/**************************************************************************
 * FILE NAME: S32Z2.h                                                     *
 * REVISION:  2.3                                                         *
 *                                                                        *
 * DESCRIPTION:                                                           *
 * This file contains all of the register and bit field definitions for   *
 * the S32Z2.                                                             *
 *========================================================================*/
/*==================================================================================================
*   Copyright 2024 NXP
*   NXP Confidential and Proprietary. This software is owned or controlled by NXP and may only
*   be used strictly in accordance with the applicable license terms. By expressly accepting
*   such terms or by downloading, installing, activating and/or otherwise using the software,
*   you are agreeing that you have read, and that you agree to comply with and are bound by,
*   such license terms. If you do not agree to be bound by the applicable license terms, then
*   you may not retain, install, activate or otherwise use the software.
*
*   THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
*   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
*   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
*   IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
*   INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
*   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
*   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
*   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
*   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*   THE POSSIBILITY OF SUCH DAMAGE.
==================================================================================================*/
/*>>>>>>> NOTE! this file is auto-generated please do not edit it! <<<<<<<*/

/**************************************************************************
 * Example register & bit field write:                                    *
 *                                                                        *
 *  <MODULE>.<REGISTER>.B.<BIT> = 1;                                      *
 *  <MODULE>.<REGISTER>.R       = 0x10000000;                             *
 *                                                                        *
 **************************************************************************/

#ifndef _S32Z2_H_
#define _S32Z2_H_
#include "typedefs.h"
#ifdef __cplusplus
extern "C" {
#endif
#ifdef __MWERKS__
#pragma push
#pragma ANSI_strict off
#endif
#ifdef __ghs__
#pragma ghs nowarning 618
#endif
#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wpedantic"
#endif


/* ============================================================================
   =============================== Module: ACCESS_PROTECTION ==================
   ============================================================================ */

typedef union ACCESS_PROTECTION_MAPAR_union_tag { /* Memory Access Protection Address Register */
  vuint32_t R;
  struct {
    vuint32_t MAPA:32;
  } B;
} ACCESS_PROTECTION_MAPAR_tag;

typedef union ACCESS_PROTECTION_D_DTAP_union_tag { /* IDM Access Configuration */
  vuint32_t R;
  struct {
    vuint32_t AP:3;
    vuint32_t _unused_3:29;
  } B;
} ACCESS_PROTECTION_D_DTAP_tag;

struct ACCESS_PROTECTION_tag {
  ACCESS_PROTECTION_MAPAR_tag MAPAR;   /* Memory Access Protection Address Register */
  uint8_t ACCESS_PROTECTION_reserved0[4];
  ACCESS_PROTECTION_D_DTAP_tag D_DTAP; /* IDM Access Configuration */
};


/* ============================================================================
   =============================== Module: ACE ================================
   ============================================================================ */

typedef union ACE_INIT_VEC_union_tag { /* Initial Value */
  vuint64_t R;
  struct {
    vuint64_t DATA:64;
  } B;
} ACE_INIT_VEC_tag;

typedef union ACE_DATA_IN_union_tag {  /* Data Input */
  vuint64_t R;
  struct {
    vuint64_t DATA:64;
  } B;
} ACE_DATA_IN_tag;

typedef union ACE_STATUS_union_tag {   /* Status */
  vuint64_t R;
  struct {
    vuint64_t _unused_0:1;
    vuint64_t BUSY:1;                  /* Busy */
    vuint64_t ERROR:3;                 /* Error */
    vuint64_t _unused_5:11;
    vuint64_t K_ERR:1;                 /* AES Internal error (Functional error) */
    vuint64_t CMP_ERR:1;               /* Secure comparison error (Safety error) */
    vuint64_t KD_ERR:1;                /* Key digest error (Functional error) */
    vuint64_t KP_ERR:1;                /* Key property check error (Functional error) */
    vuint64_t VERIFY_ERR:1;            /* Verify error */
    vuint64_t DMA_ERR:1;               /* DMA error (Functional Error) */
    vuint64_t ECC_ERR:1;               /* ECC error (Safety error) */
    vuint64_t DMA1_DID_ERR:1;          /* DMA1 DID error (Safety error) */
    vuint64_t DMA2_DID_ERR:1;          /* DMA2 DID error (Safety error) */
    vuint64_t KP_CRC_ERR:1;            /* Key Property CRC Error (Safety error) */
    vuint64_t _unused_26:38;
  } B;
} ACE_STATUS_tag;

typedef union ACE_CONTROL_union_tag {  /* Control */
  vuint64_t R;
  struct {
    vuint64_t ENABLE:1;                /* Enable */
    vuint64_t RESET:1;                 /* Reset */
    vuint64_t MASK:7;                  /* Result mask */
    vuint64_t _unused_9:55;
  } B;
} ACE_CONTROL_tag;

typedef union ACE_RESULT_union_tag {   /* Result */
  vuint64_t R;
  struct {
    vuint64_t DATA:64;                 /* Result data */
  } B;
} ACE_RESULT_tag;

typedef union ACE_AUTH_TAG_union_tag { /* Auth Tag Output */
  vuint64_t R;
  struct {
    vuint64_t DATA:64;                 /* Auth tag result data */
  } B;
} ACE_AUTH_TAG_tag;

typedef union ACE_VERSION_union_tag {  /* ACE Version */
  vuint64_t R;
  struct {
    vuint64_t EXT_REV:4;               /* Extended revision */
    vuint64_t MINOR_REV2:4;            /* Minor revision number 2 */
    vuint64_t MINOR_REV1:4;            /* Minor revision number 1 */
    vuint64_t MAJOR_REV:4;             /* Major revision number */
    vuint64_t MILESTONE:2;             /* ACE version milestone */
    vuint64_t _unused_18:46;
  } B;
} ACE_VERSION_tag;

struct ACE_tag {
  ACE_INIT_VEC_tag INIT_VEC;           /* Initial Value */
  ACE_DATA_IN_tag DATA_IN;             /* Data Input */
  uint8_t ACE_reserved0[240];
  ACE_STATUS_tag STATUS;               /* Status */
  ACE_CONTROL_tag CONTROL;             /* Control */
  uint8_t ACE_reserved1[8];
  ACE_RESULT_tag RESULT;               /* Result */
  ACE_AUTH_TAG_tag AUTH_TAG;           /* Auth Tag Output */
  ACE_VERSION_tag VERSION;             /* ACE Version */
};


/* ============================================================================
   =============================== Module: ADC ================================
   ============================================================================ */

typedef union ADC_MCR_union_tag {      /* Main Configuration */
  vuint32_t R;
  struct {
    vuint32_t PWDN:1;                  /* Power-Down Enable */
    vuint32_t _unused_1:3;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t ACKO:1;                  /* Auto-Clock-Off Mode Enable */
    vuint32_t ABORT:1;                 /* Abort Conversion */
    vuint32_t ABORTCHAIN:1;            /* Abort Conversion Chain */
    vuint32_t ADCLKSE:1;               /* Analog Clock Frequency Select */
    vuint32_t TSAMP:2;                 /* Sample Time for Calibration */
    vuint32_t NRSMPL:2;                /* Number of Averaging Samples */
    vuint32_t AVGEN:1;                 /* Average Enable */
    vuint32_t CALSTART:1;              /* Calibration Start */
    vuint32_t STCL:1;                  /* Self-Testing Configuration Lock */
    vuint32_t CTU_MODE:1;              /* Cross Trigger Unit Mode */
    vuint32_t CTUEN:1;                 /* Cross Trigger Unit Enable */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t JSTART:1;                /* Start Injection Conversion */
    vuint32_t JEDGE:1;                 /* Injection Trigger Edge Selection */
    vuint32_t JTRGEN:1;                /* Injection External Trigger Enable */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t NSTART:1;                /* Normal Conversion Start */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t EDGE:1;                  /* Trigger Edge Select */
    vuint32_t TRGEN:1;                 /* External Trigger Enable */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t MODE:1;                  /* Normal Scan Mode Select */
    vuint32_t WLSIDE:1;                /* Write Left Side */
    vuint32_t OWREN:1;                 /* Overwrite Enable */
  } B;
} ADC_MCR_tag;

typedef union ADC_MSR_union_tag {      /* Main Status */
  vuint32_t R;
  struct {
    vuint32_t ADCSTATUS:3;             /* ADC Status */
    vuint32_t _unused_3:2;             /* Reserved */
    vuint32_t ACKO:1;                  /* Auto-Clock-Off Enable */
    vuint32_t _unused_6:3;             /* Reserved */
    vuint32_t CHADDR:7;                /* Channel Address */
    vuint32_t CTUSTART:1;              /* CTU Conversion Status */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t SELF_TEST_S:1;           /* Self-Test Status */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t JSTART:1;                /* Injected Conversion Status */
    vuint32_t _unused_21:2;            /* Reserved */
    vuint32_t JABORT:1;                /* Injected Conversion Abort Status */
    vuint32_t NSTART:1;                /* Normal Conversion Status */
    vuint32_t _unused_25:4;            /* Reserved */
    vuint32_t CALBUSY:1;               /* Calibration Busy */
    vuint32_t CALFAIL:1;               /* Calibration Failed */
    vuint32_t CALIBRTD:1;              /* Calibration Status */
  } B;
} ADC_MSR_tag;

typedef union ADC_ISR_union_tag {      /* Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t ECH:1;                   /* End of Conversion Chain */
    vuint32_t EOC:1;                   /* End of Channel Conversion */
    vuint32_t JECH:1;                  /* Injected End of Conversion Chain */
    vuint32_t JEOC:1;                  /* Injected Channel End of Conversion */
    vuint32_t EOCTU:1;                 /* End of CTU Conversion */
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} ADC_ISR_tag;

typedef union ADC_CEOCFR0_union_tag {  /* Channel Pending 0 */
  vuint32_t R;
  struct {
    vuint32_t EOC_CH0:1;               /* Channel 0 EOC Status */
    vuint32_t EOC_CH1:1;               /* Channel 1 EOC Status */
    vuint32_t EOC_CH2:1;               /* Channel 2 EOC Status */
    vuint32_t EOC_CH3:1;               /* Channel 3 EOC Status */
    vuint32_t EOC_CH4:1;               /* Channel 4 EOC Status */
    vuint32_t EOC_CH5:1;               /* Channel 5 EOC Status */
    vuint32_t EOC_CH6:1;               /* Channel 6 EOC Status */
    vuint32_t EOC_CH7:1;               /* Channel 7 EOC Status */
    vuint32_t _unused_8:24;
  } B;
} ADC_CEOCFR0_tag;

typedef union ADC_CEOCFR1_union_tag {  /* Channel Pending 1 */
  vuint32_t R;
  struct {
    vuint32_t EOC_CH32:1;              /* Channel 32 EOC Status */
    vuint32_t EOC_CH33:1;              /* Channel 33 EOC Status */
    vuint32_t EOC_CH34:1;              /* Channel 34 EOC Status */
    vuint32_t EOC_CH35:1;              /* Channel 35 EOC Status */
    vuint32_t EOC_CH36:1;              /* Channel 36 EOC Status */
    vuint32_t EOC_CH37:1;              /* Channel 37 EOC Status */
    vuint32_t EOC_CH38:1;              /* Channel 38 EOC Status */
    vuint32_t EOC_CH39:1;              /* Channel 39 EOC Status */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:16;
  } B;
} ADC_CEOCFR1_tag;

typedef union ADC_IMR_union_tag {      /* Interrupt Mask */
  vuint32_t R;
  struct {
    vuint32_t MSKECH:1;                /* End of Chain Conversion Interrupt Mask */
    vuint32_t MSKEOC:1;                /* End of Conversion Interrupt Mask */
    vuint32_t MSKJECH:1;               /* End of Injected Chain Conversion Interrupt Mask */
    vuint32_t MSKJEOC:1;               /* End of Injected Conversion Interrupt Mask */
    vuint32_t MSKEOCTU:1;              /* End of CTU Conversion Interrupt Mask */
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} ADC_IMR_tag;

typedef union ADC_CIMR0_union_tag {    /* Channel Interrupt Mask 0 */
  vuint32_t R;
  struct {
    vuint32_t CIM0:1;                  /* Channel 0 Interrupt Enable */
    vuint32_t CIM1:1;                  /* Channel 1 Interrupt Enable */
    vuint32_t CIM2:1;                  /* Channel 2 Interrupt Enable */
    vuint32_t CIM3:1;                  /* Channel 3 Interrupt Enable */
    vuint32_t CIM4:1;                  /* Channel 4 Interrupt Enable */
    vuint32_t CIM5:1;                  /* Channel 5 Interrupt Enable */
    vuint32_t CIM6:1;                  /* Channel 6 Interrupt Enable */
    vuint32_t CIM7:1;                  /* Channel 7 Interrupt Enable */
    vuint32_t _unused_8:24;
  } B;
} ADC_CIMR0_tag;

typedef union ADC_CIMR1_union_tag {    /* Channel Interrupt Mask 1 */
  vuint32_t R;
  struct {
    vuint32_t CIM32:1;                 /* Channel 32 Interrupt Enable */
    vuint32_t CIM33:1;                 /* Channel 33 Interrupt Enable */
    vuint32_t CIM34:1;                 /* Channel 34 Interrupt Enable */
    vuint32_t CIM35:1;                 /* Channel 35 Interrupt Enable */
    vuint32_t CIM36:1;                 /* Channel 36 Interrupt Enable */
    vuint32_t CIM37:1;                 /* Channel 37 Interrupt Enable */
    vuint32_t CIM38:1;                 /* Channel 38 Interrupt Enable */
    vuint32_t CIM39:1;                 /* Channel 39 Interrupt Enable */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:16;
  } B;
} ADC_CIMR1_tag;

typedef union ADC_WTISR_union_tag {    /* Watchdog Threshold Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t WDG0L:1;                 /* Channel 0 Watchdog Low Threshold Interrupt */
    vuint32_t WDG0H:1;                 /* Channel 0 Watchdog High Threshold Interrupt */
    vuint32_t WDG1L:1;                 /* Channel 1 Watchdog Low Threshold Interrupt */
    vuint32_t WDG1H:1;                 /* Channel 1 Watchdog High Threshold Interrupt */
    vuint32_t WDG2L:1;                 /* Channel 2 Watchdog Low Threshold Interrupt */
    vuint32_t WDG2H:1;                 /* Channel 2 Watchdog High Threshold Interrupt */
    vuint32_t WDG3L:1;                 /* Channel 3 Watchdog Low Threshold Interrupt */
    vuint32_t WDG3H:1;                 /* Channel 3 Watchdog High Threshold Interrupt */
    vuint32_t WDG4L:1;                 /* Channel 4 Watchdog Low Threshold Interrupt */
    vuint32_t WDG4H:1;                 /* Channel 4 Watchdog High Threshold Interrupt */
    vuint32_t WDG5L:1;                 /* Channel 5 Watchdog Low Threshold Interrupt */
    vuint32_t WDG5H:1;                 /* Channel 5 Watchdog High Threshold Interrupt */
    vuint32_t WDG6L:1;                 /* Channel 6 Watchdog Low Threshold Interrupt */
    vuint32_t WDG6H:1;                 /* Channel 6 Watchdog High Threshold Interrupt */
    vuint32_t WDG7L:1;                 /* Channel 7 Watchdog Low Threshold Interrupt */
    vuint32_t WDG7H:1;                 /* Channel 7 Watchdog High Threshold Interrupt */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t _unused_22:2;            /* Reserved */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t _unused_26:2;            /* Reserved */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} ADC_WTISR_tag;

typedef union ADC_WTIMR_union_tag {    /* Watchdog Threshold Interrupt Mask */
  vuint32_t R;
  struct {
    vuint32_t MSKWDG0L:1;              /* Channel 0 Watchdog Low Threshold Interrupt Mask */
    vuint32_t MSKWDG0H:1;              /* Channel 0 Watchdog High Threshold Interrupt Mask */
    vuint32_t MSKWDG1L:1;              /* Channel 1 Watchdog Low Threshold Interrupt Mask */
    vuint32_t MSKWDG1H:1;              /* Channel 1 Watchdog High Threshold Interrupt Mask */
    vuint32_t MSKWDG2L:1;              /* Channel 2 Watchdog Low Threshold Interrupt Mask */
    vuint32_t MSKWDG2H:1;              /* Channel 2 Watchdog High Threshold Interrupt Mask */
    vuint32_t MSKWDG3L:1;              /* Channel 3 Watchdog Low Threshold Interrupt Mask */
    vuint32_t MSKWDG3H:1;              /* Channel 3 Watchdog High Threshold Interrupt Mask */
    vuint32_t MSKWDG4L:1;              /* Channel 4 Watchdog Low Threshold Interrupt Mask */
    vuint32_t MSKWDG4H:1;              /* Channel 4 Watchdog High Threshold Interrupt Mask */
    vuint32_t MSKWDG5L:1;              /* Channel 5 Watchdog Low Threshold Interrupt Mask */
    vuint32_t MSKWDG5H:1;              /* Channel 5 Watchdog High Threshold Interrupt Mask */
    vuint32_t MSKWDG6L:1;              /* Channel 6 Watchdog Low Threshold Interrupt Mask */
    vuint32_t MSKWDG6H:1;              /* Channel 6 Watchdog High Threshold Interrupt Mask */
    vuint32_t MSKWDG7L:1;              /* Channel 7 Watchdog Low Threshold Interrupt Mask */
    vuint32_t MSKWDG7H:1;              /* Channel 7 Watchdog High Threshold Interrupt Mask */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t _unused_22:2;            /* Reserved */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t _unused_26:2;            /* Reserved */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} ADC_WTIMR_tag;

typedef union ADC_DMAE_union_tag {     /* DMAE */
  vuint32_t R;
  struct {
    vuint32_t DMAEN:1;                 /* DMA Global Enable */
    vuint32_t DCLR:1;                  /* DMA Clear Sequence Enable */
    vuint32_t _unused_2:30;            /* Reserved */
  } B;
} ADC_DMAE_tag;

typedef union ADC_DMAR0_union_tag {    /* DMA 0 */
  vuint32_t R;
  struct {
    vuint32_t DMA0:1;                  /* Channel 0 DMA Enable */
    vuint32_t DMA1:1;                  /* Channel 1 DMA Enable */
    vuint32_t DMA2:1;                  /* Channel 2 DMA Enable */
    vuint32_t DMA3:1;                  /* Channel 3 DMA Enable */
    vuint32_t DMA4:1;                  /* Channel 4 DMA Enable */
    vuint32_t DMA5:1;                  /* Channel 5 DMA Enable */
    vuint32_t DMA6:1;                  /* Channel 6 DMA Enable */
    vuint32_t DMA7:1;                  /* Channel 7 DMA Enable */
    vuint32_t _unused_8:24;
  } B;
} ADC_DMAR0_tag;

typedef union ADC_DMAR1_union_tag {    /* DMA 1 */
  vuint32_t R;
  struct {
    vuint32_t DMA32:1;                 /* Channel 32 DMA Enable */
    vuint32_t DMA33:1;                 /* Channel 33 DMA Enable */
    vuint32_t DMA34:1;                 /* Channel 34 DMA Enable */
    vuint32_t DMA35:1;                 /* Channel 35 DMA Enable */
    vuint32_t DMA36:1;                 /* Channel 36 DMA Enable */
    vuint32_t DMA37:1;                 /* Channel 37 DMA Enable */
    vuint32_t DMA38:1;                 /* Channel 38 DMA Enable */
    vuint32_t DMA39:1;                 /* Channel 39 DMA Enable */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:16;
  } B;
} ADC_DMAR1_tag;

typedef union ADC_THRHLR0_union_tag {  /* Analog Watchdog Threshold 0 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_THRHLR0_tag;

typedef union ADC_THRHLR1_union_tag {  /* Analog Watchdog Threshold 1 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_THRHLR1_tag;

typedef union ADC_THRHLR2_union_tag {  /* Analog Watchdog Threshold 2 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_THRHLR2_tag;

typedef union ADC_THRHLR3_union_tag {  /* Analog Watchdog Threshold 3 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_THRHLR3_tag;

typedef union ADC_PSCR_union_tag {     /* Presampling Control */
  vuint32_t R;
  struct {
    vuint32_t PRECONV:1;               /* Convert Presampled Value */
    vuint32_t PREVAL0:2;               /* Internal Presampling Voltage Selection */
    vuint32_t PREVAL1:2;               /* Internal Presampling Voltage Selection. */
    vuint32_t _unused_5:2;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} ADC_PSCR_tag;

typedef union ADC_PSR0_union_tag {     /* Presampling 0 */
  vuint32_t R;
  struct {
    vuint32_t PRES0:1;                 /* Presampling Enable for Channel 0 */
    vuint32_t PRES1:1;                 /* Presampling Enable for Channel 1 */
    vuint32_t PRES2:1;                 /* Presampling Enable for Channel 2 */
    vuint32_t PRES3:1;                 /* Presampling Enable for Channel 3 */
    vuint32_t PRES4:1;                 /* Presampling Enable for Channel 4 */
    vuint32_t PRES5:1;                 /* Presampling Enable for Channel 5 */
    vuint32_t PRES6:1;                 /* Presampling Enable for Channel 6 */
    vuint32_t PRES7:1;                 /* Presampling Enable for Channel 7 */
    vuint32_t _unused_8:24;
  } B;
} ADC_PSR0_tag;

typedef union ADC_PSR1_union_tag {     /* Presampling 1 */
  vuint32_t R;
  struct {
    vuint32_t PRES32:1;                /* Presampling Enable for Channel 32 */
    vuint32_t PRES33:1;                /* Presampling Enable for Channel 33 */
    vuint32_t PRES34:1;                /* Presampling Enable for Channel 34 */
    vuint32_t PRES35:1;                /* Presampling Enable for Channel 35 */
    vuint32_t PRES36:1;                /* Presampling Enable for Channel 36 */
    vuint32_t PRES37:1;                /* Presampling Enable for Channel 37 */
    vuint32_t PRES38:1;                /* Presampling Enable for Channel 38 */
    vuint32_t PRES39:1;                /* Presampling Enable for Channel 39 */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:16;
  } B;
} ADC_PSR1_tag;

typedef union ADC_CTR0_union_tag {     /* Conversion Timing 0 */
  vuint32_t R;
  struct {
    vuint32_t INPSAMP:8;               /* Sampling Phase Duration */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ADC_CTR0_tag;

typedef union ADC_CTR1_union_tag {     /* Conversion Timing 1 */
  vuint32_t R;
  struct {
    vuint32_t INPSAMP:8;               /* Sampling Phase Duration */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ADC_CTR1_tag;

typedef union ADC_NCMR0_union_tag {    /* Normal Conversion Mask 0 */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;                   /* Normal Conversion Mask for Channel 0 */
    vuint32_t CH1:1;                   /* Normal Conversion Mask for Channel 1 */
    vuint32_t CH2:1;                   /* Normal Conversion Mask for Channel 2 */
    vuint32_t CH3:1;                   /* Normal Conversion Mask for Channel 3 */
    vuint32_t CH4:1;                   /* Normal Conversion Mask for Channel 4 */
    vuint32_t CH5:1;                   /* Normal Conversion Mask for Channel 5 */
    vuint32_t CH6:1;                   /* Normal Conversion Mask for Channel 6 */
    vuint32_t CH7:1;                   /* Normal Conversion Mask for Channel 7 */
    vuint32_t _unused_8:24;
  } B;
} ADC_NCMR0_tag;

typedef union ADC_NCMR1_union_tag {    /* Normal Conversion Mask 1 */
  vuint32_t R;
  struct {
    vuint32_t CH32:1;                  /* Normal Conversion Mask for Channel 32 */
    vuint32_t CH33:1;                  /* Normal Conversion Mask for Channel 33 */
    vuint32_t CH34:1;                  /* Normal Conversion Mask for Channel 34 */
    vuint32_t CH35:1;                  /* Normal Conversion Mask for Channel 35 */
    vuint32_t CH36:1;                  /* Normal Conversion Mask for Channel 36 */
    vuint32_t CH37:1;                  /* Normal Conversion Mask for Channel 37 */
    vuint32_t CH38:1;                  /* Normal Conversion Mask for Channel 38 */
    vuint32_t CH39:1;                  /* Normal Conversion Mask for Channel 39 */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:16;
  } B;
} ADC_NCMR1_tag;

typedef union ADC_JCMR0_union_tag {    /* Injected Conversion Mask 0 */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;                   /* Injected Conversion Mask for Channel 0 */
    vuint32_t CH1:1;                   /* Injected Conversion Mask for Channel 1 */
    vuint32_t CH2:1;                   /* Injected Conversion Mask for Channel 2 */
    vuint32_t CH3:1;                   /* Injected Conversion Mask for Channel 3 */
    vuint32_t CH4:1;                   /* Injected Conversion Mask for Channel 4 */
    vuint32_t CH5:1;                   /* Injected Conversion Mask for Channel 5 */
    vuint32_t CH6:1;                   /* Injected Conversion Mask for Channel 6 */
    vuint32_t CH7:1;                   /* Injected Conversion Mask for Channel 7 */
    vuint32_t _unused_8:24;
  } B;
} ADC_JCMR0_tag;

typedef union ADC_JCMR1_union_tag {    /* Injected Conversion Mask 1 */
  vuint32_t R;
  struct {
    vuint32_t CH32:1;                  /* Injected Conversion Mask for Channel 32 */
    vuint32_t CH33:1;                  /* Injected Conversion Mask for Channel 33 */
    vuint32_t CH34:1;                  /* Injected Conversion Mask for Channel 34 */
    vuint32_t CH35:1;                  /* Injected Conversion Mask for Channel 35 */
    vuint32_t CH36:1;                  /* Injected Conversion Mask for Channel 36 */
    vuint32_t CH37:1;                  /* Injected Conversion Mask for Channel 37 */
    vuint32_t CH38:1;                  /* Injected Conversion Mask for Channel 38 */
    vuint32_t CH39:1;                  /* Injected Conversion Mask for Channel 39 */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:16;
  } B;
} ADC_JCMR1_tag;

typedef union ADC_USROFSGN_union_tag { /* User OFFSET and Gain */
  vuint32_t R;
  struct {
    vuint32_t OFFSUSER:8;              /* User Defined Offset */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t GAINUSER:10;             /* User-Defined Gain Value */
    vuint32_t _unused_26:6;            /* Reserved */
  } B;
} ADC_USROFSGN_tag;

typedef union ADC_PDEDR_union_tag {    /* Power Down Exit Delay */
  vuint32_t R;
  struct {
    vuint32_t PDED:8;                  /* Power Down Exist Delay */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ADC_PDEDR_tag;

typedef union ADC_PCDR_union_tag {     /* Precision Channel n Data */
  vuint32_t R;
  struct {
    vuint32_t CDATA:12;                /* Channel Converted Data */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t RESULT:2;                /* Mode of Conversion Status */
    vuint32_t OVERW:1;                 /* Data Overwrite */
    vuint32_t VALID:1;                 /* Conversion Data Valid */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} ADC_PCDR_tag;

typedef union ADC_ICDR_union_tag {     /* Internal Channel n Data */
  vuint32_t R;
  struct {
    vuint32_t CDATA:12;                /* Channel Converted Data */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t RESULT:2;                /* Mode of Conversion Status */
    vuint32_t OVERW:1;                 /* Data Overwrite */
    vuint32_t VALID:1;                 /* Conversion Data Valid */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} ADC_ICDR_tag;

typedef union ADC_THRHLR4_union_tag {  /* Analog Watchdog Threshold 4 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_THRHLR4_tag;

typedef union ADC_THRHLR5_union_tag {  /* Analog Watchdog Threshold 5 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_THRHLR5_tag;

typedef union ADC_THRHLR6_union_tag {  /* Analog Watchdog Threshold 6 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_THRHLR6_tag;

typedef union ADC_THRHLR7_union_tag {  /* Analog Watchdog Threshold 7 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_THRHLR7_tag;

typedef union ADC_CWSELR0_union_tag {  /* Channel Watchdog Select 0 */
  vuint32_t R;
  struct {
    vuint32_t WSEL_CH0:3;              /* Channel Watchdog Select for Channel 0 */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t WSEL_CH1:3;              /* Channel Watchdog Select for Channel 1 */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t WSEL_CH2:3;              /* Channel Watchdog Select for Channel 2 */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t WSEL_CH3:3;              /* Channel Watchdog Select for Channel 3 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t WSEL_CH4:3;              /* Channel Watchdog Select for Channel 4 */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t WSEL_CH5:3;              /* Channel Watchdog Select for Channel 5 */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t WSEL_CH6:3;              /* Channel Watchdog Select for Channel 6 */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t WSEL_CH7:3;              /* Channel Watchdog Select for Channel 7 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} ADC_CWSELR0_tag;

typedef union ADC_CWSELR4_union_tag {  /* Channel Watchdog Select 4 */
  vuint32_t R;
  struct {
    vuint32_t WSEL_CH32:3;             /* Channel Watchdog Select for Channel 32 */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t WSEL_CH33:3;             /* Channel Watchdog Select for Channel 33 */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t WSEL_CH34:3;             /* Channel Watchdog Select for Channel 34 */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t WSEL_CH35:3;             /* Channel Watchdog Select for Channel 35 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t WSEL_CH36:3;             /* Channel Watchdog Select for Channel 36 */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t WSEL_CH37:3;             /* Channel Watchdog Select for Channel 37 */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t WSEL_CH38:3;             /* Channel Watchdog Select for Channel 38 */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t WSEL_CH39:3;             /* Channel Watchdog Select for Channel 39 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} ADC_CWSELR4_tag;

typedef union ADC_CWENR0_union_tag {   /* Channel Watchdog Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t CWEN0:1;                 /* Watchdog Enable for Channel 0 */
    vuint32_t CWEN1:1;                 /* Watchdog Enable for Channel 1 */
    vuint32_t CWEN2:1;                 /* Watchdog Enable for Channel 2 */
    vuint32_t CWEN3:1;                 /* Watchdog Enable for Channel 3 */
    vuint32_t CWEN4:1;                 /* Watchdog Enable for Channel 4 */
    vuint32_t CWEN5:1;                 /* Watchdog Enable for Channel 5 */
    vuint32_t CWEN6:1;                 /* Watchdog Enable for Channel 6 */
    vuint32_t CWEN7:1;                 /* Watchdog Enable for Channel 7 */
    vuint32_t _unused_8:24;
  } B;
} ADC_CWENR0_tag;

typedef union ADC_CWENR1_union_tag {   /* Channel Watchdog Enable 1 */
  vuint32_t R;
  struct {
    vuint32_t CWEN32:1;                /* Watchdog Enable for Channel 32 */
    vuint32_t CWEN33:1;                /* Watchdog Enable for Channel 33 */
    vuint32_t CWEN34:1;                /* Watchdog Enable for Channel 34 */
    vuint32_t CWEN35:1;                /* Watchdog Enable for Channel 35 */
    vuint32_t CWEN36:1;                /* Watchdog Enable for Channel 36 */
    vuint32_t CWEN37:1;                /* Watchdog Enable for Channel 37 */
    vuint32_t CWEN38:1;                /* Watchdog Enable for Channel 38 */
    vuint32_t CWEN39:1;                /* Watchdog Enable for Channel 39 */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:16;
  } B;
} ADC_CWENR1_tag;

typedef union ADC_AWORR0_union_tag {   /* Analog Watchdog Out of Range 0 */
  vuint32_t R;
  struct {
    vuint32_t AWOR_CH0:1;              /* Analog Watchdog Out of Range for Channel 0 */
    vuint32_t AWOR_CH1:1;              /* Analog Watchdog Out of Range for Channel 1 */
    vuint32_t AWOR_CH2:1;              /* Analog Watchdog Out of Range for Channel 2 */
    vuint32_t AWOR_CH3:1;              /* Analog Watchdog Out of Range for Channel 3 */
    vuint32_t AWOR_CH4:1;              /* Analog Watchdog Out of Range for Channel 4 */
    vuint32_t AWOR_CH5:1;              /* Analog Watchdog Out of Range for Channel 5 */
    vuint32_t AWOR_CH6:1;              /* Analog Watchdog Out of Range for Channel 6 */
    vuint32_t AWOR_CH7:1;              /* Analog Watchdog Out of Range for Channel 7 */
    vuint32_t _unused_8:24;
  } B;
} ADC_AWORR0_tag;

typedef union ADC_AWORR1_union_tag {   /* Analog Watchdog Out of Range 1 */
  vuint32_t R;
  struct {
    vuint32_t AWOR_CH32:1;             /* Analog Watchdog Out of Range for Channel 32 */
    vuint32_t AWOR_CH33:1;             /* Analog Watchdog Out of Range for Channel 33 */
    vuint32_t AWOR_CH34:1;             /* Analog Watchdog Out of Range for Channel 34 */
    vuint32_t AWOR_CH35:1;             /* Analog Watchdog Out of Range for Channel 35 */
    vuint32_t AWOR_CH36:1;             /* Analog Watchdog Out of Range for Channel 36 */
    vuint32_t AWOR_CH37:1;             /* Analog Watchdog Out of Range for Channel 37 */
    vuint32_t AWOR_CH38:1;             /* Analog Watchdog Out of Range for Channel 38 */
    vuint32_t AWOR_CH39:1;             /* Analog Watchdog Out of Range for Channel 39 */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:16;
  } B;
} ADC_AWORR1_tag;

typedef union ADC_STCR1_union_tag {    /* Self-Test Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t INPSAMP_S:8;             /* Sampling Configuration for Algorithm S */
    vuint32_t _unused_16:8;            /* Reserved */
    vuint32_t INPSAMP_C:8;             /* Sampling Configuration for Algorithm C */
  } B;
} ADC_STCR1_tag;

typedef union ADC_STCR2_union_tag {    /* Self-Test Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t FMA_S:1;                 /* Fault Mapping for BGAP Algorithm */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t FMA_C:1;                 /* Fault Mapping for Algorithm C */
    vuint32_t FMA_WDTERR:1;            /* Fault Mapping for Watchdog Timer Error */
    vuint32_t FMA_WDSERR:1;            /* Fault Mapping for Watchdog Sequence Error */
    vuint32_t _unused_5:2;             /* Reserved */
    vuint32_t EN:1;                    /* Self-Testing Channel Enable */
    vuint32_t _unused_8:3;             /* Reserved */
    vuint32_t MSKERR_S0:1;             /* Error on Algorithm S0 Channel Interrupt Mask */
    vuint32_t MSKERR_S1:1;             /* Error on Algorithm S1 Channel Interrupt Mask */
    vuint32_t MSKERR_S2:1;             /* Error on Algorithm S2 Channel Interrupt Mask */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t MSKERR_C:1;              /* Error on Algorithm C Channel Interrupt Mask */
    vuint32_t MSKWDG_EOA_S:1;          /* End of Algorithm S Interrupt Mask */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t MSKWDG_EOA_C:1;          /* End of Algorithm C Interrupt Mask */
    vuint32_t _unused_19:4;            /* Reserved */
    vuint32_t MSKST_EOC:1;             /* Self-Test EOC Interrupt Mask */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t MSKWDTERR:1;             /* Watchdog Timer Error Interrupt Mask */
    vuint32_t SERR:1;                  /* Error Fault Injection Field (write-only) */
    vuint32_t MSKWDSERR:1;             /* Watchdog Sequence Error Interrupt Mask */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_STCR2_tag;

typedef union ADC_STCR3_union_tag {    /* Self-Test Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t MSTEP:5;                 /* Self-Test Step Select */
    vuint32_t _unused_5:3;             /* Reserved */
    vuint32_t ALG:2;                   /* Self-Test Algorithm Select */
    vuint32_t _unused_10:22;           /* Reserved */
  } B;
} ADC_STCR3_tag;

typedef union ADC_STBRR_union_tag {    /* Self-Test Baud Rate */
  vuint32_t R;
  struct {
    vuint32_t BR:8;                    /* Algorithm Baud Rate */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t WDT:3;                   /* Watchdog Timer Value */
    vuint32_t _unused_19:13;           /* Reserved */
  } B;
} ADC_STBRR_tag;

typedef union ADC_STSR1_union_tag {    /* Self-Test Status 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;             /* Reserved */
    vuint32_t STEP_C:5;                /* Algorithm C Step Number Error */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t ERR_S0:1;                /* Algorithm S0 Error */
    vuint32_t ERR_S1:1;                /* Algorithm S1 Error */
    vuint32_t ERR_S2:1;                /* Algorithm S2 Error */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t ERR_C:1;                 /* Algorithm C Error */
    vuint32_t WDG_EOA_S:1;             /* Watchdog End of Algorithm S */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t WDG_EOA_C:1;             /* Watchdog End of Algorithm C */
    vuint32_t _unused_19:4;            /* Reserved */
    vuint32_t ST_EOC:1;                /* Self-Test EOC */
    vuint32_t OVERWR:1;                /* Overwrite Error */
    vuint32_t WDTERR:1;                /* Watchdog Timer Error */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t WDSERR:1;                /* Watchdog Sequence Errors */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_STSR1_tag;

typedef union ADC_STSR2_union_tag {    /* Self-Test Status 2 */
  vuint32_t R;
  struct {
    vuint32_t DATA0:12;                /* Test Channel Converted Data when ERR_S1 Occurred */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t DATA1:12;                /* Test Channel Converted Data when ERR_S1 Occurred */
    vuint32_t _unused_28:3;            /* Reserved */
    vuint32_t OVFL:1;                  /* Overflow Bit */
  } B;
} ADC_STSR2_tag;

typedef union ADC_STSR3_union_tag {    /* Self-Test Status 3 */
  vuint32_t R;
  struct {
    vuint32_t DATA0:12;                /* Test Channel Converted Data when ERR_S0 Occurred */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t DATA1:12;                /* Test Channel Converted Data when ERR_C Occurred */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_STSR3_tag;

typedef union ADC_STSR4_union_tag {    /* Self-Test Status 4 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t DATA1:12;                /* Test Channel Converted Data When ERR_C Occurred */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_STSR4_tag;

typedef union ADC_STDR1_union_tag {    /* Self-Test Data 1 */
  vuint32_t R;
  struct {
    vuint32_t TCDATA:12;               /* Test Channel Converted Data */
    vuint32_t _unused_12:6;            /* Reserved */
    vuint32_t OWERWR:1;                /* Overwrite Data */
    vuint32_t VALID:1;                 /* Valid Data */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} ADC_STDR1_tag;

typedef union ADC_STDR2_union_tag {    /* Self-Test Data 2 */
  vuint32_t R;
  struct {
    vuint32_t IDATA:12;                /* Integer Data */
    vuint32_t _unused_12:6;            /* Reserved */
    vuint32_t OVERWR:1;                /* Overwrite Data */
    vuint32_t VALID:1;                 /* Valid Data */
    vuint32_t FDATA:12;                /* Fractional Data */
  } B;
} ADC_STDR2_tag;

typedef union ADC_STAW0R_union_tag {   /* Self-Test Analog Watchdog 0 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold Value for Algorithm S Step0 */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold Value for Algorithm S Step0 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t WDTE:1;                  /* Watchdog Timer Enable */
    vuint32_t AWDE:1;                  /* Analog Watchdog Enable */
  } B;
} ADC_STAW0R_tag;

typedef union ADC_STAW1AR_union_tag {  /* Self-Test Analog Watchdog 1A */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold Value for Algorithm S Step1 */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold Value for Algorithm S Step1 */
    vuint32_t _unused_28:3;            /* Reserved */
    vuint32_t AWDE:1;                  /* Analog Watchdog Enable */
  } B;
} ADC_STAW1AR_tag;

typedef union ADC_STAW1BR_union_tag {  /* Self-Test Analog Watchdog 1B */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold Value for Algorithm S Step1 */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold Value for Algorithm S Step1 */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_STAW1BR_tag;

typedef union ADC_STAW2R_union_tag {   /* Self-Test Analog Watchdog 2 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Threshold level low */
    vuint32_t _unused_12:19;           /* Reserved */
    vuint32_t AWDE:1;                  /* Analog Watchdog Enable */
  } B;
} ADC_STAW2R_tag;

typedef union ADC_STAW3R_union_tag {   /* Self-Test Analog Watchdog 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;            /* Reserved */
  } B;
} ADC_STAW3R_tag;

typedef union ADC_STAW4R_union_tag {   /* Self-Test Analog Watchdog 4 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold Value for Step0 of Algorithm C */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold Value for Step0 of Algorithm C */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t WDTE:1;                  /* Watchdog Timer Enable */
    vuint32_t AWDE:1;                  /* Analog Watchdog Enable */
  } B;
} ADC_STAW4R_tag;

typedef union ADC_STAW5R_union_tag {   /* Self-Test Analog Watchdog 5 */
  vuint32_t R;
  struct {
    vuint32_t THRL:12;                 /* Low Threshold Value for Step0 of Algorithm C */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t THRH:12;                 /* High Threshold Value for Step N of Algorithm C */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} ADC_STAW5R_tag;

typedef union ADC_CALSTAT_union_tag {  /* Calibration Status */
  vuint32_t R;
  struct {
    vuint32_t STAT_1:1;                /* Status of Calibration Step 1 */
    vuint32_t STAT_2:1;                /* Status of Calibration Step 2 */
    vuint32_t STAT_3:1;                /* Status of Calibration Step 3 */
    vuint32_t STAT_4:1;                /* Status of calibration step 4 */
    vuint32_t STAT_5:1;                /* Status of Calibration Step 5 */
    vuint32_t STAT_6:1;                /* Status of Calibration Step 6 */
    vuint32_t STAT_7:1;                /* Status of Calibration Step 7 */
    vuint32_t STAT_8:1;                /* Status of Calibration Step 8 */
    vuint32_t STAT_9:1;                /* Status of Calibration Step 9 */
    vuint32_t STAT_10:1;               /* Status of Calibration Step 10 */
    vuint32_t STAT_11:1;               /* Status of Calibration Step 11 */
    vuint32_t STAT_12:1;               /* Status of Calibration Step 12 */
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t TEST_RESULT:16;          /* TEST_RESULT */
  } B;
} ADC_CALSTAT_tag;

struct ADC_tag {
  ADC_MCR_tag MCR;                     /* Main Configuration */
  ADC_MSR_tag MSR;                     /* Main Status */
  uint8_t ADC_reserved0[8];
  ADC_ISR_tag ISR;                     /* Interrupt Status */
  ADC_CEOCFR0_tag CEOCFR0;             /* Channel Pending 0 */
  ADC_CEOCFR1_tag CEOCFR1;             /* Channel Pending 1 */
  uint8_t ADC_reserved1[4];
  ADC_IMR_tag IMR;                     /* Interrupt Mask */
  ADC_CIMR0_tag CIMR0;                 /* Channel Interrupt Mask 0 */
  ADC_CIMR1_tag CIMR1;                 /* Channel Interrupt Mask 1 */
  uint8_t ADC_reserved2[4];
  ADC_WTISR_tag WTISR;                 /* Watchdog Threshold Interrupt Status */
  ADC_WTIMR_tag WTIMR;                 /* Watchdog Threshold Interrupt Mask */
  uint8_t ADC_reserved3[8];
  ADC_DMAE_tag DMAE;                   /* DMAE */
  ADC_DMAR0_tag DMAR0;                 /* DMA 0 */
  ADC_DMAR1_tag DMAR1;                 /* DMA 1 */
  uint8_t ADC_reserved4[20];
  ADC_THRHLR0_tag THRHLR0;             /* Analog Watchdog Threshold 0 */
  ADC_THRHLR1_tag THRHLR1;             /* Analog Watchdog Threshold 1 */
  ADC_THRHLR2_tag THRHLR2;             /* Analog Watchdog Threshold 2 */
  ADC_THRHLR3_tag THRHLR3;             /* Analog Watchdog Threshold 3 */
  uint8_t ADC_reserved5[16];
  ADC_PSCR_tag PSCR;                   /* Presampling Control */
  ADC_PSR0_tag PSR0;                   /* Presampling 0 */
  ADC_PSR1_tag PSR1;                   /* Presampling 1 */
  uint8_t ADC_reserved6[8];
  ADC_CTR0_tag CTR0;                   /* Conversion Timing 0 */
  ADC_CTR1_tag CTR1;                   /* Conversion Timing 1 */
  uint8_t ADC_reserved7[8];
  ADC_NCMR0_tag NCMR0;                 /* Normal Conversion Mask 0 */
  ADC_NCMR1_tag NCMR1;                 /* Normal Conversion Mask 1 */
  uint8_t ADC_reserved8[8];
  ADC_JCMR0_tag JCMR0;                 /* Injected Conversion Mask 0 */
  ADC_JCMR1_tag JCMR1;                 /* Injected Conversion Mask 1 */
  uint8_t ADC_reserved9[4];
  ADC_USROFSGN_tag USROFSGN;           /* User OFFSET and Gain */
  uint8_t ADC_reserved10[4];
  ADC_PDEDR_tag PDEDR;                 /* Power Down Exit Delay */
  uint8_t ADC_reserved11[52];
  ADC_PCDR_tag PCDR[8];                /* Precision Channel n Data */
  uint8_t ADC_reserved12[96];
  ADC_ICDR_tag ICDR[8];                /* Internal Channel n Data */
  uint8_t ADC_reserved13[224];
  ADC_THRHLR4_tag THRHLR4;             /* Analog Watchdog Threshold 4 */
  ADC_THRHLR5_tag THRHLR5;             /* Analog Watchdog Threshold 5 */
  ADC_THRHLR6_tag THRHLR6;             /* Analog Watchdog Threshold 6 */
  ADC_THRHLR7_tag THRHLR7;             /* Analog Watchdog Threshold 7 */
  uint8_t ADC_reserved14[32];
  ADC_CWSELR0_tag CWSELR0;             /* Channel Watchdog Select 0 */
  uint8_t ADC_reserved15[12];
  ADC_CWSELR4_tag CWSELR4;             /* Channel Watchdog Select 4 */
  uint8_t ADC_reserved16[28];
  ADC_CWENR0_tag CWENR0;               /* Channel Watchdog Enable 0 */
  ADC_CWENR1_tag CWENR1;               /* Channel Watchdog Enable 1 */
  uint8_t ADC_reserved17[8];
  ADC_AWORR0_tag AWORR0;               /* Analog Watchdog Out of Range 0 */
  ADC_AWORR1_tag AWORR1;               /* Analog Watchdog Out of Range 1 */
  uint8_t ADC_reserved18[72];
  ADC_STCR1_tag STCR1;                 /* Self-Test Configuration 1 */
  ADC_STCR2_tag STCR2;                 /* Self-Test Configuration 2 */
  ADC_STCR3_tag STCR3;                 /* Self-Test Configuration 3 */
  ADC_STBRR_tag STBRR;                 /* Self-Test Baud Rate */
  ADC_STSR1_tag STSR1;                 /* Self-Test Status 1 */
  ADC_STSR2_tag STSR2;                 /* Self-Test Status 2 */
  ADC_STSR3_tag STSR3;                 /* Self-Test Status 3 */
  ADC_STSR4_tag STSR4;                 /* Self-Test Status 4 */
  uint8_t ADC_reserved19[16];
  ADC_STDR1_tag STDR1;                 /* Self-Test Data 1 */
  ADC_STDR2_tag STDR2;                 /* Self-Test Data 2 */
  uint8_t ADC_reserved20[8];
  ADC_STAW0R_tag STAW0R;               /* Self-Test Analog Watchdog 0 */
  ADC_STAW1AR_tag STAW1AR;             /* Self-Test Analog Watchdog 1A */
  ADC_STAW1BR_tag STAW1BR;             /* Self-Test Analog Watchdog 1B */
  ADC_STAW2R_tag STAW2R;               /* Self-Test Analog Watchdog 2 */
  ADC_STAW3R_tag STAW3R;               /* Self-Test Analog Watchdog 3 */
  ADC_STAW4R_tag STAW4R;               /* Self-Test Analog Watchdog 4 */
  ADC_STAW5R_tag STAW5R;               /* Self-Test Analog Watchdog 5 */
  ADC_CALSTAT_tag CALSTAT;             /* Calibration Status */
};


/* ============================================================================
   =============================== Module: AES_ACCEL ==========================
   ============================================================================ */

typedef union AES_ACCEL_ACCEL_LEN_union_tag { /* LEN */
  vuint32_t R;
  struct {
    vuint32_t TLEN:19;                 /* TLEN */
    vuint32_t _unused_19:13;           /* Reserved */
  } B;
} AES_ACCEL_ACCEL_LEN_tag;

typedef union AES_ACCEL_ACCEL_AILEN_union_tag { /* AAD/IV length */
  vuint32_t R;
  struct {
    vuint32_t AAD:19;                  /* Additional Authenticated Data */
    vuint32_t _unused_19:5;            /* Reserved */
    vuint32_t IVLEN:7;                 /* IV length */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} AES_ACCEL_ACCEL_AILEN_tag;

typedef union AES_ACCEL_ACCEL_CRYPT_union_tag { /* CRYPT */
  vuint32_t R;
  struct {
    vuint32_t MASK:7;                  /* MASK */
    vuint32_t _unused_7:1;
    vuint32_t CMODE:4;                 /* Cryptographic Mode */
    vuint32_t CO:2;                    /* Cryptographic Operation */
    vuint32_t _unused_14:2;
    vuint32_t KSLOT:7;                 /* Key Slot */
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:8;
  } B;
} AES_ACCEL_ACCEL_CRYPT_tag;

typedef union AES_ACCEL_ACCEL_OWNSTAT_union_tag { /* OWN_Status */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t NS:1;                    /* Non-Secure */
    vuint32_t PRIV:1;                  /* Privilege */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} AES_ACCEL_ACCEL_OWNSTAT_tag;

typedef union AES_ACCEL_ACCEL_TLVAL_union_tag { /* Timer Load Value */
  vuint32_t R;
  struct {
    vuint32_t TSV:16;                  /* Timer Start Value */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} AES_ACCEL_ACCEL_TLVAL_tag;

typedef union AES_ACCEL_ACCEL_CVAL_union_tag { /* Current Timer Value */
  vuint32_t R;
  struct {
    vuint32_t TVL:16;                  /* Current Timer Value */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} AES_ACCEL_ACCEL_CVAL_tag;

typedef union AES_ACCEL_ACCEL_TCTRL_union_tag { /* Timer Control */
  vuint32_t R;
  struct {
    vuint32_t TEN:1;                   /* Timer Enable Bit */
    vuint32_t TIE:1;                   /* Timer Interrupt Enable */
    vuint32_t FRZ:1;                   /* Freeze */
    vuint32_t _unused_3:5;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:23;
  } B;
} AES_ACCEL_ACCEL_TCTRL_tag;

typedef union AES_ACCEL_ACCEL_TFLG_union_tag { /* Timer Flag */
  vuint32_t R;
  struct {
    vuint32_t TIF:1;                   /* Timer Interrupt Flag */
    vuint32_t _unused_1:7;
    vuint32_t _unused_8:2;
    vuint32_t _unused_10:22;
  } B;
} AES_ACCEL_ACCEL_TFLG_tag;

typedef union AES_ACCEL_ACCEL_FEEDINTMAP_union_tag { /* FEEDINTMAP */
  vuint32_t R;
  struct {
    vuint32_t FIEN0:1;                 /* FIENn */
    vuint32_t FIEN1:1;                 /* FIENn */
    vuint32_t FIEN2:1;                 /* FIENn */
    vuint32_t FIEN3:1;                 /* FIENn */
    vuint32_t FIEN4:1;                 /* FIENn */
    vuint32_t FIEN5:1;                 /* FIENn */
    vuint32_t FIEN6:1;                 /* FIENn */
    vuint32_t FIEN7:1;                 /* FIENn */
    vuint32_t FIEN8:1;                 /* FIENn */
    vuint32_t FIEN9:1;                 /* FIENn */
    vuint32_t FIEN10:1;                /* FIENn */
    vuint32_t FIEN11:1;                /* FIENn */
    vuint32_t FIEN12:1;                /* FIENn */
    vuint32_t FIEN13:1;                /* FIENn */
    vuint32_t FIEN14:1;                /* FIENn */
    vuint32_t FIEN15:1;                /* FIENn */
    vuint32_t FIEN16:1;                /* FIENn */
    vuint32_t FIEN17:1;                /* FIENn */
    vuint32_t FIEN18:1;                /* FIENn */
    vuint32_t FIEN19:1;                /* FIENn */
    vuint32_t FIEN20:1;                /* FIENn */
    vuint32_t FIEN21:1;                /* FIENn */
    vuint32_t FIEN22:1;                /* FIENn */
    vuint32_t FIEN23:1;                /* FIENn */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} AES_ACCEL_ACCEL_FEEDINTMAP_tag;

typedef union AES_ACCEL_ACCEL_RESULTINTMAP_union_tag { /* RESULTINTMAP */
  vuint32_t R;
  struct {
    vuint32_t RIEN0:1;                 /* RIENn */
    vuint32_t RIEN1:1;                 /* RIENn */
    vuint32_t RIEN2:1;                 /* RIENn */
    vuint32_t RIEN3:1;                 /* RIENn */
    vuint32_t RIEN4:1;                 /* RIENn */
    vuint32_t RIEN5:1;                 /* RIENn */
    vuint32_t RIEN6:1;                 /* RIENn */
    vuint32_t RIEN7:1;                 /* RIENn */
    vuint32_t RIEN8:1;                 /* RIENn */
    vuint32_t RIEN9:1;                 /* RIENn */
    vuint32_t RIEN10:1;                /* RIENn */
    vuint32_t RIEN11:1;                /* RIENn */
    vuint32_t RIEN12:1;                /* RIENn */
    vuint32_t RIEN13:1;                /* RIENn */
    vuint32_t RIEN14:1;                /* RIENn */
    vuint32_t RIEN15:1;                /* RIENn */
    vuint32_t RIEN16:1;                /* RIENn */
    vuint32_t RIEN17:1;                /* RIENn */
    vuint32_t RIEN18:1;                /* RIENn */
    vuint32_t RIEN19:1;                /* RIENn */
    vuint32_t RIEN20:1;                /* RIENn */
    vuint32_t RIEN21:1;                /* RIENn */
    vuint32_t RIEN22:1;                /* RIENn */
    vuint32_t RIEN23:1;                /* RIENn */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} AES_ACCEL_ACCEL_RESULTINTMAP_tag;

typedef struct AES_ACCEL_ACCEL_struct_tag {
  AES_ACCEL_ACCEL_LEN_tag LEN;         /* LEN */
  AES_ACCEL_ACCEL_AILEN_tag AILEN;     /* AAD/IV length */
  AES_ACCEL_ACCEL_CRYPT_tag CRYPT;     /* CRYPT */
  AES_ACCEL_ACCEL_OWNSTAT_tag OWNSTAT; /* OWN_Status */
  AES_ACCEL_ACCEL_TLVAL_tag TLVAL;     /* Timer Load Value */
  AES_ACCEL_ACCEL_CVAL_tag CVAL;       /* Current Timer Value */
  AES_ACCEL_ACCEL_TCTRL_tag TCTRL;     /* Timer Control */
  AES_ACCEL_ACCEL_TFLG_tag TFLG;       /* Timer Flag */
  AES_ACCEL_ACCEL_FEEDINTMAP_tag FEEDINTMAP; /* FEEDINTMAP */
  AES_ACCEL_ACCEL_RESULTINTMAP_tag RESULTINTMAP; /* RESULTINTMAP */
  uint8_t ACCEL_reserved0[4056];
} AES_ACCEL_ACCEL_tag;

struct AES_ACCEL_tag {
  AES_ACCEL_ACCEL_tag ACCEL[8];
};


/* ============================================================================
   =============================== Module: ATP ================================
   ============================================================================ */

typedef union ATP_PLLC_union_tag {     /* PLL Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:23;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:7;
    vuint32_t PLLPD:1;                 /* PLL Reset */
  } B;
} ATP_PLLC_tag;

typedef union ATP_PLLS_union_tag {     /* PLL Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t LOCK:1;                  /* PLL Lock Status */
    vuint32_t LOL:1;                   /* PLL Loss of Lock Status */
    vuint32_t _unused_4:4;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:22;
  } B;
} ATP_PLLS_tag;

typedef union ATP_PLLDIV_union_tag {   /* PLL Divider */
  vuint32_t R;
  struct {
    vuint32_t MFID:8;                  /* PLL Multiplication Factor Integer Divider */
    vuint32_t _unused_8:4;
    vuint32_t RDIV:3;                  /* PLL Input Clock Predivider */
    vuint32_t _unused_15:1;
    vuint32_t ODIV1:6;                 /* PLL Output Division Factor 1 */
    vuint32_t _unused_22:3;
    vuint32_t _unused_25:6;
    vuint32_t _unused_31:1;
  } B;
} ATP_PLLDIV_tag;

typedef union ATP_PLLCAL_2_union_tag { /* PLL Calibration 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:17;
    vuint32_t CALDPER:2;               /* Calibration DAC Period */
    vuint32_t _unused_19:3;
    vuint32_t _unused_22:10;
  } B;
} ATP_PLLCAL_2_tag;

typedef union ATP_PLLCKMUX_union_tag { /* PLL Clock MUX */
  vuint32_t R;
  struct {
    vuint32_t REFCLKSEL:2;             /* Reference Clock Selection */
    vuint32_t _unused_2:30;
  } B;
} ATP_PLLCKMUX_tag;

typedef union ATP_ALS_union_tag {      /* AL Status */
  vuint32_t R;
  struct {
    vuint32_t AS:1;                    /* Aurora Status */
    vuint32_t CS:1;                    /* Channel Status */
    vuint32_t TS:2;                    /* Training Status */
    vuint32_t _unused_4:5;
    vuint32_t PRST:1;                  /* Channel Partner Reset */
    vuint32_t TXCFG:3;                 /* TX Lane Configuration */
    vuint32_t _unused_13:2;
    vuint32_t _unused_15:4;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:10;
  } B;
} ATP_ALS_tag;

typedef union ATP_ALGC_union_tag {     /* AL General Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CCOEN:1;                 /* Clock Compensation Override Enable */
    vuint32_t CRCEN:1;                 /* CRC Enable */
    vuint32_t _unused_4:4;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:3;
    vuint32_t _unused_13:1;
    vuint32_t PCRST:1;                 /* Protocol Converter Reset */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:12;
    vuint32_t _unused_30:1;
    vuint32_t RST:1;                   /* Aurora Channel Reset */
  } B;
} ATP_ALGC_tag;

typedef union ATP_ALTC_union_tag {     /* AL Training Control */
  vuint32_t R;
  struct {
    vuint32_t VTC:4;                   /* Verify Timer Count */
    vuint32_t _unused_4:6;
    vuint32_t BTC:4;                   /* Bond Timer Count */
    vuint32_t _unused_14:5;
    vuint32_t ATC:4;                   /* Align Timer Count */
    vuint32_t _unused_23:3;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t VHD:1;                   /* Hold in Verify */
    vuint32_t BHD:1;                   /* Hold in Bond */
    vuint32_t AHD:1;                   /* Hold in Align */
    vuint32_t STE:1;                   /* Static Training Enable */
  } B;
} ATP_ALTC_tag;

typedef union ATP_LVDSTX_union_tag {   /* LVDS TX IO Configuration */
  vuint32_t R;
  struct {
    vuint32_t TXAMODE:1;
    vuint32_t CREF_EN:1;               /* Current Reference Enable */
    vuint32_t TX_TREF_EN:1;            /* TX LVDS Termination Reference Enable */
    vuint32_t _unused_3:2;
    vuint32_t PREMPH:2;                /* Pre-Emphasis Configuration */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:4;
    vuint32_t TX_CONF:4;               /* LVDS PADS TX Control Configuration */
    vuint32_t PADS_TX_CONF_EN:1;       /* PADS TX Configuration Enable */
    vuint32_t _unused_17:15;
  } B;
} ATP_LVDSTX_tag;

typedef union ATP_LVDSRX_union_tag {   /* LVDS RX IO Configuration */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t RXICE:1;
    vuint32_t _unused_2:1;
    vuint32_t RXCB:1;                  /* RX LVDS Current Boost */
    vuint32_t _unused_4:12;
    vuint32_t RX_TREF_EN:1;            /* RX LVDS Termination Reference Enable */
    vuint32_t _unused_17:15;
  } B;
} ATP_LVDSRX_tag;

typedef union ATP_LVDSTXOBE_union_tag { /* LVDS TX OBE Configuration */
  vuint32_t R;
  struct {
    vuint32_t OBETX0:1;
    vuint32_t OBETX1:1;
    vuint32_t OBETX2:1;
    vuint32_t OBETX3:1;
    vuint32_t _unused_4:28;
  } B;
} ATP_LVDSTXOBE_tag;

typedef union ATP_CIAC_union_tag {     /* CIA Control */
  vuint32_t R;
  struct {
    vuint32_t NUM_LANE:4;              /* Number of Aurora Lanes */
    vuint32_t TPIUCM:3;                /* TPIU Control Mode */
    vuint32_t _unused_7:1;
    vuint32_t DBITR:1;
    vuint32_t DBYTER:1;
    vuint32_t _unused_10:4;
    vuint32_t _unused_14:2;
    vuint32_t _unused_16:12;
    vuint32_t TPIU_CLK_SEL:1;          /* TPIU clock mux select signal */
    vuint32_t TPIU_CLK_DISABLE:1;      /* TPIU clock disable */
    vuint32_t _unused_30:2;
  } B;
} ATP_CIAC_tag;

typedef union ATP_ATPE_union_tag {     /* Aurora Trace Port Enable */
  vuint32_t R;
  struct {
    vuint32_t ATPEN:1;
    vuint32_t APHYEN:1;
    vuint32_t _unused_2:30;
  } B;
} ATP_ATPE_tag;

typedef union ATP_ITCTRL_union_tag {   /* Integration Mode Control */
  vuint32_t R;
  struct {
    vuint32_t IME:1;
    vuint32_t _unused_1:31;
  } B;
} ATP_ITCTRL_tag;

typedef union ATP_CLAIMSET_union_tag { /* Claim Tag Set */
  vuint32_t R;
  struct {
    vuint32_t SET:32;
  } B;
} ATP_CLAIMSET_tag;

typedef union ATP_CLAIMCLR_union_tag { /* Claim Tag Clear */
  vuint32_t R;
  struct {
    vuint32_t CLR:32;
  } B;
} ATP_CLAIMCLR_tag;

typedef union ATP_DEVAFF0_union_tag {  /* Device Affinity 0 */
  vuint32_t R;
  struct {
    vuint32_t DEVAFF0:32;              /* Value of this field is set to 24h. */
  } B;
} ATP_DEVAFF0_tag;

typedef union ATP_DEVAFF1_union_tag {  /* Device Affinity 1 */
  vuint32_t R;
  struct {
    vuint32_t DEVAFF1:32;              /* Value of this field is set to 24h. */
  } B;
} ATP_DEVAFF1_tag;

typedef union ATP_LAR_union_tag {      /* Lock Access Register */
  vuint32_t R;
  struct {
    vuint32_t KEY:32;
  } B;
} ATP_LAR_tag;

typedef union ATP_LSR_union_tag {      /* Lock Status Register */
  vuint32_t R;
  struct {
    vuint32_t SLI:1;
    vuint32_t SLK:1;
    vuint32_t NTT:1;
    vuint32_t _unused_3:29;
  } B;
} ATP_LSR_tag;

typedef union ATP_AUTHSTATUS_union_tag { /* Authentication Status */
  vuint32_t R;
  struct {
    vuint32_t AUTHSTATUS:8;
    vuint32_t _unused_8:24;
  } B;
} ATP_AUTHSTATUS_tag;

typedef union ATP_DEVARCH_union_tag {  /* Device Architecture */
  vuint32_t R;
  struct {
    vuint32_t ARCHID:16;               /* Architecture ID */
    vuint32_t REVISION:4;              /* Architecture revision */
    vuint32_t PRESENT:1;               /* Indicates the presence of this register. */
    vuint32_t ARCHITECT:11;            /* Defines the architect of the component. */
  } B;
} ATP_DEVARCH_tag;

typedef union ATP_DEVID2_union_tag {   /* Device Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t DEVID2:32;
  } B;
} ATP_DEVID2_tag;

typedef union ATP_DEVID1_union_tag {   /* Device Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DEVID1:32;
  } B;
} ATP_DEVID1_tag;

typedef union ATP_DEVID_union_tag {    /* Device Configuration */
  vuint32_t R;
  struct {
    vuint32_t DEVID:32;
  } B;
} ATP_DEVID_tag;

typedef union ATP_DEVTYPE_union_tag {  /* Device Type Identifier */
  vuint32_t R;
  struct {
    vuint32_t MAJOR:4;                 /* Major Type */
    vuint32_t SUB:4;                   /* Sub Type */
    vuint32_t _unused_8:24;
  } B;
} ATP_DEVTYPE_tag;

typedef union ATP_PIDR4_union_tag {    /* Peripheral Identification Register 4 */
  vuint32_t R;
  struct {
    vuint32_t DES_2:4;                 /* JEP106 Continuation Code */
    vuint32_t SIZE:4;                  /* 4KB Count */
    vuint32_t _unused_8:24;
  } B;
} ATP_PIDR4_tag;

typedef union ATP_PIDR0_union_tag {    /* Peripheral Identification Register 0 */
  vuint32_t R;
  struct {
    vuint32_t PART_0:8;                /* Part number, bits[7:0] */
    vuint32_t _unused_8:24;
  } B;
} ATP_PIDR0_tag;

typedef union ATP_PIDR1_union_tag {    /* Peripheral Identification Register 1 */
  vuint32_t R;
  struct {
    vuint32_t PART_1:4;                /* Part number, bits[11:8] */
    vuint32_t DES_0:4;                 /* JEP106 identification code, bits[3:0] */
    vuint32_t _unused_8:24;
  } B;
} ATP_PIDR1_tag;

typedef union ATP_PIDR2_union_tag {    /* Peripheral Identification Register 2 */
  vuint32_t R;
  struct {
    vuint32_t DES_1:3;                 /* JEP106 identification code, bits[6:4] */
    vuint32_t JEDEC:1;
    vuint32_t REVISION:4;              /* Revision */
    vuint32_t _unused_8:24;
  } B;
} ATP_PIDR2_tag;

typedef union ATP_PIDR3_union_tag {    /* Peripheral Identification Register 3 */
  vuint32_t R;
  struct {
    vuint32_t REVAND:4;                /* RevAnd */
    vuint32_t CMOD:4;                  /* Customer Modified */
    vuint32_t _unused_8:24;
  } B;
} ATP_PIDR3_tag;

typedef union ATP_CIDR0_union_tag {    /* Component Identification Register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_0:8;               /* Preamble */
    vuint32_t _unused_8:24;
  } B;
} ATP_CIDR0_tag;

typedef union ATP_CIDR1_union_tag {    /* Component Identification Register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_1:4;               /* Preamble */
    vuint32_t CLASS:4;                 /* Component class */
    vuint32_t _unused_8:24;
  } B;
} ATP_CIDR1_tag;

typedef union ATP_CIDR2_union_tag {    /* Component Identification Register 2 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_2:8;               /* Preamble */
    vuint32_t _unused_8:24;
  } B;
} ATP_CIDR2_tag;

typedef union ATP_CIDR3_union_tag {    /* Component Identification Register 3 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_3:8;               /* Preamble */
    vuint32_t _unused_8:24;
  } B;
} ATP_CIDR3_tag;

struct ATP_tag {
  ATP_PLLC_tag PLLC;                   /* PLL Control */
  ATP_PLLS_tag PLLS;                   /* PLL Status */
  ATP_PLLDIV_tag PLLDIV;               /* PLL Divider */
  uint8_t ATP_reserved0[12];
  ATP_PLLCAL_2_tag PLLCAL_2;           /* PLL Calibration 2 */
  uint8_t ATP_reserved1[4];
  ATP_PLLCKMUX_tag PLLCKMUX;           /* PLL Clock MUX */
  uint8_t ATP_reserved2[732];
  ATP_ALS_tag ALS;                     /* AL Status */
  uint8_t ATP_reserved3[4];
  ATP_ALGC_tag ALGC;                   /* AL General Control */
  ATP_ALTC_tag ALTC;                   /* AL Training Control */
  uint8_t ATP_reserved4[304];
  ATP_LVDSTX_tag LVDSTX;               /* LVDS TX IO Configuration */
  ATP_LVDSRX_tag LVDSRX;               /* LVDS RX IO Configuration */
  ATP_LVDSTXOBE_tag LVDSTXOBE;         /* LVDS TX OBE Configuration */
  uint8_t ATP_reserved5[52];
  ATP_CIAC_tag CIAC;                   /* CIA Control */
  uint8_t ATP_reserved6[4];
  ATP_ATPE_tag ATPE;                   /* Aurora Trace Port Enable */
  uint8_t ATP_reserved7[2676];
  ATP_ITCTRL_tag ITCTRL;               /* Integration Mode Control */
  uint8_t ATP_reserved8[156];
  ATP_CLAIMSET_tag CLAIMSET;           /* Claim Tag Set */
  ATP_CLAIMCLR_tag CLAIMCLR;           /* Claim Tag Clear */
  ATP_DEVAFF0_tag DEVAFF0;             /* Device Affinity 0 */
  ATP_DEVAFF1_tag DEVAFF1;             /* Device Affinity 1 */
  ATP_LAR_tag LAR;                     /* Lock Access Register */
  ATP_LSR_tag LSR;                     /* Lock Status Register */
  ATP_AUTHSTATUS_tag AUTHSTATUS;       /* Authentication Status */
  ATP_DEVARCH_tag DEVARCH;             /* Device Architecture */
  ATP_DEVID2_tag DEVID2;               /* Device Configuration 2 */
  ATP_DEVID1_tag DEVID1;               /* Device Configuration 1 */
  ATP_DEVID_tag DEVID;                 /* Device Configuration */
  ATP_DEVTYPE_tag DEVTYPE;             /* Device Type Identifier */
  ATP_PIDR4_tag PIDR4;                 /* Peripheral Identification Register 4 */
  uint8_t ATP_reserved9[12];
  ATP_PIDR0_tag PIDR0;                 /* Peripheral Identification Register 0 */
  ATP_PIDR1_tag PIDR1;                 /* Peripheral Identification Register 1 */
  ATP_PIDR2_tag PIDR2;                 /* Peripheral Identification Register 2 */
  ATP_PIDR3_tag PIDR3;                 /* Peripheral Identification Register 3 */
  ATP_CIDR0_tag CIDR0;                 /* Component Identification Register 0 */
  ATP_CIDR1_tag CIDR1;                 /* Component Identification Register 1 */
  ATP_CIDR2_tag CIDR2;                 /* Component Identification Register 2 */
  ATP_CIDR3_tag CIDR3;                 /* Component Identification Register 3 */
};


/* ============================================================================
   =============================== Module: AXBS ===============================
   ============================================================================ */

typedef union AXBS_PORT_PRS_union_tag { /* Priority Slave Registers */
  vuint32_t R;
  struct {
    vuint32_t M0:3;                    /* Master 0 priority */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t M1:3;                    /* Master 1 priority */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t M2:3;                    /* Master 2 priority */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t M3:3;                    /* Master 3 priority */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t M4:3;                    /* Master 4 priority */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t M5:3;                    /* Master 5 priority */
    vuint32_t _unused_23:9;            /* Reserved */
  } B;
} AXBS_PORT_PRS_tag;

typedef union AXBS_PORT_CRS_union_tag { /* Control Register */
  vuint32_t R;
  struct {
    vuint32_t PARK:3;                  /* Park */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t PCTL:2;                  /* Parking Control */
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t ARB:2;                   /* Arbitration Mode */
    vuint32_t _unused_10:6;            /* Reserved */
    vuint32_t HPE0:1;                  /* HPE0 */
    vuint32_t HPE1:1;                  /* HPE1 */
    vuint32_t HPE2:1;                  /* HPE2 */
    vuint32_t HPE3:1;                  /* HPE3 */
    vuint32_t HPE4:1;                  /* HPE4 */
    vuint32_t HPE5:1;                  /* HPE5 */
    vuint32_t HPE6:1;                  /* HPE6 */
    vuint32_t HPE7:1;                  /* HPE7 */
    vuint32_t _unused_24:6;            /* Reserved */
    vuint32_t HLP:1;                   /* Halt Low Priority */
    vuint32_t RO:1;                    /* Read Only */
  } B;
} AXBS_PORT_CRS_tag;

typedef struct AXBS_PORT_struct_tag {
  AXBS_PORT_PRS_tag PRS;               /* Priority Slave Registers */
  uint8_t PORT_reserved0[12];
  AXBS_PORT_CRS_tag CRS;               /* Control Register */
  uint8_t PORT_reserved1[236];
} AXBS_PORT_tag;

typedef union AXBS_MGPCR0_union_tag {  /* Master General Purpose Control Register */
  vuint32_t R;
  struct {
    vuint32_t AULB:3;                  /* Arbitrates On Undefined Length Bursts */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} AXBS_MGPCR0_tag;

typedef union AXBS_MGPCR1_union_tag {  /* Master General Purpose Control Register */
  vuint32_t R;
  struct {
    vuint32_t AULB:3;                  /* Arbitrates On Undefined Length Bursts */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} AXBS_MGPCR1_tag;

typedef union AXBS_MGPCR2_union_tag {  /* Master General Purpose Control Register */
  vuint32_t R;
  struct {
    vuint32_t AULB:3;                  /* Arbitrates On Undefined Length Bursts */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} AXBS_MGPCR2_tag;

typedef union AXBS_MGPCR3_union_tag {  /* Master General Purpose Control Register */
  vuint32_t R;
  struct {
    vuint32_t AULB:3;                  /* Arbitrates On Undefined Length Bursts */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} AXBS_MGPCR3_tag;

typedef union AXBS_MGPCR4_union_tag {  /* Master General Purpose Control Register */
  vuint32_t R;
  struct {
    vuint32_t AULB:3;                  /* Arbitrates On Undefined Length Bursts */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} AXBS_MGPCR4_tag;

typedef union AXBS_MGPCR5_union_tag {  /* Master General Purpose Control Register */
  vuint32_t R;
  struct {
    vuint32_t AULB:3;                  /* Arbitrates On Undefined Length Bursts */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} AXBS_MGPCR5_tag;

struct AXBS_tag {
  AXBS_PORT_tag PORT[8];
  AXBS_MGPCR0_tag MGPCR0;              /* Master General Purpose Control Register */
  uint8_t AXBS_reserved0[252];
  AXBS_MGPCR1_tag MGPCR1;              /* Master General Purpose Control Register */
  uint8_t AXBS_reserved1[252];
  AXBS_MGPCR2_tag MGPCR2;              /* Master General Purpose Control Register */
  uint8_t AXBS_reserved2[252];
  AXBS_MGPCR3_tag MGPCR3;              /* Master General Purpose Control Register */
  uint8_t AXBS_reserved3[252];
  AXBS_MGPCR4_tag MGPCR4;              /* Master General Purpose Control Register */
  uint8_t AXBS_reserved4[252];
  AXBS_MGPCR5_tag MGPCR5;              /* Master General Purpose Control Register */
};


/* ============================================================================
   =============================== Module: Boot ===============================
   ============================================================================ */

typedef union Boot_BOOT_GPR_BMR1_union_tag { /* Boot Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t BOOT_CFG:32;             /* Boot Configuration */
  } B;
} Boot_BOOT_GPR_BMR1_tag;

typedef union Boot_BOOT_GPR_BMR2_union_tag { /* Boot Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t FUSE_SEL:1;              /* Select Boot from Fuses */
    vuint32_t _unused_5:19;
    vuint32_t BOOTMODE2:1;             /* Boot Mode Pin 2 */
    vuint32_t BOOTMODE1:1;             /* Boot Mode Pin 1 */
    vuint32_t _unused_26:6;
  } B;
} Boot_BOOT_GPR_BMR2_tag;

typedef union Boot_BOOT_POR_CTRL_REG_union_tag { /* Boot POR Control */
  vuint32_t R;
  struct {
    vuint32_t HSE_FW_ROLLBACK_COUNT_A:8; /* HSE Firmware Rollback Count - Primary Image */
    vuint32_t HSE_FW_ROLLBACK_COUNT_B:8; /* HSE Firmware Rollback Count - Backup Image */
    vuint32_t HSE_FW_ROLLBACK_MARKER:16; /* HSE_H Firmware Rollback Marker */
  } B;
} Boot_BOOT_POR_CTRL_REG_tag;

typedef union Boot_BOOT_POR_C1_REG_union_tag { /* BOOT POR Control 1 Register */
  vuint32_t R;
  struct {
    vuint32_t SELF_TEST_DCD_IMAGE:8;   /* Self-Test DCD Image */
    vuint32_t DCD_IMAGE_SELECT:8;      /* DCD_IMAGE_SELECT marker */
    vuint32_t APP_IMAGE:8;             /* Application image */
    vuint32_t DDRC_IIA_IMAGE:8;        /* DDRC_IIA Application Image (Applicable only in case of LPDDR4 flash boot) */
  } B;
} Boot_BOOT_POR_C1_REG_tag;

typedef union Boot_BOOT_DEST_C0_REG_union_tag { /* BOOT DEST Control 0 Register */
  vuint32_t R;
  struct {
    vuint32_t LPDDR4_FUNC_RESET_COUNTER:3; /* LPDDR4 Functional reset counter */
    vuint32_t _unused_3:29;
  } B;
} Boot_BOOT_DEST_C0_REG_tag;

typedef union Boot_BOOT_FUNC_C0_REG_union_tag { /* BOOT FUNC Control 0 Register */
  vuint32_t R;
  struct {
    vuint32_t DDRC_IIA_STATUS:32;      /* DDRC_IIA Status */
  } B;
} Boot_BOOT_FUNC_C0_REG_tag;

struct Boot_tag {
  Boot_BOOT_GPR_BMR1_tag BOOT_GPR_BMR1; /* Boot Configuration 1 */
  Boot_BOOT_GPR_BMR2_tag BOOT_GPR_BMR2; /* Boot Configuration 2 */
  uint8_t Boot_reserved0[100];
  Boot_BOOT_POR_CTRL_REG_tag BOOT_POR_CTRL_REG; /* Boot POR Control */
  Boot_BOOT_POR_C1_REG_tag BOOT_POR_C1_REG; /* BOOT POR Control 1 Register */
  uint8_t Boot_reserved1[4];
  Boot_BOOT_DEST_C0_REG_tag BOOT_DEST_C0_REG; /* BOOT DEST Control 0 Register */
  uint8_t Boot_reserved2[8];
  Boot_BOOT_FUNC_C0_REG_tag BOOT_FUNC_C0_REG; /* BOOT FUNC Control 0 Register */
};


/* ============================================================================
   =============================== Module: CANXL_DSC_CONTROL ==================
   ============================================================================ */

typedef union CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSTA_union_tag { /* Descriptor Control Status */
  vuint32_t R;
  struct {
    vuint32_t STATE:3;                 /* Message Descriptor State */
    vuint32_t _unused_3:3;
    vuint32_t HWLOCK:1;                /* Hardware Lock */
    vuint32_t SYSLOCK:1;               /* System lock */
    vuint32_t HWPOINTER:5;             /* Hardware Pointer */
    vuint32_t _unused_13:3;
    vuint32_t SYSPOINTER:5;            /* System Pointer */
    vuint32_t _unused_21:3;
    vuint32_t WTMARK:4;                /* Watermark */
    vuint32_t _unused_28:4;
  } B;
} CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSTA_tag;

typedef union CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSYSPUSH_union_tag { /* Descriptor Control System Push */
  vuint32_t R;
  struct {
    vuint32_t PUSH:4;                  /* System Transmission Push Control */
    vuint32_t _unused_4:28;
  } B;
} CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSYSPUSH_tag;

typedef union CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSYSLOCK_union_tag { /* Descriptor Control System Lock */
  vuint32_t R;
  struct {
    vuint32_t SYSLOCK:1;               /* System Lock */
    vuint32_t _unused_1:31;
  } B;
} CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSYSLOCK_tag;

typedef union CANXL_DSC_CONTROL_DSCMBCTRLAR_DCACT_union_tag { /* Descriptor Control Activation */
  vuint32_t R;
  struct {
    vuint32_t ACT:1;                   /* Descriptor Activation */
    vuint32_t _unused_1:31;
  } B;
} CANXL_DSC_CONTROL_DSCMBCTRLAR_DCACT_tag;

typedef struct CANXL_DSC_CONTROL_DSCMBCTRLAR_struct_tag {
  CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSTA_tag DCSTA; /* Descriptor Control Status */
  CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSYSPUSH_tag DCSYSPUSH; /* Descriptor Control System Push */
  CANXL_DSC_CONTROL_DSCMBCTRLAR_DCSYSLOCK_tag DCSYSLOCK; /* Descriptor Control System Lock */
  uint8_t DSCMBCTRLAR_reserved0[8];
  CANXL_DSC_CONTROL_DSCMBCTRLAR_DCACT_tag DCACT; /* Descriptor Control Activation */
  uint8_t DSCMBCTRLAR_reserved1[8];
} CANXL_DSC_CONTROL_DSCMBCTRLAR_tag;

struct CANXL_DSC_CONTROL_tag {
  CANXL_DSC_CONTROL_DSCMBCTRLAR_tag DSCMBCTRLAR[128];
};


/* ============================================================================
   =============================== Module: CANXL_FILTER_BANK ==================
   ============================================================================ */

typedef union CANXL_FILTER_BANK_AFCFG0_union_tag { /* Acceptance Filter Configuration */
  vuint32_t R;
  struct {
    vuint32_t ACPTVCAN:5;              /* Number of VCAN Acceptance Filters */
    vuint32_t _unused_5:2;
    vuint32_t AVCANEN:1;               /* Acceptance VCAN Enable */
    vuint32_t ACPTSDU:5;               /* Number of SDU Acceptance Filters */
    vuint32_t _unused_13:2;
    vuint32_t ASDUEN:1;                /* Acceptance SDU Enable */
    vuint32_t ACPTADDR:5;              /* Number of ADDR Acceptance Filters */
    vuint32_t _unused_21:2;
    vuint32_t AADDREN:1;               /* Acceptance ADDR Enable */
    vuint32_t _unused_24:8;
  } B;
} CANXL_FILTER_BANK_AFCFG0_tag;

typedef union CANXL_FILTER_BANK_VAMRCFG0_union_tag { /* VCAN Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t VMSK0R0:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R1:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R2:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R3:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R4:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R5:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R6:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R7:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R8:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R9:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R10:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R11:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R12:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R13:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R14:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R15:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R16:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R17:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R18:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R19:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R20:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R21:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R22:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R23:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R24:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R25:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R26:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R27:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R28:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R29:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R30:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R31:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_FILTER_BANK_VAMRCFG0_tag;

typedef union CANXL_FILTER_BANK_SAMRCFG0_union_tag { /* SDU Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t SMSK0R0:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R1:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R2:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R3:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R4:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R5:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R6:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R7:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R8:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R9:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R10:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R11:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R12:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R13:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R14:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R15:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R16:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R17:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R18:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R19:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R20:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R21:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R22:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R23:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R24:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R25:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R26:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R27:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R28:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R29:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R30:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R31:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_FILTER_BANK_SAMRCFG0_tag;

typedef union CANXL_FILTER_BANK_AAMRCFG0_union_tag { /* ADDR Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t AMSK0R0:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R1:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R2:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R3:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R4:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R5:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R6:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R7:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R8:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R9:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R10:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R11:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R12:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R13:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R14:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R15:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R16:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R17:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R18:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R19:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R20:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R21:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R22:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R23:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R24:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R25:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R26:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R27:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R28:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R29:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R30:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R31:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_FILTER_BANK_AAMRCFG0_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_0_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_0_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_2_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_2_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_4_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_4_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_6_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_6_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_8_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_8_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_10_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_10_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_12_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_12_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_14_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_14_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_16_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_16_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_18_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_18_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_20_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_20_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_22_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_22_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_24_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_24_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_26_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_26_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_28_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_28_tag;

typedef union CANXL_FILTER_BANK_VAFLT0_30_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT0_30_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_0_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_0_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_2_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_2_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_4_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_4_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_6_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_6_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_8_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_8_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_10_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_10_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_12_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_12_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_14_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_14_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_16_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_16_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_18_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_18_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_20_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_20_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_22_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_22_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_24_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_24_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_26_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_26_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_28_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_28_tag;

typedef union CANXL_FILTER_BANK_SAFLT0_30_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT0_30_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_0L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_0L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_0H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_0H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_1L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_1L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_1H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_1H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_2L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_2L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_2H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_2H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_3L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_3L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_3H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_3H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_4L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_4L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_4H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_4H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_5L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_5L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_5H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_5H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_6L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_6L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_6H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_6H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_7L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_7L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_7H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_7H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_8L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_8L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_8H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_8H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_9L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_9L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_9H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_9H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_10L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_10L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_10H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_10H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_11L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_11L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_11H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_11H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_12L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_12L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_12H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_12H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_13L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_13L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_13H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_13H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_14L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_14L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_14H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_14H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_15L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_15L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_15H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_15H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_16L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_16L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_16H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_16H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_17L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_17L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_17H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_17H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_18L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_18L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_18H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_18H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_19L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_19L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_19H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_19H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_20L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_20L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_20H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_20H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_21L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_21L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_21H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_21H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_22L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_22L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_22H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_22H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_23L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_23L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_23H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_23H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_24L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_24L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_24H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_24H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_25L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_25L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_25H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_25H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_26L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_26L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_26H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_26H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_27L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_27L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_27H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_27H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_28L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_28L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_28H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_28H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_29L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_29L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_29H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_29H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_30L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_30L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_30H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_30H_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_31L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT0_31L_tag;

typedef union CANXL_FILTER_BANK_AAFLT0_31H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT0_31H_tag;

typedef union CANXL_FILTER_BANK_RFCFG0_union_tag { /* Rejection Filter Configuration */
  vuint32_t R;
  struct {
    vuint32_t REJVCAN:5;               /* Number of VCAN Rejection Filters */
    vuint32_t _unused_5:2;
    vuint32_t RVCANEN:1;               /* Rejection VCAN Enable */
    vuint32_t REJSDU:5;                /* Number of SDU Rejection Filters */
    vuint32_t _unused_13:2;
    vuint32_t RSDUEN:1;                /* Rejection SDU Enable */
    vuint32_t REJADDR:5;               /* Number of ADDR Rejection Filters */
    vuint32_t _unused_21:2;
    vuint32_t RADDREN:1;               /* Rejection ADDR Enable */
    vuint32_t _unused_24:8;
  } B;
} CANXL_FILTER_BANK_RFCFG0_tag;

typedef union CANXL_FILTER_BANK_VRMRCFG0_union_tag { /* VCAN Rejection Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t VMSK0R0:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R1:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R2:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R3:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R4:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R5:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R6:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R7:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R8:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R9:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R10:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R11:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R12:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R13:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R14:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R15:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R16:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R17:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R18:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R19:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R20:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R21:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R22:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R23:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R24:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R25:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R26:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R27:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R28:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R29:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R30:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R31:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
  } B;
} CANXL_FILTER_BANK_VRMRCFG0_tag;

typedef union CANXL_FILTER_BANK_SRMRCFG0_union_tag { /* SDU Rejection Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t SMSK0R0:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R1:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R2:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R3:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R4:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R5:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R6:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R7:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R8:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R9:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R10:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R11:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R12:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R13:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R14:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R15:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R16:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R17:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R18:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R19:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R20:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R21:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R22:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R23:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R24:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R25:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R26:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R27:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R28:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R29:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R30:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R31:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
  } B;
} CANXL_FILTER_BANK_SRMRCFG0_tag;

typedef union CANXL_FILTER_BANK_ARMRCFG0_union_tag { /* ADDR Rejection Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t AMSK0R0:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R1:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R2:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R3:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R4:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R5:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R6:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R7:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R8:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R9:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R10:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R11:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R12:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R13:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R14:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R15:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R16:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R17:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R18:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R19:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R20:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R21:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R22:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R23:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R24:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R25:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R26:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R27:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R28:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R29:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R30:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R31:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
  } B;
} CANXL_FILTER_BANK_ARMRCFG0_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_0_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_0_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_2_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_2_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_4_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_4_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_6_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_6_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_8_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_8_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_10_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_10_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_12_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_12_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_14_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_14_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_16_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_16_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_18_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_18_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_20_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_20_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_22_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_22_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_24_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_24_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_26_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_26_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_28_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_28_tag;

typedef union CANXL_FILTER_BANK_VRFLT0_30_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT0_30_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_0_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_0_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_2_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_2_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_4_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_4_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_6_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_6_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_8_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_8_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_10_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_10_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_12_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_12_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_14_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_14_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_16_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_16_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_18_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_18_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_20_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_20_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_22_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_22_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_24_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_24_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_26_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_26_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_28_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_28_tag;

typedef union CANXL_FILTER_BANK_SRFLT0_30_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT0_30_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_0L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_0L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_0H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_0H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_1L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_1L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_1H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_1H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_2L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_2L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_2H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_2H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_3L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_3L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_3H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_3H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_4L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_4L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_4H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_4H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_5L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_5L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_5H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_5H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_6L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_6L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_6H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_6H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_7L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_7L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_7H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_7H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_8L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_8L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_8H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_8H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_9L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_9L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_9H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_9H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_10L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_10L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_10H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_10H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_11L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_11L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_11H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_11H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_12L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_12L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_12H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_12H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_13L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_13L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_13H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_13H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_14L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_14L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_14H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_14H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_15L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_15L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_15H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_15H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_16L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_16L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_16H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_16H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_17L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_17L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_17H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_17H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_18L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_18L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_18H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_18H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_19L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_19L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_19H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_19H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_20L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_20L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_20H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_20H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_21L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_21L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_21H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_21H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_22L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_22L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_22H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_22H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_23L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_23L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_23H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_23H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_24L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_24L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_24H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_24H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_25L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_25L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_25H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_25H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_26L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_26L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_26H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_26H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_27L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_27L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_27H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_27H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_28L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_28L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_28H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_28H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_29L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_29L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_29H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_29H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_30L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_30L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_30H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_30H_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_31L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT0_31L_tag;

typedef union CANXL_FILTER_BANK_ARFLT0_31H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT0_31H_tag;

typedef union CANXL_FILTER_BANK_AFCFG1_union_tag { /* Acceptance Filter Configuration */
  vuint32_t R;
  struct {
    vuint32_t ACPTVCAN:5;              /* Number of VCAN Acceptance Filters */
    vuint32_t _unused_5:2;
    vuint32_t AVCANEN:1;               /* Acceptance VCAN Enable */
    vuint32_t ACPTSDU:5;               /* Number of SDU Acceptance Filters */
    vuint32_t _unused_13:2;
    vuint32_t ASDUEN:1;                /* Acceptance SDU Enable */
    vuint32_t ACPTADDR:5;              /* Number of ADDR Acceptance Filters */
    vuint32_t _unused_21:2;
    vuint32_t AADDREN:1;               /* Acceptance ADDR Enable */
    vuint32_t _unused_24:8;
  } B;
} CANXL_FILTER_BANK_AFCFG1_tag;

typedef union CANXL_FILTER_BANK_VAMRCFG1_union_tag { /* VCAN Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t VMSK0R0:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R1:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R2:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R3:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R4:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R5:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R6:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R7:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R8:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R9:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R10:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R11:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R12:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R13:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R14:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R15:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R16:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R17:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R18:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R19:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R20:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R21:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R22:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R23:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R24:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R25:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R26:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R27:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R28:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R29:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R30:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R31:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_FILTER_BANK_VAMRCFG1_tag;

typedef union CANXL_FILTER_BANK_SAMRCFG1_union_tag { /* SDU Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t SMSK0R0:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R1:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R2:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R3:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R4:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R5:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R6:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R7:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R8:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R9:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R10:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R11:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R12:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R13:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R14:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R15:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R16:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R17:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R18:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R19:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R20:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R21:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R22:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R23:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R24:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R25:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R26:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R27:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R28:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R29:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R30:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R31:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_FILTER_BANK_SAMRCFG1_tag;

typedef union CANXL_FILTER_BANK_AAMRCFG1_union_tag { /* ADDR Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t AMSK0R0:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R1:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R2:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R3:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R4:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R5:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R6:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R7:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R8:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R9:1;               /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R10:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R11:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R12:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R13:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R14:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R15:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R16:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R17:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R18:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R19:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R20:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R21:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R22:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R23:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R24:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R25:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R26:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R27:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R28:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R29:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R30:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R31:1;              /* ADDR filter pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_FILTER_BANK_AAMRCFG1_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_0_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_0_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_2_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_2_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_4_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_4_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_6_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_6_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_8_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_8_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_10_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_10_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_12_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_12_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_14_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_14_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_16_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_16_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_18_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_18_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_20_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_20_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_22_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_22_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_24_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_24_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_26_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_26_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_28_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_28_tag;

typedef union CANXL_FILTER_BANK_VAFLT1_30_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Acceptance Filter High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_VAFLT1_30_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_0_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_0_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_2_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_2_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_4_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_4_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_6_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_6_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_8_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_8_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_10_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_10_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_12_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_12_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_14_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_14_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_16_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_16_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_18_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_18_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_20_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_20_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_22_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_22_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_24_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_24_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_26_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_26_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_28_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_28_tag;

typedef union CANXL_FILTER_BANK_SAFLT1_30_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_SAFLT1_30_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_0L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_0L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_0H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_0H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_1L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_1L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_1H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_1H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_2L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_2L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_2H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_2H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_3L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_3L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_3H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_3H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_4L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_4L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_4H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_4H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_5L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_5L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_5H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_5H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_6L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_6L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_6H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_6H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_7L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_7L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_7H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_7H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_8L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_8L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_8H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_8H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_9L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_9L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_9H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_9H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_10L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_10L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_10H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_10H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_11L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_11L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_11H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_11H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_12L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_12L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_12H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_12H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_13L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_13L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_13H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_13H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_14L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_14L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_14H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_14H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_15L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_15L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_15H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_15H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_16L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_16L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_16H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_16H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_17L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_17L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_17H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_17H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_18L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_18L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_18H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_18H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_19L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_19L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_19H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_19H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_20L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_20L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_20H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_20H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_21L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_21L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_21H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_21H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_22L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_22L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_22H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_22H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_23L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_23L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_23H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_23H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_24L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_24L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_24H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_24H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_25L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_25L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_25H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_25H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_26L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_26L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_26H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_26H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_27L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_27L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_27H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_27H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_28L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_28L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_28H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_28H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_29L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_29L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_29H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_29H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_30L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_30L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_30H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_30H_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_31L_union_tag { /* ADDR Acceptance Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_FILTER_BANK_AAFLT1_31L_tag;

typedef union CANXL_FILTER_BANK_AAFLT1_31H_union_tag { /* ADDR Acceptance Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_FILTER_BANK_AAFLT1_31H_tag;

typedef union CANXL_FILTER_BANK_RFCFG1_union_tag { /* Rejection Filter Configuration */
  vuint32_t R;
  struct {
    vuint32_t REJVCAN:5;               /* Number of VCAN Rejection Filters */
    vuint32_t _unused_5:2;
    vuint32_t RVCANEN:1;               /* Rejection VCAN Enable */
    vuint32_t REJSDU:5;                /* Number of SDU Rejection Filters */
    vuint32_t _unused_13:2;
    vuint32_t RSDUEN:1;                /* Rejection SDU Enable */
    vuint32_t REJADDR:5;               /* Number of ADDR Rejection Filters */
    vuint32_t _unused_21:2;
    vuint32_t RADDREN:1;               /* Rejection ADDR Enable */
    vuint32_t _unused_24:8;
  } B;
} CANXL_FILTER_BANK_RFCFG1_tag;

typedef union CANXL_FILTER_BANK_VRMRCFG1_union_tag { /* VCAN Rejection Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t VMSK0R0:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R1:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R2:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R3:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R4:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R5:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R6:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R7:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R8:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R9:1;               /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R10:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R11:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R12:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R13:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R14:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R15:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R16:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R17:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R18:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R19:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R20:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R21:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R22:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R23:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R24:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R25:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R26:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R27:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R28:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R29:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R30:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t VMSK0R31:1;              /* VCAN Filter Pair Mask/Range Selection n in Rejection Filter */
  } B;
} CANXL_FILTER_BANK_VRMRCFG1_tag;

typedef union CANXL_FILTER_BANK_SRMRCFG1_union_tag { /* SDU Rejection Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t SMSK0R0:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R1:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R2:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R3:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R4:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R5:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R6:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R7:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R8:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R9:1;               /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R10:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R11:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R12:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R13:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R14:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R15:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R16:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R17:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R18:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R19:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R20:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R21:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R22:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R23:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R24:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R25:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R26:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R27:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R28:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R29:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R30:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t SMSK0R31:1;              /* SDU Filter Pair Mask/Range Selection n in Rejection Filter */
  } B;
} CANXL_FILTER_BANK_SRMRCFG1_tag;

typedef union CANXL_FILTER_BANK_ARMRCFG1_union_tag { /* ADDR Rejection Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t AMSK0R0:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R1:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R2:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R3:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R4:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R5:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R6:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R7:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R8:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R9:1;               /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R10:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R11:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R12:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R13:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R14:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R15:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R16:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R17:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R18:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R19:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R20:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R21:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R22:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R23:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R24:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R25:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R26:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R27:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R28:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R29:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R30:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
    vuint32_t AMSK0R31:1;              /* ADDR Filter Pair Mask/Range Selection n in Rejection Filter */
  } B;
} CANXL_FILTER_BANK_ARMRCFG1_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_0_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_0_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_2_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_2_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_4_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_4_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_6_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_6_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_8_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_8_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_10_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_10_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_12_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_12_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_14_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_14_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_16_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_16_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_18_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_18_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_20_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_20_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_22_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_22_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_24_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_24_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_26_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_26_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_28_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_28_tag;

typedef union CANXL_FILTER_BANK_VRFLT1_30_union_tag { /* VCAN Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANA_H:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_L:8;               /* VCAN Rejection Filter Element */
    vuint32_t VCANB_H:8;               /* VCAN Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_VRFLT1_30_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_0_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_0_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_2_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_2_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_4_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_4_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_6_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_6_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_8_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_8_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_10_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_10_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_12_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_12_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_14_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_14_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_16_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_16_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_18_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_18_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_20_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_20_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_22_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_22_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_24_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_24_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_26_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_26_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_28_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_28_tag;

typedef union CANXL_FILTER_BANK_SRFLT1_30_union_tag { /* SDU Rejection Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Rejection Filter */
    vuint32_t SDUA_H:8;                /* SDU Rejection Filter High */
    vuint32_t SDUB_L:8;                /* SDU Rejection Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_SRFLT1_30_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_0L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_0L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_0H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_0H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_1L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_1L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_1H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_1H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_2L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_2L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_2H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_2H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_3L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_3L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_3H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_3H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_4L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_4L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_4H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_4H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_5L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_5L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_5H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_5H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_6L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_6L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_6H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_6H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_7L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_7L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_7H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_7H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_8L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_8L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_8H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_8H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_9L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_9L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_9H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_9H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_10L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_10L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_10H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_10H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_11L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_11L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_11H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_11H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_12L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_12L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_12H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_12H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_13L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_13L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_13H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_13H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_14L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_14L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_14H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_14H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_15L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_15L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_15H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_15H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_16L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_16L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_16H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_16H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_17L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_17L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_17H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_17H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_18L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_18L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_18H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_18H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_19L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_19L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_19H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_19H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_20L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_20L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_20H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_20H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_21L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_21L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_21H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_21H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_22L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_22L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_22H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_22H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_23L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_23L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_23H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_23H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_24L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_24L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_24H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_24H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_25L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_25L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_25H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_25H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_26L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_26L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_26H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_26H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_27L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_27L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_27H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_27H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_28L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_28L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_28H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_28H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_29L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_29L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_29H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_29H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_30L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_30L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_30H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_30H_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_31L_union_tag { /* ADDR Rejection Filter Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Rejection Filter Low */
  } B;
} CANXL_FILTER_BANK_ARFLT1_31L_tag;

typedef union CANXL_FILTER_BANK_ARFLT1_31H_union_tag { /* ADDR Rejection Filter High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Rejection Filter High */
  } B;
} CANXL_FILTER_BANK_ARFLT1_31H_tag;

struct CANXL_FILTER_BANK_tag {
  CANXL_FILTER_BANK_AFCFG0_tag AFCFG0; /* Acceptance Filter Configuration */
  CANXL_FILTER_BANK_VAMRCFG0_tag VAMRCFG0; /* VCAN Acceptance Mask or Range Configuration */
  CANXL_FILTER_BANK_SAMRCFG0_tag SAMRCFG0; /* SDU Acceptance Mask or Range Configuration */
  CANXL_FILTER_BANK_AAMRCFG0_tag AAMRCFG0; /* ADDR Acceptance Mask or Range Configuration */
  uint8_t CANXL_FILTER_BANK_reserved0[12];
  CANXL_FILTER_BANK_VAFLT0_0_tag VAFLT0_0; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_2_tag VAFLT0_2; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_4_tag VAFLT0_4; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_6_tag VAFLT0_6; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_8_tag VAFLT0_8; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_10_tag VAFLT0_10; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_12_tag VAFLT0_12; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_14_tag VAFLT0_14; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_16_tag VAFLT0_16; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_18_tag VAFLT0_18; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_20_tag VAFLT0_20; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_22_tag VAFLT0_22; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_24_tag VAFLT0_24; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_26_tag VAFLT0_26; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_28_tag VAFLT0_28; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT0_30_tag VAFLT0_30; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_0_tag SAFLT0_0; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_2_tag SAFLT0_2; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_4_tag SAFLT0_4; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_6_tag SAFLT0_6; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_8_tag SAFLT0_8; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_10_tag SAFLT0_10; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_12_tag SAFLT0_12; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_14_tag SAFLT0_14; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_16_tag SAFLT0_16; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_18_tag SAFLT0_18; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_20_tag SAFLT0_20; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_22_tag SAFLT0_22; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_24_tag SAFLT0_24; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_26_tag SAFLT0_26; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_28_tag SAFLT0_28; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT0_30_tag SAFLT0_30; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_AAFLT0_0L_tag AAFLT0_0L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_0H_tag AAFLT0_0H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_1L_tag AAFLT0_1L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_1H_tag AAFLT0_1H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_2L_tag AAFLT0_2L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_2H_tag AAFLT0_2H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_3L_tag AAFLT0_3L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_3H_tag AAFLT0_3H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_4L_tag AAFLT0_4L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_4H_tag AAFLT0_4H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_5L_tag AAFLT0_5L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_5H_tag AAFLT0_5H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_6L_tag AAFLT0_6L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_6H_tag AAFLT0_6H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_7L_tag AAFLT0_7L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_7H_tag AAFLT0_7H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_8L_tag AAFLT0_8L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_8H_tag AAFLT0_8H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_9L_tag AAFLT0_9L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_9H_tag AAFLT0_9H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_10L_tag AAFLT0_10L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_10H_tag AAFLT0_10H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_11L_tag AAFLT0_11L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_11H_tag AAFLT0_11H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_12L_tag AAFLT0_12L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_12H_tag AAFLT0_12H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_13L_tag AAFLT0_13L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_13H_tag AAFLT0_13H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_14L_tag AAFLT0_14L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_14H_tag AAFLT0_14H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_15L_tag AAFLT0_15L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_15H_tag AAFLT0_15H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_16L_tag AAFLT0_16L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_16H_tag AAFLT0_16H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_17L_tag AAFLT0_17L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_17H_tag AAFLT0_17H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_18L_tag AAFLT0_18L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_18H_tag AAFLT0_18H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_19L_tag AAFLT0_19L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_19H_tag AAFLT0_19H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_20L_tag AAFLT0_20L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_20H_tag AAFLT0_20H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_21L_tag AAFLT0_21L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_21H_tag AAFLT0_21H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_22L_tag AAFLT0_22L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_22H_tag AAFLT0_22H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_23L_tag AAFLT0_23L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_23H_tag AAFLT0_23H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_24L_tag AAFLT0_24L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_24H_tag AAFLT0_24H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_25L_tag AAFLT0_25L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_25H_tag AAFLT0_25H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_26L_tag AAFLT0_26L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_26H_tag AAFLT0_26H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_27L_tag AAFLT0_27L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_27H_tag AAFLT0_27H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_28L_tag AAFLT0_28L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_28H_tag AAFLT0_28H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_29L_tag AAFLT0_29L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_29H_tag AAFLT0_29H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_30L_tag AAFLT0_30L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_30H_tag AAFLT0_30H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT0_31L_tag AAFLT0_31L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT0_31H_tag AAFLT0_31H; /* ADDR Acceptance Filter High */
  uint8_t CANXL_FILTER_BANK_reserved1[100];
  CANXL_FILTER_BANK_RFCFG0_tag RFCFG0; /* Rejection Filter Configuration */
  CANXL_FILTER_BANK_VRMRCFG0_tag VRMRCFG0; /* VCAN Rejection Mask or Range Configuration */
  CANXL_FILTER_BANK_SRMRCFG0_tag SRMRCFG0; /* SDU Rejection Mask or Range Configuration */
  CANXL_FILTER_BANK_ARMRCFG0_tag ARMRCFG0; /* ADDR Rejection Mask or Range Configuration */
  uint8_t CANXL_FILTER_BANK_reserved2[12];
  CANXL_FILTER_BANK_VRFLT0_0_tag VRFLT0_0; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_2_tag VRFLT0_2; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_4_tag VRFLT0_4; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_6_tag VRFLT0_6; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_8_tag VRFLT0_8; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_10_tag VRFLT0_10; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_12_tag VRFLT0_12; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_14_tag VRFLT0_14; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_16_tag VRFLT0_16; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_18_tag VRFLT0_18; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_20_tag VRFLT0_20; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_22_tag VRFLT0_22; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_24_tag VRFLT0_24; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_26_tag VRFLT0_26; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_28_tag VRFLT0_28; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT0_30_tag VRFLT0_30; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_0_tag SRFLT0_0; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_2_tag SRFLT0_2; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_4_tag SRFLT0_4; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_6_tag SRFLT0_6; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_8_tag SRFLT0_8; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_10_tag SRFLT0_10; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_12_tag SRFLT0_12; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_14_tag SRFLT0_14; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_16_tag SRFLT0_16; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_18_tag SRFLT0_18; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_20_tag SRFLT0_20; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_22_tag SRFLT0_22; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_24_tag SRFLT0_24; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_26_tag SRFLT0_26; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_28_tag SRFLT0_28; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT0_30_tag SRFLT0_30; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_ARFLT0_0L_tag ARFLT0_0L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_0H_tag ARFLT0_0H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_1L_tag ARFLT0_1L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_1H_tag ARFLT0_1H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_2L_tag ARFLT0_2L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_2H_tag ARFLT0_2H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_3L_tag ARFLT0_3L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_3H_tag ARFLT0_3H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_4L_tag ARFLT0_4L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_4H_tag ARFLT0_4H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_5L_tag ARFLT0_5L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_5H_tag ARFLT0_5H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_6L_tag ARFLT0_6L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_6H_tag ARFLT0_6H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_7L_tag ARFLT0_7L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_7H_tag ARFLT0_7H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_8L_tag ARFLT0_8L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_8H_tag ARFLT0_8H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_9L_tag ARFLT0_9L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_9H_tag ARFLT0_9H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_10L_tag ARFLT0_10L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_10H_tag ARFLT0_10H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_11L_tag ARFLT0_11L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_11H_tag ARFLT0_11H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_12L_tag ARFLT0_12L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_12H_tag ARFLT0_12H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_13L_tag ARFLT0_13L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_13H_tag ARFLT0_13H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_14L_tag ARFLT0_14L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_14H_tag ARFLT0_14H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_15L_tag ARFLT0_15L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_15H_tag ARFLT0_15H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_16L_tag ARFLT0_16L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_16H_tag ARFLT0_16H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_17L_tag ARFLT0_17L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_17H_tag ARFLT0_17H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_18L_tag ARFLT0_18L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_18H_tag ARFLT0_18H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_19L_tag ARFLT0_19L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_19H_tag ARFLT0_19H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_20L_tag ARFLT0_20L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_20H_tag ARFLT0_20H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_21L_tag ARFLT0_21L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_21H_tag ARFLT0_21H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_22L_tag ARFLT0_22L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_22H_tag ARFLT0_22H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_23L_tag ARFLT0_23L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_23H_tag ARFLT0_23H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_24L_tag ARFLT0_24L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_24H_tag ARFLT0_24H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_25L_tag ARFLT0_25L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_25H_tag ARFLT0_25H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_26L_tag ARFLT0_26L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_26H_tag ARFLT0_26H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_27L_tag ARFLT0_27L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_27H_tag ARFLT0_27H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_28L_tag ARFLT0_28L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_28H_tag ARFLT0_28H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_29L_tag ARFLT0_29L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_29H_tag ARFLT0_29H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_30L_tag ARFLT0_30L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_30H_tag ARFLT0_30H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT0_31L_tag ARFLT0_31L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT0_31H_tag ARFLT0_31H; /* ADDR Rejection Filter High */
  uint8_t CANXL_FILTER_BANK_reserved3[100];
  CANXL_FILTER_BANK_AFCFG1_tag AFCFG1; /* Acceptance Filter Configuration */
  CANXL_FILTER_BANK_VAMRCFG1_tag VAMRCFG1; /* VCAN Acceptance Mask or Range Configuration */
  CANXL_FILTER_BANK_SAMRCFG1_tag SAMRCFG1; /* SDU Acceptance Mask or Range Configuration */
  CANXL_FILTER_BANK_AAMRCFG1_tag AAMRCFG1; /* ADDR Acceptance Mask or Range Configuration */
  uint8_t CANXL_FILTER_BANK_reserved4[12];
  CANXL_FILTER_BANK_VAFLT1_0_tag VAFLT1_0; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_2_tag VAFLT1_2; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_4_tag VAFLT1_4; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_6_tag VAFLT1_6; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_8_tag VAFLT1_8; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_10_tag VAFLT1_10; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_12_tag VAFLT1_12; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_14_tag VAFLT1_14; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_16_tag VAFLT1_16; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_18_tag VAFLT1_18; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_20_tag VAFLT1_20; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_22_tag VAFLT1_22; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_24_tag VAFLT1_24; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_26_tag VAFLT1_26; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_28_tag VAFLT1_28; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_VAFLT1_30_tag VAFLT1_30; /* VCAN Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_0_tag SAFLT1_0; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_2_tag SAFLT1_2; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_4_tag SAFLT1_4; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_6_tag SAFLT1_6; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_8_tag SAFLT1_8; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_10_tag SAFLT1_10; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_12_tag SAFLT1_12; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_14_tag SAFLT1_14; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_16_tag SAFLT1_16; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_18_tag SAFLT1_18; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_20_tag SAFLT1_20; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_22_tag SAFLT1_22; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_24_tag SAFLT1_24; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_26_tag SAFLT1_26; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_28_tag SAFLT1_28; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_SAFLT1_30_tag SAFLT1_30; /* SDU Acceptance Filter */
  CANXL_FILTER_BANK_AAFLT1_0L_tag AAFLT1_0L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_0H_tag AAFLT1_0H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_1L_tag AAFLT1_1L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_1H_tag AAFLT1_1H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_2L_tag AAFLT1_2L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_2H_tag AAFLT1_2H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_3L_tag AAFLT1_3L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_3H_tag AAFLT1_3H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_4L_tag AAFLT1_4L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_4H_tag AAFLT1_4H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_5L_tag AAFLT1_5L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_5H_tag AAFLT1_5H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_6L_tag AAFLT1_6L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_6H_tag AAFLT1_6H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_7L_tag AAFLT1_7L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_7H_tag AAFLT1_7H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_8L_tag AAFLT1_8L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_8H_tag AAFLT1_8H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_9L_tag AAFLT1_9L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_9H_tag AAFLT1_9H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_10L_tag AAFLT1_10L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_10H_tag AAFLT1_10H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_11L_tag AAFLT1_11L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_11H_tag AAFLT1_11H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_12L_tag AAFLT1_12L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_12H_tag AAFLT1_12H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_13L_tag AAFLT1_13L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_13H_tag AAFLT1_13H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_14L_tag AAFLT1_14L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_14H_tag AAFLT1_14H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_15L_tag AAFLT1_15L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_15H_tag AAFLT1_15H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_16L_tag AAFLT1_16L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_16H_tag AAFLT1_16H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_17L_tag AAFLT1_17L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_17H_tag AAFLT1_17H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_18L_tag AAFLT1_18L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_18H_tag AAFLT1_18H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_19L_tag AAFLT1_19L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_19H_tag AAFLT1_19H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_20L_tag AAFLT1_20L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_20H_tag AAFLT1_20H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_21L_tag AAFLT1_21L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_21H_tag AAFLT1_21H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_22L_tag AAFLT1_22L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_22H_tag AAFLT1_22H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_23L_tag AAFLT1_23L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_23H_tag AAFLT1_23H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_24L_tag AAFLT1_24L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_24H_tag AAFLT1_24H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_25L_tag AAFLT1_25L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_25H_tag AAFLT1_25H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_26L_tag AAFLT1_26L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_26H_tag AAFLT1_26H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_27L_tag AAFLT1_27L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_27H_tag AAFLT1_27H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_28L_tag AAFLT1_28L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_28H_tag AAFLT1_28H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_29L_tag AAFLT1_29L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_29H_tag AAFLT1_29H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_30L_tag AAFLT1_30L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_30H_tag AAFLT1_30H; /* ADDR Acceptance Filter High */
  CANXL_FILTER_BANK_AAFLT1_31L_tag AAFLT1_31L; /* ADDR Acceptance Filter Low */
  CANXL_FILTER_BANK_AAFLT1_31H_tag AAFLT1_31H; /* ADDR Acceptance Filter High */
  uint8_t CANXL_FILTER_BANK_reserved5[100];
  CANXL_FILTER_BANK_RFCFG1_tag RFCFG1; /* Rejection Filter Configuration */
  CANXL_FILTER_BANK_VRMRCFG1_tag VRMRCFG1; /* VCAN Rejection Mask or Range Configuration */
  CANXL_FILTER_BANK_SRMRCFG1_tag SRMRCFG1; /* SDU Rejection Mask or Range Configuration */
  CANXL_FILTER_BANK_ARMRCFG1_tag ARMRCFG1; /* ADDR Rejection Mask or Range Configuration */
  uint8_t CANXL_FILTER_BANK_reserved6[12];
  CANXL_FILTER_BANK_VRFLT1_0_tag VRFLT1_0; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_2_tag VRFLT1_2; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_4_tag VRFLT1_4; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_6_tag VRFLT1_6; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_8_tag VRFLT1_8; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_10_tag VRFLT1_10; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_12_tag VRFLT1_12; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_14_tag VRFLT1_14; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_16_tag VRFLT1_16; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_18_tag VRFLT1_18; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_20_tag VRFLT1_20; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_22_tag VRFLT1_22; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_24_tag VRFLT1_24; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_26_tag VRFLT1_26; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_28_tag VRFLT1_28; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_VRFLT1_30_tag VRFLT1_30; /* VCAN Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_0_tag SRFLT1_0; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_2_tag SRFLT1_2; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_4_tag SRFLT1_4; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_6_tag SRFLT1_6; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_8_tag SRFLT1_8; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_10_tag SRFLT1_10; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_12_tag SRFLT1_12; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_14_tag SRFLT1_14; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_16_tag SRFLT1_16; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_18_tag SRFLT1_18; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_20_tag SRFLT1_20; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_22_tag SRFLT1_22; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_24_tag SRFLT1_24; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_26_tag SRFLT1_26; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_28_tag SRFLT1_28; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_SRFLT1_30_tag SRFLT1_30; /* SDU Rejection Filter */
  CANXL_FILTER_BANK_ARFLT1_0L_tag ARFLT1_0L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_0H_tag ARFLT1_0H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_1L_tag ARFLT1_1L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_1H_tag ARFLT1_1H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_2L_tag ARFLT1_2L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_2H_tag ARFLT1_2H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_3L_tag ARFLT1_3L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_3H_tag ARFLT1_3H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_4L_tag ARFLT1_4L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_4H_tag ARFLT1_4H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_5L_tag ARFLT1_5L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_5H_tag ARFLT1_5H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_6L_tag ARFLT1_6L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_6H_tag ARFLT1_6H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_7L_tag ARFLT1_7L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_7H_tag ARFLT1_7H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_8L_tag ARFLT1_8L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_8H_tag ARFLT1_8H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_9L_tag ARFLT1_9L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_9H_tag ARFLT1_9H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_10L_tag ARFLT1_10L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_10H_tag ARFLT1_10H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_11L_tag ARFLT1_11L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_11H_tag ARFLT1_11H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_12L_tag ARFLT1_12L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_12H_tag ARFLT1_12H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_13L_tag ARFLT1_13L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_13H_tag ARFLT1_13H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_14L_tag ARFLT1_14L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_14H_tag ARFLT1_14H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_15L_tag ARFLT1_15L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_15H_tag ARFLT1_15H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_16L_tag ARFLT1_16L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_16H_tag ARFLT1_16H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_17L_tag ARFLT1_17L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_17H_tag ARFLT1_17H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_18L_tag ARFLT1_18L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_18H_tag ARFLT1_18H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_19L_tag ARFLT1_19L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_19H_tag ARFLT1_19H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_20L_tag ARFLT1_20L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_20H_tag ARFLT1_20H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_21L_tag ARFLT1_21L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_21H_tag ARFLT1_21H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_22L_tag ARFLT1_22L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_22H_tag ARFLT1_22H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_23L_tag ARFLT1_23L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_23H_tag ARFLT1_23H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_24L_tag ARFLT1_24L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_24H_tag ARFLT1_24H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_25L_tag ARFLT1_25L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_25H_tag ARFLT1_25H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_26L_tag ARFLT1_26L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_26H_tag ARFLT1_26H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_27L_tag ARFLT1_27L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_27H_tag ARFLT1_27H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_28L_tag ARFLT1_28L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_28H_tag ARFLT1_28H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_29L_tag ARFLT1_29L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_29H_tag ARFLT1_29H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_30L_tag ARFLT1_30L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_30H_tag ARFLT1_30H; /* ADDR Rejection Filter High */
  CANXL_FILTER_BANK_ARFLT1_31L_tag ARFLT1_31L; /* ADDR Rejection Filter Low */
  CANXL_FILTER_BANK_ARFLT1_31H_tag ARFLT1_31H; /* ADDR Rejection Filter High */
};


/* ============================================================================
   =============================== Module: CANXL_GRP_CONTROL ==================
   ============================================================================ */

typedef union CANXL_GRP_CONTROL_DSCCTRL_union_tag { /* Descriptors Control */
  vuint32_t R;
  struct {
    vuint32_t TXDSC:7;                 /* Tx Descriptors Setup */
    vuint32_t _unused_7:25;
  } B;
} CANXL_GRP_CONTROL_DSCCTRL_tag;

typedef union CANXL_GRP_CONTROL_DSCACTIVE1_union_tag { /* Descriptor Activation 1 */
  vuint32_t R;
  struct {
    vuint32_t DCACT0:1;                /* Descriptor Active */
    vuint32_t DCACT1:1;                /* Descriptor Active */
    vuint32_t DCACT2:1;                /* Descriptor Active */
    vuint32_t DCACT3:1;                /* Descriptor Active */
    vuint32_t DCACT4:1;                /* Descriptor Active */
    vuint32_t DCACT5:1;                /* Descriptor Active */
    vuint32_t DCACT6:1;                /* Descriptor Active */
    vuint32_t DCACT7:1;                /* Descriptor Active */
    vuint32_t DCACT8:1;                /* Descriptor Active */
    vuint32_t DCACT9:1;                /* Descriptor Active */
    vuint32_t DCACT10:1;               /* Descriptor Active */
    vuint32_t DCACT11:1;               /* Descriptor Active */
    vuint32_t DCACT12:1;               /* Descriptor Active */
    vuint32_t DCACT13:1;               /* Descriptor Active */
    vuint32_t DCACT14:1;               /* Descriptor Active */
    vuint32_t DCACT15:1;               /* Descriptor Active */
    vuint32_t DCACT16:1;               /* Descriptor Active */
    vuint32_t DCACT17:1;               /* Descriptor Active */
    vuint32_t DCACT18:1;               /* Descriptor Active */
    vuint32_t DCACT19:1;               /* Descriptor Active */
    vuint32_t DCACT20:1;               /* Descriptor Active */
    vuint32_t DCACT21:1;               /* Descriptor Active */
    vuint32_t DCACT22:1;               /* Descriptor Active */
    vuint32_t DCACT23:1;               /* Descriptor Active */
    vuint32_t DCACT24:1;               /* Descriptor Active */
    vuint32_t DCACT25:1;               /* Descriptor Active */
    vuint32_t DCACT26:1;               /* Descriptor Active */
    vuint32_t DCACT27:1;               /* Descriptor Active */
    vuint32_t DCACT28:1;               /* Descriptor Active */
    vuint32_t DCACT29:1;               /* Descriptor Active */
    vuint32_t DCACT30:1;               /* Descriptor Active */
    vuint32_t DCACT31:1;               /* Descriptor Active */
  } B;
} CANXL_GRP_CONTROL_DSCACTIVE1_tag;

typedef union CANXL_GRP_CONTROL_DSCACTIVE2_union_tag { /* Descriptor Activation 2 */
  vuint32_t R;
  struct {
    vuint32_t DCACT32:1;               /* Descriptor Active */
    vuint32_t DCACT33:1;               /* Descriptor Active */
    vuint32_t DCACT34:1;               /* Descriptor Active */
    vuint32_t DCACT35:1;               /* Descriptor Active */
    vuint32_t DCACT36:1;               /* Descriptor Active */
    vuint32_t DCACT37:1;               /* Descriptor Active */
    vuint32_t DCACT38:1;               /* Descriptor Active */
    vuint32_t DCACT39:1;               /* Descriptor Active */
    vuint32_t DCACT40:1;               /* Descriptor Active */
    vuint32_t DCACT41:1;               /* Descriptor Active */
    vuint32_t DCACT42:1;               /* Descriptor Active */
    vuint32_t DCACT43:1;               /* Descriptor Active */
    vuint32_t DCACT44:1;               /* Descriptor Active */
    vuint32_t DCACT45:1;               /* Descriptor Active */
    vuint32_t DCACT46:1;               /* Descriptor Active */
    vuint32_t DCACT47:1;               /* Descriptor Active */
    vuint32_t DCACT48:1;               /* Descriptor Active */
    vuint32_t DCACT49:1;               /* Descriptor Active */
    vuint32_t DCACT50:1;               /* Descriptor Active */
    vuint32_t DCACT51:1;               /* Descriptor Active */
    vuint32_t DCACT52:1;               /* Descriptor Active */
    vuint32_t DCACT53:1;               /* Descriptor Active */
    vuint32_t DCACT54:1;               /* Descriptor Active */
    vuint32_t DCACT55:1;               /* Descriptor Active */
    vuint32_t DCACT56:1;               /* Descriptor Active */
    vuint32_t DCACT57:1;               /* Descriptor Active */
    vuint32_t DCACT58:1;               /* Descriptor Active */
    vuint32_t DCACT59:1;               /* Descriptor Active */
    vuint32_t DCACT60:1;               /* Descriptor Active */
    vuint32_t DCACT61:1;               /* Descriptor Active */
    vuint32_t DCACT62:1;               /* Descriptor Active */
    vuint32_t DCACT63:1;               /* Descriptor Active */
  } B;
} CANXL_GRP_CONTROL_DSCACTIVE2_tag;

typedef union CANXL_GRP_CONTROL_DSCACTIVE3_union_tag { /* Descriptor Activation 3 */
  vuint32_t R;
  struct {
    vuint32_t DCACT64:1;               /* Descriptor Active */
    vuint32_t DCACT65:1;               /* Descriptor Active */
    vuint32_t DCACT66:1;               /* Descriptor Active */
    vuint32_t DCACT67:1;               /* Descriptor Active */
    vuint32_t DCACT68:1;               /* Descriptor Active */
    vuint32_t DCACT69:1;               /* Descriptor Active */
    vuint32_t DCACT70:1;               /* Descriptor Active */
    vuint32_t DCACT71:1;               /* Descriptor Active */
    vuint32_t DCACT72:1;               /* Descriptor Active */
    vuint32_t DCACT73:1;               /* Descriptor Active */
    vuint32_t DCACT74:1;               /* Descriptor Active */
    vuint32_t DCACT75:1;               /* Descriptor Active */
    vuint32_t DCACT76:1;               /* Descriptor Active */
    vuint32_t DCACT77:1;               /* Descriptor Active */
    vuint32_t DCACT78:1;               /* Descriptor Active */
    vuint32_t DCACT79:1;               /* Descriptor Active */
    vuint32_t DCACT80:1;               /* Descriptor Active */
    vuint32_t DCACT81:1;               /* Descriptor Active */
    vuint32_t DCACT82:1;               /* Descriptor Active */
    vuint32_t DCACT83:1;               /* Descriptor Active */
    vuint32_t DCACT84:1;               /* Descriptor Active */
    vuint32_t DCACT85:1;               /* Descriptor Active */
    vuint32_t DCACT86:1;               /* Descriptor Active */
    vuint32_t DCACT87:1;               /* Descriptor Active */
    vuint32_t DCACT88:1;               /* Descriptor Active */
    vuint32_t DCACT89:1;               /* Descriptor Active */
    vuint32_t DCACT90:1;               /* Descriptor Active */
    vuint32_t DCACT91:1;               /* Descriptor Active */
    vuint32_t DCACT92:1;               /* Descriptor Active */
    vuint32_t DCACT93:1;               /* Descriptor Active */
    vuint32_t DCACT94:1;               /* Descriptor Active */
    vuint32_t DCACT95:1;               /* Descriptor Active */
  } B;
} CANXL_GRP_CONTROL_DSCACTIVE3_tag;

typedef union CANXL_GRP_CONTROL_DSCACTIVE4_union_tag { /* Descriptor Activation 4 */
  vuint32_t R;
  struct {
    vuint32_t DCACT96:1;               /* Descriptor Active */
    vuint32_t DCACT97:1;               /* Descriptor Active */
    vuint32_t DCACT98:1;               /* Descriptor Active */
    vuint32_t DCACT99:1;               /* Descriptor Active */
    vuint32_t DCACT100:1;              /* Descriptor Active */
    vuint32_t DCACT101:1;              /* Descriptor Active */
    vuint32_t DCACT102:1;              /* Descriptor Active */
    vuint32_t DCACT103:1;              /* Descriptor Active */
    vuint32_t DCACT104:1;              /* Descriptor Active */
    vuint32_t DCACT105:1;              /* Descriptor Active */
    vuint32_t DCACT106:1;              /* Descriptor Active */
    vuint32_t DCACT107:1;              /* Descriptor Active */
    vuint32_t DCACT108:1;              /* Descriptor Active */
    vuint32_t DCACT109:1;              /* Descriptor Active */
    vuint32_t DCACT110:1;              /* Descriptor Active */
    vuint32_t DCACT111:1;              /* Descriptor Active */
    vuint32_t DCACT112:1;              /* Descriptor Active */
    vuint32_t DCACT113:1;              /* Descriptor Active */
    vuint32_t DCACT114:1;              /* Descriptor Active */
    vuint32_t DCACT115:1;              /* Descriptor Active */
    vuint32_t DCACT116:1;              /* Descriptor Active */
    vuint32_t DCACT117:1;              /* Descriptor Active */
    vuint32_t DCACT118:1;              /* Descriptor Active */
    vuint32_t DCACT119:1;              /* Descriptor Active */
    vuint32_t DCACT120:1;              /* Descriptor Active */
    vuint32_t DCACT121:1;              /* Descriptor Active */
    vuint32_t DCACT122:1;              /* Descriptor Active */
    vuint32_t DCACT123:1;              /* Descriptor Active */
    vuint32_t DCACT124:1;              /* Descriptor Active */
    vuint32_t DCACT125:1;              /* Descriptor Active */
    vuint32_t DCACT126:1;              /* Descriptor Active */
    vuint32_t DCACT127:1;              /* Descriptor Active */
  } B;
} CANXL_GRP_CONTROL_DSCACTIVE4_tag;

typedef union CANXL_GRP_CONTROL_FIFOCTRL1_union_tag { /* Message FIFO Control 1 */
  vuint32_t R;
  struct {
    vuint32_t FIFODPH1:4;              /* Message FIFO Depth 1 */
    vuint32_t _unused_4:4;
    vuint32_t FIFOWTM1:4;              /* Message FIFO Watermark 1 */
    vuint32_t _unused_12:4;
    vuint32_t FIFODPH2:4;              /* Message FIFO Depth 2 */
    vuint32_t _unused_20:4;
    vuint32_t FIFOWTM2:4;              /* Message FIFO Watermark 2 */
    vuint32_t _unused_28:4;
  } B;
} CANXL_GRP_CONTROL_FIFOCTRL1_tag;

typedef union CANXL_GRP_CONTROL_FIFOCTRL2_union_tag { /* Message FIFO Control 2 */
  vuint32_t R;
  struct {
    vuint32_t FIFODPH3:4;              /* Message FIFO Depth 3 */
    vuint32_t _unused_4:4;
    vuint32_t FIFOWTM3:4;              /* Message FIFO Watermark 3 */
    vuint32_t _unused_12:4;
    vuint32_t FIFODPH4:4;              /* Message FIFO Depth 4 */
    vuint32_t _unused_20:4;
    vuint32_t FIFOWTM4:4;              /* Message FIFO Watermark 4 */
    vuint32_t _unused_28:4;
  } B;
} CANXL_GRP_CONTROL_FIFOCTRL2_tag;

typedef union CANXL_GRP_CONTROL_FIFOCTRL3_union_tag { /* Message FIFO Control 3 */
  vuint32_t R;
  struct {
    vuint32_t FIFODPH5:4;              /* Message FIFO Depth 5 */
    vuint32_t _unused_4:4;
    vuint32_t FIFOWTM5:4;              /* Message FIFO Watermark 5 */
    vuint32_t _unused_12:4;
    vuint32_t FIFODPH6:4;              /* Message FIFO Depth 6 */
    vuint32_t _unused_20:4;
    vuint32_t FIFOWTM6:4;              /* Message FIFO Watermark 6 */
    vuint32_t _unused_28:4;
  } B;
} CANXL_GRP_CONTROL_FIFOCTRL3_tag;

typedef union CANXL_GRP_CONTROL_FIFOCTRL4_union_tag { /* Message FIFO Control 4 */
  vuint32_t R;
  struct {
    vuint32_t FIFODPH7:4;              /* Message FIFO Depth 7 */
    vuint32_t _unused_4:4;
    vuint32_t FIFOWTM7:4;              /* Message FIFO Watermark 7 */
    vuint32_t _unused_12:4;
    vuint32_t FIFODPH8:4;              /* Message FIFO Depth 8 */
    vuint32_t _unused_20:4;
    vuint32_t FIFOWTM8:4;              /* Message FIFO Watermark 8 */
    vuint32_t _unused_28:4;
  } B;
} CANXL_GRP_CONTROL_FIFOCTRL4_tag;

typedef union CANXL_GRP_CONTROL_MSGIFLAG1_union_tag { /* Message Interrupt FLAG 1 */
  vuint32_t R;
  struct {
    vuint32_t MSGFLAG0:1;              /* Message Flag n */
    vuint32_t MSGFLAG1:1;              /* Message Flag n */
    vuint32_t MSGFLAG2:1;              /* Message Flag n */
    vuint32_t MSGFLAG3:1;              /* Message Flag n */
    vuint32_t MSGFLAG4:1;              /* Message Flag n */
    vuint32_t MSGFLAG5:1;              /* Message Flag n */
    vuint32_t MSGFLAG6:1;              /* Message Flag n */
    vuint32_t MSGFLAG7:1;              /* Message Flag n */
    vuint32_t MSGFLAG8:1;              /* Message Flag n */
    vuint32_t MSGFLAG9:1;              /* Message Flag n */
    vuint32_t MSGFLAG10:1;             /* Message Flag n */
    vuint32_t MSGFLAG11:1;             /* Message Flag n */
    vuint32_t MSGFLAG12:1;             /* Message Flag n */
    vuint32_t MSGFLAG13:1;             /* Message Flag n */
    vuint32_t MSGFLAG14:1;             /* Message Flag n */
    vuint32_t MSGFLAG15:1;             /* Message Flag n */
    vuint32_t MSGFLAG16:1;             /* Message Flag n */
    vuint32_t MSGFLAG17:1;             /* Message Flag n */
    vuint32_t MSGFLAG18:1;             /* Message Flag n */
    vuint32_t MSGFLAG19:1;             /* Message Flag n */
    vuint32_t MSGFLAG20:1;             /* Message Flag n */
    vuint32_t MSGFLAG21:1;             /* Message Flag n */
    vuint32_t MSGFLAG22:1;             /* Message Flag n */
    vuint32_t MSGFLAG23:1;             /* Message Flag n */
    vuint32_t MSGFLAG24:1;             /* Message Flag n */
    vuint32_t MSGFLAG25:1;             /* Message Flag n */
    vuint32_t MSGFLAG26:1;             /* Message Flag n */
    vuint32_t MSGFLAG27:1;             /* Message Flag n */
    vuint32_t MSGFLAG28:1;             /* Message Flag n */
    vuint32_t MSGFLAG29:1;             /* Message Flag n */
    vuint32_t MSGFLAG30:1;             /* Message Flag n */
    vuint32_t MSGFLAG31:1;             /* Message Flag n */
  } B;
} CANXL_GRP_CONTROL_MSGIFLAG1_tag;

typedef union CANXL_GRP_CONTROL_MSGIFLAG2_union_tag { /* Message Interrupt FLAG 2 */
  vuint32_t R;
  struct {
    vuint32_t MSGFLAG32:1;             /* Message Flag n */
    vuint32_t MSGFLAG33:1;             /* Message Flag n */
    vuint32_t MSGFLAG34:1;             /* Message Flag n */
    vuint32_t MSGFLAG35:1;             /* Message Flag n */
    vuint32_t MSGFLAG36:1;             /* Message Flag n */
    vuint32_t MSGFLAG37:1;             /* Message Flag n */
    vuint32_t MSGFLAG38:1;             /* Message Flag n */
    vuint32_t MSGFLAG39:1;             /* Message Flag n */
    vuint32_t MSGFLAG40:1;             /* Message Flag n */
    vuint32_t MSGFLAG41:1;             /* Message Flag n */
    vuint32_t MSGFLAG42:1;             /* Message Flag n */
    vuint32_t MSGFLAG43:1;             /* Message Flag n */
    vuint32_t MSGFLAG44:1;             /* Message Flag n */
    vuint32_t MSGFLAG45:1;             /* Message Flag n */
    vuint32_t MSGFLAG46:1;             /* Message Flag n */
    vuint32_t MSGFLAG47:1;             /* Message Flag n */
    vuint32_t MSGFLAG48:1;             /* Message Flag n */
    vuint32_t MSGFLAG49:1;             /* Message Flag n */
    vuint32_t MSGFLAG50:1;             /* Message Flag n */
    vuint32_t MSGFLAG51:1;             /* Message Flag n */
    vuint32_t MSGFLAG52:1;             /* Message Flag n */
    vuint32_t MSGFLAG53:1;             /* Message Flag n */
    vuint32_t MSGFLAG54:1;             /* Message Flag n */
    vuint32_t MSGFLAG55:1;             /* Message Flag n */
    vuint32_t MSGFLAG56:1;             /* Message Flag n */
    vuint32_t MSGFLAG57:1;             /* Message Flag n */
    vuint32_t MSGFLAG58:1;             /* Message Flag n */
    vuint32_t MSGFLAG59:1;             /* Message Flag n */
    vuint32_t MSGFLAG60:1;             /* Message Flag n */
    vuint32_t MSGFLAG61:1;             /* Message Flag n */
    vuint32_t MSGFLAG62:1;             /* Message Flag n */
    vuint32_t MSGFLAG63:1;             /* Message Flag n */
  } B;
} CANXL_GRP_CONTROL_MSGIFLAG2_tag;

typedef union CANXL_GRP_CONTROL_MSGIFLAG3_union_tag { /* Message Interrupt FLAG 3 */
  vuint32_t R;
  struct {
    vuint32_t MSGFLAG64:1;             /* Message Flag n */
    vuint32_t MSGFLAG65:1;             /* Message Flag n */
    vuint32_t MSGFLAG66:1;             /* Message Flag n */
    vuint32_t MSGFLAG67:1;             /* Message Flag n */
    vuint32_t MSGFLAG68:1;             /* Message Flag n */
    vuint32_t MSGFLAG69:1;             /* Message Flag n */
    vuint32_t MSGFLAG70:1;             /* Message Flag n */
    vuint32_t MSGFLAG71:1;             /* Message Flag n */
    vuint32_t MSGFLAG72:1;             /* Message Flag n */
    vuint32_t MSGFLAG73:1;             /* Message Flag n */
    vuint32_t MSGFLAG74:1;             /* Message Flag n */
    vuint32_t MSGFLAG75:1;             /* Message Flag n */
    vuint32_t MSGFLAG76:1;             /* Message Flag n */
    vuint32_t MSGFLAG77:1;             /* Message Flag n */
    vuint32_t MSGFLAG78:1;             /* Message Flag n */
    vuint32_t MSGFLAG79:1;             /* Message Flag n */
    vuint32_t MSGFLAG80:1;             /* Message Flag n */
    vuint32_t MSGFLAG81:1;             /* Message Flag n */
    vuint32_t MSGFLAG82:1;             /* Message Flag n */
    vuint32_t MSGFLAG83:1;             /* Message Flag n */
    vuint32_t MSGFLAG84:1;             /* Message Flag n */
    vuint32_t MSGFLAG85:1;             /* Message Flag n */
    vuint32_t MSGFLAG86:1;             /* Message Flag n */
    vuint32_t MSGFLAG87:1;             /* Message Flag n */
    vuint32_t MSGFLAG88:1;             /* Message Flag n */
    vuint32_t MSGFLAG89:1;             /* Message Flag n */
    vuint32_t MSGFLAG90:1;             /* Message Flag n */
    vuint32_t MSGFLAG91:1;             /* Message Flag n */
    vuint32_t MSGFLAG92:1;             /* Message Flag n */
    vuint32_t MSGFLAG93:1;             /* Message Flag n */
    vuint32_t MSGFLAG94:1;             /* Message Flag n */
    vuint32_t MSGFLAG95:1;             /* Message Flag n */
  } B;
} CANXL_GRP_CONTROL_MSGIFLAG3_tag;

typedef union CANXL_GRP_CONTROL_MSGIFLAG4_union_tag { /* Message Interrupt FLAG 4 */
  vuint32_t R;
  struct {
    vuint32_t MSGFLAG96:1;             /* Message Flag n */
    vuint32_t MSGFLAG97:1;             /* Message Flag n */
    vuint32_t MSGFLAG98:1;             /* Message Flag n */
    vuint32_t MSGFLAG99:1;             /* Message Flag n */
    vuint32_t MSGFLAG100:1;            /* Message Flag n */
    vuint32_t MSGFLAG101:1;            /* Message Flag n */
    vuint32_t MSGFLAG102:1;            /* Message Flag n */
    vuint32_t MSGFLAG103:1;            /* Message Flag n */
    vuint32_t MSGFLAG104:1;            /* Message Flag n */
    vuint32_t MSGFLAG105:1;            /* Message Flag n */
    vuint32_t MSGFLAG106:1;            /* Message Flag n */
    vuint32_t MSGFLAG107:1;            /* Message Flag n */
    vuint32_t MSGFLAG108:1;            /* Message Flag n */
    vuint32_t MSGFLAG109:1;            /* Message Flag n */
    vuint32_t MSGFLAG110:1;            /* Message Flag n */
    vuint32_t MSGFLAG111:1;            /* Message Flag n */
    vuint32_t MSGFLAG112:1;            /* Message Flag n */
    vuint32_t MSGFLAG113:1;            /* Message Flag n */
    vuint32_t MSGFLAG114:1;            /* Message Flag n */
    vuint32_t MSGFLAG115:1;            /* Message Flag n */
    vuint32_t MSGFLAG116:1;            /* Message Flag n */
    vuint32_t MSGFLAG117:1;            /* Message Flag n */
    vuint32_t MSGFLAG118:1;            /* Message Flag n */
    vuint32_t MSGFLAG119:1;            /* Message Flag n */
    vuint32_t MSGFLAG120:1;            /* Message Flag n */
    vuint32_t MSGFLAG121:1;            /* Message Flag n */
    vuint32_t MSGFLAG122:1;            /* Message Flag n */
    vuint32_t MSGFLAG123:1;            /* Message Flag n */
    vuint32_t MSGFLAG124:1;            /* Message Flag n */
    vuint32_t MSGFLAG125:1;            /* Message Flag n */
    vuint32_t MSGFLAG126:1;            /* Message Flag n */
    vuint32_t MSGFLAG127:1;            /* Message Flag n */
  } B;
} CANXL_GRP_CONTROL_MSGIFLAG4_tag;

typedef union CANXL_GRP_CONTROL_MSGIMASK1_union_tag { /* Message Interrupt Mask 1 */
  vuint32_t R;
  struct {
    vuint32_t MSGDIM0:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM1:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM2:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM3:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM4:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM5:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM6:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM7:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM8:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM9:1;               /* Message Done Interrupt Mask */
    vuint32_t MSGDIM10:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM11:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM12:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM13:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM14:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM15:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM16:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM17:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM18:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM19:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM20:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM21:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM22:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM23:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM24:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM25:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM26:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM27:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM28:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM29:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM30:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM31:1;              /* Message Done Interrupt Mask */
  } B;
} CANXL_GRP_CONTROL_MSGIMASK1_tag;

typedef union CANXL_GRP_CONTROL_MSGIMASK2_union_tag { /* Message Interrupt Mask 2 */
  vuint32_t R;
  struct {
    vuint32_t MSGDIM32:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM33:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM34:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM35:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM36:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM37:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM38:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM39:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM40:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM41:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM42:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM43:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM44:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM45:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM46:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM47:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM48:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM49:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM50:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM51:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM52:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM53:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM54:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM55:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM56:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM57:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM58:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM59:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM60:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM61:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM62:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM63:1;              /* Message Done Interrupt Mask */
  } B;
} CANXL_GRP_CONTROL_MSGIMASK2_tag;

typedef union CANXL_GRP_CONTROL_MSGIMASK3_union_tag { /* Message Interrupt Mask 3 */
  vuint32_t R;
  struct {
    vuint32_t MSGDIM64:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM65:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM66:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM67:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM68:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM69:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM70:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM71:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM72:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM73:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM74:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM75:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM76:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM77:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM78:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM79:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM80:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM81:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM82:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM83:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM84:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM85:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM86:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM87:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM88:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM89:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM90:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM91:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM92:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM93:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM94:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM95:1;              /* Message Done Interrupt Mask */
  } B;
} CANXL_GRP_CONTROL_MSGIMASK3_tag;

typedef union CANXL_GRP_CONTROL_MSGIMASK4_union_tag { /* Message Interrupt Mask 4 */
  vuint32_t R;
  struct {
    vuint32_t MSGDIM96:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM97:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM98:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM99:1;              /* Message Done Interrupt Mask */
    vuint32_t MSGDIM100:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM101:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM102:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM103:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM104:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM105:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM106:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM107:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM108:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM109:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM110:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM111:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM112:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM113:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM114:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM115:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM116:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM117:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM118:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM119:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM120:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM121:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM122:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM123:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM124:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM125:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM126:1;             /* Message Done Interrupt Mask */
    vuint32_t MSGDIM127:1;             /* Message Done Interrupt Mask */
  } B;
} CANXL_GRP_CONTROL_MSGIMASK4_tag;

typedef union CANXL_GRP_CONTROL_FREEDSC0_union_tag { /* Free Descriptor Index 0 */
  vuint32_t R;
  struct {
    vuint32_t TMWMRK:3;                /* TX MSG Watermark */
    vuint32_t _unused_3:29;
  } B;
} CANXL_GRP_CONTROL_FREEDSC0_tag;

typedef union CANXL_GRP_CONTROL_OVERRUNF1_union_tag { /* Descriptor Overrun Flag 1 */
  vuint32_t R;
  struct {
    vuint32_t OVRRF0:1;                /* Overrun Flag */
    vuint32_t OVRRF1:1;                /* Overrun Flag */
    vuint32_t OVRRF2:1;                /* Overrun Flag */
    vuint32_t OVRRF3:1;                /* Overrun Flag */
    vuint32_t OVRRF4:1;                /* Overrun Flag */
    vuint32_t OVRRF5:1;                /* Overrun Flag */
    vuint32_t OVRRF6:1;                /* Overrun Flag */
    vuint32_t OVRRF7:1;                /* Overrun Flag */
    vuint32_t OVRRF8:1;                /* Overrun Flag */
    vuint32_t OVRRF9:1;                /* Overrun Flag */
    vuint32_t OVRRF10:1;               /* Overrun Flag */
    vuint32_t OVRRF11:1;               /* Overrun Flag */
    vuint32_t OVRRF12:1;               /* Overrun Flag */
    vuint32_t OVRRF13:1;               /* Overrun Flag */
    vuint32_t OVRRF14:1;               /* Overrun Flag */
    vuint32_t OVRRF15:1;               /* Overrun Flag */
    vuint32_t OVRRF16:1;               /* Overrun Flag */
    vuint32_t OVRRF17:1;               /* Overrun Flag */
    vuint32_t OVRRF18:1;               /* Overrun Flag */
    vuint32_t OVRRF19:1;               /* Overrun Flag */
    vuint32_t OVRRF20:1;               /* Overrun Flag */
    vuint32_t OVRRF21:1;               /* Overrun Flag */
    vuint32_t OVRRF22:1;               /* Overrun Flag */
    vuint32_t OVRRF23:1;               /* Overrun Flag */
    vuint32_t OVRRF24:1;               /* Overrun Flag */
    vuint32_t OVRRF25:1;               /* Overrun Flag */
    vuint32_t OVRRF26:1;               /* Overrun Flag */
    vuint32_t OVRRF27:1;               /* Overrun Flag */
    vuint32_t OVRRF28:1;               /* Overrun Flag */
    vuint32_t OVRRF29:1;               /* Overrun Flag */
    vuint32_t OVRRF30:1;               /* Overrun Flag */
    vuint32_t OVRRF31:1;               /* Overrun Flag */
  } B;
} CANXL_GRP_CONTROL_OVERRUNF1_tag;

typedef union CANXL_GRP_CONTROL_OVERRUNF2_union_tag { /* Descriptor Overrun Flag 2 */
  vuint32_t R;
  struct {
    vuint32_t OVRRF32:1;               /* Overrun Flag */
    vuint32_t OVRRF33:1;               /* Overrun Flag */
    vuint32_t OVRRF34:1;               /* Overrun Flag */
    vuint32_t OVRRF35:1;               /* Overrun Flag */
    vuint32_t OVRRF36:1;               /* Overrun Flag */
    vuint32_t OVRRF37:1;               /* Overrun Flag */
    vuint32_t OVRRF38:1;               /* Overrun Flag */
    vuint32_t OVRRF39:1;               /* Overrun Flag */
    vuint32_t OVRRF40:1;               /* Overrun Flag */
    vuint32_t OVRRF41:1;               /* Overrun Flag */
    vuint32_t OVRRF42:1;               /* Overrun Flag */
    vuint32_t OVRRF43:1;               /* Overrun Flag */
    vuint32_t OVRRF44:1;               /* Overrun Flag */
    vuint32_t OVRRF45:1;               /* Overrun Flag */
    vuint32_t OVRRF46:1;               /* Overrun Flag */
    vuint32_t OVRRF47:1;               /* Overrun Flag */
    vuint32_t OVRRF48:1;               /* Overrun Flag */
    vuint32_t OVRRF49:1;               /* Overrun Flag */
    vuint32_t OVRRF50:1;               /* Overrun Flag */
    vuint32_t OVRRF51:1;               /* Overrun Flag */
    vuint32_t OVRRF52:1;               /* Overrun Flag */
    vuint32_t OVRRF53:1;               /* Overrun Flag */
    vuint32_t OVRRF54:1;               /* Overrun Flag */
    vuint32_t OVRRF55:1;               /* Overrun Flag */
    vuint32_t OVRRF56:1;               /* Overrun Flag */
    vuint32_t OVRRF57:1;               /* Overrun Flag */
    vuint32_t OVRRF58:1;               /* Overrun Flag */
    vuint32_t OVRRF59:1;               /* Overrun Flag */
    vuint32_t OVRRF60:1;               /* Overrun Flag */
    vuint32_t OVRRF61:1;               /* Overrun Flag */
    vuint32_t OVRRF62:1;               /* Overrun Flag */
    vuint32_t OVRRF63:1;               /* Overrun Flag */
  } B;
} CANXL_GRP_CONTROL_OVERRUNF2_tag;

typedef union CANXL_GRP_CONTROL_OVERRUNF3_union_tag { /* Descriptor Overrun Flag 3 */
  vuint32_t R;
  struct {
    vuint32_t OVRRF64:1;               /* Overrun Flag */
    vuint32_t OVRRF65:1;               /* Overrun Flag */
    vuint32_t OVRRF66:1;               /* Overrun Flag */
    vuint32_t OVRRF67:1;               /* Overrun Flag */
    vuint32_t OVRRF68:1;               /* Overrun Flag */
    vuint32_t OVRRF69:1;               /* Overrun Flag */
    vuint32_t OVRRF70:1;               /* Overrun Flag */
    vuint32_t OVRRF71:1;               /* Overrun Flag */
    vuint32_t OVRRF72:1;               /* Overrun Flag */
    vuint32_t OVRRF73:1;               /* Overrun Flag */
    vuint32_t OVRRF74:1;               /* Overrun Flag */
    vuint32_t OVRRF75:1;               /* Overrun Flag */
    vuint32_t OVRRF76:1;               /* Overrun Flag */
    vuint32_t OVRRF77:1;               /* Overrun Flag */
    vuint32_t OVRRF78:1;               /* Overrun Flag */
    vuint32_t OVRRF79:1;               /* Overrun Flag */
    vuint32_t OVRRF80:1;               /* Overrun Flag */
    vuint32_t OVRRF81:1;               /* Overrun Flag */
    vuint32_t OVRRF82:1;               /* Overrun Flag */
    vuint32_t OVRRF83:1;               /* Overrun Flag */
    vuint32_t OVRRF84:1;               /* Overrun Flag */
    vuint32_t OVRRF85:1;               /* Overrun Flag */
    vuint32_t OVRRF86:1;               /* Overrun Flag */
    vuint32_t OVRRF87:1;               /* Overrun Flag */
    vuint32_t OVRRF88:1;               /* Overrun Flag */
    vuint32_t OVRRF89:1;               /* Overrun Flag */
    vuint32_t OVRRF90:1;               /* Overrun Flag */
    vuint32_t OVRRF91:1;               /* Overrun Flag */
    vuint32_t OVRRF92:1;               /* Overrun Flag */
    vuint32_t OVRRF93:1;               /* Overrun Flag */
    vuint32_t OVRRF94:1;               /* Overrun Flag */
    vuint32_t OVRRF95:1;               /* Overrun Flag */
  } B;
} CANXL_GRP_CONTROL_OVERRUNF3_tag;

typedef union CANXL_GRP_CONTROL_OVERRUNF4_union_tag { /* Descriptor Overrun Flag 4 */
  vuint32_t R;
  struct {
    vuint32_t OVRRF96:1;               /* Overrun Flag */
    vuint32_t OVRRF97:1;               /* Overrun Flag */
    vuint32_t OVRRF98:1;               /* Overrun Flag */
    vuint32_t OVRRF99:1;               /* Overrun Flag */
    vuint32_t OVRRF100:1;              /* Overrun Flag */
    vuint32_t OVRRF101:1;              /* Overrun Flag */
    vuint32_t OVRRF102:1;              /* Overrun Flag */
    vuint32_t OVRRF103:1;              /* Overrun Flag */
    vuint32_t OVRRF104:1;              /* Overrun Flag */
    vuint32_t OVRRF105:1;              /* Overrun Flag */
    vuint32_t OVRRF106:1;              /* Overrun Flag */
    vuint32_t OVRRF107:1;              /* Overrun Flag */
    vuint32_t OVRRF108:1;              /* Overrun Flag */
    vuint32_t OVRRF109:1;              /* Overrun Flag */
    vuint32_t OVRRF110:1;              /* Overrun Flag */
    vuint32_t OVRRF111:1;              /* Overrun Flag */
    vuint32_t OVRRF112:1;              /* Overrun Flag */
    vuint32_t OVRRF113:1;              /* Overrun Flag */
    vuint32_t OVRRF114:1;              /* Overrun Flag */
    vuint32_t OVRRF115:1;              /* Overrun Flag */
    vuint32_t OVRRF116:1;              /* Overrun Flag */
    vuint32_t OVRRF117:1;              /* Overrun Flag */
    vuint32_t OVRRF118:1;              /* Overrun Flag */
    vuint32_t OVRRF119:1;              /* Overrun Flag */
    vuint32_t OVRRF120:1;              /* Overrun Flag */
    vuint32_t OVRRF121:1;              /* Overrun Flag */
    vuint32_t OVRRF122:1;              /* Overrun Flag */
    vuint32_t OVRRF123:1;              /* Overrun Flag */
    vuint32_t OVRRF124:1;              /* Overrun Flag */
    vuint32_t OVRRF125:1;              /* Overrun Flag */
    vuint32_t OVRRF126:1;              /* Overrun Flag */
    vuint32_t OVRRF127:1;              /* Overrun Flag */
  } B;
} CANXL_GRP_CONTROL_OVERRUNF4_tag;

typedef union CANXL_GRP_CONTROL_URUNF1_union_tag { /* Under Run Flag 1 */
  vuint32_t R;
  struct {
    vuint32_t URUNF0:1;                /* Under Run Flag */
    vuint32_t URUNF1:1;                /* Under Run Flag */
    vuint32_t URUNF2:1;                /* Under Run Flag */
    vuint32_t URUNF3:1;                /* Under Run Flag */
    vuint32_t URUNF4:1;                /* Under Run Flag */
    vuint32_t URUNF5:1;                /* Under Run Flag */
    vuint32_t URUNF6:1;                /* Under Run Flag */
    vuint32_t URUNF7:1;                /* Under Run Flag */
    vuint32_t URUNF8:1;                /* Under Run Flag */
    vuint32_t URUNF9:1;                /* Under Run Flag */
    vuint32_t URUNF10:1;               /* Under Run Flag */
    vuint32_t URUNF11:1;               /* Under Run Flag */
    vuint32_t URUNF12:1;               /* Under Run Flag */
    vuint32_t URUNF13:1;               /* Under Run Flag */
    vuint32_t URUNF14:1;               /* Under Run Flag */
    vuint32_t URUNF15:1;               /* Under Run Flag */
    vuint32_t URUNF16:1;               /* Under Run Flag */
    vuint32_t URUNF17:1;               /* Under Run Flag */
    vuint32_t URUNF18:1;               /* Under Run Flag */
    vuint32_t URUNF19:1;               /* Under Run Flag */
    vuint32_t URUNF20:1;               /* Under Run Flag */
    vuint32_t URUNF21:1;               /* Under Run Flag */
    vuint32_t URUNF22:1;               /* Under Run Flag */
    vuint32_t URUNF23:1;               /* Under Run Flag */
    vuint32_t URUNF24:1;               /* Under Run Flag */
    vuint32_t URUNF25:1;               /* Under Run Flag */
    vuint32_t URUNF26:1;               /* Under Run Flag */
    vuint32_t URUNF27:1;               /* Under Run Flag */
    vuint32_t URUNF28:1;               /* Under Run Flag */
    vuint32_t URUNF29:1;               /* Under Run Flag */
    vuint32_t URUNF30:1;               /* Under Run Flag */
    vuint32_t URUNF31:1;               /* Under Run Flag */
  } B;
} CANXL_GRP_CONTROL_URUNF1_tag;

typedef union CANXL_GRP_CONTROL_URUNF2_union_tag { /* Under Run Flag 2 */
  vuint32_t R;
  struct {
    vuint32_t URUNF32:1;               /* Under Run Flag */
    vuint32_t URUNF33:1;               /* Under Run Flag */
    vuint32_t URUNF34:1;               /* Under Run Flag */
    vuint32_t URUNF35:1;               /* Under Run Flag */
    vuint32_t URUNF36:1;               /* Under Run Flag */
    vuint32_t URUNF37:1;               /* Under Run Flag */
    vuint32_t URUNF38:1;               /* Under Run Flag */
    vuint32_t URUNF39:1;               /* Under Run Flag */
    vuint32_t URUNF40:1;               /* Under Run Flag */
    vuint32_t URUNF41:1;               /* Under Run Flag */
    vuint32_t URUNF42:1;               /* Under Run Flag */
    vuint32_t URUNF43:1;               /* Under Run Flag */
    vuint32_t URUNF44:1;               /* Under Run Flag */
    vuint32_t URUNF45:1;               /* Under Run Flag */
    vuint32_t URUNF46:1;               /* Under Run Flag */
    vuint32_t URUNF47:1;               /* Under Run Flag */
    vuint32_t URUNF48:1;               /* Under Run Flag */
    vuint32_t URUNF49:1;               /* Under Run Flag */
    vuint32_t URUNF50:1;               /* Under Run Flag */
    vuint32_t URUNF51:1;               /* Under Run Flag */
    vuint32_t URUNF52:1;               /* Under Run Flag */
    vuint32_t URUNF53:1;               /* Under Run Flag */
    vuint32_t URUNF54:1;               /* Under Run Flag */
    vuint32_t URUNF55:1;               /* Under Run Flag */
    vuint32_t URUNF56:1;               /* Under Run Flag */
    vuint32_t URUNF57:1;               /* Under Run Flag */
    vuint32_t URUNF58:1;               /* Under Run Flag */
    vuint32_t URUNF59:1;               /* Under Run Flag */
    vuint32_t URUNF60:1;               /* Under Run Flag */
    vuint32_t URUNF61:1;               /* Under Run Flag */
    vuint32_t URUNF62:1;               /* Under Run Flag */
    vuint32_t URUNF63:1;               /* Under Run Flag */
  } B;
} CANXL_GRP_CONTROL_URUNF2_tag;

typedef union CANXL_GRP_CONTROL_URUNF3_union_tag { /* Under Run Flag 3 */
  vuint32_t R;
  struct {
    vuint32_t URUNF64:1;               /* Under Run Flag */
    vuint32_t URUNF65:1;               /* Under Run Flag */
    vuint32_t URUNF66:1;               /* Under Run Flag */
    vuint32_t URUNF67:1;               /* Under Run Flag */
    vuint32_t URUNF68:1;               /* Under Run Flag */
    vuint32_t URUNF69:1;               /* Under Run Flag */
    vuint32_t URUNF70:1;               /* Under Run Flag */
    vuint32_t URUNF71:1;               /* Under Run Flag */
    vuint32_t URUNF72:1;               /* Under Run Flag */
    vuint32_t URUNF73:1;               /* Under Run Flag */
    vuint32_t URUNF74:1;               /* Under Run Flag */
    vuint32_t URUNF75:1;               /* Under Run Flag */
    vuint32_t URUNF76:1;               /* Under Run Flag */
    vuint32_t URUNF77:1;               /* Under Run Flag */
    vuint32_t URUNF78:1;               /* Under Run Flag */
    vuint32_t URUNF79:1;               /* Under Run Flag */
    vuint32_t URUNF80:1;               /* Under Run Flag */
    vuint32_t URUNF81:1;               /* Under Run Flag */
    vuint32_t URUNF82:1;               /* Under Run Flag */
    vuint32_t URUNF83:1;               /* Under Run Flag */
    vuint32_t URUNF84:1;               /* Under Run Flag */
    vuint32_t URUNF85:1;               /* Under Run Flag */
    vuint32_t URUNF86:1;               /* Under Run Flag */
    vuint32_t URUNF87:1;               /* Under Run Flag */
    vuint32_t URUNF88:1;               /* Under Run Flag */
    vuint32_t URUNF89:1;               /* Under Run Flag */
    vuint32_t URUNF90:1;               /* Under Run Flag */
    vuint32_t URUNF91:1;               /* Under Run Flag */
    vuint32_t URUNF92:1;               /* Under Run Flag */
    vuint32_t URUNF93:1;               /* Under Run Flag */
    vuint32_t URUNF94:1;               /* Under Run Flag */
    vuint32_t URUNF95:1;               /* Under Run Flag */
  } B;
} CANXL_GRP_CONTROL_URUNF3_tag;

typedef union CANXL_GRP_CONTROL_URUNF4_union_tag { /* Under Run Flag 4 */
  vuint32_t R;
  struct {
    vuint32_t URUNF96:1;               /* Under Run Flag */
    vuint32_t URUNF97:1;               /* Under Run Flag */
    vuint32_t URUNF98:1;               /* Under Run Flag */
    vuint32_t URUNF99:1;               /* Under Run Flag */
    vuint32_t URUNF100:1;              /* Under Run Flag */
    vuint32_t URUNF101:1;              /* Under Run Flag */
    vuint32_t URUNF102:1;              /* Under Run Flag */
    vuint32_t URUNF103:1;              /* Under Run Flag */
    vuint32_t URUNF104:1;              /* Under Run Flag */
    vuint32_t URUNF105:1;              /* Under Run Flag */
    vuint32_t URUNF106:1;              /* Under Run Flag */
    vuint32_t URUNF107:1;              /* Under Run Flag */
    vuint32_t URUNF108:1;              /* Under Run Flag */
    vuint32_t URUNF109:1;              /* Under Run Flag */
    vuint32_t URUNF110:1;              /* Under Run Flag */
    vuint32_t URUNF111:1;              /* Under Run Flag */
    vuint32_t URUNF112:1;              /* Under Run Flag */
    vuint32_t URUNF113:1;              /* Under Run Flag */
    vuint32_t URUNF114:1;              /* Under Run Flag */
    vuint32_t URUNF115:1;              /* Under Run Flag */
    vuint32_t URUNF116:1;              /* Under Run Flag */
    vuint32_t URUNF117:1;              /* Under Run Flag */
    vuint32_t URUNF118:1;              /* Under Run Flag */
    vuint32_t URUNF119:1;              /* Under Run Flag */
    vuint32_t URUNF120:1;              /* Under Run Flag */
    vuint32_t URUNF121:1;              /* Under Run Flag */
    vuint32_t URUNF122:1;              /* Under Run Flag */
    vuint32_t URUNF123:1;              /* Under Run Flag */
    vuint32_t URUNF124:1;              /* Under Run Flag */
    vuint32_t URUNF125:1;              /* Under Run Flag */
    vuint32_t URUNF126:1;              /* Under Run Flag */
    vuint32_t URUNF127:1;              /* Under Run Flag */
  } B;
} CANXL_GRP_CONTROL_URUNF4_tag;

struct CANXL_GRP_CONTROL_tag {
  CANXL_GRP_CONTROL_DSCCTRL_tag DSCCTRL; /* Descriptors Control */
  CANXL_GRP_CONTROL_DSCACTIVE1_tag DSCACTIVE1; /* Descriptor Activation 1 */
  CANXL_GRP_CONTROL_DSCACTIVE2_tag DSCACTIVE2; /* Descriptor Activation 2 */
  CANXL_GRP_CONTROL_DSCACTIVE3_tag DSCACTIVE3; /* Descriptor Activation 3 */
  CANXL_GRP_CONTROL_DSCACTIVE4_tag DSCACTIVE4; /* Descriptor Activation 4 */
  CANXL_GRP_CONTROL_FIFOCTRL1_tag FIFOCTRL1; /* Message FIFO Control 1 */
  CANXL_GRP_CONTROL_FIFOCTRL2_tag FIFOCTRL2; /* Message FIFO Control 2 */
  CANXL_GRP_CONTROL_FIFOCTRL3_tag FIFOCTRL3; /* Message FIFO Control 3 */
  CANXL_GRP_CONTROL_FIFOCTRL4_tag FIFOCTRL4; /* Message FIFO Control 4 */
  uint8_t CANXL_GRP_CONTROL_reserved0[48];
  CANXL_GRP_CONTROL_MSGIFLAG1_tag MSGIFLAG1; /* Message Interrupt FLAG 1 */
  CANXL_GRP_CONTROL_MSGIFLAG2_tag MSGIFLAG2; /* Message Interrupt FLAG 2 */
  CANXL_GRP_CONTROL_MSGIFLAG3_tag MSGIFLAG3; /* Message Interrupt FLAG 3 */
  CANXL_GRP_CONTROL_MSGIFLAG4_tag MSGIFLAG4; /* Message Interrupt FLAG 4 */
  uint8_t CANXL_GRP_CONTROL_reserved1[16];
  CANXL_GRP_CONTROL_MSGIMASK1_tag MSGIMASK1; /* Message Interrupt Mask 1 */
  CANXL_GRP_CONTROL_MSGIMASK2_tag MSGIMASK2; /* Message Interrupt Mask 2 */
  CANXL_GRP_CONTROL_MSGIMASK3_tag MSGIMASK3; /* Message Interrupt Mask 3 */
  CANXL_GRP_CONTROL_MSGIMASK4_tag MSGIMASK4; /* Message Interrupt Mask 4 */
  uint8_t CANXL_GRP_CONTROL_reserved2[16];
  CANXL_GRP_CONTROL_FREEDSC0_tag FREEDSC0; /* Free Descriptor Index 0 */
  uint8_t CANXL_GRP_CONTROL_reserved3[112];
  CANXL_GRP_CONTROL_OVERRUNF1_tag OVERRUNF1; /* Descriptor Overrun Flag 1 */
  CANXL_GRP_CONTROL_OVERRUNF2_tag OVERRUNF2; /* Descriptor Overrun Flag 2 */
  CANXL_GRP_CONTROL_OVERRUNF3_tag OVERRUNF3; /* Descriptor Overrun Flag 3 */
  CANXL_GRP_CONTROL_OVERRUNF4_tag OVERRUNF4; /* Descriptor Overrun Flag 4 */
  uint8_t CANXL_GRP_CONTROL_reserved4[32];
  CANXL_GRP_CONTROL_URUNF1_tag URUNF1; /* Under Run Flag 1 */
  CANXL_GRP_CONTROL_URUNF2_tag URUNF2; /* Under Run Flag 2 */
  CANXL_GRP_CONTROL_URUNF3_tag URUNF3; /* Under Run Flag 3 */
  CANXL_GRP_CONTROL_URUNF4_tag URUNF4; /* Under Run Flag 4 */
};


/* ============================================================================
   =============================== Module: CANXL_MRU ==========================
   ============================================================================ */

typedef union CANXL_MRU_CHXCONFIG_CH_CFG0_union_tag { /* Channel (x) Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t CHE:1;                   /* Channel Enable */
    vuint32_t CHR:1;                   /* Channel Reset */
    vuint32_t IE:1;                    /* Interrupt Enable */
    vuint32_t _unused_3:13;
    vuint32_t MBE0:1;                  /* Mailbox Enable */
    vuint32_t MBE1:1;                  /* Mailbox Enable */
    vuint32_t MBE2:1;                  /* Mailbox Enable */
    vuint32_t MBE3:1;                  /* Mailbox Enable */
    vuint32_t _unused_20:12;
  } B;
} CANXL_MRU_CHXCONFIG_CH_CFG0_tag;

typedef union CANXL_MRU_CHXCONFIG_CH_CFG1_union_tag { /* Channel (x) Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MBIC0:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC1:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC2:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC3:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t _unused_20:12;
  } B;
} CANXL_MRU_CHXCONFIG_CH_CFG1_tag;

typedef union CANXL_MRU_CHXCONFIG_CH_MBSTAT_union_tag { /* Channel (x) Mailbox Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MBS0:1;                  /* Mailbox Status */
    vuint32_t MBS1:1;                  /* Mailbox Status */
    vuint32_t MBS2:1;                  /* Mailbox Status */
    vuint32_t MBS3:1;                  /* Mailbox Status */
    vuint32_t _unused_20:12;
  } B;
} CANXL_MRU_CHXCONFIG_CH_MBSTAT_tag;

typedef struct CANXL_MRU_CHXCONFIG_struct_tag {
  CANXL_MRU_CHXCONFIG_CH_CFG0_tag CH_CFG0; /* Channel (x) Configuration 0 */
  CANXL_MRU_CHXCONFIG_CH_CFG1_tag CH_CFG1; /* Channel (x) Configuration 1 */
  CANXL_MRU_CHXCONFIG_CH_MBSTAT_tag CH_MBSTAT; /* Channel (x) Mailbox Status */
} CANXL_MRU_CHXCONFIG_tag;

typedef union CANXL_MRU_NOTIFY0_union_tag { /* Notification 0 Status */
  vuint32_t R;
  struct {
    vuint32_t CH1_IS0:1;               /* Interrupt 0 Status */
    vuint32_t _unused_1:31;
  } B;
} CANXL_MRU_NOTIFY0_tag;

typedef union CANXL_MRU_NOTIFY1_union_tag { /* Notification 1 Status */
  vuint32_t R;
  struct {
    vuint32_t CH1_IS1:1;               /* Interrupt 1 Status */
    vuint32_t _unused_1:31;
  } B;
} CANXL_MRU_NOTIFY1_tag;

typedef union CANXL_MRU_CH1_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} CANXL_MRU_CH1_MB0_tag;

typedef union CANXL_MRU_CH1_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} CANXL_MRU_CH1_MB1_tag;

typedef union CANXL_MRU_CH1_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} CANXL_MRU_CH1_MB2_tag;

typedef union CANXL_MRU_CH1_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} CANXL_MRU_CH1_MB3_tag;

struct CANXL_MRU_tag {
  CANXL_MRU_CHXCONFIG_tag CHXCONFIG[1];
  uint8_t CANXL_MRU_reserved0[500];
  CANXL_MRU_NOTIFY0_tag NOTIFY0;       /* Notification 0 Status */
  CANXL_MRU_NOTIFY1_tag NOTIFY1;       /* Notification 1 Status */
  uint8_t CANXL_MRU_reserved1[3576];
  CANXL_MRU_CH1_MB0_tag CH1_MB0;       /* Channel (x) Mailbox (n) */
  CANXL_MRU_CH1_MB1_tag CH1_MB1;       /* Channel (x) Mailbox (n) */
  CANXL_MRU_CH1_MB2_tag CH1_MB2;       /* Channel (x) Mailbox (n) */
  CANXL_MRU_CH1_MB3_tag CH1_MB3;       /* Channel (x) Mailbox (n) */
};


/* ============================================================================
   =============================== Module: CANXL_MSG_DESCRIPTORS ==============
   ============================================================================ */

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_RXCTRL_union_tag { /* Message Descriptor Control (In Rx Mode) */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MBSIZE:11;               /* Message Buffer Size */
    vuint32_t _unused_19:1;
    vuint32_t ADDRROA:1;               /* ADDR Rejection Over Acceptance */
    vuint32_t SDUROA:1;                /* SDU Rejection Over Acceptance */
    vuint32_t VCANROA:1;               /* VCAN Rejection Over Acceptance */
    vuint32_t _unused_23:1;
    vuint32_t KEEPLST:1;               /* Keep Latest */
    vuint32_t _unused_25:4;
    vuint32_t RXFDFRM:1;               /* FD Type Frame Configuration */
    vuint32_t RXXLFRM:1;               /* XL Type Frame Configuration */
    vuint32_t MODE:1;                  /* RX Mode Indication */
  } B;
} CANXL_MSG_DESCRIPTORS_MSGDSC_RXCTRL_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_MDFLT1FD_union_tag { /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
  vuint32_t R;
  struct {
    vuint32_t IDEXTA_L:18;             /* IDEXT Acceptance Filter Low */
    vuint32_t IDSTDA_L:11;             /* IDSTD Acceptance Filter Low */
    vuint32_t MSKRNG:1;                /* ID Filter Pair Mask/Range Selection */
    vuint32_t IDEMSK:1;                /* IDE Mask */
    vuint32_t RTRMSK:1;                /* RTR Mask */
  } B;
} CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_MDFLT1FD_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_MDFLT1XL_union_tag { /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
  vuint32_t R;
  struct {
    vuint32_t ACCPT_VCAN:5;            /* Acceptance VCAN Filter Number */
    vuint32_t ACCPT_SDU:5;             /* Acceptance SDU Filter Number */
    vuint32_t ACCPT_ADDR:5;            /* Acceptance ADDR Filter Number */
    vuint32_t _unused_15:1;
    vuint32_t FBSEL:1;                 /* Filter Bank Select */
    vuint32_t SECMSK:1;                /* SEC Mask */
    vuint32_t IDSTDA_L:11;             /* IDSTD Acceptance Filter Low */
    vuint32_t MSKRNG:1;                /* ID Filter Pair Mask/Range Selection */
    vuint32_t IDEMSK:1;                /* IDE Mask */
    vuint32_t RRSMSK:1;                /* RRS Mask */
  } B;
} CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_MDFLT1XL_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_union_tag {
  CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_MDFLT1FD_tag MDFLT1FD; /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
  CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_MDFLT1XL_tag MDFLT1XL; /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
} CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_MDFLT2FD_union_tag { /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
  vuint32_t R;
  struct {
    vuint32_t IDEXTA_H:18;             /* IDEXT Acceptance Filter High */
    vuint32_t IDSTDA_H:11;             /* IDSTD Acceptance Filter High */
    vuint32_t _unused_29:1;
    vuint32_t IDE:1;                   /* IDE Value */
    vuint32_t RTR:1;                   /* RTR Value */
  } B;
} CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_MDFLT2FD_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_MDFLT2XL_union_tag { /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
  vuint32_t R;
  struct {
    vuint32_t RJCT_VCAN:5;             /* Rejection VCAN Filter Number */
    vuint32_t RJCT_SDU:5;              /* Rejection SDU Filter Number */
    vuint32_t RJCT_ADDR:5;             /* Rejection ADDR Filter Number */
    vuint32_t _unused_15:2;
    vuint32_t SECf:1;                  /* SEC Value */
    vuint32_t IDSTDA_H:11;             /* IDSTD Acceptance Filter High */
    vuint32_t _unused_29:1;
    vuint32_t IDE:1;                   /* IDE Value */
    vuint32_t RRS:1;                   /* RRS Value */
  } B;
} CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_MDFLT2XL_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_union_tag {
  CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_MDFLT2FD_tag MDFLT2FD; /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
  CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_MDFLT2XL_tag MDFLT2XL; /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
} CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_RXLSTPTR_union_tag { /* Message List Pointer (In Rx Mode) */
  vuint32_t R;
  struct {
    vuint32_t RXLST_POINTER:32;        /* Receive List Pointer */
  } B;
} CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_RXLSTPTR_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_TXLSTPTR_union_tag { /* Message List Pointer (In Tx Mode) */
  vuint32_t R;
  struct {
    vuint32_t TXLST_POINTER:32;        /* Transmit List Pointer */
  } B;
} CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_TXLSTPTR_tag;

typedef union CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_union_tag {
  CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_RXLSTPTR_tag RXLSTPTR; /* Message List Pointer (In Rx Mode) */
  CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_TXLSTPTR_tag TXLSTPTR; /* Message List Pointer (In Tx Mode) */
} CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_tag;

typedef struct CANXL_MSG_DESCRIPTORS_MSGDSC_struct_tag {
  CANXL_MSG_DESCRIPTORS_MSGDSC_RXCTRL_tag RXCTRL; /* Message Descriptor Control (In Rx Mode) */
  CANXL_MSG_DESCRIPTORS_MSGDSC_CFG1_tag CFG1;
  CANXL_MSG_DESCRIPTORS_MSGDSC_CFG2_tag CFG2;
  CANXL_MSG_DESCRIPTORS_MSGDSC_LSTPNT_tag LSTPNT;
  uint8_t MSGDSC_reserved0[16];
} CANXL_MSG_DESCRIPTORS_MSGDSC_tag;

struct CANXL_MSG_DESCRIPTORS_tag {
  CANXL_MSG_DESCRIPTORS_MSGDSC_tag MSGDSC[128];
};


/* ============================================================================
   =============================== Module: CANXL_RAMECC =======================
   ============================================================================ */

typedef union CANXL_RAMECC_DRMEC_union_tag { /* Data RAM Error Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t ECCDIS:1;                /* Error Correction Disable */
    vuint32_t ERRRPTDIS:1;             /* Error Report Disable */
    vuint32_t _unused_10:4;
    vuint32_t ERRINJEN:1;              /* CANEXCEL Access Error Injection Enable */
    vuint32_t _unused_15:1;
    vuint32_t CEIEN:1;                 /* Correctable Errors Interrupt Enable */
    vuint32_t _unused_17:1;
    vuint32_t NCEIEN:1;                /* Non-Correctable Errors Interrupt Enable */
    vuint32_t _unused_19:12;
    vuint32_t ECRWRDIS:1;              /* Error Configuration Register Write Disable */
  } B;
} CANXL_RAMECC_DRMEC_tag;

typedef union CANXL_RAMECC_DRERRIA_union_tag { /* Data RAM Error Injection Address */
  vuint32_t R;
  struct {
    vuint32_t INJADDR_L:2;             /* Error Injection Address Low */
    vuint32_t INJADDR_H:13;            /* Error Injection Address High */
    vuint32_t _unused_15:17;
  } B;
} CANXL_RAMECC_DRERRIA_tag;

typedef union CANXL_RAMECC_DRERRIDP_union_tag { /* Data RAM Error Injection Data Pattern */
  vuint32_t R;
  struct {
    vuint32_t DFLIP:32;                /* Data flip pattern */
  } B;
} CANXL_RAMECC_DRERRIDP_tag;

typedef union CANXL_RAMECC_DRERRIPP_union_tag { /* Data RAM Error Injection Parity Pattern */
  vuint32_t R;
  struct {
    vuint32_t PFLIP0:5;                /* Parity Flip Pattern For Byte 0 (Least Significant) */
    vuint32_t _unused_5:3;
    vuint32_t PFLIP1:5;                /* Parity Flip Pattern For Byte 1 */
    vuint32_t _unused_13:3;
    vuint32_t PFLIP2:5;                /* Parity Flip Pattern For Byte 2 */
    vuint32_t _unused_21:3;
    vuint32_t PFLIP3:5;                /* Parity Flip Pattern For Byte 3 (most significant) */
    vuint32_t _unused_29:3;
  } B;
} CANXL_RAMECC_DRERRIPP_tag;

typedef union CANXL_RAMECC_DRRERRA_union_tag { /* Data RAM Error Report Address */
  vuint32_t R;
  struct {
    vuint32_t ERRADDR:15;              /* Error Address */
    vuint32_t _unused_15:9;
    vuint32_t NCE:1;                   /* Non-Correctable Error */
    vuint32_t _unused_25:7;
  } B;
} CANXL_RAMECC_DRRERRA_tag;

typedef union CANXL_RAMECC_DRRERRD_union_tag { /* Data RAM Error Report Data */
  vuint32_t R;
  struct {
    vuint32_t RDATA:32;                /* Raw data word read from memory with error */
  } B;
} CANXL_RAMECC_DRRERRD_tag;

typedef union CANXL_RAMECC_DRRERRSYN_union_tag { /* Data RAM Error Report Syndrome */
  vuint32_t R;
  struct {
    vuint32_t SYND0:5;                 /* Error Syndrome For Byte 0 (least significant) */
    vuint32_t _unused_5:2;
    vuint32_t BE0:1;                   /* Byte Enabled For Byte 0 (least significant) */
    vuint32_t SYND1:5;                 /* Error Syndrome for Byte 1 */
    vuint32_t _unused_13:2;
    vuint32_t BE1:1;                   /* Byte Enabled For Byte 1 */
    vuint32_t SYND2:5;                 /* Error Syndrome For Byte 2 */
    vuint32_t _unused_21:2;
    vuint32_t BE2:1;                   /* Byte Enabled For Byte 2 */
    vuint32_t SYND3:5;                 /* Error Syndrome For Byte 3 (most significant) */
    vuint32_t _unused_29:2;
    vuint32_t BE3:1;                   /* Byte Enabled For Byte 3 (most significant) */
  } B;
} CANXL_RAMECC_DRRERRSYN_tag;

typedef union CANXL_RAMECC_DRERRS_union_tag { /* Data RAM Error Status */
  vuint32_t R;
  struct {
    vuint32_t CEIOF:1;                 /* Correctable Error Interrupt Overrun Flag */
    vuint32_t _unused_1:1;
    vuint32_t NCEIOF:1;                /* Non-Correctable Error Interrupt Overrun Flag */
    vuint32_t _unused_3:13;
    vuint32_t CEIF:1;                  /* Correctable Error Interrupt Flag */
    vuint32_t _unused_17:1;
    vuint32_t NCEIF:1;                 /* Non-Correctable Error Interrupt Flag */
    vuint32_t _unused_19:13;
  } B;
} CANXL_RAMECC_DRERRS_tag;

struct CANXL_RAMECC_tag {
  CANXL_RAMECC_DRMEC_tag DRMEC;        /* Data RAM Error Control */
  CANXL_RAMECC_DRERRIA_tag DRERRIA;    /* Data RAM Error Injection Address */
  CANXL_RAMECC_DRERRIDP_tag DRERRIDP;  /* Data RAM Error Injection Data Pattern */
  CANXL_RAMECC_DRERRIPP_tag DRERRIPP;  /* Data RAM Error Injection Parity Pattern */
  CANXL_RAMECC_DRRERRA_tag DRRERRA;    /* Data RAM Error Report Address */
  CANXL_RAMECC_DRRERRD_tag DRRERRD;    /* Data RAM Error Report Data */
  CANXL_RAMECC_DRRERRSYN_tag DRRERRSYN; /* Data RAM Error Report Syndrome */
  CANXL_RAMECC_DRERRS_tag DRERRS;      /* Data RAM Error Status */
};


/* ============================================================================
   =============================== Module: CANXL_RXFIFO =======================
   ============================================================================ */

typedef union CANXL_RXFIFO_RXFFCTR_union_tag { /* Receive FIFO Message Buffer Control */
  vuint32_t R;
  struct {
    vuint32_t MBSIZE:11;               /* Message Buffer Size */
    vuint32_t _unused_11:4;
    vuint32_t KEEPLST:1;               /* Keep Latest */
    vuint32_t _unused_16:16;
  } B;
} CANXL_RXFIFO_RXFFCTR_tag;

typedef union CANXL_RXFIFO_RXFMBP_union_tag { /* Receive FIFO Message Buffer Pointer */
  vuint32_t R;
  struct {
    vuint32_t RXPT:32;                 /* Receive Message Buffer Pointer */
  } B;
} CANXL_RXFIFO_RXFMBP_tag;

typedef union CANXL_RXFIFO_SECAM_union_tag { /* SEC Acceptance Mask */
  vuint32_t R;
  struct {
    vuint32_t SECAM0:1;                /* SEC Acceptance Mask for Filter 0 */
    vuint32_t SECAM1:1;                /* SEC Acceptance Mask for Filter 1 */
    vuint32_t SECAM2:1;                /* SEC Acceptance Mask for Filter 2 */
    vuint32_t SECAM3:1;                /* SEC Acceptance Mask for Filter 3 */
    vuint32_t SECAM4:1;                /* SEC Acceptance Mask for Filter 4 */
    vuint32_t SECAM5:1;                /* SEC Acceptance Mask for Filter 5 */
    vuint32_t SECAM6:1;                /* SEC Acceptance Mask for Filter 6 */
    vuint32_t SECAM7:1;                /* SEC Acceptance Mask for Filter 7 */
    vuint32_t SECAM8:1;                /* SEC Acceptance Mask for Filter 8 */
    vuint32_t SECAM9:1;                /* SEC Acceptance Mask for Filter 9 */
    vuint32_t SECAM10:1;               /* SEC Acceptance Mask for Filter 10 */
    vuint32_t SECAM11:1;               /* SEC Acceptance Mask for Filter 11 */
    vuint32_t SECAM12:1;               /* SEC Acceptance Mask for Filter 12 */
    vuint32_t SECAM13:1;               /* SEC Acceptance Mask for Filter 13 */
    vuint32_t SECAM14:1;               /* SEC Acceptance Mask for Filter 14 */
    vuint32_t SECAM15:1;               /* SEC Acceptance Mask for Filter 15 */
    vuint32_t SECAM16:1;               /* SEC Acceptance Mask for Filter 16 */
    vuint32_t SECAM17:1;               /* SEC Acceptance Mask for Filter 17 */
    vuint32_t SECAM18:1;               /* SEC Acceptance Mask for Filter 18 */
    vuint32_t SECAM19:1;               /* SEC Acceptance Mask for Filter 19 */
    vuint32_t SECAM20:1;               /* SEC Acceptance Mask for Filter 20 */
    vuint32_t SECAM21:1;               /* SEC Acceptance Mask for Filter 21 */
    vuint32_t SECAM22:1;               /* SEC Acceptance Mask for Filter 22 */
    vuint32_t SECAM23:1;               /* SEC Acceptance Mask for Filter 23 */
    vuint32_t SECAM24:1;               /* SEC Acceptance Mask for Filter 24 */
    vuint32_t SECAM25:1;               /* SEC Acceptance Mask for Filter 25 */
    vuint32_t SECAM26:1;               /* SEC Acceptance Mask for Filter 26 */
    vuint32_t SECAM27:1;               /* SEC Acceptance Mask for Filter 27 */
    vuint32_t SECAM28:1;               /* SEC Acceptance Mask for Filter 28 */
    vuint32_t SECAM29:1;               /* SEC Acceptance Mask for Filter 29 */
    vuint32_t SECAM30:1;               /* SEC Acceptance Mask for Filter 30 */
    vuint32_t SECAM31:1;               /* SEC Acceptance Mask for Filter 31 */
  } B;
} CANXL_RXFIFO_SECAM_tag;

typedef union CANXL_RXFIFO_SECAV_union_tag { /* SEC Acceptance Value */
  vuint32_t R;
  struct {
    vuint32_t SECAV0:1;                /* SEC Acceptance Value for Filter 0 */
    vuint32_t SECAV1:1;                /* SEC Acceptance Value for Filter 1 */
    vuint32_t SECAV2:1;                /* SEC Acceptance Value for Filter 2 */
    vuint32_t SECAV3:1;                /* SEC Acceptance Value for Filter 3 */
    vuint32_t SECAV4:1;                /* SEC Acceptance Value for Filter 4 */
    vuint32_t SECAV5:1;                /* SEC Acceptance Value for Filter 5 */
    vuint32_t SECAV6:1;                /* SEC Acceptance Value for Filter 6 */
    vuint32_t SECAV7:1;                /* SEC Acceptance Value for Filter 7 */
    vuint32_t SECAV8:1;                /* SEC Acceptance Value for Filter 8 */
    vuint32_t SECAV9:1;                /* SEC Acceptance Value for Filter 9 */
    vuint32_t SECAV10:1;               /* SEC Acceptance Value for Filter 10 */
    vuint32_t SECAV11:1;               /* SEC Acceptance Value for Filter 11 */
    vuint32_t SECAV12:1;               /* SEC Acceptance Value for Filter 12 */
    vuint32_t SECAV13:1;               /* SEC Acceptance Value for Filter 13 */
    vuint32_t SECAV14:1;               /* SEC Acceptance Value for Filter 14 */
    vuint32_t SECAV15:1;               /* SEC Acceptance Value for Filter 15 */
    vuint32_t SECAV16:1;               /* SEC Acceptance Value for Filter 16 */
    vuint32_t SECAV17:1;               /* SEC Acceptance Value for Filter 17 */
    vuint32_t SECAV18:1;               /* SEC Acceptance Value for Filter 18 */
    vuint32_t SECAV19:1;               /* SEC Acceptance Value for Filter 19 */
    vuint32_t SECAV20:1;               /* SEC Acceptance Value for Filter 20 */
    vuint32_t SECAV21:1;               /* SEC Acceptance Value for Filter 21 */
    vuint32_t SECAV22:1;               /* SEC Acceptance Value for Filter 22 */
    vuint32_t SECAV23:1;               /* SEC Acceptance Value for Filter 23 */
    vuint32_t SECAV24:1;               /* SEC Acceptance Value for Filter 24 */
    vuint32_t SECAV25:1;               /* SEC Acceptance Value for Filter 25 */
    vuint32_t SECAV26:1;               /* SEC Acceptance Value for Filter 26 */
    vuint32_t SECAV27:1;               /* SEC Acceptance Value for Filter 27 */
    vuint32_t SECAV28:1;               /* SEC Acceptance Value for Filter 28 */
    vuint32_t SECAV29:1;               /* SEC Acceptance Value for Filter 29 */
    vuint32_t SECAV30:1;               /* SEC Acceptance Value for Filter 30 */
    vuint32_t SECAV31:1;               /* SEC Acceptance Value for Filter 31 */
  } B;
} CANXL_RXFIFO_SECAV_tag;

typedef union CANXL_RXFIFO_AFCFG_union_tag { /* Acceptance Filter Configuration */
  vuint32_t R;
  struct {
    vuint32_t ACPTVCAN:5;              /* Number of VCAN Acceptance Filters */
    vuint32_t _unused_5:2;
    vuint32_t AVCANEN:1;               /* Acceptance VCAN Enable */
    vuint32_t ACPTSDU:5;               /* Number of SDU Acceptance Filters */
    vuint32_t _unused_13:2;
    vuint32_t ASDUEN:1;                /* Acceptance SDU Enable */
    vuint32_t ACPTADDR:5;              /* Number of ADDR Acceptance Filters */
    vuint32_t _unused_21:2;
    vuint32_t AADDREN:1;               /* Acceptance ADDR Enable */
    vuint32_t ACPTID:5;                /* Number of ID Acceptance Filters */
    vuint32_t _unused_29:3;
  } B;
} CANXL_RXFIFO_AFCFG_tag;

typedef union CANXL_RXFIFO_VAMRCFG_union_tag { /* VCAN Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t VMSK0R0:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R1:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R2:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R3:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R4:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R5:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R6:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R7:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R8:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R9:1;               /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R10:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R11:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R12:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R13:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R14:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R15:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R16:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R17:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R18:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R19:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R20:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R21:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R22:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R23:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R24:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R25:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R26:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R27:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R28:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R29:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R30:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t VMSK0R31:1;              /* VCAN Filter Pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_RXFIFO_VAMRCFG_tag;

typedef union CANXL_RXFIFO_SAMRCFG_union_tag { /* SDU Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t SMSK0R0:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R1:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R2:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R3:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R4:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R5:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R6:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R7:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R8:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R9:1;               /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R10:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R11:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R12:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R13:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R14:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R15:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R16:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R17:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R18:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R19:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R20:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R21:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R22:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R23:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R24:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R25:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R26:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R27:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R28:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R29:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R30:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t SMSK0R31:1;              /* SDU Filter Pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_RXFIFO_SAMRCFG_tag;

typedef union CANXL_RXFIFO_AAMRCFG_union_tag { /* ADDR Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t AMSK0R0:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R1:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R2:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R3:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R4:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R5:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R6:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R7:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R8:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R9:1;               /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R10:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R11:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R12:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R13:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R14:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R15:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R16:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R17:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R18:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R19:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R20:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R21:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R22:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R23:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R24:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R25:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R26:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R27:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R28:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R29:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R30:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t AMSK0R31:1;              /* ADDR Filter Pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_RXFIFO_AAMRCFG_tag;

typedef union CANXL_RXFIFO_ACPTIDMR_union_tag { /* ID Acceptance Mask or Range Configuration */
  vuint32_t R;
  struct {
    vuint32_t IDAMSK0R0:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R1:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R2:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R3:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R4:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R5:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R6:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R7:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R8:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R9:1;             /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R10:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R11:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R12:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R13:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R14:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R15:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R16:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R17:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R18:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R19:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R20:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R21:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R22:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R23:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R24:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R25:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R26:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R27:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R28:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R29:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R30:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
    vuint32_t IDAMSK0R31:1;            /* ID Filter Pair Mask/Range Selection n in Acceptance Filter */
  } B;
} CANXL_RXFIFO_ACPTIDMR_tag;

typedef union CANXL_RXFIFO_VAFLT_union_tag { /* VCAN Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t VCANA_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANA_H:8;               /* VCAN Filter Element High */
    vuint32_t VCANB_L:8;               /* VCAN Acceptance Filter Low */
    vuint32_t VCANB_H:8;               /* VCAN Acceptance Filter High */
  } B;
} CANXL_RXFIFO_VAFLT_tag;

typedef union CANXL_RXFIFO_SAFLT_union_tag { /* SDU Acceptance Filter */
  vuint32_t R;
  struct {
    vuint32_t SDUA_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUA_H:8;                /* SDU Acceptance Filter High */
    vuint32_t SDUB_L:8;                /* SDU Acceptance Filter Low */
    vuint32_t SDUB_H:8;                /* SDU Acceptance Filter High */
  } B;
} CANXL_RXFIFO_SAFLT_tag;

typedef union CANXL_RXFIFO_ADDRACPTFLTAR_AAFLTL_union_tag { /* ADDR Acceptance Filter Element Low */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_L:32;              /* ADDR Acceptance Filter Low */
  } B;
} CANXL_RXFIFO_ADDRACPTFLTAR_AAFLTL_tag;

typedef union CANXL_RXFIFO_ADDRACPTFLTAR_AAFLTH_union_tag { /* ADDR Acceptance Filter Element High */
  vuint32_t R;
  struct {
    vuint32_t ADDRN_H:32;              /* ADDR Acceptance Filter High */
  } B;
} CANXL_RXFIFO_ADDRACPTFLTAR_AAFLTH_tag;

typedef struct CANXL_RXFIFO_ADDRACPTFLTAR_struct_tag {
  CANXL_RXFIFO_ADDRACPTFLTAR_AAFLTL_tag AAFLTL; /* ADDR Acceptance Filter Element Low */
  CANXL_RXFIFO_ADDRACPTFLTAR_AAFLTH_tag AAFLTH; /* ADDR Acceptance Filter Element High */
} CANXL_RXFIFO_ADDRACPTFLTAR_tag;

typedef union CANXL_RXFIFO_IDACPTFLTAR_IDAFLTL_union_tag { /* ID Acceptance Filter Element Low */
  vuint32_t R;
  struct {
    vuint32_t IDEXTENDN_L:18;          /* IDEXT Acceptance Filter Low */
    vuint32_t IDSTANDN_L:11;           /* IDSTD Acceptance Filter Low */
    vuint32_t _unused_29:1;
    vuint32_t IDEN_L:1;                /* IDE Acceptance Filter Low */
    vuint32_t RTRN_L:1;                /* RTR/RRS Acceptance Filter Low */
  } B;
} CANXL_RXFIFO_IDACPTFLTAR_IDAFLTL_tag;

typedef union CANXL_RXFIFO_IDACPTFLTAR_IDAFLTH_union_tag { /* ID Acceptance Filter Element High */
  vuint32_t R;
  struct {
    vuint32_t IDEXTENDN_H:18;          /* IDEXT Acceptance Filter High */
    vuint32_t IDSTANDN_H:11;           /* IDSTD Acceptance Filter High */
    vuint32_t _unused_29:1;
    vuint32_t IDEN_H:1;                /* IDE Acceptance Filter High */
    vuint32_t RTRN_H:1;                /* RTR/RRS Acceptance Filter High */
  } B;
} CANXL_RXFIFO_IDACPTFLTAR_IDAFLTH_tag;

typedef struct CANXL_RXFIFO_IDACPTFLTAR_struct_tag {
  CANXL_RXFIFO_IDACPTFLTAR_IDAFLTL_tag IDAFLTL; /* ID Acceptance Filter Element Low */
  CANXL_RXFIFO_IDACPTFLTAR_IDAFLTH_tag IDAFLTH; /* ID Acceptance Filter Element High */
} CANXL_RXFIFO_IDACPTFLTAR_tag;

struct CANXL_RXFIFO_tag {
  CANXL_RXFIFO_RXFFCTR_tag RXFFCTR;    /* Receive FIFO Message Buffer Control */
  CANXL_RXFIFO_RXFMBP_tag RXFMBP[32];  /* Receive FIFO Message Buffer Pointer */
  uint8_t CANXL_RXFIFO_reserved0[116];
  CANXL_RXFIFO_SECAM_tag SECAM;        /* SEC Acceptance Mask */
  CANXL_RXFIFO_SECAV_tag SECAV;        /* SEC Acceptance Value */
  CANXL_RXFIFO_AFCFG_tag AFCFG;        /* Acceptance Filter Configuration */
  CANXL_RXFIFO_VAMRCFG_tag VAMRCFG;    /* VCAN Acceptance Mask or Range Configuration */
  CANXL_RXFIFO_SAMRCFG_tag SAMRCFG;    /* SDU Acceptance Mask or Range Configuration */
  CANXL_RXFIFO_AAMRCFG_tag AAMRCFG;    /* ADDR Acceptance Mask or Range Configuration */
  CANXL_RXFIFO_ACPTIDMR_tag ACPTIDMR;  /* ID Acceptance Mask or Range Configuration */
  uint8_t CANXL_RXFIFO_reserved1[16];
  CANXL_RXFIFO_VAFLT_tag VAFLT[16];    /* VCAN Acceptance Filter */
  CANXL_RXFIFO_SAFLT_tag SAFLT[16];    /* SDU Acceptance Filter */
  CANXL_RXFIFO_ADDRACPTFLTAR_tag ADDRACPTFLTAR[32];
  CANXL_RXFIFO_IDACPTFLTAR_tag IDACPTFLTAR[32];
};


/* ============================================================================
   =============================== Module: CANXL_RXFIFO_CONTROL ===============
   ============================================================================ */

typedef union CANXL_RXFIFO_CONTROL_RXFCSTA_union_tag { /* RxFIFO Control Status */
  vuint32_t R;
  struct {
    vuint32_t STATE:3;                 /* RxFIFO State */
    vuint32_t _unused_3:3;
    vuint32_t HWLOCK:1;                /* Hardware Lock */
    vuint32_t SYSLOCK:1;               /* System Lock */
    vuint32_t HWPOINTER:6;             /* Hardware Pointer */
    vuint32_t _unused_14:2;
    vuint32_t SYSPOINTER:6;            /* System Pointer */
    vuint32_t _unused_22:10;
  } B;
} CANXL_RXFIFO_CONTROL_RXFCSTA_tag;

typedef union CANXL_RXFIFO_CONTROL_RXFSYSPOP_union_tag { /* RxFIFO POP Control */
  vuint32_t R;
  struct {
    vuint32_t RXPOP:5;                 /* System Host RxFIFO Pop Control */
    vuint32_t _unused_5:27;
  } B;
} CANXL_RXFIFO_CONTROL_RXFSYSPOP_tag;

typedef union CANXL_RXFIFO_CONTROL_RXFSYSLOCK_union_tag { /* RxFIFO System Lock Control */
  vuint32_t R;
  struct {
    vuint32_t SYSLOCK:1;               /* System Host Lock */
    vuint32_t _unused_1:31;
  } B;
} CANXL_RXFIFO_CONTROL_RXFSYSLOCK_tag;

typedef union CANXL_RXFIFO_CONTROL_RXFSYSACT_union_tag { /* RxFIFO Activation */
  vuint32_t R;
  struct {
    vuint32_t ACT:1;                   /* RxFIFO Activation */
    vuint32_t _unused_1:31;
  } B;
} CANXL_RXFIFO_CONTROL_RXFSYSACT_tag;

typedef union CANXL_RXFIFO_CONTROL_RXFIEN_union_tag { /* RxFIFO Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t RXFIE:1;                 /* RxFIFO Interrupt Enable */
    vuint32_t RXFEIE:1;                /* RxFIFO Error Interrupt Enable */
    vuint32_t _unused_2:30;
  } B;
} CANXL_RXFIFO_CONTROL_RXFIEN_tag;

typedef union CANXL_RXFIFO_CONTROL_RXFS_union_tag { /* RxFIFO Status */
  vuint32_t R;
  struct {
    vuint32_t RXFF:1;                  /* RxFIFO Flag */
    vuint32_t RXFEF:1;                 /* RxFIFO Error Flag */
    vuint32_t _unused_2:30;
  } B;
} CANXL_RXFIFO_CONTROL_RXFS_tag;

typedef union CANXL_RXFIFO_CONTROL_RXFC_union_tag { /* RxFIFO Configuration */
  vuint32_t R;
  struct {
    vuint32_t RXFD:5;                  /* RxFIFO Depth */
    vuint32_t _unused_5:3;
    vuint32_t RXFWTM:5;                /* RxFIFO Watermark Indication */
    vuint32_t _unused_13:19;
  } B;
} CANXL_RXFIFO_CONTROL_RXFC_tag;

struct CANXL_RXFIFO_CONTROL_tag {
  CANXL_RXFIFO_CONTROL_RXFCSTA_tag RXFCSTA; /* RxFIFO Control Status */
  CANXL_RXFIFO_CONTROL_RXFSYSPOP_tag RXFSYSPOP; /* RxFIFO POP Control */
  CANXL_RXFIFO_CONTROL_RXFSYSLOCK_tag RXFSYSLOCK; /* RxFIFO System Lock Control */
  uint8_t CANXL_RXFIFO_CONTROL_reserved0[8];
  CANXL_RXFIFO_CONTROL_RXFSYSACT_tag RXFSYSACT; /* RxFIFO Activation */
  uint8_t CANXL_RXFIFO_CONTROL_reserved1[8];
  CANXL_RXFIFO_CONTROL_RXFIEN_tag RXFIEN; /* RxFIFO Interrupt Enable */
  CANXL_RXFIFO_CONTROL_RXFS_tag RXFS;  /* RxFIFO Status */
  CANXL_RXFIFO_CONTROL_RXFC_tag RXFC;  /* RxFIFO Configuration */
};


/* ============================================================================
   =============================== Module: CANXL_SIC ==========================
   ============================================================================ */

typedef union CANXL_SIC_SYSMC_union_tag { /* System Module Control */
  vuint32_t R;
  struct {
    vuint32_t FRZREQ:1;                /* Freeze Mode Request */
    vuint32_t LPMREQ:1;                /* Low-Power Mode Request */
    vuint32_t _unused_2:29;
    vuint32_t SOFRST:1;                /* Software Reset */
  } B;
} CANXL_SIC_SYSMC_tag;

typedef union CANXL_SIC_SYSMCFG_union_tag { /* System Module Configuration */
  vuint32_t R;
  struct {
    vuint32_t FB1EN:1;                 /* Filter Bank 1 Enable */
    vuint32_t DRWRDIS:1;               /* Data RAM Write Access Disabled */
    vuint32_t _unused_2:1;
    vuint32_t MRP:1;                   /* Match Reception Priority */
    vuint32_t _unused_4:12;
    vuint32_t MAXTXMB:7;               /* Maximum Number of Transmit Message Descriptor */
    vuint32_t _unused_23:1;
    vuint32_t MAXRXMB:7;               /* Maximum Number of Receive Message Descriptor */
    vuint32_t _unused_31:1;
  } B;
} CANXL_SIC_SYSMCFG_tag;

typedef union CANXL_SIC_SYSIE_union_tag { /* System Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t FRZACKIE:1;              /* Freeze Acknowledge Interrupt Enable */
    vuint32_t LPMACKIE:1;              /* Low-power Mode Acknowledge Interrupt Enable */
    vuint32_t _unused_2:6;
    vuint32_t IERRIE:4;                /* Internal Error Interrupt Enable */
    vuint32_t _unused_12:4;
    vuint32_t CERRIE:1;                /* CANEXCEL Error Interrupt Enable */
    vuint32_t CPERRIE:1;               /* CANEXCEL Passive Error Interrupt Enable */
    vuint32_t CBOFFIE:1;               /* CANEXCEL Bus-off Interrupt Enable */
    vuint32_t CBDONEIE:1;              /* CANEXCEL Bus-Off Done Interrupt Enable */
    vuint32_t CRXWRNIE:1;              /* CANEXCEL RX Error Warning Interrupt Enable */
    vuint32_t CTXWRNIE:1;              /* CANEXCEL TX Error Warning Interrupt Enable */
    vuint32_t CFDPERRIE:1;             /* CANEXCEL FD Data Phase Error Interrupt Enable */
    vuint32_t CPREXCIE:1;              /* CANEXCEL Protocol Exception Interrupt Enable */
    vuint32_t CDPERRIE:1;              /* CANEXCEL Data Phase Error Interrupt Enable */
    vuint32_t CTFUERRIR:1;             /* CANEXCEL TX FIFO Underflow Error Interrupt Enable */
    vuint32_t CRFOERRIR:1;             /* CANEXCEL RX FIFO Overflow Interrupt Enable */
    vuint32_t CLSERRIE:1;              /* CANEXCEL LOM State Error Interrupt Enable */
    vuint32_t CMDOERRIE:1;             /* CANEXCEL MD Overrun Error Interrupt Enable */
    vuint32_t CMDUERRIE:1;             /* CANEXCEL MD Underrun Error Interrupt Enable */
    vuint32_t CRXSOERRIE:1;            /* CANEXCEL RX SMB Overrun Error Interrupt Enable */
    vuint32_t _unused_31:1;
  } B;
} CANXL_SIC_SYSIE_tag;

typedef union CANXL_SIC_SYSS_union_tag { /* System Status */
  vuint32_t R;
  struct {
    vuint32_t FRZACKF:1;               /* Freeze Mode Acknowledgment Flag */
    vuint32_t LPMACKF:1;               /* Low-Power Mode Acknowledgment Flag */
    vuint32_t NTRDY:1;                 /* CANEXCEL Not Ready */
    vuint32_t _unused_3:5;
    vuint32_t IERR:4;                  /* Internal Error Indication */
    vuint32_t _unused_12:4;
    vuint32_t CERR:1;                  /* CANEXCEL Error */
    vuint32_t CPASERR:1;               /* CANEXCEL Passive Error */
    vuint32_t CBOFF:1;                 /* CANEXCEL Bus-off */
    vuint32_t CBDONE:1;                /* CANEXCEL Bus-Off Done */
    vuint32_t CRXWRN:1;                /* CANEXCEL RX Error Warning */
    vuint32_t CTXWRN:1;                /* CANEXCEL TX Error Warning */
    vuint32_t CFDPERR:1;               /* CANEXCEL FD Data Phase Error */
    vuint32_t CPREXC:1;                /* CANEXCEL Protocol Exception */
    vuint32_t CXDPERR:1;               /* CANEXCEL Data Phase Error */
    vuint32_t CTFUFLW:1;               /* CANEXCEL TX FIFO Underflow */
    vuint32_t CRFOVR:1;                /* CANEXCEL RX FIFO Overflow */
    vuint32_t CLOMSERR:1;              /* CANEXCEL LOM State Error */
    vuint32_t CMDOERR:1;               /* CANEXCEL MD Overrun Error */
    vuint32_t CMDUERR:1;               /* CANEXCEL MD Underrun Error */
    vuint32_t CRXOERR:1;               /* CANEXCEL RX SMB Overrun Error */
    vuint32_t _unused_31:1;
  } B;
} CANXL_SIC_SYSS_tag;

typedef union CANXL_SIC_SYSFRTL_union_tag { /* System Free Running Timer Low */
  vuint32_t R;
  struct {
    vuint32_t FRTL:32;                 /* Free Running Timer Low */
  } B;
} CANXL_SIC_SYSFRTL_tag;

typedef union CANXL_SIC_SYSFRTH_union_tag { /* System Free Running Timer High */
  vuint32_t R;
  struct {
    vuint32_t FRTH:32;                 /* Free Running Timer High */
  } B;
} CANXL_SIC_SYSFRTH_tag;

typedef union CANXL_SIC_SYSMDHICFG_union_tag { /* System Message Descriptor Host Interrupt Configuration */
  vuint32_t R;
  struct {
    vuint32_t TXDCFGHI0_0:1;           /* Transmit Descriptor Configuration for Host Interrupt 0 */
    vuint32_t TXDCFGHI0_1:1;           /* Transmit Descriptor Configuration for Host Interrupt 0 */
    vuint32_t TXDCFGHI0_2:1;           /* Transmit Descriptor Configuration for Host Interrupt 0 */
    vuint32_t TXDCFGHI0_3:1;           /* Transmit Descriptor Configuration for Host Interrupt 0 */
    vuint32_t _unused_4:4;
    vuint32_t TXDCFGHI1_0:1;           /* Transmit Descriptor Configuration for Host Interrupt 1 */
    vuint32_t TXDCFGHI1_1:1;           /* Transmit Descriptor Configuration for Host Interrupt 1 */
    vuint32_t TXDCFGHI1_2:1;           /* Transmit Descriptor Configuration for Host Interrupt 1 */
    vuint32_t TXDCFGHI1_3:1;           /* Transmit Descriptor Configuration for Host Interrupt 1 */
    vuint32_t _unused_12:4;
    vuint32_t RXDCFGHI0_0:1;           /* Receive Descriptor Configuration for Host Interrupt 0 */
    vuint32_t RXDCFGHI0_1:1;           /* Receive Descriptor Configuration for Host Interrupt 0 */
    vuint32_t RXDCFGHI0_2:1;           /* Receive Descriptor Configuration for Host Interrupt 0 */
    vuint32_t RXDCFGHI0_3:1;           /* Receive Descriptor Configuration for Host Interrupt 0 */
    vuint32_t _unused_20:4;
    vuint32_t RXDCFGHI1_0:1;           /* Receive Descriptor Configuration for Host Interrupt 1 */
    vuint32_t RXDCFGHI1_1:1;           /* Receive Descriptor Configuration for Host Interrupt 1 */
    vuint32_t RXDCFGHI1_2:1;           /* Receive Descriptor Configuration for Host Interrupt 1 */
    vuint32_t RXDCFGHI1_3:1;           /* Receive Descriptor Configuration for Host Interrupt 1 */
    vuint32_t _unused_28:4;
  } B;
} CANXL_SIC_SYSMDHICFG_tag;

typedef union CANXL_SIC_BCFG1_union_tag { /* BCANXL Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t ABRDIS:1;                /* Automatic Bus-Off Recovery Disable */
    vuint32_t FDRSDIS:1;               /* FD Data Phase Rate Switch Disable */
    vuint32_t _unused_2:30;
  } B;
} CANXL_SIC_BCFG1_tag;

typedef union CANXL_SIC_BCFG2_union_tag { /* BCANXL Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t LPB:1;                   /* Loop-Back Mode */
    vuint32_t LOM:1;                   /* Listen-Only Mode */
    vuint32_t TSCAP:2;                 /* Time Stamp Capture Point */
    vuint32_t SRXEN:1;                 /* Self Reception Enable */
    vuint32_t FDEN:1;                  /* CAN FD Protocol Enable */
    vuint32_t PEXCE:1;                 /* CAN FD Protocol Exception Enable */
    vuint32_t EDFLTDIS:1;              /* Integration Edge Filter Disable */
    vuint32_t ROM:1;                   /* Restricted Operation Mode */
    vuint32_t XLEN:1;                  /* CAN XL Protocol Enable */
    vuint32_t XLER:1;                  /* CAN XL Error Response */
    vuint32_t TMSE:1;                  /* Transceiver Mode Switching Enable */
    vuint32_t TFER:1;                  /* Tx SMB Error Response */
    vuint32_t TSS:1;                   /* Time Stamp Size */
    vuint32_t _unused_14:15;
    vuint32_t FPES:1;                  /* FIFO Payload Endianness Selection */
    vuint32_t _unused_30:2;
  } B;
} CANXL_SIC_BCFG2_tag;

typedef union CANXL_SIC_BBPRS_union_tag { /* BCANXL Bit Time Quanta Prescaler */
  vuint32_t R;
  struct {
    vuint32_t PRESDIV:8;               /* Prescaler Divisor Factor */
    vuint32_t _unused_8:24;
  } B;
} CANXL_SIC_BBPRS_tag;

typedef union CANXL_SIC_BNCBT_union_tag { /* BCANXL Nominal Bit Timing */
  vuint32_t R;
  struct {
    vuint32_t NTSEG1:9;                /* Nominal Time Segment 1 */
    vuint32_t _unused_9:3;
    vuint32_t NTSEG2:7;                /* Nominal Time Segment 2 */
    vuint32_t _unused_19:3;
    vuint32_t NRJW:7;                  /* Nominal Resynchronization Jump Width */
    vuint32_t _unused_29:3;
  } B;
} CANXL_SIC_BNCBT_tag;

typedef union CANXL_SIC_BFDCBT_union_tag { /* BCANXL FD Data Phase Bit Timing */
  vuint32_t R;
  struct {
    vuint32_t FTSEG1:8;                /* FD Data Time Segment 1 */
    vuint32_t _unused_8:4;
    vuint32_t FTSEG2:7;                /* FD Data Time Segment 2 */
    vuint32_t _unused_19:3;
    vuint32_t FRJW:7;                  /* FD Data Phase Resynchronization Jump Width */
    vuint32_t _unused_29:3;
  } B;
} CANXL_SIC_BFDCBT_tag;

typedef union CANXL_SIC_BXDCBT_union_tag { /* BCANXL XL Data Phase Bit Timing */
  vuint32_t R;
  struct {
    vuint32_t XTSEG1:8;                /* XL Data Time Segment 1 */
    vuint32_t _unused_8:4;
    vuint32_t XTSEG2:7;                /* XL Data Time Segment 2 */
    vuint32_t _unused_19:3;
    vuint32_t XRJW:7;                  /* XL Data Phase Resynchronization Jump Width */
    vuint32_t _unused_29:3;
  } B;
} CANXL_SIC_BXDCBT_tag;

typedef union CANXL_SIC_BTDCC_union_tag { /* BCANXL Transceiver Delay Compensation Control */
  vuint32_t R;
  struct {
    vuint32_t FTDCOFF:7;               /* CAN FD Transceiver Delay Compensation Offset */
    vuint32_t _unused_7:7;
    vuint32_t FTDMDIS:1;               /* CAN FD Transceiver Delay Measurement Disabled */
    vuint32_t FTDCEN:1;                /* CAN FD Transceiver Delay Compensation Enable */
    vuint32_t XTDCOFF:8;               /* CAN XL Transceiver Delay Compensation Offset */
    vuint32_t _unused_24:6;
    vuint32_t XTDMDIS:1;               /* CAN XL Transceiver Delay Measurement Disabled */
    vuint32_t XTDCEN:1;                /* CAN XL Transceiver Delay Compensation Enable */
  } B;
} CANXL_SIC_BTDCC_tag;

typedef union CANXL_SIC_BMICI_union_tag { /* BCANXL PWM Configuration */
  vuint32_t R;
  struct {
    vuint32_t PWMO:6;                  /* PWM Offset */
    vuint32_t _unused_6:2;
    vuint32_t PWML:6;                  /* PWM Long Phase */
    vuint32_t _unused_14:2;
    vuint32_t PWMS:6;                  /* PWM Short Phase */
    vuint32_t _unused_22:10;
  } B;
} CANXL_SIC_BMICI_tag;

struct CANXL_SIC_tag {
  uint8_t CANXL_SIC_reserved0[64];
  CANXL_SIC_SYSMC_tag SYSMC;           /* System Module Control */
  CANXL_SIC_SYSMCFG_tag SYSMCFG;       /* System Module Configuration */
  CANXL_SIC_SYSIE_tag SYSIE;           /* System Interrupt Enable */
  CANXL_SIC_SYSS_tag SYSS;             /* System Status */
  CANXL_SIC_SYSFRTL_tag SYSFRTL;       /* System Free Running Timer Low */
  CANXL_SIC_SYSFRTH_tag SYSFRTH;       /* System Free Running Timer High */
  CANXL_SIC_SYSMDHICFG_tag SYSMDHICFG; /* System Message Descriptor Host Interrupt Configuration */
  uint8_t CANXL_SIC_reserved1[36];
  CANXL_SIC_BCFG1_tag BCFG1;           /* BCANXL Configuration 1 */
  CANXL_SIC_BCFG2_tag BCFG2;           /* BCANXL Configuration 2 */
  uint8_t CANXL_SIC_reserved2[4];
  CANXL_SIC_BBPRS_tag BBPRS;           /* BCANXL Bit Time Quanta Prescaler */
  CANXL_SIC_BNCBT_tag BNCBT;           /* BCANXL Nominal Bit Timing */
  CANXL_SIC_BFDCBT_tag BFDCBT;         /* BCANXL FD Data Phase Bit Timing */
  CANXL_SIC_BXDCBT_tag BXDCBT;         /* BCANXL XL Data Phase Bit Timing */
  CANXL_SIC_BTDCC_tag BTDCC;           /* BCANXL Transceiver Delay Compensation Control */
  CANXL_SIC_BMICI_tag BMICI;           /* BCANXL PWM Configuration */
};


/* ============================================================================
   =============================== Module: CAN_HUB ============================
   ============================================================================ */

typedef union CAN_HUB_CANNETW_union_tag { /* CAN Module Connected to Bus Flag */
  vuint32_t R;
  struct {
    vuint32_t STATUS:24;               /* CAN Module Connected to Bus Flag */
    vuint32_t _unused_24:8;
  } B;
} CAN_HUB_CANNETW_tag;

typedef union CAN_HUB_CAN0ALWGRP_union_tag { /* CAN0 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN0 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN0ALWGRP_tag;

typedef union CAN_HUB_CAN1ALWGRP_union_tag { /* CAN1 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN1 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN1ALWGRP_tag;

typedef union CAN_HUB_CAN2ALWGRP_union_tag { /* CAN2 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN2 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN2ALWGRP_tag;

typedef union CAN_HUB_CAN3ALWGRP_union_tag { /* CAN3 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN3 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN3ALWGRP_tag;

typedef union CAN_HUB_CAN4ALWGRP_union_tag { /* CAN4 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN4 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN4ALWGRP_tag;

typedef union CAN_HUB_CAN5ALWGRP_union_tag { /* CAN5 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN4 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN5ALWGRP_tag;

typedef union CAN_HUB_CAN6ALWGRP_union_tag { /* CAN6 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN0 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN6ALWGRP_tag;

typedef union CAN_HUB_CAN7ALWGRP_union_tag { /* CAN7 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN7 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN7ALWGRP_tag;

typedef union CAN_HUB_CAN8ALWGRP_union_tag { /* CAN8 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN8 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN8ALWGRP_tag;

typedef union CAN_HUB_CAN9ALWGRP_union_tag { /* CAN9 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN9 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN9ALWGRP_tag;

typedef union CAN_HUB_CAN10ALWGRP_union_tag { /* CAN10 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN10 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN10ALWGRP_tag;

typedef union CAN_HUB_CAN11ALWGRP_union_tag { /* CAN11 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN4 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN11ALWGRP_tag;

typedef union CAN_HUB_CAN12ALWGRP_union_tag { /* CAN12 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN12 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN12ALWGRP_tag;

typedef union CAN_HUB_CAN13ALWGRP_union_tag { /* CAN13 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN13 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN13ALWGRP_tag;

typedef union CAN_HUB_CAN14ALWGRP_union_tag { /* CAN14 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN14 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN14ALWGRP_tag;

typedef union CAN_HUB_CAN15ALWGRP_union_tag { /* CAN15 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN15 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN15ALWGRP_tag;

typedef union CAN_HUB_CAN16ALWGRP_union_tag { /* CAN16 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN16 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN16ALWGRP_tag;

typedef union CAN_HUB_CAN17ALWGRP_union_tag { /* CAN17 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN17 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN17ALWGRP_tag;

typedef union CAN_HUB_CAN18ALWGRP_union_tag { /* CAN18 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN18 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN18ALWGRP_tag;

typedef union CAN_HUB_CAN19ALWGRP_union_tag { /* CAN19 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN19 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN19ALWGRP_tag;

typedef union CAN_HUB_CAN20ALWGRP_union_tag { /* CAN20 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN20 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN20ALWGRP_tag;

typedef union CAN_HUB_CAN21ALWGRP_union_tag { /* CAN21 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN21 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN21ALWGRP_tag;

typedef union CAN_HUB_CAN22ALWGRP_union_tag { /* CAN22 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN22 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN22ALWGRP_tag;

typedef union CAN_HUB_CAN23ALWGRP_union_tag { /* CAN23 Allowable Groups */
  vuint32_t R;
  struct {
    vuint32_t ALWGRP:27;               /* CAN23 Allowable Groups */
    vuint32_t _unused_27:5;
  } B;
} CAN_HUB_CAN23ALWGRP_tag;

typedef union CAN_HUB_CAN0GRPCTL_union_tag { /* CAN0 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN0GRPCTL_tag;

typedef union CAN_HUB_CAN1GRPCTL_union_tag { /* CAN1 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN1GRPCTL_tag;

typedef union CAN_HUB_CAN2GRPCTL_union_tag { /* CAN2 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN2GRPCTL_tag;

typedef union CAN_HUB_CAN3GRPCTL_union_tag { /* CAN3 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN3GRPCTL_tag;

typedef union CAN_HUB_CAN4GRPCTL_union_tag { /* CAN4 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN4GRPCTL_tag;

typedef union CAN_HUB_CAN5GRPCTL_union_tag { /* CAN5 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN5GRPCTL_tag;

typedef union CAN_HUB_CAN6GRPCTL_union_tag { /* CAN6 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN6GRPCTL_tag;

typedef union CAN_HUB_CAN7GRPCTL_union_tag { /* CAN7 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN7GRPCTL_tag;

typedef union CAN_HUB_CAN8GRPCTL_union_tag { /* CAN8 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN8GRPCTL_tag;

typedef union CAN_HUB_CAN9GRPCTL_union_tag { /* CAN9 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN9GRPCTL_tag;

typedef union CAN_HUB_CAN10GRPCTL_union_tag { /* CAN10 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN10GRPCTL_tag;

typedef union CAN_HUB_CAN11GRPCTL_union_tag { /* CAN11 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN11GRPCTL_tag;

typedef union CAN_HUB_CAN12GRPCTL_union_tag { /* CAN12 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN12GRPCTL_tag;

typedef union CAN_HUB_CAN13GRPCTL_union_tag { /* CAN13 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN13GRPCTL_tag;

typedef union CAN_HUB_CAN14GRPCTL_union_tag { /* CAN14 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN14GRPCTL_tag;

typedef union CAN_HUB_CAN15GRPCTL_union_tag { /* CAN15 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN15GRPCTL_tag;

typedef union CAN_HUB_CAN16GRPCTL_union_tag { /* CAN16 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN16GRPCTL_tag;

typedef union CAN_HUB_CAN17GRPCTL_union_tag { /* CAN17 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN17GRPCTL_tag;

typedef union CAN_HUB_CAN18GRPCTL_union_tag { /* CAN18 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN18GRPCTL_tag;

typedef union CAN_HUB_CAN19GRPCTL_union_tag { /* CAN19 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN19GRPCTL_tag;

typedef union CAN_HUB_CAN20GRPCTL_union_tag { /* CAN20 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN20GRPCTL_tag;

typedef union CAN_HUB_CAN21GRPCTL_union_tag { /* CAN21 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN21GRPCTL_tag;

typedef union CAN_HUB_CAN22GRPCTL_union_tag { /* CAN22 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN22GRPCTL_tag;

typedef union CAN_HUB_CAN23GRPCTL_union_tag { /* CAN23 Module Group Control Register */
  vuint32_t R;
  struct {
    vuint32_t USECANBUS:5;             /* Use CAN Bus */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:8;
    vuint32_t HWFLTR:6;                /* HW Fault Reaction */
    vuint32_t _unused_22:10;
  } B;
} CAN_HUB_CAN23GRPCTL_tag;

struct CAN_HUB_tag {
  CAN_HUB_CANNETW_tag CANNETW;         /* CAN Module Connected to Bus Flag */
  uint8_t CAN_HUB_reserved0[124];
  CAN_HUB_CAN0ALWGRP_tag CAN0ALWGRP;   /* CAN0 Allowable Groups */
  CAN_HUB_CAN1ALWGRP_tag CAN1ALWGRP;   /* CAN1 Allowable Groups */
  CAN_HUB_CAN2ALWGRP_tag CAN2ALWGRP;   /* CAN2 Allowable Groups */
  CAN_HUB_CAN3ALWGRP_tag CAN3ALWGRP;   /* CAN3 Allowable Groups */
  CAN_HUB_CAN4ALWGRP_tag CAN4ALWGRP;   /* CAN4 Allowable Groups */
  CAN_HUB_CAN5ALWGRP_tag CAN5ALWGRP;   /* CAN5 Allowable Groups */
  CAN_HUB_CAN6ALWGRP_tag CAN6ALWGRP;   /* CAN6 Allowable Groups */
  CAN_HUB_CAN7ALWGRP_tag CAN7ALWGRP;   /* CAN7 Allowable Groups */
  CAN_HUB_CAN8ALWGRP_tag CAN8ALWGRP;   /* CAN8 Allowable Groups */
  CAN_HUB_CAN9ALWGRP_tag CAN9ALWGRP;   /* CAN9 Allowable Groups */
  CAN_HUB_CAN10ALWGRP_tag CAN10ALWGRP; /* CAN10 Allowable Groups */
  CAN_HUB_CAN11ALWGRP_tag CAN11ALWGRP; /* CAN11 Allowable Groups */
  CAN_HUB_CAN12ALWGRP_tag CAN12ALWGRP; /* CAN12 Allowable Groups */
  CAN_HUB_CAN13ALWGRP_tag CAN13ALWGRP; /* CAN13 Allowable Groups */
  CAN_HUB_CAN14ALWGRP_tag CAN14ALWGRP; /* CAN14 Allowable Groups */
  CAN_HUB_CAN15ALWGRP_tag CAN15ALWGRP; /* CAN15 Allowable Groups */
  CAN_HUB_CAN16ALWGRP_tag CAN16ALWGRP; /* CAN16 Allowable Groups */
  CAN_HUB_CAN17ALWGRP_tag CAN17ALWGRP; /* CAN17 Allowable Groups */
  CAN_HUB_CAN18ALWGRP_tag CAN18ALWGRP; /* CAN18 Allowable Groups */
  CAN_HUB_CAN19ALWGRP_tag CAN19ALWGRP; /* CAN19 Allowable Groups */
  CAN_HUB_CAN20ALWGRP_tag CAN20ALWGRP; /* CAN20 Allowable Groups */
  CAN_HUB_CAN21ALWGRP_tag CAN21ALWGRP; /* CAN21 Allowable Groups */
  CAN_HUB_CAN22ALWGRP_tag CAN22ALWGRP; /* CAN22 Allowable Groups */
  CAN_HUB_CAN23ALWGRP_tag CAN23ALWGRP; /* CAN23 Allowable Groups */
  uint8_t CAN_HUB_reserved1[32];
  CAN_HUB_CAN0GRPCTL_tag CAN0GRPCTL;   /* CAN0 Module Group Control Register */
  CAN_HUB_CAN1GRPCTL_tag CAN1GRPCTL;   /* CAN1 Module Group Control Register */
  CAN_HUB_CAN2GRPCTL_tag CAN2GRPCTL;   /* CAN2 Module Group Control Register */
  CAN_HUB_CAN3GRPCTL_tag CAN3GRPCTL;   /* CAN3 Module Group Control Register */
  CAN_HUB_CAN4GRPCTL_tag CAN4GRPCTL;   /* CAN4 Module Group Control Register */
  CAN_HUB_CAN5GRPCTL_tag CAN5GRPCTL;   /* CAN5 Module Group Control Register */
  CAN_HUB_CAN6GRPCTL_tag CAN6GRPCTL;   /* CAN6 Module Group Control Register */
  CAN_HUB_CAN7GRPCTL_tag CAN7GRPCTL;   /* CAN7 Module Group Control Register */
  CAN_HUB_CAN8GRPCTL_tag CAN8GRPCTL;   /* CAN8 Module Group Control Register */
  CAN_HUB_CAN9GRPCTL_tag CAN9GRPCTL;   /* CAN9 Module Group Control Register */
  CAN_HUB_CAN10GRPCTL_tag CAN10GRPCTL; /* CAN10 Module Group Control Register */
  CAN_HUB_CAN11GRPCTL_tag CAN11GRPCTL; /* CAN11 Module Group Control Register */
  CAN_HUB_CAN12GRPCTL_tag CAN12GRPCTL; /* CAN12 Module Group Control Register */
  CAN_HUB_CAN13GRPCTL_tag CAN13GRPCTL; /* CAN13 Module Group Control Register */
  CAN_HUB_CAN14GRPCTL_tag CAN14GRPCTL; /* CAN14 Module Group Control Register */
  CAN_HUB_CAN15GRPCTL_tag CAN15GRPCTL; /* CAN15 Module Group Control Register */
  CAN_HUB_CAN16GRPCTL_tag CAN16GRPCTL; /* CAN16 Module Group Control Register */
  CAN_HUB_CAN17GRPCTL_tag CAN17GRPCTL; /* CAN17 Module Group Control Register */
  CAN_HUB_CAN18GRPCTL_tag CAN18GRPCTL; /* CAN18 Module Group Control Register */
  CAN_HUB_CAN19GRPCTL_tag CAN19GRPCTL; /* CAN19 Module Group Control Register */
  CAN_HUB_CAN20GRPCTL_tag CAN20GRPCTL; /* CAN20 Module Group Control Register */
  CAN_HUB_CAN21GRPCTL_tag CAN21GRPCTL; /* CAN21 Module Group Control Register */
  CAN_HUB_CAN22GRPCTL_tag CAN22GRPCTL; /* CAN22 Module Group Control Register */
  CAN_HUB_CAN23GRPCTL_tag CAN23GRPCTL; /* CAN23 Module Group Control Register */
};


/* ============================================================================
   =============================== Module: CAN_TBS ============================
   ============================================================================ */

typedef union CAN_TBS_CAN_TB_VALUE_union_tag { /* CAN Time Base Output Value */
  vuint32_t R;
  struct {
    vuint32_t TB_VALUE:32;             /* CAN Time Base Output Value */
  } B;
} CAN_TBS_CAN_TB_VALUE_tag;

typedef union CAN_TBS_CAN_TS_SEL_union_tag { /* CAN Timestamp Mux Selector */
  vuint32_t R;
  struct {
    vuint32_t TS_SEL:2;                /* CAN Timestamp Source Selection */
    vuint32_t _unused_2:30;
  } B;
} CAN_TBS_CAN_TS_SEL_tag;

struct CAN_TBS_tag {
  CAN_TBS_CAN_TB_VALUE_tag CAN_TB_VALUE; /* CAN Time Base Output Value */
  uint8_t CAN_TBS_reserved0[2044];
  CAN_TBS_CAN_TS_SEL_tag CAN_TS_SEL;   /* CAN Timestamp Mux Selector */
};


/* ============================================================================
   =============================== Module: CE_DMAMUX ==========================
   ============================================================================ */

typedef union CE_DMAMUX_CHCONF_union_tag { /* Channel Configuration */
  vuint8_t R;
  struct {
    vuint8_t SOURCE:6;                 /* DMA Channel Source (Slot) */
    vuint8_t TRIG:1;                   /* DMA Channel Trigger Enable */
    vuint8_t ENBL:1;                   /* DMA Channel Enable */
  } B;
} CE_DMAMUX_CHCONF_tag;

struct CE_DMAMUX_tag {
  CE_DMAMUX_CHCONF_tag CHCONF[32];     /* Channel Configuration */
};


/* ============================================================================
   =============================== Module: CE_L_VFCCU =========================
   ============================================================================ */

typedef union CE_L_VFCCU_GDFHID_C0_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID0:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID1:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID2:3;                /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID3:3;                /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID4:3;                /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID5:3;                /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID6:3;                /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID7:3;                /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} CE_L_VFCCU_GDFHID_C0_tag;

typedef union CE_L_VFCCU_GDFHID_C1_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID8:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID9:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID10:3;               /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID11:3;               /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID12:3;               /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID13:3;               /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID14:3;               /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID15:3;               /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} CE_L_VFCCU_GDFHID_C1_tag;

typedef union CE_L_VFCCU_GFLTPO_C0_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS0:1;                   /* Polarity Selection */
    vuint32_t PS1:1;                   /* Polarity Selection */
    vuint32_t PS2:1;                   /* Polarity Selection */
    vuint32_t PS3:1;                   /* Polarity Selection */
    vuint32_t PS4:1;                   /* Polarity Selection */
    vuint32_t PS5:1;                   /* Polarity Selection */
    vuint32_t PS6:1;                   /* Polarity Selection */
    vuint32_t PS7:1;                   /* Polarity Selection */
    vuint32_t PS8:1;                   /* Polarity Selection */
    vuint32_t PS9:1;                   /* Polarity Selection */
    vuint32_t PS10:1;                  /* Polarity Selection */
    vuint32_t PS11:1;                  /* Polarity Selection */
    vuint32_t PS12:1;                  /* Polarity Selection */
    vuint32_t PS13:1;                  /* Polarity Selection */
    vuint32_t PS14:1;                  /* Polarity Selection */
    vuint32_t PS15:1;                  /* Polarity Selection */
    vuint32_t PS16:1;                  /* Polarity Selection */
    vuint32_t PS17:1;                  /* Polarity Selection */
    vuint32_t PS18:1;                  /* Polarity Selection */
    vuint32_t PS19:1;                  /* Polarity Selection */
    vuint32_t PS20:1;                  /* Polarity Selection */
    vuint32_t PS21:1;                  /* Polarity Selection */
    vuint32_t PS22:1;                  /* Polarity Selection */
    vuint32_t PS23:1;                  /* Polarity Selection */
    vuint32_t PS24:1;                  /* Polarity Selection */
    vuint32_t PS25:1;                  /* Polarity Selection */
    vuint32_t PS26:1;                  /* Polarity Selection */
    vuint32_t PS27:1;                  /* Polarity Selection */
    vuint32_t PS28:1;                  /* Polarity Selection */
    vuint32_t PS29:1;                  /* Polarity Selection */
    vuint32_t PS30:1;                  /* Polarity Selection */
    vuint32_t PS31:1;                  /* Polarity Selection */
  } B;
} CE_L_VFCCU_GFLTPO_C0_tag;

typedef union CE_L_VFCCU_GFLTPO_C1_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS32:1;                  /* Polarity Selection */
    vuint32_t PS33:1;                  /* Polarity Selection */
    vuint32_t PS34:1;                  /* Polarity Selection */
    vuint32_t PS35:1;                  /* Polarity Selection */
    vuint32_t PS36:1;                  /* Polarity Selection */
    vuint32_t PS37:1;                  /* Polarity Selection */
    vuint32_t PS38:1;                  /* Polarity Selection */
    vuint32_t PS39:1;                  /* Polarity Selection */
    vuint32_t PS40:1;                  /* Polarity Selection */
    vuint32_t PS41:1;                  /* Polarity Selection */
    vuint32_t PS42:1;                  /* Polarity Selection */
    vuint32_t PS43:1;                  /* Polarity Selection */
    vuint32_t PS44:1;                  /* Polarity Selection */
    vuint32_t PS45:1;                  /* Polarity Selection */
    vuint32_t PS46:1;                  /* Polarity Selection */
    vuint32_t PS47:1;                  /* Polarity Selection */
    vuint32_t PS48:1;                  /* Polarity Selection */
    vuint32_t PS49:1;                  /* Polarity Selection */
    vuint32_t PS50:1;                  /* Polarity Selection */
    vuint32_t PS51:1;                  /* Polarity Selection */
    vuint32_t PS52:1;                  /* Polarity Selection */
    vuint32_t _unused_21:11;
  } B;
} CE_L_VFCCU_GFLTPO_C1_tag;

typedef union CE_L_VFCCU_GFLTRC_C0_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW0:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW1:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW2:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW3:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW4:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW5:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW6:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW7:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW8:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW9:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW10:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW11:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW12:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW13:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW14:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW15:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW16:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW17:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW18:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW19:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW20:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW21:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW22:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW23:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW24:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW25:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW26:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW27:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW28:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW29:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW30:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW31:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
  } B;
} CE_L_VFCCU_GFLTRC_C0_tag;

typedef union CE_L_VFCCU_GFLTRC_C1_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW32:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW33:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW34:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW35:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW36:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW37:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW38:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW39:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW40:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW41:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW42:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW43:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW44:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW45:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW46:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW47:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW48:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW49:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW50:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW51:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW52:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t _unused_21:11;
  } B;
} CE_L_VFCCU_GFLTRC_C1_tag;

typedef union CE_L_VFCCU_GFLTOVDC0_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS0:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS1:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS2:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS3:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS4:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS5:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS6:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS7:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS8:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS9:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS10:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS11:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS12:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS13:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS14:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS15:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS16:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS17:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS18:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS19:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS20:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS21:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS22:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS23:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS24:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS25:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS26:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS27:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS28:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS29:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS30:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS31:1;             /* Fault Overflow Detection Disable */
  } B;
} CE_L_VFCCU_GFLTOVDC0_tag;

typedef union CE_L_VFCCU_GFLTOVDC1_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS32:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS33:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS34:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS35:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS36:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS37:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS38:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS39:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS40:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS41:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS42:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS43:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS44:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS45:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS46:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS47:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS48:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS49:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS50:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS51:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS52:1;             /* Fault Overflow Detection Disable */
    vuint32_t _unused_21:11;
  } B;
} CE_L_VFCCU_GFLTOVDC1_tag;

typedef union CE_L_VFCCU_GSWFLODC_union_tag { /* Global Software Fault Overflow Detection Disable */
  vuint32_t R;
  struct {
    vuint32_t SWOVF_DIS0:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS1:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS2:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS3:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS4:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS5:1;            /* Software Overflow Disable Detection */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_GSWFLODC_tag;

typedef union CE_L_VFCCU_GCTRL_union_tag { /* Global Space Control */
  vuint32_t R;
  struct {
    vuint32_t OVF_EN:1;                /* Overflow Enable */
    vuint32_t _unused_1:31;
  } B;
} CE_L_VFCCU_GCTRL_tag;

typedef union CE_L_VFCCU_GINTOVFS_union_tag { /* Global DID FSM Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t FLTSERV:1;               /* Fault Serve */
    vuint32_t OVF_DET:1;               /* Overflow Detect */
    vuint32_t _unused_9:7;
    vuint32_t SERV_DID:4;              /* Domain Being Serviced */
    vuint32_t _unused_20:4;
    vuint32_t OVF_DID:4;               /* Overflow DID */
    vuint32_t _unused_28:4;
  } B;
} CE_L_VFCCU_GINTOVFS_tag;

typedef union CE_L_VFCCU_GDBGCFG_union_tag { /* Global Debug */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t FRZ:1;                   /* Freeze On First Fault */
    vuint32_t _unused_17:15;
  } B;
} CE_L_VFCCU_GDBGCFG_tag;

typedef union CE_L_VFCCU_GDBGSTAT_union_tag { /* Global Debug Status */
  vuint32_t R;
  struct {
    vuint32_t FLTIND:8;                /* Fault Index */
    vuint32_t _unused_8:24;
  } B;
} CE_L_VFCCU_GDBGSTAT_tag;

typedef union CE_L_VFCCU_SW_RKN_SWRPTDID_union_tag { /* Software Fault Reported DID */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Software Fault DID */
    vuint32_t _unused_4:28;
  } B;
} CE_L_VFCCU_SW_RKN_SWRPTDID_tag;

typedef union CE_L_VFCCU_SW_RKN_SWRKSET_0_union_tag { /* Software Reaction Set */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_SW_RKN_SWRKSET_0_tag;

typedef union CE_L_VFCCU_SW_RKN_SWRKCLR_0_union_tag { /* Software Reaction Clear */
  vuint32_t R;
  struct {
    vuint32_t RKNCLR0:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR1:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR2:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR3:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR4:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR5:1;               /* Reaction Line Clear */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_SW_RKN_SWRKCLR_0_tag;

typedef struct CE_L_VFCCU_SW_RKN_struct_tag {
  CE_L_VFCCU_SW_RKN_SWRPTDID_tag SWRPTDID; /* Software Fault Reported DID */
  CE_L_VFCCU_SW_RKN_SWRKSET_0_tag SWRKSET_0; /* Software Reaction Set */
  uint8_t SW_RKN_reserved0[4];
  CE_L_VFCCU_SW_RKN_SWRKCLR_0_tag SWRKCLR_0; /* Software Reaction Clear */
  uint8_t SW_RKN_reserved1[16];
} CE_L_VFCCU_SW_RKN_tag;

typedef union CE_L_VFCCU_FHCFG0_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} CE_L_VFCCU_FHCFG0_tag;

typedef union CE_L_VFCCU_FHSRVDS0_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} CE_L_VFCCU_FHSRVDS0_tag;

typedef union CE_L_VFCCU_FHFLTENC00_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} CE_L_VFCCU_FHFLTENC00_tag;

typedef union CE_L_VFCCU_FHFLTENC01_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t _unused_21:11;
  } B;
} CE_L_VFCCU_FHFLTENC01_tag;

typedef union CE_L_VFCCU_FHFLTS00_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} CE_L_VFCCU_FHFLTS00_tag;

typedef union CE_L_VFCCU_FHFLTS01_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t _unused_21:11;
  } B;
} CE_L_VFCCU_FHFLTS01_tag;

typedef union CE_L_VFCCU_FHFLTRKC00_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} CE_L_VFCCU_FHFLTRKC00_tag;

typedef union CE_L_VFCCU_FHFLTRKC01_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} CE_L_VFCCU_FHFLTRKC01_tag;

typedef union CE_L_VFCCU_FHFLTRKC02_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} CE_L_VFCCU_FHFLTRKC02_tag;

typedef union CE_L_VFCCU_FHFLTRKC03_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} CE_L_VFCCU_FHFLTRKC03_tag;

typedef union CE_L_VFCCU_FHFLTRKC04_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} CE_L_VFCCU_FHFLTRKC04_tag;

typedef union CE_L_VFCCU_FHFLTRKC05_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} CE_L_VFCCU_FHFLTRKC05_tag;

typedef union CE_L_VFCCU_FHFLTRKC06_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:12;
  } B;
} CE_L_VFCCU_FHFLTRKC06_tag;

typedef union CE_L_VFCCU_FHIMRKC0_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_FHIMRKC0_00_tag;

typedef union CE_L_VFCCU_FHIMRKC0_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_FHIMRKC0_10_tag;

typedef union CE_L_VFCCU_FHIMRKC0_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_FHIMRKC0_20_tag;

typedef union CE_L_VFCCU_FHIMRKC0_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_FHIMRKC0_30_tag;

typedef union CE_L_VFCCU_FHIMRKC0_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_FHIMRKC0_40_tag;

typedef union CE_L_VFCCU_FHIMRKC0_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} CE_L_VFCCU_FHIMRKC0_50_tag;

struct CE_L_VFCCU_tag {
  CE_L_VFCCU_GDFHID_C0_tag GDFHID_C0;  /* Global DID-FHID Map */
  CE_L_VFCCU_GDFHID_C1_tag GDFHID_C1;  /* Global DID-FHID Map */
  uint8_t CE_L_VFCCU_reserved0[24];
  CE_L_VFCCU_GFLTPO_C0_tag GFLTPO_C0;  /* Global Fault Polarity */
  CE_L_VFCCU_GFLTPO_C1_tag GFLTPO_C1;  /* Global Fault Polarity */
  uint8_t CE_L_VFCCU_reserved1[56];
  CE_L_VFCCU_GFLTRC_C0_tag GFLTRC_C0;  /* Global Fault Recovery */
  CE_L_VFCCU_GFLTRC_C1_tag GFLTRC_C1;  /* Global Fault Recovery */
  uint8_t CE_L_VFCCU_reserved2[56];
  CE_L_VFCCU_GFLTOVDC0_tag GFLTOVDC0;  /* Global Fault Overflow Detection */
  CE_L_VFCCU_GFLTOVDC1_tag GFLTOVDC1;  /* Global Fault Overflow Detection */
  uint8_t CE_L_VFCCU_reserved3[104];
  CE_L_VFCCU_GSWFLODC_tag GSWFLODC;    /* Global Software Fault Overflow Detection Disable */
  uint8_t CE_L_VFCCU_reserved4[28];
  CE_L_VFCCU_GCTRL_tag GCTRL;          /* Global Space Control */
  CE_L_VFCCU_GINTOVFS_tag GINTOVFS;    /* Global DID FSM Status */
  uint8_t CE_L_VFCCU_reserved5[428];
  CE_L_VFCCU_GDBGCFG_tag GDBGCFG;      /* Global Debug */
  CE_L_VFCCU_GDBGSTAT_tag GDBGSTAT;    /* Global Debug Status */
  uint8_t CE_L_VFCCU_reserved6[15636];
  CE_L_VFCCU_SW_RKN_tag SW_RKN[6];
  uint8_t CE_L_VFCCU_reserved7[16192];
  CE_L_VFCCU_FHCFG0_tag FHCFG0;        /* Fault Handler */
  CE_L_VFCCU_FHSRVDS0_tag FHSRVDS0;    /* Fault Handler Status */
  uint8_t CE_L_VFCCU_reserved8[8];
  CE_L_VFCCU_FHFLTENC00_tag FHFLTENC00; /* Fault Enable */
  CE_L_VFCCU_FHFLTENC01_tag FHFLTENC01; /* Fault Enable */
  uint8_t CE_L_VFCCU_reserved9[56];
  CE_L_VFCCU_FHFLTS00_tag FHFLTS00;    /* Fault Status */
  CE_L_VFCCU_FHFLTS01_tag FHFLTS01;    /* Fault Status */
  uint8_t CE_L_VFCCU_reserved10[56];
  CE_L_VFCCU_FHFLTRKC00_tag FHFLTRKC00; /* Fault Reaction Set Configuration */
  CE_L_VFCCU_FHFLTRKC01_tag FHFLTRKC01; /* Fault Reaction Set Configuration */
  CE_L_VFCCU_FHFLTRKC02_tag FHFLTRKC02; /* Fault Reaction Set Configuration */
  CE_L_VFCCU_FHFLTRKC03_tag FHFLTRKC03; /* Fault Reaction Set Configuration */
  CE_L_VFCCU_FHFLTRKC04_tag FHFLTRKC04; /* Fault Reaction Set Configuration */
  CE_L_VFCCU_FHFLTRKC05_tag FHFLTRKC05; /* Fault Reaction Set Configuration */
  CE_L_VFCCU_FHFLTRKC06_tag FHFLTRKC06; /* Fault Reaction Set Configuration */
  uint8_t CE_L_VFCCU_reserved11[228];
  CE_L_VFCCU_FHIMRKC0_00_tag FHIMRKC0_00; /* Immediate Reaction Configuration */
  uint8_t CE_L_VFCCU_reserved12[12];
  CE_L_VFCCU_FHIMRKC0_10_tag FHIMRKC0_10; /* Immediate Reaction Configuration */
  uint8_t CE_L_VFCCU_reserved13[12];
  CE_L_VFCCU_FHIMRKC0_20_tag FHIMRKC0_20; /* Immediate Reaction Configuration */
  uint8_t CE_L_VFCCU_reserved14[12];
  CE_L_VFCCU_FHIMRKC0_30_tag FHIMRKC0_30; /* Immediate Reaction Configuration */
  uint8_t CE_L_VFCCU_reserved15[12];
  CE_L_VFCCU_FHIMRKC0_40_tag FHIMRKC0_40; /* Immediate Reaction Configuration */
  uint8_t CE_L_VFCCU_reserved16[12];
  CE_L_VFCCU_FHIMRKC0_50_tag FHIMRKC0_50; /* Immediate Reaction Configuration */
};


/* ============================================================================
   =============================== Module: CE_SEMA42 ==========================
   ============================================================================ */

typedef union CE_SEMA42_GATE_union_tag { /* Gate */
  vuint8_t R;
  struct {
    vuint8_t GTFSM:4;                  /* Gate Finite State Machine */
    vuint8_t _unused_4:4;              /* Reserved */
  } B;
} CE_SEMA42_GATE_tag;

typedef union CE_SEMA42_RSTGT_R_union_tag { /* Reset Gate Read */
  vuint16_t R;
  struct {
    vuint16_t RSTGTN:8;                /* Reset Gate Number */
    vuint16_t RSTGMS:4;                /* Reset Gate Domain */
    vuint16_t RSTGSM:2;                /* Reset Gate Finite State Machine */
    vuint16_t _unused_14:2;
  } B;
} CE_SEMA42_RSTGT_R_tag;

typedef union CE_SEMA42_RSTGT_W_union_tag { /* Reset Gate Write */
  vuint16_t R;
  struct {
    vuint16_t RSTGTN:8;                /* Reset Gate Number */
    vuint16_t RSTGDP:8;                /* Reset Gate Data Pattern */
  } B;
} CE_SEMA42_RSTGT_W_tag;

typedef union CE_SEMA42_RSTGT_union_tag {
  CE_SEMA42_RSTGT_R_tag R;             /* Reset Gate Read */
  CE_SEMA42_RSTGT_W_tag W;             /* Reset Gate Write */
} CE_SEMA42_RSTGT_tag;

struct CE_SEMA42_tag {
  CE_SEMA42_GATE_tag GATE[16];         /* Gate */
  uint8_t CE_SEMA42_reserved0[50];
  CE_SEMA42_RSTGT_tag RSTGT;
};


/* ============================================================================
   =============================== Module: CE_SRG_S ===========================
   ============================================================================ */

typedef union CE_SRG_S_CR0_union_tag { /* Control 0 */
  vuint32_t R;
  struct {
    vuint32_t MSTR0DOW:1;              /* MSTR0 Disable Optimized Write */
    vuint32_t C0CCMDOW:1;              /* Core 0 CCM Disable Optimized Write */
    vuint32_t C0CSMDOW:1;              /* Core 0 CSM Disable Optimized Write */
    vuint32_t DEDOW:1;                 /* DMA ETR Disable Optimized Write */
    vuint32_t C1CCMDOW:1;              /* Core 1 CCM Disable Optimized Write */
    vuint32_t C1CSMDOW:1;              /* Core 1 CSM Disable Optimized Write */
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t MSTR0HPE:1;              /* MSTR High Priority Elevation */
    vuint32_t C0CCMHPE:1;              /* Core 0 CCM High Priority Elevation */
    vuint32_t C0CSMHPE:1;              /* Core 0 CSM High Priority Elevation */
    vuint32_t _unused_11:1;
    vuint32_t C1CCMHPE:1;              /* Core 1 CCM CE High Priority Elevation */
    vuint32_t C1CSMHPE:1;              /* Core 1 CSM CE High Priority Elevation */
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t SLV0SBE:1;               /* SLV0 Suppress Bus Error */
    vuint32_t AIPS0SBE:1;              /* CE_AIPS_0 Suppress Bus Error */
    vuint32_t AIPS1SBE:1;              /* CE_AIPS_1 Suppress Bus Error */
    vuint32_t AIPS2SBE:1;              /* CE_AIPS_2 Suppress Bus Error */
    vuint32_t AIPS3SBE:1;              /* CE_AIPS_3 Suppress Bus Error */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} CE_SRG_S_CR0_tag;

typedef union CE_SRG_S_LCR0_union_tag { /* Lock Control 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t LNSMPU:1;                /* Lock Non-Secure MPU */
    vuint32_t _unused_17:1;
    vuint32_t LNSVTOR:1;               /* Lock Non-Secure Vector Table */
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} CE_SRG_S_LCR0_tag;

typedef union CE_SRG_S_LCR1_union_tag { /* Lock Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t LNSMPU:1;                /* Lock Non-Secure MPU */
    vuint32_t _unused_17:1;
    vuint32_t LNSVTOR:1;               /* Lock Non-Secure Vector Table */
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} CE_SRG_S_LCR1_tag;

typedef union CE_SRG_S_LCR2_union_tag { /* Lock Control 2 */
  vuint32_t R;
  struct {
    vuint32_t REMAP_EN:1;              /* Remap Enable */
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t REMAP0:1;                /* Remap RAM0 */
    vuint32_t REMAP1:1;                /* Remap RAM1 */
    vuint32_t REMAP2:1;                /* Remap RAM2 */
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} CE_SRG_S_LCR2_tag;

typedef union CE_SRG_S_LCR3_union_tag { /* Lock Control 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t R0RO:5;                  /* RAM0 Remap Offset */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t R1RO:5;                  /* RAM1 Remap Offset */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} CE_SRG_S_LCR3_tag;

typedef union CE_SRG_S_LCR4_union_tag { /* Lock Control 4 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t R2RO:5;                  /* RAM2 Remap Offset */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} CE_SRG_S_LCR4_tag;

typedef union CE_SRG_S_LCR7_union_tag { /* Lock Control 7 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t C0DBGSA:1;               /* Core 0 Debug Secure Access */
    vuint32_t C1DBGSA:1;               /* Core 1 Debug Secure Access */
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} CE_SRG_S_LCR7_tag;

typedef union CE_SRG_S_SR0_union_tag { /* Status 0 */
  vuint32_t R;
  struct {
    vuint32_t R0EA:1;                  /* RAM0 Error Alarm */
    vuint32_t R1EA:1;                  /* RAM1 Error Alarm */
    vuint32_t R2EA:1;                  /* RAM2 Error Alarm */
    vuint32_t _unused_3:1;
    vuint32_t R0WEA:1;                 /* RAM0 Write Error Alarm */
    vuint32_t R1WEA:1;                 /* RAM1 Write Error Alarm */
    vuint32_t R2WEA:1;                 /* RAM2 Write Error Alarm */
    vuint32_t _unused_7:1;
    vuint32_t C0CTRE:1;                /* Core 0 Code Bus TAG Array Recoverable Error */
    vuint32_t C0CDRE:1;                /* Core 0 Code Bus DATA Array Recoverable Error */
    vuint32_t C0STRE:1;                /* Core 0 System Bus TAG Array Recoverable Error */
    vuint32_t C0SDRE:1;                /* Core 0 System Bus DATA Array Recoverable Error */
    vuint32_t C1CTRE:1;                /* Core 1 Code Bus TAG Array Recoverable Error */
    vuint32_t C1CDRE:1;                /* Core 1 Code Bus DATA Array Recoverable Error */
    vuint32_t C1STRE:1;                /* Core 1 System Bus TAG Array Recoverable Error */
    vuint32_t C1SDRE:1;                /* Core 1 System Bus DATA Array Recoverable Error */
    vuint32_t C0CCCMCRE:1;             /* Core 0 Code Bus LMEM Cache Array Cache Maintenance Command Recoverable Error */
    vuint32_t C0SCCMCRE:1;             /* Core 0 System Bus LMEM Cache Array Cache Maintenance Command Recoverable Error */
    vuint32_t C1CCCMCRE:1;             /* Core 1 Code Bus LMEM Cache Array Cache Maintenance Command Recoverable Error */
    vuint32_t C1SCCMCRE:1;             /* Core 1 System Bus LMEM Cache Array Cache Maintenance Command Recoverable Error */
    vuint32_t CORE0H:1;                /* Core 0 Halted */
    vuint32_t CORE0TU:1;               /* Core 0 Trace Units Enabled */
    vuint32_t CORE1H:1;                /* Core 1 Halted */
    vuint32_t CORE1TU:1;               /* Core 1 Trace Units Enabled */
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} CE_SRG_S_SR0_tag;

struct CE_SRG_S_tag {
  CE_SRG_S_CR0_tag CR0;                /* Control 0 */
  uint8_t CE_SRG_S_reserved0[252];
  CE_SRG_S_LCR0_tag LCR0;              /* Lock Control 0 */
  CE_SRG_S_LCR1_tag LCR1;              /* Lock Control 1 */
  CE_SRG_S_LCR2_tag LCR2;              /* Lock Control 2 */
  CE_SRG_S_LCR3_tag LCR3;              /* Lock Control 3 */
  CE_SRG_S_LCR4_tag LCR4;              /* Lock Control 4 */
  uint8_t CE_SRG_S_reserved1[8];
  CE_SRG_S_LCR7_tag LCR7;              /* Lock Control 7 */
  uint8_t CE_SRG_S_reserved2[224];
  CE_SRG_S_SR0_tag SR0;                /* Status 0 */
};


/* ============================================================================
   =============================== Module: CE_XRDC ============================
   ============================================================================ */

typedef union CE_XRDC_CR_union_tag {   /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} CE_XRDC_CR_tag;

typedef union CE_XRDC_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} CE_XRDC_HWCFG0_tag;

typedef union CE_XRDC_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} CE_XRDC_HWCFG1_tag;

typedef union CE_XRDC_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} CE_XRDC_MDACFG_tag;

typedef union CE_XRDC_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} CE_XRDC_MRCFG_tag;

typedef union CE_XRDC_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} CE_XRDC_DERRLOC_tag;

typedef union CE_XRDC_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} CE_XRDC_DERRW0_DERR_W0_tag;

typedef union CE_XRDC_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} CE_XRDC_DERRW0_DERR_W1_tag;

typedef union CE_XRDC_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} CE_XRDC_DERRW0_DERR_W3_tag;

typedef struct CE_XRDC_DERRW0_struct_tag {
  CE_XRDC_DERRW0_DERR_W0_tag DERR_W0;  /* Domain Error Word 0 */
  CE_XRDC_DERRW0_DERR_W1_tag DERR_W1;  /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  CE_XRDC_DERRW0_DERR_W3_tag DERR_W3;  /* Domain Error Word 3 */
} CE_XRDC_DERRW0_tag;

typedef union CE_XRDC_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W0_0_DFMT1_tag;

typedef union CE_XRDC_MDA_W1_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W1_0_DFMT1_tag;

typedef union CE_XRDC_MDA_W2_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W2_0_DFMT1_tag;

typedef union CE_XRDC_MDA_W3_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W3_0_DFMT1_tag;

typedef union CE_XRDC_MDA_W4_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W4_0_DFMT1_tag;

typedef union CE_XRDC_MDA_W5_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W5_0_DFMT1_tag;

typedef union CE_XRDC_MDA_W6_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W6_0_DFMT1_tag;

typedef union CE_XRDC_MDA_W7_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W7_0_DFMT1_tag;

typedef union CE_XRDC_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W0_1_DFMT1_tag;

typedef union CE_XRDC_MDA_W0_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W0_2_DFMT1_tag;

typedef union CE_XRDC_MDA_W1_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W1_2_DFMT1_tag;

typedef union CE_XRDC_MDA_W2_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W2_2_DFMT1_tag;

typedef union CE_XRDC_MDA_W3_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W3_2_DFMT1_tag;

typedef union CE_XRDC_MDA_W4_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W4_2_DFMT1_tag;

typedef union CE_XRDC_MDA_W5_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W5_2_DFMT1_tag;

typedef union CE_XRDC_MDA_W6_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W6_2_DFMT1_tag;

typedef union CE_XRDC_MDA_W7_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MDA_W7_2_DFMT1_tag;

typedef union CE_XRDC_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} CE_XRDC_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union CE_XRDC_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct CE_XRDC_PDAC_SLOT_PDACN_struct_tag {
  CE_XRDC_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  CE_XRDC_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} CE_XRDC_PDAC_SLOT_PDACN_tag;

typedef struct CE_XRDC_PDAC_SLOT_struct_tag {
  CE_XRDC_PDAC_SLOT_PDACN_tag PDACN[127]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[8];
} CE_XRDC_PDAC_SLOT_tag;

typedef union CE_XRDC_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} CE_XRDC_MRGDN_XRDC_MRGD_W0_tag;

typedef union CE_XRDC_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} CE_XRDC_MRGDN_XRDC_MRGD_W1_tag;

typedef union CE_XRDC_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} CE_XRDC_MRGDN_XRDC_MRGD_W2_tag;

typedef union CE_XRDC_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} CE_XRDC_MRGDN_XRDC_MRGD_W3_tag;

typedef struct CE_XRDC_MRGDN_struct_tag {
  CE_XRDC_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  CE_XRDC_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  CE_XRDC_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  CE_XRDC_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} CE_XRDC_MRGDN_tag;

struct CE_XRDC_tag {
  CE_XRDC_CR_tag CR;                   /* Control */
  uint8_t CE_XRDC_reserved0[236];
  CE_XRDC_HWCFG0_tag HWCFG0;           /* Hardware Configuration 0 */
  CE_XRDC_HWCFG1_tag HWCFG1;           /* Hardware Configuration 1 */
  uint8_t CE_XRDC_reserved1[8];
  CE_XRDC_MDACFG_tag MDACFG[3];        /* Master Domain Assignment Configuration */
  uint8_t CE_XRDC_reserved2[61];
  CE_XRDC_MRCFG_tag MRCFG[3];          /* Memory Region Configuration */
  uint8_t CE_XRDC_reserved3[189];
  CE_XRDC_DERRLOC_tag DERRLOC[16];     /* Domain Error Location */
  uint8_t CE_XRDC_reserved4[448];
  CE_XRDC_DERRW0_tag DERRW0[20];       /* Valid array indices: 0-2, 16-19 */
  uint8_t CE_XRDC_reserved5[704];
  CE_XRDC_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W1_0_DFMT1_tag MDA_W1_0_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W2_0_DFMT1_tag MDA_W2_0_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W3_0_DFMT1_tag MDA_W3_0_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W4_0_DFMT1_tag MDA_W4_0_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W5_0_DFMT1_tag MDA_W5_0_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W6_0_DFMT1_tag MDA_W6_0_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W7_0_DFMT1_tag MDA_W7_0_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  uint8_t CE_XRDC_reserved6[28];
  CE_XRDC_MDA_W0_2_DFMT1_tag MDA_W0_2_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W1_2_DFMT1_tag MDA_W1_2_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W2_2_DFMT1_tag MDA_W2_2_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W3_2_DFMT1_tag MDA_W3_2_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W4_2_DFMT1_tag MDA_W4_2_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W5_2_DFMT1_tag MDA_W5_2_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W6_2_DFMT1_tag MDA_W6_2_DFMT1; /* Master Domain Assignment */
  CE_XRDC_MDA_W7_2_DFMT1_tag MDA_W7_2_DFMT1; /* Master Domain Assignment */
  uint8_t CE_XRDC_reserved7[1952];
  CE_XRDC_PDAC_SLOT_tag PDAC_SLOT[4];
  CE_XRDC_MRGDN_tag MRGDN[3][16];
};


/* ============================================================================
   =============================== Module: CLASS_0X9_CORESIGHT_COMPONENT ======
   ============================================================================ */

typedef union CLASS_0X9_CORESIGHT_COMPONENT_DTIR_union_tag { /* DTIR Register */
  vuint32_t R;
  struct {
    vuint32_t MAJ_T:4;
    vuint32_t SUB_T:4;
    vuint32_t _unused_8:24;
  } B;
} CLASS_0X9_CORESIGHT_COMPONENT_DTIR_tag;

struct CLASS_0X9_CORESIGHT_COMPONENT_tag {
  CLASS_0X9_CORESIGHT_COMPONENT_DTIR_tag DTIR; /* DTIR Register */
};


/* ============================================================================
   =============================== Module: CMU_FC =============================
   ============================================================================ */

typedef union CMU_FC_GCR_union_tag {   /* Global Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t FCE:1;                   /* Frequency Check Enable */
    vuint32_t _unused_1:7;
    vuint32_t FI_SCE:1;                /* Fault Injection Software Configuration Enable */
    vuint32_t FI_START:1;              /* Fault Injection Start */
    vuint32_t FI_AUTO_EN:1;            /* Automatic Fault Injection mode enabled */
    vuint32_t FI_INT_MASK:1;           /* Standalone fault injection mode interrupts mask */
    vuint32_t _unused_12:4;
    vuint32_t FI_AUTO_RCOUNT:16;       /* Automatic Fault Injection Repeat Count */
  } B;
} CMU_FC_GCR_tag;

typedef union CMU_FC_RCCR_union_tag {  /* Reference Count Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t REF_CNT:16;              /* Reference clock count */
    vuint32_t _unused_16:16;
  } B;
} CMU_FC_RCCR_tag;

typedef union CMU_FC_HTCR_union_tag {  /* High Threshold Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t HFREF:24;                /* High frequency reference threshold */
    vuint32_t _unused_24:8;
  } B;
} CMU_FC_HTCR_tag;

typedef union CMU_FC_LTCR_union_tag {  /* Low Threshold Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t LFREF:24;                /* Low Frequency Reference Threshold */
    vuint32_t _unused_24:8;
  } B;
} CMU_FC_LTCR_tag;

typedef union CMU_FC_SR_union_tag {    /* Status Register */
  vuint32_t R;
  struct {
    vuint32_t FLL:1;                   /* Frequency lower than low frequency reference threshold event status */
    vuint32_t FHH:1;                   /* Frequency higher than high frequency reference threshold event status */
    vuint32_t _unused_2:2;
    vuint32_t RS:1;                    /* Run Status */
    vuint32_t _unused_5:3;
    vuint32_t _unused_8:3;
    vuint32_t _unused_11:5;
    vuint32_t AFI_FAIL:1;              /* Indicates status of fault injection in automatic FI */
    vuint32_t _unused_17:3;
    vuint32_t FI_PASS:1;               /* Indicates standalone fault injection status of latent faults in FHH and FLL generation logic */
    vuint32_t _unused_21:2;
    vuint32_t FI_COMP:1;               /* Indicates the completion of standalone fault injection sequence */
    vuint32_t _unused_24:8;
  } B;
} CMU_FC_SR_tag;

typedef union CMU_FC_IER_union_tag {   /* Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t FLLIE:1;                 /* Frequency Lower than Low Frequency Reference Threshold Synchronous Interrupt Enable */
    vuint32_t FHHIE:1;                 /* Frequency Higher than High Frequency Reference Threshold Synchronous Interrupt Enable */
    vuint32_t FLLAIE:1;                /* Frequency Lower than Low Frequency Reference Threshold Asynchronous Interrupt Enable */
    vuint32_t FHHAIE:1;                /* Frequency Higher than High Frequency Reference Threshold Asynchronous Interrupt Enable */
    vuint32_t _unused_4:12;
    vuint32_t AFIFIE:1;                /* Auto FI sync fail interrupt enable */
    vuint32_t AFIAFIE:1;               /* Auto FI async fail interrupt enable */
    vuint32_t _unused_18:2;
    vuint32_t FIPIE:1;                 /* FI sync pass interrupt enable */
    vuint32_t FIAPIE:1;                /* FI async pass interrupt enable */
    vuint32_t _unused_22:1;
    vuint32_t FICIE:1;                 /* FI complete interrupt enable */
    vuint32_t _unused_24:8;
  } B;
} CMU_FC_IER_tag;

struct CMU_FC_tag {
  CMU_FC_GCR_tag GCR;                  /* Global Configuration Register */
  CMU_FC_RCCR_tag RCCR;                /* Reference Count Configuration Register */
  CMU_FC_HTCR_tag HTCR;                /* High Threshold Configuration Register */
  CMU_FC_LTCR_tag LTCR;                /* Low Threshold Configuration Register */
  CMU_FC_SR_tag SR;                    /* Status Register */
  CMU_FC_IER_tag IER;                  /* Interrupt Enable Register */
};


/* ============================================================================
   =============================== Module: CORESIGHT_OCEM =====================
   ============================================================================ */

typedef union CORESIGHT_OCEM_SCCO_union_tag { /* SCCO Register */
  vuint32_t R;
  struct {
    vuint32_t SCC:16;
    vuint32_t DIREC:1;
    vuint32_t _unused_17:7;
    vuint32_t SIZE:8;
  } B;
} CORESIGHT_OCEM_SCCO_tag;

typedef union CORESIGHT_OCEM_SCDA_union_tag { /* SCDA Register */
  vuint32_t R;
  struct {
    vuint32_t SCD:32;
  } B;
} CORESIGHT_OCEM_SCDA_tag;

struct CORESIGHT_OCEM_tag {
  CORESIGHT_OCEM_SCCO_tag SCCO;        /* SCCO Register */
  CORESIGHT_OCEM_SCDA_tag SCDA;        /* SCDA Register */
};


/* ============================================================================
   =============================== Module: CORESIGHT_PROGRAMMING_MODEL ========
   ============================================================================ */

typedef union CORESIGHT_PROGRAMMING_MODEL_P_ID4_union_tag { /* PMSS ID4 Register */
  vuint32_t R;
  struct {
    vuint32_t JEP106_CC:4;
    vuint32_t FOURKB_C:4;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_P_ID4_tag;

typedef union CORESIGHT_PROGRAMMING_MODEL_P_ID0_union_tag { /* PMSS ID0 Register */
  vuint32_t R;
  struct {
    vuint32_t PN_L:8;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_P_ID0_tag;

typedef union CORESIGHT_PROGRAMMING_MODEL_P_ID1_union_tag { /* PMSS ID1 Register */
  vuint32_t R;
  struct {
    vuint32_t PN_M:4;
    vuint32_t JEP106_ICL:4;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_P_ID1_tag;

typedef union CORESIGHT_PROGRAMMING_MODEL_P_ID2_union_tag { /* PMSS ID2 Register */
  vuint32_t R;
  struct {
    vuint32_t JEP106_ICH:2;
    vuint32_t _unused_2:1;
    vuint32_t JEDEC_A:1;
    vuint32_t REV:4;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_P_ID2_tag;

typedef union CORESIGHT_PROGRAMMING_MODEL_P_ID3_union_tag { /* PMSS ID3 Register */
  vuint32_t R;
  struct {
    vuint32_t CU_MO:4;
    vuint32_t RE_AN:4;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_P_ID3_tag;

typedef union CORESIGHT_PROGRAMMING_MODEL_C_ID0_union_tag { /* Core ID0 Register */
  vuint32_t R;
  struct {
    vuint32_t ID0_VAL:8;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_C_ID0_tag;

typedef union CORESIGHT_PROGRAMMING_MODEL_C_ID1_union_tag { /* Core ID1 Register */
  vuint32_t R;
  struct {
    vuint32_t ID1_VAL:4;
    vuint32_t CCLA:4;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_C_ID1_tag;

typedef union CORESIGHT_PROGRAMMING_MODEL_C_ID2_union_tag { /* Core ID2 Register */
  vuint32_t R;
  struct {
    vuint32_t ID2_VAL:8;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_C_ID2_tag;

typedef union CORESIGHT_PROGRAMMING_MODEL_C_ID3_union_tag { /* Core ID3 Register */
  vuint32_t R;
  struct {
    vuint32_t ID3_VAL:8;
    vuint32_t _unused_8:24;
  } B;
} CORESIGHT_PROGRAMMING_MODEL_C_ID3_tag;

struct CORESIGHT_PROGRAMMING_MODEL_tag {
  CORESIGHT_PROGRAMMING_MODEL_P_ID4_tag P_ID4; /* PMSS ID4 Register */
  uint8_t CORESIGHT_PROGRAMMING_MODEL_reserved0[12];
  CORESIGHT_PROGRAMMING_MODEL_P_ID0_tag P_ID0; /* PMSS ID0 Register */
  CORESIGHT_PROGRAMMING_MODEL_P_ID1_tag P_ID1; /* PMSS ID1 Register */
  CORESIGHT_PROGRAMMING_MODEL_P_ID2_tag P_ID2; /* PMSS ID2 Register */
  CORESIGHT_PROGRAMMING_MODEL_P_ID3_tag P_ID3; /* PMSS ID3 Register */
  CORESIGHT_PROGRAMMING_MODEL_C_ID0_tag C_ID0; /* Core ID0 Register */
  CORESIGHT_PROGRAMMING_MODEL_C_ID1_tag C_ID1; /* Core ID1 Register */
  CORESIGHT_PROGRAMMING_MODEL_C_ID2_tag C_ID2; /* Core ID2 Register */
  CORESIGHT_PROGRAMMING_MODEL_C_ID3_tag C_ID3; /* Core ID3 Register */
};


/* ============================================================================
   =============================== Module: CORE_DEBUGGER_INTERFACE ============
   ============================================================================ */

typedef union CORE_DEBUGGER_INTERFACE_CORE_DRD3_RD_union_tag { /* Core DRD3 RD Register */
  vuint32_t R;
  struct {
    vuint32_t CORE_DRD3_RD:32;
  } B;
} CORE_DEBUGGER_INTERFACE_CORE_DRD3_RD_tag;

typedef union CORE_DEBUGGER_INTERFACE_CORE_DRD2_RD_union_tag { /* Core DRD2 RD Register */
  vuint32_t R;
  struct {
    vuint32_t CORE_DRD2_RD:32;
  } B;
} CORE_DEBUGGER_INTERFACE_CORE_DRD2_RD_tag;

typedef union CORE_DEBUGGER_INTERFACE_CORE_DRD1_RD_union_tag { /* Core DRD1 RD Register */
  vuint32_t R;
  struct {
    vuint32_t CORE_DRD1_RD:32;
  } B;
} CORE_DEBUGGER_INTERFACE_CORE_DRD1_RD_tag;

typedef union CORE_DEBUGGER_INTERFACE_CORE_DRD_RD_union_tag { /* Core DRD RD Register */
  vuint32_t R;
  struct {
    vuint32_t CORE_DRD_RD:32;
  } B;
} CORE_DEBUGGER_INTERFACE_CORE_DRD_RD_tag;

struct CORE_DEBUGGER_INTERFACE_tag {
  CORE_DEBUGGER_INTERFACE_CORE_DRD3_RD_tag CORE_DRD3_RD; /* Core DRD3 RD Register */
  CORE_DEBUGGER_INTERFACE_CORE_DRD2_RD_tag CORE_DRD2_RD; /* Core DRD2 RD Register */
  CORE_DEBUGGER_INTERFACE_CORE_DRD1_RD_tag CORE_DRD1_RD; /* Core DRD1 RD Register */
  CORE_DEBUGGER_INTERFACE_CORE_DRD_RD_tag CORE_DRD_RD; /* Core DRD RD Register */
};


/* ============================================================================
   =============================== Module: CORE_PLL ===========================
   ============================================================================ */

typedef union CORE_PLL_PLLCR_union_tag { /* PLL Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;            /* Reserved */
    vuint32_t PLLPD:1;                 /* PLL Power Down */
  } B;
} CORE_PLL_PLLCR_tag;

typedef union CORE_PLL_PLLSR_union_tag { /* PLL Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t LOCK:1;                  /* Lock Status */
    vuint32_t LOL:1;                   /* Loss-Of-Lock Flag */
    vuint32_t _unused_4:4;
    vuint32_t _unused_8:3;
    vuint32_t _unused_11:21;
  } B;
} CORE_PLL_PLLSR_tag;

typedef union CORE_PLL_PLLDV_union_tag { /* PLL Divider */
  vuint32_t R;
  struct {
    vuint32_t MFI:8;                   /* Integer Portion Of Loop Divider */
    vuint32_t _unused_8:4;
    vuint32_t RDIV:3;                  /* Input Clock Predivider */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:6;
    vuint32_t _unused_22:3;
    vuint32_t _unused_25:6;
    vuint32_t _unused_31:1;
  } B;
} CORE_PLL_PLLDV_tag;

typedef union CORE_PLL_PLLFM_union_tag { /* PLL Frequency Modulation */
  vuint32_t R;
  struct {
    vuint32_t STEPNO:11;               /* Number Of Steps Of Modulation Period Or Frequency Modulation */
    vuint32_t _unused_11:5;
    vuint32_t STEPSIZE:10;             /* Frequency Modulation Step Size */
    vuint32_t _unused_26:3;
    vuint32_t SPREADCTL:1;             /* Modulation Type Selection */
    vuint32_t SSCGBYP:1;               /* Frequency Modulation (Spread Spectrum Clock Generation) Bypass */
    vuint32_t _unused_31:1;
  } B;
} CORE_PLL_PLLFM_tag;

typedef union CORE_PLL_PLLFD_union_tag { /* PLL Fractional Divider */
  vuint32_t R;
  struct {
    vuint32_t MFN:15;                  /* Numerator Of Fractional Loop Division Factor */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:4;
    vuint32_t _unused_22:6;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t SDMEN:1;                 /* Fractional Mode Enable */
    vuint32_t _unused_31:1;
  } B;
} CORE_PLL_PLLFD_tag;

typedef union CORE_PLL_PLLCLKMUX_union_tag { /* PLL Clock Multiplexer */
  vuint32_t R;
  struct {
    vuint32_t REFCLKSEL:1;             /* Reference Clock Select */
    vuint32_t _unused_1:31;
  } B;
} CORE_PLL_PLLCLKMUX_tag;

typedef union CORE_PLL_PLLODIV_union_tag { /* PLL Output Divider */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t _unused_2:14;
    vuint32_t DIV:8;                   /* Division Value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider Enable */
  } B;
} CORE_PLL_PLLODIV_tag;

struct CORE_PLL_tag {
  CORE_PLL_PLLCR_tag PLLCR;            /* PLL Control */
  CORE_PLL_PLLSR_tag PLLSR;            /* PLL Status */
  CORE_PLL_PLLDV_tag PLLDV;            /* PLL Divider */
  CORE_PLL_PLLFM_tag PLLFM;            /* PLL Frequency Modulation */
  CORE_PLL_PLLFD_tag PLLFD;            /* PLL Fractional Divider */
  uint8_t CORE_PLL_reserved0[12];
  CORE_PLL_PLLCLKMUX_tag PLLCLKMUX;    /* PLL Clock Multiplexer */
  uint8_t CORE_PLL_reserved1[92];
  CORE_PLL_PLLODIV_tag PLLODIV[1];     /* PLL Output Divider */
};


/* ============================================================================
   =============================== Module: CORE_SAFETY ========================
   ============================================================================ */

typedef union CORE_SAFETY_C_UCPERR_union_tag { /* Core UCP ERR Register */
  vuint32_t R;
  struct {
    vuint32_t C_UCPERR:1;
    vuint32_t P_ECC_ERR:1;
    vuint32_t _unused_2:1;
    vuint32_t P_EAC_ERR:1;
    vuint32_t _unused_4:4;
    vuint32_t C_ILPE:1;
    vuint32_t _unused_9:23;
  } B;
} CORE_SAFETY_C_UCPERR_tag;

typedef union CORE_SAFETY_C_UCPERR_S_union_tag { /* Core UCP ERR Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t P_ECC_ERR_S:1;
    vuint32_t _unused_2:30;
  } B;
} CORE_SAFETY_C_UCPERR_S_tag;

struct CORE_SAFETY_tag {
  CORE_SAFETY_C_UCPERR_tag C_UCPERR;   /* Core UCP ERR Register */
  CORE_SAFETY_C_UCPERR_S_tag C_UCPERR_S; /* Core UCP ERR Shadow Register */
};


/* ============================================================================
   =============================== Module: CRC ================================
   ============================================================================ */

typedef union CRC_CHANNEL_CFG_union_tag { /* Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t INV:1;                   /* Inversion selection */
    vuint32_t SWAP:1;                  /* Swap selection */
    vuint32_t POLYG:2;                 /* Polynomial selection */
    vuint32_t SWAP_BITWISE:1;          /* Swap CRC_INP bit-wise */
    vuint32_t SWAP_BYTEWISE:1;         /* Swap CRC_INP byte-wise */
    vuint32_t _unused_6:18;
    vuint32_t _unused_24:8;
  } B;
} CRC_CHANNEL_CFG_tag;

typedef union CRC_CHANNEL_INP_union_tag { /* Input Register */
  vuint32_t R;
  struct {
    vuint32_t INP:32;                  /* Input data for the CRC computation */
  } B;
} CRC_CHANNEL_INP_tag;

typedef union CRC_CHANNEL_CSTAT_union_tag { /* Current Status Register */
  vuint32_t R;
  struct {
    vuint32_t CSTAT:32;                /* CRC signature status */
  } B;
} CRC_CHANNEL_CSTAT_tag;

typedef union CRC_CHANNEL_OUTP_union_tag { /* Output Register */
  vuint32_t R;
  struct {
    vuint32_t OUTP:32;                 /* Final CRC signature */
  } B;
} CRC_CHANNEL_OUTP_tag;

typedef struct CRC_CHANNEL_struct_tag {
  CRC_CHANNEL_CFG_tag CFG;             /* Configuration Register */
  CRC_CHANNEL_INP_tag INP;             /* Input Register */
  CRC_CHANNEL_CSTAT_tag CSTAT;         /* Current Status Register */
  CRC_CHANNEL_OUTP_tag OUTP;           /* Output Register */
} CRC_CHANNEL_tag;

struct CRC_tag {
  CRC_CHANNEL_tag CHANNEL[3];
};


/* ============================================================================
   =============================== Module: CSTCU ==============================
   ============================================================================ */

typedef union CSTCU_RUNSWREG_union_tag { /* Run Software */
  vuint32_t R;
  struct {
    vuint32_t RUNSW:1;                 /* Run Software */
    vuint32_t _unused_1:3;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:27;
  } B;
} CSTCU_RUNSWREG_tag;

typedef union CSTCU_LSCHLVLD_union_tag { /* LSTCU Scheduler List Valid */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:7;
    vuint32_t MBPLVLD:1;               /* LSTCU MBIST Pointer List Valid */
    vuint32_t _unused_9:7;
    vuint32_t LBPLVLD:1;               /* LSTCU LBIST Pointer List Valid */
    vuint32_t _unused_17:7;
    vuint32_t _unused_24:8;
  } B;
} CSTCU_LSCHLVLD_tag;

typedef union CSTCU_WDG_union_tag {    /* Watchdog Timer */
  vuint32_t R;
  struct {
    vuint32_t WDGEOC:32;               /* Watchdog End of Counter */
  } B;
} CSTCU_WDG_tag;

typedef union CSTCU_IE_union_tag {     /* Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t STEND_IE:1;              /* Self-Test End Interrupt Enable */
    vuint32_t _unused_1:31;
  } B;
} CSTCU_IE_tag;

typedef union CSTCU_IF_union_tag {     /* Interrupt Flag */
  vuint32_t R;
  struct {
    vuint32_t STEND_IF:1;              /* Self-Test End Interrupt Flag */
    vuint32_t _unused_1:31;
  } B;
} CSTCU_IF_tag;

typedef union CSTCU_ERR_STAT_union_tag { /* Error Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVP_MB:1;               /* Invalid LSTCU Pointer During MBIST Scheduling */
    vuint32_t INVP_LB:1;               /* Invalid LSTCU Pointer During LBIST Scheduling */
    vuint32_t _unused_3:5;
    vuint32_t ABORTHW:1;               /* Hardware Abort Flag */
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:6;
    vuint32_t UFSF:1;                  /* Unrecoverable Fault Status Flag */
    vuint32_t RFSF:1;                  /* Recoverable Fault Status Flag */
    vuint32_t _unused_18:14;
  } B;
} CSTCU_ERR_STAT_tag;

typedef union CSTCU_ERR_FM_union_tag { /* Error Fault Mapping */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVPFMMB:1;              /* MBIST Fault Mapping */
    vuint32_t INVPFMLB:1;              /* FBIST Fault Mapping */
    vuint32_t _unused_3:29;
  } B;
} CSTCU_ERR_FM_tag;

typedef union CSTCU_LRFSTAT_union_tag { /* LSTCU Recoverable Fault Status */
  vuint32_t R;
  struct {
    vuint32_t RFSLSTCU0:1;             /* Recoverable Fault Occurrence Status from LSTCU0 */
    vuint32_t RFSLSTCU1:1;             /* Recoverable Fault Occurrence Status from LSTCU1 */
    vuint32_t RFSLSTCU2:1;             /* Recoverable Fault Occurrence Status from LSTCU2 */
    vuint32_t RFSLSTCU3:1;             /* Recoverable Fault Occurrence Status from LSTCU3 */
    vuint32_t RFSLSTCU4:1;             /* Recoverable Fault Occurrence Status from LSTCU4 */
    vuint32_t RFSLSTCU5:1;             /* Recoverable Fault Occurrence Status from LSTCU5 */
    vuint32_t RFSLSTCU6:1;             /* Recoverable Fault Occurrence Status from LSTCU6 */
    vuint32_t RFSLSTCU7:1;             /* Recoverable Fault Occurrence Status from LSTCU7 */
    vuint32_t RFSLSTCU8:1;             /* Recoverable Fault Occurrence Status from LSTCU8 */
    vuint32_t RFSLSTCU9:1;             /* Recoverable Fault Occurrence Status from LSTCU9 */
    vuint32_t RFSLSTCU10:1;            /* Recoverable Fault Occurrence Status from LSTCU10 */
    vuint32_t RFSLSTCU11:1;            /* Recoverable Fault Occurrence Status from LSTCU11 */
    vuint32_t RFSLSTCU12:1;            /* Recoverable Fault Occurrence Status from LSTCU12 */
    vuint32_t RFSLSTCU13:1;            /* Recoverable Fault Occurrence Status from LSTCU13 */
    vuint32_t RFSLSTCU14:1;            /* Recoverable Fault Occurrence Status from LSTCU14 */
    vuint32_t RFSLSTCU15:1;            /* Recoverable Fault Occurrence Status from LSTCU15 */
    vuint32_t RFSLSTCU16:1;            /* Recoverable Fault Occurrence Status from LSTCU16 */
    vuint32_t RFSLSTCU17:1;            /* Recoverable Fault Occurrence Status from LSTCU17 */
    vuint32_t RFSLSTCU18:1;            /* Recoverable Fault Occurrence Status from LSTCU18 */
    vuint32_t RFSLSTCU19:1;            /* Recoverable Fault Occurrence Status from LSTCU19 */
    vuint32_t _unused_20:12;
  } B;
} CSTCU_LRFSTAT_tag;

typedef union CSTCU_LUFSTAT_union_tag { /* LSTCU Unrecoverable Fault Status */
  vuint32_t R;
  struct {
    vuint32_t UFSLSTCU0:1;             /* Unrecoverable Fault Occurrence Status from LSTCU0 */
    vuint32_t UFSLSTCU1:1;             /* Unrecoverable Fault Occurrence Status from LSTCU1 */
    vuint32_t UFSLSTCU2:1;             /* Unrecoverable Fault Occurrence Status from LSTCU2 */
    vuint32_t UFSLSTCU3:1;             /* Unrecoverable Fault Occurrence Status from LSTCU3 */
    vuint32_t UFSLSTCU4:1;             /* Unrecoverable Fault Occurrence Status from LSTCU4 */
    vuint32_t UFSLSTCU5:1;             /* Unrecoverable Fault Occurrence Status from LSTCU5 */
    vuint32_t UFSLSTCU6:1;             /* Unrecoverable Fault Occurrence Status from LSTCU6 */
    vuint32_t UFSLSTCU7:1;             /* Unrecoverable Fault Occurrence Status from LSTCU7 */
    vuint32_t UFSLSTCU8:1;             /* Unrecoverable Fault Occurrence Status from LSTCU8 */
    vuint32_t UFSLSTCU9:1;             /* Unrecoverable Fault Occurrence Status from LSTCU9 */
    vuint32_t UFSLSTCU10:1;            /* Unrecoverable Fault Occurrence Status from LSTCU10 */
    vuint32_t UFSLSTCU11:1;            /* Unrecoverable Fault Occurrence Status from LSTCU11 */
    vuint32_t UFSLSTCU12:1;            /* Unrecoverable Fault Occurrence Status from LSTCU12 */
    vuint32_t UFSLSTCU13:1;            /* Unrecoverable Fault Occurrence Status from LSTCU13 */
    vuint32_t UFSLSTCU14:1;            /* Unrecoverable Fault Occurrence Status from LSTCU14 */
    vuint32_t UFSLSTCU15:1;            /* Unrecoverable Fault Occurrence Status from LSTCU15 */
    vuint32_t UFSLSTCU16:1;            /* Unrecoverable Fault Occurrence Status from LSTCU16 */
    vuint32_t UFSLSTCU17:1;            /* Unrecoverable Fault Occurrence Status from LSTCU17 */
    vuint32_t UFSLSTCU18:1;            /* Unrecoverable Fault Occurrence Status from LSTCU18 */
    vuint32_t UFSLSTCU19:1;            /* Unrecoverable Fault Occurrence Status from LSTCU19 */
    vuint32_t _unused_20:12;
  } B;
} CSTCU_LUFSTAT_tag;

typedef union CSTCU_RDEN_union_tag {   /* Reset Domain Self-Test Enable */
  vuint32_t R;
  struct {
    vuint32_t SERD0:1;                 /* Enable Self-Test in Reset Domain 0 */
    vuint32_t SERD1:1;                 /* Enable Self-Test in Reset Domain 1 */
    vuint32_t SERD2:1;                 /* Enable Self-Test in Reset Domain 2 */
    vuint32_t SERD3:1;                 /* Enable Self-Test in Reset Domain 3 */
    vuint32_t _unused_4:28;
  } B;
} CSTCU_RDEN_tag;

typedef union CSTCU_RDENSTAT_union_tag { /* Reset Domain Enable Status */
  vuint32_t R;
  struct {
    vuint32_t SESRD0:1;                /* Self-Test Enable Status in Reset Domain 0 */
    vuint32_t SESRD1:1;                /* Self-Test Enable Status in Reset Domain 1 */
    vuint32_t SESRD2:1;                /* Self-Test Enable Status in Reset Domain 2 */
    vuint32_t SESRD3:1;                /* Self-Test Enable Status in Reset Domain 3 */
    vuint32_t _unused_4:28;
  } B;
} CSTCU_RDENSTAT_tag;

typedef union CSTCU_LASTRDEN_union_tag { /* Last Run Reset Domain Enable */
  vuint32_t R;
  struct {
    vuint32_t LRSERD0:1;               /* Last Run Self-Test Enable Status in Reset Domain 0 */
    vuint32_t LRSERD1:1;               /* Last Run Self-Test Enable Status in Reset Domain 1 */
    vuint32_t LRSERD2:1;               /* Last Run Self-Test Enable Status in Reset Domain 2 */
    vuint32_t LRSERD3:1;               /* Last Run Self-Test Enable Status in Reset Domain 3 */
    vuint32_t _unused_4:28;
  } B;
} CSTCU_LASTRDEN_tag;

typedef union CSTCU_BYPLSTCU_union_tag { /* Bypass LSTCU */
  vuint32_t R;
  struct {
    vuint32_t BYP_L0:1;                /* Bypass LSTCU 0 */
    vuint32_t BYP_L1:1;                /* Bypass LSTCU 1 */
    vuint32_t BYP_L2:1;                /* Bypass LSTCU 2 */
    vuint32_t BYP_L3:1;                /* Bypass LSTCU 3 */
    vuint32_t BYP_L4:1;                /* Bypass LSTCU 4 */
    vuint32_t BYP_L5:1;                /* Bypass LSTCU 5 */
    vuint32_t BYP_L6:1;                /* Bypass LSTCU 6 */
    vuint32_t BYP_L7:1;                /* Bypass LSTCU 7 */
    vuint32_t BYP_L8:1;                /* Bypass LSTCU 8 */
    vuint32_t BYP_L9:1;                /* Bypass LSTCU 9 */
    vuint32_t BYP_L10:1;               /* Bypass LSTCU 10 */
    vuint32_t BYP_L11:1;               /* Bypass LSTCU 11 */
    vuint32_t BYP_L12:1;               /* Bypass LSTCU 12 */
    vuint32_t BYP_L13:1;               /* Bypass LSTCU 13 */
    vuint32_t BYP_L14:1;               /* Bypass LSTCU 14 */
    vuint32_t BYP_L15:1;               /* Bypass LSTCU 15 */
    vuint32_t BYP_L16:1;               /* Bypass LSTCU 16 */
    vuint32_t BYP_L17:1;               /* Bypass LSTCU 17 */
    vuint32_t BYP_L18:1;               /* Bypass LSTCU 18 */
    vuint32_t BYP_L19:1;               /* Bypass LSTCU 19 */
    vuint32_t _unused_20:12;
  } B;
} CSTCU_BYPLSTCU_tag;

typedef union CSTCU_STAG_union_tag {   /* Stagger */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MB_DELAY:8;              /* MBIST Delay */
    vuint32_t LB_DELAY:8;              /* LBIST Delay */
    vuint32_t _unused_24:8;
  } B;
} CSTCU_STAG_tag;

typedef union CSTCU_LMBPTR_union_tag { /* LSTCU MBIST Run Phase Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t MBPTR:8;                 /* LSTCU Pointer Scheduled During MBIST Run Phase */
    vuint32_t MBCSM:1;                 /* MBIST Concurrent or Sequential Mode */
    vuint32_t _unused_9:22;
    vuint32_t MBEOL:1;                 /* MBIST End of List */
  } B;
} CSTCU_LMBPTR_tag;

typedef union CSTCU_LLBPTR_union_tag { /* LSTCU LBIST Run phase Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t LBPTR:8;                 /* LSTCU Pointer Scheduled During LBIST Run Phase */
    vuint32_t LBCSM:1;                 /* LBIST Concurrent or Sequential Mode */
    vuint32_t _unused_9:22;
    vuint32_t LBEOL:1;                 /* LBIST End of List */
  } B;
} CSTCU_LLBPTR_tag;

struct CSTCU_tag {
  CSTCU_RUNSWREG_tag RUNSWREG;         /* Run Software */
  CSTCU_LSCHLVLD_tag LSCHLVLD;         /* LSTCU Scheduler List Valid */
  uint8_t CSTCU_reserved0[4];
  CSTCU_WDG_tag WDG;                   /* Watchdog Timer */
  CSTCU_IE_tag IE;                     /* Interrupt Enable */
  CSTCU_IF_tag IF;                     /* Interrupt Flag */
  uint8_t CSTCU_reserved1[8];
  CSTCU_ERR_STAT_tag ERR_STAT;         /* Error Status */
  uint8_t CSTCU_reserved2[4];
  CSTCU_ERR_FM_tag ERR_FM;             /* Error Fault Mapping */
  uint8_t CSTCU_reserved3[4];
  CSTCU_LRFSTAT_tag LRFSTAT[1];        /* LSTCU Recoverable Fault Status */
  uint8_t CSTCU_reserved4[12];
  CSTCU_LUFSTAT_tag LUFSTAT[1];        /* LSTCU Unrecoverable Fault Status */
  uint8_t CSTCU_reserved5[12];
  CSTCU_RDEN_tag RDEN;                 /* Reset Domain Self-Test Enable */
  CSTCU_RDENSTAT_tag RDENSTAT;         /* Reset Domain Enable Status */
  CSTCU_LASTRDEN_tag LASTRDEN;         /* Last Run Reset Domain Enable */
  uint8_t CSTCU_reserved6[4];
  CSTCU_BYPLSTCU_tag BYPLSTCU[1];      /* Bypass LSTCU */
  uint8_t CSTCU_reserved7[12];
  CSTCU_STAG_tag STAG;                 /* Stagger */
  uint8_t CSTCU_reserved8[396];
  CSTCU_LMBPTR_tag LMBPTR[20];         /* LSTCU MBIST Run Phase Scheduler Pointer */
  uint8_t CSTCU_reserved9[176];
  CSTCU_LLBPTR_tag LLBPTR[20];         /* LSTCU LBIST Run phase Scheduler Pointer */
};


/* ============================================================================
   =============================== Module: CTU ================================
   ============================================================================ */

typedef union CTU_TGSISR_union_tag {   /* Trigger Generator Subunit Input Selection Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t I1_RE:1;                 /* Input 1 Rising edge Enable */
    vuint32_t I1_FE:1;                 /* Input 1 Falling Edge Enable. */
    vuint32_t I2_RE:1;                 /* Input 2 Rising Edge Enable. */
    vuint32_t I2_FE:1;                 /* Input 2 Falling Edge Enable. */
    vuint32_t I3_RE:1;                 /* Input 3 Rising edge Enable */
    vuint32_t I3_FE:1;                 /* Input 3 Falling Edge Enable. */
    vuint32_t I4_RE:1;                 /* Input 4 Rising Edge Enable. */
    vuint32_t I4_FE:1;                 /* Input 4 Falling Edge Enable. */
    vuint32_t I5_RE:1;                 /* Input 5 Rising Edge Enable. */
    vuint32_t I5_FE:1;                 /* Input 5 Falling Edge Enable. */
    vuint32_t I6_RE:1;                 /* Input 6 Rising Edge Enable. */
    vuint32_t I6_FE:1;                 /* Input 6 Falling Edge Enable. */
    vuint32_t I7_RE:1;                 /* Input 7 Rising Edge Enable. */
    vuint32_t I7_FE:1;                 /* Input 7 Falling Edge Enable. */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t I9_RE:1;                 /* Input 9 Rising Edge Enable. */
    vuint32_t I9_FE:1;                 /* Input 9 Falling Edge Enable. */
    vuint32_t I10_RE:1;                /* Input 10 Rising Edge Enable. */
    vuint32_t I10_FE:1;                /* Input 10 Falling Edge Enable. */
    vuint32_t I11_RE:1;                /* Input 11 Rising Edge Enable. */
    vuint32_t I11_FE:1;                /* Input 11 Falling Edge Enable. */
    vuint32_t I12_RE:1;                /* Input 12 Rising Edge Enable. */
    vuint32_t I12_FE:1;                /* Input 12 Falling Edge Enable. */
    vuint32_t _unused_26:2;            /* Reserved */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t I15_RE:1;                /* Input 15 Rising Edge Enable. */
    vuint32_t I15_FE:1;                /* Input 15 Falling Edge Enable. */
  } B;
} CTU_TGSISR_tag;

typedef union CTU_TGSCR_union_tag {    /* Trigger Generator Subunit Control Register */
  vuint16_t R;
  struct {
    vuint16_t TGS_M:1;                 /* Trigger Generator Subunit Mode */
    vuint16_t MRS_SM:5;
    vuint16_t PRES:2;
    vuint16_t ET_TM:1;
    vuint16_t _unused_9:7;             /* Reserved */
  } B;
} CTU_TGSCR_tag;

typedef union CTU_TCR_union_tag {      /* Trigger Compare Register */
  vuint16_t R;
  struct {
    vuint16_t TCRV:16;                 /* Trigger Compare Register Value */
  } B;
} CTU_TCR_tag;

typedef union CTU_TGSCCR_union_tag {   /* TGS Counter Compare Register */
  vuint16_t R;
  struct {
    vuint16_t TGSCCV:16;
  } B;
} CTU_TGSCCR_tag;

typedef union CTU_TGSCRR_union_tag {   /* TGS Counter Reload Register */
  vuint16_t R;
  struct {
    vuint16_t TGSCRV:16;               /* TGSCRV */
  } B;
} CTU_TGSCRR_tag;

typedef union CTU_CLCR1_union_tag {    /* Commands List Control Register 1 */
  vuint32_t R;
  struct {
    vuint32_t T0_INDEX:5;
    vuint32_t _unused_5:3;             /* Reserved */
    vuint32_t T1_INDEX:5;
    vuint32_t _unused_13:3;            /* Reserved */
    vuint32_t T2_INDEX:5;
    vuint32_t _unused_21:3;            /* Reserved */
    vuint32_t T3_INDEX:5;
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} CTU_CLCR1_tag;

typedef union CTU_CLCR2_union_tag {    /* Commands List Control Register 2 */
  vuint32_t R;
  struct {
    vuint32_t T4_INDEX:5;
    vuint32_t _unused_5:3;             /* Reserved */
    vuint32_t T5_INDEX:5;
    vuint32_t _unused_13:3;            /* Reserved */
    vuint32_t T6_INDEX:5;
    vuint32_t _unused_21:3;            /* Reserved */
    vuint32_t T7_INDEX:5;              /* T7_INDEX */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} CTU_CLCR2_tag;

typedef union CTU_THCR1_union_tag {    /* Trigger Handler Control Register 1 */
  vuint32_t R;
  struct {
    vuint32_t T0_ADCE:1;
    vuint32_t _unused_1:1;             /* Reserved. */
    vuint32_t _unused_2:1;             /* Reserved. */
    vuint32_t _unused_3:1;             /* Reserved. */
    vuint32_t _unused_4:1;             /* Reserved. */
    vuint32_t _unused_5:1;             /* Reserved. */
    vuint32_t T0_E:1;
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t T1_ADCE:1;
    vuint32_t _unused_9:1;             /* Reserved. */
    vuint32_t _unused_10:1;            /* Reserved. */
    vuint32_t _unused_11:1;            /* Reserved. */
    vuint32_t _unused_12:1;            /* Reserved. */
    vuint32_t _unused_13:1;            /* Reserved. */
    vuint32_t T1_E:1;
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t T2_ADCE:1;
    vuint32_t _unused_17:1;            /* Reserved. */
    vuint32_t _unused_18:1;            /* Reserved. */
    vuint32_t _unused_19:1;            /* Reserved. */
    vuint32_t _unused_20:1;            /* Reserved. */
    vuint32_t _unused_21:1;            /* Reserved. */
    vuint32_t T2_E:1;
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t T3_ADCE:1;
    vuint32_t _unused_25:1;            /* Reserved. */
    vuint32_t _unused_26:1;            /* Reserved. */
    vuint32_t _unused_27:1;            /* Reserved. */
    vuint32_t _unused_28:1;            /* Reserved. */
    vuint32_t _unused_29:1;            /* Reserved. */
    vuint32_t T3_E:1;
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} CTU_THCR1_tag;

typedef union CTU_THCR2_union_tag {    /* Trigger Handler Control Register 2 */
  vuint32_t R;
  struct {
    vuint32_t T4_ADCE:1;
    vuint32_t _unused_1:1;             /* Reserved. */
    vuint32_t _unused_2:1;             /* Reserved. */
    vuint32_t _unused_3:1;             /* Reserved. */
    vuint32_t _unused_4:1;             /* Reserved. */
    vuint32_t _unused_5:1;             /* Reserved. */
    vuint32_t T4_E:1;
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t T5_ADCE:1;
    vuint32_t _unused_9:1;             /* Reserved. */
    vuint32_t _unused_10:1;            /* Reserved. */
    vuint32_t _unused_11:1;            /* Reserved. */
    vuint32_t _unused_12:1;            /* Reserved. */
    vuint32_t _unused_13:1;            /* Reserved. */
    vuint32_t T5_E:1;
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t T6_ADCE:1;
    vuint32_t _unused_17:1;            /* Reserved. */
    vuint32_t _unused_18:1;            /* Reserved. */
    vuint32_t _unused_19:1;            /* Reserved. */
    vuint32_t _unused_20:1;            /* Reserved. */
    vuint32_t _unused_21:1;            /* Reserved. */
    vuint32_t T6_E:1;
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t T7_ADCE:1;
    vuint32_t _unused_25:1;            /* Reserved. */
    vuint32_t _unused_26:1;            /* Reserved. */
    vuint32_t _unused_27:1;            /* Reserved. */
    vuint32_t _unused_28:1;            /* Reserved. */
    vuint32_t _unused_29:1;            /* Reserved. */
    vuint32_t T7_E:1;
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} CTU_THCR2_tag;

typedef union CTU_CLR_A_union_tag {    /* Commands List Register A for ADC single-conversion mode commands */
  vuint16_t R;
  struct {
    vuint16_t CH:4;                    /* ADC Port channel number. */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t SU:1;                    /* ADC Port A / Port B selection. */
    vuint16_t _unused_6:4;             /* Reserved */
    vuint16_t FIFO:3;                  /* FIFO used for ADC Port A / Port B. */
    vuint16_t CMS:1;
    vuint16_t LC:1;                    /* Last Command bit */
    vuint16_t CIR:1;
  } B;
} CTU_CLR_A_tag;

typedef union CTU_CLR_B_union_tag {    /* Command List Register B for ADC dual-conversion mode commands */
  vuint16_t R;
  struct {
    vuint16_t CH_A:4;                  /* ADC Port A Channel Number */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t CH_B:4;                  /* ADC Port B Channel Number */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t FIFO:3;                  /* FIFO used for ADC Port A / Port B. */
    vuint16_t CMS:1;                   /* Conversion Mode Selection */
    vuint16_t LC:1;                    /* Last Command Bit */
    vuint16_t CIR:1;
  } B;
} CTU_CLR_B_tag;

typedef union CTU_CLR_union_tag {
  CTU_CLR_A_tag A[24];                 /* Commands List Register A for ADC single-conversion mode commands */
  CTU_CLR_B_tag B[24];                 /* Command List Register B for ADC dual-conversion mode commands */
} CTU_CLR_tag;

typedef union CTU_FDCR_union_tag {     /* FIFO DMA Control Register */
  vuint16_t R;
  struct {
    vuint16_t DE0:1;
    vuint16_t DE1:1;
    vuint16_t DE2:1;
    vuint16_t DE3:1;
    vuint16_t _unused_4:4;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:4;            /* Reserved */
  } B;
} CTU_FDCR_tag;

typedef union CTU_FCR_union_tag {      /* FIFO Control Register */
  vuint32_t R;
  struct {
    vuint32_t FULL_EN0:1;
    vuint32_t EMPTY_EN0:1;
    vuint32_t OF_EN0:1;
    vuint32_t OR_EN0:1;
    vuint32_t FULL_EN1:1;
    vuint32_t EMPTY_EN1:1;
    vuint32_t OF_EN1:1;
    vuint32_t OR_EN1:1;
    vuint32_t FULL_EN2:1;
    vuint32_t EMPTY_EN2:1;
    vuint32_t OF_EN2:1;
    vuint32_t OR_EN2:1;
    vuint32_t FULL_EN3:1;
    vuint32_t EMPTY_EN3:1;
    vuint32_t OF_EN3:1;
    vuint32_t OR_EN3:1;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} CTU_FCR_tag;

typedef union CTU_FTH_union_tag {      /* FIFO Threshold Register */
  vuint32_t R;
  struct {
    vuint32_t TH0:8;
    vuint32_t TH1:8;
    vuint32_t TH2:8;
    vuint32_t TH3:8;
  } B;
} CTU_FTH_tag;

typedef union CTU_FST_union_tag {      /* FIFO Status Register */
  vuint32_t R;
  struct {
    vuint32_t FULL0:1;
    vuint32_t EMP0:1;
    vuint32_t OF0:1;
    vuint32_t OR0:1;
    vuint32_t FULL1:1;                 /* FULL1 */
    vuint32_t EMP1:1;
    vuint32_t OF1:1;
    vuint32_t OR1:1;
    vuint32_t FULL2:1;
    vuint32_t EMP2:1;
    vuint32_t OF2:1;
    vuint32_t OR2:1;
    vuint32_t FULL3:1;
    vuint32_t EMP3:1;
    vuint32_t OF3:1;
    vuint32_t OR3:1;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} CTU_FST_tag;

typedef union CTU_FR_union_tag {       /* FIFO Right Aligned Data Register */
  vuint32_t R;
  struct {
    vuint32_t DATA:12;
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t N_CH:3;
    vuint32_t ADC:1;                   /* This bit indicates from which ADC Port the value in the DATA field corresponds to. */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} CTU_FR_tag;

typedef union CTU_FL_union_tag {       /* FIFO Signed Left Aligned Data Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;             /* Reserved */
    vuint32_t LA_DATA:12;              /* LA_DATA */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t N_CH:3;
    vuint32_t ADC:1;                   /* This bit indicates from which ADC Port the value in the DATA field corresponds to. */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} CTU_FL_tag;

typedef union CTU_EFR_union_tag {      /* Error Flag Register */
  vuint16_t R;
  struct {
    vuint16_t MRS_RE:1;
    vuint16_t SM_TO:1;
    vuint16_t ICE:1;
    vuint16_t MRS_O:1;
    vuint16_t TGS_OSM:1;
    vuint16_t ADC_OE:1;
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t ERRCMP:1;
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t LIST_BE:1;
    vuint16_t _unused_14:2;            /* Reserved */
  } B;
} CTU_EFR_tag;

typedef union CTU_IFR_union_tag {      /* Interrupt Flag Register */
  vuint16_t R;
  struct {
    vuint16_t MRS_I:1;
    vuint16_t T0_I:1;
    vuint16_t T1_I:1;
    vuint16_t T2_I:1;
    vuint16_t T3_I:1;
    vuint16_t T4_I:1;
    vuint16_t T5_I:1;
    vuint16_t T6_I:1;
    vuint16_t T7_I:1;
    vuint16_t ADC_I:1;
    vuint16_t SERR_A:1;
    vuint16_t SERR_B:1;
    vuint16_t _unused_12:4;            /* Reserved */
  } B;
} CTU_IFR_tag;

typedef union CTU_IR_union_tag {       /* Interrupt/DMA Register */
  vuint16_t R;
  struct {
    vuint16_t IEE:1;
    vuint16_t MRS_IE:1;
    vuint16_t MRS_DMAE:1;
    vuint16_t DMA_DE:1;
    vuint16_t SAF_CNT_A_EN:1;          /* Enable the ADC Port A counter to check the conversion time. */
    vuint16_t SAF_CNT_B_EN:1;          /* Enable the ADC Port B counter to check the conversion time. */
    vuint16_t _unused_6:2;             /* Reserved */
    vuint16_t T0_IE:1;                 /* Trigger 0 Interrupt Enable */
    vuint16_t T1_IE:1;                 /* Trigger 1 Interrupt Enable */
    vuint16_t T2_IE:1;                 /* Trigger 2 Interrupt Enable */
    vuint16_t T3_IE:1;                 /* Trigger 3 Interrupt Enable */
    vuint16_t T4_IE:1;                 /* Trigger 4 Interrupt Enable */
    vuint16_t T5_IE:1;                 /* Trigger 5 Interrupt Enable */
    vuint16_t T6_IE:1;                 /* Trigger 6 Interrupt Enable */
    vuint16_t T7_IE:1;                 /* Trigger 7 Interrupt Enable */
  } B;
} CTU_IR_tag;

typedef union CTU_COTR_union_tag {     /* Control ON Time Register */
  vuint16_t R;
  struct {
    vuint16_t COTGT:8;                 /* COTGT */
    vuint16_t _unused_8:8;             /* Reserved */
  } B;
} CTU_COTR_tag;

typedef union CTU_CR_union_tag {       /* Control Register */
  vuint16_t R;
  struct {
    vuint16_t TGSISR_RE:1;
    vuint16_t GRE:1;
    vuint16_t MRS_SG:1;
    vuint16_t FGRE:1;
    vuint16_t CGRE:1;
    vuint16_t DFE:1;
    vuint16_t CTU_ODIS:1;
    vuint16_t CTU_ADC_R:1;
    vuint16_t T0_SG:1;
    vuint16_t T1_SG:1;
    vuint16_t T2_SG:1;
    vuint16_t T3_SG:1;
    vuint16_t T4_SG:1;
    vuint16_t T5_SG:1;
    vuint16_t T6_SG:1;
    vuint16_t T7_SG:1;
  } B;
} CTU_CR_tag;

typedef union CTU_DFR_union_tag {      /* Digital Filter Register */
  vuint16_t R;
  struct {
    vuint16_t FILTER_N:8;
    vuint16_t _unused_8:8;             /* Reserved */
  } B;
} CTU_DFR_tag;

typedef union CTU_EXPAR_union_tag {    /* Expected Value A Register */
  vuint16_t R;
  struct {
    vuint16_t EXPA:16;                 /* This value is the expected number of system clock cycles needed by ADC Port A to complete a conversion. */
  } B;
} CTU_EXPAR_tag;

typedef union CTU_EXPBR_union_tag {    /* Expected Value B Register */
  vuint16_t R;
  struct {
    vuint16_t EXPB:16;                 /* This value is the expected number of system clock cycles needed by ADC Port B to complete a conversion. */
  } B;
} CTU_EXPBR_tag;

typedef union CTU_CNTRNGR_union_tag {  /* Counter Range Register */
  vuint16_t R;
  struct {
    vuint16_t CNTRNG:8;                /* CNTRNG */
    vuint16_t _unused_8:8;             /* Reserved */
  } B;
} CTU_CNTRNGR_tag;

typedef union CTU_LISTCSR_union_tag {  /* List Control/Status Register */
  vuint32_t R;
  struct {
    vuint32_t PAR_LIST:1;
    vuint32_t _unused_1:15;            /* Reserved */
    vuint32_t LIST0_ADDR:5;            /* List Address 0. Indicates the command address being executed when LIST_BE flag in EFR register was set. */
    vuint32_t _unused_21:2;            /* Reserved */
    vuint32_t LIST0_BLK:1;
    vuint32_t LIST1_ADDR:5;
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t LIST1_BLK:1;
  } B;
} CTU_LISTCSR_tag;

struct CTU_tag {
  CTU_TGSISR_tag TGSISR;               /* Trigger Generator Subunit Input Selection Register */
  CTU_TGSCR_tag TGSCR;                 /* Trigger Generator Subunit Control Register */
  CTU_TCR_tag TCR[8];                  /* Trigger Compare Register */
  CTU_TGSCCR_tag TGSCCR;               /* TGS Counter Compare Register */
  CTU_TGSCRR_tag TGSCRR;               /* TGS Counter Reload Register */
  uint8_t CTU_reserved0[2];
  CTU_CLCR1_tag CLCR1;                 /* Commands List Control Register 1 */
  CTU_CLCR2_tag CLCR2;                 /* Commands List Control Register 2 */
  CTU_THCR1_tag THCR1;                 /* Trigger Handler Control Register 1 */
  CTU_THCR2_tag THCR2;                 /* Trigger Handler Control Register 2 */
  CTU_CLR_tag CLR;
  uint8_t CTU_reserved1[16];
  CTU_FDCR_tag FDCR;                   /* FIFO DMA Control Register */
  uint8_t CTU_reserved2[2];
  CTU_FCR_tag FCR;                     /* FIFO Control Register */
  CTU_FTH_tag FTH;                     /* FIFO Threshold Register */
  uint8_t CTU_reserved3[4];
  CTU_FST_tag FST;                     /* FIFO Status Register */
  CTU_FR_tag FR[4];                    /* FIFO Right Aligned Data Register */
  uint8_t CTU_reserved4[16];
  CTU_FL_tag FL[4];                    /* FIFO Signed Left Aligned Data Register */
  uint8_t CTU_reserved5[16];
  CTU_EFR_tag EFR;                     /* Error Flag Register */
  CTU_IFR_tag IFR;                     /* Interrupt Flag Register */
  CTU_IR_tag IR;                       /* Interrupt/DMA Register */
  CTU_COTR_tag COTR;                   /* Control ON Time Register */
  CTU_CR_tag CR;                       /* Control Register */
  CTU_DFR_tag DFR;                     /* Digital Filter Register */
  CTU_EXPAR_tag EXPAR;                 /* Expected Value A Register */
  CTU_EXPBR_tag EXPBR;                 /* Expected Value B Register */
  CTU_CNTRNGR_tag CNTRNGR;             /* Counter Range Register */
  uint8_t CTU_reserved6[2];
  CTU_LISTCSR_tag LISTCSR;             /* List Control/Status Register */
};


/* ============================================================================
   =============================== Module: C_VFCCU ============================
   ============================================================================ */

typedef union C_VFCCU_GDFHID_C0_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID0:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID1:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID2:3;                /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID3:3;                /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID4:3;                /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID5:3;                /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID6:3;                /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID7:3;                /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_GDFHID_C0_tag;

typedef union C_VFCCU_GDFHID_C1_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID8:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID9:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID10:3;               /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID11:3;               /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID12:3;               /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID13:3;               /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID14:3;               /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID15:3;               /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_GDFHID_C1_tag;

typedef union C_VFCCU_GFLTPO_C0_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS0:1;                   /* Polarity Selection */
    vuint32_t PS1:1;                   /* Polarity Selection */
    vuint32_t PS2:1;                   /* Polarity Selection */
    vuint32_t PS3:1;                   /* Polarity Selection */
    vuint32_t PS4:1;                   /* Polarity Selection */
    vuint32_t PS5:1;                   /* Polarity Selection */
    vuint32_t PS6:1;                   /* Polarity Selection */
    vuint32_t PS7:1;                   /* Polarity Selection */
    vuint32_t PS8:1;                   /* Polarity Selection */
    vuint32_t PS9:1;                   /* Polarity Selection */
    vuint32_t PS10:1;                  /* Polarity Selection */
    vuint32_t PS11:1;                  /* Polarity Selection */
    vuint32_t PS12:1;                  /* Polarity Selection */
    vuint32_t PS13:1;                  /* Polarity Selection */
    vuint32_t PS14:1;                  /* Polarity Selection */
    vuint32_t PS15:1;                  /* Polarity Selection */
    vuint32_t PS16:1;                  /* Polarity Selection */
    vuint32_t PS17:1;                  /* Polarity Selection */
    vuint32_t PS18:1;                  /* Polarity Selection */
    vuint32_t PS19:1;                  /* Polarity Selection */
    vuint32_t PS20:1;                  /* Polarity Selection */
    vuint32_t PS21:1;                  /* Polarity Selection */
    vuint32_t PS22:1;                  /* Polarity Selection */
    vuint32_t PS23:1;                  /* Polarity Selection */
    vuint32_t PS24:1;                  /* Polarity Selection */
    vuint32_t PS25:1;                  /* Polarity Selection */
    vuint32_t PS26:1;                  /* Polarity Selection */
    vuint32_t PS27:1;                  /* Polarity Selection */
    vuint32_t PS28:1;                  /* Polarity Selection */
    vuint32_t PS29:1;                  /* Polarity Selection */
    vuint32_t PS30:1;                  /* Polarity Selection */
    vuint32_t PS31:1;                  /* Polarity Selection */
  } B;
} C_VFCCU_GFLTPO_C0_tag;

typedef union C_VFCCU_GFLTPO_C1_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS32:1;                  /* Polarity Selection */
    vuint32_t PS33:1;                  /* Polarity Selection */
    vuint32_t PS34:1;                  /* Polarity Selection */
    vuint32_t PS35:1;                  /* Polarity Selection */
    vuint32_t PS36:1;                  /* Polarity Selection */
    vuint32_t PS37:1;                  /* Polarity Selection */
    vuint32_t PS38:1;                  /* Polarity Selection */
    vuint32_t PS39:1;                  /* Polarity Selection */
    vuint32_t PS40:1;                  /* Polarity Selection */
    vuint32_t PS41:1;                  /* Polarity Selection */
    vuint32_t PS42:1;                  /* Polarity Selection */
    vuint32_t PS43:1;                  /* Polarity Selection */
    vuint32_t PS44:1;                  /* Polarity Selection */
    vuint32_t PS45:1;                  /* Polarity Selection */
    vuint32_t PS46:1;                  /* Polarity Selection */
    vuint32_t PS47:1;                  /* Polarity Selection */
    vuint32_t PS48:1;                  /* Polarity Selection */
    vuint32_t PS49:1;                  /* Polarity Selection */
    vuint32_t PS50:1;                  /* Polarity Selection */
    vuint32_t PS51:1;                  /* Polarity Selection */
    vuint32_t PS52:1;                  /* Polarity Selection */
    vuint32_t PS53:1;                  /* Polarity Selection */
    vuint32_t PS54:1;                  /* Polarity Selection */
    vuint32_t PS55:1;                  /* Polarity Selection */
    vuint32_t PS56:1;                  /* Polarity Selection */
    vuint32_t PS57:1;                  /* Polarity Selection */
    vuint32_t PS58:1;                  /* Polarity Selection */
    vuint32_t PS59:1;                  /* Polarity Selection */
    vuint32_t PS60:1;                  /* Polarity Selection */
    vuint32_t PS61:1;                  /* Polarity Selection */
    vuint32_t PS62:1;                  /* Polarity Selection */
    vuint32_t PS63:1;                  /* Polarity Selection */
  } B;
} C_VFCCU_GFLTPO_C1_tag;

typedef union C_VFCCU_GFLTPO_C2_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS64:1;                  /* Polarity Selection */
    vuint32_t PS65:1;                  /* Polarity Selection */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_GFLTPO_C2_tag;

typedef union C_VFCCU_GFLTRC_C0_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW0:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW1:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW2:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW3:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW4:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW5:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW6:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW7:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW8:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW9:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW10:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW11:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW12:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW13:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW14:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW15:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW16:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW17:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW18:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW19:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW20:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW21:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW22:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW23:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW24:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW25:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW26:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW27:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW28:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW29:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW30:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW31:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
  } B;
} C_VFCCU_GFLTRC_C0_tag;

typedef union C_VFCCU_GFLTRC_C1_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW32:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW33:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW34:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW35:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW36:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW37:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW38:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW39:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW40:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW41:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW42:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW43:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW44:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW45:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW46:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW47:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW48:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW49:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW50:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW51:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW52:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW53:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW54:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW55:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW56:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW57:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW58:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW59:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW60:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW61:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW62:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW63:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
  } B;
} C_VFCCU_GFLTRC_C1_tag;

typedef union C_VFCCU_GFLTRC_C2_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW64:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW65:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_GFLTRC_C2_tag;

typedef union C_VFCCU_GFLTOVDC0_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS0:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS1:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS2:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS3:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS4:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS5:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS6:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS7:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS8:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS9:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS10:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS11:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS12:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS13:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS14:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS15:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS16:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS17:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS18:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS19:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS20:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS21:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS22:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS23:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS24:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS25:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS26:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS27:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS28:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS29:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS30:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS31:1;             /* Fault Overflow Detection Disable */
  } B;
} C_VFCCU_GFLTOVDC0_tag;

typedef union C_VFCCU_GFLTOVDC1_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS32:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS33:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS34:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS35:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS36:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS37:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS38:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS39:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS40:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS41:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS42:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS43:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS44:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS45:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS46:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS47:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS48:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS49:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS50:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS51:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS52:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS53:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS54:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS55:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS56:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS57:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS58:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS59:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS60:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS61:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS62:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS63:1;             /* Fault Overflow Detection Disable */
  } B;
} C_VFCCU_GFLTOVDC1_tag;

typedef union C_VFCCU_GFLTOVDC2_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS64:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS65:1;             /* Fault Overflow Detection Disable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_GFLTOVDC2_tag;

typedef union C_VFCCU_GCTRL_union_tag { /* Global Space Control */
  vuint32_t R;
  struct {
    vuint32_t OVF_EN:1;                /* Overflow Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_GCTRL_tag;

typedef union C_VFCCU_GINTOVFS_union_tag { /* Global DID FSM Status */
  vuint32_t R;
  struct {
    vuint32_t FSMSTATE:2;              /* FSM State */
    vuint32_t _unused_2:5;
    vuint32_t FLTSERV:1;               /* Fault Serve */
    vuint32_t OVF_DET:1;               /* Overflow Detect */
    vuint32_t _unused_9:7;
    vuint32_t SERV_DID:4;              /* Domain Being Serviced */
    vuint32_t _unused_20:4;
    vuint32_t OVF_DID:4;               /* Overflow DID */
    vuint32_t _unused_28:4;
  } B;
} C_VFCCU_GINTOVFS_tag;

typedef union C_VFCCU_GEXTOVFS_union_tag { /* Global External Overflow Status */
  vuint32_t R;
  struct {
    vuint32_t EXTOVFS0:1;              /* External Overflow Status */
    vuint32_t EXTOVFS1:1;              /* External Overflow Status */
    vuint32_t EXTOVFS2:1;              /* External Overflow Status */
    vuint32_t EXTOVFS3:1;              /* External Overflow Status */
    vuint32_t EXTOVFS4:1;              /* External Overflow Status */
    vuint32_t EXTOVFS5:1;              /* External Overflow Status */
    vuint32_t EXTOVFS6:1;              /* External Overflow Status */
    vuint32_t EXTOVFS7:1;              /* External Overflow Status */
    vuint32_t EXTOVFS8:1;              /* External Overflow Status */
    vuint32_t EXTOVFS9:1;              /* External Overflow Status */
    vuint32_t EXTOVFS10:1;             /* External Overflow Status */
    vuint32_t _unused_11:21;
  } B;
} C_VFCCU_GEXTOVFS_tag;

typedef union C_VFCCU_GRKNTIMC_union_tag { /* Global Reaction Timer Period */
  vuint32_t R;
  struct {
    vuint32_t RKTIMCFG:32;             /* Reaction Timer Period */
  } B;
} C_VFCCU_GRKNTIMC_tag;

typedef union C_VFCCU_GRKNTIMS_union_tag { /* Global Reaction Timer Status */
  vuint32_t R;
  struct {
    vuint32_t RKTIMVAL:32;             /* Current Reaction Timer */
  } B;
} C_VFCCU_GRKNTIMS_tag;

typedef union C_VFCCU_GOVFRKC0_union_tag { /* Global Overflow Reaction */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_GOVFRKC0_tag;

typedef union C_VFCCU_GOVFRKC1_union_tag { /* Global Overflow Reaction */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_GOVFRKC1_tag;

typedef union C_VFCCU_GMEOUTDC_union_tag { /* Global Minimum EOUT Duration */
  vuint32_t R;
  struct {
    vuint32_t EOUTMIND:32;             /* EOUT Minimum Duration */
  } B;
} C_VFCCU_GMEOUTDC_tag;

typedef union C_VFCCU_GEOUTTCT_union_tag { /* Global EOUT Timer Disable */
  vuint32_t R;
  struct {
    vuint32_t TMRDIS0:1;               /* Timer Disable */
    vuint32_t TMRDIS1:1;               /* Timer Disable */
    vuint32_t TMRDIS2:1;               /* Timer Disable */
    vuint32_t TMRDIS3:1;               /* Timer Disable */
    vuint32_t _unused_4:28;
  } B;
} C_VFCCU_GEOUTTCT_tag;

typedef union C_VFCCU_GLB_EOUT_GEOUTPNC_union_tag { /* Global EOUT Pin */
  vuint32_t R;
  struct {
    vuint32_t OBE_VALID:1;             /* Output Buffer Enable Valid */
    vuint32_t OBE_STAT:1;              /* Output Buffer Enable Status */
    vuint32_t _unused_2:1;
    vuint32_t DO_STAT:1;               /* Data Output Status */
    vuint32_t VAL_CTRL:2;              /* Invert Output */
    vuint32_t _unused_6:2;
    vuint32_t IBE:1;                   /* Input Buffer Enable */
    vuint32_t IND_STAT:1;              /* Input Data From Pad */
    vuint32_t INV_IP:1;                /* Invert Input */
    vuint32_t _unused_11:5;
    vuint32_t SOC_PAD_CTRL1:1;         /* SoC Pad Control 1: SRE0 */
    vuint32_t SOC_PAD_CTRL2:1;         /* SoC Pad Control 2: SRE1 */
    vuint32_t SOC_PAD_CTRL3:1;         /* SoC Pad Control 3: SRE2 */
    vuint32_t SOC_PAD_CTRL4:1;         /* SoC Pad Control 4: PUE */
    vuint32_t SOC_PAD_CTRL5:1;         /* SoC Pad Control 5: PUS */
    vuint32_t SOC_PAD_CTRL6:1;         /* SoC Pad Control 6: RCVR */
    vuint32_t _unused_22:10;           /* Reserved */
  } B;
} C_VFCCU_GLB_EOUT_GEOUTPNC_tag;

typedef union C_VFCCU_GLB_EOUT_GEOUTPMC_union_tag { /* Global EOUT Pin Map */
  vuint32_t R;
  struct {
    vuint32_t SPLENB0:1;               /* Special Enable Pins */
    vuint32_t SPLENB1:1;               /* Special Enable Pins */
    vuint32_t SPLENB2:1;               /* Special Enable Pins */
    vuint32_t _unused_3:29;
  } B;
} C_VFCCU_GLB_EOUT_GEOUTPMC_tag;

typedef union C_VFCCU_GLB_EOUT_GEOUTMC_union_tag { /* Global EOUT Mode */
  vuint32_t R;
  struct {
    vuint32_t EOUTM:3;                 /* EOUT Pin Mode */
    vuint32_t INV:1;                   /* Invert EOUT */
    vuint32_t _unused_4:28;
  } B;
} C_VFCCU_GLB_EOUT_GEOUTMC_tag;

typedef union C_VFCCU_GLB_EOUT_GEOUTTMS_union_tag { /* Global EOUT Timer */
  vuint32_t R;
  struct {
    vuint32_t EOUTDVAL:32;             /* EOUT Timer Current Value */
  } B;
} C_VFCCU_GLB_EOUT_GEOUTTMS_tag;

typedef union C_VFCCU_GLB_EOUT_GEOUTDIC_union_tag { /* Global EOUT DID */
  vuint32_t R;
  struct {
    vuint32_t DID_EOUT:4;              /* DID for Fault on EOUT */
    vuint32_t _unused_4:28;
  } B;
} C_VFCCU_GLB_EOUT_GEOUTDIC_tag;

typedef struct C_VFCCU_GLB_EOUT_struct_tag {
  C_VFCCU_GLB_EOUT_GEOUTPNC_tag GEOUTPNC; /* Global EOUT Pin */
  C_VFCCU_GLB_EOUT_GEOUTPMC_tag GEOUTPMC; /* Global EOUT Pin Map */
  C_VFCCU_GLB_EOUT_GEOUTMC_tag GEOUTMC; /* Global EOUT Mode */
  C_VFCCU_GLB_EOUT_GEOUTTMS_tag GEOUTTMS; /* Global EOUT Timer */
  C_VFCCU_GLB_EOUT_GEOUTDIC_tag GEOUTDIC; /* Global EOUT DID */
} C_VFCCU_GLB_EOUT_tag;

typedef union C_VFCCU_GDBGCFG_union_tag { /* Global Debug */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t FRZ:1;                   /* Freeze On First Fault */
    vuint32_t _unused_17:15;
  } B;
} C_VFCCU_GDBGCFG_tag;

typedef union C_VFCCU_GDBGSTAT_union_tag { /* Global Debug Status */
  vuint32_t R;
  struct {
    vuint32_t FLTIND:8;                /* Fault Index */
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_GDBGSTAT_tag;

typedef union C_VFCCU_FHCFG0_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_FHCFG0_tag;

typedef union C_VFCCU_FHSRVDS0_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} C_VFCCU_FHSRVDS0_tag;

typedef union C_VFCCU_FHFLTENC00_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC00_tag;

typedef union C_VFCCU_FHFLTENC01_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t EN53:1;                  /* Fault Enable */
    vuint32_t EN54:1;                  /* Fault Enable */
    vuint32_t EN55:1;                  /* Fault Enable */
    vuint32_t EN56:1;                  /* Fault Enable */
    vuint32_t EN57:1;                  /* Fault Enable */
    vuint32_t EN58:1;                  /* Fault Enable */
    vuint32_t EN59:1;                  /* Fault Enable */
    vuint32_t EN60:1;                  /* Fault Enable */
    vuint32_t EN61:1;                  /* Fault Enable */
    vuint32_t EN62:1;                  /* Fault Enable */
    vuint32_t EN63:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC01_tag;

typedef union C_VFCCU_FHFLTENC02_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN64:1;                  /* Fault Enable */
    vuint32_t EN65:1;                  /* Fault Enable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTENC02_tag;

typedef union C_VFCCU_FHFLTS00_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS00_tag;

typedef union C_VFCCU_FHFLTS01_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t STAT53:1;                /* Fault Status */
    vuint32_t STAT54:1;                /* Fault Status */
    vuint32_t STAT55:1;                /* Fault Status */
    vuint32_t STAT56:1;                /* Fault Status */
    vuint32_t STAT57:1;                /* Fault Status */
    vuint32_t STAT58:1;                /* Fault Status */
    vuint32_t STAT59:1;                /* Fault Status */
    vuint32_t STAT60:1;                /* Fault Status */
    vuint32_t STAT61:1;                /* Fault Status */
    vuint32_t STAT62:1;                /* Fault Status */
    vuint32_t STAT63:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS01_tag;

typedef union C_VFCCU_FHFLTS02_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT64:1;                /* Fault Status */
    vuint32_t STAT65:1;                /* Fault Status */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTS02_tag;

typedef union C_VFCCU_FHFLTRKC00_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC00_tag;

typedef union C_VFCCU_FHFLTRKC01_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC01_tag;

typedef union C_VFCCU_FHFLTRKC02_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC02_tag;

typedef union C_VFCCU_FHFLTRKC03_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC03_tag;

typedef union C_VFCCU_FHFLTRKC04_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC04_tag;

typedef union C_VFCCU_FHFLTRKC05_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC05_tag;

typedef union C_VFCCU_FHFLTRKC06_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL53:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL54:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL55:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC06_tag;

typedef union C_VFCCU_FHFLTRKC07_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL56:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL57:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL58:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL59:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL60:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL61:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL62:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL63:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC07_tag;

typedef union C_VFCCU_FHFLTRKC08_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL64:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL65:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_FHFLTRKC08_tag;

typedef union C_VFCCU_FHIMRKC0_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC0_00_tag;

typedef union C_VFCCU_FHIMRKC0_01_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC0_01_tag;

typedef union C_VFCCU_FHDLRKC0_00_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC0_00_tag;

typedef union C_VFCCU_FHDLRKC0_01_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC0_01_tag;

typedef union C_VFCCU_FHIMRKC0_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC0_10_tag;

typedef union C_VFCCU_FHIMRKC0_11_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC0_11_tag;

typedef union C_VFCCU_FHDLRKC0_10_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC0_10_tag;

typedef union C_VFCCU_FHDLRKC0_11_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC0_11_tag;

typedef union C_VFCCU_FHIMRKC0_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC0_20_tag;

typedef union C_VFCCU_FHIMRKC0_21_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC0_21_tag;

typedef union C_VFCCU_FHDLRKC0_20_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC0_20_tag;

typedef union C_VFCCU_FHDLRKC0_21_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC0_21_tag;

typedef union C_VFCCU_FHIMRKC0_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC0_30_tag;

typedef union C_VFCCU_FHIMRKC0_31_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC0_31_tag;

typedef union C_VFCCU_FHDLRKC0_30_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC0_30_tag;

typedef union C_VFCCU_FHDLRKC0_31_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC0_31_tag;

typedef union C_VFCCU_FHIMRKC0_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC0_40_tag;

typedef union C_VFCCU_FHIMRKC0_41_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC0_41_tag;

typedef union C_VFCCU_FHDLRKC0_40_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC0_40_tag;

typedef union C_VFCCU_FHDLRKC0_41_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC0_41_tag;

typedef union C_VFCCU_FHIMRKC0_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC0_50_tag;

typedef union C_VFCCU_FHIMRKC0_51_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC0_51_tag;

typedef union C_VFCCU_FHDLRKC0_50_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC0_50_tag;

typedef union C_VFCCU_FHDLRKC0_51_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC0_51_tag;

typedef union C_VFCCU_FHCFG1_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_FHCFG1_tag;

typedef union C_VFCCU_FHSRVDS1_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} C_VFCCU_FHSRVDS1_tag;

typedef union C_VFCCU_FHFLTENC10_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC10_tag;

typedef union C_VFCCU_FHFLTENC11_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t EN53:1;                  /* Fault Enable */
    vuint32_t EN54:1;                  /* Fault Enable */
    vuint32_t EN55:1;                  /* Fault Enable */
    vuint32_t EN56:1;                  /* Fault Enable */
    vuint32_t EN57:1;                  /* Fault Enable */
    vuint32_t EN58:1;                  /* Fault Enable */
    vuint32_t EN59:1;                  /* Fault Enable */
    vuint32_t EN60:1;                  /* Fault Enable */
    vuint32_t EN61:1;                  /* Fault Enable */
    vuint32_t EN62:1;                  /* Fault Enable */
    vuint32_t EN63:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC11_tag;

typedef union C_VFCCU_FHFLTENC12_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN64:1;                  /* Fault Enable */
    vuint32_t EN65:1;                  /* Fault Enable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTENC12_tag;

typedef union C_VFCCU_FHFLTS10_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS10_tag;

typedef union C_VFCCU_FHFLTS11_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t STAT53:1;                /* Fault Status */
    vuint32_t STAT54:1;                /* Fault Status */
    vuint32_t STAT55:1;                /* Fault Status */
    vuint32_t STAT56:1;                /* Fault Status */
    vuint32_t STAT57:1;                /* Fault Status */
    vuint32_t STAT58:1;                /* Fault Status */
    vuint32_t STAT59:1;                /* Fault Status */
    vuint32_t STAT60:1;                /* Fault Status */
    vuint32_t STAT61:1;                /* Fault Status */
    vuint32_t STAT62:1;                /* Fault Status */
    vuint32_t STAT63:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS11_tag;

typedef union C_VFCCU_FHFLTS12_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT64:1;                /* Fault Status */
    vuint32_t STAT65:1;                /* Fault Status */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTS12_tag;

typedef union C_VFCCU_FHFLTRKC10_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC10_tag;

typedef union C_VFCCU_FHFLTRKC11_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC11_tag;

typedef union C_VFCCU_FHFLTRKC12_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC12_tag;

typedef union C_VFCCU_FHFLTRKC13_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC13_tag;

typedef union C_VFCCU_FHFLTRKC14_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC14_tag;

typedef union C_VFCCU_FHFLTRKC15_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC15_tag;

typedef union C_VFCCU_FHFLTRKC16_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL53:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL54:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL55:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC16_tag;

typedef union C_VFCCU_FHFLTRKC17_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL56:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL57:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL58:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL59:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL60:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL61:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL62:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL63:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC17_tag;

typedef union C_VFCCU_FHFLTRKC18_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL64:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL65:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_FHFLTRKC18_tag;

typedef union C_VFCCU_FHIMRKC1_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC1_00_tag;

typedef union C_VFCCU_FHIMRKC1_01_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC1_01_tag;

typedef union C_VFCCU_FHDLRKC1_00_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC1_00_tag;

typedef union C_VFCCU_FHDLRKC1_01_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC1_01_tag;

typedef union C_VFCCU_FHIMRKC1_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC1_10_tag;

typedef union C_VFCCU_FHIMRKC1_11_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC1_11_tag;

typedef union C_VFCCU_FHDLRKC1_10_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC1_10_tag;

typedef union C_VFCCU_FHDLRKC1_11_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC1_11_tag;

typedef union C_VFCCU_FHIMRKC1_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC1_20_tag;

typedef union C_VFCCU_FHIMRKC1_21_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC1_21_tag;

typedef union C_VFCCU_FHDLRKC1_20_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC1_20_tag;

typedef union C_VFCCU_FHDLRKC1_21_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC1_21_tag;

typedef union C_VFCCU_FHIMRKC1_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC1_30_tag;

typedef union C_VFCCU_FHIMRKC1_31_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC1_31_tag;

typedef union C_VFCCU_FHDLRKC1_30_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC1_30_tag;

typedef union C_VFCCU_FHDLRKC1_31_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC1_31_tag;

typedef union C_VFCCU_FHIMRKC1_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC1_40_tag;

typedef union C_VFCCU_FHIMRKC1_41_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC1_41_tag;

typedef union C_VFCCU_FHDLRKC1_40_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC1_40_tag;

typedef union C_VFCCU_FHDLRKC1_41_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC1_41_tag;

typedef union C_VFCCU_FHIMRKC1_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC1_50_tag;

typedef union C_VFCCU_FHIMRKC1_51_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC1_51_tag;

typedef union C_VFCCU_FHDLRKC1_50_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC1_50_tag;

typedef union C_VFCCU_FHDLRKC1_51_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC1_51_tag;

typedef union C_VFCCU_FHCFG2_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_FHCFG2_tag;

typedef union C_VFCCU_FHSRVDS2_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} C_VFCCU_FHSRVDS2_tag;

typedef union C_VFCCU_FHFLTENC20_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC20_tag;

typedef union C_VFCCU_FHFLTENC21_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t EN53:1;                  /* Fault Enable */
    vuint32_t EN54:1;                  /* Fault Enable */
    vuint32_t EN55:1;                  /* Fault Enable */
    vuint32_t EN56:1;                  /* Fault Enable */
    vuint32_t EN57:1;                  /* Fault Enable */
    vuint32_t EN58:1;                  /* Fault Enable */
    vuint32_t EN59:1;                  /* Fault Enable */
    vuint32_t EN60:1;                  /* Fault Enable */
    vuint32_t EN61:1;                  /* Fault Enable */
    vuint32_t EN62:1;                  /* Fault Enable */
    vuint32_t EN63:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC21_tag;

typedef union C_VFCCU_FHFLTENC22_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN64:1;                  /* Fault Enable */
    vuint32_t EN65:1;                  /* Fault Enable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTENC22_tag;

typedef union C_VFCCU_FHFLTS20_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS20_tag;

typedef union C_VFCCU_FHFLTS21_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t STAT53:1;                /* Fault Status */
    vuint32_t STAT54:1;                /* Fault Status */
    vuint32_t STAT55:1;                /* Fault Status */
    vuint32_t STAT56:1;                /* Fault Status */
    vuint32_t STAT57:1;                /* Fault Status */
    vuint32_t STAT58:1;                /* Fault Status */
    vuint32_t STAT59:1;                /* Fault Status */
    vuint32_t STAT60:1;                /* Fault Status */
    vuint32_t STAT61:1;                /* Fault Status */
    vuint32_t STAT62:1;                /* Fault Status */
    vuint32_t STAT63:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS21_tag;

typedef union C_VFCCU_FHFLTS22_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT64:1;                /* Fault Status */
    vuint32_t STAT65:1;                /* Fault Status */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTS22_tag;

typedef union C_VFCCU_FHFLTRKC20_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC20_tag;

typedef union C_VFCCU_FHFLTRKC21_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC21_tag;

typedef union C_VFCCU_FHFLTRKC22_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC22_tag;

typedef union C_VFCCU_FHFLTRKC23_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC23_tag;

typedef union C_VFCCU_FHFLTRKC24_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC24_tag;

typedef union C_VFCCU_FHFLTRKC25_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC25_tag;

typedef union C_VFCCU_FHFLTRKC26_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL53:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL54:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL55:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC26_tag;

typedef union C_VFCCU_FHFLTRKC27_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL56:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL57:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL58:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL59:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL60:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL61:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL62:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL63:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC27_tag;

typedef union C_VFCCU_FHFLTRKC28_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL64:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL65:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_FHFLTRKC28_tag;

typedef union C_VFCCU_FHIMRKC2_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC2_00_tag;

typedef union C_VFCCU_FHIMRKC2_01_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC2_01_tag;

typedef union C_VFCCU_FHDLRKC2_00_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC2_00_tag;

typedef union C_VFCCU_FHDLRKC2_01_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC2_01_tag;

typedef union C_VFCCU_FHIMRKC2_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC2_10_tag;

typedef union C_VFCCU_FHIMRKC2_11_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC2_11_tag;

typedef union C_VFCCU_FHDLRKC2_10_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC2_10_tag;

typedef union C_VFCCU_FHDLRKC2_11_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC2_11_tag;

typedef union C_VFCCU_FHIMRKC2_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC2_20_tag;

typedef union C_VFCCU_FHIMRKC2_21_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC2_21_tag;

typedef union C_VFCCU_FHDLRKC2_20_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC2_20_tag;

typedef union C_VFCCU_FHDLRKC2_21_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC2_21_tag;

typedef union C_VFCCU_FHIMRKC2_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC2_30_tag;

typedef union C_VFCCU_FHIMRKC2_31_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC2_31_tag;

typedef union C_VFCCU_FHDLRKC2_30_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC2_30_tag;

typedef union C_VFCCU_FHDLRKC2_31_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC2_31_tag;

typedef union C_VFCCU_FHIMRKC2_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC2_40_tag;

typedef union C_VFCCU_FHIMRKC2_41_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC2_41_tag;

typedef union C_VFCCU_FHDLRKC2_40_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC2_40_tag;

typedef union C_VFCCU_FHDLRKC2_41_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC2_41_tag;

typedef union C_VFCCU_FHIMRKC2_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC2_50_tag;

typedef union C_VFCCU_FHIMRKC2_51_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC2_51_tag;

typedef union C_VFCCU_FHDLRKC2_50_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC2_50_tag;

typedef union C_VFCCU_FHDLRKC2_51_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC2_51_tag;

typedef union C_VFCCU_FHCFG3_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_FHCFG3_tag;

typedef union C_VFCCU_FHSRVDS3_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} C_VFCCU_FHSRVDS3_tag;

typedef union C_VFCCU_FHFLTENC30_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC30_tag;

typedef union C_VFCCU_FHFLTENC31_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t EN53:1;                  /* Fault Enable */
    vuint32_t EN54:1;                  /* Fault Enable */
    vuint32_t EN55:1;                  /* Fault Enable */
    vuint32_t EN56:1;                  /* Fault Enable */
    vuint32_t EN57:1;                  /* Fault Enable */
    vuint32_t EN58:1;                  /* Fault Enable */
    vuint32_t EN59:1;                  /* Fault Enable */
    vuint32_t EN60:1;                  /* Fault Enable */
    vuint32_t EN61:1;                  /* Fault Enable */
    vuint32_t EN62:1;                  /* Fault Enable */
    vuint32_t EN63:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC31_tag;

typedef union C_VFCCU_FHFLTENC32_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN64:1;                  /* Fault Enable */
    vuint32_t EN65:1;                  /* Fault Enable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTENC32_tag;

typedef union C_VFCCU_FHFLTS30_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS30_tag;

typedef union C_VFCCU_FHFLTS31_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t STAT53:1;                /* Fault Status */
    vuint32_t STAT54:1;                /* Fault Status */
    vuint32_t STAT55:1;                /* Fault Status */
    vuint32_t STAT56:1;                /* Fault Status */
    vuint32_t STAT57:1;                /* Fault Status */
    vuint32_t STAT58:1;                /* Fault Status */
    vuint32_t STAT59:1;                /* Fault Status */
    vuint32_t STAT60:1;                /* Fault Status */
    vuint32_t STAT61:1;                /* Fault Status */
    vuint32_t STAT62:1;                /* Fault Status */
    vuint32_t STAT63:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS31_tag;

typedef union C_VFCCU_FHFLTS32_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT64:1;                /* Fault Status */
    vuint32_t STAT65:1;                /* Fault Status */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTS32_tag;

typedef union C_VFCCU_FHFLTRKC30_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC30_tag;

typedef union C_VFCCU_FHFLTRKC31_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC31_tag;

typedef union C_VFCCU_FHFLTRKC32_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC32_tag;

typedef union C_VFCCU_FHFLTRKC33_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC33_tag;

typedef union C_VFCCU_FHFLTRKC34_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC34_tag;

typedef union C_VFCCU_FHFLTRKC35_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC35_tag;

typedef union C_VFCCU_FHFLTRKC36_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL53:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL54:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL55:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC36_tag;

typedef union C_VFCCU_FHFLTRKC37_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL56:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL57:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL58:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL59:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL60:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL61:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL62:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL63:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC37_tag;

typedef union C_VFCCU_FHFLTRKC38_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL64:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL65:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_FHFLTRKC38_tag;

typedef union C_VFCCU_FHIMRKC3_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC3_00_tag;

typedef union C_VFCCU_FHIMRKC3_01_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC3_01_tag;

typedef union C_VFCCU_FHDLRKC3_00_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC3_00_tag;

typedef union C_VFCCU_FHDLRKC3_01_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC3_01_tag;

typedef union C_VFCCU_FHIMRKC3_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC3_10_tag;

typedef union C_VFCCU_FHIMRKC3_11_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC3_11_tag;

typedef union C_VFCCU_FHDLRKC3_10_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC3_10_tag;

typedef union C_VFCCU_FHDLRKC3_11_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC3_11_tag;

typedef union C_VFCCU_FHIMRKC3_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC3_20_tag;

typedef union C_VFCCU_FHIMRKC3_21_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC3_21_tag;

typedef union C_VFCCU_FHDLRKC3_20_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC3_20_tag;

typedef union C_VFCCU_FHDLRKC3_21_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC3_21_tag;

typedef union C_VFCCU_FHIMRKC3_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC3_30_tag;

typedef union C_VFCCU_FHIMRKC3_31_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC3_31_tag;

typedef union C_VFCCU_FHDLRKC3_30_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC3_30_tag;

typedef union C_VFCCU_FHDLRKC3_31_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC3_31_tag;

typedef union C_VFCCU_FHIMRKC3_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC3_40_tag;

typedef union C_VFCCU_FHIMRKC3_41_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC3_41_tag;

typedef union C_VFCCU_FHDLRKC3_40_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC3_40_tag;

typedef union C_VFCCU_FHDLRKC3_41_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC3_41_tag;

typedef union C_VFCCU_FHIMRKC3_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC3_50_tag;

typedef union C_VFCCU_FHIMRKC3_51_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC3_51_tag;

typedef union C_VFCCU_FHDLRKC3_50_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC3_50_tag;

typedef union C_VFCCU_FHDLRKC3_51_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC3_51_tag;

typedef union C_VFCCU_FHCFG4_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_FHCFG4_tag;

typedef union C_VFCCU_FHSRVDS4_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} C_VFCCU_FHSRVDS4_tag;

typedef union C_VFCCU_FHFLTENC40_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC40_tag;

typedef union C_VFCCU_FHFLTENC41_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t EN53:1;                  /* Fault Enable */
    vuint32_t EN54:1;                  /* Fault Enable */
    vuint32_t EN55:1;                  /* Fault Enable */
    vuint32_t EN56:1;                  /* Fault Enable */
    vuint32_t EN57:1;                  /* Fault Enable */
    vuint32_t EN58:1;                  /* Fault Enable */
    vuint32_t EN59:1;                  /* Fault Enable */
    vuint32_t EN60:1;                  /* Fault Enable */
    vuint32_t EN61:1;                  /* Fault Enable */
    vuint32_t EN62:1;                  /* Fault Enable */
    vuint32_t EN63:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC41_tag;

typedef union C_VFCCU_FHFLTENC42_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN64:1;                  /* Fault Enable */
    vuint32_t EN65:1;                  /* Fault Enable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTENC42_tag;

typedef union C_VFCCU_FHFLTS40_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS40_tag;

typedef union C_VFCCU_FHFLTS41_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t STAT53:1;                /* Fault Status */
    vuint32_t STAT54:1;                /* Fault Status */
    vuint32_t STAT55:1;                /* Fault Status */
    vuint32_t STAT56:1;                /* Fault Status */
    vuint32_t STAT57:1;                /* Fault Status */
    vuint32_t STAT58:1;                /* Fault Status */
    vuint32_t STAT59:1;                /* Fault Status */
    vuint32_t STAT60:1;                /* Fault Status */
    vuint32_t STAT61:1;                /* Fault Status */
    vuint32_t STAT62:1;                /* Fault Status */
    vuint32_t STAT63:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS41_tag;

typedef union C_VFCCU_FHFLTS42_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT64:1;                /* Fault Status */
    vuint32_t STAT65:1;                /* Fault Status */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTS42_tag;

typedef union C_VFCCU_FHFLTRKC40_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC40_tag;

typedef union C_VFCCU_FHFLTRKC41_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC41_tag;

typedef union C_VFCCU_FHFLTRKC42_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC42_tag;

typedef union C_VFCCU_FHFLTRKC43_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC43_tag;

typedef union C_VFCCU_FHFLTRKC44_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC44_tag;

typedef union C_VFCCU_FHFLTRKC45_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC45_tag;

typedef union C_VFCCU_FHFLTRKC46_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL53:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL54:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL55:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC46_tag;

typedef union C_VFCCU_FHFLTRKC47_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL56:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL57:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL58:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL59:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL60:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL61:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL62:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL63:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC47_tag;

typedef union C_VFCCU_FHFLTRKC48_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL64:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL65:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_FHFLTRKC48_tag;

typedef union C_VFCCU_FHIMRKC4_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC4_00_tag;

typedef union C_VFCCU_FHIMRKC4_01_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC4_01_tag;

typedef union C_VFCCU_FHDLRKC4_00_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC4_00_tag;

typedef union C_VFCCU_FHDLRKC4_01_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC4_01_tag;

typedef union C_VFCCU_FHIMRKC4_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC4_10_tag;

typedef union C_VFCCU_FHIMRKC4_11_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC4_11_tag;

typedef union C_VFCCU_FHDLRKC4_10_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC4_10_tag;

typedef union C_VFCCU_FHDLRKC4_11_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC4_11_tag;

typedef union C_VFCCU_FHIMRKC4_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC4_20_tag;

typedef union C_VFCCU_FHIMRKC4_21_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC4_21_tag;

typedef union C_VFCCU_FHDLRKC4_20_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC4_20_tag;

typedef union C_VFCCU_FHDLRKC4_21_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC4_21_tag;

typedef union C_VFCCU_FHIMRKC4_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC4_30_tag;

typedef union C_VFCCU_FHIMRKC4_31_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC4_31_tag;

typedef union C_VFCCU_FHDLRKC4_30_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC4_30_tag;

typedef union C_VFCCU_FHDLRKC4_31_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC4_31_tag;

typedef union C_VFCCU_FHIMRKC4_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC4_40_tag;

typedef union C_VFCCU_FHIMRKC4_41_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC4_41_tag;

typedef union C_VFCCU_FHDLRKC4_40_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC4_40_tag;

typedef union C_VFCCU_FHDLRKC4_41_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC4_41_tag;

typedef union C_VFCCU_FHIMRKC4_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC4_50_tag;

typedef union C_VFCCU_FHIMRKC4_51_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC4_51_tag;

typedef union C_VFCCU_FHDLRKC4_50_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC4_50_tag;

typedef union C_VFCCU_FHDLRKC4_51_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC4_51_tag;

typedef union C_VFCCU_FHCFG5_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_FHCFG5_tag;

typedef union C_VFCCU_FHSRVDS5_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} C_VFCCU_FHSRVDS5_tag;

typedef union C_VFCCU_FHFLTENC50_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC50_tag;

typedef union C_VFCCU_FHFLTENC51_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t EN53:1;                  /* Fault Enable */
    vuint32_t EN54:1;                  /* Fault Enable */
    vuint32_t EN55:1;                  /* Fault Enable */
    vuint32_t EN56:1;                  /* Fault Enable */
    vuint32_t EN57:1;                  /* Fault Enable */
    vuint32_t EN58:1;                  /* Fault Enable */
    vuint32_t EN59:1;                  /* Fault Enable */
    vuint32_t EN60:1;                  /* Fault Enable */
    vuint32_t EN61:1;                  /* Fault Enable */
    vuint32_t EN62:1;                  /* Fault Enable */
    vuint32_t EN63:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC51_tag;

typedef union C_VFCCU_FHFLTENC52_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN64:1;                  /* Fault Enable */
    vuint32_t EN65:1;                  /* Fault Enable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTENC52_tag;

typedef union C_VFCCU_FHFLTS50_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS50_tag;

typedef union C_VFCCU_FHFLTS51_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t STAT53:1;                /* Fault Status */
    vuint32_t STAT54:1;                /* Fault Status */
    vuint32_t STAT55:1;                /* Fault Status */
    vuint32_t STAT56:1;                /* Fault Status */
    vuint32_t STAT57:1;                /* Fault Status */
    vuint32_t STAT58:1;                /* Fault Status */
    vuint32_t STAT59:1;                /* Fault Status */
    vuint32_t STAT60:1;                /* Fault Status */
    vuint32_t STAT61:1;                /* Fault Status */
    vuint32_t STAT62:1;                /* Fault Status */
    vuint32_t STAT63:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS51_tag;

typedef union C_VFCCU_FHFLTS52_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT64:1;                /* Fault Status */
    vuint32_t STAT65:1;                /* Fault Status */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTS52_tag;

typedef union C_VFCCU_FHFLTRKC50_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC50_tag;

typedef union C_VFCCU_FHFLTRKC51_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC51_tag;

typedef union C_VFCCU_FHFLTRKC52_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC52_tag;

typedef union C_VFCCU_FHFLTRKC53_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC53_tag;

typedef union C_VFCCU_FHFLTRKC54_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC54_tag;

typedef union C_VFCCU_FHFLTRKC55_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC55_tag;

typedef union C_VFCCU_FHFLTRKC56_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL53:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL54:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL55:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC56_tag;

typedef union C_VFCCU_FHFLTRKC57_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL56:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL57:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL58:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL59:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL60:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL61:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL62:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL63:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC57_tag;

typedef union C_VFCCU_FHFLTRKC58_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL64:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL65:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_FHFLTRKC58_tag;

typedef union C_VFCCU_FHIMRKC5_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC5_00_tag;

typedef union C_VFCCU_FHIMRKC5_01_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC5_01_tag;

typedef union C_VFCCU_FHDLRKC5_00_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC5_00_tag;

typedef union C_VFCCU_FHDLRKC5_01_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC5_01_tag;

typedef union C_VFCCU_FHIMRKC5_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC5_10_tag;

typedef union C_VFCCU_FHIMRKC5_11_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC5_11_tag;

typedef union C_VFCCU_FHDLRKC5_10_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC5_10_tag;

typedef union C_VFCCU_FHDLRKC5_11_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC5_11_tag;

typedef union C_VFCCU_FHIMRKC5_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC5_20_tag;

typedef union C_VFCCU_FHIMRKC5_21_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC5_21_tag;

typedef union C_VFCCU_FHDLRKC5_20_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC5_20_tag;

typedef union C_VFCCU_FHDLRKC5_21_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC5_21_tag;

typedef union C_VFCCU_FHIMRKC5_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC5_30_tag;

typedef union C_VFCCU_FHIMRKC5_31_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC5_31_tag;

typedef union C_VFCCU_FHDLRKC5_30_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC5_30_tag;

typedef union C_VFCCU_FHDLRKC5_31_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC5_31_tag;

typedef union C_VFCCU_FHIMRKC5_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC5_40_tag;

typedef union C_VFCCU_FHIMRKC5_41_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC5_41_tag;

typedef union C_VFCCU_FHDLRKC5_40_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC5_40_tag;

typedef union C_VFCCU_FHDLRKC5_41_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC5_41_tag;

typedef union C_VFCCU_FHIMRKC5_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC5_50_tag;

typedef union C_VFCCU_FHIMRKC5_51_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC5_51_tag;

typedef union C_VFCCU_FHDLRKC5_50_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC5_50_tag;

typedef union C_VFCCU_FHDLRKC5_51_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC5_51_tag;

typedef union C_VFCCU_FHCFG6_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_FHCFG6_tag;

typedef union C_VFCCU_FHSRVDS6_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} C_VFCCU_FHSRVDS6_tag;

typedef union C_VFCCU_FHFLTENC60_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC60_tag;

typedef union C_VFCCU_FHFLTENC61_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t EN53:1;                  /* Fault Enable */
    vuint32_t EN54:1;                  /* Fault Enable */
    vuint32_t EN55:1;                  /* Fault Enable */
    vuint32_t EN56:1;                  /* Fault Enable */
    vuint32_t EN57:1;                  /* Fault Enable */
    vuint32_t EN58:1;                  /* Fault Enable */
    vuint32_t EN59:1;                  /* Fault Enable */
    vuint32_t EN60:1;                  /* Fault Enable */
    vuint32_t EN61:1;                  /* Fault Enable */
    vuint32_t EN62:1;                  /* Fault Enable */
    vuint32_t EN63:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC61_tag;

typedef union C_VFCCU_FHFLTENC62_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN64:1;                  /* Fault Enable */
    vuint32_t EN65:1;                  /* Fault Enable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTENC62_tag;

typedef union C_VFCCU_FHFLTS60_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS60_tag;

typedef union C_VFCCU_FHFLTS61_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t STAT53:1;                /* Fault Status */
    vuint32_t STAT54:1;                /* Fault Status */
    vuint32_t STAT55:1;                /* Fault Status */
    vuint32_t STAT56:1;                /* Fault Status */
    vuint32_t STAT57:1;                /* Fault Status */
    vuint32_t STAT58:1;                /* Fault Status */
    vuint32_t STAT59:1;                /* Fault Status */
    vuint32_t STAT60:1;                /* Fault Status */
    vuint32_t STAT61:1;                /* Fault Status */
    vuint32_t STAT62:1;                /* Fault Status */
    vuint32_t STAT63:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS61_tag;

typedef union C_VFCCU_FHFLTS62_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT64:1;                /* Fault Status */
    vuint32_t STAT65:1;                /* Fault Status */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTS62_tag;

typedef union C_VFCCU_FHFLTRKC60_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC60_tag;

typedef union C_VFCCU_FHFLTRKC61_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC61_tag;

typedef union C_VFCCU_FHFLTRKC62_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC62_tag;

typedef union C_VFCCU_FHFLTRKC63_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC63_tag;

typedef union C_VFCCU_FHFLTRKC64_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC64_tag;

typedef union C_VFCCU_FHFLTRKC65_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC65_tag;

typedef union C_VFCCU_FHFLTRKC66_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL53:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL54:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL55:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC66_tag;

typedef union C_VFCCU_FHFLTRKC67_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL56:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL57:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL58:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL59:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL60:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL61:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL62:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL63:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC67_tag;

typedef union C_VFCCU_FHFLTRKC68_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL64:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL65:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_FHFLTRKC68_tag;

typedef union C_VFCCU_FHIMRKC6_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC6_00_tag;

typedef union C_VFCCU_FHIMRKC6_01_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC6_01_tag;

typedef union C_VFCCU_FHDLRKC6_00_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC6_00_tag;

typedef union C_VFCCU_FHDLRKC6_01_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC6_01_tag;

typedef union C_VFCCU_FHIMRKC6_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC6_10_tag;

typedef union C_VFCCU_FHIMRKC6_11_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC6_11_tag;

typedef union C_VFCCU_FHDLRKC6_10_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC6_10_tag;

typedef union C_VFCCU_FHDLRKC6_11_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC6_11_tag;

typedef union C_VFCCU_FHIMRKC6_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC6_20_tag;

typedef union C_VFCCU_FHIMRKC6_21_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC6_21_tag;

typedef union C_VFCCU_FHDLRKC6_20_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC6_20_tag;

typedef union C_VFCCU_FHDLRKC6_21_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC6_21_tag;

typedef union C_VFCCU_FHIMRKC6_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC6_30_tag;

typedef union C_VFCCU_FHIMRKC6_31_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC6_31_tag;

typedef union C_VFCCU_FHDLRKC6_30_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC6_30_tag;

typedef union C_VFCCU_FHDLRKC6_31_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC6_31_tag;

typedef union C_VFCCU_FHIMRKC6_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC6_40_tag;

typedef union C_VFCCU_FHIMRKC6_41_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC6_41_tag;

typedef union C_VFCCU_FHDLRKC6_40_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC6_40_tag;

typedef union C_VFCCU_FHDLRKC6_41_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC6_41_tag;

typedef union C_VFCCU_FHIMRKC6_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC6_50_tag;

typedef union C_VFCCU_FHIMRKC6_51_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC6_51_tag;

typedef union C_VFCCU_FHDLRKC6_50_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC6_50_tag;

typedef union C_VFCCU_FHDLRKC6_51_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC6_51_tag;

typedef union C_VFCCU_FHCFG7_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} C_VFCCU_FHCFG7_tag;

typedef union C_VFCCU_FHSRVDS7_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} C_VFCCU_FHSRVDS7_tag;

typedef union C_VFCCU_FHFLTENC70_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC70_tag;

typedef union C_VFCCU_FHFLTENC71_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t EN53:1;                  /* Fault Enable */
    vuint32_t EN54:1;                  /* Fault Enable */
    vuint32_t EN55:1;                  /* Fault Enable */
    vuint32_t EN56:1;                  /* Fault Enable */
    vuint32_t EN57:1;                  /* Fault Enable */
    vuint32_t EN58:1;                  /* Fault Enable */
    vuint32_t EN59:1;                  /* Fault Enable */
    vuint32_t EN60:1;                  /* Fault Enable */
    vuint32_t EN61:1;                  /* Fault Enable */
    vuint32_t EN62:1;                  /* Fault Enable */
    vuint32_t EN63:1;                  /* Fault Enable */
  } B;
} C_VFCCU_FHFLTENC71_tag;

typedef union C_VFCCU_FHFLTENC72_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN64:1;                  /* Fault Enable */
    vuint32_t EN65:1;                  /* Fault Enable */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTENC72_tag;

typedef union C_VFCCU_FHFLTS70_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS70_tag;

typedef union C_VFCCU_FHFLTS71_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t STAT53:1;                /* Fault Status */
    vuint32_t STAT54:1;                /* Fault Status */
    vuint32_t STAT55:1;                /* Fault Status */
    vuint32_t STAT56:1;                /* Fault Status */
    vuint32_t STAT57:1;                /* Fault Status */
    vuint32_t STAT58:1;                /* Fault Status */
    vuint32_t STAT59:1;                /* Fault Status */
    vuint32_t STAT60:1;                /* Fault Status */
    vuint32_t STAT61:1;                /* Fault Status */
    vuint32_t STAT62:1;                /* Fault Status */
    vuint32_t STAT63:1;                /* Fault Status */
  } B;
} C_VFCCU_FHFLTS71_tag;

typedef union C_VFCCU_FHFLTS72_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT64:1;                /* Fault Status */
    vuint32_t STAT65:1;                /* Fault Status */
    vuint32_t _unused_2:30;
  } B;
} C_VFCCU_FHFLTS72_tag;

typedef union C_VFCCU_FHFLTRKC70_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC70_tag;

typedef union C_VFCCU_FHFLTRKC71_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC71_tag;

typedef union C_VFCCU_FHFLTRKC72_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC72_tag;

typedef union C_VFCCU_FHFLTRKC73_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC73_tag;

typedef union C_VFCCU_FHFLTRKC74_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC74_tag;

typedef union C_VFCCU_FHFLTRKC75_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC75_tag;

typedef union C_VFCCU_FHFLTRKC76_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL53:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL54:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL55:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC76_tag;

typedef union C_VFCCU_FHFLTRKC77_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL56:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL57:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL58:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL59:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL60:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL61:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL62:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL63:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} C_VFCCU_FHFLTRKC77_tag;

typedef union C_VFCCU_FHFLTRKC78_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL64:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL65:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} C_VFCCU_FHFLTRKC78_tag;

typedef union C_VFCCU_FHIMRKC7_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC7_00_tag;

typedef union C_VFCCU_FHIMRKC7_01_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC7_01_tag;

typedef union C_VFCCU_FHDLRKC7_00_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC7_00_tag;

typedef union C_VFCCU_FHDLRKC7_01_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC7_01_tag;

typedef union C_VFCCU_FHIMRKC7_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC7_10_tag;

typedef union C_VFCCU_FHIMRKC7_11_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC7_11_tag;

typedef union C_VFCCU_FHDLRKC7_10_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC7_10_tag;

typedef union C_VFCCU_FHDLRKC7_11_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC7_11_tag;

typedef union C_VFCCU_FHIMRKC7_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC7_20_tag;

typedef union C_VFCCU_FHIMRKC7_21_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC7_21_tag;

typedef union C_VFCCU_FHDLRKC7_20_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC7_20_tag;

typedef union C_VFCCU_FHDLRKC7_21_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC7_21_tag;

typedef union C_VFCCU_FHIMRKC7_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC7_30_tag;

typedef union C_VFCCU_FHIMRKC7_31_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC7_31_tag;

typedef union C_VFCCU_FHDLRKC7_30_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC7_30_tag;

typedef union C_VFCCU_FHDLRKC7_31_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC7_31_tag;

typedef union C_VFCCU_FHIMRKC7_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC7_40_tag;

typedef union C_VFCCU_FHIMRKC7_41_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC7_41_tag;

typedef union C_VFCCU_FHDLRKC7_40_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC7_40_tag;

typedef union C_VFCCU_FHDLRKC7_41_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC7_41_tag;

typedef union C_VFCCU_FHIMRKC7_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHIMRKC7_50_tag;

typedef union C_VFCCU_FHIMRKC7_51_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHIMRKC7_51_tag;

typedef union C_VFCCU_FHDLRKC7_50_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t EOUTEN0:1;               /* EOUT Enable */
    vuint32_t EOUTEN1:1;               /* EOUT Enable */
    vuint32_t EOUTEN2:1;               /* EOUT Enable */
    vuint32_t EOUTEN3:1;               /* EOUT Enable */
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t RKNEN6:1;                /* Reaction Line Enable */
    vuint32_t RKNEN7:1;                /* Reaction Line Enable */
    vuint32_t RKNEN8:1;                /* Reaction Line Enable */
    vuint32_t RKNEN9:1;                /* Reaction Line Enable */
    vuint32_t RKNEN10:1;               /* Reaction Line Enable */
    vuint32_t RKNEN11:1;               /* Reaction Line Enable */
    vuint32_t RKNEN12:1;               /* Reaction Line Enable */
    vuint32_t RKNEN13:1;               /* Reaction Line Enable */
    vuint32_t RKNEN14:1;               /* Reaction Line Enable */
    vuint32_t RKNEN15:1;               /* Reaction Line Enable */
    vuint32_t RKNEN16:1;               /* Reaction Line Enable */
    vuint32_t RKNEN17:1;               /* Reaction Line Enable */
    vuint32_t RKNEN18:1;               /* Reaction Line Enable */
    vuint32_t RKNEN19:1;               /* Reaction Line Enable */
    vuint32_t RKNEN20:1;               /* Reaction Line Enable */
    vuint32_t RKNEN21:1;               /* Reaction Line Enable */
    vuint32_t RKNEN22:1;               /* Reaction Line Enable */
    vuint32_t RKNEN23:1;               /* Reaction Line Enable */
    vuint32_t RKNEN24:1;               /* Reaction Line Enable */
    vuint32_t RKNEN25:1;               /* Reaction Line Enable */
    vuint32_t RKNEN26:1;               /* Reaction Line Enable */
    vuint32_t RKNEN27:1;               /* Reaction Line Enable */
  } B;
} C_VFCCU_FHDLRKC7_50_tag;

typedef union C_VFCCU_FHDLRKC7_51_union_tag { /* Delayed Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN28:1;               /* Reaction Line Enable */
    vuint32_t RKNEN29:1;               /* Reaction Line Enable */
    vuint32_t RKNEN30:1;               /* Reaction Line Enable */
    vuint32_t RKNEN31:1;               /* Reaction Line Enable */
    vuint32_t RKNEN32:1;               /* Reaction Line Enable */
    vuint32_t RKNEN33:1;               /* Reaction Line Enable */
    vuint32_t RKNEN34:1;               /* Reaction Line Enable */
    vuint32_t RKNEN35:1;               /* Reaction Line Enable */
    vuint32_t RKNEN36:1;               /* Reaction Line Enable */
    vuint32_t RKNEN37:1;               /* Reaction Line Enable */
    vuint32_t RKNEN38:1;               /* Reaction Line Enable */
    vuint32_t RKNEN39:1;               /* Reaction Line Enable */
    vuint32_t RKNEN40:1;               /* Reaction Line Enable */
    vuint32_t RKNEN41:1;               /* Reaction Line Enable */
    vuint32_t RKNEN42:1;               /* Reaction Line Enable */
    vuint32_t RKNEN43:1;               /* Reaction Line Enable */
    vuint32_t RKNEN44:1;               /* Reaction Line Enable */
    vuint32_t RKNEN45:1;               /* Reaction Line Enable */
    vuint32_t RKNEN46:1;               /* Reaction Line Enable */
    vuint32_t RKNEN47:1;               /* Reaction Line Enable */
    vuint32_t RKNEN48:1;               /* Reaction Line Enable */
    vuint32_t RKNEN49:1;               /* Reaction Line Enable */
    vuint32_t _unused_22:10;
  } B;
} C_VFCCU_FHDLRKC7_51_tag;

struct C_VFCCU_tag {
  C_VFCCU_GDFHID_C0_tag GDFHID_C0;     /* Global DID-FHID Map */
  C_VFCCU_GDFHID_C1_tag GDFHID_C1;     /* Global DID-FHID Map */
  uint8_t C_VFCCU_reserved0[24];
  C_VFCCU_GFLTPO_C0_tag GFLTPO_C0;     /* Global Fault Polarity */
  C_VFCCU_GFLTPO_C1_tag GFLTPO_C1;     /* Global Fault Polarity */
  C_VFCCU_GFLTPO_C2_tag GFLTPO_C2;     /* Global Fault Polarity */
  uint8_t C_VFCCU_reserved1[52];
  C_VFCCU_GFLTRC_C0_tag GFLTRC_C0;     /* Global Fault Recovery */
  C_VFCCU_GFLTRC_C1_tag GFLTRC_C1;     /* Global Fault Recovery */
  C_VFCCU_GFLTRC_C2_tag GFLTRC_C2;     /* Global Fault Recovery */
  uint8_t C_VFCCU_reserved2[52];
  C_VFCCU_GFLTOVDC0_tag GFLTOVDC0;     /* Global Fault Overflow Detection */
  C_VFCCU_GFLTOVDC1_tag GFLTOVDC1;     /* Global Fault Overflow Detection */
  C_VFCCU_GFLTOVDC2_tag GFLTOVDC2;     /* Global Fault Overflow Detection */
  uint8_t C_VFCCU_reserved3[132];
  C_VFCCU_GCTRL_tag GCTRL;             /* Global Space Control */
  C_VFCCU_GINTOVFS_tag GINTOVFS;       /* Global DID FSM Status */
  C_VFCCU_GEXTOVFS_tag GEXTOVFS;       /* Global External Overflow Status */
  uint8_t C_VFCCU_reserved4[4];
  C_VFCCU_GRKNTIMC_tag GRKNTIMC;       /* Global Reaction Timer Period */
  uint8_t C_VFCCU_reserved5[4];
  C_VFCCU_GRKNTIMS_tag GRKNTIMS;       /* Global Reaction Timer Status */
  uint8_t C_VFCCU_reserved6[12];
  C_VFCCU_GOVFRKC0_tag GOVFRKC0;       /* Global Overflow Reaction */
  C_VFCCU_GOVFRKC1_tag GOVFRKC1;       /* Global Overflow Reaction */
  uint8_t C_VFCCU_reserved7[60];
  C_VFCCU_GMEOUTDC_tag GMEOUTDC;       /* Global Minimum EOUT Duration */
  C_VFCCU_GEOUTTCT_tag GEOUTTCT;       /* Global EOUT Timer Disable */
  C_VFCCU_GLB_EOUT_tag GLB_EOUT[4];
  uint8_t C_VFCCU_reserved8[240];
  C_VFCCU_GDBGCFG_tag GDBGCFG;         /* Global Debug */
  C_VFCCU_GDBGSTAT_tag GDBGSTAT;       /* Global Debug Status */
  uint8_t C_VFCCU_reserved9[64788];
  C_VFCCU_FHCFG0_tag FHCFG0;           /* Fault Handler */
  C_VFCCU_FHSRVDS0_tag FHSRVDS0;       /* Fault Handler Status */
  uint8_t C_VFCCU_reserved10[8];
  C_VFCCU_FHFLTENC00_tag FHFLTENC00;   /* Fault Enable */
  C_VFCCU_FHFLTENC01_tag FHFLTENC01;   /* Fault Enable */
  C_VFCCU_FHFLTENC02_tag FHFLTENC02;   /* Fault Enable */
  uint8_t C_VFCCU_reserved11[52];
  C_VFCCU_FHFLTS00_tag FHFLTS00;       /* Fault Status */
  C_VFCCU_FHFLTS01_tag FHFLTS01;       /* Fault Status */
  C_VFCCU_FHFLTS02_tag FHFLTS02;       /* Fault Status */
  uint8_t C_VFCCU_reserved12[52];
  C_VFCCU_FHFLTRKC00_tag FHFLTRKC00;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC01_tag FHFLTRKC01;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC02_tag FHFLTRKC02;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC03_tag FHFLTRKC03;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC04_tag FHFLTRKC04;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC05_tag FHFLTRKC05;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC06_tag FHFLTRKC06;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC07_tag FHFLTRKC07;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC08_tag FHFLTRKC08;   /* Fault Reaction Set Configuration */
  uint8_t C_VFCCU_reserved13[220];
  C_VFCCU_FHIMRKC0_00_tag FHIMRKC0_00; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC0_01_tag FHIMRKC0_01; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC0_00_tag FHDLRKC0_00; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC0_01_tag FHDLRKC0_01; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC0_10_tag FHIMRKC0_10; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC0_11_tag FHIMRKC0_11; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC0_10_tag FHDLRKC0_10; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC0_11_tag FHDLRKC0_11; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC0_20_tag FHIMRKC0_20; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC0_21_tag FHIMRKC0_21; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC0_20_tag FHDLRKC0_20; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC0_21_tag FHDLRKC0_21; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC0_30_tag FHIMRKC0_30; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC0_31_tag FHIMRKC0_31; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC0_30_tag FHDLRKC0_30; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC0_31_tag FHDLRKC0_31; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC0_40_tag FHIMRKC0_40; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC0_41_tag FHIMRKC0_41; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC0_40_tag FHDLRKC0_40; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC0_41_tag FHDLRKC0_41; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC0_50_tag FHIMRKC0_50; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC0_51_tag FHIMRKC0_51; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC0_50_tag FHDLRKC0_50; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC0_51_tag FHDLRKC0_51; /* Delayed Reaction Configuration */
  uint8_t C_VFCCU_reserved14[65040];
  C_VFCCU_FHCFG1_tag FHCFG1;           /* Fault Handler */
  C_VFCCU_FHSRVDS1_tag FHSRVDS1;       /* Fault Handler Status */
  uint8_t C_VFCCU_reserved15[8];
  C_VFCCU_FHFLTENC10_tag FHFLTENC10;   /* Fault Enable */
  C_VFCCU_FHFLTENC11_tag FHFLTENC11;   /* Fault Enable */
  C_VFCCU_FHFLTENC12_tag FHFLTENC12;   /* Fault Enable */
  uint8_t C_VFCCU_reserved16[52];
  C_VFCCU_FHFLTS10_tag FHFLTS10;       /* Fault Status */
  C_VFCCU_FHFLTS11_tag FHFLTS11;       /* Fault Status */
  C_VFCCU_FHFLTS12_tag FHFLTS12;       /* Fault Status */
  uint8_t C_VFCCU_reserved17[52];
  C_VFCCU_FHFLTRKC10_tag FHFLTRKC10;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC11_tag FHFLTRKC11;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC12_tag FHFLTRKC12;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC13_tag FHFLTRKC13;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC14_tag FHFLTRKC14;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC15_tag FHFLTRKC15;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC16_tag FHFLTRKC16;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC17_tag FHFLTRKC17;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC18_tag FHFLTRKC18;   /* Fault Reaction Set Configuration */
  uint8_t C_VFCCU_reserved18[220];
  C_VFCCU_FHIMRKC1_00_tag FHIMRKC1_00; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC1_01_tag FHIMRKC1_01; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC1_00_tag FHDLRKC1_00; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC1_01_tag FHDLRKC1_01; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC1_10_tag FHIMRKC1_10; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC1_11_tag FHIMRKC1_11; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC1_10_tag FHDLRKC1_10; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC1_11_tag FHDLRKC1_11; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC1_20_tag FHIMRKC1_20; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC1_21_tag FHIMRKC1_21; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC1_20_tag FHDLRKC1_20; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC1_21_tag FHDLRKC1_21; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC1_30_tag FHIMRKC1_30; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC1_31_tag FHIMRKC1_31; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC1_30_tag FHDLRKC1_30; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC1_31_tag FHDLRKC1_31; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC1_40_tag FHIMRKC1_40; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC1_41_tag FHIMRKC1_41; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC1_40_tag FHDLRKC1_40; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC1_41_tag FHDLRKC1_41; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC1_50_tag FHIMRKC1_50; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC1_51_tag FHIMRKC1_51; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC1_50_tag FHDLRKC1_50; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC1_51_tag FHDLRKC1_51; /* Delayed Reaction Configuration */
  uint8_t C_VFCCU_reserved19[65040];
  C_VFCCU_FHCFG2_tag FHCFG2;           /* Fault Handler */
  C_VFCCU_FHSRVDS2_tag FHSRVDS2;       /* Fault Handler Status */
  uint8_t C_VFCCU_reserved20[8];
  C_VFCCU_FHFLTENC20_tag FHFLTENC20;   /* Fault Enable */
  C_VFCCU_FHFLTENC21_tag FHFLTENC21;   /* Fault Enable */
  C_VFCCU_FHFLTENC22_tag FHFLTENC22;   /* Fault Enable */
  uint8_t C_VFCCU_reserved21[52];
  C_VFCCU_FHFLTS20_tag FHFLTS20;       /* Fault Status */
  C_VFCCU_FHFLTS21_tag FHFLTS21;       /* Fault Status */
  C_VFCCU_FHFLTS22_tag FHFLTS22;       /* Fault Status */
  uint8_t C_VFCCU_reserved22[52];
  C_VFCCU_FHFLTRKC20_tag FHFLTRKC20;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC21_tag FHFLTRKC21;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC22_tag FHFLTRKC22;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC23_tag FHFLTRKC23;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC24_tag FHFLTRKC24;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC25_tag FHFLTRKC25;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC26_tag FHFLTRKC26;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC27_tag FHFLTRKC27;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC28_tag FHFLTRKC28;   /* Fault Reaction Set Configuration */
  uint8_t C_VFCCU_reserved23[220];
  C_VFCCU_FHIMRKC2_00_tag FHIMRKC2_00; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC2_01_tag FHIMRKC2_01; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC2_00_tag FHDLRKC2_00; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC2_01_tag FHDLRKC2_01; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC2_10_tag FHIMRKC2_10; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC2_11_tag FHIMRKC2_11; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC2_10_tag FHDLRKC2_10; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC2_11_tag FHDLRKC2_11; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC2_20_tag FHIMRKC2_20; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC2_21_tag FHIMRKC2_21; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC2_20_tag FHDLRKC2_20; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC2_21_tag FHDLRKC2_21; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC2_30_tag FHIMRKC2_30; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC2_31_tag FHIMRKC2_31; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC2_30_tag FHDLRKC2_30; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC2_31_tag FHDLRKC2_31; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC2_40_tag FHIMRKC2_40; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC2_41_tag FHIMRKC2_41; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC2_40_tag FHDLRKC2_40; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC2_41_tag FHDLRKC2_41; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC2_50_tag FHIMRKC2_50; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC2_51_tag FHIMRKC2_51; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC2_50_tag FHDLRKC2_50; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC2_51_tag FHDLRKC2_51; /* Delayed Reaction Configuration */
  uint8_t C_VFCCU_reserved24[65040];
  C_VFCCU_FHCFG3_tag FHCFG3;           /* Fault Handler */
  C_VFCCU_FHSRVDS3_tag FHSRVDS3;       /* Fault Handler Status */
  uint8_t C_VFCCU_reserved25[8];
  C_VFCCU_FHFLTENC30_tag FHFLTENC30;   /* Fault Enable */
  C_VFCCU_FHFLTENC31_tag FHFLTENC31;   /* Fault Enable */
  C_VFCCU_FHFLTENC32_tag FHFLTENC32;   /* Fault Enable */
  uint8_t C_VFCCU_reserved26[52];
  C_VFCCU_FHFLTS30_tag FHFLTS30;       /* Fault Status */
  C_VFCCU_FHFLTS31_tag FHFLTS31;       /* Fault Status */
  C_VFCCU_FHFLTS32_tag FHFLTS32;       /* Fault Status */
  uint8_t C_VFCCU_reserved27[52];
  C_VFCCU_FHFLTRKC30_tag FHFLTRKC30;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC31_tag FHFLTRKC31;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC32_tag FHFLTRKC32;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC33_tag FHFLTRKC33;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC34_tag FHFLTRKC34;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC35_tag FHFLTRKC35;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC36_tag FHFLTRKC36;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC37_tag FHFLTRKC37;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC38_tag FHFLTRKC38;   /* Fault Reaction Set Configuration */
  uint8_t C_VFCCU_reserved28[220];
  C_VFCCU_FHIMRKC3_00_tag FHIMRKC3_00; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC3_01_tag FHIMRKC3_01; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC3_00_tag FHDLRKC3_00; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC3_01_tag FHDLRKC3_01; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC3_10_tag FHIMRKC3_10; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC3_11_tag FHIMRKC3_11; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC3_10_tag FHDLRKC3_10; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC3_11_tag FHDLRKC3_11; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC3_20_tag FHIMRKC3_20; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC3_21_tag FHIMRKC3_21; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC3_20_tag FHDLRKC3_20; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC3_21_tag FHDLRKC3_21; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC3_30_tag FHIMRKC3_30; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC3_31_tag FHIMRKC3_31; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC3_30_tag FHDLRKC3_30; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC3_31_tag FHDLRKC3_31; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC3_40_tag FHIMRKC3_40; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC3_41_tag FHIMRKC3_41; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC3_40_tag FHDLRKC3_40; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC3_41_tag FHDLRKC3_41; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC3_50_tag FHIMRKC3_50; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC3_51_tag FHIMRKC3_51; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC3_50_tag FHDLRKC3_50; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC3_51_tag FHDLRKC3_51; /* Delayed Reaction Configuration */
  uint8_t C_VFCCU_reserved29[65040];
  C_VFCCU_FHCFG4_tag FHCFG4;           /* Fault Handler */
  C_VFCCU_FHSRVDS4_tag FHSRVDS4;       /* Fault Handler Status */
  uint8_t C_VFCCU_reserved30[8];
  C_VFCCU_FHFLTENC40_tag FHFLTENC40;   /* Fault Enable */
  C_VFCCU_FHFLTENC41_tag FHFLTENC41;   /* Fault Enable */
  C_VFCCU_FHFLTENC42_tag FHFLTENC42;   /* Fault Enable */
  uint8_t C_VFCCU_reserved31[52];
  C_VFCCU_FHFLTS40_tag FHFLTS40;       /* Fault Status */
  C_VFCCU_FHFLTS41_tag FHFLTS41;       /* Fault Status */
  C_VFCCU_FHFLTS42_tag FHFLTS42;       /* Fault Status */
  uint8_t C_VFCCU_reserved32[52];
  C_VFCCU_FHFLTRKC40_tag FHFLTRKC40;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC41_tag FHFLTRKC41;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC42_tag FHFLTRKC42;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC43_tag FHFLTRKC43;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC44_tag FHFLTRKC44;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC45_tag FHFLTRKC45;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC46_tag FHFLTRKC46;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC47_tag FHFLTRKC47;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC48_tag FHFLTRKC48;   /* Fault Reaction Set Configuration */
  uint8_t C_VFCCU_reserved33[220];
  C_VFCCU_FHIMRKC4_00_tag FHIMRKC4_00; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC4_01_tag FHIMRKC4_01; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC4_00_tag FHDLRKC4_00; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC4_01_tag FHDLRKC4_01; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC4_10_tag FHIMRKC4_10; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC4_11_tag FHIMRKC4_11; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC4_10_tag FHDLRKC4_10; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC4_11_tag FHDLRKC4_11; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC4_20_tag FHIMRKC4_20; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC4_21_tag FHIMRKC4_21; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC4_20_tag FHDLRKC4_20; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC4_21_tag FHDLRKC4_21; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC4_30_tag FHIMRKC4_30; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC4_31_tag FHIMRKC4_31; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC4_30_tag FHDLRKC4_30; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC4_31_tag FHDLRKC4_31; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC4_40_tag FHIMRKC4_40; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC4_41_tag FHIMRKC4_41; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC4_40_tag FHDLRKC4_40; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC4_41_tag FHDLRKC4_41; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC4_50_tag FHIMRKC4_50; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC4_51_tag FHIMRKC4_51; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC4_50_tag FHDLRKC4_50; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC4_51_tag FHDLRKC4_51; /* Delayed Reaction Configuration */
  uint8_t C_VFCCU_reserved34[65040];
  C_VFCCU_FHCFG5_tag FHCFG5;           /* Fault Handler */
  C_VFCCU_FHSRVDS5_tag FHSRVDS5;       /* Fault Handler Status */
  uint8_t C_VFCCU_reserved35[8];
  C_VFCCU_FHFLTENC50_tag FHFLTENC50;   /* Fault Enable */
  C_VFCCU_FHFLTENC51_tag FHFLTENC51;   /* Fault Enable */
  C_VFCCU_FHFLTENC52_tag FHFLTENC52;   /* Fault Enable */
  uint8_t C_VFCCU_reserved36[52];
  C_VFCCU_FHFLTS50_tag FHFLTS50;       /* Fault Status */
  C_VFCCU_FHFLTS51_tag FHFLTS51;       /* Fault Status */
  C_VFCCU_FHFLTS52_tag FHFLTS52;       /* Fault Status */
  uint8_t C_VFCCU_reserved37[52];
  C_VFCCU_FHFLTRKC50_tag FHFLTRKC50;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC51_tag FHFLTRKC51;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC52_tag FHFLTRKC52;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC53_tag FHFLTRKC53;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC54_tag FHFLTRKC54;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC55_tag FHFLTRKC55;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC56_tag FHFLTRKC56;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC57_tag FHFLTRKC57;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC58_tag FHFLTRKC58;   /* Fault Reaction Set Configuration */
  uint8_t C_VFCCU_reserved38[220];
  C_VFCCU_FHIMRKC5_00_tag FHIMRKC5_00; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC5_01_tag FHIMRKC5_01; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC5_00_tag FHDLRKC5_00; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC5_01_tag FHDLRKC5_01; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC5_10_tag FHIMRKC5_10; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC5_11_tag FHIMRKC5_11; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC5_10_tag FHDLRKC5_10; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC5_11_tag FHDLRKC5_11; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC5_20_tag FHIMRKC5_20; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC5_21_tag FHIMRKC5_21; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC5_20_tag FHDLRKC5_20; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC5_21_tag FHDLRKC5_21; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC5_30_tag FHIMRKC5_30; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC5_31_tag FHIMRKC5_31; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC5_30_tag FHDLRKC5_30; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC5_31_tag FHDLRKC5_31; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC5_40_tag FHIMRKC5_40; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC5_41_tag FHIMRKC5_41; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC5_40_tag FHDLRKC5_40; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC5_41_tag FHDLRKC5_41; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC5_50_tag FHIMRKC5_50; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC5_51_tag FHIMRKC5_51; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC5_50_tag FHDLRKC5_50; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC5_51_tag FHDLRKC5_51; /* Delayed Reaction Configuration */
  uint8_t C_VFCCU_reserved39[65040];
  C_VFCCU_FHCFG6_tag FHCFG6;           /* Fault Handler */
  C_VFCCU_FHSRVDS6_tag FHSRVDS6;       /* Fault Handler Status */
  uint8_t C_VFCCU_reserved40[8];
  C_VFCCU_FHFLTENC60_tag FHFLTENC60;   /* Fault Enable */
  C_VFCCU_FHFLTENC61_tag FHFLTENC61;   /* Fault Enable */
  C_VFCCU_FHFLTENC62_tag FHFLTENC62;   /* Fault Enable */
  uint8_t C_VFCCU_reserved41[52];
  C_VFCCU_FHFLTS60_tag FHFLTS60;       /* Fault Status */
  C_VFCCU_FHFLTS61_tag FHFLTS61;       /* Fault Status */
  C_VFCCU_FHFLTS62_tag FHFLTS62;       /* Fault Status */
  uint8_t C_VFCCU_reserved42[52];
  C_VFCCU_FHFLTRKC60_tag FHFLTRKC60;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC61_tag FHFLTRKC61;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC62_tag FHFLTRKC62;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC63_tag FHFLTRKC63;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC64_tag FHFLTRKC64;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC65_tag FHFLTRKC65;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC66_tag FHFLTRKC66;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC67_tag FHFLTRKC67;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC68_tag FHFLTRKC68;   /* Fault Reaction Set Configuration */
  uint8_t C_VFCCU_reserved43[220];
  C_VFCCU_FHIMRKC6_00_tag FHIMRKC6_00; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC6_01_tag FHIMRKC6_01; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC6_00_tag FHDLRKC6_00; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC6_01_tag FHDLRKC6_01; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC6_10_tag FHIMRKC6_10; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC6_11_tag FHIMRKC6_11; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC6_10_tag FHDLRKC6_10; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC6_11_tag FHDLRKC6_11; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC6_20_tag FHIMRKC6_20; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC6_21_tag FHIMRKC6_21; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC6_20_tag FHDLRKC6_20; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC6_21_tag FHDLRKC6_21; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC6_30_tag FHIMRKC6_30; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC6_31_tag FHIMRKC6_31; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC6_30_tag FHDLRKC6_30; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC6_31_tag FHDLRKC6_31; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC6_40_tag FHIMRKC6_40; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC6_41_tag FHIMRKC6_41; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC6_40_tag FHDLRKC6_40; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC6_41_tag FHDLRKC6_41; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC6_50_tag FHIMRKC6_50; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC6_51_tag FHIMRKC6_51; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC6_50_tag FHDLRKC6_50; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC6_51_tag FHDLRKC6_51; /* Delayed Reaction Configuration */
  uint8_t C_VFCCU_reserved44[65040];
  C_VFCCU_FHCFG7_tag FHCFG7;           /* Fault Handler */
  C_VFCCU_FHSRVDS7_tag FHSRVDS7;       /* Fault Handler Status */
  uint8_t C_VFCCU_reserved45[8];
  C_VFCCU_FHFLTENC70_tag FHFLTENC70;   /* Fault Enable */
  C_VFCCU_FHFLTENC71_tag FHFLTENC71;   /* Fault Enable */
  C_VFCCU_FHFLTENC72_tag FHFLTENC72;   /* Fault Enable */
  uint8_t C_VFCCU_reserved46[52];
  C_VFCCU_FHFLTS70_tag FHFLTS70;       /* Fault Status */
  C_VFCCU_FHFLTS71_tag FHFLTS71;       /* Fault Status */
  C_VFCCU_FHFLTS72_tag FHFLTS72;       /* Fault Status */
  uint8_t C_VFCCU_reserved47[52];
  C_VFCCU_FHFLTRKC70_tag FHFLTRKC70;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC71_tag FHFLTRKC71;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC72_tag FHFLTRKC72;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC73_tag FHFLTRKC73;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC74_tag FHFLTRKC74;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC75_tag FHFLTRKC75;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC76_tag FHFLTRKC76;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC77_tag FHFLTRKC77;   /* Fault Reaction Set Configuration */
  C_VFCCU_FHFLTRKC78_tag FHFLTRKC78;   /* Fault Reaction Set Configuration */
  uint8_t C_VFCCU_reserved48[220];
  C_VFCCU_FHIMRKC7_00_tag FHIMRKC7_00; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC7_01_tag FHIMRKC7_01; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC7_00_tag FHDLRKC7_00; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC7_01_tag FHDLRKC7_01; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC7_10_tag FHIMRKC7_10; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC7_11_tag FHIMRKC7_11; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC7_10_tag FHDLRKC7_10; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC7_11_tag FHDLRKC7_11; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC7_20_tag FHIMRKC7_20; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC7_21_tag FHIMRKC7_21; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC7_20_tag FHDLRKC7_20; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC7_21_tag FHDLRKC7_21; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC7_30_tag FHIMRKC7_30; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC7_31_tag FHIMRKC7_31; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC7_30_tag FHDLRKC7_30; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC7_31_tag FHDLRKC7_31; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC7_40_tag FHIMRKC7_40; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC7_41_tag FHIMRKC7_41; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC7_40_tag FHDLRKC7_40; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC7_41_tag FHDLRKC7_41; /* Delayed Reaction Configuration */
  C_VFCCU_FHIMRKC7_50_tag FHIMRKC7_50; /* Immediate Reaction Configuration */
  C_VFCCU_FHIMRKC7_51_tag FHIMRKC7_51; /* Immediate Reaction Configuration */
  C_VFCCU_FHDLRKC7_50_tag FHDLRKC7_50; /* Delayed Reaction Configuration */
  C_VFCCU_FHDLRKC7_51_tag FHDLRKC7_51; /* Delayed Reaction Configuration */
};


/* ============================================================================
   =============================== Module: DATA_MEMORY_SUBSYSTEM ==============
   ============================================================================ */

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DMBA_union_tag { /* IDM Base Address */
  vuint32_t R;
  struct {
    vuint32_t DMBA:32;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DMBA_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DMBE_union_tag { /* DMSS Idle Indication */
  vuint32_t R;
  struct {
    vuint32_t WBE:1;
    vuint32_t EWBE:1;
    vuint32_t EDPE:1;
    vuint32_t DMSS_IDLE:1;
    vuint32_t ADLB:1;
    vuint32_t _unused_5:2;
    vuint32_t AXIP:1;
    vuint32_t DMA:1;
    vuint32_t QMAN:1;
    vuint32_t _unused_10:6;
    vuint32_t L1WBB:1;
    vuint32_t _unused_17:15;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DMBE_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_HDCFG_union_tag { /* DMSS Hardware Configuration */
  vuint32_t R;
  struct {
    vuint32_t BLK_NUM:1;
    vuint32_t BNK_NUM:1;
    vuint32_t TCM_SIZE:3;
    vuint32_t EDAP_AXI_WID:1;
    vuint32_t _unused_6:1;
    vuint32_t EDP_AXI_WID:2;
    vuint32_t _unused_9:23;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_HDCFG_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_GPOUT_union_tag { /* MSS General Purpose 2 */
  vuint32_t R;
  struct {
    vuint32_t GPOUT:32;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_GPOUT_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DACC_union_tag { /* DDMA Access Configuration */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t DOL:4;
    vuint32_t UOL:3;
    vuint32_t _unused_11:21;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DACC_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_SDCFG_union_tag { /* MSS SW Configuration */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t DAPE:1;
    vuint32_t _unused_23:2;
    vuint32_t EDAP_DS:1;
    vuint32_t _unused_26:6;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_SDCFG_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_BARRIER_union_tag { /* MSS Memory Barrier Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t INT_ACT:1;
    vuint32_t _unused_8:24;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_BARRIER_tag;

typedef union DATA_MEMORY_SUBSYSTEM_D_DMAB_union_tag { /* DDMA Status */
  vuint32_t R;
  struct {
    vuint32_t RESERVED0:1;
    vuint32_t _unused_1:31;
  } B;
} DATA_MEMORY_SUBSYSTEM_D_DMAB_tag;

typedef union DATA_MEMORY_SUBSYSTEM_DBG_WRC0_union_tag { /* EDP Write Response Counter */
  vuint32_t R;
  struct {
    vuint32_t WRC0:7;
    vuint32_t MAX_OS0:2;
    vuint32_t _unused_9:23;
  } B;
} DATA_MEMORY_SUBSYSTEM_DBG_WRC0_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DDTC_union_tag { /* DDMA Linear Transfer Size */
  vuint32_t R;
  struct {
    vuint32_t DDTC:22;
    vuint32_t _unused_22:10;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DDTC_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_2DCFG1_union_tag { /* DDMA Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t TDT:1;
    vuint32_t _unused_1:1;
    vuint32_t CLIP_EN:2;
    vuint32_t TRTYP:3;
    vuint32_t _unused_7:1;
    vuint32_t UNPACK:4;
    vuint32_t ELEMENT_SZ:2;
    vuint32_t CH_NUM:2;
    vuint32_t _unused_16:16;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_2DCFG1_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_2DCFG2_union_tag { /* DDMA Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t TILE_WIDTH:16;
    vuint32_t TILE_HEIGHT:16;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_2DCFG2_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_2DCFG3_union_tag { /* DDMA Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t INT_STRIDE:16;
    vuint32_t EXT_STRIDE:16;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_2DCFG3_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_2DCFG4_union_tag { /* DDMA Configuration 4 */
  vuint32_t R;
  struct {
    vuint32_t PADVAL:32;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_2DCFG4_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_2DCFG5_union_tag { /* DDMA Configuration 5 */
  vuint32_t R;
  struct {
    vuint32_t TCLIP:16;
    vuint32_t BCLIP:16;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_2DCFG5_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_2DCFG6_union_tag { /* DDMA Configuration 6 */
  vuint32_t R;
  struct {
    vuint32_t LCLIP:16;
    vuint32_t RCLIP:16;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_2DCFG6_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DDQS_union_tag { /* DDMA Status */
  vuint32_t R;
  struct {
    vuint32_t QEMPTY:1;
    vuint32_t QFULL:1;
    vuint32_t QCOUNT:2;
    vuint32_t _unused_4:2;
    vuint32_t QAUTO:1;
    vuint32_t QOVFL:1;
    vuint32_t STS0:2;
    vuint32_t STS1:2;
    vuint32_t STS2:2;
    vuint32_t _unused_14:2;
    vuint32_t LSID:2;
    vuint32_t _unused_18:14;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DDQS_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DDEA_union_tag { /* DDMA Address 1 */
  vuint32_t R;
  struct {
    vuint32_t DDEA:32;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DDEA_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DDIA_union_tag { /* DDMA Address 2 */
  vuint32_t R;
  struct {
    vuint32_t DDIA:32;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DDIA_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DDCL_union_tag { /* DDMA Control */
  vuint32_t R;
  struct {
    vuint32_t IIT:1;
    vuint32_t TRID:3;
    vuint32_t BSZ:4;
    vuint32_t DDST:1;
    vuint32_t EXTW:1;
    vuint32_t DDIE:1;
    vuint32_t _unused_11:5;
    vuint32_t MESSAGE:1;
    vuint32_t _unused_17:7;
    vuint32_t DDMA_TASK_ID:7;
    vuint32_t _unused_31:1;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DDCL_tag;

typedef union DATA_MEMORY_SUBSYSTEM_DDMA_PAUSE_union_tag { /* DDMA Pause Control */
  vuint32_t R;
  struct {
    vuint32_t DDMA_PAUSE:1;
    vuint32_t DDMA_PAUSE_STATUS:1;
    vuint32_t _unused_2:30;
  } B;
} DATA_MEMORY_SUBSYSTEM_DDMA_PAUSE_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DDESC_ID_union_tag { /* DDMA Interrupt Source */
  vuint32_t R;
  struct {
    vuint32_t Q0_TASK:1;
    vuint32_t Q1_TASK:1;
    vuint32_t Q2_TASK:1;
    vuint32_t Q3_TASK:1;
    vuint32_t _unused_4:12;
    vuint32_t MSTR_TASK:1;
    vuint32_t _unused_17:15;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DDESC_ID_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSS_DDESC_OV_union_tag { /* DDMA Task ID Overwrite Indication */
  vuint32_t R;
  struct {
    vuint32_t Q0_TASK_OV:1;
    vuint32_t Q1_TASK_OV:1;
    vuint32_t Q2_TASK_OV:1;
    vuint32_t Q3_TASK_OV:1;
    vuint32_t _unused_4:12;
    vuint32_t MSTR_TASK_OV:1;
    vuint32_t _unused_17:15;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSS_DDESC_OV_tag;

typedef union DATA_MEMORY_SUBSYSTEM_MSTR_TASK_ID_union_tag { /* DDMA Task (Master) ID */
  vuint32_t R;
  struct {
    vuint32_t DDMA_TASK_ID:7;
    vuint32_t _unused_7:25;
  } B;
} DATA_MEMORY_SUBSYSTEM_MSTR_TASK_ID_tag;

typedef union DATA_MEMORY_SUBSYSTEM_QMAN_TASK_IDX_union_tag { /* QMAN Task IDx */
  vuint32_t R;
  struct {
    vuint32_t DDMA_TASK_ID:7;
    vuint32_t _unused_7:25;
  } B;
} DATA_MEMORY_SUBSYSTEM_QMAN_TASK_IDX_tag;

struct DATA_MEMORY_SUBSYSTEM_tag {
  DATA_MEMORY_SUBSYSTEM_MSS_DMBA_tag MSS_DMBA; /* IDM Base Address */
  DATA_MEMORY_SUBSYSTEM_MSS_DMBE_tag MSS_DMBE; /* DMSS Idle Indication */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved0[20];
  DATA_MEMORY_SUBSYSTEM_MSS_HDCFG_tag MSS_HDCFG; /* DMSS Hardware Configuration */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved1[4];
  DATA_MEMORY_SUBSYSTEM_MSS_GPOUT_tag MSS_GPOUT; /* MSS General Purpose 2 */
  DATA_MEMORY_SUBSYSTEM_MSS_DACC_tag MSS_DACC; /* DDMA Access Configuration */
  DATA_MEMORY_SUBSYSTEM_MSS_SDCFG_tag MSS_SDCFG; /* MSS SW Configuration */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved2[8];
  DATA_MEMORY_SUBSYSTEM_MSS_BARRIER_tag MSS_BARRIER; /* MSS Memory Barrier Control */
  DATA_MEMORY_SUBSYSTEM_D_DMAB_tag D_DMAB; /* DDMA Status */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved3[4];
  DATA_MEMORY_SUBSYSTEM_DBG_WRC0_tag DBG_WRC0; /* EDP Write Response Counter */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved4[16];
  DATA_MEMORY_SUBSYSTEM_MSS_DDTC_tag MSS_DDTC; /* DDMA Linear Transfer Size */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved5[4];
  DATA_MEMORY_SUBSYSTEM_MSS_2DCFG1_tag MSS_2DCFG1; /* DDMA Configuration 1 */
  DATA_MEMORY_SUBSYSTEM_MSS_2DCFG2_tag MSS_2DCFG2; /* DDMA Configuration 2 */
  DATA_MEMORY_SUBSYSTEM_MSS_2DCFG3_tag MSS_2DCFG3; /* DDMA Configuration 3 */
  DATA_MEMORY_SUBSYSTEM_MSS_2DCFG4_tag MSS_2DCFG4; /* DDMA Configuration 4 */
  DATA_MEMORY_SUBSYSTEM_MSS_2DCFG5_tag MSS_2DCFG5; /* DDMA Configuration 5 */
  DATA_MEMORY_SUBSYSTEM_MSS_2DCFG6_tag MSS_2DCFG6; /* DDMA Configuration 6 */
  DATA_MEMORY_SUBSYSTEM_MSS_DDQS_tag MSS_DDQS; /* DDMA Status */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved6[4];
  DATA_MEMORY_SUBSYSTEM_MSS_DDEA_tag MSS_DDEA; /* DDMA Address 1 */
  DATA_MEMORY_SUBSYSTEM_MSS_DDIA_tag MSS_DDIA; /* DDMA Address 2 */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved7[4];
  DATA_MEMORY_SUBSYSTEM_MSS_DDCL_tag MSS_DDCL; /* DDMA Control */
  DATA_MEMORY_SUBSYSTEM_DDMA_PAUSE_tag DDMA_PAUSE; /* DDMA Pause Control */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved8[12];
  DATA_MEMORY_SUBSYSTEM_MSS_DDESC_ID_tag MSS_DDESC_ID; /* DDMA Interrupt Source */
  DATA_MEMORY_SUBSYSTEM_MSS_DDESC_OV_tag MSS_DDESC_OV; /* DDMA Task ID Overwrite Indication */
  DATA_MEMORY_SUBSYSTEM_MSTR_TASK_ID_tag MSTR_TASK_ID; /* DDMA Task (Master) ID */
  uint8_t DATA_MEMORY_SUBSYSTEM_reserved9[4];
  DATA_MEMORY_SUBSYSTEM_QMAN_TASK_IDX_tag QMAN_TASK_IDX[4]; /* QMAN Task IDx */
};


/* ============================================================================
   =============================== Module: DBG ================================
   ============================================================================ */

typedef union DBG_DBG_DDAM_union_tag { /* DDMA Breakpoint Register */
  vuint32_t R;
  struct {
    vuint32_t DMADAM:32;
  } B;
} DBG_DBG_DDAM_tag;

typedef union DBG_DBGACS_union_tag {   /* Error Access Control Register */
  vuint32_t R;
  struct {
    vuint32_t DBG_ACS:16;
    vuint32_t _unused_16:16;
  } B;
} DBG_DBGACS_tag;

typedef union DBG_DBG_DDC_union_tag {  /* DDMA Breakpoint Control Register */
  vuint32_t R;
  struct {
    vuint32_t DMADAMS:3;
    vuint32_t _unused_3:29;
  } B;
} DBG_DBG_DDC_tag;

typedef union DBG_DBG_DDS_union_tag {  /* DDMA Breakpoint Status Register */
  vuint32_t R;
  struct {
    vuint32_t DMADAMST:3;
    vuint32_t _unused_3:29;
  } B;
} DBG_DBG_DDS_tag;

typedef union DBG_CENSIRQ_union_tag {  /* Critical Error Interrupt Requests Register */
  vuint32_t R;
  struct {
    vuint32_t CNSE:1;
    vuint32_t CNSIPV:1;
    vuint32_t CNSILOP:1;
    vuint32_t _unused_3:1;
    vuint32_t CNSIRE:1;
    vuint32_t _unused_5:3;
    vuint32_t IN_PRIV:1;
    vuint32_t OUT_PRIV:1;
    vuint32_t LD_PRIV_ACC:1;
    vuint32_t ST_PRIV_ACC:1;
    vuint32_t LD_BLANK_ACC:1;
    vuint32_t ST_BLANK_ACC:1;
    vuint32_t LD_RG_CROS:1;
    vuint32_t ST_RG_CROS:1;
    vuint32_t UNMAPPED_EXCPTN:1;
    vuint32_t STACK_VIOL_EXCPTN:1;
    vuint32_t EXC_E:1;
    vuint32_t ER_EXOK:1;
    vuint32_t LD_IDM_CROS:1;
    vuint32_t ST_IDM_CROS:1;
    vuint32_t VLD_ADR_ERR:1;
    vuint32_t VST_ADR_ERR:1;
    vuint32_t HIST_IDM_CROS:1;
    vuint32_t _unused_25:7;
  } B;
} DBG_CENSIRQ_tag;

typedef union DBG_CENSIRQ2_union_tag { /* Critical Errors Interrupt 2 Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t DDMA_ERR_ACC:1;
    vuint32_t QMAN_ERR_ACC:1;
    vuint32_t QMAN_IDM_CROS:1;
    vuint32_t _unused_4:3;
    vuint32_t DDMA_RG_CROS:1;
    vuint32_t DDMA_IDM_CROS:1;
    vuint32_t _unused_9:1;
    vuint32_t ER_IOP:1;
    vuint32_t _unused_11:1;
    vuint32_t ER_EDP:1;
    vuint32_t _unused_13:17;
    vuint32_t D_UCSERR:1;              /* Uncorrectable Data Safety errors */
    vuint32_t P_UCSERR:1;              /* Uncorrectable Data Safety errors */
  } B;
} DBG_CENSIRQ2_tag;

typedef union DBG_CWDOGIRQ_union_tag { /* Critical Error Interrupt Requests Register */
  vuint32_t R;
  struct {
    vuint32_t CNSE:1;
    vuint32_t _unused_1:15;
    vuint32_t WDOG_MAX:1;
    vuint32_t WDOG_MIN:1;
    vuint32_t _unused_18:2;
    vuint32_t ICUWD:1;
    vuint32_t _unused_21:8;
    vuint32_t IOPWDOG_V:1;
    vuint32_t EDPWDOG_V:1;
    vuint32_t EPPWDOG_V:1;
  } B;
} DBG_CWDOGIRQ_tag;

typedef union DBG_NENSIRQ_union_tag {  /* Normal Error Interrupt Requests Register */
  vuint32_t R;
  struct {
    vuint32_t NNSE:1;
    vuint32_t DIV_0_V:1;
    vuint32_t OVRFLW_EXCPTN:1;
    vuint32_t _unused_3:11;
    vuint32_t EDAP_R_OOR:1;
    vuint32_t EDAP_W_OOR:1;
    vuint32_t _unused_16:2;
    vuint32_t EPP_CF_ERR:1;
    vuint32_t _unused_19:11;
    vuint32_t D_CSERR:1;
    vuint32_t P_CSERR:1;
  } B;
} DBG_NENSIRQ_tag;

typedef union DBG_TIMER_IRQ_union_tag { /* Timer Interrupt Source Register */
  vuint32_t R;
  struct {
    vuint32_t TIMER0_IRQ:1;
    vuint32_t TIMER1_IRQ:1;
    vuint32_t TIMER2_IRQ:1;
    vuint32_t TIMER3_IRQ:1;
    vuint32_t _unused_4:28;
  } B;
} DBG_TIMER_IRQ_tag;

typedef union DBG_DMA_IRQ_union_tag {  /* DMA Interrupt Source Register */
  vuint32_t R;
  struct {
    vuint32_t DDMA_IRQ:1;
    vuint32_t PDMA_IRQ:1;
    vuint32_t _unused_2:30;
  } B;
} DBG_DMA_IRQ_tag;

typedef union DBG_POSCINT_union_tag {  /* Posted Core Interrupt Register */
  vuint32_t R;
  struct {
    vuint32_t PCINT0EN:1;
    vuint32_t PCINT1EN:1;
    vuint32_t PCINT2EN:1;
    vuint32_t PCINT3EN:1;
    vuint32_t _unused_4:4;
    vuint32_t PCINT0ST:1;
    vuint32_t PCINT1ST:1;
    vuint32_t PCINT2ST:1;
    vuint32_t PCINT3ST:1;
    vuint32_t _unused_12:20;
  } B;
} DBG_POSCINT_tag;

typedef union DBG_NQBIRQ_union_tag {   /* Normal QMAN and Buffers Interrupt Register */
  vuint32_t R;
  struct {
    vuint32_t NQBIRQ:1;
    vuint32_t QMAN_IRQ_STATUS:1;
    vuint32_t QMAN_QFULL:1;
    vuint32_t _unused_3:5;
    vuint32_t SNOOP_CSR0:1;
    vuint32_t SNOOP_CSR1:1;
    vuint32_t _unused_10:22;
  } B;
} DBG_NQBIRQ_tag;

typedef union DBG_NENSIRQ_M_union_tag { /* Normal Error Interrupt Request Mask Register */
  vuint32_t R;
  struct {
    vuint32_t NNSE_M:1;
    vuint32_t DIV_0_V_M:1;
    vuint32_t OVRFLW_EXCPTN_M:1;
    vuint32_t _unused_3:11;
    vuint32_t EDAP_R_OOR_M:1;
    vuint32_t EDAP_W_OOR_M:1;
    vuint32_t _unused_16:2;
    vuint32_t EPP_CF_ERR_M:1;
    vuint32_t _unused_19:13;
  } B;
} DBG_NENSIRQ_M_tag;

typedef union DBG_TIMER_IRQ_M_union_tag { /* Timer Interrupt Mask Register */
  vuint32_t R;
  struct {
    vuint32_t TIMER0_MASK:1;
    vuint32_t TIMER1_MASK:1;
    vuint32_t TIMER2_MASK:1;
    vuint32_t TIMER3_MASK:1;
    vuint32_t _unused_4:28;
  } B;
} DBG_TIMER_IRQ_M_tag;

typedef union DBG_NQBIRQ_M_union_tag { /* Normal QMAN and Buffers Interrupt Mask Register */
  vuint32_t R;
  struct {
    vuint32_t NQBIRQ_M:1;
    vuint32_t QMAN_IRQ_STATUS_M:1;
    vuint32_t QMAN_QFULL_M:1;
    vuint32_t _unused_3:5;
    vuint32_t SNOOP_CSR0_M:1;
    vuint32_t SNOOP_CSR1_M:1;
    vuint32_t _unused_10:22;
  } B;
} DBG_NQBIRQ_M_tag;

typedef union DBG_DBG_STACK_START_union_tag { /* Stack Violation Start Address Register */
  vuint32_t R;
  struct {
    vuint32_t STK_VIOLTN_START_ADD:32;
  } B;
} DBG_DBG_STACK_START_tag;

typedef union DBG_DBG_STACK_END_union_tag { /* Stack Violation End Address Register */
  vuint32_t R;
  struct {
    vuint32_t STACK_VIOLTN_END_ADD:32;
  } B;
} DBG_DBG_STACK_END_tag;

typedef union DBG_DBG_DUNMPD_MSK_union_tag { /* DBG DMSS Unmapped Access Mask Register */
  vuint32_t R;
  struct {
    vuint32_t DEXCPTN_UNMAPD_MASK:32;
  } B;
} DBG_DBG_DUNMPD_MSK_tag;

typedef union DBG_DBG_DUNMPD_union_tag { /* DMSS Unmapped Access Status Register */
  vuint32_t R;
  struct {
    vuint32_t DEXCPTN_UNMAPD:32;
  } B;
} DBG_DBG_DUNMPD_tag;

typedef union DBG_CENSIRQ_S_union_tag { /* Critical Error Interrupt Request Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t CNSIPV_S:1;
    vuint32_t CNSILOP_S:1;
    vuint32_t _unused_3:1;
    vuint32_t CNSIRE_S:1;
    vuint32_t _unused_5:3;
    vuint32_t IN_PRIV_S:1;
    vuint32_t OUT_PRIV_S:1;
    vuint32_t LD_PRIV_ACC_S:1;
    vuint32_t ST_PRIV_ACC_S:1;
    vuint32_t LD_BLANK_ACC_S:1;
    vuint32_t ST_BLANK_ACC_S:1;
    vuint32_t LD_RG_CROS_S:1;
    vuint32_t ST_RG_CROS_S:1;
    vuint32_t _unused_16:1;
    vuint32_t STACK_VIOL_EXCPTN_S:1;
    vuint32_t EXC_E_S:1;
    vuint32_t ER_EXOK_S:1;
    vuint32_t LD_IDM_CROS_S:1;
    vuint32_t ST_IDM_CROS_S:1;
    vuint32_t VLD_ADR_ERR_S:1;
    vuint32_t VST_ADR_ERR_S:1;
    vuint32_t HIST_IDM_CROS_S:1;
    vuint32_t _unused_25:7;
  } B;
} DBG_CENSIRQ_S_tag;

typedef union DBG_CENSIRQ2_S_union_tag { /* Critical Error Interrupt Request 2 Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t DDMA_RG_CROS_S:1;
    vuint32_t DDMA_IDM_CROS_S:1;
    vuint32_t _unused_9:1;
    vuint32_t ER_IOP_S:1;
    vuint32_t ER_EPP_S:1;
    vuint32_t ER_EDP_S:1;
    vuint32_t _unused_13:19;
  } B;
} DBG_CENSIRQ2_S_tag;

typedef union DBG_CWDOGIRQ_S_union_tag { /* Critical Error Interrupt Request Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t WDOG_MAX_S:1;
    vuint32_t WDOG_MIN_S:1;
    vuint32_t _unused_18:2;
    vuint32_t ICUWD_S:1;
    vuint32_t _unused_21:8;
    vuint32_t IOPWDOG_V_S:1;
    vuint32_t EDPWDOG_V_S:1;
    vuint32_t EPPWDOG_V_S:1;
  } B;
} DBG_CWDOGIRQ_S_tag;

typedef union DBG_NENSIRQ_S_union_tag { /* Normal Error Interrupt Requests Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t DIV_0_V_S:1;
    vuint32_t OVRFLW_EXCPTN_S:1;
    vuint32_t _unused_3:11;
    vuint32_t EDAP_R_OOR_S:1;
    vuint32_t EDAP_W_OOR_S:1;
    vuint32_t _unused_16:2;
    vuint32_t EPP_CF_ERR_S:1;
    vuint32_t _unused_19:13;
  } B;
} DBG_NENSIRQ_S_tag;

typedef union DBG_TIMER_IRQ_S_union_tag { /* Timer Interrupt Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t TIMER0_IRQ_S:1;
    vuint32_t TIMER1_IRQ_S:1;
    vuint32_t TIMER2_IRQ_S:1;
    vuint32_t TIMER3_IRQ_S:1;
    vuint32_t _unused_4:28;
  } B;
} DBG_TIMER_IRQ_S_tag;

typedef union DBG_DMA_IRQ_S_union_tag { /* DMA Interrupt Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t DDMA_IRQ_S:1;
    vuint32_t PDMA_IRQ_S:1;
    vuint32_t _unused_2:30;
  } B;
} DBG_DMA_IRQ_S_tag;

typedef union DBG_DBG_DESC_ID_union_tag { /* DBG DESC ID Register */
  vuint32_t R;
  struct {
    vuint32_t Q0_BLANK_ACC:1;
    vuint32_t Q1_BLANK_ACC:1;
    vuint32_t Q2_BLANK_ACC:1;
    vuint32_t Q3_BLANK_ACC:1;
    vuint32_t _unused_4:12;
    vuint32_t MSTR_BLANK_ACC:1;
    vuint32_t _unused_17:15;
  } B;
} DBG_DBG_DESC_ID_tag;

typedef union DBG_DBG_QMAN_ID_union_tag { /* DBG QMAN ID Register */
  vuint32_t R;
  struct {
    vuint32_t Q0_BLANK_ACC:1;
    vuint32_t Q1_BLANK_ACC:1;
    vuint32_t Q2_BLANK_ACC:1;
    vuint32_t Q3_BLANK_ACC:1;
    vuint32_t _unused_4:28;
  } B;
} DBG_DBG_QMAN_ID_tag;

typedef union DBG_XCI_COR_union_tag {  /* MSS Acknowledge Control Register */
  vuint32_t R;
  struct {
    vuint32_t INT_COUNT:4;
    vuint32_t _unused_4:28;
  } B;
} DBG_XCI_COR_tag;

struct DBG_tag {
  DBG_DBG_DDAM_tag DBG_DDAM;           /* DDMA Breakpoint Register */
  DBG_DBGACS_tag DBGACS;               /* Error Access Control Register */
  DBG_DBG_DDC_tag DBG_DDC;             /* DDMA Breakpoint Control Register */
  DBG_DBG_DDS_tag DBG_DDS;             /* DDMA Breakpoint Status Register */
  DBG_CENSIRQ_tag CENSIRQ;             /* Critical Error Interrupt Requests Register */
  DBG_CENSIRQ2_tag CENSIRQ2;           /* Critical Errors Interrupt 2 Register */
  DBG_CWDOGIRQ_tag CWDOGIRQ;           /* Critical Error Interrupt Requests Register */
  DBG_NENSIRQ_tag NENSIRQ;             /* Normal Error Interrupt Requests Register */
  DBG_TIMER_IRQ_tag TIMER_IRQ;         /* Timer Interrupt Source Register */
  DBG_DMA_IRQ_tag DMA_IRQ;             /* DMA Interrupt Source Register */
  DBG_POSCINT_tag POSCINT;             /* Posted Core Interrupt Register */
  uint8_t DBG_reserved0[4];
  DBG_NQBIRQ_tag NQBIRQ;               /* Normal QMAN and Buffers Interrupt Register */
  uint8_t DBG_reserved1[8];
  DBG_NENSIRQ_M_tag NENSIRQ_M;         /* Normal Error Interrupt Request Mask Register */
  DBG_TIMER_IRQ_M_tag TIMER_IRQ_M;     /* Timer Interrupt Mask Register */
  DBG_NQBIRQ_M_tag NQBIRQ_M;           /* Normal QMAN and Buffers Interrupt Mask Register */
  uint8_t DBG_reserved2[8];
  DBG_DBG_STACK_START_tag DBG_STACK_START; /* Stack Violation Start Address Register */
  DBG_DBG_STACK_END_tag DBG_STACK_END; /* Stack Violation End Address Register */
  DBG_DBG_DUNMPD_MSK_tag DBG_DUNMPD_MSK; /* DBG DMSS Unmapped Access Mask Register */
  DBG_DBG_DUNMPD_tag DBG_DUNMPD;       /* DMSS Unmapped Access Status Register */
  DBG_CENSIRQ_S_tag CENSIRQ_S;         /* Critical Error Interrupt Request Shadow Register */
  DBG_CENSIRQ2_S_tag CENSIRQ2_S;       /* Critical Error Interrupt Request 2 Shadow Register */
  DBG_CWDOGIRQ_S_tag CWDOGIRQ_S;       /* Critical Error Interrupt Request Shadow Register */
  DBG_NENSIRQ_S_tag NENSIRQ_S;         /* Normal Error Interrupt Requests Shadow Register */
  DBG_TIMER_IRQ_S_tag TIMER_IRQ_S;     /* Timer Interrupt Shadow Register */
  DBG_DMA_IRQ_S_tag DMA_IRQ_S;         /* DMA Interrupt Shadow Register */
  uint8_t DBG_reserved3[56];
  DBG_DBG_DESC_ID_tag DBG_DESC_ID;     /* DBG DESC ID Register */
  DBG_DBG_QMAN_ID_tag DBG_QMAN_ID;     /* DBG QMAN ID Register */
  uint8_t DBG_reserved4[8];
  DBG_XCI_COR_tag XCI_COR;             /* MSS Acknowledge Control Register */
};


/* ============================================================================
   =============================== Module: DCU ================================
   ============================================================================ */

typedef union DCU_OCM_PADD1_union_tag { /* OCM PADD1 Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_PADD1:32;
  } B;
} DCU_OCM_PADD1_tag;

typedef union DCU_OCM_PADD2_union_tag { /* OCM PADD2 Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_PADD2:32;
  } B;
} DCU_OCM_PADD2_tag;

typedef union DCU_OCM_PCOUNT1_union_tag { /* OCM PCOUNT1 Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_PCOUNT1:16;
    vuint32_t _unused_16:16;
  } B;
} DCU_OCM_PCOUNT1_tag;

typedef union DCU_OCM_PCOUNT2_union_tag { /* OCM PCOUNT2 Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_PCOUNT2:16;
    vuint32_t _unused_16:16;
  } B;
} DCU_OCM_PCOUNT2_tag;

typedef union DCU_CEVAX_A_PC_union_tag { /* CEVAX A PC Register */
  vuint32_t R;
  struct {
    vuint32_t CEVAX_A_PC:32;
  } B;
} DCU_CEVAX_A_PC_tag;

typedef union DCU_OCM_DADD_LOW_union_tag { /* OCM DDD Low Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_DADD_LOW:32;
  } B;
} DCU_OCM_DADD_LOW_tag;

typedef union DCU_OCM_DADD_HIGH_union_tag { /* OCM DADD HIGH Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_DADD_HIGH:32;
  } B;
} DCU_OCM_DADD_HIGH_tag;

typedef union DCU_OCM_ACOUNT_union_tag { /* OCM ACOUNT Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_ACOUNT:16;
    vuint32_t _unused_16:16;
  } B;
} DCU_OCM_ACOUNT_tag;

typedef union DCU_OCM_DCOUNT_union_tag { /* OCM DCOUNT Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_DCOUNT:16;
    vuint32_t _unused_16:16;
  } B;
} DCU_OCM_DCOUNT_tag;

typedef union DCU_OCM_DVM_union_tag {  /* OCM DVM Register */
  vuint32_t R;
  struct {
    vuint32_t OCM_DVM:32;
  } B;
} DCU_OCM_DVM_tag;

typedef union DCU_OCM_CONTROL_union_tag { /* OCM CONTROL Register */
  vuint32_t R;
  struct {
    vuint32_t DEBUG_ENABLE:1;
    vuint32_t BP_DETECT_EN:1;
    vuint32_t CORE_RST:1;
    vuint32_t MSS_RST:1;
    vuint32_t TRAPE_EN:1;
    vuint32_t STOP:1;
    vuint32_t STEP:1;
    vuint32_t PADD_INC:1;
    vuint32_t NIDIS:1;
    vuint32_t CIDIS:1;
    vuint32_t TINIS:1;
    vuint32_t _unused_11:1;
    vuint32_t GP_OUT:4;
    vuint32_t DVM_TYPE:2;
    vuint32_t _unused_18:2;
    vuint32_t RST_STATUS:1;
    vuint32_t AUTO_CLK:1;
    vuint32_t _unused_22:1;
    vuint32_t BOOT_MASK:1;
    vuint32_t APB_ERROR:1;
    vuint32_t _unused_25:7;
  } B;
} DCU_OCM_CONTROL_tag;

typedef union DCU_OCM_SA_BP_EN_union_tag { /* OCM SA BP EN Register */
  vuint32_t R;
  struct {
    vuint32_t SA_EXT:2;
    vuint32_t _unused_2:1;
    vuint32_t DMA_DA_E:1;
    vuint32_t SA_PA:2;
    vuint32_t _unused_6:2;
    vuint32_t SA_DA_RD:1;
    vuint32_t _unused_9:3;
    vuint32_t SA_DA_WR:1;
    vuint32_t CO_A_EN:1;
    vuint32_t CO_D_EN:1;
    vuint32_t _unused_15:2;
    vuint32_t SA_DVM:1;
    vuint32_t SA_COMB_RD:1;
    vuint32_t SA_COMB_WR:1;
    vuint32_t _unused_20:12;
  } B;
} DCU_OCM_SA_BP_EN_tag;

typedef union DCU_MEM_CONFIG_union_tag { /* MEM CONFIG Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t PMEM_TCM_SIZE:4;
    vuint32_t PMEM_CAC_SIZE:4;
    vuint32_t PMEM_CAC_WAY:1;
    vuint32_t _unused_13:3;
    vuint32_t DMEM_TCM_SIZE:4;
    vuint32_t DMEM_CAC_SIZE:4;
    vuint32_t DMSS_BLK:2;
    vuint32_t _unused_26:1;
    vuint32_t DMEM_CAC_WAY:1;
    vuint32_t PECC:2;
    vuint32_t DECC:2;
  } B;
} DCU_MEM_CONFIG_tag;

typedef union DCU_MSS_CONFIG_union_tag { /* MSS CONFIG Register */
  vuint32_t R;
  struct {
    vuint32_t NUM_QMAN:5;
    vuint32_t _unused_5:11;
    vuint32_t NUM_ICU_GRP:4;
    vuint32_t NUM_TIMER:2;
    vuint32_t _unused_22:10;
  } B;
} DCU_MSS_CONFIG_tag;

typedef union DCU_OCM_STATUS_union_tag { /* OCM STATUS Register */
  vuint32_t R;
  struct {
    vuint32_t DBG_MODE:1;
    vuint32_t RST_DUR_DEBUG:1;
    vuint32_t PMEM_WR:1;
    vuint32_t PCACHE_WR:1;
    vuint32_t _unused_4:1;
    vuint32_t CORE_NOT_IDLE:1;
    vuint32_t _unused_6:1;
    vuint32_t PACTIVE_TRANS:1;
    vuint32_t TRAPE_INST:1;
    vuint32_t PMSS_IDLE:1;
    vuint32_t _unused_10:2;
    vuint32_t DACTIVE_TRANS:1;
    vuint32_t DMSS_IDLE_OCEM:1;
    vuint32_t _unused_14:1;
    vuint32_t DMSS_SLAVE_ACT:1;
    vuint32_t _unused_16:2;
    vuint32_t WAIT:1;
    vuint32_t _unused_19:9;
    vuint32_t AUTH_IF:4;
  } B;
} DCU_OCM_STATUS_tag;

typedef union DCU_OCM_SA_BP_ST_union_tag { /* OCM SA BP ST Register */
  vuint32_t R;
  struct {
    vuint32_t SA_EXT_ST:2;
    vuint32_t _unused_2:1;
    vuint32_t DMA_DA_ST:1;
    vuint32_t SA_PA_ST:2;
    vuint32_t _unused_6:2;
    vuint32_t SA_DA_RD_ST:1;
    vuint32_t _unused_9:3;
    vuint32_t SA_DA_WR_ST:1;
    vuint32_t _unused_13:4;
    vuint32_t SA_DVM_ST:1;
    vuint32_t SA_COMB_RD_ST:1;
    vuint32_t SA_COMB_WR_ST:1;
    vuint32_t _unused_20:12;
  } B;
} DCU_OCM_SA_BP_ST_tag;

typedef union DCU_M_CONFIG_union_tag { /* M CONFIG Register */
  vuint32_t R;
  struct {
    vuint32_t M0_WIDTH:3;
    vuint32_t _unused_3:1;
    vuint32_t M1_WIDTH:3;
    vuint32_t _unused_7:1;
    vuint32_t EDP_WIDTH:3;
    vuint32_t _unused_11:1;
    vuint32_t EPP_WIDTH:3;
    vuint32_t _unused_15:17;
  } B;
} DCU_M_CONFIG_tag;

typedef union DCU_S_CONFIG_union_tag { /* S_CONFIG Register */
  vuint32_t R;
  struct {
    vuint32_t S0_WIDTH:3;
    vuint32_t _unused_3:1;
    vuint32_t S1_WIDTH:3;
    vuint32_t _unused_7:1;
    vuint32_t S2_WIDTH:3;
    vuint32_t _unused_11:1;
    vuint32_t EDAP_WIDTH:3;
    vuint32_t _unused_15:17;
  } B;
} DCU_S_CONFIG_tag;

typedef union DCU_DEBUG_CONFIG_union_tag { /* DEBUG CONFIG Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t DVBP:2;
    vuint32_t _unused_3:5;
    vuint32_t PROFILER:2;
    vuint32_t _unused_10:2;
    vuint32_t RTT:2;
    vuint32_t _unused_14:2;
    vuint32_t APB:1;
    vuint32_t _unused_17:15;
  } B;
} DCU_DEBUG_CONFIG_tag;

typedef union DCU_CORE_VERSION_union_tag { /* CORE VERSION Register */
  vuint32_t R;
  struct {
    vuint32_t DSP_RTL_REVISION:4;
    vuint32_t DSP_RTL_VERSION:12;
    vuint32_t DSP_CORE_TYPE:16;
  } B;
} DCU_CORE_VERSION_tag;

typedef union DCU_CORE_ID_union_tag {  /* CORE ID Register */
  vuint32_t R;
  struct {
    vuint32_t CORE_ID:32;
  } B;
} DCU_CORE_ID_tag;

typedef union DCU_CORE_CONFIG_union_tag { /* CORE CONFIG Register */
  vuint32_t R;
  struct {
    vuint32_t SPU_XTEND:1;
    vuint32_t VPU_XTEND:1;
    vuint32_t _unused_2:2;
    vuint32_t BTB_ENTRIES:2;
    vuint32_t _unused_6:1;
    vuint32_t BTB_WAYS:1;
    vuint32_t _unused_8:1;
    vuint32_t SFP:1;
    vuint32_t HPF:1;
    vuint32_t _unused_11:5;
    vuint32_t NUM_VCU:1;
    vuint32_t LVPU:1;
    vuint32_t _unused_18:2;
    vuint32_t VEC_MPY:1;
    vuint32_t VEC_MPY_XTD:1;
    vuint32_t VSP_FP:2;
    vuint32_t VHP_FP:1;
    vuint32_t NL:1;
    vuint32_t BNN:1;
    vuint32_t _unused_27:5;
  } B;
} DCU_CORE_CONFIG_tag;

struct DCU_tag {
  DCU_OCM_PADD1_tag OCM_PADD1;         /* OCM PADD1 Register */
  DCU_OCM_PADD2_tag OCM_PADD2;         /* OCM PADD2 Register */
  uint8_t DCU_reserved0[8];
  DCU_OCM_PCOUNT1_tag OCM_PCOUNT1;     /* OCM PCOUNT1 Register */
  DCU_OCM_PCOUNT2_tag OCM_PCOUNT2;     /* OCM PCOUNT2 Register */
  uint8_t DCU_reserved1[8];
  DCU_CEVAX_A_PC_tag CEVAX_A_PC;       /* CEVAX A PC Register */
  DCU_OCM_DADD_LOW_tag OCM_DADD_LOW;   /* OCM DDD Low Register */
  DCU_OCM_DADD_HIGH_tag OCM_DADD_HIGH; /* OCM DADD HIGH Register */
  uint8_t DCU_reserved2[4];
  DCU_OCM_ACOUNT_tag OCM_ACOUNT;       /* OCM ACOUNT Register */
  DCU_OCM_DCOUNT_tag OCM_DCOUNT;       /* OCM DCOUNT Register */
  DCU_OCM_DVM_tag OCM_DVM;             /* OCM DVM Register */
  uint8_t DCU_reserved3[4];
  DCU_OCM_CONTROL_tag OCM_CONTROL;     /* OCM CONTROL Register */
  DCU_OCM_SA_BP_EN_tag OCM_SA_BP_EN;   /* OCM SA BP EN Register */
  DCU_MEM_CONFIG_tag MEM_CONFIG;       /* MEM CONFIG Register */
  DCU_MSS_CONFIG_tag MSS_CONFIG;       /* MSS CONFIG Register */
  DCU_OCM_STATUS_tag OCM_STATUS;       /* OCM STATUS Register */
  DCU_OCM_SA_BP_ST_tag OCM_SA_BP_ST;   /* OCM SA BP ST Register */
  DCU_M_CONFIG_tag M_CONFIG;           /* M CONFIG Register */
  DCU_S_CONFIG_tag S_CONFIG;           /* S_CONFIG Register */
  DCU_DEBUG_CONFIG_tag DEBUG_CONFIG;   /* DEBUG CONFIG Register */
  DCU_CORE_VERSION_tag CORE_VERSION;   /* CORE VERSION Register */
  DCU_CORE_ID_tag CORE_ID;             /* CORE ID Register */
  DCU_CORE_CONFIG_tag CORE_CONFIG;     /* CORE CONFIG Register */
};


/* ============================================================================
   =============================== Module: DDRC ===============================
   ============================================================================ */

typedef union DDRC_CS_BNDS_CS_BNDS_union_tag { /* Rank 0 Memory Bounds */
  vuint32_t R;
  struct {
    vuint32_t EA:16;                   /* Ending Address */
    vuint32_t SA:16;                   /* Starting Address */
  } B;
} DDRC_CS_BNDS_CS_BNDS_tag;

typedef struct DDRC_CS_BNDS_struct_tag {
  DDRC_CS_BNDS_CS_BNDS_tag CS_BNDS;    /* Rank 0 Memory Bounds */
  uint8_t CS_BNDS_reserved0[4];
} DDRC_CS_BNDS_tag;

typedef union DDRC_REMAP_0A_union_tag { /* Remap Region 0A Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_0_REMAP_ADDR:12;     /* Region 0 Remap Starting Address */
    vuint32_t _unused_12:19;
    vuint32_t REG_0_REMAP_EN:1;        /* Region 0 Remap Enable */
  } B;
} DDRC_REMAP_0A_tag;

typedef union DDRC_REMAP_0B_union_tag { /* Remap Region 0B Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_0_EA:12;             /* Region 0 Ending Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_0_SA:12;             /* Region 0 Starting Address */
    vuint32_t _unused_28:4;
  } B;
} DDRC_REMAP_0B_tag;

typedef union DDRC_REMAP_1A_union_tag { /* Remap Region 1A Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_1_REMAP_ADDR:12;     /* Region 1 Remap Starting Address */
    vuint32_t _unused_12:19;
    vuint32_t REG_1_REMAP_EN:1;        /* Region 1 Remap Enable */
  } B;
} DDRC_REMAP_1A_tag;

typedef union DDRC_REMAP_1B_union_tag { /* Remap Region 1B Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_1_EA:12;             /* Region 1 Ending Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_1_SA:12;             /* Region 1 Starting Address */
    vuint32_t _unused_28:4;
  } B;
} DDRC_REMAP_1B_tag;

typedef union DDRC_REMAP_2A_union_tag { /* Remap Region 2A Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_2_REMAP_ADDR:12;     /* Region 2 Remap Starting Address */
    vuint32_t _unused_12:19;
    vuint32_t REG_2_REMAP_EN:1;        /* Region 2 Remap Enable */
  } B;
} DDRC_REMAP_2A_tag;

typedef union DDRC_REMAP_2B_union_tag { /* Remap Region 2B Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_2_EA:12;             /* Region 2 Ending Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_2_SA:12;             /* Region 2 Starting Address */
    vuint32_t _unused_28:4;
  } B;
} DDRC_REMAP_2B_tag;

typedef union DDRC_REMAP_3A_union_tag { /* Remap Region 3A Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_3_REMAP_ADDR:12;     /* Region 3 Remap Starting Address */
    vuint32_t _unused_12:19;
    vuint32_t REG_3_REMAP_EN:1;        /* Region 3 Remap Enable */
  } B;
} DDRC_REMAP_3A_tag;

typedef union DDRC_REMAP_3B_union_tag { /* Remap Region 3B Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_3_EA:12;             /* Region 3 Ending Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_3_SA:12;             /* Region 3 Starting Address */
    vuint32_t _unused_28:4;
  } B;
} DDRC_REMAP_3B_tag;

typedef union DDRC_DDR_ADDR_DEC_0_union_tag { /* DDRC Address Decode 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t ROW14_OVRD:6;            /* Row 14 Override */
    vuint32_t _unused_8:2;
    vuint32_t ROW15_OVRD:6;            /* Row 15 Override */
    vuint32_t _unused_16:2;
    vuint32_t ROW16_OVRD:6;            /* Row 16 Override */
    vuint32_t _unused_24:2;
    vuint32_t ROW17_OVRD:6;            /* Row 17 Override */
  } B;
} DDRC_DDR_ADDR_DEC_0_tag;

typedef union DDRC_DDR_ADDR_DEC_1_union_tag { /* DDRC Address Decode 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t ROW10_OVRD:6;            /* Row 10 Override */
    vuint32_t _unused_8:2;
    vuint32_t ROW11_OVRD:6;            /* Row 11 Override */
    vuint32_t _unused_16:2;
    vuint32_t ROW12_OVRD:6;            /* Row 12 Override */
    vuint32_t _unused_24:2;
    vuint32_t ROW13_OVRD:6;            /* Row 13 Override */
  } B;
} DDRC_DDR_ADDR_DEC_1_tag;

typedef union DDRC_DDR_ADDR_DEC_2_union_tag { /* DDRC Address Decode 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t ROW6_OVRD:6;             /* Row 6 Override */
    vuint32_t _unused_8:2;
    vuint32_t ROW7_OVRD:6;             /* Row 7 Override */
    vuint32_t _unused_16:2;
    vuint32_t ROW8_OVRD:6;             /* Row 8 Override */
    vuint32_t _unused_24:2;
    vuint32_t ROW9_OVRD:6;             /* Row 9 Override */
  } B;
} DDRC_DDR_ADDR_DEC_2_tag;

typedef union DDRC_DDR_ADDR_DEC_3_union_tag { /* DDRC Address Decode 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t ROW2_OVRD:6;             /* Row 2 Override */
    vuint32_t _unused_8:2;
    vuint32_t ROW3_OVRD:6;             /* Row 3 Override */
    vuint32_t _unused_16:2;
    vuint32_t ROW4_OVRD:6;             /* Row 4 Override */
    vuint32_t _unused_24:2;
    vuint32_t ROW5_OVRD:6;             /* Row 5 Override */
  } B;
} DDRC_DDR_ADDR_DEC_3_tag;

typedef union DDRC_DDR_ADDR_DEC_4_union_tag { /* DDRC Address Decode 4 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t COL9_OVRD:6;             /* Col 9 Override */
    vuint32_t _unused_8:2;
    vuint32_t COL10_OVRD:6;            /* Col 10 Override */
    vuint32_t _unused_16:2;
    vuint32_t ROW0_OVRD:6;             /* Row 0 Override */
    vuint32_t _unused_24:2;
    vuint32_t ROW1_OVRD:6;             /* Row 1 Override */
  } B;
} DDRC_DDR_ADDR_DEC_4_tag;

typedef union DDRC_DDR_ADDR_DEC_5_union_tag { /* DDRC Address Decode 5 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t COL5_OVRD:6;             /* Col 5 Override */
    vuint32_t _unused_8:2;
    vuint32_t COL6_OVRD:6;             /* Col 6 Override */
    vuint32_t _unused_16:2;
    vuint32_t COL7_OVRD:6;             /* Col 7 Override */
    vuint32_t _unused_24:2;
    vuint32_t COL8_OVRD:6;             /* Col 8 Override */
  } B;
} DDRC_DDR_ADDR_DEC_5_tag;

typedef union DDRC_DDR_ADDR_DEC_6_union_tag { /* DDRC Address Decode 6 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t COL1_OVRD:6;             /* Col 1 Override */
    vuint32_t _unused_8:2;
    vuint32_t COL2_OVRD:6;             /* Col 2 Override */
    vuint32_t _unused_16:2;
    vuint32_t COL3_OVRD:6;             /* Col 3 Override */
    vuint32_t _unused_24:2;
    vuint32_t COL4_OVRD:6;             /* Col 4 Override */
  } B;
} DDRC_DDR_ADDR_DEC_6_tag;

typedef union DDRC_DDR_ADDR_DEC_7_union_tag { /* DDRC Address Decode 7 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CID1_OVRD:6;             /* CID 1 Override */
    vuint32_t _unused_8:2;
    vuint32_t BA0_OVRD:6;              /* Bank 0 Override */
    vuint32_t _unused_16:2;
    vuint32_t BA1_OVRD:6;              /* Bank 1 Override */
    vuint32_t _unused_24:2;
    vuint32_t COL0_OVRD:6;             /* Col 0 Override */
  } B;
} DDRC_DDR_ADDR_DEC_7_tag;

typedef union DDRC_DDR_ADDR_DEC_8_union_tag { /* DDRC Address Decode 8 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t BG1_OVRD:6;              /* Bank Group 1 Override */
    vuint32_t _unused_8:2;
    vuint32_t CS0_OVRD:6;              /* Interleaved Rank 0 Override */
    vuint32_t _unused_16:2;
    vuint32_t CS1_OVRD:6;              /* Interleaved Rank 1 Override */
    vuint32_t _unused_24:2;
    vuint32_t CID0_OVRD:6;             /* CID 0 Override */
  } B;
} DDRC_DDR_ADDR_DEC_8_tag;

typedef union DDRC_DDR_ADDR_DEC_9_union_tag { /* DDRC Address Decode 9 */
  vuint32_t R;
  struct {
    vuint32_t ADDR_DEC_OVRD:1;         /* Address Decode Override */
    vuint32_t _unused_1:25;
    vuint32_t BG0_OVRD:6;              /* Bank Group 0 Override */
  } B;
} DDRC_DDR_ADDR_DEC_9_tag;

typedef union DDRC_CS_CONFIG_union_tag { /* Rank 0 Configuration */
  vuint32_t R;
  struct {
    vuint32_t COL_BITS_CS:3;           /* Column Bits */
    vuint32_t _unused_3:1;
    vuint32_t BG_BITS_CS:2;            /* Bank Group Bits */
    vuint32_t _unused_6:2;
    vuint32_t ROW_BITS_CS:3;           /* Row Bits */
    vuint32_t _unused_11:12;
    vuint32_t AP_EN:1;                 /* Auto-Precharge Enable */
    vuint32_t _unused_24:7;
    vuint32_t CS_EN:1;                 /* Rank Enable */
  } B;
} DDRC_CS_CONFIG_tag;

typedef union DDRC_TIMING_CFG_3_union_tag { /* DDR SDRAM Timing Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t EXT_WRTORD:1;            /* Extended Write-To-Read Time */
    vuint32_t EXT_ACTTOACT:1;          /* Extended Activate-To-Activate Time */
    vuint32_t _unused_2:1;
    vuint32_t EXT_FOUR_ACT:1;          /* Extended Four Activate */
    vuint32_t EXT_CKE_PLS:2;           /* Extended MCKE Pulse */
    vuint32_t _unused_6:2;
    vuint32_t EXT_WRREC:2;             /* Extended Write Recovery */
    vuint32_t _unused_10:1;
    vuint32_t EXT_WR_LAT_2:1;          /* Extended Write Latency 2 */
    vuint32_t EXT_CASLAT:3;            /* Extended CAS Latency */
    vuint32_t _unused_15:1;
    vuint32_t EXT_REFREC:6;            /* Extended Refresh Recovery */
    vuint32_t EXT_ACTTORW:2;           /* Extended Activate To Read Or Write Time */
    vuint32_t EXT_ACTTOPRE:3;          /* Extended Activate-To-Precharge Time */
    vuint32_t _unused_27:1;
    vuint32_t EXT_PRETOACT:2;          /* Extended Precharge-To-Activate Time */
    vuint32_t _unused_30:2;
  } B;
} DDRC_TIMING_CFG_3_tag;

typedef union DDRC_TIMING_CFG_0_union_tag { /* DDR SDRAM Timing Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t MRS_CYC:6;               /* MRW Cycle Time */
    vuint32_t _unused_6:6;
    vuint32_t EXT_ACT_PD_EXIT:1;       /* Extended Active Power-Down Exit */
    vuint32_t _unused_13:1;
    vuint32_t EXT_PRE_PD_EXIT:2;       /* Extended Precharge Power-Down Exit */
    vuint32_t PRE_PD_EXIT:4;           /* Precharge Power-Down Exit */
    vuint32_t ACT_PD_EXIT:4;           /* Active Powerdown Exit */
    vuint32_t WWT:2;                   /* Write-To-Write Turnaround To Different Ranks */
    vuint32_t RRT:2;                   /* Read-To-Read Turnaround To Different Ranks */
    vuint32_t WRT:2;                   /* Write-To-Read Turnaround To Different Ranks */
    vuint32_t RWT:2;                   /* Read-To-Write Turnaround To Different Ranks */
  } B;
} DDRC_TIMING_CFG_0_tag;

typedef union DDRC_TIMING_CFG_1_union_tag { /* DDR SDRAM Timing Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t WRTORD:4;                /* Write-To-Read Interval */
    vuint32_t ACTTOACT:4;              /* Activate-To-Activate Interval */
    vuint32_t WRREC:4;                 /* Write Recovery */
    vuint32_t REFREC:4;                /* Refresh Recovery */
    vuint32_t _unused_16:1;
    vuint32_t CASLAT:3;                /* CAS Latency */
    vuint32_t ACTTORW:4;               /* Activate To Read Or Write */
    vuint32_t ACTTOPRE:4;              /* Activate-To-Precharge Time */
    vuint32_t PRETOACT:4;              /* Precharge-To-Activate Time */
  } B;
} DDRC_TIMING_CFG_1_tag;

typedef union DDRC_TIMING_CFG_2_union_tag { /* DDR SDRAM Timing Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t FOUR_ACT:6;              /* Four Activate */
    vuint32_t CKE_PLS:3;               /* MCKE Pulse */
    vuint32_t _unused_9:4;
    vuint32_t RD_TO_PRE:5;             /* Read-To-Precharge Time */
    vuint32_t EXT_WR_LAT:1;            /* Extended Write Latency */
    vuint32_t WR_LAT:4;                /* Write Latency */
    vuint32_t _unused_23:5;
    vuint32_t DERATE_VAL:4;            /* Derate Value */
  } B;
} DDRC_TIMING_CFG_2_tag;

typedef union DDRC_DDR_SDRAM_CFG_union_tag { /* DDR SDRAM Control Configuration */
  vuint32_t R;
  struct {
    vuint32_t BI:1;                    /* Bypass Initialization */
    vuint32_t MEM_HALT:1;              /* DDRC Halt */
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:4;
    vuint32_t BA_INTLV_CTL:7;          /* Rank Interleaving Control */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t DBW:2;                   /* DDR SDRAM Data Bus Width */
    vuint32_t DYN_PWR:1;               /* Dynamic Power Management */
    vuint32_t _unused_22:2;
    vuint32_t SDRAM_TYPE:3;            /* DDR SDRAM Type */
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t SREN:1;                  /* Self-Refresh Enable */
    vuint32_t MEM_EN:1;                /* DDRC Enable */
  } B;
} DDRC_DDR_SDRAM_CFG_tag;

typedef union DDRC_DDR_SDRAM_CFG_2_union_tag { /* DDR SDRAM Control Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t D_INIT:1;                /* DDR SDRAM Data Initialization */
    vuint32_t _unused_5:2;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t NUM_PR:4;                /* Number Of Posted Refreshes */
    vuint32_t _unused_16:5;
    vuint32_t _unused_21:2;
    vuint32_t _unused_23:1;
    vuint32_t MCK_DIS:4;               /* MCK Disable */
    vuint32_t _unused_28:2;
    vuint32_t _unused_30:1;
    vuint32_t FRC_SR:1;                /* Force Self-Refresh */
  } B;
} DDRC_DDR_SDRAM_CFG_2_tag;

typedef union DDRC_DDR_SDRAM_MD_CNTL_union_tag { /* DDR SDRAM Mode Control */
  vuint32_t R;
  struct {
    vuint32_t MD_VALUE:18;             /* Mode Register Value */
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t CKE_CNTL:2;              /* Clock Enable Control */
    vuint32_t SET_PRE:1;               /* Set Precharge */
    vuint32_t SET_REF:1;               /* Set Refresh */
    vuint32_t MD_SEL:4;                /* Mode Register Select */
    vuint32_t CS_SEL:3;                /* Select Rank */
    vuint32_t MD_EN:1;                 /* Mode Enable */
  } B;
} DDRC_DDR_SDRAM_MD_CNTL_tag;

typedef union DDRC_DDR_SDRAM_INTERVAL_union_tag { /* DDR SDRAM Interval Configuration */
  vuint32_t R;
  struct {
    vuint32_t BSTOPRE:14;              /* Precharge Interval */
    vuint32_t _unused_14:2;
    vuint32_t REFINT:16;               /* Refresh Interval */
  } B;
} DDRC_DDR_SDRAM_INTERVAL_tag;

typedef union DDRC_DDR_DATA_INIT_union_tag { /* DDR SDRAM Data Initialization */
  vuint32_t R;
  struct {
    vuint32_t INIT_VALUE:32;           /* Initialization Value */
  } B;
} DDRC_DDR_DATA_INIT_tag;

typedef union DDRC_TIMING_CFG_4_union_tag { /* DDR SDRAM Timing Configuration 4 */
  vuint32_t R;
  struct {
    vuint32_t DLL_LOCK:2;              /* DDR SDRAM DLL Lock Time */
    vuint32_t _unused_2:2;
    vuint32_t EXT_REFINT:1;            /* Extended Refresh Interval */
    vuint32_t _unused_5:3;
    vuint32_t EXT_WWT:2;               /* Extended Write-To-Write Turnaround */
    vuint32_t EXT_RRT:2;               /* Extended Read-To-Read Turnaround */
    vuint32_t EXT_WRT:2;               /* Extended Write-To-Read Turnaround */
    vuint32_t EXT_RWT:2;               /* Extended Read-To-Write Turnaround */
    vuint32_t WWT:4;                   /* Write-To-Write Turnaround For Same Rank */
    vuint32_t RRT:4;                   /* Read-To-Read Turnaround For Same Rank */
    vuint32_t WRT:4;                   /* Write-To-Read Turnaround For Same Rank */
    vuint32_t RWT:4;                   /* Read-To-Write Turnaround For Same Rank */
  } B;
} DDRC_TIMING_CFG_4_tag;

typedef union DDRC_TIMING_CFG_7_union_tag { /* DDR SDRAM Timing Configuration 7 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t _unused_4:4;
    vuint32_t _unused_8:8;
    vuint32_t _unused_16:4;
    vuint32_t CKSRX:4;                 /* Clock After Self-Refresh Exit */
    vuint32_t CKSRE:4;                 /* Clock After Self-Refresh Entry */
    vuint32_t CKE_RST:2;               /* MCKE Reset Time */
    vuint32_t _unused_30:2;
  } B;
} DDRC_TIMING_CFG_7_tag;

typedef union DDRC_DDR_ZQ_CNTL_union_tag { /* DDR SDRAM ZQ Calibration Control */
  vuint32_t R;
  struct {
    vuint32_t ZQCS_INT:4;              /* ZQCS Interval */
    vuint32_t _unused_4:4;
    vuint32_t ZQCS:4;                  /* ZQ Calibration Short Time */
    vuint32_t _unused_12:4;
    vuint32_t ZQOPER:4;                /* ZQ Calibration Operation Time */
    vuint32_t _unused_20:4;
    vuint32_t ZQINIT:4;                /* ZQ Calibration Initialization Time */
    vuint32_t _unused_28:3;
    vuint32_t ZQ_EN:1;                 /* ZQ Calibration Enable */
  } B;
} DDRC_DDR_ZQ_CNTL_tag;

typedef union DDRC_DDR_SR_CNTR_union_tag { /* DDR SDRAM Self-Refresh Counter */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t SR_IT:4;                 /* Self-Refresh Idle Threshold */
    vuint32_t _unused_20:12;
  } B;
} DDRC_DDR_SR_CNTR_tag;

typedef union DDRC_TIMING_CFG_8_union_tag { /* DDR SDRAM Timing Configuration 8 */
  vuint32_t R;
  struct {
    vuint32_t PRE_ALL_REC:6;           /* Precharge All-To-Activate Interval */
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:25;
  } B;
} DDRC_TIMING_CFG_8_tag;

typedef union DDRC_TIMING_CFG_9_union_tag { /* DDR SDRAM timing configuration 9 */
  vuint32_t R;
  struct {
    vuint32_t REFTOREF_PB:10;          /* Refresh-to-refresh interval for per-bank refresh. */
    vuint32_t _unused_10:2;
    vuint32_t _unused_12:4;
    vuint32_t REFREC_PB:10;            /* Refresh Recovery Per-Bank Refresh */
    vuint32_t _unused_26:6;
  } B;
} DDRC_TIMING_CFG_9_tag;

typedef union DDRC_TIMING_CFG_11_union_tag { /* DDR SDRAM Timing Configuration 11 */
  vuint32_t R;
  struct {
    vuint32_t MWWT:4;                  /* Masked Write-To-Write Turnaround (tCCDMW) */
    vuint32_t _unused_4:4;
    vuint32_t PRE_TO_PRE:4;            /* Precharge-To-Precharge Time */
    vuint32_t _unused_12:20;
  } B;
} DDRC_TIMING_CFG_11_tag;

typedef union DDRC_DDR_SDRAM_CFG_3_union_tag { /* DDR SDRAM Control Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t SR_FAST_WK_EN:1;         /* Self Refresh Fast Wakeup Enable */
    vuint32_t _unused_2:1;
    vuint32_t NON_PWR_2:1;             /* Non Power of 2 Enable */
    vuint32_t _unused_4:2;
    vuint32_t _unused_6:1;
    vuint32_t DYN_REF_RATE_EN:1;       /* Dynamic Refresh Rate Enable */
    vuint32_t _unused_8:2;
    vuint32_t _unused_10:1;
    vuint32_t DRAIN_FOR_SR:1;          /* Drain Queues For Self-Refresh */
    vuint32_t DM_CFG:3;                /* Data Mask Configuration */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:2;
    vuint32_t _unused_22:2;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:2;
    vuint32_t _unused_30:1;
    vuint32_t DDRC_RST:1;              /* DDRC Reset */
  } B;
} DDRC_DDR_SDRAM_CFG_3_tag;

typedef union DDRC_DDR_SDRAM_REF_RATE_union_tag { /* DDR Refresh Rate */
  vuint32_t R;
  struct {
    vuint32_t REF_RATE_CS1:8;          /* Refresh Rate Rank 1 */
    vuint32_t REF_RATE_CS0:8;          /* Refresh Rate Rank 0 */
    vuint32_t _unused_16:16;
  } B;
} DDRC_DDR_SDRAM_REF_RATE_tag;

typedef union DDRC_TX_CFG_1_union_tag { /* Transaction Configuration Register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t _unused_4:3;
    vuint32_t TS_DEPTH:5;              /* Transaction Scheduler Depth */
    vuint32_t _unused_12:9;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:10;
  } B;
} DDRC_TX_CFG_1_tag;

typedef union DDRC_FFI_CFG_union_tag { /* Freedom From Interference Configuration */
  vuint32_t R;
  struct {
    vuint32_t GRP_3_DID:4;             /* Group 3 Domain ID */
    vuint32_t BNS_CNT_EN:1;            /* Bonus Arbitration Enable */
    vuint32_t GRP_DEF_EN:1;            /* Group Default Enable */
    vuint32_t GRP_3_AEN:1;             /* Group 3 Arbitration Enable */
    vuint32_t GRP_3_EN:1;              /* Group 3 Enable */
    vuint32_t GRP_2_DID:4;             /* Group 2 Domain ID */
    vuint32_t _unused_12:3;
    vuint32_t GRP_2_EN:1;              /* Group 2 Enable */
    vuint32_t GRP_1_DID:4;             /* Group 1 Domain ID */
    vuint32_t _unused_20:3;
    vuint32_t GRP_1_EN:1;              /* Group 1 Enable */
    vuint32_t GRP_0_DID:4;             /* Group 0 Domain ID */
    vuint32_t _unused_28:3;
    vuint32_t GRP_0_EN:1;              /* Group 0 Enable */
  } B;
} DDRC_FFI_CFG_tag;

typedef union DDRC_FFI_CFG2_union_tag { /* Freedom From Interference Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t INF_DEF_BNS:1;           /* Infinite Default Bonus */
    vuint32_t _unused_1:3;
    vuint32_t BNS_CNT_CNFG:5;          /* Bonus Count Config */
    vuint32_t _unused_9:23;
  } B;
} DDRC_FFI_CFG2_tag;

typedef union DDRC_DDRDSR_2_union_tag { /* DDR SDRAM Debug Status 2 */
  vuint32_t R;
  struct {
    vuint32_t RPD_END:1;               /* Rapid Clear Of Memory End */
    vuint32_t RPD_ST:1;                /* Rapid Clear Of Memory Start */
    vuint32_t PHY_INIT_CMPLT:1;        /* DDR PHY Initialization Complete */
    vuint32_t _unused_3:27;
    vuint32_t NML:1;                   /* No Modified Lines */
    vuint32_t IDLE:1;                  /* Memory controller idle (read only). */
  } B;
} DDRC_DDRDSR_2_tag;

typedef union DDRC_DDR_IP_REV1_union_tag { /* DDRC Revision 1 */
  vuint32_t R;
  struct {
    vuint32_t IP_MN:8;                 /* Minor Revision */
    vuint32_t IP_MJ:8;                 /* Major Revision */
    vuint32_t IP_ID:16;                /* IP Block ID */
  } B;
} DDRC_DDR_IP_REV1_tag;

typedef union DDRC_DDR_MTCR_union_tag { /* DDR SDRAM Memory Test Control */
  vuint32_t R;
  struct {
    vuint32_t MT_STAT:1;               /* Memory Test Status */
    vuint32_t _unused_1:8;
    vuint32_t MT_ADDR_EN:1;            /* Memory Test Address Range Enable */
    vuint32_t _unused_10:6;
    vuint32_t MT_TRNARND:4;            /* Memory Test Turnaround */
    vuint32_t _unused_20:4;
    vuint32_t MT_TYP:2;                /* Memory Test Type */
    vuint32_t _unused_26:5;
    vuint32_t MT_EN:1;                 /* Memory Test Enable */
  } B;
} DDRC_DDR_MTCR_tag;

typedef union DDRC_DDR_MTP_union_tag { /* DDR SDRAM Memory Test Pattern n */
  vuint32_t R;
  struct {
    vuint32_t DDR_PATT:32;             /* DDR SDRAM Pattern */
  } B;
} DDRC_DDR_MTP_tag;

typedef union DDRC_DDR_MT_ST_EXT_ADDR_union_tag { /* DDR SDRAM Memory Test Start Extended Address */
  vuint32_t R;
  struct {
    vuint32_t MT_ST_EXT_ADDR:8;        /* Memory Test Start Extended Address */
    vuint32_t _unused_8:24;
  } B;
} DDRC_DDR_MT_ST_EXT_ADDR_tag;

typedef union DDRC_DDR_MT_ST_ADDR_union_tag { /* DDR SDRAM Memory Test Start Address */
  vuint32_t R;
  struct {
    vuint32_t MT_ST_ADDR:32;           /* Memory Test Start Address */
  } B;
} DDRC_DDR_MT_ST_ADDR_tag;

typedef union DDRC_DDR_MT_END_EXT_ADDR_union_tag { /* DDR SDRAM Memory Test End Extended Address */
  vuint32_t R;
  struct {
    vuint32_t MT_END_EXT_ADDR:8;       /* Memory Test End Extended Address */
    vuint32_t _unused_8:24;
  } B;
} DDRC_DDR_MT_END_EXT_ADDR_tag;

typedef union DDRC_DDR_MT_END_ADDR_union_tag { /* DDR SDRAM Memory Test End Address */
  vuint32_t R;
  struct {
    vuint32_t MT_END_ADDR:32;          /* Memory Test End Address */
  } B;
} DDRC_DDR_MT_END_ADDR_tag;

typedef union DDRC_PMGC0_union_tag {   /* Performance Monitor Global Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:29;
    vuint32_t FCECE:1;                 /* Freeze Counters On Enabled Condition Or Event */
    vuint32_t PMIE:1;                  /* Performance Monitor Interrupt Enable */
    vuint32_t FAC:1;                   /* Freeze All Counters */
  } B;
} DDRC_PMGC0_tag;

typedef union DDRC_PMLCA0_union_tag {  /* Performance Monitor Local Control A0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:26;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA0_tag;

typedef union DDRC_PMLCB0_union_tag {  /* Performance Monitor Local Control B0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB0_tag;

typedef union DDRC_PMC0A_union_tag {   /* PMC 0a */
  vuint32_t R;
  struct {
    vuint32_t PMC0:32;                 /* Counter 0 */
  } B;
} DDRC_PMC0A_tag;

typedef union DDRC_PMC0B_union_tag {   /* PMC 0b */
  vuint32_t R;
  struct {
    vuint32_t PMC0:32;                 /* Counter 0 */
  } B;
} DDRC_PMC0B_tag;

typedef union DDRC_PMLCA1_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA1_tag;

typedef union DDRC_PMLCB1_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB1_tag;

typedef union DDRC_PMC1_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC1:32;                 /* Event Count */
  } B;
} DDRC_PMC1_tag;

typedef union DDRC_PMLCA2_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA2_tag;

typedef union DDRC_PMLCB2_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB2_tag;

typedef union DDRC_PMC2_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC2:32;                 /* Event Count */
  } B;
} DDRC_PMC2_tag;

typedef union DDRC_PMLCA3_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA3_tag;

typedef union DDRC_PMLCB3_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB3_tag;

typedef union DDRC_PMC3_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC3:32;                 /* Event Count */
  } B;
} DDRC_PMC3_tag;

typedef union DDRC_PMLCA4_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA4_tag;

typedef union DDRC_PMLCB4_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB4_tag;

typedef union DDRC_PMC4_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC4:32;                 /* Event Count */
  } B;
} DDRC_PMC4_tag;

typedef union DDRC_PMLCA5_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA5_tag;

typedef union DDRC_PMLCB5_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB5_tag;

typedef union DDRC_PMC5_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC5:32;                 /* Event Count */
  } B;
} DDRC_PMC5_tag;

typedef union DDRC_PMLCA6_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA6_tag;

typedef union DDRC_PMLCB6_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB6_tag;

typedef union DDRC_PMC6_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC6:32;                 /* Event Count */
  } B;
} DDRC_PMC6_tag;

typedef union DDRC_PMLCA7_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA7_tag;

typedef union DDRC_PMLCB7_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB7_tag;

typedef union DDRC_PMC7_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC7:32;                 /* Event Count */
  } B;
} DDRC_PMC7_tag;

typedef union DDRC_PMLCA8_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA8_tag;

typedef union DDRC_PMLCB8_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB8_tag;

typedef union DDRC_PMC8_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC8:32;                 /* Event Count */
  } B;
} DDRC_PMC8_tag;

typedef union DDRC_PMLCA9_union_tag {  /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA9_tag;

typedef union DDRC_PMLCB9_union_tag {  /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB9_tag;

typedef union DDRC_PMC9_union_tag {    /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC9:32;                 /* Event Count */
  } B;
} DDRC_PMC9_tag;

typedef union DDRC_PMLCA10_union_tag { /* Performance Monitor Local Control A */
  vuint32_t R;
  struct {
    vuint32_t BDIST:6;                 /* Burst Distance */
    vuint32_t BGRAN:5;                 /* Burst Granularity */
    vuint32_t BSIZE:5;                 /* Burst Size */
    vuint32_t EVENT:7;                 /* Event Selector */
    vuint32_t _unused_23:3;
    vuint32_t CE:1;                    /* Condition Enable */
    vuint32_t _unused_27:4;
    vuint32_t FC:1;                    /* Freeze Counter */
  } B;
} DDRC_PMLCA10_tag;

typedef union DDRC_PMLCB10_union_tag { /* Performance Monitor Local Control B */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:6;             /* Threshold */
    vuint32_t _unused_6:2;
    vuint32_t TBMULT:3;                /* Threshold And Burstiness Multiplier */
    vuint32_t _unused_11:5;
    vuint32_t TRIGOFFCNTL:2;           /* Trigger-Off Control */
    vuint32_t TRIGONCNTL:2;            /* Trigger-On Control */
    vuint32_t TRIGOFFSEL:4;            /* Trigger-Off Select */
    vuint32_t _unused_24:2;
    vuint32_t TRIGONSEL:4;             /* Trigger-On Select */
    vuint32_t _unused_30:2;
  } B;
} DDRC_PMLCB10_tag;

typedef union DDRC_PMC10_union_tag {   /* Performance Monitor Counter */
  vuint32_t R;
  struct {
    vuint32_t PMC10:32;                /* Event Count */
  } B;
} DDRC_PMC10_tag;

typedef union DDRC_ERR_EN_union_tag {  /* Error Enable */
  vuint32_t R;
  struct {
    vuint32_t WTE_EN:1;                /* Write Tag Error Enable */
    vuint32_t RTE_EN:1;                /* Read Tag Error Enable */
    vuint32_t LKSTP_2_EN:1;            /* Lockstep 2 Enable */
    vuint32_t LKSTP_1_EN:1;            /* Lockstep 1 Enable */
    vuint32_t _unused_4:1;
    vuint32_t PAR_1_EN:1;              /* Parity Enable For Internal Errors */
    vuint32_t ECC_EN_RAM_2:1;          /* ECC Enable For On-Chip RAM 2 */
    vuint32_t ECC_EN_RAM_1:1;          /* ECC Enable For On-Chip RAM 1 */
    vuint32_t CRC_2_EN:1;              /* CRC Enable For Group 2 Registers */
    vuint32_t CRC_1_EN:1;              /* CRC Enable For Group 1 Registers */
    vuint32_t _unused_10:20;
    vuint32_t INLINE_ECC_EN:1;         /* Inline ECC Enable */
    vuint32_t ECC_EN:1;                /* ECC Enable */
  } B;
} DDRC_ERR_EN_tag;

typedef union DDRC_DATA_ERR_INJECT_HI_union_tag { /* Memory Data Path Error Injection Mask High */
  vuint32_t R;
  struct {
    vuint32_t EIMH:32;                 /* Error Injection Mask High Data Path */
  } B;
} DDRC_DATA_ERR_INJECT_HI_tag;

typedef union DDRC_DATA_ERR_INJECT_LO_union_tag { /* Memory Data Path Error Injection Mask Low */
  vuint32_t R;
  struct {
    vuint32_t EIML:32;                 /* Error Injection Mask Low Data Bit */
  } B;
} DDRC_DATA_ERR_INJECT_LO_tag;

typedef union DDRC_ERR_INJECT_union_tag { /* Memory Data Path Error Injection Mask ECC */
  vuint32_t R;
  struct {
    vuint32_t EEIM:8;                  /* ECC Error Injection Mask */
    vuint32_t EIEN:1;                  /* Error Injection Enable */
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:2;
    vuint32_t NUM_ECC_INJ:4;           /* Number Of ECC Errors Injected */
    vuint32_t PIEN:1;                  /* Parity Error Injection Enable */
    vuint32_t INTEIN:1;                /* Internal Error Injection Enable */
    vuint32_t INTIES:3;                /* Internal Error Injection Source */
    vuint32_t ECC_INJ_SRC:2;           /* ECC Injection Source */
    vuint32_t FRC2B:1;                 /* Force 2-Bit Error */
    vuint32_t PAR_INJ_SRC:2;           /* Parity Error Injection Source */
    vuint32_t _unused_26:5;
    vuint32_t ADDR_TEN:1;              /* Address Trigger Enable */
  } B;
} DDRC_ERR_INJECT_tag;

typedef union DDRC_ADDR_ERR_INJ_union_tag { /* Address Error Inject */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} DDRC_ADDR_ERR_INJ_tag;

typedef union DDRC_CAPTURE_EXT_DATA_HI_union_tag { /* Memory Extended Data Path Read Capture High */
  vuint32_t R;
  struct {
    vuint32_t ECEHD:32;                /* Error Capture Extended High Data Path */
  } B;
} DDRC_CAPTURE_EXT_DATA_HI_tag;

typedef union DDRC_CAPTURE_EXT_DATA_LO_union_tag { /* Memory Extended Data Path Read Capture Low */
  vuint32_t R;
  struct {
    vuint32_t ECELD:32;                /* Error Capture Extended Low Data Path */
  } B;
} DDRC_CAPTURE_EXT_DATA_LO_tag;

typedef union DDRC_CAPTURE_DATA_HI_union_tag { /* Memory Data Path Read Capture High */
  vuint32_t R;
  struct {
    vuint32_t ECHD:32;                 /* Error Capture High Data Path */
  } B;
} DDRC_CAPTURE_DATA_HI_tag;

typedef union DDRC_CAPTURE_DATA_LO_union_tag { /* Memory Data Path Read Capture Low */
  vuint32_t R;
  struct {
    vuint32_t ECLD:32;                 /* Error Capture Low Data Path */
  } B;
} DDRC_CAPTURE_DATA_LO_tag;

typedef union DDRC_CAPTURE_ECC_union_tag { /* Memory Data Path Read Capture ECC */
  vuint32_t R;
  struct {
    vuint32_t ECE:32;                  /* Error Capture ECC */
  } B;
} DDRC_CAPTURE_ECC_tag;

typedef union DDRC_ERR_DETECT_union_tag { /* Memory Error Detect */
  vuint32_t R;
  struct {
    vuint32_t MSE:1;                   /* Memory-Select Error */
    vuint32_t _unused_1:1;
    vuint32_t SBE:1;                   /* Single-Bit ECC Errors */
    vuint32_t MBE:1;                   /* Multiple-Bit Error */
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t REFRATEE:1;              /* Refresh rate error. */
    vuint32_t LKSTP2E:1;               /* Lockstep 2 error. */
    vuint32_t LKSTP1E:1;               /* Lockstep 1 error. */
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:2;
    vuint32_t PHYE:1;                  /* PHY error. */
    vuint32_t _unused_17:2;
    vuint32_t IPE:1;                   /* Internal Parity Error */
    vuint32_t UPDTMTE:1;               /* Update Timeout Error */
    vuint32_t CRCE:1;                  /* Configuration CRC Error */
    vuint32_t SMBE2:1;                 /* SRAM Multi-Bit Error 2 */
    vuint32_t SMBE1:1;                 /* SRAM Multi-Bit Error 1 */
    vuint32_t SSBE2:1;                 /* SRAM SBE 2 */
    vuint32_t SSBE1:1;                 /* SRAM SBE 1 */
    vuint32_t WTAGE:1;                 /* Write Tag Error */
    vuint32_t RTAGE:1;                 /* Read Tag Error */
    vuint32_t WTTE:1;                  /* Write Tag Timeout Error */
    vuint32_t RTTE:1;                  /* Read Tag Timeout Error */
    vuint32_t RTMTE:1;                 /* Read Timeout Error */
    vuint32_t MME:1;                   /* Multiple Memory Errors */
  } B;
} DDRC_ERR_DETECT_tag;

typedef union DDRC_ERR_DISABLE_union_tag { /* Memory Error Disable */
  vuint32_t R;
  struct {
    vuint32_t MSED:1;                  /* Memory-Select Error Disable */
    vuint32_t _unused_1:1;
    vuint32_t SBED:1;                  /* Single-Bit ECC Error Disable */
    vuint32_t MBED:1;                  /* Multiple-Bit ECC Error Disable */
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:2;
    vuint32_t REFRATEED:1;             /* Refresh Rate Error Disable */
    vuint32_t _unused_8:2;
    vuint32_t _unused_10:2;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:3;
    vuint32_t PHYED:1;                 /* PHY Error Disable */
    vuint32_t _unused_17:3;
    vuint32_t UPDTMTED:1;              /* Update Timeout Error Disable */
    vuint32_t _unused_21:11;
  } B;
} DDRC_ERR_DISABLE_tag;

typedef union DDRC_ERR_INT_EN_union_tag { /* Memory Error Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t MSEE:1;                  /* Memory-Select Error Interrupt Enable */
    vuint32_t _unused_1:1;
    vuint32_t SBEE:1;                  /* Single-Bit ECC Error Interrupt Enable */
    vuint32_t MBEE:1;                  /* Multiple-Bit ECC Error Interrupt Enable */
    vuint32_t SSBE12E:1;               /* SRAM Single-Bit Error Interrupt Enable */
    vuint32_t _unused_5:2;
    vuint32_t REFRATEEE:1;             /* Refresh Rate Interrupt Enable */
    vuint32_t _unused_8:2;
    vuint32_t _unused_10:2;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:3;
    vuint32_t PHYEE:1;                 /* PHY error interrupt enable. */
    vuint32_t _unused_17:3;
    vuint32_t UPDTMTEE:1;              /* Update Timeout Interrupt Enable */
    vuint32_t _unused_21:11;
  } B;
} DDRC_ERR_INT_EN_tag;

typedef union DDRC_CAPTURE_ATTRIBUTES_union_tag { /* Memory Error Attributes Capture */
  vuint32_t R;
  struct {
    vuint32_t VLD:1;                   /* Valid */
    vuint32_t _unused_1:3;
    vuint32_t _unused_4:4;
    vuint32_t _unused_8:4;
    vuint32_t TTYP:2;                  /* Error Transaction Type */
    vuint32_t _unused_14:2;
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:3;
    vuint32_t _unused_27:1;
    vuint32_t BNUM:3;                  /* Data Beat Number */
    vuint32_t _unused_31:1;
  } B;
} DDRC_CAPTURE_ATTRIBUTES_tag;

typedef union DDRC_CAPTURE_ADDRESS_union_tag { /* Memory Error Address Capture */
  vuint32_t R;
  struct {
    vuint32_t CADDR:32;                /* Captured Address */
  } B;
} DDRC_CAPTURE_ADDRESS_tag;

typedef union DDRC_CAPTURE_EXT_ADDRESS_union_tag { /* Memory Error Extended Address Capture */
  vuint32_t R;
  struct {
    vuint32_t CEADDR:8;                /* Captured Extended Address */
    vuint32_t _unused_8:24;
  } B;
} DDRC_CAPTURE_EXT_ADDRESS_tag;

typedef union DDRC_ERR_SBE_union_tag { /* Single-Bit ECC Memory Error Management */
  vuint32_t R;
  struct {
    vuint32_t SBEC:8;                  /* SBE Counter */
    vuint32_t _unused_8:8;
    vuint32_t SBET:8;                  /* SBE Threshold */
    vuint32_t _unused_24:8;
  } B;
} DDRC_ERR_SBE_tag;

typedef union DDRC_REG_LKSTP_CNTL_union_tag { /* Lockstep Register Control */
  vuint32_t R;
  struct {
    vuint32_t CLR_LKSTP2:1;            /* Clear Lockstep 2 Error */
    vuint32_t CLR_LKSTP1:1;            /* Clear Lockstep 1 Error */
    vuint32_t _unused_2:30;
  } B;
} DDRC_REG_LKSTP_CNTL_tag;

typedef union DDRC_REG_CRC_GRP_1_union_tag { /* Register CRC Code For Group 1 */
  vuint32_t R;
  struct {
    vuint32_t CRC_1:32;                /* Programmed CRC Code */
  } B;
} DDRC_REG_CRC_GRP_1_tag;

typedef union DDRC_REG_CRC_GRP_2_union_tag { /* Register CRC Code For Group 2 */
  vuint32_t R;
  struct {
    vuint32_t CRC_2:32;                /* Programmed CRC Code */
  } B;
} DDRC_REG_CRC_GRP_2_tag;

typedef union DDRC_ECC_REG_0_union_tag { /* ECC Region 0 Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_0_EA:12;             /* Region 0 End Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_0_SA:12;             /* Region 0 Start Address */
    vuint32_t _unused_28:3;
    vuint32_t REG_0_EN:1;              /* Region 0 Enable */
  } B;
} DDRC_ECC_REG_0_tag;

typedef union DDRC_ECC_REG_1_union_tag { /* ECC Region 1 Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_1_EA:12;             /* Region 1 End Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_1_SA:12;             /* Region 1 Start Address */
    vuint32_t _unused_28:3;
    vuint32_t REG_1_EN:1;              /* Region 1 Enable */
  } B;
} DDRC_ECC_REG_1_tag;

typedef union DDRC_ECC_REG_2_union_tag { /* ECC Region 2 Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_2_EA:12;             /* Region 2 End Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_2_SA:12;             /* Region 2 Start Address */
    vuint32_t _unused_28:3;
    vuint32_t REG_2_EN:1;              /* Region 2 Enable */
  } B;
} DDRC_ECC_REG_2_tag;

typedef union DDRC_ECC_REG_3_union_tag { /* ECC Region 3 Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_3_EA:12;             /* Region 3 End Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_3_SA:12;             /* Region 3 Start Address */
    vuint32_t _unused_28:3;
    vuint32_t REG_3_EN:1;              /* Region 3 Enable */
  } B;
} DDRC_ECC_REG_3_tag;

typedef union DDRC_ECC_REG_4_union_tag { /* ECC Region 4 Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_4_EA:12;             /* Region 4 End Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_4_SA:12;             /* Region 4 Start Address */
    vuint32_t _unused_28:3;
    vuint32_t REG_4_EN:1;              /* Region 4 Enable */
  } B;
} DDRC_ECC_REG_4_tag;

typedef union DDRC_ECC_REG_5_union_tag { /* ECC Region 5 Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_5_EA:12;             /* Region 5 End Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_5_SA:12;             /* Region 5 Start Address */
    vuint32_t _unused_28:3;
    vuint32_t REG_5_EN:1;              /* Region 5 Enable */
  } B;
} DDRC_ECC_REG_5_tag;

typedef union DDRC_ECC_REG_6_union_tag { /* ECC Region 6 Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_6_EA:12;             /* Region 6 End Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_6_SA:12;             /* Region 6 Start Address */
    vuint32_t _unused_28:3;
    vuint32_t REG_6_EN:1;              /* Region 6 Enable */
  } B;
} DDRC_ECC_REG_6_tag;

typedef union DDRC_ECC_REG_7_union_tag { /* ECC Region 7 Configuration */
  vuint32_t R;
  struct {
    vuint32_t REG_7_EA:12;             /* Region 7 End Address */
    vuint32_t _unused_12:4;
    vuint32_t REG_7_SA:12;             /* Region 7 Start Address */
    vuint32_t _unused_28:3;
    vuint32_t REG_7_EN:1;              /* Region 7 Enable */
  } B;
} DDRC_ECC_REG_7_tag;

struct DDRC_tag {
  DDRC_CS_BNDS_tag CS_BNDS[2];
  uint8_t DDRC_reserved0[16];
  DDRC_REMAP_0A_tag REMAP_0A;          /* Remap Region 0A Configuration */
  DDRC_REMAP_0B_tag REMAP_0B;          /* Remap Region 0B Configuration */
  DDRC_REMAP_1A_tag REMAP_1A;          /* Remap Region 1A Configuration */
  DDRC_REMAP_1B_tag REMAP_1B;          /* Remap Region 1B Configuration */
  DDRC_REMAP_2A_tag REMAP_2A;          /* Remap Region 2A Configuration */
  DDRC_REMAP_2B_tag REMAP_2B;          /* Remap Region 2B Configuration */
  DDRC_REMAP_3A_tag REMAP_3A;          /* Remap Region 3A Configuration */
  DDRC_REMAP_3B_tag REMAP_3B;          /* Remap Region 3B Configuration */
  DDRC_DDR_ADDR_DEC_0_tag DDR_ADDR_DEC_0; /* DDRC Address Decode 0 */
  DDRC_DDR_ADDR_DEC_1_tag DDR_ADDR_DEC_1; /* DDRC Address Decode 1 */
  DDRC_DDR_ADDR_DEC_2_tag DDR_ADDR_DEC_2; /* DDRC Address Decode 2 */
  DDRC_DDR_ADDR_DEC_3_tag DDR_ADDR_DEC_3; /* DDRC Address Decode 3 */
  DDRC_DDR_ADDR_DEC_4_tag DDR_ADDR_DEC_4; /* DDRC Address Decode 4 */
  DDRC_DDR_ADDR_DEC_5_tag DDR_ADDR_DEC_5; /* DDRC Address Decode 5 */
  DDRC_DDR_ADDR_DEC_6_tag DDR_ADDR_DEC_6; /* DDRC Address Decode 6 */
  DDRC_DDR_ADDR_DEC_7_tag DDR_ADDR_DEC_7; /* DDRC Address Decode 7 */
  DDRC_DDR_ADDR_DEC_8_tag DDR_ADDR_DEC_8; /* DDRC Address Decode 8 */
  DDRC_DDR_ADDR_DEC_9_tag DDR_ADDR_DEC_9; /* DDRC Address Decode 9 */
  uint8_t DDRC_reserved1[24];
  DDRC_CS_CONFIG_tag CS_CONFIG[2];     /* Rank 0 Configuration */
  uint8_t DDRC_reserved2[120];
  DDRC_TIMING_CFG_3_tag TIMING_CFG_3;  /* DDR SDRAM Timing Configuration 3 */
  DDRC_TIMING_CFG_0_tag TIMING_CFG_0;  /* DDR SDRAM Timing Configuration 0 */
  DDRC_TIMING_CFG_1_tag TIMING_CFG_1;  /* DDR SDRAM Timing Configuration 1 */
  DDRC_TIMING_CFG_2_tag TIMING_CFG_2;  /* DDR SDRAM Timing Configuration 2 */
  DDRC_DDR_SDRAM_CFG_tag DDR_SDRAM_CFG; /* DDR SDRAM Control Configuration */
  DDRC_DDR_SDRAM_CFG_2_tag DDR_SDRAM_CFG_2; /* DDR SDRAM Control Configuration 2 */
  uint8_t DDRC_reserved3[8];
  DDRC_DDR_SDRAM_MD_CNTL_tag DDR_SDRAM_MD_CNTL; /* DDR SDRAM Mode Control */
  DDRC_DDR_SDRAM_INTERVAL_tag DDR_SDRAM_INTERVAL; /* DDR SDRAM Interval Configuration */
  DDRC_DDR_DATA_INIT_tag DDR_DATA_INIT; /* DDR SDRAM Data Initialization */
  uint8_t DDRC_reserved4[52];
  DDRC_TIMING_CFG_4_tag TIMING_CFG_4;  /* DDR SDRAM Timing Configuration 4 */
  uint8_t DDRC_reserved5[8];
  DDRC_TIMING_CFG_7_tag TIMING_CFG_7;  /* DDR SDRAM Timing Configuration 7 */
  DDRC_DDR_ZQ_CNTL_tag DDR_ZQ_CNTL;    /* DDR SDRAM ZQ Calibration Control */
  uint8_t DDRC_reserved6[8];
  DDRC_DDR_SR_CNTR_tag DDR_SR_CNTR;    /* DDR SDRAM Self-Refresh Counter */
  uint8_t DDRC_reserved7[208];
  DDRC_TIMING_CFG_8_tag TIMING_CFG_8;  /* DDR SDRAM Timing Configuration 8 */
  DDRC_TIMING_CFG_9_tag TIMING_CFG_9;  /* DDR SDRAM timing configuration 9 */
  uint8_t DDRC_reserved8[4];
  DDRC_TIMING_CFG_11_tag TIMING_CFG_11; /* DDR SDRAM Timing Configuration 11 */
  DDRC_DDR_SDRAM_CFG_3_tag DDR_SDRAM_CFG_3; /* DDR SDRAM Control Configuration 3 */
  uint8_t DDRC_reserved9[92];
  DDRC_DDR_SDRAM_REF_RATE_tag DDR_SDRAM_REF_RATE; /* DDR Refresh Rate */
  uint8_t DDRC_reserved10[1340];
  DDRC_TX_CFG_1_tag TX_CFG_1;          /* Transaction Configuration Register 1 */
  uint8_t DDRC_reserved11[28];
  DDRC_FFI_CFG_tag FFI_CFG;            /* Freedom From Interference Configuration */
  DDRC_FFI_CFG2_tag FFI_CFG2;          /* Freedom From Interference Configuration 2 */
  uint8_t DDRC_reserved12[764];
  DDRC_DDRDSR_2_tag DDRDSR_2;          /* DDR SDRAM Debug Status 2 */
  uint8_t DDRC_reserved13[208];
  DDRC_DDR_IP_REV1_tag DDR_IP_REV1;    /* DDRC Revision 1 */
  uint8_t DDRC_reserved14[260];
  DDRC_DDR_MTCR_tag DDR_MTCR;          /* DDR SDRAM Memory Test Control */
  uint8_t DDRC_reserved15[28];
  DDRC_DDR_MTP_tag DDR_MTP[10];        /* DDR SDRAM Memory Test Pattern n */
  uint8_t DDRC_reserved16[24];
  DDRC_DDR_MT_ST_EXT_ADDR_tag DDR_MT_ST_EXT_ADDR; /* DDR SDRAM Memory Test Start Extended Address */
  DDRC_DDR_MT_ST_ADDR_tag DDR_MT_ST_ADDR; /* DDR SDRAM Memory Test Start Address */
  DDRC_DDR_MT_END_EXT_ADDR_tag DDR_MT_END_EXT_ADDR; /* DDR SDRAM Memory Test End Extended Address */
  DDRC_DDR_MT_END_ADDR_tag DDR_MT_END_ADDR; /* DDR SDRAM Memory Test End Address */
  uint8_t DDRC_reserved17[144];
  DDRC_PMGC0_tag PMGC0;                /* Performance Monitor Global Control */
  uint8_t DDRC_reserved18[12];
  DDRC_PMLCA0_tag PMLCA0;              /* Performance Monitor Local Control A0 */
  DDRC_PMLCB0_tag PMLCB0;              /* Performance Monitor Local Control B0 */
  DDRC_PMC0A_tag PMC0A;                /* PMC 0a */
  DDRC_PMC0B_tag PMC0B;                /* PMC 0b */
  DDRC_PMLCA1_tag PMLCA1;              /* Performance Monitor Local Control A */
  DDRC_PMLCB1_tag PMLCB1;              /* Performance Monitor Local Control B */
  DDRC_PMC1_tag PMC1;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved19[4];
  DDRC_PMLCA2_tag PMLCA2;              /* Performance Monitor Local Control A */
  DDRC_PMLCB2_tag PMLCB2;              /* Performance Monitor Local Control B */
  DDRC_PMC2_tag PMC2;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved20[4];
  DDRC_PMLCA3_tag PMLCA3;              /* Performance Monitor Local Control A */
  DDRC_PMLCB3_tag PMLCB3;              /* Performance Monitor Local Control B */
  DDRC_PMC3_tag PMC3;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved21[4];
  DDRC_PMLCA4_tag PMLCA4;              /* Performance Monitor Local Control A */
  DDRC_PMLCB4_tag PMLCB4;              /* Performance Monitor Local Control B */
  DDRC_PMC4_tag PMC4;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved22[4];
  DDRC_PMLCA5_tag PMLCA5;              /* Performance Monitor Local Control A */
  DDRC_PMLCB5_tag PMLCB5;              /* Performance Monitor Local Control B */
  DDRC_PMC5_tag PMC5;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved23[4];
  DDRC_PMLCA6_tag PMLCA6;              /* Performance Monitor Local Control A */
  DDRC_PMLCB6_tag PMLCB6;              /* Performance Monitor Local Control B */
  DDRC_PMC6_tag PMC6;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved24[4];
  DDRC_PMLCA7_tag PMLCA7;              /* Performance Monitor Local Control A */
  DDRC_PMLCB7_tag PMLCB7;              /* Performance Monitor Local Control B */
  DDRC_PMC7_tag PMC7;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved25[4];
  DDRC_PMLCA8_tag PMLCA8;              /* Performance Monitor Local Control A */
  DDRC_PMLCB8_tag PMLCB8;              /* Performance Monitor Local Control B */
  DDRC_PMC8_tag PMC8;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved26[4];
  DDRC_PMLCA9_tag PMLCA9;              /* Performance Monitor Local Control A */
  DDRC_PMLCB9_tag PMLCB9;              /* Performance Monitor Local Control B */
  DDRC_PMC9_tag PMC9;                  /* Performance Monitor Counter */
  uint8_t DDRC_reserved27[4];
  DDRC_PMLCA10_tag PMLCA10;            /* Performance Monitor Local Control A */
  DDRC_PMLCB10_tag PMLCB10;            /* Performance Monitor Local Control B */
  DDRC_PMC10_tag PMC10;                /* Performance Monitor Counter */
  uint8_t DDRC_reserved28[324];
  DDRC_ERR_EN_tag ERR_EN;              /* Error Enable */
  uint8_t DDRC_reserved29[252];
  DDRC_DATA_ERR_INJECT_HI_tag DATA_ERR_INJECT_HI; /* Memory Data Path Error Injection Mask High */
  DDRC_DATA_ERR_INJECT_LO_tag DATA_ERR_INJECT_LO; /* Memory Data Path Error Injection Mask Low */
  DDRC_ERR_INJECT_tag ERR_INJECT;      /* Memory Data Path Error Injection Mask ECC */
  DDRC_ADDR_ERR_INJ_tag ADDR_ERR_INJ;  /* Address Error Inject */
  uint8_t DDRC_reserved30[8];
  DDRC_CAPTURE_EXT_DATA_HI_tag CAPTURE_EXT_DATA_HI; /* Memory Extended Data Path Read Capture High */
  DDRC_CAPTURE_EXT_DATA_LO_tag CAPTURE_EXT_DATA_LO; /* Memory Extended Data Path Read Capture Low */
  DDRC_CAPTURE_DATA_HI_tag CAPTURE_DATA_HI; /* Memory Data Path Read Capture High */
  DDRC_CAPTURE_DATA_LO_tag CAPTURE_DATA_LO; /* Memory Data Path Read Capture Low */
  DDRC_CAPTURE_ECC_tag CAPTURE_ECC;    /* Memory Data Path Read Capture ECC */
  uint8_t DDRC_reserved31[20];
  DDRC_ERR_DETECT_tag ERR_DETECT;      /* Memory Error Detect */
  DDRC_ERR_DISABLE_tag ERR_DISABLE;    /* Memory Error Disable */
  DDRC_ERR_INT_EN_tag ERR_INT_EN;      /* Memory Error Interrupt Enable */
  DDRC_CAPTURE_ATTRIBUTES_tag CAPTURE_ATTRIBUTES; /* Memory Error Attributes Capture */
  DDRC_CAPTURE_ADDRESS_tag CAPTURE_ADDRESS; /* Memory Error Address Capture */
  DDRC_CAPTURE_EXT_ADDRESS_tag CAPTURE_EXT_ADDRESS; /* Memory Error Extended Address Capture */
  DDRC_ERR_SBE_tag ERR_SBE;            /* Single-Bit ECC Memory Error Management */
  uint8_t DDRC_reserved32[164];
  DDRC_REG_LKSTP_CNTL_tag REG_LKSTP_CNTL; /* Lockstep Register Control */
  uint8_t DDRC_reserved33[12];
  DDRC_REG_CRC_GRP_1_tag REG_CRC_GRP_1; /* Register CRC Code For Group 1 */
  DDRC_REG_CRC_GRP_2_tag REG_CRC_GRP_2; /* Register CRC Code For Group 2 */
  uint8_t DDRC_reserved34[40];
  DDRC_ECC_REG_0_tag ECC_REG_0;        /* ECC Region 0 Configuration */
  DDRC_ECC_REG_1_tag ECC_REG_1;        /* ECC Region 1 Configuration */
  DDRC_ECC_REG_2_tag ECC_REG_2;        /* ECC Region 2 Configuration */
  DDRC_ECC_REG_3_tag ECC_REG_3;        /* ECC Region 3 Configuration */
  DDRC_ECC_REG_4_tag ECC_REG_4;        /* ECC Region 4 Configuration */
  DDRC_ECC_REG_5_tag ECC_REG_5;        /* ECC Region 5 Configuration */
  DDRC_ECC_REG_6_tag ECC_REG_6;        /* ECC Region 6 Configuration */
  DDRC_ECC_REG_7_tag ECC_REG_7;        /* ECC Region 7 Configuration */
};


/* ============================================================================
   =============================== Module: DDR_PLL ============================
   ============================================================================ */

typedef union DDR_PLL_PLLCR_union_tag { /* PLL Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;            /* Reserved */
    vuint32_t PLLPD:1;                 /* PLL Power Down */
  } B;
} DDR_PLL_PLLCR_tag;

typedef union DDR_PLL_PLLSR_union_tag { /* PLL Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t LOCK:1;                  /* Lock Status */
    vuint32_t LOL:1;                   /* Loss-Of-Lock Flag */
    vuint32_t _unused_4:4;
    vuint32_t _unused_8:3;
    vuint32_t _unused_11:21;
  } B;
} DDR_PLL_PLLSR_tag;

typedef union DDR_PLL_PLLDV_union_tag { /* PLL Divider */
  vuint32_t R;
  struct {
    vuint32_t MFI:8;                   /* Integer Portion Of Loop Divider */
    vuint32_t _unused_8:4;
    vuint32_t RDIV:3;                  /* Input Clock Predivider */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:6;
    vuint32_t _unused_22:3;
    vuint32_t _unused_25:6;
    vuint32_t _unused_31:1;
  } B;
} DDR_PLL_PLLDV_tag;

typedef union DDR_PLL_PLLFM_union_tag { /* PLL Frequency Modulation */
  vuint32_t R;
  struct {
    vuint32_t STEPNO:11;               /* Number Of Steps Of Modulation Period Or Frequency Modulation */
    vuint32_t _unused_11:5;
    vuint32_t STEPSIZE:10;             /* Frequency Modulation Step Size */
    vuint32_t _unused_26:3;
    vuint32_t SPREADCTL:1;             /* Modulation Type Selection */
    vuint32_t SSCGBYP:1;               /* Frequency Modulation (Spread Spectrum Clock Generation) Bypass */
    vuint32_t _unused_31:1;
  } B;
} DDR_PLL_PLLFM_tag;

typedef union DDR_PLL_PLLFD_union_tag { /* PLL Fractional Divider */
  vuint32_t R;
  struct {
    vuint32_t MFN:15;                  /* Numerator Of Fractional Loop Division Factor */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:4;
    vuint32_t _unused_22:6;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t SDMEN:1;                 /* Fractional Mode Enable */
    vuint32_t _unused_31:1;
  } B;
} DDR_PLL_PLLFD_tag;

typedef union DDR_PLL_PLLCLKMUX_union_tag { /* PLL Clock Multiplexer */
  vuint32_t R;
  struct {
    vuint32_t REFCLKSEL:1;             /* Reference Clock Select */
    vuint32_t _unused_1:31;
  } B;
} DDR_PLL_PLLCLKMUX_tag;

typedef union DDR_PLL_PLLODIV_union_tag { /* PLL Output Divider */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t _unused_2:14;
    vuint32_t DIV:8;                   /* Division Value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider Enable */
  } B;
} DDR_PLL_PLLODIV_tag;

struct DDR_PLL_tag {
  DDR_PLL_PLLCR_tag PLLCR;             /* PLL Control */
  DDR_PLL_PLLSR_tag PLLSR;             /* PLL Status */
  DDR_PLL_PLLDV_tag PLLDV;             /* PLL Divider */
  DDR_PLL_PLLFM_tag PLLFM;             /* PLL Frequency Modulation */
  DDR_PLL_PLLFD_tag PLLFD;             /* PLL Fractional Divider */
  uint8_t DDR_PLL_reserved0[12];
  DDR_PLL_PLLCLKMUX_tag PLLCLKMUX;     /* PLL Clock Multiplexer */
  uint8_t DDR_PLL_reserved1[92];
  DDR_PLL_PLLODIV_tag PLLODIV[1];      /* PLL Output Divider */
};


/* ============================================================================
   =============================== Module: DFS ================================
   ============================================================================ */

typedef union DFS_PORTSR_union_tag {   /* Port Status Register */
  vuint32_t R;
  struct {
    vuint32_t PORTSTAT0:1;             /* Lock status for port 0 */
    vuint32_t PORTSTAT1:1;             /* Lock status for port 1 */
    vuint32_t PORTSTAT2:1;             /* Lock status for port 2 */
    vuint32_t PORTSTAT3:1;             /* Lock status for port 3 */
    vuint32_t PORTSTAT4:1;             /* Lock status for port 4 */
    vuint32_t PORTSTAT5:1;             /* Lock status for port 5 */
    vuint32_t _unused_6:26;
  } B;
} DFS_PORTSR_tag;

typedef union DFS_PORTLOLSR_union_tag { /* Port Loss of Lock Status */
  vuint32_t R;
  struct {
    vuint32_t LOLF0:1;                 /* Loss of lock flag */
    vuint32_t LOLF1:1;                 /* Loss of lock flag */
    vuint32_t LOLF2:1;                 /* Loss of lock flag */
    vuint32_t LOLF3:1;                 /* Loss of lock flag */
    vuint32_t LOLF4:1;                 /* Loss of lock flag */
    vuint32_t LOLF5:1;                 /* Loss of lock flag */
    vuint32_t _unused_6:26;
  } B;
} DFS_PORTLOLSR_tag;

typedef union DFS_PORTRESET_union_tag { /* Port Reset */
  vuint32_t R;
  struct {
    vuint32_t RESET0:1;                /* Enable control for port n */
    vuint32_t RESET1:1;                /* Enable control for port n */
    vuint32_t RESET2:1;                /* Enable control for port n */
    vuint32_t RESET3:1;                /* Enable control for port n */
    vuint32_t RESET4:1;                /* Enable control for port n */
    vuint32_t RESET5:1;                /* Enable control for port n */
    vuint32_t _unused_6:26;
  } B;
} DFS_PORTRESET_tag;

typedef union DFS_CTL_union_tag {      /* Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t DFS_RESET:1;             /* Reset of DFS */
    vuint32_t _unused_2:30;
  } B;
} DFS_CTL_tag;

typedef union DFS_DVPORT_union_tag {   /* Divider for Port 0 */
  vuint32_t R;
  struct {
    vuint32_t MFN:6;                   /* Numerator of fractional part of division value */
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t MFI:8;                   /* Integer part of division value */
    vuint32_t _unused_16:16;
  } B;
} DFS_DVPORT_tag;

struct DFS_tag {
  uint8_t DFS_reserved0[12];
  DFS_PORTSR_tag PORTSR;               /* Port Status Register */
  DFS_PORTLOLSR_tag PORTLOLSR;         /* Port Loss of Lock Status */
  DFS_PORTRESET_tag PORTRESET;         /* Port Reset */
  DFS_CTL_tag CTL;                     /* Control */
  DFS_DVPORT_tag DVPORT[6];            /* Divider for Port 0 */
};


/* ============================================================================
   =============================== Module: DIPORTSD ===========================
   ============================================================================ */

typedef union DIPORTSD_ECR_union_tag { /* Error and Control Register */
  vuint32_t R;
  struct {
    vuint32_t NCRC_ERR_EN:1;           /* Normal Mode CRC Error Enable */
    vuint32_t MFRM_ERR_EN:1;           /* Message Frame Error Enable */
    vuint32_t MID_ERR_EN:1;            /* Message ID Error Enable */
    vuint32_t MPRC_ERR_EN:1;           /* Message Processing Error Enable */
    vuint32_t MADD_ERR_EN:1;           /* Message Address Error Enable */
    vuint32_t RSP_ERR_EN:1;            /* Response Error Enable */
    vuint32_t _unused_6:25;
    vuint32_t REG_LOCK:1;              /* Register Lock */
  } B;
} DIPORTSD_ECR_tag;

typedef union DIPORTSD_SESR_union_tag { /* Error Status Register */
  vuint32_t R;
  struct {
    vuint32_t NCRC_ERR:1;              /* Normal Mode CRC Error Status */
    vuint32_t MFRM_ERR:1;              /* Message Frame Error Status */
    vuint32_t MID_ERR:1;               /* Message ID Error Status */
    vuint32_t MPRC_ERR:1;              /* Message Processing Error Status */
    vuint32_t MADD_ERR:1;              /* Message Address Error Status */
    vuint32_t RSP_ERR:1;               /* Response Error Status */
    vuint32_t _unused_6:26;
  } B;
} DIPORTSD_SESR_tag;

typedef union DIPORTSD_QOS_union_tag { /* Quality Of Service */
  vuint32_t R;
  struct {
    vuint32_t SQOS:2;                  /* Quality of Service */
    vuint32_t _unused_2:6;
    vuint32_t AABW:5;                  /* Active AXI Bandwidth */
    vuint32_t _unused_13:19;
  } B;
} DIPORTSD_QOS_tag;

typedef union DIPORTSD_SIGEN_union_tag { /* Signaling Enable */
  vuint32_t R;
  struct {
    vuint32_t SIGEN:8;                 /* Signaling Enable */
    vuint32_t _unused_8:24;
  } B;
} DIPORTSD_SIGEN_tag;

typedef union DIPORTSD_RSRV0_union_tag { /* Reserved 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;             /* Reserved. Do not change field value. */
    vuint32_t _unused_3:29;
  } B;
} DIPORTSD_RSRV0_tag;

typedef union DIPORTSD_LAR0_union_tag { /* Logical Address Region 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Logical Address for Expansion Region 0 */
  } B;
} DIPORTSD_LAR0_tag;

typedef union DIPORTSD_PAR0_union_tag { /* Physical Address Region 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Physical Address for Expansion Region 0 */
  } B;
} DIPORTSD_PAR0_tag;

typedef union DIPORTSD_RS0_union_tag { /* Region Size 0 */
  vuint32_t R;
  struct {
    vuint32_t BLOCK_SIZE:5;            /* Block Size of the Expansion Address Region 0 */
    vuint32_t _unused_5:27;
  } B;
} DIPORTSD_RS0_tag;

typedef union DIPORTSD_LAR1_union_tag { /* Logical Address Region 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Logical Address for Expansion Region 1 */
  } B;
} DIPORTSD_LAR1_tag;

typedef union DIPORTSD_PAR1_union_tag { /* Physical Address Region 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Physical Address for Expansion Region 1 */
  } B;
} DIPORTSD_PAR1_tag;

typedef union DIPORTSD_RS1_union_tag { /* Region Size 1 */
  vuint32_t R;
  struct {
    vuint32_t BLOCK_SIZE:5;            /* Block Size of the Expansion Address Region 1 */
    vuint32_t _unused_5:27;
  } B;
} DIPORTSD_RS1_tag;

typedef union DIPORTSD_LAR2_union_tag { /* Logical Address Region 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Logical Address for Expansion Region 2 */
  } B;
} DIPORTSD_LAR2_tag;

typedef union DIPORTSD_PAR2_union_tag { /* Physical Address Region 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Physical Address for Expansion Region 2 */
  } B;
} DIPORTSD_PAR2_tag;

typedef union DIPORTSD_RS2_union_tag { /* Region Size 2 */
  vuint32_t R;
  struct {
    vuint32_t BLOCK_SIZE:5;            /* Block Size of the Expansion Address Region 2 */
    vuint32_t _unused_5:27;
  } B;
} DIPORTSD_RS2_tag;

typedef union DIPORTSD_LAR3_union_tag { /* Logical Address Region 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Logical Address for Expansion Region 3 */
  } B;
} DIPORTSD_LAR3_tag;

typedef union DIPORTSD_PAR3_union_tag { /* Physical Address Region 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Physical Address for Expansion Region 3 */
  } B;
} DIPORTSD_PAR3_tag;

typedef union DIPORTSD_RS3_union_tag { /* Region Size 3 */
  vuint32_t R;
  struct {
    vuint32_t BLOCK_SIZE:5;            /* Block Size of the Expansion Address Region 3 */
    vuint32_t _unused_5:27;
  } B;
} DIPORTSD_RS3_tag;

typedef union DIPORTSD_LAR4_union_tag { /* Logical Address Region 4 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Logical Address for Expansion Region 4 */
  } B;
} DIPORTSD_LAR4_tag;

typedef union DIPORTSD_PAR4_union_tag { /* Physical Address Region 4 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Physical Address for Expansion Region 4 */
  } B;
} DIPORTSD_PAR4_tag;

typedef union DIPORTSD_RS4_union_tag { /* Region Size 4 */
  vuint32_t R;
  struct {
    vuint32_t BLOCK_SIZE:5;            /* Block Size of the Expansion Address Region 4 */
    vuint32_t _unused_5:27;
  } B;
} DIPORTSD_RS4_tag;

typedef union DIPORTSD_LAR5_union_tag { /* Logical Address Region 5 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Logical Address for Expansion Region 5 */
  } B;
} DIPORTSD_LAR5_tag;

typedef union DIPORTSD_PAR5_union_tag { /* Physical Address Region 5 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Physical Address for Expansion Region 5 */
  } B;
} DIPORTSD_PAR5_tag;

typedef union DIPORTSD_RS5_union_tag { /* Region Size 5 */
  vuint32_t R;
  struct {
    vuint32_t BLOCK_SIZE:5;            /* Block Size of the Expansion Address Region 5 */
    vuint32_t _unused_5:27;
  } B;
} DIPORTSD_RS5_tag;

typedef union DIPORTSD_LAR6_union_tag { /* Logical Address Region 6 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Logical Address for Expansion Region 6 */
  } B;
} DIPORTSD_LAR6_tag;

typedef union DIPORTSD_PAR6_union_tag { /* Physical Address Region 6 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Physical Address for Expansion Region 6 */
  } B;
} DIPORTSD_PAR6_tag;

typedef union DIPORTSD_RS6_union_tag { /* Region Size 6 */
  vuint32_t R;
  struct {
    vuint32_t BLOCK_SIZE:5;            /* Block Size of the Expansion Address Region 6 */
    vuint32_t _unused_5:27;
  } B;
} DIPORTSD_RS6_tag;

typedef union DIPORTSD_LAR7_union_tag { /* Logical Address Region 7 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Logical Address for Expansion Region 7 */
  } B;
} DIPORTSD_LAR7_tag;

typedef union DIPORTSD_PAR7_union_tag { /* Physical Address Region 7 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t START_ADDR:20;           /* Starting Physical Address for Expansion Region 7 */
  } B;
} DIPORTSD_PAR7_tag;

typedef union DIPORTSD_RS7_union_tag { /* Region Size 7 */
  vuint32_t R;
  struct {
    vuint32_t BLOCK_SIZE:5;            /* Block Size of the Expansion Address Region 7 */
    vuint32_t _unused_5:27;
  } B;
} DIPORTSD_RS7_tag;

typedef union DIPORTSD_RSRV1_union_tag { /* Reserved 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved. Do not change field value. */
    vuint32_t _unused_8:11;
    vuint32_t _unused_19:5;            /* Reserved. Do not change field value. */
    vuint32_t _unused_24:6;
    vuint32_t _unused_30:2;            /* Reserved. Do not change field value. */
  } B;
} DIPORTSD_RSRV1_tag;

typedef union DIPORTSD_RSRV2_union_tag { /* Reserved 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} DIPORTSD_RSRV2_tag;

typedef union DIPORTSD_RSRV3_union_tag { /* Reserved 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} DIPORTSD_RSRV3_tag;

typedef union DIPORTSD_RSRV4_union_tag { /* Reserved 4 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} DIPORTSD_RSRV4_tag;

typedef union DIPORTSD_RSRV5_union_tag { /* Reserved 5 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} DIPORTSD_RSRV5_tag;

struct DIPORTSD_tag {
  uint8_t DIPORTSD_reserved0[256];
  DIPORTSD_ECR_tag ECR;                /* Error and Control Register */
  DIPORTSD_SESR_tag SESR;              /* Error Status Register */
  DIPORTSD_QOS_tag QOS;                /* Quality Of Service */
  DIPORTSD_SIGEN_tag SIGEN;            /* Signaling Enable */
  uint8_t DIPORTSD_reserved1[8];
  DIPORTSD_RSRV0_tag RSRV0;            /* Reserved 0 */
  uint8_t DIPORTSD_reserved2[12];
  DIPORTSD_LAR0_tag LAR0;              /* Logical Address Region 0 */
  DIPORTSD_PAR0_tag PAR0;              /* Physical Address Region 0 */
  DIPORTSD_RS0_tag RS0;                /* Region Size 0 */
  DIPORTSD_LAR1_tag LAR1;              /* Logical Address Region 1 */
  DIPORTSD_PAR1_tag PAR1;              /* Physical Address Region 1 */
  DIPORTSD_RS1_tag RS1;                /* Region Size 1 */
  DIPORTSD_LAR2_tag LAR2;              /* Logical Address Region 2 */
  DIPORTSD_PAR2_tag PAR2;              /* Physical Address Region 2 */
  DIPORTSD_RS2_tag RS2;                /* Region Size 2 */
  DIPORTSD_LAR3_tag LAR3;              /* Logical Address Region 3 */
  DIPORTSD_PAR3_tag PAR3;              /* Physical Address Region 3 */
  DIPORTSD_RS3_tag RS3;                /* Region Size 3 */
  DIPORTSD_LAR4_tag LAR4;              /* Logical Address Region 4 */
  DIPORTSD_PAR4_tag PAR4;              /* Physical Address Region 4 */
  DIPORTSD_RS4_tag RS4;                /* Region Size 4 */
  DIPORTSD_LAR5_tag LAR5;              /* Logical Address Region 5 */
  DIPORTSD_PAR5_tag PAR5;              /* Physical Address Region 5 */
  DIPORTSD_RS5_tag RS5;                /* Region Size 5 */
  DIPORTSD_LAR6_tag LAR6;              /* Logical Address Region 6 */
  DIPORTSD_PAR6_tag PAR6;              /* Physical Address Region 6 */
  DIPORTSD_RS6_tag RS6;                /* Region Size 6 */
  DIPORTSD_LAR7_tag LAR7;              /* Logical Address Region 7 */
  DIPORTSD_PAR7_tag PAR7;              /* Physical Address Region 7 */
  DIPORTSD_RS7_tag RS7;                /* Region Size 7 */
  uint8_t DIPORTSD_reserved3[120];
  DIPORTSD_RSRV1_tag RSRV1;            /* Reserved 1 */
  DIPORTSD_RSRV2_tag RSRV2;            /* Reserved 2 */
  DIPORTSD_RSRV3_tag RSRV3;            /* Reserved 3 */
  DIPORTSD_RSRV4_tag RSRV4;            /* Reserved 4 */
  DIPORTSD_RSRV5_tag RSRV5;            /* Reserved 5 */
};


/* ============================================================================
   =============================== Module: DMAMUX =============================
   ============================================================================ */

typedef union DMAMUX_CHCONF_union_tag { /* Channel Configuration */
  vuint8_t R;
  struct {
    vuint8_t SOURCE:6;                 /* DMA Channel Source (Slot) */
    vuint8_t TRIG:1;                   /* DMA Channel Trigger Enable */
    vuint8_t ENBL:1;                   /* DMA Channel Enable */
  } B;
} DMAMUX_CHCONF_tag;

struct DMAMUX_tag {
  DMAMUX_CHCONF_tag CHCONF[16];        /* Channel Configuration */
};


/* ============================================================================
   =============================== Module: DMA_CRC ============================
   ============================================================================ */

typedef union DMA_CRC_GEC_union_tag {  /* Global Enable CRC Register */
  vuint32_t R;
  struct {
    vuint32_t GBL_EN:1;                /* Global Enable bit */
    vuint32_t _unused_1:6;             /* Reserved */
    vuint32_t SWAP_BYTE:1;             /* Swap Byte */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} DMA_CRC_GEC_tag;

typedef union DMA_CRC_CONTROL_REGISTER_CTL_union_tag { /* CRC Control Register */
  vuint32_t R;
  struct {
    vuint32_t CH_SEL:5;                /* Channel Select, merged from more bitFields with different positions or sizes */
    vuint32_t _unused_5:3;
    vuint32_t POLY_SEL:3;              /* Polynomial Select */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t CS_XOR:1;
    vuint32_t CS_SWAP:1;
    vuint32_t SWAP_BIT:1;              /* Swap Bit */
    vuint32_t INIT_SEL:1;              /* Initial values of the CRC */
    vuint32_t MODE:3;                  /* CRC Mode */
    vuint32_t _unused_19:12;           /* Reserved */
    vuint32_t EN:1;                    /* CRC Logic */
  } B;
} DMA_CRC_CONTROL_REGISTER_CTL_tag;

typedef union DMA_CRC_CONTROL_REGISTER_ICRC_union_tag { /* Initial CRC Value Register */
  vuint32_t R;
  struct {
    vuint32_t INI_CRC_VAL:32;          /* Initial CRC Value */
  } B;
} DMA_CRC_CONTROL_REGISTER_ICRC_tag;

typedef union DMA_CRC_CONTROL_REGISTER_FCRC_union_tag { /* Final CRC Value Register */
  vuint32_t R;
  struct {
    vuint32_t CHKSUM_VAL:32;           /* Final CRC Value */
  } B;
} DMA_CRC_CONTROL_REGISTER_FCRC_tag;

typedef struct DMA_CRC_CONTROL_REGISTER_struct_tag {
  DMA_CRC_CONTROL_REGISTER_CTL_tag CTL; /* CRC Control Register */
  DMA_CRC_CONTROL_REGISTER_ICRC_tag ICRC; /* Initial CRC Value Register */
  DMA_CRC_CONTROL_REGISTER_FCRC_tag FCRC; /* Final CRC Value Register */
  uint8_t CONTROL_REGISTER_reserved0[4];
} DMA_CRC_CONTROL_REGISTER_tag;

struct DMA_CRC_tag {
  DMA_CRC_GEC_tag GEC;                 /* Global Enable CRC Register */
  uint8_t DMA_CRC_reserved0[12];
  DMA_CRC_CONTROL_REGISTER_tag CONTROL_REGISTER[8];
};


/* ============================================================================
   =============================== Module: DMSS_SAFETY ========================
   ============================================================================ */

typedef union DMSS_SAFETY_D_UCSERR_union_tag { /* DMSS UCS ERR Register */
  vuint32_t R;
  struct {
    vuint32_t D_UCSERR:1;
    vuint32_t D_ECC_ERR:1;
    vuint32_t _unused_2:7;
    vuint32_t D_MSS_PERR:1;
    vuint32_t _unused_10:6;
    vuint32_t D_UCECNTEV:1;
    vuint32_t D_UCECNTOV:1;
    vuint32_t _unused_18:12;
    vuint32_t CWDOGIRQ:1;
    vuint32_t CENSIRQ:1;
  } B;
} DMSS_SAFETY_D_UCSERR_tag;

typedef union DMSS_SAFETY_D_CSERR_union_tag { /* DMSS CS ERR Register */
  vuint32_t R;
  struct {
    vuint32_t D_CSERR:1;
    vuint32_t D_ECC_COR:1;
    vuint32_t _unused_2:14;
    vuint32_t D_CECNTEV:1;
    vuint32_t D_CECNTOV:1;
    vuint32_t _unused_18:13;
    vuint32_t NNSE:1;
  } B;
} DMSS_SAFETY_D_CSERR_tag;

typedef union DMSS_SAFETY_D_UCSERR_S_union_tag { /* DMSS UCS ERR Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t D_ECC_ERR_S:1;
    vuint32_t _unused_2:7;
    vuint32_t D_MSS_PERR_S:1;
    vuint32_t _unused_10:22;
  } B;
} DMSS_SAFETY_D_UCSERR_S_tag;

typedef union DMSS_SAFETY_D_CSERR_S_union_tag { /* DMSS CS ERR Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t D_ECC_COR_S:1;
    vuint32_t _unused_2:30;
  } B;
} DMSS_SAFETY_D_CSERR_S_tag;

typedef union DMSS_SAFETY_D_CSERR_M_union_tag { /* DMSS CS ERR Mask Register */
  vuint32_t R;
  struct {
    vuint32_t D_CSRERR_M:1;
    vuint32_t D_ECC_COR_M:1;
    vuint32_t _unused_2:30;
  } B;
} DMSS_SAFETY_D_CSERR_M_tag;

typedef union DMSS_SAFETY_D_ECC_TCFG_union_tag { /* DMSS ECC TCFG Register */
  vuint32_t R;
  struct {
    vuint32_t D_FLIPCFG:32;
  } B;
} DMSS_SAFETY_D_ECC_TCFG_tag;

typedef union DMSS_SAFETY_D_ECC_TCFG_2_union_tag { /* DMSS ECC TCFG 2 Register */
  vuint32_t R;
  struct {
    vuint32_t D_FLIPCFGE:7;
    vuint32_t _unused_7:24;
    vuint32_t D_TRIG:1;
  } B;
} DMSS_SAFETY_D_ECC_TCFG_2_tag;

typedef union DMSS_SAFETY_D_SFT_SCPD_0_union_tag { /* DMSS SFT SCPD 0 Register */
  vuint32_t R;
  struct {
    vuint32_t SFT_SCPD0:32;
  } B;
} DMSS_SAFETY_D_SFT_SCPD_0_tag;

typedef union DMSS_SAFETY_D_SFT_SCPD_1_union_tag { /* DMSS SFT SCPD 1 Register */
  vuint32_t R;
  struct {
    vuint32_t SFT_SCPD1:32;
  } B;
} DMSS_SAFETY_D_SFT_SCPD_1_tag;

typedef union DMSS_SAFETY_D_SFT_SCPD_2_union_tag { /* DMSS SFT SCPD 2 Register */
  vuint32_t R;
  struct {
    vuint32_t SFT_SCPD2:32;
  } B;
} DMSS_SAFETY_D_SFT_SCPD_2_tag;

typedef union DMSS_SAFETY_D_SFT_SCPD_3_union_tag { /* DMSS SFT SCPD 3 Register */
  vuint32_t R;
  struct {
    vuint32_t SFT_SCPD3:32;
  } B;
} DMSS_SAFETY_D_SFT_SCPD_3_tag;

typedef union DMSS_SAFETY_D_UCSERR_CNT_union_tag { /* DMSS UCS ERR Register */
  vuint32_t R;
  struct {
    vuint32_t D_UCSERR_CNT_V:16;
    vuint32_t _unused_16:16;
  } B;
} DMSS_SAFETY_D_UCSERR_CNT_tag;

typedef union DMSS_SAFETY_D_CSERR_CNT_union_tag { /* DMSS CS ERR CNT Register */
  vuint32_t R;
  struct {
    vuint32_t D_CSERR_CNT_V:16;
    vuint32_t _unused_16:16;
  } B;
} DMSS_SAFETY_D_CSERR_CNT_tag;

struct DMSS_SAFETY_tag {
  DMSS_SAFETY_D_UCSERR_tag D_UCSERR;   /* DMSS UCS ERR Register */
  uint8_t DMSS_SAFETY_reserved0[4];
  DMSS_SAFETY_D_CSERR_tag D_CSERR;     /* DMSS CS ERR Register */
  uint8_t DMSS_SAFETY_reserved1[4];
  DMSS_SAFETY_D_UCSERR_S_tag D_UCSERR_S; /* DMSS UCS ERR Shadow Register */
  DMSS_SAFETY_D_CSERR_S_tag D_CSERR_S; /* DMSS CS ERR Shadow Register */
  uint8_t DMSS_SAFETY_reserved2[4];
  DMSS_SAFETY_D_CSERR_M_tag D_CSERR_M; /* DMSS CS ERR Mask Register */
  DMSS_SAFETY_D_ECC_TCFG_tag D_ECC_TCFG; /* DMSS ECC TCFG Register */
  DMSS_SAFETY_D_ECC_TCFG_2_tag D_ECC_TCFG_2; /* DMSS ECC TCFG 2 Register */
  uint8_t DMSS_SAFETY_reserved3[8];
  DMSS_SAFETY_D_SFT_SCPD_0_tag D_SFT_SCPD_0; /* DMSS SFT SCPD 0 Register */
  DMSS_SAFETY_D_SFT_SCPD_1_tag D_SFT_SCPD_1; /* DMSS SFT SCPD 1 Register */
  DMSS_SAFETY_D_SFT_SCPD_2_tag D_SFT_SCPD_2; /* DMSS SFT SCPD 2 Register */
  DMSS_SAFETY_D_SFT_SCPD_3_tag D_SFT_SCPD_3; /* DMSS SFT SCPD 3 Register */
  DMSS_SAFETY_D_UCSERR_CNT_tag D_UCSERR_CNT; /* DMSS UCS ERR Register */
  DMSS_SAFETY_D_CSERR_CNT_tag D_CSERR_CNT; /* DMSS CS ERR CNT Register */
};


/* ============================================================================
   =============================== Module: DSPI_10 ============================
   ============================================================================ */

typedef union DSPI_10_MCR_union_tag {  /* Module Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t HALT:1;                  /* Halt */
    vuint32_t PES:1;                   /* Parity Error Stop */
    vuint32_t FCPCS:1;                 /* Fast Continuous PCS Mode */
    vuint32_t XSPI:1;                  /* Extended SPI Mode */
    vuint32_t _unused_4:4;             /* Reserved */
    vuint32_t SMPL_PT:2;               /* Sample Point */
    vuint32_t CLR_RXF:1;               /* Clear RX FIFO */
    vuint32_t CLR_TXF:1;               /* Clear TX FIFO */
    vuint32_t DIS_RXF:1;               /* Disable Receive FIFO */
    vuint32_t DIS_TXF:1;               /* Disable Transmit FIFO */
    vuint32_t MDIS:1;                  /* Module Disable */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t PCSIS:3;                 /* Peripheral Chip Select x Inactive State */
    vuint32_t _unused_19:5;            /* Always write the reset value to this field. */
    vuint32_t ROOE:1;                  /* Receive FIFO Overflow Overwrite Enable */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t MTFE:1;                  /* Modified Transfer Format Enable */
    vuint32_t FRZ:1;                   /* Freeze */
    vuint32_t DCONF:2;                 /* Interface Configuration */
    vuint32_t CONT_SCKE:1;             /* Continuous SCK Enable */
    vuint32_t MSTR:1;                  /* Master/Slave Mode Select */
  } B;
} DSPI_10_MCR_tag;

typedef union DSPI_10_TCR_union_tag {  /* Transfer Count Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t SPI_TCNT:16;             /* DSPI Transfer Counter */
  } B;
} DSPI_10_TCR_tag;

typedef union DSPI_10_CTAR_union_tag { /* Clock and Transfer Attributes Register (In Master Mode) */
  vuint32_t R;
  struct {
    vuint32_t BR:4;                    /* Baud Rate Scaler */
    vuint32_t DT:4;                    /* Delay After Transfer Scaler */
    vuint32_t ASC:4;                   /* After SCK Delay Scaler */
    vuint32_t CSSCK:4;                 /* PCS to SCK Delay Scaler */
    vuint32_t PBR:2;                   /* Baud Rate Prescaler */
    vuint32_t PDT:2;                   /* Delay after Transfer Prescaler */
    vuint32_t PASC:2;                  /* After SCK Delay Prescaler */
    vuint32_t PCSSCK:2;                /* PCS to SCK Delay Prescaler */
    vuint32_t LSBFE:1;                 /* LSB First */
    vuint32_t CPHA:1;                  /* Clock Phase */
    vuint32_t CPOL:1;                  /* Clock Polarity */
    vuint32_t FMSZ:4;                  /* Frame Size */
    vuint32_t DBR:1;                   /* Double Baud Rate */
  } B;
} DSPI_10_CTAR_tag;

typedef union DSPI_10_SR_union_tag {   /* Status Register */
  vuint32_t R;
  struct {
    vuint32_t POPNXTPTR:4;             /* Pop Next Pointer */
    vuint32_t RXCTR:4;                 /* RX FIFO Counter */
    vuint32_t TXNXTPTR:4;              /* Transmit Next Pointer */
    vuint32_t TXCTR:4;                 /* TX FIFO Counter */
    vuint32_t CMDFFF:1;                /* Command FIFO Fill Flag */
    vuint32_t RFDF:1;                  /* Receive FIFO Drain Flag */
    vuint32_t TFIWF:1;                 /* Transmit FIFO Invalid Write Flag */
    vuint32_t RFOF:1;                  /* Receive FIFO Overflow Flag */
    vuint32_t DDIF:1;                  /* DSI Data Received with Active Fields */
    vuint32_t SPEF:1;                  /* DSPI Parity Error Flag */
    vuint32_t DPEF:1;                  /* DSI Parity Error Flag */
    vuint32_t CMDTCF:1;                /* Command Transfer Complete Flag */
    vuint32_t BSYF:1;                  /* Busy Flag */
    vuint32_t TFFF:1;                  /* Transmit FIFO Fill Flag */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t EOQF:1;                  /* End of Queue Flag */
    vuint32_t SPITCF:1;                /* DSPI Frame Transfer Complete Flag. */
    vuint32_t TXRXS:1;                 /* TX and RX Status */
    vuint32_t TCF:1;                   /* Transfer Complete Flag */
  } B;
} DSPI_10_SR_tag;

typedef union DSPI_10_RSER_union_tag { /* DMA/Interrupt Request Select and Enable Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;            /* Reserved */
    vuint32_t DDIF_DIRS:1;             /* DSI Data Received (with active bits, DMA, or interrupt request select) */
    vuint32_t CMDFFF_DIRS:1;           /* Command FIFO Fill DMA or Interrupt Request Select */
    vuint32_t RFDF_DIRS:1;             /* Receive FIFO Drain DMA or Interrupt Request Select */
    vuint32_t RFDF_RE:1;               /* Receive FIFO Drain Request Enable */
    vuint32_t TFIWF_RE:1;              /* Transmit FIFO Invalid Write Request Enable */
    vuint32_t RFOF_RE:1;               /* Receive FIFO Overflow Request Enable */
    vuint32_t DDIF_RE:1;               /* DSI Data Received (with active bits request enable) */
    vuint32_t SPEF_RE:1;               /* DSPI Parity Error Request Enable */
    vuint32_t DPEF_RE:1;               /* DSI Parity Error Request Enable */
    vuint32_t CMDTCF_RE:1;             /* Command Transmission Complete Request Enable */
    vuint32_t TFFF_DIRS:1;             /* Transmit FIFO Fill DMA or Interrupt Request Select */
    vuint32_t TFFF_RE:1;               /* Transmit FIFO Fill Request Enable */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t EOQF_RE:1;               /* Finished Request Enable */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t CMDFFF_RE:1;             /* Command FIFO Fill Flag Request Enable */
    vuint32_t TCF_RE:1;                /* Transmission Complete Request Enable */
  } B;
} DSPI_10_RSER_tag;

typedef union DSPI_10_PUSHR_union_tag { /* PUSH TX FIFO Register In Master Mode */
  vuint32_t R;
  struct {
    vuint32_t TXDATA:16;               /* Transmit Data */
    vuint32_t PCS:3;                   /* PCS */
    vuint32_t _unused_19:5;            /* Always write the reset value to this field. */
    vuint32_t PP_MCSC:1;               /* Parity Polarity or Mask T CSC delay in the next frame */
    vuint32_t PE_MASC:1;               /* Parity Enable or Mask TASC delay in the current frame */
    vuint32_t CTCNT:1;                 /* Clear Transfer Counter */
    vuint32_t EOQ:1;                   /* End Of Queue */
    vuint32_t CTAS:3;                  /* Clock and Transfer Attributes Select */
    vuint32_t CONT:1;                  /* Continuous Peripheral Chip Select Enable */
  } B;
} DSPI_10_PUSHR_tag;

typedef union DSPI_10_POPR_union_tag { /* POP RX FIFO Register */
  vuint32_t R;
  struct {
    vuint32_t RXDATA:32;               /* Received Data */
  } B;
} DSPI_10_POPR_tag;

typedef union DSPI_10_TXFR_union_tag { /* Transmit FIFO Registers */
  vuint32_t R;
  struct {
    vuint32_t TXDATA:16;               /* Transmit Data */
    vuint32_t TXCMD_TXDATA:16;         /* Transmit Command or Transmit Data */
  } B;
} DSPI_10_TXFR_tag;

typedef union DSPI_10_RXFR_union_tag { /* Receive FIFO Registers */
  vuint32_t R;
  struct {
    vuint32_t RXDATA:32;               /* Receive Data */
  } B;
} DSPI_10_RXFR_tag;

typedef union DSPI_10_DSICR0_union_tag { /* DSI Configuration Register 0 */
  vuint32_t R;
  struct {
    vuint32_t DPCSX:8;                 /* DSI Peripheral Chip Select 0-7 */
    vuint32_t PP:1;                    /* Parity Polarity */
    vuint32_t PE:1;                    /* Parity Enable */
    vuint32_t PES:1;                   /* Parity Error Stop */
    vuint32_t DMS:1;                   /* Data Match Stop */
    vuint32_t DSICTAS:3;               /* DSI Clock and Transfer Attributes Select */
    vuint32_t DCONT:1;                 /* DSI Continuous Peripheral Chip Select Enable */
    vuint32_t CID:1;                   /* Change In Data Transfer Enable */
    vuint32_t _unused_17:2;            /* Reserved */
    vuint32_t TXSS:1;                  /* Transmit Data Source Select */
    vuint32_t TSBC:1;                  /* Timed Serial Bus Configuration. */
    vuint32_t ITSB:1;                  /* Interleaved TSB mode. */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t FMSZ5:1;                 /* MSB of Frame Size in Master Mode (when DSI is used in 64-bit mode) */
    vuint32_t _unused_24:6;            /* Reserved */
    vuint32_t FMSZ4:1;                 /* MSB of Frame Size in Master Mode */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} DSPI_10_DSICR0_tag;

typedef union DSPI_10_SDR0_union_tag { /* DSI Serialization Data Register 0 */
  vuint32_t R;
  struct {
    vuint32_t SER_DATA:32;             /* Serialized Data */
  } B;
} DSPI_10_SDR0_tag;

typedef union DSPI_10_ASDR0_union_tag { /* DSI Alternate Serialization Data Register 0 */
  vuint32_t R;
  struct {
    vuint32_t ASER_DATA:32;            /* Alternate Serialized Data */
  } B;
} DSPI_10_ASDR0_tag;

typedef union DSPI_10_COMPR0_union_tag { /* DSI Transmit Comparison Register 0 */
  vuint32_t R;
  struct {
    vuint32_t COMP_DATA:32;            /* Compare Data */
  } B;
} DSPI_10_COMPR0_tag;

typedef union DSPI_10_DDR0_union_tag { /* DSI Deserialization Data Register 0 */
  vuint32_t R;
  struct {
    vuint32_t DESER_DATA:32;           /* Deserialized Data */
  } B;
} DSPI_10_DDR0_tag;

typedef union DSPI_10_DSICR1_union_tag { /* DSI Configuration Register 1 */
  vuint32_t R;
  struct {
    vuint32_t DPCS1_X:8;               /* DSI Peripheral Chip Select 0-7 */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t DSE0:1;                  /* Data Select Enable 0 */
    vuint32_t DSE1:1;                  /* Data Select Enable 1 */
    vuint32_t DSI64E:1;                /* DSI 64-bit Mode Enable */
    vuint32_t CSE:1;                   /* Command Select Enable */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:3;            /* Reserved */
    vuint32_t TSBCNT:6;                /* Timed Serial Bus Operation Count */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} DSPI_10_DSICR1_tag;

typedef union DSPI_10_SSR0_union_tag { /* DSI Serialization Source Select Register 0 */
  vuint32_t R;
  struct {
    vuint32_t SS:32;                   /* Source Select */
  } B;
} DSPI_10_SSR0_tag;

typedef union DSPI_10_DIMR0_union_tag { /* DSI Deserialized Data Interrupt Mask Register 0 */
  vuint32_t R;
  struct {
    vuint32_t MASK:32;                 /* Mask */
  } B;
} DSPI_10_DIMR0_tag;

typedef union DSPI_10_DPIR0_union_tag { /* DSI Deserialized Data Polarity Interrupt Register 0 */
  vuint32_t R;
  struct {
    vuint32_t DP:32;                   /* Data Polarity */
  } B;
} DSPI_10_DPIR0_tag;

typedef union DSPI_10_SDR1_union_tag { /* DSI Serialization Data Register 1 */
  vuint32_t R;
  struct {
    vuint32_t SER_DATA:32;             /* Serialized Data */
  } B;
} DSPI_10_SDR1_tag;

typedef union DSPI_10_ASDR1_union_tag { /* DSI Alternate Serialization Data Register 1 */
  vuint32_t R;
  struct {
    vuint32_t ASER_DATA:32;            /* Alternate Serialized Data */
  } B;
} DSPI_10_ASDR1_tag;

typedef union DSPI_10_COMPR1_union_tag { /* DSI Transmit Comparison Register 1 */
  vuint32_t R;
  struct {
    vuint32_t COMP_DATA:32;            /* Compare Data */
  } B;
} DSPI_10_COMPR1_tag;

typedef union DSPI_10_DDR1_union_tag { /* DSI Deserialization Data Register 1 */
  vuint32_t R;
  struct {
    vuint32_t DESER_DATA:32;           /* Deserialized Data */
  } B;
} DSPI_10_DDR1_tag;

typedef union DSPI_10_SSR1_union_tag { /* DSI Serialization Source Select Register 1 */
  vuint32_t R;
  struct {
    vuint32_t SS:32;                   /* Source Select */
  } B;
} DSPI_10_SSR1_tag;

typedef union DSPI_10_DIMR1_union_tag { /* DSI Deserialized Data Interrupt Mask Register 1 */
  vuint32_t R;
  struct {
    vuint32_t MASK:32;                 /* Mask */
  } B;
} DSPI_10_DIMR1_tag;

typedef union DSPI_10_DPIR1_union_tag { /* DSI Deserialized Data Polarity Interrupt Register 1 */
  vuint32_t R;
  struct {
    vuint32_t DP:32;                   /* Data Polarity */
  } B;
} DSPI_10_DPIR1_tag;

typedef union DSPI_10_CTARE_union_tag { /* Clock and Transfer Attributes Register Extended */
  vuint32_t R;
  struct {
    vuint32_t DTCP:11;                 /* Data Transfer Count Preload */
    vuint32_t _unused_11:5;            /* Reserved */
    vuint32_t FMSZE:1;                 /* Frame Size Extended */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} DSPI_10_CTARE_tag;

typedef union DSPI_10_SREX_union_tag { /* Status Register Extended */
  vuint32_t R;
  struct {
    vuint32_t CMDNXTPTR:4;             /* Command Next Pointer */
    vuint32_t CMDCTR:5;                /* CMD FIFO Counter */
    vuint32_t _unused_9:2;             /* Reserved */
    vuint32_t RXCTR4:1;                /* RX FIFO Counter[4] */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t TXCTR4:1;                /* TX FIFO Counter[4] */
    vuint32_t _unused_15:17;           /* Reserved */
  } B;
} DSPI_10_SREX_tag;

typedef union DSPI_10_TSL_union_tag {  /* Time Slot Length Register */
  vuint32_t R;
  struct {
    vuint32_t TS0_LEN:7;               /* Time Slot 0 Length */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t TS1_LEN:7;               /* Time Slot 1 Length */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t TS2_LEN:7;               /* Time Slot 2 Length */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t TS3_LEN:7;               /* Time Slot 3 Length */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} DSPI_10_TSL_tag;

typedef union DSPI_10_TS_CONF_union_tag { /* Time Slot Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t TS0:4;                   /* Time Slot 0 */
    vuint32_t TS1:4;                   /* Time Slot 1 */
    vuint32_t TS2:4;                   /* Time Slot 2 */
    vuint32_t TS3:4;                   /* Time Slot 3 */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} DSPI_10_TS_CONF_tag;

struct DSPI_10_tag {
  DSPI_10_MCR_tag MCR;                 /* Module Configuration Register */
  uint8_t DSPI_10_reserved0[4];
  DSPI_10_TCR_tag TCR;                 /* Transfer Count Register */
  DSPI_10_CTAR_tag CTAR[6];            /* Clock and Transfer Attributes Register (In Master Mode) */
  uint8_t DSPI_10_reserved1[8];
  DSPI_10_SR_tag SR;                   /* Status Register */
  DSPI_10_RSER_tag RSER;               /* DMA/Interrupt Request Select and Enable Register */
  DSPI_10_PUSHR_tag PUSHR;             /* PUSH TX FIFO Register In Master Mode */
  DSPI_10_POPR_tag POPR;               /* POP RX FIFO Register */
  DSPI_10_TXFR_tag TXFR[16];           /* Transmit FIFO Registers */
  DSPI_10_RXFR_tag RXFR[16];           /* Receive FIFO Registers */
  DSPI_10_DSICR0_tag DSICR0;           /* DSI Configuration Register 0 */
  DSPI_10_SDR0_tag SDR0;               /* DSI Serialization Data Register 0 */
  DSPI_10_ASDR0_tag ASDR0;             /* DSI Alternate Serialization Data Register 0 */
  DSPI_10_COMPR0_tag COMPR0;           /* DSI Transmit Comparison Register 0 */
  DSPI_10_DDR0_tag DDR0;               /* DSI Deserialization Data Register 0 */
  DSPI_10_DSICR1_tag DSICR1;           /* DSI Configuration Register 1 */
  DSPI_10_SSR0_tag SSR0;               /* DSI Serialization Source Select Register 0 */
  uint8_t DSPI_10_reserved2[16];
  DSPI_10_DIMR0_tag DIMR0;             /* DSI Deserialized Data Interrupt Mask Register 0 */
  DSPI_10_DPIR0_tag DPIR0;             /* DSI Deserialized Data Polarity Interrupt Register 0 */
  DSPI_10_SDR1_tag SDR1;               /* DSI Serialization Data Register 1 */
  DSPI_10_ASDR1_tag ASDR1;             /* DSI Alternate Serialization Data Register 1 */
  DSPI_10_COMPR1_tag COMPR1;           /* DSI Transmit Comparison Register 1 */
  DSPI_10_DDR1_tag DDR1;               /* DSI Deserialization Data Register 1 */
  DSPI_10_SSR1_tag SSR1;               /* DSI Serialization Source Select Register 1 */
  uint8_t DSPI_10_reserved3[16];
  DSPI_10_DIMR1_tag DIMR1;             /* DSI Deserialized Data Interrupt Mask Register 1 */
  DSPI_10_DPIR1_tag DPIR1;             /* DSI Deserialized Data Polarity Interrupt Register 1 */
  DSPI_10_CTARE_tag CTARE[6];          /* Clock and Transfer Attributes Register Extended */
  uint8_t DSPI_10_reserved4[8];
  DSPI_10_SREX_tag SREX;               /* Status Register Extended */
  uint8_t DSPI_10_reserved5[16];
  DSPI_10_TSL_tag TSL;                 /* Time Slot Length Register */
  DSPI_10_TS_CONF_tag TS_CONF;         /* Time Slot Configuration Register */
};


/* ============================================================================
   =============================== Module: EDMA3_MP ===========================
   ============================================================================ */

typedef union EDMA3_MP_CSR_union_tag { /* Management Page Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t EDBG:1;                  /* Enable Debug */
    vuint32_t ERCA:1;                  /* Enable Round Robin Channel Arbitration */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t HAE:1;                   /* Halt After Error */
    vuint32_t HALT:1;                  /* Halt DMA Operations */
    vuint32_t GCLC:1;                  /* Global Channel Linking Control */
    vuint32_t GMRC:1;                  /* Global Master ID Replication Control */
    vuint32_t ECX:1;                   /* Cancel Transfer With Error */
    vuint32_t CX:1;                    /* Cancel Transfer */
    vuint32_t _unused_10:14;           /* Reserved */
    vuint32_t ACTIVE_ID:5;             /* Active Channel ID, merged from more bitFields with different positions or sizes */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t ACTIVE:1;                /* DMA Active Status */
  } B;
} EDMA3_MP_CSR_tag;

typedef union EDMA3_MP_ES_union_tag {  /* Management Page Error Status */
  vuint32_t R;
  struct {
    vuint32_t DBE:1;                   /* Destination Bus Error */
    vuint32_t SBE:1;                   /* Source Bus Error */
    vuint32_t SGE:1;                   /* Scatter/Gather Configuration Error */
    vuint32_t NCE:1;                   /* NBYTES/CITER Configuration Error */
    vuint32_t DOE:1;                   /* Destination Offset Error */
    vuint32_t DAE:1;                   /* Destination Address Error */
    vuint32_t SOE:1;                   /* Source Offset Error */
    vuint32_t SAE:1;                   /* Source Address Error */
    vuint32_t ECX:1;                   /* Transfer Canceled */
    vuint32_t UCE:1;                   /* Uncorrectable TCD Error During Channel Execution */
    vuint32_t _unused_10:14;           /* Reserved */
    vuint32_t ERRCHN:5;                /* Error Channel Number or Canceled Channel Number, merged from more bitFields with different positions or sizes */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} EDMA3_MP_ES_tag;

typedef union EDMA3_MP_INT_union_tag { /* Management Page Interrupt Request Status */
  vuint32_t R;
  struct {
    vuint32_t INT:32;                  /* Interrupt Request Status, merged from more bitFields with different positions or sizes */
  } B;
} EDMA3_MP_INT_tag;

typedef union EDMA3_MP_HRS_union_tag { /* Management Page Hardware Request Status */
  vuint32_t R;
  struct {
    vuint32_t HRS:32;                  /* Hardware Request Status */
  } B;
} EDMA3_MP_HRS_tag;

typedef union EDMA3_MP_CH_GRPRI_union_tag { /* Channel Arbitration Group */
  vuint32_t R;
  struct {
    vuint32_t GRPRI:5;                 /* Arbitration Group For Channel n */
    vuint32_t _unused_5:27;
  } B;
} EDMA3_MP_CH_GRPRI_tag;

struct EDMA3_MP_tag {
  EDMA3_MP_CSR_tag CSR;                /* Management Page Control */
  EDMA3_MP_ES_tag ES;                  /* Management Page Error Status */
  EDMA3_MP_INT_tag INT;                /* Management Page Interrupt Request Status */
  EDMA3_MP_HRS_tag HRS;                /* Management Page Hardware Request Status */
  uint8_t EDMA3_MP_reserved0[240];
  EDMA3_MP_CH_GRPRI_tag CH_GRPRI[32];  /* Channel Arbitration Group */
};


/* ============================================================================
   =============================== Module: EDMA3_TCD ==========================
   ============================================================================ */

typedef union EDMA3_TCD_TCD_CH_CSR_union_tag { /* Channel Control and Status */
  vuint32_t R;
  struct {
    vuint32_t ERQ:1;                   /* Enable DMA Request */
    vuint32_t EARQ:1;                  /* Enable Asynchronous DMA Request In Stop Mode For Channel */
    vuint32_t EEI:1;                   /* Enable Error Interrupt */
    vuint32_t EBW:1;                   /* Enable Buffered Writes */
    vuint32_t _unused_4:26;            /* Reserved */
    vuint32_t DONE:1;                  /* Channel Done */
    vuint32_t ACTIVE:1;                /* Channel Active */
  } B;
} EDMA3_TCD_TCD_CH_CSR_tag;

typedef union EDMA3_TCD_TCD_CH_ES_union_tag { /* Channel Error Status */
  vuint32_t R;
  struct {
    vuint32_t DBE:1;                   /* Destination Bus Error */
    vuint32_t SBE:1;                   /* Source Bus Error */
    vuint32_t SGE:1;                   /* Scatter/Gather Configuration Error */
    vuint32_t NCE:1;                   /* NBYTES/CITER Configuration Error */
    vuint32_t DOE:1;                   /* Destination Offset Error */
    vuint32_t DAE:1;                   /* Destination Address Error */
    vuint32_t SOE:1;                   /* Source Offset Error */
    vuint32_t SAE:1;                   /* Source Address Error */
    vuint32_t _unused_8:23;            /* Reserved */
    vuint32_t ERR:1;                   /* Error In Channel */
  } B;
} EDMA3_TCD_TCD_CH_ES_tag;

typedef union EDMA3_TCD_TCD_CH_INT_union_tag { /* Channel Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t INT:1;                   /* Interrupt Request */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} EDMA3_TCD_TCD_CH_INT_tag;

typedef union EDMA3_TCD_TCD_CH_SBR_union_tag { /* Channel System Bus */
  vuint32_t R;
  struct {
    vuint32_t MID:6;                   /* Master ID */
    vuint32_t _unused_6:9;             /* Reserved */
    vuint32_t PAL:1;                   /* Privileged Access Level */
    vuint32_t EMI:1;                   /* Enable Master ID Replication */
    vuint32_t ATTR:6;                  /* Attribute Output, merged from more bitFields with different positions or sizes */
    vuint32_t _unused_23:9;            /* Reserved */
  } B;
} EDMA3_TCD_TCD_CH_SBR_tag;

typedef union EDMA3_TCD_TCD_CH_PRI_union_tag { /* Channel Priority */
  vuint32_t R;
  struct {
    vuint32_t APL:3;                   /* Arbitration Priority Level */
    vuint32_t _unused_3:27;            /* Reserved */
    vuint32_t DPA:1;                   /* Disable Preempt Ability */
    vuint32_t ECP:1;                   /* Enable Channel Preemption */
  } B;
} EDMA3_TCD_TCD_CH_PRI_tag;

typedef union EDMA3_TCD_TCD_SADDR_union_tag { /* TCD Source Address */
  vuint32_t R;
  struct {
    vuint32_t SADDR:32;                /* Source Address */
  } B;
} EDMA3_TCD_TCD_SADDR_tag;

typedef union EDMA3_TCD_TCD_SOFF_union_tag { /* TCD Signed Source Address Offset */
  vuint16_t R;
  struct {
    vuint16_t SOFF:16;                 /* Source Address Signed Offset */
  } B;
} EDMA3_TCD_TCD_SOFF_tag;

typedef union EDMA3_TCD_TCD_ATTR_union_tag { /* TCD Transfer Attributes */
  vuint16_t R;
  struct {
    vuint16_t DSIZE:3;                 /* Destination Data Transfer Size */
    vuint16_t DMOD:5;                  /* Destination Address Modulo */
    vuint16_t SSIZE:3;                 /* Source Data Transfer Size */
    vuint16_t SMOD:5;                  /* Source Address Modulo */
  } B;
} EDMA3_TCD_TCD_ATTR_tag;

typedef union EDMA3_TCD_TCD_NBYTES_NBYTES_MLOFFNO_union_tag { /* TCD Transfer Size Without Minor Loop Offsets */
  vuint32_t R;
  struct {
    vuint32_t NBYTES:30;               /* Number of Bytes To Transfer Per Service Request */
    vuint32_t DMLOE:1;                 /* Destination Minor Loop Offset Enable */
    vuint32_t SMLOE:1;                 /* Source Minor Loop Offset Enable */
  } B;
} EDMA3_TCD_TCD_NBYTES_NBYTES_MLOFFNO_tag;

typedef union EDMA3_TCD_TCD_NBYTES_NBYTES_MLOFFYES_union_tag { /* TCD Transfer Size with Minor Loop Offsets */
  vuint32_t R;
  struct {
    vuint32_t NBYTES:10;               /* Number of Bytes To Transfer Per Service Request */
    vuint32_t MLOFF:20;                /* Minor Loop Offset */
    vuint32_t DMLOE:1;                 /* Destination Minor Loop Offset Enable */
    vuint32_t SMLOE:1;                 /* Source Minor Loop Offset Enable */
  } B;
} EDMA3_TCD_TCD_NBYTES_NBYTES_MLOFFYES_tag;

typedef union EDMA3_TCD_TCD_NBYTES_union_tag {
  EDMA3_TCD_TCD_NBYTES_NBYTES_MLOFFNO_tag NBYTES_MLOFFNO; /* TCD Transfer Size Without Minor Loop Offsets */
  EDMA3_TCD_TCD_NBYTES_NBYTES_MLOFFYES_tag NBYTES_MLOFFYES; /* TCD Transfer Size with Minor Loop Offsets */
} EDMA3_TCD_TCD_NBYTES_tag;

typedef union EDMA3_TCD_TCD_SLAST_SDA_union_tag { /* TCD Last Source Address Adjustment / Store DADDR Address */
  vuint32_t R;
  struct {
    vuint32_t SLAST_SDA:32;            /* Last Source Address Adjustment / Store DADDR Address */
  } B;
} EDMA3_TCD_TCD_SLAST_SDA_tag;

typedef union EDMA3_TCD_TCD_DADDR_union_tag { /* TCD Destination Address */
  vuint32_t R;
  struct {
    vuint32_t DADDR:32;                /* Destination Address */
  } B;
} EDMA3_TCD_TCD_DADDR_tag;

typedef union EDMA3_TCD_TCD_DOFF_union_tag { /* TCD Signed Destination Address Offset */
  vuint16_t R;
  struct {
    vuint16_t DOFF:16;                 /* Destination Address Signed Offset */
  } B;
} EDMA3_TCD_TCD_DOFF_tag;

typedef union EDMA3_TCD_TCD_CITER_CITER_ELINKNO_union_tag { /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
  vuint16_t R;
  struct {
    vuint16_t CITER:15;                /* Current Major Iteration Count */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} EDMA3_TCD_TCD_CITER_CITER_ELINKNO_tag;

typedef union EDMA3_TCD_TCD_CITER_CITER_ELINKYES_union_tag { /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
  vuint16_t R;
  struct {
    vuint16_t CITER:9;                 /* Current Major Iteration Count */
    vuint16_t LINKCH:5;                /* Minor Loop Link Channel Number, merged from more bitFields with different positions or sizes */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} EDMA3_TCD_TCD_CITER_CITER_ELINKYES_tag;

typedef union EDMA3_TCD_TCD_CITER_union_tag {
  EDMA3_TCD_TCD_CITER_CITER_ELINKNO_tag CITER_ELINKNO; /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
  EDMA3_TCD_TCD_CITER_CITER_ELINKYES_tag CITER_ELINKYES; /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
} EDMA3_TCD_TCD_CITER_tag;

typedef union EDMA3_TCD_TCD_DLAST_SGA_union_tag { /* TCD Last Destination Address Adjustment / Scatter Gather Address */
  vuint32_t R;
  struct {
    vuint32_t DLAST_SGA:32;            /* Last Destination Address Adjustment / Scatter Gather Address */
  } B;
} EDMA3_TCD_TCD_DLAST_SGA_tag;

typedef union EDMA3_TCD_TCD_CSR_union_tag { /* TCD Control and Status */
  vuint16_t R;
  struct {
    vuint16_t START:1;                 /* Channel Start */
    vuint16_t INTMAJOR:1;              /* Enable Interrupt If Major count complete */
    vuint16_t INTHALF:1;               /* Enable Interrupt If Major Counter Half-complete */
    vuint16_t DREQ:1;                  /* Disable Request */
    vuint16_t ESG:1;                   /* Enable Scatter/Gather Processing */
    vuint16_t MAJORELINK:1;            /* Enable Link When Major Loop Complete */
    vuint16_t _unused_6:1;
    vuint16_t ESDA:1;                  /* Enable Store Destination Address */
    vuint16_t MAJORLINKCH:5;           /* Major Loop Link Channel Number, merged from more bitFields with different positions or sizes */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t BWC:2;                   /* Bandwidth Control */
  } B;
} EDMA3_TCD_TCD_CSR_tag;

typedef union EDMA3_TCD_TCD_BITER_BITER_ELINKNO_union_tag { /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
  vuint16_t R;
  struct {
    vuint16_t BITER:15;                /* Starting Major Iteration Count */
    vuint16_t ELINK:1;                 /* Enables Link */
  } B;
} EDMA3_TCD_TCD_BITER_BITER_ELINKNO_tag;

typedef union EDMA3_TCD_TCD_BITER_BITER_ELINKYES_union_tag { /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
  vuint16_t R;
  struct {
    vuint16_t BITER:9;                 /* Starting Major Iteration Count */
    vuint16_t LINKCH:5;                /* Link Channel Number, merged from more bitFields with different positions or sizes */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} EDMA3_TCD_TCD_BITER_BITER_ELINKYES_tag;

typedef union EDMA3_TCD_TCD_BITER_union_tag {
  EDMA3_TCD_TCD_BITER_BITER_ELINKNO_tag BITER_ELINKNO; /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
  EDMA3_TCD_TCD_BITER_BITER_ELINKYES_tag BITER_ELINKYES; /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
} EDMA3_TCD_TCD_BITER_tag;

typedef struct EDMA3_TCD_TCD_struct_tag {
  EDMA3_TCD_TCD_CH_CSR_tag CH_CSR;     /* Channel Control and Status */
  EDMA3_TCD_TCD_CH_ES_tag CH_ES;       /* Channel Error Status */
  EDMA3_TCD_TCD_CH_INT_tag CH_INT;     /* Channel Interrupt Status */
  EDMA3_TCD_TCD_CH_SBR_tag CH_SBR;     /* Channel System Bus */
  EDMA3_TCD_TCD_CH_PRI_tag CH_PRI;     /* Channel Priority */
  uint8_t TCD_reserved0[12];
  EDMA3_TCD_TCD_SADDR_tag SADDR;       /* TCD Source Address */
  EDMA3_TCD_TCD_SOFF_tag SOFF;         /* TCD Signed Source Address Offset */
  EDMA3_TCD_TCD_ATTR_tag ATTR;         /* TCD Transfer Attributes */
  EDMA3_TCD_TCD_NBYTES_tag NBYTES;
  EDMA3_TCD_TCD_SLAST_SDA_tag SLAST_SDA; /* TCD Last Source Address Adjustment / Store DADDR Address */
  EDMA3_TCD_TCD_DADDR_tag DADDR;       /* TCD Destination Address */
  EDMA3_TCD_TCD_DOFF_tag DOFF;         /* TCD Signed Destination Address Offset */
  EDMA3_TCD_TCD_CITER_tag CITER;
  EDMA3_TCD_TCD_DLAST_SGA_tag DLAST_SGA; /* TCD Last Destination Address Adjustment / Scatter Gather Address */
  EDMA3_TCD_TCD_CSR_tag CSR;           /* TCD Control and Status */
  EDMA3_TCD_TCD_BITER_tag BITER;
  uint8_t TCD_reserved1[65472];
} EDMA3_TCD_TCD_tag;

struct EDMA3_TCD_tag {
  EDMA3_TCD_TCD_tag TCD[32];
};


/* ============================================================================
   =============================== Module: EDMA4_MP ===========================
   ============================================================================ */

typedef union EDMA4_MP_CSR_union_tag { /* Management Page Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t EDBG:1;                  /* Enable Debug */
    vuint32_t ERCA:1;                  /* Enable Round Robin Channel Arbitration */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t HAE:1;                   /* Halt After Error */
    vuint32_t HALT:1;                  /* Halt DMA Operations */
    vuint32_t GCLC:1;                  /* Global Channel Linking Control */
    vuint32_t GMRC:1;                  /* Global Master ID Replication Control */
    vuint32_t _unused_8:2;
    vuint32_t _unused_10:6;            /* Reserved */
    vuint32_t VER:8;                   /* eDMA version */
    vuint32_t ACTIVE_ID:5;             /* Active channel ID */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t ACTIVE:1;                /* DMA Active Status */
  } B;
} EDMA4_MP_CSR_tag;

typedef union EDMA4_MP_ES_union_tag {  /* Management Page Error Status Register */
  vuint32_t R;
  struct {
    vuint32_t DBE:1;                   /* Destination Bus Error */
    vuint32_t SBE:1;                   /* Source Bus Error */
    vuint32_t SGE:1;                   /* Scatter/Gather Configuration Error */
    vuint32_t NCE:1;                   /* NBYTES/CITER Configuration Error */
    vuint32_t DOE:1;                   /* Destination Offset Error */
    vuint32_t DAE:1;                   /* Destination Address Error */
    vuint32_t SOE:1;                   /* Source Offset Error */
    vuint32_t SAE:1;                   /* Source Address Error */
    vuint32_t ECX:1;                   /* Transfer Canceled */
    vuint32_t UCE:1;                   /* Uncorrectable TCD error during channel execution. */
    vuint32_t _unused_10:14;           /* Reserved */
    vuint32_t ERRCHN:5;                /* Error Channel Number or Canceled Channel Number */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} EDMA4_MP_ES_tag;

typedef union EDMA4_MP_INT_union_tag { /* Management Page Interrupt Request Status Register */
  vuint32_t R;
  struct {
    vuint32_t INT:32;                  /* Interrupt Request Status */
  } B;
} EDMA4_MP_INT_tag;

typedef union EDMA4_MP_HRS_union_tag { /* Management Page Hardware Request Status Register */
  vuint32_t R;
  struct {
    vuint32_t HRS:32;                  /* Hardware Request Status */
  } B;
} EDMA4_MP_HRS_tag;

typedef union EDMA4_MP_CH_GRPRI_union_tag { /* Channel Arbitration Group Register */
  vuint32_t R;
  struct {
    vuint32_t GRPRI:6;                 /* Arbitration group per channel. */
    vuint32_t _unused_6:26;
  } B;
} EDMA4_MP_CH_GRPRI_tag;

struct EDMA4_MP_tag {
  EDMA4_MP_CSR_tag CSR;                /* Management Page Control Register */
  EDMA4_MP_ES_tag ES;                  /* Management Page Error Status Register */
  EDMA4_MP_INT_tag INT;                /* Management Page Interrupt Request Status Register */
  uint8_t EDMA4_MP_reserved0[4];
  EDMA4_MP_HRS_tag HRS;                /* Management Page Hardware Request Status Register */
  uint8_t EDMA4_MP_reserved1[236];
  EDMA4_MP_CH_GRPRI_tag CH_GRPRI[32];  /* Channel Arbitration Group Register */
};


/* ============================================================================
   =============================== Module: EDMA4_TCD ==========================
   ============================================================================ */

typedef union EDMA4_TCD_TCD_CH_CSR_union_tag { /* Channel Control and Status Register */
  vuint32_t R;
  struct {
    vuint32_t ERQ:1;                   /* Enable DMA Request */
    vuint32_t EARQ:1;                  /* Enable Asynchronous DMA Request */
    vuint32_t EEI:1;                   /* Enable Error Interrupt */
    vuint32_t _unused_3:9;
    vuint32_t SWAP:4;                  /* Swap size */
    vuint32_t SIGNEXT:6;               /* Sign Extension */
    vuint32_t _unused_22:8;
    vuint32_t DONE:1;                  /* Channel Done */
    vuint32_t ACTIVE:1;                /* Channel Active */
  } B;
} EDMA4_TCD_TCD_CH_CSR_tag;

typedef union EDMA4_TCD_TCD_CH_ES_union_tag { /* Channel Error Status Register */
  vuint32_t R;
  struct {
    vuint32_t DBE:1;                   /* Destination Bus Error */
    vuint32_t SBE:1;                   /* Source Bus Error */
    vuint32_t SGE:1;                   /* Scatter/Gather Configuration Error */
    vuint32_t NCE:1;                   /* NBYTES/CITER Configuration Error */
    vuint32_t DOE:1;                   /* Destination Offset Error */
    vuint32_t DAE:1;                   /* Destination Address Error */
    vuint32_t SOE:1;                   /* Source Offset Error */
    vuint32_t SAE:1;                   /* Source Address Error */
    vuint32_t _unused_8:23;            /* Reserved */
    vuint32_t ERR:1;                   /* Error In Channel */
  } B;
} EDMA4_TCD_TCD_CH_ES_tag;

typedef union EDMA4_TCD_TCD_CH_INT_union_tag { /* Channel Interrupt Status Register */
  vuint32_t R;
  struct {
    vuint32_t INT:1;                   /* Interrupt Request */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} EDMA4_TCD_TCD_CH_INT_tag;

typedef union EDMA4_TCD_TCD_CH_SBR_union_tag { /* Channel System Bus Register */
  vuint32_t R;
  struct {
    vuint32_t MID:6;                   /* Master ID */
    vuint32_t _unused_6:7;             /* Reserved */
    vuint32_t INSTR:1;                 /* Instruction/Data Access */
    vuint32_t SECf:1;                  /* Security Level */
    vuint32_t PAL:1;                   /* Privileged Access Level */
    vuint32_t EMI:1;                   /* Enable Master ID replication */
    vuint32_t ATTR:6;                  /* Attribute Output */
    vuint32_t _unused_23:9;            /* Reserved */
  } B;
} EDMA4_TCD_TCD_CH_SBR_tag;

typedef union EDMA4_TCD_TCD_CH_PRI_union_tag { /* Channel Priority Register */
  vuint32_t R;
  struct {
    vuint32_t APL:3;                   /* Arbitration Priority Level */
    vuint32_t _unused_3:27;            /* Reserved */
    vuint32_t DPA:1;                   /* Disable Preempt Ability. */
    vuint32_t ECP:1;                   /* Enable Channel Preemption. */
  } B;
} EDMA4_TCD_TCD_CH_PRI_tag;

typedef union EDMA4_TCD_TCD_CH_MATTR_union_tag { /* Memory Attributes Register */
  vuint16_t R;
  struct {
    vuint16_t RCACHE:4;                /* Read Cache Attributes */
    vuint16_t WCACHE:4;                /* Write Cache Attributes */
    vuint16_t _unused_8:8;             /* Reserved */
  } B;
} EDMA4_TCD_TCD_CH_MATTR_tag;

typedef union EDMA4_TCD_TCD_SADDR_union_tag { /* TCD Source Address Register */
  vuint32_t R;
  struct {
    vuint32_t SADDR:32;                /* Source Address */
  } B;
} EDMA4_TCD_TCD_SADDR_tag;

typedef union EDMA4_TCD_TCD_SOFF_union_tag { /* TCD Signed Source Address Offset Register */
  vuint16_t R;
  struct {
    vuint16_t SOFF:16;                 /* Source address signed offset */
  } B;
} EDMA4_TCD_TCD_SOFF_tag;

typedef union EDMA4_TCD_TCD_ATTR_union_tag { /* TCD Transfer Attributes Register */
  vuint16_t R;
  struct {
    vuint16_t DSIZE:3;                 /* Destination data transfer size */
    vuint16_t DMOD:5;                  /* Destination address modulo */
    vuint16_t SSIZE:3;                 /* Source data transfer size */
    vuint16_t SMOD:5;                  /* Source address modulo */
  } B;
} EDMA4_TCD_TCD_ATTR_tag;

typedef union EDMA4_TCD_TCD_NBYTES_NBYTES_MLOFFNO_union_tag { /* TCD Transfer Size without Minor Loop Offsets Register */
  vuint32_t R;
  struct {
    vuint32_t NBYTES:30;               /* Number of Bytes to transfer per service request */
    vuint32_t DMLOE:1;                 /* Destination Minor Loop Offset Enable */
    vuint32_t SMLOE:1;                 /* Source Minor Loop Offset Enable */
  } B;
} EDMA4_TCD_TCD_NBYTES_NBYTES_MLOFFNO_tag;

typedef union EDMA4_TCD_TCD_NBYTES_NBYTES_MLOFFYES_union_tag { /* TCD Transfer Size with Minor Loop Offsets Register */
  vuint32_t R;
  struct {
    vuint32_t NBYTES:10;               /* Number of Bytes to transfer per service request */
    vuint32_t MLOFF:20;                /* Minor Loop Offset */
    vuint32_t DMLOE:1;                 /* Destination Minor Loop Offset Enable */
    vuint32_t SMLOE:1;                 /* Source Minor Loop Offset Enable */
  } B;
} EDMA4_TCD_TCD_NBYTES_NBYTES_MLOFFYES_tag;

typedef union EDMA4_TCD_TCD_NBYTES_union_tag {
  EDMA4_TCD_TCD_NBYTES_NBYTES_MLOFFNO_tag NBYTES_MLOFFNO; /* TCD Transfer Size without Minor Loop Offsets Register */
  EDMA4_TCD_TCD_NBYTES_NBYTES_MLOFFYES_tag NBYTES_MLOFFYES; /* TCD Transfer Size with Minor Loop Offsets Register */
} EDMA4_TCD_TCD_NBYTES_tag;

typedef union EDMA4_TCD_TCD_SLAST_SDA_union_tag { /* TCD Last Source Address Adjustment / Store DADDR Address Register */
  vuint32_t R;
  struct {
    vuint32_t SLAST_SDA:32;            /* Last Source Address Adjustment / Store DADDR Address */
  } B;
} EDMA4_TCD_TCD_SLAST_SDA_tag;

typedef union EDMA4_TCD_TCD_DADDR_union_tag { /* TCD Destination Address Register */
  vuint32_t R;
  struct {
    vuint32_t DADDR:32;                /* Destination Address */
  } B;
} EDMA4_TCD_TCD_DADDR_tag;

typedef union EDMA4_TCD_TCD_DOFF_union_tag { /* TCD Signed Destination Address Offset Register */
  vuint16_t R;
  struct {
    vuint16_t DOFF:16;                 /* Destination Address Signed Offset */
  } B;
} EDMA4_TCD_TCD_DOFF_tag;

typedef union EDMA4_TCD_TCD_CITER_CITER_ELINKNO_union_tag { /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
  vuint16_t R;
  struct {
    vuint16_t CITER:15;                /* Current Major Iteration Count */
    vuint16_t ELINK:1;                 /* Enable channel-to-channel linking on minor-loop complete */
  } B;
} EDMA4_TCD_TCD_CITER_CITER_ELINKNO_tag;

typedef union EDMA4_TCD_TCD_CITER_CITER_ELINKYES_union_tag { /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
  vuint16_t R;
  struct {
    vuint16_t CITER:9;                 /* Current Major Iteration Count */
    vuint16_t LINKCH:5;                /* Minor Loop Link Channel Number */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t ELINK:1;                 /* Enable channel-to-channel linking on minor-loop complete */
  } B;
} EDMA4_TCD_TCD_CITER_CITER_ELINKYES_tag;

typedef union EDMA4_TCD_TCD_CITER_union_tag {
  EDMA4_TCD_TCD_CITER_CITER_ELINKNO_tag CITER_ELINKNO; /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
  EDMA4_TCD_TCD_CITER_CITER_ELINKYES_tag CITER_ELINKYES; /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
} EDMA4_TCD_TCD_CITER_tag;

typedef union EDMA4_TCD_TCD_DLAST_SGA_union_tag { /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
  vuint32_t R;
  struct {
    vuint32_t DLAST_SGA:32;            /* Final Destination Address Adjustment / Scatter Gather Address */
  } B;
} EDMA4_TCD_TCD_DLAST_SGA_tag;

typedef union EDMA4_TCD_TCD_CSR_union_tag { /* TCD Control and Status Register */
  vuint16_t R;
  struct {
    vuint16_t START:1;                 /* Channel Start */
    vuint16_t INTMAJOR:1;              /* Enable an interrupt when major iteration count completes. */
    vuint16_t INTHALF:1;               /* Enable an interrupt when major counter is half complete. */
    vuint16_t DREQ:1;                  /* Disable request */
    vuint16_t ESG:1;                   /* Enable Scatter/Gather processing */
    vuint16_t MAJORELINK:1;            /* Enable channel-to-channel linking on major loop complete */
    vuint16_t _unused_6:1;
    vuint16_t ESDA:1;                  /* Enable store destination address */
    vuint16_t MAJORLINKCH:5;           /* Major loop link channel number */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t TMC:2;                   /* Transfer Mode Control */
  } B;
} EDMA4_TCD_TCD_CSR_tag;

typedef union EDMA4_TCD_TCD_BITER_BITER_ELINKNO_union_tag { /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
  vuint16_t R;
  struct {
    vuint16_t BITER:15;                /* Starting Major Iteration Count */
    vuint16_t ELINK:1;                 /* Enables channel-to-channel linking on minor loop complete */
  } B;
} EDMA4_TCD_TCD_BITER_BITER_ELINKNO_tag;

typedef union EDMA4_TCD_TCD_BITER_BITER_ELINKYES_union_tag { /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
  vuint16_t R;
  struct {
    vuint16_t BITER:9;                 /* Starting major iteration count */
    vuint16_t LINKCH:5;                /* Link Channel Number */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t ELINK:1;                 /* Enables channel-to-channel linking on minor loop complete */
  } B;
} EDMA4_TCD_TCD_BITER_BITER_ELINKYES_tag;

typedef union EDMA4_TCD_TCD_BITER_union_tag {
  EDMA4_TCD_TCD_BITER_BITER_ELINKNO_tag BITER_ELINKNO; /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
  EDMA4_TCD_TCD_BITER_BITER_ELINKYES_tag BITER_ELINKYES; /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
} EDMA4_TCD_TCD_BITER_tag;

typedef struct EDMA4_TCD_TCD_struct_tag {
  EDMA4_TCD_TCD_CH_CSR_tag CH_CSR;     /* Channel Control and Status Register */
  EDMA4_TCD_TCD_CH_ES_tag CH_ES;       /* Channel Error Status Register */
  EDMA4_TCD_TCD_CH_INT_tag CH_INT;     /* Channel Interrupt Status Register */
  EDMA4_TCD_TCD_CH_SBR_tag CH_SBR;     /* Channel System Bus Register */
  EDMA4_TCD_TCD_CH_PRI_tag CH_PRI;     /* Channel Priority Register */
  uint8_t TCD_reserved0[4];
  EDMA4_TCD_TCD_CH_MATTR_tag CH_MATTR; /* Memory Attributes Register */
  uint8_t TCD_reserved1[6];
  EDMA4_TCD_TCD_SADDR_tag SADDR;       /* TCD Source Address Register */
  EDMA4_TCD_TCD_SOFF_tag SOFF;         /* TCD Signed Source Address Offset Register */
  EDMA4_TCD_TCD_ATTR_tag ATTR;         /* TCD Transfer Attributes Register */
  EDMA4_TCD_TCD_NBYTES_tag NBYTES;
  EDMA4_TCD_TCD_SLAST_SDA_tag SLAST_SDA; /* TCD Last Source Address Adjustment / Store DADDR Address Register */
  EDMA4_TCD_TCD_DADDR_tag DADDR;       /* TCD Destination Address Register */
  EDMA4_TCD_TCD_DOFF_tag DOFF;         /* TCD Signed Destination Address Offset Register */
  EDMA4_TCD_TCD_CITER_tag CITER;
  EDMA4_TCD_TCD_DLAST_SGA_tag DLAST_SGA; /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
  EDMA4_TCD_TCD_CSR_tag CSR;           /* TCD Control and Status Register */
  EDMA4_TCD_TCD_BITER_tag BITER;
  uint8_t TCD_reserved2[65472];
} EDMA4_TCD_TCD_tag;

struct EDMA4_TCD_tag {
  EDMA4_TCD_TCD_tag TCD[32];
};


/* ============================================================================
   =============================== Module: EIM ================================
   ============================================================================ */

typedef union EIM_EIMCR_union_tag {    /* Error Injection Module Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t GEIEN:1;                 /* Global Error Injection Enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} EIM_EIMCR_tag;

typedef union EIM_EICHEN_union_tag {   /* Error Injection Channel Enable register */
  vuint32_t R;
  struct {
    vuint32_t EICH31EN:1;              /* Error Injection Channel 31 Enable */
    vuint32_t EICH30EN:1;              /* Error Injection Channel 30 Enable */
    vuint32_t EICH29EN:1;              /* Error Injection Channel 29 Enable */
    vuint32_t EICH28EN:1;              /* Error Injection Channel 28 Enable */
    vuint32_t EICH27EN:1;              /* Error Injection Channel 27 Enable */
    vuint32_t EICH26EN:1;              /* Error Injection Channel 26 Enable */
    vuint32_t EICH25EN:1;              /* Error Injection Channel 25 Enable */
    vuint32_t EICH24EN:1;              /* Error Injection Channel 24 Enable */
    vuint32_t EICH23EN:1;              /* Error Injection Channel 23 Enable */
    vuint32_t EICH22EN:1;              /* Error Injection Channel 22 Enable */
    vuint32_t EICH21EN:1;              /* Error Injection Channel 21 Enable */
    vuint32_t EICH20EN:1;              /* Error Injection Channel 20 Enable */
    vuint32_t EICH19EN:1;              /* Error Injection Channel 19 Enable */
    vuint32_t EICH18EN:1;              /* Error Injection Channel 18 Enable */
    vuint32_t EICH17EN:1;              /* Error Injection Channel 17 Enable */
    vuint32_t EICH16EN:1;              /* Error Injection Channel 16 Enable */
    vuint32_t EICH15EN:1;              /* Error Injection Channel 15 Enable */
    vuint32_t EICH14EN:1;              /* Error Injection Channel 14 Enable */
    vuint32_t EICH13EN:1;              /* Error Injection Channel 13 Enable */
    vuint32_t EICH12EN:1;              /* Error Injection Channel 12 Enable */
    vuint32_t EICH11EN:1;              /* Error Injection Channel 11 Enable */
    vuint32_t EICH10EN:1;              /* Error Injection Channel 10 Enable */
    vuint32_t EICH9EN:1;               /* Error Injection Channel 9 Enable */
    vuint32_t EICH8EN:1;               /* Error Injection Channel 8 Enable */
    vuint32_t EICH7EN:1;               /* Error Injection Channel 7 Enable */
    vuint32_t EICH6EN:1;               /* Error Injection Channel 6 Enable */
    vuint32_t EICH5EN:1;               /* Error Injection Channel 5 Enable */
    vuint32_t EICH4EN:1;               /* Error Injection Channel 4 Enable */
    vuint32_t EICH3EN:1;               /* Error Injection Channel 3 Enable */
    vuint32_t EICH2EN:1;               /* Error Injection Channel 2 Enable */
    vuint32_t EICH1EN:1;               /* Error Injection Channel 1 Enable */
    vuint32_t EICH0EN:1;               /* Error Injection Channel 0 Enable */
  } B;
} EIM_EICHEN_tag;

typedef union EIM_EICHD0_WORD0_union_tag { /* Error Injection Channel Descriptor 0, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD0_WORD0_tag;

typedef union EIM_EICHD0_WORD1_union_tag { /* Error Injection Channel Descriptor 0, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD0_WORD1_tag;

typedef union EIM_EICHD0_WORD2_union_tag { /* Error Injection Channel Descriptor 0, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD0_WORD2_tag;

typedef union EIM_EICHD0_WORD3_union_tag { /* Error Injection Channel Descriptor 0, Word3 */
  vuint32_t R;
  struct {
    vuint32_t B8_11DATA_MASK:32;       /* Data Mask Bytes 8-11 */
  } B;
} EIM_EICHD0_WORD3_tag;

typedef union EIM_EICHD1_WORD0_union_tag { /* Error Injection Channel Descriptor 1, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD1_WORD0_tag;

typedef union EIM_EICHD1_WORD1_union_tag { /* Error Injection Channel Descriptor 1, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD1_WORD1_tag;

typedef union EIM_EICHD1_WORD2_union_tag { /* Error Injection Channel Descriptor 1, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD1_WORD2_tag;

typedef union EIM_EICHD1_WORD3_union_tag { /* Error Injection Channel Descriptor 1, Word3 */
  vuint32_t R;
  struct {
    vuint32_t B8_11DATA_MASK:32;       /* Data Mask Bytes 8-11 */
  } B;
} EIM_EICHD1_WORD3_tag;

typedef union EIM_EICHD1_WORD4_union_tag { /* Error Injection Channel Descriptor 1, Word4 */
  vuint32_t R;
  struct {
    vuint32_t B12_15DATA_MASK:32;      /* Data Mask Bytes 12-15 */
  } B;
} EIM_EICHD1_WORD4_tag;

typedef union EIM_EICHD1_WORD5_union_tag { /* Error Injection Channel Descriptor 1, Word5 */
  vuint32_t R;
  struct {
    vuint32_t B16_19DATA_MASK:32;      /* Data Mask Bytes 16-19 */
  } B;
} EIM_EICHD1_WORD5_tag;

typedef union EIM_EICHD1_WORD6_union_tag { /* Error Injection Channel Descriptor 1, Word6 */
  vuint32_t R;
  struct {
    vuint32_t B20_23DATA_MASK:32;      /* Data Mask Bytes 20-23 */
  } B;
} EIM_EICHD1_WORD6_tag;

typedef union EIM_EICHD1_WORD7_union_tag { /* Error Injection Channel Descriptor 1, Word7 */
  vuint32_t R;
  struct {
    vuint32_t B24_27DATA_MASK:32;      /* Data Mask Bytes 24-27 */
  } B;
} EIM_EICHD1_WORD7_tag;

typedef union EIM_EICHD1_WORD8_union_tag { /* Error Injection Channel Descriptor 1, Word8 */
  vuint32_t R;
  struct {
    vuint32_t B28_31DATA_MASK:32;      /* Data Mask Bytes 28-31 */
  } B;
} EIM_EICHD1_WORD8_tag;

typedef union EIM_EICHD2_WORD0_union_tag { /* Error Injection Channel Descriptor 2, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD2_WORD0_tag;

typedef union EIM_EICHD2_WORD1_union_tag { /* Error Injection Channel Descriptor 2, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD2_WORD1_tag;

typedef union EIM_EICHD2_WORD2_union_tag { /* Error Injection Channel Descriptor 2, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD2_WORD2_tag;

typedef union EIM_EICHD3_WORD0_union_tag { /* Error Injection Channel Descriptor 3, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD3_WORD0_tag;

typedef union EIM_EICHD3_WORD1_union_tag { /* Error Injection Channel Descriptor 3, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD3_WORD1_tag;

typedef union EIM_EICHD3_WORD2_union_tag { /* Error Injection Channel Descriptor 3, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD3_WORD2_tag;

typedef union EIM_EICHD4_WORD0_union_tag { /* Error Injection Channel Descriptor 4, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD4_WORD0_tag;

typedef union EIM_EICHD4_WORD1_union_tag { /* Error Injection Channel Descriptor 4, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD4_WORD1_tag;

typedef union EIM_EICHD4_WORD2_union_tag { /* Error Injection Channel Descriptor 4, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD4_WORD2_tag;

typedef union EIM_EICHD5_WORD0_union_tag { /* Error Injection Channel Descriptor 5, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD5_WORD0_tag;

typedef union EIM_EICHD5_WORD1_union_tag { /* Error Injection Channel Descriptor 5, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD5_WORD1_tag;

typedef union EIM_EICHD5_WORD2_union_tag { /* Error Injection Channel Descriptor 5, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD5_WORD2_tag;

typedef union EIM_EICHD6_WORD0_union_tag { /* Error Injection Channel Descriptor 6, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD6_WORD0_tag;

typedef union EIM_EICHD6_WORD1_union_tag { /* Error Injection Channel Descriptor 6, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD6_WORD1_tag;

typedef union EIM_EICHD6_WORD2_union_tag { /* Error Injection Channel Descriptor 6, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD6_WORD2_tag;

typedef union EIM_EICHD7_WORD0_union_tag { /* Error Injection Channel Descriptor 7, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:18;
    vuint32_t CHKBIT_MASK:14;          /* Checkbit Mask, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD7_WORD0_tag;

typedef union EIM_EICHD7_WORD1_union_tag { /* Error Injection Channel Descriptor 7, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD7_WORD1_tag;

typedef union EIM_EICHD7_WORD2_union_tag { /* Error Injection Channel Descriptor 7, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD7_WORD2_tag;

typedef union EIM_EICHD8_WORD0_union_tag { /* Error Injection Channel Descriptor 8, Word0 */
  vuint32_t R;
  struct {
    vuint32_t CHKBIT_MASK:32;          /* Checkbit Mask */
  } B;
} EIM_EICHD8_WORD0_tag;

typedef union EIM_EICHD8_WORD1_union_tag { /* Error Injection Channel Descriptor 8, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD8_WORD1_tag;

typedef union EIM_EICHD8_WORD2_union_tag { /* Error Injection Channel Descriptor 8, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD8_WORD2_tag;

typedef union EIM_EICHD8_WORD3_union_tag { /* Error Injection Channel Descriptor 8, Word3 */
  vuint32_t R;
  struct {
    vuint32_t B8_11DATA_MASK:32;       /* Data Mask Bytes 8-11 */
  } B;
} EIM_EICHD8_WORD3_tag;

typedef union EIM_EICHD8_WORD4_union_tag { /* Error Injection Channel Descriptor 8, Word4 */
  vuint32_t R;
  struct {
    vuint32_t B12_15DATA_MASK:32;      /* Data Mask Bytes 12-15 */
  } B;
} EIM_EICHD8_WORD4_tag;

typedef union EIM_EICHD8_WORD5_union_tag { /* Error Injection Channel Descriptor 8, Word5 */
  vuint32_t R;
  struct {
    vuint32_t B16_19DATA_MASK:32;      /* Data Mask Bytes 16-19 */
  } B;
} EIM_EICHD8_WORD5_tag;

typedef union EIM_EICHD8_WORD6_union_tag { /* Error Injection Channel Descriptor 8, Word6 */
  vuint32_t R;
  struct {
    vuint32_t B20_23DATA_MASK:32;      /* Data Mask Bytes 20-23 */
  } B;
} EIM_EICHD8_WORD6_tag;

typedef union EIM_EICHD8_WORD7_union_tag { /* Error Injection Channel Descriptor 8, Word7 */
  vuint32_t R;
  struct {
    vuint32_t B24_27DATA_MASK:32;      /* Data Mask Bytes 24-27 */
  } B;
} EIM_EICHD8_WORD7_tag;

typedef union EIM_EICHD8_WORD8_union_tag { /* Error Injection Channel Descriptor 8, Word8 */
  vuint32_t R;
  struct {
    vuint32_t B28_31DATA_MASK:32;      /* Data Mask Bytes 28-31 */
  } B;
} EIM_EICHD8_WORD8_tag;

typedef union EIM_EICHD9_WORD0_union_tag { /* Error Injection Channel Descriptor 9, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:18;
    vuint32_t CHKBIT_MASK:14;          /* Checkbit Mask */
  } B;
} EIM_EICHD9_WORD0_tag;

typedef union EIM_EICHD9_WORD1_union_tag { /* Error Injection Channel Descriptor 9, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD9_WORD1_tag;

typedef union EIM_EICHD9_WORD2_union_tag { /* Error Injection Channel Descriptor 9, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD9_WORD2_tag;

typedef union EIM_EICHD9_WORD3_union_tag { /* Error Injection Channel Descriptor 9, Word3 */
  vuint32_t R;
  struct {
    vuint32_t B8_11DATA_MASK:32;       /* Data Mask Bytes 8-11 */
  } B;
} EIM_EICHD9_WORD3_tag;

typedef union EIM_EICHD9_WORD4_union_tag { /* Error Injection Channel Descriptor 9, Word4 */
  vuint32_t R;
  struct {
    vuint32_t B12_15DATA_MASK:32;      /* Data Mask Bytes 12-15 */
  } B;
} EIM_EICHD9_WORD4_tag;

typedef union EIM_EICHD9_WORD5_union_tag { /* Error Injection Channel Descriptor 9, Word5 */
  vuint32_t R;
  struct {
    vuint32_t B16_19DATA_MASK:32;      /* Data Mask Bytes 16-19 */
  } B;
} EIM_EICHD9_WORD5_tag;

typedef union EIM_EICHD9_WORD6_union_tag { /* Error Injection Channel Descriptor 9, Word6 */
  vuint32_t R;
  struct {
    vuint32_t B20_23DATA_MASK:32;      /* Data Mask Bytes 20-23 */
  } B;
} EIM_EICHD9_WORD6_tag;

typedef union EIM_EICHD9_WORD7_union_tag { /* Error Injection Channel Descriptor 9, Word7 */
  vuint32_t R;
  struct {
    vuint32_t B24_27DATA_MASK:32;      /* Data Mask Bytes 24-27 */
  } B;
} EIM_EICHD9_WORD7_tag;

typedef union EIM_EICHD9_WORD8_union_tag { /* Error Injection Channel Descriptor 9, Word8 */
  vuint32_t R;
  struct {
    vuint32_t B28_31DATA_MASK:32;      /* Data Mask Bytes 28-31 */
  } B;
} EIM_EICHD9_WORD8_tag;

typedef union EIM_EICHD10_WORD0_union_tag { /* Error Injection Channel Descriptor 10, Word0 */
  vuint32_t R;
  struct {
    vuint32_t CHKBIT_MASK:32;          /* Checkbit Mask */
  } B;
} EIM_EICHD10_WORD0_tag;

typedef union EIM_EICHD10_WORD1_union_tag { /* Error Injection Channel Descriptor 10, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD10_WORD1_tag;

typedef union EIM_EICHD10_WORD2_union_tag { /* Error Injection Channel Descriptor 10, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD10_WORD2_tag;

typedef union EIM_EICHD10_WORD3_union_tag { /* Error Injection Channel Descriptor 10, Word3 */
  vuint32_t R;
  struct {
    vuint32_t B8_11DATA_MASK:32;       /* Data Mask Bytes 8-11 */
  } B;
} EIM_EICHD10_WORD3_tag;

typedef union EIM_EICHD10_WORD4_union_tag { /* Error Injection Channel Descriptor 10, Word4 */
  vuint32_t R;
  struct {
    vuint32_t B12_15DATA_MASK:32;      /* Data Mask Bytes 12-15 */
  } B;
} EIM_EICHD10_WORD4_tag;

typedef union EIM_EICHD11_WORD0_union_tag { /* Error Injection Channel Descriptor 11, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD11_WORD0_tag;

typedef union EIM_EICHD11_WORD1_union_tag { /* Error Injection Channel Descriptor 11, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD11_WORD1_tag;

typedef union EIM_EICHD11_WORD2_union_tag { /* Error Injection Channel Descriptor 11, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD11_WORD2_tag;

typedef union EIM_EICHD12_WORD0_union_tag { /* Error Injection Channel Descriptor 12, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD12_WORD0_tag;

typedef union EIM_EICHD12_WORD1_union_tag { /* Error Injection Channel Descriptor 12, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD12_WORD1_tag;

typedef union EIM_EICHD12_WORD2_union_tag { /* Error Injection Channel Descriptor 12, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD12_WORD2_tag;

typedef union EIM_EICHD13_WORD0_union_tag { /* Error Injection Channel Descriptor 13, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD13_WORD0_tag;

typedef union EIM_EICHD13_WORD1_union_tag { /* Error Injection Channel Descriptor 13, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD13_WORD1_tag;

typedef union EIM_EICHD13_WORD2_union_tag { /* Error Injection Channel Descriptor 13, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD13_WORD2_tag;

typedef union EIM_EICHD14_WORD0_union_tag { /* Error Injection Channel Descriptor 14, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD14_WORD0_tag;

typedef union EIM_EICHD14_WORD1_union_tag { /* Error Injection Channel Descriptor 14, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD14_WORD1_tag;

typedef union EIM_EICHD14_WORD2_union_tag { /* Error Injection Channel Descriptor 14, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD14_WORD2_tag;

typedef union EIM_EICHD15_WORD0_union_tag { /* Error Injection Channel Descriptor 15, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD15_WORD0_tag;

typedef union EIM_EICHD15_WORD1_union_tag { /* Error Injection Channel Descriptor 15, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD15_WORD1_tag;

typedef union EIM_EICHD15_WORD2_union_tag { /* Error Injection Channel Descriptor 15, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD15_WORD2_tag;

typedef union EIM_EICHD16_WORD0_union_tag { /* Error Injection Channel Descriptor 16, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD16_WORD0_tag;

typedef union EIM_EICHD16_WORD1_union_tag { /* Error Injection Channel Descriptor 16, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD16_WORD1_tag;

typedef union EIM_EICHD16_WORD2_union_tag { /* Error Injection Channel Descriptor 16, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD16_WORD2_tag;

typedef union EIM_EICHD17_WORD0_union_tag { /* Error Injection Channel Descriptor 17, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD17_WORD0_tag;

typedef union EIM_EICHD17_WORD1_union_tag { /* Error Injection Channel Descriptor 17, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD17_WORD1_tag;

typedef union EIM_EICHD17_WORD2_union_tag { /* Error Injection Channel Descriptor 17, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD17_WORD2_tag;

typedef union EIM_EICHD18_WORD1_union_tag { /* Error Injection Channel Descriptor 18, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:28;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
    vuint32_t _unused_28:4;
  } B;
} EIM_EICHD18_WORD1_tag;

typedef union EIM_EICHD18_WORD2_union_tag { /* Error Injection Channel Descriptor 18, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD18_WORD2_tag;

typedef union EIM_EICHD19_WORD0_union_tag { /* Error Injection Channel Descriptor 19, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD19_WORD0_tag;

typedef union EIM_EICHD19_WORD1_union_tag { /* Error Injection Channel Descriptor 19, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD19_WORD1_tag;

typedef union EIM_EICHD19_WORD2_union_tag { /* Error Injection Channel Descriptor 19, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD19_WORD2_tag;

typedef union EIM_EICHD20_WORD0_union_tag { /* Error Injection Channel Descriptor 20, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD20_WORD0_tag;

typedef union EIM_EICHD20_WORD1_union_tag { /* Error Injection Channel Descriptor 20, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD20_WORD1_tag;

typedef union EIM_EICHD20_WORD2_union_tag { /* Error Injection Channel Descriptor 20, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD20_WORD2_tag;

typedef union EIM_EICHD21_WORD0_union_tag { /* Error Injection Channel Descriptor 21, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD21_WORD0_tag;

typedef union EIM_EICHD21_WORD1_union_tag { /* Error Injection Channel Descriptor 21, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD21_WORD1_tag;

typedef union EIM_EICHD21_WORD2_union_tag { /* Error Injection Channel Descriptor 21, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD21_WORD2_tag;

typedef union EIM_EICHD22_WORD0_union_tag { /* Error Injection Channel Descriptor 22, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD22_WORD0_tag;

typedef union EIM_EICHD22_WORD1_union_tag { /* Error Injection Channel Descriptor 22, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD22_WORD1_tag;

typedef union EIM_EICHD22_WORD2_union_tag { /* Error Injection Channel Descriptor 22, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD22_WORD2_tag;

typedef union EIM_EICHD23_WORD1_union_tag { /* Error Injection Channel Descriptor 23, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:11;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
    vuint32_t _unused_11:21;
  } B;
} EIM_EICHD23_WORD1_tag;

typedef union EIM_EICHD24_WORD1_union_tag { /* Error Injection Channel Descriptor 24, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:30;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
    vuint32_t _unused_30:2;
  } B;
} EIM_EICHD24_WORD1_tag;

typedef union EIM_EICHD25_WORD1_union_tag { /* Error Injection Channel Descriptor 25, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:2;         /* Data Mask Bytes 0-3 */
    vuint32_t _unused_2:30;
  } B;
} EIM_EICHD25_WORD1_tag;

typedef union EIM_EICHD26_WORD0_union_tag { /* Error Injection Channel Descriptor 26, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD26_WORD0_tag;

typedef union EIM_EICHD26_WORD1_union_tag { /* Error Injection Channel Descriptor 26, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD26_WORD1_tag;

typedef union EIM_EICHD26_WORD2_union_tag { /* Error Injection Channel Descriptor 26, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD26_WORD2_tag;

typedef union EIM_EICHD27_WORD0_union_tag { /* Error Injection Channel Descriptor 27, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD27_WORD0_tag;

typedef union EIM_EICHD27_WORD1_union_tag { /* Error Injection Channel Descriptor 27, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3, merged from more bitFields with different positions or sizes */
  } B;
} EIM_EICHD27_WORD1_tag;

typedef union EIM_EICHD27_WORD2_union_tag { /* Error Injection Channel Descriptor 27, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD27_WORD2_tag;

typedef union EIM_EICHD28_WORD0_union_tag { /* Error Injection Channel Descriptor 28, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD28_WORD0_tag;

typedef union EIM_EICHD28_WORD1_union_tag { /* Error Injection Channel Descriptor 28, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD28_WORD1_tag;

typedef union EIM_EICHD28_WORD2_union_tag { /* Error Injection Channel Descriptor 28, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD28_WORD2_tag;

typedef union EIM_EICHD29_WORD0_union_tag { /* Error Injection Channel Descriptor 29, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD29_WORD0_tag;

typedef union EIM_EICHD29_WORD1_union_tag { /* Error Injection Channel Descriptor 29, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD29_WORD1_tag;

typedef union EIM_EICHD29_WORD2_union_tag { /* Error Injection Channel Descriptor 29, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD29_WORD2_tag;

typedef union EIM_EICHD30_WORD0_union_tag { /* Error Injection Channel Descriptor 30, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_EICHD30_WORD0_tag;

typedef union EIM_EICHD30_WORD1_union_tag { /* Error Injection Channel Descriptor 30, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_EICHD30_WORD1_tag;

typedef union EIM_EICHD30_WORD2_union_tag { /* Error Injection Channel Descriptor 30, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} EIM_EICHD30_WORD2_tag;

typedef union EIM_EICHD31_WORD1_union_tag { /* Error Injection Channel Descriptor 31, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:13;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_13:19;
  } B;
} EIM_EICHD31_WORD1_tag;

struct EIM_tag {
  EIM_EIMCR_tag EIMCR;                 /* Error Injection Module Configuration Register */
  EIM_EICHEN_tag EICHEN;               /* Error Injection Channel Enable register */
  uint8_t EIM_reserved0[248];
  EIM_EICHD0_WORD0_tag EICHD0_WORD0;   /* Error Injection Channel Descriptor 0, Word0 */
  EIM_EICHD0_WORD1_tag EICHD0_WORD1;   /* Error Injection Channel Descriptor 0, Word1 */
  EIM_EICHD0_WORD2_tag EICHD0_WORD2;   /* Error Injection Channel Descriptor 0, Word2 */
  EIM_EICHD0_WORD3_tag EICHD0_WORD3;   /* Error Injection Channel Descriptor 0, Word3 */
  uint8_t EIM_reserved1[48];
  EIM_EICHD1_WORD0_tag EICHD1_WORD0;   /* Error Injection Channel Descriptor 1, Word0 */
  EIM_EICHD1_WORD1_tag EICHD1_WORD1;   /* Error Injection Channel Descriptor 1, Word1 */
  EIM_EICHD1_WORD2_tag EICHD1_WORD2;   /* Error Injection Channel Descriptor 1, Word2 */
  EIM_EICHD1_WORD3_tag EICHD1_WORD3;   /* Error Injection Channel Descriptor 1, Word3 */
  EIM_EICHD1_WORD4_tag EICHD1_WORD4;   /* Error Injection Channel Descriptor 1, Word4 */
  EIM_EICHD1_WORD5_tag EICHD1_WORD5;   /* Error Injection Channel Descriptor 1, Word5 */
  EIM_EICHD1_WORD6_tag EICHD1_WORD6;   /* Error Injection Channel Descriptor 1, Word6 */
  EIM_EICHD1_WORD7_tag EICHD1_WORD7;   /* Error Injection Channel Descriptor 1, Word7 */
  EIM_EICHD1_WORD8_tag EICHD1_WORD8;   /* Error Injection Channel Descriptor 1, Word8 */
  uint8_t EIM_reserved2[28];
  EIM_EICHD2_WORD0_tag EICHD2_WORD0;   /* Error Injection Channel Descriptor 2, Word0 */
  EIM_EICHD2_WORD1_tag EICHD2_WORD1;   /* Error Injection Channel Descriptor 2, Word1 */
  EIM_EICHD2_WORD2_tag EICHD2_WORD2;   /* Error Injection Channel Descriptor 2, Word2 */
  uint8_t EIM_reserved3[52];
  EIM_EICHD3_WORD0_tag EICHD3_WORD0;   /* Error Injection Channel Descriptor 3, Word0 */
  EIM_EICHD3_WORD1_tag EICHD3_WORD1;   /* Error Injection Channel Descriptor 3, Word1 */
  EIM_EICHD3_WORD2_tag EICHD3_WORD2;   /* Error Injection Channel Descriptor 3, Word2 */
  uint8_t EIM_reserved4[52];
  EIM_EICHD4_WORD0_tag EICHD4_WORD0;   /* Error Injection Channel Descriptor 4, Word0 */
  EIM_EICHD4_WORD1_tag EICHD4_WORD1;   /* Error Injection Channel Descriptor 4, Word1 */
  EIM_EICHD4_WORD2_tag EICHD4_WORD2;   /* Error Injection Channel Descriptor 4, Word2 */
  uint8_t EIM_reserved5[52];
  EIM_EICHD5_WORD0_tag EICHD5_WORD0;   /* Error Injection Channel Descriptor 5, Word0 */
  EIM_EICHD5_WORD1_tag EICHD5_WORD1;   /* Error Injection Channel Descriptor 5, Word1 */
  EIM_EICHD5_WORD2_tag EICHD5_WORD2;   /* Error Injection Channel Descriptor 5, Word2 */
  uint8_t EIM_reserved6[52];
  EIM_EICHD6_WORD0_tag EICHD6_WORD0;   /* Error Injection Channel Descriptor 6, Word0 */
  EIM_EICHD6_WORD1_tag EICHD6_WORD1;   /* Error Injection Channel Descriptor 6, Word1 */
  EIM_EICHD6_WORD2_tag EICHD6_WORD2;   /* Error Injection Channel Descriptor 6, Word2 */
  uint8_t EIM_reserved7[52];
  EIM_EICHD7_WORD0_tag EICHD7_WORD0;   /* Error Injection Channel Descriptor 7, Word0 */
  EIM_EICHD7_WORD1_tag EICHD7_WORD1;   /* Error Injection Channel Descriptor 7, Word1 */
  EIM_EICHD7_WORD2_tag EICHD7_WORD2;   /* Error Injection Channel Descriptor 7, Word2 */
  uint8_t EIM_reserved8[52];
  EIM_EICHD8_WORD0_tag EICHD8_WORD0;   /* Error Injection Channel Descriptor 8, Word0 */
  EIM_EICHD8_WORD1_tag EICHD8_WORD1;   /* Error Injection Channel Descriptor 8, Word1 */
  EIM_EICHD8_WORD2_tag EICHD8_WORD2;   /* Error Injection Channel Descriptor 8, Word2 */
  EIM_EICHD8_WORD3_tag EICHD8_WORD3;   /* Error Injection Channel Descriptor 8, Word3 */
  EIM_EICHD8_WORD4_tag EICHD8_WORD4;   /* Error Injection Channel Descriptor 8, Word4 */
  EIM_EICHD8_WORD5_tag EICHD8_WORD5;   /* Error Injection Channel Descriptor 8, Word5 */
  EIM_EICHD8_WORD6_tag EICHD8_WORD6;   /* Error Injection Channel Descriptor 8, Word6 */
  EIM_EICHD8_WORD7_tag EICHD8_WORD7;   /* Error Injection Channel Descriptor 8, Word7 */
  EIM_EICHD8_WORD8_tag EICHD8_WORD8;   /* Error Injection Channel Descriptor 8, Word8 */
  uint8_t EIM_reserved9[28];
  EIM_EICHD9_WORD0_tag EICHD9_WORD0;   /* Error Injection Channel Descriptor 9, Word0 */
  EIM_EICHD9_WORD1_tag EICHD9_WORD1;   /* Error Injection Channel Descriptor 9, Word1 */
  EIM_EICHD9_WORD2_tag EICHD9_WORD2;   /* Error Injection Channel Descriptor 9, Word2 */
  EIM_EICHD9_WORD3_tag EICHD9_WORD3;   /* Error Injection Channel Descriptor 9, Word3 */
  EIM_EICHD9_WORD4_tag EICHD9_WORD4;   /* Error Injection Channel Descriptor 9, Word4 */
  EIM_EICHD9_WORD5_tag EICHD9_WORD5;   /* Error Injection Channel Descriptor 9, Word5 */
  EIM_EICHD9_WORD6_tag EICHD9_WORD6;   /* Error Injection Channel Descriptor 9, Word6 */
  EIM_EICHD9_WORD7_tag EICHD9_WORD7;   /* Error Injection Channel Descriptor 9, Word7 */
  EIM_EICHD9_WORD8_tag EICHD9_WORD8;   /* Error Injection Channel Descriptor 9, Word8 */
  uint8_t EIM_reserved10[28];
  EIM_EICHD10_WORD0_tag EICHD10_WORD0; /* Error Injection Channel Descriptor 10, Word0 */
  EIM_EICHD10_WORD1_tag EICHD10_WORD1; /* Error Injection Channel Descriptor 10, Word1 */
  EIM_EICHD10_WORD2_tag EICHD10_WORD2; /* Error Injection Channel Descriptor 10, Word2 */
  EIM_EICHD10_WORD3_tag EICHD10_WORD3; /* Error Injection Channel Descriptor 10, Word3 */
  EIM_EICHD10_WORD4_tag EICHD10_WORD4; /* Error Injection Channel Descriptor 10, Word4 */
  uint8_t EIM_reserved11[44];
  EIM_EICHD11_WORD0_tag EICHD11_WORD0; /* Error Injection Channel Descriptor 11, Word0 */
  EIM_EICHD11_WORD1_tag EICHD11_WORD1; /* Error Injection Channel Descriptor 11, Word1 */
  EIM_EICHD11_WORD2_tag EICHD11_WORD2; /* Error Injection Channel Descriptor 11, Word2 */
  uint8_t EIM_reserved12[52];
  EIM_EICHD12_WORD0_tag EICHD12_WORD0; /* Error Injection Channel Descriptor 12, Word0 */
  EIM_EICHD12_WORD1_tag EICHD12_WORD1; /* Error Injection Channel Descriptor 12, Word1 */
  EIM_EICHD12_WORD2_tag EICHD12_WORD2; /* Error Injection Channel Descriptor 12, Word2 */
  uint8_t EIM_reserved13[52];
  EIM_EICHD13_WORD0_tag EICHD13_WORD0; /* Error Injection Channel Descriptor 13, Word0 */
  EIM_EICHD13_WORD1_tag EICHD13_WORD1; /* Error Injection Channel Descriptor 13, Word1 */
  EIM_EICHD13_WORD2_tag EICHD13_WORD2; /* Error Injection Channel Descriptor 13, Word2 */
  uint8_t EIM_reserved14[52];
  EIM_EICHD14_WORD0_tag EICHD14_WORD0; /* Error Injection Channel Descriptor 14, Word0 */
  EIM_EICHD14_WORD1_tag EICHD14_WORD1; /* Error Injection Channel Descriptor 14, Word1 */
  EIM_EICHD14_WORD2_tag EICHD14_WORD2; /* Error Injection Channel Descriptor 14, Word2 */
  uint8_t EIM_reserved15[52];
  EIM_EICHD15_WORD0_tag EICHD15_WORD0; /* Error Injection Channel Descriptor 15, Word0 */
  EIM_EICHD15_WORD1_tag EICHD15_WORD1; /* Error Injection Channel Descriptor 15, Word1 */
  EIM_EICHD15_WORD2_tag EICHD15_WORD2; /* Error Injection Channel Descriptor 15, Word2 */
  uint8_t EIM_reserved16[52];
  EIM_EICHD16_WORD0_tag EICHD16_WORD0; /* Error Injection Channel Descriptor 16, Word0 */
  EIM_EICHD16_WORD1_tag EICHD16_WORD1; /* Error Injection Channel Descriptor 16, Word1 */
  EIM_EICHD16_WORD2_tag EICHD16_WORD2; /* Error Injection Channel Descriptor 16, Word2 */
  uint8_t EIM_reserved17[52];
  EIM_EICHD17_WORD0_tag EICHD17_WORD0; /* Error Injection Channel Descriptor 17, Word0 */
  EIM_EICHD17_WORD1_tag EICHD17_WORD1; /* Error Injection Channel Descriptor 17, Word1 */
  EIM_EICHD17_WORD2_tag EICHD17_WORD2; /* Error Injection Channel Descriptor 17, Word2 */
  uint8_t EIM_reserved18[56];
  EIM_EICHD18_WORD1_tag EICHD18_WORD1; /* Error Injection Channel Descriptor 18, Word1 */
  EIM_EICHD18_WORD2_tag EICHD18_WORD2; /* Error Injection Channel Descriptor 18, Word2 */
  uint8_t EIM_reserved19[52];
  EIM_EICHD19_WORD0_tag EICHD19_WORD0; /* Error Injection Channel Descriptor 19, Word0 */
  EIM_EICHD19_WORD1_tag EICHD19_WORD1; /* Error Injection Channel Descriptor 19, Word1 */
  EIM_EICHD19_WORD2_tag EICHD19_WORD2; /* Error Injection Channel Descriptor 19, Word2 */
  uint8_t EIM_reserved20[52];
  EIM_EICHD20_WORD0_tag EICHD20_WORD0; /* Error Injection Channel Descriptor 20, Word0 */
  EIM_EICHD20_WORD1_tag EICHD20_WORD1; /* Error Injection Channel Descriptor 20, Word1 */
  EIM_EICHD20_WORD2_tag EICHD20_WORD2; /* Error Injection Channel Descriptor 20, Word2 */
  uint8_t EIM_reserved21[52];
  EIM_EICHD21_WORD0_tag EICHD21_WORD0; /* Error Injection Channel Descriptor 21, Word0 */
  EIM_EICHD21_WORD1_tag EICHD21_WORD1; /* Error Injection Channel Descriptor 21, Word1 */
  EIM_EICHD21_WORD2_tag EICHD21_WORD2; /* Error Injection Channel Descriptor 21, Word2 */
  uint8_t EIM_reserved22[52];
  EIM_EICHD22_WORD0_tag EICHD22_WORD0; /* Error Injection Channel Descriptor 22, Word0 */
  EIM_EICHD22_WORD1_tag EICHD22_WORD1; /* Error Injection Channel Descriptor 22, Word1 */
  EIM_EICHD22_WORD2_tag EICHD22_WORD2; /* Error Injection Channel Descriptor 22, Word2 */
  uint8_t EIM_reserved23[56];
  EIM_EICHD23_WORD1_tag EICHD23_WORD1; /* Error Injection Channel Descriptor 23, Word1 */
  uint8_t EIM_reserved24[60];
  EIM_EICHD24_WORD1_tag EICHD24_WORD1; /* Error Injection Channel Descriptor 24, Word1 */
  uint8_t EIM_reserved25[60];
  EIM_EICHD25_WORD1_tag EICHD25_WORD1; /* Error Injection Channel Descriptor 25, Word1 */
  uint8_t EIM_reserved26[56];
  EIM_EICHD26_WORD0_tag EICHD26_WORD0; /* Error Injection Channel Descriptor 26, Word0 */
  EIM_EICHD26_WORD1_tag EICHD26_WORD1; /* Error Injection Channel Descriptor 26, Word1 */
  EIM_EICHD26_WORD2_tag EICHD26_WORD2; /* Error Injection Channel Descriptor 26, Word2 */
  uint8_t EIM_reserved27[52];
  EIM_EICHD27_WORD0_tag EICHD27_WORD0; /* Error Injection Channel Descriptor 27, Word0 */
  EIM_EICHD27_WORD1_tag EICHD27_WORD1; /* Error Injection Channel Descriptor 27, Word1 */
  EIM_EICHD27_WORD2_tag EICHD27_WORD2; /* Error Injection Channel Descriptor 27, Word2 */
  uint8_t EIM_reserved28[52];
  EIM_EICHD28_WORD0_tag EICHD28_WORD0; /* Error Injection Channel Descriptor 28, Word0 */
  EIM_EICHD28_WORD1_tag EICHD28_WORD1; /* Error Injection Channel Descriptor 28, Word1 */
  EIM_EICHD28_WORD2_tag EICHD28_WORD2; /* Error Injection Channel Descriptor 28, Word2 */
  uint8_t EIM_reserved29[52];
  EIM_EICHD29_WORD0_tag EICHD29_WORD0; /* Error Injection Channel Descriptor 29, Word0 */
  EIM_EICHD29_WORD1_tag EICHD29_WORD1; /* Error Injection Channel Descriptor 29, Word1 */
  EIM_EICHD29_WORD2_tag EICHD29_WORD2; /* Error Injection Channel Descriptor 29, Word2 */
  uint8_t EIM_reserved30[52];
  EIM_EICHD30_WORD0_tag EICHD30_WORD0; /* Error Injection Channel Descriptor 30, Word0 */
  EIM_EICHD30_WORD1_tag EICHD30_WORD1; /* Error Injection Channel Descriptor 30, Word1 */
  EIM_EICHD30_WORD2_tag EICHD30_WORD2; /* Error Injection Channel Descriptor 30, Word2 */
  uint8_t EIM_reserved31[56];
  EIM_EICHD31_WORD1_tag EICHD31_WORD1; /* Error Injection Channel Descriptor 31, Word1 */
};


/* ============================================================================
   =============================== Module: EIM_GTM ============================
   ============================================================================ */

typedef union EIM_GTM_EIMCR_union_tag { /* Error Injection Module Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t GEIEN:1;                 /* Global Error Injection Enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} EIM_GTM_EIMCR_tag;

typedef union EIM_GTM_EICHEN_union_tag { /* Error Injection Channel Enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t EICH11EN:1;              /* Error Injection Channel 11 Enable */
    vuint32_t EICH10EN:1;              /* Error Injection Channel 10 Enable */
    vuint32_t EICH9EN:1;               /* Error Injection Channel 9 Enable */
    vuint32_t EICH8EN:1;               /* Error Injection Channel 8 Enable */
    vuint32_t EICH7EN:1;               /* Error Injection Channel 7 Enable */
    vuint32_t EICH6EN:1;               /* Error Injection Channel 6 Enable */
    vuint32_t EICH5EN:1;               /* Error Injection Channel 5 Enable */
    vuint32_t EICH4EN:1;               /* Error Injection Channel 4 Enable */
    vuint32_t EICH3EN:1;               /* Error Injection Channel 3 Enable */
    vuint32_t EICH2EN:1;               /* Error Injection Channel 2 Enable */
    vuint32_t EICH1EN:1;               /* Error Injection Channel 1 Enable */
    vuint32_t EICH0EN:1;               /* Error Injection Channel 0 Enable */
  } B;
} EIM_GTM_EICHEN_tag;

typedef union EIM_GTM_EICHD0_WORD0_union_tag { /* Error Injection Channel Descriptor 0, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD0_WORD0_tag;

typedef union EIM_GTM_EICHD0_WORD1_union_tag { /* Error Injection Channel Descriptor 0, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_GTM_EICHD0_WORD1_tag;

typedef union EIM_GTM_EICHD1_WORD0_union_tag { /* Error Injection Channel Descriptor 1, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD1_WORD0_tag;

typedef union EIM_GTM_EICHD1_WORD1_union_tag { /* Error Injection Channel Descriptor 1, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_GTM_EICHD1_WORD1_tag;

typedef union EIM_GTM_EICHD2_WORD0_union_tag { /* Error Injection Channel Descriptor 2, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD2_WORD0_tag;

typedef union EIM_GTM_EICHD2_WORD1_union_tag { /* Error Injection Channel Descriptor 2, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:24;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_24:8;
  } B;
} EIM_GTM_EICHD2_WORD1_tag;

typedef union EIM_GTM_EICHD3_WORD0_union_tag { /* Error Injection Channel Descriptor 3, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD3_WORD0_tag;

typedef union EIM_GTM_EICHD3_WORD1_union_tag { /* Error Injection Channel Descriptor 3, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:24;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_24:8;
  } B;
} EIM_GTM_EICHD3_WORD1_tag;

typedef union EIM_GTM_EICHD4_WORD0_union_tag { /* Error Injection Channel Descriptor 4, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD4_WORD0_tag;

typedef union EIM_GTM_EICHD4_WORD1_union_tag { /* Error Injection Channel Descriptor 4, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:24;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_24:8;
  } B;
} EIM_GTM_EICHD4_WORD1_tag;

typedef union EIM_GTM_EICHD5_WORD0_union_tag { /* Error Injection Channel Descriptor 5, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD5_WORD0_tag;

typedef union EIM_GTM_EICHD5_WORD1_union_tag { /* Error Injection Channel Descriptor 5, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:29;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_29:3;
  } B;
} EIM_GTM_EICHD5_WORD1_tag;

typedef union EIM_GTM_EICHD6_WORD0_union_tag { /* Error Injection Channel Descriptor 6, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD6_WORD0_tag;

typedef union EIM_GTM_EICHD6_WORD1_union_tag { /* Error Injection Channel Descriptor 6, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_GTM_EICHD6_WORD1_tag;

typedef union EIM_GTM_EICHD7_WORD0_union_tag { /* Error Injection Channel Descriptor 7, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD7_WORD0_tag;

typedef union EIM_GTM_EICHD7_WORD1_union_tag { /* Error Injection Channel Descriptor 7, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_GTM_EICHD7_WORD1_tag;

typedef union EIM_GTM_EICHD8_WORD0_union_tag { /* Error Injection Channel Descriptor 8, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD8_WORD0_tag;

typedef union EIM_GTM_EICHD8_WORD1_union_tag { /* Error Injection Channel Descriptor 8, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_GTM_EICHD8_WORD1_tag;

typedef union EIM_GTM_EICHD9_WORD0_union_tag { /* Error Injection Channel Descriptor 9, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD9_WORD0_tag;

typedef union EIM_GTM_EICHD9_WORD1_union_tag { /* Error Injection Channel Descriptor 9, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_GTM_EICHD9_WORD1_tag;

typedef union EIM_GTM_EICHD10_WORD0_union_tag { /* Error Injection Channel Descriptor 10, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD10_WORD0_tag;

typedef union EIM_GTM_EICHD10_WORD1_union_tag { /* Error Injection Channel Descriptor 10, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_GTM_EICHD10_WORD1_tag;

typedef union EIM_GTM_EICHD11_WORD0_union_tag { /* Error Injection Channel Descriptor 11, Word0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CHKBIT_MASK:8;           /* Checkbit Mask */
  } B;
} EIM_GTM_EICHD11_WORD0_tag;

typedef union EIM_GTM_EICHD11_WORD1_union_tag { /* Error Injection Channel Descriptor 11, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:32;        /* Data Mask Bytes 0-3 */
  } B;
} EIM_GTM_EICHD11_WORD1_tag;

struct EIM_GTM_tag {
  EIM_GTM_EIMCR_tag EIMCR;             /* Error Injection Module Configuration Register */
  EIM_GTM_EICHEN_tag EICHEN;           /* Error Injection Channel Enable register */
  uint8_t EIM_GTM_reserved0[248];
  EIM_GTM_EICHD0_WORD0_tag EICHD0_WORD0; /* Error Injection Channel Descriptor 0, Word0 */
  EIM_GTM_EICHD0_WORD1_tag EICHD0_WORD1; /* Error Injection Channel Descriptor 0, Word1 */
  uint8_t EIM_GTM_reserved1[56];
  EIM_GTM_EICHD1_WORD0_tag EICHD1_WORD0; /* Error Injection Channel Descriptor 1, Word0 */
  EIM_GTM_EICHD1_WORD1_tag EICHD1_WORD1; /* Error Injection Channel Descriptor 1, Word1 */
  uint8_t EIM_GTM_reserved2[56];
  EIM_GTM_EICHD2_WORD0_tag EICHD2_WORD0; /* Error Injection Channel Descriptor 2, Word0 */
  EIM_GTM_EICHD2_WORD1_tag EICHD2_WORD1; /* Error Injection Channel Descriptor 2, Word1 */
  uint8_t EIM_GTM_reserved3[56];
  EIM_GTM_EICHD3_WORD0_tag EICHD3_WORD0; /* Error Injection Channel Descriptor 3, Word0 */
  EIM_GTM_EICHD3_WORD1_tag EICHD3_WORD1; /* Error Injection Channel Descriptor 3, Word1 */
  uint8_t EIM_GTM_reserved4[56];
  EIM_GTM_EICHD4_WORD0_tag EICHD4_WORD0; /* Error Injection Channel Descriptor 4, Word0 */
  EIM_GTM_EICHD4_WORD1_tag EICHD4_WORD1; /* Error Injection Channel Descriptor 4, Word1 */
  uint8_t EIM_GTM_reserved5[56];
  EIM_GTM_EICHD5_WORD0_tag EICHD5_WORD0; /* Error Injection Channel Descriptor 5, Word0 */
  EIM_GTM_EICHD5_WORD1_tag EICHD5_WORD1; /* Error Injection Channel Descriptor 5, Word1 */
  uint8_t EIM_GTM_reserved6[56];
  EIM_GTM_EICHD6_WORD0_tag EICHD6_WORD0; /* Error Injection Channel Descriptor 6, Word0 */
  EIM_GTM_EICHD6_WORD1_tag EICHD6_WORD1; /* Error Injection Channel Descriptor 6, Word1 */
  uint8_t EIM_GTM_reserved7[56];
  EIM_GTM_EICHD7_WORD0_tag EICHD7_WORD0; /* Error Injection Channel Descriptor 7, Word0 */
  EIM_GTM_EICHD7_WORD1_tag EICHD7_WORD1; /* Error Injection Channel Descriptor 7, Word1 */
  uint8_t EIM_GTM_reserved8[56];
  EIM_GTM_EICHD8_WORD0_tag EICHD8_WORD0; /* Error Injection Channel Descriptor 8, Word0 */
  EIM_GTM_EICHD8_WORD1_tag EICHD8_WORD1; /* Error Injection Channel Descriptor 8, Word1 */
  uint8_t EIM_GTM_reserved9[56];
  EIM_GTM_EICHD9_WORD0_tag EICHD9_WORD0; /* Error Injection Channel Descriptor 9, Word0 */
  EIM_GTM_EICHD9_WORD1_tag EICHD9_WORD1; /* Error Injection Channel Descriptor 9, Word1 */
  uint8_t EIM_GTM_reserved10[56];
  EIM_GTM_EICHD10_WORD0_tag EICHD10_WORD0; /* Error Injection Channel Descriptor 10, Word0 */
  EIM_GTM_EICHD10_WORD1_tag EICHD10_WORD1; /* Error Injection Channel Descriptor 10, Word1 */
  uint8_t EIM_GTM_reserved11[56];
  EIM_GTM_EICHD11_WORD0_tag EICHD11_WORD0; /* Error Injection Channel Descriptor 11, Word0 */
  EIM_GTM_EICHD11_WORD1_tag EICHD11_WORD1; /* Error Injection Channel Descriptor 11, Word1 */
};


/* ============================================================================
   =============================== Module: EMIOS ==============================
   ============================================================================ */

typedef union EMIOS_MCR_union_tag {    /* Module Configuration */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t GPRE:8;                  /* Global Prescaler */
    vuint32_t _unused_16:10;
    vuint32_t GPREN:1;                 /* Global Prescaler Enable */
    vuint32_t _unused_27:1;
    vuint32_t GTBE:1;                  /* Global Timebase Enable */
    vuint32_t FRZ:1;                   /* Freeze */
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} EMIOS_MCR_tag;

typedef union EMIOS_GFLAG_union_tag {  /* Global Flag */
  vuint32_t R;
  struct {
    vuint32_t F0:1;                    /* Mirror of UC 0 FLAG */
    vuint32_t F1:1;                    /* Mirror of UC 1 FLAG */
    vuint32_t F2:1;                    /* Mirror of UC 2 FLAG */
    vuint32_t F3:1;                    /* Mirror of UC 3 FLAG */
    vuint32_t F4:1;                    /* Mirror of UC 4 FLAG */
    vuint32_t F5:1;                    /* Mirror of UC 5 FLAG */
    vuint32_t F6:1;                    /* Mirror of UC 6 FLAG */
    vuint32_t F7:1;                    /* Mirror of UC 7 FLAG */
    vuint32_t F8:1;                    /* Mirror of WSC 8 flags (except FLAGFF) */
    vuint32_t F9:1;                    /* Mirror of WSC 8 FLAGFF */
    vuint32_t F10:1;                   /* Mirror of WSC 10 flags (except FLAGFF) */
    vuint32_t F11:1;                   /* Mirror of WSC 10 FLAGFF */
    vuint32_t F12:1;                   /* Mirror of WSC 12 flags (except FLAGFF) */
    vuint32_t F13:1;                   /* Mirror of WSC 12 FLAGFF */
    vuint32_t F14:1;                   /* Mirror of WSC 14 flags (except FLAGFF) */
    vuint32_t F15:1;                   /* Mirror of WSC 14 FLAGFF */
    vuint32_t F16:1;                   /* Mirror of UC 16 FLAG */
    vuint32_t F17:1;                   /* Mirror of UC 17 FLAG */
    vuint32_t F18:1;                   /* Mirror of UC 18 FLAG */
    vuint32_t F19:1;                   /* Mirror of UC 19 FLAG */
    vuint32_t F20:1;                   /* Mirror of UC 20 FLAG */
    vuint32_t F21:1;                   /* Mirror of UC 21 FLAG */
    vuint32_t F22:1;                   /* Mirror of UC 22 FLAG */
    vuint32_t F23:1;                   /* Mirror of UC 23 FLAG */
    vuint32_t F24:1;                   /* Mirror of UC 24 FLAG */
    vuint32_t F25:1;                   /* Mirror of UC 25 FLAG */
    vuint32_t F26:1;                   /* Mirror of UC 26 FLAG */
    vuint32_t F27:1;                   /* Mirror of UC 27 FLAG */
    vuint32_t F28:1;                   /* Mirror of UC 28 FLAG */
    vuint32_t F29:1;                   /* Mirror of UC 29 FLAG */
    vuint32_t F30:1;                   /* Mirror of UC 30 FLAG */
    vuint32_t F31:1;                   /* Mirror of UC 31 FLAG */
  } B;
} EMIOS_GFLAG_tag;

typedef union EMIOS_OUDIS_union_tag {  /* Output Update Disable */
  vuint32_t R;
  struct {
    vuint32_t OU0:1;                   /* Channel 0 Output Update Disable */
    vuint32_t OU1:1;                   /* Channel 1 Output Update Disable */
    vuint32_t OU2:1;                   /* Channel 2 Output Update Disable */
    vuint32_t OU3:1;                   /* Channel 3 Output Update Disable */
    vuint32_t OU4:1;                   /* Channel 4 Output Update Disable */
    vuint32_t OU5:1;                   /* Channel 5 Output Update Disable */
    vuint32_t OU6:1;                   /* Channel 6 Output Update Disable */
    vuint32_t OU7:1;                   /* Channel 7 Output Update Disable */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t OU16:1;                  /* Channel 16 Output Update Disable */
    vuint32_t OU17:1;                  /* Channel 17 Output Update Disable */
    vuint32_t OU18:1;                  /* Channel 18 Output Update Disable */
    vuint32_t OU19:1;                  /* Channel 19 Output Update Disable */
    vuint32_t OU20:1;                  /* Channel 20 Output Update Disable */
    vuint32_t OU21:1;                  /* Channel 21 Output Update Disable */
    vuint32_t OU22:1;                  /* Channel 22 Output Update Disable */
    vuint32_t OU23:1;                  /* Channel 23 Output Update Disable */
    vuint32_t OU24:1;                  /* Channel 24 Output Update Disable */
    vuint32_t OU25:1;                  /* Channel 25 Output Update Disable */
    vuint32_t OU26:1;                  /* Channel 26 Output Update Disable */
    vuint32_t OU27:1;                  /* Channel 27 Output Update Disable */
    vuint32_t OU28:1;                  /* Channel 28 Output Update Disable */
    vuint32_t OU29:1;                  /* Channel 29 Output Update Disable */
    vuint32_t OU30:1;                  /* Channel 30 Output Update Disable */
    vuint32_t OU31:1;                  /* Channel 31 Output Update Disable */
  } B;
} EMIOS_OUDIS_tag;

typedef union EMIOS_CH_UC_A_union_tag { /* UC A 0 */
  vuint32_t R;
  struct {
    vuint32_t A:24;                    /* A */
    vuint32_t _unused_24:7;
    vuint32_t RISE_FALL:1;             /* Rising and falling edge detection */
  } B;
} EMIOS_CH_UC_A_tag;

typedef union EMIOS_CH_UC_B_union_tag { /* UC B 0 */
  vuint32_t R;
  struct {
    vuint32_t B:24;                    /* B */
    vuint32_t _unused_24:8;
  } B;
} EMIOS_CH_UC_B_tag;

typedef union EMIOS_CH_UC_CNT_union_tag { /* UC Counter 0 */
  vuint32_t R;
  struct {
    vuint32_t C:24;                    /* Internal Counter Value */
    vuint32_t _unused_24:8;
  } B;
} EMIOS_CH_UC_CNT_tag;

typedef union EMIOS_CH_UC_C_union_tag { /* UC Control 0 */
  vuint32_t R;
  struct {
    vuint32_t MODE:7;                  /* Mode Selection */
    vuint32_t EDPOL:1;                 /* Edge Polarity */
    vuint32_t EDSEL:1;                 /* Edge Selection */
    vuint32_t BSL:2;                   /* Bus Select */
    vuint32_t _unused_11:1;
    vuint32_t FORCMB:1;                /* Force Match B */
    vuint32_t FORCMA:1;                /* Force Match A */
    vuint32_t _unused_14:3;
    vuint32_t FEN:1;                   /* Flag Enable */
    vuint32_t FCK:1;                   /* Filter Clock Select */
    vuint32_t IF:4;                    /* Input Filter */
    vuint32_t _unused_23:1;
    vuint32_t DMA:1;                   /* Direct Memory Access */
    vuint32_t UCPREN:1;                /* Prescaler Enable */
    vuint32_t UCPRE:2;                 /* Prescaler */
    vuint32_t ODISSL:2;                /* Output Disable Select */
    vuint32_t ODIS:1;                  /* Output Disable */
    vuint32_t FREN:1;                  /* Freeze Enable */
  } B;
} EMIOS_CH_UC_C_tag;

typedef union EMIOS_CH_UC_S_union_tag { /* UC Status 0 */
  vuint32_t R;
  struct {
    vuint32_t FLAG:1;                  /* Flag */
    vuint32_t UCOUT:1;                 /* UC Output Pin */
    vuint32_t UCIN:1;                  /* UC Input Pin */
    vuint32_t _unused_3:12;
    vuint32_t OVFL:1;                  /* Overflow */
    vuint32_t _unused_16:15;
    vuint32_t OVR:1;                   /* Overrun */
  } B;
} EMIOS_CH_UC_S_tag;

typedef union EMIOS_CH_UC_ALTA_union_tag { /* Alternate Address 0 */
  vuint32_t R;
  struct {
    vuint32_t ALTA:24;                 /* Alternate Address */
    vuint32_t _unused_24:8;
  } B;
} EMIOS_CH_UC_ALTA_tag;

typedef union EMIOS_CH_UC_C2_union_tag { /* UC Control 2 0 */
  vuint32_t R;
  struct {
    vuint32_t UCRELDEL_INT:5;          /* Reload Signal Output Delay Interval */
    vuint32_t _unused_5:9;
    vuint32_t UCPRECLK:1;              /* Prescaler Clock Source */
    vuint32_t _unused_15:1;
    vuint32_t UCEXTPRE:14;             /* Extended Prescaler */
    vuint32_t _unused_30:2;
  } B;
} EMIOS_CH_UC_C2_tag;

typedef struct EMIOS_CH_UC_struct_tag {
  EMIOS_CH_UC_A_tag A;                 /* UC A 0 */
  EMIOS_CH_UC_B_tag B;                 /* UC B 0 */
  EMIOS_CH_UC_CNT_tag CNT;             /* UC Counter 0 */
  EMIOS_CH_UC_C_tag C;                 /* UC Control 0 */
  EMIOS_CH_UC_S_tag S;                 /* UC Status 0 */
  EMIOS_CH_UC_ALTA_tag ALTA;           /* Alternate Address 0 */
  EMIOS_CH_UC_C2_tag C2;               /* UC Control 2 0 */
  uint8_t UC_reserved0[4];
} EMIOS_CH_UC_tag;

typedef union EMIOS_CH_WSC_WSCAEC_union_tag { /* WSC Capture A and Event Counter 8 */
  vuint32_t R;
  struct {
    vuint32_t T24CAPA:24;              /* Timebase Capture A */
    vuint32_t EVCNT:8;                 /* Event Count */
  } B;
} EMIOS_CH_WSC_WSCAEC_tag;

typedef union EMIOS_CH_WSC_WSCAPB_union_tag { /* WSC Capture B 8 */
  vuint32_t R;
  struct {
    vuint32_t T24CAPB:24;              /* T24CAPB */
    vuint32_t _unused_24:8;
  } B;
} EMIOS_CH_WSC_WSCAPB_tag;

typedef union EMIOS_CH_WSC_WSC1_union_tag { /* WSC Control 1 8 */
  vuint32_t R;
  struct {
    vuint32_t MODE:7;                  /* Mode */
    vuint32_t EDPOLCAP:1;              /* Capture Edge Polarity */
    vuint32_t EDSELCAP:1;              /* Capture Edge Selection */
    vuint32_t BSL:2;                   /* Bus Select */
    vuint32_t _unused_11:6;
    vuint32_t FEN:1;                   /* Flag and Overrun Enable */
    vuint32_t FCK:1;                   /* Filter Clock */
    vuint32_t IF0_3:4;                 /* Input Filter */
    vuint32_t _unused_23:1;
    vuint32_t DMA:1;                   /* Direct Memory Access */
    vuint32_t CPREN:1;                 /* T16PWCNT Counter Prescaler Enable */
    vuint32_t IF4_7:4;                 /* Input Filter */
    vuint32_t _unused_30:1;
    vuint32_t FREN:1;                  /* Freeze Enable */
  } B;
} EMIOS_CH_WSC_WSC1_tag;

typedef union EMIOS_CH_WSC_WSC2_union_tag { /* WSC Control 2 8 */
  vuint32_t R;
  struct {
    vuint32_t FLAGSEL:6;               /* Flag Selection */
    vuint32_t _unused_6:1;
    vuint32_t EDPOLPW:1;               /* Pulse Width Detection Edge Polarity */
    vuint32_t EDSELPW:1;               /* Pulse Width Detection Edge Selection */
    vuint32_t _unused_9:3;
    vuint32_t PWREN:1;                 /* T16PWCNT Reset Enable */
    vuint32_t PWSWR:1;                 /* T16PWCNT Software Reset */
    vuint32_t WSPREMODE:1;             /* WSC Internal Prescaler */
    vuint32_t _unused_15:1;
    vuint32_t WSPRE:12;                /* Prescaler */
    vuint32_t _unused_28:4;
  } B;
} EMIOS_CH_WSC_WSC2_tag;

typedef union EMIOS_CH_WSC_WSS_union_tag { /* WSC Status 8 */
  vuint32_t R;
  struct {
    vuint32_t FLAGCE:1;                /* Compare Event Flag */
    vuint32_t FLAGECO:1;               /* Event Counter Overflow Flag */
    vuint32_t FLAGPW:1;                /* Pulse Width Capture Event Flag */
    vuint32_t FLAGCAP:1;               /* Capture Event Flag */
    vuint32_t FLAGPWO:1;               /* T16PWCNT Pulse Width Counter Overflow Flag */
    vuint32_t FLAGFF:1;                /* FIFO Full */
    vuint32_t _unused_6:2;
    vuint32_t OVRCE:1;                 /* FLAGCE Overrun Flag */
    vuint32_t OVRECO:1;                /* FLAGECO Overrun Flag */
    vuint32_t OVRPW:1;                 /* FLAGPW Overrun Flag */
    vuint32_t OVRCAP:1;                /* FLAGCAP Overrun Flag */
    vuint32_t OVRPWO:1;                /* FLAGPWO Overrun Flag */
    vuint32_t _unused_13:2;
    vuint32_t WSCIN:1;                 /* Wheel Speed Channel Input */
    vuint32_t FLAGCEC:1;               /* FLAGCE Clear */
    vuint32_t FLAGECOC:1;              /* FLAGECO Clear */
    vuint32_t FLAGPWC:1;               /* FLAGPW Clear */
    vuint32_t FLAGCAPC:1;              /* FLAGCAP Clear */
    vuint32_t FLAGPWOC:1;              /* FLAGPWO Clear */
    vuint32_t FLAGFFC:1;               /* FLAGFF Clear */
    vuint32_t _unused_22:2;
    vuint32_t OVRCEC:1;                /* OVRCE Clear */
    vuint32_t OVRECOC:1;               /* OVRECO Clear */
    vuint32_t OVRPWC:1;                /* OVRPW Clear */
    vuint32_t OVRCAPC:1;               /* OVRCAP Clear */
    vuint32_t OVRPWOC:1;               /* OVRPWO Clear */
    vuint32_t _unused_29:3;
  } B;
} EMIOS_CH_WSC_WSS_tag;

typedef union EMIOS_CH_WSC_WSEV_union_tag { /* WSC Event 8 */
  vuint32_t R;
  struct {
    vuint32_t EVENT:8;                 /* EVENT */
    vuint32_t _unused_8:24;
  } B;
} EMIOS_CH_WSC_WSEV_tag;

typedef union EMIOS_CH_WSC_WSCEV_union_tag { /* WSC Capture Event 8 */
  vuint32_t R;
  struct {
    vuint32_t T24CAPEV:24;             /* T24CAPEV */
    vuint32_t _unused_24:8;
  } B;
} EMIOS_CH_WSC_WSCEV_tag;

typedef union EMIOS_CH_WSC_WSPW_union_tag { /* WSC Pulse Width 8 */
  vuint32_t R;
  struct {
    vuint32_t T16PWCAP:16;             /* T16PWCAP */
    vuint32_t _unused_16:16;
  } B;
} EMIOS_CH_WSC_WSPW_tag;

typedef union EMIOS_CH_WSC_WSPWCNT_union_tag { /* WSC Pulse Width Counter 8 */
  vuint32_t R;
  struct {
    vuint32_t T16PWCNT:16;             /* T16PWCNT */
    vuint32_t _unused_16:16;
  } B;
} EMIOS_CH_WSC_WSPWCNT_tag;

typedef union EMIOS_CH_WSC_WSFC_union_tag { /* WSC FIFO Control 8 */
  vuint32_t R;
  struct {
    vuint32_t CLRF:1;                  /* Clear FIFO */
    vuint32_t FOOE:1;                  /* FIFO Overflow Overwrite Enable */
    vuint32_t _unused_2:30;
  } B;
} EMIOS_CH_WSC_WSFC_tag;

typedef union EMIOS_CH_WSC_WSFR_union_tag { /* WSC FIFO Read 8 */
  vuint32_t R;
  struct {
    vuint32_t T24CAPA_T16PWD_VAL:16;   /* Input Capture or Pulse Width Measurement */
    vuint32_t E:1;                     /* Captured Edge Type */
    vuint32_t _unused_17:15;
  } B;
} EMIOS_CH_WSC_WSFR_tag;

typedef union EMIOS_CH_WSC_WSFCNT_union_tag { /* WSC FIFO Counter 8 */
  vuint32_t R;
  struct {
    vuint32_t FCNTR:5;                 /* FIFO Counter */
    vuint32_t _unused_5:27;
  } B;
} EMIOS_CH_WSC_WSFCNT_tag;

typedef union EMIOS_CH_WSC_WSFPNT_union_tag { /* WSC FIFO Pointer 8 */
  vuint32_t R;
  struct {
    vuint32_t FIFORDPT:4;              /* FIFO Read Pointer */
    vuint32_t _unused_4:12;
    vuint32_t FIFOWRPT:4;              /* FIFO Write Pointer */
    vuint32_t _unused_20:12;
  } B;
} EMIOS_CH_WSC_WSFPNT_tag;

typedef struct EMIOS_CH_WSC_struct_tag {
  EMIOS_CH_WSC_WSCAEC_tag WSCAEC;      /* WSC Capture A and Event Counter 8 */
  EMIOS_CH_WSC_WSCAPB_tag WSCAPB;      /* WSC Capture B 8 */
  EMIOS_CH_WSC_WSC1_tag WSC1;          /* WSC Control 1 8 */
  EMIOS_CH_WSC_WSC2_tag WSC2;          /* WSC Control 2 8 */
  EMIOS_CH_WSC_WSS_tag WSS;            /* WSC Status 8 */
  EMIOS_CH_WSC_WSEV_tag WSEV;          /* WSC Event 8 */
  EMIOS_CH_WSC_WSCEV_tag WSCEV;        /* WSC Capture Event 8 */
  EMIOS_CH_WSC_WSPW_tag WSPW;          /* WSC Pulse Width 8 */
  EMIOS_CH_WSC_WSPWCNT_tag WSPWCNT;    /* WSC Pulse Width Counter 8 */
  EMIOS_CH_WSC_WSFC_tag WSFC;          /* WSC FIFO Control 8 */
  EMIOS_CH_WSC_WSFR_tag WSFR;          /* WSC FIFO Read 8 */
  EMIOS_CH_WSC_WSFCNT_tag WSFCNT;      /* WSC FIFO Counter 8 */
  EMIOS_CH_WSC_WSFPNT_tag WSFPNT;      /* WSC FIFO Pointer 8 */
  uint8_t WSC_reserved0[12];
} EMIOS_CH_WSC_tag;

typedef union EMIOS_CH_union_tag {
  EMIOS_CH_UC_tag UC[32];              /* Valid array indices: 0-7, 16-31 */
  EMIOS_CH_WSC_tag WSC[8];             /* Valid array indices: 4-7 */
} EMIOS_CH_tag;

struct EMIOS_tag {
  EMIOS_MCR_tag MCR;                   /* Module Configuration */
  EMIOS_GFLAG_tag GFLAG;               /* Global Flag */
  EMIOS_OUDIS_tag OUDIS;               /* Output Update Disable */
  uint8_t EMIOS_reserved0[20];
  EMIOS_CH_tag CH;
};


/* ============================================================================
   =============================== Module: ENETC_PORT =========================
   ============================================================================ */

typedef union ENETC_PORT_PCAPR_union_tag { /* Port capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t LINK_TYPE:1;
    vuint32_t _unused_5:7;
    vuint32_t NUM_TC:4;
    vuint32_t NUM_Q:4;
    vuint32_t _unused_20:4;
    vuint32_t NUM_CG:4;
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t CBS:1;                   /* Credit Based Shaping */
    vuint32_t _unused_30:2;
  } B;
} ENETC_PORT_PCAPR_tag;

typedef union ENETC_PORT_PMCAPR_union_tag { /* Port MAC capability register */
  vuint32_t R;
  struct {
    vuint32_t MAC_VAR:3;               /* MAC Variant */
    vuint32_t _unused_3:1;
    vuint32_t EFPAD:2;
    vuint32_t _unused_6:2;
    vuint32_t HD:1;
    vuint32_t FP:2;                    /* Indicates if frame preemption is supported */
    vuint32_t _unused_11:13;
    vuint32_t MII_PROT:4;              /* Indicates the MII protocol supported */
    vuint32_t _unused_28:4;
  } B;
} ENETC_PORT_PMCAPR_tag;

typedef union ENETC_PORT_PIOCAPR_union_tag { /* Port I/O capability register */
  vuint32_t R;
  struct {
    vuint32_t PCS_PROT:16;             /* PCS protocols supported */
    vuint32_t _unused_16:7;
    vuint32_t _unused_23:1;
    vuint32_t IO_VAR:4;                /* IO Variants supported */
    vuint32_t EMDIO:1;
    vuint32_t _unused_29:1;
    vuint32_t REVMII_RATE:1;           /* RevMII MII rate */
    vuint32_t REVMII:1;                /* Reverse Mode Device Configuration */
  } B;
} ENETC_PORT_PIOCAPR_tag;

typedef union ENETC_PORT_PCR_union_tag { /* Port configuration register */
  vuint32_t R;
  struct {
    vuint32_t HDR_FMT:1;
    vuint32_t _unused_1:3;
    vuint32_t L2DOSE:1;
    vuint32_t _unused_5:3;
    vuint32_t TIMER_CS:1;
    vuint32_t _unused_9:3;
    vuint32_t FCSEA:1;                 /* FCS Error Action */
    vuint32_t _unused_13:3;
    vuint32_t PSPEED:14;
    vuint32_t _unused_30:2;
  } B;
} ENETC_PORT_PCR_tag;

typedef union ENETC_PORT_PMAR0_union_tag { /* Port MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} ENETC_PORT_PMAR0_tag;

typedef union ENETC_PORT_PMAR1_union_tag { /* Port MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} ENETC_PORT_PMAR1_tag;

typedef union ENETC_PORT_PTAR_union_tag { /* Port TPID acceptance register */
  vuint32_t R;
  struct {
    vuint32_t OVTPIDL:4;
    vuint32_t IVTPIDL:4;
    vuint32_t _unused_8:24;
  } B;
} ENETC_PORT_PTAR_tag;

typedef union ENETC_PORT_PQOSMR_union_tag { /* Port QoS mode register */
  vuint32_t R;
  struct {
    vuint32_t VS:1;
    vuint32_t VE:1;
    vuint32_t DDRf:2;
    vuint32_t DIPV:3;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:8;
    vuint32_t VQMP:4;                  /* Mapping profile index */
    vuint32_t _unused_20:12;
  } B;
} ENETC_PORT_PQOSMR_tag;

typedef union ENETC_PORT_PPCR_union_tag { /* Port parser configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t L1PFS:5;
    vuint32_t _unused_6:3;
    vuint32_t L2PFS:5;
    vuint32_t _unused_14:2;
    vuint32_t L3HFP:1;                 /* L3 header fields present */
    vuint32_t L3PFS:5;                 /* L3 payload fields size in bytes */
    vuint32_t _unused_22:2;
    vuint32_t L4HFP:1;                 /* L4 Header fields present */
    vuint32_t L4PFS:5;                 /* L4 payload fields size in bytes */
    vuint32_t _unused_30:2;
  } B;
} ENETC_PORT_PPCR_tag;

typedef union ENETC_PORT_PIPFCR_union_tag { /* Port ingress port filter configuration register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t _unused_1:31;
  } B;
} ENETC_PORT_PIPFCR_tag;

typedef union ENETC_PORT_PSGCR_union_tag { /* Port stream gate configuration register */
  vuint32_t R;
  struct {
    vuint32_t PDELAY:24;
    vuint32_t _unused_24:7;
    vuint32_t OGC:1;                   /* Stream Gate Open Gate Check mode */
  } B;
} ENETC_PORT_PSGCR_tag;

typedef union ENETC_PORT_POR_union_tag { /* Port operational register */
  vuint32_t R;
  struct {
    vuint32_t TXDIS:1;                 /* Tx Disable. */
    vuint32_t RXDIS:1;                 /* Rx Disable. */
    vuint32_t _unused_2:30;
  } B;
} ENETC_PORT_POR_tag;

typedef union ENETC_PORT_PSR_union_tag { /* Port status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;               /* Transmit busy. */
    vuint32_t RX_BUSY:1;               /* Receive busy. */
    vuint32_t _unused_2:30;
  } B;
} ENETC_PORT_PSR_tag;

typedef union ENETC_PORT_PRXSDUOR_union_tag { /* Port receive SDU overhead register */
  vuint32_t R;
  struct {
    vuint32_t PPDU_BCO:5;              /* PPDU Byte count overhead */
    vuint32_t _unused_5:3;
    vuint32_t MACSEC_BCO:5;            /* MACsec byte count overhead */
    vuint32_t _unused_13:19;
  } B;
} ENETC_PORT_PRXSDUOR_tag;

typedef union ENETC_PORT_PTXSDUOR_union_tag { /* Port transmit SDU overhead register */
  vuint32_t R;
  struct {
    vuint32_t PPDU_BCO:5;              /* PPDU Byte count overhead */
    vuint32_t _unused_5:3;
    vuint32_t MACSEC_BCO:5;            /* MACsec byte count overhead */
    vuint32_t _unused_13:19;
  } B;
} ENETC_PORT_PTXSDUOR_tag;

typedef union ENETC_PORT_PTGSCR_union_tag { /* Port time gate scheduling control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t TGE:1;                   /* Time Gating Enable */
  } B;
} ENETC_PORT_PTGSCR_tag;

typedef union ENETC_PORT_PTGAGLSR_union_tag { /* Port time gate scheduling admin gate list status register */
  vuint32_t R;
  struct {
    vuint32_t TG:1;
    vuint32_t CFG_PEND:1;
    vuint32_t _unused_2:30;
  } B;
} ENETC_PORT_PTGAGLSR_tag;

typedef union ENETC_PORT_PTGAGLLR_union_tag { /* Port time gate scheduling admin gate list length register */
  vuint32_t R;
  struct {
    vuint32_t ADMIN_GATE_LIST_LENGTH:16;
    vuint32_t _unused_16:16;
  } B;
} ENETC_PORT_PTGAGLLR_tag;

typedef union ENETC_PORT_PTGOGLLR_union_tag { /* Port time gating operational gate list length register */
  vuint32_t R;
  struct {
    vuint32_t OPER_GATE_LIST_LENGTH:16;
    vuint32_t _unused_16:16;
  } B;
} ENETC_PORT_PTGOGLLR_tag;

typedef union ENETC_PORT_PTGSATOR_union_tag { /* Port time gate scheduling advance time offset register */
  vuint32_t R;
  struct {
    vuint32_t ADV_TIME_OFFSET:16;
    vuint32_t _unused_16:16;
  } B;
} ENETC_PORT_PTGSATOR_tag;

typedef union ENETC_PORT_PRXDCR_union_tag { /* Port Rx discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} ENETC_PORT_PRXDCR_tag;

typedef union ENETC_PORT_PRXDCRR0_union_tag { /* Port Rx discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCDR:1;
    vuint32_t SMREDR:1;
    vuint32_t RXDISDR:1;
    vuint32_t IPFDR:1;
    vuint32_t RPDR:1;
    vuint32_t ISFDR:1;
    vuint32_t SGCDR:1;
    vuint32_t SGOEDR:1;
    vuint32_t MSDUEDR:1;
    vuint32_t _unused_9:2;
    vuint32_t ITEDR:1;
    vuint32_t ECCEDR:1;
    vuint32_t SIFDR:1;
    vuint32_t L2DOSDR:1;
    vuint32_t _unused_15:1;
    vuint32_t PEDR:1;
    vuint32_t NODESTDR:1;              /* No Destination Discard Reason */
    vuint32_t _unused_18:14;
  } B;
} ENETC_PORT_PRXDCRR0_tag;

typedef union ENETC_PORT_PRXDCRR1_union_tag { /* Port Rx discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:16;
    vuint32_t _unused_16:12;
    vuint32_t TT:4;
  } B;
} ENETC_PORT_PRXDCRR1_tag;

typedef union ENETC_PORT_TCT_NUM_PTGSTCSR_union_tag { /* Port time gate scheduling traffic class 0 status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:15;
    vuint32_t LH_STATE:1;
    vuint32_t _unused_17:15;
  } B;
} ENETC_PORT_TCT_NUM_PTGSTCSR_tag;

typedef union ENETC_PORT_TCT_NUM_PTCTMSDUR_union_tag { /* Port traffic class 0 transmit maximum SDU register */
  vuint32_t R;
  struct {
    vuint32_t MAXSDU:16;               /* Tx Maximum Service Data Unit Size */
    vuint32_t SDU_TYPE:2;              /* SDU type */
    vuint32_t _unused_18:6;
    vuint32_t SF_MAXSDU_DIS:1;
    vuint32_t _unused_25:7;
  } B;
} ENETC_PORT_TCT_NUM_PTCTMSDUR_tag;

typedef union ENETC_PORT_TCT_NUM_PTCCBSR0_union_tag { /* Port transmit traffic class 0 credit based shaper register 0 */
  vuint32_t R;
  struct {
    vuint32_t BW:7;                    /* Bandwidth */
    vuint32_t _unused_7:24;
    vuint32_t CBSE:1;                  /* Credit Based Shaper Enable */
  } B;
} ENETC_PORT_TCT_NUM_PTCCBSR0_tag;

typedef union ENETC_PORT_TCT_NUM_PTCCBSR1_union_tag { /* Port traffic class 0 credit based shaper register 1 */
  vuint32_t R;
  struct {
    vuint32_t HI_CREDIT:32;            /* hicredit (in credit units) */
  } B;
} ENETC_PORT_TCT_NUM_PTCCBSR1_tag;

typedef struct ENETC_PORT_TCT_NUM_struct_tag {
  ENETC_PORT_TCT_NUM_PTGSTCSR_tag PTGSTCSR; /* Port time gate scheduling traffic class 0 status register */
  uint8_t TCT_NUM_reserved0[4];
  ENETC_PORT_TCT_NUM_PTCTMSDUR_tag PTCTMSDUR; /* Port traffic class 0 transmit maximum SDU register */
  uint8_t TCT_NUM_reserved1[4];
  ENETC_PORT_TCT_NUM_PTCCBSR0_tag PTCCBSR0; /* Port transmit traffic class 0 credit based shaper register 0 */
  ENETC_PORT_TCT_NUM_PTCCBSR1_tag PTCCBSR1; /* Port traffic class 0 credit based shaper register 1 */
  uint8_t TCT_NUM_reserved2[8];
} ENETC_PORT_TCT_NUM_tag;

typedef union ENETC_PORT_PPCPDEIMR_union_tag { /* Port PCP DEI mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPCPMP:4;
    vuint32_t _unused_4:3;
    vuint32_t IPCPMPV:1;
    vuint32_t EPCPMP:4;
    vuint32_t _unused_12:3;
    vuint32_t EPCPMPV:1;
    vuint32_t DR0DEI:1;
    vuint32_t DR1DEI:1;
    vuint32_t DR2DEI:1;
    vuint32_t DR3DEI:1;
    vuint32_t DRME:1;
    vuint32_t _unused_21:11;
  } B;
} ENETC_PORT_PPCPDEIMR_tag;

typedef union ENETC_PORT_PISIDCR_union_tag { /* Port ingress stream identification configuration register */
  vuint32_t R;
  struct {
    vuint32_t KCPAIR:1;
    vuint32_t KC0EN:1;                 /* Key Construction 0 Enable */
    vuint32_t KC1EN:1;                 /* Key Construction 1 Enable */
    vuint32_t _unused_3:13;
    vuint32_t ISEID:16;
  } B;
} ENETC_PORT_PISIDCR_tag;

struct ENETC_PORT_tag {
  ENETC_PORT_PCAPR_tag PCAPR;          /* Port capability register */
  ENETC_PORT_PMCAPR_tag PMCAPR;        /* Port MAC capability register */
  ENETC_PORT_PIOCAPR_tag PIOCAPR;      /* Port I/O capability register */
  uint8_t ENETC_PORT_reserved0[4];
  ENETC_PORT_PCR_tag PCR;              /* Port configuration register */
  uint8_t ENETC_PORT_reserved1[12];
  ENETC_PORT_PMAR0_tag PMAR0;          /* Port MAC address register 0 */
  ENETC_PORT_PMAR1_tag PMAR1;          /* Port MAC address register 1 */
  uint8_t ENETC_PORT_reserved2[40];
  ENETC_PORT_PTAR_tag PTAR;            /* Port TPID acceptance register */
  ENETC_PORT_PQOSMR_tag PQOSMR;        /* Port QoS mode register */
  uint8_t ENETC_PORT_reserved3[40];
  ENETC_PORT_PPCR_tag PPCR;            /* Port parser configuration register */
  ENETC_PORT_PIPFCR_tag PIPFCR;        /* Port ingress port filter configuration register */
  uint8_t ENETC_PORT_reserved4[24];
  ENETC_PORT_PSGCR_tag PSGCR;          /* Port stream gate configuration register */
  uint8_t ENETC_PORT_reserved5[92];
  ENETC_PORT_POR_tag PORr;             /* Port operational register */
  ENETC_PORT_PSR_tag PSR;              /* Port status register */
  ENETC_PORT_PRXSDUOR_tag PRXSDUOR;    /* Port receive SDU overhead register */
  ENETC_PORT_PTXSDUOR_tag PTXSDUOR;    /* Port transmit SDU overhead register */
  ENETC_PORT_PTGSCR_tag PTGSCR;        /* Port time gate scheduling control register */
  ENETC_PORT_PTGAGLSR_tag PTGAGLSR;    /* Port time gate scheduling admin gate list status register */
  ENETC_PORT_PTGAGLLR_tag PTGAGLLR;    /* Port time gate scheduling admin gate list length register */
  ENETC_PORT_PTGOGLLR_tag PTGOGLLR;    /* Port time gating operational gate list length register */
  ENETC_PORT_PTGSATOR_tag PTGSATOR;    /* Port time gate scheduling advance time offset register */
  uint8_t ENETC_PORT_reserved6[156];
  ENETC_PORT_PRXDCR_tag PRXDCR;        /* Port Rx discard count register */
  uint8_t ENETC_PORT_reserved7[4];
  ENETC_PORT_PRXDCRR0_tag PRXDCRR0;    /* Port Rx discard count reason register 0 */
  ENETC_PORT_PRXDCRR1_tag PRXDCRR1;    /* Port Rx discard count reason register 1 */
  uint8_t ENETC_PORT_reserved8[48];
  ENETC_PORT_TCT_NUM_tag TCT_NUM[8];
  uint8_t ENETC_PORT_reserved9[312];
  ENETC_PORT_PPCPDEIMR_tag PPCPDEIMR;  /* Port PCP DEI mapping register */
  uint8_t ENETC_PORT_reserved10[36];
  ENETC_PORT_PISIDCR_tag PISIDCR;      /* Port ingress stream identification configuration register */
};


/* ============================================================================
   =============================== Module: ENETC_PSEUDO_MAC_PORT2 =============
   ============================================================================ */

typedef union ENETC_PSEUDO_MAC_PORT2_PPMSR_union_tag { /* Port pseudo MAC status register */
  vuint32_t R;
  struct {
    vuint32_t LSTATE:1;
    vuint32_t _unused_1:7;
    vuint32_t RSTATE:1;
    vuint32_t _unused_9:23;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMSR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMCR_union_tag { /* Port pseudo MAC configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TXPAD:1;                 /* Transmit Padding */
    vuint32_t _unused_17:15;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMCR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMROCR_union_tag { /* Port pseudo MAC receive octets counter */
  vuint32_t R;
  struct {
    vuint32_t ROCT:32;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMROCR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMRUFCR_union_tag { /* Port pseudo MAC receive unicast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t RUCA:32;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMRUFCR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMRMFCR_union_tag { /* Port pseudo MAC receive multicast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t RMCA:32;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMRMFCR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMRBFCR_union_tag { /* Port pseudo MAC receive broadcast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t RBCA:32;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMRBFCR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMTOCR_union_tag { /* Port pseudo MAC transmit octets counter */
  vuint32_t R;
  struct {
    vuint32_t TOCT:32;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMTOCR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMTUFCR_union_tag { /* Port pseudo MAC transmit unicast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t TUCA:32;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMTUFCR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMTMFCR_union_tag { /* Port pseudo MAC transmit multicast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t TMCA:32;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMTMFCR_tag;

typedef union ENETC_PSEUDO_MAC_PORT2_PPMTBFCR_union_tag { /* Port pseudo MAC transmit broadcast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t TBCA:32;
  } B;
} ENETC_PSEUDO_MAC_PORT2_PPMTBFCR_tag;

struct ENETC_PSEUDO_MAC_PORT2_tag {
  ENETC_PSEUDO_MAC_PORT2_PPMSR_tag PPMSR; /* Port pseudo MAC status register */
  uint8_t ENETC_PSEUDO_MAC_PORT2_reserved0[12];
  ENETC_PSEUDO_MAC_PORT2_PPMCR_tag PPMCR; /* Port pseudo MAC configuration register */
  uint8_t ENETC_PSEUDO_MAC_PORT2_reserved1[108];
  ENETC_PSEUDO_MAC_PORT2_PPMROCR_tag PPMROCR[2]; /* Port pseudo MAC receive octets counter */
  ENETC_PSEUDO_MAC_PORT2_PPMRUFCR_tag PPMRUFCR[2]; /* Port pseudo MAC receive unicast frame counter register */
  ENETC_PSEUDO_MAC_PORT2_PPMRMFCR_tag PPMRMFCR[2]; /* Port pseudo MAC receive multicast frame counter register */
  ENETC_PSEUDO_MAC_PORT2_PPMRBFCR_tag PPMRBFCR[2]; /* Port pseudo MAC receive broadcast frame counter register */
  uint8_t ENETC_PSEUDO_MAC_PORT2_reserved2[32];
  ENETC_PSEUDO_MAC_PORT2_PPMTOCR_tag PPMTOCR[2]; /* Port pseudo MAC transmit octets counter */
  ENETC_PSEUDO_MAC_PORT2_PPMTUFCR_tag PPMTUFCR[2]; /* Port pseudo MAC transmit unicast frame counter register */
  ENETC_PSEUDO_MAC_PORT2_PPMTMFCR_tag PPMTMFCR[2]; /* Port pseudo MAC transmit multicast frame counter register */
  ENETC_PSEUDO_MAC_PORT2_PPMTBFCR_tag PPMTBFCR[2]; /* Port pseudo MAC transmit broadcast frame counter register */
};


/* ============================================================================
   =============================== Module: ERM ================================
   ============================================================================ */

typedef union ERM_CR0_union_tag {      /* ERM Configuration Register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ENCIE7:1;                /* ENCIE7 */
    vuint32_t ESCIE7:1;                /* ESCIE7 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t ENCIE6:1;                /* ENCIE6 */
    vuint32_t ESCIE6:1;                /* ESCIE6 */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t ENCIE5:1;                /* ENCIE5 */
    vuint32_t ESCIE5:1;                /* ESCIE5 */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t ENCIE4:1;                /* ENCIE4 */
    vuint32_t ESCIE4:1;                /* ESCIE4 */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t ENCIE3:1;                /* ENCIE3 */
    vuint32_t ESCIE3:1;                /* ESCIE3 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t ENCIE2:1;                /* ENCIE2 */
    vuint32_t ESCIE2:1;                /* ESCIE2 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t ENCIE1:1;                /* ENCIE1 */
    vuint32_t ESCIE1:1;                /* ESCIE1 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t ENCIE0:1;                /* ENCIE0 */
    vuint32_t ESCIE0:1;                /* ESCIE0 */
  } B;
} ERM_CR0_tag;

typedef union ERM_CR1_union_tag {      /* ERM Configuration Register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ENCIE15:1;               /* ENCIE15 */
    vuint32_t ESCIE15:1;               /* ESCIE15 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t ENCIE14:1;               /* ENCIE14 */
    vuint32_t ESCIE14:1;               /* ESCIE14 */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t ENCIE13:1;               /* ENCIE13 */
    vuint32_t ESCIE13:1;               /* ESCIE13 */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t ENCIE12:1;               /* ENCIE12 */
    vuint32_t ESCIE12:1;               /* ESCIE12 */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t ENCIE11:1;               /* ENCIE11 */
    vuint32_t ESCIE11:1;               /* ESCIE11 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t ENCIE10:1;               /* ENCIE10 */
    vuint32_t ESCIE10:1;               /* ESCIE10 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t ENCIE9:1;                /* ENCIE9 */
    vuint32_t ESCIE9:1;                /* ESCIE9 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t ENCIE8:1;                /* ENCIE8 */
    vuint32_t ESCIE8:1;                /* ESCIE8 */
  } B;
} ERM_CR1_tag;

typedef union ERM_CR2_union_tag {      /* ERM Configuration Register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ENCIE23:1;               /* ENCIE23 */
    vuint32_t ESCIE23:1;               /* ESCIE23 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t ENCIE22:1;               /* ENCIE22 */
    vuint32_t ESCIE22:1;               /* ESCIE22 */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t ENCIE21:1;               /* ENCIE21 */
    vuint32_t ESCIE21:1;               /* ESCIE21 */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t ENCIE20:1;               /* ENCIE20 */
    vuint32_t ESCIE20:1;               /* ESCIE20 */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t ENCIE19:1;               /* ENCIE19 */
    vuint32_t ESCIE19:1;               /* ESCIE19 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t ENCIE18:1;               /* ENCIE18 */
    vuint32_t ESCIE18:1;               /* ESCIE18 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t ENCIE17:1;               /* ENCIE17 */
    vuint32_t ESCIE17:1;               /* ESCIE17 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t ENCIE16:1;               /* ENCIE16 */
    vuint32_t ESCIE16:1;               /* ESCIE16 */
  } B;
} ERM_CR2_tag;

typedef union ERM_SR0_union_tag {      /* ERM Status Register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t NCE7:1;                  /* NCE7 */
    vuint32_t SBC7:1;                  /* SBC7 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t NCE6:1;                  /* NCE6 */
    vuint32_t SBC6:1;                  /* SBC6 */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t NCE5:1;                  /* NCE5 */
    vuint32_t SBC5:1;                  /* SBC5 */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t NCE4:1;                  /* NCE4 */
    vuint32_t SBC4:1;                  /* SBC4 */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t NCE3:1;                  /* NCE3 */
    vuint32_t SBC3:1;                  /* SBC3 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t NCE2:1;                  /* NCE2 */
    vuint32_t SBC2:1;                  /* SBC2 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t NCE1:1;                  /* NCE1 */
    vuint32_t SBC1:1;                  /* SBC1 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t NCE0:1;                  /* NCE0 */
    vuint32_t SBC0:1;                  /* SBC0 */
  } B;
} ERM_SR0_tag;

typedef union ERM_SR1_union_tag {      /* ERM Status Register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t NCE15:1;                 /* NCE15 */
    vuint32_t SBC15:1;                 /* SBC15 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t NCE14:1;                 /* NCE14 */
    vuint32_t SBC14:1;                 /* SBC14 */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t NCE13:1;                 /* NCE13 */
    vuint32_t SBC13:1;                 /* SBC13 */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t NCE12:1;                 /* NCE12 */
    vuint32_t SBC12:1;                 /* SBC12 */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t NCE11:1;                 /* NCE11 */
    vuint32_t SBC11:1;                 /* SBC11 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t NCE10:1;                 /* NCE10 */
    vuint32_t SBC10:1;                 /* SBC10 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t NCE9:1;                  /* NCE9 */
    vuint32_t SBC9:1;                  /* SBC9 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t NCE8:1;                  /* NCE8 */
    vuint32_t SBC8:1;                  /* SBC8 */
  } B;
} ERM_SR1_tag;

typedef union ERM_SR2_union_tag {      /* ERM Status Register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t NCE23:1;                 /* NCE23 */
    vuint32_t SBC23:1;                 /* SBC23 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t NCE22:1;                 /* NCE22 */
    vuint32_t SBC22:1;                 /* SBC22 */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t NCE21:1;                 /* NCE21 */
    vuint32_t SBC21:1;                 /* SBC21 */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t NCE20:1;                 /* NCE20 */
    vuint32_t SBC20:1;                 /* SBC20 */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t NCE19:1;                 /* NCE19 */
    vuint32_t SBC19:1;                 /* SBC19 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t NCE18:1;                 /* NCE18 */
    vuint32_t SBC18:1;                 /* SBC18 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t NCE17:1;                 /* NCE17 */
    vuint32_t SBC17:1;                 /* SBC17 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t NCE16:1;                 /* NCE16 */
    vuint32_t SBC16:1;                 /* SBC16 */
  } B;
} ERM_SR2_tag;

typedef union ERM_EAR0_union_tag {     /* ERM Memory 0 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR0_tag;

typedef union ERM_SYN0_union_tag {     /* ERM Memory 0 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN0_tag;

typedef union ERM_CORR_ERR_CNT0_union_tag { /* ERM Memory 0 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT0_tag;

typedef union ERM_EAR1_union_tag {     /* ERM Memory 1 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR1_tag;

typedef union ERM_SYN1_union_tag {     /* ERM Memory 1 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN1_tag;

typedef union ERM_CORR_ERR_CNT1_union_tag { /* ERM Memory 1 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT1_tag;

typedef union ERM_EAR2_union_tag {     /* ERM Memory 2 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR2_tag;

typedef union ERM_SYN2_union_tag {     /* ERM Memory 2 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN2_tag;

typedef union ERM_CORR_ERR_CNT2_union_tag { /* ERM Memory 2 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT2_tag;

typedef union ERM_EAR3_union_tag {     /* ERM Memory 3 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR3_tag;

typedef union ERM_SYN3_union_tag {     /* ERM Memory 3 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN3_tag;

typedef union ERM_CORR_ERR_CNT3_union_tag { /* ERM Memory 3 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT3_tag;

typedef union ERM_EAR4_union_tag {     /* ERM Memory 4 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR4_tag;

typedef union ERM_CORR_ERR_CNT4_union_tag { /* ERM Memory 4 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT4_tag;

typedef union ERM_EAR5_union_tag {     /* ERM Memory 5 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR5_tag;

typedef union ERM_SYN5_union_tag {     /* ERM Memory 5 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN5_tag;

typedef union ERM_CORR_ERR_CNT5_union_tag { /* ERM Memory 5 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT5_tag;

typedef union ERM_EAR6_union_tag {     /* ERM Memory 6 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR6_tag;

typedef union ERM_SYN6_union_tag {     /* ERM Memory 6 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME_EXT:2;
    vuint32_t _unused_2:22;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN6_tag;

typedef union ERM_CORR_ERR_CNT6_union_tag { /* ERM Memory 6 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT6_tag;

typedef union ERM_EAR7_union_tag {     /* ERM Memory 7 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR7_tag;

typedef union ERM_SYN7_union_tag {     /* ERM Memory 7 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME_EXT:2;
    vuint32_t _unused_2:22;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN7_tag;

typedef union ERM_CORR_ERR_CNT7_union_tag { /* ERM Memory 7 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT7_tag;

typedef union ERM_EAR8_union_tag {     /* ERM Memory 8 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR8_tag;

typedef union ERM_SYN8_union_tag {     /* ERM Memory 8 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME_EXT:2;
    vuint32_t _unused_2:22;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN8_tag;

typedef union ERM_CORR_ERR_CNT8_union_tag { /* ERM Memory 8 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT8_tag;

typedef union ERM_EAR9_union_tag {     /* ERM Memory 9 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR9_tag;

typedef union ERM_SYN9_union_tag {     /* ERM Memory 9 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME_EXT:2;
    vuint32_t _unused_2:22;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN9_tag;

typedef union ERM_CORR_ERR_CNT9_union_tag { /* ERM Memory 9 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT9_tag;

typedef union ERM_EAR10_union_tag {    /* ERM Memory 10 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR10_tag;

typedef union ERM_SYN10_union_tag {    /* ERM Memory 10 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN10_tag;

typedef union ERM_CORR_ERR_CNT10_union_tag { /* ERM Memory 10 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT10_tag;

typedef union ERM_EAR11_union_tag {    /* ERM Memory 11 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR11_tag;

typedef union ERM_SYN11_union_tag {    /* ERM Memory 11 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN11_tag;

typedef union ERM_CORR_ERR_CNT11_union_tag { /* ERM Memory 11 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT11_tag;

typedef union ERM_EAR12_union_tag {    /* ERM Memory 12 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR12_tag;

typedef union ERM_SYN12_union_tag {    /* ERM Memory 12 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN12_tag;

typedef union ERM_CORR_ERR_CNT12_union_tag { /* ERM Memory 12 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT12_tag;

typedef union ERM_EAR13_union_tag {    /* ERM Memory 13 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR13_tag;

typedef union ERM_SYN13_union_tag {    /* ERM Memory 13 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN13_tag;

typedef union ERM_CORR_ERR_CNT13_union_tag { /* ERM Memory 13 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT13_tag;

typedef union ERM_EAR14_union_tag {    /* ERM Memory 14 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR14_tag;

typedef union ERM_SYN14_union_tag {    /* ERM Memory 14 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN14_tag;

typedef union ERM_CORR_ERR_CNT14_union_tag { /* ERM Memory 14 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT14_tag;

typedef union ERM_EAR15_union_tag {    /* ERM Memory 15 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR15_tag;

typedef union ERM_CORR_ERR_CNT15_union_tag { /* ERM Memory 15 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT15_tag;

typedef union ERM_EAR16_union_tag {    /* ERM Memory 16 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR16_tag;

typedef union ERM_CORR_ERR_CNT16_union_tag { /* ERM Memory 16 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT16_tag;

typedef union ERM_EAR17_union_tag {    /* ERM Memory 17 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR17_tag;

typedef union ERM_CORR_ERR_CNT17_union_tag { /* ERM Memory 17 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT17_tag;

typedef union ERM_EAR18_union_tag {    /* ERM Memory 18 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR18_tag;

typedef union ERM_SYN18_union_tag {    /* ERM Memory 18 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN18_tag;

typedef union ERM_CORR_ERR_CNT18_union_tag { /* ERM Memory 18 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT18_tag;

typedef union ERM_EAR19_union_tag {    /* ERM Memory 19 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR19_tag;

typedef union ERM_CORR_ERR_CNT19_union_tag { /* ERM Memory 19 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT19_tag;

typedef union ERM_EAR20_union_tag {    /* ERM Memory 20 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR20_tag;

typedef union ERM_SYN20_union_tag {    /* ERM Memory 20 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME_EXT:2;
    vuint32_t _unused_2:22;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_SYN20_tag;

typedef union ERM_CORR_ERR_CNT20_union_tag { /* ERM Memory 20 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT20_tag;

typedef union ERM_EAR21_union_tag {    /* ERM Memory 21 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR21_tag;

typedef union ERM_CORR_ERR_CNT21_union_tag { /* ERM Memory 21 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT21_tag;

typedef union ERM_EAR22_union_tag {    /* ERM Memory 22 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR22_tag;

typedef union ERM_CORR_ERR_CNT22_union_tag { /* ERM Memory 22 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT22_tag;

typedef union ERM_EAR23_union_tag {    /* ERM Memory 23 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_EAR23_tag;

typedef union ERM_CORR_ERR_CNT23_union_tag { /* ERM Memory 23 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_CORR_ERR_CNT23_tag;

struct ERM_tag {
  ERM_CR0_tag CR0;                     /* ERM Configuration Register 0 */
  ERM_CR1_tag CR1;                     /* ERM Configuration Register 1 */
  ERM_CR2_tag CR2;                     /* ERM Configuration Register 2 */
  uint8_t ERM_reserved0[4];
  ERM_SR0_tag SR0;                     /* ERM Status Register 0 */
  ERM_SR1_tag SR1;                     /* ERM Status Register 1 */
  ERM_SR2_tag SR2;                     /* ERM Status Register 2 */
  uint8_t ERM_reserved1[228];
  ERM_EAR0_tag EAR0;                   /* ERM Memory 0 Error Address Register */
  ERM_SYN0_tag SYN0;                   /* ERM Memory 0 Syndrome Register */
  ERM_CORR_ERR_CNT0_tag CORR_ERR_CNT0; /* ERM Memory 0 Correctable Error Count Register */
  uint8_t ERM_reserved2[4];
  ERM_EAR1_tag EAR1;                   /* ERM Memory 1 Error Address Register */
  ERM_SYN1_tag SYN1;                   /* ERM Memory 1 Syndrome Register */
  ERM_CORR_ERR_CNT1_tag CORR_ERR_CNT1; /* ERM Memory 1 Correctable Error Count Register */
  uint8_t ERM_reserved3[4];
  ERM_EAR2_tag EAR2;                   /* ERM Memory 2 Error Address Register */
  ERM_SYN2_tag SYN2;                   /* ERM Memory 2 Syndrome Register */
  ERM_CORR_ERR_CNT2_tag CORR_ERR_CNT2; /* ERM Memory 2 Correctable Error Count Register */
  uint8_t ERM_reserved4[4];
  ERM_EAR3_tag EAR3;                   /* ERM Memory 3 Error Address Register */
  ERM_SYN3_tag SYN3;                   /* ERM Memory 3 Syndrome Register */
  ERM_CORR_ERR_CNT3_tag CORR_ERR_CNT3; /* ERM Memory 3 Correctable Error Count Register */
  uint8_t ERM_reserved5[4];
  ERM_EAR4_tag EAR4;                   /* ERM Memory 4 Error Address Register */
  uint8_t ERM_reserved6[4];
  ERM_CORR_ERR_CNT4_tag CORR_ERR_CNT4; /* ERM Memory 4 Correctable Error Count Register */
  uint8_t ERM_reserved7[4];
  ERM_EAR5_tag EAR5;                   /* ERM Memory 5 Error Address Register */
  ERM_SYN5_tag SYN5;                   /* ERM Memory 5 Syndrome Register */
  ERM_CORR_ERR_CNT5_tag CORR_ERR_CNT5; /* ERM Memory 5 Correctable Error Count Register */
  uint8_t ERM_reserved8[4];
  ERM_EAR6_tag EAR6;                   /* ERM Memory 6 Error Address Register */
  ERM_SYN6_tag SYN6;                   /* ERM Memory 6 Syndrome Register */
  ERM_CORR_ERR_CNT6_tag CORR_ERR_CNT6; /* ERM Memory 6 Correctable Error Count Register */
  uint8_t ERM_reserved9[4];
  ERM_EAR7_tag EAR7;                   /* ERM Memory 7 Error Address Register */
  ERM_SYN7_tag SYN7;                   /* ERM Memory 7 Syndrome Register */
  ERM_CORR_ERR_CNT7_tag CORR_ERR_CNT7; /* ERM Memory 7 Correctable Error Count Register */
  uint8_t ERM_reserved10[4];
  ERM_EAR8_tag EAR8;                   /* ERM Memory 8 Error Address Register */
  ERM_SYN8_tag SYN8;                   /* ERM Memory 8 Syndrome Register */
  ERM_CORR_ERR_CNT8_tag CORR_ERR_CNT8; /* ERM Memory 8 Correctable Error Count Register */
  uint8_t ERM_reserved11[4];
  ERM_EAR9_tag EAR9;                   /* ERM Memory 9 Error Address Register */
  ERM_SYN9_tag SYN9;                   /* ERM Memory 9 Syndrome Register */
  ERM_CORR_ERR_CNT9_tag CORR_ERR_CNT9; /* ERM Memory 9 Correctable Error Count Register */
  uint8_t ERM_reserved12[4];
  ERM_EAR10_tag EAR10;                 /* ERM Memory 10 Error Address Register */
  ERM_SYN10_tag SYN10;                 /* ERM Memory 10 Syndrome Register */
  ERM_CORR_ERR_CNT10_tag CORR_ERR_CNT10; /* ERM Memory 10 Correctable Error Count Register */
  uint8_t ERM_reserved13[4];
  ERM_EAR11_tag EAR11;                 /* ERM Memory 11 Error Address Register */
  ERM_SYN11_tag SYN11;                 /* ERM Memory 11 Syndrome Register */
  ERM_CORR_ERR_CNT11_tag CORR_ERR_CNT11; /* ERM Memory 11 Correctable Error Count Register */
  uint8_t ERM_reserved14[4];
  ERM_EAR12_tag EAR12;                 /* ERM Memory 12 Error Address Register */
  ERM_SYN12_tag SYN12;                 /* ERM Memory 12 Syndrome Register */
  ERM_CORR_ERR_CNT12_tag CORR_ERR_CNT12; /* ERM Memory 12 Correctable Error Count Register */
  uint8_t ERM_reserved15[4];
  ERM_EAR13_tag EAR13;                 /* ERM Memory 13 Error Address Register */
  ERM_SYN13_tag SYN13;                 /* ERM Memory 13 Syndrome Register */
  ERM_CORR_ERR_CNT13_tag CORR_ERR_CNT13; /* ERM Memory 13 Correctable Error Count Register */
  uint8_t ERM_reserved16[4];
  ERM_EAR14_tag EAR14;                 /* ERM Memory 14 Error Address Register */
  ERM_SYN14_tag SYN14;                 /* ERM Memory 14 Syndrome Register */
  ERM_CORR_ERR_CNT14_tag CORR_ERR_CNT14; /* ERM Memory 14 Correctable Error Count Register */
  uint8_t ERM_reserved17[4];
  ERM_EAR15_tag EAR15;                 /* ERM Memory 15 Error Address Register */
  uint8_t ERM_reserved18[4];
  ERM_CORR_ERR_CNT15_tag CORR_ERR_CNT15; /* ERM Memory 15 Correctable Error Count Register */
  uint8_t ERM_reserved19[4];
  ERM_EAR16_tag EAR16;                 /* ERM Memory 16 Error Address Register */
  uint8_t ERM_reserved20[4];
  ERM_CORR_ERR_CNT16_tag CORR_ERR_CNT16; /* ERM Memory 16 Correctable Error Count Register */
  uint8_t ERM_reserved21[4];
  ERM_EAR17_tag EAR17;                 /* ERM Memory 17 Error Address Register */
  uint8_t ERM_reserved22[4];
  ERM_CORR_ERR_CNT17_tag CORR_ERR_CNT17; /* ERM Memory 17 Correctable Error Count Register */
  uint8_t ERM_reserved23[4];
  ERM_EAR18_tag EAR18;                 /* ERM Memory 18 Error Address Register */
  ERM_SYN18_tag SYN18;                 /* ERM Memory 18 Syndrome Register */
  ERM_CORR_ERR_CNT18_tag CORR_ERR_CNT18; /* ERM Memory 18 Correctable Error Count Register */
  uint8_t ERM_reserved24[4];
  ERM_EAR19_tag EAR19;                 /* ERM Memory 19 Error Address Register */
  uint8_t ERM_reserved25[4];
  ERM_CORR_ERR_CNT19_tag CORR_ERR_CNT19; /* ERM Memory 19 Correctable Error Count Register */
  uint8_t ERM_reserved26[4];
  ERM_EAR20_tag EAR20;                 /* ERM Memory 20 Error Address Register */
  ERM_SYN20_tag SYN20;                 /* ERM Memory 20 Syndrome Register */
  ERM_CORR_ERR_CNT20_tag CORR_ERR_CNT20; /* ERM Memory 20 Correctable Error Count Register */
  uint8_t ERM_reserved27[4];
  ERM_EAR21_tag EAR21;                 /* ERM Memory 21 Error Address Register */
  uint8_t ERM_reserved28[4];
  ERM_CORR_ERR_CNT21_tag CORR_ERR_CNT21; /* ERM Memory 21 Correctable Error Count Register */
  uint8_t ERM_reserved29[4];
  ERM_EAR22_tag EAR22;                 /* ERM Memory 22 Error Address Register */
  uint8_t ERM_reserved30[4];
  ERM_CORR_ERR_CNT22_tag CORR_ERR_CNT22; /* ERM Memory 22 Correctable Error Count Register */
  uint8_t ERM_reserved31[4];
  ERM_EAR23_tag EAR23;                 /* ERM Memory 23 Error Address Register */
  uint8_t ERM_reserved32[4];
  ERM_CORR_ERR_CNT23_tag CORR_ERR_CNT23; /* ERM Memory 23 Correctable Error Count Register */
};


/* ============================================================================
   =============================== Module: ERM_GTM ============================
   ============================================================================ */

typedef union ERM_GTM_CR0_union_tag {  /* ERM Configuration Register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ENCIE7:1;                /* ENCIE7 */
    vuint32_t ESCIE7:1;                /* ESCIE7 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t ENCIE6:1;                /* ENCIE6 */
    vuint32_t ESCIE6:1;                /* ESCIE6 */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t ENCIE5:1;                /* ENCIE5 */
    vuint32_t ESCIE5:1;                /* ESCIE5 */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t ENCIE4:1;                /* ENCIE4 */
    vuint32_t ESCIE4:1;                /* ESCIE4 */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t ENCIE3:1;                /* ENCIE3 */
    vuint32_t ESCIE3:1;                /* ESCIE3 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t ENCIE2:1;                /* ENCIE2 */
    vuint32_t ESCIE2:1;                /* ESCIE2 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t ENCIE1:1;                /* ENCIE1 */
    vuint32_t ESCIE1:1;                /* ESCIE1 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t ENCIE0:1;                /* ENCIE0 */
    vuint32_t ESCIE0:1;                /* ESCIE0 */
  } B;
} ERM_GTM_CR0_tag;

typedef union ERM_GTM_CR1_union_tag {  /* ERM Configuration Register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t ENCIE11:1;               /* ENCIE11 */
    vuint32_t ESCIE11:1;               /* ESCIE11 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t ENCIE10:1;               /* ENCIE10 */
    vuint32_t ESCIE10:1;               /* ESCIE10 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t ENCIE9:1;                /* ENCIE9 */
    vuint32_t ESCIE9:1;                /* ESCIE9 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t ENCIE8:1;                /* ENCIE8 */
    vuint32_t ESCIE8:1;                /* ESCIE8 */
  } B;
} ERM_GTM_CR1_tag;

typedef union ERM_GTM_SR0_union_tag {  /* ERM Status Register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t NCE7:1;                  /* NCE7 */
    vuint32_t SBC7:1;                  /* SBC7 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t NCE6:1;                  /* NCE6 */
    vuint32_t SBC6:1;                  /* SBC6 */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t NCE5:1;                  /* NCE5 */
    vuint32_t SBC5:1;                  /* SBC5 */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t NCE4:1;                  /* NCE4 */
    vuint32_t SBC4:1;                  /* SBC4 */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t NCE3:1;                  /* NCE3 */
    vuint32_t SBC3:1;                  /* SBC3 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t NCE2:1;                  /* NCE2 */
    vuint32_t SBC2:1;                  /* SBC2 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t NCE1:1;                  /* NCE1 */
    vuint32_t SBC1:1;                  /* SBC1 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t NCE0:1;                  /* NCE0 */
    vuint32_t SBC0:1;                  /* SBC0 */
  } B;
} ERM_GTM_SR0_tag;

typedef union ERM_GTM_SR1_union_tag {  /* ERM Status Register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;             /* Reserved */
    vuint32_t _unused_4:4;             /* Reserved */
    vuint32_t _unused_8:4;             /* Reserved */
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t NCE11:1;                 /* NCE11 */
    vuint32_t SBC11:1;                 /* SBC11 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t NCE10:1;                 /* NCE10 */
    vuint32_t SBC10:1;                 /* SBC10 */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t NCE9:1;                  /* NCE9 */
    vuint32_t SBC9:1;                  /* SBC9 */
    vuint32_t _unused_28:2;            /* Reserved */
    vuint32_t NCE8:1;                  /* NCE8 */
    vuint32_t SBC8:1;                  /* SBC8 */
  } B;
} ERM_GTM_SR1_tag;

typedef union ERM_GTM_EAR0_union_tag { /* ERM Memory 0 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR0_tag;

typedef union ERM_GTM_SYN0_union_tag { /* ERM Memory 0 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN0_tag;

typedef union ERM_GTM_CORR_ERR_CNT0_union_tag { /* ERM Memory 0 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT0_tag;

typedef union ERM_GTM_EAR1_union_tag { /* ERM Memory 1 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR1_tag;

typedef union ERM_GTM_SYN1_union_tag { /* ERM Memory 1 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN1_tag;

typedef union ERM_GTM_CORR_ERR_CNT1_union_tag { /* ERM Memory 1 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT1_tag;

typedef union ERM_GTM_EAR2_union_tag { /* ERM Memory 2 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR2_tag;

typedef union ERM_GTM_SYN2_union_tag { /* ERM Memory 2 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN2_tag;

typedef union ERM_GTM_CORR_ERR_CNT2_union_tag { /* ERM Memory 2 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT2_tag;

typedef union ERM_GTM_EAR3_union_tag { /* ERM Memory 3 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR3_tag;

typedef union ERM_GTM_SYN3_union_tag { /* ERM Memory 3 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN3_tag;

typedef union ERM_GTM_CORR_ERR_CNT3_union_tag { /* ERM Memory 3 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT3_tag;

typedef union ERM_GTM_EAR4_union_tag { /* ERM Memory 4 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR4_tag;

typedef union ERM_GTM_SYN4_union_tag { /* ERM Memory 4 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN4_tag;

typedef union ERM_GTM_CORR_ERR_CNT4_union_tag { /* ERM Memory 4 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT4_tag;

typedef union ERM_GTM_EAR5_union_tag { /* ERM Memory 5 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR5_tag;

typedef union ERM_GTM_SYN5_union_tag { /* ERM Memory 5 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN5_tag;

typedef union ERM_GTM_CORR_ERR_CNT5_union_tag { /* ERM Memory 5 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT5_tag;

typedef union ERM_GTM_EAR6_union_tag { /* ERM Memory 6 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR6_tag;

typedef union ERM_GTM_SYN6_union_tag { /* ERM Memory 6 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN6_tag;

typedef union ERM_GTM_CORR_ERR_CNT6_union_tag { /* ERM Memory 6 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT6_tag;

typedef union ERM_GTM_EAR7_union_tag { /* ERM Memory 7 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR7_tag;

typedef union ERM_GTM_SYN7_union_tag { /* ERM Memory 7 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN7_tag;

typedef union ERM_GTM_CORR_ERR_CNT7_union_tag { /* ERM Memory 7 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT7_tag;

typedef union ERM_GTM_EAR8_union_tag { /* ERM Memory 8 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR8_tag;

typedef union ERM_GTM_SYN8_union_tag { /* ERM Memory 8 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN8_tag;

typedef union ERM_GTM_CORR_ERR_CNT8_union_tag { /* ERM Memory 8 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT8_tag;

typedef union ERM_GTM_EAR9_union_tag { /* ERM Memory 9 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR9_tag;

typedef union ERM_GTM_SYN9_union_tag { /* ERM Memory 9 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN9_tag;

typedef union ERM_GTM_CORR_ERR_CNT9_union_tag { /* ERM Memory 9 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT9_tag;

typedef union ERM_GTM_EAR10_union_tag { /* ERM Memory 10 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR10_tag;

typedef union ERM_GTM_SYN10_union_tag { /* ERM Memory 10 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN10_tag;

typedef union ERM_GTM_CORR_ERR_CNT10_union_tag { /* ERM Memory 10 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT10_tag;

typedef union ERM_GTM_EAR11_union_tag { /* ERM Memory 11 Error Address Register */
  vuint32_t R;
  struct {
    vuint32_t EAR:32;                  /* EAR */
  } B;
} ERM_GTM_EAR11_tag;

typedef union ERM_GTM_SYN11_union_tag { /* ERM Memory 11 Syndrome Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;            /* Reserved */
    vuint32_t SYNDROME:8;              /* SYNDROME */
  } B;
} ERM_GTM_SYN11_tag;

typedef union ERM_GTM_CORR_ERR_CNT11_union_tag { /* ERM Memory 11 Correctable Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Memory n Correctable Error Count */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} ERM_GTM_CORR_ERR_CNT11_tag;

struct ERM_GTM_tag {
  ERM_GTM_CR0_tag CR0;                 /* ERM Configuration Register 0 */
  ERM_GTM_CR1_tag CR1;                 /* ERM Configuration Register 1 */
  uint8_t ERM_GTM_reserved0[8];
  ERM_GTM_SR0_tag SR0;                 /* ERM Status Register 0 */
  ERM_GTM_SR1_tag SR1;                 /* ERM Status Register 1 */
  uint8_t ERM_GTM_reserved1[232];
  ERM_GTM_EAR0_tag EAR0;               /* ERM Memory 0 Error Address Register */
  ERM_GTM_SYN0_tag SYN0;               /* ERM Memory 0 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT0_tag CORR_ERR_CNT0; /* ERM Memory 0 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved2[4];
  ERM_GTM_EAR1_tag EAR1;               /* ERM Memory 1 Error Address Register */
  ERM_GTM_SYN1_tag SYN1;               /* ERM Memory 1 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT1_tag CORR_ERR_CNT1; /* ERM Memory 1 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved3[4];
  ERM_GTM_EAR2_tag EAR2;               /* ERM Memory 2 Error Address Register */
  ERM_GTM_SYN2_tag SYN2;               /* ERM Memory 2 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT2_tag CORR_ERR_CNT2; /* ERM Memory 2 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved4[4];
  ERM_GTM_EAR3_tag EAR3;               /* ERM Memory 3 Error Address Register */
  ERM_GTM_SYN3_tag SYN3;               /* ERM Memory 3 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT3_tag CORR_ERR_CNT3; /* ERM Memory 3 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved5[4];
  ERM_GTM_EAR4_tag EAR4;               /* ERM Memory 4 Error Address Register */
  ERM_GTM_SYN4_tag SYN4;               /* ERM Memory 4 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT4_tag CORR_ERR_CNT4; /* ERM Memory 4 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved6[4];
  ERM_GTM_EAR5_tag EAR5;               /* ERM Memory 5 Error Address Register */
  ERM_GTM_SYN5_tag SYN5;               /* ERM Memory 5 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT5_tag CORR_ERR_CNT5; /* ERM Memory 5 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved7[4];
  ERM_GTM_EAR6_tag EAR6;               /* ERM Memory 6 Error Address Register */
  ERM_GTM_SYN6_tag SYN6;               /* ERM Memory 6 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT6_tag CORR_ERR_CNT6; /* ERM Memory 6 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved8[4];
  ERM_GTM_EAR7_tag EAR7;               /* ERM Memory 7 Error Address Register */
  ERM_GTM_SYN7_tag SYN7;               /* ERM Memory 7 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT7_tag CORR_ERR_CNT7; /* ERM Memory 7 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved9[4];
  ERM_GTM_EAR8_tag EAR8;               /* ERM Memory 8 Error Address Register */
  ERM_GTM_SYN8_tag SYN8;               /* ERM Memory 8 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT8_tag CORR_ERR_CNT8; /* ERM Memory 8 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved10[4];
  ERM_GTM_EAR9_tag EAR9;               /* ERM Memory 9 Error Address Register */
  ERM_GTM_SYN9_tag SYN9;               /* ERM Memory 9 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT9_tag CORR_ERR_CNT9; /* ERM Memory 9 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved11[4];
  ERM_GTM_EAR10_tag EAR10;             /* ERM Memory 10 Error Address Register */
  ERM_GTM_SYN10_tag SYN10;             /* ERM Memory 10 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT10_tag CORR_ERR_CNT10; /* ERM Memory 10 Correctable Error Count Register */
  uint8_t ERM_GTM_reserved12[4];
  ERM_GTM_EAR11_tag EAR11;             /* ERM Memory 11 Error Address Register */
  ERM_GTM_SYN11_tag SYN11;             /* ERM Memory 11 Syndrome Register */
  ERM_GTM_CORR_ERR_CNT11_tag CORR_ERR_CNT11; /* ERM Memory 11 Correctable Error Count Register */
};


/* ============================================================================
   =============================== Module: ERROR_CORRECTION_CODES =============
   ============================================================================ */

typedef union ERROR_CORRECTION_CODES_D_ECCCOR_union_tag { /* ECC Correctable Error Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t D_ECCCORADD:30;
  } B;
} ERROR_CORRECTION_CODES_D_ECCCOR_tag;

typedef union ERROR_CORRECTION_CODES_D_ECCERR_union_tag { /* ECC Uncorrectable Error Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t D_ECCERRADD:30;
  } B;
} ERROR_CORRECTION_CODES_D_ECCERR_tag;

struct ERROR_CORRECTION_CODES_tag {
  uint8_t ERROR_CORRECTION_CODES_reserved0[4];
  ERROR_CORRECTION_CODES_D_ECCCOR_tag D_ECCCOR; /* ECC Correctable Error Address */
  ERROR_CORRECTION_CODES_D_ECCERR_tag D_ECCERR; /* ECC Uncorrectable Error Address */
};


/* ============================================================================
   =============================== Module: FEED_DMACRC ========================
   ============================================================================ */

typedef union FEED_DMACRC_GEC_union_tag { /* Global Enable CRC Register */
  vuint32_t R;
  struct {
    vuint32_t GBL_EN:1;                /* Global Enable bit */
    vuint32_t _unused_1:6;             /* Reserved */
    vuint32_t SWAP_BYTE:1;             /* Swap Byte */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} FEED_DMACRC_GEC_tag;

typedef union FEED_DMACRC_CONTROL_REGISTER_CTL_union_tag { /* CRC Control Register */
  vuint32_t R;
  struct {
    vuint32_t CH_SEL:5;                /* Channel Select */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t POLY_SEL:3;              /* Polynomial Select */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t CS_XOR:1;
    vuint32_t CS_SWAP:1;
    vuint32_t SWAP_BIT:1;              /* Swap Bit */
    vuint32_t INIT_SEL:1;              /* Initial values of the CRC */
    vuint32_t MODE:3;                  /* CRC Mode */
    vuint32_t _unused_19:12;           /* Reserved */
    vuint32_t EN:1;                    /* CRC Logic */
  } B;
} FEED_DMACRC_CONTROL_REGISTER_CTL_tag;

typedef union FEED_DMACRC_CONTROL_REGISTER_ICRC_union_tag { /* Initial CRC Value Register */
  vuint32_t R;
  struct {
    vuint32_t INI_CRC_VAL:32;          /* Initial CRC Value */
  } B;
} FEED_DMACRC_CONTROL_REGISTER_ICRC_tag;

typedef union FEED_DMACRC_CONTROL_REGISTER_FCRC_union_tag { /* Final CRC Value Register */
  vuint32_t R;
  struct {
    vuint32_t CHKSUM_VAL:32;           /* Final CRC Value */
  } B;
} FEED_DMACRC_CONTROL_REGISTER_FCRC_tag;

typedef struct FEED_DMACRC_CONTROL_REGISTER_struct_tag {
  FEED_DMACRC_CONTROL_REGISTER_CTL_tag CTL; /* CRC Control Register */
  FEED_DMACRC_CONTROL_REGISTER_ICRC_tag ICRC; /* Initial CRC Value Register */
  FEED_DMACRC_CONTROL_REGISTER_FCRC_tag FCRC; /* Final CRC Value Register */
  uint8_t CONTROL_REGISTER_reserved0[4];
} FEED_DMACRC_CONTROL_REGISTER_tag;

struct FEED_DMACRC_tag {
  FEED_DMACRC_GEC_tag GEC;             /* Global Enable CRC Register */
  uint8_t FEED_DMACRC_reserved0[12];
  FEED_DMACRC_CONTROL_REGISTER_tag CONTROL_REGISTER[8];
};


/* ============================================================================
   =============================== Module: FEED_DMA_MP ========================
   ============================================================================ */

typedef union FEED_DMA_MP_CSR_union_tag { /* Management Page Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t EDBG:1;                  /* Enable Debug */
    vuint32_t ERCA:1;                  /* Enable Round Robin Channel Arbitration */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t HAE:1;                   /* Halt After Error */
    vuint32_t HALT:1;                  /* Halt DMA Operations */
    vuint32_t GCLC:1;                  /* Global Channel Linking Control */
    vuint32_t GMRC:1;                  /* Global Master ID Replication Control */
    vuint32_t ECX:1;                   /* Cancel Transfer With Error */
    vuint32_t CX:1;                    /* Cancel Transfer */
    vuint32_t _unused_10:6;            /* Reserved */
    vuint32_t _unused_16:8;            /* Reserved */
    vuint32_t ACTIVE_ID:5;             /* Active Channel ID */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t ACTIVE:1;                /* DMA Active Status */
  } B;
} FEED_DMA_MP_CSR_tag;

typedef union FEED_DMA_MP_ES_union_tag { /* Management Page Error Status */
  vuint32_t R;
  struct {
    vuint32_t DBE:1;                   /* Destination Bus Error */
    vuint32_t SBE:1;                   /* Source Bus Error */
    vuint32_t SGE:1;                   /* Scatter/Gather Configuration Error */
    vuint32_t NCE:1;                   /* NBYTES/CITER Configuration Error */
    vuint32_t DOE:1;                   /* Destination Offset Error */
    vuint32_t DAE:1;                   /* Destination Address Error */
    vuint32_t SOE:1;                   /* Source Offset Error */
    vuint32_t SAE:1;                   /* Source Address Error */
    vuint32_t ECX:1;                   /* Transfer Canceled */
    vuint32_t UCE:1;                   /* Uncorrectable TCD Error During Channel Execution */
    vuint32_t _unused_10:14;           /* Reserved */
    vuint32_t ERRCHN:5;                /* Error Channel Number or Canceled Channel Number */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} FEED_DMA_MP_ES_tag;

typedef union FEED_DMA_MP_INT_union_tag { /* Management Page Interrupt Request Status */
  vuint32_t R;
  struct {
    vuint32_t INT:24;                  /* Interrupt Request Status */
    vuint32_t _unused_24:8;
  } B;
} FEED_DMA_MP_INT_tag;

typedef union FEED_DMA_MP_HRS_union_tag { /* Management Page Hardware Request Status */
  vuint32_t R;
  struct {
    vuint32_t HRS:32;                  /* Hardware Request Status */
  } B;
} FEED_DMA_MP_HRS_tag;

typedef union FEED_DMA_MP_CH_GRPRI_union_tag { /* Channel Arbitration Group */
  vuint32_t R;
  struct {
    vuint32_t GRPRI:5;                 /* Arbitration Group For Channel n */
    vuint32_t _unused_5:27;
  } B;
} FEED_DMA_MP_CH_GRPRI_tag;

struct FEED_DMA_MP_tag {
  FEED_DMA_MP_CSR_tag CSR;             /* Management Page Control */
  FEED_DMA_MP_ES_tag ES;               /* Management Page Error Status */
  FEED_DMA_MP_INT_tag INT;             /* Management Page Interrupt Request Status */
  FEED_DMA_MP_HRS_tag HRS;             /* Management Page Hardware Request Status */
  uint8_t FEED_DMA_MP_reserved0[240];
  FEED_DMA_MP_CH_GRPRI_tag CH_GRPRI[24]; /* Channel Arbitration Group */
};


/* ============================================================================
   =============================== Module: FEED_DMA_TCD =======================
   ============================================================================ */

typedef union FEED_DMA_TCD_TCD_CH_CSR_union_tag { /* Channel Control and Status */
  vuint32_t R;
  struct {
    vuint32_t ERQ:1;                   /* Enable DMA Request */
    vuint32_t EARQ:1;                  /* Enable Asynchronous DMA Request In Stop Mode For Channel */
    vuint32_t EEI:1;                   /* Enable Error Interrupt */
    vuint32_t EBW:1;                   /* Enable Buffered Writes */
    vuint32_t _unused_4:26;            /* Reserved */
    vuint32_t DONE:1;                  /* Channel Done */
    vuint32_t ACTIVE:1;                /* Channel Active */
  } B;
} FEED_DMA_TCD_TCD_CH_CSR_tag;

typedef union FEED_DMA_TCD_TCD_CH_ES_union_tag { /* Channel Error Status */
  vuint32_t R;
  struct {
    vuint32_t DBE:1;                   /* Destination Bus Error */
    vuint32_t SBE:1;                   /* Source Bus Error */
    vuint32_t SGE:1;                   /* Scatter/Gather Configuration Error */
    vuint32_t NCE:1;                   /* NBYTES/CITER Configuration Error */
    vuint32_t DOE:1;                   /* Destination Offset Error */
    vuint32_t DAE:1;                   /* Destination Address Error */
    vuint32_t SOE:1;                   /* Source Offset Error */
    vuint32_t SAE:1;                   /* Source Address Error */
    vuint32_t _unused_8:23;            /* Reserved */
    vuint32_t ERR:1;                   /* Error In Channel */
  } B;
} FEED_DMA_TCD_TCD_CH_ES_tag;

typedef union FEED_DMA_TCD_TCD_CH_INT_union_tag { /* Channel Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t INT:1;                   /* Interrupt Request */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} FEED_DMA_TCD_TCD_CH_INT_tag;

typedef union FEED_DMA_TCD_TCD_CH_SBR_union_tag { /* Channel System Bus */
  vuint32_t R;
  struct {
    vuint32_t MID:6;                   /* Master ID */
    vuint32_t _unused_6:9;             /* Reserved */
    vuint32_t PAL:1;                   /* Privileged Access Level */
    vuint32_t EMI:1;                   /* Enable Master ID Replication */
    vuint32_t ATTR:3;                  /* Attribute Output */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} FEED_DMA_TCD_TCD_CH_SBR_tag;

typedef union FEED_DMA_TCD_TCD_CH_PRI_union_tag { /* Channel Priority */
  vuint32_t R;
  struct {
    vuint32_t APL:3;                   /* Arbitration Priority Level */
    vuint32_t _unused_3:27;            /* Reserved */
    vuint32_t DPA:1;                   /* Disable Preempt Ability */
    vuint32_t ECP:1;                   /* Enable Channel Preemption */
  } B;
} FEED_DMA_TCD_TCD_CH_PRI_tag;

typedef union FEED_DMA_TCD_TCD_SADDR_union_tag { /* TCD Source Address */
  vuint32_t R;
  struct {
    vuint32_t SADDR:32;                /* Source Address */
  } B;
} FEED_DMA_TCD_TCD_SADDR_tag;

typedef union FEED_DMA_TCD_TCD_SOFF_union_tag { /* TCD Signed Source Address Offset */
  vuint16_t R;
  struct {
    vuint16_t SOFF:16;                 /* Source Address Signed Offset */
  } B;
} FEED_DMA_TCD_TCD_SOFF_tag;

typedef union FEED_DMA_TCD_TCD_ATTR_union_tag { /* TCD Transfer Attributes */
  vuint16_t R;
  struct {
    vuint16_t DSIZE:3;                 /* Destination Data Transfer Size */
    vuint16_t DMOD:5;                  /* Destination Address Modulo */
    vuint16_t SSIZE:3;                 /* Source Data Transfer Size */
    vuint16_t SMOD:5;                  /* Source Address Modulo */
  } B;
} FEED_DMA_TCD_TCD_ATTR_tag;

typedef union FEED_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFNO_union_tag { /* TCD Transfer Size Without Minor Loop Offsets */
  vuint32_t R;
  struct {
    vuint32_t NBYTES:30;               /* Number of Bytes To Transfer Per Service Request */
    vuint32_t DMLOE:1;                 /* Destination Minor Loop Offset Enable */
    vuint32_t SMLOE:1;                 /* Source Minor Loop Offset Enable */
  } B;
} FEED_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFNO_tag;

typedef union FEED_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFYES_union_tag { /* TCD Transfer Size with Minor Loop Offsets */
  vuint32_t R;
  struct {
    vuint32_t NBYTES:10;               /* Number of Bytes To Transfer Per Service Request */
    vuint32_t MLOFF:20;                /* Minor Loop Offset */
    vuint32_t DMLOE:1;                 /* Destination Minor Loop Offset Enable */
    vuint32_t SMLOE:1;                 /* Source Minor Loop Offset Enable */
  } B;
} FEED_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFYES_tag;

typedef union FEED_DMA_TCD_TCD_NBYTES_union_tag {
  FEED_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFNO_tag NBYTES_MLOFFNO; /* TCD Transfer Size Without Minor Loop Offsets */
  FEED_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFYES_tag NBYTES_MLOFFYES; /* TCD Transfer Size with Minor Loop Offsets */
} FEED_DMA_TCD_TCD_NBYTES_tag;

typedef union FEED_DMA_TCD_TCD_SLAST_SDA_union_tag { /* TCD Last Source Address Adjustment / Store DADDR Address */
  vuint32_t R;
  struct {
    vuint32_t SLAST_SDA:32;            /* Last Source Address Adjustment / Store DADDR Address */
  } B;
} FEED_DMA_TCD_TCD_SLAST_SDA_tag;

typedef union FEED_DMA_TCD_TCD_DADDR_union_tag { /* TCD Destination Address */
  vuint32_t R;
  struct {
    vuint32_t DADDR:32;                /* Destination Address */
  } B;
} FEED_DMA_TCD_TCD_DADDR_tag;

typedef union FEED_DMA_TCD_TCD_DOFF_union_tag { /* TCD Signed Destination Address Offset */
  vuint16_t R;
  struct {
    vuint16_t DOFF:16;                 /* Destination Address Signed Offset */
  } B;
} FEED_DMA_TCD_TCD_DOFF_tag;

typedef union FEED_DMA_TCD_TCD_CITER_CITER_ELINKNO_union_tag { /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
  vuint16_t R;
  struct {
    vuint16_t CITER:15;                /* Current Major Iteration Count */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} FEED_DMA_TCD_TCD_CITER_CITER_ELINKNO_tag;

typedef union FEED_DMA_TCD_TCD_CITER_CITER_ELINKYES_union_tag { /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
  vuint16_t R;
  struct {
    vuint16_t CITER:9;                 /* Current Major Iteration Count */
    vuint16_t LINKCH:5;                /* Minor Loop Link Channel Number */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} FEED_DMA_TCD_TCD_CITER_CITER_ELINKYES_tag;

typedef union FEED_DMA_TCD_TCD_CITER_union_tag {
  FEED_DMA_TCD_TCD_CITER_CITER_ELINKNO_tag CITER_ELINKNO; /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
  FEED_DMA_TCD_TCD_CITER_CITER_ELINKYES_tag CITER_ELINKYES; /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
} FEED_DMA_TCD_TCD_CITER_tag;

typedef union FEED_DMA_TCD_TCD_DLAST_SGA_union_tag { /* TCD Last Destination Address Adjustment / Scatter Gather Address */
  vuint32_t R;
  struct {
    vuint32_t DLAST_SGA:32;            /* Last Destination Address Adjustment / Scatter Gather Address */
  } B;
} FEED_DMA_TCD_TCD_DLAST_SGA_tag;

typedef union FEED_DMA_TCD_TCD_CSR_union_tag { /* TCD Control and Status */
  vuint16_t R;
  struct {
    vuint16_t START:1;                 /* Channel Start */
    vuint16_t INTMAJOR:1;              /* Enable Interrupt If Major count complete */
    vuint16_t INTHALF:1;               /* Enable Interrupt If Major Counter Half-complete */
    vuint16_t DREQ:1;                  /* Disable Request */
    vuint16_t ESG:1;                   /* Enable Scatter/Gather Processing */
    vuint16_t MAJORELINK:1;            /* Enable Link When Major Loop Complete */
    vuint16_t _unused_6:1;
    vuint16_t ESDA:1;                  /* Enable Store Destination Address */
    vuint16_t MAJORLINKCH:5;           /* Major Loop Link Channel Number */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t BWC:2;                   /* Bandwidth Control */
  } B;
} FEED_DMA_TCD_TCD_CSR_tag;

typedef union FEED_DMA_TCD_TCD_BITER_BITER_ELINKNO_union_tag { /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
  vuint16_t R;
  struct {
    vuint16_t BITER:15;                /* Starting Major Iteration Count */
    vuint16_t ELINK:1;                 /* Enables Link */
  } B;
} FEED_DMA_TCD_TCD_BITER_BITER_ELINKNO_tag;

typedef union FEED_DMA_TCD_TCD_BITER_BITER_ELINKYES_union_tag { /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
  vuint16_t R;
  struct {
    vuint16_t BITER:9;                 /* Starting Major Iteration Count */
    vuint16_t LINKCH:5;                /* Link Channel Number */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} FEED_DMA_TCD_TCD_BITER_BITER_ELINKYES_tag;

typedef union FEED_DMA_TCD_TCD_BITER_union_tag {
  FEED_DMA_TCD_TCD_BITER_BITER_ELINKNO_tag BITER_ELINKNO; /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
  FEED_DMA_TCD_TCD_BITER_BITER_ELINKYES_tag BITER_ELINKYES; /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
} FEED_DMA_TCD_TCD_BITER_tag;

typedef struct FEED_DMA_TCD_TCD_struct_tag {
  FEED_DMA_TCD_TCD_CH_CSR_tag CH_CSR;  /* Channel Control and Status */
  FEED_DMA_TCD_TCD_CH_ES_tag CH_ES;    /* Channel Error Status */
  FEED_DMA_TCD_TCD_CH_INT_tag CH_INT;  /* Channel Interrupt Status */
  FEED_DMA_TCD_TCD_CH_SBR_tag CH_SBR;  /* Channel System Bus */
  FEED_DMA_TCD_TCD_CH_PRI_tag CH_PRI;  /* Channel Priority */
  uint8_t TCD_reserved0[12];
  FEED_DMA_TCD_TCD_SADDR_tag SADDR;    /* TCD Source Address */
  FEED_DMA_TCD_TCD_SOFF_tag SOFF;      /* TCD Signed Source Address Offset */
  FEED_DMA_TCD_TCD_ATTR_tag ATTR;      /* TCD Transfer Attributes */
  FEED_DMA_TCD_TCD_NBYTES_tag NBYTES;
  FEED_DMA_TCD_TCD_SLAST_SDA_tag SLAST_SDA; /* TCD Last Source Address Adjustment / Store DADDR Address */
  FEED_DMA_TCD_TCD_DADDR_tag DADDR;    /* TCD Destination Address */
  FEED_DMA_TCD_TCD_DOFF_tag DOFF;      /* TCD Signed Destination Address Offset */
  FEED_DMA_TCD_TCD_CITER_tag CITER;
  FEED_DMA_TCD_TCD_DLAST_SGA_tag DLAST_SGA; /* TCD Last Destination Address Adjustment / Scatter Gather Address */
  FEED_DMA_TCD_TCD_CSR_tag CSR;        /* TCD Control and Status */
  FEED_DMA_TCD_TCD_BITER_tag BITER;
  uint8_t TCD_reserved1[4032];
} FEED_DMA_TCD_TCD_tag;

struct FEED_DMA_TCD_tag {
  FEED_DMA_TCD_TCD_tag TCD[24];
};


/* ============================================================================
   =============================== Module: FLEXCAN ============================
   ============================================================================ */

typedef union FLEXCAN_MCR_union_tag {  /* Module Configuration */
  vuint32_t R;
  struct {
    vuint32_t MAXMB:7;                 /* Number of the Last Message Buffer */
    vuint32_t _unused_7:1;
    vuint32_t IDAM:2;                  /* ID Acceptance Mode */
    vuint32_t _unused_10:1;
    vuint32_t FDEN:1;                  /* CAN FD Operation Enable */
    vuint32_t AEN:1;                   /* Abort Enable */
    vuint32_t LPRIOEN:1;               /* Local Priority Enable */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t DMA:1;                   /* DMA Enable */
    vuint32_t IRMQ:1;                  /* Individual RX Masking and Queue Enable */
    vuint32_t SRXDIS:1;                /* Self-Reception Disable */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t LPMACK:1;                /* Low-Power Mode Acknowledge */
    vuint32_t WRNEN:1;                 /* Warning Interrupt Enable */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;
    vuint32_t FRZACK:1;                /* Freeze Mode Acknowledge */
    vuint32_t SOFTRST:1;               /* Soft Reset */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t NOTRDY:1;                /* FlexCAN Not Ready */
    vuint32_t HALT:1;                  /* Halt FlexCAN */
    vuint32_t RFEN:1;                  /* Legacy RX FIFO Enable */
    vuint32_t FRZ:1;                   /* Freeze Enable */
    vuint32_t MDIS:1;                  /* Module Disable */
  } B;
} FLEXCAN_MCR_tag;

typedef union FLEXCAN_CTRL1_union_tag { /* Control 1 */
  vuint32_t R;
  struct {
    vuint32_t PROPSEG:3;               /* Propagation Segment */
    vuint32_t LOM:1;                   /* Listen-Only Mode */
    vuint32_t LBUF:1;                  /* Lowest Buffer Transmitted First */
    vuint32_t TSYN:1;                  /* Timer Sync */
    vuint32_t BOFFREC:1;               /* Bus Off Recovery */
    vuint32_t SMP:1;                   /* CAN Bit Sampling */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t RWRNMSK:1;               /* RX Warning Interrupt Mask */
    vuint32_t TWRNMSK:1;               /* TX Warning Interrupt Mask */
    vuint32_t LPB:1;                   /* Loopback Mode */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t ERRMSK:1;                /* Error Interrupt Mask */
    vuint32_t BOFFMSK:1;               /* Bus Off Interrupt Mask */
    vuint32_t PSEG2:3;                 /* Phase Segment 2 */
    vuint32_t PSEG1:3;                 /* Phase Segment 1 */
    vuint32_t RJW:2;                   /* Resync Jump Width */
    vuint32_t PRESDIV:8;               /* Prescaler Division Factor */
  } B;
} FLEXCAN_CTRL1_tag;

typedef union FLEXCAN_TIMER_union_tag { /* Free-Running Timer */
  vuint32_t R;
  struct {
    vuint32_t TIMER:16;                /* Timer Value */
    vuint32_t _unused_16:16;
  } B;
} FLEXCAN_TIMER_tag;

typedef union FLEXCAN_RXMGMASK_union_tag { /* RX Message Buffers Global Mask */
  vuint32_t R;
  struct {
    vuint32_t MG:32;                   /* Global Mask for RX Message Buffers */
  } B;
} FLEXCAN_RXMGMASK_tag;

typedef union FLEXCAN_RX14MASK_union_tag { /* Receive 14 Mask */
  vuint32_t R;
  struct {
    vuint32_t RX14M:32;                /* RX Buffer 14 Mask Bits */
  } B;
} FLEXCAN_RX14MASK_tag;

typedef union FLEXCAN_RX15MASK_union_tag { /* Receive 15 Mask */
  vuint32_t R;
  struct {
    vuint32_t RX15M:32;                /* RX Buffer 15 Mask Bits */
  } B;
} FLEXCAN_RX15MASK_tag;

typedef union FLEXCAN_ECR_union_tag {  /* Error Counter */
  vuint32_t R;
  struct {
    vuint32_t TXERRCNT:8;              /* Transmit Error Counter */
    vuint32_t RXERRCNT:8;              /* Receive Error Counter */
    vuint32_t TXERRCNT_FAST:8;         /* Transmit Error Counter for Fast Bits */
    vuint32_t RXERRCNT_FAST:8;         /* Receive Error Counter for Fast Bits */
  } B;
} FLEXCAN_ECR_tag;

typedef union FLEXCAN_ESR1_union_tag { /* Error and Status 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t ERRINT:1;                /* Error Interrupt Flag */
    vuint32_t BOFFINT:1;               /* Bus Off Interrupt Flag */
    vuint32_t RX:1;                    /* FlexCAN in Reception Flag */
    vuint32_t FLTCONF:2;               /* Fault Confinement State */
    vuint32_t TX:1;                    /* FlexCAN In Transmission */
    vuint32_t IDLE:1;                  /* Idle */
    vuint32_t RXWRN:1;                 /* RX Error Warning Flag */
    vuint32_t TXWRN:1;                 /* TX Error Warning Flag */
    vuint32_t STFERR:1;                /* Stuffing Error Flag */
    vuint32_t FRMERR:1;                /* Form Error Flag */
    vuint32_t CRCERR:1;                /* Cyclic Redundancy Check Error Flag */
    vuint32_t ACKERR:1;                /* Acknowledge Error Flag */
    vuint32_t BIT0ERR:1;               /* Bit0 Error Flag */
    vuint32_t BIT1ERR:1;               /* Bit1 Error Flag */
    vuint32_t RWRNINT:1;               /* RX Warning Interrupt Flag */
    vuint32_t TWRNINT:1;               /* TX Warning Interrupt Flag */
    vuint32_t SYNCH:1;                 /* CAN Synchronization Status Flag */
    vuint32_t BOFFDONEINT:1;           /* Bus Off Done Interrupt Flag */
    vuint32_t ERRINT_FAST:1;           /* Fast Error Interrupt Flag */
    vuint32_t ERROVR:1;                /* Error Overrun Flag */
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:2;
    vuint32_t STFERR_FAST:1;           /* Fast Stuffing Error Flag */
    vuint32_t FRMERR_FAST:1;           /* Fast Form Error Flag */
    vuint32_t CRCERR_FAST:1;           /* Fast Cyclic Redundancy Check Error Flag */
    vuint32_t _unused_29:1;
    vuint32_t BIT0ERR_FAST:1;          /* Fast Bit0 Error Flag */
    vuint32_t BIT1ERR_FAST:1;          /* Fast Bit1 Error Flag */
  } B;
} FLEXCAN_ESR1_tag;

typedef union FLEXCAN_IMASK2_union_tag { /* Interrupt Masks 2 */
  vuint32_t R;
  struct {
    vuint32_t BUF63TO32M:32;           /* Buffer MBi Mask */
  } B;
} FLEXCAN_IMASK2_tag;

typedef union FLEXCAN_IMASK1_union_tag { /* Interrupt Masks 1 */
  vuint32_t R;
  struct {
    vuint32_t BUF31TO0M:32;            /* Buffer MBi Mask */
  } B;
} FLEXCAN_IMASK1_tag;

typedef union FLEXCAN_IFLAG2_union_tag { /* Interrupt Flags 2 */
  vuint32_t R;
  struct {
    vuint32_t BUF63TO32I:32;           /* Buffer MBi Interrupt */
  } B;
} FLEXCAN_IFLAG2_tag;

typedef union FLEXCAN_IFLAG1_union_tag { /* Interrupt Flags 1 */
  vuint32_t R;
  struct {
    vuint32_t BUF0I:1;                 /* Buffer MB0 Interrupt or Clear Legacy FIFO bit */
    vuint32_t BUF4TO1I:4;              /* Buffer MBi Interrupt or Reserved */
    vuint32_t BUF5I:1;                 /* Buffer MB5 Interrupt or Frames available in Legacy RX FIFO */
    vuint32_t BUF6I:1;                 /* Buffer MB6 Interrupt or Legacy RX FIFO Warning */
    vuint32_t BUF7I:1;                 /* Buffer MB7 Interrupt or Legacy RX FIFO Overflow */
    vuint32_t BUF31TO8I:24;            /* Buffer MBi Interrupt */
  } B;
} FLEXCAN_IFLAG1_tag;

typedef union FLEXCAN_CTRL2_union_tag { /* Control 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t _unused_2:3;
    vuint32_t _unused_5:1;
    vuint32_t TSTAMPCAP:2;             /* Timestamp Capture Point */
    vuint32_t MBTSBASE:2;              /* Message Buffer Timestamp Base */
    vuint32_t _unused_10:1;
    vuint32_t EDFLTDIS:1;              /* Edge Filter Disable */
    vuint32_t ISOCANFDEN:1;            /* ISO CAN FD Enable */
    vuint32_t BTE:1;                   /* Bit Timing Expansion Enable */
    vuint32_t PREXCEN:1;               /* Protocol Exception Enable */
    vuint32_t TIMER_SRC:1;             /* Timer Source */
    vuint32_t EACEN:1;                 /* Entire Frame Arbitration Field Comparison Enable for RX Message Buffers */
    vuint32_t RRS:1;                   /* Remote Request Storing */
    vuint32_t MRP:1;                   /* Message Buffers Reception Priority */
    vuint32_t TASD:5;                  /* Transmission Arbitration Start Delay */
    vuint32_t RFFN:4;                  /* Number of Legacy Receive FIFO Filters */
    vuint32_t WRMFRZ:1;                /* Write Access to Memory in Freeze Mode */
    vuint32_t ECRWRE:1;                /* Error Correction Configuration Register Write Enable */
    vuint32_t BOFFDONEMSK:1;           /* Bus Off Done Interrupt Mask */
    vuint32_t ERRMSK_FAST:1;           /* Error Interrupt Mask for Errors Detected in the Data Phase of Fast CAN FD Frames */
  } B;
} FLEXCAN_CTRL2_tag;

typedef union FLEXCAN_ESR2_union_tag { /* Error and Status 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t _unused_12:1;
    vuint32_t IMB:1;                   /* Inactive Message Buffer */
    vuint32_t VPS:1;                   /* Valid Priority Status */
    vuint32_t _unused_15:1;
    vuint32_t LPTM:7;                  /* Lowest Priority TX Message Buffer */
    vuint32_t _unused_23:9;
  } B;
} FLEXCAN_ESR2_tag;

typedef union FLEXCAN_CRCR_union_tag { /* Cyclic Redundancy Check */
  vuint32_t R;
  struct {
    vuint32_t TXCRC:15;                /* Transmitted CRC value */
    vuint32_t _unused_15:1;
    vuint32_t MBCRC:7;                 /* CRC Message Buffer */
    vuint32_t _unused_23:9;
  } B;
} FLEXCAN_CRCR_tag;

typedef union FLEXCAN_RXFGMASK_union_tag { /* Legacy RX FIFO Global Mask */
  vuint32_t R;
  struct {
    vuint32_t FGM:32;                  /* Legacy RX FIFO Global Mask Bits */
  } B;
} FLEXCAN_RXFGMASK_tag;

typedef union FLEXCAN_RXFIR_union_tag { /* Legacy RX FIFO Information */
  vuint32_t R;
  struct {
    vuint32_t IDHIT:9;                 /* Identifier Acceptance Filter Hit Indicator */
    vuint32_t _unused_9:23;
  } B;
} FLEXCAN_RXFIR_tag;

typedef union FLEXCAN_CBT_union_tag {  /* CAN Bit Timing */
  vuint32_t R;
  struct {
    vuint32_t EPSEG2:5;                /* Extended Phase Segment 2 */
    vuint32_t EPSEG1:5;                /* Extended Phase Segment 1 */
    vuint32_t EPROPSEG:6;              /* Extended Propagation Segment */
    vuint32_t ERJW:5;                  /* Extended Resync Jump Width */
    vuint32_t EPRESDIV:10;             /* Extended Prescaler Division Factor */
    vuint32_t BTF:1;                   /* Bit Timing Format Enable */
  } B;
} FLEXCAN_CBT_tag;

typedef union FLEXCAN_IMASK4_union_tag { /* Interrupt Masks 4 */
  vuint32_t R;
  struct {
    vuint32_t BUF127TO96M:32;          /* Buffer MBi Mask */
  } B;
} FLEXCAN_IMASK4_tag;

typedef union FLEXCAN_IMASK3_union_tag { /* Interrupt Masks 3 */
  vuint32_t R;
  struct {
    vuint32_t BUF95TO64M:32;           /* Buffer MBi Mask */
  } B;
} FLEXCAN_IMASK3_tag;

typedef union FLEXCAN_IFLAG4_union_tag { /* Interrupt Flags 4 */
  vuint32_t R;
  struct {
    vuint32_t BUF127TO96:32;           /* Buffer MBi Interrupt */
  } B;
} FLEXCAN_IFLAG4_tag;

typedef union FLEXCAN_IFLAG3_union_tag { /* Interrupt Flags 3 */
  vuint32_t R;
  struct {
    vuint32_t BUF95TO64:32;            /* Buffer MBi Interrupt */
  } B;
} FLEXCAN_IFLAG3_tag;




typedef union CE_CAN_MB_CS_union_tag { /* Message Buffer 0 CS Register */
  vuint32_t R;
  struct {
    vuint32_t TIMESTAMP:16;            /* Free-Running Counter Time stamp. This 16-bit field is a copy of the Free-Running Timer, captured for Tx and Rx frames at the time when the beginning of the Identifier field appears on the CAN bus. */
	vuint32_t DLC:4;                   /* Length of the data to be stored/transmitted. */
	vuint32_t RTR:1;                   /* Remote Transmission Request. One/zero for remote/data frame. */
	vuint32_t IDE:1;                   /* ID Extended. One/zero for extended/standard format frame. */
	vuint32_t SRR:1;                   /* Substitute Remote Request. Contains a fixed recessive bit. */
    vuint32_t _unused_23:1;
    vuint32_t CODE:4;                  /* Message Buffer Code */
    vuint32_t _unused_28:1;
	vuint32_t ESI:1;                   /* Error State Indicator */
	vuint32_t BRS:1;                   /* Bit Rate Switch */
    vuint32_t EDL:1;                   /* Extended Data Length */
  } B;
} CE_CAN_MB_CS_tag;

typedef union CE_CAN_MB_ID_union_tag { /* Message Buffer 0 ID Register */
  vuint32_t R;
  struct {
    vuint32_t ID_EXT:18;               /* Frame Identifier Extended */
    vuint32_t ID_STD:11;               /* Frame Identifier Standard */
    vuint32_t PRIO:3;                  /* Local Priority */
  } B;
} CE_CAN_MB_ID_tag;

typedef union CE_CAN_MB_DATA_union_tag {
    vuint8_t B[8];
    vuint16_t H[4];
    vuint32_t W[2];
} CE_CAN_MB_DATA_tag;

typedef struct CE_CAN_MB_struct_tag {
   CE_CAN_MB_CS_tag CS;                    /* Message Buffer 0 CS Register */
   CE_CAN_MB_ID_tag ID;                    /* Message Buffer 0 ID Register */
   CE_CAN_MB_DATA_tag DATA;
} CE_CAN_MB_tag;


typedef union FLEXCAN_RXIMR_union_tag { /* Receive Individual Mask */
  vuint32_t R;
  struct {
    vuint32_t MI:32;                   /* Individual Mask Bits */
  } B;
} FLEXCAN_RXIMR_tag;

typedef union FLEXCAN_MECR_union_tag { /* Memory Error Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t NCEFAFRZ:1;              /* Noncorrectable Errors in FlexCAN Access Put Chip in Freeze Mode */
    vuint32_t ECCDIS:1;                /* Error Correction Disable */
    vuint32_t RERRDIS:1;               /* Error Report Disable */
    vuint32_t _unused_10:3;
    vuint32_t EXTERRIE:1;              /* Extended Error Injection Enable */
    vuint32_t FAERRIE:1;               /* FlexCAN Access Error Injection Enable */
    vuint32_t HAERRIE:1;               /* Host Access Error Injection Enable */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:11;
    vuint32_t ECRWRDIS:1;              /* Error Configuration Register Write Disable */
  } B;
} FLEXCAN_MECR_tag;

typedef union FLEXCAN_ERRIAR_union_tag { /* Error Injection Address */
  vuint32_t R;
  struct {
    vuint32_t INJADDR_L:2;             /* Error Injection Address Low */
    vuint32_t INJADDR_H:12;            /* Error Injection Address High */
    vuint32_t _unused_14:18;
  } B;
} FLEXCAN_ERRIAR_tag;

typedef union FLEXCAN_ERRIDPR_union_tag { /* Error Injection Data Pattern */
  vuint32_t R;
  struct {
    vuint32_t DFLIP:32;                /* Data Flip Pattern */
  } B;
} FLEXCAN_ERRIDPR_tag;

typedef union FLEXCAN_ERRIPPR_union_tag { /* Error Injection Parity Pattern */
  vuint32_t R;
  struct {
    vuint32_t PFLIP0:5;                /* Parity Flip Pattern for Byte 0 (Least Significant) */
    vuint32_t _unused_5:3;
    vuint32_t PFLIP1:5;                /* Parity Flip Pattern for Byte 1 */
    vuint32_t _unused_13:3;
    vuint32_t PFLIP2:5;                /* Parity Flip Pattern for Byte 2 */
    vuint32_t _unused_21:3;
    vuint32_t PFLIP3:5;                /* Parity Flip Pattern for Byte 3 (Most Significant) */
    vuint32_t _unused_29:3;
  } B;
} FLEXCAN_ERRIPPR_tag;

typedef union FLEXCAN_RERRAR_union_tag { /* Error Report Address */
  vuint32_t R;
  struct {
    vuint32_t ERRADDR:14;              /* Address Where Error Detected */
    vuint32_t _unused_14:2;
    vuint32_t SAID:3;                  /* SAID */
    vuint32_t _unused_19:5;
    vuint32_t NCE:1;                   /* Noncorrectable Error */
    vuint32_t _unused_25:7;
  } B;
} FLEXCAN_RERRAR_tag;

typedef union FLEXCAN_RERRDR_union_tag { /* Error Report Data */
  vuint32_t R;
  struct {
    vuint32_t RDATA:32;                /* Raw Data Word Read from Memory with Error */
  } B;
} FLEXCAN_RERRDR_tag;

typedef union FLEXCAN_RERRSYNR_union_tag { /* Error Report Syndrome */
  vuint32_t R;
  struct {
    vuint32_t SYND0:5;                 /* Error Syndrome for Byte 0 (Least Significant) */
    vuint32_t _unused_5:2;
    vuint32_t BE0:1;                   /* Byte Enabled for Byte 0 (Least Significant) */
    vuint32_t SYND1:5;                 /* Error Syndrome for Byte 1 */
    vuint32_t _unused_13:2;
    vuint32_t BE1:1;                   /* Byte Enabled for Byte 1 */
    vuint32_t SYND2:5;                 /* Error Syndrome for Byte 2 */
    vuint32_t _unused_21:2;
    vuint32_t BE2:1;                   /* Byte Enabled for Byte 2 */
    vuint32_t SYND3:5;                 /* Error Syndrome for Byte 3 (Most Significant) */
    vuint32_t _unused_29:2;
    vuint32_t BE3:1;                   /* Byte Enabled for Byte 3 (Most Significant) */
  } B;
} FLEXCAN_RERRSYNR_tag;

typedef union FLEXCAN_ERRSR_union_tag { /* Error Status */
  vuint32_t R;
  struct {
    vuint32_t CEIOF:1;                 /* Correctable Error Interrupt Overrun Flag */
    vuint32_t _unused_1:1;
    vuint32_t FANCEIOF:1;              /* FlexCAN Access with Noncorrectable Error Interrupt Overrun Flag */
    vuint32_t HANCEIOF:1;              /* Host Access With Noncorrectable Error Interrupt Overrun Flag */
    vuint32_t _unused_4:12;
    vuint32_t CEIF:1;                  /* Correctable Error Interrupt Flag */
    vuint32_t _unused_17:1;
    vuint32_t FANCEIF:1;               /* FlexCAN Access with Noncorrectable Error Interrupt Flag */
    vuint32_t HANCEIF:1;               /* Host Access with Noncorrectable Error Interrupt Flag */
    vuint32_t _unused_20:12;
  } B;
} FLEXCAN_ERRSR_tag;

typedef union FLEXCAN_EPRS_union_tag { /* Enhanced CAN Bit Timing Prescalers */
  vuint32_t R;
  struct {
    vuint32_t ENPRESDIV:10;            /* Extended Nominal Prescaler Division Factor */
    vuint32_t _unused_10:6;
    vuint32_t EDPRESDIV:10;            /* Extended Data Phase Prescaler Division Factor */
    vuint32_t _unused_26:6;
  } B;
} FLEXCAN_EPRS_tag;

typedef union FLEXCAN_ENCBT_union_tag { /* Enhanced Nominal CAN Bit Timing */
  vuint32_t R;
  struct {
    vuint32_t NTSEG1:8;                /* Nominal Time Segment 1 */
    vuint32_t _unused_8:4;
    vuint32_t NTSEG2:7;                /* Nominal Time Segment 2 */
    vuint32_t _unused_19:3;
    vuint32_t NRJW:7;                  /* Nominal Resynchronization Jump Width */
    vuint32_t _unused_29:3;
  } B;
} FLEXCAN_ENCBT_tag;

typedef union FLEXCAN_EDCBT_union_tag { /* Enhanced Data Phase CAN Bit Timing */
  vuint32_t R;
  struct {
    vuint32_t DTSEG1:5;                /* Data Phase Segment 1 */
    vuint32_t _unused_5:7;
    vuint32_t DTSEG2:4;                /* Data Phase Time Segment 2 */
    vuint32_t _unused_16:6;
    vuint32_t DRJW:4;                  /* Data Phase Resynchronization Jump Width */
    vuint32_t _unused_26:6;
  } B;
} FLEXCAN_EDCBT_tag;

typedef union FLEXCAN_ETDC_union_tag { /* Enhanced Transceiver Delay Compensation */
  vuint32_t R;
  struct {
    vuint32_t ETDCVAL:8;               /* Enhanced Transceiver Delay Compensation Value */
    vuint32_t _unused_8:7;
    vuint32_t ETDCFAIL:1;              /* Transceiver Delay Compensation Fail */
    vuint32_t ETDCOFF:7;               /* Enhanced Transceiver Delay Compensation Offset */
    vuint32_t _unused_23:7;
    vuint32_t TDMDIS:1;                /* Transceiver Delay Measurement Disable */
    vuint32_t ETDCEN:1;                /* Transceiver Delay Compensation Enable */
  } B;
} FLEXCAN_ETDC_tag;

typedef union FLEXCAN_FDCTRL_union_tag { /* CAN FD Control */
  vuint32_t R;
  struct {
    vuint32_t TDCVAL:6;                /* Transceiver Delay Compensation Value */
    vuint32_t _unused_6:2;
    vuint32_t TDCOFF:5;                /* Transceiver Delay Compensation Offset */
    vuint32_t _unused_13:1;
    vuint32_t TDCFAIL:1;               /* Transceiver Delay Compensation Fail */
    vuint32_t TDCEN:1;                 /* Transceiver Delay Compensation Enable */
    vuint32_t MBDSR0:2;                /* Message Buffer Data Size for Region 0 */
    vuint32_t _unused_18:1;
    vuint32_t MBDSR1:2;                /* Message Buffer Data Size for Region 1 */
    vuint32_t _unused_21:1;
    vuint32_t MBDSR2:2;                /* Message Buffer Data Size for Region 2 */
    vuint32_t _unused_24:1;
    vuint32_t MBDSR3:2;                /* Message Buffer Data Size for Region 3 */
    vuint32_t _unused_27:4;
    vuint32_t FDRATE:1;                /* Bit Rate Switch Enable */
  } B;
} FLEXCAN_FDCTRL_tag;

typedef union FLEXCAN_FDCBT_union_tag { /* CAN FD Bit Timing */
  vuint32_t R;
  struct {
    vuint32_t FPSEG2:3;                /* Fast Phase Segment 2 */
    vuint32_t _unused_3:2;
    vuint32_t FPSEG1:3;                /* Fast Phase Segment 1 */
    vuint32_t _unused_8:2;
    vuint32_t FPROPSEG:5;              /* Fast Propagation Segment */
    vuint32_t _unused_15:1;
    vuint32_t FRJW:3;                  /* Fast Resync Jump Width */
    vuint32_t _unused_19:1;
    vuint32_t FPRESDIV:10;             /* Fast Prescaler Division Factor */
    vuint32_t _unused_30:2;
  } B;
} FLEXCAN_FDCBT_tag;

typedef union FLEXCAN_FDCRC_union_tag { /* CAN FD CRC */
  vuint32_t R;
  struct {
    vuint32_t FD_TXCRC:21;             /* Extended Transmitted CRC value */
    vuint32_t _unused_21:3;
    vuint32_t FD_MBCRC:7;              /* CRC Message Buffer Number for FD_TXCRC */
    vuint32_t _unused_31:1;
  } B;
} FLEXCAN_FDCRC_tag;

typedef union FLEXCAN_ERFCR_union_tag { /* Enhanced RX FIFO Control */
  vuint32_t R;
  struct {
    vuint32_t ERFWM:5;                 /* Enhanced RX FIFO Watermark */
    vuint32_t _unused_5:3;
    vuint32_t NFE:6;                   /* Number of Enhanced RX FIFO Filter Elements */
    vuint32_t _unused_14:2;
    vuint32_t NEXIF:7;                 /* Number of Extended ID Filter Elements */
    vuint32_t _unused_23:3;
    vuint32_t DMALW:5;                 /* DMA Last Word */
    vuint32_t ERFEN:1;                 /* Enhanced RX FIFO enable */
  } B;
} FLEXCAN_ERFCR_tag;

typedef union FLEXCAN_ERFIER_union_tag { /* Enhanced RX FIFO Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:15;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:12;
    vuint32_t ERFDAIE:1;               /* Enhanced RX FIFO Data Available Interrupt Enable */
    vuint32_t ERFWMIIE:1;              /* Enhanced RX FIFO Watermark Indication Interrupt Enable */
    vuint32_t ERFOVFIE:1;              /* Enhanced RX FIFO Overflow Interrupt Enable */
    vuint32_t ERFUFWIE:1;              /* Enhanced RX FIFO Underflow Interrupt Enable */
  } B;
} FLEXCAN_ERFIER_tag;

typedef union FLEXCAN_ERFSR_union_tag { /* Enhanced RX FIFO Status */
  vuint32_t R;
  struct {
    vuint32_t ERFEL:6;                 /* Enhanced RX FIFO Elements */
    vuint32_t _unused_6:9;
    vuint32_t _unused_15:1;
    vuint32_t ERFF:1;                  /* Enhanced RX FIFO Full Flag */
    vuint32_t ERFE:1;                  /* Enhanced RX FIFO Empty Flag */
    vuint32_t _unused_18:9;
    vuint32_t ERFCLR:1;                /* Enhanced RX FIFO Clear */
    vuint32_t ERFDA:1;                 /* Enhanced RX FIFO Data Available Flag */
    vuint32_t ERFWMI:1;                /* Enhanced RX FIFO Watermark Indication Flag */
    vuint32_t ERFOVF:1;                /* Enhanced RX FIFO Overflow Flag */
    vuint32_t ERFUFW:1;                /* Enhanced RX FIFO Underflow Flag */
  } B;
} FLEXCAN_ERFSR_tag;

typedef union FLEXCAN_HR_TIME_STAMP_union_tag { /* High-Resolution Timestamp */
  vuint32_t R;
  struct {
    vuint32_t TS:32;                   /* High-Resolution Timestamp */
  } B;
} FLEXCAN_HR_TIME_STAMP_tag;

typedef union FLEXCAN_ERFFEL_union_tag { /* Enhanced RX FIFO Filter Element */
  vuint32_t R;
  struct {
    vuint32_t FEL:32;                  /* Filter Element Bits */
  } B;
} FLEXCAN_ERFFEL_tag;

struct FLEXCAN_tag {
  FLEXCAN_MCR_tag MCR;                 /* Module Configuration */
  FLEXCAN_CTRL1_tag CTRL1;             /* Control 1 */
  FLEXCAN_TIMER_tag TIMER;             /* Free-Running Timer */
  uint8_t FLEXCAN_reserved0[4];
  FLEXCAN_RXMGMASK_tag RXMGMASK;       /* RX Message Buffers Global Mask */
  FLEXCAN_RX14MASK_tag RX14MASK;       /* Receive 14 Mask */
  FLEXCAN_RX15MASK_tag RX15MASK;       /* Receive 15 Mask */
  FLEXCAN_ECR_tag ECR;                 /* Error Counter */
  FLEXCAN_ESR1_tag ESR1;               /* Error and Status 1 */
  FLEXCAN_IMASK2_tag IMASK2;           /* Interrupt Masks 2 */
  FLEXCAN_IMASK1_tag IMASK1;           /* Interrupt Masks 1 */
  FLEXCAN_IFLAG2_tag IFLAG2;           /* Interrupt Flags 2 */
  FLEXCAN_IFLAG1_tag IFLAG1;           /* Interrupt Flags 1 */
  FLEXCAN_CTRL2_tag CTRL2;             /* Control 2 */
  FLEXCAN_ESR2_tag ESR2;               /* Error and Status 2 */
  uint8_t FLEXCAN_reserved1[8];
  FLEXCAN_CRCR_tag CRCR;               /* Cyclic Redundancy Check */
  FLEXCAN_RXFGMASK_tag RXFGMASK;       /* Legacy RX FIFO Global Mask */
  FLEXCAN_RXFIR_tag RXFIR;             /* Legacy RX FIFO Information */
  FLEXCAN_CBT_tag CBT;                 /* CAN Bit Timing */
  uint8_t FLEXCAN_reserved2[20];
  FLEXCAN_IMASK4_tag IMASK4;           /* Interrupt Masks 4 */
  FLEXCAN_IMASK3_tag IMASK3;           /* Interrupt Masks 3 */
  FLEXCAN_IFLAG4_tag IFLAG4;           /* Interrupt Flags 4 */
  FLEXCAN_IFLAG3_tag IFLAG3;           /* Interrupt Flags 3 */
  uint8_t FLEXCAN_reserved3[8];
  CE_CAN_MB_tag MB[128];
  FLEXCAN_RXIMR_tag RXIMR[128];        /* Receive Individual Mask */
  uint8_t FLEXCAN_reserved4[96];
  FLEXCAN_MECR_tag MECR;               /* Memory Error Control */
  FLEXCAN_ERRIAR_tag ERRIAR;           /* Error Injection Address */
  FLEXCAN_ERRIDPR_tag ERRIDPR;         /* Error Injection Data Pattern */
  FLEXCAN_ERRIPPR_tag ERRIPPR;         /* Error Injection Parity Pattern */
  FLEXCAN_RERRAR_tag RERRAR;           /* Error Report Address */
  FLEXCAN_RERRDR_tag RERRDR;           /* Error Report Data */
  FLEXCAN_RERRSYNR_tag RERRSYNR;       /* Error Report Syndrome */
  FLEXCAN_ERRSR_tag ERRSR;             /* Error Status */
  uint8_t FLEXCAN_reserved5[240];
  FLEXCAN_EPRS_tag EPRS;               /* Enhanced CAN Bit Timing Prescalers */
  FLEXCAN_ENCBT_tag ENCBT;             /* Enhanced Nominal CAN Bit Timing */
  FLEXCAN_EDCBT_tag EDCBT;             /* Enhanced Data Phase CAN Bit Timing */
  FLEXCAN_ETDC_tag ETDC;               /* Enhanced Transceiver Delay Compensation */
  FLEXCAN_FDCTRL_tag FDCTRL;           /* CAN FD Control */
  FLEXCAN_FDCBT_tag FDCBT;             /* CAN FD Bit Timing */
  FLEXCAN_FDCRC_tag FDCRC;             /* CAN FD CRC */
  FLEXCAN_ERFCR_tag ERFCR;             /* Enhanced RX FIFO Control */
  FLEXCAN_ERFIER_tag ERFIER;           /* Enhanced RX FIFO Interrupt Enable */
  FLEXCAN_ERFSR_tag ERFSR;             /* Enhanced RX FIFO Status */
  uint8_t FLEXCAN_reserved6[24];
  FLEXCAN_HR_TIME_STAMP_tag HR_TIME_STAMP[128]; /* High-Resolution Timestamp */
  uint8_t FLEXCAN_reserved7[8656];
  FLEXCAN_ERFFEL_tag ERFFEL[128];      /* Enhanced RX FIFO Filter Element */
};


/* ============================================================================
   =============================== Module: FLEXRAY ============================
   ============================================================================ */

typedef union FLEXRAY_MVR_union_tag {  /* Module Version */
  vuint16_t R;
  struct {
    vuint16_t PEVER:8;                 /* PE Version Number */
    vuint16_t CHIVER:8;                /* CHI Version Number */
  } B;
} FLEXRAY_MVR_tag;

typedef union FLEXRAY_MCR_union_tag {  /* Module Configuration */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t BITRATE:3;               /* FlexRay Bus Bit Rate */
    vuint16_t _unused_4:1;
    vuint16_t _unused_5:1;
    vuint16_t FAM:1;                   /* FIFO Address Mode */
    vuint16_t FUM:1;                   /* FIFO Update Mode */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t ECCE:1;                  /* ECC Functionality Enable */
    vuint16_t SFFE:1;                  /* Synchronization Frame Filter Enable */
    vuint16_t CHA:1;                   /* Channel A Enable */
    vuint16_t CHB:1;                   /* Channel B Enable */
    vuint16_t SCM:1;                   /* Single Channel Device Mode */
    vuint16_t SBFF:1;                  /* System Bus Failure Freeze */
    vuint16_t MEN:1;                   /* Module Enable */
  } B;
} FLEXRAY_MCR_tag;

typedef union FLEXRAY_SYMBADHR_union_tag { /* System Memory Base Address High */
  vuint16_t R;
  struct {
    vuint16_t SMBA:16;                 /* System Memory Base Address High */
  } B;
} FLEXRAY_SYMBADHR_tag;

typedef union FLEXRAY_SYMBADLR_union_tag { /* System Memory Base Address Low */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t SMBA:12;                 /* System Memory Base Address Low */
  } B;
} FLEXRAY_SYMBADLR_tag;

typedef union FLEXRAY_STBSCR_union_tag { /* Strobe Signal Control */
  vuint16_t R;
  struct {
    vuint16_t STBPSEL:2;               /* Strobe Port Select */
    vuint16_t _unused_2:2;
    vuint16_t ENB:1;                   /* Strobe Signal Enable */
    vuint16_t _unused_5:3;
    vuint16_t SEL:4;                   /* Strobe Signal Select */
    vuint16_t _unused_12:3;
    vuint16_t WMD:1;                   /* Write Mode */
  } B;
} FLEXRAY_STBSCR_tag;

typedef union FLEXRAY_MBDSR_union_tag { /* MB Data Size */
  vuint16_t R;
  struct {
    vuint16_t MBSEG1DS:7;              /* MB Segment 1 Data Size */
    vuint16_t _unused_7:1;
    vuint16_t MBSEG2DS:7;              /* MB Segment 2 Data Size */
    vuint16_t _unused_15:1;
  } B;
} FLEXRAY_MBDSR_tag;

typedef union FLEXRAY_MBSSUTR_union_tag { /* MB Segment Size and Utilization */
  vuint16_t R;
  struct {
    vuint16_t LAST_MB_UTIL:8;          /* Last MB Utilized */
    vuint16_t LAST_MB_SEG1:8;          /* Last MB In Segment 1 */
  } B;
} FLEXRAY_MBSSUTR_tag;

typedef union FLEXRAY_PEDRAR_union_tag { /* PE DRAM Access */
  vuint16_t R;
  struct {
    vuint16_t DAD:1;                   /* PE DRAM Access Done */
    vuint16_t ADDR:11;                 /* PE DRAM Access Address */
    vuint16_t INST:4;                  /* PE DRAM Access Instruction */
  } B;
} FLEXRAY_PEDRAR_tag;

typedef union FLEXRAY_PEDRDR_union_tag { /* PE DRAM Data */
  vuint16_t R;
  struct {
    vuint16_t DATA:16;                 /* Data To Or From PE DRAM */
  } B;
} FLEXRAY_PEDRDR_tag;

typedef union FLEXRAY_POCR_union_tag { /* Protocol Operation Control */
  vuint16_t R;
  struct {
    vuint16_t POCCMD:4;                /* Protocol Control Command */
    vuint16_t _unused_4:3;
    vuint16_t BSY_WMC:1;               /* Protocol Control Command Busy Or Command Write */
    vuint16_t ERC_AP:2;                /* External Rate Correction Application */
    vuint16_t EOC_AP:2;                /* External Offset Correction Application */
    vuint16_t _unused_12:3;
    vuint16_t WME:1;                   /* Write Mode External Correction */
  } B;
} FLEXRAY_POCR_tag;

typedef union FLEXRAY_GIFER_union_tag { /* Global Interrupt Flag And Enable */
  vuint16_t R;
  struct {
    vuint16_t TBIE:1;                  /* Transmit MB Interrupt Enable */
    vuint16_t RBIE:1;                  /* Receive MB Interrupt Enable */
    vuint16_t FAFAIE:1;                /* Receive FIFO Channel A Almost Full Interrupt Enable */
    vuint16_t FAFBIE:1;                /* Receive FIFO Channel B Almost Full Interrupt Enable */
    vuint16_t WUPIE:1;                 /* Wakeup Interrupt Enable */
    vuint16_t CHIE:1;                  /* CHI Interrupt Enable */
    vuint16_t PRIE:1;                  /* Protocol Interrupt Enable */
    vuint16_t MIE:1;                   /* Module Interrupt Enable */
    vuint16_t TBIF:1;                  /* Transmit MB Interrupt Flag */
    vuint16_t RBIF:1;                  /* Receive MB Interrupt Flag */
    vuint16_t FAFAIF:1;                /* Receive FIFO Channel A Almost Full Interrupt Flag */
    vuint16_t FAFBIF:1;                /* Receive FIFO Channel B Almost Full Interrupt Flag */
    vuint16_t WUPIF:1;                 /* Wakeup Interrupt Flag */
    vuint16_t CHIF:1;                  /* CHI Interrupt Flag */
    vuint16_t PRIF:1;                  /* Protocol Interrupt Flag */
    vuint16_t MIF:1;                   /* Module Interrupt Flag */
  } B;
} FLEXRAY_GIFER_tag;

typedef union FLEXRAY_PIFR0_union_tag { /* Protocol Interrupt Flag 0 */
  vuint16_t R;
  struct {
    vuint16_t CYS_IF:1;                /* Cycle Start Interrupt Flag */
    vuint16_t TI1_IF:1;                /* Timer 1 Expired Interrupt Flag */
    vuint16_t TI2_IF:1;                /* Timer 2 Expired Interrupt Flag */
    vuint16_t TBVA_IF:1;               /* Transmission Across Boundary On Channel A Interrupt Flag */
    vuint16_t TBVB_IF:1;               /* Transmission Across Boundary On Channel B Interrupt Flag */
    vuint16_t LTXA_IF:1;               /* pLatestTx Violation on Channel A Interrupt Flag */
    vuint16_t LTXB_IF:1;               /* pLatestTx Violation on Channel B Interrupt Flag */
    vuint16_t MTX_IF:1;                /* Media Access Test Symbol Received Interrupt Flag */
    vuint16_t MXS_IF:1;                /* Max Sync Frames Detected Interrupt Flag */
    vuint16_t CCL_IF:1;                /* Clock Correction Limit Reached Interrupt Flag */
    vuint16_t MOC_IF:1;                /* Missing Offset Correction Interrupt Flag */
    vuint16_t MRC_IF:1;                /* Missing Rate Correction Interrupt Flag */
    vuint16_t CSA_IF:1;                /* Cold Start Abort Interrupt Flag */
    vuint16_t ILCF_IF:1;               /* Illegal Protocol Configuration Interrupt Flag */
    vuint16_t INTL_IF:1;               /* Internal Protocol Error Interrupt Flag */
    vuint16_t FATL_IF:1;               /* Fatal Protocol Error Interrupt Flag */
  } B;
} FLEXRAY_PIFR0_tag;

typedef union FLEXRAY_PIFR1_union_tag { /* Protocol Interrupt Flag 1 */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t ODT_IF:1;                /* Odd Cycle Table Written Interrupt Flag */
    vuint16_t EVT_IF:1;                /* Even Cycle Table Written Interrupt Flag */
    vuint16_t _unused_6:2;
    vuint16_t SSI0_IF:1;               /* Slot Status Counter Incremented Interrupt Flag 0 */
    vuint16_t SSI1_IF:1;               /* Slot Status Counter Incremented Interrupt Flag 1 */
    vuint16_t SSI2_IF:1;               /* Slot Status Counter Incremented Interrupt Flag 2 */
    vuint16_t SSI3_IF:1;               /* Slot Status Counter Incremented Interrupt Flag 3 */
    vuint16_t PSC_IF:1;                /* Protocol State Changed Interrupt Flag */
    vuint16_t PECF_IF:1;               /* PE Communication Failure Interrupt Flag */
    vuint16_t IPC_IF:1;                /* Illegal Protocol Control Command Interrupt Flag */
    vuint16_t EMC_IF:1;                /* Error Mode Changed Interrupt Flag */
  } B;
} FLEXRAY_PIFR1_tag;

typedef union FLEXRAY_PIER0_union_tag { /* Protocol Interrupt Enable 0 */
  vuint16_t R;
  struct {
    vuint16_t CYS_IE:1;                /* Cycle Start Interrupt Enable */
    vuint16_t TI1_IE:1;                /* Timer 1 Expired Interrupt Enable */
    vuint16_t TI2_IE:1;                /* Timer 2 Expired Interrupt Enable */
    vuint16_t TBVA_IE:1;               /* Transmission Across Boundary On Channel A Interrupt Enable */
    vuint16_t TBVB_IE:1;               /* Transmission Across Boundary On Channel B Interrupt Enable */
    vuint16_t LTXA_IE:1;               /* pLatestTx Violation On Channel A Interrupt Enable */
    vuint16_t LTXB_IE:1;               /* pLatestTx Violation on Channel B Interrupt Enable */
    vuint16_t MTX_IE:1;                /* Media Access Test Symbol Received Interrupt Enable */
    vuint16_t MXS_IE:1;                /* Max Sync Frames Detected Interrupt Enable */
    vuint16_t CCL_IE:1;                /* Clock Correction Limit Reached Interrupt Enable */
    vuint16_t MOC_IE:1;                /* Missing Offset Correction Interrupt Enable */
    vuint16_t MRC_IE:1;                /* Missing Rate Correction Interrupt Enable */
    vuint16_t CSA_IE:1;                /* Cold Start Abort Interrupt Enable */
    vuint16_t ILCF_IE:1;               /* Illegal Protocol Configuration Interrupt Enable */
    vuint16_t INTL_IE:1;               /* Internal Protocol Error Interrupt Enable */
    vuint16_t FATL_IE:1;               /* Fatal Protocol Error Interrupt Enable */
  } B;
} FLEXRAY_PIER0_tag;

typedef union FLEXRAY_PIER1_union_tag { /* Protocol Interrupt Enable 1 */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t ODT_IE:1;                /* Odd Cycle Table Written Interrupt Enable */
    vuint16_t EVT_IE:1;                /* Even Cycle Table Written Interrupt Enable */
    vuint16_t _unused_6:2;
    vuint16_t SSI0_IE:1;               /* Slot Status Counter Incremented Interrupt Enable */
    vuint16_t SSI1_IE:1;               /* Slot Status Counter Incremented Interrupt Enable */
    vuint16_t SSI2_IE:1;               /* Slot Status Counter Incremented Interrupt Enable */
    vuint16_t SSI3_IE:1;               /* Slot Status Counter Incremented Interrupt Enable */
    vuint16_t PSC_IE:1;                /* Protocol State Changed Interrupt Enable */
    vuint16_t PECF_IE:1;               /* PE Communication Failure Interrupt Enable */
    vuint16_t IPC_IE:1;                /* Illegal Protocol Control Command Interrupt Enable */
    vuint16_t EMC_IE:1;                /* Error Mode Changed Interrupt Enable */
  } B;
} FLEXRAY_PIER1_tag;

typedef union FLEXRAY_CHIERFR_union_tag { /* CHI Error Flag */
  vuint16_t R;
  struct {
    vuint16_t ILSA_EF:1;               /* Illegal System Bus Address Error Flag */
    vuint16_t NMF_EF:1;                /* Network Management Frame Error Flag */
    vuint16_t NML_EF:1;                /* Network Management Length Error Flag */
    vuint16_t SPL_EF:1;                /* Static Payload Length Error Flag */
    vuint16_t DPL_EF:1;                /* Dynamic Payload Length Error Flag */
    vuint16_t FID_EF:1;                /* FID Error Flag */
    vuint16_t SBCF_EF:1;               /* System Bus Communication Failure Error Flag */
    vuint16_t _unused_7:1;
    vuint16_t LCK_EF:1;                /* Lock Error Flag */
    vuint16_t MBU_EF:1;                /* MB Usage Error Flag */
    vuint16_t MBS_EF:1;                /* MB Search Error Flag */
    vuint16_t FOVA_EF:1;               /* Receive FIFO Overrun Channel A Error Flag */
    vuint16_t FOVB_EF:1;               /* Receive FIFO Overrun Channel B Error Flag */
    vuint16_t PCMI_EF:1;               /* Protocol Command Ignored Error Flag */
    vuint16_t FRLA_EF:1;               /* Frame Lost Channel A Error Flag */
    vuint16_t FRLB_EF:1;               /* Frame Lost Channel B Error Flag */
  } B;
} FLEXRAY_CHIERFR_tag;

typedef union FLEXRAY_MBIVEC_union_tag { /* MB Interrupt Vector */
  vuint16_t R;
  struct {
    vuint16_t RBIVEC:8;                /* Receive Buffer Interrupt Vector */
    vuint16_t TBIVEC:8;                /* Transmit Buffer Interrupt Vector */
  } B;
} FLEXRAY_MBIVEC_tag;

typedef union FLEXRAY_CASERCR_union_tag { /* Channel A Status Error Counter Register */
  vuint16_t R;
  struct {
    vuint16_t CHAERSCNT:16;            /* Channel A Status Error Counter */
  } B;
} FLEXRAY_CASERCR_tag;

typedef union FLEXRAY_CBSERCR_union_tag { /* Channel B Status Error Counter */
  vuint16_t R;
  struct {
    vuint16_t CHBERSCNT:16;            /* Channel B Status Error Counter */
  } B;
} FLEXRAY_CBSERCR_tag;

typedef union FLEXRAY_PSR0_union_tag { /* Protocol Status 0 */
  vuint16_t R;
  struct {
    vuint16_t WAKEUPSTATUS:3;          /* Wakeup Status */
    vuint16_t _unused_3:1;
    vuint16_t STARTUPSTATE:4;          /* Startup State */
    vuint16_t PROTSTATE:3;             /* Protocol State */
    vuint16_t _unused_11:1;
    vuint16_t SLOTMODE:2;              /* Slot Mode */
    vuint16_t ERRMODE:2;               /* Error Mode */
  } B;
} FLEXRAY_PSR0_tag;

typedef union FLEXRAY_PSR1_union_tag { /* Protocol Status 1 */
  vuint16_t R;
  struct {
    vuint16_t APTAC:5;                 /* Allow Passive To Active Counter */
    vuint16_t FRZ:1;                   /* Freeze Occurred */
    vuint16_t HHR:1;                   /* Host Halt Request Pending */
    vuint16_t CPN:1;                   /* Leading Cold Start Path Noise */
    vuint16_t REMCSAT:5;               /* Remaining Coldstart Attempts */
    vuint16_t _unused_13:1;
    vuint16_t CSP:1;                   /* Leading Cold Start Path */
    vuint16_t CSAA:1;                  /* Cold Start Attempt Aborted Flag */
  } B;
} FLEXRAY_PSR1_tag;

typedef union FLEXRAY_PSR2_union_tag { /* Protocol Status 2 */
  vuint16_t R;
  struct {
    vuint16_t CKCORFCNT:4;             /* Clock Correction Failed Counter */
    vuint16_t MTA:1;                   /* MTS Received On Channel A */
    vuint16_t SSEA:1;                  /* Symbol Window Syntax Error On Channel A */
    vuint16_t SBVA:1;                  /* Symbol Window Boundary Violation On Channel A */
    vuint16_t STCA:1;                  /* Symbol Window Transmit Conflict On Channel A */
    vuint16_t NSEA:1;                  /* NIT Syntax Error On Channel A */
    vuint16_t NBVA:1;                  /* NIT Boundary Violation On Channel A */
    vuint16_t MTB:1;                   /* MTS Received On Channel B */
    vuint16_t SSEB:1;                  /* Symbol Window Syntax Error On Channel B */
    vuint16_t SBVB:1;                  /* Symbol Window Boundary Violation On Channel B */
    vuint16_t STCB:1;                  /* Symbol Window Transmit Conflict On Channel B */
    vuint16_t NSEB:1;                  /* NIT Syntax Error On Channel B */
    vuint16_t NBVB:1;                  /* NIT Boundary Violation On Channel B */
  } B;
} FLEXRAY_PSR2_tag;

typedef union FLEXRAY_PSR3_union_tag { /* Protocol Status 3 */
  vuint16_t R;
  struct {
    vuint16_t AVFA:1;                  /* Aggregated Valid Frame On Channel A */
    vuint16_t ASEA:1;                  /* Aggregated Syntax Error On Channel A */
    vuint16_t ACEA:1;                  /* Aggregated Content Error On Channel A */
    vuint16_t AACA:1;                  /* Aggregated Additional Communication On Channel A */
    vuint16_t ABVA:1;                  /* Aggregated Boundary Violation On Channel A */
    vuint16_t WUA:1;                   /* Wakeup Symbol Received On Channel A */
    vuint16_t _unused_6:2;
    vuint16_t AVFB:1;                  /* Aggregated Valid Frame On Channel B */
    vuint16_t ASEB:1;                  /* Aggregated Syntax Error On Channel B */
    vuint16_t ACEB:1;                  /* Aggregated Content Error On Channel B */
    vuint16_t AACB:1;                  /* Aggregated Additional Communication On Channel B */
    vuint16_t ABVB:1;                  /* Aggregated Boundary Violation On Channel B */
    vuint16_t WUB:1;                   /* Wakeup Symbol Received On Channel B */
    vuint16_t _unused_14:2;
  } B;
} FLEXRAY_PSR3_tag;

typedef union FLEXRAY_MTCTR_union_tag { /* MT Counter */
  vuint16_t R;
  struct {
    vuint16_t MTCT:14;                 /* MT Counter */
    vuint16_t _unused_14:2;
  } B;
} FLEXRAY_MTCTR_tag;

typedef union FLEXRAY_CYCTR_union_tag { /* Cycle Counter */
  vuint16_t R;
  struct {
    vuint16_t CYCCNT:6;                /* Cycle Counter */
    vuint16_t _unused_6:10;
  } B;
} FLEXRAY_CYCTR_tag;

typedef union FLEXRAY_SLTCTAR_union_tag { /* Slot Counter Channel A */
  vuint16_t R;
  struct {
    vuint16_t SLOTCNTA:11;             /* Slot Counter Value For Channel A */
    vuint16_t _unused_11:5;
  } B;
} FLEXRAY_SLTCTAR_tag;

typedef union FLEXRAY_SLTCTBR_union_tag { /* Slot Counter Channel B */
  vuint16_t R;
  struct {
    vuint16_t SLOTCNTB:11;             /* Slot Counter Value For Channel B */
    vuint16_t _unused_11:5;
  } B;
} FLEXRAY_SLTCTBR_tag;

typedef union FLEXRAY_RTCORVR_union_tag { /* Rate Correction Value */
  vuint16_t R;
  struct {
    vuint16_t RATECORR:16;             /* Rate Correction Value */
  } B;
} FLEXRAY_RTCORVR_tag;

typedef union FLEXRAY_OFCORVR_union_tag { /* Offset Correction Value */
  vuint16_t R;
  struct {
    vuint16_t OFFSETCORR:16;           /* Offset Correction Value */
  } B;
} FLEXRAY_OFCORVR_tag;

typedef union FLEXRAY_CIFR_union_tag { /* Combined Interrupt Flag */
  vuint16_t R;
  struct {
    vuint16_t TBIF:1;                  /* Transmit MB Interrupt Flag */
    vuint16_t RBIF:1;                  /* Receive MB Interrupt Flag */
    vuint16_t FAFAIF:1;                /* Receive FIFO Channel A Almost Full Interrupt Flag */
    vuint16_t FAFBIF:1;                /* Receive FIFO Channel B Almost Full Interrupt Flag */
    vuint16_t WUPIF:1;                 /* Wakeup Interrupt Flag */
    vuint16_t CHIF:1;                  /* CHI Interrupt Flag */
    vuint16_t PRIF:1;                  /* Protocol Interrupt Flag */
    vuint16_t MIF:1;                   /* Module Interrupt Flag */
    vuint16_t _unused_8:8;
  } B;
} FLEXRAY_CIFR_tag;

typedef union FLEXRAY_SYMATOR_union_tag { /* System Memory Access Timeout */
  vuint16_t R;
  struct {
    vuint16_t TIMEOUT:8;               /* System Memory Access Timeout */
    vuint16_t _unused_8:8;
  } B;
} FLEXRAY_SYMATOR_tag;

typedef union FLEXRAY_SFCNTR_union_tag { /* Sync Frame Counter */
  vuint16_t R;
  struct {
    vuint16_t SFODA:4;                 /* Sync Frames Channel A, Odd Cycle */
    vuint16_t SFODB:4;                 /* Sync Frames Channel B, Odd cycle */
    vuint16_t SFEVA:4;                 /* Sync Frames Channel A, Even Cycle */
    vuint16_t SFEVB:4;                 /* Sync Frames Channel B, Even Cycle */
  } B;
} FLEXRAY_SFCNTR_tag;

typedef union FLEXRAY_SFTOR_union_tag { /* Sync Frame Table Offset */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t SFT_OFFSET:15;           /* Sync Frame Table Offset */
  } B;
} FLEXRAY_SFTOR_tag;

typedef union FLEXRAY_SFTCCSR_union_tag { /* Sync Frame Table Configuration Control Status */
  vuint16_t R;
  struct {
    vuint16_t SIDEN:1;                 /* Sync Frame ID Table Enable */
    vuint16_t SDVEN:1;                 /* Sync Frame Deviation Table Enable */
    vuint16_t OPT:1;                   /* One Pair Trigger */
    vuint16_t _unused_3:1;
    vuint16_t OVAL:1;                  /* Odd Cycle Tables Valid */
    vuint16_t EVAL:1;                  /* Even Cycle Tables Valid */
    vuint16_t OLKS:1;                  /* Odd Cycle Tables Lock Status */
    vuint16_t ELKS:1;                  /* Even Cycle Tables Lock Status */
    vuint16_t CYCNUM:6;                /* Cycle Number */
    vuint16_t OLKT:1;                  /* Odd Cycle Tables Lock And Unlock Trigger */
    vuint16_t ELKT:1;                  /* Even Cycle Tables Lock/Unlock Trigger */
  } B;
} FLEXRAY_SFTCCSR_tag;

typedef union FLEXRAY_SFIDRFR_union_tag { /* Sync Frame ID Rejection Filter */
  vuint16_t R;
  struct {
    vuint16_t SYNFRID:10;              /* Sync Frame Rejection ID */
    vuint16_t _unused_10:6;
  } B;
} FLEXRAY_SFIDRFR_tag;

typedef union FLEXRAY_SFIDAFVR_union_tag { /* Sync Frame ID Acceptance Filter Value */
  vuint16_t R;
  struct {
    vuint16_t FVAL:10;                 /* Filter Value */
    vuint16_t _unused_10:6;
  } B;
} FLEXRAY_SFIDAFVR_tag;

typedef union FLEXRAY_SFIDAFMR_union_tag { /* Sync Frame ID Acceptance Filter Mask */
  vuint16_t R;
  struct {
    vuint16_t FMSK:10;                 /* Filter Mask */
    vuint16_t _unused_10:6;
  } B;
} FLEXRAY_SFIDAFMR_tag;

typedef union FLEXRAY_NMVR_union_tag { /* NMV 0 */
  vuint16_t R;
  struct {
    vuint16_t NMVP:16;                 /* NMV Part */
  } B;
} FLEXRAY_NMVR_tag;

typedef union FLEXRAY_NMVLR_union_tag { /* Network Management Vector Length Register */
  vuint16_t R;
  struct {
    vuint16_t NMVL:4;                  /* Network Management Vector Length */
    vuint16_t _unused_4:12;            /* Reserved */
  } B;
} FLEXRAY_NMVLR_tag;

typedef union FLEXRAY_TICCR_union_tag { /* Timer Configuration And Control */
  vuint16_t R;
  struct {
    vuint16_t T1ST:1;                  /* Timer T1 State */
    vuint16_t T1TR:1;                  /* Timer T1 Trigger */
    vuint16_t T1SP:1;                  /* Timer T1 Stop */
    vuint16_t _unused_3:1;
    vuint16_t T1_REP:1;                /* Timer T1 Repetitive Mode */
    vuint16_t _unused_5:3;
    vuint16_t T2ST:1;                  /* Timer T2 State */
    vuint16_t T2TR:1;                  /* Timer T2 Trigger */
    vuint16_t T2SP:1;                  /* Timer T2 Stop */
    vuint16_t _unused_11:1;
    vuint16_t T2_REP:1;                /* Timer T2 Repetitive Mode */
    vuint16_t T2_CFG:1;                /* Timer T2 Configuration */
    vuint16_t _unused_14:2;
  } B;
} FLEXRAY_TICCR_tag;

typedef union FLEXRAY_TI1CYSR_union_tag { /* Timer 1 Cycle Set */
  vuint16_t R;
  struct {
    vuint16_t T1_CYC_MSK:6;            /* Timer T1 Cycle Filter Mask */
    vuint16_t _unused_6:2;
    vuint16_t T1_CYC_VAL:6;            /* Timer T1 Cycle Filter Value */
    vuint16_t _unused_14:2;
  } B;
} FLEXRAY_TI1CYSR_tag;

typedef union FLEXRAY_TI1MTOR_union_tag { /* Timer 1 MT Offset */
  vuint16_t R;
  struct {
    vuint16_t T1_MTOFFSET:14;          /* Timer 1 MT Offset */
    vuint16_t _unused_14:2;
  } B;
} FLEXRAY_TI1MTOR_tag;

typedef union FLEXRAY_TI2CR0_ABS_union_tag { /* Timer 2 Configuration 0 (Absolute Timer Configuration) */
  vuint16_t R;
  struct {
    vuint16_t T2CYCMSK:6;              /* Timer T2 Cycle Filter Mask */
    vuint16_t _unused_6:2;             /* Reserved */
    vuint16_t T2CYCVAL:6;              /* Timer T2 Cycle Filter Mask */
    vuint16_t _unused_14:2;            /* Reserved */
  } B;
} FLEXRAY_TI2CR0_ABS_tag;

typedef union FLEXRAY_TI2CR0_REL_union_tag { /* Timer 2 Configuration 0 (Relative Timer Configuration) */
  vuint16_t R;
  struct {
    vuint16_t T2MTCNT:16;              /* Timer T2 MT High Word */
  } B;
} FLEXRAY_TI2CR0_REL_tag;

typedef union FLEXRAY_TI2CR0_union_tag {
  FLEXRAY_TI2CR0_ABS_tag ABS;          /* Timer 2 Configuration 0 (Absolute Timer Configuration) */
  FLEXRAY_TI2CR0_REL_tag REL;          /* Timer 2 Configuration 0 (Relative Timer Configuration) */
} FLEXRAY_TI2CR0_tag;

typedef union FLEXRAY_TI2CR1_ABS_union_tag { /* Timer 2 Configuration 1 (Absolute Timer Configuration) */
  vuint16_t R;
  struct {
    vuint16_t T2MOFF:14;               /* Timer T2 MT Offset */
    vuint16_t _unused_14:2;            /* Reserved */
  } B;
} FLEXRAY_TI2CR1_ABS_tag;

typedef union FLEXRAY_TI2CR1_REL_union_tag { /* Timer 2 Configuration 1 (Relative Timer Configuration) */
  vuint16_t R;
  struct {
    vuint16_t T2MTCNT:16;              /* Timer T2 Macrotick Low Word */
  } B;
} FLEXRAY_TI2CR1_REL_tag;

typedef union FLEXRAY_TI2CR1_union_tag {
  FLEXRAY_TI2CR1_ABS_tag ABS;          /* Timer 2 Configuration 1 (Absolute Timer Configuration) */
  FLEXRAY_TI2CR1_REL_tag REL;          /* Timer 2 Configuration 1 (Relative Timer Configuration) */
} FLEXRAY_TI2CR1_tag;

typedef union FLEXRAY_SSSR_union_tag { /* Slot Status Selection */
  vuint16_t R;
  struct {
    vuint16_t SLOTNUMBER:11;           /* Slot Number */
    vuint16_t _unused_11:1;
    vuint16_t SEL:2;                   /* Selector */
    vuint16_t _unused_14:1;
    vuint16_t WMD:1;                   /* Write Mode */
  } B;
} FLEXRAY_SSSR_tag;

typedef union FLEXRAY_SSCCR_union_tag { /* Slot Status Counter Condition */
  vuint16_t R;
  struct {
    vuint16_t STATUSMASK:4;            /* Slot Status Mask */
    vuint16_t SUF:1;                   /* Startup Frame Restriction */
    vuint16_t NUF:1;                   /* Null Frame Restriction */
    vuint16_t SYF:1;                   /* Sync Frame Restriction */
    vuint16_t VFR:1;                   /* Valid Frame Restriction */
    vuint16_t MCY:1;                   /* Multi Cycle Selection */
    vuint16_t CNTCFG:2;                /* Counter Configuration */
    vuint16_t _unused_11:1;
    vuint16_t SEL:2;                   /* Selector */
    vuint16_t _unused_14:1;
    vuint16_t WMD:1;                   /* Write Mode */
  } B;
} FLEXRAY_SSCCR_tag;

typedef union FLEXRAY_SSR_union_tag {  /* Slot Status */
  vuint16_t R;
  struct {
    vuint16_t TCA:1;                   /* Transmission Conflict On Channel A */
    vuint16_t BVA:1;                   /* Boundary Violation On Channel A */
    vuint16_t CEA:1;                   /* Content Error On Channel A */
    vuint16_t SEA:1;                   /* Syntax Error On Channel A */
    vuint16_t SUA:1;                   /* Startup Frame Indicator Channel A */
    vuint16_t NFA:1;                   /* Null Frame Indicator Channel A */
    vuint16_t SYA:1;                   /* Sync Frame Indicator Channel A */
    vuint16_t VFA:1;                   /* Valid Frame On Channel A */
    vuint16_t TCB:1;                   /* Transmission Conflict On Channel B */
    vuint16_t BVB:1;                   /* Boundary Violation On Channel B */
    vuint16_t CEB:1;                   /* Content Error On Channel B */
    vuint16_t SEB:1;                   /* Syntax Error On Channel B */
    vuint16_t SUB:1;                   /* Startup Frame Indicator Channel B */
    vuint16_t NFB:1;                   /* Null Frame Indicator Channel B */
    vuint16_t SYB:1;                   /* Sync Frame Indicator Channel B */
    vuint16_t VFB:1;                   /* Valid Frame On Channel B */
  } B;
} FLEXRAY_SSR_tag;

typedef union FLEXRAY_SSCR_union_tag { /* Slot Status Counter */
  vuint16_t R;
  struct {
    vuint16_t SLOTSTATUSCNT:16;        /* Slot Status Counter */
  } B;
} FLEXRAY_SSCR_tag;

typedef union FLEXRAY_MTSACFR_union_tag { /* MTS A Configuration */
  vuint16_t R;
  struct {
    vuint16_t CYCCNTVAL:6;             /* Cycle Counter Value */
    vuint16_t _unused_6:2;
    vuint16_t CYCCNTMSK:6;             /* Cycle Counter Mask */
    vuint16_t _unused_14:1;
    vuint16_t MTE:1;                   /* MTS Transmission Enable */
  } B;
} FLEXRAY_MTSACFR_tag;

typedef union FLEXRAY_MTSBCFR_union_tag { /* MTS B Configuration */
  vuint16_t R;
  struct {
    vuint16_t CYCCNTVAL:6;             /* Cycle Counter Value */
    vuint16_t _unused_6:2;
    vuint16_t CYCCNTMSK:6;             /* Cycle Counter Mask */
    vuint16_t _unused_14:1;
    vuint16_t MTE:1;                   /* MTS Transmission Enable */
  } B;
} FLEXRAY_MTSBCFR_tag;

typedef union FLEXRAY_RSBIR_union_tag { /* Receive Shadow Buffer Index */
  vuint16_t R;
  struct {
    vuint16_t RSBIDX:9;                /* RSBIDXA1/RSBIDXA2/RSBIDXB1/RSBIDXB2- Receive Shadow Buffer Index */
    vuint16_t _unused_9:3;
    vuint16_t SEL:2;                   /* Selector */
    vuint16_t _unused_14:1;
    vuint16_t WMD:1;                   /* Write Mode */
  } B;
} FLEXRAY_RSBIR_tag;

typedef union FLEXRAY_RFWMSR_union_tag { /* Receive FIFO Watermark And Selection */
  vuint16_t R;
  struct {
    vuint16_t SEL:1;                   /* Select */
    vuint16_t _unused_1:7;
    vuint16_t WM:8;                    /* WMA/WMB - Watermark */
  } B;
} FLEXRAY_RFWMSR_tag;

typedef union FLEXRAY_RFSIR_union_tag { /* Receive FIFO Start Index */
  vuint16_t R;
  struct {
    vuint16_t SIDX:10;                 /* SIDXA Or SIDXB Start Index */
    vuint16_t _unused_10:6;
  } B;
} FLEXRAY_RFSIR_tag;

typedef union FLEXRAY_RFDSR_union_tag { /* Receive FIFO Depth And Size */
  vuint16_t R;
  struct {
    vuint16_t ENTRY_SIZE:7;            /* ENTRY_SIZEA Or ENTRY_SIZEB Entry Size */
    vuint16_t _unused_7:1;
    vuint16_t FIFO_DEPTH:8;            /* FIFO_DEPTHA And FIFO_DEPTHB FIFO Depth */
  } B;
} FLEXRAY_RFDSR_tag;

typedef union FLEXRAY_RFARIR_union_tag { /* Receive FIFO A Read Index */
  vuint16_t R;
  struct {
    vuint16_t RDIDX:10;                /* Read Index */
    vuint16_t _unused_10:6;
  } B;
} FLEXRAY_RFARIR_tag;

typedef union FLEXRAY_RFBRIR_union_tag { /* Receive FIFO B Read Index */
  vuint16_t R;
  struct {
    vuint16_t RDIDX:10;                /* Read Index */
    vuint16_t _unused_10:6;
  } B;
} FLEXRAY_RFBRIR_tag;

typedef union FLEXRAY_RFMIDAFVR_union_tag { /* Receive FIFO Message ID Acceptance Filter Value */
  vuint16_t R;
  struct {
    vuint16_t MIDAFVAL:16;             /* MIDAFVALA Or MIDAFVALB Message ID Acceptance Filter Value */
  } B;
} FLEXRAY_RFMIDAFVR_tag;

typedef union FLEXRAY_RFMIDAFMR_union_tag { /* Receive FIFO Message ID Acceptance Filter Mask */
  vuint16_t R;
  struct {
    vuint16_t MIDAFMSK:16;             /* MIDAFMSKA Or MIDAFMSKB Message ID Acceptance Filter Mask */
  } B;
} FLEXRAY_RFMIDAFMR_tag;

typedef union FLEXRAY_RFFIDRFVR_union_tag { /* Receive FIFO Frame ID Rejection Filter Value */
  vuint16_t R;
  struct {
    vuint16_t FIDRFVAL:11;             /* FIDRFVALA Or FIDRFVALB Frame ID Rejection Filter Value */
    vuint16_t _unused_11:5;
  } B;
} FLEXRAY_RFFIDRFVR_tag;

typedef union FLEXRAY_RFFIDRFMR_union_tag { /* Receive FIFO Frame ID Rejection Filter Mask */
  vuint16_t R;
  struct {
    vuint16_t FIDRFMSK:11;             /* Frame ID Rejection Filter Mask */
    vuint16_t _unused_11:5;
  } B;
} FLEXRAY_RFFIDRFMR_tag;

typedef union FLEXRAY_RFRFCFR_union_tag { /* Receive FIFO Range Filter Configuration */
  vuint16_t R;
  struct {
    vuint16_t SID:11;                  /* Slot ID */
    vuint16_t _unused_11:1;
    vuint16_t SEL:2;                   /* Filter Selector */
    vuint16_t IBD:1;                   /* Interval Boundary */
    vuint16_t WMD:1;                   /* Write Mode */
  } B;
} FLEXRAY_RFRFCFR_tag;

typedef union FLEXRAY_RFRFCTR_union_tag { /* Receive FIFO Range Filter Control */
  vuint16_t R;
  struct {
    vuint16_t F0EN:1;                  /* Range Filter 0 Enable */
    vuint16_t F1EN:1;                  /* Range Filter 1 Enable */
    vuint16_t F2EN:1;                  /* Range Filter 2 Enable */
    vuint16_t F3EN:1;                  /* Range Filter 3 Enable */
    vuint16_t _unused_4:4;
    vuint16_t F0MD:1;                  /* Range Filter 0 Mode */
    vuint16_t F1MD:1;                  /* Range Filter 1 Mode */
    vuint16_t F2MD:1;                  /* Range Filter 2 Mode */
    vuint16_t F3MD:1;                  /* Range Filter 3 Mode */
    vuint16_t _unused_12:4;
  } B;
} FLEXRAY_RFRFCTR_tag;

typedef union FLEXRAY_LDTXSLAR_union_tag { /* Last Dynamic Transmit Slot Channel A */
  vuint16_t R;
  struct {
    vuint16_t LDYNTXSLOTA:11;          /* Last Dynamic Transmission Slot Channel A */
    vuint16_t _unused_11:5;
  } B;
} FLEXRAY_LDTXSLAR_tag;

typedef union FLEXRAY_LDTXSLBR_union_tag { /* Last Dynamic Transmit Slot Channel B */
  vuint16_t R;
  struct {
    vuint16_t LDYNTXSLOTB:11;          /* Last Dynamic Transmission Slot Channel B */
    vuint16_t _unused_11:5;
  } B;
} FLEXRAY_LDTXSLBR_tag;

typedef union FLEXRAY_PCR0_union_tag { /* Protocol Configuration 0 */
  vuint16_t R;
  struct {
    vuint16_t STATIC_SLOT_LENGTH:10;   /* static_slot_length */
    vuint16_t ACTION_POINT_OFFSET:6;   /* action_point_offset */
  } B;
} FLEXRAY_PCR0_tag;

typedef union FLEXRAY_PCR1_union_tag { /* Protocol Configuration 1 */
  vuint16_t R;
  struct {
    vuint16_t MACRO_AFTER_FIRST_STATIC_SLOT:14; /* macro_after_first_static_slot */
    vuint16_t _unused_14:2;
  } B;
} FLEXRAY_PCR1_tag;

typedef union FLEXRAY_PCR2_union_tag { /* Protocol Configuration 2 */
  vuint16_t R;
  struct {
    vuint16_t NUMBER_OF_STATIC_SLOTS:10; /* gNumberOfStaticSlots */
    vuint16_t MINISLOT_AFTER_ACTION_POINT:6; /* minislot_after_action_point */
  } B;
} FLEXRAY_PCR2_tag;

typedef union FLEXRAY_PCR3_union_tag { /* Protocol Configuration 3 */
  vuint16_t R;
  struct {
    vuint16_t COLDSTART_ATTEMPTS:5;    /* coldstart_attempts */
    vuint16_t MINISLOT_ACTION_POINT_OFFSET:5; /* minislot_action_point_offset */
    vuint16_t WAKEUP_SYMBOL_RX_LOW:6;  /* wakeup_symbol_rx_low */
  } B;
} FLEXRAY_PCR3_tag;

typedef union FLEXRAY_PCR4_union_tag { /* Protocol Configuration 4 */
  vuint16_t R;
  struct {
    vuint16_t WAKEUP_SYMBOL_RX_WINDOW:9; /* wakeup_symbol_rx_window */
    vuint16_t CAS_RX_LOW_MAX:7;        /* cas_rx_low_max */
  } B;
} FLEXRAY_PCR4_tag;

typedef union FLEXRAY_PCR5_union_tag { /* Protocol Configuration 5 */
  vuint16_t R;
  struct {
    vuint16_t WAKEUP_SYMBOL_RX_IDLE:6; /* wakeup_symbol_rx_idle */
    vuint16_t WAKEUP_SYMBOL_TX_LOW:6;  /* wakeup_symbol_tx_low */
    vuint16_t TSS_TRANSMITTER:4;       /* tss_transmitter */
  } B;
} FLEXRAY_PCR5_tag;

typedef union FLEXRAY_PCR6_union_tag { /* Protocol Configuration 6 */
  vuint16_t R;
  struct {
    vuint16_t MACRO_INITIAL_OFFSET_A:7; /* macro_initial_offset_a */
    vuint16_t SYMBOL_WINDOW_AFTER_ACTION_POINT:8; /* symbol_window_after_action_point */
    vuint16_t _unused_15:1;
  } B;
} FLEXRAY_PCR6_tag;

typedef union FLEXRAY_PCR7_union_tag { /* Protocol Configuration 7 */
  vuint16_t R;
  struct {
    vuint16_t MICRO_PER_MACRO_NOM_HALF:7; /* micro_per_macro_nom_half */
    vuint16_t DECODING_CORRECTION_B:9; /* decoding_correction_b */
  } B;
} FLEXRAY_PCR7_tag;

typedef union FLEXRAY_PCR8_union_tag { /* Protocol Configuration 8 */
  vuint16_t R;
  struct {
    vuint16_t WAKEUP_SYMBOL_TX_IDLE:8; /* wakeup_symbol_tx_idle */
    vuint16_t MAX_WITHOUT_CLOCK_CORRECTION_PASSIVE:4; /* max_without_clock_correction_passive */
    vuint16_t MAX_WITHOUT_CLOCK_CORRECTION_FATAL:4; /* max_without_clock_correction_fatal */
  } B;
} FLEXRAY_PCR8_tag;

typedef union FLEXRAY_PCR9_union_tag { /* Protocol Configuration 9 */
  vuint16_t R;
  struct {
    vuint16_t OFFSET_CORRECTION_OUT:14; /* offset_correction_out */
    vuint16_t SYMBOL_WINDOW_EXISTS:1;  /* symbol_window_exists */
    vuint16_t MINISLOT_EXISTS:1;       /* minislot_exists */
  } B;
} FLEXRAY_PCR9_tag;

typedef union FLEXRAY_PCR10_union_tag { /* Protocol Configuration 10 */
  vuint16_t R;
  struct {
    vuint16_t MACRO_PER_CYCLE:14;      /* macro_per_cycle */
    vuint16_t WAKEUP_CHANNEL:1;        /* wakeup_channel */
    vuint16_t SINGLE_SLOT_ENABLED:1;   /* single_slot_enabled */
  } B;
} FLEXRAY_PCR10_tag;

typedef union FLEXRAY_PCR11_union_tag { /* Protocol Configuration 11 */
  vuint16_t R;
  struct {
    vuint16_t OFFSET_CORRECTION_START:14; /* offset_correction_start */
    vuint16_t KEY_SLOT_USED_FOR_SYNC:1; /* key_slot_used_for_sync */
    vuint16_t KEY_SLOT_USED_FOR_STARTUP:1; /* key_slot_used_for_startup */
  } B;
} FLEXRAY_PCR11_tag;

typedef union FLEXRAY_PCR12_union_tag { /* Protocol Configuration 12 */
  vuint16_t R;
  struct {
    vuint16_t KEY_SLOT_HEADER_CRC:11;  /* key_slot_header_crc */
    vuint16_t ALLOW_PASSIVE_TO_ACTIVE:5; /* allow_passive_to_active */
  } B;
} FLEXRAY_PCR12_tag;

typedef union FLEXRAY_PCR13_union_tag { /* Protocol Configuration 13 */
  vuint16_t R;
  struct {
    vuint16_t STATIC_SLOT_AFTER_ACTION_POINT:10; /* static_slot_after_action_point */
    vuint16_t FIRST_MINISLOT_ACTION_POINT_OFFSET:6; /* first_minislot_action_point_offset */
  } B;
} FLEXRAY_PCR13_tag;

typedef union FLEXRAY_PCR14_union_tag { /* Protocol Configuration 14 */
  vuint16_t R;
  struct {
    vuint16_t LISTEN_TIMEOUT:5;        /* listen_timeout */
    vuint16_t RATE_CORRECTION_OUT:11;  /* rate_correction_out */
  } B;
} FLEXRAY_PCR14_tag;

typedef union FLEXRAY_PCR15_union_tag { /* Protocol Configuration 15 */
  vuint16_t R;
  struct {
    vuint16_t LISTEN_TIMEOUT:16;       /* listen_timeout */
  } B;
} FLEXRAY_PCR15_tag;

typedef union FLEXRAY_PCR16_union_tag { /* Protocol Configuration 16 */
  vuint16_t R;
  struct {
    vuint16_t NOISE_LISTEN_TIMEOUT:9;  /* noise_listen_timeout */
    vuint16_t MACRO_INITIAL_OFFSET_B:7; /* macro_initial_offset_b */
  } B;
} FLEXRAY_PCR16_tag;

typedef union FLEXRAY_PCR17_union_tag { /* Protocol Configuration 17 */
  vuint16_t R;
  struct {
    vuint16_t NOISE_LISTEN_TIMEOUT:16; /* noise_listen_timeout */
  } B;
} FLEXRAY_PCR17_tag;

typedef union FLEXRAY_PCR18_union_tag { /* Protocol Configuration 18 */
  vuint16_t R;
  struct {
    vuint16_t KEY_SLOT_ID:10;          /* key_slot_id */
    vuint16_t WAKEUP_PATTERN:6;        /* wakeup_pattern */
  } B;
} FLEXRAY_PCR18_tag;

typedef union FLEXRAY_PCR19_union_tag { /* Protocol Configuration 19 */
  vuint16_t R;
  struct {
    vuint16_t PAYLOAD_LENGTH_STATIC:7; /* payload_length_static */
    vuint16_t DECODING_CORRECTION_A:9; /* decoding_correction_a */
  } B;
} FLEXRAY_PCR19_tag;

typedef union FLEXRAY_PCR20_union_tag { /* Protocol Configuration 20 */
  vuint16_t R;
  struct {
    vuint16_t MICRO_INITIAL_OFFSET_A:8; /* micro_initial_offset_a */
    vuint16_t MICRO_INITIAL_OFFSET_B:8; /* micro_initial_offset_b */
  } B;
} FLEXRAY_PCR20_tag;

typedef union FLEXRAY_PCR21_union_tag { /* Protocol Configuration 21 */
  vuint16_t R;
  struct {
    vuint16_t LATEST_TX:13;            /* latest_tx */
    vuint16_t EXTERN_RATE_CORRECTION:3; /* extern_rate_correction */
  } B;
} FLEXRAY_PCR21_tag;

typedef union FLEXRAY_PCR22_union_tag { /* Protocol Configuration 22 */
  vuint16_t R;
  struct {
    vuint16_t MICRO_PER_CYCLE:4;       /* micro_per_cycle */
    vuint16_t COMP_ACCEPTED_STARTUP_RANGE_A:11; /* comp_accepted_startup_range_a */
    vuint16_t _unused_15:1;            /* Reserved bit, will not be changed. Do not write any value different from the reset value. */
  } B;
} FLEXRAY_PCR22_tag;

typedef union FLEXRAY_PCR23_union_tag { /* Protocol Configuration 23 */
  vuint16_t R;
  struct {
    vuint16_t MICRO_PER_CYCLE:16;      /* micro_per_cycle */
  } B;
} FLEXRAY_PCR23_tag;

typedef union FLEXRAY_PCR24_union_tag { /* Protocol Configuration 24 */
  vuint16_t R;
  struct {
    vuint16_t MICRO_PER_CYCLE_MIN:4;   /* micro_per_cycle_min */
    vuint16_t MAX_PAYLOAD_LENGTH_DYNAMIC:7; /* max_payload_length_dynamic */
    vuint16_t CLUSTER_DRIFT_DAMPING:5; /* cluster_drift_damping */
  } B;
} FLEXRAY_PCR24_tag;

typedef union FLEXRAY_PCR25_union_tag { /* Protocol Configuration 25 */
  vuint16_t R;
  struct {
    vuint16_t MICRO_PER_CYCLE_MIN:16;  /* micro_per_cycle_min */
  } B;
} FLEXRAY_PCR25_tag;

typedef union FLEXRAY_PCR26_union_tag { /* Protocol Configuration 26 */
  vuint16_t R;
  struct {
    vuint16_t MICRO_PER_CYCLE_MAX:4;   /* micro_per_cycle_max */
    vuint16_t COMP_ACCEPTED_STARTUP_RANGE_B:11; /* comp_accepted_startup_range_b */
    vuint16_t ALLOW_HALT_DUE_TO_CLOCK:1; /* allow_halt_due_to_clock */
  } B;
} FLEXRAY_PCR26_tag;

typedef union FLEXRAY_PCR27_union_tag { /* Protocol Configuration 27 */
  vuint16_t R;
  struct {
    vuint16_t MICRO_PER_CYCLE_MAX:16;  /* micro_per_cycle_max */
  } B;
} FLEXRAY_PCR27_tag;

typedef union FLEXRAY_PCR28_union_tag { /* Protocol Configuration 28 */
  vuint16_t R;
  struct {
    vuint16_t MACRO_AFTER_OFFSET_CORRECTION:14; /* macro_after_offset_correction */
    vuint16_t DYNAMIC_SLOT_IDLE_PHASE:2; /* dynamic_slot_idle_phase */
  } B;
} FLEXRAY_PCR28_tag;

typedef union FLEXRAY_PCR29_union_tag { /* Protocol Configuration 29 */
  vuint16_t R;
  struct {
    vuint16_t MINISLOTS_MAX:13;        /* minislots_max */
    vuint16_t EXTERN_OFFSET_CORRECTION:3; /* extern_offset_correction */
  } B;
} FLEXRAY_PCR29_tag;

typedef union FLEXRAY_PCR30_union_tag { /* Protocol Configuration 30 */
  vuint16_t R;
  struct {
    vuint16_t SYNC_NODE_MAX:4;         /* sync_node_max */
    vuint16_t _unused_4:12;
  } B;
} FLEXRAY_PCR30_tag;

typedef union FLEXRAY_STPWHR_union_tag { /* Stopwatch Count High */
  vuint16_t R;
  struct {
    vuint16_t STPW_S_H:16;             /* Stopwatch Count High */
  } B;
} FLEXRAY_STPWHR_tag;

typedef union FLEXRAY_STPWLR_union_tag { /* Stopwatch Count Low */
  vuint16_t R;
  struct {
    vuint16_t STPW_S_L:16;             /* Stopwatch Count Low */
  } B;
} FLEXRAY_STPWLR_tag;

typedef union FLEXRAY_PEOER_union_tag { /* Protocol Event Output Enable And Stopwatch Control */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:3;
    vuint16_t _unused_3:5;
    vuint16_t STPW_EN:1;               /* Stopwatch Count Enable */
    vuint16_t _unused_9:7;
  } B;
} FLEXRAY_PEOER_tag;

typedef union FLEXRAY_RFSDOR_union_tag { /* Receive FIFO Start Data Offset */
  vuint16_t R;
  struct {
    vuint16_t SDO:16;                  /* SDOA And SDOB Start Data Field Offset */
  } B;
} FLEXRAY_RFSDOR_tag;

typedef union FLEXRAY_RFSYMBADHR_union_tag { /* Receive FIFO System Memory Base Address High */
  vuint16_t R;
  struct {
    vuint16_t SMBA:16;                 /* System Memory Base Address */
  } B;
} FLEXRAY_RFSYMBADHR_tag;

typedef union FLEXRAY_RFSYMBADLR_union_tag { /* Receive FIFO System Memory Base Address Low */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t SMBA:12;                 /* System Memory Base Address */
  } B;
} FLEXRAY_RFSYMBADLR_tag;

typedef union FLEXRAY_RFPTR_union_tag { /* Receive FIFO Periodic Timer */
  vuint16_t R;
  struct {
    vuint16_t PTD:14;                  /* Periodic Timer Duration */
    vuint16_t _unused_14:2;
  } B;
} FLEXRAY_RFPTR_tag;

typedef union FLEXRAY_RFFLPCR_union_tag { /* Receive FIFO Fill Level and Pop Count */
  vuint16_t R;
  struct {
    vuint16_t FLA_OR_PCA:8;            /* FLA Or PCA */
    vuint16_t FLB_OR_PCB:8;            /* FLB Or PCB */
  } B;
} FLEXRAY_RFFLPCR_tag;

typedef union FLEXRAY_EEIFER_union_tag { /* ECC Error Interrupt Flag And Enable */
  vuint16_t R;
  struct {
    vuint16_t DRCE_IE:1;               /* DRAM Corrected Error Interrupt Enable */
    vuint16_t DRNE_IE:1;               /* DRAM Non-Corrected Error Interrupt Enable */
    vuint16_t LRCE_IE:1;               /* LRAM Corrected Error Interrupt Enable */
    vuint16_t LRNE_IE:1;               /* LRAM Non-Corrected Error Interrupt Enable */
    vuint16_t _unused_4:4;
    vuint16_t DRCE_IF:1;               /* DRAM Corrected Error Interrupt Flag */
    vuint16_t DRNE_IF:1;               /* DRAM Non-Corrected Error Interrupt Flag */
    vuint16_t LRCE_IF:1;               /* LRAM Corrected Error Interrupt Flag */
    vuint16_t LRNE_IF:1;               /* LRAM Non-Corrected Error Interrupt Flag */
    vuint16_t DRCE_OF:1;               /* DRAM Corrected Error Overflow Flag */
    vuint16_t DRNE_OF:1;               /* DRAM Non-Corrected Error Overflow Flag */
    vuint16_t LRCE_OF:1;               /* LRAM Corrected Error Overflow Flag */
    vuint16_t LRNE_OF:1;               /* LRAM Non-Corrected Error Overflow Flag */
  } B;
} FLEXRAY_EEIFER_tag;

typedef union FLEXRAY_EERICR_union_tag { /* ECC Error Report And Injection Control */
  vuint16_t R;
  struct {
    vuint16_t EIE:1;                   /* Error Injection Enable */
    vuint16_t EIM:1;                   /* Error Injection Mode */
    vuint16_t _unused_2:2;
    vuint16_t ERM:1;                   /* Error Report Mode */
    vuint16_t _unused_5:3;
    vuint16_t ERS:2;                   /* Error Report Select */
    vuint16_t _unused_10:5;
    vuint16_t BSY:1;                   /* Register Update Busy */
  } B;
} FLEXRAY_EERICR_tag;

typedef union FLEXRAY_EERAR_union_tag { /* ECC Error Report Address */
  vuint16_t R;
  struct {
    vuint16_t ADDR:12;                 /* Memory Address */
    vuint16_t BANK:3;                  /* Memory Bank */
    vuint16_t MID:1;                   /* Memory Identifier */
  } B;
} FLEXRAY_EERAR_tag;

typedef union FLEXRAY_EERDR_union_tag { /* ECC Error Report Data */
  vuint16_t R;
  struct {
    vuint16_t DATA:16;                 /* Data */
  } B;
} FLEXRAY_EERDR_tag;

typedef union FLEXRAY_EERCR_union_tag { /* ECC Error Report Code */
  vuint16_t R;
  struct {
    vuint16_t CODE:5;                  /* Code */
    vuint16_t _unused_5:11;
  } B;
} FLEXRAY_EERCR_tag;

typedef union FLEXRAY_EEIAR_union_tag { /* ECC Error Injection Address */
  vuint16_t R;
  struct {
    vuint16_t ADDR:12;                 /* Memory Address */
    vuint16_t BANK:3;                  /* Memory Bank */
    vuint16_t MID:1;                   /* Memory Identifier */
  } B;
} FLEXRAY_EEIAR_tag;

typedef union FLEXRAY_EEIDR_union_tag { /* ECC Error Injection Data */
  vuint16_t R;
  struct {
    vuint16_t DATA:16;                 /* Data */
  } B;
} FLEXRAY_EEIDR_tag;

typedef union FLEXRAY_EEICR_union_tag { /* ECC Error Injection Code */
  vuint16_t R;
  struct {
    vuint16_t CODE:5;                  /* Code */
    vuint16_t _unused_5:11;
  } B;
} FLEXRAY_EEICR_tag;

typedef union FLEXRAY_MB_CCSR_union_tag { /* MB Configuration Control Status */
  vuint16_t R;
  struct {
    vuint16_t MBIF:1;                  /* MB Interrupt Flag */
    vuint16_t LCKS:1;                  /* Lock Status */
    vuint16_t EDS:1;                   /* Enable And Disable Status */
    vuint16_t DVAL:1;                  /* Data Valid */
    vuint16_t DUP:1;                   /* Data Updated */
    vuint16_t _unused_5:3;
    vuint16_t MBIE:1;                  /* MB Interrupt Enable */
    vuint16_t LCKT:1;                  /* Lock And Unlock Trigger */
    vuint16_t EDT:1;                   /* Enable And Disable Trigger */
    vuint16_t CMT:1;                   /* Commit To Transmission */
    vuint16_t MTD:1;                   /* MB Transfer Direction */
    vuint16_t _unused_13:3;
  } B;
} FLEXRAY_MB_CCSR_tag;

typedef union FLEXRAY_MB_CCFR_union_tag { /* MB Cycle Counter Filter */
  vuint16_t R;
  struct {
    vuint16_t CCFVAL:6;                /* Cycle Counter Filtering Value */
    vuint16_t CCFMSK:6;                /* Cycle Counter Filtering Mask */
    vuint16_t CCFE:1;                  /* Cycle Counter Filtering Enable */
    vuint16_t CHB:1;                   /* Channel Assignment B */
    vuint16_t CHA:1;                   /* Channel Assignment A */
    vuint16_t MTM:1;                   /* MB Transmission Mode */
  } B;
} FLEXRAY_MB_CCFR_tag;

typedef union FLEXRAY_MB_FIDR_union_tag { /* MB FID */
  vuint16_t R;
  struct {
    vuint16_t FID:11;                  /* FID */
    vuint16_t _unused_11:5;
  } B;
} FLEXRAY_MB_FIDR_tag;

typedef union FLEXRAY_MB_IDXR_union_tag { /* MB Index */
  vuint16_t R;
  struct {
    vuint16_t MBIDX:9;                 /* MB Index */
    vuint16_t _unused_9:7;
  } B;
} FLEXRAY_MB_IDXR_tag;

typedef struct FLEXRAY_MB_struct_tag {
  FLEXRAY_MB_CCSR_tag CCSR;            /* MB Configuration Control Status */
  FLEXRAY_MB_CCFR_tag CCFR;            /* MB Cycle Counter Filter */
  FLEXRAY_MB_FIDR_tag FIDR;            /* MB FID */
  FLEXRAY_MB_IDXR_tag IDXR;            /* MB Index */
} FLEXRAY_MB_tag;

typedef union FLEXRAY_MBDOR_union_tag { /* MB Data Field Offset */
  vuint16_t R;
  struct {
    vuint16_t MBDO:16;                 /* MB Data Field Offset */
  } B;
} FLEXRAY_MBDOR_tag;

typedef union FLEXRAY_LEETR_union_tag { /* LRAM ECC Error Test Register */
  vuint16_t R;
  struct {
    vuint16_t LEETD:16;                /* LRAM ECC Error Test Data */
  } B;
} FLEXRAY_LEETR_tag;

struct FLEXRAY_tag {
  FLEXRAY_MVR_tag MVR;                 /* Module Version */
  FLEXRAY_MCR_tag MCR;                 /* Module Configuration */
  FLEXRAY_SYMBADHR_tag SYMBADHR;       /* System Memory Base Address High */
  FLEXRAY_SYMBADLR_tag SYMBADLR;       /* System Memory Base Address Low */
  FLEXRAY_STBSCR_tag STBSCR;           /* Strobe Signal Control */
  uint8_t FLEXRAY_reserved0[2];
  FLEXRAY_MBDSR_tag MBDSR;             /* MB Data Size */
  FLEXRAY_MBSSUTR_tag MBSSUTR;         /* MB Segment Size and Utilization */
  FLEXRAY_PEDRAR_tag PEDRAR;           /* PE DRAM Access */
  FLEXRAY_PEDRDR_tag PEDRDR;           /* PE DRAM Data */
  FLEXRAY_POCR_tag POCR;               /* Protocol Operation Control */
  FLEXRAY_GIFER_tag GIFER;             /* Global Interrupt Flag And Enable */
  FLEXRAY_PIFR0_tag PIFR0;             /* Protocol Interrupt Flag 0 */
  FLEXRAY_PIFR1_tag PIFR1;             /* Protocol Interrupt Flag 1 */
  FLEXRAY_PIER0_tag PIER0;             /* Protocol Interrupt Enable 0 */
  FLEXRAY_PIER1_tag PIER1;             /* Protocol Interrupt Enable 1 */
  FLEXRAY_CHIERFR_tag CHIERFR;         /* CHI Error Flag */
  FLEXRAY_MBIVEC_tag MBIVEC;           /* MB Interrupt Vector */
  FLEXRAY_CASERCR_tag CASERCR;         /* Channel A Status Error Counter Register */
  FLEXRAY_CBSERCR_tag CBSERCR;         /* Channel B Status Error Counter */
  FLEXRAY_PSR0_tag PSR0;               /* Protocol Status 0 */
  FLEXRAY_PSR1_tag PSR1;               /* Protocol Status 1 */
  FLEXRAY_PSR2_tag PSR2;               /* Protocol Status 2 */
  FLEXRAY_PSR3_tag PSR3;               /* Protocol Status 3 */
  FLEXRAY_MTCTR_tag MTCTR;             /* MT Counter */
  FLEXRAY_CYCTR_tag CYCTR;             /* Cycle Counter */
  FLEXRAY_SLTCTAR_tag SLTCTAR;         /* Slot Counter Channel A */
  FLEXRAY_SLTCTBR_tag SLTCTBR;         /* Slot Counter Channel B */
  FLEXRAY_RTCORVR_tag RTCORVR;         /* Rate Correction Value */
  FLEXRAY_OFCORVR_tag OFCORVR;         /* Offset Correction Value */
  FLEXRAY_CIFR_tag CIFR;               /* Combined Interrupt Flag */
  FLEXRAY_SYMATOR_tag SYMATOR;         /* System Memory Access Timeout */
  FLEXRAY_SFCNTR_tag SFCNTR;           /* Sync Frame Counter */
  FLEXRAY_SFTOR_tag SFTOR;             /* Sync Frame Table Offset */
  FLEXRAY_SFTCCSR_tag SFTCCSR;         /* Sync Frame Table Configuration Control Status */
  FLEXRAY_SFIDRFR_tag SFIDRFR;         /* Sync Frame ID Rejection Filter */
  FLEXRAY_SFIDAFVR_tag SFIDAFVR;       /* Sync Frame ID Acceptance Filter Value */
  FLEXRAY_SFIDAFMR_tag SFIDAFMR;       /* Sync Frame ID Acceptance Filter Mask */
  FLEXRAY_NMVR_tag NMVR[6];            /* NMV 0 */
  FLEXRAY_NMVLR_tag NMVLR;             /* Network Management Vector Length Register */
  FLEXRAY_TICCR_tag TICCR;             /* Timer Configuration And Control */
  FLEXRAY_TI1CYSR_tag TI1CYSR;         /* Timer 1 Cycle Set */
  FLEXRAY_TI1MTOR_tag TI1MTOR;         /* Timer 1 MT Offset */
  FLEXRAY_TI2CR0_tag TI2CR0;
  FLEXRAY_TI2CR1_tag TI2CR1;
  FLEXRAY_SSSR_tag SSSR;               /* Slot Status Selection */
  FLEXRAY_SSCCR_tag SSCCR;             /* Slot Status Counter Condition */
  FLEXRAY_SSR_tag SSR[8];              /* Slot Status */
  FLEXRAY_SSCR_tag SSCR[4];            /* Slot Status Counter */
  FLEXRAY_MTSACFR_tag MTSACFR;         /* MTS A Configuration */
  FLEXRAY_MTSBCFR_tag MTSBCFR;         /* MTS B Configuration */
  FLEXRAY_RSBIR_tag RSBIR;             /* Receive Shadow Buffer Index */
  FLEXRAY_RFWMSR_tag RFWMSR;           /* Receive FIFO Watermark And Selection */
  FLEXRAY_RFSIR_tag RFSIR;             /* Receive FIFO Start Index */
  FLEXRAY_RFDSR_tag RFDSR;             /* Receive FIFO Depth And Size */
  FLEXRAY_RFARIR_tag RFARIR;           /* Receive FIFO A Read Index */
  FLEXRAY_RFBRIR_tag RFBRIR;           /* Receive FIFO B Read Index */
  FLEXRAY_RFMIDAFVR_tag RFMIDAFVR;     /* Receive FIFO Message ID Acceptance Filter Value */
  FLEXRAY_RFMIDAFMR_tag RFMIDAFMR;     /* Receive FIFO Message ID Acceptance Filter Mask */
  FLEXRAY_RFFIDRFVR_tag RFFIDRFVR;     /* Receive FIFO Frame ID Rejection Filter Value */
  FLEXRAY_RFFIDRFMR_tag RFFIDRFMR;     /* Receive FIFO Frame ID Rejection Filter Mask */
  FLEXRAY_RFRFCFR_tag RFRFCFR;         /* Receive FIFO Range Filter Configuration */
  FLEXRAY_RFRFCTR_tag RFRFCTR;         /* Receive FIFO Range Filter Control */
  FLEXRAY_LDTXSLAR_tag LDTXSLAR;       /* Last Dynamic Transmit Slot Channel A */
  FLEXRAY_LDTXSLBR_tag LDTXSLBR;       /* Last Dynamic Transmit Slot Channel B */
  FLEXRAY_PCR0_tag PCR0;               /* Protocol Configuration 0 */
  FLEXRAY_PCR1_tag PCR1;               /* Protocol Configuration 1 */
  FLEXRAY_PCR2_tag PCR2;               /* Protocol Configuration 2 */
  FLEXRAY_PCR3_tag PCR3;               /* Protocol Configuration 3 */
  FLEXRAY_PCR4_tag PCR4;               /* Protocol Configuration 4 */
  FLEXRAY_PCR5_tag PCR5;               /* Protocol Configuration 5 */
  FLEXRAY_PCR6_tag PCR6;               /* Protocol Configuration 6 */
  FLEXRAY_PCR7_tag PCR7;               /* Protocol Configuration 7 */
  FLEXRAY_PCR8_tag PCR8;               /* Protocol Configuration 8 */
  FLEXRAY_PCR9_tag PCR9;               /* Protocol Configuration 9 */
  FLEXRAY_PCR10_tag PCR10;             /* Protocol Configuration 10 */
  FLEXRAY_PCR11_tag PCR11;             /* Protocol Configuration 11 */
  FLEXRAY_PCR12_tag PCR12;             /* Protocol Configuration 12 */
  FLEXRAY_PCR13_tag PCR13;             /* Protocol Configuration 13 */
  FLEXRAY_PCR14_tag PCR14;             /* Protocol Configuration 14 */
  FLEXRAY_PCR15_tag PCR15;             /* Protocol Configuration 15 */
  FLEXRAY_PCR16_tag PCR16;             /* Protocol Configuration 16 */
  FLEXRAY_PCR17_tag PCR17;             /* Protocol Configuration 17 */
  FLEXRAY_PCR18_tag PCR18;             /* Protocol Configuration 18 */
  FLEXRAY_PCR19_tag PCR19;             /* Protocol Configuration 19 */
  FLEXRAY_PCR20_tag PCR20;             /* Protocol Configuration 20 */
  FLEXRAY_PCR21_tag PCR21;             /* Protocol Configuration 21 */
  FLEXRAY_PCR22_tag PCR22;             /* Protocol Configuration 22 */
  FLEXRAY_PCR23_tag PCR23;             /* Protocol Configuration 23 */
  FLEXRAY_PCR24_tag PCR24;             /* Protocol Configuration 24 */
  FLEXRAY_PCR25_tag PCR25;             /* Protocol Configuration 25 */
  FLEXRAY_PCR26_tag PCR26;             /* Protocol Configuration 26 */
  FLEXRAY_PCR27_tag PCR27;             /* Protocol Configuration 27 */
  FLEXRAY_PCR28_tag PCR28;             /* Protocol Configuration 28 */
  FLEXRAY_PCR29_tag PCR29;             /* Protocol Configuration 29 */
  FLEXRAY_PCR30_tag PCR30;             /* Protocol Configuration 30 */
  FLEXRAY_STPWHR_tag STPWHR;           /* Stopwatch Count High */
  FLEXRAY_STPWLR_tag STPWLR;           /* Stopwatch Count Low */
  FLEXRAY_PEOER_tag PEOER;             /* Protocol Event Output Enable And Stopwatch Control */
  uint8_t FLEXRAY_reserved1[2];
  FLEXRAY_RFSDOR_tag RFSDOR;           /* Receive FIFO Start Data Offset */
  FLEXRAY_RFSYMBADHR_tag RFSYMBADHR;   /* Receive FIFO System Memory Base Address High */
  FLEXRAY_RFSYMBADLR_tag RFSYMBADLR;   /* Receive FIFO System Memory Base Address Low */
  FLEXRAY_RFPTR_tag RFPTR;             /* Receive FIFO Periodic Timer */
  FLEXRAY_RFFLPCR_tag RFFLPCR;         /* Receive FIFO Fill Level and Pop Count */
  FLEXRAY_EEIFER_tag EEIFER;           /* ECC Error Interrupt Flag And Enable */
  FLEXRAY_EERICR_tag EERICR;           /* ECC Error Report And Injection Control */
  FLEXRAY_EERAR_tag EERAR;             /* ECC Error Report Address */
  FLEXRAY_EERDR_tag EERDR;             /* ECC Error Report Data */
  FLEXRAY_EERCR_tag EERCR;             /* ECC Error Report Code */
  FLEXRAY_EEIAR_tag EEIAR;             /* ECC Error Injection Address */
  FLEXRAY_EEIDR_tag EEIDR;             /* ECC Error Injection Data */
  FLEXRAY_EEICR_tag EEICR;             /* ECC Error Injection Code */
  uint8_t FLEXRAY_reserved2[1792];
  FLEXRAY_MB_tag MB[256];
  FLEXRAY_MBDOR_tag MBDOR[260];        /* MB Data Field Offset */
  uint8_t FLEXRAY_reserved3[8];
  FLEXRAY_LEETR_tag LEETR[6];          /* LRAM ECC Error Test Register */
};


/* ============================================================================
   =============================== Module: FXOSC ==============================
   ============================================================================ */

typedef union FXOSC_CTRL_union_tag {   /* FXOSC Control Register */
  vuint32_t R;
  struct {
    vuint32_t OSCON:1;                 /* Crystal oscillator power-down control */
    vuint32_t _unused_1:1;
    vuint32_t ALC_D:1;                 /* Automatic level controller enable */
    vuint32_t _unused_3:1;
    vuint32_t GM_SEL:4;                /* Crystal overdrive protection */
    vuint32_t _unused_8:8;
    vuint32_t EOCV:8;                  /* End of count value */
    vuint32_t COMP_EN:1;               /* Comparator enable */
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:4;
    vuint32_t _unused_30:1;
    vuint32_t OSC_BYP:1;               /* Oscillator bypass */
  } B;
} FXOSC_CTRL_tag;

typedef union FXOSC_STAT_union_tag {   /* Oscillator Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t OSC_STAT:1;              /* Crystal oscillator status */
  } B;
} FXOSC_STAT_tag;

struct FXOSC_tag {
  FXOSC_CTRL_tag CTRL;                 /* FXOSC Control Register */
  FXOSC_STAT_tag STAT;                 /* Oscillator Status Register */
};


/* ============================================================================
   =============================== Module: GPR0 ===============================
   ============================================================================ */

typedef union GPR0_LVFCCUGD0_union_tag { /* VFCCU Global DID register 0 */
  vuint32_t R;
  struct {
    vuint32_t FHID:4;                  /* Global static DID for VFCCU */
    vuint32_t _unused_4:28;
  } B;
} GPR0_LVFCCUGD0_tag;

typedef union GPR0_LVFCCULD0_union_tag { /* VFCCU Local DID register 0 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR0_LVFCCULD0_tag;

typedef union GPR0_LVFCCULD1_union_tag { /* VFCCU Local DID register 1 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR0_LVFCCULD1_tag;

typedef union GPR0_LVFCCULD2_union_tag { /* VFCCU Local DID register 2 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR0_LVFCCULD2_tag;

typedef union GPR0_FROSTICKC_union_tag { /* FlexRay OS Tick Mux Selector */
  vuint32_t R;
  struct {
    vuint32_t SEL:2;                   /* FlexRay OS Tick Mux Selector */
    vuint32_t _unused_2:30;
  } B;
} GPR0_FROSTICKC_tag;

typedef union GPR0_FUNCC0_union_tag {  /* Source Control for SINC HTRIGs */
  vuint32_t R;
  struct {
    vuint32_t CTRL:32;                 /* TRGMUX or eMIOS Source Selection for SINC HTRIGs */
  } B;
} GPR0_FUNCC0_tag;

typedef union GPR0_INITM0_union_tag {  /* NoC Initiator NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 0 NoC initiator timeout status */
  } B;
} GPR0_INITM0_tag;

typedef union GPR0_TARGTMS0_union_tag { /* NoC Target NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 0 NoC target timeout status */
  } B;
} GPR0_TARGTMS0_tag;

typedef union GPR0_TARGTMC0_union_tag { /* NoC Target NIU Timeout Control */
  vuint32_t R;
  struct {
    vuint32_t EN:32;                   /* Peripheral group 0 NoC target timeout enable */
  } B;
} GPR0_TARGTMC0_tag;

typedef union GPR0_CLKOUT0SEL_union_tag { /* CLKOUT_0 MUX select */
  vuint32_t R;
  struct {
    vuint32_t MUXSEL:6;                /* MUX select for CLKOUT_0 */
    vuint32_t CGEN:1;                  /* CG EN for DDR PLL PHI0 */
    vuint32_t _unused_7:25;
  } B;
} GPR0_CLKOUT0SEL_tag;

struct GPR0_tag {
  GPR0_LVFCCUGD0_tag LVFCCUGD0;        /* VFCCU Global DID register 0 */
  GPR0_LVFCCULD0_tag LVFCCULD0;        /* VFCCU Local DID register 0 */
  GPR0_LVFCCULD1_tag LVFCCULD1;        /* VFCCU Local DID register 1 */
  GPR0_LVFCCULD2_tag LVFCCULD2;        /* VFCCU Local DID register 2 */
  GPR0_FROSTICKC_tag FROSTICKC;        /* FlexRay OS Tick Mux Selector */
  uint8_t GPR0_reserved0[36];
  GPR0_FUNCC0_tag FUNCC0;              /* Source Control for SINC HTRIGs */
  uint8_t GPR0_reserved1[12];
  GPR0_INITM0_tag INITM0;              /* NoC Initiator NIU Timeout Status */
  GPR0_TARGTMS0_tag TARGTMS0;          /* NoC Target NIU Timeout Status */
  GPR0_TARGTMC0_tag TARGTMC0;          /* NoC Target NIU Timeout Control */
  uint8_t GPR0_reserved2[4];
  GPR0_CLKOUT0SEL_tag CLKOUT0SEL;      /* CLKOUT_0 MUX select */
};


/* ============================================================================
   =============================== Module: GPR0_PCTL ==========================
   ============================================================================ */

typedef union GPR0_PCTL_DSPI0PCTL_union_tag { /* SPI_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_DSPI0PCTL_tag;

typedef union GPR0_PCTL_FR0PCTL_union_tag { /* FlexRay_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for FlexRay_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_FR0PCTL_tag;

typedef union GPR0_PCTL_FR1PCTL_union_tag { /* FlexRay_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for FlexRay_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_FR1PCTL_tag;

typedef union GPR0_PCTL_I3C0PCTL_union_tag { /* I3C_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for I3C_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_I3C0PCTL_tag;

typedef union GPR0_PCTL_EDMA0PCTL_union_tag { /* eDMA_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL_0:1;                /* Clock Gating Control for eDMA_0 */
    vuint32_t PCTL_1:1;                /* Clock Gating Control for DMA_CRC_0 */
    vuint32_t PCTL_2:1;                /* Clock Gating Control for DMAMUX_0 */
    vuint32_t PCTL_3:1;                /* Clock Gating Control for PIT_0 */
    vuint32_t _unused_4:28;
  } B;
} GPR0_PCTL_EDMA0PCTL_tag;

typedef union GPR0_PCTL_MSCDSPIPCTL_union_tag { /* MSC_0_DSPI Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for MSC_0_DSPI */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_MSCDSPIPCTL_tag;

typedef union GPR0_PCTL_DSPI1PCTL_union_tag { /* SPI_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_DSPI1PCTL_tag;

typedef union GPR0_PCTL_LIN0PCTL_union_tag { /* LINFlex_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_LIN0PCTL_tag;

typedef union GPR0_PCTL_LIN1PCTL_union_tag { /* LINFlex_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_LIN1PCTL_tag;

typedef union GPR0_PCTL_LIN2PCTL_union_tag { /* LINFlex_2 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_2 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_LIN2PCTL_tag;

typedef union GPR0_PCTL_MSCLINPCTL_union_tag { /* MSC_0_LIN Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for MSC_0_LIN */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_MSCLINPCTL_tag;

typedef union GPR0_PCTL_ADC1PCTL_union_tag { /* ADC_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for ADC_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_ADC1PCTL_tag;

typedef union GPR0_PCTL_ADC0PCTL_union_tag { /* ADC_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for ADC_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_ADC0PCTL_tag;

typedef union GPR0_PCTL_PSI50PCTL_union_tag { /* PSI5_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for PSI5_0 clocks */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_PSI50PCTL_tag;

typedef union GPR0_PCTL_CTUPCTL_union_tag { /* CTU Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CTU */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_CTUPCTL_tag;

typedef union GPR0_PCTL_SIUL0PCTL_union_tag { /* SIUL2_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SIUL2_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_SIUL0PCTL_tag;

typedef union GPR0_PCTL_GTMNANOPCTL_union_tag { /* GTM NANO Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL_GTM:1;              /* Clock Gating Control for GTM */
    vuint32_t PCTL_NANO:1;             /* Clock Gating Control for NANO */
    vuint32_t _unused_2:30;
  } B;
} GPR0_PCTL_GTMNANOPCTL_tag;

typedef union GPR0_PCTL_PSI5S0PCTL_union_tag { /* PSI5_S_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for PSI5_S_0 clocks */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_PSI5S0PCTL_tag;

typedef union GPR0_PCTL_SINCPCTL_union_tag { /* SINC Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SINC clocks */
    vuint32_t _unused_1:31;
  } B;
} GPR0_PCTL_SINCPCTL_tag;

struct GPR0_PCTL_tag {
  uint8_t GPR0_PCTL_reserved0[4];
  GPR0_PCTL_DSPI0PCTL_tag DSPI0PCTL;   /* SPI_0 Clock Control Enable */
  GPR0_PCTL_FR0PCTL_tag FR0PCTL;       /* FlexRay_0 Clock Control Enable */
  GPR0_PCTL_FR1PCTL_tag FR1PCTL;       /* FlexRay_1 Clock Control Enable */
  GPR0_PCTL_I3C0PCTL_tag I3C0PCTL;     /* I3C_0 Clock Control Enable */
  GPR0_PCTL_EDMA0PCTL_tag EDMA0PCTL;   /* eDMA_0 Clock Control Enable */
  GPR0_PCTL_MSCDSPIPCTL_tag MSCDSPIPCTL; /* MSC_0_DSPI Clock Control Enable */
  GPR0_PCTL_DSPI1PCTL_tag DSPI1PCTL;   /* SPI_1 Clock Control Enable */
  GPR0_PCTL_LIN0PCTL_tag LIN0PCTL;     /* LINFlex_0 Clock Control Enable */
  GPR0_PCTL_LIN1PCTL_tag LIN1PCTL;     /* LINFlex_1 Clock Control Enable */
  GPR0_PCTL_LIN2PCTL_tag LIN2PCTL;     /* LINFlex_2 Clock Control Enable */
  GPR0_PCTL_MSCLINPCTL_tag MSCLINPCTL; /* MSC_0_LIN Clock Control Enable */
  GPR0_PCTL_ADC1PCTL_tag ADC1PCTL;     /* ADC_1 Clock Control Enable */
  GPR0_PCTL_ADC0PCTL_tag ADC0PCTL;     /* ADC_0 Clock Control Enable */
  uint8_t GPR0_PCTL_reserved1[20];
  GPR0_PCTL_PSI50PCTL_tag PSI50PCTL;   /* PSI5_0 Clock Control Enable */
  GPR0_PCTL_CTUPCTL_tag CTUPCTL;       /* CTU Clock Control Enable */
  GPR0_PCTL_SIUL0PCTL_tag SIUL0PCTL;   /* SIUL2_0 Clock Control Enable */
  GPR0_PCTL_GTMNANOPCTL_tag GTMNANOPCTL; /* GTM NANO Clock Control Enable */
  GPR0_PCTL_PSI5S0PCTL_tag PSI5S0PCTL; /* PSI5_S_0 Clock Control Enable */
  GPR0_PCTL_SINCPCTL_tag SINCPCTL;     /* SINC Clock Control Enable */
};


/* ============================================================================
   =============================== Module: GPR1 ===============================
   ============================================================================ */

typedef union GPR1_LVFCCUGD1_union_tag { /* VFCCU Global DID register 1 */
  vuint32_t R;
  struct {
    vuint32_t FHID:4;                  /* Global static DID for VFCCU */
    vuint32_t _unused_4:28;
  } B;
} GPR1_LVFCCUGD1_tag;

typedef union GPR1_LVFCCULD3_union_tag { /* VFCCU Local DID register 3 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR1_LVFCCULD3_tag;

typedef union GPR1_LVFCCULD4_union_tag { /* VFCCU Local DID register 4 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR1_LVFCCULD4_tag;

typedef union GPR1_LVFCCULD5_union_tag { /* VFCCU Local DID register 5 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR1_LVFCCULD5_tag;

typedef union GPR1_DESTC2_union_tag {  /* Source Control for RGMII TX CLK */
  vuint32_t R;
  struct {
    vuint32_t CTRL:32;                 /* MC_CGM_1 Source for RGMII TX CLK */
  } B;
} GPR1_DESTC2_tag;

typedef union GPR1_FUNCC2_union_tag {  /* Zipwire Configuration */
  vuint32_t R;
  struct {
    vuint32_t CTRL:32;                 /* Zipwire Configuration Control */
  } B;
} GPR1_FUNCC2_tag;

typedef union GPR1_INITM1_union_tag {  /* NoC Initiator NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 1 NoC initiator timeout status */
  } B;
} GPR1_INITM1_tag;

typedef union GPR1_TARGTMS1_union_tag { /* NoC Target NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 1 NoC target timeout status */
  } B;
} GPR1_TARGTMS1_tag;

typedef union GPR1_TARGTMC1_union_tag { /* NoC Target NIU Timeout Control */
  vuint32_t R;
  struct {
    vuint32_t EN:32;                   /* Peripheral group 1 NoC target timeout Enable */
  } B;
} GPR1_TARGTMC1_tag;

typedef union GPR1_NETCC0_union_tag {  /* NETC Control register 0 */
  vuint32_t R;
  struct {
    vuint32_t IERBLOCK:1;              /* Default value for IERB NETCRR[LOCK] bit. */
    vuint32_t _unused_1:31;
  } B;
} GPR1_NETCC0_tag;

typedef union GPR1_NETCC1_union_tag {  /* NETC Control register 1 */
  vuint32_t R;
  struct {
    vuint32_t LINK0MIIP:4;             /* Defines the MII protocol for L0MCAPR */
    vuint32_t LINK1MIIP:4;             /* Defines the MII protocol for L1MCAPR */
    vuint32_t _unused_8:24;
  } B;
} GPR1_NETCC1_tag;

typedef union GPR1_CLKOUT1SEL_union_tag { /* CLKOUT_1 MUX select */
  vuint32_t R;
  struct {
    vuint32_t MUXSEL:6;                /* MUX select for CLKOUT_1 */
    vuint32_t _unused_6:26;
  } B;
} GPR1_CLKOUT1SEL_tag;

typedef union GPR1_XPAR1_union_tag {   /* Interface parity control and status register */
  vuint32_t R;
  struct {
    vuint32_t DIS:1;                   /* NETC AXI parity disable */
    vuint32_t _unused_1:7;
    vuint32_t STAT:1;                  /* NETC AXI parity error status */
    vuint32_t _unused_9:23;
  } B;
} GPR1_XPAR1_tag;

struct GPR1_tag {
  GPR1_LVFCCUGD1_tag LVFCCUGD1;        /* VFCCU Global DID register 1 */
  GPR1_LVFCCULD3_tag LVFCCULD3;        /* VFCCU Local DID register 3 */
  GPR1_LVFCCULD4_tag LVFCCULD4;        /* VFCCU Local DID register 4 */
  GPR1_LVFCCULD5_tag LVFCCULD5;        /* VFCCU Local DID register 5 */
  uint8_t GPR1_reserved0[20];
  GPR1_DESTC2_tag DESTC2;              /* Source Control for RGMII TX CLK */
  uint8_t GPR1_reserved1[12];
  GPR1_FUNCC2_tag FUNCC2;              /* Zipwire Configuration */
  uint8_t GPR1_reserved2[12];
  GPR1_INITM1_tag INITM1;              /* NoC Initiator NIU Timeout Status */
  GPR1_TARGTMS1_tag TARGTMS1;          /* NoC Target NIU Timeout Status */
  GPR1_TARGTMC1_tag TARGTMC1;          /* NoC Target NIU Timeout Control */
  uint8_t GPR1_reserved3[4];
  GPR1_NETCC0_tag NETCC0;              /* NETC Control register 0 */
  GPR1_NETCC1_tag NETCC1;              /* NETC Control register 1 */
  GPR1_CLKOUT1SEL_tag CLKOUT1SEL;      /* CLKOUT_1 MUX select */
  uint8_t GPR1_reserved4[20];
  GPR1_XPAR1_tag XPAR1;                /* Interface parity control and status register */
};


/* ============================================================================
   =============================== Module: GPR1_PCTL ==========================
   ============================================================================ */

typedef union GPR1_PCTL_I3C1PCTL_union_tag { /* LPI2C_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LPI2C_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_I3C1PCTL_tag;

typedef union GPR1_PCTL_EDMA1PCTL_union_tag { /* eDMA_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL_0:1;                /* Clock Gating Control for eDMA_1 */
    vuint32_t PCTL_1:1;                /* Clock Gating Control for DMA_CRC_1 */
    vuint32_t PCTL_2:1;                /* Clock Gating Control for DMAMUX_1 */
    vuint32_t PCTL_3:1;                /* Clock Gating Control for PIT_1 */
    vuint32_t _unused_4:28;
  } B;
} GPR1_PCTL_EDMA1PCTL_tag;

typedef union GPR1_PCTL_DSPI2PCTL_union_tag { /* SPI_2 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_2 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_DSPI2PCTL_tag;

typedef union GPR1_PCTL_DSPI3PCTL_union_tag { /* SPI_3 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_3 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_DSPI3PCTL_tag;

typedef union GPR1_PCTL_DSPI4PCTL_union_tag { /* SPI_4 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_4 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_DSPI4PCTL_tag;

typedef union GPR1_PCTL_LIN3PCTL_union_tag { /* LINFlex_3 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_3 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_LIN3PCTL_tag;

typedef union GPR1_PCTL_LIN4PCTL_union_tag { /* LINFlex_4 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_4 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_LIN4PCTL_tag;

typedef union GPR1_PCTL_LIN5PCTL_union_tag { /* LINFlex_5 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_5 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_LIN5PCTL_tag;

typedef union GPR1_PCTL_SIPI0PCTL_union_tag { /* SIPI_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SIPI_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_SIPI0PCTL_tag;

typedef union GPR1_PCTL_SIPI1PCTL_union_tag { /* SIPI_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SIPI_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_SIPI1PCTL_tag;

typedef union GPR1_PCTL_SRX0PCTL_union_tag { /* SRX_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SRX_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_SRX0PCTL_tag;

typedef union GPR1_PCTL_ENET0PCTL_union_tag { /* ENET_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for NETC_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_ENET0PCTL_tag;

typedef union GPR1_PCTL_SIUL1PCTL_union_tag { /* SIUL2_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SIUL2_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR1_PCTL_SIUL1PCTL_tag;

struct GPR1_PCTL_tag {
  GPR1_PCTL_I3C1PCTL_tag I3C1PCTL;     /* LPI2C_1 Clock Control Enable */
  GPR1_PCTL_EDMA1PCTL_tag EDMA1PCTL;   /* eDMA_1 Clock Control Enable */
  GPR1_PCTL_DSPI2PCTL_tag DSPI2PCTL;   /* SPI_2 Clock Control Enable */
  GPR1_PCTL_DSPI3PCTL_tag DSPI3PCTL;   /* SPI_3 Clock Control Enable */
  GPR1_PCTL_DSPI4PCTL_tag DSPI4PCTL;   /* SPI_4 Clock Control Enable */
  GPR1_PCTL_LIN3PCTL_tag LIN3PCTL;     /* LINFlex_3 Clock Control Enable */
  GPR1_PCTL_LIN4PCTL_tag LIN4PCTL;     /* LINFlex_4 Clock Control Enable */
  GPR1_PCTL_LIN5PCTL_tag LIN5PCTL;     /* LINFlex_5 Clock Control Enable */
  GPR1_PCTL_SIPI0PCTL_tag SIPI0PCTL;   /* SIPI_0 Clock Control Enable */
  GPR1_PCTL_SIPI1PCTL_tag SIPI1PCTL;   /* SIPI_1 Clock Control Enable */
  GPR1_PCTL_SRX0PCTL_tag SRX0PCTL;     /* SRX_0 Clock Control Enable */
  uint8_t GPR1_PCTL_reserved0[4];
  GPR1_PCTL_ENET0PCTL_tag ENET0PCTL;   /* ENET_0 Clock Control Enable */
  uint8_t GPR1_PCTL_reserved1[4];
  GPR1_PCTL_SIUL1PCTL_tag SIUL1PCTL;   /* SIUL2_1 Clock Control Enable */
};


/* ============================================================================
   =============================== Module: GPR2 ===============================
   ============================================================================ */

typedef union GPR2_LVFCCUGD2_union_tag { /* VFCCU Global DID register 2 */
  vuint32_t R;
  struct {
    vuint32_t FHID:4;                  /* Global static DID for VFCCU */
    vuint32_t _unused_4:28;
  } B;
} GPR2_LVFCCUGD2_tag;

typedef union GPR2_LVFCCULD6_union_tag { /* VFCCU Local DID register 6 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR2_LVFCCULD6_tag;

typedef union GPR2_LVFCCULD7_union_tag { /* VFCCU Local DID register 7 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR2_LVFCCULD7_tag;

typedef union GPR2_LVFCCULD8_union_tag { /* VFCCU Local DID register 8 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR2_LVFCCULD8_tag;

typedef union GPR2_SPF2C_union_tag {   /* CEVA_SPF2 Control */
  vuint32_t R;
  struct {
    vuint32_t PGEN:1;                  /* Control PGEN in CEVA_SPF2 Power Down/Up */
    vuint32_t RET1N:1;                 /* Control RET1N in CEVA_SPF2 Power Down/Up */
    vuint32_t RET2N:1;                 /* Reserved bit */
    vuint32_t ACULOCK:1;               /* control acu_lock */
    vuint32_t ICULOCK:1;               /* control icu_lock */
    vuint32_t _unused_5:27;
  } B;
} GPR2_SPF2C_tag;

typedef union GPR2_FUNCC4_union_tag {  /* Miscellaneous Control */
  vuint32_t R;
  struct {
    vuint32_t CTRL:32;                 /* Miscellaneous Control */
  } B;
} GPR2_FUNCC4_tag;

typedef union GPR2_INITM2_union_tag {  /* NoC Initiator NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 2 system sub-NoC initiator timeout status */
  } B;
} GPR2_INITM2_tag;

typedef union GPR2_TARGTMS2_union_tag { /* NoC Target NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 2 system sub-NoC target timeout status */
  } B;
} GPR2_TARGTMS2_tag;

typedef union GPR2_TARGTMC2_union_tag { /* NoC Target NIU Timeout Control */
  vuint32_t R;
  struct {
    vuint32_t EN:32;                   /* Peripheral group 2 NoC target timeout Enable */
  } B;
} GPR2_TARGTMC2_tag;

typedef union GPR2_SMURSTCNT_union_tag { /* SMU Cortex-M33 Core Reset Counter */
  vuint32_t R;
  struct {
    vuint32_t CNTVAL:6;                /* Reset Counter Control */
    vuint32_t _unused_6:26;
  } B;
} GPR2_SMURSTCNT_tag;

typedef union GPR2_SMUS_union_tag {    /* SMU Cortex-M33 Core Status */
  vuint32_t R;
  struct {
    vuint32_t SLEEPDEEP:1;             /* Deep Sleep State */
    vuint32_t SLEEPHOLDAN:1;           /* SLEEPHOLDREQn Acknowledge */
    vuint32_t SLEEPING:1;              /* Sleep State */
    vuint32_t CORECLKEN:1;             /* Core Clock Enable */
    vuint32_t INTVFETCH:1;             /* Interrupt Vector Fetch State */
    vuint32_t CPULOCKUP:1;             /* Lockup State */
    vuint32_t CURRNS:1;                /* Current Security State */
    vuint32_t SYSRSTREQ:1;             /* System Reset Request */
    vuint32_t _unused_8:24;
  } B;
} GPR2_SMUS_tag;

typedef union GPR2_XPAR2_union_tag {   /* Interface parity control and status register */
  vuint32_t R;
  struct {
    vuint32_t DIS:5;                   /* NoC parity disable */
    vuint32_t _unused_5:3;
    vuint32_t STAT:5;                  /* NoC parity error status */
    vuint32_t _unused_13:19;
  } B;
} GPR2_XPAR2_tag;

struct GPR2_tag {
  GPR2_LVFCCUGD2_tag LVFCCUGD2;        /* VFCCU Global DID register 2 */
  GPR2_LVFCCULD6_tag LVFCCULD6;        /* VFCCU Local DID register 6 */
  GPR2_LVFCCULD7_tag LVFCCULD7;        /* VFCCU Local DID register 7 */
  GPR2_LVFCCULD8_tag LVFCCULD8;        /* VFCCU Local DID register 8 */
  GPR2_SPF2C_tag SPF2C;                /* CEVA_SPF2 Control */
  uint8_t GPR2_reserved0[36];
  GPR2_FUNCC4_tag FUNCC4;              /* Miscellaneous Control */
  uint8_t GPR2_reserved1[12];
  GPR2_INITM2_tag INITM2;              /* NoC Initiator NIU Timeout Status */
  GPR2_TARGTMS2_tag TARGTMS2;          /* NoC Target NIU Timeout Status */
  GPR2_TARGTMC2_tag TARGTMC2;          /* NoC Target NIU Timeout Control */
  uint8_t GPR2_reserved2[4];
  GPR2_SMURSTCNT_tag SMURSTCNT;        /* SMU Cortex-M33 Core Reset Counter */
  uint8_t GPR2_reserved3[4];
  GPR2_SMUS_tag SMUS;                  /* SMU Cortex-M33 Core Status */
  uint8_t GPR2_reserved4[28];
  GPR2_XPAR2_tag XPAR2;                /* Interface parity control and status register */
};


/* ============================================================================
   =============================== Module: GPR3 ===============================
   ============================================================================ */

typedef union GPR3_LVFCCUGD3_union_tag { /* VFCCU Global DID register 3 */
  vuint32_t R;
  struct {
    vuint32_t FHID:4;                  /* Global static DID for VFCCU */
    vuint32_t _unused_4:28;
  } B;
} GPR3_LVFCCUGD3_tag;

typedef union GPR3_LVFCCULD9_union_tag { /* VFCCU Local DID register 9 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR3_LVFCCULD9_tag;

typedef union GPR3_LVFCCULD10_union_tag { /* VFCCU Local DID register 10 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR3_LVFCCULD10_tag;

typedef union GPR3_LVFCCULD11_union_tag { /* VFCCU Local DID register 11 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR3_LVFCCULD11_tag;

typedef union GPR3_DDRCS_union_tag {   /* DDRC Status */
  vuint32_t R;
  struct {
    vuint32_t RCMC:1;                  /* DDRC Rapid Clear of Memory Complete */
    vuint32_t _unused_1:31;
  } B;
} GPR3_DDRCS_tag;

typedef union GPR3_DDRCC_union_tag {   /* DDRC Control */
  vuint32_t R;
  struct {
    vuint32_t CMR:1;                   /* DDRC Clear Memory Request */
    vuint32_t DSRR:1;                  /* DDRC DRAM Self-Refresh Request */
    vuint32_t _unused_2:30;
  } B;
} GPR3_DDRCC_tag;

typedef union GPR3_GENC0_union_tag {   /* Generic control register 0 */
  vuint32_t R;
  struct {
    vuint32_t CTRL0:32;                /* Generic control register for general purpose use */
  } B;
} GPR3_GENC0_tag;

typedef union GPR3_FUNCC6_union_tag {  /* Enable for TX Ack Suppress Logic */
  vuint32_t R;
  struct {
    vuint32_t CTRL:32;                 /* Enable for TX Ack Suppress Logic */
  } B;
} GPR3_FUNCC6_tag;

typedef union GPR3_INITM3_union_tag {  /* NoC Initiator NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 3 system NoC initiator timeout status */
  } B;
} GPR3_INITM3_tag;

typedef union GPR3_TARGTMS3_union_tag { /* NoC Target NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 3 system NoC target timeout status */
  } B;
} GPR3_TARGTMS3_tag;

typedef union GPR3_TARGTMC3_union_tag { /* NoC Target NIU Timeout Control */
  vuint32_t R;
  struct {
    vuint32_t EN:32;                   /* Peripheral group 3 NoC target timeout Enable */
  } B;
} GPR3_TARGTMC3_tag;

typedef union GPR3_RTU0FDC_union_tag { /* RTU0 Fence and Drain Control */
  vuint32_t R;
  struct {
    vuint32_t FDREQ_SYS_NOC:1;         /* RTU0 Fence and Drain Control */
    vuint32_t FDREQ_LLC:1;             /* RTU0 LLC I/F Fence and Drain Request */
    vuint32_t SRAM_KILL_ACCESS:1;      /* RTU0 SRAM Kill Access Request */
    vuint32_t _unused_3:29;
  } B;
} GPR3_RTU0FDC_tag;

typedef union GPR3_RTU0FDS_union_tag { /* RTU0 Fence and Drain Status */
  vuint32_t R;
  struct {
    vuint32_t IDLE_SYS_NOC:1;          /* RTU0 Fence and Drain Status */
    vuint32_t IDLE_LLC:1;              /* RTU0 LLC I/F Fence and Drain Idle status */
    vuint32_t _unused_2:30;
  } B;
} GPR3_RTU0FDS_tag;

typedef union GPR3_RTU1FDC_union_tag { /* RTU1 Fence and Drain Control */
  vuint32_t R;
  struct {
    vuint32_t FDREQ_SYS_NOC:1;         /* RTU1 Fence and Drain Control */
    vuint32_t FDREQ_LLC:1;             /* RTU1 LLC I/F Fence and Drain Request */
    vuint32_t SRAM_KILL_ACCESS:1;      /* RTU1 SRAM Kill Access Request */
    vuint32_t _unused_3:29;
  } B;
} GPR3_RTU1FDC_tag;

typedef union GPR3_RTU1FDS_union_tag { /* RTU1 Fence and Drain Status */
  vuint32_t R;
  struct {
    vuint32_t IDLE_SYS_NOC:1;          /* RTU1 Fence and Drain Status */
    vuint32_t IDLE_LLC:1;              /* RTU1 LLC I/F Fence and Drain Idle status */
    vuint32_t _unused_2:30;
  } B;
} GPR3_RTU1FDS_tag;

typedef union GPR3_CE0RSTCNT_union_tag { /* FlexLLCE Cortex-M33 Core 0 Reset Counter */
  vuint32_t R;
  struct {
    vuint32_t CNTVAL:6;                /* Reset Counter Control */
    vuint32_t _unused_6:26;
  } B;
} GPR3_CE0RSTCNT_tag;

typedef union GPR3_CE1RSTCNT_union_tag { /* FlexLLCE Cortex-M33 Core 1 Reset Counter */
  vuint32_t R;
  struct {
    vuint32_t CNTVAL:6;                /* Reset Counter Control */
    vuint32_t _unused_6:26;
  } B;
} GPR3_CE1RSTCNT_tag;

typedef union GPR3_CLKOUT4SEL_union_tag { /* CLKOUT_4 MUX select */
  vuint32_t R;
  struct {
    vuint32_t MUXSEL:4;                /* MUX select for CLKOUT_4 */
    vuint32_t _unused_4:28;
  } B;
} GPR3_CLKOUT4SEL_tag;

typedef union GPR3_RGMC_union_tag {    /* MC_RGM Control */
  vuint32_t R;
  struct {
    vuint32_t FUNC_RST_ENTRY_PROC_DONE:8; /* Reserved */
    vuint32_t FUNC_RST_EXIT_PROC_DONE:2; /* Reserved */
    vuint32_t DEST_RST_DISABLE:15;     /* Bypass for Destructive Reset Source */
    vuint32_t _unused_25:7;
  } B;
} GPR3_RGMC_tag;

typedef union GPR3_XRDC_INT_S_union_tag { /* XRDC Interrupt Source */
  vuint32_t R;
  struct {
    vuint32_t XRDC_INT_SRC:11;         /* XRDC Interrupt Source Status */
    vuint32_t _unused_11:21;
  } B;
} GPR3_XRDC_INT_S_tag;

typedef union GPR3_FLEXLLCECM33S_union_tag { /* FlexLLCE Cortex-M33 Core 0 Status */
  vuint32_t R;
  struct {
    vuint32_t SLEEPDEEP:1;             /* Deep Sleep State */
    vuint32_t SLEEPHOLDAN:1;           /* SLEEPHOLDREQn Acknowledge */
    vuint32_t SLEEPING:1;              /* Sleep State */
    vuint32_t CORECLKEN:1;             /* Core Clock Enable */
    vuint32_t INTVFETCH:1;             /* Interrupt Vector Fetch State */
    vuint32_t CPULOCKUP:1;             /* Lockup State */
    vuint32_t CURRNS:1;                /* Current Security State */
    vuint32_t SYSRSTREQ:1;             /* System Reset Request */
    vuint32_t _unused_8:24;
  } B;
} GPR3_FLEXLLCECM33S_tag;

typedef union GPR3_FLEXLLCECM33CES_union_tag { /* FlexLLCE Cortex-M33 Core 1 Status */
  vuint32_t R;
  struct {
    vuint32_t SLEEPDEEP:1;             /* Deep Sleep State */
    vuint32_t SLEEPHOLDAN:1;           /* SLEEPHOLDREQn Acknowledge */
    vuint32_t SLEEPING:1;              /* Sleep State */
    vuint32_t CORECLKEN:1;             /* Core Clock Enable */
    vuint32_t INTVFETCH:1;             /* Interrupt Vector Fetch State */
    vuint32_t CPULOCKUP:1;             /* Lockup State */
    vuint32_t CURRNS:1;                /* Current Security State */
    vuint32_t SYSRSTREQ:1;             /* System Reset Request */
    vuint32_t _unused_8:24;
  } B;
} GPR3_FLEXLLCECM33CES_tag;

typedef union GPR3_XPAR3_union_tag {   /* Interface parity control and status register */
  vuint32_t R;
  struct {
    vuint32_t DIS:6;                   /* NoC parity disable */
    vuint32_t _unused_6:2;
    vuint32_t STAT:6;                  /* NoC parity error status */
    vuint32_t _unused_14:18;
  } B;
} GPR3_XPAR3_tag;

typedef union GPR3_XPAR4_union_tag {   /* Interface parity control and status register */
  vuint32_t R;
  struct {
    vuint32_t DIS:6;                   /* NoC parity disable */
    vuint32_t _unused_6:2;
    vuint32_t STAT:6;                  /* NoC parity error status */
    vuint32_t _unused_14:18;
  } B;
} GPR3_XPAR4_tag;

typedef union GPR3_RTU0XPAR_union_tag { /* RTU0 global parity control register */
  vuint32_t R;
  struct {
    vuint32_t DIS:1;                   /* RTU0 global parity disable */
    vuint32_t _unused_1:31;
  } B;
} GPR3_RTU0XPAR_tag;

typedef union GPR3_RTU1XPAR_union_tag { /* RTU1 global parity control register */
  vuint32_t R;
  struct {
    vuint32_t DIS:1;                   /* RTU1 global parity disable */
    vuint32_t _unused_1:31;
  } B;
} GPR3_RTU1XPAR_tag;

struct GPR3_tag {
  GPR3_LVFCCUGD3_tag LVFCCUGD3;        /* VFCCU Global DID register 3 */
  GPR3_LVFCCULD9_tag LVFCCULD9;        /* VFCCU Local DID register 9 */
  GPR3_LVFCCULD10_tag LVFCCULD10;      /* VFCCU Local DID register 10 */
  GPR3_LVFCCULD11_tag LVFCCULD11;      /* VFCCU Local DID register 11 */
  GPR3_DDRCS_tag DDRCS;                /* DDRC Status */
  GPR3_DDRCC_tag DDRCC;                /* DDRC Control */
  GPR3_GENC0_tag GENC0;                /* Generic control register 0 */
  uint8_t GPR3_reserved0[36];
  GPR3_FUNCC6_tag FUNCC6;              /* Enable for TX Ack Suppress Logic */
  uint8_t GPR3_reserved1[12];
  GPR3_INITM3_tag INITM3;              /* NoC Initiator NIU Timeout Status */
  GPR3_TARGTMS3_tag TARGTMS3;          /* NoC Target NIU Timeout Status */
  GPR3_TARGTMC3_tag TARGTMC3;          /* NoC Target NIU Timeout Control */
  GPR3_RTU0FDC_tag RTU0FDC;            /* RTU0 Fence and Drain Control */
  GPR3_RTU0FDS_tag RTU0FDS;            /* RTU0 Fence and Drain Status */
  GPR3_RTU1FDC_tag RTU1FDC;            /* RTU1 Fence and Drain Control */
  GPR3_RTU1FDS_tag RTU1FDS;            /* RTU1 Fence and Drain Status */
  uint8_t GPR3_reserved2[4];
  GPR3_CE0RSTCNT_tag CE0RSTCNT;        /* FlexLLCE Cortex-M33 Core 0 Reset Counter */
  GPR3_CE1RSTCNT_tag CE1RSTCNT;        /* FlexLLCE Cortex-M33 Core 1 Reset Counter */
  GPR3_CLKOUT4SEL_tag CLKOUT4SEL;      /* CLKOUT_4 MUX select */
  GPR3_RGMC_tag RGMC;                  /* MC_RGM Control */
  GPR3_XRDC_INT_S_tag XRDC_INT_S;      /* XRDC Interrupt Source */
  uint8_t GPR3_reserved3[32];
  GPR3_FLEXLLCECM33S_tag FLEXLLCECM33S; /* FlexLLCE Cortex-M33 Core 0 Status */
  GPR3_FLEXLLCECM33CES_tag FLEXLLCECM33CES; /* FlexLLCE Cortex-M33 Core 1 Status */
  uint8_t GPR3_reserved4[12];
  GPR3_XPAR3_tag XPAR3;                /* Interface parity control and status register */
  GPR3_XPAR4_tag XPAR4;                /* Interface parity control and status register */
  GPR3_RTU0XPAR_tag RTU0XPAR;          /* RTU0 global parity control register */
  GPR3_RTU1XPAR_tag RTU1XPAR;          /* RTU1 global parity control register */
};


/* ============================================================================
   =============================== Module: GPR3_PCTL ==========================
   ============================================================================ */

typedef union GPR3_PCTL_EDMA3PCTL_union_tag { /* eDMA_3 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for eDMA_3 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_EDMA3PCTL_tag;

typedef union GPR3_PCTL_EDMACEPCTL_union_tag { /* CE_eDMA Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_eDMA */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_EDMACEPCTL_tag;

typedef union GPR3_PCTL_CAN0PCTL_union_tag { /* CE_CAN_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN0PCTL_tag;

typedef union GPR3_PCTL_CAN1PCTL_union_tag { /* CE_CAN_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN1PCTL_tag;

typedef union GPR3_PCTL_CAN2PCTL_union_tag { /* CE_CAN_2 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_2 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN2PCTL_tag;

typedef union GPR3_PCTL_CAN3PCTL_union_tag { /* CE_CAN_3 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_3 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN3PCTL_tag;

typedef union GPR3_PCTL_CAN4PCTL_union_tag { /* CE_CAN_4 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_4 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN4PCTL_tag;

typedef union GPR3_PCTL_CAN5PCTL_union_tag { /* CE_CAN_5 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_5 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN5PCTL_tag;

typedef union GPR3_PCTL_CAN6PCTL_union_tag { /* CE_CAN_6 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_6 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN6PCTL_tag;

typedef union GPR3_PCTL_CAN7PCTL_union_tag { /* CE_CAN_7 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_7 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN7PCTL_tag;

typedef union GPR3_PCTL_CAN8PCTL_union_tag { /* CE_CAN_8 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_8 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN8PCTL_tag;

typedef union GPR3_PCTL_CAN9PCTL_union_tag { /* CE_CAN_9 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_9 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN9PCTL_tag;

typedef union GPR3_PCTL_CAN10PCTL_union_tag { /* CE_CAN_10 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_10 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN10PCTL_tag;

typedef union GPR3_PCTL_CAN11PCTL_union_tag { /* CE_CAN_11 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_11 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN11PCTL_tag;

typedef union GPR3_PCTL_CAN12PCTL_union_tag { /* CE_CAN_12 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_12 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN12PCTL_tag;

typedef union GPR3_PCTL_CAN13PCTL_union_tag { /* CE_CAN_13 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_13 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN13PCTL_tag;

typedef union GPR3_PCTL_CAN14PCTL_union_tag { /* CE_CAN_14 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_14 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN14PCTL_tag;

typedef union GPR3_PCTL_CAN15PCTL_union_tag { /* CE_CAN_15 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_15 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN15PCTL_tag;

typedef union GPR3_PCTL_CAN16PCTL_union_tag { /* CE_CAN_16 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_16 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN16PCTL_tag;

typedef union GPR3_PCTL_CAN17PCTL_union_tag { /* CE_CAN_17 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_17 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN17PCTL_tag;

typedef union GPR3_PCTL_CAN18PCTL_union_tag { /* CE_CAN_18 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_18 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN18PCTL_tag;

typedef union GPR3_PCTL_CAN19PCTL_union_tag { /* CE_CAN_19 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_19 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN19PCTL_tag;

typedef union GPR3_PCTL_CAN20PCTL_union_tag { /* CE_CAN_20 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_20 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN20PCTL_tag;

typedef union GPR3_PCTL_CAN21PCTL_union_tag { /* CE_CAN_21 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_21 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN21PCTL_tag;

typedef union GPR3_PCTL_CAN22PCTL_union_tag { /* CE_CAN_22 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_22 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN22PCTL_tag;

typedef union GPR3_PCTL_CAN23PCTL_union_tag { /* CE_CAN_23 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_CAN_23 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_CAN23PCTL_tag;

typedef union GPR3_PCTL_PIT0PCTL_union_tag { /* CE_PIT_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_PIT_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_PIT0PCTL_tag;

typedef union GPR3_PCTL_PIT1PCTL_union_tag { /* CE_PIT_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_PIT_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_PIT1PCTL_tag;

typedef union GPR3_PCTL_PIT2PCTL_union_tag { /* CE_PIT_2 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_PIT_2 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_PIT2PCTL_tag;

typedef union GPR3_PCTL_PIT3PCTL_union_tag { /* CE_PIT_3 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_PIT_3 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_PIT3PCTL_tag;

typedef union GPR3_PCTL_PIT4PCTL_union_tag { /* CE_PIT_4 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_PIT_4 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_PIT4PCTL_tag;

typedef union GPR3_PCTL_PIT5PCTL_union_tag { /* CE_PIT_5 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for CE_PIT_5 */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_PIT5PCTL_tag;

typedef union GPR3_PCTL_RXLUTPCTL_union_tag { /* RXLUT Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for RXLUT */
    vuint32_t _unused_1:31;
  } B;
} GPR3_PCTL_RXLUTPCTL_tag;

struct GPR3_PCTL_tag {
  GPR3_PCTL_EDMA3PCTL_tag EDMA3PCTL;   /* eDMA_3 Clock Control Enable */
  GPR3_PCTL_EDMACEPCTL_tag EDMACEPCTL; /* CE_eDMA Clock Control Enable */
  uint8_t GPR3_PCTL_reserved0[4];
  GPR3_PCTL_CAN0PCTL_tag CAN0PCTL;     /* CE_CAN_0 Clock Control Enable */
  GPR3_PCTL_CAN1PCTL_tag CAN1PCTL;     /* CE_CAN_1 Clock Control Enable */
  GPR3_PCTL_CAN2PCTL_tag CAN2PCTL;     /* CE_CAN_2 Clock Control Enable */
  GPR3_PCTL_CAN3PCTL_tag CAN3PCTL;     /* CE_CAN_3 Clock Control Enable */
  GPR3_PCTL_CAN4PCTL_tag CAN4PCTL;     /* CE_CAN_4 Clock Control Enable */
  GPR3_PCTL_CAN5PCTL_tag CAN5PCTL;     /* CE_CAN_5 Clock Control Enable */
  GPR3_PCTL_CAN6PCTL_tag CAN6PCTL;     /* CE_CAN_6 Clock Control Enable */
  GPR3_PCTL_CAN7PCTL_tag CAN7PCTL;     /* CE_CAN_7 Clock Control Enable */
  GPR3_PCTL_CAN8PCTL_tag CAN8PCTL;     /* CE_CAN_8 Clock Control Enable */
  GPR3_PCTL_CAN9PCTL_tag CAN9PCTL;     /* CE_CAN_9 Clock Control Enable */
  GPR3_PCTL_CAN10PCTL_tag CAN10PCTL;   /* CE_CAN_10 Clock Control Enable */
  GPR3_PCTL_CAN11PCTL_tag CAN11PCTL;   /* CE_CAN_11 Clock Control Enable */
  GPR3_PCTL_CAN12PCTL_tag CAN12PCTL;   /* CE_CAN_12 Clock Control Enable */
  GPR3_PCTL_CAN13PCTL_tag CAN13PCTL;   /* CE_CAN_13 Clock Control Enable */
  GPR3_PCTL_CAN14PCTL_tag CAN14PCTL;   /* CE_CAN_14 Clock Control Enable */
  GPR3_PCTL_CAN15PCTL_tag CAN15PCTL;   /* CE_CAN_15 Clock Control Enable */
  GPR3_PCTL_CAN16PCTL_tag CAN16PCTL;   /* CE_CAN_16 Clock Control Enable */
  GPR3_PCTL_CAN17PCTL_tag CAN17PCTL;   /* CE_CAN_17 Clock Control Enable */
  GPR3_PCTL_CAN18PCTL_tag CAN18PCTL;   /* CE_CAN_18 Clock Control Enable */
  GPR3_PCTL_CAN19PCTL_tag CAN19PCTL;   /* CE_CAN_19 Clock Control Enable */
  GPR3_PCTL_CAN20PCTL_tag CAN20PCTL;   /* CE_CAN_20 Clock Control Enable */
  GPR3_PCTL_CAN21PCTL_tag CAN21PCTL;   /* CE_CAN_21 Clock Control Enable */
  GPR3_PCTL_CAN22PCTL_tag CAN22PCTL;   /* CE_CAN_22 Clock Control Enable */
  GPR3_PCTL_CAN23PCTL_tag CAN23PCTL;   /* CE_CAN_23 Clock Control Enable */
  GPR3_PCTL_PIT0PCTL_tag PIT0PCTL;     /* CE_PIT_0 Clock Control Enable */
  GPR3_PCTL_PIT1PCTL_tag PIT1PCTL;     /* CE_PIT_1 Clock Control Enable */
  GPR3_PCTL_PIT2PCTL_tag PIT2PCTL;     /* CE_PIT_2 Clock Control Enable */
  GPR3_PCTL_PIT3PCTL_tag PIT3PCTL;     /* CE_PIT_3 Clock Control Enable */
  GPR3_PCTL_PIT4PCTL_tag PIT4PCTL;     /* CE_PIT_4 Clock Control Enable */
  GPR3_PCTL_PIT5PCTL_tag PIT5PCTL;     /* CE_PIT_5 Clock Control Enable */
  GPR3_PCTL_RXLUTPCTL_tag RXLUTPCTL;   /* RXLUT Clock Control Enable */
};


/* ============================================================================
   =============================== Module: GPR4 ===============================
   ============================================================================ */

typedef union GPR4_LVFCCUGD4_union_tag { /* VFCCU Global DID register 4 */
  vuint32_t R;
  struct {
    vuint32_t FHID:4;                  /* Global static DID for VFCCU */
    vuint32_t _unused_4:28;
  } B;
} GPR4_LVFCCUGD4_tag;

typedef union GPR4_LVFCCULD12_union_tag { /* VFCCU Local DID register 12 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR4_LVFCCULD12_tag;

typedef union GPR4_LVFCCULD13_union_tag { /* VFCCU Local DID register 13 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR4_LVFCCULD13_tag;

typedef union GPR4_LVFCCULD14_union_tag { /* VFCCU Local DID register 14 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR4_LVFCCULD14_tag;

typedef union GPR4_INITM4_union_tag {  /* NoC Initiator NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 4 NoC initiator timeout status */
  } B;
} GPR4_INITM4_tag;

typedef union GPR4_TARGTMS4_union_tag { /* NoC Target NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 4 NoC target timeout status */
  } B;
} GPR4_TARGTMS4_tag;

typedef union GPR4_TARGTMC4_union_tag { /* NoC Target NIU Timeout Control */
  vuint32_t R;
  struct {
    vuint32_t EN:32;                   /* Peripheral group 4 NoC target timeout Enable */
  } B;
} GPR4_TARGTMC4_tag;

typedef union GPR4_CLKOUT2SEL_union_tag { /* CLKOUT_2 MUX select */
  vuint32_t R;
  struct {
    vuint32_t MUXSEL:5;                /* MUX select for CLKOUT_2 */
    vuint32_t _unused_5:27;
  } B;
} GPR4_CLKOUT2SEL_tag;

typedef union GPR4_MEW0MSTDIS_union_tag { /* MEW_0 Master ID Disable */
  vuint32_t R;
  struct {
    vuint32_t MSTRDIS:1;               /* MEW for QuadSPI_0 Master ID Disable */
    vuint32_t MID:6;                   /* QuadSPI_0 MASTERID */
    vuint32_t _unused_7:25;
  } B;
} GPR4_MEW0MSTDIS_tag;

typedef union GPR4_MEW1MSTDIS_union_tag { /* MEW_1 Master ID Disable */
  vuint32_t R;
  struct {
    vuint32_t MSTRDIS:1;               /* MEW for QuadSPI_1 Master ID Disable */
    vuint32_t MID:6;                   /* QuadSPI_1 MASTERID */
    vuint32_t _unused_7:25;
  } B;
} GPR4_MEW1MSTDIS_tag;

typedef union GPR4_HSE_SYS_FUSEC_union_tag { /* System Fuse Update Disable */
  vuint32_t R;
  struct {
    vuint32_t CTRL:1;                  /* System Fuse Lock Control */
    vuint32_t _unused_1:31;
  } B;
} GPR4_HSE_SYS_FUSEC_tag;

struct GPR4_tag {
  GPR4_LVFCCUGD4_tag LVFCCUGD4;        /* VFCCU Global DID register 4 */
  GPR4_LVFCCULD12_tag LVFCCULD12;      /* VFCCU Local DID register 12 */
  GPR4_LVFCCULD13_tag LVFCCULD13;      /* VFCCU Local DID register 13 */
  GPR4_LVFCCULD14_tag LVFCCULD14;      /* VFCCU Local DID register 14 */
  uint8_t GPR4_reserved0[52];
  GPR4_INITM4_tag INITM4;              /* NoC Initiator NIU Timeout Status */
  GPR4_TARGTMS4_tag TARGTMS4;          /* NoC Target NIU Timeout Status */
  GPR4_TARGTMC4_tag TARGTMC4;          /* NoC Target NIU Timeout Control */
  uint8_t GPR4_reserved1[4];
  GPR4_CLKOUT2SEL_tag CLKOUT2SEL;      /* CLKOUT_2 MUX select */
  uint8_t GPR4_reserved2[56];
  GPR4_MEW0MSTDIS_tag MEW0MSTDIS;      /* MEW_0 Master ID Disable */
  GPR4_MEW1MSTDIS_tag MEW1MSTDIS;      /* MEW_1 Master ID Disable */
  uint8_t GPR4_reserved3[8];
  GPR4_HSE_SYS_FUSEC_tag HSE_SYS_FUSEC; /* System Fuse Update Disable */
};


/* ============================================================================
   =============================== Module: GPR4_PCTL ==========================
   ============================================================================ */

typedef union GPR4_PCTL_QSPI0PCTL_union_tag { /* QuadSPI_0 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for QuadSPI_0 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_QSPI0PCTL_tag;

typedef union GPR4_PCTL_QSPI1PCTL_union_tag { /* QuadSPI_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for QuadSPI_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_QSPI1PCTL_tag;

typedef union GPR4_PCTL_EDMA4PCTL_union_tag { /* eDMA_4 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL_0:1;                /* Clock Gating Control for eDMA_4 */
    vuint32_t PCTL_1:1;                /* Clock Gating Control for DMA_CRC_4 */
    vuint32_t PCTL_2:1;                /* Clock Gating Control for DMAMUX_4 */
    vuint32_t PCTL_3:1;                /* Clock Gating Control for PIT_4 */
    vuint32_t _unused_4:28;
  } B;
} GPR4_PCTL_EDMA4PCTL_tag;

typedef union GPR4_PCTL_DSPI5PCTL_union_tag { /* SPI_5 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_5 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_DSPI5PCTL_tag;

typedef union GPR4_PCTL_DSPI6PCTL_union_tag { /* SPI_6 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_6 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_DSPI6PCTL_tag;

typedef union GPR4_PCTL_DSPI7PCTL_union_tag { /* SPI_7 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_7 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_DSPI7PCTL_tag;

typedef union GPR4_PCTL_LIN6PCTL_union_tag { /* LINFlex_6 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_6 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_LIN6PCTL_tag;

typedef union GPR4_PCTL_LIN7PCTL_union_tag { /* LINFlex_7 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_7 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_LIN7PCTL_tag;

typedef union GPR4_PCTL_LIN8PCTL_union_tag { /* LINFlex_8 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_8 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_LIN8PCTL_tag;

typedef union GPR4_PCTL_SDHCPCTL_union_tag { /* uSDHC Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for uSDHC */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_SDHCPCTL_tag;

typedef union GPR4_PCTL_SRX1PCTL_union_tag { /* SRX_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SRX_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_SRX1PCTL_tag;

typedef union GPR4_PCTL_I3C2PCTL_union_tag { /* LPI2C_2 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LPI2C_2 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_I3C2PCTL_tag;

typedef union GPR4_PCTL_PSI51PCTL_union_tag { /* PSI5_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for PSI5_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_PSI51PCTL_tag;

typedef union GPR4_PCTL_SIUL4PCTL_union_tag { /* SIUL2_4 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SIUL2_4 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_SIUL4PCTL_tag;

typedef union GPR4_PCTL_PSI5S1PCTL_union_tag { /* PSI5_S_1 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for PSI5_S_1 */
    vuint32_t _unused_1:31;
  } B;
} GPR4_PCTL_PSI5S1PCTL_tag;

struct GPR4_PCTL_tag {
  GPR4_PCTL_QSPI0PCTL_tag QSPI0PCTL;   /* QuadSPI_0 Clock Control Enable */
  GPR4_PCTL_QSPI1PCTL_tag QSPI1PCTL;   /* QuadSPI_1 Clock Control Enable */
  GPR4_PCTL_EDMA4PCTL_tag EDMA4PCTL;   /* eDMA_4 Clock Control Enable */
  GPR4_PCTL_DSPI5PCTL_tag DSPI5PCTL;   /* SPI_5 Clock Control Enable */
  GPR4_PCTL_DSPI6PCTL_tag DSPI6PCTL;   /* SPI_6 Clock Control Enable */
  GPR4_PCTL_DSPI7PCTL_tag DSPI7PCTL;   /* SPI_7 Clock Control Enable */
  GPR4_PCTL_LIN6PCTL_tag LIN6PCTL;     /* LINFlex_6 Clock Control Enable */
  GPR4_PCTL_LIN7PCTL_tag LIN7PCTL;     /* LINFlex_7 Clock Control Enable */
  GPR4_PCTL_LIN8PCTL_tag LIN8PCTL;     /* LINFlex_8 Clock Control Enable */
  GPR4_PCTL_SDHCPCTL_tag SDHCPCTL;     /* uSDHC Clock Control Enable */
  GPR4_PCTL_SRX1PCTL_tag SRX1PCTL;     /* SRX_1 Clock Control Enable */
  GPR4_PCTL_I3C2PCTL_tag I3C2PCTL;     /* LPI2C_2 Clock Control Enable */
  GPR4_PCTL_PSI51PCTL_tag PSI51PCTL;   /* PSI5_1 Clock Control Enable */
  GPR4_PCTL_SIUL4PCTL_tag SIUL4PCTL;   /* SIUL2_4 Clock Control Enable */
  GPR4_PCTL_PSI5S1PCTL_tag PSI5S1PCTL; /* PSI5_S_1 Clock Control Enable */
};


/* ============================================================================
   =============================== Module: GPR5 ===============================
   ============================================================================ */

typedef union GPR5_LVFCCUGD5_union_tag { /* VFCCU Global DID register 5 */
  vuint32_t R;
  struct {
    vuint32_t FHID:4;                  /* Global static DID for VFCCU */
    vuint32_t _unused_4:28;
  } B;
} GPR5_LVFCCUGD5_tag;

typedef union GPR5_LVFCCULD15_union_tag { /* VFCCU Local DID register 15 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR5_LVFCCULD15_tag;

typedef union GPR5_LVFCCULD16_union_tag { /* VFCCU Local DID register 16 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR5_LVFCCULD16_tag;

typedef union GPR5_LVFCCULD17_union_tag { /* VFCCU Local DID register 17 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR5_LVFCCULD17_tag;

typedef union GPR5_DESTC10_union_tag { /* CANXL_PE_CLK Configuration */
  vuint32_t R;
  struct {
    vuint32_t CTRL:32;                 /* CANXL_PE_CLK Configuration Control */
  } B;
} GPR5_DESTC10_tag;

typedef union GPR5_INITM5_union_tag {  /* NoC Initiator NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 5 NoC initiator timeout status */
  } B;
} GPR5_INITM5_tag;

typedef union GPR5_TARGTMS5_union_tag { /* NoC Target NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 5 NoC target timeout status */
  } B;
} GPR5_TARGTMS5_tag;

typedef union GPR5_TARGTMC5_union_tag { /* NoC Target NIU Timeout Control */
  vuint32_t R;
  struct {
    vuint32_t EN:32;                   /* Peripheral group 5 NoC target timeout Enable */
  } B;
} GPR5_TARGTMC5_tag;

typedef union GPR5_CLKOUT3SEL_union_tag { /* CLKOUT_3 MUX select */
  vuint32_t R;
  struct {
    vuint32_t MUXSEL:4;                /* MUX select for CLKOUT_3 */
    vuint32_t _unused_4:28;
  } B;
} GPR5_CLKOUT3SEL_tag;

struct GPR5_tag {
  GPR5_LVFCCUGD5_tag LVFCCUGD5;        /* VFCCU Global DID register 5 */
  GPR5_LVFCCULD15_tag LVFCCULD15;      /* VFCCU Local DID register 15 */
  GPR5_LVFCCULD16_tag LVFCCULD16;      /* VFCCU Local DID register 16 */
  GPR5_LVFCCULD17_tag LVFCCULD17;      /* VFCCU Local DID register 17 */
  uint8_t GPR5_reserved0[20];
  GPR5_DESTC10_tag DESTC10;            /* CANXL_PE_CLK Configuration */
  uint8_t GPR5_reserved1[28];
  GPR5_INITM5_tag INITM5;              /* NoC Initiator NIU Timeout Status */
  GPR5_TARGTMS5_tag TARGTMS5;          /* NoC Target NIU Timeout Status */
  GPR5_TARGTMC5_tag TARGTMC5;          /* NoC Target NIU Timeout Control */
  uint8_t GPR5_reserved2[4];
  GPR5_CLKOUT3SEL_tag CLKOUT3SEL;      /* CLKOUT_3 MUX select */
};


/* ============================================================================
   =============================== Module: GPR5_PCTL ==========================
   ============================================================================ */

typedef union GPR5_PCTL_EDMA5PCTL_union_tag { /* eDMA_5 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL_0:1;                /* Clock Gating Control for eDMA_5 */
    vuint32_t PCTL_1:1;                /* Clock Gating Control for DMA_CRC_5 */
    vuint32_t PCTL_2:1;                /* Clock Gating Control for DMAMUX_5 */
    vuint32_t PCTL_3:1;                /* Clock Gating Control for PIT_5 */
    vuint32_t _unused_4:28;
  } B;
} GPR5_PCTL_EDMA5PCTL_tag;

typedef union GPR5_PCTL_DSPI8PCTL_union_tag { /* SPI_8 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_8 */
    vuint32_t _unused_1:31;
  } B;
} GPR5_PCTL_DSPI8PCTL_tag;

typedef union GPR5_PCTL_DSPI9PCTL_union_tag { /* SPI_9 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gate Enable for SPI_9 */
    vuint32_t _unused_1:31;
  } B;
} GPR5_PCTL_DSPI9PCTL_tag;

typedef union GPR5_PCTL_LIN9PCTL_union_tag { /* LINFlex_9 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_9 */
    vuint32_t _unused_1:31;
  } B;
} GPR5_PCTL_LIN9PCTL_tag;

typedef union GPR5_PCTL_LIN10PCTL_union_tag { /* LINFlex_10 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_10 */
    vuint32_t _unused_1:31;
  } B;
} GPR5_PCTL_LIN10PCTL_tag;

typedef union GPR5_PCTL_LIN11PCTL_union_tag { /* LINFlex_11 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for LINFlex_11 */
    vuint32_t _unused_1:31;
  } B;
} GPR5_PCTL_LIN11PCTL_tag;

typedef union GPR5_PCTL_SIUL5PCTL_union_tag { /* SIUL2_5 Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for SIUL2_5 */
    vuint32_t _unused_1:31;
  } B;
} GPR5_PCTL_SIUL5PCTL_tag;

struct GPR5_PCTL_tag {
  GPR5_PCTL_EDMA5PCTL_tag EDMA5PCTL;   /* eDMA_5 Clock Control Enable */
  GPR5_PCTL_DSPI8PCTL_tag DSPI8PCTL;   /* SPI_8 Clock Control Enable */
  GPR5_PCTL_DSPI9PCTL_tag DSPI9PCTL;   /* SPI_9 Clock Control Enable */
  GPR5_PCTL_LIN9PCTL_tag LIN9PCTL;     /* LINFlex_9 Clock Control Enable */
  GPR5_PCTL_LIN10PCTL_tag LIN10PCTL;   /* LINFlex_10 Clock Control Enable */
  GPR5_PCTL_LIN11PCTL_tag LIN11PCTL;   /* LINFlex_11 Clock Control Enable */
  GPR5_PCTL_SIUL5PCTL_tag SIUL5PCTL;   /* SIUL2_5 Clock Control Enable */
};


/* ============================================================================
   =============================== Module: GPR6 ===============================
   ============================================================================ */

typedef union GPR6_LVFCCUGD6_union_tag { /* VFCCU Global DID register 6 */
  vuint32_t R;
  struct {
    vuint32_t FHID:4;                  /* Global static DID for VFCCU */
    vuint32_t _unused_4:28;
  } B;
} GPR6_LVFCCUGD6_tag;

typedef union GPR6_LVFCCULD18_union_tag { /* VFCCU Local DID register 18 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR6_LVFCCULD18_tag;

typedef union GPR6_LVFCCULD19_union_tag { /* VFCCU Local DID register 19 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR6_LVFCCULD19_tag;

typedef union GPR6_LVFCCULD20_union_tag { /* VFCCU Local DID register 20 */
  vuint32_t R;
  struct {
    vuint32_t FHID:32;                 /* Local static DID for VFCCU */
  } B;
} GPR6_LVFCCULD20_tag;

typedef union GPR6_TARGTMS6_union_tag { /* NoC Target NIU Timeout Status */
  vuint32_t R;
  struct {
    vuint32_t STAT:32;                 /* Peripheral group 6 system NoC target timeout status */
  } B;
} GPR6_TARGTMS6_tag;

typedef union GPR6_TARGTMC6_union_tag { /* NoC Target NIU Timeout Control */
  vuint32_t R;
  struct {
    vuint32_t EN:32;                   /* Peripheral group 6 NoC target timeout enable */
  } B;
} GPR6_TARGTMC6_tag;

typedef union GPR6_XPAR5_union_tag {   /* Interface parity control and status register */
  vuint32_t R;
  struct {
    vuint32_t DIS:1;                   /* NoC parity disable */
    vuint32_t _unused_1:7;
    vuint32_t STAT:1;                  /* NoC parity error status */
    vuint32_t _unused_9:23;
  } B;
} GPR6_XPAR5_tag;

struct GPR6_tag {
  GPR6_LVFCCUGD6_tag LVFCCUGD6;        /* VFCCU Global DID register 6 */
  GPR6_LVFCCULD18_tag LVFCCULD18;      /* VFCCU Local DID register 18 */
  GPR6_LVFCCULD19_tag LVFCCULD19;      /* VFCCU Local DID register 19 */
  GPR6_LVFCCULD20_tag LVFCCULD20;      /* VFCCU Local DID register 20 */
  uint8_t GPR6_reserved0[4];
  GPR6_TARGTMS6_tag TARGTMS6;          /* NoC Target NIU Timeout Status */
  GPR6_TARGTMC6_tag TARGTMC6;          /* NoC Target NIU Timeout Control */
  uint8_t GPR6_reserved1[28];
  GPR6_XPAR5_tag XPAR5;                /* Interface parity control and status register */
};


/* ============================================================================
   =============================== Module: GPR6_PCTL ==========================
   ============================================================================ */

typedef union GPR6_PCTL_DDRPCTL_union_tag { /* DDRC Clock Control Enable */
  vuint32_t R;
  struct {
    vuint32_t PCTL:1;                  /* Clock Gating Control for DDRC */
    vuint32_t _unused_1:31;
  } B;
} GPR6_PCTL_DDRPCTL_tag;

struct GPR6_PCTL_tag {
  GPR6_PCTL_DDRPCTL_tag DDRPCTL;       /* DDRC Clock Control Enable */
};


/* ============================================================================
   =============================== Module: GTMDI ==============================
   ============================================================================ */

typedef union GTMDI_DID_union_tag {    /* Device Identity */
  vuint32_t R;
  struct {
    vuint32_t IDCODE:1;                /* IDCODE Register ID */
    vuint32_t MIC:11;                  /* Manufacturer Identity Code */
    vuint32_t PIN:10;                  /* Part Identification Number */
    vuint32_t DC:6;                    /* Design Center */
    vuint32_t PRN:4;                   /* Part Revision Number */
  } B;
} GTMDI_DID_tag;

typedef union GTMDI_DC_union_tag {     /* GTMDI Development Control */
  vuint32_t R;
  struct {
    vuint32_t TSFT:1;                  /* Timestamp Force Transmission */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t SDBE:1;                  /* System Debug Enable */
    vuint32_t DBR:1;                   /* Debug Request */
    vuint32_t DBE:1;                   /* Debug Enable */
    vuint32_t _unused_5:3;             /* Reserved */
    vuint32_t CHR:1;                   /* Clear Halt Request */
    vuint32_t _unused_9:7;             /* Reserved */
    vuint32_t EOS1:5;                  /* EVTO1 Output Selection */
    vuint32_t _unused_21:3;            /* Reserved */
    vuint32_t EOS0:5;                  /* EVTO0 Output Selection */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t GDE:1;                   /* Global Debug Enable */
  } B;
} GTMDI_DC_tag;

typedef union GTMDI_DS_union_tag {     /* GTMDI Development Status */
  vuint32_t R;
  struct {
    vuint32_t HS2:16;                  /* Halt Status 2 */
    vuint32_t HS1:15;                  /* Halt Status 1 */
    vuint32_t HLT:1;                   /* Halt Indication */
  } B;
} GTMDI_DS_tag;

typedef union GTMDI_TIM_WPC1_union_tag { /* TIM Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t CHSEL2:3;                /* TIM Channel Selection 2 */
    vuint32_t _unused_3:3;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t SSEL2:3;                 /* TIM Sub-Module Source Selection 2 */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t TSS2:2;                  /* TIM Channel Slope Selection 2 */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t CHSEL1:3;                /* TIM Channel Selection 1 */
    vuint32_t _unused_19:3;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t SSEL1:3;                 /* TIM Sub-Module Source Selection 1 */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t TSS1:2;                  /* TIM Channel Slope Selection 1 */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_TIM_WPC1_tag;

typedef union GTMDI_TIM_WPC2_union_tag { /* TIM Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TIM_WPC2_tag;

typedef union GTMDI_TOM_WPC1_union_tag { /* TOM Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t CHSEL2:4;                /* TOM Channel Selection 2 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t SSEL2:3;                 /* TOM Sub-Module Source Selection 2 */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t TSS2:2;                  /* TOM Channel Slope Selection 2 */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t POL2:1;                  /* Polarity Selection 2 */
    vuint32_t CHSEL1:4;                /* TOM Channel Selection 1 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t SSEL1:3;                 /* TOM Sub-Module Source Selection 1 */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t TSS1:2;                  /* TOM Channel Slope Selection 1 */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t POL1:1;                  /* Polarity Selection 1 */
  } B;
} GTMDI_TOM_WPC1_tag;

typedef union GTMDI_TOM_WPC2_union_tag { /* TOM Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TOM_WPC2_tag;

typedef union GTMDI_ATOM_WPC1_union_tag { /* ATOM Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t CHSEL2:3;                /* ATOM Channel Selection 2 */
    vuint32_t _unused_3:3;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t SSEL2:4;                 /* ATOM Sub-Module Source Selection 2 */
    vuint32_t TSS2:2;                  /* ATOM Channel Slope Selection 2 */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t POL2:1;                  /* Polarity Selection 2 */
    vuint32_t CHSEL1:3;                /* ATOM Channel Selection 1 */
    vuint32_t _unused_19:3;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t SSEL1:4;                 /* ATOM Sub-Module Source Selection 1 */
    vuint32_t TSS1:2;                  /* ATOM Channel Slope Selection 1 */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t POL1:1;                  /* Polarity Selection 1 */
  } B;
} GTMDI_ATOM_WPC1_tag;

typedef union GTMDI_ATOM_WPC2_union_tag { /* ATOM Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_ATOM_WPC2_tag;

typedef union GTMDI_SPEA_WPC1_union_tag { /* SPEA Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t TSS2:2;                  /* SPEA DIR Slope Selection */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t _unused_7:10;            /* Reserved */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t TSS1:2;                  /* SPEA NIPD Slope Selection */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t SSEL:3;                  /* SPEA Sub-Module Source Selection */
    vuint32_t _unused_27:5;            /* Reserved */
  } B;
} GTMDI_SPEA_WPC1_tag;

typedef union GTMDI_SPEA_WPC2_union_tag { /* SPEA Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_SPEA_WPC2_tag;

typedef union GTMDI_SPEB_WPC1_union_tag { /* SPEB Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t TSS2:2;                  /* SPEB DIR Slope Selection */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t _unused_7:10;            /* Reserved */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t TSS1:2;                  /* SPEB NIPD Slope Selection */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t SSEL:3;                  /* SPEB Sub-Module Source Selection */
    vuint32_t _unused_27:5;            /* Reserved */
  } B;
} GTMDI_SPEB_WPC1_tag;

typedef union GTMDI_SPEB_WPC2_union_tag { /* SPEB Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_SPEB_WPC2_tag;

typedef union GTMDI_DPLL_WPC1_union_tag { /* DPLL Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t _unused_2:4;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t TSEL2A:2;                /* Watchpoint Selection 2 */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t TSEL2B:2;                /* Watchpoint Selection 2B */
    vuint32_t _unused_14:3;            /* Reserved */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t TSS1:2;                  /* TASI/SASI Slope Selection */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t TSEL1:2;                 /* Watchpoint Selection 1 */
    vuint32_t _unused_26:6;            /* Reserved */
  } B;
} GTMDI_DPLL_WPC1_tag;

typedef union GTMDI_DPLL_WPC2_union_tag { /* DPLL Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_DPLL_WPC2_tag;

typedef union GTMDI_DPLL_WPC3_union_tag { /* DPLL Watchpoint Control 3 */
  vuint32_t R;
  struct {
    vuint32_t RAM_DATA:24;             /* Data Value */
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t DMASK:1;                 /* Data Mask */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} GTMDI_DPLL_WPC3_tag;

typedef union GTMDI_DPLL_WPC4_union_tag { /* DPLL Watchpoint Control 4 */
  vuint32_t R;
  struct {
    vuint32_t RAM_ADDR_MASK:12;        /* RAM Address Mask */
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} GTMDI_DPLL_WPC4_tag;

typedef union GTMDI_DPLL_WPC5_union_tag { /* DPLL Watchpoint Control 5 */
  vuint32_t R;
  struct {
    vuint32_t RAM_ADDR:12;             /* RAM Address Compare */
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} GTMDI_DPLL_WPC5_tag;

typedef union GTMDI_DPLL_DTC_union_tag { /* DPLL Data Trace Control */
  vuint32_t R;
  struct {
    vuint32_t SEN:1;                   /* Start/Stop Enable */
    vuint32_t _unused_1:3;             /* Reserved */
    vuint32_t STSEL:2;                 /* Start/Stop Input Signal Selection 1 */
    vuint32_t DMC:1;                   /* Data Trace Message Control */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t RAMSEL:2;                /* AM Module Selection */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t RWC:2;                   /* Read/Write Control */
    vuint32_t _unused_14:18;           /* Reserved */
  } B;
} GTMDI_DPLL_DTC_tag;

typedef union GTMDI_ARU_WPC1_union_tag { /* ARU Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t _unused_8:4;             /* Reserved */
    vuint32_t TSS2:1;                  /* ARU Debugging Channel 1 Data Activity Selection */
    vuint32_t _unused_13:9;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t TSS1:1;                  /* ARU Debugging Channel 0 Data Activity Selection 1 */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} GTMDI_ARU_WPC1_tag;

typedef union GTMDI_ARU_WPC2_union_tag { /* ARU Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_ARU_WPC2_tag;

typedef union GTMDI_ARU_D0H_union_tag { /* ARU Watchpoint DATA0H */
  vuint32_t R;
  struct {
    vuint32_t DATA0H:29;               /* Watchpoint DATA0H */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} GTMDI_ARU_D0H_tag;

typedef union GTMDI_ARU_D0L_union_tag { /* ARU Watchpoint DATA0L */
  vuint32_t R;
  struct {
    vuint32_t DATA0L:29;               /* Watchpoint DATA0L */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} GTMDI_ARU_D0L_tag;

typedef union GTMDI_ARU_D1H_union_tag { /* ARU Watchpoint DATA1H */
  vuint32_t R;
  struct {
    vuint32_t DATA1H:29;               /* Watchpoint DATA1H */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} GTMDI_ARU_D1H_tag;

typedef union GTMDI_ARU_D1L_union_tag { /* ARU Watchpoint ARUD1L */
  vuint32_t R;
  struct {
    vuint32_t DATA1L:29;               /* Watchpoint ARUD1L */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} GTMDI_ARU_D1L_tag;

typedef union GTMDI_ARU_DTC_union_tag { /* ARU Data Trace Control */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t DMC2:1;                  /* Data Trace Message Control 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:10;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t DMC1:1;                  /* Data Trace Message Control 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:10;           /* Reserved */
  } B;
} GTMDI_ARU_DTC_tag;

typedef union GTMDI_MCS_HBC_union_tag { /* MCS Hardware Breakpoint (HBP) Control */
  vuint32_t R;
  struct {
    vuint32_t MCS0BE:1;                /* MCS 0 Breakpoint Enable */
    vuint32_t MCS1BE:1;                /* MCS 1 Breakpoint Enable */
    vuint32_t MCS2BE:1;                /* MCS 2 Breakpoint Enable */
    vuint32_t MCS3BE:1;                /* MCS 3 Breakpoint Enable */
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} GTMDI_MCS_HBC_tag;

typedef union GTMDI_MCS_HLT_union_tag { /* MCS Halt Control */
  vuint32_t R;
  struct {
    vuint32_t MCS0HBP0ICLR:1;          /* MCS 0 HBP 0 IRQ Clear */
    vuint32_t MCS0HBP1ICLR:1;          /* MCS 0 HBP 1 IRQ Clear */
    vuint32_t MCS1HBP0ICLR:1;          /* MCS 1 HBP 0 IRQ Clear */
    vuint32_t MCS1HBP1ICLR:1;          /* MCS 1 HBP 1 IRQ Clear */
    vuint32_t MCS2HBP0ICLR:1;          /* MCS 2 HBP 0 IRQ Clear */
    vuint32_t MCS2HBP1ICLR:1;          /* MCS 2 HBP 1 IRQ Clear */
    vuint32_t MCS3HBP0ICLR:1;          /* MCS 3 HBP 0 IRQ Clear */
    vuint32_t MCS3HBP1ICLR:1;          /* MCS 3 HBP 1 IRQ Clear */
    vuint32_t _unused_8:7;             /* Reserved */
    vuint32_t MCSSS:1;                 /* MCS Single Step */
    vuint32_t MCS0HBP0IS:1;            /* MCS0 HBPO IRQ STATE */
    vuint32_t MCS0HBP1IS:1;            /* MCS0 HBP1 IRQ STATE */
    vuint32_t MCS1HBP0IS:1;            /* MCS1 HBPO IRQ STATE */
    vuint32_t MCS1HBP1IS:1;            /* MCS1 HBP1 IRQ STATE */
    vuint32_t MCS2HBP0IS:1;            /* MCS2 HBPO IRQ STATE */
    vuint32_t MCS2HBP1IS:1;            /* MCS2 HBP1 IRQ STATE */
    vuint32_t MCS3HBP0IS:1;            /* MCS3 HBPO IRQ STATE */
    vuint32_t MCS3HBP1IS:1;            /* MCS3 HBP1 IRQ STATE */
    vuint32_t HALTSTAT:3;              /* Halt FSM Status */
    vuint32_t _unused_27:5;            /* Reserved */
  } B;
} GTMDI_MCS_HLT_tag;

typedef union GTMDI_MCSA_DC_union_tag { /* MCSA Development Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t MCS_SEL:2;               /* MCS_SEL Selection field */
    vuint32_t _unused_10:21;           /* Reserved */
    vuint32_t INDA:1;                  /* Instruction or Data trace */
  } B;
} GTMDI_MCSA_DC_tag;

typedef union GTMDI_MCSA_WPC_union_tag { /* MCSA Watchpoint Control */
  vuint32_t R;
  struct {
    vuint32_t SEN:1;                   /* Start/Stop Enable */
    vuint32_t TEN:1;                   /* Watchpoint Trigger Enable */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL:2;                 /* Start/Stop Input Signal Selection */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t HEN:1;                   /* Halt Enable */
    vuint32_t EOC:1;                   /* EVTO_b Control */
    vuint32_t _unused_9:5;             /* Reserved */
    vuint32_t HWO:2;                   /* Halt/Watchpoint Operand */
    vuint32_t HME:4;                   /* Halt/Watchpoint Data Mask Enable */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t WMC:1;                   /* Watchpoint Message Control */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t HACH:2;                  /* Halt/Watchpoint on Active Channel */
    vuint32_t _unused_26:6;            /* Reserved */
  } B;
} GTMDI_MCSA_WPC_tag;

typedef union GTMDI_MCSA_PTC_union_tag { /* MCSA Program Fetch Trace Control */
  vuint32_t R;
  struct {
    vuint32_t SEN:1;                   /* Start/Stop Enable */
    vuint32_t _unused_1:5;             /* Reserved */
    vuint32_t FTC:1;                   /* Fetch Trace Control */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t PFTE:2;                  /* Program Fetch Trace End */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t PFTS:2;                  /* Program Fetch Trace Start */
    vuint32_t _unused_14:18;           /* Reserved */
  } B;
} GTMDI_MCSA_PTC_tag;

typedef union GTMDI_MCSA_DTC_union_tag { /* MCSA Data Trace Control */
  vuint32_t R;
  struct {
    vuint32_t SEN:1;                   /* Start/Stop Enable */
    vuint32_t DTE:2;                   /* Data Trace End */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t DTS:2;                   /* Data Trace Start */
    vuint32_t DTC:1;                   /* Data Trace Control */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t RWT1:2;                  /* MCS Read/Write Trace Control 1 */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t RWT0:2;                  /* MCS Read/Write Trace Control 0 */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t RC1:1;                   /* Range Control 1 */
    vuint32_t RC0:1;                   /* Range Control 0 */
    vuint32_t _unused_18:14;           /* Reserved */
  } B;
} GTMDI_MCSA_DTC_tag;

typedef union GTMDI_MCSA_WPA1_union_tag { /* MCSA Watchpoint Address 1 */
  vuint32_t R;
  struct {
    vuint32_t HWA:14;                  /* Halt/Watchpoint Address */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t HWAM:14;                 /* Halt/Watchpoint Address Mask */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_MCSA_WPA1_tag;

typedef union GTMDI_MCSA_WPA2_union_tag { /* MCSA Watchpoint Address 2 */
  vuint32_t R;
  struct {
    vuint32_t HWA:14;                  /* Halt/Watchpoint Address */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t HWAM:14;                 /* Halt/Watchpoint Address Mask */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_MCSA_WPA2_tag;

typedef union GTMDI_MCSA_WPD1_union_tag { /* MCSA Watchpoint Data 1 */
  vuint32_t R;
  struct {
    vuint32_t HWD:32;                  /* Halt/Watchpoint Data */
  } B;
} GTMDI_MCSA_WPD1_tag;

typedef union GTMDI_MCSA_WPD2_union_tag { /* MCSA Watchpoint Data 2 */
  vuint32_t R;
  struct {
    vuint32_t HWD:32;                  /* Halt/Watchpoint Data */
  } B;
} GTMDI_MCSA_WPD2_tag;

typedef union GTMDI_MCSA_CE_union_tag { /* MCSA Channel Enable */
  vuint32_t R;
  struct {
    vuint32_t WPCE:8;                  /* MCS Channel Enable */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t CDTE:8;                  /* MCS Channel Data Trace Enable */
    vuint32_t CFTE:8;                  /* MCS Channel Fetch Trace Enable */
  } B;
} GTMDI_MCSA_CE_tag;

typedef union GTMDI_MCSA_DTAR1_union_tag { /* MCSA Data Trace Address Range 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t DTEA:13;                 /* Data Trace End Address */
    vuint32_t _unused_15:3;            /* Reserved */
    vuint32_t DTSA:13;                 /* Data Trace Start Address */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_MCSA_DTAR1_tag;

typedef union GTMDI_MCSA_DTAR2_union_tag { /* MCSA Data Trace Address Range 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t DTEA:13;                 /* Data Trace End Address */
    vuint32_t _unused_15:3;            /* Reserved */
    vuint32_t DTSA:13;                 /* Data Trace Start Address */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_MCSA_DTAR2_tag;

typedef union GTMDI_MCSB_DC_union_tag { /* MCSB Development Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t MCS_SEL:2;               /* MCS_SEL Selection field */
    vuint32_t _unused_10:21;           /* Reserved */
    vuint32_t INDA:1;                  /* Instruction or Data trace */
  } B;
} GTMDI_MCSB_DC_tag;

typedef union GTMDI_MCSB_WPC_union_tag { /* MCSB Watchpoint Control */
  vuint32_t R;
  struct {
    vuint32_t SEN:1;                   /* Start/Stop Enable */
    vuint32_t TEN:1;                   /* Watchpoint Trigger Enable */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL:2;                 /* Start/Stop Input Signal Selection */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t HEN:1;                   /* Halt Enable */
    vuint32_t EOC:1;                   /* EVTO_b Control */
    vuint32_t _unused_9:5;             /* Reserved */
    vuint32_t HWO:2;                   /* Halt/Watchpoint Operand */
    vuint32_t HME:4;                   /* Halt/Watchpoint Data Mask Enable */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t WMC:1;                   /* Watchpoint Message Control */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t HACH:2;                  /* Halt/Watchpoint on Active Channel */
    vuint32_t _unused_26:6;            /* Reserved */
  } B;
} GTMDI_MCSB_WPC_tag;

typedef union GTMDI_MCSB_PTC_union_tag { /* MCSB Program Fetch Trace Control */
  vuint32_t R;
  struct {
    vuint32_t SEN:1;                   /* Start/Stop Enable */
    vuint32_t _unused_1:5;             /* Reserved */
    vuint32_t FTC:1;                   /* Fetch Trace Control */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t PFTE:2;                  /* Program Fetch Trace End */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t PFTS:2;                  /* Program Fetch Trace Start */
    vuint32_t _unused_14:18;           /* Reserved */
  } B;
} GTMDI_MCSB_PTC_tag;

typedef union GTMDI_MCSB_DTC_union_tag { /* MCSB Data Trace Control */
  vuint32_t R;
  struct {
    vuint32_t SEN:1;                   /* Start/Stop Enable */
    vuint32_t DTE:2;                   /* Data Trace End */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t DTS:2;                   /* Data Trace Start */
    vuint32_t DTC:1;                   /* Data Trace Control */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t RWT1:2;                  /* MCS Read/Write Trace Control 1 */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t RWT0:2;                  /* MCS Read/Write Trace Control 0 */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t RC1:1;                   /* Range Control 1 */
    vuint32_t RC0:1;                   /* Range Control 0 */
    vuint32_t _unused_18:14;           /* Reserved */
  } B;
} GTMDI_MCSB_DTC_tag;

typedef union GTMDI_MCSB_WPA1_union_tag { /* MCSB Watchpoint Address 1 */
  vuint32_t R;
  struct {
    vuint32_t HWA:14;                  /* Halt/Watchpoint Address */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t HWAM:14;                 /* Halt/Watchpoint Address Mask */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_MCSB_WPA1_tag;

typedef union GTMDI_MCSB_WPA2_union_tag { /* MCSB Watchpoint Address 2 */
  vuint32_t R;
  struct {
    vuint32_t HWA:14;                  /* Halt/Watchpoint Address */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t HWAM:14;                 /* Halt/Watchpoint Address Mask */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_MCSB_WPA2_tag;

typedef union GTMDI_MCSB_WPD1_union_tag { /* MCSB Watchpoint Data 1 */
  vuint32_t R;
  struct {
    vuint32_t HWD:32;                  /* Halt/Watchpoint Data */
  } B;
} GTMDI_MCSB_WPD1_tag;

typedef union GTMDI_MCSB_WPD2_union_tag { /* MCSB Watchpoint Data 2 */
  vuint32_t R;
  struct {
    vuint32_t HWD:32;                  /* Halt/Watchpoint Data */
  } B;
} GTMDI_MCSB_WPD2_tag;

typedef union GTMDI_MCSB_CE_union_tag { /* MCSB Channel Enable */
  vuint32_t R;
  struct {
    vuint32_t WPCE:8;                  /* MCS Channel Enable */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t CDTE:8;                  /* MCS Channel Data Trace Enable */
    vuint32_t CFTE:8;                  /* MCS Channel Fetch Trace Enable */
  } B;
} GTMDI_MCSB_CE_tag;

typedef union GTMDI_MCSB_DTAR1_union_tag { /* MCSB Data Trace Address Range 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t DTEA:13;                 /* Data Trace End Address */
    vuint32_t _unused_15:3;            /* Reserved */
    vuint32_t DTSA:13;                 /* Data Trace Start Address */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_MCSB_DTAR1_tag;

typedef union GTMDI_MCSB_DTAR2_union_tag { /* MCSB Data Trace Address Range 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t DTEA:13;                 /* Data Trace End Address */
    vuint32_t _unused_15:3;            /* Reserved */
    vuint32_t DTSA:13;                 /* Data Trace Start Address */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_MCSB_DTAR2_tag;

typedef union GTMDI_TBU0_WPC1_union_tag { /* TBU0 Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t _unused_8:4;             /* Reserved */
    vuint32_t TSS2:3;                  /* TBU0 Watchpoint 2 Compare Selection 2 */
    vuint32_t _unused_15:7;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t TSS1:3;                  /* TBU0 Watchpoint 1 Compare Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TBU0_WPC1_tag;

typedef union GTMDI_TBU0_WPC2_union_tag { /* TBUn Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TBU0_WPC2_tag;

typedef union GTMDI_TBU0_D_union_tag { /* TBU0 Watchpoint DATA */
  vuint32_t R;
  struct {
    vuint32_t TBU0_DATA:27;            /* TBUn DATA */
    vuint32_t _unused_27:5;            /* Reserved */
  } B;
} GTMDI_TBU0_D_tag;

typedef union GTMDI_TBU1_WPC1_union_tag { /* TBUn Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t _unused_8:4;             /* Reserved */
    vuint32_t TSS2:2;                  /* TBUn Watchpoint 2 Compare Selection 2 */
    vuint32_t _unused_14:8;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t TSS1:2;                  /* TBUn Watchpoint 1 Compare Selection 1 */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_TBU1_WPC1_tag;

typedef union GTMDI_TBU1_WPC2_union_tag { /* TBUn Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TBU1_WPC2_tag;

typedef union GTMDI_TBU1_D_union_tag { /* TBUn Watchpoint DATA */
  vuint32_t R;
  struct {
    vuint32_t TBUN_DATA:24;            /* TBUn DATA */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} GTMDI_TBU1_D_tag;

typedef union GTMDI_TBU2_WPC1_union_tag { /* TBUn Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t _unused_8:4;             /* Reserved */
    vuint32_t TSS2:2;                  /* TBUn Watchpoint 2 Compare Selection 2 */
    vuint32_t _unused_14:8;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t TSS1:2;                  /* TBUn Watchpoint 1 Compare Selection 1 */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_TBU2_WPC1_tag;

typedef union GTMDI_TBU2_WPC2_union_tag { /* TBUn Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TBU2_WPC2_tag;

typedef union GTMDI_TBU2_D_union_tag { /* TBUn Watchpoint DATA */
  vuint32_t R;
  struct {
    vuint32_t TBUN_DATA:24;            /* TBUn DATA */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} GTMDI_TBU2_D_tag;

typedef union GTMDI_TBU3_WPC1_union_tag { /* TBUn Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t _unused_8:4;             /* Reserved */
    vuint32_t TSS2:2;                  /* TBUn Watchpoint 2 Compare Selection 2 */
    vuint32_t _unused_14:8;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t TSS1:2;                  /* TBUn Watchpoint 1 Compare Selection 1 */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_TBU3_WPC1_tag;

typedef union GTMDI_TBU3_WPC2_union_tag { /* TBUn Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TBU3_WPC2_tag;

typedef union GTMDI_TBU3_D_union_tag { /* TBUn Watchpoint DATA */
  vuint32_t R;
  struct {
    vuint32_t TBUN_DATA:24;            /* TBUn DATA */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} GTMDI_TBU3_D_tag;

typedef union GTMDI_TIO_IN_WPC1_union_tag { /* TIO IN watchpoint control register 1 */
  vuint32_t R;
  struct {
    vuint32_t CHSEL2:3;                /* TIO Channel Selection 2 */
    vuint32_t _unused_3:3;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t SSEL2:4;                 /* TIO Sub-Module Source Selection 2 */
    vuint32_t TSS2:2;                  /* TIO Channel Slope Selection 2 */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t CHSEL1:3;                /* TIO Channel Selection 1 */
    vuint32_t _unused_19:3;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t SSEL1:4;                 /* TIO Sub-Module Source Selection 1 */
    vuint32_t TSS1:2;                  /* TIO Channel Slope Selection 1 */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} GTMDI_TIO_IN_WPC1_tag;

typedef union GTMDI_TIO_IN_WPC2_union_tag { /* TIO IN watchpoint control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TIO_IN_WPC2_tag;

typedef union GTMDI_TIO_OUT_WPC1_union_tag { /* TIO OUT Watchpoint Control 1 */
  vuint32_t R;
  struct {
    vuint32_t CHSEL2:4;                /* TIO Channel Selection 2 */
    vuint32_t _unused_4:2;             /* Reserved */
    vuint32_t WMC2:1;                  /* Watchpoint Message Control 2 */
    vuint32_t HEN2:1;                  /* Halt Enable 2 */
    vuint32_t SSEL2:4;                 /* TIO Sub-Module Source Selection 2 */
    vuint32_t TSS2:2;                  /* TIO Channel Slope Selection 2 */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t POL2:1;                  /* Polarity Selection 2 */
    vuint32_t CHSEL1:4;                /* TIO Channel Selection 1 */
    vuint32_t _unused_20:2;            /* Reserved */
    vuint32_t WMC1:1;                  /* Watchpoint Message Control 1 */
    vuint32_t HEN1:1;                  /* Halt Enable 1 */
    vuint32_t SSEL1:4;                 /* TIO Sub-Module Source Selection 1 */
    vuint32_t TSS1:2;                  /* TIO Channel Slope Selection 1 */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t POL1:1;                  /* Polarity Selection 2 */
  } B;
} GTMDI_TIO_OUT_WPC1_tag;

typedef union GTMDI_TIO_OUT_WPC2_union_tag { /* TIO OUT Watchpoint Control 2 */
  vuint32_t R;
  struct {
    vuint32_t SEN2:1;                  /* Start/Stop Enable 2 */
    vuint32_t TEN2:1;                  /* Watchpoint Trigger Enable 2 */
    vuint32_t _unused_2:2;             /* Reserved */
    vuint32_t STSEL2:2;                /* Start/Stop Input Signal Selection 2 */
    vuint32_t _unused_6:6;             /* Reserved */
    vuint32_t WTSEL2:3;                /* Watchpoint Trigger Output Selection 2 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SEN1:1;                  /* Start/Stop Enable 1 */
    vuint32_t TEN1:1;                  /* Watchpoint Trigger Enable 1 */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t STSEL1:2;                /* Start/Stop Input Signal Selection 1 */
    vuint32_t _unused_22:6;            /* Reserved */
    vuint32_t WTSEL1:3;                /* Watchpoint Trigger Output Selection 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} GTMDI_TIO_OUT_WPC2_tag;

typedef union GTMDI_ATID_union_tag {   /* ATB Bus ID Control */
  vuint32_t R;
  struct {
    vuint32_t ATID:7;
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} GTMDI_ATID_tag;

typedef union GTMDI_DEVARCH_union_tag { /* Device Architecture */
  vuint32_t R;
  struct {
    vuint32_t ARCHID:16;               /* Architecture ID */
    vuint32_t ARCHREV:4;               /* Architecture Revision */
    vuint32_t PRESENT:1;               /* Architecture Present */
    vuint32_t ARCHITECT1:7;            /* Architecture 1 */
    vuint32_t ARCHITECT2:4;            /* Architecture 2 */
  } B;
} GTMDI_DEVARCH_tag;

typedef union GTMDI_DEVTYPE_union_tag { /* Device Type Identifier */
  vuint32_t R;
  struct {
    vuint32_t MAJOR:4;                 /* Major Type for Component Device Type */
    vuint32_t SUB:4;                   /* Sub Type for Component Device Type */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_DEVTYPE_tag;

typedef union GTMDI_PIDR4_union_tag {  /* Peripheral Identification 4 */
  vuint32_t R;
  struct {
    vuint32_t DES_2:4;                 /* DES */
    vuint32_t SIZE:4;                  /* SIZE */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_PIDR4_tag;

typedef union GTMDI_PIDR5_union_tag {  /* Peripheral Identification 5 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;            /* Reserved */
  } B;
} GTMDI_PIDR5_tag;

typedef union GTMDI_PIDR6_union_tag {  /* Peripheral Identification 6 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;            /* Reserved */
  } B;
} GTMDI_PIDR6_tag;

typedef union GTMDI_PIDR7_union_tag {  /* Peripheral Identification 7 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;            /* Reserved */
  } B;
} GTMDI_PIDR7_tag;

typedef union GTMDI_PIDR0_union_tag {  /* Peripheral Identification 0 */
  vuint32_t R;
  struct {
    vuint32_t PART_0:8;                /* Part Number Bits [7:0] */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_PIDR0_tag;

typedef union GTMDI_PIDR1_union_tag {  /* Peripheral Identification 1 */
  vuint32_t R;
  struct {
    vuint32_t PART_1:4;                /* Part Number Bits [11:8] */
    vuint32_t DES_0:4;                 /* JEP106 Identification Code Bits [3:0] */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_PIDR1_tag;

typedef union GTMDI_PIDR2_union_tag {  /* Peripheral Identification 2 */
  vuint32_t R;
  struct {
    vuint32_t DES_1:3;                 /* JEP106 Identification Code Bits [6:4] */
    vuint32_t JEDEC:1;                 /* JEDEC */
    vuint32_t REVISION:4;              /* Revision field */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_PIDR2_tag;

typedef union GTMDI_PIDR3_union_tag {  /* Peripheral Identification 3 */
  vuint32_t R;
  struct {
    vuint32_t CMOD:4;                  /* Customer Modified */
    vuint32_t REVAND:4;                /* REVAND */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_PIDR3_tag;

typedef union GTMDI_CIDR0_union_tag {  /* Component Identification 0 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_1:8;               /* Preamble Segment 0 */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_CIDR0_tag;

typedef union GTMDI_CIDR1_union_tag {  /* Component Identification 1 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_1:4;               /* Preamble Segment 1 */
    vuint32_t CLASS:4;                 /* Component class */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_CIDR1_tag;

typedef union GTMDI_CIDR2_union_tag {  /* Component Identification 2 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_2:8;               /* Preamble Segment 2 */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_CIDR2_tag;

typedef union GTMDI_CIDR3_union_tag {  /* Component Identification 3 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_3:8;               /* Preamble Segment 3 */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} GTMDI_CIDR3_tag;

struct GTMDI_tag {
  GTMDI_DID_tag DID;                   /* Device Identity */
  uint8_t GTMDI_reserved0[16];
  GTMDI_DC_tag DC;                     /* GTMDI Development Control */
  GTMDI_DS_tag DS;                     /* GTMDI Development Status */
  uint8_t GTMDI_reserved1[16];
  GTMDI_TIM_WPC1_tag TIM_WPC1;         /* TIM Watchpoint Control 1 */
  GTMDI_TIM_WPC2_tag TIM_WPC2;         /* TIM Watchpoint Control 2 */
  uint8_t GTMDI_reserved2[16];
  GTMDI_TOM_WPC1_tag TOM_WPC1;         /* TOM Watchpoint Control 1 */
  GTMDI_TOM_WPC2_tag TOM_WPC2;         /* TOM Watchpoint Control 2 */
  uint8_t GTMDI_reserved3[16];
  GTMDI_ATOM_WPC1_tag ATOM_WPC1;       /* ATOM Watchpoint Control 1 */
  GTMDI_ATOM_WPC2_tag ATOM_WPC2;       /* ATOM Watchpoint Control 2 */
  uint8_t GTMDI_reserved4[16];
  GTMDI_SPEA_WPC1_tag SPEA_WPC1;       /* SPEA Watchpoint Control 1 */
  GTMDI_SPEA_WPC2_tag SPEA_WPC2;       /* SPEA Watchpoint Control 2 */
  uint8_t GTMDI_reserved5[16];
  GTMDI_SPEB_WPC1_tag SPEB_WPC1;       /* SPEB Watchpoint Control 1 */
  GTMDI_SPEB_WPC2_tag SPEB_WPC2;       /* SPEB Watchpoint Control 2 */
  uint8_t GTMDI_reserved6[16];
  GTMDI_DPLL_WPC1_tag DPLL_WPC1;       /* DPLL Watchpoint Control 1 */
  GTMDI_DPLL_WPC2_tag DPLL_WPC2;       /* DPLL Watchpoint Control 2 */
  GTMDI_DPLL_WPC3_tag DPLL_WPC3;       /* DPLL Watchpoint Control 3 */
  GTMDI_DPLL_WPC4_tag DPLL_WPC4;       /* DPLL Watchpoint Control 4 */
  GTMDI_DPLL_WPC5_tag DPLL_WPC5;       /* DPLL Watchpoint Control 5 */
  GTMDI_DPLL_DTC_tag DPLL_DTC;         /* DPLL Data Trace Control */
  uint8_t GTMDI_reserved7[12];
  GTMDI_ARU_WPC1_tag ARU_WPC1;         /* ARU Watchpoint Control 1 */
  GTMDI_ARU_WPC2_tag ARU_WPC2;         /* ARU Watchpoint Control 2 */
  GTMDI_ARU_D0H_tag ARU_D0H;           /* ARU Watchpoint DATA0H */
  GTMDI_ARU_D0L_tag ARU_D0L;           /* ARU Watchpoint DATA0L */
  GTMDI_ARU_D1H_tag ARU_D1H;           /* ARU Watchpoint DATA1H */
  GTMDI_ARU_D1L_tag ARU_D1L;           /* ARU Watchpoint ARUD1L */
  GTMDI_ARU_DTC_tag ARU_DTC;           /* ARU Data Trace Control */
  uint8_t GTMDI_reserved8[4];
  GTMDI_MCS_HBC_tag MCS_HBC;           /* MCS Hardware Breakpoint (HBP) Control */
  GTMDI_MCS_HLT_tag MCS_HLT;           /* MCS Halt Control */
  GTMDI_MCSA_DC_tag MCSA_DC;           /* MCSA Development Control */
  uint8_t GTMDI_reserved9[4];
  GTMDI_MCSA_WPC_tag MCSA_WPC;         /* MCSA Watchpoint Control */
  GTMDI_MCSA_PTC_tag MCSA_PTC;         /* MCSA Program Fetch Trace Control */
  GTMDI_MCSA_DTC_tag MCSA_DTC;         /* MCSA Data Trace Control */
  GTMDI_MCSA_WPA1_tag MCSA_WPA1;       /* MCSA Watchpoint Address 1 */
  GTMDI_MCSA_WPA2_tag MCSA_WPA2;       /* MCSA Watchpoint Address 2 */
  GTMDI_MCSA_WPD1_tag MCSA_WPD1;       /* MCSA Watchpoint Data 1 */
  GTMDI_MCSA_WPD2_tag MCSA_WPD2;       /* MCSA Watchpoint Data 2 */
  GTMDI_MCSA_CE_tag MCSA_CE;           /* MCSA Channel Enable */
  uint8_t GTMDI_reserved10[4];
  GTMDI_MCSA_DTAR1_tag MCSA_DTAR1;     /* MCSA Data Trace Address Range 1 */
  GTMDI_MCSA_DTAR2_tag MCSA_DTAR2;     /* MCSA Data Trace Address Range 2 */
  uint8_t GTMDI_reserved11[16];
  GTMDI_MCSB_DC_tag MCSB_DC;           /* MCSB Development Control */
  uint8_t GTMDI_reserved12[4];
  GTMDI_MCSB_WPC_tag MCSB_WPC;         /* MCSB Watchpoint Control */
  GTMDI_MCSB_PTC_tag MCSB_PTC;         /* MCSB Program Fetch Trace Control */
  GTMDI_MCSB_DTC_tag MCSB_DTC;         /* MCSB Data Trace Control */
  GTMDI_MCSB_WPA1_tag MCSB_WPA1;       /* MCSB Watchpoint Address 1 */
  GTMDI_MCSB_WPA2_tag MCSB_WPA2;       /* MCSB Watchpoint Address 2 */
  GTMDI_MCSB_WPD1_tag MCSB_WPD1;       /* MCSB Watchpoint Data 1 */
  GTMDI_MCSB_WPD2_tag MCSB_WPD2;       /* MCSB Watchpoint Data 2 */
  GTMDI_MCSB_CE_tag MCSB_CE;           /* MCSB Channel Enable */
  uint8_t GTMDI_reserved13[4];
  GTMDI_MCSB_DTAR1_tag MCSB_DTAR1;     /* MCSB Data Trace Address Range 1 */
  GTMDI_MCSB_DTAR2_tag MCSB_DTAR2;     /* MCSB Data Trace Address Range 2 */
  uint8_t GTMDI_reserved14[16];
  GTMDI_TBU0_WPC1_tag TBU0_WPC1;       /* TBU0 Watchpoint Control 1 */
  GTMDI_TBU0_WPC2_tag TBU0_WPC2;       /* TBUn Watchpoint Control 2 */
  GTMDI_TBU0_D_tag TBU0_D;             /* TBU0 Watchpoint DATA */
  uint8_t GTMDI_reserved15[16];
  GTMDI_TBU1_WPC1_tag TBU1_WPC1;       /* TBUn Watchpoint Control 1 */
  GTMDI_TBU1_WPC2_tag TBU1_WPC2;       /* TBUn Watchpoint Control 2 */
  GTMDI_TBU1_D_tag TBU1_D;             /* TBUn Watchpoint DATA */
  uint8_t GTMDI_reserved16[16];
  GTMDI_TBU2_WPC1_tag TBU2_WPC1;       /* TBUn Watchpoint Control 1 */
  GTMDI_TBU2_WPC2_tag TBU2_WPC2;       /* TBUn Watchpoint Control 2 */
  GTMDI_TBU2_D_tag TBU2_D;             /* TBUn Watchpoint DATA */
  uint8_t GTMDI_reserved17[16];
  GTMDI_TBU3_WPC1_tag TBU3_WPC1;       /* TBUn Watchpoint Control 1 */
  GTMDI_TBU3_WPC2_tag TBU3_WPC2;       /* TBUn Watchpoint Control 2 */
  GTMDI_TBU3_D_tag TBU3_D;             /* TBUn Watchpoint DATA */
  uint8_t GTMDI_reserved18[16];
  GTMDI_TIO_IN_WPC1_tag TIO_IN_WPC1;   /* TIO IN watchpoint control register 1 */
  GTMDI_TIO_IN_WPC2_tag TIO_IN_WPC2;   /* TIO IN watchpoint control register 2 */
  uint8_t GTMDI_reserved19[16];
  GTMDI_TIO_OUT_WPC1_tag TIO_OUT_WPC1; /* TIO OUT Watchpoint Control 1 */
  GTMDI_TIO_OUT_WPC2_tag TIO_OUT_WPC2; /* TIO OUT Watchpoint Control 2 */
  uint8_t GTMDI_reserved20[352];
  GTMDI_ATID_tag ATID;                 /* ATB Bus ID Control */
  uint8_t GTMDI_reserved21[3152];
  GTMDI_DEVARCH_tag DEVARCH;           /* Device Architecture */
  uint8_t GTMDI_reserved22[12];
  GTMDI_DEVTYPE_tag DEVTYPE;           /* Device Type Identifier */
  GTMDI_PIDR4_tag PIDR4;               /* Peripheral Identification 4 */
  GTMDI_PIDR5_tag PIDR5;               /* Peripheral Identification 5 */
  GTMDI_PIDR6_tag PIDR6;               /* Peripheral Identification 6 */
  GTMDI_PIDR7_tag PIDR7;               /* Peripheral Identification 7 */
  GTMDI_PIDR0_tag PIDR0;               /* Peripheral Identification 0 */
  GTMDI_PIDR1_tag PIDR1;               /* Peripheral Identification 1 */
  GTMDI_PIDR2_tag PIDR2;               /* Peripheral Identification 2 */
  GTMDI_PIDR3_tag PIDR3;               /* Peripheral Identification 3 */
  GTMDI_CIDR0_tag CIDR0;               /* Component Identification 0 */
  GTMDI_CIDR1_tag CIDR1;               /* Component Identification 1 */
  GTMDI_CIDR2_tag CIDR2;               /* Component Identification 2 */
  GTMDI_CIDR3_tag CIDR3;               /* Component Identification 3 */
};


/* ============================================================================
   =============================== Module: GTMSS ==============================
   ============================================================================ */

typedef union GTMSS_IRQ_CFG0_union_tag { /* IRQ Configuration Register for Cluster 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;             /* Reserved */
    vuint32_t FIFO0:1;                 /* FIFO0_CH0_IRQ select bit */
    vuint32_t FIFO1:1;                 /* FIFO0_CH1_IRQ select bit */
    vuint32_t FIFO2:1;                 /* FIFO0_CH2_IRQ select bit */
    vuint32_t FIFO3:1;                 /* FIFO0_CH3_IRQ select bit */
    vuint32_t TIM6:1;                  /* TIM0_CH6_IRQ select bit */
    vuint32_t TIM7:1;                  /* TIM0_CH7_IRQ select bit */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t ATOM1:1;                 /* ATOM0_CH1_IRQ select bit */
    vuint32_t ATOM3:1;                 /* ATOM0_CH3_IRQ select bit */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t MCS0:1;                  /* MCS0_CH0_IRQ select bit */
    vuint32_t MCS1:1;                  /* MCS0_CH1_IRQ select bit */
    vuint32_t MCS2:1;                  /* MCS0_CH2_IRQ select bit */
    vuint32_t MCS3:1;                  /* MCS0_CH3_IRQ select bit */
    vuint32_t _unused_20:11;           /* Reserved */
    vuint32_t LCK:1;                   /* Lock bit */
  } B;
} GTMSS_IRQ_CFG0_tag;

typedef union GTMSS_IRQ_CFG1_union_tag { /* IRQ Confoguration Register for Cluster 1 */
  vuint32_t R;
  struct {
    vuint32_t TIO3:1;                  /* TIO1_G0_CH3_IRQ select bit */
    vuint32_t TIO5:1;                  /* TIO1_G0_CH5_IRQ select bit */
    vuint32_t TIO7:1;                  /* TIO1_G0_CH7_IRQ select bit */
    vuint32_t _unused_3:5;             /* Reserved */
    vuint32_t TIM6:1;                  /* TIM1_CH6_IRQ select bit */
    vuint32_t TIM7:1;                  /* TIM1_CH7_IRQ select bit */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t ATOM1:1;                 /* ATOM1_CH1_IRQ select bit */
    vuint32_t ATOM3:1;                 /* ATOM1_CH3_IRQ select bit */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t MCS0:1;                  /* MCS1_CH0_IRQ select bit */
    vuint32_t MCS1:1;                  /* MCS1_CH1_IRQ select bit */
    vuint32_t MCS2:1;                  /* MCS1_CH2_IRQ select bit */
    vuint32_t MCS3:1;                  /* MCS1_CH3_IRQ select bit */
    vuint32_t _unused_20:11;           /* Reserved */
    vuint32_t LCK:1;                   /* Lock bit */
  } B;
} GTMSS_IRQ_CFG1_tag;

typedef union GTMSS_IRQ_CFG2_union_tag { /* IRQ Confoguration Register for Cluster 2 */
  vuint32_t R;
  struct {
    vuint32_t TIO3:1;                  /* TIO2_G0_CH3_IRQ select bit */
    vuint32_t TIO5:1;                  /* TIO2_G0_CH5_IRQ select bit */
    vuint32_t TIO7:1;                  /* TIO2_G0_CH7_IRQ select bit */
    vuint32_t _unused_3:5;             /* Reserved */
    vuint32_t TIM6:1;                  /* TIM2_CH6_IRQ select bit */
    vuint32_t TIM7:1;                  /* TIM2_CH7_IRQ select bit */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t ATOM1:1;                 /* ATOM2_CH1_IRQ select bit */
    vuint32_t ATOM3:1;                 /* ATOM2_CH3_IRQ select bit */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t MCS0:1;                  /* MCS2_CH0_IRQ select bit */
    vuint32_t MCS1:1;                  /* MCS2_CH1_IRQ select bit */
    vuint32_t MCS2:1;                  /* MCS2_CH2_IRQ select bit */
    vuint32_t MCS3:1;                  /* MCS2_CH3_IRQ select bit */
    vuint32_t _unused_20:11;           /* Reserved */
    vuint32_t LCK:1;                   /* Lock bit */
  } B;
} GTMSS_IRQ_CFG2_tag;

typedef union GTMSS_IRQ_CFG3_union_tag { /* IRQ Confoguration Register for Cluster 3 */
  vuint32_t R;
  struct {
    vuint32_t TIO3:1;                  /* TIO3_G0_CH3_IRQ select bit */
    vuint32_t TIO5:1;                  /* TIO3_G0_CH5_IRQ select bit */
    vuint32_t TIO7:1;                  /* TIO3_G0_CH7_IRQ select bit */
    vuint32_t _unused_3:9;             /* Reserved */
    vuint32_t ATOM1:1;                 /* ATOM3_CH1_IRQ select bit */
    vuint32_t ATOM3:1;                 /* ATOM3_CH3_IRQ select bit */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t MCS0:1;                  /* MCS3_CH0_IRQ select bit */
    vuint32_t MCS1:1;                  /* MCS3_CH1_IRQ select bit */
    vuint32_t MCS2:1;                  /* MCS3_CH2_IRQ select bit */
    vuint32_t MCS3:1;                  /* MCS3_CH3_IRQ select bit */
    vuint32_t _unused_20:11;           /* Reserved */
    vuint32_t LCK:1;                   /* Lock bit */
  } B;
} GTMSS_IRQ_CFG3_tag;

typedef union GTMSS_DBG_DID_union_tag { /* Debug domain ID register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;             /* Reserved */
    vuint32_t DID:4;                   /* Debug domain ID selector */
    vuint32_t _unused_7:23;            /* Reserved */
    vuint32_t EN:1;                    /* DID enable bit */
    vuint32_t LCK:1;                   /* Lock bit */
  } B;
} GTMSS_DBG_DID_tag;

struct GTMSS_tag {
  GTMSS_IRQ_CFG0_tag IRQ_CFG0;         /* IRQ Configuration Register for Cluster 0 */
  GTMSS_IRQ_CFG1_tag IRQ_CFG1;         /* IRQ Confoguration Register for Cluster 1 */
  GTMSS_IRQ_CFG2_tag IRQ_CFG2;         /* IRQ Confoguration Register for Cluster 2 */
  GTMSS_IRQ_CFG3_tag IRQ_CFG3;         /* IRQ Confoguration Register for Cluster 3 */
  uint8_t GTMSS_reserved0[8176];
  GTMSS_DBG_DID_tag DBG_DID;           /* Debug domain ID register */
};


/* ============================================================================
   =============================== Module: GTM_gtm_cls0 =======================
   ============================================================================ */

typedef union GTM_gtm_cls0_GTM_REV_union_tag { /* GTM version control register */
  vuint32_t R;
  struct {
    vuint32_t REL_ITER:4;
    vuint32_t REL_BASE:8;
    vuint32_t _unused_12:4;
    vuint32_t VENDOR_CODE:4;
    vuint32_t DEVICE_CODE:4;
    vuint32_t VER_MINOR:4;
    vuint32_t VER_MAJOR:4;
  } B;
} GTM_gtm_cls0_GTM_REV_tag;

typedef union GTM_gtm_cls0_GTM_RST_union_tag { /* GTM global reset register */
  vuint32_t R;
  struct {
    vuint32_t RST:1;
    vuint32_t _unused_1:26;
    vuint32_t BRIDGE_MODE_WRDIS:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_GTM_RST_tag;

typedef union GTM_gtm_cls0_GTM_CTRL_union_tag { /* GTM global control register */
  vuint32_t R;
  struct {
    vuint32_t RF_PROT:1;
    vuint32_t TO_MODE:2;
    vuint32_t _unused_3:1;
    vuint32_t TO_VAL:8;
    vuint32_t AEIM_CLUSTER:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_GTM_CTRL_tag;

typedef union GTM_gtm_cls0_GTM_CFG_union_tag { /* GTM configuration register */
  vuint32_t R;
  struct {
    vuint32_t SRC_IN_MUX:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_GTM_CFG_tag;

typedef union GTM_gtm_cls0_GTM_AEI_ADDR_XPT_union_tag { /* GTM AEI timeout exception address register */
  vuint32_t R;
  struct {
    vuint32_t TO_ADDR:21;
    vuint32_t _unused_21:3;
    vuint32_t TO_W1R0:1;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_GTM_AEI_ADDR_XPT_tag;

typedef union GTM_gtm_cls0_GTM_AEI_STA_XPT_union_tag { /* GTM AEI non zero status register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:21;
    vuint32_t _unused_21:3;
    vuint32_t W1R0:1;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_GTM_AEI_STA_XPT_tag;

typedef union GTM_gtm_cls0_GTM_IRQ_NOTIFY_union_tag { /* GTM Interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t AEI_TO_XPT:1;
    vuint32_t AEI_USP_ADDR:1;
    vuint32_t AEI_IM_ADDR:1;
    vuint32_t AEI_USP_BE:1;
    vuint32_t AEIM_USP_ADDR:1;
    vuint32_t AEIM_IM_ADDR:1;
    vuint32_t AEIM_USP_BE:1;
    vuint32_t CLK_EN_ERR:1;
    vuint32_t CLK_PER_ERR:1;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_GTM_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_GTM_IRQ_EN_union_tag { /* GTM interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t AEI_TO_XPT_IRQ_EN:1;
    vuint32_t AEI_USP_ADDR_IRQ_EN:1;
    vuint32_t AEI_IM_ADDR_IRQ_EN:1;
    vuint32_t AEI_USP_BE_IRQ_EN:1;
    vuint32_t AEIM_USP_ADDR_IRQ_EN:1;
    vuint32_t AEIM_IM_ADDR_IRQ_EN:1;
    vuint32_t AEIM_USP_BE_IRQ_EN:1;
    vuint32_t CLK_EN_ERR_IRQ_EN:1;
    vuint32_t CLK_PER_ERR_IRQ_EN:1;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_GTM_IRQ_EN_tag;

typedef union GTM_gtm_cls0_GTM_EIRQ_EN_union_tag { /* GTM error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t AEI_TO_XPT_EIRQ_EN:1;
    vuint32_t AEI_USP_ADDR_EIRQ_EN:1;
    vuint32_t AEI_IM_ADDR_EIRQ_EN:1;
    vuint32_t AEI_USP_BE_EIRQ_EN:1;
    vuint32_t AEIM_USP_ADDR_EIRQ_EN:1;
    vuint32_t AEIM_IM_ADDR_EIRQ_EN:1;
    vuint32_t AEIM_USP_BE_EIRQ_EN:1;
    vuint32_t CLK_EN_ERR_EIRQ_EN:1;
    vuint32_t CLK_PER_ERR_EIRQ_EN:1;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_GTM_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_GTM_IRQ_FORCINT_union_tag { /* GTM Software interrupt generation register */
  vuint32_t R;
  struct {
    vuint32_t TRG_AEI_TO_XPT:1;
    vuint32_t TRG_AEI_USP_ADDR:1;
    vuint32_t TRG_AEI_IM_ADDR:1;
    vuint32_t TRG_AEI_USP_BE:1;
    vuint32_t TRG_AEIM_USP_ADDR:1;
    vuint32_t TRG_AEIM_IM_ADDR:1;
    vuint32_t TRG_AEIM_USP_BE:1;
    vuint32_t TRG_CLK_EN_ERR:1;
    vuint32_t TRG_CLK_PER_ERR:1;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_GTM_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_GTM_IRQ_MODE_union_tag { /* GTM top level interrupts mode selection */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_GTM_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_GTM_CLS_CLK_CFG_union_tag { /* GTM Cluster Clock Configuration */
  vuint32_t R;
  struct {
    vuint32_t CLS0_CLK_DIV:2;
    vuint32_t CLS1_CLK_DIV:2;
    vuint32_t CLS2_CLK_DIV:2;
    vuint32_t CLS3_CLK_DIV:2;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_GTM_CLS_CLK_CFG_tag;

typedef union GTM_gtm_cls0_GTM_ARU_COM_DIS_union_tag { /* GTM ARU communication disable */
  vuint32_t R;
  struct {
    vuint32_t CLS0_DIS:1;
    vuint32_t CLS1_DIS:1;
    vuint32_t CLS2_DIS:1;
    vuint32_t CLS3_DIS:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_GTM_ARU_COM_DIS_tag;

typedef union GTM_gtm_cls0_BRIDGE_MODE_union_tag { /* GTM AEI bridge mode register */
  vuint32_t R;
  struct {
    vuint32_t BRG_MODE:1;
    vuint32_t MSK_WR_RSP:1;
    vuint32_t BYPASS_SYNC:1;
    vuint32_t _unused_3:5;
    vuint32_t MODE_UP_PGR:1;
    vuint32_t BUFF_OVL:1;
    vuint32_t _unused_10:2;
    vuint32_t SYNC_INPUT_REG:1;
    vuint32_t _unused_13:3;
    vuint32_t BRG_RST:1;
    vuint32_t _unused_17:7;
    vuint32_t BUFF_DPT:8;
  } B;
} GTM_gtm_cls0_BRIDGE_MODE_tag;

typedef union GTM_gtm_cls0_BRIDGE_PTR1_union_tag { /* GTM AEI bridge pointer 1 register */
  vuint32_t R;
  struct {
    vuint32_t NEW_TRAN_PTR:5;
    vuint32_t FIRST_RSP_PTR:5;
    vuint32_t TRAN_IN_PGR:5;
    vuint32_t ABT_TRAN_PGR:5;
    vuint32_t FBC:6;
    vuint32_t RSP_TRAN_RDY:6;
  } B;
} GTM_gtm_cls0_BRIDGE_PTR1_tag;

typedef union GTM_gtm_cls0_BRIDGE_PTR2_union_tag { /* GTM AEI bridge pointer 2 register */
  vuint32_t R;
  struct {
    vuint32_t TRAN_IN_PGR2:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_BRIDGE_PTR2_tag;

typedef union GTM_gtm_cls0_MCS_AEM_DIS_union_tag { /* GTM MCS master port disable register */
  vuint32_t R;
  struct {
    vuint32_t DIS_CLS0:2;
    vuint32_t DIS_CLS1:2;
    vuint32_t DIS_CLS2:2;
    vuint32_t DIS_CLS3:2;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS_AEM_DIS_tag;

typedef union GTM_gtm_cls0_CMU_CLK_EN_union_tag { /* CMU clock enable */
  vuint32_t R;
  struct {
    vuint32_t EN_CLK0:2;
    vuint32_t EN_CLK1:2;
    vuint32_t EN_CLK2:2;
    vuint32_t EN_CLK3:2;
    vuint32_t EN_CLK4:2;
    vuint32_t EN_CLK5:2;
    vuint32_t EN_CLK6:2;
    vuint32_t EN_CLK7:2;
    vuint32_t EN_ECLK0:2;
    vuint32_t EN_ECLK1:2;
    vuint32_t EN_ECLK2:2;
    vuint32_t EN_FXCLK:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_CLK_EN_tag;

typedef union GTM_gtm_cls0_CMU_GCLK_NUM_union_tag { /* The numerator for CMU global clock resolution generator */
  vuint32_t R;
  struct {
    vuint32_t GCLK_NUM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_GCLK_NUM_tag;

typedef union GTM_gtm_cls0_CMU_GCLK_DEN_union_tag { /* The denominator for CMU global clock resolution generator */
  vuint32_t R;
  struct {
    vuint32_t GCLK_DEN:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_GCLK_DEN_tag;

typedef union GTM_gtm_cls0_CMU_CLK_0_CTRL_union_tag { /* CMU control for clock resolution generator [x] */
  vuint32_t R;
  struct {
    vuint32_t CLK_CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_CLK_0_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_CLK_1_CTRL_union_tag { /* CMU control for clock resolution generator [x] */
  vuint32_t R;
  struct {
    vuint32_t CLK_CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_CLK_1_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_CLK_2_CTRL_union_tag { /* CMU control for clock resolution generator [x] */
  vuint32_t R;
  struct {
    vuint32_t CLK_CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_CLK_2_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_CLK_3_CTRL_union_tag { /* CMU control for clock resolution generator [x] */
  vuint32_t R;
  struct {
    vuint32_t CLK_CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_CLK_3_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_CLK_4_CTRL_union_tag { /* CMU control for clock resolution generator [x] */
  vuint32_t R;
  struct {
    vuint32_t CLK_CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_CLK_4_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_CLK_5_CTRL_union_tag { /* CMU control for clock resolution generator [x] */
  vuint32_t R;
  struct {
    vuint32_t CLK_CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_CLK_5_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_CLK_6_CTRL_union_tag { /* CMU control for clock resolution generator 6 */
  vuint32_t R;
  struct {
    vuint32_t CLK_CNT:24;
    vuint32_t CLK_SEL:2;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_CMU_CLK_6_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_CLK_7_CTRL_union_tag { /* CMU control for clock resolution generator 7 */
  vuint32_t R;
  struct {
    vuint32_t CLK_CNT:24;
    vuint32_t CLK_SEL:2;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_CMU_CLK_7_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_ECLK_0_NUM_union_tag { /* The numerator for the external clock resolution generator [z] */
  vuint32_t R;
  struct {
    vuint32_t ECLK_NUM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_ECLK_0_NUM_tag;

typedef union GTM_gtm_cls0_CMU_ECLK_0_DEN_union_tag { /* The denominator for the external clock resolution generator [z] */
  vuint32_t R;
  struct {
    vuint32_t ECLK_DEN:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_ECLK_0_DEN_tag;

typedef union GTM_gtm_cls0_CMU_ECLK_1_NUM_union_tag { /* The numerator for the external clock resolution generator [z] */
  vuint32_t R;
  struct {
    vuint32_t ECLK_NUM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_ECLK_1_NUM_tag;

typedef union GTM_gtm_cls0_CMU_ECLK_1_DEN_union_tag { /* The denominator for the external clock resolution generator [z] */
  vuint32_t R;
  struct {
    vuint32_t ECLK_DEN:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_ECLK_1_DEN_tag;

typedef union GTM_gtm_cls0_CMU_ECLK_2_NUM_union_tag { /* The numerator for the external clock resolution generator [z] */
  vuint32_t R;
  struct {
    vuint32_t ECLK_NUM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_ECLK_2_NUM_tag;

typedef union GTM_gtm_cls0_CMU_ECLK_2_DEN_union_tag { /* The denominator for the external clock resolution generator [z] */
  vuint32_t R;
  struct {
    vuint32_t ECLK_DEN:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CMU_ECLK_2_DEN_tag;

typedef union GTM_gtm_cls0_CMU_FXCLK_CTRL_union_tag { /* CMU control for selection of FCR subblock input */
  vuint32_t R;
  struct {
    vuint32_t FXCLK_SEL:4;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_CMU_FXCLK_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_GLB_CTRL_union_tag { /* CMU synchronizing ARU and clock source */
  vuint32_t R;
  struct {
    vuint32_t ARU_ADDR_RSTGLB:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_CMU_GLB_CTRL_tag;

typedef union GTM_gtm_cls0_CMU_CLK_CTRL_union_tag { /* CMU control for clock resolution generator */
  vuint32_t R;
  struct {
    vuint32_t CLK0_EXT_DIVIDER:1;
    vuint32_t CLK1_EXT_DIVIDER:1;
    vuint32_t CLK2_EXT_DIVIDER:1;
    vuint32_t CLK3_EXT_DIVIDER:1;
    vuint32_t CLK4_EXT_DIVIDER:1;
    vuint32_t CLK5_EXT_DIVIDER:1;
    vuint32_t CLK6_EXT_DIVIDER:1;
    vuint32_t CLK7_EXT_DIVIDER:1;
    vuint32_t CLK8_EXT_DIVIDER:1;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_CMU_CLK_CTRL_tag;

typedef union GTM_gtm_cls0_TBU_CHEN_union_tag { /* TBU global channel enable */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CH0:2;
    vuint32_t ENDIS_CH1:2;
    vuint32_t ENDIS_CH2:2;
    vuint32_t ENDIS_CH3:2;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_TBU_CHEN_tag;

typedef union GTM_gtm_cls0_TBU_CH0_CTRL_union_tag { /* TBU channel 0 control */
  vuint32_t R;
  struct {
    vuint32_t LOW_RES:1;
    vuint32_t CH_CLK_SRC:3;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_TBU_CH0_CTRL_tag;

typedef union GTM_gtm_cls0_TBU_CH0_BASE_union_tag { /* TBU channel 0 base */
  vuint32_t R;
  struct {
    vuint32_t BASE:27;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls0_TBU_CH0_BASE_tag;

typedef union GTM_gtm_cls0_TBU_CH1_CTRL_union_tag { /* TBU channel 1 control */
  vuint32_t R;
  struct {
    vuint32_t CH_MODE:1;
    vuint32_t CH_CLK_SRC:3;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_TBU_CH1_CTRL_tag;

typedef union GTM_gtm_cls0_TBU_CH1_BASE_union_tag { /* TBU channel [x] base */
  vuint32_t R;
  struct {
    vuint32_t BASE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TBU_CH1_BASE_tag;

typedef union GTM_gtm_cls0_TBU_CH2_CTRL_union_tag { /* TBU channel 2 control */
  vuint32_t R;
  struct {
    vuint32_t CH_MODE:1;
    vuint32_t CH_CLK_SRC:3;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_TBU_CH2_CTRL_tag;

typedef union GTM_gtm_cls0_TBU_CH2_BASE_union_tag { /* TBU channel [x] base */
  vuint32_t R;
  struct {
    vuint32_t BASE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TBU_CH2_BASE_tag;

typedef union GTM_gtm_cls0_TBU_CH3_CTRL_union_tag { /* TBU channel 3 control */
  vuint32_t R;
  struct {
    vuint32_t CH_MODE:1;
    vuint32_t _unused_1:3;
    vuint32_t USE_CH2:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_TBU_CH3_CTRL_tag;

typedef union GTM_gtm_cls0_TBU_CH3_BASE_union_tag { /* TBU channel 3 base */
  vuint32_t R;
  struct {
    vuint32_t BASE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TBU_CH3_BASE_tag;

typedef union GTM_gtm_cls0_TBU_CH3_BASE_MARK_union_tag { /* TBU channel 3 modulo value */
  vuint32_t R;
  struct {
    vuint32_t BASE_MARK:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TBU_CH3_BASE_MARK_tag;

typedef union GTM_gtm_cls0_TBU_CH3_BASE_CAPTURE_union_tag { /* TBU channel 3 base captured */
  vuint32_t R;
  struct {
    vuint32_t BASE_CAPTURE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TBU_CH3_BASE_CAPTURE_tag;

typedef union GTM_gtm_cls0_ARU_ACCESS_union_tag { /* ARU access register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t RREQ:1;
    vuint32_t WREQ:1;
    vuint32_t _unused_14:18;
  } B;
} GTM_gtm_cls0_ARU_ACCESS_tag;

typedef union GTM_gtm_cls0_ARU_DATA_H_union_tag { /* ARU access register upper data word */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_ARU_DATA_H_tag;

typedef union GTM_gtm_cls0_ARU_DATA_L_union_tag { /* ARU access register lower data word */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_ARU_DATA_L_tag;

typedef union GTM_gtm_cls0_ARU_DBG_ACCESS0_union_tag { /* ARU debug access channel 0 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_ARU_DBG_ACCESS0_tag;

typedef union GTM_gtm_cls0_ARU_DBG_DATA0_H_union_tag { /* ARU debug access 0 transfer register upper data word */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_ARU_DBG_DATA0_H_tag;

typedef union GTM_gtm_cls0_ARU_DBG_DATA0_L_union_tag { /* ARU debug access 0 transfer register lower data word */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_ARU_DBG_DATA0_L_tag;

typedef union GTM_gtm_cls0_ARU_DBG_ACCESS1_union_tag { /* ARU debug access channel 0 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_ARU_DBG_ACCESS1_tag;

typedef union GTM_gtm_cls0_ARU_DBG_DATA1_H_union_tag { /* ARU debug access 1 transfer register upper data word */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_ARU_DBG_DATA1_H_tag;

typedef union GTM_gtm_cls0_ARU_DBG_DATA1_L_union_tag { /* ARU debug access 1 transfer register lower data word */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_ARU_DBG_DATA1_L_tag;

typedef union GTM_gtm_cls0_ARU_IRQ_NOTIFY_union_tag { /* ARU interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEW_DATA0:1;
    vuint32_t NEW_DATA1:1;
    vuint32_t ACC_ACK:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_ARU_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ARU_IRQ_EN_union_tag { /* ARU interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEW_DATA0_IRQ_EN:1;
    vuint32_t NEW_DATA1_IRQ_EN:1;
    vuint32_t ACC_ACK_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_ARU_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ARU_IRQ_FORCINT_union_tag { /* ARU force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEW_DATA0:1;
    vuint32_t TRG_NEW_DATA1:1;
    vuint32_t TRG_ACC_ACK:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_ARU_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ARU_IRQ_MODE_union_tag { /* ARU interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ARU_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ARU_CADDR_END_union_tag { /* ARU caddr counter end value */
  vuint32_t R;
  struct {
    vuint32_t CADDR_END:7;
    vuint32_t _unused_7:25;
  } B;
} GTM_gtm_cls0_ARU_CADDR_END_tag;

typedef union GTM_gtm_cls0_ARU_CTRL_union_tag { /* ARU enable dynamic routing */
  vuint32_t R;
  struct {
    vuint32_t ARU_0_DYN_EN:2;
    vuint32_t ARU_1_DYN_EN:2;
    vuint32_t ARU_DYN_RING_MODE:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_ARU_CTRL_tag;

typedef union GTM_gtm_cls0_ARU_0_DYN_CTRL_union_tag { /* ARU [g] dynamic routing control register */
  vuint32_t R;
  struct {
    vuint32_t DYN_ARU_UPDATE_EN:1;
    vuint32_t DYN_ROUTE_SWAP:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ARU_0_DYN_CTRL_tag;

typedef union GTM_gtm_cls0_ARU_1_DYN_CTRL_union_tag { /* ARU [g] dynamic routing control register */
  vuint32_t R;
  struct {
    vuint32_t DYN_ARU_UPDATE_EN:1;
    vuint32_t DYN_ROUTE_SWAP:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ARU_1_DYN_CTRL_tag;

typedef union GTM_gtm_cls0_ARU_0_DYN_ROUTE_LOW_union_tag { /* ARU [g] lower bits of DYN_ROUTE register */
  vuint32_t R;
  struct {
    vuint32_t DYN_READ_ID0:8;
    vuint32_t DYN_READ_ID1:8;
    vuint32_t DYN_READ_ID2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ARU_0_DYN_ROUTE_LOW_tag;

typedef union GTM_gtm_cls0_ARU_1_DYN_ROUTE_LOW_union_tag { /* ARU [g] lower bits of DYN_ROUTE register */
  vuint32_t R;
  struct {
    vuint32_t DYN_READ_ID0:8;
    vuint32_t DYN_READ_ID1:8;
    vuint32_t DYN_READ_ID2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ARU_1_DYN_ROUTE_LOW_tag;

typedef union GTM_gtm_cls0_ARU_0_DYN_ROUTE_HIGH_union_tag { /* ARU [g] higher bits of DYN_ROUTE register */
  vuint32_t R;
  struct {
    vuint32_t DYN_READ_ID3:8;
    vuint32_t DYN_READ_ID4:8;
    vuint32_t DYN_READ_ID5:8;
    vuint32_t DYN_CLK_WAIT:4;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_ARU_0_DYN_ROUTE_HIGH_tag;

typedef union GTM_gtm_cls0_ARU_1_DYN_ROUTE_HIGH_union_tag { /* ARU [g] higher bits of DYN_ROUTE register */
  vuint32_t R;
  struct {
    vuint32_t DYN_READ_ID3:8;
    vuint32_t DYN_READ_ID4:8;
    vuint32_t DYN_READ_ID5:8;
    vuint32_t DYN_CLK_WAIT:4;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_ARU_1_DYN_ROUTE_HIGH_tag;

typedef union GTM_gtm_cls0_ARU_0_DYN_ROUTE_SR_LOW_union_tag { /* ARU [g] shadow register of DYN_ROUTE register lower bits */
  vuint32_t R;
  struct {
    vuint32_t DYN_READ_ID6:8;
    vuint32_t DYN_READ_ID7:8;
    vuint32_t DYN_READ_ID8:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ARU_0_DYN_ROUTE_SR_LOW_tag;

typedef union GTM_gtm_cls0_ARU_1_DYN_ROUTE_SR_LOW_union_tag { /* ARU [g] shadow register of DYN_ROUTE register lower bits */
  vuint32_t R;
  struct {
    vuint32_t DYN_READ_ID6:8;
    vuint32_t DYN_READ_ID7:8;
    vuint32_t DYN_READ_ID8:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ARU_1_DYN_ROUTE_SR_LOW_tag;

typedef union GTM_gtm_cls0_ARU_0_DYN_ROUTE_SR_HIGH_union_tag { /* ARU [g] shadow register of DYN_ROUTE register higher bits */
  vuint32_t R;
  struct {
    vuint32_t DYN_READ_ID9:8;
    vuint32_t DYN_READ_ID10:8;
    vuint32_t DYN_READ_ID11:8;
    vuint32_t DYN_CLK_WAIT:4;
    vuint32_t DYN_UPDATE_EN:1;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_ARU_0_DYN_ROUTE_SR_HIGH_tag;

typedef union GTM_gtm_cls0_ARU_1_DYN_ROUTE_SR_HIGH_union_tag { /* ARU [g] shadow register of DYN_ROUTE register higher bits */
  vuint32_t R;
  struct {
    vuint32_t DYN_READ_ID9:8;
    vuint32_t DYN_READ_ID10:8;
    vuint32_t DYN_READ_ID11:8;
    vuint32_t DYN_CLK_WAIT:4;
    vuint32_t DYN_UPDATE_EN:1;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_ARU_1_DYN_ROUTE_SR_HIGH_tag;

typedef union GTM_gtm_cls0_ARU_0_DYN_RDADDR_union_tag { /* ARU [g] master ID for dynamic routing */
  vuint32_t R;
  struct {
    vuint32_t DYN_ARU_RDADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_ARU_0_DYN_RDADDR_tag;

typedef union GTM_gtm_cls0_ARU_1_DYN_RDADDR_union_tag { /* ARU [g] master ID for dynamic routing */
  vuint32_t R;
  struct {
    vuint32_t DYN_ARU_RDADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_ARU_1_DYN_RDADDR_tag;

typedef union GTM_gtm_cls0_ARU_CADDR_union_tag { /* ARU caddr counter value */
  vuint32_t R;
  struct {
    vuint32_t CADDR_0:7;
    vuint32_t _unused_7:9;
    vuint32_t CADDR_1:7;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_ARU_CADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_0_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_0_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_0_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_0_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_1_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_1_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_1_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_1_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_2_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_2_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_2_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_2_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_3_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_3_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_3_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_3_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_4_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_4_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_4_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_4_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_5_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_5_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_5_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_5_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_6_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_6_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_6_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_6_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_7_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_7_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_7_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_7_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_8_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_8_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_8_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_8_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_9_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_9_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_9_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_9_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_10_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_10_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_10_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_10_DEST_tag;

typedef union GTM_gtm_cls0_BRC_SRC_11_ADDR_union_tag { /* BRC read address for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:3;
    vuint32_t BRC_MODE:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_SRC_11_ADDR_tag;

typedef union GTM_gtm_cls0_BRC_SRC_11_DEST_union_tag { /* BRC destination channels for input channel [x] */
  vuint32_t R;
  struct {
    vuint32_t EN_DEST0:1;
    vuint32_t EN_DEST1:1;
    vuint32_t EN_DEST2:1;
    vuint32_t EN_DEST3:1;
    vuint32_t EN_DEST4:1;
    vuint32_t EN_DEST5:1;
    vuint32_t EN_DEST6:1;
    vuint32_t EN_DEST7:1;
    vuint32_t EN_DEST8:1;
    vuint32_t EN_DEST9:1;
    vuint32_t EN_DEST10:1;
    vuint32_t EN_DEST11:1;
    vuint32_t EN_DEST12:1;
    vuint32_t EN_DEST13:1;
    vuint32_t EN_DEST14:1;
    vuint32_t EN_DEST15:1;
    vuint32_t EN_DEST16:1;
    vuint32_t EN_DEST17:1;
    vuint32_t EN_DEST18:1;
    vuint32_t EN_DEST19:1;
    vuint32_t EN_DEST20:1;
    vuint32_t EN_DEST21:1;
    vuint32_t EN_TRASHBIN:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_BRC_SRC_11_DEST_tag;

typedef union GTM_gtm_cls0_BRC_IRQ_NOTIFY_union_tag { /* BRC interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t DEST_ERR:1;
    vuint32_t DID0:1;
    vuint32_t DID1:1;
    vuint32_t DID2:1;
    vuint32_t DID3:1;
    vuint32_t DID4:1;
    vuint32_t DID5:1;
    vuint32_t DID6:1;
    vuint32_t DID7:1;
    vuint32_t DID8:1;
    vuint32_t DID9:1;
    vuint32_t DID10:1;
    vuint32_t DID11:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_BRC_IRQ_EN_union_tag { /* BRC interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t DEST_ERR_IRQ_EN:1;
    vuint32_t DID_IRQ_EN0:1;
    vuint32_t DID_IRQ_EN1:1;
    vuint32_t DID_IRQ_EN2:1;
    vuint32_t DID_IRQ_EN3:1;
    vuint32_t DID_IRQ_EN4:1;
    vuint32_t DID_IRQ_EN5:1;
    vuint32_t DID_IRQ_EN6:1;
    vuint32_t DID_IRQ_EN7:1;
    vuint32_t DID_IRQ_EN8:1;
    vuint32_t DID_IRQ_EN9:1;
    vuint32_t DID_IRQ_EN10:1;
    vuint32_t DID_IRQ_EN11:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_IRQ_EN_tag;

typedef union GTM_gtm_cls0_BRC_IRQ_FORCINT_union_tag { /* BRC force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_DEST_ERR:1;
    vuint32_t TRG_DID0:1;
    vuint32_t TRG_DID1:1;
    vuint32_t TRG_DID2:1;
    vuint32_t TRG_DID3:1;
    vuint32_t TRG_DID4:1;
    vuint32_t TRG_DID5:1;
    vuint32_t TRG_DID6:1;
    vuint32_t TRG_DID7:1;
    vuint32_t TRG_DID8:1;
    vuint32_t TRG_DID9:1;
    vuint32_t TRG_DID10:1;
    vuint32_t TRG_DID11:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_BRC_IRQ_MODE_union_tag { /* BRC interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_BRC_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_BRC_RST_union_tag { /* BRC software reset register */
  vuint32_t R;
  struct {
    vuint32_t RST:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_BRC_RST_tag;

typedef union GTM_gtm_cls0_BRC_EIRQ_EN_union_tag { /* BRC error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t DEST_ERR_EIRQ_EN:1;
    vuint32_t DID_EIRQ_EN0:1;
    vuint32_t DID_EIRQ_EN1:1;
    vuint32_t DID_EIRQ_EN2:1;
    vuint32_t DID_EIRQ_EN3:1;
    vuint32_t DID_EIRQ_EN4:1;
    vuint32_t DID_EIRQ_EN5:1;
    vuint32_t DID_EIRQ_EN6:1;
    vuint32_t DID_EIRQ_EN7:1;
    vuint32_t DID_EIRQ_EN8:1;
    vuint32_t DID_EIRQ_EN9:1;
    vuint32_t DID_EIRQ_EN10:1;
    vuint32_t DID_EIRQ_EN11:1;
    vuint32_t _unused_13:19;
  } B;
} GTM_gtm_cls0_BRC_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_0_union_tag { /* ICM Interrupt group register covering infrastructural and safety components (ARU, BRC, AEI, PSM0, PSM1, MAP, CMP, SPE) */
  vuint32_t R;
  struct {
    vuint32_t ARU_NEW_DATA0_IRQ:1;
    vuint32_t ARU_NEW_DATA1_IRQ:1;
    vuint32_t ARU_ACC_ACK_IRQ:1;
    vuint32_t BRC_IRQ:1;
    vuint32_t AEI_IRQ:1;
    vuint32_t CMP_IRQ:1;
    vuint32_t SPE0_IRQ:1;
    vuint32_t SPE1_IRQ:1;
    vuint32_t _unused_8:8;
    vuint32_t PSM0_CH0_IRQ:1;
    vuint32_t PSM0_CH1_IRQ:1;
    vuint32_t PSM0_CH2_IRQ:1;
    vuint32_t PSM0_CH3_IRQ:1;
    vuint32_t PSM0_CH4_IRQ:1;
    vuint32_t PSM0_CH5_IRQ:1;
    vuint32_t PSM0_CH6_IRQ:1;
    vuint32_t PSM0_CH7_IRQ:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ICM_IRQG_0_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_1_union_tag { /* ICM Interrupt group register covering DPLL */
  vuint32_t R;
  struct {
    vuint32_t DPLL_DCGI_IRQ:1;
    vuint32_t DPLL_EDI_IRQ:1;
    vuint32_t DPLL_TINI_IRQ:1;
    vuint32_t DPLL_TAXI_IRQ:1;
    vuint32_t DPLL_SISI_IRQ:1;
    vuint32_t DPLL_TISI_IRQ:1;
    vuint32_t DPLL_MSI_IRQ:1;
    vuint32_t DPLL_MTI_IRQ:1;
    vuint32_t DPLL_SASI_IRQ:1;
    vuint32_t DPLL_TASI_IRQ:1;
    vuint32_t DPLL_PWI_IRQ:1;
    vuint32_t DPLL_W2I_IRQ:1;
    vuint32_t DPLL_W1I_IRQ:1;
    vuint32_t DPLL_GL1I_IRQ:1;
    vuint32_t DPLL_LL1I_IRQ:1;
    vuint32_t DPLL_EI_IRQ:1;
    vuint32_t DPLL_GL2I_IRQ:1;
    vuint32_t DPLL_LL2I_IRQ:1;
    vuint32_t DPLL_TE0I_IRQ:1;
    vuint32_t DPLL_TE1I_IRQ:1;
    vuint32_t DPLL_TE2I_IRQ:1;
    vuint32_t DPLL_TE3I_IRQ:1;
    vuint32_t DPLL_TE4I_IRQ:1;
    vuint32_t DPLL_CDTI_IRQ:1;
    vuint32_t DPLL_CDSI_IRQ:1;
    vuint32_t DPLL_TORI_IRQ:1;
    vuint32_t DPLL_SORI_IRQ:1;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls0_ICM_IRQG_1_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_2_union_tag { /* ICM Interrupt group register covering TIM0, TIM1, TIM2, TIM3 */
  vuint32_t R;
  struct {
    vuint32_t TIM0_CH0_IRQ:1;
    vuint32_t TIM0_CH1_IRQ:1;
    vuint32_t TIM0_CH2_IRQ:1;
    vuint32_t TIM0_CH3_IRQ:1;
    vuint32_t TIM0_CH4_IRQ:1;
    vuint32_t TIM0_CH5_IRQ:1;
    vuint32_t TIM0_CH6_IRQ:1;
    vuint32_t TIM0_CH7_IRQ:1;
    vuint32_t TIM1_CH0_IRQ:1;
    vuint32_t TIM1_CH1_IRQ:1;
    vuint32_t TIM1_CH2_IRQ:1;
    vuint32_t TIM1_CH3_IRQ:1;
    vuint32_t TIM1_CH4_IRQ:1;
    vuint32_t TIM1_CH5_IRQ:1;
    vuint32_t TIM1_CH6_IRQ:1;
    vuint32_t TIM1_CH7_IRQ:1;
    vuint32_t TIM2_CH0_IRQ:1;
    vuint32_t TIM2_CH1_IRQ:1;
    vuint32_t TIM2_CH2_IRQ:1;
    vuint32_t TIM2_CH3_IRQ:1;
    vuint32_t TIM2_CH4_IRQ:1;
    vuint32_t TIM2_CH5_IRQ:1;
    vuint32_t TIM2_CH6_IRQ:1;
    vuint32_t TIM2_CH7_IRQ:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ICM_IRQG_2_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_4_union_tag { /* ICM Interrupt group register covering MCS0 to MCS3 submodules */
  vuint32_t R;
  struct {
    vuint32_t MCS0_CH0_IRQ:1;
    vuint32_t MCS0_CH1_IRQ:1;
    vuint32_t MCS0_CH2_IRQ:1;
    vuint32_t MCS0_CH3_IRQ:1;
    vuint32_t MCS0_CH4_IRQ:1;
    vuint32_t MCS0_CH5_IRQ:1;
    vuint32_t MCS0_CH6_IRQ:1;
    vuint32_t MCS0_CH7_IRQ:1;
    vuint32_t MCS1_CH0_IRQ:1;
    vuint32_t MCS1_CH1_IRQ:1;
    vuint32_t MCS1_CH2_IRQ:1;
    vuint32_t MCS1_CH3_IRQ:1;
    vuint32_t MCS1_CH4_IRQ:1;
    vuint32_t MCS1_CH5_IRQ:1;
    vuint32_t MCS1_CH6_IRQ:1;
    vuint32_t MCS1_CH7_IRQ:1;
    vuint32_t MCS2_CH0_IRQ:1;
    vuint32_t MCS2_CH1_IRQ:1;
    vuint32_t MCS2_CH2_IRQ:1;
    vuint32_t MCS2_CH3_IRQ:1;
    vuint32_t MCS2_CH4_IRQ:1;
    vuint32_t MCS2_CH5_IRQ:1;
    vuint32_t MCS2_CH6_IRQ:1;
    vuint32_t MCS2_CH7_IRQ:1;
    vuint32_t MCS3_CH0_IRQ:1;
    vuint32_t MCS3_CH1_IRQ:1;
    vuint32_t MCS3_CH2_IRQ:1;
    vuint32_t MCS3_CH3_IRQ:1;
    vuint32_t MCS3_CH4_IRQ:1;
    vuint32_t MCS3_CH5_IRQ:1;
    vuint32_t MCS3_CH6_IRQ:1;
    vuint32_t MCS3_CH7_IRQ:1;
  } B;
} GTM_gtm_cls0_ICM_IRQG_4_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MEI_union_tag { /* ICM Interrupt group register for module error interrupt information */
  vuint32_t R;
  struct {
    vuint32_t GTM_EIRQ:1;
    vuint32_t BRC_EIRQ:1;
    vuint32_t FIFO0_EIRQ:1;
    vuint32_t _unused_3:1;
    vuint32_t TIM0_EIRQ:1;
    vuint32_t TIM1_EIRQ:1;
    vuint32_t TIM2_EIRQ:1;
    vuint32_t _unused_7:5;
    vuint32_t MCS0_EIRQ:1;
    vuint32_t MCS1_EIRQ:1;
    vuint32_t MCS2_EIRQ:1;
    vuint32_t MCS3_EIRQ:1;
    vuint32_t _unused_16:4;
    vuint32_t SPE0_EIRQ:1;
    vuint32_t SPE1_EIRQ:1;
    vuint32_t _unused_22:2;
    vuint32_t CMP_EIRQ:1;
    vuint32_t DPLL_EIRQ:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MEI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_CEI0_union_tag { /* ICM Interrupt group register 0 for channel error interrupt information */
  vuint32_t R;
  struct {
    vuint32_t FIFO0_CH0_EIRQ:1;
    vuint32_t FIFO0_CH1_EIRQ:1;
    vuint32_t FIFO0_CH2_EIRQ:1;
    vuint32_t FIFO0_CH3_EIRQ:1;
    vuint32_t FIFO0_CH4_EIRQ:1;
    vuint32_t FIFO0_CH5_EIRQ:1;
    vuint32_t FIFO0_CH6_EIRQ:1;
    vuint32_t FIFO0_CH7_EIRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_CEI0_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_CEI1_union_tag { /* ICM Interrupt group register 1 for channel error interrupt information */
  vuint32_t R;
  struct {
    vuint32_t TIM0_CH0_EIRQ:1;
    vuint32_t TIM0_CH1_EIRQ:1;
    vuint32_t TIM0_CH2_EIRQ:1;
    vuint32_t TIM0_CH3_EIRQ:1;
    vuint32_t TIM0_CH4_EIRQ:1;
    vuint32_t TIM0_CH5_EIRQ:1;
    vuint32_t TIM0_CH6_EIRQ:1;
    vuint32_t TIM0_CH7_EIRQ:1;
    vuint32_t TIM1_CH0_EIRQ:1;
    vuint32_t TIM1_CH1_EIRQ:1;
    vuint32_t TIM1_CH2_EIRQ:1;
    vuint32_t TIM1_CH3_EIRQ:1;
    vuint32_t TIM1_CH4_EIRQ:1;
    vuint32_t TIM1_CH5_EIRQ:1;
    vuint32_t TIM1_CH6_EIRQ:1;
    vuint32_t TIM1_CH7_EIRQ:1;
    vuint32_t TIM2_CH0_EIRQ:1;
    vuint32_t TIM2_CH1_EIRQ:1;
    vuint32_t TIM2_CH2_EIRQ:1;
    vuint32_t TIM2_CH3_EIRQ:1;
    vuint32_t TIM2_CH4_EIRQ:1;
    vuint32_t TIM2_CH5_EIRQ:1;
    vuint32_t TIM2_CH6_EIRQ:1;
    vuint32_t TIM2_CH7_EIRQ:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ICM_IRQG_CEI1_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_CEI3_union_tag { /* ICM Interrupt group register 3 for channel error interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS0_CH0_EIRQ:1;
    vuint32_t MCS0_CH1_EIRQ:1;
    vuint32_t MCS0_CH2_EIRQ:1;
    vuint32_t MCS0_CH3_EIRQ:1;
    vuint32_t MCS0_CH4_EIRQ:1;
    vuint32_t MCS0_CH5_EIRQ:1;
    vuint32_t MCS0_CH6_EIRQ:1;
    vuint32_t MCS0_CH7_EIRQ:1;
    vuint32_t MCS1_CH0_EIRQ:1;
    vuint32_t MCS1_CH1_EIRQ:1;
    vuint32_t MCS1_CH2_EIRQ:1;
    vuint32_t MCS1_CH3_EIRQ:1;
    vuint32_t MCS1_CH4_EIRQ:1;
    vuint32_t MCS1_CH5_EIRQ:1;
    vuint32_t MCS1_CH6_EIRQ:1;
    vuint32_t MCS1_CH7_EIRQ:1;
    vuint32_t MCS2_CH0_EIRQ:1;
    vuint32_t MCS2_CH1_EIRQ:1;
    vuint32_t MCS2_CH2_EIRQ:1;
    vuint32_t MCS2_CH3_EIRQ:1;
    vuint32_t MCS2_CH4_EIRQ:1;
    vuint32_t MCS2_CH5_EIRQ:1;
    vuint32_t MCS2_CH6_EIRQ:1;
    vuint32_t MCS2_CH7_EIRQ:1;
    vuint32_t MCS3_CH0_EIRQ:1;
    vuint32_t MCS3_CH1_EIRQ:1;
    vuint32_t MCS3_CH2_EIRQ:1;
    vuint32_t MCS3_CH3_EIRQ:1;
    vuint32_t MCS3_CH4_EIRQ:1;
    vuint32_t MCS3_CH5_EIRQ:1;
    vuint32_t MCS3_CH6_EIRQ:1;
    vuint32_t MCS3_CH7_EIRQ:1;
  } B;
} GTM_gtm_cls0_ICM_IRQG_CEI3_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MCS0_CEI_union_tag { /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS_CH0_EIRQ:1;
    vuint32_t MCS_CH1_EIRQ:1;
    vuint32_t MCS_CH2_EIRQ:1;
    vuint32_t MCS_CH3_EIRQ:1;
    vuint32_t MCS_CH4_EIRQ:1;
    vuint32_t MCS_CH5_EIRQ:1;
    vuint32_t MCS_CH6_EIRQ:1;
    vuint32_t MCS_CH7_EIRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MCS0_CEI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MCS1_CEI_union_tag { /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS_CH0_EIRQ:1;
    vuint32_t MCS_CH1_EIRQ:1;
    vuint32_t MCS_CH2_EIRQ:1;
    vuint32_t MCS_CH3_EIRQ:1;
    vuint32_t MCS_CH4_EIRQ:1;
    vuint32_t MCS_CH5_EIRQ:1;
    vuint32_t MCS_CH6_EIRQ:1;
    vuint32_t MCS_CH7_EIRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MCS1_CEI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MCS2_CEI_union_tag { /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS_CH0_EIRQ:1;
    vuint32_t MCS_CH1_EIRQ:1;
    vuint32_t MCS_CH2_EIRQ:1;
    vuint32_t MCS_CH3_EIRQ:1;
    vuint32_t MCS_CH4_EIRQ:1;
    vuint32_t MCS_CH5_EIRQ:1;
    vuint32_t MCS_CH6_EIRQ:1;
    vuint32_t MCS_CH7_EIRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MCS2_CEI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MCS3_CEI_union_tag { /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS_CH0_EIRQ:1;
    vuint32_t MCS_CH1_EIRQ:1;
    vuint32_t MCS_CH2_EIRQ:1;
    vuint32_t MCS_CH3_EIRQ:1;
    vuint32_t MCS_CH4_EIRQ:1;
    vuint32_t MCS_CH5_EIRQ:1;
    vuint32_t MCS_CH6_EIRQ:1;
    vuint32_t MCS_CH7_EIRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MCS3_CEI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_PSM_0_CEI_union_tag { /* ICM Interrupt group PSM 0 for Channel Error Interrupt information of FIFO0, FIFO1, FIFO2 */
  vuint32_t R;
  struct {
    vuint32_t PSM_M0_CH0_EIRQ:1;
    vuint32_t PSM_M0_CH1_EIRQ:1;
    vuint32_t PSM_M0_CH2_EIRQ:1;
    vuint32_t PSM_M0_CH3_EIRQ:1;
    vuint32_t PSM_M0_CH4_EIRQ:1;
    vuint32_t PSM_M0_CH5_EIRQ:1;
    vuint32_t PSM_M0_CH6_EIRQ:1;
    vuint32_t PSM_M0_CH7_EIRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_PSM_0_CEI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_SPE_CEI_union_tag { /* ICM Interrupt group SPE for module Error Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t SPE0_EIRQ:1;
    vuint32_t SPE1_EIRQ:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ICM_IRQG_SPE_CEI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_CLS_0_MEI_union_tag { /* ICM Interrupt group for module Error Interrupt information for each TIM[j], MCS[j], SPE[j], FIFO[j] (j=4*[g]+k, k∈{0, ... ,3}) */
  vuint32_t R;
  struct {
    vuint32_t TIM_M0_EIRQ:1;
    vuint32_t MCS_M0_EIRQ:1;
    vuint32_t SPE_M0_EIRQ:1;
    vuint32_t FIFO_M0_EIRQ:1;
    vuint32_t _unused_4:4;
    vuint32_t TIM_M1_EIRQ:1;
    vuint32_t MCS_M1_EIRQ:1;
    vuint32_t SPE_M1_EIRQ:1;
    vuint32_t _unused_11:5;
    vuint32_t TIM_M2_EIRQ:1;
    vuint32_t MCS_M2_EIRQ:1;
    vuint32_t _unused_18:7;
    vuint32_t MCS_M3_EIRQ:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_ICM_IRQG_CLS_0_MEI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MCS0_CI_union_tag { /* ICM Interrupt group MCS[j] for Channel Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS_CH0_IRQ:1;
    vuint32_t MCS_CH1_IRQ:1;
    vuint32_t MCS_CH2_IRQ:1;
    vuint32_t MCS_CH3_IRQ:1;
    vuint32_t MCS_CH4_IRQ:1;
    vuint32_t MCS_CH5_IRQ:1;
    vuint32_t MCS_CH6_IRQ:1;
    vuint32_t MCS_CH7_IRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MCS0_CI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MCS1_CI_union_tag { /* ICM Interrupt group MCS[j] for Channel Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS_CH0_IRQ:1;
    vuint32_t MCS_CH1_IRQ:1;
    vuint32_t MCS_CH2_IRQ:1;
    vuint32_t MCS_CH3_IRQ:1;
    vuint32_t MCS_CH4_IRQ:1;
    vuint32_t MCS_CH5_IRQ:1;
    vuint32_t MCS_CH6_IRQ:1;
    vuint32_t MCS_CH7_IRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MCS1_CI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MCS2_CI_union_tag { /* ICM Interrupt group MCS[j] for Channel Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS_CH0_IRQ:1;
    vuint32_t MCS_CH1_IRQ:1;
    vuint32_t MCS_CH2_IRQ:1;
    vuint32_t MCS_CH3_IRQ:1;
    vuint32_t MCS_CH4_IRQ:1;
    vuint32_t MCS_CH5_IRQ:1;
    vuint32_t MCS_CH6_IRQ:1;
    vuint32_t MCS_CH7_IRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MCS2_CI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_MCS3_CI_union_tag { /* ICM Interrupt group MCS[j] for Channel Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t MCS_CH0_IRQ:1;
    vuint32_t MCS_CH1_IRQ:1;
    vuint32_t MCS_CH2_IRQ:1;
    vuint32_t MCS_CH3_IRQ:1;
    vuint32_t MCS_CH4_IRQ:1;
    vuint32_t MCS_CH5_IRQ:1;
    vuint32_t MCS_CH6_IRQ:1;
    vuint32_t MCS_CH7_IRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_MCS3_CI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_PSM_0_CI_union_tag { /* ICM Interrupt group PSM 0 for Channel Interrupt information of FIFO0, FIFO1, FIFO2 */
  vuint32_t R;
  struct {
    vuint32_t PSM_M0_CH0_IRQ:1;
    vuint32_t PSM_M0_CH1_IRQ:1;
    vuint32_t PSM_M0_CH2_IRQ:1;
    vuint32_t PSM_M0_CH3_IRQ:1;
    vuint32_t PSM_M0_CH4_IRQ:1;
    vuint32_t PSM_M0_CH5_IRQ:1;
    vuint32_t PSM_M0_CH6_IRQ:1;
    vuint32_t PSM_M0_CH7_IRQ:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_ICM_IRQG_PSM_0_CI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_SPE_CI_union_tag { /* ICM Interrupt group SPE for module Interrupt information */
  vuint32_t R;
  struct {
    vuint32_t SPE0_IRQ:1;
    vuint32_t SPE1_IRQ:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ICM_IRQG_SPE_CI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_ATOM_0_CI_union_tag { /* ICM Interrupt group ATOM [g] for Channel Interrupt information of ATOM[j] (m=4*[g]+(0..3)) */
  vuint32_t R;
  struct {
    vuint32_t ATOM_M0_CH0_IRQ:1;
    vuint32_t ATOM_M0_CH1_IRQ:1;
    vuint32_t ATOM_M0_CH2_IRQ:1;
    vuint32_t ATOM_M0_CH3_IRQ:1;
    vuint32_t ATOM_M0_CH4_IRQ:1;
    vuint32_t ATOM_M0_CH5_IRQ:1;
    vuint32_t ATOM_M0_CH6_IRQ:1;
    vuint32_t ATOM_M0_CH7_IRQ:1;
    vuint32_t ATOM_M1_CH0_IRQ:1;
    vuint32_t ATOM_M1_CH1_IRQ:1;
    vuint32_t ATOM_M1_CH2_IRQ:1;
    vuint32_t ATOM_M1_CH3_IRQ:1;
    vuint32_t ATOM_M1_CH4_IRQ:1;
    vuint32_t ATOM_M1_CH5_IRQ:1;
    vuint32_t ATOM_M1_CH6_IRQ:1;
    vuint32_t ATOM_M1_CH7_IRQ:1;
    vuint32_t ATOM_M2_CH0_IRQ:1;
    vuint32_t ATOM_M2_CH1_IRQ:1;
    vuint32_t ATOM_M2_CH2_IRQ:1;
    vuint32_t ATOM_M2_CH3_IRQ:1;
    vuint32_t ATOM_M2_CH4_IRQ:1;
    vuint32_t ATOM_M2_CH5_IRQ:1;
    vuint32_t ATOM_M2_CH6_IRQ:1;
    vuint32_t ATOM_M2_CH7_IRQ:1;
    vuint32_t ATOM_M3_CH0_IRQ:1;
    vuint32_t ATOM_M3_CH1_IRQ:1;
    vuint32_t ATOM_M3_CH2_IRQ:1;
    vuint32_t ATOM_M3_CH3_IRQ:1;
    vuint32_t ATOM_M3_CH4_IRQ:1;
    vuint32_t ATOM_M3_CH5_IRQ:1;
    vuint32_t ATOM_M3_CH6_IRQ:1;
    vuint32_t ATOM_M3_CH7_IRQ:1;
  } B;
} GTM_gtm_cls0_ICM_IRQG_ATOM_0_CI_tag;

typedef union GTM_gtm_cls0_ICM_IRQG_TOM_0_CI_union_tag { /* ICM Interrupt group TOM [g] for Channel Interrupt information of TOMm ([j]=2*[g]+(0..1)) */
  vuint32_t R;
  struct {
    vuint32_t TOM_M0_CH0_IRQ:1;
    vuint32_t TOM_M0_CH1_IRQ:1;
    vuint32_t TOM_M0_CH2_IRQ:1;
    vuint32_t TOM_M0_CH3_IRQ:1;
    vuint32_t TOM_M0_CH4_IRQ:1;
    vuint32_t TOM_M0_CH5_IRQ:1;
    vuint32_t TOM_M0_CH6_IRQ:1;
    vuint32_t TOM_M0_CH7_IRQ:1;
    vuint32_t TOM_M0_CH8_IRQ:1;
    vuint32_t TOM_M0_CH9_IRQ:1;
    vuint32_t TOM_M0_CH10_IRQ:1;
    vuint32_t TOM_M0_CH11_IRQ:1;
    vuint32_t TOM_M0_CH12_IRQ:1;
    vuint32_t TOM_M0_CH13_IRQ:1;
    vuint32_t TOM_M0_CH14_IRQ:1;
    vuint32_t TOM_M0_CH15_IRQ:1;
    vuint32_t TOM_M1_CH0_IRQ:1;
    vuint32_t TOM_M1_CH1_IRQ:1;
    vuint32_t TOM_M1_CH2_IRQ:1;
    vuint32_t TOM_M1_CH3_IRQ:1;
    vuint32_t TOM_M1_CH4_IRQ:1;
    vuint32_t TOM_M1_CH5_IRQ:1;
    vuint32_t TOM_M1_CH6_IRQ:1;
    vuint32_t TOM_M1_CH7_IRQ:1;
    vuint32_t TOM_M1_CH8_IRQ:1;
    vuint32_t TOM_M1_CH9_IRQ:1;
    vuint32_t TOM_M1_CH10_IRQ:1;
    vuint32_t TOM_M1_CH11_IRQ:1;
    vuint32_t TOM_M1_CH12_IRQ:1;
    vuint32_t TOM_M1_CH13_IRQ:1;
    vuint32_t TOM_M1_CH14_IRQ:1;
    vuint32_t TOM_M1_CH15_IRQ:1;
  } B;
} GTM_gtm_cls0_ICM_IRQG_TOM_0_CI_tag;

typedef union GTM_gtm_cls0_MAP_CTRL_union_tag { /* MAP Control register */
  vuint32_t R;
  struct {
    vuint32_t TSEL:1;
    vuint32_t SSL:3;
    vuint32_t LSEL:1;
    vuint32_t _unused_5:11;
    vuint32_t TSPP0_EN:1;
    vuint32_t TSPP0_DLD:1;
    vuint32_t _unused_18:2;
    vuint32_t TSPP0_I0V:1;
    vuint32_t TSPP0_I1V:1;
    vuint32_t TSPP0_I2V:1;
    vuint32_t _unused_23:1;
    vuint32_t TSPP1_EN:1;
    vuint32_t TSPP1_DLD:1;
    vuint32_t _unused_26:2;
    vuint32_t TSPP1_I0V:1;
    vuint32_t TSPP1_I1V:1;
    vuint32_t TSPP1_I2V:1;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_MAP_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH0_GPR0_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH0_GPR1_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH0_CNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TIM0_CH0_ECNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH0_CNTS_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH0_TDUC_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_TIM0_CH0_TDUV_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH0_FLT_RE_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH0_FLT_FE_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t TBU0_SEL:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls0_TIM0_CH0_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls0_TIM0_CH0_ECTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TIM0_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TIM0_CH0_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH0_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH1_GPR0_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH1_GPR1_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH1_CNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TIM0_CH1_ECNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH1_CNTS_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH1_TDUC_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_TIM0_CH1_TDUV_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH1_FLT_RE_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH1_FLT_FE_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t TBU0_SEL:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls0_TIM0_CH1_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls0_TIM0_CH1_ECTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TIM0_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TIM0_CH1_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH1_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH2_GPR0_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH2_GPR1_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH2_CNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TIM0_CH2_ECNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH2_CNTS_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH2_TDUC_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_TIM0_CH2_TDUV_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH2_FLT_RE_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH2_FLT_FE_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t TBU0_SEL:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls0_TIM0_CH2_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls0_TIM0_CH2_ECTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TIM0_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TIM0_CH2_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH2_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH3_GPR0_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH3_GPR1_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH3_CNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TIM0_CH3_ECNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH3_CNTS_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH3_TDUC_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_TIM0_CH3_TDUV_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH3_FLT_RE_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH3_FLT_FE_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t TBU0_SEL:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls0_TIM0_CH3_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls0_TIM0_CH3_ECTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TIM0_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TIM0_CH3_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH3_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH4_GPR0_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH4_GPR1_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH4_CNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TIM0_CH4_ECNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH4_CNTS_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH4_TDUC_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_TIM0_CH4_TDUV_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH4_FLT_RE_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH4_FLT_FE_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t TBU0_SEL:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls0_TIM0_CH4_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls0_TIM0_CH4_ECTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TIM0_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TIM0_CH4_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH4_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH5_GPR0_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH5_GPR1_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH5_CNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TIM0_CH5_ECNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH5_CNTS_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH5_TDUC_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_TIM0_CH5_TDUV_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH5_FLT_RE_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH5_FLT_FE_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t TBU0_SEL:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls0_TIM0_CH5_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls0_TIM0_CH5_ECTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TIM0_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TIM0_CH5_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH5_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH6_GPR0_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH6_GPR1_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH6_CNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TIM0_CH6_ECNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH6_CNTS_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH6_TDUC_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_TIM0_CH6_TDUV_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH6_FLT_RE_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH6_FLT_FE_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t TBU0_SEL:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls0_TIM0_CH6_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls0_TIM0_CH6_ECTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TIM0_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TIM0_CH6_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH6_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH7_GPR0_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH7_GPR1_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH7_CNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TIM0_CH7_ECNT_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls0_TIM0_CH7_CNTS_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH7_TDUC_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_TIM0_CH7_TDUV_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH7_FLT_RE_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_CH7_FLT_FE_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t TBU0_SEL:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls0_TIM0_CH7_CTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls0_TIM0_CH7_ECTRL_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TIM0_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TIM0_CH7_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_TIM0_CH7_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_TIM0_INP_VAL_union_tag { /* TIM[i] input value observation register */
  vuint32_t R;
  struct {
    vuint32_t F_OUT0:1;
    vuint32_t F_OUT1:1;
    vuint32_t F_OUT2:1;
    vuint32_t F_OUT3:1;
    vuint32_t F_OUT4:1;
    vuint32_t F_OUT5:1;
    vuint32_t F_OUT6:1;
    vuint32_t F_OUT7:1;
    vuint32_t F_IN0:1;
    vuint32_t F_IN1:1;
    vuint32_t F_IN2:1;
    vuint32_t F_IN3:1;
    vuint32_t F_IN4:1;
    vuint32_t F_IN5:1;
    vuint32_t F_IN6:1;
    vuint32_t F_IN7:1;
    vuint32_t TIM_IN0:1;
    vuint32_t TIM_IN1:1;
    vuint32_t TIM_IN2:1;
    vuint32_t TIM_IN3:1;
    vuint32_t TIM_IN4:1;
    vuint32_t TIM_IN5:1;
    vuint32_t TIM_IN6:1;
    vuint32_t TIM_IN7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_TIM0_INP_VAL_tag;

typedef union GTM_gtm_cls0_TIM0_IN_SRC_union_tag { /* TIM[i] AUX IN source selection register */
  vuint32_t R;
  struct {
    vuint32_t VAL_0:2;
    vuint32_t MODE_0:2;
    vuint32_t VAL_1:2;
    vuint32_t MODE_1:2;
    vuint32_t VAL_2:2;
    vuint32_t MODE_2:2;
    vuint32_t VAL_3:2;
    vuint32_t MODE_3:2;
    vuint32_t VAL_4:2;
    vuint32_t MODE_4:2;
    vuint32_t VAL_5:2;
    vuint32_t MODE_5:2;
    vuint32_t VAL_6:2;
    vuint32_t MODE_6:2;
    vuint32_t VAL_7:2;
    vuint32_t MODE_7:2;
  } B;
} GTM_gtm_cls0_TIM0_IN_SRC_tag;

typedef union GTM_gtm_cls0_TIM0_RST_union_tag { /* TIM[i] global software reset register */
  vuint32_t R;
  struct {
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_TIM0_RST_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH0_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH0_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH0_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH0_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH0_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH0_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH0_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH0_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH0_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH1_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH1_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH1_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH1_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH1_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH1_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH1_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH1_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH1_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH2_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH2_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH2_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH2_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH2_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH2_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH2_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH2_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH2_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH3_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH3_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH3_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH3_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH3_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH3_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH3_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH3_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH3_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH4_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH4_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH4_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH4_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH4_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH4_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH4_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH4_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH4_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH5_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH5_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH5_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH5_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH5_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH5_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH5_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH5_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH5_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH6_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH6_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH6_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH6_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH6_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH6_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH6_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH6_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH6_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH7_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH7_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH7_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH7_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH7_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH7_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH7_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH7_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH7_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH8_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH8_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH8_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH8_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH8_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH8_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH8_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH8_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH8_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH8_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH8_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH8_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH8_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH9_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH9_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH9_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH9_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH9_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH9_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH9_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH9_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH9_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH9_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH9_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH9_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH9_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH10_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH10_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH10_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH10_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH10_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH10_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH10_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH10_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH10_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH10_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH10_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH10_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH10_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH11_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH11_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH11_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH11_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH11_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH11_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH11_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH11_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH11_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH11_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH11_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH11_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH11_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH12_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH12_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH12_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH12_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH12_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH12_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH12_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH12_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH12_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH12_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH12_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH12_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH12_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH13_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH13_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH13_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH13_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH13_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH13_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH13_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH13_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH13_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH13_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH13_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH13_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH13_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH14_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH14_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH14_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH14_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH14_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH14_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH14_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH14_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH14_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH14_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH14_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH14_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH14_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t BITREV:1;
    vuint32_t _unused_28:3;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_TOM0_CH15_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH15_SR0_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH15_SR1_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH15_CM0_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH15_CM1_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH15_CN0_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_TOM0_CH15_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH15_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH15_IRQ_EN_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH15_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_TOM0_CH15_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_TOM0_CH15_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_CH15_CTRL_SR_tag;

typedef union GTM_gtm_cls0_TOM0_TGC0_GLB_CTRL_union_tag { /* TOM[i] TGC [g] global control register */
  vuint32_t R;
  struct {
    vuint32_t HOST_TRIG:1;
    vuint32_t _unused_1:7;
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t UPEN_CTRL0:2;
    vuint32_t UPEN_CTRL1:2;
    vuint32_t UPEN_CTRL2:2;
    vuint32_t UPEN_CTRL3:2;
    vuint32_t UPEN_CTRL4:2;
    vuint32_t UPEN_CTRL5:2;
    vuint32_t UPEN_CTRL6:2;
    vuint32_t UPEN_CTRL7:2;
  } B;
} GTM_gtm_cls0_TOM0_TGC0_GLB_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_TGC0_ACT_TB_union_tag { /* TOM[i] TGC [g] action time base register */
  vuint32_t R;
  struct {
    vuint32_t ACT_TB:24;
    vuint32_t TB_TRIG:1;
    vuint32_t TBU_SEL:2;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls0_TOM0_TGC0_ACT_TB_tag;

typedef union GTM_gtm_cls0_TOM0_TGC0_FUPD_CTRL_union_tag { /* TOM[i] TGC [g] force update control register */
  vuint32_t R;
  struct {
    vuint32_t FUPD_CTRL0:2;
    vuint32_t FUPD_CTRL1:2;
    vuint32_t FUPD_CTRL2:2;
    vuint32_t FUPD_CTRL3:2;
    vuint32_t FUPD_CTRL4:2;
    vuint32_t FUPD_CTRL5:2;
    vuint32_t FUPD_CTRL6:2;
    vuint32_t FUPD_CTRL7:2;
    vuint32_t RSTCN0_CH0:2;
    vuint32_t RSTCN0_CH1:2;
    vuint32_t RSTCN0_CH2:2;
    vuint32_t RSTCN0_CH3:2;
    vuint32_t RSTCN0_CH4:2;
    vuint32_t RSTCN0_CH5:2;
    vuint32_t RSTCN0_CH6:2;
    vuint32_t RSTCN0_CH7:2;
  } B;
} GTM_gtm_cls0_TOM0_TGC0_FUPD_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_TGC0_INT_TRIG_union_tag { /* TOM[i] TGC [g] internal trigger control register */
  vuint32_t R;
  struct {
    vuint32_t INT_TRIG0:2;
    vuint32_t INT_TRIG1:2;
    vuint32_t INT_TRIG2:2;
    vuint32_t INT_TRIG3:2;
    vuint32_t INT_TRIG4:2;
    vuint32_t INT_TRIG5:2;
    vuint32_t INT_TRIG6:2;
    vuint32_t INT_TRIG7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC0_INT_TRIG_tag;

typedef union GTM_gtm_cls0_TOM0_TGC0_ENDIS_CTRL_union_tag { /* TOM[i] TGC [g] enable/disable control register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CTRL0:2;
    vuint32_t ENDIS_CTRL1:2;
    vuint32_t ENDIS_CTRL2:2;
    vuint32_t ENDIS_CTRL3:2;
    vuint32_t ENDIS_CTRL4:2;
    vuint32_t ENDIS_CTRL5:2;
    vuint32_t ENDIS_CTRL6:2;
    vuint32_t ENDIS_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC0_ENDIS_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_TGC0_ENDIS_STAT_union_tag { /* TOM[i] TGC [g] enable/disable status register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_STAT0:2;
    vuint32_t ENDIS_STAT1:2;
    vuint32_t ENDIS_STAT2:2;
    vuint32_t ENDIS_STAT3:2;
    vuint32_t ENDIS_STAT4:2;
    vuint32_t ENDIS_STAT5:2;
    vuint32_t ENDIS_STAT6:2;
    vuint32_t ENDIS_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC0_ENDIS_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_TGC0_OUTEN_CTRL_union_tag { /* TOM[i] TGC [g] output enable control register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_CTRL0:2;
    vuint32_t OUTEN_CTRL1:2;
    vuint32_t OUTEN_CTRL2:2;
    vuint32_t OUTEN_CTRL3:2;
    vuint32_t OUTEN_CTRL4:2;
    vuint32_t OUTEN_CTRL5:2;
    vuint32_t OUTEN_CTRL6:2;
    vuint32_t OUTEN_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC0_OUTEN_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_TGC0_OUTEN_STAT_union_tag { /* TOM[i] TGC [g] output enable status register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_STAT0:2;
    vuint32_t OUTEN_STAT1:2;
    vuint32_t OUTEN_STAT2:2;
    vuint32_t OUTEN_STAT3:2;
    vuint32_t OUTEN_STAT4:2;
    vuint32_t OUTEN_STAT5:2;
    vuint32_t OUTEN_STAT6:2;
    vuint32_t OUTEN_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC0_OUTEN_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_TGC1_GLB_CTRL_union_tag { /* TOM[i] TGC [g] global control register */
  vuint32_t R;
  struct {
    vuint32_t HOST_TRIG:1;
    vuint32_t _unused_1:7;
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t UPEN_CTRL0:2;
    vuint32_t UPEN_CTRL1:2;
    vuint32_t UPEN_CTRL2:2;
    vuint32_t UPEN_CTRL3:2;
    vuint32_t UPEN_CTRL4:2;
    vuint32_t UPEN_CTRL5:2;
    vuint32_t UPEN_CTRL6:2;
    vuint32_t UPEN_CTRL7:2;
  } B;
} GTM_gtm_cls0_TOM0_TGC1_GLB_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_TGC1_ACT_TB_union_tag { /* TOM[i] TGC [g] action time base register */
  vuint32_t R;
  struct {
    vuint32_t ACT_TB:24;
    vuint32_t TB_TRIG:1;
    vuint32_t TBU_SEL:2;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls0_TOM0_TGC1_ACT_TB_tag;

typedef union GTM_gtm_cls0_TOM0_TGC1_FUPD_CTRL_union_tag { /* TOM[i] TGC [g] force update control register */
  vuint32_t R;
  struct {
    vuint32_t FUPD_CTRL0:2;
    vuint32_t FUPD_CTRL1:2;
    vuint32_t FUPD_CTRL2:2;
    vuint32_t FUPD_CTRL3:2;
    vuint32_t FUPD_CTRL4:2;
    vuint32_t FUPD_CTRL5:2;
    vuint32_t FUPD_CTRL6:2;
    vuint32_t FUPD_CTRL7:2;
    vuint32_t RSTCN0_CH0:2;
    vuint32_t RSTCN0_CH1:2;
    vuint32_t RSTCN0_CH2:2;
    vuint32_t RSTCN0_CH3:2;
    vuint32_t RSTCN0_CH4:2;
    vuint32_t RSTCN0_CH5:2;
    vuint32_t RSTCN0_CH6:2;
    vuint32_t RSTCN0_CH7:2;
  } B;
} GTM_gtm_cls0_TOM0_TGC1_FUPD_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_TGC1_INT_TRIG_union_tag { /* TOM[i] TGC [g] internal trigger control register */
  vuint32_t R;
  struct {
    vuint32_t INT_TRIG0:2;
    vuint32_t INT_TRIG1:2;
    vuint32_t INT_TRIG2:2;
    vuint32_t INT_TRIG3:2;
    vuint32_t INT_TRIG4:2;
    vuint32_t INT_TRIG5:2;
    vuint32_t INT_TRIG6:2;
    vuint32_t INT_TRIG7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC1_INT_TRIG_tag;

typedef union GTM_gtm_cls0_TOM0_TGC1_ENDIS_CTRL_union_tag { /* TOM[i] TGC [g] enable/disable control register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CTRL0:2;
    vuint32_t ENDIS_CTRL1:2;
    vuint32_t ENDIS_CTRL2:2;
    vuint32_t ENDIS_CTRL3:2;
    vuint32_t ENDIS_CTRL4:2;
    vuint32_t ENDIS_CTRL5:2;
    vuint32_t ENDIS_CTRL6:2;
    vuint32_t ENDIS_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC1_ENDIS_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_TGC1_ENDIS_STAT_union_tag { /* TOM[i] TGC [g] enable/disable status register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_STAT0:2;
    vuint32_t ENDIS_STAT1:2;
    vuint32_t ENDIS_STAT2:2;
    vuint32_t ENDIS_STAT3:2;
    vuint32_t ENDIS_STAT4:2;
    vuint32_t ENDIS_STAT5:2;
    vuint32_t ENDIS_STAT6:2;
    vuint32_t ENDIS_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC1_ENDIS_STAT_tag;

typedef union GTM_gtm_cls0_TOM0_TGC1_OUTEN_CTRL_union_tag { /* TOM[i] TGC [g] output enable control register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_CTRL0:2;
    vuint32_t OUTEN_CTRL1:2;
    vuint32_t OUTEN_CTRL2:2;
    vuint32_t OUTEN_CTRL3:2;
    vuint32_t OUTEN_CTRL4:2;
    vuint32_t OUTEN_CTRL5:2;
    vuint32_t OUTEN_CTRL6:2;
    vuint32_t OUTEN_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC1_OUTEN_CTRL_tag;

typedef union GTM_gtm_cls0_TOM0_TGC1_OUTEN_STAT_union_tag { /* TOM[i] TGC [g] output enable status register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_STAT0:2;
    vuint32_t OUTEN_STAT1:2;
    vuint32_t OUTEN_STAT2:2;
    vuint32_t OUTEN_STAT3:2;
    vuint32_t OUTEN_STAT4:2;
    vuint32_t OUTEN_STAT5:2;
    vuint32_t OUTEN_STAT6:2;
    vuint32_t OUTEN_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_TOM0_TGC1_OUTEN_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_RDADDR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_SR0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_SR1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_CM0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_CM1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_CN0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ATOM0_CH0_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_CH0_CTRL_SR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_RDADDR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_SR0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_SR1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_CM0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_CM1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_CN0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ATOM0_CH1_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_CH1_CTRL_SR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_RDADDR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_SR0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_SR1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_CM0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_CM1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_CN0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ATOM0_CH2_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_CH2_CTRL_SR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_RDADDR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_SR0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_SR1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_CM0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_CM1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_CN0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ATOM0_CH3_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_CH3_CTRL_SR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_RDADDR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_SR0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_SR1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_CM0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_CM1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_CN0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ATOM0_CH4_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_CH4_CTRL_SR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_RDADDR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_SR0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_SR1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_CM0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_CM1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_CN0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ATOM0_CH5_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_CH5_CTRL_SR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_RDADDR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_SR0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_SR1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_CM0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_CM1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_CN0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ATOM0_CH6_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_CH6_CTRL_SR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_RDADDR_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_SR0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_SR1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_CM0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_CM1_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_CN0_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_ATOM0_CH7_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_CH7_CTRL_SR_tag;

typedef union GTM_gtm_cls0_ATOM0_AGC_GLB_CTRL_union_tag { /* ATOM[i] AGC global control register */
  vuint32_t R;
  struct {
    vuint32_t HOST_TRIG:1;
    vuint32_t _unused_1:7;
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t UPEN_CTRL0:2;
    vuint32_t UPEN_CTRL1:2;
    vuint32_t UPEN_CTRL2:2;
    vuint32_t UPEN_CTRL3:2;
    vuint32_t UPEN_CTRL4:2;
    vuint32_t UPEN_CTRL5:2;
    vuint32_t UPEN_CTRL6:2;
    vuint32_t UPEN_CTRL7:2;
  } B;
} GTM_gtm_cls0_ATOM0_AGC_GLB_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_AGC_ENDIS_CTRL_union_tag { /* ATOM[i] AGC enable/disable control register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CTRL0:2;
    vuint32_t ENDIS_CTRL1:2;
    vuint32_t ENDIS_CTRL2:2;
    vuint32_t ENDIS_CTRL3:2;
    vuint32_t ENDIS_CTRL4:2;
    vuint32_t ENDIS_CTRL5:2;
    vuint32_t ENDIS_CTRL6:2;
    vuint32_t ENDIS_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_AGC_ENDIS_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_AGC_ENDIS_STAT_union_tag { /* ATOM[i] AGC enable/disable status register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_STAT0:2;
    vuint32_t ENDIS_STAT1:2;
    vuint32_t ENDIS_STAT2:2;
    vuint32_t ENDIS_STAT3:2;
    vuint32_t ENDIS_STAT4:2;
    vuint32_t ENDIS_STAT5:2;
    vuint32_t ENDIS_STAT6:2;
    vuint32_t ENDIS_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_AGC_ENDIS_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_AGC_ACT_TB_union_tag { /* ATOM[i] AGC action time base register */
  vuint32_t R;
  struct {
    vuint32_t ACT_TB:24;
    vuint32_t TB_TRIG:1;
    vuint32_t TBU_SEL:2;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls0_ATOM0_AGC_ACT_TB_tag;

typedef union GTM_gtm_cls0_ATOM0_AGC_OUTEN_CTRL_union_tag { /* ATOM[i] AGC output enable control register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_CTRL0:2;
    vuint32_t OUTEN_CTRL1:2;
    vuint32_t OUTEN_CTRL2:2;
    vuint32_t OUTEN_CTRL3:2;
    vuint32_t OUTEN_CTRL4:2;
    vuint32_t OUTEN_CTRL5:2;
    vuint32_t OUTEN_CTRL6:2;
    vuint32_t OUTEN_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_AGC_OUTEN_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_AGC_OUTEN_STAT_union_tag { /* ATOM[i] AGC output enable status register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_STAT0:2;
    vuint32_t OUTEN_STAT1:2;
    vuint32_t OUTEN_STAT2:2;
    vuint32_t OUTEN_STAT3:2;
    vuint32_t OUTEN_STAT4:2;
    vuint32_t OUTEN_STAT5:2;
    vuint32_t OUTEN_STAT6:2;
    vuint32_t OUTEN_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_AGC_OUTEN_STAT_tag;

typedef union GTM_gtm_cls0_ATOM0_AGC_FUPD_CTRL_union_tag { /* ATOM[i] AGC force update control register */
  vuint32_t R;
  struct {
    vuint32_t FUPD_CTRL0:2;
    vuint32_t FUPD_CTRL1:2;
    vuint32_t FUPD_CTRL2:2;
    vuint32_t FUPD_CTRL3:2;
    vuint32_t FUPD_CTRL4:2;
    vuint32_t FUPD_CTRL5:2;
    vuint32_t FUPD_CTRL6:2;
    vuint32_t FUPD_CTRL7:2;
    vuint32_t RSTCN0_CH0:2;
    vuint32_t RSTCN0_CH1:2;
    vuint32_t RSTCN0_CH2:2;
    vuint32_t RSTCN0_CH3:2;
    vuint32_t RSTCN0_CH4:2;
    vuint32_t RSTCN0_CH5:2;
    vuint32_t RSTCN0_CH6:2;
    vuint32_t RSTCN0_CH7:2;
  } B;
} GTM_gtm_cls0_ATOM0_AGC_FUPD_CTRL_tag;

typedef union GTM_gtm_cls0_ATOM0_AGC_INT_TRIG_union_tag { /* ATOM[i] AGC internal trigger control register */
  vuint32_t R;
  struct {
    vuint32_t INT_TRIG0:2;
    vuint32_t INT_TRIG1:2;
    vuint32_t INT_TRIG2:2;
    vuint32_t INT_TRIG3:2;
    vuint32_t INT_TRIG4:2;
    vuint32_t INT_TRIG5:2;
    vuint32_t INT_TRIG6:2;
    vuint32_t INT_TRIG7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_ATOM0_AGC_INT_TRIG_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH0_R0_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH0_R1_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH0_R2_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH0_R3_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH0_R4_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH0_R5_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH0_R6_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH0_R7_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_MCS0_CH0_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_MCS0_CH0_ACB_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CH0_MHB_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_CH0_PC_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_CH0_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH0_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH1_R0_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH1_R1_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH1_R2_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH1_R3_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH1_R4_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH1_R5_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH1_R6_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH1_R7_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_MCS0_CH1_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_MCS0_CH1_ACB_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CH1_MHB_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_CH1_PC_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_CH1_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH1_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH2_R0_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH2_R1_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH2_R2_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH2_R3_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH2_R4_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH2_R5_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH2_R6_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH2_R7_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_MCS0_CH2_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_MCS0_CH2_ACB_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CH2_MHB_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_CH2_PC_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_CH2_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH2_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH3_R0_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH3_R1_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH3_R2_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH3_R3_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH3_R4_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH3_R5_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH3_R6_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH3_R7_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_MCS0_CH3_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_MCS0_CH3_ACB_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CH3_MHB_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_CH3_PC_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_CH3_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH3_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH4_R0_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH4_R1_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH4_R2_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH4_R3_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH4_R4_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH4_R5_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH4_R6_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH4_R7_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_MCS0_CH4_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_MCS0_CH4_ACB_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CH4_MHB_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_CH4_PC_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_CH4_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH4_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH5_R0_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH5_R1_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH5_R2_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH5_R3_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH5_R4_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH5_R5_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH5_R6_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH5_R7_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_MCS0_CH5_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_MCS0_CH5_ACB_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CH5_MHB_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_CH5_PC_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_CH5_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH5_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH6_R0_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH6_R1_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH6_R2_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH6_R3_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH6_R4_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH6_R5_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH6_R6_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH6_R7_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_MCS0_CH6_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_MCS0_CH6_ACB_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CH6_MHB_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_CH6_PC_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_CH6_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH6_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH7_R0_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH7_R1_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH7_R2_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH7_R3_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH7_R4_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH7_R5_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH7_R6_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CH7_R7_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_MCS0_CH7_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_MCS0_CH7_ACB_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CH7_MHB_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_CH7_PC_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_CH7_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls0_MCS0_CH7_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_CTRG_union_tag { /* MCS[i] clear trigger control register */
  vuint32_t R;
  struct {
    vuint32_t TRG0:1;
    vuint32_t TRG1:1;
    vuint32_t TRG2:1;
    vuint32_t TRG3:1;
    vuint32_t TRG4:1;
    vuint32_t TRG5:1;
    vuint32_t TRG6:1;
    vuint32_t TRG7:1;
    vuint32_t TRG8:1;
    vuint32_t TRG9:1;
    vuint32_t TRG10:1;
    vuint32_t TRG11:1;
    vuint32_t TRG12:1;
    vuint32_t TRG13:1;
    vuint32_t TRG14:1;
    vuint32_t TRG15:1;
    vuint32_t TRG16:1;
    vuint32_t TRG17:1;
    vuint32_t TRG18:1;
    vuint32_t TRG19:1;
    vuint32_t TRG20:1;
    vuint32_t TRG21:1;
    vuint32_t TRG22:1;
    vuint32_t TRG23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_CTRG_tag;

typedef union GTM_gtm_cls0_MCS0_STRG_union_tag { /* MCS[i] set trigger control register */
  vuint32_t R;
  struct {
    vuint32_t TRG0:1;
    vuint32_t TRG1:1;
    vuint32_t TRG2:1;
    vuint32_t TRG3:1;
    vuint32_t TRG4:1;
    vuint32_t TRG5:1;
    vuint32_t TRG6:1;
    vuint32_t TRG7:1;
    vuint32_t TRG8:1;
    vuint32_t TRG9:1;
    vuint32_t TRG10:1;
    vuint32_t TRG11:1;
    vuint32_t TRG12:1;
    vuint32_t TRG13:1;
    vuint32_t TRG14:1;
    vuint32_t TRG15:1;
    vuint32_t TRG16:1;
    vuint32_t TRG17:1;
    vuint32_t TRG18:1;
    vuint32_t TRG19:1;
    vuint32_t TRG20:1;
    vuint32_t TRG21:1;
    vuint32_t TRG22:1;
    vuint32_t TRG23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_MCS0_STRG_tag;

typedef union GTM_gtm_cls0_MCS0_CTRL_STAT_union_tag { /* MCS[i] control and status register */
  vuint32_t R;
  struct {
    vuint32_t SCD_MODE:2;
    vuint32_t _unused_2:6;
    vuint32_t SCD_CH:4;
    vuint32_t _unused_12:4;
    vuint32_t RAM_RST:1;
    vuint32_t _unused_17:3;
    vuint32_t ERR_SRC_ID:3;
    vuint32_t _unused_23:1;
    vuint32_t EN_TIM_FOUT:1;
    vuint32_t EN_HVD:1;
    vuint32_t HLT_AEIM_ERR:1;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls0_MCS0_CTRL_STAT_tag;

typedef union GTM_gtm_cls0_MCS0_RESET_union_tag { /* MCS[i] reset register */
  vuint32_t R;
  struct {
    vuint32_t RST0:1;
    vuint32_t RST1:1;
    vuint32_t RST2:1;
    vuint32_t RST3:1;
    vuint32_t RST4:1;
    vuint32_t RST5:1;
    vuint32_t RST6:1;
    vuint32_t RST7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_RESET_tag;

typedef union GTM_gtm_cls0_MCS0_CAT_union_tag { /* MCS[i] cancel ARU transfer instruction */
  vuint32_t R;
  struct {
    vuint32_t CAT0:1;
    vuint32_t CAT1:1;
    vuint32_t CAT2:1;
    vuint32_t CAT3:1;
    vuint32_t CAT4:1;
    vuint32_t CAT5:1;
    vuint32_t CAT6:1;
    vuint32_t CAT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CAT_tag;

typedef union GTM_gtm_cls0_MCS0_CWT_union_tag { /* MCS[i] cancel waiting instruction */
  vuint32_t R;
  struct {
    vuint32_t CWT0:1;
    vuint32_t CWT1:1;
    vuint32_t CWT2:1;
    vuint32_t CWT3:1;
    vuint32_t CWT4:1;
    vuint32_t CWT5:1;
    vuint32_t CWT6:1;
    vuint32_t CWT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_CWT_tag;

typedef union GTM_gtm_cls0_MCS0_ERR_union_tag { /* MCS[i] error register */
  vuint32_t R;
  struct {
    vuint32_t ERR0:1;
    vuint32_t ERR1:1;
    vuint32_t ERR2:1;
    vuint32_t ERR3:1;
    vuint32_t ERR4:1;
    vuint32_t ERR5:1;
    vuint32_t ERR6:1;
    vuint32_t ERR7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_ERR_tag;

typedef union GTM_gtm_cls0_MCS0_REG_PROT_union_tag { /* MCS[i] write protection register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:2;
    vuint32_t WPROT1:2;
    vuint32_t WPROT2:2;
    vuint32_t WPROT3:2;
    vuint32_t WPROT4:2;
    vuint32_t WPROT5:2;
    vuint32_t WPROT6:2;
    vuint32_t WPROT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_MCS0_REG_PROT_tag;

typedef union GTM_gtm_cls0_MCS0_SINT_IRQ_NOTIFY_union_tag { /* MCS[i] shared interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_IRQ0:1;
    vuint32_t S_IRQ1:1;
    vuint32_t S_IRQ2:1;
    vuint32_t S_IRQ3:1;
    vuint32_t S_IRQ4:1;
    vuint32_t S_IRQ5:1;
    vuint32_t S_IRQ6:1;
    vuint32_t S_IRQ7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_SINT_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_SINT_IRQ_EN_union_tag { /* MCS[i] shared interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_IRQ0_EN:1;
    vuint32_t S_IRQ1_EN:1;
    vuint32_t S_IRQ2_EN:1;
    vuint32_t S_IRQ3_EN:1;
    vuint32_t S_IRQ4_EN:1;
    vuint32_t S_IRQ5_EN:1;
    vuint32_t S_IRQ6_EN:1;
    vuint32_t S_IRQ7_EN:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_SINT_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_SINT_IRQ_FORCINT_union_tag { /* MCS[i] force shared interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_IRQ0:1;
    vuint32_t TRG_S_IRQ1:1;
    vuint32_t TRG_S_IRQ2:1;
    vuint32_t TRG_S_IRQ3:1;
    vuint32_t TRG_S_IRQ4:1;
    vuint32_t TRG_S_IRQ5:1;
    vuint32_t TRG_S_IRQ6:1;
    vuint32_t TRG_S_IRQ7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_SINT_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_SINT_IRQ_MODE_union_tag { /* MCS[i] shared interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_SINT_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_HBP0_CTRL_union_tag { /* MCS[i] hardware break point h control register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH0:1;
    vuint32_t EN_CH1:1;
    vuint32_t EN_CH2:1;
    vuint32_t EN_CH3:1;
    vuint32_t EN_CH4:1;
    vuint32_t EN_CH5:1;
    vuint32_t EN_CH6:1;
    vuint32_t EN_CH7:1;
    vuint32_t SCOPE:2;
    vuint32_t _unused_10:2;
    vuint32_t TYPE:3;
    vuint32_t _unused_15:1;
    vuint32_t AND:1;
    vuint32_t NOT:1;
    vuint32_t _unused_18:14;
  } B;
} GTM_gtm_cls0_MCS0_HBP0_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_HBP0_PATTERN_union_tag { /* MCS[i] hardware break point pattern register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls0_MCS0_HBP0_PATTERN_tag;

typedef union GTM_gtm_cls0_MCS0_HBP0_STATUS_union_tag { /* MCS[i] hardware break point status register */
  vuint32_t R;
  struct {
    vuint32_t HALT_CH0:1;
    vuint32_t HALT_CH1:1;
    vuint32_t HALT_CH2:1;
    vuint32_t HALT_CH3:1;
    vuint32_t HALT_CH4:1;
    vuint32_t HALT_CH5:1;
    vuint32_t HALT_CH6:1;
    vuint32_t HALT_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_HBP0_STATUS_tag;

typedef union GTM_gtm_cls0_MCS0_HBP0_IRQ_NOTIFY_union_tag { /* MCS[i] hardware break point interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_MCS0_HBP0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_HBP0_IRQ_EN_union_tag { /* MCS[i] hardware break point interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ_EN:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_MCS0_HBP0_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_HBP0_IRQ_FORCINT_union_tag { /* MCS[i] force hardware break point interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_MCS0_HBP0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_HBP0_IRQ_MODE_union_tag { /* MCS[i] break point h interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_HBP0_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_MCS0_HBP1_CTRL_union_tag { /* MCS[i] hardware break point h control register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH0:1;
    vuint32_t EN_CH1:1;
    vuint32_t EN_CH2:1;
    vuint32_t EN_CH3:1;
    vuint32_t EN_CH4:1;
    vuint32_t EN_CH5:1;
    vuint32_t EN_CH6:1;
    vuint32_t EN_CH7:1;
    vuint32_t SCOPE:2;
    vuint32_t _unused_10:2;
    vuint32_t TYPE:3;
    vuint32_t _unused_15:1;
    vuint32_t AND:1;
    vuint32_t NOT:1;
    vuint32_t _unused_18:14;
  } B;
} GTM_gtm_cls0_MCS0_HBP1_CTRL_tag;

typedef union GTM_gtm_cls0_MCS0_HBP1_PATTERN_union_tag { /* MCS[i] hardware break point pattern register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls0_MCS0_HBP1_PATTERN_tag;

typedef union GTM_gtm_cls0_MCS0_HBP1_STATUS_union_tag { /* MCS[i] hardware break point status register */
  vuint32_t R;
  struct {
    vuint32_t HALT_CH0:1;
    vuint32_t HALT_CH1:1;
    vuint32_t HALT_CH2:1;
    vuint32_t HALT_CH3:1;
    vuint32_t HALT_CH4:1;
    vuint32_t HALT_CH5:1;
    vuint32_t HALT_CH6:1;
    vuint32_t HALT_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_MCS0_HBP1_STATUS_tag;

typedef union GTM_gtm_cls0_MCS0_HBP1_IRQ_NOTIFY_union_tag { /* MCS[i] hardware break point interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_MCS0_HBP1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_MCS0_HBP1_IRQ_EN_union_tag { /* MCS[i] hardware break point interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ_EN:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_MCS0_HBP1_IRQ_EN_tag;

typedef union GTM_gtm_cls0_MCS0_HBP1_IRQ_FORCINT_union_tag { /* MCS[i] force hardware break point interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_MCS0_HBP1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_MCS0_HBP1_IRQ_MODE_union_tag { /* MCS[i] break point h interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_MCS0_HBP1_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_CCM0_ARP0_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP0_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP0_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP0_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP1_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP1_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP1_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP1_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP2_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP2_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP2_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP2_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP3_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP3_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP3_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP3_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP4_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP4_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP4_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP4_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP5_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP5_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP5_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP5_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP6_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP6_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP6_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP6_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP7_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP7_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP7_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP7_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP8_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP8_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP8_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP8_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_ARP9_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls0_CCM0_ARP9_CTRL_tag;

typedef union GTM_gtm_cls0_CCM0_ARP9_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CCM0_ARP9_PROT_tag;

typedef union GTM_gtm_cls0_CCM0_HW_CONF2_union_tag { /* CCM[i] 2. Hardware Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t AXIM_ID_WIDTH:5;
    vuint32_t AXIM_PRIV_ACC:1;
    vuint32_t AXIM_SEC_ACC:1;
    vuint32_t AXIM_POSTED_WRITE:1;
    vuint32_t _unused_8:1;
    vuint32_t TIO_OUT_RST:1;
    vuint32_t _unused_10:6;
    vuint32_t AXIS_DATA_SIZE:1;
    vuint32_t _unused_17:1;
    vuint32_t AXIM_DATA_SIZE:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_CCM0_HW_CONF2_tag;

typedef union GTM_gtm_cls0_CCM0_AEIM_STA_union_tag { /* CCM[i] MCS Bus Master Status Register */
  vuint32_t R;
  struct {
    vuint32_t AEIM_XPT_ADDR:16;
    vuint32_t _unused_16:8;
    vuint32_t AEIM_XPT_STA:2;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_CCM0_AEIM_STA_tag;

typedef union GTM_gtm_cls0_CCM0_HW_CONF_union_tag { /* CCM[i] Hardware Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t GRSTEN:1;
    vuint32_t BRIDGE_MODE_RST:1;
    vuint32_t SYNC_INPUT_REG:1;
    vuint32_t CFG_CLOCK_RATE:1;
    vuint32_t ATOM_OUT_RST:1;
    vuint32_t ATOM_TRIG_CHAIN:3;
    vuint32_t TOM_OUT_RST:1;
    vuint32_t TOM_TRIG_CHAIN:3;
    vuint32_t RAM_INIT_RST:1;
    vuint32_t ERM:1;
    vuint32_t _unused_14:1;
    vuint32_t RESET_ACTIVE:1;
    vuint32_t IRQ_MODE_LEVEL:1;
    vuint32_t IRQ_MODE_PULSE:1;
    vuint32_t IRQ_MODE_PULSE_NOTIFY:1;
    vuint32_t IRQ_MODE_SINGLE_PULSE:1;
    vuint32_t ATOM_TRIG_INTCHAIN:4;
    vuint32_t TOM_TRIG_INTCHAIN:5;
    vuint32_t INT_CLK_EN_GEN:1;
    vuint32_t AEI_ADDR_PIPELINE_STAGE:1;
    vuint32_t AEI_RDATA_PIPELINE_STAGE:1;
  } B;
} GTM_gtm_cls0_CCM0_HW_CONF_tag;

typedef union GTM_gtm_cls0_CCM0_TIM_AUX_IN_SRC_union_tag { /* CCM[i] TIM AUX Input Source Register */
  vuint32_t R;
  struct {
    vuint32_t SRC_CH0:1;
    vuint32_t SRC_CH1:1;
    vuint32_t SRC_CH2:1;
    vuint32_t SRC_CH3:1;
    vuint32_t SRC_CH4:1;
    vuint32_t SRC_CH5:1;
    vuint32_t SRC_CH6:1;
    vuint32_t SRC_CH7:1;
    vuint32_t _unused_8:8;
    vuint32_t SEL_OUT_N_CH0:1;
    vuint32_t SEL_OUT_N_CH1:1;
    vuint32_t SEL_OUT_N_CH2:1;
    vuint32_t SEL_OUT_N_CH3:1;
    vuint32_t SEL_OUT_N_CH4:1;
    vuint32_t SEL_OUT_N_CH5:1;
    vuint32_t SEL_OUT_N_CH6:1;
    vuint32_t SEL_OUT_N_CH7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_CCM0_TIM_AUX_IN_SRC_tag;

typedef union GTM_gtm_cls0_CCM0_EXT_CAP_EN_union_tag { /* CCM[i] External Capture Enable Register */
  vuint32_t R;
  struct {
    vuint32_t TIM_I_EXT_CAP_EN0:1;
    vuint32_t TIM_I_EXT_CAP_EN1:1;
    vuint32_t TIM_I_EXT_CAP_EN2:1;
    vuint32_t TIM_I_EXT_CAP_EN3:1;
    vuint32_t TIM_I_EXT_CAP_EN4:1;
    vuint32_t TIM_I_EXT_CAP_EN5:1;
    vuint32_t TIM_I_EXT_CAP_EN6:1;
    vuint32_t TIM_I_EXT_CAP_EN7:1;
    vuint32_t TIM_IP1_EXT_CAP_EN0:1;
    vuint32_t TIM_IP1_EXT_CAP_EN1:1;
    vuint32_t TIM_IP1_EXT_CAP_EN2:1;
    vuint32_t TIM_IP1_EXT_CAP_EN3:1;
    vuint32_t TIM_IP1_EXT_CAP_EN4:1;
    vuint32_t TIM_IP1_EXT_CAP_EN5:1;
    vuint32_t TIM_IP1_EXT_CAP_EN6:1;
    vuint32_t TIM_IP1_EXT_CAP_EN7:1;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_CCM0_EXT_CAP_EN_tag;

typedef union GTM_gtm_cls0_CCM0_TOM_OUT_union_tag { /* CCM[i] TOM Output Register */
  vuint32_t R;
  struct {
    vuint32_t TOM_OUT0:1;
    vuint32_t TOM_OUT1:1;
    vuint32_t TOM_OUT2:1;
    vuint32_t TOM_OUT3:1;
    vuint32_t TOM_OUT4:1;
    vuint32_t TOM_OUT5:1;
    vuint32_t TOM_OUT6:1;
    vuint32_t TOM_OUT7:1;
    vuint32_t TOM_OUT8:1;
    vuint32_t TOM_OUT9:1;
    vuint32_t TOM_OUT10:1;
    vuint32_t TOM_OUT11:1;
    vuint32_t TOM_OUT12:1;
    vuint32_t TOM_OUT13:1;
    vuint32_t TOM_OUT14:1;
    vuint32_t TOM_OUT15:1;
    vuint32_t TOM_OUT_N0:1;
    vuint32_t TOM_OUT_N1:1;
    vuint32_t TOM_OUT_N2:1;
    vuint32_t TOM_OUT_N3:1;
    vuint32_t TOM_OUT_N4:1;
    vuint32_t TOM_OUT_N5:1;
    vuint32_t TOM_OUT_N6:1;
    vuint32_t TOM_OUT_N7:1;
    vuint32_t TOM_OUT_N8:1;
    vuint32_t TOM_OUT_N9:1;
    vuint32_t TOM_OUT_N10:1;
    vuint32_t TOM_OUT_N11:1;
    vuint32_t TOM_OUT_N12:1;
    vuint32_t TOM_OUT_N13:1;
    vuint32_t TOM_OUT_N14:1;
    vuint32_t TOM_OUT_N15:1;
  } B;
} GTM_gtm_cls0_CCM0_TOM_OUT_tag;

typedef union GTM_gtm_cls0_CCM0_ATOM_OUT_union_tag { /* CCM[i] ATOM Output Register */
  vuint32_t R;
  struct {
    vuint32_t ATOM_I_OUT0:1;
    vuint32_t ATOM_I_OUT1:1;
    vuint32_t ATOM_I_OUT2:1;
    vuint32_t ATOM_I_OUT3:1;
    vuint32_t ATOM_I_OUT4:1;
    vuint32_t ATOM_I_OUT5:1;
    vuint32_t ATOM_I_OUT6:1;
    vuint32_t ATOM_I_OUT7:1;
    vuint32_t ATOM_I_OUT_N0:1;
    vuint32_t ATOM_I_OUT_N1:1;
    vuint32_t ATOM_I_OUT_N2:1;
    vuint32_t ATOM_I_OUT_N3:1;
    vuint32_t ATOM_I_OUT_N4:1;
    vuint32_t ATOM_I_OUT_N5:1;
    vuint32_t ATOM_I_OUT_N6:1;
    vuint32_t ATOM_I_OUT_N7:1;
    vuint32_t ATOM_IP1_OUT0:1;
    vuint32_t ATOM_IP1_OUT1:1;
    vuint32_t ATOM_IP1_OUT2:1;
    vuint32_t ATOM_IP1_OUT3:1;
    vuint32_t ATOM_IP1_OUT4:1;
    vuint32_t ATOM_IP1_OUT5:1;
    vuint32_t ATOM_IP1_OUT6:1;
    vuint32_t ATOM_IP1_OUT7:1;
    vuint32_t ATOM_IP1_OUT_N0:1;
    vuint32_t ATOM_IP1_OUT_N1:1;
    vuint32_t ATOM_IP1_OUT_N2:1;
    vuint32_t ATOM_IP1_OUT_N3:1;
    vuint32_t ATOM_IP1_OUT_N4:1;
    vuint32_t ATOM_IP1_OUT_N5:1;
    vuint32_t ATOM_IP1_OUT_N6:1;
    vuint32_t ATOM_IP1_OUT_N7:1;
  } B;
} GTM_gtm_cls0_CCM0_ATOM_OUT_tag;

typedef union GTM_gtm_cls0_CCM0_CMU_CLK_CFG_union_tag { /* CCM[i] CMU Clock Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CLK0_SRC:2;
    vuint32_t _unused_2:2;
    vuint32_t CLK1_SRC:2;
    vuint32_t _unused_6:2;
    vuint32_t CLK2_SRC:2;
    vuint32_t _unused_10:2;
    vuint32_t CLK3_SRC:2;
    vuint32_t _unused_14:2;
    vuint32_t CLK4_SRC:2;
    vuint32_t _unused_18:2;
    vuint32_t CLK5_SRC:2;
    vuint32_t _unused_22:2;
    vuint32_t CLK6_SRC:2;
    vuint32_t _unused_26:2;
    vuint32_t CLK7_SRC:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_CCM0_CMU_CLK_CFG_tag;

typedef union GTM_gtm_cls0_CCM0_CMU_FXCLK_CFG_union_tag { /* CCM[i] CMU Fixed Clock Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t FXCLK0_SRC:4;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_CCM0_CMU_FXCLK_CFG_tag;

typedef union GTM_gtm_cls0_CCM0_CFG_union_tag { /* CCM[i] Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t EN_TIM:1;
    vuint32_t EN_TOM_SPE_TDTM:1;
    vuint32_t EN_ATOM_ADTM:1;
    vuint32_t EN_MCS:1;
    vuint32_t EN_DPLL_MAP:1;
    vuint32_t EN_BRC:1;
    vuint32_t EN_PSM:1;
    vuint32_t _unused_7:9;
    vuint32_t CLS_CLK_DIV:2;
    vuint32_t _unused_18:12;
    vuint32_t TBU_DIR1:1;
    vuint32_t TBU_DIR2:1;
  } B;
} GTM_gtm_cls0_CCM0_CFG_tag;

typedef union GTM_gtm_cls0_CCM0_PROT_union_tag { /* CCM[i] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t CLS_PROT:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls0_CCM0_PROT_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CTRL_union_tag { /* CDTM[i]_DTM[d] global configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t CLK_SEL:2;
    vuint32_t DTM_SEL:2;
    vuint32_t UPD_MODE:3;
    vuint32_t CH_SHUTOFF_EN:1;
    vuint32_t SR_UPD_EN:1;
    vuint32_t _unused_9:7;
    vuint32_t SHUT_OFF_RST:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CTRL_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL1_union_tag { /* CDTM[i]_DTM[d] channel control register 1 */
  vuint32_t R;
  struct {
    vuint32_t O1SEL_0:1;
    vuint32_t _unused_1:2;
    vuint32_t SWAP_0:1;
    vuint32_t O1F_0:2;
    vuint32_t XDT_EN_0_1:1;
    vuint32_t _unused_7:1;
    vuint32_t O1SEL_1:1;
    vuint32_t I1SEL_1:1;
    vuint32_t SH_EN_1:1;
    vuint32_t SWAP_1:1;
    vuint32_t O1F_1:2;
    vuint32_t _unused_14:2;
    vuint32_t O1SEL_2:1;
    vuint32_t I1SEL_2:1;
    vuint32_t SH_EN_2:1;
    vuint32_t SWAP_2:1;
    vuint32_t O1F_2:2;
    vuint32_t XDT_EN_2_3:1;
    vuint32_t _unused_23:1;
    vuint32_t O1SEL_3:1;
    vuint32_t I1SEL_3:1;
    vuint32_t SH_EN_3:1;
    vuint32_t SWAP_3:1;
    vuint32_t O1F_3:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL1_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL2_union_tag { /* CDTM[i]_DTM[d] channel control register 2 */
  vuint32_t R;
  struct {
    vuint32_t POL0_0:1;
    vuint32_t OC0_0:1;
    vuint32_t SL0_0:1;
    vuint32_t DT0_0:1;
    vuint32_t POL1_0:1;
    vuint32_t OC1_0:1;
    vuint32_t SL1_0:1;
    vuint32_t DT1_0:1;
    vuint32_t POL0_1:1;
    vuint32_t OC0_1:1;
    vuint32_t SL0_1:1;
    vuint32_t DT0_1:1;
    vuint32_t POL1_1:1;
    vuint32_t OC1_1:1;
    vuint32_t SL1_1:1;
    vuint32_t DT1_1:1;
    vuint32_t POL0_2:1;
    vuint32_t OC0_2:1;
    vuint32_t SL0_2:1;
    vuint32_t DT0_2:1;
    vuint32_t POL1_2:1;
    vuint32_t OC1_2:1;
    vuint32_t SL1_2:1;
    vuint32_t DT1_2:1;
    vuint32_t POL0_3:1;
    vuint32_t OC0_3:1;
    vuint32_t SL0_3:1;
    vuint32_t DT0_3:1;
    vuint32_t POL1_3:1;
    vuint32_t OC1_3:1;
    vuint32_t SL1_3:1;
    vuint32_t DT1_3:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL2_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL2_SR_union_tag { /* CDTM[i] DTM[j] channel control register 2 shadow */
  vuint32_t R;
  struct {
    vuint32_t POL0_0_SR:1;
    vuint32_t OC0_0_SR:1;
    vuint32_t SL0_0_SR:1;
    vuint32_t DT0_0_SR:1;
    vuint32_t POL1_0_SR:1;
    vuint32_t OC1_0_SR:1;
    vuint32_t SL1_0_SR:1;
    vuint32_t DT1_0_SR:1;
    vuint32_t POL0_1_SR:1;
    vuint32_t OC0_1_SR:1;
    vuint32_t SL0_1_SR:1;
    vuint32_t DT0_1_SR:1;
    vuint32_t POL1_1_SR:1;
    vuint32_t OC1_1_SR:1;
    vuint32_t SL1_1_SR:1;
    vuint32_t DT1_1_SR:1;
    vuint32_t POL0_2_SR:1;
    vuint32_t OC0_2_SR:1;
    vuint32_t SL0_2_SR:1;
    vuint32_t DT0_2_SR:1;
    vuint32_t POL1_2_SR:1;
    vuint32_t OC1_2_SR:1;
    vuint32_t SL1_2_SR:1;
    vuint32_t DT1_2_SR:1;
    vuint32_t POL0_3_SR:1;
    vuint32_t OC0_3_SR:1;
    vuint32_t SL0_3_SR:1;
    vuint32_t DT0_3_SR:1;
    vuint32_t POL1_3_SR:1;
    vuint32_t OC1_3_SR:1;
    vuint32_t SL1_3_SR:1;
    vuint32_t DT1_3_SR:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL2_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_PS_CTRL_union_tag { /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t RELBLK:10;
    vuint32_t _unused_10:6;
    vuint32_t PSU_IN_SEL:1;
    vuint32_t IN_POL:1;
    vuint32_t TIM_SEL:1;
    vuint32_t _unused_19:1;
    vuint32_t SHIFT_SEL:2;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_PS_CTRL_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH_DTV_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE:13;
    vuint32_t _unused_13:3;
    vuint32_t RELFALL:13;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH_DTV_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH_SR_union_tag { /* CDTM[i]_DTM[d] channel shadow register */
  vuint32_t R;
  struct {
    vuint32_t SL0_0_SR_SR:1;
    vuint32_t SL1_0_SR_SR:1;
    vuint32_t SL0_1_SR_SR:1;
    vuint32_t SL1_1_SR_SR:1;
    vuint32_t SL0_2_SR_SR:1;
    vuint32_t SL1_2_SR_SR:1;
    vuint32_t SL0_3_SR_SR:1;
    vuint32_t SL1_3_SR_SR:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL3_union_tag { /* CDTM[i]_DTM[d] channel control register 3 */
  vuint32_t R;
  struct {
    vuint32_t CII0:1;
    vuint32_t CIS0:1;
    vuint32_t TSEL0_0:1;
    vuint32_t TSEL1_0:1;
    vuint32_t _unused_4:4;
    vuint32_t CII1:1;
    vuint32_t CIS1:1;
    vuint32_t TSEL0_1:1;
    vuint32_t TSEL1_1:1;
    vuint32_t _unused_12:4;
    vuint32_t CII2:1;
    vuint32_t CIS2:1;
    vuint32_t TSEL0_2:1;
    vuint32_t TSEL1_2:1;
    vuint32_t _unused_20:4;
    vuint32_t CII3:1;
    vuint32_t CIS3:1;
    vuint32_t TSEL0_3:1;
    vuint32_t TSEL1_3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL3_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CTRL2_union_tag { /* CDTM[i]_DTM[d] global configuration and control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SHUTOFF_SEL_0:3;
    vuint32_t SHUTOFF_POL_0:1;
    vuint32_t UPD_MODE_0:2;
    vuint32_t SHUT_OFF_RST_0:1;
    vuint32_t WR_EN_0:1;
    vuint32_t SHUTOFF_SEL_1:3;
    vuint32_t SHUTOFF_POL_1:1;
    vuint32_t UPD_MODE_1:2;
    vuint32_t SHUT_OFF_RST_1:1;
    vuint32_t WR_EN_1:1;
    vuint32_t SHUTOFF_SEL_2:3;
    vuint32_t SHUTOFF_POL_2:1;
    vuint32_t UPD_MODE_2:2;
    vuint32_t SHUT_OFF_RST_2:1;
    vuint32_t WR_EN_2:1;
    vuint32_t SHUTOFF_SEL_3:3;
    vuint32_t SHUTOFF_POL_3:1;
    vuint32_t UPD_MODE_3:2;
    vuint32_t SHUT_OFF_RST_3:1;
    vuint32_t WR_EN_3:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CTRL2_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH0_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH0_DTV_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH1_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH1_DTV_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH2_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH2_DTV_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM4_CH3_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM4_CH3_DTV_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CTRL_union_tag { /* CDTM[i]_DTM[d] global configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t CLK_SEL:2;
    vuint32_t DTM_SEL:2;
    vuint32_t UPD_MODE:3;
    vuint32_t CH_SHUTOFF_EN:1;
    vuint32_t SR_UPD_EN:1;
    vuint32_t _unused_9:7;
    vuint32_t SHUT_OFF_RST:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CTRL_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL1_union_tag { /* CDTM[i]_DTM[d] channel control register 1 */
  vuint32_t R;
  struct {
    vuint32_t O1SEL_0:1;
    vuint32_t I1SEL_0:1;
    vuint32_t _unused_2:1;
    vuint32_t SWAP_0:1;
    vuint32_t O1F_0:2;
    vuint32_t XDT_EN_0_1:1;
    vuint32_t _unused_7:1;
    vuint32_t O1SEL_1:1;
    vuint32_t I1SEL_1:1;
    vuint32_t SH_EN_1:1;
    vuint32_t SWAP_1:1;
    vuint32_t O1F_1:2;
    vuint32_t _unused_14:2;
    vuint32_t O1SEL_2:1;
    vuint32_t I1SEL_2:1;
    vuint32_t SH_EN_2:1;
    vuint32_t SWAP_2:1;
    vuint32_t O1F_2:2;
    vuint32_t XDT_EN_2_3:1;
    vuint32_t _unused_23:1;
    vuint32_t O1SEL_3:1;
    vuint32_t I1SEL_3:1;
    vuint32_t SH_EN_3:1;
    vuint32_t SWAP_3:1;
    vuint32_t O1F_3:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL1_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL2_union_tag { /* CDTM[i]_DTM[d] channel control register 2 */
  vuint32_t R;
  struct {
    vuint32_t POL0_0:1;
    vuint32_t OC0_0:1;
    vuint32_t SL0_0:1;
    vuint32_t DT0_0:1;
    vuint32_t POL1_0:1;
    vuint32_t OC1_0:1;
    vuint32_t SL1_0:1;
    vuint32_t DT1_0:1;
    vuint32_t POL0_1:1;
    vuint32_t OC0_1:1;
    vuint32_t SL0_1:1;
    vuint32_t DT0_1:1;
    vuint32_t POL1_1:1;
    vuint32_t OC1_1:1;
    vuint32_t SL1_1:1;
    vuint32_t DT1_1:1;
    vuint32_t POL0_2:1;
    vuint32_t OC0_2:1;
    vuint32_t SL0_2:1;
    vuint32_t DT0_2:1;
    vuint32_t POL1_2:1;
    vuint32_t OC1_2:1;
    vuint32_t SL1_2:1;
    vuint32_t DT1_2:1;
    vuint32_t POL0_3:1;
    vuint32_t OC0_3:1;
    vuint32_t SL0_3:1;
    vuint32_t DT0_3:1;
    vuint32_t POL1_3:1;
    vuint32_t OC1_3:1;
    vuint32_t SL1_3:1;
    vuint32_t DT1_3:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL2_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL2_SR_union_tag { /* CDTM[i] DTM[j] channel control register 2 shadow */
  vuint32_t R;
  struct {
    vuint32_t POL0_0_SR:1;
    vuint32_t OC0_0_SR:1;
    vuint32_t SL0_0_SR:1;
    vuint32_t DT0_0_SR:1;
    vuint32_t POL1_0_SR:1;
    vuint32_t OC1_0_SR:1;
    vuint32_t SL1_0_SR:1;
    vuint32_t DT1_0_SR:1;
    vuint32_t POL0_1_SR:1;
    vuint32_t OC0_1_SR:1;
    vuint32_t SL0_1_SR:1;
    vuint32_t DT0_1_SR:1;
    vuint32_t POL1_1_SR:1;
    vuint32_t OC1_1_SR:1;
    vuint32_t SL1_1_SR:1;
    vuint32_t DT1_1_SR:1;
    vuint32_t POL0_2_SR:1;
    vuint32_t OC0_2_SR:1;
    vuint32_t SL0_2_SR:1;
    vuint32_t DT0_2_SR:1;
    vuint32_t POL1_2_SR:1;
    vuint32_t OC1_2_SR:1;
    vuint32_t SL1_2_SR:1;
    vuint32_t DT1_2_SR:1;
    vuint32_t POL0_3_SR:1;
    vuint32_t OC0_3_SR:1;
    vuint32_t SL0_3_SR:1;
    vuint32_t DT0_3_SR:1;
    vuint32_t POL1_3_SR:1;
    vuint32_t OC1_3_SR:1;
    vuint32_t SL1_3_SR:1;
    vuint32_t DT1_3_SR:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL2_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_PS_CTRL_union_tag { /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t RELBLK:10;
    vuint32_t _unused_10:6;
    vuint32_t PSU_IN_SEL:1;
    vuint32_t IN_POL:1;
    vuint32_t TIM_SEL:1;
    vuint32_t _unused_19:1;
    vuint32_t SHIFT_SEL:2;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_PS_CTRL_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH_DTV_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE:13;
    vuint32_t _unused_13:3;
    vuint32_t RELFALL:13;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH_DTV_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH_SR_union_tag { /* CDTM[i]_DTM[d] channel shadow register */
  vuint32_t R;
  struct {
    vuint32_t SL0_0_SR_SR:1;
    vuint32_t SL1_0_SR_SR:1;
    vuint32_t SL0_1_SR_SR:1;
    vuint32_t SL1_1_SR_SR:1;
    vuint32_t SL0_2_SR_SR:1;
    vuint32_t SL1_2_SR_SR:1;
    vuint32_t SL0_3_SR_SR:1;
    vuint32_t SL1_3_SR_SR:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL3_union_tag { /* CDTM[i]_DTM[d] channel control register 3 */
  vuint32_t R;
  struct {
    vuint32_t CII0:1;
    vuint32_t CIS0:1;
    vuint32_t TSEL0_0:1;
    vuint32_t TSEL1_0:1;
    vuint32_t _unused_4:4;
    vuint32_t CII1:1;
    vuint32_t CIS1:1;
    vuint32_t TSEL0_1:1;
    vuint32_t TSEL1_1:1;
    vuint32_t _unused_12:4;
    vuint32_t CII2:1;
    vuint32_t CIS2:1;
    vuint32_t TSEL0_2:1;
    vuint32_t TSEL1_2:1;
    vuint32_t _unused_20:4;
    vuint32_t CII3:1;
    vuint32_t CIS3:1;
    vuint32_t TSEL0_3:1;
    vuint32_t TSEL1_3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL3_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CTRL2_union_tag { /* CDTM[i]_DTM[d] global configuration and control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SHUTOFF_SEL_0:3;
    vuint32_t SHUTOFF_POL_0:1;
    vuint32_t UPD_MODE_0:2;
    vuint32_t SHUT_OFF_RST_0:1;
    vuint32_t WR_EN_0:1;
    vuint32_t SHUTOFF_SEL_1:3;
    vuint32_t SHUTOFF_POL_1:1;
    vuint32_t UPD_MODE_1:2;
    vuint32_t SHUT_OFF_RST_1:1;
    vuint32_t WR_EN_1:1;
    vuint32_t SHUTOFF_SEL_2:3;
    vuint32_t SHUTOFF_POL_2:1;
    vuint32_t UPD_MODE_2:2;
    vuint32_t SHUT_OFF_RST_2:1;
    vuint32_t WR_EN_2:1;
    vuint32_t SHUTOFF_SEL_3:3;
    vuint32_t SHUTOFF_POL_3:1;
    vuint32_t UPD_MODE_3:2;
    vuint32_t SHUT_OFF_RST_3:1;
    vuint32_t WR_EN_3:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CTRL2_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH0_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH0_DTV_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH1_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH1_DTV_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH2_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH2_DTV_SR_tag;

typedef union GTM_gtm_cls0_CDTM0_DTM5_CH3_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls0_CDTM0_DTM5_CH3_DTV_SR_tag;

typedef union GTM_gtm_cls0_F2A0_CH0_ARU_RD_FIFO_union_tag { /* F2A[i] stream [x] ARU read register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_F2A0_CH0_ARU_RD_FIFO_tag;

typedef union GTM_gtm_cls0_F2A0_CH1_ARU_RD_FIFO_union_tag { /* F2A[i] stream [x] ARU read register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_F2A0_CH1_ARU_RD_FIFO_tag;

typedef union GTM_gtm_cls0_F2A0_CH2_ARU_RD_FIFO_union_tag { /* F2A[i] stream [x] ARU read register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_F2A0_CH2_ARU_RD_FIFO_tag;

typedef union GTM_gtm_cls0_F2A0_CH3_ARU_RD_FIFO_union_tag { /* F2A[i] stream [x] ARU read register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_F2A0_CH3_ARU_RD_FIFO_tag;

typedef union GTM_gtm_cls0_F2A0_CH4_ARU_RD_FIFO_union_tag { /* F2A[i] stream [x] ARU read register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_F2A0_CH4_ARU_RD_FIFO_tag;

typedef union GTM_gtm_cls0_F2A0_CH5_ARU_RD_FIFO_union_tag { /* F2A[i] stream [x] ARU read register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_F2A0_CH5_ARU_RD_FIFO_tag;

typedef union GTM_gtm_cls0_F2A0_CH6_ARU_RD_FIFO_union_tag { /* F2A[i] stream [x] ARU read register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_F2A0_CH6_ARU_RD_FIFO_tag;

typedef union GTM_gtm_cls0_F2A0_CH7_ARU_RD_FIFO_union_tag { /* F2A[i] stream [x] ARU read register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_F2A0_CH7_ARU_RD_FIFO_tag;

typedef union GTM_gtm_cls0_F2A0_CH0_STR_CFG_union_tag { /* F2A[i] stream [x] configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TMODE:2;
    vuint32_t DIR:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_F2A0_CH0_STR_CFG_tag;

typedef union GTM_gtm_cls0_F2A0_CH1_STR_CFG_union_tag { /* F2A[i] stream [x] configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TMODE:2;
    vuint32_t DIR:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_F2A0_CH1_STR_CFG_tag;

typedef union GTM_gtm_cls0_F2A0_CH2_STR_CFG_union_tag { /* F2A[i] stream [x] configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TMODE:2;
    vuint32_t DIR:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_F2A0_CH2_STR_CFG_tag;

typedef union GTM_gtm_cls0_F2A0_CH3_STR_CFG_union_tag { /* F2A[i] stream [x] configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TMODE:2;
    vuint32_t DIR:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_F2A0_CH3_STR_CFG_tag;

typedef union GTM_gtm_cls0_F2A0_CH4_STR_CFG_union_tag { /* F2A[i] stream [x] configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TMODE:2;
    vuint32_t DIR:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_F2A0_CH4_STR_CFG_tag;

typedef union GTM_gtm_cls0_F2A0_CH5_STR_CFG_union_tag { /* F2A[i] stream [x] configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TMODE:2;
    vuint32_t DIR:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_F2A0_CH5_STR_CFG_tag;

typedef union GTM_gtm_cls0_F2A0_CH6_STR_CFG_union_tag { /* F2A[i] stream [x] configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TMODE:2;
    vuint32_t DIR:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_F2A0_CH6_STR_CFG_tag;

typedef union GTM_gtm_cls0_F2A0_CH7_STR_CFG_union_tag { /* F2A[i] stream [x] configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t TMODE:2;
    vuint32_t DIR:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls0_F2A0_CH7_STR_CFG_tag;

typedef union GTM_gtm_cls0_F2A0_ENABLE_union_tag { /* F2A[i] stream activation register */
  vuint32_t R;
  struct {
    vuint32_t STR0_EN:2;
    vuint32_t STR1_EN:2;
    vuint32_t STR2_EN:2;
    vuint32_t STR3_EN:2;
    vuint32_t STR4_EN:2;
    vuint32_t STR5_EN:2;
    vuint32_t STR6_EN:2;
    vuint32_t STR7_EN:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_F2A0_ENABLE_tag;

typedef union GTM_gtm_cls0_F2A0_CTRL_union_tag { /* F2A[i] stream control register */
  vuint32_t R;
  struct {
    vuint32_t STR4_CONF:2;
    vuint32_t STR5_CONF:2;
    vuint32_t STR6_CONF:2;
    vuint32_t STR7_CONF:2;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_F2A0_CTRL_tag;

typedef union GTM_gtm_cls0_AFD0_CH0_BUF_ACC_union_tag { /* AFD [i] FIFO [x] buffer access register */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_AFD0_CH0_BUF_ACC_tag;

typedef union GTM_gtm_cls0_AFD0_CH1_BUF_ACC_union_tag { /* AFD [i] FIFO [x] buffer access register */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_AFD0_CH1_BUF_ACC_tag;

typedef union GTM_gtm_cls0_AFD0_CH2_BUF_ACC_union_tag { /* AFD [i] FIFO [x] buffer access register */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_AFD0_CH2_BUF_ACC_tag;

typedef union GTM_gtm_cls0_AFD0_CH3_BUF_ACC_union_tag { /* AFD [i] FIFO [x] buffer access register */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_AFD0_CH3_BUF_ACC_tag;

typedef union GTM_gtm_cls0_AFD0_CH4_BUF_ACC_union_tag { /* AFD [i] FIFO [x] buffer access register */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_AFD0_CH4_BUF_ACC_tag;

typedef union GTM_gtm_cls0_AFD0_CH5_BUF_ACC_union_tag { /* AFD [i] FIFO [x] buffer access register */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_AFD0_CH5_BUF_ACC_tag;

typedef union GTM_gtm_cls0_AFD0_CH6_BUF_ACC_union_tag { /* AFD [i] FIFO [x] buffer access register */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_AFD0_CH6_BUF_ACC_tag;

typedef union GTM_gtm_cls0_AFD0_CH7_BUF_ACC_union_tag { /* AFD [i] FIFO [x] buffer access register */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_AFD0_CH7_BUF_ACC_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_CTRL_union_tag { /* FIFO[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t RBM:1;
    vuint32_t RAP:1;
    vuint32_t FLUSH:1;
    vuint32_t WULOCK:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_CTRL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_END_ADDR_union_tag { /* FIFO[i] channel [x] end address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_END_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_START_ADDR_union_tag { /* FIFO[i] channel [x] start address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_START_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_UPPER_WM_union_tag { /* FIFO[i] channel [x] upper watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_UPPER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_LOWER_WM_union_tag { /* FIFO[i] channel [x] lower watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_LOWER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_STATUS_union_tag { /* FIFO[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t FULL:1;
    vuint32_t LOW_WM:1;
    vuint32_t UP_WM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_FILL_LEVEL_union_tag { /* FIFO[i] channel [x] fill level register */
  vuint32_t R;
  struct {
    vuint32_t LEVEL:11;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_FILL_LEVEL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_WR_PTR_union_tag { /* FIFO[i] channel [x] write pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_WR_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_RD_PTR_union_tag { /* FIFO[i] channel [x] read pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_RD_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_IRQ_NOTIFY_union_tag { /* FIFO[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY:1;
    vuint32_t FIFO_FULL:1;
    vuint32_t FIFO_LWM:1;
    vuint32_t FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_IRQ_EN_union_tag { /* FIFO[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_IRQ_EN:1;
    vuint32_t FIFO_FULL_IRQ_EN:1;
    vuint32_t FIFO_LWM_IRQ_EN:1;
    vuint32_t FIFO_UWM_IRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_IRQ_FORCINT_union_tag { /* FIFO[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_FIFO_EMPTY:1;
    vuint32_t TRG_FIFO_FULL:1;
    vuint32_t TRG_FIFO_LWM:1;
    vuint32_t TRG_FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_IRQ_MODE_union_tag { /* FIFO[i] channel [x] interrupt mode control register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t DMA_HYSTERESIS:1;
    vuint32_t DMA_HYST_DIR:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_FIFO0_CH0_EIRQ_EN_union_tag { /* FIFO[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_EIRQ_EN:1;
    vuint32_t FIFO_FULL_EIRQ_EN:1;
    vuint32_t FIFO_LWM_EIRQ_EN:1;
    vuint32_t FIFO_UWM_EIRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH0_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_CTRL_union_tag { /* FIFO[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t RBM:1;
    vuint32_t RAP:1;
    vuint32_t FLUSH:1;
    vuint32_t WULOCK:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_CTRL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_END_ADDR_union_tag { /* FIFO[i] channel [x] end address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_END_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_START_ADDR_union_tag { /* FIFO[i] channel [x] start address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_START_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_UPPER_WM_union_tag { /* FIFO[i] channel [x] upper watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_UPPER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_LOWER_WM_union_tag { /* FIFO[i] channel [x] lower watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_LOWER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_STATUS_union_tag { /* FIFO[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t FULL:1;
    vuint32_t LOW_WM:1;
    vuint32_t UP_WM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_FILL_LEVEL_union_tag { /* FIFO[i] channel [x] fill level register */
  vuint32_t R;
  struct {
    vuint32_t LEVEL:11;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_FILL_LEVEL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_WR_PTR_union_tag { /* FIFO[i] channel [x] write pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_WR_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_RD_PTR_union_tag { /* FIFO[i] channel [x] read pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_RD_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_IRQ_NOTIFY_union_tag { /* FIFO[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY:1;
    vuint32_t FIFO_FULL:1;
    vuint32_t FIFO_LWM:1;
    vuint32_t FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_IRQ_EN_union_tag { /* FIFO[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_IRQ_EN:1;
    vuint32_t FIFO_FULL_IRQ_EN:1;
    vuint32_t FIFO_LWM_IRQ_EN:1;
    vuint32_t FIFO_UWM_IRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_IRQ_FORCINT_union_tag { /* FIFO[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_FIFO_EMPTY:1;
    vuint32_t TRG_FIFO_FULL:1;
    vuint32_t TRG_FIFO_LWM:1;
    vuint32_t TRG_FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_IRQ_MODE_union_tag { /* FIFO[i] channel [x] interrupt mode control register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t DMA_HYSTERESIS:1;
    vuint32_t DMA_HYST_DIR:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_FIFO0_CH1_EIRQ_EN_union_tag { /* FIFO[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_EIRQ_EN:1;
    vuint32_t FIFO_FULL_EIRQ_EN:1;
    vuint32_t FIFO_LWM_EIRQ_EN:1;
    vuint32_t FIFO_UWM_EIRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH1_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_CTRL_union_tag { /* FIFO[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t RBM:1;
    vuint32_t RAP:1;
    vuint32_t FLUSH:1;
    vuint32_t WULOCK:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_CTRL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_END_ADDR_union_tag { /* FIFO[i] channel [x] end address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_END_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_START_ADDR_union_tag { /* FIFO[i] channel [x] start address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_START_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_UPPER_WM_union_tag { /* FIFO[i] channel [x] upper watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_UPPER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_LOWER_WM_union_tag { /* FIFO[i] channel [x] lower watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_LOWER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_STATUS_union_tag { /* FIFO[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t FULL:1;
    vuint32_t LOW_WM:1;
    vuint32_t UP_WM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_FILL_LEVEL_union_tag { /* FIFO[i] channel [x] fill level register */
  vuint32_t R;
  struct {
    vuint32_t LEVEL:11;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_FILL_LEVEL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_WR_PTR_union_tag { /* FIFO[i] channel [x] write pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_WR_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_RD_PTR_union_tag { /* FIFO[i] channel [x] read pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_RD_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_IRQ_NOTIFY_union_tag { /* FIFO[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY:1;
    vuint32_t FIFO_FULL:1;
    vuint32_t FIFO_LWM:1;
    vuint32_t FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_IRQ_EN_union_tag { /* FIFO[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_IRQ_EN:1;
    vuint32_t FIFO_FULL_IRQ_EN:1;
    vuint32_t FIFO_LWM_IRQ_EN:1;
    vuint32_t FIFO_UWM_IRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_IRQ_FORCINT_union_tag { /* FIFO[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_FIFO_EMPTY:1;
    vuint32_t TRG_FIFO_FULL:1;
    vuint32_t TRG_FIFO_LWM:1;
    vuint32_t TRG_FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_IRQ_MODE_union_tag { /* FIFO[i] channel [x] interrupt mode control register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t DMA_HYSTERESIS:1;
    vuint32_t DMA_HYST_DIR:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_FIFO0_CH2_EIRQ_EN_union_tag { /* FIFO[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_EIRQ_EN:1;
    vuint32_t FIFO_FULL_EIRQ_EN:1;
    vuint32_t FIFO_LWM_EIRQ_EN:1;
    vuint32_t FIFO_UWM_EIRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH2_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_CTRL_union_tag { /* FIFO[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t RBM:1;
    vuint32_t RAP:1;
    vuint32_t FLUSH:1;
    vuint32_t WULOCK:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_CTRL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_END_ADDR_union_tag { /* FIFO[i] channel [x] end address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_END_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_START_ADDR_union_tag { /* FIFO[i] channel [x] start address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_START_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_UPPER_WM_union_tag { /* FIFO[i] channel [x] upper watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_UPPER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_LOWER_WM_union_tag { /* FIFO[i] channel [x] lower watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_LOWER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_STATUS_union_tag { /* FIFO[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t FULL:1;
    vuint32_t LOW_WM:1;
    vuint32_t UP_WM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_FILL_LEVEL_union_tag { /* FIFO[i] channel [x] fill level register */
  vuint32_t R;
  struct {
    vuint32_t LEVEL:11;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_FILL_LEVEL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_WR_PTR_union_tag { /* FIFO[i] channel [x] write pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_WR_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_RD_PTR_union_tag { /* FIFO[i] channel [x] read pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_RD_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_IRQ_NOTIFY_union_tag { /* FIFO[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY:1;
    vuint32_t FIFO_FULL:1;
    vuint32_t FIFO_LWM:1;
    vuint32_t FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_IRQ_EN_union_tag { /* FIFO[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_IRQ_EN:1;
    vuint32_t FIFO_FULL_IRQ_EN:1;
    vuint32_t FIFO_LWM_IRQ_EN:1;
    vuint32_t FIFO_UWM_IRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_IRQ_FORCINT_union_tag { /* FIFO[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_FIFO_EMPTY:1;
    vuint32_t TRG_FIFO_FULL:1;
    vuint32_t TRG_FIFO_LWM:1;
    vuint32_t TRG_FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_IRQ_MODE_union_tag { /* FIFO[i] channel [x] interrupt mode control register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t DMA_HYSTERESIS:1;
    vuint32_t DMA_HYST_DIR:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_FIFO0_CH3_EIRQ_EN_union_tag { /* FIFO[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_EIRQ_EN:1;
    vuint32_t FIFO_FULL_EIRQ_EN:1;
    vuint32_t FIFO_LWM_EIRQ_EN:1;
    vuint32_t FIFO_UWM_EIRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH3_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_CTRL_union_tag { /* FIFO[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t RBM:1;
    vuint32_t RAP:1;
    vuint32_t FLUSH:1;
    vuint32_t WULOCK:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_CTRL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_END_ADDR_union_tag { /* FIFO[i] channel [x] end address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_END_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_START_ADDR_union_tag { /* FIFO[i] channel [x] start address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_START_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_UPPER_WM_union_tag { /* FIFO[i] channel [x] upper watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_UPPER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_LOWER_WM_union_tag { /* FIFO[i] channel [x] lower watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_LOWER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_STATUS_union_tag { /* FIFO[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t FULL:1;
    vuint32_t LOW_WM:1;
    vuint32_t UP_WM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_FILL_LEVEL_union_tag { /* FIFO[i] channel [x] fill level register */
  vuint32_t R;
  struct {
    vuint32_t LEVEL:11;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_FILL_LEVEL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_WR_PTR_union_tag { /* FIFO[i] channel [x] write pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_WR_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_RD_PTR_union_tag { /* FIFO[i] channel [x] read pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_RD_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_IRQ_NOTIFY_union_tag { /* FIFO[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY:1;
    vuint32_t FIFO_FULL:1;
    vuint32_t FIFO_LWM:1;
    vuint32_t FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_IRQ_EN_union_tag { /* FIFO[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_IRQ_EN:1;
    vuint32_t FIFO_FULL_IRQ_EN:1;
    vuint32_t FIFO_LWM_IRQ_EN:1;
    vuint32_t FIFO_UWM_IRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_IRQ_FORCINT_union_tag { /* FIFO[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_FIFO_EMPTY:1;
    vuint32_t TRG_FIFO_FULL:1;
    vuint32_t TRG_FIFO_LWM:1;
    vuint32_t TRG_FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_IRQ_MODE_union_tag { /* FIFO[i] channel [x] interrupt mode control register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t DMA_HYSTERESIS:1;
    vuint32_t DMA_HYST_DIR:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_FIFO0_CH4_EIRQ_EN_union_tag { /* FIFO[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_EIRQ_EN:1;
    vuint32_t FIFO_FULL_EIRQ_EN:1;
    vuint32_t FIFO_LWM_EIRQ_EN:1;
    vuint32_t FIFO_UWM_EIRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH4_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_CTRL_union_tag { /* FIFO[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t RBM:1;
    vuint32_t RAP:1;
    vuint32_t FLUSH:1;
    vuint32_t WULOCK:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_CTRL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_END_ADDR_union_tag { /* FIFO[i] channel [x] end address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_END_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_START_ADDR_union_tag { /* FIFO[i] channel [x] start address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_START_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_UPPER_WM_union_tag { /* FIFO[i] channel [x] upper watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_UPPER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_LOWER_WM_union_tag { /* FIFO[i] channel [x] lower watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_LOWER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_STATUS_union_tag { /* FIFO[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t FULL:1;
    vuint32_t LOW_WM:1;
    vuint32_t UP_WM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_FILL_LEVEL_union_tag { /* FIFO[i] channel [x] fill level register */
  vuint32_t R;
  struct {
    vuint32_t LEVEL:11;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_FILL_LEVEL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_WR_PTR_union_tag { /* FIFO[i] channel [x] write pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_WR_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_RD_PTR_union_tag { /* FIFO[i] channel [x] read pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_RD_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_IRQ_NOTIFY_union_tag { /* FIFO[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY:1;
    vuint32_t FIFO_FULL:1;
    vuint32_t FIFO_LWM:1;
    vuint32_t FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_IRQ_EN_union_tag { /* FIFO[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_IRQ_EN:1;
    vuint32_t FIFO_FULL_IRQ_EN:1;
    vuint32_t FIFO_LWM_IRQ_EN:1;
    vuint32_t FIFO_UWM_IRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_IRQ_FORCINT_union_tag { /* FIFO[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_FIFO_EMPTY:1;
    vuint32_t TRG_FIFO_FULL:1;
    vuint32_t TRG_FIFO_LWM:1;
    vuint32_t TRG_FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_IRQ_MODE_union_tag { /* FIFO[i] channel [x] interrupt mode control register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t DMA_HYSTERESIS:1;
    vuint32_t DMA_HYST_DIR:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_FIFO0_CH5_EIRQ_EN_union_tag { /* FIFO[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_EIRQ_EN:1;
    vuint32_t FIFO_FULL_EIRQ_EN:1;
    vuint32_t FIFO_LWM_EIRQ_EN:1;
    vuint32_t FIFO_UWM_EIRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH5_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_CTRL_union_tag { /* FIFO[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t RBM:1;
    vuint32_t RAP:1;
    vuint32_t FLUSH:1;
    vuint32_t WULOCK:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_CTRL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_END_ADDR_union_tag { /* FIFO[i] channel [x] end address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_END_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_START_ADDR_union_tag { /* FIFO[i] channel [x] start address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_START_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_UPPER_WM_union_tag { /* FIFO[i] channel [x] upper watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_UPPER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_LOWER_WM_union_tag { /* FIFO[i] channel [x] lower watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_LOWER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_STATUS_union_tag { /* FIFO[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t FULL:1;
    vuint32_t LOW_WM:1;
    vuint32_t UP_WM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_FILL_LEVEL_union_tag { /* FIFO[i] channel [x] fill level register */
  vuint32_t R;
  struct {
    vuint32_t LEVEL:11;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_FILL_LEVEL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_WR_PTR_union_tag { /* FIFO[i] channel [x] write pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_WR_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_RD_PTR_union_tag { /* FIFO[i] channel [x] read pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_RD_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_IRQ_NOTIFY_union_tag { /* FIFO[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY:1;
    vuint32_t FIFO_FULL:1;
    vuint32_t FIFO_LWM:1;
    vuint32_t FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_IRQ_EN_union_tag { /* FIFO[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_IRQ_EN:1;
    vuint32_t FIFO_FULL_IRQ_EN:1;
    vuint32_t FIFO_LWM_IRQ_EN:1;
    vuint32_t FIFO_UWM_IRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_IRQ_FORCINT_union_tag { /* FIFO[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_FIFO_EMPTY:1;
    vuint32_t TRG_FIFO_FULL:1;
    vuint32_t TRG_FIFO_LWM:1;
    vuint32_t TRG_FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_IRQ_MODE_union_tag { /* FIFO[i] channel [x] interrupt mode control register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t DMA_HYSTERESIS:1;
    vuint32_t DMA_HYST_DIR:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_FIFO0_CH6_EIRQ_EN_union_tag { /* FIFO[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_EIRQ_EN:1;
    vuint32_t FIFO_FULL_EIRQ_EN:1;
    vuint32_t FIFO_LWM_EIRQ_EN:1;
    vuint32_t FIFO_UWM_EIRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH6_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_CTRL_union_tag { /* FIFO[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t RBM:1;
    vuint32_t RAP:1;
    vuint32_t FLUSH:1;
    vuint32_t WULOCK:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_CTRL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_END_ADDR_union_tag { /* FIFO[i] channel [x] end address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_END_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_START_ADDR_union_tag { /* FIFO[i] channel [x] start address register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_START_ADDR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_UPPER_WM_union_tag { /* FIFO[i] channel [x] upper watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_UPPER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_LOWER_WM_union_tag { /* FIFO[i] channel [x] lower watermark register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_LOWER_WM_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_STATUS_union_tag { /* FIFO[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t FULL:1;
    vuint32_t LOW_WM:1;
    vuint32_t UP_WM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_FILL_LEVEL_union_tag { /* FIFO[i] channel [x] fill level register */
  vuint32_t R;
  struct {
    vuint32_t LEVEL:11;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_FILL_LEVEL_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_WR_PTR_union_tag { /* FIFO[i] channel [x] write pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_WR_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_RD_PTR_union_tag { /* FIFO[i] channel [x] read pointer register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_RD_PTR_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_IRQ_NOTIFY_union_tag { /* FIFO[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY:1;
    vuint32_t FIFO_FULL:1;
    vuint32_t FIFO_LWM:1;
    vuint32_t FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_IRQ_EN_union_tag { /* FIFO[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_IRQ_EN:1;
    vuint32_t FIFO_FULL_IRQ_EN:1;
    vuint32_t FIFO_LWM_IRQ_EN:1;
    vuint32_t FIFO_UWM_IRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_IRQ_FORCINT_union_tag { /* FIFO[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_FIFO_EMPTY:1;
    vuint32_t TRG_FIFO_FULL:1;
    vuint32_t TRG_FIFO_LWM:1;
    vuint32_t TRG_FIFO_UWM:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_IRQ_MODE_union_tag { /* FIFO[i] channel [x] interrupt mode control register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t DMA_HYSTERESIS:1;
    vuint32_t DMA_HYST_DIR:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_FIFO0_CH7_EIRQ_EN_union_tag { /* FIFO[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t FIFO_EMPTY_EIRQ_EN:1;
    vuint32_t FIFO_FULL_EIRQ_EN:1;
    vuint32_t FIFO_LWM_EIRQ_EN:1;
    vuint32_t FIFO_UWM_EIRQ_EN:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_FIFO0_CH7_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_SPE0_CTRL_STAT_union_tag { /* SPE[i] Control Status Register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t SIE0:1;
    vuint32_t SIE1:1;
    vuint32_t SIE2:1;
    vuint32_t TRIG_SEL:2;
    vuint32_t TIM_SEL:1;
    vuint32_t FSOM:1;
    vuint32_t SPE_PAT_PTR:3;
    vuint32_t _unused_11:1;
    vuint32_t AIP:3;
    vuint32_t ADIR:1;
    vuint32_t PIP:3;
    vuint32_t PDIR:1;
    vuint32_t NIP:3;
    vuint32_t ETRIG_SEL:1;
    vuint32_t FSOL:8;
  } B;
} GTM_gtm_cls0_SPE0_CTRL_STAT_tag;

typedef union GTM_gtm_cls0_SPE0_PAT_union_tag { /* SPE[i] Input Pattern Definition Register */
  vuint32_t R;
  struct {
    vuint32_t IP0_VAL:1;
    vuint32_t IP0_PAT:3;
    vuint32_t IP1_VAL:1;
    vuint32_t IP1_PAT:3;
    vuint32_t IP2_VAL:1;
    vuint32_t IP2_PAT:3;
    vuint32_t IP3_VAL:1;
    vuint32_t IP3_PAT:3;
    vuint32_t IP4_VAL:1;
    vuint32_t IP4_PAT:3;
    vuint32_t IP5_VAL:1;
    vuint32_t IP5_PAT:3;
    vuint32_t IP6_VAL:1;
    vuint32_t IP6_PAT:3;
    vuint32_t IP7_VAL:1;
    vuint32_t IP7_PAT:3;
  } B;
} GTM_gtm_cls0_SPE0_PAT_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_PAT0_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_PAT0_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_PAT1_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_PAT1_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_PAT2_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_PAT2_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_PAT3_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_PAT3_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_PAT4_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_PAT4_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_PAT5_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_PAT5_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_PAT6_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_PAT6_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_PAT7_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_PAT7_tag;

typedef union GTM_gtm_cls0_SPE0_OUT_CTRL_union_tag { /* SPE[i] Output Control Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_CTRL0:2;
    vuint32_t SPE_OUT_CTRL1:2;
    vuint32_t SPE_OUT_CTRL2:2;
    vuint32_t SPE_OUT_CTRL3:2;
    vuint32_t SPE_OUT_CTRL4:2;
    vuint32_t SPE_OUT_CTRL5:2;
    vuint32_t SPE_OUT_CTRL6:2;
    vuint32_t SPE_OUT_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_SPE0_OUT_CTRL_tag;

typedef union GTM_gtm_cls0_SPE0_IRQ_NOTIFY_union_tag { /* SPE[i] Interrupt Notification Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_NIPD:1;
    vuint32_t SPE_DCHG:1;
    vuint32_t SPE_PERR:1;
    vuint32_t SPE_BIS:1;
    vuint32_t SPE_RCMP:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_SPE0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_SPE0_IRQ_EN_union_tag { /* SPE[i] Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_NIPD_IRQ_EN:1;
    vuint32_t SPE_DCHG_IRQ_EN:1;
    vuint32_t SPE_PERR_IRQ_EN:1;
    vuint32_t SPE_BIS_IRQ_EN:1;
    vuint32_t SPE_RCMP_IRQ_EN:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_SPE0_IRQ_EN_tag;

typedef union GTM_gtm_cls0_SPE0_IRQ_FORCINT_union_tag { /* SPE[i] Interrupt Generation By Software */
  vuint32_t R;
  struct {
    vuint32_t TRG_SPE_NIPD:1;
    vuint32_t TRG_SPE_DCHG:1;
    vuint32_t TRG_SPE_PERR:1;
    vuint32_t TRG_SPE_BIS:1;
    vuint32_t TRG_SPE_RCMP:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_SPE0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_SPE0_IRQ_MODE_union_tag { /* SPE[i] Interrupt Mode Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_SPE0_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_SPE0_EIRQ_EN_union_tag { /* SPE[i] Error Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_NIPD_EIRQ_EN:1;
    vuint32_t SPE_DCHG_EIRQ_EN:1;
    vuint32_t SPE_PERR_EIRQ_EN:1;
    vuint32_t SPE_BIS_EIRQ_EN:1;
    vuint32_t SPE_RCMP_EIRQ_EN:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_SPE0_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_SPE0_REV_CNT_union_tag { /* SPE[i] Input Revolution Counter */
  vuint32_t R;
  struct {
    vuint32_t REV_CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_SPE0_REV_CNT_tag;

typedef union GTM_gtm_cls0_SPE0_REV_CMP_union_tag { /* SPE[i] Revolution Counter Compare Value */
  vuint32_t R;
  struct {
    vuint32_t REV_CMP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_SPE0_REV_CMP_tag;

typedef union GTM_gtm_cls0_SPE0_CTRL_STAT2_union_tag { /* SPE[i] Control Status Register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t SPE_PAT_PTR_BWD:3;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_SPE0_CTRL_STAT2_tag;

typedef union GTM_gtm_cls0_SPE0_CMD_union_tag { /* SPE[i] Command Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_CTRL_CMD:2;
    vuint32_t _unused_2:14;
    vuint32_t SPE_UPD_TRIG:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls0_SPE0_CMD_tag;

typedef union GTM_gtm_cls0_AXIM0_FREE_union_tag { /* AXIM[i] slot allocation status. */
  vuint32_t R;
  struct {
    vuint32_t FREE0:1;
    vuint32_t FREE1:1;
    vuint32_t FREE2:1;
    vuint32_t FREE3:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_AXIM0_FREE_tag;

typedef union GTM_gtm_cls0_AXIM0_REQUEST_union_tag { /* AXIM[i] slot request (allocation). */
  vuint32_t R;
  struct {
    vuint32_t REQ1HOT0:1;
    vuint32_t REQ1HOT1:1;
    vuint32_t REQ1HOT2:1;
    vuint32_t REQ1HOT3:1;
    vuint32_t _unused_4:20;
    vuint32_t REQID:8;
  } B;
} GTM_gtm_cls0_AXIM0_REQUEST_tag;

typedef union GTM_gtm_cls0_AXIM0_RELEASE_union_tag { /* AXIM[i] slot release (de-allocation). */
  vuint32_t R;
  struct {
    vuint32_t RELREQ0:1;
    vuint32_t RELREQ1:1;
    vuint32_t RELREQ2:1;
    vuint32_t RELREQ3:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls0_AXIM0_RELEASE_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT0_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT0_ADDR_LOW_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT0_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT0_DATA_LOW_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT0_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT0_CFG1_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT0_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT0_CFG2_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT0_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT0_STATUS_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT1_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT1_ADDR_LOW_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT1_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT1_DATA_LOW_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT1_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT1_CFG1_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT1_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT1_CFG2_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT1_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT1_STATUS_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT2_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT2_ADDR_LOW_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT2_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT2_DATA_LOW_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT2_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT2_CFG1_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT2_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT2_CFG2_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT2_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT2_STATUS_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT3_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT3_ADDR_LOW_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT3_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT3_DATA_LOW_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT3_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT3_CFG1_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT3_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT3_CFG2_tag;

typedef union GTM_gtm_cls0_AXIM0_SLOT3_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls0_AXIM0_SLOT3_STATUS_tag;

typedef union GTM_gtm_cls0_FIFO0_MEMORY_union_tag { /* FIFO data memory */
  vuint32_t R;
  struct {
    vuint32_t DATA:29;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_FIFO0_MEMORY_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_0_union_tag { /* Control Register 0 */
  vuint32_t R;
  struct {
    vuint32_t MLT:10;
    vuint32_t IFP:1;
    vuint32_t SNU:5;
    vuint32_t TNU:9;
    vuint32_t AMS:1;
    vuint32_t AMT:1;
    vuint32_t IDS:1;
    vuint32_t IDT:1;
    vuint32_t SEN:1;
    vuint32_t TEN:1;
    vuint32_t RMO:1;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_0_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_1_union_tag { /* Control Register 1 */
  vuint32_t R;
  struct {
    vuint32_t DMO:1;
    vuint32_t DEN:1;
    vuint32_t IDDS:1;
    vuint32_t COA:1;
    vuint32_t PIT:1;
    vuint32_t SGE1:1;
    vuint32_t DLM1:1;
    vuint32_t PCM1:1;
    vuint32_t SGE2:1;
    vuint32_t DLM2:1;
    vuint32_t PCM2:1;
    vuint32_t SYN_NS:5;
    vuint32_t SYN_NT:6;
    vuint32_t LCD:1;
    vuint32_t SWR:1;
    vuint32_t SYSF:1;
    vuint32_t TS0_HRS:1;
    vuint32_t TS0_HRT:1;
    vuint32_t SMC:1;
    vuint32_t SSL:2;
    vuint32_t TSL:2;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_1_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_2_union_tag { /* Action Enable Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t AEN0:1;
    vuint32_t AEN1:1;
    vuint32_t AEN2:1;
    vuint32_t AEN3:1;
    vuint32_t AEN4:1;
    vuint32_t AEN5:1;
    vuint32_t AEN6:1;
    vuint32_t AEN7:1;
    vuint32_t WAD0:1;
    vuint32_t WAD1:1;
    vuint32_t WAD2:1;
    vuint32_t WAD3:1;
    vuint32_t WAD4:1;
    vuint32_t WAD5:1;
    vuint32_t WAD6:1;
    vuint32_t WAD7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_2_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_3_union_tag { /* Action Enable Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t AEN8:1;
    vuint32_t AEN9:1;
    vuint32_t AEN10:1;
    vuint32_t AEN11:1;
    vuint32_t AEN12:1;
    vuint32_t AEN13:1;
    vuint32_t AEN14:1;
    vuint32_t AEN15:1;
    vuint32_t WAD8:1;
    vuint32_t WAD9:1;
    vuint32_t WAD10:1;
    vuint32_t WAD11:1;
    vuint32_t WAD12:1;
    vuint32_t WAD13:1;
    vuint32_t WAD14:1;
    vuint32_t WAD15:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_3_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_4_union_tag { /* Action Enable Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t AEN16:1;
    vuint32_t AEN17:1;
    vuint32_t AEN18:1;
    vuint32_t AEN19:1;
    vuint32_t AEN20:1;
    vuint32_t AEN21:1;
    vuint32_t AEN22:1;
    vuint32_t AEN23:1;
    vuint32_t WAD16:1;
    vuint32_t WAD17:1;
    vuint32_t WAD18:1;
    vuint32_t WAD19:1;
    vuint32_t WAD20:1;
    vuint32_t WAD21:1;
    vuint32_t WAD22:1;
    vuint32_t WAD23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_4_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_5_union_tag { /* Action Enable Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t AEN24:1;
    vuint32_t AEN25:1;
    vuint32_t AEN26:1;
    vuint32_t AEN27:1;
    vuint32_t AEN28:1;
    vuint32_t AEN29:1;
    vuint32_t AEN30:1;
    vuint32_t AEN31:1;
    vuint32_t WAD24:1;
    vuint32_t WAD25:1;
    vuint32_t WAD26:1;
    vuint32_t WAD27:1;
    vuint32_t WAD28:1;
    vuint32_t WAD29:1;
    vuint32_t WAD30:1;
    vuint32_t WAD31:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_5_tag;

typedef union GTM_gtm_cls0_DPLL_ACT_STA_union_tag { /* Action Status Register including Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t ACT_N0:1;
    vuint32_t ACT_N1:1;
    vuint32_t ACT_N2:1;
    vuint32_t ACT_N3:1;
    vuint32_t ACT_N4:1;
    vuint32_t ACT_N5:1;
    vuint32_t ACT_N6:1;
    vuint32_t ACT_N7:1;
    vuint32_t ACT_N8:1;
    vuint32_t ACT_N9:1;
    vuint32_t ACT_N10:1;
    vuint32_t ACT_N11:1;
    vuint32_t ACT_N12:1;
    vuint32_t ACT_N13:1;
    vuint32_t ACT_N14:1;
    vuint32_t ACT_N15:1;
    vuint32_t ACT_N16:1;
    vuint32_t ACT_N17:1;
    vuint32_t ACT_N18:1;
    vuint32_t ACT_N19:1;
    vuint32_t ACT_N20:1;
    vuint32_t ACT_N21:1;
    vuint32_t ACT_N22:1;
    vuint32_t ACT_N23:1;
    vuint32_t ACT_N24:1;
    vuint32_t ACT_N25:1;
    vuint32_t ACT_N26:1;
    vuint32_t ACT_N27:1;
    vuint32_t ACT_N28:1;
    vuint32_t ACT_N29:1;
    vuint32_t ACT_N30:1;
    vuint32_t ACT_N31:1;
  } B;
} GTM_gtm_cls0_DPLL_ACT_STA_tag;

typedef union GTM_gtm_cls0_DPLL_OSW_union_tag { /* Offset and Switch old/new Address Register */
  vuint32_t R;
  struct {
    vuint32_t SWON_S:1;
    vuint32_t SWON_T:1;
    vuint32_t _unused_2:6;
    vuint32_t OSS:2;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_DPLL_OSW_tag;

typedef union GTM_gtm_cls0_DPLL_AOSV_2_union_tag { /* Address Offset Register of RAM 2 Regions */
  vuint32_t R;
  struct {
    vuint32_t AOSV_2A:8;
    vuint32_t AOSV_2B:8;
    vuint32_t AOSV_2C:8;
    vuint32_t AOSV_2D:8;
  } B;
} GTM_gtm_cls0_DPLL_AOSV_2_tag;

typedef union GTM_gtm_cls0_DPLL_APT_union_tag { /* Actual RAM Pointer Address for TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t WAPT:1;
    vuint32_t APT:10;
    vuint32_t _unused_12:1;
    vuint32_t WAPT_2B:1;
    vuint32_t APT_2B:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_APT_tag;

typedef union GTM_gtm_cls0_DPLL_APS_union_tag { /* Actual RAM Pointer Address for STATE */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t WAPS:1;
    vuint32_t APS:6;
    vuint32_t _unused_8:5;
    vuint32_t WAPS_1C2:1;
    vuint32_t APS_1C2:6;
    vuint32_t _unused_20:12;
  } B;
} GTM_gtm_cls0_DPLL_APS_tag;

typedef union GTM_gtm_cls0_DPLL_APT_2C_union_tag { /* Actual RAM Pointer Address for Region 2c */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t APT_2C:10;
    vuint32_t _unused_12:20;
  } B;
} GTM_gtm_cls0_DPLL_APT_2C_tag;

typedef union GTM_gtm_cls0_DPLL_APS_1C3_union_tag { /* Actual RAM Pointer Address for RAM region 1c3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t APS_1C3:6;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_DPLL_APS_1C3_tag;

typedef union GTM_gtm_cls0_DPLL_NUTC_union_tag { /* Number of Recent TRIGGER Events used for Calculations */
  vuint32_t R;
  struct {
    vuint32_t NUTE:10;
    vuint32_t FST:1;
    vuint32_t _unused_11:2;
    vuint32_t SYN_T:3;
    vuint32_t SYN_T_OLD:3;
    vuint32_t VTN:6;
    vuint32_t _unused_25:4;
    vuint32_t WNUT:1;
    vuint32_t WSYN:1;
    vuint32_t WVTN:1;
  } B;
} GTM_gtm_cls0_DPLL_NUTC_tag;

typedef union GTM_gtm_cls0_DPLL_NUSC_union_tag { /* Number of Recent STATE Events used for Calculations */
  vuint32_t R;
  struct {
    vuint32_t NUSE:6;
    vuint32_t FSS:1;
    vuint32_t SYN_S:6;
    vuint32_t SYN_S_OLD:6;
    vuint32_t VSN:6;
    vuint32_t _unused_25:4;
    vuint32_t WNUS:1;
    vuint32_t WSYN:1;
    vuint32_t WVSN:1;
  } B;
} GTM_gtm_cls0_DPLL_NUSC_tag;

typedef union GTM_gtm_cls0_DPLL_NTI_CNT_union_tag { /* Number of Active TRIGGER Events to Interrupt */
  vuint32_t R;
  struct {
    vuint32_t NTI_CNT:10;
    vuint32_t _unused_10:22;
  } B;
} GTM_gtm_cls0_DPLL_NTI_CNT_tag;

typedef union GTM_gtm_cls0_DPLL_IRQ_NOTIFY_union_tag { /* Interrupt Register */
  vuint32_t R;
  struct {
    vuint32_t PDI:1;
    vuint32_t PEI:1;
    vuint32_t TINI:1;
    vuint32_t TAXI:1;
    vuint32_t SISI:1;
    vuint32_t TISI:1;
    vuint32_t MSI:1;
    vuint32_t MTI:1;
    vuint32_t SASI:1;
    vuint32_t TASI:1;
    vuint32_t PWI:1;
    vuint32_t W2I:1;
    vuint32_t W1I:1;
    vuint32_t GL1I:1;
    vuint32_t LL1I:1;
    vuint32_t EI:1;
    vuint32_t GL2I:1;
    vuint32_t LL2I:1;
    vuint32_t TE0I:1;
    vuint32_t TE1I:1;
    vuint32_t TE2I:1;
    vuint32_t TE3I:1;
    vuint32_t TE4I:1;
    vuint32_t CDTI:1;
    vuint32_t CDSI:1;
    vuint32_t TORI:1;
    vuint32_t SORI:1;
    vuint32_t DCGI:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_DPLL_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls0_DPLL_IRQ_EN_union_tag { /* Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t PDI_IRQ_EN:1;
    vuint32_t PEI_IRQ_EN:1;
    vuint32_t TINI_IRQ_EN:1;
    vuint32_t TAXI_IRQ_EN:1;
    vuint32_t SISI_IRQ_EN:1;
    vuint32_t TISI_IRQ_EN:1;
    vuint32_t MSI_IRQ_EN:1;
    vuint32_t MTI_IRQ_EN:1;
    vuint32_t SASI_IRQ_EN:1;
    vuint32_t TASI_IRQ_EN:1;
    vuint32_t PWI_IRQ_EN:1;
    vuint32_t W2I_IRQ_EN:1;
    vuint32_t W1I_IRQ_EN:1;
    vuint32_t GL1I_IRQ_EN:1;
    vuint32_t LL1I_IRQ_EN:1;
    vuint32_t EI_IRQ_EN:1;
    vuint32_t GL2I_IRQ_EN:1;
    vuint32_t LL2I_IRQ_EN:1;
    vuint32_t TE0I_IRQ_EN:1;
    vuint32_t TE1I_IRQ_EN:1;
    vuint32_t TE2I_IRQ_EN:1;
    vuint32_t TE3I_IRQ_EN:1;
    vuint32_t TE4I_IRQ_EN:1;
    vuint32_t CDTI_IRQ_EN:1;
    vuint32_t CDSI_IRQ_EN:1;
    vuint32_t TORI_IRQ_EN:1;
    vuint32_t SORI_IRQ_EN:1;
    vuint32_t DCGI_IRQ_EN:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_DPLL_IRQ_EN_tag;

typedef union GTM_gtm_cls0_DPLL_IRQ_FORCINT_union_tag { /* Force Interrupt Register */
  vuint32_t R;
  struct {
    vuint32_t TRG_PDI:1;
    vuint32_t TRG_PEI:1;
    vuint32_t TRG_TINI:1;
    vuint32_t TRG_TAXI:1;
    vuint32_t TRG_SISI:1;
    vuint32_t TRG_TISI:1;
    vuint32_t TRG_MSI:1;
    vuint32_t TRG_MTI:1;
    vuint32_t TRG_SASI:1;
    vuint32_t TRG_TASI:1;
    vuint32_t TRG_PWI:1;
    vuint32_t TRG_W2I:1;
    vuint32_t TRG_W1I:1;
    vuint32_t TRG_GL1I:1;
    vuint32_t TRG_LL1I:1;
    vuint32_t TRG_EI:1;
    vuint32_t TRG_GL2I:1;
    vuint32_t TRG_LL2I:1;
    vuint32_t TRG_TE0I:1;
    vuint32_t TRG_TE1I:1;
    vuint32_t TRG_TE2I:1;
    vuint32_t TRG_TE3I:1;
    vuint32_t TRG_TE4I:1;
    vuint32_t TRG_CDTI:1;
    vuint32_t TRG_CDSI:1;
    vuint32_t TRG_TORI:1;
    vuint32_t TRG_SORI:1;
    vuint32_t TRG_DCGI:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_DPLL_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls0_DPLL_IRQ_MODE_union_tag { /* Interrupt Request Mode */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls0_DPLL_IRQ_MODE_tag;

typedef union GTM_gtm_cls0_DPLL_EIRQ_EN_union_tag { /* Error Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t PDI_EIRQ_EN:1;
    vuint32_t PEI_EIRQ_EN:1;
    vuint32_t TINI_EIRQ_EN:1;
    vuint32_t TAXI_EIRQ_EN:1;
    vuint32_t SISI_EIRQ_EN:1;
    vuint32_t TISI_EIRQ_EN:1;
    vuint32_t MSI_EIRQ_EN:1;
    vuint32_t MTI_EIRQ_EN:1;
    vuint32_t SASI_EIRQ_EN:1;
    vuint32_t TASI_EIRQ_EN:1;
    vuint32_t PWI_EIRQ_EN:1;
    vuint32_t W2I_EIRQ_EN:1;
    vuint32_t W1I_EIRQ_EN:1;
    vuint32_t GL1I_EIRQ_EN:1;
    vuint32_t LL1I_EIRQ_EN:1;
    vuint32_t EI_EIRQ_EN:1;
    vuint32_t GL2I_EIRQ_EN:1;
    vuint32_t LL2I_EIRQ_EN:1;
    vuint32_t TE0I_EIRQ_EN:1;
    vuint32_t TE1I_EIRQ_EN:1;
    vuint32_t TE2I_EIRQ_EN:1;
    vuint32_t TE3I_EIRQ_EN:1;
    vuint32_t TE4I_EIRQ_EN:1;
    vuint32_t CDTI_EIRQ_EN:1;
    vuint32_t CDSI_EIRQ_EN:1;
    vuint32_t TORI_EIRQ_EN:1;
    vuint32_t SORI_EIRQ_EN:1;
    vuint32_t DCGI_EIRQ_EN:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls0_DPLL_EIRQ_EN_tag;

typedef union GTM_gtm_cls0_DPLL_INC_CNT1_union_tag { /* Counter Value of Sent SUB_INC1 Pulses */
  vuint32_t R;
  struct {
    vuint32_t INC_CNT1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_INC_CNT1_tag;

typedef union GTM_gtm_cls0_DPLL_INC_CNT2_union_tag { /* Counter Value of sent SUB_INC2 values (for DPLL_CTRL_1.SMC=1 and DPLL_CTRL_0.RMO=1) */
  vuint32_t R;
  struct {
    vuint32_t INC_CNT2:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_INC_CNT2_tag;

typedef union GTM_gtm_cls0_DPLL_APT_SYNC_union_tag { /* TRIGGER Time Stamp Field Offset at Synchronization Time */
  vuint32_t R;
  struct {
    vuint32_t APT_2B_EXT:6;
    vuint32_t APT_2B_STATUS:1;
    vuint32_t _unused_7:7;
    vuint32_t APT_2B_OLD:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_APT_SYNC_tag;

typedef union GTM_gtm_cls0_DPLL_APS_SYNC_union_tag { /* STATE Time Stamp Field Offset at Synchronization Time */
  vuint32_t R;
  struct {
    vuint32_t APS_1C2_EXT:6;
    vuint32_t APS_1C2_STATUS:1;
    vuint32_t _unused_7:7;
    vuint32_t APS_1C2_OLD:6;
    vuint32_t _unused_20:12;
  } B;
} GTM_gtm_cls0_DPLL_APS_SYNC_tag;

typedef union GTM_gtm_cls0_DPLL_TBU_TS0_T_union_tag { /* Time Stamp Value for the last active TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t DPLL_TBU_TS0_T:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TBU_TS0_T_tag;

typedef union GTM_gtm_cls0_DPLL_TBU_TS0_S_union_tag { /* Time Stamp Value for the last active STATE */
  vuint32_t R;
  struct {
    vuint32_t DPLL_TBU_TS0_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TBU_TS0_S_tag;

typedef union GTM_gtm_cls0_DPLL_ADD_IN_LD1_union_tag { /* ADD_IN Value in Direct Load Mode for TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t ADD_IN_LD1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADD_IN_LD1_tag;

typedef union GTM_gtm_cls0_DPLL_ADD_IN_LD2_union_tag { /* ADD_IN Value in Direct Load Mode for STATE */
  vuint32_t R;
  struct {
    vuint32_t ADD_IN_LD2:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADD_IN_LD2_tag;

typedef union GTM_gtm_cls0_DPLL_STATUS_union_tag { /* Status Register */
  vuint32_t R;
  struct {
    vuint32_t FPCE:1;
    vuint32_t CSO:1;
    vuint32_t _unused_2:1;
    vuint32_t CTO:1;
    vuint32_t CRO:1;
    vuint32_t RCS:1;
    vuint32_t RCT:1;
    vuint32_t PSE:1;
    vuint32_t SOR:1;
    vuint32_t MS:1;
    vuint32_t TOR:1;
    vuint32_t MT:1;
    vuint32_t RAM2_ERR:1;
    vuint32_t _unused_13:2;
    vuint32_t LOW_RES:1;
    vuint32_t CSVS:1;
    vuint32_t CSVT:1;
    vuint32_t CAIP2:1;
    vuint32_t CAIP1:1;
    vuint32_t ISN:1;
    vuint32_t ITN:1;
    vuint32_t BWD2:1;
    vuint32_t BWD1:1;
    vuint32_t _unused_24:1;
    vuint32_t LOCK2:1;
    vuint32_t SYS:1;
    vuint32_t SYT:1;
    vuint32_t FSD:1;
    vuint32_t FTD:1;
    vuint32_t LOCK1:1;
    vuint32_t ERR:1;
  } B;
} GTM_gtm_cls0_DPLL_STATUS_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_0_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_0_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_1_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_1_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_2_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_2_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_3_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_3_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_4_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_4_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_5_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_5_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_6_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_6_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_7_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_7_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_8_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_8_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_9_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_9_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_10_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_10_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_11_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_11_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_12_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_12_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_13_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_13_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_14_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_14_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_15_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_15_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_16_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_16_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_17_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_17_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_18_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_18_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_19_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_19_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_20_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_20_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_21_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_21_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_22_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_22_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_23_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_23_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_24_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_24_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_25_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_25_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_26_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_26_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_27_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_27_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_28_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_28_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_29_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_29_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_30_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_30_tag;

typedef union GTM_gtm_cls0_DPLL_ID_PMTR_31_union_tag { /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  vuint32_t R;
  struct {
    vuint32_t ID_PMTR:9;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_ID_PMTR_31_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_0_SHADOW_TRIGGER_union_tag { /* Shadow Register of DPLL_CTRL_0 controlled by an active TRIGGER Slope */
  vuint32_t R;
  struct {
    vuint32_t MLT:10;
    vuint32_t IFP:1;
    vuint32_t _unused_11:15;
    vuint32_t AMT:1;
    vuint32_t _unused_27:1;
    vuint32_t IDT:1;
    vuint32_t _unused_29:2;
    vuint32_t RMO:1;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_0_SHADOW_TRIGGER_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_0_SHADOW_STATE_union_tag { /* Shadow Register of DPLL_CTRL_0 controlled by an active STATE Slope */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:10;
    vuint32_t IFP:1;
    vuint32_t _unused_11:14;
    vuint32_t AMS:1;
    vuint32_t _unused_26:1;
    vuint32_t IDS:1;
    vuint32_t _unused_28:3;
    vuint32_t RMO:1;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_0_SHADOW_STATE_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_1_SHADOW_TRIGGER_union_tag { /* Shadow Register of DPLL_CTRL_1 controlled by an active TRIGGER Slope */
  vuint32_t R;
  struct {
    vuint32_t DMO:1;
    vuint32_t _unused_1:2;
    vuint32_t COA:1;
    vuint32_t PIT:1;
    vuint32_t SGE1:1;
    vuint32_t DLM1:1;
    vuint32_t PCM1:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_1_SHADOW_TRIGGER_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_1_SHADOW_STATE_union_tag { /* DPLL Shadow Register of DPLL_CTRL_1 controlled by an active STATE Slope */
  vuint32_t R;
  struct {
    vuint32_t DMO:1;
    vuint32_t _unused_1:2;
    vuint32_t COA:1;
    vuint32_t _unused_4:1;
    vuint32_t SGE1:1;
    vuint32_t DLM1:1;
    vuint32_t PCM1:1;
    vuint32_t SGE2:1;
    vuint32_t DLM2:1;
    vuint32_t PCM2:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_1_SHADOW_STATE_tag;

typedef union GTM_gtm_cls0_DPLL_RAM_INI_union_tag { /* Register to control the RAM Initialization */
  vuint32_t R;
  struct {
    vuint32_t INIT_1A:1;
    vuint32_t INIT_1BC:1;
    vuint32_t INIT_2:1;
    vuint32_t _unused_3:1;
    vuint32_t INIT_RAM:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls0_DPLL_RAM_INI_tag;

typedef union GTM_gtm_cls0_DPLL_PSA0_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA0_tag;

typedef union GTM_gtm_cls0_DPLL_PSA1_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA1_tag;

typedef union GTM_gtm_cls0_DPLL_PSA2_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA2_tag;

typedef union GTM_gtm_cls0_DPLL_PSA3_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA3_tag;

typedef union GTM_gtm_cls0_DPLL_PSA4_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA4_tag;

typedef union GTM_gtm_cls0_DPLL_PSA5_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA5_tag;

typedef union GTM_gtm_cls0_DPLL_PSA6_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA6_tag;

typedef union GTM_gtm_cls0_DPLL_PSA7_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA7_tag;

typedef union GTM_gtm_cls0_DPLL_PSA8_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA8_tag;

typedef union GTM_gtm_cls0_DPLL_PSA9_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA9_tag;

typedef union GTM_gtm_cls0_DPLL_PSA10_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA10_tag;

typedef union GTM_gtm_cls0_DPLL_PSA11_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA11_tag;

typedef union GTM_gtm_cls0_DPLL_PSA12_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA12_tag;

typedef union GTM_gtm_cls0_DPLL_PSA13_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA13_tag;

typedef union GTM_gtm_cls0_DPLL_PSA14_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA14_tag;

typedef union GTM_gtm_cls0_DPLL_PSA15_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA15_tag;

typedef union GTM_gtm_cls0_DPLL_PSA16_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA16_tag;

typedef union GTM_gtm_cls0_DPLL_PSA17_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA17_tag;

typedef union GTM_gtm_cls0_DPLL_PSA18_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA18_tag;

typedef union GTM_gtm_cls0_DPLL_PSA19_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA19_tag;

typedef union GTM_gtm_cls0_DPLL_PSA20_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA20_tag;

typedef union GTM_gtm_cls0_DPLL_PSA21_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA21_tag;

typedef union GTM_gtm_cls0_DPLL_PSA22_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA22_tag;

typedef union GTM_gtm_cls0_DPLL_PSA23_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA23_tag;

typedef union GTM_gtm_cls0_DPLL_PSA24_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA24_tag;

typedef union GTM_gtm_cls0_DPLL_PSA25_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA25_tag;

typedef union GTM_gtm_cls0_DPLL_PSA26_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA26_tag;

typedef union GTM_gtm_cls0_DPLL_PSA27_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA27_tag;

typedef union GTM_gtm_cls0_DPLL_PSA28_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA28_tag;

typedef union GTM_gtm_cls0_DPLL_PSA29_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA29_tag;

typedef union GTM_gtm_cls0_DPLL_PSA30_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA30_tag;

typedef union GTM_gtm_cls0_DPLL_PSA31_union_tag { /* Position Request for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSA31_tag;

typedef union GTM_gtm_cls0_DPLL_DLA0_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA0_tag;

typedef union GTM_gtm_cls0_DPLL_DLA1_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA1_tag;

typedef union GTM_gtm_cls0_DPLL_DLA2_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA2_tag;

typedef union GTM_gtm_cls0_DPLL_DLA3_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA3_tag;

typedef union GTM_gtm_cls0_DPLL_DLA4_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA4_tag;

typedef union GTM_gtm_cls0_DPLL_DLA5_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA5_tag;

typedef union GTM_gtm_cls0_DPLL_DLA6_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA6_tag;

typedef union GTM_gtm_cls0_DPLL_DLA7_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA7_tag;

typedef union GTM_gtm_cls0_DPLL_DLA8_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA8_tag;

typedef union GTM_gtm_cls0_DPLL_DLA9_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA9_tag;

typedef union GTM_gtm_cls0_DPLL_DLA10_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA10_tag;

typedef union GTM_gtm_cls0_DPLL_DLA11_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA11_tag;

typedef union GTM_gtm_cls0_DPLL_DLA12_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA12_tag;

typedef union GTM_gtm_cls0_DPLL_DLA13_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA13_tag;

typedef union GTM_gtm_cls0_DPLL_DLA14_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA14_tag;

typedef union GTM_gtm_cls0_DPLL_DLA15_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA15_tag;

typedef union GTM_gtm_cls0_DPLL_DLA16_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA16_tag;

typedef union GTM_gtm_cls0_DPLL_DLA17_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA17_tag;

typedef union GTM_gtm_cls0_DPLL_DLA18_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA18_tag;

typedef union GTM_gtm_cls0_DPLL_DLA19_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA19_tag;

typedef union GTM_gtm_cls0_DPLL_DLA20_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA20_tag;

typedef union GTM_gtm_cls0_DPLL_DLA21_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA21_tag;

typedef union GTM_gtm_cls0_DPLL_DLA22_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA22_tag;

typedef union GTM_gtm_cls0_DPLL_DLA23_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA23_tag;

typedef union GTM_gtm_cls0_DPLL_DLA24_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA24_tag;

typedef union GTM_gtm_cls0_DPLL_DLA25_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA25_tag;

typedef union GTM_gtm_cls0_DPLL_DLA26_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA26_tag;

typedef union GTM_gtm_cls0_DPLL_DLA27_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA27_tag;

typedef union GTM_gtm_cls0_DPLL_DLA28_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA28_tag;

typedef union GTM_gtm_cls0_DPLL_DLA29_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA29_tag;

typedef union GTM_gtm_cls0_DPLL_DLA30_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA30_tag;

typedef union GTM_gtm_cls0_DPLL_DLA31_union_tag { /* Time to React for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DLA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DLA31_tag;

typedef union GTM_gtm_cls0_DPLL_NA0_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA0_tag;

typedef union GTM_gtm_cls0_DPLL_NA1_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA1_tag;

typedef union GTM_gtm_cls0_DPLL_NA2_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA2_tag;

typedef union GTM_gtm_cls0_DPLL_NA3_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA3_tag;

typedef union GTM_gtm_cls0_DPLL_NA4_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA4_tag;

typedef union GTM_gtm_cls0_DPLL_NA5_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA5_tag;

typedef union GTM_gtm_cls0_DPLL_NA6_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA6_tag;

typedef union GTM_gtm_cls0_DPLL_NA7_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA7_tag;

typedef union GTM_gtm_cls0_DPLL_NA8_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA8_tag;

typedef union GTM_gtm_cls0_DPLL_NA9_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA9_tag;

typedef union GTM_gtm_cls0_DPLL_NA10_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA10_tag;

typedef union GTM_gtm_cls0_DPLL_NA11_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA11_tag;

typedef union GTM_gtm_cls0_DPLL_NA12_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA12_tag;

typedef union GTM_gtm_cls0_DPLL_NA13_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA13_tag;

typedef union GTM_gtm_cls0_DPLL_NA14_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA14_tag;

typedef union GTM_gtm_cls0_DPLL_NA15_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA15_tag;

typedef union GTM_gtm_cls0_DPLL_NA16_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA16_tag;

typedef union GTM_gtm_cls0_DPLL_NA17_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA17_tag;

typedef union GTM_gtm_cls0_DPLL_NA18_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA18_tag;

typedef union GTM_gtm_cls0_DPLL_NA19_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA19_tag;

typedef union GTM_gtm_cls0_DPLL_NA20_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA20_tag;

typedef union GTM_gtm_cls0_DPLL_NA21_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA21_tag;

typedef union GTM_gtm_cls0_DPLL_NA22_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA22_tag;

typedef union GTM_gtm_cls0_DPLL_NA23_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA23_tag;

typedef union GTM_gtm_cls0_DPLL_NA24_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA24_tag;

typedef union GTM_gtm_cls0_DPLL_NA25_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA25_tag;

typedef union GTM_gtm_cls0_DPLL_NA26_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA26_tag;

typedef union GTM_gtm_cls0_DPLL_NA27_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA27_tag;

typedef union GTM_gtm_cls0_DPLL_NA28_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA28_tag;

typedef union GTM_gtm_cls0_DPLL_NA29_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA29_tag;

typedef union GTM_gtm_cls0_DPLL_NA30_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA30_tag;

typedef union GTM_gtm_cls0_DPLL_NA31_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:10;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NA31_tag;

typedef union GTM_gtm_cls0_DPLL_DTA0_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA0_tag;

typedef union GTM_gtm_cls0_DPLL_DTA1_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA1_tag;

typedef union GTM_gtm_cls0_DPLL_DTA2_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA2_tag;

typedef union GTM_gtm_cls0_DPLL_DTA3_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA3_tag;

typedef union GTM_gtm_cls0_DPLL_DTA4_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA4_tag;

typedef union GTM_gtm_cls0_DPLL_DTA5_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA5_tag;

typedef union GTM_gtm_cls0_DPLL_DTA6_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA6_tag;

typedef union GTM_gtm_cls0_DPLL_DTA7_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA7_tag;

typedef union GTM_gtm_cls0_DPLL_DTA8_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA8_tag;

typedef union GTM_gtm_cls0_DPLL_DTA9_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA9_tag;

typedef union GTM_gtm_cls0_DPLL_DTA10_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA10_tag;

typedef union GTM_gtm_cls0_DPLL_DTA11_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA11_tag;

typedef union GTM_gtm_cls0_DPLL_DTA12_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA12_tag;

typedef union GTM_gtm_cls0_DPLL_DTA13_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA13_tag;

typedef union GTM_gtm_cls0_DPLL_DTA14_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA14_tag;

typedef union GTM_gtm_cls0_DPLL_DTA15_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA15_tag;

typedef union GTM_gtm_cls0_DPLL_DTA16_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA16_tag;

typedef union GTM_gtm_cls0_DPLL_DTA17_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA17_tag;

typedef union GTM_gtm_cls0_DPLL_DTA18_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA18_tag;

typedef union GTM_gtm_cls0_DPLL_DTA19_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA19_tag;

typedef union GTM_gtm_cls0_DPLL_DTA20_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA20_tag;

typedef union GTM_gtm_cls0_DPLL_DTA21_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA21_tag;

typedef union GTM_gtm_cls0_DPLL_DTA22_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA22_tag;

typedef union GTM_gtm_cls0_DPLL_DTA23_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA23_tag;

typedef union GTM_gtm_cls0_DPLL_DTA24_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA24_tag;

typedef union GTM_gtm_cls0_DPLL_DTA25_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA25_tag;

typedef union GTM_gtm_cls0_DPLL_DTA26_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA26_tag;

typedef union GTM_gtm_cls0_DPLL_DTA27_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA27_tag;

typedef union GTM_gtm_cls0_DPLL_DTA28_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA28_tag;

typedef union GTM_gtm_cls0_DPLL_DTA29_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA29_tag;

typedef union GTM_gtm_cls0_DPLL_DTA30_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA30_tag;

typedef union GTM_gtm_cls0_DPLL_DTA31_union_tag { /* Calculated Relative Time to Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DTA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DTA31_tag;

typedef union GTM_gtm_cls0_DPLL_TS_T_union_tag { /* Actual TRIGGER Time Stamp Value */
  vuint32_t R;
  struct {
    vuint32_t TRIGGER_TS:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TS_T_tag;

typedef union GTM_gtm_cls0_DPLL_TS_T_OLD_union_tag { /* Previous TRIGGER Time Stamp Value */
  vuint32_t R;
  struct {
    vuint32_t TRIGGER_TS_OLD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TS_T_OLD_tag;

typedef union GTM_gtm_cls0_DPLL_FTV_T_union_tag { /* Actual TRIGGER Filter value */
  vuint32_t R;
  struct {
    vuint32_t TRIGGER_FT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_FTV_T_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_0_union_tag { /* DPLL RAM1B reserved data */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_0_tag;

typedef union GTM_gtm_cls0_DPLL_TS_S_union_tag { /* Actual STATE Time Stamp Register */
  vuint32_t R;
  struct {
    vuint32_t STATE_TS:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TS_S_tag;

typedef union GTM_gtm_cls0_DPLL_TS_S_OLD_union_tag { /* Previous STATE Time Stamp Register */
  vuint32_t R;
  struct {
    vuint32_t STATE_TS_OLD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TS_S_OLD_tag;

typedef union GTM_gtm_cls0_DPLL_FTV_S_union_tag { /* Actual STATE Filter Value */
  vuint32_t R;
  struct {
    vuint32_t STATE_FT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_FTV_S_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_1_union_tag { /* DPLL RAM1B reserved data */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_1_tag;

typedef union GTM_gtm_cls0_DPLL_THMI_union_tag { /* TRIGGER Hold Time Min. Value */
  vuint32_t R;
  struct {
    vuint32_t THMI:16;
    vuint32_t NOT_USED:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_THMI_tag;

typedef union GTM_gtm_cls0_DPLL_THMA_union_tag { /* TRIGGER Hold Time Max. Value */
  vuint32_t R;
  struct {
    vuint32_t THMA:16;
    vuint32_t NOT_USED:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_THMA_tag;

typedef union GTM_gtm_cls0_DPLL_THVAL_union_tag { /* Measured TRIGGER Hold Time Value */
  vuint32_t R;
  struct {
    vuint32_t THVAL:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_THVAL_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_2_union_tag { /* DPLL RAM1B reserved data */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_2_tag;

typedef union GTM_gtm_cls0_DPLL_TOV_union_tag { /* Time Out Value of Active TRIGGER Slope (for missing TRIGGER generation) */
  vuint32_t R;
  struct {
    vuint32_t TOV_DB:10;
    vuint32_t TOV_DW:6;
    vuint32_t NOT_USED:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TOV_tag;

typedef union GTM_gtm_cls0_DPLL_TOV_S_union_tag { /* Time Out Value of active STATE Slope (for missing STATE generation) */
  vuint32_t R;
  struct {
    vuint32_t DB:10;
    vuint32_t DW:6;
    vuint32_t NOT_USED:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TOV_S_tag;

typedef union GTM_gtm_cls0_DPLL_ADD_IN_CAL1_union_tag { /* Calculated ADD_IN Value for SUB_INC1 Generation */
  vuint32_t R;
  struct {
    vuint32_t ADD_IN_CAL1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADD_IN_CAL1_tag;

typedef union GTM_gtm_cls0_DPLL_ADD_IN_CAL2_union_tag { /* Calculated ADD_IN Value for SUB_INC2 Generation */
  vuint32_t R;
  struct {
    vuint32_t ADD_IN_CAL2:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADD_IN_CAL2_tag;

typedef union GTM_gtm_cls0_DPLL_MPVAL1_union_tag { /* Missing Pulses to be Added or Subtracted Directly */
  vuint32_t R;
  struct {
    vuint32_t MPVAL1:16;
    vuint32_t SIX1:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_MPVAL1_tag;

typedef union GTM_gtm_cls0_DPLL_MPVAL2_union_tag { /* Missing Pulses to be Added or Subtracted Directly */
  vuint32_t R;
  struct {
    vuint32_t MPVAL2:16;
    vuint32_t SIX2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_MPVAL2_tag;

typedef union GTM_gtm_cls0_DPLL_NMB_T_TAR_union_tag { /* Target Number of Pulses to be sent in Normal Mode */
  vuint32_t R;
  struct {
    vuint32_t NMB_T_TAR:16;
    vuint32_t NOT_USED:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NMB_T_TAR_tag;

typedef union GTM_gtm_cls0_DPLL_NMB_T_TAR_OLD_union_tag { /* Last but one Target Number of Pulses to be sent in Normal Mode */
  vuint32_t R;
  struct {
    vuint32_t NMB_T_TAR_OLD:16;
    vuint32_t NOT_USED:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NMB_T_TAR_OLD_tag;

typedef union GTM_gtm_cls0_DPLL_NMB_S_TAR_union_tag { /* Target Number of Pulses to be sent in Emergency Mode */
  vuint32_t R;
  struct {
    vuint32_t NMB_S_TAR:20;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NMB_S_TAR_tag;

typedef union GTM_gtm_cls0_DPLL_NMB_S_TAR_OLD_union_tag { /* Last but one Target Number of Pulses to be sent in Emergency Mode */
  vuint32_t R;
  struct {
    vuint32_t NMB_S_TAR_OLD:20;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NMB_S_TAR_OLD_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_3_0_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_3_0_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_3_1_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_3_1_tag;

typedef union GTM_gtm_cls0_DPLL_RCDT_TX_union_tag { /* Reciprocal Value of the Expected Increment Duration of TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t RCDT_TX:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RCDT_TX_tag;

typedef union GTM_gtm_cls0_DPLL_RCDT_SX_union_tag { /* Reciprocal Value of the Expected Increment Duration of STATE */
  vuint32_t R;
  struct {
    vuint32_t RCDT_SX:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RCDT_SX_tag;

typedef union GTM_gtm_cls0_DPLL_RCDT_TX_NOM_union_tag { /* Reciprocal Value of the Expected Nominal Increment Duration of TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t RCDT_TX_NOM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RCDT_TX_NOM_tag;

typedef union GTM_gtm_cls0_DPLL_RCDT_SX_NOM_union_tag { /* Reciprocal Value of the Expected Nominal Increment Duration of STATE */
  vuint32_t R;
  struct {
    vuint32_t RCDT_SX_NOM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RCDT_SX_NOM_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_T_ACT_union_tag { /* Reciprocal Value of the Last Increment of TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t RDT_T_ACT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_T_ACT_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S_ACT_union_tag { /* Reciprocal Value of the Last Increment of STATE */
  vuint32_t R;
  struct {
    vuint32_t RDT_S_ACT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S_ACT_tag;

typedef union GTM_gtm_cls0_DPLL_DT_T_ACT_union_tag { /* Duration of the Last TRIGGER Increment */
  vuint32_t R;
  struct {
    vuint32_t DT_T_ACT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_T_ACT_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S_ACT_union_tag { /* Duration of the Last STATE Increment */
  vuint32_t R;
  struct {
    vuint32_t DT_S_ACT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S_ACT_tag;

typedef union GTM_gtm_cls0_DPLL_EDT_T_union_tag { /* Difference of Prediction to Actual Value of the Last TRIGGER Increment */
  vuint32_t R;
  struct {
    vuint32_t EDT_T:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_EDT_T_tag;

typedef union GTM_gtm_cls0_DPLL_MEDT_T_union_tag { /* Weighted Difference of Prediction Errors of TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t MEDT_T:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_MEDT_T_tag;

typedef union GTM_gtm_cls0_DPLL_EDT_S_union_tag { /* Difference of Prediction to Actual Value of the Last STATE Increment */
  vuint32_t R;
  struct {
    vuint32_t EDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_EDT_S_tag;

typedef union GTM_gtm_cls0_DPLL_MEDT_S_union_tag { /* Weighted Difference of Prediction Errors of STATE */
  vuint32_t R;
  struct {
    vuint32_t MEDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_MEDT_S_tag;

typedef union GTM_gtm_cls0_DPLL_CDT_TX_union_tag { /* Prediction of the Actual TRIGGER Increment Duration */
  vuint32_t R;
  struct {
    vuint32_t CDT_TX:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CDT_TX_tag;

typedef union GTM_gtm_cls0_DPLL_CDT_SX_union_tag { /* Prediction of the Actual STATE Increment Duration */
  vuint32_t R;
  struct {
    vuint32_t CDT_SX:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CDT_SX_tag;

typedef union GTM_gtm_cls0_DPLL_CDT_TX_NOM_union_tag { /* Prediction of the Nominal TRIGGER Increment Duration */
  vuint32_t R;
  struct {
    vuint32_t CDT_TX_NOM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CDT_TX_NOM_tag;

typedef union GTM_gtm_cls0_DPLL_CDT_SX_NOM_union_tag { /* Prediction of the Nominal STATE Increment Duration */
  vuint32_t R;
  struct {
    vuint32_t CDT_SX_NOM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CDT_SX_NOM_tag;

typedef union GTM_gtm_cls0_DPLL_TLR_union_tag { /* TRIGGER Locking Range */
  vuint32_t R;
  struct {
    vuint32_t TLR:8;
    vuint32_t NOT_USED:16;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TLR_tag;

typedef union GTM_gtm_cls0_DPLL_SLR_union_tag { /* STATE Locking Range */
  vuint32_t R;
  struct {
    vuint32_t SLR:8;
    vuint32_t NOT_USED:16;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_SLR_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_0_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_0_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_1_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_1_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_2_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_2_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_3_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_3_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_4_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_4_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_5_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_5_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_6_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_6_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_7_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_7_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_8_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_8_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_9_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_9_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_10_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_10_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_11_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_11_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_12_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_12_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_13_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_13_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_14_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_14_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_15_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_15_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_16_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_16_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_17_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_17_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_18_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_18_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_19_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_19_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_20_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_20_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_21_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_21_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_0_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_0_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_1_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_1_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_2_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_2_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_3_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_3_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_4_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_4_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_5_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_5_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_6_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_6_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_7_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_7_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_8_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_8_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_9_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_9_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_10_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_10_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_11_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_11_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_12_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_12_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_13_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_13_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_14_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_14_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_15_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_15_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_16_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_16_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_17_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_17_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_18_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_18_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_19_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_19_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_20_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_20_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_21_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_21_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_22_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_22_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_23_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_23_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_24_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_24_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_25_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_25_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_26_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_26_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_27_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_27_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_28_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_28_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_29_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_29_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_30_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_30_tag;

typedef union GTM_gtm_cls0_DPLL_PDT_31_union_tag { /* Projected Increment Sum Relations for Action [n] */
  vuint32_t R;
  struct {
    vuint32_t DB:14;
    vuint32_t DW:10;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PDT_31_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_0_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_0_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_1_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_1_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_2_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_2_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_3_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_3_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_4_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_4_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_5_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_5_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_6_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_6_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_7_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_7_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_8_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_8_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_9_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_9_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_10_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_10_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_11_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_11_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_12_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_12_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_13_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_13_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_14_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_14_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_15_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_15_tag;

typedef union GTM_gtm_cls0_DPLL_MLS1_union_tag { /* Calculated Number of Sub-Pulses between two nominal STATE Events for DPLL_CTRL_1.SMC = 0 */
  vuint32_t R;
  struct {
    vuint32_t MLS1:18;
    vuint32_t NOT_USED:6;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_MLS1_tag;

typedef union GTM_gtm_cls0_DPLL_MLS2_union_tag { /* Calculated Number of Sub-Pulses between two nominal STATE Events for DPLL_CTRL_1.SMC=1 and RMO=1 */
  vuint32_t R;
  struct {
    vuint32_t MLS2:18;
    vuint32_t NOT_USED:6;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_MLS2_tag;

typedef union GTM_gtm_cls0_DPLL_CNT_NUM_1_union_tag { /* Number of SUB_INC1 pulses in continuous mode */
  vuint32_t R;
  struct {
    vuint32_t CNT_NUM_1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CNT_NUM_1_tag;

typedef union GTM_gtm_cls0_DPLL_CNT_NUM_2_union_tag { /* Number of SUB_INC2 pulses in continuous mode */
  vuint32_t R;
  struct {
    vuint32_t CNT_NUM_2:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CNT_NUM_2_tag;

typedef union GTM_gtm_cls0_DPLL_PVT_union_tag { /* Plausibility Value of Next TRIGGER Slope */
  vuint32_t R;
  struct {
    vuint32_t PVT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PVT_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_0_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_0_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_1_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_1_tag;

typedef union GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_2_union_tag { /* DPLL RAM1B reserved data [k] */
  vuint32_t R;
  struct {
    vuint32_t RSVD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_2_tag;

typedef union GTM_gtm_cls0_DPLL_PSTC_union_tag { /* Actual Calculated Position Stamp of TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t PSTC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSTC_tag;

typedef union GTM_gtm_cls0_DPLL_PSSC_union_tag { /* Actual Calculated Position Stamp of STATE */
  vuint32_t R;
  struct {
    vuint32_t PSSC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSSC_tag;

typedef union GTM_gtm_cls0_DPLL_PSTM_union_tag { /* Measured Position Stamp at Last TRIGGER Input */
  vuint32_t R;
  struct {
    vuint32_t PSTM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSTM_tag;

typedef union GTM_gtm_cls0_DPLL_PSTM_OLD_union_tag { /* Measured Position Stamp at Last but one TRIGGER Input */
  vuint32_t R;
  struct {
    vuint32_t PSTM_OLD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSTM_OLD_tag;

typedef union GTM_gtm_cls0_DPLL_PSSM_union_tag { /* Measured Position Stamp at Last STATE Input */
  vuint32_t R;
  struct {
    vuint32_t PSSM:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSSM_tag;

typedef union GTM_gtm_cls0_DPLL_PSSM_OLD_union_tag { /* Measured Position Stamp at Last but one STATE Input */
  vuint32_t R;
  struct {
    vuint32_t PSSM_OLD:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSSM_OLD_tag;

typedef union GTM_gtm_cls0_DPLL_NMB_T_union_tag { /* Number of Pulses to be sent in Normal Mode */
  vuint32_t R;
  struct {
    vuint32_t NMB_T:16;
    vuint32_t NOT_USED:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NMB_T_tag;

typedef union GTM_gtm_cls0_DPLL_NMB_S_union_tag { /* Number of Pulses to be sent in Emergency Mode */
  vuint32_t R;
  struct {
    vuint32_t NMB_S:20;
    vuint32_t NOT_USED:4;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_NMB_S_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S0_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S0_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S1_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S1_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S2_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S2_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S3_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S3_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S4_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S4_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S5_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S5_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S6_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S6_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S7_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S7_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S8_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S8_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S9_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S9_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S10_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S10_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S11_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S11_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S12_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S12_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S13_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S13_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S14_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S14_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S15_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S15_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S16_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S16_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S17_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S17_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S18_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S18_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S19_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S19_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S20_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S20_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S21_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S21_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S22_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S22_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S23_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S23_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S24_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S24_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S25_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S25_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S26_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S26_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S27_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S27_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S28_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S28_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S29_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S29_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S30_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S30_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S31_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S31_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S32_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S32_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S33_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S33_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S34_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S34_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S35_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S35_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S36_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S36_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S37_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S37_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S38_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S38_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S39_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S39_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S40_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S40_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S41_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S41_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S42_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S42_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S43_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S43_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S44_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S44_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S45_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S45_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S46_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S46_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S47_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S47_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S48_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S48_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S49_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S49_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S50_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S50_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S51_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S51_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S52_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S52_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S53_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S53_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S54_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S54_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S55_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S55_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S56_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S56_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S57_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S57_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S58_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S58_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S59_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S59_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S60_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S60_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S61_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S61_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S62_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S62_tag;

typedef union GTM_gtm_cls0_DPLL_RDT_S63_union_tag { /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t RDT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RDT_S63_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S0_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S0_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S1_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S1_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S2_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S2_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S3_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S3_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S4_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S4_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S5_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S5_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S6_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S6_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S7_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S7_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S8_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S8_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S9_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S9_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S10_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S10_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S11_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S11_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S12_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S12_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S13_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S13_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S14_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S14_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S15_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S15_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S16_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S16_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S17_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S17_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S18_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S18_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S19_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S19_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S20_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S20_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S21_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S21_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S22_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S22_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S23_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S23_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S24_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S24_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S25_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S25_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S26_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S26_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S27_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S27_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S28_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S28_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S29_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S29_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S30_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S30_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S31_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S31_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S32_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S32_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S33_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S33_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S34_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S34_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S35_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S35_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S36_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S36_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S37_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S37_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S38_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S38_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S39_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S39_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S40_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S40_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S41_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S41_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S42_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S42_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S43_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S43_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S44_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S44_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S45_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S45_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S46_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S46_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S47_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S47_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S48_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S48_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S49_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S49_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S50_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S50_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S51_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S51_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S52_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S52_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S53_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S53_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S54_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S54_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S55_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S55_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S56_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S56_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S57_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S57_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S58_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S58_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S59_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S59_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S60_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S60_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S61_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S61_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S62_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S62_tag;

typedef union GTM_gtm_cls0_DPLL_TSF_S63_union_tag { /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t TSF_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSF_S63_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S0_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S0_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S1_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S1_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S2_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S2_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S3_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S3_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S4_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S4_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S5_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S5_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S6_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S6_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S7_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S7_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S8_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S8_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S9_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S9_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S10_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S10_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S11_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S11_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S12_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S12_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S13_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S13_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S14_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S14_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S15_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S15_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S16_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S16_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S17_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S17_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S18_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S18_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S19_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S19_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S20_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S20_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S21_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S21_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S22_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S22_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S23_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S23_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S24_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S24_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S25_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S25_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S26_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S26_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S27_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S27_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S28_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S28_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S29_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S29_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S30_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S30_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S31_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S31_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S32_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S32_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S33_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S33_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S34_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S34_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S35_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S35_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S36_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S36_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S37_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S37_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S38_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S38_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S39_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S39_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S40_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S40_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S41_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S41_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S42_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S42_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S43_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S43_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S44_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S44_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S45_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S45_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S46_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S46_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S47_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S47_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S48_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S48_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S49_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S49_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S50_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S50_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S51_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S51_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S52_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S52_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S53_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S53_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S54_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S54_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S55_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S55_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S56_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S56_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S57_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S57_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S58_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S58_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S59_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S59_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S60_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S60_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S61_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S61_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S62_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S62_tag;

typedef union GTM_gtm_cls0_DPLL_ADT_S63_union_tag { /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  vuint32_t R;
  struct {
    vuint32_t PD_S:16;
    vuint32_t NS:6;
    vuint32_t NOT_USED:2;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_ADT_S63_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S0_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S0_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S1_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S1_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S2_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S2_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S3_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S3_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S4_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S4_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S5_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S5_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S6_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S6_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S7_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S7_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S8_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S8_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S9_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S9_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S10_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S10_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S11_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S11_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S12_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S12_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S13_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S13_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S14_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S14_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S15_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S15_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S16_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S16_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S17_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S17_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S18_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S18_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S19_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S19_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S20_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S20_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S21_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S21_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S22_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S22_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S23_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S23_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S24_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S24_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S25_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S25_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S26_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S26_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S27_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S27_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S28_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S28_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S29_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S29_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S30_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S30_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S31_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S31_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S32_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S32_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S33_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S33_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S34_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S34_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S35_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S35_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S36_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S36_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S37_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S37_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S38_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S38_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S39_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S39_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S40_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S40_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S41_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S41_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S42_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S42_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S43_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S43_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S44_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S44_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S45_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S45_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S46_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S46_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S47_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S47_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S48_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S48_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S49_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S49_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S50_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S50_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S51_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S51_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S52_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S52_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S53_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S53_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S54_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S54_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S55_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S55_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S56_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S56_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S57_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S57_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S58_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S58_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S59_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S59_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S60_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S60_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S61_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S61_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S62_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S62_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S63_union_tag { /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  vuint32_t R;
  struct {
    vuint32_t DT_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S63_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC0_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC0_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC1_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC1_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC2_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC2_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC3_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC3_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC4_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC4_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC5_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC5_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC6_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC6_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC7_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC7_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC8_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC8_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC9_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC9_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC10_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC10_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC11_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC11_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC12_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC12_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC13_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC13_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC14_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC14_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC15_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC15_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC16_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC16_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC17_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC17_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC18_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC18_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC19_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC19_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC20_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC20_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC21_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC21_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC22_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC22_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC23_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC23_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC24_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC24_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC25_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC25_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC26_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC26_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC27_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC27_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC28_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC28_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC29_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC29_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC30_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC30_tag;

typedef union GTM_gtm_cls0_DPLL_TSAC31_union_tag { /* Calculated Time Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t TSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TSAC31_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC0_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC0_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC1_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC1_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC2_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC2_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC3_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC3_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC4_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC4_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC5_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC5_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC6_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC6_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC7_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC7_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC8_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC8_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC9_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC9_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC10_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC10_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC11_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC11_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC12_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC12_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC13_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC13_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC14_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC14_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC15_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC15_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC16_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC16_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC17_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC17_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC18_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC18_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC19_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC19_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC20_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC20_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC21_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC21_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC22_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC22_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC23_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC23_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC24_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC24_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC25_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC25_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC26_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC26_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC27_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC27_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC28_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC28_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC29_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC29_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC30_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC30_tag;

typedef union GTM_gtm_cls0_DPLL_PSAC31_union_tag { /* Calculated Position Value to start Action [n] */
  vuint32_t R;
  struct {
    vuint32_t PSAC:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_PSAC31_tag;

typedef union GTM_gtm_cls0_DPLL_ACB_0_union_tag { /* Control Bits for NOAC Actions */
  vuint32_t R;
  struct {
    vuint32_t ACB_0:5;
    vuint32_t _unused_5:3;
    vuint32_t ACB_1:5;
    vuint32_t _unused_13:3;
    vuint32_t ACB_2:5;
    vuint32_t _unused_21:3;
    vuint32_t ACB_3:5;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_DPLL_ACB_0_tag;

typedef union GTM_gtm_cls0_DPLL_ACB_1_union_tag { /* Control Bits for NOAC Actions */
  vuint32_t R;
  struct {
    vuint32_t ACB_0:5;
    vuint32_t _unused_5:3;
    vuint32_t ACB_1:5;
    vuint32_t _unused_13:3;
    vuint32_t ACB_2:5;
    vuint32_t _unused_21:3;
    vuint32_t ACB_3:5;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_DPLL_ACB_1_tag;

typedef union GTM_gtm_cls0_DPLL_ACB_2_union_tag { /* Control Bits for NOAC Actions */
  vuint32_t R;
  struct {
    vuint32_t ACB_0:5;
    vuint32_t _unused_5:3;
    vuint32_t ACB_1:5;
    vuint32_t _unused_13:3;
    vuint32_t ACB_2:5;
    vuint32_t _unused_21:3;
    vuint32_t ACB_3:5;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_DPLL_ACB_2_tag;

typedef union GTM_gtm_cls0_DPLL_ACB_3_union_tag { /* Control Bits for NOAC Actions */
  vuint32_t R;
  struct {
    vuint32_t ACB_0:5;
    vuint32_t _unused_5:3;
    vuint32_t ACB_1:5;
    vuint32_t _unused_13:3;
    vuint32_t ACB_2:5;
    vuint32_t _unused_21:3;
    vuint32_t ACB_3:5;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_DPLL_ACB_3_tag;

typedef union GTM_gtm_cls0_DPLL_ACB_4_union_tag { /* Control Bits for NOAC Actions */
  vuint32_t R;
  struct {
    vuint32_t ACB_0:5;
    vuint32_t _unused_5:3;
    vuint32_t ACB_1:5;
    vuint32_t _unused_13:3;
    vuint32_t ACB_2:5;
    vuint32_t _unused_21:3;
    vuint32_t ACB_3:5;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_DPLL_ACB_4_tag;

typedef union GTM_gtm_cls0_DPLL_ACB_5_union_tag { /* Control Bits for NOAC Actions */
  vuint32_t R;
  struct {
    vuint32_t ACB_0:5;
    vuint32_t _unused_5:3;
    vuint32_t ACB_1:5;
    vuint32_t _unused_13:3;
    vuint32_t ACB_2:5;
    vuint32_t _unused_21:3;
    vuint32_t ACB_3:5;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_DPLL_ACB_5_tag;

typedef union GTM_gtm_cls0_DPLL_ACB_6_union_tag { /* Control Bits for NOAC Actions */
  vuint32_t R;
  struct {
    vuint32_t ACB_0:5;
    vuint32_t _unused_5:3;
    vuint32_t ACB_1:5;
    vuint32_t _unused_13:3;
    vuint32_t ACB_2:5;
    vuint32_t _unused_21:3;
    vuint32_t ACB_3:5;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_DPLL_ACB_6_tag;

typedef union GTM_gtm_cls0_DPLL_ACB_7_union_tag { /* Control Bits for NOAC Actions */
  vuint32_t R;
  struct {
    vuint32_t ACB_0:5;
    vuint32_t _unused_5:3;
    vuint32_t ACB_1:5;
    vuint32_t _unused_13:3;
    vuint32_t ACB_2:5;
    vuint32_t _unused_21:3;
    vuint32_t ACB_3:5;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls0_DPLL_ACB_7_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_11_union_tag { /* Control Register 11 */
  vuint32_t R;
  struct {
    vuint32_t SIP1:1;
    vuint32_t ERZ1:1;
    vuint32_t PCMF1:1;
    vuint32_t FSYL1:1;
    vuint32_t INCF1:1;
    vuint32_t PCMF1_INCCNT_B:1;
    vuint32_t ADT:1;
    vuint32_t ADS:1;
    vuint32_t SIP2:1;
    vuint32_t ERZ2:1;
    vuint32_t PCMF2:1;
    vuint32_t FSYL2:1;
    vuint32_t INCF2:1;
    vuint32_t PCMF2_INCCNT_B:1;
    vuint32_t STATE_EXT:1;
    vuint32_t ACBU:1;
    vuint32_t WSIP1:1;
    vuint32_t WERZ1:1;
    vuint32_t WPCMF1:1;
    vuint32_t WFSYL1:1;
    vuint32_t WINCF1:1;
    vuint32_t WPCMF1_INCCNT_B:1;
    vuint32_t WADT:1;
    vuint32_t WADS:1;
    vuint32_t WSIP2:1;
    vuint32_t WERZ2:1;
    vuint32_t WPCMF2:1;
    vuint32_t WFSYL2:1;
    vuint32_t WINCF2:1;
    vuint32_t WPCMF2_INCCNT_B:1;
    vuint32_t WSTATE_EXT:1;
    vuint32_t WACBU:1;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_11_tag;

typedef union GTM_gtm_cls0_DPLL_THVAL2_union_tag { /* Measured TRIGGER Hold Time Value 2 */
  vuint32_t R;
  struct {
    vuint32_t THVAL:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_THVAL2_tag;

typedef union GTM_gtm_cls0_DPLL_TIDEL_union_tag { /* TRIGGER input delay */
  vuint32_t R;
  struct {
    vuint32_t TIDEL:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_TIDEL_tag;

typedef union GTM_gtm_cls0_DPLL_SIDEL_union_tag { /* STATE input delay */
  vuint32_t R;
  struct {
    vuint32_t SIDEL:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_SIDEL_tag;

typedef union GTM_gtm_cls0_DPLL_APS_SYNC_EXT_union_tag { /* STATE Time Stamp Field Offset at Synchronization Time */
  vuint32_t R;
  struct {
    vuint32_t APS_1C2_EXT:7;
    vuint32_t _unused_7:8;
    vuint32_t APS_1C2_STATUS:1;
    vuint32_t APS_1C2_OLD:7;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls0_DPLL_APS_SYNC_EXT_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_EXT_union_tag { /* STATE Time Stamp Field Offset at Synchronization Time */
  vuint32_t R;
  struct {
    vuint32_t SNU:6;
    vuint32_t _unused_6:10;
    vuint32_t SYN_NS:6;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_EXT_tag;

typedef union GTM_gtm_cls0_DPLL_APS_EXT_union_tag { /* Actual RAM Pointer Address for STATE */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t WAPS:1;
    vuint32_t APS:7;
    vuint32_t _unused_9:4;
    vuint32_t WAPS_1C2:1;
    vuint32_t APS_1C2:7;
    vuint32_t _unused_21:11;
  } B;
} GTM_gtm_cls0_DPLL_APS_EXT_tag;

typedef union GTM_gtm_cls0_DPLL_APS_1C3_EXT_union_tag { /* Actual RAM Pointer Address for RAM region 1c3 (DPLL_CTRL_11.STATE_EXT=1) */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t APS_1C3:7;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls0_DPLL_APS_1C3_EXT_tag;

typedef union GTM_gtm_cls0_DPLL_STA_union_tag { /* Status of the state machine states */
  vuint32_t R;
  struct {
    vuint32_t STA_T:8;
    vuint32_t _unused_8:1;
    vuint32_t CNT_T:3;
    vuint32_t STA_S:8;
    vuint32_t _unused_20:1;
    vuint32_t CNT_S:3;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_STA_tag;

typedef union GTM_gtm_cls0_DPLL_INCF1_OFFSET_union_tag { /* Start value of ADD_IN_ADDER1 */
  vuint32_t R;
  struct {
    vuint32_t INCF1_OFFSET:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_INCF1_OFFSET_tag;

typedef union GTM_gtm_cls0_DPLL_INCF2_OFFSET_union_tag { /* Start value of the ADD_IN_ADDER2 */
  vuint32_t R;
  struct {
    vuint32_t INCF2_OFFSET:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_INCF2_OFFSET_tag;

typedef union GTM_gtm_cls0_DPLL_DT_T_START_union_tag { /* Start value of DT_T_ACT */
  vuint32_t R;
  struct {
    vuint32_t DT_T_START:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_T_START_tag;

typedef union GTM_gtm_cls0_DPLL_DT_S_START_union_tag { /* Start value of DT_S_ACT */
  vuint32_t R;
  struct {
    vuint32_t DT_S_START:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_DT_S_START_tag;

typedef union GTM_gtm_cls0_DPLL_STA_MASK_union_tag { /* Notify values for DPLL_STA */
  vuint32_t R;
  struct {
    vuint32_t STA_NOTIFY_T:8;
    vuint32_t STA_NOTIFY_S:8;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls0_DPLL_STA_MASK_tag;

typedef union GTM_gtm_cls0_DPLL_STA_FLAG_union_tag { /* DPLL STA Flags */
  vuint32_t R;
  struct {
    vuint32_t STA_FLAG_T:1;
    vuint32_t _unused_1:7;
    vuint32_t STA_FLAG_S:1;
    vuint32_t INC_CNT1_FLAG:1;
    vuint32_t INC_CNT2_FLAG:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls0_DPLL_STA_FLAG_tag;

typedef union GTM_gtm_cls0_DPLL_INC_CNT1_MASK_union_tag { /* Notify value of DPLL_INC_CNT1 */
  vuint32_t R;
  struct {
    vuint32_t INC_CNT1_NOTIFY:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_INC_CNT1_MASK_tag;

typedef union GTM_gtm_cls0_DPLL_INC_CNT2_MASK_union_tag { /* Notify value of DPLL_INC_CNT2 */
  vuint32_t R;
  struct {
    vuint32_t INC_CNT2_NOTIFY:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_INC_CNT2_MASK_tag;

typedef union GTM_gtm_cls0_DPLL_NUSC_EXT1_union_tag { /* Number of Recent STATE Events used for Calculations */
  vuint32_t R;
  struct {
    vuint32_t SYN_S:7;
    vuint32_t _unused_7:9;
    vuint32_t SYN_S_OLD:7;
    vuint32_t _unused_23:7;
    vuint32_t WSYN:1;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls0_DPLL_NUSC_EXT1_tag;

typedef union GTM_gtm_cls0_DPLL_NUSC_EXT2_union_tag { /* Number of Recent STATE Events used for Calculations */
  vuint32_t R;
  struct {
    vuint32_t NUSE:7;
    vuint32_t _unused_7:8;
    vuint32_t FSS:1;
    vuint32_t VSN:7;
    vuint32_t _unused_23:6;
    vuint32_t WNUS:1;
    vuint32_t _unused_30:1;
    vuint32_t WVSN:1;
  } B;
} GTM_gtm_cls0_DPLL_NUSC_EXT2_tag;

typedef union GTM_gtm_cls0_DPLL_CTN_MIN_union_tag { /* Minimum value of predicted nominal increment of TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t CTN_MIN:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CTN_MIN_tag;

typedef union GTM_gtm_cls0_DPLL_CTN_MAX_union_tag { /* Maximum value of predicted nominal increment of TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t CTN_MAX:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CTN_MAX_tag;

typedef union GTM_gtm_cls0_DPLL_CSN_MIN_union_tag { /* Minimum value of predicted nominal increment of STATE */
  vuint32_t R;
  struct {
    vuint32_t CSN_MIN:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CSN_MIN_tag;

typedef union GTM_gtm_cls0_DPLL_CSN_MAX_union_tag { /* Maximum value of predicted nominal increment of STATE */
  vuint32_t R;
  struct {
    vuint32_t CSN_MAX:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_CSN_MAX_tag;

typedef union GTM_gtm_cls0_DPLL_SW_TRIG_union_tag { /* Software triggered input events */
  vuint32_t R;
  struct {
    vuint32_t TRIG_EVENT:1;
    vuint32_t WTRIG_EVENT:1;
    vuint32_t TRIG_LEVEL:1;
    vuint32_t WTRIG_LEVEL:1;
    vuint32_t STATE_EVENT:1;
    vuint32_t WSTATE_EVENT:1;
    vuint32_t STATE_LEVEL:1;
    vuint32_t WSTATE_LEVEL:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls0_DPLL_SW_TRIG_tag;

typedef union GTM_gtm_cls0_DPLL_MP_T_union_tag { /* Missing pulses of TRIGGER */
  vuint32_t R;
  struct {
    vuint32_t MP_T:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_MP_T_tag;

typedef union GTM_gtm_cls0_DPLL_MP_S_union_tag { /* Missing pulses of STATE */
  vuint32_t R;
  struct {
    vuint32_t MP_S:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_MP_S_tag;

typedef union GTM_gtm_cls0_DPLL_CTRL_12_union_tag { /* DPLL control register 12 */
  vuint32_t R;
  struct {
    vuint32_t SUBINC_MUX_SEL:1;
    vuint32_t _unused_1:15;
    vuint32_t WSUBINC_MUX_SEL:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls0_DPLL_CTRL_12_tag;

typedef union GTM_gtm_cls0_DPLL_RR2_union_tag { /* DPLL memory RR2 */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls0_DPLL_RR2_tag;

typedef union GTM_gtm_cls0_MCS0_MEM_union_tag { /* MCS[i] memory region */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls0_MCS0_MEM_tag;

struct GTM_gtm_cls0_tag {
  GTM_gtm_cls0_GTM_REV_tag GTM_REV;    /* GTM version control register */
  GTM_gtm_cls0_GTM_RST_tag GTM_RST;    /* GTM global reset register */
  GTM_gtm_cls0_GTM_CTRL_tag GTM_CTRL;  /* GTM global control register */
  GTM_gtm_cls0_GTM_CFG_tag GTM_CFG;    /* GTM configuration register */
  GTM_gtm_cls0_GTM_AEI_ADDR_XPT_tag GTM_AEI_ADDR_XPT; /* GTM AEI timeout exception address register */
  GTM_gtm_cls0_GTM_AEI_STA_XPT_tag GTM_AEI_STA_XPT; /* GTM AEI non zero status register */
  GTM_gtm_cls0_GTM_IRQ_NOTIFY_tag GTM_IRQ_NOTIFY; /* GTM Interrupt notification register */
  GTM_gtm_cls0_GTM_IRQ_EN_tag GTM_IRQ_EN; /* GTM interrupt enable register */
  GTM_gtm_cls0_GTM_EIRQ_EN_tag GTM_EIRQ_EN; /* GTM error interrupt enable register */
  GTM_gtm_cls0_GTM_IRQ_FORCINT_tag GTM_IRQ_FORCINT; /* GTM Software interrupt generation register */
  GTM_gtm_cls0_GTM_IRQ_MODE_tag GTM_IRQ_MODE; /* GTM top level interrupts mode selection */
  GTM_gtm_cls0_GTM_CLS_CLK_CFG_tag GTM_CLS_CLK_CFG; /* GTM Cluster Clock Configuration */
  GTM_gtm_cls0_GTM_ARU_COM_DIS_tag GTM_ARU_COM_DIS; /* GTM ARU communication disable */
  uint8_t GTM_gtm_cls0_reserved0[12];
  GTM_gtm_cls0_BRIDGE_MODE_tag BRIDGE_MODE; /* GTM AEI bridge mode register */
  GTM_gtm_cls0_BRIDGE_PTR1_tag BRIDGE_PTR1; /* GTM AEI bridge pointer 1 register */
  GTM_gtm_cls0_BRIDGE_PTR2_tag BRIDGE_PTR2; /* GTM AEI bridge pointer 2 register */
  GTM_gtm_cls0_MCS_AEM_DIS_tag MCS_AEM_DIS; /* GTM MCS master port disable register */
  uint8_t GTM_gtm_cls0_reserved1[48];
  GTM_gtm_cls0_CMU_CLK_EN_tag CMU_CLK_EN; /* CMU clock enable */
  GTM_gtm_cls0_CMU_GCLK_NUM_tag CMU_GCLK_NUM; /* The numerator for CMU global clock resolution generator */
  GTM_gtm_cls0_CMU_GCLK_DEN_tag CMU_GCLK_DEN; /* The denominator for CMU global clock resolution generator */
  GTM_gtm_cls0_CMU_CLK_0_CTRL_tag CMU_CLK_0_CTRL; /* CMU control for clock resolution generator [x] */
  GTM_gtm_cls0_CMU_CLK_1_CTRL_tag CMU_CLK_1_CTRL; /* CMU control for clock resolution generator [x] */
  GTM_gtm_cls0_CMU_CLK_2_CTRL_tag CMU_CLK_2_CTRL; /* CMU control for clock resolution generator [x] */
  GTM_gtm_cls0_CMU_CLK_3_CTRL_tag CMU_CLK_3_CTRL; /* CMU control for clock resolution generator [x] */
  GTM_gtm_cls0_CMU_CLK_4_CTRL_tag CMU_CLK_4_CTRL; /* CMU control for clock resolution generator [x] */
  GTM_gtm_cls0_CMU_CLK_5_CTRL_tag CMU_CLK_5_CTRL; /* CMU control for clock resolution generator [x] */
  GTM_gtm_cls0_CMU_CLK_6_CTRL_tag CMU_CLK_6_CTRL; /* CMU control for clock resolution generator 6 */
  GTM_gtm_cls0_CMU_CLK_7_CTRL_tag CMU_CLK_7_CTRL; /* CMU control for clock resolution generator 7 */
  GTM_gtm_cls0_CMU_ECLK_0_NUM_tag CMU_ECLK_0_NUM; /* The numerator for the external clock resolution generator [z] */
  GTM_gtm_cls0_CMU_ECLK_0_DEN_tag CMU_ECLK_0_DEN; /* The denominator for the external clock resolution generator [z] */
  GTM_gtm_cls0_CMU_ECLK_1_NUM_tag CMU_ECLK_1_NUM; /* The numerator for the external clock resolution generator [z] */
  GTM_gtm_cls0_CMU_ECLK_1_DEN_tag CMU_ECLK_1_DEN; /* The denominator for the external clock resolution generator [z] */
  GTM_gtm_cls0_CMU_ECLK_2_NUM_tag CMU_ECLK_2_NUM; /* The numerator for the external clock resolution generator [z] */
  GTM_gtm_cls0_CMU_ECLK_2_DEN_tag CMU_ECLK_2_DEN; /* The denominator for the external clock resolution generator [z] */
  GTM_gtm_cls0_CMU_FXCLK_CTRL_tag CMU_FXCLK_CTRL; /* CMU control for selection of FCR subblock input */
  GTM_gtm_cls0_CMU_GLB_CTRL_tag CMU_GLB_CTRL; /* CMU synchronizing ARU and clock source */
  GTM_gtm_cls0_CMU_CLK_CTRL_tag CMU_CLK_CTRL; /* CMU control for clock resolution generator */
  uint8_t GTM_gtm_cls0_reserved2[48];
  GTM_gtm_cls0_TBU_CHEN_tag TBU_CHEN;  /* TBU global channel enable */
  GTM_gtm_cls0_TBU_CH0_CTRL_tag TBU_CH0_CTRL; /* TBU channel 0 control */
  GTM_gtm_cls0_TBU_CH0_BASE_tag TBU_CH0_BASE; /* TBU channel 0 base */
  GTM_gtm_cls0_TBU_CH1_CTRL_tag TBU_CH1_CTRL; /* TBU channel 1 control */
  GTM_gtm_cls0_TBU_CH1_BASE_tag TBU_CH1_BASE; /* TBU channel [x] base */
  GTM_gtm_cls0_TBU_CH2_CTRL_tag TBU_CH2_CTRL; /* TBU channel 2 control */
  GTM_gtm_cls0_TBU_CH2_BASE_tag TBU_CH2_BASE; /* TBU channel [x] base */
  GTM_gtm_cls0_TBU_CH3_CTRL_tag TBU_CH3_CTRL; /* TBU channel 3 control */
  GTM_gtm_cls0_TBU_CH3_BASE_tag TBU_CH3_BASE; /* TBU channel 3 base */
  GTM_gtm_cls0_TBU_CH3_BASE_MARK_tag TBU_CH3_BASE_MARK; /* TBU channel 3 modulo value */
  GTM_gtm_cls0_TBU_CH3_BASE_CAPTURE_tag TBU_CH3_BASE_CAPTURE; /* TBU channel 3 base captured */
  uint8_t GTM_gtm_cls0_reserved3[84];
  GTM_gtm_cls0_ARU_ACCESS_tag ARU_ACCESS; /* ARU access register */
  GTM_gtm_cls0_ARU_DATA_H_tag ARU_DATA_H; /* ARU access register upper data word */
  GTM_gtm_cls0_ARU_DATA_L_tag ARU_DATA_L; /* ARU access register lower data word */
  GTM_gtm_cls0_ARU_DBG_ACCESS0_tag ARU_DBG_ACCESS0; /* ARU debug access channel 0 */
  GTM_gtm_cls0_ARU_DBG_DATA0_H_tag ARU_DBG_DATA0_H; /* ARU debug access 0 transfer register upper data word */
  GTM_gtm_cls0_ARU_DBG_DATA0_L_tag ARU_DBG_DATA0_L; /* ARU debug access 0 transfer register lower data word */
  GTM_gtm_cls0_ARU_DBG_ACCESS1_tag ARU_DBG_ACCESS1; /* ARU debug access channel 0 */
  GTM_gtm_cls0_ARU_DBG_DATA1_H_tag ARU_DBG_DATA1_H; /* ARU debug access 1 transfer register upper data word */
  GTM_gtm_cls0_ARU_DBG_DATA1_L_tag ARU_DBG_DATA1_L; /* ARU debug access 1 transfer register lower data word */
  GTM_gtm_cls0_ARU_IRQ_NOTIFY_tag ARU_IRQ_NOTIFY; /* ARU interrupt notification register */
  GTM_gtm_cls0_ARU_IRQ_EN_tag ARU_IRQ_EN; /* ARU interrupt enable register */
  GTM_gtm_cls0_ARU_IRQ_FORCINT_tag ARU_IRQ_FORCINT; /* ARU force interrupt register */
  GTM_gtm_cls0_ARU_IRQ_MODE_tag ARU_IRQ_MODE; /* ARU interrupt mode register */
  GTM_gtm_cls0_ARU_CADDR_END_tag ARU_CADDR_END; /* ARU caddr counter end value */
  uint8_t GTM_gtm_cls0_reserved4[4];
  GTM_gtm_cls0_ARU_CTRL_tag ARU_CTRL;  /* ARU enable dynamic routing */
  GTM_gtm_cls0_ARU_0_DYN_CTRL_tag ARU_0_DYN_CTRL; /* ARU [g] dynamic routing control register */
  GTM_gtm_cls0_ARU_1_DYN_CTRL_tag ARU_1_DYN_CTRL; /* ARU [g] dynamic routing control register */
  GTM_gtm_cls0_ARU_0_DYN_ROUTE_LOW_tag ARU_0_DYN_ROUTE_LOW; /* ARU [g] lower bits of DYN_ROUTE register */
  GTM_gtm_cls0_ARU_1_DYN_ROUTE_LOW_tag ARU_1_DYN_ROUTE_LOW; /* ARU [g] lower bits of DYN_ROUTE register */
  GTM_gtm_cls0_ARU_0_DYN_ROUTE_HIGH_tag ARU_0_DYN_ROUTE_HIGH; /* ARU [g] higher bits of DYN_ROUTE register */
  GTM_gtm_cls0_ARU_1_DYN_ROUTE_HIGH_tag ARU_1_DYN_ROUTE_HIGH; /* ARU [g] higher bits of DYN_ROUTE register */
  GTM_gtm_cls0_ARU_0_DYN_ROUTE_SR_LOW_tag ARU_0_DYN_ROUTE_SR_LOW; /* ARU [g] shadow register of DYN_ROUTE register lower bits */
  GTM_gtm_cls0_ARU_1_DYN_ROUTE_SR_LOW_tag ARU_1_DYN_ROUTE_SR_LOW; /* ARU [g] shadow register of DYN_ROUTE register lower bits */
  GTM_gtm_cls0_ARU_0_DYN_ROUTE_SR_HIGH_tag ARU_0_DYN_ROUTE_SR_HIGH; /* ARU [g] shadow register of DYN_ROUTE register higher bits */
  GTM_gtm_cls0_ARU_1_DYN_ROUTE_SR_HIGH_tag ARU_1_DYN_ROUTE_SR_HIGH; /* ARU [g] shadow register of DYN_ROUTE register higher bits */
  GTM_gtm_cls0_ARU_0_DYN_RDADDR_tag ARU_0_DYN_RDADDR; /* ARU [g] master ID for dynamic routing */
  GTM_gtm_cls0_ARU_1_DYN_RDADDR_tag ARU_1_DYN_RDADDR; /* ARU [g] master ID for dynamic routing */
  uint8_t GTM_gtm_cls0_reserved5[12];
  GTM_gtm_cls0_ARU_CADDR_tag ARU_CADDR; /* ARU caddr counter value */
  GTM_gtm_cls0_BRC_SRC_0_ADDR_tag BRC_SRC_0_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_0_DEST_tag BRC_SRC_0_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_1_ADDR_tag BRC_SRC_1_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_1_DEST_tag BRC_SRC_1_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_2_ADDR_tag BRC_SRC_2_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_2_DEST_tag BRC_SRC_2_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_3_ADDR_tag BRC_SRC_3_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_3_DEST_tag BRC_SRC_3_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_4_ADDR_tag BRC_SRC_4_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_4_DEST_tag BRC_SRC_4_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_5_ADDR_tag BRC_SRC_5_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_5_DEST_tag BRC_SRC_5_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_6_ADDR_tag BRC_SRC_6_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_6_DEST_tag BRC_SRC_6_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_7_ADDR_tag BRC_SRC_7_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_7_DEST_tag BRC_SRC_7_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_8_ADDR_tag BRC_SRC_8_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_8_DEST_tag BRC_SRC_8_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_9_ADDR_tag BRC_SRC_9_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_9_DEST_tag BRC_SRC_9_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_10_ADDR_tag BRC_SRC_10_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_10_DEST_tag BRC_SRC_10_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_11_ADDR_tag BRC_SRC_11_ADDR; /* BRC read address for input channel [x] */
  GTM_gtm_cls0_BRC_SRC_11_DEST_tag BRC_SRC_11_DEST; /* BRC destination channels for input channel [x] */
  GTM_gtm_cls0_BRC_IRQ_NOTIFY_tag BRC_IRQ_NOTIFY; /* BRC interrupt notification register */
  GTM_gtm_cls0_BRC_IRQ_EN_tag BRC_IRQ_EN; /* BRC interrupt enable register */
  GTM_gtm_cls0_BRC_IRQ_FORCINT_tag BRC_IRQ_FORCINT; /* BRC force interrupt register */
  GTM_gtm_cls0_BRC_IRQ_MODE_tag BRC_IRQ_MODE; /* BRC interrupt mode configuration register */
  GTM_gtm_cls0_BRC_RST_tag BRC_RST;    /* BRC software reset register */
  GTM_gtm_cls0_BRC_EIRQ_EN_tag BRC_EIRQ_EN; /* BRC error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved6[392];
  GTM_gtm_cls0_ICM_IRQG_0_tag ICM_IRQG_0; /* ICM Interrupt group register covering infrastructural and safety components (ARU, BRC, AEI, PSM0, PSM1, MAP, CMP, SPE) */
  GTM_gtm_cls0_ICM_IRQG_1_tag ICM_IRQG_1; /* ICM Interrupt group register covering DPLL */
  GTM_gtm_cls0_ICM_IRQG_2_tag ICM_IRQG_2; /* ICM Interrupt group register covering TIM0, TIM1, TIM2, TIM3 */
  uint8_t GTM_gtm_cls0_reserved7[4];
  GTM_gtm_cls0_ICM_IRQG_4_tag ICM_IRQG_4; /* ICM Interrupt group register covering MCS0 to MCS3 submodules */
  uint8_t GTM_gtm_cls0_reserved8[28];
  GTM_gtm_cls0_ICM_IRQG_MEI_tag ICM_IRQG_MEI; /* ICM Interrupt group register for module error interrupt information */
  GTM_gtm_cls0_ICM_IRQG_CEI0_tag ICM_IRQG_CEI0; /* ICM Interrupt group register 0 for channel error interrupt information */
  GTM_gtm_cls0_ICM_IRQG_CEI1_tag ICM_IRQG_CEI1; /* ICM Interrupt group register 1 for channel error interrupt information */
  uint8_t GTM_gtm_cls0_reserved9[4];
  GTM_gtm_cls0_ICM_IRQG_CEI3_tag ICM_IRQG_CEI3; /* ICM Interrupt group register 3 for channel error interrupt information */
  uint8_t GTM_gtm_cls0_reserved10[32];
  GTM_gtm_cls0_ICM_IRQG_MCS0_CEI_tag ICM_IRQG_MCS0_CEI; /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
  GTM_gtm_cls0_ICM_IRQG_MCS1_CEI_tag ICM_IRQG_MCS1_CEI; /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
  GTM_gtm_cls0_ICM_IRQG_MCS2_CEI_tag ICM_IRQG_MCS2_CEI; /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
  GTM_gtm_cls0_ICM_IRQG_MCS3_CEI_tag ICM_IRQG_MCS3_CEI; /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
  uint8_t GTM_gtm_cls0_reserved11[48];
  GTM_gtm_cls0_ICM_IRQG_PSM_0_CEI_tag ICM_IRQG_PSM_0_CEI; /* ICM Interrupt group PSM 0 for Channel Error Interrupt information of FIFO0, FIFO1, FIFO2 */
  uint8_t GTM_gtm_cls0_reserved12[12];
  GTM_gtm_cls0_ICM_IRQG_SPE_CEI_tag ICM_IRQG_SPE_CEI; /* ICM Interrupt group SPE for module Error Interrupt information */
  uint8_t GTM_gtm_cls0_reserved13[88];
  GTM_gtm_cls0_ICM_IRQG_CLS_0_MEI_tag ICM_IRQG_CLS_0_MEI; /* ICM Interrupt group for module Error Interrupt information for each TIM[j], MCS[j], SPE[j], FIFO[j] (j=4*[g]+k, k∈{0, ... ,3}) */
  uint8_t GTM_gtm_cls0_reserved14[12];
  GTM_gtm_cls0_ICM_IRQG_MCS0_CI_tag ICM_IRQG_MCS0_CI; /* ICM Interrupt group MCS[j] for Channel Interrupt information */
  GTM_gtm_cls0_ICM_IRQG_MCS1_CI_tag ICM_IRQG_MCS1_CI; /* ICM Interrupt group MCS[j] for Channel Interrupt information */
  GTM_gtm_cls0_ICM_IRQG_MCS2_CI_tag ICM_IRQG_MCS2_CI; /* ICM Interrupt group MCS[j] for Channel Interrupt information */
  GTM_gtm_cls0_ICM_IRQG_MCS3_CI_tag ICM_IRQG_MCS3_CI; /* ICM Interrupt group MCS[j] for Channel Interrupt information */
  uint8_t GTM_gtm_cls0_reserved15[48];
  GTM_gtm_cls0_ICM_IRQG_PSM_0_CI_tag ICM_IRQG_PSM_0_CI; /* ICM Interrupt group PSM 0 for Channel Interrupt information of FIFO0, FIFO1, FIFO2 */
  uint8_t GTM_gtm_cls0_reserved16[12];
  GTM_gtm_cls0_ICM_IRQG_SPE_CI_tag ICM_IRQG_SPE_CI; /* ICM Interrupt group SPE for module Interrupt information */
  uint8_t GTM_gtm_cls0_reserved17[28];
  GTM_gtm_cls0_ICM_IRQG_ATOM_0_CI_tag ICM_IRQG_ATOM_0_CI; /* ICM Interrupt group ATOM [g] for Channel Interrupt information of ATOM[j] (m=4*[g]+(0..3)) */
  uint8_t GTM_gtm_cls0_reserved18[12];
  GTM_gtm_cls0_ICM_IRQG_TOM_0_CI_tag ICM_IRQG_TOM_0_CI; /* ICM Interrupt group TOM [g] for Channel Interrupt information of TOMm ([j]=2*[g]+(0..1)) */
  uint8_t GTM_gtm_cls0_reserved19[156];
  GTM_gtm_cls0_MAP_CTRL_tag MAP_CTRL;  /* MAP Control register */
  uint8_t GTM_gtm_cls0_reserved20[444];
  GTM_gtm_cls0_TIM0_CH0_GPR0_tag TIM0_CH0_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH0_GPR1_tag TIM0_CH0_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH0_CNT_tag TIM0_CH0_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls0_TIM0_CH0_ECNT_tag TIM0_CH0_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls0_TIM0_CH0_CNTS_tag TIM0_CH0_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls0_TIM0_CH0_TDUC_tag TIM0_CH0_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls0_TIM0_CH0_TDUV_tag TIM0_CH0_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls0_TIM0_CH0_FLT_RE_tag TIM0_CH0_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls0_TIM0_CH0_FLT_FE_tag TIM0_CH0_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls0_TIM0_CH0_CTRL_tag TIM0_CH0_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls0_TIM0_CH0_ECTRL_tag TIM0_CH0_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls0_TIM0_CH0_IRQ_NOTIFY_tag TIM0_CH0_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TIM0_CH0_IRQ_EN_tag TIM0_CH0_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TIM0_CH0_IRQ_FORCINT_tag TIM0_CH0_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TIM0_CH0_IRQ_MODE_tag TIM0_CH0_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls0_TIM0_CH0_EIRQ_EN_tag TIM0_CH0_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved21[64];
  GTM_gtm_cls0_TIM0_CH1_GPR0_tag TIM0_CH1_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH1_GPR1_tag TIM0_CH1_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH1_CNT_tag TIM0_CH1_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls0_TIM0_CH1_ECNT_tag TIM0_CH1_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls0_TIM0_CH1_CNTS_tag TIM0_CH1_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls0_TIM0_CH1_TDUC_tag TIM0_CH1_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls0_TIM0_CH1_TDUV_tag TIM0_CH1_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls0_TIM0_CH1_FLT_RE_tag TIM0_CH1_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls0_TIM0_CH1_FLT_FE_tag TIM0_CH1_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls0_TIM0_CH1_CTRL_tag TIM0_CH1_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls0_TIM0_CH1_ECTRL_tag TIM0_CH1_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls0_TIM0_CH1_IRQ_NOTIFY_tag TIM0_CH1_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TIM0_CH1_IRQ_EN_tag TIM0_CH1_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TIM0_CH1_IRQ_FORCINT_tag TIM0_CH1_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TIM0_CH1_IRQ_MODE_tag TIM0_CH1_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls0_TIM0_CH1_EIRQ_EN_tag TIM0_CH1_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved22[64];
  GTM_gtm_cls0_TIM0_CH2_GPR0_tag TIM0_CH2_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH2_GPR1_tag TIM0_CH2_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH2_CNT_tag TIM0_CH2_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls0_TIM0_CH2_ECNT_tag TIM0_CH2_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls0_TIM0_CH2_CNTS_tag TIM0_CH2_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls0_TIM0_CH2_TDUC_tag TIM0_CH2_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls0_TIM0_CH2_TDUV_tag TIM0_CH2_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls0_TIM0_CH2_FLT_RE_tag TIM0_CH2_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls0_TIM0_CH2_FLT_FE_tag TIM0_CH2_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls0_TIM0_CH2_CTRL_tag TIM0_CH2_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls0_TIM0_CH2_ECTRL_tag TIM0_CH2_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls0_TIM0_CH2_IRQ_NOTIFY_tag TIM0_CH2_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TIM0_CH2_IRQ_EN_tag TIM0_CH2_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TIM0_CH2_IRQ_FORCINT_tag TIM0_CH2_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TIM0_CH2_IRQ_MODE_tag TIM0_CH2_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls0_TIM0_CH2_EIRQ_EN_tag TIM0_CH2_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved23[64];
  GTM_gtm_cls0_TIM0_CH3_GPR0_tag TIM0_CH3_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH3_GPR1_tag TIM0_CH3_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH3_CNT_tag TIM0_CH3_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls0_TIM0_CH3_ECNT_tag TIM0_CH3_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls0_TIM0_CH3_CNTS_tag TIM0_CH3_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls0_TIM0_CH3_TDUC_tag TIM0_CH3_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls0_TIM0_CH3_TDUV_tag TIM0_CH3_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls0_TIM0_CH3_FLT_RE_tag TIM0_CH3_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls0_TIM0_CH3_FLT_FE_tag TIM0_CH3_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls0_TIM0_CH3_CTRL_tag TIM0_CH3_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls0_TIM0_CH3_ECTRL_tag TIM0_CH3_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls0_TIM0_CH3_IRQ_NOTIFY_tag TIM0_CH3_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TIM0_CH3_IRQ_EN_tag TIM0_CH3_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TIM0_CH3_IRQ_FORCINT_tag TIM0_CH3_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TIM0_CH3_IRQ_MODE_tag TIM0_CH3_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls0_TIM0_CH3_EIRQ_EN_tag TIM0_CH3_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved24[64];
  GTM_gtm_cls0_TIM0_CH4_GPR0_tag TIM0_CH4_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH4_GPR1_tag TIM0_CH4_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH4_CNT_tag TIM0_CH4_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls0_TIM0_CH4_ECNT_tag TIM0_CH4_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls0_TIM0_CH4_CNTS_tag TIM0_CH4_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls0_TIM0_CH4_TDUC_tag TIM0_CH4_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls0_TIM0_CH4_TDUV_tag TIM0_CH4_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls0_TIM0_CH4_FLT_RE_tag TIM0_CH4_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls0_TIM0_CH4_FLT_FE_tag TIM0_CH4_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls0_TIM0_CH4_CTRL_tag TIM0_CH4_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls0_TIM0_CH4_ECTRL_tag TIM0_CH4_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls0_TIM0_CH4_IRQ_NOTIFY_tag TIM0_CH4_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TIM0_CH4_IRQ_EN_tag TIM0_CH4_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TIM0_CH4_IRQ_FORCINT_tag TIM0_CH4_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TIM0_CH4_IRQ_MODE_tag TIM0_CH4_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls0_TIM0_CH4_EIRQ_EN_tag TIM0_CH4_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved25[64];
  GTM_gtm_cls0_TIM0_CH5_GPR0_tag TIM0_CH5_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH5_GPR1_tag TIM0_CH5_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH5_CNT_tag TIM0_CH5_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls0_TIM0_CH5_ECNT_tag TIM0_CH5_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls0_TIM0_CH5_CNTS_tag TIM0_CH5_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls0_TIM0_CH5_TDUC_tag TIM0_CH5_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls0_TIM0_CH5_TDUV_tag TIM0_CH5_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls0_TIM0_CH5_FLT_RE_tag TIM0_CH5_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls0_TIM0_CH5_FLT_FE_tag TIM0_CH5_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls0_TIM0_CH5_CTRL_tag TIM0_CH5_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls0_TIM0_CH5_ECTRL_tag TIM0_CH5_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls0_TIM0_CH5_IRQ_NOTIFY_tag TIM0_CH5_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TIM0_CH5_IRQ_EN_tag TIM0_CH5_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TIM0_CH5_IRQ_FORCINT_tag TIM0_CH5_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TIM0_CH5_IRQ_MODE_tag TIM0_CH5_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls0_TIM0_CH5_EIRQ_EN_tag TIM0_CH5_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved26[64];
  GTM_gtm_cls0_TIM0_CH6_GPR0_tag TIM0_CH6_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH6_GPR1_tag TIM0_CH6_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH6_CNT_tag TIM0_CH6_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls0_TIM0_CH6_ECNT_tag TIM0_CH6_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls0_TIM0_CH6_CNTS_tag TIM0_CH6_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls0_TIM0_CH6_TDUC_tag TIM0_CH6_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls0_TIM0_CH6_TDUV_tag TIM0_CH6_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls0_TIM0_CH6_FLT_RE_tag TIM0_CH6_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls0_TIM0_CH6_FLT_FE_tag TIM0_CH6_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls0_TIM0_CH6_CTRL_tag TIM0_CH6_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls0_TIM0_CH6_ECTRL_tag TIM0_CH6_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls0_TIM0_CH6_IRQ_NOTIFY_tag TIM0_CH6_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TIM0_CH6_IRQ_EN_tag TIM0_CH6_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TIM0_CH6_IRQ_FORCINT_tag TIM0_CH6_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TIM0_CH6_IRQ_MODE_tag TIM0_CH6_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls0_TIM0_CH6_EIRQ_EN_tag TIM0_CH6_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved27[64];
  GTM_gtm_cls0_TIM0_CH7_GPR0_tag TIM0_CH7_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH7_GPR1_tag TIM0_CH7_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls0_TIM0_CH7_CNT_tag TIM0_CH7_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls0_TIM0_CH7_ECNT_tag TIM0_CH7_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls0_TIM0_CH7_CNTS_tag TIM0_CH7_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls0_TIM0_CH7_TDUC_tag TIM0_CH7_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls0_TIM0_CH7_TDUV_tag TIM0_CH7_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls0_TIM0_CH7_FLT_RE_tag TIM0_CH7_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls0_TIM0_CH7_FLT_FE_tag TIM0_CH7_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls0_TIM0_CH7_CTRL_tag TIM0_CH7_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls0_TIM0_CH7_ECTRL_tag TIM0_CH7_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls0_TIM0_CH7_IRQ_NOTIFY_tag TIM0_CH7_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TIM0_CH7_IRQ_EN_tag TIM0_CH7_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TIM0_CH7_IRQ_FORCINT_tag TIM0_CH7_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TIM0_CH7_IRQ_MODE_tag TIM0_CH7_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls0_TIM0_CH7_EIRQ_EN_tag TIM0_CH7_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved28[64];
  GTM_gtm_cls0_TIM0_INP_VAL_tag TIM0_INP_VAL; /* TIM[i] input value observation register */
  GTM_gtm_cls0_TIM0_IN_SRC_tag TIM0_IN_SRC; /* TIM[i] AUX IN source selection register */
  GTM_gtm_cls0_TIM0_RST_tag TIM0_RST;  /* TIM[i] global software reset register */
  uint8_t GTM_gtm_cls0_reserved29[1012];
  GTM_gtm_cls0_TOM0_CH0_CTRL_tag TOM0_CH0_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH0_SR0_tag TOM0_CH0_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH0_SR1_tag TOM0_CH0_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH0_CM0_tag TOM0_CH0_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH0_CM1_tag TOM0_CH0_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH0_CN0_tag TOM0_CH0_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH0_STAT_tag TOM0_CH0_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH0_IRQ_NOTIFY_tag TOM0_CH0_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH0_IRQ_EN_tag TOM0_CH0_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH0_IRQ_FORCINT_tag TOM0_CH0_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH0_IRQ_MODE_tag TOM0_CH0_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved30[4];
  GTM_gtm_cls0_TOM0_CH0_CTRL_SR_tag TOM0_CH0_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved31[12];
  GTM_gtm_cls0_TOM0_CH1_CTRL_tag TOM0_CH1_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH1_SR0_tag TOM0_CH1_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH1_SR1_tag TOM0_CH1_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH1_CM0_tag TOM0_CH1_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH1_CM1_tag TOM0_CH1_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH1_CN0_tag TOM0_CH1_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH1_STAT_tag TOM0_CH1_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH1_IRQ_NOTIFY_tag TOM0_CH1_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH1_IRQ_EN_tag TOM0_CH1_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH1_IRQ_FORCINT_tag TOM0_CH1_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH1_IRQ_MODE_tag TOM0_CH1_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved32[4];
  GTM_gtm_cls0_TOM0_CH1_CTRL_SR_tag TOM0_CH1_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved33[12];
  GTM_gtm_cls0_TOM0_CH2_CTRL_tag TOM0_CH2_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH2_SR0_tag TOM0_CH2_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH2_SR1_tag TOM0_CH2_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH2_CM0_tag TOM0_CH2_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH2_CM1_tag TOM0_CH2_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH2_CN0_tag TOM0_CH2_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH2_STAT_tag TOM0_CH2_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH2_IRQ_NOTIFY_tag TOM0_CH2_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH2_IRQ_EN_tag TOM0_CH2_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH2_IRQ_FORCINT_tag TOM0_CH2_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH2_IRQ_MODE_tag TOM0_CH2_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved34[4];
  GTM_gtm_cls0_TOM0_CH2_CTRL_SR_tag TOM0_CH2_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved35[12];
  GTM_gtm_cls0_TOM0_CH3_CTRL_tag TOM0_CH3_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH3_SR0_tag TOM0_CH3_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH3_SR1_tag TOM0_CH3_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH3_CM0_tag TOM0_CH3_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH3_CM1_tag TOM0_CH3_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH3_CN0_tag TOM0_CH3_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH3_STAT_tag TOM0_CH3_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH3_IRQ_NOTIFY_tag TOM0_CH3_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH3_IRQ_EN_tag TOM0_CH3_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH3_IRQ_FORCINT_tag TOM0_CH3_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH3_IRQ_MODE_tag TOM0_CH3_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved36[4];
  GTM_gtm_cls0_TOM0_CH3_CTRL_SR_tag TOM0_CH3_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved37[12];
  GTM_gtm_cls0_TOM0_CH4_CTRL_tag TOM0_CH4_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH4_SR0_tag TOM0_CH4_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH4_SR1_tag TOM0_CH4_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH4_CM0_tag TOM0_CH4_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH4_CM1_tag TOM0_CH4_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH4_CN0_tag TOM0_CH4_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH4_STAT_tag TOM0_CH4_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH4_IRQ_NOTIFY_tag TOM0_CH4_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH4_IRQ_EN_tag TOM0_CH4_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH4_IRQ_FORCINT_tag TOM0_CH4_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH4_IRQ_MODE_tag TOM0_CH4_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved38[4];
  GTM_gtm_cls0_TOM0_CH4_CTRL_SR_tag TOM0_CH4_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved39[12];
  GTM_gtm_cls0_TOM0_CH5_CTRL_tag TOM0_CH5_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH5_SR0_tag TOM0_CH5_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH5_SR1_tag TOM0_CH5_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH5_CM0_tag TOM0_CH5_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH5_CM1_tag TOM0_CH5_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH5_CN0_tag TOM0_CH5_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH5_STAT_tag TOM0_CH5_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH5_IRQ_NOTIFY_tag TOM0_CH5_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH5_IRQ_EN_tag TOM0_CH5_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH5_IRQ_FORCINT_tag TOM0_CH5_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH5_IRQ_MODE_tag TOM0_CH5_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved40[4];
  GTM_gtm_cls0_TOM0_CH5_CTRL_SR_tag TOM0_CH5_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved41[12];
  GTM_gtm_cls0_TOM0_CH6_CTRL_tag TOM0_CH6_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH6_SR0_tag TOM0_CH6_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH6_SR1_tag TOM0_CH6_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH6_CM0_tag TOM0_CH6_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH6_CM1_tag TOM0_CH6_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH6_CN0_tag TOM0_CH6_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH6_STAT_tag TOM0_CH6_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH6_IRQ_NOTIFY_tag TOM0_CH6_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH6_IRQ_EN_tag TOM0_CH6_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH6_IRQ_FORCINT_tag TOM0_CH6_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH6_IRQ_MODE_tag TOM0_CH6_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved42[4];
  GTM_gtm_cls0_TOM0_CH6_CTRL_SR_tag TOM0_CH6_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved43[12];
  GTM_gtm_cls0_TOM0_CH7_CTRL_tag TOM0_CH7_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH7_SR0_tag TOM0_CH7_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH7_SR1_tag TOM0_CH7_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH7_CM0_tag TOM0_CH7_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH7_CM1_tag TOM0_CH7_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH7_CN0_tag TOM0_CH7_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH7_STAT_tag TOM0_CH7_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH7_IRQ_NOTIFY_tag TOM0_CH7_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH7_IRQ_EN_tag TOM0_CH7_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH7_IRQ_FORCINT_tag TOM0_CH7_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH7_IRQ_MODE_tag TOM0_CH7_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved44[4];
  GTM_gtm_cls0_TOM0_CH7_CTRL_SR_tag TOM0_CH7_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved45[12];
  GTM_gtm_cls0_TOM0_CH8_CTRL_tag TOM0_CH8_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH8_SR0_tag TOM0_CH8_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH8_SR1_tag TOM0_CH8_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH8_CM0_tag TOM0_CH8_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH8_CM1_tag TOM0_CH8_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH8_CN0_tag TOM0_CH8_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH8_STAT_tag TOM0_CH8_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH8_IRQ_NOTIFY_tag TOM0_CH8_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH8_IRQ_EN_tag TOM0_CH8_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH8_IRQ_FORCINT_tag TOM0_CH8_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH8_IRQ_MODE_tag TOM0_CH8_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved46[4];
  GTM_gtm_cls0_TOM0_CH8_CTRL_SR_tag TOM0_CH8_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved47[12];
  GTM_gtm_cls0_TOM0_CH9_CTRL_tag TOM0_CH9_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH9_SR0_tag TOM0_CH9_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH9_SR1_tag TOM0_CH9_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH9_CM0_tag TOM0_CH9_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH9_CM1_tag TOM0_CH9_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH9_CN0_tag TOM0_CH9_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH9_STAT_tag TOM0_CH9_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH9_IRQ_NOTIFY_tag TOM0_CH9_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH9_IRQ_EN_tag TOM0_CH9_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH9_IRQ_FORCINT_tag TOM0_CH9_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH9_IRQ_MODE_tag TOM0_CH9_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved48[4];
  GTM_gtm_cls0_TOM0_CH9_CTRL_SR_tag TOM0_CH9_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved49[12];
  GTM_gtm_cls0_TOM0_CH10_CTRL_tag TOM0_CH10_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH10_SR0_tag TOM0_CH10_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH10_SR1_tag TOM0_CH10_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH10_CM0_tag TOM0_CH10_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH10_CM1_tag TOM0_CH10_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH10_CN0_tag TOM0_CH10_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH10_STAT_tag TOM0_CH10_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH10_IRQ_NOTIFY_tag TOM0_CH10_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH10_IRQ_EN_tag TOM0_CH10_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH10_IRQ_FORCINT_tag TOM0_CH10_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH10_IRQ_MODE_tag TOM0_CH10_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved50[4];
  GTM_gtm_cls0_TOM0_CH10_CTRL_SR_tag TOM0_CH10_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved51[12];
  GTM_gtm_cls0_TOM0_CH11_CTRL_tag TOM0_CH11_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH11_SR0_tag TOM0_CH11_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH11_SR1_tag TOM0_CH11_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH11_CM0_tag TOM0_CH11_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH11_CM1_tag TOM0_CH11_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH11_CN0_tag TOM0_CH11_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH11_STAT_tag TOM0_CH11_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH11_IRQ_NOTIFY_tag TOM0_CH11_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH11_IRQ_EN_tag TOM0_CH11_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH11_IRQ_FORCINT_tag TOM0_CH11_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH11_IRQ_MODE_tag TOM0_CH11_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved52[4];
  GTM_gtm_cls0_TOM0_CH11_CTRL_SR_tag TOM0_CH11_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved53[12];
  GTM_gtm_cls0_TOM0_CH12_CTRL_tag TOM0_CH12_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH12_SR0_tag TOM0_CH12_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH12_SR1_tag TOM0_CH12_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH12_CM0_tag TOM0_CH12_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH12_CM1_tag TOM0_CH12_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH12_CN0_tag TOM0_CH12_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH12_STAT_tag TOM0_CH12_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH12_IRQ_NOTIFY_tag TOM0_CH12_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH12_IRQ_EN_tag TOM0_CH12_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH12_IRQ_FORCINT_tag TOM0_CH12_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH12_IRQ_MODE_tag TOM0_CH12_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved54[4];
  GTM_gtm_cls0_TOM0_CH12_CTRL_SR_tag TOM0_CH12_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved55[12];
  GTM_gtm_cls0_TOM0_CH13_CTRL_tag TOM0_CH13_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH13_SR0_tag TOM0_CH13_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH13_SR1_tag TOM0_CH13_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH13_CM0_tag TOM0_CH13_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH13_CM1_tag TOM0_CH13_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH13_CN0_tag TOM0_CH13_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH13_STAT_tag TOM0_CH13_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH13_IRQ_NOTIFY_tag TOM0_CH13_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH13_IRQ_EN_tag TOM0_CH13_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH13_IRQ_FORCINT_tag TOM0_CH13_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH13_IRQ_MODE_tag TOM0_CH13_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved56[4];
  GTM_gtm_cls0_TOM0_CH13_CTRL_SR_tag TOM0_CH13_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved57[12];
  GTM_gtm_cls0_TOM0_CH14_CTRL_tag TOM0_CH14_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH14_SR0_tag TOM0_CH14_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH14_SR1_tag TOM0_CH14_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH14_CM0_tag TOM0_CH14_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH14_CM1_tag TOM0_CH14_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH14_CN0_tag TOM0_CH14_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH14_STAT_tag TOM0_CH14_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH14_IRQ_NOTIFY_tag TOM0_CH14_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH14_IRQ_EN_tag TOM0_CH14_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH14_IRQ_FORCINT_tag TOM0_CH14_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH14_IRQ_MODE_tag TOM0_CH14_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved58[4];
  GTM_gtm_cls0_TOM0_CH14_CTRL_SR_tag TOM0_CH14_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved59[12];
  GTM_gtm_cls0_TOM0_CH15_CTRL_tag TOM0_CH15_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls0_TOM0_CH15_SR0_tag TOM0_CH15_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_TOM0_CH15_SR1_tag TOM0_CH15_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls0_TOM0_CH15_CM0_tag TOM0_CH15_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_TOM0_CH15_CM1_tag TOM0_CH15_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls0_TOM0_CH15_CN0_tag TOM0_CH15_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls0_TOM0_CH15_STAT_tag TOM0_CH15_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls0_TOM0_CH15_IRQ_NOTIFY_tag TOM0_CH15_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_TOM0_CH15_IRQ_EN_tag TOM0_CH15_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_TOM0_CH15_IRQ_FORCINT_tag TOM0_CH15_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls0_TOM0_CH15_IRQ_MODE_tag TOM0_CH15_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls0_reserved60[4];
  GTM_gtm_cls0_TOM0_CH15_CTRL_SR_tag TOM0_CH15_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved61[60];
  GTM_gtm_cls0_TOM0_TGC0_GLB_CTRL_tag TOM0_TGC0_GLB_CTRL; /* TOM[i] TGC [g] global control register */
  GTM_gtm_cls0_TOM0_TGC0_ACT_TB_tag TOM0_TGC0_ACT_TB; /* TOM[i] TGC [g] action time base register */
  GTM_gtm_cls0_TOM0_TGC0_FUPD_CTRL_tag TOM0_TGC0_FUPD_CTRL; /* TOM[i] TGC [g] force update control register */
  GTM_gtm_cls0_TOM0_TGC0_INT_TRIG_tag TOM0_TGC0_INT_TRIG; /* TOM[i] TGC [g] internal trigger control register */
  uint8_t GTM_gtm_cls0_reserved62[48];
  GTM_gtm_cls0_TOM0_TGC0_ENDIS_CTRL_tag TOM0_TGC0_ENDIS_CTRL; /* TOM[i] TGC [g] enable/disable control register */
  GTM_gtm_cls0_TOM0_TGC0_ENDIS_STAT_tag TOM0_TGC0_ENDIS_STAT; /* TOM[i] TGC [g] enable/disable status register */
  GTM_gtm_cls0_TOM0_TGC0_OUTEN_CTRL_tag TOM0_TGC0_OUTEN_CTRL; /* TOM[i] TGC [g] output enable control register */
  GTM_gtm_cls0_TOM0_TGC0_OUTEN_STAT_tag TOM0_TGC0_OUTEN_STAT; /* TOM[i] TGC [g] output enable status register */
  uint8_t GTM_gtm_cls0_reserved63[48];
  GTM_gtm_cls0_TOM0_TGC1_GLB_CTRL_tag TOM0_TGC1_GLB_CTRL; /* TOM[i] TGC [g] global control register */
  GTM_gtm_cls0_TOM0_TGC1_ACT_TB_tag TOM0_TGC1_ACT_TB; /* TOM[i] TGC [g] action time base register */
  GTM_gtm_cls0_TOM0_TGC1_FUPD_CTRL_tag TOM0_TGC1_FUPD_CTRL; /* TOM[i] TGC [g] force update control register */
  GTM_gtm_cls0_TOM0_TGC1_INT_TRIG_tag TOM0_TGC1_INT_TRIG; /* TOM[i] TGC [g] internal trigger control register */
  uint8_t GTM_gtm_cls0_reserved64[48];
  GTM_gtm_cls0_TOM0_TGC1_ENDIS_CTRL_tag TOM0_TGC1_ENDIS_CTRL; /* TOM[i] TGC [g] enable/disable control register */
  GTM_gtm_cls0_TOM0_TGC1_ENDIS_STAT_tag TOM0_TGC1_ENDIS_STAT; /* TOM[i] TGC [g] enable/disable status register */
  GTM_gtm_cls0_TOM0_TGC1_OUTEN_CTRL_tag TOM0_TGC1_OUTEN_CTRL; /* TOM[i] TGC [g] output enable control register */
  GTM_gtm_cls0_TOM0_TGC1_OUTEN_STAT_tag TOM0_TGC1_OUTEN_STAT; /* TOM[i] TGC [g] output enable status register */
  uint8_t GTM_gtm_cls0_reserved65[768];
  GTM_gtm_cls0_ATOM0_CH0_RDADDR_tag ATOM0_CH0_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls0_ATOM0_CH0_CTRL_tag ATOM0_CH0_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls0_ATOM0_CH0_SR0_tag ATOM0_CH0_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH0_SR1_tag ATOM0_CH0_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH0_CM0_tag ATOM0_CH0_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH0_CM1_tag ATOM0_CH0_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH0_CN0_tag ATOM0_CH0_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls0_ATOM0_CH0_STAT_tag ATOM0_CH0_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls0_ATOM0_CH0_IRQ_NOTIFY_tag ATOM0_CH0_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_ATOM0_CH0_IRQ_EN_tag ATOM0_CH0_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_ATOM0_CH0_IRQ_FORCINT_tag ATOM0_CH0_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls0_ATOM0_CH0_IRQ_MODE_tag ATOM0_CH0_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved66[4];
  GTM_gtm_cls0_ATOM0_CH0_CTRL_SR_tag ATOM0_CH0_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved67[72];
  GTM_gtm_cls0_ATOM0_CH1_RDADDR_tag ATOM0_CH1_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls0_ATOM0_CH1_CTRL_tag ATOM0_CH1_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls0_ATOM0_CH1_SR0_tag ATOM0_CH1_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH1_SR1_tag ATOM0_CH1_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH1_CM0_tag ATOM0_CH1_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH1_CM1_tag ATOM0_CH1_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH1_CN0_tag ATOM0_CH1_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls0_ATOM0_CH1_STAT_tag ATOM0_CH1_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls0_ATOM0_CH1_IRQ_NOTIFY_tag ATOM0_CH1_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_ATOM0_CH1_IRQ_EN_tag ATOM0_CH1_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_ATOM0_CH1_IRQ_FORCINT_tag ATOM0_CH1_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls0_ATOM0_CH1_IRQ_MODE_tag ATOM0_CH1_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved68[4];
  GTM_gtm_cls0_ATOM0_CH1_CTRL_SR_tag ATOM0_CH1_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved69[72];
  GTM_gtm_cls0_ATOM0_CH2_RDADDR_tag ATOM0_CH2_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls0_ATOM0_CH2_CTRL_tag ATOM0_CH2_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls0_ATOM0_CH2_SR0_tag ATOM0_CH2_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH2_SR1_tag ATOM0_CH2_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH2_CM0_tag ATOM0_CH2_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH2_CM1_tag ATOM0_CH2_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH2_CN0_tag ATOM0_CH2_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls0_ATOM0_CH2_STAT_tag ATOM0_CH2_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls0_ATOM0_CH2_IRQ_NOTIFY_tag ATOM0_CH2_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_ATOM0_CH2_IRQ_EN_tag ATOM0_CH2_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_ATOM0_CH2_IRQ_FORCINT_tag ATOM0_CH2_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls0_ATOM0_CH2_IRQ_MODE_tag ATOM0_CH2_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved70[4];
  GTM_gtm_cls0_ATOM0_CH2_CTRL_SR_tag ATOM0_CH2_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved71[72];
  GTM_gtm_cls0_ATOM0_CH3_RDADDR_tag ATOM0_CH3_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls0_ATOM0_CH3_CTRL_tag ATOM0_CH3_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls0_ATOM0_CH3_SR0_tag ATOM0_CH3_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH3_SR1_tag ATOM0_CH3_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH3_CM0_tag ATOM0_CH3_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH3_CM1_tag ATOM0_CH3_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH3_CN0_tag ATOM0_CH3_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls0_ATOM0_CH3_STAT_tag ATOM0_CH3_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls0_ATOM0_CH3_IRQ_NOTIFY_tag ATOM0_CH3_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_ATOM0_CH3_IRQ_EN_tag ATOM0_CH3_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_ATOM0_CH3_IRQ_FORCINT_tag ATOM0_CH3_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls0_ATOM0_CH3_IRQ_MODE_tag ATOM0_CH3_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved72[4];
  GTM_gtm_cls0_ATOM0_CH3_CTRL_SR_tag ATOM0_CH3_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved73[72];
  GTM_gtm_cls0_ATOM0_CH4_RDADDR_tag ATOM0_CH4_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls0_ATOM0_CH4_CTRL_tag ATOM0_CH4_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls0_ATOM0_CH4_SR0_tag ATOM0_CH4_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH4_SR1_tag ATOM0_CH4_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH4_CM0_tag ATOM0_CH4_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH4_CM1_tag ATOM0_CH4_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH4_CN0_tag ATOM0_CH4_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls0_ATOM0_CH4_STAT_tag ATOM0_CH4_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls0_ATOM0_CH4_IRQ_NOTIFY_tag ATOM0_CH4_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_ATOM0_CH4_IRQ_EN_tag ATOM0_CH4_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_ATOM0_CH4_IRQ_FORCINT_tag ATOM0_CH4_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls0_ATOM0_CH4_IRQ_MODE_tag ATOM0_CH4_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved74[4];
  GTM_gtm_cls0_ATOM0_CH4_CTRL_SR_tag ATOM0_CH4_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved75[72];
  GTM_gtm_cls0_ATOM0_CH5_RDADDR_tag ATOM0_CH5_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls0_ATOM0_CH5_CTRL_tag ATOM0_CH5_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls0_ATOM0_CH5_SR0_tag ATOM0_CH5_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH5_SR1_tag ATOM0_CH5_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH5_CM0_tag ATOM0_CH5_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH5_CM1_tag ATOM0_CH5_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH5_CN0_tag ATOM0_CH5_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls0_ATOM0_CH5_STAT_tag ATOM0_CH5_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls0_ATOM0_CH5_IRQ_NOTIFY_tag ATOM0_CH5_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_ATOM0_CH5_IRQ_EN_tag ATOM0_CH5_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_ATOM0_CH5_IRQ_FORCINT_tag ATOM0_CH5_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls0_ATOM0_CH5_IRQ_MODE_tag ATOM0_CH5_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved76[4];
  GTM_gtm_cls0_ATOM0_CH5_CTRL_SR_tag ATOM0_CH5_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved77[72];
  GTM_gtm_cls0_ATOM0_CH6_RDADDR_tag ATOM0_CH6_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls0_ATOM0_CH6_CTRL_tag ATOM0_CH6_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls0_ATOM0_CH6_SR0_tag ATOM0_CH6_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH6_SR1_tag ATOM0_CH6_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH6_CM0_tag ATOM0_CH6_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH6_CM1_tag ATOM0_CH6_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH6_CN0_tag ATOM0_CH6_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls0_ATOM0_CH6_STAT_tag ATOM0_CH6_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls0_ATOM0_CH6_IRQ_NOTIFY_tag ATOM0_CH6_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_ATOM0_CH6_IRQ_EN_tag ATOM0_CH6_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_ATOM0_CH6_IRQ_FORCINT_tag ATOM0_CH6_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls0_ATOM0_CH6_IRQ_MODE_tag ATOM0_CH6_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved78[4];
  GTM_gtm_cls0_ATOM0_CH6_CTRL_SR_tag ATOM0_CH6_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved79[72];
  GTM_gtm_cls0_ATOM0_CH7_RDADDR_tag ATOM0_CH7_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls0_ATOM0_CH7_CTRL_tag ATOM0_CH7_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls0_ATOM0_CH7_SR0_tag ATOM0_CH7_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH7_SR1_tag ATOM0_CH7_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls0_ATOM0_CH7_CM0_tag ATOM0_CH7_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH7_CM1_tag ATOM0_CH7_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls0_ATOM0_CH7_CN0_tag ATOM0_CH7_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls0_ATOM0_CH7_STAT_tag ATOM0_CH7_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls0_ATOM0_CH7_IRQ_NOTIFY_tag ATOM0_CH7_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_ATOM0_CH7_IRQ_EN_tag ATOM0_CH7_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_ATOM0_CH7_IRQ_FORCINT_tag ATOM0_CH7_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls0_ATOM0_CH7_IRQ_MODE_tag ATOM0_CH7_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved80[4];
  GTM_gtm_cls0_ATOM0_CH7_CTRL_SR_tag ATOM0_CH7_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls0_reserved81[136];
  GTM_gtm_cls0_ATOM0_AGC_GLB_CTRL_tag ATOM0_AGC_GLB_CTRL; /* ATOM[i] AGC global control register */
  GTM_gtm_cls0_ATOM0_AGC_ENDIS_CTRL_tag ATOM0_AGC_ENDIS_CTRL; /* ATOM[i] AGC enable/disable control register */
  GTM_gtm_cls0_ATOM0_AGC_ENDIS_STAT_tag ATOM0_AGC_ENDIS_STAT; /* ATOM[i] AGC enable/disable status register */
  GTM_gtm_cls0_ATOM0_AGC_ACT_TB_tag ATOM0_AGC_ACT_TB; /* ATOM[i] AGC action time base register */
  GTM_gtm_cls0_ATOM0_AGC_OUTEN_CTRL_tag ATOM0_AGC_OUTEN_CTRL; /* ATOM[i] AGC output enable control register */
  GTM_gtm_cls0_ATOM0_AGC_OUTEN_STAT_tag ATOM0_AGC_OUTEN_STAT; /* ATOM[i] AGC output enable status register */
  GTM_gtm_cls0_ATOM0_AGC_FUPD_CTRL_tag ATOM0_AGC_FUPD_CTRL; /* ATOM[i] AGC force update control register */
  GTM_gtm_cls0_ATOM0_AGC_INT_TRIG_tag ATOM0_AGC_INT_TRIG; /* ATOM[i] AGC internal trigger control register */
  uint8_t GTM_gtm_cls0_reserved82[928];
  GTM_gtm_cls0_MCS0_CH0_R0_tag MCS0_CH0_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH0_R1_tag MCS0_CH0_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH0_R2_tag MCS0_CH0_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH0_R3_tag MCS0_CH0_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH0_R4_tag MCS0_CH0_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH0_R5_tag MCS0_CH0_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH0_R6_tag MCS0_CH0_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH0_R7_tag MCS0_CH0_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH0_CTRL_tag MCS0_CH0_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls0_MCS0_CH0_ACB_tag MCS0_CH0_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls0_reserved83[20];
  GTM_gtm_cls0_MCS0_CH0_MHB_tag MCS0_CH0_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls0_reserved84[160];
  GTM_gtm_cls0_MCS0_CH0_PC_tag MCS0_CH0_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls0_MCS0_CH0_IRQ_NOTIFY_tag MCS0_CH0_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls0_MCS0_CH0_IRQ_EN_tag MCS0_CH0_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls0_MCS0_CH0_IRQ_FORCINT_tag MCS0_CH0_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls0_MCS0_CH0_IRQ_MODE_tag MCS0_CH0_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls0_MCS0_CH0_EIRQ_EN_tag MCS0_CH0_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved85[8];
  GTM_gtm_cls0_MCS0_CH1_R0_tag MCS0_CH1_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH1_R1_tag MCS0_CH1_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH1_R2_tag MCS0_CH1_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH1_R3_tag MCS0_CH1_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH1_R4_tag MCS0_CH1_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH1_R5_tag MCS0_CH1_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH1_R6_tag MCS0_CH1_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH1_R7_tag MCS0_CH1_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH1_CTRL_tag MCS0_CH1_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls0_MCS0_CH1_ACB_tag MCS0_CH1_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls0_reserved86[20];
  GTM_gtm_cls0_MCS0_CH1_MHB_tag MCS0_CH1_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls0_reserved87[160];
  GTM_gtm_cls0_MCS0_CH1_PC_tag MCS0_CH1_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls0_MCS0_CH1_IRQ_NOTIFY_tag MCS0_CH1_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls0_MCS0_CH1_IRQ_EN_tag MCS0_CH1_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls0_MCS0_CH1_IRQ_FORCINT_tag MCS0_CH1_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls0_MCS0_CH1_IRQ_MODE_tag MCS0_CH1_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls0_MCS0_CH1_EIRQ_EN_tag MCS0_CH1_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved88[8];
  GTM_gtm_cls0_MCS0_CH2_R0_tag MCS0_CH2_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH2_R1_tag MCS0_CH2_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH2_R2_tag MCS0_CH2_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH2_R3_tag MCS0_CH2_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH2_R4_tag MCS0_CH2_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH2_R5_tag MCS0_CH2_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH2_R6_tag MCS0_CH2_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH2_R7_tag MCS0_CH2_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH2_CTRL_tag MCS0_CH2_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls0_MCS0_CH2_ACB_tag MCS0_CH2_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls0_reserved89[20];
  GTM_gtm_cls0_MCS0_CH2_MHB_tag MCS0_CH2_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls0_reserved90[160];
  GTM_gtm_cls0_MCS0_CH2_PC_tag MCS0_CH2_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls0_MCS0_CH2_IRQ_NOTIFY_tag MCS0_CH2_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls0_MCS0_CH2_IRQ_EN_tag MCS0_CH2_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls0_MCS0_CH2_IRQ_FORCINT_tag MCS0_CH2_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls0_MCS0_CH2_IRQ_MODE_tag MCS0_CH2_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls0_MCS0_CH2_EIRQ_EN_tag MCS0_CH2_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved91[8];
  GTM_gtm_cls0_MCS0_CH3_R0_tag MCS0_CH3_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH3_R1_tag MCS0_CH3_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH3_R2_tag MCS0_CH3_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH3_R3_tag MCS0_CH3_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH3_R4_tag MCS0_CH3_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH3_R5_tag MCS0_CH3_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH3_R6_tag MCS0_CH3_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH3_R7_tag MCS0_CH3_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH3_CTRL_tag MCS0_CH3_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls0_MCS0_CH3_ACB_tag MCS0_CH3_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls0_reserved92[20];
  GTM_gtm_cls0_MCS0_CH3_MHB_tag MCS0_CH3_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls0_reserved93[160];
  GTM_gtm_cls0_MCS0_CH3_PC_tag MCS0_CH3_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls0_MCS0_CH3_IRQ_NOTIFY_tag MCS0_CH3_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls0_MCS0_CH3_IRQ_EN_tag MCS0_CH3_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls0_MCS0_CH3_IRQ_FORCINT_tag MCS0_CH3_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls0_MCS0_CH3_IRQ_MODE_tag MCS0_CH3_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls0_MCS0_CH3_EIRQ_EN_tag MCS0_CH3_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved94[8];
  GTM_gtm_cls0_MCS0_CH4_R0_tag MCS0_CH4_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH4_R1_tag MCS0_CH4_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH4_R2_tag MCS0_CH4_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH4_R3_tag MCS0_CH4_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH4_R4_tag MCS0_CH4_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH4_R5_tag MCS0_CH4_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH4_R6_tag MCS0_CH4_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH4_R7_tag MCS0_CH4_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH4_CTRL_tag MCS0_CH4_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls0_MCS0_CH4_ACB_tag MCS0_CH4_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls0_reserved95[20];
  GTM_gtm_cls0_MCS0_CH4_MHB_tag MCS0_CH4_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls0_reserved96[160];
  GTM_gtm_cls0_MCS0_CH4_PC_tag MCS0_CH4_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls0_MCS0_CH4_IRQ_NOTIFY_tag MCS0_CH4_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls0_MCS0_CH4_IRQ_EN_tag MCS0_CH4_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls0_MCS0_CH4_IRQ_FORCINT_tag MCS0_CH4_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls0_MCS0_CH4_IRQ_MODE_tag MCS0_CH4_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls0_MCS0_CH4_EIRQ_EN_tag MCS0_CH4_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved97[8];
  GTM_gtm_cls0_MCS0_CH5_R0_tag MCS0_CH5_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH5_R1_tag MCS0_CH5_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH5_R2_tag MCS0_CH5_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH5_R3_tag MCS0_CH5_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH5_R4_tag MCS0_CH5_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH5_R5_tag MCS0_CH5_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH5_R6_tag MCS0_CH5_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH5_R7_tag MCS0_CH5_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH5_CTRL_tag MCS0_CH5_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls0_MCS0_CH5_ACB_tag MCS0_CH5_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls0_reserved98[20];
  GTM_gtm_cls0_MCS0_CH5_MHB_tag MCS0_CH5_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls0_reserved99[160];
  GTM_gtm_cls0_MCS0_CH5_PC_tag MCS0_CH5_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls0_MCS0_CH5_IRQ_NOTIFY_tag MCS0_CH5_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls0_MCS0_CH5_IRQ_EN_tag MCS0_CH5_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls0_MCS0_CH5_IRQ_FORCINT_tag MCS0_CH5_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls0_MCS0_CH5_IRQ_MODE_tag MCS0_CH5_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls0_MCS0_CH5_EIRQ_EN_tag MCS0_CH5_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved100[8];
  GTM_gtm_cls0_MCS0_CH6_R0_tag MCS0_CH6_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH6_R1_tag MCS0_CH6_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH6_R2_tag MCS0_CH6_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH6_R3_tag MCS0_CH6_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH6_R4_tag MCS0_CH6_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH6_R5_tag MCS0_CH6_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH6_R6_tag MCS0_CH6_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH6_R7_tag MCS0_CH6_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH6_CTRL_tag MCS0_CH6_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls0_MCS0_CH6_ACB_tag MCS0_CH6_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls0_reserved101[20];
  GTM_gtm_cls0_MCS0_CH6_MHB_tag MCS0_CH6_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls0_reserved102[160];
  GTM_gtm_cls0_MCS0_CH6_PC_tag MCS0_CH6_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls0_MCS0_CH6_IRQ_NOTIFY_tag MCS0_CH6_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls0_MCS0_CH6_IRQ_EN_tag MCS0_CH6_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls0_MCS0_CH6_IRQ_FORCINT_tag MCS0_CH6_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls0_MCS0_CH6_IRQ_MODE_tag MCS0_CH6_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls0_MCS0_CH6_EIRQ_EN_tag MCS0_CH6_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved103[8];
  GTM_gtm_cls0_MCS0_CH7_R0_tag MCS0_CH7_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH7_R1_tag MCS0_CH7_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH7_R2_tag MCS0_CH7_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH7_R3_tag MCS0_CH7_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH7_R4_tag MCS0_CH7_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH7_R5_tag MCS0_CH7_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH7_R6_tag MCS0_CH7_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH7_R7_tag MCS0_CH7_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls0_MCS0_CH7_CTRL_tag MCS0_CH7_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls0_MCS0_CH7_ACB_tag MCS0_CH7_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls0_reserved104[20];
  GTM_gtm_cls0_MCS0_CH7_MHB_tag MCS0_CH7_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls0_reserved105[160];
  GTM_gtm_cls0_MCS0_CH7_PC_tag MCS0_CH7_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls0_MCS0_CH7_IRQ_NOTIFY_tag MCS0_CH7_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls0_MCS0_CH7_IRQ_EN_tag MCS0_CH7_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls0_MCS0_CH7_IRQ_FORCINT_tag MCS0_CH7_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls0_MCS0_CH7_IRQ_MODE_tag MCS0_CH7_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls0_MCS0_CH7_EIRQ_EN_tag MCS0_CH7_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved106[1584];
  GTM_gtm_cls0_MCS0_CTRG_tag MCS0_CTRG; /* MCS[i] clear trigger control register */
  GTM_gtm_cls0_MCS0_STRG_tag MCS0_STRG; /* MCS[i] set trigger control register */
  uint8_t GTM_gtm_cls0_reserved107[208];
  GTM_gtm_cls0_MCS0_CTRL_STAT_tag MCS0_CTRL_STAT; /* MCS[i] control and status register */
  GTM_gtm_cls0_MCS0_RESET_tag MCS0_RESET; /* MCS[i] reset register */
  GTM_gtm_cls0_MCS0_CAT_tag MCS0_CAT;  /* MCS[i] cancel ARU transfer instruction */
  GTM_gtm_cls0_MCS0_CWT_tag MCS0_CWT;  /* MCS[i] cancel waiting instruction */
  GTM_gtm_cls0_MCS0_ERR_tag MCS0_ERR;  /* MCS[i] error register */
  uint8_t GTM_gtm_cls0_reserved108[8];
  GTM_gtm_cls0_MCS0_REG_PROT_tag MCS0_REG_PROT; /* MCS[i] write protection register */
  GTM_gtm_cls0_MCS0_SINT_IRQ_NOTIFY_tag MCS0_SINT_IRQ_NOTIFY; /* MCS[i] shared interrupt notification register */
  GTM_gtm_cls0_MCS0_SINT_IRQ_EN_tag MCS0_SINT_IRQ_EN; /* MCS[i] shared interrupt enable register */
  GTM_gtm_cls0_MCS0_SINT_IRQ_FORCINT_tag MCS0_SINT_IRQ_FORCINT; /* MCS[i] force shared interrupt register */
  GTM_gtm_cls0_MCS0_SINT_IRQ_MODE_tag MCS0_SINT_IRQ_MODE; /* MCS[i] shared interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved109[16];
  GTM_gtm_cls0_MCS0_HBP0_CTRL_tag MCS0_HBP0_CTRL; /* MCS[i] hardware break point h control register */
  GTM_gtm_cls0_MCS0_HBP0_PATTERN_tag MCS0_HBP0_PATTERN; /* MCS[i] hardware break point pattern register */
  GTM_gtm_cls0_MCS0_HBP0_STATUS_tag MCS0_HBP0_STATUS; /* MCS[i] hardware break point status register */
  GTM_gtm_cls0_MCS0_HBP0_IRQ_NOTIFY_tag MCS0_HBP0_IRQ_NOTIFY; /* MCS[i] hardware break point interrupt notification register */
  GTM_gtm_cls0_MCS0_HBP0_IRQ_EN_tag MCS0_HBP0_IRQ_EN; /* MCS[i] hardware break point interrupt enable register */
  GTM_gtm_cls0_MCS0_HBP0_IRQ_FORCINT_tag MCS0_HBP0_IRQ_FORCINT; /* MCS[i] force hardware break point interrupt register */
  GTM_gtm_cls0_MCS0_HBP0_IRQ_MODE_tag MCS0_HBP0_IRQ_MODE; /* MCS[i] break point h interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved110[4];
  GTM_gtm_cls0_MCS0_HBP1_CTRL_tag MCS0_HBP1_CTRL; /* MCS[i] hardware break point h control register */
  GTM_gtm_cls0_MCS0_HBP1_PATTERN_tag MCS0_HBP1_PATTERN; /* MCS[i] hardware break point pattern register */
  GTM_gtm_cls0_MCS0_HBP1_STATUS_tag MCS0_HBP1_STATUS; /* MCS[i] hardware break point status register */
  GTM_gtm_cls0_MCS0_HBP1_IRQ_NOTIFY_tag MCS0_HBP1_IRQ_NOTIFY; /* MCS[i] hardware break point interrupt notification register */
  GTM_gtm_cls0_MCS0_HBP1_IRQ_EN_tag MCS0_HBP1_IRQ_EN; /* MCS[i] hardware break point interrupt enable register */
  GTM_gtm_cls0_MCS0_HBP1_IRQ_FORCINT_tag MCS0_HBP1_IRQ_FORCINT; /* MCS[i] force hardware break point interrupt register */
  GTM_gtm_cls0_MCS0_HBP1_IRQ_MODE_tag MCS0_HBP1_IRQ_MODE; /* MCS[i] break point h interrupt mode configuration register */
  uint8_t GTM_gtm_cls0_reserved111[4228];
  GTM_gtm_cls0_CCM0_ARP0_CTRL_tag CCM0_ARP0_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP0_PROT_tag CCM0_ARP0_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP1_CTRL_tag CCM0_ARP1_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP1_PROT_tag CCM0_ARP1_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP2_CTRL_tag CCM0_ARP2_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP2_PROT_tag CCM0_ARP2_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP3_CTRL_tag CCM0_ARP3_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP3_PROT_tag CCM0_ARP3_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP4_CTRL_tag CCM0_ARP4_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP4_PROT_tag CCM0_ARP4_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP5_CTRL_tag CCM0_ARP5_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP5_PROT_tag CCM0_ARP5_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP6_CTRL_tag CCM0_ARP6_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP6_PROT_tag CCM0_ARP6_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP7_CTRL_tag CCM0_ARP7_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP7_PROT_tag CCM0_ARP7_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP8_CTRL_tag CCM0_ARP8_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP8_PROT_tag CCM0_ARP8_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls0_CCM0_ARP9_CTRL_tag CCM0_ARP9_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls0_CCM0_ARP9_PROT_tag CCM0_ARP9_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  uint8_t GTM_gtm_cls0_reserved112[388];
  GTM_gtm_cls0_CCM0_HW_CONF2_tag CCM0_HW_CONF2; /* CCM[i] 2. Hardware Configuration Register */
  GTM_gtm_cls0_CCM0_AEIM_STA_tag CCM0_AEIM_STA; /* CCM[i] MCS Bus Master Status Register */
  GTM_gtm_cls0_CCM0_HW_CONF_tag CCM0_HW_CONF; /* CCM[i] Hardware Configuration Register */
  GTM_gtm_cls0_CCM0_TIM_AUX_IN_SRC_tag CCM0_TIM_AUX_IN_SRC; /* CCM[i] TIM AUX Input Source Register */
  GTM_gtm_cls0_CCM0_EXT_CAP_EN_tag CCM0_EXT_CAP_EN; /* CCM[i] External Capture Enable Register */
  GTM_gtm_cls0_CCM0_TOM_OUT_tag CCM0_TOM_OUT; /* CCM[i] TOM Output Register */
  GTM_gtm_cls0_CCM0_ATOM_OUT_tag CCM0_ATOM_OUT; /* CCM[i] ATOM Output Register */
  GTM_gtm_cls0_CCM0_CMU_CLK_CFG_tag CCM0_CMU_CLK_CFG; /* CCM[i] CMU Clock Configuration Register */
  GTM_gtm_cls0_CCM0_CMU_FXCLK_CFG_tag CCM0_CMU_FXCLK_CFG; /* CCM[i] CMU Fixed Clock Configuration Register */
  GTM_gtm_cls0_CCM0_CFG_tag CCM0_CFG;  /* CCM[i] Configuration Register */
  GTM_gtm_cls0_CCM0_PROT_tag CCM0_PROT; /* CCM[i] Protection Register */
  uint8_t GTM_gtm_cls0_reserved113[768];
  GTM_gtm_cls0_CDTM0_DTM4_CTRL_tag CDTM0_DTM4_CTRL; /* CDTM[i]_DTM[d] global configuration and control register */
  GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL1_tag CDTM0_DTM4_CH_CTRL1; /* CDTM[i]_DTM[d] channel control register 1 */
  GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL2_tag CDTM0_DTM4_CH_CTRL2; /* CDTM[i]_DTM[d] channel control register 2 */
  GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL2_SR_tag CDTM0_DTM4_CH_CTRL2_SR; /* CDTM[i] DTM[j] channel control register 2 shadow */
  GTM_gtm_cls0_CDTM0_DTM4_PS_CTRL_tag CDTM0_DTM4_PS_CTRL; /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  GTM_gtm_cls0_CDTM0_DTM4_CH_DTV_tag CDTM0_DTM4_CH_DTV[4]; /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  GTM_gtm_cls0_CDTM0_DTM4_CH_SR_tag CDTM0_DTM4_CH_SR; /* CDTM[i]_DTM[d] channel shadow register */
  GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL3_tag CDTM0_DTM4_CH_CTRL3; /* CDTM[i]_DTM[d] channel control register 3 */
  GTM_gtm_cls0_CDTM0_DTM4_CTRL2_tag CDTM0_DTM4_CTRL2; /* CDTM[i]_DTM[d] global configuration and control register 2 */
  GTM_gtm_cls0_CDTM0_DTM4_CH0_DTV_SR_tag CDTM0_DTM4_CH0_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls0_CDTM0_DTM4_CH1_DTV_SR_tag CDTM0_DTM4_CH1_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls0_CDTM0_DTM4_CH2_DTV_SR_tag CDTM0_DTM4_CH2_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls0_CDTM0_DTM4_CH3_DTV_SR_tag CDTM0_DTM4_CH3_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls0_CDTM0_DTM5_CTRL_tag CDTM0_DTM5_CTRL; /* CDTM[i]_DTM[d] global configuration and control register */
  GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL1_tag CDTM0_DTM5_CH_CTRL1; /* CDTM[i]_DTM[d] channel control register 1 */
  GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL2_tag CDTM0_DTM5_CH_CTRL2; /* CDTM[i]_DTM[d] channel control register 2 */
  GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL2_SR_tag CDTM0_DTM5_CH_CTRL2_SR; /* CDTM[i] DTM[j] channel control register 2 shadow */
  GTM_gtm_cls0_CDTM0_DTM5_PS_CTRL_tag CDTM0_DTM5_PS_CTRL; /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  GTM_gtm_cls0_CDTM0_DTM5_CH_DTV_tag CDTM0_DTM5_CH_DTV[4]; /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  GTM_gtm_cls0_CDTM0_DTM5_CH_SR_tag CDTM0_DTM5_CH_SR; /* CDTM[i]_DTM[d] channel shadow register */
  GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL3_tag CDTM0_DTM5_CH_CTRL3; /* CDTM[i]_DTM[d] channel control register 3 */
  GTM_gtm_cls0_CDTM0_DTM5_CTRL2_tag CDTM0_DTM5_CTRL2; /* CDTM[i]_DTM[d] global configuration and control register 2 */
  GTM_gtm_cls0_CDTM0_DTM5_CH0_DTV_SR_tag CDTM0_DTM5_CH0_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls0_CDTM0_DTM5_CH1_DTV_SR_tag CDTM0_DTM5_CH1_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls0_CDTM0_DTM5_CH2_DTV_SR_tag CDTM0_DTM5_CH2_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls0_CDTM0_DTM5_CH3_DTV_SR_tag CDTM0_DTM5_CH3_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  uint8_t GTM_gtm_cls0_reserved114[640];
  GTM_gtm_cls0_F2A0_CH0_ARU_RD_FIFO_tag F2A0_CH0_ARU_RD_FIFO; /* F2A[i] stream [x] ARU read register */
  GTM_gtm_cls0_F2A0_CH1_ARU_RD_FIFO_tag F2A0_CH1_ARU_RD_FIFO; /* F2A[i] stream [x] ARU read register */
  GTM_gtm_cls0_F2A0_CH2_ARU_RD_FIFO_tag F2A0_CH2_ARU_RD_FIFO; /* F2A[i] stream [x] ARU read register */
  GTM_gtm_cls0_F2A0_CH3_ARU_RD_FIFO_tag F2A0_CH3_ARU_RD_FIFO; /* F2A[i] stream [x] ARU read register */
  GTM_gtm_cls0_F2A0_CH4_ARU_RD_FIFO_tag F2A0_CH4_ARU_RD_FIFO; /* F2A[i] stream [x] ARU read register */
  GTM_gtm_cls0_F2A0_CH5_ARU_RD_FIFO_tag F2A0_CH5_ARU_RD_FIFO; /* F2A[i] stream [x] ARU read register */
  GTM_gtm_cls0_F2A0_CH6_ARU_RD_FIFO_tag F2A0_CH6_ARU_RD_FIFO; /* F2A[i] stream [x] ARU read register */
  GTM_gtm_cls0_F2A0_CH7_ARU_RD_FIFO_tag F2A0_CH7_ARU_RD_FIFO; /* F2A[i] stream [x] ARU read register */
  GTM_gtm_cls0_F2A0_CH0_STR_CFG_tag F2A0_CH0_STR_CFG; /* F2A[i] stream [x] configuration register */
  GTM_gtm_cls0_F2A0_CH1_STR_CFG_tag F2A0_CH1_STR_CFG; /* F2A[i] stream [x] configuration register */
  GTM_gtm_cls0_F2A0_CH2_STR_CFG_tag F2A0_CH2_STR_CFG; /* F2A[i] stream [x] configuration register */
  GTM_gtm_cls0_F2A0_CH3_STR_CFG_tag F2A0_CH3_STR_CFG; /* F2A[i] stream [x] configuration register */
  GTM_gtm_cls0_F2A0_CH4_STR_CFG_tag F2A0_CH4_STR_CFG; /* F2A[i] stream [x] configuration register */
  GTM_gtm_cls0_F2A0_CH5_STR_CFG_tag F2A0_CH5_STR_CFG; /* F2A[i] stream [x] configuration register */
  GTM_gtm_cls0_F2A0_CH6_STR_CFG_tag F2A0_CH6_STR_CFG; /* F2A[i] stream [x] configuration register */
  GTM_gtm_cls0_F2A0_CH7_STR_CFG_tag F2A0_CH7_STR_CFG; /* F2A[i] stream [x] configuration register */
  GTM_gtm_cls0_F2A0_ENABLE_tag F2A0_ENABLE; /* F2A[i] stream activation register */
  GTM_gtm_cls0_F2A0_CTRL_tag F2A0_CTRL; /* F2A[i] stream control register */
  uint8_t GTM_gtm_cls0_reserved115[56];
  GTM_gtm_cls0_AFD0_CH0_BUF_ACC_tag AFD0_CH0_BUF_ACC; /* AFD [i] FIFO [x] buffer access register */
  uint8_t GTM_gtm_cls0_reserved116[12];
  GTM_gtm_cls0_AFD0_CH1_BUF_ACC_tag AFD0_CH1_BUF_ACC; /* AFD [i] FIFO [x] buffer access register */
  uint8_t GTM_gtm_cls0_reserved117[12];
  GTM_gtm_cls0_AFD0_CH2_BUF_ACC_tag AFD0_CH2_BUF_ACC; /* AFD [i] FIFO [x] buffer access register */
  uint8_t GTM_gtm_cls0_reserved118[12];
  GTM_gtm_cls0_AFD0_CH3_BUF_ACC_tag AFD0_CH3_BUF_ACC; /* AFD [i] FIFO [x] buffer access register */
  uint8_t GTM_gtm_cls0_reserved119[12];
  GTM_gtm_cls0_AFD0_CH4_BUF_ACC_tag AFD0_CH4_BUF_ACC; /* AFD [i] FIFO [x] buffer access register */
  uint8_t GTM_gtm_cls0_reserved120[12];
  GTM_gtm_cls0_AFD0_CH5_BUF_ACC_tag AFD0_CH5_BUF_ACC; /* AFD [i] FIFO [x] buffer access register */
  uint8_t GTM_gtm_cls0_reserved121[12];
  GTM_gtm_cls0_AFD0_CH6_BUF_ACC_tag AFD0_CH6_BUF_ACC; /* AFD [i] FIFO [x] buffer access register */
  uint8_t GTM_gtm_cls0_reserved122[12];
  GTM_gtm_cls0_AFD0_CH7_BUF_ACC_tag AFD0_CH7_BUF_ACC; /* AFD [i] FIFO [x] buffer access register */
  uint8_t GTM_gtm_cls0_reserved123[268];
  GTM_gtm_cls0_FIFO0_CH0_CTRL_tag FIFO0_CH0_CTRL; /* FIFO[i] channel [x] control register */
  GTM_gtm_cls0_FIFO0_CH0_END_ADDR_tag FIFO0_CH0_END_ADDR; /* FIFO[i] channel [x] end address register */
  GTM_gtm_cls0_FIFO0_CH0_START_ADDR_tag FIFO0_CH0_START_ADDR; /* FIFO[i] channel [x] start address register */
  GTM_gtm_cls0_FIFO0_CH0_UPPER_WM_tag FIFO0_CH0_UPPER_WM; /* FIFO[i] channel [x] upper watermark register */
  GTM_gtm_cls0_FIFO0_CH0_LOWER_WM_tag FIFO0_CH0_LOWER_WM; /* FIFO[i] channel [x] lower watermark register */
  GTM_gtm_cls0_FIFO0_CH0_STATUS_tag FIFO0_CH0_STATUS; /* FIFO[i] channel [x] status register */
  GTM_gtm_cls0_FIFO0_CH0_FILL_LEVEL_tag FIFO0_CH0_FILL_LEVEL; /* FIFO[i] channel [x] fill level register */
  GTM_gtm_cls0_FIFO0_CH0_WR_PTR_tag FIFO0_CH0_WR_PTR; /* FIFO[i] channel [x] write pointer register */
  GTM_gtm_cls0_FIFO0_CH0_RD_PTR_tag FIFO0_CH0_RD_PTR; /* FIFO[i] channel [x] read pointer register */
  GTM_gtm_cls0_FIFO0_CH0_IRQ_NOTIFY_tag FIFO0_CH0_IRQ_NOTIFY; /* FIFO[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_FIFO0_CH0_IRQ_EN_tag FIFO0_CH0_IRQ_EN; /* FIFO[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_FIFO0_CH0_IRQ_FORCINT_tag FIFO0_CH0_IRQ_FORCINT; /* FIFO[i] channel [x] force interrupt register */
  GTM_gtm_cls0_FIFO0_CH0_IRQ_MODE_tag FIFO0_CH0_IRQ_MODE; /* FIFO[i] channel [x] interrupt mode control register */
  GTM_gtm_cls0_FIFO0_CH0_EIRQ_EN_tag FIFO0_CH0_EIRQ_EN; /* FIFO[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved124[8];
  GTM_gtm_cls0_FIFO0_CH1_CTRL_tag FIFO0_CH1_CTRL; /* FIFO[i] channel [x] control register */
  GTM_gtm_cls0_FIFO0_CH1_END_ADDR_tag FIFO0_CH1_END_ADDR; /* FIFO[i] channel [x] end address register */
  GTM_gtm_cls0_FIFO0_CH1_START_ADDR_tag FIFO0_CH1_START_ADDR; /* FIFO[i] channel [x] start address register */
  GTM_gtm_cls0_FIFO0_CH1_UPPER_WM_tag FIFO0_CH1_UPPER_WM; /* FIFO[i] channel [x] upper watermark register */
  GTM_gtm_cls0_FIFO0_CH1_LOWER_WM_tag FIFO0_CH1_LOWER_WM; /* FIFO[i] channel [x] lower watermark register */
  GTM_gtm_cls0_FIFO0_CH1_STATUS_tag FIFO0_CH1_STATUS; /* FIFO[i] channel [x] status register */
  GTM_gtm_cls0_FIFO0_CH1_FILL_LEVEL_tag FIFO0_CH1_FILL_LEVEL; /* FIFO[i] channel [x] fill level register */
  GTM_gtm_cls0_FIFO0_CH1_WR_PTR_tag FIFO0_CH1_WR_PTR; /* FIFO[i] channel [x] write pointer register */
  GTM_gtm_cls0_FIFO0_CH1_RD_PTR_tag FIFO0_CH1_RD_PTR; /* FIFO[i] channel [x] read pointer register */
  GTM_gtm_cls0_FIFO0_CH1_IRQ_NOTIFY_tag FIFO0_CH1_IRQ_NOTIFY; /* FIFO[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_FIFO0_CH1_IRQ_EN_tag FIFO0_CH1_IRQ_EN; /* FIFO[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_FIFO0_CH1_IRQ_FORCINT_tag FIFO0_CH1_IRQ_FORCINT; /* FIFO[i] channel [x] force interrupt register */
  GTM_gtm_cls0_FIFO0_CH1_IRQ_MODE_tag FIFO0_CH1_IRQ_MODE; /* FIFO[i] channel [x] interrupt mode control register */
  GTM_gtm_cls0_FIFO0_CH1_EIRQ_EN_tag FIFO0_CH1_EIRQ_EN; /* FIFO[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved125[8];
  GTM_gtm_cls0_FIFO0_CH2_CTRL_tag FIFO0_CH2_CTRL; /* FIFO[i] channel [x] control register */
  GTM_gtm_cls0_FIFO0_CH2_END_ADDR_tag FIFO0_CH2_END_ADDR; /* FIFO[i] channel [x] end address register */
  GTM_gtm_cls0_FIFO0_CH2_START_ADDR_tag FIFO0_CH2_START_ADDR; /* FIFO[i] channel [x] start address register */
  GTM_gtm_cls0_FIFO0_CH2_UPPER_WM_tag FIFO0_CH2_UPPER_WM; /* FIFO[i] channel [x] upper watermark register */
  GTM_gtm_cls0_FIFO0_CH2_LOWER_WM_tag FIFO0_CH2_LOWER_WM; /* FIFO[i] channel [x] lower watermark register */
  GTM_gtm_cls0_FIFO0_CH2_STATUS_tag FIFO0_CH2_STATUS; /* FIFO[i] channel [x] status register */
  GTM_gtm_cls0_FIFO0_CH2_FILL_LEVEL_tag FIFO0_CH2_FILL_LEVEL; /* FIFO[i] channel [x] fill level register */
  GTM_gtm_cls0_FIFO0_CH2_WR_PTR_tag FIFO0_CH2_WR_PTR; /* FIFO[i] channel [x] write pointer register */
  GTM_gtm_cls0_FIFO0_CH2_RD_PTR_tag FIFO0_CH2_RD_PTR; /* FIFO[i] channel [x] read pointer register */
  GTM_gtm_cls0_FIFO0_CH2_IRQ_NOTIFY_tag FIFO0_CH2_IRQ_NOTIFY; /* FIFO[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_FIFO0_CH2_IRQ_EN_tag FIFO0_CH2_IRQ_EN; /* FIFO[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_FIFO0_CH2_IRQ_FORCINT_tag FIFO0_CH2_IRQ_FORCINT; /* FIFO[i] channel [x] force interrupt register */
  GTM_gtm_cls0_FIFO0_CH2_IRQ_MODE_tag FIFO0_CH2_IRQ_MODE; /* FIFO[i] channel [x] interrupt mode control register */
  GTM_gtm_cls0_FIFO0_CH2_EIRQ_EN_tag FIFO0_CH2_EIRQ_EN; /* FIFO[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved126[8];
  GTM_gtm_cls0_FIFO0_CH3_CTRL_tag FIFO0_CH3_CTRL; /* FIFO[i] channel [x] control register */
  GTM_gtm_cls0_FIFO0_CH3_END_ADDR_tag FIFO0_CH3_END_ADDR; /* FIFO[i] channel [x] end address register */
  GTM_gtm_cls0_FIFO0_CH3_START_ADDR_tag FIFO0_CH3_START_ADDR; /* FIFO[i] channel [x] start address register */
  GTM_gtm_cls0_FIFO0_CH3_UPPER_WM_tag FIFO0_CH3_UPPER_WM; /* FIFO[i] channel [x] upper watermark register */
  GTM_gtm_cls0_FIFO0_CH3_LOWER_WM_tag FIFO0_CH3_LOWER_WM; /* FIFO[i] channel [x] lower watermark register */
  GTM_gtm_cls0_FIFO0_CH3_STATUS_tag FIFO0_CH3_STATUS; /* FIFO[i] channel [x] status register */
  GTM_gtm_cls0_FIFO0_CH3_FILL_LEVEL_tag FIFO0_CH3_FILL_LEVEL; /* FIFO[i] channel [x] fill level register */
  GTM_gtm_cls0_FIFO0_CH3_WR_PTR_tag FIFO0_CH3_WR_PTR; /* FIFO[i] channel [x] write pointer register */
  GTM_gtm_cls0_FIFO0_CH3_RD_PTR_tag FIFO0_CH3_RD_PTR; /* FIFO[i] channel [x] read pointer register */
  GTM_gtm_cls0_FIFO0_CH3_IRQ_NOTIFY_tag FIFO0_CH3_IRQ_NOTIFY; /* FIFO[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_FIFO0_CH3_IRQ_EN_tag FIFO0_CH3_IRQ_EN; /* FIFO[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_FIFO0_CH3_IRQ_FORCINT_tag FIFO0_CH3_IRQ_FORCINT; /* FIFO[i] channel [x] force interrupt register */
  GTM_gtm_cls0_FIFO0_CH3_IRQ_MODE_tag FIFO0_CH3_IRQ_MODE; /* FIFO[i] channel [x] interrupt mode control register */
  GTM_gtm_cls0_FIFO0_CH3_EIRQ_EN_tag FIFO0_CH3_EIRQ_EN; /* FIFO[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved127[8];
  GTM_gtm_cls0_FIFO0_CH4_CTRL_tag FIFO0_CH4_CTRL; /* FIFO[i] channel [x] control register */
  GTM_gtm_cls0_FIFO0_CH4_END_ADDR_tag FIFO0_CH4_END_ADDR; /* FIFO[i] channel [x] end address register */
  GTM_gtm_cls0_FIFO0_CH4_START_ADDR_tag FIFO0_CH4_START_ADDR; /* FIFO[i] channel [x] start address register */
  GTM_gtm_cls0_FIFO0_CH4_UPPER_WM_tag FIFO0_CH4_UPPER_WM; /* FIFO[i] channel [x] upper watermark register */
  GTM_gtm_cls0_FIFO0_CH4_LOWER_WM_tag FIFO0_CH4_LOWER_WM; /* FIFO[i] channel [x] lower watermark register */
  GTM_gtm_cls0_FIFO0_CH4_STATUS_tag FIFO0_CH4_STATUS; /* FIFO[i] channel [x] status register */
  GTM_gtm_cls0_FIFO0_CH4_FILL_LEVEL_tag FIFO0_CH4_FILL_LEVEL; /* FIFO[i] channel [x] fill level register */
  GTM_gtm_cls0_FIFO0_CH4_WR_PTR_tag FIFO0_CH4_WR_PTR; /* FIFO[i] channel [x] write pointer register */
  GTM_gtm_cls0_FIFO0_CH4_RD_PTR_tag FIFO0_CH4_RD_PTR; /* FIFO[i] channel [x] read pointer register */
  GTM_gtm_cls0_FIFO0_CH4_IRQ_NOTIFY_tag FIFO0_CH4_IRQ_NOTIFY; /* FIFO[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_FIFO0_CH4_IRQ_EN_tag FIFO0_CH4_IRQ_EN; /* FIFO[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_FIFO0_CH4_IRQ_FORCINT_tag FIFO0_CH4_IRQ_FORCINT; /* FIFO[i] channel [x] force interrupt register */
  GTM_gtm_cls0_FIFO0_CH4_IRQ_MODE_tag FIFO0_CH4_IRQ_MODE; /* FIFO[i] channel [x] interrupt mode control register */
  GTM_gtm_cls0_FIFO0_CH4_EIRQ_EN_tag FIFO0_CH4_EIRQ_EN; /* FIFO[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved128[8];
  GTM_gtm_cls0_FIFO0_CH5_CTRL_tag FIFO0_CH5_CTRL; /* FIFO[i] channel [x] control register */
  GTM_gtm_cls0_FIFO0_CH5_END_ADDR_tag FIFO0_CH5_END_ADDR; /* FIFO[i] channel [x] end address register */
  GTM_gtm_cls0_FIFO0_CH5_START_ADDR_tag FIFO0_CH5_START_ADDR; /* FIFO[i] channel [x] start address register */
  GTM_gtm_cls0_FIFO0_CH5_UPPER_WM_tag FIFO0_CH5_UPPER_WM; /* FIFO[i] channel [x] upper watermark register */
  GTM_gtm_cls0_FIFO0_CH5_LOWER_WM_tag FIFO0_CH5_LOWER_WM; /* FIFO[i] channel [x] lower watermark register */
  GTM_gtm_cls0_FIFO0_CH5_STATUS_tag FIFO0_CH5_STATUS; /* FIFO[i] channel [x] status register */
  GTM_gtm_cls0_FIFO0_CH5_FILL_LEVEL_tag FIFO0_CH5_FILL_LEVEL; /* FIFO[i] channel [x] fill level register */
  GTM_gtm_cls0_FIFO0_CH5_WR_PTR_tag FIFO0_CH5_WR_PTR; /* FIFO[i] channel [x] write pointer register */
  GTM_gtm_cls0_FIFO0_CH5_RD_PTR_tag FIFO0_CH5_RD_PTR; /* FIFO[i] channel [x] read pointer register */
  GTM_gtm_cls0_FIFO0_CH5_IRQ_NOTIFY_tag FIFO0_CH5_IRQ_NOTIFY; /* FIFO[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_FIFO0_CH5_IRQ_EN_tag FIFO0_CH5_IRQ_EN; /* FIFO[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_FIFO0_CH5_IRQ_FORCINT_tag FIFO0_CH5_IRQ_FORCINT; /* FIFO[i] channel [x] force interrupt register */
  GTM_gtm_cls0_FIFO0_CH5_IRQ_MODE_tag FIFO0_CH5_IRQ_MODE; /* FIFO[i] channel [x] interrupt mode control register */
  GTM_gtm_cls0_FIFO0_CH5_EIRQ_EN_tag FIFO0_CH5_EIRQ_EN; /* FIFO[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved129[8];
  GTM_gtm_cls0_FIFO0_CH6_CTRL_tag FIFO0_CH6_CTRL; /* FIFO[i] channel [x] control register */
  GTM_gtm_cls0_FIFO0_CH6_END_ADDR_tag FIFO0_CH6_END_ADDR; /* FIFO[i] channel [x] end address register */
  GTM_gtm_cls0_FIFO0_CH6_START_ADDR_tag FIFO0_CH6_START_ADDR; /* FIFO[i] channel [x] start address register */
  GTM_gtm_cls0_FIFO0_CH6_UPPER_WM_tag FIFO0_CH6_UPPER_WM; /* FIFO[i] channel [x] upper watermark register */
  GTM_gtm_cls0_FIFO0_CH6_LOWER_WM_tag FIFO0_CH6_LOWER_WM; /* FIFO[i] channel [x] lower watermark register */
  GTM_gtm_cls0_FIFO0_CH6_STATUS_tag FIFO0_CH6_STATUS; /* FIFO[i] channel [x] status register */
  GTM_gtm_cls0_FIFO0_CH6_FILL_LEVEL_tag FIFO0_CH6_FILL_LEVEL; /* FIFO[i] channel [x] fill level register */
  GTM_gtm_cls0_FIFO0_CH6_WR_PTR_tag FIFO0_CH6_WR_PTR; /* FIFO[i] channel [x] write pointer register */
  GTM_gtm_cls0_FIFO0_CH6_RD_PTR_tag FIFO0_CH6_RD_PTR; /* FIFO[i] channel [x] read pointer register */
  GTM_gtm_cls0_FIFO0_CH6_IRQ_NOTIFY_tag FIFO0_CH6_IRQ_NOTIFY; /* FIFO[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_FIFO0_CH6_IRQ_EN_tag FIFO0_CH6_IRQ_EN; /* FIFO[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_FIFO0_CH6_IRQ_FORCINT_tag FIFO0_CH6_IRQ_FORCINT; /* FIFO[i] channel [x] force interrupt register */
  GTM_gtm_cls0_FIFO0_CH6_IRQ_MODE_tag FIFO0_CH6_IRQ_MODE; /* FIFO[i] channel [x] interrupt mode control register */
  GTM_gtm_cls0_FIFO0_CH6_EIRQ_EN_tag FIFO0_CH6_EIRQ_EN; /* FIFO[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved130[8];
  GTM_gtm_cls0_FIFO0_CH7_CTRL_tag FIFO0_CH7_CTRL; /* FIFO[i] channel [x] control register */
  GTM_gtm_cls0_FIFO0_CH7_END_ADDR_tag FIFO0_CH7_END_ADDR; /* FIFO[i] channel [x] end address register */
  GTM_gtm_cls0_FIFO0_CH7_START_ADDR_tag FIFO0_CH7_START_ADDR; /* FIFO[i] channel [x] start address register */
  GTM_gtm_cls0_FIFO0_CH7_UPPER_WM_tag FIFO0_CH7_UPPER_WM; /* FIFO[i] channel [x] upper watermark register */
  GTM_gtm_cls0_FIFO0_CH7_LOWER_WM_tag FIFO0_CH7_LOWER_WM; /* FIFO[i] channel [x] lower watermark register */
  GTM_gtm_cls0_FIFO0_CH7_STATUS_tag FIFO0_CH7_STATUS; /* FIFO[i] channel [x] status register */
  GTM_gtm_cls0_FIFO0_CH7_FILL_LEVEL_tag FIFO0_CH7_FILL_LEVEL; /* FIFO[i] channel [x] fill level register */
  GTM_gtm_cls0_FIFO0_CH7_WR_PTR_tag FIFO0_CH7_WR_PTR; /* FIFO[i] channel [x] write pointer register */
  GTM_gtm_cls0_FIFO0_CH7_RD_PTR_tag FIFO0_CH7_RD_PTR; /* FIFO[i] channel [x] read pointer register */
  GTM_gtm_cls0_FIFO0_CH7_IRQ_NOTIFY_tag FIFO0_CH7_IRQ_NOTIFY; /* FIFO[i] channel [x] interrupt notification register */
  GTM_gtm_cls0_FIFO0_CH7_IRQ_EN_tag FIFO0_CH7_IRQ_EN; /* FIFO[i] channel [x] interrupt enable register */
  GTM_gtm_cls0_FIFO0_CH7_IRQ_FORCINT_tag FIFO0_CH7_IRQ_FORCINT; /* FIFO[i] channel [x] force interrupt register */
  GTM_gtm_cls0_FIFO0_CH7_IRQ_MODE_tag FIFO0_CH7_IRQ_MODE; /* FIFO[i] channel [x] interrupt mode control register */
  GTM_gtm_cls0_FIFO0_CH7_EIRQ_EN_tag FIFO0_CH7_EIRQ_EN; /* FIFO[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls0_reserved131[8];
  GTM_gtm_cls0_SPE0_CTRL_STAT_tag SPE0_CTRL_STAT; /* SPE[i] Control Status Register */
  GTM_gtm_cls0_SPE0_PAT_tag SPE0_PAT;  /* SPE[i] Input Pattern Definition Register */
  GTM_gtm_cls0_SPE0_OUT_PAT0_tag SPE0_OUT_PAT0; /* SPE[i] Output Definition Register */
  GTM_gtm_cls0_SPE0_OUT_PAT1_tag SPE0_OUT_PAT1; /* SPE[i] Output Definition Register */
  GTM_gtm_cls0_SPE0_OUT_PAT2_tag SPE0_OUT_PAT2; /* SPE[i] Output Definition Register */
  GTM_gtm_cls0_SPE0_OUT_PAT3_tag SPE0_OUT_PAT3; /* SPE[i] Output Definition Register */
  GTM_gtm_cls0_SPE0_OUT_PAT4_tag SPE0_OUT_PAT4; /* SPE[i] Output Definition Register */
  GTM_gtm_cls0_SPE0_OUT_PAT5_tag SPE0_OUT_PAT5; /* SPE[i] Output Definition Register */
  GTM_gtm_cls0_SPE0_OUT_PAT6_tag SPE0_OUT_PAT6; /* SPE[i] Output Definition Register */
  GTM_gtm_cls0_SPE0_OUT_PAT7_tag SPE0_OUT_PAT7; /* SPE[i] Output Definition Register */
  GTM_gtm_cls0_SPE0_OUT_CTRL_tag SPE0_OUT_CTRL; /* SPE[i] Output Control Register */
  GTM_gtm_cls0_SPE0_IRQ_NOTIFY_tag SPE0_IRQ_NOTIFY; /* SPE[i] Interrupt Notification Register */
  GTM_gtm_cls0_SPE0_IRQ_EN_tag SPE0_IRQ_EN; /* SPE[i] Interrupt Enable Register */
  GTM_gtm_cls0_SPE0_IRQ_FORCINT_tag SPE0_IRQ_FORCINT; /* SPE[i] Interrupt Generation By Software */
  GTM_gtm_cls0_SPE0_IRQ_MODE_tag SPE0_IRQ_MODE; /* SPE[i] Interrupt Mode Configuration Register */
  GTM_gtm_cls0_SPE0_EIRQ_EN_tag SPE0_EIRQ_EN; /* SPE[i] Error Interrupt Enable Register */
  GTM_gtm_cls0_SPE0_REV_CNT_tag SPE0_REV_CNT; /* SPE[i] Input Revolution Counter */
  GTM_gtm_cls0_SPE0_REV_CMP_tag SPE0_REV_CMP; /* SPE[i] Revolution Counter Compare Value */
  GTM_gtm_cls0_SPE0_CTRL_STAT2_tag SPE0_CTRL_STAT2; /* SPE[i] Control Status Register 2 */
  GTM_gtm_cls0_SPE0_CMD_tag SPE0_CMD;  /* SPE[i] Command Register */
  uint8_t GTM_gtm_cls0_reserved132[944];
  GTM_gtm_cls0_AXIM0_FREE_tag AXIM0_FREE; /* AXIM[i] slot allocation status. */
  GTM_gtm_cls0_AXIM0_REQUEST_tag AXIM0_REQUEST; /* AXIM[i] slot request (allocation). */
  GTM_gtm_cls0_AXIM0_RELEASE_tag AXIM0_RELEASE; /* AXIM[i] slot release (de-allocation). */
  uint8_t GTM_gtm_cls0_reserved133[20];
  GTM_gtm_cls0_AXIM0_SLOT0_ADDR_LOW_tag AXIM0_SLOT0_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls0_reserved134[4];
  GTM_gtm_cls0_AXIM0_SLOT0_DATA_LOW_tag AXIM0_SLOT0_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls0_reserved135[4];
  GTM_gtm_cls0_AXIM0_SLOT0_CFG1_tag AXIM0_SLOT0_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls0_AXIM0_SLOT0_CFG2_tag AXIM0_SLOT0_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls0_AXIM0_SLOT0_STATUS_tag AXIM0_SLOT0_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls0_reserved136[4];
  GTM_gtm_cls0_AXIM0_SLOT1_ADDR_LOW_tag AXIM0_SLOT1_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls0_reserved137[4];
  GTM_gtm_cls0_AXIM0_SLOT1_DATA_LOW_tag AXIM0_SLOT1_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls0_reserved138[4];
  GTM_gtm_cls0_AXIM0_SLOT1_CFG1_tag AXIM0_SLOT1_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls0_AXIM0_SLOT1_CFG2_tag AXIM0_SLOT1_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls0_AXIM0_SLOT1_STATUS_tag AXIM0_SLOT1_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls0_reserved139[4];
  GTM_gtm_cls0_AXIM0_SLOT2_ADDR_LOW_tag AXIM0_SLOT2_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls0_reserved140[4];
  GTM_gtm_cls0_AXIM0_SLOT2_DATA_LOW_tag AXIM0_SLOT2_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls0_reserved141[4];
  GTM_gtm_cls0_AXIM0_SLOT2_CFG1_tag AXIM0_SLOT2_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls0_AXIM0_SLOT2_CFG2_tag AXIM0_SLOT2_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls0_AXIM0_SLOT2_STATUS_tag AXIM0_SLOT2_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls0_reserved142[4];
  GTM_gtm_cls0_AXIM0_SLOT3_ADDR_LOW_tag AXIM0_SLOT3_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls0_reserved143[4];
  GTM_gtm_cls0_AXIM0_SLOT3_DATA_LOW_tag AXIM0_SLOT3_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls0_reserved144[4];
  GTM_gtm_cls0_AXIM0_SLOT3_CFG1_tag AXIM0_SLOT3_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls0_AXIM0_SLOT3_CFG2_tag AXIM0_SLOT3_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls0_AXIM0_SLOT3_STATUS_tag AXIM0_SLOT3_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls0_reserved145[3940];
  GTM_gtm_cls0_FIFO0_MEMORY_tag FIFO0_MEMORY[1024]; /* FIFO data memory */
  uint8_t GTM_gtm_cls0_reserved146[4096];
  GTM_gtm_cls0_DPLL_CTRL_0_tag DPLL_CTRL_0; /* Control Register 0 */
  GTM_gtm_cls0_DPLL_CTRL_1_tag DPLL_CTRL_1; /* Control Register 1 */
  GTM_gtm_cls0_DPLL_CTRL_2_tag DPLL_CTRL_2; /* Action Enable Register */
  GTM_gtm_cls0_DPLL_CTRL_3_tag DPLL_CTRL_3; /* Action Enable Register */
  GTM_gtm_cls0_DPLL_CTRL_4_tag DPLL_CTRL_4; /* Action Enable Register */
  GTM_gtm_cls0_DPLL_CTRL_5_tag DPLL_CTRL_5; /* Action Enable Register */
  GTM_gtm_cls0_DPLL_ACT_STA_tag DPLL_ACT_STA; /* Action Status Register including Shadow Register */
  GTM_gtm_cls0_DPLL_OSW_tag DPLL_OSW;  /* Offset and Switch old/new Address Register */
  GTM_gtm_cls0_DPLL_AOSV_2_tag DPLL_AOSV_2; /* Address Offset Register of RAM 2 Regions */
  GTM_gtm_cls0_DPLL_APT_tag DPLL_APT;  /* Actual RAM Pointer Address for TRIGGER */
  GTM_gtm_cls0_DPLL_APS_tag DPLL_APS;  /* Actual RAM Pointer Address for STATE */
  GTM_gtm_cls0_DPLL_APT_2C_tag DPLL_APT_2C; /* Actual RAM Pointer Address for Region 2c */
  GTM_gtm_cls0_DPLL_APS_1C3_tag DPLL_APS_1C3; /* Actual RAM Pointer Address for RAM region 1c3 */
  GTM_gtm_cls0_DPLL_NUTC_tag DPLL_NUTC; /* Number of Recent TRIGGER Events used for Calculations */
  GTM_gtm_cls0_DPLL_NUSC_tag DPLL_NUSC; /* Number of Recent STATE Events used for Calculations */
  GTM_gtm_cls0_DPLL_NTI_CNT_tag DPLL_NTI_CNT; /* Number of Active TRIGGER Events to Interrupt */
  GTM_gtm_cls0_DPLL_IRQ_NOTIFY_tag DPLL_IRQ_NOTIFY; /* Interrupt Register */
  GTM_gtm_cls0_DPLL_IRQ_EN_tag DPLL_IRQ_EN; /* Interrupt Enable Register */
  GTM_gtm_cls0_DPLL_IRQ_FORCINT_tag DPLL_IRQ_FORCINT; /* Force Interrupt Register */
  GTM_gtm_cls0_DPLL_IRQ_MODE_tag DPLL_IRQ_MODE; /* Interrupt Request Mode */
  GTM_gtm_cls0_DPLL_EIRQ_EN_tag DPLL_EIRQ_EN; /* Error Interrupt Enable Register */
  uint8_t GTM_gtm_cls0_reserved147[92];
  GTM_gtm_cls0_DPLL_INC_CNT1_tag DPLL_INC_CNT1; /* Counter Value of Sent SUB_INC1 Pulses */
  GTM_gtm_cls0_DPLL_INC_CNT2_tag DPLL_INC_CNT2; /* Counter Value of sent SUB_INC2 values (for DPLL_CTRL_1.SMC=1 and DPLL_CTRL_0.RMO=1) */
  GTM_gtm_cls0_DPLL_APT_SYNC_tag DPLL_APT_SYNC; /* TRIGGER Time Stamp Field Offset at Synchronization Time */
  GTM_gtm_cls0_DPLL_APS_SYNC_tag DPLL_APS_SYNC; /* STATE Time Stamp Field Offset at Synchronization Time */
  GTM_gtm_cls0_DPLL_TBU_TS0_T_tag DPLL_TBU_TS0_T; /* Time Stamp Value for the last active TRIGGER */
  GTM_gtm_cls0_DPLL_TBU_TS0_S_tag DPLL_TBU_TS0_S; /* Time Stamp Value for the last active STATE */
  GTM_gtm_cls0_DPLL_ADD_IN_LD1_tag DPLL_ADD_IN_LD1; /* ADD_IN Value in Direct Load Mode for TRIGGER */
  GTM_gtm_cls0_DPLL_ADD_IN_LD2_tag DPLL_ADD_IN_LD2; /* ADD_IN Value in Direct Load Mode for STATE */
  uint8_t GTM_gtm_cls0_reserved148[44];
  GTM_gtm_cls0_DPLL_STATUS_tag DPLL_STATUS; /* Status Register */
  GTM_gtm_cls0_DPLL_ID_PMTR_0_tag DPLL_ID_PMTR_0; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_1_tag DPLL_ID_PMTR_1; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_2_tag DPLL_ID_PMTR_2; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_3_tag DPLL_ID_PMTR_3; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_4_tag DPLL_ID_PMTR_4; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_5_tag DPLL_ID_PMTR_5; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_6_tag DPLL_ID_PMTR_6; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_7_tag DPLL_ID_PMTR_7; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_8_tag DPLL_ID_PMTR_8; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_9_tag DPLL_ID_PMTR_9; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_10_tag DPLL_ID_PMTR_10; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_11_tag DPLL_ID_PMTR_11; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_12_tag DPLL_ID_PMTR_12; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_13_tag DPLL_ID_PMTR_13; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_14_tag DPLL_ID_PMTR_14; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_15_tag DPLL_ID_PMTR_15; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_16_tag DPLL_ID_PMTR_16; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_17_tag DPLL_ID_PMTR_17; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_18_tag DPLL_ID_PMTR_18; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_19_tag DPLL_ID_PMTR_19; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_20_tag DPLL_ID_PMTR_20; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_21_tag DPLL_ID_PMTR_21; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_22_tag DPLL_ID_PMTR_22; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_23_tag DPLL_ID_PMTR_23; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_24_tag DPLL_ID_PMTR_24; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_25_tag DPLL_ID_PMTR_25; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_26_tag DPLL_ID_PMTR_26; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_27_tag DPLL_ID_PMTR_27; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_28_tag DPLL_ID_PMTR_28; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_29_tag DPLL_ID_PMTR_29; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_30_tag DPLL_ID_PMTR_30; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  GTM_gtm_cls0_DPLL_ID_PMTR_31_tag DPLL_ID_PMTR_31; /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
  uint8_t GTM_gtm_cls0_reserved149[96];
  GTM_gtm_cls0_DPLL_CTRL_0_SHADOW_TRIGGER_tag DPLL_CTRL_0_SHADOW_TRIGGER; /* Shadow Register of DPLL_CTRL_0 controlled by an active TRIGGER Slope */
  GTM_gtm_cls0_DPLL_CTRL_0_SHADOW_STATE_tag DPLL_CTRL_0_SHADOW_STATE; /* Shadow Register of DPLL_CTRL_0 controlled by an active STATE Slope */
  GTM_gtm_cls0_DPLL_CTRL_1_SHADOW_TRIGGER_tag DPLL_CTRL_1_SHADOW_TRIGGER; /* Shadow Register of DPLL_CTRL_1 controlled by an active TRIGGER Slope */
  GTM_gtm_cls0_DPLL_CTRL_1_SHADOW_STATE_tag DPLL_CTRL_1_SHADOW_STATE; /* DPLL Shadow Register of DPLL_CTRL_1 controlled by an active STATE Slope */
  uint8_t GTM_gtm_cls0_reserved150[12];
  GTM_gtm_cls0_DPLL_RAM_INI_tag DPLL_RAM_INI; /* Register to control the RAM Initialization */
  GTM_gtm_cls0_DPLL_PSA0_tag DPLL_PSA0; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA1_tag DPLL_PSA1; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA2_tag DPLL_PSA2; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA3_tag DPLL_PSA3; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA4_tag DPLL_PSA4; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA5_tag DPLL_PSA5; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA6_tag DPLL_PSA6; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA7_tag DPLL_PSA7; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA8_tag DPLL_PSA8; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA9_tag DPLL_PSA9; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA10_tag DPLL_PSA10; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA11_tag DPLL_PSA11; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA12_tag DPLL_PSA12; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA13_tag DPLL_PSA13; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA14_tag DPLL_PSA14; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA15_tag DPLL_PSA15; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA16_tag DPLL_PSA16; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA17_tag DPLL_PSA17; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA18_tag DPLL_PSA18; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA19_tag DPLL_PSA19; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA20_tag DPLL_PSA20; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA21_tag DPLL_PSA21; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA22_tag DPLL_PSA22; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA23_tag DPLL_PSA23; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA24_tag DPLL_PSA24; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA25_tag DPLL_PSA25; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA26_tag DPLL_PSA26; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA27_tag DPLL_PSA27; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA28_tag DPLL_PSA28; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA29_tag DPLL_PSA29; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA30_tag DPLL_PSA30; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_PSA31_tag DPLL_PSA31; /* Position Request for Action [n] */
  GTM_gtm_cls0_DPLL_DLA0_tag DPLL_DLA0; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA1_tag DPLL_DLA1; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA2_tag DPLL_DLA2; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA3_tag DPLL_DLA3; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA4_tag DPLL_DLA4; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA5_tag DPLL_DLA5; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA6_tag DPLL_DLA6; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA7_tag DPLL_DLA7; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA8_tag DPLL_DLA8; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA9_tag DPLL_DLA9; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA10_tag DPLL_DLA10; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA11_tag DPLL_DLA11; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA12_tag DPLL_DLA12; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA13_tag DPLL_DLA13; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA14_tag DPLL_DLA14; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA15_tag DPLL_DLA15; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA16_tag DPLL_DLA16; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA17_tag DPLL_DLA17; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA18_tag DPLL_DLA18; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA19_tag DPLL_DLA19; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA20_tag DPLL_DLA20; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA21_tag DPLL_DLA21; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA22_tag DPLL_DLA22; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA23_tag DPLL_DLA23; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA24_tag DPLL_DLA24; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA25_tag DPLL_DLA25; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA26_tag DPLL_DLA26; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA27_tag DPLL_DLA27; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA28_tag DPLL_DLA28; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA29_tag DPLL_DLA29; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA30_tag DPLL_DLA30; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_DLA31_tag DPLL_DLA31; /* Time to React for Action [n] */
  GTM_gtm_cls0_DPLL_NA0_tag DPLL_NA0;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA1_tag DPLL_NA1;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA2_tag DPLL_NA2;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA3_tag DPLL_NA3;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA4_tag DPLL_NA4;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA5_tag DPLL_NA5;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA6_tag DPLL_NA6;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA7_tag DPLL_NA7;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA8_tag DPLL_NA8;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA9_tag DPLL_NA9;  /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA10_tag DPLL_NA10; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA11_tag DPLL_NA11; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA12_tag DPLL_NA12; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA13_tag DPLL_NA13; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA14_tag DPLL_NA14; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA15_tag DPLL_NA15; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA16_tag DPLL_NA16; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA17_tag DPLL_NA17; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA18_tag DPLL_NA18; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA19_tag DPLL_NA19; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA20_tag DPLL_NA20; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA21_tag DPLL_NA21; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA22_tag DPLL_NA22; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA23_tag DPLL_NA23; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA24_tag DPLL_NA24; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA25_tag DPLL_NA25; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA26_tag DPLL_NA26; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA27_tag DPLL_NA27; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA28_tag DPLL_NA28; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA29_tag DPLL_NA29; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA30_tag DPLL_NA30; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_NA31_tag DPLL_NA31; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA0_tag DPLL_DTA0; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA1_tag DPLL_DTA1; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA2_tag DPLL_DTA2; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA3_tag DPLL_DTA3; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA4_tag DPLL_DTA4; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA5_tag DPLL_DTA5; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA6_tag DPLL_DTA6; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA7_tag DPLL_DTA7; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA8_tag DPLL_DTA8; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA9_tag DPLL_DTA9; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA10_tag DPLL_DTA10; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA11_tag DPLL_DTA11; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA12_tag DPLL_DTA12; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA13_tag DPLL_DTA13; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA14_tag DPLL_DTA14; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA15_tag DPLL_DTA15; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA16_tag DPLL_DTA16; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA17_tag DPLL_DTA17; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA18_tag DPLL_DTA18; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA19_tag DPLL_DTA19; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA20_tag DPLL_DTA20; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA21_tag DPLL_DTA21; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA22_tag DPLL_DTA22; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA23_tag DPLL_DTA23; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA24_tag DPLL_DTA24; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA25_tag DPLL_DTA25; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA26_tag DPLL_DTA26; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA27_tag DPLL_DTA27; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA28_tag DPLL_DTA28; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA29_tag DPLL_DTA29; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA30_tag DPLL_DTA30; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_DTA31_tag DPLL_DTA31; /* Calculated Relative Time to Action [n] */
  GTM_gtm_cls0_DPLL_TS_T_tag DPLL_TS_T; /* Actual TRIGGER Time Stamp Value */
  GTM_gtm_cls0_DPLL_TS_T_OLD_tag DPLL_TS_T_OLD; /* Previous TRIGGER Time Stamp Value */
  GTM_gtm_cls0_DPLL_FTV_T_tag DPLL_FTV_T; /* Actual TRIGGER Filter value */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_0_tag DPLL_RAM1B_RSVD_0; /* DPLL RAM1B reserved data */
  GTM_gtm_cls0_DPLL_TS_S_tag DPLL_TS_S; /* Actual STATE Time Stamp Register */
  GTM_gtm_cls0_DPLL_TS_S_OLD_tag DPLL_TS_S_OLD; /* Previous STATE Time Stamp Register */
  GTM_gtm_cls0_DPLL_FTV_S_tag DPLL_FTV_S; /* Actual STATE Filter Value */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_1_tag DPLL_RAM1B_RSVD_1; /* DPLL RAM1B reserved data */
  GTM_gtm_cls0_DPLL_THMI_tag DPLL_THMI; /* TRIGGER Hold Time Min. Value */
  GTM_gtm_cls0_DPLL_THMA_tag DPLL_THMA; /* TRIGGER Hold Time Max. Value */
  GTM_gtm_cls0_DPLL_THVAL_tag DPLL_THVAL; /* Measured TRIGGER Hold Time Value */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_2_tag DPLL_RAM1B_RSVD_2; /* DPLL RAM1B reserved data */
  GTM_gtm_cls0_DPLL_TOV_tag DPLL_TOV;  /* Time Out Value of Active TRIGGER Slope (for missing TRIGGER generation) */
  GTM_gtm_cls0_DPLL_TOV_S_tag DPLL_TOV_S; /* Time Out Value of active STATE Slope (for missing STATE generation) */
  GTM_gtm_cls0_DPLL_ADD_IN_CAL1_tag DPLL_ADD_IN_CAL1; /* Calculated ADD_IN Value for SUB_INC1 Generation */
  GTM_gtm_cls0_DPLL_ADD_IN_CAL2_tag DPLL_ADD_IN_CAL2; /* Calculated ADD_IN Value for SUB_INC2 Generation */
  GTM_gtm_cls0_DPLL_MPVAL1_tag DPLL_MPVAL1; /* Missing Pulses to be Added or Subtracted Directly */
  GTM_gtm_cls0_DPLL_MPVAL2_tag DPLL_MPVAL2; /* Missing Pulses to be Added or Subtracted Directly */
  GTM_gtm_cls0_DPLL_NMB_T_TAR_tag DPLL_NMB_T_TAR; /* Target Number of Pulses to be sent in Normal Mode */
  GTM_gtm_cls0_DPLL_NMB_T_TAR_OLD_tag DPLL_NMB_T_TAR_OLD; /* Last but one Target Number of Pulses to be sent in Normal Mode */
  GTM_gtm_cls0_DPLL_NMB_S_TAR_tag DPLL_NMB_S_TAR; /* Target Number of Pulses to be sent in Emergency Mode */
  GTM_gtm_cls0_DPLL_NMB_S_TAR_OLD_tag DPLL_NMB_S_TAR_OLD; /* Last but one Target Number of Pulses to be sent in Emergency Mode */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_3_0_tag DPLL_RAM1B_RSVD_3_0; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_3_1_tag DPLL_RAM1B_RSVD_3_1; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RCDT_TX_tag DPLL_RCDT_TX; /* Reciprocal Value of the Expected Increment Duration of TRIGGER */
  GTM_gtm_cls0_DPLL_RCDT_SX_tag DPLL_RCDT_SX; /* Reciprocal Value of the Expected Increment Duration of STATE */
  GTM_gtm_cls0_DPLL_RCDT_TX_NOM_tag DPLL_RCDT_TX_NOM; /* Reciprocal Value of the Expected Nominal Increment Duration of TRIGGER */
  GTM_gtm_cls0_DPLL_RCDT_SX_NOM_tag DPLL_RCDT_SX_NOM; /* Reciprocal Value of the Expected Nominal Increment Duration of STATE */
  GTM_gtm_cls0_DPLL_RDT_T_ACT_tag DPLL_RDT_T_ACT; /* Reciprocal Value of the Last Increment of TRIGGER */
  GTM_gtm_cls0_DPLL_RDT_S_ACT_tag DPLL_RDT_S_ACT; /* Reciprocal Value of the Last Increment of STATE */
  GTM_gtm_cls0_DPLL_DT_T_ACT_tag DPLL_DT_T_ACT; /* Duration of the Last TRIGGER Increment */
  GTM_gtm_cls0_DPLL_DT_S_ACT_tag DPLL_DT_S_ACT; /* Duration of the Last STATE Increment */
  GTM_gtm_cls0_DPLL_EDT_T_tag DPLL_EDT_T; /* Difference of Prediction to Actual Value of the Last TRIGGER Increment */
  GTM_gtm_cls0_DPLL_MEDT_T_tag DPLL_MEDT_T; /* Weighted Difference of Prediction Errors of TRIGGER */
  GTM_gtm_cls0_DPLL_EDT_S_tag DPLL_EDT_S; /* Difference of Prediction to Actual Value of the Last STATE Increment */
  GTM_gtm_cls0_DPLL_MEDT_S_tag DPLL_MEDT_S; /* Weighted Difference of Prediction Errors of STATE */
  GTM_gtm_cls0_DPLL_CDT_TX_tag DPLL_CDT_TX; /* Prediction of the Actual TRIGGER Increment Duration */
  GTM_gtm_cls0_DPLL_CDT_SX_tag DPLL_CDT_SX; /* Prediction of the Actual STATE Increment Duration */
  GTM_gtm_cls0_DPLL_CDT_TX_NOM_tag DPLL_CDT_TX_NOM; /* Prediction of the Nominal TRIGGER Increment Duration */
  GTM_gtm_cls0_DPLL_CDT_SX_NOM_tag DPLL_CDT_SX_NOM; /* Prediction of the Nominal STATE Increment Duration */
  GTM_gtm_cls0_DPLL_TLR_tag DPLL_TLR;  /* TRIGGER Locking Range */
  GTM_gtm_cls0_DPLL_SLR_tag DPLL_SLR;  /* STATE Locking Range */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_0_tag DPLL_RAM1B_RSVD_4_0; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_1_tag DPLL_RAM1B_RSVD_4_1; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_2_tag DPLL_RAM1B_RSVD_4_2; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_3_tag DPLL_RAM1B_RSVD_4_3; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_4_tag DPLL_RAM1B_RSVD_4_4; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_5_tag DPLL_RAM1B_RSVD_4_5; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_6_tag DPLL_RAM1B_RSVD_4_6; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_7_tag DPLL_RAM1B_RSVD_4_7; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_8_tag DPLL_RAM1B_RSVD_4_8; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_9_tag DPLL_RAM1B_RSVD_4_9; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_10_tag DPLL_RAM1B_RSVD_4_10; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_11_tag DPLL_RAM1B_RSVD_4_11; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_12_tag DPLL_RAM1B_RSVD_4_12; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_13_tag DPLL_RAM1B_RSVD_4_13; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_14_tag DPLL_RAM1B_RSVD_4_14; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_15_tag DPLL_RAM1B_RSVD_4_15; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_16_tag DPLL_RAM1B_RSVD_4_16; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_17_tag DPLL_RAM1B_RSVD_4_17; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_18_tag DPLL_RAM1B_RSVD_4_18; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_19_tag DPLL_RAM1B_RSVD_4_19; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_20_tag DPLL_RAM1B_RSVD_4_20; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_21_tag DPLL_RAM1B_RSVD_4_21; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_PDT_0_tag DPLL_PDT_0; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_1_tag DPLL_PDT_1; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_2_tag DPLL_PDT_2; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_3_tag DPLL_PDT_3; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_4_tag DPLL_PDT_4; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_5_tag DPLL_PDT_5; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_6_tag DPLL_PDT_6; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_7_tag DPLL_PDT_7; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_8_tag DPLL_PDT_8; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_9_tag DPLL_PDT_9; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_10_tag DPLL_PDT_10; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_11_tag DPLL_PDT_11; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_12_tag DPLL_PDT_12; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_13_tag DPLL_PDT_13; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_14_tag DPLL_PDT_14; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_15_tag DPLL_PDT_15; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_16_tag DPLL_PDT_16; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_17_tag DPLL_PDT_17; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_18_tag DPLL_PDT_18; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_19_tag DPLL_PDT_19; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_20_tag DPLL_PDT_20; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_21_tag DPLL_PDT_21; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_22_tag DPLL_PDT_22; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_23_tag DPLL_PDT_23; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_24_tag DPLL_PDT_24; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_25_tag DPLL_PDT_25; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_26_tag DPLL_PDT_26; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_27_tag DPLL_PDT_27; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_28_tag DPLL_PDT_28; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_29_tag DPLL_PDT_29; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_30_tag DPLL_PDT_30; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_PDT_31_tag DPLL_PDT_31; /* Projected Increment Sum Relations for Action [n] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_0_tag DPLL_RAM1B_RSVD_5_0; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_1_tag DPLL_RAM1B_RSVD_5_1; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_2_tag DPLL_RAM1B_RSVD_5_2; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_3_tag DPLL_RAM1B_RSVD_5_3; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_4_tag DPLL_RAM1B_RSVD_5_4; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_5_tag DPLL_RAM1B_RSVD_5_5; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_6_tag DPLL_RAM1B_RSVD_5_6; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_7_tag DPLL_RAM1B_RSVD_5_7; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_8_tag DPLL_RAM1B_RSVD_5_8; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_9_tag DPLL_RAM1B_RSVD_5_9; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_10_tag DPLL_RAM1B_RSVD_5_10; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_11_tag DPLL_RAM1B_RSVD_5_11; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_12_tag DPLL_RAM1B_RSVD_5_12; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_13_tag DPLL_RAM1B_RSVD_5_13; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_14_tag DPLL_RAM1B_RSVD_5_14; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_15_tag DPLL_RAM1B_RSVD_5_15; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_MLS1_tag DPLL_MLS1; /* Calculated Number of Sub-Pulses between two nominal STATE Events for DPLL_CTRL_1.SMC = 0 */
  GTM_gtm_cls0_DPLL_MLS2_tag DPLL_MLS2; /* Calculated Number of Sub-Pulses between two nominal STATE Events for DPLL_CTRL_1.SMC=1 and RMO=1 */
  GTM_gtm_cls0_DPLL_CNT_NUM_1_tag DPLL_CNT_NUM_1; /* Number of SUB_INC1 pulses in continuous mode */
  GTM_gtm_cls0_DPLL_CNT_NUM_2_tag DPLL_CNT_NUM_2; /* Number of SUB_INC2 pulses in continuous mode */
  GTM_gtm_cls0_DPLL_PVT_tag DPLL_PVT;  /* Plausibility Value of Next TRIGGER Slope */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_0_tag DPLL_RAM1B_RSVD_6_0; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_1_tag DPLL_RAM1B_RSVD_6_1; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_2_tag DPLL_RAM1B_RSVD_6_2; /* DPLL RAM1B reserved data [k] */
  GTM_gtm_cls0_DPLL_PSTC_tag DPLL_PSTC; /* Actual Calculated Position Stamp of TRIGGER */
  GTM_gtm_cls0_DPLL_PSSC_tag DPLL_PSSC; /* Actual Calculated Position Stamp of STATE */
  GTM_gtm_cls0_DPLL_PSTM_tag DPLL_PSTM; /* Measured Position Stamp at Last TRIGGER Input */
  GTM_gtm_cls0_DPLL_PSTM_OLD_tag DPLL_PSTM_OLD; /* Measured Position Stamp at Last but one TRIGGER Input */
  GTM_gtm_cls0_DPLL_PSSM_tag DPLL_PSSM; /* Measured Position Stamp at Last STATE Input */
  GTM_gtm_cls0_DPLL_PSSM_OLD_tag DPLL_PSSM_OLD; /* Measured Position Stamp at Last but one STATE Input */
  GTM_gtm_cls0_DPLL_NMB_T_tag DPLL_NMB_T; /* Number of Pulses to be sent in Normal Mode */
  GTM_gtm_cls0_DPLL_NMB_S_tag DPLL_NMB_S; /* Number of Pulses to be sent in Emergency Mode */
  GTM_gtm_cls0_DPLL_RDT_S0_tag DPLL_RDT_S0; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S1_tag DPLL_RDT_S1; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S2_tag DPLL_RDT_S2; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S3_tag DPLL_RDT_S3; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S4_tag DPLL_RDT_S4; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S5_tag DPLL_RDT_S5; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S6_tag DPLL_RDT_S6; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S7_tag DPLL_RDT_S7; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S8_tag DPLL_RDT_S8; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S9_tag DPLL_RDT_S9; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S10_tag DPLL_RDT_S10; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S11_tag DPLL_RDT_S11; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S12_tag DPLL_RDT_S12; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S13_tag DPLL_RDT_S13; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S14_tag DPLL_RDT_S14; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S15_tag DPLL_RDT_S15; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S16_tag DPLL_RDT_S16; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S17_tag DPLL_RDT_S17; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S18_tag DPLL_RDT_S18; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S19_tag DPLL_RDT_S19; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S20_tag DPLL_RDT_S20; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S21_tag DPLL_RDT_S21; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S22_tag DPLL_RDT_S22; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S23_tag DPLL_RDT_S23; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S24_tag DPLL_RDT_S24; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S25_tag DPLL_RDT_S25; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S26_tag DPLL_RDT_S26; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S27_tag DPLL_RDT_S27; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S28_tag DPLL_RDT_S28; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S29_tag DPLL_RDT_S29; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S30_tag DPLL_RDT_S30; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S31_tag DPLL_RDT_S31; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S32_tag DPLL_RDT_S32; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S33_tag DPLL_RDT_S33; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S34_tag DPLL_RDT_S34; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S35_tag DPLL_RDT_S35; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S36_tag DPLL_RDT_S36; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S37_tag DPLL_RDT_S37; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S38_tag DPLL_RDT_S38; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S39_tag DPLL_RDT_S39; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S40_tag DPLL_RDT_S40; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S41_tag DPLL_RDT_S41; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S42_tag DPLL_RDT_S42; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S43_tag DPLL_RDT_S43; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S44_tag DPLL_RDT_S44; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S45_tag DPLL_RDT_S45; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S46_tag DPLL_RDT_S46; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S47_tag DPLL_RDT_S47; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S48_tag DPLL_RDT_S48; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S49_tag DPLL_RDT_S49; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S50_tag DPLL_RDT_S50; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S51_tag DPLL_RDT_S51; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S52_tag DPLL_RDT_S52; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S53_tag DPLL_RDT_S53; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S54_tag DPLL_RDT_S54; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S55_tag DPLL_RDT_S55; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S56_tag DPLL_RDT_S56; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S57_tag DPLL_RDT_S57; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S58_tag DPLL_RDT_S58; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S59_tag DPLL_RDT_S59; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S60_tag DPLL_RDT_S60; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S61_tag DPLL_RDT_S61; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S62_tag DPLL_RDT_S62; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_RDT_S63_tag DPLL_RDT_S63; /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S0_tag DPLL_TSF_S0; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S1_tag DPLL_TSF_S1; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S2_tag DPLL_TSF_S2; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S3_tag DPLL_TSF_S3; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S4_tag DPLL_TSF_S4; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S5_tag DPLL_TSF_S5; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S6_tag DPLL_TSF_S6; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S7_tag DPLL_TSF_S7; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S8_tag DPLL_TSF_S8; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S9_tag DPLL_TSF_S9; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S10_tag DPLL_TSF_S10; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S11_tag DPLL_TSF_S11; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S12_tag DPLL_TSF_S12; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S13_tag DPLL_TSF_S13; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S14_tag DPLL_TSF_S14; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S15_tag DPLL_TSF_S15; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S16_tag DPLL_TSF_S16; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S17_tag DPLL_TSF_S17; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S18_tag DPLL_TSF_S18; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S19_tag DPLL_TSF_S19; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S20_tag DPLL_TSF_S20; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S21_tag DPLL_TSF_S21; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S22_tag DPLL_TSF_S22; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S23_tag DPLL_TSF_S23; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S24_tag DPLL_TSF_S24; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S25_tag DPLL_TSF_S25; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S26_tag DPLL_TSF_S26; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S27_tag DPLL_TSF_S27; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S28_tag DPLL_TSF_S28; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S29_tag DPLL_TSF_S29; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S30_tag DPLL_TSF_S30; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S31_tag DPLL_TSF_S31; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S32_tag DPLL_TSF_S32; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S33_tag DPLL_TSF_S33; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S34_tag DPLL_TSF_S34; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S35_tag DPLL_TSF_S35; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S36_tag DPLL_TSF_S36; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S37_tag DPLL_TSF_S37; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S38_tag DPLL_TSF_S38; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S39_tag DPLL_TSF_S39; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S40_tag DPLL_TSF_S40; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S41_tag DPLL_TSF_S41; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S42_tag DPLL_TSF_S42; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S43_tag DPLL_TSF_S43; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S44_tag DPLL_TSF_S44; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S45_tag DPLL_TSF_S45; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S46_tag DPLL_TSF_S46; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S47_tag DPLL_TSF_S47; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S48_tag DPLL_TSF_S48; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S49_tag DPLL_TSF_S49; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S50_tag DPLL_TSF_S50; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S51_tag DPLL_TSF_S51; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S52_tag DPLL_TSF_S52; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S53_tag DPLL_TSF_S53; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S54_tag DPLL_TSF_S54; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S55_tag DPLL_TSF_S55; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S56_tag DPLL_TSF_S56; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S57_tag DPLL_TSF_S57; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S58_tag DPLL_TSF_S58; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S59_tag DPLL_TSF_S59; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S60_tag DPLL_TSF_S60; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S61_tag DPLL_TSF_S61; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S62_tag DPLL_TSF_S62; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_TSF_S63_tag DPLL_TSF_S63; /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_ADT_S0_tag DPLL_ADT_S0; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S1_tag DPLL_ADT_S1; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S2_tag DPLL_ADT_S2; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S3_tag DPLL_ADT_S3; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S4_tag DPLL_ADT_S4; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S5_tag DPLL_ADT_S5; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S6_tag DPLL_ADT_S6; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S7_tag DPLL_ADT_S7; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S8_tag DPLL_ADT_S8; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S9_tag DPLL_ADT_S9; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S10_tag DPLL_ADT_S10; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S11_tag DPLL_ADT_S11; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S12_tag DPLL_ADT_S12; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S13_tag DPLL_ADT_S13; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S14_tag DPLL_ADT_S14; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S15_tag DPLL_ADT_S15; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S16_tag DPLL_ADT_S16; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S17_tag DPLL_ADT_S17; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S18_tag DPLL_ADT_S18; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S19_tag DPLL_ADT_S19; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S20_tag DPLL_ADT_S20; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S21_tag DPLL_ADT_S21; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S22_tag DPLL_ADT_S22; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S23_tag DPLL_ADT_S23; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S24_tag DPLL_ADT_S24; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S25_tag DPLL_ADT_S25; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S26_tag DPLL_ADT_S26; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S27_tag DPLL_ADT_S27; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S28_tag DPLL_ADT_S28; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S29_tag DPLL_ADT_S29; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S30_tag DPLL_ADT_S30; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S31_tag DPLL_ADT_S31; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S32_tag DPLL_ADT_S32; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S33_tag DPLL_ADT_S33; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S34_tag DPLL_ADT_S34; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S35_tag DPLL_ADT_S35; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S36_tag DPLL_ADT_S36; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S37_tag DPLL_ADT_S37; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S38_tag DPLL_ADT_S38; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S39_tag DPLL_ADT_S39; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S40_tag DPLL_ADT_S40; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S41_tag DPLL_ADT_S41; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S42_tag DPLL_ADT_S42; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S43_tag DPLL_ADT_S43; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S44_tag DPLL_ADT_S44; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S45_tag DPLL_ADT_S45; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S46_tag DPLL_ADT_S46; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S47_tag DPLL_ADT_S47; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S48_tag DPLL_ADT_S48; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S49_tag DPLL_ADT_S49; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S50_tag DPLL_ADT_S50; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S51_tag DPLL_ADT_S51; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S52_tag DPLL_ADT_S52; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S53_tag DPLL_ADT_S53; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S54_tag DPLL_ADT_S54; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S55_tag DPLL_ADT_S55; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S56_tag DPLL_ADT_S56; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S57_tag DPLL_ADT_S57; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S58_tag DPLL_ADT_S58; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S59_tag DPLL_ADT_S59; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S60_tag DPLL_ADT_S60; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S61_tag DPLL_ADT_S61; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S62_tag DPLL_ADT_S62; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_ADT_S63_tag DPLL_ADT_S63; /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
  GTM_gtm_cls0_DPLL_DT_S0_tag DPLL_DT_S0; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S1_tag DPLL_DT_S1; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S2_tag DPLL_DT_S2; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S3_tag DPLL_DT_S3; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S4_tag DPLL_DT_S4; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S5_tag DPLL_DT_S5; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S6_tag DPLL_DT_S6; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S7_tag DPLL_DT_S7; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S8_tag DPLL_DT_S8; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S9_tag DPLL_DT_S9; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S10_tag DPLL_DT_S10; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S11_tag DPLL_DT_S11; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S12_tag DPLL_DT_S12; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S13_tag DPLL_DT_S13; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S14_tag DPLL_DT_S14; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S15_tag DPLL_DT_S15; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S16_tag DPLL_DT_S16; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S17_tag DPLL_DT_S17; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S18_tag DPLL_DT_S18; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S19_tag DPLL_DT_S19; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S20_tag DPLL_DT_S20; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S21_tag DPLL_DT_S21; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S22_tag DPLL_DT_S22; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S23_tag DPLL_DT_S23; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S24_tag DPLL_DT_S24; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S25_tag DPLL_DT_S25; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S26_tag DPLL_DT_S26; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S27_tag DPLL_DT_S27; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S28_tag DPLL_DT_S28; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S29_tag DPLL_DT_S29; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S30_tag DPLL_DT_S30; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S31_tag DPLL_DT_S31; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S32_tag DPLL_DT_S32; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S33_tag DPLL_DT_S33; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S34_tag DPLL_DT_S34; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S35_tag DPLL_DT_S35; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S36_tag DPLL_DT_S36; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S37_tag DPLL_DT_S37; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S38_tag DPLL_DT_S38; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S39_tag DPLL_DT_S39; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S40_tag DPLL_DT_S40; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S41_tag DPLL_DT_S41; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S42_tag DPLL_DT_S42; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S43_tag DPLL_DT_S43; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S44_tag DPLL_DT_S44; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S45_tag DPLL_DT_S45; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S46_tag DPLL_DT_S46; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S47_tag DPLL_DT_S47; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S48_tag DPLL_DT_S48; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S49_tag DPLL_DT_S49; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S50_tag DPLL_DT_S50; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S51_tag DPLL_DT_S51; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S52_tag DPLL_DT_S52; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S53_tag DPLL_DT_S53; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S54_tag DPLL_DT_S54; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S55_tag DPLL_DT_S55; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S56_tag DPLL_DT_S56; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S57_tag DPLL_DT_S57; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S58_tag DPLL_DT_S58; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S59_tag DPLL_DT_S59; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S60_tag DPLL_DT_S60; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S61_tag DPLL_DT_S61; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S62_tag DPLL_DT_S62; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  GTM_gtm_cls0_DPLL_DT_S63_tag DPLL_DT_S63; /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
  uint8_t GTM_gtm_cls0_reserved151[1024];
  GTM_gtm_cls0_DPLL_TSAC0_tag DPLL_TSAC0; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC1_tag DPLL_TSAC1; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC2_tag DPLL_TSAC2; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC3_tag DPLL_TSAC3; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC4_tag DPLL_TSAC4; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC5_tag DPLL_TSAC5; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC6_tag DPLL_TSAC6; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC7_tag DPLL_TSAC7; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC8_tag DPLL_TSAC8; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC9_tag DPLL_TSAC9; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC10_tag DPLL_TSAC10; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC11_tag DPLL_TSAC11; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC12_tag DPLL_TSAC12; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC13_tag DPLL_TSAC13; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC14_tag DPLL_TSAC14; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC15_tag DPLL_TSAC15; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC16_tag DPLL_TSAC16; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC17_tag DPLL_TSAC17; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC18_tag DPLL_TSAC18; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC19_tag DPLL_TSAC19; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC20_tag DPLL_TSAC20; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC21_tag DPLL_TSAC21; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC22_tag DPLL_TSAC22; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC23_tag DPLL_TSAC23; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC24_tag DPLL_TSAC24; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC25_tag DPLL_TSAC25; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC26_tag DPLL_TSAC26; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC27_tag DPLL_TSAC27; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC28_tag DPLL_TSAC28; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC29_tag DPLL_TSAC29; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC30_tag DPLL_TSAC30; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_TSAC31_tag DPLL_TSAC31; /* Calculated Time Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC0_tag DPLL_PSAC0; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC1_tag DPLL_PSAC1; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC2_tag DPLL_PSAC2; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC3_tag DPLL_PSAC3; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC4_tag DPLL_PSAC4; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC5_tag DPLL_PSAC5; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC6_tag DPLL_PSAC6; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC7_tag DPLL_PSAC7; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC8_tag DPLL_PSAC8; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC9_tag DPLL_PSAC9; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC10_tag DPLL_PSAC10; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC11_tag DPLL_PSAC11; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC12_tag DPLL_PSAC12; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC13_tag DPLL_PSAC13; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC14_tag DPLL_PSAC14; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC15_tag DPLL_PSAC15; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC16_tag DPLL_PSAC16; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC17_tag DPLL_PSAC17; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC18_tag DPLL_PSAC18; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC19_tag DPLL_PSAC19; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC20_tag DPLL_PSAC20; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC21_tag DPLL_PSAC21; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC22_tag DPLL_PSAC22; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC23_tag DPLL_PSAC23; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC24_tag DPLL_PSAC24; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC25_tag DPLL_PSAC25; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC26_tag DPLL_PSAC26; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC27_tag DPLL_PSAC27; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC28_tag DPLL_PSAC28; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC29_tag DPLL_PSAC29; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC30_tag DPLL_PSAC30; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_PSAC31_tag DPLL_PSAC31; /* Calculated Position Value to start Action [n] */
  GTM_gtm_cls0_DPLL_ACB_0_tag DPLL_ACB_0; /* Control Bits for NOAC Actions */
  GTM_gtm_cls0_DPLL_ACB_1_tag DPLL_ACB_1; /* Control Bits for NOAC Actions */
  GTM_gtm_cls0_DPLL_ACB_2_tag DPLL_ACB_2; /* Control Bits for NOAC Actions */
  GTM_gtm_cls0_DPLL_ACB_3_tag DPLL_ACB_3; /* Control Bits for NOAC Actions */
  GTM_gtm_cls0_DPLL_ACB_4_tag DPLL_ACB_4; /* Control Bits for NOAC Actions */
  GTM_gtm_cls0_DPLL_ACB_5_tag DPLL_ACB_5; /* Control Bits for NOAC Actions */
  GTM_gtm_cls0_DPLL_ACB_6_tag DPLL_ACB_6; /* Control Bits for NOAC Actions */
  GTM_gtm_cls0_DPLL_ACB_7_tag DPLL_ACB_7; /* Control Bits for NOAC Actions */
  GTM_gtm_cls0_DPLL_CTRL_11_tag DPLL_CTRL_11; /* Control Register 11 */
  GTM_gtm_cls0_DPLL_THVAL2_tag DPLL_THVAL2; /* Measured TRIGGER Hold Time Value 2 */
  GTM_gtm_cls0_DPLL_TIDEL_tag DPLL_TIDEL; /* TRIGGER input delay */
  GTM_gtm_cls0_DPLL_SIDEL_tag DPLL_SIDEL; /* STATE input delay */
  GTM_gtm_cls0_DPLL_APS_SYNC_EXT_tag DPLL_APS_SYNC_EXT; /* STATE Time Stamp Field Offset at Synchronization Time */
  GTM_gtm_cls0_DPLL_CTRL_EXT_tag DPLL_CTRL_EXT; /* STATE Time Stamp Field Offset at Synchronization Time */
  GTM_gtm_cls0_DPLL_APS_EXT_tag DPLL_APS_EXT; /* Actual RAM Pointer Address for STATE */
  GTM_gtm_cls0_DPLL_APS_1C3_EXT_tag DPLL_APS_1C3_EXT; /* Actual RAM Pointer Address for RAM region 1c3 (DPLL_CTRL_11.STATE_EXT=1) */
  GTM_gtm_cls0_DPLL_STA_tag DPLL_STA;  /* Status of the state machine states */
  GTM_gtm_cls0_DPLL_INCF1_OFFSET_tag DPLL_INCF1_OFFSET; /* Start value of ADD_IN_ADDER1 */
  GTM_gtm_cls0_DPLL_INCF2_OFFSET_tag DPLL_INCF2_OFFSET; /* Start value of the ADD_IN_ADDER2 */
  GTM_gtm_cls0_DPLL_DT_T_START_tag DPLL_DT_T_START; /* Start value of DT_T_ACT */
  GTM_gtm_cls0_DPLL_DT_S_START_tag DPLL_DT_S_START; /* Start value of DT_S_ACT */
  GTM_gtm_cls0_DPLL_STA_MASK_tag DPLL_STA_MASK; /* Notify values for DPLL_STA */
  GTM_gtm_cls0_DPLL_STA_FLAG_tag DPLL_STA_FLAG; /* DPLL STA Flags */
  GTM_gtm_cls0_DPLL_INC_CNT1_MASK_tag DPLL_INC_CNT1_MASK; /* Notify value of DPLL_INC_CNT1 */
  GTM_gtm_cls0_DPLL_INC_CNT2_MASK_tag DPLL_INC_CNT2_MASK; /* Notify value of DPLL_INC_CNT2 */
  GTM_gtm_cls0_DPLL_NUSC_EXT1_tag DPLL_NUSC_EXT1; /* Number of Recent STATE Events used for Calculations */
  GTM_gtm_cls0_DPLL_NUSC_EXT2_tag DPLL_NUSC_EXT2; /* Number of Recent STATE Events used for Calculations */
  GTM_gtm_cls0_DPLL_CTN_MIN_tag DPLL_CTN_MIN; /* Minimum value of predicted nominal increment of TRIGGER */
  GTM_gtm_cls0_DPLL_CTN_MAX_tag DPLL_CTN_MAX; /* Maximum value of predicted nominal increment of TRIGGER */
  GTM_gtm_cls0_DPLL_CSN_MIN_tag DPLL_CSN_MIN; /* Minimum value of predicted nominal increment of STATE */
  GTM_gtm_cls0_DPLL_CSN_MAX_tag DPLL_CSN_MAX; /* Maximum value of predicted nominal increment of STATE */
  GTM_gtm_cls0_DPLL_SW_TRIG_tag DPLL_SW_TRIG; /* Software triggered input events */
  GTM_gtm_cls0_DPLL_MP_T_tag DPLL_MP_T; /* Missing pulses of TRIGGER */
  GTM_gtm_cls0_DPLL_MP_S_tag DPLL_MP_S; /* Missing pulses of STATE */
  GTM_gtm_cls0_DPLL_CTRL_12_tag DPLL_CTRL_12; /* DPLL control register 12 */
  uint8_t GTM_gtm_cls0_reserved152[12404];
  GTM_gtm_cls0_DPLL_RR2_tag DPLL_RR2[4096]; /* DPLL memory RR2 */
  GTM_gtm_cls0_MCS0_MEM_tag MCS0_MEM[6144]; /* MCS[i] memory region */
};


/* ============================================================================
   =============================== Module: GTM_gtm_cls1 =======================
   ============================================================================ */

typedef union GTM_gtm_cls1_MON_STATUS_union_tag { /* MON status register */
  vuint32_t R;
  struct {
    vuint32_t ACT_CMU0:1;
    vuint32_t ACT_CMU1:1;
    vuint32_t ACT_CMU2:1;
    vuint32_t ACT_CMU3:1;
    vuint32_t ACT_CMU4:1;
    vuint32_t ACT_CMU5:1;
    vuint32_t ACT_CMU6:1;
    vuint32_t ACT_CMU7:1;
    vuint32_t ACT_CMUFX0:1;
    vuint32_t ACT_CMUFX1:1;
    vuint32_t ACT_CMUFX2:1;
    vuint32_t ACT_CMUFX3:1;
    vuint32_t ACT_CMUFX4:1;
    vuint32_t _unused_13:1;
    vuint32_t ACT_CMU8:1;
    vuint32_t _unused_15:1;
    vuint32_t CMP_ERR:1;
    vuint32_t _unused_17:3;
    vuint32_t MCS0_ERR:1;
    vuint32_t MCS1_ERR:1;
    vuint32_t MCS2_ERR:1;
    vuint32_t MCS3_ERR:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MON_STATUS_tag;

typedef union GTM_gtm_cls1_MON_ACTIVITY_0_union_tag { /* MON activity register 0 */
  vuint32_t R;
  struct {
    vuint32_t MCA_0_0:1;
    vuint32_t MCA_0_1:1;
    vuint32_t MCA_0_2:1;
    vuint32_t MCA_0_3:1;
    vuint32_t MCA_0_4:1;
    vuint32_t MCA_0_5:1;
    vuint32_t MCA_0_6:1;
    vuint32_t MCA_0_7:1;
    vuint32_t MCA_1_0:1;
    vuint32_t MCA_1_1:1;
    vuint32_t MCA_1_2:1;
    vuint32_t MCA_1_3:1;
    vuint32_t MCA_1_4:1;
    vuint32_t MCA_1_5:1;
    vuint32_t MCA_1_6:1;
    vuint32_t MCA_1_7:1;
    vuint32_t MCA_2_0:1;
    vuint32_t MCA_2_1:1;
    vuint32_t MCA_2_2:1;
    vuint32_t MCA_2_3:1;
    vuint32_t MCA_2_4:1;
    vuint32_t MCA_2_5:1;
    vuint32_t MCA_2_6:1;
    vuint32_t MCA_2_7:1;
    vuint32_t MCA_3_0:1;
    vuint32_t MCA_3_1:1;
    vuint32_t MCA_3_2:1;
    vuint32_t MCA_3_3:1;
    vuint32_t MCA_3_4:1;
    vuint32_t MCA_3_5:1;
    vuint32_t MCA_3_6:1;
    vuint32_t MCA_3_7:1;
  } B;
} GTM_gtm_cls1_MON_ACTIVITY_0_tag;

typedef union GTM_gtm_cls1_MON_ACTIVITY_MCS0_union_tag { /* MON activity register for MCS [j] */
  vuint32_t R;
  struct {
    vuint32_t MCA_0:1;
    vuint32_t MCA_1:1;
    vuint32_t MCA_2:1;
    vuint32_t MCA_3:1;
    vuint32_t MCA_4:1;
    vuint32_t MCA_5:1;
    vuint32_t MCA_6:1;
    vuint32_t MCA_7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MON_ACTIVITY_MCS0_tag;

typedef union GTM_gtm_cls1_MON_ACTIVITY_MCS1_union_tag { /* MON activity register for MCS [j] */
  vuint32_t R;
  struct {
    vuint32_t MCA_0:1;
    vuint32_t MCA_1:1;
    vuint32_t MCA_2:1;
    vuint32_t MCA_3:1;
    vuint32_t MCA_4:1;
    vuint32_t MCA_5:1;
    vuint32_t MCA_6:1;
    vuint32_t MCA_7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MON_ACTIVITY_MCS1_tag;

typedef union GTM_gtm_cls1_MON_ACTIVITY_MCS2_union_tag { /* MON activity register for MCS [j] */
  vuint32_t R;
  struct {
    vuint32_t MCA_0:1;
    vuint32_t MCA_1:1;
    vuint32_t MCA_2:1;
    vuint32_t MCA_3:1;
    vuint32_t MCA_4:1;
    vuint32_t MCA_5:1;
    vuint32_t MCA_6:1;
    vuint32_t MCA_7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MON_ACTIVITY_MCS2_tag;

typedef union GTM_gtm_cls1_MON_ACTIVITY_MCS3_union_tag { /* MON activity register for MCS [j] */
  vuint32_t R;
  struct {
    vuint32_t MCA_0:1;
    vuint32_t MCA_1:1;
    vuint32_t MCA_2:1;
    vuint32_t MCA_3:1;
    vuint32_t MCA_4:1;
    vuint32_t MCA_5:1;
    vuint32_t MCA_6:1;
    vuint32_t MCA_7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MON_ACTIVITY_MCS3_tag;

typedef union GTM_gtm_cls1_CMP_EN_union_tag { /* CMP comparator enable register */
  vuint32_t R;
  struct {
    vuint32_t ABWC0_EN:1;
    vuint32_t ABWC1_EN:1;
    vuint32_t ABWC2_EN:1;
    vuint32_t ABWC3_EN:1;
    vuint32_t ABWC4_EN:1;
    vuint32_t ABWC5_EN:1;
    vuint32_t ABWC6_EN:1;
    vuint32_t ABWC7_EN:1;
    vuint32_t ABWC8_EN:1;
    vuint32_t ABWC9_EN:1;
    vuint32_t ABWC10_EN:1;
    vuint32_t ABWC11_EN:1;
    vuint32_t TBWC0_EN:1;
    vuint32_t TBWC1_EN:1;
    vuint32_t TBWC2_EN:1;
    vuint32_t TBWC3_EN:1;
    vuint32_t TBWC4_EN:1;
    vuint32_t TBWC5_EN:1;
    vuint32_t TBWC6_EN:1;
    vuint32_t TBWC7_EN:1;
    vuint32_t TBWC8_EN:1;
    vuint32_t TBWC9_EN:1;
    vuint32_t TBWC10_EN:1;
    vuint32_t TBWC11_EN:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_CMP_EN_tag;

typedef union GTM_gtm_cls1_CMP_IRQ_NOTIFY_union_tag { /* CMP event notification register */
  vuint32_t R;
  struct {
    vuint32_t ABWC0:1;
    vuint32_t ABWC1:1;
    vuint32_t ABWC2:1;
    vuint32_t ABWC3:1;
    vuint32_t ABWC4:1;
    vuint32_t ABWC5:1;
    vuint32_t ABWC6:1;
    vuint32_t ABWC7:1;
    vuint32_t ABWC8:1;
    vuint32_t ABWC9:1;
    vuint32_t ABWC10:1;
    vuint32_t ABWC11:1;
    vuint32_t TBWC0:1;
    vuint32_t TBWC1:1;
    vuint32_t TBWC2:1;
    vuint32_t TBWC3:1;
    vuint32_t TBWC4:1;
    vuint32_t TBWC5:1;
    vuint32_t TBWC6:1;
    vuint32_t TBWC7:1;
    vuint32_t TBWC8:1;
    vuint32_t TBWC9:1;
    vuint32_t TBWC10:1;
    vuint32_t TBWC11:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_CMP_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_CMP_IRQ_EN_union_tag { /* CMP interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t ABWC0_EN_IRQ:1;
    vuint32_t ABWC1_EN_IRQ:1;
    vuint32_t ABWC2_EN_IRQ:1;
    vuint32_t ABWC3_EN_IRQ:1;
    vuint32_t ABWC4_EN_IRQ:1;
    vuint32_t ABWC5_EN_IRQ:1;
    vuint32_t ABWC6_EN_IRQ:1;
    vuint32_t ABWC7_EN_IRQ:1;
    vuint32_t ABWC8_EN_IRQ:1;
    vuint32_t ABWC9_EN_IRQ:1;
    vuint32_t ABWC10_EN_IRQ:1;
    vuint32_t ABWC11_EN_IRQ:1;
    vuint32_t TBWC0_EN_IRQ:1;
    vuint32_t TBWC1_EN_IRQ:1;
    vuint32_t TBWC2_EN_IRQ:1;
    vuint32_t TBWC3_EN_IRQ:1;
    vuint32_t TBWC4_EN_IRQ:1;
    vuint32_t TBWC5_EN_IRQ:1;
    vuint32_t TBWC6_EN_IRQ:1;
    vuint32_t TBWC7_EN_IRQ:1;
    vuint32_t TBWC8_EN_IRQ:1;
    vuint32_t TBWC9_EN_IRQ:1;
    vuint32_t TBWC10_EN_IRQ:1;
    vuint32_t TBWC11_EN_IRQ:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_CMP_IRQ_EN_tag;

typedef union GTM_gtm_cls1_CMP_IRQ_FORCINT_union_tag { /* CMP interrupt force register */
  vuint32_t R;
  struct {
    vuint32_t TRG_ABWC0:1;
    vuint32_t TRG_ABWC1:1;
    vuint32_t TRG_ABWC2:1;
    vuint32_t TRG_ABWC3:1;
    vuint32_t TRG_ABWC4:1;
    vuint32_t TRG_ABWC5:1;
    vuint32_t TRG_ABWC6:1;
    vuint32_t TRG_ABWC7:1;
    vuint32_t TRG_ABWC8:1;
    vuint32_t TRG_ABWC9:1;
    vuint32_t TRG_ABWC10:1;
    vuint32_t TRG_ABWC11:1;
    vuint32_t TRG_TBWC0:1;
    vuint32_t TRG_TBWC1:1;
    vuint32_t TRG_TBWC2:1;
    vuint32_t TRG_TBWC3:1;
    vuint32_t TRG_TBWC4:1;
    vuint32_t TRG_TBWC5:1;
    vuint32_t TRG_TBWC6:1;
    vuint32_t TRG_TBWC7:1;
    vuint32_t TRG_TBWC8:1;
    vuint32_t TRG_TBWC9:1;
    vuint32_t TRG_TBWC10:1;
    vuint32_t TRG_TBWC11:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_CMP_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_CMP_IRQ_MODE_union_tag { /* CMP interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_CMP_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_CMP_EIRQ_EN_union_tag { /* CMP error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t ABWC0_EN_EIRQ:1;
    vuint32_t ABWC1_EN_EIRQ:1;
    vuint32_t ABWC2_EN_EIRQ:1;
    vuint32_t ABWC3_EN_EIRQ:1;
    vuint32_t ABWC4_EN_EIRQ:1;
    vuint32_t ABWC5_EN_EIRQ:1;
    vuint32_t ABWC6_EN_EIRQ:1;
    vuint32_t ABWC7_EN_EIRQ:1;
    vuint32_t ABWC8_EN_EIRQ:1;
    vuint32_t ABWC9_EN_EIRQ:1;
    vuint32_t ABWC10_EN_EIRQ:1;
    vuint32_t ABWC11_EN_EIRQ:1;
    vuint32_t TBWC0_EN_EIRQ:1;
    vuint32_t TBWC1_EN_EIRQ:1;
    vuint32_t TBWC2_EN_EIRQ:1;
    vuint32_t TBWC3_EN_EIRQ:1;
    vuint32_t TBWC4_EN_EIRQ:1;
    vuint32_t TBWC5_EN_EIRQ:1;
    vuint32_t TBWC6_EN_EIRQ:1;
    vuint32_t TBWC7_EN_EIRQ:1;
    vuint32_t TBWC8_EN_EIRQ:1;
    vuint32_t TBWC9_EN_EIRQ:1;
    vuint32_t TBWC10_EN_EIRQ:1;
    vuint32_t TBWC11_EN_EIRQ:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_CMP_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH0_GPR0_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH0_GPR1_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH0_CNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TIM1_CH0_ECNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH0_CNTS_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH0_TDUC_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls1_TIM1_CH0_TDUV_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH0_FLT_RE_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH0_FLT_FE_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls1_TIM1_CH0_CTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls1_TIM1_CH0_ECTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIM1_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIM1_CH0_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH0_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH1_GPR0_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH1_GPR1_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH1_CNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TIM1_CH1_ECNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH1_CNTS_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH1_TDUC_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls1_TIM1_CH1_TDUV_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH1_FLT_RE_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH1_FLT_FE_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls1_TIM1_CH1_CTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls1_TIM1_CH1_ECTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIM1_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIM1_CH1_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH1_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH2_GPR0_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH2_GPR1_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH2_CNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TIM1_CH2_ECNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH2_CNTS_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH2_TDUC_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls1_TIM1_CH2_TDUV_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH2_FLT_RE_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH2_FLT_FE_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls1_TIM1_CH2_CTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls1_TIM1_CH2_ECTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIM1_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIM1_CH2_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH2_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH3_GPR0_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH3_GPR1_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH3_CNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TIM1_CH3_ECNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH3_CNTS_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH3_TDUC_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls1_TIM1_CH3_TDUV_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH3_FLT_RE_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH3_FLT_FE_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls1_TIM1_CH3_CTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls1_TIM1_CH3_ECTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIM1_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIM1_CH3_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH3_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH4_GPR0_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH4_GPR1_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH4_CNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TIM1_CH4_ECNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH4_CNTS_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH4_TDUC_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls1_TIM1_CH4_TDUV_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH4_FLT_RE_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH4_FLT_FE_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls1_TIM1_CH4_CTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls1_TIM1_CH4_ECTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIM1_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIM1_CH4_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH4_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH5_GPR0_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH5_GPR1_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH5_CNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TIM1_CH5_ECNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH5_CNTS_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH5_TDUC_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls1_TIM1_CH5_TDUV_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH5_FLT_RE_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH5_FLT_FE_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls1_TIM1_CH5_CTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls1_TIM1_CH5_ECTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIM1_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIM1_CH5_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH5_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH6_GPR0_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH6_GPR1_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH6_CNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TIM1_CH6_ECNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH6_CNTS_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH6_TDUC_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls1_TIM1_CH6_TDUV_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH6_FLT_RE_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH6_FLT_FE_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls1_TIM1_CH6_CTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls1_TIM1_CH6_ECTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIM1_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIM1_CH6_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH6_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH7_GPR0_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH7_GPR1_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH7_CNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TIM1_CH7_ECNT_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls1_TIM1_CH7_CNTS_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH7_TDUC_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls1_TIM1_CH7_TDUV_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH7_FLT_RE_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_CH7_FLT_FE_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls1_TIM1_CH7_CTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls1_TIM1_CH7_ECTRL_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIM1_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIM1_CH7_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIM1_CH7_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_TIM1_INP_VAL_union_tag { /* TIM[i] input value observation register */
  vuint32_t R;
  struct {
    vuint32_t F_OUT0:1;
    vuint32_t F_OUT1:1;
    vuint32_t F_OUT2:1;
    vuint32_t F_OUT3:1;
    vuint32_t F_OUT4:1;
    vuint32_t F_OUT5:1;
    vuint32_t F_OUT6:1;
    vuint32_t F_OUT7:1;
    vuint32_t F_IN0:1;
    vuint32_t F_IN1:1;
    vuint32_t F_IN2:1;
    vuint32_t F_IN3:1;
    vuint32_t F_IN4:1;
    vuint32_t F_IN5:1;
    vuint32_t F_IN6:1;
    vuint32_t F_IN7:1;
    vuint32_t TIM_IN0:1;
    vuint32_t TIM_IN1:1;
    vuint32_t TIM_IN2:1;
    vuint32_t TIM_IN3:1;
    vuint32_t TIM_IN4:1;
    vuint32_t TIM_IN5:1;
    vuint32_t TIM_IN6:1;
    vuint32_t TIM_IN7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIM1_INP_VAL_tag;

typedef union GTM_gtm_cls1_TIM1_IN_SRC_union_tag { /* TIM[i] AUX IN source selection register */
  vuint32_t R;
  struct {
    vuint32_t VAL_0:2;
    vuint32_t MODE_0:2;
    vuint32_t VAL_1:2;
    vuint32_t MODE_1:2;
    vuint32_t VAL_2:2;
    vuint32_t MODE_2:2;
    vuint32_t VAL_3:2;
    vuint32_t MODE_3:2;
    vuint32_t VAL_4:2;
    vuint32_t MODE_4:2;
    vuint32_t VAL_5:2;
    vuint32_t MODE_5:2;
    vuint32_t VAL_6:2;
    vuint32_t MODE_6:2;
    vuint32_t VAL_7:2;
    vuint32_t MODE_7:2;
  } B;
} GTM_gtm_cls1_TIM1_IN_SRC_tag;

typedef union GTM_gtm_cls1_TIM1_RST_union_tag { /* TIM[i] global software reset register */
  vuint32_t R;
  struct {
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIM1_RST_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH0_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH0_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH0_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH0_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH0_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH0_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH0_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH0_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH0_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH1_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH1_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH1_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH1_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH1_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH1_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH1_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH1_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH1_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH2_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH2_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH2_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH2_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH2_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH2_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH2_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH2_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH2_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH3_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH3_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH3_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH3_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH3_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH3_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH3_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH3_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH3_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH4_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH4_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH4_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH4_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH4_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH4_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH4_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH4_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH4_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH5_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH5_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH5_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH5_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH5_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH5_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH5_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH5_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH5_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH6_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH6_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH6_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH6_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH6_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH6_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH6_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH6_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH6_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:1;
    vuint32_t SPEM:1;
    vuint32_t GCM:1;
    vuint32_t _unused_30:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH7_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH7_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH7_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH7_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH7_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH7_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH7_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH7_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH7_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH8_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH8_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH8_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH8_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH8_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH8_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH8_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH8_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH8_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH8_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH8_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH8_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH8_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SPE_TRIG:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH9_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH9_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH9_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH9_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH9_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH9_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH9_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH9_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH9_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH9_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH9_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH9_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH9_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH10_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH10_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH10_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH10_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH10_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH10_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH10_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH10_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH10_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH10_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH10_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH10_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH10_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH11_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH11_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH11_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH11_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH11_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH11_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH11_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH11_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH11_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH11_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH11_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH11_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH11_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH12_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH12_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH12_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH12_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH12_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH12_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH12_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH12_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH12_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH12_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH12_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH12_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH12_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH13_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH13_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH13_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH13_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH13_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH13_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH13_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH13_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH13_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH13_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH13_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH13_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH13_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t _unused_27:4;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH14_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH14_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH14_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH14_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH14_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH14_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH14_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH14_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH14_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH14_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH14_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH14_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH14_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_CTRL_union_tag { /* TOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t SR0_TRIG:1;
    vuint32_t _unused_8:3;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t _unused_16:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t _unused_25:1;
    vuint32_t OSM:1;
    vuint32_t BITREV:1;
    vuint32_t _unused_28:3;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_TOM1_CH15_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_SR0_union_tag { /* TOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH15_SR0_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_SR1_union_tag { /* TOM[i] channel [x] CCU1 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH15_SR1_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_CM0_union_tag { /* TOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH15_CM0_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_CM1_union_tag { /* TOM[i] channel [x] CCU1 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH15_CM1_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_CN0_union_tag { /* TOM[i] channel [x] CCU0 counter */
  vuint32_t R;
  struct {
    vuint32_t CN0:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH15_CN0_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_STAT_union_tag { /* TOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:28;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_TOM1_CH15_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_IRQ_NOTIFY_union_tag { /* TOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH15_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_IRQ_EN_union_tag { /* TOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH15_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_IRQ_FORCINT_union_tag { /* TOM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH15_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_IRQ_MODE_union_tag { /* TOM[i] channel [x] interrupt mode register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TOM1_CH15_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TOM1_CH15_CTRL_SR_union_tag { /* TOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_CH15_CTRL_SR_tag;

typedef union GTM_gtm_cls1_TOM1_TGC0_GLB_CTRL_union_tag { /* TOM[i] TGC [g] global control register */
  vuint32_t R;
  struct {
    vuint32_t HOST_TRIG:1;
    vuint32_t _unused_1:7;
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t UPEN_CTRL0:2;
    vuint32_t UPEN_CTRL1:2;
    vuint32_t UPEN_CTRL2:2;
    vuint32_t UPEN_CTRL3:2;
    vuint32_t UPEN_CTRL4:2;
    vuint32_t UPEN_CTRL5:2;
    vuint32_t UPEN_CTRL6:2;
    vuint32_t UPEN_CTRL7:2;
  } B;
} GTM_gtm_cls1_TOM1_TGC0_GLB_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_TGC0_ACT_TB_union_tag { /* TOM[i] TGC [g] action time base register */
  vuint32_t R;
  struct {
    vuint32_t ACT_TB:24;
    vuint32_t TB_TRIG:1;
    vuint32_t TBU_SEL:2;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls1_TOM1_TGC0_ACT_TB_tag;

typedef union GTM_gtm_cls1_TOM1_TGC0_FUPD_CTRL_union_tag { /* TOM[i] TGC [g] force update control register */
  vuint32_t R;
  struct {
    vuint32_t FUPD_CTRL0:2;
    vuint32_t FUPD_CTRL1:2;
    vuint32_t FUPD_CTRL2:2;
    vuint32_t FUPD_CTRL3:2;
    vuint32_t FUPD_CTRL4:2;
    vuint32_t FUPD_CTRL5:2;
    vuint32_t FUPD_CTRL6:2;
    vuint32_t FUPD_CTRL7:2;
    vuint32_t RSTCN0_CH0:2;
    vuint32_t RSTCN0_CH1:2;
    vuint32_t RSTCN0_CH2:2;
    vuint32_t RSTCN0_CH3:2;
    vuint32_t RSTCN0_CH4:2;
    vuint32_t RSTCN0_CH5:2;
    vuint32_t RSTCN0_CH6:2;
    vuint32_t RSTCN0_CH7:2;
  } B;
} GTM_gtm_cls1_TOM1_TGC0_FUPD_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_TGC0_INT_TRIG_union_tag { /* TOM[i] TGC [g] internal trigger control register */
  vuint32_t R;
  struct {
    vuint32_t INT_TRIG0:2;
    vuint32_t INT_TRIG1:2;
    vuint32_t INT_TRIG2:2;
    vuint32_t INT_TRIG3:2;
    vuint32_t INT_TRIG4:2;
    vuint32_t INT_TRIG5:2;
    vuint32_t INT_TRIG6:2;
    vuint32_t INT_TRIG7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC0_INT_TRIG_tag;

typedef union GTM_gtm_cls1_TOM1_TGC0_ENDIS_CTRL_union_tag { /* TOM[i] TGC [g] enable/disable control register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CTRL0:2;
    vuint32_t ENDIS_CTRL1:2;
    vuint32_t ENDIS_CTRL2:2;
    vuint32_t ENDIS_CTRL3:2;
    vuint32_t ENDIS_CTRL4:2;
    vuint32_t ENDIS_CTRL5:2;
    vuint32_t ENDIS_CTRL6:2;
    vuint32_t ENDIS_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC0_ENDIS_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_TGC0_ENDIS_STAT_union_tag { /* TOM[i] TGC [g] enable/disable status register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_STAT0:2;
    vuint32_t ENDIS_STAT1:2;
    vuint32_t ENDIS_STAT2:2;
    vuint32_t ENDIS_STAT3:2;
    vuint32_t ENDIS_STAT4:2;
    vuint32_t ENDIS_STAT5:2;
    vuint32_t ENDIS_STAT6:2;
    vuint32_t ENDIS_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC0_ENDIS_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_TGC0_OUTEN_CTRL_union_tag { /* TOM[i] TGC [g] output enable control register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_CTRL0:2;
    vuint32_t OUTEN_CTRL1:2;
    vuint32_t OUTEN_CTRL2:2;
    vuint32_t OUTEN_CTRL3:2;
    vuint32_t OUTEN_CTRL4:2;
    vuint32_t OUTEN_CTRL5:2;
    vuint32_t OUTEN_CTRL6:2;
    vuint32_t OUTEN_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC0_OUTEN_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_TGC0_OUTEN_STAT_union_tag { /* TOM[i] TGC [g] output enable status register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_STAT0:2;
    vuint32_t OUTEN_STAT1:2;
    vuint32_t OUTEN_STAT2:2;
    vuint32_t OUTEN_STAT3:2;
    vuint32_t OUTEN_STAT4:2;
    vuint32_t OUTEN_STAT5:2;
    vuint32_t OUTEN_STAT6:2;
    vuint32_t OUTEN_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC0_OUTEN_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_TGC1_GLB_CTRL_union_tag { /* TOM[i] TGC [g] global control register */
  vuint32_t R;
  struct {
    vuint32_t HOST_TRIG:1;
    vuint32_t _unused_1:7;
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t UPEN_CTRL0:2;
    vuint32_t UPEN_CTRL1:2;
    vuint32_t UPEN_CTRL2:2;
    vuint32_t UPEN_CTRL3:2;
    vuint32_t UPEN_CTRL4:2;
    vuint32_t UPEN_CTRL5:2;
    vuint32_t UPEN_CTRL6:2;
    vuint32_t UPEN_CTRL7:2;
  } B;
} GTM_gtm_cls1_TOM1_TGC1_GLB_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_TGC1_ACT_TB_union_tag { /* TOM[i] TGC [g] action time base register */
  vuint32_t R;
  struct {
    vuint32_t ACT_TB:24;
    vuint32_t TB_TRIG:1;
    vuint32_t TBU_SEL:2;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls1_TOM1_TGC1_ACT_TB_tag;

typedef union GTM_gtm_cls1_TOM1_TGC1_FUPD_CTRL_union_tag { /* TOM[i] TGC [g] force update control register */
  vuint32_t R;
  struct {
    vuint32_t FUPD_CTRL0:2;
    vuint32_t FUPD_CTRL1:2;
    vuint32_t FUPD_CTRL2:2;
    vuint32_t FUPD_CTRL3:2;
    vuint32_t FUPD_CTRL4:2;
    vuint32_t FUPD_CTRL5:2;
    vuint32_t FUPD_CTRL6:2;
    vuint32_t FUPD_CTRL7:2;
    vuint32_t RSTCN0_CH0:2;
    vuint32_t RSTCN0_CH1:2;
    vuint32_t RSTCN0_CH2:2;
    vuint32_t RSTCN0_CH3:2;
    vuint32_t RSTCN0_CH4:2;
    vuint32_t RSTCN0_CH5:2;
    vuint32_t RSTCN0_CH6:2;
    vuint32_t RSTCN0_CH7:2;
  } B;
} GTM_gtm_cls1_TOM1_TGC1_FUPD_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_TGC1_INT_TRIG_union_tag { /* TOM[i] TGC [g] internal trigger control register */
  vuint32_t R;
  struct {
    vuint32_t INT_TRIG0:2;
    vuint32_t INT_TRIG1:2;
    vuint32_t INT_TRIG2:2;
    vuint32_t INT_TRIG3:2;
    vuint32_t INT_TRIG4:2;
    vuint32_t INT_TRIG5:2;
    vuint32_t INT_TRIG6:2;
    vuint32_t INT_TRIG7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC1_INT_TRIG_tag;

typedef union GTM_gtm_cls1_TOM1_TGC1_ENDIS_CTRL_union_tag { /* TOM[i] TGC [g] enable/disable control register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CTRL0:2;
    vuint32_t ENDIS_CTRL1:2;
    vuint32_t ENDIS_CTRL2:2;
    vuint32_t ENDIS_CTRL3:2;
    vuint32_t ENDIS_CTRL4:2;
    vuint32_t ENDIS_CTRL5:2;
    vuint32_t ENDIS_CTRL6:2;
    vuint32_t ENDIS_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC1_ENDIS_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_TGC1_ENDIS_STAT_union_tag { /* TOM[i] TGC [g] enable/disable status register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_STAT0:2;
    vuint32_t ENDIS_STAT1:2;
    vuint32_t ENDIS_STAT2:2;
    vuint32_t ENDIS_STAT3:2;
    vuint32_t ENDIS_STAT4:2;
    vuint32_t ENDIS_STAT5:2;
    vuint32_t ENDIS_STAT6:2;
    vuint32_t ENDIS_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC1_ENDIS_STAT_tag;

typedef union GTM_gtm_cls1_TOM1_TGC1_OUTEN_CTRL_union_tag { /* TOM[i] TGC [g] output enable control register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_CTRL0:2;
    vuint32_t OUTEN_CTRL1:2;
    vuint32_t OUTEN_CTRL2:2;
    vuint32_t OUTEN_CTRL3:2;
    vuint32_t OUTEN_CTRL4:2;
    vuint32_t OUTEN_CTRL5:2;
    vuint32_t OUTEN_CTRL6:2;
    vuint32_t OUTEN_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC1_OUTEN_CTRL_tag;

typedef union GTM_gtm_cls1_TOM1_TGC1_OUTEN_STAT_union_tag { /* TOM[i] TGC [g] output enable status register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_STAT0:2;
    vuint32_t OUTEN_STAT1:2;
    vuint32_t OUTEN_STAT2:2;
    vuint32_t OUTEN_STAT3:2;
    vuint32_t OUTEN_STAT4:2;
    vuint32_t OUTEN_STAT5:2;
    vuint32_t OUTEN_STAT6:2;
    vuint32_t OUTEN_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_TOM1_TGC1_OUTEN_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_RDADDR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_SR0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_SR1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_CM0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_CM1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_CN0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_CTRL2_tag;

typedef union GTM_gtm_cls1_ATOM1_CH0_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_CH0_CTRL_SR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_RDADDR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_SR0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_SR1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_CM0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_CM1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_CN0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_CTRL2_tag;

typedef union GTM_gtm_cls1_ATOM1_CH1_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_CH1_CTRL_SR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_RDADDR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_SR0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_SR1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_CM0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_CM1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_CN0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_CTRL2_tag;

typedef union GTM_gtm_cls1_ATOM1_CH2_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_CH2_CTRL_SR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_RDADDR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_SR0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_SR1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_CM0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_CM1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_CN0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_CTRL2_tag;

typedef union GTM_gtm_cls1_ATOM1_CH3_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_CH3_CTRL_SR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_RDADDR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_SR0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_SR1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_CM0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_CM1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_CN0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_CTRL2_tag;

typedef union GTM_gtm_cls1_ATOM1_CH4_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_CH4_CTRL_SR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_RDADDR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_SR0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_SR1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_CM0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_CM1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_CN0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_CTRL2_tag;

typedef union GTM_gtm_cls1_ATOM1_CH5_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_CH5_CTRL_SR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_RDADDR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_SR0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_SR1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_CM0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_CM1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_CN0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_CTRL2_tag;

typedef union GTM_gtm_cls1_ATOM1_CH6_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_CH6_CTRL_SR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_RDADDR_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_SR0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_SR1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_CM0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_CM1_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_CN0_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_CTRL2_tag;

typedef union GTM_gtm_cls1_ATOM1_CH7_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_CH7_CTRL_SR_tag;

typedef union GTM_gtm_cls1_ATOM1_AGC_GLB_CTRL_union_tag { /* ATOM[i] AGC global control register */
  vuint32_t R;
  struct {
    vuint32_t HOST_TRIG:1;
    vuint32_t _unused_1:7;
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t UPEN_CTRL0:2;
    vuint32_t UPEN_CTRL1:2;
    vuint32_t UPEN_CTRL2:2;
    vuint32_t UPEN_CTRL3:2;
    vuint32_t UPEN_CTRL4:2;
    vuint32_t UPEN_CTRL5:2;
    vuint32_t UPEN_CTRL6:2;
    vuint32_t UPEN_CTRL7:2;
  } B;
} GTM_gtm_cls1_ATOM1_AGC_GLB_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_AGC_ENDIS_CTRL_union_tag { /* ATOM[i] AGC enable/disable control register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CTRL0:2;
    vuint32_t ENDIS_CTRL1:2;
    vuint32_t ENDIS_CTRL2:2;
    vuint32_t ENDIS_CTRL3:2;
    vuint32_t ENDIS_CTRL4:2;
    vuint32_t ENDIS_CTRL5:2;
    vuint32_t ENDIS_CTRL6:2;
    vuint32_t ENDIS_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_AGC_ENDIS_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_AGC_ENDIS_STAT_union_tag { /* ATOM[i] AGC enable/disable status register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_STAT0:2;
    vuint32_t ENDIS_STAT1:2;
    vuint32_t ENDIS_STAT2:2;
    vuint32_t ENDIS_STAT3:2;
    vuint32_t ENDIS_STAT4:2;
    vuint32_t ENDIS_STAT5:2;
    vuint32_t ENDIS_STAT6:2;
    vuint32_t ENDIS_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_AGC_ENDIS_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_AGC_ACT_TB_union_tag { /* ATOM[i] AGC action time base register */
  vuint32_t R;
  struct {
    vuint32_t ACT_TB:24;
    vuint32_t TB_TRIG:1;
    vuint32_t TBU_SEL:2;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls1_ATOM1_AGC_ACT_TB_tag;

typedef union GTM_gtm_cls1_ATOM1_AGC_OUTEN_CTRL_union_tag { /* ATOM[i] AGC output enable control register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_CTRL0:2;
    vuint32_t OUTEN_CTRL1:2;
    vuint32_t OUTEN_CTRL2:2;
    vuint32_t OUTEN_CTRL3:2;
    vuint32_t OUTEN_CTRL4:2;
    vuint32_t OUTEN_CTRL5:2;
    vuint32_t OUTEN_CTRL6:2;
    vuint32_t OUTEN_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_AGC_OUTEN_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_AGC_OUTEN_STAT_union_tag { /* ATOM[i] AGC output enable status register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_STAT0:2;
    vuint32_t OUTEN_STAT1:2;
    vuint32_t OUTEN_STAT2:2;
    vuint32_t OUTEN_STAT3:2;
    vuint32_t OUTEN_STAT4:2;
    vuint32_t OUTEN_STAT5:2;
    vuint32_t OUTEN_STAT6:2;
    vuint32_t OUTEN_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_AGC_OUTEN_STAT_tag;

typedef union GTM_gtm_cls1_ATOM1_AGC_FUPD_CTRL_union_tag { /* ATOM[i] AGC force update control register */
  vuint32_t R;
  struct {
    vuint32_t FUPD_CTRL0:2;
    vuint32_t FUPD_CTRL1:2;
    vuint32_t FUPD_CTRL2:2;
    vuint32_t FUPD_CTRL3:2;
    vuint32_t FUPD_CTRL4:2;
    vuint32_t FUPD_CTRL5:2;
    vuint32_t FUPD_CTRL6:2;
    vuint32_t FUPD_CTRL7:2;
    vuint32_t RSTCN0_CH0:2;
    vuint32_t RSTCN0_CH1:2;
    vuint32_t RSTCN0_CH2:2;
    vuint32_t RSTCN0_CH3:2;
    vuint32_t RSTCN0_CH4:2;
    vuint32_t RSTCN0_CH5:2;
    vuint32_t RSTCN0_CH6:2;
    vuint32_t RSTCN0_CH7:2;
  } B;
} GTM_gtm_cls1_ATOM1_AGC_FUPD_CTRL_tag;

typedef union GTM_gtm_cls1_ATOM1_AGC_INT_TRIG_union_tag { /* ATOM[i] AGC internal trigger control register */
  vuint32_t R;
  struct {
    vuint32_t INT_TRIG0:2;
    vuint32_t INT_TRIG1:2;
    vuint32_t INT_TRIG2:2;
    vuint32_t INT_TRIG3:2;
    vuint32_t INT_TRIG4:2;
    vuint32_t INT_TRIG5:2;
    vuint32_t INT_TRIG6:2;
    vuint32_t INT_TRIG7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_ATOM1_AGC_INT_TRIG_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH0_R0_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH0_R1_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH0_R2_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH0_R3_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH0_R4_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH0_R5_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH0_R6_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH0_R7_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_MCS1_CH0_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_MCS1_CH0_ACB_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CH0_MHB_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_CH0_PC_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_CH0_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH0_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH1_R0_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH1_R1_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH1_R2_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH1_R3_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH1_R4_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH1_R5_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH1_R6_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH1_R7_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_MCS1_CH1_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_MCS1_CH1_ACB_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CH1_MHB_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_CH1_PC_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_CH1_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH1_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH2_R0_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH2_R1_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH2_R2_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH2_R3_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH2_R4_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH2_R5_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH2_R6_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH2_R7_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_MCS1_CH2_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_MCS1_CH2_ACB_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CH2_MHB_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_CH2_PC_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_CH2_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH2_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH3_R0_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH3_R1_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH3_R2_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH3_R3_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH3_R4_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH3_R5_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH3_R6_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH3_R7_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_MCS1_CH3_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_MCS1_CH3_ACB_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CH3_MHB_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_CH3_PC_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_CH3_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH3_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH4_R0_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH4_R1_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH4_R2_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH4_R3_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH4_R4_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH4_R5_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH4_R6_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH4_R7_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_MCS1_CH4_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_MCS1_CH4_ACB_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CH4_MHB_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_CH4_PC_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_CH4_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH4_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH5_R0_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH5_R1_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH5_R2_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH5_R3_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH5_R4_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH5_R5_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH5_R6_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH5_R7_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_MCS1_CH5_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_MCS1_CH5_ACB_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CH5_MHB_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_CH5_PC_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_CH5_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH5_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH6_R0_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH6_R1_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH6_R2_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH6_R3_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH6_R4_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH6_R5_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH6_R6_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH6_R7_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_MCS1_CH6_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_MCS1_CH6_ACB_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CH6_MHB_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_CH6_PC_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_CH6_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH6_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH7_R0_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH7_R1_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH7_R2_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH7_R3_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH7_R4_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH7_R5_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH7_R6_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CH7_R7_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_MCS1_CH7_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_MCS1_CH7_ACB_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CH7_MHB_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_CH7_PC_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_CH7_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls1_MCS1_CH7_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_CTRG_union_tag { /* MCS[i] clear trigger control register */
  vuint32_t R;
  struct {
    vuint32_t TRG0:1;
    vuint32_t TRG1:1;
    vuint32_t TRG2:1;
    vuint32_t TRG3:1;
    vuint32_t TRG4:1;
    vuint32_t TRG5:1;
    vuint32_t TRG6:1;
    vuint32_t TRG7:1;
    vuint32_t TRG8:1;
    vuint32_t TRG9:1;
    vuint32_t TRG10:1;
    vuint32_t TRG11:1;
    vuint32_t TRG12:1;
    vuint32_t TRG13:1;
    vuint32_t TRG14:1;
    vuint32_t TRG15:1;
    vuint32_t TRG16:1;
    vuint32_t TRG17:1;
    vuint32_t TRG18:1;
    vuint32_t TRG19:1;
    vuint32_t TRG20:1;
    vuint32_t TRG21:1;
    vuint32_t TRG22:1;
    vuint32_t TRG23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_CTRG_tag;

typedef union GTM_gtm_cls1_MCS1_STRG_union_tag { /* MCS[i] set trigger control register */
  vuint32_t R;
  struct {
    vuint32_t TRG0:1;
    vuint32_t TRG1:1;
    vuint32_t TRG2:1;
    vuint32_t TRG3:1;
    vuint32_t TRG4:1;
    vuint32_t TRG5:1;
    vuint32_t TRG6:1;
    vuint32_t TRG7:1;
    vuint32_t TRG8:1;
    vuint32_t TRG9:1;
    vuint32_t TRG10:1;
    vuint32_t TRG11:1;
    vuint32_t TRG12:1;
    vuint32_t TRG13:1;
    vuint32_t TRG14:1;
    vuint32_t TRG15:1;
    vuint32_t TRG16:1;
    vuint32_t TRG17:1;
    vuint32_t TRG18:1;
    vuint32_t TRG19:1;
    vuint32_t TRG20:1;
    vuint32_t TRG21:1;
    vuint32_t TRG22:1;
    vuint32_t TRG23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_MCS1_STRG_tag;

typedef union GTM_gtm_cls1_MCS1_CTRL_STAT_union_tag { /* MCS[i] control and status register */
  vuint32_t R;
  struct {
    vuint32_t SCD_MODE:2;
    vuint32_t _unused_2:6;
    vuint32_t SCD_CH:4;
    vuint32_t _unused_12:4;
    vuint32_t RAM_RST:1;
    vuint32_t _unused_17:3;
    vuint32_t ERR_SRC_ID:3;
    vuint32_t _unused_23:1;
    vuint32_t EN_TIM_FOUT:1;
    vuint32_t EN_HVD:1;
    vuint32_t HLT_AEIM_ERR:1;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls1_MCS1_CTRL_STAT_tag;

typedef union GTM_gtm_cls1_MCS1_RESET_union_tag { /* MCS[i] reset register */
  vuint32_t R;
  struct {
    vuint32_t RST0:1;
    vuint32_t RST1:1;
    vuint32_t RST2:1;
    vuint32_t RST3:1;
    vuint32_t RST4:1;
    vuint32_t RST5:1;
    vuint32_t RST6:1;
    vuint32_t RST7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_RESET_tag;

typedef union GTM_gtm_cls1_MCS1_CAT_union_tag { /* MCS[i] cancel ARU transfer instruction */
  vuint32_t R;
  struct {
    vuint32_t CAT0:1;
    vuint32_t CAT1:1;
    vuint32_t CAT2:1;
    vuint32_t CAT3:1;
    vuint32_t CAT4:1;
    vuint32_t CAT5:1;
    vuint32_t CAT6:1;
    vuint32_t CAT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CAT_tag;

typedef union GTM_gtm_cls1_MCS1_CWT_union_tag { /* MCS[i] cancel waiting instruction */
  vuint32_t R;
  struct {
    vuint32_t CWT0:1;
    vuint32_t CWT1:1;
    vuint32_t CWT2:1;
    vuint32_t CWT3:1;
    vuint32_t CWT4:1;
    vuint32_t CWT5:1;
    vuint32_t CWT6:1;
    vuint32_t CWT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_CWT_tag;

typedef union GTM_gtm_cls1_MCS1_ERR_union_tag { /* MCS[i] error register */
  vuint32_t R;
  struct {
    vuint32_t ERR0:1;
    vuint32_t ERR1:1;
    vuint32_t ERR2:1;
    vuint32_t ERR3:1;
    vuint32_t ERR4:1;
    vuint32_t ERR5:1;
    vuint32_t ERR6:1;
    vuint32_t ERR7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_ERR_tag;

typedef union GTM_gtm_cls1_MCS1_REG_PROT_union_tag { /* MCS[i] write protection register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:2;
    vuint32_t WPROT1:2;
    vuint32_t WPROT2:2;
    vuint32_t WPROT3:2;
    vuint32_t WPROT4:2;
    vuint32_t WPROT5:2;
    vuint32_t WPROT6:2;
    vuint32_t WPROT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_MCS1_REG_PROT_tag;

typedef union GTM_gtm_cls1_MCS1_SINT_IRQ_NOTIFY_union_tag { /* MCS[i] shared interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_IRQ0:1;
    vuint32_t S_IRQ1:1;
    vuint32_t S_IRQ2:1;
    vuint32_t S_IRQ3:1;
    vuint32_t S_IRQ4:1;
    vuint32_t S_IRQ5:1;
    vuint32_t S_IRQ6:1;
    vuint32_t S_IRQ7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_SINT_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_SINT_IRQ_EN_union_tag { /* MCS[i] shared interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_IRQ0_EN:1;
    vuint32_t S_IRQ1_EN:1;
    vuint32_t S_IRQ2_EN:1;
    vuint32_t S_IRQ3_EN:1;
    vuint32_t S_IRQ4_EN:1;
    vuint32_t S_IRQ5_EN:1;
    vuint32_t S_IRQ6_EN:1;
    vuint32_t S_IRQ7_EN:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_SINT_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_SINT_IRQ_FORCINT_union_tag { /* MCS[i] force shared interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_IRQ0:1;
    vuint32_t TRG_S_IRQ1:1;
    vuint32_t TRG_S_IRQ2:1;
    vuint32_t TRG_S_IRQ3:1;
    vuint32_t TRG_S_IRQ4:1;
    vuint32_t TRG_S_IRQ5:1;
    vuint32_t TRG_S_IRQ6:1;
    vuint32_t TRG_S_IRQ7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_SINT_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_SINT_IRQ_MODE_union_tag { /* MCS[i] shared interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_SINT_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_HBP0_CTRL_union_tag { /* MCS[i] hardware break point h control register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH0:1;
    vuint32_t EN_CH1:1;
    vuint32_t EN_CH2:1;
    vuint32_t EN_CH3:1;
    vuint32_t EN_CH4:1;
    vuint32_t EN_CH5:1;
    vuint32_t EN_CH6:1;
    vuint32_t EN_CH7:1;
    vuint32_t SCOPE:2;
    vuint32_t _unused_10:2;
    vuint32_t TYPE:3;
    vuint32_t _unused_15:1;
    vuint32_t AND:1;
    vuint32_t NOT:1;
    vuint32_t _unused_18:14;
  } B;
} GTM_gtm_cls1_MCS1_HBP0_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_HBP0_PATTERN_union_tag { /* MCS[i] hardware break point pattern register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls1_MCS1_HBP0_PATTERN_tag;

typedef union GTM_gtm_cls1_MCS1_HBP0_STATUS_union_tag { /* MCS[i] hardware break point status register */
  vuint32_t R;
  struct {
    vuint32_t HALT_CH0:1;
    vuint32_t HALT_CH1:1;
    vuint32_t HALT_CH2:1;
    vuint32_t HALT_CH3:1;
    vuint32_t HALT_CH4:1;
    vuint32_t HALT_CH5:1;
    vuint32_t HALT_CH6:1;
    vuint32_t HALT_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_HBP0_STATUS_tag;

typedef union GTM_gtm_cls1_MCS1_HBP0_IRQ_NOTIFY_union_tag { /* MCS[i] hardware break point interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_MCS1_HBP0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_HBP0_IRQ_EN_union_tag { /* MCS[i] hardware break point interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ_EN:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_MCS1_HBP0_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_HBP0_IRQ_FORCINT_union_tag { /* MCS[i] force hardware break point interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_MCS1_HBP0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_HBP0_IRQ_MODE_union_tag { /* MCS[i] break point h interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_HBP0_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_MCS1_HBP1_CTRL_union_tag { /* MCS[i] hardware break point h control register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH0:1;
    vuint32_t EN_CH1:1;
    vuint32_t EN_CH2:1;
    vuint32_t EN_CH3:1;
    vuint32_t EN_CH4:1;
    vuint32_t EN_CH5:1;
    vuint32_t EN_CH6:1;
    vuint32_t EN_CH7:1;
    vuint32_t SCOPE:2;
    vuint32_t _unused_10:2;
    vuint32_t TYPE:3;
    vuint32_t _unused_15:1;
    vuint32_t AND:1;
    vuint32_t NOT:1;
    vuint32_t _unused_18:14;
  } B;
} GTM_gtm_cls1_MCS1_HBP1_CTRL_tag;

typedef union GTM_gtm_cls1_MCS1_HBP1_PATTERN_union_tag { /* MCS[i] hardware break point pattern register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls1_MCS1_HBP1_PATTERN_tag;

typedef union GTM_gtm_cls1_MCS1_HBP1_STATUS_union_tag { /* MCS[i] hardware break point status register */
  vuint32_t R;
  struct {
    vuint32_t HALT_CH0:1;
    vuint32_t HALT_CH1:1;
    vuint32_t HALT_CH2:1;
    vuint32_t HALT_CH3:1;
    vuint32_t HALT_CH4:1;
    vuint32_t HALT_CH5:1;
    vuint32_t HALT_CH6:1;
    vuint32_t HALT_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_MCS1_HBP1_STATUS_tag;

typedef union GTM_gtm_cls1_MCS1_HBP1_IRQ_NOTIFY_union_tag { /* MCS[i] hardware break point interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_MCS1_HBP1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_MCS1_HBP1_IRQ_EN_union_tag { /* MCS[i] hardware break point interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ_EN:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_MCS1_HBP1_IRQ_EN_tag;

typedef union GTM_gtm_cls1_MCS1_HBP1_IRQ_FORCINT_union_tag { /* MCS[i] force hardware break point interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_MCS1_HBP1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_MCS1_HBP1_IRQ_MODE_union_tag { /* MCS[i] break point h interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_MCS1_HBP1_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_CTRL_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_SINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_SCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_SOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_OINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_OCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_OOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH0_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH0_SHIFTCNT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_CTRL_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_SINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_SCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_SOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_OINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_OCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_OOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH1_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH1_SHIFTCNT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_CTRL_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_SINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_SCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_SOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_OINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_OCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_OOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH2_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH2_SHIFTCNT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_CTRL_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_SINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_SCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_SOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_OINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_OCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_OOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH3_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH3_SHIFTCNT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_CTRL_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_SINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_SCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_SOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_OINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_OCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_OOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH4_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH4_SHIFTCNT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_CTRL_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_SINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_SCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_SOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_OINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_OCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_OOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH5_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH5_SHIFTCNT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_CTRL_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_SINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_SCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_SOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_OINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_OCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_OOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH6_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH6_SHIFTCNT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_CTRL_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_SINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_SCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_SOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_OINST_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_OCMD_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_OOP_tag;

typedef union GTM_gtm_cls1_TIO1_G0_CH7_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_G0_CH7_SHIFTCNT_tag;

typedef union GTM_gtm_cls1_TIO1_G0_ISEL0_CTRL1_union_tag { /* TIO[i] input selection register 1 */
  vuint32_t R;
  struct {
    vuint32_t LUT2_0:4;
    vuint32_t LUT2_1:4;
    vuint32_t LUT2_2:4;
    vuint32_t LUT2_3:4;
    vuint32_t OUT_SEL0:1;
    vuint32_t OUT_SEL1:1;
    vuint32_t OUT_SEL2:1;
    vuint32_t OUT_SEL3:1;
    vuint32_t _unused_20:4;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls1_TIO1_G0_ISEL0_CTRL1_tag;

typedef union GTM_gtm_cls1_TIO1_G0_ISEL0_CTRL2_union_tag { /* TIO[i] input selection register 2 */
  vuint32_t R;
  struct {
    vuint32_t LUT3:8;
    vuint32_t _unused_8:8;
    vuint32_t QOUT_SEL:2;
    vuint32_t _unused_18:2;
    vuint32_t LUT3IN_SEL0:1;
    vuint32_t LUT3IN_SEL1:1;
    vuint32_t LUT3IN_SEL2:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls1_TIO1_G0_ISEL0_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_ISEL1_CTRL1_union_tag { /* TIO[i] input selection register 1 */
  vuint32_t R;
  struct {
    vuint32_t LUT2_0:4;
    vuint32_t LUT2_1:4;
    vuint32_t LUT2_2:4;
    vuint32_t LUT2_3:4;
    vuint32_t OUT_SEL0:1;
    vuint32_t OUT_SEL1:1;
    vuint32_t OUT_SEL2:1;
    vuint32_t OUT_SEL3:1;
    vuint32_t _unused_20:4;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls1_TIO1_G0_ISEL1_CTRL1_tag;

typedef union GTM_gtm_cls1_TIO1_G0_ISEL1_CTRL2_union_tag { /* TIO[i] input selection register 2 */
  vuint32_t R;
  struct {
    vuint32_t LUT3:8;
    vuint32_t _unused_8:8;
    vuint32_t QOUT_SEL:2;
    vuint32_t _unused_18:2;
    vuint32_t LUT3IN_SEL0:1;
    vuint32_t LUT3IN_SEL1:1;
    vuint32_t LUT3IN_SEL2:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls1_TIO1_G0_ISEL1_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_G0_OP_USAGE_union_tag { /* TIO[i] operand usage selection register */
  vuint32_t R;
  struct {
    vuint32_t MODE0:3;
    vuint32_t MODE1:3;
    vuint32_t MODE2:3;
    vuint32_t MODE3:3;
    vuint32_t MODE4:3;
    vuint32_t MODE5:3;
    vuint32_t MODE6:3;
    vuint32_t MODE7:3;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t WRITE_EN4:1;
    vuint32_t WRITE_EN5:1;
    vuint32_t WRITE_EN6:1;
    vuint32_t WRITE_EN7:1;
  } B;
} GTM_gtm_cls1_TIO1_G0_OP_USAGE_tag;

typedef union GTM_gtm_cls1_TIO1_S_union_tag { /* TIO[i] signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_S_tag;

typedef union GTM_gtm_cls1_TIO1_O_union_tag { /* TIO[i] output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_O_tag;

typedef union GTM_gtm_cls1_TIO1_ENDIS_union_tag { /* TIO[i] enable/disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_ENDIS_tag;

typedef union GTM_gtm_cls1_TIO1_INVERT_union_tag { /* TIO[i] signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_INVERT_tag;

typedef union GTM_gtm_cls1_TIO1_INPUT_MODE_union_tag { /* TIO[i] input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_INPUT_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_CYCLIC_MODE_union_tag { /* TIO[i] cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CYCLIC_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_TRIG_OUT_GATE_EN_union_tag { /* TIO[i] enable Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_TRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls1_TIO1_PLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] enable PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_PLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls1_TIO1_CS_union_tag { /* TIO[i] clear signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CS_tag;

typedef union GTM_gtm_cls1_TIO1_CO_union_tag { /* TIO[i] clear output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CO_tag;

typedef union GTM_gtm_cls1_TIO1_CENDIS_union_tag { /* TIO[i] disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CENDIS_tag;

typedef union GTM_gtm_cls1_TIO1_CINVERT_union_tag { /* TIO[i] clear signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CINVERT_tag;

typedef union GTM_gtm_cls1_TIO1_CINPUT_MODE_union_tag { /* TIO[i] disable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CINPUT_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_CCYCLIC_MODE_union_tag { /* TIO[i] disable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CCYCLIC_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_CTRIG_OUT_GATE_EN_union_tag { /* TIO[i] clear Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls1_TIO1_CPLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] clear PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_CPLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls1_TIO1_SS_union_tag { /* TIO[i] set signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_SS_tag;

typedef union GTM_gtm_cls1_TIO1_SO_union_tag { /* TIO[i] set output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_SO_tag;

typedef union GTM_gtm_cls1_TIO1_SENDIS_union_tag { /* TIO[i] enable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_SENDIS_tag;

typedef union GTM_gtm_cls1_TIO1_SINVERT_union_tag { /* TIO[i] set signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_SINVERT_tag;

typedef union GTM_gtm_cls1_TIO1_SINPUT_MODE_union_tag { /* TIO[i] enable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_SINPUT_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_SCYCLIC_MODE_union_tag { /* TIO[i] enable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_SCYCLIC_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_STRIG_OUT_GATE_EN_union_tag { /* TIO[i] set Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_STRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls1_TIO1_SPLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] set PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_SPLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls1_TIO1_IS_union_tag { /* TIO[i] invert signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_IS_tag;

typedef union GTM_gtm_cls1_TIO1_IO_union_tag { /* TIO[i] invert output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_IO_tag;

typedef union GTM_gtm_cls1_TIO1_IENDIS_union_tag { /* TIO[i] toggle enable/disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_IENDIS_tag;

typedef union GTM_gtm_cls1_TIO1_IINVERT_union_tag { /* TIO[i] toggle signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_IINVERT_tag;

typedef union GTM_gtm_cls1_TIO1_IINPUT_MODE_union_tag { /* TIO[i] enable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_IINPUT_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_ICYCLIC_MODE_union_tag { /* TIO[i] enable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_ICYCLIC_MODE_tag;

typedef union GTM_gtm_cls1_TIO1_FUPD_union_tag { /* TIO[i] force update register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_FUPD_tag;

typedef union GTM_gtm_cls1_TIO1_HW_CONF_union_tag { /* TIO[i] configuration register */
  vuint32_t R;
  struct {
    vuint32_t NTIO_CH8:2;
    vuint32_t _unused_2:2;
    vuint32_t TIO_PLUS:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_TIO1_HW_CONF_tag;

typedef union GTM_gtm_cls1_TIO1_RSEL_CTRL1_union_tag { /* TIO[i] resource selection control register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t SEL_CLKEN6_0:1;
    vuint32_t _unused_25:3;
    vuint32_t SEL_CLKEN7_0:1;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls1_TIO1_RSEL_CTRL1_tag;

typedef union GTM_gtm_cls1_TIO1_RSEL_CTRL2_union_tag { /* TIO[i] resource selection control register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t SEL_TB1_0:1;
    vuint32_t _unused_5:3;
    vuint32_t SEL_TB2_0:1;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls1_TIO1_RSEL_CTRL2_tag;

typedef union GTM_gtm_cls1_TIO1_PL_SWRST_union_tag { /* TIO[i] software reset for TIO Plus functionality */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_TIO1_PL_SWRST_tag;

typedef union GTM_gtm_cls1_CCM1_ARP0_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP0_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP0_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP0_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP1_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP1_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP1_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP1_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP2_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP2_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP2_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP2_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP3_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP3_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP3_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP3_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP4_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP4_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP4_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP4_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP5_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP5_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP5_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP5_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP6_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP6_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP6_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP6_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP7_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP7_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP7_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP7_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP8_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP8_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP8_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP8_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_ARP9_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls1_CCM1_ARP9_CTRL_tag;

typedef union GTM_gtm_cls1_CCM1_ARP9_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CCM1_ARP9_PROT_tag;

typedef union GTM_gtm_cls1_CCM1_TIO_G0_OUT_union_tag { /* CCM[i] TIO Group 0,1 Output Register */
  vuint32_t R;
  struct {
    vuint32_t TIO_G0_OUT0:1;
    vuint32_t TIO_G0_OUT1:1;
    vuint32_t TIO_G0_OUT2:1;
    vuint32_t TIO_G0_OUT3:1;
    vuint32_t TIO_G0_OUT4:1;
    vuint32_t TIO_G0_OUT5:1;
    vuint32_t TIO_G0_OUT6:1;
    vuint32_t TIO_G0_OUT7:1;
    vuint32_t _unused_8:8;
    vuint32_t TIO_G0_OUT_N0:1;
    vuint32_t TIO_G0_OUT_N1:1;
    vuint32_t TIO_G0_OUT_N2:1;
    vuint32_t TIO_G0_OUT_N3:1;
    vuint32_t TIO_G0_OUT_N4:1;
    vuint32_t TIO_G0_OUT_N5:1;
    vuint32_t TIO_G0_OUT_N6:1;
    vuint32_t TIO_G0_OUT_N7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_CCM1_TIO_G0_OUT_tag;

typedef union GTM_gtm_cls1_CCM1_HW_CONF2_union_tag { /* CCM[i] 2. Hardware Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t AXIM_ID_WIDTH:5;
    vuint32_t AXIM_PRIV_ACC:1;
    vuint32_t AXIM_SEC_ACC:1;
    vuint32_t AXIM_POSTED_WRITE:1;
    vuint32_t _unused_8:1;
    vuint32_t TIO_OUT_RST:1;
    vuint32_t _unused_10:6;
    vuint32_t AXIS_DATA_SIZE:1;
    vuint32_t _unused_17:1;
    vuint32_t AXIM_DATA_SIZE:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls1_CCM1_HW_CONF2_tag;

typedef union GTM_gtm_cls1_CCM1_AEIM_STA_union_tag { /* CCM[i] MCS Bus Master Status Register */
  vuint32_t R;
  struct {
    vuint32_t AEIM_XPT_ADDR:16;
    vuint32_t _unused_16:8;
    vuint32_t AEIM_XPT_STA:2;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls1_CCM1_AEIM_STA_tag;

typedef union GTM_gtm_cls1_CCM1_HW_CONF_union_tag { /* CCM[i] Hardware Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t GRSTEN:1;
    vuint32_t BRIDGE_MODE_RST:1;
    vuint32_t SYNC_INPUT_REG:1;
    vuint32_t CFG_CLOCK_RATE:1;
    vuint32_t ATOM_OUT_RST:1;
    vuint32_t ATOM_TRIG_CHAIN:3;
    vuint32_t TOM_OUT_RST:1;
    vuint32_t TOM_TRIG_CHAIN:3;
    vuint32_t RAM_INIT_RST:1;
    vuint32_t ERM:1;
    vuint32_t _unused_14:1;
    vuint32_t RESET_ACTIVE:1;
    vuint32_t IRQ_MODE_LEVEL:1;
    vuint32_t IRQ_MODE_PULSE:1;
    vuint32_t IRQ_MODE_PULSE_NOTIFY:1;
    vuint32_t IRQ_MODE_SINGLE_PULSE:1;
    vuint32_t ATOM_TRIG_INTCHAIN:4;
    vuint32_t TOM_TRIG_INTCHAIN:5;
    vuint32_t INT_CLK_EN_GEN:1;
    vuint32_t AEI_ADDR_PIPELINE_STAGE:1;
    vuint32_t AEI_RDATA_PIPELINE_STAGE:1;
  } B;
} GTM_gtm_cls1_CCM1_HW_CONF_tag;

typedef union GTM_gtm_cls1_CCM1_TIM_AUX_IN_SRC_union_tag { /* CCM[i] TIM AUX Input Source Register */
  vuint32_t R;
  struct {
    vuint32_t SRC_CH0:1;
    vuint32_t SRC_CH1:1;
    vuint32_t SRC_CH2:1;
    vuint32_t SRC_CH3:1;
    vuint32_t SRC_CH4:1;
    vuint32_t SRC_CH5:1;
    vuint32_t SRC_CH6:1;
    vuint32_t SRC_CH7:1;
    vuint32_t _unused_8:8;
    vuint32_t SEL_OUT_N_CH0:1;
    vuint32_t SEL_OUT_N_CH1:1;
    vuint32_t SEL_OUT_N_CH2:1;
    vuint32_t SEL_OUT_N_CH3:1;
    vuint32_t SEL_OUT_N_CH4:1;
    vuint32_t SEL_OUT_N_CH5:1;
    vuint32_t SEL_OUT_N_CH6:1;
    vuint32_t SEL_OUT_N_CH7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_CCM1_TIM_AUX_IN_SRC_tag;

typedef union GTM_gtm_cls1_CCM1_EXT_CAP_EN_union_tag { /* CCM[i] External Capture Enable Register */
  vuint32_t R;
  struct {
    vuint32_t TIM_I_EXT_CAP_EN0:1;
    vuint32_t TIM_I_EXT_CAP_EN1:1;
    vuint32_t TIM_I_EXT_CAP_EN2:1;
    vuint32_t TIM_I_EXT_CAP_EN3:1;
    vuint32_t TIM_I_EXT_CAP_EN4:1;
    vuint32_t TIM_I_EXT_CAP_EN5:1;
    vuint32_t TIM_I_EXT_CAP_EN6:1;
    vuint32_t TIM_I_EXT_CAP_EN7:1;
    vuint32_t TIM_IP1_EXT_CAP_EN0:1;
    vuint32_t TIM_IP1_EXT_CAP_EN1:1;
    vuint32_t TIM_IP1_EXT_CAP_EN2:1;
    vuint32_t TIM_IP1_EXT_CAP_EN3:1;
    vuint32_t TIM_IP1_EXT_CAP_EN4:1;
    vuint32_t TIM_IP1_EXT_CAP_EN5:1;
    vuint32_t TIM_IP1_EXT_CAP_EN6:1;
    vuint32_t TIM_IP1_EXT_CAP_EN7:1;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_CCM1_EXT_CAP_EN_tag;

typedef union GTM_gtm_cls1_CCM1_TOM_OUT_union_tag { /* CCM[i] TOM Output Register */
  vuint32_t R;
  struct {
    vuint32_t TOM_OUT0:1;
    vuint32_t TOM_OUT1:1;
    vuint32_t TOM_OUT2:1;
    vuint32_t TOM_OUT3:1;
    vuint32_t TOM_OUT4:1;
    vuint32_t TOM_OUT5:1;
    vuint32_t TOM_OUT6:1;
    vuint32_t TOM_OUT7:1;
    vuint32_t TOM_OUT8:1;
    vuint32_t TOM_OUT9:1;
    vuint32_t TOM_OUT10:1;
    vuint32_t TOM_OUT11:1;
    vuint32_t TOM_OUT12:1;
    vuint32_t TOM_OUT13:1;
    vuint32_t TOM_OUT14:1;
    vuint32_t TOM_OUT15:1;
    vuint32_t TOM_OUT_N0:1;
    vuint32_t TOM_OUT_N1:1;
    vuint32_t TOM_OUT_N2:1;
    vuint32_t TOM_OUT_N3:1;
    vuint32_t TOM_OUT_N4:1;
    vuint32_t TOM_OUT_N5:1;
    vuint32_t TOM_OUT_N6:1;
    vuint32_t TOM_OUT_N7:1;
    vuint32_t TOM_OUT_N8:1;
    vuint32_t TOM_OUT_N9:1;
    vuint32_t TOM_OUT_N10:1;
    vuint32_t TOM_OUT_N11:1;
    vuint32_t TOM_OUT_N12:1;
    vuint32_t TOM_OUT_N13:1;
    vuint32_t TOM_OUT_N14:1;
    vuint32_t TOM_OUT_N15:1;
  } B;
} GTM_gtm_cls1_CCM1_TOM_OUT_tag;

typedef union GTM_gtm_cls1_CCM1_ATOM_OUT_union_tag { /* CCM[i] ATOM Output Register */
  vuint32_t R;
  struct {
    vuint32_t ATOM_I_OUT0:1;
    vuint32_t ATOM_I_OUT1:1;
    vuint32_t ATOM_I_OUT2:1;
    vuint32_t ATOM_I_OUT3:1;
    vuint32_t ATOM_I_OUT4:1;
    vuint32_t ATOM_I_OUT5:1;
    vuint32_t ATOM_I_OUT6:1;
    vuint32_t ATOM_I_OUT7:1;
    vuint32_t ATOM_I_OUT_N0:1;
    vuint32_t ATOM_I_OUT_N1:1;
    vuint32_t ATOM_I_OUT_N2:1;
    vuint32_t ATOM_I_OUT_N3:1;
    vuint32_t ATOM_I_OUT_N4:1;
    vuint32_t ATOM_I_OUT_N5:1;
    vuint32_t ATOM_I_OUT_N6:1;
    vuint32_t ATOM_I_OUT_N7:1;
    vuint32_t ATOM_IP1_OUT0:1;
    vuint32_t ATOM_IP1_OUT1:1;
    vuint32_t ATOM_IP1_OUT2:1;
    vuint32_t ATOM_IP1_OUT3:1;
    vuint32_t ATOM_IP1_OUT4:1;
    vuint32_t ATOM_IP1_OUT5:1;
    vuint32_t ATOM_IP1_OUT6:1;
    vuint32_t ATOM_IP1_OUT7:1;
    vuint32_t ATOM_IP1_OUT_N0:1;
    vuint32_t ATOM_IP1_OUT_N1:1;
    vuint32_t ATOM_IP1_OUT_N2:1;
    vuint32_t ATOM_IP1_OUT_N3:1;
    vuint32_t ATOM_IP1_OUT_N4:1;
    vuint32_t ATOM_IP1_OUT_N5:1;
    vuint32_t ATOM_IP1_OUT_N6:1;
    vuint32_t ATOM_IP1_OUT_N7:1;
  } B;
} GTM_gtm_cls1_CCM1_ATOM_OUT_tag;

typedef union GTM_gtm_cls1_CCM1_CMU_CLK_CFG_union_tag { /* CCM[i] CMU Clock Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CLK0_SRC:2;
    vuint32_t _unused_2:2;
    vuint32_t CLK1_SRC:2;
    vuint32_t _unused_6:2;
    vuint32_t CLK2_SRC:2;
    vuint32_t _unused_10:2;
    vuint32_t CLK3_SRC:2;
    vuint32_t _unused_14:2;
    vuint32_t CLK4_SRC:2;
    vuint32_t _unused_18:2;
    vuint32_t CLK5_SRC:2;
    vuint32_t _unused_22:2;
    vuint32_t CLK6_SRC:2;
    vuint32_t _unused_26:2;
    vuint32_t CLK7_SRC:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_CCM1_CMU_CLK_CFG_tag;

typedef union GTM_gtm_cls1_CCM1_CMU_FXCLK_CFG_union_tag { /* CCM[i] CMU Fixed Clock Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t FXCLK0_SRC:4;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls1_CCM1_CMU_FXCLK_CFG_tag;

typedef union GTM_gtm_cls1_CCM1_CFG_union_tag { /* CCM[i] Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t EN_TIM:1;
    vuint32_t EN_TOM_SPE_TDTM:1;
    vuint32_t EN_ATOM_ADTM:1;
    vuint32_t EN_MCS:1;
    vuint32_t _unused_4:3;
    vuint32_t EN_CMP_MON:1;
    vuint32_t EN_TIO_DTM:1;
    vuint32_t _unused_9:7;
    vuint32_t CLS_CLK_DIV:2;
    vuint32_t _unused_18:12;
    vuint32_t TBU_DIR1:1;
    vuint32_t TBU_DIR2:1;
  } B;
} GTM_gtm_cls1_CCM1_CFG_tag;

typedef union GTM_gtm_cls1_CCM1_PROT_union_tag { /* CCM[i] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t CLS_PROT:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls1_CCM1_PROT_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CTRL_union_tag { /* CDTM[i]_DTM[d] global configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t CLK_SEL:2;
    vuint32_t DTM_SEL:2;
    vuint32_t UPD_MODE:3;
    vuint32_t CH_SHUTOFF_EN:1;
    vuint32_t SR_UPD_EN:1;
    vuint32_t _unused_9:7;
    vuint32_t SHUT_OFF_RST:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CTRL_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL1_union_tag { /* CDTM[i]_DTM[d] channel control register 1 */
  vuint32_t R;
  struct {
    vuint32_t O1SEL_0:1;
    vuint32_t _unused_1:2;
    vuint32_t SWAP_0:1;
    vuint32_t O1F_0:2;
    vuint32_t XDT_EN_0_1:1;
    vuint32_t _unused_7:1;
    vuint32_t O1SEL_1:1;
    vuint32_t I1SEL_1:1;
    vuint32_t SH_EN_1:1;
    vuint32_t SWAP_1:1;
    vuint32_t O1F_1:2;
    vuint32_t _unused_14:2;
    vuint32_t O1SEL_2:1;
    vuint32_t I1SEL_2:1;
    vuint32_t SH_EN_2:1;
    vuint32_t SWAP_2:1;
    vuint32_t O1F_2:2;
    vuint32_t XDT_EN_2_3:1;
    vuint32_t _unused_23:1;
    vuint32_t O1SEL_3:1;
    vuint32_t I1SEL_3:1;
    vuint32_t SH_EN_3:1;
    vuint32_t SWAP_3:1;
    vuint32_t O1F_3:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL1_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL2_union_tag { /* CDTM[i]_DTM[d] channel control register 2 */
  vuint32_t R;
  struct {
    vuint32_t POL0_0:1;
    vuint32_t OC0_0:1;
    vuint32_t SL0_0:1;
    vuint32_t DT0_0:1;
    vuint32_t POL1_0:1;
    vuint32_t OC1_0:1;
    vuint32_t SL1_0:1;
    vuint32_t DT1_0:1;
    vuint32_t POL0_1:1;
    vuint32_t OC0_1:1;
    vuint32_t SL0_1:1;
    vuint32_t DT0_1:1;
    vuint32_t POL1_1:1;
    vuint32_t OC1_1:1;
    vuint32_t SL1_1:1;
    vuint32_t DT1_1:1;
    vuint32_t POL0_2:1;
    vuint32_t OC0_2:1;
    vuint32_t SL0_2:1;
    vuint32_t DT0_2:1;
    vuint32_t POL1_2:1;
    vuint32_t OC1_2:1;
    vuint32_t SL1_2:1;
    vuint32_t DT1_2:1;
    vuint32_t POL0_3:1;
    vuint32_t OC0_3:1;
    vuint32_t SL0_3:1;
    vuint32_t DT0_3:1;
    vuint32_t POL1_3:1;
    vuint32_t OC1_3:1;
    vuint32_t SL1_3:1;
    vuint32_t DT1_3:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL2_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL2_SR_union_tag { /* CDTM[i] DTM[j] channel control register 2 shadow */
  vuint32_t R;
  struct {
    vuint32_t POL0_0_SR:1;
    vuint32_t OC0_0_SR:1;
    vuint32_t SL0_0_SR:1;
    vuint32_t DT0_0_SR:1;
    vuint32_t POL1_0_SR:1;
    vuint32_t OC1_0_SR:1;
    vuint32_t SL1_0_SR:1;
    vuint32_t DT1_0_SR:1;
    vuint32_t POL0_1_SR:1;
    vuint32_t OC0_1_SR:1;
    vuint32_t SL0_1_SR:1;
    vuint32_t DT0_1_SR:1;
    vuint32_t POL1_1_SR:1;
    vuint32_t OC1_1_SR:1;
    vuint32_t SL1_1_SR:1;
    vuint32_t DT1_1_SR:1;
    vuint32_t POL0_2_SR:1;
    vuint32_t OC0_2_SR:1;
    vuint32_t SL0_2_SR:1;
    vuint32_t DT0_2_SR:1;
    vuint32_t POL1_2_SR:1;
    vuint32_t OC1_2_SR:1;
    vuint32_t SL1_2_SR:1;
    vuint32_t DT1_2_SR:1;
    vuint32_t POL0_3_SR:1;
    vuint32_t OC0_3_SR:1;
    vuint32_t SL0_3_SR:1;
    vuint32_t DT0_3_SR:1;
    vuint32_t POL1_3_SR:1;
    vuint32_t OC1_3_SR:1;
    vuint32_t SL1_3_SR:1;
    vuint32_t DT1_3_SR:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL2_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_PS_CTRL_union_tag { /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t RELBLK:10;
    vuint32_t _unused_10:6;
    vuint32_t PSU_IN_SEL:1;
    vuint32_t IN_POL:1;
    vuint32_t TIM_SEL:1;
    vuint32_t _unused_19:1;
    vuint32_t SHIFT_SEL:2;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_PS_CTRL_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH_DTV_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE:13;
    vuint32_t _unused_13:3;
    vuint32_t RELFALL:13;
    vuint32_t _unused_29:2;
    vuint32_t HRES:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH_DTV_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH_SR_union_tag { /* CDTM[i]_DTM[d] channel shadow register */
  vuint32_t R;
  struct {
    vuint32_t SL0_0_SR_SR:1;
    vuint32_t SL1_0_SR_SR:1;
    vuint32_t SL0_1_SR_SR:1;
    vuint32_t SL1_1_SR_SR:1;
    vuint32_t SL0_2_SR_SR:1;
    vuint32_t SL1_2_SR_SR:1;
    vuint32_t SL0_3_SR_SR:1;
    vuint32_t SL1_3_SR_SR:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL3_union_tag { /* CDTM[i]_DTM[d] channel control register 3 */
  vuint32_t R;
  struct {
    vuint32_t CII0:1;
    vuint32_t CIS0:1;
    vuint32_t TSEL0_0:1;
    vuint32_t TSEL1_0:1;
    vuint32_t _unused_4:4;
    vuint32_t CII1:1;
    vuint32_t CIS1:1;
    vuint32_t TSEL0_1:1;
    vuint32_t TSEL1_1:1;
    vuint32_t _unused_12:4;
    vuint32_t CII2:1;
    vuint32_t CIS2:1;
    vuint32_t TSEL0_2:1;
    vuint32_t TSEL1_2:1;
    vuint32_t _unused_20:4;
    vuint32_t CII3:1;
    vuint32_t CIS3:1;
    vuint32_t TSEL0_3:1;
    vuint32_t TSEL1_3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL3_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CTRL2_union_tag { /* CDTM[i]_DTM[d] global configuration and control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SHUTOFF_SEL_0:3;
    vuint32_t SHUTOFF_POL_0:1;
    vuint32_t UPD_MODE_0:2;
    vuint32_t SHUT_OFF_RST_0:1;
    vuint32_t WR_EN_0:1;
    vuint32_t SHUTOFF_SEL_1:3;
    vuint32_t SHUTOFF_POL_1:1;
    vuint32_t UPD_MODE_1:2;
    vuint32_t SHUT_OFF_RST_1:1;
    vuint32_t WR_EN_1:1;
    vuint32_t SHUTOFF_SEL_2:3;
    vuint32_t SHUTOFF_POL_2:1;
    vuint32_t UPD_MODE_2:2;
    vuint32_t SHUT_OFF_RST_2:1;
    vuint32_t WR_EN_2:1;
    vuint32_t SHUTOFF_SEL_3:3;
    vuint32_t SHUTOFF_POL_3:1;
    vuint32_t UPD_MODE_3:2;
    vuint32_t SHUT_OFF_RST_3:1;
    vuint32_t WR_EN_3:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CTRL2_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH0_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH0_DTV_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH1_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH1_DTV_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH2_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH2_DTV_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM4_CH3_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM4_CH3_DTV_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CTRL_union_tag { /* CDTM[i]_DTM[d] global configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t CLK_SEL:2;
    vuint32_t DTM_SEL:2;
    vuint32_t UPD_MODE:3;
    vuint32_t CH_SHUTOFF_EN:1;
    vuint32_t SR_UPD_EN:1;
    vuint32_t _unused_9:7;
    vuint32_t SHUT_OFF_RST:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CTRL_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL1_union_tag { /* CDTM[i]_DTM[d] channel control register 1 */
  vuint32_t R;
  struct {
    vuint32_t O1SEL_0:1;
    vuint32_t I1SEL_0:1;
    vuint32_t _unused_2:1;
    vuint32_t SWAP_0:1;
    vuint32_t O1F_0:2;
    vuint32_t XDT_EN_0_1:1;
    vuint32_t _unused_7:1;
    vuint32_t O1SEL_1:1;
    vuint32_t I1SEL_1:1;
    vuint32_t SH_EN_1:1;
    vuint32_t SWAP_1:1;
    vuint32_t O1F_1:2;
    vuint32_t _unused_14:2;
    vuint32_t O1SEL_2:1;
    vuint32_t I1SEL_2:1;
    vuint32_t SH_EN_2:1;
    vuint32_t SWAP_2:1;
    vuint32_t O1F_2:2;
    vuint32_t XDT_EN_2_3:1;
    vuint32_t _unused_23:1;
    vuint32_t O1SEL_3:1;
    vuint32_t I1SEL_3:1;
    vuint32_t SH_EN_3:1;
    vuint32_t SWAP_3:1;
    vuint32_t O1F_3:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL1_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL2_union_tag { /* CDTM[i]_DTM[d] channel control register 2 */
  vuint32_t R;
  struct {
    vuint32_t POL0_0:1;
    vuint32_t OC0_0:1;
    vuint32_t SL0_0:1;
    vuint32_t DT0_0:1;
    vuint32_t POL1_0:1;
    vuint32_t OC1_0:1;
    vuint32_t SL1_0:1;
    vuint32_t DT1_0:1;
    vuint32_t POL0_1:1;
    vuint32_t OC0_1:1;
    vuint32_t SL0_1:1;
    vuint32_t DT0_1:1;
    vuint32_t POL1_1:1;
    vuint32_t OC1_1:1;
    vuint32_t SL1_1:1;
    vuint32_t DT1_1:1;
    vuint32_t POL0_2:1;
    vuint32_t OC0_2:1;
    vuint32_t SL0_2:1;
    vuint32_t DT0_2:1;
    vuint32_t POL1_2:1;
    vuint32_t OC1_2:1;
    vuint32_t SL1_2:1;
    vuint32_t DT1_2:1;
    vuint32_t POL0_3:1;
    vuint32_t OC0_3:1;
    vuint32_t SL0_3:1;
    vuint32_t DT0_3:1;
    vuint32_t POL1_3:1;
    vuint32_t OC1_3:1;
    vuint32_t SL1_3:1;
    vuint32_t DT1_3:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL2_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL2_SR_union_tag { /* CDTM[i] DTM[j] channel control register 2 shadow */
  vuint32_t R;
  struct {
    vuint32_t POL0_0_SR:1;
    vuint32_t OC0_0_SR:1;
    vuint32_t SL0_0_SR:1;
    vuint32_t DT0_0_SR:1;
    vuint32_t POL1_0_SR:1;
    vuint32_t OC1_0_SR:1;
    vuint32_t SL1_0_SR:1;
    vuint32_t DT1_0_SR:1;
    vuint32_t POL0_1_SR:1;
    vuint32_t OC0_1_SR:1;
    vuint32_t SL0_1_SR:1;
    vuint32_t DT0_1_SR:1;
    vuint32_t POL1_1_SR:1;
    vuint32_t OC1_1_SR:1;
    vuint32_t SL1_1_SR:1;
    vuint32_t DT1_1_SR:1;
    vuint32_t POL0_2_SR:1;
    vuint32_t OC0_2_SR:1;
    vuint32_t SL0_2_SR:1;
    vuint32_t DT0_2_SR:1;
    vuint32_t POL1_2_SR:1;
    vuint32_t OC1_2_SR:1;
    vuint32_t SL1_2_SR:1;
    vuint32_t DT1_2_SR:1;
    vuint32_t POL0_3_SR:1;
    vuint32_t OC0_3_SR:1;
    vuint32_t SL0_3_SR:1;
    vuint32_t DT0_3_SR:1;
    vuint32_t POL1_3_SR:1;
    vuint32_t OC1_3_SR:1;
    vuint32_t SL1_3_SR:1;
    vuint32_t DT1_3_SR:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL2_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_PS_CTRL_union_tag { /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t RELBLK:10;
    vuint32_t _unused_10:6;
    vuint32_t PSU_IN_SEL:1;
    vuint32_t IN_POL:1;
    vuint32_t TIM_SEL:1;
    vuint32_t _unused_19:1;
    vuint32_t SHIFT_SEL:2;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_PS_CTRL_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH_DTV_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE:13;
    vuint32_t _unused_13:3;
    vuint32_t RELFALL:13;
    vuint32_t _unused_29:2;
    vuint32_t HRES:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH_DTV_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH_SR_union_tag { /* CDTM[i]_DTM[d] channel shadow register */
  vuint32_t R;
  struct {
    vuint32_t SL0_0_SR_SR:1;
    vuint32_t SL1_0_SR_SR:1;
    vuint32_t SL0_1_SR_SR:1;
    vuint32_t SL1_1_SR_SR:1;
    vuint32_t SL0_2_SR_SR:1;
    vuint32_t SL1_2_SR_SR:1;
    vuint32_t SL0_3_SR_SR:1;
    vuint32_t SL1_3_SR_SR:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL3_union_tag { /* CDTM[i]_DTM[d] channel control register 3 */
  vuint32_t R;
  struct {
    vuint32_t CII0:1;
    vuint32_t CIS0:1;
    vuint32_t TSEL0_0:1;
    vuint32_t TSEL1_0:1;
    vuint32_t _unused_4:4;
    vuint32_t CII1:1;
    vuint32_t CIS1:1;
    vuint32_t TSEL0_1:1;
    vuint32_t TSEL1_1:1;
    vuint32_t _unused_12:4;
    vuint32_t CII2:1;
    vuint32_t CIS2:1;
    vuint32_t TSEL0_2:1;
    vuint32_t TSEL1_2:1;
    vuint32_t _unused_20:4;
    vuint32_t CII3:1;
    vuint32_t CIS3:1;
    vuint32_t TSEL0_3:1;
    vuint32_t TSEL1_3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL3_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CTRL2_union_tag { /* CDTM[i]_DTM[d] global configuration and control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SHUTOFF_SEL_0:3;
    vuint32_t SHUTOFF_POL_0:1;
    vuint32_t UPD_MODE_0:2;
    vuint32_t SHUT_OFF_RST_0:1;
    vuint32_t WR_EN_0:1;
    vuint32_t SHUTOFF_SEL_1:3;
    vuint32_t SHUTOFF_POL_1:1;
    vuint32_t UPD_MODE_1:2;
    vuint32_t SHUT_OFF_RST_1:1;
    vuint32_t WR_EN_1:1;
    vuint32_t SHUTOFF_SEL_2:3;
    vuint32_t SHUTOFF_POL_2:1;
    vuint32_t UPD_MODE_2:2;
    vuint32_t SHUT_OFF_RST_2:1;
    vuint32_t WR_EN_2:1;
    vuint32_t SHUTOFF_SEL_3:3;
    vuint32_t SHUTOFF_POL_3:1;
    vuint32_t UPD_MODE_3:2;
    vuint32_t SHUT_OFF_RST_3:1;
    vuint32_t WR_EN_3:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CTRL2_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH0_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH0_DTV_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH1_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH1_DTV_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH2_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH2_DTV_SR_tag;

typedef union GTM_gtm_cls1_CDTM1_DTM5_CH3_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls1_CDTM1_DTM5_CH3_DTV_SR_tag;

typedef union GTM_gtm_cls1_SPE1_CTRL_STAT_union_tag { /* SPE[i] Control Status Register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t SIE0:1;
    vuint32_t SIE1:1;
    vuint32_t SIE2:1;
    vuint32_t TRIG_SEL:2;
    vuint32_t TIM_SEL:1;
    vuint32_t FSOM:1;
    vuint32_t SPE_PAT_PTR:3;
    vuint32_t _unused_11:1;
    vuint32_t AIP:3;
    vuint32_t ADIR:1;
    vuint32_t PIP:3;
    vuint32_t PDIR:1;
    vuint32_t NIP:3;
    vuint32_t ETRIG_SEL:1;
    vuint32_t FSOL:8;
  } B;
} GTM_gtm_cls1_SPE1_CTRL_STAT_tag;

typedef union GTM_gtm_cls1_SPE1_PAT_union_tag { /* SPE[i] Input Pattern Definition Register */
  vuint32_t R;
  struct {
    vuint32_t IP0_VAL:1;
    vuint32_t IP0_PAT:3;
    vuint32_t IP1_VAL:1;
    vuint32_t IP1_PAT:3;
    vuint32_t IP2_VAL:1;
    vuint32_t IP2_PAT:3;
    vuint32_t IP3_VAL:1;
    vuint32_t IP3_PAT:3;
    vuint32_t IP4_VAL:1;
    vuint32_t IP4_PAT:3;
    vuint32_t IP5_VAL:1;
    vuint32_t IP5_PAT:3;
    vuint32_t IP6_VAL:1;
    vuint32_t IP6_PAT:3;
    vuint32_t IP7_VAL:1;
    vuint32_t IP7_PAT:3;
  } B;
} GTM_gtm_cls1_SPE1_PAT_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_PAT0_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_PAT0_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_PAT1_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_PAT1_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_PAT2_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_PAT2_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_PAT3_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_PAT3_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_PAT4_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_PAT4_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_PAT5_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_PAT5_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_PAT6_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_PAT6_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_PAT7_union_tag { /* SPE[i] Output Definition Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_PAT0:2;
    vuint32_t SPE_OUT_PAT1:2;
    vuint32_t SPE_OUT_PAT2:2;
    vuint32_t SPE_OUT_PAT3:2;
    vuint32_t SPE_OUT_PAT4:2;
    vuint32_t SPE_OUT_PAT5:2;
    vuint32_t SPE_OUT_PAT6:2;
    vuint32_t SPE_OUT_PAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_PAT7_tag;

typedef union GTM_gtm_cls1_SPE1_OUT_CTRL_union_tag { /* SPE[i] Output Control Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_OUT_CTRL0:2;
    vuint32_t SPE_OUT_CTRL1:2;
    vuint32_t SPE_OUT_CTRL2:2;
    vuint32_t SPE_OUT_CTRL3:2;
    vuint32_t SPE_OUT_CTRL4:2;
    vuint32_t SPE_OUT_CTRL5:2;
    vuint32_t SPE_OUT_CTRL6:2;
    vuint32_t SPE_OUT_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_SPE1_OUT_CTRL_tag;

typedef union GTM_gtm_cls1_SPE1_IRQ_NOTIFY_union_tag { /* SPE[i] Interrupt Notification Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_NIPD:1;
    vuint32_t SPE_DCHG:1;
    vuint32_t SPE_PERR:1;
    vuint32_t SPE_BIS:1;
    vuint32_t SPE_RCMP:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_SPE1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls1_SPE1_IRQ_EN_union_tag { /* SPE[i] Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_NIPD_IRQ_EN:1;
    vuint32_t SPE_DCHG_IRQ_EN:1;
    vuint32_t SPE_PERR_IRQ_EN:1;
    vuint32_t SPE_BIS_IRQ_EN:1;
    vuint32_t SPE_RCMP_IRQ_EN:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_SPE1_IRQ_EN_tag;

typedef union GTM_gtm_cls1_SPE1_IRQ_FORCINT_union_tag { /* SPE[i] Interrupt Generation By Software */
  vuint32_t R;
  struct {
    vuint32_t TRG_SPE_NIPD:1;
    vuint32_t TRG_SPE_DCHG:1;
    vuint32_t TRG_SPE_PERR:1;
    vuint32_t TRG_SPE_BIS:1;
    vuint32_t TRG_SPE_RCMP:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_SPE1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls1_SPE1_IRQ_MODE_union_tag { /* SPE[i] Interrupt Mode Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls1_SPE1_IRQ_MODE_tag;

typedef union GTM_gtm_cls1_SPE1_EIRQ_EN_union_tag { /* SPE[i] Error Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_NIPD_EIRQ_EN:1;
    vuint32_t SPE_DCHG_EIRQ_EN:1;
    vuint32_t SPE_PERR_EIRQ_EN:1;
    vuint32_t SPE_BIS_EIRQ_EN:1;
    vuint32_t SPE_RCMP_EIRQ_EN:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls1_SPE1_EIRQ_EN_tag;

typedef union GTM_gtm_cls1_SPE1_REV_CNT_union_tag { /* SPE[i] Input Revolution Counter */
  vuint32_t R;
  struct {
    vuint32_t REV_CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_SPE1_REV_CNT_tag;

typedef union GTM_gtm_cls1_SPE1_REV_CMP_union_tag { /* SPE[i] Revolution Counter Compare Value */
  vuint32_t R;
  struct {
    vuint32_t REV_CMP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls1_SPE1_REV_CMP_tag;

typedef union GTM_gtm_cls1_SPE1_CTRL_STAT2_union_tag { /* SPE[i] Control Status Register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t SPE_PAT_PTR_BWD:3;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls1_SPE1_CTRL_STAT2_tag;

typedef union GTM_gtm_cls1_SPE1_CMD_union_tag { /* SPE[i] Command Register */
  vuint32_t R;
  struct {
    vuint32_t SPE_CTRL_CMD:2;
    vuint32_t _unused_2:14;
    vuint32_t SPE_UPD_TRIG:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls1_SPE1_CMD_tag;

typedef union GTM_gtm_cls1_AXIM1_FREE_union_tag { /* AXIM[i] slot allocation status. */
  vuint32_t R;
  struct {
    vuint32_t FREE0:1;
    vuint32_t FREE1:1;
    vuint32_t FREE2:1;
    vuint32_t FREE3:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls1_AXIM1_FREE_tag;

typedef union GTM_gtm_cls1_AXIM1_REQUEST_union_tag { /* AXIM[i] slot request (allocation). */
  vuint32_t R;
  struct {
    vuint32_t REQ1HOT0:1;
    vuint32_t REQ1HOT1:1;
    vuint32_t REQ1HOT2:1;
    vuint32_t REQ1HOT3:1;
    vuint32_t _unused_4:20;
    vuint32_t REQID:8;
  } B;
} GTM_gtm_cls1_AXIM1_REQUEST_tag;

typedef union GTM_gtm_cls1_AXIM1_RELEASE_union_tag { /* AXIM[i] slot release (de-allocation). */
  vuint32_t R;
  struct {
    vuint32_t RELREQ0:1;
    vuint32_t RELREQ1:1;
    vuint32_t RELREQ2:1;
    vuint32_t RELREQ3:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls1_AXIM1_RELEASE_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT0_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT0_ADDR_LOW_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT0_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT0_DATA_LOW_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT0_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT0_CFG1_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT0_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT0_CFG2_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT0_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT0_STATUS_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT1_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT1_ADDR_LOW_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT1_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT1_DATA_LOW_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT1_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT1_CFG1_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT1_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT1_CFG2_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT1_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT1_STATUS_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT2_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT2_ADDR_LOW_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT2_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT2_DATA_LOW_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT2_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT2_CFG1_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT2_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT2_CFG2_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT2_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT2_STATUS_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT3_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT3_ADDR_LOW_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT3_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT3_DATA_LOW_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT3_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT3_CFG1_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT3_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT3_CFG2_tag;

typedef union GTM_gtm_cls1_AXIM1_SLOT3_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls1_AXIM1_SLOT3_STATUS_tag;

typedef union GTM_gtm_cls1_MCS1_MEM_union_tag { /* MCS[i] memory region */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls1_MCS1_MEM_tag;

struct GTM_gtm_cls1_tag {
  uint8_t GTM_gtm_cls1_reserved0[1664];
  GTM_gtm_cls1_MON_STATUS_tag MON_STATUS; /* MON status register */
  GTM_gtm_cls1_MON_ACTIVITY_0_tag MON_ACTIVITY_0; /* MON activity register 0 */
  uint8_t GTM_gtm_cls1_reserved1[4];
  GTM_gtm_cls1_MON_ACTIVITY_MCS0_tag MON_ACTIVITY_MCS0; /* MON activity register for MCS [j] */
  GTM_gtm_cls1_MON_ACTIVITY_MCS1_tag MON_ACTIVITY_MCS1; /* MON activity register for MCS [j] */
  GTM_gtm_cls1_MON_ACTIVITY_MCS2_tag MON_ACTIVITY_MCS2; /* MON activity register for MCS [j] */
  GTM_gtm_cls1_MON_ACTIVITY_MCS3_tag MON_ACTIVITY_MCS3; /* MON activity register for MCS [j] */
  uint8_t GTM_gtm_cls1_reserved2[36];
  GTM_gtm_cls1_CMP_EN_tag CMP_EN;      /* CMP comparator enable register */
  GTM_gtm_cls1_CMP_IRQ_NOTIFY_tag CMP_IRQ_NOTIFY; /* CMP event notification register */
  GTM_gtm_cls1_CMP_IRQ_EN_tag CMP_IRQ_EN; /* CMP interrupt enable register */
  GTM_gtm_cls1_CMP_IRQ_FORCINT_tag CMP_IRQ_FORCINT; /* CMP interrupt force register */
  GTM_gtm_cls1_CMP_IRQ_MODE_tag CMP_IRQ_MODE; /* CMP interrupt mode configuration register */
  GTM_gtm_cls1_CMP_EIRQ_EN_tag CMP_EIRQ_EN; /* CMP error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved3[296];
  GTM_gtm_cls1_TIM1_CH0_GPR0_tag TIM1_CH0_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH0_GPR1_tag TIM1_CH0_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH0_CNT_tag TIM1_CH0_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls1_TIM1_CH0_ECNT_tag TIM1_CH0_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls1_TIM1_CH0_CNTS_tag TIM1_CH0_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls1_TIM1_CH0_TDUC_tag TIM1_CH0_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls1_TIM1_CH0_TDUV_tag TIM1_CH0_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls1_TIM1_CH0_FLT_RE_tag TIM1_CH0_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls1_TIM1_CH0_FLT_FE_tag TIM1_CH0_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls1_TIM1_CH0_CTRL_tag TIM1_CH0_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls1_TIM1_CH0_ECTRL_tag TIM1_CH0_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls1_TIM1_CH0_IRQ_NOTIFY_tag TIM1_CH0_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TIM1_CH0_IRQ_EN_tag TIM1_CH0_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TIM1_CH0_IRQ_FORCINT_tag TIM1_CH0_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TIM1_CH0_IRQ_MODE_tag TIM1_CH0_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_TIM1_CH0_EIRQ_EN_tag TIM1_CH0_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved4[64];
  GTM_gtm_cls1_TIM1_CH1_GPR0_tag TIM1_CH1_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH1_GPR1_tag TIM1_CH1_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH1_CNT_tag TIM1_CH1_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls1_TIM1_CH1_ECNT_tag TIM1_CH1_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls1_TIM1_CH1_CNTS_tag TIM1_CH1_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls1_TIM1_CH1_TDUC_tag TIM1_CH1_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls1_TIM1_CH1_TDUV_tag TIM1_CH1_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls1_TIM1_CH1_FLT_RE_tag TIM1_CH1_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls1_TIM1_CH1_FLT_FE_tag TIM1_CH1_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls1_TIM1_CH1_CTRL_tag TIM1_CH1_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls1_TIM1_CH1_ECTRL_tag TIM1_CH1_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls1_TIM1_CH1_IRQ_NOTIFY_tag TIM1_CH1_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TIM1_CH1_IRQ_EN_tag TIM1_CH1_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TIM1_CH1_IRQ_FORCINT_tag TIM1_CH1_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TIM1_CH1_IRQ_MODE_tag TIM1_CH1_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_TIM1_CH1_EIRQ_EN_tag TIM1_CH1_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved5[64];
  GTM_gtm_cls1_TIM1_CH2_GPR0_tag TIM1_CH2_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH2_GPR1_tag TIM1_CH2_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH2_CNT_tag TIM1_CH2_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls1_TIM1_CH2_ECNT_tag TIM1_CH2_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls1_TIM1_CH2_CNTS_tag TIM1_CH2_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls1_TIM1_CH2_TDUC_tag TIM1_CH2_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls1_TIM1_CH2_TDUV_tag TIM1_CH2_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls1_TIM1_CH2_FLT_RE_tag TIM1_CH2_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls1_TIM1_CH2_FLT_FE_tag TIM1_CH2_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls1_TIM1_CH2_CTRL_tag TIM1_CH2_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls1_TIM1_CH2_ECTRL_tag TIM1_CH2_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls1_TIM1_CH2_IRQ_NOTIFY_tag TIM1_CH2_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TIM1_CH2_IRQ_EN_tag TIM1_CH2_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TIM1_CH2_IRQ_FORCINT_tag TIM1_CH2_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TIM1_CH2_IRQ_MODE_tag TIM1_CH2_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_TIM1_CH2_EIRQ_EN_tag TIM1_CH2_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved6[64];
  GTM_gtm_cls1_TIM1_CH3_GPR0_tag TIM1_CH3_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH3_GPR1_tag TIM1_CH3_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH3_CNT_tag TIM1_CH3_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls1_TIM1_CH3_ECNT_tag TIM1_CH3_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls1_TIM1_CH3_CNTS_tag TIM1_CH3_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls1_TIM1_CH3_TDUC_tag TIM1_CH3_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls1_TIM1_CH3_TDUV_tag TIM1_CH3_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls1_TIM1_CH3_FLT_RE_tag TIM1_CH3_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls1_TIM1_CH3_FLT_FE_tag TIM1_CH3_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls1_TIM1_CH3_CTRL_tag TIM1_CH3_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls1_TIM1_CH3_ECTRL_tag TIM1_CH3_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls1_TIM1_CH3_IRQ_NOTIFY_tag TIM1_CH3_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TIM1_CH3_IRQ_EN_tag TIM1_CH3_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TIM1_CH3_IRQ_FORCINT_tag TIM1_CH3_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TIM1_CH3_IRQ_MODE_tag TIM1_CH3_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_TIM1_CH3_EIRQ_EN_tag TIM1_CH3_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved7[64];
  GTM_gtm_cls1_TIM1_CH4_GPR0_tag TIM1_CH4_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH4_GPR1_tag TIM1_CH4_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH4_CNT_tag TIM1_CH4_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls1_TIM1_CH4_ECNT_tag TIM1_CH4_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls1_TIM1_CH4_CNTS_tag TIM1_CH4_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls1_TIM1_CH4_TDUC_tag TIM1_CH4_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls1_TIM1_CH4_TDUV_tag TIM1_CH4_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls1_TIM1_CH4_FLT_RE_tag TIM1_CH4_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls1_TIM1_CH4_FLT_FE_tag TIM1_CH4_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls1_TIM1_CH4_CTRL_tag TIM1_CH4_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls1_TIM1_CH4_ECTRL_tag TIM1_CH4_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls1_TIM1_CH4_IRQ_NOTIFY_tag TIM1_CH4_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TIM1_CH4_IRQ_EN_tag TIM1_CH4_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TIM1_CH4_IRQ_FORCINT_tag TIM1_CH4_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TIM1_CH4_IRQ_MODE_tag TIM1_CH4_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_TIM1_CH4_EIRQ_EN_tag TIM1_CH4_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved8[64];
  GTM_gtm_cls1_TIM1_CH5_GPR0_tag TIM1_CH5_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH5_GPR1_tag TIM1_CH5_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH5_CNT_tag TIM1_CH5_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls1_TIM1_CH5_ECNT_tag TIM1_CH5_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls1_TIM1_CH5_CNTS_tag TIM1_CH5_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls1_TIM1_CH5_TDUC_tag TIM1_CH5_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls1_TIM1_CH5_TDUV_tag TIM1_CH5_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls1_TIM1_CH5_FLT_RE_tag TIM1_CH5_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls1_TIM1_CH5_FLT_FE_tag TIM1_CH5_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls1_TIM1_CH5_CTRL_tag TIM1_CH5_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls1_TIM1_CH5_ECTRL_tag TIM1_CH5_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls1_TIM1_CH5_IRQ_NOTIFY_tag TIM1_CH5_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TIM1_CH5_IRQ_EN_tag TIM1_CH5_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TIM1_CH5_IRQ_FORCINT_tag TIM1_CH5_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TIM1_CH5_IRQ_MODE_tag TIM1_CH5_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_TIM1_CH5_EIRQ_EN_tag TIM1_CH5_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved9[64];
  GTM_gtm_cls1_TIM1_CH6_GPR0_tag TIM1_CH6_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH6_GPR1_tag TIM1_CH6_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH6_CNT_tag TIM1_CH6_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls1_TIM1_CH6_ECNT_tag TIM1_CH6_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls1_TIM1_CH6_CNTS_tag TIM1_CH6_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls1_TIM1_CH6_TDUC_tag TIM1_CH6_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls1_TIM1_CH6_TDUV_tag TIM1_CH6_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls1_TIM1_CH6_FLT_RE_tag TIM1_CH6_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls1_TIM1_CH6_FLT_FE_tag TIM1_CH6_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls1_TIM1_CH6_CTRL_tag TIM1_CH6_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls1_TIM1_CH6_ECTRL_tag TIM1_CH6_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls1_TIM1_CH6_IRQ_NOTIFY_tag TIM1_CH6_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TIM1_CH6_IRQ_EN_tag TIM1_CH6_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TIM1_CH6_IRQ_FORCINT_tag TIM1_CH6_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TIM1_CH6_IRQ_MODE_tag TIM1_CH6_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_TIM1_CH6_EIRQ_EN_tag TIM1_CH6_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved10[64];
  GTM_gtm_cls1_TIM1_CH7_GPR0_tag TIM1_CH7_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH7_GPR1_tag TIM1_CH7_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls1_TIM1_CH7_CNT_tag TIM1_CH7_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls1_TIM1_CH7_ECNT_tag TIM1_CH7_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls1_TIM1_CH7_CNTS_tag TIM1_CH7_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls1_TIM1_CH7_TDUC_tag TIM1_CH7_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls1_TIM1_CH7_TDUV_tag TIM1_CH7_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls1_TIM1_CH7_FLT_RE_tag TIM1_CH7_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls1_TIM1_CH7_FLT_FE_tag TIM1_CH7_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls1_TIM1_CH7_CTRL_tag TIM1_CH7_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls1_TIM1_CH7_ECTRL_tag TIM1_CH7_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls1_TIM1_CH7_IRQ_NOTIFY_tag TIM1_CH7_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TIM1_CH7_IRQ_EN_tag TIM1_CH7_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TIM1_CH7_IRQ_FORCINT_tag TIM1_CH7_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TIM1_CH7_IRQ_MODE_tag TIM1_CH7_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_TIM1_CH7_EIRQ_EN_tag TIM1_CH7_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved11[64];
  GTM_gtm_cls1_TIM1_INP_VAL_tag TIM1_INP_VAL; /* TIM[i] input value observation register */
  GTM_gtm_cls1_TIM1_IN_SRC_tag TIM1_IN_SRC; /* TIM[i] AUX IN source selection register */
  GTM_gtm_cls1_TIM1_RST_tag TIM1_RST;  /* TIM[i] global software reset register */
  uint8_t GTM_gtm_cls1_reserved12[1012];
  GTM_gtm_cls1_TOM1_CH0_CTRL_tag TOM1_CH0_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH0_SR0_tag TOM1_CH0_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH0_SR1_tag TOM1_CH0_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH0_CM0_tag TOM1_CH0_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH0_CM1_tag TOM1_CH0_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH0_CN0_tag TOM1_CH0_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH0_STAT_tag TOM1_CH0_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH0_IRQ_NOTIFY_tag TOM1_CH0_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH0_IRQ_EN_tag TOM1_CH0_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH0_IRQ_FORCINT_tag TOM1_CH0_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH0_IRQ_MODE_tag TOM1_CH0_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved13[4];
  GTM_gtm_cls1_TOM1_CH0_CTRL_SR_tag TOM1_CH0_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved14[12];
  GTM_gtm_cls1_TOM1_CH1_CTRL_tag TOM1_CH1_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH1_SR0_tag TOM1_CH1_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH1_SR1_tag TOM1_CH1_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH1_CM0_tag TOM1_CH1_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH1_CM1_tag TOM1_CH1_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH1_CN0_tag TOM1_CH1_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH1_STAT_tag TOM1_CH1_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH1_IRQ_NOTIFY_tag TOM1_CH1_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH1_IRQ_EN_tag TOM1_CH1_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH1_IRQ_FORCINT_tag TOM1_CH1_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH1_IRQ_MODE_tag TOM1_CH1_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved15[4];
  GTM_gtm_cls1_TOM1_CH1_CTRL_SR_tag TOM1_CH1_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved16[12];
  GTM_gtm_cls1_TOM1_CH2_CTRL_tag TOM1_CH2_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH2_SR0_tag TOM1_CH2_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH2_SR1_tag TOM1_CH2_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH2_CM0_tag TOM1_CH2_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH2_CM1_tag TOM1_CH2_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH2_CN0_tag TOM1_CH2_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH2_STAT_tag TOM1_CH2_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH2_IRQ_NOTIFY_tag TOM1_CH2_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH2_IRQ_EN_tag TOM1_CH2_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH2_IRQ_FORCINT_tag TOM1_CH2_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH2_IRQ_MODE_tag TOM1_CH2_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved17[4];
  GTM_gtm_cls1_TOM1_CH2_CTRL_SR_tag TOM1_CH2_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved18[12];
  GTM_gtm_cls1_TOM1_CH3_CTRL_tag TOM1_CH3_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH3_SR0_tag TOM1_CH3_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH3_SR1_tag TOM1_CH3_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH3_CM0_tag TOM1_CH3_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH3_CM1_tag TOM1_CH3_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH3_CN0_tag TOM1_CH3_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH3_STAT_tag TOM1_CH3_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH3_IRQ_NOTIFY_tag TOM1_CH3_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH3_IRQ_EN_tag TOM1_CH3_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH3_IRQ_FORCINT_tag TOM1_CH3_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH3_IRQ_MODE_tag TOM1_CH3_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved19[4];
  GTM_gtm_cls1_TOM1_CH3_CTRL_SR_tag TOM1_CH3_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved20[12];
  GTM_gtm_cls1_TOM1_CH4_CTRL_tag TOM1_CH4_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH4_SR0_tag TOM1_CH4_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH4_SR1_tag TOM1_CH4_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH4_CM0_tag TOM1_CH4_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH4_CM1_tag TOM1_CH4_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH4_CN0_tag TOM1_CH4_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH4_STAT_tag TOM1_CH4_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH4_IRQ_NOTIFY_tag TOM1_CH4_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH4_IRQ_EN_tag TOM1_CH4_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH4_IRQ_FORCINT_tag TOM1_CH4_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH4_IRQ_MODE_tag TOM1_CH4_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved21[4];
  GTM_gtm_cls1_TOM1_CH4_CTRL_SR_tag TOM1_CH4_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved22[12];
  GTM_gtm_cls1_TOM1_CH5_CTRL_tag TOM1_CH5_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH5_SR0_tag TOM1_CH5_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH5_SR1_tag TOM1_CH5_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH5_CM0_tag TOM1_CH5_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH5_CM1_tag TOM1_CH5_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH5_CN0_tag TOM1_CH5_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH5_STAT_tag TOM1_CH5_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH5_IRQ_NOTIFY_tag TOM1_CH5_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH5_IRQ_EN_tag TOM1_CH5_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH5_IRQ_FORCINT_tag TOM1_CH5_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH5_IRQ_MODE_tag TOM1_CH5_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved23[4];
  GTM_gtm_cls1_TOM1_CH5_CTRL_SR_tag TOM1_CH5_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved24[12];
  GTM_gtm_cls1_TOM1_CH6_CTRL_tag TOM1_CH6_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH6_SR0_tag TOM1_CH6_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH6_SR1_tag TOM1_CH6_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH6_CM0_tag TOM1_CH6_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH6_CM1_tag TOM1_CH6_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH6_CN0_tag TOM1_CH6_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH6_STAT_tag TOM1_CH6_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH6_IRQ_NOTIFY_tag TOM1_CH6_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH6_IRQ_EN_tag TOM1_CH6_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH6_IRQ_FORCINT_tag TOM1_CH6_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH6_IRQ_MODE_tag TOM1_CH6_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved25[4];
  GTM_gtm_cls1_TOM1_CH6_CTRL_SR_tag TOM1_CH6_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved26[12];
  GTM_gtm_cls1_TOM1_CH7_CTRL_tag TOM1_CH7_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH7_SR0_tag TOM1_CH7_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH7_SR1_tag TOM1_CH7_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH7_CM0_tag TOM1_CH7_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH7_CM1_tag TOM1_CH7_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH7_CN0_tag TOM1_CH7_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH7_STAT_tag TOM1_CH7_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH7_IRQ_NOTIFY_tag TOM1_CH7_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH7_IRQ_EN_tag TOM1_CH7_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH7_IRQ_FORCINT_tag TOM1_CH7_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH7_IRQ_MODE_tag TOM1_CH7_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved27[4];
  GTM_gtm_cls1_TOM1_CH7_CTRL_SR_tag TOM1_CH7_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved28[12];
  GTM_gtm_cls1_TOM1_CH8_CTRL_tag TOM1_CH8_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH8_SR0_tag TOM1_CH8_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH8_SR1_tag TOM1_CH8_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH8_CM0_tag TOM1_CH8_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH8_CM1_tag TOM1_CH8_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH8_CN0_tag TOM1_CH8_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH8_STAT_tag TOM1_CH8_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH8_IRQ_NOTIFY_tag TOM1_CH8_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH8_IRQ_EN_tag TOM1_CH8_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH8_IRQ_FORCINT_tag TOM1_CH8_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH8_IRQ_MODE_tag TOM1_CH8_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved29[4];
  GTM_gtm_cls1_TOM1_CH8_CTRL_SR_tag TOM1_CH8_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved30[12];
  GTM_gtm_cls1_TOM1_CH9_CTRL_tag TOM1_CH9_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH9_SR0_tag TOM1_CH9_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH9_SR1_tag TOM1_CH9_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH9_CM0_tag TOM1_CH9_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH9_CM1_tag TOM1_CH9_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH9_CN0_tag TOM1_CH9_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH9_STAT_tag TOM1_CH9_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH9_IRQ_NOTIFY_tag TOM1_CH9_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH9_IRQ_EN_tag TOM1_CH9_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH9_IRQ_FORCINT_tag TOM1_CH9_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH9_IRQ_MODE_tag TOM1_CH9_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved31[4];
  GTM_gtm_cls1_TOM1_CH9_CTRL_SR_tag TOM1_CH9_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved32[12];
  GTM_gtm_cls1_TOM1_CH10_CTRL_tag TOM1_CH10_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH10_SR0_tag TOM1_CH10_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH10_SR1_tag TOM1_CH10_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH10_CM0_tag TOM1_CH10_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH10_CM1_tag TOM1_CH10_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH10_CN0_tag TOM1_CH10_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH10_STAT_tag TOM1_CH10_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH10_IRQ_NOTIFY_tag TOM1_CH10_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH10_IRQ_EN_tag TOM1_CH10_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH10_IRQ_FORCINT_tag TOM1_CH10_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH10_IRQ_MODE_tag TOM1_CH10_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved33[4];
  GTM_gtm_cls1_TOM1_CH10_CTRL_SR_tag TOM1_CH10_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved34[12];
  GTM_gtm_cls1_TOM1_CH11_CTRL_tag TOM1_CH11_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH11_SR0_tag TOM1_CH11_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH11_SR1_tag TOM1_CH11_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH11_CM0_tag TOM1_CH11_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH11_CM1_tag TOM1_CH11_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH11_CN0_tag TOM1_CH11_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH11_STAT_tag TOM1_CH11_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH11_IRQ_NOTIFY_tag TOM1_CH11_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH11_IRQ_EN_tag TOM1_CH11_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH11_IRQ_FORCINT_tag TOM1_CH11_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH11_IRQ_MODE_tag TOM1_CH11_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved35[4];
  GTM_gtm_cls1_TOM1_CH11_CTRL_SR_tag TOM1_CH11_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved36[12];
  GTM_gtm_cls1_TOM1_CH12_CTRL_tag TOM1_CH12_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH12_SR0_tag TOM1_CH12_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH12_SR1_tag TOM1_CH12_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH12_CM0_tag TOM1_CH12_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH12_CM1_tag TOM1_CH12_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH12_CN0_tag TOM1_CH12_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH12_STAT_tag TOM1_CH12_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH12_IRQ_NOTIFY_tag TOM1_CH12_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH12_IRQ_EN_tag TOM1_CH12_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH12_IRQ_FORCINT_tag TOM1_CH12_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH12_IRQ_MODE_tag TOM1_CH12_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved37[4];
  GTM_gtm_cls1_TOM1_CH12_CTRL_SR_tag TOM1_CH12_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved38[12];
  GTM_gtm_cls1_TOM1_CH13_CTRL_tag TOM1_CH13_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH13_SR0_tag TOM1_CH13_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH13_SR1_tag TOM1_CH13_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH13_CM0_tag TOM1_CH13_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH13_CM1_tag TOM1_CH13_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH13_CN0_tag TOM1_CH13_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH13_STAT_tag TOM1_CH13_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH13_IRQ_NOTIFY_tag TOM1_CH13_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH13_IRQ_EN_tag TOM1_CH13_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH13_IRQ_FORCINT_tag TOM1_CH13_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH13_IRQ_MODE_tag TOM1_CH13_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved39[4];
  GTM_gtm_cls1_TOM1_CH13_CTRL_SR_tag TOM1_CH13_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved40[12];
  GTM_gtm_cls1_TOM1_CH14_CTRL_tag TOM1_CH14_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH14_SR0_tag TOM1_CH14_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH14_SR1_tag TOM1_CH14_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH14_CM0_tag TOM1_CH14_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH14_CM1_tag TOM1_CH14_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH14_CN0_tag TOM1_CH14_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH14_STAT_tag TOM1_CH14_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH14_IRQ_NOTIFY_tag TOM1_CH14_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH14_IRQ_EN_tag TOM1_CH14_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH14_IRQ_FORCINT_tag TOM1_CH14_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH14_IRQ_MODE_tag TOM1_CH14_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved41[4];
  GTM_gtm_cls1_TOM1_CH14_CTRL_SR_tag TOM1_CH14_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved42[12];
  GTM_gtm_cls1_TOM1_CH15_CTRL_tag TOM1_CH15_CTRL; /* TOM[i] channel [x] control register */
  GTM_gtm_cls1_TOM1_CH15_SR0_tag TOM1_CH15_SR0; /* TOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_TOM1_CH15_SR1_tag TOM1_CH15_SR1; /* TOM[i] channel [x] CCU1 compare shadow register */
  GTM_gtm_cls1_TOM1_CH15_CM0_tag TOM1_CH15_CM0; /* TOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_TOM1_CH15_CM1_tag TOM1_CH15_CM1; /* TOM[i] channel [x] CCU1 compare register */
  GTM_gtm_cls1_TOM1_CH15_CN0_tag TOM1_CH15_CN0; /* TOM[i] channel [x] CCU0 counter */
  GTM_gtm_cls1_TOM1_CH15_STAT_tag TOM1_CH15_STAT; /* TOM[i] channel [x] status register */
  GTM_gtm_cls1_TOM1_CH15_IRQ_NOTIFY_tag TOM1_CH15_IRQ_NOTIFY; /* TOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_TOM1_CH15_IRQ_EN_tag TOM1_CH15_IRQ_EN; /* TOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_TOM1_CH15_IRQ_FORCINT_tag TOM1_CH15_IRQ_FORCINT; /* TOM[i] channel [x] force interrupt register */
  GTM_gtm_cls1_TOM1_CH15_IRQ_MODE_tag TOM1_CH15_IRQ_MODE; /* TOM[i] channel [x] interrupt mode register */
  uint8_t GTM_gtm_cls1_reserved43[4];
  GTM_gtm_cls1_TOM1_CH15_CTRL_SR_tag TOM1_CH15_CTRL_SR; /* TOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved44[60];
  GTM_gtm_cls1_TOM1_TGC0_GLB_CTRL_tag TOM1_TGC0_GLB_CTRL; /* TOM[i] TGC [g] global control register */
  GTM_gtm_cls1_TOM1_TGC0_ACT_TB_tag TOM1_TGC0_ACT_TB; /* TOM[i] TGC [g] action time base register */
  GTM_gtm_cls1_TOM1_TGC0_FUPD_CTRL_tag TOM1_TGC0_FUPD_CTRL; /* TOM[i] TGC [g] force update control register */
  GTM_gtm_cls1_TOM1_TGC0_INT_TRIG_tag TOM1_TGC0_INT_TRIG; /* TOM[i] TGC [g] internal trigger control register */
  uint8_t GTM_gtm_cls1_reserved45[48];
  GTM_gtm_cls1_TOM1_TGC0_ENDIS_CTRL_tag TOM1_TGC0_ENDIS_CTRL; /* TOM[i] TGC [g] enable/disable control register */
  GTM_gtm_cls1_TOM1_TGC0_ENDIS_STAT_tag TOM1_TGC0_ENDIS_STAT; /* TOM[i] TGC [g] enable/disable status register */
  GTM_gtm_cls1_TOM1_TGC0_OUTEN_CTRL_tag TOM1_TGC0_OUTEN_CTRL; /* TOM[i] TGC [g] output enable control register */
  GTM_gtm_cls1_TOM1_TGC0_OUTEN_STAT_tag TOM1_TGC0_OUTEN_STAT; /* TOM[i] TGC [g] output enable status register */
  uint8_t GTM_gtm_cls1_reserved46[48];
  GTM_gtm_cls1_TOM1_TGC1_GLB_CTRL_tag TOM1_TGC1_GLB_CTRL; /* TOM[i] TGC [g] global control register */
  GTM_gtm_cls1_TOM1_TGC1_ACT_TB_tag TOM1_TGC1_ACT_TB; /* TOM[i] TGC [g] action time base register */
  GTM_gtm_cls1_TOM1_TGC1_FUPD_CTRL_tag TOM1_TGC1_FUPD_CTRL; /* TOM[i] TGC [g] force update control register */
  GTM_gtm_cls1_TOM1_TGC1_INT_TRIG_tag TOM1_TGC1_INT_TRIG; /* TOM[i] TGC [g] internal trigger control register */
  uint8_t GTM_gtm_cls1_reserved47[48];
  GTM_gtm_cls1_TOM1_TGC1_ENDIS_CTRL_tag TOM1_TGC1_ENDIS_CTRL; /* TOM[i] TGC [g] enable/disable control register */
  GTM_gtm_cls1_TOM1_TGC1_ENDIS_STAT_tag TOM1_TGC1_ENDIS_STAT; /* TOM[i] TGC [g] enable/disable status register */
  GTM_gtm_cls1_TOM1_TGC1_OUTEN_CTRL_tag TOM1_TGC1_OUTEN_CTRL; /* TOM[i] TGC [g] output enable control register */
  GTM_gtm_cls1_TOM1_TGC1_OUTEN_STAT_tag TOM1_TGC1_OUTEN_STAT; /* TOM[i] TGC [g] output enable status register */
  uint8_t GTM_gtm_cls1_reserved48[768];
  GTM_gtm_cls1_ATOM1_CH0_RDADDR_tag ATOM1_CH0_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls1_ATOM1_CH0_CTRL_tag ATOM1_CH0_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls1_ATOM1_CH0_SR0_tag ATOM1_CH0_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH0_SR1_tag ATOM1_CH0_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH0_CM0_tag ATOM1_CH0_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH0_CM1_tag ATOM1_CH0_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH0_CN0_tag ATOM1_CH0_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls1_ATOM1_CH0_STAT_tag ATOM1_CH0_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls1_ATOM1_CH0_IRQ_NOTIFY_tag ATOM1_CH0_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_ATOM1_CH0_IRQ_EN_tag ATOM1_CH0_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_ATOM1_CH0_IRQ_FORCINT_tag ATOM1_CH0_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls1_ATOM1_CH0_IRQ_MODE_tag ATOM1_CH0_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_ATOM1_CH0_CTRL2_tag ATOM1_CH0_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls1_ATOM1_CH0_CTRL_SR_tag ATOM1_CH0_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved49[72];
  GTM_gtm_cls1_ATOM1_CH1_RDADDR_tag ATOM1_CH1_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls1_ATOM1_CH1_CTRL_tag ATOM1_CH1_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls1_ATOM1_CH1_SR0_tag ATOM1_CH1_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH1_SR1_tag ATOM1_CH1_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH1_CM0_tag ATOM1_CH1_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH1_CM1_tag ATOM1_CH1_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH1_CN0_tag ATOM1_CH1_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls1_ATOM1_CH1_STAT_tag ATOM1_CH1_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls1_ATOM1_CH1_IRQ_NOTIFY_tag ATOM1_CH1_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_ATOM1_CH1_IRQ_EN_tag ATOM1_CH1_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_ATOM1_CH1_IRQ_FORCINT_tag ATOM1_CH1_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls1_ATOM1_CH1_IRQ_MODE_tag ATOM1_CH1_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_ATOM1_CH1_CTRL2_tag ATOM1_CH1_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls1_ATOM1_CH1_CTRL_SR_tag ATOM1_CH1_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved50[72];
  GTM_gtm_cls1_ATOM1_CH2_RDADDR_tag ATOM1_CH2_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls1_ATOM1_CH2_CTRL_tag ATOM1_CH2_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls1_ATOM1_CH2_SR0_tag ATOM1_CH2_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH2_SR1_tag ATOM1_CH2_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH2_CM0_tag ATOM1_CH2_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH2_CM1_tag ATOM1_CH2_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH2_CN0_tag ATOM1_CH2_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls1_ATOM1_CH2_STAT_tag ATOM1_CH2_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls1_ATOM1_CH2_IRQ_NOTIFY_tag ATOM1_CH2_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_ATOM1_CH2_IRQ_EN_tag ATOM1_CH2_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_ATOM1_CH2_IRQ_FORCINT_tag ATOM1_CH2_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls1_ATOM1_CH2_IRQ_MODE_tag ATOM1_CH2_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_ATOM1_CH2_CTRL2_tag ATOM1_CH2_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls1_ATOM1_CH2_CTRL_SR_tag ATOM1_CH2_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved51[72];
  GTM_gtm_cls1_ATOM1_CH3_RDADDR_tag ATOM1_CH3_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls1_ATOM1_CH3_CTRL_tag ATOM1_CH3_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls1_ATOM1_CH3_SR0_tag ATOM1_CH3_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH3_SR1_tag ATOM1_CH3_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH3_CM0_tag ATOM1_CH3_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH3_CM1_tag ATOM1_CH3_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH3_CN0_tag ATOM1_CH3_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls1_ATOM1_CH3_STAT_tag ATOM1_CH3_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls1_ATOM1_CH3_IRQ_NOTIFY_tag ATOM1_CH3_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_ATOM1_CH3_IRQ_EN_tag ATOM1_CH3_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_ATOM1_CH3_IRQ_FORCINT_tag ATOM1_CH3_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls1_ATOM1_CH3_IRQ_MODE_tag ATOM1_CH3_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_ATOM1_CH3_CTRL2_tag ATOM1_CH3_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls1_ATOM1_CH3_CTRL_SR_tag ATOM1_CH3_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved52[72];
  GTM_gtm_cls1_ATOM1_CH4_RDADDR_tag ATOM1_CH4_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls1_ATOM1_CH4_CTRL_tag ATOM1_CH4_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls1_ATOM1_CH4_SR0_tag ATOM1_CH4_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH4_SR1_tag ATOM1_CH4_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH4_CM0_tag ATOM1_CH4_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH4_CM1_tag ATOM1_CH4_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH4_CN0_tag ATOM1_CH4_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls1_ATOM1_CH4_STAT_tag ATOM1_CH4_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls1_ATOM1_CH4_IRQ_NOTIFY_tag ATOM1_CH4_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_ATOM1_CH4_IRQ_EN_tag ATOM1_CH4_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_ATOM1_CH4_IRQ_FORCINT_tag ATOM1_CH4_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls1_ATOM1_CH4_IRQ_MODE_tag ATOM1_CH4_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_ATOM1_CH4_CTRL2_tag ATOM1_CH4_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls1_ATOM1_CH4_CTRL_SR_tag ATOM1_CH4_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved53[72];
  GTM_gtm_cls1_ATOM1_CH5_RDADDR_tag ATOM1_CH5_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls1_ATOM1_CH5_CTRL_tag ATOM1_CH5_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls1_ATOM1_CH5_SR0_tag ATOM1_CH5_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH5_SR1_tag ATOM1_CH5_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH5_CM0_tag ATOM1_CH5_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH5_CM1_tag ATOM1_CH5_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH5_CN0_tag ATOM1_CH5_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls1_ATOM1_CH5_STAT_tag ATOM1_CH5_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls1_ATOM1_CH5_IRQ_NOTIFY_tag ATOM1_CH5_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_ATOM1_CH5_IRQ_EN_tag ATOM1_CH5_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_ATOM1_CH5_IRQ_FORCINT_tag ATOM1_CH5_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls1_ATOM1_CH5_IRQ_MODE_tag ATOM1_CH5_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_ATOM1_CH5_CTRL2_tag ATOM1_CH5_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls1_ATOM1_CH5_CTRL_SR_tag ATOM1_CH5_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved54[72];
  GTM_gtm_cls1_ATOM1_CH6_RDADDR_tag ATOM1_CH6_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls1_ATOM1_CH6_CTRL_tag ATOM1_CH6_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls1_ATOM1_CH6_SR0_tag ATOM1_CH6_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH6_SR1_tag ATOM1_CH6_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH6_CM0_tag ATOM1_CH6_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH6_CM1_tag ATOM1_CH6_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH6_CN0_tag ATOM1_CH6_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls1_ATOM1_CH6_STAT_tag ATOM1_CH6_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls1_ATOM1_CH6_IRQ_NOTIFY_tag ATOM1_CH6_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_ATOM1_CH6_IRQ_EN_tag ATOM1_CH6_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_ATOM1_CH6_IRQ_FORCINT_tag ATOM1_CH6_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls1_ATOM1_CH6_IRQ_MODE_tag ATOM1_CH6_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_ATOM1_CH6_CTRL2_tag ATOM1_CH6_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls1_ATOM1_CH6_CTRL_SR_tag ATOM1_CH6_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved55[72];
  GTM_gtm_cls1_ATOM1_CH7_RDADDR_tag ATOM1_CH7_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls1_ATOM1_CH7_CTRL_tag ATOM1_CH7_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls1_ATOM1_CH7_SR0_tag ATOM1_CH7_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH7_SR1_tag ATOM1_CH7_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls1_ATOM1_CH7_CM0_tag ATOM1_CH7_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH7_CM1_tag ATOM1_CH7_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls1_ATOM1_CH7_CN0_tag ATOM1_CH7_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls1_ATOM1_CH7_STAT_tag ATOM1_CH7_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls1_ATOM1_CH7_IRQ_NOTIFY_tag ATOM1_CH7_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls1_ATOM1_CH7_IRQ_EN_tag ATOM1_CH7_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls1_ATOM1_CH7_IRQ_FORCINT_tag ATOM1_CH7_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls1_ATOM1_CH7_IRQ_MODE_tag ATOM1_CH7_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls1_ATOM1_CH7_CTRL2_tag ATOM1_CH7_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls1_ATOM1_CH7_CTRL_SR_tag ATOM1_CH7_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls1_reserved56[136];
  GTM_gtm_cls1_ATOM1_AGC_GLB_CTRL_tag ATOM1_AGC_GLB_CTRL; /* ATOM[i] AGC global control register */
  GTM_gtm_cls1_ATOM1_AGC_ENDIS_CTRL_tag ATOM1_AGC_ENDIS_CTRL; /* ATOM[i] AGC enable/disable control register */
  GTM_gtm_cls1_ATOM1_AGC_ENDIS_STAT_tag ATOM1_AGC_ENDIS_STAT; /* ATOM[i] AGC enable/disable status register */
  GTM_gtm_cls1_ATOM1_AGC_ACT_TB_tag ATOM1_AGC_ACT_TB; /* ATOM[i] AGC action time base register */
  GTM_gtm_cls1_ATOM1_AGC_OUTEN_CTRL_tag ATOM1_AGC_OUTEN_CTRL; /* ATOM[i] AGC output enable control register */
  GTM_gtm_cls1_ATOM1_AGC_OUTEN_STAT_tag ATOM1_AGC_OUTEN_STAT; /* ATOM[i] AGC output enable status register */
  GTM_gtm_cls1_ATOM1_AGC_FUPD_CTRL_tag ATOM1_AGC_FUPD_CTRL; /* ATOM[i] AGC force update control register */
  GTM_gtm_cls1_ATOM1_AGC_INT_TRIG_tag ATOM1_AGC_INT_TRIG; /* ATOM[i] AGC internal trigger control register */
  uint8_t GTM_gtm_cls1_reserved57[928];
  GTM_gtm_cls1_MCS1_CH0_R0_tag MCS1_CH0_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH0_R1_tag MCS1_CH0_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH0_R2_tag MCS1_CH0_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH0_R3_tag MCS1_CH0_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH0_R4_tag MCS1_CH0_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH0_R5_tag MCS1_CH0_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH0_R6_tag MCS1_CH0_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH0_R7_tag MCS1_CH0_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH0_CTRL_tag MCS1_CH0_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls1_MCS1_CH0_ACB_tag MCS1_CH0_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls1_reserved58[20];
  GTM_gtm_cls1_MCS1_CH0_MHB_tag MCS1_CH0_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls1_reserved59[160];
  GTM_gtm_cls1_MCS1_CH0_PC_tag MCS1_CH0_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls1_MCS1_CH0_IRQ_NOTIFY_tag MCS1_CH0_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls1_MCS1_CH0_IRQ_EN_tag MCS1_CH0_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls1_MCS1_CH0_IRQ_FORCINT_tag MCS1_CH0_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls1_MCS1_CH0_IRQ_MODE_tag MCS1_CH0_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls1_MCS1_CH0_EIRQ_EN_tag MCS1_CH0_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved60[8];
  GTM_gtm_cls1_MCS1_CH1_R0_tag MCS1_CH1_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH1_R1_tag MCS1_CH1_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH1_R2_tag MCS1_CH1_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH1_R3_tag MCS1_CH1_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH1_R4_tag MCS1_CH1_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH1_R5_tag MCS1_CH1_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH1_R6_tag MCS1_CH1_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH1_R7_tag MCS1_CH1_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH1_CTRL_tag MCS1_CH1_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls1_MCS1_CH1_ACB_tag MCS1_CH1_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls1_reserved61[20];
  GTM_gtm_cls1_MCS1_CH1_MHB_tag MCS1_CH1_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls1_reserved62[160];
  GTM_gtm_cls1_MCS1_CH1_PC_tag MCS1_CH1_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls1_MCS1_CH1_IRQ_NOTIFY_tag MCS1_CH1_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls1_MCS1_CH1_IRQ_EN_tag MCS1_CH1_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls1_MCS1_CH1_IRQ_FORCINT_tag MCS1_CH1_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls1_MCS1_CH1_IRQ_MODE_tag MCS1_CH1_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls1_MCS1_CH1_EIRQ_EN_tag MCS1_CH1_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved63[8];
  GTM_gtm_cls1_MCS1_CH2_R0_tag MCS1_CH2_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH2_R1_tag MCS1_CH2_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH2_R2_tag MCS1_CH2_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH2_R3_tag MCS1_CH2_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH2_R4_tag MCS1_CH2_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH2_R5_tag MCS1_CH2_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH2_R6_tag MCS1_CH2_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH2_R7_tag MCS1_CH2_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH2_CTRL_tag MCS1_CH2_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls1_MCS1_CH2_ACB_tag MCS1_CH2_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls1_reserved64[20];
  GTM_gtm_cls1_MCS1_CH2_MHB_tag MCS1_CH2_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls1_reserved65[160];
  GTM_gtm_cls1_MCS1_CH2_PC_tag MCS1_CH2_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls1_MCS1_CH2_IRQ_NOTIFY_tag MCS1_CH2_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls1_MCS1_CH2_IRQ_EN_tag MCS1_CH2_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls1_MCS1_CH2_IRQ_FORCINT_tag MCS1_CH2_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls1_MCS1_CH2_IRQ_MODE_tag MCS1_CH2_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls1_MCS1_CH2_EIRQ_EN_tag MCS1_CH2_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved66[8];
  GTM_gtm_cls1_MCS1_CH3_R0_tag MCS1_CH3_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH3_R1_tag MCS1_CH3_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH3_R2_tag MCS1_CH3_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH3_R3_tag MCS1_CH3_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH3_R4_tag MCS1_CH3_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH3_R5_tag MCS1_CH3_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH3_R6_tag MCS1_CH3_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH3_R7_tag MCS1_CH3_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH3_CTRL_tag MCS1_CH3_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls1_MCS1_CH3_ACB_tag MCS1_CH3_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls1_reserved67[20];
  GTM_gtm_cls1_MCS1_CH3_MHB_tag MCS1_CH3_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls1_reserved68[160];
  GTM_gtm_cls1_MCS1_CH3_PC_tag MCS1_CH3_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls1_MCS1_CH3_IRQ_NOTIFY_tag MCS1_CH3_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls1_MCS1_CH3_IRQ_EN_tag MCS1_CH3_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls1_MCS1_CH3_IRQ_FORCINT_tag MCS1_CH3_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls1_MCS1_CH3_IRQ_MODE_tag MCS1_CH3_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls1_MCS1_CH3_EIRQ_EN_tag MCS1_CH3_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved69[8];
  GTM_gtm_cls1_MCS1_CH4_R0_tag MCS1_CH4_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH4_R1_tag MCS1_CH4_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH4_R2_tag MCS1_CH4_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH4_R3_tag MCS1_CH4_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH4_R4_tag MCS1_CH4_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH4_R5_tag MCS1_CH4_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH4_R6_tag MCS1_CH4_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH4_R7_tag MCS1_CH4_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH4_CTRL_tag MCS1_CH4_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls1_MCS1_CH4_ACB_tag MCS1_CH4_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls1_reserved70[20];
  GTM_gtm_cls1_MCS1_CH4_MHB_tag MCS1_CH4_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls1_reserved71[160];
  GTM_gtm_cls1_MCS1_CH4_PC_tag MCS1_CH4_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls1_MCS1_CH4_IRQ_NOTIFY_tag MCS1_CH4_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls1_MCS1_CH4_IRQ_EN_tag MCS1_CH4_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls1_MCS1_CH4_IRQ_FORCINT_tag MCS1_CH4_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls1_MCS1_CH4_IRQ_MODE_tag MCS1_CH4_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls1_MCS1_CH4_EIRQ_EN_tag MCS1_CH4_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved72[8];
  GTM_gtm_cls1_MCS1_CH5_R0_tag MCS1_CH5_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH5_R1_tag MCS1_CH5_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH5_R2_tag MCS1_CH5_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH5_R3_tag MCS1_CH5_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH5_R4_tag MCS1_CH5_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH5_R5_tag MCS1_CH5_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH5_R6_tag MCS1_CH5_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH5_R7_tag MCS1_CH5_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH5_CTRL_tag MCS1_CH5_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls1_MCS1_CH5_ACB_tag MCS1_CH5_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls1_reserved73[20];
  GTM_gtm_cls1_MCS1_CH5_MHB_tag MCS1_CH5_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls1_reserved74[160];
  GTM_gtm_cls1_MCS1_CH5_PC_tag MCS1_CH5_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls1_MCS1_CH5_IRQ_NOTIFY_tag MCS1_CH5_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls1_MCS1_CH5_IRQ_EN_tag MCS1_CH5_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls1_MCS1_CH5_IRQ_FORCINT_tag MCS1_CH5_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls1_MCS1_CH5_IRQ_MODE_tag MCS1_CH5_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls1_MCS1_CH5_EIRQ_EN_tag MCS1_CH5_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved75[8];
  GTM_gtm_cls1_MCS1_CH6_R0_tag MCS1_CH6_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH6_R1_tag MCS1_CH6_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH6_R2_tag MCS1_CH6_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH6_R3_tag MCS1_CH6_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH6_R4_tag MCS1_CH6_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH6_R5_tag MCS1_CH6_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH6_R6_tag MCS1_CH6_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH6_R7_tag MCS1_CH6_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH6_CTRL_tag MCS1_CH6_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls1_MCS1_CH6_ACB_tag MCS1_CH6_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls1_reserved76[20];
  GTM_gtm_cls1_MCS1_CH6_MHB_tag MCS1_CH6_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls1_reserved77[160];
  GTM_gtm_cls1_MCS1_CH6_PC_tag MCS1_CH6_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls1_MCS1_CH6_IRQ_NOTIFY_tag MCS1_CH6_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls1_MCS1_CH6_IRQ_EN_tag MCS1_CH6_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls1_MCS1_CH6_IRQ_FORCINT_tag MCS1_CH6_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls1_MCS1_CH6_IRQ_MODE_tag MCS1_CH6_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls1_MCS1_CH6_EIRQ_EN_tag MCS1_CH6_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved78[8];
  GTM_gtm_cls1_MCS1_CH7_R0_tag MCS1_CH7_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH7_R1_tag MCS1_CH7_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH7_R2_tag MCS1_CH7_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH7_R3_tag MCS1_CH7_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH7_R4_tag MCS1_CH7_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH7_R5_tag MCS1_CH7_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH7_R6_tag MCS1_CH7_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH7_R7_tag MCS1_CH7_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls1_MCS1_CH7_CTRL_tag MCS1_CH7_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls1_MCS1_CH7_ACB_tag MCS1_CH7_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls1_reserved79[20];
  GTM_gtm_cls1_MCS1_CH7_MHB_tag MCS1_CH7_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls1_reserved80[160];
  GTM_gtm_cls1_MCS1_CH7_PC_tag MCS1_CH7_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls1_MCS1_CH7_IRQ_NOTIFY_tag MCS1_CH7_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls1_MCS1_CH7_IRQ_EN_tag MCS1_CH7_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls1_MCS1_CH7_IRQ_FORCINT_tag MCS1_CH7_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls1_MCS1_CH7_IRQ_MODE_tag MCS1_CH7_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls1_MCS1_CH7_EIRQ_EN_tag MCS1_CH7_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls1_reserved81[1584];
  GTM_gtm_cls1_MCS1_CTRG_tag MCS1_CTRG; /* MCS[i] clear trigger control register */
  GTM_gtm_cls1_MCS1_STRG_tag MCS1_STRG; /* MCS[i] set trigger control register */
  uint8_t GTM_gtm_cls1_reserved82[208];
  GTM_gtm_cls1_MCS1_CTRL_STAT_tag MCS1_CTRL_STAT; /* MCS[i] control and status register */
  GTM_gtm_cls1_MCS1_RESET_tag MCS1_RESET; /* MCS[i] reset register */
  GTM_gtm_cls1_MCS1_CAT_tag MCS1_CAT;  /* MCS[i] cancel ARU transfer instruction */
  GTM_gtm_cls1_MCS1_CWT_tag MCS1_CWT;  /* MCS[i] cancel waiting instruction */
  GTM_gtm_cls1_MCS1_ERR_tag MCS1_ERR;  /* MCS[i] error register */
  uint8_t GTM_gtm_cls1_reserved83[8];
  GTM_gtm_cls1_MCS1_REG_PROT_tag MCS1_REG_PROT; /* MCS[i] write protection register */
  GTM_gtm_cls1_MCS1_SINT_IRQ_NOTIFY_tag MCS1_SINT_IRQ_NOTIFY; /* MCS[i] shared interrupt notification register */
  GTM_gtm_cls1_MCS1_SINT_IRQ_EN_tag MCS1_SINT_IRQ_EN; /* MCS[i] shared interrupt enable register */
  GTM_gtm_cls1_MCS1_SINT_IRQ_FORCINT_tag MCS1_SINT_IRQ_FORCINT; /* MCS[i] force shared interrupt register */
  GTM_gtm_cls1_MCS1_SINT_IRQ_MODE_tag MCS1_SINT_IRQ_MODE; /* MCS[i] shared interrupt mode configuration register */
  uint8_t GTM_gtm_cls1_reserved84[16];
  GTM_gtm_cls1_MCS1_HBP0_CTRL_tag MCS1_HBP0_CTRL; /* MCS[i] hardware break point h control register */
  GTM_gtm_cls1_MCS1_HBP0_PATTERN_tag MCS1_HBP0_PATTERN; /* MCS[i] hardware break point pattern register */
  GTM_gtm_cls1_MCS1_HBP0_STATUS_tag MCS1_HBP0_STATUS; /* MCS[i] hardware break point status register */
  GTM_gtm_cls1_MCS1_HBP0_IRQ_NOTIFY_tag MCS1_HBP0_IRQ_NOTIFY; /* MCS[i] hardware break point interrupt notification register */
  GTM_gtm_cls1_MCS1_HBP0_IRQ_EN_tag MCS1_HBP0_IRQ_EN; /* MCS[i] hardware break point interrupt enable register */
  GTM_gtm_cls1_MCS1_HBP0_IRQ_FORCINT_tag MCS1_HBP0_IRQ_FORCINT; /* MCS[i] force hardware break point interrupt register */
  GTM_gtm_cls1_MCS1_HBP0_IRQ_MODE_tag MCS1_HBP0_IRQ_MODE; /* MCS[i] break point h interrupt mode configuration register */
  uint8_t GTM_gtm_cls1_reserved85[4];
  GTM_gtm_cls1_MCS1_HBP1_CTRL_tag MCS1_HBP1_CTRL; /* MCS[i] hardware break point h control register */
  GTM_gtm_cls1_MCS1_HBP1_PATTERN_tag MCS1_HBP1_PATTERN; /* MCS[i] hardware break point pattern register */
  GTM_gtm_cls1_MCS1_HBP1_STATUS_tag MCS1_HBP1_STATUS; /* MCS[i] hardware break point status register */
  GTM_gtm_cls1_MCS1_HBP1_IRQ_NOTIFY_tag MCS1_HBP1_IRQ_NOTIFY; /* MCS[i] hardware break point interrupt notification register */
  GTM_gtm_cls1_MCS1_HBP1_IRQ_EN_tag MCS1_HBP1_IRQ_EN; /* MCS[i] hardware break point interrupt enable register */
  GTM_gtm_cls1_MCS1_HBP1_IRQ_FORCINT_tag MCS1_HBP1_IRQ_FORCINT; /* MCS[i] force hardware break point interrupt register */
  GTM_gtm_cls1_MCS1_HBP1_IRQ_MODE_tag MCS1_HBP1_IRQ_MODE; /* MCS[i] break point h interrupt mode configuration register */
  uint8_t GTM_gtm_cls1_reserved86[132];
  GTM_gtm_cls1_TIO1_G0_CH0_CTRL_tag TIO1_G0_CH0_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls1_TIO1_G0_CH0_IRQ_NOTIFY_tag TIO1_G0_CH0_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls1_TIO1_G0_CH0_IRQ_EN_tag TIO1_G0_CH0_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls1_TIO1_G0_CH0_IRQ_FORCINT_tag TIO1_G0_CH0_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls1_TIO1_G0_CH0_IRQ_MODE_tag TIO1_G0_CH0_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls1_TIO1_G0_CH0_CTRL2_tag TIO1_G0_CH0_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls1_reserved87[8];
  GTM_gtm_cls1_TIO1_G0_CH0_SINST_tag TIO1_G0_CH0_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH0_SCMD_tag TIO1_G0_CH0_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH0_SOP_tag TIO1_G0_CH0_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls1_reserved88[4];
  GTM_gtm_cls1_TIO1_G0_CH0_OINST_tag TIO1_G0_CH0_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls1_TIO1_G0_CH0_OCMD_tag TIO1_G0_CH0_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls1_TIO1_G0_CH0_OOP_tag TIO1_G0_CH0_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls1_TIO1_G0_CH0_SHIFTCNT_tag TIO1_G0_CH0_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls1_TIO1_G0_CH1_CTRL_tag TIO1_G0_CH1_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls1_TIO1_G0_CH1_IRQ_NOTIFY_tag TIO1_G0_CH1_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls1_TIO1_G0_CH1_IRQ_EN_tag TIO1_G0_CH1_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls1_TIO1_G0_CH1_IRQ_FORCINT_tag TIO1_G0_CH1_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls1_TIO1_G0_CH1_IRQ_MODE_tag TIO1_G0_CH1_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls1_TIO1_G0_CH1_CTRL2_tag TIO1_G0_CH1_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls1_reserved89[8];
  GTM_gtm_cls1_TIO1_G0_CH1_SINST_tag TIO1_G0_CH1_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH1_SCMD_tag TIO1_G0_CH1_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH1_SOP_tag TIO1_G0_CH1_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls1_reserved90[4];
  GTM_gtm_cls1_TIO1_G0_CH1_OINST_tag TIO1_G0_CH1_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls1_TIO1_G0_CH1_OCMD_tag TIO1_G0_CH1_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls1_TIO1_G0_CH1_OOP_tag TIO1_G0_CH1_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls1_TIO1_G0_CH1_SHIFTCNT_tag TIO1_G0_CH1_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls1_TIO1_G0_CH2_CTRL_tag TIO1_G0_CH2_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls1_TIO1_G0_CH2_IRQ_NOTIFY_tag TIO1_G0_CH2_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls1_TIO1_G0_CH2_IRQ_EN_tag TIO1_G0_CH2_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls1_TIO1_G0_CH2_IRQ_FORCINT_tag TIO1_G0_CH2_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls1_TIO1_G0_CH2_IRQ_MODE_tag TIO1_G0_CH2_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls1_TIO1_G0_CH2_CTRL2_tag TIO1_G0_CH2_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls1_reserved91[8];
  GTM_gtm_cls1_TIO1_G0_CH2_SINST_tag TIO1_G0_CH2_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH2_SCMD_tag TIO1_G0_CH2_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH2_SOP_tag TIO1_G0_CH2_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls1_reserved92[4];
  GTM_gtm_cls1_TIO1_G0_CH2_OINST_tag TIO1_G0_CH2_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls1_TIO1_G0_CH2_OCMD_tag TIO1_G0_CH2_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls1_TIO1_G0_CH2_OOP_tag TIO1_G0_CH2_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls1_TIO1_G0_CH2_SHIFTCNT_tag TIO1_G0_CH2_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls1_TIO1_G0_CH3_CTRL_tag TIO1_G0_CH3_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls1_TIO1_G0_CH3_IRQ_NOTIFY_tag TIO1_G0_CH3_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls1_TIO1_G0_CH3_IRQ_EN_tag TIO1_G0_CH3_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls1_TIO1_G0_CH3_IRQ_FORCINT_tag TIO1_G0_CH3_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls1_TIO1_G0_CH3_IRQ_MODE_tag TIO1_G0_CH3_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls1_TIO1_G0_CH3_CTRL2_tag TIO1_G0_CH3_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls1_reserved93[8];
  GTM_gtm_cls1_TIO1_G0_CH3_SINST_tag TIO1_G0_CH3_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH3_SCMD_tag TIO1_G0_CH3_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH3_SOP_tag TIO1_G0_CH3_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls1_reserved94[4];
  GTM_gtm_cls1_TIO1_G0_CH3_OINST_tag TIO1_G0_CH3_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls1_TIO1_G0_CH3_OCMD_tag TIO1_G0_CH3_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls1_TIO1_G0_CH3_OOP_tag TIO1_G0_CH3_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls1_TIO1_G0_CH3_SHIFTCNT_tag TIO1_G0_CH3_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls1_TIO1_G0_CH4_CTRL_tag TIO1_G0_CH4_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls1_TIO1_G0_CH4_IRQ_NOTIFY_tag TIO1_G0_CH4_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls1_TIO1_G0_CH4_IRQ_EN_tag TIO1_G0_CH4_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls1_TIO1_G0_CH4_IRQ_FORCINT_tag TIO1_G0_CH4_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls1_TIO1_G0_CH4_IRQ_MODE_tag TIO1_G0_CH4_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls1_TIO1_G0_CH4_CTRL2_tag TIO1_G0_CH4_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls1_reserved95[8];
  GTM_gtm_cls1_TIO1_G0_CH4_SINST_tag TIO1_G0_CH4_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH4_SCMD_tag TIO1_G0_CH4_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH4_SOP_tag TIO1_G0_CH4_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls1_reserved96[4];
  GTM_gtm_cls1_TIO1_G0_CH4_OINST_tag TIO1_G0_CH4_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls1_TIO1_G0_CH4_OCMD_tag TIO1_G0_CH4_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls1_TIO1_G0_CH4_OOP_tag TIO1_G0_CH4_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls1_TIO1_G0_CH4_SHIFTCNT_tag TIO1_G0_CH4_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls1_TIO1_G0_CH5_CTRL_tag TIO1_G0_CH5_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls1_TIO1_G0_CH5_IRQ_NOTIFY_tag TIO1_G0_CH5_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls1_TIO1_G0_CH5_IRQ_EN_tag TIO1_G0_CH5_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls1_TIO1_G0_CH5_IRQ_FORCINT_tag TIO1_G0_CH5_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls1_TIO1_G0_CH5_IRQ_MODE_tag TIO1_G0_CH5_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls1_TIO1_G0_CH5_CTRL2_tag TIO1_G0_CH5_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls1_reserved97[8];
  GTM_gtm_cls1_TIO1_G0_CH5_SINST_tag TIO1_G0_CH5_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH5_SCMD_tag TIO1_G0_CH5_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH5_SOP_tag TIO1_G0_CH5_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls1_reserved98[4];
  GTM_gtm_cls1_TIO1_G0_CH5_OINST_tag TIO1_G0_CH5_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls1_TIO1_G0_CH5_OCMD_tag TIO1_G0_CH5_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls1_TIO1_G0_CH5_OOP_tag TIO1_G0_CH5_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls1_TIO1_G0_CH5_SHIFTCNT_tag TIO1_G0_CH5_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls1_TIO1_G0_CH6_CTRL_tag TIO1_G0_CH6_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls1_TIO1_G0_CH6_IRQ_NOTIFY_tag TIO1_G0_CH6_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls1_TIO1_G0_CH6_IRQ_EN_tag TIO1_G0_CH6_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls1_TIO1_G0_CH6_IRQ_FORCINT_tag TIO1_G0_CH6_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls1_TIO1_G0_CH6_IRQ_MODE_tag TIO1_G0_CH6_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls1_TIO1_G0_CH6_CTRL2_tag TIO1_G0_CH6_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls1_reserved99[8];
  GTM_gtm_cls1_TIO1_G0_CH6_SINST_tag TIO1_G0_CH6_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH6_SCMD_tag TIO1_G0_CH6_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH6_SOP_tag TIO1_G0_CH6_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls1_reserved100[4];
  GTM_gtm_cls1_TIO1_G0_CH6_OINST_tag TIO1_G0_CH6_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls1_TIO1_G0_CH6_OCMD_tag TIO1_G0_CH6_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls1_TIO1_G0_CH6_OOP_tag TIO1_G0_CH6_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls1_TIO1_G0_CH6_SHIFTCNT_tag TIO1_G0_CH6_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls1_TIO1_G0_CH7_CTRL_tag TIO1_G0_CH7_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls1_TIO1_G0_CH7_IRQ_NOTIFY_tag TIO1_G0_CH7_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls1_TIO1_G0_CH7_IRQ_EN_tag TIO1_G0_CH7_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls1_TIO1_G0_CH7_IRQ_FORCINT_tag TIO1_G0_CH7_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls1_TIO1_G0_CH7_IRQ_MODE_tag TIO1_G0_CH7_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls1_TIO1_G0_CH7_CTRL2_tag TIO1_G0_CH7_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls1_reserved101[8];
  GTM_gtm_cls1_TIO1_G0_CH7_SINST_tag TIO1_G0_CH7_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH7_SCMD_tag TIO1_G0_CH7_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls1_TIO1_G0_CH7_SOP_tag TIO1_G0_CH7_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls1_reserved102[4];
  GTM_gtm_cls1_TIO1_G0_CH7_OINST_tag TIO1_G0_CH7_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls1_TIO1_G0_CH7_OCMD_tag TIO1_G0_CH7_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls1_TIO1_G0_CH7_OOP_tag TIO1_G0_CH7_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls1_TIO1_G0_CH7_SHIFTCNT_tag TIO1_G0_CH7_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls1_TIO1_G0_ISEL0_CTRL1_tag TIO1_G0_ISEL0_CTRL1; /* TIO[i] input selection register 1 */
  GTM_gtm_cls1_TIO1_G0_ISEL0_CTRL2_tag TIO1_G0_ISEL0_CTRL2; /* TIO[i] input selection register 2 */
  uint8_t GTM_gtm_cls1_reserved103[24];
  GTM_gtm_cls1_TIO1_G0_ISEL1_CTRL1_tag TIO1_G0_ISEL1_CTRL1; /* TIO[i] input selection register 1 */
  GTM_gtm_cls1_TIO1_G0_ISEL1_CTRL2_tag TIO1_G0_ISEL1_CTRL2; /* TIO[i] input selection register 2 */
  uint8_t GTM_gtm_cls1_reserved104[24];
  GTM_gtm_cls1_TIO1_G0_OP_USAGE_tag TIO1_G0_OP_USAGE; /* TIO[i] operand usage selection register */
  uint8_t GTM_gtm_cls1_reserved105[2492];
  GTM_gtm_cls1_TIO1_S_tag TIO1_S;      /* TIO[i] signal sampling register */
  GTM_gtm_cls1_TIO1_O_tag TIO1_O;      /* TIO[i] output register */
  GTM_gtm_cls1_TIO1_ENDIS_tag TIO1_ENDIS; /* TIO[i] enable/disable register */
  GTM_gtm_cls1_TIO1_INVERT_tag TIO1_INVERT; /* TIO[i] signal invert register */
  GTM_gtm_cls1_TIO1_INPUT_MODE_tag TIO1_INPUT_MODE; /* TIO[i] input mode register */
  GTM_gtm_cls1_TIO1_CYCLIC_MODE_tag TIO1_CYCLIC_MODE; /* TIO[i] cyclic mode register */
  GTM_gtm_cls1_TIO1_TRIG_OUT_GATE_EN_tag TIO1_TRIG_OUT_GATE_EN; /* TIO[i] enable Trigger Output, output gating register */
  GTM_gtm_cls1_TIO1_PLTRIG_OUT_GATE_EN_tag TIO1_PLTRIG_OUT_GATE_EN; /* TIO[i] enable PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls1_reserved106[32];
  GTM_gtm_cls1_TIO1_CS_tag TIO1_CS;    /* TIO[i] clear signal sampling register */
  GTM_gtm_cls1_TIO1_CO_tag TIO1_CO;    /* TIO[i] clear output register */
  GTM_gtm_cls1_TIO1_CENDIS_tag TIO1_CENDIS; /* TIO[i] disable register */
  GTM_gtm_cls1_TIO1_CINVERT_tag TIO1_CINVERT; /* TIO[i] clear signal invert register */
  GTM_gtm_cls1_TIO1_CINPUT_MODE_tag TIO1_CINPUT_MODE; /* TIO[i] disable input mode register */
  GTM_gtm_cls1_TIO1_CCYCLIC_MODE_tag TIO1_CCYCLIC_MODE; /* TIO[i] disable cyclic mode register */
  GTM_gtm_cls1_TIO1_CTRIG_OUT_GATE_EN_tag TIO1_CTRIG_OUT_GATE_EN; /* TIO[i] clear Trigger Output, output gating register */
  GTM_gtm_cls1_TIO1_CPLTRIG_OUT_GATE_EN_tag TIO1_CPLTRIG_OUT_GATE_EN; /* TIO[i] clear PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls1_reserved107[32];
  GTM_gtm_cls1_TIO1_SS_tag TIO1_SS;    /* TIO[i] set signal sampling register */
  GTM_gtm_cls1_TIO1_SO_tag TIO1_SO;    /* TIO[i] set output register */
  GTM_gtm_cls1_TIO1_SENDIS_tag TIO1_SENDIS; /* TIO[i] enable register */
  GTM_gtm_cls1_TIO1_SINVERT_tag TIO1_SINVERT; /* TIO[i] set signal invert register */
  GTM_gtm_cls1_TIO1_SINPUT_MODE_tag TIO1_SINPUT_MODE; /* TIO[i] enable input mode register */
  GTM_gtm_cls1_TIO1_SCYCLIC_MODE_tag TIO1_SCYCLIC_MODE; /* TIO[i] enable cyclic mode register */
  GTM_gtm_cls1_TIO1_STRIG_OUT_GATE_EN_tag TIO1_STRIG_OUT_GATE_EN; /* TIO[i] set Trigger Output, output gating register */
  GTM_gtm_cls1_TIO1_SPLTRIG_OUT_GATE_EN_tag TIO1_SPLTRIG_OUT_GATE_EN; /* TIO[i] set PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls1_reserved108[32];
  GTM_gtm_cls1_TIO1_IS_tag TIO1_IS;    /* TIO[i] invert signal sampling register */
  GTM_gtm_cls1_TIO1_IO_tag TIO1_IO;    /* TIO[i] invert output register */
  GTM_gtm_cls1_TIO1_IENDIS_tag TIO1_IENDIS; /* TIO[i] toggle enable/disable register */
  GTM_gtm_cls1_TIO1_IINVERT_tag TIO1_IINVERT; /* TIO[i] toggle signal invert register */
  GTM_gtm_cls1_TIO1_IINPUT_MODE_tag TIO1_IINPUT_MODE; /* TIO[i] enable input mode register */
  GTM_gtm_cls1_TIO1_ICYCLIC_MODE_tag TIO1_ICYCLIC_MODE; /* TIO[i] enable cyclic mode register */
  uint8_t GTM_gtm_cls1_reserved109[40];
  GTM_gtm_cls1_TIO1_FUPD_tag TIO1_FUPD; /* TIO[i] force update register */
  GTM_gtm_cls1_TIO1_HW_CONF_tag TIO1_HW_CONF; /* TIO[i] configuration register */
  GTM_gtm_cls1_TIO1_RSEL_CTRL1_tag TIO1_RSEL_CTRL1; /* TIO[i] resource selection control register 1 */
  GTM_gtm_cls1_TIO1_RSEL_CTRL2_tag TIO1_RSEL_CTRL2; /* TIO[i] resource selection control register 2 */
  GTM_gtm_cls1_TIO1_PL_SWRST_tag TIO1_PL_SWRST; /* TIO[i] software reset for TIO Plus functionality */
  uint8_t GTM_gtm_cls1_reserved110[748];
  GTM_gtm_cls1_CCM1_ARP0_CTRL_tag CCM1_ARP0_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP0_PROT_tag CCM1_ARP0_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP1_CTRL_tag CCM1_ARP1_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP1_PROT_tag CCM1_ARP1_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP2_CTRL_tag CCM1_ARP2_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP2_PROT_tag CCM1_ARP2_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP3_CTRL_tag CCM1_ARP3_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP3_PROT_tag CCM1_ARP3_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP4_CTRL_tag CCM1_ARP4_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP4_PROT_tag CCM1_ARP4_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP5_CTRL_tag CCM1_ARP5_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP5_PROT_tag CCM1_ARP5_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP6_CTRL_tag CCM1_ARP6_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP6_PROT_tag CCM1_ARP6_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP7_CTRL_tag CCM1_ARP7_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP7_PROT_tag CCM1_ARP7_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP8_CTRL_tag CCM1_ARP8_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP8_PROT_tag CCM1_ARP8_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls1_CCM1_ARP9_CTRL_tag CCM1_ARP9_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls1_CCM1_ARP9_PROT_tag CCM1_ARP9_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  uint8_t GTM_gtm_cls1_reserved111[380];
  GTM_gtm_cls1_CCM1_TIO_G0_OUT_tag CCM1_TIO_G0_OUT; /* CCM[i] TIO Group 0,1 Output Register */
  uint8_t GTM_gtm_cls1_reserved112[4];
  GTM_gtm_cls1_CCM1_HW_CONF2_tag CCM1_HW_CONF2; /* CCM[i] 2. Hardware Configuration Register */
  GTM_gtm_cls1_CCM1_AEIM_STA_tag CCM1_AEIM_STA; /* CCM[i] MCS Bus Master Status Register */
  GTM_gtm_cls1_CCM1_HW_CONF_tag CCM1_HW_CONF; /* CCM[i] Hardware Configuration Register */
  GTM_gtm_cls1_CCM1_TIM_AUX_IN_SRC_tag CCM1_TIM_AUX_IN_SRC; /* CCM[i] TIM AUX Input Source Register */
  GTM_gtm_cls1_CCM1_EXT_CAP_EN_tag CCM1_EXT_CAP_EN; /* CCM[i] External Capture Enable Register */
  GTM_gtm_cls1_CCM1_TOM_OUT_tag CCM1_TOM_OUT; /* CCM[i] TOM Output Register */
  GTM_gtm_cls1_CCM1_ATOM_OUT_tag CCM1_ATOM_OUT; /* CCM[i] ATOM Output Register */
  GTM_gtm_cls1_CCM1_CMU_CLK_CFG_tag CCM1_CMU_CLK_CFG; /* CCM[i] CMU Clock Configuration Register */
  GTM_gtm_cls1_CCM1_CMU_FXCLK_CFG_tag CCM1_CMU_FXCLK_CFG; /* CCM[i] CMU Fixed Clock Configuration Register */
  GTM_gtm_cls1_CCM1_CFG_tag CCM1_CFG;  /* CCM[i] Configuration Register */
  GTM_gtm_cls1_CCM1_PROT_tag CCM1_PROT; /* CCM[i] Protection Register */
  uint8_t GTM_gtm_cls1_reserved113[768];
  GTM_gtm_cls1_CDTM1_DTM4_CTRL_tag CDTM1_DTM4_CTRL; /* CDTM[i]_DTM[d] global configuration and control register */
  GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL1_tag CDTM1_DTM4_CH_CTRL1; /* CDTM[i]_DTM[d] channel control register 1 */
  GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL2_tag CDTM1_DTM4_CH_CTRL2; /* CDTM[i]_DTM[d] channel control register 2 */
  GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL2_SR_tag CDTM1_DTM4_CH_CTRL2_SR; /* CDTM[i] DTM[j] channel control register 2 shadow */
  GTM_gtm_cls1_CDTM1_DTM4_PS_CTRL_tag CDTM1_DTM4_PS_CTRL; /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  GTM_gtm_cls1_CDTM1_DTM4_CH_DTV_tag CDTM1_DTM4_CH_DTV[4]; /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  GTM_gtm_cls1_CDTM1_DTM4_CH_SR_tag CDTM1_DTM4_CH_SR; /* CDTM[i]_DTM[d] channel shadow register */
  GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL3_tag CDTM1_DTM4_CH_CTRL3; /* CDTM[i]_DTM[d] channel control register 3 */
  GTM_gtm_cls1_CDTM1_DTM4_CTRL2_tag CDTM1_DTM4_CTRL2; /* CDTM[i]_DTM[d] global configuration and control register 2 */
  GTM_gtm_cls1_CDTM1_DTM4_CH0_DTV_SR_tag CDTM1_DTM4_CH0_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls1_CDTM1_DTM4_CH1_DTV_SR_tag CDTM1_DTM4_CH1_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls1_CDTM1_DTM4_CH2_DTV_SR_tag CDTM1_DTM4_CH2_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls1_CDTM1_DTM4_CH3_DTV_SR_tag CDTM1_DTM4_CH3_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls1_CDTM1_DTM5_CTRL_tag CDTM1_DTM5_CTRL; /* CDTM[i]_DTM[d] global configuration and control register */
  GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL1_tag CDTM1_DTM5_CH_CTRL1; /* CDTM[i]_DTM[d] channel control register 1 */
  GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL2_tag CDTM1_DTM5_CH_CTRL2; /* CDTM[i]_DTM[d] channel control register 2 */
  GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL2_SR_tag CDTM1_DTM5_CH_CTRL2_SR; /* CDTM[i] DTM[j] channel control register 2 shadow */
  GTM_gtm_cls1_CDTM1_DTM5_PS_CTRL_tag CDTM1_DTM5_PS_CTRL; /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  GTM_gtm_cls1_CDTM1_DTM5_CH_DTV_tag CDTM1_DTM5_CH_DTV[4]; /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  GTM_gtm_cls1_CDTM1_DTM5_CH_SR_tag CDTM1_DTM5_CH_SR; /* CDTM[i]_DTM[d] channel shadow register */
  GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL3_tag CDTM1_DTM5_CH_CTRL3; /* CDTM[i]_DTM[d] channel control register 3 */
  GTM_gtm_cls1_CDTM1_DTM5_CTRL2_tag CDTM1_DTM5_CTRL2; /* CDTM[i]_DTM[d] global configuration and control register 2 */
  GTM_gtm_cls1_CDTM1_DTM5_CH0_DTV_SR_tag CDTM1_DTM5_CH0_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls1_CDTM1_DTM5_CH1_DTV_SR_tag CDTM1_DTM5_CH1_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls1_CDTM1_DTM5_CH2_DTV_SR_tag CDTM1_DTM5_CH2_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls1_CDTM1_DTM5_CH3_DTV_SR_tag CDTM1_DTM5_CH3_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  uint8_t GTM_gtm_cls1_reserved114[1664];
  GTM_gtm_cls1_SPE1_CTRL_STAT_tag SPE1_CTRL_STAT; /* SPE[i] Control Status Register */
  GTM_gtm_cls1_SPE1_PAT_tag SPE1_PAT;  /* SPE[i] Input Pattern Definition Register */
  GTM_gtm_cls1_SPE1_OUT_PAT0_tag SPE1_OUT_PAT0; /* SPE[i] Output Definition Register */
  GTM_gtm_cls1_SPE1_OUT_PAT1_tag SPE1_OUT_PAT1; /* SPE[i] Output Definition Register */
  GTM_gtm_cls1_SPE1_OUT_PAT2_tag SPE1_OUT_PAT2; /* SPE[i] Output Definition Register */
  GTM_gtm_cls1_SPE1_OUT_PAT3_tag SPE1_OUT_PAT3; /* SPE[i] Output Definition Register */
  GTM_gtm_cls1_SPE1_OUT_PAT4_tag SPE1_OUT_PAT4; /* SPE[i] Output Definition Register */
  GTM_gtm_cls1_SPE1_OUT_PAT5_tag SPE1_OUT_PAT5; /* SPE[i] Output Definition Register */
  GTM_gtm_cls1_SPE1_OUT_PAT6_tag SPE1_OUT_PAT6; /* SPE[i] Output Definition Register */
  GTM_gtm_cls1_SPE1_OUT_PAT7_tag SPE1_OUT_PAT7; /* SPE[i] Output Definition Register */
  GTM_gtm_cls1_SPE1_OUT_CTRL_tag SPE1_OUT_CTRL; /* SPE[i] Output Control Register */
  GTM_gtm_cls1_SPE1_IRQ_NOTIFY_tag SPE1_IRQ_NOTIFY; /* SPE[i] Interrupt Notification Register */
  GTM_gtm_cls1_SPE1_IRQ_EN_tag SPE1_IRQ_EN; /* SPE[i] Interrupt Enable Register */
  GTM_gtm_cls1_SPE1_IRQ_FORCINT_tag SPE1_IRQ_FORCINT; /* SPE[i] Interrupt Generation By Software */
  GTM_gtm_cls1_SPE1_IRQ_MODE_tag SPE1_IRQ_MODE; /* SPE[i] Interrupt Mode Configuration Register */
  GTM_gtm_cls1_SPE1_EIRQ_EN_tag SPE1_EIRQ_EN; /* SPE[i] Error Interrupt Enable Register */
  GTM_gtm_cls1_SPE1_REV_CNT_tag SPE1_REV_CNT; /* SPE[i] Input Revolution Counter */
  GTM_gtm_cls1_SPE1_REV_CMP_tag SPE1_REV_CMP; /* SPE[i] Revolution Counter Compare Value */
  GTM_gtm_cls1_SPE1_CTRL_STAT2_tag SPE1_CTRL_STAT2; /* SPE[i] Control Status Register 2 */
  GTM_gtm_cls1_SPE1_CMD_tag SPE1_CMD;  /* SPE[i] Command Register */
  uint8_t GTM_gtm_cls1_reserved115[944];
  GTM_gtm_cls1_AXIM1_FREE_tag AXIM1_FREE; /* AXIM[i] slot allocation status. */
  GTM_gtm_cls1_AXIM1_REQUEST_tag AXIM1_REQUEST; /* AXIM[i] slot request (allocation). */
  GTM_gtm_cls1_AXIM1_RELEASE_tag AXIM1_RELEASE; /* AXIM[i] slot release (de-allocation). */
  uint8_t GTM_gtm_cls1_reserved116[20];
  GTM_gtm_cls1_AXIM1_SLOT0_ADDR_LOW_tag AXIM1_SLOT0_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls1_reserved117[4];
  GTM_gtm_cls1_AXIM1_SLOT0_DATA_LOW_tag AXIM1_SLOT0_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls1_reserved118[4];
  GTM_gtm_cls1_AXIM1_SLOT0_CFG1_tag AXIM1_SLOT0_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls1_AXIM1_SLOT0_CFG2_tag AXIM1_SLOT0_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls1_AXIM1_SLOT0_STATUS_tag AXIM1_SLOT0_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls1_reserved119[4];
  GTM_gtm_cls1_AXIM1_SLOT1_ADDR_LOW_tag AXIM1_SLOT1_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls1_reserved120[4];
  GTM_gtm_cls1_AXIM1_SLOT1_DATA_LOW_tag AXIM1_SLOT1_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls1_reserved121[4];
  GTM_gtm_cls1_AXIM1_SLOT1_CFG1_tag AXIM1_SLOT1_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls1_AXIM1_SLOT1_CFG2_tag AXIM1_SLOT1_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls1_AXIM1_SLOT1_STATUS_tag AXIM1_SLOT1_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls1_reserved122[4];
  GTM_gtm_cls1_AXIM1_SLOT2_ADDR_LOW_tag AXIM1_SLOT2_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls1_reserved123[4];
  GTM_gtm_cls1_AXIM1_SLOT2_DATA_LOW_tag AXIM1_SLOT2_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls1_reserved124[4];
  GTM_gtm_cls1_AXIM1_SLOT2_CFG1_tag AXIM1_SLOT2_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls1_AXIM1_SLOT2_CFG2_tag AXIM1_SLOT2_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls1_AXIM1_SLOT2_STATUS_tag AXIM1_SLOT2_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls1_reserved125[4];
  GTM_gtm_cls1_AXIM1_SLOT3_ADDR_LOW_tag AXIM1_SLOT3_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls1_reserved126[4];
  GTM_gtm_cls1_AXIM1_SLOT3_DATA_LOW_tag AXIM1_SLOT3_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls1_reserved127[4];
  GTM_gtm_cls1_AXIM1_SLOT3_CFG1_tag AXIM1_SLOT3_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls1_AXIM1_SLOT3_CFG2_tag AXIM1_SLOT3_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls1_AXIM1_SLOT3_STATUS_tag AXIM1_SLOT3_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls1_reserved128[44900];
  GTM_gtm_cls1_MCS1_MEM_tag MCS1_MEM[3072]; /* MCS[i] memory region */
};


/* ============================================================================
   =============================== Module: GTM_gtm_cls2 =======================
   ============================================================================ */

typedef union GTM_gtm_cls2_TIM2_CH0_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH0_GPR0_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH0_GPR1_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH0_CNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_TIM2_CH0_ECNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH0_CNTS_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH0_TDUC_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls2_TIM2_CH0_TDUV_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH0_FLT_RE_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH0_FLT_FE_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls2_TIM2_CH0_CTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls2_TIM2_CH0_ECTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIM2_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIM2_CH0_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH0_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH1_GPR0_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH1_GPR1_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH1_CNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_TIM2_CH1_ECNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH1_CNTS_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH1_TDUC_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls2_TIM2_CH1_TDUV_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH1_FLT_RE_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH1_FLT_FE_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls2_TIM2_CH1_CTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls2_TIM2_CH1_ECTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIM2_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIM2_CH1_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH1_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH2_GPR0_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH2_GPR1_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH2_CNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_TIM2_CH2_ECNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH2_CNTS_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH2_TDUC_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls2_TIM2_CH2_TDUV_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH2_FLT_RE_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH2_FLT_FE_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls2_TIM2_CH2_CTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls2_TIM2_CH2_ECTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIM2_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIM2_CH2_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH2_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH3_GPR0_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH3_GPR1_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH3_CNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_TIM2_CH3_ECNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH3_CNTS_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH3_TDUC_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls2_TIM2_CH3_TDUV_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH3_FLT_RE_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH3_FLT_FE_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls2_TIM2_CH3_CTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls2_TIM2_CH3_ECTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIM2_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIM2_CH3_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH3_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH4_GPR0_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH4_GPR1_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH4_CNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_TIM2_CH4_ECNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH4_CNTS_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH4_TDUC_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls2_TIM2_CH4_TDUV_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH4_FLT_RE_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH4_FLT_FE_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls2_TIM2_CH4_CTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls2_TIM2_CH4_ECTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIM2_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIM2_CH4_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH4_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH5_GPR0_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH5_GPR1_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH5_CNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_TIM2_CH5_ECNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH5_CNTS_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH5_TDUC_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls2_TIM2_CH5_TDUV_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH5_FLT_RE_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH5_FLT_FE_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls2_TIM2_CH5_CTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls2_TIM2_CH5_ECTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIM2_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIM2_CH5_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH5_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH6_GPR0_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH6_GPR1_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH6_CNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_TIM2_CH6_ECNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH6_CNTS_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH6_TDUC_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls2_TIM2_CH6_TDUV_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH6_FLT_RE_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH6_FLT_FE_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls2_TIM2_CH6_CTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls2_TIM2_CH6_ECTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIM2_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIM2_CH6_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH6_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_GPR0_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR0f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH7_GPR0_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_GPR1_union_tag { /* TIM[i] channel [x] general purpose 0 register */
  vuint32_t R;
  struct {
    vuint32_t GPR1f:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH7_GPR1_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_CNT_union_tag { /* TIM[i] channel [x] SMU counter register */
  vuint32_t R;
  struct {
    vuint32_t CNT:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH7_CNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_ECNT_union_tag { /* TIM[i] channel [x] SMU edge counter register */
  vuint32_t R;
  struct {
    vuint32_t ECNT:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_TIM2_CH7_ECNT_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_CNTS_union_tag { /* TIM[i] channel [x] SMU shadow counter register */
  vuint32_t R;
  struct {
    vuint32_t CNTS:24;
    vuint32_t ECNT:8;
  } B;
} GTM_gtm_cls2_TIM2_CH7_CNTS_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_TDUC_union_tag { /* TIM[i] channel [x] TDU counter register */
  vuint32_t R;
  struct {
    vuint32_t TO_CNT:8;
    vuint32_t TO_CNT1:8;
    vuint32_t TO_CNT2:8;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH7_TDUC_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_TDUV_union_tag { /* TIM[i] channel [x] TDU control register */
  vuint32_t R;
  struct {
    vuint32_t TOV:8;
    vuint32_t TOV1:8;
    vuint32_t TOV2:8;
    vuint32_t SLICING:2;
    vuint32_t TCS_USE_SAMPLE_EVT:1;
    vuint32_t TDU_SAME_CNT_CLK:1;
    vuint32_t TCS:3;
    vuint32_t _unused_31:1;
  } B;
} GTM_gtm_cls2_TIM2_CH7_TDUV_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_FLT_RE_union_tag { /* TIM[i] channel [x] filter parameter 0 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_RE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH7_FLT_RE_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_FLT_FE_union_tag { /* TIM[i] channel [x] filter parameter 1 register */
  vuint32_t R;
  struct {
    vuint32_t FLT_FE:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_CH7_FLT_FE_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_CTRL_union_tag { /* TIM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t TIM_EN:1;
    vuint32_t TIM_MODE:3;
    vuint32_t OSM:1;
    vuint32_t ARU_EN:1;
    vuint32_t CICTRL:1;
    vuint32_t _unused_7:1;
    vuint32_t GPR0_SEL:2;
    vuint32_t GPR1_SEL:2;
    vuint32_t CNTS_SEL:1;
    vuint32_t DSL:1;
    vuint32_t ISL:1;
    vuint32_t ECNT_RESET:1;
    vuint32_t FLT_EN:1;
    vuint32_t FLT_CNT_FRQ:2;
    vuint32_t EXT_CAP_EN:1;
    vuint32_t FLT_MODE_RE:1;
    vuint32_t FLT_CTR_RE:1;
    vuint32_t FLT_MODE_FE:1;
    vuint32_t FLT_CTR_FE:1;
    vuint32_t CLK_SEL:3;
    vuint32_t FR_ECNT_OFL:1;
    vuint32_t EGPR0_SEL:1;
    vuint32_t EGPR1_SEL:1;
    vuint32_t TOCTRL:2;
  } B;
} GTM_gtm_cls2_TIM2_CH7_CTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_ECTRL_union_tag { /* TIM[i] channel [x] extended control register */
  vuint32_t R;
  struct {
    vuint32_t EXT_CAP_SRC:4;
    vuint32_t _unused_4:1;
    vuint32_t USE_PREV_TDU_IN:1;
    vuint32_t TODET_IRQ_SRC:2;
    vuint32_t TDU_START:3;
    vuint32_t _unused_11:1;
    vuint32_t TDU_STOP:3;
    vuint32_t _unused_15:1;
    vuint32_t TDU_RESYNC:4;
    vuint32_t _unused_20:2;
    vuint32_t USE_LUT:2;
    vuint32_t EFLT_CTR_RE:1;
    vuint32_t EFLT_CTR_FE:1;
    vuint32_t _unused_26:2;
    vuint32_t SWAP_CAPTURE:1;
    vuint32_t IMM_START:1;
    vuint32_t ECLK_SEL:1;
    vuint32_t USE_PREV_CH_IN:1;
  } B;
} GTM_gtm_cls2_TIM2_CH7_ECTRL_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_IRQ_NOTIFY_union_tag { /* TIM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL:1;
    vuint32_t ECNTOFL:1;
    vuint32_t CNTOFL:1;
    vuint32_t GPROFL:1;
    vuint32_t TODET:1;
    vuint32_t GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_IRQ_EN_union_tag { /* TIM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_IRQ_EN:1;
    vuint32_t ECNTOFL_IRQ_EN:1;
    vuint32_t CNTOFL_IRQ_EN:1;
    vuint32_t GPROFL_IRQ_EN:1;
    vuint32_t TODET_IRQ_EN:1;
    vuint32_t GLITCHDET_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_IRQ_FORCINT_union_tag { /* TIM[i] channel [x] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_NEWVAL:1;
    vuint32_t TRG_ECNTOFL:1;
    vuint32_t TRG_CNTOFL:1;
    vuint32_t TRG_GPROFL:1;
    vuint32_t TRG_TODET:1;
    vuint32_t TRG_GLITCHDET:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_IRQ_MODE_union_tag { /* TIM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIM2_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIM2_CH7_EIRQ_EN_union_tag { /* TIM[i] channel [x] error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t NEWVAL_EIRQ_EN:1;
    vuint32_t ECNTOFL_EIRQ_EN:1;
    vuint32_t CNTOFL_EIRQ_EN:1;
    vuint32_t GPROFL_EIRQ_EN:1;
    vuint32_t TODET_EIRQ_EN:1;
    vuint32_t GLITCHDET_EIRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIM2_CH7_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_TIM2_INP_VAL_union_tag { /* TIM[i] input value observation register */
  vuint32_t R;
  struct {
    vuint32_t F_OUT0:1;
    vuint32_t F_OUT1:1;
    vuint32_t F_OUT2:1;
    vuint32_t F_OUT3:1;
    vuint32_t F_OUT4:1;
    vuint32_t F_OUT5:1;
    vuint32_t F_OUT6:1;
    vuint32_t F_OUT7:1;
    vuint32_t F_IN0:1;
    vuint32_t F_IN1:1;
    vuint32_t F_IN2:1;
    vuint32_t F_IN3:1;
    vuint32_t F_IN4:1;
    vuint32_t F_IN5:1;
    vuint32_t F_IN6:1;
    vuint32_t F_IN7:1;
    vuint32_t TIM_IN0:1;
    vuint32_t TIM_IN1:1;
    vuint32_t TIM_IN2:1;
    vuint32_t TIM_IN3:1;
    vuint32_t TIM_IN4:1;
    vuint32_t TIM_IN5:1;
    vuint32_t TIM_IN6:1;
    vuint32_t TIM_IN7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIM2_INP_VAL_tag;

typedef union GTM_gtm_cls2_TIM2_IN_SRC_union_tag { /* TIM[i] AUX IN source selection register */
  vuint32_t R;
  struct {
    vuint32_t VAL_0:2;
    vuint32_t MODE_0:2;
    vuint32_t VAL_1:2;
    vuint32_t MODE_1:2;
    vuint32_t VAL_2:2;
    vuint32_t MODE_2:2;
    vuint32_t VAL_3:2;
    vuint32_t MODE_3:2;
    vuint32_t VAL_4:2;
    vuint32_t MODE_4:2;
    vuint32_t VAL_5:2;
    vuint32_t MODE_5:2;
    vuint32_t VAL_6:2;
    vuint32_t MODE_6:2;
    vuint32_t VAL_7:2;
    vuint32_t MODE_7:2;
  } B;
} GTM_gtm_cls2_TIM2_IN_SRC_tag;

typedef union GTM_gtm_cls2_TIM2_RST_union_tag { /* TIM[i] global software reset register */
  vuint32_t R;
  struct {
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIM2_RST_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_RDADDR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_SR0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_SR1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_CM0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_CM1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_CN0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_CTRL2_tag;

typedef union GTM_gtm_cls2_ATOM2_CH0_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_CH0_CTRL_SR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_RDADDR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_SR0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_SR1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_CM0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_CM1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_CN0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_CTRL2_tag;

typedef union GTM_gtm_cls2_ATOM2_CH1_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_CH1_CTRL_SR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_RDADDR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_SR0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_SR1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_CM0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_CM1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_CN0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_CTRL2_tag;

typedef union GTM_gtm_cls2_ATOM2_CH2_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_CH2_CTRL_SR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_RDADDR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_SR0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_SR1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_CM0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_CM1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_CN0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_CTRL2_tag;

typedef union GTM_gtm_cls2_ATOM2_CH3_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_CH3_CTRL_SR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_RDADDR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_SR0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_SR1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_CM0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_CM1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_CN0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_CTRL2_tag;

typedef union GTM_gtm_cls2_ATOM2_CH4_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_CH4_CTRL_SR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_RDADDR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_SR0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_SR1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_CM0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_CM1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_CN0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_CTRL2_tag;

typedef union GTM_gtm_cls2_ATOM2_CH5_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_CH5_CTRL_SR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_RDADDR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_SR0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_SR1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_CM0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_CM1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_CN0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_CTRL2_tag;

typedef union GTM_gtm_cls2_ATOM2_CH6_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_CH6_CTRL_SR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_RDADDR_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_SR0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_SR1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_CM0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_CM1_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_CN0_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_CTRL2_union_tag { /* ATOM[i] channel [x] control2 register */
  vuint32_t R;
  struct {
    vuint32_t HRES:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_CTRL2_tag;

typedef union GTM_gtm_cls2_ATOM2_CH7_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_CH7_CTRL_SR_tag;

typedef union GTM_gtm_cls2_ATOM2_AGC_GLB_CTRL_union_tag { /* ATOM[i] AGC global control register */
  vuint32_t R;
  struct {
    vuint32_t HOST_TRIG:1;
    vuint32_t _unused_1:7;
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t UPEN_CTRL0:2;
    vuint32_t UPEN_CTRL1:2;
    vuint32_t UPEN_CTRL2:2;
    vuint32_t UPEN_CTRL3:2;
    vuint32_t UPEN_CTRL4:2;
    vuint32_t UPEN_CTRL5:2;
    vuint32_t UPEN_CTRL6:2;
    vuint32_t UPEN_CTRL7:2;
  } B;
} GTM_gtm_cls2_ATOM2_AGC_GLB_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_AGC_ENDIS_CTRL_union_tag { /* ATOM[i] AGC enable/disable control register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CTRL0:2;
    vuint32_t ENDIS_CTRL1:2;
    vuint32_t ENDIS_CTRL2:2;
    vuint32_t ENDIS_CTRL3:2;
    vuint32_t ENDIS_CTRL4:2;
    vuint32_t ENDIS_CTRL5:2;
    vuint32_t ENDIS_CTRL6:2;
    vuint32_t ENDIS_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_AGC_ENDIS_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_AGC_ENDIS_STAT_union_tag { /* ATOM[i] AGC enable/disable status register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_STAT0:2;
    vuint32_t ENDIS_STAT1:2;
    vuint32_t ENDIS_STAT2:2;
    vuint32_t ENDIS_STAT3:2;
    vuint32_t ENDIS_STAT4:2;
    vuint32_t ENDIS_STAT5:2;
    vuint32_t ENDIS_STAT6:2;
    vuint32_t ENDIS_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_AGC_ENDIS_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_AGC_ACT_TB_union_tag { /* ATOM[i] AGC action time base register */
  vuint32_t R;
  struct {
    vuint32_t ACT_TB:24;
    vuint32_t TB_TRIG:1;
    vuint32_t TBU_SEL:2;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls2_ATOM2_AGC_ACT_TB_tag;

typedef union GTM_gtm_cls2_ATOM2_AGC_OUTEN_CTRL_union_tag { /* ATOM[i] AGC output enable control register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_CTRL0:2;
    vuint32_t OUTEN_CTRL1:2;
    vuint32_t OUTEN_CTRL2:2;
    vuint32_t OUTEN_CTRL3:2;
    vuint32_t OUTEN_CTRL4:2;
    vuint32_t OUTEN_CTRL5:2;
    vuint32_t OUTEN_CTRL6:2;
    vuint32_t OUTEN_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_AGC_OUTEN_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_AGC_OUTEN_STAT_union_tag { /* ATOM[i] AGC output enable status register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_STAT0:2;
    vuint32_t OUTEN_STAT1:2;
    vuint32_t OUTEN_STAT2:2;
    vuint32_t OUTEN_STAT3:2;
    vuint32_t OUTEN_STAT4:2;
    vuint32_t OUTEN_STAT5:2;
    vuint32_t OUTEN_STAT6:2;
    vuint32_t OUTEN_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_AGC_OUTEN_STAT_tag;

typedef union GTM_gtm_cls2_ATOM2_AGC_FUPD_CTRL_union_tag { /* ATOM[i] AGC force update control register */
  vuint32_t R;
  struct {
    vuint32_t FUPD_CTRL0:2;
    vuint32_t FUPD_CTRL1:2;
    vuint32_t FUPD_CTRL2:2;
    vuint32_t FUPD_CTRL3:2;
    vuint32_t FUPD_CTRL4:2;
    vuint32_t FUPD_CTRL5:2;
    vuint32_t FUPD_CTRL6:2;
    vuint32_t FUPD_CTRL7:2;
    vuint32_t RSTCN0_CH0:2;
    vuint32_t RSTCN0_CH1:2;
    vuint32_t RSTCN0_CH2:2;
    vuint32_t RSTCN0_CH3:2;
    vuint32_t RSTCN0_CH4:2;
    vuint32_t RSTCN0_CH5:2;
    vuint32_t RSTCN0_CH6:2;
    vuint32_t RSTCN0_CH7:2;
  } B;
} GTM_gtm_cls2_ATOM2_AGC_FUPD_CTRL_tag;

typedef union GTM_gtm_cls2_ATOM2_AGC_INT_TRIG_union_tag { /* ATOM[i] AGC internal trigger control register */
  vuint32_t R;
  struct {
    vuint32_t INT_TRIG0:2;
    vuint32_t INT_TRIG1:2;
    vuint32_t INT_TRIG2:2;
    vuint32_t INT_TRIG3:2;
    vuint32_t INT_TRIG4:2;
    vuint32_t INT_TRIG5:2;
    vuint32_t INT_TRIG6:2;
    vuint32_t INT_TRIG7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_ATOM2_AGC_INT_TRIG_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH0_R0_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH0_R1_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH0_R2_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH0_R3_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH0_R4_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH0_R5_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH0_R6_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH0_R7_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls2_MCS2_CH0_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_MCS2_CH0_ACB_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CH0_MHB_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_CH0_PC_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_CH0_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH0_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH1_R0_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH1_R1_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH1_R2_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH1_R3_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH1_R4_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH1_R5_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH1_R6_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH1_R7_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls2_MCS2_CH1_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_MCS2_CH1_ACB_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CH1_MHB_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_CH1_PC_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_CH1_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH1_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH2_R0_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH2_R1_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH2_R2_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH2_R3_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH2_R4_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH2_R5_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH2_R6_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH2_R7_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls2_MCS2_CH2_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_MCS2_CH2_ACB_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CH2_MHB_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_CH2_PC_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_CH2_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH2_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH3_R0_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH3_R1_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH3_R2_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH3_R3_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH3_R4_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH3_R5_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH3_R6_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH3_R7_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls2_MCS2_CH3_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_MCS2_CH3_ACB_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CH3_MHB_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_CH3_PC_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_CH3_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH3_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH4_R0_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH4_R1_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH4_R2_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH4_R3_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH4_R4_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH4_R5_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH4_R6_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH4_R7_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls2_MCS2_CH4_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_MCS2_CH4_ACB_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CH4_MHB_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_CH4_PC_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_CH4_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH4_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH5_R0_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH5_R1_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH5_R2_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH5_R3_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH5_R4_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH5_R5_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH5_R6_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH5_R7_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls2_MCS2_CH5_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_MCS2_CH5_ACB_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CH5_MHB_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_CH5_PC_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_CH5_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH5_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH6_R0_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH6_R1_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH6_R2_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH6_R3_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH6_R4_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH6_R5_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH6_R6_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH6_R7_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls2_MCS2_CH6_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_MCS2_CH6_ACB_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CH6_MHB_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_CH6_PC_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_CH6_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH6_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH7_R0_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH7_R1_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH7_R2_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH7_R3_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH7_R4_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH7_R5_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH7_R6_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CH7_R7_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls2_MCS2_CH7_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_MCS2_CH7_ACB_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CH7_MHB_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_CH7_PC_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_CH7_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls2_MCS2_CH7_EIRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_CTRG_union_tag { /* MCS[i] clear trigger control register */
  vuint32_t R;
  struct {
    vuint32_t TRG0:1;
    vuint32_t TRG1:1;
    vuint32_t TRG2:1;
    vuint32_t TRG3:1;
    vuint32_t TRG4:1;
    vuint32_t TRG5:1;
    vuint32_t TRG6:1;
    vuint32_t TRG7:1;
    vuint32_t TRG8:1;
    vuint32_t TRG9:1;
    vuint32_t TRG10:1;
    vuint32_t TRG11:1;
    vuint32_t TRG12:1;
    vuint32_t TRG13:1;
    vuint32_t TRG14:1;
    vuint32_t TRG15:1;
    vuint32_t TRG16:1;
    vuint32_t TRG17:1;
    vuint32_t TRG18:1;
    vuint32_t TRG19:1;
    vuint32_t TRG20:1;
    vuint32_t TRG21:1;
    vuint32_t TRG22:1;
    vuint32_t TRG23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_CTRG_tag;

typedef union GTM_gtm_cls2_MCS2_STRG_union_tag { /* MCS[i] set trigger control register */
  vuint32_t R;
  struct {
    vuint32_t TRG0:1;
    vuint32_t TRG1:1;
    vuint32_t TRG2:1;
    vuint32_t TRG3:1;
    vuint32_t TRG4:1;
    vuint32_t TRG5:1;
    vuint32_t TRG6:1;
    vuint32_t TRG7:1;
    vuint32_t TRG8:1;
    vuint32_t TRG9:1;
    vuint32_t TRG10:1;
    vuint32_t TRG11:1;
    vuint32_t TRG12:1;
    vuint32_t TRG13:1;
    vuint32_t TRG14:1;
    vuint32_t TRG15:1;
    vuint32_t TRG16:1;
    vuint32_t TRG17:1;
    vuint32_t TRG18:1;
    vuint32_t TRG19:1;
    vuint32_t TRG20:1;
    vuint32_t TRG21:1;
    vuint32_t TRG22:1;
    vuint32_t TRG23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_MCS2_STRG_tag;

typedef union GTM_gtm_cls2_MCS2_CTRL_STAT_union_tag { /* MCS[i] control and status register */
  vuint32_t R;
  struct {
    vuint32_t SCD_MODE:2;
    vuint32_t _unused_2:6;
    vuint32_t SCD_CH:4;
    vuint32_t _unused_12:4;
    vuint32_t RAM_RST:1;
    vuint32_t _unused_17:3;
    vuint32_t ERR_SRC_ID:3;
    vuint32_t _unused_23:1;
    vuint32_t EN_TIM_FOUT:1;
    vuint32_t EN_HVD:1;
    vuint32_t HLT_AEIM_ERR:1;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls2_MCS2_CTRL_STAT_tag;

typedef union GTM_gtm_cls2_MCS2_RESET_union_tag { /* MCS[i] reset register */
  vuint32_t R;
  struct {
    vuint32_t RST0:1;
    vuint32_t RST1:1;
    vuint32_t RST2:1;
    vuint32_t RST3:1;
    vuint32_t RST4:1;
    vuint32_t RST5:1;
    vuint32_t RST6:1;
    vuint32_t RST7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_RESET_tag;

typedef union GTM_gtm_cls2_MCS2_CAT_union_tag { /* MCS[i] cancel ARU transfer instruction */
  vuint32_t R;
  struct {
    vuint32_t CAT0:1;
    vuint32_t CAT1:1;
    vuint32_t CAT2:1;
    vuint32_t CAT3:1;
    vuint32_t CAT4:1;
    vuint32_t CAT5:1;
    vuint32_t CAT6:1;
    vuint32_t CAT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CAT_tag;

typedef union GTM_gtm_cls2_MCS2_CWT_union_tag { /* MCS[i] cancel waiting instruction */
  vuint32_t R;
  struct {
    vuint32_t CWT0:1;
    vuint32_t CWT1:1;
    vuint32_t CWT2:1;
    vuint32_t CWT3:1;
    vuint32_t CWT4:1;
    vuint32_t CWT5:1;
    vuint32_t CWT6:1;
    vuint32_t CWT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_CWT_tag;

typedef union GTM_gtm_cls2_MCS2_ERR_union_tag { /* MCS[i] error register */
  vuint32_t R;
  struct {
    vuint32_t ERR0:1;
    vuint32_t ERR1:1;
    vuint32_t ERR2:1;
    vuint32_t ERR3:1;
    vuint32_t ERR4:1;
    vuint32_t ERR5:1;
    vuint32_t ERR6:1;
    vuint32_t ERR7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_ERR_tag;

typedef union GTM_gtm_cls2_MCS2_REG_PROT_union_tag { /* MCS[i] write protection register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:2;
    vuint32_t WPROT1:2;
    vuint32_t WPROT2:2;
    vuint32_t WPROT3:2;
    vuint32_t WPROT4:2;
    vuint32_t WPROT5:2;
    vuint32_t WPROT6:2;
    vuint32_t WPROT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_MCS2_REG_PROT_tag;

typedef union GTM_gtm_cls2_MCS2_SINT_IRQ_NOTIFY_union_tag { /* MCS[i] shared interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_IRQ0:1;
    vuint32_t S_IRQ1:1;
    vuint32_t S_IRQ2:1;
    vuint32_t S_IRQ3:1;
    vuint32_t S_IRQ4:1;
    vuint32_t S_IRQ5:1;
    vuint32_t S_IRQ6:1;
    vuint32_t S_IRQ7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_SINT_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_SINT_IRQ_EN_union_tag { /* MCS[i] shared interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_IRQ0_EN:1;
    vuint32_t S_IRQ1_EN:1;
    vuint32_t S_IRQ2_EN:1;
    vuint32_t S_IRQ3_EN:1;
    vuint32_t S_IRQ4_EN:1;
    vuint32_t S_IRQ5_EN:1;
    vuint32_t S_IRQ6_EN:1;
    vuint32_t S_IRQ7_EN:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_SINT_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_SINT_IRQ_FORCINT_union_tag { /* MCS[i] force shared interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_IRQ0:1;
    vuint32_t TRG_S_IRQ1:1;
    vuint32_t TRG_S_IRQ2:1;
    vuint32_t TRG_S_IRQ3:1;
    vuint32_t TRG_S_IRQ4:1;
    vuint32_t TRG_S_IRQ5:1;
    vuint32_t TRG_S_IRQ6:1;
    vuint32_t TRG_S_IRQ7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_SINT_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_SINT_IRQ_MODE_union_tag { /* MCS[i] shared interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_SINT_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_HBP0_CTRL_union_tag { /* MCS[i] hardware break point h control register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH0:1;
    vuint32_t EN_CH1:1;
    vuint32_t EN_CH2:1;
    vuint32_t EN_CH3:1;
    vuint32_t EN_CH4:1;
    vuint32_t EN_CH5:1;
    vuint32_t EN_CH6:1;
    vuint32_t EN_CH7:1;
    vuint32_t SCOPE:2;
    vuint32_t _unused_10:2;
    vuint32_t TYPE:3;
    vuint32_t _unused_15:1;
    vuint32_t AND:1;
    vuint32_t NOT:1;
    vuint32_t _unused_18:14;
  } B;
} GTM_gtm_cls2_MCS2_HBP0_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_HBP0_PATTERN_union_tag { /* MCS[i] hardware break point pattern register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls2_MCS2_HBP0_PATTERN_tag;

typedef union GTM_gtm_cls2_MCS2_HBP0_STATUS_union_tag { /* MCS[i] hardware break point status register */
  vuint32_t R;
  struct {
    vuint32_t HALT_CH0:1;
    vuint32_t HALT_CH1:1;
    vuint32_t HALT_CH2:1;
    vuint32_t HALT_CH3:1;
    vuint32_t HALT_CH4:1;
    vuint32_t HALT_CH5:1;
    vuint32_t HALT_CH6:1;
    vuint32_t HALT_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_HBP0_STATUS_tag;

typedef union GTM_gtm_cls2_MCS2_HBP0_IRQ_NOTIFY_union_tag { /* MCS[i] hardware break point interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_MCS2_HBP0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_HBP0_IRQ_EN_union_tag { /* MCS[i] hardware break point interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ_EN:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_MCS2_HBP0_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_HBP0_IRQ_FORCINT_union_tag { /* MCS[i] force hardware break point interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_MCS2_HBP0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_HBP0_IRQ_MODE_union_tag { /* MCS[i] break point h interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_HBP0_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_MCS2_HBP1_CTRL_union_tag { /* MCS[i] hardware break point h control register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH0:1;
    vuint32_t EN_CH1:1;
    vuint32_t EN_CH2:1;
    vuint32_t EN_CH3:1;
    vuint32_t EN_CH4:1;
    vuint32_t EN_CH5:1;
    vuint32_t EN_CH6:1;
    vuint32_t EN_CH7:1;
    vuint32_t SCOPE:2;
    vuint32_t _unused_10:2;
    vuint32_t TYPE:3;
    vuint32_t _unused_15:1;
    vuint32_t AND:1;
    vuint32_t NOT:1;
    vuint32_t _unused_18:14;
  } B;
} GTM_gtm_cls2_MCS2_HBP1_CTRL_tag;

typedef union GTM_gtm_cls2_MCS2_HBP1_PATTERN_union_tag { /* MCS[i] hardware break point pattern register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls2_MCS2_HBP1_PATTERN_tag;

typedef union GTM_gtm_cls2_MCS2_HBP1_STATUS_union_tag { /* MCS[i] hardware break point status register */
  vuint32_t R;
  struct {
    vuint32_t HALT_CH0:1;
    vuint32_t HALT_CH1:1;
    vuint32_t HALT_CH2:1;
    vuint32_t HALT_CH3:1;
    vuint32_t HALT_CH4:1;
    vuint32_t HALT_CH5:1;
    vuint32_t HALT_CH6:1;
    vuint32_t HALT_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_MCS2_HBP1_STATUS_tag;

typedef union GTM_gtm_cls2_MCS2_HBP1_IRQ_NOTIFY_union_tag { /* MCS[i] hardware break point interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_MCS2_HBP1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_MCS2_HBP1_IRQ_EN_union_tag { /* MCS[i] hardware break point interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ_EN:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_MCS2_HBP1_IRQ_EN_tag;

typedef union GTM_gtm_cls2_MCS2_HBP1_IRQ_FORCINT_union_tag { /* MCS[i] force hardware break point interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_MCS2_HBP1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_MCS2_HBP1_IRQ_MODE_union_tag { /* MCS[i] break point h interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_MCS2_HBP1_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_CTRL_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_SINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_SCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_SOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_OINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_OCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_OOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH0_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH0_SHIFTCNT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_CTRL_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_SINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_SCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_SOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_OINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_OCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_OOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH1_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH1_SHIFTCNT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_CTRL_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_SINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_SCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_SOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_OINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_OCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_OOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH2_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH2_SHIFTCNT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_CTRL_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_SINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_SCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_SOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_OINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_OCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_OOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH3_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH3_SHIFTCNT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_CTRL_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_SINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_SCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_SOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_OINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_OCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_OOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH4_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH4_SHIFTCNT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_CTRL_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_SINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_SCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_SOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_OINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_OCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_OOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH5_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH5_SHIFTCNT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_CTRL_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_SINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_SCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_SOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_OINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_OCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_OOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH6_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH6_SHIFTCNT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_CTRL_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_SINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_SCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_SOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_OINST_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_OCMD_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_OOP_tag;

typedef union GTM_gtm_cls2_TIO2_G0_CH7_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_G0_CH7_SHIFTCNT_tag;

typedef union GTM_gtm_cls2_TIO2_G0_ISEL0_CTRL1_union_tag { /* TIO[i] input selection register 1 */
  vuint32_t R;
  struct {
    vuint32_t LUT2_0:4;
    vuint32_t LUT2_1:4;
    vuint32_t LUT2_2:4;
    vuint32_t LUT2_3:4;
    vuint32_t OUT_SEL0:1;
    vuint32_t OUT_SEL1:1;
    vuint32_t OUT_SEL2:1;
    vuint32_t OUT_SEL3:1;
    vuint32_t _unused_20:4;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls2_TIO2_G0_ISEL0_CTRL1_tag;

typedef union GTM_gtm_cls2_TIO2_G0_ISEL0_CTRL2_union_tag { /* TIO[i] input selection register 2 */
  vuint32_t R;
  struct {
    vuint32_t LUT3:8;
    vuint32_t _unused_8:8;
    vuint32_t QOUT_SEL:2;
    vuint32_t _unused_18:2;
    vuint32_t LUT3IN_SEL0:1;
    vuint32_t LUT3IN_SEL1:1;
    vuint32_t LUT3IN_SEL2:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls2_TIO2_G0_ISEL0_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_ISEL1_CTRL1_union_tag { /* TIO[i] input selection register 1 */
  vuint32_t R;
  struct {
    vuint32_t LUT2_0:4;
    vuint32_t LUT2_1:4;
    vuint32_t LUT2_2:4;
    vuint32_t LUT2_3:4;
    vuint32_t OUT_SEL0:1;
    vuint32_t OUT_SEL1:1;
    vuint32_t OUT_SEL2:1;
    vuint32_t OUT_SEL3:1;
    vuint32_t _unused_20:4;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls2_TIO2_G0_ISEL1_CTRL1_tag;

typedef union GTM_gtm_cls2_TIO2_G0_ISEL1_CTRL2_union_tag { /* TIO[i] input selection register 2 */
  vuint32_t R;
  struct {
    vuint32_t LUT3:8;
    vuint32_t _unused_8:8;
    vuint32_t QOUT_SEL:2;
    vuint32_t _unused_18:2;
    vuint32_t LUT3IN_SEL0:1;
    vuint32_t LUT3IN_SEL1:1;
    vuint32_t LUT3IN_SEL2:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls2_TIO2_G0_ISEL1_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_G0_OP_USAGE_union_tag { /* TIO[i] operand usage selection register */
  vuint32_t R;
  struct {
    vuint32_t MODE0:3;
    vuint32_t MODE1:3;
    vuint32_t MODE2:3;
    vuint32_t MODE3:3;
    vuint32_t MODE4:3;
    vuint32_t MODE5:3;
    vuint32_t MODE6:3;
    vuint32_t MODE7:3;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t WRITE_EN4:1;
    vuint32_t WRITE_EN5:1;
    vuint32_t WRITE_EN6:1;
    vuint32_t WRITE_EN7:1;
  } B;
} GTM_gtm_cls2_TIO2_G0_OP_USAGE_tag;

typedef union GTM_gtm_cls2_TIO2_S_union_tag { /* TIO[i] signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_S_tag;

typedef union GTM_gtm_cls2_TIO2_O_union_tag { /* TIO[i] output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_O_tag;

typedef union GTM_gtm_cls2_TIO2_ENDIS_union_tag { /* TIO[i] enable/disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_ENDIS_tag;

typedef union GTM_gtm_cls2_TIO2_INVERT_union_tag { /* TIO[i] signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_INVERT_tag;

typedef union GTM_gtm_cls2_TIO2_INPUT_MODE_union_tag { /* TIO[i] input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_INPUT_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_CYCLIC_MODE_union_tag { /* TIO[i] cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CYCLIC_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_TRIG_OUT_GATE_EN_union_tag { /* TIO[i] enable Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_TRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls2_TIO2_PLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] enable PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_PLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls2_TIO2_CS_union_tag { /* TIO[i] clear signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CS_tag;

typedef union GTM_gtm_cls2_TIO2_CO_union_tag { /* TIO[i] clear output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CO_tag;

typedef union GTM_gtm_cls2_TIO2_CENDIS_union_tag { /* TIO[i] disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CENDIS_tag;

typedef union GTM_gtm_cls2_TIO2_CINVERT_union_tag { /* TIO[i] clear signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CINVERT_tag;

typedef union GTM_gtm_cls2_TIO2_CINPUT_MODE_union_tag { /* TIO[i] disable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CINPUT_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_CCYCLIC_MODE_union_tag { /* TIO[i] disable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CCYCLIC_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_CTRIG_OUT_GATE_EN_union_tag { /* TIO[i] clear Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls2_TIO2_CPLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] clear PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_CPLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls2_TIO2_SS_union_tag { /* TIO[i] set signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_SS_tag;

typedef union GTM_gtm_cls2_TIO2_SO_union_tag { /* TIO[i] set output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_SO_tag;

typedef union GTM_gtm_cls2_TIO2_SENDIS_union_tag { /* TIO[i] enable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_SENDIS_tag;

typedef union GTM_gtm_cls2_TIO2_SINVERT_union_tag { /* TIO[i] set signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_SINVERT_tag;

typedef union GTM_gtm_cls2_TIO2_SINPUT_MODE_union_tag { /* TIO[i] enable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_SINPUT_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_SCYCLIC_MODE_union_tag { /* TIO[i] enable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_SCYCLIC_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_STRIG_OUT_GATE_EN_union_tag { /* TIO[i] set Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_STRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls2_TIO2_SPLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] set PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_SPLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls2_TIO2_IS_union_tag { /* TIO[i] invert signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_IS_tag;

typedef union GTM_gtm_cls2_TIO2_IO_union_tag { /* TIO[i] invert output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_IO_tag;

typedef union GTM_gtm_cls2_TIO2_IENDIS_union_tag { /* TIO[i] toggle enable/disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_IENDIS_tag;

typedef union GTM_gtm_cls2_TIO2_IINVERT_union_tag { /* TIO[i] toggle signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_IINVERT_tag;

typedef union GTM_gtm_cls2_TIO2_IINPUT_MODE_union_tag { /* TIO[i] enable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_IINPUT_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_ICYCLIC_MODE_union_tag { /* TIO[i] enable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_ICYCLIC_MODE_tag;

typedef union GTM_gtm_cls2_TIO2_FUPD_union_tag { /* TIO[i] force update register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_FUPD_tag;

typedef union GTM_gtm_cls2_TIO2_HW_CONF_union_tag { /* TIO[i] configuration register */
  vuint32_t R;
  struct {
    vuint32_t NTIO_CH8:2;
    vuint32_t _unused_2:2;
    vuint32_t TIO_PLUS:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls2_TIO2_HW_CONF_tag;

typedef union GTM_gtm_cls2_TIO2_RSEL_CTRL1_union_tag { /* TIO[i] resource selection control register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t SEL_CLKEN6_0:1;
    vuint32_t _unused_25:3;
    vuint32_t SEL_CLKEN7_0:1;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls2_TIO2_RSEL_CTRL1_tag;

typedef union GTM_gtm_cls2_TIO2_RSEL_CTRL2_union_tag { /* TIO[i] resource selection control register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t SEL_TB1_0:1;
    vuint32_t _unused_5:3;
    vuint32_t SEL_TB2_0:1;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls2_TIO2_RSEL_CTRL2_tag;

typedef union GTM_gtm_cls2_TIO2_PL_SWRST_union_tag { /* TIO[i] software reset for TIO Plus functionality */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_TIO2_PL_SWRST_tag;

typedef union GTM_gtm_cls2_CCM2_ARP0_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP0_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP0_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP0_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP1_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP1_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP1_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP1_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP2_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP2_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP2_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP2_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP3_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP3_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP3_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP3_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP4_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP4_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP4_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP4_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP5_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP5_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP5_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP5_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP6_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP6_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP6_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP6_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP7_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP7_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP7_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP7_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP8_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP8_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP8_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP8_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_ARP9_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls2_CCM2_ARP9_CTRL_tag;

typedef union GTM_gtm_cls2_CCM2_ARP9_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_ARP9_PROT_tag;

typedef union GTM_gtm_cls2_CCM2_TIO_G0_OUT_union_tag { /* CCM[i] TIO Group 0,1 Output Register */
  vuint32_t R;
  struct {
    vuint32_t TIO_G0_OUT0:1;
    vuint32_t TIO_G0_OUT1:1;
    vuint32_t TIO_G0_OUT2:1;
    vuint32_t TIO_G0_OUT3:1;
    vuint32_t TIO_G0_OUT4:1;
    vuint32_t TIO_G0_OUT5:1;
    vuint32_t TIO_G0_OUT6:1;
    vuint32_t TIO_G0_OUT7:1;
    vuint32_t _unused_8:8;
    vuint32_t TIO_G0_OUT_N0:1;
    vuint32_t TIO_G0_OUT_N1:1;
    vuint32_t TIO_G0_OUT_N2:1;
    vuint32_t TIO_G0_OUT_N3:1;
    vuint32_t TIO_G0_OUT_N4:1;
    vuint32_t TIO_G0_OUT_N5:1;
    vuint32_t TIO_G0_OUT_N6:1;
    vuint32_t TIO_G0_OUT_N7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_CCM2_TIO_G0_OUT_tag;

typedef union GTM_gtm_cls2_CCM2_HW_CONF2_union_tag { /* CCM[i] 2. Hardware Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t AXIM_ID_WIDTH:5;
    vuint32_t AXIM_PRIV_ACC:1;
    vuint32_t AXIM_SEC_ACC:1;
    vuint32_t AXIM_POSTED_WRITE:1;
    vuint32_t _unused_8:1;
    vuint32_t TIO_OUT_RST:1;
    vuint32_t _unused_10:6;
    vuint32_t AXIS_DATA_SIZE:1;
    vuint32_t _unused_17:1;
    vuint32_t AXIM_DATA_SIZE:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls2_CCM2_HW_CONF2_tag;

typedef union GTM_gtm_cls2_CCM2_AEIM_STA_union_tag { /* CCM[i] MCS Bus Master Status Register */
  vuint32_t R;
  struct {
    vuint32_t AEIM_XPT_ADDR:16;
    vuint32_t _unused_16:8;
    vuint32_t AEIM_XPT_STA:2;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls2_CCM2_AEIM_STA_tag;

typedef union GTM_gtm_cls2_CCM2_HW_CONF_union_tag { /* CCM[i] Hardware Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t GRSTEN:1;
    vuint32_t BRIDGE_MODE_RST:1;
    vuint32_t SYNC_INPUT_REG:1;
    vuint32_t CFG_CLOCK_RATE:1;
    vuint32_t ATOM_OUT_RST:1;
    vuint32_t ATOM_TRIG_CHAIN:3;
    vuint32_t TOM_OUT_RST:1;
    vuint32_t TOM_TRIG_CHAIN:3;
    vuint32_t RAM_INIT_RST:1;
    vuint32_t ERM:1;
    vuint32_t _unused_14:1;
    vuint32_t RESET_ACTIVE:1;
    vuint32_t IRQ_MODE_LEVEL:1;
    vuint32_t IRQ_MODE_PULSE:1;
    vuint32_t IRQ_MODE_PULSE_NOTIFY:1;
    vuint32_t IRQ_MODE_SINGLE_PULSE:1;
    vuint32_t ATOM_TRIG_INTCHAIN:4;
    vuint32_t TOM_TRIG_INTCHAIN:5;
    vuint32_t INT_CLK_EN_GEN:1;
    vuint32_t AEI_ADDR_PIPELINE_STAGE:1;
    vuint32_t AEI_RDATA_PIPELINE_STAGE:1;
  } B;
} GTM_gtm_cls2_CCM2_HW_CONF_tag;

typedef union GTM_gtm_cls2_CCM2_TIM_AUX_IN_SRC_union_tag { /* CCM[i] TIM AUX Input Source Register */
  vuint32_t R;
  struct {
    vuint32_t SRC_CH0:1;
    vuint32_t SRC_CH1:1;
    vuint32_t SRC_CH2:1;
    vuint32_t SRC_CH3:1;
    vuint32_t SRC_CH4:1;
    vuint32_t SRC_CH5:1;
    vuint32_t SRC_CH6:1;
    vuint32_t SRC_CH7:1;
    vuint32_t _unused_8:8;
    vuint32_t SEL_OUT_N_CH0:1;
    vuint32_t SEL_OUT_N_CH1:1;
    vuint32_t SEL_OUT_N_CH2:1;
    vuint32_t SEL_OUT_N_CH3:1;
    vuint32_t SEL_OUT_N_CH4:1;
    vuint32_t SEL_OUT_N_CH5:1;
    vuint32_t SEL_OUT_N_CH6:1;
    vuint32_t SEL_OUT_N_CH7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls2_CCM2_TIM_AUX_IN_SRC_tag;

typedef union GTM_gtm_cls2_CCM2_EXT_CAP_EN_union_tag { /* CCM[i] External Capture Enable Register */
  vuint32_t R;
  struct {
    vuint32_t TIM_I_EXT_CAP_EN0:1;
    vuint32_t TIM_I_EXT_CAP_EN1:1;
    vuint32_t TIM_I_EXT_CAP_EN2:1;
    vuint32_t TIM_I_EXT_CAP_EN3:1;
    vuint32_t TIM_I_EXT_CAP_EN4:1;
    vuint32_t TIM_I_EXT_CAP_EN5:1;
    vuint32_t TIM_I_EXT_CAP_EN6:1;
    vuint32_t TIM_I_EXT_CAP_EN7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CCM2_EXT_CAP_EN_tag;

typedef union GTM_gtm_cls2_CCM2_ATOM_OUT_union_tag { /* CCM[i] ATOM Output Register */
  vuint32_t R;
  struct {
    vuint32_t ATOM_I_OUT0:1;
    vuint32_t ATOM_I_OUT1:1;
    vuint32_t ATOM_I_OUT2:1;
    vuint32_t ATOM_I_OUT3:1;
    vuint32_t ATOM_I_OUT4:1;
    vuint32_t ATOM_I_OUT5:1;
    vuint32_t ATOM_I_OUT6:1;
    vuint32_t ATOM_I_OUT7:1;
    vuint32_t ATOM_I_OUT_N0:1;
    vuint32_t ATOM_I_OUT_N1:1;
    vuint32_t ATOM_I_OUT_N2:1;
    vuint32_t ATOM_I_OUT_N3:1;
    vuint32_t ATOM_I_OUT_N4:1;
    vuint32_t ATOM_I_OUT_N5:1;
    vuint32_t ATOM_I_OUT_N6:1;
    vuint32_t ATOM_I_OUT_N7:1;
    vuint32_t ATOM_IP1_OUT0:1;
    vuint32_t ATOM_IP1_OUT1:1;
    vuint32_t ATOM_IP1_OUT2:1;
    vuint32_t ATOM_IP1_OUT3:1;
    vuint32_t ATOM_IP1_OUT4:1;
    vuint32_t ATOM_IP1_OUT5:1;
    vuint32_t ATOM_IP1_OUT6:1;
    vuint32_t ATOM_IP1_OUT7:1;
    vuint32_t ATOM_IP1_OUT_N0:1;
    vuint32_t ATOM_IP1_OUT_N1:1;
    vuint32_t ATOM_IP1_OUT_N2:1;
    vuint32_t ATOM_IP1_OUT_N3:1;
    vuint32_t ATOM_IP1_OUT_N4:1;
    vuint32_t ATOM_IP1_OUT_N5:1;
    vuint32_t ATOM_IP1_OUT_N6:1;
    vuint32_t ATOM_IP1_OUT_N7:1;
  } B;
} GTM_gtm_cls2_CCM2_ATOM_OUT_tag;

typedef union GTM_gtm_cls2_CCM2_CMU_CLK_CFG_union_tag { /* CCM[i] CMU Clock Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CLK0_SRC:2;
    vuint32_t _unused_2:2;
    vuint32_t CLK1_SRC:2;
    vuint32_t _unused_6:2;
    vuint32_t CLK2_SRC:2;
    vuint32_t _unused_10:2;
    vuint32_t CLK3_SRC:2;
    vuint32_t _unused_14:2;
    vuint32_t CLK4_SRC:2;
    vuint32_t _unused_18:2;
    vuint32_t CLK5_SRC:2;
    vuint32_t _unused_22:2;
    vuint32_t CLK6_SRC:2;
    vuint32_t _unused_26:2;
    vuint32_t CLK7_SRC:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_CCM2_CMU_CLK_CFG_tag;

typedef union GTM_gtm_cls2_CCM2_CFG_union_tag { /* CCM[i] Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t EN_TIM:1;
    vuint32_t _unused_1:1;
    vuint32_t EN_ATOM_ADTM:1;
    vuint32_t EN_MCS:1;
    vuint32_t _unused_4:4;
    vuint32_t EN_TIO_DTM:1;
    vuint32_t _unused_9:7;
    vuint32_t CLS_CLK_DIV:2;
    vuint32_t _unused_18:12;
    vuint32_t TBU_DIR1:1;
    vuint32_t TBU_DIR2:1;
  } B;
} GTM_gtm_cls2_CCM2_CFG_tag;

typedef union GTM_gtm_cls2_CCM2_PROT_union_tag { /* CCM[i] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t CLS_PROT:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls2_CCM2_PROT_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CTRL_union_tag { /* CDTM[i]_DTM[d] global configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t CLK_SEL:2;
    vuint32_t DTM_SEL:2;
    vuint32_t UPD_MODE:3;
    vuint32_t CH_SHUTOFF_EN:1;
    vuint32_t SR_UPD_EN:1;
    vuint32_t _unused_9:7;
    vuint32_t SHUT_OFF_RST:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CTRL_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL1_union_tag { /* CDTM[i]_DTM[d] channel control register 1 */
  vuint32_t R;
  struct {
    vuint32_t O1SEL_0:1;
    vuint32_t _unused_1:2;
    vuint32_t SWAP_0:1;
    vuint32_t O1F_0:2;
    vuint32_t XDT_EN_0_1:1;
    vuint32_t _unused_7:1;
    vuint32_t O1SEL_1:1;
    vuint32_t I1SEL_1:1;
    vuint32_t SH_EN_1:1;
    vuint32_t SWAP_1:1;
    vuint32_t O1F_1:2;
    vuint32_t _unused_14:2;
    vuint32_t O1SEL_2:1;
    vuint32_t I1SEL_2:1;
    vuint32_t SH_EN_2:1;
    vuint32_t SWAP_2:1;
    vuint32_t O1F_2:2;
    vuint32_t XDT_EN_2_3:1;
    vuint32_t _unused_23:1;
    vuint32_t O1SEL_3:1;
    vuint32_t I1SEL_3:1;
    vuint32_t SH_EN_3:1;
    vuint32_t SWAP_3:1;
    vuint32_t O1F_3:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL1_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL2_union_tag { /* CDTM[i]_DTM[d] channel control register 2 */
  vuint32_t R;
  struct {
    vuint32_t POL0_0:1;
    vuint32_t OC0_0:1;
    vuint32_t SL0_0:1;
    vuint32_t DT0_0:1;
    vuint32_t POL1_0:1;
    vuint32_t OC1_0:1;
    vuint32_t SL1_0:1;
    vuint32_t DT1_0:1;
    vuint32_t POL0_1:1;
    vuint32_t OC0_1:1;
    vuint32_t SL0_1:1;
    vuint32_t DT0_1:1;
    vuint32_t POL1_1:1;
    vuint32_t OC1_1:1;
    vuint32_t SL1_1:1;
    vuint32_t DT1_1:1;
    vuint32_t POL0_2:1;
    vuint32_t OC0_2:1;
    vuint32_t SL0_2:1;
    vuint32_t DT0_2:1;
    vuint32_t POL1_2:1;
    vuint32_t OC1_2:1;
    vuint32_t SL1_2:1;
    vuint32_t DT1_2:1;
    vuint32_t POL0_3:1;
    vuint32_t OC0_3:1;
    vuint32_t SL0_3:1;
    vuint32_t DT0_3:1;
    vuint32_t POL1_3:1;
    vuint32_t OC1_3:1;
    vuint32_t SL1_3:1;
    vuint32_t DT1_3:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL2_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL2_SR_union_tag { /* CDTM[i] DTM[j] channel control register 2 shadow */
  vuint32_t R;
  struct {
    vuint32_t POL0_0_SR:1;
    vuint32_t OC0_0_SR:1;
    vuint32_t SL0_0_SR:1;
    vuint32_t DT0_0_SR:1;
    vuint32_t POL1_0_SR:1;
    vuint32_t OC1_0_SR:1;
    vuint32_t SL1_0_SR:1;
    vuint32_t DT1_0_SR:1;
    vuint32_t POL0_1_SR:1;
    vuint32_t OC0_1_SR:1;
    vuint32_t SL0_1_SR:1;
    vuint32_t DT0_1_SR:1;
    vuint32_t POL1_1_SR:1;
    vuint32_t OC1_1_SR:1;
    vuint32_t SL1_1_SR:1;
    vuint32_t DT1_1_SR:1;
    vuint32_t POL0_2_SR:1;
    vuint32_t OC0_2_SR:1;
    vuint32_t SL0_2_SR:1;
    vuint32_t DT0_2_SR:1;
    vuint32_t POL1_2_SR:1;
    vuint32_t OC1_2_SR:1;
    vuint32_t SL1_2_SR:1;
    vuint32_t DT1_2_SR:1;
    vuint32_t POL0_3_SR:1;
    vuint32_t OC0_3_SR:1;
    vuint32_t SL0_3_SR:1;
    vuint32_t DT0_3_SR:1;
    vuint32_t POL1_3_SR:1;
    vuint32_t OC1_3_SR:1;
    vuint32_t SL1_3_SR:1;
    vuint32_t DT1_3_SR:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL2_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_PS_CTRL_union_tag { /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t RELBLK:10;
    vuint32_t _unused_10:6;
    vuint32_t PSU_IN_SEL:1;
    vuint32_t IN_POL:1;
    vuint32_t TIM_SEL:1;
    vuint32_t _unused_19:1;
    vuint32_t SHIFT_SEL:2;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_PS_CTRL_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH_DTV_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE:13;
    vuint32_t _unused_13:3;
    vuint32_t RELFALL:13;
    vuint32_t _unused_29:2;
    vuint32_t HRES:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH_DTV_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH_SR_union_tag { /* CDTM[i]_DTM[d] channel shadow register */
  vuint32_t R;
  struct {
    vuint32_t SL0_0_SR_SR:1;
    vuint32_t SL1_0_SR_SR:1;
    vuint32_t SL0_1_SR_SR:1;
    vuint32_t SL1_1_SR_SR:1;
    vuint32_t SL0_2_SR_SR:1;
    vuint32_t SL1_2_SR_SR:1;
    vuint32_t SL0_3_SR_SR:1;
    vuint32_t SL1_3_SR_SR:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL3_union_tag { /* CDTM[i]_DTM[d] channel control register 3 */
  vuint32_t R;
  struct {
    vuint32_t CII0:1;
    vuint32_t CIS0:1;
    vuint32_t TSEL0_0:1;
    vuint32_t TSEL1_0:1;
    vuint32_t _unused_4:4;
    vuint32_t CII1:1;
    vuint32_t CIS1:1;
    vuint32_t TSEL0_1:1;
    vuint32_t TSEL1_1:1;
    vuint32_t _unused_12:4;
    vuint32_t CII2:1;
    vuint32_t CIS2:1;
    vuint32_t TSEL0_2:1;
    vuint32_t TSEL1_2:1;
    vuint32_t _unused_20:4;
    vuint32_t CII3:1;
    vuint32_t CIS3:1;
    vuint32_t TSEL0_3:1;
    vuint32_t TSEL1_3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL3_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CTRL2_union_tag { /* CDTM[i]_DTM[d] global configuration and control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SHUTOFF_SEL_0:3;
    vuint32_t SHUTOFF_POL_0:1;
    vuint32_t UPD_MODE_0:2;
    vuint32_t SHUT_OFF_RST_0:1;
    vuint32_t WR_EN_0:1;
    vuint32_t SHUTOFF_SEL_1:3;
    vuint32_t SHUTOFF_POL_1:1;
    vuint32_t UPD_MODE_1:2;
    vuint32_t SHUT_OFF_RST_1:1;
    vuint32_t WR_EN_1:1;
    vuint32_t SHUTOFF_SEL_2:3;
    vuint32_t SHUTOFF_POL_2:1;
    vuint32_t UPD_MODE_2:2;
    vuint32_t SHUT_OFF_RST_2:1;
    vuint32_t WR_EN_2:1;
    vuint32_t SHUTOFF_SEL_3:3;
    vuint32_t SHUTOFF_POL_3:1;
    vuint32_t UPD_MODE_3:2;
    vuint32_t SHUT_OFF_RST_3:1;
    vuint32_t WR_EN_3:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CTRL2_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH0_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH0_DTV_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH1_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH1_DTV_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH2_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH2_DTV_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM4_CH3_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM4_CH3_DTV_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CTRL_union_tag { /* CDTM[i]_DTM[d] global configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t CLK_SEL:2;
    vuint32_t DTM_SEL:2;
    vuint32_t UPD_MODE:3;
    vuint32_t CH_SHUTOFF_EN:1;
    vuint32_t SR_UPD_EN:1;
    vuint32_t _unused_9:7;
    vuint32_t SHUT_OFF_RST:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CTRL_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL1_union_tag { /* CDTM[i]_DTM[d] channel control register 1 */
  vuint32_t R;
  struct {
    vuint32_t O1SEL_0:1;
    vuint32_t I1SEL_0:1;
    vuint32_t _unused_2:1;
    vuint32_t SWAP_0:1;
    vuint32_t O1F_0:2;
    vuint32_t XDT_EN_0_1:1;
    vuint32_t _unused_7:1;
    vuint32_t O1SEL_1:1;
    vuint32_t I1SEL_1:1;
    vuint32_t SH_EN_1:1;
    vuint32_t SWAP_1:1;
    vuint32_t O1F_1:2;
    vuint32_t _unused_14:2;
    vuint32_t O1SEL_2:1;
    vuint32_t I1SEL_2:1;
    vuint32_t SH_EN_2:1;
    vuint32_t SWAP_2:1;
    vuint32_t O1F_2:2;
    vuint32_t XDT_EN_2_3:1;
    vuint32_t _unused_23:1;
    vuint32_t O1SEL_3:1;
    vuint32_t I1SEL_3:1;
    vuint32_t SH_EN_3:1;
    vuint32_t SWAP_3:1;
    vuint32_t O1F_3:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL1_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL2_union_tag { /* CDTM[i]_DTM[d] channel control register 2 */
  vuint32_t R;
  struct {
    vuint32_t POL0_0:1;
    vuint32_t OC0_0:1;
    vuint32_t SL0_0:1;
    vuint32_t DT0_0:1;
    vuint32_t POL1_0:1;
    vuint32_t OC1_0:1;
    vuint32_t SL1_0:1;
    vuint32_t DT1_0:1;
    vuint32_t POL0_1:1;
    vuint32_t OC0_1:1;
    vuint32_t SL0_1:1;
    vuint32_t DT0_1:1;
    vuint32_t POL1_1:1;
    vuint32_t OC1_1:1;
    vuint32_t SL1_1:1;
    vuint32_t DT1_1:1;
    vuint32_t POL0_2:1;
    vuint32_t OC0_2:1;
    vuint32_t SL0_2:1;
    vuint32_t DT0_2:1;
    vuint32_t POL1_2:1;
    vuint32_t OC1_2:1;
    vuint32_t SL1_2:1;
    vuint32_t DT1_2:1;
    vuint32_t POL0_3:1;
    vuint32_t OC0_3:1;
    vuint32_t SL0_3:1;
    vuint32_t DT0_3:1;
    vuint32_t POL1_3:1;
    vuint32_t OC1_3:1;
    vuint32_t SL1_3:1;
    vuint32_t DT1_3:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL2_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL2_SR_union_tag { /* CDTM[i] DTM[j] channel control register 2 shadow */
  vuint32_t R;
  struct {
    vuint32_t POL0_0_SR:1;
    vuint32_t OC0_0_SR:1;
    vuint32_t SL0_0_SR:1;
    vuint32_t DT0_0_SR:1;
    vuint32_t POL1_0_SR:1;
    vuint32_t OC1_0_SR:1;
    vuint32_t SL1_0_SR:1;
    vuint32_t DT1_0_SR:1;
    vuint32_t POL0_1_SR:1;
    vuint32_t OC0_1_SR:1;
    vuint32_t SL0_1_SR:1;
    vuint32_t DT0_1_SR:1;
    vuint32_t POL1_1_SR:1;
    vuint32_t OC1_1_SR:1;
    vuint32_t SL1_1_SR:1;
    vuint32_t DT1_1_SR:1;
    vuint32_t POL0_2_SR:1;
    vuint32_t OC0_2_SR:1;
    vuint32_t SL0_2_SR:1;
    vuint32_t DT0_2_SR:1;
    vuint32_t POL1_2_SR:1;
    vuint32_t OC1_2_SR:1;
    vuint32_t SL1_2_SR:1;
    vuint32_t DT1_2_SR:1;
    vuint32_t POL0_3_SR:1;
    vuint32_t OC0_3_SR:1;
    vuint32_t SL0_3_SR:1;
    vuint32_t DT0_3_SR:1;
    vuint32_t POL1_3_SR:1;
    vuint32_t OC1_3_SR:1;
    vuint32_t SL1_3_SR:1;
    vuint32_t DT1_3_SR:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL2_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_PS_CTRL_union_tag { /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t RELBLK:10;
    vuint32_t _unused_10:6;
    vuint32_t PSU_IN_SEL:1;
    vuint32_t IN_POL:1;
    vuint32_t TIM_SEL:1;
    vuint32_t _unused_19:1;
    vuint32_t SHIFT_SEL:2;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_PS_CTRL_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH_DTV_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE:13;
    vuint32_t _unused_13:3;
    vuint32_t RELFALL:13;
    vuint32_t _unused_29:2;
    vuint32_t HRES:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH_DTV_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH_SR_union_tag { /* CDTM[i]_DTM[d] channel shadow register */
  vuint32_t R;
  struct {
    vuint32_t SL0_0_SR_SR:1;
    vuint32_t SL1_0_SR_SR:1;
    vuint32_t SL0_1_SR_SR:1;
    vuint32_t SL1_1_SR_SR:1;
    vuint32_t SL0_2_SR_SR:1;
    vuint32_t SL1_2_SR_SR:1;
    vuint32_t SL0_3_SR_SR:1;
    vuint32_t SL1_3_SR_SR:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL3_union_tag { /* CDTM[i]_DTM[d] channel control register 3 */
  vuint32_t R;
  struct {
    vuint32_t CII0:1;
    vuint32_t CIS0:1;
    vuint32_t TSEL0_0:1;
    vuint32_t TSEL1_0:1;
    vuint32_t _unused_4:4;
    vuint32_t CII1:1;
    vuint32_t CIS1:1;
    vuint32_t TSEL0_1:1;
    vuint32_t TSEL1_1:1;
    vuint32_t _unused_12:4;
    vuint32_t CII2:1;
    vuint32_t CIS2:1;
    vuint32_t TSEL0_2:1;
    vuint32_t TSEL1_2:1;
    vuint32_t _unused_20:4;
    vuint32_t CII3:1;
    vuint32_t CIS3:1;
    vuint32_t TSEL0_3:1;
    vuint32_t TSEL1_3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL3_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CTRL2_union_tag { /* CDTM[i]_DTM[d] global configuration and control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SHUTOFF_SEL_0:3;
    vuint32_t SHUTOFF_POL_0:1;
    vuint32_t UPD_MODE_0:2;
    vuint32_t SHUT_OFF_RST_0:1;
    vuint32_t WR_EN_0:1;
    vuint32_t SHUTOFF_SEL_1:3;
    vuint32_t SHUTOFF_POL_1:1;
    vuint32_t UPD_MODE_1:2;
    vuint32_t SHUT_OFF_RST_1:1;
    vuint32_t WR_EN_1:1;
    vuint32_t SHUTOFF_SEL_2:3;
    vuint32_t SHUTOFF_POL_2:1;
    vuint32_t UPD_MODE_2:2;
    vuint32_t SHUT_OFF_RST_2:1;
    vuint32_t WR_EN_2:1;
    vuint32_t SHUTOFF_SEL_3:3;
    vuint32_t SHUTOFF_POL_3:1;
    vuint32_t UPD_MODE_3:2;
    vuint32_t SHUT_OFF_RST_3:1;
    vuint32_t WR_EN_3:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CTRL2_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH0_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH0_DTV_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH1_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH1_DTV_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH2_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH2_DTV_SR_tag;

typedef union GTM_gtm_cls2_CDTM2_DTM5_CH3_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls2_CDTM2_DTM5_CH3_DTV_SR_tag;

typedef union GTM_gtm_cls2_AXIM2_FREE_union_tag { /* AXIM[i] slot allocation status. */
  vuint32_t R;
  struct {
    vuint32_t FREE0:1;
    vuint32_t FREE1:1;
    vuint32_t FREE2:1;
    vuint32_t FREE3:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls2_AXIM2_FREE_tag;

typedef union GTM_gtm_cls2_AXIM2_REQUEST_union_tag { /* AXIM[i] slot request (allocation). */
  vuint32_t R;
  struct {
    vuint32_t REQ1HOT0:1;
    vuint32_t REQ1HOT1:1;
    vuint32_t REQ1HOT2:1;
    vuint32_t REQ1HOT3:1;
    vuint32_t _unused_4:20;
    vuint32_t REQID:8;
  } B;
} GTM_gtm_cls2_AXIM2_REQUEST_tag;

typedef union GTM_gtm_cls2_AXIM2_RELEASE_union_tag { /* AXIM[i] slot release (de-allocation). */
  vuint32_t R;
  struct {
    vuint32_t RELREQ0:1;
    vuint32_t RELREQ1:1;
    vuint32_t RELREQ2:1;
    vuint32_t RELREQ3:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls2_AXIM2_RELEASE_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT0_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT0_ADDR_LOW_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT0_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT0_DATA_LOW_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT0_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT0_CFG1_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT0_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT0_CFG2_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT0_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT0_STATUS_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT1_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT1_ADDR_LOW_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT1_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT1_DATA_LOW_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT1_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT1_CFG1_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT1_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT1_CFG2_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT1_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT1_STATUS_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT2_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT2_ADDR_LOW_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT2_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT2_DATA_LOW_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT2_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT2_CFG1_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT2_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT2_CFG2_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT2_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT2_STATUS_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT3_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT3_ADDR_LOW_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT3_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT3_DATA_LOW_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT3_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT3_CFG1_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT3_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT3_CFG2_tag;

typedef union GTM_gtm_cls2_AXIM2_SLOT3_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls2_AXIM2_SLOT3_STATUS_tag;

typedef union GTM_gtm_cls2_MCS2_MEM_union_tag { /* MCS[i] memory region */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls2_MCS2_MEM_tag;

struct GTM_gtm_cls2_tag {
  uint8_t GTM_gtm_cls2_reserved0[2048];
  GTM_gtm_cls2_TIM2_CH0_GPR0_tag TIM2_CH0_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH0_GPR1_tag TIM2_CH0_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH0_CNT_tag TIM2_CH0_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls2_TIM2_CH0_ECNT_tag TIM2_CH0_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls2_TIM2_CH0_CNTS_tag TIM2_CH0_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls2_TIM2_CH0_TDUC_tag TIM2_CH0_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls2_TIM2_CH0_TDUV_tag TIM2_CH0_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls2_TIM2_CH0_FLT_RE_tag TIM2_CH0_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls2_TIM2_CH0_FLT_FE_tag TIM2_CH0_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls2_TIM2_CH0_CTRL_tag TIM2_CH0_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls2_TIM2_CH0_ECTRL_tag TIM2_CH0_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls2_TIM2_CH0_IRQ_NOTIFY_tag TIM2_CH0_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_TIM2_CH0_IRQ_EN_tag TIM2_CH0_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_TIM2_CH0_IRQ_FORCINT_tag TIM2_CH0_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls2_TIM2_CH0_IRQ_MODE_tag TIM2_CH0_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_TIM2_CH0_EIRQ_EN_tag TIM2_CH0_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved1[64];
  GTM_gtm_cls2_TIM2_CH1_GPR0_tag TIM2_CH1_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH1_GPR1_tag TIM2_CH1_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH1_CNT_tag TIM2_CH1_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls2_TIM2_CH1_ECNT_tag TIM2_CH1_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls2_TIM2_CH1_CNTS_tag TIM2_CH1_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls2_TIM2_CH1_TDUC_tag TIM2_CH1_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls2_TIM2_CH1_TDUV_tag TIM2_CH1_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls2_TIM2_CH1_FLT_RE_tag TIM2_CH1_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls2_TIM2_CH1_FLT_FE_tag TIM2_CH1_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls2_TIM2_CH1_CTRL_tag TIM2_CH1_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls2_TIM2_CH1_ECTRL_tag TIM2_CH1_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls2_TIM2_CH1_IRQ_NOTIFY_tag TIM2_CH1_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_TIM2_CH1_IRQ_EN_tag TIM2_CH1_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_TIM2_CH1_IRQ_FORCINT_tag TIM2_CH1_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls2_TIM2_CH1_IRQ_MODE_tag TIM2_CH1_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_TIM2_CH1_EIRQ_EN_tag TIM2_CH1_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved2[64];
  GTM_gtm_cls2_TIM2_CH2_GPR0_tag TIM2_CH2_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH2_GPR1_tag TIM2_CH2_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH2_CNT_tag TIM2_CH2_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls2_TIM2_CH2_ECNT_tag TIM2_CH2_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls2_TIM2_CH2_CNTS_tag TIM2_CH2_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls2_TIM2_CH2_TDUC_tag TIM2_CH2_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls2_TIM2_CH2_TDUV_tag TIM2_CH2_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls2_TIM2_CH2_FLT_RE_tag TIM2_CH2_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls2_TIM2_CH2_FLT_FE_tag TIM2_CH2_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls2_TIM2_CH2_CTRL_tag TIM2_CH2_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls2_TIM2_CH2_ECTRL_tag TIM2_CH2_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls2_TIM2_CH2_IRQ_NOTIFY_tag TIM2_CH2_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_TIM2_CH2_IRQ_EN_tag TIM2_CH2_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_TIM2_CH2_IRQ_FORCINT_tag TIM2_CH2_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls2_TIM2_CH2_IRQ_MODE_tag TIM2_CH2_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_TIM2_CH2_EIRQ_EN_tag TIM2_CH2_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved3[64];
  GTM_gtm_cls2_TIM2_CH3_GPR0_tag TIM2_CH3_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH3_GPR1_tag TIM2_CH3_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH3_CNT_tag TIM2_CH3_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls2_TIM2_CH3_ECNT_tag TIM2_CH3_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls2_TIM2_CH3_CNTS_tag TIM2_CH3_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls2_TIM2_CH3_TDUC_tag TIM2_CH3_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls2_TIM2_CH3_TDUV_tag TIM2_CH3_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls2_TIM2_CH3_FLT_RE_tag TIM2_CH3_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls2_TIM2_CH3_FLT_FE_tag TIM2_CH3_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls2_TIM2_CH3_CTRL_tag TIM2_CH3_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls2_TIM2_CH3_ECTRL_tag TIM2_CH3_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls2_TIM2_CH3_IRQ_NOTIFY_tag TIM2_CH3_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_TIM2_CH3_IRQ_EN_tag TIM2_CH3_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_TIM2_CH3_IRQ_FORCINT_tag TIM2_CH3_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls2_TIM2_CH3_IRQ_MODE_tag TIM2_CH3_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_TIM2_CH3_EIRQ_EN_tag TIM2_CH3_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved4[64];
  GTM_gtm_cls2_TIM2_CH4_GPR0_tag TIM2_CH4_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH4_GPR1_tag TIM2_CH4_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH4_CNT_tag TIM2_CH4_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls2_TIM2_CH4_ECNT_tag TIM2_CH4_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls2_TIM2_CH4_CNTS_tag TIM2_CH4_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls2_TIM2_CH4_TDUC_tag TIM2_CH4_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls2_TIM2_CH4_TDUV_tag TIM2_CH4_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls2_TIM2_CH4_FLT_RE_tag TIM2_CH4_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls2_TIM2_CH4_FLT_FE_tag TIM2_CH4_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls2_TIM2_CH4_CTRL_tag TIM2_CH4_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls2_TIM2_CH4_ECTRL_tag TIM2_CH4_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls2_TIM2_CH4_IRQ_NOTIFY_tag TIM2_CH4_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_TIM2_CH4_IRQ_EN_tag TIM2_CH4_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_TIM2_CH4_IRQ_FORCINT_tag TIM2_CH4_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls2_TIM2_CH4_IRQ_MODE_tag TIM2_CH4_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_TIM2_CH4_EIRQ_EN_tag TIM2_CH4_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved5[64];
  GTM_gtm_cls2_TIM2_CH5_GPR0_tag TIM2_CH5_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH5_GPR1_tag TIM2_CH5_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH5_CNT_tag TIM2_CH5_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls2_TIM2_CH5_ECNT_tag TIM2_CH5_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls2_TIM2_CH5_CNTS_tag TIM2_CH5_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls2_TIM2_CH5_TDUC_tag TIM2_CH5_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls2_TIM2_CH5_TDUV_tag TIM2_CH5_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls2_TIM2_CH5_FLT_RE_tag TIM2_CH5_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls2_TIM2_CH5_FLT_FE_tag TIM2_CH5_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls2_TIM2_CH5_CTRL_tag TIM2_CH5_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls2_TIM2_CH5_ECTRL_tag TIM2_CH5_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls2_TIM2_CH5_IRQ_NOTIFY_tag TIM2_CH5_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_TIM2_CH5_IRQ_EN_tag TIM2_CH5_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_TIM2_CH5_IRQ_FORCINT_tag TIM2_CH5_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls2_TIM2_CH5_IRQ_MODE_tag TIM2_CH5_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_TIM2_CH5_EIRQ_EN_tag TIM2_CH5_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved6[64];
  GTM_gtm_cls2_TIM2_CH6_GPR0_tag TIM2_CH6_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH6_GPR1_tag TIM2_CH6_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH6_CNT_tag TIM2_CH6_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls2_TIM2_CH6_ECNT_tag TIM2_CH6_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls2_TIM2_CH6_CNTS_tag TIM2_CH6_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls2_TIM2_CH6_TDUC_tag TIM2_CH6_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls2_TIM2_CH6_TDUV_tag TIM2_CH6_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls2_TIM2_CH6_FLT_RE_tag TIM2_CH6_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls2_TIM2_CH6_FLT_FE_tag TIM2_CH6_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls2_TIM2_CH6_CTRL_tag TIM2_CH6_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls2_TIM2_CH6_ECTRL_tag TIM2_CH6_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls2_TIM2_CH6_IRQ_NOTIFY_tag TIM2_CH6_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_TIM2_CH6_IRQ_EN_tag TIM2_CH6_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_TIM2_CH6_IRQ_FORCINT_tag TIM2_CH6_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls2_TIM2_CH6_IRQ_MODE_tag TIM2_CH6_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_TIM2_CH6_EIRQ_EN_tag TIM2_CH6_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved7[64];
  GTM_gtm_cls2_TIM2_CH7_GPR0_tag TIM2_CH7_GPR0; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH7_GPR1_tag TIM2_CH7_GPR1; /* TIM[i] channel [x] general purpose 0 register */
  GTM_gtm_cls2_TIM2_CH7_CNT_tag TIM2_CH7_CNT; /* TIM[i] channel [x] SMU counter register */
  GTM_gtm_cls2_TIM2_CH7_ECNT_tag TIM2_CH7_ECNT; /* TIM[i] channel [x] SMU edge counter register */
  GTM_gtm_cls2_TIM2_CH7_CNTS_tag TIM2_CH7_CNTS; /* TIM[i] channel [x] SMU shadow counter register */
  GTM_gtm_cls2_TIM2_CH7_TDUC_tag TIM2_CH7_TDUC; /* TIM[i] channel [x] TDU counter register */
  GTM_gtm_cls2_TIM2_CH7_TDUV_tag TIM2_CH7_TDUV; /* TIM[i] channel [x] TDU control register */
  GTM_gtm_cls2_TIM2_CH7_FLT_RE_tag TIM2_CH7_FLT_RE; /* TIM[i] channel [x] filter parameter 0 register */
  GTM_gtm_cls2_TIM2_CH7_FLT_FE_tag TIM2_CH7_FLT_FE; /* TIM[i] channel [x] filter parameter 1 register */
  GTM_gtm_cls2_TIM2_CH7_CTRL_tag TIM2_CH7_CTRL; /* TIM[i] channel [x] control register */
  GTM_gtm_cls2_TIM2_CH7_ECTRL_tag TIM2_CH7_ECTRL; /* TIM[i] channel [x] extended control register */
  GTM_gtm_cls2_TIM2_CH7_IRQ_NOTIFY_tag TIM2_CH7_IRQ_NOTIFY; /* TIM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_TIM2_CH7_IRQ_EN_tag TIM2_CH7_IRQ_EN; /* TIM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_TIM2_CH7_IRQ_FORCINT_tag TIM2_CH7_IRQ_FORCINT; /* TIM[i] channel [x] force interrupt register */
  GTM_gtm_cls2_TIM2_CH7_IRQ_MODE_tag TIM2_CH7_IRQ_MODE; /* TIM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_TIM2_CH7_EIRQ_EN_tag TIM2_CH7_EIRQ_EN; /* TIM[i] channel [x] error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved8[64];
  GTM_gtm_cls2_TIM2_INP_VAL_tag TIM2_INP_VAL; /* TIM[i] input value observation register */
  GTM_gtm_cls2_TIM2_IN_SRC_tag TIM2_IN_SRC; /* TIM[i] AUX IN source selection register */
  GTM_gtm_cls2_TIM2_RST_tag TIM2_RST;  /* TIM[i] global software reset register */
  uint8_t GTM_gtm_cls2_reserved9[3060];
  GTM_gtm_cls2_ATOM2_CH0_RDADDR_tag ATOM2_CH0_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls2_ATOM2_CH0_CTRL_tag ATOM2_CH0_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls2_ATOM2_CH0_SR0_tag ATOM2_CH0_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH0_SR1_tag ATOM2_CH0_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH0_CM0_tag ATOM2_CH0_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH0_CM1_tag ATOM2_CH0_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH0_CN0_tag ATOM2_CH0_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls2_ATOM2_CH0_STAT_tag ATOM2_CH0_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls2_ATOM2_CH0_IRQ_NOTIFY_tag ATOM2_CH0_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_ATOM2_CH0_IRQ_EN_tag ATOM2_CH0_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_ATOM2_CH0_IRQ_FORCINT_tag ATOM2_CH0_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls2_ATOM2_CH0_IRQ_MODE_tag ATOM2_CH0_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_ATOM2_CH0_CTRL2_tag ATOM2_CH0_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls2_ATOM2_CH0_CTRL_SR_tag ATOM2_CH0_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls2_reserved10[72];
  GTM_gtm_cls2_ATOM2_CH1_RDADDR_tag ATOM2_CH1_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls2_ATOM2_CH1_CTRL_tag ATOM2_CH1_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls2_ATOM2_CH1_SR0_tag ATOM2_CH1_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH1_SR1_tag ATOM2_CH1_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH1_CM0_tag ATOM2_CH1_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH1_CM1_tag ATOM2_CH1_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH1_CN0_tag ATOM2_CH1_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls2_ATOM2_CH1_STAT_tag ATOM2_CH1_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls2_ATOM2_CH1_IRQ_NOTIFY_tag ATOM2_CH1_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_ATOM2_CH1_IRQ_EN_tag ATOM2_CH1_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_ATOM2_CH1_IRQ_FORCINT_tag ATOM2_CH1_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls2_ATOM2_CH1_IRQ_MODE_tag ATOM2_CH1_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_ATOM2_CH1_CTRL2_tag ATOM2_CH1_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls2_ATOM2_CH1_CTRL_SR_tag ATOM2_CH1_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls2_reserved11[72];
  GTM_gtm_cls2_ATOM2_CH2_RDADDR_tag ATOM2_CH2_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls2_ATOM2_CH2_CTRL_tag ATOM2_CH2_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls2_ATOM2_CH2_SR0_tag ATOM2_CH2_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH2_SR1_tag ATOM2_CH2_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH2_CM0_tag ATOM2_CH2_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH2_CM1_tag ATOM2_CH2_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH2_CN0_tag ATOM2_CH2_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls2_ATOM2_CH2_STAT_tag ATOM2_CH2_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls2_ATOM2_CH2_IRQ_NOTIFY_tag ATOM2_CH2_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_ATOM2_CH2_IRQ_EN_tag ATOM2_CH2_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_ATOM2_CH2_IRQ_FORCINT_tag ATOM2_CH2_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls2_ATOM2_CH2_IRQ_MODE_tag ATOM2_CH2_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_ATOM2_CH2_CTRL2_tag ATOM2_CH2_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls2_ATOM2_CH2_CTRL_SR_tag ATOM2_CH2_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls2_reserved12[72];
  GTM_gtm_cls2_ATOM2_CH3_RDADDR_tag ATOM2_CH3_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls2_ATOM2_CH3_CTRL_tag ATOM2_CH3_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls2_ATOM2_CH3_SR0_tag ATOM2_CH3_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH3_SR1_tag ATOM2_CH3_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH3_CM0_tag ATOM2_CH3_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH3_CM1_tag ATOM2_CH3_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH3_CN0_tag ATOM2_CH3_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls2_ATOM2_CH3_STAT_tag ATOM2_CH3_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls2_ATOM2_CH3_IRQ_NOTIFY_tag ATOM2_CH3_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_ATOM2_CH3_IRQ_EN_tag ATOM2_CH3_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_ATOM2_CH3_IRQ_FORCINT_tag ATOM2_CH3_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls2_ATOM2_CH3_IRQ_MODE_tag ATOM2_CH3_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_ATOM2_CH3_CTRL2_tag ATOM2_CH3_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls2_ATOM2_CH3_CTRL_SR_tag ATOM2_CH3_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls2_reserved13[72];
  GTM_gtm_cls2_ATOM2_CH4_RDADDR_tag ATOM2_CH4_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls2_ATOM2_CH4_CTRL_tag ATOM2_CH4_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls2_ATOM2_CH4_SR0_tag ATOM2_CH4_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH4_SR1_tag ATOM2_CH4_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH4_CM0_tag ATOM2_CH4_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH4_CM1_tag ATOM2_CH4_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH4_CN0_tag ATOM2_CH4_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls2_ATOM2_CH4_STAT_tag ATOM2_CH4_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls2_ATOM2_CH4_IRQ_NOTIFY_tag ATOM2_CH4_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_ATOM2_CH4_IRQ_EN_tag ATOM2_CH4_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_ATOM2_CH4_IRQ_FORCINT_tag ATOM2_CH4_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls2_ATOM2_CH4_IRQ_MODE_tag ATOM2_CH4_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_ATOM2_CH4_CTRL2_tag ATOM2_CH4_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls2_ATOM2_CH4_CTRL_SR_tag ATOM2_CH4_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls2_reserved14[72];
  GTM_gtm_cls2_ATOM2_CH5_RDADDR_tag ATOM2_CH5_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls2_ATOM2_CH5_CTRL_tag ATOM2_CH5_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls2_ATOM2_CH5_SR0_tag ATOM2_CH5_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH5_SR1_tag ATOM2_CH5_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH5_CM0_tag ATOM2_CH5_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH5_CM1_tag ATOM2_CH5_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH5_CN0_tag ATOM2_CH5_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls2_ATOM2_CH5_STAT_tag ATOM2_CH5_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls2_ATOM2_CH5_IRQ_NOTIFY_tag ATOM2_CH5_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_ATOM2_CH5_IRQ_EN_tag ATOM2_CH5_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_ATOM2_CH5_IRQ_FORCINT_tag ATOM2_CH5_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls2_ATOM2_CH5_IRQ_MODE_tag ATOM2_CH5_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_ATOM2_CH5_CTRL2_tag ATOM2_CH5_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls2_ATOM2_CH5_CTRL_SR_tag ATOM2_CH5_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls2_reserved15[72];
  GTM_gtm_cls2_ATOM2_CH6_RDADDR_tag ATOM2_CH6_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls2_ATOM2_CH6_CTRL_tag ATOM2_CH6_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls2_ATOM2_CH6_SR0_tag ATOM2_CH6_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH6_SR1_tag ATOM2_CH6_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH6_CM0_tag ATOM2_CH6_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH6_CM1_tag ATOM2_CH6_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH6_CN0_tag ATOM2_CH6_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls2_ATOM2_CH6_STAT_tag ATOM2_CH6_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls2_ATOM2_CH6_IRQ_NOTIFY_tag ATOM2_CH6_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_ATOM2_CH6_IRQ_EN_tag ATOM2_CH6_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_ATOM2_CH6_IRQ_FORCINT_tag ATOM2_CH6_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls2_ATOM2_CH6_IRQ_MODE_tag ATOM2_CH6_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_ATOM2_CH6_CTRL2_tag ATOM2_CH6_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls2_ATOM2_CH6_CTRL_SR_tag ATOM2_CH6_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls2_reserved16[72];
  GTM_gtm_cls2_ATOM2_CH7_RDADDR_tag ATOM2_CH7_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls2_ATOM2_CH7_CTRL_tag ATOM2_CH7_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls2_ATOM2_CH7_SR0_tag ATOM2_CH7_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH7_SR1_tag ATOM2_CH7_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls2_ATOM2_CH7_CM0_tag ATOM2_CH7_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH7_CM1_tag ATOM2_CH7_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls2_ATOM2_CH7_CN0_tag ATOM2_CH7_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls2_ATOM2_CH7_STAT_tag ATOM2_CH7_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls2_ATOM2_CH7_IRQ_NOTIFY_tag ATOM2_CH7_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls2_ATOM2_CH7_IRQ_EN_tag ATOM2_CH7_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls2_ATOM2_CH7_IRQ_FORCINT_tag ATOM2_CH7_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls2_ATOM2_CH7_IRQ_MODE_tag ATOM2_CH7_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  GTM_gtm_cls2_ATOM2_CH7_CTRL2_tag ATOM2_CH7_CTRL2; /* ATOM[i] channel [x] control2 register */
  GTM_gtm_cls2_ATOM2_CH7_CTRL_SR_tag ATOM2_CH7_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls2_reserved17[136];
  GTM_gtm_cls2_ATOM2_AGC_GLB_CTRL_tag ATOM2_AGC_GLB_CTRL; /* ATOM[i] AGC global control register */
  GTM_gtm_cls2_ATOM2_AGC_ENDIS_CTRL_tag ATOM2_AGC_ENDIS_CTRL; /* ATOM[i] AGC enable/disable control register */
  GTM_gtm_cls2_ATOM2_AGC_ENDIS_STAT_tag ATOM2_AGC_ENDIS_STAT; /* ATOM[i] AGC enable/disable status register */
  GTM_gtm_cls2_ATOM2_AGC_ACT_TB_tag ATOM2_AGC_ACT_TB; /* ATOM[i] AGC action time base register */
  GTM_gtm_cls2_ATOM2_AGC_OUTEN_CTRL_tag ATOM2_AGC_OUTEN_CTRL; /* ATOM[i] AGC output enable control register */
  GTM_gtm_cls2_ATOM2_AGC_OUTEN_STAT_tag ATOM2_AGC_OUTEN_STAT; /* ATOM[i] AGC output enable status register */
  GTM_gtm_cls2_ATOM2_AGC_FUPD_CTRL_tag ATOM2_AGC_FUPD_CTRL; /* ATOM[i] AGC force update control register */
  GTM_gtm_cls2_ATOM2_AGC_INT_TRIG_tag ATOM2_AGC_INT_TRIG; /* ATOM[i] AGC internal trigger control register */
  uint8_t GTM_gtm_cls2_reserved18[928];
  GTM_gtm_cls2_MCS2_CH0_R0_tag MCS2_CH0_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH0_R1_tag MCS2_CH0_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH0_R2_tag MCS2_CH0_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH0_R3_tag MCS2_CH0_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH0_R4_tag MCS2_CH0_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH0_R5_tag MCS2_CH0_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH0_R6_tag MCS2_CH0_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH0_R7_tag MCS2_CH0_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH0_CTRL_tag MCS2_CH0_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls2_MCS2_CH0_ACB_tag MCS2_CH0_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls2_reserved19[20];
  GTM_gtm_cls2_MCS2_CH0_MHB_tag MCS2_CH0_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls2_reserved20[160];
  GTM_gtm_cls2_MCS2_CH0_PC_tag MCS2_CH0_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls2_MCS2_CH0_IRQ_NOTIFY_tag MCS2_CH0_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls2_MCS2_CH0_IRQ_EN_tag MCS2_CH0_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls2_MCS2_CH0_IRQ_FORCINT_tag MCS2_CH0_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls2_MCS2_CH0_IRQ_MODE_tag MCS2_CH0_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls2_MCS2_CH0_EIRQ_EN_tag MCS2_CH0_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved21[8];
  GTM_gtm_cls2_MCS2_CH1_R0_tag MCS2_CH1_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH1_R1_tag MCS2_CH1_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH1_R2_tag MCS2_CH1_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH1_R3_tag MCS2_CH1_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH1_R4_tag MCS2_CH1_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH1_R5_tag MCS2_CH1_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH1_R6_tag MCS2_CH1_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH1_R7_tag MCS2_CH1_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH1_CTRL_tag MCS2_CH1_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls2_MCS2_CH1_ACB_tag MCS2_CH1_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls2_reserved22[20];
  GTM_gtm_cls2_MCS2_CH1_MHB_tag MCS2_CH1_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls2_reserved23[160];
  GTM_gtm_cls2_MCS2_CH1_PC_tag MCS2_CH1_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls2_MCS2_CH1_IRQ_NOTIFY_tag MCS2_CH1_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls2_MCS2_CH1_IRQ_EN_tag MCS2_CH1_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls2_MCS2_CH1_IRQ_FORCINT_tag MCS2_CH1_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls2_MCS2_CH1_IRQ_MODE_tag MCS2_CH1_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls2_MCS2_CH1_EIRQ_EN_tag MCS2_CH1_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved24[8];
  GTM_gtm_cls2_MCS2_CH2_R0_tag MCS2_CH2_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH2_R1_tag MCS2_CH2_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH2_R2_tag MCS2_CH2_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH2_R3_tag MCS2_CH2_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH2_R4_tag MCS2_CH2_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH2_R5_tag MCS2_CH2_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH2_R6_tag MCS2_CH2_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH2_R7_tag MCS2_CH2_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH2_CTRL_tag MCS2_CH2_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls2_MCS2_CH2_ACB_tag MCS2_CH2_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls2_reserved25[20];
  GTM_gtm_cls2_MCS2_CH2_MHB_tag MCS2_CH2_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls2_reserved26[160];
  GTM_gtm_cls2_MCS2_CH2_PC_tag MCS2_CH2_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls2_MCS2_CH2_IRQ_NOTIFY_tag MCS2_CH2_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls2_MCS2_CH2_IRQ_EN_tag MCS2_CH2_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls2_MCS2_CH2_IRQ_FORCINT_tag MCS2_CH2_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls2_MCS2_CH2_IRQ_MODE_tag MCS2_CH2_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls2_MCS2_CH2_EIRQ_EN_tag MCS2_CH2_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved27[8];
  GTM_gtm_cls2_MCS2_CH3_R0_tag MCS2_CH3_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH3_R1_tag MCS2_CH3_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH3_R2_tag MCS2_CH3_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH3_R3_tag MCS2_CH3_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH3_R4_tag MCS2_CH3_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH3_R5_tag MCS2_CH3_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH3_R6_tag MCS2_CH3_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH3_R7_tag MCS2_CH3_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH3_CTRL_tag MCS2_CH3_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls2_MCS2_CH3_ACB_tag MCS2_CH3_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls2_reserved28[20];
  GTM_gtm_cls2_MCS2_CH3_MHB_tag MCS2_CH3_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls2_reserved29[160];
  GTM_gtm_cls2_MCS2_CH3_PC_tag MCS2_CH3_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls2_MCS2_CH3_IRQ_NOTIFY_tag MCS2_CH3_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls2_MCS2_CH3_IRQ_EN_tag MCS2_CH3_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls2_MCS2_CH3_IRQ_FORCINT_tag MCS2_CH3_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls2_MCS2_CH3_IRQ_MODE_tag MCS2_CH3_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls2_MCS2_CH3_EIRQ_EN_tag MCS2_CH3_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved30[8];
  GTM_gtm_cls2_MCS2_CH4_R0_tag MCS2_CH4_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH4_R1_tag MCS2_CH4_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH4_R2_tag MCS2_CH4_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH4_R3_tag MCS2_CH4_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH4_R4_tag MCS2_CH4_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH4_R5_tag MCS2_CH4_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH4_R6_tag MCS2_CH4_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH4_R7_tag MCS2_CH4_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH4_CTRL_tag MCS2_CH4_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls2_MCS2_CH4_ACB_tag MCS2_CH4_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls2_reserved31[20];
  GTM_gtm_cls2_MCS2_CH4_MHB_tag MCS2_CH4_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls2_reserved32[160];
  GTM_gtm_cls2_MCS2_CH4_PC_tag MCS2_CH4_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls2_MCS2_CH4_IRQ_NOTIFY_tag MCS2_CH4_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls2_MCS2_CH4_IRQ_EN_tag MCS2_CH4_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls2_MCS2_CH4_IRQ_FORCINT_tag MCS2_CH4_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls2_MCS2_CH4_IRQ_MODE_tag MCS2_CH4_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls2_MCS2_CH4_EIRQ_EN_tag MCS2_CH4_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved33[8];
  GTM_gtm_cls2_MCS2_CH5_R0_tag MCS2_CH5_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH5_R1_tag MCS2_CH5_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH5_R2_tag MCS2_CH5_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH5_R3_tag MCS2_CH5_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH5_R4_tag MCS2_CH5_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH5_R5_tag MCS2_CH5_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH5_R6_tag MCS2_CH5_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH5_R7_tag MCS2_CH5_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH5_CTRL_tag MCS2_CH5_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls2_MCS2_CH5_ACB_tag MCS2_CH5_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls2_reserved34[20];
  GTM_gtm_cls2_MCS2_CH5_MHB_tag MCS2_CH5_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls2_reserved35[160];
  GTM_gtm_cls2_MCS2_CH5_PC_tag MCS2_CH5_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls2_MCS2_CH5_IRQ_NOTIFY_tag MCS2_CH5_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls2_MCS2_CH5_IRQ_EN_tag MCS2_CH5_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls2_MCS2_CH5_IRQ_FORCINT_tag MCS2_CH5_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls2_MCS2_CH5_IRQ_MODE_tag MCS2_CH5_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls2_MCS2_CH5_EIRQ_EN_tag MCS2_CH5_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved36[8];
  GTM_gtm_cls2_MCS2_CH6_R0_tag MCS2_CH6_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH6_R1_tag MCS2_CH6_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH6_R2_tag MCS2_CH6_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH6_R3_tag MCS2_CH6_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH6_R4_tag MCS2_CH6_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH6_R5_tag MCS2_CH6_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH6_R6_tag MCS2_CH6_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH6_R7_tag MCS2_CH6_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH6_CTRL_tag MCS2_CH6_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls2_MCS2_CH6_ACB_tag MCS2_CH6_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls2_reserved37[20];
  GTM_gtm_cls2_MCS2_CH6_MHB_tag MCS2_CH6_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls2_reserved38[160];
  GTM_gtm_cls2_MCS2_CH6_PC_tag MCS2_CH6_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls2_MCS2_CH6_IRQ_NOTIFY_tag MCS2_CH6_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls2_MCS2_CH6_IRQ_EN_tag MCS2_CH6_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls2_MCS2_CH6_IRQ_FORCINT_tag MCS2_CH6_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls2_MCS2_CH6_IRQ_MODE_tag MCS2_CH6_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls2_MCS2_CH6_EIRQ_EN_tag MCS2_CH6_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved39[8];
  GTM_gtm_cls2_MCS2_CH7_R0_tag MCS2_CH7_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH7_R1_tag MCS2_CH7_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH7_R2_tag MCS2_CH7_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH7_R3_tag MCS2_CH7_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH7_R4_tag MCS2_CH7_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH7_R5_tag MCS2_CH7_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH7_R6_tag MCS2_CH7_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH7_R7_tag MCS2_CH7_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls2_MCS2_CH7_CTRL_tag MCS2_CH7_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls2_MCS2_CH7_ACB_tag MCS2_CH7_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls2_reserved40[20];
  GTM_gtm_cls2_MCS2_CH7_MHB_tag MCS2_CH7_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls2_reserved41[160];
  GTM_gtm_cls2_MCS2_CH7_PC_tag MCS2_CH7_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls2_MCS2_CH7_IRQ_NOTIFY_tag MCS2_CH7_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls2_MCS2_CH7_IRQ_EN_tag MCS2_CH7_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls2_MCS2_CH7_IRQ_FORCINT_tag MCS2_CH7_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls2_MCS2_CH7_IRQ_MODE_tag MCS2_CH7_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls2_MCS2_CH7_EIRQ_EN_tag MCS2_CH7_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls2_reserved42[1584];
  GTM_gtm_cls2_MCS2_CTRG_tag MCS2_CTRG; /* MCS[i] clear trigger control register */
  GTM_gtm_cls2_MCS2_STRG_tag MCS2_STRG; /* MCS[i] set trigger control register */
  uint8_t GTM_gtm_cls2_reserved43[208];
  GTM_gtm_cls2_MCS2_CTRL_STAT_tag MCS2_CTRL_STAT; /* MCS[i] control and status register */
  GTM_gtm_cls2_MCS2_RESET_tag MCS2_RESET; /* MCS[i] reset register */
  GTM_gtm_cls2_MCS2_CAT_tag MCS2_CAT;  /* MCS[i] cancel ARU transfer instruction */
  GTM_gtm_cls2_MCS2_CWT_tag MCS2_CWT;  /* MCS[i] cancel waiting instruction */
  GTM_gtm_cls2_MCS2_ERR_tag MCS2_ERR;  /* MCS[i] error register */
  uint8_t GTM_gtm_cls2_reserved44[8];
  GTM_gtm_cls2_MCS2_REG_PROT_tag MCS2_REG_PROT; /* MCS[i] write protection register */
  GTM_gtm_cls2_MCS2_SINT_IRQ_NOTIFY_tag MCS2_SINT_IRQ_NOTIFY; /* MCS[i] shared interrupt notification register */
  GTM_gtm_cls2_MCS2_SINT_IRQ_EN_tag MCS2_SINT_IRQ_EN; /* MCS[i] shared interrupt enable register */
  GTM_gtm_cls2_MCS2_SINT_IRQ_FORCINT_tag MCS2_SINT_IRQ_FORCINT; /* MCS[i] force shared interrupt register */
  GTM_gtm_cls2_MCS2_SINT_IRQ_MODE_tag MCS2_SINT_IRQ_MODE; /* MCS[i] shared interrupt mode configuration register */
  uint8_t GTM_gtm_cls2_reserved45[16];
  GTM_gtm_cls2_MCS2_HBP0_CTRL_tag MCS2_HBP0_CTRL; /* MCS[i] hardware break point h control register */
  GTM_gtm_cls2_MCS2_HBP0_PATTERN_tag MCS2_HBP0_PATTERN; /* MCS[i] hardware break point pattern register */
  GTM_gtm_cls2_MCS2_HBP0_STATUS_tag MCS2_HBP0_STATUS; /* MCS[i] hardware break point status register */
  GTM_gtm_cls2_MCS2_HBP0_IRQ_NOTIFY_tag MCS2_HBP0_IRQ_NOTIFY; /* MCS[i] hardware break point interrupt notification register */
  GTM_gtm_cls2_MCS2_HBP0_IRQ_EN_tag MCS2_HBP0_IRQ_EN; /* MCS[i] hardware break point interrupt enable register */
  GTM_gtm_cls2_MCS2_HBP0_IRQ_FORCINT_tag MCS2_HBP0_IRQ_FORCINT; /* MCS[i] force hardware break point interrupt register */
  GTM_gtm_cls2_MCS2_HBP0_IRQ_MODE_tag MCS2_HBP0_IRQ_MODE; /* MCS[i] break point h interrupt mode configuration register */
  uint8_t GTM_gtm_cls2_reserved46[4];
  GTM_gtm_cls2_MCS2_HBP1_CTRL_tag MCS2_HBP1_CTRL; /* MCS[i] hardware break point h control register */
  GTM_gtm_cls2_MCS2_HBP1_PATTERN_tag MCS2_HBP1_PATTERN; /* MCS[i] hardware break point pattern register */
  GTM_gtm_cls2_MCS2_HBP1_STATUS_tag MCS2_HBP1_STATUS; /* MCS[i] hardware break point status register */
  GTM_gtm_cls2_MCS2_HBP1_IRQ_NOTIFY_tag MCS2_HBP1_IRQ_NOTIFY; /* MCS[i] hardware break point interrupt notification register */
  GTM_gtm_cls2_MCS2_HBP1_IRQ_EN_tag MCS2_HBP1_IRQ_EN; /* MCS[i] hardware break point interrupt enable register */
  GTM_gtm_cls2_MCS2_HBP1_IRQ_FORCINT_tag MCS2_HBP1_IRQ_FORCINT; /* MCS[i] force hardware break point interrupt register */
  GTM_gtm_cls2_MCS2_HBP1_IRQ_MODE_tag MCS2_HBP1_IRQ_MODE; /* MCS[i] break point h interrupt mode configuration register */
  uint8_t GTM_gtm_cls2_reserved47[132];
  GTM_gtm_cls2_TIO2_G0_CH0_CTRL_tag TIO2_G0_CH0_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls2_TIO2_G0_CH0_IRQ_NOTIFY_tag TIO2_G0_CH0_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls2_TIO2_G0_CH0_IRQ_EN_tag TIO2_G0_CH0_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls2_TIO2_G0_CH0_IRQ_FORCINT_tag TIO2_G0_CH0_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls2_TIO2_G0_CH0_IRQ_MODE_tag TIO2_G0_CH0_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls2_TIO2_G0_CH0_CTRL2_tag TIO2_G0_CH0_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls2_reserved48[8];
  GTM_gtm_cls2_TIO2_G0_CH0_SINST_tag TIO2_G0_CH0_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH0_SCMD_tag TIO2_G0_CH0_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH0_SOP_tag TIO2_G0_CH0_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls2_reserved49[4];
  GTM_gtm_cls2_TIO2_G0_CH0_OINST_tag TIO2_G0_CH0_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls2_TIO2_G0_CH0_OCMD_tag TIO2_G0_CH0_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls2_TIO2_G0_CH0_OOP_tag TIO2_G0_CH0_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls2_TIO2_G0_CH0_SHIFTCNT_tag TIO2_G0_CH0_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls2_TIO2_G0_CH1_CTRL_tag TIO2_G0_CH1_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls2_TIO2_G0_CH1_IRQ_NOTIFY_tag TIO2_G0_CH1_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls2_TIO2_G0_CH1_IRQ_EN_tag TIO2_G0_CH1_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls2_TIO2_G0_CH1_IRQ_FORCINT_tag TIO2_G0_CH1_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls2_TIO2_G0_CH1_IRQ_MODE_tag TIO2_G0_CH1_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls2_TIO2_G0_CH1_CTRL2_tag TIO2_G0_CH1_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls2_reserved50[8];
  GTM_gtm_cls2_TIO2_G0_CH1_SINST_tag TIO2_G0_CH1_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH1_SCMD_tag TIO2_G0_CH1_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH1_SOP_tag TIO2_G0_CH1_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls2_reserved51[4];
  GTM_gtm_cls2_TIO2_G0_CH1_OINST_tag TIO2_G0_CH1_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls2_TIO2_G0_CH1_OCMD_tag TIO2_G0_CH1_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls2_TIO2_G0_CH1_OOP_tag TIO2_G0_CH1_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls2_TIO2_G0_CH1_SHIFTCNT_tag TIO2_G0_CH1_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls2_TIO2_G0_CH2_CTRL_tag TIO2_G0_CH2_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls2_TIO2_G0_CH2_IRQ_NOTIFY_tag TIO2_G0_CH2_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls2_TIO2_G0_CH2_IRQ_EN_tag TIO2_G0_CH2_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls2_TIO2_G0_CH2_IRQ_FORCINT_tag TIO2_G0_CH2_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls2_TIO2_G0_CH2_IRQ_MODE_tag TIO2_G0_CH2_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls2_TIO2_G0_CH2_CTRL2_tag TIO2_G0_CH2_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls2_reserved52[8];
  GTM_gtm_cls2_TIO2_G0_CH2_SINST_tag TIO2_G0_CH2_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH2_SCMD_tag TIO2_G0_CH2_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH2_SOP_tag TIO2_G0_CH2_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls2_reserved53[4];
  GTM_gtm_cls2_TIO2_G0_CH2_OINST_tag TIO2_G0_CH2_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls2_TIO2_G0_CH2_OCMD_tag TIO2_G0_CH2_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls2_TIO2_G0_CH2_OOP_tag TIO2_G0_CH2_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls2_TIO2_G0_CH2_SHIFTCNT_tag TIO2_G0_CH2_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls2_TIO2_G0_CH3_CTRL_tag TIO2_G0_CH3_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls2_TIO2_G0_CH3_IRQ_NOTIFY_tag TIO2_G0_CH3_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls2_TIO2_G0_CH3_IRQ_EN_tag TIO2_G0_CH3_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls2_TIO2_G0_CH3_IRQ_FORCINT_tag TIO2_G0_CH3_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls2_TIO2_G0_CH3_IRQ_MODE_tag TIO2_G0_CH3_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls2_TIO2_G0_CH3_CTRL2_tag TIO2_G0_CH3_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls2_reserved54[8];
  GTM_gtm_cls2_TIO2_G0_CH3_SINST_tag TIO2_G0_CH3_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH3_SCMD_tag TIO2_G0_CH3_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH3_SOP_tag TIO2_G0_CH3_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls2_reserved55[4];
  GTM_gtm_cls2_TIO2_G0_CH3_OINST_tag TIO2_G0_CH3_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls2_TIO2_G0_CH3_OCMD_tag TIO2_G0_CH3_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls2_TIO2_G0_CH3_OOP_tag TIO2_G0_CH3_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls2_TIO2_G0_CH3_SHIFTCNT_tag TIO2_G0_CH3_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls2_TIO2_G0_CH4_CTRL_tag TIO2_G0_CH4_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls2_TIO2_G0_CH4_IRQ_NOTIFY_tag TIO2_G0_CH4_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls2_TIO2_G0_CH4_IRQ_EN_tag TIO2_G0_CH4_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls2_TIO2_G0_CH4_IRQ_FORCINT_tag TIO2_G0_CH4_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls2_TIO2_G0_CH4_IRQ_MODE_tag TIO2_G0_CH4_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls2_TIO2_G0_CH4_CTRL2_tag TIO2_G0_CH4_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls2_reserved56[8];
  GTM_gtm_cls2_TIO2_G0_CH4_SINST_tag TIO2_G0_CH4_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH4_SCMD_tag TIO2_G0_CH4_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH4_SOP_tag TIO2_G0_CH4_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls2_reserved57[4];
  GTM_gtm_cls2_TIO2_G0_CH4_OINST_tag TIO2_G0_CH4_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls2_TIO2_G0_CH4_OCMD_tag TIO2_G0_CH4_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls2_TIO2_G0_CH4_OOP_tag TIO2_G0_CH4_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls2_TIO2_G0_CH4_SHIFTCNT_tag TIO2_G0_CH4_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls2_TIO2_G0_CH5_CTRL_tag TIO2_G0_CH5_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls2_TIO2_G0_CH5_IRQ_NOTIFY_tag TIO2_G0_CH5_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls2_TIO2_G0_CH5_IRQ_EN_tag TIO2_G0_CH5_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls2_TIO2_G0_CH5_IRQ_FORCINT_tag TIO2_G0_CH5_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls2_TIO2_G0_CH5_IRQ_MODE_tag TIO2_G0_CH5_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls2_TIO2_G0_CH5_CTRL2_tag TIO2_G0_CH5_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls2_reserved58[8];
  GTM_gtm_cls2_TIO2_G0_CH5_SINST_tag TIO2_G0_CH5_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH5_SCMD_tag TIO2_G0_CH5_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH5_SOP_tag TIO2_G0_CH5_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls2_reserved59[4];
  GTM_gtm_cls2_TIO2_G0_CH5_OINST_tag TIO2_G0_CH5_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls2_TIO2_G0_CH5_OCMD_tag TIO2_G0_CH5_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls2_TIO2_G0_CH5_OOP_tag TIO2_G0_CH5_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls2_TIO2_G0_CH5_SHIFTCNT_tag TIO2_G0_CH5_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls2_TIO2_G0_CH6_CTRL_tag TIO2_G0_CH6_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls2_TIO2_G0_CH6_IRQ_NOTIFY_tag TIO2_G0_CH6_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls2_TIO2_G0_CH6_IRQ_EN_tag TIO2_G0_CH6_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls2_TIO2_G0_CH6_IRQ_FORCINT_tag TIO2_G0_CH6_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls2_TIO2_G0_CH6_IRQ_MODE_tag TIO2_G0_CH6_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls2_TIO2_G0_CH6_CTRL2_tag TIO2_G0_CH6_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls2_reserved60[8];
  GTM_gtm_cls2_TIO2_G0_CH6_SINST_tag TIO2_G0_CH6_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH6_SCMD_tag TIO2_G0_CH6_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH6_SOP_tag TIO2_G0_CH6_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls2_reserved61[4];
  GTM_gtm_cls2_TIO2_G0_CH6_OINST_tag TIO2_G0_CH6_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls2_TIO2_G0_CH6_OCMD_tag TIO2_G0_CH6_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls2_TIO2_G0_CH6_OOP_tag TIO2_G0_CH6_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls2_TIO2_G0_CH6_SHIFTCNT_tag TIO2_G0_CH6_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls2_TIO2_G0_CH7_CTRL_tag TIO2_G0_CH7_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls2_TIO2_G0_CH7_IRQ_NOTIFY_tag TIO2_G0_CH7_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls2_TIO2_G0_CH7_IRQ_EN_tag TIO2_G0_CH7_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls2_TIO2_G0_CH7_IRQ_FORCINT_tag TIO2_G0_CH7_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls2_TIO2_G0_CH7_IRQ_MODE_tag TIO2_G0_CH7_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls2_TIO2_G0_CH7_CTRL2_tag TIO2_G0_CH7_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls2_reserved62[8];
  GTM_gtm_cls2_TIO2_G0_CH7_SINST_tag TIO2_G0_CH7_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH7_SCMD_tag TIO2_G0_CH7_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls2_TIO2_G0_CH7_SOP_tag TIO2_G0_CH7_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls2_reserved63[4];
  GTM_gtm_cls2_TIO2_G0_CH7_OINST_tag TIO2_G0_CH7_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls2_TIO2_G0_CH7_OCMD_tag TIO2_G0_CH7_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls2_TIO2_G0_CH7_OOP_tag TIO2_G0_CH7_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls2_TIO2_G0_CH7_SHIFTCNT_tag TIO2_G0_CH7_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls2_TIO2_G0_ISEL0_CTRL1_tag TIO2_G0_ISEL0_CTRL1; /* TIO[i] input selection register 1 */
  GTM_gtm_cls2_TIO2_G0_ISEL0_CTRL2_tag TIO2_G0_ISEL0_CTRL2; /* TIO[i] input selection register 2 */
  uint8_t GTM_gtm_cls2_reserved64[24];
  GTM_gtm_cls2_TIO2_G0_ISEL1_CTRL1_tag TIO2_G0_ISEL1_CTRL1; /* TIO[i] input selection register 1 */
  GTM_gtm_cls2_TIO2_G0_ISEL1_CTRL2_tag TIO2_G0_ISEL1_CTRL2; /* TIO[i] input selection register 2 */
  uint8_t GTM_gtm_cls2_reserved65[24];
  GTM_gtm_cls2_TIO2_G0_OP_USAGE_tag TIO2_G0_OP_USAGE; /* TIO[i] operand usage selection register */
  uint8_t GTM_gtm_cls2_reserved66[2492];
  GTM_gtm_cls2_TIO2_S_tag TIO2_S;      /* TIO[i] signal sampling register */
  GTM_gtm_cls2_TIO2_O_tag TIO2_O;      /* TIO[i] output register */
  GTM_gtm_cls2_TIO2_ENDIS_tag TIO2_ENDIS; /* TIO[i] enable/disable register */
  GTM_gtm_cls2_TIO2_INVERT_tag TIO2_INVERT; /* TIO[i] signal invert register */
  GTM_gtm_cls2_TIO2_INPUT_MODE_tag TIO2_INPUT_MODE; /* TIO[i] input mode register */
  GTM_gtm_cls2_TIO2_CYCLIC_MODE_tag TIO2_CYCLIC_MODE; /* TIO[i] cyclic mode register */
  GTM_gtm_cls2_TIO2_TRIG_OUT_GATE_EN_tag TIO2_TRIG_OUT_GATE_EN; /* TIO[i] enable Trigger Output, output gating register */
  GTM_gtm_cls2_TIO2_PLTRIG_OUT_GATE_EN_tag TIO2_PLTRIG_OUT_GATE_EN; /* TIO[i] enable PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls2_reserved67[32];
  GTM_gtm_cls2_TIO2_CS_tag TIO2_CS;    /* TIO[i] clear signal sampling register */
  GTM_gtm_cls2_TIO2_CO_tag TIO2_CO;    /* TIO[i] clear output register */
  GTM_gtm_cls2_TIO2_CENDIS_tag TIO2_CENDIS; /* TIO[i] disable register */
  GTM_gtm_cls2_TIO2_CINVERT_tag TIO2_CINVERT; /* TIO[i] clear signal invert register */
  GTM_gtm_cls2_TIO2_CINPUT_MODE_tag TIO2_CINPUT_MODE; /* TIO[i] disable input mode register */
  GTM_gtm_cls2_TIO2_CCYCLIC_MODE_tag TIO2_CCYCLIC_MODE; /* TIO[i] disable cyclic mode register */
  GTM_gtm_cls2_TIO2_CTRIG_OUT_GATE_EN_tag TIO2_CTRIG_OUT_GATE_EN; /* TIO[i] clear Trigger Output, output gating register */
  GTM_gtm_cls2_TIO2_CPLTRIG_OUT_GATE_EN_tag TIO2_CPLTRIG_OUT_GATE_EN; /* TIO[i] clear PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls2_reserved68[32];
  GTM_gtm_cls2_TIO2_SS_tag TIO2_SS;    /* TIO[i] set signal sampling register */
  GTM_gtm_cls2_TIO2_SO_tag TIO2_SO;    /* TIO[i] set output register */
  GTM_gtm_cls2_TIO2_SENDIS_tag TIO2_SENDIS; /* TIO[i] enable register */
  GTM_gtm_cls2_TIO2_SINVERT_tag TIO2_SINVERT; /* TIO[i] set signal invert register */
  GTM_gtm_cls2_TIO2_SINPUT_MODE_tag TIO2_SINPUT_MODE; /* TIO[i] enable input mode register */
  GTM_gtm_cls2_TIO2_SCYCLIC_MODE_tag TIO2_SCYCLIC_MODE; /* TIO[i] enable cyclic mode register */
  GTM_gtm_cls2_TIO2_STRIG_OUT_GATE_EN_tag TIO2_STRIG_OUT_GATE_EN; /* TIO[i] set Trigger Output, output gating register */
  GTM_gtm_cls2_TIO2_SPLTRIG_OUT_GATE_EN_tag TIO2_SPLTRIG_OUT_GATE_EN; /* TIO[i] set PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls2_reserved69[32];
  GTM_gtm_cls2_TIO2_IS_tag TIO2_IS;    /* TIO[i] invert signal sampling register */
  GTM_gtm_cls2_TIO2_IO_tag TIO2_IO;    /* TIO[i] invert output register */
  GTM_gtm_cls2_TIO2_IENDIS_tag TIO2_IENDIS; /* TIO[i] toggle enable/disable register */
  GTM_gtm_cls2_TIO2_IINVERT_tag TIO2_IINVERT; /* TIO[i] toggle signal invert register */
  GTM_gtm_cls2_TIO2_IINPUT_MODE_tag TIO2_IINPUT_MODE; /* TIO[i] enable input mode register */
  GTM_gtm_cls2_TIO2_ICYCLIC_MODE_tag TIO2_ICYCLIC_MODE; /* TIO[i] enable cyclic mode register */
  uint8_t GTM_gtm_cls2_reserved70[40];
  GTM_gtm_cls2_TIO2_FUPD_tag TIO2_FUPD; /* TIO[i] force update register */
  GTM_gtm_cls2_TIO2_HW_CONF_tag TIO2_HW_CONF; /* TIO[i] configuration register */
  GTM_gtm_cls2_TIO2_RSEL_CTRL1_tag TIO2_RSEL_CTRL1; /* TIO[i] resource selection control register 1 */
  GTM_gtm_cls2_TIO2_RSEL_CTRL2_tag TIO2_RSEL_CTRL2; /* TIO[i] resource selection control register 2 */
  GTM_gtm_cls2_TIO2_PL_SWRST_tag TIO2_PL_SWRST; /* TIO[i] software reset for TIO Plus functionality */
  uint8_t GTM_gtm_cls2_reserved71[748];
  GTM_gtm_cls2_CCM2_ARP0_CTRL_tag CCM2_ARP0_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP0_PROT_tag CCM2_ARP0_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP1_CTRL_tag CCM2_ARP1_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP1_PROT_tag CCM2_ARP1_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP2_CTRL_tag CCM2_ARP2_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP2_PROT_tag CCM2_ARP2_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP3_CTRL_tag CCM2_ARP3_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP3_PROT_tag CCM2_ARP3_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP4_CTRL_tag CCM2_ARP4_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP4_PROT_tag CCM2_ARP4_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP5_CTRL_tag CCM2_ARP5_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP5_PROT_tag CCM2_ARP5_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP6_CTRL_tag CCM2_ARP6_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP6_PROT_tag CCM2_ARP6_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP7_CTRL_tag CCM2_ARP7_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP7_PROT_tag CCM2_ARP7_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP8_CTRL_tag CCM2_ARP8_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP8_PROT_tag CCM2_ARP8_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls2_CCM2_ARP9_CTRL_tag CCM2_ARP9_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls2_CCM2_ARP9_PROT_tag CCM2_ARP9_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  uint8_t GTM_gtm_cls2_reserved72[380];
  GTM_gtm_cls2_CCM2_TIO_G0_OUT_tag CCM2_TIO_G0_OUT; /* CCM[i] TIO Group 0,1 Output Register */
  uint8_t GTM_gtm_cls2_reserved73[4];
  GTM_gtm_cls2_CCM2_HW_CONF2_tag CCM2_HW_CONF2; /* CCM[i] 2. Hardware Configuration Register */
  GTM_gtm_cls2_CCM2_AEIM_STA_tag CCM2_AEIM_STA; /* CCM[i] MCS Bus Master Status Register */
  GTM_gtm_cls2_CCM2_HW_CONF_tag CCM2_HW_CONF; /* CCM[i] Hardware Configuration Register */
  GTM_gtm_cls2_CCM2_TIM_AUX_IN_SRC_tag CCM2_TIM_AUX_IN_SRC; /* CCM[i] TIM AUX Input Source Register */
  GTM_gtm_cls2_CCM2_EXT_CAP_EN_tag CCM2_EXT_CAP_EN; /* CCM[i] External Capture Enable Register */
  uint8_t GTM_gtm_cls2_reserved74[4];
  GTM_gtm_cls2_CCM2_ATOM_OUT_tag CCM2_ATOM_OUT; /* CCM[i] ATOM Output Register */
  GTM_gtm_cls2_CCM2_CMU_CLK_CFG_tag CCM2_CMU_CLK_CFG; /* CCM[i] CMU Clock Configuration Register */
  uint8_t GTM_gtm_cls2_reserved75[4];
  GTM_gtm_cls2_CCM2_CFG_tag CCM2_CFG;  /* CCM[i] Configuration Register */
  GTM_gtm_cls2_CCM2_PROT_tag CCM2_PROT; /* CCM[i] Protection Register */
  uint8_t GTM_gtm_cls2_reserved76[768];
  GTM_gtm_cls2_CDTM2_DTM4_CTRL_tag CDTM2_DTM4_CTRL; /* CDTM[i]_DTM[d] global configuration and control register */
  GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL1_tag CDTM2_DTM4_CH_CTRL1; /* CDTM[i]_DTM[d] channel control register 1 */
  GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL2_tag CDTM2_DTM4_CH_CTRL2; /* CDTM[i]_DTM[d] channel control register 2 */
  GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL2_SR_tag CDTM2_DTM4_CH_CTRL2_SR; /* CDTM[i] DTM[j] channel control register 2 shadow */
  GTM_gtm_cls2_CDTM2_DTM4_PS_CTRL_tag CDTM2_DTM4_PS_CTRL; /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  GTM_gtm_cls2_CDTM2_DTM4_CH_DTV_tag CDTM2_DTM4_CH_DTV[4]; /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  GTM_gtm_cls2_CDTM2_DTM4_CH_SR_tag CDTM2_DTM4_CH_SR; /* CDTM[i]_DTM[d] channel shadow register */
  GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL3_tag CDTM2_DTM4_CH_CTRL3; /* CDTM[i]_DTM[d] channel control register 3 */
  GTM_gtm_cls2_CDTM2_DTM4_CTRL2_tag CDTM2_DTM4_CTRL2; /* CDTM[i]_DTM[d] global configuration and control register 2 */
  GTM_gtm_cls2_CDTM2_DTM4_CH0_DTV_SR_tag CDTM2_DTM4_CH0_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls2_CDTM2_DTM4_CH1_DTV_SR_tag CDTM2_DTM4_CH1_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls2_CDTM2_DTM4_CH2_DTV_SR_tag CDTM2_DTM4_CH2_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls2_CDTM2_DTM4_CH3_DTV_SR_tag CDTM2_DTM4_CH3_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls2_CDTM2_DTM5_CTRL_tag CDTM2_DTM5_CTRL; /* CDTM[i]_DTM[d] global configuration and control register */
  GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL1_tag CDTM2_DTM5_CH_CTRL1; /* CDTM[i]_DTM[d] channel control register 1 */
  GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL2_tag CDTM2_DTM5_CH_CTRL2; /* CDTM[i]_DTM[d] channel control register 2 */
  GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL2_SR_tag CDTM2_DTM5_CH_CTRL2_SR; /* CDTM[i] DTM[j] channel control register 2 shadow */
  GTM_gtm_cls2_CDTM2_DTM5_PS_CTRL_tag CDTM2_DTM5_PS_CTRL; /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  GTM_gtm_cls2_CDTM2_DTM5_CH_DTV_tag CDTM2_DTM5_CH_DTV[4]; /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  GTM_gtm_cls2_CDTM2_DTM5_CH_SR_tag CDTM2_DTM5_CH_SR; /* CDTM[i]_DTM[d] channel shadow register */
  GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL3_tag CDTM2_DTM5_CH_CTRL3; /* CDTM[i]_DTM[d] channel control register 3 */
  GTM_gtm_cls2_CDTM2_DTM5_CTRL2_tag CDTM2_DTM5_CTRL2; /* CDTM[i]_DTM[d] global configuration and control register 2 */
  GTM_gtm_cls2_CDTM2_DTM5_CH0_DTV_SR_tag CDTM2_DTM5_CH0_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls2_CDTM2_DTM5_CH1_DTV_SR_tag CDTM2_DTM5_CH1_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls2_CDTM2_DTM5_CH2_DTV_SR_tag CDTM2_DTM5_CH2_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls2_CDTM2_DTM5_CH3_DTV_SR_tag CDTM2_DTM5_CH3_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  uint8_t GTM_gtm_cls2_reserved77[2688];
  GTM_gtm_cls2_AXIM2_FREE_tag AXIM2_FREE; /* AXIM[i] slot allocation status. */
  GTM_gtm_cls2_AXIM2_REQUEST_tag AXIM2_REQUEST; /* AXIM[i] slot request (allocation). */
  GTM_gtm_cls2_AXIM2_RELEASE_tag AXIM2_RELEASE; /* AXIM[i] slot release (de-allocation). */
  uint8_t GTM_gtm_cls2_reserved78[20];
  GTM_gtm_cls2_AXIM2_SLOT0_ADDR_LOW_tag AXIM2_SLOT0_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls2_reserved79[4];
  GTM_gtm_cls2_AXIM2_SLOT0_DATA_LOW_tag AXIM2_SLOT0_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls2_reserved80[4];
  GTM_gtm_cls2_AXIM2_SLOT0_CFG1_tag AXIM2_SLOT0_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls2_AXIM2_SLOT0_CFG2_tag AXIM2_SLOT0_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls2_AXIM2_SLOT0_STATUS_tag AXIM2_SLOT0_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls2_reserved81[4];
  GTM_gtm_cls2_AXIM2_SLOT1_ADDR_LOW_tag AXIM2_SLOT1_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls2_reserved82[4];
  GTM_gtm_cls2_AXIM2_SLOT1_DATA_LOW_tag AXIM2_SLOT1_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls2_reserved83[4];
  GTM_gtm_cls2_AXIM2_SLOT1_CFG1_tag AXIM2_SLOT1_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls2_AXIM2_SLOT1_CFG2_tag AXIM2_SLOT1_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls2_AXIM2_SLOT1_STATUS_tag AXIM2_SLOT1_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls2_reserved84[4];
  GTM_gtm_cls2_AXIM2_SLOT2_ADDR_LOW_tag AXIM2_SLOT2_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls2_reserved85[4];
  GTM_gtm_cls2_AXIM2_SLOT2_DATA_LOW_tag AXIM2_SLOT2_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls2_reserved86[4];
  GTM_gtm_cls2_AXIM2_SLOT2_CFG1_tag AXIM2_SLOT2_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls2_AXIM2_SLOT2_CFG2_tag AXIM2_SLOT2_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls2_AXIM2_SLOT2_STATUS_tag AXIM2_SLOT2_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls2_reserved87[4];
  GTM_gtm_cls2_AXIM2_SLOT3_ADDR_LOW_tag AXIM2_SLOT3_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls2_reserved88[4];
  GTM_gtm_cls2_AXIM2_SLOT3_DATA_LOW_tag AXIM2_SLOT3_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls2_reserved89[4];
  GTM_gtm_cls2_AXIM2_SLOT3_CFG1_tag AXIM2_SLOT3_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls2_AXIM2_SLOT3_CFG2_tag AXIM2_SLOT3_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls2_AXIM2_SLOT3_STATUS_tag AXIM2_SLOT3_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls2_reserved90[44900];
  GTM_gtm_cls2_MCS2_MEM_tag MCS2_MEM[3072]; /* MCS[i] memory region */
};


/* ============================================================================
   =============================== Module: GTM_gtm_cls3 =======================
   ============================================================================ */

typedef union GTM_gtm_cls3_ATOM3_CH0_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_RDADDR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_SR0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_SR1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_CM0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_CM1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_CN0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_ATOM3_CH0_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_CH0_CTRL_SR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_RDADDR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_SR0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_SR1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_CM0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_CM1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_CN0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_ATOM3_CH1_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_CH1_CTRL_SR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_RDADDR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_SR0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_SR1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_CM0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_CM1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_CN0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_ATOM3_CH2_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_CH2_CTRL_SR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_RDADDR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_SR0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_SR1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_CM0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_CM1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_CN0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_ATOM3_CH3_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_CH3_CTRL_SR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_RDADDR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_SR0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_SR1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_CM0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_CM1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_CN0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_ATOM3_CH4_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_CH4_CTRL_SR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_RDADDR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_SR0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_SR1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_CM0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_CM1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_CN0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_ATOM3_CH5_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_CH5_CTRL_SR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_RDADDR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_SR0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_SR1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_CM0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_CM1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_CN0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_ATOM3_CH6_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_CH6_CTRL_SR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_RDADDR_union_tag { /* ATOM[i] channel[x] ARU read address register */
  vuint32_t R;
  struct {
    vuint32_t RDADDR0:9;
    vuint32_t _unused_9:7;
    vuint32_t RDADDR1:9;
    vuint32_t _unused_25:7;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_RDADDR_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_CTRL_union_tag { /* ATOM[i] channel [x] control register */
  vuint32_t R;
  struct {
    vuint32_t MODE:2;
    vuint32_t TB12_SEL:1;
    vuint32_t ARU_EN:1;
    vuint32_t ACB:5;
    vuint32_t CMP_CTRL:1;
    vuint32_t EUPM:1;
    vuint32_t SL:1;
    vuint32_t CLK_SRC:4;
    vuint32_t WR_REQ:1;
    vuint32_t TRIG_PULSE:1;
    vuint32_t UDMODE:2;
    vuint32_t RST_CCU0:1;
    vuint32_t OSM_TRIG:1;
    vuint32_t EXT_TRIG:1;
    vuint32_t EXTTRIGOUT:1;
    vuint32_t TRIGOUT:1;
    vuint32_t SLA:1;
    vuint32_t OSM:1;
    vuint32_t ABM:1;
    vuint32_t _unused_28:1;
    vuint32_t EXT_FUPD:1;
    vuint32_t SOMB:1;
    vuint32_t FREEZE:1;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_SR0_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_SR0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_SR1_union_tag { /* ATOM[i] channel [x] CCU0 compare shadow register */
  vuint32_t R;
  struct {
    vuint32_t SR1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_SR1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_CM0_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_CM0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_CM1_union_tag { /* ATOM[i] channel [x] CCU0 compare register */
  vuint32_t R;
  struct {
    vuint32_t CM1:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_CM1_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_CN0_union_tag { /* ATOM[i] channel [x] CCU0 counter register */
  vuint32_t R;
  struct {
    vuint32_t CN0:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_CN0_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_STAT_union_tag { /* ATOM[i] channel [x] status register */
  vuint32_t R;
  struct {
    vuint32_t OL:1;
    vuint32_t _unused_1:15;
    vuint32_t ACBI:5;
    vuint32_t DV:1;
    vuint32_t WRF:1;
    vuint32_t DR:1;
    vuint32_t ACBO:5;
    vuint32_t OSM_RTF:1;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_IRQ_NOTIFY_union_tag { /* ATOM[i] channel [x] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC:1;
    vuint32_t CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_IRQ_EN_union_tag { /* ATOM[i] channel [x] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CCU0TC_IRQ_EN:1;
    vuint32_t CCU1TC_IRQ_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_IRQ_FORCINT_union_tag { /* ATOM[i] channel [x] software interrupt generation */
  vuint32_t R;
  struct {
    vuint32_t TRG_CCU0TC:1;
    vuint32_t TRG_CCU1TC:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_IRQ_MODE_union_tag { /* ATOM[i] channel [x] interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_ATOM3_CH7_CTRL_SR_union_tag { /* ATOM[i] channel [x] control shadow register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t SL_SR:1;
    vuint32_t CLK_SRC_SR:4;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_CH7_CTRL_SR_tag;

typedef union GTM_gtm_cls3_ATOM3_AGC_GLB_CTRL_union_tag { /* ATOM[i] AGC global control register */
  vuint32_t R;
  struct {
    vuint32_t HOST_TRIG:1;
    vuint32_t _unused_1:7;
    vuint32_t RST_CH0:1;
    vuint32_t RST_CH1:1;
    vuint32_t RST_CH2:1;
    vuint32_t RST_CH3:1;
    vuint32_t RST_CH4:1;
    vuint32_t RST_CH5:1;
    vuint32_t RST_CH6:1;
    vuint32_t RST_CH7:1;
    vuint32_t UPEN_CTRL0:2;
    vuint32_t UPEN_CTRL1:2;
    vuint32_t UPEN_CTRL2:2;
    vuint32_t UPEN_CTRL3:2;
    vuint32_t UPEN_CTRL4:2;
    vuint32_t UPEN_CTRL5:2;
    vuint32_t UPEN_CTRL6:2;
    vuint32_t UPEN_CTRL7:2;
  } B;
} GTM_gtm_cls3_ATOM3_AGC_GLB_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_AGC_ENDIS_CTRL_union_tag { /* ATOM[i] AGC enable/disable control register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_CTRL0:2;
    vuint32_t ENDIS_CTRL1:2;
    vuint32_t ENDIS_CTRL2:2;
    vuint32_t ENDIS_CTRL3:2;
    vuint32_t ENDIS_CTRL4:2;
    vuint32_t ENDIS_CTRL5:2;
    vuint32_t ENDIS_CTRL6:2;
    vuint32_t ENDIS_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_AGC_ENDIS_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_AGC_ENDIS_STAT_union_tag { /* ATOM[i] AGC enable/disable status register */
  vuint32_t R;
  struct {
    vuint32_t ENDIS_STAT0:2;
    vuint32_t ENDIS_STAT1:2;
    vuint32_t ENDIS_STAT2:2;
    vuint32_t ENDIS_STAT3:2;
    vuint32_t ENDIS_STAT4:2;
    vuint32_t ENDIS_STAT5:2;
    vuint32_t ENDIS_STAT6:2;
    vuint32_t ENDIS_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_AGC_ENDIS_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_AGC_ACT_TB_union_tag { /* ATOM[i] AGC action time base register */
  vuint32_t R;
  struct {
    vuint32_t ACT_TB:24;
    vuint32_t TB_TRIG:1;
    vuint32_t TBU_SEL:2;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls3_ATOM3_AGC_ACT_TB_tag;

typedef union GTM_gtm_cls3_ATOM3_AGC_OUTEN_CTRL_union_tag { /* ATOM[i] AGC output enable control register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_CTRL0:2;
    vuint32_t OUTEN_CTRL1:2;
    vuint32_t OUTEN_CTRL2:2;
    vuint32_t OUTEN_CTRL3:2;
    vuint32_t OUTEN_CTRL4:2;
    vuint32_t OUTEN_CTRL5:2;
    vuint32_t OUTEN_CTRL6:2;
    vuint32_t OUTEN_CTRL7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_AGC_OUTEN_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_AGC_OUTEN_STAT_union_tag { /* ATOM[i] AGC output enable status register */
  vuint32_t R;
  struct {
    vuint32_t OUTEN_STAT0:2;
    vuint32_t OUTEN_STAT1:2;
    vuint32_t OUTEN_STAT2:2;
    vuint32_t OUTEN_STAT3:2;
    vuint32_t OUTEN_STAT4:2;
    vuint32_t OUTEN_STAT5:2;
    vuint32_t OUTEN_STAT6:2;
    vuint32_t OUTEN_STAT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_AGC_OUTEN_STAT_tag;

typedef union GTM_gtm_cls3_ATOM3_AGC_FUPD_CTRL_union_tag { /* ATOM[i] AGC force update control register */
  vuint32_t R;
  struct {
    vuint32_t FUPD_CTRL0:2;
    vuint32_t FUPD_CTRL1:2;
    vuint32_t FUPD_CTRL2:2;
    vuint32_t FUPD_CTRL3:2;
    vuint32_t FUPD_CTRL4:2;
    vuint32_t FUPD_CTRL5:2;
    vuint32_t FUPD_CTRL6:2;
    vuint32_t FUPD_CTRL7:2;
    vuint32_t RSTCN0_CH0:2;
    vuint32_t RSTCN0_CH1:2;
    vuint32_t RSTCN0_CH2:2;
    vuint32_t RSTCN0_CH3:2;
    vuint32_t RSTCN0_CH4:2;
    vuint32_t RSTCN0_CH5:2;
    vuint32_t RSTCN0_CH6:2;
    vuint32_t RSTCN0_CH7:2;
  } B;
} GTM_gtm_cls3_ATOM3_AGC_FUPD_CTRL_tag;

typedef union GTM_gtm_cls3_ATOM3_AGC_INT_TRIG_union_tag { /* ATOM[i] AGC internal trigger control register */
  vuint32_t R;
  struct {
    vuint32_t INT_TRIG0:2;
    vuint32_t INT_TRIG1:2;
    vuint32_t INT_TRIG2:2;
    vuint32_t INT_TRIG3:2;
    vuint32_t INT_TRIG4:2;
    vuint32_t INT_TRIG5:2;
    vuint32_t INT_TRIG6:2;
    vuint32_t INT_TRIG7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_ATOM3_AGC_INT_TRIG_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH0_R0_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH0_R1_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH0_R2_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH0_R3_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH0_R4_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH0_R5_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH0_R6_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH0_R7_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls3_MCS3_CH0_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_MCS3_CH0_ACB_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CH0_MHB_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_CH0_PC_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_CH0_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH0_EIRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH1_R0_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH1_R1_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH1_R2_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH1_R3_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH1_R4_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH1_R5_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH1_R6_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH1_R7_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls3_MCS3_CH1_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_MCS3_CH1_ACB_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CH1_MHB_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_CH1_PC_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_CH1_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH1_EIRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH2_R0_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH2_R1_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH2_R2_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH2_R3_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH2_R4_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH2_R5_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH2_R6_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH2_R7_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls3_MCS3_CH2_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_MCS3_CH2_ACB_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CH2_MHB_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_CH2_PC_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_CH2_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH2_EIRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH3_R0_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH3_R1_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH3_R2_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH3_R3_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH3_R4_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH3_R5_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH3_R6_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH3_R7_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls3_MCS3_CH3_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_MCS3_CH3_ACB_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CH3_MHB_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_CH3_PC_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_CH3_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH3_EIRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH4_R0_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH4_R1_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH4_R2_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH4_R3_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH4_R4_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH4_R5_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH4_R6_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH4_R7_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls3_MCS3_CH4_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_MCS3_CH4_ACB_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CH4_MHB_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_CH4_PC_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_CH4_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH4_EIRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH5_R0_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH5_R1_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH5_R2_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH5_R3_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH5_R4_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH5_R5_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH5_R6_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH5_R7_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls3_MCS3_CH5_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_MCS3_CH5_ACB_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CH5_MHB_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_CH5_PC_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_CH5_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH5_EIRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH6_R0_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH6_R1_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH6_R2_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH6_R3_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH6_R4_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH6_R5_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH6_R6_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH6_R7_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls3_MCS3_CH6_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_MCS3_CH6_ACB_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CH6_MHB_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_CH6_PC_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_CH6_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH6_EIRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_R0_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH7_R0_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_R1_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH7_R1_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_R2_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH7_R2_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_R3_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH7_R3_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_R4_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH7_R4_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_R5_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH7_R5_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_R6_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH7_R6_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_R7_union_tag { /* MCS[i] channel x general purpose register [y] */
  vuint32_t R;
  struct {
    vuint32_t DATA:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CH7_R7_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_CTRL_union_tag { /* MCS[i] channel x control register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IRQ:1;
    vuint32_t ERR:1;
    vuint32_t _unused_3:1;
    vuint32_t CY:1;
    vuint32_t Z:1;
    vuint32_t V:1;
    vuint32_t N:1;
    vuint32_t CAT:1;
    vuint32_t CWT:1;
    vuint32_t SAT:1;
    vuint32_t _unused_11:21;
  } B;
} GTM_gtm_cls3_MCS3_CH7_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_ACB_union_tag { /* MCS[i] channel x ARU control Bit register */
  vuint32_t R;
  struct {
    vuint32_t ACB0:1;
    vuint32_t ACB1:1;
    vuint32_t ACB2:1;
    vuint32_t ACB3:1;
    vuint32_t ACB4:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_MCS3_CH7_ACB_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_MHB_union_tag { /* MCS[i] channel x memory high byte register */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CH7_MHB_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_PC_union_tag { /* MCS[i] channel x program counter register */
  vuint32_t R;
  struct {
    vuint32_t PC:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_CH7_PC_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_IRQ_NOTIFY_union_tag { /* MCS[i] channel x interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_IRQ_EN_union_tag { /* MCS[i] channel x interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_IRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_IRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_IRQ_FORCINT_union_tag { /* MCS[i] channel x force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_MCS_IRQ:1;
    vuint32_t _unused_1:1;
    vuint32_t TRG_ERR_IRQ:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_IRQ_MODE_union_tag { /* MCS[i] channel x IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_CH7_EIRQ_EN_union_tag { /* MCS[i] channel x error interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t MCS_EIRQ_EN:1;
    vuint32_t _unused_1:1;
    vuint32_t ERR_EIRQ_EN:1;
    vuint32_t _unused_3:29;
  } B;
} GTM_gtm_cls3_MCS3_CH7_EIRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_CTRG_union_tag { /* MCS[i] clear trigger control register */
  vuint32_t R;
  struct {
    vuint32_t TRG0:1;
    vuint32_t TRG1:1;
    vuint32_t TRG2:1;
    vuint32_t TRG3:1;
    vuint32_t TRG4:1;
    vuint32_t TRG5:1;
    vuint32_t TRG6:1;
    vuint32_t TRG7:1;
    vuint32_t TRG8:1;
    vuint32_t TRG9:1;
    vuint32_t TRG10:1;
    vuint32_t TRG11:1;
    vuint32_t TRG12:1;
    vuint32_t TRG13:1;
    vuint32_t TRG14:1;
    vuint32_t TRG15:1;
    vuint32_t TRG16:1;
    vuint32_t TRG17:1;
    vuint32_t TRG18:1;
    vuint32_t TRG19:1;
    vuint32_t TRG20:1;
    vuint32_t TRG21:1;
    vuint32_t TRG22:1;
    vuint32_t TRG23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_CTRG_tag;

typedef union GTM_gtm_cls3_MCS3_STRG_union_tag { /* MCS[i] set trigger control register */
  vuint32_t R;
  struct {
    vuint32_t TRG0:1;
    vuint32_t TRG1:1;
    vuint32_t TRG2:1;
    vuint32_t TRG3:1;
    vuint32_t TRG4:1;
    vuint32_t TRG5:1;
    vuint32_t TRG6:1;
    vuint32_t TRG7:1;
    vuint32_t TRG8:1;
    vuint32_t TRG9:1;
    vuint32_t TRG10:1;
    vuint32_t TRG11:1;
    vuint32_t TRG12:1;
    vuint32_t TRG13:1;
    vuint32_t TRG14:1;
    vuint32_t TRG15:1;
    vuint32_t TRG16:1;
    vuint32_t TRG17:1;
    vuint32_t TRG18:1;
    vuint32_t TRG19:1;
    vuint32_t TRG20:1;
    vuint32_t TRG21:1;
    vuint32_t TRG22:1;
    vuint32_t TRG23:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_MCS3_STRG_tag;

typedef union GTM_gtm_cls3_MCS3_CTRL_STAT_union_tag { /* MCS[i] control and status register */
  vuint32_t R;
  struct {
    vuint32_t SCD_MODE:2;
    vuint32_t _unused_2:6;
    vuint32_t SCD_CH:4;
    vuint32_t _unused_12:4;
    vuint32_t RAM_RST:1;
    vuint32_t _unused_17:3;
    vuint32_t ERR_SRC_ID:3;
    vuint32_t _unused_23:1;
    vuint32_t EN_TIM_FOUT:1;
    vuint32_t EN_HVD:1;
    vuint32_t HLT_AEIM_ERR:1;
    vuint32_t _unused_27:5;
  } B;
} GTM_gtm_cls3_MCS3_CTRL_STAT_tag;

typedef union GTM_gtm_cls3_MCS3_RESET_union_tag { /* MCS[i] reset register */
  vuint32_t R;
  struct {
    vuint32_t RST0:1;
    vuint32_t RST1:1;
    vuint32_t RST2:1;
    vuint32_t RST3:1;
    vuint32_t RST4:1;
    vuint32_t RST5:1;
    vuint32_t RST6:1;
    vuint32_t RST7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_RESET_tag;

typedef union GTM_gtm_cls3_MCS3_CAT_union_tag { /* MCS[i] cancel ARU transfer instruction */
  vuint32_t R;
  struct {
    vuint32_t CAT0:1;
    vuint32_t CAT1:1;
    vuint32_t CAT2:1;
    vuint32_t CAT3:1;
    vuint32_t CAT4:1;
    vuint32_t CAT5:1;
    vuint32_t CAT6:1;
    vuint32_t CAT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CAT_tag;

typedef union GTM_gtm_cls3_MCS3_CWT_union_tag { /* MCS[i] cancel waiting instruction */
  vuint32_t R;
  struct {
    vuint32_t CWT0:1;
    vuint32_t CWT1:1;
    vuint32_t CWT2:1;
    vuint32_t CWT3:1;
    vuint32_t CWT4:1;
    vuint32_t CWT5:1;
    vuint32_t CWT6:1;
    vuint32_t CWT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_CWT_tag;

typedef union GTM_gtm_cls3_MCS3_ERR_union_tag { /* MCS[i] error register */
  vuint32_t R;
  struct {
    vuint32_t ERR0:1;
    vuint32_t ERR1:1;
    vuint32_t ERR2:1;
    vuint32_t ERR3:1;
    vuint32_t ERR4:1;
    vuint32_t ERR5:1;
    vuint32_t ERR6:1;
    vuint32_t ERR7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_ERR_tag;

typedef union GTM_gtm_cls3_MCS3_REG_PROT_union_tag { /* MCS[i] write protection register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:2;
    vuint32_t WPROT1:2;
    vuint32_t WPROT2:2;
    vuint32_t WPROT3:2;
    vuint32_t WPROT4:2;
    vuint32_t WPROT5:2;
    vuint32_t WPROT6:2;
    vuint32_t WPROT7:2;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_MCS3_REG_PROT_tag;

typedef union GTM_gtm_cls3_MCS3_SINT_IRQ_NOTIFY_union_tag { /* MCS[i] shared interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_IRQ0:1;
    vuint32_t S_IRQ1:1;
    vuint32_t S_IRQ2:1;
    vuint32_t S_IRQ3:1;
    vuint32_t S_IRQ4:1;
    vuint32_t S_IRQ5:1;
    vuint32_t S_IRQ6:1;
    vuint32_t S_IRQ7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_SINT_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_SINT_IRQ_EN_union_tag { /* MCS[i] shared interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_IRQ0_EN:1;
    vuint32_t S_IRQ1_EN:1;
    vuint32_t S_IRQ2_EN:1;
    vuint32_t S_IRQ3_EN:1;
    vuint32_t S_IRQ4_EN:1;
    vuint32_t S_IRQ5_EN:1;
    vuint32_t S_IRQ6_EN:1;
    vuint32_t S_IRQ7_EN:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_SINT_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_SINT_IRQ_FORCINT_union_tag { /* MCS[i] force shared interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_IRQ0:1;
    vuint32_t TRG_S_IRQ1:1;
    vuint32_t TRG_S_IRQ2:1;
    vuint32_t TRG_S_IRQ3:1;
    vuint32_t TRG_S_IRQ4:1;
    vuint32_t TRG_S_IRQ5:1;
    vuint32_t TRG_S_IRQ6:1;
    vuint32_t TRG_S_IRQ7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_SINT_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_SINT_IRQ_MODE_union_tag { /* MCS[i] shared interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_SINT_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_HBP0_CTRL_union_tag { /* MCS[i] hardware break point h control register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH0:1;
    vuint32_t EN_CH1:1;
    vuint32_t EN_CH2:1;
    vuint32_t EN_CH3:1;
    vuint32_t EN_CH4:1;
    vuint32_t EN_CH5:1;
    vuint32_t EN_CH6:1;
    vuint32_t EN_CH7:1;
    vuint32_t SCOPE:2;
    vuint32_t _unused_10:2;
    vuint32_t TYPE:3;
    vuint32_t _unused_15:1;
    vuint32_t AND:1;
    vuint32_t NOT:1;
    vuint32_t _unused_18:14;
  } B;
} GTM_gtm_cls3_MCS3_HBP0_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_HBP0_PATTERN_union_tag { /* MCS[i] hardware break point pattern register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls3_MCS3_HBP0_PATTERN_tag;

typedef union GTM_gtm_cls3_MCS3_HBP0_STATUS_union_tag { /* MCS[i] hardware break point status register */
  vuint32_t R;
  struct {
    vuint32_t HALT_CH0:1;
    vuint32_t HALT_CH1:1;
    vuint32_t HALT_CH2:1;
    vuint32_t HALT_CH3:1;
    vuint32_t HALT_CH4:1;
    vuint32_t HALT_CH5:1;
    vuint32_t HALT_CH6:1;
    vuint32_t HALT_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_HBP0_STATUS_tag;

typedef union GTM_gtm_cls3_MCS3_HBP0_IRQ_NOTIFY_union_tag { /* MCS[i] hardware break point interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls3_MCS3_HBP0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_HBP0_IRQ_EN_union_tag { /* MCS[i] hardware break point interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ_EN:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls3_MCS3_HBP0_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_HBP0_IRQ_FORCINT_union_tag { /* MCS[i] force hardware break point interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls3_MCS3_HBP0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_HBP0_IRQ_MODE_union_tag { /* MCS[i] break point h interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_HBP0_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_MCS3_HBP1_CTRL_union_tag { /* MCS[i] hardware break point h control register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH0:1;
    vuint32_t EN_CH1:1;
    vuint32_t EN_CH2:1;
    vuint32_t EN_CH3:1;
    vuint32_t EN_CH4:1;
    vuint32_t EN_CH5:1;
    vuint32_t EN_CH6:1;
    vuint32_t EN_CH7:1;
    vuint32_t SCOPE:2;
    vuint32_t _unused_10:2;
    vuint32_t TYPE:3;
    vuint32_t _unused_15:1;
    vuint32_t AND:1;
    vuint32_t NOT:1;
    vuint32_t _unused_18:14;
  } B;
} GTM_gtm_cls3_MCS3_HBP1_CTRL_tag;

typedef union GTM_gtm_cls3_MCS3_HBP1_PATTERN_union_tag { /* MCS[i] hardware break point pattern register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls3_MCS3_HBP1_PATTERN_tag;

typedef union GTM_gtm_cls3_MCS3_HBP1_STATUS_union_tag { /* MCS[i] hardware break point status register */
  vuint32_t R;
  struct {
    vuint32_t HALT_CH0:1;
    vuint32_t HALT_CH1:1;
    vuint32_t HALT_CH2:1;
    vuint32_t HALT_CH3:1;
    vuint32_t HALT_CH4:1;
    vuint32_t HALT_CH5:1;
    vuint32_t HALT_CH6:1;
    vuint32_t HALT_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_MCS3_HBP1_STATUS_tag;

typedef union GTM_gtm_cls3_MCS3_HBP1_IRQ_NOTIFY_union_tag { /* MCS[i] hardware break point interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls3_MCS3_HBP1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_MCS3_HBP1_IRQ_EN_union_tag { /* MCS[i] hardware break point interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t HBP_IRQ_EN:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls3_MCS3_HBP1_IRQ_EN_tag;

typedef union GTM_gtm_cls3_MCS3_HBP1_IRQ_FORCINT_union_tag { /* MCS[i] force hardware break point interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_HBP_IRQ:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls3_MCS3_HBP1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_MCS3_HBP1_IRQ_MODE_union_tag { /* MCS[i] break point h interrupt mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_MCS3_HBP1_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_CTRL_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_IRQ_EN_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_SINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_SCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_SOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_OINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_OCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_OOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH0_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH0_SHIFTCNT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_CTRL_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_IRQ_EN_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_SINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_SCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_SOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_OINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_OCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_OOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH1_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH1_SHIFTCNT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_CTRL_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_IRQ_EN_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_SINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_SCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_SOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_OINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_OCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_OOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH2_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH2_SHIFTCNT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_CTRL_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_IRQ_EN_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_SINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_SCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_SOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_OINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_OCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_OOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH3_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH3_SHIFTCNT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_CTRL_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_IRQ_EN_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_SINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_SCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_SOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_OINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_OCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_OOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH4_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH4_SHIFTCNT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_CTRL_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_IRQ_EN_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_SINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_SCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_SOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_OINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_OCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_OOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH5_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH5_SHIFTCNT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_CTRL_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_IRQ_EN_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_SINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_SCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_SOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_OINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_OCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_OOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH6_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH6_SHIFTCNT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_CTRL_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t TRIG_OUT_EN_S_RE:1;
    vuint32_t TRIG_OUT_EN_S_FE:1;
    vuint32_t TRIG_OUT_EN_O_RE:1;
    vuint32_t TRIG_OUT_EN_O_FE:1;
    vuint32_t TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t TRIG_OUT_EN_PL_EVT:1;
    vuint32_t TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_CYCLIC_INIT_TRIG_EN:1;
    vuint32_t UPDATE_SRC:4;
    vuint32_t PL_S_MODE:2;
    vuint32_t PL_FREEZE_S_EN:1;
    vuint32_t PL_CYCLIC_BUFF:1;
    vuint32_t PL_O_MODE:3;
    vuint32_t PL_FREEZE_O_EN:1;
    vuint32_t PL_ODIS:1;
    vuint32_t PL_SEL_IN:1;
    vuint32_t PL_O_TRIG_OUT_EN:1;
    vuint32_t PL_S_TRIG_OUT_EN:1;
    vuint32_t PL_TRIG_OUT_EN_S_RE:1;
    vuint32_t PL_TRIG_OUT_EN_S_FE:1;
    vuint32_t PL_TRIG_OUT_EN_O_RE:1;
    vuint32_t PL_TRIG_OUT_EN_O_FE:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_TRIG:1;
    vuint32_t PL_TRIG_OUT_EN_PL_EVT:1;
    vuint32_t PL_TRIG_OUT_EN_PREV_PL_TRIG:1;
    vuint32_t PL_TRIG_OUT_UPD_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_CTRL_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_IRQ_NOTIFY_union_tag { /* TIO[i] channel [c] interrupt notification register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ:1;
    vuint32_t S_FE_IRQ:1;
    vuint32_t O_RE_IRQ:1;
    vuint32_t O_FE_IRQ:1;
    vuint32_t UPDATE_IRQ:1;
    vuint32_t PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_IRQ_NOTIFY_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_IRQ_EN_union_tag { /* TIO[i] channel [c] interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t S_RE_IRQ_EN:1;
    vuint32_t S_FE_IRQ_EN:1;
    vuint32_t O_RE_IRQ_EN:1;
    vuint32_t O_FE_IRQ_EN:1;
    vuint32_t UPDATE_IRQ_EN:1;
    vuint32_t PL_EVT_IRQ_EN:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_IRQ_EN_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_IRQ_FORCINT_union_tag { /* TIO[i] channel [c] force interrupt register */
  vuint32_t R;
  struct {
    vuint32_t TRG_S_RE_IRQ:1;
    vuint32_t TRG_S_FE_IRQ:1;
    vuint32_t TRG_O_RE_IRQ:1;
    vuint32_t TRG_O_FE_IRQ:1;
    vuint32_t TRG_UPDATE_IRQ:1;
    vuint32_t TRG_PL_EVT_IRQ:1;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_IRQ_FORCINT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_IRQ_MODE_union_tag { /* TIO[i] channel [c] IRQ mode configuration register */
  vuint32_t R;
  struct {
    vuint32_t IRQ_MODE:2;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_IRQ_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_CTRL2_union_tag { /* TIO[i] group [g] channel [c] control register */
  vuint32_t R;
  struct {
    vuint32_t DUAL_CMP_EN:1;
    vuint32_t DUAL_CMP_MST_EN:1;
    vuint32_t _unused_2:30;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_SINST_union_tag { /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_SINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_SCMD_union_tag { /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_SCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_SOP_union_tag { /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_SOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_OINST_union_tag { /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_OINST_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_OCMD_union_tag { /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t CMD:6;
    vuint32_t DATA_PUSH_EN:1;
    vuint32_t INSTR_PULL_EN:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_OCMD_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_OOP_union_tag { /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  vuint32_t R;
  struct {
    vuint32_t OP:24;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_OOP_tag;

typedef union GTM_gtm_cls3_TIO3_G0_CH7_SHIFTCNT_union_tag { /* TIO[i] channel [c] resource shift count register */
  vuint32_t R;
  struct {
    vuint32_t CNT:5;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_G0_CH7_SHIFTCNT_tag;

typedef union GTM_gtm_cls3_TIO3_G0_ISEL0_CTRL1_union_tag { /* TIO[i] input selection register 1 */
  vuint32_t R;
  struct {
    vuint32_t LUT2_0:4;
    vuint32_t LUT2_1:4;
    vuint32_t LUT2_2:4;
    vuint32_t LUT2_3:4;
    vuint32_t OUT_SEL0:1;
    vuint32_t OUT_SEL1:1;
    vuint32_t OUT_SEL2:1;
    vuint32_t OUT_SEL3:1;
    vuint32_t _unused_20:4;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls3_TIO3_G0_ISEL0_CTRL1_tag;

typedef union GTM_gtm_cls3_TIO3_G0_ISEL0_CTRL2_union_tag { /* TIO[i] input selection register 2 */
  vuint32_t R;
  struct {
    vuint32_t LUT3:8;
    vuint32_t _unused_8:8;
    vuint32_t QOUT_SEL:2;
    vuint32_t _unused_18:2;
    vuint32_t LUT3IN_SEL0:1;
    vuint32_t LUT3IN_SEL1:1;
    vuint32_t LUT3IN_SEL2:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls3_TIO3_G0_ISEL0_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_ISEL1_CTRL1_union_tag { /* TIO[i] input selection register 1 */
  vuint32_t R;
  struct {
    vuint32_t LUT2_0:4;
    vuint32_t LUT2_1:4;
    vuint32_t LUT2_2:4;
    vuint32_t LUT2_3:4;
    vuint32_t OUT_SEL0:1;
    vuint32_t OUT_SEL1:1;
    vuint32_t OUT_SEL2:1;
    vuint32_t OUT_SEL3:1;
    vuint32_t _unused_20:4;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls3_TIO3_G0_ISEL1_CTRL1_tag;

typedef union GTM_gtm_cls3_TIO3_G0_ISEL1_CTRL2_union_tag { /* TIO[i] input selection register 2 */
  vuint32_t R;
  struct {
    vuint32_t LUT3:8;
    vuint32_t _unused_8:8;
    vuint32_t QOUT_SEL:2;
    vuint32_t _unused_18:2;
    vuint32_t LUT3IN_SEL0:1;
    vuint32_t LUT3IN_SEL1:1;
    vuint32_t LUT3IN_SEL2:1;
    vuint32_t _unused_23:9;
  } B;
} GTM_gtm_cls3_TIO3_G0_ISEL1_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_G0_OP_USAGE_union_tag { /* TIO[i] operand usage selection register */
  vuint32_t R;
  struct {
    vuint32_t MODE0:3;
    vuint32_t MODE1:3;
    vuint32_t MODE2:3;
    vuint32_t MODE3:3;
    vuint32_t MODE4:3;
    vuint32_t MODE5:3;
    vuint32_t MODE6:3;
    vuint32_t MODE7:3;
    vuint32_t WRITE_EN0:1;
    vuint32_t WRITE_EN1:1;
    vuint32_t WRITE_EN2:1;
    vuint32_t WRITE_EN3:1;
    vuint32_t WRITE_EN4:1;
    vuint32_t WRITE_EN5:1;
    vuint32_t WRITE_EN6:1;
    vuint32_t WRITE_EN7:1;
  } B;
} GTM_gtm_cls3_TIO3_G0_OP_USAGE_tag;

typedef union GTM_gtm_cls3_TIO3_S_union_tag { /* TIO[i] signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_S_tag;

typedef union GTM_gtm_cls3_TIO3_O_union_tag { /* TIO[i] output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_O_tag;

typedef union GTM_gtm_cls3_TIO3_ENDIS_union_tag { /* TIO[i] enable/disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_ENDIS_tag;

typedef union GTM_gtm_cls3_TIO3_INVERT_union_tag { /* TIO[i] signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_INVERT_tag;

typedef union GTM_gtm_cls3_TIO3_INPUT_MODE_union_tag { /* TIO[i] input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_INPUT_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_CYCLIC_MODE_union_tag { /* TIO[i] cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CYCLIC_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_TRIG_OUT_GATE_EN_union_tag { /* TIO[i] enable Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_TRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls3_TIO3_PLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] enable PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_PLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls3_TIO3_CS_union_tag { /* TIO[i] clear signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CS_tag;

typedef union GTM_gtm_cls3_TIO3_CO_union_tag { /* TIO[i] clear output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CO_tag;

typedef union GTM_gtm_cls3_TIO3_CENDIS_union_tag { /* TIO[i] disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CENDIS_tag;

typedef union GTM_gtm_cls3_TIO3_CINVERT_union_tag { /* TIO[i] clear signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CINVERT_tag;

typedef union GTM_gtm_cls3_TIO3_CINPUT_MODE_union_tag { /* TIO[i] disable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CINPUT_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_CCYCLIC_MODE_union_tag { /* TIO[i] disable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CCYCLIC_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_CTRIG_OUT_GATE_EN_union_tag { /* TIO[i] clear Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls3_TIO3_CPLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] clear PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_CPLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls3_TIO3_SS_union_tag { /* TIO[i] set signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_SS_tag;

typedef union GTM_gtm_cls3_TIO3_SO_union_tag { /* TIO[i] set output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_SO_tag;

typedef union GTM_gtm_cls3_TIO3_SENDIS_union_tag { /* TIO[i] enable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_SENDIS_tag;

typedef union GTM_gtm_cls3_TIO3_SINVERT_union_tag { /* TIO[i] set signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_SINVERT_tag;

typedef union GTM_gtm_cls3_TIO3_SINPUT_MODE_union_tag { /* TIO[i] enable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_SINPUT_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_SCYCLIC_MODE_union_tag { /* TIO[i] enable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_SCYCLIC_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_STRIG_OUT_GATE_EN_union_tag { /* TIO[i] set Trigger Output, output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_STRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls3_TIO3_SPLTRIG_OUT_GATE_EN_union_tag { /* TIO[i] set PL_TRIG_OUT output gating register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_SPLTRIG_OUT_GATE_EN_tag;

typedef union GTM_gtm_cls3_TIO3_IS_union_tag { /* TIO[i] invert signal sampling register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_IS_tag;

typedef union GTM_gtm_cls3_TIO3_IO_union_tag { /* TIO[i] invert output register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_IO_tag;

typedef union GTM_gtm_cls3_TIO3_IENDIS_union_tag { /* TIO[i] toggle enable/disable register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_IENDIS_tag;

typedef union GTM_gtm_cls3_TIO3_IINVERT_union_tag { /* TIO[i] toggle signal invert register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_IINVERT_tag;

typedef union GTM_gtm_cls3_TIO3_IINPUT_MODE_union_tag { /* TIO[i] enable input mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_IINPUT_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_ICYCLIC_MODE_union_tag { /* TIO[i] enable cyclic mode register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_ICYCLIC_MODE_tag;

typedef union GTM_gtm_cls3_TIO3_FUPD_union_tag { /* TIO[i] force update register */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_FUPD_tag;

typedef union GTM_gtm_cls3_TIO3_HW_CONF_union_tag { /* TIO[i] configuration register */
  vuint32_t R;
  struct {
    vuint32_t NTIO_CH8:2;
    vuint32_t _unused_2:2;
    vuint32_t TIO_PLUS:1;
    vuint32_t _unused_5:27;
  } B;
} GTM_gtm_cls3_TIO3_HW_CONF_tag;

typedef union GTM_gtm_cls3_TIO3_RSEL_CTRL1_union_tag { /* TIO[i] resource selection control register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t SEL_CLKEN6_0:1;
    vuint32_t _unused_25:3;
    vuint32_t SEL_CLKEN7_0:1;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls3_TIO3_RSEL_CTRL1_tag;

typedef union GTM_gtm_cls3_TIO3_RSEL_CTRL2_union_tag { /* TIO[i] resource selection control register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t SEL_TB1_0:1;
    vuint32_t _unused_5:3;
    vuint32_t SEL_TB2_0:1;
    vuint32_t _unused_9:23;
  } B;
} GTM_gtm_cls3_TIO3_RSEL_CTRL2_tag;

typedef union GTM_gtm_cls3_TIO3_PL_SWRST_union_tag { /* TIO[i] software reset for TIO Plus functionality */
  vuint32_t R;
  struct {
    vuint32_t CH0:1;
    vuint32_t CH1:1;
    vuint32_t CH2:1;
    vuint32_t CH3:1;
    vuint32_t CH4:1;
    vuint32_t CH5:1;
    vuint32_t CH6:1;
    vuint32_t CH7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_TIO3_PL_SWRST_tag;

typedef union GTM_gtm_cls3_CCM3_ARP0_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP0_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP0_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP0_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP1_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP1_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP1_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP1_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP2_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP2_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP2_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP2_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP3_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP3_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP3_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP3_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP4_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP4_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP4_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP4_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP5_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP5_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP5_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP5_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP6_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP6_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP6_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP6_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP7_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP7_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP7_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP7_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP8_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP8_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP8_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP8_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_ARP9_CTRL_union_tag { /* CCM[i] Address Range Protector [a] Control Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;
    vuint32_t SIZE:4;
    vuint32_t _unused_20:4;
    vuint32_t DIS_PROT:1;
    vuint32_t _unused_25:6;
    vuint32_t WPROT_AEI:1;
  } B;
} GTM_gtm_cls3_CCM3_ARP9_CTRL_tag;

typedef union GTM_gtm_cls3_CCM3_ARP9_PROT_union_tag { /* CCM[i] Address Range Protector [a] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t WPROT0:1;
    vuint32_t WPROT1:1;
    vuint32_t WPROT2:1;
    vuint32_t WPROT3:1;
    vuint32_t WPROT4:1;
    vuint32_t WPROT5:1;
    vuint32_t WPROT6:1;
    vuint32_t WPROT7:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CCM3_ARP9_PROT_tag;

typedef union GTM_gtm_cls3_CCM3_TIO_G0_OUT_union_tag { /* CCM[i] TIO Group 0,1 Output Register */
  vuint32_t R;
  struct {
    vuint32_t TIO_G0_OUT0:1;
    vuint32_t TIO_G0_OUT1:1;
    vuint32_t TIO_G0_OUT2:1;
    vuint32_t TIO_G0_OUT3:1;
    vuint32_t TIO_G0_OUT4:1;
    vuint32_t TIO_G0_OUT5:1;
    vuint32_t TIO_G0_OUT6:1;
    vuint32_t TIO_G0_OUT7:1;
    vuint32_t _unused_8:8;
    vuint32_t TIO_G0_OUT_N0:1;
    vuint32_t TIO_G0_OUT_N1:1;
    vuint32_t TIO_G0_OUT_N2:1;
    vuint32_t TIO_G0_OUT_N3:1;
    vuint32_t TIO_G0_OUT_N4:1;
    vuint32_t TIO_G0_OUT_N5:1;
    vuint32_t TIO_G0_OUT_N6:1;
    vuint32_t TIO_G0_OUT_N7:1;
    vuint32_t _unused_24:8;
  } B;
} GTM_gtm_cls3_CCM3_TIO_G0_OUT_tag;

typedef union GTM_gtm_cls3_CCM3_HW_CONF2_union_tag { /* CCM[i] 2. Hardware Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t AXIM_ID_WIDTH:5;
    vuint32_t AXIM_PRIV_ACC:1;
    vuint32_t AXIM_SEC_ACC:1;
    vuint32_t AXIM_POSTED_WRITE:1;
    vuint32_t _unused_8:1;
    vuint32_t TIO_OUT_RST:1;
    vuint32_t _unused_10:6;
    vuint32_t AXIS_DATA_SIZE:1;
    vuint32_t _unused_17:1;
    vuint32_t AXIM_DATA_SIZE:1;
    vuint32_t _unused_19:13;
  } B;
} GTM_gtm_cls3_CCM3_HW_CONF2_tag;

typedef union GTM_gtm_cls3_CCM3_AEIM_STA_union_tag { /* CCM[i] MCS Bus Master Status Register */
  vuint32_t R;
  struct {
    vuint32_t AEIM_XPT_ADDR:16;
    vuint32_t _unused_16:8;
    vuint32_t AEIM_XPT_STA:2;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls3_CCM3_AEIM_STA_tag;

typedef union GTM_gtm_cls3_CCM3_HW_CONF_union_tag { /* CCM[i] Hardware Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t GRSTEN:1;
    vuint32_t BRIDGE_MODE_RST:1;
    vuint32_t SYNC_INPUT_REG:1;
    vuint32_t CFG_CLOCK_RATE:1;
    vuint32_t ATOM_OUT_RST:1;
    vuint32_t ATOM_TRIG_CHAIN:3;
    vuint32_t TOM_OUT_RST:1;
    vuint32_t TOM_TRIG_CHAIN:3;
    vuint32_t RAM_INIT_RST:1;
    vuint32_t ERM:1;
    vuint32_t _unused_14:1;
    vuint32_t RESET_ACTIVE:1;
    vuint32_t IRQ_MODE_LEVEL:1;
    vuint32_t IRQ_MODE_PULSE:1;
    vuint32_t IRQ_MODE_PULSE_NOTIFY:1;
    vuint32_t IRQ_MODE_SINGLE_PULSE:1;
    vuint32_t ATOM_TRIG_INTCHAIN:4;
    vuint32_t TOM_TRIG_INTCHAIN:5;
    vuint32_t INT_CLK_EN_GEN:1;
    vuint32_t AEI_ADDR_PIPELINE_STAGE:1;
    vuint32_t AEI_RDATA_PIPELINE_STAGE:1;
  } B;
} GTM_gtm_cls3_CCM3_HW_CONF_tag;

typedef union GTM_gtm_cls3_CCM3_ATOM_OUT_union_tag { /* CCM[i] ATOM Output Register */
  vuint32_t R;
  struct {
    vuint32_t ATOM_I_OUT0:1;
    vuint32_t ATOM_I_OUT1:1;
    vuint32_t ATOM_I_OUT2:1;
    vuint32_t ATOM_I_OUT3:1;
    vuint32_t ATOM_I_OUT4:1;
    vuint32_t ATOM_I_OUT5:1;
    vuint32_t ATOM_I_OUT6:1;
    vuint32_t ATOM_I_OUT7:1;
    vuint32_t ATOM_I_OUT_N0:1;
    vuint32_t ATOM_I_OUT_N1:1;
    vuint32_t ATOM_I_OUT_N2:1;
    vuint32_t ATOM_I_OUT_N3:1;
    vuint32_t ATOM_I_OUT_N4:1;
    vuint32_t ATOM_I_OUT_N5:1;
    vuint32_t ATOM_I_OUT_N6:1;
    vuint32_t ATOM_I_OUT_N7:1;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_CCM3_ATOM_OUT_tag;

typedef union GTM_gtm_cls3_CCM3_CMU_CLK_CFG_union_tag { /* CCM[i] CMU Clock Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CLK0_SRC:2;
    vuint32_t _unused_2:2;
    vuint32_t CLK1_SRC:2;
    vuint32_t _unused_6:2;
    vuint32_t CLK2_SRC:2;
    vuint32_t _unused_10:2;
    vuint32_t CLK3_SRC:2;
    vuint32_t _unused_14:2;
    vuint32_t CLK4_SRC:2;
    vuint32_t _unused_18:2;
    vuint32_t CLK5_SRC:2;
    vuint32_t _unused_22:2;
    vuint32_t CLK6_SRC:2;
    vuint32_t _unused_26:2;
    vuint32_t CLK7_SRC:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_CCM3_CMU_CLK_CFG_tag;

typedef union GTM_gtm_cls3_CCM3_CFG_union_tag { /* CCM[i] Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t EN_ATOM_ADTM:1;
    vuint32_t EN_MCS:1;
    vuint32_t _unused_4:4;
    vuint32_t EN_TIO_DTM:1;
    vuint32_t _unused_9:7;
    vuint32_t CLS_CLK_DIV:2;
    vuint32_t _unused_18:12;
    vuint32_t TBU_DIR1:1;
    vuint32_t TBU_DIR2:1;
  } B;
} GTM_gtm_cls3_CCM3_CFG_tag;

typedef union GTM_gtm_cls3_CCM3_PROT_union_tag { /* CCM[i] Protection Register */
  vuint32_t R;
  struct {
    vuint32_t CLS_PROT:1;
    vuint32_t _unused_1:31;
  } B;
} GTM_gtm_cls3_CCM3_PROT_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CTRL_union_tag { /* CDTM[i]_DTM[d] global configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t CLK_SEL:2;
    vuint32_t DTM_SEL:2;
    vuint32_t UPD_MODE:3;
    vuint32_t CH_SHUTOFF_EN:1;
    vuint32_t SR_UPD_EN:1;
    vuint32_t _unused_9:7;
    vuint32_t SHUT_OFF_RST:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CTRL_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL1_union_tag { /* CDTM[i]_DTM[d] channel control register 1 */
  vuint32_t R;
  struct {
    vuint32_t O1SEL_0:1;
    vuint32_t _unused_1:2;
    vuint32_t SWAP_0:1;
    vuint32_t O1F_0:2;
    vuint32_t XDT_EN_0_1:1;
    vuint32_t _unused_7:1;
    vuint32_t O1SEL_1:1;
    vuint32_t I1SEL_1:1;
    vuint32_t SH_EN_1:1;
    vuint32_t SWAP_1:1;
    vuint32_t O1F_1:2;
    vuint32_t _unused_14:2;
    vuint32_t O1SEL_2:1;
    vuint32_t I1SEL_2:1;
    vuint32_t SH_EN_2:1;
    vuint32_t SWAP_2:1;
    vuint32_t O1F_2:2;
    vuint32_t XDT_EN_2_3:1;
    vuint32_t _unused_23:1;
    vuint32_t O1SEL_3:1;
    vuint32_t I1SEL_3:1;
    vuint32_t SH_EN_3:1;
    vuint32_t SWAP_3:1;
    vuint32_t O1F_3:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL1_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL2_union_tag { /* CDTM[i]_DTM[d] channel control register 2 */
  vuint32_t R;
  struct {
    vuint32_t POL0_0:1;
    vuint32_t OC0_0:1;
    vuint32_t SL0_0:1;
    vuint32_t DT0_0:1;
    vuint32_t POL1_0:1;
    vuint32_t OC1_0:1;
    vuint32_t SL1_0:1;
    vuint32_t DT1_0:1;
    vuint32_t POL0_1:1;
    vuint32_t OC0_1:1;
    vuint32_t SL0_1:1;
    vuint32_t DT0_1:1;
    vuint32_t POL1_1:1;
    vuint32_t OC1_1:1;
    vuint32_t SL1_1:1;
    vuint32_t DT1_1:1;
    vuint32_t POL0_2:1;
    vuint32_t OC0_2:1;
    vuint32_t SL0_2:1;
    vuint32_t DT0_2:1;
    vuint32_t POL1_2:1;
    vuint32_t OC1_2:1;
    vuint32_t SL1_2:1;
    vuint32_t DT1_2:1;
    vuint32_t POL0_3:1;
    vuint32_t OC0_3:1;
    vuint32_t SL0_3:1;
    vuint32_t DT0_3:1;
    vuint32_t POL1_3:1;
    vuint32_t OC1_3:1;
    vuint32_t SL1_3:1;
    vuint32_t DT1_3:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL2_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL2_SR_union_tag { /* CDTM[i] DTM[j] channel control register 2 shadow */
  vuint32_t R;
  struct {
    vuint32_t POL0_0_SR:1;
    vuint32_t OC0_0_SR:1;
    vuint32_t SL0_0_SR:1;
    vuint32_t DT0_0_SR:1;
    vuint32_t POL1_0_SR:1;
    vuint32_t OC1_0_SR:1;
    vuint32_t SL1_0_SR:1;
    vuint32_t DT1_0_SR:1;
    vuint32_t POL0_1_SR:1;
    vuint32_t OC0_1_SR:1;
    vuint32_t SL0_1_SR:1;
    vuint32_t DT0_1_SR:1;
    vuint32_t POL1_1_SR:1;
    vuint32_t OC1_1_SR:1;
    vuint32_t SL1_1_SR:1;
    vuint32_t DT1_1_SR:1;
    vuint32_t POL0_2_SR:1;
    vuint32_t OC0_2_SR:1;
    vuint32_t SL0_2_SR:1;
    vuint32_t DT0_2_SR:1;
    vuint32_t POL1_2_SR:1;
    vuint32_t OC1_2_SR:1;
    vuint32_t SL1_2_SR:1;
    vuint32_t DT1_2_SR:1;
    vuint32_t POL0_3_SR:1;
    vuint32_t OC0_3_SR:1;
    vuint32_t SL0_3_SR:1;
    vuint32_t DT0_3_SR:1;
    vuint32_t POL1_3_SR:1;
    vuint32_t OC1_3_SR:1;
    vuint32_t SL1_3_SR:1;
    vuint32_t DT1_3_SR:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL2_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_PS_CTRL_union_tag { /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t RELBLK:10;
    vuint32_t _unused_10:6;
    vuint32_t PSU_IN_SEL:1;
    vuint32_t IN_POL:1;
    vuint32_t TIM_SEL:1;
    vuint32_t _unused_19:1;
    vuint32_t SHIFT_SEL:2;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_PS_CTRL_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH_DTV_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE:13;
    vuint32_t _unused_13:3;
    vuint32_t RELFALL:13;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH_DTV_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH_SR_union_tag { /* CDTM[i]_DTM[d] channel shadow register */
  vuint32_t R;
  struct {
    vuint32_t SL0_0_SR_SR:1;
    vuint32_t SL1_0_SR_SR:1;
    vuint32_t SL0_1_SR_SR:1;
    vuint32_t SL1_1_SR_SR:1;
    vuint32_t SL0_2_SR_SR:1;
    vuint32_t SL1_2_SR_SR:1;
    vuint32_t SL0_3_SR_SR:1;
    vuint32_t SL1_3_SR_SR:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL3_union_tag { /* CDTM[i]_DTM[d] channel control register 3 */
  vuint32_t R;
  struct {
    vuint32_t CII0:1;
    vuint32_t CIS0:1;
    vuint32_t TSEL0_0:1;
    vuint32_t TSEL1_0:1;
    vuint32_t _unused_4:4;
    vuint32_t CII1:1;
    vuint32_t CIS1:1;
    vuint32_t TSEL0_1:1;
    vuint32_t TSEL1_1:1;
    vuint32_t _unused_12:4;
    vuint32_t CII2:1;
    vuint32_t CIS2:1;
    vuint32_t TSEL0_2:1;
    vuint32_t TSEL1_2:1;
    vuint32_t _unused_20:4;
    vuint32_t CII3:1;
    vuint32_t CIS3:1;
    vuint32_t TSEL0_3:1;
    vuint32_t TSEL1_3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL3_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CTRL2_union_tag { /* CDTM[i]_DTM[d] global configuration and control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SHUTOFF_SEL_0:3;
    vuint32_t SHUTOFF_POL_0:1;
    vuint32_t UPD_MODE_0:2;
    vuint32_t SHUT_OFF_RST_0:1;
    vuint32_t WR_EN_0:1;
    vuint32_t SHUTOFF_SEL_1:3;
    vuint32_t SHUTOFF_POL_1:1;
    vuint32_t UPD_MODE_1:2;
    vuint32_t SHUT_OFF_RST_1:1;
    vuint32_t WR_EN_1:1;
    vuint32_t SHUTOFF_SEL_2:3;
    vuint32_t SHUTOFF_POL_2:1;
    vuint32_t UPD_MODE_2:2;
    vuint32_t SHUT_OFF_RST_2:1;
    vuint32_t WR_EN_2:1;
    vuint32_t SHUTOFF_SEL_3:3;
    vuint32_t SHUTOFF_POL_3:1;
    vuint32_t UPD_MODE_3:2;
    vuint32_t SHUT_OFF_RST_3:1;
    vuint32_t WR_EN_3:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CTRL2_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH0_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH0_DTV_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH1_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH1_DTV_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH2_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH2_DTV_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM4_CH3_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM4_CH3_DTV_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CTRL_union_tag { /* CDTM[i]_DTM[d] global configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t CLK_SEL:2;
    vuint32_t DTM_SEL:2;
    vuint32_t UPD_MODE:3;
    vuint32_t CH_SHUTOFF_EN:1;
    vuint32_t SR_UPD_EN:1;
    vuint32_t _unused_9:7;
    vuint32_t SHUT_OFF_RST:1;
    vuint32_t _unused_17:15;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CTRL_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL1_union_tag { /* CDTM[i]_DTM[d] channel control register 1 */
  vuint32_t R;
  struct {
    vuint32_t O1SEL_0:1;
    vuint32_t I1SEL_0:1;
    vuint32_t _unused_2:1;
    vuint32_t SWAP_0:1;
    vuint32_t O1F_0:2;
    vuint32_t XDT_EN_0_1:1;
    vuint32_t _unused_7:1;
    vuint32_t O1SEL_1:1;
    vuint32_t I1SEL_1:1;
    vuint32_t SH_EN_1:1;
    vuint32_t SWAP_1:1;
    vuint32_t O1F_1:2;
    vuint32_t _unused_14:2;
    vuint32_t O1SEL_2:1;
    vuint32_t I1SEL_2:1;
    vuint32_t SH_EN_2:1;
    vuint32_t SWAP_2:1;
    vuint32_t O1F_2:2;
    vuint32_t XDT_EN_2_3:1;
    vuint32_t _unused_23:1;
    vuint32_t O1SEL_3:1;
    vuint32_t I1SEL_3:1;
    vuint32_t SH_EN_3:1;
    vuint32_t SWAP_3:1;
    vuint32_t O1F_3:2;
    vuint32_t _unused_30:2;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL1_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL2_union_tag { /* CDTM[i]_DTM[d] channel control register 2 */
  vuint32_t R;
  struct {
    vuint32_t POL0_0:1;
    vuint32_t OC0_0:1;
    vuint32_t SL0_0:1;
    vuint32_t DT0_0:1;
    vuint32_t POL1_0:1;
    vuint32_t OC1_0:1;
    vuint32_t SL1_0:1;
    vuint32_t DT1_0:1;
    vuint32_t POL0_1:1;
    vuint32_t OC0_1:1;
    vuint32_t SL0_1:1;
    vuint32_t DT0_1:1;
    vuint32_t POL1_1:1;
    vuint32_t OC1_1:1;
    vuint32_t SL1_1:1;
    vuint32_t DT1_1:1;
    vuint32_t POL0_2:1;
    vuint32_t OC0_2:1;
    vuint32_t SL0_2:1;
    vuint32_t DT0_2:1;
    vuint32_t POL1_2:1;
    vuint32_t OC1_2:1;
    vuint32_t SL1_2:1;
    vuint32_t DT1_2:1;
    vuint32_t POL0_3:1;
    vuint32_t OC0_3:1;
    vuint32_t SL0_3:1;
    vuint32_t DT0_3:1;
    vuint32_t POL1_3:1;
    vuint32_t OC1_3:1;
    vuint32_t SL1_3:1;
    vuint32_t DT1_3:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL2_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL2_SR_union_tag { /* CDTM[i] DTM[j] channel control register 2 shadow */
  vuint32_t R;
  struct {
    vuint32_t POL0_0_SR:1;
    vuint32_t OC0_0_SR:1;
    vuint32_t SL0_0_SR:1;
    vuint32_t DT0_0_SR:1;
    vuint32_t POL1_0_SR:1;
    vuint32_t OC1_0_SR:1;
    vuint32_t SL1_0_SR:1;
    vuint32_t DT1_0_SR:1;
    vuint32_t POL0_1_SR:1;
    vuint32_t OC0_1_SR:1;
    vuint32_t SL0_1_SR:1;
    vuint32_t DT0_1_SR:1;
    vuint32_t POL1_1_SR:1;
    vuint32_t OC1_1_SR:1;
    vuint32_t SL1_1_SR:1;
    vuint32_t DT1_1_SR:1;
    vuint32_t POL0_2_SR:1;
    vuint32_t OC0_2_SR:1;
    vuint32_t SL0_2_SR:1;
    vuint32_t DT0_2_SR:1;
    vuint32_t POL1_2_SR:1;
    vuint32_t OC1_2_SR:1;
    vuint32_t SL1_2_SR:1;
    vuint32_t DT1_2_SR:1;
    vuint32_t POL0_3_SR:1;
    vuint32_t OC0_3_SR:1;
    vuint32_t SL0_3_SR:1;
    vuint32_t DT0_3_SR:1;
    vuint32_t POL1_3_SR:1;
    vuint32_t OC1_3_SR:1;
    vuint32_t SL1_3_SR:1;
    vuint32_t DT1_3_SR:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL2_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_PS_CTRL_union_tag { /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  vuint32_t R;
  struct {
    vuint32_t RELBLK:10;
    vuint32_t _unused_10:6;
    vuint32_t PSU_IN_SEL:1;
    vuint32_t IN_POL:1;
    vuint32_t TIM_SEL:1;
    vuint32_t _unused_19:1;
    vuint32_t SHIFT_SEL:2;
    vuint32_t _unused_22:10;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_PS_CTRL_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH_DTV_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE:13;
    vuint32_t _unused_13:3;
    vuint32_t RELFALL:13;
    vuint32_t _unused_29:3;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH_DTV_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH_SR_union_tag { /* CDTM[i]_DTM[d] channel shadow register */
  vuint32_t R;
  struct {
    vuint32_t SL0_0_SR_SR:1;
    vuint32_t SL1_0_SR_SR:1;
    vuint32_t SL0_1_SR_SR:1;
    vuint32_t SL1_1_SR_SR:1;
    vuint32_t SL0_2_SR_SR:1;
    vuint32_t SL1_2_SR_SR:1;
    vuint32_t SL0_3_SR_SR:1;
    vuint32_t SL1_3_SR_SR:1;
    vuint32_t _unused_8:24;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL3_union_tag { /* CDTM[i]_DTM[d] channel control register 3 */
  vuint32_t R;
  struct {
    vuint32_t CII0:1;
    vuint32_t CIS0:1;
    vuint32_t TSEL0_0:1;
    vuint32_t TSEL1_0:1;
    vuint32_t _unused_4:4;
    vuint32_t CII1:1;
    vuint32_t CIS1:1;
    vuint32_t TSEL0_1:1;
    vuint32_t TSEL1_1:1;
    vuint32_t _unused_12:4;
    vuint32_t CII2:1;
    vuint32_t CIS2:1;
    vuint32_t TSEL0_2:1;
    vuint32_t TSEL1_2:1;
    vuint32_t _unused_20:4;
    vuint32_t CII3:1;
    vuint32_t CIS3:1;
    vuint32_t TSEL0_3:1;
    vuint32_t TSEL1_3:1;
    vuint32_t _unused_28:4;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL3_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CTRL2_union_tag { /* CDTM[i]_DTM[d] global configuration and control register 2 */
  vuint32_t R;
  struct {
    vuint32_t SHUTOFF_SEL_0:3;
    vuint32_t SHUTOFF_POL_0:1;
    vuint32_t UPD_MODE_0:2;
    vuint32_t SHUT_OFF_RST_0:1;
    vuint32_t WR_EN_0:1;
    vuint32_t SHUTOFF_SEL_1:3;
    vuint32_t SHUTOFF_POL_1:1;
    vuint32_t UPD_MODE_1:2;
    vuint32_t SHUT_OFF_RST_1:1;
    vuint32_t WR_EN_1:1;
    vuint32_t SHUTOFF_SEL_2:3;
    vuint32_t SHUTOFF_POL_2:1;
    vuint32_t UPD_MODE_2:2;
    vuint32_t SHUT_OFF_RST_2:1;
    vuint32_t WR_EN_2:1;
    vuint32_t SHUTOFF_SEL_3:3;
    vuint32_t SHUTOFF_POL_3:1;
    vuint32_t UPD_MODE_3:2;
    vuint32_t SHUT_OFF_RST_3:1;
    vuint32_t WR_EN_3:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CTRL2_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH0_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH0_DTV_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH1_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH1_DTV_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH2_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH2_DTV_SR_tag;

typedef union GTM_gtm_cls3_CDTM3_DTM5_CH3_DTV_SR_union_tag { /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  vuint32_t R;
  struct {
    vuint32_t RELRISE_SR:13;
    vuint32_t _unused_13:1;
    vuint32_t RELRISE_UPD_FE0RE1:1;
    vuint32_t RELRISE_UPD_EN:1;
    vuint32_t RELFALL_SR:13;
    vuint32_t _unused_29:1;
    vuint32_t RELFALL_UPD_FE0RE1:1;
    vuint32_t RELFALL_UPD_EN:1;
  } B;
} GTM_gtm_cls3_CDTM3_DTM5_CH3_DTV_SR_tag;

typedef union GTM_gtm_cls3_AXIM3_FREE_union_tag { /* AXIM[i] slot allocation status. */
  vuint32_t R;
  struct {
    vuint32_t FREE0:1;
    vuint32_t FREE1:1;
    vuint32_t FREE2:1;
    vuint32_t FREE3:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls3_AXIM3_FREE_tag;

typedef union GTM_gtm_cls3_AXIM3_REQUEST_union_tag { /* AXIM[i] slot request (allocation). */
  vuint32_t R;
  struct {
    vuint32_t REQ1HOT0:1;
    vuint32_t REQ1HOT1:1;
    vuint32_t REQ1HOT2:1;
    vuint32_t REQ1HOT3:1;
    vuint32_t _unused_4:20;
    vuint32_t REQID:8;
  } B;
} GTM_gtm_cls3_AXIM3_REQUEST_tag;

typedef union GTM_gtm_cls3_AXIM3_RELEASE_union_tag { /* AXIM[i] slot release (de-allocation). */
  vuint32_t R;
  struct {
    vuint32_t RELREQ0:1;
    vuint32_t RELREQ1:1;
    vuint32_t RELREQ2:1;
    vuint32_t RELREQ3:1;
    vuint32_t _unused_4:28;
  } B;
} GTM_gtm_cls3_AXIM3_RELEASE_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT0_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT0_ADDR_LOW_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT0_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT0_DATA_LOW_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT0_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT0_CFG1_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT0_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT0_CFG2_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT0_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT0_STATUS_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT1_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT1_ADDR_LOW_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT1_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT1_DATA_LOW_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT1_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT1_CFG1_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT1_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT1_CFG2_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT1_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT1_STATUS_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT2_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT2_ADDR_LOW_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT2_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT2_DATA_LOW_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT2_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT2_CFG1_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT2_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT2_CFG2_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT2_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT2_STATUS_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT3_ADDR_LOW_union_tag { /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_ADDR:32;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT3_ADDR_LOW_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT3_DATA_LOW_union_tag { /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  vuint32_t R;
  struct {
    vuint32_t AXI_DATA_LOW:32;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT3_DATA_LOW_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT3_CFG1_union_tag { /* AXIM[i] slot [s] configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t INCR:4;
    vuint32_t AUTO_INCR:1;
    vuint32_t PRIO:2;
    vuint32_t _unused_7:4;
    vuint32_t AXI_PROT:3;
    vuint32_t AXI_CACHE:4;
    vuint32_t AXI_LOCK:2;
    vuint32_t _unused_20:2;
    vuint32_t AXI_SIZE:3;
    vuint32_t AXI_RW:1;
    vuint32_t _unused_26:6;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT3_CFG1_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT3_CFG2_union_tag { /* AXIM[i] slot[s] configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t AXI_ID:16;
    vuint32_t _unused_16:16;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT3_CFG2_tag;

typedef union GTM_gtm_cls3_AXIM3_SLOT3_STATUS_union_tag { /* AXIM[i] slot[s] status */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:1;
    vuint32_t QUEUED:1;
    vuint32_t STARTED:1;
    vuint32_t READY:1;
    vuint32_t RESP:2;
    vuint32_t _unused_6:26;
  } B;
} GTM_gtm_cls3_AXIM3_SLOT3_STATUS_tag;

typedef union GTM_gtm_cls3_MCS3_MEM_union_tag { /* MCS[i] memory region */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;
  } B;
} GTM_gtm_cls3_MCS3_MEM_tag;

struct GTM_gtm_cls3_tag {
  uint8_t GTM_gtm_cls3_reserved0[6144];
  GTM_gtm_cls3_ATOM3_CH0_RDADDR_tag ATOM3_CH0_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls3_ATOM3_CH0_CTRL_tag ATOM3_CH0_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls3_ATOM3_CH0_SR0_tag ATOM3_CH0_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH0_SR1_tag ATOM3_CH0_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH0_CM0_tag ATOM3_CH0_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH0_CM1_tag ATOM3_CH0_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH0_CN0_tag ATOM3_CH0_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls3_ATOM3_CH0_STAT_tag ATOM3_CH0_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls3_ATOM3_CH0_IRQ_NOTIFY_tag ATOM3_CH0_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls3_ATOM3_CH0_IRQ_EN_tag ATOM3_CH0_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls3_ATOM3_CH0_IRQ_FORCINT_tag ATOM3_CH0_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls3_ATOM3_CH0_IRQ_MODE_tag ATOM3_CH0_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved1[4];
  GTM_gtm_cls3_ATOM3_CH0_CTRL_SR_tag ATOM3_CH0_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls3_reserved2[72];
  GTM_gtm_cls3_ATOM3_CH1_RDADDR_tag ATOM3_CH1_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls3_ATOM3_CH1_CTRL_tag ATOM3_CH1_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls3_ATOM3_CH1_SR0_tag ATOM3_CH1_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH1_SR1_tag ATOM3_CH1_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH1_CM0_tag ATOM3_CH1_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH1_CM1_tag ATOM3_CH1_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH1_CN0_tag ATOM3_CH1_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls3_ATOM3_CH1_STAT_tag ATOM3_CH1_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls3_ATOM3_CH1_IRQ_NOTIFY_tag ATOM3_CH1_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls3_ATOM3_CH1_IRQ_EN_tag ATOM3_CH1_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls3_ATOM3_CH1_IRQ_FORCINT_tag ATOM3_CH1_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls3_ATOM3_CH1_IRQ_MODE_tag ATOM3_CH1_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved3[4];
  GTM_gtm_cls3_ATOM3_CH1_CTRL_SR_tag ATOM3_CH1_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls3_reserved4[72];
  GTM_gtm_cls3_ATOM3_CH2_RDADDR_tag ATOM3_CH2_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls3_ATOM3_CH2_CTRL_tag ATOM3_CH2_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls3_ATOM3_CH2_SR0_tag ATOM3_CH2_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH2_SR1_tag ATOM3_CH2_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH2_CM0_tag ATOM3_CH2_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH2_CM1_tag ATOM3_CH2_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH2_CN0_tag ATOM3_CH2_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls3_ATOM3_CH2_STAT_tag ATOM3_CH2_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls3_ATOM3_CH2_IRQ_NOTIFY_tag ATOM3_CH2_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls3_ATOM3_CH2_IRQ_EN_tag ATOM3_CH2_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls3_ATOM3_CH2_IRQ_FORCINT_tag ATOM3_CH2_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls3_ATOM3_CH2_IRQ_MODE_tag ATOM3_CH2_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved5[4];
  GTM_gtm_cls3_ATOM3_CH2_CTRL_SR_tag ATOM3_CH2_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls3_reserved6[72];
  GTM_gtm_cls3_ATOM3_CH3_RDADDR_tag ATOM3_CH3_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls3_ATOM3_CH3_CTRL_tag ATOM3_CH3_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls3_ATOM3_CH3_SR0_tag ATOM3_CH3_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH3_SR1_tag ATOM3_CH3_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH3_CM0_tag ATOM3_CH3_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH3_CM1_tag ATOM3_CH3_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH3_CN0_tag ATOM3_CH3_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls3_ATOM3_CH3_STAT_tag ATOM3_CH3_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls3_ATOM3_CH3_IRQ_NOTIFY_tag ATOM3_CH3_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls3_ATOM3_CH3_IRQ_EN_tag ATOM3_CH3_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls3_ATOM3_CH3_IRQ_FORCINT_tag ATOM3_CH3_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls3_ATOM3_CH3_IRQ_MODE_tag ATOM3_CH3_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved7[4];
  GTM_gtm_cls3_ATOM3_CH3_CTRL_SR_tag ATOM3_CH3_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls3_reserved8[72];
  GTM_gtm_cls3_ATOM3_CH4_RDADDR_tag ATOM3_CH4_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls3_ATOM3_CH4_CTRL_tag ATOM3_CH4_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls3_ATOM3_CH4_SR0_tag ATOM3_CH4_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH4_SR1_tag ATOM3_CH4_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH4_CM0_tag ATOM3_CH4_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH4_CM1_tag ATOM3_CH4_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH4_CN0_tag ATOM3_CH4_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls3_ATOM3_CH4_STAT_tag ATOM3_CH4_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls3_ATOM3_CH4_IRQ_NOTIFY_tag ATOM3_CH4_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls3_ATOM3_CH4_IRQ_EN_tag ATOM3_CH4_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls3_ATOM3_CH4_IRQ_FORCINT_tag ATOM3_CH4_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls3_ATOM3_CH4_IRQ_MODE_tag ATOM3_CH4_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved9[4];
  GTM_gtm_cls3_ATOM3_CH4_CTRL_SR_tag ATOM3_CH4_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls3_reserved10[72];
  GTM_gtm_cls3_ATOM3_CH5_RDADDR_tag ATOM3_CH5_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls3_ATOM3_CH5_CTRL_tag ATOM3_CH5_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls3_ATOM3_CH5_SR0_tag ATOM3_CH5_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH5_SR1_tag ATOM3_CH5_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH5_CM0_tag ATOM3_CH5_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH5_CM1_tag ATOM3_CH5_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH5_CN0_tag ATOM3_CH5_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls3_ATOM3_CH5_STAT_tag ATOM3_CH5_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls3_ATOM3_CH5_IRQ_NOTIFY_tag ATOM3_CH5_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls3_ATOM3_CH5_IRQ_EN_tag ATOM3_CH5_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls3_ATOM3_CH5_IRQ_FORCINT_tag ATOM3_CH5_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls3_ATOM3_CH5_IRQ_MODE_tag ATOM3_CH5_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved11[4];
  GTM_gtm_cls3_ATOM3_CH5_CTRL_SR_tag ATOM3_CH5_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls3_reserved12[72];
  GTM_gtm_cls3_ATOM3_CH6_RDADDR_tag ATOM3_CH6_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls3_ATOM3_CH6_CTRL_tag ATOM3_CH6_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls3_ATOM3_CH6_SR0_tag ATOM3_CH6_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH6_SR1_tag ATOM3_CH6_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH6_CM0_tag ATOM3_CH6_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH6_CM1_tag ATOM3_CH6_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH6_CN0_tag ATOM3_CH6_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls3_ATOM3_CH6_STAT_tag ATOM3_CH6_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls3_ATOM3_CH6_IRQ_NOTIFY_tag ATOM3_CH6_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls3_ATOM3_CH6_IRQ_EN_tag ATOM3_CH6_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls3_ATOM3_CH6_IRQ_FORCINT_tag ATOM3_CH6_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls3_ATOM3_CH6_IRQ_MODE_tag ATOM3_CH6_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved13[4];
  GTM_gtm_cls3_ATOM3_CH6_CTRL_SR_tag ATOM3_CH6_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls3_reserved14[72];
  GTM_gtm_cls3_ATOM3_CH7_RDADDR_tag ATOM3_CH7_RDADDR; /* ATOM[i] channel[x] ARU read address register */
  GTM_gtm_cls3_ATOM3_CH7_CTRL_tag ATOM3_CH7_CTRL; /* ATOM[i] channel [x] control register */
  GTM_gtm_cls3_ATOM3_CH7_SR0_tag ATOM3_CH7_SR0; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH7_SR1_tag ATOM3_CH7_SR1; /* ATOM[i] channel [x] CCU0 compare shadow register */
  GTM_gtm_cls3_ATOM3_CH7_CM0_tag ATOM3_CH7_CM0; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH7_CM1_tag ATOM3_CH7_CM1; /* ATOM[i] channel [x] CCU0 compare register */
  GTM_gtm_cls3_ATOM3_CH7_CN0_tag ATOM3_CH7_CN0; /* ATOM[i] channel [x] CCU0 counter register */
  GTM_gtm_cls3_ATOM3_CH7_STAT_tag ATOM3_CH7_STAT; /* ATOM[i] channel [x] status register */
  GTM_gtm_cls3_ATOM3_CH7_IRQ_NOTIFY_tag ATOM3_CH7_IRQ_NOTIFY; /* ATOM[i] channel [x] interrupt notification register */
  GTM_gtm_cls3_ATOM3_CH7_IRQ_EN_tag ATOM3_CH7_IRQ_EN; /* ATOM[i] channel [x] interrupt enable register */
  GTM_gtm_cls3_ATOM3_CH7_IRQ_FORCINT_tag ATOM3_CH7_IRQ_FORCINT; /* ATOM[i] channel [x] software interrupt generation */
  GTM_gtm_cls3_ATOM3_CH7_IRQ_MODE_tag ATOM3_CH7_IRQ_MODE; /* ATOM[i] channel [x] interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved15[4];
  GTM_gtm_cls3_ATOM3_CH7_CTRL_SR_tag ATOM3_CH7_CTRL_SR; /* ATOM[i] channel [x] control shadow register */
  uint8_t GTM_gtm_cls3_reserved16[136];
  GTM_gtm_cls3_ATOM3_AGC_GLB_CTRL_tag ATOM3_AGC_GLB_CTRL; /* ATOM[i] AGC global control register */
  GTM_gtm_cls3_ATOM3_AGC_ENDIS_CTRL_tag ATOM3_AGC_ENDIS_CTRL; /* ATOM[i] AGC enable/disable control register */
  GTM_gtm_cls3_ATOM3_AGC_ENDIS_STAT_tag ATOM3_AGC_ENDIS_STAT; /* ATOM[i] AGC enable/disable status register */
  GTM_gtm_cls3_ATOM3_AGC_ACT_TB_tag ATOM3_AGC_ACT_TB; /* ATOM[i] AGC action time base register */
  GTM_gtm_cls3_ATOM3_AGC_OUTEN_CTRL_tag ATOM3_AGC_OUTEN_CTRL; /* ATOM[i] AGC output enable control register */
  GTM_gtm_cls3_ATOM3_AGC_OUTEN_STAT_tag ATOM3_AGC_OUTEN_STAT; /* ATOM[i] AGC output enable status register */
  GTM_gtm_cls3_ATOM3_AGC_FUPD_CTRL_tag ATOM3_AGC_FUPD_CTRL; /* ATOM[i] AGC force update control register */
  GTM_gtm_cls3_ATOM3_AGC_INT_TRIG_tag ATOM3_AGC_INT_TRIG; /* ATOM[i] AGC internal trigger control register */
  uint8_t GTM_gtm_cls3_reserved17[928];
  GTM_gtm_cls3_MCS3_CH0_R0_tag MCS3_CH0_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH0_R1_tag MCS3_CH0_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH0_R2_tag MCS3_CH0_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH0_R3_tag MCS3_CH0_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH0_R4_tag MCS3_CH0_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH0_R5_tag MCS3_CH0_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH0_R6_tag MCS3_CH0_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH0_R7_tag MCS3_CH0_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH0_CTRL_tag MCS3_CH0_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls3_MCS3_CH0_ACB_tag MCS3_CH0_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls3_reserved18[20];
  GTM_gtm_cls3_MCS3_CH0_MHB_tag MCS3_CH0_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls3_reserved19[160];
  GTM_gtm_cls3_MCS3_CH0_PC_tag MCS3_CH0_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls3_MCS3_CH0_IRQ_NOTIFY_tag MCS3_CH0_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls3_MCS3_CH0_IRQ_EN_tag MCS3_CH0_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls3_MCS3_CH0_IRQ_FORCINT_tag MCS3_CH0_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls3_MCS3_CH0_IRQ_MODE_tag MCS3_CH0_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls3_MCS3_CH0_EIRQ_EN_tag MCS3_CH0_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls3_reserved20[8];
  GTM_gtm_cls3_MCS3_CH1_R0_tag MCS3_CH1_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH1_R1_tag MCS3_CH1_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH1_R2_tag MCS3_CH1_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH1_R3_tag MCS3_CH1_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH1_R4_tag MCS3_CH1_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH1_R5_tag MCS3_CH1_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH1_R6_tag MCS3_CH1_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH1_R7_tag MCS3_CH1_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH1_CTRL_tag MCS3_CH1_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls3_MCS3_CH1_ACB_tag MCS3_CH1_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls3_reserved21[20];
  GTM_gtm_cls3_MCS3_CH1_MHB_tag MCS3_CH1_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls3_reserved22[160];
  GTM_gtm_cls3_MCS3_CH1_PC_tag MCS3_CH1_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls3_MCS3_CH1_IRQ_NOTIFY_tag MCS3_CH1_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls3_MCS3_CH1_IRQ_EN_tag MCS3_CH1_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls3_MCS3_CH1_IRQ_FORCINT_tag MCS3_CH1_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls3_MCS3_CH1_IRQ_MODE_tag MCS3_CH1_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls3_MCS3_CH1_EIRQ_EN_tag MCS3_CH1_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls3_reserved23[8];
  GTM_gtm_cls3_MCS3_CH2_R0_tag MCS3_CH2_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH2_R1_tag MCS3_CH2_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH2_R2_tag MCS3_CH2_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH2_R3_tag MCS3_CH2_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH2_R4_tag MCS3_CH2_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH2_R5_tag MCS3_CH2_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH2_R6_tag MCS3_CH2_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH2_R7_tag MCS3_CH2_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH2_CTRL_tag MCS3_CH2_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls3_MCS3_CH2_ACB_tag MCS3_CH2_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls3_reserved24[20];
  GTM_gtm_cls3_MCS3_CH2_MHB_tag MCS3_CH2_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls3_reserved25[160];
  GTM_gtm_cls3_MCS3_CH2_PC_tag MCS3_CH2_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls3_MCS3_CH2_IRQ_NOTIFY_tag MCS3_CH2_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls3_MCS3_CH2_IRQ_EN_tag MCS3_CH2_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls3_MCS3_CH2_IRQ_FORCINT_tag MCS3_CH2_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls3_MCS3_CH2_IRQ_MODE_tag MCS3_CH2_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls3_MCS3_CH2_EIRQ_EN_tag MCS3_CH2_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls3_reserved26[8];
  GTM_gtm_cls3_MCS3_CH3_R0_tag MCS3_CH3_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH3_R1_tag MCS3_CH3_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH3_R2_tag MCS3_CH3_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH3_R3_tag MCS3_CH3_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH3_R4_tag MCS3_CH3_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH3_R5_tag MCS3_CH3_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH3_R6_tag MCS3_CH3_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH3_R7_tag MCS3_CH3_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH3_CTRL_tag MCS3_CH3_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls3_MCS3_CH3_ACB_tag MCS3_CH3_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls3_reserved27[20];
  GTM_gtm_cls3_MCS3_CH3_MHB_tag MCS3_CH3_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls3_reserved28[160];
  GTM_gtm_cls3_MCS3_CH3_PC_tag MCS3_CH3_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls3_MCS3_CH3_IRQ_NOTIFY_tag MCS3_CH3_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls3_MCS3_CH3_IRQ_EN_tag MCS3_CH3_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls3_MCS3_CH3_IRQ_FORCINT_tag MCS3_CH3_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls3_MCS3_CH3_IRQ_MODE_tag MCS3_CH3_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls3_MCS3_CH3_EIRQ_EN_tag MCS3_CH3_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls3_reserved29[8];
  GTM_gtm_cls3_MCS3_CH4_R0_tag MCS3_CH4_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH4_R1_tag MCS3_CH4_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH4_R2_tag MCS3_CH4_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH4_R3_tag MCS3_CH4_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH4_R4_tag MCS3_CH4_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH4_R5_tag MCS3_CH4_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH4_R6_tag MCS3_CH4_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH4_R7_tag MCS3_CH4_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH4_CTRL_tag MCS3_CH4_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls3_MCS3_CH4_ACB_tag MCS3_CH4_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls3_reserved30[20];
  GTM_gtm_cls3_MCS3_CH4_MHB_tag MCS3_CH4_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls3_reserved31[160];
  GTM_gtm_cls3_MCS3_CH4_PC_tag MCS3_CH4_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls3_MCS3_CH4_IRQ_NOTIFY_tag MCS3_CH4_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls3_MCS3_CH4_IRQ_EN_tag MCS3_CH4_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls3_MCS3_CH4_IRQ_FORCINT_tag MCS3_CH4_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls3_MCS3_CH4_IRQ_MODE_tag MCS3_CH4_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls3_MCS3_CH4_EIRQ_EN_tag MCS3_CH4_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls3_reserved32[8];
  GTM_gtm_cls3_MCS3_CH5_R0_tag MCS3_CH5_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH5_R1_tag MCS3_CH5_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH5_R2_tag MCS3_CH5_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH5_R3_tag MCS3_CH5_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH5_R4_tag MCS3_CH5_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH5_R5_tag MCS3_CH5_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH5_R6_tag MCS3_CH5_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH5_R7_tag MCS3_CH5_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH5_CTRL_tag MCS3_CH5_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls3_MCS3_CH5_ACB_tag MCS3_CH5_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls3_reserved33[20];
  GTM_gtm_cls3_MCS3_CH5_MHB_tag MCS3_CH5_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls3_reserved34[160];
  GTM_gtm_cls3_MCS3_CH5_PC_tag MCS3_CH5_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls3_MCS3_CH5_IRQ_NOTIFY_tag MCS3_CH5_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls3_MCS3_CH5_IRQ_EN_tag MCS3_CH5_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls3_MCS3_CH5_IRQ_FORCINT_tag MCS3_CH5_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls3_MCS3_CH5_IRQ_MODE_tag MCS3_CH5_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls3_MCS3_CH5_EIRQ_EN_tag MCS3_CH5_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls3_reserved35[8];
  GTM_gtm_cls3_MCS3_CH6_R0_tag MCS3_CH6_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH6_R1_tag MCS3_CH6_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH6_R2_tag MCS3_CH6_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH6_R3_tag MCS3_CH6_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH6_R4_tag MCS3_CH6_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH6_R5_tag MCS3_CH6_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH6_R6_tag MCS3_CH6_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH6_R7_tag MCS3_CH6_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH6_CTRL_tag MCS3_CH6_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls3_MCS3_CH6_ACB_tag MCS3_CH6_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls3_reserved36[20];
  GTM_gtm_cls3_MCS3_CH6_MHB_tag MCS3_CH6_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls3_reserved37[160];
  GTM_gtm_cls3_MCS3_CH6_PC_tag MCS3_CH6_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls3_MCS3_CH6_IRQ_NOTIFY_tag MCS3_CH6_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls3_MCS3_CH6_IRQ_EN_tag MCS3_CH6_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls3_MCS3_CH6_IRQ_FORCINT_tag MCS3_CH6_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls3_MCS3_CH6_IRQ_MODE_tag MCS3_CH6_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls3_MCS3_CH6_EIRQ_EN_tag MCS3_CH6_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls3_reserved38[8];
  GTM_gtm_cls3_MCS3_CH7_R0_tag MCS3_CH7_R0; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH7_R1_tag MCS3_CH7_R1; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH7_R2_tag MCS3_CH7_R2; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH7_R3_tag MCS3_CH7_R3; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH7_R4_tag MCS3_CH7_R4; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH7_R5_tag MCS3_CH7_R5; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH7_R6_tag MCS3_CH7_R6; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH7_R7_tag MCS3_CH7_R7; /* MCS[i] channel x general purpose register [y] */
  GTM_gtm_cls3_MCS3_CH7_CTRL_tag MCS3_CH7_CTRL; /* MCS[i] channel x control register */
  GTM_gtm_cls3_MCS3_CH7_ACB_tag MCS3_CH7_ACB; /* MCS[i] channel x ARU control Bit register */
  uint8_t GTM_gtm_cls3_reserved39[20];
  GTM_gtm_cls3_MCS3_CH7_MHB_tag MCS3_CH7_MHB; /* MCS[i] channel x memory high byte register */
  uint8_t GTM_gtm_cls3_reserved40[160];
  GTM_gtm_cls3_MCS3_CH7_PC_tag MCS3_CH7_PC; /* MCS[i] channel x program counter register */
  GTM_gtm_cls3_MCS3_CH7_IRQ_NOTIFY_tag MCS3_CH7_IRQ_NOTIFY; /* MCS[i] channel x interrupt notification register */
  GTM_gtm_cls3_MCS3_CH7_IRQ_EN_tag MCS3_CH7_IRQ_EN; /* MCS[i] channel x interrupt enable register */
  GTM_gtm_cls3_MCS3_CH7_IRQ_FORCINT_tag MCS3_CH7_IRQ_FORCINT; /* MCS[i] channel x force interrupt register */
  GTM_gtm_cls3_MCS3_CH7_IRQ_MODE_tag MCS3_CH7_IRQ_MODE; /* MCS[i] channel x IRQ mode configuration register */
  GTM_gtm_cls3_MCS3_CH7_EIRQ_EN_tag MCS3_CH7_EIRQ_EN; /* MCS[i] channel x error interrupt enable register */
  uint8_t GTM_gtm_cls3_reserved41[1584];
  GTM_gtm_cls3_MCS3_CTRG_tag MCS3_CTRG; /* MCS[i] clear trigger control register */
  GTM_gtm_cls3_MCS3_STRG_tag MCS3_STRG; /* MCS[i] set trigger control register */
  uint8_t GTM_gtm_cls3_reserved42[208];
  GTM_gtm_cls3_MCS3_CTRL_STAT_tag MCS3_CTRL_STAT; /* MCS[i] control and status register */
  GTM_gtm_cls3_MCS3_RESET_tag MCS3_RESET; /* MCS[i] reset register */
  GTM_gtm_cls3_MCS3_CAT_tag MCS3_CAT;  /* MCS[i] cancel ARU transfer instruction */
  GTM_gtm_cls3_MCS3_CWT_tag MCS3_CWT;  /* MCS[i] cancel waiting instruction */
  GTM_gtm_cls3_MCS3_ERR_tag MCS3_ERR;  /* MCS[i] error register */
  uint8_t GTM_gtm_cls3_reserved43[8];
  GTM_gtm_cls3_MCS3_REG_PROT_tag MCS3_REG_PROT; /* MCS[i] write protection register */
  GTM_gtm_cls3_MCS3_SINT_IRQ_NOTIFY_tag MCS3_SINT_IRQ_NOTIFY; /* MCS[i] shared interrupt notification register */
  GTM_gtm_cls3_MCS3_SINT_IRQ_EN_tag MCS3_SINT_IRQ_EN; /* MCS[i] shared interrupt enable register */
  GTM_gtm_cls3_MCS3_SINT_IRQ_FORCINT_tag MCS3_SINT_IRQ_FORCINT; /* MCS[i] force shared interrupt register */
  GTM_gtm_cls3_MCS3_SINT_IRQ_MODE_tag MCS3_SINT_IRQ_MODE; /* MCS[i] shared interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved44[16];
  GTM_gtm_cls3_MCS3_HBP0_CTRL_tag MCS3_HBP0_CTRL; /* MCS[i] hardware break point h control register */
  GTM_gtm_cls3_MCS3_HBP0_PATTERN_tag MCS3_HBP0_PATTERN; /* MCS[i] hardware break point pattern register */
  GTM_gtm_cls3_MCS3_HBP0_STATUS_tag MCS3_HBP0_STATUS; /* MCS[i] hardware break point status register */
  GTM_gtm_cls3_MCS3_HBP0_IRQ_NOTIFY_tag MCS3_HBP0_IRQ_NOTIFY; /* MCS[i] hardware break point interrupt notification register */
  GTM_gtm_cls3_MCS3_HBP0_IRQ_EN_tag MCS3_HBP0_IRQ_EN; /* MCS[i] hardware break point interrupt enable register */
  GTM_gtm_cls3_MCS3_HBP0_IRQ_FORCINT_tag MCS3_HBP0_IRQ_FORCINT; /* MCS[i] force hardware break point interrupt register */
  GTM_gtm_cls3_MCS3_HBP0_IRQ_MODE_tag MCS3_HBP0_IRQ_MODE; /* MCS[i] break point h interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved45[4];
  GTM_gtm_cls3_MCS3_HBP1_CTRL_tag MCS3_HBP1_CTRL; /* MCS[i] hardware break point h control register */
  GTM_gtm_cls3_MCS3_HBP1_PATTERN_tag MCS3_HBP1_PATTERN; /* MCS[i] hardware break point pattern register */
  GTM_gtm_cls3_MCS3_HBP1_STATUS_tag MCS3_HBP1_STATUS; /* MCS[i] hardware break point status register */
  GTM_gtm_cls3_MCS3_HBP1_IRQ_NOTIFY_tag MCS3_HBP1_IRQ_NOTIFY; /* MCS[i] hardware break point interrupt notification register */
  GTM_gtm_cls3_MCS3_HBP1_IRQ_EN_tag MCS3_HBP1_IRQ_EN; /* MCS[i] hardware break point interrupt enable register */
  GTM_gtm_cls3_MCS3_HBP1_IRQ_FORCINT_tag MCS3_HBP1_IRQ_FORCINT; /* MCS[i] force hardware break point interrupt register */
  GTM_gtm_cls3_MCS3_HBP1_IRQ_MODE_tag MCS3_HBP1_IRQ_MODE; /* MCS[i] break point h interrupt mode configuration register */
  uint8_t GTM_gtm_cls3_reserved46[132];
  GTM_gtm_cls3_TIO3_G0_CH0_CTRL_tag TIO3_G0_CH0_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls3_TIO3_G0_CH0_IRQ_NOTIFY_tag TIO3_G0_CH0_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls3_TIO3_G0_CH0_IRQ_EN_tag TIO3_G0_CH0_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls3_TIO3_G0_CH0_IRQ_FORCINT_tag TIO3_G0_CH0_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls3_TIO3_G0_CH0_IRQ_MODE_tag TIO3_G0_CH0_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls3_TIO3_G0_CH0_CTRL2_tag TIO3_G0_CH0_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls3_reserved47[8];
  GTM_gtm_cls3_TIO3_G0_CH0_SINST_tag TIO3_G0_CH0_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH0_SCMD_tag TIO3_G0_CH0_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH0_SOP_tag TIO3_G0_CH0_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls3_reserved48[4];
  GTM_gtm_cls3_TIO3_G0_CH0_OINST_tag TIO3_G0_CH0_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls3_TIO3_G0_CH0_OCMD_tag TIO3_G0_CH0_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls3_TIO3_G0_CH0_OOP_tag TIO3_G0_CH0_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls3_TIO3_G0_CH0_SHIFTCNT_tag TIO3_G0_CH0_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls3_TIO3_G0_CH1_CTRL_tag TIO3_G0_CH1_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls3_TIO3_G0_CH1_IRQ_NOTIFY_tag TIO3_G0_CH1_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls3_TIO3_G0_CH1_IRQ_EN_tag TIO3_G0_CH1_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls3_TIO3_G0_CH1_IRQ_FORCINT_tag TIO3_G0_CH1_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls3_TIO3_G0_CH1_IRQ_MODE_tag TIO3_G0_CH1_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls3_TIO3_G0_CH1_CTRL2_tag TIO3_G0_CH1_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls3_reserved49[8];
  GTM_gtm_cls3_TIO3_G0_CH1_SINST_tag TIO3_G0_CH1_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH1_SCMD_tag TIO3_G0_CH1_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH1_SOP_tag TIO3_G0_CH1_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls3_reserved50[4];
  GTM_gtm_cls3_TIO3_G0_CH1_OINST_tag TIO3_G0_CH1_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls3_TIO3_G0_CH1_OCMD_tag TIO3_G0_CH1_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls3_TIO3_G0_CH1_OOP_tag TIO3_G0_CH1_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls3_TIO3_G0_CH1_SHIFTCNT_tag TIO3_G0_CH1_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls3_TIO3_G0_CH2_CTRL_tag TIO3_G0_CH2_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls3_TIO3_G0_CH2_IRQ_NOTIFY_tag TIO3_G0_CH2_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls3_TIO3_G0_CH2_IRQ_EN_tag TIO3_G0_CH2_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls3_TIO3_G0_CH2_IRQ_FORCINT_tag TIO3_G0_CH2_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls3_TIO3_G0_CH2_IRQ_MODE_tag TIO3_G0_CH2_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls3_TIO3_G0_CH2_CTRL2_tag TIO3_G0_CH2_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls3_reserved51[8];
  GTM_gtm_cls3_TIO3_G0_CH2_SINST_tag TIO3_G0_CH2_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH2_SCMD_tag TIO3_G0_CH2_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH2_SOP_tag TIO3_G0_CH2_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls3_reserved52[4];
  GTM_gtm_cls3_TIO3_G0_CH2_OINST_tag TIO3_G0_CH2_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls3_TIO3_G0_CH2_OCMD_tag TIO3_G0_CH2_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls3_TIO3_G0_CH2_OOP_tag TIO3_G0_CH2_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls3_TIO3_G0_CH2_SHIFTCNT_tag TIO3_G0_CH2_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls3_TIO3_G0_CH3_CTRL_tag TIO3_G0_CH3_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls3_TIO3_G0_CH3_IRQ_NOTIFY_tag TIO3_G0_CH3_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls3_TIO3_G0_CH3_IRQ_EN_tag TIO3_G0_CH3_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls3_TIO3_G0_CH3_IRQ_FORCINT_tag TIO3_G0_CH3_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls3_TIO3_G0_CH3_IRQ_MODE_tag TIO3_G0_CH3_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls3_TIO3_G0_CH3_CTRL2_tag TIO3_G0_CH3_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls3_reserved53[8];
  GTM_gtm_cls3_TIO3_G0_CH3_SINST_tag TIO3_G0_CH3_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH3_SCMD_tag TIO3_G0_CH3_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH3_SOP_tag TIO3_G0_CH3_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls3_reserved54[4];
  GTM_gtm_cls3_TIO3_G0_CH3_OINST_tag TIO3_G0_CH3_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls3_TIO3_G0_CH3_OCMD_tag TIO3_G0_CH3_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls3_TIO3_G0_CH3_OOP_tag TIO3_G0_CH3_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls3_TIO3_G0_CH3_SHIFTCNT_tag TIO3_G0_CH3_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls3_TIO3_G0_CH4_CTRL_tag TIO3_G0_CH4_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls3_TIO3_G0_CH4_IRQ_NOTIFY_tag TIO3_G0_CH4_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls3_TIO3_G0_CH4_IRQ_EN_tag TIO3_G0_CH4_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls3_TIO3_G0_CH4_IRQ_FORCINT_tag TIO3_G0_CH4_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls3_TIO3_G0_CH4_IRQ_MODE_tag TIO3_G0_CH4_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls3_TIO3_G0_CH4_CTRL2_tag TIO3_G0_CH4_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls3_reserved55[8];
  GTM_gtm_cls3_TIO3_G0_CH4_SINST_tag TIO3_G0_CH4_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH4_SCMD_tag TIO3_G0_CH4_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH4_SOP_tag TIO3_G0_CH4_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls3_reserved56[4];
  GTM_gtm_cls3_TIO3_G0_CH4_OINST_tag TIO3_G0_CH4_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls3_TIO3_G0_CH4_OCMD_tag TIO3_G0_CH4_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls3_TIO3_G0_CH4_OOP_tag TIO3_G0_CH4_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls3_TIO3_G0_CH4_SHIFTCNT_tag TIO3_G0_CH4_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls3_TIO3_G0_CH5_CTRL_tag TIO3_G0_CH5_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls3_TIO3_G0_CH5_IRQ_NOTIFY_tag TIO3_G0_CH5_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls3_TIO3_G0_CH5_IRQ_EN_tag TIO3_G0_CH5_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls3_TIO3_G0_CH5_IRQ_FORCINT_tag TIO3_G0_CH5_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls3_TIO3_G0_CH5_IRQ_MODE_tag TIO3_G0_CH5_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls3_TIO3_G0_CH5_CTRL2_tag TIO3_G0_CH5_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls3_reserved57[8];
  GTM_gtm_cls3_TIO3_G0_CH5_SINST_tag TIO3_G0_CH5_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH5_SCMD_tag TIO3_G0_CH5_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH5_SOP_tag TIO3_G0_CH5_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls3_reserved58[4];
  GTM_gtm_cls3_TIO3_G0_CH5_OINST_tag TIO3_G0_CH5_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls3_TIO3_G0_CH5_OCMD_tag TIO3_G0_CH5_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls3_TIO3_G0_CH5_OOP_tag TIO3_G0_CH5_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls3_TIO3_G0_CH5_SHIFTCNT_tag TIO3_G0_CH5_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls3_TIO3_G0_CH6_CTRL_tag TIO3_G0_CH6_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls3_TIO3_G0_CH6_IRQ_NOTIFY_tag TIO3_G0_CH6_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls3_TIO3_G0_CH6_IRQ_EN_tag TIO3_G0_CH6_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls3_TIO3_G0_CH6_IRQ_FORCINT_tag TIO3_G0_CH6_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls3_TIO3_G0_CH6_IRQ_MODE_tag TIO3_G0_CH6_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls3_TIO3_G0_CH6_CTRL2_tag TIO3_G0_CH6_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls3_reserved59[8];
  GTM_gtm_cls3_TIO3_G0_CH6_SINST_tag TIO3_G0_CH6_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH6_SCMD_tag TIO3_G0_CH6_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH6_SOP_tag TIO3_G0_CH6_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls3_reserved60[4];
  GTM_gtm_cls3_TIO3_G0_CH6_OINST_tag TIO3_G0_CH6_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls3_TIO3_G0_CH6_OCMD_tag TIO3_G0_CH6_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls3_TIO3_G0_CH6_OOP_tag TIO3_G0_CH6_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls3_TIO3_G0_CH6_SHIFTCNT_tag TIO3_G0_CH6_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls3_TIO3_G0_CH7_CTRL_tag TIO3_G0_CH7_CTRL; /* TIO[i] group [g] channel [c] control register */
  GTM_gtm_cls3_TIO3_G0_CH7_IRQ_NOTIFY_tag TIO3_G0_CH7_IRQ_NOTIFY; /* TIO[i] channel [c] interrupt notification register */
  GTM_gtm_cls3_TIO3_G0_CH7_IRQ_EN_tag TIO3_G0_CH7_IRQ_EN; /* TIO[i] channel [c] interrupt enable register */
  GTM_gtm_cls3_TIO3_G0_CH7_IRQ_FORCINT_tag TIO3_G0_CH7_IRQ_FORCINT; /* TIO[i] channel [c] force interrupt register */
  GTM_gtm_cls3_TIO3_G0_CH7_IRQ_MODE_tag TIO3_G0_CH7_IRQ_MODE; /* TIO[i] channel [c] IRQ mode configuration register */
  GTM_gtm_cls3_TIO3_G0_CH7_CTRL2_tag TIO3_G0_CH7_CTRL2; /* TIO[i] group [g] channel [c] control register */
  uint8_t GTM_gtm_cls3_reserved61[8];
  GTM_gtm_cls3_TIO3_G0_CH7_SINST_tag TIO3_G0_CH7_SINST; /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH7_SCMD_tag TIO3_G0_CH7_SCMD; /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
  GTM_gtm_cls3_TIO3_G0_CH7_SOP_tag TIO3_G0_CH7_SOP; /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
  uint8_t GTM_gtm_cls3_reserved62[4];
  GTM_gtm_cls3_TIO3_G0_CH7_OINST_tag TIO3_G0_CH7_OINST; /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
  GTM_gtm_cls3_TIO3_G0_CH7_OCMD_tag TIO3_G0_CH7_OCMD; /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
  GTM_gtm_cls3_TIO3_G0_CH7_OOP_tag TIO3_G0_CH7_OOP; /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
  GTM_gtm_cls3_TIO3_G0_CH7_SHIFTCNT_tag TIO3_G0_CH7_SHIFTCNT; /* TIO[i] channel [c] resource shift count register */
  GTM_gtm_cls3_TIO3_G0_ISEL0_CTRL1_tag TIO3_G0_ISEL0_CTRL1; /* TIO[i] input selection register 1 */
  GTM_gtm_cls3_TIO3_G0_ISEL0_CTRL2_tag TIO3_G0_ISEL0_CTRL2; /* TIO[i] input selection register 2 */
  uint8_t GTM_gtm_cls3_reserved63[24];
  GTM_gtm_cls3_TIO3_G0_ISEL1_CTRL1_tag TIO3_G0_ISEL1_CTRL1; /* TIO[i] input selection register 1 */
  GTM_gtm_cls3_TIO3_G0_ISEL1_CTRL2_tag TIO3_G0_ISEL1_CTRL2; /* TIO[i] input selection register 2 */
  uint8_t GTM_gtm_cls3_reserved64[24];
  GTM_gtm_cls3_TIO3_G0_OP_USAGE_tag TIO3_G0_OP_USAGE; /* TIO[i] operand usage selection register */
  uint8_t GTM_gtm_cls3_reserved65[2492];
  GTM_gtm_cls3_TIO3_S_tag TIO3_S;      /* TIO[i] signal sampling register */
  GTM_gtm_cls3_TIO3_O_tag TIO3_O;      /* TIO[i] output register */
  GTM_gtm_cls3_TIO3_ENDIS_tag TIO3_ENDIS; /* TIO[i] enable/disable register */
  GTM_gtm_cls3_TIO3_INVERT_tag TIO3_INVERT; /* TIO[i] signal invert register */
  GTM_gtm_cls3_TIO3_INPUT_MODE_tag TIO3_INPUT_MODE; /* TIO[i] input mode register */
  GTM_gtm_cls3_TIO3_CYCLIC_MODE_tag TIO3_CYCLIC_MODE; /* TIO[i] cyclic mode register */
  GTM_gtm_cls3_TIO3_TRIG_OUT_GATE_EN_tag TIO3_TRIG_OUT_GATE_EN; /* TIO[i] enable Trigger Output, output gating register */
  GTM_gtm_cls3_TIO3_PLTRIG_OUT_GATE_EN_tag TIO3_PLTRIG_OUT_GATE_EN; /* TIO[i] enable PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls3_reserved66[32];
  GTM_gtm_cls3_TIO3_CS_tag TIO3_CS;    /* TIO[i] clear signal sampling register */
  GTM_gtm_cls3_TIO3_CO_tag TIO3_CO;    /* TIO[i] clear output register */
  GTM_gtm_cls3_TIO3_CENDIS_tag TIO3_CENDIS; /* TIO[i] disable register */
  GTM_gtm_cls3_TIO3_CINVERT_tag TIO3_CINVERT; /* TIO[i] clear signal invert register */
  GTM_gtm_cls3_TIO3_CINPUT_MODE_tag TIO3_CINPUT_MODE; /* TIO[i] disable input mode register */
  GTM_gtm_cls3_TIO3_CCYCLIC_MODE_tag TIO3_CCYCLIC_MODE; /* TIO[i] disable cyclic mode register */
  GTM_gtm_cls3_TIO3_CTRIG_OUT_GATE_EN_tag TIO3_CTRIG_OUT_GATE_EN; /* TIO[i] clear Trigger Output, output gating register */
  GTM_gtm_cls3_TIO3_CPLTRIG_OUT_GATE_EN_tag TIO3_CPLTRIG_OUT_GATE_EN; /* TIO[i] clear PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls3_reserved67[32];
  GTM_gtm_cls3_TIO3_SS_tag TIO3_SS;    /* TIO[i] set signal sampling register */
  GTM_gtm_cls3_TIO3_SO_tag TIO3_SO;    /* TIO[i] set output register */
  GTM_gtm_cls3_TIO3_SENDIS_tag TIO3_SENDIS; /* TIO[i] enable register */
  GTM_gtm_cls3_TIO3_SINVERT_tag TIO3_SINVERT; /* TIO[i] set signal invert register */
  GTM_gtm_cls3_TIO3_SINPUT_MODE_tag TIO3_SINPUT_MODE; /* TIO[i] enable input mode register */
  GTM_gtm_cls3_TIO3_SCYCLIC_MODE_tag TIO3_SCYCLIC_MODE; /* TIO[i] enable cyclic mode register */
  GTM_gtm_cls3_TIO3_STRIG_OUT_GATE_EN_tag TIO3_STRIG_OUT_GATE_EN; /* TIO[i] set Trigger Output, output gating register */
  GTM_gtm_cls3_TIO3_SPLTRIG_OUT_GATE_EN_tag TIO3_SPLTRIG_OUT_GATE_EN; /* TIO[i] set PL_TRIG_OUT output gating register */
  uint8_t GTM_gtm_cls3_reserved68[32];
  GTM_gtm_cls3_TIO3_IS_tag TIO3_IS;    /* TIO[i] invert signal sampling register */
  GTM_gtm_cls3_TIO3_IO_tag TIO3_IO;    /* TIO[i] invert output register */
  GTM_gtm_cls3_TIO3_IENDIS_tag TIO3_IENDIS; /* TIO[i] toggle enable/disable register */
  GTM_gtm_cls3_TIO3_IINVERT_tag TIO3_IINVERT; /* TIO[i] toggle signal invert register */
  GTM_gtm_cls3_TIO3_IINPUT_MODE_tag TIO3_IINPUT_MODE; /* TIO[i] enable input mode register */
  GTM_gtm_cls3_TIO3_ICYCLIC_MODE_tag TIO3_ICYCLIC_MODE; /* TIO[i] enable cyclic mode register */
  uint8_t GTM_gtm_cls3_reserved69[40];
  GTM_gtm_cls3_TIO3_FUPD_tag TIO3_FUPD; /* TIO[i] force update register */
  GTM_gtm_cls3_TIO3_HW_CONF_tag TIO3_HW_CONF; /* TIO[i] configuration register */
  GTM_gtm_cls3_TIO3_RSEL_CTRL1_tag TIO3_RSEL_CTRL1; /* TIO[i] resource selection control register 1 */
  GTM_gtm_cls3_TIO3_RSEL_CTRL2_tag TIO3_RSEL_CTRL2; /* TIO[i] resource selection control register 2 */
  GTM_gtm_cls3_TIO3_PL_SWRST_tag TIO3_PL_SWRST; /* TIO[i] software reset for TIO Plus functionality */
  uint8_t GTM_gtm_cls3_reserved70[748];
  GTM_gtm_cls3_CCM3_ARP0_CTRL_tag CCM3_ARP0_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP0_PROT_tag CCM3_ARP0_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP1_CTRL_tag CCM3_ARP1_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP1_PROT_tag CCM3_ARP1_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP2_CTRL_tag CCM3_ARP2_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP2_PROT_tag CCM3_ARP2_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP3_CTRL_tag CCM3_ARP3_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP3_PROT_tag CCM3_ARP3_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP4_CTRL_tag CCM3_ARP4_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP4_PROT_tag CCM3_ARP4_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP5_CTRL_tag CCM3_ARP5_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP5_PROT_tag CCM3_ARP5_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP6_CTRL_tag CCM3_ARP6_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP6_PROT_tag CCM3_ARP6_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP7_CTRL_tag CCM3_ARP7_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP7_PROT_tag CCM3_ARP7_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP8_CTRL_tag CCM3_ARP8_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP8_PROT_tag CCM3_ARP8_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  GTM_gtm_cls3_CCM3_ARP9_CTRL_tag CCM3_ARP9_CTRL; /* CCM[i] Address Range Protector [a] Control Register */
  GTM_gtm_cls3_CCM3_ARP9_PROT_tag CCM3_ARP9_PROT; /* CCM[i] Address Range Protector [a] Protection Register */
  uint8_t GTM_gtm_cls3_reserved71[380];
  GTM_gtm_cls3_CCM3_TIO_G0_OUT_tag CCM3_TIO_G0_OUT; /* CCM[i] TIO Group 0,1 Output Register */
  uint8_t GTM_gtm_cls3_reserved72[4];
  GTM_gtm_cls3_CCM3_HW_CONF2_tag CCM3_HW_CONF2; /* CCM[i] 2. Hardware Configuration Register */
  GTM_gtm_cls3_CCM3_AEIM_STA_tag CCM3_AEIM_STA; /* CCM[i] MCS Bus Master Status Register */
  GTM_gtm_cls3_CCM3_HW_CONF_tag CCM3_HW_CONF; /* CCM[i] Hardware Configuration Register */
  uint8_t GTM_gtm_cls3_reserved73[12];
  GTM_gtm_cls3_CCM3_ATOM_OUT_tag CCM3_ATOM_OUT; /* CCM[i] ATOM Output Register */
  GTM_gtm_cls3_CCM3_CMU_CLK_CFG_tag CCM3_CMU_CLK_CFG; /* CCM[i] CMU Clock Configuration Register */
  uint8_t GTM_gtm_cls3_reserved74[4];
  GTM_gtm_cls3_CCM3_CFG_tag CCM3_CFG;  /* CCM[i] Configuration Register */
  GTM_gtm_cls3_CCM3_PROT_tag CCM3_PROT; /* CCM[i] Protection Register */
  uint8_t GTM_gtm_cls3_reserved75[768];
  GTM_gtm_cls3_CDTM3_DTM4_CTRL_tag CDTM3_DTM4_CTRL; /* CDTM[i]_DTM[d] global configuration and control register */
  GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL1_tag CDTM3_DTM4_CH_CTRL1; /* CDTM[i]_DTM[d] channel control register 1 */
  GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL2_tag CDTM3_DTM4_CH_CTRL2; /* CDTM[i]_DTM[d] channel control register 2 */
  GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL2_SR_tag CDTM3_DTM4_CH_CTRL2_SR; /* CDTM[i] DTM[j] channel control register 2 shadow */
  GTM_gtm_cls3_CDTM3_DTM4_PS_CTRL_tag CDTM3_DTM4_PS_CTRL; /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  GTM_gtm_cls3_CDTM3_DTM4_CH_DTV_tag CDTM3_DTM4_CH_DTV[4]; /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  GTM_gtm_cls3_CDTM3_DTM4_CH_SR_tag CDTM3_DTM4_CH_SR; /* CDTM[i]_DTM[d] channel shadow register */
  GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL3_tag CDTM3_DTM4_CH_CTRL3; /* CDTM[i]_DTM[d] channel control register 3 */
  GTM_gtm_cls3_CDTM3_DTM4_CTRL2_tag CDTM3_DTM4_CTRL2; /* CDTM[i]_DTM[d] global configuration and control register 2 */
  GTM_gtm_cls3_CDTM3_DTM4_CH0_DTV_SR_tag CDTM3_DTM4_CH0_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls3_CDTM3_DTM4_CH1_DTV_SR_tag CDTM3_DTM4_CH1_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls3_CDTM3_DTM4_CH2_DTV_SR_tag CDTM3_DTM4_CH2_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls3_CDTM3_DTM4_CH3_DTV_SR_tag CDTM3_DTM4_CH3_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls3_CDTM3_DTM5_CTRL_tag CDTM3_DTM5_CTRL; /* CDTM[i]_DTM[d] global configuration and control register */
  GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL1_tag CDTM3_DTM5_CH_CTRL1; /* CDTM[i]_DTM[d] channel control register 1 */
  GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL2_tag CDTM3_DTM5_CH_CTRL2; /* CDTM[i]_DTM[d] channel control register 2 */
  GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL2_SR_tag CDTM3_DTM5_CH_CTRL2_SR; /* CDTM[i] DTM[j] channel control register 2 shadow */
  GTM_gtm_cls3_CDTM3_DTM5_PS_CTRL_tag CDTM3_DTM5_PS_CTRL; /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
  GTM_gtm_cls3_CDTM3_DTM5_CH_DTV_tag CDTM3_DTM5_CH_DTV[4]; /* CDTM[i]_DTM[d] channel [x] dead time reload values */
  GTM_gtm_cls3_CDTM3_DTM5_CH_SR_tag CDTM3_DTM5_CH_SR; /* CDTM[i]_DTM[d] channel shadow register */
  GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL3_tag CDTM3_DTM5_CH_CTRL3; /* CDTM[i]_DTM[d] channel control register 3 */
  GTM_gtm_cls3_CDTM3_DTM5_CTRL2_tag CDTM3_DTM5_CTRL2; /* CDTM[i]_DTM[d] global configuration and control register 2 */
  GTM_gtm_cls3_CDTM3_DTM5_CH0_DTV_SR_tag CDTM3_DTM5_CH0_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls3_CDTM3_DTM5_CH1_DTV_SR_tag CDTM3_DTM5_CH1_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls3_CDTM3_DTM5_CH2_DTV_SR_tag CDTM3_DTM5_CH2_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  GTM_gtm_cls3_CDTM3_DTM5_CH3_DTV_SR_tag CDTM3_DTM5_CH3_DTV_SR; /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
  uint8_t GTM_gtm_cls3_reserved76[2688];
  GTM_gtm_cls3_AXIM3_FREE_tag AXIM3_FREE; /* AXIM[i] slot allocation status. */
  GTM_gtm_cls3_AXIM3_REQUEST_tag AXIM3_REQUEST; /* AXIM[i] slot request (allocation). */
  GTM_gtm_cls3_AXIM3_RELEASE_tag AXIM3_RELEASE; /* AXIM[i] slot release (de-allocation). */
  uint8_t GTM_gtm_cls3_reserved77[20];
  GTM_gtm_cls3_AXIM3_SLOT0_ADDR_LOW_tag AXIM3_SLOT0_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls3_reserved78[4];
  GTM_gtm_cls3_AXIM3_SLOT0_DATA_LOW_tag AXIM3_SLOT0_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls3_reserved79[4];
  GTM_gtm_cls3_AXIM3_SLOT0_CFG1_tag AXIM3_SLOT0_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls3_AXIM3_SLOT0_CFG2_tag AXIM3_SLOT0_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls3_AXIM3_SLOT0_STATUS_tag AXIM3_SLOT0_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls3_reserved80[4];
  GTM_gtm_cls3_AXIM3_SLOT1_ADDR_LOW_tag AXIM3_SLOT1_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls3_reserved81[4];
  GTM_gtm_cls3_AXIM3_SLOT1_DATA_LOW_tag AXIM3_SLOT1_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls3_reserved82[4];
  GTM_gtm_cls3_AXIM3_SLOT1_CFG1_tag AXIM3_SLOT1_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls3_AXIM3_SLOT1_CFG2_tag AXIM3_SLOT1_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls3_AXIM3_SLOT1_STATUS_tag AXIM3_SLOT1_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls3_reserved83[4];
  GTM_gtm_cls3_AXIM3_SLOT2_ADDR_LOW_tag AXIM3_SLOT2_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls3_reserved84[4];
  GTM_gtm_cls3_AXIM3_SLOT2_DATA_LOW_tag AXIM3_SLOT2_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls3_reserved85[4];
  GTM_gtm_cls3_AXIM3_SLOT2_CFG1_tag AXIM3_SLOT2_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls3_AXIM3_SLOT2_CFG2_tag AXIM3_SLOT2_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls3_AXIM3_SLOT2_STATUS_tag AXIM3_SLOT2_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls3_reserved86[4];
  GTM_gtm_cls3_AXIM3_SLOT3_ADDR_LOW_tag AXIM3_SLOT3_ADDR_LOW; /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls3_reserved87[4];
  GTM_gtm_cls3_AXIM3_SLOT3_DATA_LOW_tag AXIM3_SLOT3_DATA_LOW; /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
  uint8_t GTM_gtm_cls3_reserved88[4];
  GTM_gtm_cls3_AXIM3_SLOT3_CFG1_tag AXIM3_SLOT3_CFG1; /* AXIM[i] slot [s] configuration 1 */
  GTM_gtm_cls3_AXIM3_SLOT3_CFG2_tag AXIM3_SLOT3_CFG2; /* AXIM[i] slot[s] configuration 2 */
  GTM_gtm_cls3_AXIM3_SLOT3_STATUS_tag AXIM3_SLOT3_STATUS; /* AXIM[i] slot[s] status */
  uint8_t GTM_gtm_cls3_reserved89[44900];
  GTM_gtm_cls3_MCS3_MEM_tag MCS3_MEM[3072]; /* MCS[i] memory region */
};


/* ============================================================================
   =============================== Module: I3C ================================
   ============================================================================ */

typedef union I3C_MCONFIG_union_tag {  /* Controller Configuration */
  vuint32_t R;
  struct {
    vuint32_t MSTENA:2;                /* Controller Enable */
    vuint32_t _unused_2:1;
    vuint32_t DISTO:1;                 /* Disable Timeout */
    vuint32_t HKEEP:2;                 /* High-Keeper */
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t PPBAUD:4;                /* Push-Pull Baud Rate */
    vuint32_t PPLOW:4;                 /* Push-Pull Low */
    vuint32_t ODBAUD:8;                /* Open Drain Baud Rate */
    vuint32_t _unused_24:1;            /* Reserved value must not be changed. */
    vuint32_t SKEW:3;                  /* Skew */
    vuint32_t I2CBAUD:4;               /* I2C Baud Rate */
  } B;
} I3C_MCONFIG_tag;

typedef union I3C_SCONFIG_union_tag {  /* Target Configuration */
  vuint32_t R;
  struct {
    vuint32_t SLVENA:1;                /* Target Enable */
    vuint32_t NACK:1;                  /* Not Acknowledge */
    vuint32_t MATCHSS:1;               /* Match Start or Stop */
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:2;
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:2;
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t BAMATCH:6;               /* Bus Available Match */
    vuint32_t _unused_22:2;
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t SADDR:7;                 /* Static Address */
  } B;
} I3C_SCONFIG_tag;

typedef union I3C_SSTATUS_union_tag {  /* Target Status */
  vuint32_t R;
  struct {
    vuint32_t STNOTSTOP:1;             /* Status not Stop */
    vuint32_t STMSG:1;                 /* Status Message */
    vuint32_t _unused_2:1;
    vuint32_t STREQRD:1;               /* Status Request Read */
    vuint32_t STREQWR:1;               /* Status Request Write */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t START:1;                 /* Start */
    vuint32_t MATCHED:1;               /* Matched */
    vuint32_t STOP:1;                  /* Stop */
    vuint32_t RX_PEND:1;               /* Received Message Pending */
    vuint32_t TXNOTFULL:1;             /* Transmit Buffer Not Full */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t ERRWARN:1;               /* Error Warning */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:2;
    vuint32_t _unused_22:2;
    vuint32_t _unused_24:2;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:5;
  } B;
} I3C_SSTATUS_tag;

typedef union I3C_SINTSET_union_tag {  /* Target Interrupt Set */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t START:1;                 /* Start Interrupt Enable */
    vuint32_t MATCHED:1;               /* Match Interrupt Enable */
    vuint32_t STOP:1;                  /* Stop Interrupt Enable */
    vuint32_t RXPEND:1;                /* Receive Interrupt Enable */
    vuint32_t TXSEND:1;                /* Transmit Interrupt Enable */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t ERRWARN:1;               /* Error or Warning Interrupt Enable */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} I3C_SINTSET_tag;

typedef union I3C_SINTCLR_union_tag {  /* Target Interrupt Clear */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t START:1;                 /* START Interrupt Enable Clear */
    vuint32_t MATCHED:1;               /* Matched Interrupt Enable Clear */
    vuint32_t STOP:1;                  /* STOP Interrupt Enable Clear */
    vuint32_t RXPEND:1;                /* RXPEND Interrupt Enable Clear */
    vuint32_t TXSEND:1;                /* TXSEND Interrupt Enable Clear */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t ERRWARN:1;               /* ERRWARN Interrupt Enable Clear */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} I3C_SINTCLR_tag;

typedef union I3C_SINTMASKED_union_tag { /* Target Interrupt Mask */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t START:1;                 /* START Interrupt Mask */
    vuint32_t MATCHED:1;               /* MATCHED Interrupt Mask */
    vuint32_t STOP:1;                  /* STOP Interrupt Mask */
    vuint32_t RXPEND:1;                /* RXPEND Interrupt Mask */
    vuint32_t TXSEND:1;                /* TXSEND Interrupt Mask */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t ERRWARN:1;               /* ERRWARN Interrupt Mask */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:12;
  } B;
} I3C_SINTMASKED_tag;

typedef union I3C_SERRWARN_union_tag { /* Target Errors and Warnings */
  vuint32_t R;
  struct {
    vuint32_t ORUN:1;                  /* Overrun Error */
    vuint32_t URUN:1;                  /* Underrun Error */
    vuint32_t URUNNACK:1;              /* Underrun and Not Acknowledged (NACKed) Error */
    vuint32_t TERM:1;                  /* Terminated Error */
    vuint32_t INVSTART:1;              /* Invalid Start Error */
    vuint32_t _unused_5:3;
    vuint32_t SPAR:1;                  /* SDR Parity Error */
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:2;
    vuint32_t OREAD:1;                 /* Over-Read Error */
    vuint32_t OWRITE:1;                /* Over-Write Error */
    vuint32_t _unused_18:3;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:10;
  } B;
} I3C_SERRWARN_tag;

typedef union I3C_SDMACTRL_union_tag { /* Target DMA Control */
  vuint32_t R;
  struct {
    vuint32_t DMAFB:2;                 /* DMA Read (From-Bus) Trigger */
    vuint32_t DMATB:2;                 /* DMA Write (To-Bus) Trigger */
    vuint32_t DMAWIDTH:2;              /* Width of DMA Operations */
    vuint32_t _unused_6:2;
    vuint32_t _unused_8:24;
  } B;
} I3C_SDMACTRL_tag;

typedef union I3C_SDATACTRL_union_tag { /* Target Data Control */
  vuint32_t R;
  struct {
    vuint32_t FLUSHTB:1;               /* Flush To-Bus Buffer or FIFO */
    vuint32_t FLUSHFB:1;               /* Flush From-Bus Buffer or FIFO */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t UNLOCK:1;                /* Unlock */
    vuint32_t TXTRIG:2;                /* Transmit Trigger Level */
    vuint32_t RXTRIG:2;                /* Receive Trigger Level */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t TXCOUNT:5;               /* Count of Bytes in Transmit */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:2;
    vuint32_t RXCOUNT:5;               /* Count of Bytes in Receive */
    vuint32_t _unused_29:1;
    vuint32_t TXFULL:1;                /* Transmit is Full */
    vuint32_t RXEMPTY:1;               /* Receive is Empty */
  } B;
} I3C_SDATACTRL_tag;

typedef union I3C_SWDATAB_union_tag {  /* Target Write Data Byte */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Data */
    vuint32_t END:1;                   /* End */
    vuint32_t _unused_9:7;
    vuint32_t END_ALSO:1;              /* End Also */
    vuint32_t _unused_17:15;
  } B;
} I3C_SWDATAB_tag;

typedef union I3C_SWDATABE_union_tag { /* Target Write Data Byte End */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Data */
    vuint32_t _unused_8:24;
  } B;
} I3C_SWDATABE_tag;

typedef union I3C_SWDATAH_union_tag {  /* Target Write Data Halfword */
  vuint32_t R;
  struct {
    vuint32_t DATA0:8;                 /* Data 0 */
    vuint32_t DATA1:8;                 /* Data 1 */
    vuint32_t END:1;                   /* End of Message */
    vuint32_t _unused_17:15;
  } B;
} I3C_SWDATAH_tag;

typedef union I3C_SWDATAHE_union_tag { /* Target Write Data Halfword End */
  vuint32_t R;
  struct {
    vuint32_t DATA0:8;                 /* Data 0 */
    vuint32_t DATA1:8;                 /* Data 1 */
    vuint32_t _unused_16:16;
  } B;
} I3C_SWDATAHE_tag;

typedef union I3C_SRDATAB_union_tag {  /* Target Read Data Byte */
  vuint32_t R;
  struct {
    vuint32_t DATA0:8;                 /* Data 0 */
    vuint32_t _unused_8:24;
  } B;
} I3C_SRDATAB_tag;

typedef union I3C_SRDATAH_MRDATAH_MRDATAH_union_tag { /* Controller Read Data Halfword */
  vuint32_t R;
  struct {
    vuint32_t LSB:8;                   /* Low Byte */
    vuint32_t MSB:8;                   /* High Byte */
    vuint32_t _unused_16:16;
  } B;
} I3C_SRDATAH_MRDATAH_MRDATAH_tag;

typedef union I3C_SRDATAH_MRDATAH_SRDATAH_union_tag { /* Target Read Data Halfword */
  vuint32_t R;
  struct {
    vuint32_t LSB:8;                   /* Low Byte */
    vuint32_t MSB:8;                   /* High Byte */
    vuint32_t _unused_16:16;
  } B;
} I3C_SRDATAH_MRDATAH_SRDATAH_tag;

typedef union I3C_SRDATAH_MRDATAH_union_tag {
  I3C_SRDATAH_MRDATAH_MRDATAH_tag MRDATAH; /* Controller Read Data Halfword */
  I3C_SRDATAH_MRDATAH_SRDATAH_tag SRDATAH; /* Target Read Data Halfword */
} I3C_SRDATAH_MRDATAH_tag;

typedef union I3C_SWDATAB1_union_tag { /* Target Write Data Byte */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Data */
    vuint32_t _unused_8:24;
  } B;
} I3C_SWDATAB1_tag;

typedef union I3C_SCAPABILITIES2_union_tag { /* Target Capabilities 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t I2C10B:1;                /* I2C 10-bit Address */
    vuint32_t I2CRST:1;                /* I2C Software Reset */
    vuint32_t I2CDEVID:1;              /* I2C Device ID */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:2;
    vuint32_t _unused_12:2;
    vuint32_t _unused_14:2;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:2;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:8;
  } B;
} I3C_SCAPABILITIES2_tag;

typedef union I3C_SCAPABILITIES_union_tag { /* Target Capabilities */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t _unused_2:4;
    vuint32_t _unused_6:3;
    vuint32_t MASTER:1;                /* Controller */
    vuint32_t SADDR:2;                 /* Static Address */
    vuint32_t _unused_12:4;
    vuint32_t _unused_16:5;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t EXTFIFO:3;               /* External FIFO */
    vuint32_t FIFOTX:2;                /* FIFO Transmit */
    vuint32_t FIFORX:2;                /* FIFO Receive */
    vuint32_t INT:1;                   /* Interrupts */
    vuint32_t DMA:1;                   /* Direct Memory Access */
  } B;
} I3C_SCAPABILITIES_tag;

typedef union I3C_SIDEXT_union_tag {   /* Target ID Extension */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t _unused_4:3;
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:8;
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} I3C_SIDEXT_tag;

typedef union I3C_SMSGLAST_union_tag { /* Target Message Last Matched */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t LASTSTATIC:1;            /* Last Static Address Matched */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:2;
    vuint32_t _unused_8:4;
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:2;
    vuint32_t _unused_16:4;
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:2;
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} I3C_SMSGLAST_tag;

typedef union I3C_MCONFIG_EXT_union_tag { /* Controller Extended Configuration */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:2;
    vuint32_t _unused_8:4;
    vuint32_t _unused_12:4;
    vuint32_t _unused_16:4;
    vuint32_t _unused_20:12;
  } B;
} I3C_MCONFIG_EXT_tag;

typedef union I3C_MCTRL_union_tag {    /* Controller Control */
  vuint32_t R;
  struct {
    vuint32_t REQUEST:3;               /* Request */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t TYPE:2;                  /* Bus Type with EmitStartAddr */
    vuint32_t _unused_6:2;
    vuint32_t DIR:1;                   /* Direction */
    vuint32_t ADDR:7;                  /* Address */
    vuint32_t RDTERM:8;                /* Read Terminate Counter */
    vuint32_t _unused_24:8;
  } B;
} I3C_MCTRL_tag;

typedef union I3C_MSTATUS_union_tag {  /* Controller Status */
  vuint32_t R;
  struct {
    vuint32_t STATE:3;                 /* State of the Controller */
    vuint32_t _unused_3:1;
    vuint32_t BETWEEN:1;               /* Between */
    vuint32_t NACKED:1;                /* Not Acknowledged */
    vuint32_t _unused_6:2;
    vuint32_t _unused_8:1;
    vuint32_t MCTRLDONE:1;             /* Controller Control Done */
    vuint32_t COMPLETE:1;              /* Complete */
    vuint32_t RXPEND:1;                /* RXPEND */
    vuint32_t TXNOTFULL:1;             /* TX Buffer or FIFO Not Full */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t ERRWARN:1;               /* Error or Warning */
    vuint32_t _unused_16:3;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:4;
    vuint32_t _unused_24:7;
    vuint32_t _unused_31:1;
  } B;
} I3C_MSTATUS_tag;

typedef union I3C_MINTSET_union_tag {  /* Controller Interrupt Set */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t _unused_8:1;
    vuint32_t MCTRLDONE:1;             /* Controller Control Done Interrupt Enable */
    vuint32_t COMPLETE:1;              /* Completed Message Interrupt Enable */
    vuint32_t RXPEND:1;                /* Receive Pending Interrupt Enable */
    vuint32_t TXNOTFULL:1;             /* Transmit Buffer/FIFO Not Full Interrupt Enable */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t ERRWARN:1;               /* Error or Warning (ERRWARN) Interrupt Enable */
    vuint32_t _unused_16:3;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:12;
  } B;
} I3C_MINTSET_tag;

typedef union I3C_MINTCLR_union_tag {  /* Controller Interrupt Clear */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t _unused_8:1;
    vuint32_t MCTRLDONE:1;             /* MCTRLDONE Interrupt Enable Clear */
    vuint32_t COMPLETE:1;              /* COMPLETE Interrupt Enable Clear */
    vuint32_t RXPEND:1;                /* RXPEND Interrupt Enable Clear */
    vuint32_t TXNOTFULL:1;             /* TXNOTFULL Interrupt Enable Clear */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t ERRWARN:1;               /* ERRWARN Interrupt Enable Clear */
    vuint32_t _unused_16:3;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:12;
  } B;
} I3C_MINTCLR_tag;

typedef union I3C_MINTMASKED_union_tag { /* Controller Interrupt Mask */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t _unused_8:1;
    vuint32_t MCTRLDONE:1;             /* MCTRLDONE Interrupt Mask */
    vuint32_t COMPLETE:1;              /* COMPLETE Interrupt Mask */
    vuint32_t RXPEND:1;                /* RXPEND Interrupt Mask */
    vuint32_t TXNOTFULL:1;             /* TXNOTFULL Interrupt Mask */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t ERRWARN:1;               /* ERRWARN Interrupt Mask */
    vuint32_t _unused_16:3;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:12;
  } B;
} I3C_MINTMASKED_tag;

typedef union I3C_MERRWARN_union_tag { /* Controller Errors and Warnings */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t NACK:1;                  /* Not Acknowledge Error */
    vuint32_t WRABT:1;                 /* Write Abort Error */
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:4;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:3;
    vuint32_t OREAD:1;                 /* Overread Error */
    vuint32_t OWRITE:1;                /* Overwrite Error */
    vuint32_t MSGERR:1;                /* Message Error */
    vuint32_t _unused_19:1;
    vuint32_t TIMEOUT:1;               /* Timeout Error */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:10;
  } B;
} I3C_MERRWARN_tag;

typedef union I3C_MDMACTRL_union_tag { /* Controller DMA Control */
  vuint32_t R;
  struct {
    vuint32_t DMAFB:2;                 /* DMA from Bus */
    vuint32_t DMATB:2;                 /* DMA to Bus */
    vuint32_t DMAWIDTH:2;              /* DMA Width */
    vuint32_t _unused_6:2;
    vuint32_t _unused_8:24;
  } B;
} I3C_MDMACTRL_tag;

typedef union I3C_MDATACTRL_union_tag { /* Controller Data Control */
  vuint32_t R;
  struct {
    vuint32_t FLUSHTB:1;               /* Flush To-Bus Buffer or FIFO */
    vuint32_t FLUSHFB:1;               /* Flush From-Bus Buffer or FIFO */
    vuint32_t _unused_2:1;
    vuint32_t UNLOCK:1;                /* Unlock */
    vuint32_t TXTRIG:2;                /* Transmit Trigger Level */
    vuint32_t RXTRIG:2;                /* Receive Trigger Level */
    vuint32_t _unused_8:8;
    vuint32_t TXCOUNT:5;               /* Transmit Byte Count */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:2;
    vuint32_t RXCOUNT:5;               /* Receive Byte Count */
    vuint32_t _unused_29:1;
    vuint32_t TXFULL:1;                /* Transmit is Full */
    vuint32_t RXEMPTY:1;               /* Receive is Empty */
  } B;
} I3C_MDATACTRL_tag;

typedef union I3C_MWDATAB_union_tag {  /* Controller Write Data Byte */
  vuint32_t R;
  struct {
    vuint32_t VALUE:8;                 /* Data Byte */
    vuint32_t END:1;                   /* End of Message */
    vuint32_t _unused_9:7;
    vuint32_t END_ALSO:1;              /* End of Message ALSO */
    vuint32_t _unused_17:15;
  } B;
} I3C_MWDATAB_tag;

typedef union I3C_MWDATABE_union_tag { /* Controller Write Data Byte End */
  vuint32_t R;
  struct {
    vuint32_t VALUE:8;                 /* Data */
    vuint32_t _unused_8:24;
  } B;
} I3C_MWDATABE_tag;

typedef union I3C_MWDATAH_union_tag {  /* Controller Write Data Halfword */
  vuint32_t R;
  struct {
    vuint32_t DATA0:8;                 /* Data Byte 0 */
    vuint32_t DATA1:8;                 /* Data Byte 1 */
    vuint32_t END:1;                   /* End of Message */
    vuint32_t _unused_17:15;
  } B;
} I3C_MWDATAH_tag;

typedef union I3C_MWDATAHE_union_tag { /* Controller Write Data Halfword End */
  vuint32_t R;
  struct {
    vuint32_t DATA0:8;                 /* Data Byte 0 */
    vuint32_t DATA1:8;                 /* Data Byte 1 */
    vuint32_t _unused_16:16;
  } B;
} I3C_MWDATAHE_tag;

typedef union I3C_MRDATAB_union_tag {  /* Controller Read Data Byte */
  vuint32_t R;
  struct {
    vuint32_t VALUE:8;                 /* Value */
    vuint32_t _unused_8:24;
  } B;
} I3C_MRDATAB_tag;

typedef union I3C_MWDATAB1_union_tag { /* Controller Write Byte Data 1 (to Bus) */
  vuint32_t R;
  struct {
    vuint32_t VALUE:8;                 /* Value */
    vuint32_t _unused_8:24;
  } B;
} I3C_MWDATAB1_tag;

typedef union I3C_MWMSG_SDR_MWMSG_SDR_CONTROL_union_tag { /* Controller Write Message Control in SDR mode */
  vuint32_t R;
  struct {
    vuint32_t DIR:1;                   /* Direction */
    vuint32_t ADDR:7;                  /* Address */
    vuint32_t END:1;                   /* End of SDR Message */
    vuint32_t _unused_9:1;
    vuint32_t I2C:1;                   /* I2C */
    vuint32_t LEN:5;                   /* Length */
    vuint32_t _unused_16:16;
  } B;
} I3C_MWMSG_SDR_MWMSG_SDR_CONTROL_tag;

typedef union I3C_MWMSG_SDR_MWMSG_SDR_DATA_union_tag { /* Controller Write Message Data in SDR mode */
  vuint32_t R;
  struct {
    vuint32_t DATA16B:16;              /* Data */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:15;
  } B;
} I3C_MWMSG_SDR_MWMSG_SDR_DATA_tag;

typedef union I3C_MWMSG_SDR_union_tag {
  I3C_MWMSG_SDR_MWMSG_SDR_CONTROL_tag MWMSG_SDR_CONTROL; /* Controller Write Message Control in SDR mode */
  I3C_MWMSG_SDR_MWMSG_SDR_DATA_tag MWMSG_SDR_DATA; /* Controller Write Message Data in SDR mode */
} I3C_MWMSG_SDR_tag;

typedef union I3C_MRMSG_SDR_union_tag { /* Controller Read Message in SDR mode */
  vuint32_t R;
  struct {
    vuint32_t DATA:16;                 /* Data */
    vuint32_t _unused_16:16;
  } B;
} I3C_MRMSG_SDR_tag;

typedef union I3C_SMAPCTRL1_union_tag { /* Map Feature Control 1 */
  vuint32_t R;
  struct {
    vuint32_t ENA:1;                   /* Enable */
    vuint32_t ADDR:7;                  /* Address */
    vuint32_t MAPSA:1;                 /* MAP Static Address */
    vuint32_t _unused_9:3;
    vuint32_t NACK:1;                  /* Not Acknowledged */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:10;
    vuint32_t _unused_24:8;
  } B;
} I3C_SMAPCTRL1_tag;

typedef union I3C_SELFRESET_union_tag { /* Self Reset */
  vuint32_t R;
  struct {
    vuint32_t RST:1;                   /* Reset */
    vuint32_t _unused_1:7;
    vuint32_t KEY:24;                  /* Key */
  } B;
} I3C_SELFRESET_tag;

struct I3C_tag {
  I3C_MCONFIG_tag MCONFIG;             /* Controller Configuration */
  I3C_SCONFIG_tag SCONFIG;             /* Target Configuration */
  I3C_SSTATUS_tag SSTATUS;             /* Target Status */
  uint8_t I3C_reserved0[4];
  I3C_SINTSET_tag SINTSET;             /* Target Interrupt Set */
  I3C_SINTCLR_tag SINTCLR;             /* Target Interrupt Clear */
  I3C_SINTMASKED_tag SINTMASKED;       /* Target Interrupt Mask */
  I3C_SERRWARN_tag SERRWARN;           /* Target Errors and Warnings */
  I3C_SDMACTRL_tag SDMACTRL;           /* Target DMA Control */
  uint8_t I3C_reserved1[8];
  I3C_SDATACTRL_tag SDATACTRL;         /* Target Data Control */
  I3C_SWDATAB_tag SWDATAB;             /* Target Write Data Byte */
  I3C_SWDATABE_tag SWDATABE;           /* Target Write Data Byte End */
  I3C_SWDATAH_tag SWDATAH;             /* Target Write Data Halfword */
  I3C_SWDATAHE_tag SWDATAHE;           /* Target Write Data Halfword End */
  I3C_SRDATAB_tag SRDATAB;             /* Target Read Data Byte */
  uint8_t I3C_reserved2[4];
  I3C_SRDATAH_MRDATAH_tag SRDATAH_MRDATAH;
  uint8_t I3C_reserved3[8];
  I3C_SWDATAB1_tag SWDATAB1;           /* Target Write Data Byte */
  uint8_t I3C_reserved4[4];
  I3C_SCAPABILITIES2_tag SCAPABILITIES2; /* Target Capabilities 2 */
  I3C_SCAPABILITIES_tag SCAPABILITIES; /* Target Capabilities */
  uint8_t I3C_reserved5[12];
  I3C_SIDEXT_tag SIDEXT;               /* Target ID Extension */
  uint8_t I3C_reserved6[8];
  I3C_SMSGLAST_tag SMSGLAST;           /* Target Message Last Matched */
  I3C_MCONFIG_EXT_tag MCONFIG_EXT;     /* Controller Extended Configuration */
  I3C_MCTRL_tag MCTRL;                 /* Controller Control */
  I3C_MSTATUS_tag MSTATUS;             /* Controller Status */
  uint8_t I3C_reserved7[4];
  I3C_MINTSET_tag MINTSET;             /* Controller Interrupt Set */
  I3C_MINTCLR_tag MINTCLR;             /* Controller Interrupt Clear */
  I3C_MINTMASKED_tag MINTMASKED;       /* Controller Interrupt Mask */
  I3C_MERRWARN_tag MERRWARN;           /* Controller Errors and Warnings */
  I3C_MDMACTRL_tag MDMACTRL;           /* Controller DMA Control */
  uint8_t I3C_reserved8[8];
  I3C_MDATACTRL_tag MDATACTRL;         /* Controller Data Control */
  I3C_MWDATAB_tag MWDATAB;             /* Controller Write Data Byte */
  I3C_MWDATABE_tag MWDATABE;           /* Controller Write Data Byte End */
  I3C_MWDATAH_tag MWDATAH;             /* Controller Write Data Halfword */
  I3C_MWDATAHE_tag MWDATAHE;           /* Controller Write Data Halfword End */
  I3C_MRDATAB_tag MRDATAB;             /* Controller Read Data Byte */
  uint8_t I3C_reserved9[8];
  I3C_MWDATAB1_tag MWDATAB1;           /* Controller Write Byte Data 1 (to Bus) */
  I3C_MWMSG_SDR_tag MWMSG_SDR;
  I3C_MRMSG_SDR_tag MRMSG_SDR;         /* Controller Read Message in SDR mode */
  uint8_t I3C_reserved10[72];
  I3C_SMAPCTRL1_tag SMAPCTRL1;         /* Map Feature Control 1 */
  uint8_t I3C_reserved11[3788];
  I3C_SELFRESET_tag SELFRESET;         /* Self Reset */
};


/* ============================================================================
   =============================== Module: INTERFACE_CONFIGURATION ============
   ============================================================================ */

typedef union INTERFACE_CONFIGURATION_D_PORT_RID_union_tag { /* DMSS AXI Read ID Control */
  vuint32_t R;
  struct {
    vuint32_t LS0_ID:4;
    vuint32_t LS1_ID:4;
    vuint32_t _unused_8:8;
    vuint32_t DDMA_ID:4;
    vuint32_t QMAN_ID:4;
    vuint32_t _unused_24:8;
  } B;
} INTERFACE_CONFIGURATION_D_PORT_RID_tag;

typedef union INTERFACE_CONFIGURATION_D_PORT_WID_union_tag { /* DMSS AXI Port Write ID Control */
  vuint32_t R;
  struct {
    vuint32_t WB_ID:4;
    vuint32_t WBU_ID:4;
    vuint32_t _unused_8:8;
    vuint32_t DDMA_N_WID:4;
    vuint32_t QMAN_WID:4;
    vuint32_t _unused_24:8;
  } B;
} INTERFACE_CONFIGURATION_D_PORT_WID_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD0_START_union_tag { /* DMSS ADD0 START */
  vuint32_t R;
  struct {
    vuint32_t REGION_START:20;
    vuint32_t REGION_MID:8;
    vuint32_t _unused_28:1;
    vuint32_t DPRAW:1;
    vuint32_t _unused_30:2;
  } B;
} INTERFACE_CONFIGURATION_D_ADD0_START_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD0_ATT0_union_tag { /* DMSS ADD0 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MOM:1;
    vuint32_t AP:3;
    vuint32_t _unused_20:4;
    vuint32_t RQOS:4;
    vuint32_t WQOS:4;
  } B;
} INTERFACE_CONFIGURATION_D_ADD0_ATT0_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD0_ATT1_union_tag { /* DMSS ADD0 ATT1 */
  vuint32_t R;
  struct {
    vuint32_t EXACT_RD:1;
    vuint32_t L2A_WR:4;
    vuint32_t L2A_RD:4;
    vuint32_t DABSZ:4;
    vuint32_t DADOL:4;
    vuint32_t _unused_17:1;
    vuint32_t DAUOL:3;
    vuint32_t _unused_21:11;
  } B;
} INTERFACE_CONFIGURATION_D_ADD0_ATT1_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD1_START_union_tag { /* DMSS ADD1 START */
  vuint32_t R;
  struct {
    vuint32_t REGION_START:20;
    vuint32_t REGION_MID:8;
    vuint32_t INACTIVE:1;
    vuint32_t DPRAW:1;
    vuint32_t _unused_30:2;
  } B;
} INTERFACE_CONFIGURATION_D_ADD1_START_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD1_ATT0_union_tag { /* DMSS ADD1 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MOM:1;
    vuint32_t AP:3;
    vuint32_t _unused_20:4;
    vuint32_t RQOS:4;
    vuint32_t WQOS:4;
  } B;
} INTERFACE_CONFIGURATION_D_ADD1_ATT0_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD1_ATT1_union_tag { /* DMSS ADD1 ATT1 */
  vuint32_t R;
  struct {
    vuint32_t EXACT_RD:1;
    vuint32_t L2A_WR:4;
    vuint32_t L2A_RD:4;
    vuint32_t DABSZ:4;
    vuint32_t DADOL:4;
    vuint32_t _unused_17:1;
    vuint32_t DAUOL:3;
    vuint32_t _unused_21:11;
  } B;
} INTERFACE_CONFIGURATION_D_ADD1_ATT1_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD2_START_union_tag { /* DMSS ADD2 START */
  vuint32_t R;
  struct {
    vuint32_t REGION_START:20;
    vuint32_t REGION_MID:8;
    vuint32_t INACTIVE:1;
    vuint32_t DPRAW:1;
    vuint32_t _unused_30:2;
  } B;
} INTERFACE_CONFIGURATION_D_ADD2_START_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD2_ATT0_union_tag { /* DMSS ADD2 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MOM:1;
    vuint32_t AP:3;
    vuint32_t _unused_20:4;
    vuint32_t RQOS:4;
    vuint32_t WQOS:4;
  } B;
} INTERFACE_CONFIGURATION_D_ADD2_ATT0_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD2_ATT1_union_tag { /* DMSS ADD2 ATT1 */
  vuint32_t R;
  struct {
    vuint32_t EXACT_RD:1;
    vuint32_t L2A_WR:4;
    vuint32_t L2A_RD:4;
    vuint32_t DABSZ:4;
    vuint32_t DADOL:4;
    vuint32_t _unused_17:1;
    vuint32_t DAUOL:3;
    vuint32_t _unused_21:11;
  } B;
} INTERFACE_CONFIGURATION_D_ADD2_ATT1_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD3_START_union_tag { /* DMSS ADD3 START */
  vuint32_t R;
  struct {
    vuint32_t REGION_START:20;
    vuint32_t REGION_MID:8;
    vuint32_t INACTIVE:1;
    vuint32_t DPRAW:1;
    vuint32_t _unused_30:2;
  } B;
} INTERFACE_CONFIGURATION_D_ADD3_START_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD3_ATT0_union_tag { /* DMSS ADD3 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MOM:1;
    vuint32_t AP:3;
    vuint32_t _unused_20:4;
    vuint32_t RQOS:4;
    vuint32_t WQOS:4;
  } B;
} INTERFACE_CONFIGURATION_D_ADD3_ATT0_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD3_ATT1_union_tag { /* DMSS ADD3 ATT1 */
  vuint32_t R;
  struct {
    vuint32_t EXACT_RD:1;
    vuint32_t L2A_WR:4;
    vuint32_t L2A_RD:4;
    vuint32_t DABSZ:4;
    vuint32_t DADOL:4;
    vuint32_t _unused_17:1;
    vuint32_t DAUOL:3;
    vuint32_t _unused_21:11;
  } B;
} INTERFACE_CONFIGURATION_D_ADD3_ATT1_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD4_START_union_tag { /* DMSS ADD4 START */
  vuint32_t R;
  struct {
    vuint32_t REGION_START:20;
    vuint32_t REGION_MID:8;
    vuint32_t INACTIVE:1;
    vuint32_t DPRAW:1;
    vuint32_t _unused_30:2;
  } B;
} INTERFACE_CONFIGURATION_D_ADD4_START_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD4_ATT0_union_tag { /* DMSS ADD4 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MOM:1;
    vuint32_t AP:3;
    vuint32_t _unused_20:4;
    vuint32_t RQOS:4;
    vuint32_t WQOS:4;
  } B;
} INTERFACE_CONFIGURATION_D_ADD4_ATT0_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD4_ATT1_union_tag { /* DMSS ADD4 ATT1 */
  vuint32_t R;
  struct {
    vuint32_t EXACT_RD:1;
    vuint32_t L2A_WR:4;
    vuint32_t L2A_RD:4;
    vuint32_t DABSZ:4;
    vuint32_t DADOL:4;
    vuint32_t _unused_17:1;
    vuint32_t DAUOL:3;
    vuint32_t _unused_21:11;
  } B;
} INTERFACE_CONFIGURATION_D_ADD4_ATT1_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD5_START_union_tag { /* DMSS ADD5 START */
  vuint32_t R;
  struct {
    vuint32_t REGION_START:20;
    vuint32_t REGION_MID:8;
    vuint32_t INACTIVE:1;
    vuint32_t DPRAW:1;
    vuint32_t _unused_30:2;
  } B;
} INTERFACE_CONFIGURATION_D_ADD5_START_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD5_ATT0_union_tag { /* DMSS ADD5 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MOM:1;
    vuint32_t AP:3;
    vuint32_t _unused_20:4;
    vuint32_t RQOS:4;
    vuint32_t WQOS:4;
  } B;
} INTERFACE_CONFIGURATION_D_ADD5_ATT0_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD5_ATT1_union_tag { /* DMSS ADD5 ATT1 */
  vuint32_t R;
  struct {
    vuint32_t EXACT_RD:1;
    vuint32_t L2A_WR:4;
    vuint32_t L2A_RD:4;
    vuint32_t DABSZ:4;
    vuint32_t DADOL:4;
    vuint32_t _unused_17:1;
    vuint32_t DAUOL:3;
    vuint32_t _unused_21:11;
  } B;
} INTERFACE_CONFIGURATION_D_ADD5_ATT1_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD6_START_union_tag { /* DMSS ADD6 START */
  vuint32_t R;
  struct {
    vuint32_t REGION_START:20;
    vuint32_t REGION_MID:8;
    vuint32_t INACTIVE:1;
    vuint32_t DPRAW:1;
    vuint32_t _unused_30:2;
  } B;
} INTERFACE_CONFIGURATION_D_ADD6_START_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD6_ATT0_union_tag { /* DMSS ADD6 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MOM:1;
    vuint32_t AP:3;
    vuint32_t _unused_20:4;
    vuint32_t RQOS:4;
    vuint32_t WQOS:4;
  } B;
} INTERFACE_CONFIGURATION_D_ADD6_ATT0_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD6_ATT1_union_tag { /* DMSS ADD6 ATT1 */
  vuint32_t R;
  struct {
    vuint32_t EXACT_RD:1;
    vuint32_t L2A_WR:4;
    vuint32_t L2A_RD:4;
    vuint32_t DABSZ:4;
    vuint32_t DADOL:4;
    vuint32_t _unused_17:1;
    vuint32_t DAUOL:3;
    vuint32_t _unused_21:11;
  } B;
} INTERFACE_CONFIGURATION_D_ADD6_ATT1_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD7_START_union_tag { /* DMSS ADD7 START */
  vuint32_t R;
  struct {
    vuint32_t REGION_START:20;
    vuint32_t REGION_MID:8;
    vuint32_t INACTIVE:1;
    vuint32_t DPRAW:1;
    vuint32_t _unused_30:2;
  } B;
} INTERFACE_CONFIGURATION_D_ADD7_START_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD7_ATT0_union_tag { /* DMSS ADD7 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MOM:1;
    vuint32_t AP:3;
    vuint32_t _unused_20:4;
    vuint32_t RQOS:4;
    vuint32_t WQOS:4;
  } B;
} INTERFACE_CONFIGURATION_D_ADD7_ATT0_tag;

typedef union INTERFACE_CONFIGURATION_D_ADD7_ATT1_union_tag { /* DMSS ADD7 ATT1 */
  vuint32_t R;
  struct {
    vuint32_t EXACT_RD:1;
    vuint32_t L2A_WR:4;
    vuint32_t L2A_RD:4;
    vuint32_t DABSZ:4;
    vuint32_t DADOL:4;
    vuint32_t _unused_17:1;
    vuint32_t DAUOL:3;
    vuint32_t _unused_21:11;
  } B;
} INTERFACE_CONFIGURATION_D_ADD7_ATT1_tag;

struct INTERFACE_CONFIGURATION_tag {
  INTERFACE_CONFIGURATION_D_PORT_RID_tag D_PORT_RID; /* DMSS AXI Read ID Control */
  INTERFACE_CONFIGURATION_D_PORT_WID_tag D_PORT_WID; /* DMSS AXI Port Write ID Control */
  uint8_t INTERFACE_CONFIGURATION_reserved0[24];
  INTERFACE_CONFIGURATION_D_ADD0_START_tag D_ADD0_START; /* DMSS ADD0 START */
  INTERFACE_CONFIGURATION_D_ADD0_ATT0_tag D_ADD0_ATT0; /* DMSS ADD0 ATT0 */
  INTERFACE_CONFIGURATION_D_ADD0_ATT1_tag D_ADD0_ATT1; /* DMSS ADD0 ATT1 */
  uint8_t INTERFACE_CONFIGURATION_reserved1[4];
  INTERFACE_CONFIGURATION_D_ADD1_START_tag D_ADD1_START; /* DMSS ADD1 START */
  INTERFACE_CONFIGURATION_D_ADD1_ATT0_tag D_ADD1_ATT0; /* DMSS ADD1 ATT0 */
  INTERFACE_CONFIGURATION_D_ADD1_ATT1_tag D_ADD1_ATT1; /* DMSS ADD1 ATT1 */
  uint8_t INTERFACE_CONFIGURATION_reserved2[4];
  INTERFACE_CONFIGURATION_D_ADD2_START_tag D_ADD2_START; /* DMSS ADD2 START */
  INTERFACE_CONFIGURATION_D_ADD2_ATT0_tag D_ADD2_ATT0; /* DMSS ADD2 ATT0 */
  INTERFACE_CONFIGURATION_D_ADD2_ATT1_tag D_ADD2_ATT1; /* DMSS ADD2 ATT1 */
  uint8_t INTERFACE_CONFIGURATION_reserved3[4];
  INTERFACE_CONFIGURATION_D_ADD3_START_tag D_ADD3_START; /* DMSS ADD3 START */
  INTERFACE_CONFIGURATION_D_ADD3_ATT0_tag D_ADD3_ATT0; /* DMSS ADD3 ATT0 */
  INTERFACE_CONFIGURATION_D_ADD3_ATT1_tag D_ADD3_ATT1; /* DMSS ADD3 ATT1 */
  uint8_t INTERFACE_CONFIGURATION_reserved4[4];
  INTERFACE_CONFIGURATION_D_ADD4_START_tag D_ADD4_START; /* DMSS ADD4 START */
  INTERFACE_CONFIGURATION_D_ADD4_ATT0_tag D_ADD4_ATT0; /* DMSS ADD4 ATT0 */
  INTERFACE_CONFIGURATION_D_ADD4_ATT1_tag D_ADD4_ATT1; /* DMSS ADD4 ATT1 */
  uint8_t INTERFACE_CONFIGURATION_reserved5[4];
  INTERFACE_CONFIGURATION_D_ADD5_START_tag D_ADD5_START; /* DMSS ADD5 START */
  INTERFACE_CONFIGURATION_D_ADD5_ATT0_tag D_ADD5_ATT0; /* DMSS ADD5 ATT0 */
  INTERFACE_CONFIGURATION_D_ADD5_ATT1_tag D_ADD5_ATT1; /* DMSS ADD5 ATT1 */
  uint8_t INTERFACE_CONFIGURATION_reserved6[4];
  INTERFACE_CONFIGURATION_D_ADD6_START_tag D_ADD6_START; /* DMSS ADD6 START */
  INTERFACE_CONFIGURATION_D_ADD6_ATT0_tag D_ADD6_ATT0; /* DMSS ADD6 ATT0 */
  INTERFACE_CONFIGURATION_D_ADD6_ATT1_tag D_ADD6_ATT1; /* DMSS ADD6 ATT1 */
  uint8_t INTERFACE_CONFIGURATION_reserved7[4];
  INTERFACE_CONFIGURATION_D_ADD7_START_tag D_ADD7_START; /* DMSS ADD7 START */
  INTERFACE_CONFIGURATION_D_ADD7_ATT0_tag D_ADD7_ATT0; /* DMSS ADD7 ATT0 */
  INTERFACE_CONFIGURATION_D_ADD7_ATT1_tag D_ADD7_ATT1; /* DMSS ADD7 ATT1 */
};


/* ============================================================================
   =============================== Module: INTERRUPT_CONTROL ==================
   ============================================================================ */

typedef union INTERRUPT_CONTROL_ISRX_union_tag { /* Interrupt Status Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:32;
  } B;
} INTERRUPT_CONTROL_ISRX_tag;

typedef union INTERRUPT_CONTROL_ICRX_union_tag { /* Interrupt Clear Register */
  vuint32_t R;
  struct {
    vuint32_t ICR:32;
  } B;
} INTERRUPT_CONTROL_ICRX_tag;

typedef union INTERRUPT_CONTROL_IGRX_union_tag { /* Interrupt Generation Register */
  vuint32_t R;
  struct {
    vuint32_t IGR:32;
  } B;
} INTERRUPT_CONTROL_IGRX_tag;

typedef union INTERRUPT_CONTROL_IMR0_union_tag { /* Interrupt Mask Register */
  vuint32_t R;
  struct {
    vuint32_t IMR_0:1;
    vuint32_t IMR0L:4;
    vuint32_t IMR0_5:1;
    vuint32_t IMR0H:26;
  } B;
} INTERRUPT_CONTROL_IMR0_tag;

typedef union INTERRUPT_CONTROL_IGMR_union_tag { /* IGM Register */
  vuint32_t R;
  struct {
    vuint32_t GM0:1;
    vuint32_t _unused_1:15;
    vuint32_t CIL:1;
    vuint32_t CILP:1;
    vuint32_t _unused_18:13;
    vuint32_t ICUE:1;
  } B;
} INTERRUPT_CONTROL_IGMR_tag;

typedef union INTERRUPT_CONTROL_IVAX_union_tag { /* Interrupt Vector Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t IVAX:24;
  } B;
} INTERRUPT_CONTROL_IVAX_tag;

typedef union INTERRUPT_CONTROL_ICFG0_union_tag { /* ICFG0 Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFG0_tag;

typedef union INTERRUPT_CONTROL_ICFG1_union_tag { /* ICFG1 Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFG1_tag;

typedef union INTERRUPT_CONTROL_ICFG2_union_tag { /* ICFG2 Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFG2_tag;

typedef union INTERRUPT_CONTROL_ICFG3_union_tag { /* ICFG3 Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFG3_tag;

typedef union INTERRUPT_CONTROL_ICFG4_union_tag { /* ICFG4 Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFG4_tag;

typedef union INTERRUPT_CONTROL_ICFG5_union_tag { /* ICFG5 Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFG5_tag;

typedef union INTERRUPT_CONTROL_ICFG6_union_tag { /* ICFG6 Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFG6_tag;

typedef union INTERRUPT_CONTROL_ICFG8_union_tag { /* ICFG8 Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFG8_tag;

typedef union INTERRUPT_CONTROL_ICFGX_union_tag { /* ICFGx Register */
  vuint32_t R;
  struct {
    vuint32_t ISR:1;
    vuint32_t IMR:1;
    vuint32_t IMOD:2;
    vuint32_t _unused_4:2;
    vuint32_t IVI:3;
    vuint32_t _unused_9:3;
    vuint32_t IPR:3;
    vuint32_t _unused_15:17;
  } B;
} INTERRUPT_CONTROL_ICFGX_tag;

struct INTERRUPT_CONTROL_tag {
  INTERRUPT_CONTROL_ISRX_tag ISRX[1];  /* Interrupt Status Register */
  uint8_t INTERRUPT_CONTROL_reserved0[28];
  INTERRUPT_CONTROL_ICRX_tag ICRX[1];  /* Interrupt Clear Register */
  uint8_t INTERRUPT_CONTROL_reserved1[28];
  INTERRUPT_CONTROL_IGRX_tag IGRX[1];  /* Interrupt Generation Register */
  uint8_t INTERRUPT_CONTROL_reserved2[28];
  INTERRUPT_CONTROL_IMR0_tag IMR0;     /* Interrupt Mask Register */
  uint8_t INTERRUPT_CONTROL_reserved3[124];
  INTERRUPT_CONTROL_IGMR_tag IGMR;     /* IGM Register */
  uint8_t INTERRUPT_CONTROL_reserved4[36];
  INTERRUPT_CONTROL_IVAX_tag IVAX[6];  /* Interrupt Vector Address */
  uint8_t INTERRUPT_CONTROL_reserved5[224];
  INTERRUPT_CONTROL_ICFG0_tag ICFG0;   /* ICFG0 Register */
  INTERRUPT_CONTROL_ICFG1_tag ICFG1;   /* ICFG1 Register */
  INTERRUPT_CONTROL_ICFG2_tag ICFG2;   /* ICFG2 Register */
  INTERRUPT_CONTROL_ICFG3_tag ICFG3;   /* ICFG3 Register */
  INTERRUPT_CONTROL_ICFG4_tag ICFG4;   /* ICFG4 Register */
  INTERRUPT_CONTROL_ICFG5_tag ICFG5;   /* ICFG5 Register */
  INTERRUPT_CONTROL_ICFG6_tag ICFG6;   /* ICFG6 Register */
  uint8_t INTERRUPT_CONTROL_reserved6[4];
  INTERRUPT_CONTROL_ICFG8_tag ICFG8;   /* ICFG8 Register */
  INTERRUPT_CONTROL_ICFGX_tag ICFGX[23]; /* ICFGx Register */
};


/* ============================================================================
   =============================== Module: LCU ================================
   ============================================================================ */

typedef union LCU_LC_LUTCTRL_union_tag { /* LC 0 Output 0 LUT Control */
  vuint32_t R;
  struct {
    vuint32_t LUTCTRL:16;              /* LUT Control */
    vuint32_t _unused_16:16;
  } B;
} LCU_LC_LUTCTRL_tag;

typedef union LCU_LC_FILT_union_tag {  /* LC 0 Output 0 Filter */
  vuint32_t R;
  struct {
    vuint32_t LUT_FALL_FILT:16;        /* Fall Filter */
    vuint32_t LUT_RISE_FILT:16;        /* Rise Filter */
  } B;
} LCU_LC_FILT_tag;

typedef union LCU_LC_INTDMAEN_union_tag { /* LC 0 Interrupt and DMA Enable */
  vuint32_t R;
  struct {
    vuint32_t LUT_INT_EN:4;            /* LUT Interrupt Enable */
    vuint32_t _unused_4:4;
    vuint32_t LUT_DMA_EN:4;            /* LUT DMA Enable */
    vuint32_t _unused_12:4;
    vuint32_t FORCE_INT_EN:4;          /* Force Interrupt Enable */
    vuint32_t _unused_20:4;
    vuint32_t FORCE_DMA_EN:4;          /* Force DMA Enable */
    vuint32_t _unused_28:4;
  } B;
} LCU_LC_INTDMAEN_tag;

typedef union LCU_LC_STS_union_tag {   /* LC 0 Status */
  vuint32_t R;
  struct {
    vuint32_t LUT_STS:4;               /* LUT Event */
    vuint32_t _unused_4:4;
    vuint32_t FORCESTS:4;              /* Force Event */
    vuint32_t _unused_12:20;
  } B;
} LCU_LC_STS_tag;

typedef union LCU_LC_OUTPOL_union_tag { /* LC 0 Output Polarity Control */
  vuint32_t R;
  struct {
    vuint32_t OUTPOL:4;                /* Output Polarity */
    vuint32_t _unused_4:28;
  } B;
} LCU_LC_OUTPOL_tag;

typedef union LCU_LC_FFILT_union_tag { /* LC 0 Force Filter */
  vuint32_t R;
  struct {
    vuint32_t FORCE_FILT:8;            /* Force Filter */
    vuint32_t _unused_8:8;
    vuint32_t FORCE_POL:4;             /* Force Input Polarity */
    vuint32_t _unused_20:4;
    vuint32_t COMB_EN:4;               /* Combinational Force Path (CFP) Enable */
    vuint32_t COMB_FORCE:4;            /* Combined Sensed Force Input */
  } B;
} LCU_LC_FFILT_tag;

typedef union LCU_LC_FCTRL_union_tag { /* LC 0 Force Control */
  vuint32_t R;
  struct {
    vuint32_t FORCE_SENSE0:4;          /* Force Input Sensitivity */
    vuint32_t FORCE_MODE0:2;           /* Force Clearing Mode */
    vuint32_t SYNC_SEL0:2;             /* Sync Select */
    vuint32_t FORCE_SENSE1:4;          /* Force Input Sensitivity */
    vuint32_t FORCE_MODE1:2;           /* Force Clearing Mode */
    vuint32_t SYNC_SEL1:2;             /* Sync Select */
    vuint32_t FORCE_SENSE2:4;          /* Force Input Sensitivity */
    vuint32_t FORCE_MODE2:2;           /* Force Clearing Mode */
    vuint32_t SYNC_SEL2:2;             /* Sync Select */
    vuint32_t FORCE_SENSE3:4;          /* Force Input Sensitivity */
    vuint32_t FORCE_MODE3:2;           /* Force Clearing Mode */
    vuint32_t SYNC_SEL3:2;             /* Sync Select */
  } B;
} LCU_LC_FCTRL_tag;

typedef union LCU_LC_SCTRL_union_tag { /* LC 0 Sync Control */
  vuint32_t R;
  struct {
    vuint32_t SW_MODE:4;               /* Software Sync Mode */
    vuint32_t _unused_4:4;
    vuint32_t SW_SYNC_SEL:1;           /* Software Sync Select */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:22;
  } B;
} LCU_LC_SCTRL_tag;

typedef struct LCU_LC_struct_tag {
  LCU_LC_LUTCTRL_tag LUTCTRL[4];       /* LC 0 Output 0 LUT Control */
  LCU_LC_FILT_tag FILT[4];             /* LC 0 Output 0 Filter */
  LCU_LC_INTDMAEN_tag INTDMAEN;        /* LC 0 Interrupt and DMA Enable */
  LCU_LC_STS_tag STS;                  /* LC 0 Status */
  LCU_LC_OUTPOL_tag OUTPOL;            /* LC 0 Output Polarity Control */
  LCU_LC_FFILT_tag FFILT;              /* LC 0 Force Filter */
  LCU_LC_FCTRL_tag FCTRL;              /* LC 0 Force Control */
  LCU_LC_SCTRL_tag SCTRL;              /* LC 0 Sync Control */
  uint8_t LC_reserved0[8];
} LCU_LC_tag;

typedef union LCU_MUXSEL_union_tag {   /* Mux Select */
  vuint32_t R;
  struct {
    vuint32_t MUXSEL:8;                /* Mux Select */
    vuint32_t _unused_8:24;
  } B;
} LCU_MUXSEL_tag;

typedef union LCU_CFG_union_tag {      /* Configuration */
  vuint32_t R;
  struct {
    vuint32_t WP:1;                    /* Write Protect */
    vuint32_t _unused_1:6;
    vuint32_t INCL_MUXES:1;            /* Input Muxing */
    vuint32_t NUM_SYNCS:8;             /* Sync Inputs */
    vuint32_t NUM_FORCES:8;            /* Force Inputs */
    vuint32_t NUM_LOGIC_CELLS:8;       /* LCs */
  } B;
} LCU_CFG_tag;

typedef union LCU_SWEN_union_tag {     /* Software Override Enable */
  vuint32_t R;
  struct {
    vuint32_t SWEN:12;                 /* Software Override Enable */
    vuint32_t _unused_12:20;
  } B;
} LCU_SWEN_tag;

typedef union LCU_SWVALUE_union_tag {  /* Software Override Value */
  vuint32_t R;
  struct {
    vuint32_t SWVALUE:12;              /* Software Override Value */
    vuint32_t _unused_12:20;
  } B;
} LCU_SWVALUE_tag;

typedef union LCU_OUTEN_union_tag {    /* Output Enable */
  vuint32_t R;
  struct {
    vuint32_t OUTEN:12;                /* Output Enables */
    vuint32_t _unused_12:20;
  } B;
} LCU_OUTEN_tag;

typedef union LCU_LCIN_union_tag {     /* Logic Inputs */
  vuint32_t R;
  struct {
    vuint32_t LC_INPUTS:12;            /* Logic Inputs */
    vuint32_t _unused_12:20;
  } B;
} LCU_LCIN_tag;

typedef union LCU_SWOUT_union_tag {    /* Overridden Inputs */
  vuint32_t R;
  struct {
    vuint32_t SWOUT:12;                /* Overridden Inputs */
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} LCU_SWOUT_tag;

typedef union LCU_LCOUT_union_tag {    /* Logic Outputs */
  vuint32_t R;
  struct {
    vuint32_t LCOUT:12;                /* Logic Outputs */
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} LCU_LCOUT_tag;

typedef union LCU_FORCEOUT_union_tag { /* Forced Outputs */
  vuint32_t R;
  struct {
    vuint32_t FORCEOUT:12;             /* Forced Outputs */
    vuint32_t _unused_12:20;
  } B;
} LCU_FORCEOUT_tag;

typedef union LCU_FORCESTS_union_tag { /* Force Status */
  vuint32_t R;
  struct {
    vuint32_t FORCESTS:12;             /* Force Status */
    vuint32_t _unused_12:20;
  } B;
} LCU_FORCESTS_tag;

typedef union LCU_STPEN_union_tag {    /* Stop Mode Enable */
  vuint32_t R;
  struct {
    vuint32_t STPEN:12;                /* Stop Mode Enable */
    vuint32_t _unused_12:20;
  } B;
} LCU_STPEN_tag;

typedef union LCU_DBGEN_union_tag {    /* Debug Mode Enable */
  vuint32_t R;
  struct {
    vuint32_t DBGEN:12;                /* Debug Mode Enable */
    vuint32_t _unused_12:20;
  } B;
} LCU_DBGEN_tag;

struct LCU_tag {
  LCU_LC_tag LC[3];
  uint8_t LCU_reserved0[320];
  LCU_MUXSEL_tag MUXSEL[12];           /* Mux Select */
  uint8_t LCU_reserved1[80];
  LCU_CFG_tag CFG;                     /* Configuration */
  LCU_SWEN_tag SWEN;                   /* Software Override Enable */
  LCU_SWVALUE_tag SWVALUE;             /* Software Override Value */
  LCU_OUTEN_tag OUTEN;                 /* Output Enable */
  LCU_LCIN_tag LCIN;                   /* Logic Inputs */
  LCU_SWOUT_tag SWOUT;                 /* Overridden Inputs */
  LCU_LCOUT_tag LCOUT;                 /* Logic Outputs */
  LCU_FORCEOUT_tag FORCEOUT;           /* Forced Outputs */
  LCU_FORCESTS_tag FORCESTS;           /* Force Status */
  LCU_STPEN_tag STPEN;                 /* Stop Mode Enable */
  LCU_DBGEN_tag DBGEN;                 /* Debug Mode Enable */
};


/* ============================================================================
   =============================== Module: LFAST ==============================
   ============================================================================ */

typedef union LFAST_MCR_union_tag {    /* LFAST Mode Configuration */
  vuint32_t R;
  struct {
    vuint32_t DATAEN:1;                /* DATA Frame Enable */
    vuint32_t DRFRST:1;                /* LFAST Soft Reset */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t CTSEN:1;                 /* CTS Enable */
    vuint32_t TXARBD:1;                /* Tx Arbiter Disable */
    vuint32_t _unused_5:8;             /* Reserved */
    vuint32_t TXEN:1;                  /* LFAST Transmitter Enable */
    vuint32_t RXEN:1;                  /* LFAST Receiver Enable */
    vuint32_t DRFEN:1;                 /* LFAST Enable */
    vuint32_t LSSEL:1;                 /* Low Speed Select Mode */
    vuint32_t _unused_17:7;            /* Reserved */
    vuint32_t IPGDBG:1;                /* IPG Debug mode */
    vuint32_t _unused_25:6;            /* Reserved */
    vuint32_t MSEN:1;                  /* LFAST Master or Slave mode Enable */
  } B;
} LFAST_MCR_tag;

typedef union LFAST_SCR_union_tag {    /* LFAST Speed Control */
  vuint32_t R;
  struct {
    vuint32_t TDR:1;                   /* Transmit Data Rate */
    vuint32_t _unused_1:7;             /* Reserved */
    vuint32_t RDR:1;                   /* Receiver Data Rate */
    vuint32_t _unused_9:7;             /* Reserved */
    vuint32_t DRMD:1;                  /* Data Rate Controller mode */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} LFAST_SCR_tag;

typedef union LFAST_COCR_union_tag {   /* LFAST Correlator Control */
  vuint32_t R;
  struct {
    vuint32_t PHSSEL:1;
    vuint32_t CORRTH:3;                /* Correlator Threshold Level */
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t _unused_16:2;            /* Reserved */
    vuint32_t _unused_18:6;            /* Reserved */
    vuint32_t SMPSEL:8;                /* Sampler Data Path Selector (overrides the correlator selection) */
  } B;
} LFAST_COCR_tag;

typedef union LFAST_TMCR_union_tag {   /* LFAST Test Mode Control */
  vuint32_t R;
  struct {
    vuint32_t LPFRMTH:16;
    vuint32_t LPMOD:3;                 /* Loopback Mode */
    vuint32_t _unused_19:5;            /* Reserved */
    vuint32_t LPON:1;                  /* Loopback Mode Logic Enable */
    vuint32_t CLKTST:1;                /* Clock Test Mode */
    vuint32_t _unused_26:6;            /* Reserved */
  } B;
} LFAST_TMCR_tag;

typedef union LFAST_ALCR_union_tag {   /* LFAST Auto Loopback Control */
  vuint32_t R;
  struct {
    vuint32_t LPFMCNT:16;              /* Auto Loopback Frame Transmission Count */
    vuint32_t LPCNTEN:1;               /* Auto Loopback Frame Transmission Count Enable */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} LFAST_ALCR_tag;

typedef union LFAST_RCDCR_union_tag {  /* LFAST Rate Change Delay Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t DRCNT:4;                 /* Data Rate Controller Counter Value */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} LFAST_RCDCR_tag;

typedef union LFAST_ICR_union_tag {    /* LFAST ICLC Control */
  vuint32_t R;
  struct {
    vuint32_t ICLCPLD:8;               /* ICLC Payload */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t SNDICLC:1;               /* ICLC Frame Request */
    vuint32_t ICLCSEQ:1;               /* ICLC Enabled */
    vuint32_t _unused_18:14;           /* Reserved */
  } B;
} LFAST_ICR_tag;

typedef union LFAST_PICR_union_tag {   /* LFAST Ping Control */
  vuint32_t R;
  struct {
    vuint32_t PNGPYLD:8;
    vuint32_t _unused_8:7;             /* Reserved */
    vuint32_t PNGAUTO:1;               /* Ping Response Enable */
    vuint32_t PNGREQ:1;                /* Ping Response Frame Request */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} LFAST_PICR_tag;

typedef union LFAST_RFCR_union_tag {   /* LFAST Rx FIFO CTS Control */
  vuint32_t R;
  struct {
    vuint32_t RCTSMN:6;                /* Rx FIFO Minimum Threshold */
    vuint32_t _unused_6:10;            /* Reserved */
    vuint32_t RCTSMX:6;                /* Rx FIFO Maximum Threshold */
    vuint32_t _unused_22:10;           /* Reserved */
  } B;
} LFAST_RFCR_tag;

typedef union LFAST_TIER_union_tag {   /* LFAST Tx Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t TXDTIE:1;
    vuint32_t TXICLCIE:1;              /* This field is 0 when MCR[MSEN] is 0. */
    vuint32_t TXUNSIE:1;
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t TXPNGIE:1;
    vuint32_t _unused_5:11;            /* Reserved */
    vuint32_t TXOVIE:1;
    vuint32_t TXIIE:1;
    vuint32_t _unused_18:14;           /* Reserved */
  } B;
} LFAST_TIER_tag;

typedef union LFAST_RIER_union_tag {   /* LFAST Rx Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t RXUNSIE:1;
    vuint32_t RXDIE:1;
    vuint32_t RXCTSIE:1;
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t RXLCEIE:1;
    vuint32_t RXICIE:1;
    vuint32_t RXSZIE:1;
    vuint32_t RXOFIE:1;
    vuint32_t RXUFIE:1;
    vuint32_t RXMXIE:1;
    vuint32_t RXMNIE:1;
    vuint32_t RXUOIE:1;
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} LFAST_RIER_tag;

typedef union LFAST_RIIER_union_tag {  /* LFAST Rx ICLC Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t ICPONIE:1;
    vuint32_t ICPOFIE:1;
    vuint32_t ICTSIE:1;
    vuint32_t ICTFIE:1;
    vuint32_t ICRSIE:1;
    vuint32_t ICRFIE:1;
    vuint32_t ICTEIE:1;
    vuint32_t ICTDIE:1;
    vuint32_t ICCTIE:1;
    vuint32_t ICLPIE:1;
    vuint32_t ICTOIE:1;
    vuint32_t ICPRIE:1;
    vuint32_t ICPSIE:1;                /* This field is 0 when MCR[MSEN] is 0. */
    vuint32_t ICPFIE:1;                /* This field is 0 when MCR[MSEN] is 0. */
    vuint32_t _unused_14:18;           /* Reserved */
  } B;
} LFAST_RIIER_tag;

typedef union LFAST_PLLCR_union_tag {  /* LFAST PLL Control */
  vuint32_t R;
  struct {
    vuint32_t PREDIV:2;
    vuint32_t FBDIV:6;
    vuint32_t FDIVEN:1;
    vuint32_t PLCKCW:2;                /* PLL Lock Ready Count Width */
    vuint32_t _unused_11:2;            /* Reserved */
    vuint32_t LPCFG:2;
    vuint32_t REFINV:1;
    vuint32_t SWPON:1;
    vuint32_t SWPOFF:1;
    vuint32_t _unused_18:11;           /* Reserved */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} LFAST_PLLCR_tag;

typedef union LFAST_UNSTCR_union_tag { /* LFAST Unsolicited Tx Control */
  vuint32_t R;
  struct {
    vuint32_t UNSHDR:7;                /* Tx Unsolicited Message Header */
    vuint32_t _unused_7:9;             /* Reserved */
    vuint32_t USNDRQ:1;                /* Tx Unsolicited send request */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} LFAST_UNSTCR_tag;

typedef union LFAST_UNSTDR_union_tag { /* LFAST Unsolicited Tx Data Registers */
  vuint32_t R;
  struct {
    vuint32_t UNTXD:32;
  } B;
} LFAST_UNSTDR_tag;

typedef union LFAST_GSR_union_tag {    /* LFAST Global Status */
  vuint32_t R;
  struct {
    vuint32_t LPCSDV:1;
    vuint32_t LPCHDV:1;
    vuint32_t LPCPDV:1;
    vuint32_t LPFPDV:1;
    vuint32_t LPTXDN:1;
    vuint32_t _unused_5:11;            /* Reserved */
    vuint32_t DRSM:1;                  /* Receive Interface Data Rate Status */
    vuint32_t LDSM:1;                  /* Transmit Interface Data Rate Status */
    vuint32_t LRMD:1;
    vuint32_t _unused_19:12;           /* Reserved */
    vuint32_t DUALMD:1;
  } B;
} LFAST_GSR_tag;

typedef union LFAST_PISR_union_tag {   /* LFAST Ping Status */
  vuint32_t R;
  struct {
    vuint32_t RXPNGD:8;                /* Ping Data Register */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} LFAST_PISR_tag;

typedef union LFAST_DFSR_union_tag {   /* LFAST Data Frame Status */
  vuint32_t R;
  struct {
    vuint32_t TXFCNT:3;                /* Unread Tx Frame Count */
    vuint32_t _unused_3:5;             /* Reserved */
    vuint32_t TXDCNT:6;                /* Unread Tx Frame Data Count */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t RXFCNT:3;                /* Unread Rx Frame Count */
    vuint32_t _unused_19:5;            /* Reserved */
    vuint32_t RXDCNT:6;                /* Unread Rx Frame Data Count */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} LFAST_DFSR_tag;

typedef union LFAST_TISR_union_tag {   /* LFAST Tx Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t TXDTF:1;
    vuint32_t TXICLCF:1;
    vuint32_t TXUNSF:1;                /* TXUNSF */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t TXPNGF:1;
    vuint32_t _unused_5:11;            /* Reserved */
    vuint32_t TXOVF:1;
    vuint32_t TXIEF:1;
    vuint32_t _unused_18:14;           /* Reserved */
  } B;
} LFAST_TISR_tag;

typedef union LFAST_RISR_union_tag {   /* LFAST Rx Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t RXUNSF:1;
    vuint32_t RXDF:1;
    vuint32_t RXCTSF:1;
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t RXLCEF:1;
    vuint32_t RXICF:1;
    vuint32_t RXSZF:1;
    vuint32_t RXOFF:1;
    vuint32_t RXUFF:1;
    vuint32_t RXMXF:1;
    vuint32_t RXMNF:1;
    vuint32_t RXUOF:1;
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} LFAST_RISR_tag;

typedef union LFAST_RIISR_union_tag {  /* LFAST Rx ICLC Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t ICPONF:1;
    vuint32_t ICPOFF:1;
    vuint32_t ICTSF:1;
    vuint32_t ICTFF:1;
    vuint32_t ICRSF:1;
    vuint32_t ICRFF:1;
    vuint32_t ICTEF:1;
    vuint32_t ICTDF:1;
    vuint32_t ICCTF:1;
    vuint32_t ICLPF:1;
    vuint32_t ICTOF:1;
    vuint32_t ICPRF:1;
    vuint32_t ICPSF:1;
    vuint32_t ICPFF:1;
    vuint32_t _unused_14:18;           /* Reserved */
  } B;
} LFAST_RIISR_tag;

typedef union LFAST_PLLLSR_union_tag { /* LFAST PLL and LVDS Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t LDSLPS:1;
    vuint32_t LRSLPS:1;
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t PLDCR:1;                 /* PLL Lock Delay Counter Ready */
    vuint32_t PLLDIS:1;                /* PLL Disable Status */
    vuint32_t _unused_18:14;           /* Reserved */
  } B;
} LFAST_PLLLSR_tag;

typedef union LFAST_UNSRSR_union_tag { /* LFAST Unsolicited Rx Status */
  vuint32_t R;
  struct {
    vuint32_t URPCNT:3;                /* Rx Unsolicited Payload */
    vuint32_t _unused_3:5;             /* Reserved */
    vuint32_t URXDV:1;                 /* Unsolicited Data Valid */
    vuint32_t _unused_9:23;            /* Reserved */
  } B;
} LFAST_UNSRSR_tag;

typedef union LFAST_UNSRDR_union_tag { /* LFAST Unsolicited Rx Data */
  vuint32_t R;
  struct {
    vuint32_t UNRXD:32;                /* Unsolicited Receive Data */
  } B;
} LFAST_UNSRDR_tag;

typedef union LFAST_GPR_union_tag {    /* General Purpose registers */
  vuint32_t R;
  struct {
    vuint32_t GPR:32;
  } B;
} LFAST_GPR_tag;

struct LFAST_tag {
  LFAST_MCR_tag MCR;                   /* LFAST Mode Configuration */
  LFAST_SCR_tag SCR;                   /* LFAST Speed Control */
  LFAST_COCR_tag COCR;                 /* LFAST Correlator Control */
  LFAST_TMCR_tag TMCR;                 /* LFAST Test Mode Control */
  LFAST_ALCR_tag ALCR;                 /* LFAST Auto Loopback Control */
  LFAST_RCDCR_tag RCDCR;               /* LFAST Rate Change Delay Control */
  uint8_t LFAST_reserved0[4];
  LFAST_ICR_tag ICR;                   /* LFAST ICLC Control */
  LFAST_PICR_tag PICR;                 /* LFAST Ping Control */
  uint8_t LFAST_reserved1[8];
  LFAST_RFCR_tag RFCR;                 /* LFAST Rx FIFO CTS Control */
  LFAST_TIER_tag TIER;                 /* LFAST Tx Interrupt Enable */
  LFAST_RIER_tag RIER;                 /* LFAST Rx Interrupt Enable */
  LFAST_RIIER_tag RIIER;               /* LFAST Rx ICLC Interrupt Enable */
  LFAST_PLLCR_tag PLLCR;               /* LFAST PLL Control */
  uint8_t LFAST_reserved2[4];
  LFAST_UNSTCR_tag UNSTCR;             /* LFAST Unsolicited Tx Control */
  LFAST_UNSTDR_tag UNSTDR[9];          /* LFAST Unsolicited Tx Data Registers */
  uint8_t LFAST_reserved3[20];
  LFAST_GSR_tag GSR;                   /* LFAST Global Status */
  LFAST_PISR_tag PISR;                 /* LFAST Ping Status */
  uint8_t LFAST_reserved4[12];
  LFAST_DFSR_tag DFSR;                 /* LFAST Data Frame Status */
  LFAST_TISR_tag TISR;                 /* LFAST Tx Interrupt Status */
  LFAST_RISR_tag RISR;                 /* LFAST Rx Interrupt Status */
  LFAST_RIISR_tag RIISR;               /* LFAST Rx ICLC Interrupt Status */
  LFAST_PLLLSR_tag PLLLSR;             /* LFAST PLL and LVDS Status */
  LFAST_UNSRSR_tag UNSRSR;             /* LFAST Unsolicited Rx Status */
  LFAST_UNSRDR_tag UNSRDR[9];          /* LFAST Unsolicited Rx Data */
  uint8_t LFAST_reserved5[4];
  LFAST_GPR_tag GPR[1];                /* General Purpose registers */
};


/* ============================================================================
   =============================== Module: LINFLEXD ===========================
   ============================================================================ */

typedef union LINFLEXD_LINCR1_union_tag { /* LIN Control Register 1 */
  vuint32_t R;
  struct {
    vuint32_t INIT:1;
    vuint32_t SLEEP:1;
    vuint32_t RBLM:1;
    vuint32_t SSBL:1;
    vuint32_t MME:1;
    vuint32_t LBKM:1;
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t MBL:4;
    vuint32_t AUTOWU:1;
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t CFD:1;
    vuint32_t CCD:1;
    vuint32_t NLSE:1;
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} LINFLEXD_LINCR1_tag;

typedef union LINFLEXD_LINIER_union_tag { /* LIN Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t HRIE:1;
    vuint32_t DTIE:1;
    vuint32_t DRIE:1;
    vuint32_t TOIE:1;
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t WUIE:1;
    vuint32_t LSIE:1;
    vuint32_t BOIE:1;
    vuint32_t FEIE:1;
    vuint32_t _unused_9:2;             /* Reserved */
    vuint32_t HEIE:1;
    vuint32_t CEIE:1;
    vuint32_t BEIE:1;
    vuint32_t OCIE:1;
    vuint32_t SZIE:1;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} LINFLEXD_LINIER_tag;

typedef union LINFLEXD_LINSR_union_tag { /* LIN Status */
  vuint32_t R;
  struct {
    vuint32_t HRF:1;
    vuint32_t DTF:1;
    vuint32_t DRF:1;
    vuint32_t _unused_3:2;             /* Reserved */
    vuint32_t WUF:1;
    vuint32_t RDI:1;
    vuint32_t RXBUSY:1;
    vuint32_t DRBNE:1;
    vuint32_t RMB:1;
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t LINS:4;                  /* LIN State */
    vuint32_t RDC:3;
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} LINFLEXD_LINSR_tag;

typedef union LINFLEXD_LINESR_union_tag { /* LIN Error Status */
  vuint32_t R;
  struct {
    vuint32_t NF:1;
    vuint32_t _unused_1:6;             /* Reserved */
    vuint32_t BOF:1;
    vuint32_t FEF:1;
    vuint32_t IDPEF:1;
    vuint32_t SDEF:1;
    vuint32_t SFEF:1;
    vuint32_t CEF:1;
    vuint32_t BEF:1;
    vuint32_t OCF:1;                   /* Output Compare Flag */
    vuint32_t SZF:1;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} LINFLEXD_LINESR_tag;

typedef union LINFLEXD_UARTCR_union_tag { /* UART Mode Control */
  vuint32_t R;
  struct {
    vuint32_t UART:1;
    vuint32_t WL0:1;
    vuint32_t PCE:1;
    vuint32_t PC0:1;
    vuint32_t TXEN:1;
    vuint32_t RXEN:1;
    vuint32_t PC1:1;
    vuint32_t WL1:1;
    vuint32_t TFBM:1;
    vuint32_t RFBM:1;
    vuint32_t RDFL_RFC:3;
    vuint32_t TDFL_TFC:3;
    vuint32_t WLS:1;
    vuint32_t SBUR:2;
    vuint32_t DTU_PCETX:1;             /* Disable Timeout in UART mode */
    vuint32_t NEF:3;                   /* Number of expected frames */
    vuint32_t ROSE:1;                  /* Reduced Over Sampling Enable */
    vuint32_t OSR:4;                   /* Over Sampling Rate */
    vuint32_t CSP:3;
    vuint32_t MIS:1;
  } B;
} LINFLEXD_UARTCR_tag;

typedef union LINFLEXD_UARTSR_union_tag { /* UART Mode Status */
  vuint32_t R;
  struct {
    vuint32_t NF:1;
    vuint32_t DTFTFF:1;
    vuint32_t DRFRFE:1;
    vuint32_t TO:1;
    vuint32_t RFNE:1;
    vuint32_t WUF:1;
    vuint32_t RDI:1;
    vuint32_t BOF:1;
    vuint32_t FEF:1;
    vuint32_t RMB:1;
    vuint32_t PE:4;
    vuint32_t OCF:1;
    vuint32_t SZF:1;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} LINFLEXD_UARTSR_tag;

typedef union LINFLEXD_LINTCSR_union_tag { /* LIN Time-Out Control Status */
  vuint32_t R;
  struct {
    vuint32_t CNT:8;
    vuint32_t TOCE:1;
    vuint32_t IOT:1;
    vuint32_t MODE:1;
    vuint32_t _unused_11:21;           /* Reserved */
  } B;
} LINFLEXD_LINTCSR_tag;

typedef union LINFLEXD_LINOCR_union_tag { /* LIN Output Compare */
  vuint32_t R;
  struct {
    vuint32_t OC1:8;
    vuint32_t OC2:8;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} LINFLEXD_LINOCR_tag;

typedef union LINFLEXD_LINTOCR_union_tag { /* LIN Time-Out Control */
  vuint32_t R;
  struct {
    vuint32_t HTO:7;
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t RTO:4;
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} LINFLEXD_LINTOCR_tag;

typedef union LINFLEXD_LINFBRR_union_tag { /* LIN Fractional Baud Rate */
  vuint32_t R;
  struct {
    vuint32_t FBR:4;
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} LINFLEXD_LINFBRR_tag;

typedef union LINFLEXD_LINIBRR_union_tag { /* LIN Integer Baud Rate */
  vuint32_t R;
  struct {
    vuint32_t IBR:20;
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} LINFLEXD_LINIBRR_tag;

typedef union LINFLEXD_LINCFR_union_tag { /* LIN Checksum Field */
  vuint32_t R;
  struct {
    vuint32_t CF:8;
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} LINFLEXD_LINCFR_tag;

typedef union LINFLEXD_LINCR2_union_tag { /* LIN Control Register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t HTRQ:1;
    vuint32_t ABRQ:1;
    vuint32_t DTRQ:1;
    vuint32_t DDRQ:1;
    vuint32_t WURQ:1;
    vuint32_t IOPE:1;
    vuint32_t IOBE:1;
    vuint32_t TBDE:1;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} LINFLEXD_LINCR2_tag;

typedef union LINFLEXD_BIDR_union_tag { /* Buffer Identifier */
  vuint32_t R;
  struct {
    vuint32_t ID:6;
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t CCS:1;
    vuint32_t DIR:1;
    vuint32_t DFL:3;
    vuint32_t _unused_13:3;            /* Reserved */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} LINFLEXD_BIDR_tag;

typedef union LINFLEXD_BDRL_union_tag { /* Buffer Data Register Least Significant */
  vuint32_t R;
  struct {
    vuint32_t DATA0:8;
    vuint32_t DATA1:8;
    vuint32_t DATA2:8;
    vuint32_t DATA3:8;
  } B;
} LINFLEXD_BDRL_tag;

typedef union LINFLEXD_BDRM_union_tag { /* Buffer Data Register Most Significant */
  vuint32_t R;
  struct {
    vuint32_t DATA4:8;
    vuint32_t DATA5:8;
    vuint32_t DATA6:8;
    vuint32_t DATA7:8;
  } B;
} LINFLEXD_BDRM_tag;

typedef union LINFLEXD_GCR_union_tag { /* Global Control */
  vuint32_t R;
  struct {
    vuint32_t SR:1;
    vuint32_t STOP:1;
    vuint32_t RDLIS:1;
    vuint32_t TDLIS:1;
    vuint32_t RDFBM:1;
    vuint32_t TDFBM:1;
    vuint32_t _unused_6:26;            /* Reserved */
  } B;
} LINFLEXD_GCR_tag;

typedef union LINFLEXD_UARTPTO_union_tag { /* UART Preset Timeout */
  vuint32_t R;
  struct {
    vuint32_t PTO:12;
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} LINFLEXD_UARTPTO_tag;

typedef union LINFLEXD_UARTCTO_union_tag { /* UART Current Timeout */
  vuint32_t R;
  struct {
    vuint32_t CTO:12;
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} LINFLEXD_UARTCTO_tag;

typedef union LINFLEXD_DMATXE_union_tag { /* DMA Tx Enable */
  vuint32_t R;
  struct {
    vuint32_t DTE0:1;                  /* DMA Tx channel enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} LINFLEXD_DMATXE_tag;

typedef union LINFLEXD_DMARXE_union_tag { /* DMA Rx Enable */
  vuint32_t R;
  struct {
    vuint32_t DRE0:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} LINFLEXD_DMARXE_tag;

struct LINFLEXD_tag {
  LINFLEXD_LINCR1_tag LINCR1;          /* LIN Control Register 1 */
  LINFLEXD_LINIER_tag LINIER;          /* LIN Interrupt Enable */
  LINFLEXD_LINSR_tag LINSR;            /* LIN Status */
  LINFLEXD_LINESR_tag LINESR;          /* LIN Error Status */
  LINFLEXD_UARTCR_tag UARTCR;          /* UART Mode Control */
  LINFLEXD_UARTSR_tag UARTSR;          /* UART Mode Status */
  LINFLEXD_LINTCSR_tag LINTCSR;        /* LIN Time-Out Control Status */
  LINFLEXD_LINOCR_tag LINOCR;          /* LIN Output Compare */
  LINFLEXD_LINTOCR_tag LINTOCR;        /* LIN Time-Out Control */
  LINFLEXD_LINFBRR_tag LINFBRR;        /* LIN Fractional Baud Rate */
  LINFLEXD_LINIBRR_tag LINIBRR;        /* LIN Integer Baud Rate */
  LINFLEXD_LINCFR_tag LINCFR;          /* LIN Checksum Field */
  LINFLEXD_LINCR2_tag LINCR2;          /* LIN Control Register 2 */
  LINFLEXD_BIDR_tag BIDR;              /* Buffer Identifier */
  LINFLEXD_BDRL_tag BDRL;              /* Buffer Data Register Least Significant */
  LINFLEXD_BDRM_tag BDRM;              /* Buffer Data Register Most Significant */
  uint8_t LINFLEXD_reserved0[12];
  LINFLEXD_GCR_tag GCR;                /* Global Control */
  LINFLEXD_UARTPTO_tag UARTPTO;        /* UART Preset Timeout */
  LINFLEXD_UARTCTO_tag UARTCTO;        /* UART Current Timeout */
  LINFLEXD_DMATXE_tag DMATXE;          /* DMA Tx Enable */
  LINFLEXD_DMARXE_tag DMARXE;          /* DMA Rx Enable */
};


/* ============================================================================
   =============================== Module: LLC_CSR ============================
   ============================================================================ */

typedef union LLC_CSR_CCUTCR_union_tag { /* Transaction Control */
  vuint32_t R;
  struct {
    vuint32_t TRANSEN:1;               /* Transaction Access Enable */
    vuint32_t _unused_1:31;
  } B;
} LLC_CSR_CCUTCR_tag;

typedef union LLC_CSR_CCUTAR_union_tag { /* Transaction Activity */
  vuint32_t R;
  struct {
    vuint32_t TRANSACTV:1;             /* Transaction Active */
    vuint32_t _unused_1:31;
  } B;
} LLC_CSR_CCUTAR_tag;

typedef union LLC_CSR_CCUCTCR_union_tag { /* Cache Transaction Control */
  vuint32_t R;
  struct {
    vuint32_t LOOKUPEN:1;              /* Lookup Enable */
    vuint32_t FILLEN:1;                /* Fill Enable */
    vuint32_t _unused_2:30;
  } B;
} LLC_CSR_CCUCTCR_tag;

typedef union LLC_CSR_CCUCTAR_union_tag { /* Cache Transaction Activity */
  vuint32_t R;
  struct {
    vuint32_t EVICTACTV:1;             /* Eviction Active */
    vuint32_t FILLACTV:1;              /* Fill Active */
    vuint32_t FLUSHACTV:1;             /* Flush Active */
    vuint32_t _unused_3:29;
  } B;
} LLC_CSR_CCUCTAR_tag;

typedef union LLC_CSR_CCUCAOR_union_tag { /* Allocation Override */
  vuint32_t R;
  struct {
    vuint32_t AWALLOCEN:1;             /* Enable Write Channel Override */
    vuint32_t ARALLOCEN:1;             /* Enable Read Channel Override */
    vuint32_t WRALLOCPARTIALEN:1;      /* Enable Partial Write Allocation */
    vuint32_t _unused_3:13;
    vuint32_t AWALLOCVALUE:4;          /* Write Channel Cache Override */
    vuint32_t ARALLOCVALUE:4;          /* Read Channel Cache Override */
    vuint32_t _unused_24:8;
  } B;
} LLC_CSR_CCUCAOR_tag;

typedef union LLC_CSR_CCUSPCR0_union_tag { /* Scratchpad Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t SCPADEN:1;               /* Scratchpad RAM Enable */
    vuint32_t _unused_1:15;
    vuint32_t NUMSCPADWAYS:3;          /* Scratchpad Ways */
    vuint32_t _unused_19:13;
  } B;
} LLC_CSR_CCUSPCR0_tag;

typedef union LLC_CSR_CCUSPCR1_union_tag { /* Scratchpad Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t SCPADSIZE:32;            /* Scratchpad Size */
  } B;
} LLC_CSR_CCUSPCR1_tag;

typedef union LLC_CSR_CCUSPBR0_union_tag { /* Scratchpad Base Address 0 */
  vuint32_t R;
  struct {
    vuint32_t SCPADBASEADDR:32;        /* Scratchpad Base Address */
  } B;
} LLC_CSR_CCUSPBR0_tag;

typedef union LLC_CSR_CCUWPCR_CCUWPCRA_union_tag { /* Way Partitioning Control A 0 */
  vuint32_t R;
  struct {
    vuint32_t WPAGENTID:4;             /* Way Partitioning Agent ID */
    vuint32_t _unused_4:27;
    vuint32_t WPAGENTIDVALID:1;        /* Way Partitioning Agent ID Valid */
  } B;
} LLC_CSR_CCUWPCR_CCUWPCRA_tag;

typedef union LLC_CSR_CCUWPCR_CCUWPCRB_union_tag { /* Way Partitioning Control B 0 */
  vuint32_t R;
  struct {
    vuint32_t WPWAYVECTOR:8;           /* Way Partitioning Vector Enable */
    vuint32_t _unused_8:24;
  } B;
} LLC_CSR_CCUWPCR_CCUWPCRB_tag;

typedef struct LLC_CSR_CCUWPCR_struct_tag {
  LLC_CSR_CCUWPCR_CCUWPCRA_tag CCUWPCRA; /* Way Partitioning Control A 0 */
  LLC_CSR_CCUWPCR_CCUWPCRB_tag CCUWPCRB; /* Way Partitioning Control B 0 */
} LLC_CSR_CCUWPCR_tag;

typedef union LLC_CSR_CCUCMCR_union_tag { /* Maintenance Control */
  vuint32_t R;
  struct {
    vuint32_t MNTOP:4;                 /* Maintenance Operation */
    vuint32_t _unused_4:12;
    vuint32_t ARRAYID:6;               /* Array ID */
    vuint32_t SECATTR:1;               /* Security Attribute */
    vuint32_t _unused_23:9;
  } B;
} LLC_CSR_CCUCMCR_tag;

typedef union LLC_CSR_CCUCMAR_union_tag { /* Maintenance Activity */
  vuint32_t R;
  struct {
    vuint32_t MNTOPACTV:1;             /* Maintenance Operation Active */
    vuint32_t _unused_1:31;
  } B;
} LLC_CSR_CCUCMAR_tag;

typedef union LLC_CSR_CCUCMLR0_union_tag { /* Maintenance Location 0 */
  vuint32_t R;
  struct {
    vuint32_t MNTSET:15;               /* Maintenance Set */
    vuint32_t MNTWAY:6;                /* Maintenance Way */
    vuint32_t MNTWORD:11;              /* Maintenance Word */
  } B;
} LLC_CSR_CCUCMLR0_tag;

typedef union LLC_CSR_CCUCMLR1_union_tag { /* Maintenance Location 1 */
  vuint32_t R;
  struct {
    vuint32_t MNTADDR:16;              /* Maintenance Address */
    vuint32_t _unused_16:16;
  } B;
} LLC_CSR_CCUCMLR1_tag;

typedef union LLC_CSR_CCUCMLR2_union_tag { /* Maintenance Location 2 */
  vuint32_t R;
  struct {
    vuint32_t MNTRANGE:16;             /* Maintenance Range */
    vuint32_t _unused_16:16;
  } B;
} LLC_CSR_CCUCMLR2_tag;

typedef union LLC_CSR_CCUCMDR_union_tag { /* Maintenance Data */
  vuint32_t R;
  struct {
    vuint32_t MNTDATA:32;              /* Maintenance Data */
  } B;
} LLC_CSR_CCUCMDR_tag;

typedef union LLC_CSR_CCUCECR_union_tag { /* Correctable Error Control */
  vuint32_t R;
  struct {
    vuint32_t ERRDETEN:1;              /* Error Detection Enable */
    vuint32_t ERRINTEN:1;              /* Error Interrupt Enable */
    vuint32_t _unused_2:2;
    vuint32_t ERRTHRESHOLD:8;          /* Error Threshold */
    vuint32_t _unused_12:20;
  } B;
} LLC_CSR_CCUCECR_tag;

typedef union LLC_CSR_CCUCESR_union_tag { /* Correctable Error Status */
  vuint32_t R;
  struct {
    vuint32_t CERRVLD:1;               /* Correctable Error Valid */
    vuint32_t CERRCOUNTOVERFLOW:1;     /* Correctable Error Count Overflow */
    vuint32_t CERRCOUNT:8;             /* Correctable Error Count */
    vuint32_t _unused_10:1;
    vuint32_t CERRTYPE:5;              /* Correctable Error Type */
    vuint32_t CERRINFO:16;             /* Correctable Error Array */
  } B;
} LLC_CSR_CCUCESR_tag;

typedef union LLC_CSR_CCUCESAR_union_tag { /* Correctable Error Status Alias */
  vuint32_t R;
  struct {
    vuint32_t CERRVLD:1;               /* Alias Correctable Error Valid */
    vuint32_t CERRCOUNTOVERFLOW:1;     /* Alias Correctable Error Count Overflow */
    vuint32_t CERRCOUNT:8;             /* Alias Correctable Error Count */
    vuint32_t _unused_10:1;
    vuint32_t CERRTYPE:5;              /* Alias Correctable Error Type */
    vuint32_t CERRINFO:16;             /* Alias Correctable Error Info */
  } B;
} LLC_CSR_CCUCESAR_tag;

typedef union LLC_CSR_CCUCELR0_union_tag { /* Correctable Error Location 0 */
  vuint32_t R;
  struct {
    vuint32_t ERRENTRY:15;             /* Error Entry */
    vuint32_t ERRWAY:6;                /* Error Way */
    vuint32_t ERRWORD:11;              /* Error Word */
  } B;
} LLC_CSR_CCUCELR0_tag;

typedef union LLC_CSR_CCUCELR1_union_tag { /* Correctable Error Location 1 */
  vuint32_t R;
  struct {
    vuint32_t ERRADDR:16;              /* Error Address */
    vuint32_t _unused_16:16;
  } B;
} LLC_CSR_CCUCELR1_tag;

typedef union LLC_CSR_CCUUEDR_union_tag { /* Uncorrectable Error Detect */
  vuint32_t R;
  struct {
    vuint32_t PROTERRDETEN:1;          /* AXI Downstream Protocol Error Detect Enable */
    vuint32_t MEMERRDETEN:1;           /* Memory Protection Error Detect Enable */
    vuint32_t _unused_2:30;
  } B;
} LLC_CSR_CCUUEDR_tag;

typedef union LLC_CSR_CCUUEIR_union_tag { /* Uncorrectable Error Interrupt */
  vuint32_t R;
  struct {
    vuint32_t PROTERRINTEN:1;          /* AXI Downstream Protocol Error Interrupt Enable */
    vuint32_t MEMERRINTEN:1;           /* Memory Protection Error Interrupt Enable */
    vuint32_t _unused_2:30;
  } B;
} LLC_CSR_CCUUEIR_tag;

typedef union LLC_CSR_CCUUESR_union_tag { /* Uncorrectable Error Status */
  vuint32_t R;
  struct {
    vuint32_t UERRVLD:1;               /* Uncorrectable Error Valid */
    vuint32_t _unused_1:3;
    vuint32_t UERRTYPE:5;              /* Uncorrectable Error Type */
    vuint32_t _unused_9:7;
    vuint32_t UERRINFO:16;             /* Uncorrectable Error Info */
  } B;
} LLC_CSR_CCUUESR_tag;

typedef union LLC_CSR_CCUUESAR_union_tag { /* Uncorrectable Error Status Alias */
  vuint32_t R;
  struct {
    vuint32_t UERRVLD:1;               /* Alias Uncorrectable Error Valid */
    vuint32_t _unused_1:3;
    vuint32_t UERRTYPE:5;              /* Alias Uncorrectable Error Type */
    vuint32_t _unused_9:7;
    vuint32_t UERRINFO:16;             /* Alias Uncorrectable Error Info */
  } B;
} LLC_CSR_CCUUESAR_tag;

typedef union LLC_CSR_CCUUELR0_union_tag { /* Uncorrectable Error Location 0 */
  vuint32_t R;
  struct {
    vuint32_t ERRENTRY:15;             /* Error Entry */
    vuint32_t ERRWAY:6;                /* Error Way */
    vuint32_t ERRWORD:11;              /* Error Word */
  } B;
} LLC_CSR_CCUUELR0_tag;

typedef union LLC_CSR_CCUUELR1_union_tag { /* Uncorrectable Error Location 1 */
  vuint32_t R;
  struct {
    vuint32_t ERRADDR:16;              /* Error Address */
    vuint32_t _unused_16:16;
  } B;
} LLC_CSR_CCUUELR1_tag;

typedef union LLC_CSR_CCUIDR_union_tag { /* Identification */
  vuint32_t R;
  struct {
    vuint32_t PATCHRELVER:4;           /* Patch Release Version */
    vuint32_t MINRELVER:4;             /* Minor Release Version */
    vuint32_t MAJRELVER:4;             /* Major Release Version */
    vuint32_t _unused_12:20;
  } B;
} LLC_CSR_CCUIDR_tag;

typedef union LLC_CSR_CCUCRTR_union_tag { /* Correctable Resiliency Threshold */
  vuint32_t R;
  struct {
    vuint32_t RESTHRESHOLD:8;          /* Resiliency Threshold */
    vuint32_t _unused_8:24;
  } B;
} LLC_CSR_CCUCRTR_tag;

struct LLC_CSR_tag {
  LLC_CSR_CCUTCR_tag CCUTCR;           /* Transaction Control */
  LLC_CSR_CCUTAR_tag CCUTAR;           /* Transaction Activity */
  uint8_t LLC_CSR_reserved0[8];
  LLC_CSR_CCUCTCR_tag CCUCTCR;         /* Cache Transaction Control */
  LLC_CSR_CCUCTAR_tag CCUCTAR;         /* Cache Transaction Activity */
  LLC_CSR_CCUCAOR_tag CCUCAOR;         /* Allocation Override */
  uint8_t LLC_CSR_reserved1[4];
  LLC_CSR_CCUSPCR0_tag CCUSPCR0;       /* Scratchpad Configuration 0 */
  LLC_CSR_CCUSPCR1_tag CCUSPCR1;       /* Scratchpad Configuration 1 */
  LLC_CSR_CCUSPBR0_tag CCUSPBR0;       /* Scratchpad Base Address 0 */
  uint8_t LLC_CSR_reserved2[20];
  LLC_CSR_CCUWPCR_tag CCUWPCR[8];
  uint8_t LLC_CSR_reserved3[128];
  LLC_CSR_CCUCMCR_tag CCUCMCR;         /* Maintenance Control */
  LLC_CSR_CCUCMAR_tag CCUCMAR;         /* Maintenance Activity */
  LLC_CSR_CCUCMLR0_tag CCUCMLR0;       /* Maintenance Location 0 */
  LLC_CSR_CCUCMLR1_tag CCUCMLR1;       /* Maintenance Location 1 */
  LLC_CSR_CCUCMLR2_tag CCUCMLR2;       /* Maintenance Location 2 */
  LLC_CSR_CCUCMDR_tag CCUCMDR;         /* Maintenance Data */
  uint8_t LLC_CSR_reserved4[40];
  LLC_CSR_CCUCECR_tag CCUCECR;         /* Correctable Error Control */
  LLC_CSR_CCUCESR_tag CCUCESR;         /* Correctable Error Status */
  LLC_CSR_CCUCESAR_tag CCUCESAR;       /* Correctable Error Status Alias */
  LLC_CSR_CCUCELR0_tag CCUCELR0;       /* Correctable Error Location 0 */
  LLC_CSR_CCUCELR1_tag CCUCELR1;       /* Correctable Error Location 1 */
  LLC_CSR_CCUUEDR_tag CCUUEDR;         /* Uncorrectable Error Detect */
  LLC_CSR_CCUUEIR_tag CCUUEIR;         /* Uncorrectable Error Interrupt */
  LLC_CSR_CCUUESR_tag CCUUESR;         /* Uncorrectable Error Status */
  LLC_CSR_CCUUESAR_tag CCUUESAR;       /* Uncorrectable Error Status Alias */
  LLC_CSR_CCUUELR0_tag CCUUELR0;       /* Uncorrectable Error Location 0 */
  LLC_CSR_CCUUELR1_tag CCUUELR1;       /* Uncorrectable Error Location 1 */
  uint8_t LLC_CSR_reserved5[84];
  LLC_CSR_CCUIDR_tag CCUIDR;           /* Identification */
  LLC_CSR_CCUCRTR_tag CCUCRTR;         /* Correctable Resiliency Threshold */
};


/* ============================================================================
   =============================== Module: LLC_FSC ============================
   ============================================================================ */

typedef union LLC_FSC_FSCBISTCR_union_tag { /* FSC BIST Control */
  vuint32_t R;
  struct {
    vuint32_t BIST_START:1;            /* BIST Start */
    vuint32_t BIST_STEP:1;             /* BIST Step */
    vuint32_t _unused_2:30;
  } B;
} LLC_FSC_FSCBISTCR_tag;

typedef union LLC_FSC_FSCBISTAR_union_tag { /* FSC BIST Activity */
  vuint32_t R;
  struct {
    vuint32_t BIST_DONE:5;             /* BIST Done */
    vuint32_t BIST_ERR:5;              /* BIST Error */
    vuint32_t _unused_10:22;
  } B;
} LLC_FSC_FSCBISTAR_tag;

typedef union LLC_FSC_FSCLF0_union_tag { /* FSC Latent Fault Inputs 0 */
  vuint32_t R;
  struct {
    vuint32_t LATENT_FAULT:32;         /* Latent Fault */
  } B;
} LLC_FSC_FSCLF0_tag;

typedef union LLC_FSC_FSCLF_union_tag { /* FSC Latent Fault Inputs */
  vuint32_t R;
  struct {
    vuint32_t LATENT_FAULT:32;         /* Latent Fault */
  } B;
} LLC_FSC_FSCLF_tag;

typedef union LLC_FSC_FSCMF0_union_tag { /* FSC Mission Fault 0 */
  vuint32_t R;
  struct {
    vuint32_t MISSION_FAULT:32;        /* Mission Fault */
  } B;
} LLC_FSC_FSCMF0_tag;

typedef union LLC_FSC_FSCMF_union_tag { /* FSC Mission Fault Inputs */
  vuint32_t R;
  struct {
    vuint32_t MISSION_FAULT:32;        /* Mission Fault */
  } B;
} LLC_FSC_FSCMF_tag;

typedef union LLC_FSC_FSCCETHF0_union_tag { /* FSC Correctable Error Over Theshold 0 */
  vuint32_t R;
  struct {
    vuint32_t CERR_OVER_THRESH:32;     /* Correctable Errors Over Threshold */
  } B;
} LLC_FSC_FSCCETHF0_tag;

typedef union LLC_FSC_FSCCETHF_union_tag { /* FSC Correctable Error Over Threshold 1 */
  vuint32_t R;
  struct {
    vuint32_t CERR_OVER_THRESH:32;     /* Correctable Errors Over Threshold */
  } B;
} LLC_FSC_FSCCETHF_tag;

struct LLC_FSC_tag {
  LLC_FSC_FSCBISTCR_tag FSCBISTCR;     /* FSC BIST Control */
  LLC_FSC_FSCBISTAR_tag FSCBISTAR;     /* FSC BIST Activity */
  uint8_t LLC_FSC_reserved0[8];
  LLC_FSC_FSCLF0_tag FSCLF0;           /* FSC Latent Fault Inputs 0 */
  LLC_FSC_FSCLF_tag FSCLF[3];          /* FSC Latent Fault Inputs */
  LLC_FSC_FSCMF0_tag FSCMF0;           /* FSC Mission Fault 0 */
  LLC_FSC_FSCMF_tag FSCMF[3];          /* FSC Mission Fault Inputs */
  LLC_FSC_FSCCETHF0_tag FSCCETHF0;     /* FSC Correctable Error Over Theshold 0 */
  LLC_FSC_FSCCETHF_tag FSCCETHF[3];    /* FSC Correctable Error Over Threshold 1 */
};


/* ============================================================================
   =============================== Module: LMEM64 =============================
   ============================================================================ */

typedef union LMEM64_PCCCR_union_tag { /* PC bus Cache control register */
  vuint32_t R;
  struct {
    vuint32_t ENCACHE:1;               /* Cache enable */
    vuint32_t ENWRBUF:1;               /* Enable Write Buffer */
    vuint32_t PCCR2:1;                 /* Forces all cacheable spaces to write through */
    vuint32_t PCCR3:1;                 /* Forces no allocation on cache misses */
    vuint32_t _unused_4:20;            /* Reserved */
    vuint32_t INVW0:1;                 /* Invalidate Way 0 */
    vuint32_t PUSHW0:1;                /* Push Way 0 */
    vuint32_t INVW1:1;                 /* Invalidate Way 1 */
    vuint32_t PUSHW1:1;                /* Push Way 1 */
    vuint32_t _unused_28:3;            /* Reserved */
    vuint32_t GO:1;                    /* Initiate Cache Command */
  } B;
} LMEM64_PCCCR_tag;

typedef union LMEM64_PCCLCR_union_tag { /* PC bus Cache line control register */
  vuint32_t R;
  struct {
    vuint32_t LGO:1;                   /* Initiate Cache Line Command */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t CACHEADDR:12;            /* Cache address */
    vuint32_t WSEL:1;                  /* Way select */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t TDSEL:1;                 /* Tag/Data Select */
    vuint32_t _unused_17:3;            /* Reserved */
    vuint32_t LCIVB:1;                 /* Line Command Initial Valid Bit */
    vuint32_t LCIMB:1;                 /* Line Command Initial Modified Bit */
    vuint32_t LCWAY:1;                 /* Line Command Way */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t LCMD:2;                  /* Line Command */
    vuint32_t LADSEL:1;                /* Line Address Select */
    vuint32_t LACC:1;                  /* Line access type */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} LMEM64_PCCLCR_tag;

typedef union LMEM64_PCCSAR_union_tag { /* PC bus Cache search address register */
  vuint32_t R;
  struct {
    vuint32_t LGO:1;                   /* Initiate Cache Line Command */
    vuint32_t PHYADDR:31;              /* Physical Address */
  } B;
} LMEM64_PCCSAR_tag;

typedef union LMEM64_PCCCVR_union_tag { /* PC bus Cache read/write value register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;                 /* Cache read/write Data */
  } B;
} LMEM64_PCCCVR_tag;

typedef union LMEM64_PSCCR_union_tag { /* PS bus Cache control register */
  vuint32_t R;
  struct {
    vuint32_t ENCACHE:1;               /* Cache enable */
    vuint32_t ENWRBUF:1;               /* Enable Write Buffer */
    vuint32_t PSCR2:1;                 /* Forces all cacheable spaces to write through */
    vuint32_t PSCR3:1;                 /* Forces no allocation on cache misses */
    vuint32_t _unused_4:20;            /* Reserved */
    vuint32_t INVW0:1;                 /* Invalidate Way 0 */
    vuint32_t PUSHW0:1;                /* Push Way 0 */
    vuint32_t INVW1:1;                 /* Invalidate Way 1 */
    vuint32_t PUSHW1:1;                /* Push Way 1 */
    vuint32_t _unused_28:3;            /* Reserved */
    vuint32_t GO:1;                    /* Initiate Cache Command */
  } B;
} LMEM64_PSCCR_tag;

typedef union LMEM64_PSCLCR_union_tag { /* PS bus Cache line control register */
  vuint32_t R;
  struct {
    vuint32_t LGO:1;                   /* Initiate Cache Line Command */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t CACHEADDR:12;            /* Cache address */
    vuint32_t WSEL:1;                  /* Way select */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t TDSEL:1;                 /* Tag/Data Select */
    vuint32_t _unused_17:3;            /* Reserved */
    vuint32_t LCIVB:1;                 /* Line Command Initial Valid Bit */
    vuint32_t LCIMB:1;                 /* Line Command Initial Modified Bit */
    vuint32_t LCWAY:1;                 /* Line Command Way */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t LCMD:2;                  /* Line Command */
    vuint32_t LADSEL:1;                /* Line Address Select */
    vuint32_t LACC:1;                  /* Line access type */
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} LMEM64_PSCLCR_tag;

typedef union LMEM64_PSCSAR_union_tag { /* PS bus Cache search address register */
  vuint32_t R;
  struct {
    vuint32_t LGO:1;                   /* Initiate Cache Line Command */
    vuint32_t PHYADDR:31;              /* Physical Address */
  } B;
} LMEM64_PSCSAR_tag;

typedef union LMEM64_PSCCVR_union_tag { /* PS bus Cache read/write value register */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;                 /* Cache read/write Data */
  } B;
} LMEM64_PSCCVR_tag;

struct LMEM64_tag {
  LMEM64_PCCCR_tag PCCCR;              /* PC bus Cache control register */
  LMEM64_PCCLCR_tag PCCLCR;            /* PC bus Cache line control register */
  LMEM64_PCCSAR_tag PCCSAR;            /* PC bus Cache search address register */
  LMEM64_PCCCVR_tag PCCCVR;            /* PC bus Cache read/write value register */
  uint8_t LMEM64_reserved0[2032];
  LMEM64_PSCCR_tag PSCCR;              /* PS bus Cache control register */
  LMEM64_PSCLCR_tag PSCLCR;            /* PS bus Cache line control register */
  LMEM64_PSCSAR_tag PSCSAR;            /* PS bus Cache search address register */
  LMEM64_PSCCVR_tag PSCCVR;            /* PS bus Cache read/write value register */
};


/* ============================================================================
   =============================== Module: LPI2C ==============================
   ============================================================================ */

typedef union LPI2C_VERID_union_tag {  /* Version ID */
  vuint32_t R;
  struct {
    vuint32_t FEATURE:16;              /* Feature Specification Number */
    vuint32_t MINOR:8;                 /* Minor Version Number */
    vuint32_t MAJOR:8;                 /* Major Version Number */
  } B;
} LPI2C_VERID_tag;

typedef union LPI2C_PARAM_union_tag {  /* Parameter */
  vuint32_t R;
  struct {
    vuint32_t MTXFIFO:4;               /* Controller Transmit FIFO Size */
    vuint32_t _unused_4:4;
    vuint32_t MRXFIFO:4;               /* Controller Receive FIFO Size */
    vuint32_t _unused_12:4;
    vuint32_t _unused_16:16;
  } B;
} LPI2C_PARAM_tag;

typedef union LPI2C_MCR_union_tag {    /* Controller Control */
  vuint32_t R;
  struct {
    vuint32_t MEN:1;                   /* Controller Enable */
    vuint32_t RST:1;                   /* Software Reset */
    vuint32_t DOZEN:1;                 /* Doze Mode Enable */
    vuint32_t DBGEN:1;                 /* Debug Enable */
    vuint32_t _unused_4:4;
    vuint32_t RTF:1;                   /* Reset Transmit FIFO */
    vuint32_t RRF:1;                   /* Reset Receive FIFO */
    vuint32_t _unused_10:22;
  } B;
} LPI2C_MCR_tag;

typedef union LPI2C_MSR_union_tag {    /* Controller Status */
  vuint32_t R;
  struct {
    vuint32_t TDF:1;                   /* Transmit Data Flag */
    vuint32_t RDF:1;                   /* Receive Data Flag */
    vuint32_t _unused_2:6;
    vuint32_t EPF:1;                   /* End Packet Flag */
    vuint32_t SDF:1;                   /* Stop Detect Flag */
    vuint32_t NDF:1;                   /* NACK Detect Flag */
    vuint32_t ALF:1;                   /* Arbitration Lost Flag */
    vuint32_t FEF:1;                   /* FIFO Error Flag */
    vuint32_t PLTF:1;                  /* Pin Low Timeout Flag */
    vuint32_t DMF:1;                   /* Data Match Flag */
    vuint32_t STF:1;                   /* Start Flag */
    vuint32_t _unused_16:8;
    vuint32_t MBF:1;                   /* Controller Busy Flag */
    vuint32_t BBF:1;                   /* Bus Busy Flag */
    vuint32_t _unused_26:6;
  } B;
} LPI2C_MSR_tag;

typedef union LPI2C_MIER_union_tag {   /* Controller Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t TDIE:1;                  /* Transmit Data Interrupt Enable */
    vuint32_t RDIE:1;                  /* Receive Data Interrupt Enable */
    vuint32_t _unused_2:6;
    vuint32_t EPIE:1;                  /* End Packet Interrupt Enable */
    vuint32_t SDIE:1;                  /* Stop Detect Interrupt Enable */
    vuint32_t NDIE:1;                  /* NACK Detect Interrupt Enable */
    vuint32_t ALIE:1;                  /* Arbitration Lost Interrupt Enable */
    vuint32_t FEIE:1;                  /* FIFO Error Interrupt Enable */
    vuint32_t PLTIE:1;                 /* Pin Low Timeout Interrupt Enable */
    vuint32_t DMIE:1;                  /* Data Match Interrupt Enable */
    vuint32_t STIE:1;                  /* Start Interrupt Enable */
    vuint32_t _unused_16:16;
  } B;
} LPI2C_MIER_tag;

typedef union LPI2C_MDER_union_tag {   /* Controller DMA Enable */
  vuint32_t R;
  struct {
    vuint32_t TDDE:1;                  /* Transmit Data DMA Enable */
    vuint32_t RDDE:1;                  /* Receive Data DMA Enable */
    vuint32_t _unused_2:30;
  } B;
} LPI2C_MDER_tag;

typedef union LPI2C_MCFGR0_union_tag { /* Controller Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t HREN:1;                  /* Host Request Enable */
    vuint32_t HRPOL:1;                 /* Host Request Polarity */
    vuint32_t HRSEL:1;                 /* Host Request Select */
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:4;
    vuint32_t CIRFIFO:1;               /* Circular FIFO Enable */
    vuint32_t RDMO:1;                  /* Receive Data Match Only */
    vuint32_t _unused_10:6;
    vuint32_t RELAX:1;                 /* Relaxed Mode */
    vuint32_t ABORT:1;                 /* Abort Transfer */
    vuint32_t _unused_18:14;
  } B;
} LPI2C_MCFGR0_tag;

typedef union LPI2C_MCFGR1_union_tag { /* Controller Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t PRESCALE:3;              /* Prescaler */
    vuint32_t _unused_3:5;
    vuint32_t AUTOSTOP:1;              /* Automatic Stop Generation */
    vuint32_t IGNACK:1;                /* Ignore NACK */
    vuint32_t TIMECFG:1;               /* Timeout Configuration */
    vuint32_t STOPCFG:1;               /* Stop Configuration */
    vuint32_t STARTCFG:1;              /* Start Configuration */
    vuint32_t _unused_13:3;
    vuint32_t MATCFG:3;                /* Match Configuration */
    vuint32_t _unused_19:5;
    vuint32_t PINCFG:3;                /* Pin Configuration */
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:4;
  } B;
} LPI2C_MCFGR1_tag;

typedef union LPI2C_MCFGR2_union_tag { /* Controller Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t BUSIDLE:12;              /* Bus Idle Timeout */
    vuint32_t _unused_12:4;
    vuint32_t FILTSCL:4;               /* Glitch Filter SCL */
    vuint32_t _unused_20:4;
    vuint32_t FILTSDA:4;               /* Glitch Filter SDA */
    vuint32_t _unused_28:4;
  } B;
} LPI2C_MCFGR2_tag;

typedef union LPI2C_MCFGR3_union_tag { /* Controller Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t PINLOW:12;               /* Pin Low Timeout */
    vuint32_t _unused_20:12;
  } B;
} LPI2C_MCFGR3_tag;

typedef union LPI2C_MDMR_union_tag {   /* Controller Data Match */
  vuint32_t R;
  struct {
    vuint32_t MATCH0:8;                /* Match 0 Value */
    vuint32_t _unused_8:8;
    vuint32_t MATCH1:8;                /* Match 1 Value */
    vuint32_t _unused_24:8;
  } B;
} LPI2C_MDMR_tag;

typedef union LPI2C_MCCR0_union_tag {  /* Controller Clock Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t CLKLO:6;                 /* Clock Low Period */
    vuint32_t _unused_6:2;
    vuint32_t CLKHI:6;                 /* Clock High Period */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t SETHOLD:6;               /* Setup Hold Delay */
    vuint32_t _unused_22:2;
    vuint32_t DATAVD:6;                /* Data Valid Delay */
    vuint32_t _unused_30:2;
  } B;
} LPI2C_MCCR0_tag;

typedef union LPI2C_MCCR1_union_tag {  /* Controller Clock Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t CLKLO:6;                 /* Clock Low Period */
    vuint32_t _unused_6:2;
    vuint32_t CLKHI:6;                 /* Clock High Period */
    vuint32_t _unused_14:2;
    vuint32_t SETHOLD:6;               /* Setup Hold Delay */
    vuint32_t _unused_22:2;
    vuint32_t DATAVD:6;                /* Data Valid Delay */
    vuint32_t _unused_30:2;
  } B;
} LPI2C_MCCR1_tag;

typedef union LPI2C_MFCR_union_tag {   /* Controller FIFO Control */
  vuint32_t R;
  struct {
    vuint32_t TXWATER:3;               /* Transmit FIFO Watermark */
    vuint32_t _unused_3:13;
    vuint32_t RXWATER:3;               /* Receive FIFO Watermark */
    vuint32_t _unused_19:13;
  } B;
} LPI2C_MFCR_tag;

typedef union LPI2C_MFSR_union_tag {   /* Controller FIFO Status */
  vuint32_t R;
  struct {
    vuint32_t TXCOUNT:4;               /* Transmit FIFO Count */
    vuint32_t _unused_4:12;
    vuint32_t RXCOUNT:4;               /* Receive FIFO Count */
    vuint32_t _unused_20:12;
  } B;
} LPI2C_MFSR_tag;

typedef union LPI2C_MTDR_union_tag {   /* Controller Transmit Data */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Transmit Data */
    vuint32_t CMD:3;                   /* Command Data */
    vuint32_t _unused_11:21;
  } B;
} LPI2C_MTDR_tag;

typedef union LPI2C_MRDR_union_tag {   /* Controller Receive Data */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Receive Data */
    vuint32_t _unused_8:6;
    vuint32_t RXEMPTY:1;               /* Receive Empty */
    vuint32_t _unused_15:17;
  } B;
} LPI2C_MRDR_tag;

typedef union LPI2C_MRDROR_union_tag { /* Controller Receive Data Read Only */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Receive Data */
    vuint32_t _unused_8:6;
    vuint32_t RXEMPTY:1;               /* RX Empty */
    vuint32_t _unused_15:17;
  } B;
} LPI2C_MRDROR_tag;

typedef union LPI2C_SCR_union_tag {    /* Target Control */
  vuint32_t R;
  struct {
    vuint32_t SEN:1;                   /* Target Enable */
    vuint32_t RST:1;                   /* Software Reset */
    vuint32_t _unused_2:2;
    vuint32_t FILTEN:1;                /* Filter Enable */
    vuint32_t FILTDZ:1;                /* Filter Doze Enable */
    vuint32_t _unused_6:2;
    vuint32_t RTF:1;                   /* Reset Transmit FIFO */
    vuint32_t RRF:1;                   /* Reset Receive FIFO */
    vuint32_t _unused_10:22;
  } B;
} LPI2C_SCR_tag;

typedef union LPI2C_SSR_union_tag {    /* Target Status */
  vuint32_t R;
  struct {
    vuint32_t TDF:1;                   /* Transmit Data Flag */
    vuint32_t RDF:1;                   /* Receive Data Flag */
    vuint32_t AVF:1;                   /* Address Valid Flag */
    vuint32_t TAF:1;                   /* Transmit ACK Flag */
    vuint32_t _unused_4:4;
    vuint32_t RSF:1;                   /* Repeated Start Flag */
    vuint32_t SDF:1;                   /* Stop Detect Flag */
    vuint32_t BEF:1;                   /* Bit Error Flag */
    vuint32_t FEF:1;                   /* FIFO Error Flag */
    vuint32_t AM0F:1;                  /* Address Match 0 Flag */
    vuint32_t AM1F:1;                  /* Address Match 1 Flag */
    vuint32_t GCF:1;                   /* General Call Flag */
    vuint32_t SARF:1;                  /* SMBus Alert Response Flag */
    vuint32_t _unused_16:8;
    vuint32_t SBF:1;                   /* Target Busy Flag */
    vuint32_t BBF:1;                   /* Bus Busy Flag */
    vuint32_t _unused_26:6;
  } B;
} LPI2C_SSR_tag;

typedef union LPI2C_SIER_union_tag {   /* Target Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t TDIE:1;                  /* Transmit Data Interrupt Enable */
    vuint32_t RDIE:1;                  /* Receive Data Interrupt Enable */
    vuint32_t AVIE:1;                  /* Address Valid Interrupt Enable */
    vuint32_t TAIE:1;                  /* Transmit ACK Interrupt Enable */
    vuint32_t _unused_4:4;
    vuint32_t RSIE:1;                  /* Repeated Start Interrupt Enable */
    vuint32_t SDIE:1;                  /* Stop Detect Interrupt Enable */
    vuint32_t BEIE:1;                  /* Bit Error Interrupt Enable */
    vuint32_t FEIE:1;                  /* FIFO Error Interrupt Enable */
    vuint32_t AM0IE:1;                 /* Address Match 0 Interrupt Enable */
    vuint32_t AM1IE:1;                 /* Address Match 1 Interrupt Enable */
    vuint32_t GCIE:1;                  /* General Call Interrupt Enable */
    vuint32_t SARIE:1;                 /* SMBus Alert Response Interrupt Enable */
    vuint32_t _unused_16:16;
  } B;
} LPI2C_SIER_tag;

typedef union LPI2C_SDER_union_tag {   /* Target DMA Enable */
  vuint32_t R;
  struct {
    vuint32_t TDDE:1;                  /* Transmit Data DMA Enable */
    vuint32_t RDDE:1;                  /* Receive Data DMA Enable */
    vuint32_t AVDE:1;                  /* Address Valid DMA Enable */
    vuint32_t _unused_3:29;
  } B;
} LPI2C_SDER_tag;

typedef union LPI2C_SCFGR0_union_tag { /* Target Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t RDREQ:1;                 /* Read Request */
    vuint32_t RDACK:1;                 /* Read Acknowledge Flag */
    vuint32_t _unused_2:30;
  } B;
} LPI2C_SCFGR0_tag;

typedef union LPI2C_SCFGR1_union_tag { /* Target Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t ADRSTALL:1;              /* Address SCL Stall */
    vuint32_t RXSTALL:1;               /* RX SCL Stall */
    vuint32_t TXDSTALL:1;              /* Transmit Data SCL Stall */
    vuint32_t ACKSTALL:1;              /* ACK SCL Stall */
    vuint32_t RXNACK:1;                /* Receive NACK */
    vuint32_t _unused_5:3;
    vuint32_t GCEN:1;                  /* General Call Enable */
    vuint32_t SAEN:1;                  /* SMBus Alert Enable */
    vuint32_t TXCFG:1;                 /* Transmit Flag Configuration */
    vuint32_t RXCFG:1;                 /* Receive Data Configuration */
    vuint32_t IGNACK:1;                /* Ignore NACK */
    vuint32_t HSMEN:1;                 /* HS Mode Enable */
    vuint32_t _unused_14:2;
    vuint32_t ADDRCFG:3;               /* Address Configuration */
    vuint32_t _unused_19:5;
    vuint32_t RXALL:1;                 /* Receive All */
    vuint32_t RSCFG:1;                 /* Repeated Start Configuration */
    vuint32_t SDCFG:1;                 /* Stop Detect Configuration */
    vuint32_t _unused_27:5;
  } B;
} LPI2C_SCFGR1_tag;

typedef union LPI2C_SCFGR2_union_tag { /* Target Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t CLKHOLD:4;               /* Clock Hold Time */
    vuint32_t _unused_4:4;
    vuint32_t DATAVD:6;                /* Data Valid Delay */
    vuint32_t _unused_14:2;
    vuint32_t FILTSCL:4;               /* Glitch Filter SCL */
    vuint32_t _unused_20:4;
    vuint32_t FILTSDA:4;               /* Glitch Filter SDA */
    vuint32_t _unused_28:4;
  } B;
} LPI2C_SCFGR2_tag;

typedef union LPI2C_SAMR_union_tag {   /* Target Address Match */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t ADDR0:10;                /* Address 0 Value */
    vuint32_t _unused_11:6;
    vuint32_t ADDR1:10;                /* Address 1 Value */
    vuint32_t _unused_27:5;
  } B;
} LPI2C_SAMR_tag;

typedef union LPI2C_SASR_union_tag {   /* Target Address Status */
  vuint32_t R;
  struct {
    vuint32_t RADDR:11;                /* Received Address */
    vuint32_t _unused_11:3;
    vuint32_t ANV:1;                   /* Address Not Valid */
    vuint32_t _unused_15:17;
  } B;
} LPI2C_SASR_tag;

typedef union LPI2C_STAR_union_tag {   /* Target Transmit ACK */
  vuint32_t R;
  struct {
    vuint32_t TXNACK:1;                /* Transmit NACK */
    vuint32_t _unused_1:31;
  } B;
} LPI2C_STAR_tag;

typedef union LPI2C_STDR_union_tag {   /* Target Transmit Data */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Transmit Data */
    vuint32_t _unused_8:24;
  } B;
} LPI2C_STDR_tag;

typedef union LPI2C_SRDR_union_tag {   /* Target Receive Data */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Received Data */
    vuint32_t RADDR:3;                 /* Received Address */
    vuint32_t _unused_11:3;
    vuint32_t RXEMPTY:1;               /* Receive Empty */
    vuint32_t SOF:1;                   /* Start of Frame */
    vuint32_t _unused_16:16;
  } B;
} LPI2C_SRDR_tag;

typedef union LPI2C_SRDROR_union_tag { /* Target Receive Data Read Only */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Receive Data */
    vuint32_t RADDR:3;                 /* Received Address */
    vuint32_t _unused_11:3;
    vuint32_t RXEMPTY:1;               /* Receive Empty */
    vuint32_t SOF:1;                   /* Start of Frame */
    vuint32_t _unused_16:16;
  } B;
} LPI2C_SRDROR_tag;

typedef union LPI2C_MTCBR_union_tag {  /* Controller Transmit Command Burst */
  vuint32_t R;
  struct {
    vuint32_t DATA:8;                  /* Data */
    vuint32_t CMD:3;                   /* Command */
    vuint32_t _unused_11:21;           /* Reserved */
  } B;
} LPI2C_MTCBR_tag;

typedef union LPI2C_MTDBR_union_tag {  /* Transmit Data Burst */
  vuint32_t R;
  struct {
    vuint32_t DATA0:8;                 /* Data */
    vuint32_t DATA1:8;                 /* Data */
    vuint32_t DATA2:8;                 /* Data */
    vuint32_t DATA3:8;                 /* Data */
  } B;
} LPI2C_MTDBR_tag;

struct LPI2C_tag {
  LPI2C_VERID_tag VERID;               /* Version ID */
  LPI2C_PARAM_tag PARAM;               /* Parameter */
  uint8_t LPI2C_reserved0[8];
  LPI2C_MCR_tag MCR;                   /* Controller Control */
  LPI2C_MSR_tag MSR;                   /* Controller Status */
  LPI2C_MIER_tag MIER;                 /* Controller Interrupt Enable */
  LPI2C_MDER_tag MDER;                 /* Controller DMA Enable */
  LPI2C_MCFGR0_tag MCFGR0;             /* Controller Configuration 0 */
  LPI2C_MCFGR1_tag MCFGR1;             /* Controller Configuration 1 */
  LPI2C_MCFGR2_tag MCFGR2;             /* Controller Configuration 2 */
  LPI2C_MCFGR3_tag MCFGR3;             /* Controller Configuration 3 */
  uint8_t LPI2C_reserved1[16];
  LPI2C_MDMR_tag MDMR;                 /* Controller Data Match */
  uint8_t LPI2C_reserved2[4];
  LPI2C_MCCR0_tag MCCR0;               /* Controller Clock Configuration 0 */
  uint8_t LPI2C_reserved3[4];
  LPI2C_MCCR1_tag MCCR1;               /* Controller Clock Configuration 1 */
  uint8_t LPI2C_reserved4[4];
  LPI2C_MFCR_tag MFCR;                 /* Controller FIFO Control */
  LPI2C_MFSR_tag MFSR;                 /* Controller FIFO Status */
  LPI2C_MTDR_tag MTDR;                 /* Controller Transmit Data */
  uint8_t LPI2C_reserved5[12];
  LPI2C_MRDR_tag MRDR;                 /* Controller Receive Data */
  uint8_t LPI2C_reserved6[4];
  LPI2C_MRDROR_tag MRDROR;             /* Controller Receive Data Read Only */
  uint8_t LPI2C_reserved7[148];
  LPI2C_SCR_tag SCR;                   /* Target Control */
  LPI2C_SSR_tag SSR;                   /* Target Status */
  LPI2C_SIER_tag SIER;                 /* Target Interrupt Enable */
  LPI2C_SDER_tag SDER;                 /* Target DMA Enable */
  LPI2C_SCFGR0_tag SCFGR0;             /* Target Configuration 0 */
  LPI2C_SCFGR1_tag SCFGR1;             /* Target Configuration 1 */
  LPI2C_SCFGR2_tag SCFGR2;             /* Target Configuration 2 */
  uint8_t LPI2C_reserved8[20];
  LPI2C_SAMR_tag SAMR;                 /* Target Address Match */
  uint8_t LPI2C_reserved9[12];
  LPI2C_SASR_tag SASR;                 /* Target Address Status */
  LPI2C_STAR_tag STAR;                 /* Target Transmit ACK */
  uint8_t LPI2C_reserved10[8];
  LPI2C_STDR_tag STDR;                 /* Target Transmit Data */
  uint8_t LPI2C_reserved11[12];
  LPI2C_SRDR_tag SRDR;                 /* Target Receive Data */
  uint8_t LPI2C_reserved12[4];
  LPI2C_SRDROR_tag SRDROR;             /* Target Receive Data Read Only */
  uint8_t LPI2C_reserved13[132];
  LPI2C_MTCBR_tag MTCBR[128];          /* Controller Transmit Command Burst */
  LPI2C_MTDBR_tag MTDBR[256];          /* Transmit Data Burst */
};


/* ============================================================================
   =============================== Module: LSTCU ==============================
   ============================================================================ */

typedef union LSTCU_ERR_STAT_union_tag { /* Error Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVP_MB:1;               /* Invalid Pointer MBIST */
    vuint32_t INVP_LB:1;               /* Invalid Pointer LBIST */
    vuint32_t _unused_3:13;
    vuint32_t UFSF:1;                  /* Unrecoverable Fault Status */
    vuint32_t RFSF:1;                  /* Recoverable Fault Status */
    vuint32_t _unused_18:14;
  } B;
} LSTCU_ERR_STAT_tag;

typedef union LSTCU_ERR_FM_union_tag { /* Error Fault Mapping */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVPFMMB:1;              /* Invalid BIST Pointer Fault Mapping During MBIST Scheduling */
    vuint32_t INVPFMLB:1;              /* Invalid BIST Pointer Fault Mapping During LBIST Scheduling */
    vuint32_t _unused_3:29;
  } B;
} LSTCU_ERR_FM_tag;

typedef union LSTCU_MB_RSTAT0_union_tag { /* MBIST Run Status 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTAT0:1;               /* MBIST Run Result Status 0 */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_MB_RSTAT0_tag;

typedef union LSTCU_LB_RSTAT0_union_tag { /* LBIST Run Status 0 */
  vuint32_t R;
  struct {
    vuint32_t LBSTAT0:1;               /* LBIST Run Result Status 0 */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_LB_RSTAT0_tag;

typedef union LSTCU_MBFM0_union_tag {  /* MBIST Fault Mapping 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTATFM0:1;             /* MBIST Fault Mapping n */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_MBFM0_tag;

typedef union LSTCU_LBFM0_union_tag {  /* LBIST Fault Mapping 0 */
  vuint32_t R;
  struct {
    vuint32_t LBSTATFM0:1;             /* LBIST Fault Mapping n */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_LBFM0_tag;

typedef union LSTCU_STAG_union_tag {   /* Stagger */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MB_DELAY:8;              /* MBIST Delay */
    vuint32_t LB_DELAY:8;              /* LBIST Delay */
    vuint32_t _unused_24:8;
  } B;
} LSTCU_STAG_tag;

typedef union LSTCU_PH1_DUR_union_tag { /* Phase 1 Duration */
  vuint32_t R;
  struct {
    vuint32_t PH1DUR:10;               /* Phase 1 Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_PH1_DUR_tag;

typedef union LSTCU_ISODUR_union_tag { /* Isolation Duration */
  vuint32_t R;
  struct {
    vuint32_t EN_DUR:10;               /* Enable Duration */
    vuint32_t _unused_10:6;
    vuint32_t DIS_DUR:10;              /* Disable Duration */
    vuint32_t _unused_26:6;
  } B;
} LSTCU_ISODUR_tag;

typedef union LSTCU_SRCENDUR_union_tag { /* Synchronous Reset Clock Enable Duration */
  vuint32_t R;
  struct {
    vuint32_t CLKENDUR:10;             /* Synchronous Reset Clock Enable Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_SRCENDUR_tag;

typedef union LSTCU_MBPTR_union_tag {  /* MBIST Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t MBPTR:8;                 /* MBIST Pointer */
    vuint32_t MBCSM:1;                 /* MBIST Mode */
    vuint32_t _unused_9:22;
    vuint32_t MBEOL:1;                 /* MBIST End of List */
  } B;
} LSTCU_MBPTR_tag;

typedef union LSTCU_LBPTR_union_tag {  /* LBIST Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t LBPTR:8;                 /* LBIST Pointer */
    vuint32_t LBCSM:1;                 /* LBIST Mode */
    vuint32_t _unused_9:22;
    vuint32_t LBEOL:1;                 /* LBIST End of List */
  } B;
} LSTCU_LBPTR_tag;

struct LSTCU_tag {
  uint8_t LSTCU_reserved0[8];
  LSTCU_ERR_STAT_tag ERR_STAT;         /* Error Status */
  uint8_t LSTCU_reserved1[4];
  LSTCU_ERR_FM_tag ERR_FM;             /* Error Fault Mapping */
  uint8_t LSTCU_reserved2[76];
  LSTCU_MB_RSTAT0_tag MB_RSTAT0;       /* MBIST Run Status 0 */
  uint8_t LSTCU_reserved3[156];
  LSTCU_LB_RSTAT0_tag LB_RSTAT0;       /* LBIST Run Status 0 */
  uint8_t LSTCU_reserved4[124];
  LSTCU_MBFM0_tag MBFM0;               /* MBIST Fault Mapping 0 */
  uint8_t LSTCU_reserved5[156];
  LSTCU_LBFM0_tag LBFM0;               /* LBIST Fault Mapping 0 */
  uint8_t LSTCU_reserved6[60];
  LSTCU_STAG_tag STAG;                 /* Stagger */
  uint8_t LSTCU_reserved7[12];
  LSTCU_PH1_DUR_tag PH1_DUR;           /* Phase 1 Duration */
  LSTCU_ISODUR_tag ISODUR;             /* Isolation Duration */
  LSTCU_SRCENDUR_tag SRCENDUR;         /* Synchronous Reset Clock Enable Duration */
  uint8_t LSTCU_reserved8[132];
  LSTCU_MBPTR_tag MBPTR[1];            /* MBIST Scheduler Pointer */
  uint8_t LSTCU_reserved9[156];
  LSTCU_LBPTR_tag LBPTR[1];            /* LBIST Scheduler Pointer */
};


/* ============================================================================
   =============================== Module: LSTCU_14_15_17_18 ==================
   ============================================================================ */

typedef union LSTCU_14_15_17_18_ERR_STAT_union_tag { /* Error Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVP_MB:1;               /* Invalid Pointer MBIST */
    vuint32_t INVP_LB:1;               /* Invalid Pointer LBIST */
    vuint32_t _unused_3:13;
    vuint32_t UFSF:1;                  /* Unrecoverable Fault Status */
    vuint32_t RFSF:1;                  /* Recoverable Fault Status */
    vuint32_t _unused_18:14;
  } B;
} LSTCU_14_15_17_18_ERR_STAT_tag;

typedef union LSTCU_14_15_17_18_ERR_FM_union_tag { /* Error Fault Mapping */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVPFMMB:1;              /* Invalid BIST Pointer Fault Mapping During MBIST Scheduling */
    vuint32_t INVPFMLB:1;              /* Invalid BIST Pointer Fault Mapping During LBIST Scheduling */
    vuint32_t _unused_3:29;
  } B;
} LSTCU_14_15_17_18_ERR_FM_tag;

typedef union LSTCU_14_15_17_18_MB_RSTAT0_union_tag { /* MBIST Run Status 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTAT0:1;               /* MBIST Run Result Status 0 */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_14_15_17_18_MB_RSTAT0_tag;

typedef union LSTCU_14_15_17_18_MB_RSTAT1_union_tag { /* MBIST Run Status 1 */
  vuint32_t R;
  struct {
    vuint32_t MBSTAT0:1;               /* MBIST Run Result Status 0 */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_14_15_17_18_MB_RSTAT1_tag;

typedef union LSTCU_14_15_17_18_LB_RSTAT0_union_tag { /* LBIST Run Status 0 */
  vuint32_t R;
  struct {
    vuint32_t LBSTAT0:1;               /* LBIST Run Result Status 0 */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_14_15_17_18_LB_RSTAT0_tag;

typedef union LSTCU_14_15_17_18_MBFM0_union_tag { /* MBIST Fault Mapping 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTATFM0:1;             /* MBIST Fault Mapping n */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_14_15_17_18_MBFM0_tag;

typedef union LSTCU_14_15_17_18_MBFM1_union_tag { /* MBIST Fault Mapping 1 */
  vuint32_t R;
  struct {
    vuint32_t MBSTATFM0:1;             /* MBIST Fault Mapping n */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_14_15_17_18_MBFM1_tag;

typedef union LSTCU_14_15_17_18_LBFM0_union_tag { /* LBIST Fault Mapping 0 */
  vuint32_t R;
  struct {
    vuint32_t LBSTATFM0:1;             /* LBIST Fault Mapping n */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_14_15_17_18_LBFM0_tag;

typedef union LSTCU_14_15_17_18_STAG_union_tag { /* Stagger */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MB_DELAY:8;              /* MBIST Delay */
    vuint32_t LB_DELAY:8;              /* LBIST Delay */
    vuint32_t _unused_24:8;
  } B;
} LSTCU_14_15_17_18_STAG_tag;

typedef union LSTCU_14_15_17_18_PH1_DUR_union_tag { /* Phase 1 Duration */
  vuint32_t R;
  struct {
    vuint32_t PH1DUR:10;               /* Phase 1 Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_14_15_17_18_PH1_DUR_tag;

typedef union LSTCU_14_15_17_18_MBPTR_union_tag { /* MBIST Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t MBPTR:8;                 /* MBIST Pointer */
    vuint32_t MBCSM:1;                 /* MBIST Mode */
    vuint32_t _unused_9:22;
    vuint32_t MBEOL:1;                 /* MBIST End of List */
  } B;
} LSTCU_14_15_17_18_MBPTR_tag;

typedef union LSTCU_14_15_17_18_LBPTR_union_tag { /* LBIST Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t LBPTR:8;                 /* LBIST Pointer */
    vuint32_t LBCSM:1;                 /* LBIST Mode */
    vuint32_t _unused_9:22;
    vuint32_t LBEOL:1;                 /* LBIST End of List */
  } B;
} LSTCU_14_15_17_18_LBPTR_tag;

struct LSTCU_14_15_17_18_tag {
  uint8_t LSTCU_14_15_17_18_reserved0[8];
  LSTCU_14_15_17_18_ERR_STAT_tag ERR_STAT; /* Error Status */
  uint8_t LSTCU_14_15_17_18_reserved1[4];
  LSTCU_14_15_17_18_ERR_FM_tag ERR_FM; /* Error Fault Mapping */
  uint8_t LSTCU_14_15_17_18_reserved2[76];
  LSTCU_14_15_17_18_MB_RSTAT0_tag MB_RSTAT0; /* MBIST Run Status 0 */
  LSTCU_14_15_17_18_MB_RSTAT1_tag MB_RSTAT1; /* MBIST Run Status 1 */
  uint8_t LSTCU_14_15_17_18_reserved3[152];
  LSTCU_14_15_17_18_LB_RSTAT0_tag LB_RSTAT0; /* LBIST Run Status 0 */
  uint8_t LSTCU_14_15_17_18_reserved4[124];
  LSTCU_14_15_17_18_MBFM0_tag MBFM0;   /* MBIST Fault Mapping 0 */
  LSTCU_14_15_17_18_MBFM1_tag MBFM1;   /* MBIST Fault Mapping 1 */
  uint8_t LSTCU_14_15_17_18_reserved5[152];
  LSTCU_14_15_17_18_LBFM0_tag LBFM0;   /* LBIST Fault Mapping 0 */
  uint8_t LSTCU_14_15_17_18_reserved6[60];
  LSTCU_14_15_17_18_STAG_tag STAG;     /* Stagger */
  uint8_t LSTCU_14_15_17_18_reserved7[12];
  LSTCU_14_15_17_18_PH1_DUR_tag PH1_DUR; /* Phase 1 Duration */
  uint8_t LSTCU_14_15_17_18_reserved8[140];
  LSTCU_14_15_17_18_MBPTR_tag MBPTR[2]; /* MBIST Scheduler Pointer */
  uint8_t LSTCU_14_15_17_18_reserved9[152];
  LSTCU_14_15_17_18_LBPTR_tag LBPTR[1]; /* LBIST Scheduler Pointer */
};


/* ============================================================================
   =============================== Module: LSTCU_19 ===========================
   ============================================================================ */

typedef union LSTCU_19_ERR_STAT_union_tag { /* Error Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVP_MB:1;               /* Invalid Pointer MBIST */
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:13;
    vuint32_t UFSF:1;                  /* Unrecoverable Fault Status */
    vuint32_t RFSF:1;                  /* Recoverable Fault Status */
    vuint32_t _unused_18:14;
  } B;
} LSTCU_19_ERR_STAT_tag;

typedef union LSTCU_19_ERR_FM_union_tag { /* Error Fault Mapping */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVPFMMB:1;              /* Invalid BIST Pointer Fault Mapping During MBIST Scheduling */
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:29;
  } B;
} LSTCU_19_ERR_FM_tag;

typedef union LSTCU_19_MB_RSTAT0_union_tag { /* MBIST Run Status 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTAT0:1;               /* MBIST Run Result Status 0 */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_19_MB_RSTAT0_tag;

typedef union LSTCU_19_MBFM0_union_tag { /* MBIST Fault Mapping 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTATFM0:1;             /* MBIST Fault Mapping n */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_19_MBFM0_tag;

typedef union LSTCU_19_STAG_union_tag { /* Stagger */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MB_DELAY:8;              /* MBIST Delay */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:8;
  } B;
} LSTCU_19_STAG_tag;

typedef union LSTCU_19_PH1_DUR_union_tag { /* Phase 1 Duration */
  vuint32_t R;
  struct {
    vuint32_t PH1DUR:10;               /* Phase 1 Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_19_PH1_DUR_tag;

typedef union LSTCU_19_ISODUR_union_tag { /* Isolation Duration */
  vuint32_t R;
  struct {
    vuint32_t EN_DUR:10;               /* Enable Duration */
    vuint32_t _unused_10:6;
    vuint32_t DIS_DUR:10;              /* Disable Duration */
    vuint32_t _unused_26:6;
  } B;
} LSTCU_19_ISODUR_tag;

typedef union LSTCU_19_SRCENDUR_union_tag { /* Synchronous Reset Clock Enable Duration */
  vuint32_t R;
  struct {
    vuint32_t CLKENDUR:10;             /* Synchronous Reset Clock Enable Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_19_SRCENDUR_tag;

typedef union LSTCU_19_MBPTR_union_tag { /* MBIST Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t MBPTR:8;                 /* MBIST Pointer */
    vuint32_t MBCSM:1;                 /* MBIST Mode */
    vuint32_t _unused_9:22;
    vuint32_t MBEOL:1;                 /* MBIST End of List */
  } B;
} LSTCU_19_MBPTR_tag;

struct LSTCU_19_tag {
  uint8_t LSTCU_19_reserved0[8];
  LSTCU_19_ERR_STAT_tag ERR_STAT;      /* Error Status */
  uint8_t LSTCU_19_reserved1[4];
  LSTCU_19_ERR_FM_tag ERR_FM;          /* Error Fault Mapping */
  uint8_t LSTCU_19_reserved2[76];
  LSTCU_19_MB_RSTAT0_tag MB_RSTAT0;    /* MBIST Run Status 0 */
  uint8_t LSTCU_19_reserved3[284];
  LSTCU_19_MBFM0_tag MBFM0;            /* MBIST Fault Mapping 0 */
  uint8_t LSTCU_19_reserved4[220];
  LSTCU_19_STAG_tag STAG;              /* Stagger */
  uint8_t LSTCU_19_reserved5[12];
  LSTCU_19_PH1_DUR_tag PH1_DUR;        /* Phase 1 Duration */
  LSTCU_19_ISODUR_tag ISODUR;          /* Isolation Duration */
  LSTCU_19_SRCENDUR_tag SRCENDUR;      /* Synchronous Reset Clock Enable Duration */
  uint8_t LSTCU_19_reserved6[132];
  LSTCU_19_MBPTR_tag MBPTR[1];         /* MBIST Scheduler Pointer */
};


/* ============================================================================
   =============================== Module: LSTCU_3_7 ==========================
   ============================================================================ */

typedef union LSTCU_3_7_ERR_STAT_union_tag { /* Error Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVP_MB:1;               /* Invalid Pointer MBIST */
    vuint32_t INVP_LB:1;               /* Invalid Pointer LBIST */
    vuint32_t _unused_3:13;
    vuint32_t UFSF:1;                  /* Unrecoverable Fault Status */
    vuint32_t RFSF:1;                  /* Recoverable Fault Status */
    vuint32_t _unused_18:14;
  } B;
} LSTCU_3_7_ERR_STAT_tag;

typedef union LSTCU_3_7_ERR_FM_union_tag { /* Error Fault Mapping */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVPFMMB:1;              /* Invalid BIST Pointer Fault Mapping During MBIST Scheduling */
    vuint32_t INVPFMLB:1;              /* Invalid BIST Pointer Fault Mapping During LBIST Scheduling */
    vuint32_t _unused_3:29;
  } B;
} LSTCU_3_7_ERR_FM_tag;

typedef union LSTCU_3_7_MB_RSTAT0_union_tag { /* MBIST Run Status 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTAT0:1;               /* MBIST Run Result Status 0 */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_3_7_MB_RSTAT0_tag;

typedef union LSTCU_3_7_MBFM0_union_tag { /* MBIST Fault Mapping 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTATFM0:1;             /* MBIST Fault Mapping n */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_3_7_MBFM0_tag;

typedef union LSTCU_3_7_STAG_union_tag { /* Stagger */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MB_DELAY:8;              /* MBIST Delay */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:8;
  } B;
} LSTCU_3_7_STAG_tag;

typedef union LSTCU_3_7_PH1_DUR_union_tag { /* Phase 1 Duration */
  vuint32_t R;
  struct {
    vuint32_t PH1DUR:10;               /* Phase 1 Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_3_7_PH1_DUR_tag;

typedef union LSTCU_3_7_ISODUR_union_tag { /* Isolation Duration */
  vuint32_t R;
  struct {
    vuint32_t EN_DUR:10;               /* Enable Duration */
    vuint32_t _unused_10:6;
    vuint32_t DIS_DUR:10;              /* Disable Duration */
    vuint32_t _unused_26:6;
  } B;
} LSTCU_3_7_ISODUR_tag;

typedef union LSTCU_3_7_SRCENDUR_union_tag { /* Synchronous Reset Clock Enable Duration */
  vuint32_t R;
  struct {
    vuint32_t CLKENDUR:10;             /* Synchronous Reset Clock Enable Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_3_7_SRCENDUR_tag;

typedef union LSTCU_3_7_MBPTR_union_tag { /* MBIST Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t MBPTR:8;                 /* MBIST Pointer */
    vuint32_t MBCSM:1;                 /* MBIST Mode */
    vuint32_t _unused_9:22;
    vuint32_t MBEOL:1;                 /* MBIST End of List */
  } B;
} LSTCU_3_7_MBPTR_tag;

struct LSTCU_3_7_tag {
  uint8_t LSTCU_3_7_reserved0[8];
  LSTCU_3_7_ERR_STAT_tag ERR_STAT;     /* Error Status */
  uint8_t LSTCU_3_7_reserved1[4];
  LSTCU_3_7_ERR_FM_tag ERR_FM;         /* Error Fault Mapping */
  uint8_t LSTCU_3_7_reserved2[76];
  LSTCU_3_7_MB_RSTAT0_tag MB_RSTAT0;   /* MBIST Run Status 0 */
  uint8_t LSTCU_3_7_reserved3[284];
  LSTCU_3_7_MBFM0_tag MBFM0;           /* MBIST Fault Mapping 0 */
  uint8_t LSTCU_3_7_reserved4[220];
  LSTCU_3_7_STAG_tag STAG;             /* Stagger */
  uint8_t LSTCU_3_7_reserved5[12];
  LSTCU_3_7_PH1_DUR_tag PH1_DUR;       /* Phase 1 Duration */
  LSTCU_3_7_ISODUR_tag ISODUR;         /* Isolation Duration */
  LSTCU_3_7_SRCENDUR_tag SRCENDUR;     /* Synchronous Reset Clock Enable Duration */
  uint8_t LSTCU_3_7_reserved6[132];
  LSTCU_3_7_MBPTR_tag MBPTR[1];        /* MBIST Scheduler Pointer */
};


/* ============================================================================
   =============================== Module: LSTCU_8 ============================
   ============================================================================ */

typedef union LSTCU_8_ERR_STAT_union_tag { /* Error Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVP_MB:1;               /* Invalid Pointer MBIST */
    vuint32_t INVP_LB:1;               /* Invalid Pointer LBIST */
    vuint32_t _unused_3:13;
    vuint32_t UFSF:1;                  /* Unrecoverable Fault Status */
    vuint32_t RFSF:1;                  /* Recoverable Fault Status */
    vuint32_t _unused_18:14;
  } B;
} LSTCU_8_ERR_STAT_tag;

typedef union LSTCU_8_ERR_FM_union_tag { /* Error Fault Mapping */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t INVPFMMB:1;              /* Invalid BIST Pointer Fault Mapping During MBIST Scheduling */
    vuint32_t INVPFMLB:1;              /* Invalid BIST Pointer Fault Mapping During LBIST Scheduling */
    vuint32_t _unused_3:29;
  } B;
} LSTCU_8_ERR_FM_tag;

typedef union LSTCU_8_MB_RSTAT0_union_tag { /* MBIST Run Status 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTAT0:1;               /* MBIST Run Result Status 0 */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_8_MB_RSTAT0_tag;

typedef union LSTCU_8_MBFM0_union_tag { /* MBIST Fault Mapping 0 */
  vuint32_t R;
  struct {
    vuint32_t MBSTATFM0:1;             /* MBIST Fault Mapping n */
    vuint32_t _unused_1:15;
    vuint32_t _unused_16:16;
  } B;
} LSTCU_8_MBFM0_tag;

typedef union LSTCU_8_STAG_union_tag { /* Stagger */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MB_DELAY:8;              /* MBIST Delay */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:8;
  } B;
} LSTCU_8_STAG_tag;

typedef union LSTCU_8_PH1_DUR_union_tag { /* Phase 1 Duration */
  vuint32_t R;
  struct {
    vuint32_t PH1DUR:10;               /* Phase 1 Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_8_PH1_DUR_tag;

typedef union LSTCU_8_ISODUR_union_tag { /* Isolation Duration */
  vuint32_t R;
  struct {
    vuint32_t EN_DUR:10;               /* Enable Duration */
    vuint32_t _unused_10:6;
    vuint32_t DIS_DUR:10;              /* Disable Duration */
    vuint32_t _unused_26:6;
  } B;
} LSTCU_8_ISODUR_tag;

typedef union LSTCU_8_SRCENDUR_union_tag { /* Synchronous Reset Clock Enable Duration */
  vuint32_t R;
  struct {
    vuint32_t CLKENDUR:10;             /* Synchronous Reset Clock Enable Duration */
    vuint32_t _unused_10:22;
  } B;
} LSTCU_8_SRCENDUR_tag;

typedef union LSTCU_8_MBPTR_union_tag { /* MBIST Scheduler Pointer */
  vuint32_t R;
  struct {
    vuint32_t MBPTR:8;                 /* MBIST Pointer */
    vuint32_t MBCSM:1;                 /* MBIST Mode */
    vuint32_t _unused_9:22;
    vuint32_t MBEOL:1;                 /* MBIST End of List */
  } B;
} LSTCU_8_MBPTR_tag;

struct LSTCU_8_tag {
  uint8_t LSTCU_8_reserved0[8];
  LSTCU_8_ERR_STAT_tag ERR_STAT;       /* Error Status */
  uint8_t LSTCU_8_reserved1[4];
  LSTCU_8_ERR_FM_tag ERR_FM;           /* Error Fault Mapping */
  uint8_t LSTCU_8_reserved2[76];
  LSTCU_8_MB_RSTAT0_tag MB_RSTAT0;     /* MBIST Run Status 0 */
  uint8_t LSTCU_8_reserved3[284];
  LSTCU_8_MBFM0_tag MBFM0;             /* MBIST Fault Mapping 0 */
  uint8_t LSTCU_8_reserved4[220];
  LSTCU_8_STAG_tag STAG;               /* Stagger */
  uint8_t LSTCU_8_reserved5[12];
  LSTCU_8_PH1_DUR_tag PH1_DUR;         /* Phase 1 Duration */
  LSTCU_8_ISODUR_tag ISODUR;           /* Isolation Duration */
  LSTCU_8_SRCENDUR_tag SRCENDUR;       /* Synchronous Reset Clock Enable Duration */
  uint8_t LSTCU_8_reserved6[132];
  LSTCU_8_MBPTR_tag MBPTR[1];          /* MBIST Scheduler Pointer */
};


/* ============================================================================
   =============================== Module: L_VFCCU ============================
   ============================================================================ */

typedef union L_VFCCU_GDFHID_C0_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID0:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID1:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID2:3;                /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID3:3;                /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID4:3;                /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID5:3;                /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID6:3;                /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID7:3;                /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} L_VFCCU_GDFHID_C0_tag;

typedef union L_VFCCU_GDFHID_C1_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID8:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID9:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID10:3;               /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID11:3;               /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID12:3;               /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID13:3;               /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID14:3;               /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID15:3;               /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} L_VFCCU_GDFHID_C1_tag;

typedef union L_VFCCU_GFLTPO_C0_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS0:1;                   /* Polarity Selection */
    vuint32_t PS1:1;                   /* Polarity Selection */
    vuint32_t PS2:1;                   /* Polarity Selection */
    vuint32_t PS3:1;                   /* Polarity Selection */
    vuint32_t PS4:1;                   /* Polarity Selection */
    vuint32_t PS5:1;                   /* Polarity Selection */
    vuint32_t PS6:1;                   /* Polarity Selection */
    vuint32_t PS7:1;                   /* Polarity Selection */
    vuint32_t PS8:1;                   /* Polarity Selection */
    vuint32_t PS9:1;                   /* Polarity Selection */
    vuint32_t PS10:1;                  /* Polarity Selection */
    vuint32_t PS11:1;                  /* Polarity Selection */
    vuint32_t PS12:1;                  /* Polarity Selection */
    vuint32_t PS13:1;                  /* Polarity Selection */
    vuint32_t PS14:1;                  /* Polarity Selection */
    vuint32_t PS15:1;                  /* Polarity Selection */
    vuint32_t PS16:1;                  /* Polarity Selection */
    vuint32_t PS17:1;                  /* Polarity Selection */
    vuint32_t PS18:1;                  /* Polarity Selection */
    vuint32_t PS19:1;                  /* Polarity Selection */
    vuint32_t PS20:1;                  /* Polarity Selection */
    vuint32_t PS21:1;                  /* Polarity Selection */
    vuint32_t PS22:1;                  /* Polarity Selection */
    vuint32_t PS23:1;                  /* Polarity Selection */
    vuint32_t PS24:1;                  /* Polarity Selection */
    vuint32_t PS25:1;                  /* Polarity Selection */
    vuint32_t PS26:1;                  /* Polarity Selection */
    vuint32_t PS27:1;                  /* Polarity Selection */
    vuint32_t PS28:1;                  /* Polarity Selection */
    vuint32_t PS29:1;                  /* Polarity Selection */
    vuint32_t PS30:1;                  /* Polarity Selection */
    vuint32_t PS31:1;                  /* Polarity Selection */
  } B;
} L_VFCCU_GFLTPO_C0_tag;

typedef union L_VFCCU_GFLTRC_C0_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW0:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW1:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW2:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW3:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW4:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW5:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW6:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW7:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW8:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW9:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW10:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW11:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW12:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW13:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW14:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW15:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW16:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW17:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW18:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW19:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW20:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW21:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW22:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW23:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW24:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW25:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW26:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW27:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW28:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW29:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW30:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW31:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
  } B;
} L_VFCCU_GFLTRC_C0_tag;

typedef union L_VFCCU_GFLTOVDC0_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS0:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS1:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS2:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS3:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS4:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS5:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS6:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS7:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS8:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS9:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS10:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS11:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS12:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS13:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS14:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS15:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS16:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS17:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS18:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS19:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS20:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS21:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS22:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS23:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS24:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS25:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS26:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS27:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS28:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS29:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS30:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS31:1;             /* Fault Overflow Detection Disable */
  } B;
} L_VFCCU_GFLTOVDC0_tag;

typedef union L_VFCCU_GCTRL_union_tag { /* Global Space Control */
  vuint32_t R;
  struct {
    vuint32_t OVF_EN:1;                /* Overflow Enable */
    vuint32_t _unused_1:31;
  } B;
} L_VFCCU_GCTRL_tag;

typedef union L_VFCCU_GINTOVFS_union_tag { /* Global DID FSM Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t FLTSERV:1;               /* Fault Serve */
    vuint32_t OVF_DET:1;               /* Overflow Detect */
    vuint32_t _unused_9:7;
    vuint32_t SERV_DID:4;              /* Domain Being Serviced */
    vuint32_t _unused_20:4;
    vuint32_t OVF_DID:4;               /* Overflow DID */
    vuint32_t _unused_28:4;
  } B;
} L_VFCCU_GINTOVFS_tag;

typedef union L_VFCCU_GDBGCFG_union_tag { /* Global Debug */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t FRZ:1;                   /* Freeze On First Fault */
    vuint32_t _unused_17:15;
  } B;
} L_VFCCU_GDBGCFG_tag;

typedef union L_VFCCU_GDBGSTAT_union_tag { /* Global Debug Status */
  vuint32_t R;
  struct {
    vuint32_t FLTIND:8;                /* Fault Index */
    vuint32_t _unused_8:24;
  } B;
} L_VFCCU_GDBGSTAT_tag;

typedef union L_VFCCU_FHCFG0_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} L_VFCCU_FHCFG0_tag;

typedef union L_VFCCU_FHSRVDS0_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} L_VFCCU_FHSRVDS0_tag;

typedef union L_VFCCU_FHFLTENC00_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} L_VFCCU_FHFLTENC00_tag;

typedef union L_VFCCU_FHFLTS00_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} L_VFCCU_FHFLTS00_tag;

typedef union L_VFCCU_FHFLTRKC00_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} L_VFCCU_FHFLTRKC00_tag;

typedef union L_VFCCU_FHFLTRKC01_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} L_VFCCU_FHFLTRKC01_tag;

typedef union L_VFCCU_FHFLTRKC02_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} L_VFCCU_FHFLTRKC02_tag;

typedef union L_VFCCU_FHFLTRKC03_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} L_VFCCU_FHFLTRKC03_tag;

typedef union L_VFCCU_FHIMRKC0_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} L_VFCCU_FHIMRKC0_00_tag;

typedef union L_VFCCU_FHIMRKC0_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} L_VFCCU_FHIMRKC0_10_tag;

typedef union L_VFCCU_FHIMRKC0_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} L_VFCCU_FHIMRKC0_20_tag;

typedef union L_VFCCU_FHIMRKC0_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} L_VFCCU_FHIMRKC0_30_tag;

typedef union L_VFCCU_FHIMRKC0_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} L_VFCCU_FHIMRKC0_40_tag;

typedef union L_VFCCU_FHIMRKC0_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} L_VFCCU_FHIMRKC0_50_tag;

struct L_VFCCU_tag {
  L_VFCCU_GDFHID_C0_tag GDFHID_C0;     /* Global DID-FHID Map */
  L_VFCCU_GDFHID_C1_tag GDFHID_C1;     /* Global DID-FHID Map */
  uint8_t L_VFCCU_reserved0[24];
  L_VFCCU_GFLTPO_C0_tag GFLTPO_C0;     /* Global Fault Polarity */
  uint8_t L_VFCCU_reserved1[60];
  L_VFCCU_GFLTRC_C0_tag GFLTRC_C0;     /* Global Fault Recovery */
  uint8_t L_VFCCU_reserved2[60];
  L_VFCCU_GFLTOVDC0_tag GFLTOVDC0;     /* Global Fault Overflow Detection */
  uint8_t L_VFCCU_reserved3[140];
  L_VFCCU_GCTRL_tag GCTRL;             /* Global Space Control */
  L_VFCCU_GINTOVFS_tag GINTOVFS;       /* Global DID FSM Status */
  uint8_t L_VFCCU_reserved4[428];
  L_VFCCU_GDBGCFG_tag GDBGCFG;         /* Global Debug */
  L_VFCCU_GDBGSTAT_tag GDBGSTAT;       /* Global Debug Status */
  uint8_t L_VFCCU_reserved5[64788];
  L_VFCCU_FHCFG0_tag FHCFG0;           /* Fault Handler */
  L_VFCCU_FHSRVDS0_tag FHSRVDS0;       /* Fault Handler Status */
  uint8_t L_VFCCU_reserved6[8];
  L_VFCCU_FHFLTENC00_tag FHFLTENC00;   /* Fault Enable */
  uint8_t L_VFCCU_reserved7[60];
  L_VFCCU_FHFLTS00_tag FHFLTS00;       /* Fault Status */
  uint8_t L_VFCCU_reserved8[60];
  L_VFCCU_FHFLTRKC00_tag FHFLTRKC00;   /* Fault Reaction Set Configuration */
  L_VFCCU_FHFLTRKC01_tag FHFLTRKC01;   /* Fault Reaction Set Configuration */
  L_VFCCU_FHFLTRKC02_tag FHFLTRKC02;   /* Fault Reaction Set Configuration */
  L_VFCCU_FHFLTRKC03_tag FHFLTRKC03;   /* Fault Reaction Set Configuration */
  uint8_t L_VFCCU_reserved9[240];
  L_VFCCU_FHIMRKC0_00_tag FHIMRKC0_00; /* Immediate Reaction Configuration */
  uint8_t L_VFCCU_reserved10[12];
  L_VFCCU_FHIMRKC0_10_tag FHIMRKC0_10; /* Immediate Reaction Configuration */
  uint8_t L_VFCCU_reserved11[12];
  L_VFCCU_FHIMRKC0_20_tag FHIMRKC0_20; /* Immediate Reaction Configuration */
  uint8_t L_VFCCU_reserved12[12];
  L_VFCCU_FHIMRKC0_30_tag FHIMRKC0_30; /* Immediate Reaction Configuration */
  uint8_t L_VFCCU_reserved13[12];
  L_VFCCU_FHIMRKC0_40_tag FHIMRKC0_40; /* Immediate Reaction Configuration */
  uint8_t L_VFCCU_reserved14[12];
  L_VFCCU_FHIMRKC0_50_tag FHIMRKC0_50; /* Immediate Reaction Configuration */
};


/* ============================================================================
   =============================== Module: MCM ================================
   ============================================================================ */

typedef union MCM_PLASC_union_tag {    /* Crossbar Switch (AXBS) Slave Configuration */
  vuint16_t R;
  struct {
    vuint16_t ASC0:1;                  /* AXBS Slave Configuration n */
    vuint16_t ASC1:1;                  /* AXBS Slave Configuration n */
    vuint16_t ASC2:1;                  /* AXBS Slave Configuration n */
    vuint16_t ASC3:1;                  /* AXBS Slave Configuration n */
    vuint16_t ASC4:1;                  /* AXBS Slave Configuration n */
    vuint16_t ASC5:1;                  /* AXBS Slave Configuration n */
    vuint16_t ASC6:1;                  /* AXBS Slave Configuration n */
    vuint16_t ASC7:1;                  /* AXBS Slave Configuration n */
    vuint16_t _unused_8:8;             /* Reserved */
  } B;
} MCM_PLASC_tag;

typedef union MCM_PLAMC_union_tag {    /* Crossbar Switch Master Configuration */
  vuint16_t R;
  struct {
    vuint16_t AMC0:1;                  /* AXBS Master Configuration */
    vuint16_t AMC1:1;                  /* AXBS Master Configuration */
    vuint16_t AMC2:1;                  /* AXBS Master Configuration */
    vuint16_t AMC3:1;                  /* AXBS Master Configuration */
    vuint16_t AMC4:1;                  /* AXBS Master Configuration */
    vuint16_t AMC5:1;                  /* AXBS Master Configuration */
    vuint16_t AMC6:1;                  /* AXBS Master Configuration */
    vuint16_t AMC7:1;                  /* AXBS Master Configuration */
    vuint16_t _unused_8:8;             /* Reserved */
  } B;
} MCM_PLAMC_tag;

typedef union MCM_ISCR_union_tag {     /* Interrupt Status and Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t CWBER:1;                 /* Cache Write Buffer Error Status */
    vuint32_t _unused_5:3;
    vuint32_t FIOC:1;                  /* FPU Invalid Operation Interrupt Status */
    vuint32_t FDZC:1;                  /* FPU Divide-by-Zero Interrupt Status */
    vuint32_t FOFC:1;                  /* FPU Overflow Interrupt Status */
    vuint32_t FUFC:1;                  /* FPU Underflow Interrupt status */
    vuint32_t FIXC:1;                  /* FPU Inexact Interrupt Status */
    vuint32_t _unused_13:2;
    vuint32_t FIDC:1;                  /* FPU Input Denormal Interrupt Status */
    vuint32_t _unused_16:4;
    vuint32_t CWBEE:1;                 /* Cache Write Buffer Error Enable */
    vuint32_t _unused_21:3;
    vuint32_t FIOCE:1;                 /* FPU Invalid Operation Interrupt Enable */
    vuint32_t FDZCE:1;                 /* FPU Divide-by-Zero Interrupt Enable */
    vuint32_t FOFCE:1;                 /* FPU Overflow Interrupt Enable */
    vuint32_t FUFCE:1;                 /* FPU Underflow Interrupt Enable */
    vuint32_t FIXCE:1;                 /* FPU Inexact Interrupt Enable */
    vuint32_t _unused_29:2;
    vuint32_t FIDCE:1;                 /* FPU Input Denormal Interrupt Enable */
  } B;
} MCM_ISCR_tag;

typedef union MCM_FADR_union_tag {     /* Write Buffer Fault Address */
  vuint32_t R;
  struct {
    vuint32_t ADDRESS:32;              /* Fault Address */
  } B;
} MCM_FADR_tag;

typedef union MCM_FATR_union_tag {     /* Store Buffer Fault Attributes */
  vuint32_t R;
  struct {
    vuint32_t BEDA:1;                  /* Bus Error Data Access Type */
    vuint32_t BEMD:1;                  /* Bus Error Privilege Level */
    vuint32_t _unused_2:2;
    vuint32_t BESZ:2;                  /* Bus Error Size */
    vuint32_t _unused_6:1;
    vuint32_t BEWT:1;                  /* Bus Error Write */
    vuint32_t BEMN:4;                  /* Bus Error Master Number */
    vuint32_t _unused_12:19;
    vuint32_t BEOVR:1;                 /* Bus Error Overrun */
  } B;
} MCM_FATR_tag;

typedef union MCM_FDR_union_tag {      /* Store Buffer Fault Data */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;                 /* Fault Data */
  } B;
} MCM_FDR_tag;

typedef union MCM_CPCR2_union_tag {    /* Core Platform Control 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t CBCS:4;                  /* Code Bus Cache Size */
    vuint32_t _unused_8:4;
    vuint32_t SBCS:4;                  /* System Bus Cache Size */
    vuint32_t _unused_16:2;
    vuint32_t _unused_18:2;
    vuint32_t _unused_20:2;
    vuint32_t _unused_22:10;
  } B;
} MCM_CPCR2_tag;

typedef union MCM_GPCR0_union_tag {    /* General Purpose Control 0 */
  vuint32_t R;
  struct {
    vuint32_t CORE_TASKID:4;           /* Cortex-M33 Core Task ID */
    vuint32_t _unused_4:4;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:23;            /* Reserved */
  } B;
} MCM_GPCR0_tag;

typedef union MCM_LMDR0_union_tag {    /* Local Memory Descriptor 0 */
  vuint32_t R;
  struct {
    vuint32_t EEWG:1;                  /* Enable ECC Write Generation */
    vuint32_t EERC:1;                  /* Enable ECC Read Check */
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:4;
    vuint32_t _unused_12:1;
    vuint32_t MT:3;                    /* Memory Type */
    vuint32_t RO:1;                    /* Read-Only */
    vuint32_t DPW:3;                   /* LMEM Data Path Width */
    vuint32_t WY:4;                    /* Level 1 Cache Ways */
    vuint32_t LMSZ:4;                  /* LMEM Size */
    vuint32_t LMSZH:1;                 /* LMEM Size Hole */
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t V:1;                     /* Valid */
  } B;
} MCM_LMDR0_tag;

typedef union MCM_LMDR1_union_tag {    /* Local Memory Descriptor 1 */
  vuint32_t R;
  struct {
    vuint32_t EEWG:1;                  /* Enable ECC Write Generation */
    vuint32_t EERC:1;                  /* Enable ECC Read Check */
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:4;
    vuint32_t _unused_12:1;
    vuint32_t MT:3;                    /* Memory Type */
    vuint32_t RO:1;                    /* Read-Only */
    vuint32_t DPW:3;                   /* LMEM Data Path Width */
    vuint32_t WY:4;                    /* Level 1 Cache Ways */
    vuint32_t LMSZ:4;                  /* LMEM Size */
    vuint32_t LMSZH:1;                 /* LMEM Size Hole */
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t V:1;                     /* Valid */
  } B;
} MCM_LMDR1_tag;

struct MCM_tag {
  uint8_t MCM_reserved0[8];
  MCM_PLASC_tag PLASC;                 /* Crossbar Switch (AXBS) Slave Configuration */
  MCM_PLAMC_tag PLAMC;                 /* Crossbar Switch Master Configuration */
  uint8_t MCM_reserved1[4];
  MCM_ISCR_tag ISCR;                   /* Interrupt Status and Control */
  uint8_t MCM_reserved2[12];
  MCM_FADR_tag FADR;                   /* Write Buffer Fault Address */
  MCM_FATR_tag FATR;                   /* Store Buffer Fault Attributes */
  MCM_FDR_tag FDR;                     /* Store Buffer Fault Data */
  uint8_t MCM_reserved3[8];
  MCM_CPCR2_tag CPCR2;                 /* Core Platform Control 2 */
  uint8_t MCM_reserved4[24];
  MCM_GPCR0_tag GPCR0;                 /* General Purpose Control 0 */
  uint8_t MCM_reserved5[940];
  MCM_LMDR0_tag LMDR0;                 /* Local Memory Descriptor 0 */
  MCM_LMDR1_tag LMDR1;                 /* Local Memory Descriptor 1 */
};


/* ============================================================================
   =============================== Module: MC_CGM_0 ===========================
   ============================================================================ */

typedef union MC_CGM_0_PCFS_SDUR_union_tag { /* PCFS Step Duration */
  vuint32_t R;
  struct {
    vuint32_t SDUR:16;                 /* Step duration */
    vuint32_t _unused_16:16;
  } B;
} MC_CGM_0_PCFS_SDUR_tag;

typedef union MC_CGM_0_PCFS_DIVC12_union_tag { /* PCFS Divider Change 12 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} MC_CGM_0_PCFS_DIVC12_tag;

typedef union MC_CGM_0_PCFS_DIVE12_union_tag { /* PCFS Divider End 12 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_0_PCFS_DIVE12_tag;

typedef union MC_CGM_0_PCFS_DIVS12_union_tag { /* PCFS Divider Start 12 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_0_PCFS_DIVS12_tag;

typedef union MC_CGM_0_MUX_0_CSC_union_tag { /* Clock Mux 0 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_0_MUX_0_CSC_tag;

typedef union MC_CGM_0_MUX_0_CSS_union_tag { /* Clock Mux 0 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_0_MUX_0_CSS_tag;

typedef union MC_CGM_0_MUX_1_CSC_union_tag { /* Clock Mux 1 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_0_MUX_1_CSC_tag;

typedef union MC_CGM_0_MUX_1_CSS_union_tag { /* Clock Mux 1 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_0_MUX_1_CSS_tag;

typedef union MC_CGM_0_MUX_1_DC_0_union_tag { /* Clock Mux 1 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_1_DC_0_tag;

typedef union MC_CGM_0_MUX_1_DC_1_union_tag { /* Clock Mux 1 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_1_DC_1_tag;

typedef union MC_CGM_0_MUX_1_DIV_UPD_STAT_union_tag { /* Clock Mux 1 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 1 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_1_DIV_UPD_STAT_tag;

typedef union MC_CGM_0_MUX_2_CSC_union_tag { /* Clock Mux 2 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_2_CSC_tag;

typedef union MC_CGM_0_MUX_2_CSS_union_tag { /* Clock Mux 2 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_2_CSS_tag;

typedef union MC_CGM_0_MUX_2_DC_0_union_tag { /* Clock Mux 2 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_2_DC_0_tag;

typedef union MC_CGM_0_MUX_2_DC_1_union_tag { /* Clock Mux 2 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_2_DC_1_tag;

typedef union MC_CGM_0_MUX_2_DC_2_union_tag { /* Clock Mux 2 Divider 2 Control Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_FMT:2;               /* Division format */
    vuint32_t _unused_2:14;
    vuint32_t DIV:13;                  /* Division value */
    vuint32_t _unused_29:2;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_2_DC_2_tag;

typedef union MC_CGM_0_MUX_2_DC_3_union_tag { /* Clock Mux 2 Divider 3 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_2_DC_3_tag;

typedef union MC_CGM_0_MUX_2_DC_4_union_tag { /* Clock Mux 2 Divider 4 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_2_DC_4_tag;

typedef union MC_CGM_0_MUX_2_DC_5_union_tag { /* Clock Mux 2 Divider 5 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_2_DC_5_tag;

typedef union MC_CGM_0_MUX_2_DIV_UPD_STAT_union_tag { /* Clock Mux 2 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 2 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_2_DIV_UPD_STAT_tag;

typedef union MC_CGM_0_MUX_3_CSC_union_tag { /* Clock Mux 3 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_0_MUX_3_CSC_tag;

typedef union MC_CGM_0_MUX_3_CSS_union_tag { /* Clock Mux 3 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_0_MUX_3_CSS_tag;

typedef union MC_CGM_0_MUX_3_DC_0_union_tag { /* Clock Mux 3 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_3_DC_0_tag;

typedef union MC_CGM_0_MUX_3_DC_1_union_tag { /* Clock Mux 3 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_3_DC_1_tag;

typedef union MC_CGM_0_MUX_3_DC_2_union_tag { /* Clock Mux 3 Divider 2 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_3_DC_2_tag;

typedef union MC_CGM_0_MUX_3_DC_3_union_tag { /* Clock Mux 3 Divider 3 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_3_DC_3_tag;

typedef union MC_CGM_0_MUX_3_DC_4_union_tag { /* Clock Mux 3 Divider 4 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_3_DC_4_tag;

typedef union MC_CGM_0_MUX_3_DC_5_union_tag { /* Clock Mux 3 Divider 5 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_3_DC_5_tag;

typedef union MC_CGM_0_MUX_3_DC_6_union_tag { /* Clock Mux 3 Divider 6 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_3_DC_6_tag;

typedef union MC_CGM_0_MUX_3_DC_7_union_tag { /* Clock Mux 3 Divider 7 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_3_DC_7_tag;

typedef union MC_CGM_0_MUX_3_DIV_UPD_STAT_union_tag { /* Clock Mux 3 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 3 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_3_DIV_UPD_STAT_tag;

typedef union MC_CGM_0_MUX_4_CSC_union_tag { /* Clock Mux 4 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_4_CSC_tag;

typedef union MC_CGM_0_MUX_4_CSS_union_tag { /* Clock Mux 4 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_4_CSS_tag;

typedef union MC_CGM_0_MUX_4_DC_0_union_tag { /* Clock Mux 4 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_4_DC_0_tag;

typedef union MC_CGM_0_MUX_4_DIV_UPD_STAT_union_tag { /* Clock Mux 4 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 4 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_4_DIV_UPD_STAT_tag;

typedef union MC_CGM_0_MUX_5_CSC_union_tag { /* Clock Mux 5 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_5_CSC_tag;

typedef union MC_CGM_0_MUX_5_CSS_union_tag { /* Clock Mux 5 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_5_CSS_tag;

typedef union MC_CGM_0_MUX_6_CSC_union_tag { /* Clock Mux 6 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_6_CSC_tag;

typedef union MC_CGM_0_MUX_6_CSS_union_tag { /* Clock Mux 6 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_6_CSS_tag;

typedef union MC_CGM_0_MUX_6_DC_0_union_tag { /* Clock Mux 6 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_6_DC_0_tag;

typedef union MC_CGM_0_MUX_6_DIV_UPD_STAT_union_tag { /* Clock Mux 6 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 6 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_6_DIV_UPD_STAT_tag;

typedef union MC_CGM_0_MUX_7_CSC_union_tag { /* Clock Mux 7 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_7_CSC_tag;

typedef union MC_CGM_0_MUX_7_CSS_union_tag { /* Clock Mux 7 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_7_CSS_tag;

typedef union MC_CGM_0_MUX_7_DC_0_union_tag { /* Clock Mux 7 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_7_DC_0_tag;

typedef union MC_CGM_0_MUX_7_DC_1_union_tag { /* Clock Mux 7 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_7_DC_1_tag;

typedef union MC_CGM_0_MUX_7_DIV_TRIG_CTRL_union_tag { /* Clock Mux 7 Divider Trigger Control Register */
  vuint32_t R;
  struct {
    vuint32_t TCTL:1;                  /* Trigger control */
    vuint32_t _unused_1:30;
    vuint32_t HHEN:1;                  /* Halt handshake enable */
  } B;
} MC_CGM_0_MUX_7_DIV_TRIG_CTRL_tag;

typedef union MC_CGM_0_MUX_7_DIV_TRIG_union_tag { /* Clock Mux 7 Divider Trigger Register */
  vuint32_t R;
  struct {
    vuint32_t TRIGGER:32;              /* Trigger for divider update */
  } B;
} MC_CGM_0_MUX_7_DIV_TRIG_tag;

typedef union MC_CGM_0_MUX_7_DIV_UPD_STAT_union_tag { /* Clock Mux 7 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 7 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_7_DIV_UPD_STAT_tag;

typedef union MC_CGM_0_MUX_8_CSC_union_tag { /* Clock Mux 8 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_8_CSC_tag;

typedef union MC_CGM_0_MUX_8_CSS_union_tag { /* Clock Mux 8 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_8_CSS_tag;

typedef union MC_CGM_0_MUX_8_DC_0_union_tag { /* Clock Mux 8 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_8_DC_0_tag;

typedef union MC_CGM_0_MUX_8_DIV_UPD_STAT_union_tag { /* Clock Mux 8 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 8 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_8_DIV_UPD_STAT_tag;

typedef union MC_CGM_0_MUX_9_CSC_union_tag { /* Clock Mux 9 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_9_CSC_tag;

typedef union MC_CGM_0_MUX_9_CSS_union_tag { /* Clock Mux 9 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_0_MUX_9_CSS_tag;

typedef union MC_CGM_0_MUX_9_DC_0_union_tag { /* Clock Mux 9 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_9_DC_0_tag;

typedef union MC_CGM_0_MUX_9_DIV_UPD_STAT_union_tag { /* Clock Mux 9 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 9 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_9_DIV_UPD_STAT_tag;

typedef union MC_CGM_0_MUX_10_CSC_union_tag { /* Clock Mux 10 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CG:1;                    /* Clock gate */
    vuint32_t FCG:1;                   /* Force clock gate */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_0_MUX_10_CSC_tag;

typedef union MC_CGM_0_MUX_10_CSS_union_tag { /* Clock Mux 10 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t GRIP:1;                  /* Gating request is in progress. */
    vuint32_t CS:1;                    /* Clock status */
    vuint32_t _unused_18:2;
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_0_MUX_10_CSS_tag;

typedef union MC_CGM_0_MUX_10_DC_0_union_tag { /* Clock Mux 10 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_0_MUX_10_DC_0_tag;

typedef union MC_CGM_0_MUX_10_DIV_UPD_STAT_union_tag { /* Clock Mux 10 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 10 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_0_MUX_10_DIV_UPD_STAT_tag;

struct MC_CGM_0_tag {
  MC_CGM_0_PCFS_SDUR_tag PCFS_SDUR;    /* PCFS Step Duration */
  uint8_t MC_CGM_0_reserved0[132];
  MC_CGM_0_PCFS_DIVC12_tag PCFS_DIVC12; /* PCFS Divider Change 12 Register */
  MC_CGM_0_PCFS_DIVE12_tag PCFS_DIVE12; /* PCFS Divider End 12 Register */
  MC_CGM_0_PCFS_DIVS12_tag PCFS_DIVS12; /* PCFS Divider Start 12 Register */
  uint8_t MC_CGM_0_reserved1[620];
  MC_CGM_0_MUX_0_CSC_tag MUX_0_CSC;    /* Clock Mux 0 Select Control Register */
  MC_CGM_0_MUX_0_CSS_tag MUX_0_CSS;    /* Clock Mux 0 Select Status Register */
  uint8_t MC_CGM_0_reserved2[56];
  MC_CGM_0_MUX_1_CSC_tag MUX_1_CSC;    /* Clock Mux 1 Select Control Register */
  MC_CGM_0_MUX_1_CSS_tag MUX_1_CSS;    /* Clock Mux 1 Select Status Register */
  MC_CGM_0_MUX_1_DC_0_tag MUX_1_DC_0;  /* Clock Mux 1 Divider 0 Control Register */
  MC_CGM_0_MUX_1_DC_1_tag MUX_1_DC_1;  /* Clock Mux 1 Divider 1 Control Register */
  uint8_t MC_CGM_0_reserved3[44];
  MC_CGM_0_MUX_1_DIV_UPD_STAT_tag MUX_1_DIV_UPD_STAT; /* Clock Mux 1 Divider Update Status Register */
  MC_CGM_0_MUX_2_CSC_tag MUX_2_CSC;    /* Clock Mux 2 Select Control Register */
  MC_CGM_0_MUX_2_CSS_tag MUX_2_CSS;    /* Clock Mux 2 Select Status Register */
  MC_CGM_0_MUX_2_DC_0_tag MUX_2_DC_0;  /* Clock Mux 2 Divider 0 Control Register */
  MC_CGM_0_MUX_2_DC_1_tag MUX_2_DC_1;  /* Clock Mux 2 Divider 1 Control Register */
  MC_CGM_0_MUX_2_DC_2_tag MUX_2_DC_2;  /* Clock Mux 2 Divider 2 Control Register */
  MC_CGM_0_MUX_2_DC_3_tag MUX_2_DC_3;  /* Clock Mux 2 Divider 3 Control Register */
  MC_CGM_0_MUX_2_DC_4_tag MUX_2_DC_4;  /* Clock Mux 2 Divider 4 Control Register */
  MC_CGM_0_MUX_2_DC_5_tag MUX_2_DC_5;  /* Clock Mux 2 Divider 5 Control Register */
  uint8_t MC_CGM_0_reserved4[28];
  MC_CGM_0_MUX_2_DIV_UPD_STAT_tag MUX_2_DIV_UPD_STAT; /* Clock Mux 2 Divider Update Status Register */
  MC_CGM_0_MUX_3_CSC_tag MUX_3_CSC;    /* Clock Mux 3 Select Control Register */
  MC_CGM_0_MUX_3_CSS_tag MUX_3_CSS;    /* Clock Mux 3 Select Status Register */
  MC_CGM_0_MUX_3_DC_0_tag MUX_3_DC_0;  /* Clock Mux 3 Divider 0 Control Register */
  MC_CGM_0_MUX_3_DC_1_tag MUX_3_DC_1;  /* Clock Mux 3 Divider 1 Control Register */
  MC_CGM_0_MUX_3_DC_2_tag MUX_3_DC_2;  /* Clock Mux 3 Divider 2 Control Register */
  MC_CGM_0_MUX_3_DC_3_tag MUX_3_DC_3;  /* Clock Mux 3 Divider 3 Control Register */
  MC_CGM_0_MUX_3_DC_4_tag MUX_3_DC_4;  /* Clock Mux 3 Divider 4 Control Register */
  MC_CGM_0_MUX_3_DC_5_tag MUX_3_DC_5;  /* Clock Mux 3 Divider 5 Control Register */
  MC_CGM_0_MUX_3_DC_6_tag MUX_3_DC_6;  /* Clock Mux 3 Divider 6 Control Register */
  MC_CGM_0_MUX_3_DC_7_tag MUX_3_DC_7;  /* Clock Mux 3 Divider 7 Control Register */
  uint8_t MC_CGM_0_reserved5[20];
  MC_CGM_0_MUX_3_DIV_UPD_STAT_tag MUX_3_DIV_UPD_STAT; /* Clock Mux 3 Divider Update Status Register */
  MC_CGM_0_MUX_4_CSC_tag MUX_4_CSC;    /* Clock Mux 4 Select Control Register */
  MC_CGM_0_MUX_4_CSS_tag MUX_4_CSS;    /* Clock Mux 4 Select Status Register */
  MC_CGM_0_MUX_4_DC_0_tag MUX_4_DC_0;  /* Clock Mux 4 Divider 0 Control Register */
  uint8_t MC_CGM_0_reserved6[48];
  MC_CGM_0_MUX_4_DIV_UPD_STAT_tag MUX_4_DIV_UPD_STAT; /* Clock Mux 4 Divider Update Status Register */
  MC_CGM_0_MUX_5_CSC_tag MUX_5_CSC;    /* Clock Mux 5 Select Control Register */
  MC_CGM_0_MUX_5_CSS_tag MUX_5_CSS;    /* Clock Mux 5 Select Status Register */
  uint8_t MC_CGM_0_reserved7[56];
  MC_CGM_0_MUX_6_CSC_tag MUX_6_CSC;    /* Clock Mux 6 Select Control Register */
  MC_CGM_0_MUX_6_CSS_tag MUX_6_CSS;    /* Clock Mux 6 Select Status Register */
  MC_CGM_0_MUX_6_DC_0_tag MUX_6_DC_0;  /* Clock Mux 6 Divider 0 Control Register */
  uint8_t MC_CGM_0_reserved8[48];
  MC_CGM_0_MUX_6_DIV_UPD_STAT_tag MUX_6_DIV_UPD_STAT; /* Clock Mux 6 Divider Update Status Register */
  MC_CGM_0_MUX_7_CSC_tag MUX_7_CSC;    /* Clock Mux 7 Select Control Register */
  MC_CGM_0_MUX_7_CSS_tag MUX_7_CSS;    /* Clock Mux 7 Select Status Register */
  MC_CGM_0_MUX_7_DC_0_tag MUX_7_DC_0;  /* Clock Mux 7 Divider 0 Control Register */
  MC_CGM_0_MUX_7_DC_1_tag MUX_7_DC_1;  /* Clock Mux 7 Divider 1 Control Register */
  uint8_t MC_CGM_0_reserved9[36];
  MC_CGM_0_MUX_7_DIV_TRIG_CTRL_tag MUX_7_DIV_TRIG_CTRL; /* Clock Mux 7 Divider Trigger Control Register */
  MC_CGM_0_MUX_7_DIV_TRIG_tag MUX_7_DIV_TRIG; /* Clock Mux 7 Divider Trigger Register */
  MC_CGM_0_MUX_7_DIV_UPD_STAT_tag MUX_7_DIV_UPD_STAT; /* Clock Mux 7 Divider Update Status Register */
  MC_CGM_0_MUX_8_CSC_tag MUX_8_CSC;    /* Clock Mux 8 Select Control Register */
  MC_CGM_0_MUX_8_CSS_tag MUX_8_CSS;    /* Clock Mux 8 Select Status Register */
  MC_CGM_0_MUX_8_DC_0_tag MUX_8_DC_0;  /* Clock Mux 8 Divider 0 Control Register */
  uint8_t MC_CGM_0_reserved10[48];
  MC_CGM_0_MUX_8_DIV_UPD_STAT_tag MUX_8_DIV_UPD_STAT; /* Clock Mux 8 Divider Update Status Register */
  MC_CGM_0_MUX_9_CSC_tag MUX_9_CSC;    /* Clock Mux 9 Select Control Register */
  MC_CGM_0_MUX_9_CSS_tag MUX_9_CSS;    /* Clock Mux 9 Select Status Register */
  MC_CGM_0_MUX_9_DC_0_tag MUX_9_DC_0;  /* Clock Mux 9 Divider 0 Control Register */
  uint8_t MC_CGM_0_reserved11[48];
  MC_CGM_0_MUX_9_DIV_UPD_STAT_tag MUX_9_DIV_UPD_STAT; /* Clock Mux 9 Divider Update Status Register */
  MC_CGM_0_MUX_10_CSC_tag MUX_10_CSC;  /* Clock Mux 10 Select Control Register */
  MC_CGM_0_MUX_10_CSS_tag MUX_10_CSS;  /* Clock Mux 10 Select Status Register */
  MC_CGM_0_MUX_10_DC_0_tag MUX_10_DC_0; /* Clock Mux 10 Divider 0 Control Register */
  uint8_t MC_CGM_0_reserved12[48];
  MC_CGM_0_MUX_10_DIV_UPD_STAT_tag MUX_10_DIV_UPD_STAT; /* Clock Mux 10 Divider Update Status Register */
};


/* ============================================================================
   =============================== Module: MC_CGM_1 ===========================
   ============================================================================ */

typedef union MC_CGM_1_PCFS_SDUR_union_tag { /* PCFS Step Duration */
  vuint32_t R;
  struct {
    vuint32_t SDUR:16;                 /* Step duration */
    vuint32_t _unused_16:16;
  } B;
} MC_CGM_1_PCFS_SDUR_tag;

typedef union MC_CGM_1_PCFS_DIVC12_union_tag { /* PCFS Divider Change 12 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} MC_CGM_1_PCFS_DIVC12_tag;

typedef union MC_CGM_1_PCFS_DIVE12_union_tag { /* PCFS Divider End 12 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_1_PCFS_DIVE12_tag;

typedef union MC_CGM_1_PCFS_DIVS12_union_tag { /* PCFS Divider Start 12 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_1_PCFS_DIVS12_tag;

typedef union MC_CGM_1_MUX_0_CSC_union_tag { /* Clock Mux 0 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_1_MUX_0_CSC_tag;

typedef union MC_CGM_1_MUX_0_CSS_union_tag { /* Clock Mux 0 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_1_MUX_0_CSS_tag;

typedef union MC_CGM_1_MUX_1_CSC_union_tag { /* Clock Mux 1 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_1_MUX_1_CSC_tag;

typedef union MC_CGM_1_MUX_1_CSS_union_tag { /* Clock Mux 1 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_1_MUX_1_CSS_tag;

typedef union MC_CGM_1_MUX_1_DC_0_union_tag { /* Clock Mux 1 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_1_DC_0_tag;

typedef union MC_CGM_1_MUX_1_DIV_UPD_STAT_union_tag { /* Clock Mux 1 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 1 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_1_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_2_CSC_union_tag { /* Clock Mux 2 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_1_MUX_2_CSC_tag;

typedef union MC_CGM_1_MUX_2_CSS_union_tag { /* Clock Mux 2 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_1_MUX_2_CSS_tag;

typedef union MC_CGM_1_MUX_3_CSC_union_tag { /* Clock Mux 3 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_1_MUX_3_CSC_tag;

typedef union MC_CGM_1_MUX_3_CSS_union_tag { /* Clock Mux 3 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_1_MUX_3_CSS_tag;

typedef union MC_CGM_1_MUX_4_CSC_union_tag { /* Clock Mux 4 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_1_MUX_4_CSC_tag;

typedef union MC_CGM_1_MUX_4_CSS_union_tag { /* Clock Mux 4 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_1_MUX_4_CSS_tag;

typedef union MC_CGM_1_MUX_4_DC_0_union_tag { /* Clock Mux 4 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_4_DC_0_tag;

typedef union MC_CGM_1_MUX_4_DIV_UPD_STAT_union_tag { /* Clock Mux 4 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 4 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_4_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_5_CSC_union_tag { /* Clock Mux 5 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_5_CSC_tag;

typedef union MC_CGM_1_MUX_5_CSS_union_tag { /* Clock Mux 5 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_5_CSS_tag;

typedef union MC_CGM_1_MUX_5_DC_0_union_tag { /* Clock Mux 5 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_5_DC_0_tag;

typedef union MC_CGM_1_MUX_5_DIV_UPD_STAT_union_tag { /* Clock Mux 5 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 5 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_5_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_6_CSC_union_tag { /* Clock Mux 6 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_6_CSC_tag;

typedef union MC_CGM_1_MUX_6_CSS_union_tag { /* Clock Mux 6 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_6_CSS_tag;

typedef union MC_CGM_1_MUX_6_DC_0_union_tag { /* Clock Mux 6 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_6_DC_0_tag;

typedef union MC_CGM_1_MUX_6_DC_1_union_tag { /* Clock Mux 6 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t _unused_2:14;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_6_DC_1_tag;

typedef union MC_CGM_1_MUX_6_DIV_UPD_STAT_union_tag { /* Clock Mux 6 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 6 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_6_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_7_CSC_union_tag { /* Clock Mux 7 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_7_CSC_tag;

typedef union MC_CGM_1_MUX_7_CSS_union_tag { /* Clock Mux 7 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_7_CSS_tag;

typedef union MC_CGM_1_MUX_7_DC_0_union_tag { /* Clock Mux 7 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_7_DC_0_tag;

typedef union MC_CGM_1_MUX_7_DC_1_union_tag { /* Clock Mux 7 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_7_DC_1_tag;

typedef union MC_CGM_1_MUX_7_DC_2_union_tag { /* Clock Mux 7 Divider 2 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_7_DC_2_tag;

typedef union MC_CGM_1_MUX_7_DIV_UPD_STAT_union_tag { /* Clock Mux 7 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 7 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_7_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_8_CSC_union_tag { /* Clock Mux 8 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_8_CSC_tag;

typedef union MC_CGM_1_MUX_8_CSS_union_tag { /* Clock Mux 8 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_8_CSS_tag;

typedef union MC_CGM_1_MUX_8_DC_0_union_tag { /* Clock Mux 8 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_8_DC_0_tag;

typedef union MC_CGM_1_MUX_8_DC_1_union_tag { /* Clock Mux 8 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t _unused_2:14;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_8_DC_1_tag;

typedef union MC_CGM_1_MUX_8_DIV_UPD_STAT_union_tag { /* Clock Mux 8 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 8 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_8_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_9_CSC_union_tag { /* Clock Mux 9 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_9_CSC_tag;

typedef union MC_CGM_1_MUX_9_CSS_union_tag { /* Clock Mux 9 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_9_CSS_tag;

typedef union MC_CGM_1_MUX_9_DC_0_union_tag { /* Clock Mux 9 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_9_DC_0_tag;

typedef union MC_CGM_1_MUX_9_DC_1_union_tag { /* Clock Mux 9 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_9_DC_1_tag;

typedef union MC_CGM_1_MUX_9_DC_2_union_tag { /* Clock Mux 9 Divider 2 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_9_DC_2_tag;

typedef union MC_CGM_1_MUX_9_DIV_UPD_STAT_union_tag { /* Clock Mux 9 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 9 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_9_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_10_CSC_union_tag { /* Clock Mux 10 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CG:1;                    /* Clock gate */
    vuint32_t FCG:1;                   /* Force clock gate */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_10_CSC_tag;

typedef union MC_CGM_1_MUX_10_CSS_union_tag { /* Clock Mux 10 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t GRIP:1;                  /* Gating request is in progress. */
    vuint32_t CS:1;                    /* Clock status */
    vuint32_t _unused_18:2;
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_10_CSS_tag;

typedef union MC_CGM_1_MUX_10_DC_0_union_tag { /* Clock Mux 10 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_10_DC_0_tag;

typedef union MC_CGM_1_MUX_10_DIV_UPD_STAT_union_tag { /* Clock Mux 10 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 10 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_10_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_11_CSC_union_tag { /* Clock Mux 11 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_11_CSC_tag;

typedef union MC_CGM_1_MUX_11_CSS_union_tag { /* Clock Mux 11 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_11_CSS_tag;

typedef union MC_CGM_1_MUX_11_DC_0_union_tag { /* Clock Mux 11 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_11_DC_0_tag;

typedef union MC_CGM_1_MUX_11_DIV_UPD_STAT_union_tag { /* Clock Mux 11 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 11 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_11_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_12_CSC_union_tag { /* Clock Mux 12 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_12_CSC_tag;

typedef union MC_CGM_1_MUX_12_CSS_union_tag { /* Clock Mux 12 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_12_CSS_tag;

typedef union MC_CGM_1_MUX_12_DC_0_union_tag { /* Clock Mux 12 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_12_DC_0_tag;

typedef union MC_CGM_1_MUX_12_DIV_UPD_STAT_union_tag { /* Clock Mux 12 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 12 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_12_DIV_UPD_STAT_tag;

typedef union MC_CGM_1_MUX_13_CSC_union_tag { /* Clock Mux 13 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_13_CSC_tag;

typedef union MC_CGM_1_MUX_13_CSS_union_tag { /* Clock Mux 13 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_13_CSS_tag;

typedef union MC_CGM_1_MUX_14_CSC_union_tag { /* Clock Mux 14 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_14_CSC_tag;

typedef union MC_CGM_1_MUX_14_CSS_union_tag { /* Clock Mux 14 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_1_MUX_14_CSS_tag;

typedef union MC_CGM_1_MUX_14_DC_0_union_tag { /* Clock Mux 14 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_1_MUX_14_DC_0_tag;

typedef union MC_CGM_1_MUX_14_DIV_UPD_STAT_union_tag { /* Clock Mux 14 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 14 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_1_MUX_14_DIV_UPD_STAT_tag;

struct MC_CGM_1_tag {
  MC_CGM_1_PCFS_SDUR_tag PCFS_SDUR;    /* PCFS Step Duration */
  uint8_t MC_CGM_1_reserved0[132];
  MC_CGM_1_PCFS_DIVC12_tag PCFS_DIVC12; /* PCFS Divider Change 12 Register */
  MC_CGM_1_PCFS_DIVE12_tag PCFS_DIVE12; /* PCFS Divider End 12 Register */
  MC_CGM_1_PCFS_DIVS12_tag PCFS_DIVS12; /* PCFS Divider Start 12 Register */
  uint8_t MC_CGM_1_reserved1[620];
  MC_CGM_1_MUX_0_CSC_tag MUX_0_CSC;    /* Clock Mux 0 Select Control Register */
  MC_CGM_1_MUX_0_CSS_tag MUX_0_CSS;    /* Clock Mux 0 Select Status Register */
  uint8_t MC_CGM_1_reserved2[56];
  MC_CGM_1_MUX_1_CSC_tag MUX_1_CSC;    /* Clock Mux 1 Select Control Register */
  MC_CGM_1_MUX_1_CSS_tag MUX_1_CSS;    /* Clock Mux 1 Select Status Register */
  MC_CGM_1_MUX_1_DC_0_tag MUX_1_DC_0;  /* Clock Mux 1 Divider 0 Control Register */
  uint8_t MC_CGM_1_reserved3[48];
  MC_CGM_1_MUX_1_DIV_UPD_STAT_tag MUX_1_DIV_UPD_STAT; /* Clock Mux 1 Divider Update Status Register */
  MC_CGM_1_MUX_2_CSC_tag MUX_2_CSC;    /* Clock Mux 2 Select Control Register */
  MC_CGM_1_MUX_2_CSS_tag MUX_2_CSS;    /* Clock Mux 2 Select Status Register */
  uint8_t MC_CGM_1_reserved4[56];
  MC_CGM_1_MUX_3_CSC_tag MUX_3_CSC;    /* Clock Mux 3 Select Control Register */
  MC_CGM_1_MUX_3_CSS_tag MUX_3_CSS;    /* Clock Mux 3 Select Status Register */
  uint8_t MC_CGM_1_reserved5[56];
  MC_CGM_1_MUX_4_CSC_tag MUX_4_CSC;    /* Clock Mux 4 Select Control Register */
  MC_CGM_1_MUX_4_CSS_tag MUX_4_CSS;    /* Clock Mux 4 Select Status Register */
  MC_CGM_1_MUX_4_DC_0_tag MUX_4_DC_0;  /* Clock Mux 4 Divider 0 Control Register */
  uint8_t MC_CGM_1_reserved6[48];
  MC_CGM_1_MUX_4_DIV_UPD_STAT_tag MUX_4_DIV_UPD_STAT; /* Clock Mux 4 Divider Update Status Register */
  MC_CGM_1_MUX_5_CSC_tag MUX_5_CSC;    /* Clock Mux 5 Select Control Register */
  MC_CGM_1_MUX_5_CSS_tag MUX_5_CSS;    /* Clock Mux 5 Select Status Register */
  MC_CGM_1_MUX_5_DC_0_tag MUX_5_DC_0;  /* Clock Mux 5 Divider 0 Control Register */
  uint8_t MC_CGM_1_reserved7[48];
  MC_CGM_1_MUX_5_DIV_UPD_STAT_tag MUX_5_DIV_UPD_STAT; /* Clock Mux 5 Divider Update Status Register */
  MC_CGM_1_MUX_6_CSC_tag MUX_6_CSC;    /* Clock Mux 6 Select Control Register */
  MC_CGM_1_MUX_6_CSS_tag MUX_6_CSS;    /* Clock Mux 6 Select Status Register */
  MC_CGM_1_MUX_6_DC_0_tag MUX_6_DC_0;  /* Clock Mux 6 Divider 0 Control Register */
  MC_CGM_1_MUX_6_DC_1_tag MUX_6_DC_1;  /* Clock Mux 6 Divider 1 Control Register */
  uint8_t MC_CGM_1_reserved8[44];
  MC_CGM_1_MUX_6_DIV_UPD_STAT_tag MUX_6_DIV_UPD_STAT; /* Clock Mux 6 Divider Update Status Register */
  MC_CGM_1_MUX_7_CSC_tag MUX_7_CSC;    /* Clock Mux 7 Select Control Register */
  MC_CGM_1_MUX_7_CSS_tag MUX_7_CSS;    /* Clock Mux 7 Select Status Register */
  MC_CGM_1_MUX_7_DC_0_tag MUX_7_DC_0;  /* Clock Mux 7 Divider 0 Control Register */
  MC_CGM_1_MUX_7_DC_1_tag MUX_7_DC_1;  /* Clock Mux 7 Divider 1 Control Register */
  MC_CGM_1_MUX_7_DC_2_tag MUX_7_DC_2;  /* Clock Mux 7 Divider 2 Control Register */
  uint8_t MC_CGM_1_reserved9[40];
  MC_CGM_1_MUX_7_DIV_UPD_STAT_tag MUX_7_DIV_UPD_STAT; /* Clock Mux 7 Divider Update Status Register */
  MC_CGM_1_MUX_8_CSC_tag MUX_8_CSC;    /* Clock Mux 8 Select Control Register */
  MC_CGM_1_MUX_8_CSS_tag MUX_8_CSS;    /* Clock Mux 8 Select Status Register */
  MC_CGM_1_MUX_8_DC_0_tag MUX_8_DC_0;  /* Clock Mux 8 Divider 0 Control Register */
  MC_CGM_1_MUX_8_DC_1_tag MUX_8_DC_1;  /* Clock Mux 8 Divider 1 Control Register */
  uint8_t MC_CGM_1_reserved10[44];
  MC_CGM_1_MUX_8_DIV_UPD_STAT_tag MUX_8_DIV_UPD_STAT; /* Clock Mux 8 Divider Update Status Register */
  MC_CGM_1_MUX_9_CSC_tag MUX_9_CSC;    /* Clock Mux 9 Select Control Register */
  MC_CGM_1_MUX_9_CSS_tag MUX_9_CSS;    /* Clock Mux 9 Select Status Register */
  MC_CGM_1_MUX_9_DC_0_tag MUX_9_DC_0;  /* Clock Mux 9 Divider 0 Control Register */
  MC_CGM_1_MUX_9_DC_1_tag MUX_9_DC_1;  /* Clock Mux 9 Divider 1 Control Register */
  MC_CGM_1_MUX_9_DC_2_tag MUX_9_DC_2;  /* Clock Mux 9 Divider 2 Control Register */
  uint8_t MC_CGM_1_reserved11[40];
  MC_CGM_1_MUX_9_DIV_UPD_STAT_tag MUX_9_DIV_UPD_STAT; /* Clock Mux 9 Divider Update Status Register */
  MC_CGM_1_MUX_10_CSC_tag MUX_10_CSC;  /* Clock Mux 10 Select Control Register */
  MC_CGM_1_MUX_10_CSS_tag MUX_10_CSS;  /* Clock Mux 10 Select Status Register */
  MC_CGM_1_MUX_10_DC_0_tag MUX_10_DC_0; /* Clock Mux 10 Divider 0 Control Register */
  uint8_t MC_CGM_1_reserved12[48];
  MC_CGM_1_MUX_10_DIV_UPD_STAT_tag MUX_10_DIV_UPD_STAT; /* Clock Mux 10 Divider Update Status Register */
  MC_CGM_1_MUX_11_CSC_tag MUX_11_CSC;  /* Clock Mux 11 Select Control Register */
  MC_CGM_1_MUX_11_CSS_tag MUX_11_CSS;  /* Clock Mux 11 Select Status Register */
  MC_CGM_1_MUX_11_DC_0_tag MUX_11_DC_0; /* Clock Mux 11 Divider 0 Control Register */
  uint8_t MC_CGM_1_reserved13[48];
  MC_CGM_1_MUX_11_DIV_UPD_STAT_tag MUX_11_DIV_UPD_STAT; /* Clock Mux 11 Divider Update Status Register */
  MC_CGM_1_MUX_12_CSC_tag MUX_12_CSC;  /* Clock Mux 12 Select Control Register */
  MC_CGM_1_MUX_12_CSS_tag MUX_12_CSS;  /* Clock Mux 12 Select Status Register */
  MC_CGM_1_MUX_12_DC_0_tag MUX_12_DC_0; /* Clock Mux 12 Divider 0 Control Register */
  uint8_t MC_CGM_1_reserved14[48];
  MC_CGM_1_MUX_12_DIV_UPD_STAT_tag MUX_12_DIV_UPD_STAT; /* Clock Mux 12 Divider Update Status Register */
  MC_CGM_1_MUX_13_CSC_tag MUX_13_CSC;  /* Clock Mux 13 Select Control Register */
  MC_CGM_1_MUX_13_CSS_tag MUX_13_CSS;  /* Clock Mux 13 Select Status Register */
  uint8_t MC_CGM_1_reserved15[56];
  MC_CGM_1_MUX_14_CSC_tag MUX_14_CSC;  /* Clock Mux 14 Select Control Register */
  MC_CGM_1_MUX_14_CSS_tag MUX_14_CSS;  /* Clock Mux 14 Select Status Register */
  MC_CGM_1_MUX_14_DC_0_tag MUX_14_DC_0; /* Clock Mux 14 Divider 0 Control Register */
  uint8_t MC_CGM_1_reserved16[48];
  MC_CGM_1_MUX_14_DIV_UPD_STAT_tag MUX_14_DIV_UPD_STAT; /* Clock Mux 14 Divider Update Status Register */
};


/* ============================================================================
   =============================== Module: MC_CGM_2 ===========================
   ============================================================================ */

typedef union MC_CGM_2_PCFS_SDUR_union_tag { /* PCFS Step Duration */
  vuint32_t R;
  struct {
    vuint32_t SDUR:16;                 /* Step duration */
    vuint32_t _unused_16:16;
  } B;
} MC_CGM_2_PCFS_SDUR_tag;

typedef union MC_CGM_2_PCFS_DIVC15_union_tag { /* PCFS Divider Change 15 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} MC_CGM_2_PCFS_DIVC15_tag;

typedef union MC_CGM_2_PCFS_DIVE15_union_tag { /* PCFS Divider End 15 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_2_PCFS_DIVE15_tag;

typedef union MC_CGM_2_PCFS_DIVS15_union_tag { /* PCFS Divider Start 15 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_2_PCFS_DIVS15_tag;

typedef union MC_CGM_2_MUX_0_CSC_union_tag { /* Clock Mux 0 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_2_MUX_0_CSC_tag;

typedef union MC_CGM_2_MUX_0_CSS_union_tag { /* Clock Mux 0 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_2_MUX_0_CSS_tag;

typedef union MC_CGM_2_MUX_1_CSC_union_tag { /* Clock Mux 1 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_2_MUX_1_CSC_tag;

typedef union MC_CGM_2_MUX_1_CSS_union_tag { /* Clock Mux 1 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_2_MUX_1_CSS_tag;

typedef union MC_CGM_2_MUX_1_DC_0_union_tag { /* Clock Mux 1 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_2_MUX_1_DC_0_tag;

typedef union MC_CGM_2_MUX_1_DIV_UPD_STAT_union_tag { /* Clock Mux 1 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 1 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_2_MUX_1_DIV_UPD_STAT_tag;

typedef union MC_CGM_2_MUX_2_CSC_union_tag { /* Clock Mux 2 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_2_MUX_2_CSC_tag;

typedef union MC_CGM_2_MUX_2_CSS_union_tag { /* Clock Mux 2 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_2_MUX_2_CSS_tag;

typedef union MC_CGM_2_MUX_3_CSC_union_tag { /* Clock Mux 3 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_2_MUX_3_CSC_tag;

typedef union MC_CGM_2_MUX_3_CSS_union_tag { /* Clock Mux 3 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_2_MUX_3_CSS_tag;

struct MC_CGM_2_tag {
  MC_CGM_2_PCFS_SDUR_tag PCFS_SDUR;    /* PCFS Step Duration */
  uint8_t MC_CGM_2_reserved0[168];
  MC_CGM_2_PCFS_DIVC15_tag PCFS_DIVC15; /* PCFS Divider Change 15 Register */
  MC_CGM_2_PCFS_DIVE15_tag PCFS_DIVE15; /* PCFS Divider End 15 Register */
  MC_CGM_2_PCFS_DIVS15_tag PCFS_DIVS15; /* PCFS Divider Start 15 Register */
  uint8_t MC_CGM_2_reserved1[584];
  MC_CGM_2_MUX_0_CSC_tag MUX_0_CSC;    /* Clock Mux 0 Select Control Register */
  MC_CGM_2_MUX_0_CSS_tag MUX_0_CSS;    /* Clock Mux 0 Select Status Register */
  uint8_t MC_CGM_2_reserved2[56];
  MC_CGM_2_MUX_1_CSC_tag MUX_1_CSC;    /* Clock Mux 1 Select Control Register */
  MC_CGM_2_MUX_1_CSS_tag MUX_1_CSS;    /* Clock Mux 1 Select Status Register */
  MC_CGM_2_MUX_1_DC_0_tag MUX_1_DC_0;  /* Clock Mux 1 Divider 0 Control Register */
  uint8_t MC_CGM_2_reserved3[48];
  MC_CGM_2_MUX_1_DIV_UPD_STAT_tag MUX_1_DIV_UPD_STAT; /* Clock Mux 1 Divider Update Status Register */
  MC_CGM_2_MUX_2_CSC_tag MUX_2_CSC;    /* Clock Mux 2 Select Control Register */
  MC_CGM_2_MUX_2_CSS_tag MUX_2_CSS;    /* Clock Mux 2 Select Status Register */
  uint8_t MC_CGM_2_reserved4[56];
  MC_CGM_2_MUX_3_CSC_tag MUX_3_CSC;    /* Clock Mux 3 Select Control Register */
  MC_CGM_2_MUX_3_CSS_tag MUX_3_CSS;    /* Clock Mux 3 Select Status Register */
};


/* ============================================================================
   =============================== Module: MC_CGM_3 ===========================
   ============================================================================ */

typedef union MC_CGM_3_PCFS_SDUR_union_tag { /* PCFS Step Duration */
  vuint32_t R;
  struct {
    vuint32_t SDUR:16;                 /* Step duration */
    vuint32_t _unused_16:16;
  } B;
} MC_CGM_3_PCFS_SDUR_tag;

typedef union MC_CGM_3_PCFS_DIVC12_union_tag { /* PCFS Divider Change 12 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} MC_CGM_3_PCFS_DIVC12_tag;

typedef union MC_CGM_3_PCFS_DIVE12_union_tag { /* PCFS Divider End 12 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_3_PCFS_DIVE12_tag;

typedef union MC_CGM_3_PCFS_DIVS12_union_tag { /* PCFS Divider Start 12 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_3_PCFS_DIVS12_tag;

typedef union MC_CGM_3_MUX_0_CSC_union_tag { /* Clock Mux 0 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_3_MUX_0_CSC_tag;

typedef union MC_CGM_3_MUX_0_CSS_union_tag { /* Clock Mux 0 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_3_MUX_0_CSS_tag;

typedef union MC_CGM_3_MUX_1_CSC_union_tag { /* Clock Mux 1 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_3_MUX_1_CSC_tag;

typedef union MC_CGM_3_MUX_1_CSS_union_tag { /* Clock Mux 1 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_3_MUX_1_CSS_tag;

typedef union MC_CGM_3_MUX_1_DC_0_union_tag { /* Clock Mux 1 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_3_MUX_1_DC_0_tag;

typedef union MC_CGM_3_MUX_1_DIV_UPD_STAT_union_tag { /* Clock Mux 1 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 1 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_3_MUX_1_DIV_UPD_STAT_tag;

typedef union MC_CGM_3_MUX_2_CSC_union_tag { /* Clock Mux 2 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_3_MUX_2_CSC_tag;

typedef union MC_CGM_3_MUX_2_CSS_union_tag { /* Clock Mux 2 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_3_MUX_2_CSS_tag;

typedef union MC_CGM_3_MUX_2_DC_0_union_tag { /* Clock Mux 2 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_3_MUX_2_DC_0_tag;

typedef union MC_CGM_3_MUX_2_DC_1_union_tag { /* Clock Mux 2 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_3_MUX_2_DC_1_tag;

typedef union MC_CGM_3_MUX_2_DIV_UPD_STAT_union_tag { /* Clock Mux 2 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 2 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_3_MUX_2_DIV_UPD_STAT_tag;

typedef union MC_CGM_3_MUX_3_CSC_union_tag { /* Clock Mux 3 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_3_MUX_3_CSC_tag;

typedef union MC_CGM_3_MUX_3_CSS_union_tag { /* Clock Mux 3 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_3_MUX_3_CSS_tag;

typedef union MC_CGM_3_MUX_3_DC_0_union_tag { /* Clock Mux 3 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_3_MUX_3_DC_0_tag;

typedef union MC_CGM_3_MUX_3_DIV_UPD_STAT_union_tag { /* Clock Mux 3 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 3 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_3_MUX_3_DIV_UPD_STAT_tag;

typedef union MC_CGM_3_MUX_4_CSC_union_tag { /* Clock Mux 4 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CG:1;                    /* Clock gate */
    vuint32_t FCG:1;                   /* Force clock gate */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_3_MUX_4_CSC_tag;

typedef union MC_CGM_3_MUX_4_CSS_union_tag { /* Clock Mux 4 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t GRIP:1;                  /* Gating request is in progress. */
    vuint32_t CS:1;                    /* Clock status */
    vuint32_t _unused_18:2;
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_3_MUX_4_CSS_tag;

typedef union MC_CGM_3_MUX_4_DC_0_union_tag { /* Clock Mux 4 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_3_MUX_4_DC_0_tag;

typedef union MC_CGM_3_MUX_4_DIV_UPD_STAT_union_tag { /* Clock Mux 4 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 4 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_3_MUX_4_DIV_UPD_STAT_tag;

struct MC_CGM_3_tag {
  MC_CGM_3_PCFS_SDUR_tag PCFS_SDUR;    /* PCFS Step Duration */
  uint8_t MC_CGM_3_reserved0[132];
  MC_CGM_3_PCFS_DIVC12_tag PCFS_DIVC12; /* PCFS Divider Change 12 Register */
  MC_CGM_3_PCFS_DIVE12_tag PCFS_DIVE12; /* PCFS Divider End 12 Register */
  MC_CGM_3_PCFS_DIVS12_tag PCFS_DIVS12; /* PCFS Divider Start 12 Register */
  uint8_t MC_CGM_3_reserved1[620];
  MC_CGM_3_MUX_0_CSC_tag MUX_0_CSC;    /* Clock Mux 0 Select Control Register */
  MC_CGM_3_MUX_0_CSS_tag MUX_0_CSS;    /* Clock Mux 0 Select Status Register */
  uint8_t MC_CGM_3_reserved2[56];
  MC_CGM_3_MUX_1_CSC_tag MUX_1_CSC;    /* Clock Mux 1 Select Control Register */
  MC_CGM_3_MUX_1_CSS_tag MUX_1_CSS;    /* Clock Mux 1 Select Status Register */
  MC_CGM_3_MUX_1_DC_0_tag MUX_1_DC_0;  /* Clock Mux 1 Divider 0 Control Register */
  uint8_t MC_CGM_3_reserved3[48];
  MC_CGM_3_MUX_1_DIV_UPD_STAT_tag MUX_1_DIV_UPD_STAT; /* Clock Mux 1 Divider Update Status Register */
  MC_CGM_3_MUX_2_CSC_tag MUX_2_CSC;    /* Clock Mux 2 Select Control Register */
  MC_CGM_3_MUX_2_CSS_tag MUX_2_CSS;    /* Clock Mux 2 Select Status Register */
  MC_CGM_3_MUX_2_DC_0_tag MUX_2_DC_0;  /* Clock Mux 2 Divider 0 Control Register */
  MC_CGM_3_MUX_2_DC_1_tag MUX_2_DC_1;  /* Clock Mux 2 Divider 1 Control Register */
  uint8_t MC_CGM_3_reserved4[44];
  MC_CGM_3_MUX_2_DIV_UPD_STAT_tag MUX_2_DIV_UPD_STAT; /* Clock Mux 2 Divider Update Status Register */
  MC_CGM_3_MUX_3_CSC_tag MUX_3_CSC;    /* Clock Mux 3 Select Control Register */
  MC_CGM_3_MUX_3_CSS_tag MUX_3_CSS;    /* Clock Mux 3 Select Status Register */
  MC_CGM_3_MUX_3_DC_0_tag MUX_3_DC_0;  /* Clock Mux 3 Divider 0 Control Register */
  uint8_t MC_CGM_3_reserved5[48];
  MC_CGM_3_MUX_3_DIV_UPD_STAT_tag MUX_3_DIV_UPD_STAT; /* Clock Mux 3 Divider Update Status Register */
  MC_CGM_3_MUX_4_CSC_tag MUX_4_CSC;    /* Clock Mux 4 Select Control Register */
  MC_CGM_3_MUX_4_CSS_tag MUX_4_CSS;    /* Clock Mux 4 Select Status Register */
  MC_CGM_3_MUX_4_DC_0_tag MUX_4_DC_0;  /* Clock Mux 4 Divider 0 Control Register */
  uint8_t MC_CGM_3_reserved6[48];
  MC_CGM_3_MUX_4_DIV_UPD_STAT_tag MUX_4_DIV_UPD_STAT; /* Clock Mux 4 Divider Update Status Register */
};


/* ============================================================================
   =============================== Module: MC_CGM_4 ===========================
   ============================================================================ */

typedef union MC_CGM_4_PCFS_SDUR_union_tag { /* PCFS Step Duration */
  vuint32_t R;
  struct {
    vuint32_t SDUR:16;                 /* Step duration */
    vuint32_t _unused_16:16;
  } B;
} MC_CGM_4_PCFS_SDUR_tag;

typedef union MC_CGM_4_PCFS_DIVC15_union_tag { /* PCFS Divider Change 15 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} MC_CGM_4_PCFS_DIVC15_tag;

typedef union MC_CGM_4_PCFS_DIVE15_union_tag { /* PCFS Divider End 15 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_4_PCFS_DIVE15_tag;

typedef union MC_CGM_4_PCFS_DIVS15_union_tag { /* PCFS Divider Start 15 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_4_PCFS_DIVS15_tag;

typedef union MC_CGM_4_MUX_0_CSC_union_tag { /* Clock Mux 0 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_4_MUX_0_CSC_tag;

typedef union MC_CGM_4_MUX_0_CSS_union_tag { /* Clock Mux 0 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_4_MUX_0_CSS_tag;

typedef union MC_CGM_4_MUX_1_CSC_union_tag { /* Clock Mux 1 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_4_MUX_1_CSC_tag;

typedef union MC_CGM_4_MUX_1_CSS_union_tag { /* Clock Mux 1 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_4_MUX_1_CSS_tag;

typedef union MC_CGM_4_MUX_1_DC_0_union_tag { /* Clock Mux 1 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_1_DC_0_tag;

typedef union MC_CGM_4_MUX_1_DC_1_union_tag { /* Clock Mux 1 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_1_DC_1_tag;

typedef union MC_CGM_4_MUX_1_DIV_UPD_STAT_union_tag { /* Clock Mux 1 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 1 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_4_MUX_1_DIV_UPD_STAT_tag;

typedef union MC_CGM_4_MUX_2_CSC_union_tag { /* Clock Mux 2 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_2_CSC_tag;

typedef union MC_CGM_4_MUX_2_CSS_union_tag { /* Clock Mux 2 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_2_CSS_tag;

typedef union MC_CGM_4_MUX_2_DC_0_union_tag { /* Clock Mux 2 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_2_DC_0_tag;

typedef union MC_CGM_4_MUX_2_DC_1_union_tag { /* Clock Mux 2 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_2_DC_1_tag;

typedef union MC_CGM_4_MUX_2_DC_2_union_tag { /* Clock Mux 2 Divider 2 Control Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_FMT:2;               /* Division format */
    vuint32_t _unused_2:14;
    vuint32_t DIV:13;                  /* Division value */
    vuint32_t _unused_29:2;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_2_DC_2_tag;

typedef union MC_CGM_4_MUX_2_DC_3_union_tag { /* Clock Mux 2 Divider 3 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_2_DC_3_tag;

typedef union MC_CGM_4_MUX_2_DC_4_union_tag { /* Clock Mux 2 Divider 4 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_2_DC_4_tag;

typedef union MC_CGM_4_MUX_2_DC_5_union_tag { /* Clock Mux 2 Divider 5 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_2_DC_5_tag;

typedef union MC_CGM_4_MUX_2_DIV_UPD_STAT_union_tag { /* Clock Mux 2 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 2 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_4_MUX_2_DIV_UPD_STAT_tag;

typedef union MC_CGM_4_MUX_3_CSC_union_tag { /* Clock Mux 3 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_4_MUX_3_CSC_tag;

typedef union MC_CGM_4_MUX_3_CSS_union_tag { /* Clock Mux 3 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_4_MUX_3_CSS_tag;

typedef union MC_CGM_4_MUX_3_DC_0_union_tag { /* Clock Mux 3 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_3_DC_0_tag;

typedef union MC_CGM_4_MUX_3_DC_1_union_tag { /* Clock Mux 3 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_3_DC_1_tag;

typedef union MC_CGM_4_MUX_3_DC_2_union_tag { /* Clock Mux 3 Divider 2 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_3_DC_2_tag;

typedef union MC_CGM_4_MUX_3_DC_3_union_tag { /* Clock Mux 3 Divider 3 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_3_DC_3_tag;

typedef union MC_CGM_4_MUX_3_DC_4_union_tag { /* Clock Mux 3 Divider 4 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_3_DC_4_tag;

typedef union MC_CGM_4_MUX_3_DC_5_union_tag { /* Clock Mux 3 Divider 5 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_3_DC_5_tag;

typedef union MC_CGM_4_MUX_3_DC_6_union_tag { /* Clock Mux 3 Divider 6 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_3_DC_6_tag;

typedef union MC_CGM_4_MUX_3_DC_7_union_tag { /* Clock Mux 3 Divider 7 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:10;                  /* Division value */
    vuint32_t _unused_26:5;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_3_DC_7_tag;

typedef union MC_CGM_4_MUX_3_DIV_UPD_STAT_union_tag { /* Clock Mux 3 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 3 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_4_MUX_3_DIV_UPD_STAT_tag;

typedef union MC_CGM_4_MUX_4_CSC_union_tag { /* Clock Mux 4 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_4_CSC_tag;

typedef union MC_CGM_4_MUX_4_CSS_union_tag { /* Clock Mux 4 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_4_CSS_tag;

typedef union MC_CGM_4_MUX_5_CSC_union_tag { /* Clock Mux 5 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_5_CSC_tag;

typedef union MC_CGM_4_MUX_5_CSS_union_tag { /* Clock Mux 5 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_5_CSS_tag;

typedef union MC_CGM_4_MUX_6_CSC_union_tag { /* Clock Mux 6 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CG:1;                    /* Clock gate */
    vuint32_t FCG:1;                   /* Force clock gate */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_4_MUX_6_CSC_tag;

typedef union MC_CGM_4_MUX_6_CSS_union_tag { /* Clock Mux 6 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t GRIP:1;                  /* Gating request is in progress. */
    vuint32_t CS:1;                    /* Clock status */
    vuint32_t _unused_18:2;
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_4_MUX_6_CSS_tag;

typedef union MC_CGM_4_MUX_6_DC_0_union_tag { /* Clock Mux 6 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_6_DC_0_tag;

typedef union MC_CGM_4_MUX_6_DIV_UPD_STAT_union_tag { /* Clock Mux 6 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 6 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_4_MUX_6_DIV_UPD_STAT_tag;

typedef union MC_CGM_4_MUX_7_CSC_union_tag { /* Clock Mux 7 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_7_CSC_tag;

typedef union MC_CGM_4_MUX_7_CSS_union_tag { /* Clock Mux 7 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_7_CSS_tag;

typedef union MC_CGM_4_MUX_7_DC_0_union_tag { /* Clock Mux 7 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_7_DC_0_tag;

typedef union MC_CGM_4_MUX_7_DIV_UPD_STAT_union_tag { /* Clock Mux 7 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 7 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_4_MUX_7_DIV_UPD_STAT_tag;

typedef union MC_CGM_4_MUX_8_CSC_union_tag { /* Clock Mux 8 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_8_CSC_tag;

typedef union MC_CGM_4_MUX_8_CSS_union_tag { /* Clock Mux 8 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_8_CSS_tag;

typedef union MC_CGM_4_MUX_8_DC_0_union_tag { /* Clock Mux 8 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_8_DC_0_tag;

typedef union MC_CGM_4_MUX_8_DIV_UPD_STAT_union_tag { /* Clock Mux 8 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 8 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_4_MUX_8_DIV_UPD_STAT_tag;

typedef union MC_CGM_4_MUX_9_CSC_union_tag { /* Clock Mux 9 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_4_MUX_9_CSC_tag;

typedef union MC_CGM_4_MUX_9_CSS_union_tag { /* Clock Mux 9 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_4_MUX_9_CSS_tag;

typedef union MC_CGM_4_MUX_9_DC_0_union_tag { /* Clock Mux 9 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_9_DC_0_tag;

typedef union MC_CGM_4_MUX_9_DC_1_union_tag { /* Clock Mux 9 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_4_MUX_9_DC_1_tag;

typedef union MC_CGM_4_MUX_9_DIV_UPD_STAT_union_tag { /* Clock Mux 9 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 9 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_4_MUX_9_DIV_UPD_STAT_tag;

typedef union MC_CGM_4_MUX_10_CSC_union_tag { /* Clock Mux 10 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_10_CSC_tag;

typedef union MC_CGM_4_MUX_10_CSS_union_tag { /* Clock Mux 10 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_10_CSS_tag;

typedef union MC_CGM_4_MUX_11_CSC_union_tag { /* Clock Mux 11 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_11_CSC_tag;

typedef union MC_CGM_4_MUX_11_CSS_union_tag { /* Clock Mux 11 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_4_MUX_11_CSS_tag;

struct MC_CGM_4_tag {
  MC_CGM_4_PCFS_SDUR_tag PCFS_SDUR;    /* PCFS Step Duration */
  uint8_t MC_CGM_4_reserved0[168];
  MC_CGM_4_PCFS_DIVC15_tag PCFS_DIVC15; /* PCFS Divider Change 15 Register */
  MC_CGM_4_PCFS_DIVE15_tag PCFS_DIVE15; /* PCFS Divider End 15 Register */
  MC_CGM_4_PCFS_DIVS15_tag PCFS_DIVS15; /* PCFS Divider Start 15 Register */
  uint8_t MC_CGM_4_reserved1[584];
  MC_CGM_4_MUX_0_CSC_tag MUX_0_CSC;    /* Clock Mux 0 Select Control Register */
  MC_CGM_4_MUX_0_CSS_tag MUX_0_CSS;    /* Clock Mux 0 Select Status Register */
  uint8_t MC_CGM_4_reserved2[56];
  MC_CGM_4_MUX_1_CSC_tag MUX_1_CSC;    /* Clock Mux 1 Select Control Register */
  MC_CGM_4_MUX_1_CSS_tag MUX_1_CSS;    /* Clock Mux 1 Select Status Register */
  MC_CGM_4_MUX_1_DC_0_tag MUX_1_DC_0;  /* Clock Mux 1 Divider 0 Control Register */
  MC_CGM_4_MUX_1_DC_1_tag MUX_1_DC_1;  /* Clock Mux 1 Divider 1 Control Register */
  uint8_t MC_CGM_4_reserved3[44];
  MC_CGM_4_MUX_1_DIV_UPD_STAT_tag MUX_1_DIV_UPD_STAT; /* Clock Mux 1 Divider Update Status Register */
  MC_CGM_4_MUX_2_CSC_tag MUX_2_CSC;    /* Clock Mux 2 Select Control Register */
  MC_CGM_4_MUX_2_CSS_tag MUX_2_CSS;    /* Clock Mux 2 Select Status Register */
  MC_CGM_4_MUX_2_DC_0_tag MUX_2_DC_0;  /* Clock Mux 2 Divider 0 Control Register */
  MC_CGM_4_MUX_2_DC_1_tag MUX_2_DC_1;  /* Clock Mux 2 Divider 1 Control Register */
  MC_CGM_4_MUX_2_DC_2_tag MUX_2_DC_2;  /* Clock Mux 2 Divider 2 Control Register */
  MC_CGM_4_MUX_2_DC_3_tag MUX_2_DC_3;  /* Clock Mux 2 Divider 3 Control Register */
  MC_CGM_4_MUX_2_DC_4_tag MUX_2_DC_4;  /* Clock Mux 2 Divider 4 Control Register */
  MC_CGM_4_MUX_2_DC_5_tag MUX_2_DC_5;  /* Clock Mux 2 Divider 5 Control Register */
  uint8_t MC_CGM_4_reserved4[28];
  MC_CGM_4_MUX_2_DIV_UPD_STAT_tag MUX_2_DIV_UPD_STAT; /* Clock Mux 2 Divider Update Status Register */
  MC_CGM_4_MUX_3_CSC_tag MUX_3_CSC;    /* Clock Mux 3 Select Control Register */
  MC_CGM_4_MUX_3_CSS_tag MUX_3_CSS;    /* Clock Mux 3 Select Status Register */
  MC_CGM_4_MUX_3_DC_0_tag MUX_3_DC_0;  /* Clock Mux 3 Divider 0 Control Register */
  MC_CGM_4_MUX_3_DC_1_tag MUX_3_DC_1;  /* Clock Mux 3 Divider 1 Control Register */
  MC_CGM_4_MUX_3_DC_2_tag MUX_3_DC_2;  /* Clock Mux 3 Divider 2 Control Register */
  MC_CGM_4_MUX_3_DC_3_tag MUX_3_DC_3;  /* Clock Mux 3 Divider 3 Control Register */
  MC_CGM_4_MUX_3_DC_4_tag MUX_3_DC_4;  /* Clock Mux 3 Divider 4 Control Register */
  MC_CGM_4_MUX_3_DC_5_tag MUX_3_DC_5;  /* Clock Mux 3 Divider 5 Control Register */
  MC_CGM_4_MUX_3_DC_6_tag MUX_3_DC_6;  /* Clock Mux 3 Divider 6 Control Register */
  MC_CGM_4_MUX_3_DC_7_tag MUX_3_DC_7;  /* Clock Mux 3 Divider 7 Control Register */
  uint8_t MC_CGM_4_reserved5[20];
  MC_CGM_4_MUX_3_DIV_UPD_STAT_tag MUX_3_DIV_UPD_STAT; /* Clock Mux 3 Divider Update Status Register */
  MC_CGM_4_MUX_4_CSC_tag MUX_4_CSC;    /* Clock Mux 4 Select Control Register */
  MC_CGM_4_MUX_4_CSS_tag MUX_4_CSS;    /* Clock Mux 4 Select Status Register */
  uint8_t MC_CGM_4_reserved6[56];
  MC_CGM_4_MUX_5_CSC_tag MUX_5_CSC;    /* Clock Mux 5 Select Control Register */
  MC_CGM_4_MUX_5_CSS_tag MUX_5_CSS;    /* Clock Mux 5 Select Status Register */
  uint8_t MC_CGM_4_reserved7[56];
  MC_CGM_4_MUX_6_CSC_tag MUX_6_CSC;    /* Clock Mux 6 Select Control Register */
  MC_CGM_4_MUX_6_CSS_tag MUX_6_CSS;    /* Clock Mux 6 Select Status Register */
  MC_CGM_4_MUX_6_DC_0_tag MUX_6_DC_0;  /* Clock Mux 6 Divider 0 Control Register */
  uint8_t MC_CGM_4_reserved8[48];
  MC_CGM_4_MUX_6_DIV_UPD_STAT_tag MUX_6_DIV_UPD_STAT; /* Clock Mux 6 Divider Update Status Register */
  MC_CGM_4_MUX_7_CSC_tag MUX_7_CSC;    /* Clock Mux 7 Select Control Register */
  MC_CGM_4_MUX_7_CSS_tag MUX_7_CSS;    /* Clock Mux 7 Select Status Register */
  MC_CGM_4_MUX_7_DC_0_tag MUX_7_DC_0;  /* Clock Mux 7 Divider 0 Control Register */
  uint8_t MC_CGM_4_reserved9[48];
  MC_CGM_4_MUX_7_DIV_UPD_STAT_tag MUX_7_DIV_UPD_STAT; /* Clock Mux 7 Divider Update Status Register */
  MC_CGM_4_MUX_8_CSC_tag MUX_8_CSC;    /* Clock Mux 8 Select Control Register */
  MC_CGM_4_MUX_8_CSS_tag MUX_8_CSS;    /* Clock Mux 8 Select Status Register */
  MC_CGM_4_MUX_8_DC_0_tag MUX_8_DC_0;  /* Clock Mux 8 Divider 0 Control Register */
  uint8_t MC_CGM_4_reserved10[48];
  MC_CGM_4_MUX_8_DIV_UPD_STAT_tag MUX_8_DIV_UPD_STAT; /* Clock Mux 8 Divider Update Status Register */
  MC_CGM_4_MUX_9_CSC_tag MUX_9_CSC;    /* Clock Mux 9 Select Control Register */
  MC_CGM_4_MUX_9_CSS_tag MUX_9_CSS;    /* Clock Mux 9 Select Status Register */
  MC_CGM_4_MUX_9_DC_0_tag MUX_9_DC_0;  /* Clock Mux 9 Divider 0 Control Register */
  MC_CGM_4_MUX_9_DC_1_tag MUX_9_DC_1;  /* Clock Mux 9 Divider 1 Control Register */
  uint8_t MC_CGM_4_reserved11[44];
  MC_CGM_4_MUX_9_DIV_UPD_STAT_tag MUX_9_DIV_UPD_STAT; /* Clock Mux 9 Divider Update Status Register */
  MC_CGM_4_MUX_10_CSC_tag MUX_10_CSC;  /* Clock Mux 10 Select Control Register */
  MC_CGM_4_MUX_10_CSS_tag MUX_10_CSS;  /* Clock Mux 10 Select Status Register */
  uint8_t MC_CGM_4_reserved12[56];
  MC_CGM_4_MUX_11_CSC_tag MUX_11_CSC;  /* Clock Mux 11 Select Control Register */
  MC_CGM_4_MUX_11_CSS_tag MUX_11_CSS;  /* Clock Mux 11 Select Status Register */
};


/* ============================================================================
   =============================== Module: MC_CGM_5 ===========================
   ============================================================================ */

typedef union MC_CGM_5_PCFS_SDUR_union_tag { /* PCFS Step Duration */
  vuint32_t R;
  struct {
    vuint32_t SDUR:16;                 /* Step duration */
    vuint32_t _unused_16:16;
  } B;
} MC_CGM_5_PCFS_SDUR_tag;

typedef union MC_CGM_5_PCFS_DIVC15_union_tag { /* PCFS Divider Change 15 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} MC_CGM_5_PCFS_DIVC15_tag;

typedef union MC_CGM_5_PCFS_DIVE15_union_tag { /* PCFS Divider End 15 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_5_PCFS_DIVE15_tag;

typedef union MC_CGM_5_PCFS_DIVS15_union_tag { /* PCFS Divider Start 15 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_5_PCFS_DIVS15_tag;

typedef union MC_CGM_5_MUX_0_CSC_union_tag { /* Clock Mux 0 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_5_MUX_0_CSC_tag;

typedef union MC_CGM_5_MUX_0_CSS_union_tag { /* Clock Mux 0 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_5_MUX_0_CSS_tag;

typedef union MC_CGM_5_MUX_0_DC_0_union_tag { /* Clock Mux 0 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_5_MUX_0_DC_0_tag;

typedef union MC_CGM_5_MUX_0_DIV_UPD_STAT_union_tag { /* Clock Mux 0 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 0 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_5_MUX_0_DIV_UPD_STAT_tag;

typedef union MC_CGM_5_MUX_1_CSC_union_tag { /* Clock Mux 1 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_5_MUX_1_CSC_tag;

typedef union MC_CGM_5_MUX_1_CSS_union_tag { /* Clock Mux 1 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_5_MUX_1_CSS_tag;

typedef union MC_CGM_5_MUX_1_DC_0_union_tag { /* Clock Mux 1 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_5_MUX_1_DC_0_tag;

typedef union MC_CGM_5_MUX_1_DIV_UPD_STAT_union_tag { /* Clock Mux 1 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 1 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_5_MUX_1_DIV_UPD_STAT_tag;

typedef union MC_CGM_5_MUX_2_CSC_union_tag { /* Clock Mux 2 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_5_MUX_2_CSC_tag;

typedef union MC_CGM_5_MUX_2_CSS_union_tag { /* Clock Mux 2 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_5_MUX_2_CSS_tag;

typedef union MC_CGM_5_MUX_2_DC_0_union_tag { /* Clock Mux 2 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_5_MUX_2_DC_0_tag;

typedef union MC_CGM_5_MUX_2_DIV_UPD_STAT_union_tag { /* Clock Mux 2 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 2 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_5_MUX_2_DIV_UPD_STAT_tag;

typedef union MC_CGM_5_MUX_3_CSC_union_tag { /* Clock Mux 3 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_5_MUX_3_CSC_tag;

typedef union MC_CGM_5_MUX_3_CSS_union_tag { /* Clock Mux 3 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_5_MUX_3_CSS_tag;

typedef union MC_CGM_5_MUX_4_CSC_union_tag { /* Clock Mux 4 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CG:1;                    /* Clock gate */
    vuint32_t FCG:1;                   /* Force clock gate */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_5_MUX_4_CSC_tag;

typedef union MC_CGM_5_MUX_4_CSS_union_tag { /* Clock Mux 4 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t GRIP:1;                  /* Gating request is in progress. */
    vuint32_t CS:1;                    /* Clock status */
    vuint32_t _unused_18:2;
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_5_MUX_4_CSS_tag;

typedef union MC_CGM_5_MUX_4_DC_0_union_tag { /* Clock Mux 4 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_5_MUX_4_DC_0_tag;

typedef union MC_CGM_5_MUX_4_DIV_UPD_STAT_union_tag { /* Clock Mux 4 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 4 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_5_MUX_4_DIV_UPD_STAT_tag;

typedef union MC_CGM_5_MUX_5_CSC_union_tag { /* Clock Mux 5 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:5;                /* Clock source selection control */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_5_MUX_5_CSC_tag;

typedef union MC_CGM_5_MUX_5_CSS_union_tag { /* Clock Mux 5 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:5;               /* Clock source selection status */
    vuint32_t _unused_29:3;
  } B;
} MC_CGM_5_MUX_5_CSS_tag;

typedef union MC_CGM_5_MUX_5_DC_0_union_tag { /* Clock Mux 5 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_5_MUX_5_DC_0_tag;

typedef union MC_CGM_5_MUX_5_DC_1_union_tag { /* Clock Mux 5 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_5_MUX_5_DC_1_tag;

typedef union MC_CGM_5_MUX_5_DC_2_union_tag { /* Clock Mux 5 Divider 2 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_5_MUX_5_DC_2_tag;

typedef union MC_CGM_5_MUX_5_DIV_UPD_STAT_union_tag { /* Clock Mux 5 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 5 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_5_MUX_5_DIV_UPD_STAT_tag;

struct MC_CGM_5_tag {
  MC_CGM_5_PCFS_SDUR_tag PCFS_SDUR;    /* PCFS Step Duration */
  uint8_t MC_CGM_5_reserved0[168];
  MC_CGM_5_PCFS_DIVC15_tag PCFS_DIVC15; /* PCFS Divider Change 15 Register */
  MC_CGM_5_PCFS_DIVE15_tag PCFS_DIVE15; /* PCFS Divider End 15 Register */
  MC_CGM_5_PCFS_DIVS15_tag PCFS_DIVS15; /* PCFS Divider Start 15 Register */
  uint8_t MC_CGM_5_reserved1[584];
  MC_CGM_5_MUX_0_CSC_tag MUX_0_CSC;    /* Clock Mux 0 Select Control Register */
  MC_CGM_5_MUX_0_CSS_tag MUX_0_CSS;    /* Clock Mux 0 Select Status Register */
  MC_CGM_5_MUX_0_DC_0_tag MUX_0_DC_0;  /* Clock Mux 0 Divider 0 Control Register */
  uint8_t MC_CGM_5_reserved2[48];
  MC_CGM_5_MUX_0_DIV_UPD_STAT_tag MUX_0_DIV_UPD_STAT; /* Clock Mux 0 Divider Update Status Register */
  MC_CGM_5_MUX_1_CSC_tag MUX_1_CSC;    /* Clock Mux 1 Select Control Register */
  MC_CGM_5_MUX_1_CSS_tag MUX_1_CSS;    /* Clock Mux 1 Select Status Register */
  MC_CGM_5_MUX_1_DC_0_tag MUX_1_DC_0;  /* Clock Mux 1 Divider 0 Control Register */
  uint8_t MC_CGM_5_reserved3[48];
  MC_CGM_5_MUX_1_DIV_UPD_STAT_tag MUX_1_DIV_UPD_STAT; /* Clock Mux 1 Divider Update Status Register */
  MC_CGM_5_MUX_2_CSC_tag MUX_2_CSC;    /* Clock Mux 2 Select Control Register */
  MC_CGM_5_MUX_2_CSS_tag MUX_2_CSS;    /* Clock Mux 2 Select Status Register */
  MC_CGM_5_MUX_2_DC_0_tag MUX_2_DC_0;  /* Clock Mux 2 Divider 0 Control Register */
  uint8_t MC_CGM_5_reserved4[48];
  MC_CGM_5_MUX_2_DIV_UPD_STAT_tag MUX_2_DIV_UPD_STAT; /* Clock Mux 2 Divider Update Status Register */
  MC_CGM_5_MUX_3_CSC_tag MUX_3_CSC;    /* Clock Mux 3 Select Control Register */
  MC_CGM_5_MUX_3_CSS_tag MUX_3_CSS;    /* Clock Mux 3 Select Status Register */
  uint8_t MC_CGM_5_reserved5[56];
  MC_CGM_5_MUX_4_CSC_tag MUX_4_CSC;    /* Clock Mux 4 Select Control Register */
  MC_CGM_5_MUX_4_CSS_tag MUX_4_CSS;    /* Clock Mux 4 Select Status Register */
  MC_CGM_5_MUX_4_DC_0_tag MUX_4_DC_0;  /* Clock Mux 4 Divider 0 Control Register */
  uint8_t MC_CGM_5_reserved6[48];
  MC_CGM_5_MUX_4_DIV_UPD_STAT_tag MUX_4_DIV_UPD_STAT; /* Clock Mux 4 Divider Update Status Register */
  MC_CGM_5_MUX_5_CSC_tag MUX_5_CSC;    /* Clock Mux 5 Select Control Register */
  MC_CGM_5_MUX_5_CSS_tag MUX_5_CSS;    /* Clock Mux 5 Select Status Register */
  MC_CGM_5_MUX_5_DC_0_tag MUX_5_DC_0;  /* Clock Mux 5 Divider 0 Control Register */
  MC_CGM_5_MUX_5_DC_1_tag MUX_5_DC_1;  /* Clock Mux 5 Divider 1 Control Register */
  MC_CGM_5_MUX_5_DC_2_tag MUX_5_DC_2;  /* Clock Mux 5 Divider 2 Control Register */
  uint8_t MC_CGM_5_reserved7[40];
  MC_CGM_5_MUX_5_DIV_UPD_STAT_tag MUX_5_DIV_UPD_STAT; /* Clock Mux 5 Divider Update Status Register */
};


/* ============================================================================
   =============================== Module: MC_CGM_6 ===========================
   ============================================================================ */

typedef union MC_CGM_6_PCFS_SDUR_union_tag { /* PCFS Step Duration */
  vuint32_t R;
  struct {
    vuint32_t SDUR:16;                 /* Step duration */
    vuint32_t _unused_16:16;
  } B;
} MC_CGM_6_PCFS_SDUR_tag;

typedef union MC_CGM_6_PCFS_DIVC40_union_tag { /* PCFS Divider Change 40 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} MC_CGM_6_PCFS_DIVC40_tag;

typedef union MC_CGM_6_PCFS_DIVE40_union_tag { /* PCFS Divider End 40 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_6_PCFS_DIVE40_tag;

typedef union MC_CGM_6_PCFS_DIVS40_union_tag { /* PCFS Divider Start 40 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} MC_CGM_6_PCFS_DIVS40_tag;

typedef union MC_CGM_6_MUX_0_CSC_union_tag { /* Clock Mux 0 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:6;                /* Clock source selection control */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_6_MUX_0_CSC_tag;

typedef union MC_CGM_6_MUX_0_CSS_union_tag { /* Clock Mux 0 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:6;               /* Clock source selection status */
    vuint32_t _unused_30:2;
  } B;
} MC_CGM_6_MUX_0_CSS_tag;

typedef union MC_CGM_6_MUX_0_DC_0_union_tag { /* Clock Mux 0 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_6_MUX_0_DC_0_tag;

typedef union MC_CGM_6_MUX_0_DC_1_union_tag { /* Clock Mux 0 Divider 1 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_6_MUX_0_DC_1_tag;

typedef union MC_CGM_6_MUX_0_DIV_UPD_STAT_union_tag { /* Clock Mux 0 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 0 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_6_MUX_0_DIV_UPD_STAT_tag;

typedef union MC_CGM_6_MUX_1_CSC_union_tag { /* Clock Mux 1 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_6_MUX_1_CSC_tag;

typedef union MC_CGM_6_MUX_1_CSS_union_tag { /* Clock Mux 1 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} MC_CGM_6_MUX_1_CSS_tag;

typedef union MC_CGM_6_MUX_1_DC_0_union_tag { /* Clock Mux 1 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} MC_CGM_6_MUX_1_DC_0_tag;

typedef union MC_CGM_6_MUX_1_DIV_UPD_STAT_union_tag { /* Clock Mux 1 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 1 */
    vuint32_t _unused_1:31;
  } B;
} MC_CGM_6_MUX_1_DIV_UPD_STAT_tag;

struct MC_CGM_6_tag {
  MC_CGM_6_PCFS_SDUR_tag PCFS_SDUR;    /* PCFS Step Duration */
  uint8_t MC_CGM_6_reserved0[468];
  MC_CGM_6_PCFS_DIVC40_tag PCFS_DIVC40; /* PCFS Divider Change 40 Register */
  MC_CGM_6_PCFS_DIVE40_tag PCFS_DIVE40; /* PCFS Divider End 40 Register */
  MC_CGM_6_PCFS_DIVS40_tag PCFS_DIVS40; /* PCFS Divider Start 40 Register */
  uint8_t MC_CGM_6_reserved1[284];
  MC_CGM_6_MUX_0_CSC_tag MUX_0_CSC;    /* Clock Mux 0 Select Control Register */
  MC_CGM_6_MUX_0_CSS_tag MUX_0_CSS;    /* Clock Mux 0 Select Status Register */
  MC_CGM_6_MUX_0_DC_0_tag MUX_0_DC_0;  /* Clock Mux 0 Divider 0 Control Register */
  MC_CGM_6_MUX_0_DC_1_tag MUX_0_DC_1;  /* Clock Mux 0 Divider 1 Control Register */
  uint8_t MC_CGM_6_reserved2[44];
  MC_CGM_6_MUX_0_DIV_UPD_STAT_tag MUX_0_DIV_UPD_STAT; /* Clock Mux 0 Divider Update Status Register */
  MC_CGM_6_MUX_1_CSC_tag MUX_1_CSC;    /* Clock Mux 1 Select Control Register */
  MC_CGM_6_MUX_1_CSS_tag MUX_1_CSS;    /* Clock Mux 1 Select Status Register */
  MC_CGM_6_MUX_1_DC_0_tag MUX_1_DC_0;  /* Clock Mux 1 Divider 0 Control Register */
  uint8_t MC_CGM_6_reserved3[48];
  MC_CGM_6_MUX_1_DIV_UPD_STAT_tag MUX_1_DIV_UPD_STAT; /* Clock Mux 1 Divider Update Status Register */
};


/* ============================================================================
   =============================== Module: MC_ME ==============================
   ============================================================================ */

typedef union MC_ME_CTL_KEY_union_tag { /* Control Key Register */
  vuint32_t R;
  struct {
    vuint32_t KEY:16;                  /* Control key */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} MC_ME_CTL_KEY_tag;

typedef union MC_ME_MODE_CONF_union_tag { /* Mode Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t DEST_RST:1;              /* Destructive reset request */
    vuint32_t FUNC_RST:1;              /* Functional reset request */
    vuint32_t _unused_2:14;            /* Reserved */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} MC_ME_MODE_CONF_tag;

typedef union MC_ME_MODE_UPD_union_tag { /* Mode Update Register */
  vuint32_t R;
  struct {
    vuint32_t MODE_UPD:1;              /* Mode update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_MODE_UPD_tag;

typedef union MC_ME_PRTN0_PCONF_union_tag { /* Partition 0 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t PCE:1;                   /* Partition clock enable */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN0_PCONF_tag;

typedef union MC_ME_PRTN0_PUPD_union_tag { /* Partition 0 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t PCUD:1;                  /* Partition clock update */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN0_PUPD_tag;

typedef union MC_ME_PRTN0_STAT_union_tag { /* Partition 0 Status Register */
  vuint32_t R;
  struct {
    vuint32_t PCS:1;                   /* Partition clock status */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN0_STAT_tag;

typedef union MC_ME_PRTN0_CORE0_PCONF_union_tag { /* Partition 0 Core 0 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 0 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE0_PCONF_tag;

typedef union MC_ME_PRTN0_CORE0_PUPD_union_tag { /* Partition 0 Core 0 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 0 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE0_PUPD_tag;

typedef union MC_ME_PRTN0_CORE0_STAT_union_tag { /* Partition 0 Core 0 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 0 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN0_CORE0_STAT_tag;

typedef union MC_ME_PRTN0_CORE0_ADDR_union_tag { /* Partition 0 Core 0 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN0_CORE0_ADDR_tag;

typedef union MC_ME_PRTN0_CORE1_PCONF_union_tag { /* Partition 0 Core 1 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 1 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE1_PCONF_tag;

typedef union MC_ME_PRTN0_CORE1_PUPD_union_tag { /* Partition 0 Core 1 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 1 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE1_PUPD_tag;

typedef union MC_ME_PRTN0_CORE1_STAT_union_tag { /* Partition 0 Core 1 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 1 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN0_CORE1_STAT_tag;

typedef union MC_ME_PRTN0_CORE1_ADDR_union_tag { /* Partition 0 Core 1 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN0_CORE1_ADDR_tag;

typedef union MC_ME_PRTN0_CORE2_PCONF_union_tag { /* Partition 0 Core 2 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 2 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE2_PCONF_tag;

typedef union MC_ME_PRTN0_CORE2_PUPD_union_tag { /* Partition 0 Core 2 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 2 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE2_PUPD_tag;

typedef union MC_ME_PRTN0_CORE2_STAT_union_tag { /* Partition 0 Core 2 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 2 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN0_CORE2_STAT_tag;

typedef union MC_ME_PRTN0_CORE2_ADDR_union_tag { /* Partition 0 Core 2 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN0_CORE2_ADDR_tag;

typedef union MC_ME_PRTN0_CORE3_PCONF_union_tag { /* Partition 0 Core 3 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 3 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE3_PCONF_tag;

typedef union MC_ME_PRTN0_CORE3_PUPD_union_tag { /* Partition 0 Core 3 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 3 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE3_PUPD_tag;

typedef union MC_ME_PRTN0_CORE3_STAT_union_tag { /* Partition 0 Core 3 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 3 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN0_CORE3_STAT_tag;

typedef union MC_ME_PRTN0_CORE3_ADDR_union_tag { /* Partition 0 Core 3 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN0_CORE3_ADDR_tag;

typedef union MC_ME_PRTN0_CORE4_PCONF_union_tag { /* Partition 0 Core 4 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 4 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE4_PCONF_tag;

typedef union MC_ME_PRTN0_CORE4_PUPD_union_tag { /* Partition 0 Core 4 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 4 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE4_PUPD_tag;

typedef union MC_ME_PRTN0_CORE4_STAT_union_tag { /* Partition 0 Core 4 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 4 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN0_CORE4_STAT_tag;

typedef union MC_ME_PRTN0_CORE4_ADDR_union_tag { /* Partition 0 Core 4 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN0_CORE4_ADDR_tag;

typedef union MC_ME_PRTN0_CORE5_PCONF_union_tag { /* Partition 0 Core 5 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 5 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE5_PCONF_tag;

typedef union MC_ME_PRTN0_CORE5_PUPD_union_tag { /* Partition 0 Core 5 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 5 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE5_PUPD_tag;

typedef union MC_ME_PRTN0_CORE5_STAT_union_tag { /* Partition 0 Core 5 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 5 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN0_CORE5_STAT_tag;

typedef union MC_ME_PRTN0_CORE5_ADDR_union_tag { /* Partition 0 Core 5 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN0_CORE5_ADDR_tag;

typedef union MC_ME_PRTN0_CORE6_PCONF_union_tag { /* Partition 0 Core 6 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 6 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE6_PCONF_tag;

typedef union MC_ME_PRTN0_CORE6_PUPD_union_tag { /* Partition 0 Core 6 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 6 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE6_PUPD_tag;

typedef union MC_ME_PRTN0_CORE6_STAT_union_tag { /* Partition 0 Core 6 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 6 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN0_CORE6_STAT_tag;

typedef union MC_ME_PRTN0_CORE6_ADDR_union_tag { /* Partition 0 Core 6 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN0_CORE6_ADDR_tag;

typedef union MC_ME_PRTN0_CORE7_PCONF_union_tag { /* Partition 0 Core 7 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 7 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE7_PCONF_tag;

typedef union MC_ME_PRTN0_CORE7_PUPD_union_tag { /* Partition 0 Core 7 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 7 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN0_CORE7_PUPD_tag;

typedef union MC_ME_PRTN0_CORE7_STAT_union_tag { /* Partition 0 Core 7 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 7 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN0_CORE7_STAT_tag;

typedef union MC_ME_PRTN0_CORE7_ADDR_union_tag { /* Partition 0 Core 7 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN0_CORE7_ADDR_tag;

typedef union MC_ME_PRTN1_PCONF_union_tag { /* Partition 1 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t PCE:1;                   /* Partition clock enable */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t OSSE:1;                  /* Output safe stating enable */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN1_PCONF_tag;

typedef union MC_ME_PRTN1_PUPD_union_tag { /* Partition 1 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t PCUD:1;                  /* Partition clock update */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t OSSUD:1;                 /* Output safe stating update */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN1_PUPD_tag;

typedef union MC_ME_PRTN1_STAT_union_tag { /* Partition 1 Status Register */
  vuint32_t R;
  struct {
    vuint32_t PCS:1;                   /* Partition clock status */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t OSSS:1;                  /* Output safe stating status */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN1_STAT_tag;

typedef union MC_ME_PRTN1_CORE0_PCONF_union_tag { /* Partition 1 Core 0 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 0 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN1_CORE0_PCONF_tag;

typedef union MC_ME_PRTN1_CORE0_PUPD_union_tag { /* Partition 1 Core 0 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 0 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN1_CORE0_PUPD_tag;

typedef union MC_ME_PRTN1_CORE0_STAT_union_tag { /* Partition 1 Core 0 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 0 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN1_CORE0_STAT_tag;

typedef union MC_ME_PRTN1_CORE0_ADDR_union_tag { /* Partition 1 Core 0 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN1_CORE0_ADDR_tag;

typedef union MC_ME_PRTN1_CORE1_PCONF_union_tag { /* Partition 1 Core 1 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 1 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN1_CORE1_PCONF_tag;

typedef union MC_ME_PRTN1_CORE1_PUPD_union_tag { /* Partition 1 Core 1 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 1 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN1_CORE1_PUPD_tag;

typedef union MC_ME_PRTN1_CORE1_STAT_union_tag { /* Partition 1 Core 1 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 1 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN1_CORE1_STAT_tag;

typedef union MC_ME_PRTN1_CORE1_ADDR_union_tag { /* Partition 1 Core 1 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN1_CORE1_ADDR_tag;

typedef union MC_ME_PRTN1_CORE2_PCONF_union_tag { /* Partition 1 Core 2 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 2 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN1_CORE2_PCONF_tag;

typedef union MC_ME_PRTN1_CORE2_PUPD_union_tag { /* Partition 1 Core 2 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 2 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN1_CORE2_PUPD_tag;

typedef union MC_ME_PRTN1_CORE2_STAT_union_tag { /* Partition 1 Core 2 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 2 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN1_CORE2_STAT_tag;

typedef union MC_ME_PRTN1_CORE2_ADDR_union_tag { /* Partition 1 Core 2 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN1_CORE2_ADDR_tag;

typedef union MC_ME_PRTN1_CORE3_PCONF_union_tag { /* Partition 1 Core 3 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 3 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN1_CORE3_PCONF_tag;

typedef union MC_ME_PRTN1_CORE3_PUPD_union_tag { /* Partition 1 Core 3 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 3 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN1_CORE3_PUPD_tag;

typedef union MC_ME_PRTN1_CORE3_STAT_union_tag { /* Partition 1 Core 3 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 3 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN1_CORE3_STAT_tag;

typedef union MC_ME_PRTN1_CORE3_ADDR_union_tag { /* Partition 1 Core 3 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN1_CORE3_ADDR_tag;

typedef union MC_ME_PRTN2_PCONF_union_tag { /* Partition 2 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t PCE:1;                   /* Partition clock enable */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t OSSE:1;                  /* Output safe stating enable */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN2_PCONF_tag;

typedef union MC_ME_PRTN2_PUPD_union_tag { /* Partition 2 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t PCUD:1;                  /* Partition clock update */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t OSSUD:1;                 /* Output safe stating update */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN2_PUPD_tag;

typedef union MC_ME_PRTN2_STAT_union_tag { /* Partition 2 Status Register */
  vuint32_t R;
  struct {
    vuint32_t PCS:1;                   /* Partition clock status */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t OSSS:1;                  /* Output safe stating status */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MC_ME_PRTN2_STAT_tag;

typedef union MC_ME_PRTN2_CORE0_PCONF_union_tag { /* Partition 2 Core 0 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 0 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN2_CORE0_PCONF_tag;

typedef union MC_ME_PRTN2_CORE0_PUPD_union_tag { /* Partition 2 Core 0 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 0 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN2_CORE0_PUPD_tag;

typedef union MC_ME_PRTN2_CORE0_STAT_union_tag { /* Partition 2 Core 0 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 0 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN2_CORE0_STAT_tag;

typedef union MC_ME_PRTN2_CORE0_ADDR_union_tag { /* Partition 2 Core 0 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN2_CORE0_ADDR_tag;

typedef union MC_ME_PRTN2_CORE1_PCONF_union_tag { /* Partition 2 Core 1 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 1 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN2_CORE1_PCONF_tag;

typedef union MC_ME_PRTN2_CORE1_PUPD_union_tag { /* Partition 2 Core 1 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 1 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN2_CORE1_PUPD_tag;

typedef union MC_ME_PRTN2_CORE1_STAT_union_tag { /* Partition 2 Core 1 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 1 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN2_CORE1_STAT_tag;

typedef union MC_ME_PRTN2_CORE1_ADDR_union_tag { /* Partition 2 Core 1 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN2_CORE1_ADDR_tag;

typedef union MC_ME_PRTN2_CORE2_PCONF_union_tag { /* Partition 2 Core 2 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 2 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN2_CORE2_PCONF_tag;

typedef union MC_ME_PRTN2_CORE2_PUPD_union_tag { /* Partition 2 Core 2 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 2 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN2_CORE2_PUPD_tag;

typedef union MC_ME_PRTN2_CORE2_STAT_union_tag { /* Partition 2 Core 2 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 2 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN2_CORE2_STAT_tag;

typedef union MC_ME_PRTN2_CORE2_ADDR_union_tag { /* Partition 2 Core 2 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN2_CORE2_ADDR_tag;

typedef union MC_ME_PRTN2_CORE3_PCONF_union_tag { /* Partition 2 Core 3 Process Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CCE:1;                   /* Core 3 clock enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN2_CORE3_PCONF_tag;

typedef union MC_ME_PRTN2_CORE3_PUPD_union_tag { /* Partition 2 Core 3 Process Update Register */
  vuint32_t R;
  struct {
    vuint32_t CCUPD:1;                 /* Core 3 clock update */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MC_ME_PRTN2_CORE3_PUPD_tag;

typedef union MC_ME_PRTN2_CORE3_STAT_union_tag { /* Partition 2 Core 3 Status Register */
  vuint32_t R;
  struct {
    vuint32_t CCS:1;                   /* Core 3 clock process status */
    vuint32_t _unused_1:30;            /* Reserved */
    vuint32_t WFI:1;                   /* Wait for interrupt status */
  } B;
} MC_ME_PRTN2_CORE3_STAT_tag;

typedef union MC_ME_PRTN2_CORE3_ADDR_union_tag { /* Partition 2 Core 3 Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADDR:30;                 /* Address */
  } B;
} MC_ME_PRTN2_CORE3_ADDR_tag;

struct MC_ME_tag {
  MC_ME_CTL_KEY_tag CTL_KEY;           /* Control Key Register */
  MC_ME_MODE_CONF_tag MODE_CONF;       /* Mode Configuration Register */
  MC_ME_MODE_UPD_tag MODE_UPD;         /* Mode Update Register */
  uint8_t MC_ME_reserved0[244];
  MC_ME_PRTN0_PCONF_tag PRTN0_PCONF;   /* Partition 0 Process Configuration Register */
  MC_ME_PRTN0_PUPD_tag PRTN0_PUPD;     /* Partition 0 Process Update Register */
  MC_ME_PRTN0_STAT_tag PRTN0_STAT;     /* Partition 0 Status Register */
  uint8_t MC_ME_reserved1[52];
  MC_ME_PRTN0_CORE0_PCONF_tag PRTN0_CORE0_PCONF; /* Partition 0 Core 0 Process Configuration Register */
  MC_ME_PRTN0_CORE0_PUPD_tag PRTN0_CORE0_PUPD; /* Partition 0 Core 0 Process Update Register */
  MC_ME_PRTN0_CORE0_STAT_tag PRTN0_CORE0_STAT; /* Partition 0 Core 0 Status Register */
  MC_ME_PRTN0_CORE0_ADDR_tag PRTN0_CORE0_ADDR; /* Partition 0 Core 0 Address Register */
  uint8_t MC_ME_reserved2[16];
  MC_ME_PRTN0_CORE1_PCONF_tag PRTN0_CORE1_PCONF; /* Partition 0 Core 1 Process Configuration Register */
  MC_ME_PRTN0_CORE1_PUPD_tag PRTN0_CORE1_PUPD; /* Partition 0 Core 1 Process Update Register */
  MC_ME_PRTN0_CORE1_STAT_tag PRTN0_CORE1_STAT; /* Partition 0 Core 1 Status Register */
  MC_ME_PRTN0_CORE1_ADDR_tag PRTN0_CORE1_ADDR; /* Partition 0 Core 1 Address Register */
  uint8_t MC_ME_reserved3[16];
  MC_ME_PRTN0_CORE2_PCONF_tag PRTN0_CORE2_PCONF; /* Partition 0 Core 2 Process Configuration Register */
  MC_ME_PRTN0_CORE2_PUPD_tag PRTN0_CORE2_PUPD; /* Partition 0 Core 2 Process Update Register */
  MC_ME_PRTN0_CORE2_STAT_tag PRTN0_CORE2_STAT; /* Partition 0 Core 2 Status Register */
  MC_ME_PRTN0_CORE2_ADDR_tag PRTN0_CORE2_ADDR; /* Partition 0 Core 2 Address Register */
  uint8_t MC_ME_reserved4[16];
  MC_ME_PRTN0_CORE3_PCONF_tag PRTN0_CORE3_PCONF; /* Partition 0 Core 3 Process Configuration Register */
  MC_ME_PRTN0_CORE3_PUPD_tag PRTN0_CORE3_PUPD; /* Partition 0 Core 3 Process Update Register */
  MC_ME_PRTN0_CORE3_STAT_tag PRTN0_CORE3_STAT; /* Partition 0 Core 3 Status Register */
  MC_ME_PRTN0_CORE3_ADDR_tag PRTN0_CORE3_ADDR; /* Partition 0 Core 3 Address Register */
  uint8_t MC_ME_reserved5[16];
  MC_ME_PRTN0_CORE4_PCONF_tag PRTN0_CORE4_PCONF; /* Partition 0 Core 4 Process Configuration Register */
  MC_ME_PRTN0_CORE4_PUPD_tag PRTN0_CORE4_PUPD; /* Partition 0 Core 4 Process Update Register */
  MC_ME_PRTN0_CORE4_STAT_tag PRTN0_CORE4_STAT; /* Partition 0 Core 4 Status Register */
  MC_ME_PRTN0_CORE4_ADDR_tag PRTN0_CORE4_ADDR; /* Partition 0 Core 4 Address Register */
  uint8_t MC_ME_reserved6[16];
  MC_ME_PRTN0_CORE5_PCONF_tag PRTN0_CORE5_PCONF; /* Partition 0 Core 5 Process Configuration Register */
  MC_ME_PRTN0_CORE5_PUPD_tag PRTN0_CORE5_PUPD; /* Partition 0 Core 5 Process Update Register */
  MC_ME_PRTN0_CORE5_STAT_tag PRTN0_CORE5_STAT; /* Partition 0 Core 5 Status Register */
  MC_ME_PRTN0_CORE5_ADDR_tag PRTN0_CORE5_ADDR; /* Partition 0 Core 5 Address Register */
  uint8_t MC_ME_reserved7[16];
  MC_ME_PRTN0_CORE6_PCONF_tag PRTN0_CORE6_PCONF; /* Partition 0 Core 6 Process Configuration Register */
  MC_ME_PRTN0_CORE6_PUPD_tag PRTN0_CORE6_PUPD; /* Partition 0 Core 6 Process Update Register */
  MC_ME_PRTN0_CORE6_STAT_tag PRTN0_CORE6_STAT; /* Partition 0 Core 6 Status Register */
  MC_ME_PRTN0_CORE6_ADDR_tag PRTN0_CORE6_ADDR; /* Partition 0 Core 6 Address Register */
  uint8_t MC_ME_reserved8[16];
  MC_ME_PRTN0_CORE7_PCONF_tag PRTN0_CORE7_PCONF; /* Partition 0 Core 7 Process Configuration Register */
  MC_ME_PRTN0_CORE7_PUPD_tag PRTN0_CORE7_PUPD; /* Partition 0 Core 7 Process Update Register */
  MC_ME_PRTN0_CORE7_STAT_tag PRTN0_CORE7_STAT; /* Partition 0 Core 7 Status Register */
  MC_ME_PRTN0_CORE7_ADDR_tag PRTN0_CORE7_ADDR; /* Partition 0 Core 7 Address Register */
  uint8_t MC_ME_reserved9[208];
  MC_ME_PRTN1_PCONF_tag PRTN1_PCONF;   /* Partition 1 Process Configuration Register */
  MC_ME_PRTN1_PUPD_tag PRTN1_PUPD;     /* Partition 1 Process Update Register */
  MC_ME_PRTN1_STAT_tag PRTN1_STAT;     /* Partition 1 Status Register */
  uint8_t MC_ME_reserved10[52];
  MC_ME_PRTN1_CORE0_PCONF_tag PRTN1_CORE0_PCONF; /* Partition 1 Core 0 Process Configuration Register */
  MC_ME_PRTN1_CORE0_PUPD_tag PRTN1_CORE0_PUPD; /* Partition 1 Core 0 Process Update Register */
  MC_ME_PRTN1_CORE0_STAT_tag PRTN1_CORE0_STAT; /* Partition 1 Core 0 Status Register */
  MC_ME_PRTN1_CORE0_ADDR_tag PRTN1_CORE0_ADDR; /* Partition 1 Core 0 Address Register */
  uint8_t MC_ME_reserved11[16];
  MC_ME_PRTN1_CORE1_PCONF_tag PRTN1_CORE1_PCONF; /* Partition 1 Core 1 Process Configuration Register */
  MC_ME_PRTN1_CORE1_PUPD_tag PRTN1_CORE1_PUPD; /* Partition 1 Core 1 Process Update Register */
  MC_ME_PRTN1_CORE1_STAT_tag PRTN1_CORE1_STAT; /* Partition 1 Core 1 Status Register */
  MC_ME_PRTN1_CORE1_ADDR_tag PRTN1_CORE1_ADDR; /* Partition 1 Core 1 Address Register */
  uint8_t MC_ME_reserved12[16];
  MC_ME_PRTN1_CORE2_PCONF_tag PRTN1_CORE2_PCONF; /* Partition 1 Core 2 Process Configuration Register */
  MC_ME_PRTN1_CORE2_PUPD_tag PRTN1_CORE2_PUPD; /* Partition 1 Core 2 Process Update Register */
  MC_ME_PRTN1_CORE2_STAT_tag PRTN1_CORE2_STAT; /* Partition 1 Core 2 Status Register */
  MC_ME_PRTN1_CORE2_ADDR_tag PRTN1_CORE2_ADDR; /* Partition 1 Core 2 Address Register */
  uint8_t MC_ME_reserved13[16];
  MC_ME_PRTN1_CORE3_PCONF_tag PRTN1_CORE3_PCONF; /* Partition 1 Core 3 Process Configuration Register */
  MC_ME_PRTN1_CORE3_PUPD_tag PRTN1_CORE3_PUPD; /* Partition 1 Core 3 Process Update Register */
  MC_ME_PRTN1_CORE3_STAT_tag PRTN1_CORE3_STAT; /* Partition 1 Core 3 Status Register */
  MC_ME_PRTN1_CORE3_ADDR_tag PRTN1_CORE3_ADDR; /* Partition 1 Core 3 Address Register */
  uint8_t MC_ME_reserved14[336];
  MC_ME_PRTN2_PCONF_tag PRTN2_PCONF;   /* Partition 2 Process Configuration Register */
  MC_ME_PRTN2_PUPD_tag PRTN2_PUPD;     /* Partition 2 Process Update Register */
  MC_ME_PRTN2_STAT_tag PRTN2_STAT;     /* Partition 2 Status Register */
  uint8_t MC_ME_reserved15[52];
  MC_ME_PRTN2_CORE0_PCONF_tag PRTN2_CORE0_PCONF; /* Partition 2 Core 0 Process Configuration Register */
  MC_ME_PRTN2_CORE0_PUPD_tag PRTN2_CORE0_PUPD; /* Partition 2 Core 0 Process Update Register */
  MC_ME_PRTN2_CORE0_STAT_tag PRTN2_CORE0_STAT; /* Partition 2 Core 0 Status Register */
  MC_ME_PRTN2_CORE0_ADDR_tag PRTN2_CORE0_ADDR; /* Partition 2 Core 0 Address Register */
  uint8_t MC_ME_reserved16[16];
  MC_ME_PRTN2_CORE1_PCONF_tag PRTN2_CORE1_PCONF; /* Partition 2 Core 1 Process Configuration Register */
  MC_ME_PRTN2_CORE1_PUPD_tag PRTN2_CORE1_PUPD; /* Partition 2 Core 1 Process Update Register */
  MC_ME_PRTN2_CORE1_STAT_tag PRTN2_CORE1_STAT; /* Partition 2 Core 1 Status Register */
  MC_ME_PRTN2_CORE1_ADDR_tag PRTN2_CORE1_ADDR; /* Partition 2 Core 1 Address Register */
  uint8_t MC_ME_reserved17[16];
  MC_ME_PRTN2_CORE2_PCONF_tag PRTN2_CORE2_PCONF; /* Partition 2 Core 2 Process Configuration Register */
  MC_ME_PRTN2_CORE2_PUPD_tag PRTN2_CORE2_PUPD; /* Partition 2 Core 2 Process Update Register */
  MC_ME_PRTN2_CORE2_STAT_tag PRTN2_CORE2_STAT; /* Partition 2 Core 2 Status Register */
  MC_ME_PRTN2_CORE2_ADDR_tag PRTN2_CORE2_ADDR; /* Partition 2 Core 2 Address Register */
  uint8_t MC_ME_reserved18[16];
  MC_ME_PRTN2_CORE3_PCONF_tag PRTN2_CORE3_PCONF; /* Partition 2 Core 3 Process Configuration Register */
  MC_ME_PRTN2_CORE3_PUPD_tag PRTN2_CORE3_PUPD; /* Partition 2 Core 3 Process Update Register */
  MC_ME_PRTN2_CORE3_STAT_tag PRTN2_CORE3_STAT; /* Partition 2 Core 3 Status Register */
  MC_ME_PRTN2_CORE3_ADDR_tag PRTN2_CORE3_ADDR; /* Partition 2 Core 3 Address Register */
};


/* ============================================================================
   =============================== Module: MC_RGM =============================
   ============================================================================ */

typedef union MC_RGM_DES_union_tag {   /* Destructive Event Status Register */
  vuint32_t R;
  struct {
    vuint32_t F_POR:1;                 /* Flag for power-on reset */
    vuint32_t F_DR_1:1;                /* Flag for 'Destructive' Reset F_DR_1 */
    vuint32_t _unused_2:1;
    vuint32_t F_DR_3:1;                /* Flag for 'Destructive' Reset F_DR_3 */
    vuint32_t F_DR_4:1;                /* Flag for 'Destructive' Reset F_DR_4 */
    vuint32_t _unused_5:1;
    vuint32_t F_DR_6:1;                /* Flag for 'Destructive' Reset F_DR_6 */
    vuint32_t _unused_7:1;
    vuint32_t F_DR_8:1;                /* Flag for 'Destructive' Reset F_DR_8 */
    vuint32_t F_DR_9:1;                /* Flag for 'Destructive' Reset F_DR_9 */
    vuint32_t F_DR_10:1;               /* Flag for 'Destructive' Reset F_DR_10 */
    vuint32_t F_DR_11:1;               /* Flag for 'Destructive' Reset F_DR_11 */
    vuint32_t F_DR_12:1;               /* Flag for 'Destructive' Reset F_DR_12 */
    vuint32_t F_DR_13:1;               /* Flag for 'Destructive' Reset F_DR_13 */
    vuint32_t F_DR_14:1;               /* Flag for 'Destructive' Reset F_DR_14 */
    vuint32_t F_DR_15:1;               /* Flag for 'Destructive' Reset F_DR_15 */
    vuint32_t F_DR_16:1;               /* Flag for 'Destructive' Reset F_DR_16 */
    vuint32_t F_DR_17:1;               /* Flag for 'Destructive' Reset F_DR_17 */
    vuint32_t F_DR_18:1;               /* Flag for 'Destructive' Reset F_DR_18 */
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t F_DR_30:1;               /* Flag for 'Destructive' Reset F_DR_30 */
    vuint32_t F_DR_31:1;               /* Flag for 'Destructive' Reset F_DR_31 */
  } B;
} MC_RGM_DES_tag;

typedef union MC_RGM_FES_union_tag {   /* Functional /External Reset Status Register */
  vuint32_t R;
  struct {
    vuint32_t F_EXR:1;                 /* Flag for External Reset */
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t F_FR_3:1;                /* Flag for 'Functional' Reset F_FR_3 */
    vuint32_t F_FR_4:1;                /* Flag for 'Functional' Reset F_FR_4 */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t F_FR_20:1;               /* Flag for 'Functional' Reset F_FR_20 */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t F_FR_30:1;               /* Flag for 'Functional' Reset F_FR_30 */
    vuint32_t F_FR_31:1;               /* Flag for 'Functional' Reset F_FR_31 */
  } B;
} MC_RGM_FES_tag;

typedef union MC_RGM_FERD_union_tag {  /* Functional Event Reset Disable Register */
  vuint32_t R;
  struct {
    vuint32_t D_EXR:1;                 /* External Reset event demote to interrupt */
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t D_F_FR_3:1;              /* F_FR_3 Disable Control */
    vuint32_t D_F_FR_4:1;              /* F_FR_4 Disable Control */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t D_F_FR_20:1;             /* F_FR_20 Disable Control */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t D_F_FR_31:1;             /* F_FR_31 Disable Control */
  } B;
} MC_RGM_FERD_tag;

typedef union MC_RGM_FBRE_union_tag {  /* Functional Bidirectional Reset Enable Register */
  vuint32_t R;
  struct {
    vuint32_t BE_EXR:1;                /* Bidirectional Reset Enable for External Reset */
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t BE_F_FR_3:1;             /* Bidirectional Reset Enables for 'Functional' Reset F_FR_3 */
    vuint32_t BE_F_FR_4:1;             /* Bidirectional Reset Enables for 'Functional' Reset F_FR_4 */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t BE_F_FR_20:1;            /* Bidirectional Reset Enables for 'Functional' Reset F_FR_20 */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t BE_F_FR_30:1;            /* Bidirectional Reset Enables for 'Functional' Reset F_FR_30 */
    vuint32_t BE_F_FR_31:1;            /* Bidirectional Reset Enables for 'Functional' Reset F_FR_31 */
  } B;
} MC_RGM_FBRE_tag;

typedef union MC_RGM_FREC_union_tag {  /* Functional Reset Escalation Counter Register */
  vuint32_t R;
  struct {
    vuint32_t FREC:4;                  /* Functional' Reset Escalation Counter */
    vuint32_t _unused_4:28;
  } B;
} MC_RGM_FREC_tag;

typedef union MC_RGM_FRET_union_tag {  /* Functional Reset Escalation Threshold Register */
  vuint32_t R;
  struct {
    vuint32_t FRET:4;                  /* 'Functional' Reset Escalation Threshold */
    vuint32_t _unused_4:28;
  } B;
} MC_RGM_FRET_tag;

typedef union MC_RGM_DRET_union_tag {  /* Destructive Reset Escalation Threshold Register */
  vuint32_t R;
  struct {
    vuint32_t DRET:4;                  /* 'Destructive' Reset Escalation Threshold */
    vuint32_t _unused_4:28;
  } B;
} MC_RGM_DRET_tag;

typedef union MC_RGM_ERCTRL_union_tag { /* External Reset Control Register */
  vuint32_t R;
  struct {
    vuint32_t ERASSERT:1;              /* ERASSERT */
    vuint32_t _unused_1:31;
  } B;
} MC_RGM_ERCTRL_tag;

typedef union MC_RGM_FRENTC_union_tag { /* Functional Reset Entry Timeout Control Register */
  vuint32_t R;
  struct {
    vuint32_t FRET_EN:1;               /* Functional Reset Entry Timer Enable/Disable */
    vuint32_t FRET_TIMEOUT:31;         /* Functional Reset Entry Timer Value */
  } B;
} MC_RGM_FRENTC_tag;

typedef union MC_RGM_PRST0_0_union_tag { /* Peripheral Reset */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t PERIPH_3_RST:1;          /* Reset Control value for peripheral PERIPH_3_RST */
    vuint32_t PERIPH_4_RST:1;          /* Reset Control value for peripheral PERIPH_4_RST */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t PERIPH_8_RST:1;          /* Reset Control value for peripheral PERIPH_8_RST */
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t PERIPH_16_RST:1;         /* Reset Control value for peripheral PERIPH_16_RST */
    vuint32_t PERIPH_17_RST:1;         /* Reset Control value for peripheral PERIPH_17_RST */
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t PERIPH_24_RST:1;         /* Reset Control value for peripheral PERIPH_24_RST */
    vuint32_t PERIPH_25_RST:1;         /* Reset Control value for peripheral PERIPH_25_RST */
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} MC_RGM_PRST0_0_tag;

typedef union MC_RGM_PRST1_0_union_tag { /* Peripheral Reset */
  vuint32_t R;
  struct {
    vuint32_t PERIPH_64_RST:1;         /* Reset Control value for peripheral PERIPH_64_RST */
    vuint32_t PERIPH_65_RST:1;         /* Reset Control value for peripheral PERIPH_65_RST */
    vuint32_t PERIPH_66_RST:1;         /* Reset Control value for peripheral PERIPH_66_RST */
    vuint32_t PERIPH_67_RST:1;         /* Reset Control value for peripheral PERIPH_67_RST */
    vuint32_t PERIPH_68_RST:1;         /* Reset Control value for peripheral PERIPH_68_RST */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} MC_RGM_PRST1_0_tag;

typedef union MC_RGM_PRST2_0_union_tag { /* Peripheral Reset */
  vuint32_t R;
  struct {
    vuint32_t PERIPH_128_RST:1;        /* Reset Control value for peripheral PERIPH_128_RST */
    vuint32_t PERIPH_129_RST:1;        /* Reset Control value for peripheral PERIPH_129_RST */
    vuint32_t PERIPH_130_RST:1;        /* Reset Control value for peripheral PERIPH_130_RST */
    vuint32_t PERIPH_131_RST:1;        /* Reset Control value for peripheral PERIPH_131_RST */
    vuint32_t PERIPH_132_RST:1;        /* Reset Control value for peripheral PERIPH_132_RST */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} MC_RGM_PRST2_0_tag;

typedef union MC_RGM_PSTAT0_0_union_tag { /* Peripheral Reset Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t PERIPH_3_STAT:1;         /* Reset Status for peripheral PERIPH_3_STAT */
    vuint32_t PERIPH_4_STAT:1;         /* Reset Status for peripheral PERIPH_4_STAT */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t PERIPH_8_STAT:1;         /* Reset Status for peripheral PERIPH_8_STAT */
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t PERIPH_16_STAT:1;        /* Reset Status for peripheral PERIPH_16_STAT */
    vuint32_t PERIPH_17_STAT:1;        /* Reset Status for peripheral PERIPH_17_STAT */
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t PERIPH_24_STAT:1;        /* Reset Status for peripheral PERIPH_24_STAT */
    vuint32_t PERIPH_25_STAT:1;        /* Reset Status for peripheral PERIPH_25_STAT */
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} MC_RGM_PSTAT0_0_tag;

typedef union MC_RGM_PSTAT1_0_union_tag { /* Peripheral Reset Status Register */
  vuint32_t R;
  struct {
    vuint32_t PERIPH_64_STAT:1;        /* Reset Status for peripheral PERIPH_64_STAT */
    vuint32_t PERIPH_65_STAT:1;        /* Reset Status for peripheral PERIPH_65_STAT */
    vuint32_t PERIPH_66_STAT:1;        /* Reset Status for peripheral PERIPH_66_STAT */
    vuint32_t PERIPH_67_STAT:1;        /* Reset Status for peripheral PERIPH_67_STAT */
    vuint32_t PERIPH_68_STAT:1;        /* Reset Status for peripheral PERIPH_68_STAT */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} MC_RGM_PSTAT1_0_tag;

typedef union MC_RGM_PSTAT2_0_union_tag { /* Peripheral Reset Status Register */
  vuint32_t R;
  struct {
    vuint32_t PERIPH_128_STAT:1;       /* Reset Status for peripheral PERIPH_128_STAT */
    vuint32_t PERIPH_129_STAT:1;       /* Reset Status for peripheral PERIPH_129_STAT */
    vuint32_t PERIPH_130_STAT:1;       /* Reset Status for peripheral PERIPH_130_STAT */
    vuint32_t PERIPH_131_STAT:1;       /* Reset Status for peripheral PERIPH_131_STAT */
    vuint32_t PERIPH_132_STAT:1;       /* Reset Status for peripheral PERIPH_132_STAT */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} MC_RGM_PSTAT2_0_tag;

struct MC_RGM_tag {
  MC_RGM_DES_tag DES;                  /* Destructive Event Status Register */
  uint8_t MC_RGM_reserved0[4];
  MC_RGM_FES_tag FES;                  /* Functional /External Reset Status Register */
  MC_RGM_FERD_tag FERD;                /* Functional Event Reset Disable Register */
  MC_RGM_FBRE_tag FBRE;                /* Functional Bidirectional Reset Enable Register */
  MC_RGM_FREC_tag FREC;                /* Functional Reset Escalation Counter Register */
  MC_RGM_FRET_tag FRET;                /* Functional Reset Escalation Threshold Register */
  MC_RGM_DRET_tag DRET;                /* Destructive Reset Escalation Threshold Register */
  MC_RGM_ERCTRL_tag ERCTRL;            /* External Reset Control Register */
  uint8_t MC_RGM_reserved1[4];
  MC_RGM_FRENTC_tag FRENTC;            /* Functional Reset Entry Timeout Control Register */
  uint8_t MC_RGM_reserved2[20];
  MC_RGM_PRST0_0_tag PRST0_0;          /* Peripheral Reset */
  uint8_t MC_RGM_reserved3[4];
  MC_RGM_PRST1_0_tag PRST1_0;          /* Peripheral Reset */
  uint8_t MC_RGM_reserved4[4];
  MC_RGM_PRST2_0_tag PRST2_0;          /* Peripheral Reset */
  uint8_t MC_RGM_reserved5[236];
  MC_RGM_PSTAT0_0_tag PSTAT0_0;        /* Peripheral Reset Status Register */
  uint8_t MC_RGM_reserved6[4];
  MC_RGM_PSTAT1_0_tag PSTAT1_0;        /* Peripheral Reset Status Register */
  uint8_t MC_RGM_reserved7[4];
  MC_RGM_PSTAT2_0_tag PSTAT2_0;        /* Peripheral Reset Status Register */
};


/* ============================================================================
   =============================== Module: MDM_AP =============================
   ============================================================================ */

typedef union MDM_AP_STATUS_union_tag { /* MDM_AP status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t SYSTEM_RESET:1;
    vuint32_t _unused_3:3;             /* Reserved */
    vuint32_t CSYSPWRUPREQ:1;          /* Status of CSYSPWRUPREQ from SWJ-DP */
    vuint32_t CDBGPWRUPREQ:1;          /* Status of CDBGPWRUPREQ from SWJ-DP */
    vuint32_t JTAG_ACTIVE:1;
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:2;            /* Reserved */
    vuint32_t SMU_HALTED:1;            /* SMU Core is halted */
    vuint32_t CE_A_HALTED:1;           /* Communication Engine Cortex-M33 core 0 is halted */
    vuint32_t CE_B_HALTED:1;           /* Communication Engine Cortex-M33 core 1 is halted */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SMU_SLEEPDEEP:1;         /* SMU SLEEPDEEP */
    vuint32_t CE_A_SLEEPDEEP:1;        /* Communication Engine Cortex-M33 Core 0 SLEEPDEEP */
    vuint32_t CE_B_SLEEPDEEP:1;        /* Communication Engine Cortex-M33 Core 1 SLEEPDEEP */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t SMU_SLEEPING:1;          /* SMU Sleeping */
    vuint32_t CE_A_SLEEPING:1;         /* Communication Engine Cortex-M33 Core 0 sleeping */
    vuint32_t CE_B_SLEEPING:1;         /* Communication Engine Cortex-M33 Core 1 sleeping */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t SMU_DBGRSTRTS:1;         /* Handshake Signal for SMU_DBGRSTRT */
    vuint32_t CE_A_DBGRSTRTS:1;        /* Handshake Signal for CE_A_DBGRSTRT */
    vuint32_t CE_B_DBGRSTRTS:1;        /* Handshake Signal for CE_B_DBGRSTRT */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} MDM_AP_STATUS_tag;

typedef union MDM_AP_CONTROL_union_tag { /* MDM_AP control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t ETR_HANDSHAKE_0:1;       /* ETR Handshake_0 */
    vuint32_t ETR_HANDSHAKE_1:1;       /* ETR Handshake_1 */
    vuint32_t SMU_EDBGREQ:1;           /* EDBGREQ Input for SMU Cortex-M33 Core */
    vuint32_t COMMS_0_EDBGREQ:1;       /* EDBGREQ Input for CE Cortex-M33 Core 0 */
    vuint32_t COMMS_1_EDBGREQ:1;       /* EDBGREQ Input for CE Cortex-M33 Core 1 */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:3;            /* Reserved */
    vuint32_t DIS_POR_WDOG_MSK:1;      /* Mask Power on watchdog timer */
    vuint32_t _unused_16:4;            /* Reserved */
    vuint32_t TPIU_OVERRIDE:1;         /* TPIU Override */
    vuint32_t ETR_OVERRIDE_SMU:1;      /* ETR in SMU subsystem Override */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t MSK_DBG_FAULT:1;         /* Mask Debug Faults */
    vuint32_t MSK_FCCU_RST_TRIGGER:1;  /* Mask Reset Trigger from FCCU */
    vuint32_t _unused_25:3;            /* Reserved */
    vuint32_t SMU_DBGRSTRT:1;          /* Debug Restart Input to SMU Cortex-M33 Core */
    vuint32_t CE_A_DBGRSTRT:1;         /* Debug Restart Input to CE Cortex-M33 Core 0 */
    vuint32_t CE_B_DBGRSTRT:1;         /* Debug Restart Input to CE Cortex-M33 Core 1 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} MDM_AP_CONTROL_tag;

typedef union MDM_AP_STATUS2_union_tag { /* MDM_AP status 2 */
  vuint32_t R;
  struct {
    vuint32_t CR52_RTU0_0_DBGMODE:1;
    vuint32_t CR52_RTU0_1_DBGMODE:1;
    vuint32_t CR52_RTU0_2_DBGMODE:1;
    vuint32_t CR52_RTU0_3_DBGMODE:1;
    vuint32_t CR52_RTU0_0_WFI:1;
    vuint32_t CR52_RTU0_1_WFI:1;
    vuint32_t CR52_RTU0_2_WFI:1;
    vuint32_t CR52_RTU0_3_WFI:1;
    vuint32_t CR52_RTU0_0_WFE:1;
    vuint32_t CR52_RTU0_1_WFE:1;
    vuint32_t CR52_RTU0_2_WFE:1;
    vuint32_t CR52_RTU0_3_WFE:1;
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t CR52_RTU1_0_DBGMODE:1;
    vuint32_t CR52_RTU1_1_DBGMODE:1;
    vuint32_t CR52_RTU1_2_DBGMODE:1;
    vuint32_t CR52_RTU1_3_DBGMODE:1;
    vuint32_t CR52_RTU1_0_WFI:1;
    vuint32_t CR52_RTU1_1_WFI:1;
    vuint32_t CR52_RTU1_2_WFI:1;
    vuint32_t CR52_RTU1_3_WFI:1;
    vuint32_t CR52_RTU1_0_WFE:1;
    vuint32_t CR52_RTU1_1_WFE:1;
    vuint32_t CR52_RTU1_2_WFE:1;
    vuint32_t CR52_RTU1_3_WFE:1;
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} MDM_AP_STATUS2_tag;

typedef union MDM_AP_CONTROL2_union_tag { /* MDM_AP Control 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t RTU0_ETR_HANDSHAKE_0:1;  /* ETR Handshake_0 */
    vuint32_t RTU0_ETR_HANDSHAKE_1:1;  /* ETR Handshake_1 */
    vuint32_t _unused_8:7;             /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t CR52_RTU0_0_EDBGREQ:1;   /* EDBGREQ Input for RTU0 core 0 */
    vuint32_t CR52_RTU0_1_EDBGREQ:1;   /* EDBGREQ Input for RTU0 core 1 */
    vuint32_t CR52_RTU0_2_EDBGREQ:1;   /* EDBGREQ Input for RTU0 core 2 */
    vuint32_t CR52_RTU0_3_EDBGREQ:1;   /* EDBGREQ Input for RTU0 core 3 */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} MDM_AP_CONTROL2_tag;

typedef union MDM_AP_CONTROL3_union_tag { /* MDM_AP Control 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t RTU1_ETR_HANDSHAKE_0:1;  /* ETR Handshake_0 */
    vuint32_t RTU1_ETR_HANDSHAKE_1:1;  /* ETR Handshake_1 */
    vuint32_t _unused_8:7;             /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t CR52_RTU1_0_EDBGREQ:1;   /* EDBGREQ Input for RTU1 core 0 */
    vuint32_t CR52_RTU1_1_EDBGREQ:1;   /* EDBGREQ Input for RTU1 core 1 */
    vuint32_t CR52_RTU1_2_EDBGREQ:1;   /* EDBGREQ Input for RTU1 core 2 */
    vuint32_t CR52_RTU1_3_EDBGREQ:1;   /* EDBGREQ Input for RTU1 core 3 */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} MDM_AP_CONTROL3_tag;

typedef union MDM_AP_RTU0_CORE_0_IP_DBG_union_tag { /* RTU0 core 0 debug */
  vuint32_t R;
  struct {
    vuint32_t RTU0_0_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_RTU0_CORE_0_IP_DBG_tag;

typedef union MDM_AP_RTU0_CORE_1_IP_DBG_union_tag { /* RTU0 core 1 debug */
  vuint32_t R;
  struct {
    vuint32_t RTU0_1_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_RTU0_CORE_1_IP_DBG_tag;

typedef union MDM_AP_RTU0_CORE_2_IP_DBG_union_tag { /* RTU0 core 2 debug */
  vuint32_t R;
  struct {
    vuint32_t RTU0_2_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_RTU0_CORE_2_IP_DBG_tag;

typedef union MDM_AP_RTU0_CORE_3_IP_DBG_union_tag { /* RTU0 core 3 debug */
  vuint32_t R;
  struct {
    vuint32_t RTU0_3_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_RTU0_CORE_3_IP_DBG_tag;

typedef union MDM_AP_RTU1_CORE_0_IP_DBG_union_tag { /* RTU1 core 0 debug */
  vuint32_t R;
  struct {
    vuint32_t RTU1_0_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_RTU1_CORE_0_IP_DBG_tag;

typedef union MDM_AP_RTU1_CORE_1_IP_DBG_union_tag { /* RTU1 core 1 debug */
  vuint32_t R;
  struct {
    vuint32_t RTU1_1_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_RTU1_CORE_1_IP_DBG_tag;

typedef union MDM_AP_RTU1_CORE_2_IP_DBG_union_tag { /* RTU1 core 2 debug */
  vuint32_t R;
  struct {
    vuint32_t RTU1_2_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_RTU1_CORE_2_IP_DBG_tag;

typedef union MDM_AP_RTU1_CORE_3_IP_DBG_union_tag { /* RTU1 core 3 debug */
  vuint32_t R;
  struct {
    vuint32_t RTU1_3_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_RTU1_CORE_3_IP_DBG_tag;

typedef union MDM_AP_SMU_CORTEX_M33_IP_DBG_union_tag { /* Cortex-M33 Core Debug */
  vuint32_t R;
  struct {
    vuint32_t CM33_SMU_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_SMU_CORTEX_M33_IP_DBG_tag;

typedef union MDM_AP_CE_CORTEX_M33_0_IP_DBG_union_tag { /* CE Cortex-M33 Core 0 Debug */
  vuint32_t R;
  struct {
    vuint32_t CE_CM33_0_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_CE_CORTEX_M33_0_IP_DBG_tag;

typedef union MDM_AP_CE_CORTEX_M33_1_IP_DBG_union_tag { /* CE Cortex-M33 Core 1 Debug */
  vuint32_t R;
  struct {
    vuint32_t CE_CM33_1_IP_DBGDIS:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MDM_AP_CE_CORTEX_M33_1_IP_DBG_tag;

typedef union MDM_AP_RTU0_SWTMASK_IP_DBG_union_tag { /* RTU0 SWT debug mask register */
  vuint32_t R;
  struct {
    vuint32_t SWT0_MASK_DBG:1;
    vuint32_t SWT1_MASK_DBG:1;
    vuint32_t SWT2_MASK_DBG:1;
    vuint32_t SWT3_MASK_DBG:1;
    vuint32_t SWT4_MASK_DBG:1;
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} MDM_AP_RTU0_SWTMASK_IP_DBG_tag;

typedef union MDM_AP_RTU1_SWTMASK_IP_DBG_union_tag { /* RTU1 SWT debug mask register */
  vuint32_t R;
  struct {
    vuint32_t SWT0_MASK_DBG:1;
    vuint32_t SWT1_MASK_DBG:1;
    vuint32_t SWT2_MASK_DBG:1;
    vuint32_t SWT3_MASK_DBG:1;
    vuint32_t SWT4_MASK_DBG:1;
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} MDM_AP_RTU1_SWTMASK_IP_DBG_tag;

typedef union MDM_AP_RTU0_STMMASK_IP_DBG_union_tag { /* RTU0 STM debug mask register */
  vuint32_t R;
  struct {
    vuint32_t STM0_MASK_DBG:1;
    vuint32_t STM1_MASK_DBG:1;
    vuint32_t STM2_MASK_DBG:1;
    vuint32_t STM3_MASK_DBG:1;
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} MDM_AP_RTU0_STMMASK_IP_DBG_tag;

typedef union MDM_AP_RTU1_STMMASK_IP_DBG_union_tag { /* RTU1 STM debug mask register */
  vuint32_t R;
  struct {
    vuint32_t STM0_MASK_DBG:1;
    vuint32_t STM1_MASK_DBG:1;
    vuint32_t STM2_MASK_DBG:1;
    vuint32_t STM3_MASK_DBG:1;
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} MDM_AP_RTU1_STMMASK_IP_DBG_tag;

typedef union MDM_AP_DBGENCNTRL_union_tag { /* Debug enable control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t RTU0_HIDEN3:1;           /* Hypervisor Core Debug (invasive debug) */
    vuint32_t RTU0_HNIDEN3:1;          /* Hypervisor Core Debug (non-invasive debug) */
    vuint32_t RTU0_DBGEN3:1;           /* Core Debug (invasive debug) */
    vuint32_t RTU0_NIDEN3:1;           /* Core Debug (non-invasive debug) */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t RTU0_HIDEN2:1;           /* Hypervisor Core Debug (invasive debug) */
    vuint32_t RTU0_HNIDEN2:1;          /* Hypervisor Core Debug (non-invasive debug) */
    vuint32_t RTU0_DBGEN2:1;           /* Core Debug (invasive debug) */
    vuint32_t RTU0_NIDEN2:1;           /* Core Debug (non-invasive debug) */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t RTU0_HIDEN1:1;           /* Hypervisor Core Debug (invasive debug) */
    vuint32_t RTU0_HNIDEN1:1;          /* Hypervisor Core Debug (non-invasive debug) */
    vuint32_t RTU0_DBGEN1:1;           /* Core Debug (invasive debug) */
    vuint32_t RTU0_NIDEN1:1;           /* Core Debug (non-invasive debug) */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t RTU0_HIDEN0:1;           /* Hypervisor Core Debug (invasive debug) */
    vuint32_t RTU0_HNIDEN0:1;          /* Hypervisor Core Debug (non-invasive debug) */
    vuint32_t RTU0_DBGEN0:1;           /* Core Debug (invasive debug) */
    vuint32_t RTU0_NIDEN0:1;           /* Core Debug (non-invasive debug) */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} MDM_AP_DBGENCNTRL_tag;

typedef union MDM_AP_DBGENCNTRL2_union_tag { /* Debug enable control 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t RTU1_HIDEN3:1;           /* Hypervisor Core Debug (invasive debug) */
    vuint32_t RTU1_HNIDEN3:1;          /* Hypervisor Core Debug (non-invasive debug) */
    vuint32_t RTU1_DBGEN3:1;           /* Core Debug (invasive debug) */
    vuint32_t RTU1_NIDEN3:1;           /* Core Debug (non-invasive debug) */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t RTU1_HIDEN2:1;           /* Hypervisor Core Debug (invasive debug) */
    vuint32_t RTU1_HNIDEN2:1;          /* Hypervisor Core Debug (non-invasive debug) */
    vuint32_t RTU1_DBGEN2:1;           /* Core Debug (invasive debug) */
    vuint32_t RTU1_NIDEN2:1;           /* Core Debug (non-invasive debug) */
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t RTU1_HIDEN1:1;           /* Hypervisor Core Debug (invasive debug) */
    vuint32_t RTU1_HNIDEN1:1;          /* Hypervisor Core Debug (non-invasive debug) */
    vuint32_t RTU1_DBGEN1:1;           /* Core Debug (invasive debug) */
    vuint32_t RTU1_NIDEN1:1;           /* Core Debug (non-invasive debug) */
    vuint32_t _unused_24:2;            /* Reserved */
    vuint32_t RTU1_HIDEN0:1;           /* Hypervisor Core Debug (invasive debug) */
    vuint32_t RTU1_HNIDEN0:1;          /* Hypervisor Core Debug (non-invasive debug) */
    vuint32_t RTU1_DBGEN0:1;           /* Core Debug (invasive debug) */
    vuint32_t RTU1_NIDEN0:1;           /* Core Debug (non-invasive debug) */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} MDM_AP_DBGENCNTRL2_tag;

typedef union MDM_AP_DBGENCNTRL3_union_tag { /* Debug enable control 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;             /* Reserved */
    vuint32_t GDBGEN:1;                /* Global Invasive debug enable */
    vuint32_t GNIDEN:1;                /* Global Noninvasive debug enable */
    vuint32_t GSPIDEN:1;               /* Global Secure invasive debug enable */
    vuint32_t GSPNIDEN:1;              /* Global Secure noninvasive debug enable */
    vuint32_t _unused_8:2;             /* Reserved */
    vuint32_t CEVA_SPF2_DBGEN:1;       /* Ceva-SensPro Float 2 (CEVA_PF2) Invasive debug enable */
    vuint32_t CEVA_SPF2_NIDEN:1;       /* Ceva-SensPro Float 2 (CEVA_PF2) Noninvasive debug enable */
    vuint32_t CEVA_SPF2_SPIDEN:1;      /* >Ceva-SensPro Float 2 (CEVA_PF2) Secure invasive debug enable */
    vuint32_t CEVA_SPF2_SPNIDEN:1;     /* >Ceva-SensPro Float 2 (CEVA_PF2) Secure noninvasive debug enable */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t GTM_DBGEN:1;             /* GTM debug enable */
    vuint32_t _unused_16:4;            /* Reserved */
    vuint32_t CE_A_DBGEN:1;            /* CE Cortex-M33 core 0 Invasive debug enable */
    vuint32_t CE_A_NIDEN:1;            /* CE Cortex-M33 core 0 Noninvasive debug enable */
    vuint32_t CE_A_SPIDEN:1;           /* CE Cortex-M33 core 0 Secure invasive debug enable */
    vuint32_t CE_A_SPNIDEN:1;          /* CE Cortex-M33 core 0 Secure noninvasive debug enable */
    vuint32_t CE_B_DBGEN:1;            /* CE Cortex-M33 core 1 Invasive debug enable */
    vuint32_t CE_B_NIDEN:1;            /* CE Cortex-M33 core 1 Noninvasive debug enable */
    vuint32_t CE_B_SPIDEN:1;           /* CE Cortex-M33 core 1 Secure invasive debug enable */
    vuint32_t CE_B_SPNIDEN:1;          /* CE Cortex-M33 core 1 Secure noninvasive debug enable */
    vuint32_t SYSM_DBGEN:1;            /* System Manager Invasive debug enable */
    vuint32_t SYSM_NIDEN:1;            /* System Manager Noninvasive debug enable */
    vuint32_t SYSM_SPIDEN:1;           /* System Manager Secure invasive debug enable */
    vuint32_t SYSM_SPNIDEN:1;          /* System Manager Secure noninvasive debug enable */
  } B;
} MDM_AP_DBGENCNTRL3_tag;

typedef union MDM_AP_DTS_ENABLE_union_tag { /* DTS output enable register */
  vuint32_t R;
  struct {
    vuint32_t DTS_EN:1;                /* DTS Enable */
    vuint32_t DTS_EN_B:1;              /* DTS Enable B */
    vuint32_t DTS_EN_C:1;              /* DTS Enable C */
    vuint32_t DTS_EN_D:1;              /* DTS Enable D */
    vuint32_t _unused_4:28;
  } B;
} MDM_AP_DTS_ENABLE_tag;

typedef union MDM_AP_DTS_STARTUP_union_tag { /* DTS startup register */
  vuint32_t R;
  struct {
    vuint32_t AD:32;                   /* Application Dependent register bits. */
  } B;
} MDM_AP_DTS_STARTUP_tag;

typedef union MDM_AP_DTS_STARTUP_B_union_tag { /* DTS startup register B */
  vuint32_t R;
  struct {
    vuint32_t AD:32;                   /* Application Dependent register bits. */
  } B;
} MDM_AP_DTS_STARTUP_B_tag;

typedef union MDM_AP_DTS_STARTUP_C_union_tag { /* DTS startup register C */
  vuint32_t R;
  struct {
    vuint32_t AD:32;                   /* Application Dependent register bits. */
  } B;
} MDM_AP_DTS_STARTUP_C_tag;

typedef union MDM_AP_DTS_STARTUP_D_union_tag { /* DTS startup register D */
  vuint32_t R;
  struct {
    vuint32_t AD:32;                   /* Application Dependent register bits. */
  } B;
} MDM_AP_DTS_STARTUP_D_tag;

typedef union MDM_AP_DTS_SEMAPHORE_union_tag { /* DTS SEMAPHORE register */
  vuint32_t R;
  struct {
    vuint32_t ST:32;                   /* Semaphore Trigger */
  } B;
} MDM_AP_DTS_SEMAPHORE_tag;

typedef union MDM_AP_DTS_SEMAPHORE_B_union_tag { /* DTS SEMAPHORE_B register */
  vuint32_t R;
  struct {
    vuint32_t ST_B:32;                 /* Semaphore Trigger Extension */
  } B;
} MDM_AP_DTS_SEMAPHORE_B_tag;

typedef union MDM_AP_DTS_SEMAPHORE_C_union_tag { /* DTS SEMAPHORE_C register */
  vuint32_t R;
  struct {
    vuint32_t ST_C:32;                 /* Semaphore Trigger Extension */
  } B;
} MDM_AP_DTS_SEMAPHORE_C_tag;

typedef union MDM_AP_DTS_SEMAPHORE_D_union_tag { /* DTS SEMAPHORE_D register */
  vuint32_t R;
  struct {
    vuint32_t ST_D:32;                 /* Semaphore Trigger Extension */
  } B;
} MDM_AP_DTS_SEMAPHORE_D_tag;

typedef union MDM_AP_IDR_union_tag {   /* Identification Register */
  vuint32_t R;
  struct {
    vuint32_t TYPE:4;                  /* Indicates Type of component */
    vuint32_t VARIANT:4;               /* Variation from base type specified by IDR TYPE */
    vuint32_t _unused_8:5;
    vuint32_t CLASS:4;                 /* Class */
    vuint32_t JEDEC_CODE:7;            /* The JEP106 identification code */
    vuint32_t JEDEC_BANK:4;            /* The JEP106 continuation code */
    vuint32_t REVISION:4;              /* Revision */
  } B;
} MDM_AP_IDR_tag;

typedef union MDM_AP_DEVARCH_union_tag { /* CoreSight Device Architecture Register */
  vuint32_t R;
  struct {
    vuint32_t ARCHID:16;               /* Architecture ID */
    vuint32_t REVISION:4;              /* Architecture revision */
    vuint32_t PRESENT:1;               /* DEVARCH register is present */
    vuint32_t ARCHITECT:11;            /* Architect of the component - JEDEC code */
  } B;
} MDM_AP_DEVARCH_tag;

typedef union MDM_AP_DEVTYPE_union_tag { /* CoreSight Device Type Identifier Register */
  vuint32_t R;
  struct {
    vuint32_t MAJOR:4;                 /* Major Classification */
    vuint32_t SUB:4;                   /* Minor Classification */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_DEVTYPE_tag;

typedef union MDM_AP_PIDR4_union_tag { /* CoreSight Peripheral Identification Register 4 */
  vuint32_t R;
  struct {
    vuint32_t DES_2:4;                 /* DES_2 */
    vuint32_t SIZE:4;                  /* SIZE */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_PIDR4_tag;

typedef union MDM_AP_PIDR0_union_tag { /* CoreSight Peripheral Identification Register 0 */
  vuint32_t R;
  struct {
    vuint32_t PART_0:8;                /* PART_0 */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_PIDR0_tag;

typedef union MDM_AP_PIDR1_union_tag { /* CoreSight Peripheral Identification Register 1 */
  vuint32_t R;
  struct {
    vuint32_t PART_1:4;                /* PART_1 */
    vuint32_t DES_0:4;                 /* DES_0 */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_PIDR1_tag;

typedef union MDM_AP_PIDR2_union_tag { /* CoreSight Peripheral Identification Register 2 */
  vuint32_t R;
  struct {
    vuint32_t DES_1:3;                 /* DES_1 */
    vuint32_t JEDEC:1;                 /* JEDEC */
    vuint32_t REVISION:4;              /* Revision */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_PIDR2_tag;

typedef union MDM_AP_PIDR3_union_tag { /* CoreSight Peripheral Identification Register 3 */
  vuint32_t R;
  struct {
    vuint32_t CMOD:4;                  /* Customer Modified */
    vuint32_t REVAND:4;                /* REVAND */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_PIDR3_tag;

typedef union MDM_AP_CIDR0_union_tag { /* CoreSight Component Identification Register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_0:8;               /* Preamble */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_CIDR0_tag;

typedef union MDM_AP_CIDR1_union_tag { /* CoreSight Component Identification Register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_1:4;               /* PRMBL_1 */
    vuint32_t CLASS:4;                 /* Component Class */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_CIDR1_tag;

typedef union MDM_AP_CIDR2_union_tag { /* CoreSight Component Identification Register 2 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_2:8;               /* PRMBL_2 */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_CIDR2_tag;

typedef union MDM_AP_CIDR3_union_tag { /* CoreSight Component Identification Register 3 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_3:8;               /* PRMBL_3 */
    vuint32_t _unused_8:24;
  } B;
} MDM_AP_CIDR3_tag;

struct MDM_AP_tag {
  MDM_AP_STATUS_tag STATUS;            /* MDM_AP status */
  MDM_AP_CONTROL_tag CONTROL;          /* MDM_AP control */
  uint8_t MDM_AP_reserved0[56];
  MDM_AP_STATUS2_tag STATUS2;          /* MDM_AP status 2 */
  MDM_AP_CONTROL2_tag CONTROL2;        /* MDM_AP Control 2 */
  uint8_t MDM_AP_reserved1[4];
  MDM_AP_CONTROL3_tag CONTROL3;        /* MDM_AP Control 3 */
  uint8_t MDM_AP_reserved2[32];
  MDM_AP_RTU0_CORE_0_IP_DBG_tag RTU0_CORE_0_IP_DBG; /* RTU0 core 0 debug */
  MDM_AP_RTU0_CORE_1_IP_DBG_tag RTU0_CORE_1_IP_DBG; /* RTU0 core 1 debug */
  MDM_AP_RTU0_CORE_2_IP_DBG_tag RTU0_CORE_2_IP_DBG; /* RTU0 core 2 debug */
  MDM_AP_RTU0_CORE_3_IP_DBG_tag RTU0_CORE_3_IP_DBG; /* RTU0 core 3 debug */
  MDM_AP_RTU1_CORE_0_IP_DBG_tag RTU1_CORE_0_IP_DBG; /* RTU1 core 0 debug */
  MDM_AP_RTU1_CORE_1_IP_DBG_tag RTU1_CORE_1_IP_DBG; /* RTU1 core 1 debug */
  MDM_AP_RTU1_CORE_2_IP_DBG_tag RTU1_CORE_2_IP_DBG; /* RTU1 core 2 debug */
  MDM_AP_RTU1_CORE_3_IP_DBG_tag RTU1_CORE_3_IP_DBG; /* RTU1 core 3 debug */
  MDM_AP_SMU_CORTEX_M33_IP_DBG_tag SMU_CORTEX_M33_IP_DBG; /* Cortex-M33 Core Debug */
  MDM_AP_CE_CORTEX_M33_0_IP_DBG_tag CE_CORTEX_M33_0_IP_DBG; /* CE Cortex-M33 Core 0 Debug */
  MDM_AP_CE_CORTEX_M33_1_IP_DBG_tag CE_CORTEX_M33_1_IP_DBG; /* CE Cortex-M33 Core 1 Debug */
  uint8_t MDM_AP_reserved3[4];
  MDM_AP_RTU0_SWTMASK_IP_DBG_tag RTU0_SWTMASK_IP_DBG; /* RTU0 SWT debug mask register */
  MDM_AP_RTU1_SWTMASK_IP_DBG_tag RTU1_SWTMASK_IP_DBG; /* RTU1 SWT debug mask register */
  uint8_t MDM_AP_reserved4[8];
  MDM_AP_RTU0_STMMASK_IP_DBG_tag RTU0_STMMASK_IP_DBG; /* RTU0 STM debug mask register */
  MDM_AP_RTU1_STMMASK_IP_DBG_tag RTU1_STMMASK_IP_DBG; /* RTU1 STM debug mask register */
  uint8_t MDM_AP_reserved5[8];
  MDM_AP_DBGENCNTRL_tag DBGENCNTRL;    /* Debug enable control */
  uint8_t MDM_AP_reserved6[4];
  MDM_AP_DBGENCNTRL2_tag DBGENCNTRL2;  /* Debug enable control 2 */
  uint8_t MDM_AP_reserved7[4];
  MDM_AP_DBGENCNTRL3_tag DBGENCNTRL3;  /* Debug enable control 3 */
  uint8_t MDM_AP_reserved8[44];
  MDM_AP_DTS_ENABLE_tag DTS_ENABLE;    /* DTS output enable register */
  uint8_t MDM_AP_reserved9[12];
  MDM_AP_DTS_STARTUP_tag DTS_STARTUP;  /* DTS startup register */
  MDM_AP_DTS_STARTUP_B_tag DTS_STARTUP_B; /* DTS startup register B */
  MDM_AP_DTS_STARTUP_C_tag DTS_STARTUP_C; /* DTS startup register C */
  MDM_AP_DTS_STARTUP_D_tag DTS_STARTUP_D; /* DTS startup register D */
  MDM_AP_DTS_SEMAPHORE_tag DTS_SEMAPHORE; /* DTS SEMAPHORE register */
  uint8_t MDM_AP_reserved10[12];
  MDM_AP_DTS_SEMAPHORE_B_tag DTS_SEMAPHORE_B; /* DTS SEMAPHORE_B register */
  uint8_t MDM_AP_reserved11[12];
  MDM_AP_DTS_SEMAPHORE_C_tag DTS_SEMAPHORE_C; /* DTS SEMAPHORE_C register */
  uint8_t MDM_AP_reserved12[12];
  MDM_AP_DTS_SEMAPHORE_D_tag DTS_SEMAPHORE_D; /* DTS SEMAPHORE_D register */
  uint8_t MDM_AP_reserved13[3240];
  MDM_AP_IDR_tag IDR;                  /* Identification Register */
  uint8_t MDM_AP_reserved14[444];
  MDM_AP_DEVARCH_tag DEVARCH;          /* CoreSight Device Architecture Register */
  uint8_t MDM_AP_reserved15[12];
  MDM_AP_DEVTYPE_tag DEVTYPE;          /* CoreSight Device Type Identifier Register */
  MDM_AP_PIDR4_tag PIDR4;              /* CoreSight Peripheral Identification Register 4 */
  uint8_t MDM_AP_reserved16[12];
  MDM_AP_PIDR0_tag PIDR0;              /* CoreSight Peripheral Identification Register 0 */
  MDM_AP_PIDR1_tag PIDR1;              /* CoreSight Peripheral Identification Register 1 */
  MDM_AP_PIDR2_tag PIDR2;              /* CoreSight Peripheral Identification Register 2 */
  MDM_AP_PIDR3_tag PIDR3;              /* CoreSight Peripheral Identification Register 3 */
  MDM_AP_CIDR0_tag CIDR0;              /* CoreSight Component Identification Register 0 */
  MDM_AP_CIDR1_tag CIDR1;              /* CoreSight Component Identification Register 1 */
  MDM_AP_CIDR2_tag CIDR2;              /* CoreSight Component Identification Register 2 */
  MDM_AP_CIDR3_tag CIDR3;              /* CoreSight Component Identification Register 3 */
};


/* ============================================================================
   =============================== Module: MEW ================================
   ============================================================================ */

typedef union MEW_ECC_GLBL_CTRL_union_tag { /* AXI ECC Global Control */
  vuint32_t R;
  struct {
    vuint32_t RD_EN:1;                 /* RD_EN */
    vuint32_t _unused_1:2;             /* Reserved */
    vuint32_t RD_EDCEN:1;              /* RD_EDCEN */
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t WR_EN:1;                 /* WR_EN */
    vuint32_t _unused_17:2;            /* Reserved */
    vuint32_t WR_EDCEN:1;              /* WR_EDCEN */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} MEW_ECC_GLBL_CTRL_tag;

typedef union MEW_ECC_MX_EPA_union_tag { /* AXI ECC Maximum ECC Protected Address */
  vuint32_t R;
  struct {
    vuint32_t MX_EPA:32;               /* MX_EPA */
  } B;
} MEW_ECC_MX_EPA_tag;

typedef union MEW_ECC_MN_EPA_union_tag { /* AXI ECC Minimum ECC Protected Address */
  vuint32_t R;
  struct {
    vuint32_t MN_EPA:32;               /* MN_EPA */
  } B;
} MEW_ECC_MN_EPA_tag;

typedef union MEW_ECC_LK_PTN_union_tag { /* AXI ECC Lock Pattern */
  vuint32_t R;
  struct {
    vuint32_t LK_PTN:32;               /* LK_PTN */
  } B;
} MEW_ECC_LK_PTN_tag;

typedef union MEW_ECC_ULK_PTN_union_tag { /* AXI ECC Unlock Pattern */
  vuint32_t R;
  struct {
    vuint32_t ULK_PTN:32;              /* ULK_PTN */
  } B;
} MEW_ECC_ULK_PTN_tag;

typedef union MEW_ECC_EERAR_union_tag { /* ECC Error Report Address */
  vuint32_t R;
  struct {
    vuint32_t EERA:32;                 /* EERA */
  } B;
} MEW_ECC_EERAR_tag;

typedef union MEW_ECC_EERDSR_union_tag { /* AXI ECC Error Report Data And Syndrome */
  vuint32_t R;
  struct {
    vuint32_t EERD:8;                  /* EERD */
    vuint32_t EECC:8;                  /* EECC */
    vuint32_t EERS:8;                  /* EERS */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} MEW_ECC_EERDSR_tag;

typedef union MEW_ECC_ERR_IE_union_tag { /* AXI ECC Error Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t NCE_IE1:1;               /* Non-Corrected Error Logging And Interrupt Enable For ECC Module 1 */
    vuint32_t NCE_IE2:1;               /* Non-Corrected Error Logging And Interrupt Enable For ECC Module 2 */
    vuint32_t NCE_IE3:1;               /* Non-Corrected Error Logging And Interrupt Enable For ECC Module 3 */
    vuint32_t NCE_IE4:1;               /* Non-Corrected Error Logging And Interrupt Enable For ECC Module 4 */
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t CE_IE1:1;                /* Corrected Error Logging And Interrupt Enable For ECC Module 1 */
    vuint32_t CE_IE2:1;                /* Corrected Error Logging And Interrupt Enable For ECC Module 2 */
    vuint32_t CE_IE3:1;                /* Corrected Error Logging And Interrupt Enable For ECC Module 3 */
    vuint32_t CE_IE4:1;                /* Corrected Error Logging And Interrupt Enable For ECC Module 4 */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} MEW_ECC_ERR_IE_tag;

typedef union MEW_ECC_ERR_IN_STCLR_union_tag { /* AXI ECC Error Interupt Status And Clear */
  vuint32_t R;
  struct {
    vuint32_t NCE_IF1:1;               /* Non-Corrected Error Interrupt Flag For ECC Module 1 */
    vuint32_t NCE_IF2:1;               /* Non-Corrected Error Interrupt Flag For ECC Module 2 */
    vuint32_t NCE_IF3:1;               /* Non-Corrected Error Interrupt Flag For ECC Module 3 */
    vuint32_t NCE_IF4:1;               /* Non-Corrected Error Interrupt Flag For ECC Module 4 */
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t CE_IF1:1;                /* Corrected Error Interrupt Flag For ECC Module 1 */
    vuint32_t CE_IF2:1;                /* Corrected Error Interrupt Flag For ECC module 2 */
    vuint32_t CE_IF3:1;                /* Corrected Error Interrupt Flag For ECC module 3 */
    vuint32_t CE_IF4:1;                /* Corrected Error Interrupt Flag For ECC module 4 */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} MEW_ECC_ERR_IN_STCLR_tag;

typedef union MEW_EDC_ERR_IE_union_tag { /* AXI EDC Error Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t EDC_EIE1:1;              /* EDC Encoder Error Interrupt Enable */
    vuint32_t EDC_EIE2:1;              /* EDC Encoder Error Interrupt Enable */
    vuint32_t EDC_EIE3:1;              /* EDC Encoder Error Interrupt Enable */
    vuint32_t EDC_EIE4:1;              /* EDC Encoder Error Interrupt Enable */
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t EDC_DIE1:1;              /* EDC Decoder Error Interrupt Enable */
    vuint32_t EDC_DIE2:1;              /* EDC Decoder Error Interrupt Enable */
    vuint32_t EDC_DIE3:1;              /* EDC Decoder Error Interrupt Enable */
    vuint32_t EDC_DIE4:1;              /* EDC Decoder Error Interrupt Enable */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} MEW_EDC_ERR_IE_tag;

typedef union MEW_EDC_ERR_IN_STCLR_union_tag { /* AXI EDC Error Interrupt Status And Clear */
  vuint32_t R;
  struct {
    vuint32_t EDC_ENC1:1;              /* EDC_ENC1 */
    vuint32_t EDC_ENC2:1;              /* EDC_ENC2 */
    vuint32_t EDC_ENC3:1;              /* EDC_ENC3 */
    vuint32_t EDC_ENC4:1;              /* EDC_ENC4 */
    vuint32_t _unused_4:12;            /* Reserved */
    vuint32_t EDC_DEC1:1;              /* EDC_DEC1 */
    vuint32_t EDC_DEC2:1;              /* EDC_DEC2 */
    vuint32_t EDC_DEC3:1;              /* EDC_DEC3 */
    vuint32_t EDC_DEC4:1;              /* EDC_DEC4 */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} MEW_EDC_ERR_IN_STCLR_tag;

typedef union MEW_ECC_SHD_STAT_CTRL_union_tag { /* Shadow Control, RW Path Status And Status Clear */
  vuint32_t R;
  struct {
    vuint32_t STAT_CLR:1;              /* All Stat Clear */
    vuint32_t COUNT_CLR:1;             /* All-Counter Clear */
    vuint32_t _unused_2:6;             /* Reserved */
    vuint32_t RD_TX_FULL:1;            /* Read Region Tx-Table Full */
    vuint32_t WR_TX_FULL:1;            /* Write Region Tx-Table Full */
    vuint32_t WR_RES_FULL:1;           /* Write Region Resp-Table Full */
    vuint32_t _unused_11:5;            /* Reserved */
    vuint32_t RD_TX_EMP:1;             /* Read Region Tx-Table Empty */
    vuint32_t WR_TX_EMP:1;             /* Write Region Tx-Table Empty */
    vuint32_t WR_RES_EMP:1;            /* Write Region Resp-Table Empty */
    vuint32_t _unused_19:5;            /* Reserved */
    vuint32_t SHD_RGN_SLT:1;           /* Shadow Region Select */
    vuint32_t _unused_25:7;            /* Reserved */
  } B;
} MEW_ECC_SHD_STAT_CTRL_tag;

typedef union MEW_ECC_CBL_UNCBL_BIT_EC_union_tag { /* Correctable And Uncorrectable Bit Error Counter */
  vuint32_t R;
  struct {
    vuint32_t CLB_BIT_EC:15;           /* Correctable Bit Error Count */
    vuint32_t CBL_BIT_OF:1;            /* Correctable Bit Overflow */
    vuint32_t UCBL_BIT_EC:15;          /* Uncorrectable Bit Error Count */
    vuint32_t UCBL_BIT_OF:1;           /* Uncorrectable Bit Overflow */
  } B;
} MEW_ECC_CBL_UNCBL_BIT_EC_tag;

typedef union MEW_ECC_CBL_UCBL_BEAT_EC_union_tag { /* Correctable And Uncorrectable Beat Error Counter */
  vuint32_t R;
  struct {
    vuint32_t CBL_BEAT_EC:15;          /* Correctable Beat Error Count */
    vuint32_t CBL_BEAT_OF:1;           /* Correctable Beat Overflow */
    vuint32_t UCBL_BEAT_EC:15;         /* Un-correctable Beat Error Count */
    vuint32_t UCBL_BEAT_OF:1;          /* Un-correctable Beat Overflow */
  } B;
} MEW_ECC_CBL_UCBL_BEAT_EC_tag;

typedef union MEW_ECC_DBG_CTRL_union_tag { /* Debug And Debug Control */
  vuint32_t R;
  struct {
    vuint32_t ADD_EN_ECC_DIS:1;        /* Address Mapper Enabled But ECC Disabled */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MEW_ECC_DBG_CTRL_tag;

struct MEW_tag {
  MEW_ECC_GLBL_CTRL_tag ECC_GLBL_CTRL; /* AXI ECC Global Control */
  MEW_ECC_MX_EPA_tag ECC_MX_EPA;       /* AXI ECC Maximum ECC Protected Address */
  MEW_ECC_MN_EPA_tag ECC_MN_EPA;       /* AXI ECC Minimum ECC Protected Address */
  MEW_ECC_LK_PTN_tag ECC_LK_PTN;       /* AXI ECC Lock Pattern */
  MEW_ECC_ULK_PTN_tag ECC_ULK_PTN;     /* AXI ECC Unlock Pattern */
  uint8_t MEW_reserved0[4];
  MEW_ECC_EERAR_tag ECC_EERAR;         /* ECC Error Report Address */
  MEW_ECC_EERDSR_tag ECC_EERDSR[4];    /* AXI ECC Error Report Data And Syndrome */
  MEW_ECC_ERR_IE_tag ECC_ERR_IE;       /* AXI ECC Error Interrupt Enable */
  MEW_ECC_ERR_IN_STCLR_tag ECC_ERR_IN_STCLR; /* AXI ECC Error Interupt Status And Clear */
  MEW_EDC_ERR_IE_tag EDC_ERR_IE;       /* AXI EDC Error Interrupt Enable */
  MEW_EDC_ERR_IN_STCLR_tag EDC_ERR_IN_STCLR; /* AXI EDC Error Interrupt Status And Clear */
  uint8_t MEW_reserved1[4];
  MEW_ECC_SHD_STAT_CTRL_tag ECC_SHD_STAT_CTRL; /* Shadow Control, RW Path Status And Status Clear */
  MEW_ECC_CBL_UNCBL_BIT_EC_tag ECC_CBL_UNCBL_BIT_EC; /* Correctable And Uncorrectable Bit Error Counter */
  MEW_ECC_CBL_UCBL_BEAT_EC_tag ECC_CBL_UCBL_BEAT_EC; /* Correctable And Uncorrectable Beat Error Counter */
  uint8_t MEW_reserved2[52];
  MEW_ECC_DBG_CTRL_tag ECC_DBG_CTRL;   /* Debug And Debug Control */
};


/* ============================================================================
   =============================== Module: MRU ================================
   ============================================================================ */

typedef union MRU_CHXCONFIG_CH_CFG0_union_tag { /* Channel (x) Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t CHE:1;                   /* Channel Enable */
    vuint32_t CHR:1;                   /* Channel Reset */
    vuint32_t IE:1;                    /* Interrupt Enable */
    vuint32_t _unused_3:13;
    vuint32_t MBE0:1;                  /* Mailbox Enable */
    vuint32_t MBE1:1;                  /* Mailbox Enable */
    vuint32_t MBE2:1;                  /* Mailbox Enable */
    vuint32_t MBE3:1;                  /* Mailbox Enable */
    vuint32_t MBE4:1;                  /* Mailbox Enable */
    vuint32_t MBE5:1;                  /* Mailbox Enable */
    vuint32_t MBE6:1;                  /* Mailbox Enable */
    vuint32_t MBE7:1;                  /* Mailbox Enable */
    vuint32_t MBE8:1;                  /* Mailbox Enable */
    vuint32_t MBE9:1;                  /* Mailbox Enable */
    vuint32_t MBE10:1;                 /* Mailbox Enable */
    vuint32_t MBE11:1;                 /* Mailbox Enable */
    vuint32_t MBE12:1;                 /* Mailbox Enable */
    vuint32_t MBE13:1;                 /* Mailbox Enable */
    vuint32_t MBE14:1;                 /* Mailbox Enable */
    vuint32_t MBE15:1;                 /* Mailbox Enable */
  } B;
} MRU_CHXCONFIG_CH_CFG0_tag;

typedef union MRU_CHXCONFIG_CH_CFG1_union_tag { /* Channel (x) Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MBIC0:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC1:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC2:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC3:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC4:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC5:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC6:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC7:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC8:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC9:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC10:1;                /* Mailbox Interrupt Configuration */
    vuint32_t MBIC11:1;                /* Mailbox Interrupt Configuration */
    vuint32_t MBIC12:1;                /* Mailbox Interrupt Configuration */
    vuint32_t MBIC13:1;                /* Mailbox Interrupt Configuration */
    vuint32_t MBIC14:1;                /* Mailbox Interrupt Configuration */
    vuint32_t MBIC15:1;                /* Mailbox Interrupt Configuration */
  } B;
} MRU_CHXCONFIG_CH_CFG1_tag;

typedef union MRU_CHXCONFIG_CH_MBSTAT_union_tag { /* Channel (x) Mailbox Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MBS0:1;                  /* Mailbox Status */
    vuint32_t MBS1:1;                  /* Mailbox Status */
    vuint32_t MBS2:1;                  /* Mailbox Status */
    vuint32_t MBS3:1;                  /* Mailbox Status */
    vuint32_t MBS4:1;                  /* Mailbox Status */
    vuint32_t MBS5:1;                  /* Mailbox Status */
    vuint32_t MBS6:1;                  /* Mailbox Status */
    vuint32_t MBS7:1;                  /* Mailbox Status */
    vuint32_t MBS8:1;                  /* Mailbox Status */
    vuint32_t MBS9:1;                  /* Mailbox Status */
    vuint32_t MBS10:1;                 /* Mailbox Status */
    vuint32_t MBS11:1;                 /* Mailbox Status */
    vuint32_t MBS12:1;                 /* Mailbox Status */
    vuint32_t MBS13:1;                 /* Mailbox Status */
    vuint32_t MBS14:1;                 /* Mailbox Status */
    vuint32_t MBS15:1;                 /* Mailbox Status */
  } B;
} MRU_CHXCONFIG_CH_MBSTAT_tag;

typedef struct MRU_CHXCONFIG_struct_tag {
  MRU_CHXCONFIG_CH_CFG0_tag CH_CFG0;   /* Channel (x) Configuration 0 */
  MRU_CHXCONFIG_CH_CFG1_tag CH_CFG1;   /* Channel (x) Configuration 1 */
  MRU_CHXCONFIG_CH_MBSTAT_tag CH_MBSTAT; /* Channel (x) Mailbox Status */
  uint8_t CHXCONFIG_reserved0[4];
} MRU_CHXCONFIG_tag;

typedef union MRU_NOTIFY0_union_tag {  /* Notification 0 Status */
  vuint32_t R;
  struct {
    vuint32_t CH1_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH2_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH3_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH4_IS0:1;               /* Interrupt 0 Status */
    vuint32_t _unused_4:28;
  } B;
} MRU_NOTIFY0_tag;

typedef union MRU_NOTIFY1_union_tag {  /* Notification 1 Status */
  vuint32_t R;
  struct {
    vuint32_t CH1_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH2_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH3_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH4_IS1:1;               /* Interrupt 1 Status */
    vuint32_t _unused_4:28;
  } B;
} MRU_NOTIFY1_tag;

typedef union MRU_CH1_MB0_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB0_tag;

typedef union MRU_CH1_MB1_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB1_tag;

typedef union MRU_CH1_MB2_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB2_tag;

typedef union MRU_CH1_MB3_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB3_tag;

typedef union MRU_CH1_MB4_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB4_tag;

typedef union MRU_CH1_MB5_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB5_tag;

typedef union MRU_CH1_MB6_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB6_tag;

typedef union MRU_CH1_MB7_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB7_tag;

typedef union MRU_CH1_MB8_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB8_tag;

typedef union MRU_CH1_MB9_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB9_tag;

typedef union MRU_CH1_MB10_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB10_tag;

typedef union MRU_CH1_MB11_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB11_tag;

typedef union MRU_CH1_MB12_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB12_tag;

typedef union MRU_CH1_MB13_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB13_tag;

typedef union MRU_CH1_MB14_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB14_tag;

typedef union MRU_CH1_MB15_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH1_MB15_tag;

typedef union MRU_CH2_MB0_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB0_tag;

typedef union MRU_CH2_MB1_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB1_tag;

typedef union MRU_CH2_MB2_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB2_tag;

typedef union MRU_CH2_MB3_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB3_tag;

typedef union MRU_CH2_MB4_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB4_tag;

typedef union MRU_CH2_MB5_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB5_tag;

typedef union MRU_CH2_MB6_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB6_tag;

typedef union MRU_CH2_MB7_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB7_tag;

typedef union MRU_CH2_MB8_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB8_tag;

typedef union MRU_CH2_MB9_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB9_tag;

typedef union MRU_CH2_MB10_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB10_tag;

typedef union MRU_CH2_MB11_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB11_tag;

typedef union MRU_CH2_MB12_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB12_tag;

typedef union MRU_CH2_MB13_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB13_tag;

typedef union MRU_CH2_MB14_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB14_tag;

typedef union MRU_CH2_MB15_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH2_MB15_tag;

typedef union MRU_CH3_MB0_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB0_tag;

typedef union MRU_CH3_MB1_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB1_tag;

typedef union MRU_CH3_MB2_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB2_tag;

typedef union MRU_CH3_MB3_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB3_tag;

typedef union MRU_CH3_MB4_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB4_tag;

typedef union MRU_CH3_MB5_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB5_tag;

typedef union MRU_CH3_MB6_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB6_tag;

typedef union MRU_CH3_MB7_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB7_tag;

typedef union MRU_CH3_MB8_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB8_tag;

typedef union MRU_CH3_MB9_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB9_tag;

typedef union MRU_CH3_MB10_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB10_tag;

typedef union MRU_CH3_MB11_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB11_tag;

typedef union MRU_CH3_MB12_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB12_tag;

typedef union MRU_CH3_MB13_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB13_tag;

typedef union MRU_CH3_MB14_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB14_tag;

typedef union MRU_CH3_MB15_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH3_MB15_tag;

typedef union MRU_CH4_MB0_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB0_tag;

typedef union MRU_CH4_MB1_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB1_tag;

typedef union MRU_CH4_MB2_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB2_tag;

typedef union MRU_CH4_MB3_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB3_tag;

typedef union MRU_CH4_MB4_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB4_tag;

typedef union MRU_CH4_MB5_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB5_tag;

typedef union MRU_CH4_MB6_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB6_tag;

typedef union MRU_CH4_MB7_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB7_tag;

typedef union MRU_CH4_MB8_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB8_tag;

typedef union MRU_CH4_MB9_union_tag {  /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB9_tag;

typedef union MRU_CH4_MB10_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB10_tag;

typedef union MRU_CH4_MB11_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB11_tag;

typedef union MRU_CH4_MB12_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB12_tag;

typedef union MRU_CH4_MB13_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB13_tag;

typedef union MRU_CH4_MB14_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB14_tag;

typedef union MRU_CH4_MB15_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} MRU_CH4_MB15_tag;

struct MRU_tag {
  MRU_CHXCONFIG_tag CHXCONFIG[4];
  uint8_t MRU_reserved0[448];
  MRU_NOTIFY0_tag NOTIFY0;             /* Notification 0 Status */
  MRU_NOTIFY1_tag NOTIFY1;             /* Notification 1 Status */
  uint8_t MRU_reserved1[15864];
  MRU_CH1_MB0_tag CH1_MB0;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB1_tag CH1_MB1;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB2_tag CH1_MB2;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB3_tag CH1_MB3;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB4_tag CH1_MB4;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB5_tag CH1_MB5;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB6_tag CH1_MB6;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB7_tag CH1_MB7;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB8_tag CH1_MB8;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB9_tag CH1_MB9;             /* Channel (x) Mailbox (n) */
  MRU_CH1_MB10_tag CH1_MB10;           /* Channel (x) Mailbox (n) */
  MRU_CH1_MB11_tag CH1_MB11;           /* Channel (x) Mailbox (n) */
  MRU_CH1_MB12_tag CH1_MB12;           /* Channel (x) Mailbox (n) */
  MRU_CH1_MB13_tag CH1_MB13;           /* Channel (x) Mailbox (n) */
  MRU_CH1_MB14_tag CH1_MB14;           /* Channel (x) Mailbox (n) */
  MRU_CH1_MB15_tag CH1_MB15;           /* Channel (x) Mailbox (n) */
  uint8_t MRU_reserved2[16320];
  MRU_CH2_MB0_tag CH2_MB0;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB1_tag CH2_MB1;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB2_tag CH2_MB2;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB3_tag CH2_MB3;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB4_tag CH2_MB4;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB5_tag CH2_MB5;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB6_tag CH2_MB6;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB7_tag CH2_MB7;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB8_tag CH2_MB8;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB9_tag CH2_MB9;             /* Channel (x) Mailbox (n) */
  MRU_CH2_MB10_tag CH2_MB10;           /* Channel (x) Mailbox (n) */
  MRU_CH2_MB11_tag CH2_MB11;           /* Channel (x) Mailbox (n) */
  MRU_CH2_MB12_tag CH2_MB12;           /* Channel (x) Mailbox (n) */
  MRU_CH2_MB13_tag CH2_MB13;           /* Channel (x) Mailbox (n) */
  MRU_CH2_MB14_tag CH2_MB14;           /* Channel (x) Mailbox (n) */
  MRU_CH2_MB15_tag CH2_MB15;           /* Channel (x) Mailbox (n) */
  uint8_t MRU_reserved3[16320];
  MRU_CH3_MB0_tag CH3_MB0;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB1_tag CH3_MB1;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB2_tag CH3_MB2;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB3_tag CH3_MB3;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB4_tag CH3_MB4;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB5_tag CH3_MB5;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB6_tag CH3_MB6;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB7_tag CH3_MB7;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB8_tag CH3_MB8;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB9_tag CH3_MB9;             /* Channel (x) Mailbox (n) */
  MRU_CH3_MB10_tag CH3_MB10;           /* Channel (x) Mailbox (n) */
  MRU_CH3_MB11_tag CH3_MB11;           /* Channel (x) Mailbox (n) */
  MRU_CH3_MB12_tag CH3_MB12;           /* Channel (x) Mailbox (n) */
  MRU_CH3_MB13_tag CH3_MB13;           /* Channel (x) Mailbox (n) */
  MRU_CH3_MB14_tag CH3_MB14;           /* Channel (x) Mailbox (n) */
  MRU_CH3_MB15_tag CH3_MB15;           /* Channel (x) Mailbox (n) */
  uint8_t MRU_reserved4[16320];
  MRU_CH4_MB0_tag CH4_MB0;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB1_tag CH4_MB1;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB2_tag CH4_MB2;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB3_tag CH4_MB3;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB4_tag CH4_MB4;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB5_tag CH4_MB5;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB6_tag CH4_MB6;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB7_tag CH4_MB7;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB8_tag CH4_MB8;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB9_tag CH4_MB9;             /* Channel (x) Mailbox (n) */
  MRU_CH4_MB10_tag CH4_MB10;           /* Channel (x) Mailbox (n) */
  MRU_CH4_MB11_tag CH4_MB11;           /* Channel (x) Mailbox (n) */
  MRU_CH4_MB12_tag CH4_MB12;           /* Channel (x) Mailbox (n) */
  MRU_CH4_MB13_tag CH4_MB13;           /* Channel (x) Mailbox (n) */
  MRU_CH4_MB14_tag CH4_MB14;           /* Channel (x) Mailbox (n) */
  MRU_CH4_MB15_tag CH4_MB15;           /* Channel (x) Mailbox (n) */
};


/* ============================================================================
   =============================== Module: MSCM ===============================
   ============================================================================ */

typedef union MSCM_CPXTYPE_union_tag { /* Core Processor x Type */
  vuint32_t R;
  struct {
    vuint32_t PERSONALITY:32;          /* Personality */
  } B;
} MSCM_CPXTYPE_tag;

typedef union MSCM_CPXNUM_union_tag {  /* Core Processor x Number */
  vuint32_t R;
  struct {
    vuint32_t CPN:4;                   /* Processor Number */
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} MSCM_CPXNUM_tag;

typedef union MSCM_CPXREV_union_tag {  /* Core Processor x Revision */
  vuint32_t R;
  struct {
    vuint32_t RYPZ:8;                  /* Processor Revision */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} MSCM_CPXREV_tag;

typedef union MSCM_CPXCFG0_union_tag { /* Core Processor x Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t DCWY:8;                  /* Level 1 Data Cache Ways */
    vuint32_t DCSZ:8;                  /* Level 1 Data Cache Size */
    vuint32_t ICWY:8;                  /* Level 1 Instruction Cache Ways */
    vuint32_t ICSZ:8;                  /* Level 1 Instruction Cache Size */
  } B;
} MSCM_CPXCFG0_tag;

typedef union MSCM_CPXCFG1_union_tag { /* Core or Cluster Processor x Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t L2WY:8;                  /* Level 2 Cache Ways */
    vuint32_t L2SZ:8;                  /* Level 2 Cache Size */
  } B;
} MSCM_CPXCFG1_tag;

typedef union MSCM_CPXCFG2_union_tag { /* Core Processor x Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t ITCM:16;                 /* Instruction TCM Size */
    vuint32_t TCM:16;                  /* TCM Size */
  } B;
} MSCM_CPXCFG2_tag;

typedef union MSCM_CPXCFG3_union_tag { /* Core Processor x Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t FPU:1;                   /* Floating Point Unit */
    vuint32_t SIMD:1;                  /* SIMD/Neon instruction support */
    vuint32_t MMU:1;                   /* Memory Management Unit */
    vuint32_t CMP:1;                   /* Core Memory Protection Unit */
    vuint32_t CPY:1;                   /* Cryptography */
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} MSCM_CPXCFG3_tag;

typedef union MSCM_CP0TYPE_union_tag { /* Cluster Processor 0 Type */
  vuint32_t R;
  struct {
    vuint32_t PERSONALITY:32;          /* Personality */
  } B;
} MSCM_CP0TYPE_tag;

typedef union MSCM_CP0NUM_union_tag {  /* Cluster Processor 0 Number */
  vuint32_t R;
  struct {
    vuint32_t CPN:4;                   /* Processor Number */
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} MSCM_CP0NUM_tag;

typedef union MSCM_CP0REV_union_tag {  /* Core Processor 0 Revision */
  vuint32_t R;
  struct {
    vuint32_t RYPZ:8;                  /* Processor Revision */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} MSCM_CP0REV_tag;

typedef union MSCM_CP0CFG0_union_tag { /* Core Processor 0 Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t DCWY:8;                  /* Level 1 Data Cache Ways */
    vuint32_t DCSZ:8;                  /* Level 1 Data Cache Size */
    vuint32_t ICWY:8;                  /* Level 1 Instruction Cache Ways */
    vuint32_t ICSZ:8;                  /* Level 1 Instruction Cache Size */
  } B;
} MSCM_CP0CFG0_tag;

typedef union MSCM_CP0CFG1_union_tag { /* Cluster Processor 0 Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t L2WY:8;                  /* Level 2 Cache Ways */
    vuint32_t L2SZ:8;                  /* Level 2 Cache Size */
  } B;
} MSCM_CP0CFG1_tag;

typedef union MSCM_CP0CFG2_union_tag { /* Core Processor 0 Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t TCM:16;                  /* TCM Size */
  } B;
} MSCM_CP0CFG2_tag;

typedef union MSCM_CP0CFG3_union_tag { /* Core Processor 0 Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t FPU:1;                   /* Floating Point Unit */
    vuint32_t SIMD:1;                  /* SIMD/Neon instruction support */
    vuint32_t MMU:1;                   /* Memory Management Unit */
    vuint32_t CMP:1;                   /* Core Memory Protection Unit */
    vuint32_t CPY:1;                   /* Cryptography */
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} MSCM_CP0CFG3_tag;

typedef union MSCM_CP1TYPE_union_tag { /* Cluster Processor 1 Type */
  vuint32_t R;
  struct {
    vuint32_t PERSONALITY:32;          /* Personality */
  } B;
} MSCM_CP1TYPE_tag;

typedef union MSCM_CP1NUM_union_tag {  /* Cluster Processor 1 Number */
  vuint32_t R;
  struct {
    vuint32_t CPN:4;                   /* Processor Number */
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} MSCM_CP1NUM_tag;

typedef union MSCM_CP1REV_union_tag {  /* Core Processor 1 Revision */
  vuint32_t R;
  struct {
    vuint32_t RYPZ:8;                  /* Processor Revision */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} MSCM_CP1REV_tag;

typedef union MSCM_CP1CFG0_union_tag { /* Core Processor 1 Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t DCWY:8;                  /* Level 1 Data Cache Ways */
    vuint32_t DCSZ:8;                  /* Level 1 Data Cache Size */
    vuint32_t ICWY:8;                  /* Level 1 Instruction Cache Ways */
    vuint32_t ICSZ:8;                  /* Level 1 Instruction Cache Size */
  } B;
} MSCM_CP1CFG0_tag;

typedef union MSCM_CP1CFG1_union_tag { /* Cluster Processor 1 Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t L2WY:8;                  /* Level 2 Cache Ways */
    vuint32_t L2SZ:8;                  /* Level 2 Cache Size */
  } B;
} MSCM_CP1CFG1_tag;

typedef union MSCM_CP1CFG2_union_tag { /* Core Processor 1 Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t TCM:16;                  /* TCM Size */
  } B;
} MSCM_CP1CFG2_tag;

typedef union MSCM_CP1CFG3_union_tag { /* Core Processor 1 Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t FPU:1;                   /* Floating Point Unit */
    vuint32_t SIMD:1;                  /* SIMD/Neon instruction support */
    vuint32_t MMU:1;                   /* Memory Management Unit */
    vuint32_t CMP:1;                   /* Core Memory Protection Unit */
    vuint32_t CPY:1;                   /* Cryptography */
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} MSCM_CP1CFG3_tag;

typedef union MSCM_CP2TYPE_union_tag { /* Core Processor 2 Type */
  vuint32_t R;
  struct {
    vuint32_t PERSONALITY:32;          /* Personality */
  } B;
} MSCM_CP2TYPE_tag;

typedef union MSCM_CP2NUM_union_tag {  /* Core Processor 2 Number */
  vuint32_t R;
  struct {
    vuint32_t CPN:4;                   /* Processor Number */
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} MSCM_CP2NUM_tag;

typedef union MSCM_CP2REV_union_tag {  /* Core Processor 2 Revision */
  vuint32_t R;
  struct {
    vuint32_t RYPZ:8;                  /* Processor Revision */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} MSCM_CP2REV_tag;

typedef union MSCM_CP2CFG0_union_tag { /* Core Processor 2 Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t DCWY:8;                  /* Level 1 Data Cache Ways */
    vuint32_t DCSZ:8;                  /* Level 1 Data Cache Size */
    vuint32_t ICWY:8;                  /* Level 1 Instruction Cache Ways */
    vuint32_t ICSZ:8;                  /* Level 1 Instruction Cache Size */
  } B;
} MSCM_CP2CFG0_tag;

typedef union MSCM_CP2CFG1_union_tag { /* Core Processor 2 Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t L2WY:8;                  /* Level 2 Cache Ways */
    vuint32_t L2SZ:8;                  /* Level 2 Cache Size */
  } B;
} MSCM_CP2CFG1_tag;

typedef union MSCM_CP2CFG2_union_tag { /* Core Processor 2 Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t ITCM:16;                 /* Instruction TCM Size */
    vuint32_t TCM:16;                  /* Data TCM Size */
  } B;
} MSCM_CP2CFG2_tag;

typedef union MSCM_CP2CFG3_union_tag { /* Core Processor 2 Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t FPU:1;                   /* Floating Point Unit */
    vuint32_t SIMD:1;                  /* SIMD/Neon instruction support */
    vuint32_t MMU:1;                   /* Memory Management Unit */
    vuint32_t CMP:1;                   /* Core Memory Protection Unit */
    vuint32_t CPY:1;                   /* Cryptography */
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} MSCM_CP2CFG3_tag;

typedef union MSCM_CP3TYPE_union_tag { /* Core Processor 3 Type */
  vuint32_t R;
  struct {
    vuint32_t PERSONALITY:32;          /* Personality */
  } B;
} MSCM_CP3TYPE_tag;

typedef union MSCM_CP3NUM_union_tag {  /* Core Processor 3 Number */
  vuint32_t R;
  struct {
    vuint32_t CPN:4;                   /* Processor Number */
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} MSCM_CP3NUM_tag;

typedef union MSCM_CP3REV_union_tag {  /* Core Processor 3 Revision */
  vuint32_t R;
  struct {
    vuint32_t RYPZ:8;                  /* Processor Revision */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} MSCM_CP3REV_tag;

typedef union MSCM_CP3CFG0_union_tag { /* Core Processor 3 Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t DCWY:8;                  /* Level 1 Data Cache Ways */
    vuint32_t DCSZ:8;                  /* Level 1 Data Cache Size */
    vuint32_t ICWY:8;                  /* Level 1 Instruction Cache Ways */
    vuint32_t ICSZ:8;                  /* Level 1 Instruction Cache Size */
  } B;
} MSCM_CP3CFG0_tag;

typedef union MSCM_CP3CFG1_union_tag { /* Core Processor 3 Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t L2WY:8;                  /* Level 2 Cache Ways */
    vuint32_t L2SZ:8;                  /* Level 2 Cache Size */
  } B;
} MSCM_CP3CFG1_tag;

typedef union MSCM_CP3CFG2_union_tag { /* Core Processor 3 Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t ITCM:16;                 /* Instruction TCM Size */
    vuint32_t TCM:16;                  /* Data TCM Size */
  } B;
} MSCM_CP3CFG2_tag;

typedef union MSCM_CP3CFG3_union_tag { /* Core Processor 3 Configuration 3 */
  vuint32_t R;
  struct {
    vuint32_t FPU:1;                   /* Floating Point Unit */
    vuint32_t SIMD:1;                  /* SIMD/Neon instruction support */
    vuint32_t MMU:1;                   /* Memory Management Unit */
    vuint32_t CMP:1;                   /* Core Memory Protection Unit */
    vuint32_t CPY:1;                   /* Cryptography */
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} MSCM_CP3CFG3_tag;

typedef union MSCM_IRSPRC_union_tag {  /* Interrupt Router Shared Peripheral Routing Control */
  vuint16_t R;
  struct {
    vuint16_t RTU0_GIC:1;              /* Enable RTU0 GIC Interrupt Steering */
    vuint16_t RTU1_GIC:1;              /* Enable RTU1 GIC Interrupt Steering */
    vuint16_t SMU_CORE:1;              /* Enable SMU Core Interrupt Steering */
    vuint16_t _unused_3:12;
    vuint16_t LOCK:1;                  /* Lock */
  } B;
} MSCM_IRSPRC_tag;

struct MSCM_tag {
  MSCM_CPXTYPE_tag CPXTYPE;            /* Core Processor x Type */
  MSCM_CPXNUM_tag CPXNUM;              /* Core Processor x Number */
  MSCM_CPXREV_tag CPXREV;              /* Core Processor x Revision */
  MSCM_CPXCFG0_tag CPXCFG0;            /* Core Processor x Configuration 0 */
  MSCM_CPXCFG1_tag CPXCFG1;            /* Core or Cluster Processor x Configuration 1 */
  MSCM_CPXCFG2_tag CPXCFG2;            /* Core Processor x Configuration 2 */
  MSCM_CPXCFG3_tag CPXCFG3;            /* Core Processor x Configuration 3 */
  uint8_t MSCM_reserved0[4];
  MSCM_CP0TYPE_tag CP0TYPE;            /* Cluster Processor 0 Type */
  MSCM_CP0NUM_tag CP0NUM;              /* Cluster Processor 0 Number */
  MSCM_CP0REV_tag CP0REV;              /* Core Processor 0 Revision */
  MSCM_CP0CFG0_tag CP0CFG0;            /* Core Processor 0 Configuration 0 */
  MSCM_CP0CFG1_tag CP0CFG1;            /* Cluster Processor 0 Configuration 1 */
  MSCM_CP0CFG2_tag CP0CFG2;            /* Core Processor 0 Configuration 2 */
  MSCM_CP0CFG3_tag CP0CFG3;            /* Core Processor 0 Configuration 3 */
  uint8_t MSCM_reserved1[4];
  MSCM_CP1TYPE_tag CP1TYPE;            /* Cluster Processor 1 Type */
  MSCM_CP1NUM_tag CP1NUM;              /* Cluster Processor 1 Number */
  MSCM_CP1REV_tag CP1REV;              /* Core Processor 1 Revision */
  MSCM_CP1CFG0_tag CP1CFG0;            /* Core Processor 1 Configuration 0 */
  MSCM_CP1CFG1_tag CP1CFG1;            /* Cluster Processor 1 Configuration 1 */
  MSCM_CP1CFG2_tag CP1CFG2;            /* Core Processor 1 Configuration 2 */
  MSCM_CP1CFG3_tag CP1CFG3;            /* Core Processor 1 Configuration 3 */
  uint8_t MSCM_reserved2[4];
  MSCM_CP2TYPE_tag CP2TYPE;            /* Core Processor 2 Type */
  MSCM_CP2NUM_tag CP2NUM;              /* Core Processor 2 Number */
  MSCM_CP2REV_tag CP2REV;              /* Core Processor 2 Revision */
  MSCM_CP2CFG0_tag CP2CFG0;            /* Core Processor 2 Configuration 0 */
  MSCM_CP2CFG1_tag CP2CFG1;            /* Core Processor 2 Configuration 1 */
  MSCM_CP2CFG2_tag CP2CFG2;            /* Core Processor 2 Configuration 2 */
  MSCM_CP2CFG3_tag CP2CFG3;            /* Core Processor 2 Configuration 3 */
  uint8_t MSCM_reserved3[4];
  MSCM_CP3TYPE_tag CP3TYPE;            /* Core Processor 3 Type */
  MSCM_CP3NUM_tag CP3NUM;              /* Core Processor 3 Number */
  MSCM_CP3REV_tag CP3REV;              /* Core Processor 3 Revision */
  MSCM_CP3CFG0_tag CP3CFG0;            /* Core Processor 3 Configuration 0 */
  MSCM_CP3CFG1_tag CP3CFG1;            /* Core Processor 3 Configuration 1 */
  MSCM_CP3CFG2_tag CP3CFG2;            /* Core Processor 3 Configuration 2 */
  MSCM_CP3CFG3_tag CP3CFG3;            /* Core Processor 3 Configuration 3 */
  uint8_t MSCM_reserved4[2020];
  MSCM_IRSPRC_tag IRSPRC[789];         /* Interrupt Router Shared Peripheral Routing Control */
};


/* ============================================================================
   =============================== Module: MU =================================
   ============================================================================ */

typedef union MU_VER_union_tag {       /* Version ID */
  vuint32_t R;
  struct {
    vuint32_t FEATURE:16;              /* Feature Set Number */
    vuint32_t MINOR:8;                 /* Minor Version Number */
    vuint32_t MAJOR:8;                 /* Major Version Number */
  } B;
} MU_VER_tag;

typedef union MU_PAR_union_tag {       /* Parameter Register */
  vuint32_t R;
  struct {
    vuint32_t TR_NUM:8;                /* Transmit Register Number */
    vuint32_t RR_NUM:8;                /* Receive Register Number */
    vuint32_t GIR_NUM:8;               /* General-purpose Interrupt Request Number */
    vuint32_t FLAG_WIDTH:8;            /* Flag Width */
  } B;
} MU_PAR_tag;

typedef union MU_CR_union_tag {        /* Control Register */
  vuint32_t R;
  struct {
    vuint32_t MUR:1;                   /* MU Reset */
    vuint32_t MURIE:1;                 /* MUB Reset Interrupt Enable */
    vuint32_t _unused_2:30;            /* Reserved */
  } B;
} MU_CR_tag;

typedef union MU_SR_union_tag {        /* Status Register */
  vuint32_t R;
  struct {
    vuint32_t MURS:1;                  /* MUA and MUB Reset State */
    vuint32_t MURIP:1;                 /* MU Reset Interrupt Pending */
    vuint32_t EP:1;                    /* MUA Side Event Pending */
    vuint32_t FUP:1;                   /* MUA Flags Update Pending */
    vuint32_t GIRP:1;                  /* MUA General-purpose Interrupt Pending */
    vuint32_t TEP:1;                   /* MUA Transmit Empty Pending */
    vuint32_t RFP:1;                   /* MUA Receive Full Pending Flag */
    vuint32_t _unused_7:25;            /* Reserved */
  } B;
} MU_SR_tag;

typedef union MU_CCR0_union_tag {      /* Core Control Register 0 */
  vuint32_t R;
  struct {
    vuint32_t NMI:1;                   /* MUB Non-maskable Interrupt Request */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MU_CCR0_tag;

typedef union MU_CSSR0_union_tag {     /* Core Sticky Status Register 0 */
  vuint32_t R;
  struct {
    vuint32_t NMIC:1;                  /* Processor A Non-Maskable-Interrupt Clear */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} MU_CSSR0_tag;

typedef union MU_FCR_union_tag {       /* Flag Control Register */
  vuint32_t R;
  struct {
    vuint32_t F0:1;                    /* MUA to MUB Flag n */
    vuint32_t F1:1;                    /* MUA to MUB Flag n */
    vuint32_t F2:1;                    /* MUA to MUB Flag n */
    vuint32_t F3:1;                    /* MUA to MUB Flag n */
    vuint32_t F4:1;                    /* MUA to MUB Flag n */
    vuint32_t F5:1;                    /* MUA to MUB Flag n */
    vuint32_t F6:1;                    /* MUA to MUB Flag n */
    vuint32_t F7:1;                    /* MUA to MUB Flag n */
    vuint32_t F8:1;                    /* MUA to MUB Flag n */
    vuint32_t F9:1;                    /* MUA to MUB Flag n */
    vuint32_t F10:1;                   /* MUA to MUB Flag n */
    vuint32_t F11:1;                   /* MUA to MUB Flag n */
    vuint32_t F12:1;                   /* MUA to MUB Flag n */
    vuint32_t F13:1;                   /* MUA to MUB Flag n */
    vuint32_t F14:1;                   /* MUA to MUB Flag n */
    vuint32_t F15:1;                   /* MUA to MUB Flag n */
    vuint32_t F16:1;                   /* MUA to MUB Flag n */
    vuint32_t F17:1;                   /* MUA to MUB Flag n */
    vuint32_t F18:1;                   /* MUA to MUB Flag n */
    vuint32_t F19:1;                   /* MUA to MUB Flag n */
    vuint32_t F20:1;                   /* MUA to MUB Flag n */
    vuint32_t F21:1;                   /* MUA to MUB Flag n */
    vuint32_t F22:1;                   /* MUA to MUB Flag n */
    vuint32_t F23:1;                   /* MUA to MUB Flag n */
    vuint32_t F24:1;                   /* MUA to MUB Flag n */
    vuint32_t F25:1;                   /* MUA to MUB Flag n */
    vuint32_t F26:1;                   /* MUA to MUB Flag n */
    vuint32_t F27:1;                   /* MUA to MUB Flag n */
    vuint32_t F28:1;                   /* MUA to MUB Flag n */
    vuint32_t F29:1;                   /* MUA to MUB Flag n */
    vuint32_t F30:1;                   /* MUA to MUB Flag n */
    vuint32_t F31:1;                   /* MUA to MUB Flag n */
  } B;
} MU_FCR_tag;

typedef union MU_FSR_union_tag {       /* Flag Status Register */
  vuint32_t R;
  struct {
    vuint32_t F0:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F1:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F2:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F3:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F4:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F5:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F6:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F7:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F8:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F9:1;                    /* MUB to MUA-Side Flag n */
    vuint32_t F10:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F11:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F12:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F13:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F14:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F15:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F16:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F17:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F18:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F19:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F20:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F21:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F22:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F23:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F24:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F25:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F26:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F27:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F28:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F29:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F30:1;                   /* MUB to MUA-Side Flag n */
    vuint32_t F31:1;                   /* MUB to MUA-Side Flag n */
  } B;
} MU_FSR_tag;

typedef union MU_GIER_union_tag {      /* General-purpose Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t GIE0:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE1:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE2:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE3:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE4:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE5:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE6:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE7:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE8:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE9:1;                  /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE10:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE11:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE12:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE13:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE14:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE15:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE16:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE17:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE18:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE19:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE20:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE21:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE22:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE23:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE24:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE25:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE26:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE27:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE28:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE29:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE30:1;                 /* MUA General-purpose Interrupt Enable n */
    vuint32_t GIE31:1;                 /* MUA General-purpose Interrupt Enable n */
  } B;
} MU_GIER_tag;

typedef union MU_GCR_union_tag {       /* General-purpose Control Register */
  vuint32_t R;
  struct {
    vuint32_t GIR0:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR1:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR2:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR3:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR4:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR5:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR6:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR7:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR8:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR9:1;                  /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR10:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR11:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR12:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR13:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR14:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR15:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR16:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR17:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR18:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR19:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR20:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR21:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR22:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR23:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR24:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR25:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR26:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR27:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR28:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR29:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR30:1;                 /* MUA General-purpose Interrupt Request n */
    vuint32_t GIR31:1;                 /* MUA General-purpose Interrupt Request n */
  } B;
} MU_GCR_tag;

typedef union MU_GSR_union_tag {       /* General-purpose Status Register */
  vuint32_t R;
  struct {
    vuint32_t GIP0:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP1:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP2:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP3:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP4:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP5:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP6:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP7:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP8:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP9:1;                  /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP10:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP11:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP12:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP13:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP14:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP15:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP16:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP17:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP18:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP19:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP20:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP21:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP22:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP23:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP24:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP25:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP26:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP27:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP28:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP29:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP30:1;                 /* MUA General-purpose Interrupt Request Pending n */
    vuint32_t GIP31:1;                 /* MUA General-purpose Interrupt Request Pending n */
  } B;
} MU_GSR_tag;

typedef union MU_TCR_union_tag {       /* Transmit Control Register */
  vuint32_t R;
  struct {
    vuint32_t TIE0:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE1:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE2:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE3:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE4:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE5:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE6:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE7:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE8:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE9:1;                  /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE10:1;                 /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE11:1;                 /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE12:1;                 /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE13:1;                 /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE14:1;                 /* MUA Transmit Interrupt Enable n */
    vuint32_t TIE15:1;                 /* MUA Transmit Interrupt Enable n */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} MU_TCR_tag;

typedef union MU_TSR_union_tag {       /* Transmit Status Register */
  vuint32_t R;
  struct {
    vuint32_t TE0:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE1:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE2:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE3:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE4:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE5:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE6:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE7:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE8:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE9:1;                   /* MUA Transmit Register n Empty */
    vuint32_t TE10:1;                  /* MUA Transmit Register n Empty */
    vuint32_t TE11:1;                  /* MUA Transmit Register n Empty */
    vuint32_t TE12:1;                  /* MUA Transmit Register n Empty */
    vuint32_t TE13:1;                  /* MUA Transmit Register n Empty */
    vuint32_t TE14:1;                  /* MUA Transmit Register n Empty */
    vuint32_t TE15:1;                  /* MUA Transmit Register n Empty */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} MU_TSR_tag;

typedef union MU_RCR_union_tag {       /* Receive Control Register */
  vuint32_t R;
  struct {
    vuint32_t RIE0:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE1:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE2:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE3:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE4:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE5:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE6:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE7:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE8:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE9:1;                  /* MUA Receive Interrupt Enable n */
    vuint32_t RIE10:1;                 /* MUA Receive Interrupt Enable n */
    vuint32_t RIE11:1;                 /* MUA Receive Interrupt Enable n */
    vuint32_t RIE12:1;                 /* MUA Receive Interrupt Enable n */
    vuint32_t RIE13:1;                 /* MUA Receive Interrupt Enable n */
    vuint32_t RIE14:1;                 /* MUA Receive Interrupt Enable n */
    vuint32_t RIE15:1;                 /* MUA Receive Interrupt Enable n */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} MU_RCR_tag;

typedef union MU_RSR_union_tag {       /* Receive Status Register */
  vuint32_t R;
  struct {
    vuint32_t RF0:1;                   /* MUA Receive Register n Full */
    vuint32_t RF1:1;                   /* MUA Receive Register n Full */
    vuint32_t RF2:1;                   /* MUA Receive Register n Full */
    vuint32_t RF3:1;                   /* MUA Receive Register n Full */
    vuint32_t RF4:1;                   /* MUA Receive Register n Full */
    vuint32_t RF5:1;                   /* MUA Receive Register n Full */
    vuint32_t RF6:1;                   /* MUA Receive Register n Full */
    vuint32_t RF7:1;                   /* MUA Receive Register n Full */
    vuint32_t RF8:1;                   /* MUA Receive Register n Full */
    vuint32_t RF9:1;                   /* MUA Receive Register n Full */
    vuint32_t RF10:1;                  /* MUA Receive Register n Full */
    vuint32_t RF11:1;                  /* MUA Receive Register n Full */
    vuint32_t RF12:1;                  /* MUA Receive Register n Full */
    vuint32_t RF13:1;                  /* MUA Receive Register n Full */
    vuint32_t RF14:1;                  /* MUA Receive Register n Full */
    vuint32_t RF15:1;                  /* MUA Receive Register n Full */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} MU_RSR_tag;

typedef union MU_TR_union_tag {        /* Transmit Register */
  vuint32_t R;
  struct {
    vuint32_t TR_DATA:32;              /* MUA Transmit Data */
  } B;
} MU_TR_tag;

typedef union MU_RR_union_tag {        /* Receive Register */
  vuint32_t R;
  struct {
    vuint32_t RR_DATA:32;              /* MUA Receive Data */
  } B;
} MU_RR_tag;

struct MU_tag {
  MU_VER_tag VER;                      /* Version ID */
  MU_PAR_tag PAR;                      /* Parameter Register */
  MU_CR_tag CR;                        /* Control Register */
  MU_SR_tag SR;                        /* Status Register */
  MU_CCR0_tag CCR0;                    /* Core Control Register 0 */
  uint8_t MU_reserved0[4];
  MU_CSSR0_tag CSSR0;                  /* Core Sticky Status Register 0 */
  uint8_t MU_reserved1[228];
  MU_FCR_tag FCR;                      /* Flag Control Register */
  MU_FSR_tag FSR;                      /* Flag Status Register */
  uint8_t MU_reserved2[8];
  MU_GIER_tag GIER;                    /* General-purpose Interrupt Enable Register */
  MU_GCR_tag GCR;                      /* General-purpose Control Register */
  MU_GSR_tag GSR;                      /* General-purpose Status Register */
  uint8_t MU_reserved3[4];
  MU_TCR_tag TCR;                      /* Transmit Control Register */
  MU_TSR_tag TSR;                      /* Transmit Status Register */
  MU_RCR_tag RCR;                      /* Receive Control Register */
  MU_RSR_tag RSR;                      /* Receive Status Register */
  uint8_t MU_reserved4[208];
  MU_TR_tag TR[16];                    /* Transmit Register */
  uint8_t MU_reserved5[64];
  MU_RR_tag RR[16];                    /* Receive Register */
};


/* ============================================================================
   =============================== Module: MULTICORE_CONFIGURATION ============
   ============================================================================ */

typedef union MULTICORE_CONFIGURATION_COM_REGX_union_tag { /* MCCI Message Register */
  vuint32_t R;
  struct {
    vuint32_t COM_REG:32;
  } B;
} MULTICORE_CONFIGURATION_COM_REGX_tag;

typedef union MULTICORE_CONFIGURATION_COM_STS_union_tag { /* MCCI Status */
  vuint32_t R;
  struct {
    vuint32_t COM_STS:8;
    vuint32_t _unused_8:24;
  } B;
} MULTICORE_CONFIGURATION_COM_STS_tag;

typedef union MULTICORE_CONFIGURATION_COM_INT_EN_union_tag { /* MCCI Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t COM_INT_EN:8;
    vuint32_t _unused_8:24;
  } B;
} MULTICORE_CONFIGURATION_COM_INT_EN_tag;

typedef union MULTICORE_CONFIGURATION_START_ADDR0_union_tag { /* IDM Snoop Start Address 0 */
  vuint32_t R;
  struct {
    vuint32_t START_ADD:21;
    vuint32_t _unused_21:11;
  } B;
} MULTICORE_CONFIGURATION_START_ADDR0_tag;

typedef union MULTICORE_CONFIGURATION_TOP_ADDR0_union_tag { /* IDM Snoop End Address 0 */
  vuint32_t R;
  struct {
    vuint32_t TOP_ADD:21;
    vuint32_t _unused_21:11;
  } B;
} MULTICORE_CONFIGURATION_TOP_ADDR0_tag;

typedef union MULTICORE_CONFIGURATION_SNOOP_CSR0_union_tag { /* IDM Snoop Control 0 */
  vuint32_t R;
  struct {
    vuint32_t SNOOP_RD_WR:2;
    vuint32_t SNOOP_EDAP_STS:1;
    vuint32_t SNOOP_EDAP_INT_EN:1;
    vuint32_t _unused_4:28;
  } B;
} MULTICORE_CONFIGURATION_SNOOP_CSR0_tag;

typedef union MULTICORE_CONFIGURATION_START_ADDR1_union_tag { /* IDM Snoop Start Address 1 */
  vuint32_t R;
  struct {
    vuint32_t START_ADD:21;
    vuint32_t _unused_21:11;
  } B;
} MULTICORE_CONFIGURATION_START_ADDR1_tag;

typedef union MULTICORE_CONFIGURATION_TOP_ADDR1_union_tag { /* IDM Snoop End Address 1 */
  vuint32_t R;
  struct {
    vuint32_t TOP_ADD:21;
    vuint32_t _unused_21:11;
  } B;
} MULTICORE_CONFIGURATION_TOP_ADDR1_tag;

typedef union MULTICORE_CONFIGURATION_SNOOP_CSR1_union_tag { /* IDM Snoop Control 1 */
  vuint32_t R;
  struct {
    vuint32_t SNOOP_RD_WR:2;
    vuint32_t SNOOP_EDAP_STS:1;
    vuint32_t SNOOP_EDAP_INT_EN:1;
    vuint32_t _unused_4:28;
  } B;
} MULTICORE_CONFIGURATION_SNOOP_CSR1_tag;

struct MULTICORE_CONFIGURATION_tag {
  MULTICORE_CONFIGURATION_COM_REGX_tag COM_REGX[8]; /* MCCI Message Register */
  uint8_t MULTICORE_CONFIGURATION_reserved0[96];
  MULTICORE_CONFIGURATION_COM_STS_tag COM_STS; /* MCCI Status */
  MULTICORE_CONFIGURATION_COM_INT_EN_tag COM_INT_EN; /* MCCI Interrupt Enable */
  uint8_t MULTICORE_CONFIGURATION_reserved1[8];
  MULTICORE_CONFIGURATION_START_ADDR0_tag START_ADDR0; /* IDM Snoop Start Address 0 */
  MULTICORE_CONFIGURATION_TOP_ADDR0_tag TOP_ADDR0; /* IDM Snoop End Address 0 */
  MULTICORE_CONFIGURATION_SNOOP_CSR0_tag SNOOP_CSR0; /* IDM Snoop Control 0 */
  MULTICORE_CONFIGURATION_START_ADDR1_tag START_ADDR1; /* IDM Snoop Start Address 1 */
  MULTICORE_CONFIGURATION_TOP_ADDR1_tag TOP_ADDR1; /* IDM Snoop End Address 1 */
  MULTICORE_CONFIGURATION_SNOOP_CSR1_tag SNOOP_CSR1; /* IDM Snoop Control 1 */
};


/* ============================================================================
   =============================== Module: NETC_F0_GLOBAL =====================
   ============================================================================ */

typedef union NETC_F0_GLOBAL_SMCAPR_union_tag { /* Shared memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_SMCAPR_tag;

typedef union NETC_F0_GLOBAL_SMDTR_union_tag { /* Shared memory depletion threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_SMDTR_tag;

typedef union NETC_F0_GLOBAL_SMACR_union_tag { /* Shared memory available count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_SMACR_tag;

typedef union NETC_F0_GLOBAL_SMCLWMR_union_tag { /* Shared memory count low watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_SMCLWMR_tag;

typedef union NETC_F0_GLOBAL_SMBUCR_union_tag { /* Shared memory buffer unassigned count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_SMBUCR_tag;

typedef union NETC_F0_GLOBAL_SMBUCHWMR_union_tag { /* Shared memory buffer unassigned count high watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_SMBUCHWMR_tag;

typedef union NETC_F0_GLOBAL_SMLCR_union_tag { /* Shared memory loss count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:6;
    vuint32_t IFLC:1;
    vuint32_t IFDC:1;
  } B;
} NETC_F0_GLOBAL_SMLCR_tag;

typedef union NETC_F0_GLOBAL_HBTCAPR_union_tag { /* Hash bucket table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:8;
    vuint32_t MAX_COL:3;
    vuint32_t _unused_27:1;
    vuint32_t MAX_VISITS:4;
  } B;
} NETC_F0_GLOBAL_HBTCAPR_tag;

typedef union NETC_F0_GLOBAL_HBTOR0_union_tag { /* Hash bucket table operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t HWM_ENTRIES:16;
  } B;
} NETC_F0_GLOBAL_HBTOR0_tag;

typedef union NETC_F0_GLOBAL_HBTOR2_union_tag { /* Hash bucket table operational register 2 */
  vuint32_t R;
  struct {
    vuint32_t RUN_AVG_FRACT:8;
    vuint32_t RUN_AVG_INT:8;
    vuint32_t HWM_COL:4;
    vuint32_t _unused_20:12;
  } B;
} NETC_F0_GLOBAL_HBTOR2_tag;

typedef union NETC_F0_GLOBAL_SMERBCAPR_union_tag { /* Shared memory ENETC receive buffer capability register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:4;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;
  } B;
} NETC_F0_GLOBAL_SMERBCAPR_tag;

typedef union NETC_F0_GLOBAL_SMERBOR0_union_tag { /* Shared memory ENETC receive buffer operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t AMOUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_SMERBOR0_tag;

typedef union NETC_F0_GLOBAL_SMERBOR1_union_tag { /* Shared memory ENETC receive buffer operational 1 */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_SMERBOR1_tag;

typedef union NETC_F0_GLOBAL_PCE_SL_PCEOR_union_tag { /* PCE 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_FRAMES:6;
    vuint32_t _unused_6:2;
    vuint32_t HWM_FRAMES:6;
    vuint32_t _unused_14:2;
    vuint32_t MAX_FRAMES:6;
    vuint32_t _unused_22:10;
  } B;
} NETC_F0_GLOBAL_PCE_SL_PCEOR_tag;

typedef union NETC_F0_GLOBAL_PCE_SL_RFEOR_union_tag { /* Replication Forwarding Engine 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_FRAMES:6;
    vuint32_t _unused_6:2;
    vuint32_t HWM_FRAMES:6;
    vuint32_t _unused_14:2;
    vuint32_t MAX_FRAMES:6;
    vuint32_t _unused_22:10;
  } B;
} NETC_F0_GLOBAL_PCE_SL_RFEOR_tag;

typedef struct NETC_F0_GLOBAL_PCE_SL_struct_tag {
  NETC_F0_GLOBAL_PCE_SL_PCEOR_tag PCEOR; /* PCE 0 operational register */
  NETC_F0_GLOBAL_PCE_SL_RFEOR_tag RFEOR; /* Replication Forwarding Engine 0 operational register */
} NETC_F0_GLOBAL_PCE_SL_tag;

typedef union NETC_F0_GLOBAL_NETCCLKR_union_tag { /* NETC clock register */
  vuint32_t R;
  struct {
    vuint32_t FREQ:11;                 /* Frequency */
    vuint32_t _unused_11:21;
  } B;
} NETC_F0_GLOBAL_NETCCLKR_tag;

typedef union NETC_F0_GLOBAL_HTA_LOOP_HTACAPR_union_tag { /* HTA 0 capability register */
  vuint32_t R;
  struct {
    vuint32_t MAX_RX_FRAMES:8;
    vuint32_t MAX_TX_FRAMES:8;
    vuint32_t _unused_16:16;
  } B;
} NETC_F0_GLOBAL_HTA_LOOP_HTACAPR_tag;

typedef union NETC_F0_GLOBAL_HTA_LOOP_HTARFCOR_union_tag { /* HTA 0 receive frame count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:8;
    vuint32_t HP_HWM:8;
    vuint32_t LP_COUNT:8;
    vuint32_t LP_HWM:8;
  } B;
} NETC_F0_GLOBAL_HTA_LOOP_HTARFCOR_tag;

typedef union NETC_F0_GLOBAL_HTA_LOOP_HTAHPBCOR_union_tag { /* HTA 0 high priority byte count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:16;
    vuint32_t HWM:16;
  } B;
} NETC_F0_GLOBAL_HTA_LOOP_HTAHPBCOR_tag;

typedef union NETC_F0_GLOBAL_HTA_LOOP_HTALPBCOR_union_tag { /* HTA 0 low priority byte count operational register */
  vuint32_t R;
  struct {
    vuint32_t LP_COUNT:16;
    vuint32_t HWM:16;
  } B;
} NETC_F0_GLOBAL_HTA_LOOP_HTALPBCOR_tag;

typedef union NETC_F0_GLOBAL_HTA_LOOP_HTATFCOR_union_tag { /* HTA 0 transmit frame count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:8;
    vuint32_t HP_HWM:8;
    vuint32_t LP_COUNT:8;
    vuint32_t LP_HWM:8;
  } B;
} NETC_F0_GLOBAL_HTA_LOOP_HTATFCOR_tag;

typedef struct NETC_F0_GLOBAL_HTA_LOOP_struct_tag {
  NETC_F0_GLOBAL_HTA_LOOP_HTACAPR_tag HTACAPR; /* HTA 0 capability register */
  NETC_F0_GLOBAL_HTA_LOOP_HTARFCOR_tag HTARFCOR; /* HTA 0 receive frame count operational register */
  NETC_F0_GLOBAL_HTA_LOOP_HTAHPBCOR_tag HTAHPBCOR; /* HTA 0 high priority byte count operational register */
  NETC_F0_GLOBAL_HTA_LOOP_HTALPBCOR_tag HTALPBCOR; /* HTA 0 low priority byte count operational register */
  uint8_t HTA_LOOP_reserved0[20];
  NETC_F0_GLOBAL_HTA_LOOP_HTATFCOR_tag HTATFCOR; /* HTA 0 transmit frame count operational register */
} NETC_F0_GLOBAL_HTA_LOOP_tag;

typedef union NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_union_tag { /* Root complex 0 system bus read latency average register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_tag;

typedef union NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_union_tag { /* Root complex 0 system bus read latency high watermark register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_tag;

typedef union NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_union_tag { /* Root complex 0 system bus write latency average register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_tag;

typedef union NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_union_tag { /* Root complex 0 system bus write latency high watermark register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_tag;

typedef struct NETC_F0_GLOBAL_ARRAY_NUM_RC_struct_tag {
  NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_tag RCSBRLAR; /* Root complex 0 system bus read latency average register */
  NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_tag RCSBRLHWMR; /* Root complex 0 system bus read latency high watermark register */
  NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_tag RCSBWLAR; /* Root complex 0 system bus write latency average register */
  NETC_F0_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_tag RCSBWLHWMR; /* Root complex 0 system bus write latency high watermark register */
} NETC_F0_GLOBAL_ARRAY_NUM_RC_tag;

typedef union NETC_F0_GLOBAL_IPBRR0_union_tag { /* IP block revision register 0 */
  vuint32_t R;
  struct {
    vuint32_t IP_MN:8;
    vuint32_t IP_MJ:8;
    vuint32_t IP_ID:16;
  } B;
} NETC_F0_GLOBAL_IPBRR0_tag;

typedef union NETC_F0_GLOBAL_IPBRR1_union_tag { /* IP block revision register 1 */
  vuint32_t R;
  struct {
    vuint32_t IP_CFG:8;
    vuint32_t IP_MNT:8;
    vuint32_t IP_INT:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_GLOBAL_IPBRR1_tag;

typedef union NETC_F0_GLOBAL_FBLPR_union_tag { /* Function boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F0_GLOBAL_FBLPR_tag;

typedef union NETC_F0_GLOBAL_TIMER_TUFSBECR_union_tag { /* Timer uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F0_GLOBAL_TIMER_TUFSBECR_tag;

typedef union NETC_F0_GLOBAL_TIMER_TUFSBESR_union_tag { /* Timer uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F0_GLOBAL_TIMER_TUFSBESR_tag;

typedef struct NETC_F0_GLOBAL_TIMER_struct_tag {
  NETC_F0_GLOBAL_TIMER_TUFSBECR_tag TUFSBECR; /* Timer uncorrectable fatal system bus error configuration register */
  NETC_F0_GLOBAL_TIMER_TUFSBESR_tag TUFSBESR; /* Timer uncorrectable fatal system bus error status register */
} NETC_F0_GLOBAL_TIMER_tag;

struct NETC_F0_GLOBAL_tag {
  NETC_F0_GLOBAL_SMCAPR_tag SMCAPR;    /* Shared memory capability register */
  NETC_F0_GLOBAL_SMDTR_tag SMDTR;      /* Shared memory depletion threshold register */
  NETC_F0_GLOBAL_SMACR_tag SMACR;      /* Shared memory available count register */
  uint8_t NETC_F0_GLOBAL_reserved0[4];
  NETC_F0_GLOBAL_SMCLWMR_tag SMCLWMR;  /* Shared memory count low watermark register */
  NETC_F0_GLOBAL_SMBUCR_tag SMBUCR;    /* Shared memory buffer unassigned count register */
  NETC_F0_GLOBAL_SMBUCHWMR_tag SMBUCHWMR; /* Shared memory buffer unassigned count high watermark register */
  NETC_F0_GLOBAL_SMLCR_tag SMLCR;      /* Shared memory loss count register */
  NETC_F0_GLOBAL_HBTCAPR_tag HBTCAPR;  /* Hash bucket table capability register */
  NETC_F0_GLOBAL_HBTOR0_tag HBTOR0;    /* Hash bucket table operational register 0 */
  uint8_t NETC_F0_GLOBAL_reserved1[4];
  NETC_F0_GLOBAL_HBTOR2_tag HBTOR2;    /* Hash bucket table operational register 2 */
  uint8_t NETC_F0_GLOBAL_reserved2[16];
  NETC_F0_GLOBAL_SMERBCAPR_tag SMERBCAPR; /* Shared memory ENETC receive buffer capability register */
  NETC_F0_GLOBAL_SMERBOR0_tag SMERBOR0; /* Shared memory ENETC receive buffer operational register 0 */
  NETC_F0_GLOBAL_SMERBOR1_tag SMERBOR1; /* Shared memory ENETC receive buffer operational 1 */
  uint8_t NETC_F0_GLOBAL_reserved3[180];
  NETC_F0_GLOBAL_PCE_SL_tag PCE_SL[1];
  uint8_t NETC_F0_GLOBAL_reserved4[92];
  NETC_F0_GLOBAL_NETCCLKR_tag NETCCLKR; /* NETC clock register */
  uint8_t NETC_F0_GLOBAL_reserved5[152];
  NETC_F0_GLOBAL_HTA_LOOP_tag HTA_LOOP[1];
  uint8_t NETC_F0_GLOBAL_reserved6[216];
  NETC_F0_GLOBAL_ARRAY_NUM_RC_tag ARRAY_NUM_RC[1];
  uint8_t NETC_F0_GLOBAL_reserved7[2280];
  NETC_F0_GLOBAL_IPBRR0_tag IPBRR0;    /* IP block revision register 0 */
  NETC_F0_GLOBAL_IPBRR1_tag IPBRR1;    /* IP block revision register 1 */
  uint8_t NETC_F0_GLOBAL_reserved8[256];
  NETC_F0_GLOBAL_FBLPR_tag FBLPR[2];   /* Function boot loader parameter register 0 */
  uint8_t NETC_F0_GLOBAL_reserved9[280];
  NETC_F0_GLOBAL_TIMER_tag TIMER;
};


/* ============================================================================
   =============================== Module: NETC_F0_PCI_HDR_TYPE0 ==============
   ============================================================================ */

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t MEM_ACCESS:1;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_ID:16;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_union_tag { /* PCI PCI-PM capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_union_tag { /* PCI PCI-PM capabilities register */
  vuint16_t R;
  struct {
    vuint16_t VERSION:3;
    vuint16_t _unused_3:8;
    vuint16_t PME_SUPPORT:5;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_union_tag { /* PCI PCI-PM control and status register */
  vuint16_t R;
  struct {
    vuint16_t PWR_STATE:2;
    vuint16_t _unused_2:1;
    vuint16_t NO_SOFT_RST:1;
    vuint16_t _unused_4:12;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_union_tag { /* PCI PCI-PM capabilities data register */
  vuint8_t R;
  struct {
    vuint8_t _unused_0:8;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_union_tag { /* PCI EA capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_union_tag { /* PCI EA capabilities register */
  vuint16_t R;
  struct {
    vuint16_t NUM_ENTRIES:6;
    vuint16_t _unused_6:10;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_union_tag { /* PCI EA per-entry 0 format register */
  vuint32_t R;
  struct {
    vuint32_t ENTRY_SIZE:3;
    vuint32_t _unused_3:1;
    vuint32_t BEI:4;
    vuint32_t PRIM_PROP:8;
    vuint32_t SEC_PROP:8;
    vuint32_t _unused_24:6;
    vuint32_t WRITABLE:1;
    vuint32_t ENABLE:1;
  } B;
} NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_union_tag { /* PCI EA per-entry 0 base register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t S:1;
    vuint32_t BASE:30;
  } B;
} NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_union_tag { /* PCI EA per-entry 0 max offset register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t S:1;
    vuint32_t MAX_OFFSET:30;
  } B;
} NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_union_tag { /* PCI EA per-entry 0 extended base register */
  vuint32_t R;
  struct {
    vuint32_t BASE:32;
  } B;
} NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_tag;

typedef struct NETC_F0_PCI_HDR_TYPE0_NUM_EA_struct_tag {
  NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_tag PCI_CFC_EA_PE_FMT; /* PCI EA per-entry 0 format register */
  NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_tag PCI_CFC_EA_PE_BASE; /* PCI EA per-entry 0 base register */
  NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_tag PCI_CFC_EA_PE_MAXOFF; /* PCI EA per-entry 0 max offset register */
  NETC_F0_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_tag PCI_CFC_EA_PE_EXT_BASE; /* PCI EA per-entry 0 extended base register */
} NETC_F0_PCI_HDR_TYPE0_NUM_EA_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:21;
    vuint32_t ACS_VIOLATION_STAT:1;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_F0_PCI_HDR_TYPE0_tag {
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved0[1];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved1[1];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved2[4];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved3[4];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved4[11];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved5[24];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved6[22];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved7[4];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_tag PCI_CFC_PCIPM_CAP_LIST; /* PCI PCI-PM capabilities list register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_tag PCI_CFC_PCIPM_CAP; /* PCI PCI-PM capabilities register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_tag PCI_CFC_PCIPM_CTL_STAT; /* PCI PCI-PM control and status register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved8[1];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_tag PCI_CFC_PCIPM_DATA; /* PCI PCI-PM capabilities data register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved9[4];
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_tag PCI_CFC_EA_CAP_LIST; /* PCI EA capabilities list register */
  NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_tag PCI_CFC_EA_CAP; /* PCI EA capabilities register */
  NETC_F0_PCI_HDR_TYPE0_NUM_EA_tag NUM_EA[2];
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved10[64];
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved11[20];
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_F0_PCI_HDR_TYPE0_reserved12[8];
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_F1 ============================
   ============================================================================ */

typedef union NETC_F1_EMDIO_CFG_union_tag { /* External MDIO configuration register */
  vuint32_t R;
  struct {
    vuint32_t BSY2:1;                  /* Busy 2 (same as bit 31) */
    vuint32_t MDIO_RD_ER:1;
    vuint32_t MDIO_HOLD:3;
    vuint32_t PRE_DIS:1;
    vuint32_t ENC45:1;
    vuint32_t MDIO_CLK_DIV:9;          /* MDIO Clock Divisor */
    vuint32_t WHOAMI:3;                /* Returns the virtual port ID. */
    vuint32_t _unused_19:3;
    vuint32_t EHOLD:1;                 /* Extended HOLD */
    vuint32_t NEG:1;
    vuint32_t _unused_24:4;
    vuint32_t ADDR_ERR:1;
    vuint32_t CIM:1;
    vuint32_t CMP:1;                   /* MDIO Command Completion */
    vuint32_t BSY1:1;                  /* Busy 1 */
  } B;
} NETC_F1_EMDIO_CFG_tag;

typedef union NETC_F1_EMDIO_CTL_union_tag { /* External MDIO interface control register */
  vuint32_t R;
  struct {
    vuint32_t DEV_ADDR:5;              /* 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
    vuint32_t PORT_ADDR:5;             /* 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
    vuint32_t _unused_10:4;
    vuint32_t POST_INC:1;              /* MDIO read with address post-increment initiation. Self-clearing once transaction is complete. */
    vuint32_t READ:1;                  /* MDIO read initiation. */
    vuint32_t _unused_16:15;
    vuint32_t BSY:1;                   /* MDIO busy */
  } B;
} NETC_F1_EMDIO_CTL_tag;

typedef union NETC_F1_EMDIO_DATA_union_tag { /* External MDIO interface data register */
  vuint32_t R;
  struct {
    vuint32_t MDIO_DATA:16;            /* 16-bit MDIO data. */
    vuint32_t _unused_16:16;
  } B;
} NETC_F1_EMDIO_DATA_tag;

typedef union NETC_F1_EMDIO_ADDR_union_tag { /* External MDIO register address register */
  vuint32_t R;
  struct {
    vuint32_t REGADDR:16;              /* MDIO PHY register address. */
    vuint32_t _unused_16:16;
  } B;
} NETC_F1_EMDIO_ADDR_tag;

typedef union NETC_F1_EMDIO_STAT_union_tag { /* External MDIO status register */
  vuint32_t R;
  struct {
    vuint32_t BSY:1;                   /* Global MDIO busy */
    vuint32_t _unused_1:7;
    vuint32_t WHT_LIST:5;              /* PHY white list */
    vuint32_t _unused_13:2;
    vuint32_t WHT_LIST_ENA:1;          /* PHY white list enable */
    vuint32_t PORT_ID:3;               /* Port ID */
    vuint32_t REQ_TYPE:1;              /* Port ID */
    vuint32_t _unused_20:12;
  } B;
} NETC_F1_EMDIO_STAT_tag;

typedef union NETC_F1_PHY_STATUS_CFG_union_tag { /* PHY status configuration register */
  vuint32_t R;
  struct {
    vuint32_t BSY:1;                   /* MDIO busy */
    vuint32_t MDIO_RD_ER:1;            /* MDIO read error */
    vuint32_t _unused_2:14;
    vuint32_t STATUS_INTERVAL:16;      /* PHY status read interval */
  } B;
} NETC_F1_PHY_STATUS_CFG_tag;

typedef union NETC_F1_PHY_STATUS_CTL_union_tag { /* PHY status control register */
  vuint32_t R;
  struct {
    vuint32_t DEV_ADDR:5;              /* 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
    vuint32_t PORT_ADDR:5;             /* 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
    vuint32_t _unused_10:22;
  } B;
} NETC_F1_PHY_STATUS_CTL_tag;

typedef union NETC_F1_PHY_STATUS_DATA_union_tag { /* PHY status data register */
  vuint32_t R;
  struct {
    vuint32_t MDIO_DATA:16;            /* 16-bit MDIO data */
    vuint32_t CURR_CNT:16;             /* Current count */
  } B;
} NETC_F1_PHY_STATUS_DATA_tag;

typedef union NETC_F1_PHY_STATUS_ADDR_union_tag { /* PHY status register address register */
  vuint32_t R;
  struct {
    vuint32_t REGADDR:16;              /* MDIO PHY register address. Address of the register within the Clause 45 PHY device from which data is to be read. */
    vuint32_t _unused_16:16;
  } B;
} NETC_F1_PHY_STATUS_ADDR_tag;

typedef union NETC_F1_PHY_STATUS_EVENT_union_tag { /* PHY status event register */
  vuint32_t R;
  struct {
    vuint32_t STATUS_EVENT_HL:16;      /* Status event high-to-low. Set to 1 if a 1->0 transition on a corresponding data bit has occurred. Write 1 to clear. */
    vuint32_t STATUS_EVENT_LH:16;      /* Status event low-to-high. Set to 1 if a 0->1 transition on a corresponding data bit has occurred. Write 1 to clear. */
  } B;
} NETC_F1_PHY_STATUS_EVENT_tag;

typedef union NETC_F1_PHY_STATUS_MASK_union_tag { /* PHY status mask register */
  vuint32_t R;
  struct {
    vuint32_t STATUS_MASK_HL:16;       /* Status high-to-low mask. If set to 1, assert an interrupt if the corresponding event bit is set. */
    vuint32_t STATUS_MASK_LH:16;       /* Status mask low-to-high. If set to 1, assert an interrupt if the corresponding event bit is set. */
  } B;
} NETC_F1_PHY_STATUS_MASK_tag;

typedef union NETC_F1_MDIO_CFG_union_tag { /* MDIO configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t MDIO_MODE:1;             /* MDIO pin mode */
    vuint32_t MDC_MODE:1;              /* MDC pin mode */
    vuint32_t _unused_6:26;
  } B;
} NETC_F1_MDIO_CFG_tag;

struct NETC_F1_tag {
  uint8_t NETC_F1_reserved0[7168];
  NETC_F1_EMDIO_CFG_tag EMDIO_CFG;     /* External MDIO configuration register */
  NETC_F1_EMDIO_CTL_tag EMDIO_CTL;     /* External MDIO interface control register */
  NETC_F1_EMDIO_DATA_tag EMDIO_DATA;   /* External MDIO interface data register */
  NETC_F1_EMDIO_ADDR_tag EMDIO_ADDR;   /* External MDIO register address register */
  NETC_F1_EMDIO_STAT_tag EMDIO_STAT;   /* External MDIO status register */
  uint8_t NETC_F1_reserved1[12];
  NETC_F1_PHY_STATUS_CFG_tag PHY_STATUS_CFG; /* PHY status configuration register */
  NETC_F1_PHY_STATUS_CTL_tag PHY_STATUS_CTL; /* PHY status control register */
  NETC_F1_PHY_STATUS_DATA_tag PHY_STATUS_DATA; /* PHY status data register */
  NETC_F1_PHY_STATUS_ADDR_tag PHY_STATUS_ADDR; /* PHY status register address register */
  NETC_F1_PHY_STATUS_EVENT_tag PHY_STATUS_EVENT; /* PHY status event register */
  NETC_F1_PHY_STATUS_MASK_tag PHY_STATUS_MASK; /* PHY status mask register */
  uint8_t NETC_F1_reserved2[8];
  NETC_F1_MDIO_CFG_tag MDIO_CFG;       /* MDIO configuration register */
};


/* ============================================================================
   =============================== Module: NETC_F1_GLOBAL =====================
   ============================================================================ */

typedef union NETC_F1_GLOBAL_SMCAPR_union_tag { /* Shared memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_SMCAPR_tag;

typedef union NETC_F1_GLOBAL_SMDTR_union_tag { /* Shared memory depletion threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_SMDTR_tag;

typedef union NETC_F1_GLOBAL_SMACR_union_tag { /* Shared memory available count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_SMACR_tag;

typedef union NETC_F1_GLOBAL_SMCLWMR_union_tag { /* Shared memory count low watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_SMCLWMR_tag;

typedef union NETC_F1_GLOBAL_SMBUCR_union_tag { /* Shared memory buffer unassigned count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_SMBUCR_tag;

typedef union NETC_F1_GLOBAL_SMBUCHWMR_union_tag { /* Shared memory buffer unassigned count high watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_SMBUCHWMR_tag;

typedef union NETC_F1_GLOBAL_SMLCR_union_tag { /* Shared memory loss count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:6;
    vuint32_t IFLC:1;
    vuint32_t IFDC:1;
  } B;
} NETC_F1_GLOBAL_SMLCR_tag;

typedef union NETC_F1_GLOBAL_HBTCAPR_union_tag { /* Hash bucket table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:8;
    vuint32_t MAX_COL:3;
    vuint32_t _unused_27:1;
    vuint32_t MAX_VISITS:4;
  } B;
} NETC_F1_GLOBAL_HBTCAPR_tag;

typedef union NETC_F1_GLOBAL_HBTOR0_union_tag { /* Hash bucket table operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t HWM_ENTRIES:16;
  } B;
} NETC_F1_GLOBAL_HBTOR0_tag;

typedef union NETC_F1_GLOBAL_HBTOR2_union_tag { /* Hash bucket table operational register 2 */
  vuint32_t R;
  struct {
    vuint32_t RUN_AVG_FRACT:8;
    vuint32_t RUN_AVG_INT:8;
    vuint32_t HWM_COL:4;
    vuint32_t _unused_20:12;
  } B;
} NETC_F1_GLOBAL_HBTOR2_tag;

typedef union NETC_F1_GLOBAL_SMERBCAPR_union_tag { /* Shared memory ENETC receive buffer capability register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:4;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;
  } B;
} NETC_F1_GLOBAL_SMERBCAPR_tag;

typedef union NETC_F1_GLOBAL_SMERBOR0_union_tag { /* Shared memory ENETC receive buffer operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t AMOUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_SMERBOR0_tag;

typedef union NETC_F1_GLOBAL_SMERBOR1_union_tag { /* Shared memory ENETC receive buffer operational 1 */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_SMERBOR1_tag;

typedef union NETC_F1_GLOBAL_PCE_SL_PCEOR_union_tag { /* PCE 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_FRAMES:6;
    vuint32_t _unused_6:2;
    vuint32_t HWM_FRAMES:6;
    vuint32_t _unused_14:2;
    vuint32_t MAX_FRAMES:6;
    vuint32_t _unused_22:10;
  } B;
} NETC_F1_GLOBAL_PCE_SL_PCEOR_tag;

typedef union NETC_F1_GLOBAL_PCE_SL_RFEOR_union_tag { /* Replication Forwarding Engine 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_FRAMES:6;
    vuint32_t _unused_6:2;
    vuint32_t HWM_FRAMES:6;
    vuint32_t _unused_14:2;
    vuint32_t MAX_FRAMES:6;
    vuint32_t _unused_22:10;
  } B;
} NETC_F1_GLOBAL_PCE_SL_RFEOR_tag;

typedef struct NETC_F1_GLOBAL_PCE_SL_struct_tag {
  NETC_F1_GLOBAL_PCE_SL_PCEOR_tag PCEOR; /* PCE 0 operational register */
  NETC_F1_GLOBAL_PCE_SL_RFEOR_tag RFEOR; /* Replication Forwarding Engine 0 operational register */
} NETC_F1_GLOBAL_PCE_SL_tag;

typedef union NETC_F1_GLOBAL_NETCCLKR_union_tag { /* NETC clock register */
  vuint32_t R;
  struct {
    vuint32_t FREQ:11;                 /* Frequency */
    vuint32_t _unused_11:21;
  } B;
} NETC_F1_GLOBAL_NETCCLKR_tag;

typedef union NETC_F1_GLOBAL_HTA_LOOP_HTACAPR_union_tag { /* HTA 0 capability register */
  vuint32_t R;
  struct {
    vuint32_t MAX_RX_FRAMES:8;
    vuint32_t MAX_TX_FRAMES:8;
    vuint32_t _unused_16:16;
  } B;
} NETC_F1_GLOBAL_HTA_LOOP_HTACAPR_tag;

typedef union NETC_F1_GLOBAL_HTA_LOOP_HTARFCOR_union_tag { /* HTA 0 receive frame count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:8;
    vuint32_t HP_HWM:8;
    vuint32_t LP_COUNT:8;
    vuint32_t LP_HWM:8;
  } B;
} NETC_F1_GLOBAL_HTA_LOOP_HTARFCOR_tag;

typedef union NETC_F1_GLOBAL_HTA_LOOP_HTAHPBCOR_union_tag { /* HTA 0 high priority byte count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:16;
    vuint32_t HWM:16;
  } B;
} NETC_F1_GLOBAL_HTA_LOOP_HTAHPBCOR_tag;

typedef union NETC_F1_GLOBAL_HTA_LOOP_HTALPBCOR_union_tag { /* HTA 0 low priority byte count operational register */
  vuint32_t R;
  struct {
    vuint32_t LP_COUNT:16;
    vuint32_t HWM:16;
  } B;
} NETC_F1_GLOBAL_HTA_LOOP_HTALPBCOR_tag;

typedef union NETC_F1_GLOBAL_HTA_LOOP_HTATFCOR_union_tag { /* HTA 0 transmit frame count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:8;
    vuint32_t HP_HWM:8;
    vuint32_t LP_COUNT:8;
    vuint32_t LP_HWM:8;
  } B;
} NETC_F1_GLOBAL_HTA_LOOP_HTATFCOR_tag;

typedef struct NETC_F1_GLOBAL_HTA_LOOP_struct_tag {
  NETC_F1_GLOBAL_HTA_LOOP_HTACAPR_tag HTACAPR; /* HTA 0 capability register */
  NETC_F1_GLOBAL_HTA_LOOP_HTARFCOR_tag HTARFCOR; /* HTA 0 receive frame count operational register */
  NETC_F1_GLOBAL_HTA_LOOP_HTAHPBCOR_tag HTAHPBCOR; /* HTA 0 high priority byte count operational register */
  NETC_F1_GLOBAL_HTA_LOOP_HTALPBCOR_tag HTALPBCOR; /* HTA 0 low priority byte count operational register */
  uint8_t HTA_LOOP_reserved0[20];
  NETC_F1_GLOBAL_HTA_LOOP_HTATFCOR_tag HTATFCOR; /* HTA 0 transmit frame count operational register */
} NETC_F1_GLOBAL_HTA_LOOP_tag;

typedef union NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_union_tag { /* Root complex 0 system bus read latency average register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_tag;

typedef union NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_union_tag { /* Root complex 0 system bus read latency high watermark register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_tag;

typedef union NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_union_tag { /* Root complex 0 system bus write latency average register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_tag;

typedef union NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_union_tag { /* Root complex 0 system bus write latency high watermark register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_tag;

typedef struct NETC_F1_GLOBAL_ARRAY_NUM_RC_struct_tag {
  NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_tag RCSBRLAR; /* Root complex 0 system bus read latency average register */
  NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_tag RCSBRLHWMR; /* Root complex 0 system bus read latency high watermark register */
  NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_tag RCSBWLAR; /* Root complex 0 system bus write latency average register */
  NETC_F1_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_tag RCSBWLHWMR; /* Root complex 0 system bus write latency high watermark register */
} NETC_F1_GLOBAL_ARRAY_NUM_RC_tag;

typedef union NETC_F1_GLOBAL_IPBRR0_union_tag { /* IP block revision register 0 */
  vuint32_t R;
  struct {
    vuint32_t IP_MN:8;
    vuint32_t IP_MJ:8;
    vuint32_t IP_ID:16;
  } B;
} NETC_F1_GLOBAL_IPBRR0_tag;

typedef union NETC_F1_GLOBAL_IPBRR1_union_tag { /* IP block revision register 1 */
  vuint32_t R;
  struct {
    vuint32_t IP_CFG:8;
    vuint32_t IP_MNT:8;
    vuint32_t IP_INT:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_GLOBAL_IPBRR1_tag;

typedef union NETC_F1_GLOBAL_FBLPR_union_tag { /* Function boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F1_GLOBAL_FBLPR_tag;

typedef union NETC_F1_GLOBAL_EMDIO_EMDIOUFSBECR_union_tag { /* EMDIO uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F1_GLOBAL_EMDIO_EMDIOUFSBECR_tag;

typedef union NETC_F1_GLOBAL_EMDIO_EMDIOUFSBESR_union_tag { /* EMDIO uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F1_GLOBAL_EMDIO_EMDIOUFSBESR_tag;

typedef union NETC_F1_GLOBAL_EMDIO_EMDIOUNIECR_union_tag { /* EMDIO uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F1_GLOBAL_EMDIO_EMDIOUNIECR_tag;

typedef union NETC_F1_GLOBAL_EMDIO_EMDIOUNIESR_union_tag { /* EMDIO uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F1_GLOBAL_EMDIO_EMDIOUNIESR_tag;

typedef union NETC_F1_GLOBAL_EMDIO_EMDIOUNIECTR_union_tag { /* EMDIO uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F1_GLOBAL_EMDIO_EMDIOUNIECTR_tag;

typedef struct NETC_F1_GLOBAL_EMDIO_struct_tag {
  NETC_F1_GLOBAL_EMDIO_EMDIOUFSBECR_tag EMDIOUFSBECR; /* EMDIO uncorrectable fatal system bus error configuration register */
  NETC_F1_GLOBAL_EMDIO_EMDIOUFSBESR_tag EMDIOUFSBESR; /* EMDIO uncorrectable fatal system bus error status register */
  uint8_t EMDIO_reserved0[40];
  NETC_F1_GLOBAL_EMDIO_EMDIOUNIECR_tag EMDIOUNIECR; /* EMDIO uncorrectable non-fatal integrity error configuration register */
  NETC_F1_GLOBAL_EMDIO_EMDIOUNIESR_tag EMDIOUNIESR; /* EMDIO uncorrectable non-fatal integrity error status register */
  uint8_t EMDIO_reserved1[4];
  NETC_F1_GLOBAL_EMDIO_EMDIOUNIECTR_tag EMDIOUNIECTR; /* EMDIO uncorrectable non-fatal integrity error count register */
} NETC_F1_GLOBAL_EMDIO_tag;

struct NETC_F1_GLOBAL_tag {
  NETC_F1_GLOBAL_SMCAPR_tag SMCAPR;    /* Shared memory capability register */
  NETC_F1_GLOBAL_SMDTR_tag SMDTR;      /* Shared memory depletion threshold register */
  NETC_F1_GLOBAL_SMACR_tag SMACR;      /* Shared memory available count register */
  uint8_t NETC_F1_GLOBAL_reserved0[4];
  NETC_F1_GLOBAL_SMCLWMR_tag SMCLWMR;  /* Shared memory count low watermark register */
  NETC_F1_GLOBAL_SMBUCR_tag SMBUCR;    /* Shared memory buffer unassigned count register */
  NETC_F1_GLOBAL_SMBUCHWMR_tag SMBUCHWMR; /* Shared memory buffer unassigned count high watermark register */
  NETC_F1_GLOBAL_SMLCR_tag SMLCR;      /* Shared memory loss count register */
  NETC_F1_GLOBAL_HBTCAPR_tag HBTCAPR;  /* Hash bucket table capability register */
  NETC_F1_GLOBAL_HBTOR0_tag HBTOR0;    /* Hash bucket table operational register 0 */
  uint8_t NETC_F1_GLOBAL_reserved1[4];
  NETC_F1_GLOBAL_HBTOR2_tag HBTOR2;    /* Hash bucket table operational register 2 */
  uint8_t NETC_F1_GLOBAL_reserved2[16];
  NETC_F1_GLOBAL_SMERBCAPR_tag SMERBCAPR; /* Shared memory ENETC receive buffer capability register */
  NETC_F1_GLOBAL_SMERBOR0_tag SMERBOR0; /* Shared memory ENETC receive buffer operational register 0 */
  NETC_F1_GLOBAL_SMERBOR1_tag SMERBOR1; /* Shared memory ENETC receive buffer operational 1 */
  uint8_t NETC_F1_GLOBAL_reserved3[180];
  NETC_F1_GLOBAL_PCE_SL_tag PCE_SL[1];
  uint8_t NETC_F1_GLOBAL_reserved4[92];
  NETC_F1_GLOBAL_NETCCLKR_tag NETCCLKR; /* NETC clock register */
  uint8_t NETC_F1_GLOBAL_reserved5[152];
  NETC_F1_GLOBAL_HTA_LOOP_tag HTA_LOOP[1];
  uint8_t NETC_F1_GLOBAL_reserved6[216];
  NETC_F1_GLOBAL_ARRAY_NUM_RC_tag ARRAY_NUM_RC[1];
  uint8_t NETC_F1_GLOBAL_reserved7[2280];
  NETC_F1_GLOBAL_IPBRR0_tag IPBRR0;    /* IP block revision register 0 */
  NETC_F1_GLOBAL_IPBRR1_tag IPBRR1;    /* IP block revision register 1 */
  uint8_t NETC_F1_GLOBAL_reserved8[256];
  NETC_F1_GLOBAL_FBLPR_tag FBLPR[2];   /* Function boot loader parameter register 0 */
  uint8_t NETC_F1_GLOBAL_reserved9[280];
  NETC_F1_GLOBAL_EMDIO_tag EMDIO;
};


/* ============================================================================
   =============================== Module: NETC_F1_PCI_HDR_TYPE0 ==============
   ============================================================================ */

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t MEM_ACCESS:1;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_ID:16;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_union_tag { /* PCI PCI-PM capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_union_tag { /* PCI PCI-PM capabilities register */
  vuint16_t R;
  struct {
    vuint16_t VERSION:3;
    vuint16_t _unused_3:8;
    vuint16_t PME_SUPPORT:5;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_union_tag { /* PCI PCI-PM control and status register */
  vuint16_t R;
  struct {
    vuint16_t PWR_STATE:2;
    vuint16_t _unused_2:1;
    vuint16_t NO_SOFT_RST:1;
    vuint16_t _unused_4:12;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_union_tag { /* PCI PCI-PM capabilities data register */
  vuint8_t R;
  struct {
    vuint8_t _unused_0:8;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_union_tag { /* PCI EA capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_union_tag { /* PCI EA capabilities register */
  vuint16_t R;
  struct {
    vuint16_t NUM_ENTRIES:6;
    vuint16_t _unused_6:10;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_union_tag { /* PCI EA per-entry 0 format register */
  vuint32_t R;
  struct {
    vuint32_t ENTRY_SIZE:3;
    vuint32_t _unused_3:1;
    vuint32_t BEI:4;
    vuint32_t PRIM_PROP:8;
    vuint32_t SEC_PROP:8;
    vuint32_t _unused_24:6;
    vuint32_t WRITABLE:1;
    vuint32_t ENABLE:1;
  } B;
} NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_union_tag { /* PCI EA per-entry 0 base register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t S:1;
    vuint32_t BASE:30;
  } B;
} NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_union_tag { /* PCI EA per-entry 0 max offset register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t S:1;
    vuint32_t MAX_OFFSET:30;
  } B;
} NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_union_tag { /* PCI EA per-entry 0 extended base register */
  vuint32_t R;
  struct {
    vuint32_t BASE:32;
  } B;
} NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_tag;

typedef struct NETC_F1_PCI_HDR_TYPE0_NUM_EA_struct_tag {
  NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_tag PCI_CFC_EA_PE_FMT; /* PCI EA per-entry 0 format register */
  NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_tag PCI_CFC_EA_PE_BASE; /* PCI EA per-entry 0 base register */
  NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_tag PCI_CFC_EA_PE_MAXOFF; /* PCI EA per-entry 0 max offset register */
  NETC_F1_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_tag PCI_CFC_EA_PE_EXT_BASE; /* PCI EA per-entry 0 extended base register */
} NETC_F1_PCI_HDR_TYPE0_NUM_EA_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:21;
    vuint32_t ACS_VIOLATION_STAT:1;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_F1_PCI_HDR_TYPE0_tag {
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved0[1];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved1[1];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved2[4];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved3[4];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved4[11];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved5[24];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved6[22];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved7[4];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_tag PCI_CFC_PCIPM_CAP_LIST; /* PCI PCI-PM capabilities list register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_tag PCI_CFC_PCIPM_CAP; /* PCI PCI-PM capabilities register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_tag PCI_CFC_PCIPM_CTL_STAT; /* PCI PCI-PM control and status register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved8[1];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_tag PCI_CFC_PCIPM_DATA; /* PCI PCI-PM capabilities data register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved9[4];
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_tag PCI_CFC_EA_CAP_LIST; /* PCI EA capabilities list register */
  NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_tag PCI_CFC_EA_CAP; /* PCI EA capabilities register */
  NETC_F1_PCI_HDR_TYPE0_NUM_EA_tag NUM_EA[2];
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved10[64];
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved11[20];
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_F1_PCI_HDR_TYPE0_reserved12[8];
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_F2 ============================
   ============================================================================ */

typedef union NETC_F2_SCAPR0_union_tag { /* Switch capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_PORT:5;
    vuint32_t _unused_5:3;
    vuint32_t NUM_IPV:1;
    vuint32_t _unused_9:7;
    vuint32_t NUM_MSIX:4;
    vuint32_t NUM_PCPMP:4;
    vuint32_t NUM_QVMP:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F2_SCAPR0_tag;

typedef union NETC_F2_SCAPR1_union_tag { /* Switch capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t FS:1;                    /* Functional safety capability supported. */
    vuint32_t _unused_1:1;
    vuint32_t CTF:1;
    vuint32_t TIMCAP:1;
    vuint32_t IMIR:1;
    vuint32_t _unused_5:11;
    vuint32_t SQ_TAGS:5;
    vuint32_t _unused_21:11;
  } B;
} NETC_F2_SCAPR1_tag;

typedef union NETC_F2_BPCAPR_union_tag { /* Buffer pool capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_BP:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_SPB:5;
    vuint32_t _unused_21:11;
  } B;
} NETC_F2_BPCAPR_tag;

typedef union NETC_F2_FCAPR_union_tag { /* Forwarding capability register */
  vuint32_t R;
  struct {
    vuint32_t BR:1;                    /* 802.1Q bridge forwarding support. */
    vuint32_t SF:1;                    /* Stream forwarding supported */
    vuint32_t _unused_2:30;
  } B;
} NETC_F2_FCAPR_tag;

typedef union NETC_F2_SMBCAPR_union_tag { /* Shared memory buffer capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:24;
    vuint32_t _unused_24:4;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;                  /* Indicates memory location */
  } B;
} NETC_F2_SMBCAPR_tag;

typedef union NETC_F2_SMBOR0_union_tag { /* Shared memory buffer operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_SMBOR0_tag;

typedef union NETC_F2_SMBOR1_union_tag { /* Shared memory buffer operational register 1 */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_SMBOR1_tag;

typedef union NETC_F2_CCAR_union_tag { /* Command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F2_CCAR_tag;

typedef union NETC_F2_MPCR_union_tag { /* Management port configuration register */
  vuint32_t R;
  struct {
    vuint32_t PORT:5;                  /* Switch Management Port */
    vuint32_t _unused_5:27;
  } B;
} NETC_F2_MPCR_tag;

typedef union NETC_F2_IMDCR0_union_tag { /* Ingress mirror destination configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t MIREN:1;                 /* Mirror enable. */
    vuint32_t MIRDEST:1;               /* Mirror destination */
    vuint32_t IPV:3;                   /* Internal Priority Value (IPV) */
    vuint32_t _unused_5:1;
    vuint32_t DR:2;                    /* Drop Resilience (DR) */
    vuint32_t PORT:5;                  /* Port number. */
    vuint32_t _unused_13:19;
  } B;
} NETC_F2_IMDCR0_tag;

typedef union NETC_F2_IMDCR1_union_tag { /* Ingress mirror destination configuration register 1 */
  vuint32_t R;
  struct {
    vuint32_t EFMEID:16;
    vuint32_t EFM_LEN_CHANGE:7;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_IMDCR1_tag;

typedef union NETC_F2_CTFCR_union_tag { /* Cut-through forwarding count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F2_CTFCR_tag;

typedef union NETC_F2_NUM_CBDR_CBDRMR_union_tag { /* Command BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;                    /* Enable command buffer descriptor ring */
  } B;
} NETC_F2_NUM_CBDR_CBDRMR_tag;

typedef union NETC_F2_NUM_CBDR_CBDRSR_union_tag { /* Command BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;                  /* Busy. */
    vuint32_t _unused_1:31;
  } B;
} NETC_F2_NUM_CBDR_CBDRSR_tag;

typedef union NETC_F2_NUM_CBDR_CBDRBAR0_union_tag { /* Command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F2_NUM_CBDR_CBDRBAR0_tag;

typedef union NETC_F2_NUM_CBDR_CBDRBAR1_union_tag { /* Command BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F2_NUM_CBDR_CBDRBAR1_tag;

typedef union NETC_F2_NUM_CBDR_CBDRPIR_union_tag { /* Command BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F2_NUM_CBDR_CBDRPIR_tag;

typedef union NETC_F2_NUM_CBDR_CBDRCIR_union_tag { /* Command BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:21;
    vuint32_t _unused_31:1;
  } B;
} NETC_F2_NUM_CBDR_CBDRCIR_tag;

typedef union NETC_F2_NUM_CBDR_CBDRLENR_union_tag { /* Command BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F2_NUM_CBDR_CBDRLENR_tag;

typedef struct NETC_F2_NUM_CBDR_struct_tag {
  NETC_F2_NUM_CBDR_CBDRMR_tag CBDRMR;  /* Command BDR 0 mode register */
  NETC_F2_NUM_CBDR_CBDRSR_tag CBDRSR;  /* Command BDR 0 status register */
  uint8_t NUM_CBDR_reserved0[8];
  NETC_F2_NUM_CBDR_CBDRBAR0_tag CBDRBAR0; /* Command BDR base address register 0 */
  NETC_F2_NUM_CBDR_CBDRBAR1_tag CBDRBAR1; /* Command BDR 0 base address register 1 */
  NETC_F2_NUM_CBDR_CBDRPIR_tag CBDRPIR; /* Command BDR 0 producer index register */
  NETC_F2_NUM_CBDR_CBDRCIR_tag CBDRCIR; /* Command BDR 0 consumer index register */
  NETC_F2_NUM_CBDR_CBDRLENR_tag CBDRLENR; /* Command BDR 0 length register */
  uint8_t NUM_CBDR_reserved1[12];
} NETC_F2_NUM_CBDR_tag;

typedef union NETC_F2_NUM_CBDR_INT_CBDRIER_union_tag { /* Command BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F2_NUM_CBDR_INT_CBDRIER_tag;

typedef union NETC_F2_NUM_CBDR_INT_CBDRIDR_union_tag { /* Command BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F2_NUM_CBDR_INT_CBDRIDR_tag;

typedef union NETC_F2_NUM_CBDR_INT_CBDRMSIVR_union_tag { /* Command BDR 0 MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F2_NUM_CBDR_INT_CBDRMSIVR_tag;

typedef struct NETC_F2_NUM_CBDR_INT_struct_tag {
  NETC_F2_NUM_CBDR_INT_CBDRIER_tag CBDRIER; /* Command BDR 0 interrupt enable register */
  NETC_F2_NUM_CBDR_INT_CBDRIDR_tag CBDRIDR; /* Command BDR 0 interrupt detect register */
  NETC_F2_NUM_CBDR_INT_CBDRMSIVR_tag CBDRMSIVR; /* Command BDR 0 MSI-X vector register */
  uint8_t NUM_CBDR_INT_reserved0[4];
} NETC_F2_NUM_CBDR_INT_tag;

typedef union NETC_F2_QOSVLANMP0R0_union_tag { /* QoS to VLAN mapping profile 0 register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0_DR0:4;
    vuint32_t IPV0_DR1:4;
    vuint32_t IPV0_DR2:4;
    vuint32_t IPV0_DR3:4;
    vuint32_t IPV1_DR0:4;
    vuint32_t IPV1_DR1:4;
    vuint32_t IPV1_DR2:4;
    vuint32_t IPV1_DR3:4;
  } B;
} NETC_F2_QOSVLANMP0R0_tag;

typedef union NETC_F2_QOSVLANMP0R1_union_tag { /* QoS to VLAN mapping profile 0 register 1 */
  vuint32_t R;
  struct {
    vuint32_t IPV2_DR0:4;
    vuint32_t IPV2_DR1:4;
    vuint32_t IPV2_DR2:4;
    vuint32_t IPV2_DR3:4;
    vuint32_t IPV3_DR0:4;
    vuint32_t IPV3_DR1:4;
    vuint32_t IPV3_DR2:4;
    vuint32_t IPV3_DR3:4;
  } B;
} NETC_F2_QOSVLANMP0R1_tag;

typedef union NETC_F2_QOSVLANMP0R2_union_tag { /* QoS to VLAN mapping profile 0 register 2 */
  vuint32_t R;
  struct {
    vuint32_t IPV4_DR0:4;
    vuint32_t IPV4_DR1:4;
    vuint32_t IPV4_DR2:4;
    vuint32_t IPV4_DR3:4;
    vuint32_t IPV5_DR0:4;
    vuint32_t IPV5_DR1:4;
    vuint32_t IPV5_DR2:4;
    vuint32_t IPV5_DR3:4;
  } B;
} NETC_F2_QOSVLANMP0R2_tag;

typedef union NETC_F2_QOSVLANMP0R3_union_tag { /* QoS to VLAN mapping profile 0 register 3 */
  vuint32_t R;
  struct {
    vuint32_t IPV6_DR0:4;
    vuint32_t IPV6_DR1:4;
    vuint32_t IPV6_DR2:4;
    vuint32_t IPV6_DR3:4;
    vuint32_t IPV7_DR0:4;
    vuint32_t IPV7_DR1:4;
    vuint32_t IPV7_DR2:4;
    vuint32_t IPV7_DR3:4;
  } B;
} NETC_F2_QOSVLANMP0R3_tag;

typedef union NETC_F2_QOSVLANMP1R0_union_tag { /* QoS to VLAN mapping profile 1 register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0_DR0:4;
    vuint32_t IPV0_DR1:4;
    vuint32_t IPV0_DR2:4;
    vuint32_t IPV0_DR3:4;
    vuint32_t IPV1_DR0:4;
    vuint32_t IPV1_DR1:4;
    vuint32_t IPV1_DR2:4;
    vuint32_t IPV1_DR3:4;
  } B;
} NETC_F2_QOSVLANMP1R0_tag;

typedef union NETC_F2_QOSVLANMP1R1_union_tag { /* QoS to VLAN mapping profile 1 register 1 */
  vuint32_t R;
  struct {
    vuint32_t IPV2_DR0:4;
    vuint32_t IPV2_DR1:4;
    vuint32_t IPV2_DR2:4;
    vuint32_t IPV2_DR3:4;
    vuint32_t IPV3_DR0:4;
    vuint32_t IPV3_DR1:4;
    vuint32_t IPV3_DR2:4;
    vuint32_t IPV3_DR3:4;
  } B;
} NETC_F2_QOSVLANMP1R1_tag;

typedef union NETC_F2_QOSVLANMP1R2_union_tag { /* QoS to VLAN mapping profile 1 register 2 */
  vuint32_t R;
  struct {
    vuint32_t IPV4_DR0:4;
    vuint32_t IPV4_DR1:4;
    vuint32_t IPV4_DR2:4;
    vuint32_t IPV4_DR3:4;
    vuint32_t IPV5_DR0:4;
    vuint32_t IPV5_DR1:4;
    vuint32_t IPV5_DR2:4;
    vuint32_t IPV5_DR3:4;
  } B;
} NETC_F2_QOSVLANMP1R2_tag;

typedef union NETC_F2_QOSVLANMP1R3_union_tag { /* QoS to VLAN mapping profile 1 register 3 */
  vuint32_t R;
  struct {
    vuint32_t IPV6_DR0:4;
    vuint32_t IPV6_DR1:4;
    vuint32_t IPV6_DR2:4;
    vuint32_t IPV6_DR3:4;
    vuint32_t IPV7_DR0:4;
    vuint32_t IPV7_DR1:4;
    vuint32_t IPV7_DR2:4;
    vuint32_t IPV7_DR3:4;
  } B;
} NETC_F2_QOSVLANMP1R3_tag;

typedef union NETC_F2_PCP2PCPMPR_union_tag { /* PCP to PCP mapping profile 0 register */
  vuint32_t R;
  struct {
    vuint32_t PCP0:3;
    vuint32_t _unused_3:1;
    vuint32_t PCP1:3;
    vuint32_t _unused_7:1;
    vuint32_t PCP2:3;
    vuint32_t _unused_11:1;
    vuint32_t PCP3:3;
    vuint32_t _unused_15:1;
    vuint32_t PCP4:3;
    vuint32_t _unused_19:1;
    vuint32_t PCP5:3;
    vuint32_t _unused_23:1;
    vuint32_t PCP6:3;
    vuint32_t _unused_27:1;
    vuint32_t PCP7:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F2_PCP2PCPMPR_tag;

typedef union NETC_F2_BRCAPR_union_tag { /* Bridge capability register */
  vuint32_t R;
  struct {
    vuint32_t IPV4MFLT:1;              /* L2 IPv4 multicast filtering supported. */
    vuint32_t _unused_1:1;
    vuint32_t STAMVD:1;                /* Station Move Disable supported */
    vuint32_t STRMCTRL:1;              /* Storm Control supported. */
    vuint32_t SRCPPRND:1;              /* Source port pruning disable supported */
    vuint32_t EVLANXLATE:1;            /* Egress VLAN translation supported */
    vuint32_t _unused_6:6;
    vuint32_t NUM_STG:2;
    vuint32_t _unused_14:18;
  } B;
} NETC_F2_BRCAPR_tag;

typedef union NETC_F2_VFHTCAPR_union_tag { /* VLAN filter hash table capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:20;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_VFHTCAPR_tag;

typedef union NETC_F2_VFHTOR_union_tag { /* VLAN filter hash table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:12;
    vuint32_t _unused_12:20;
  } B;
} NETC_F2_VFHTOR_tag;

typedef union NETC_F2_VFHTDECR0_union_tag { /* VLAN Filter (hash) table default entry configuration registers 0 */
  vuint32_t R;
  struct {
    vuint32_t PORT0:1;                 /* Port n */
    vuint32_t PORT1:1;                 /* Port n */
    vuint32_t PORT2:1;                 /* Port n */
    vuint32_t _unused_3:21;
    vuint32_t STG_ID:4;
    vuint32_t _unused_28:1;
    vuint32_t IPMFE:1;                 /* IP Multicast Filtering Enable */
    vuint32_t IPMFLE:1;                /* IP Multicast Flooding Enable */
    vuint32_t _unused_31:1;
  } B;
} NETC_F2_VFHTDECR0_tag;

typedef union NETC_F2_VFHTDECR1_union_tag { /* VLAN filter hash table default entry configuration registers 1 */
  vuint32_t R;
  struct {
    vuint32_t FID:12;
    vuint32_t VL_MODE:1;
    vuint32_t _unused_13:3;
    vuint32_t BASE_ETEID:16;
  } B;
} NETC_F2_VFHTDECR1_tag;

typedef union NETC_F2_VFHTDECR2_union_tag { /* VLAN filter hash table default entry configuration registers 2 */
  vuint32_t R;
  struct {
    vuint32_t ET_PORT0:1;              /* Egress Treatment Applicability Port n. */
    vuint32_t ET_PORT1:1;              /* Egress Treatment Applicability Port n. */
    vuint32_t ET_PORT2:1;              /* Egress Treatment Applicability Port n. */
    vuint32_t _unused_3:21;
    vuint32_t MLO:3;
    vuint32_t MFO:2;
    vuint32_t _unused_29:3;
  } B;
} NETC_F2_VFHTDECR2_tag;

typedef union NETC_F2_FDBHTCAPR_union_tag { /* FDB hash table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GMAC:9;
    vuint32_t _unused_9:11;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_FDBHTCAPR_tag;

typedef union NETC_F2_FDBHTMCR_union_tag { /* FDB hash table memory configuration register */
  vuint32_t R;
  struct {
    vuint32_t DYN_LIMIT:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_FDBHTMCR_tag;

typedef union NETC_F2_FDBHTOR0_union_tag { /* FDB hash table operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t STATIC_ENTRIES:16;
    vuint32_t NUM_GENTRIES:9;
    vuint32_t _unused_25:7;
  } B;
} NETC_F2_FDBHTOR0_tag;

typedef union NETC_F2_FDBHTOR1_union_tag { /* FDB hash table operational register 1 */
  vuint32_t R;
  struct {
    vuint32_t DYN_ENTRIES:16;
    vuint32_t HWM_DYN_ENTRIES:16;
  } B;
} NETC_F2_FDBHTOR1_tag;

typedef union NETC_F2_IPMFHTCAPR_union_tag { /* IP multicast filter hash table capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:20;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_IPMFHTCAPR_tag;

typedef union NETC_F2_IPV4MFHTOR_union_tag { /* IPv4 multicast filter hash table operation register */
  vuint32_t R;
  struct {
    vuint32_t ASM_ENTRIES:16;
    vuint32_t SSM_ENTRIES:16;
  } B;
} NETC_F2_IPV4MFHTOR_tag;

struct NETC_F2_tag {
  NETC_F2_SCAPR0_tag SCAPR0;           /* Switch capability register 0 */
  NETC_F2_SCAPR1_tag SCAPR1;           /* Switch capability register 1 */
  NETC_F2_BPCAPR_tag BPCAPR;           /* Buffer pool capability register */
  uint8_t NETC_F2_reserved0[12];
  NETC_F2_FCAPR_tag FCAPR;             /* Forwarding capability register */
  uint8_t NETC_F2_reserved1[36];
  NETC_F2_SMBCAPR_tag SMBCAPR;         /* Shared memory buffer capability register */
  NETC_F2_SMBOR0_tag SMBOR0;           /* Shared memory buffer operational register 0 */
  NETC_F2_SMBOR1_tag SMBOR1;           /* Shared memory buffer operational register 1 */
  uint8_t NETC_F2_reserved2[52];
  NETC_F2_CCAR_tag CCAR;               /* Command cache attribute register */
  uint8_t NETC_F2_reserved3[892];
  NETC_F2_MPCR_tag MPCR;               /* Management port configuration register */
  uint8_t NETC_F2_reserved4[28];
  NETC_F2_IMDCR0_tag IMDCR0;           /* Ingress mirror destination configuration register 0 */
  NETC_F2_IMDCR1_tag IMDCR1;           /* Ingress mirror destination configuration register 1 */
  uint8_t NETC_F2_reserved5[24];
  NETC_F2_CTFCR_tag CTFCR;             /* Cut-through forwarding count register */
  uint8_t NETC_F2_reserved6[956];
  NETC_F2_NUM_CBDR_tag NUM_CBDR[2];
  uint8_t NETC_F2_reserved7[64];
  NETC_F2_NUM_CBDR_INT_tag NUM_CBDR_INT[2];
  uint8_t NETC_F2_reserved8[64];
  NETC_F2_QOSVLANMP0R0_tag QOSVLANMP0R0; /* QoS to VLAN mapping profile 0 register 0 */
  NETC_F2_QOSVLANMP0R1_tag QOSVLANMP0R1; /* QoS to VLAN mapping profile 0 register 1 */
  NETC_F2_QOSVLANMP0R2_tag QOSVLANMP0R2; /* QoS to VLAN mapping profile 0 register 2 */
  NETC_F2_QOSVLANMP0R3_tag QOSVLANMP0R3; /* QoS to VLAN mapping profile 0 register 3 */
  uint8_t NETC_F2_reserved9[16];
  NETC_F2_QOSVLANMP1R0_tag QOSVLANMP1R0; /* QoS to VLAN mapping profile 1 register 0 */
  NETC_F2_QOSVLANMP1R1_tag QOSVLANMP1R1; /* QoS to VLAN mapping profile 1 register 1 */
  NETC_F2_QOSVLANMP1R2_tag QOSVLANMP1R2; /* QoS to VLAN mapping profile 1 register 2 */
  NETC_F2_QOSVLANMP1R3_tag QOSVLANMP1R3; /* QoS to VLAN mapping profile 1 register 3 */
  uint8_t NETC_F2_reserved10[464];
  NETC_F2_PCP2PCPMPR_tag PCP2PCPMPR[2]; /* PCP to PCP mapping profile 0 register */
  uint8_t NETC_F2_reserved11[5368];
  NETC_F2_BRCAPR_tag BRCAPR;           /* Bridge capability register */
  uint8_t NETC_F2_reserved12[4];
  NETC_F2_VFHTCAPR_tag VFHTCAPR;       /* VLAN filter hash table capability register */
  NETC_F2_VFHTOR_tag VFHTOR;           /* VLAN filter hash table operational register */
  NETC_F2_VFHTDECR0_tag VFHTDECR0;     /* VLAN Filter (hash) table default entry configuration registers 0 */
  NETC_F2_VFHTDECR1_tag VFHTDECR1;     /* VLAN filter hash table default entry configuration registers 1 */
  NETC_F2_VFHTDECR2_tag VFHTDECR2;     /* VLAN filter hash table default entry configuration registers 2 */
  uint8_t NETC_F2_reserved13[4];
  NETC_F2_FDBHTCAPR_tag FDBHTCAPR;     /* FDB hash table capability register */
  NETC_F2_FDBHTMCR_tag FDBHTMCR;       /* FDB hash table memory configuration register */
  NETC_F2_FDBHTOR0_tag FDBHTOR0;       /* FDB hash table operational register 0 */
  NETC_F2_FDBHTOR1_tag FDBHTOR1;       /* FDB hash table operational register 1 */
  uint8_t NETC_F2_reserved14[16];
  NETC_F2_IPMFHTCAPR_tag IPMFHTCAPR;   /* IP multicast filter hash table capability register */
  NETC_F2_IPV4MFHTOR_tag IPV4MFHTOR;   /* IPv4 multicast filter hash table operation register */
};


/* ============================================================================
   =============================== Module: NETC_F2_COMMON =====================
   ============================================================================ */

typedef union NETC_F2_COMMON_IPCAPR_union_tag { /* Ingress port capability register */
  vuint32_t R;
  struct {
    vuint32_t RP:1;
    vuint32_t IPFLT:1;
    vuint32_t ISID:1;
    vuint32_t _unused_3:5;
    vuint32_t SDU:5;                   /* Indicates support for various PDU/SDUs (Protocol/Service Data Unit) definitions. */
    vuint32_t _unused_13:3;
    vuint32_t NUM_VQMP:4;
    vuint32_t _unused_20:12;
  } B;
} NETC_F2_COMMON_IPCAPR_tag;

typedef union NETC_F2_COMMON_EPCAPR_union_tag { /* Egress port capability register */
  vuint32_t R;
  struct {
    vuint32_t ET:1;
    vuint32_t _unused_1:7;
    vuint32_t SDU:5;                   /* Indicates support for various PDU/SDUs (Protocol/Service Data Unit) definitions. */
    vuint32_t _unused_13:3;
    vuint32_t NUM_QVMP:4;
    vuint32_t _unused_20:12;
  } B;
} NETC_F2_COMMON_EPCAPR_tag;

typedef union NETC_F2_COMMON_OSR_union_tag { /* Operational state register */
  vuint32_t R;
  struct {
    vuint32_t STATE:1;
    vuint32_t ITM_STATE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F2_COMMON_OSR_tag;

typedef union NETC_F2_COMMON_CMECR_union_tag { /* Correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F2_COMMON_CMECR_tag;

typedef union NETC_F2_COMMON_CMESR_union_tag { /* Correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F2_COMMON_CMESR_tag;

typedef union NETC_F2_COMMON_CMECTR_union_tag { /* Correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F2_COMMON_CMECTR_tag;

typedef union NETC_F2_COMMON_UNMACECR_union_tag { /* Uncorrectable non-fatal MAC error configuration register */
  vuint32_t R;
  struct {
    vuint32_t PORT0:1;                 /* Report disable port */
    vuint32_t PORT1:1;                 /* Report disable port */
    vuint32_t PORT2:1;                 /* Report disable port */
    vuint32_t _unused_3:29;
  } B;
} NETC_F2_COMMON_UNMACECR_tag;

typedef union NETC_F2_COMMON_UNMACESR_union_tag { /* Uncorrectable non-fatal MAC error status register */
  vuint32_t R;
  struct {
    vuint32_t PORT0:1;                 /* Port 0 MAC error */
    vuint32_t PORT1:1;                 /* Port 1 MAC error */
    vuint32_t PORT2:1;                 /* Port 2 MAC error */
    vuint32_t _unused_3:29;
  } B;
} NETC_F2_COMMON_UNMACESR_tag;

typedef union NETC_F2_COMMON_UNSBECR_union_tag { /* Uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F2_COMMON_UNSBECR_tag;

typedef union NETC_F2_COMMON_UNSBESR_union_tag { /* Uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F2_COMMON_UNSBESR_tag;

typedef union NETC_F2_COMMON_UNSBECTR_union_tag { /* Uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F2_COMMON_UNSBECTR_tag;

typedef union NETC_F2_COMMON_UFSBECR_union_tag { /* Uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F2_COMMON_UFSBECR_tag;

typedef union NETC_F2_COMMON_UFSBESR_union_tag { /* Uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F2_COMMON_UFSBESR_tag;

typedef union NETC_F2_COMMON_UNMECR_union_tag { /* Uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F2_COMMON_UNMECR_tag;

typedef union NETC_F2_COMMON_UNMESR0_union_tag { /* Uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F2_COMMON_UNMESR0_tag;

typedef union NETC_F2_COMMON_UNMESR1_union_tag { /* Uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F2_COMMON_UNMESR1_tag;

typedef union NETC_F2_COMMON_UNMECTR_union_tag { /* Uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F2_COMMON_UNMECTR_tag;

typedef union NETC_F2_COMMON_UFMECR_union_tag { /* Uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F2_COMMON_UFMECR_tag;

typedef union NETC_F2_COMMON_UFMESR0_union_tag { /* Uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F2_COMMON_UFMESR0_tag;

typedef union NETC_F2_COMMON_UFMESR1_union_tag { /* Uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F2_COMMON_UFMESR1_tag;

typedef union NETC_F2_COMMON_UNIECR_union_tag { /* Uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F2_COMMON_UNIECR_tag;

typedef union NETC_F2_COMMON_UNIESR_union_tag { /* Uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t LINK_SLICE_ID:4;         /* Link or Slice ID */
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:2;
    vuint32_t ENGINE_ID:1;             /* Engine ID */
    vuint32_t _unused_17:14;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F2_COMMON_UNIESR_tag;

typedef union NETC_F2_COMMON_UNIECTR_union_tag { /* Uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F2_COMMON_UNIECTR_tag;

typedef union NETC_F2_COMMON_UFIECR_union_tag { /* Uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F2_COMMON_UFIECR_tag;

typedef union NETC_F2_COMMON_UFIESR_union_tag { /* Uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t LINK_SLICE_ID:4;         /* Link or Slice ID */
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:2;
    vuint32_t ENGINE_ID:1;             /* Engine ID */
    vuint32_t _unused_17:13;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F2_COMMON_UFIESR_tag;

typedef union NETC_F2_COMMON_EMDIOIRR_union_tag { /* External MDIO interrupt reason register */
  vuint32_t R;
  struct {
    vuint32_t PORT0:1;
    vuint32_t PORT1:1;
    vuint32_t PORT2:1;
    vuint32_t _unused_3:29;
  } B;
} NETC_F2_COMMON_EMDIOIRR_tag;

typedef union NETC_F2_COMMON_EMDIOMSIVR_union_tag { /* External MDIO MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F2_COMMON_EMDIOMSIVR_tag;

typedef union NETC_F2_COMMON_TCCR_union_tag { /* Time capture configuration register */
  vuint32_t R;
  struct {
    vuint32_t TIMEOUT:24;
    vuint32_t _unused_24:6;
    vuint32_t ARM:2;
  } B;
} NETC_F2_COMMON_TCCR_tag;

typedef union NETC_F2_COMMON_TCIER_union_tag { /* Time capture interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t TRANSMIT:1;              /* Transmit interrupt */
    vuint32_t TIMEOUT:1;               /* Timeout interrupt */
  } B;
} NETC_F2_COMMON_TCIER_tag;

typedef union NETC_F2_COMMON_TCRPIDR_union_tag { /* Time capture receive port interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TX_PORT0:1;
    vuint32_t TX_PORT1:1;
    vuint32_t TX_PORT2:1;
    vuint32_t _unused_3:27;
    vuint32_t TRANSMIT:1;              /* Transmit interrupt */
    vuint32_t TIMEOUT:1;               /* Timeout interrupt */
  } B;
} NETC_F2_COMMON_TCRPIDR_tag;

typedef union NETC_F2_COMMON_TCRPSR_union_tag { /* Time capture receive port status register */
  vuint32_t R;
  struct {
    vuint32_t RX_PORT:5;
    vuint32_t _unused_5:3;
    vuint32_t RX_CNT:2;
    vuint32_t _unused_10:22;
  } B;
} NETC_F2_COMMON_TCRPSR_tag;

typedef union NETC_F2_COMMON_TCRPTSR_union_tag { /* Time capture receive port timestamp register */
  vuint32_t R;
  struct {
    vuint32_t TIMESTAMP:32;
  } B;
} NETC_F2_COMMON_TCRPTSR_tag;

typedef union NETC_F2_COMMON_TCMSIVR_union_tag { /* Time capture MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F2_COMMON_TCMSIVR_tag;

typedef union NETC_F2_COMMON_CVLANR1_union_tag { /* Custom VLAN EtherType register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;                     /* Valid */
  } B;
} NETC_F2_COMMON_CVLANR1_tag;

typedef union NETC_F2_COMMON_CVLANR2_union_tag { /* Custom VLAN EtherType register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;                     /* Valid */
  } B;
} NETC_F2_COMMON_CVLANR2_tag;

typedef union NETC_F2_COMMON_PSRTAGETR_union_tag { /* Pre-Standard RTAG EtherType register */
  vuint32_t R;
  struct {
    vuint32_t ETHERTYPE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_PSRTAGETR_tag;

typedef union NETC_F2_COMMON_DOSL2CR_union_tag { /* DoS L2 configuration register */
  vuint32_t R;
  struct {
    vuint32_t SAMEADDR:1;
    vuint32_t MSAMCC:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F2_COMMON_DOSL2CR_tag;

typedef union NETC_F2_COMMON_NUM_PROFILE_VLANIPVMPR0_union_tag { /* VLAN to IPV mapping profile 0 register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:3;
    vuint32_t _unused_3:1;
    vuint32_t PCP_DEI_1:3;
    vuint32_t _unused_7:1;
    vuint32_t PCP_DEI_2:3;
    vuint32_t _unused_11:1;
    vuint32_t PCP_DEI_3:3;
    vuint32_t _unused_15:1;
    vuint32_t PCP_DEI_4:3;
    vuint32_t _unused_19:1;
    vuint32_t PCP_DEI_5:3;
    vuint32_t _unused_23:1;
    vuint32_t PCP_DEI_6:3;
    vuint32_t _unused_27:1;
    vuint32_t PCP_DEI_7:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F2_COMMON_NUM_PROFILE_VLANIPVMPR0_tag;

typedef union NETC_F2_COMMON_NUM_PROFILE_VLANIPVMPR1_union_tag { /* VLAN to IPV mapping profile 0 register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:3;
    vuint32_t _unused_3:1;
    vuint32_t PCP_DEI_9:3;
    vuint32_t _unused_7:1;
    vuint32_t PCP_DEI_10:3;
    vuint32_t _unused_11:1;
    vuint32_t PCP_DEI_11:3;
    vuint32_t _unused_15:1;
    vuint32_t PCP_DEI_12:3;
    vuint32_t _unused_19:1;
    vuint32_t PCP_DEI_13:3;
    vuint32_t _unused_23:1;
    vuint32_t PCP_DEI_14:3;
    vuint32_t _unused_27:1;
    vuint32_t PCP_DEI_15:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F2_COMMON_NUM_PROFILE_VLANIPVMPR1_tag;

typedef union NETC_F2_COMMON_NUM_PROFILE_VLANDRMPR_union_tag { /* VLAN to DR mapping profile 0 register */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:2;
    vuint32_t PCP_DEI_1:2;
    vuint32_t PCP_DEI_2:2;
    vuint32_t PCP_DEI_3:2;
    vuint32_t PCP_DEI_4:2;
    vuint32_t PCP_DEI_5:2;
    vuint32_t PCP_DEI_6:2;
    vuint32_t PCP_DEI_7:2;
    vuint32_t PCP_DEI_8:2;
    vuint32_t PCP_DEI_9:2;
    vuint32_t PCP_DEI_10:2;
    vuint32_t PCP_DEI_11:2;
    vuint32_t PCP_DEI_12:2;
    vuint32_t PCP_DEI_13:2;
    vuint32_t PCP_DEI_14:2;
    vuint32_t PCP_DEI_15:2;
  } B;
} NETC_F2_COMMON_NUM_PROFILE_VLANDRMPR_tag;

typedef struct NETC_F2_COMMON_NUM_PROFILE_struct_tag {
  NETC_F2_COMMON_NUM_PROFILE_VLANIPVMPR0_tag VLANIPVMPR0; /* VLAN to IPV mapping profile 0 register 0 */
  NETC_F2_COMMON_NUM_PROFILE_VLANIPVMPR1_tag VLANIPVMPR1; /* VLAN to IPV mapping profile 0 register 1 */
  NETC_F2_COMMON_NUM_PROFILE_VLANDRMPR_tag VLANDRMPR; /* VLAN to DR mapping profile 0 register */
  uint8_t NUM_PROFILE_reserved0[4];
} NETC_F2_COMMON_NUM_PROFILE_tag;

typedef union NETC_F2_COMMON_IPFCAPR_union_tag { /* Ingress port filter capability register */
  vuint32_t R;
  struct {
    vuint32_t RP:1;                    /* Rate Policer function supported */
    vuint32_t ISID:1;                  /* Ingress Stream Identification supported. */
    vuint32_t _unused_2:30;
  } B;
} NETC_F2_COMMON_IPFCAPR_tag;

typedef union NETC_F2_COMMON_IPFTCAPR_union_tag { /* Ingress port filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t MGMT:1;
    vuint32_t _unused_17:3;
    vuint32_t ACCESS_METH:4;
    vuint32_t ENTRY_MAX_WORDS:4;
    vuint32_t WORD_SIZE:2;
    vuint32_t _unused_30:2;
  } B;
} NETC_F2_COMMON_IPFTCAPR_tag;

typedef union NETC_F2_COMMON_IPFTMOR_union_tag { /* Ingress port filter table memory operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_IPFTMOR_tag;

typedef union NETC_F2_COMMON_ITMCAPR_union_tag { /* Index table memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:12;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;
  } B;
} NETC_F2_COMMON_ITMCAPR_tag;

typedef union NETC_F2_COMMON_RPCAPR_union_tag { /* Rate policer capability register */
  vuint32_t R;
  struct {
    vuint32_t TRTCM:1;                 /* Two-Rate Three-Color Marker supported per MEF 10.3 standard. */
    vuint32_t CM:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F2_COMMON_RPCAPR_tag;

typedef union NETC_F2_COMMON_RPITCAPR_union_tag { /* Rate policer index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:14;
    vuint32_t _unused_14:6;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_RPITCAPR_tag;

typedef union NETC_F2_COMMON_RPITMAR_union_tag { /* Rate policer index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_RPITMAR_tag;

typedef union NETC_F2_COMMON_RPITOR_union_tag { /* Rate policer index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:14;
    vuint32_t _unused_14:18;
  } B;
} NETC_F2_COMMON_RPITOR_tag;

typedef union NETC_F2_COMMON_ISCITCAPR_union_tag { /* Ingress stream counter index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_ISCITCAPR_tag;

typedef union NETC_F2_COMMON_ISCITMAR_union_tag { /* Ingress stream counter index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_ISCITMAR_tag;

typedef union NETC_F2_COMMON_ISCITOR_union_tag { /* Ingress stream counter index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_ISCITOR_tag;

typedef union NETC_F2_COMMON_ISCAPR_union_tag { /* Ingress stream capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t ISQG:1;
    vuint32_t _unused_2:1;
    vuint32_t SG:1;
    vuint32_t RP:1;
    vuint32_t MAXSDU:1;
    vuint32_t _unused_6:3;
    vuint32_t FWD:1;
    vuint32_t ET:1;
    vuint32_t _unused_11:21;
  } B;
} NETC_F2_COMMON_ISCAPR_tag;

typedef union NETC_F2_COMMON_ISITCAPR_union_tag { /* Ingress stream index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_ISITCAPR_tag;

typedef union NETC_F2_COMMON_ISITMAR_union_tag { /* Ingress stream index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_ISITMAR_tag;

typedef union NETC_F2_COMMON_ISITOR_union_tag { /* Ingress stream index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_ISITOR_tag;

typedef union NETC_F2_COMMON_ISQGITCAPR_union_tag { /* Ingress sequence generation index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_ISQGITCAPR_tag;

typedef union NETC_F2_COMMON_ISQGITMAR_union_tag { /* Ingress sequence generation index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:13;
    vuint32_t _unused_13:19;
  } B;
} NETC_F2_COMMON_ISQGITMAR_tag;

typedef union NETC_F2_COMMON_ISQGITOR_union_tag { /* Ingress sequence generation index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_ISQGITOR_tag;

typedef union NETC_F2_COMMON_SGCAPR_union_tag { /* Stream gate capability register */
  vuint32_t R;
  struct {
    vuint32_t GLC_AO:1;
    vuint32_t GLC_GC:1;
    vuint32_t GLC_IO:1;
    vuint32_t GLC_IPV:1;
    vuint32_t GLC_CTD:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F2_COMMON_SGCAPR_tag;

typedef union NETC_F2_COMMON_SGIITCAPR_union_tag { /* Stream gate instance index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_SGIITCAPR_tag;

typedef union NETC_F2_COMMON_SGIITMAR_union_tag { /* Stream gate instance index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_SGIITMAR_tag;

typedef union NETC_F2_COMMON_SGIITOR_union_tag { /* Stream gate instance index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_SGIITOR_tag;

typedef union NETC_F2_COMMON_SGCLITCAPR_union_tag { /* Stream gate control list index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_SGCLITCAPR_tag;

typedef union NETC_F2_COMMON_SGCLITMAR_union_tag { /* Stream gate control list index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_SGCLITMAR_tag;

typedef union NETC_F2_COMMON_SGCLTMOR_union_tag { /* Stream gate control list table memory operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_SGCLTMOR_tag;

typedef union NETC_F2_COMMON_FMICAPR_union_tag { /* Frame modification ingress capability register */
  vuint32_t R;
  struct {
    vuint32_t L2_ACT:8;                /* Layer 2 frame modification actions supported */
    vuint32_t _unused_8:24;
  } B;
} NETC_F2_COMMON_FMICAPR_tag;

typedef union NETC_F2_COMMON_FMECAPR_union_tag { /* Frame modification egress capability register */
  vuint32_t R;
  struct {
    vuint32_t L2_ACT:8;                /* Layer 2 frame modification actions */
    vuint32_t _unused_8:8;
    vuint32_t L3_ACT:8;                /* Layer 3 frame modification actions */
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_FMECAPR_tag;

typedef union NETC_F2_COMMON_FMITCAPR_union_tag { /* Frame modification index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:13;
    vuint32_t _unused_13:7;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_FMITCAPR_tag;

typedef union NETC_F2_COMMON_FMITMAR_union_tag { /* Frame modification index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:13;
    vuint32_t _unused_13:19;
  } B;
} NETC_F2_COMMON_FMITMAR_tag;

typedef union NETC_F2_COMMON_FMITOR_union_tag { /* Frame modification index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:13;
    vuint32_t _unused_13:19;
  } B;
} NETC_F2_COMMON_FMITOR_tag;

typedef union NETC_F2_COMMON_FMDITCAPR_union_tag { /* Frame modification data index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_FMDITCAPR_tag;

typedef union NETC_F2_COMMON_FMDITMAR_union_tag { /* Frame modification data index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_FMDITMAR_tag;

typedef union NETC_F2_COMMON_ETCAPR_union_tag { /* Egress treatment capability register */
  vuint32_t R;
  struct {
    vuint32_t ESQR:1;                  /* Egress Sequence Recovery supported */
    vuint32_t _unused_1:31;
  } B;
} NETC_F2_COMMON_ETCAPR_tag;

typedef union NETC_F2_COMMON_ETTCAPR_union_tag { /* Egress treatment table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_ETTCAPR_tag;

typedef union NETC_F2_COMMON_ETTOR_union_tag { /* Egress treatment table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_ETTOR_tag;

typedef union NETC_F2_COMMON_TGSTCAPR_union_tag { /* Time gate scheduling table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;            /* Number of Words */
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;           /* Access Method */
    vuint32_t MAX_GCL_LEN:2;           /* Maximum Gate Control List Length */
    vuint32_t _unused_26:6;
  } B;
} NETC_F2_COMMON_TGSTCAPR_tag;

typedef union NETC_F2_COMMON_TGSTMOR_union_tag { /* Time gate scheduling table memory operation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_TGSTMOR_tag;

typedef union NETC_F2_COMMON_ESQRCAPR_union_tag { /* Egress sequence recovery capability register */
  vuint32_t R;
  struct {
    vuint32_t SQR_TYPE:2;
    vuint32_t SQR_ALG:2;
    vuint32_t _unused_4:4;
    vuint32_t SQR_MAX_HL:3;
    vuint32_t _unused_11:21;
  } B;
} NETC_F2_COMMON_ESQRCAPR_tag;

typedef union NETC_F2_COMMON_ESQRTCAPR_union_tag { /* Egress sequence recovery table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_ESQRTCAPR_tag;

typedef union NETC_F2_COMMON_ECTCAPR_union_tag { /* Egress counter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_ECTCAPR_tag;

typedef union NETC_F2_COMMON_HTMCAPR_union_tag { /* Hash table memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:12;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;
  } B;
} NETC_F2_COMMON_HTMCAPR_tag;

typedef union NETC_F2_COMMON_HTMOR_union_tag { /* Hash table memory operational register */
  vuint32_t R;
  struct {
    vuint32_t AMOUNT:16;
    vuint32_t WATERMARK:16;
  } B;
} NETC_F2_COMMON_HTMOR_tag;

typedef union NETC_F2_COMMON_ISIDCAPR_union_tag { /* Ingress stream identification capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_KC:2;
    vuint32_t NUM_PF:3;
    vuint32_t _unused_5:3;
    vuint32_t MAX_KSIZE:5;
    vuint32_t _unused_13:3;
    vuint32_t UFT:1;
    vuint32_t ETHFT:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F2_COMMON_ISIDCAPR_tag;

typedef union NETC_F2_COMMON_ISIDHTCAPR_union_tag { /* Ingress stream identification hash table capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:20;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_ISIDHTCAPR_tag;

typedef union NETC_F2_COMMON_ISIDKC0OR_union_tag { /* Ingress stream identification key construction 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:15;
    vuint32_t EN:1;
  } B;
} NETC_F2_COMMON_ISIDKC0OR_tag;

typedef union NETC_F2_COMMON_ISIDKC0CR0_union_tag { /* Ingress stream identification key construction 0 configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t VALID:1;                 /* Valid */
    vuint32_t PORTP:1;
    vuint32_t SPMP:1;                  /* Switch Port Masquerading (flag) Present */
    vuint32_t DMACP:1;                 /* Destination MAC (address) Present */
    vuint32_t SMACP:1;                 /* Source MAC (address) Present. */
    vuint32_t OVIDP:1;                 /* Outer VID Present */
    vuint32_t OPCPP:1;                 /* Outer PCP Present */
    vuint32_t IVIDP:1;                 /* Inner VID Present. */
    vuint32_t IPCPP:1;                 /* Inner PCP Present */
    vuint32_t SQTP:1;                  /* Sequence Tag (code point) Present. */
    vuint32_t ETP:1;                   /* EtherType Present. */
    vuint32_t _unused_11:21;
  } B;
} NETC_F2_COMMON_ISIDKC0CR0_tag;

typedef union NETC_F2_COMMON_ISIDKC0PF0CR_union_tag { /* Ingress stream identification key construction 0 payload field 0 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC0PF0CR_tag;

typedef union NETC_F2_COMMON_ISIDKC0PF1CR_union_tag { /* Ingress stream identification key construction 0 payload field 1 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC0PF1CR_tag;

typedef union NETC_F2_COMMON_ISIDKC0PF2CR_union_tag { /* Ingress stream identification key construction 0 payload field 2 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC0PF2CR_tag;

typedef union NETC_F2_COMMON_ISIDKC0PF3CR_union_tag { /* Ingress stream identification key construction 0 payload field 3 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC0PF3CR_tag;

typedef union NETC_F2_COMMON_ISIDKC1OR_union_tag { /* Ingress stream identification key construction 1 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:15;
    vuint32_t EN:1;
  } B;
} NETC_F2_COMMON_ISIDKC1OR_tag;

typedef union NETC_F2_COMMON_ISIDKC1CR0_union_tag { /* Ingress stream identification key construction 1 configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t VALID:1;                 /* Valid */
    vuint32_t PORTP:1;
    vuint32_t SPMP:1;                  /* Switch Port Masquerading (flag) Present */
    vuint32_t DMACP:1;                 /* Destination MAC (address) Present */
    vuint32_t SMACP:1;                 /* Source MAC (address) Present. */
    vuint32_t OVIDP:1;                 /* Outer VID Present */
    vuint32_t OPCPP:1;                 /* Outer PCP Present */
    vuint32_t IVIDP:1;                 /* Inner VID Present. */
    vuint32_t IPCPP:1;                 /* Inner PCP Present */
    vuint32_t SQTP:1;                  /* Sequence Tag (code point) Present. */
    vuint32_t ETP:1;                   /* EtherType Present. */
    vuint32_t _unused_11:21;
  } B;
} NETC_F2_COMMON_ISIDKC1CR0_tag;

typedef union NETC_F2_COMMON_ISIDKC1PF0CR_union_tag { /* Ingress stream identification key construction 1 payload field 0 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC1PF0CR_tag;

typedef union NETC_F2_COMMON_ISIDKC1PF1CR_union_tag { /* Ingress stream identification key construction 1 payload field 1 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC1PF1CR_tag;

typedef union NETC_F2_COMMON_ISIDKC1PF2CR_union_tag { /* Ingress stream identification key construction 1 payload field 2 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC1PF2CR_tag;

typedef union NETC_F2_COMMON_ISIDKC1PF3CR_union_tag { /* Ingress stream identification key construction 1 payload field 3 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC1PF3CR_tag;

typedef union NETC_F2_COMMON_ISIDKC2OR_union_tag { /* Ingress stream identification key construction 2 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:15;
    vuint32_t EN:1;
  } B;
} NETC_F2_COMMON_ISIDKC2OR_tag;

typedef union NETC_F2_COMMON_ISIDKC2CR0_union_tag { /* Ingress stream identification key construction 2 configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t VALID:1;                 /* Valid */
    vuint32_t PORTP:1;
    vuint32_t SPMP:1;                  /* Switch Port Masquerading (flag) Present */
    vuint32_t DMACP:1;                 /* Destination MAC (address) Present */
    vuint32_t SMACP:1;                 /* Source MAC (address) Present. */
    vuint32_t OVIDP:1;                 /* Outer VID Present */
    vuint32_t OPCPP:1;                 /* Outer PCP Present */
    vuint32_t IVIDP:1;                 /* Inner VID Present. */
    vuint32_t IPCPP:1;                 /* Inner PCP Present */
    vuint32_t SQTP:1;                  /* Sequence Tag (code point) Present. */
    vuint32_t ETP:1;                   /* EtherType Present. */
    vuint32_t _unused_11:21;
  } B;
} NETC_F2_COMMON_ISIDKC2CR0_tag;

typedef union NETC_F2_COMMON_ISIDKC2PF0CR_union_tag { /* Ingress stream identification key construction 2 payload field 0 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC2PF0CR_tag;

typedef union NETC_F2_COMMON_ISIDKC2PF1CR_union_tag { /* Ingress stream identification key construction 2 payload field 1 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC2PF1CR_tag;

typedef union NETC_F2_COMMON_ISIDKC2PF2CR_union_tag { /* Ingress stream identification key construction 2 payload field 2 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC2PF2CR_tag;

typedef union NETC_F2_COMMON_ISIDKC2PF3CR_union_tag { /* Ingress stream identification key construction 2 payload field 3 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC2PF3CR_tag;

typedef union NETC_F2_COMMON_ISIDKC3OR_union_tag { /* Ingress stream identification key construction 3 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:15;
    vuint32_t EN:1;
  } B;
} NETC_F2_COMMON_ISIDKC3OR_tag;

typedef union NETC_F2_COMMON_ISIDKC3CR0_union_tag { /* Ingress stream identification key construction 3 configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t VALID:1;                 /* Valid */
    vuint32_t PORTP:1;
    vuint32_t SPMP:1;                  /* Switch Port Masquerading (flag) Present */
    vuint32_t DMACP:1;                 /* Destination MAC (address) Present */
    vuint32_t SMACP:1;                 /* Source MAC (address) Present. */
    vuint32_t OVIDP:1;                 /* Outer VID Present */
    vuint32_t OPCPP:1;                 /* Outer PCP Present */
    vuint32_t IVIDP:1;                 /* Inner VID Present. */
    vuint32_t IPCPP:1;                 /* Inner PCP Present */
    vuint32_t SQTP:1;                  /* Sequence Tag (code point) Present. */
    vuint32_t ETP:1;                   /* EtherType Present. */
    vuint32_t _unused_11:21;
  } B;
} NETC_F2_COMMON_ISIDKC3CR0_tag;

typedef union NETC_F2_COMMON_ISIDKC3PF0CR_union_tag { /* Ingress stream identification key construction 3 payload field 0 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC3PF0CR_tag;

typedef union NETC_F2_COMMON_ISIDKC3PF1CR_union_tag { /* Ingress stream identification key construction 3 payload field 1 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC3PF1CR_tag;

typedef union NETC_F2_COMMON_ISIDKC3PF2CR_union_tag { /* Ingress stream identification key construction 3 payload field 2 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC3PF2CR_tag;

typedef union NETC_F2_COMMON_ISIDKC3PF3CR_union_tag { /* Ingress stream identification key construction 3 payload field 3 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_COMMON_ISIDKC3PF3CR_tag;

typedef union NETC_F2_COMMON_ISFHTCAPR_union_tag { /* Ingress stream filter hash table capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:20;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_COMMON_ISFHTCAPR_tag;

typedef union NETC_F2_COMMON_ISFHTOR_union_tag { /* Ingress stream filter hash table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_COMMON_ISFHTOR_tag;

struct NETC_F2_COMMON_tag {
  uint8_t NETC_F2_COMMON_reserved0[4096];
  NETC_F2_COMMON_IPCAPR_tag IPCAPR;    /* Ingress port capability register */
  NETC_F2_COMMON_EPCAPR_tag EPCAPR;    /* Egress port capability register */
  uint8_t NETC_F2_COMMON_reserved1[8];
  NETC_F2_COMMON_OSR_tag OSR;          /* Operational state register */
  uint8_t NETC_F2_COMMON_reserved2[44];
  NETC_F2_COMMON_CMECR_tag CMECR;      /* Correctable memory error configuration register */
  NETC_F2_COMMON_CMESR_tag CMESR;      /* Correctable memory error status register */
  uint8_t NETC_F2_COMMON_reserved3[4];
  NETC_F2_COMMON_CMECTR_tag CMECTR;    /* Correctable memory error count register */
  uint8_t NETC_F2_COMMON_reserved4[16];
  NETC_F2_COMMON_UNMACECR_tag UNMACECR; /* Uncorrectable non-fatal MAC error configuration register */
  NETC_F2_COMMON_UNMACESR_tag UNMACESR; /* Uncorrectable non-fatal MAC error status register */
  uint8_t NETC_F2_COMMON_reserved5[8];
  NETC_F2_COMMON_UNSBECR_tag UNSBECR;  /* Uncorrectable non-fatal system bus error configuration register */
  NETC_F2_COMMON_UNSBESR_tag UNSBESR;  /* Uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F2_COMMON_reserved6[4];
  NETC_F2_COMMON_UNSBECTR_tag UNSBECTR; /* Uncorrectable non-fatal system bus error count register */
  NETC_F2_COMMON_UFSBECR_tag UFSBECR;  /* Uncorrectable fatal system bus error configuration register */
  NETC_F2_COMMON_UFSBESR_tag UFSBESR;  /* Uncorrectable fatal system bus error status register */
  uint8_t NETC_F2_COMMON_reserved7[8];
  NETC_F2_COMMON_UNMECR_tag UNMECR;    /* Uncorrectable non-fatal memory error configuration register */
  NETC_F2_COMMON_UNMESR0_tag UNMESR0;  /* Uncorrectable non-fatal memory error status register 0 */
  NETC_F2_COMMON_UNMESR1_tag UNMESR1;  /* Uncorrectable non-fatal memory error status register 1 */
  NETC_F2_COMMON_UNMECTR_tag UNMECTR;  /* Uncorrectable non-fatal memory error count register */
  NETC_F2_COMMON_UFMECR_tag UFMECR;    /* Uncorrectable fatal memory error configuration register */
  NETC_F2_COMMON_UFMESR0_tag UFMESR0;  /* Uncorrectable fatal memory error status register 0 */
  NETC_F2_COMMON_UFMESR1_tag UFMESR1;  /* Uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F2_COMMON_reserved8[4];
  NETC_F2_COMMON_UNIECR_tag UNIECR;    /* Uncorrectable non-fatal integrity error configuration register */
  NETC_F2_COMMON_UNIESR_tag UNIESR;    /* Uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F2_COMMON_reserved9[4];
  NETC_F2_COMMON_UNIECTR_tag UNIECTR;  /* Uncorrectable non-fatal integrity error count register */
  NETC_F2_COMMON_UFIECR_tag UFIECR;    /* Uncorrectable fatal integrity error configuration register */
  NETC_F2_COMMON_UFIESR_tag UFIESR;    /* Uncorrectable fatal integrity error status register */
  uint8_t NETC_F2_COMMON_reserved10[32];
  NETC_F2_COMMON_EMDIOIRR_tag EMDIOIRR; /* External MDIO interrupt reason register */
  NETC_F2_COMMON_EMDIOMSIVR_tag EMDIOMSIVR; /* External MDIO MSI-X vector register */
  uint8_t NETC_F2_COMMON_reserved11[16];
  NETC_F2_COMMON_TCCR_tag TCCR;        /* Time capture configuration register */
  NETC_F2_COMMON_TCIER_tag TCIER;      /* Time capture interrupt enable register */
  NETC_F2_COMMON_TCRPIDR_tag TCRPIDR;  /* Time capture receive port interrupt detect register */
  NETC_F2_COMMON_TCRPSR_tag TCRPSR;    /* Time capture receive port status register */
  uint8_t NETC_F2_COMMON_reserved12[4];
  NETC_F2_COMMON_TCRPTSR_tag TCRPTSR;  /* Time capture receive port timestamp register */
  NETC_F2_COMMON_TCMSIVR_tag TCMSIVR;  /* Time capture MSI-X vector register */
  uint8_t NETC_F2_COMMON_reserved13[228];
  NETC_F2_COMMON_CVLANR1_tag CVLANR1;  /* Custom VLAN EtherType register 1 */
  NETC_F2_COMMON_CVLANR2_tag CVLANR2;  /* Custom VLAN EtherType register 2 */
  NETC_F2_COMMON_PSRTAGETR_tag PSRTAGETR; /* Pre-Standard RTAG EtherType register */
  uint8_t NETC_F2_COMMON_reserved14[20];
  NETC_F2_COMMON_DOSL2CR_tag DOSL2CR;  /* DoS L2 configuration register */
  uint8_t NETC_F2_COMMON_reserved15[220];
  NETC_F2_COMMON_NUM_PROFILE_tag NUM_PROFILE[2];
  uint8_t NETC_F2_COMMON_reserved16[800];
  NETC_F2_COMMON_IPFCAPR_tag IPFCAPR;  /* Ingress port filter capability register */
  NETC_F2_COMMON_IPFTCAPR_tag IPFTCAPR; /* Ingress port filter table capability register */
  NETC_F2_COMMON_IPFTMOR_tag IPFTMOR;  /* Ingress port filter table memory operational register */
  uint8_t NETC_F2_COMMON_reserved17[436];
  NETC_F2_COMMON_ITMCAPR_tag ITMCAPR;  /* Index table memory capability register */
  uint8_t NETC_F2_COMMON_reserved18[12];
  NETC_F2_COMMON_RPCAPR_tag RPCAPR;    /* Rate policer capability register */
  NETC_F2_COMMON_RPITCAPR_tag RPITCAPR; /* Rate policer index table capability register */
  NETC_F2_COMMON_RPITMAR_tag RPITMAR;  /* Rate policer index table memory allocation register */
  NETC_F2_COMMON_RPITOR_tag RPITOR;    /* Rate policer index table operational register */
  uint8_t NETC_F2_COMMON_reserved19[4];
  NETC_F2_COMMON_ISCITCAPR_tag ISCITCAPR; /* Ingress stream counter index table capability register */
  NETC_F2_COMMON_ISCITMAR_tag ISCITMAR; /* Ingress stream counter index table memory allocation register */
  NETC_F2_COMMON_ISCITOR_tag ISCITOR;  /* Ingress stream counter index table operational register */
  NETC_F2_COMMON_ISCAPR_tag ISCAPR;    /* Ingress stream capability register */
  NETC_F2_COMMON_ISITCAPR_tag ISITCAPR; /* Ingress stream index table capability register */
  NETC_F2_COMMON_ISITMAR_tag ISITMAR;  /* Ingress stream index table memory allocation register */
  NETC_F2_COMMON_ISITOR_tag ISITOR;    /* Ingress stream index table operational register */
  uint8_t NETC_F2_COMMON_reserved20[4];
  NETC_F2_COMMON_ISQGITCAPR_tag ISQGITCAPR; /* Ingress sequence generation index table capability register */
  NETC_F2_COMMON_ISQGITMAR_tag ISQGITMAR; /* Ingress sequence generation index table memory allocation register */
  NETC_F2_COMMON_ISQGITOR_tag ISQGITOR; /* Ingress sequence generation index table operational register */
  uint8_t NETC_F2_COMMON_reserved21[16];
  NETC_F2_COMMON_SGCAPR_tag SGCAPR;    /* Stream gate capability register */
  NETC_F2_COMMON_SGIITCAPR_tag SGIITCAPR; /* Stream gate instance index table capability register */
  NETC_F2_COMMON_SGIITMAR_tag SGIITMAR; /* Stream gate instance index table memory allocation register */
  NETC_F2_COMMON_SGIITOR_tag SGIITOR;  /* Stream gate instance index table operational register */
  uint8_t NETC_F2_COMMON_reserved22[4];
  NETC_F2_COMMON_SGCLITCAPR_tag SGCLITCAPR; /* Stream gate control list index table capability register */
  NETC_F2_COMMON_SGCLITMAR_tag SGCLITMAR; /* Stream gate control list index table memory allocation register */
  NETC_F2_COMMON_SGCLTMOR_tag SGCLTMOR; /* Stream gate control list table memory operational register */
  NETC_F2_COMMON_FMICAPR_tag FMICAPR;  /* Frame modification ingress capability register */
  NETC_F2_COMMON_FMECAPR_tag FMECAPR;  /* Frame modification egress capability register */
  NETC_F2_COMMON_FMITCAPR_tag FMITCAPR; /* Frame modification index table capability register */
  NETC_F2_COMMON_FMITMAR_tag FMITMAR;  /* Frame modification index table memory allocation register */
  NETC_F2_COMMON_FMITOR_tag FMITOR;    /* Frame modification index table operational register */
  NETC_F2_COMMON_FMDITCAPR_tag FMDITCAPR; /* Frame modification data index table capability register */
  NETC_F2_COMMON_FMDITMAR_tag FMDITMAR; /* Frame modification data index table memory allocation register */
  uint8_t NETC_F2_COMMON_reserved23[36];
  NETC_F2_COMMON_ETCAPR_tag ETCAPR;    /* Egress treatment capability register */
  NETC_F2_COMMON_ETTCAPR_tag ETTCAPR;  /* Egress treatment table capability register */
  uint8_t NETC_F2_COMMON_reserved24[4];
  NETC_F2_COMMON_ETTOR_tag ETTOR;      /* Egress treatment table operational register */
  uint8_t NETC_F2_COMMON_reserved25[4];
  NETC_F2_COMMON_TGSTCAPR_tag TGSTCAPR; /* Time gate scheduling table capability register */
  uint8_t NETC_F2_COMMON_reserved26[4];
  NETC_F2_COMMON_TGSTMOR_tag TGSTMOR;  /* Time gate scheduling table memory operation register */
  NETC_F2_COMMON_ESQRCAPR_tag ESQRCAPR; /* Egress sequence recovery capability register */
  NETC_F2_COMMON_ESQRTCAPR_tag ESQRTCAPR; /* Egress sequence recovery table capability register */
  uint8_t NETC_F2_COMMON_reserved27[4];
  NETC_F2_COMMON_ECTCAPR_tag ECTCAPR;  /* Egress counter table capability register */
  uint8_t NETC_F2_COMMON_reserved28[16];
  NETC_F2_COMMON_HTMCAPR_tag HTMCAPR;  /* Hash table memory capability register */
  NETC_F2_COMMON_HTMOR_tag HTMOR;      /* Hash table memory operational register */
  uint8_t NETC_F2_COMMON_reserved29[8];
  NETC_F2_COMMON_ISIDCAPR_tag ISIDCAPR; /* Ingress stream identification capability register */
  NETC_F2_COMMON_ISIDHTCAPR_tag ISIDHTCAPR; /* Ingress stream identification hash table capability register */
  uint8_t NETC_F2_COMMON_reserved30[8];
  NETC_F2_COMMON_ISIDKC0OR_tag ISIDKC0OR; /* Ingress stream identification key construction 0 operational register */
  NETC_F2_COMMON_ISIDKC0CR0_tag ISIDKC0CR0; /* Ingress stream identification key construction 0 configuration register 0 */
  uint8_t NETC_F2_COMMON_reserved31[8];
  NETC_F2_COMMON_ISIDKC0PF0CR_tag ISIDKC0PF0CR; /* Ingress stream identification key construction 0 payload field 0 configuration register */
  NETC_F2_COMMON_ISIDKC0PF1CR_tag ISIDKC0PF1CR; /* Ingress stream identification key construction 0 payload field 1 configuration register */
  NETC_F2_COMMON_ISIDKC0PF2CR_tag ISIDKC0PF2CR; /* Ingress stream identification key construction 0 payload field 2 configuration register */
  NETC_F2_COMMON_ISIDKC0PF3CR_tag ISIDKC0PF3CR; /* Ingress stream identification key construction 0 payload field 3 configuration register */
  NETC_F2_COMMON_ISIDKC1OR_tag ISIDKC1OR; /* Ingress stream identification key construction 1 operational register */
  NETC_F2_COMMON_ISIDKC1CR0_tag ISIDKC1CR0; /* Ingress stream identification key construction 1 configuration register 0 */
  uint8_t NETC_F2_COMMON_reserved32[8];
  NETC_F2_COMMON_ISIDKC1PF0CR_tag ISIDKC1PF0CR; /* Ingress stream identification key construction 1 payload field 0 configuration register */
  NETC_F2_COMMON_ISIDKC1PF1CR_tag ISIDKC1PF1CR; /* Ingress stream identification key construction 1 payload field 1 configuration register */
  NETC_F2_COMMON_ISIDKC1PF2CR_tag ISIDKC1PF2CR; /* Ingress stream identification key construction 1 payload field 2 configuration register */
  NETC_F2_COMMON_ISIDKC1PF3CR_tag ISIDKC1PF3CR; /* Ingress stream identification key construction 1 payload field 3 configuration register */
  NETC_F2_COMMON_ISIDKC2OR_tag ISIDKC2OR; /* Ingress stream identification key construction 2 operational register */
  NETC_F2_COMMON_ISIDKC2CR0_tag ISIDKC2CR0; /* Ingress stream identification key construction 2 configuration register 0 */
  uint8_t NETC_F2_COMMON_reserved33[8];
  NETC_F2_COMMON_ISIDKC2PF0CR_tag ISIDKC2PF0CR; /* Ingress stream identification key construction 2 payload field 0 configuration register */
  NETC_F2_COMMON_ISIDKC2PF1CR_tag ISIDKC2PF1CR; /* Ingress stream identification key construction 2 payload field 1 configuration register */
  NETC_F2_COMMON_ISIDKC2PF2CR_tag ISIDKC2PF2CR; /* Ingress stream identification key construction 2 payload field 2 configuration register */
  NETC_F2_COMMON_ISIDKC2PF3CR_tag ISIDKC2PF3CR; /* Ingress stream identification key construction 2 payload field 3 configuration register */
  NETC_F2_COMMON_ISIDKC3OR_tag ISIDKC3OR; /* Ingress stream identification key construction 3 operational register */
  NETC_F2_COMMON_ISIDKC3CR0_tag ISIDKC3CR0; /* Ingress stream identification key construction 3 configuration register 0 */
  uint8_t NETC_F2_COMMON_reserved34[8];
  NETC_F2_COMMON_ISIDKC3PF0CR_tag ISIDKC3PF0CR; /* Ingress stream identification key construction 3 payload field 0 configuration register */
  NETC_F2_COMMON_ISIDKC3PF1CR_tag ISIDKC3PF1CR; /* Ingress stream identification key construction 3 payload field 1 configuration register */
  NETC_F2_COMMON_ISIDKC3PF2CR_tag ISIDKC3PF2CR; /* Ingress stream identification key construction 3 payload field 2 configuration register */
  NETC_F2_COMMON_ISIDKC3PF3CR_tag ISIDKC3PF3CR; /* Ingress stream identification key construction 3 payload field 3 configuration register */
  uint8_t NETC_F2_COMMON_reserved35[96];
  NETC_F2_COMMON_ISFHTCAPR_tag ISFHTCAPR; /* Ingress stream filter hash table capability register */
  NETC_F2_COMMON_ISFHTOR_tag ISFHTOR;  /* Ingress stream filter hash table operational register */
};


/* ============================================================================
   =============================== Module: NETC_F2_GLOBAL =====================
   ============================================================================ */

typedef union NETC_F2_GLOBAL_SMCAPR_union_tag { /* Shared memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_SMCAPR_tag;

typedef union NETC_F2_GLOBAL_SMDTR_union_tag { /* Shared memory depletion threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_SMDTR_tag;

typedef union NETC_F2_GLOBAL_SMACR_union_tag { /* Shared memory available count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_SMACR_tag;

typedef union NETC_F2_GLOBAL_SMCLWMR_union_tag { /* Shared memory count low watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_SMCLWMR_tag;

typedef union NETC_F2_GLOBAL_SMBUCR_union_tag { /* Shared memory buffer unassigned count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_SMBUCR_tag;

typedef union NETC_F2_GLOBAL_SMBUCHWMR_union_tag { /* Shared memory buffer unassigned count high watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_SMBUCHWMR_tag;

typedef union NETC_F2_GLOBAL_SMLCR_union_tag { /* Shared memory loss count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:6;
    vuint32_t IFLC:1;
    vuint32_t IFDC:1;
  } B;
} NETC_F2_GLOBAL_SMLCR_tag;

typedef union NETC_F2_GLOBAL_HBTCAPR_union_tag { /* Hash bucket table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:8;
    vuint32_t MAX_COL:3;
    vuint32_t _unused_27:1;
    vuint32_t MAX_VISITS:4;
  } B;
} NETC_F2_GLOBAL_HBTCAPR_tag;

typedef union NETC_F2_GLOBAL_HBTOR0_union_tag { /* Hash bucket table operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t HWM_ENTRIES:16;
  } B;
} NETC_F2_GLOBAL_HBTOR0_tag;

typedef union NETC_F2_GLOBAL_HBTOR2_union_tag { /* Hash bucket table operational register 2 */
  vuint32_t R;
  struct {
    vuint32_t RUN_AVG_FRACT:8;
    vuint32_t RUN_AVG_INT:8;
    vuint32_t HWM_COL:4;
    vuint32_t _unused_20:12;
  } B;
} NETC_F2_GLOBAL_HBTOR2_tag;

typedef union NETC_F2_GLOBAL_SMERBCAPR_union_tag { /* Shared memory ENETC receive buffer capability register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:4;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;
  } B;
} NETC_F2_GLOBAL_SMERBCAPR_tag;

typedef union NETC_F2_GLOBAL_SMERBOR0_union_tag { /* Shared memory ENETC receive buffer operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t AMOUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_SMERBOR0_tag;

typedef union NETC_F2_GLOBAL_SMERBOR1_union_tag { /* Shared memory ENETC receive buffer operational 1 */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_SMERBOR1_tag;

typedef union NETC_F2_GLOBAL_PCE_SL_PCEOR_union_tag { /* PCE 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_FRAMES:6;
    vuint32_t _unused_6:2;
    vuint32_t HWM_FRAMES:6;
    vuint32_t _unused_14:2;
    vuint32_t MAX_FRAMES:6;
    vuint32_t _unused_22:10;
  } B;
} NETC_F2_GLOBAL_PCE_SL_PCEOR_tag;

typedef union NETC_F2_GLOBAL_PCE_SL_RFEOR_union_tag { /* Replication Forwarding Engine 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_FRAMES:6;
    vuint32_t _unused_6:2;
    vuint32_t HWM_FRAMES:6;
    vuint32_t _unused_14:2;
    vuint32_t MAX_FRAMES:6;
    vuint32_t _unused_22:10;
  } B;
} NETC_F2_GLOBAL_PCE_SL_RFEOR_tag;

typedef struct NETC_F2_GLOBAL_PCE_SL_struct_tag {
  NETC_F2_GLOBAL_PCE_SL_PCEOR_tag PCEOR; /* PCE 0 operational register */
  NETC_F2_GLOBAL_PCE_SL_RFEOR_tag RFEOR; /* Replication Forwarding Engine 0 operational register */
} NETC_F2_GLOBAL_PCE_SL_tag;

typedef union NETC_F2_GLOBAL_NETCCLKR_union_tag { /* NETC clock register */
  vuint32_t R;
  struct {
    vuint32_t FREQ:11;                 /* Frequency */
    vuint32_t _unused_11:21;
  } B;
} NETC_F2_GLOBAL_NETCCLKR_tag;

typedef union NETC_F2_GLOBAL_HTA_LOOP_HTACAPR_union_tag { /* HTA 0 capability register */
  vuint32_t R;
  struct {
    vuint32_t MAX_RX_FRAMES:8;
    vuint32_t MAX_TX_FRAMES:8;
    vuint32_t _unused_16:16;
  } B;
} NETC_F2_GLOBAL_HTA_LOOP_HTACAPR_tag;

typedef union NETC_F2_GLOBAL_HTA_LOOP_HTARFCOR_union_tag { /* HTA 0 receive frame count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:8;
    vuint32_t HP_HWM:8;
    vuint32_t LP_COUNT:8;
    vuint32_t LP_HWM:8;
  } B;
} NETC_F2_GLOBAL_HTA_LOOP_HTARFCOR_tag;

typedef union NETC_F2_GLOBAL_HTA_LOOP_HTAHPBCOR_union_tag { /* HTA 0 high priority byte count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:16;
    vuint32_t HWM:16;
  } B;
} NETC_F2_GLOBAL_HTA_LOOP_HTAHPBCOR_tag;

typedef union NETC_F2_GLOBAL_HTA_LOOP_HTALPBCOR_union_tag { /* HTA 0 low priority byte count operational register */
  vuint32_t R;
  struct {
    vuint32_t LP_COUNT:16;
    vuint32_t HWM:16;
  } B;
} NETC_F2_GLOBAL_HTA_LOOP_HTALPBCOR_tag;

typedef union NETC_F2_GLOBAL_HTA_LOOP_HTATFCOR_union_tag { /* HTA 0 transmit frame count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:8;
    vuint32_t HP_HWM:8;
    vuint32_t LP_COUNT:8;
    vuint32_t LP_HWM:8;
  } B;
} NETC_F2_GLOBAL_HTA_LOOP_HTATFCOR_tag;

typedef struct NETC_F2_GLOBAL_HTA_LOOP_struct_tag {
  NETC_F2_GLOBAL_HTA_LOOP_HTACAPR_tag HTACAPR; /* HTA 0 capability register */
  NETC_F2_GLOBAL_HTA_LOOP_HTARFCOR_tag HTARFCOR; /* HTA 0 receive frame count operational register */
  NETC_F2_GLOBAL_HTA_LOOP_HTAHPBCOR_tag HTAHPBCOR; /* HTA 0 high priority byte count operational register */
  NETC_F2_GLOBAL_HTA_LOOP_HTALPBCOR_tag HTALPBCOR; /* HTA 0 low priority byte count operational register */
  uint8_t HTA_LOOP_reserved0[20];
  NETC_F2_GLOBAL_HTA_LOOP_HTATFCOR_tag HTATFCOR; /* HTA 0 transmit frame count operational register */
} NETC_F2_GLOBAL_HTA_LOOP_tag;

typedef union NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_union_tag { /* Root complex 0 system bus read latency average register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_tag;

typedef union NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_union_tag { /* Root complex 0 system bus read latency high watermark register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_tag;

typedef union NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_union_tag { /* Root complex 0 system bus write latency average register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_tag;

typedef union NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_union_tag { /* Root complex 0 system bus write latency high watermark register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_tag;

typedef struct NETC_F2_GLOBAL_ARRAY_NUM_RC_struct_tag {
  NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_tag RCSBRLAR; /* Root complex 0 system bus read latency average register */
  NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_tag RCSBRLHWMR; /* Root complex 0 system bus read latency high watermark register */
  NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_tag RCSBWLAR; /* Root complex 0 system bus write latency average register */
  NETC_F2_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_tag RCSBWLHWMR; /* Root complex 0 system bus write latency high watermark register */
} NETC_F2_GLOBAL_ARRAY_NUM_RC_tag;

typedef union NETC_F2_GLOBAL_IPBRR0_union_tag { /* IP block revision register 0 */
  vuint32_t R;
  struct {
    vuint32_t IP_MN:8;
    vuint32_t IP_MJ:8;
    vuint32_t IP_ID:16;
  } B;
} NETC_F2_GLOBAL_IPBRR0_tag;

typedef union NETC_F2_GLOBAL_IPBRR1_union_tag { /* IP block revision register 1 */
  vuint32_t R;
  struct {
    vuint32_t IP_CFG:8;
    vuint32_t IP_MNT:8;
    vuint32_t IP_INT:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_GLOBAL_IPBRR1_tag;

typedef union NETC_F2_GLOBAL_FBLPR_union_tag { /* Function boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F2_GLOBAL_FBLPR_tag;

struct NETC_F2_GLOBAL_tag {
  NETC_F2_GLOBAL_SMCAPR_tag SMCAPR;    /* Shared memory capability register */
  NETC_F2_GLOBAL_SMDTR_tag SMDTR;      /* Shared memory depletion threshold register */
  NETC_F2_GLOBAL_SMACR_tag SMACR;      /* Shared memory available count register */
  uint8_t NETC_F2_GLOBAL_reserved0[4];
  NETC_F2_GLOBAL_SMCLWMR_tag SMCLWMR;  /* Shared memory count low watermark register */
  NETC_F2_GLOBAL_SMBUCR_tag SMBUCR;    /* Shared memory buffer unassigned count register */
  NETC_F2_GLOBAL_SMBUCHWMR_tag SMBUCHWMR; /* Shared memory buffer unassigned count high watermark register */
  NETC_F2_GLOBAL_SMLCR_tag SMLCR;      /* Shared memory loss count register */
  NETC_F2_GLOBAL_HBTCAPR_tag HBTCAPR;  /* Hash bucket table capability register */
  NETC_F2_GLOBAL_HBTOR0_tag HBTOR0;    /* Hash bucket table operational register 0 */
  uint8_t NETC_F2_GLOBAL_reserved1[4];
  NETC_F2_GLOBAL_HBTOR2_tag HBTOR2;    /* Hash bucket table operational register 2 */
  uint8_t NETC_F2_GLOBAL_reserved2[16];
  NETC_F2_GLOBAL_SMERBCAPR_tag SMERBCAPR; /* Shared memory ENETC receive buffer capability register */
  NETC_F2_GLOBAL_SMERBOR0_tag SMERBOR0; /* Shared memory ENETC receive buffer operational register 0 */
  NETC_F2_GLOBAL_SMERBOR1_tag SMERBOR1; /* Shared memory ENETC receive buffer operational 1 */
  uint8_t NETC_F2_GLOBAL_reserved3[180];
  NETC_F2_GLOBAL_PCE_SL_tag PCE_SL[1];
  uint8_t NETC_F2_GLOBAL_reserved4[92];
  NETC_F2_GLOBAL_NETCCLKR_tag NETCCLKR; /* NETC clock register */
  uint8_t NETC_F2_GLOBAL_reserved5[152];
  NETC_F2_GLOBAL_HTA_LOOP_tag HTA_LOOP[1];
  uint8_t NETC_F2_GLOBAL_reserved6[216];
  NETC_F2_GLOBAL_ARRAY_NUM_RC_tag ARRAY_NUM_RC[1];
  uint8_t NETC_F2_GLOBAL_reserved7[2280];
  NETC_F2_GLOBAL_IPBRR0_tag IPBRR0;    /* IP block revision register 0 */
  NETC_F2_GLOBAL_IPBRR1_tag IPBRR1;    /* IP block revision register 1 */
  uint8_t NETC_F2_GLOBAL_reserved8[256];
  NETC_F2_GLOBAL_FBLPR_tag FBLPR[2];   /* Function boot loader parameter register 0 */
};


/* ============================================================================
   =============================== Module: NETC_F2_PCI_HDR_TYPE0 ==============
   ============================================================================ */

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t MEM_ACCESS:1;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_ID:16;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_union_tag { /* PCI PCI-PM capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_union_tag { /* PCI PCI-PM capabilities register */
  vuint16_t R;
  struct {
    vuint16_t VERSION:3;
    vuint16_t _unused_3:8;
    vuint16_t PME_SUPPORT:5;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_union_tag { /* PCI PCI-PM control and status register */
  vuint16_t R;
  struct {
    vuint16_t PWR_STATE:2;
    vuint16_t _unused_2:1;
    vuint16_t NO_SOFT_RST:1;
    vuint16_t _unused_4:12;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_union_tag { /* PCI PCI-PM capabilities data register */
  vuint8_t R;
  struct {
    vuint8_t _unused_0:8;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_union_tag { /* PCI EA capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_union_tag { /* PCI EA capabilities register */
  vuint16_t R;
  struct {
    vuint16_t NUM_ENTRIES:6;
    vuint16_t _unused_6:10;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_union_tag { /* PCI EA per-entry 0 format register */
  vuint32_t R;
  struct {
    vuint32_t ENTRY_SIZE:3;
    vuint32_t _unused_3:1;
    vuint32_t BEI:4;
    vuint32_t PRIM_PROP:8;
    vuint32_t SEC_PROP:8;
    vuint32_t _unused_24:6;
    vuint32_t WRITABLE:1;
    vuint32_t ENABLE:1;
  } B;
} NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_union_tag { /* PCI EA per-entry 0 base register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t S:1;
    vuint32_t BASE:30;
  } B;
} NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_union_tag { /* PCI EA per-entry 0 max offset register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t S:1;
    vuint32_t MAX_OFFSET:30;
  } B;
} NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_union_tag { /* PCI EA per-entry 0 extended base register */
  vuint32_t R;
  struct {
    vuint32_t BASE:32;
  } B;
} NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_tag;

typedef struct NETC_F2_PCI_HDR_TYPE0_NUM_EA_struct_tag {
  NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_tag PCI_CFC_EA_PE_FMT; /* PCI EA per-entry 0 format register */
  NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_tag PCI_CFC_EA_PE_BASE; /* PCI EA per-entry 0 base register */
  NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_tag PCI_CFC_EA_PE_MAXOFF; /* PCI EA per-entry 0 max offset register */
  NETC_F2_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_tag PCI_CFC_EA_PE_EXT_BASE; /* PCI EA per-entry 0 extended base register */
} NETC_F2_PCI_HDR_TYPE0_NUM_EA_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:21;
    vuint32_t ACS_VIOLATION_STAT:1;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_F2_PCI_HDR_TYPE0_tag {
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved0[1];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved1[1];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved2[4];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved3[4];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved4[11];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved5[24];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved6[22];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved7[4];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_tag PCI_CFC_PCIPM_CAP_LIST; /* PCI PCI-PM capabilities list register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_tag PCI_CFC_PCIPM_CAP; /* PCI PCI-PM capabilities register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_tag PCI_CFC_PCIPM_CTL_STAT; /* PCI PCI-PM control and status register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved8[1];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_tag PCI_CFC_PCIPM_DATA; /* PCI PCI-PM capabilities data register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved9[4];
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_tag PCI_CFC_EA_CAP_LIST; /* PCI EA capabilities list register */
  NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_tag PCI_CFC_EA_CAP; /* PCI EA capabilities register */
  NETC_F2_PCI_HDR_TYPE0_NUM_EA_tag NUM_EA[2];
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved10[64];
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved11[20];
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_F2_PCI_HDR_TYPE0_reserved12[8];
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_F3 ============================
   ============================================================================ */

typedef union NETC_F3_ECAPR0_union_tag { /* ENETC capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;
    vuint32_t _unused_3:2;
    vuint32_t TSD:1;
    vuint32_t _unused_6:2;
    vuint32_t RSS:1;
    vuint32_t _unused_9:4;
    vuint32_t WO:1;
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional safety capability supported. */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_ECAPR0_tag;

typedef union NETC_F3_ECAPR1_union_tag { /* ENETC capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:11;             /* Number of MSI-X */
    vuint32_t _unused_23:1;
    vuint32_t NUM_VSI:4;
    vuint32_t _unused_28:3;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_ECAPR1_tag;

typedef union NETC_F3_ECAPR2_union_tag { /* ENETC capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:10;
    vuint32_t _unused_10:6;
    vuint32_t NUM_RX_BDR:10;
    vuint32_t _unused_26:6;
  } B;
} NETC_F3_ECAPR2_tag;

typedef union NETC_F3_PMR_union_tag {  /* Port mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t SI0EN:1;
    vuint32_t SI1EN:1;
    vuint32_t SI2EN:1;
    vuint32_t SI3EN:1;
    vuint32_t SI4EN:1;
    vuint32_t SI5EN:1;
    vuint32_t SI6EN:1;
    vuint32_t SI7EN:1;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PMR_tag;

typedef union NETC_F3_PONVLANR_union_tag { /* Port outer native VLAN register */
  vuint32_t R;
  struct {
    vuint32_t VID:12;
    vuint32_t DEI:1;
    vuint32_t PCP:3;
    vuint32_t TPID:2;
    vuint32_t PNE:1;                   /* Port Native VLAN Enable */
    vuint32_t VZE:1;                   /* VID 0 Enable */
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_PONVLANR_tag;

typedef union NETC_F3_PINVLANR_union_tag { /* Port inner native VLAN register */
  vuint32_t R;
  struct {
    vuint32_t VID:12;
    vuint32_t DEI:1;
    vuint32_t PCP:3;
    vuint32_t TPID:2;
    vuint32_t PNE:1;                   /* Port Native VLAN Enable */
    vuint32_t VZE:1;                   /* VID 0 Enable */
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_PINVLANR_tag;

typedef union NETC_F3_PVCLCTR_union_tag { /* Port VLAN classification control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:9;
    vuint32_t OAI:1;                   /* Outer as Inner */
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_PVCLCTR_tag;

typedef union NETC_F3_PARCSCR_union_tag { /* Parser checksum configuration register */
  vuint32_t R;
  struct {
    vuint32_t L4CD:1;
    vuint32_t L3CD:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_PARCSCR_tag;

typedef union NETC_F3_PARCECR_union_tag { /* Parser custom Ethertype 0 configuration register */
  vuint32_t R;
  struct {
    vuint32_t CP:4;                    /* Code Point */
    vuint32_t _unused_4:1;             /* - */
    vuint32_t EN:1;                    /* Enable */
    vuint32_t _unused_6:10;            /* - */
    vuint32_t ETYPE:16;                /* ETYPE */
  } B;
} NETC_F3_PARCECR_tag;

typedef union NETC_F3_PPAUONTR_union_tag { /* Port pause ON threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PPAUONTR_tag;

typedef union NETC_F3_PPAUOFFTR_union_tag { /* Port pause OFF threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PPAUOFFTR_tag;

typedef union NETC_F3_PRXMBER_union_tag { /* Port receive memory buffer entitlement register */
  vuint32_t R;
  struct {
    vuint32_t AMOUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PRXMBER_tag;

typedef union NETC_F3_PRXMBLR_union_tag { /* Port receive memory buffer limit register */
  vuint32_t R;
  struct {
    vuint32_t LIMIT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PRXMBLR_tag;

typedef union NETC_F3_PRXBCR_union_tag { /* Port receive buffer count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PRXBCR_tag;

typedef union NETC_F3_PRXBCHWMR_union_tag { /* Port receive buffer count high watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PRXBCHWMR_tag;

typedef union NETC_F3_PICDRADCR_PICDRDCR_union_tag { /* Port ingress congestion DR0 discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_PICDRADCR_PICDRDCR_tag;

typedef union NETC_F3_PICDRADCR_PICDRDCRRR_union_tag { /* Port ingress congestion DR0 discard count read-reset register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_PICDRADCR_PICDRDCRRR_tag;

typedef struct NETC_F3_PICDRADCR_struct_tag {
  NETC_F3_PICDRADCR_PICDRDCR_tag PICDRDCR; /* Port ingress congestion DR0 discard count register */
  uint8_t PICDRADCR_reserved0[4];
  NETC_F3_PICDRADCR_PICDRDCRRR_tag PICDRDCRRR; /* Port ingress congestion DR0 discard count read-reset register */
  uint8_t PICDRADCR_reserved1[4];
} NETC_F3_PICDRADCR_tag;

typedef union NETC_F3_PICPDSR_union_tag { /* Port ingress congestion priority discard status register */
  vuint32_t R;
  struct {
    vuint32_t DR0_P0DS:1;
    vuint32_t _unused_1:3;
    vuint32_t DR0_P1DS:1;
    vuint32_t _unused_5:3;
    vuint32_t DR1_P0DS:1;
    vuint32_t _unused_9:3;
    vuint32_t DR1_P1DS:1;
    vuint32_t _unused_13:3;
    vuint32_t DR2_P0DS:1;
    vuint32_t _unused_17:3;
    vuint32_t DR2_P1DS:1;
    vuint32_t _unused_21:3;
    vuint32_t DR3_P0DS:1;
    vuint32_t _unused_25:3;
    vuint32_t DR3_P1DS:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_F3_PICPDSR_tag;

typedef union NETC_F3_PSIPMMR_union_tag { /* Port station interface promiscuous MAC mode register */
  vuint32_t R;
  struct {
    vuint32_t SI0_MAC_UP:1;
    vuint32_t SI1_MAC_UP:1;
    vuint32_t SI2_MAC_UP:1;
    vuint32_t SI3_MAC_UP:1;
    vuint32_t SI4_MAC_UP:1;
    vuint32_t SI5_MAC_UP:1;
    vuint32_t SI6_MAC_UP:1;
    vuint32_t SI7_MAC_UP:1;
    vuint32_t _unused_8:8;
    vuint32_t SI0_MAC_MP:1;
    vuint32_t SI1_MAC_MP:1;
    vuint32_t SI2_MAC_MP:1;
    vuint32_t SI3_MAC_MP:1;
    vuint32_t SI4_MAC_MP:1;
    vuint32_t SI5_MAC_MP:1;
    vuint32_t SI6_MAC_MP:1;
    vuint32_t SI7_MAC_MP:1;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PSIPMMR_tag;

typedef union NETC_F3_PSIPVMR_union_tag { /* Port station interface promiscuous VLAN mode register */
  vuint32_t R;
  struct {
    vuint32_t SI0_VLAN_P:1;
    vuint32_t SI1_VLAN_P:1;
    vuint32_t SI2_VLAN_P:1;
    vuint32_t SI3_VLAN_P:1;
    vuint32_t SI4_VLAN_P:1;
    vuint32_t SI5_VLAN_P:1;
    vuint32_t SI6_VLAN_P:1;
    vuint32_t SI7_VLAN_P:1;
    vuint32_t _unused_8:8;
    vuint32_t SI0_VLAN_UTA:1;
    vuint32_t SI1_VLAN_UTA:1;
    vuint32_t SI2_VLAN_UTA:1;
    vuint32_t SI3_VLAN_UTA:1;
    vuint32_t SI4_VLAN_UTA:1;
    vuint32_t SI5_VLAN_UTA:1;
    vuint32_t SI6_VLAN_UTA:1;
    vuint32_t SI7_VLAN_UTA:1;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PSIPVMR_tag;

typedef union NETC_F3_PBFDSIR_union_tag { /* Port broadcast frames dropped due to MAC filtering register */
  vuint32_t R;
  struct {
    vuint32_t FRAME_DROP:32;
  } B;
} NETC_F3_PBFDSIR_tag;

typedef union NETC_F3_PFDMSAPR_union_tag { /* Port frame drop MAC source address pruning register */
  vuint32_t R;
  struct {
    vuint32_t FRAME_DROP:32;
  } B;
} NETC_F3_PFDMSAPR_tag;

typedef union NETC_F3_PSIMAFCAPR_union_tag { /* Port station interface MAC address filtering capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:12;
    vuint32_t _unused_12:20;
  } B;
} NETC_F3_PSIMAFCAPR_tag;

typedef union NETC_F3_PUFDMFR_union_tag { /* Port unicast frames dropped due to MAC filtering register */
  vuint32_t R;
  struct {
    vuint32_t FRAME_DROP:32;
  } B;
} NETC_F3_PUFDMFR_tag;

typedef union NETC_F3_PMFDMFR_union_tag { /* Port multicast frames dropped due to MAC filtering register */
  vuint32_t R;
  struct {
    vuint32_t FRAME_DROP:32;
  } B;
} NETC_F3_PMFDMFR_tag;

typedef union NETC_F3_PSIVLANFCAPR_union_tag { /* Port station interface VLAN filtering capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:12;
    vuint32_t _unused_12:20;
  } B;
} NETC_F3_PSIVLANFCAPR_tag;

typedef union NETC_F3_PSIVLANFMR_union_tag { /* Port station interface VLAN filtering mode register */
  vuint32_t R;
  struct {
    vuint32_t VS:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_PSIVLANFMR_tag;

typedef union NETC_F3_PUFDVFR_union_tag { /* Port unicast frames dropped VLAN filtering register */
  vuint32_t R;
  struct {
    vuint32_t FRAME_DROP:32;
  } B;
} NETC_F3_PUFDVFR_tag;

typedef union NETC_F3_PMFDVFR_union_tag { /* Port multicast frames dropped VLAN filtering register */
  vuint32_t R;
  struct {
    vuint32_t FRAME_DROP:32;
  } B;
} NETC_F3_PMFDVFR_tag;

typedef union NETC_F3_PBFDVFR_union_tag { /* Port broadcast frames dropped VLAN filtering register */
  vuint32_t R;
  struct {
    vuint32_t FRAME_DROP:32;
  } B;
} NETC_F3_PBFDVFR_tag;

typedef union NETC_F3_PRFSCAPR_union_tag { /* Port RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_PRFSCAPR_tag;

typedef union NETC_F3_PRFSMR_union_tag { /* Port RFS mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RFSE:1;
  } B;
} NETC_F3_PRFSMR_tag;

typedef union NETC_F3_IPV2ICMPMR0_union_tag { /* Receive IPV to ICM priority mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0ICM:1;
    vuint32_t _unused_1:3;
    vuint32_t IPV1ICM:1;
    vuint32_t _unused_5:3;
    vuint32_t IPV2ICM:1;
    vuint32_t _unused_9:3;
    vuint32_t IPV3ICM:1;
    vuint32_t _unused_13:3;
    vuint32_t IPV4ICM:1;
    vuint32_t _unused_17:3;
    vuint32_t IPV5ICM:1;
    vuint32_t _unused_21:3;
    vuint32_t IPV6ICM:1;
    vuint32_t _unused_25:3;
    vuint32_t IPV7ICM:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_F3_IPV2ICMPMR0_tag;

typedef union NETC_F3_PRIO2TCMR0_union_tag { /* Transmit priority to traffic class mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIO0TC:3;
    vuint32_t _unused_3:1;
    vuint32_t PRIO1TC:3;
    vuint32_t _unused_7:1;
    vuint32_t PRIO2TC:3;
    vuint32_t _unused_11:1;
    vuint32_t PRIO3TC:3;
    vuint32_t _unused_15:1;
    vuint32_t PRIO4TC:3;
    vuint32_t _unused_19:1;
    vuint32_t PRIO5TC:3;
    vuint32_t _unused_23:1;
    vuint32_t PRIO6TC:3;
    vuint32_t _unused_27:1;
    vuint32_t PRIO7TC:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_PRIO2TCMR0_tag;

typedef union NETC_F3_PTCTSDR_union_tag { /* Port traffic class 0 time specific departure register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t TSDE:1;
  } B;
} NETC_F3_PTCTSDR_tag;

typedef union NETC_F3_SMCAPR_union_tag { /* Switch management capability register */
  vuint32_t R;
  struct {
    vuint32_t SM:1;                    /* Switch Management */
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SMCAPR_tag;

typedef union NETC_F3_SMHRBDRMR_union_tag { /* Switch management host reason 1 receive BD ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t RXBDR:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SMHRBDRMR_tag;

typedef union NETC_F3_NUM_SI_PSIPMAR0_union_tag { /* Port station interface 1 primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_NUM_SI_PSIPMAR0_tag;

typedef union NETC_F3_NUM_SI_PSIPMAR1_union_tag { /* Port station interface 1 primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_NUM_SI_PSIPMAR1_tag;

typedef union NETC_F3_NUM_SI_PSIVLANR_union_tag { /* Port station interface 1 VLAN register */
  vuint32_t R;
  struct {
    vuint32_t VID:12;
    vuint32_t DEI:1;
    vuint32_t PCP:3;
    vuint32_t TPID:2;
    vuint32_t _unused_18:13;
    vuint32_t E:1;                     /* Enable */
  } B;
} NETC_F3_NUM_SI_PSIVLANR_tag;

typedef union NETC_F3_NUM_SI_PSICFGR0_union_tag { /* Port station interface 1 configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:7;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:3;
    vuint32_t SPE:1;                   /* Source Pruning Enable */
    vuint32_t VTE:1;                   /* VLAN Tag Extract */
    vuint32_t _unused_13:1;
    vuint32_t SIVIE:1;                 /* SI-based VLAN Insertion Enable */
    vuint32_t ASE:1;                   /* Anti-spoofing enable */
    vuint32_t NUM_RX_BDR:7;
    vuint32_t _unused_23:1;
    vuint32_t SIVC:4;
    vuint32_t SIBW:4;
  } B;
} NETC_F3_NUM_SI_PSICFGR0_tag;

typedef union NETC_F3_NUM_SI_PSICFGR1_union_tag { /* Port station interface 1 configuration register 1 */
  vuint32_t R;
  struct {
    vuint32_t TC0_MAP:3;
    vuint32_t _unused_3:1;
    vuint32_t TC1_MAP:3;
    vuint32_t _unused_7:1;
    vuint32_t TC2_MAP:3;
    vuint32_t _unused_11:1;
    vuint32_t TC3_MAP:3;
    vuint32_t _unused_15:1;
    vuint32_t TC4_MAP:3;
    vuint32_t _unused_19:1;
    vuint32_t TC5_MAP:3;
    vuint32_t _unused_23:1;
    vuint32_t TC6_MAP:3;
    vuint32_t _unused_27:1;
    vuint32_t TC7_MAP:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_NUM_SI_PSICFGR1_tag;

typedef union NETC_F3_NUM_SI_PSICFGR2_union_tag { /* Port station interface 1 configuration register 2 */
  vuint32_t R;
  struct {
    vuint32_t NUM_MSIX:6;              /* Number of MSI-X */
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_NUM_SI_PSICFGR2_tag;

typedef union NETC_F3_NUM_SI_PSIVMAFCFGR_union_tag { /* Port station interface 1 VSI MAC address filtering configuration register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_NUM_SI_PSIVMAFCFGR_tag;

typedef union NETC_F3_NUM_SI_PSIVLANFCFGR_union_tag { /* Port station interface 1 VLAN filtering configuration register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_NUM_SI_PSIVLANFCFGR_tag;

typedef union NETC_F3_NUM_SI_PSIRFSCFGR_union_tag { /* Port station interface 1 RFS configuration register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_NUM_SI_PSIRFSCFGR_tag;

typedef union NETC_F3_NUM_SI_PSIUMHFR0_union_tag { /* Port station interface 1 unicast MAC hash filter register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_HASH_FLT_LOW:32;
  } B;
} NETC_F3_NUM_SI_PSIUMHFR0_tag;

typedef union NETC_F3_NUM_SI_PSIUMHFR1_union_tag { /* Port station interface 1 unicast MAC hash filter register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_HASH_FLT_HIGH:32;
  } B;
} NETC_F3_NUM_SI_PSIUMHFR1_tag;

typedef union NETC_F3_NUM_SI_PSIMMHFR0_union_tag { /* Port station interface 1 multicast MAC hash filter register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_HASH_FLT_LOW:32;
  } B;
} NETC_F3_NUM_SI_PSIMMHFR0_tag;

typedef union NETC_F3_NUM_SI_PSIMMHFR1_union_tag { /* Port station interface 1 multicast MAC hash filter register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_HASH_FLT_HIGH:32;
  } B;
} NETC_F3_NUM_SI_PSIMMHFR1_tag;

typedef union NETC_F3_NUM_SI_PSIVHFR0_union_tag { /* Port station interface 1 VLAN hash filter register 0 */
  vuint32_t R;
  struct {
    vuint32_t VLAN_HASH_FLT_LOW:32;
  } B;
} NETC_F3_NUM_SI_PSIVHFR0_tag;

typedef union NETC_F3_NUM_SI_PSIVHFR1_union_tag { /* Port station interface 1 VLAN hash filter register 1 */
  vuint32_t R;
  struct {
    vuint32_t VLAN_HASH_FLT_HIGH:32;
  } B;
} NETC_F3_NUM_SI_PSIVHFR1_tag;

typedef struct NETC_F3_NUM_SI_struct_tag {
  NETC_F3_NUM_SI_PSIPMAR0_tag PSIPMAR0; /* Port station interface 1 primary MAC address register 0 */
  NETC_F3_NUM_SI_PSIPMAR1_tag PSIPMAR1; /* Port station interface 1 primary MAC address register 1 */
  NETC_F3_NUM_SI_PSIVLANR_tag PSIVLANR; /* Port station interface 1 VLAN register */
  uint8_t NUM_SI_reserved0[4];
  NETC_F3_NUM_SI_PSICFGR0_tag PSICFGR0; /* Port station interface 1 configuration register 0 */
  NETC_F3_NUM_SI_PSICFGR1_tag PSICFGR1; /* Port station interface 1 configuration register 1 */
  NETC_F3_NUM_SI_PSICFGR2_tag PSICFGR2; /* Port station interface 1 configuration register 2 */
  uint8_t NUM_SI_reserved1[20];
  NETC_F3_NUM_SI_PSIVMAFCFGR_tag PSIVMAFCFGR; /* Port station interface 1 VSI MAC address filtering configuration register */
  NETC_F3_NUM_SI_PSIVLANFCFGR_tag PSIVLANFCFGR; /* Port station interface 1 VLAN filtering configuration register */
  uint8_t NUM_SI_reserved2[8];
  NETC_F3_NUM_SI_PSIRFSCFGR_tag PSIRFSCFGR; /* Port station interface 1 RFS configuration register */
  uint8_t NUM_SI_reserved3[12];
  NETC_F3_NUM_SI_PSIUMHFR0_tag PSIUMHFR0; /* Port station interface 1 unicast MAC hash filter register 0 */
  NETC_F3_NUM_SI_PSIUMHFR1_tag PSIUMHFR1; /* Port station interface 1 unicast MAC hash filter register 1 */
  NETC_F3_NUM_SI_PSIMMHFR0_tag PSIMMHFR0; /* Port station interface 1 multicast MAC hash filter register 0 */
  NETC_F3_NUM_SI_PSIMMHFR1_tag PSIMMHFR1; /* Port station interface 1 multicast MAC hash filter register 1 */
  NETC_F3_NUM_SI_PSIVHFR0_tag PSIVHFR0; /* Port station interface 1 VLAN hash filter register 0 */
  NETC_F3_NUM_SI_PSIVHFR1_tag PSIVHFR1; /* Port station interface 1 VLAN hash filter register 1 */
  uint8_t NUM_SI_reserved4[24];
} NETC_F3_NUM_SI_tag;

struct NETC_F3_tag {
  NETC_F3_ECAPR0_tag ECAPR0;           /* ENETC capability register 0 */
  NETC_F3_ECAPR1_tag ECAPR1;           /* ENETC capability register 1 */
  NETC_F3_ECAPR2_tag ECAPR2;           /* ENETC capability register 2 */
  uint8_t NETC_F3_reserved0[4];
  NETC_F3_PMR_tag PMR;                 /* Port mode register */
  uint8_t NETC_F3_reserved1[108];
  NETC_F3_PONVLANR_tag PONVLANR;       /* Port outer native VLAN register */
  NETC_F3_PINVLANR_tag PINVLANR;       /* Port inner native VLAN register */
  NETC_F3_PVCLCTR_tag PVCLCTR;         /* Port VLAN classification control register */
  uint8_t NETC_F3_reserved2[16];
  NETC_F3_PARCSCR_tag PARCSCR;         /* Parser checksum configuration register */
  NETC_F3_PARCECR_tag PARCECR[4];      /* Parser custom Ethertype 0 configuration register */
  uint8_t NETC_F3_reserved3[88];
  NETC_F3_PPAUONTR_tag PPAUONTR;       /* Port pause ON threshold register */
  NETC_F3_PPAUOFFTR_tag PPAUOFFTR;     /* Port pause OFF threshold register */
  uint8_t NETC_F3_reserved4[16];
  NETC_F3_PRXMBER_tag PRXMBER;         /* Port receive memory buffer entitlement register */
  NETC_F3_PRXMBLR_tag PRXMBLR;         /* Port receive memory buffer limit register */
  NETC_F3_PRXBCR_tag PRXBCR;           /* Port receive buffer count register */
  NETC_F3_PRXBCHWMR_tag PRXBCHWMR;     /* Port receive buffer count high watermark register */
  uint8_t NETC_F3_reserved5[16];
  NETC_F3_PICDRADCR_tag PICDRADCR[4];
  NETC_F3_PICPDSR_tag PICPDSR;         /* Port ingress congestion priority discard status register */
  uint8_t NETC_F3_reserved6[124];
  NETC_F3_PSIPMMR_tag PSIPMMR;         /* Port station interface promiscuous MAC mode register */
  NETC_F3_PSIPVMR_tag PSIPVMR;         /* Port station interface promiscuous VLAN mode register */
  NETC_F3_PBFDSIR_tag PBFDSIR;         /* Port broadcast frames dropped due to MAC filtering register */
  NETC_F3_PFDMSAPR_tag PFDMSAPR;       /* Port frame drop MAC source address pruning register */
  uint8_t NETC_F3_reserved7[112];
  NETC_F3_PSIMAFCAPR_tag PSIMAFCAPR;   /* Port station interface MAC address filtering capability register */
  NETC_F3_PUFDMFR_tag PUFDMFR;         /* Port unicast frames dropped due to MAC filtering register */
  NETC_F3_PMFDMFR_tag PMFDMFR;         /* Port multicast frames dropped due to MAC filtering register */
  uint8_t NETC_F3_reserved8[52];
  NETC_F3_PSIVLANFCAPR_tag PSIVLANFCAPR; /* Port station interface VLAN filtering capability register */
  NETC_F3_PSIVLANFMR_tag PSIVLANFMR;   /* Port station interface VLAN filtering mode register */
  uint8_t NETC_F3_reserved9[8];
  NETC_F3_PUFDVFR_tag PUFDVFR;         /* Port unicast frames dropped VLAN filtering register */
  NETC_F3_PMFDVFR_tag PMFDVFR;         /* Port multicast frames dropped VLAN filtering register */
  NETC_F3_PBFDVFR_tag PBFDVFR;         /* Port broadcast frames dropped VLAN filtering register */
  uint8_t NETC_F3_reserved10[36];
  NETC_F3_PRFSCAPR_tag PRFSCAPR;       /* Port RFS capability register */
  uint8_t NETC_F3_reserved11[12];
  NETC_F3_PRFSMR_tag PRFSMR;           /* Port RFS mode register */
  uint8_t NETC_F3_reserved12[92];
  NETC_F3_IPV2ICMPMR0_tag IPV2ICMPMR0; /* Receive IPV to ICM priority mapping register 0 */
  uint8_t NETC_F3_reserved13[12];
  NETC_F3_PRIO2TCMR0_tag PRIO2TCMR0;   /* Transmit priority to traffic class mapping register 0 */
  uint8_t NETC_F3_reserved14[12];
  NETC_F3_PTCTSDR_tag PTCTSDR[8];      /* Port traffic class 0 time specific departure register */
  uint8_t NETC_F3_reserved15[1104];
  NETC_F3_SMCAPR_tag SMCAPR;           /* Switch management capability register */
  uint8_t NETC_F3_reserved16[124];
  NETC_F3_SMHRBDRMR_tag SMHRBDRMR[15]; /* Switch management host reason 1 receive BD ring mapping register */
  uint8_t NETC_F3_reserved17[5956];
  NETC_F3_NUM_SI_tag NUM_SI[8];
};


/* ============================================================================
   =============================== Module: NETC_F3_COMMON =====================
   ============================================================================ */

typedef union NETC_F3_COMMON_IPCAPR_union_tag { /* Ingress port capability register */
  vuint32_t R;
  struct {
    vuint32_t RP:1;
    vuint32_t IPFLT:1;
    vuint32_t ISID:1;
    vuint32_t _unused_3:5;
    vuint32_t SDU:5;                   /* Indicates support for various PDU/SDUs (Protocol/Service Data Unit) definitions. */
    vuint32_t _unused_13:3;
    vuint32_t NUM_VQMP:4;
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_COMMON_IPCAPR_tag;

typedef union NETC_F3_COMMON_EPCAPR_union_tag { /* Egress port capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t SDU:5;                   /* Indicates support for various PDU/SDUs (Protocol/Service Data Unit) definitions. */
    vuint32_t _unused_13:3;
    vuint32_t NUM_QVMP:4;
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_COMMON_EPCAPR_tag;

typedef union NETC_F3_COMMON_OSR_union_tag { /* Operational state register */
  vuint32_t R;
  struct {
    vuint32_t STATE:1;
    vuint32_t ITM_STATE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_COMMON_OSR_tag;

typedef union NETC_F3_COMMON_CMECR_union_tag { /* Correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_COMMON_CMECR_tag;

typedef union NETC_F3_COMMON_CMESR_union_tag { /* Correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_COMMON_CMESR_tag;

typedef union NETC_F3_COMMON_CMECTR_union_tag { /* Correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_COMMON_CMECTR_tag;

typedef union NETC_F3_COMMON_UNMACECR_union_tag { /* Uncorrectable non-fatal MAC error configuration register */
  vuint32_t R;
  struct {
    vuint32_t PORT0:1;                 /* Report disable port */
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_COMMON_UNMACECR_tag;

typedef union NETC_F3_COMMON_UNMACESR_union_tag { /* Uncorrectable non-fatal MAC error status register */
  vuint32_t R;
  struct {
    vuint32_t PORT0:1;                 /* Port MAC error */
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_COMMON_UNMACESR_tag;

typedef union NETC_F3_COMMON_UNMECR_union_tag { /* Uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_COMMON_UNMECR_tag;

typedef union NETC_F3_COMMON_UNMESR0_union_tag { /* Uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_COMMON_UNMESR0_tag;

typedef union NETC_F3_COMMON_UNMESR1_union_tag { /* Uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_COMMON_UNMESR1_tag;

typedef union NETC_F3_COMMON_UNMECTR_union_tag { /* Uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_COMMON_UNMECTR_tag;

typedef union NETC_F3_COMMON_UFMECR_union_tag { /* Uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_COMMON_UFMECR_tag;

typedef union NETC_F3_COMMON_UFMESR0_union_tag { /* Uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_COMMON_UFMESR0_tag;

typedef union NETC_F3_COMMON_UFMESR1_union_tag { /* Uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_COMMON_UFMESR1_tag;

typedef union NETC_F3_COMMON_UNIECR_union_tag { /* Uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_COMMON_UNIECR_tag;

typedef union NETC_F3_COMMON_UNIESR_union_tag { /* Uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t LINK_SLICE_ID:4;         /* Link or Slice ID */
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:2;
    vuint32_t ENGINE_ID:1;             /* Engine ID */
    vuint32_t _unused_17:14;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_COMMON_UNIESR_tag;

typedef union NETC_F3_COMMON_UNIECTR_union_tag { /* Uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_COMMON_UNIECTR_tag;

typedef union NETC_F3_COMMON_UFIECR_union_tag { /* Uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_COMMON_UFIECR_tag;

typedef union NETC_F3_COMMON_UFIESR_union_tag { /* Uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t LINK_SLICE_ID:4;         /* Link or Slice ID */
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:2;
    vuint32_t ENGINE_ID:1;             /* Engine ID */
    vuint32_t _unused_17:13;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_COMMON_UFIESR_tag;

typedef union NETC_F3_COMMON_CVLANR1_union_tag { /* Custom VLAN EtherType register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;                     /* Valid */
  } B;
} NETC_F3_COMMON_CVLANR1_tag;

typedef union NETC_F3_COMMON_CVLANR2_union_tag { /* Custom VLAN EtherType register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;                     /* Valid */
  } B;
} NETC_F3_COMMON_CVLANR2_tag;

typedef union NETC_F3_COMMON_DOSL2CR_union_tag { /* DoS L2 configuration register */
  vuint32_t R;
  struct {
    vuint32_t SAMEADDR:1;
    vuint32_t MSAMCC:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_COMMON_DOSL2CR_tag;

typedef union NETC_F3_COMMON_NUM_PROFILE_VLANIPVMPR0_union_tag { /* VLAN to IPV mapping profile 0 register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:3;
    vuint32_t _unused_3:1;
    vuint32_t PCP_DEI_1:3;
    vuint32_t _unused_7:1;
    vuint32_t PCP_DEI_2:3;
    vuint32_t _unused_11:1;
    vuint32_t PCP_DEI_3:3;
    vuint32_t _unused_15:1;
    vuint32_t PCP_DEI_4:3;
    vuint32_t _unused_19:1;
    vuint32_t PCP_DEI_5:3;
    vuint32_t _unused_23:1;
    vuint32_t PCP_DEI_6:3;
    vuint32_t _unused_27:1;
    vuint32_t PCP_DEI_7:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_COMMON_NUM_PROFILE_VLANIPVMPR0_tag;

typedef union NETC_F3_COMMON_NUM_PROFILE_VLANIPVMPR1_union_tag { /* VLAN to IPV mapping profile 0 register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:3;
    vuint32_t _unused_3:1;
    vuint32_t PCP_DEI_9:3;
    vuint32_t _unused_7:1;
    vuint32_t PCP_DEI_10:3;
    vuint32_t _unused_11:1;
    vuint32_t PCP_DEI_11:3;
    vuint32_t _unused_15:1;
    vuint32_t PCP_DEI_12:3;
    vuint32_t _unused_19:1;
    vuint32_t PCP_DEI_13:3;
    vuint32_t _unused_23:1;
    vuint32_t PCP_DEI_14:3;
    vuint32_t _unused_27:1;
    vuint32_t PCP_DEI_15:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_COMMON_NUM_PROFILE_VLANIPVMPR1_tag;

typedef union NETC_F3_COMMON_NUM_PROFILE_VLANDRMPR_union_tag { /* VLAN to DR mapping profile 0 register */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:2;
    vuint32_t PCP_DEI_1:2;
    vuint32_t PCP_DEI_2:2;
    vuint32_t PCP_DEI_3:2;
    vuint32_t PCP_DEI_4:2;
    vuint32_t PCP_DEI_5:2;
    vuint32_t PCP_DEI_6:2;
    vuint32_t PCP_DEI_7:2;
    vuint32_t PCP_DEI_8:2;
    vuint32_t PCP_DEI_9:2;
    vuint32_t PCP_DEI_10:2;
    vuint32_t PCP_DEI_11:2;
    vuint32_t PCP_DEI_12:2;
    vuint32_t PCP_DEI_13:2;
    vuint32_t PCP_DEI_14:2;
    vuint32_t PCP_DEI_15:2;
  } B;
} NETC_F3_COMMON_NUM_PROFILE_VLANDRMPR_tag;

typedef struct NETC_F3_COMMON_NUM_PROFILE_struct_tag {
  NETC_F3_COMMON_NUM_PROFILE_VLANIPVMPR0_tag VLANIPVMPR0; /* VLAN to IPV mapping profile 0 register 0 */
  NETC_F3_COMMON_NUM_PROFILE_VLANIPVMPR1_tag VLANIPVMPR1; /* VLAN to IPV mapping profile 0 register 1 */
  NETC_F3_COMMON_NUM_PROFILE_VLANDRMPR_tag VLANDRMPR; /* VLAN to DR mapping profile 0 register */
} NETC_F3_COMMON_NUM_PROFILE_tag;

typedef union NETC_F3_COMMON_IPFCAPR_union_tag { /* Ingress port filter capability register */
  vuint32_t R;
  struct {
    vuint32_t RP:1;                    /* Rate Policer function supported */
    vuint32_t ISID:1;                  /* Ingress Stream Identification supported. */
    vuint32_t FWD_SI:1;
    vuint32_t WOL:1;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_COMMON_IPFCAPR_tag;

typedef union NETC_F3_COMMON_IPFTCAPR_union_tag { /* Ingress port filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t MGMT:1;
    vuint32_t _unused_17:3;
    vuint32_t ACCESS_METH:4;
    vuint32_t ENTRY_MAX_WORDS:4;
    vuint32_t WORD_SIZE:2;
    vuint32_t _unused_30:2;
  } B;
} NETC_F3_COMMON_IPFTCAPR_tag;

typedef union NETC_F3_COMMON_IPFTMOR_union_tag { /* Ingress port filter table memory operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_IPFTMOR_tag;

typedef union NETC_F3_COMMON_ITMCAPR_union_tag { /* Index table memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:12;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;
  } B;
} NETC_F3_COMMON_ITMCAPR_tag;

typedef union NETC_F3_COMMON_RPCAPR_union_tag { /* Rate policer capability register */
  vuint32_t R;
  struct {
    vuint32_t TRTCM:1;                 /* Two-Rate Three-Color Marker supported per MEF 10.3 standard. */
    vuint32_t CM:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_COMMON_RPCAPR_tag;

typedef union NETC_F3_COMMON_RPITCAPR_union_tag { /* Rate policer index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:14;
    vuint32_t _unused_14:6;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_COMMON_RPITCAPR_tag;

typedef union NETC_F3_COMMON_RPITMAR_union_tag { /* Rate policer index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_RPITMAR_tag;

typedef union NETC_F3_COMMON_RPITOR_union_tag { /* Rate policer index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:14;
    vuint32_t _unused_14:18;
  } B;
} NETC_F3_COMMON_RPITOR_tag;

typedef union NETC_F3_COMMON_ISCITCAPR_union_tag { /* Ingress stream counter index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_COMMON_ISCITCAPR_tag;

typedef union NETC_F3_COMMON_ISCITMAR_union_tag { /* Ingress stream counter index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_ISCITMAR_tag;

typedef union NETC_F3_COMMON_ISCITOR_union_tag { /* Ingress stream counter index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_ISCITOR_tag;

typedef union NETC_F3_COMMON_ISCAPR_union_tag { /* Ingress stream capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t SG:1;
    vuint32_t RP:1;
    vuint32_t MAXSDU:1;
    vuint32_t _unused_6:3;
    vuint32_t FWD:1;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_COMMON_ISCAPR_tag;

typedef union NETC_F3_COMMON_ISITCAPR_union_tag { /* Ingress stream index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_COMMON_ISITCAPR_tag;

typedef union NETC_F3_COMMON_ISITMAR_union_tag { /* Ingress stream index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_ISITMAR_tag;

typedef union NETC_F3_COMMON_ISITOR_union_tag { /* Ingress stream index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_ISITOR_tag;

typedef union NETC_F3_COMMON_SGCAPR_union_tag { /* Stream gate capability register */
  vuint32_t R;
  struct {
    vuint32_t GLC_AO:1;
    vuint32_t GLC_GC:1;
    vuint32_t GLC_IO:1;
    vuint32_t GLC_IPV:1;
    vuint32_t GLC_CTD:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_COMMON_SGCAPR_tag;

typedef union NETC_F3_COMMON_SGIITCAPR_union_tag { /* Stream gate instance index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_COMMON_SGIITCAPR_tag;

typedef union NETC_F3_COMMON_SGIITMAR_union_tag { /* Stream gate instance index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_SGIITMAR_tag;

typedef union NETC_F3_COMMON_SGIITOR_union_tag { /* Stream gate instance index table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_SGIITOR_tag;

typedef union NETC_F3_COMMON_SGCLITCAPR_union_tag { /* Stream gate control list index table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_COMMON_SGCLITCAPR_tag;

typedef union NETC_F3_COMMON_SGCLITMAR_union_tag { /* Stream gate control list index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_SGCLITMAR_tag;

typedef union NETC_F3_COMMON_SGCLTMOR_union_tag { /* Stream gate control list table memory operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_SGCLTMOR_tag;

typedef union NETC_F3_COMMON_TGSTCAPR_union_tag { /* Time gate scheduling table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;            /* Number of Words */
    vuint32_t _unused_16:4;
    vuint32_t ACCESS_METH:4;           /* Access Method */
    vuint32_t MAX_GCL_LEN:2;           /* Maximum Gate Control List Length */
    vuint32_t _unused_26:6;
  } B;
} NETC_F3_COMMON_TGSTCAPR_tag;

typedef union NETC_F3_COMMON_TGSTMOR_union_tag { /* Time gate scheduling table memory operation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_TGSTMOR_tag;

typedef union NETC_F3_COMMON_HTMCAPR_union_tag { /* Hash table memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:12;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;
  } B;
} NETC_F3_COMMON_HTMCAPR_tag;

typedef union NETC_F3_COMMON_HTMOR_union_tag { /* Hash table memory operational register */
  vuint32_t R;
  struct {
    vuint32_t AMOUNT:16;
    vuint32_t WATERMARK:16;
  } B;
} NETC_F3_COMMON_HTMOR_tag;

typedef union NETC_F3_COMMON_ISIDCAPR_union_tag { /* Ingress stream identification capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_KC:2;
    vuint32_t NUM_PF:3;
    vuint32_t _unused_5:3;
    vuint32_t MAX_KSIZE:5;
    vuint32_t _unused_13:3;
    vuint32_t UFT:1;
    vuint32_t ETHFT:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_COMMON_ISIDCAPR_tag;

typedef union NETC_F3_COMMON_ISIDHTCAPR_union_tag { /* Ingress stream identification hash table capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:20;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_COMMON_ISIDHTCAPR_tag;

typedef union NETC_F3_COMMON_ISIDKC0OR_union_tag { /* Ingress stream identification key construction 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:15;
    vuint32_t EN:1;
  } B;
} NETC_F3_COMMON_ISIDKC0OR_tag;

typedef union NETC_F3_COMMON_ISIDKC0CR0_union_tag { /* Ingress stream identification key construction 0 configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t VALID:1;                 /* Valid */
    vuint32_t PORTP:1;
    vuint32_t SPMP:1;                  /* Switch Port Masquerading (flag) Present */
    vuint32_t DMACP:1;                 /* Destination MAC (address) Present */
    vuint32_t SMACP:1;                 /* Source MAC (address) Present. */
    vuint32_t OVIDP:1;                 /* Outer VID Present */
    vuint32_t OPCPP:1;                 /* Outer PCP Present */
    vuint32_t IVIDP:1;                 /* Inner VID Present. */
    vuint32_t IPCPP:1;                 /* Inner PCP Present */
    vuint32_t SQTP:1;                  /* Sequence Tag (code point) Present. */
    vuint32_t ETP:1;                   /* EtherType Present. */
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_COMMON_ISIDKC0CR0_tag;

typedef union NETC_F3_COMMON_ISIDKC0PF0CR_union_tag { /* Ingress stream identification key construction 0 payload field 0 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_COMMON_ISIDKC0PF0CR_tag;

typedef union NETC_F3_COMMON_ISIDKC0PF1CR_union_tag { /* Ingress stream identification key construction 0 payload field 1 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_COMMON_ISIDKC0PF1CR_tag;

typedef union NETC_F3_COMMON_ISIDKC0PF2CR_union_tag { /* Ingress stream identification key construction 0 payload field 2 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_COMMON_ISIDKC0PF2CR_tag;

typedef union NETC_F3_COMMON_ISIDKC0PF3CR_union_tag { /* Ingress stream identification key construction 0 payload field 3 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_COMMON_ISIDKC0PF3CR_tag;

typedef union NETC_F3_COMMON_ISIDKC1OR_union_tag { /* Ingress stream identification key construction 1 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:15;
    vuint32_t EN:1;
  } B;
} NETC_F3_COMMON_ISIDKC1OR_tag;

typedef union NETC_F3_COMMON_ISIDKC1CR0_union_tag { /* Ingress stream identification key construction 1 configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t VALID:1;                 /* Valid */
    vuint32_t PORTP:1;
    vuint32_t SPMP:1;                  /* Switch Port Masquerading (flag) Present */
    vuint32_t DMACP:1;                 /* Destination MAC (address) Present */
    vuint32_t SMACP:1;                 /* Source MAC (address) Present. */
    vuint32_t OVIDP:1;                 /* Outer VID Present */
    vuint32_t OPCPP:1;                 /* Outer PCP Present */
    vuint32_t IVIDP:1;                 /* Inner VID Present. */
    vuint32_t IPCPP:1;                 /* Inner PCP Present */
    vuint32_t SQTP:1;                  /* Sequence Tag (code point) Present. */
    vuint32_t ETP:1;                   /* EtherType Present. */
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_COMMON_ISIDKC1CR0_tag;

typedef union NETC_F3_COMMON_ISIDKC1PF0CR_union_tag { /* Ingress stream identification key construction 1 payload field 0 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_COMMON_ISIDKC1PF0CR_tag;

typedef union NETC_F3_COMMON_ISIDKC1PF1CR_union_tag { /* Ingress stream identification key construction 1 payload field 1 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_COMMON_ISIDKC1PF1CR_tag;

typedef union NETC_F3_COMMON_ISIDKC1PF2CR_union_tag { /* Ingress stream identification key construction 1 payload field 2 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_COMMON_ISIDKC1PF2CR_tag;

typedef union NETC_F3_COMMON_ISIDKC1PF3CR_union_tag { /* Ingress stream identification key construction 1 payload field 3 configuration register */
  vuint32_t R;
  struct {
    vuint32_t PFP:1;
    vuint32_t NUM_BYTES:4;
    vuint32_t _unused_5:3;
    vuint32_t BYTE_OFFSET:7;
    vuint32_t _unused_15:1;
    vuint32_t FBMASK:3;
    vuint32_t _unused_19:1;
    vuint32_t LBMASK:3;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_COMMON_ISIDKC1PF3CR_tag;

typedef union NETC_F3_COMMON_ISFHTCAPR_union_tag { /* Ingress stream filter hash table capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:20;
    vuint32_t ACCESS_METH:4;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_COMMON_ISFHTCAPR_tag;

typedef union NETC_F3_COMMON_ISFHTOR_union_tag { /* Ingress stream filter hash table operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_COMMON_ISFHTOR_tag;

struct NETC_F3_COMMON_tag {
  uint8_t NETC_F3_COMMON_reserved0[4096];
  NETC_F3_COMMON_IPCAPR_tag IPCAPR;    /* Ingress port capability register */
  NETC_F3_COMMON_EPCAPR_tag EPCAPR;    /* Egress port capability register */
  uint8_t NETC_F3_COMMON_reserved1[8];
  NETC_F3_COMMON_OSR_tag OSR;          /* Operational state register */
  uint8_t NETC_F3_COMMON_reserved2[44];
  NETC_F3_COMMON_CMECR_tag CMECR;      /* Correctable memory error configuration register */
  NETC_F3_COMMON_CMESR_tag CMESR;      /* Correctable memory error status register */
  uint8_t NETC_F3_COMMON_reserved3[4];
  NETC_F3_COMMON_CMECTR_tag CMECTR;    /* Correctable memory error count register */
  uint8_t NETC_F3_COMMON_reserved4[16];
  NETC_F3_COMMON_UNMACECR_tag UNMACECR; /* Uncorrectable non-fatal MAC error configuration register */
  NETC_F3_COMMON_UNMACESR_tag UNMACESR; /* Uncorrectable non-fatal MAC error status register */
  uint8_t NETC_F3_COMMON_reserved5[40];
  NETC_F3_COMMON_UNMECR_tag UNMECR;    /* Uncorrectable non-fatal memory error configuration register */
  NETC_F3_COMMON_UNMESR0_tag UNMESR0;  /* Uncorrectable non-fatal memory error status register 0 */
  NETC_F3_COMMON_UNMESR1_tag UNMESR1;  /* Uncorrectable non-fatal memory error status register 1 */
  NETC_F3_COMMON_UNMECTR_tag UNMECTR;  /* Uncorrectable non-fatal memory error count register */
  NETC_F3_COMMON_UFMECR_tag UFMECR;    /* Uncorrectable fatal memory error configuration register */
  NETC_F3_COMMON_UFMESR0_tag UFMESR0;  /* Uncorrectable fatal memory error status register 0 */
  NETC_F3_COMMON_UFMESR1_tag UFMESR1;  /* Uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_COMMON_reserved6[4];
  NETC_F3_COMMON_UNIECR_tag UNIECR;    /* Uncorrectable non-fatal integrity error configuration register */
  NETC_F3_COMMON_UNIESR_tag UNIESR;    /* Uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_COMMON_reserved7[4];
  NETC_F3_COMMON_UNIECTR_tag UNIECTR;  /* Uncorrectable non-fatal integrity error count register */
  NETC_F3_COMMON_UFIECR_tag UFIECR;    /* Uncorrectable fatal integrity error configuration register */
  NETC_F3_COMMON_UFIESR_tag UFIESR;    /* Uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_COMMON_reserved8[312];
  NETC_F3_COMMON_CVLANR1_tag CVLANR1;  /* Custom VLAN EtherType register 1 */
  NETC_F3_COMMON_CVLANR2_tag CVLANR2;  /* Custom VLAN EtherType register 2 */
  uint8_t NETC_F3_COMMON_reserved9[24];
  NETC_F3_COMMON_DOSL2CR_tag DOSL2CR;  /* DoS L2 configuration register */
  uint8_t NETC_F3_COMMON_reserved10[220];
  NETC_F3_COMMON_NUM_PROFILE_tag NUM_PROFILE[1];
  uint8_t NETC_F3_COMMON_reserved11[820];
  NETC_F3_COMMON_IPFCAPR_tag IPFCAPR;  /* Ingress port filter capability register */
  NETC_F3_COMMON_IPFTCAPR_tag IPFTCAPR; /* Ingress port filter table capability register */
  NETC_F3_COMMON_IPFTMOR_tag IPFTMOR;  /* Ingress port filter table memory operational register */
  uint8_t NETC_F3_COMMON_reserved12[436];
  NETC_F3_COMMON_ITMCAPR_tag ITMCAPR;  /* Index table memory capability register */
  uint8_t NETC_F3_COMMON_reserved13[12];
  NETC_F3_COMMON_RPCAPR_tag RPCAPR;    /* Rate policer capability register */
  NETC_F3_COMMON_RPITCAPR_tag RPITCAPR; /* Rate policer index table capability register */
  NETC_F3_COMMON_RPITMAR_tag RPITMAR;  /* Rate policer index table memory allocation register */
  NETC_F3_COMMON_RPITOR_tag RPITOR;    /* Rate policer index table operational register */
  uint8_t NETC_F3_COMMON_reserved14[4];
  NETC_F3_COMMON_ISCITCAPR_tag ISCITCAPR; /* Ingress stream counter index table capability register */
  NETC_F3_COMMON_ISCITMAR_tag ISCITMAR; /* Ingress stream counter index table memory allocation register */
  NETC_F3_COMMON_ISCITOR_tag ISCITOR;  /* Ingress stream counter index table operational register */
  NETC_F3_COMMON_ISCAPR_tag ISCAPR;    /* Ingress stream capability register */
  NETC_F3_COMMON_ISITCAPR_tag ISITCAPR; /* Ingress stream index table capability register */
  NETC_F3_COMMON_ISITMAR_tag ISITMAR;  /* Ingress stream index table memory allocation register */
  NETC_F3_COMMON_ISITOR_tag ISITOR;    /* Ingress stream index table operational register */
  uint8_t NETC_F3_COMMON_reserved15[32];
  NETC_F3_COMMON_SGCAPR_tag SGCAPR;    /* Stream gate capability register */
  NETC_F3_COMMON_SGIITCAPR_tag SGIITCAPR; /* Stream gate instance index table capability register */
  NETC_F3_COMMON_SGIITMAR_tag SGIITMAR; /* Stream gate instance index table memory allocation register */
  NETC_F3_COMMON_SGIITOR_tag SGIITOR;  /* Stream gate instance index table operational register */
  uint8_t NETC_F3_COMMON_reserved16[4];
  NETC_F3_COMMON_SGCLITCAPR_tag SGCLITCAPR; /* Stream gate control list index table capability register */
  NETC_F3_COMMON_SGCLITMAR_tag SGCLITMAR; /* Stream gate control list index table memory allocation register */
  NETC_F3_COMMON_SGCLTMOR_tag SGCLTMOR; /* Stream gate control list table memory operational register */
  uint8_t NETC_F3_COMMON_reserved17[84];
  NETC_F3_COMMON_TGSTCAPR_tag TGSTCAPR; /* Time gate scheduling table capability register */
  uint8_t NETC_F3_COMMON_reserved18[4];
  NETC_F3_COMMON_TGSTMOR_tag TGSTMOR;  /* Time gate scheduling table memory operation register */
  uint8_t NETC_F3_COMMON_reserved19[32];
  NETC_F3_COMMON_HTMCAPR_tag HTMCAPR;  /* Hash table memory capability register */
  NETC_F3_COMMON_HTMOR_tag HTMOR;      /* Hash table memory operational register */
  uint8_t NETC_F3_COMMON_reserved20[8];
  NETC_F3_COMMON_ISIDCAPR_tag ISIDCAPR; /* Ingress stream identification capability register */
  NETC_F3_COMMON_ISIDHTCAPR_tag ISIDHTCAPR; /* Ingress stream identification hash table capability register */
  uint8_t NETC_F3_COMMON_reserved21[8];
  NETC_F3_COMMON_ISIDKC0OR_tag ISIDKC0OR; /* Ingress stream identification key construction 0 operational register */
  NETC_F3_COMMON_ISIDKC0CR0_tag ISIDKC0CR0; /* Ingress stream identification key construction 0 configuration register 0 */
  uint8_t NETC_F3_COMMON_reserved22[8];
  NETC_F3_COMMON_ISIDKC0PF0CR_tag ISIDKC0PF0CR; /* Ingress stream identification key construction 0 payload field 0 configuration register */
  NETC_F3_COMMON_ISIDKC0PF1CR_tag ISIDKC0PF1CR; /* Ingress stream identification key construction 0 payload field 1 configuration register */
  NETC_F3_COMMON_ISIDKC0PF2CR_tag ISIDKC0PF2CR; /* Ingress stream identification key construction 0 payload field 2 configuration register */
  NETC_F3_COMMON_ISIDKC0PF3CR_tag ISIDKC0PF3CR; /* Ingress stream identification key construction 0 payload field 3 configuration register */
  NETC_F3_COMMON_ISIDKC1OR_tag ISIDKC1OR; /* Ingress stream identification key construction 1 operational register */
  NETC_F3_COMMON_ISIDKC1CR0_tag ISIDKC1CR0; /* Ingress stream identification key construction 1 configuration register 0 */
  uint8_t NETC_F3_COMMON_reserved23[8];
  NETC_F3_COMMON_ISIDKC1PF0CR_tag ISIDKC1PF0CR; /* Ingress stream identification key construction 1 payload field 0 configuration register */
  NETC_F3_COMMON_ISIDKC1PF1CR_tag ISIDKC1PF1CR; /* Ingress stream identification key construction 1 payload field 1 configuration register */
  NETC_F3_COMMON_ISIDKC1PF2CR_tag ISIDKC1PF2CR; /* Ingress stream identification key construction 1 payload field 2 configuration register */
  NETC_F3_COMMON_ISIDKC1PF3CR_tag ISIDKC1PF3CR; /* Ingress stream identification key construction 1 payload field 3 configuration register */
  uint8_t NETC_F3_COMMON_reserved24[160];
  NETC_F3_COMMON_ISFHTCAPR_tag ISFHTCAPR; /* Ingress stream filter hash table capability register */
  NETC_F3_COMMON_ISFHTOR_tag ISFHTOR;  /* Ingress stream filter hash table operational register */
};


/* ============================================================================
   =============================== Module: NETC_F3_GLOBAL =====================
   ============================================================================ */

typedef union NETC_F3_GLOBAL_SMCAPR_union_tag { /* Shared memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_SMCAPR_tag;

typedef union NETC_F3_GLOBAL_SMDTR_union_tag { /* Shared memory depletion threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_SMDTR_tag;

typedef union NETC_F3_GLOBAL_SMACR_union_tag { /* Shared memory available count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_SMACR_tag;

typedef union NETC_F3_GLOBAL_SMCLWMR_union_tag { /* Shared memory count low watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_SMCLWMR_tag;

typedef union NETC_F3_GLOBAL_SMBUCR_union_tag { /* Shared memory buffer unassigned count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_SMBUCR_tag;

typedef union NETC_F3_GLOBAL_SMBUCHWMR_union_tag { /* Shared memory buffer unassigned count high watermark register */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_SMBUCHWMR_tag;

typedef union NETC_F3_GLOBAL_SMLCR_union_tag { /* Shared memory loss count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:24;
    vuint32_t _unused_24:6;
    vuint32_t IFLC:1;
    vuint32_t IFDC:1;
  } B;
} NETC_F3_GLOBAL_SMLCR_tag;

typedef union NETC_F3_GLOBAL_HBTCAPR_union_tag { /* Hash bucket table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t _unused_16:8;
    vuint32_t MAX_COL:3;
    vuint32_t _unused_27:1;
    vuint32_t MAX_VISITS:4;
  } B;
} NETC_F3_GLOBAL_HBTCAPR_tag;

typedef union NETC_F3_GLOBAL_HBTOR0_union_tag { /* Hash bucket table operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_ENTRIES:16;
    vuint32_t HWM_ENTRIES:16;
  } B;
} NETC_F3_GLOBAL_HBTOR0_tag;

typedef union NETC_F3_GLOBAL_HBTOR2_union_tag { /* Hash bucket table operational register 2 */
  vuint32_t R;
  struct {
    vuint32_t RUN_AVG_FRACT:8;
    vuint32_t RUN_AVG_INT:8;
    vuint32_t HWM_COL:4;
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_GLOBAL_HBTOR2_tag;

typedef union NETC_F3_GLOBAL_SMERBCAPR_union_tag { /* Shared memory ENETC receive buffer capability register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:4;
    vuint32_t WORD_SIZE:2;
    vuint32_t MLOC:2;
  } B;
} NETC_F3_GLOBAL_SMERBCAPR_tag;

typedef union NETC_F3_GLOBAL_SMERBOR0_union_tag { /* Shared memory ENETC receive buffer operational register 0 */
  vuint32_t R;
  struct {
    vuint32_t AMOUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_SMERBOR0_tag;

typedef union NETC_F3_GLOBAL_SMERBOR1_union_tag { /* Shared memory ENETC receive buffer operational 1 */
  vuint32_t R;
  struct {
    vuint32_t WATERMARK:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_SMERBOR1_tag;

typedef union NETC_F3_GLOBAL_PCE_SL_PCEOR_union_tag { /* PCE 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_FRAMES:6;
    vuint32_t _unused_6:2;
    vuint32_t HWM_FRAMES:6;
    vuint32_t _unused_14:2;
    vuint32_t MAX_FRAMES:6;
    vuint32_t _unused_22:10;
  } B;
} NETC_F3_GLOBAL_PCE_SL_PCEOR_tag;

typedef union NETC_F3_GLOBAL_PCE_SL_RFEOR_union_tag { /* Replication Forwarding Engine 0 operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_FRAMES:6;
    vuint32_t _unused_6:2;
    vuint32_t HWM_FRAMES:6;
    vuint32_t _unused_14:2;
    vuint32_t MAX_FRAMES:6;
    vuint32_t _unused_22:10;
  } B;
} NETC_F3_GLOBAL_PCE_SL_RFEOR_tag;

typedef struct NETC_F3_GLOBAL_PCE_SL_struct_tag {
  NETC_F3_GLOBAL_PCE_SL_PCEOR_tag PCEOR; /* PCE 0 operational register */
  NETC_F3_GLOBAL_PCE_SL_RFEOR_tag RFEOR; /* Replication Forwarding Engine 0 operational register */
} NETC_F3_GLOBAL_PCE_SL_tag;

typedef union NETC_F3_GLOBAL_NETCCLKR_union_tag { /* NETC clock register */
  vuint32_t R;
  struct {
    vuint32_t FREQ:11;                 /* Frequency */
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_GLOBAL_NETCCLKR_tag;

typedef union NETC_F3_GLOBAL_HTA_LOOP_HTACAPR_union_tag { /* HTA 0 capability register */
  vuint32_t R;
  struct {
    vuint32_t MAX_RX_FRAMES:8;
    vuint32_t MAX_TX_FRAMES:8;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_GLOBAL_HTA_LOOP_HTACAPR_tag;

typedef union NETC_F3_GLOBAL_HTA_LOOP_HTARFCOR_union_tag { /* HTA 0 receive frame count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:8;
    vuint32_t HP_HWM:8;
    vuint32_t LP_COUNT:8;
    vuint32_t LP_HWM:8;
  } B;
} NETC_F3_GLOBAL_HTA_LOOP_HTARFCOR_tag;

typedef union NETC_F3_GLOBAL_HTA_LOOP_HTAHPBCOR_union_tag { /* HTA 0 high priority byte count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:16;
    vuint32_t HWM:16;
  } B;
} NETC_F3_GLOBAL_HTA_LOOP_HTAHPBCOR_tag;

typedef union NETC_F3_GLOBAL_HTA_LOOP_HTALPBCOR_union_tag { /* HTA 0 low priority byte count operational register */
  vuint32_t R;
  struct {
    vuint32_t LP_COUNT:16;
    vuint32_t HWM:16;
  } B;
} NETC_F3_GLOBAL_HTA_LOOP_HTALPBCOR_tag;

typedef union NETC_F3_GLOBAL_HTA_LOOP_HTATFCOR_union_tag { /* HTA 0 transmit frame count operational register */
  vuint32_t R;
  struct {
    vuint32_t HP_COUNT:8;
    vuint32_t HP_HWM:8;
    vuint32_t LP_COUNT:8;
    vuint32_t LP_HWM:8;
  } B;
} NETC_F3_GLOBAL_HTA_LOOP_HTATFCOR_tag;

typedef struct NETC_F3_GLOBAL_HTA_LOOP_struct_tag {
  NETC_F3_GLOBAL_HTA_LOOP_HTACAPR_tag HTACAPR; /* HTA 0 capability register */
  NETC_F3_GLOBAL_HTA_LOOP_HTARFCOR_tag HTARFCOR; /* HTA 0 receive frame count operational register */
  NETC_F3_GLOBAL_HTA_LOOP_HTAHPBCOR_tag HTAHPBCOR; /* HTA 0 high priority byte count operational register */
  NETC_F3_GLOBAL_HTA_LOOP_HTALPBCOR_tag HTALPBCOR; /* HTA 0 low priority byte count operational register */
  uint8_t HTA_LOOP_reserved0[20];
  NETC_F3_GLOBAL_HTA_LOOP_HTATFCOR_tag HTATFCOR; /* HTA 0 transmit frame count operational register */
} NETC_F3_GLOBAL_HTA_LOOP_tag;

typedef union NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_union_tag { /* Root complex 0 system bus read latency average register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_tag;

typedef union NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_union_tag { /* Root complex 0 system bus read latency high watermark register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_tag;

typedef union NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_union_tag { /* Root complex 0 system bus write latency average register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_tag;

typedef union NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_union_tag { /* Root complex 0 system bus write latency high watermark register */
  vuint32_t R;
  struct {
    vuint32_t FRACT:8;
    vuint32_t INT:12;
    vuint32_t _unused_20:12;
  } B;
} NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_tag;

typedef struct NETC_F3_GLOBAL_ARRAY_NUM_RC_struct_tag {
  NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBRLAR_tag RCSBRLAR; /* Root complex 0 system bus read latency average register */
  NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBRLHWMR_tag RCSBRLHWMR; /* Root complex 0 system bus read latency high watermark register */
  NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBWLAR_tag RCSBWLAR; /* Root complex 0 system bus write latency average register */
  NETC_F3_GLOBAL_ARRAY_NUM_RC_RCSBWLHWMR_tag RCSBWLHWMR; /* Root complex 0 system bus write latency high watermark register */
} NETC_F3_GLOBAL_ARRAY_NUM_RC_tag;

typedef union NETC_F3_GLOBAL_IPBRR0_union_tag { /* IP block revision register 0 */
  vuint32_t R;
  struct {
    vuint32_t IP_MN:8;
    vuint32_t IP_MJ:8;
    vuint32_t IP_ID:16;
  } B;
} NETC_F3_GLOBAL_IPBRR0_tag;

typedef union NETC_F3_GLOBAL_IPBRR1_union_tag { /* IP block revision register 1 */
  vuint32_t R;
  struct {
    vuint32_t IP_CFG:8;
    vuint32_t IP_MNT:8;
    vuint32_t IP_INT:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_GLOBAL_IPBRR1_tag;

typedef union NETC_F3_GLOBAL_FBLPR_union_tag { /* Function boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F3_GLOBAL_FBLPR_tag;

struct NETC_F3_GLOBAL_tag {
  NETC_F3_GLOBAL_SMCAPR_tag SMCAPR;    /* Shared memory capability register */
  NETC_F3_GLOBAL_SMDTR_tag SMDTR;      /* Shared memory depletion threshold register */
  NETC_F3_GLOBAL_SMACR_tag SMACR;      /* Shared memory available count register */
  uint8_t NETC_F3_GLOBAL_reserved0[4];
  NETC_F3_GLOBAL_SMCLWMR_tag SMCLWMR;  /* Shared memory count low watermark register */
  NETC_F3_GLOBAL_SMBUCR_tag SMBUCR;    /* Shared memory buffer unassigned count register */
  NETC_F3_GLOBAL_SMBUCHWMR_tag SMBUCHWMR; /* Shared memory buffer unassigned count high watermark register */
  NETC_F3_GLOBAL_SMLCR_tag SMLCR;      /* Shared memory loss count register */
  NETC_F3_GLOBAL_HBTCAPR_tag HBTCAPR;  /* Hash bucket table capability register */
  NETC_F3_GLOBAL_HBTOR0_tag HBTOR0;    /* Hash bucket table operational register 0 */
  uint8_t NETC_F3_GLOBAL_reserved1[4];
  NETC_F3_GLOBAL_HBTOR2_tag HBTOR2;    /* Hash bucket table operational register 2 */
  uint8_t NETC_F3_GLOBAL_reserved2[16];
  NETC_F3_GLOBAL_SMERBCAPR_tag SMERBCAPR; /* Shared memory ENETC receive buffer capability register */
  NETC_F3_GLOBAL_SMERBOR0_tag SMERBOR0; /* Shared memory ENETC receive buffer operational register 0 */
  NETC_F3_GLOBAL_SMERBOR1_tag SMERBOR1; /* Shared memory ENETC receive buffer operational 1 */
  uint8_t NETC_F3_GLOBAL_reserved3[180];
  NETC_F3_GLOBAL_PCE_SL_tag PCE_SL[1];
  uint8_t NETC_F3_GLOBAL_reserved4[92];
  NETC_F3_GLOBAL_NETCCLKR_tag NETCCLKR; /* NETC clock register */
  uint8_t NETC_F3_GLOBAL_reserved5[152];
  NETC_F3_GLOBAL_HTA_LOOP_tag HTA_LOOP[1];
  uint8_t NETC_F3_GLOBAL_reserved6[216];
  NETC_F3_GLOBAL_ARRAY_NUM_RC_tag ARRAY_NUM_RC[1];
  uint8_t NETC_F3_GLOBAL_reserved7[2280];
  NETC_F3_GLOBAL_IPBRR0_tag IPBRR0;    /* IP block revision register 0 */
  NETC_F3_GLOBAL_IPBRR1_tag IPBRR1;    /* IP block revision register 1 */
  uint8_t NETC_F3_GLOBAL_reserved8[256];
  NETC_F3_GLOBAL_FBLPR_tag FBLPR[2];   /* Function boot loader parameter register 0 */
};


/* ============================================================================
   =============================== Module: NETC_F3_PCI_HDR_TYPE0 ==============
   ============================================================================ */

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t MEM_ACCESS:1;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_ID:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_union_tag { /* PCI PCI-PM capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_union_tag { /* PCI PCI-PM capabilities register */
  vuint16_t R;
  struct {
    vuint16_t VERSION:3;
    vuint16_t _unused_3:8;
    vuint16_t PME_SUPPORT:5;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_union_tag { /* PCI PCI-PM control and status register */
  vuint16_t R;
  struct {
    vuint16_t PWR_STATE:2;
    vuint16_t _unused_2:1;
    vuint16_t NO_SOFT_RST:1;
    vuint16_t _unused_4:12;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_union_tag { /* PCI PCI-PM capabilities data register */
  vuint8_t R;
  struct {
    vuint8_t _unused_0:8;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_union_tag { /* PCI EA capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_union_tag { /* PCI EA capabilities register */
  vuint16_t R;
  struct {
    vuint16_t NUM_ENTRIES:6;
    vuint16_t _unused_6:10;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_union_tag { /* PCI EA per-entry 0 format register */
  vuint32_t R;
  struct {
    vuint32_t ENTRY_SIZE:3;
    vuint32_t _unused_3:1;
    vuint32_t BEI:4;
    vuint32_t PRIM_PROP:8;
    vuint32_t SEC_PROP:8;
    vuint32_t _unused_24:6;
    vuint32_t WRITABLE:1;
    vuint32_t ENABLE:1;
  } B;
} NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_union_tag { /* PCI EA per-entry 0 base register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t S:1;
    vuint32_t BASE:30;
  } B;
} NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_union_tag { /* PCI EA per-entry 0 max offset register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t S:1;
    vuint32_t MAX_OFFSET:30;
  } B;
} NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_union_tag { /* PCI EA per-entry 0 extended base register */
  vuint32_t R;
  struct {
    vuint32_t BASE:32;
  } B;
} NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_tag;

typedef struct NETC_F3_PCI_HDR_TYPE0_NUM_EA_struct_tag {
  NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_FMT_tag PCI_CFC_EA_PE_FMT; /* PCI EA per-entry 0 format register */
  NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_BASE_tag PCI_CFC_EA_PE_BASE; /* PCI EA per-entry 0 base register */
  NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_MAXOFF_tag PCI_CFC_EA_PE_MAXOFF; /* PCI EA per-entry 0 max offset register */
  NETC_F3_PCI_HDR_TYPE0_NUM_EA_PCI_CFC_EA_PE_EXT_BASE_tag PCI_CFC_EA_PE_EXT_BASE; /* PCI EA per-entry 0 extended base register */
} NETC_F3_PCI_HDR_TYPE0_NUM_EA_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:21;
    vuint32_t ACS_VIOLATION_STAT:1;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_union_tag { /* PCIe SR-IOV capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CAP_union_tag { /* PCIe SR-IOV capability register */
  vuint32_t R;
  struct {
    vuint32_t VF_MIGRATION_CAP:1;
    vuint32_t ARI_CAP_HIER_PRSV:1;
    vuint32_t _unused_2:19;
    vuint32_t VF_MIGRATION_ING_MSG_NUM:11;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CAP_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CTL_union_tag { /* PCIe SR-IOV control register */
  vuint16_t R;
  struct {
    vuint16_t VF_ENABLE:1;
    vuint16_t VF_MIGRATION_ENABLE:1;
    vuint16_t VF_MIGRATION_INT_ENABLE:1;
    vuint16_t VF_MSE:1;
    vuint16_t ARI_CAP_HIERARCHY:1;
    vuint16_t _unused_5:5;
    vuint16_t D2_SUPPORT:1;
    vuint16_t PME_SUPPORT:5;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CTL_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_STAT_union_tag { /* PCIe SR-IOV status register */
  vuint16_t R;
  struct {
    vuint16_t VF_MIGRATION_STATUS:1;
    vuint16_t _unused_1:15;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_STAT_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_INIT_VFS_union_tag { /* PCIe SR-IOV initial VFs register */
  vuint16_t R;
  struct {
    vuint16_t INITIAL_VFS:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_INIT_VFS_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS_union_tag { /* PCIe SR-IOV total VFs register */
  vuint16_t R;
  struct {
    vuint16_t TOTAL_VFS:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_NUM_VFS_union_tag { /* PCIe SR-IOV num VFs register */
  vuint16_t R;
  struct {
    vuint16_t NUM_VFS:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_NUM_VFS_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST_union_tag { /* PCIe SR-IOV function dependency list register */
  vuint16_t R;
  struct {
    vuint16_t FUNC_DEP_LIST:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF_union_tag { /* PCIe SR-IOV first VF offset register */
  vuint16_t R;
  struct {
    vuint16_t FIRST_VF_OFFSET:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE_union_tag { /* PCIe SR-IOV VF stride register */
  vuint16_t R;
  struct {
    vuint16_t VF_STRIDE:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID_union_tag { /* PCIe SR-IOV VF device ID register */
  vuint16_t R;
  struct {
    vuint16_t VF_DEVICE_ID:16;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES_union_tag { /* PCIe SR-IOV supported page sizes ID register */
  vuint32_t R;
  struct {
    vuint32_t SUP_PAGE_SIZES:32;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE_union_tag { /* PCIe SR-IOV system page size ID register */
  vuint32_t R;
  struct {
    vuint32_t SYS_PAGE_SIZE:32;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR_union_tag { /* PCIe SR-IOV VF base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR_tag;

typedef union NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF_union_tag { /* PCIe SR-IOV VF migration state array offset register */
  vuint32_t R;
  struct {
    vuint32_t VF_MIG_STATE_ARR_OFF:32;
  } B;
} NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF_tag;

struct NETC_F3_PCI_HDR_TYPE0_tag {
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved0[1];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved1[1];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved2[4];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved3[4];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved4[11];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved5[24];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved6[22];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved7[4];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST_tag PCI_CFC_PCIPM_CAP_LIST; /* PCI PCI-PM capabilities list register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_tag PCI_CFC_PCIPM_CAP; /* PCI PCI-PM capabilities register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT_tag PCI_CFC_PCIPM_CTL_STAT; /* PCI PCI-PM control and status register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved8[1];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA_tag PCI_CFC_PCIPM_DATA; /* PCI PCI-PM capabilities data register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved9[4];
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST_tag PCI_CFC_EA_CAP_LIST; /* PCI EA capabilities list register */
  NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_tag PCI_CFC_EA_CAP; /* PCI EA capabilities register */
  NETC_F3_PCI_HDR_TYPE0_NUM_EA_tag NUM_EA[4];
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved10[32];
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved11[20];
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved12[8];
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved13[4];
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_tag PCIE_CFC_SRIOV_CAP_HDR; /* PCIe SR-IOV capability header */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CAP_tag PCIE_CFC_SRIOV_CAP; /* PCIe SR-IOV capability register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CTL_tag PCIE_CFC_SRIOV_CTL; /* PCIe SR-IOV control register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_STAT_tag PCIE_CFC_SRIOV_STAT; /* PCIe SR-IOV status register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_INIT_VFS_tag PCIE_CFC_SRIOV_INIT_VFS; /* PCIe SR-IOV initial VFs register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS_tag PCIE_CFC_SRIOV_TOTAL_VFS; /* PCIe SR-IOV total VFs register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_NUM_VFS_tag PCIE_CFC_SRIOV_NUM_VFS; /* PCIe SR-IOV num VFs register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST_tag PCIE_CFC_SRIOV_FUNC_DEP_LIST; /* PCIe SR-IOV function dependency list register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF_tag PCIE_CFC_SRIOV_FIRST_VF_OFF; /* PCIe SR-IOV first VF offset register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE_tag PCIE_CFC_SRIOV_VF_STRIDE; /* PCIe SR-IOV VF stride register */
  uint8_t NETC_F3_PCI_HDR_TYPE0_reserved14[2];
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID_tag PCIE_CFC_SRIOV_VF_DEV_ID; /* PCIe SR-IOV VF device ID register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES_tag PCIE_CFC_SRIOV_SUP_PAGE_SIZES; /* PCIe SR-IOV supported page sizes ID register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE_tag PCIE_CFC_SRIOV_SYS_PAGE_SIZE; /* PCIe SR-IOV system page size ID register */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR_tag PCIE_CFC_VF_BAR[6]; /* PCIe SR-IOV VF base address register 0 */
  NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF_tag PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF; /* PCIe SR-IOV VF migration state array offset register */
};


/* ============================================================================
   =============================== Module: NETC_F3_SI0 ========================
   ============================================================================ */

typedef union NETC_F3_SI0_SIMR_union_tag { /* Station interface mode register */
  vuint32_t R;
  struct {
    vuint32_t RSSE:1;
    vuint32_t RNUM:1;
    vuint32_t RNMM:1;
    vuint32_t RNBM:1;
    vuint32_t V2IPVE:1;
    vuint32_t _unused_5:11;
    vuint32_t DEFAULT_RX_GROUP:1;
    vuint32_t _unused_17:14;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI0_SIMR_tag;

typedef union NETC_F3_SI0_SISR_union_tag { /* Station interface status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;
    vuint32_t MAC_UP:1;
    vuint32_t MAC_MP:1;
    vuint32_t VLAN_P:1;
    vuint32_t VLAN_UTA:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_SI0_SISR_tag;

typedef union NETC_F3_SI0_SICTR0_union_tag { /* Station interface current time register 0 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI0_SICTR0_tag;

typedef union NETC_F3_SI0_SICTR1_union_tag { /* Station interface current time register 1 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI0_SICTR1_tag;

typedef union NETC_F3_SI0_SIPCAPR0_union_tag { /* Station interface port capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;                   /* Receive Flow Steering */
    vuint32_t FP:1;                    /* Frame Preemption */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t TSD:1;                   /* Time Specific Departure */
    vuint32_t CBS:1;                   /* Credit Based Shaping (CBS) */
    vuint32_t _unused_7:1;
    vuint32_t RSS:1;                   /* Receive Side Scaling */
    vuint32_t PSFP:1;                  /* Per-Stream Filtering and Policing (IEEE 802.1Qci) */
    vuint32_t IPFLT:1;                 /* Ingress Port Filtering */
    vuint32_t RP:1;                    /* Rate Policing */
    vuint32_t _unused_12:1;
    vuint32_t WO:1;                    /* Wake-on-LAN */
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional Safety */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI0_SIPCAPR0_tag;

typedef union NETC_F3_SI0_SIPCAPR1_union_tag { /* Station interface port capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:6;
    vuint32_t _unused_18:13;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_SI0_SIPCAPR1_tag;

typedef union NETC_F3_SI0_SITSR_union_tag { /* Station interface timer status register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;                  /* Timer synchronization */
    vuint32_t PARAM_VAL:31;            /* User specific parameter values */
  } B;
} NETC_F3_SI0_SITSR_tag;

typedef union NETC_F3_SI0_SIRBGCR_union_tag { /* Station interface receive BDR group control register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GROUPS:2;
    vuint32_t _unused_2:14;
    vuint32_t RINGS_PER_GROUP:3;
    vuint32_t _unused_19:13;
  } B;
} NETC_F3_SI0_SIRBGCR_tag;

typedef union NETC_F3_SI0_SIBCAR_union_tag { /* Station interface buffer cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI0_SIBCAR_tag;

typedef union NETC_F3_SI0_SIMCAR_union_tag { /* Station interface message cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t MSG_WRCACHE:4;
    vuint32_t MSG_WRDOMAIN:2;
    vuint32_t MSG_WRSNP:1;
    vuint32_t _unused_7:9;
    vuint32_t MSG_RDCACHE:4;
    vuint32_t MSG_RDDOMAIN:2;
    vuint32_t MSG_RDSNP:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_F3_SI0_SIMCAR_tag;

typedef union NETC_F3_SI0_SICCAR_union_tag { /* Station interface command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI0_SICCAR_tag;

typedef union NETC_F3_SI0_SIPMAR0_union_tag { /* Station interface primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_SI0_SIPMAR0_tag;

typedef union NETC_F3_SI0_SIPMAR1_union_tag { /* Station interface primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI0_SIPMAR1_tag;

typedef union NETC_F3_SI0_SICVLANR1_union_tag { /* Station interface custom VLAN register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI0_SICVLANR1_tag;

typedef union NETC_F3_SI0_SICVLANR2_union_tag { /* Station interface custom VLAN register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI0_SICVLANR2_tag;

typedef union NETC_F3_SI0_SIVLANIPVMR0_union_tag { /* Station interface VLAN to IPV mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:4;
    vuint32_t PCP_DEI_1:4;
    vuint32_t PCP_DEI_2:4;
    vuint32_t PCP_DEI_3:4;
    vuint32_t PCP_DEI_4:4;
    vuint32_t PCP_DEI_5:4;
    vuint32_t PCP_DEI_6:4;
    vuint32_t PCP_DEI_7:4;
  } B;
} NETC_F3_SI0_SIVLANIPVMR0_tag;

typedef union NETC_F3_SI0_SIVLANIPVMR1_union_tag { /* Station interface VLAN to IPV mapping register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:4;
    vuint32_t PCP_DEI_9:4;
    vuint32_t PCP_DEI_10:4;
    vuint32_t PCP_DEI_11:4;
    vuint32_t PCP_DEI_12:4;
    vuint32_t PCP_DEI_13:4;
    vuint32_t PCP_DEI_14:4;
    vuint32_t PCP_DEI_15:4;
  } B;
} NETC_F3_SI0_SIVLANIPVMR1_tag;

typedef union NETC_F3_SI0_SIIPVBDRMR0_union_tag { /* Station interface IPV to ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPV0BDR:3;
    vuint32_t _unused_3:1;
    vuint32_t IPV1BDR:3;
    vuint32_t _unused_7:1;
    vuint32_t IPV2BDR:3;
    vuint32_t _unused_11:1;
    vuint32_t IPV3BDR:3;
    vuint32_t _unused_15:1;
    vuint32_t IPV4BDR:3;
    vuint32_t _unused_19:1;
    vuint32_t IPV5BDR:3;
    vuint32_t _unused_23:1;
    vuint32_t IPV6BDR:3;
    vuint32_t _unused_27:1;
    vuint32_t IPV7BDR:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI0_SIIPVBDRMR0_tag;

typedef union NETC_F3_SI0_PSI_A_PSIMSGRR_union_tag { /* Physical station interface message receive register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t MR1:1;
    vuint32_t MR2:1;
    vuint32_t MR3:1;
    vuint32_t MR4:1;
    vuint32_t MR5:1;
    vuint32_t MR6:1;
    vuint32_t MR7:1;
    vuint32_t _unused_8:8;
    vuint32_t MC:16;                   /* Message code */
  } B;
} NETC_F3_SI0_PSI_A_PSIMSGRR_tag;

typedef union NETC_F3_SI0_PSI_A_PSIMSGSR_union_tag { /* Physical station interface message send register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t MS1:1;
    vuint32_t MS2:1;
    vuint32_t MS3:1;
    vuint32_t MS4:1;
    vuint32_t MS5:1;
    vuint32_t MS6:1;
    vuint32_t MS7:1;
    vuint32_t _unused_8:8;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI0_PSI_A_PSIMSGSR_tag;

typedef union NETC_F3_SI0_PSI_A_VSI_NUM_PSIVMSGRCVAR0_union_tag { /* PSI VSI 1 message receive address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MSIZE:5;
    vuint32_t _unused_5:1;
    vuint32_t ADDRL:26;
  } B;
} NETC_F3_SI0_PSI_A_VSI_NUM_PSIVMSGRCVAR0_tag;

typedef union NETC_F3_SI0_PSI_A_VSI_NUM_PSIVMSGRCVAR1_union_tag { /* PSI VSI 1 message receive address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI0_PSI_A_VSI_NUM_PSIVMSGRCVAR1_tag;

typedef struct NETC_F3_SI0_PSI_A_VSI_NUM_struct_tag {
  NETC_F3_SI0_PSI_A_VSI_NUM_PSIVMSGRCVAR0_tag PSIVMSGRCVAR0; /* PSI VSI 1 message receive address register 0 */
  NETC_F3_SI0_PSI_A_VSI_NUM_PSIVMSGRCVAR1_tag PSIVMSGRCVAR1; /* PSI VSI 1 message receive address register 1 */
} NETC_F3_SI0_PSI_A_VSI_NUM_tag;

typedef struct NETC_F3_SI0_PSI_A_struct_tag {
  NETC_F3_SI0_PSI_A_PSIMSGRR_tag PSIMSGRR; /* Physical station interface message receive register */
  NETC_F3_SI0_PSI_A_PSIMSGSR_tag PSIMSGSR; /* Physical station interface message send register */
  uint8_t PSI_A_reserved0[4];
  NETC_F3_SI0_PSI_A_VSI_NUM_tag VSI_NUM[7];
} NETC_F3_SI0_PSI_A_tag;

typedef union NETC_F3_SI0_SIROCT0_union_tag { /* Station interface receive octets counter (ifInOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_LOW:32;
  } B;
} NETC_F3_SI0_SIROCT0_tag;

typedef union NETC_F3_SI0_SIROCT1_union_tag { /* Station interface receive octets counter (ifInOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_HIGH:32;
  } B;
} NETC_F3_SI0_SIROCT1_tag;

typedef union NETC_F3_SI0_SIRFRM0_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_LOW:32;
  } B;
} NETC_F3_SI0_SIRFRM0_tag;

typedef union NETC_F3_SI0_SIRFRM1_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_HIGH:32;
  } B;
} NETC_F3_SI0_SIRFRM1_tag;

typedef union NETC_F3_SI0_SIRUCA0_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_LOW:32;
  } B;
} NETC_F3_SI0_SIRUCA0_tag;

typedef union NETC_F3_SI0_SIRUCA1_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_HIGH:32;
  } B;
} NETC_F3_SI0_SIRUCA1_tag;

typedef union NETC_F3_SI0_SIRMCA0_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_LOW:32;
  } B;
} NETC_F3_SI0_SIRMCA0_tag;

typedef union NETC_F3_SI0_SIRMCA1_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_HIGH:32;
  } B;
} NETC_F3_SI0_SIRMCA1_tag;

typedef union NETC_F3_SI0_SITOCT0_union_tag { /* Station interface transmit octets counter (ifOutOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_LOW:32;
  } B;
} NETC_F3_SI0_SITOCT0_tag;

typedef union NETC_F3_SI0_SITOCT1_union_tag { /* Station interface transmit octets counter (ifOutOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_HIGH:32;
  } B;
} NETC_F3_SI0_SITOCT1_tag;

typedef union NETC_F3_SI0_SITFRM0_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_LOW:32;
  } B;
} NETC_F3_SI0_SITFRM0_tag;

typedef union NETC_F3_SI0_SITFRM1_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_HIGH:32;
  } B;
} NETC_F3_SI0_SITFRM1_tag;

typedef union NETC_F3_SI0_SITUCA0_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_LOW:32;
  } B;
} NETC_F3_SI0_SITUCA0_tag;

typedef union NETC_F3_SI0_SITUCA1_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_HIGH:32;
  } B;
} NETC_F3_SI0_SITUCA1_tag;

typedef union NETC_F3_SI0_SITMCA0_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_LOW:32;
  } B;
} NETC_F3_SI0_SITMCA0_tag;

typedef union NETC_F3_SI0_SITMCA1_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_HIGH:32;
  } B;
} NETC_F3_SI0_SITMCA1_tag;

typedef union NETC_F3_SI0_SICBDRMR_union_tag { /* Station interface command BDR mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI0_SICBDRMR_tag;

typedef union NETC_F3_SI0_SICBDRSR_union_tag { /* Station interface command BDR status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI0_SICBDRSR_tag;

typedef union NETC_F3_SI0_SICBDRBAR0_union_tag { /* Station interface command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI0_SICBDRBAR0_tag;

typedef union NETC_F3_SI0_SICBDRBAR1_union_tag { /* Station interface command BDR base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI0_SICBDRBAR1_tag;

typedef union NETC_F3_SI0_SICBDRPIR_union_tag { /* Station interface command BDR producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI0_SICBDRPIR_tag;

typedef union NETC_F3_SI0_SICBDRCIR_union_tag { /* Station interface command BDR consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI0_SICBDRCIR_tag;

typedef union NETC_F3_SI0_SICBDRLENR_union_tag { /* Station interface command BDR length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_SI0_SICBDRLENR_tag;

typedef union NETC_F3_SI0_SICBDRIER_union_tag { /* Station interface command BDR interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI0_SICBDRIER_tag;

typedef union NETC_F3_SI0_SICBDRIDR_union_tag { /* Station interface command BDR interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI0_SICBDRIDR_tag;

typedef union NETC_F3_SI0_SICAPR0_union_tag { /* Station interface capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_RX_BDR:8;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F3_SI0_SICAPR0_tag;

typedef union NETC_F3_SI0_SICAPR1_union_tag { /* Station interface capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t NUM_RX_GRP:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI0_SICAPR1_tag;

typedef union NETC_F3_SI0_SICAPR2_union_tag { /* Station interface capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t VTP:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_SI0_SICAPR2_tag;

typedef union NETC_F3_SI0_PSI_PSIIER_union_tag { /* Physical station interface interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t MR1IE:1;
    vuint32_t MR2IE:1;
    vuint32_t MR3IE:1;
    vuint32_t MR4IE:1;
    vuint32_t MR5IE:1;
    vuint32_t MR6IE:1;
    vuint32_t MR7IE:1;
    vuint32_t _unused_8:9;
    vuint32_t FLR1IE:1;
    vuint32_t FLR2IE:1;
    vuint32_t FLR3IE:1;
    vuint32_t FLR4IE:1;
    vuint32_t FLR5IE:1;
    vuint32_t FLR6IE:1;
    vuint32_t FLR7IE:1;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI0_PSI_PSIIER_tag;

typedef union NETC_F3_SI0_PSI_PSIIDR_union_tag { /* Physical station interface interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXR:1;
    vuint32_t MR1:1;
    vuint32_t MR2:1;
    vuint32_t MR3:1;
    vuint32_t MR4:1;
    vuint32_t MR5:1;
    vuint32_t MR6:1;
    vuint32_t MR7:1;
    vuint32_t _unused_8:8;
    vuint32_t RXR:1;
    vuint32_t FLR1:1;
    vuint32_t FLR2:1;
    vuint32_t FLR3:1;
    vuint32_t FLR4:1;
    vuint32_t FLR5:1;
    vuint32_t FLR6:1;
    vuint32_t FLR7:1;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI0_PSI_PSIIDR_tag;

typedef struct NETC_F3_SI0_PSI_struct_tag {
  NETC_F3_SI0_PSI_PSIIER_tag PSIIER;   /* Physical station interface interrupt enable register */
  uint8_t PSI_reserved0[4];
  NETC_F3_SI0_PSI_PSIIDR_tag PSIIDR;   /* Physical station interface interrupt detect register */
} NETC_F3_SI0_PSI_tag;

typedef union NETC_F3_SI0_SITXIDR0_union_tag { /* Station interface transmit interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXT0:1;
    vuint32_t TXT1:1;
    vuint32_t TXT2:1;
    vuint32_t TXT3:1;
    vuint32_t TXT4:1;
    vuint32_t TXT5:1;
    vuint32_t TXT6:1;
    vuint32_t TXT7:1;
    vuint32_t TXT8:1;
    vuint32_t TXT9:1;
    vuint32_t TXT10:1;
    vuint32_t TXT11:1;
    vuint32_t TXT12:1;
    vuint32_t TXT13:1;
    vuint32_t TXT14:1;
    vuint32_t TXT15:1;
    vuint32_t TXF0:1;
    vuint32_t TXF1:1;
    vuint32_t TXF2:1;
    vuint32_t TXF3:1;
    vuint32_t TXF4:1;
    vuint32_t TXF5:1;
    vuint32_t TXF6:1;
    vuint32_t TXF7:1;
    vuint32_t TXF8:1;
    vuint32_t TXF9:1;
    vuint32_t TXF10:1;
    vuint32_t TXF11:1;
    vuint32_t TXF12:1;
    vuint32_t TXF13:1;
    vuint32_t TXF14:1;
    vuint32_t TXF15:1;
  } B;
} NETC_F3_SI0_SITXIDR0_tag;

typedef union NETC_F3_SI0_SITXIDR1_union_tag { /* Station interface transmit interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t TXT16:1;
    vuint32_t TXT17:1;
    vuint32_t _unused_2:14;
    vuint32_t TXF16:1;
    vuint32_t TXF17:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_SI0_SITXIDR1_tag;

typedef union NETC_F3_SI0_SIRXIDR0_union_tag { /* Station interface receive interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t RX0:1;
    vuint32_t RX1:1;
    vuint32_t RX2:1;
    vuint32_t RX3:1;
    vuint32_t RX4:1;
    vuint32_t RX5:1;
    vuint32_t RX6:1;
    vuint32_t RX7:1;
    vuint32_t RX8:1;
    vuint32_t RX9:1;
    vuint32_t RX10:1;
    vuint32_t RX11:1;
    vuint32_t RX12:1;
    vuint32_t RX13:1;
    vuint32_t RX14:1;
    vuint32_t RX15:1;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI0_SIRXIDR0_tag;

typedef union NETC_F3_SI0_SIRXIDR1_union_tag { /* Station interface receive interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t RX16:1;
    vuint32_t RX17:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI0_SIRXIDR1_tag;

typedef union NETC_F3_SI0_SIMSIVR_union_tag { /* Station interface MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI0_SIMSIVR_tag;

typedef union NETC_F3_SI0_SICMSIVR_union_tag { /* Station interface command MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI0_SICMSIVR_tag;

typedef union NETC_F3_SI0_SITMRIER_union_tag { /* Station interface timer interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t SYNCE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI0_SITMRIER_tag;

typedef union NETC_F3_SI0_SITMRIDR_union_tag { /* Station interface timer interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI0_SITMRIDR_tag;

typedef union NETC_F3_SI0_SITMRMSIVR_union_tag { /* Station interface timer MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI0_SITMRMSIVR_tag;

typedef union NETC_F3_SI0_SIMSITRVR_union_tag { /* Station interface MSI-X transmit ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI0_SIMSITRVR_tag;

typedef union NETC_F3_SI0_SIMSIRRVR_union_tag { /* Station interface MSI-X receive ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI0_SIMSIRRVR_tag;

typedef union NETC_F3_SI0_SICMECR_union_tag { /* Station interface correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI0_SICMECR_tag;

typedef union NETC_F3_SI0_SICMESR_union_tag { /* Station interface correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_SI0_SICMESR_tag;

typedef union NETC_F3_SI0_SICMECTR_union_tag { /* Station interface correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI0_SICMECTR_tag;

typedef union NETC_F3_SI0_SIUPECR_union_tag { /* Station interface uncorrectable programming error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI0_SIUPECR_tag;

typedef union NETC_F3_SI0_SIUPESR_union_tag { /* Station interface uncorrectable programming error status register */
  vuint32_t R;
  struct {
    vuint32_t DROP_SI_EN:1;
    vuint32_t DROP_RING_EN:1;
    vuint32_t DROP_GRP_SEL:1;
    vuint32_t DROP_RING_SEL:1;
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t PE:1;                    /* Programming error */
  } B;
} NETC_F3_SI0_SIUPESR_tag;

typedef union NETC_F3_SI0_SIUPECTR_union_tag { /* Station interface uncorrectable programming error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;                /* Count */
  } B;
} NETC_F3_SI0_SIUPECTR_tag;

typedef union NETC_F3_SI0_SIUNSBECR_union_tag { /* Station interface uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI0_SIUNSBECR_tag;

typedef union NETC_F3_SI0_SIUNSBESR_union_tag { /* Station interface uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI0_SIUNSBESR_tag;

typedef union NETC_F3_SI0_SIUNSBECTR_union_tag { /* Station interface uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI0_SIUNSBECTR_tag;

typedef union NETC_F3_SI0_SIUFSBECR_union_tag { /* Station interface uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI0_SIUFSBECR_tag;

typedef union NETC_F3_SI0_SIUFSBESR_union_tag { /* Station interface uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI0_SIUFSBESR_tag;

typedef union NETC_F3_SI0_SIUNMECR_union_tag { /* Station interface uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI0_SIUNMECR_tag;

typedef union NETC_F3_SI0_SIUNMESR0_union_tag { /* Station interface uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI0_SIUNMESR0_tag;

typedef union NETC_F3_SI0_SIUNMESR1_union_tag { /* Station interface uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI0_SIUNMESR1_tag;

typedef union NETC_F3_SI0_SIUNMECTR_union_tag { /* Station interface uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI0_SIUNMECTR_tag;

typedef union NETC_F3_SI0_SIUFMECR_union_tag { /* Station interface uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI0_SIUFMECR_tag;

typedef union NETC_F3_SI0_SIUFMESR0_union_tag { /* Station interface uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI0_SIUFMESR0_tag;

typedef union NETC_F3_SI0_SIUFMESR1_union_tag { /* Station interface uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI0_SIUFMESR1_tag;

typedef union NETC_F3_SI0_SIUNIECR_union_tag { /* Station interface uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI0_SIUNIECR_tag;

typedef union NETC_F3_SI0_SIUNIESR_union_tag { /* Station interface uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI0_SIUNIESR_tag;

typedef union NETC_F3_SI0_SIUNIECTR_union_tag { /* Station interface uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI0_SIUNIECTR_tag;

typedef union NETC_F3_SI0_SIUFIECR_union_tag { /* Station interface uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI0_SIUFIECR_tag;

typedef union NETC_F3_SI0_SIUFIESR_union_tag { /* Station interface uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:16;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI0_SIUFIESR_tag;

typedef union NETC_F3_SI0_SIMAFTCAPR_union_tag { /* Station interface MAC address filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI0_SIMAFTCAPR_tag;

typedef union NETC_F3_SI0_SIVFTCAPR_union_tag { /* Station interface VLAN filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI0_SIVFTCAPR_tag;

typedef union NETC_F3_SI0_SIRFSCAPR_union_tag { /* Station interface RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_SI0_SIRFSCAPR_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBMR_union_tag { /* Tx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t PRIO:3;
    vuint32_t _unused_3:1;
    vuint32_t WRR:3;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t VIH:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI0_BDR_NUM_TBMR_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBSR_union_tag { /* Tx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI0_BDR_NUM_TBSR_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBBAR0_union_tag { /* Tx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI0_BDR_NUM_TBBAR0_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBBAR1_union_tag { /* Tx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI0_BDR_NUM_TBBAR1_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBPIR_union_tag { /* Tx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI0_BDR_NUM_TBPIR_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBCIR_union_tag { /* Tx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t STAT_ID:16;
  } B;
} NETC_F3_SI0_BDR_NUM_TBCIR_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBLENR_union_tag { /* Tx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI0_BDR_NUM_TBLENR_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBIER_union_tag { /* Tx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t TXTIE:1;
    vuint32_t TXFIE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI0_BDR_NUM_TBIER_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBIDR_union_tag { /* Tx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXT:1;
    vuint32_t TXF:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI0_BDR_NUM_TBIDR_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBICR0_union_tag { /* Tx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:4;
    vuint32_t _unused_4:27;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI0_BDR_NUM_TBICR0_tag;

typedef union NETC_F3_SI0_BDR_NUM_TBICR1_union_tag { /* Tx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI0_BDR_NUM_TBICR1_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBMR_union_tag { /* Rx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t AL:1;
    vuint32_t _unused_1:1;
    vuint32_t BDS:1;
    vuint32_t _unused_3:1;
    vuint32_t CM:1;
    vuint32_t VTE:1;
    vuint32_t VTPD:1;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI0_BDR_NUM_RBMR_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBSR_union_tag { /* Rx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI0_BDR_NUM_RBSR_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBBSR_union_tag { /* Rx BDR 0 buffer size register */
  vuint32_t R;
  struct {
    vuint32_t BSIZE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI0_BDR_NUM_RBBSR_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBCIR_union_tag { /* Rx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI0_BDR_NUM_RBCIR_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBBAR0_union_tag { /* Rx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI0_BDR_NUM_RBBAR0_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBBAR1_union_tag { /* Rx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI0_BDR_NUM_RBBAR1_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBPIR_union_tag { /* Rx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI0_BDR_NUM_RBPIR_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBLENR_union_tag { /* Rx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI0_BDR_NUM_RBLENR_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBDCR_union_tag { /* Rx BDR 0 drop count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_SI0_BDR_NUM_RBDCR_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBIER_union_tag { /* Rx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t RXTIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI0_BDR_NUM_RBIER_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBIDR_union_tag { /* Rx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t RXT:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI0_BDR_NUM_RBIDR_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBICR0_union_tag { /* Rx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:9;
    vuint32_t _unused_9:22;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI0_BDR_NUM_RBICR0_tag;

typedef union NETC_F3_SI0_BDR_NUM_RBICR1_union_tag { /* Rx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI0_BDR_NUM_RBICR1_tag;

typedef struct NETC_F3_SI0_BDR_NUM_struct_tag {
  NETC_F3_SI0_BDR_NUM_TBMR_tag TBMR;   /* Tx BDR 0 mode register */
  NETC_F3_SI0_BDR_NUM_TBSR_tag TBSR;   /* Tx BDR 0 status register */
  uint8_t BDR_NUM_reserved0[8];
  NETC_F3_SI0_BDR_NUM_TBBAR0_tag TBBAR0; /* Tx BDR 0 base address register 0 */
  NETC_F3_SI0_BDR_NUM_TBBAR1_tag TBBAR1; /* Tx BDR 0 base address register 1 */
  NETC_F3_SI0_BDR_NUM_TBPIR_tag TBPIR; /* Tx BDR 0 producer index register */
  NETC_F3_SI0_BDR_NUM_TBCIR_tag TBCIR; /* Tx BDR 0 consumer index register */
  NETC_F3_SI0_BDR_NUM_TBLENR_tag TBLENR; /* Tx BDR 0 length register */
  uint8_t BDR_NUM_reserved1[124];
  NETC_F3_SI0_BDR_NUM_TBIER_tag TBIER; /* Tx BDR 0 interrupt enable register */
  NETC_F3_SI0_BDR_NUM_TBIDR_tag TBIDR; /* Tx BDR 0 interrupt detect register */
  NETC_F3_SI0_BDR_NUM_TBICR0_tag TBICR0; /* Tx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI0_BDR_NUM_TBICR1_tag TBICR1; /* Tx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved2[80];
  NETC_F3_SI0_BDR_NUM_RBMR_tag RBMR;   /* Rx BDR 0 mode register */
  NETC_F3_SI0_BDR_NUM_RBSR_tag RBSR;   /* Rx BDR 0 status register */
  NETC_F3_SI0_BDR_NUM_RBBSR_tag RBBSR; /* Rx BDR 0 buffer size register */
  NETC_F3_SI0_BDR_NUM_RBCIR_tag RBCIR; /* Rx BDR 0 consumer index register */
  NETC_F3_SI0_BDR_NUM_RBBAR0_tag RBBAR0; /* Rx BDR 0 base address register 0 */
  NETC_F3_SI0_BDR_NUM_RBBAR1_tag RBBAR1; /* Rx BDR 0 base address register 1 */
  NETC_F3_SI0_BDR_NUM_RBPIR_tag RBPIR; /* Rx BDR 0 producer index register */
  uint8_t BDR_NUM_reserved3[4];
  NETC_F3_SI0_BDR_NUM_RBLENR_tag RBLENR; /* Rx BDR 0 length register */
  uint8_t BDR_NUM_reserved4[92];
  NETC_F3_SI0_BDR_NUM_RBDCR_tag RBDCR; /* Rx BDR 0 drop count register */
  uint8_t BDR_NUM_reserved5[28];
  NETC_F3_SI0_BDR_NUM_RBIER_tag RBIER; /* Rx BDR 0 interrupt enable register */
  NETC_F3_SI0_BDR_NUM_RBIDR_tag RBIDR; /* Rx BDR 0 interrupt detect register */
  NETC_F3_SI0_BDR_NUM_RBICR0_tag RBICR0; /* Rx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI0_BDR_NUM_RBICR1_tag RBICR1; /* Rx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved6[80];
} NETC_F3_SI0_BDR_NUM_tag;

struct NETC_F3_SI0_tag {
  NETC_F3_SI0_SIMR_tag SIMR;           /* Station interface mode register */
  NETC_F3_SI0_SISR_tag SISR;           /* Station interface status register */
  uint8_t NETC_F3_SI0_reserved0[16];
  NETC_F3_SI0_SICTR0_tag SICTR0;       /* Station interface current time register 0 */
  NETC_F3_SI0_SICTR1_tag SICTR1;       /* Station interface current time register 1 */
  NETC_F3_SI0_SIPCAPR0_tag SIPCAPR0;   /* Station interface port capability register 0 */
  NETC_F3_SI0_SIPCAPR1_tag SIPCAPR1;   /* Station interface port capability register 1 */
  uint8_t NETC_F3_SI0_reserved1[8];
  NETC_F3_SI0_SITSR_tag SITSR;         /* Station interface timer status register */
  uint8_t NETC_F3_SI0_reserved2[4];
  NETC_F3_SI0_SIRBGCR_tag SIRBGCR;     /* Station interface receive BDR group control register */
  uint8_t NETC_F3_SI0_reserved3[4];
  NETC_F3_SI0_SIBCAR_tag SIBCAR;       /* Station interface buffer cache attribute register */
  NETC_F3_SI0_SIMCAR_tag SIMCAR;       /* Station interface message cache attribute register */
  NETC_F3_SI0_SICCAR_tag SICCAR;       /* Station interface command cache attribute register */
  uint8_t NETC_F3_SI0_reserved4[52];
  NETC_F3_SI0_SIPMAR0_tag SIPMAR0;     /* Station interface primary MAC address register 0 */
  NETC_F3_SI0_SIPMAR1_tag SIPMAR1;     /* Station interface primary MAC address register 1 */
  uint8_t NETC_F3_SI0_reserved5[8];
  NETC_F3_SI0_SICVLANR1_tag SICVLANR1; /* Station interface custom VLAN register 1 */
  NETC_F3_SI0_SICVLANR2_tag SICVLANR2; /* Station interface custom VLAN register 2 */
  uint8_t NETC_F3_SI0_reserved6[104];
  NETC_F3_SI0_SIVLANIPVMR0_tag SIVLANIPVMR0; /* Station interface VLAN to IPV mapping register 0 */
  NETC_F3_SI0_SIVLANIPVMR1_tag SIVLANIPVMR1; /* Station interface VLAN to IPV mapping register 1 */
  uint8_t NETC_F3_SI0_reserved7[72];
  NETC_F3_SI0_SIIPVBDRMR0_tag SIIPVBDRMR0; /* Station interface IPV to ring mapping register */
  uint8_t NETC_F3_SI0_reserved8[176];
  NETC_F3_SI0_PSI_A_tag PSI_A;
  uint8_t NETC_F3_SI0_reserved9[184];
  NETC_F3_SI0_SIROCT0_tag SIROCT0;     /* Station interface receive octets counter (ifInOctets) 0 */
  NETC_F3_SI0_SIROCT1_tag SIROCT1;     /* Station interface receive octets counter (ifInOctets) 1 */
  NETC_F3_SI0_SIRFRM0_tag SIRFRM0;     /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  NETC_F3_SI0_SIRFRM1_tag SIRFRM1;     /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  NETC_F3_SI0_SIRUCA0_tag SIRUCA0;     /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  NETC_F3_SI0_SIRUCA1_tag SIRUCA1;     /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  NETC_F3_SI0_SIRMCA0_tag SIRMCA0;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  NETC_F3_SI0_SIRMCA1_tag SIRMCA1;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  NETC_F3_SI0_SITOCT0_tag SITOCT0;     /* Station interface transmit octets counter (ifOutOctets) 0 */
  NETC_F3_SI0_SITOCT1_tag SITOCT1;     /* Station interface transmit octets counter (ifOutOctets) 1 */
  NETC_F3_SI0_SITFRM0_tag SITFRM0;     /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  NETC_F3_SI0_SITFRM1_tag SITFRM1;     /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  NETC_F3_SI0_SITUCA0_tag SITUCA0;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  NETC_F3_SI0_SITUCA1_tag SITUCA1;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  NETC_F3_SI0_SITMCA0_tag SITMCA0;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  NETC_F3_SI0_SITMCA1_tag SITMCA1;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  uint8_t NETC_F3_SI0_reserved10[1216];
  NETC_F3_SI0_SICBDRMR_tag SICBDRMR;   /* Station interface command BDR mode register */
  NETC_F3_SI0_SICBDRSR_tag SICBDRSR;   /* Station interface command BDR status register */
  uint8_t NETC_F3_SI0_reserved11[8];
  NETC_F3_SI0_SICBDRBAR0_tag SICBDRBAR0; /* Station interface command BDR base address register 0 */
  NETC_F3_SI0_SICBDRBAR1_tag SICBDRBAR1; /* Station interface command BDR base address register 1 */
  NETC_F3_SI0_SICBDRPIR_tag SICBDRPIR; /* Station interface command BDR producer index register */
  NETC_F3_SI0_SICBDRCIR_tag SICBDRCIR; /* Station interface command BDR consumer index register */
  NETC_F3_SI0_SICBDRLENR_tag SICBDRLENR; /* Station interface command BDR length register */
  uint8_t NETC_F3_SI0_reserved12[124];
  NETC_F3_SI0_SICBDRIER_tag SICBDRIER; /* Station interface command BDR interrupt enable register */
  NETC_F3_SI0_SICBDRIDR_tag SICBDRIDR; /* Station interface command BDR interrupt detect register */
  uint8_t NETC_F3_SI0_reserved13[88];
  NETC_F3_SI0_SICAPR0_tag SICAPR0;     /* Station interface capability register 0 */
  NETC_F3_SI0_SICAPR1_tag SICAPR1;     /* Station interface capability register 1 */
  NETC_F3_SI0_SICAPR2_tag SICAPR2;     /* Station interface capability register 2 */
  uint8_t NETC_F3_SI0_reserved14[244];
  NETC_F3_SI0_PSI_tag PSI;
  uint8_t NETC_F3_SI0_reserved15[12];
  NETC_F3_SI0_SITXIDR0_tag SITXIDR0;   /* Station interface transmit interrupt detect register 0 */
  NETC_F3_SI0_SITXIDR1_tag SITXIDR1;   /* Station interface transmit interrupt detect register 1 */
  uint8_t NETC_F3_SI0_reserved16[8];
  NETC_F3_SI0_SIRXIDR0_tag SIRXIDR0;   /* Station interface receive interrupt detect register 0 */
  NETC_F3_SI0_SIRXIDR1_tag SIRXIDR1;   /* Station interface receive interrupt detect register 1 */
  NETC_F3_SI0_SIMSIVR_tag SIMSIVR;     /* Station interface MSI-X vector register */
  NETC_F3_SI0_SICMSIVR_tag SICMSIVR;   /* Station interface command MSI-X vector register */
  uint8_t NETC_F3_SI0_reserved17[8];
  NETC_F3_SI0_SITMRIER_tag SITMRIER;   /* Station interface timer interrupt enable register */
  NETC_F3_SI0_SITMRIDR_tag SITMRIDR;   /* Station interface timer interrupt detect register */
  uint8_t NETC_F3_SI0_reserved18[4];
  NETC_F3_SI0_SITMRMSIVR_tag SITMRMSIVR; /* Station interface timer MSI-X vector register */
  uint8_t NETC_F3_SI0_reserved19[176];
  NETC_F3_SI0_SIMSITRVR_tag SIMSITRVR[18]; /* Station interface MSI-X transmit ring 0 vector register */
  uint8_t NETC_F3_SI0_reserved20[56];
  NETC_F3_SI0_SIMSIRRVR_tag SIMSIRRVR[18]; /* Station interface MSI-X receive ring 0 vector register */
  uint8_t NETC_F3_SI0_reserved21[568];
  NETC_F3_SI0_SICMECR_tag SICMECR;     /* Station interface correctable memory error configuration register */
  NETC_F3_SI0_SICMESR_tag SICMESR;     /* Station interface correctable memory error status register */
  uint8_t NETC_F3_SI0_reserved22[4];
  NETC_F3_SI0_SICMECTR_tag SICMECTR;   /* Station interface correctable memory error count register */
  NETC_F3_SI0_SIUPECR_tag SIUPECR;     /* Station interface uncorrectable programming error configuration register */
  NETC_F3_SI0_SIUPESR_tag SIUPESR;     /* Station interface uncorrectable programming error status register */
  uint8_t NETC_F3_SI0_reserved23[4];
  NETC_F3_SI0_SIUPECTR_tag SIUPECTR;   /* Station interface uncorrectable programming error count register */
  NETC_F3_SI0_SIUNSBECR_tag SIUNSBECR; /* Station interface uncorrectable non-fatal system bus error configuration register */
  NETC_F3_SI0_SIUNSBESR_tag SIUNSBESR; /* Station interface uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F3_SI0_reserved24[4];
  NETC_F3_SI0_SIUNSBECTR_tag SIUNSBECTR; /* Station interface uncorrectable non-fatal system bus error count register */
  NETC_F3_SI0_SIUFSBECR_tag SIUFSBECR; /* Station interface uncorrectable fatal system bus error configuration register */
  NETC_F3_SI0_SIUFSBESR_tag SIUFSBESR; /* Station interface uncorrectable fatal system bus error status register */
  uint8_t NETC_F3_SI0_reserved25[8];
  NETC_F3_SI0_SIUNMECR_tag SIUNMECR;   /* Station interface uncorrectable non-fatal memory error configuration register */
  NETC_F3_SI0_SIUNMESR0_tag SIUNMESR0; /* Station interface uncorrectable non-fatal memory error status register 0 */
  NETC_F3_SI0_SIUNMESR1_tag SIUNMESR1; /* Station interface uncorrectable non-fatal memory error status register 1 */
  NETC_F3_SI0_SIUNMECTR_tag SIUNMECTR; /* Station interface uncorrectable non-fatal memory error count register */
  NETC_F3_SI0_SIUFMECR_tag SIUFMECR;   /* Station interface uncorrectable fatal memory error configuration register */
  NETC_F3_SI0_SIUFMESR0_tag SIUFMESR0; /* Station interface uncorrectable fatal memory error status register 0 */
  NETC_F3_SI0_SIUFMESR1_tag SIUFMESR1; /* Station interface uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_SI0_reserved26[4];
  NETC_F3_SI0_SIUNIECR_tag SIUNIECR;   /* Station interface uncorrectable non-fatal integrity error configuration register */
  NETC_F3_SI0_SIUNIESR_tag SIUNIESR;   /* Station interface uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_SI0_reserved27[4];
  NETC_F3_SI0_SIUNIECTR_tag SIUNIECTR; /* Station interface uncorrectable non-fatal integrity error count register */
  NETC_F3_SI0_SIUFIECR_tag SIUFIECR;   /* Station interface uncorrectable fatal integrity error configuration register */
  NETC_F3_SI0_SIUFIESR_tag SIUFIESR;   /* Station interface uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_SI0_reserved28[392];
  NETC_F3_SI0_SIMAFTCAPR_tag SIMAFTCAPR; /* Station interface MAC address filter table capability register */
  uint8_t NETC_F3_SI0_reserved29[252];
  NETC_F3_SI0_SIVFTCAPR_tag SIVFTCAPR; /* Station interface VLAN filter table capability register */
  uint8_t NETC_F3_SI0_reserved30[252];
  NETC_F3_SI0_SIRFSCAPR_tag SIRFSCAPR; /* Station interface RFS capability register */
  uint8_t NETC_F3_SI0_reserved31[28156];
  NETC_F3_SI0_BDR_NUM_tag BDR_NUM[18];
};


/* ============================================================================
   =============================== Module: NETC_F3_SI1 ========================
   ============================================================================ */

typedef union NETC_F3_SI1_SIMR_union_tag { /* Station interface mode register */
  vuint32_t R;
  struct {
    vuint32_t RSSE:1;
    vuint32_t RNUM:1;
    vuint32_t RNMM:1;
    vuint32_t RNBM:1;
    vuint32_t V2IPVE:1;
    vuint32_t _unused_5:11;
    vuint32_t DEFAULT_RX_GROUP:1;
    vuint32_t _unused_17:14;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI1_SIMR_tag;

typedef union NETC_F3_SI1_SISR_union_tag { /* Station interface status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;
    vuint32_t MAC_UP:1;
    vuint32_t MAC_MP:1;
    vuint32_t VLAN_P:1;
    vuint32_t VLAN_UTA:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_SI1_SISR_tag;

typedef union NETC_F3_SI1_SICTR0_union_tag { /* Station interface current time register 0 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI1_SICTR0_tag;

typedef union NETC_F3_SI1_SICTR1_union_tag { /* Station interface current time register 1 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI1_SICTR1_tag;

typedef union NETC_F3_SI1_SIPCAPR0_union_tag { /* Station interface port capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;                   /* Receive Flow Steering */
    vuint32_t FP:1;                    /* Frame Preemption */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t TSD:1;                   /* Time Specific Departure */
    vuint32_t CBS:1;                   /* Credit Based Shaping (CBS) */
    vuint32_t _unused_7:1;
    vuint32_t RSS:1;                   /* Receive Side Scaling */
    vuint32_t PSFP:1;                  /* Per-Stream Filtering and Policing (IEEE 802.1Qci) */
    vuint32_t IPFLT:1;                 /* Ingress Port Filtering */
    vuint32_t RP:1;                    /* Rate Policing */
    vuint32_t _unused_12:1;
    vuint32_t WO:1;                    /* Wake-on-LAN */
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional Safety */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI1_SIPCAPR0_tag;

typedef union NETC_F3_SI1_SIPCAPR1_union_tag { /* Station interface port capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:6;
    vuint32_t _unused_18:13;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_SI1_SIPCAPR1_tag;

typedef union NETC_F3_SI1_SIRBGCR_union_tag { /* Station interface receive BDR group control register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GROUPS:2;
    vuint32_t _unused_2:14;
    vuint32_t RINGS_PER_GROUP:3;
    vuint32_t _unused_19:13;
  } B;
} NETC_F3_SI1_SIRBGCR_tag;

typedef union NETC_F3_SI1_SIBCAR_union_tag { /* Station interface buffer cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI1_SIBCAR_tag;

typedef union NETC_F3_SI1_SICCAR_union_tag { /* Station interface command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI1_SICCAR_tag;

typedef union NETC_F3_SI1_SIPMAR0_union_tag { /* Station interface primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_SI1_SIPMAR0_tag;

typedef union NETC_F3_SI1_SIPMAR1_union_tag { /* Station interface primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI1_SIPMAR1_tag;

typedef union NETC_F3_SI1_SICVLANR1_union_tag { /* Station interface custom VLAN register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI1_SICVLANR1_tag;

typedef union NETC_F3_SI1_SICVLANR2_union_tag { /* Station interface custom VLAN register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI1_SICVLANR2_tag;

typedef union NETC_F3_SI1_SIVLANIPVMR0_union_tag { /* Station interface VLAN to IPV mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:4;
    vuint32_t PCP_DEI_1:4;
    vuint32_t PCP_DEI_2:4;
    vuint32_t PCP_DEI_3:4;
    vuint32_t PCP_DEI_4:4;
    vuint32_t PCP_DEI_5:4;
    vuint32_t PCP_DEI_6:4;
    vuint32_t PCP_DEI_7:4;
  } B;
} NETC_F3_SI1_SIVLANIPVMR0_tag;

typedef union NETC_F3_SI1_SIVLANIPVMR1_union_tag { /* Station interface VLAN to IPV mapping register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:4;
    vuint32_t PCP_DEI_9:4;
    vuint32_t PCP_DEI_10:4;
    vuint32_t PCP_DEI_11:4;
    vuint32_t PCP_DEI_12:4;
    vuint32_t PCP_DEI_13:4;
    vuint32_t PCP_DEI_14:4;
    vuint32_t PCP_DEI_15:4;
  } B;
} NETC_F3_SI1_SIVLANIPVMR1_tag;

typedef union NETC_F3_SI1_SIIPVBDRMR0_union_tag { /* Station interface IPV to ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPV0BDR:3;
    vuint32_t _unused_3:1;
    vuint32_t IPV1BDR:3;
    vuint32_t _unused_7:1;
    vuint32_t IPV2BDR:3;
    vuint32_t _unused_11:1;
    vuint32_t IPV3BDR:3;
    vuint32_t _unused_15:1;
    vuint32_t IPV4BDR:3;
    vuint32_t _unused_19:1;
    vuint32_t IPV5BDR:3;
    vuint32_t _unused_23:1;
    vuint32_t IPV6BDR:3;
    vuint32_t _unused_27:1;
    vuint32_t IPV7BDR:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI1_SIIPVBDRMR0_tag;

typedef union NETC_F3_SI1_VSI_A_VSIMSGSR_union_tag { /* Virtual station interface message send register */
  vuint32_t R;
  struct {
    vuint32_t MB:1;
    vuint32_t MS:1;
    vuint32_t _unused_2:14;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI1_VSI_A_VSIMSGSR_tag;

typedef union NETC_F3_SI1_VSI_A_VSIMSGRR_union_tag { /* Virtual station interface message receive register */
  vuint32_t R;
  struct {
    vuint32_t MR:1;
    vuint32_t _unused_1:15;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI1_VSI_A_VSIMSGRR_tag;

typedef union NETC_F3_SI1_VSI_A_VSIMSGSNDAR0_union_tag { /* Virtual station interface message send register 0 */
  vuint32_t R;
  struct {
    vuint32_t MSIZE:5;
    vuint32_t _unused_5:1;
    vuint32_t ADDRL:26;
  } B;
} NETC_F3_SI1_VSI_A_VSIMSGSNDAR0_tag;

typedef union NETC_F3_SI1_VSI_A_VSIMSGSNDAR1_union_tag { /* Virtual station interface message send address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI1_VSI_A_VSIMSGSNDAR1_tag;

typedef struct NETC_F3_SI1_VSI_A_struct_tag {
  NETC_F3_SI1_VSI_A_VSIMSGSR_tag VSIMSGSR; /* Virtual station interface message send register */
  NETC_F3_SI1_VSI_A_VSIMSGRR_tag VSIMSGRR; /* Virtual station interface message receive register */
  uint8_t VSI_A_reserved0[4];
  NETC_F3_SI1_VSI_A_VSIMSGSNDAR0_tag VSIMSGSNDAR0; /* Virtual station interface message send register 0 */
  NETC_F3_SI1_VSI_A_VSIMSGSNDAR1_tag VSIMSGSNDAR1; /* Virtual station interface message send address register 1 */
} NETC_F3_SI1_VSI_A_tag;

typedef union NETC_F3_SI1_SIROCT0_union_tag { /* Station interface receive octets counter (ifInOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_LOW:32;
  } B;
} NETC_F3_SI1_SIROCT0_tag;

typedef union NETC_F3_SI1_SIROCT1_union_tag { /* Station interface receive octets counter (ifInOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_HIGH:32;
  } B;
} NETC_F3_SI1_SIROCT1_tag;

typedef union NETC_F3_SI1_SIRFRM0_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_LOW:32;
  } B;
} NETC_F3_SI1_SIRFRM0_tag;

typedef union NETC_F3_SI1_SIRFRM1_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_HIGH:32;
  } B;
} NETC_F3_SI1_SIRFRM1_tag;

typedef union NETC_F3_SI1_SIRUCA0_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_LOW:32;
  } B;
} NETC_F3_SI1_SIRUCA0_tag;

typedef union NETC_F3_SI1_SIRUCA1_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_HIGH:32;
  } B;
} NETC_F3_SI1_SIRUCA1_tag;

typedef union NETC_F3_SI1_SIRMCA0_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_LOW:32;
  } B;
} NETC_F3_SI1_SIRMCA0_tag;

typedef union NETC_F3_SI1_SIRMCA1_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_HIGH:32;
  } B;
} NETC_F3_SI1_SIRMCA1_tag;

typedef union NETC_F3_SI1_SITOCT0_union_tag { /* Station interface transmit octets counter (ifOutOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_LOW:32;
  } B;
} NETC_F3_SI1_SITOCT0_tag;

typedef union NETC_F3_SI1_SITOCT1_union_tag { /* Station interface transmit octets counter (ifOutOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_HIGH:32;
  } B;
} NETC_F3_SI1_SITOCT1_tag;

typedef union NETC_F3_SI1_SITFRM0_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_LOW:32;
  } B;
} NETC_F3_SI1_SITFRM0_tag;

typedef union NETC_F3_SI1_SITFRM1_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_HIGH:32;
  } B;
} NETC_F3_SI1_SITFRM1_tag;

typedef union NETC_F3_SI1_SITUCA0_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_LOW:32;
  } B;
} NETC_F3_SI1_SITUCA0_tag;

typedef union NETC_F3_SI1_SITUCA1_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_HIGH:32;
  } B;
} NETC_F3_SI1_SITUCA1_tag;

typedef union NETC_F3_SI1_SITMCA0_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_LOW:32;
  } B;
} NETC_F3_SI1_SITMCA0_tag;

typedef union NETC_F3_SI1_SITMCA1_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_HIGH:32;
  } B;
} NETC_F3_SI1_SITMCA1_tag;

typedef union NETC_F3_SI1_SIBLPR_union_tag { /* Station interface boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F3_SI1_SIBLPR_tag;

typedef union NETC_F3_SI1_SICBDRMR_union_tag { /* Station interface command BDR mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI1_SICBDRMR_tag;

typedef union NETC_F3_SI1_SICBDRSR_union_tag { /* Station interface command BDR status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI1_SICBDRSR_tag;

typedef union NETC_F3_SI1_SICBDRBAR0_union_tag { /* Station interface command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI1_SICBDRBAR0_tag;

typedef union NETC_F3_SI1_SICBDRBAR1_union_tag { /* Station interface command BDR base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI1_SICBDRBAR1_tag;

typedef union NETC_F3_SI1_SICBDRPIR_union_tag { /* Station interface command BDR producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI1_SICBDRPIR_tag;

typedef union NETC_F3_SI1_SICBDRCIR_union_tag { /* Station interface command BDR consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI1_SICBDRCIR_tag;

typedef union NETC_F3_SI1_SICBDRLENR_union_tag { /* Station interface command BDR length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_SI1_SICBDRLENR_tag;

typedef union NETC_F3_SI1_SICBDRIER_union_tag { /* Station interface command BDR interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI1_SICBDRIER_tag;

typedef union NETC_F3_SI1_SICBDRIDR_union_tag { /* Station interface command BDR interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI1_SICBDRIDR_tag;

typedef union NETC_F3_SI1_SICAPR0_union_tag { /* Station interface capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_RX_BDR:8;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F3_SI1_SICAPR0_tag;

typedef union NETC_F3_SI1_SICAPR1_union_tag { /* Station interface capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t NUM_RX_GRP:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI1_SICAPR1_tag;

typedef union NETC_F3_SI1_SICAPR2_union_tag { /* Station interface capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t VTP:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_SI1_SICAPR2_tag;

typedef union NETC_F3_SI1_VSI_VSIIER_union_tag { /* Virtual station interface interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MSIE:1;
    vuint32_t MRIE:1;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI1_VSI_VSIIER_tag;

typedef union NETC_F3_SI1_VSI_VSIIDR_union_tag { /* Virtual station interface interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXR:1;
    vuint32_t _unused_1:7;
    vuint32_t MS:1;
    vuint32_t MR:1;
    vuint32_t _unused_10:6;
    vuint32_t RXR:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI1_VSI_VSIIDR_tag;

typedef struct NETC_F3_SI1_VSI_struct_tag {
  NETC_F3_SI1_VSI_VSIIER_tag VSIIER;   /* Virtual station interface interrupt enable register */
  uint8_t VSI_reserved0[4];
  NETC_F3_SI1_VSI_VSIIDR_tag VSIIDR;   /* Virtual station interface interrupt detect register */
} NETC_F3_SI1_VSI_tag;

typedef union NETC_F3_SI1_SITXIDR0_union_tag { /* Station interface transmit interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXT0:1;
    vuint32_t TXT1:1;
    vuint32_t TXT2:1;
    vuint32_t TXT3:1;
    vuint32_t TXT4:1;
    vuint32_t TXT5:1;
    vuint32_t TXT6:1;
    vuint32_t TXT7:1;
    vuint32_t TXT8:1;
    vuint32_t TXT9:1;
    vuint32_t TXT10:1;
    vuint32_t TXT11:1;
    vuint32_t TXT12:1;
    vuint32_t TXT13:1;
    vuint32_t TXT14:1;
    vuint32_t TXT15:1;
    vuint32_t TXF0:1;
    vuint32_t TXF1:1;
    vuint32_t TXF2:1;
    vuint32_t TXF3:1;
    vuint32_t TXF4:1;
    vuint32_t TXF5:1;
    vuint32_t TXF6:1;
    vuint32_t TXF7:1;
    vuint32_t TXF8:1;
    vuint32_t TXF9:1;
    vuint32_t TXF10:1;
    vuint32_t TXF11:1;
    vuint32_t TXF12:1;
    vuint32_t TXF13:1;
    vuint32_t TXF14:1;
    vuint32_t TXF15:1;
  } B;
} NETC_F3_SI1_SITXIDR0_tag;

typedef union NETC_F3_SI1_SITXIDR1_union_tag { /* Station interface transmit interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t TXT16:1;
    vuint32_t TXT17:1;
    vuint32_t _unused_2:14;
    vuint32_t TXF16:1;
    vuint32_t TXF17:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_SI1_SITXIDR1_tag;

typedef union NETC_F3_SI1_SIRXIDR0_union_tag { /* Station interface receive interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t RX0:1;
    vuint32_t RX1:1;
    vuint32_t RX2:1;
    vuint32_t RX3:1;
    vuint32_t RX4:1;
    vuint32_t RX5:1;
    vuint32_t RX6:1;
    vuint32_t RX7:1;
    vuint32_t RX8:1;
    vuint32_t RX9:1;
    vuint32_t RX10:1;
    vuint32_t RX11:1;
    vuint32_t RX12:1;
    vuint32_t RX13:1;
    vuint32_t RX14:1;
    vuint32_t RX15:1;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI1_SIRXIDR0_tag;

typedef union NETC_F3_SI1_SIRXIDR1_union_tag { /* Station interface receive interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t RX16:1;
    vuint32_t RX17:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI1_SIRXIDR1_tag;

typedef union NETC_F3_SI1_SIMSIVR_union_tag { /* Station interface MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI1_SIMSIVR_tag;

typedef union NETC_F3_SI1_SICMSIVR_union_tag { /* Station interface command MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI1_SICMSIVR_tag;

typedef union NETC_F3_SI1_SITMRIER_union_tag { /* Station interface timer interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t SYNCE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI1_SITMRIER_tag;

typedef union NETC_F3_SI1_SITMRIDR_union_tag { /* Station interface timer interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI1_SITMRIDR_tag;

typedef union NETC_F3_SI1_SITMRMSIVR_union_tag { /* Station interface timer MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI1_SITMRMSIVR_tag;

typedef union NETC_F3_SI1_SIMSITRVR_union_tag { /* Station interface MSI-X transmit ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI1_SIMSITRVR_tag;

typedef union NETC_F3_SI1_SIMSIRRVR_union_tag { /* Station interface MSI-X receive ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI1_SIMSIRRVR_tag;

typedef union NETC_F3_SI1_SICMECR_union_tag { /* Station interface correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI1_SICMECR_tag;

typedef union NETC_F3_SI1_SICMESR_union_tag { /* Station interface correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_SI1_SICMESR_tag;

typedef union NETC_F3_SI1_SICMECTR_union_tag { /* Station interface correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI1_SICMECTR_tag;

typedef union NETC_F3_SI1_SIUPECR_union_tag { /* Station interface uncorrectable programming error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI1_SIUPECR_tag;

typedef union NETC_F3_SI1_SIUPESR_union_tag { /* Station interface uncorrectable programming error status register */
  vuint32_t R;
  struct {
    vuint32_t DROP_SI_EN:1;
    vuint32_t DROP_RING_EN:1;
    vuint32_t DROP_GRP_SEL:1;
    vuint32_t DROP_RING_SEL:1;
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t PE:1;                    /* Programming error */
  } B;
} NETC_F3_SI1_SIUPESR_tag;

typedef union NETC_F3_SI1_SIUPECTR_union_tag { /* Station interface uncorrectable programming error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;                /* Count */
  } B;
} NETC_F3_SI1_SIUPECTR_tag;

typedef union NETC_F3_SI1_SIUNSBECR_union_tag { /* Station interface uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI1_SIUNSBECR_tag;

typedef union NETC_F3_SI1_SIUNSBESR_union_tag { /* Station interface uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI1_SIUNSBESR_tag;

typedef union NETC_F3_SI1_SIUNSBECTR_union_tag { /* Station interface uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI1_SIUNSBECTR_tag;

typedef union NETC_F3_SI1_SIUFSBECR_union_tag { /* Station interface uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI1_SIUFSBECR_tag;

typedef union NETC_F3_SI1_SIUFSBESR_union_tag { /* Station interface uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI1_SIUFSBESR_tag;

typedef union NETC_F3_SI1_SIUNMECR_union_tag { /* Station interface uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI1_SIUNMECR_tag;

typedef union NETC_F3_SI1_SIUNMESR0_union_tag { /* Station interface uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI1_SIUNMESR0_tag;

typedef union NETC_F3_SI1_SIUNMESR1_union_tag { /* Station interface uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI1_SIUNMESR1_tag;

typedef union NETC_F3_SI1_SIUNMECTR_union_tag { /* Station interface uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI1_SIUNMECTR_tag;

typedef union NETC_F3_SI1_SIUFMECR_union_tag { /* Station interface uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI1_SIUFMECR_tag;

typedef union NETC_F3_SI1_SIUFMESR0_union_tag { /* Station interface uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI1_SIUFMESR0_tag;

typedef union NETC_F3_SI1_SIUFMESR1_union_tag { /* Station interface uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI1_SIUFMESR1_tag;

typedef union NETC_F3_SI1_SIUNIECR_union_tag { /* Station interface uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI1_SIUNIECR_tag;

typedef union NETC_F3_SI1_SIUNIESR_union_tag { /* Station interface uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI1_SIUNIESR_tag;

typedef union NETC_F3_SI1_SIUNIECTR_union_tag { /* Station interface uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI1_SIUNIECTR_tag;

typedef union NETC_F3_SI1_SIUFIECR_union_tag { /* Station interface uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI1_SIUFIECR_tag;

typedef union NETC_F3_SI1_SIUFIESR_union_tag { /* Station interface uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:16;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI1_SIUFIESR_tag;

typedef union NETC_F3_SI1_SIMAFTCAPR_union_tag { /* Station interface MAC address filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI1_SIMAFTCAPR_tag;

typedef union NETC_F3_SI1_SIVFTCAPR_union_tag { /* Station interface VLAN filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI1_SIVFTCAPR_tag;

typedef union NETC_F3_SI1_SIRFSCAPR_union_tag { /* Station interface RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_SI1_SIRFSCAPR_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBMR_union_tag { /* Tx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t PRIO:3;
    vuint32_t _unused_3:1;
    vuint32_t WRR:3;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t VIH:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI1_BDR_NUM_TBMR_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBSR_union_tag { /* Tx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI1_BDR_NUM_TBSR_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBBAR0_union_tag { /* Tx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI1_BDR_NUM_TBBAR0_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBBAR1_union_tag { /* Tx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI1_BDR_NUM_TBBAR1_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBPIR_union_tag { /* Tx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI1_BDR_NUM_TBPIR_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBCIR_union_tag { /* Tx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t STAT_ID:16;
  } B;
} NETC_F3_SI1_BDR_NUM_TBCIR_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBLENR_union_tag { /* Tx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI1_BDR_NUM_TBLENR_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBIER_union_tag { /* Tx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t TXTIE:1;
    vuint32_t TXFIE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI1_BDR_NUM_TBIER_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBIDR_union_tag { /* Tx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXT:1;
    vuint32_t TXF:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI1_BDR_NUM_TBIDR_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBICR0_union_tag { /* Tx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:4;
    vuint32_t _unused_4:27;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI1_BDR_NUM_TBICR0_tag;

typedef union NETC_F3_SI1_BDR_NUM_TBICR1_union_tag { /* Tx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI1_BDR_NUM_TBICR1_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBMR_union_tag { /* Rx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t AL:1;
    vuint32_t _unused_1:1;
    vuint32_t BDS:1;
    vuint32_t _unused_3:1;
    vuint32_t CM:1;
    vuint32_t VTE:1;
    vuint32_t VTPD:1;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI1_BDR_NUM_RBMR_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBSR_union_tag { /* Rx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI1_BDR_NUM_RBSR_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBBSR_union_tag { /* Rx BDR 0 buffer size register */
  vuint32_t R;
  struct {
    vuint32_t BSIZE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI1_BDR_NUM_RBBSR_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBCIR_union_tag { /* Rx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI1_BDR_NUM_RBCIR_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBBAR0_union_tag { /* Rx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI1_BDR_NUM_RBBAR0_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBBAR1_union_tag { /* Rx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI1_BDR_NUM_RBBAR1_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBPIR_union_tag { /* Rx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI1_BDR_NUM_RBPIR_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBLENR_union_tag { /* Rx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI1_BDR_NUM_RBLENR_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBDCR_union_tag { /* Rx BDR 0 drop count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_SI1_BDR_NUM_RBDCR_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBIER_union_tag { /* Rx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t RXTIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI1_BDR_NUM_RBIER_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBIDR_union_tag { /* Rx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t RXT:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI1_BDR_NUM_RBIDR_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBICR0_union_tag { /* Rx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:9;
    vuint32_t _unused_9:22;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI1_BDR_NUM_RBICR0_tag;

typedef union NETC_F3_SI1_BDR_NUM_RBICR1_union_tag { /* Rx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI1_BDR_NUM_RBICR1_tag;

typedef struct NETC_F3_SI1_BDR_NUM_struct_tag {
  NETC_F3_SI1_BDR_NUM_TBMR_tag TBMR;   /* Tx BDR 0 mode register */
  NETC_F3_SI1_BDR_NUM_TBSR_tag TBSR;   /* Tx BDR 0 status register */
  uint8_t BDR_NUM_reserved0[8];
  NETC_F3_SI1_BDR_NUM_TBBAR0_tag TBBAR0; /* Tx BDR 0 base address register 0 */
  NETC_F3_SI1_BDR_NUM_TBBAR1_tag TBBAR1; /* Tx BDR 0 base address register 1 */
  NETC_F3_SI1_BDR_NUM_TBPIR_tag TBPIR; /* Tx BDR 0 producer index register */
  NETC_F3_SI1_BDR_NUM_TBCIR_tag TBCIR; /* Tx BDR 0 consumer index register */
  NETC_F3_SI1_BDR_NUM_TBLENR_tag TBLENR; /* Tx BDR 0 length register */
  uint8_t BDR_NUM_reserved1[124];
  NETC_F3_SI1_BDR_NUM_TBIER_tag TBIER; /* Tx BDR 0 interrupt enable register */
  NETC_F3_SI1_BDR_NUM_TBIDR_tag TBIDR; /* Tx BDR 0 interrupt detect register */
  NETC_F3_SI1_BDR_NUM_TBICR0_tag TBICR0; /* Tx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI1_BDR_NUM_TBICR1_tag TBICR1; /* Tx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved2[80];
  NETC_F3_SI1_BDR_NUM_RBMR_tag RBMR;   /* Rx BDR 0 mode register */
  NETC_F3_SI1_BDR_NUM_RBSR_tag RBSR;   /* Rx BDR 0 status register */
  NETC_F3_SI1_BDR_NUM_RBBSR_tag RBBSR; /* Rx BDR 0 buffer size register */
  NETC_F3_SI1_BDR_NUM_RBCIR_tag RBCIR; /* Rx BDR 0 consumer index register */
  NETC_F3_SI1_BDR_NUM_RBBAR0_tag RBBAR0; /* Rx BDR 0 base address register 0 */
  NETC_F3_SI1_BDR_NUM_RBBAR1_tag RBBAR1; /* Rx BDR 0 base address register 1 */
  NETC_F3_SI1_BDR_NUM_RBPIR_tag RBPIR; /* Rx BDR 0 producer index register */
  uint8_t BDR_NUM_reserved3[4];
  NETC_F3_SI1_BDR_NUM_RBLENR_tag RBLENR; /* Rx BDR 0 length register */
  uint8_t BDR_NUM_reserved4[92];
  NETC_F3_SI1_BDR_NUM_RBDCR_tag RBDCR; /* Rx BDR 0 drop count register */
  uint8_t BDR_NUM_reserved5[28];
  NETC_F3_SI1_BDR_NUM_RBIER_tag RBIER; /* Rx BDR 0 interrupt enable register */
  NETC_F3_SI1_BDR_NUM_RBIDR_tag RBIDR; /* Rx BDR 0 interrupt detect register */
  NETC_F3_SI1_BDR_NUM_RBICR0_tag RBICR0; /* Rx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI1_BDR_NUM_RBICR1_tag RBICR1; /* Rx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved6[80];
} NETC_F3_SI1_BDR_NUM_tag;

struct NETC_F3_SI1_tag {
  NETC_F3_SI1_SIMR_tag SIMR;           /* Station interface mode register */
  NETC_F3_SI1_SISR_tag SISR;           /* Station interface status register */
  uint8_t NETC_F3_SI1_reserved0[16];
  NETC_F3_SI1_SICTR0_tag SICTR0;       /* Station interface current time register 0 */
  NETC_F3_SI1_SICTR1_tag SICTR1;       /* Station interface current time register 1 */
  NETC_F3_SI1_SIPCAPR0_tag SIPCAPR0;   /* Station interface port capability register 0 */
  NETC_F3_SI1_SIPCAPR1_tag SIPCAPR1;   /* Station interface port capability register 1 */
  uint8_t NETC_F3_SI1_reserved1[16];
  NETC_F3_SI1_SIRBGCR_tag SIRBGCR;     /* Station interface receive BDR group control register */
  uint8_t NETC_F3_SI1_reserved2[4];
  NETC_F3_SI1_SIBCAR_tag SIBCAR;       /* Station interface buffer cache attribute register */
  uint8_t NETC_F3_SI1_reserved3[4];
  NETC_F3_SI1_SICCAR_tag SICCAR;       /* Station interface command cache attribute register */
  uint8_t NETC_F3_SI1_reserved4[52];
  NETC_F3_SI1_SIPMAR0_tag SIPMAR0;     /* Station interface primary MAC address register 0 */
  NETC_F3_SI1_SIPMAR1_tag SIPMAR1;     /* Station interface primary MAC address register 1 */
  uint8_t NETC_F3_SI1_reserved5[8];
  NETC_F3_SI1_SICVLANR1_tag SICVLANR1; /* Station interface custom VLAN register 1 */
  NETC_F3_SI1_SICVLANR2_tag SICVLANR2; /* Station interface custom VLAN register 2 */
  uint8_t NETC_F3_SI1_reserved6[104];
  NETC_F3_SI1_SIVLANIPVMR0_tag SIVLANIPVMR0; /* Station interface VLAN to IPV mapping register 0 */
  NETC_F3_SI1_SIVLANIPVMR1_tag SIVLANIPVMR1; /* Station interface VLAN to IPV mapping register 1 */
  uint8_t NETC_F3_SI1_reserved7[72];
  NETC_F3_SI1_SIIPVBDRMR0_tag SIIPVBDRMR0; /* Station interface IPV to ring mapping register */
  uint8_t NETC_F3_SI1_reserved8[176];
  NETC_F3_SI1_VSI_A_tag VSI_A;
  uint8_t NETC_F3_SI1_reserved9[232];
  NETC_F3_SI1_SIROCT0_tag SIROCT0;     /* Station interface receive octets counter (ifInOctets) 0 */
  NETC_F3_SI1_SIROCT1_tag SIROCT1;     /* Station interface receive octets counter (ifInOctets) 1 */
  NETC_F3_SI1_SIRFRM0_tag SIRFRM0;     /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  NETC_F3_SI1_SIRFRM1_tag SIRFRM1;     /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  NETC_F3_SI1_SIRUCA0_tag SIRUCA0;     /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  NETC_F3_SI1_SIRUCA1_tag SIRUCA1;     /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  NETC_F3_SI1_SIRMCA0_tag SIRMCA0;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  NETC_F3_SI1_SIRMCA1_tag SIRMCA1;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  NETC_F3_SI1_SITOCT0_tag SITOCT0;     /* Station interface transmit octets counter (ifOutOctets) 0 */
  NETC_F3_SI1_SITOCT1_tag SITOCT1;     /* Station interface transmit octets counter (ifOutOctets) 1 */
  NETC_F3_SI1_SITFRM0_tag SITFRM0;     /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  NETC_F3_SI1_SITFRM1_tag SITFRM1;     /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  NETC_F3_SI1_SITUCA0_tag SITUCA0;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  NETC_F3_SI1_SITUCA1_tag SITUCA1;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  NETC_F3_SI1_SITMCA0_tag SITMCA0;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  NETC_F3_SI1_SITMCA1_tag SITMCA1;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  uint8_t NETC_F3_SI1_reserved10[176];
  NETC_F3_SI1_SIBLPR_tag SIBLPR[2];    /* Station interface boot loader parameter register 0 */
  uint8_t NETC_F3_SI1_reserved11[1032];
  NETC_F3_SI1_SICBDRMR_tag SICBDRMR;   /* Station interface command BDR mode register */
  NETC_F3_SI1_SICBDRSR_tag SICBDRSR;   /* Station interface command BDR status register */
  uint8_t NETC_F3_SI1_reserved12[8];
  NETC_F3_SI1_SICBDRBAR0_tag SICBDRBAR0; /* Station interface command BDR base address register 0 */
  NETC_F3_SI1_SICBDRBAR1_tag SICBDRBAR1; /* Station interface command BDR base address register 1 */
  NETC_F3_SI1_SICBDRPIR_tag SICBDRPIR; /* Station interface command BDR producer index register */
  NETC_F3_SI1_SICBDRCIR_tag SICBDRCIR; /* Station interface command BDR consumer index register */
  NETC_F3_SI1_SICBDRLENR_tag SICBDRLENR; /* Station interface command BDR length register */
  uint8_t NETC_F3_SI1_reserved13[124];
  NETC_F3_SI1_SICBDRIER_tag SICBDRIER; /* Station interface command BDR interrupt enable register */
  NETC_F3_SI1_SICBDRIDR_tag SICBDRIDR; /* Station interface command BDR interrupt detect register */
  uint8_t NETC_F3_SI1_reserved14[88];
  NETC_F3_SI1_SICAPR0_tag SICAPR0;     /* Station interface capability register 0 */
  NETC_F3_SI1_SICAPR1_tag SICAPR1;     /* Station interface capability register 1 */
  NETC_F3_SI1_SICAPR2_tag SICAPR2;     /* Station interface capability register 2 */
  uint8_t NETC_F3_SI1_reserved15[244];
  NETC_F3_SI1_VSI_tag VSI;
  uint8_t NETC_F3_SI1_reserved16[12];
  NETC_F3_SI1_SITXIDR0_tag SITXIDR0;   /* Station interface transmit interrupt detect register 0 */
  NETC_F3_SI1_SITXIDR1_tag SITXIDR1;   /* Station interface transmit interrupt detect register 1 */
  uint8_t NETC_F3_SI1_reserved17[8];
  NETC_F3_SI1_SIRXIDR0_tag SIRXIDR0;   /* Station interface receive interrupt detect register 0 */
  NETC_F3_SI1_SIRXIDR1_tag SIRXIDR1;   /* Station interface receive interrupt detect register 1 */
  NETC_F3_SI1_SIMSIVR_tag SIMSIVR;     /* Station interface MSI-X vector register */
  NETC_F3_SI1_SICMSIVR_tag SICMSIVR;   /* Station interface command MSI-X vector register */
  uint8_t NETC_F3_SI1_reserved18[8];
  NETC_F3_SI1_SITMRIER_tag SITMRIER;   /* Station interface timer interrupt enable register */
  NETC_F3_SI1_SITMRIDR_tag SITMRIDR;   /* Station interface timer interrupt detect register */
  uint8_t NETC_F3_SI1_reserved19[4];
  NETC_F3_SI1_SITMRMSIVR_tag SITMRMSIVR; /* Station interface timer MSI-X vector register */
  uint8_t NETC_F3_SI1_reserved20[176];
  NETC_F3_SI1_SIMSITRVR_tag SIMSITRVR[18]; /* Station interface MSI-X transmit ring 0 vector register */
  uint8_t NETC_F3_SI1_reserved21[56];
  NETC_F3_SI1_SIMSIRRVR_tag SIMSIRRVR[18]; /* Station interface MSI-X receive ring 0 vector register */
  uint8_t NETC_F3_SI1_reserved22[568];
  NETC_F3_SI1_SICMECR_tag SICMECR;     /* Station interface correctable memory error configuration register */
  NETC_F3_SI1_SICMESR_tag SICMESR;     /* Station interface correctable memory error status register */
  uint8_t NETC_F3_SI1_reserved23[4];
  NETC_F3_SI1_SICMECTR_tag SICMECTR;   /* Station interface correctable memory error count register */
  NETC_F3_SI1_SIUPECR_tag SIUPECR;     /* Station interface uncorrectable programming error configuration register */
  NETC_F3_SI1_SIUPESR_tag SIUPESR;     /* Station interface uncorrectable programming error status register */
  uint8_t NETC_F3_SI1_reserved24[4];
  NETC_F3_SI1_SIUPECTR_tag SIUPECTR;   /* Station interface uncorrectable programming error count register */
  NETC_F3_SI1_SIUNSBECR_tag SIUNSBECR; /* Station interface uncorrectable non-fatal system bus error configuration register */
  NETC_F3_SI1_SIUNSBESR_tag SIUNSBESR; /* Station interface uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F3_SI1_reserved25[4];
  NETC_F3_SI1_SIUNSBECTR_tag SIUNSBECTR; /* Station interface uncorrectable non-fatal system bus error count register */
  NETC_F3_SI1_SIUFSBECR_tag SIUFSBECR; /* Station interface uncorrectable fatal system bus error configuration register */
  NETC_F3_SI1_SIUFSBESR_tag SIUFSBESR; /* Station interface uncorrectable fatal system bus error status register */
  uint8_t NETC_F3_SI1_reserved26[8];
  NETC_F3_SI1_SIUNMECR_tag SIUNMECR;   /* Station interface uncorrectable non-fatal memory error configuration register */
  NETC_F3_SI1_SIUNMESR0_tag SIUNMESR0; /* Station interface uncorrectable non-fatal memory error status register 0 */
  NETC_F3_SI1_SIUNMESR1_tag SIUNMESR1; /* Station interface uncorrectable non-fatal memory error status register 1 */
  NETC_F3_SI1_SIUNMECTR_tag SIUNMECTR; /* Station interface uncorrectable non-fatal memory error count register */
  NETC_F3_SI1_SIUFMECR_tag SIUFMECR;   /* Station interface uncorrectable fatal memory error configuration register */
  NETC_F3_SI1_SIUFMESR0_tag SIUFMESR0; /* Station interface uncorrectable fatal memory error status register 0 */
  NETC_F3_SI1_SIUFMESR1_tag SIUFMESR1; /* Station interface uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_SI1_reserved27[4];
  NETC_F3_SI1_SIUNIECR_tag SIUNIECR;   /* Station interface uncorrectable non-fatal integrity error configuration register */
  NETC_F3_SI1_SIUNIESR_tag SIUNIESR;   /* Station interface uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_SI1_reserved28[4];
  NETC_F3_SI1_SIUNIECTR_tag SIUNIECTR; /* Station interface uncorrectable non-fatal integrity error count register */
  NETC_F3_SI1_SIUFIECR_tag SIUFIECR;   /* Station interface uncorrectable fatal integrity error configuration register */
  NETC_F3_SI1_SIUFIESR_tag SIUFIESR;   /* Station interface uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_SI1_reserved29[392];
  NETC_F3_SI1_SIMAFTCAPR_tag SIMAFTCAPR; /* Station interface MAC address filter table capability register */
  uint8_t NETC_F3_SI1_reserved30[252];
  NETC_F3_SI1_SIVFTCAPR_tag SIVFTCAPR; /* Station interface VLAN filter table capability register */
  uint8_t NETC_F3_SI1_reserved31[252];
  NETC_F3_SI1_SIRFSCAPR_tag SIRFSCAPR; /* Station interface RFS capability register */
  uint8_t NETC_F3_SI1_reserved32[28156];
  NETC_F3_SI1_BDR_NUM_tag BDR_NUM[18];
};


/* ============================================================================
   =============================== Module: NETC_F3_SI2 ========================
   ============================================================================ */

typedef union NETC_F3_SI2_SIMR_union_tag { /* Station interface mode register */
  vuint32_t R;
  struct {
    vuint32_t RSSE:1;
    vuint32_t RNUM:1;
    vuint32_t RNMM:1;
    vuint32_t RNBM:1;
    vuint32_t V2IPVE:1;
    vuint32_t _unused_5:11;
    vuint32_t DEFAULT_RX_GROUP:1;
    vuint32_t _unused_17:14;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI2_SIMR_tag;

typedef union NETC_F3_SI2_SISR_union_tag { /* Station interface status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;
    vuint32_t MAC_UP:1;
    vuint32_t MAC_MP:1;
    vuint32_t VLAN_P:1;
    vuint32_t VLAN_UTA:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_SI2_SISR_tag;

typedef union NETC_F3_SI2_SICTR0_union_tag { /* Station interface current time register 0 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI2_SICTR0_tag;

typedef union NETC_F3_SI2_SICTR1_union_tag { /* Station interface current time register 1 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI2_SICTR1_tag;

typedef union NETC_F3_SI2_SIPCAPR0_union_tag { /* Station interface port capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;                   /* Receive Flow Steering */
    vuint32_t FP:1;                    /* Frame Preemption */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t TSD:1;                   /* Time Specific Departure */
    vuint32_t CBS:1;                   /* Credit Based Shaping (CBS) */
    vuint32_t _unused_7:1;
    vuint32_t RSS:1;                   /* Receive Side Scaling */
    vuint32_t PSFP:1;                  /* Per-Stream Filtering and Policing (IEEE 802.1Qci) */
    vuint32_t IPFLT:1;                 /* Ingress Port Filtering */
    vuint32_t RP:1;                    /* Rate Policing */
    vuint32_t _unused_12:1;
    vuint32_t WO:1;                    /* Wake-on-LAN */
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional Safety */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI2_SIPCAPR0_tag;

typedef union NETC_F3_SI2_SIPCAPR1_union_tag { /* Station interface port capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:6;
    vuint32_t _unused_18:13;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_SI2_SIPCAPR1_tag;

typedef union NETC_F3_SI2_SIRBGCR_union_tag { /* Station interface receive BDR group control register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GROUPS:2;
    vuint32_t _unused_2:14;
    vuint32_t RINGS_PER_GROUP:3;
    vuint32_t _unused_19:13;
  } B;
} NETC_F3_SI2_SIRBGCR_tag;

typedef union NETC_F3_SI2_SIBCAR_union_tag { /* Station interface buffer cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI2_SIBCAR_tag;

typedef union NETC_F3_SI2_SICCAR_union_tag { /* Station interface command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI2_SICCAR_tag;

typedef union NETC_F3_SI2_SIPMAR0_union_tag { /* Station interface primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_SI2_SIPMAR0_tag;

typedef union NETC_F3_SI2_SIPMAR1_union_tag { /* Station interface primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI2_SIPMAR1_tag;

typedef union NETC_F3_SI2_SICVLANR1_union_tag { /* Station interface custom VLAN register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI2_SICVLANR1_tag;

typedef union NETC_F3_SI2_SICVLANR2_union_tag { /* Station interface custom VLAN register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI2_SICVLANR2_tag;

typedef union NETC_F3_SI2_SIVLANIPVMR0_union_tag { /* Station interface VLAN to IPV mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:4;
    vuint32_t PCP_DEI_1:4;
    vuint32_t PCP_DEI_2:4;
    vuint32_t PCP_DEI_3:4;
    vuint32_t PCP_DEI_4:4;
    vuint32_t PCP_DEI_5:4;
    vuint32_t PCP_DEI_6:4;
    vuint32_t PCP_DEI_7:4;
  } B;
} NETC_F3_SI2_SIVLANIPVMR0_tag;

typedef union NETC_F3_SI2_SIVLANIPVMR1_union_tag { /* Station interface VLAN to IPV mapping register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:4;
    vuint32_t PCP_DEI_9:4;
    vuint32_t PCP_DEI_10:4;
    vuint32_t PCP_DEI_11:4;
    vuint32_t PCP_DEI_12:4;
    vuint32_t PCP_DEI_13:4;
    vuint32_t PCP_DEI_14:4;
    vuint32_t PCP_DEI_15:4;
  } B;
} NETC_F3_SI2_SIVLANIPVMR1_tag;

typedef union NETC_F3_SI2_SIIPVBDRMR0_union_tag { /* Station interface IPV to ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPV0BDR:3;
    vuint32_t _unused_3:1;
    vuint32_t IPV1BDR:3;
    vuint32_t _unused_7:1;
    vuint32_t IPV2BDR:3;
    vuint32_t _unused_11:1;
    vuint32_t IPV3BDR:3;
    vuint32_t _unused_15:1;
    vuint32_t IPV4BDR:3;
    vuint32_t _unused_19:1;
    vuint32_t IPV5BDR:3;
    vuint32_t _unused_23:1;
    vuint32_t IPV6BDR:3;
    vuint32_t _unused_27:1;
    vuint32_t IPV7BDR:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI2_SIIPVBDRMR0_tag;

typedef union NETC_F3_SI2_VSI_A_VSIMSGSR_union_tag { /* Virtual station interface message send register */
  vuint32_t R;
  struct {
    vuint32_t MB:1;
    vuint32_t MS:1;
    vuint32_t _unused_2:14;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI2_VSI_A_VSIMSGSR_tag;

typedef union NETC_F3_SI2_VSI_A_VSIMSGRR_union_tag { /* Virtual station interface message receive register */
  vuint32_t R;
  struct {
    vuint32_t MR:1;
    vuint32_t _unused_1:15;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI2_VSI_A_VSIMSGRR_tag;

typedef union NETC_F3_SI2_VSI_A_VSIMSGSNDAR0_union_tag { /* Virtual station interface message send register 0 */
  vuint32_t R;
  struct {
    vuint32_t MSIZE:5;
    vuint32_t _unused_5:1;
    vuint32_t ADDRL:26;
  } B;
} NETC_F3_SI2_VSI_A_VSIMSGSNDAR0_tag;

typedef union NETC_F3_SI2_VSI_A_VSIMSGSNDAR1_union_tag { /* Virtual station interface message send address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI2_VSI_A_VSIMSGSNDAR1_tag;

typedef struct NETC_F3_SI2_VSI_A_struct_tag {
  NETC_F3_SI2_VSI_A_VSIMSGSR_tag VSIMSGSR; /* Virtual station interface message send register */
  NETC_F3_SI2_VSI_A_VSIMSGRR_tag VSIMSGRR; /* Virtual station interface message receive register */
  uint8_t VSI_A_reserved0[4];
  NETC_F3_SI2_VSI_A_VSIMSGSNDAR0_tag VSIMSGSNDAR0; /* Virtual station interface message send register 0 */
  NETC_F3_SI2_VSI_A_VSIMSGSNDAR1_tag VSIMSGSNDAR1; /* Virtual station interface message send address register 1 */
} NETC_F3_SI2_VSI_A_tag;

typedef union NETC_F3_SI2_SIROCT0_union_tag { /* Station interface receive octets counter (ifInOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_LOW:32;
  } B;
} NETC_F3_SI2_SIROCT0_tag;

typedef union NETC_F3_SI2_SIROCT1_union_tag { /* Station interface receive octets counter (ifInOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_HIGH:32;
  } B;
} NETC_F3_SI2_SIROCT1_tag;

typedef union NETC_F3_SI2_SIRFRM0_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_LOW:32;
  } B;
} NETC_F3_SI2_SIRFRM0_tag;

typedef union NETC_F3_SI2_SIRFRM1_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_HIGH:32;
  } B;
} NETC_F3_SI2_SIRFRM1_tag;

typedef union NETC_F3_SI2_SIRUCA0_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_LOW:32;
  } B;
} NETC_F3_SI2_SIRUCA0_tag;

typedef union NETC_F3_SI2_SIRUCA1_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_HIGH:32;
  } B;
} NETC_F3_SI2_SIRUCA1_tag;

typedef union NETC_F3_SI2_SIRMCA0_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_LOW:32;
  } B;
} NETC_F3_SI2_SIRMCA0_tag;

typedef union NETC_F3_SI2_SIRMCA1_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_HIGH:32;
  } B;
} NETC_F3_SI2_SIRMCA1_tag;

typedef union NETC_F3_SI2_SITOCT0_union_tag { /* Station interface transmit octets counter (ifOutOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_LOW:32;
  } B;
} NETC_F3_SI2_SITOCT0_tag;

typedef union NETC_F3_SI2_SITOCT1_union_tag { /* Station interface transmit octets counter (ifOutOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_HIGH:32;
  } B;
} NETC_F3_SI2_SITOCT1_tag;

typedef union NETC_F3_SI2_SITFRM0_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_LOW:32;
  } B;
} NETC_F3_SI2_SITFRM0_tag;

typedef union NETC_F3_SI2_SITFRM1_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_HIGH:32;
  } B;
} NETC_F3_SI2_SITFRM1_tag;

typedef union NETC_F3_SI2_SITUCA0_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_LOW:32;
  } B;
} NETC_F3_SI2_SITUCA0_tag;

typedef union NETC_F3_SI2_SITUCA1_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_HIGH:32;
  } B;
} NETC_F3_SI2_SITUCA1_tag;

typedef union NETC_F3_SI2_SITMCA0_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_LOW:32;
  } B;
} NETC_F3_SI2_SITMCA0_tag;

typedef union NETC_F3_SI2_SITMCA1_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_HIGH:32;
  } B;
} NETC_F3_SI2_SITMCA1_tag;

typedef union NETC_F3_SI2_SIBLPR_union_tag { /* Station interface boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F3_SI2_SIBLPR_tag;

typedef union NETC_F3_SI2_SICBDRMR_union_tag { /* Station interface command BDR mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI2_SICBDRMR_tag;

typedef union NETC_F3_SI2_SICBDRSR_union_tag { /* Station interface command BDR status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI2_SICBDRSR_tag;

typedef union NETC_F3_SI2_SICBDRBAR0_union_tag { /* Station interface command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI2_SICBDRBAR0_tag;

typedef union NETC_F3_SI2_SICBDRBAR1_union_tag { /* Station interface command BDR base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI2_SICBDRBAR1_tag;

typedef union NETC_F3_SI2_SICBDRPIR_union_tag { /* Station interface command BDR producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI2_SICBDRPIR_tag;

typedef union NETC_F3_SI2_SICBDRCIR_union_tag { /* Station interface command BDR consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI2_SICBDRCIR_tag;

typedef union NETC_F3_SI2_SICBDRLENR_union_tag { /* Station interface command BDR length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_SI2_SICBDRLENR_tag;

typedef union NETC_F3_SI2_SICBDRIER_union_tag { /* Station interface command BDR interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI2_SICBDRIER_tag;

typedef union NETC_F3_SI2_SICBDRIDR_union_tag { /* Station interface command BDR interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI2_SICBDRIDR_tag;

typedef union NETC_F3_SI2_SICAPR0_union_tag { /* Station interface capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_RX_BDR:8;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F3_SI2_SICAPR0_tag;

typedef union NETC_F3_SI2_SICAPR1_union_tag { /* Station interface capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t NUM_RX_GRP:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI2_SICAPR1_tag;

typedef union NETC_F3_SI2_SICAPR2_union_tag { /* Station interface capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t VTP:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_SI2_SICAPR2_tag;

typedef union NETC_F3_SI2_VSI_VSIIER_union_tag { /* Virtual station interface interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MSIE:1;
    vuint32_t MRIE:1;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI2_VSI_VSIIER_tag;

typedef union NETC_F3_SI2_VSI_VSIIDR_union_tag { /* Virtual station interface interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXR:1;
    vuint32_t _unused_1:7;
    vuint32_t MS:1;
    vuint32_t MR:1;
    vuint32_t _unused_10:6;
    vuint32_t RXR:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI2_VSI_VSIIDR_tag;

typedef struct NETC_F3_SI2_VSI_struct_tag {
  NETC_F3_SI2_VSI_VSIIER_tag VSIIER;   /* Virtual station interface interrupt enable register */
  uint8_t VSI_reserved0[4];
  NETC_F3_SI2_VSI_VSIIDR_tag VSIIDR;   /* Virtual station interface interrupt detect register */
} NETC_F3_SI2_VSI_tag;

typedef union NETC_F3_SI2_SITXIDR0_union_tag { /* Station interface transmit interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXT0:1;
    vuint32_t TXT1:1;
    vuint32_t TXT2:1;
    vuint32_t TXT3:1;
    vuint32_t TXT4:1;
    vuint32_t TXT5:1;
    vuint32_t TXT6:1;
    vuint32_t TXT7:1;
    vuint32_t TXT8:1;
    vuint32_t TXT9:1;
    vuint32_t TXT10:1;
    vuint32_t TXT11:1;
    vuint32_t TXT12:1;
    vuint32_t TXT13:1;
    vuint32_t TXT14:1;
    vuint32_t TXT15:1;
    vuint32_t TXF0:1;
    vuint32_t TXF1:1;
    vuint32_t TXF2:1;
    vuint32_t TXF3:1;
    vuint32_t TXF4:1;
    vuint32_t TXF5:1;
    vuint32_t TXF6:1;
    vuint32_t TXF7:1;
    vuint32_t TXF8:1;
    vuint32_t TXF9:1;
    vuint32_t TXF10:1;
    vuint32_t TXF11:1;
    vuint32_t TXF12:1;
    vuint32_t TXF13:1;
    vuint32_t TXF14:1;
    vuint32_t TXF15:1;
  } B;
} NETC_F3_SI2_SITXIDR0_tag;

typedef union NETC_F3_SI2_SITXIDR1_union_tag { /* Station interface transmit interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t TXT16:1;
    vuint32_t TXT17:1;
    vuint32_t _unused_2:14;
    vuint32_t TXF16:1;
    vuint32_t TXF17:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_SI2_SITXIDR1_tag;

typedef union NETC_F3_SI2_SIRXIDR0_union_tag { /* Station interface receive interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t RX0:1;
    vuint32_t RX1:1;
    vuint32_t RX2:1;
    vuint32_t RX3:1;
    vuint32_t RX4:1;
    vuint32_t RX5:1;
    vuint32_t RX6:1;
    vuint32_t RX7:1;
    vuint32_t RX8:1;
    vuint32_t RX9:1;
    vuint32_t RX10:1;
    vuint32_t RX11:1;
    vuint32_t RX12:1;
    vuint32_t RX13:1;
    vuint32_t RX14:1;
    vuint32_t RX15:1;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI2_SIRXIDR0_tag;

typedef union NETC_F3_SI2_SIRXIDR1_union_tag { /* Station interface receive interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t RX16:1;
    vuint32_t RX17:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI2_SIRXIDR1_tag;

typedef union NETC_F3_SI2_SIMSIVR_union_tag { /* Station interface MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI2_SIMSIVR_tag;

typedef union NETC_F3_SI2_SICMSIVR_union_tag { /* Station interface command MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI2_SICMSIVR_tag;

typedef union NETC_F3_SI2_SITMRIER_union_tag { /* Station interface timer interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t SYNCE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI2_SITMRIER_tag;

typedef union NETC_F3_SI2_SITMRIDR_union_tag { /* Station interface timer interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI2_SITMRIDR_tag;

typedef union NETC_F3_SI2_SITMRMSIVR_union_tag { /* Station interface timer MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI2_SITMRMSIVR_tag;

typedef union NETC_F3_SI2_SIMSITRVR_union_tag { /* Station interface MSI-X transmit ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI2_SIMSITRVR_tag;

typedef union NETC_F3_SI2_SIMSIRRVR_union_tag { /* Station interface MSI-X receive ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI2_SIMSIRRVR_tag;

typedef union NETC_F3_SI2_SICMECR_union_tag { /* Station interface correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI2_SICMECR_tag;

typedef union NETC_F3_SI2_SICMESR_union_tag { /* Station interface correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_SI2_SICMESR_tag;

typedef union NETC_F3_SI2_SICMECTR_union_tag { /* Station interface correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI2_SICMECTR_tag;

typedef union NETC_F3_SI2_SIUPECR_union_tag { /* Station interface uncorrectable programming error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI2_SIUPECR_tag;

typedef union NETC_F3_SI2_SIUPESR_union_tag { /* Station interface uncorrectable programming error status register */
  vuint32_t R;
  struct {
    vuint32_t DROP_SI_EN:1;
    vuint32_t DROP_RING_EN:1;
    vuint32_t DROP_GRP_SEL:1;
    vuint32_t DROP_RING_SEL:1;
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t PE:1;                    /* Programming error */
  } B;
} NETC_F3_SI2_SIUPESR_tag;

typedef union NETC_F3_SI2_SIUPECTR_union_tag { /* Station interface uncorrectable programming error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;                /* Count */
  } B;
} NETC_F3_SI2_SIUPECTR_tag;

typedef union NETC_F3_SI2_SIUNSBECR_union_tag { /* Station interface uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI2_SIUNSBECR_tag;

typedef union NETC_F3_SI2_SIUNSBESR_union_tag { /* Station interface uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI2_SIUNSBESR_tag;

typedef union NETC_F3_SI2_SIUNSBECTR_union_tag { /* Station interface uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI2_SIUNSBECTR_tag;

typedef union NETC_F3_SI2_SIUFSBECR_union_tag { /* Station interface uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI2_SIUFSBECR_tag;

typedef union NETC_F3_SI2_SIUFSBESR_union_tag { /* Station interface uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI2_SIUFSBESR_tag;

typedef union NETC_F3_SI2_SIUNMECR_union_tag { /* Station interface uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI2_SIUNMECR_tag;

typedef union NETC_F3_SI2_SIUNMESR0_union_tag { /* Station interface uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI2_SIUNMESR0_tag;

typedef union NETC_F3_SI2_SIUNMESR1_union_tag { /* Station interface uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI2_SIUNMESR1_tag;

typedef union NETC_F3_SI2_SIUNMECTR_union_tag { /* Station interface uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI2_SIUNMECTR_tag;

typedef union NETC_F3_SI2_SIUFMECR_union_tag { /* Station interface uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI2_SIUFMECR_tag;

typedef union NETC_F3_SI2_SIUFMESR0_union_tag { /* Station interface uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI2_SIUFMESR0_tag;

typedef union NETC_F3_SI2_SIUFMESR1_union_tag { /* Station interface uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI2_SIUFMESR1_tag;

typedef union NETC_F3_SI2_SIUNIECR_union_tag { /* Station interface uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI2_SIUNIECR_tag;

typedef union NETC_F3_SI2_SIUNIESR_union_tag { /* Station interface uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI2_SIUNIESR_tag;

typedef union NETC_F3_SI2_SIUNIECTR_union_tag { /* Station interface uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI2_SIUNIECTR_tag;

typedef union NETC_F3_SI2_SIUFIECR_union_tag { /* Station interface uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI2_SIUFIECR_tag;

typedef union NETC_F3_SI2_SIUFIESR_union_tag { /* Station interface uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:16;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI2_SIUFIESR_tag;

typedef union NETC_F3_SI2_SIMAFTCAPR_union_tag { /* Station interface MAC address filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI2_SIMAFTCAPR_tag;

typedef union NETC_F3_SI2_SIVFTCAPR_union_tag { /* Station interface VLAN filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI2_SIVFTCAPR_tag;

typedef union NETC_F3_SI2_SIRFSCAPR_union_tag { /* Station interface RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_SI2_SIRFSCAPR_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBMR_union_tag { /* Tx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t PRIO:3;
    vuint32_t _unused_3:1;
    vuint32_t WRR:3;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t VIH:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI2_BDR_NUM_TBMR_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBSR_union_tag { /* Tx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI2_BDR_NUM_TBSR_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBBAR0_union_tag { /* Tx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI2_BDR_NUM_TBBAR0_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBBAR1_union_tag { /* Tx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI2_BDR_NUM_TBBAR1_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBPIR_union_tag { /* Tx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI2_BDR_NUM_TBPIR_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBCIR_union_tag { /* Tx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t STAT_ID:16;
  } B;
} NETC_F3_SI2_BDR_NUM_TBCIR_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBLENR_union_tag { /* Tx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI2_BDR_NUM_TBLENR_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBIER_union_tag { /* Tx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t TXTIE:1;
    vuint32_t TXFIE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI2_BDR_NUM_TBIER_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBIDR_union_tag { /* Tx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXT:1;
    vuint32_t TXF:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI2_BDR_NUM_TBIDR_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBICR0_union_tag { /* Tx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:4;
    vuint32_t _unused_4:27;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI2_BDR_NUM_TBICR0_tag;

typedef union NETC_F3_SI2_BDR_NUM_TBICR1_union_tag { /* Tx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI2_BDR_NUM_TBICR1_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBMR_union_tag { /* Rx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t AL:1;
    vuint32_t _unused_1:1;
    vuint32_t BDS:1;
    vuint32_t _unused_3:1;
    vuint32_t CM:1;
    vuint32_t VTE:1;
    vuint32_t VTPD:1;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI2_BDR_NUM_RBMR_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBSR_union_tag { /* Rx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI2_BDR_NUM_RBSR_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBBSR_union_tag { /* Rx BDR 0 buffer size register */
  vuint32_t R;
  struct {
    vuint32_t BSIZE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI2_BDR_NUM_RBBSR_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBCIR_union_tag { /* Rx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI2_BDR_NUM_RBCIR_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBBAR0_union_tag { /* Rx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI2_BDR_NUM_RBBAR0_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBBAR1_union_tag { /* Rx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI2_BDR_NUM_RBBAR1_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBPIR_union_tag { /* Rx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI2_BDR_NUM_RBPIR_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBLENR_union_tag { /* Rx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI2_BDR_NUM_RBLENR_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBDCR_union_tag { /* Rx BDR 0 drop count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_SI2_BDR_NUM_RBDCR_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBIER_union_tag { /* Rx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t RXTIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI2_BDR_NUM_RBIER_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBIDR_union_tag { /* Rx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t RXT:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI2_BDR_NUM_RBIDR_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBICR0_union_tag { /* Rx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:9;
    vuint32_t _unused_9:22;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI2_BDR_NUM_RBICR0_tag;

typedef union NETC_F3_SI2_BDR_NUM_RBICR1_union_tag { /* Rx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI2_BDR_NUM_RBICR1_tag;

typedef struct NETC_F3_SI2_BDR_NUM_struct_tag {
  NETC_F3_SI2_BDR_NUM_TBMR_tag TBMR;   /* Tx BDR 0 mode register */
  NETC_F3_SI2_BDR_NUM_TBSR_tag TBSR;   /* Tx BDR 0 status register */
  uint8_t BDR_NUM_reserved0[8];
  NETC_F3_SI2_BDR_NUM_TBBAR0_tag TBBAR0; /* Tx BDR 0 base address register 0 */
  NETC_F3_SI2_BDR_NUM_TBBAR1_tag TBBAR1; /* Tx BDR 0 base address register 1 */
  NETC_F3_SI2_BDR_NUM_TBPIR_tag TBPIR; /* Tx BDR 0 producer index register */
  NETC_F3_SI2_BDR_NUM_TBCIR_tag TBCIR; /* Tx BDR 0 consumer index register */
  NETC_F3_SI2_BDR_NUM_TBLENR_tag TBLENR; /* Tx BDR 0 length register */
  uint8_t BDR_NUM_reserved1[124];
  NETC_F3_SI2_BDR_NUM_TBIER_tag TBIER; /* Tx BDR 0 interrupt enable register */
  NETC_F3_SI2_BDR_NUM_TBIDR_tag TBIDR; /* Tx BDR 0 interrupt detect register */
  NETC_F3_SI2_BDR_NUM_TBICR0_tag TBICR0; /* Tx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI2_BDR_NUM_TBICR1_tag TBICR1; /* Tx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved2[80];
  NETC_F3_SI2_BDR_NUM_RBMR_tag RBMR;   /* Rx BDR 0 mode register */
  NETC_F3_SI2_BDR_NUM_RBSR_tag RBSR;   /* Rx BDR 0 status register */
  NETC_F3_SI2_BDR_NUM_RBBSR_tag RBBSR; /* Rx BDR 0 buffer size register */
  NETC_F3_SI2_BDR_NUM_RBCIR_tag RBCIR; /* Rx BDR 0 consumer index register */
  NETC_F3_SI2_BDR_NUM_RBBAR0_tag RBBAR0; /* Rx BDR 0 base address register 0 */
  NETC_F3_SI2_BDR_NUM_RBBAR1_tag RBBAR1; /* Rx BDR 0 base address register 1 */
  NETC_F3_SI2_BDR_NUM_RBPIR_tag RBPIR; /* Rx BDR 0 producer index register */
  uint8_t BDR_NUM_reserved3[4];
  NETC_F3_SI2_BDR_NUM_RBLENR_tag RBLENR; /* Rx BDR 0 length register */
  uint8_t BDR_NUM_reserved4[92];
  NETC_F3_SI2_BDR_NUM_RBDCR_tag RBDCR; /* Rx BDR 0 drop count register */
  uint8_t BDR_NUM_reserved5[28];
  NETC_F3_SI2_BDR_NUM_RBIER_tag RBIER; /* Rx BDR 0 interrupt enable register */
  NETC_F3_SI2_BDR_NUM_RBIDR_tag RBIDR; /* Rx BDR 0 interrupt detect register */
  NETC_F3_SI2_BDR_NUM_RBICR0_tag RBICR0; /* Rx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI2_BDR_NUM_RBICR1_tag RBICR1; /* Rx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved6[80];
} NETC_F3_SI2_BDR_NUM_tag;

struct NETC_F3_SI2_tag {
  NETC_F3_SI2_SIMR_tag SIMR;           /* Station interface mode register */
  NETC_F3_SI2_SISR_tag SISR;           /* Station interface status register */
  uint8_t NETC_F3_SI2_reserved0[16];
  NETC_F3_SI2_SICTR0_tag SICTR0;       /* Station interface current time register 0 */
  NETC_F3_SI2_SICTR1_tag SICTR1;       /* Station interface current time register 1 */
  NETC_F3_SI2_SIPCAPR0_tag SIPCAPR0;   /* Station interface port capability register 0 */
  NETC_F3_SI2_SIPCAPR1_tag SIPCAPR1;   /* Station interface port capability register 1 */
  uint8_t NETC_F3_SI2_reserved1[16];
  NETC_F3_SI2_SIRBGCR_tag SIRBGCR;     /* Station interface receive BDR group control register */
  uint8_t NETC_F3_SI2_reserved2[4];
  NETC_F3_SI2_SIBCAR_tag SIBCAR;       /* Station interface buffer cache attribute register */
  uint8_t NETC_F3_SI2_reserved3[4];
  NETC_F3_SI2_SICCAR_tag SICCAR;       /* Station interface command cache attribute register */
  uint8_t NETC_F3_SI2_reserved4[52];
  NETC_F3_SI2_SIPMAR0_tag SIPMAR0;     /* Station interface primary MAC address register 0 */
  NETC_F3_SI2_SIPMAR1_tag SIPMAR1;     /* Station interface primary MAC address register 1 */
  uint8_t NETC_F3_SI2_reserved5[8];
  NETC_F3_SI2_SICVLANR1_tag SICVLANR1; /* Station interface custom VLAN register 1 */
  NETC_F3_SI2_SICVLANR2_tag SICVLANR2; /* Station interface custom VLAN register 2 */
  uint8_t NETC_F3_SI2_reserved6[104];
  NETC_F3_SI2_SIVLANIPVMR0_tag SIVLANIPVMR0; /* Station interface VLAN to IPV mapping register 0 */
  NETC_F3_SI2_SIVLANIPVMR1_tag SIVLANIPVMR1; /* Station interface VLAN to IPV mapping register 1 */
  uint8_t NETC_F3_SI2_reserved7[72];
  NETC_F3_SI2_SIIPVBDRMR0_tag SIIPVBDRMR0; /* Station interface IPV to ring mapping register */
  uint8_t NETC_F3_SI2_reserved8[176];
  NETC_F3_SI2_VSI_A_tag VSI_A;
  uint8_t NETC_F3_SI2_reserved9[232];
  NETC_F3_SI2_SIROCT0_tag SIROCT0;     /* Station interface receive octets counter (ifInOctets) 0 */
  NETC_F3_SI2_SIROCT1_tag SIROCT1;     /* Station interface receive octets counter (ifInOctets) 1 */
  NETC_F3_SI2_SIRFRM0_tag SIRFRM0;     /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  NETC_F3_SI2_SIRFRM1_tag SIRFRM1;     /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  NETC_F3_SI2_SIRUCA0_tag SIRUCA0;     /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  NETC_F3_SI2_SIRUCA1_tag SIRUCA1;     /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  NETC_F3_SI2_SIRMCA0_tag SIRMCA0;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  NETC_F3_SI2_SIRMCA1_tag SIRMCA1;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  NETC_F3_SI2_SITOCT0_tag SITOCT0;     /* Station interface transmit octets counter (ifOutOctets) 0 */
  NETC_F3_SI2_SITOCT1_tag SITOCT1;     /* Station interface transmit octets counter (ifOutOctets) 1 */
  NETC_F3_SI2_SITFRM0_tag SITFRM0;     /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  NETC_F3_SI2_SITFRM1_tag SITFRM1;     /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  NETC_F3_SI2_SITUCA0_tag SITUCA0;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  NETC_F3_SI2_SITUCA1_tag SITUCA1;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  NETC_F3_SI2_SITMCA0_tag SITMCA0;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  NETC_F3_SI2_SITMCA1_tag SITMCA1;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  uint8_t NETC_F3_SI2_reserved10[176];
  NETC_F3_SI2_SIBLPR_tag SIBLPR[2];    /* Station interface boot loader parameter register 0 */
  uint8_t NETC_F3_SI2_reserved11[1032];
  NETC_F3_SI2_SICBDRMR_tag SICBDRMR;   /* Station interface command BDR mode register */
  NETC_F3_SI2_SICBDRSR_tag SICBDRSR;   /* Station interface command BDR status register */
  uint8_t NETC_F3_SI2_reserved12[8];
  NETC_F3_SI2_SICBDRBAR0_tag SICBDRBAR0; /* Station interface command BDR base address register 0 */
  NETC_F3_SI2_SICBDRBAR1_tag SICBDRBAR1; /* Station interface command BDR base address register 1 */
  NETC_F3_SI2_SICBDRPIR_tag SICBDRPIR; /* Station interface command BDR producer index register */
  NETC_F3_SI2_SICBDRCIR_tag SICBDRCIR; /* Station interface command BDR consumer index register */
  NETC_F3_SI2_SICBDRLENR_tag SICBDRLENR; /* Station interface command BDR length register */
  uint8_t NETC_F3_SI2_reserved13[124];
  NETC_F3_SI2_SICBDRIER_tag SICBDRIER; /* Station interface command BDR interrupt enable register */
  NETC_F3_SI2_SICBDRIDR_tag SICBDRIDR; /* Station interface command BDR interrupt detect register */
  uint8_t NETC_F3_SI2_reserved14[88];
  NETC_F3_SI2_SICAPR0_tag SICAPR0;     /* Station interface capability register 0 */
  NETC_F3_SI2_SICAPR1_tag SICAPR1;     /* Station interface capability register 1 */
  NETC_F3_SI2_SICAPR2_tag SICAPR2;     /* Station interface capability register 2 */
  uint8_t NETC_F3_SI2_reserved15[244];
  NETC_F3_SI2_VSI_tag VSI;
  uint8_t NETC_F3_SI2_reserved16[12];
  NETC_F3_SI2_SITXIDR0_tag SITXIDR0;   /* Station interface transmit interrupt detect register 0 */
  NETC_F3_SI2_SITXIDR1_tag SITXIDR1;   /* Station interface transmit interrupt detect register 1 */
  uint8_t NETC_F3_SI2_reserved17[8];
  NETC_F3_SI2_SIRXIDR0_tag SIRXIDR0;   /* Station interface receive interrupt detect register 0 */
  NETC_F3_SI2_SIRXIDR1_tag SIRXIDR1;   /* Station interface receive interrupt detect register 1 */
  NETC_F3_SI2_SIMSIVR_tag SIMSIVR;     /* Station interface MSI-X vector register */
  NETC_F3_SI2_SICMSIVR_tag SICMSIVR;   /* Station interface command MSI-X vector register */
  uint8_t NETC_F3_SI2_reserved18[8];
  NETC_F3_SI2_SITMRIER_tag SITMRIER;   /* Station interface timer interrupt enable register */
  NETC_F3_SI2_SITMRIDR_tag SITMRIDR;   /* Station interface timer interrupt detect register */
  uint8_t NETC_F3_SI2_reserved19[4];
  NETC_F3_SI2_SITMRMSIVR_tag SITMRMSIVR; /* Station interface timer MSI-X vector register */
  uint8_t NETC_F3_SI2_reserved20[176];
  NETC_F3_SI2_SIMSITRVR_tag SIMSITRVR[18]; /* Station interface MSI-X transmit ring 0 vector register */
  uint8_t NETC_F3_SI2_reserved21[56];
  NETC_F3_SI2_SIMSIRRVR_tag SIMSIRRVR[18]; /* Station interface MSI-X receive ring 0 vector register */
  uint8_t NETC_F3_SI2_reserved22[568];
  NETC_F3_SI2_SICMECR_tag SICMECR;     /* Station interface correctable memory error configuration register */
  NETC_F3_SI2_SICMESR_tag SICMESR;     /* Station interface correctable memory error status register */
  uint8_t NETC_F3_SI2_reserved23[4];
  NETC_F3_SI2_SICMECTR_tag SICMECTR;   /* Station interface correctable memory error count register */
  NETC_F3_SI2_SIUPECR_tag SIUPECR;     /* Station interface uncorrectable programming error configuration register */
  NETC_F3_SI2_SIUPESR_tag SIUPESR;     /* Station interface uncorrectable programming error status register */
  uint8_t NETC_F3_SI2_reserved24[4];
  NETC_F3_SI2_SIUPECTR_tag SIUPECTR;   /* Station interface uncorrectable programming error count register */
  NETC_F3_SI2_SIUNSBECR_tag SIUNSBECR; /* Station interface uncorrectable non-fatal system bus error configuration register */
  NETC_F3_SI2_SIUNSBESR_tag SIUNSBESR; /* Station interface uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F3_SI2_reserved25[4];
  NETC_F3_SI2_SIUNSBECTR_tag SIUNSBECTR; /* Station interface uncorrectable non-fatal system bus error count register */
  NETC_F3_SI2_SIUFSBECR_tag SIUFSBECR; /* Station interface uncorrectable fatal system bus error configuration register */
  NETC_F3_SI2_SIUFSBESR_tag SIUFSBESR; /* Station interface uncorrectable fatal system bus error status register */
  uint8_t NETC_F3_SI2_reserved26[8];
  NETC_F3_SI2_SIUNMECR_tag SIUNMECR;   /* Station interface uncorrectable non-fatal memory error configuration register */
  NETC_F3_SI2_SIUNMESR0_tag SIUNMESR0; /* Station interface uncorrectable non-fatal memory error status register 0 */
  NETC_F3_SI2_SIUNMESR1_tag SIUNMESR1; /* Station interface uncorrectable non-fatal memory error status register 1 */
  NETC_F3_SI2_SIUNMECTR_tag SIUNMECTR; /* Station interface uncorrectable non-fatal memory error count register */
  NETC_F3_SI2_SIUFMECR_tag SIUFMECR;   /* Station interface uncorrectable fatal memory error configuration register */
  NETC_F3_SI2_SIUFMESR0_tag SIUFMESR0; /* Station interface uncorrectable fatal memory error status register 0 */
  NETC_F3_SI2_SIUFMESR1_tag SIUFMESR1; /* Station interface uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_SI2_reserved27[4];
  NETC_F3_SI2_SIUNIECR_tag SIUNIECR;   /* Station interface uncorrectable non-fatal integrity error configuration register */
  NETC_F3_SI2_SIUNIESR_tag SIUNIESR;   /* Station interface uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_SI2_reserved28[4];
  NETC_F3_SI2_SIUNIECTR_tag SIUNIECTR; /* Station interface uncorrectable non-fatal integrity error count register */
  NETC_F3_SI2_SIUFIECR_tag SIUFIECR;   /* Station interface uncorrectable fatal integrity error configuration register */
  NETC_F3_SI2_SIUFIESR_tag SIUFIESR;   /* Station interface uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_SI2_reserved29[392];
  NETC_F3_SI2_SIMAFTCAPR_tag SIMAFTCAPR; /* Station interface MAC address filter table capability register */
  uint8_t NETC_F3_SI2_reserved30[252];
  NETC_F3_SI2_SIVFTCAPR_tag SIVFTCAPR; /* Station interface VLAN filter table capability register */
  uint8_t NETC_F3_SI2_reserved31[252];
  NETC_F3_SI2_SIRFSCAPR_tag SIRFSCAPR; /* Station interface RFS capability register */
  uint8_t NETC_F3_SI2_reserved32[28156];
  NETC_F3_SI2_BDR_NUM_tag BDR_NUM[18];
};


/* ============================================================================
   =============================== Module: NETC_F3_SI3 ========================
   ============================================================================ */

typedef union NETC_F3_SI3_SIMR_union_tag { /* Station interface mode register */
  vuint32_t R;
  struct {
    vuint32_t RSSE:1;
    vuint32_t RNUM:1;
    vuint32_t RNMM:1;
    vuint32_t RNBM:1;
    vuint32_t V2IPVE:1;
    vuint32_t _unused_5:11;
    vuint32_t DEFAULT_RX_GROUP:1;
    vuint32_t _unused_17:14;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI3_SIMR_tag;

typedef union NETC_F3_SI3_SISR_union_tag { /* Station interface status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;
    vuint32_t MAC_UP:1;
    vuint32_t MAC_MP:1;
    vuint32_t VLAN_P:1;
    vuint32_t VLAN_UTA:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_SI3_SISR_tag;

typedef union NETC_F3_SI3_SICTR0_union_tag { /* Station interface current time register 0 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI3_SICTR0_tag;

typedef union NETC_F3_SI3_SICTR1_union_tag { /* Station interface current time register 1 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI3_SICTR1_tag;

typedef union NETC_F3_SI3_SIPCAPR0_union_tag { /* Station interface port capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;                   /* Receive Flow Steering */
    vuint32_t FP:1;                    /* Frame Preemption */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t TSD:1;                   /* Time Specific Departure */
    vuint32_t CBS:1;                   /* Credit Based Shaping (CBS) */
    vuint32_t _unused_7:1;
    vuint32_t RSS:1;                   /* Receive Side Scaling */
    vuint32_t PSFP:1;                  /* Per-Stream Filtering and Policing (IEEE 802.1Qci) */
    vuint32_t IPFLT:1;                 /* Ingress Port Filtering */
    vuint32_t RP:1;                    /* Rate Policing */
    vuint32_t _unused_12:1;
    vuint32_t WO:1;                    /* Wake-on-LAN */
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional Safety */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI3_SIPCAPR0_tag;

typedef union NETC_F3_SI3_SIPCAPR1_union_tag { /* Station interface port capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:6;
    vuint32_t _unused_18:13;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_SI3_SIPCAPR1_tag;

typedef union NETC_F3_SI3_SIRBGCR_union_tag { /* Station interface receive BDR group control register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GROUPS:2;
    vuint32_t _unused_2:14;
    vuint32_t RINGS_PER_GROUP:3;
    vuint32_t _unused_19:13;
  } B;
} NETC_F3_SI3_SIRBGCR_tag;

typedef union NETC_F3_SI3_SIBCAR_union_tag { /* Station interface buffer cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI3_SIBCAR_tag;

typedef union NETC_F3_SI3_SICCAR_union_tag { /* Station interface command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI3_SICCAR_tag;

typedef union NETC_F3_SI3_SIPMAR0_union_tag { /* Station interface primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_SI3_SIPMAR0_tag;

typedef union NETC_F3_SI3_SIPMAR1_union_tag { /* Station interface primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI3_SIPMAR1_tag;

typedef union NETC_F3_SI3_SICVLANR1_union_tag { /* Station interface custom VLAN register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI3_SICVLANR1_tag;

typedef union NETC_F3_SI3_SICVLANR2_union_tag { /* Station interface custom VLAN register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI3_SICVLANR2_tag;

typedef union NETC_F3_SI3_SIVLANIPVMR0_union_tag { /* Station interface VLAN to IPV mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:4;
    vuint32_t PCP_DEI_1:4;
    vuint32_t PCP_DEI_2:4;
    vuint32_t PCP_DEI_3:4;
    vuint32_t PCP_DEI_4:4;
    vuint32_t PCP_DEI_5:4;
    vuint32_t PCP_DEI_6:4;
    vuint32_t PCP_DEI_7:4;
  } B;
} NETC_F3_SI3_SIVLANIPVMR0_tag;

typedef union NETC_F3_SI3_SIVLANIPVMR1_union_tag { /* Station interface VLAN to IPV mapping register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:4;
    vuint32_t PCP_DEI_9:4;
    vuint32_t PCP_DEI_10:4;
    vuint32_t PCP_DEI_11:4;
    vuint32_t PCP_DEI_12:4;
    vuint32_t PCP_DEI_13:4;
    vuint32_t PCP_DEI_14:4;
    vuint32_t PCP_DEI_15:4;
  } B;
} NETC_F3_SI3_SIVLANIPVMR1_tag;

typedef union NETC_F3_SI3_SIIPVBDRMR0_union_tag { /* Station interface IPV to ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPV0BDR:3;
    vuint32_t _unused_3:1;
    vuint32_t IPV1BDR:3;
    vuint32_t _unused_7:1;
    vuint32_t IPV2BDR:3;
    vuint32_t _unused_11:1;
    vuint32_t IPV3BDR:3;
    vuint32_t _unused_15:1;
    vuint32_t IPV4BDR:3;
    vuint32_t _unused_19:1;
    vuint32_t IPV5BDR:3;
    vuint32_t _unused_23:1;
    vuint32_t IPV6BDR:3;
    vuint32_t _unused_27:1;
    vuint32_t IPV7BDR:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI3_SIIPVBDRMR0_tag;

typedef union NETC_F3_SI3_VSI_A_VSIMSGSR_union_tag { /* Virtual station interface message send register */
  vuint32_t R;
  struct {
    vuint32_t MB:1;
    vuint32_t MS:1;
    vuint32_t _unused_2:14;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI3_VSI_A_VSIMSGSR_tag;

typedef union NETC_F3_SI3_VSI_A_VSIMSGRR_union_tag { /* Virtual station interface message receive register */
  vuint32_t R;
  struct {
    vuint32_t MR:1;
    vuint32_t _unused_1:15;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI3_VSI_A_VSIMSGRR_tag;

typedef union NETC_F3_SI3_VSI_A_VSIMSGSNDAR0_union_tag { /* Virtual station interface message send register 0 */
  vuint32_t R;
  struct {
    vuint32_t MSIZE:5;
    vuint32_t _unused_5:1;
    vuint32_t ADDRL:26;
  } B;
} NETC_F3_SI3_VSI_A_VSIMSGSNDAR0_tag;

typedef union NETC_F3_SI3_VSI_A_VSIMSGSNDAR1_union_tag { /* Virtual station interface message send address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI3_VSI_A_VSIMSGSNDAR1_tag;

typedef struct NETC_F3_SI3_VSI_A_struct_tag {
  NETC_F3_SI3_VSI_A_VSIMSGSR_tag VSIMSGSR; /* Virtual station interface message send register */
  NETC_F3_SI3_VSI_A_VSIMSGRR_tag VSIMSGRR; /* Virtual station interface message receive register */
  uint8_t VSI_A_reserved0[4];
  NETC_F3_SI3_VSI_A_VSIMSGSNDAR0_tag VSIMSGSNDAR0; /* Virtual station interface message send register 0 */
  NETC_F3_SI3_VSI_A_VSIMSGSNDAR1_tag VSIMSGSNDAR1; /* Virtual station interface message send address register 1 */
} NETC_F3_SI3_VSI_A_tag;

typedef union NETC_F3_SI3_SIROCT0_union_tag { /* Station interface receive octets counter (ifInOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_LOW:32;
  } B;
} NETC_F3_SI3_SIROCT0_tag;

typedef union NETC_F3_SI3_SIROCT1_union_tag { /* Station interface receive octets counter (ifInOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_HIGH:32;
  } B;
} NETC_F3_SI3_SIROCT1_tag;

typedef union NETC_F3_SI3_SIRFRM0_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_LOW:32;
  } B;
} NETC_F3_SI3_SIRFRM0_tag;

typedef union NETC_F3_SI3_SIRFRM1_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_HIGH:32;
  } B;
} NETC_F3_SI3_SIRFRM1_tag;

typedef union NETC_F3_SI3_SIRUCA0_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_LOW:32;
  } B;
} NETC_F3_SI3_SIRUCA0_tag;

typedef union NETC_F3_SI3_SIRUCA1_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_HIGH:32;
  } B;
} NETC_F3_SI3_SIRUCA1_tag;

typedef union NETC_F3_SI3_SIRMCA0_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_LOW:32;
  } B;
} NETC_F3_SI3_SIRMCA0_tag;

typedef union NETC_F3_SI3_SIRMCA1_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_HIGH:32;
  } B;
} NETC_F3_SI3_SIRMCA1_tag;

typedef union NETC_F3_SI3_SITOCT0_union_tag { /* Station interface transmit octets counter (ifOutOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_LOW:32;
  } B;
} NETC_F3_SI3_SITOCT0_tag;

typedef union NETC_F3_SI3_SITOCT1_union_tag { /* Station interface transmit octets counter (ifOutOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_HIGH:32;
  } B;
} NETC_F3_SI3_SITOCT1_tag;

typedef union NETC_F3_SI3_SITFRM0_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_LOW:32;
  } B;
} NETC_F3_SI3_SITFRM0_tag;

typedef union NETC_F3_SI3_SITFRM1_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_HIGH:32;
  } B;
} NETC_F3_SI3_SITFRM1_tag;

typedef union NETC_F3_SI3_SITUCA0_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_LOW:32;
  } B;
} NETC_F3_SI3_SITUCA0_tag;

typedef union NETC_F3_SI3_SITUCA1_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_HIGH:32;
  } B;
} NETC_F3_SI3_SITUCA1_tag;

typedef union NETC_F3_SI3_SITMCA0_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_LOW:32;
  } B;
} NETC_F3_SI3_SITMCA0_tag;

typedef union NETC_F3_SI3_SITMCA1_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_HIGH:32;
  } B;
} NETC_F3_SI3_SITMCA1_tag;

typedef union NETC_F3_SI3_SIBLPR_union_tag { /* Station interface boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F3_SI3_SIBLPR_tag;

typedef union NETC_F3_SI3_SICBDRMR_union_tag { /* Station interface command BDR mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI3_SICBDRMR_tag;

typedef union NETC_F3_SI3_SICBDRSR_union_tag { /* Station interface command BDR status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI3_SICBDRSR_tag;

typedef union NETC_F3_SI3_SICBDRBAR0_union_tag { /* Station interface command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI3_SICBDRBAR0_tag;

typedef union NETC_F3_SI3_SICBDRBAR1_union_tag { /* Station interface command BDR base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI3_SICBDRBAR1_tag;

typedef union NETC_F3_SI3_SICBDRPIR_union_tag { /* Station interface command BDR producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI3_SICBDRPIR_tag;

typedef union NETC_F3_SI3_SICBDRCIR_union_tag { /* Station interface command BDR consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI3_SICBDRCIR_tag;

typedef union NETC_F3_SI3_SICBDRLENR_union_tag { /* Station interface command BDR length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_SI3_SICBDRLENR_tag;

typedef union NETC_F3_SI3_SICBDRIER_union_tag { /* Station interface command BDR interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI3_SICBDRIER_tag;

typedef union NETC_F3_SI3_SICBDRIDR_union_tag { /* Station interface command BDR interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI3_SICBDRIDR_tag;

typedef union NETC_F3_SI3_SICAPR0_union_tag { /* Station interface capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_RX_BDR:8;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F3_SI3_SICAPR0_tag;

typedef union NETC_F3_SI3_SICAPR1_union_tag { /* Station interface capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t NUM_RX_GRP:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI3_SICAPR1_tag;

typedef union NETC_F3_SI3_SICAPR2_union_tag { /* Station interface capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t VTP:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_SI3_SICAPR2_tag;

typedef union NETC_F3_SI3_VSI_VSIIER_union_tag { /* Virtual station interface interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MSIE:1;
    vuint32_t MRIE:1;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI3_VSI_VSIIER_tag;

typedef union NETC_F3_SI3_VSI_VSIIDR_union_tag { /* Virtual station interface interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXR:1;
    vuint32_t _unused_1:7;
    vuint32_t MS:1;
    vuint32_t MR:1;
    vuint32_t _unused_10:6;
    vuint32_t RXR:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI3_VSI_VSIIDR_tag;

typedef struct NETC_F3_SI3_VSI_struct_tag {
  NETC_F3_SI3_VSI_VSIIER_tag VSIIER;   /* Virtual station interface interrupt enable register */
  uint8_t VSI_reserved0[4];
  NETC_F3_SI3_VSI_VSIIDR_tag VSIIDR;   /* Virtual station interface interrupt detect register */
} NETC_F3_SI3_VSI_tag;

typedef union NETC_F3_SI3_SITXIDR0_union_tag { /* Station interface transmit interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXT0:1;
    vuint32_t TXT1:1;
    vuint32_t TXT2:1;
    vuint32_t TXT3:1;
    vuint32_t TXT4:1;
    vuint32_t TXT5:1;
    vuint32_t TXT6:1;
    vuint32_t TXT7:1;
    vuint32_t TXT8:1;
    vuint32_t TXT9:1;
    vuint32_t TXT10:1;
    vuint32_t TXT11:1;
    vuint32_t TXT12:1;
    vuint32_t TXT13:1;
    vuint32_t TXT14:1;
    vuint32_t TXT15:1;
    vuint32_t TXF0:1;
    vuint32_t TXF1:1;
    vuint32_t TXF2:1;
    vuint32_t TXF3:1;
    vuint32_t TXF4:1;
    vuint32_t TXF5:1;
    vuint32_t TXF6:1;
    vuint32_t TXF7:1;
    vuint32_t TXF8:1;
    vuint32_t TXF9:1;
    vuint32_t TXF10:1;
    vuint32_t TXF11:1;
    vuint32_t TXF12:1;
    vuint32_t TXF13:1;
    vuint32_t TXF14:1;
    vuint32_t TXF15:1;
  } B;
} NETC_F3_SI3_SITXIDR0_tag;

typedef union NETC_F3_SI3_SITXIDR1_union_tag { /* Station interface transmit interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t TXT16:1;
    vuint32_t TXT17:1;
    vuint32_t _unused_2:14;
    vuint32_t TXF16:1;
    vuint32_t TXF17:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_SI3_SITXIDR1_tag;

typedef union NETC_F3_SI3_SIRXIDR0_union_tag { /* Station interface receive interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t RX0:1;
    vuint32_t RX1:1;
    vuint32_t RX2:1;
    vuint32_t RX3:1;
    vuint32_t RX4:1;
    vuint32_t RX5:1;
    vuint32_t RX6:1;
    vuint32_t RX7:1;
    vuint32_t RX8:1;
    vuint32_t RX9:1;
    vuint32_t RX10:1;
    vuint32_t RX11:1;
    vuint32_t RX12:1;
    vuint32_t RX13:1;
    vuint32_t RX14:1;
    vuint32_t RX15:1;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI3_SIRXIDR0_tag;

typedef union NETC_F3_SI3_SIRXIDR1_union_tag { /* Station interface receive interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t RX16:1;
    vuint32_t RX17:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI3_SIRXIDR1_tag;

typedef union NETC_F3_SI3_SIMSIVR_union_tag { /* Station interface MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI3_SIMSIVR_tag;

typedef union NETC_F3_SI3_SICMSIVR_union_tag { /* Station interface command MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI3_SICMSIVR_tag;

typedef union NETC_F3_SI3_SITMRIER_union_tag { /* Station interface timer interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t SYNCE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI3_SITMRIER_tag;

typedef union NETC_F3_SI3_SITMRIDR_union_tag { /* Station interface timer interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI3_SITMRIDR_tag;

typedef union NETC_F3_SI3_SITMRMSIVR_union_tag { /* Station interface timer MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI3_SITMRMSIVR_tag;

typedef union NETC_F3_SI3_SIMSITRVR_union_tag { /* Station interface MSI-X transmit ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI3_SIMSITRVR_tag;

typedef union NETC_F3_SI3_SIMSIRRVR_union_tag { /* Station interface MSI-X receive ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI3_SIMSIRRVR_tag;

typedef union NETC_F3_SI3_SICMECR_union_tag { /* Station interface correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI3_SICMECR_tag;

typedef union NETC_F3_SI3_SICMESR_union_tag { /* Station interface correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_SI3_SICMESR_tag;

typedef union NETC_F3_SI3_SICMECTR_union_tag { /* Station interface correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI3_SICMECTR_tag;

typedef union NETC_F3_SI3_SIUPECR_union_tag { /* Station interface uncorrectable programming error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI3_SIUPECR_tag;

typedef union NETC_F3_SI3_SIUPESR_union_tag { /* Station interface uncorrectable programming error status register */
  vuint32_t R;
  struct {
    vuint32_t DROP_SI_EN:1;
    vuint32_t DROP_RING_EN:1;
    vuint32_t DROP_GRP_SEL:1;
    vuint32_t DROP_RING_SEL:1;
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t PE:1;                    /* Programming error */
  } B;
} NETC_F3_SI3_SIUPESR_tag;

typedef union NETC_F3_SI3_SIUPECTR_union_tag { /* Station interface uncorrectable programming error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;                /* Count */
  } B;
} NETC_F3_SI3_SIUPECTR_tag;

typedef union NETC_F3_SI3_SIUNSBECR_union_tag { /* Station interface uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI3_SIUNSBECR_tag;

typedef union NETC_F3_SI3_SIUNSBESR_union_tag { /* Station interface uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI3_SIUNSBESR_tag;

typedef union NETC_F3_SI3_SIUNSBECTR_union_tag { /* Station interface uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI3_SIUNSBECTR_tag;

typedef union NETC_F3_SI3_SIUFSBECR_union_tag { /* Station interface uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI3_SIUFSBECR_tag;

typedef union NETC_F3_SI3_SIUFSBESR_union_tag { /* Station interface uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI3_SIUFSBESR_tag;

typedef union NETC_F3_SI3_SIUNMECR_union_tag { /* Station interface uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI3_SIUNMECR_tag;

typedef union NETC_F3_SI3_SIUNMESR0_union_tag { /* Station interface uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI3_SIUNMESR0_tag;

typedef union NETC_F3_SI3_SIUNMESR1_union_tag { /* Station interface uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI3_SIUNMESR1_tag;

typedef union NETC_F3_SI3_SIUNMECTR_union_tag { /* Station interface uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI3_SIUNMECTR_tag;

typedef union NETC_F3_SI3_SIUFMECR_union_tag { /* Station interface uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI3_SIUFMECR_tag;

typedef union NETC_F3_SI3_SIUFMESR0_union_tag { /* Station interface uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI3_SIUFMESR0_tag;

typedef union NETC_F3_SI3_SIUFMESR1_union_tag { /* Station interface uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI3_SIUFMESR1_tag;

typedef union NETC_F3_SI3_SIUNIECR_union_tag { /* Station interface uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI3_SIUNIECR_tag;

typedef union NETC_F3_SI3_SIUNIESR_union_tag { /* Station interface uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI3_SIUNIESR_tag;

typedef union NETC_F3_SI3_SIUNIECTR_union_tag { /* Station interface uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI3_SIUNIECTR_tag;

typedef union NETC_F3_SI3_SIUFIECR_union_tag { /* Station interface uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI3_SIUFIECR_tag;

typedef union NETC_F3_SI3_SIUFIESR_union_tag { /* Station interface uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:16;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI3_SIUFIESR_tag;

typedef union NETC_F3_SI3_SIMAFTCAPR_union_tag { /* Station interface MAC address filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI3_SIMAFTCAPR_tag;

typedef union NETC_F3_SI3_SIVFTCAPR_union_tag { /* Station interface VLAN filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI3_SIVFTCAPR_tag;

typedef union NETC_F3_SI3_SIRFSCAPR_union_tag { /* Station interface RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_SI3_SIRFSCAPR_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBMR_union_tag { /* Tx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t PRIO:3;
    vuint32_t _unused_3:1;
    vuint32_t WRR:3;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t VIH:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI3_BDR_NUM_TBMR_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBSR_union_tag { /* Tx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI3_BDR_NUM_TBSR_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBBAR0_union_tag { /* Tx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI3_BDR_NUM_TBBAR0_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBBAR1_union_tag { /* Tx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI3_BDR_NUM_TBBAR1_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBPIR_union_tag { /* Tx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI3_BDR_NUM_TBPIR_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBCIR_union_tag { /* Tx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t STAT_ID:16;
  } B;
} NETC_F3_SI3_BDR_NUM_TBCIR_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBLENR_union_tag { /* Tx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI3_BDR_NUM_TBLENR_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBIER_union_tag { /* Tx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t TXTIE:1;
    vuint32_t TXFIE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI3_BDR_NUM_TBIER_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBIDR_union_tag { /* Tx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXT:1;
    vuint32_t TXF:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI3_BDR_NUM_TBIDR_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBICR0_union_tag { /* Tx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:4;
    vuint32_t _unused_4:27;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI3_BDR_NUM_TBICR0_tag;

typedef union NETC_F3_SI3_BDR_NUM_TBICR1_union_tag { /* Tx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI3_BDR_NUM_TBICR1_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBMR_union_tag { /* Rx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t AL:1;
    vuint32_t _unused_1:1;
    vuint32_t BDS:1;
    vuint32_t _unused_3:1;
    vuint32_t CM:1;
    vuint32_t VTE:1;
    vuint32_t VTPD:1;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI3_BDR_NUM_RBMR_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBSR_union_tag { /* Rx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI3_BDR_NUM_RBSR_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBBSR_union_tag { /* Rx BDR 0 buffer size register */
  vuint32_t R;
  struct {
    vuint32_t BSIZE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI3_BDR_NUM_RBBSR_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBCIR_union_tag { /* Rx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI3_BDR_NUM_RBCIR_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBBAR0_union_tag { /* Rx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI3_BDR_NUM_RBBAR0_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBBAR1_union_tag { /* Rx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI3_BDR_NUM_RBBAR1_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBPIR_union_tag { /* Rx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI3_BDR_NUM_RBPIR_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBLENR_union_tag { /* Rx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI3_BDR_NUM_RBLENR_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBDCR_union_tag { /* Rx BDR 0 drop count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_SI3_BDR_NUM_RBDCR_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBIER_union_tag { /* Rx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t RXTIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI3_BDR_NUM_RBIER_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBIDR_union_tag { /* Rx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t RXT:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI3_BDR_NUM_RBIDR_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBICR0_union_tag { /* Rx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:9;
    vuint32_t _unused_9:22;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI3_BDR_NUM_RBICR0_tag;

typedef union NETC_F3_SI3_BDR_NUM_RBICR1_union_tag { /* Rx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI3_BDR_NUM_RBICR1_tag;

typedef struct NETC_F3_SI3_BDR_NUM_struct_tag {
  NETC_F3_SI3_BDR_NUM_TBMR_tag TBMR;   /* Tx BDR 0 mode register */
  NETC_F3_SI3_BDR_NUM_TBSR_tag TBSR;   /* Tx BDR 0 status register */
  uint8_t BDR_NUM_reserved0[8];
  NETC_F3_SI3_BDR_NUM_TBBAR0_tag TBBAR0; /* Tx BDR 0 base address register 0 */
  NETC_F3_SI3_BDR_NUM_TBBAR1_tag TBBAR1; /* Tx BDR 0 base address register 1 */
  NETC_F3_SI3_BDR_NUM_TBPIR_tag TBPIR; /* Tx BDR 0 producer index register */
  NETC_F3_SI3_BDR_NUM_TBCIR_tag TBCIR; /* Tx BDR 0 consumer index register */
  NETC_F3_SI3_BDR_NUM_TBLENR_tag TBLENR; /* Tx BDR 0 length register */
  uint8_t BDR_NUM_reserved1[124];
  NETC_F3_SI3_BDR_NUM_TBIER_tag TBIER; /* Tx BDR 0 interrupt enable register */
  NETC_F3_SI3_BDR_NUM_TBIDR_tag TBIDR; /* Tx BDR 0 interrupt detect register */
  NETC_F3_SI3_BDR_NUM_TBICR0_tag TBICR0; /* Tx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI3_BDR_NUM_TBICR1_tag TBICR1; /* Tx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved2[80];
  NETC_F3_SI3_BDR_NUM_RBMR_tag RBMR;   /* Rx BDR 0 mode register */
  NETC_F3_SI3_BDR_NUM_RBSR_tag RBSR;   /* Rx BDR 0 status register */
  NETC_F3_SI3_BDR_NUM_RBBSR_tag RBBSR; /* Rx BDR 0 buffer size register */
  NETC_F3_SI3_BDR_NUM_RBCIR_tag RBCIR; /* Rx BDR 0 consumer index register */
  NETC_F3_SI3_BDR_NUM_RBBAR0_tag RBBAR0; /* Rx BDR 0 base address register 0 */
  NETC_F3_SI3_BDR_NUM_RBBAR1_tag RBBAR1; /* Rx BDR 0 base address register 1 */
  NETC_F3_SI3_BDR_NUM_RBPIR_tag RBPIR; /* Rx BDR 0 producer index register */
  uint8_t BDR_NUM_reserved3[4];
  NETC_F3_SI3_BDR_NUM_RBLENR_tag RBLENR; /* Rx BDR 0 length register */
  uint8_t BDR_NUM_reserved4[92];
  NETC_F3_SI3_BDR_NUM_RBDCR_tag RBDCR; /* Rx BDR 0 drop count register */
  uint8_t BDR_NUM_reserved5[28];
  NETC_F3_SI3_BDR_NUM_RBIER_tag RBIER; /* Rx BDR 0 interrupt enable register */
  NETC_F3_SI3_BDR_NUM_RBIDR_tag RBIDR; /* Rx BDR 0 interrupt detect register */
  NETC_F3_SI3_BDR_NUM_RBICR0_tag RBICR0; /* Rx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI3_BDR_NUM_RBICR1_tag RBICR1; /* Rx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved6[80];
} NETC_F3_SI3_BDR_NUM_tag;

struct NETC_F3_SI3_tag {
  NETC_F3_SI3_SIMR_tag SIMR;           /* Station interface mode register */
  NETC_F3_SI3_SISR_tag SISR;           /* Station interface status register */
  uint8_t NETC_F3_SI3_reserved0[16];
  NETC_F3_SI3_SICTR0_tag SICTR0;       /* Station interface current time register 0 */
  NETC_F3_SI3_SICTR1_tag SICTR1;       /* Station interface current time register 1 */
  NETC_F3_SI3_SIPCAPR0_tag SIPCAPR0;   /* Station interface port capability register 0 */
  NETC_F3_SI3_SIPCAPR1_tag SIPCAPR1;   /* Station interface port capability register 1 */
  uint8_t NETC_F3_SI3_reserved1[16];
  NETC_F3_SI3_SIRBGCR_tag SIRBGCR;     /* Station interface receive BDR group control register */
  uint8_t NETC_F3_SI3_reserved2[4];
  NETC_F3_SI3_SIBCAR_tag SIBCAR;       /* Station interface buffer cache attribute register */
  uint8_t NETC_F3_SI3_reserved3[4];
  NETC_F3_SI3_SICCAR_tag SICCAR;       /* Station interface command cache attribute register */
  uint8_t NETC_F3_SI3_reserved4[52];
  NETC_F3_SI3_SIPMAR0_tag SIPMAR0;     /* Station interface primary MAC address register 0 */
  NETC_F3_SI3_SIPMAR1_tag SIPMAR1;     /* Station interface primary MAC address register 1 */
  uint8_t NETC_F3_SI3_reserved5[8];
  NETC_F3_SI3_SICVLANR1_tag SICVLANR1; /* Station interface custom VLAN register 1 */
  NETC_F3_SI3_SICVLANR2_tag SICVLANR2; /* Station interface custom VLAN register 2 */
  uint8_t NETC_F3_SI3_reserved6[104];
  NETC_F3_SI3_SIVLANIPVMR0_tag SIVLANIPVMR0; /* Station interface VLAN to IPV mapping register 0 */
  NETC_F3_SI3_SIVLANIPVMR1_tag SIVLANIPVMR1; /* Station interface VLAN to IPV mapping register 1 */
  uint8_t NETC_F3_SI3_reserved7[72];
  NETC_F3_SI3_SIIPVBDRMR0_tag SIIPVBDRMR0; /* Station interface IPV to ring mapping register */
  uint8_t NETC_F3_SI3_reserved8[176];
  NETC_F3_SI3_VSI_A_tag VSI_A;
  uint8_t NETC_F3_SI3_reserved9[232];
  NETC_F3_SI3_SIROCT0_tag SIROCT0;     /* Station interface receive octets counter (ifInOctets) 0 */
  NETC_F3_SI3_SIROCT1_tag SIROCT1;     /* Station interface receive octets counter (ifInOctets) 1 */
  NETC_F3_SI3_SIRFRM0_tag SIRFRM0;     /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  NETC_F3_SI3_SIRFRM1_tag SIRFRM1;     /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  NETC_F3_SI3_SIRUCA0_tag SIRUCA0;     /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  NETC_F3_SI3_SIRUCA1_tag SIRUCA1;     /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  NETC_F3_SI3_SIRMCA0_tag SIRMCA0;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  NETC_F3_SI3_SIRMCA1_tag SIRMCA1;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  NETC_F3_SI3_SITOCT0_tag SITOCT0;     /* Station interface transmit octets counter (ifOutOctets) 0 */
  NETC_F3_SI3_SITOCT1_tag SITOCT1;     /* Station interface transmit octets counter (ifOutOctets) 1 */
  NETC_F3_SI3_SITFRM0_tag SITFRM0;     /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  NETC_F3_SI3_SITFRM1_tag SITFRM1;     /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  NETC_F3_SI3_SITUCA0_tag SITUCA0;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  NETC_F3_SI3_SITUCA1_tag SITUCA1;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  NETC_F3_SI3_SITMCA0_tag SITMCA0;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  NETC_F3_SI3_SITMCA1_tag SITMCA1;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  uint8_t NETC_F3_SI3_reserved10[176];
  NETC_F3_SI3_SIBLPR_tag SIBLPR[2];    /* Station interface boot loader parameter register 0 */
  uint8_t NETC_F3_SI3_reserved11[1032];
  NETC_F3_SI3_SICBDRMR_tag SICBDRMR;   /* Station interface command BDR mode register */
  NETC_F3_SI3_SICBDRSR_tag SICBDRSR;   /* Station interface command BDR status register */
  uint8_t NETC_F3_SI3_reserved12[8];
  NETC_F3_SI3_SICBDRBAR0_tag SICBDRBAR0; /* Station interface command BDR base address register 0 */
  NETC_F3_SI3_SICBDRBAR1_tag SICBDRBAR1; /* Station interface command BDR base address register 1 */
  NETC_F3_SI3_SICBDRPIR_tag SICBDRPIR; /* Station interface command BDR producer index register */
  NETC_F3_SI3_SICBDRCIR_tag SICBDRCIR; /* Station interface command BDR consumer index register */
  NETC_F3_SI3_SICBDRLENR_tag SICBDRLENR; /* Station interface command BDR length register */
  uint8_t NETC_F3_SI3_reserved13[124];
  NETC_F3_SI3_SICBDRIER_tag SICBDRIER; /* Station interface command BDR interrupt enable register */
  NETC_F3_SI3_SICBDRIDR_tag SICBDRIDR; /* Station interface command BDR interrupt detect register */
  uint8_t NETC_F3_SI3_reserved14[88];
  NETC_F3_SI3_SICAPR0_tag SICAPR0;     /* Station interface capability register 0 */
  NETC_F3_SI3_SICAPR1_tag SICAPR1;     /* Station interface capability register 1 */
  NETC_F3_SI3_SICAPR2_tag SICAPR2;     /* Station interface capability register 2 */
  uint8_t NETC_F3_SI3_reserved15[244];
  NETC_F3_SI3_VSI_tag VSI;
  uint8_t NETC_F3_SI3_reserved16[12];
  NETC_F3_SI3_SITXIDR0_tag SITXIDR0;   /* Station interface transmit interrupt detect register 0 */
  NETC_F3_SI3_SITXIDR1_tag SITXIDR1;   /* Station interface transmit interrupt detect register 1 */
  uint8_t NETC_F3_SI3_reserved17[8];
  NETC_F3_SI3_SIRXIDR0_tag SIRXIDR0;   /* Station interface receive interrupt detect register 0 */
  NETC_F3_SI3_SIRXIDR1_tag SIRXIDR1;   /* Station interface receive interrupt detect register 1 */
  NETC_F3_SI3_SIMSIVR_tag SIMSIVR;     /* Station interface MSI-X vector register */
  NETC_F3_SI3_SICMSIVR_tag SICMSIVR;   /* Station interface command MSI-X vector register */
  uint8_t NETC_F3_SI3_reserved18[8];
  NETC_F3_SI3_SITMRIER_tag SITMRIER;   /* Station interface timer interrupt enable register */
  NETC_F3_SI3_SITMRIDR_tag SITMRIDR;   /* Station interface timer interrupt detect register */
  uint8_t NETC_F3_SI3_reserved19[4];
  NETC_F3_SI3_SITMRMSIVR_tag SITMRMSIVR; /* Station interface timer MSI-X vector register */
  uint8_t NETC_F3_SI3_reserved20[176];
  NETC_F3_SI3_SIMSITRVR_tag SIMSITRVR[18]; /* Station interface MSI-X transmit ring 0 vector register */
  uint8_t NETC_F3_SI3_reserved21[56];
  NETC_F3_SI3_SIMSIRRVR_tag SIMSIRRVR[18]; /* Station interface MSI-X receive ring 0 vector register */
  uint8_t NETC_F3_SI3_reserved22[568];
  NETC_F3_SI3_SICMECR_tag SICMECR;     /* Station interface correctable memory error configuration register */
  NETC_F3_SI3_SICMESR_tag SICMESR;     /* Station interface correctable memory error status register */
  uint8_t NETC_F3_SI3_reserved23[4];
  NETC_F3_SI3_SICMECTR_tag SICMECTR;   /* Station interface correctable memory error count register */
  NETC_F3_SI3_SIUPECR_tag SIUPECR;     /* Station interface uncorrectable programming error configuration register */
  NETC_F3_SI3_SIUPESR_tag SIUPESR;     /* Station interface uncorrectable programming error status register */
  uint8_t NETC_F3_SI3_reserved24[4];
  NETC_F3_SI3_SIUPECTR_tag SIUPECTR;   /* Station interface uncorrectable programming error count register */
  NETC_F3_SI3_SIUNSBECR_tag SIUNSBECR; /* Station interface uncorrectable non-fatal system bus error configuration register */
  NETC_F3_SI3_SIUNSBESR_tag SIUNSBESR; /* Station interface uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F3_SI3_reserved25[4];
  NETC_F3_SI3_SIUNSBECTR_tag SIUNSBECTR; /* Station interface uncorrectable non-fatal system bus error count register */
  NETC_F3_SI3_SIUFSBECR_tag SIUFSBECR; /* Station interface uncorrectable fatal system bus error configuration register */
  NETC_F3_SI3_SIUFSBESR_tag SIUFSBESR; /* Station interface uncorrectable fatal system bus error status register */
  uint8_t NETC_F3_SI3_reserved26[8];
  NETC_F3_SI3_SIUNMECR_tag SIUNMECR;   /* Station interface uncorrectable non-fatal memory error configuration register */
  NETC_F3_SI3_SIUNMESR0_tag SIUNMESR0; /* Station interface uncorrectable non-fatal memory error status register 0 */
  NETC_F3_SI3_SIUNMESR1_tag SIUNMESR1; /* Station interface uncorrectable non-fatal memory error status register 1 */
  NETC_F3_SI3_SIUNMECTR_tag SIUNMECTR; /* Station interface uncorrectable non-fatal memory error count register */
  NETC_F3_SI3_SIUFMECR_tag SIUFMECR;   /* Station interface uncorrectable fatal memory error configuration register */
  NETC_F3_SI3_SIUFMESR0_tag SIUFMESR0; /* Station interface uncorrectable fatal memory error status register 0 */
  NETC_F3_SI3_SIUFMESR1_tag SIUFMESR1; /* Station interface uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_SI3_reserved27[4];
  NETC_F3_SI3_SIUNIECR_tag SIUNIECR;   /* Station interface uncorrectable non-fatal integrity error configuration register */
  NETC_F3_SI3_SIUNIESR_tag SIUNIESR;   /* Station interface uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_SI3_reserved28[4];
  NETC_F3_SI3_SIUNIECTR_tag SIUNIECTR; /* Station interface uncorrectable non-fatal integrity error count register */
  NETC_F3_SI3_SIUFIECR_tag SIUFIECR;   /* Station interface uncorrectable fatal integrity error configuration register */
  NETC_F3_SI3_SIUFIESR_tag SIUFIESR;   /* Station interface uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_SI3_reserved29[392];
  NETC_F3_SI3_SIMAFTCAPR_tag SIMAFTCAPR; /* Station interface MAC address filter table capability register */
  uint8_t NETC_F3_SI3_reserved30[252];
  NETC_F3_SI3_SIVFTCAPR_tag SIVFTCAPR; /* Station interface VLAN filter table capability register */
  uint8_t NETC_F3_SI3_reserved31[252];
  NETC_F3_SI3_SIRFSCAPR_tag SIRFSCAPR; /* Station interface RFS capability register */
  uint8_t NETC_F3_SI3_reserved32[28156];
  NETC_F3_SI3_BDR_NUM_tag BDR_NUM[18];
};


/* ============================================================================
   =============================== Module: NETC_F3_SI4 ========================
   ============================================================================ */

typedef union NETC_F3_SI4_SIMR_union_tag { /* Station interface mode register */
  vuint32_t R;
  struct {
    vuint32_t RSSE:1;
    vuint32_t RNUM:1;
    vuint32_t RNMM:1;
    vuint32_t RNBM:1;
    vuint32_t V2IPVE:1;
    vuint32_t _unused_5:11;
    vuint32_t DEFAULT_RX_GROUP:1;
    vuint32_t _unused_17:14;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI4_SIMR_tag;

typedef union NETC_F3_SI4_SISR_union_tag { /* Station interface status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;
    vuint32_t MAC_UP:1;
    vuint32_t MAC_MP:1;
    vuint32_t VLAN_P:1;
    vuint32_t VLAN_UTA:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_SI4_SISR_tag;

typedef union NETC_F3_SI4_SICTR0_union_tag { /* Station interface current time register 0 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI4_SICTR0_tag;

typedef union NETC_F3_SI4_SICTR1_union_tag { /* Station interface current time register 1 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI4_SICTR1_tag;

typedef union NETC_F3_SI4_SIPCAPR0_union_tag { /* Station interface port capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;                   /* Receive Flow Steering */
    vuint32_t FP:1;                    /* Frame Preemption */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t TSD:1;                   /* Time Specific Departure */
    vuint32_t CBS:1;                   /* Credit Based Shaping (CBS) */
    vuint32_t _unused_7:1;
    vuint32_t RSS:1;                   /* Receive Side Scaling */
    vuint32_t PSFP:1;                  /* Per-Stream Filtering and Policing (IEEE 802.1Qci) */
    vuint32_t IPFLT:1;                 /* Ingress Port Filtering */
    vuint32_t RP:1;                    /* Rate Policing */
    vuint32_t _unused_12:1;
    vuint32_t WO:1;                    /* Wake-on-LAN */
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional Safety */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI4_SIPCAPR0_tag;

typedef union NETC_F3_SI4_SIPCAPR1_union_tag { /* Station interface port capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:6;
    vuint32_t _unused_18:13;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_SI4_SIPCAPR1_tag;

typedef union NETC_F3_SI4_SIRBGCR_union_tag { /* Station interface receive BDR group control register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GROUPS:2;
    vuint32_t _unused_2:14;
    vuint32_t RINGS_PER_GROUP:3;
    vuint32_t _unused_19:13;
  } B;
} NETC_F3_SI4_SIRBGCR_tag;

typedef union NETC_F3_SI4_SIBCAR_union_tag { /* Station interface buffer cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI4_SIBCAR_tag;

typedef union NETC_F3_SI4_SICCAR_union_tag { /* Station interface command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI4_SICCAR_tag;

typedef union NETC_F3_SI4_SIPMAR0_union_tag { /* Station interface primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_SI4_SIPMAR0_tag;

typedef union NETC_F3_SI4_SIPMAR1_union_tag { /* Station interface primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI4_SIPMAR1_tag;

typedef union NETC_F3_SI4_SICVLANR1_union_tag { /* Station interface custom VLAN register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI4_SICVLANR1_tag;

typedef union NETC_F3_SI4_SICVLANR2_union_tag { /* Station interface custom VLAN register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI4_SICVLANR2_tag;

typedef union NETC_F3_SI4_SIVLANIPVMR0_union_tag { /* Station interface VLAN to IPV mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:4;
    vuint32_t PCP_DEI_1:4;
    vuint32_t PCP_DEI_2:4;
    vuint32_t PCP_DEI_3:4;
    vuint32_t PCP_DEI_4:4;
    vuint32_t PCP_DEI_5:4;
    vuint32_t PCP_DEI_6:4;
    vuint32_t PCP_DEI_7:4;
  } B;
} NETC_F3_SI4_SIVLANIPVMR0_tag;

typedef union NETC_F3_SI4_SIVLANIPVMR1_union_tag { /* Station interface VLAN to IPV mapping register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:4;
    vuint32_t PCP_DEI_9:4;
    vuint32_t PCP_DEI_10:4;
    vuint32_t PCP_DEI_11:4;
    vuint32_t PCP_DEI_12:4;
    vuint32_t PCP_DEI_13:4;
    vuint32_t PCP_DEI_14:4;
    vuint32_t PCP_DEI_15:4;
  } B;
} NETC_F3_SI4_SIVLANIPVMR1_tag;

typedef union NETC_F3_SI4_SIIPVBDRMR0_union_tag { /* Station interface IPV to ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPV0BDR:3;
    vuint32_t _unused_3:1;
    vuint32_t IPV1BDR:3;
    vuint32_t _unused_7:1;
    vuint32_t IPV2BDR:3;
    vuint32_t _unused_11:1;
    vuint32_t IPV3BDR:3;
    vuint32_t _unused_15:1;
    vuint32_t IPV4BDR:3;
    vuint32_t _unused_19:1;
    vuint32_t IPV5BDR:3;
    vuint32_t _unused_23:1;
    vuint32_t IPV6BDR:3;
    vuint32_t _unused_27:1;
    vuint32_t IPV7BDR:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI4_SIIPVBDRMR0_tag;

typedef union NETC_F3_SI4_VSI_A_VSIMSGSR_union_tag { /* Virtual station interface message send register */
  vuint32_t R;
  struct {
    vuint32_t MB:1;
    vuint32_t MS:1;
    vuint32_t _unused_2:14;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI4_VSI_A_VSIMSGSR_tag;

typedef union NETC_F3_SI4_VSI_A_VSIMSGRR_union_tag { /* Virtual station interface message receive register */
  vuint32_t R;
  struct {
    vuint32_t MR:1;
    vuint32_t _unused_1:15;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI4_VSI_A_VSIMSGRR_tag;

typedef union NETC_F3_SI4_VSI_A_VSIMSGSNDAR0_union_tag { /* Virtual station interface message send register 0 */
  vuint32_t R;
  struct {
    vuint32_t MSIZE:5;
    vuint32_t _unused_5:1;
    vuint32_t ADDRL:26;
  } B;
} NETC_F3_SI4_VSI_A_VSIMSGSNDAR0_tag;

typedef union NETC_F3_SI4_VSI_A_VSIMSGSNDAR1_union_tag { /* Virtual station interface message send address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI4_VSI_A_VSIMSGSNDAR1_tag;

typedef struct NETC_F3_SI4_VSI_A_struct_tag {
  NETC_F3_SI4_VSI_A_VSIMSGSR_tag VSIMSGSR; /* Virtual station interface message send register */
  NETC_F3_SI4_VSI_A_VSIMSGRR_tag VSIMSGRR; /* Virtual station interface message receive register */
  uint8_t VSI_A_reserved0[4];
  NETC_F3_SI4_VSI_A_VSIMSGSNDAR0_tag VSIMSGSNDAR0; /* Virtual station interface message send register 0 */
  NETC_F3_SI4_VSI_A_VSIMSGSNDAR1_tag VSIMSGSNDAR1; /* Virtual station interface message send address register 1 */
} NETC_F3_SI4_VSI_A_tag;

typedef union NETC_F3_SI4_SIROCT0_union_tag { /* Station interface receive octets counter (ifInOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_LOW:32;
  } B;
} NETC_F3_SI4_SIROCT0_tag;

typedef union NETC_F3_SI4_SIROCT1_union_tag { /* Station interface receive octets counter (ifInOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_HIGH:32;
  } B;
} NETC_F3_SI4_SIROCT1_tag;

typedef union NETC_F3_SI4_SIRFRM0_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_LOW:32;
  } B;
} NETC_F3_SI4_SIRFRM0_tag;

typedef union NETC_F3_SI4_SIRFRM1_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_HIGH:32;
  } B;
} NETC_F3_SI4_SIRFRM1_tag;

typedef union NETC_F3_SI4_SIRUCA0_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_LOW:32;
  } B;
} NETC_F3_SI4_SIRUCA0_tag;

typedef union NETC_F3_SI4_SIRUCA1_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_HIGH:32;
  } B;
} NETC_F3_SI4_SIRUCA1_tag;

typedef union NETC_F3_SI4_SIRMCA0_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_LOW:32;
  } B;
} NETC_F3_SI4_SIRMCA0_tag;

typedef union NETC_F3_SI4_SIRMCA1_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_HIGH:32;
  } B;
} NETC_F3_SI4_SIRMCA1_tag;

typedef union NETC_F3_SI4_SITOCT0_union_tag { /* Station interface transmit octets counter (ifOutOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_LOW:32;
  } B;
} NETC_F3_SI4_SITOCT0_tag;

typedef union NETC_F3_SI4_SITOCT1_union_tag { /* Station interface transmit octets counter (ifOutOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_HIGH:32;
  } B;
} NETC_F3_SI4_SITOCT1_tag;

typedef union NETC_F3_SI4_SITFRM0_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_LOW:32;
  } B;
} NETC_F3_SI4_SITFRM0_tag;

typedef union NETC_F3_SI4_SITFRM1_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_HIGH:32;
  } B;
} NETC_F3_SI4_SITFRM1_tag;

typedef union NETC_F3_SI4_SITUCA0_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_LOW:32;
  } B;
} NETC_F3_SI4_SITUCA0_tag;

typedef union NETC_F3_SI4_SITUCA1_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_HIGH:32;
  } B;
} NETC_F3_SI4_SITUCA1_tag;

typedef union NETC_F3_SI4_SITMCA0_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_LOW:32;
  } B;
} NETC_F3_SI4_SITMCA0_tag;

typedef union NETC_F3_SI4_SITMCA1_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_HIGH:32;
  } B;
} NETC_F3_SI4_SITMCA1_tag;

typedef union NETC_F3_SI4_SIBLPR_union_tag { /* Station interface boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F3_SI4_SIBLPR_tag;

typedef union NETC_F3_SI4_SICBDRMR_union_tag { /* Station interface command BDR mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI4_SICBDRMR_tag;

typedef union NETC_F3_SI4_SICBDRSR_union_tag { /* Station interface command BDR status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI4_SICBDRSR_tag;

typedef union NETC_F3_SI4_SICBDRBAR0_union_tag { /* Station interface command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI4_SICBDRBAR0_tag;

typedef union NETC_F3_SI4_SICBDRBAR1_union_tag { /* Station interface command BDR base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI4_SICBDRBAR1_tag;

typedef union NETC_F3_SI4_SICBDRPIR_union_tag { /* Station interface command BDR producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI4_SICBDRPIR_tag;

typedef union NETC_F3_SI4_SICBDRCIR_union_tag { /* Station interface command BDR consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI4_SICBDRCIR_tag;

typedef union NETC_F3_SI4_SICBDRLENR_union_tag { /* Station interface command BDR length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_SI4_SICBDRLENR_tag;

typedef union NETC_F3_SI4_SICBDRIER_union_tag { /* Station interface command BDR interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI4_SICBDRIER_tag;

typedef union NETC_F3_SI4_SICBDRIDR_union_tag { /* Station interface command BDR interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI4_SICBDRIDR_tag;

typedef union NETC_F3_SI4_SICAPR0_union_tag { /* Station interface capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_RX_BDR:8;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F3_SI4_SICAPR0_tag;

typedef union NETC_F3_SI4_SICAPR1_union_tag { /* Station interface capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t NUM_RX_GRP:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI4_SICAPR1_tag;

typedef union NETC_F3_SI4_SICAPR2_union_tag { /* Station interface capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t VTP:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_SI4_SICAPR2_tag;

typedef union NETC_F3_SI4_VSI_VSIIER_union_tag { /* Virtual station interface interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MSIE:1;
    vuint32_t MRIE:1;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI4_VSI_VSIIER_tag;

typedef union NETC_F3_SI4_VSI_VSIIDR_union_tag { /* Virtual station interface interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXR:1;
    vuint32_t _unused_1:7;
    vuint32_t MS:1;
    vuint32_t MR:1;
    vuint32_t _unused_10:6;
    vuint32_t RXR:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI4_VSI_VSIIDR_tag;

typedef struct NETC_F3_SI4_VSI_struct_tag {
  NETC_F3_SI4_VSI_VSIIER_tag VSIIER;   /* Virtual station interface interrupt enable register */
  uint8_t VSI_reserved0[4];
  NETC_F3_SI4_VSI_VSIIDR_tag VSIIDR;   /* Virtual station interface interrupt detect register */
} NETC_F3_SI4_VSI_tag;

typedef union NETC_F3_SI4_SITXIDR0_union_tag { /* Station interface transmit interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXT0:1;
    vuint32_t TXT1:1;
    vuint32_t TXT2:1;
    vuint32_t TXT3:1;
    vuint32_t TXT4:1;
    vuint32_t TXT5:1;
    vuint32_t TXT6:1;
    vuint32_t TXT7:1;
    vuint32_t TXT8:1;
    vuint32_t TXT9:1;
    vuint32_t TXT10:1;
    vuint32_t TXT11:1;
    vuint32_t TXT12:1;
    vuint32_t TXT13:1;
    vuint32_t TXT14:1;
    vuint32_t TXT15:1;
    vuint32_t TXF0:1;
    vuint32_t TXF1:1;
    vuint32_t TXF2:1;
    vuint32_t TXF3:1;
    vuint32_t TXF4:1;
    vuint32_t TXF5:1;
    vuint32_t TXF6:1;
    vuint32_t TXF7:1;
    vuint32_t TXF8:1;
    vuint32_t TXF9:1;
    vuint32_t TXF10:1;
    vuint32_t TXF11:1;
    vuint32_t TXF12:1;
    vuint32_t TXF13:1;
    vuint32_t TXF14:1;
    vuint32_t TXF15:1;
  } B;
} NETC_F3_SI4_SITXIDR0_tag;

typedef union NETC_F3_SI4_SITXIDR1_union_tag { /* Station interface transmit interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t TXT16:1;
    vuint32_t TXT17:1;
    vuint32_t _unused_2:14;
    vuint32_t TXF16:1;
    vuint32_t TXF17:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_SI4_SITXIDR1_tag;

typedef union NETC_F3_SI4_SIRXIDR0_union_tag { /* Station interface receive interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t RX0:1;
    vuint32_t RX1:1;
    vuint32_t RX2:1;
    vuint32_t RX3:1;
    vuint32_t RX4:1;
    vuint32_t RX5:1;
    vuint32_t RX6:1;
    vuint32_t RX7:1;
    vuint32_t RX8:1;
    vuint32_t RX9:1;
    vuint32_t RX10:1;
    vuint32_t RX11:1;
    vuint32_t RX12:1;
    vuint32_t RX13:1;
    vuint32_t RX14:1;
    vuint32_t RX15:1;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI4_SIRXIDR0_tag;

typedef union NETC_F3_SI4_SIRXIDR1_union_tag { /* Station interface receive interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t RX16:1;
    vuint32_t RX17:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI4_SIRXIDR1_tag;

typedef union NETC_F3_SI4_SIMSIVR_union_tag { /* Station interface MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI4_SIMSIVR_tag;

typedef union NETC_F3_SI4_SICMSIVR_union_tag { /* Station interface command MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI4_SICMSIVR_tag;

typedef union NETC_F3_SI4_SITMRIER_union_tag { /* Station interface timer interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t SYNCE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI4_SITMRIER_tag;

typedef union NETC_F3_SI4_SITMRIDR_union_tag { /* Station interface timer interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI4_SITMRIDR_tag;

typedef union NETC_F3_SI4_SITMRMSIVR_union_tag { /* Station interface timer MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI4_SITMRMSIVR_tag;

typedef union NETC_F3_SI4_SIMSITRVR_union_tag { /* Station interface MSI-X transmit ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI4_SIMSITRVR_tag;

typedef union NETC_F3_SI4_SIMSIRRVR_union_tag { /* Station interface MSI-X receive ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI4_SIMSIRRVR_tag;

typedef union NETC_F3_SI4_SICMECR_union_tag { /* Station interface correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI4_SICMECR_tag;

typedef union NETC_F3_SI4_SICMESR_union_tag { /* Station interface correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_SI4_SICMESR_tag;

typedef union NETC_F3_SI4_SICMECTR_union_tag { /* Station interface correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI4_SICMECTR_tag;

typedef union NETC_F3_SI4_SIUPECR_union_tag { /* Station interface uncorrectable programming error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI4_SIUPECR_tag;

typedef union NETC_F3_SI4_SIUPESR_union_tag { /* Station interface uncorrectable programming error status register */
  vuint32_t R;
  struct {
    vuint32_t DROP_SI_EN:1;
    vuint32_t DROP_RING_EN:1;
    vuint32_t DROP_GRP_SEL:1;
    vuint32_t DROP_RING_SEL:1;
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t PE:1;                    /* Programming error */
  } B;
} NETC_F3_SI4_SIUPESR_tag;

typedef union NETC_F3_SI4_SIUPECTR_union_tag { /* Station interface uncorrectable programming error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;                /* Count */
  } B;
} NETC_F3_SI4_SIUPECTR_tag;

typedef union NETC_F3_SI4_SIUNSBECR_union_tag { /* Station interface uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI4_SIUNSBECR_tag;

typedef union NETC_F3_SI4_SIUNSBESR_union_tag { /* Station interface uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI4_SIUNSBESR_tag;

typedef union NETC_F3_SI4_SIUNSBECTR_union_tag { /* Station interface uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI4_SIUNSBECTR_tag;

typedef union NETC_F3_SI4_SIUFSBECR_union_tag { /* Station interface uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI4_SIUFSBECR_tag;

typedef union NETC_F3_SI4_SIUFSBESR_union_tag { /* Station interface uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI4_SIUFSBESR_tag;

typedef union NETC_F3_SI4_SIUNMECR_union_tag { /* Station interface uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI4_SIUNMECR_tag;

typedef union NETC_F3_SI4_SIUNMESR0_union_tag { /* Station interface uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI4_SIUNMESR0_tag;

typedef union NETC_F3_SI4_SIUNMESR1_union_tag { /* Station interface uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI4_SIUNMESR1_tag;

typedef union NETC_F3_SI4_SIUNMECTR_union_tag { /* Station interface uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI4_SIUNMECTR_tag;

typedef union NETC_F3_SI4_SIUFMECR_union_tag { /* Station interface uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI4_SIUFMECR_tag;

typedef union NETC_F3_SI4_SIUFMESR0_union_tag { /* Station interface uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI4_SIUFMESR0_tag;

typedef union NETC_F3_SI4_SIUFMESR1_union_tag { /* Station interface uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI4_SIUFMESR1_tag;

typedef union NETC_F3_SI4_SIUNIECR_union_tag { /* Station interface uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI4_SIUNIECR_tag;

typedef union NETC_F3_SI4_SIUNIESR_union_tag { /* Station interface uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI4_SIUNIESR_tag;

typedef union NETC_F3_SI4_SIUNIECTR_union_tag { /* Station interface uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI4_SIUNIECTR_tag;

typedef union NETC_F3_SI4_SIUFIECR_union_tag { /* Station interface uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI4_SIUFIECR_tag;

typedef union NETC_F3_SI4_SIUFIESR_union_tag { /* Station interface uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:16;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI4_SIUFIESR_tag;

typedef union NETC_F3_SI4_SIMAFTCAPR_union_tag { /* Station interface MAC address filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI4_SIMAFTCAPR_tag;

typedef union NETC_F3_SI4_SIVFTCAPR_union_tag { /* Station interface VLAN filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI4_SIVFTCAPR_tag;

typedef union NETC_F3_SI4_SIRFSCAPR_union_tag { /* Station interface RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_SI4_SIRFSCAPR_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBMR_union_tag { /* Tx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t PRIO:3;
    vuint32_t _unused_3:1;
    vuint32_t WRR:3;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t VIH:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI4_BDR_NUM_TBMR_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBSR_union_tag { /* Tx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI4_BDR_NUM_TBSR_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBBAR0_union_tag { /* Tx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI4_BDR_NUM_TBBAR0_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBBAR1_union_tag { /* Tx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI4_BDR_NUM_TBBAR1_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBPIR_union_tag { /* Tx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI4_BDR_NUM_TBPIR_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBCIR_union_tag { /* Tx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t STAT_ID:16;
  } B;
} NETC_F3_SI4_BDR_NUM_TBCIR_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBLENR_union_tag { /* Tx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI4_BDR_NUM_TBLENR_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBIER_union_tag { /* Tx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t TXTIE:1;
    vuint32_t TXFIE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI4_BDR_NUM_TBIER_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBIDR_union_tag { /* Tx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXT:1;
    vuint32_t TXF:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI4_BDR_NUM_TBIDR_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBICR0_union_tag { /* Tx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:4;
    vuint32_t _unused_4:27;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI4_BDR_NUM_TBICR0_tag;

typedef union NETC_F3_SI4_BDR_NUM_TBICR1_union_tag { /* Tx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI4_BDR_NUM_TBICR1_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBMR_union_tag { /* Rx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t AL:1;
    vuint32_t _unused_1:1;
    vuint32_t BDS:1;
    vuint32_t _unused_3:1;
    vuint32_t CM:1;
    vuint32_t VTE:1;
    vuint32_t VTPD:1;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI4_BDR_NUM_RBMR_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBSR_union_tag { /* Rx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI4_BDR_NUM_RBSR_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBBSR_union_tag { /* Rx BDR 0 buffer size register */
  vuint32_t R;
  struct {
    vuint32_t BSIZE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI4_BDR_NUM_RBBSR_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBCIR_union_tag { /* Rx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI4_BDR_NUM_RBCIR_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBBAR0_union_tag { /* Rx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI4_BDR_NUM_RBBAR0_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBBAR1_union_tag { /* Rx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI4_BDR_NUM_RBBAR1_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBPIR_union_tag { /* Rx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI4_BDR_NUM_RBPIR_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBLENR_union_tag { /* Rx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI4_BDR_NUM_RBLENR_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBDCR_union_tag { /* Rx BDR 0 drop count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_SI4_BDR_NUM_RBDCR_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBIER_union_tag { /* Rx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t RXTIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI4_BDR_NUM_RBIER_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBIDR_union_tag { /* Rx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t RXT:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI4_BDR_NUM_RBIDR_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBICR0_union_tag { /* Rx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:9;
    vuint32_t _unused_9:22;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI4_BDR_NUM_RBICR0_tag;

typedef union NETC_F3_SI4_BDR_NUM_RBICR1_union_tag { /* Rx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI4_BDR_NUM_RBICR1_tag;

typedef struct NETC_F3_SI4_BDR_NUM_struct_tag {
  NETC_F3_SI4_BDR_NUM_TBMR_tag TBMR;   /* Tx BDR 0 mode register */
  NETC_F3_SI4_BDR_NUM_TBSR_tag TBSR;   /* Tx BDR 0 status register */
  uint8_t BDR_NUM_reserved0[8];
  NETC_F3_SI4_BDR_NUM_TBBAR0_tag TBBAR0; /* Tx BDR 0 base address register 0 */
  NETC_F3_SI4_BDR_NUM_TBBAR1_tag TBBAR1; /* Tx BDR 0 base address register 1 */
  NETC_F3_SI4_BDR_NUM_TBPIR_tag TBPIR; /* Tx BDR 0 producer index register */
  NETC_F3_SI4_BDR_NUM_TBCIR_tag TBCIR; /* Tx BDR 0 consumer index register */
  NETC_F3_SI4_BDR_NUM_TBLENR_tag TBLENR; /* Tx BDR 0 length register */
  uint8_t BDR_NUM_reserved1[124];
  NETC_F3_SI4_BDR_NUM_TBIER_tag TBIER; /* Tx BDR 0 interrupt enable register */
  NETC_F3_SI4_BDR_NUM_TBIDR_tag TBIDR; /* Tx BDR 0 interrupt detect register */
  NETC_F3_SI4_BDR_NUM_TBICR0_tag TBICR0; /* Tx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI4_BDR_NUM_TBICR1_tag TBICR1; /* Tx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved2[80];
  NETC_F3_SI4_BDR_NUM_RBMR_tag RBMR;   /* Rx BDR 0 mode register */
  NETC_F3_SI4_BDR_NUM_RBSR_tag RBSR;   /* Rx BDR 0 status register */
  NETC_F3_SI4_BDR_NUM_RBBSR_tag RBBSR; /* Rx BDR 0 buffer size register */
  NETC_F3_SI4_BDR_NUM_RBCIR_tag RBCIR; /* Rx BDR 0 consumer index register */
  NETC_F3_SI4_BDR_NUM_RBBAR0_tag RBBAR0; /* Rx BDR 0 base address register 0 */
  NETC_F3_SI4_BDR_NUM_RBBAR1_tag RBBAR1; /* Rx BDR 0 base address register 1 */
  NETC_F3_SI4_BDR_NUM_RBPIR_tag RBPIR; /* Rx BDR 0 producer index register */
  uint8_t BDR_NUM_reserved3[4];
  NETC_F3_SI4_BDR_NUM_RBLENR_tag RBLENR; /* Rx BDR 0 length register */
  uint8_t BDR_NUM_reserved4[92];
  NETC_F3_SI4_BDR_NUM_RBDCR_tag RBDCR; /* Rx BDR 0 drop count register */
  uint8_t BDR_NUM_reserved5[28];
  NETC_F3_SI4_BDR_NUM_RBIER_tag RBIER; /* Rx BDR 0 interrupt enable register */
  NETC_F3_SI4_BDR_NUM_RBIDR_tag RBIDR; /* Rx BDR 0 interrupt detect register */
  NETC_F3_SI4_BDR_NUM_RBICR0_tag RBICR0; /* Rx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI4_BDR_NUM_RBICR1_tag RBICR1; /* Rx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved6[80];
} NETC_F3_SI4_BDR_NUM_tag;

struct NETC_F3_SI4_tag {
  NETC_F3_SI4_SIMR_tag SIMR;           /* Station interface mode register */
  NETC_F3_SI4_SISR_tag SISR;           /* Station interface status register */
  uint8_t NETC_F3_SI4_reserved0[16];
  NETC_F3_SI4_SICTR0_tag SICTR0;       /* Station interface current time register 0 */
  NETC_F3_SI4_SICTR1_tag SICTR1;       /* Station interface current time register 1 */
  NETC_F3_SI4_SIPCAPR0_tag SIPCAPR0;   /* Station interface port capability register 0 */
  NETC_F3_SI4_SIPCAPR1_tag SIPCAPR1;   /* Station interface port capability register 1 */
  uint8_t NETC_F3_SI4_reserved1[16];
  NETC_F3_SI4_SIRBGCR_tag SIRBGCR;     /* Station interface receive BDR group control register */
  uint8_t NETC_F3_SI4_reserved2[4];
  NETC_F3_SI4_SIBCAR_tag SIBCAR;       /* Station interface buffer cache attribute register */
  uint8_t NETC_F3_SI4_reserved3[4];
  NETC_F3_SI4_SICCAR_tag SICCAR;       /* Station interface command cache attribute register */
  uint8_t NETC_F3_SI4_reserved4[52];
  NETC_F3_SI4_SIPMAR0_tag SIPMAR0;     /* Station interface primary MAC address register 0 */
  NETC_F3_SI4_SIPMAR1_tag SIPMAR1;     /* Station interface primary MAC address register 1 */
  uint8_t NETC_F3_SI4_reserved5[8];
  NETC_F3_SI4_SICVLANR1_tag SICVLANR1; /* Station interface custom VLAN register 1 */
  NETC_F3_SI4_SICVLANR2_tag SICVLANR2; /* Station interface custom VLAN register 2 */
  uint8_t NETC_F3_SI4_reserved6[104];
  NETC_F3_SI4_SIVLANIPVMR0_tag SIVLANIPVMR0; /* Station interface VLAN to IPV mapping register 0 */
  NETC_F3_SI4_SIVLANIPVMR1_tag SIVLANIPVMR1; /* Station interface VLAN to IPV mapping register 1 */
  uint8_t NETC_F3_SI4_reserved7[72];
  NETC_F3_SI4_SIIPVBDRMR0_tag SIIPVBDRMR0; /* Station interface IPV to ring mapping register */
  uint8_t NETC_F3_SI4_reserved8[176];
  NETC_F3_SI4_VSI_A_tag VSI_A;
  uint8_t NETC_F3_SI4_reserved9[232];
  NETC_F3_SI4_SIROCT0_tag SIROCT0;     /* Station interface receive octets counter (ifInOctets) 0 */
  NETC_F3_SI4_SIROCT1_tag SIROCT1;     /* Station interface receive octets counter (ifInOctets) 1 */
  NETC_F3_SI4_SIRFRM0_tag SIRFRM0;     /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  NETC_F3_SI4_SIRFRM1_tag SIRFRM1;     /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  NETC_F3_SI4_SIRUCA0_tag SIRUCA0;     /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  NETC_F3_SI4_SIRUCA1_tag SIRUCA1;     /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  NETC_F3_SI4_SIRMCA0_tag SIRMCA0;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  NETC_F3_SI4_SIRMCA1_tag SIRMCA1;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  NETC_F3_SI4_SITOCT0_tag SITOCT0;     /* Station interface transmit octets counter (ifOutOctets) 0 */
  NETC_F3_SI4_SITOCT1_tag SITOCT1;     /* Station interface transmit octets counter (ifOutOctets) 1 */
  NETC_F3_SI4_SITFRM0_tag SITFRM0;     /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  NETC_F3_SI4_SITFRM1_tag SITFRM1;     /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  NETC_F3_SI4_SITUCA0_tag SITUCA0;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  NETC_F3_SI4_SITUCA1_tag SITUCA1;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  NETC_F3_SI4_SITMCA0_tag SITMCA0;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  NETC_F3_SI4_SITMCA1_tag SITMCA1;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  uint8_t NETC_F3_SI4_reserved10[176];
  NETC_F3_SI4_SIBLPR_tag SIBLPR[2];    /* Station interface boot loader parameter register 0 */
  uint8_t NETC_F3_SI4_reserved11[1032];
  NETC_F3_SI4_SICBDRMR_tag SICBDRMR;   /* Station interface command BDR mode register */
  NETC_F3_SI4_SICBDRSR_tag SICBDRSR;   /* Station interface command BDR status register */
  uint8_t NETC_F3_SI4_reserved12[8];
  NETC_F3_SI4_SICBDRBAR0_tag SICBDRBAR0; /* Station interface command BDR base address register 0 */
  NETC_F3_SI4_SICBDRBAR1_tag SICBDRBAR1; /* Station interface command BDR base address register 1 */
  NETC_F3_SI4_SICBDRPIR_tag SICBDRPIR; /* Station interface command BDR producer index register */
  NETC_F3_SI4_SICBDRCIR_tag SICBDRCIR; /* Station interface command BDR consumer index register */
  NETC_F3_SI4_SICBDRLENR_tag SICBDRLENR; /* Station interface command BDR length register */
  uint8_t NETC_F3_SI4_reserved13[124];
  NETC_F3_SI4_SICBDRIER_tag SICBDRIER; /* Station interface command BDR interrupt enable register */
  NETC_F3_SI4_SICBDRIDR_tag SICBDRIDR; /* Station interface command BDR interrupt detect register */
  uint8_t NETC_F3_SI4_reserved14[88];
  NETC_F3_SI4_SICAPR0_tag SICAPR0;     /* Station interface capability register 0 */
  NETC_F3_SI4_SICAPR1_tag SICAPR1;     /* Station interface capability register 1 */
  NETC_F3_SI4_SICAPR2_tag SICAPR2;     /* Station interface capability register 2 */
  uint8_t NETC_F3_SI4_reserved15[244];
  NETC_F3_SI4_VSI_tag VSI;
  uint8_t NETC_F3_SI4_reserved16[12];
  NETC_F3_SI4_SITXIDR0_tag SITXIDR0;   /* Station interface transmit interrupt detect register 0 */
  NETC_F3_SI4_SITXIDR1_tag SITXIDR1;   /* Station interface transmit interrupt detect register 1 */
  uint8_t NETC_F3_SI4_reserved17[8];
  NETC_F3_SI4_SIRXIDR0_tag SIRXIDR0;   /* Station interface receive interrupt detect register 0 */
  NETC_F3_SI4_SIRXIDR1_tag SIRXIDR1;   /* Station interface receive interrupt detect register 1 */
  NETC_F3_SI4_SIMSIVR_tag SIMSIVR;     /* Station interface MSI-X vector register */
  NETC_F3_SI4_SICMSIVR_tag SICMSIVR;   /* Station interface command MSI-X vector register */
  uint8_t NETC_F3_SI4_reserved18[8];
  NETC_F3_SI4_SITMRIER_tag SITMRIER;   /* Station interface timer interrupt enable register */
  NETC_F3_SI4_SITMRIDR_tag SITMRIDR;   /* Station interface timer interrupt detect register */
  uint8_t NETC_F3_SI4_reserved19[4];
  NETC_F3_SI4_SITMRMSIVR_tag SITMRMSIVR; /* Station interface timer MSI-X vector register */
  uint8_t NETC_F3_SI4_reserved20[176];
  NETC_F3_SI4_SIMSITRVR_tag SIMSITRVR[18]; /* Station interface MSI-X transmit ring 0 vector register */
  uint8_t NETC_F3_SI4_reserved21[56];
  NETC_F3_SI4_SIMSIRRVR_tag SIMSIRRVR[18]; /* Station interface MSI-X receive ring 0 vector register */
  uint8_t NETC_F3_SI4_reserved22[568];
  NETC_F3_SI4_SICMECR_tag SICMECR;     /* Station interface correctable memory error configuration register */
  NETC_F3_SI4_SICMESR_tag SICMESR;     /* Station interface correctable memory error status register */
  uint8_t NETC_F3_SI4_reserved23[4];
  NETC_F3_SI4_SICMECTR_tag SICMECTR;   /* Station interface correctable memory error count register */
  NETC_F3_SI4_SIUPECR_tag SIUPECR;     /* Station interface uncorrectable programming error configuration register */
  NETC_F3_SI4_SIUPESR_tag SIUPESR;     /* Station interface uncorrectable programming error status register */
  uint8_t NETC_F3_SI4_reserved24[4];
  NETC_F3_SI4_SIUPECTR_tag SIUPECTR;   /* Station interface uncorrectable programming error count register */
  NETC_F3_SI4_SIUNSBECR_tag SIUNSBECR; /* Station interface uncorrectable non-fatal system bus error configuration register */
  NETC_F3_SI4_SIUNSBESR_tag SIUNSBESR; /* Station interface uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F3_SI4_reserved25[4];
  NETC_F3_SI4_SIUNSBECTR_tag SIUNSBECTR; /* Station interface uncorrectable non-fatal system bus error count register */
  NETC_F3_SI4_SIUFSBECR_tag SIUFSBECR; /* Station interface uncorrectable fatal system bus error configuration register */
  NETC_F3_SI4_SIUFSBESR_tag SIUFSBESR; /* Station interface uncorrectable fatal system bus error status register */
  uint8_t NETC_F3_SI4_reserved26[8];
  NETC_F3_SI4_SIUNMECR_tag SIUNMECR;   /* Station interface uncorrectable non-fatal memory error configuration register */
  NETC_F3_SI4_SIUNMESR0_tag SIUNMESR0; /* Station interface uncorrectable non-fatal memory error status register 0 */
  NETC_F3_SI4_SIUNMESR1_tag SIUNMESR1; /* Station interface uncorrectable non-fatal memory error status register 1 */
  NETC_F3_SI4_SIUNMECTR_tag SIUNMECTR; /* Station interface uncorrectable non-fatal memory error count register */
  NETC_F3_SI4_SIUFMECR_tag SIUFMECR;   /* Station interface uncorrectable fatal memory error configuration register */
  NETC_F3_SI4_SIUFMESR0_tag SIUFMESR0; /* Station interface uncorrectable fatal memory error status register 0 */
  NETC_F3_SI4_SIUFMESR1_tag SIUFMESR1; /* Station interface uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_SI4_reserved27[4];
  NETC_F3_SI4_SIUNIECR_tag SIUNIECR;   /* Station interface uncorrectable non-fatal integrity error configuration register */
  NETC_F3_SI4_SIUNIESR_tag SIUNIESR;   /* Station interface uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_SI4_reserved28[4];
  NETC_F3_SI4_SIUNIECTR_tag SIUNIECTR; /* Station interface uncorrectable non-fatal integrity error count register */
  NETC_F3_SI4_SIUFIECR_tag SIUFIECR;   /* Station interface uncorrectable fatal integrity error configuration register */
  NETC_F3_SI4_SIUFIESR_tag SIUFIESR;   /* Station interface uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_SI4_reserved29[392];
  NETC_F3_SI4_SIMAFTCAPR_tag SIMAFTCAPR; /* Station interface MAC address filter table capability register */
  uint8_t NETC_F3_SI4_reserved30[252];
  NETC_F3_SI4_SIVFTCAPR_tag SIVFTCAPR; /* Station interface VLAN filter table capability register */
  uint8_t NETC_F3_SI4_reserved31[252];
  NETC_F3_SI4_SIRFSCAPR_tag SIRFSCAPR; /* Station interface RFS capability register */
  uint8_t NETC_F3_SI4_reserved32[28156];
  NETC_F3_SI4_BDR_NUM_tag BDR_NUM[18];
};


/* ============================================================================
   =============================== Module: NETC_F3_SI5 ========================
   ============================================================================ */

typedef union NETC_F3_SI5_SIMR_union_tag { /* Station interface mode register */
  vuint32_t R;
  struct {
    vuint32_t RSSE:1;
    vuint32_t RNUM:1;
    vuint32_t RNMM:1;
    vuint32_t RNBM:1;
    vuint32_t V2IPVE:1;
    vuint32_t _unused_5:11;
    vuint32_t DEFAULT_RX_GROUP:1;
    vuint32_t _unused_17:14;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI5_SIMR_tag;

typedef union NETC_F3_SI5_SISR_union_tag { /* Station interface status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;
    vuint32_t MAC_UP:1;
    vuint32_t MAC_MP:1;
    vuint32_t VLAN_P:1;
    vuint32_t VLAN_UTA:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_SI5_SISR_tag;

typedef union NETC_F3_SI5_SICTR0_union_tag { /* Station interface current time register 0 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI5_SICTR0_tag;

typedef union NETC_F3_SI5_SICTR1_union_tag { /* Station interface current time register 1 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI5_SICTR1_tag;

typedef union NETC_F3_SI5_SIPCAPR0_union_tag { /* Station interface port capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;                   /* Receive Flow Steering */
    vuint32_t FP:1;                    /* Frame Preemption */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t TSD:1;                   /* Time Specific Departure */
    vuint32_t CBS:1;                   /* Credit Based Shaping (CBS) */
    vuint32_t _unused_7:1;
    vuint32_t RSS:1;                   /* Receive Side Scaling */
    vuint32_t PSFP:1;                  /* Per-Stream Filtering and Policing (IEEE 802.1Qci) */
    vuint32_t IPFLT:1;                 /* Ingress Port Filtering */
    vuint32_t RP:1;                    /* Rate Policing */
    vuint32_t _unused_12:1;
    vuint32_t WO:1;                    /* Wake-on-LAN */
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional Safety */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI5_SIPCAPR0_tag;

typedef union NETC_F3_SI5_SIPCAPR1_union_tag { /* Station interface port capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:6;
    vuint32_t _unused_18:13;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_SI5_SIPCAPR1_tag;

typedef union NETC_F3_SI5_SIRBGCR_union_tag { /* Station interface receive BDR group control register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GROUPS:2;
    vuint32_t _unused_2:14;
    vuint32_t RINGS_PER_GROUP:3;
    vuint32_t _unused_19:13;
  } B;
} NETC_F3_SI5_SIRBGCR_tag;

typedef union NETC_F3_SI5_SIBCAR_union_tag { /* Station interface buffer cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI5_SIBCAR_tag;

typedef union NETC_F3_SI5_SICCAR_union_tag { /* Station interface command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI5_SICCAR_tag;

typedef union NETC_F3_SI5_SIPMAR0_union_tag { /* Station interface primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_SI5_SIPMAR0_tag;

typedef union NETC_F3_SI5_SIPMAR1_union_tag { /* Station interface primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI5_SIPMAR1_tag;

typedef union NETC_F3_SI5_SICVLANR1_union_tag { /* Station interface custom VLAN register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI5_SICVLANR1_tag;

typedef union NETC_F3_SI5_SICVLANR2_union_tag { /* Station interface custom VLAN register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI5_SICVLANR2_tag;

typedef union NETC_F3_SI5_SIVLANIPVMR0_union_tag { /* Station interface VLAN to IPV mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:4;
    vuint32_t PCP_DEI_1:4;
    vuint32_t PCP_DEI_2:4;
    vuint32_t PCP_DEI_3:4;
    vuint32_t PCP_DEI_4:4;
    vuint32_t PCP_DEI_5:4;
    vuint32_t PCP_DEI_6:4;
    vuint32_t PCP_DEI_7:4;
  } B;
} NETC_F3_SI5_SIVLANIPVMR0_tag;

typedef union NETC_F3_SI5_SIVLANIPVMR1_union_tag { /* Station interface VLAN to IPV mapping register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:4;
    vuint32_t PCP_DEI_9:4;
    vuint32_t PCP_DEI_10:4;
    vuint32_t PCP_DEI_11:4;
    vuint32_t PCP_DEI_12:4;
    vuint32_t PCP_DEI_13:4;
    vuint32_t PCP_DEI_14:4;
    vuint32_t PCP_DEI_15:4;
  } B;
} NETC_F3_SI5_SIVLANIPVMR1_tag;

typedef union NETC_F3_SI5_SIIPVBDRMR0_union_tag { /* Station interface IPV to ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPV0BDR:3;
    vuint32_t _unused_3:1;
    vuint32_t IPV1BDR:3;
    vuint32_t _unused_7:1;
    vuint32_t IPV2BDR:3;
    vuint32_t _unused_11:1;
    vuint32_t IPV3BDR:3;
    vuint32_t _unused_15:1;
    vuint32_t IPV4BDR:3;
    vuint32_t _unused_19:1;
    vuint32_t IPV5BDR:3;
    vuint32_t _unused_23:1;
    vuint32_t IPV6BDR:3;
    vuint32_t _unused_27:1;
    vuint32_t IPV7BDR:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI5_SIIPVBDRMR0_tag;

typedef union NETC_F3_SI5_VSI_A_VSIMSGSR_union_tag { /* Virtual station interface message send register */
  vuint32_t R;
  struct {
    vuint32_t MB:1;
    vuint32_t MS:1;
    vuint32_t _unused_2:14;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI5_VSI_A_VSIMSGSR_tag;

typedef union NETC_F3_SI5_VSI_A_VSIMSGRR_union_tag { /* Virtual station interface message receive register */
  vuint32_t R;
  struct {
    vuint32_t MR:1;
    vuint32_t _unused_1:15;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI5_VSI_A_VSIMSGRR_tag;

typedef union NETC_F3_SI5_VSI_A_VSIMSGSNDAR0_union_tag { /* Virtual station interface message send register 0 */
  vuint32_t R;
  struct {
    vuint32_t MSIZE:5;
    vuint32_t _unused_5:1;
    vuint32_t ADDRL:26;
  } B;
} NETC_F3_SI5_VSI_A_VSIMSGSNDAR0_tag;

typedef union NETC_F3_SI5_VSI_A_VSIMSGSNDAR1_union_tag { /* Virtual station interface message send address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI5_VSI_A_VSIMSGSNDAR1_tag;

typedef struct NETC_F3_SI5_VSI_A_struct_tag {
  NETC_F3_SI5_VSI_A_VSIMSGSR_tag VSIMSGSR; /* Virtual station interface message send register */
  NETC_F3_SI5_VSI_A_VSIMSGRR_tag VSIMSGRR; /* Virtual station interface message receive register */
  uint8_t VSI_A_reserved0[4];
  NETC_F3_SI5_VSI_A_VSIMSGSNDAR0_tag VSIMSGSNDAR0; /* Virtual station interface message send register 0 */
  NETC_F3_SI5_VSI_A_VSIMSGSNDAR1_tag VSIMSGSNDAR1; /* Virtual station interface message send address register 1 */
} NETC_F3_SI5_VSI_A_tag;

typedef union NETC_F3_SI5_SIROCT0_union_tag { /* Station interface receive octets counter (ifInOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_LOW:32;
  } B;
} NETC_F3_SI5_SIROCT0_tag;

typedef union NETC_F3_SI5_SIROCT1_union_tag { /* Station interface receive octets counter (ifInOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_HIGH:32;
  } B;
} NETC_F3_SI5_SIROCT1_tag;

typedef union NETC_F3_SI5_SIRFRM0_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_LOW:32;
  } B;
} NETC_F3_SI5_SIRFRM0_tag;

typedef union NETC_F3_SI5_SIRFRM1_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_HIGH:32;
  } B;
} NETC_F3_SI5_SIRFRM1_tag;

typedef union NETC_F3_SI5_SIRUCA0_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_LOW:32;
  } B;
} NETC_F3_SI5_SIRUCA0_tag;

typedef union NETC_F3_SI5_SIRUCA1_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_HIGH:32;
  } B;
} NETC_F3_SI5_SIRUCA1_tag;

typedef union NETC_F3_SI5_SIRMCA0_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_LOW:32;
  } B;
} NETC_F3_SI5_SIRMCA0_tag;

typedef union NETC_F3_SI5_SIRMCA1_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_HIGH:32;
  } B;
} NETC_F3_SI5_SIRMCA1_tag;

typedef union NETC_F3_SI5_SITOCT0_union_tag { /* Station interface transmit octets counter (ifOutOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_LOW:32;
  } B;
} NETC_F3_SI5_SITOCT0_tag;

typedef union NETC_F3_SI5_SITOCT1_union_tag { /* Station interface transmit octets counter (ifOutOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_HIGH:32;
  } B;
} NETC_F3_SI5_SITOCT1_tag;

typedef union NETC_F3_SI5_SITFRM0_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_LOW:32;
  } B;
} NETC_F3_SI5_SITFRM0_tag;

typedef union NETC_F3_SI5_SITFRM1_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_HIGH:32;
  } B;
} NETC_F3_SI5_SITFRM1_tag;

typedef union NETC_F3_SI5_SITUCA0_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_LOW:32;
  } B;
} NETC_F3_SI5_SITUCA0_tag;

typedef union NETC_F3_SI5_SITUCA1_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_HIGH:32;
  } B;
} NETC_F3_SI5_SITUCA1_tag;

typedef union NETC_F3_SI5_SITMCA0_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_LOW:32;
  } B;
} NETC_F3_SI5_SITMCA0_tag;

typedef union NETC_F3_SI5_SITMCA1_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_HIGH:32;
  } B;
} NETC_F3_SI5_SITMCA1_tag;

typedef union NETC_F3_SI5_SIBLPR_union_tag { /* Station interface boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F3_SI5_SIBLPR_tag;

typedef union NETC_F3_SI5_SICBDRMR_union_tag { /* Station interface command BDR mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI5_SICBDRMR_tag;

typedef union NETC_F3_SI5_SICBDRSR_union_tag { /* Station interface command BDR status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI5_SICBDRSR_tag;

typedef union NETC_F3_SI5_SICBDRBAR0_union_tag { /* Station interface command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI5_SICBDRBAR0_tag;

typedef union NETC_F3_SI5_SICBDRBAR1_union_tag { /* Station interface command BDR base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI5_SICBDRBAR1_tag;

typedef union NETC_F3_SI5_SICBDRPIR_union_tag { /* Station interface command BDR producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI5_SICBDRPIR_tag;

typedef union NETC_F3_SI5_SICBDRCIR_union_tag { /* Station interface command BDR consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI5_SICBDRCIR_tag;

typedef union NETC_F3_SI5_SICBDRLENR_union_tag { /* Station interface command BDR length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_SI5_SICBDRLENR_tag;

typedef union NETC_F3_SI5_SICBDRIER_union_tag { /* Station interface command BDR interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI5_SICBDRIER_tag;

typedef union NETC_F3_SI5_SICBDRIDR_union_tag { /* Station interface command BDR interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI5_SICBDRIDR_tag;

typedef union NETC_F3_SI5_SICAPR0_union_tag { /* Station interface capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_RX_BDR:8;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F3_SI5_SICAPR0_tag;

typedef union NETC_F3_SI5_SICAPR1_union_tag { /* Station interface capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t NUM_RX_GRP:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI5_SICAPR1_tag;

typedef union NETC_F3_SI5_SICAPR2_union_tag { /* Station interface capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t VTP:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_SI5_SICAPR2_tag;

typedef union NETC_F3_SI5_VSI_VSIIER_union_tag { /* Virtual station interface interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MSIE:1;
    vuint32_t MRIE:1;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI5_VSI_VSIIER_tag;

typedef union NETC_F3_SI5_VSI_VSIIDR_union_tag { /* Virtual station interface interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXR:1;
    vuint32_t _unused_1:7;
    vuint32_t MS:1;
    vuint32_t MR:1;
    vuint32_t _unused_10:6;
    vuint32_t RXR:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI5_VSI_VSIIDR_tag;

typedef struct NETC_F3_SI5_VSI_struct_tag {
  NETC_F3_SI5_VSI_VSIIER_tag VSIIER;   /* Virtual station interface interrupt enable register */
  uint8_t VSI_reserved0[4];
  NETC_F3_SI5_VSI_VSIIDR_tag VSIIDR;   /* Virtual station interface interrupt detect register */
} NETC_F3_SI5_VSI_tag;

typedef union NETC_F3_SI5_SITXIDR0_union_tag { /* Station interface transmit interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXT0:1;
    vuint32_t TXT1:1;
    vuint32_t TXT2:1;
    vuint32_t TXT3:1;
    vuint32_t TXT4:1;
    vuint32_t TXT5:1;
    vuint32_t TXT6:1;
    vuint32_t TXT7:1;
    vuint32_t TXT8:1;
    vuint32_t TXT9:1;
    vuint32_t TXT10:1;
    vuint32_t TXT11:1;
    vuint32_t TXT12:1;
    vuint32_t TXT13:1;
    vuint32_t TXT14:1;
    vuint32_t TXT15:1;
    vuint32_t TXF0:1;
    vuint32_t TXF1:1;
    vuint32_t TXF2:1;
    vuint32_t TXF3:1;
    vuint32_t TXF4:1;
    vuint32_t TXF5:1;
    vuint32_t TXF6:1;
    vuint32_t TXF7:1;
    vuint32_t TXF8:1;
    vuint32_t TXF9:1;
    vuint32_t TXF10:1;
    vuint32_t TXF11:1;
    vuint32_t TXF12:1;
    vuint32_t TXF13:1;
    vuint32_t TXF14:1;
    vuint32_t TXF15:1;
  } B;
} NETC_F3_SI5_SITXIDR0_tag;

typedef union NETC_F3_SI5_SITXIDR1_union_tag { /* Station interface transmit interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t TXT16:1;
    vuint32_t TXT17:1;
    vuint32_t _unused_2:14;
    vuint32_t TXF16:1;
    vuint32_t TXF17:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_SI5_SITXIDR1_tag;

typedef union NETC_F3_SI5_SIRXIDR0_union_tag { /* Station interface receive interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t RX0:1;
    vuint32_t RX1:1;
    vuint32_t RX2:1;
    vuint32_t RX3:1;
    vuint32_t RX4:1;
    vuint32_t RX5:1;
    vuint32_t RX6:1;
    vuint32_t RX7:1;
    vuint32_t RX8:1;
    vuint32_t RX9:1;
    vuint32_t RX10:1;
    vuint32_t RX11:1;
    vuint32_t RX12:1;
    vuint32_t RX13:1;
    vuint32_t RX14:1;
    vuint32_t RX15:1;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI5_SIRXIDR0_tag;

typedef union NETC_F3_SI5_SIRXIDR1_union_tag { /* Station interface receive interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t RX16:1;
    vuint32_t RX17:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI5_SIRXIDR1_tag;

typedef union NETC_F3_SI5_SIMSIVR_union_tag { /* Station interface MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI5_SIMSIVR_tag;

typedef union NETC_F3_SI5_SICMSIVR_union_tag { /* Station interface command MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI5_SICMSIVR_tag;

typedef union NETC_F3_SI5_SITMRIER_union_tag { /* Station interface timer interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t SYNCE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI5_SITMRIER_tag;

typedef union NETC_F3_SI5_SITMRIDR_union_tag { /* Station interface timer interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI5_SITMRIDR_tag;

typedef union NETC_F3_SI5_SITMRMSIVR_union_tag { /* Station interface timer MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI5_SITMRMSIVR_tag;

typedef union NETC_F3_SI5_SIMSITRVR_union_tag { /* Station interface MSI-X transmit ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI5_SIMSITRVR_tag;

typedef union NETC_F3_SI5_SIMSIRRVR_union_tag { /* Station interface MSI-X receive ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI5_SIMSIRRVR_tag;

typedef union NETC_F3_SI5_SICMECR_union_tag { /* Station interface correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI5_SICMECR_tag;

typedef union NETC_F3_SI5_SICMESR_union_tag { /* Station interface correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_SI5_SICMESR_tag;

typedef union NETC_F3_SI5_SICMECTR_union_tag { /* Station interface correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI5_SICMECTR_tag;

typedef union NETC_F3_SI5_SIUPECR_union_tag { /* Station interface uncorrectable programming error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI5_SIUPECR_tag;

typedef union NETC_F3_SI5_SIUPESR_union_tag { /* Station interface uncorrectable programming error status register */
  vuint32_t R;
  struct {
    vuint32_t DROP_SI_EN:1;
    vuint32_t DROP_RING_EN:1;
    vuint32_t DROP_GRP_SEL:1;
    vuint32_t DROP_RING_SEL:1;
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t PE:1;                    /* Programming error */
  } B;
} NETC_F3_SI5_SIUPESR_tag;

typedef union NETC_F3_SI5_SIUPECTR_union_tag { /* Station interface uncorrectable programming error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;                /* Count */
  } B;
} NETC_F3_SI5_SIUPECTR_tag;

typedef union NETC_F3_SI5_SIUNSBECR_union_tag { /* Station interface uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI5_SIUNSBECR_tag;

typedef union NETC_F3_SI5_SIUNSBESR_union_tag { /* Station interface uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI5_SIUNSBESR_tag;

typedef union NETC_F3_SI5_SIUNSBECTR_union_tag { /* Station interface uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI5_SIUNSBECTR_tag;

typedef union NETC_F3_SI5_SIUFSBECR_union_tag { /* Station interface uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI5_SIUFSBECR_tag;

typedef union NETC_F3_SI5_SIUFSBESR_union_tag { /* Station interface uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI5_SIUFSBESR_tag;

typedef union NETC_F3_SI5_SIUNMECR_union_tag { /* Station interface uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI5_SIUNMECR_tag;

typedef union NETC_F3_SI5_SIUNMESR0_union_tag { /* Station interface uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI5_SIUNMESR0_tag;

typedef union NETC_F3_SI5_SIUNMESR1_union_tag { /* Station interface uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI5_SIUNMESR1_tag;

typedef union NETC_F3_SI5_SIUNMECTR_union_tag { /* Station interface uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI5_SIUNMECTR_tag;

typedef union NETC_F3_SI5_SIUFMECR_union_tag { /* Station interface uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI5_SIUFMECR_tag;

typedef union NETC_F3_SI5_SIUFMESR0_union_tag { /* Station interface uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI5_SIUFMESR0_tag;

typedef union NETC_F3_SI5_SIUFMESR1_union_tag { /* Station interface uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI5_SIUFMESR1_tag;

typedef union NETC_F3_SI5_SIUNIECR_union_tag { /* Station interface uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI5_SIUNIECR_tag;

typedef union NETC_F3_SI5_SIUNIESR_union_tag { /* Station interface uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI5_SIUNIESR_tag;

typedef union NETC_F3_SI5_SIUNIECTR_union_tag { /* Station interface uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI5_SIUNIECTR_tag;

typedef union NETC_F3_SI5_SIUFIECR_union_tag { /* Station interface uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI5_SIUFIECR_tag;

typedef union NETC_F3_SI5_SIUFIESR_union_tag { /* Station interface uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:16;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI5_SIUFIESR_tag;

typedef union NETC_F3_SI5_SIMAFTCAPR_union_tag { /* Station interface MAC address filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI5_SIMAFTCAPR_tag;

typedef union NETC_F3_SI5_SIVFTCAPR_union_tag { /* Station interface VLAN filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI5_SIVFTCAPR_tag;

typedef union NETC_F3_SI5_SIRFSCAPR_union_tag { /* Station interface RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_SI5_SIRFSCAPR_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBMR_union_tag { /* Tx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t PRIO:3;
    vuint32_t _unused_3:1;
    vuint32_t WRR:3;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t VIH:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI5_BDR_NUM_TBMR_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBSR_union_tag { /* Tx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI5_BDR_NUM_TBSR_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBBAR0_union_tag { /* Tx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI5_BDR_NUM_TBBAR0_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBBAR1_union_tag { /* Tx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI5_BDR_NUM_TBBAR1_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBPIR_union_tag { /* Tx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI5_BDR_NUM_TBPIR_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBCIR_union_tag { /* Tx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t STAT_ID:16;
  } B;
} NETC_F3_SI5_BDR_NUM_TBCIR_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBLENR_union_tag { /* Tx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI5_BDR_NUM_TBLENR_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBIER_union_tag { /* Tx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t TXTIE:1;
    vuint32_t TXFIE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI5_BDR_NUM_TBIER_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBIDR_union_tag { /* Tx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXT:1;
    vuint32_t TXF:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI5_BDR_NUM_TBIDR_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBICR0_union_tag { /* Tx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:4;
    vuint32_t _unused_4:27;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI5_BDR_NUM_TBICR0_tag;

typedef union NETC_F3_SI5_BDR_NUM_TBICR1_union_tag { /* Tx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI5_BDR_NUM_TBICR1_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBMR_union_tag { /* Rx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t AL:1;
    vuint32_t _unused_1:1;
    vuint32_t BDS:1;
    vuint32_t _unused_3:1;
    vuint32_t CM:1;
    vuint32_t VTE:1;
    vuint32_t VTPD:1;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI5_BDR_NUM_RBMR_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBSR_union_tag { /* Rx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI5_BDR_NUM_RBSR_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBBSR_union_tag { /* Rx BDR 0 buffer size register */
  vuint32_t R;
  struct {
    vuint32_t BSIZE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI5_BDR_NUM_RBBSR_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBCIR_union_tag { /* Rx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI5_BDR_NUM_RBCIR_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBBAR0_union_tag { /* Rx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI5_BDR_NUM_RBBAR0_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBBAR1_union_tag { /* Rx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI5_BDR_NUM_RBBAR1_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBPIR_union_tag { /* Rx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI5_BDR_NUM_RBPIR_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBLENR_union_tag { /* Rx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI5_BDR_NUM_RBLENR_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBDCR_union_tag { /* Rx BDR 0 drop count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_SI5_BDR_NUM_RBDCR_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBIER_union_tag { /* Rx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t RXTIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI5_BDR_NUM_RBIER_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBIDR_union_tag { /* Rx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t RXT:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI5_BDR_NUM_RBIDR_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBICR0_union_tag { /* Rx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:9;
    vuint32_t _unused_9:22;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI5_BDR_NUM_RBICR0_tag;

typedef union NETC_F3_SI5_BDR_NUM_RBICR1_union_tag { /* Rx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI5_BDR_NUM_RBICR1_tag;

typedef struct NETC_F3_SI5_BDR_NUM_struct_tag {
  NETC_F3_SI5_BDR_NUM_TBMR_tag TBMR;   /* Tx BDR 0 mode register */
  NETC_F3_SI5_BDR_NUM_TBSR_tag TBSR;   /* Tx BDR 0 status register */
  uint8_t BDR_NUM_reserved0[8];
  NETC_F3_SI5_BDR_NUM_TBBAR0_tag TBBAR0; /* Tx BDR 0 base address register 0 */
  NETC_F3_SI5_BDR_NUM_TBBAR1_tag TBBAR1; /* Tx BDR 0 base address register 1 */
  NETC_F3_SI5_BDR_NUM_TBPIR_tag TBPIR; /* Tx BDR 0 producer index register */
  NETC_F3_SI5_BDR_NUM_TBCIR_tag TBCIR; /* Tx BDR 0 consumer index register */
  NETC_F3_SI5_BDR_NUM_TBLENR_tag TBLENR; /* Tx BDR 0 length register */
  uint8_t BDR_NUM_reserved1[124];
  NETC_F3_SI5_BDR_NUM_TBIER_tag TBIER; /* Tx BDR 0 interrupt enable register */
  NETC_F3_SI5_BDR_NUM_TBIDR_tag TBIDR; /* Tx BDR 0 interrupt detect register */
  NETC_F3_SI5_BDR_NUM_TBICR0_tag TBICR0; /* Tx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI5_BDR_NUM_TBICR1_tag TBICR1; /* Tx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved2[80];
  NETC_F3_SI5_BDR_NUM_RBMR_tag RBMR;   /* Rx BDR 0 mode register */
  NETC_F3_SI5_BDR_NUM_RBSR_tag RBSR;   /* Rx BDR 0 status register */
  NETC_F3_SI5_BDR_NUM_RBBSR_tag RBBSR; /* Rx BDR 0 buffer size register */
  NETC_F3_SI5_BDR_NUM_RBCIR_tag RBCIR; /* Rx BDR 0 consumer index register */
  NETC_F3_SI5_BDR_NUM_RBBAR0_tag RBBAR0; /* Rx BDR 0 base address register 0 */
  NETC_F3_SI5_BDR_NUM_RBBAR1_tag RBBAR1; /* Rx BDR 0 base address register 1 */
  NETC_F3_SI5_BDR_NUM_RBPIR_tag RBPIR; /* Rx BDR 0 producer index register */
  uint8_t BDR_NUM_reserved3[4];
  NETC_F3_SI5_BDR_NUM_RBLENR_tag RBLENR; /* Rx BDR 0 length register */
  uint8_t BDR_NUM_reserved4[92];
  NETC_F3_SI5_BDR_NUM_RBDCR_tag RBDCR; /* Rx BDR 0 drop count register */
  uint8_t BDR_NUM_reserved5[28];
  NETC_F3_SI5_BDR_NUM_RBIER_tag RBIER; /* Rx BDR 0 interrupt enable register */
  NETC_F3_SI5_BDR_NUM_RBIDR_tag RBIDR; /* Rx BDR 0 interrupt detect register */
  NETC_F3_SI5_BDR_NUM_RBICR0_tag RBICR0; /* Rx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI5_BDR_NUM_RBICR1_tag RBICR1; /* Rx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved6[80];
} NETC_F3_SI5_BDR_NUM_tag;

struct NETC_F3_SI5_tag {
  NETC_F3_SI5_SIMR_tag SIMR;           /* Station interface mode register */
  NETC_F3_SI5_SISR_tag SISR;           /* Station interface status register */
  uint8_t NETC_F3_SI5_reserved0[16];
  NETC_F3_SI5_SICTR0_tag SICTR0;       /* Station interface current time register 0 */
  NETC_F3_SI5_SICTR1_tag SICTR1;       /* Station interface current time register 1 */
  NETC_F3_SI5_SIPCAPR0_tag SIPCAPR0;   /* Station interface port capability register 0 */
  NETC_F3_SI5_SIPCAPR1_tag SIPCAPR1;   /* Station interface port capability register 1 */
  uint8_t NETC_F3_SI5_reserved1[16];
  NETC_F3_SI5_SIRBGCR_tag SIRBGCR;     /* Station interface receive BDR group control register */
  uint8_t NETC_F3_SI5_reserved2[4];
  NETC_F3_SI5_SIBCAR_tag SIBCAR;       /* Station interface buffer cache attribute register */
  uint8_t NETC_F3_SI5_reserved3[4];
  NETC_F3_SI5_SICCAR_tag SICCAR;       /* Station interface command cache attribute register */
  uint8_t NETC_F3_SI5_reserved4[52];
  NETC_F3_SI5_SIPMAR0_tag SIPMAR0;     /* Station interface primary MAC address register 0 */
  NETC_F3_SI5_SIPMAR1_tag SIPMAR1;     /* Station interface primary MAC address register 1 */
  uint8_t NETC_F3_SI5_reserved5[8];
  NETC_F3_SI5_SICVLANR1_tag SICVLANR1; /* Station interface custom VLAN register 1 */
  NETC_F3_SI5_SICVLANR2_tag SICVLANR2; /* Station interface custom VLAN register 2 */
  uint8_t NETC_F3_SI5_reserved6[104];
  NETC_F3_SI5_SIVLANIPVMR0_tag SIVLANIPVMR0; /* Station interface VLAN to IPV mapping register 0 */
  NETC_F3_SI5_SIVLANIPVMR1_tag SIVLANIPVMR1; /* Station interface VLAN to IPV mapping register 1 */
  uint8_t NETC_F3_SI5_reserved7[72];
  NETC_F3_SI5_SIIPVBDRMR0_tag SIIPVBDRMR0; /* Station interface IPV to ring mapping register */
  uint8_t NETC_F3_SI5_reserved8[176];
  NETC_F3_SI5_VSI_A_tag VSI_A;
  uint8_t NETC_F3_SI5_reserved9[232];
  NETC_F3_SI5_SIROCT0_tag SIROCT0;     /* Station interface receive octets counter (ifInOctets) 0 */
  NETC_F3_SI5_SIROCT1_tag SIROCT1;     /* Station interface receive octets counter (ifInOctets) 1 */
  NETC_F3_SI5_SIRFRM0_tag SIRFRM0;     /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  NETC_F3_SI5_SIRFRM1_tag SIRFRM1;     /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  NETC_F3_SI5_SIRUCA0_tag SIRUCA0;     /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  NETC_F3_SI5_SIRUCA1_tag SIRUCA1;     /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  NETC_F3_SI5_SIRMCA0_tag SIRMCA0;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  NETC_F3_SI5_SIRMCA1_tag SIRMCA1;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  NETC_F3_SI5_SITOCT0_tag SITOCT0;     /* Station interface transmit octets counter (ifOutOctets) 0 */
  NETC_F3_SI5_SITOCT1_tag SITOCT1;     /* Station interface transmit octets counter (ifOutOctets) 1 */
  NETC_F3_SI5_SITFRM0_tag SITFRM0;     /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  NETC_F3_SI5_SITFRM1_tag SITFRM1;     /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  NETC_F3_SI5_SITUCA0_tag SITUCA0;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  NETC_F3_SI5_SITUCA1_tag SITUCA1;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  NETC_F3_SI5_SITMCA0_tag SITMCA0;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  NETC_F3_SI5_SITMCA1_tag SITMCA1;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  uint8_t NETC_F3_SI5_reserved10[176];
  NETC_F3_SI5_SIBLPR_tag SIBLPR[2];    /* Station interface boot loader parameter register 0 */
  uint8_t NETC_F3_SI5_reserved11[1032];
  NETC_F3_SI5_SICBDRMR_tag SICBDRMR;   /* Station interface command BDR mode register */
  NETC_F3_SI5_SICBDRSR_tag SICBDRSR;   /* Station interface command BDR status register */
  uint8_t NETC_F3_SI5_reserved12[8];
  NETC_F3_SI5_SICBDRBAR0_tag SICBDRBAR0; /* Station interface command BDR base address register 0 */
  NETC_F3_SI5_SICBDRBAR1_tag SICBDRBAR1; /* Station interface command BDR base address register 1 */
  NETC_F3_SI5_SICBDRPIR_tag SICBDRPIR; /* Station interface command BDR producer index register */
  NETC_F3_SI5_SICBDRCIR_tag SICBDRCIR; /* Station interface command BDR consumer index register */
  NETC_F3_SI5_SICBDRLENR_tag SICBDRLENR; /* Station interface command BDR length register */
  uint8_t NETC_F3_SI5_reserved13[124];
  NETC_F3_SI5_SICBDRIER_tag SICBDRIER; /* Station interface command BDR interrupt enable register */
  NETC_F3_SI5_SICBDRIDR_tag SICBDRIDR; /* Station interface command BDR interrupt detect register */
  uint8_t NETC_F3_SI5_reserved14[88];
  NETC_F3_SI5_SICAPR0_tag SICAPR0;     /* Station interface capability register 0 */
  NETC_F3_SI5_SICAPR1_tag SICAPR1;     /* Station interface capability register 1 */
  NETC_F3_SI5_SICAPR2_tag SICAPR2;     /* Station interface capability register 2 */
  uint8_t NETC_F3_SI5_reserved15[244];
  NETC_F3_SI5_VSI_tag VSI;
  uint8_t NETC_F3_SI5_reserved16[12];
  NETC_F3_SI5_SITXIDR0_tag SITXIDR0;   /* Station interface transmit interrupt detect register 0 */
  NETC_F3_SI5_SITXIDR1_tag SITXIDR1;   /* Station interface transmit interrupt detect register 1 */
  uint8_t NETC_F3_SI5_reserved17[8];
  NETC_F3_SI5_SIRXIDR0_tag SIRXIDR0;   /* Station interface receive interrupt detect register 0 */
  NETC_F3_SI5_SIRXIDR1_tag SIRXIDR1;   /* Station interface receive interrupt detect register 1 */
  NETC_F3_SI5_SIMSIVR_tag SIMSIVR;     /* Station interface MSI-X vector register */
  NETC_F3_SI5_SICMSIVR_tag SICMSIVR;   /* Station interface command MSI-X vector register */
  uint8_t NETC_F3_SI5_reserved18[8];
  NETC_F3_SI5_SITMRIER_tag SITMRIER;   /* Station interface timer interrupt enable register */
  NETC_F3_SI5_SITMRIDR_tag SITMRIDR;   /* Station interface timer interrupt detect register */
  uint8_t NETC_F3_SI5_reserved19[4];
  NETC_F3_SI5_SITMRMSIVR_tag SITMRMSIVR; /* Station interface timer MSI-X vector register */
  uint8_t NETC_F3_SI5_reserved20[176];
  NETC_F3_SI5_SIMSITRVR_tag SIMSITRVR[18]; /* Station interface MSI-X transmit ring 0 vector register */
  uint8_t NETC_F3_SI5_reserved21[56];
  NETC_F3_SI5_SIMSIRRVR_tag SIMSIRRVR[18]; /* Station interface MSI-X receive ring 0 vector register */
  uint8_t NETC_F3_SI5_reserved22[568];
  NETC_F3_SI5_SICMECR_tag SICMECR;     /* Station interface correctable memory error configuration register */
  NETC_F3_SI5_SICMESR_tag SICMESR;     /* Station interface correctable memory error status register */
  uint8_t NETC_F3_SI5_reserved23[4];
  NETC_F3_SI5_SICMECTR_tag SICMECTR;   /* Station interface correctable memory error count register */
  NETC_F3_SI5_SIUPECR_tag SIUPECR;     /* Station interface uncorrectable programming error configuration register */
  NETC_F3_SI5_SIUPESR_tag SIUPESR;     /* Station interface uncorrectable programming error status register */
  uint8_t NETC_F3_SI5_reserved24[4];
  NETC_F3_SI5_SIUPECTR_tag SIUPECTR;   /* Station interface uncorrectable programming error count register */
  NETC_F3_SI5_SIUNSBECR_tag SIUNSBECR; /* Station interface uncorrectable non-fatal system bus error configuration register */
  NETC_F3_SI5_SIUNSBESR_tag SIUNSBESR; /* Station interface uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F3_SI5_reserved25[4];
  NETC_F3_SI5_SIUNSBECTR_tag SIUNSBECTR; /* Station interface uncorrectable non-fatal system bus error count register */
  NETC_F3_SI5_SIUFSBECR_tag SIUFSBECR; /* Station interface uncorrectable fatal system bus error configuration register */
  NETC_F3_SI5_SIUFSBESR_tag SIUFSBESR; /* Station interface uncorrectable fatal system bus error status register */
  uint8_t NETC_F3_SI5_reserved26[8];
  NETC_F3_SI5_SIUNMECR_tag SIUNMECR;   /* Station interface uncorrectable non-fatal memory error configuration register */
  NETC_F3_SI5_SIUNMESR0_tag SIUNMESR0; /* Station interface uncorrectable non-fatal memory error status register 0 */
  NETC_F3_SI5_SIUNMESR1_tag SIUNMESR1; /* Station interface uncorrectable non-fatal memory error status register 1 */
  NETC_F3_SI5_SIUNMECTR_tag SIUNMECTR; /* Station interface uncorrectable non-fatal memory error count register */
  NETC_F3_SI5_SIUFMECR_tag SIUFMECR;   /* Station interface uncorrectable fatal memory error configuration register */
  NETC_F3_SI5_SIUFMESR0_tag SIUFMESR0; /* Station interface uncorrectable fatal memory error status register 0 */
  NETC_F3_SI5_SIUFMESR1_tag SIUFMESR1; /* Station interface uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_SI5_reserved27[4];
  NETC_F3_SI5_SIUNIECR_tag SIUNIECR;   /* Station interface uncorrectable non-fatal integrity error configuration register */
  NETC_F3_SI5_SIUNIESR_tag SIUNIESR;   /* Station interface uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_SI5_reserved28[4];
  NETC_F3_SI5_SIUNIECTR_tag SIUNIECTR; /* Station interface uncorrectable non-fatal integrity error count register */
  NETC_F3_SI5_SIUFIECR_tag SIUFIECR;   /* Station interface uncorrectable fatal integrity error configuration register */
  NETC_F3_SI5_SIUFIESR_tag SIUFIESR;   /* Station interface uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_SI5_reserved29[392];
  NETC_F3_SI5_SIMAFTCAPR_tag SIMAFTCAPR; /* Station interface MAC address filter table capability register */
  uint8_t NETC_F3_SI5_reserved30[252];
  NETC_F3_SI5_SIVFTCAPR_tag SIVFTCAPR; /* Station interface VLAN filter table capability register */
  uint8_t NETC_F3_SI5_reserved31[252];
  NETC_F3_SI5_SIRFSCAPR_tag SIRFSCAPR; /* Station interface RFS capability register */
  uint8_t NETC_F3_SI5_reserved32[28156];
  NETC_F3_SI5_BDR_NUM_tag BDR_NUM[18];
};


/* ============================================================================
   =============================== Module: NETC_F3_SI6 ========================
   ============================================================================ */

typedef union NETC_F3_SI6_SIMR_union_tag { /* Station interface mode register */
  vuint32_t R;
  struct {
    vuint32_t RSSE:1;
    vuint32_t RNUM:1;
    vuint32_t RNMM:1;
    vuint32_t RNBM:1;
    vuint32_t V2IPVE:1;
    vuint32_t _unused_5:11;
    vuint32_t DEFAULT_RX_GROUP:1;
    vuint32_t _unused_17:14;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI6_SIMR_tag;

typedef union NETC_F3_SI6_SISR_union_tag { /* Station interface status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;
    vuint32_t MAC_UP:1;
    vuint32_t MAC_MP:1;
    vuint32_t VLAN_P:1;
    vuint32_t VLAN_UTA:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_SI6_SISR_tag;

typedef union NETC_F3_SI6_SICTR0_union_tag { /* Station interface current time register 0 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI6_SICTR0_tag;

typedef union NETC_F3_SI6_SICTR1_union_tag { /* Station interface current time register 1 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI6_SICTR1_tag;

typedef union NETC_F3_SI6_SIPCAPR0_union_tag { /* Station interface port capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;                   /* Receive Flow Steering */
    vuint32_t FP:1;                    /* Frame Preemption */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t TSD:1;                   /* Time Specific Departure */
    vuint32_t CBS:1;                   /* Credit Based Shaping (CBS) */
    vuint32_t _unused_7:1;
    vuint32_t RSS:1;                   /* Receive Side Scaling */
    vuint32_t PSFP:1;                  /* Per-Stream Filtering and Policing (IEEE 802.1Qci) */
    vuint32_t IPFLT:1;                 /* Ingress Port Filtering */
    vuint32_t RP:1;                    /* Rate Policing */
    vuint32_t _unused_12:1;
    vuint32_t WO:1;                    /* Wake-on-LAN */
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional Safety */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI6_SIPCAPR0_tag;

typedef union NETC_F3_SI6_SIPCAPR1_union_tag { /* Station interface port capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:6;
    vuint32_t _unused_18:13;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_SI6_SIPCAPR1_tag;

typedef union NETC_F3_SI6_SIRBGCR_union_tag { /* Station interface receive BDR group control register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GROUPS:2;
    vuint32_t _unused_2:14;
    vuint32_t RINGS_PER_GROUP:3;
    vuint32_t _unused_19:13;
  } B;
} NETC_F3_SI6_SIRBGCR_tag;

typedef union NETC_F3_SI6_SIBCAR_union_tag { /* Station interface buffer cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI6_SIBCAR_tag;

typedef union NETC_F3_SI6_SICCAR_union_tag { /* Station interface command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI6_SICCAR_tag;

typedef union NETC_F3_SI6_SIPMAR0_union_tag { /* Station interface primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_SI6_SIPMAR0_tag;

typedef union NETC_F3_SI6_SIPMAR1_union_tag { /* Station interface primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI6_SIPMAR1_tag;

typedef union NETC_F3_SI6_SICVLANR1_union_tag { /* Station interface custom VLAN register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI6_SICVLANR1_tag;

typedef union NETC_F3_SI6_SICVLANR2_union_tag { /* Station interface custom VLAN register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI6_SICVLANR2_tag;

typedef union NETC_F3_SI6_SIVLANIPVMR0_union_tag { /* Station interface VLAN to IPV mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:4;
    vuint32_t PCP_DEI_1:4;
    vuint32_t PCP_DEI_2:4;
    vuint32_t PCP_DEI_3:4;
    vuint32_t PCP_DEI_4:4;
    vuint32_t PCP_DEI_5:4;
    vuint32_t PCP_DEI_6:4;
    vuint32_t PCP_DEI_7:4;
  } B;
} NETC_F3_SI6_SIVLANIPVMR0_tag;

typedef union NETC_F3_SI6_SIVLANIPVMR1_union_tag { /* Station interface VLAN to IPV mapping register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:4;
    vuint32_t PCP_DEI_9:4;
    vuint32_t PCP_DEI_10:4;
    vuint32_t PCP_DEI_11:4;
    vuint32_t PCP_DEI_12:4;
    vuint32_t PCP_DEI_13:4;
    vuint32_t PCP_DEI_14:4;
    vuint32_t PCP_DEI_15:4;
  } B;
} NETC_F3_SI6_SIVLANIPVMR1_tag;

typedef union NETC_F3_SI6_SIIPVBDRMR0_union_tag { /* Station interface IPV to ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPV0BDR:3;
    vuint32_t _unused_3:1;
    vuint32_t IPV1BDR:3;
    vuint32_t _unused_7:1;
    vuint32_t IPV2BDR:3;
    vuint32_t _unused_11:1;
    vuint32_t IPV3BDR:3;
    vuint32_t _unused_15:1;
    vuint32_t IPV4BDR:3;
    vuint32_t _unused_19:1;
    vuint32_t IPV5BDR:3;
    vuint32_t _unused_23:1;
    vuint32_t IPV6BDR:3;
    vuint32_t _unused_27:1;
    vuint32_t IPV7BDR:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI6_SIIPVBDRMR0_tag;

typedef union NETC_F3_SI6_VSI_A_VSIMSGSR_union_tag { /* Virtual station interface message send register */
  vuint32_t R;
  struct {
    vuint32_t MB:1;
    vuint32_t MS:1;
    vuint32_t _unused_2:14;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI6_VSI_A_VSIMSGSR_tag;

typedef union NETC_F3_SI6_VSI_A_VSIMSGRR_union_tag { /* Virtual station interface message receive register */
  vuint32_t R;
  struct {
    vuint32_t MR:1;
    vuint32_t _unused_1:15;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI6_VSI_A_VSIMSGRR_tag;

typedef union NETC_F3_SI6_VSI_A_VSIMSGSNDAR0_union_tag { /* Virtual station interface message send register 0 */
  vuint32_t R;
  struct {
    vuint32_t MSIZE:5;
    vuint32_t _unused_5:1;
    vuint32_t ADDRL:26;
  } B;
} NETC_F3_SI6_VSI_A_VSIMSGSNDAR0_tag;

typedef union NETC_F3_SI6_VSI_A_VSIMSGSNDAR1_union_tag { /* Virtual station interface message send address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI6_VSI_A_VSIMSGSNDAR1_tag;

typedef struct NETC_F3_SI6_VSI_A_struct_tag {
  NETC_F3_SI6_VSI_A_VSIMSGSR_tag VSIMSGSR; /* Virtual station interface message send register */
  NETC_F3_SI6_VSI_A_VSIMSGRR_tag VSIMSGRR; /* Virtual station interface message receive register */
  uint8_t VSI_A_reserved0[4];
  NETC_F3_SI6_VSI_A_VSIMSGSNDAR0_tag VSIMSGSNDAR0; /* Virtual station interface message send register 0 */
  NETC_F3_SI6_VSI_A_VSIMSGSNDAR1_tag VSIMSGSNDAR1; /* Virtual station interface message send address register 1 */
} NETC_F3_SI6_VSI_A_tag;

typedef union NETC_F3_SI6_SIROCT0_union_tag { /* Station interface receive octets counter (ifInOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_LOW:32;
  } B;
} NETC_F3_SI6_SIROCT0_tag;

typedef union NETC_F3_SI6_SIROCT1_union_tag { /* Station interface receive octets counter (ifInOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_HIGH:32;
  } B;
} NETC_F3_SI6_SIROCT1_tag;

typedef union NETC_F3_SI6_SIRFRM0_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_LOW:32;
  } B;
} NETC_F3_SI6_SIRFRM0_tag;

typedef union NETC_F3_SI6_SIRFRM1_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_HIGH:32;
  } B;
} NETC_F3_SI6_SIRFRM1_tag;

typedef union NETC_F3_SI6_SIRUCA0_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_LOW:32;
  } B;
} NETC_F3_SI6_SIRUCA0_tag;

typedef union NETC_F3_SI6_SIRUCA1_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_HIGH:32;
  } B;
} NETC_F3_SI6_SIRUCA1_tag;

typedef union NETC_F3_SI6_SIRMCA0_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_LOW:32;
  } B;
} NETC_F3_SI6_SIRMCA0_tag;

typedef union NETC_F3_SI6_SIRMCA1_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_HIGH:32;
  } B;
} NETC_F3_SI6_SIRMCA1_tag;

typedef union NETC_F3_SI6_SITOCT0_union_tag { /* Station interface transmit octets counter (ifOutOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_LOW:32;
  } B;
} NETC_F3_SI6_SITOCT0_tag;

typedef union NETC_F3_SI6_SITOCT1_union_tag { /* Station interface transmit octets counter (ifOutOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_HIGH:32;
  } B;
} NETC_F3_SI6_SITOCT1_tag;

typedef union NETC_F3_SI6_SITFRM0_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_LOW:32;
  } B;
} NETC_F3_SI6_SITFRM0_tag;

typedef union NETC_F3_SI6_SITFRM1_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_HIGH:32;
  } B;
} NETC_F3_SI6_SITFRM1_tag;

typedef union NETC_F3_SI6_SITUCA0_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_LOW:32;
  } B;
} NETC_F3_SI6_SITUCA0_tag;

typedef union NETC_F3_SI6_SITUCA1_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_HIGH:32;
  } B;
} NETC_F3_SI6_SITUCA1_tag;

typedef union NETC_F3_SI6_SITMCA0_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_LOW:32;
  } B;
} NETC_F3_SI6_SITMCA0_tag;

typedef union NETC_F3_SI6_SITMCA1_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_HIGH:32;
  } B;
} NETC_F3_SI6_SITMCA1_tag;

typedef union NETC_F3_SI6_SIBLPR_union_tag { /* Station interface boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F3_SI6_SIBLPR_tag;

typedef union NETC_F3_SI6_SICBDRMR_union_tag { /* Station interface command BDR mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI6_SICBDRMR_tag;

typedef union NETC_F3_SI6_SICBDRSR_union_tag { /* Station interface command BDR status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI6_SICBDRSR_tag;

typedef union NETC_F3_SI6_SICBDRBAR0_union_tag { /* Station interface command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI6_SICBDRBAR0_tag;

typedef union NETC_F3_SI6_SICBDRBAR1_union_tag { /* Station interface command BDR base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI6_SICBDRBAR1_tag;

typedef union NETC_F3_SI6_SICBDRPIR_union_tag { /* Station interface command BDR producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI6_SICBDRPIR_tag;

typedef union NETC_F3_SI6_SICBDRCIR_union_tag { /* Station interface command BDR consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI6_SICBDRCIR_tag;

typedef union NETC_F3_SI6_SICBDRLENR_union_tag { /* Station interface command BDR length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_SI6_SICBDRLENR_tag;

typedef union NETC_F3_SI6_SICBDRIER_union_tag { /* Station interface command BDR interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI6_SICBDRIER_tag;

typedef union NETC_F3_SI6_SICBDRIDR_union_tag { /* Station interface command BDR interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI6_SICBDRIDR_tag;

typedef union NETC_F3_SI6_SICAPR0_union_tag { /* Station interface capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_RX_BDR:8;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F3_SI6_SICAPR0_tag;

typedef union NETC_F3_SI6_SICAPR1_union_tag { /* Station interface capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t NUM_RX_GRP:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI6_SICAPR1_tag;

typedef union NETC_F3_SI6_SICAPR2_union_tag { /* Station interface capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t VTP:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_SI6_SICAPR2_tag;

typedef union NETC_F3_SI6_VSI_VSIIER_union_tag { /* Virtual station interface interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MSIE:1;
    vuint32_t MRIE:1;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI6_VSI_VSIIER_tag;

typedef union NETC_F3_SI6_VSI_VSIIDR_union_tag { /* Virtual station interface interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXR:1;
    vuint32_t _unused_1:7;
    vuint32_t MS:1;
    vuint32_t MR:1;
    vuint32_t _unused_10:6;
    vuint32_t RXR:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI6_VSI_VSIIDR_tag;

typedef struct NETC_F3_SI6_VSI_struct_tag {
  NETC_F3_SI6_VSI_VSIIER_tag VSIIER;   /* Virtual station interface interrupt enable register */
  uint8_t VSI_reserved0[4];
  NETC_F3_SI6_VSI_VSIIDR_tag VSIIDR;   /* Virtual station interface interrupt detect register */
} NETC_F3_SI6_VSI_tag;

typedef union NETC_F3_SI6_SITXIDR0_union_tag { /* Station interface transmit interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXT0:1;
    vuint32_t TXT1:1;
    vuint32_t TXT2:1;
    vuint32_t TXT3:1;
    vuint32_t TXT4:1;
    vuint32_t TXT5:1;
    vuint32_t TXT6:1;
    vuint32_t TXT7:1;
    vuint32_t TXT8:1;
    vuint32_t TXT9:1;
    vuint32_t TXT10:1;
    vuint32_t TXT11:1;
    vuint32_t TXT12:1;
    vuint32_t TXT13:1;
    vuint32_t TXT14:1;
    vuint32_t TXT15:1;
    vuint32_t TXF0:1;
    vuint32_t TXF1:1;
    vuint32_t TXF2:1;
    vuint32_t TXF3:1;
    vuint32_t TXF4:1;
    vuint32_t TXF5:1;
    vuint32_t TXF6:1;
    vuint32_t TXF7:1;
    vuint32_t TXF8:1;
    vuint32_t TXF9:1;
    vuint32_t TXF10:1;
    vuint32_t TXF11:1;
    vuint32_t TXF12:1;
    vuint32_t TXF13:1;
    vuint32_t TXF14:1;
    vuint32_t TXF15:1;
  } B;
} NETC_F3_SI6_SITXIDR0_tag;

typedef union NETC_F3_SI6_SITXIDR1_union_tag { /* Station interface transmit interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t TXT16:1;
    vuint32_t TXT17:1;
    vuint32_t _unused_2:14;
    vuint32_t TXF16:1;
    vuint32_t TXF17:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_SI6_SITXIDR1_tag;

typedef union NETC_F3_SI6_SIRXIDR0_union_tag { /* Station interface receive interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t RX0:1;
    vuint32_t RX1:1;
    vuint32_t RX2:1;
    vuint32_t RX3:1;
    vuint32_t RX4:1;
    vuint32_t RX5:1;
    vuint32_t RX6:1;
    vuint32_t RX7:1;
    vuint32_t RX8:1;
    vuint32_t RX9:1;
    vuint32_t RX10:1;
    vuint32_t RX11:1;
    vuint32_t RX12:1;
    vuint32_t RX13:1;
    vuint32_t RX14:1;
    vuint32_t RX15:1;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI6_SIRXIDR0_tag;

typedef union NETC_F3_SI6_SIRXIDR1_union_tag { /* Station interface receive interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t RX16:1;
    vuint32_t RX17:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI6_SIRXIDR1_tag;

typedef union NETC_F3_SI6_SIMSIVR_union_tag { /* Station interface MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI6_SIMSIVR_tag;

typedef union NETC_F3_SI6_SICMSIVR_union_tag { /* Station interface command MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI6_SICMSIVR_tag;

typedef union NETC_F3_SI6_SITMRIER_union_tag { /* Station interface timer interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t SYNCE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI6_SITMRIER_tag;

typedef union NETC_F3_SI6_SITMRIDR_union_tag { /* Station interface timer interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI6_SITMRIDR_tag;

typedef union NETC_F3_SI6_SITMRMSIVR_union_tag { /* Station interface timer MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI6_SITMRMSIVR_tag;

typedef union NETC_F3_SI6_SIMSITRVR_union_tag { /* Station interface MSI-X transmit ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI6_SIMSITRVR_tag;

typedef union NETC_F3_SI6_SIMSIRRVR_union_tag { /* Station interface MSI-X receive ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI6_SIMSIRRVR_tag;

typedef union NETC_F3_SI6_SICMECR_union_tag { /* Station interface correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI6_SICMECR_tag;

typedef union NETC_F3_SI6_SICMESR_union_tag { /* Station interface correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_SI6_SICMESR_tag;

typedef union NETC_F3_SI6_SICMECTR_union_tag { /* Station interface correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI6_SICMECTR_tag;

typedef union NETC_F3_SI6_SIUPECR_union_tag { /* Station interface uncorrectable programming error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI6_SIUPECR_tag;

typedef union NETC_F3_SI6_SIUPESR_union_tag { /* Station interface uncorrectable programming error status register */
  vuint32_t R;
  struct {
    vuint32_t DROP_SI_EN:1;
    vuint32_t DROP_RING_EN:1;
    vuint32_t DROP_GRP_SEL:1;
    vuint32_t DROP_RING_SEL:1;
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t PE:1;                    /* Programming error */
  } B;
} NETC_F3_SI6_SIUPESR_tag;

typedef union NETC_F3_SI6_SIUPECTR_union_tag { /* Station interface uncorrectable programming error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;                /* Count */
  } B;
} NETC_F3_SI6_SIUPECTR_tag;

typedef union NETC_F3_SI6_SIUNSBECR_union_tag { /* Station interface uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI6_SIUNSBECR_tag;

typedef union NETC_F3_SI6_SIUNSBESR_union_tag { /* Station interface uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI6_SIUNSBESR_tag;

typedef union NETC_F3_SI6_SIUNSBECTR_union_tag { /* Station interface uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI6_SIUNSBECTR_tag;

typedef union NETC_F3_SI6_SIUFSBECR_union_tag { /* Station interface uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI6_SIUFSBECR_tag;

typedef union NETC_F3_SI6_SIUFSBESR_union_tag { /* Station interface uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI6_SIUFSBESR_tag;

typedef union NETC_F3_SI6_SIUNMECR_union_tag { /* Station interface uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI6_SIUNMECR_tag;

typedef union NETC_F3_SI6_SIUNMESR0_union_tag { /* Station interface uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI6_SIUNMESR0_tag;

typedef union NETC_F3_SI6_SIUNMESR1_union_tag { /* Station interface uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI6_SIUNMESR1_tag;

typedef union NETC_F3_SI6_SIUNMECTR_union_tag { /* Station interface uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI6_SIUNMECTR_tag;

typedef union NETC_F3_SI6_SIUFMECR_union_tag { /* Station interface uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI6_SIUFMECR_tag;

typedef union NETC_F3_SI6_SIUFMESR0_union_tag { /* Station interface uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI6_SIUFMESR0_tag;

typedef union NETC_F3_SI6_SIUFMESR1_union_tag { /* Station interface uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI6_SIUFMESR1_tag;

typedef union NETC_F3_SI6_SIUNIECR_union_tag { /* Station interface uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI6_SIUNIECR_tag;

typedef union NETC_F3_SI6_SIUNIESR_union_tag { /* Station interface uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI6_SIUNIESR_tag;

typedef union NETC_F3_SI6_SIUNIECTR_union_tag { /* Station interface uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI6_SIUNIECTR_tag;

typedef union NETC_F3_SI6_SIUFIECR_union_tag { /* Station interface uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI6_SIUFIECR_tag;

typedef union NETC_F3_SI6_SIUFIESR_union_tag { /* Station interface uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:16;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI6_SIUFIESR_tag;

typedef union NETC_F3_SI6_SIMAFTCAPR_union_tag { /* Station interface MAC address filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI6_SIMAFTCAPR_tag;

typedef union NETC_F3_SI6_SIVFTCAPR_union_tag { /* Station interface VLAN filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI6_SIVFTCAPR_tag;

typedef union NETC_F3_SI6_SIRFSCAPR_union_tag { /* Station interface RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_SI6_SIRFSCAPR_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBMR_union_tag { /* Tx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t PRIO:3;
    vuint32_t _unused_3:1;
    vuint32_t WRR:3;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t VIH:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI6_BDR_NUM_TBMR_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBSR_union_tag { /* Tx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI6_BDR_NUM_TBSR_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBBAR0_union_tag { /* Tx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI6_BDR_NUM_TBBAR0_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBBAR1_union_tag { /* Tx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI6_BDR_NUM_TBBAR1_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBPIR_union_tag { /* Tx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI6_BDR_NUM_TBPIR_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBCIR_union_tag { /* Tx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t STAT_ID:16;
  } B;
} NETC_F3_SI6_BDR_NUM_TBCIR_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBLENR_union_tag { /* Tx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI6_BDR_NUM_TBLENR_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBIER_union_tag { /* Tx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t TXTIE:1;
    vuint32_t TXFIE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI6_BDR_NUM_TBIER_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBIDR_union_tag { /* Tx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXT:1;
    vuint32_t TXF:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI6_BDR_NUM_TBIDR_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBICR0_union_tag { /* Tx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:4;
    vuint32_t _unused_4:27;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI6_BDR_NUM_TBICR0_tag;

typedef union NETC_F3_SI6_BDR_NUM_TBICR1_union_tag { /* Tx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI6_BDR_NUM_TBICR1_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBMR_union_tag { /* Rx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t AL:1;
    vuint32_t _unused_1:1;
    vuint32_t BDS:1;
    vuint32_t _unused_3:1;
    vuint32_t CM:1;
    vuint32_t VTE:1;
    vuint32_t VTPD:1;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI6_BDR_NUM_RBMR_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBSR_union_tag { /* Rx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI6_BDR_NUM_RBSR_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBBSR_union_tag { /* Rx BDR 0 buffer size register */
  vuint32_t R;
  struct {
    vuint32_t BSIZE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI6_BDR_NUM_RBBSR_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBCIR_union_tag { /* Rx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI6_BDR_NUM_RBCIR_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBBAR0_union_tag { /* Rx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI6_BDR_NUM_RBBAR0_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBBAR1_union_tag { /* Rx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI6_BDR_NUM_RBBAR1_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBPIR_union_tag { /* Rx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI6_BDR_NUM_RBPIR_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBLENR_union_tag { /* Rx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI6_BDR_NUM_RBLENR_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBDCR_union_tag { /* Rx BDR 0 drop count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_SI6_BDR_NUM_RBDCR_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBIER_union_tag { /* Rx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t RXTIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI6_BDR_NUM_RBIER_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBIDR_union_tag { /* Rx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t RXT:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI6_BDR_NUM_RBIDR_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBICR0_union_tag { /* Rx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:9;
    vuint32_t _unused_9:22;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI6_BDR_NUM_RBICR0_tag;

typedef union NETC_F3_SI6_BDR_NUM_RBICR1_union_tag { /* Rx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI6_BDR_NUM_RBICR1_tag;

typedef struct NETC_F3_SI6_BDR_NUM_struct_tag {
  NETC_F3_SI6_BDR_NUM_TBMR_tag TBMR;   /* Tx BDR 0 mode register */
  NETC_F3_SI6_BDR_NUM_TBSR_tag TBSR;   /* Tx BDR 0 status register */
  uint8_t BDR_NUM_reserved0[8];
  NETC_F3_SI6_BDR_NUM_TBBAR0_tag TBBAR0; /* Tx BDR 0 base address register 0 */
  NETC_F3_SI6_BDR_NUM_TBBAR1_tag TBBAR1; /* Tx BDR 0 base address register 1 */
  NETC_F3_SI6_BDR_NUM_TBPIR_tag TBPIR; /* Tx BDR 0 producer index register */
  NETC_F3_SI6_BDR_NUM_TBCIR_tag TBCIR; /* Tx BDR 0 consumer index register */
  NETC_F3_SI6_BDR_NUM_TBLENR_tag TBLENR; /* Tx BDR 0 length register */
  uint8_t BDR_NUM_reserved1[124];
  NETC_F3_SI6_BDR_NUM_TBIER_tag TBIER; /* Tx BDR 0 interrupt enable register */
  NETC_F3_SI6_BDR_NUM_TBIDR_tag TBIDR; /* Tx BDR 0 interrupt detect register */
  NETC_F3_SI6_BDR_NUM_TBICR0_tag TBICR0; /* Tx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI6_BDR_NUM_TBICR1_tag TBICR1; /* Tx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved2[80];
  NETC_F3_SI6_BDR_NUM_RBMR_tag RBMR;   /* Rx BDR 0 mode register */
  NETC_F3_SI6_BDR_NUM_RBSR_tag RBSR;   /* Rx BDR 0 status register */
  NETC_F3_SI6_BDR_NUM_RBBSR_tag RBBSR; /* Rx BDR 0 buffer size register */
  NETC_F3_SI6_BDR_NUM_RBCIR_tag RBCIR; /* Rx BDR 0 consumer index register */
  NETC_F3_SI6_BDR_NUM_RBBAR0_tag RBBAR0; /* Rx BDR 0 base address register 0 */
  NETC_F3_SI6_BDR_NUM_RBBAR1_tag RBBAR1; /* Rx BDR 0 base address register 1 */
  NETC_F3_SI6_BDR_NUM_RBPIR_tag RBPIR; /* Rx BDR 0 producer index register */
  uint8_t BDR_NUM_reserved3[4];
  NETC_F3_SI6_BDR_NUM_RBLENR_tag RBLENR; /* Rx BDR 0 length register */
  uint8_t BDR_NUM_reserved4[92];
  NETC_F3_SI6_BDR_NUM_RBDCR_tag RBDCR; /* Rx BDR 0 drop count register */
  uint8_t BDR_NUM_reserved5[28];
  NETC_F3_SI6_BDR_NUM_RBIER_tag RBIER; /* Rx BDR 0 interrupt enable register */
  NETC_F3_SI6_BDR_NUM_RBIDR_tag RBIDR; /* Rx BDR 0 interrupt detect register */
  NETC_F3_SI6_BDR_NUM_RBICR0_tag RBICR0; /* Rx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI6_BDR_NUM_RBICR1_tag RBICR1; /* Rx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved6[80];
} NETC_F3_SI6_BDR_NUM_tag;

struct NETC_F3_SI6_tag {
  NETC_F3_SI6_SIMR_tag SIMR;           /* Station interface mode register */
  NETC_F3_SI6_SISR_tag SISR;           /* Station interface status register */
  uint8_t NETC_F3_SI6_reserved0[16];
  NETC_F3_SI6_SICTR0_tag SICTR0;       /* Station interface current time register 0 */
  NETC_F3_SI6_SICTR1_tag SICTR1;       /* Station interface current time register 1 */
  NETC_F3_SI6_SIPCAPR0_tag SIPCAPR0;   /* Station interface port capability register 0 */
  NETC_F3_SI6_SIPCAPR1_tag SIPCAPR1;   /* Station interface port capability register 1 */
  uint8_t NETC_F3_SI6_reserved1[16];
  NETC_F3_SI6_SIRBGCR_tag SIRBGCR;     /* Station interface receive BDR group control register */
  uint8_t NETC_F3_SI6_reserved2[4];
  NETC_F3_SI6_SIBCAR_tag SIBCAR;       /* Station interface buffer cache attribute register */
  uint8_t NETC_F3_SI6_reserved3[4];
  NETC_F3_SI6_SICCAR_tag SICCAR;       /* Station interface command cache attribute register */
  uint8_t NETC_F3_SI6_reserved4[52];
  NETC_F3_SI6_SIPMAR0_tag SIPMAR0;     /* Station interface primary MAC address register 0 */
  NETC_F3_SI6_SIPMAR1_tag SIPMAR1;     /* Station interface primary MAC address register 1 */
  uint8_t NETC_F3_SI6_reserved5[8];
  NETC_F3_SI6_SICVLANR1_tag SICVLANR1; /* Station interface custom VLAN register 1 */
  NETC_F3_SI6_SICVLANR2_tag SICVLANR2; /* Station interface custom VLAN register 2 */
  uint8_t NETC_F3_SI6_reserved6[104];
  NETC_F3_SI6_SIVLANIPVMR0_tag SIVLANIPVMR0; /* Station interface VLAN to IPV mapping register 0 */
  NETC_F3_SI6_SIVLANIPVMR1_tag SIVLANIPVMR1; /* Station interface VLAN to IPV mapping register 1 */
  uint8_t NETC_F3_SI6_reserved7[72];
  NETC_F3_SI6_SIIPVBDRMR0_tag SIIPVBDRMR0; /* Station interface IPV to ring mapping register */
  uint8_t NETC_F3_SI6_reserved8[176];
  NETC_F3_SI6_VSI_A_tag VSI_A;
  uint8_t NETC_F3_SI6_reserved9[232];
  NETC_F3_SI6_SIROCT0_tag SIROCT0;     /* Station interface receive octets counter (ifInOctets) 0 */
  NETC_F3_SI6_SIROCT1_tag SIROCT1;     /* Station interface receive octets counter (ifInOctets) 1 */
  NETC_F3_SI6_SIRFRM0_tag SIRFRM0;     /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  NETC_F3_SI6_SIRFRM1_tag SIRFRM1;     /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  NETC_F3_SI6_SIRUCA0_tag SIRUCA0;     /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  NETC_F3_SI6_SIRUCA1_tag SIRUCA1;     /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  NETC_F3_SI6_SIRMCA0_tag SIRMCA0;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  NETC_F3_SI6_SIRMCA1_tag SIRMCA1;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  NETC_F3_SI6_SITOCT0_tag SITOCT0;     /* Station interface transmit octets counter (ifOutOctets) 0 */
  NETC_F3_SI6_SITOCT1_tag SITOCT1;     /* Station interface transmit octets counter (ifOutOctets) 1 */
  NETC_F3_SI6_SITFRM0_tag SITFRM0;     /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  NETC_F3_SI6_SITFRM1_tag SITFRM1;     /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  NETC_F3_SI6_SITUCA0_tag SITUCA0;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  NETC_F3_SI6_SITUCA1_tag SITUCA1;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  NETC_F3_SI6_SITMCA0_tag SITMCA0;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  NETC_F3_SI6_SITMCA1_tag SITMCA1;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  uint8_t NETC_F3_SI6_reserved10[176];
  NETC_F3_SI6_SIBLPR_tag SIBLPR[2];    /* Station interface boot loader parameter register 0 */
  uint8_t NETC_F3_SI6_reserved11[1032];
  NETC_F3_SI6_SICBDRMR_tag SICBDRMR;   /* Station interface command BDR mode register */
  NETC_F3_SI6_SICBDRSR_tag SICBDRSR;   /* Station interface command BDR status register */
  uint8_t NETC_F3_SI6_reserved12[8];
  NETC_F3_SI6_SICBDRBAR0_tag SICBDRBAR0; /* Station interface command BDR base address register 0 */
  NETC_F3_SI6_SICBDRBAR1_tag SICBDRBAR1; /* Station interface command BDR base address register 1 */
  NETC_F3_SI6_SICBDRPIR_tag SICBDRPIR; /* Station interface command BDR producer index register */
  NETC_F3_SI6_SICBDRCIR_tag SICBDRCIR; /* Station interface command BDR consumer index register */
  NETC_F3_SI6_SICBDRLENR_tag SICBDRLENR; /* Station interface command BDR length register */
  uint8_t NETC_F3_SI6_reserved13[124];
  NETC_F3_SI6_SICBDRIER_tag SICBDRIER; /* Station interface command BDR interrupt enable register */
  NETC_F3_SI6_SICBDRIDR_tag SICBDRIDR; /* Station interface command BDR interrupt detect register */
  uint8_t NETC_F3_SI6_reserved14[88];
  NETC_F3_SI6_SICAPR0_tag SICAPR0;     /* Station interface capability register 0 */
  NETC_F3_SI6_SICAPR1_tag SICAPR1;     /* Station interface capability register 1 */
  NETC_F3_SI6_SICAPR2_tag SICAPR2;     /* Station interface capability register 2 */
  uint8_t NETC_F3_SI6_reserved15[244];
  NETC_F3_SI6_VSI_tag VSI;
  uint8_t NETC_F3_SI6_reserved16[12];
  NETC_F3_SI6_SITXIDR0_tag SITXIDR0;   /* Station interface transmit interrupt detect register 0 */
  NETC_F3_SI6_SITXIDR1_tag SITXIDR1;   /* Station interface transmit interrupt detect register 1 */
  uint8_t NETC_F3_SI6_reserved17[8];
  NETC_F3_SI6_SIRXIDR0_tag SIRXIDR0;   /* Station interface receive interrupt detect register 0 */
  NETC_F3_SI6_SIRXIDR1_tag SIRXIDR1;   /* Station interface receive interrupt detect register 1 */
  NETC_F3_SI6_SIMSIVR_tag SIMSIVR;     /* Station interface MSI-X vector register */
  NETC_F3_SI6_SICMSIVR_tag SICMSIVR;   /* Station interface command MSI-X vector register */
  uint8_t NETC_F3_SI6_reserved18[8];
  NETC_F3_SI6_SITMRIER_tag SITMRIER;   /* Station interface timer interrupt enable register */
  NETC_F3_SI6_SITMRIDR_tag SITMRIDR;   /* Station interface timer interrupt detect register */
  uint8_t NETC_F3_SI6_reserved19[4];
  NETC_F3_SI6_SITMRMSIVR_tag SITMRMSIVR; /* Station interface timer MSI-X vector register */
  uint8_t NETC_F3_SI6_reserved20[176];
  NETC_F3_SI6_SIMSITRVR_tag SIMSITRVR[18]; /* Station interface MSI-X transmit ring 0 vector register */
  uint8_t NETC_F3_SI6_reserved21[56];
  NETC_F3_SI6_SIMSIRRVR_tag SIMSIRRVR[18]; /* Station interface MSI-X receive ring 0 vector register */
  uint8_t NETC_F3_SI6_reserved22[568];
  NETC_F3_SI6_SICMECR_tag SICMECR;     /* Station interface correctable memory error configuration register */
  NETC_F3_SI6_SICMESR_tag SICMESR;     /* Station interface correctable memory error status register */
  uint8_t NETC_F3_SI6_reserved23[4];
  NETC_F3_SI6_SICMECTR_tag SICMECTR;   /* Station interface correctable memory error count register */
  NETC_F3_SI6_SIUPECR_tag SIUPECR;     /* Station interface uncorrectable programming error configuration register */
  NETC_F3_SI6_SIUPESR_tag SIUPESR;     /* Station interface uncorrectable programming error status register */
  uint8_t NETC_F3_SI6_reserved24[4];
  NETC_F3_SI6_SIUPECTR_tag SIUPECTR;   /* Station interface uncorrectable programming error count register */
  NETC_F3_SI6_SIUNSBECR_tag SIUNSBECR; /* Station interface uncorrectable non-fatal system bus error configuration register */
  NETC_F3_SI6_SIUNSBESR_tag SIUNSBESR; /* Station interface uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F3_SI6_reserved25[4];
  NETC_F3_SI6_SIUNSBECTR_tag SIUNSBECTR; /* Station interface uncorrectable non-fatal system bus error count register */
  NETC_F3_SI6_SIUFSBECR_tag SIUFSBECR; /* Station interface uncorrectable fatal system bus error configuration register */
  NETC_F3_SI6_SIUFSBESR_tag SIUFSBESR; /* Station interface uncorrectable fatal system bus error status register */
  uint8_t NETC_F3_SI6_reserved26[8];
  NETC_F3_SI6_SIUNMECR_tag SIUNMECR;   /* Station interface uncorrectable non-fatal memory error configuration register */
  NETC_F3_SI6_SIUNMESR0_tag SIUNMESR0; /* Station interface uncorrectable non-fatal memory error status register 0 */
  NETC_F3_SI6_SIUNMESR1_tag SIUNMESR1; /* Station interface uncorrectable non-fatal memory error status register 1 */
  NETC_F3_SI6_SIUNMECTR_tag SIUNMECTR; /* Station interface uncorrectable non-fatal memory error count register */
  NETC_F3_SI6_SIUFMECR_tag SIUFMECR;   /* Station interface uncorrectable fatal memory error configuration register */
  NETC_F3_SI6_SIUFMESR0_tag SIUFMESR0; /* Station interface uncorrectable fatal memory error status register 0 */
  NETC_F3_SI6_SIUFMESR1_tag SIUFMESR1; /* Station interface uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_SI6_reserved27[4];
  NETC_F3_SI6_SIUNIECR_tag SIUNIECR;   /* Station interface uncorrectable non-fatal integrity error configuration register */
  NETC_F3_SI6_SIUNIESR_tag SIUNIESR;   /* Station interface uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_SI6_reserved28[4];
  NETC_F3_SI6_SIUNIECTR_tag SIUNIECTR; /* Station interface uncorrectable non-fatal integrity error count register */
  NETC_F3_SI6_SIUFIECR_tag SIUFIECR;   /* Station interface uncorrectable fatal integrity error configuration register */
  NETC_F3_SI6_SIUFIESR_tag SIUFIESR;   /* Station interface uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_SI6_reserved29[392];
  NETC_F3_SI6_SIMAFTCAPR_tag SIMAFTCAPR; /* Station interface MAC address filter table capability register */
  uint8_t NETC_F3_SI6_reserved30[252];
  NETC_F3_SI6_SIVFTCAPR_tag SIVFTCAPR; /* Station interface VLAN filter table capability register */
  uint8_t NETC_F3_SI6_reserved31[252];
  NETC_F3_SI6_SIRFSCAPR_tag SIRFSCAPR; /* Station interface RFS capability register */
  uint8_t NETC_F3_SI6_reserved32[28156];
  NETC_F3_SI6_BDR_NUM_tag BDR_NUM[18];
};


/* ============================================================================
   =============================== Module: NETC_F3_SI7 ========================
   ============================================================================ */

typedef union NETC_F3_SI7_SIMR_union_tag { /* Station interface mode register */
  vuint32_t R;
  struct {
    vuint32_t RSSE:1;
    vuint32_t RNUM:1;
    vuint32_t RNMM:1;
    vuint32_t RNBM:1;
    vuint32_t V2IPVE:1;
    vuint32_t _unused_5:11;
    vuint32_t DEFAULT_RX_GROUP:1;
    vuint32_t _unused_17:14;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI7_SIMR_tag;

typedef union NETC_F3_SI7_SISR_union_tag { /* Station interface status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;
    vuint32_t MAC_UP:1;
    vuint32_t MAC_MP:1;
    vuint32_t VLAN_P:1;
    vuint32_t VLAN_UTA:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_F3_SI7_SISR_tag;

typedef union NETC_F3_SI7_SICTR0_union_tag { /* Station interface current time register 0 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI7_SICTR0_tag;

typedef union NETC_F3_SI7_SICTR1_union_tag { /* Station interface current time register 1 */
  vuint32_t R;
  struct {
    vuint32_t CURR_TIME:32;
  } B;
} NETC_F3_SI7_SICTR1_tag;

typedef union NETC_F3_SI7_SIPCAPR0_union_tag { /* Station interface port capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t RFS:1;                   /* Receive Flow Steering */
    vuint32_t FP:1;                    /* Frame Preemption */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t TSD:1;                   /* Time Specific Departure */
    vuint32_t CBS:1;                   /* Credit Based Shaping (CBS) */
    vuint32_t _unused_7:1;
    vuint32_t RSS:1;                   /* Receive Side Scaling */
    vuint32_t PSFP:1;                  /* Per-Stream Filtering and Policing (IEEE 802.1Qci) */
    vuint32_t IPFLT:1;                 /* Ingress Port Filtering */
    vuint32_t RP:1;                    /* Rate Policing */
    vuint32_t _unused_12:1;
    vuint32_t WO:1;                    /* Wake-on-LAN */
    vuint32_t _unused_14:2;
    vuint32_t FS:1;                    /* Functional Safety */
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI7_SIPCAPR0_tag;

typedef union NETC_F3_SI7_SIPCAPR1_union_tag { /* Station interface port capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t NUM_TCS:3;
    vuint32_t _unused_7:1;
    vuint32_t NUM_MCH:2;
    vuint32_t NUM_UCH:2;
    vuint32_t NUM_MSIX:6;
    vuint32_t _unused_18:13;
    vuint32_t NUM_IPV:1;
  } B;
} NETC_F3_SI7_SIPCAPR1_tag;

typedef union NETC_F3_SI7_SIRBGCR_union_tag { /* Station interface receive BDR group control register */
  vuint32_t R;
  struct {
    vuint32_t NUM_GROUPS:2;
    vuint32_t _unused_2:14;
    vuint32_t RINGS_PER_GROUP:3;
    vuint32_t _unused_19:13;
  } B;
} NETC_F3_SI7_SIRBGCR_tag;

typedef union NETC_F3_SI7_SIBCAR_union_tag { /* Station interface buffer cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI7_SIBCAR_tag;

typedef union NETC_F3_SI7_SICCAR_union_tag { /* Station interface command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI7_SICCAR_tag;

typedef union NETC_F3_SI7_SIPMAR0_union_tag { /* Station interface primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} NETC_F3_SI7_SIPMAR0_tag;

typedef union NETC_F3_SI7_SIPMAR1_union_tag { /* Station interface primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI7_SIPMAR1_tag;

typedef union NETC_F3_SI7_SICVLANR1_union_tag { /* Station interface custom VLAN register 1 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI7_SICVLANR1_tag;

typedef union NETC_F3_SI7_SICVLANR2_union_tag { /* Station interface custom VLAN register 2 */
  vuint32_t R;
  struct {
    vuint32_t ETYPE:16;
    vuint32_t _unused_16:15;
    vuint32_t V:1;
  } B;
} NETC_F3_SI7_SICVLANR2_tag;

typedef union NETC_F3_SI7_SIVLANIPVMR0_union_tag { /* Station interface VLAN to IPV mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_0:4;
    vuint32_t PCP_DEI_1:4;
    vuint32_t PCP_DEI_2:4;
    vuint32_t PCP_DEI_3:4;
    vuint32_t PCP_DEI_4:4;
    vuint32_t PCP_DEI_5:4;
    vuint32_t PCP_DEI_6:4;
    vuint32_t PCP_DEI_7:4;
  } B;
} NETC_F3_SI7_SIVLANIPVMR0_tag;

typedef union NETC_F3_SI7_SIVLANIPVMR1_union_tag { /* Station interface VLAN to IPV mapping register 1 */
  vuint32_t R;
  struct {
    vuint32_t PCP_DEI_8:4;
    vuint32_t PCP_DEI_9:4;
    vuint32_t PCP_DEI_10:4;
    vuint32_t PCP_DEI_11:4;
    vuint32_t PCP_DEI_12:4;
    vuint32_t PCP_DEI_13:4;
    vuint32_t PCP_DEI_14:4;
    vuint32_t PCP_DEI_15:4;
  } B;
} NETC_F3_SI7_SIVLANIPVMR1_tag;

typedef union NETC_F3_SI7_SIIPVBDRMR0_union_tag { /* Station interface IPV to ring mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPV0BDR:3;
    vuint32_t _unused_3:1;
    vuint32_t IPV1BDR:3;
    vuint32_t _unused_7:1;
    vuint32_t IPV2BDR:3;
    vuint32_t _unused_11:1;
    vuint32_t IPV3BDR:3;
    vuint32_t _unused_15:1;
    vuint32_t IPV4BDR:3;
    vuint32_t _unused_19:1;
    vuint32_t IPV5BDR:3;
    vuint32_t _unused_23:1;
    vuint32_t IPV6BDR:3;
    vuint32_t _unused_27:1;
    vuint32_t IPV7BDR:3;
    vuint32_t _unused_31:1;
  } B;
} NETC_F3_SI7_SIIPVBDRMR0_tag;

typedef union NETC_F3_SI7_VSI_A_VSIMSGSR_union_tag { /* Virtual station interface message send register */
  vuint32_t R;
  struct {
    vuint32_t MB:1;
    vuint32_t MS:1;
    vuint32_t _unused_2:14;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI7_VSI_A_VSIMSGSR_tag;

typedef union NETC_F3_SI7_VSI_A_VSIMSGRR_union_tag { /* Virtual station interface message receive register */
  vuint32_t R;
  struct {
    vuint32_t MR:1;
    vuint32_t _unused_1:15;
    vuint32_t MC:16;
  } B;
} NETC_F3_SI7_VSI_A_VSIMSGRR_tag;

typedef union NETC_F3_SI7_VSI_A_VSIMSGSNDAR0_union_tag { /* Virtual station interface message send register 0 */
  vuint32_t R;
  struct {
    vuint32_t MSIZE:5;
    vuint32_t _unused_5:1;
    vuint32_t ADDRL:26;
  } B;
} NETC_F3_SI7_VSI_A_VSIMSGSNDAR0_tag;

typedef union NETC_F3_SI7_VSI_A_VSIMSGSNDAR1_union_tag { /* Virtual station interface message send address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI7_VSI_A_VSIMSGSNDAR1_tag;

typedef struct NETC_F3_SI7_VSI_A_struct_tag {
  NETC_F3_SI7_VSI_A_VSIMSGSR_tag VSIMSGSR; /* Virtual station interface message send register */
  NETC_F3_SI7_VSI_A_VSIMSGRR_tag VSIMSGRR; /* Virtual station interface message receive register */
  uint8_t VSI_A_reserved0[4];
  NETC_F3_SI7_VSI_A_VSIMSGSNDAR0_tag VSIMSGSNDAR0; /* Virtual station interface message send register 0 */
  NETC_F3_SI7_VSI_A_VSIMSGSNDAR1_tag VSIMSGSNDAR1; /* Virtual station interface message send address register 1 */
} NETC_F3_SI7_VSI_A_tag;

typedef union NETC_F3_SI7_SIROCT0_union_tag { /* Station interface receive octets counter (ifInOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_LOW:32;
  } B;
} NETC_F3_SI7_SIROCT0_tag;

typedef union NETC_F3_SI7_SIROCT1_union_tag { /* Station interface receive octets counter (ifInOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t ROCT_HIGH:32;
  } B;
} NETC_F3_SI7_SIROCT1_tag;

typedef union NETC_F3_SI7_SIRFRM0_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_LOW:32;
  } B;
} NETC_F3_SI7_SIRFRM0_tag;

typedef union NETC_F3_SI7_SIRFRM1_union_tag { /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t RFRM_HIGH:32;
  } B;
} NETC_F3_SI7_SIRFRM1_tag;

typedef union NETC_F3_SI7_SIRUCA0_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_LOW:32;
  } B;
} NETC_F3_SI7_SIRUCA0_tag;

typedef union NETC_F3_SI7_SIRUCA1_union_tag { /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RUCA_HIGH:32;
  } B;
} NETC_F3_SI7_SIRUCA1_tag;

typedef union NETC_F3_SI7_SIRMCA0_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_LOW:32;
  } B;
} NETC_F3_SI7_SIRMCA0_tag;

typedef union NETC_F3_SI7_SIRMCA1_union_tag { /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t RMCA_HIGH:32;
  } B;
} NETC_F3_SI7_SIRMCA1_tag;

typedef union NETC_F3_SI7_SITOCT0_union_tag { /* Station interface transmit octets counter (ifOutOctets) 0 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_LOW:32;
  } B;
} NETC_F3_SI7_SITOCT0_tag;

typedef union NETC_F3_SI7_SITOCT1_union_tag { /* Station interface transmit octets counter (ifOutOctets) 1 */
  vuint32_t R;
  struct {
    vuint32_t TOCT_HIGH:32;
  } B;
} NETC_F3_SI7_SITOCT1_tag;

typedef union NETC_F3_SI7_SITFRM0_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_LOW:32;
  } B;
} NETC_F3_SI7_SITFRM0_tag;

typedef union NETC_F3_SI7_SITFRM1_union_tag { /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  vuint32_t R;
  struct {
    vuint32_t TFRM_HIGH:32;
  } B;
} NETC_F3_SI7_SITFRM1_tag;

typedef union NETC_F3_SI7_SITUCA0_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_LOW:32;
  } B;
} NETC_F3_SI7_SITUCA0_tag;

typedef union NETC_F3_SI7_SITUCA1_union_tag { /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TUCA_HIGH:32;
  } B;
} NETC_F3_SI7_SITUCA1_tag;

typedef union NETC_F3_SI7_SITMCA0_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_LOW:32;
  } B;
} NETC_F3_SI7_SITMCA0_tag;

typedef union NETC_F3_SI7_SITMCA1_union_tag { /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  vuint32_t R;
  struct {
    vuint32_t TMCA_HIGH:32;
  } B;
} NETC_F3_SI7_SITMCA1_tag;

typedef union NETC_F3_SI7_SIBLPR_union_tag { /* Station interface boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_F3_SI7_SIBLPR_tag;

typedef union NETC_F3_SI7_SICBDRMR_union_tag { /* Station interface command BDR mode register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI7_SICBDRMR_tag;

typedef union NETC_F3_SI7_SICBDRSR_union_tag { /* Station interface command BDR status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI7_SICBDRSR_tag;

typedef union NETC_F3_SI7_SICBDRBAR0_union_tag { /* Station interface command BDR base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI7_SICBDRBAR0_tag;

typedef union NETC_F3_SI7_SICBDRBAR1_union_tag { /* Station interface command BDR base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI7_SICBDRBAR1_tag;

typedef union NETC_F3_SI7_SICBDRPIR_union_tag { /* Station interface command BDR producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI7_SICBDRPIR_tag;

typedef union NETC_F3_SI7_SICBDRCIR_union_tag { /* Station interface command BDR consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:10;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI7_SICBDRCIR_tag;

typedef union NETC_F3_SI7_SICBDRLENR_union_tag { /* Station interface command BDR length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:8;
    vuint32_t _unused_11:21;
  } B;
} NETC_F3_SI7_SICBDRLENR_tag;

typedef union NETC_F3_SI7_SICBDRIER_union_tag { /* Station interface command BDR interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t CBDCIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI7_SICBDRIER_tag;

typedef union NETC_F3_SI7_SICBDRIDR_union_tag { /* Station interface command BDR interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t CBDC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI7_SICBDRIDR_tag;

typedef union NETC_F3_SI7_SICAPR0_union_tag { /* Station interface capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_TX_BDR:8;
    vuint32_t _unused_8:8;
    vuint32_t NUM_RX_BDR:8;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} NETC_F3_SI7_SICAPR0_tag;

typedef union NETC_F3_SI7_SICAPR1_union_tag { /* Station interface capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t NUM_RX_GRP:8;
    vuint32_t _unused_24:8;
  } B;
} NETC_F3_SI7_SICAPR1_tag;

typedef union NETC_F3_SI7_SICAPR2_union_tag { /* Station interface capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t VTP:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_F3_SI7_SICAPR2_tag;

typedef union NETC_F3_SI7_VSI_VSIIER_union_tag { /* Virtual station interface interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t MSIE:1;
    vuint32_t MRIE:1;
    vuint32_t _unused_10:22;
  } B;
} NETC_F3_SI7_VSI_VSIIER_tag;

typedef union NETC_F3_SI7_VSI_VSIIDR_union_tag { /* Virtual station interface interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXR:1;
    vuint32_t _unused_1:7;
    vuint32_t MS:1;
    vuint32_t MR:1;
    vuint32_t _unused_10:6;
    vuint32_t RXR:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI7_VSI_VSIIDR_tag;

typedef struct NETC_F3_SI7_VSI_struct_tag {
  NETC_F3_SI7_VSI_VSIIER_tag VSIIER;   /* Virtual station interface interrupt enable register */
  uint8_t VSI_reserved0[4];
  NETC_F3_SI7_VSI_VSIIDR_tag VSIIDR;   /* Virtual station interface interrupt detect register */
} NETC_F3_SI7_VSI_tag;

typedef union NETC_F3_SI7_SITXIDR0_union_tag { /* Station interface transmit interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXT0:1;
    vuint32_t TXT1:1;
    vuint32_t TXT2:1;
    vuint32_t TXT3:1;
    vuint32_t TXT4:1;
    vuint32_t TXT5:1;
    vuint32_t TXT6:1;
    vuint32_t TXT7:1;
    vuint32_t TXT8:1;
    vuint32_t TXT9:1;
    vuint32_t TXT10:1;
    vuint32_t TXT11:1;
    vuint32_t TXT12:1;
    vuint32_t TXT13:1;
    vuint32_t TXT14:1;
    vuint32_t TXT15:1;
    vuint32_t TXF0:1;
    vuint32_t TXF1:1;
    vuint32_t TXF2:1;
    vuint32_t TXF3:1;
    vuint32_t TXF4:1;
    vuint32_t TXF5:1;
    vuint32_t TXF6:1;
    vuint32_t TXF7:1;
    vuint32_t TXF8:1;
    vuint32_t TXF9:1;
    vuint32_t TXF10:1;
    vuint32_t TXF11:1;
    vuint32_t TXF12:1;
    vuint32_t TXF13:1;
    vuint32_t TXF14:1;
    vuint32_t TXF15:1;
  } B;
} NETC_F3_SI7_SITXIDR0_tag;

typedef union NETC_F3_SI7_SITXIDR1_union_tag { /* Station interface transmit interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t TXT16:1;
    vuint32_t TXT17:1;
    vuint32_t _unused_2:14;
    vuint32_t TXF16:1;
    vuint32_t TXF17:1;
    vuint32_t _unused_18:14;
  } B;
} NETC_F3_SI7_SITXIDR1_tag;

typedef union NETC_F3_SI7_SIRXIDR0_union_tag { /* Station interface receive interrupt detect register 0 */
  vuint32_t R;
  struct {
    vuint32_t RX0:1;
    vuint32_t RX1:1;
    vuint32_t RX2:1;
    vuint32_t RX3:1;
    vuint32_t RX4:1;
    vuint32_t RX5:1;
    vuint32_t RX6:1;
    vuint32_t RX7:1;
    vuint32_t RX8:1;
    vuint32_t RX9:1;
    vuint32_t RX10:1;
    vuint32_t RX11:1;
    vuint32_t RX12:1;
    vuint32_t RX13:1;
    vuint32_t RX14:1;
    vuint32_t RX15:1;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI7_SIRXIDR0_tag;

typedef union NETC_F3_SI7_SIRXIDR1_union_tag { /* Station interface receive interrupt detect register 1 */
  vuint32_t R;
  struct {
    vuint32_t RX16:1;
    vuint32_t RX17:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI7_SIRXIDR1_tag;

typedef union NETC_F3_SI7_SIMSIVR_union_tag { /* Station interface MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI7_SIMSIVR_tag;

typedef union NETC_F3_SI7_SICMSIVR_union_tag { /* Station interface command MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI7_SICMSIVR_tag;

typedef union NETC_F3_SI7_SITMRIER_union_tag { /* Station interface timer interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t SYNCE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI7_SITMRIER_tag;

typedef union NETC_F3_SI7_SITMRIDR_union_tag { /* Station interface timer interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI7_SITMRIDR_tag;

typedef union NETC_F3_SI7_SITMRMSIVR_union_tag { /* Station interface timer MSI-X vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI7_SITMRMSIVR_tag;

typedef union NETC_F3_SI7_SIMSITRVR_union_tag { /* Station interface MSI-X transmit ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI7_SIMSITRVR_tag;

typedef union NETC_F3_SI7_SIMSIRRVR_union_tag { /* Station interface MSI-X receive ring 0 vector register */
  vuint32_t R;
  struct {
    vuint32_t VECTOR:6;
    vuint32_t _unused_6:26;
  } B;
} NETC_F3_SI7_SIMSIRRVR_tag;

typedef union NETC_F3_SI7_SICMECR_union_tag { /* Station interface correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI7_SICMECR_tag;

typedef union NETC_F3_SI7_SICMESR_union_tag { /* Station interface correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_F3_SI7_SICMESR_tag;

typedef union NETC_F3_SI7_SICMECTR_union_tag { /* Station interface correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI7_SICMECTR_tag;

typedef union NETC_F3_SI7_SIUPECR_union_tag { /* Station interface uncorrectable programming error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI7_SIUPECR_tag;

typedef union NETC_F3_SI7_SIUPESR_union_tag { /* Station interface uncorrectable programming error status register */
  vuint32_t R;
  struct {
    vuint32_t DROP_SI_EN:1;
    vuint32_t DROP_RING_EN:1;
    vuint32_t DROP_GRP_SEL:1;
    vuint32_t DROP_RING_SEL:1;
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t PE:1;                    /* Programming error */
  } B;
} NETC_F3_SI7_SIUPESR_tag;

typedef union NETC_F3_SI7_SIUPECTR_union_tag { /* Station interface uncorrectable programming error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;                /* Count */
  } B;
} NETC_F3_SI7_SIUPECTR_tag;

typedef union NETC_F3_SI7_SIUNSBECR_union_tag { /* Station interface uncorrectable non-fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI7_SIUNSBECR_tag;

typedef union NETC_F3_SI7_SIUNSBESR_union_tag { /* Station interface uncorrectable non-fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:27;
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI7_SIUNSBESR_tag;

typedef union NETC_F3_SI7_SIUNSBECTR_union_tag { /* Station interface uncorrectable non-fatal system bus error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI7_SIUNSBECTR_tag;

typedef union NETC_F3_SI7_SIUFSBECR_union_tag { /* Station interface uncorrectable fatal system bus error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI7_SIUFSBECR_tag;

typedef union NETC_F3_SI7_SIUFSBESR_union_tag { /* Station interface uncorrectable fatal system bus error status register */
  vuint32_t R;
  struct {
    vuint32_t SB_ID:4;                 /* System Bus ID */
    vuint32_t _unused_4:26;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t SBE:1;                   /* System bus error */
  } B;
} NETC_F3_SI7_SIUFSBESR_tag;

typedef union NETC_F3_SI7_SIUNMECR_union_tag { /* Station interface uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI7_SIUNMECR_tag;

typedef union NETC_F3_SI7_SIUNMESR0_union_tag { /* Station interface uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI7_SIUNMESR0_tag;

typedef union NETC_F3_SI7_SIUNMESR1_union_tag { /* Station interface uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI7_SIUNMESR1_tag;

typedef union NETC_F3_SI7_SIUNMECTR_union_tag { /* Station interface uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI7_SIUNMECTR_tag;

typedef union NETC_F3_SI7_SIUFMECR_union_tag { /* Station interface uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI7_SIUFMECR_tag;

typedef union NETC_F3_SI7_SIUFMESR0_union_tag { /* Station interface uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_F3_SI7_SIUFMESR0_tag;

typedef union NETC_F3_SI7_SIUFMESR1_union_tag { /* Station interface uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_F3_SI7_SIUFMESR1_tag;

typedef union NETC_F3_SI7_SIUNIECR_union_tag { /* Station interface uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI7_SIUNIECR_tag;

typedef union NETC_F3_SI7_SIUNIESR_union_tag { /* Station interface uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:17;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI7_SIUNIESR_tag;

typedef union NETC_F3_SI7_SIUNIECTR_union_tag { /* Station interface uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI7_SIUNIECTR_tag;

typedef union NETC_F3_SI7_SIUFIECR_union_tag { /* Station interface uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_F3_SI7_SIUFIECR_tag;

typedef union NETC_F3_SI7_SIUFIESR_union_tag { /* Station interface uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:16;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_F3_SI7_SIUFIESR_tag;

typedef union NETC_F3_SI7_SIMAFTCAPR_union_tag { /* Station interface MAC address filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI7_SIMAFTCAPR_tag;

typedef union NETC_F3_SI7_SIVFTCAPR_union_tag { /* Station interface VLAN filter table capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VLAN_FTE:8;
    vuint32_t _unused_8:24;
  } B;
} NETC_F3_SI7_SIVFTCAPR_tag;

typedef union NETC_F3_SI7_SIRFSCAPR_union_tag { /* Station interface RFS capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_RFS:9;
    vuint32_t _unused_9:23;
  } B;
} NETC_F3_SI7_SIRFSCAPR_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBMR_union_tag { /* Tx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t PRIO:3;
    vuint32_t _unused_3:1;
    vuint32_t WRR:3;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t VIH:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI7_BDR_NUM_TBMR_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBSR_union_tag { /* Tx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI7_BDR_NUM_TBSR_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBBAR0_union_tag { /* Tx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI7_BDR_NUM_TBBAR0_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBBAR1_union_tag { /* Tx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI7_BDR_NUM_TBBAR1_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBPIR_union_tag { /* Tx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI7_BDR_NUM_TBPIR_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBCIR_union_tag { /* Tx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t STAT_ID:16;
  } B;
} NETC_F3_SI7_BDR_NUM_TBCIR_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBLENR_union_tag { /* Tx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI7_BDR_NUM_TBLENR_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBIER_union_tag { /* Tx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t TXTIE:1;
    vuint32_t TXFIE:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI7_BDR_NUM_TBIER_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBIDR_union_tag { /* Tx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t TXT:1;
    vuint32_t TXF:1;
    vuint32_t _unused_2:30;
  } B;
} NETC_F3_SI7_BDR_NUM_TBIDR_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBICR0_union_tag { /* Tx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:4;
    vuint32_t _unused_4:27;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI7_BDR_NUM_TBICR0_tag;

typedef union NETC_F3_SI7_BDR_NUM_TBICR1_union_tag { /* Tx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI7_BDR_NUM_TBICR1_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBMR_union_tag { /* Rx BDR 0 mode register */
  vuint32_t R;
  struct {
    vuint32_t AL:1;
    vuint32_t _unused_1:1;
    vuint32_t BDS:1;
    vuint32_t _unused_3:1;
    vuint32_t CM:1;
    vuint32_t VTE:1;
    vuint32_t VTPD:1;
    vuint32_t _unused_7:1;
    vuint32_t CRC:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:21;
    vuint32_t EN:1;
  } B;
} NETC_F3_SI7_BDR_NUM_RBMR_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBSR_union_tag { /* Rx BDR 0 status register */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;
    vuint32_t _unused_1:15;
    vuint32_t SBE:1;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI7_BDR_NUM_RBSR_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBBSR_union_tag { /* Rx BDR 0 buffer size register */
  vuint32_t R;
  struct {
    vuint32_t BSIZE:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI7_BDR_NUM_RBBSR_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBCIR_union_tag { /* Rx BDR 0 consumer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI7_BDR_NUM_RBCIR_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBBAR0_union_tag { /* Rx BDR 0 base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t ADDRL:25;
  } B;
} NETC_F3_SI7_BDR_NUM_RBBAR0_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBBAR1_union_tag { /* Rx BDR 0 base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDRH:32;
  } B;
} NETC_F3_SI7_BDR_NUM_RBBAR1_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBPIR_union_tag { /* Rx BDR 0 producer index register */
  vuint32_t R;
  struct {
    vuint32_t BDR_INDEX:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_F3_SI7_BDR_NUM_RBPIR_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBLENR_union_tag { /* Rx BDR 0 length register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t LENGTH:14;
    vuint32_t _unused_17:15;
  } B;
} NETC_F3_SI7_BDR_NUM_RBLENR_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBDCR_union_tag { /* Rx BDR 0 drop count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} NETC_F3_SI7_BDR_NUM_RBDCR_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBIER_union_tag { /* Rx BDR 0 interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t RXTIE:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI7_BDR_NUM_RBIER_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBIDR_union_tag { /* Rx BDR 0 interrupt detect register */
  vuint32_t R;
  struct {
    vuint32_t RXT:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_F3_SI7_BDR_NUM_RBIDR_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBICR0_union_tag { /* Rx BDR 0 interrupt coalescing register 0 */
  vuint32_t R;
  struct {
    vuint32_t ICPT:9;
    vuint32_t _unused_9:22;
    vuint32_t ICEN:1;
  } B;
} NETC_F3_SI7_BDR_NUM_RBICR0_tag;

typedef union NETC_F3_SI7_BDR_NUM_RBICR1_union_tag { /* Rx BDR 0 interrupt coalescing register 1 */
  vuint32_t R;
  struct {
    vuint32_t ICTT:32;
  } B;
} NETC_F3_SI7_BDR_NUM_RBICR1_tag;

typedef struct NETC_F3_SI7_BDR_NUM_struct_tag {
  NETC_F3_SI7_BDR_NUM_TBMR_tag TBMR;   /* Tx BDR 0 mode register */
  NETC_F3_SI7_BDR_NUM_TBSR_tag TBSR;   /* Tx BDR 0 status register */
  uint8_t BDR_NUM_reserved0[8];
  NETC_F3_SI7_BDR_NUM_TBBAR0_tag TBBAR0; /* Tx BDR 0 base address register 0 */
  NETC_F3_SI7_BDR_NUM_TBBAR1_tag TBBAR1; /* Tx BDR 0 base address register 1 */
  NETC_F3_SI7_BDR_NUM_TBPIR_tag TBPIR; /* Tx BDR 0 producer index register */
  NETC_F3_SI7_BDR_NUM_TBCIR_tag TBCIR; /* Tx BDR 0 consumer index register */
  NETC_F3_SI7_BDR_NUM_TBLENR_tag TBLENR; /* Tx BDR 0 length register */
  uint8_t BDR_NUM_reserved1[124];
  NETC_F3_SI7_BDR_NUM_TBIER_tag TBIER; /* Tx BDR 0 interrupt enable register */
  NETC_F3_SI7_BDR_NUM_TBIDR_tag TBIDR; /* Tx BDR 0 interrupt detect register */
  NETC_F3_SI7_BDR_NUM_TBICR0_tag TBICR0; /* Tx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI7_BDR_NUM_TBICR1_tag TBICR1; /* Tx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved2[80];
  NETC_F3_SI7_BDR_NUM_RBMR_tag RBMR;   /* Rx BDR 0 mode register */
  NETC_F3_SI7_BDR_NUM_RBSR_tag RBSR;   /* Rx BDR 0 status register */
  NETC_F3_SI7_BDR_NUM_RBBSR_tag RBBSR; /* Rx BDR 0 buffer size register */
  NETC_F3_SI7_BDR_NUM_RBCIR_tag RBCIR; /* Rx BDR 0 consumer index register */
  NETC_F3_SI7_BDR_NUM_RBBAR0_tag RBBAR0; /* Rx BDR 0 base address register 0 */
  NETC_F3_SI7_BDR_NUM_RBBAR1_tag RBBAR1; /* Rx BDR 0 base address register 1 */
  NETC_F3_SI7_BDR_NUM_RBPIR_tag RBPIR; /* Rx BDR 0 producer index register */
  uint8_t BDR_NUM_reserved3[4];
  NETC_F3_SI7_BDR_NUM_RBLENR_tag RBLENR; /* Rx BDR 0 length register */
  uint8_t BDR_NUM_reserved4[92];
  NETC_F3_SI7_BDR_NUM_RBDCR_tag RBDCR; /* Rx BDR 0 drop count register */
  uint8_t BDR_NUM_reserved5[28];
  NETC_F3_SI7_BDR_NUM_RBIER_tag RBIER; /* Rx BDR 0 interrupt enable register */
  NETC_F3_SI7_BDR_NUM_RBIDR_tag RBIDR; /* Rx BDR 0 interrupt detect register */
  NETC_F3_SI7_BDR_NUM_RBICR0_tag RBICR0; /* Rx BDR 0 interrupt coalescing register 0 */
  NETC_F3_SI7_BDR_NUM_RBICR1_tag RBICR1; /* Rx BDR 0 interrupt coalescing register 1 */
  uint8_t BDR_NUM_reserved6[80];
} NETC_F3_SI7_BDR_NUM_tag;

struct NETC_F3_SI7_tag {
  NETC_F3_SI7_SIMR_tag SIMR;           /* Station interface mode register */
  NETC_F3_SI7_SISR_tag SISR;           /* Station interface status register */
  uint8_t NETC_F3_SI7_reserved0[16];
  NETC_F3_SI7_SICTR0_tag SICTR0;       /* Station interface current time register 0 */
  NETC_F3_SI7_SICTR1_tag SICTR1;       /* Station interface current time register 1 */
  NETC_F3_SI7_SIPCAPR0_tag SIPCAPR0;   /* Station interface port capability register 0 */
  NETC_F3_SI7_SIPCAPR1_tag SIPCAPR1;   /* Station interface port capability register 1 */
  uint8_t NETC_F3_SI7_reserved1[16];
  NETC_F3_SI7_SIRBGCR_tag SIRBGCR;     /* Station interface receive BDR group control register */
  uint8_t NETC_F3_SI7_reserved2[4];
  NETC_F3_SI7_SIBCAR_tag SIBCAR;       /* Station interface buffer cache attribute register */
  uint8_t NETC_F3_SI7_reserved3[4];
  NETC_F3_SI7_SICCAR_tag SICCAR;       /* Station interface command cache attribute register */
  uint8_t NETC_F3_SI7_reserved4[52];
  NETC_F3_SI7_SIPMAR0_tag SIPMAR0;     /* Station interface primary MAC address register 0 */
  NETC_F3_SI7_SIPMAR1_tag SIPMAR1;     /* Station interface primary MAC address register 1 */
  uint8_t NETC_F3_SI7_reserved5[8];
  NETC_F3_SI7_SICVLANR1_tag SICVLANR1; /* Station interface custom VLAN register 1 */
  NETC_F3_SI7_SICVLANR2_tag SICVLANR2; /* Station interface custom VLAN register 2 */
  uint8_t NETC_F3_SI7_reserved6[104];
  NETC_F3_SI7_SIVLANIPVMR0_tag SIVLANIPVMR0; /* Station interface VLAN to IPV mapping register 0 */
  NETC_F3_SI7_SIVLANIPVMR1_tag SIVLANIPVMR1; /* Station interface VLAN to IPV mapping register 1 */
  uint8_t NETC_F3_SI7_reserved7[72];
  NETC_F3_SI7_SIIPVBDRMR0_tag SIIPVBDRMR0; /* Station interface IPV to ring mapping register */
  uint8_t NETC_F3_SI7_reserved8[176];
  NETC_F3_SI7_VSI_A_tag VSI_A;
  uint8_t NETC_F3_SI7_reserved9[232];
  NETC_F3_SI7_SIROCT0_tag SIROCT0;     /* Station interface receive octets counter (ifInOctets) 0 */
  NETC_F3_SI7_SIROCT1_tag SIROCT1;     /* Station interface receive octets counter (ifInOctets) 1 */
  NETC_F3_SI7_SIRFRM0_tag SIRFRM0;     /* Station interface receive frame counter (aFrameReceivedOK) 0 */
  NETC_F3_SI7_SIRFRM1_tag SIRFRM1;     /* Station interface receive frame counter (aFrameReceivedOK) 1 */
  NETC_F3_SI7_SIRUCA0_tag SIRUCA0;     /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
  NETC_F3_SI7_SIRUCA1_tag SIRUCA1;     /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
  NETC_F3_SI7_SIRMCA0_tag SIRMCA0;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
  NETC_F3_SI7_SIRMCA1_tag SIRMCA1;     /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
  NETC_F3_SI7_SITOCT0_tag SITOCT0;     /* Station interface transmit octets counter (ifOutOctets) 0 */
  NETC_F3_SI7_SITOCT1_tag SITOCT1;     /* Station interface transmit octets counter (ifOutOctets) 1 */
  NETC_F3_SI7_SITFRM0_tag SITFRM0;     /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
  NETC_F3_SI7_SITFRM1_tag SITFRM1;     /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
  NETC_F3_SI7_SITUCA0_tag SITUCA0;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
  NETC_F3_SI7_SITUCA1_tag SITUCA1;     /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
  NETC_F3_SI7_SITMCA0_tag SITMCA0;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
  NETC_F3_SI7_SITMCA1_tag SITMCA1;     /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
  uint8_t NETC_F3_SI7_reserved10[176];
  NETC_F3_SI7_SIBLPR_tag SIBLPR[2];    /* Station interface boot loader parameter register 0 */
  uint8_t NETC_F3_SI7_reserved11[1032];
  NETC_F3_SI7_SICBDRMR_tag SICBDRMR;   /* Station interface command BDR mode register */
  NETC_F3_SI7_SICBDRSR_tag SICBDRSR;   /* Station interface command BDR status register */
  uint8_t NETC_F3_SI7_reserved12[8];
  NETC_F3_SI7_SICBDRBAR0_tag SICBDRBAR0; /* Station interface command BDR base address register 0 */
  NETC_F3_SI7_SICBDRBAR1_tag SICBDRBAR1; /* Station interface command BDR base address register 1 */
  NETC_F3_SI7_SICBDRPIR_tag SICBDRPIR; /* Station interface command BDR producer index register */
  NETC_F3_SI7_SICBDRCIR_tag SICBDRCIR; /* Station interface command BDR consumer index register */
  NETC_F3_SI7_SICBDRLENR_tag SICBDRLENR; /* Station interface command BDR length register */
  uint8_t NETC_F3_SI7_reserved13[124];
  NETC_F3_SI7_SICBDRIER_tag SICBDRIER; /* Station interface command BDR interrupt enable register */
  NETC_F3_SI7_SICBDRIDR_tag SICBDRIDR; /* Station interface command BDR interrupt detect register */
  uint8_t NETC_F3_SI7_reserved14[88];
  NETC_F3_SI7_SICAPR0_tag SICAPR0;     /* Station interface capability register 0 */
  NETC_F3_SI7_SICAPR1_tag SICAPR1;     /* Station interface capability register 1 */
  NETC_F3_SI7_SICAPR2_tag SICAPR2;     /* Station interface capability register 2 */
  uint8_t NETC_F3_SI7_reserved15[244];
  NETC_F3_SI7_VSI_tag VSI;
  uint8_t NETC_F3_SI7_reserved16[12];
  NETC_F3_SI7_SITXIDR0_tag SITXIDR0;   /* Station interface transmit interrupt detect register 0 */
  NETC_F3_SI7_SITXIDR1_tag SITXIDR1;   /* Station interface transmit interrupt detect register 1 */
  uint8_t NETC_F3_SI7_reserved17[8];
  NETC_F3_SI7_SIRXIDR0_tag SIRXIDR0;   /* Station interface receive interrupt detect register 0 */
  NETC_F3_SI7_SIRXIDR1_tag SIRXIDR1;   /* Station interface receive interrupt detect register 1 */
  NETC_F3_SI7_SIMSIVR_tag SIMSIVR;     /* Station interface MSI-X vector register */
  NETC_F3_SI7_SICMSIVR_tag SICMSIVR;   /* Station interface command MSI-X vector register */
  uint8_t NETC_F3_SI7_reserved18[8];
  NETC_F3_SI7_SITMRIER_tag SITMRIER;   /* Station interface timer interrupt enable register */
  NETC_F3_SI7_SITMRIDR_tag SITMRIDR;   /* Station interface timer interrupt detect register */
  uint8_t NETC_F3_SI7_reserved19[4];
  NETC_F3_SI7_SITMRMSIVR_tag SITMRMSIVR; /* Station interface timer MSI-X vector register */
  uint8_t NETC_F3_SI7_reserved20[176];
  NETC_F3_SI7_SIMSITRVR_tag SIMSITRVR[18]; /* Station interface MSI-X transmit ring 0 vector register */
  uint8_t NETC_F3_SI7_reserved21[56];
  NETC_F3_SI7_SIMSIRRVR_tag SIMSIRRVR[18]; /* Station interface MSI-X receive ring 0 vector register */
  uint8_t NETC_F3_SI7_reserved22[568];
  NETC_F3_SI7_SICMECR_tag SICMECR;     /* Station interface correctable memory error configuration register */
  NETC_F3_SI7_SICMESR_tag SICMESR;     /* Station interface correctable memory error status register */
  uint8_t NETC_F3_SI7_reserved23[4];
  NETC_F3_SI7_SICMECTR_tag SICMECTR;   /* Station interface correctable memory error count register */
  NETC_F3_SI7_SIUPECR_tag SIUPECR;     /* Station interface uncorrectable programming error configuration register */
  NETC_F3_SI7_SIUPESR_tag SIUPESR;     /* Station interface uncorrectable programming error status register */
  uint8_t NETC_F3_SI7_reserved24[4];
  NETC_F3_SI7_SIUPECTR_tag SIUPECTR;   /* Station interface uncorrectable programming error count register */
  NETC_F3_SI7_SIUNSBECR_tag SIUNSBECR; /* Station interface uncorrectable non-fatal system bus error configuration register */
  NETC_F3_SI7_SIUNSBESR_tag SIUNSBESR; /* Station interface uncorrectable non-fatal system bus error status register */
  uint8_t NETC_F3_SI7_reserved25[4];
  NETC_F3_SI7_SIUNSBECTR_tag SIUNSBECTR; /* Station interface uncorrectable non-fatal system bus error count register */
  NETC_F3_SI7_SIUFSBECR_tag SIUFSBECR; /* Station interface uncorrectable fatal system bus error configuration register */
  NETC_F3_SI7_SIUFSBESR_tag SIUFSBESR; /* Station interface uncorrectable fatal system bus error status register */
  uint8_t NETC_F3_SI7_reserved26[8];
  NETC_F3_SI7_SIUNMECR_tag SIUNMECR;   /* Station interface uncorrectable non-fatal memory error configuration register */
  NETC_F3_SI7_SIUNMESR0_tag SIUNMESR0; /* Station interface uncorrectable non-fatal memory error status register 0 */
  NETC_F3_SI7_SIUNMESR1_tag SIUNMESR1; /* Station interface uncorrectable non-fatal memory error status register 1 */
  NETC_F3_SI7_SIUNMECTR_tag SIUNMECTR; /* Station interface uncorrectable non-fatal memory error count register */
  NETC_F3_SI7_SIUFMECR_tag SIUFMECR;   /* Station interface uncorrectable fatal memory error configuration register */
  NETC_F3_SI7_SIUFMESR0_tag SIUFMESR0; /* Station interface uncorrectable fatal memory error status register 0 */
  NETC_F3_SI7_SIUFMESR1_tag SIUFMESR1; /* Station interface uncorrectable fatal memory error status register 1 */
  uint8_t NETC_F3_SI7_reserved27[4];
  NETC_F3_SI7_SIUNIECR_tag SIUNIECR;   /* Station interface uncorrectable non-fatal integrity error configuration register */
  NETC_F3_SI7_SIUNIESR_tag SIUNIESR;   /* Station interface uncorrectable non-fatal integrity error status register */
  uint8_t NETC_F3_SI7_reserved28[4];
  NETC_F3_SI7_SIUNIECTR_tag SIUNIECTR; /* Station interface uncorrectable non-fatal integrity error count register */
  NETC_F3_SI7_SIUFIECR_tag SIUFIECR;   /* Station interface uncorrectable fatal integrity error configuration register */
  NETC_F3_SI7_SIUFIESR_tag SIUFIESR;   /* Station interface uncorrectable fatal integrity error status register */
  uint8_t NETC_F3_SI7_reserved29[392];
  NETC_F3_SI7_SIMAFTCAPR_tag SIMAFTCAPR; /* Station interface MAC address filter table capability register */
  uint8_t NETC_F3_SI7_reserved30[252];
  NETC_F3_SI7_SIVFTCAPR_tag SIVFTCAPR; /* Station interface VLAN filter table capability register */
  uint8_t NETC_F3_SI7_reserved31[252];
  NETC_F3_SI7_SIRFSCAPR_tag SIRFSCAPR; /* Station interface RFS capability register */
  uint8_t NETC_F3_SI7_reserved32[28156];
  NETC_F3_SI7_BDR_NUM_tag BDR_NUM[18];
};


/* ============================================================================
   =============================== Module: NETC_IERB ==========================
   ============================================================================ */

typedef union NETC_IERB_CAPR0_union_tag { /* Capability register 0 */
  vuint32_t R;
  struct {
    vuint32_t NUM_RC:4;
    vuint32_t NUM_EMDIO:1;
    vuint32_t _unused_5:1;
    vuint32_t NUM_TMR:2;
    vuint32_t NUM_LINKS:5;
    vuint32_t _unused_13:3;
    vuint32_t NUM_SW:2;
    vuint32_t _unused_18:1;
    vuint32_t NUM_ENETC:5;
    vuint32_t NUM_VSI:7;
    vuint32_t _unused_31:1;
  } B;
} NETC_IERB_CAPR0_tag;

typedef union NETC_IERB_CAPR1_union_tag { /* Capability register 1 */
  vuint32_t R;
  struct {
    vuint32_t NUM_RX_BDR:10;
    vuint32_t _unused_10:6;
    vuint32_t NUM_TX_BDR:10;
    vuint32_t _unused_26:6;
  } B;
} NETC_IERB_CAPR1_tag;

typedef union NETC_IERB_CAPR2_union_tag { /* Capability register 2 */
  vuint32_t R;
  struct {
    vuint32_t NUM_MSIX:11;
    vuint32_t _unused_11:21;
  } B;
} NETC_IERB_CAPR2_tag;

typedef union NETC_IERB_CAPR3_union_tag { /* Capability register 3 */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:12;
    vuint32_t _unused_12:4;
    vuint32_t NUM_VLAN_FTE:12;
    vuint32_t _unused_28:4;
  } B;
} NETC_IERB_CAPR3_tag;

typedef union NETC_IERB_CMCAPR_union_tag { /* Common memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:24;
    vuint32_t _unused_24:4;
    vuint32_t WORD_SIZE:2;
    vuint32_t _unused_30:2;
  } B;
} NETC_IERB_CMCAPR_tag;

typedef union NETC_IERB_IPFTMCAPR_union_tag { /* Ingress port filter ternary memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:12;
    vuint32_t WORD_SIZE:2;
    vuint32_t _unused_30:2;
  } B;
} NETC_IERB_IPFTMCAPR_tag;

typedef union NETC_IERB_TGSMCAPR_union_tag { /* Time gate scheduling memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_TGSMCAPR_tag;

typedef union NETC_IERB_SMDTR_union_tag { /* Shared memory depletion threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_IERB_SMDTR_tag;

typedef union NETC_IERB_ERSMBAR_union_tag { /* ENETC receive shared memory buffer allotment register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_IERB_ERSMBAR_tag;

typedef union NETC_IERB_HTA_NUM_HTAHPCR_union_tag { /* HTA 0 HP configuration register */
  vuint32_t R;
  struct {
    vuint32_t BLIMIT:16;
    vuint32_t _unused_16:8;
    vuint32_t FLIMIT:8;
  } B;
} NETC_IERB_HTA_NUM_HTAHPCR_tag;

typedef union NETC_IERB_HTA_NUM_HTALPCR_union_tag { /* HTA 0 LP configuration register */
  vuint32_t R;
  struct {
    vuint32_t BLIMIT:16;
    vuint32_t _unused_16:8;
    vuint32_t FLIMIT:8;
  } B;
} NETC_IERB_HTA_NUM_HTALPCR_tag;

typedef struct NETC_IERB_HTA_NUM_struct_tag {
  NETC_IERB_HTA_NUM_HTAHPCR_tag HTAHPCR; /* HTA 0 HP configuration register */
  NETC_IERB_HTA_NUM_HTALPCR_tag HTALPCR; /* HTA 0 LP configuration register */
} NETC_IERB_HTA_NUM_tag;

typedef union NETC_IERB_HBTMAR_union_tag { /* Hash bucket table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:14;
    vuint32_t _unused_14:2;
    vuint32_t MIN_WORDS:8;
    vuint32_t NEPW:3;
    vuint32_t _unused_27:3;
    vuint32_t MLOC:2;
  } B;
} NETC_IERB_HBTMAR_tag;

typedef union NETC_IERB_HBTCR_union_tag { /* Hash bucket table configuration register */
  vuint32_t R;
  struct {
    vuint32_t MAX_COL:3;
    vuint32_t _unused_3:1;
    vuint32_t MAX_VISITS:4;
    vuint32_t _unused_8:24;
  } B;
} NETC_IERB_HBTCR_tag;

typedef union NETC_IERB_GHTEMCAPR_union_tag { /* Guaranteed hash table entry memory capability register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:9;
    vuint32_t _unused_9:21;
    vuint32_t MLOC:2;
  } B;
} NETC_IERB_GHTEMCAPR_tag;

typedef union NETC_IERB_NETCFLRCR_union_tag { /* NETC FLR configuration register */
  vuint32_t R;
  struct {
    vuint32_t VALUE:9;
    vuint32_t SCALE:3;                 /* Scale */
    vuint32_t _unused_12:20;
  } B;
} NETC_IERB_NETCFLRCR_tag;

typedef union NETC_IERB_NETCCLKFR_union_tag { /* NETC clock period fractional register */
  vuint32_t R;
  struct {
    vuint32_t FRAC:32;
  } B;
} NETC_IERB_NETCCLKFR_tag;

typedef union NETC_IERB_NETCCLKCR_union_tag { /* NETC clock configuration register */
  vuint32_t R;
  struct {
    vuint32_t FREQ:11;                 /* Frequency */
    vuint32_t _unused_11:5;
    vuint32_t PERIOD:10;               /* Period */
    vuint32_t _unused_26:6;
  } B;
} NETC_IERB_NETCCLKCR_tag;

typedef union NETC_IERB_SBCR_union_tag { /* System bus configuration register */
  vuint32_t R;
  struct {
    vuint32_t WBS:2;
    vuint32_t RBS:2;
    vuint32_t _unused_4:28;
  } B;
} NETC_IERB_SBCR_tag;

typedef union NETC_IERB_SBOTCR_union_tag { /* System bus outstanding transaction control register */
  vuint32_t R;
  struct {
    vuint32_t OT_LIMIT:32;
  } B;
} NETC_IERB_SBOTCR_tag;

typedef union NETC_IERB_SGLTTR_union_tag { /* Stream gating lag time for refresh register */
  vuint32_t R;
  struct {
    vuint32_t LAG_TIME:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_IERB_SGLTTR_tag;

typedef union NETC_IERB_ARRAY_NUM_RC_RBCR_union_tag { /* Root complex 0 binding configuration register */
  vuint32_t R;
  struct {
    vuint32_t TYPE:1;
    vuint32_t _unused_1:3;
    vuint32_t PORT:4;
    vuint32_t _unused_8:24;
  } B;
} NETC_IERB_ARRAY_NUM_RC_RBCR_tag;

typedef union NETC_IERB_ARRAY_NUM_RC_RCMSICAR_union_tag { /* Root complex 0 MSI-X cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t MSI_WRCACHE:4;
    vuint32_t MSI_WRDOMAIN:2;
    vuint32_t MSI_WRSNP:1;
    vuint32_t _unused_7:25;
  } B;
} NETC_IERB_ARRAY_NUM_RC_RCMSICAR_tag;

typedef union NETC_IERB_ARRAY_NUM_RC_RCMSIAMQR_union_tag { /* Root complex 0 MSI access management qualifier register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:20;
    vuint32_t AWQOS:4;
    vuint32_t _unused_24:7;
    vuint32_t BMT:1;
  } B;
} NETC_IERB_ARRAY_NUM_RC_RCMSIAMQR_tag;

typedef struct NETC_IERB_ARRAY_NUM_RC_struct_tag {
  NETC_IERB_ARRAY_NUM_RC_RBCR_tag RBCR; /* Root complex 0 binding configuration register */
  uint8_t ARRAY_NUM_RC_reserved0[4];
  NETC_IERB_ARRAY_NUM_RC_RCMSICAR_tag RCMSICAR; /* Root complex 0 MSI-X cache attribute register */
  NETC_IERB_ARRAY_NUM_RC_RCMSIAMQR_tag RCMSIAMQR; /* Root complex 0 MSI access management qualifier register */
} NETC_IERB_ARRAY_NUM_RC_tag;

typedef union NETC_IERB_EMDIOBCR_union_tag { /* EMDIO binding configuration register */
  vuint32_t R;
  struct {
    vuint32_t RC_INST:4;
    vuint32_t _unused_4:4;
    vuint32_t FN:4;
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;
  } B;
} NETC_IERB_EMDIOBCR_tag;

typedef union NETC_IERB_EMDIOMCR_union_tag { /* EMDIO MSI-X configuration register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MSIX:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_IERB_EMDIOMCR_tag;

typedef union NETC_IERB_EMDIO_CFH_DIDVID_union_tag { /* EMDIO config header device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_IERB_EMDIO_CFH_DIDVID_tag;

typedef union NETC_IERB_EMDIO_CFH_SIDSVID_union_tag { /* EMDIO config header subsystem ID and subsystem vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t SUBSYSTEM_VENDOR_ID:16;
    vuint32_t SUBSYSTEM_DEVICE_ID:16;
  } B;
} NETC_IERB_EMDIO_CFH_SIDSVID_tag;

typedef union NETC_IERB_EMDIOBLPR_union_tag { /* EMDIO boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_IERB_EMDIOBLPR_tag;

typedef union NETC_IERB_EMDIO_CFG_union_tag { /* EMDIO configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t MDIO_MODE:1;
    vuint32_t MDC_MODE:1;
    vuint32_t _unused_6:26;
  } B;
} NETC_IERB_EMDIO_CFG_tag;

typedef union NETC_IERB_NUM_TMR_ARRAY_TBCR_union_tag { /* Timer 0 binding configuration register */
  vuint32_t R;
  struct {
    vuint32_t RC_INST:4;
    vuint32_t _unused_4:4;
    vuint32_t FN:4;
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;
  } B;
} NETC_IERB_NUM_TMR_ARRAY_TBCR_tag;

typedef union NETC_IERB_NUM_TMR_ARRAY_TMCR_union_tag { /* Timer 0 MSI-X configuration register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MSIX:1;
    vuint32_t _unused_1:31;
  } B;
} NETC_IERB_NUM_TMR_ARRAY_TMCR_tag;

typedef union NETC_IERB_NUM_TMR_ARRAY_T_CFH_DIDVID_union_tag { /* Timer 0 config header device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_IERB_NUM_TMR_ARRAY_T_CFH_DIDVID_tag;

typedef union NETC_IERB_NUM_TMR_ARRAY_T_CFH_SIDSVID_union_tag { /* Timer 0 config header subsystem ID and subsystem vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t SUBSYSTEM_VENDOR_ID:16;
    vuint32_t SUBSYSTEM_DEVICE_ID:16;
  } B;
} NETC_IERB_NUM_TMR_ARRAY_T_CFH_SIDSVID_tag;

typedef union NETC_IERB_NUM_TMR_ARRAY_TBLPR_union_tag { /* Timer 0 boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_IERB_NUM_TMR_ARRAY_TBLPR_tag;

typedef struct NETC_IERB_NUM_TMR_ARRAY_struct_tag {
  NETC_IERB_NUM_TMR_ARRAY_TBCR_tag TBCR; /* Timer 0 binding configuration register */
  uint8_t NUM_TMR_ARRAY_reserved0[16];
  NETC_IERB_NUM_TMR_ARRAY_TMCR_tag TMCR; /* Timer 0 MSI-X configuration register */
  uint8_t NUM_TMR_ARRAY_reserved1[8];
  NETC_IERB_NUM_TMR_ARRAY_T_CFH_DIDVID_tag T_CFH_DIDVID; /* Timer 0 config header device ID and vendor ID register */
  NETC_IERB_NUM_TMR_ARRAY_T_CFH_SIDSVID_tag T_CFH_SIDSVID; /* Timer 0 config header subsystem ID and subsystem vendor ID register */
  uint8_t NUM_TMR_ARRAY_reserved2[32];
  NETC_IERB_NUM_TMR_ARRAY_TBLPR_tag TBLPR[2]; /* Timer 0 boot loader parameter register 0 */
} NETC_IERB_NUM_TMR_ARRAY_tag;

typedef union NETC_IERB_L0CAPR_union_tag { /* Link 0 capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t LINK_TYPE:1;
    vuint32_t _unused_5:7;
    vuint32_t NUM_TC:4;
    vuint32_t NUM_Q:4;                 /* Number of Egress Traffic Management (ETM) class queues supported */
    vuint32_t _unused_20:4;
    vuint32_t NUM_CG:4;                /* Number of congestion groups supported */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t CBS:1;                   /* Credit Based Shaping */
    vuint32_t _unused_30:2;
  } B;
} NETC_IERB_L0CAPR_tag;

typedef union NETC_IERB_L0MCAPR_union_tag { /* Link 0 MAC capability register */
  vuint32_t R;
  struct {
    vuint32_t MAC_VAR:3;               /* MAC Variant */
    vuint32_t _unused_3:1;
    vuint32_t EFPAD:2;                 /* Egress frame padding capability */
    vuint32_t _unused_6:2;
    vuint32_t HD:1;                    /* Half Duplex capability */
    vuint32_t FP:2;                    /* Indicates if frame preemption is supported */
    vuint32_t _unused_11:13;
    vuint32_t MII_PROT:4;              /* Indicates the MII protocol supported */
    vuint32_t _unused_28:4;
  } B;
} NETC_IERB_L0MCAPR_tag;

typedef union NETC_IERB_L0IOCAPR_union_tag { /* Link 0 I/O capability register */
  vuint32_t R;
  struct {
    vuint32_t PCS_PROT:16;             /* PCS protocols supported */
    vuint32_t _unused_16:7;
    vuint32_t _unused_23:1;
    vuint32_t IO_VAR:4;                /* IO Variants supported */
    vuint32_t EMDIO:1;                 /* External MDIO supported. */
    vuint32_t _unused_29:1;
    vuint32_t REVMII_RATE:1;           /* RevMII MII rate */
    vuint32_t REVMII:1;                /* Reverse Mode Device Configuration */
  } B;
} NETC_IERB_L0IOCAPR_tag;

typedef union NETC_IERB_L0BCR_union_tag { /* Link 0 binding configuration register */
  vuint32_t R;
  struct {
    vuint32_t SW_PORT_ENETC_INST:5;
    vuint32_t _unused_5:1;
    vuint32_t NETC_FUNC:1;
    vuint32_t _unused_7:1;
    vuint32_t MDIO_PHYAD_PRTAD:5;
    vuint32_t _unused_13:3;
    vuint32_t SPL_SW_PORT:5;
    vuint32_t _unused_21:11;
  } B;
} NETC_IERB_L0BCR_tag;

typedef union NETC_IERB_L0TXBCCTR_union_tag { /* Link 0 transmit byte credit comfort threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_L0TXBCCTR_tag;

typedef union NETC_IERB_L0E0MAR0_union_tag { /* Link 0 end 0 MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:32;
  } B;
} NETC_IERB_L0E0MAR0_tag;

typedef union NETC_IERB_L0E0MAR1_union_tag { /* Link 0 end 0 MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_L0E0MAR1_tag;

typedef union NETC_IERB_L1CAPR_union_tag { /* Link 1 capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t LINK_TYPE:1;
    vuint32_t _unused_5:7;
    vuint32_t NUM_TC:4;
    vuint32_t NUM_Q:4;                 /* Number of Egress Traffic Management (ETM) class queues supported */
    vuint32_t _unused_20:4;
    vuint32_t NUM_CG:4;                /* Number of congestion groups supported */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t CBS:1;                   /* Credit Based Shaping */
    vuint32_t _unused_30:2;
  } B;
} NETC_IERB_L1CAPR_tag;

typedef union NETC_IERB_L1MCAPR_union_tag { /* Link 1 MAC capability register */
  vuint32_t R;
  struct {
    vuint32_t MAC_VAR:3;               /* MAC Variant */
    vuint32_t _unused_3:1;
    vuint32_t EFPAD:2;                 /* Egress frame padding capability */
    vuint32_t _unused_6:2;
    vuint32_t HD:1;                    /* Half Duplex capability */
    vuint32_t FP:2;                    /* Indicates if frame preemption is supported */
    vuint32_t _unused_11:13;
    vuint32_t MII_PROT:4;              /* Indicates the MII protocol supported */
    vuint32_t _unused_28:4;
  } B;
} NETC_IERB_L1MCAPR_tag;

typedef union NETC_IERB_L1IOCAPR_union_tag { /* Link 1 I/O capability register */
  vuint32_t R;
  struct {
    vuint32_t PCS_PROT:16;             /* PCS protocols supported */
    vuint32_t _unused_16:7;
    vuint32_t _unused_23:1;
    vuint32_t IO_VAR:4;                /* IO Variants supported */
    vuint32_t EMDIO:1;                 /* External MDIO supported. */
    vuint32_t _unused_29:1;
    vuint32_t REVMII_RATE:1;           /* RevMII MII rate */
    vuint32_t REVMII:1;                /* Reverse Mode Device Configuration */
  } B;
} NETC_IERB_L1IOCAPR_tag;

typedef union NETC_IERB_L1BCR_union_tag { /* Link 1 binding configuration register */
  vuint32_t R;
  struct {
    vuint32_t SW_PORT_ENETC_INST:5;
    vuint32_t _unused_5:1;
    vuint32_t NETC_FUNC:1;
    vuint32_t _unused_7:1;
    vuint32_t MDIO_PHYAD_PRTAD:5;
    vuint32_t _unused_13:3;
    vuint32_t SPL_SW_PORT:5;
    vuint32_t _unused_21:11;
  } B;
} NETC_IERB_L1BCR_tag;

typedef union NETC_IERB_L1TXBCCTR_union_tag { /* Link 1 transmit byte credit comfort threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_L1TXBCCTR_tag;

typedef union NETC_IERB_L1E0MAR0_union_tag { /* Link 1 end 0 MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:32;
  } B;
} NETC_IERB_L1E0MAR0_tag;

typedef union NETC_IERB_L1E0MAR1_union_tag { /* Link 1 end 0 MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_L1E0MAR1_tag;

typedef union NETC_IERB_L2CAPR_union_tag { /* Link 2 capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t LINK_TYPE:1;
    vuint32_t _unused_5:7;
    vuint32_t NUM_TC:4;
    vuint32_t NUM_Q:4;                 /* Number of Egress Traffic Management (ETM) class queues supported */
    vuint32_t _unused_20:4;
    vuint32_t NUM_CG:4;                /* Number of congestion groups supported */
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t CBS:1;                   /* Credit Based Shaping */
    vuint32_t _unused_30:2;
  } B;
} NETC_IERB_L2CAPR_tag;

typedef union NETC_IERB_L2MCAPR_union_tag { /* Link 2 MAC capability register */
  vuint32_t R;
  struct {
    vuint32_t MAC_VAR:3;               /* MAC Variant */
    vuint32_t _unused_3:1;
    vuint32_t EFPAD:2;                 /* Egress frame padding capability */
    vuint32_t _unused_6:2;
    vuint32_t HD:1;                    /* Half Duplex capability */
    vuint32_t FP:2;                    /* Indicates if frame preemption is supported */
    vuint32_t _unused_11:13;
    vuint32_t MII_PROT:4;              /* Indicates the MII protocol supported */
    vuint32_t _unused_28:4;
  } B;
} NETC_IERB_L2MCAPR_tag;

typedef union NETC_IERB_L2BCR_union_tag { /* Link 2 binding configuration register */
  vuint32_t R;
  struct {
    vuint32_t SW_PORT_ENETC_INST:5;
    vuint32_t _unused_5:1;
    vuint32_t NETC_FUNC:1;
    vuint32_t _unused_7:6;
    vuint32_t _unused_13:3;
    vuint32_t SPL_SW_PORT:5;
    vuint32_t _unused_21:11;
  } B;
} NETC_IERB_L2BCR_tag;

typedef union NETC_IERB_L2TXBCCTR_union_tag { /* Link 2 transmit byte credit comfort threshold register */
  vuint32_t R;
  struct {
    vuint32_t THRESH:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_L2TXBCCTR_tag;

typedef union NETC_IERB_L2E0MAR0_union_tag { /* Link 2 end 0 MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:32;
  } B;
} NETC_IERB_L2E0MAR0_tag;

typedef union NETC_IERB_L2E0MAR1_union_tag { /* Link 2 end 0 MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_L2E0MAR1_tag;

typedef union NETC_IERB_L2E1MAR0_union_tag { /* Link 2 end 1 MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:32;
  } B;
} NETC_IERB_L2E1MAR0_tag;

typedef union NETC_IERB_L2E1MAR1_union_tag { /* Link 2 end 1 MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_L2E1MAR1_tag;

typedef union NETC_IERB_CFG_SW_INST_SBCR_union_tag { /* Switch 0 binding configuration register */
  vuint32_t R;
  struct {
    vuint32_t RC_INST:4;
    vuint32_t _unused_4:4;
    vuint32_t FN:4;
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;
  } B;
} NETC_IERB_CFG_SW_INST_SBCR_tag;

typedef union NETC_IERB_CFG_SW_INST_SMCR_union_tag { /* Switch 0 MSI-X configuration register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MSIX:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_IERB_CFG_SW_INST_SMCR_tag;

typedef union NETC_IERB_CFG_SW_INST_S_CFH_DIDVID_union_tag { /* Switch 0 config header device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_IERB_CFG_SW_INST_S_CFH_DIDVID_tag;

typedef union NETC_IERB_CFG_SW_INST_S_CFH_SIDSVID_union_tag { /* Switch 0 config header subsystem ID and subsystem vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t SUBSYSTEM_VENDOR_ID:16;
    vuint32_t SUBSYSTEM_DEVICE_ID:16;
  } B;
} NETC_IERB_CFG_SW_INST_S_CFH_SIDSVID_tag;

typedef union NETC_IERB_CFG_SW_INST_SCCAR_union_tag { /* Switch 0 command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_IERB_CFG_SW_INST_SCCAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SAMQR_union_tag { /* Switch 0 access management qualifier register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t ARQOS:4;
    vuint32_t AWQOS:4;
    vuint32_t _unused_24:7;
    vuint32_t BMT:1;
  } B;
} NETC_IERB_CFG_SW_INST_SAMQR_tag;

typedef union NETC_IERB_CFG_SW_INST_SBLPR_union_tag { /* Switch 0 boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_IERB_CFG_SW_INST_SBLPR_tag;

typedef union NETC_IERB_CFG_SW_INST_SSMBAR_union_tag { /* Switch 0 shared memory buffer allotment register */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:24;
    vuint32_t _unused_24:6;
    vuint32_t MLOC:2;
  } B;
} NETC_IERB_CFG_SW_INST_SSMBAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SHTMAR_union_tag { /* Switch 0 hash table memory allotment register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:14;
    vuint32_t MLOC:2;
  } B;
} NETC_IERB_CFG_SW_INST_SHTMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SITMAR_union_tag { /* Switch 0 index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:14;
    vuint32_t MLOC:2;
  } B;
} NETC_IERB_CFG_SW_INST_SITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SIPFTMAR_union_tag { /* Switch 0 ingress port filter table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_SW_INST_SIPFTMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SRPITMAR_union_tag { /* Switch 0 rate policer index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_SW_INST_SRPITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SISCITMAR_union_tag { /* Switch 0 ingress stream counter index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_SW_INST_SISCITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SISITMAR_union_tag { /* Switch 0 ingress stream index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_SW_INST_SISITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SISQGITMAR_union_tag { /* Switch 0 ingress sequence generation index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:13;
    vuint32_t _unused_13:19;
  } B;
} NETC_IERB_CFG_SW_INST_SISQGITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SSGIITMAR_union_tag { /* Switch 0 stream gate instance index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_SW_INST_SSGIITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SSGCLITMAR_union_tag { /* Switch 0 stream gate control list index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_SW_INST_SSGCLITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SFMITMAR_union_tag { /* Switch 0 frame modification index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:13;
    vuint32_t _unused_13:19;
  } B;
} NETC_IERB_CFG_SW_INST_SFMITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_SFMDITMAR_union_tag { /* Switch 0 frame modification data index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_SW_INST_SFMDITMAR_tag;

typedef union NETC_IERB_CFG_SW_INST_STGSTAR_union_tag { /* Switch 0 time gate scheduling table allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:12;
    vuint32_t _unused_12:20;
  } B;
} NETC_IERB_CFG_SW_INST_STGSTAR_tag;

typedef union NETC_IERB_CFG_SW_INST_STGSLR_union_tag { /* Switch 0 time gate scheduling lookahead register */
  vuint32_t R;
  struct {
    vuint32_t MIN_LOOKAHEAD:20;
    vuint32_t _unused_20:12;
  } B;
} NETC_IERB_CFG_SW_INST_STGSLR_tag;

typedef union NETC_IERB_CFG_SW_INST_SMPCR_union_tag { /* Switch 0 management port configuration register */
  vuint32_t R;
  struct {
    vuint32_t PORT:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_IERB_CFG_SW_INST_SMPCR_tag;

typedef union NETC_IERB_CFG_SW_INST_SVFHTDECR0_union_tag { /* Switch 0 VLAN Filter (hash) table default entry configuration registers 0 */
  vuint32_t R;
  struct {
    vuint32_t PORT0:1;                 /* Port n. */
    vuint32_t PORT1:1;                 /* Port n. */
    vuint32_t PORT2:1;                 /* Port n. */
    vuint32_t _unused_3:21;
    vuint32_t STG_ID:4;
    vuint32_t _unused_28:1;
    vuint32_t IPMFE:1;
    vuint32_t IPMFLE:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_IERB_CFG_SW_INST_SVFHTDECR0_tag;

typedef union NETC_IERB_CFG_SW_INST_SVFHTDECR1_union_tag { /* Switch 0 VLAN filter hash table default entry configuration registers 1 */
  vuint32_t R;
  struct {
    vuint32_t FID:12;
    vuint32_t VL_MODE:1;
    vuint32_t _unused_13:3;
    vuint32_t BASE_ETEID:16;
  } B;
} NETC_IERB_CFG_SW_INST_SVFHTDECR1_tag;

typedef union NETC_IERB_CFG_SW_INST_SVFHTDECR2_union_tag { /* Switch 0 VLAN filter hash table default entry configuration registers 2 */
  vuint32_t R;
  struct {
    vuint32_t ES_PORT0:1;
    vuint32_t ES_PORT1:1;
    vuint32_t ES_PORT2:1;
    vuint32_t _unused_3:21;
    vuint32_t MLO:3;
    vuint32_t MFO:2;
    vuint32_t _unused_29:3;
  } B;
} NETC_IERB_CFG_SW_INST_SVFHTDECR2_tag;

typedef struct NETC_IERB_CFG_SW_INST_struct_tag {
  NETC_IERB_CFG_SW_INST_SBCR_tag SBCR; /* Switch 0 binding configuration register */
  uint8_t CFG_SW_INST_reserved0[16];
  NETC_IERB_CFG_SW_INST_SMCR_tag SMCR; /* Switch 0 MSI-X configuration register */
  uint8_t CFG_SW_INST_reserved1[8];
  NETC_IERB_CFG_SW_INST_S_CFH_DIDVID_tag S_CFH_DIDVID; /* Switch 0 config header device ID and vendor ID register */
  NETC_IERB_CFG_SW_INST_S_CFH_SIDSVID_tag S_CFH_SIDSVID; /* Switch 0 config header subsystem ID and subsystem vendor ID register */
  uint8_t CFG_SW_INST_reserved2[16];
  NETC_IERB_CFG_SW_INST_SCCAR_tag SCCAR; /* Switch 0 command cache attribute register */
  uint8_t CFG_SW_INST_reserved3[4];
  NETC_IERB_CFG_SW_INST_SAMQR_tag SAMQR; /* Switch 0 access management qualifier register */
  uint8_t CFG_SW_INST_reserved4[4];
  NETC_IERB_CFG_SW_INST_SBLPR_tag SBLPR[2]; /* Switch 0 boot loader parameter register 0 */
  uint8_t CFG_SW_INST_reserved5[16];
  NETC_IERB_CFG_SW_INST_SSMBAR_tag SSMBAR; /* Switch 0 shared memory buffer allotment register */
  uint8_t CFG_SW_INST_reserved6[28];
  NETC_IERB_CFG_SW_INST_SHTMAR_tag SHTMAR; /* Switch 0 hash table memory allotment register */
  NETC_IERB_CFG_SW_INST_SITMAR_tag SITMAR; /* Switch 0 index table memory allocation register */
  NETC_IERB_CFG_SW_INST_SIPFTMAR_tag SIPFTMAR; /* Switch 0 ingress port filter table memory allocation register */
  uint8_t CFG_SW_INST_reserved7[20];
  NETC_IERB_CFG_SW_INST_SRPITMAR_tag SRPITMAR; /* Switch 0 rate policer index table memory allocation register */
  NETC_IERB_CFG_SW_INST_SISCITMAR_tag SISCITMAR; /* Switch 0 ingress stream counter index table memory allocation register */
  NETC_IERB_CFG_SW_INST_SISITMAR_tag SISITMAR; /* Switch 0 ingress stream index table memory allocation register */
  NETC_IERB_CFG_SW_INST_SISQGITMAR_tag SISQGITMAR; /* Switch 0 ingress sequence generation index table memory allocation register */
  uint8_t CFG_SW_INST_reserved8[4];
  NETC_IERB_CFG_SW_INST_SSGIITMAR_tag SSGIITMAR; /* Switch 0 stream gate instance index table memory allocation register */
  NETC_IERB_CFG_SW_INST_SSGCLITMAR_tag SSGCLITMAR; /* Switch 0 stream gate control list index table memory allocation register */
  NETC_IERB_CFG_SW_INST_SFMITMAR_tag SFMITMAR; /* Switch 0 frame modification index table memory allocation register */
  NETC_IERB_CFG_SW_INST_SFMDITMAR_tag SFMDITMAR; /* Switch 0 frame modification data index table memory allocation register */
  uint8_t CFG_SW_INST_reserved9[44];
  NETC_IERB_CFG_SW_INST_STGSTAR_tag STGSTAR; /* Switch 0 time gate scheduling table allocation register */
  NETC_IERB_CFG_SW_INST_STGSLR_tag STGSLR; /* Switch 0 time gate scheduling lookahead register */
  uint8_t CFG_SW_INST_reserved10[268];
  NETC_IERB_CFG_SW_INST_SMPCR_tag SMPCR; /* Switch 0 management port configuration register */
  uint8_t CFG_SW_INST_reserved11[8];
  NETC_IERB_CFG_SW_INST_SVFHTDECR0_tag SVFHTDECR0; /* Switch 0 VLAN Filter (hash) table default entry configuration registers 0 */
  NETC_IERB_CFG_SW_INST_SVFHTDECR1_tag SVFHTDECR1; /* Switch 0 VLAN filter hash table default entry configuration registers 1 */
  NETC_IERB_CFG_SW_INST_SVFHTDECR2_tag SVFHTDECR2; /* Switch 0 VLAN filter hash table default entry configuration registers 2 */
} NETC_IERB_CFG_SW_INST_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EBCR0_union_tag { /* ENETC 0 binding configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t RC_INST:4;
    vuint32_t _unused_4:4;
    vuint32_t FN:4;
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;
  } B;
} NETC_IERB_CFG_ENETC_INST_EBCR0_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EBCR1_union_tag { /* ENETC 0 binding configuration register 1 */
  vuint32_t R;
  struct {
    vuint32_t NUM_RX_BDR:10;
    vuint32_t _unused_10:6;
    vuint32_t NUM_TX_BDR:10;
    vuint32_t _unused_26:6;
  } B;
} NETC_IERB_CFG_ENETC_INST_EBCR1_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EBCR2_union_tag { /* ENETC 0 binding configuration register 2 */
  vuint32_t R;
  struct {
    vuint32_t NUM_MAC_AFTE:12;
    vuint32_t _unused_12:4;
    vuint32_t NUM_VLAN_FTE:12;
    vuint32_t _unused_28:4;
  } B;
} NETC_IERB_CFG_ENETC_INST_EBCR2_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EVBCR_union_tag { /* ENETC 0 VSI binding configuration register */
  vuint32_t R;
  struct {
    vuint32_t NUM_VSI:4;
    vuint32_t _unused_4:28;
  } B;
} NETC_IERB_CFG_ENETC_INST_EVBCR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EMCR_union_tag { /* ENETC 0 MSI-X configuration register */
  vuint32_t R;
  struct {
    vuint32_t NUM_MSIX:11;
    vuint32_t _unused_11:21;
  } B;
} NETC_IERB_CFG_ENETC_INST_EMCR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_E_CFH_DIDVID_union_tag { /* ENETC 0 config header device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_E_CFH_DIDVID_tag;

typedef union NETC_IERB_CFG_ENETC_INST_E_CFH_SIDSVID_union_tag { /* ENETC 0 config header subsystem ID and subsystem vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t SUBSYSTEM_VENDOR_ID:16;
    vuint32_t SUBSYSTEM_DEVICE_ID:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_E_CFH_SIDSVID_tag;

typedef union NETC_IERB_CFG_ENETC_INST_E_CFC_VFDID_union_tag { /* ENETC 0 config capability VF device ID register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t VF_DEVICE_ID:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_E_CFC_VFDID_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EBCAR_union_tag { /* ENETC 0 buffer cache attribute register 0 */
  vuint32_t R;
  struct {
    vuint32_t BD_WRCACHE:4;
    vuint32_t BD_WRDOMAIN:2;
    vuint32_t BD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t WRCACHE:4;
    vuint32_t WRDOMAIN:2;
    vuint32_t WRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t BD_RDCACHE:4;
    vuint32_t BD_RDDOMAIN:2;
    vuint32_t BD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t RDCACHE:4;
    vuint32_t RDDOMAIN:2;
    vuint32_t RDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_IERB_CFG_ENETC_INST_EBCAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EMCAR_union_tag { /* ENETC 0 message cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t MSG_WRCACHE:4;
    vuint32_t MSG_WRDOMAIN:2;
    vuint32_t MSG_WRSNP:1;
    vuint32_t _unused_7:9;
    vuint32_t MSG_RDCACHE:4;
    vuint32_t MSG_RDDOMAIN:2;
    vuint32_t MSG_RDSNP:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_IERB_CFG_ENETC_INST_EMCAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ECAR_union_tag { /* ENETC 0 command cache attribute register */
  vuint32_t R;
  struct {
    vuint32_t CBD_WRCACHE:4;
    vuint32_t CBD_WRDOMAIN:2;
    vuint32_t CBD_WRSNP:1;
    vuint32_t _unused_7:1;
    vuint32_t CWRCACHE:4;
    vuint32_t CWRDOMAIN:2;
    vuint32_t CWRSNP:1;
    vuint32_t _unused_15:1;
    vuint32_t CBD_RDCACHE:4;
    vuint32_t CBD_RDDOMAIN:2;
    vuint32_t CBD_RDSNP:1;
    vuint32_t _unused_23:1;
    vuint32_t CRDCACHE:4;
    vuint32_t CRDDOMAIN:2;
    vuint32_t CRDSNP:1;
    vuint32_t _unused_31:1;
  } B;
} NETC_IERB_CFG_ENETC_INST_ECAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EAMQR_union_tag { /* ENETC 0 access management qualifier register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t ARQOS:4;
    vuint32_t AWQOS:4;
    vuint32_t _unused_24:7;
    vuint32_t BMT:1;
  } B;
} NETC_IERB_CFG_ENETC_INST_EAMQR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EBLPR_union_tag { /* ENETC 0 boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_IERB_CFG_ENETC_INST_EBLPR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ERXMBER_union_tag { /* ENETC 0 receive memory buffer entitlement register */
  vuint32_t R;
  struct {
    vuint32_t AMOUNT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_IERB_CFG_ENETC_INST_ERXMBER_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ERXMBLR_union_tag { /* ENETC 0 receive memory buffer limit register */
  vuint32_t R;
  struct {
    vuint32_t LIMIT:24;
    vuint32_t _unused_24:8;
  } B;
} NETC_IERB_CFG_ENETC_INST_ERXMBLR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ETXHPTBCR_union_tag { /* ENETC 0 transmit high priority tier byte credit register */
  vuint32_t R;
  struct {
    vuint32_t BYTE_CREDIT:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_ETXHPTBCR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ETXLPTBCR_union_tag { /* ENETC 0 transmit low priority tier byte credit register */
  vuint32_t R;
  struct {
    vuint32_t BYTE_CREDIT:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_ETXLPTBCR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EHTMAR_union_tag { /* ENETC 0 hash table memory allotment register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:14;
    vuint32_t MLOC:2;
  } B;
} NETC_IERB_CFG_ENETC_INST_EHTMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EITMAR_union_tag { /* ENETC 0 index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:14;
    vuint32_t MLOC:2;
  } B;
} NETC_IERB_CFG_ENETC_INST_EITMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EIPFTMAR_union_tag { /* ENETC 0 ingress port filter table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_EIPFTMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ERTMAR_union_tag { /* ENETC 0 RFS ternary memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t ALLOC:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_ERTMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ERPITMAR_union_tag { /* ENETC 0 rate policer index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_ERPITMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EISCITMAR_union_tag { /* ENETC 0 ingress stream counter index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_EISCITMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_EISITMAR_union_tag { /* ENETC 0 ingress stream index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_EISITMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ESGIITMAR_union_tag { /* ENETC 0 stream gate instance index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_ESGIITMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ESGCLITMAR_union_tag { /* ENETC 0 stream gate control list index table memory allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_ENETC_INST_ESGCLITMAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ETGSTAR_union_tag { /* ENETC 0 time gate scheduling table allocation register */
  vuint32_t R;
  struct {
    vuint32_t NUM_WORDS:12;
    vuint32_t _unused_12:20;
  } B;
} NETC_IERB_CFG_ENETC_INST_ETGSTAR_tag;

typedef union NETC_IERB_CFG_ENETC_INST_ETGSLR_union_tag { /* ENETC 0 time gate scheduling lookahead register */
  vuint32_t R;
  struct {
    vuint32_t MIN_LOOKAHEAD:20;
    vuint32_t _unused_20:11;
    vuint32_t ZERO_LOOKAHEAD:1;        /* Zero Lookahead */
  } B;
} NETC_IERB_CFG_ENETC_INST_ETGSLR_tag;

typedef struct NETC_IERB_CFG_ENETC_INST_struct_tag {
  NETC_IERB_CFG_ENETC_INST_EBCR0_tag EBCR0; /* ENETC 0 binding configuration register 0 */
  NETC_IERB_CFG_ENETC_INST_EBCR1_tag EBCR1; /* ENETC 0 binding configuration register 1 */
  NETC_IERB_CFG_ENETC_INST_EBCR2_tag EBCR2; /* ENETC 0 binding configuration register 2 */
  uint8_t CFG_ENETC_INST_reserved0[4];
  NETC_IERB_CFG_ENETC_INST_EVBCR_tag EVBCR; /* ENETC 0 VSI binding configuration register */
  NETC_IERB_CFG_ENETC_INST_EMCR_tag EMCR; /* ENETC 0 MSI-X configuration register */
  uint8_t CFG_ENETC_INST_reserved1[8];
  NETC_IERB_CFG_ENETC_INST_E_CFH_DIDVID_tag E_CFH_DIDVID; /* ENETC 0 config header device ID and vendor ID register */
  NETC_IERB_CFG_ENETC_INST_E_CFH_SIDSVID_tag E_CFH_SIDSVID; /* ENETC 0 config header subsystem ID and subsystem vendor ID register */
  NETC_IERB_CFG_ENETC_INST_E_CFC_VFDID_tag E_CFC_VFDID; /* ENETC 0 config capability VF device ID register */
  uint8_t CFG_ENETC_INST_reserved2[4];
  NETC_IERB_CFG_ENETC_INST_EBCAR_tag EBCAR; /* ENETC 0 buffer cache attribute register 0 */
  NETC_IERB_CFG_ENETC_INST_EMCAR_tag EMCAR; /* ENETC 0 message cache attribute register */
  NETC_IERB_CFG_ENETC_INST_ECAR_tag ECAR; /* ENETC 0 command cache attribute register */
  uint8_t CFG_ENETC_INST_reserved3[4];
  NETC_IERB_CFG_ENETC_INST_EAMQR_tag EAMQR; /* ENETC 0 access management qualifier register */
  uint8_t CFG_ENETC_INST_reserved4[4];
  NETC_IERB_CFG_ENETC_INST_EBLPR_tag EBLPR[2]; /* ENETC 0 boot loader parameter register 0 */
  NETC_IERB_CFG_ENETC_INST_ERXMBER_tag ERXMBER; /* ENETC 0 receive memory buffer entitlement register */
  NETC_IERB_CFG_ENETC_INST_ERXMBLR_tag ERXMBLR; /* ENETC 0 receive memory buffer limit register */
  uint8_t CFG_ENETC_INST_reserved5[24];
  NETC_IERB_CFG_ENETC_INST_ETXHPTBCR_tag ETXHPTBCR; /* ENETC 0 transmit high priority tier byte credit register */
  NETC_IERB_CFG_ENETC_INST_ETXLPTBCR_tag ETXLPTBCR; /* ENETC 0 transmit low priority tier byte credit register */
  uint8_t CFG_ENETC_INST_reserved6[8];
  NETC_IERB_CFG_ENETC_INST_EHTMAR_tag EHTMAR; /* ENETC 0 hash table memory allotment register */
  NETC_IERB_CFG_ENETC_INST_EITMAR_tag EITMAR; /* ENETC 0 index table memory allocation register */
  NETC_IERB_CFG_ENETC_INST_EIPFTMAR_tag EIPFTMAR; /* ENETC 0 ingress port filter table memory allocation register */
  uint8_t CFG_ENETC_INST_reserved7[4];
  NETC_IERB_CFG_ENETC_INST_ERTMAR_tag ERTMAR; /* ENETC 0 RFS ternary memory allocation register */
  uint8_t CFG_ENETC_INST_reserved8[12];
  NETC_IERB_CFG_ENETC_INST_ERPITMAR_tag ERPITMAR; /* ENETC 0 rate policer index table memory allocation register */
  NETC_IERB_CFG_ENETC_INST_EISCITMAR_tag EISCITMAR; /* ENETC 0 ingress stream counter index table memory allocation register */
  NETC_IERB_CFG_ENETC_INST_EISITMAR_tag EISITMAR; /* ENETC 0 ingress stream index table memory allocation register */
  uint8_t CFG_ENETC_INST_reserved9[8];
  NETC_IERB_CFG_ENETC_INST_ESGIITMAR_tag ESGIITMAR; /* ENETC 0 stream gate instance index table memory allocation register */
  NETC_IERB_CFG_ENETC_INST_ESGCLITMAR_tag ESGCLITMAR; /* ENETC 0 stream gate control list index table memory allocation register */
  uint8_t CFG_ENETC_INST_reserved10[52];
  NETC_IERB_CFG_ENETC_INST_ETGSTAR_tag ETGSTAR; /* ENETC 0 time gate scheduling table allocation register */
  NETC_IERB_CFG_ENETC_INST_ETGSLR_tag ETGSLR; /* ENETC 0 time gate scheduling lookahead register */
} NETC_IERB_CFG_ENETC_INST_tag;

typedef union NETC_IERB_CFG_VSI_INST_VAMQR_union_tag { /* VSI 0 access management qualifier register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t ARQOS:4;
    vuint32_t AWQOS:4;
    vuint32_t _unused_24:7;
    vuint32_t BMT:1;
  } B;
} NETC_IERB_CFG_VSI_INST_VAMQR_tag;

typedef union NETC_IERB_CFG_VSI_INST_VBLPR_union_tag { /* VSI 0 boot loader parameter register 0 */
  vuint32_t R;
  struct {
    vuint32_t PARAM_VAL:32;
  } B;
} NETC_IERB_CFG_VSI_INST_VBLPR_tag;

typedef union NETC_IERB_CFG_VSI_INST_VPMAR0_union_tag { /* VSI 0 primary MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:32;
  } B;
} NETC_IERB_CFG_VSI_INST_VPMAR0_tag;

typedef union NETC_IERB_CFG_VSI_INST_VPMAR1_union_tag { /* VSI 0 primary MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} NETC_IERB_CFG_VSI_INST_VPMAR1_tag;

typedef struct NETC_IERB_CFG_VSI_INST_struct_tag {
  NETC_IERB_CFG_VSI_INST_VAMQR_tag VAMQR; /* VSI 0 access management qualifier register */
  uint8_t CFG_VSI_INST_reserved0[4];
  NETC_IERB_CFG_VSI_INST_VBLPR_tag VBLPR[2]; /* VSI 0 boot loader parameter register 0 */
  NETC_IERB_CFG_VSI_INST_VPMAR0_tag VPMAR0; /* VSI 0 primary MAC address register 0 */
  NETC_IERB_CFG_VSI_INST_VPMAR1_tag VPMAR1; /* VSI 0 primary MAC address register 1 */
  uint8_t CFG_VSI_INST_reserved1[40];
} NETC_IERB_CFG_VSI_INST_tag;

struct NETC_IERB_tag {
  NETC_IERB_CAPR0_tag CAPR0;           /* Capability register 0 */
  NETC_IERB_CAPR1_tag CAPR1;           /* Capability register 1 */
  NETC_IERB_CAPR2_tag CAPR2;           /* Capability register 2 */
  NETC_IERB_CAPR3_tag CAPR3;           /* Capability register 3 */
  uint8_t NETC_IERB_reserved0[16];
  NETC_IERB_CMCAPR_tag CMCAPR;         /* Common memory capability register */
  uint8_t NETC_IERB_reserved1[12];
  NETC_IERB_IPFTMCAPR_tag IPFTMCAPR;   /* Ingress port filter ternary memory capability register */
  uint8_t NETC_IERB_reserved2[16];
  NETC_IERB_TGSMCAPR_tag TGSMCAPR;     /* Time gate scheduling memory capability register */
  uint8_t NETC_IERB_reserved3[56];
  NETC_IERB_SMDTR_tag SMDTR;           /* Shared memory depletion threshold register */
  NETC_IERB_ERSMBAR_tag ERSMBAR;       /* ENETC receive shared memory buffer allotment register */
  uint8_t NETC_IERB_reserved4[56];
  NETC_IERB_HTA_NUM_tag HTA_NUM[1];
  uint8_t NETC_IERB_reserved5[56];
  NETC_IERB_HBTMAR_tag HBTMAR;         /* Hash bucket table memory allocation register */
  NETC_IERB_HBTCR_tag HBTCR;           /* Hash bucket table configuration register */
  NETC_IERB_GHTEMCAPR_tag GHTEMCAPR;   /* Guaranteed hash table entry memory capability register */
  uint8_t NETC_IERB_reserved6[100];
  NETC_IERB_NETCFLRCR_tag NETCFLRCR;   /* NETC FLR configuration register */
  uint8_t NETC_IERB_reserved7[4];
  NETC_IERB_NETCCLKFR_tag NETCCLKFR;   /* NETC clock period fractional register */
  NETC_IERB_NETCCLKCR_tag NETCCLKCR;   /* NETC clock configuration register */
  NETC_IERB_SBCR_tag SBCR;             /* System bus configuration register */
  NETC_IERB_SBOTCR_tag SBOTCR;         /* System bus outstanding transaction control register */
  uint8_t NETC_IERB_reserved8[8];
  NETC_IERB_SGLTTR_tag SGLTTR;         /* Stream gating lag time for refresh register */
  uint8_t NETC_IERB_reserved9[108];
  NETC_IERB_ARRAY_NUM_RC_tag ARRAY_NUM_RC[1];
  uint8_t NETC_IERB_reserved10[240];
  NETC_IERB_EMDIOBCR_tag EMDIOBCR;     /* EMDIO binding configuration register */
  uint8_t NETC_IERB_reserved11[16];
  NETC_IERB_EMDIOMCR_tag EMDIOMCR;     /* EMDIO MSI-X configuration register */
  uint8_t NETC_IERB_reserved12[8];
  NETC_IERB_EMDIO_CFH_DIDVID_tag EMDIO_CFH_DIDVID; /* EMDIO config header device ID and vendor ID register */
  NETC_IERB_EMDIO_CFH_SIDSVID_tag EMDIO_CFH_SIDSVID; /* EMDIO config header subsystem ID and subsystem vendor ID register */
  uint8_t NETC_IERB_reserved13[32];
  NETC_IERB_EMDIOBLPR_tag EMDIOBLPR[2]; /* EMDIO boot loader parameter register 0 */
  NETC_IERB_EMDIO_CFG_tag EMDIO_CFG;   /* EMDIO configuration register */
  uint8_t NETC_IERB_reserved14[172];
  NETC_IERB_NUM_TMR_ARRAY_tag NUM_TMR_ARRAY[1];
  uint8_t NETC_IERB_reserved15[2992];
  NETC_IERB_L0CAPR_tag L0CAPR;         /* Link 0 capability register */
  NETC_IERB_L0MCAPR_tag L0MCAPR;       /* Link 0 MAC capability register */
  NETC_IERB_L0IOCAPR_tag L0IOCAPR;     /* Link 0 I/O capability register */
  uint8_t NETC_IERB_reserved16[4];
  NETC_IERB_L0BCR_tag L0BCR;           /* Link 0 binding configuration register */
  NETC_IERB_L0TXBCCTR_tag L0TXBCCTR;   /* Link 0 transmit byte credit comfort threshold register */
  uint8_t NETC_IERB_reserved17[8];
  NETC_IERB_L0E0MAR0_tag L0E0MAR0;     /* Link 0 end 0 MAC address register 0 */
  NETC_IERB_L0E0MAR1_tag L0E0MAR1;     /* Link 0 end 0 MAC address register 1 */
  uint8_t NETC_IERB_reserved18[24];
  NETC_IERB_L1CAPR_tag L1CAPR;         /* Link 1 capability register */
  NETC_IERB_L1MCAPR_tag L1MCAPR;       /* Link 1 MAC capability register */
  NETC_IERB_L1IOCAPR_tag L1IOCAPR;     /* Link 1 I/O capability register */
  uint8_t NETC_IERB_reserved19[4];
  NETC_IERB_L1BCR_tag L1BCR;           /* Link 1 binding configuration register */
  NETC_IERB_L1TXBCCTR_tag L1TXBCCTR;   /* Link 1 transmit byte credit comfort threshold register */
  uint8_t NETC_IERB_reserved20[8];
  NETC_IERB_L1E0MAR0_tag L1E0MAR0;     /* Link 1 end 0 MAC address register 0 */
  NETC_IERB_L1E0MAR1_tag L1E0MAR1;     /* Link 1 end 0 MAC address register 1 */
  uint8_t NETC_IERB_reserved21[24];
  NETC_IERB_L2CAPR_tag L2CAPR;         /* Link 2 capability register */
  NETC_IERB_L2MCAPR_tag L2MCAPR;       /* Link 2 MAC capability register */
  uint8_t NETC_IERB_reserved22[8];
  NETC_IERB_L2BCR_tag L2BCR;           /* Link 2 binding configuration register */
  NETC_IERB_L2TXBCCTR_tag L2TXBCCTR;   /* Link 2 transmit byte credit comfort threshold register */
  uint8_t NETC_IERB_reserved23[8];
  NETC_IERB_L2E0MAR0_tag L2E0MAR0;     /* Link 2 end 0 MAC address register 0 */
  NETC_IERB_L2E0MAR1_tag L2E0MAR1;     /* Link 2 end 0 MAC address register 1 */
  NETC_IERB_L2E1MAR0_tag L2E1MAR0;     /* Link 2 end 1 MAC address register 0 */
  NETC_IERB_L2E1MAR1_tag L2E1MAR1;     /* Link 2 end 1 MAC address register 1 */
  uint8_t NETC_IERB_reserved24[3920];
  NETC_IERB_CFG_SW_INST_tag CFG_SW_INST[1];
  uint8_t NETC_IERB_reserved25[3556];
  NETC_IERB_CFG_ENETC_INST_tag CFG_ENETC_INST[1];
  uint8_t NETC_IERB_reserved26[3848];
  NETC_IERB_CFG_VSI_INST_tag CFG_VSI_INST[7];
};


/* ============================================================================
   =============================== Module: NETC_PRIV ==========================
   ============================================================================ */

typedef union NETC_PRIV_NETCRR_union_tag { /* NETC reset register */
  vuint32_t R;
  struct {
    vuint32_t SR:1;                    /* Soft reset */
    vuint32_t LOCK:1;                  /* Lock */
    vuint32_t _unused_2:30;
  } B;
} NETC_PRIV_NETCRR_tag;

typedef union NETC_PRIV_NETCSR_union_tag { /* NETC status register */
  vuint32_t R;
  struct {
    vuint32_t ERROR:1;                 /* Error */
    vuint32_t STATE:1;                 /* Indicates NETC's global operational state */
    vuint32_t _unused_2:30;
  } B;
} NETC_PRIV_NETCSR_tag;

typedef union NETC_PRIV_IEICR0_union_tag { /* Integrity Error Injection Config Register 0 */
  vuint32_t R;
  struct {
    vuint32_t LINK_SLICE_ID:4;         /* Link or Slice ID */
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:2;
    vuint32_t ENGINE_ID:1;             /* Engine ID */
    vuint32_t _unused_17:5;
    vuint32_t ARM:1;                   /* Armed */
    vuint32_t _unused_23:1;
    vuint32_t EN:8;                    /* Enable */
  } B;
} NETC_PRIV_IEICR0_tag;

typedef union NETC_PRIV_IEICR1_union_tag { /* Integrity Error Injection Config Register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENGINE_EN0:1;            /* Engine enable */
    vuint32_t ENGINE_EN1:1;            /* Engine enable */
    vuint32_t _unused_2:30;
  } B;
} NETC_PRIV_IEICR1_tag;

typedef union NETC_PRIV_MEICR_union_tag { /* Memory Error Injection Config Register */
  vuint32_t R;
  struct {
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_5:17;
    vuint32_t ARM:2;                   /* Armed */
    vuint32_t EN:8;                    /* Enable */
  } B;
} NETC_PRIV_MEICR_tag;

typedef union NETC_PRIV_NETCWDTR_union_tag { /* NETC watch dog timer register */
  vuint32_t R;
  struct {
    vuint32_t MCED:1;                  /* MAC Client Express Disable */
    vuint32_t MCPD:1;                  /* MAC Client Preemptable Disable */
    vuint32_t PCED:1;                  /* PCE Disable */
    vuint32_t HTAD:1;                  /* HTA Disable */
    vuint32_t _unused_4:28;
  } B;
} NETC_PRIV_NETCWDTR_tag;

typedef union NETC_PRIV_CMECR_union_tag { /* Correctable memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:24;
  } B;
} NETC_PRIV_CMECR_tag;

typedef union NETC_PRIV_CMESR_union_tag { /* Correctable memory error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t SBEE:1;                  /* Single-bit ECC error */
  } B;
} NETC_PRIV_CMESR_tag;

typedef union NETC_PRIV_CMECTR_union_tag { /* Correctable memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_PRIV_CMECTR_tag;

typedef union NETC_PRIV_UNMECR_union_tag { /* Uncorrectable non-fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_PRIV_UNMECR_tag;

typedef union NETC_PRIV_UNMESR0_union_tag { /* Uncorrectable non-fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:10;
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_PRIV_UNMESR0_tag;

typedef union NETC_PRIV_UNMESR1_union_tag { /* Uncorrectable non-fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_PRIV_UNMESR1_tag;

typedef union NETC_PRIV_UNMECTR_union_tag { /* Uncorrectable non-fatal memory error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_PRIV_UNMECTR_tag;

typedef union NETC_PRIV_UFMECR_union_tag { /* Uncorrectable fatal memory error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_PRIV_UFMECR_tag;

typedef union NETC_PRIV_UFMESR0_union_tag { /* Uncorrectable fatal memory error status register 0 */
  vuint32_t R;
  struct {
    vuint32_t SYNDROME:11;             /* Syndrome */
    vuint32_t _unused_11:5;
    vuint32_t MEM_ID:5;                /* Memory ID */
    vuint32_t _unused_21:9;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t MBEE:1;                  /* Multi-bit ECC error */
  } B;
} NETC_PRIV_UFMESR0_tag;

typedef union NETC_PRIV_UFMESR1_union_tag { /* Uncorrectable fatal memory error status register 1 */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Address */
  } B;
} NETC_PRIV_UFMESR1_tag;

typedef union NETC_PRIV_UNIECR_union_tag { /* Uncorrectable non-fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t THRESHOLD:8;             /* Threshold */
    vuint32_t _unused_8:23;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_PRIV_UNIECR_tag;

typedef union NETC_PRIV_UNIESR_union_tag { /* Uncorrectable non-fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t LINK_SLICE_ID:4;         /* Link or Slice ID */
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:2;
    vuint32_t ENGINE_ID:1;             /* Engine ID */
    vuint32_t _unused_17:14;
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_PRIV_UNIESR_tag;

typedef union NETC_PRIV_UNIECTR_union_tag { /* Uncorrectable non-fatal integrity error count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:8;                 /* Count */
    vuint32_t _unused_8:24;
  } B;
} NETC_PRIV_UNIECTR_tag;

typedef union NETC_PRIV_UFIECR_union_tag { /* Uncorrectable fatal integrity error configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t RD:1;                    /* Report disable */
  } B;
} NETC_PRIV_UFIECR_tag;

typedef union NETC_PRIV_UFIESR_union_tag { /* Uncorrectable fatal integrity error status register */
  vuint32_t R;
  struct {
    vuint32_t LINK_SLICE_ID:4;         /* Link or Slice ID */
    vuint32_t BLOCK_ID:4;              /* Block ID */
    vuint32_t SM_ID:6;                 /* Safety mechanism ID */
    vuint32_t _unused_14:2;
    vuint32_t ENGINE_ID:1;             /* Engine ID */
    vuint32_t _unused_17:13;
    vuint32_t M:1;                     /* Multiple */
    vuint32_t INTERR:1;                /* Integrity error */
  } B;
} NETC_PRIV_UFIESR_tag;

struct NETC_PRIV_tag {
  uint8_t NETC_PRIV_reserved0[256];
  NETC_PRIV_NETCRR_tag NETCRR;         /* NETC reset register */
  NETC_PRIV_NETCSR_tag NETCSR;         /* NETC status register */
  uint8_t NETC_PRIV_reserved1[248];
  NETC_PRIV_IEICR0_tag IEICR0;         /* Integrity Error Injection Config Register 0 */
  NETC_PRIV_IEICR1_tag IEICR1;         /* Integrity Error Injection Config Register 1 */
  NETC_PRIV_MEICR_tag MEICR;           /* Memory Error Injection Config Register */
  uint8_t NETC_PRIV_reserved2[244];
  NETC_PRIV_NETCWDTR_tag NETCWDTR;     /* NETC watch dog timer register */
  uint8_t NETC_PRIV_reserved3[2812];
  NETC_PRIV_CMECR_tag CMECR;           /* Correctable memory error configuration register */
  NETC_PRIV_CMESR_tag CMESR;           /* Correctable memory error status register */
  uint8_t NETC_PRIV_reserved4[4];
  NETC_PRIV_CMECTR_tag CMECTR;         /* Correctable memory error count register */
  uint8_t NETC_PRIV_reserved5[32];
  NETC_PRIV_UNMECR_tag UNMECR;         /* Uncorrectable non-fatal memory error configuration register */
  NETC_PRIV_UNMESR0_tag UNMESR0;       /* Uncorrectable non-fatal memory error status register 0 */
  NETC_PRIV_UNMESR1_tag UNMESR1;       /* Uncorrectable non-fatal memory error status register 1 */
  NETC_PRIV_UNMECTR_tag UNMECTR;       /* Uncorrectable non-fatal memory error count register */
  NETC_PRIV_UFMECR_tag UFMECR;         /* Uncorrectable fatal memory error configuration register */
  NETC_PRIV_UFMESR0_tag UFMESR0;       /* Uncorrectable fatal memory error status register 0 */
  NETC_PRIV_UFMESR1_tag UFMESR1;       /* Uncorrectable fatal memory error status register 1 */
  uint8_t NETC_PRIV_reserved6[4];
  NETC_PRIV_UNIECR_tag UNIECR;         /* Uncorrectable non-fatal integrity error configuration register */
  NETC_PRIV_UNIESR_tag UNIESR;         /* Uncorrectable non-fatal integrity error status register */
  uint8_t NETC_PRIV_reserved7[4];
  NETC_PRIV_UNIECTR_tag UNIECTR;       /* Uncorrectable non-fatal integrity error count register */
  NETC_PRIV_UFIECR_tag UFIECR;         /* Uncorrectable fatal integrity error configuration register */
  NETC_PRIV_UFIESR_tag UFIESR;         /* Uncorrectable fatal integrity error status register */
};


/* ============================================================================
   =============================== Module: NETC_VF1_PCI_HDR_TYPE0 =============
   ============================================================================ */

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:2;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_union_tag { /* PCI latency timer register */
  vuint8_t R;
  struct {
    vuint8_t LATENCY_TIMER:8;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BIST_union_tag { /* PCI BIST register */
  vuint8_t R;
  struct {
    vuint8_t BIST:8;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BIST_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_union_tag { /* PCI cardbus CIS register */
  vuint32_t R;
  struct {
    vuint32_t CARDBUS_CIS_PTR:32;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_ID:16;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_union_tag { /* PCI expansion ROM base address register */
  vuint32_t R;
  struct {
    vuint32_t EXP_ROM_BA:32;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_VF1_PCI_HDR_TYPE0_tag {
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag PCI_CFH_LAT_TIMER; /* PCI latency timer register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BIST_tag PCI_CFH_BIST; /* PCI BIST register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag PCI_CFH_CARDBUS_CIS; /* PCI cardbus CIS register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag PCI_CFH_EXP_ROM_BA; /* PCI expansion ROM base address register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_VF1_PCI_HDR_TYPE0_reserved0[11];
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_VF1_PCI_HDR_TYPE0_reserved1[24];
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_VF1_PCI_HDR_TYPE0_reserved2[22];
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_VF1_PCI_HDR_TYPE0_reserved3[116];
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_VF1_PCI_HDR_TYPE0_reserved4[20];
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_VF1_PCI_HDR_TYPE0_reserved5[8];
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_VF2_PCI_HDR_TYPE0 =============
   ============================================================================ */

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:2;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_union_tag { /* PCI latency timer register */
  vuint8_t R;
  struct {
    vuint8_t LATENCY_TIMER:8;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BIST_union_tag { /* PCI BIST register */
  vuint8_t R;
  struct {
    vuint8_t BIST:8;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BIST_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_union_tag { /* PCI cardbus CIS register */
  vuint32_t R;
  struct {
    vuint32_t CARDBUS_CIS_PTR:32;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_ID:16;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_union_tag { /* PCI expansion ROM base address register */
  vuint32_t R;
  struct {
    vuint32_t EXP_ROM_BA:32;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_VF2_PCI_HDR_TYPE0_tag {
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag PCI_CFH_LAT_TIMER; /* PCI latency timer register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BIST_tag PCI_CFH_BIST; /* PCI BIST register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag PCI_CFH_CARDBUS_CIS; /* PCI cardbus CIS register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag PCI_CFH_EXP_ROM_BA; /* PCI expansion ROM base address register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_VF2_PCI_HDR_TYPE0_reserved0[11];
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_VF2_PCI_HDR_TYPE0_reserved1[24];
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_VF2_PCI_HDR_TYPE0_reserved2[22];
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_VF2_PCI_HDR_TYPE0_reserved3[116];
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_VF2_PCI_HDR_TYPE0_reserved4[20];
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_VF2_PCI_HDR_TYPE0_reserved5[8];
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_VF3_PCI_HDR_TYPE0 =============
   ============================================================================ */

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:2;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_union_tag { /* PCI latency timer register */
  vuint8_t R;
  struct {
    vuint8_t LATENCY_TIMER:8;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BIST_union_tag { /* PCI BIST register */
  vuint8_t R;
  struct {
    vuint8_t BIST:8;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BIST_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_union_tag { /* PCI cardbus CIS register */
  vuint32_t R;
  struct {
    vuint32_t CARDBUS_CIS_PTR:32;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_ID:16;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_union_tag { /* PCI expansion ROM base address register */
  vuint32_t R;
  struct {
    vuint32_t EXP_ROM_BA:32;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_VF3_PCI_HDR_TYPE0_tag {
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag PCI_CFH_LAT_TIMER; /* PCI latency timer register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BIST_tag PCI_CFH_BIST; /* PCI BIST register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag PCI_CFH_CARDBUS_CIS; /* PCI cardbus CIS register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag PCI_CFH_EXP_ROM_BA; /* PCI expansion ROM base address register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_VF3_PCI_HDR_TYPE0_reserved0[11];
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_VF3_PCI_HDR_TYPE0_reserved1[24];
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_VF3_PCI_HDR_TYPE0_reserved2[22];
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_VF3_PCI_HDR_TYPE0_reserved3[116];
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_VF3_PCI_HDR_TYPE0_reserved4[20];
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_VF3_PCI_HDR_TYPE0_reserved5[8];
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_VF4_PCI_HDR_TYPE0 =============
   ============================================================================ */

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:2;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_union_tag { /* PCI latency timer register */
  vuint8_t R;
  struct {
    vuint8_t LATENCY_TIMER:8;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BIST_union_tag { /* PCI BIST register */
  vuint8_t R;
  struct {
    vuint8_t BIST:8;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BIST_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_union_tag { /* PCI cardbus CIS register */
  vuint32_t R;
  struct {
    vuint32_t CARDBUS_CIS_PTR:32;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_ID:16;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_union_tag { /* PCI expansion ROM base address register */
  vuint32_t R;
  struct {
    vuint32_t EXP_ROM_BA:32;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_VF4_PCI_HDR_TYPE0_tag {
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag PCI_CFH_LAT_TIMER; /* PCI latency timer register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BIST_tag PCI_CFH_BIST; /* PCI BIST register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag PCI_CFH_CARDBUS_CIS; /* PCI cardbus CIS register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag PCI_CFH_EXP_ROM_BA; /* PCI expansion ROM base address register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_VF4_PCI_HDR_TYPE0_reserved0[11];
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_VF4_PCI_HDR_TYPE0_reserved1[24];
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_VF4_PCI_HDR_TYPE0_reserved2[22];
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_VF4_PCI_HDR_TYPE0_reserved3[116];
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_VF4_PCI_HDR_TYPE0_reserved4[20];
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_VF4_PCI_HDR_TYPE0_reserved5[8];
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_VF5_PCI_HDR_TYPE0 =============
   ============================================================================ */

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:2;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_union_tag { /* PCI latency timer register */
  vuint8_t R;
  struct {
    vuint8_t LATENCY_TIMER:8;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BIST_union_tag { /* PCI BIST register */
  vuint8_t R;
  struct {
    vuint8_t BIST:8;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BIST_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_union_tag { /* PCI cardbus CIS register */
  vuint32_t R;
  struct {
    vuint32_t CARDBUS_CIS_PTR:32;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_ID:16;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_union_tag { /* PCI expansion ROM base address register */
  vuint32_t R;
  struct {
    vuint32_t EXP_ROM_BA:32;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_VF5_PCI_HDR_TYPE0_tag {
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag PCI_CFH_LAT_TIMER; /* PCI latency timer register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BIST_tag PCI_CFH_BIST; /* PCI BIST register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag PCI_CFH_CARDBUS_CIS; /* PCI cardbus CIS register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag PCI_CFH_EXP_ROM_BA; /* PCI expansion ROM base address register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_VF5_PCI_HDR_TYPE0_reserved0[11];
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_VF5_PCI_HDR_TYPE0_reserved1[24];
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_VF5_PCI_HDR_TYPE0_reserved2[22];
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_VF5_PCI_HDR_TYPE0_reserved3[116];
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_VF5_PCI_HDR_TYPE0_reserved4[20];
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_VF5_PCI_HDR_TYPE0_reserved5[8];
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_VF6_PCI_HDR_TYPE0 =============
   ============================================================================ */

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:2;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_union_tag { /* PCI latency timer register */
  vuint8_t R;
  struct {
    vuint8_t LATENCY_TIMER:8;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BIST_union_tag { /* PCI BIST register */
  vuint8_t R;
  struct {
    vuint8_t BIST:8;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BIST_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_union_tag { /* PCI cardbus CIS register */
  vuint32_t R;
  struct {
    vuint32_t CARDBUS_CIS_PTR:32;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_ID:16;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_union_tag { /* PCI expansion ROM base address register */
  vuint32_t R;
  struct {
    vuint32_t EXP_ROM_BA:32;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_VF6_PCI_HDR_TYPE0_tag {
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag PCI_CFH_LAT_TIMER; /* PCI latency timer register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BIST_tag PCI_CFH_BIST; /* PCI BIST register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag PCI_CFH_CARDBUS_CIS; /* PCI cardbus CIS register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag PCI_CFH_EXP_ROM_BA; /* PCI expansion ROM base address register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_VF6_PCI_HDR_TYPE0_reserved0[11];
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_VF6_PCI_HDR_TYPE0_reserved1[24];
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_VF6_PCI_HDR_TYPE0_reserved2[22];
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_VF6_PCI_HDR_TYPE0_reserved3[116];
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_VF6_PCI_HDR_TYPE0_reserved4[20];
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_VF6_PCI_HDR_TYPE0_reserved5[8];
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: NETC_VF7_PCI_HDR_TYPE0 =============
   ============================================================================ */

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:2;
    vuint16_t BUS_MASTER_EN:1;
    vuint16_t _unused_3:13;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CMD_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:4;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_STAT_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_union_tag { /* PCI latency timer register */
  vuint8_t R;
  struct {
    vuint8_t LATENCY_TIMER:8;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BIST_union_tag { /* PCI BIST register */
  vuint8_t R;
  struct {
    vuint8_t BIST:8;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BIST_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR0_union_tag { /* PCI base address register 0 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR1_union_tag { /* PCI base address register 1 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR2_union_tag { /* PCI base address register 2 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR3_union_tag { /* PCI base address register 3 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR4_union_tag { /* PCI base address register 4 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR5_union_tag { /* PCI base address register 5 */
  vuint32_t R;
  struct {
    vuint32_t MEM_IO_IND:1;
    vuint32_t MEM_TYPE:2;
    vuint32_t PF_MEM:1;
    vuint32_t ADDR:28;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_union_tag { /* PCI cardbus CIS register */
  vuint32_t R;
  struct {
    vuint32_t CARDBUS_CIS_PTR:32;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_VENDOR_ID:16;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SYBSYSTEM_ID:16;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_union_tag { /* PCI expansion ROM base address register */
  vuint32_t R;
  struct {
    vuint32_t EXP_ROM_BA:32;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_union_tag { /* PCI PCIe device control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:15;
    vuint16_t INIT_FLR:1;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_union_tag { /* PCI PCIe device capabilities 2 register */
  vuint32_t R;
  struct {
    vuint32_t CMPL_TO_RNG_SUPP:4;
    vuint32_t CMPL_TO_DIS_SUPP:1;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_union_tag { /* PCI PCIe device control 2 register */
  vuint16_t R;
  struct {
    vuint16_t CMPL_TO_VALUE:4;
    vuint16_t CMPL_TO_EN:1;
    vuint16_t _unused_5:11;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_union_tag { /* PCI MSI-X capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_PTR:8;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_union_tag { /* PCI MSI-X message control register */
  vuint16_t R;
  struct {
    vuint16_t TABLE_SIZE:11;
    vuint16_t _unused_11:3;
    vuint16_t FUNC_MASK:1;
    vuint16_t MSIX_EN:1;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_union_tag { /* PCI MSI-X table offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t TABLE_BIR:3;
    vuint32_t TABLE_OFFSET:29;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_union_tag { /* PCI MSI-X PBA offset/BIR register */
  vuint32_t R;
  struct {
    vuint32_t PBA_BIR:3;
    vuint32_t PBA_OFFSET:29;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_union_tag { /* PCIe AER uncorrectable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_union_tag { /* PCIe AER uncorrectable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_ERR_MASK:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_union_tag { /* PCIe AER uncorrectable error severity register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;
    vuint32_t UCORR_INT_SEV:1;
    vuint32_t _unused_23:9;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_union_tag { /* PCIe AER correctable error status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_ERR:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_union_tag { /* PCIe AER correctable error mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t CORR_INT_MASK:1;
    vuint32_t _unused_15:17;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_union_tag { /* PCIe AER capabilities and control register */
  vuint32_t R;
  struct {
    vuint32_t FIRST_ERR_PTR:5;
    vuint32_t _unused_5:27;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_union_tag { /* PCIe ACS capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_union_tag { /* PCIe ACS capability register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK:1;
    vuint16_t ACS_P2P_REQ_REDIR:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_union_tag { /* PCIe ACS control register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;
    vuint16_t ACS_TRANS_BLOCK_EN:1;
    vuint16_t ACS_P2P_REQ_REDIR_EN:1;
    vuint16_t _unused_3:3;
    vuint16_t ACS_DIR_TRANS_P2P_EN:1;
    vuint16_t _unused_7:9;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_union_tag { /* PCIe readiness time reporting capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_union_tag { /* PCIe RTR readiness time reporting 1 register */
  vuint32_t R;
  struct {
    vuint32_t RESET_TIME:12;           /* Reset Time */
    vuint32_t _unused_12:19;
    vuint32_t VALID:1;                 /* Valid */
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag;

typedef union NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_union_tag { /* PCIe RTR readiness time reporting 2 register */
  vuint32_t R;
  struct {
    vuint32_t FLR_TIME:12;             /* FLR Time */
    vuint32_t D3HOT_D0_TIME:12;        /* D3 hot to D0 time */
    vuint32_t _unused_24:8;
  } B;
} NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag;

struct NETC_VF7_PCI_HDR_TYPE0_tag {
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER_tag PCI_CFH_LAT_TIMER; /* PCI latency timer register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BIST_tag PCI_CFH_BIST; /* PCI BIST register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR0_tag PCI_CFH_BAR0; /* PCI base address register 0 */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR1_tag PCI_CFH_BAR1; /* PCI base address register 1 */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR2_tag PCI_CFH_BAR2; /* PCI base address register 2 */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR3_tag PCI_CFH_BAR3; /* PCI base address register 3 */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR4_tag PCI_CFH_BAR4; /* PCI base address register 4 */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR5_tag PCI_CFH_BAR5; /* PCI base address register 5 */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS_tag PCI_CFH_CARDBUS_CIS; /* PCI cardbus CIS register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA_tag PCI_CFH_EXP_ROM_BA; /* PCI expansion ROM base address register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t NETC_VF7_PCI_HDR_TYPE0_reserved0[11];
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL_tag PCI_CFC_PCIE_DEV_CTL; /* PCI PCIe device control register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t NETC_VF7_PCI_HDR_TYPE0_reserved1[24];
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2_tag PCI_CFC_PCIE_DEV_CAP2; /* PCI PCIe device capabilities 2 register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2_tag PCI_CFC_PCIE_DEV_CTL2; /* PCI PCIe device control 2 register */
  uint8_t NETC_VF7_PCI_HDR_TYPE0_reserved2[22];
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST_tag PCI_CFC_MSIX_CAP_LIST; /* PCI MSI-X capabilities list register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL_tag PCI_CFC_MSIX_MSG_CTL; /* PCI MSI-X message control register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_tag PCI_CFC_MSIX_TABLE_OFF_BIR; /* PCI MSI-X table offset/BIR register */
  NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_tag PCI_CFC_MSIX_PBA_OFF_BIR; /* PCI MSI-X PBA offset/BIR register */
  uint8_t NETC_VF7_PCI_HDR_TYPE0_reserved3[116];
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_tag PCIE_CFC_AER_UCORR_ERR_STAT; /* PCIe AER uncorrectable error status register */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_tag PCIE_CFC_AER_UCORR_ERR_MASK; /* PCIe AER uncorrectable error mask register */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_tag PCIE_CFC_AER_UCORR_ERR_SEV; /* PCIe AER uncorrectable error severity register */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_tag PCIE_CFC_AER_CORR_ERR_STAT; /* PCIe AER correctable error status register */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_tag PCIE_CFC_AER_CORR_ERR_MASK; /* PCIe AER correctable error mask register */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL_tag PCIE_CFC_AER_CAP_CTL; /* PCIe AER capabilities and control register */
  uint8_t NETC_VF7_PCI_HDR_TYPE0_reserved4[20];
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR_tag PCIE_CFC_ACS_CAP_HDR; /* PCIe ACS capability header */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_tag PCIE_CFC_ACS_CAP; /* PCIe ACS capability register */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL_tag PCIE_CFC_ACS_CTL; /* PCIe ACS control register */
  uint8_t NETC_VF7_PCI_HDR_TYPE0_reserved5[8];
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR_tag PCIE_CFC_RTR_CAP_HDR; /* PCIe readiness time reporting capability header */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1_tag PCIE_CFC_RTR_RTR1; /* PCIe RTR readiness time reporting 1 register */
  NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2_tag PCIE_CFC_RTR_RTR2; /* PCIe RTR readiness time reporting 2 register */
};


/* ============================================================================
   =============================== Module: OCOTP ==============================
   ============================================================================ */

typedef union OCOTP_CTRL_SYS_union_tag { /* System master's control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t CRC_TEST:1;              /* CRC (Polynomial CRC-32/MPEG-2) test for system master */
    vuint32_t RD_WR:4;                 /* eFuse read and write control for the system master */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:9;
    vuint32_t AUTH_KEY:16;             /* Key to unlock read and write operation */
  } B;
} OCOTP_CTRL_SYS_tag;

typedef union OCOTP_ADDR_SYS_union_tag { /* System master's eFuse address for read or write operation */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;                 /* eFuse read or write address */
    vuint32_t _unused_16:16;
  } B;
} OCOTP_ADDR_SYS_tag;

typedef union OCOTP_WRDATA_SYS_union_tag { /* System master's eFuse write data */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;                 /* eFuse data for eFuse write operation */
  } B;
} OCOTP_WRDATA_SYS_tag;

typedef union OCOTP_RDATA_SYS_union_tag { /* System master's eFuse read data */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;                 /* Read data */
  } B;
} OCOTP_RDATA_SYS_tag;

typedef union OCOTP_CTRL_FBX_union_tag { /* FBXC control */
  vuint32_t R;
  struct {
    vuint32_t SEC_RST:1;               /* Security reset for FBXC */
    vuint32_t _unused_1:7;
    vuint32_t SEC_MODE:1;              /* Indicates that the FBXC is in Secure mode. */
    vuint32_t _unused_9:7;
    vuint32_t _unused_16:1;
    vuint32_t IPG_STOP:1;              /* Low Power entry */
    vuint32_t IPG_STOP_ACK:1;          /* Low-power mode acknowledgment */
    vuint32_t _unused_19:13;
  } B;
} OCOTP_CTRL_FBX_tag;

typedef union OCOTP_CRC_RGN_SYS_union_tag { /* eFuse region for CRC validation */
  vuint32_t R;
  struct {
    vuint32_t START_ADDR:16;           /* Start eFuse address for CRC calculation */
    vuint32_t END_ADDR:16;             /* End eFuse address for CRC calculation */
  } B;
} OCOTP_CRC_RGN_SYS_tag;

typedef union OCOTP_CRC_ADDR_SYS_union_tag { /* Pre-calculated CRC eFuse address for comparison */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;                 /* CRC eFuse address used for comparison in CRC test */
    vuint32_t _unused_16:16;
  } B;
} OCOTP_CRC_ADDR_SYS_tag;

typedef union OCOTP_CRC_VALUE_SYS_union_tag { /* CRC calculated from the eFuses */
  vuint32_t R;
  struct {
    vuint32_t CRC:32;                  /* CRC value */
  } B;
} OCOTP_CRC_VALUE_SYS_tag;

typedef union OCOTP_GPR_union_tag {    /* General purpose */
  vuint32_t R;
  struct {
    vuint32_t GPR:32;                  /* General-purpose field that you can use to store information for your application */
  } B;
} OCOTP_GPR_tag;

typedef union OCOTP_STATUS_SYS_union_tag { /* System master's access status */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;                  /* OCOTP busy indication */
    vuint32_t CRC_FAIL:1;              /* CRC compare status */
    vuint32_t ERROR:1;                 /* Error status flag */
    vuint32_t _unused_3:29;
  } B;
} OCOTP_STATUS_SYS_tag;

typedef union OCOTP_VERSION_union_tag { /* OCOTP design version */
  vuint32_t R;
  struct {
    vuint32_t STEP:16;                 /* The step part of the design version */
    vuint32_t MINOR:8;                 /* The minor part of the design version */
    vuint32_t MAJOR:8;                 /* The major part of the design version */
  } B;
} OCOTP_VERSION_tag;

typedef union OCOTP_STR_union_tag {    /* Sticky */
  vuint32_t R;
  struct {
    vuint32_t STR:32;                  /* Sticky field for your use */
  } B;
} OCOTP_STR_tag;

typedef union OCOTP_SEC0_union_tag {   /* ECC status for single-bit ECC error */
  vuint32_t R;
  struct {
    vuint32_t SEC1:1;                  /* SEC error status */
    vuint32_t SEC2:1;                  /* SEC error status */
    vuint32_t SEC3:1;                  /* SEC error status */
    vuint32_t SEC4:1;                  /* SEC error status */
    vuint32_t SEC5:1;                  /* SEC error status */
    vuint32_t SEC6:1;                  /* SEC error status */
    vuint32_t SEC7:1;                  /* SEC error status */
    vuint32_t SEC8:1;                  /* SEC error status */
    vuint32_t SEC9:1;                  /* SEC error status */
    vuint32_t SEC10:1;                 /* SEC error status */
    vuint32_t SEC11:1;                 /* SEC error status */
    vuint32_t SEC12:1;                 /* SEC error status */
    vuint32_t SEC13:1;                 /* SEC error status */
    vuint32_t SEC14:1;                 /* SEC error status */
    vuint32_t SEC15:1;                 /* SEC error status */
    vuint32_t SEC16:1;                 /* SEC error status */
    vuint32_t SEC17:1;                 /* SEC error status */
    vuint32_t SEC18:1;                 /* SEC error status */
    vuint32_t SEC19:1;                 /* SEC error status */
    vuint32_t SEC20:1;                 /* SEC error status */
    vuint32_t SEC21:1;                 /* SEC error status */
    vuint32_t SEC22:1;                 /* SEC error status */
    vuint32_t SEC23:1;                 /* SEC error status */
    vuint32_t SEC24:1;                 /* SEC error status */
    vuint32_t SEC25:1;                 /* SEC error status */
    vuint32_t SEC26:1;                 /* SEC error status */
    vuint32_t SEC27:1;                 /* SEC error status */
    vuint32_t SEC28:1;                 /* SEC error status */
    vuint32_t SEC29:1;                 /* SEC error status */
    vuint32_t SEC30:1;                 /* SEC error status */
    vuint32_t SEC31:1;                 /* SEC error status */
    vuint32_t SEC32:1;                 /* SEC error status */
  } B;
} OCOTP_SEC0_tag;

typedef union OCOTP_SEC1_union_tag {   /* ECC status for single-bit ECC error */
  vuint32_t R;
  struct {
    vuint32_t SEC33:1;                 /* SEC error status */
    vuint32_t SEC34:1;                 /* SEC error status */
    vuint32_t SEC35:1;                 /* SEC error status */
    vuint32_t SEC36:1;                 /* SEC error status */
    vuint32_t SEC37:1;                 /* SEC error status */
    vuint32_t SEC38:1;                 /* SEC error status */
    vuint32_t SEC39:1;                 /* SEC error status */
    vuint32_t SEC40:1;                 /* SEC error status */
    vuint32_t SEC41:1;                 /* SEC error status */
    vuint32_t SEC42:1;                 /* SEC error status */
    vuint32_t SEC43:1;                 /* SEC error status */
    vuint32_t SEC44:1;                 /* SEC error status */
    vuint32_t SEC45:1;                 /* SEC error status */
    vuint32_t SEC46:1;                 /* SEC error status */
    vuint32_t SEC47:1;                 /* SEC error status */
    vuint32_t SEC48:1;                 /* SEC error status */
    vuint32_t SEC49:1;                 /* SEC error status */
    vuint32_t SEC50:1;                 /* SEC error status */
    vuint32_t SEC51:1;                 /* SEC error status */
    vuint32_t SEC52:1;                 /* SEC error status */
    vuint32_t SEC53:1;                 /* SEC error status */
    vuint32_t SEC54:1;                 /* SEC error status */
    vuint32_t SEC55:1;                 /* SEC error status */
    vuint32_t SEC56:1;                 /* SEC error status */
    vuint32_t SEC57:1;                 /* SEC error status */
    vuint32_t SEC58:1;                 /* SEC error status */
    vuint32_t SEC59:1;                 /* SEC error status */
    vuint32_t SEC60:1;                 /* SEC error status */
    vuint32_t SEC61:1;                 /* SEC error status */
    vuint32_t SEC62:1;                 /* SEC error status */
    vuint32_t SEC63:1;                 /* SEC error status */
    vuint32_t SEC64:1;                 /* SEC error status */
  } B;
} OCOTP_SEC1_tag;

typedef union OCOTP_SEC2_union_tag {   /* ECC status for single-bit ECC error */
  vuint32_t R;
  struct {
    vuint32_t SEC65:1;                 /* SEC error status */
    vuint32_t SEC66:1;                 /* SEC error status */
    vuint32_t SEC67:1;                 /* SEC error status */
    vuint32_t SEC68:1;                 /* SEC error status */
    vuint32_t SEC69:1;                 /* SEC error status */
    vuint32_t SEC70:1;                 /* SEC error status */
    vuint32_t SEC71:1;                 /* SEC error status */
    vuint32_t SEC72:1;                 /* SEC error status */
    vuint32_t SEC73:1;                 /* SEC error status */
    vuint32_t SEC74:1;                 /* SEC error status */
    vuint32_t SEC75:1;                 /* SEC error status */
    vuint32_t SEC76:1;                 /* SEC error status */
    vuint32_t SEC77:1;                 /* SEC error status */
    vuint32_t SEC78:1;                 /* SEC error status */
    vuint32_t SEC79:1;                 /* SEC error status */
    vuint32_t SEC80:1;                 /* SEC error status */
    vuint32_t SEC81:1;                 /* SEC error status */
    vuint32_t SEC82:1;                 /* SEC error status */
    vuint32_t SEC83:1;                 /* SEC error status */
    vuint32_t SEC84:1;                 /* SEC error status */
    vuint32_t SEC85:1;                 /* SEC error status */
    vuint32_t SEC86:1;                 /* SEC error status */
    vuint32_t SEC87:1;                 /* SEC error status */
    vuint32_t SEC88:1;                 /* SEC error status */
    vuint32_t SEC89:1;                 /* SEC error status */
    vuint32_t SEC90:1;                 /* SEC error status */
    vuint32_t SEC91:1;                 /* SEC error status */
    vuint32_t SEC92:1;                 /* SEC error status */
    vuint32_t SEC93:1;                 /* SEC error status */
    vuint32_t SEC94:1;                 /* SEC error status */
    vuint32_t SEC95:1;                 /* SEC error status */
    vuint32_t SEC96:1;                 /* SEC error status */
  } B;
} OCOTP_SEC2_tag;

typedef union OCOTP_SEC3_union_tag {   /* ECC status for single-bit ECC error */
  vuint32_t R;
  struct {
    vuint32_t SEC97:1;                 /* SEC error status */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} OCOTP_SEC3_tag;

typedef union OCOTP_DED0_union_tag {   /* ECC status for double-bit ECC error */
  vuint32_t R;
  struct {
    vuint32_t DED1:1;                  /* DED error status */
    vuint32_t DED2:1;                  /* DED error status */
    vuint32_t DED3:1;                  /* DED error status */
    vuint32_t DED4:1;                  /* DED error status */
    vuint32_t DED5:1;                  /* DED error status */
    vuint32_t DED6:1;                  /* DED error status */
    vuint32_t DED7:1;                  /* DED error status */
    vuint32_t DED8:1;                  /* DED error status */
    vuint32_t DED9:1;                  /* DED error status */
    vuint32_t DED10:1;                 /* DED error status */
    vuint32_t DED11:1;                 /* DED error status */
    vuint32_t DED12:1;                 /* DED error status */
    vuint32_t DED13:1;                 /* DED error status */
    vuint32_t DED14:1;                 /* DED error status */
    vuint32_t DED15:1;                 /* DED error status */
    vuint32_t DED16:1;                 /* DED error status */
    vuint32_t DED17:1;                 /* DED error status */
    vuint32_t DED18:1;                 /* DED error status */
    vuint32_t DED19:1;                 /* DED error status */
    vuint32_t DED20:1;                 /* DED error status */
    vuint32_t DED21:1;                 /* DED error status */
    vuint32_t DED22:1;                 /* DED error status */
    vuint32_t DED23:1;                 /* DED error status */
    vuint32_t DED24:1;                 /* DED error status */
    vuint32_t DED25:1;                 /* DED error status */
    vuint32_t DED26:1;                 /* DED error status */
    vuint32_t DED27:1;                 /* DED error status */
    vuint32_t DED28:1;                 /* DED error status */
    vuint32_t DED29:1;                 /* DED error status */
    vuint32_t DED30:1;                 /* DED error status */
    vuint32_t DED31:1;                 /* DED error status */
    vuint32_t DED32:1;                 /* DED error status */
  } B;
} OCOTP_DED0_tag;

typedef union OCOTP_DED1_union_tag {   /* ECC status for double-bit ECC error */
  vuint32_t R;
  struct {
    vuint32_t DED33:1;                 /* DED error status */
    vuint32_t DED34:1;                 /* DED error status */
    vuint32_t DED35:1;                 /* DED error status */
    vuint32_t DED36:1;                 /* DED error status */
    vuint32_t DED37:1;                 /* DED error status */
    vuint32_t DED38:1;                 /* DED error status */
    vuint32_t DED39:1;                 /* DED error status */
    vuint32_t DED40:1;                 /* DED error status */
    vuint32_t DED41:1;                 /* DED error status */
    vuint32_t DED42:1;                 /* DED error status */
    vuint32_t DED43:1;                 /* DED error status */
    vuint32_t DED44:1;                 /* DED error status */
    vuint32_t DED45:1;                 /* DED error status */
    vuint32_t DED46:1;                 /* DED error status */
    vuint32_t DED47:1;                 /* DED error status */
    vuint32_t DED48:1;                 /* DED error status */
    vuint32_t DED49:1;                 /* DED error status */
    vuint32_t DED50:1;                 /* DED error status */
    vuint32_t DED51:1;                 /* DED error status */
    vuint32_t DED52:1;                 /* DED error status */
    vuint32_t DED53:1;                 /* DED error status */
    vuint32_t DED54:1;                 /* DED error status */
    vuint32_t DED55:1;                 /* DED error status */
    vuint32_t DED56:1;                 /* DED error status */
    vuint32_t DED57:1;                 /* DED error status */
    vuint32_t DED58:1;                 /* DED error status */
    vuint32_t DED59:1;                 /* DED error status */
    vuint32_t DED60:1;                 /* DED error status */
    vuint32_t DED61:1;                 /* DED error status */
    vuint32_t DED62:1;                 /* DED error status */
    vuint32_t DED63:1;                 /* DED error status */
    vuint32_t DED64:1;                 /* DED error status */
  } B;
} OCOTP_DED1_tag;

typedef union OCOTP_DED2_union_tag {   /* ECC status for double-bit ECC error */
  vuint32_t R;
  struct {
    vuint32_t DED65:1;                 /* DED error status */
    vuint32_t DED66:1;                 /* DED error status */
    vuint32_t DED67:1;                 /* DED error status */
    vuint32_t DED68:1;                 /* DED error status */
    vuint32_t DED69:1;                 /* DED error status */
    vuint32_t DED70:1;                 /* DED error status */
    vuint32_t DED71:1;                 /* DED error status */
    vuint32_t DED72:1;                 /* DED error status */
    vuint32_t DED73:1;                 /* DED error status */
    vuint32_t DED74:1;                 /* DED error status */
    vuint32_t DED75:1;                 /* DED error status */
    vuint32_t DED76:1;                 /* DED error status */
    vuint32_t DED77:1;                 /* DED error status */
    vuint32_t DED78:1;                 /* DED error status */
    vuint32_t DED79:1;                 /* DED error status */
    vuint32_t DED80:1;                 /* DED error status */
    vuint32_t DED81:1;                 /* DED error status */
    vuint32_t DED82:1;                 /* DED error status */
    vuint32_t DED83:1;                 /* DED error status */
    vuint32_t DED84:1;                 /* DED error status */
    vuint32_t DED85:1;                 /* DED error status */
    vuint32_t DED86:1;                 /* DED error status */
    vuint32_t DED87:1;                 /* DED error status */
    vuint32_t DED88:1;                 /* DED error status */
    vuint32_t DED89:1;                 /* DED error status */
    vuint32_t DED90:1;                 /* DED error status */
    vuint32_t DED91:1;                 /* DED error status */
    vuint32_t DED92:1;                 /* DED error status */
    vuint32_t DED93:1;                 /* DED error status */
    vuint32_t DED94:1;                 /* DED error status */
    vuint32_t DED95:1;                 /* DED error status */
    vuint32_t DED96:1;                 /* DED error status */
  } B;
} OCOTP_DED2_tag;

typedef union OCOTP_DED3_union_tag {   /* ECC status for double-bit ECC error */
  vuint32_t R;
  struct {
    vuint32_t DED97:1;                 /* DED error status */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} OCOTP_DED3_tag;

typedef union OCOTP_ERR_INJCTR_union_tag { /* ECC error injector */
  vuint32_t R;
  struct {
    vuint32_t ADDR:12;                 /* The address of the bit in the shadow register where you want to inject the single-bit ECC error. The OCOTP inverts that bit. */
    vuint32_t _unused_12:4;
    vuint32_t _unused_16:16;
  } B;
} OCOTP_ERR_INJCTR_tag;

typedef union OCOTP_SHADOWS_union_tag { /* Shadow */
  vuint32_t R;
  struct {
    vuint32_t EFUSES:32;               /* OTP data */
  } B;
} OCOTP_SHADOWS_tag;

struct OCOTP_tag {
  OCOTP_CTRL_SYS_tag CTRL_SYS;         /* System master's control */
  OCOTP_ADDR_SYS_tag ADDR_SYS;         /* System master's eFuse address for read or write operation */
  OCOTP_WRDATA_SYS_tag WRDATA_SYS;     /* System master's eFuse write data */
  OCOTP_RDATA_SYS_tag RDATA_SYS;       /* System master's eFuse read data */
  uint8_t OCOTP_reserved0[32];
  OCOTP_CTRL_FBX_tag CTRL_FBX;         /* FBXC control */
  OCOTP_CRC_RGN_SYS_tag CRC_RGN_SYS;   /* eFuse region for CRC validation */
  OCOTP_CRC_ADDR_SYS_tag CRC_ADDR_SYS; /* Pre-calculated CRC eFuse address for comparison */
  OCOTP_CRC_VALUE_SYS_tag CRC_VALUE_SYS; /* CRC calculated from the eFuses */
  OCOTP_GPR_tag GPR[2];                /* General purpose */
  uint8_t OCOTP_reserved1[8];
  OCOTP_STATUS_SYS_tag STATUS_SYS;     /* System master's access status */
  uint8_t OCOTP_reserved2[8];
  OCOTP_VERSION_tag VERSION;           /* OCOTP design version */
  OCOTP_STR_tag STR[2];                /* Sticky */
  uint8_t OCOTP_reserved3[8];
  OCOTP_SEC0_tag SEC0;                 /* ECC status for single-bit ECC error */
  OCOTP_SEC1_tag SEC1;                 /* ECC status for single-bit ECC error */
  OCOTP_SEC2_tag SEC2;                 /* ECC status for single-bit ECC error */
  OCOTP_SEC3_tag SEC3;                 /* ECC status for single-bit ECC error */
  uint8_t OCOTP_reserved4[48];
  OCOTP_DED0_tag DED0;                 /* ECC status for double-bit ECC error */
  OCOTP_DED1_tag DED1;                 /* ECC status for double-bit ECC error */
  OCOTP_DED2_tag DED2;                 /* ECC status for double-bit ECC error */
  OCOTP_DED3_tag DED3;                 /* ECC status for double-bit ECC error */
  uint8_t OCOTP_reserved5[48];
  OCOTP_ERR_INJCTR_tag ERR_INJCTR;     /* ECC error injector */
  uint8_t OCOTP_reserved6[268];
  OCOTP_SHADOWS_tag SHADOWS[128];      /* Shadow */
};


/* ============================================================================
   =============================== Module: OMU ================================
   ============================================================================ */

typedef union OMU_Z0ZRSR_union_tag {   /* Zone Request Status 0 */
  vuint32_t R;
  struct {
    vuint32_t Z0RS:1;                  /* Zone 0 Request Status */
    vuint32_t Z1RS:1;                  /* Zone 1 Request Status */
    vuint32_t _unused_2:30;
  } B;
} OMU_Z0ZRSR_tag;

typedef union OMU_ZER0_union_tag {     /* Zone Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t Z0E:1;                   /* Zone 0 Enable */
    vuint32_t Z1E:1;                   /* Zone 1 Enable */
    vuint32_t _unused_2:30;
  } B;
} OMU_ZER0_tag;

typedef union OMU_Z1ZRSR_union_tag {   /* Zone Request Status 1 */
  vuint32_t R;
  struct {
    vuint32_t Z2RS:1;                  /* Zone 2 Request Status */
    vuint32_t Z3RS:1;                  /* Zone 3 Request Status */
    vuint32_t _unused_2:30;
  } B;
} OMU_Z1ZRSR_tag;

typedef union OMU_ZER1_union_tag {     /* Zone Enable 1 */
  vuint32_t R;
  struct {
    vuint32_t Z2E:1;                   /* Zone 2 Enable */
    vuint32_t Z3E:1;                   /* Zone 3 Enable */
    vuint32_t _unused_2:30;
  } B;
} OMU_ZER1_tag;

typedef union OMU_Z2ZRSR_union_tag {   /* Zone Request Status 2 */
  vuint32_t R;
  struct {
    vuint32_t Z4RS:1;                  /* Zone 4 Request Status */
    vuint32_t Z5RS:1;                  /* Zone 5 Request Status */
    vuint32_t _unused_2:30;
  } B;
} OMU_Z2ZRSR_tag;

typedef union OMU_ZER2_union_tag {     /* Zone Enable 2 */
  vuint32_t R;
  struct {
    vuint32_t Z4E:1;                   /* Zone 4 Enable */
    vuint32_t Z5E:1;                   /* Zone 5 Enable */
    vuint32_t _unused_2:30;
  } B;
} OMU_ZER2_tag;

typedef union OMU_Z3ZRSR_union_tag {   /* Zone Request Status 3 */
  vuint32_t R;
  struct {
    vuint32_t Z6RS:1;                  /* Zone 6 Request Status */
    vuint32_t Z7RS:1;                  /* Zone 7 Request Status */
    vuint32_t _unused_2:30;
  } B;
} OMU_Z3ZRSR_tag;

typedef union OMU_ZER3_union_tag {     /* Zone Enable 3 */
  vuint32_t R;
  struct {
    vuint32_t Z6E:1;                   /* Zone 6 Enable */
    vuint32_t Z7E:1;                   /* Zone 7 Enable */
    vuint32_t _unused_2:30;
  } B;
} OMU_ZER3_tag;

typedef union OMU_OER_union_tag {      /* OMU Enable */
  vuint32_t R;
  struct {
    vuint32_t OE:1;                    /* OMU Enable */
    vuint32_t _unused_1:31;
  } B;
} OMU_OER_tag;

typedef union OMU_ZRR_union_tag {      /* Zone Request */
  vuint32_t R;
  struct {
    vuint32_t Z0R:1;                   /* Zone 0 Request */
    vuint32_t Z1R:1;                   /* Zone 1 Request */
    vuint32_t Z2R:1;                   /* Zone 2 Request */
    vuint32_t Z3R:1;                   /* Zone 3 Request */
    vuint32_t Z4R:1;                   /* Zone 4 Request */
    vuint32_t Z5R:1;                   /* Zone 5 Request */
    vuint32_t Z6R:1;                   /* Zone 6 Request */
    vuint32_t Z7R:1;                   /* Zone 7 Request */
    vuint32_t _unused_8:24;
  } B;
} OMU_ZRR_tag;

typedef union OMU_ZESR_union_tag {     /* Zone Enable Status */
  vuint32_t R;
  struct {
    vuint32_t Z0ES:1;                  /* Zone 0 Enable Status */
    vuint32_t Z1ES:1;                  /* Zone 1 Enable Status */
    vuint32_t Z2ES:1;                  /* Zone 2 Enable Status */
    vuint32_t Z3ES:1;                  /* Zone 3 Enable Status */
    vuint32_t Z4ES:1;                  /* Zone 4 Enable Status */
    vuint32_t Z5ES:1;                  /* Zone 5 Enable Status */
    vuint32_t Z6ES:1;                  /* Zone 6 Enable Status */
    vuint32_t Z7ES:1;                  /* Zone 7 Enable Status */
    vuint32_t _unused_8:24;
  } B;
} OMU_ZESR_tag;

typedef union OMU_OVERLAY_REGION_DESCRIPTOR_ORDLSA_union_tag { /* ORD0 Logical Start Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:10;
    vuint32_t LSA:22;                  /* Logical Start Address */
  } B;
} OMU_OVERLAY_REGION_DESCRIPTOR_ORDLSA_tag;

typedef union OMU_OVERLAY_REGION_DESCRIPTOR_ORDPSA_union_tag { /* ORD0 Physical Start Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:10;
    vuint32_t PSA:22;                  /* Physical Start Address */
  } B;
} OMU_OVERLAY_REGION_DESCRIPTOR_ORDPSA_tag;

typedef union OMU_OVERLAY_REGION_DESCRIPTOR_ORDRS_union_tag { /* ORD0 Region Size */
  vuint32_t R;
  struct {
    vuint32_t RS:4;                    /* Region Size */
    vuint32_t _unused_4:28;
  } B;
} OMU_OVERLAY_REGION_DESCRIPTOR_ORDRS_tag;

typedef union OMU_OVERLAY_REGION_DESCRIPTOR_ORDRZA_union_tag { /* ORD0 Region Zone Assignment */
  vuint32_t R;
  struct {
    vuint32_t RZA:4;                   /* Region Zone Assignment */
    vuint32_t _unused_4:28;
  } B;
} OMU_OVERLAY_REGION_DESCRIPTOR_ORDRZA_tag;

typedef struct OMU_OVERLAY_REGION_DESCRIPTOR_struct_tag {
  OMU_OVERLAY_REGION_DESCRIPTOR_ORDLSA_tag ORDLSA; /* ORD0 Logical Start Address */
  OMU_OVERLAY_REGION_DESCRIPTOR_ORDPSA_tag ORDPSA; /* ORD0 Physical Start Address */
  OMU_OVERLAY_REGION_DESCRIPTOR_ORDRS_tag ORDRS; /* ORD0 Region Size */
  OMU_OVERLAY_REGION_DESCRIPTOR_ORDRZA_tag ORDRZA; /* ORD0 Region Zone Assignment */
} OMU_OVERLAY_REGION_DESCRIPTOR_tag;

struct OMU_tag {
  OMU_Z0ZRSR_tag Z0ZRSR;               /* Zone Request Status 0 */
  OMU_ZER0_tag ZER0;                   /* Zone Enable 0 */
  uint8_t OMU_reserved0[4088];
  OMU_Z1ZRSR_tag Z1ZRSR;               /* Zone Request Status 1 */
  OMU_ZER1_tag ZER1;                   /* Zone Enable 1 */
  uint8_t OMU_reserved1[4088];
  OMU_Z2ZRSR_tag Z2ZRSR;               /* Zone Request Status 2 */
  OMU_ZER2_tag ZER2;                   /* Zone Enable 2 */
  uint8_t OMU_reserved2[4088];
  OMU_Z3ZRSR_tag Z3ZRSR;               /* Zone Request Status 3 */
  OMU_ZER3_tag ZER3;                   /* Zone Enable 3 */
  uint8_t OMU_reserved3[4088];
  OMU_OER_tag OER;                     /* OMU Enable */
  OMU_ZRR_tag ZRR;                     /* Zone Request */
  OMU_ZESR_tag ZESR;                   /* Zone Enable Status */
  uint8_t OMU_reserved4[1012];
  OMU_OVERLAY_REGION_DESCRIPTOR_tag OVERLAY_REGION_DESCRIPTOR[64];
};


/* ============================================================================
   =============================== Module: PERIPH_PLL =========================
   ============================================================================ */

typedef union PERIPH_PLL_PLLCR_union_tag { /* PLL Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;            /* Reserved */
    vuint32_t PLLPD:1;                 /* PLL Power Down */
  } B;
} PERIPH_PLL_PLLCR_tag;

typedef union PERIPH_PLL_PLLSR_union_tag { /* PLL Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t LOCK:1;                  /* Lock Status */
    vuint32_t LOL:1;                   /* Loss-Of-Lock Flag */
    vuint32_t _unused_4:4;
    vuint32_t _unused_8:3;
    vuint32_t _unused_11:21;
  } B;
} PERIPH_PLL_PLLSR_tag;

typedef union PERIPH_PLL_PLLDV_union_tag { /* PLL Divider */
  vuint32_t R;
  struct {
    vuint32_t MFI:8;                   /* Integer Portion Of Loop Divider */
    vuint32_t _unused_8:4;
    vuint32_t RDIV:3;                  /* Input Clock Predivider */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:6;
    vuint32_t _unused_22:3;
    vuint32_t _unused_25:6;
    vuint32_t _unused_31:1;
  } B;
} PERIPH_PLL_PLLDV_tag;

typedef union PERIPH_PLL_PLLFD_union_tag { /* PLL Fractional Divider */
  vuint32_t R;
  struct {
    vuint32_t MFN:15;                  /* Numerator Of Fractional Loop Division Factor */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:4;
    vuint32_t _unused_22:6;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t SDMEN:1;                 /* Fractional Mode Enable */
    vuint32_t _unused_31:1;
  } B;
} PERIPH_PLL_PLLFD_tag;

typedef union PERIPH_PLL_PLLCLKMUX_union_tag { /* PLL Clock Multiplexer */
  vuint32_t R;
  struct {
    vuint32_t REFCLKSEL:1;             /* Reference Clock Select */
    vuint32_t _unused_1:31;
  } B;
} PERIPH_PLL_PLLCLKMUX_tag;

typedef union PERIPH_PLL_PLLODIV_union_tag { /* PLL Output Divider */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t DIV:8;                   /* Division Value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider Enable */
  } B;
} PERIPH_PLL_PLLODIV_tag;

struct PERIPH_PLL_tag {
  PERIPH_PLL_PLLCR_tag PLLCR;          /* PLL Control */
  PERIPH_PLL_PLLSR_tag PLLSR;          /* PLL Status */
  PERIPH_PLL_PLLDV_tag PLLDV;          /* PLL Divider */
  uint8_t PERIPH_PLL_reserved0[4];
  PERIPH_PLL_PLLFD_tag PLLFD;          /* PLL Fractional Divider */
  uint8_t PERIPH_PLL_reserved1[12];
  PERIPH_PLL_PLLCLKMUX_tag PLLCLKMUX;  /* PLL Clock Multiplexer */
  uint8_t PERIPH_PLL_reserved2[92];
  PERIPH_PLL_PLLODIV_tag PLLODIV[7];   /* PLL Output Divider */
};


/* ============================================================================
   =============================== Module: PIT ================================
   ============================================================================ */

typedef union PIT_MCR_union_tag {      /* PIT Module Control */
  vuint32_t R;
  struct {
    vuint32_t FRZ:1;                   /* Freeze */
    vuint32_t MDIS:1;                  /* Module Disable for PIT */
    vuint32_t _unused_2:30;
  } B;
} PIT_MCR_tag;

typedef union PIT_LTMR64H_union_tag {  /* PIT Upper Lifetimer */
  vuint32_t R;
  struct {
    vuint32_t LTH:32;                  /* Lifetimer Value */
  } B;
} PIT_LTMR64H_tag;

typedef union PIT_LTMR64L_union_tag {  /* PIT Lower Lifetimer */
  vuint32_t R;
  struct {
    vuint32_t LTL:32;                  /* Lifetimer Value */
  } B;
} PIT_LTMR64L_tag;

typedef union PIT_TIMER_LDVAL_union_tag { /* Timer Load Value */
  vuint32_t R;
  struct {
    vuint32_t TSV:32;                  /* Timer Start Value, merged from more bitFields with different positions or sizes */
  } B;
} PIT_TIMER_LDVAL_tag;

typedef union PIT_TIMER_CVAL_union_tag { /* Current Timer Value */
  vuint32_t R;
  struct {
    vuint32_t TVL:32;                  /* Timer Value */
  } B;
} PIT_TIMER_CVAL_tag;

typedef union PIT_TIMER_TCTRL_union_tag { /* Timer Control */
  vuint32_t R;
  struct {
    vuint32_t TEN:1;                   /* Timer Enable */
    vuint32_t TIE:1;                   /* Timer Interrupt Enable */
    vuint32_t CHN:1;                   /* Chain Mode */
    vuint32_t _unused_3:29;
  } B;
} PIT_TIMER_TCTRL_tag;

typedef union PIT_TIMER_TFLG_union_tag { /* Timer Flag */
  vuint32_t R;
  struct {
    vuint32_t TIF:1;                   /* Timer Interrupt Flag */
    vuint32_t _unused_1:31;
  } B;
} PIT_TIMER_TFLG_tag;

typedef struct PIT_TIMER_struct_tag {
  PIT_TIMER_LDVAL_tag LDVAL;           /* Timer Load Value */
  PIT_TIMER_CVAL_tag CVAL;             /* Current Timer Value */
  PIT_TIMER_TCTRL_tag TCTRL;           /* Timer Control */
  PIT_TIMER_TFLG_tag TFLG;             /* Timer Flag */
} PIT_TIMER_tag;

struct PIT_tag {
  PIT_MCR_tag MCR;                     /* PIT Module Control */
  uint8_t PIT_reserved0[220];
  PIT_LTMR64H_tag LTMR64H;             /* PIT Upper Lifetimer */
  PIT_LTMR64L_tag LTMR64L;             /* PIT Lower Lifetimer */
  uint8_t PIT_reserved1[24];
  PIT_TIMER_tag TIMER[7];
};


/* ============================================================================
   =============================== Module: PMC ================================
   ============================================================================ */

typedef union PMC_SSR_union_tag {      /* Source of System Reset */
  vuint32_t R;
  struct {
    vuint32_t POR_SUP:1;               /* POR on Core Supply CSPD Event Flag */
    vuint32_t CSPD_EVENT:1;            /* Critical Supply Presence Detector Event Flag */
    vuint32_t POR_WDOG_EVENT:1;        /* POR Watchdog Event Flag */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} PMC_SSR_tag;

typedef union PMC_NCSPD_CTL_union_tag { /* Non-Critical Supply Presence Detector Control */
  vuint32_t R;
  struct {
    vuint32_t NCSPD_CTL0:1;            /* NCSPD0 Control */
    vuint32_t NCSPD_CTL1:1;            /* NCSPD1 Control */
    vuint32_t NCSPD_CTL2:1;            /* NCSPD2 Control */
    vuint32_t NCSPD_CTL3:1;            /* NCSPD3 Control */
    vuint32_t NCSPD_CTL4:1;            /* NCSPD4 Control */
    vuint32_t NCSPD_CTL5:1;            /* NCSPD5 Control */
    vuint32_t NCSPD_CTL6:1;            /* NCSPD6 Control */
    vuint32_t NCSPD_CTL7:1;            /* NCSPD7 Control */
    vuint32_t NCSPD_CTL8:1;            /* NCSPD8 Control */
    vuint32_t NCSPD_CTL9:1;            /* NCSPD9 Control */
    vuint32_t NCSPD_CTL10:1;           /* NCSPD10 Control */
    vuint32_t NCSPD_CTL11:1;           /* NCSPD11 Control */
    vuint32_t NCSPD_CTL12:1;           /* NCSPD12 Control */
    vuint32_t NCSPD_CTL13:1;           /* NCSPD13 Control */
    vuint32_t NCSPD_CTL14:1;           /* NCSPD14 Control */
    vuint32_t NCSPD_CTL15:1;           /* NCSPD15 Control */
    vuint32_t NCSPD_CTL16:1;           /* NCSPD16 Control */
    vuint32_t NCSPD_CTL17:1;           /* NCSPD17 Control */
    vuint32_t NCSPD_CTL18:1;           /* NCSPD18 Control */
    vuint32_t NCSPD_CTL19:1;           /* NCSPD19 Control */
    vuint32_t NCSPD_CTL20:1;           /* NCSPD20 Control */
    vuint32_t NCSPD_CTL21:1;           /* NCSPD21 Control */
    vuint32_t NCSPD_CTL22:1;           /* NCSPD22 Control */
    vuint32_t NCSPD_CTL23:1;           /* NCSPD23 Control */
    vuint32_t NCSPD_CTL24:1;           /* NCSPD24 Control */
    vuint32_t NCSPD_CTL25:1;           /* NCSPD25 Control */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} PMC_NCSPD_CTL_tag;

typedef union PMC_NCSPD_STAT_union_tag { /* NCSPD Status Register */
  vuint32_t R;
  struct {
    vuint32_t NCSPD_STAT0:1;           /* NCSPD0 Status */
    vuint32_t NCSPD_STAT1:1;           /* NCSPD1 Status */
    vuint32_t NCSPD_STAT2:1;           /* NCSPD2 Status */
    vuint32_t NCSPD_STAT3:1;           /* NCSPD3 Status */
    vuint32_t NCSPD_STAT4:1;           /* NCSPD4 Status */
    vuint32_t NCSPD_STAT5:1;           /* NCSPD5 Status */
    vuint32_t NCSPD_STAT6:1;           /* NCSPD6 Status */
    vuint32_t NCSPD_STAT7:1;           /* NCSPD7 Status */
    vuint32_t NCSPD_STAT8:1;           /* NCSPD8 Status */
    vuint32_t NCSPD_STAT9:1;           /* NCSPD9 Status */
    vuint32_t NCSPD_STAT10:1;          /* NCSPD10 Status */
    vuint32_t NCSPD_STAT11:1;          /* NCSPD11 Status */
    vuint32_t NCSPD_STAT12:1;          /* NCSPD12 Status */
    vuint32_t NCSPD_STAT13:1;          /* NCSPD13 Status */
    vuint32_t NCSPD_STAT14:1;          /* NCSPD14 Status */
    vuint32_t NCSPD_STAT15:1;          /* NCSPD15 Status */
    vuint32_t NCSPD_STAT16:1;          /* NCSPD16 Status */
    vuint32_t NCSPD_STAT17:1;          /* NCSPD17 Status */
    vuint32_t NCSPD_STAT18:1;          /* NCSPD18 Status */
    vuint32_t NCSPD_STAT19:1;          /* NCSPD19 Status */
    vuint32_t NCSPD_STAT20:1;          /* NCSPD20 Status */
    vuint32_t NCSPD_STAT21:1;          /* NCSPD21 Status */
    vuint32_t NCSPD_STAT22:1;          /* NCSPD22 Status */
    vuint32_t NCSPD_STAT23:1;          /* NCSPD23 Status */
    vuint32_t NCSPD_STAT24:1;          /* NCSPD24 Status */
    vuint32_t NCSPD_STAT25:1;          /* NCSPD25 Status */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} PMC_NCSPD_STAT_tag;

typedef union PMC_CSPD_EVENT_CAPTURE_union_tag { /* CSPD Event Capture (CSPDEF) */
  vuint32_t R;
  struct {
    vuint32_t HVCPOREF:1;              /* PMC Supply CSPD Event Flag */
    vuint32_t CSPD_EVENT_CAPTURE0:1;   /* CSPD0 Event Flag */
    vuint32_t CSPD_EVENT_CAPTURE1:1;   /* CSPD1 Event Flag */
    vuint32_t CSPD_EVENT_CAPTURE2:1;   /* CSPD2 Event Flag */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} PMC_CSPD_EVENT_CAPTURE_tag;

typedef union PMC_NCSPD_EVENT_CAPTURE_union_tag { /* NCSPD Event Flag (NCSPDEF) */
  vuint32_t R;
  struct {
    vuint32_t NCSPD_EVENT_CAPTURE0:1;  /* NCSPD0 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE1:1;  /* NCSPD1 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE2:1;  /* NCSPD2 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE3:1;  /* NCSPD3 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE4:1;  /* NCSPD4 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE5:1;  /* NCSPD5 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE6:1;  /* NCSPD6 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE7:1;  /* NCSPD7 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE8:1;  /* NCSPD8 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE9:1;  /* NCSPD9 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE10:1; /* NCSPD10 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE11:1; /* NCSPD11 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE12:1; /* NCSPD12 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE13:1; /* NCSPD13 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE14:1; /* NCSPD14 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE15:1; /* NCSPD15 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE16:1; /* NCSPD16 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE17:1; /* NCSPD17 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE18:1; /* NCSPD18 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE19:1; /* NCSPD19 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE20:1; /* NCSPD20 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE21:1; /* NCSPD21 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE22:1; /* NCSPD22 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE23:1; /* NCSPD23 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE24:1; /* NCSPD24 Event Flag */
    vuint32_t NCSPD_EVENT_CAPTURE25:1; /* NCSPD25 Event Flag */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} PMC_NCSPD_EVENT_CAPTURE_tag;

typedef union PMC_POR_WDOG_EVENT_CAPTURE0_union_tag { /* Device Status Flag (DSF0) */
  vuint32_t R;
  struct {
    vuint32_t DSFBIT0:32;
  } B;
} PMC_POR_WDOG_EVENT_CAPTURE0_tag;

typedef union PMC_POR_WDOG_EVENT_CAPTURE1_union_tag { /* Device Status Flag (DSF1) */
  vuint32_t R;
  struct {
    vuint32_t DSFBIT1:32;
  } B;
} PMC_POR_WDOG_EVENT_CAPTURE1_tag;

typedef union PMC_POR_WDOG_EVENT_CAPTURE2_union_tag { /* Device Status Flag (DSF2) */
  vuint32_t R;
  struct {
    vuint32_t DSFBIT2:32;
  } B;
} PMC_POR_WDOG_EVENT_CAPTURE2_tag;

struct PMC_tag {
  PMC_SSR_tag SSR;                     /* Source of System Reset */
  PMC_NCSPD_CTL_tag NCSPD_CTL;         /* Non-Critical Supply Presence Detector Control */
  uint8_t PMC_reserved0[4];
  PMC_NCSPD_STAT_tag NCSPD_STAT;       /* NCSPD Status Register */
  PMC_CSPD_EVENT_CAPTURE_tag CSPD_EVENT_CAPTURE; /* CSPD Event Capture (CSPDEF) */
  PMC_NCSPD_EVENT_CAPTURE_tag NCSPD_EVENT_CAPTURE; /* NCSPD Event Flag (NCSPDEF) */
  uint8_t PMC_reserved1[4];
  PMC_POR_WDOG_EVENT_CAPTURE0_tag POR_WDOG_EVENT_CAPTURE0; /* Device Status Flag (DSF0) */
  PMC_POR_WDOG_EVENT_CAPTURE1_tag POR_WDOG_EVENT_CAPTURE1; /* Device Status Flag (DSF1) */
  PMC_POR_WDOG_EVENT_CAPTURE2_tag POR_WDOG_EVENT_CAPTURE2; /* Device Status Flag (DSF2) */
};


/* ============================================================================
   =============================== Module: PMSS_SAFETY ========================
   ============================================================================ */

typedef union PMSS_SAFETY_P_UCSERR_union_tag { /* PMSS UCS ERR Register */
  vuint32_t R;
  struct {
    vuint32_t P_UCSERR:1;
    vuint32_t _unused_1:4;
    vuint32_t P_CORE_PERR:1;
    vuint32_t P_MSS_PERR:1;
    vuint32_t _unused_7:9;
    vuint32_t P_UCECNTEV:1;
    vuint32_t P_UCECNTOV:1;
    vuint32_t _unused_18:14;
  } B;
} PMSS_SAFETY_P_UCSERR_tag;

typedef union PMSS_SAFETY_P_CSERR_union_tag { /* PMSS CS ERR Register */
  vuint32_t R;
  struct {
    vuint32_t P_CSERR:1;
    vuint32_t P_ECC_COR:1;
    vuint32_t P_CTECC_COR:1;
    vuint32_t P_BTBECC_COR:1;
    vuint32_t _unused_4:12;
    vuint32_t P_CECNTEV:1;
    vuint32_t P_CECNTOV:1;
    vuint32_t _unused_18:14;
  } B;
} PMSS_SAFETY_P_CSERR_tag;

typedef union PMSS_SAFETY_P_UCSERR_S_union_tag { /* PMSS UCS ERR Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t P_CORE_PERR_S:1;
    vuint32_t P_MSS_PERR_S:1;
    vuint32_t _unused_7:25;
  } B;
} PMSS_SAFETY_P_UCSERR_S_tag;

typedef union PMSS_SAFETY_P_CSERR_S_union_tag { /* PMSS CE ERR Shadow Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t P_ECC_COR_S:1;
    vuint32_t P_CTECC_COR_S:1;
    vuint32_t P_BTBECC_COR_S:1;
    vuint32_t _unused_4:28;
  } B;
} PMSS_SAFETY_P_CSERR_S_tag;

typedef union PMSS_SAFETY_P_CSERR_M_union_tag { /* PMSS CS ERR Mask Register */
  vuint32_t R;
  struct {
    vuint32_t P_CSERR_M:1;
    vuint32_t P_ECC_COR_M:1;
    vuint32_t P_CTECC_COR_M:1;
    vuint32_t P_BTBECC_COR_M:1;
    vuint32_t _unused_4:28;
  } B;
} PMSS_SAFETY_P_CSERR_M_tag;

typedef union PMSS_SAFETY_P_ECC_TCFG_union_tag { /* PMSS ECC TCFG Register */
  vuint32_t R;
  struct {
    vuint32_t P_FLIPCFG:32;
  } B;
} PMSS_SAFETY_P_ECC_TCFG_tag;

typedef union PMSS_SAFETY_P_ECC_TCFG_2_union_tag { /* PMSS ECC TCFG 2 Register */
  vuint32_t R;
  struct {
    vuint32_t P_FLIPCFGE:7;
    vuint32_t _unused_7:24;
    vuint32_t P_TRIG:1;
  } B;
} PMSS_SAFETY_P_ECC_TCFG_2_tag;

typedef union PMSS_SAFETY_P_CTECC_TCFG_union_tag { /* PMSS CT ECC TCFG Register */
  vuint32_t R;
  struct {
    vuint32_t PT_FLIPCFG:11;
    vuint32_t PT_FLIPCFGE:4;
    vuint32_t _unused_15:16;
    vuint32_t PT_TRIG:1;
  } B;
} PMSS_SAFETY_P_CTECC_TCFG_tag;

typedef union PMSS_SAFETY_P_BTBECC_TCFG_union_tag { /* PMSS BTB ECC TCFG Register */
  vuint32_t R;
  struct {
    vuint32_t PB_FLIPCFG:11;
    vuint32_t PB_FLIPCFGE:4;
    vuint32_t _unused_15:16;
    vuint32_t PB_TRIG:1;
  } B;
} PMSS_SAFETY_P_BTBECC_TCFG_tag;

typedef union PMSS_SAFETY_P_UCSERR_CNT_union_tag { /* PMSS UCSERR CNT Register */
  vuint32_t R;
  struct {
    vuint32_t P_UCSERR_CNT_V:16;
    vuint32_t _unused_16:16;
  } B;
} PMSS_SAFETY_P_UCSERR_CNT_tag;

typedef union PMSS_SAFETY_P_CSERR_CNT_union_tag { /* PMSS CSERR CNT Register */
  vuint32_t R;
  struct {
    vuint32_t P_CSERR_CNT_V:16;
    vuint32_t _unused_16:16;
  } B;
} PMSS_SAFETY_P_CSERR_CNT_tag;

typedef union PMSS_SAFETY_P_LPCS_union_tag { /* PMSS LPCS Register */
  vuint32_t R;
  struct {
    vuint32_t TS:1;
    vuint32_t _unused_1:3;
    vuint32_t WSEL:2;
    vuint32_t _unused_6:26;
  } B;
} PMSS_SAFETY_P_LPCS_tag;

struct PMSS_SAFETY_tag {
  PMSS_SAFETY_P_UCSERR_tag P_UCSERR;   /* PMSS UCS ERR Register */
  uint8_t PMSS_SAFETY_reserved0[4];
  PMSS_SAFETY_P_CSERR_tag P_CSERR;     /* PMSS CS ERR Register */
  uint8_t PMSS_SAFETY_reserved1[4];
  PMSS_SAFETY_P_UCSERR_S_tag P_UCSERR_S; /* PMSS UCS ERR Shadow Register */
  PMSS_SAFETY_P_CSERR_S_tag P_CSERR_S; /* PMSS CE ERR Shadow Register */
  uint8_t PMSS_SAFETY_reserved2[4];
  PMSS_SAFETY_P_CSERR_M_tag P_CSERR_M; /* PMSS CS ERR Mask Register */
  uint8_t PMSS_SAFETY_reserved3[16];
  PMSS_SAFETY_P_ECC_TCFG_tag P_ECC_TCFG; /* PMSS ECC TCFG Register */
  PMSS_SAFETY_P_ECC_TCFG_2_tag P_ECC_TCFG_2; /* PMSS ECC TCFG 2 Register */
  uint8_t PMSS_SAFETY_reserved4[8];
  PMSS_SAFETY_P_CTECC_TCFG_tag P_CTECC_TCFG; /* PMSS CT ECC TCFG Register */
  uint8_t PMSS_SAFETY_reserved5[12];
  PMSS_SAFETY_P_BTBECC_TCFG_tag P_BTBECC_TCFG; /* PMSS BTB ECC TCFG Register */
  uint8_t PMSS_SAFETY_reserved6[28];
  PMSS_SAFETY_P_UCSERR_CNT_tag P_UCSERR_CNT; /* PMSS UCSERR CNT Register */
  PMSS_SAFETY_P_CSERR_CNT_tag P_CSERR_CNT; /* PMSS CSERR CNT Register */
  PMSS_SAFETY_P_LPCS_tag P_LPCS;       /* PMSS LPCS Register */
};


/* ============================================================================
   =============================== Module: POWER_SCALING_UNIT =================
   ============================================================================ */

typedef union POWER_SCALING_UNIT_PSVM_union_tag { /* PSU Power Save Mode Register */
  vuint32_t R;
  struct {
    vuint32_t PMOD:2;
    vuint32_t DBC:1;
    vuint32_t _unused_3:29;
  } B;
} POWER_SCALING_UNIT_PSVM_tag;

typedef union POWER_SCALING_UNIT_PGR_union_tag { /* PSU General Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t MSS_AE:1;
    vuint32_t EDAP_AE:1;
    vuint32_t _unused_4:3;
    vuint32_t QMAN_AE:1;
    vuint32_t _unused_8:24;
  } B;
} POWER_SCALING_UNIT_PGR_tag;

struct POWER_SCALING_UNIT_tag {
  POWER_SCALING_UNIT_PSVM_tag PSVM;    /* PSU Power Save Mode Register */
  POWER_SCALING_UNIT_PGR_tag PGR;      /* PSU General Register */
};


/* ============================================================================
   =============================== Module: PROFILER ===========================
   ============================================================================ */

typedef union PROFILER_PROF_CTL_union_tag { /* PROF CTL Register */
  vuint32_t R;
  struct {
    vuint32_t FRCC_RST:1;
    vuint32_t PROF_CNT0_RST:1;
    vuint32_t PROF_CNT1_RST:1;
    vuint32_t PROF_CNT2_RST:1;
    vuint32_t PROF_CNT3_RST:1;
    vuint32_t PROF_CNT4_RST:1;
    vuint32_t PROF_CNT5_RST:1;
    vuint32_t PROF_CNT6_RST:1;
    vuint32_t PROF_CNT7_RST:1;
    vuint32_t _unused_9:23;
  } B;
} PROFILER_PROF_CTL_tag;

typedef union PROFILER_PROF_PAUSE_union_tag { /* PROF PAUSE Register */
  vuint32_t R;
  struct {
    vuint32_t FRCC_PAUSE:1;
    vuint32_t PROF_CNT0_PAUSE:1;
    vuint32_t PROF_CNT1_PAUSE:1;
    vuint32_t PROF_CNT2_PAUSE:1;
    vuint32_t PROF_CNT3_PAUSE:1;
    vuint32_t PROF_CNT4_PAUSE:1;
    vuint32_t PROF_CNT5_PAUSE:1;
    vuint32_t PROF_CNT6_PAUSE:1;
    vuint32_t PROF_CNT7_PAUSE:1;
    vuint32_t _unused_9:23;
  } B;
} PROFILER_PROF_PAUSE_tag;

typedef union PROFILER_PROF_CTRL0_union_tag { /* PROF CTRL0 Register */
  vuint32_t R;
  struct {
    vuint32_t PROF_CNT0_SEL:8;
    vuint32_t PROF_CNT1_SEL:8;
    vuint32_t PROF_CNT2_SEL:8;
    vuint32_t PROF_CNT3_SEL:8;
  } B;
} PROFILER_PROF_CTRL0_tag;

typedef union PROFILER_PROF_CTRL1_union_tag { /* PROF CTRL1 Register */
  vuint32_t R;
  struct {
    vuint32_t PROF_CNT4_SEL:8;
    vuint32_t PROF_CNT5_SEL:8;
    vuint32_t PROF_CNT6_SEL:8;
    vuint32_t PROF_CNT7_SEL:8;
  } B;
} PROFILER_PROF_CTRL1_tag;

typedef union PROFILER_FRCC_union_tag { /* FRCC Register */
  vuint32_t R;
  struct {
    vuint32_t FRCC:32;
  } B;
} PROFILER_FRCC_tag;

typedef union PROFILER_PROF_CNTX_union_tag { /* PROF CNTx Register */
  vuint32_t R;
  struct {
    vuint32_t PROF_CNTX:32;
  } B;
} PROFILER_PROF_CNTX_tag;

typedef union PROFILER_PROF_EV_MASKX_union_tag { /* PROF EV MASKx Register */
  vuint32_t R;
  struct {
    vuint32_t PROF_MF0:8;
    vuint32_t PROF_MF1:8;
    vuint32_t PROF_MF2:4;
    vuint32_t PROF_MF3:4;
    vuint32_t PROF_MF4:4;
    vuint32_t PROF_MF5:4;
  } B;
} PROFILER_PROF_EV_MASKX_tag;

struct PROFILER_tag {
  PROFILER_PROF_CTL_tag PROF_CTL;      /* PROF CTL Register */
  PROFILER_PROF_PAUSE_tag PROF_PAUSE;  /* PROF PAUSE Register */
  PROFILER_PROF_CTRL0_tag PROF_CTRL0;  /* PROF CTRL0 Register */
  PROFILER_PROF_CTRL1_tag PROF_CTRL1;  /* PROF CTRL1 Register */
  PROFILER_FRCC_tag FRCC;              /* FRCC Register */
  uint8_t PROFILER_reserved0[12];
  PROFILER_PROF_CNTX_tag PROF_CNTX[8]; /* PROF CNTx Register */
  uint8_t PROFILER_reserved1[64];
  PROFILER_PROF_EV_MASKX_tag PROF_EV_MASKX[8]; /* PROF EV MASKx Register */
};


/* ============================================================================
   =============================== Module: PROGRAM_MEMORY_SUBSYSTEM ===========
   ============================================================================ */

typedef union PROGRAM_MEMORY_SUBSYSTEM_MSS_PCR_union_tag { /* Program Control */
  vuint32_t R;
  struct {
    vuint32_t PAPE:1;
    vuint32_t _unused_1:1;
    vuint32_t CAC_PFE:1;
    vuint32_t _unused_3:3;
    vuint32_t CLBE:1;
    vuint32_t PCAC_EN:1;
    vuint32_t HWPF_SZ:2;
    vuint32_t _unused_10:2;
    vuint32_t CAC_WPE:1;
    vuint32_t _unused_13:19;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_MSS_PCR_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_SYS_OU_C_union_tag { /* PMSS AXI Outstanding Configuration */
  vuint32_t R;
  struct {
    vuint32_t TOTAL_OS:4;
    vuint32_t _unused_4:4;
    vuint32_t SW_OS:4;
    vuint32_t _unused_12:20;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_SYS_OU_C_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_HDCFG_union_tag { /* PMSS Hardware Configuration */
  vuint32_t R;
  struct {
    vuint32_t PTCM_SZE:3;
    vuint32_t _unused_3:1;
    vuint32_t PCAC_SZE:3;
    vuint32_t _unused_7:1;
    vuint32_t PECC:1;
    vuint32_t EPP_AXI_WID:1;
    vuint32_t _unused_10:20;
    vuint32_t L1ICA:2;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_HDCFG_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_PORT_ID_union_tag { /* Program Port AXI ID */
  vuint32_t R;
  struct {
    vuint32_t PDMA_ID:4;
    vuint32_t PF_ID:4;
    vuint32_t OCEM_ID:4;
    vuint32_t _unused_12:20;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_PORT_ID_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD0_START_union_tag { /* PMSS ADD0 START */
  vuint32_t R;
  struct {
    vuint32_t P_REGION_START:20;
    vuint32_t _unused_20:8;
    vuint32_t INACTIVE:1;
    vuint32_t _unused_29:3;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD0_START_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD0_ATT0_union_tag { /* PMSS ADD0 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t L1IC:1;
    vuint32_t L1IC_LOCK:1;
    vuint32_t _unused_2:2;
    vuint32_t P_AP:2;
    vuint32_t _unused_6:2;
    vuint32_t P_L2A:4;
    vuint32_t _unused_12:12;
    vuint32_t P_RQOS:4;
    vuint32_t _unused_28:4;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD0_ATT0_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD1_START_union_tag { /* PMSS ADD1 START */
  vuint32_t R;
  struct {
    vuint32_t P_REGION_START:20;
    vuint32_t _unused_20:8;
    vuint32_t INACTIVE:1;
    vuint32_t _unused_29:3;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD1_START_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD1_ATT0_union_tag { /* PMSS ADD1 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t L1IC:1;
    vuint32_t L1IC_LOCK:1;
    vuint32_t _unused_2:2;
    vuint32_t P_AP:2;
    vuint32_t _unused_6:2;
    vuint32_t P_L2A:4;
    vuint32_t _unused_12:12;
    vuint32_t P_RQOS:4;
    vuint32_t _unused_28:4;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD1_ATT0_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD2_START_union_tag { /* PMSS ADD2 START */
  vuint32_t R;
  struct {
    vuint32_t P_REGION_START:20;
    vuint32_t _unused_20:8;
    vuint32_t INACTIVE:1;
    vuint32_t _unused_29:3;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD2_START_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD2_ATT0_union_tag { /* PMSS ADD2 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t L1IC:1;
    vuint32_t L1IC_LOCK:1;
    vuint32_t _unused_2:2;
    vuint32_t P_AP:2;
    vuint32_t _unused_6:2;
    vuint32_t P_L2A:4;
    vuint32_t _unused_12:12;
    vuint32_t P_RQOS:4;
    vuint32_t _unused_28:4;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD2_ATT0_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD3_START_union_tag { /* PMSS ADD3 START */
  vuint32_t R;
  struct {
    vuint32_t P_REGION_START:20;
    vuint32_t _unused_20:8;
    vuint32_t INACTIVE:1;
    vuint32_t _unused_29:3;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD3_START_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD3_ATT0_union_tag { /* PMSS ADD3 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t L1IC:1;
    vuint32_t L1IC_LOCK:1;
    vuint32_t _unused_2:2;
    vuint32_t P_AP:2;
    vuint32_t _unused_6:2;
    vuint32_t P_L2A:4;
    vuint32_t _unused_12:12;
    vuint32_t P_RQOS:4;
    vuint32_t _unused_28:4;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD3_ATT0_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD4_START_union_tag { /* PMSS ADD4 START */
  vuint32_t R;
  struct {
    vuint32_t P_REGION_START:20;
    vuint32_t _unused_20:8;
    vuint32_t INACTIVE:1;
    vuint32_t _unused_29:3;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD4_START_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD4_ATT0_union_tag { /* PMSS ADD4 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t L1IC:1;
    vuint32_t L1IC_LOCK:1;
    vuint32_t _unused_2:2;
    vuint32_t P_AP:2;
    vuint32_t _unused_6:2;
    vuint32_t P_L2A:4;
    vuint32_t _unused_12:12;
    vuint32_t P_RQOS:4;
    vuint32_t _unused_28:4;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD4_ATT0_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD5_START_union_tag { /* PMSS ADD5 START */
  vuint32_t R;
  struct {
    vuint32_t P_REGION_START:20;
    vuint32_t _unused_20:8;
    vuint32_t INACTIVE:1;
    vuint32_t _unused_29:3;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD5_START_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD5_ATT0_union_tag { /* PMSS ADD5 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t L1IC:1;
    vuint32_t L1IC_LOCK:1;
    vuint32_t _unused_2:2;
    vuint32_t P_AP:2;
    vuint32_t _unused_6:2;
    vuint32_t P_L2A:4;
    vuint32_t _unused_12:12;
    vuint32_t P_RQOS:4;
    vuint32_t _unused_28:4;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD5_ATT0_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD6_START_union_tag { /* PMSS ADD6 START */
  vuint32_t R;
  struct {
    vuint32_t P_REGION_START:20;
    vuint32_t _unused_20:8;
    vuint32_t INACTIVE:1;
    vuint32_t _unused_29:3;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD6_START_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD6_ATT0_union_tag { /* PMSS ADD6 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t L1IC:1;
    vuint32_t L1IC_LOCK:1;
    vuint32_t _unused_2:2;
    vuint32_t P_AP:2;
    vuint32_t _unused_6:2;
    vuint32_t P_L2A:4;
    vuint32_t _unused_12:12;
    vuint32_t P_RQOS:4;
    vuint32_t _unused_28:4;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD6_ATT0_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD7_START_union_tag { /* PMSS ADD7 START */
  vuint32_t R;
  struct {
    vuint32_t P_REGION_START:20;
    vuint32_t _unused_20:8;
    vuint32_t INACTIVE:1;
    vuint32_t _unused_29:3;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD7_START_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ADD7_ATT0_union_tag { /* PMSS ADD7 ATT0 */
  vuint32_t R;
  struct {
    vuint32_t L1IC:1;
    vuint32_t L1IC_LOCK:1;
    vuint32_t _unused_2:2;
    vuint32_t P_AP:2;
    vuint32_t _unused_6:2;
    vuint32_t P_L2A:4;
    vuint32_t _unused_12:12;
    vuint32_t P_RQOS:4;
    vuint32_t _unused_28:4;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ADD7_ATT0_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_CCOSAR_union_tag { /* Program Cache Software Operations */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;
    vuint32_t P_CCOSA:26;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_CCOSAR_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_CCOCR_union_tag { /* Program Cache Software Operations Control */
  vuint32_t R;
  struct {
    vuint32_t P_ACTIVE:1;
    vuint32_t P_L1ICO:1;
    vuint32_t P_OT:4;
    vuint32_t _unused_6:1;
    vuint32_t P_OS:1;
    vuint32_t _unused_8:8;
    vuint32_t P_NOBPL:16;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_CCOCR_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_P_ECADD_union_tag { /* Program ECC Error Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ECADD:27;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_P_ECADD_tag;

typedef union PROGRAM_MEMORY_SUBSYSTEM_PMSSACS_union_tag { /* PMSS Access Control */
  vuint32_t R;
  struct {
    vuint32_t PMSS_ACS:16;
    vuint32_t _unused_16:16;
  } B;
} PROGRAM_MEMORY_SUBSYSTEM_PMSSACS_tag;

struct PROGRAM_MEMORY_SUBSYSTEM_tag {
  PROGRAM_MEMORY_SUBSYSTEM_MSS_PCR_tag MSS_PCR; /* Program Control */
  PROGRAM_MEMORY_SUBSYSTEM_P_SYS_OU_C_tag P_SYS_OU_C; /* PMSS AXI Outstanding Configuration */
  PROGRAM_MEMORY_SUBSYSTEM_P_HDCFG_tag P_HDCFG; /* PMSS Hardware Configuration */
  uint8_t PROGRAM_MEMORY_SUBSYSTEM_reserved0[16];
  PROGRAM_MEMORY_SUBSYSTEM_P_PORT_ID_tag P_PORT_ID; /* Program Port AXI ID */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD0_START_tag P_ADD0_START; /* PMSS ADD0 START */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD0_ATT0_tag P_ADD0_ATT0; /* PMSS ADD0 ATT0 */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD1_START_tag P_ADD1_START; /* PMSS ADD1 START */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD1_ATT0_tag P_ADD1_ATT0; /* PMSS ADD1 ATT0 */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD2_START_tag P_ADD2_START; /* PMSS ADD2 START */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD2_ATT0_tag P_ADD2_ATT0; /* PMSS ADD2 ATT0 */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD3_START_tag P_ADD3_START; /* PMSS ADD3 START */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD3_ATT0_tag P_ADD3_ATT0; /* PMSS ADD3 ATT0 */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD4_START_tag P_ADD4_START; /* PMSS ADD4 START */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD4_ATT0_tag P_ADD4_ATT0; /* PMSS ADD4 ATT0 */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD5_START_tag P_ADD5_START; /* PMSS ADD5 START */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD5_ATT0_tag P_ADD5_ATT0; /* PMSS ADD5 ATT0 */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD6_START_tag P_ADD6_START; /* PMSS ADD6 START */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD6_ATT0_tag P_ADD6_ATT0; /* PMSS ADD6 ATT0 */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD7_START_tag P_ADD7_START; /* PMSS ADD7 START */
  PROGRAM_MEMORY_SUBSYSTEM_P_ADD7_ATT0_tag P_ADD7_ATT0; /* PMSS ADD7 ATT0 */
  uint8_t PROGRAM_MEMORY_SUBSYSTEM_reserved1[128];
  PROGRAM_MEMORY_SUBSYSTEM_P_CCOSAR_tag P_CCOSAR; /* Program Cache Software Operations */
  PROGRAM_MEMORY_SUBSYSTEM_P_CCOCR_tag P_CCOCR; /* Program Cache Software Operations Control */
  uint8_t PROGRAM_MEMORY_SUBSYSTEM_reserved2[8];
  PROGRAM_MEMORY_SUBSYSTEM_P_ECADD_tag P_ECADD; /* Program ECC Error Address */
  uint8_t PROGRAM_MEMORY_SUBSYSTEM_reserved3[76];
  PROGRAM_MEMORY_SUBSYSTEM_PMSSACS_tag PMSSACS; /* PMSS Access Control */
};


/* ============================================================================
   =============================== Module: PSI5 ===============================
   ============================================================================ */

typedef union PSI5_GCR_union_tag {     /* Global Control Register */
  vuint16_t R;
  struct {
    vuint16_t GLOBAL_DISABLE_REQ:1;    /* GLOBAL_DISABLE_REQ */
    vuint16_t CTC_GED:1;               /* CTC_GED */
    vuint16_t _unused_2:14;            /* Reserved */
  } B;
} PSI5_GCR_tag;

typedef union PSI5_CHANNEL_CH_PCCR_union_tag { /* PSI5 Channel Control Register */
  vuint32_t R;
  struct {
    vuint32_t PSI5_CH_EN:1;            /* PSI5_CH_EN */
    vuint32_t PSI5_CH_CONFIG:1;        /* PSI5_CH_CONFIG */
    vuint32_t MODE:1;                  /* MODE */
    vuint32_t BIT_RATE:1;              /* Bit Rate */
    vuint32_t FAST_CLR_PSI5:1;         /* FAST_CLR_PSI5 */
    vuint32_t FAST_CLR_SMC:1;          /* FAST_CLR_SMC */
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t SP_TS_CLK_SEL:1;         /* SP_TS_CLK_SEL */
    vuint32_t DEBUG_FREEZE_CTRL:1;     /* DEBUG_FREEZE_CTRL */
    vuint32_t DEBUG_EN:1;              /* DEBUG_EN */
    vuint32_t _unused_11:3;            /* Reserved */
    vuint32_t GTM_RESET_ASYNC_EN:1;    /* GTM_RESET_ASYNC_EN */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t ERROR_SELECT0:1;         /* ERROR_SELECT0 */
    vuint32_t ERROR_SELECT1:1;         /* ERROR_SELECT1 */
    vuint32_t ERROR_SELECT2:1;         /* ERROR_SELECT2 */
    vuint32_t ERROR_SELECT3:1;         /* ERROR_SELECT3 */
    vuint32_t ERROR_SELECT4:1;         /* ERROR_SELECT4 */
    vuint32_t _unused_21:3;            /* Reserved */
    vuint32_t MEM_DEPTH:5;             /* MEM_DEPTH */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t CTC_ED:1;                /* CTC_ED */
    vuint32_t CTC_GED_SEL:1;           /* CTC_GED_SEL */
  } B;
} PSI5_CHANNEL_CH_PCCR_tag;

typedef union PSI5_CHANNEL_CH_DCR_union_tag { /* DMA Control Register */
  vuint32_t R;
  struct {
    vuint32_t DMA_PM_DS_CONFIG:2;      /* DMA_PM_DS_CONFIG */
    vuint32_t DMA_EN_SF:1;             /* DMA_EN_SF */
    vuint32_t _unused_3:5;             /* Reserved */
    vuint32_t IE_DMA_PM_DS_UF:1;       /* IE_DMA_PM_DS_UF */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t IE_DMA_SFUF:1;           /* IE_DMA_SFUF */
    vuint32_t IE_DMA_PM_DS_FIFO_FULL:1; /* IE_DMA_PM_DS_FIFO_FULL */
    vuint32_t _unused_12:5;            /* Reserved */
    vuint32_t IE_DMA_TF_PM_DS:1;       /* IE_DMA_TF_PM_DS */
    vuint32_t IE_DMA_TF_SF:1;          /* IE_DMA_TF_SF */
    vuint32_t _unused_19:5;            /* Reserved */
    vuint32_t DMA_PM_DS_WM:5;          /* DMA_PM_DS_WM */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_DCR_tag;

typedef union PSI5_CHANNEL_CH_DSR_union_tag { /* DMA Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;             /* Reserved */
    vuint32_t IS_DMA_PM_DS_UF:1;       /* IS_DMA_PM_DS_UF */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t IS_DMA_SFUF:1;           /* IS_DMA_SFUF */
    vuint32_t IS_DMA_PM_DS_FIFO_FULL:1; /* IS_DMA_PM_DS_FIFO_FULL */
    vuint32_t _unused_12:5;            /* Reserved */
    vuint32_t IS_DMA_TF_PM_DS:1;       /* IS_DMA_TF_PM_DS */
    vuint32_t IS_DMA_TF_SF:1;          /* IS_DMA_TF_SF */
    vuint32_t _unused_19:12;           /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_DSR_tag;

typedef union PSI5_CHANNEL_CH_GICR_union_tag { /* General Interrupt Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t _unused_8:6;             /* Reserved */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t IE_PRR:1;                /* IE_PRR */
    vuint32_t IE_BRR:1;                /* IE_BRR */
    vuint32_t IE_DSRR:1;               /* IE_DSRR */
    vuint32_t IE_PROW:1;               /* IE_PROW */
    vuint32_t IE_BROW:1;               /* IE_BROW */
    vuint32_t IE_DSROW:1;              /* IE_DSROW */
    vuint32_t IE_DTS:1;                /* IE_DTS */
    vuint32_t IE_STS:1;                /* IE_STS */
    vuint32_t _unused_24:6;            /* Reserved */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_GICR_tag;

typedef union PSI5_CHANNEL_CH_NDICR_union_tag { /* New Data Interrupt Control Register */
  vuint32_t R;
  struct {
    vuint32_t IE_ND:32;                /* IE_ND */
  } B;
} PSI5_CHANNEL_CH_NDICR_tag;

typedef union PSI5_CHANNEL_CH_OWICR_union_tag { /* Overwrite Interrupt Control Register */
  vuint32_t R;
  struct {
    vuint32_t IE_OW:32;                /* IE_OW */
  } B;
} PSI5_CHANNEL_CH_OWICR_tag;

typedef union PSI5_CHANNEL_CH_EICR_union_tag { /* Error Interrupt Control Register */
  vuint32_t R;
  struct {
    vuint32_t IE_ERROR:32;             /* IE_ERROR */
  } B;
} PSI5_CHANNEL_CH_EICR_tag;

typedef union PSI5_CHANNEL_CH_GISR_union_tag { /* General Interrupt Status Register */
  vuint32_t R;
  struct {
    vuint32_t IS_NVSM:6;               /* IS_NVSM */
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t IS_OWSM:6;               /* IS_OWSM */
    vuint32_t _unused_14:2;            /* Reserved */
    vuint32_t DPR_RDY:1;               /* DPR_RDY */
    vuint32_t DBR_RDY:1;               /* DBR_RDY */
    vuint32_t DSR_RDY:1;               /* DSR_RDY */
    vuint32_t IS_PROW:1;               /* IS_PROW */
    vuint32_t IS_BROW:1;               /* IS_BROW */
    vuint32_t IS_DSROW:1;              /* IS_DSROW */
    vuint32_t IS_DTS:1;                /* IS_DTS */
    vuint32_t IS_STS:1;                /* IS_STS */
    vuint32_t IS_CESM:6;               /* IS_CESM */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t IS_DB_FR:1;              /* IS_DB_FR */
  } B;
} PSI5_CHANNEL_CH_GISR_tag;

typedef union PSI5_CHANNEL_CH_DPMR_union_tag { /* DMA PSI5 Message Register */
  vuint32_t R;
  struct {
    vuint32_t PSI5_RXDATA:32;          /* PSI5_RXDATA */
  } B;
} PSI5_CHANNEL_CH_DPMR_tag;

typedef union PSI5_CHANNEL_CH_DSFR_union_tag { /* DMA SMC Frame Register */
  vuint32_t R;
  struct {
    vuint32_t SMC_RXDATA:32;           /* SMC_RXDATA */
  } B;
} PSI5_CHANNEL_CH_DSFR_tag;

typedef union PSI5_CHANNEL_CH_DDSR_union_tag { /* DMA Diagnostic Status Register */
  vuint32_t R;
  struct {
    vuint32_t DDS:32;                  /* DDS */
  } B;
} PSI5_CHANNEL_CH_DDSR_tag;

typedef union PSI5_CHANNEL_CH_PMRRL_union_tag { /* PSI5 Message Receive Register Low */
  vuint32_t R;
  struct {
    vuint32_t C:1;                     /* C */
    vuint32_t CRC:3;                   /* CRC */
    vuint32_t DATA_REGION:28;          /* DATA_REGION */
  } B;
} PSI5_CHANNEL_CH_PMRRL_tag;

typedef union PSI5_CHANNEL_CH_PMRRH_union_tag { /* PSI5 Message Receive Register High */
  vuint32_t R;
  struct {
    vuint32_t TIMESTAMPVALUE:24;       /* TimeStampValue */
    vuint32_t SLOTCOUNTER:3;           /* SlotCounter */
    vuint32_t T:1;                     /* T */
    vuint32_t E:1;                     /* E */
    vuint32_t EM:1;                    /* EM */
    vuint32_t F:1;                     /* F */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_PMRRH_tag;

typedef union PSI5_CHANNEL_CH_PMR_CH_PMRL_union_tag { /* PSI5 Message Register Low i */
  vuint32_t R;
  struct {
    vuint32_t C:1;                     /* C */
    vuint32_t CRCP:3;                  /* CRCP */
    vuint32_t DATA_REGION:28;          /* DATA_REGION */
  } B;
} PSI5_CHANNEL_CH_PMR_CH_PMRL_tag;

typedef union PSI5_CHANNEL_CH_PMR_CH_PMRH_union_tag { /* PSI5 Message Register High i */
  vuint32_t R;
  struct {
    vuint32_t TIMESTAMPVALUE:24;       /* TimeStampValue */
    vuint32_t SLOT_COUNTER:3;          /* Slot_Counter */
    vuint32_t T:1;                     /* T */
    vuint32_t E:1;                     /* E */
    vuint32_t EM:1;                    /* EM */
    vuint32_t F:1;                     /* F */
    vuint32_t O:1;                     /* O */
  } B;
} PSI5_CHANNEL_CH_PMR_CH_PMRH_tag;

typedef struct PSI5_CHANNEL_CH_PMR_struct_tag {
  PSI5_CHANNEL_CH_PMR_CH_PMRL_tag CH_PMRL; /* PSI5 Message Register Low i */
  PSI5_CHANNEL_CH_PMR_CH_PMRH_tag CH_PMRH; /* PSI5 Message Register High i */
} PSI5_CHANNEL_CH_PMR_tag;

typedef union PSI5_CHANNEL_CH_SFR_union_tag { /* SMC Frame Register n */
  vuint32_t R;
  struct {
    vuint32_t DATA:12;                 /* DATA */
    vuint32_t IDDATA:4;                /* IDDATA */
    vuint32_t ID:4;                    /* ID */
    vuint32_t C:1;                     /* C */
    vuint32_t CRC:6;                   /* CRC */
    vuint32_t OW:1;                    /* OW */
    vuint32_t CER:1;                   /* CER */
    vuint32_t SLOT_NO:3;               /* SLOT_NO */
  } B;
} PSI5_CHANNEL_CH_SFR_tag;

typedef union PSI5_CHANNEL_CH_NDSR_union_tag { /* New Data Status Register */
  vuint32_t R;
  struct {
    vuint32_t NDS:32;
  } B;
} PSI5_CHANNEL_CH_NDSR_tag;

typedef union PSI5_CHANNEL_CH_OWSR_union_tag { /* Overwrite Status Register */
  vuint32_t R;
  struct {
    vuint32_t OWS:32;                  /* OWS */
  } B;
} PSI5_CHANNEL_CH_OWSR_tag;

typedef union PSI5_CHANNEL_CH_EISR_union_tag { /* Error Indication Status Register */
  vuint32_t R;
  struct {
    vuint32_t ERROR:32;                /* ERROR */
  } B;
} PSI5_CHANNEL_CH_EISR_tag;

typedef union PSI5_CHANNEL_CH_SNDSR_union_tag { /* Set New Data Status Register */
  vuint32_t R;
  struct {
    vuint32_t SNDS:32;                 /* SNDS */
  } B;
} PSI5_CHANNEL_CH_SNDSR_tag;

typedef union PSI5_CHANNEL_CH_SOWSR_union_tag { /* Set Overwrite Status Register */
  vuint32_t R;
  struct {
    vuint32_t SOWS:32;                 /* SOWS */
  } B;
} PSI5_CHANNEL_CH_SOWSR_tag;

typedef union PSI5_CHANNEL_CH_SEISR_union_tag { /* Set Error Status Register */
  vuint32_t R;
  struct {
    vuint32_t SERROR:32;               /* SERROR */
  } B;
} PSI5_CHANNEL_CH_SEISR_tag;

typedef union PSI5_CHANNEL_CH_SSESR_union_tag { /* Set SMC Error Status Register */
  vuint32_t R;
  struct {
    vuint32_t SNVSM:6;                 /* SNVSM */
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t SOWSM:6;                 /* SOWSM */
    vuint32_t _unused_14:10;           /* Reserved */
    vuint32_t SCESM:6;                 /* SCESM */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_SSESR_tag;

typedef union PSI5_CHANNEL_CH_STSRR_union_tag { /* Sync Time Stamp Read Register */
  vuint32_t R;
  struct {
    vuint32_t STSV:24;                 /* STSV */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_STSRR_tag;

typedef union PSI5_CHANNEL_CH_DTSRR_union_tag { /* Data Time Stamp Read Register */
  vuint32_t R;
  struct {
    vuint32_t DTSV:24;                 /* DTSV */
    vuint32_t SLOT_COUNTER:3;          /* SLOT_COUNTER */
    vuint32_t _unused_27:5;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_DTSRR_tag;

typedef union PSI5_CHANNEL_CH_SFCR_union_tag { /* Slot n Frame Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CRCP:1;                  /* CRCP */
    vuint32_t DRL:5;                   /* DRL */
    vuint32_t _unused_6:9;             /* Reserved */
    vuint32_t SMCL:1;                  /* SMCL */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t TS_CAPT:1;               /* TS_CAPT */
    vuint32_t SLOT_EN:1;               /* SLOT_EN */
    vuint32_t _unused_19:13;           /* Reserved */
  } B;
} PSI5_CHANNEL_CH_SFCR_tag;

typedef union PSI5_CHANNEL_CH_S2SBR_union_tag { /* Slot 2 Start Boundary Register */
  vuint16_t R;
  struct {
    vuint16_t S2SBT:15;                /* S2SBT */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_S2SBR_tag;

typedef union PSI5_CHANNEL_CH_S1SBR_union_tag { /* Slot 1 Start Boundary Register */
  vuint16_t R;
  struct {
    vuint16_t S1SBT:15;                /* S1SBT */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_S1SBR_tag;

typedef union PSI5_CHANNEL_CH_S4SBR_union_tag { /* Slot 4 Start Boundary Register */
  vuint16_t R;
  struct {
    vuint16_t S4SBT:15;                /* S4SBT */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_S4SBR_tag;

typedef union PSI5_CHANNEL_CH_S3SBR_union_tag { /* Slot 3 Start Boundary Register */
  vuint16_t R;
  struct {
    vuint16_t S3SBT:15;                /* S3SBT */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_S3SBR_tag;

typedef union PSI5_CHANNEL_CH_S6SBR_union_tag { /* Slot 6 Start Boundary Register */
  vuint16_t R;
  struct {
    vuint16_t S6SBT:15;                /* S6SBT */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_S6SBR_tag;

typedef union PSI5_CHANNEL_CH_S5SBR_union_tag { /* Slot 5 Start Boundary Register */
  vuint16_t R;
  struct {
    vuint16_t S5SBT:15;                /* S5SBT */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_S5SBR_tag;

typedef union PSI5_CHANNEL_CH_SNEBR_union_tag { /* Slot n End Boundary Register */
  vuint32_t R;
  struct {
    vuint32_t SNEBT:15;                /* SnEBT */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t SLOT_NO:3;               /* SLOT_NO */
    vuint32_t _unused_19:13;           /* Reserved */
  } B;
} PSI5_CHANNEL_CH_SNEBR_tag;

typedef union PSI5_CHANNEL_CH_MDDIS_OFF_union_tag { /* Manchestor Decoder Disable Offset */
  vuint16_t R;
  struct {
    vuint16_t MDDIS_OFF:7;             /* MDDIS_OFF */
    vuint16_t _unused_7:9;             /* Reserved */
  } B;
} PSI5_CHANNEL_CH_MDDIS_OFF_tag;

typedef union PSI5_CHANNEL_CH_DOBCR_union_tag { /* Data Output Block Configuration Register */
  vuint16_t R;
  struct {
    vuint16_t SW_READY:1;              /* SW_READY */
    vuint16_t OP_SEL:1;                /* OP_SEL */
    vuint16_t SP_PULSE_SEL:1;          /* SP_PULSE_SEL */
    vuint16_t GTM_TRIG_SEL:1;          /* GTM_TRIG_SEL */
    vuint16_t DEFAULT_SYNC:1;          /* DEFAULT_SYNC */
    vuint16_t CMD_TYPE:3;              /* CMD_TYPE */
    vuint16_t DSR_RST:1;               /* DSR_RST */
    vuint16_t DBR_RST:1;               /* DBR_RST */
    vuint16_t DATA_LENGTH:6;           /* DATA_LENGTH */
  } B;
} PSI5_CHANNEL_CH_DOBCR_tag;

typedef union PSI5_CHANNEL_CH_PW1D_union_tag { /* Pulse Width for Data Bit Value 1 */
  vuint16_t R;
  struct {
    vuint16_t PULSE_WIDTH1:7;          /* Pulse_Width1 */
    vuint16_t _unused_7:9;             /* Reserved */
  } B;
} PSI5_CHANNEL_CH_PW1D_tag;

typedef union PSI5_CHANNEL_CH_PW0D_union_tag { /* Pulse Width for Data Bit Value 0 */
  vuint16_t R;
  struct {
    vuint16_t PULSE_WIDTH0:7;          /* Pulse_Width0 */
    vuint16_t _unused_7:9;             /* Reserved */
  } B;
} PSI5_CHANNEL_CH_PW0D_tag;

typedef union PSI5_CHANNEL_CH_CIPR_union_tag { /* Counter Initialize Pulse Register */
  vuint16_t R;
  struct {
    vuint16_t CIPR:16;                 /* CIPR */
  } B;
} PSI5_CHANNEL_CH_CIPR_tag;

typedef union PSI5_CHANNEL_CH_CTPR_union_tag { /* Counter Target Pulse Register */
  vuint16_t R;
  struct {
    vuint16_t CTPR:16;                 /* CTPR */
  } B;
} PSI5_CHANNEL_CH_CTPR_tag;

typedef union PSI5_CHANNEL_CH_DPRL_union_tag { /* Data Preparation Register Low */
  vuint32_t R;
  struct {
    vuint32_t DPR:24;                  /* DPR */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_DPRL_tag;

typedef union PSI5_CHANNEL_CH_DPRH_union_tag { /* Data Preparation Register High */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;            /* Reserved */
  } B;
} PSI5_CHANNEL_CH_DPRH_tag;

typedef union PSI5_CHANNEL_CH_DBRL_union_tag { /* Data Buffer Register Low */
  vuint32_t R;
  struct {
    vuint32_t DBR:32;                  /* DBR */
  } B;
} PSI5_CHANNEL_CH_DBRL_tag;

typedef union PSI5_CHANNEL_CH_DBRH_union_tag { /* Data Buffer Register High */
  vuint32_t R;
  struct {
    vuint32_t DBR:32;                  /* DBR */
  } B;
} PSI5_CHANNEL_CH_DBRH_tag;

typedef union PSI5_CHANNEL_CH_DSRL_union_tag { /* Data Shift Register Low */
  vuint32_t R;
  struct {
    vuint32_t DSR:32;                  /* DSR */
  } B;
} PSI5_CHANNEL_CH_DSRL_tag;

typedef union PSI5_CHANNEL_CH_DSRH_union_tag { /* Data Shift Register High */
  vuint32_t R;
  struct {
    vuint32_t DSR:32;                  /* DSR */
  } B;
} PSI5_CHANNEL_CH_DSRH_tag;

typedef struct PSI5_CHANNEL_struct_tag {
  PSI5_CHANNEL_CH_PCCR_tag CH_PCCR;    /* PSI5 Channel Control Register */
  PSI5_CHANNEL_CH_DCR_tag CH_DCR;      /* DMA Control Register */
  PSI5_CHANNEL_CH_DSR_tag CH_DSR;      /* DMA Status Register */
  PSI5_CHANNEL_CH_GICR_tag CH_GICR;    /* General Interrupt Control Register */
  PSI5_CHANNEL_CH_NDICR_tag CH_NDICR;  /* New Data Interrupt Control Register */
  PSI5_CHANNEL_CH_OWICR_tag CH_OWICR;  /* Overwrite Interrupt Control Register */
  PSI5_CHANNEL_CH_EICR_tag CH_EICR;    /* Error Interrupt Control Register */
  PSI5_CHANNEL_CH_GISR_tag CH_GISR;    /* General Interrupt Status Register */
  PSI5_CHANNEL_CH_DPMR_tag CH_DPMR;    /* DMA PSI5 Message Register */
  PSI5_CHANNEL_CH_DSFR_tag CH_DSFR;    /* DMA SMC Frame Register */
  PSI5_CHANNEL_CH_DDSR_tag CH_DDSR;    /* DMA Diagnostic Status Register */
  PSI5_CHANNEL_CH_PMRRL_tag CH_PMRRL;  /* PSI5 Message Receive Register Low */
  PSI5_CHANNEL_CH_PMRRH_tag CH_PMRRH;  /* PSI5 Message Receive Register High */
  PSI5_CHANNEL_CH_PMR_tag CH_PMR[32];
  PSI5_CHANNEL_CH_SFR_tag CH_SFR[6];   /* SMC Frame Register n */
  PSI5_CHANNEL_CH_NDSR_tag CH_NDSR;    /* New Data Status Register */
  PSI5_CHANNEL_CH_OWSR_tag CH_OWSR;    /* Overwrite Status Register */
  PSI5_CHANNEL_CH_EISR_tag CH_EISR;    /* Error Indication Status Register */
  PSI5_CHANNEL_CH_SNDSR_tag CH_SNDSR;  /* Set New Data Status Register */
  PSI5_CHANNEL_CH_SOWSR_tag CH_SOWSR;  /* Set Overwrite Status Register */
  PSI5_CHANNEL_CH_SEISR_tag CH_SEISR;  /* Set Error Status Register */
  PSI5_CHANNEL_CH_SSESR_tag CH_SSESR;  /* Set SMC Error Status Register */
  PSI5_CHANNEL_CH_STSRR_tag CH_STSRR;  /* Sync Time Stamp Read Register */
  PSI5_CHANNEL_CH_DTSRR_tag CH_DTSRR;  /* Data Time Stamp Read Register */
  PSI5_CHANNEL_CH_SFCR_tag CH_SFCR[6]; /* Slot n Frame Configuration Register */
  PSI5_CHANNEL_CH_S2SBR_tag CH_S2SBR;  /* Slot 2 Start Boundary Register */
  PSI5_CHANNEL_CH_S1SBR_tag CH_S1SBR;  /* Slot 1 Start Boundary Register */
  PSI5_CHANNEL_CH_S4SBR_tag CH_S4SBR;  /* Slot 4 Start Boundary Register */
  PSI5_CHANNEL_CH_S3SBR_tag CH_S3SBR;  /* Slot 3 Start Boundary Register */
  PSI5_CHANNEL_CH_S6SBR_tag CH_S6SBR;  /* Slot 6 Start Boundary Register */
  PSI5_CHANNEL_CH_S5SBR_tag CH_S5SBR;  /* Slot 5 Start Boundary Register */
  PSI5_CHANNEL_CH_SNEBR_tag CH_SNEBR;  /* Slot n End Boundary Register */
  PSI5_CHANNEL_CH_MDDIS_OFF_tag CH_MDDIS_OFF; /* Manchestor Decoder Disable Offset */
  PSI5_CHANNEL_CH_DOBCR_tag CH_DOBCR;  /* Data Output Block Configuration Register */
  PSI5_CHANNEL_CH_PW1D_tag CH_PW1D;    /* Pulse Width for Data Bit Value 1 */
  PSI5_CHANNEL_CH_PW0D_tag CH_PW0D;    /* Pulse Width for Data Bit Value 0 */
  PSI5_CHANNEL_CH_CIPR_tag CH_CIPR;    /* Counter Initialize Pulse Register */
  PSI5_CHANNEL_CH_CTPR_tag CH_CTPR;    /* Counter Target Pulse Register */
  PSI5_CHANNEL_CH_DPRL_tag CH_DPRL;    /* Data Preparation Register Low */
  PSI5_CHANNEL_CH_DPRH_tag CH_DPRH;    /* Data Preparation Register High */
  PSI5_CHANNEL_CH_DBRL_tag CH_DBRL;    /* Data Buffer Register Low */
  PSI5_CHANNEL_CH_DBRH_tag CH_DBRH;    /* Data Buffer Register High */
  PSI5_CHANNEL_CH_DSRL_tag CH_DSRL;    /* Data Shift Register Low */
  PSI5_CHANNEL_CH_DSRH_tag CH_DSRH;    /* Data Shift Register High */
  uint8_t CHANNEL_reserved0[4];
} PSI5_CHANNEL_tag;

struct PSI5_tag {
  uint8_t PSI5_reserved0[2];
  PSI5_GCR_tag GCR;                    /* Global Control Register */
  uint8_t PSI5_reserved1[4];
  PSI5_CHANNEL_tag CHANNEL[4];
};


/* ============================================================================
   =============================== Module: PSI5_S =============================
   ============================================================================ */

typedef union PSI5_S_LINCR1_union_tag { /* PSI5_S LIN Control Register 1 */
  vuint32_t R;
  struct {
    vuint32_t INIT:1;
    vuint32_t SLEEP:1;
    vuint32_t RBLM:1;
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t LBKM:1;
    vuint32_t SLFM:1;
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:4;             /* Reserved */
    vuint32_t AUTOWU:1;
    vuint32_t _unused_13:19;           /* Reserved */
  } B;
} PSI5_S_LINCR1_tag;

typedef union PSI5_S_LINIER_union_tag { /* PSI5_S LIN Interrupt enable register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t DTIE:1;                  /* Data Transmitted Interrupt enable */
    vuint32_t DRIE:1;                  /* Data Reception complete Interrupt enable */
    vuint32_t TOIE:1;                  /* Timeout Interrupt Enable */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t WUIE:1;                  /* PS_NOUSE_BIT */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t BOIE:1;                  /* Buffer Overrun Error Interrupt Enable */
    vuint32_t FEIE:1;                  /* Frame Error Interrupt Enable */
    vuint32_t _unused_9:5;             /* Reserved */
    vuint32_t OCIE:1;                  /* PS_NOUSE_BIT */
    vuint32_t SZIE:1;                  /* Stuck at Zero Interrupt Enable */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} PSI5_S_LINIER_tag;

typedef union PSI5_S_LINSR_union_tag { /* PSI5_S LIN Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:5;             /* Reserved */
    vuint32_t LINS0:1;
    vuint32_t LINS1:1;
    vuint32_t LINS2:1;
    vuint32_t LINS3:1;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} PSI5_S_LINSR_tag;

typedef union PSI5_S_UARTCR_union_tag { /* PSI5_S UART Mode Control Register */
  vuint32_t R;
  struct {
    vuint32_t UART:1;
    vuint32_t WL0:1;
    vuint32_t PCE_RX:1;
    vuint32_t PC0:1;
    vuint32_t TXEN:1;
    vuint32_t RXEN:1;
    vuint32_t PC1:1;
    vuint32_t WL1:1;
    vuint32_t TFBM:1;
    vuint32_t RFBM:1;
    vuint32_t RDFLRFC:3;
    vuint32_t TDFLTFC:3;
    vuint32_t WLS:1;
    vuint32_t SBUR:2;
    vuint32_t PCE_TXDTU:1;
    vuint32_t NEF:3;
    vuint32_t ROSE:1;
    vuint32_t OSR:4;
    vuint32_t CSP:3;
    vuint32_t MIS:1;
  } B;
} PSI5_S_UARTCR_tag;

typedef union PSI5_S_UARTSR_union_tag { /* PSI5_S UART Mode Status Register */
  vuint32_t R;
  struct {
    vuint32_t NF:1;                    /* NF */
    vuint32_t DTF:1;
    vuint32_t DRF:1;                   /* Data Reception Completed Flag */
    vuint32_t TO:1;
    vuint32_t RFNE:1;
    vuint32_t WUF:1;
    vuint32_t RDI:1;
    vuint32_t BOF:1;
    vuint32_t FEF:1;
    vuint32_t RMB:1;
    vuint32_t PE0:1;
    vuint32_t PE1:1;
    vuint32_t PE2:1;
    vuint32_t PE3:1;
    vuint32_t OCF:1;
    vuint32_t SZF:1;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} PSI5_S_UARTSR_tag;

typedef union PSI5_S_LINFBRR_union_tag { /* PSI5_S LIN Fractional Baud Rate Register */
  vuint32_t R;
  struct {
    vuint32_t FBR:4;
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} PSI5_S_LINFBRR_tag;

typedef union PSI5_S_LINIBRR_union_tag { /* PSI5_S LIN Integer Baud Rate Register */
  vuint32_t R;
  struct {
    vuint32_t IBR:20;
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} PSI5_S_LINIBRR_tag;

typedef union PSI5_S_LINCR2_union_tag { /* PSI5_S LIN Control Register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:9;             /* Reserved */
    vuint32_t ABRQ:1;
    vuint32_t _unused_10:4;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:17;           /* Reserved */
  } B;
} PSI5_S_LINCR2_tag;

typedef union PSI5_S_BDRL_union_tag {  /* PSI5_S Buffer Data Register Least Significant */
  vuint32_t R;
  struct {
    vuint32_t DATA_TX0:8;
    vuint32_t DATA_TX1:8;
    vuint32_t DATA_TX2:8;
    vuint32_t DATA_TX3:8;
  } B;
} PSI5_S_BDRL_tag;

typedef union PSI5_S_BDRM_union_tag {  /* PSI5_S Buffer Data Register Most Significant */
  vuint32_t R;
  struct {
    vuint32_t DATA_RX0:8;
    vuint32_t DATA_RX1:8;
    vuint32_t DATA_RX2:8;
    vuint32_t DATA_RX3:8;
  } B;
} PSI5_S_BDRM_tag;

typedef union PSI5_S_GCR_union_tag {   /* PSI5_S Global Control register */
  vuint32_t R;
  struct {
    vuint32_t SR:1;                    /* Soft reset */
    vuint32_t STOP:1;
    vuint32_t RDLIS:1;                 /* Received data level inversion selection */
    vuint32_t TDLIS:1;                 /* Transmit data level inversion selection */
    vuint32_t RDFBM:1;                 /* Received data first bit MSB */
    vuint32_t TDFBM:1;                 /* Transmit data first bit MSB */
    vuint32_t _unused_6:26;            /* Reserved */
  } B;
} PSI5_S_GCR_tag;

typedef union PSI5_S_UARTPTO_union_tag { /* PSI5_S UART Preset Timeout Register */
  vuint32_t R;
  struct {
    vuint32_t PTO:12;                  /* Preset Timeout */
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} PSI5_S_UARTPTO_tag;

typedef union PSI5_S_UARTCTO_union_tag { /* PSI5_S UART Current Timeout register */
  vuint32_t R;
  struct {
    vuint32_t CTO:12;
    vuint32_t _unused_12:20;           /* Reserved */
  } B;
} PSI5_S_UARTCTO_tag;

typedef union PSI5_S_DMATXE_union_tag { /* DMA Tx Enable Register */
  vuint32_t R;
  struct {
    vuint32_t DTE0:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} PSI5_S_DMATXE_tag;

typedef union PSI5_S_DMARXE_union_tag { /* DMA Rx Enable Register */
  vuint32_t R;
  struct {
    vuint32_t DRE0:1;
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} PSI5_S_DMARXE_tag;

typedef union PSI5_S_PTD_union_tag {   /* PSI5_S UART Tx Idle Delay Time Register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t IFD:4;
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} PSI5_S_PTD_tag;

typedef union PSI5_S_GLCR_union_tag {  /* PSI5_S Global Control Register */
  vuint32_t R;
  struct {
    vuint32_t GLOBAL_MODE:3;
    vuint32_t TSCNTEN_G_L:1;
    vuint32_t TSCNTEN_G:1;
    vuint32_t TSCNT_EN_A:1;
    vuint32_t TSCNT_EN_B:1;
    vuint32_t CLRTSCNT_G_L:1;
    vuint32_t CLRTSCNT_G:1;
    vuint32_t CLR_CNTR_A:1;
    vuint32_t CLR_CNTR_B:1;
    vuint32_t TSCS_A:1;
    vuint32_t TSCS_B:1;
    vuint32_t MRU_ERR_EN:1;
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t GL_DDSR_TRIG:1;
    vuint32_t GTM_TRIG_SEL:2;
    vuint32_t DEBUG_EN:1;
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t IE_DIRCMD_RDY:1;
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t DIRCMD_LEN:2;
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} PSI5_S_GLCR_tag;

typedef union PSI5_S_GLSR_union_tag {  /* PSI5_S Global Status Register */
  vuint32_t R;
  struct {
    vuint32_t CHID:3;
    vuint32_t FID:3;
    vuint32_t _unused_6:7;             /* Reserved */
    vuint32_t MRU_ERR:1;
    vuint32_t GL_MODETR_DONE:1;
    vuint32_t _unused_15:5;            /* Reserved */
    vuint32_t DIRCMD_RDY:1;
    vuint32_t _unused_21:11;           /* Reserved */
  } B;
} PSI5_S_GLSR_tag;

typedef union PSI5_S_CH_BASE_ADDR_union_tag { /* PSI5_S CHANNEL_BASE_ADDRESS */
  vuint32_t R;
  struct {
    vuint32_t MAILBOX_BASE_ADDR:32;
  } B;
} PSI5_S_CH_BASE_ADDR_tag;

typedef union PSI5_S_MRU_BUF2_REG0_union_tag { /* PSI5_S MRU OUTPUT BUFFER2 REGISTER0 */
  vuint32_t R;
  struct {
    vuint32_t CHANNEL_SPECIFIC_MAILBOX_ADDR:32;
  } B;
} PSI5_S_MRU_BUF2_REG0_tag;

typedef union PSI5_S_MRU_BUF2_REG1_union_tag { /* PSI5_S MRU OUTPUT BUFFER2 REGISTER1 */
  vuint32_t R;
  struct {
    vuint32_t XCRC:6;
    vuint32_t XCRC_ERR:1;
    vuint32_t CRC:3;
    vuint32_t CRC_ERR_P_ERR:1;
    vuint32_t ERR:2;
    vuint32_t HD_ERR:1;
    vuint32_t SCI_P_ERR:1;
    vuint32_t SCI_F_ERR:1;
    vuint32_t SCI_O_ERR:1;
    vuint32_t F_WD_ERR:1;
    vuint32_t R_OVL_ERR:1;
    vuint32_t FID:3;
    vuint32_t CHID:3;
    vuint32_t N_ERR:1;
    vuint32_t R_UVL_ERR:1;
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t DCI:4;
  } B;
} PSI5_S_MRU_BUF2_REG1_tag;

typedef union PSI5_S_MRU_BUF2_REG2_union_tag { /* PSI5_S MRU OUTPUT BUFFER2 REGISTER2 */
  vuint32_t R;
  struct {
    vuint32_t PS_DATA:28;
    vuint32_t DCI:4;
  } B;
} PSI5_S_MRU_BUF2_REG2_tag;

typedef union PSI5_S_MRU_BUF2_REG3_union_tag { /* PSI5_S MRU OUTPUT BUFFER2 REGISTER3 */
  vuint32_t R;
  struct {
    vuint32_t TIMESTAMP:24;
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t DCI:4;
  } B;
} PSI5_S_MRU_BUF2_REG3_tag;

typedef union PSI5_S_MBOX_SR_IRQ_union_tag { /* PSI5_S Mbox Status Irq */
  vuint32_t R;
  struct {
    vuint32_t MBOX_CH0:1;
    vuint32_t MBOX_CH1:1;
    vuint32_t MBOX_CH2:1;
    vuint32_t MBOX_CH3:1;
    vuint32_t MBOX_CH4:1;
    vuint32_t MBOX_CH5:1;
    vuint32_t MBOX_CH6:1;
    vuint32_t MBOX_CH7:1;
    vuint32_t _unused_8:24;
  } B;
} PSI5_S_MBOX_SR_IRQ_tag;

typedef union PSI5_S_ERR_SR_IRQ_union_tag { /* PSI5_S Error Status IRQ */
  vuint32_t R;
  struct {
    vuint32_t XCRC_ERR:1;
    vuint32_t CRC_ERR_P_ERR:1;
    vuint32_t HD_ERR:1;
    vuint32_t SCI_P_ERR:1;
    vuint32_t SCI_F_ERR:1;
    vuint32_t SCI_O_ERR:1;
    vuint32_t F_WD_ERR:1;
    vuint32_t R_OVL_ERR:1;
    vuint32_t N_ERR:1;
    vuint32_t R_UVL_ERR:1;
    vuint32_t _unused_10:22;           /* Reserved */
  } B;
} PSI5_S_ERR_SR_IRQ_tag;

typedef union PSI5_S_IRQ_MBOX_SEL_IRQ_union_tag { /* PSI5_S Mailbox select IRQ[irq_n] */
  vuint32_t R;
  struct {
    vuint32_t MBOX_CH0_SEL:1;
    vuint32_t MBOX_CH1_SEL:1;
    vuint32_t MBOX_CH2_SEL:1;
    vuint32_t MBOX_CH3_SEL:1;
    vuint32_t MBOX_CH4_SEL:1;
    vuint32_t MBOX_CH5_SEL:1;
    vuint32_t MBOX_CH6_SEL:1;
    vuint32_t MBOX_CH7_SEL:1;
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} PSI5_S_IRQ_MBOX_SEL_IRQ_tag;

typedef union PSI5_S_IRQ_ERR_SEL_IRQ_union_tag { /* PSI5_S Error Select IRQ[irq_n] */
  vuint32_t R;
  struct {
    vuint32_t XCRC_ERR_SEL:1;
    vuint32_t CRC_ERR_SEL:1;
    vuint32_t HD_ERR_SEL:1;
    vuint32_t SCI_P_ERR_SEL:1;
    vuint32_t SCI_F_ERR_SEL:1;
    vuint32_t SCI_O_ERR_SEL:1;
    vuint32_t F_WD_ERR_SEL:1;
    vuint32_t R_OVL_ERR_SEL:1;
    vuint32_t N_ERR_SEL:1;
    vuint32_t R_UVL_ERR_SEL:1;
    vuint32_t _unused_10:22;           /* Reserved */
  } B;
} PSI5_S_IRQ_ERR_SEL_IRQ_tag;

typedef struct PSI5_S_IRQ_struct_tag {
  PSI5_S_IRQ_MBOX_SEL_IRQ_tag MBOX_SEL_IRQ; /* PSI5_S Mailbox select IRQ[irq_n] */
  PSI5_S_IRQ_ERR_SEL_IRQ_tag ERR_SEL_IRQ; /* PSI5_S Error Select IRQ[irq_n] */
} PSI5_S_IRQ_tag;

typedef union PSI5_S_WDGTSSR_union_tag { /* PSI5_S Watchdog Error Status and Watchdog Timestamp status register */
  vuint32_t R;
  struct {
    vuint32_t WDTS_STATUS:24;
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t F_WD_ERR_STATUS:7;
  } B;
} PSI5_S_WDGTSSR_tag;

typedef union PSI5_S_DIRCMD_union_tag { /* PSI5_S ECU to Sensor Direct Command Write register */
  vuint32_t R;
  struct {
    vuint32_t DIRCMD_BYTE0:8;
    vuint32_t DIRCMD_BYTE1:8;
    vuint32_t DIRCMD_BYTE2:8;
    vuint32_t DIRCMD_BYTE3:8;
  } B;
} PSI5_S_DIRCMD_tag;

typedef union PSI5_S_CH0_MSGA_union_tag { /* PSI5_S channel 0 message configuration register A */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t TSBUF_EN:1;
    vuint32_t TSBUF_CLR:1;
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t TIMESTAMP_A_B_SEL:1;
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t F0_BYTE:3;
    vuint32_t _unused_11:21;           /* Reserved */
  } B;
} PSI5_S_CH0_MSGA_tag;

typedef union PSI5_S_CH0_MSGB_union_tag { /* PSI5_S channel 0 message configuration register B */
  vuint32_t R;
  struct {
    vuint32_t F0_PAYLOAD:5;
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} PSI5_S_CH0_MSGB_tag;

typedef union PSI5_S_CH0_MBOX_SR_union_tag { /* PSI5_S Mailbox status register channel0 */
  vuint32_t R;
  struct {
    vuint32_t F0_READ:1;
    vuint32_t F0_OV:1;
    vuint32_t F0_ERR:1;
    vuint32_t F1_READ:1;
    vuint32_t F1_OV:1;
    vuint32_t F1_ERR:1;
    vuint32_t _unused_6:26;            /* Reserved */
  } B;
} PSI5_S_CH0_MBOX_SR_tag;

typedef union PSI5_S_CH_MSGA_union_tag { /* PSI5_S channel message configuration register A */
  vuint32_t R;
  struct {
    vuint32_t CH_EN:1;
    vuint32_t G_PC:1;
    vuint32_t TSBUF_EN:1;
    vuint32_t TSBUF_CLR:1;
    vuint32_t TMSG_TCMD:1;
    vuint32_t TIME_STAMP_A_B_SEL:1;
    vuint32_t MODE:1;
    vuint32_t L_PC0:1;
    vuint32_t F0_BYTE:3;
    vuint32_t L_PC1:1;
    vuint32_t F1_BYTE:3;
    vuint32_t L_PC2:1;
    vuint32_t F2_BYTE:3;
    vuint32_t L_PC3:1;
    vuint32_t F3_BYTE:3;
    vuint32_t L_PC4:1;
    vuint32_t F4_BYTE:3;
    vuint32_t L_PC5:1;
    vuint32_t F5_BYTE:3;
    vuint32_t L_PC_EN:1;
  } B;
} PSI5_S_CH_MSGA_tag;

typedef union PSI5_S_CH_MSGB_union_tag { /* PSI5_S channel message configuration register B */
  vuint32_t R;
  struct {
    vuint32_t F0_PAYLOAD:5;
    vuint32_t F1_PAYLOAD:5;
    vuint32_t F2_PAYLOAD:5;
    vuint32_t F3_PAYLOAD:5;
    vuint32_t F4_PAYLOAD:5;
    vuint32_t F5_PAYLOAD:5;
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} PSI5_S_CH_MSGB_tag;

typedef union PSI5_S_CH_MBOX_SR_union_tag { /* PSI5_S Mailbox status register channel */
  vuint32_t R;
  struct {
    vuint32_t F0_READ:1;
    vuint32_t F0_OV:1;
    vuint32_t F0_ERR:1;
    vuint32_t F1_READ:1;
    vuint32_t F1_OV:1;
    vuint32_t F1_ERR:1;
    vuint32_t F2_READ:1;
    vuint32_t F2_OV:1;
    vuint32_t F2_ERR:1;
    vuint32_t F3_READ:1;
    vuint32_t F3_OV:1;
    vuint32_t F3_ERR:1;
    vuint32_t F4_READ:1;
    vuint32_t F4_OV:1;
    vuint32_t F4_ERR:1;
    vuint32_t F5_READ:1;
    vuint32_t F5_OV:1;
    vuint32_t F5_ERR:1;
    vuint32_t _unused_18:14;           /* Reserved */
  } B;
} PSI5_S_CH_MBOX_SR_tag;

typedef union PSI5_S_CH_WD_CFGR_union_tag { /* PSI5_S channel watchdog configuration register */
  vuint32_t R;
  struct {
    vuint32_t WD_TO:24;
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t WDEN:1;
    vuint32_t WDCS:1;
    vuint32_t WDRST:1;
    vuint32_t _unused_28:4;            /* Reserved */
  } B;
} PSI5_S_CH_WD_CFGR_tag;

typedef union PSI5_S_CH_DDTRIG_OFFR_union_tag { /* PSI5_S DDSR Trigger offset register channel */
  vuint32_t R;
  struct {
    vuint32_t DDTRIG_OFFR:16;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} PSI5_S_CH_DDTRIG_OFFR_tag;

typedef union PSI5_S_CH_DDTRIG_PERR_union_tag { /* PSI5_S DDSR Trigger period register channel */
  vuint32_t R;
  struct {
    vuint32_t DDTRIG_PERR:16;
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} PSI5_S_CH_DDTRIG_PERR_tag;

typedef union PSI5_S_CH_E2SCR_union_tag { /* PSI5_S ECU to Sensor Control Register */
  vuint32_t R;
  struct {
    vuint32_t CMD_TYPE:3;
    vuint32_t _unused_3:2;             /* Reserved */
    vuint32_t DDSR_RDY_IE:1;
    vuint32_t CMDPR_BZY_IE:1;
    vuint32_t CMDTR_NWRT_IE:1;
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t SYNCHRO_OVF_IE:1;
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t CMDTR_SW_CTRL:1;
    vuint32_t DDSR_CLK_SEL:1;
    vuint32_t DDSR_CLR:1;
    vuint32_t DDSR_SHIFT_SEL:1;
    vuint32_t DEFAULT_SYNC:1;
    vuint32_t GL_TRIG_SEL:1;
    vuint32_t CH_TRIG:1;
    vuint32_t _unused_18:2;            /* Reserved */
    vuint32_t ACMD:5;
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t CMD:5;
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} PSI5_S_CH_E2SCR_tag;

typedef union PSI5_S_CH_E2SSR_union_tag { /* PSI5_S ECU to Sensor Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;             /* Reserved */
    vuint32_t DDSR_RDY:1;
    vuint32_t CMDPR_BZY:1;
    vuint32_t CMDTR_NWRT:1;
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t SYCHRO_OVF:1;
    vuint32_t _unused_10:22;           /* Reserved */
  } B;
} PSI5_S_CH_E2SSR_tag;

typedef union PSI5_S_CH_DDSR_H_union_tag { /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
  vuint32_t R;
  struct {
    vuint32_t DDSR_H:11;
    vuint32_t _unused_11:21;           /* Reserved */
  } B;
} PSI5_S_CH_DDSR_H_tag;

typedef union PSI5_S_CH_DDSR_L_union_tag { /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
  vuint32_t R;
  struct {
    vuint32_t DDSR_L2:24;
    vuint32_t DDSR_L1:8;
  } B;
} PSI5_S_CH_DDSR_L_tag;

typedef struct PSI5_S_CH_struct_tag {
  PSI5_S_CH_MSGA_tag MSGA;             /* PSI5_S channel message configuration register A */
  PSI5_S_CH_MSGB_tag MSGB;             /* PSI5_S channel message configuration register B */
  uint8_t CH_reserved0[4];
  PSI5_S_CH_MBOX_SR_tag MBOX_SR;       /* PSI5_S Mailbox status register channel */
  PSI5_S_CH_WD_CFGR_tag WD_CFGR;       /* PSI5_S channel watchdog configuration register */
  PSI5_S_CH_DDTRIG_OFFR_tag DDTRIG_OFFR; /* PSI5_S DDSR Trigger offset register channel */
  PSI5_S_CH_DDTRIG_PERR_tag DDTRIG_PERR; /* PSI5_S DDSR Trigger period register channel */
  PSI5_S_CH_E2SCR_tag E2SCR;           /* PSI5_S ECU to Sensor Control Register */
  PSI5_S_CH_E2SSR_tag E2SSR;           /* PSI5_S ECU to Sensor Status Register */
  PSI5_S_CH_DDSR_H_tag DDSR_H;         /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
  PSI5_S_CH_DDSR_L_tag DDSR_L;         /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
  uint8_t CH_reserved1[16];
} PSI5_S_CH_tag;

struct PSI5_S_tag {
  PSI5_S_LINCR1_tag LINCR1;            /* PSI5_S LIN Control Register 1 */
  PSI5_S_LINIER_tag LINIER;            /* PSI5_S LIN Interrupt enable register */
  PSI5_S_LINSR_tag LINSR;              /* PSI5_S LIN Status Register */
  uint8_t PSI5_S_reserved0[4];
  PSI5_S_UARTCR_tag UARTCR;            /* PSI5_S UART Mode Control Register */
  PSI5_S_UARTSR_tag UARTSR;            /* PSI5_S UART Mode Status Register */
  uint8_t PSI5_S_reserved1[12];
  PSI5_S_LINFBRR_tag LINFBRR;          /* PSI5_S LIN Fractional Baud Rate Register */
  PSI5_S_LINIBRR_tag LINIBRR;          /* PSI5_S LIN Integer Baud Rate Register */
  uint8_t PSI5_S_reserved2[4];
  PSI5_S_LINCR2_tag LINCR2;            /* PSI5_S LIN Control Register 2 */
  uint8_t PSI5_S_reserved3[4];
  PSI5_S_BDRL_tag BDRL;                /* PSI5_S Buffer Data Register Least Significant */
  PSI5_S_BDRM_tag BDRM;                /* PSI5_S Buffer Data Register Most Significant */
  uint8_t PSI5_S_reserved4[12];
  PSI5_S_GCR_tag GCR;                  /* PSI5_S Global Control register */
  PSI5_S_UARTPTO_tag UARTPTO;          /* PSI5_S UART Preset Timeout Register */
  PSI5_S_UARTCTO_tag UARTCTO;          /* PSI5_S UART Current Timeout register */
  PSI5_S_DMATXE_tag DMATXE;            /* DMA Tx Enable Register */
  PSI5_S_DMARXE_tag DMARXE;            /* DMA Rx Enable Register */
  PSI5_S_PTD_tag PTD;                  /* PSI5_S UART Tx Idle Delay Time Register */
  uint8_t PSI5_S_reserved5[80];
  PSI5_S_GLCR_tag GLCR;                /* PSI5_S Global Control Register */
  PSI5_S_GLSR_tag GLSR;                /* PSI5_S Global Status Register */
  PSI5_S_CH_BASE_ADDR_tag CH_BASE_ADDR; /* PSI5_S CHANNEL_BASE_ADDRESS */
  PSI5_S_MRU_BUF2_REG0_tag MRU_BUF2_REG0; /* PSI5_S MRU OUTPUT BUFFER2 REGISTER0 */
  PSI5_S_MRU_BUF2_REG1_tag MRU_BUF2_REG1; /* PSI5_S MRU OUTPUT BUFFER2 REGISTER1 */
  PSI5_S_MRU_BUF2_REG2_tag MRU_BUF2_REG2; /* PSI5_S MRU OUTPUT BUFFER2 REGISTER2 */
  PSI5_S_MRU_BUF2_REG3_tag MRU_BUF2_REG3; /* PSI5_S MRU OUTPUT BUFFER2 REGISTER3 */
  uint8_t PSI5_S_reserved6[16];
  PSI5_S_MBOX_SR_IRQ_tag MBOX_SR_IRQ;  /* PSI5_S Mbox Status Irq */
  PSI5_S_ERR_SR_IRQ_tag ERR_SR_IRQ;    /* PSI5_S Error Status IRQ */
  PSI5_S_IRQ_tag IRQ[8];
  PSI5_S_WDGTSSR_tag WDGTSSR;          /* PSI5_S Watchdog Error Status and Watchdog Timestamp status register */
  PSI5_S_DIRCMD_tag DIRCMD;            /* PSI5_S ECU to Sensor Direct Command Write register */
  uint8_t PSI5_S_reserved7[60];
  PSI5_S_CH0_MSGA_tag CH0_MSGA;        /* PSI5_S channel 0 message configuration register A */
  PSI5_S_CH0_MSGB_tag CH0_MSGB;        /* PSI5_S channel 0 message configuration register B */
  uint8_t PSI5_S_reserved8[4];
  PSI5_S_CH0_MBOX_SR_tag CH0_MBOX_SR;  /* PSI5_S Mailbox status register channel0 */
  uint8_t PSI5_S_reserved9[20];
  PSI5_S_CH_tag CH[7];
};


/* ============================================================================
   =============================== Module: QMAN_CNTRL =========================
   ============================================================================ */

typedef union QMAN_CNTRL_QMAN_IDM_CROSS_union_tag { /* QMAN IDM CROSS */
  vuint32_t R;
  struct {
    vuint32_t Q0_IDM_CROS:1;
    vuint32_t Q1_IDM_CROS:1;
    vuint32_t Q2_IDM_CROS:1;
    vuint32_t Q3_IDM_CROS:1;
    vuint32_t _unused_4:28;
  } B;
} QMAN_CNTRL_QMAN_IDM_CROSS_tag;

typedef union QMAN_CNTRL_QMAN_ACTIVE_union_tag { /* QMAN ACTIVE */
  vuint32_t R;
  struct {
    vuint32_t Q0_ACTIVE:1;
    vuint32_t Q1_ACTIVE:1;
    vuint32_t Q2_ACTIVE:1;
    vuint32_t Q3_ACTIVE:1;
    vuint32_t _unused_4:28;
  } B;
} QMAN_CNTRL_QMAN_ACTIVE_tag;

typedef union QMAN_CNTRL_QMAN_VI_MASK_union_tag { /* QMAN VI MASK */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t FNUM_VI_MASK:1;
  } B;
} QMAN_CNTRL_QMAN_VI_MASK_tag;

typedef union QMAN_CNTRL_QMAN_IRQ_STATUS_union_tag { /* QMAN IRQ STATUS */
  vuint32_t R;
  struct {
    vuint32_t Q0_EMPTY_VI:1;
    vuint32_t Q1_EMPTY_VI:1;
    vuint32_t Q2_EMPTY_VI:1;
    vuint32_t Q3_EMPTY_VI:1;
    vuint32_t _unused_4:27;
    vuint32_t FNUM_VI:1;
  } B;
} QMAN_CNTRL_QMAN_IRQ_STATUS_tag;

typedef union QMAN_CNTRL_QMAN_RST_union_tag { /* QMAN RST */
  vuint32_t R;
  struct {
    vuint32_t Q0_RST:1;
    vuint32_t Q1_RST:1;
    vuint32_t Q2_RST:1;
    vuint32_t Q3_RST:1;
    vuint32_t _unused_4:28;
  } B;
} QMAN_CNTRL_QMAN_RST_tag;

typedef union QMAN_CNTRL_QMAN_QFULL_union_tag { /* QMAN QFULL */
  vuint32_t R;
  struct {
    vuint32_t Q0_FULL:1;
    vuint32_t Q1_FULL:1;
    vuint32_t Q2_FULL:1;
    vuint32_t Q3_FULL:1;
    vuint32_t _unused_4:28;
  } B;
} QMAN_CNTRL_QMAN_QFULL_tag;

typedef union QMAN_CNTRL_QMAN_QFULL_MASK_union_tag { /* QMAN QFULL MASK */
  vuint32_t R;
  struct {
    vuint32_t Q0_QFULL_MASK:1;
    vuint32_t Q1_QFULL_MASK:1;
    vuint32_t Q2_QFULL_MASK:1;
    vuint32_t Q3_QFULL_MASK:1;
    vuint32_t _unused_4:28;
  } B;
} QMAN_CNTRL_QMAN_QFULL_MASK_tag;

typedef union QMAN_CNTRL_QMAN_PAUSE_union_tag { /* QMAN PAUSE */
  vuint32_t R;
  struct {
    vuint32_t QMAN_PAUSE:1;
    vuint32_t QMAN_PAUSE_STATUS:1;
    vuint32_t _unused_2:30;
  } B;
} QMAN_CNTRL_QMAN_PAUSE_tag;

struct QMAN_CNTRL_tag {
  QMAN_CNTRL_QMAN_IDM_CROSS_tag QMAN_IDM_CROSS; /* QMAN IDM CROSS */
  QMAN_CNTRL_QMAN_ACTIVE_tag QMAN_ACTIVE; /* QMAN ACTIVE */
  uint8_t QMAN_CNTRL_reserved0[12];
  QMAN_CNTRL_QMAN_VI_MASK_tag QMAN_VI_MASK; /* QMAN VI MASK */
  QMAN_CNTRL_QMAN_IRQ_STATUS_tag QMAN_IRQ_STATUS; /* QMAN IRQ STATUS */
  QMAN_CNTRL_QMAN_RST_tag QMAN_RST;    /* QMAN RST */
  uint8_t QMAN_CNTRL_reserved1[4];
  QMAN_CNTRL_QMAN_QFULL_tag QMAN_QFULL; /* QMAN QFULL */
  QMAN_CNTRL_QMAN_QFULL_MASK_tag QMAN_QFULL_MASK; /* QMAN QFULL MASK */
  uint8_t QMAN_CNTRL_reserved2[4];
  QMAN_CNTRL_QMAN_PAUSE_tag QMAN_PAUSE; /* QMAN PAUSE */
};


/* ============================================================================
   =============================== Module: QUEUE_DESCRIPTOR ===================
   ============================================================================ */

typedef union QUEUE_DESCRIPTOR_QPUSH_DSC_CFG_union_tag { /* QPUSH DSC CFG */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_DSC_CFG:32;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_DSC_CFG_tag;

typedef union QUEUE_DESCRIPTOR_QPUSH_DSC_SRCP_union_tag { /* QPUSH DSC SRCP */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_DSC_SRCP:32;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_DSC_SRCP_tag;

typedef union QUEUE_DESCRIPTOR_QPUSH_DSC_DSTP_union_tag { /* QPUSH DSC DSTP */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_DSC_DSTP:32;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_DSC_DSTP_tag;

typedef union QUEUE_DESCRIPTOR_QPUSH_DSC_DMASZ_union_tag { /* QPUSH DSC DMASZ */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_DSC_DMASZ:32;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_DSC_DMASZ_tag;

typedef union QUEUE_DESCRIPTOR_QPUSH_DSC_SRC_union_tag { /* QPUSH DSC SRC */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_DSC_SRC:32;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_DSC_SRC_tag;

typedef union QUEUE_DESCRIPTOR_QPUSH_DSC_LINE_union_tag { /* QPUSH DSC LINE */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_DSC_LINE:32;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_DSC_LINE_tag;

typedef union QUEUE_DESCRIPTOR_QPUSH_DSC_DST_union_tag { /* QPUSH DSC DST */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_DSC_DST:32;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_DSC_DST_tag;

typedef union QUEUE_DESCRIPTOR_QPUSH_DSC_HGHT_union_tag { /* QPUSH DSC HGHT */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_DSC_HGHT:32;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_DSC_HGHT_tag;

typedef union QUEUE_DESCRIPTOR_QPUSH_NUM_union_tag { /* QPUSH NUM */
  vuint32_t R;
  struct {
    vuint32_t QPUSH_NUM:5;
    vuint32_t _unused_5:1;
    vuint32_t QPUSH_AUTO_INC:1;
    vuint32_t _unused_7:9;
    vuint32_t QPUSH_STATUS_RLS:1;
    vuint32_t QPUSH_STATUS:1;
    vuint32_t _unused_18:14;
  } B;
} QUEUE_DESCRIPTOR_QPUSH_NUM_tag;

struct QUEUE_DESCRIPTOR_tag {
  QUEUE_DESCRIPTOR_QPUSH_DSC_CFG_tag QPUSH_DSC_CFG; /* QPUSH DSC CFG */
  QUEUE_DESCRIPTOR_QPUSH_DSC_SRCP_tag QPUSH_DSC_SRCP; /* QPUSH DSC SRCP */
  QUEUE_DESCRIPTOR_QPUSH_DSC_DSTP_tag QPUSH_DSC_DSTP; /* QPUSH DSC DSTP */
  QUEUE_DESCRIPTOR_QPUSH_DSC_DMASZ_tag QPUSH_DSC_DMASZ; /* QPUSH DSC DMASZ */
  QUEUE_DESCRIPTOR_QPUSH_DSC_SRC_tag QPUSH_DSC_SRC; /* QPUSH DSC SRC */
  QUEUE_DESCRIPTOR_QPUSH_DSC_LINE_tag QPUSH_DSC_LINE; /* QPUSH DSC LINE */
  QUEUE_DESCRIPTOR_QPUSH_DSC_DST_tag QPUSH_DSC_DST; /* QPUSH DSC DST */
  QUEUE_DESCRIPTOR_QPUSH_DSC_HGHT_tag QPUSH_DSC_HGHT; /* QPUSH DSC HGHT */
  QUEUE_DESCRIPTOR_QPUSH_NUM_tag QPUSH_NUM; /* QPUSH NUM */
};


/* ============================================================================
   =============================== Module: QUEUE_MANAGER ======================
   ============================================================================ */

typedef union QUEUE_MANAGER_Q0_EN_DEPTH_union_tag { /* QX EN DEPTH */
  vuint32_t R;
  struct {
    vuint32_t QX_DEPTH:13;
    vuint32_t _unused_13:3;
    vuint32_t QX_PRI_ABS:4;            /* Absolute priority for Queue x */
    vuint32_t QX_PRI_ABS_SEL:1;
    vuint32_t QX_AUTO:1;
    vuint32_t QX_CEX_PAUSE:1;
    vuint32_t _unused_23:7;
    vuint32_t QX_CONT_FRAME:1;
    vuint32_t QX_EN:1;                 /* Enable bit for activating the queue */
  } B;
} QUEUE_MANAGER_Q0_EN_DEPTH_tag;

typedef union QUEUE_MANAGER_Q0_FIRST_ADDR_union_tag { /* Q0 FIRST ADDR */
  vuint32_t R;
  struct {
    vuint32_t QX_FIRST_ADDR:32;
  } B;
} QUEUE_MANAGER_Q0_FIRST_ADDR_tag;

typedef union QUEUE_MANAGER_Q0_BASE_PTR_union_tag { /* Q0 BASE PTR */
  vuint32_t R;
  struct {
    vuint32_t QX_BASE_PTR:32;
  } B;
} QUEUE_MANAGER_Q0_BASE_PTR_tag;

typedef union QUEUE_MANAGER_Q0_CHNK_SIZE_union_tag { /* Q0 CHNK SIZE */
  vuint32_t R;
  struct {
    vuint32_t QX_CHNK_SIZE:14;
    vuint32_t _unused_14:2;
    vuint32_t QX_CHNK_LINES:16;
  } B;
} QUEUE_MANAGER_Q0_CHNK_SIZE_tag;

typedef union QUEUE_MANAGER_Q0_DSC_EN_INC0_union_tag { /* Q0 DSC EN INC0 */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_INC_VAL0:11;
    vuint32_t _unused_11:5;
    vuint32_t QX_DSC_EN0:1;
    vuint32_t QX_DSC_CNT_CFG_0:1;
    vuint32_t _unused_18:13;
    vuint32_t QX_EN_CNT_DIS0:1;
  } B;
} QUEUE_MANAGER_Q0_DSC_EN_INC0_tag;

typedef union QUEUE_MANAGER_Q0_STATUS_union_tag { /* Q0 STATUS */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t QX_WD_WAIT:1;
    vuint32_t QX_STATUS:1;
  } B;
} QUEUE_MANAGER_Q0_STATUS_tag;

typedef union QUEUE_MANAGER_Q0_RPTR_INT_union_tag { /* Q0 RPTR INT */
  vuint32_t R;
  struct {
    vuint32_t QX_RPTR_INT:18;
    vuint32_t _unused_18:14;
  } B;
} QUEUE_MANAGER_Q0_RPTR_INT_tag;

typedef union QUEUE_MANAGER_Q0_DSC_CFG_union_tag { /* Q0 DSC CFG */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_CFG:32;
  } B;
} QUEUE_MANAGER_Q0_DSC_CFG_tag;

typedef union QUEUE_MANAGER_Q0_DSC_SRCP_union_tag { /* Q0 DSC SRCP */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_SRCP:32;
  } B;
} QUEUE_MANAGER_Q0_DSC_SRCP_tag;

typedef union QUEUE_MANAGER_Q0_DSC_DSTP_union_tag { /* Q0 DSC DSTP */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DSTP:32;
  } B;
} QUEUE_MANAGER_Q0_DSC_DSTP_tag;

typedef union QUEUE_MANAGER_Q0_DSC_DMASZ_union_tag { /* Q0 DSC DMASZ */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DMASZ:32;
  } B;
} QUEUE_MANAGER_Q0_DSC_DMASZ_tag;

typedef union QUEUE_MANAGER_Q0_DSC_SRC_union_tag { /* Q0 DSC SRC */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_SRC:32;
  } B;
} QUEUE_MANAGER_Q0_DSC_SRC_tag;

typedef union QUEUE_MANAGER_Q0_DSC_LINE_union_tag { /* Q0 DSC LINE */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_LINE:32;
  } B;
} QUEUE_MANAGER_Q0_DSC_LINE_tag;

typedef union QUEUE_MANAGER_Q0_DSC_DST_union_tag { /* Q0 DSC DST */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DST:32;
  } B;
} QUEUE_MANAGER_Q0_DSC_DST_tag;

typedef union QUEUE_MANAGER_Q0_DSC_HGHT_union_tag { /* Q0 DSC HGHT */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_HGHT:32;
  } B;
} QUEUE_MANAGER_Q0_DSC_HGHT_tag;

typedef union QUEUE_MANAGER_Q0_DSC_EN_INC1_union_tag { /* Q0 DSC EN INC1 */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_INC_VAL1:11;
    vuint32_t _unused_11:5;
    vuint32_t QX_DSC_EN1:1;
    vuint32_t QX_DSC_CNT_CFG_1:1;
    vuint32_t _unused_18:13;
    vuint32_t QX_EN_CNT_DIS1:1;
  } B;
} QUEUE_MANAGER_Q0_DSC_EN_INC1_tag;

typedef union QUEUE_MANAGER_Q0_EN_CNT_union_tag { /* Q0 EN CNT */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_CNT0:14;
    vuint32_t _unused_14:2;
    vuint32_t QX_EN_CNT1:14;
    vuint32_t QX_WD_WAIT:1;
    vuint32_t _unused_31:1;
  } B;
} QUEUE_MANAGER_Q0_EN_CNT_tag;

struct QUEUE_MANAGER_tag {
  QUEUE_MANAGER_Q0_EN_DEPTH_tag Q0_EN_DEPTH; /* QX EN DEPTH */
  QUEUE_MANAGER_Q0_FIRST_ADDR_tag Q0_FIRST_ADDR; /* Q0 FIRST ADDR */
  QUEUE_MANAGER_Q0_BASE_PTR_tag Q0_BASE_PTR; /* Q0 BASE PTR */
  QUEUE_MANAGER_Q0_CHNK_SIZE_tag Q0_CHNK_SIZE; /* Q0 CHNK SIZE */
  QUEUE_MANAGER_Q0_DSC_EN_INC0_tag Q0_DSC_EN_INC0; /* Q0 DSC EN INC0 */
  QUEUE_MANAGER_Q0_STATUS_tag Q0_STATUS; /* Q0 STATUS */
  QUEUE_MANAGER_Q0_RPTR_INT_tag Q0_RPTR_INT; /* Q0 RPTR INT */
  QUEUE_MANAGER_Q0_DSC_CFG_tag Q0_DSC_CFG; /* Q0 DSC CFG */
  QUEUE_MANAGER_Q0_DSC_SRCP_tag Q0_DSC_SRCP; /* Q0 DSC SRCP */
  QUEUE_MANAGER_Q0_DSC_DSTP_tag Q0_DSC_DSTP; /* Q0 DSC DSTP */
  QUEUE_MANAGER_Q0_DSC_DMASZ_tag Q0_DSC_DMASZ; /* Q0 DSC DMASZ */
  QUEUE_MANAGER_Q0_DSC_SRC_tag Q0_DSC_SRC; /* Q0 DSC SRC */
  QUEUE_MANAGER_Q0_DSC_LINE_tag Q0_DSC_LINE; /* Q0 DSC LINE */
  QUEUE_MANAGER_Q0_DSC_DST_tag Q0_DSC_DST; /* Q0 DSC DST */
  QUEUE_MANAGER_Q0_DSC_HGHT_tag Q0_DSC_HGHT; /* Q0 DSC HGHT */
  uint8_t QUEUE_MANAGER_reserved0[4];
  QUEUE_MANAGER_Q0_DSC_EN_INC1_tag Q0_DSC_EN_INC1; /* Q0 DSC EN INC1 */
  QUEUE_MANAGER_Q0_EN_CNT_tag Q0_EN_CNT; /* Q0 EN CNT */
};


/* ============================================================================
   =============================== Module: QUEUE_MANAGER2 =====================
   ============================================================================ */

typedef union QUEUE_MANAGER2_Q2_EN_DEPTH_union_tag { /* QX EN DEPTH */
  vuint32_t R;
  struct {
    vuint32_t QX_DEPTH:13;
    vuint32_t _unused_13:3;
    vuint32_t QX_PRI_ABS:4;            /* Absolute priority for Queue x */
    vuint32_t QX_PRI_ABS_SEL:1;
    vuint32_t QX_AUTO:1;
    vuint32_t QX_CEX_PAUSE:1;
    vuint32_t _unused_23:7;
    vuint32_t QX_CONT_FRAME:1;
    vuint32_t QX_EN:1;                 /* Enable bit for activating the queue */
  } B;
} QUEUE_MANAGER2_Q2_EN_DEPTH_tag;

typedef union QUEUE_MANAGER2_Q2_FIRST_ADDR_union_tag { /* Q2 FIRST ADDR */
  vuint32_t R;
  struct {
    vuint32_t QX_FIRST_ADDR:32;
  } B;
} QUEUE_MANAGER2_Q2_FIRST_ADDR_tag;

typedef union QUEUE_MANAGER2_Q2_BASE_PTR_union_tag { /* Q2 BASE PTR */
  vuint32_t R;
  struct {
    vuint32_t QX_BASE_PTR:32;
  } B;
} QUEUE_MANAGER2_Q2_BASE_PTR_tag;

typedef union QUEUE_MANAGER2_Q2_CHNK_SIZE_union_tag { /* Q2 CHNK SIZE */
  vuint32_t R;
  struct {
    vuint32_t QX_CHNK_SIZE:14;
    vuint32_t _unused_14:2;
    vuint32_t QX_CHNK_LINES:16;
  } B;
} QUEUE_MANAGER2_Q2_CHNK_SIZE_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_EN_INC0_union_tag { /* Q2 DSC EN INC0 */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_INC_VAL0:11;
    vuint32_t _unused_11:5;
    vuint32_t QX_DSC_EN0:1;
    vuint32_t QX_DSC_CNT_CFG_0:1;
    vuint32_t _unused_18:13;
    vuint32_t QX_EN_CNT_DIS0:1;
  } B;
} QUEUE_MANAGER2_Q2_DSC_EN_INC0_tag;

typedef union QUEUE_MANAGER2_Q2_STATUS_union_tag { /* Q2 STATUS */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t QX_WD_WAIT:1;
    vuint32_t QX_STATUS:1;
  } B;
} QUEUE_MANAGER2_Q2_STATUS_tag;

typedef union QUEUE_MANAGER2_Q2_RPTR_INT_union_tag { /* Q2 RPTR INT */
  vuint32_t R;
  struct {
    vuint32_t QX_RPTR_INT:18;
    vuint32_t _unused_18:14;
  } B;
} QUEUE_MANAGER2_Q2_RPTR_INT_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_CFG_union_tag { /* Q2 DSC CFG */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_CFG:32;
  } B;
} QUEUE_MANAGER2_Q2_DSC_CFG_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_SRCP_union_tag { /* Q2 DSC SRCP */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_SRCP:32;
  } B;
} QUEUE_MANAGER2_Q2_DSC_SRCP_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_DSTP_union_tag { /* Q2 DSC DSTP */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DSTP:32;
  } B;
} QUEUE_MANAGER2_Q2_DSC_DSTP_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_DMASZ_union_tag { /* Q2 DSC DMASZ */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DMASZ:32;
  } B;
} QUEUE_MANAGER2_Q2_DSC_DMASZ_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_SRC_union_tag { /* Q2 DSC SRC */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_SRC:32;
  } B;
} QUEUE_MANAGER2_Q2_DSC_SRC_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_LINE_union_tag { /* Q2 DSC LINE */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_LINE:32;
  } B;
} QUEUE_MANAGER2_Q2_DSC_LINE_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_DST_union_tag { /* Q2 DSC DST */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DST:32;
  } B;
} QUEUE_MANAGER2_Q2_DSC_DST_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_HGHT_union_tag { /* Q2 DSC HGHT */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_HGHT:32;
  } B;
} QUEUE_MANAGER2_Q2_DSC_HGHT_tag;

typedef union QUEUE_MANAGER2_Q2_DSC_EN_INC1_union_tag { /* Q2 DSC EN INC1 */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_INC_VAL1:11;
    vuint32_t _unused_11:5;
    vuint32_t QX_DSC_EN1:1;
    vuint32_t QX_DSC_CNT_CFG_1:1;
    vuint32_t _unused_18:13;
    vuint32_t QX_EN_CNT_DIS1:1;
  } B;
} QUEUE_MANAGER2_Q2_DSC_EN_INC1_tag;

typedef union QUEUE_MANAGER2_Q2_EN_CNT_union_tag { /* Q2 EN CNT */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_CNT0:14;
    vuint32_t _unused_14:2;
    vuint32_t QX_EN_CNT1:14;
    vuint32_t QX_WD_WAIT:1;
    vuint32_t _unused_31:1;
  } B;
} QUEUE_MANAGER2_Q2_EN_CNT_tag;

struct QUEUE_MANAGER2_tag {
  QUEUE_MANAGER2_Q2_EN_DEPTH_tag Q2_EN_DEPTH; /* QX EN DEPTH */
  QUEUE_MANAGER2_Q2_FIRST_ADDR_tag Q2_FIRST_ADDR; /* Q2 FIRST ADDR */
  QUEUE_MANAGER2_Q2_BASE_PTR_tag Q2_BASE_PTR; /* Q2 BASE PTR */
  QUEUE_MANAGER2_Q2_CHNK_SIZE_tag Q2_CHNK_SIZE; /* Q2 CHNK SIZE */
  QUEUE_MANAGER2_Q2_DSC_EN_INC0_tag Q2_DSC_EN_INC0; /* Q2 DSC EN INC0 */
  QUEUE_MANAGER2_Q2_STATUS_tag Q2_STATUS; /* Q2 STATUS */
  QUEUE_MANAGER2_Q2_RPTR_INT_tag Q2_RPTR_INT; /* Q2 RPTR INT */
  QUEUE_MANAGER2_Q2_DSC_CFG_tag Q2_DSC_CFG; /* Q2 DSC CFG */
  QUEUE_MANAGER2_Q2_DSC_SRCP_tag Q2_DSC_SRCP; /* Q2 DSC SRCP */
  QUEUE_MANAGER2_Q2_DSC_DSTP_tag Q2_DSC_DSTP; /* Q2 DSC DSTP */
  QUEUE_MANAGER2_Q2_DSC_DMASZ_tag Q2_DSC_DMASZ; /* Q2 DSC DMASZ */
  QUEUE_MANAGER2_Q2_DSC_SRC_tag Q2_DSC_SRC; /* Q2 DSC SRC */
  QUEUE_MANAGER2_Q2_DSC_LINE_tag Q2_DSC_LINE; /* Q2 DSC LINE */
  QUEUE_MANAGER2_Q2_DSC_DST_tag Q2_DSC_DST; /* Q2 DSC DST */
  QUEUE_MANAGER2_Q2_DSC_HGHT_tag Q2_DSC_HGHT; /* Q2 DSC HGHT */
  uint8_t QUEUE_MANAGER2_reserved0[4];
  QUEUE_MANAGER2_Q2_DSC_EN_INC1_tag Q2_DSC_EN_INC1; /* Q2 DSC EN INC1 */
  QUEUE_MANAGER2_Q2_EN_CNT_tag Q2_EN_CNT; /* Q2 EN CNT */
};


/* ============================================================================
   =============================== Module: QUEUE_MANAGER3 =====================
   ============================================================================ */

typedef union QUEUE_MANAGER3_Q3_EN_DEPTH_union_tag { /* QX EN DEPTH */
  vuint32_t R;
  struct {
    vuint32_t QX_DEPTH:13;
    vuint32_t _unused_13:3;
    vuint32_t QX_PRI_ABS:4;            /* Absolute priority for Queue x */
    vuint32_t QX_PRI_ABS_SEL:1;
    vuint32_t QX_AUTO:1;
    vuint32_t QX_CEX_PAUSE:1;
    vuint32_t _unused_23:7;
    vuint32_t QX_CONT_FRAME:1;
    vuint32_t QX_EN:1;                 /* Enable bit for activating the queue */
  } B;
} QUEUE_MANAGER3_Q3_EN_DEPTH_tag;

typedef union QUEUE_MANAGER3_Q3_FIRST_ADDR_union_tag { /* Q3 FIRST ADDR */
  vuint32_t R;
  struct {
    vuint32_t QX_FIRST_ADDR:32;
  } B;
} QUEUE_MANAGER3_Q3_FIRST_ADDR_tag;

typedef union QUEUE_MANAGER3_Q3_BASE_PTR_union_tag { /* Q3 BASE PTR */
  vuint32_t R;
  struct {
    vuint32_t QX_BASE_PTR:32;
  } B;
} QUEUE_MANAGER3_Q3_BASE_PTR_tag;

typedef union QUEUE_MANAGER3_Q3_CHNK_SIZE_union_tag { /* Q3 CHNK SIZE */
  vuint32_t R;
  struct {
    vuint32_t QX_CHNK_SIZE:14;
    vuint32_t _unused_14:2;
    vuint32_t QX_CHNK_LINES:16;
  } B;
} QUEUE_MANAGER3_Q3_CHNK_SIZE_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_EN_INC0_union_tag { /* Q3 DSC EN INC0 */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_INC_VAL0:11;
    vuint32_t _unused_11:5;
    vuint32_t QX_DSC_EN0:1;
    vuint32_t QX_DSC_CNT_CFG_0:1;
    vuint32_t _unused_18:13;
    vuint32_t QX_EN_CNT_DIS0:1;
  } B;
} QUEUE_MANAGER3_Q3_DSC_EN_INC0_tag;

typedef union QUEUE_MANAGER3_Q3_STATUS_union_tag { /* Q3 STATUS */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t QX_WD_WAIT:1;
    vuint32_t QX_STATUS:1;
  } B;
} QUEUE_MANAGER3_Q3_STATUS_tag;

typedef union QUEUE_MANAGER3_Q3_RPTR_INT_union_tag { /* Q3 RPTR INT */
  vuint32_t R;
  struct {
    vuint32_t QX_RPTR_INT:18;
    vuint32_t _unused_18:14;
  } B;
} QUEUE_MANAGER3_Q3_RPTR_INT_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_CFG_union_tag { /* Q3 DSC CFG */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_CFG:32;
  } B;
} QUEUE_MANAGER3_Q3_DSC_CFG_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_SRCP_union_tag { /* Q3 DSC SRCP */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_SRCP:32;
  } B;
} QUEUE_MANAGER3_Q3_DSC_SRCP_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_DSTP_union_tag { /* Q3 DSC DSTP */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DSTP:32;
  } B;
} QUEUE_MANAGER3_Q3_DSC_DSTP_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_DMASZ_union_tag { /* Q3 DSC DMASZ */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DMASZ:32;
  } B;
} QUEUE_MANAGER3_Q3_DSC_DMASZ_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_SRC_union_tag { /* Q3 DSC SRC */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_SRC:32;
  } B;
} QUEUE_MANAGER3_Q3_DSC_SRC_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_LINE_union_tag { /* Q3 DSC LINE */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_LINE:32;
  } B;
} QUEUE_MANAGER3_Q3_DSC_LINE_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_DST_union_tag { /* Q3 DSC DST */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DST:32;
  } B;
} QUEUE_MANAGER3_Q3_DSC_DST_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_HGHT_union_tag { /* Q3 DSC HGHT */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_HGHT:32;
  } B;
} QUEUE_MANAGER3_Q3_DSC_HGHT_tag;

typedef union QUEUE_MANAGER3_Q3_DSC_EN_INC1_union_tag { /* Q3 DSC EN INC1 */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_INC_VAL1:11;
    vuint32_t _unused_11:5;
    vuint32_t QX_DSC_EN1:1;
    vuint32_t QX_DSC_CNT_CFG_1:1;
    vuint32_t _unused_18:13;
    vuint32_t QX_EN_CNT_DIS1:1;
  } B;
} QUEUE_MANAGER3_Q3_DSC_EN_INC1_tag;

typedef union QUEUE_MANAGER3_Q3_EN_CNT_union_tag { /* Q3 EN CNT */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_CNT0:14;
    vuint32_t _unused_14:2;
    vuint32_t QX_EN_CNT1:14;
    vuint32_t QX_WD_WAIT:1;
    vuint32_t _unused_31:1;
  } B;
} QUEUE_MANAGER3_Q3_EN_CNT_tag;

struct QUEUE_MANAGER3_tag {
  QUEUE_MANAGER3_Q3_EN_DEPTH_tag Q3_EN_DEPTH; /* QX EN DEPTH */
  QUEUE_MANAGER3_Q3_FIRST_ADDR_tag Q3_FIRST_ADDR; /* Q3 FIRST ADDR */
  QUEUE_MANAGER3_Q3_BASE_PTR_tag Q3_BASE_PTR; /* Q3 BASE PTR */
  QUEUE_MANAGER3_Q3_CHNK_SIZE_tag Q3_CHNK_SIZE; /* Q3 CHNK SIZE */
  QUEUE_MANAGER3_Q3_DSC_EN_INC0_tag Q3_DSC_EN_INC0; /* Q3 DSC EN INC0 */
  QUEUE_MANAGER3_Q3_STATUS_tag Q3_STATUS; /* Q3 STATUS */
  QUEUE_MANAGER3_Q3_RPTR_INT_tag Q3_RPTR_INT; /* Q3 RPTR INT */
  QUEUE_MANAGER3_Q3_DSC_CFG_tag Q3_DSC_CFG; /* Q3 DSC CFG */
  QUEUE_MANAGER3_Q3_DSC_SRCP_tag Q3_DSC_SRCP; /* Q3 DSC SRCP */
  QUEUE_MANAGER3_Q3_DSC_DSTP_tag Q3_DSC_DSTP; /* Q3 DSC DSTP */
  QUEUE_MANAGER3_Q3_DSC_DMASZ_tag Q3_DSC_DMASZ; /* Q3 DSC DMASZ */
  QUEUE_MANAGER3_Q3_DSC_SRC_tag Q3_DSC_SRC; /* Q3 DSC SRC */
  QUEUE_MANAGER3_Q3_DSC_LINE_tag Q3_DSC_LINE; /* Q3 DSC LINE */
  QUEUE_MANAGER3_Q3_DSC_DST_tag Q3_DSC_DST; /* Q3 DSC DST */
  QUEUE_MANAGER3_Q3_DSC_HGHT_tag Q3_DSC_HGHT; /* Q3 DSC HGHT */
  uint8_t QUEUE_MANAGER3_reserved0[4];
  QUEUE_MANAGER3_Q3_DSC_EN_INC1_tag Q3_DSC_EN_INC1; /* Q3 DSC EN INC1 */
  QUEUE_MANAGER3_Q3_EN_CNT_tag Q3_EN_CNT; /* Q3 EN CNT */
};


/* ============================================================================
   =============================== Module: QuadSPI ============================
   ============================================================================ */

typedef union QuadSPI_MCR_union_tag {  /* Module Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SWRSTSD:1;               /* Software reset for serial flash memory domain */
    vuint32_t SWRSTHD:1;               /* Software reset for AHB domain */
    vuint32_t _unused_2:2;
    vuint32_t DQS_OUT_EN:1;            /* DQS as an output */
    vuint32_t _unused_5:1;
    vuint32_t DQS_EN:1;                /* DQS enable */
    vuint32_t DDR_EN:1;                /* DDR mode enable */
    vuint32_t VAR_LAT_EN:1;            /* Variable latency */
    vuint32_t _unused_9:1;
    vuint32_t CLR_RXF:1;               /* Clear RX FIFO */
    vuint32_t CLR_TXF:1;               /* Clear TX FIFO/buffer */
    vuint32_t DLPEN:1;                 /* Data learning pattern enable */
    vuint32_t _unused_13:1;
    vuint32_t MDIS:1;                  /* Module disable */
    vuint32_t _unused_15:1;
    vuint32_t ISD2FA:1;                /* Idle signal drive IOFA[2] flash memory A */
    vuint32_t ISD3FA:1;                /* Idle signal drive IOFA[3] flash memory A */
    vuint32_t ISD2FB:1;                /* Idle signal drive IOFB[2] flash memory B */
    vuint32_t ISD3FB:1;                /* Idle signal drive IOFB[3] flash memory B */
    vuint32_t _unused_20:4;
    vuint32_t DQS_FA_SEL:2;            /* DQS clock for sampling read data at flash memory A */
    vuint32_t CKN_FA_EN:1;             /* CKN pad for flash memory A */
    vuint32_t CK2_DCARS_FA:1;          /* CK2 DDR center-aligned read strobe for flash memory A */
    vuint32_t DQS_FB_SEL:2;            /* DQS clock for sampling read data at flash memory B */
    vuint32_t CKN_FB_EN:1;             /* Enable clock on differential CKN pad of flash memory B */
    vuint32_t CK2_DCARS_FB:1;          /* CK2 DDR center-aligned read strobe for flash memory B */
  } B;
} QuadSPI_MCR_tag;

typedef union QuadSPI_IPCR_union_tag { /* IP Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t IDATSZ:16;               /* IP data transfer size */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:5;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t SEQID:4;                 /* Points to a sequence in the LUT */
    vuint32_t _unused_28:4;
  } B;
} QuadSPI_IPCR_tag;

typedef union QuadSPI_FLSHCR_union_tag { /* Flash Memory Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t TCSS:4;                  /* Serial flash memory CS setup time */
    vuint32_t _unused_4:4;
    vuint32_t TCSH:4;                  /* Serial flash memory CS hold time */
    vuint32_t _unused_12:4;
    vuint32_t TDH:2;                   /* Serial flash memory data in hold time */
    vuint32_t _unused_18:14;
  } B;
} QuadSPI_FLSHCR_tag;

typedef union QuadSPI_BUF0CR_union_tag { /* Buffer 0 Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t MSTRID:6;                /* Master ID */
    vuint32_t _unused_6:2;
    vuint32_t ADATSZ:8;                /* AHB data transfer size */
    vuint32_t _unused_16:15;
    vuint32_t _unused_31:1;
  } B;
} QuadSPI_BUF0CR_tag;

typedef union QuadSPI_BUF1CR_union_tag { /* Buffer 1 Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t MSTRID:6;                /* Master ID */
    vuint32_t _unused_6:2;
    vuint32_t ADATSZ:8;                /* AHB data transfer size */
    vuint32_t _unused_16:16;
  } B;
} QuadSPI_BUF1CR_tag;

typedef union QuadSPI_BUF2CR_union_tag { /* Buffer 2 Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t MSTRID:6;                /* Master ID */
    vuint32_t _unused_6:2;
    vuint32_t ADATSZ:8;                /* AHB data transfer size */
    vuint32_t _unused_16:16;
  } B;
} QuadSPI_BUF2CR_tag;

typedef union QuadSPI_BUF3CR_union_tag { /* Buffer 3 Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t MSTRID:6;                /* Master ID */
    vuint32_t _unused_6:2;
    vuint32_t ADATSZ:10;               /* AHB data transfer size */
    vuint32_t _unused_18:13;
    vuint32_t ALLMST:1;                /* All master enable */
  } B;
} QuadSPI_BUF3CR_tag;

typedef union QuadSPI_BFGENCR_union_tag { /* Buffer Generic Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:12;
    vuint32_t SEQID:4;                 /* Points to a sequence in the LUT. */
    vuint32_t _unused_16:1;
    vuint32_t SEQID_WR_EN:1;           /* Enable Write Sequence ID */
    vuint32_t _unused_18:10;
    vuint32_t SEQID_WR:4;              /* Write Sequence ID */
  } B;
} QuadSPI_BFGENCR_tag;

typedef union QuadSPI_SOCCR_union_tag { /* SOC Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SOCCFG:24;               /* SOC configuration */
    vuint32_t _unused_24:8;
  } B;
} QuadSPI_SOCCR_tag;

typedef union QuadSPI_BUF0IND_union_tag { /* Buffer 0 Top Index Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t TPINDX0:8;               /* Top index of buffer 0 */
    vuint32_t _unused_11:21;
  } B;
} QuadSPI_BUF0IND_tag;

typedef union QuadSPI_BUF1IND_union_tag { /* Buffer 1 Top Index Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t TPINDX1:8;               /* Top index of buffer 1 */
    vuint32_t _unused_11:21;
  } B;
} QuadSPI_BUF1IND_tag;

typedef union QuadSPI_BUF2IND_union_tag { /* Buffer 2 Top Index Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;
    vuint32_t TPINDX2:8;               /* Top index of buffer 2 */
    vuint32_t _unused_11:21;
  } B;
} QuadSPI_BUF2IND_tag;

typedef union QuadSPI_AWRCR_union_tag { /* AHB Write Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t AWTRGLVL:8;              /* AHB write trigger level */
    vuint32_t _unused_8:6;
    vuint32_t PPW_RD_DIS:1;            /* Page program wait read disabled */
    vuint32_t PPW_WR_DIS:1;            /* Page program wait write disabled */
    vuint32_t _unused_16:16;
  } B;
} QuadSPI_AWRCR_tag;

typedef union QuadSPI_DLLCRA_union_tag { /* DLL Flash Memory A Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SLV_UPD:1;               /* Slave update */
    vuint32_t SLV_DLL_BYPASS:1;        /* Slave DLL bypass */
    vuint32_t SLV_EN:1;                /* Slave enable */
    vuint32_t SLAVE_AUTO_UPDT:1;       /* Slave chain update */
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:3;
    vuint32_t SLV_DLY_COARSE:4;        /* Delay elements in each delay tap */
    vuint32_t SLV_DLY_OFFSET:3;        /* T/16 offset delay elements in incoming DQS */
    vuint32_t _unused_15:1;
    vuint32_t SLV_FINE_OFFSET:4;       /* Fine offset delay elements in incoming DQS */
    vuint32_t DLLRES:4;                /* DLL resolution */
    vuint32_t DLL_REFCNTR:4;           /* DLL reference counter */
    vuint32_t _unused_28:2;
    vuint32_t FREQEN:1;                /* Frequency enable */
    vuint32_t DLLEN:1;                 /* DLL enable */
  } B;
} QuadSPI_DLLCRA_tag;

typedef union QuadSPI_DLLCRB_union_tag { /* DLL Flash Memory B Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SLV_UPD:1;               /* Slave update */
    vuint32_t SLV_DLL_BYPASS:1;        /* Slave DLL Bypass */
    vuint32_t SLV_EN:1;                /* Slave enable */
    vuint32_t SLAVE_AUTO_UPDT:1;       /* Update slave chain */
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:3;
    vuint32_t SLV_DLY_COARSE:4;        /* Number of delay elements in each delay tap */
    vuint32_t SLV_DLY_OFFSET:3;        /* Number of delay elements in each delay tap */
    vuint32_t _unused_15:1;
    vuint32_t SLV_FINE_OFFSET:4;       /* Number of delay elements in each delay tap */
    vuint32_t DLLRES:4;                /* DLL resolution */
    vuint32_t DLL_REFCNTR:4;           /* DLL reference counter */
    vuint32_t _unused_28:2;
    vuint32_t FREQEN:1;                /* Frequency enable */
    vuint32_t DLLEN:1;                 /* DLL enable */
  } B;
} QuadSPI_DLLCRB_tag;

typedef union QuadSPI_PARITYCR_union_tag { /* Parity Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t CRCBIN_FA:1;             /* Adds CRC bar parity to flash memory A input from QuadSPI controller */
    vuint32_t CRCBEN_FA:1;             /* Adds CRC bar parity from flash memory A output to QuadSPI controller */
    vuint32_t CRCEN_FA:1;              /* CRC parity checker logic */
    vuint32_t BYTE_SIZE_FA:1;          /* Byte size for flash memory A */
    vuint32_t CHUNKSIZE_FA:6;          /* Chunk size for flash memory A */
    vuint32_t CRC_WNDW_FA:1;           /* CRC address window configuration */
    vuint32_t _unused_16:5;
    vuint32_t CRCBIN_FB:1;             /* Adds CRC bar parity to flash memory B input from QuadSPI controller */
    vuint32_t CRCBEN_FB:1;             /* Adds CRC bar parity from flash memory B output to QuadSPI controller. */
    vuint32_t CRCEN_FB:1;              /* CRC parity checker logic */
    vuint32_t BYTE_SIZE_FB:1;          /* Byte size for flash memory B */
    vuint32_t CHUNKSIZE_FB:6;          /* Chunk size for flash memory B */
    vuint32_t CRC_WNDW_FB:1;           /* CRC address window configuration */
  } B;
} QuadSPI_PARITYCR_tag;

typedef union QuadSPI_SFAR_union_tag { /* Serial Flash Memory Address Register */
  vuint32_t R;
  struct {
    vuint32_t SFADR:32;                /* Serial flash memory address */
  } B;
} QuadSPI_SFAR_tag;

typedef union QuadSPI_SFACR_union_tag { /* Serial Flash Memory Address Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t CAS:4;                   /* Column address space */
    vuint32_t _unused_4:4;
    vuint32_t PPWB:5;                  /* Page program boundary */
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:2;
    vuint32_t WA:1;                    /* Word addressable */
    vuint32_t BYTE_SWAP:1;             /* Byte swapping */
    vuint32_t _unused_18:2;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:11;
  } B;
} QuadSPI_SFACR_tag;

typedef union QuadSPI_SMPR_union_tag { /* Sampling Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t FSPHS:1;                 /* Full-speed phase selection for SDR instructions */
    vuint32_t FSDLY:1;                 /* Full-speed delay selection for internal/pad loop back DQS sampling */
    vuint32_t _unused_7:17;
    vuint32_t DLLFSMPFA:3;             /* Selects the nth tap provided by slave delay chain for flash memory A */
    vuint32_t _unused_27:1;
    vuint32_t DLLFSMPFB:3;             /* Selects the nth tap provided by slave delay chain for flash memory B */
    vuint32_t _unused_31:1;
  } B;
} QuadSPI_SMPR_tag;

typedef union QuadSPI_RBSR_union_tag { /* RX Buffer Status Register */
  vuint32_t R;
  struct {
    vuint32_t RDBFL:8;                 /* RX buffer fill level */
    vuint32_t _unused_8:8;
    vuint32_t RDCTR:16;                /* Read counter */
  } B;
} QuadSPI_RBSR_tag;

typedef union QuadSPI_RBCT_union_tag { /* RX Buffer Control Register */
  vuint32_t R;
  struct {
    vuint32_t WMRK:7;                  /* RX buffer watermark */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:23;
  } B;
} QuadSPI_RBCT_tag;

typedef union QuadSPI_AWRSR_union_tag { /* AHB Write Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t SEQAUJOIN:1;             /* Sequence auto join */
    vuint32_t _unused_3:29;
  } B;
} QuadSPI_AWRSR_tag;

typedef union QuadSPI_DLLSR_union_tag { /* DLL Status Register */
  vuint32_t R;
  struct {
    vuint32_t DLLA_SLV_COARSE_VAL:4;   /* Coarse delay cells in slave delay chain */
    vuint32_t DLLA_SLV_FINE_VAL:4;     /* Fine delay cells in slave delay chain */
    vuint32_t _unused_8:4;
    vuint32_t DLLA_FINE_UNDERFLOW:1;   /* Fine delay chain underflow */
    vuint32_t DLLA_RANGE_ERR:1;        /* DLL master delay chain */
    vuint32_t SLVA_LOCK:1;             /* Slave high lock status */
    vuint32_t DLLA_LOCK:1;             /* DLL A lock status */
    vuint32_t DLLB_SLV_COARSE_VAL:4;   /* Coarse delay cells */
    vuint32_t DLLB_SLV_FINE_VAL:4;     /* Fine delay cells in slave delay chain */
    vuint32_t _unused_24:4;
    vuint32_t DLLB_FINE_UNDERFLOW:1;   /* Fine delay chain underflow */
    vuint32_t DLLB_RANGE_ERR:1;        /* DLL master delay chain */
    vuint32_t SLVB_LOCK:1;             /* High frequency slave delay chain locked */
    vuint32_t DLLB_LOCK:1;
  } B;
} QuadSPI_DLLSR_tag;

typedef union QuadSPI_DLCR_union_tag { /* Data Learning Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;
    vuint32_t DLP_SEL_FA:2;            /* Selects pattern matching IO pads */
    vuint32_t _unused_16:8;
    vuint32_t DL_NONDLP_FLSH:1;        /* Data learning enabled for non-DLP flash memory */
    vuint32_t _unused_25:5;
    vuint32_t DLP_SEL_FB:2;            /* Selects a pattern matching IO pads */
  } B;
} QuadSPI_DLCR_tag;

typedef union QuadSPI_DLSR_FA_union_tag { /* Data Learning Status Flash Memory A Register */
  vuint32_t R;
  struct {
    vuint32_t NEG_EDGE:8;
    vuint32_t POS_EDGE:8;
    vuint32_t _unused_16:15;
    vuint32_t DLPFFA:1;                /* Data learning pattern fail */
  } B;
} QuadSPI_DLSR_FA_tag;

typedef union QuadSPI_DLSR_FB_union_tag { /* Data Learning Status Flash Memory B Register */
  vuint32_t R;
  struct {
    vuint32_t NEG_EDGE:8;
    vuint32_t POS_EDGE:8;
    vuint32_t _unused_16:15;
    vuint32_t DLPFFB:1;                /* Data learning pattern fail */
  } B;
} QuadSPI_DLSR_FB_tag;

typedef union QuadSPI_TBSR_union_tag { /* TX Buffer Status Register */
  vuint32_t R;
  struct {
    vuint32_t TRBFL:9;                 /* TX buffer fill level */
    vuint32_t _unused_9:7;
    vuint32_t TRCTR:16;                /* Transmit counter */
  } B;
} QuadSPI_TBSR_tag;

typedef union QuadSPI_TBDR_union_tag { /* TX Buffer Data Register */
  vuint32_t R;
  struct {
    vuint32_t TXDATA:32;               /* TX data */
  } B;
} QuadSPI_TBDR_tag;

typedef union QuadSPI_TBCT_union_tag { /* TX Buffer Control Register */
  vuint32_t R;
  struct {
    vuint32_t WMRK:8;                  /* Watermark for TX buffer */
    vuint32_t _unused_8:24;
  } B;
} QuadSPI_TBCT_tag;

typedef union QuadSPI_SR_union_tag {   /* Status Register */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;                  /* Module busy */
    vuint32_t IP_ACC:1;                /* IP access */
    vuint32_t AHB_ACC:1;               /* AHB read access */
    vuint32_t _unused_3:1;
    vuint32_t AWRACC:1;                /* AHB write access */
    vuint32_t _unused_5:1;
    vuint32_t AHBTRN:1;                /* AHB access transaction pending */
    vuint32_t AHB0NE:1;                /* AHB 0 buffer not empty */
    vuint32_t AHB1NE:1;                /* AHB 1 buffer not empty */
    vuint32_t AHB2NE:1;                /* AHB 2 buffer not empty */
    vuint32_t AHB3NE:1;                /* AHB 3 buffer not empty */
    vuint32_t AHB0FUL:1;               /* AHB 0 buffer full */
    vuint32_t AHB1FUL:1;               /* AHB 1 buffer full */
    vuint32_t AHB2FUL:1;               /* AHB 2 buffer full */
    vuint32_t AHB3FUL:1;               /* AHB 3 buffer full */
    vuint32_t _unused_15:1;
    vuint32_t RXWE:1;                  /* RX buffer watermark exceeded */
    vuint32_t _unused_17:2;
    vuint32_t RXFULL:1;                /* RX buffer full */
    vuint32_t _unused_20:3;
    vuint32_t RXDMA:1;                 /* RX buffer DMA */
    vuint32_t TXEDA:1;                 /* TX buffer enough data available */
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t TXFULL:1;                /* TX buffer full */
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:3;
  } B;
} QuadSPI_SR_tag;

typedef union QuadSPI_FR_union_tag {   /* Flag Register */
  vuint32_t R;
  struct {
    vuint32_t TFF:1;                   /* IP command transaction finished flag */
    vuint32_t _unused_1:5;
    vuint32_t IPIEF:1;                 /* IP command trigger could not be executed error flag */
    vuint32_t _unused_7:1;
    vuint32_t PPWF:1;                  /* Page-program wait flag after flash memory write flag */
    vuint32_t CRCBEF:1;                /* Sets when there is CRC or ECC error for flash memory B */
    vuint32_t CRCAEF:1;                /* Sets when there is CRC or ECC error for flash memory A */
    vuint32_t _unused_11:1;
    vuint32_t ABOF:1;                  /* AHB buffer overflow flag */
    vuint32_t AIBSEF:1;                /* AHB illegal burst size error flag */
    vuint32_t AITEF:1;                 /* AHB illegal transaction error flag */
    vuint32_t AAEF:1;                  /* AHB abort error flag */
    vuint32_t RBDF:1;                  /* RX buffer drain flag */
    vuint32_t RBOF:1;                  /* RX buffer overflow flag */
    vuint32_t _unused_18:5;
    vuint32_t ILLINE:1;                /* Illegal instruction error flag */
    vuint32_t DLLUNLCK:1;              /* DLL unlock */
    vuint32_t _unused_25:1;
    vuint32_t TBUF:1;                  /* TX buffer underrun flag */
    vuint32_t TBFF:1;                  /* TX buffer fill flag */
    vuint32_t DLLABRT:1;               /* DLL abort */
    vuint32_t _unused_29:2;
    vuint32_t DLPFF:1;                 /* Data learning pattern failure flag */
  } B;
} QuadSPI_FR_tag;

typedef union QuadSPI_RSER_union_tag { /* Interrupt and DMA Request Select and Enable Register */
  vuint32_t R;
  struct {
    vuint32_t TFIE:1;                  /* Transaction finished interrupt enable flag */
    vuint32_t _unused_1:5;
    vuint32_t IPIEIE:1;                /* IP command trigger during IP access error interrupt enable flag */
    vuint32_t _unused_7:1;
    vuint32_t PPWIE:1;                 /* Page-program wait interrupt flag */
    vuint32_t CRCBIE:1;                /* Sets when there is CRC or ECC error for flash memory B */
    vuint32_t CRCAIE:1;                /* CRC and ECC interrupt enable for flash memory A */
    vuint32_t _unused_11:1;
    vuint32_t ABOIE:1;                 /* AHB buffer overflow interrupt enable flag */
    vuint32_t _unused_13:2;
    vuint32_t AAIE:1;                  /* AHB abort error interrupt enable */
    vuint32_t RBDIE:1;                 /* RX buffer drain interrupt enable */
    vuint32_t RBOIE:1;                 /* RX buffer overflow interrupt enable */
    vuint32_t _unused_18:3;
    vuint32_t RBDDE:1;                 /* RX buffer drain DMA enable */
    vuint32_t _unused_22:1;
    vuint32_t ILLINIE:1;               /* Illegal instruction error interrupt enable */
    vuint32_t DLLULIE:1;               /* DLL unlock interrupt enable */
    vuint32_t TBFDE:1;                 /* TX buffer fill DMA enable */
    vuint32_t TBUIE:1;                 /* TX buffer underrun interrupt enable flag */
    vuint32_t TBFIE:1;                 /* TX buffer fill interrupt enable flag */
    vuint32_t _unused_28:3;
    vuint32_t DLPFIE:1;                /* Data learning pattern failure interrupt enable */
  } B;
} QuadSPI_RSER_tag;

typedef union QuadSPI_SPTRCLR_union_tag { /* Sequence Pointer Clear Register */
  vuint32_t R;
  struct {
    vuint32_t BFPTRC:1;                /* Buffer pointer clear */
    vuint32_t _unused_1:7;
    vuint32_t IPPTRC:1;                /* IP pointer clear */
    vuint32_t _unused_9:7;
    vuint32_t ABRT_CLR:1;              /* Flash memory Abort/AHB buffer clear */
    vuint32_t PREFETCH_DIS:1;          /* Prefetch disable */
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:4;
    vuint32_t OTFAD_BNDRY:2;           /* Otfad Prefetch Address Boundary */
    vuint32_t _unused_26:6;
  } B;
} QuadSPI_SPTRCLR_tag;

typedef union QuadSPI_SFA1AD_union_tag { /* Serial Flash Memory A1 Top Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:10;
    vuint32_t TPADA1:22;               /* Top address for serial flash memory A1 */
  } B;
} QuadSPI_SFA1AD_tag;

typedef union QuadSPI_SFA2AD_union_tag { /* Serial Flash Memory A2 Top Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:10;
    vuint32_t TPADA2:22;               /* Top address for serial flash memory A2 */
  } B;
} QuadSPI_SFA2AD_tag;

typedef union QuadSPI_SFB1AD_union_tag { /* Serial Flash Memory B1 Top Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:10;
    vuint32_t TPADB1:22;               /* Top address for serial flash memory B1. */
  } B;
} QuadSPI_SFB1AD_tag;

typedef union QuadSPI_SFB2AD_union_tag { /* Serial Flash Memory B2 Top Address Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:10;
    vuint32_t TPADB2:22;               /* Top address for serial flash memory B2. */
  } B;
} QuadSPI_SFB2AD_tag;

typedef union QuadSPI_DLPR_union_tag { /* Data Learn Pattern Register */
  vuint32_t R;
  struct {
    vuint32_t DLPV:32;                 /* Data learning pattern value */
  } B;
} QuadSPI_DLPR_tag;

typedef union QuadSPI_FAILA_ADDR_union_tag { /* Flash Memory A Failing Address Status Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Failing address for flash memory A */
  } B;
} QuadSPI_FAILA_ADDR_tag;

typedef union QuadSPI_FAILB_ADDR_union_tag { /* flash Memory B Failing Address Status Register */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Failing address for flash memory B */
  } B;
} QuadSPI_FAILB_ADDR_tag;

typedef union QuadSPI_RBDR_union_tag { /* RX Buffer Data Register */
  vuint32_t R;
  struct {
    vuint32_t RXDATA:32;               /* RX data */
  } B;
} QuadSPI_RBDR_tag;

typedef union QuadSPI_LUTKEY_union_tag { /* LUT Key Register */
  vuint32_t R;
  struct {
    vuint32_t KEY:32;                  /* Key to lock or unlock the LUT */
  } B;
} QuadSPI_LUTKEY_tag;

typedef union QuadSPI_LCKCR_union_tag { /* LUT Lock Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t LOCK:1;                  /* Lock LUT */
    vuint32_t UNLOCK:1;                /* Unlock LUT */
    vuint32_t _unused_2:30;
  } B;
} QuadSPI_LCKCR_tag;

typedef union QuadSPI_LUT_union_tag {  /* LUT Register */
  vuint32_t R;
  struct {
    vuint32_t OPRND0:8;                /* Operand for INSTR0 */
    vuint32_t PAD0:2;                  /* Pad information for INSTR0 */
    vuint32_t INSTR0:6;                /* Instruction 0 */
    vuint32_t OPRND1:8;                /* Operand for INSTR1 */
    vuint32_t PAD1:2;                  /* Pad information for INSTR1 */
    vuint32_t INSTR1:6;                /* Instruction 1 */
  } B;
} QuadSPI_LUT_tag;

typedef union QuadSPI_FRAD_WORD0_union_tag { /* Flash Region Start Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t STARTADR:16;             /* Start Address */
  } B;
} QuadSPI_FRAD_WORD0_tag;

typedef union QuadSPI_FRAD_WORD1_union_tag { /* Flash Region End Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t ENDADR:16;               /* End Address */
  } B;
} QuadSPI_FRAD_WORD1_tag;

typedef union QuadSPI_FRAD_WORD2_union_tag { /* Flash Region Privileges */
  vuint32_t R;
  struct {
    vuint32_t MD0ACP:3;                /* Master Domain Access Control Policy */
    vuint32_t MD1ACP:3;                /* Master Domain Access Control Policy */
    vuint32_t _unused_6:18;
    vuint32_t EALO:6;                  /* Exclusive Access Lock Owner */
    vuint32_t _unused_30:2;
  } B;
} QuadSPI_FRAD_WORD2_tag;

typedef union QuadSPI_FRAD_WORD3_union_tag { /* Flash Region Lock Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t EAL:2;                   /* Exclusive Access Lock */
    vuint32_t _unused_26:3;
    vuint32_t LOCK:2;                  /* Descriptor Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} QuadSPI_FRAD_WORD3_tag;

typedef union QuadSPI_FRAD_WORD4_union_tag { /* Flash Region Compare Address Status */
  vuint32_t R;
  struct {
    vuint32_t CMP_ADDR:32;             /* Capture Address */
  } B;
} QuadSPI_FRAD_WORD4_tag;

typedef union QuadSPI_FRAD_WORD5_union_tag { /* Flash Region Compare Status Data */
  vuint32_t R;
  struct {
    vuint32_t CMP_MDID:6;              /* Capture MDID Value */
    vuint32_t CMP_SA:1;                /* Capture Secure Attribute */
    vuint32_t CMP_PA:1;                /* Capture Privilege Attribute */
    vuint32_t _unused_8:21;
    vuint32_t CMP_ERR:1;               /* Comparison Error */
    vuint32_t CMPVALID:1;              /* Comparison Valid */
    vuint32_t _unused_31:1;
  } B;
} QuadSPI_FRAD_WORD5_tag;

typedef struct QuadSPI_FRAD_struct_tag {
  QuadSPI_FRAD_WORD0_tag WORD0;        /* Flash Region Start Address */
  QuadSPI_FRAD_WORD1_tag WORD1;        /* Flash Region End Address */
  QuadSPI_FRAD_WORD2_tag WORD2;        /* Flash Region Privileges */
  QuadSPI_FRAD_WORD3_tag WORD3;        /* Flash Region Lock Control */
  QuadSPI_FRAD_WORD4_tag WORD4;        /* Flash Region Compare Address Status */
  QuadSPI_FRAD_WORD5_tag WORD5;        /* Flash Region Compare Status Data */
  uint8_t FRAD_reserved0[8];
} QuadSPI_FRAD_tag;

typedef union QuadSPI_MDAD_TGMDAD_union_tag { /* Target Group n Master Domain Access Descriptor */
  vuint32_t R;
  struct {
    vuint32_t MIDMATCH:6;              /* Domain ID Reference */
    vuint32_t MASK:6;                  /* Mask */
    vuint32_t MASKTYPE:1;              /* Mask Type */
    vuint32_t _unused_13:1;
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t _unused_16:13;
    vuint32_t LCK:1;                   /* Descriptor Lock */
    vuint32_t _unused_30:1;
    vuint32_t VLD:1;                   /* Valid */
  } B;
} QuadSPI_MDAD_TGMDAD_tag;

typedef union QuadSPI_MDAD_TGSFAR_union_tag { /* Target Group n SFAR Address */
  vuint32_t R;
  struct {
    vuint32_t SFARADDR:32;             /* SFAR Address */
  } B;
} QuadSPI_MDAD_TGSFAR_tag;

typedef union QuadSPI_MDAD_TGSFARS_union_tag { /* Target Group n SFAR Status */
  vuint32_t R;
  struct {
    vuint32_t TG_MID:6;                /* Transaction Master ID */
    vuint32_t _unused_6:4;
    vuint32_t SA:1;                    /* Secure Attribute */
    vuint32_t _unused_11:1;
    vuint32_t PA:1;                    /* Privileged Attribute */
    vuint32_t _unused_13:16;
    vuint32_t CLR:1;                   /* Clear */
    vuint32_t ERR:1;                   /* Error */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} QuadSPI_MDAD_TGSFARS_tag;

typedef union QuadSPI_MDAD_TGIPCRS_union_tag { /* Target Group n IPCR Status */
  vuint32_t R;
  struct {
    vuint32_t IDATSZ:16;               /* IDATSZ Value */
    vuint32_t SEQID:4;                 /* SEQID Value */
    vuint32_t PAR:1;                   /* Parallel Mode Enable Value */
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:5;
    vuint32_t CLR:1;                   /* Clear */
    vuint32_t ERR:2;                   /* Error */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} QuadSPI_MDAD_TGIPCRS_tag;

typedef struct QuadSPI_MDAD_struct_tag {
  QuadSPI_MDAD_TGMDAD_tag TGMDAD;      /* Target Group n Master Domain Access Descriptor */
  QuadSPI_MDAD_TGSFAR_tag TGSFAR;      /* Target Group n SFAR Address */
  QuadSPI_MDAD_TGSFARS_tag TGSFARS;    /* Target Group n SFAR Status */
  QuadSPI_MDAD_TGIPCRS_tag TGIPCRS;    /* Target Group n IPCR Status */
} QuadSPI_MDAD_tag;

typedef union QuadSPI_MGC_union_tag {  /* Master Global Configuration */
  vuint32_t R;
  struct {
    vuint32_t GCLCKMID:6;              /* Global configuration Lock Owner Status */
    vuint32_t _unused_6:4;
    vuint32_t GCLCK:2;                 /* Global Configuration Lock */
    vuint32_t _unused_12:15;
    vuint32_t GVLDFRAD:1;              /* Global Valid FRAD */
    vuint32_t _unused_28:1;
    vuint32_t GVLDMDAD:1;              /* Global Valid MDAD */
    vuint32_t _unused_30:1;
    vuint32_t GVLD:1;                  /* Global Valid access control */
  } B;
} QuadSPI_MGC_tag;

typedef union QuadSPI_MRC_union_tag {  /* Master Read Command */
  vuint32_t R;
  struct {
    vuint32_t READ_CMD0:6;             /* Read Command 0 */
    vuint32_t _unused_6:2;
    vuint32_t READ_CMD1:6;             /* Read Command 1 */
    vuint32_t _unused_14:2;
    vuint32_t READ_CMD2:6;             /* Read Command 2 */
    vuint32_t VLDCMD02:1;              /* Valid command */
    vuint32_t _unused_23:1;
    vuint32_t READ_CMD3:6;             /* Read Command 3 */
    vuint32_t VLDCMD03:1;              /* Valid command */
    vuint32_t _unused_31:1;
  } B;
} QuadSPI_MRC_tag;

typedef union QuadSPI_MTO_union_tag {  /* Master Timeout */
  vuint32_t R;
  struct {
    vuint32_t WRITE_TO:32;             /* Write Timeout */
  } B;
} QuadSPI_MTO_tag;

typedef union QuadSPI_FLSEQREQ_union_tag { /* FlashSeq Request */
  vuint32_t R;
  struct {
    vuint32_t REQ_MID:6;               /* FlashSeq Request Domain ID */
    vuint32_t REQ_TG:1;                /* FlashSeq Request Target Group */
    vuint32_t _unused_7:1;
    vuint32_t SA:1;                    /* Secure Attribute */
    vuint32_t PA:1;                    /* Privilege Attribute */
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t FRAD:3;                  /* Flash Region Descriptor Number */
    vuint32_t _unused_15:1;
    vuint32_t SEQID:4;                 /* Sequence ID */
    vuint32_t _unused_20:2;
    vuint32_t CMD:1;                   /* Instruction Type */
    vuint32_t _unused_23:2;
    vuint32_t _unused_25:2;
    vuint32_t TIMEOUT:1;               /* Timeout Error Status */
    vuint32_t _unused_28:1;
    vuint32_t CLR:1;                   /* Clear */
    vuint32_t _unused_30:1;
    vuint32_t VLD:1;                   /* Valid */
  } B;
} QuadSPI_FLSEQREQ_tag;

typedef union QuadSPI_FSMSTAT_union_tag { /* FSM Status */
  vuint32_t R;
  struct {
    vuint32_t STATE:2;                 /* FSM State Status */
    vuint32_t _unused_2:6;
    vuint32_t MID:6;                   /* Domain ID */
    vuint32_t _unused_14:2;
    vuint32_t CMD:1;                   /* Command */
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:13;
    vuint32_t VLD:1;                   /* Valid */
  } B;
} QuadSPI_FSMSTAT_tag;

typedef union QuadSPI_IPSERROR_union_tag { /* IPS Error */
  vuint32_t R;
  struct {
    vuint32_t MID:6;                   /* IPS DID Domain ID */
    vuint32_t _unused_6:2;
    vuint32_t TG0LCK:1;                /* TGn Lock */
    vuint32_t TG1LCK:1;                /* TGn Lock */
    vuint32_t TG0SEC:1;                /* TGn Security Status */
    vuint32_t TG1SEC:1;                /* TGn Security Status */
    vuint32_t TG0MID:1;                /* TGn Domain-ID Status */
    vuint32_t TG1MID:1;                /* TGn Domain-ID Status */
    vuint32_t MDADPROG:1;              /* TG/MDAD Descriptor Program Status */
    vuint32_t FRADPROG:1;              /* FRAD Descriptor Program Status */
    vuint32_t _unused_16:13;
    vuint32_t CLR:1;                   /* Clear */
    vuint32_t _unused_30:2;
  } B;
} QuadSPI_IPSERROR_tag;

typedef union QuadSPI_ERRSTAT_union_tag { /* Error Status */
  vuint32_t R;
  struct {
    vuint32_t FRADMTCH:1;              /* No FRAD Match Error */
    vuint32_t FRAD0ACC:1;              /* FRADn Access Error */
    vuint32_t FRAD1ACC:1;              /* FRADn Access Error */
    vuint32_t FRAD2ACC:1;              /* FRADn Access Error */
    vuint32_t FRAD3ACC:1;              /* FRADn Access Error */
    vuint32_t FRAD4ACC:1;              /* FRADn Access Error */
    vuint32_t FRAD5ACC:1;              /* FRADn Access Error */
    vuint32_t FRAD6ACC:1;              /* FRADn Access Error */
    vuint32_t FRAD7ACC:1;              /* FRADn Access Error */
    vuint32_t IPS_ERR:1;               /* IPS Error */
    vuint32_t TG0SFAR:1;               /* TGn SFAR Error */
    vuint32_t TG1SFAR:1;               /* TGn SFAR Error */
    vuint32_t TG0IPCR:1;               /* TGn IPCR Error */
    vuint32_t TG1IPCR:1;               /* TGn IPCR Error */
    vuint32_t TO_ERR:1;                /* Timeout Error */
    vuint32_t _unused_15:17;
  } B;
} QuadSPI_ERRSTAT_tag;

typedef union QuadSPI_INT_EN_union_tag { /* Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t FRADMTCH:1;              /* No FRAD Match Error Interrupt Enable */
    vuint32_t FRAD0ACC:1;              /* FRADn Access Error Interrupt Enable */
    vuint32_t FRAD1ACC:1;              /* FRADn Access Error Interrupt Enable */
    vuint32_t FRAD2ACC:1;              /* FRADn Access Error Interrupt Enable */
    vuint32_t FRAD3ACC:1;              /* FRADn Access Error Interrupt Enable */
    vuint32_t FRAD4ACC:1;              /* FRADn Access Error Interrupt Enable */
    vuint32_t FRAD5ACC:1;              /* FRADn Access Error Interrupt Enable */
    vuint32_t FRAD6ACC:1;              /* FRADn Access Error Interrupt Enable */
    vuint32_t FRAD7ACC:1;              /* FRADn Access Error Interrupt Enable */
    vuint32_t IPS_ERR:1;               /* IPS Error Interrupt Enable */
    vuint32_t TG0SFAR:1;               /* TGn SFAR Error Interrupt Enable */
    vuint32_t TG1SFAR:1;               /* TGn SFAR Error Interrupt Enable */
    vuint32_t TG0IPCR:1;               /* TGn IPCR Error Interrupt Enable */
    vuint32_t TG1IPCR:1;               /* TGn IPCR Error Interrupt Enable */
    vuint32_t TO_ERR:1;                /* Timeout Error Interrupt Enable */
    vuint32_t _unused_15:17;
  } B;
} QuadSPI_INT_EN_tag;

struct QuadSPI_tag {
  QuadSPI_MCR_tag MCR;                 /* Module Configuration Register */
  uint8_t QuadSPI_reserved0[4];
  QuadSPI_IPCR_tag IPCR;               /* IP Configuration Register */
  QuadSPI_FLSHCR_tag FLSHCR;           /* Flash Memory Configuration Register */
  QuadSPI_BUF0CR_tag BUF0CR;           /* Buffer 0 Configuration Register */
  QuadSPI_BUF1CR_tag BUF1CR;           /* Buffer 1 Configuration Register */
  QuadSPI_BUF2CR_tag BUF2CR;           /* Buffer 2 Configuration Register */
  QuadSPI_BUF3CR_tag BUF3CR;           /* Buffer 3 Configuration Register */
  QuadSPI_BFGENCR_tag BFGENCR;         /* Buffer Generic Configuration Register */
  QuadSPI_SOCCR_tag SOCCR;             /* SOC Configuration Register */
  uint8_t QuadSPI_reserved1[8];
  QuadSPI_BUF0IND_tag BUF0IND;         /* Buffer 0 Top Index Register */
  QuadSPI_BUF1IND_tag BUF1IND;         /* Buffer 1 Top Index Register */
  QuadSPI_BUF2IND_tag BUF2IND;         /* Buffer 2 Top Index Register */
  uint8_t QuadSPI_reserved2[20];
  QuadSPI_AWRCR_tag AWRCR;             /* AHB Write Configuration Register */
  uint8_t QuadSPI_reserved3[12];
  QuadSPI_DLLCRA_tag DLLCRA;           /* DLL Flash Memory A Configuration Register */
  QuadSPI_DLLCRB_tag DLLCRB;           /* DLL Flash Memory B Configuration Register */
  uint8_t QuadSPI_reserved4[4];
  QuadSPI_PARITYCR_tag PARITYCR;       /* Parity Configuration Register */
  uint8_t QuadSPI_reserved5[144];
  QuadSPI_SFAR_tag SFAR;               /* Serial Flash Memory Address Register */
  QuadSPI_SFACR_tag SFACR;             /* Serial Flash Memory Address Configuration Register */
  QuadSPI_SMPR_tag SMPR;               /* Sampling Register */
  QuadSPI_RBSR_tag RBSR;               /* RX Buffer Status Register */
  QuadSPI_RBCT_tag RBCT;               /* RX Buffer Control Register */
  uint8_t QuadSPI_reserved6[12];
  QuadSPI_AWRSR_tag AWRSR;             /* AHB Write Status Register */
  uint8_t QuadSPI_reserved7[8];
  QuadSPI_DLLSR_tag DLLSR;             /* DLL Status Register */
  QuadSPI_DLCR_tag DLCR;               /* Data Learning Configuration Register */
  QuadSPI_DLSR_FA_tag DLSR_FA;         /* Data Learning Status Flash Memory A Register */
  QuadSPI_DLSR_FB_tag DLSR_FB;         /* Data Learning Status Flash Memory B Register */
  uint8_t QuadSPI_reserved8[20];
  QuadSPI_TBSR_tag TBSR;               /* TX Buffer Status Register */
  QuadSPI_TBDR_tag TBDR;               /* TX Buffer Data Register */
  QuadSPI_TBCT_tag TBCT;               /* TX Buffer Control Register */
  QuadSPI_SR_tag SR;                   /* Status Register */
  QuadSPI_FR_tag FR;                   /* Flag Register */
  QuadSPI_RSER_tag RSER;               /* Interrupt and DMA Request Select and Enable Register */
  uint8_t QuadSPI_reserved9[4];
  QuadSPI_SPTRCLR_tag SPTRCLR;         /* Sequence Pointer Clear Register */
  uint8_t QuadSPI_reserved10[16];
  QuadSPI_SFA1AD_tag SFA1AD;           /* Serial Flash Memory A1 Top Address Register */
  QuadSPI_SFA2AD_tag SFA2AD;           /* Serial Flash Memory A2 Top Address Register */
  QuadSPI_SFB1AD_tag SFB1AD;           /* Serial Flash Memory B1 Top Address Register */
  QuadSPI_SFB2AD_tag SFB2AD;           /* Serial Flash Memory B2 Top Address Register */
  QuadSPI_DLPR_tag DLPR;               /* Data Learn Pattern Register */
  QuadSPI_FAILA_ADDR_tag FAILA_ADDR;   /* Flash Memory A Failing Address Status Register */
  QuadSPI_FAILB_ADDR_tag FAILB_ADDR;   /* flash Memory B Failing Address Status Register */
  uint8_t QuadSPI_reserved11[100];
  QuadSPI_RBDR_tag RBDR[64];           /* RX Buffer Data Register */
  QuadSPI_LUTKEY_tag LUTKEY;           /* LUT Key Register */
  QuadSPI_LCKCR_tag LCKCR;             /* LUT Lock Configuration Register */
  uint8_t QuadSPI_reserved12[8];
  QuadSPI_LUT_tag LUT[80];             /* LUT Register */
  uint8_t QuadSPI_reserved13[944];
  QuadSPI_FRAD_tag FRAD[8];
  QuadSPI_MDAD_tag MDAD[2];
  QuadSPI_MGC_tag MGC;                 /* Master Global Configuration */
  QuadSPI_MRC_tag MRC;                 /* Master Read Command */
  QuadSPI_MTO_tag MTO;                 /* Master Timeout */
  QuadSPI_FLSEQREQ_tag FLSEQREQ;       /* FlashSeq Request */
  QuadSPI_FSMSTAT_tag FSMSTAT;         /* FSM Status */
  QuadSPI_IPSERROR_tag IPSERROR;       /* IPS Error */
  QuadSPI_ERRSTAT_tag ERRSTAT;         /* Error Status */
  QuadSPI_INT_EN_tag INT_EN;           /* Interrupt Enable */
};


/* ============================================================================
   =============================== Module: QuadSPI_ARDB =======================
   ============================================================================ */

typedef union QuadSPI_ARDB_ARDB_union_tag { /* AHB RX Data Buffer Register */
  vuint32_t R;
  struct {
    vuint32_t ARXD:32;                 /* ARDB provided RX buffer data */
  } B;
} QuadSPI_ARDB_ARDB_tag;

struct QuadSPI_ARDB_tag {
  QuadSPI_ARDB_ARDB_tag ARDB[128];     /* AHB RX Data Buffer Register */
};


/* ============================================================================
   =============================== Module: Queue_Manager1 =====================
   ============================================================================ */

typedef union Queue_Manager1_Q1_EN_DEPTH_union_tag { /* QX EN DEPTH */
  vuint32_t R;
  struct {
    vuint32_t QX_DEPTH:13;
    vuint32_t _unused_13:3;
    vuint32_t QX_PRI_ABS:4;            /* Absolute priority for Queue x */
    vuint32_t QX_PRI_ABS_SEL:1;
    vuint32_t QX_AUTO:1;
    vuint32_t QX_CEX_PAUSE:1;
    vuint32_t _unused_23:7;
    vuint32_t QX_CONT_FRAME:1;
    vuint32_t QX_EN:1;                 /* Enable bit for activating the queue */
  } B;
} Queue_Manager1_Q1_EN_DEPTH_tag;

typedef union Queue_Manager1_Q1_FIRST_ADDR_union_tag { /* Q1 FIRST ADDR */
  vuint32_t R;
  struct {
    vuint32_t QX_FIRST_ADDR:32;
  } B;
} Queue_Manager1_Q1_FIRST_ADDR_tag;

typedef union Queue_Manager1_Q1_BASE_PTR_union_tag { /* Q1 BASE PTR */
  vuint32_t R;
  struct {
    vuint32_t QX_BASE_PTR:32;
  } B;
} Queue_Manager1_Q1_BASE_PTR_tag;

typedef union Queue_Manager1_Q1_CHNK_SIZE_union_tag { /* Q1 CHNK SIZE */
  vuint32_t R;
  struct {
    vuint32_t QX_CHNK_SIZE:14;
    vuint32_t _unused_14:2;
    vuint32_t QX_CHNK_LINES:16;
  } B;
} Queue_Manager1_Q1_CHNK_SIZE_tag;

typedef union Queue_Manager1_Q1_DSC_EN_INC0_union_tag { /* Q1 DSC EN INC0 */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_INC_VAL0:11;
    vuint32_t _unused_11:5;
    vuint32_t QX_DSC_EN0:1;
    vuint32_t QX_DSC_CNT_CFG_0:1;
    vuint32_t _unused_18:13;
    vuint32_t QX_EN_CNT_DIS0:1;
  } B;
} Queue_Manager1_Q1_DSC_EN_INC0_tag;

typedef union Queue_Manager1_Q1_STATUS_union_tag { /* Q1 STATUS */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t QX_WD_WAIT:1;
    vuint32_t QX_STATUS:1;
  } B;
} Queue_Manager1_Q1_STATUS_tag;

typedef union Queue_Manager1_Q1_RPTR_INT_union_tag { /* Q1 RPTR INT */
  vuint32_t R;
  struct {
    vuint32_t QX_RPTR_INT:18;
    vuint32_t _unused_18:14;
  } B;
} Queue_Manager1_Q1_RPTR_INT_tag;

typedef union Queue_Manager1_Q1_DSC_CFG_union_tag { /* Q1 DSC CFG */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_CFG:32;
  } B;
} Queue_Manager1_Q1_DSC_CFG_tag;

typedef union Queue_Manager1_Q1_DSC_SRCP_union_tag { /* Q1 DSC SRCP */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_SRCP:32;
  } B;
} Queue_Manager1_Q1_DSC_SRCP_tag;

typedef union Queue_Manager1_Q1_DSC_DSTP_union_tag { /* Q1 DSC DSTP */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DSTP:32;
  } B;
} Queue_Manager1_Q1_DSC_DSTP_tag;

typedef union Queue_Manager1_Q1_DSC_DMASZ_union_tag { /* Q1 DSC DMASZ */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DMASZ:32;
  } B;
} Queue_Manager1_Q1_DSC_DMASZ_tag;

typedef union Queue_Manager1_Q1_DSC_SRC_union_tag { /* Q1 DSC SRC */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_SRC:32;
  } B;
} Queue_Manager1_Q1_DSC_SRC_tag;

typedef union Queue_Manager1_Q1_DSC_LINE_union_tag { /* Q1 DSC LINE */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_LINE:32;
  } B;
} Queue_Manager1_Q1_DSC_LINE_tag;

typedef union Queue_Manager1_Q1_DSC_DST_union_tag { /* Q1 DSC DST */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_DST:32;
  } B;
} Queue_Manager1_Q1_DSC_DST_tag;

typedef union Queue_Manager1_Q1_DSC_HGHT_union_tag { /* Q1 DSC HGHT */
  vuint32_t R;
  struct {
    vuint32_t QX_DSC_HGHT:32;
  } B;
} Queue_Manager1_Q1_DSC_HGHT_tag;

typedef union Queue_Manager1_Q1_DSC_EN_INC1_union_tag { /* Q1 DSC EN INC1 */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_INC_VAL1:11;
    vuint32_t _unused_11:5;
    vuint32_t QX_DSC_EN1:1;
    vuint32_t QX_DSC_CNT_CFG_1:1;
    vuint32_t _unused_18:13;
    vuint32_t QX_EN_CNT_DIS1:1;
  } B;
} Queue_Manager1_Q1_DSC_EN_INC1_tag;

typedef union Queue_Manager1_Q1_EN_CNT_union_tag { /* Q1 EN CNT */
  vuint32_t R;
  struct {
    vuint32_t QX_EN_CNT0:14;
    vuint32_t _unused_14:2;
    vuint32_t QX_EN_CNT1:14;
    vuint32_t QX_WD_WAIT:1;
    vuint32_t _unused_31:1;
  } B;
} Queue_Manager1_Q1_EN_CNT_tag;

struct Queue_Manager1_tag {
  Queue_Manager1_Q1_EN_DEPTH_tag Q1_EN_DEPTH; /* QX EN DEPTH */
  Queue_Manager1_Q1_FIRST_ADDR_tag Q1_FIRST_ADDR; /* Q1 FIRST ADDR */
  Queue_Manager1_Q1_BASE_PTR_tag Q1_BASE_PTR; /* Q1 BASE PTR */
  Queue_Manager1_Q1_CHNK_SIZE_tag Q1_CHNK_SIZE; /* Q1 CHNK SIZE */
  Queue_Manager1_Q1_DSC_EN_INC0_tag Q1_DSC_EN_INC0; /* Q1 DSC EN INC0 */
  Queue_Manager1_Q1_STATUS_tag Q1_STATUS; /* Q1 STATUS */
  Queue_Manager1_Q1_RPTR_INT_tag Q1_RPTR_INT; /* Q1 RPTR INT */
  Queue_Manager1_Q1_DSC_CFG_tag Q1_DSC_CFG; /* Q1 DSC CFG */
  Queue_Manager1_Q1_DSC_SRCP_tag Q1_DSC_SRCP; /* Q1 DSC SRCP */
  Queue_Manager1_Q1_DSC_DSTP_tag Q1_DSC_DSTP; /* Q1 DSC DSTP */
  Queue_Manager1_Q1_DSC_DMASZ_tag Q1_DSC_DMASZ; /* Q1 DSC DMASZ */
  Queue_Manager1_Q1_DSC_SRC_tag Q1_DSC_SRC; /* Q1 DSC SRC */
  Queue_Manager1_Q1_DSC_LINE_tag Q1_DSC_LINE; /* Q1 DSC LINE */
  Queue_Manager1_Q1_DSC_DST_tag Q1_DSC_DST; /* Q1 DSC DST */
  Queue_Manager1_Q1_DSC_HGHT_tag Q1_DSC_HGHT; /* Q1 DSC HGHT */
  uint8_t Queue_Manager1_reserved0[4];
  Queue_Manager1_Q1_DSC_EN_INC1_tag Q1_DSC_EN_INC1; /* Q1 DSC EN INC1 */
  Queue_Manager1_Q1_EN_CNT_tag Q1_EN_CNT; /* Q1 EN CNT */
};


/* ============================================================================
   =============================== Module: RDC ================================
   ============================================================================ */

typedef union RDC_RD1_CTRL_REG_union_tag { /* Software Reset Domain Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;             /* Reserved */
    vuint32_t RD1_INTERCONNECT_INTERFACE_DISABLE:1; /* Interconnect Interface Disable of Software Reset Domain */
    vuint32_t _unused_4:27;            /* Reserved */
    vuint32_t RD1_CTRL_UNLOCK:1;       /* Reset Domain Control Register Unlock */
  } B;
} RDC_RD1_CTRL_REG_tag;

typedef union RDC_RD1_STAT_REG_union_tag { /* Software Reset Domain Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;             /* Reserved */
    vuint32_t RD1_INTERCONNECT_INTERFACE_DISABLE_REQ_ACK_STAT:1; /* Interconnect Interface Disable Request Acknowledgment Status of Software Reset Domain */
    vuint32_t RD1_INTERCONNECT_INTERFACE_DISABLE_STAT:1; /* Interconnect Interface Disable Status of Software Reset Domain */
    vuint32_t _unused_5:27;            /* Reserved */
  } B;
} RDC_RD1_STAT_REG_tag;

struct RDC_tag {
  uint8_t RDC_reserved0[4];
  RDC_RD1_CTRL_REG_tag RD1_CTRL_REG;   /* Software Reset Domain Control */
  uint8_t RDC_reserved1[124];
  RDC_RD1_STAT_REG_tag RD1_STAT_REG;   /* Software Reset Domain Status */
};


/* ============================================================================
   =============================== Module: RESULT_DMACRC ======================
   ============================================================================ */

typedef union RESULT_DMACRC_GEC_union_tag { /* Global Enable CRC Register */
  vuint32_t R;
  struct {
    vuint32_t GBL_EN:1;                /* Global Enable bit */
    vuint32_t _unused_1:6;             /* Reserved */
    vuint32_t SWAP_BYTE:1;             /* Swap Byte */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} RESULT_DMACRC_GEC_tag;

typedef union RESULT_DMACRC_CONTROL_REGISTER_CTL_union_tag { /* CRC Control Register */
  vuint32_t R;
  struct {
    vuint32_t CH_SEL:5;                /* Channel Select */
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:2;             /* Reserved */
    vuint32_t POLY_SEL:3;              /* Polynomial Select */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t CS_XOR:1;
    vuint32_t CS_SWAP:1;
    vuint32_t SWAP_BIT:1;              /* Swap Bit */
    vuint32_t INIT_SEL:1;              /* Initial values of the CRC */
    vuint32_t MODE:3;                  /* CRC Mode */
    vuint32_t _unused_19:12;           /* Reserved */
    vuint32_t EN:1;                    /* CRC Logic */
  } B;
} RESULT_DMACRC_CONTROL_REGISTER_CTL_tag;

typedef union RESULT_DMACRC_CONTROL_REGISTER_ICRC_union_tag { /* Initial CRC Value Register */
  vuint32_t R;
  struct {
    vuint32_t INI_CRC_VAL:32;          /* Initial CRC Value */
  } B;
} RESULT_DMACRC_CONTROL_REGISTER_ICRC_tag;

typedef union RESULT_DMACRC_CONTROL_REGISTER_FCRC_union_tag { /* Final CRC Value Register */
  vuint32_t R;
  struct {
    vuint32_t CHKSUM_VAL:32;           /* Final CRC Value */
  } B;
} RESULT_DMACRC_CONTROL_REGISTER_FCRC_tag;

typedef struct RESULT_DMACRC_CONTROL_REGISTER_struct_tag {
  RESULT_DMACRC_CONTROL_REGISTER_CTL_tag CTL; /* CRC Control Register */
  RESULT_DMACRC_CONTROL_REGISTER_ICRC_tag ICRC; /* Initial CRC Value Register */
  RESULT_DMACRC_CONTROL_REGISTER_FCRC_tag FCRC; /* Final CRC Value Register */
  uint8_t CONTROL_REGISTER_reserved0[4];
} RESULT_DMACRC_CONTROL_REGISTER_tag;

struct RESULT_DMACRC_tag {
  RESULT_DMACRC_GEC_tag GEC;           /* Global Enable CRC Register */
  uint8_t RESULT_DMACRC_reserved0[12];
  RESULT_DMACRC_CONTROL_REGISTER_tag CONTROL_REGISTER[8];
};


/* ============================================================================
   =============================== Module: RESULT_DMA_MP ======================
   ============================================================================ */

typedef union RESULT_DMA_MP_CSR_union_tag { /* Management Page Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t EDBG:1;                  /* Enable Debug */
    vuint32_t ERCA:1;                  /* Enable Round Robin Channel Arbitration */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t HAE:1;                   /* Halt After Error */
    vuint32_t HALT:1;                  /* Halt DMA Operations */
    vuint32_t GCLC:1;                  /* Global Channel Linking Control */
    vuint32_t GMRC:1;                  /* Global Master ID Replication Control */
    vuint32_t ECX:1;                   /* Cancel Transfer With Error */
    vuint32_t CX:1;                    /* Cancel Transfer */
    vuint32_t _unused_10:6;            /* Reserved */
    vuint32_t _unused_16:8;            /* Reserved */
    vuint32_t ACTIVE_ID:5;             /* Active Channel ID */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t ACTIVE:1;                /* DMA Active Status */
  } B;
} RESULT_DMA_MP_CSR_tag;

typedef union RESULT_DMA_MP_ES_union_tag { /* Management Page Error Status */
  vuint32_t R;
  struct {
    vuint32_t DBE:1;                   /* Destination Bus Error */
    vuint32_t SBE:1;                   /* Source Bus Error */
    vuint32_t SGE:1;                   /* Scatter/Gather Configuration Error */
    vuint32_t NCE:1;                   /* NBYTES/CITER Configuration Error */
    vuint32_t DOE:1;                   /* Destination Offset Error */
    vuint32_t DAE:1;                   /* Destination Address Error */
    vuint32_t SOE:1;                   /* Source Offset Error */
    vuint32_t SAE:1;                   /* Source Address Error */
    vuint32_t ECX:1;                   /* Transfer Canceled */
    vuint32_t UCE:1;                   /* Uncorrectable TCD Error During Channel Execution */
    vuint32_t _unused_10:14;           /* Reserved */
    vuint32_t ERRCHN:5;                /* Error Channel Number or Canceled Channel Number */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RESULT_DMA_MP_ES_tag;

typedef union RESULT_DMA_MP_INT_union_tag { /* Management Page Interrupt Request Status */
  vuint32_t R;
  struct {
    vuint32_t INT:24;                  /* Interrupt Request Status */
    vuint32_t _unused_24:8;
  } B;
} RESULT_DMA_MP_INT_tag;

typedef union RESULT_DMA_MP_HRS_union_tag { /* Management Page Hardware Request Status */
  vuint32_t R;
  struct {
    vuint32_t HRS:32;                  /* Hardware Request Status */
  } B;
} RESULT_DMA_MP_HRS_tag;

typedef union RESULT_DMA_MP_CH_GRPRI_union_tag { /* Channel Arbitration Group */
  vuint32_t R;
  struct {
    vuint32_t GRPRI:5;                 /* Arbitration Group For Channel n */
    vuint32_t _unused_5:27;
  } B;
} RESULT_DMA_MP_CH_GRPRI_tag;

struct RESULT_DMA_MP_tag {
  RESULT_DMA_MP_CSR_tag CSR;           /* Management Page Control */
  RESULT_DMA_MP_ES_tag ES;             /* Management Page Error Status */
  RESULT_DMA_MP_INT_tag INT;           /* Management Page Interrupt Request Status */
  RESULT_DMA_MP_HRS_tag HRS;           /* Management Page Hardware Request Status */
  uint8_t RESULT_DMA_MP_reserved0[240];
  RESULT_DMA_MP_CH_GRPRI_tag CH_GRPRI[24]; /* Channel Arbitration Group */
};


/* ============================================================================
   =============================== Module: RESULT_DMA_TCD =====================
   ============================================================================ */

typedef union RESULT_DMA_TCD_TCD_CH_CSR_union_tag { /* Channel Control and Status */
  vuint32_t R;
  struct {
    vuint32_t ERQ:1;                   /* Enable DMA Request */
    vuint32_t EARQ:1;                  /* Enable Asynchronous DMA Request In Stop Mode For Channel */
    vuint32_t EEI:1;                   /* Enable Error Interrupt */
    vuint32_t EBW:1;                   /* Enable Buffered Writes */
    vuint32_t _unused_4:26;            /* Reserved */
    vuint32_t DONE:1;                  /* Channel Done */
    vuint32_t ACTIVE:1;                /* Channel Active */
  } B;
} RESULT_DMA_TCD_TCD_CH_CSR_tag;

typedef union RESULT_DMA_TCD_TCD_CH_ES_union_tag { /* Channel Error Status */
  vuint32_t R;
  struct {
    vuint32_t DBE:1;                   /* Destination Bus Error */
    vuint32_t SBE:1;                   /* Source Bus Error */
    vuint32_t SGE:1;                   /* Scatter/Gather Configuration Error */
    vuint32_t NCE:1;                   /* NBYTES/CITER Configuration Error */
    vuint32_t DOE:1;                   /* Destination Offset Error */
    vuint32_t DAE:1;                   /* Destination Address Error */
    vuint32_t SOE:1;                   /* Source Offset Error */
    vuint32_t SAE:1;                   /* Source Address Error */
    vuint32_t _unused_8:23;            /* Reserved */
    vuint32_t ERR:1;                   /* Error In Channel */
  } B;
} RESULT_DMA_TCD_TCD_CH_ES_tag;

typedef union RESULT_DMA_TCD_TCD_CH_INT_union_tag { /* Channel Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t INT:1;                   /* Interrupt Request */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} RESULT_DMA_TCD_TCD_CH_INT_tag;

typedef union RESULT_DMA_TCD_TCD_CH_SBR_union_tag { /* Channel System Bus */
  vuint32_t R;
  struct {
    vuint32_t MID:6;                   /* Master ID */
    vuint32_t _unused_6:9;             /* Reserved */
    vuint32_t PAL:1;                   /* Privileged Access Level */
    vuint32_t EMI:1;                   /* Enable Master ID Replication */
    vuint32_t ATTR:3;                  /* Attribute Output */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} RESULT_DMA_TCD_TCD_CH_SBR_tag;

typedef union RESULT_DMA_TCD_TCD_CH_PRI_union_tag { /* Channel Priority */
  vuint32_t R;
  struct {
    vuint32_t APL:3;                   /* Arbitration Priority Level */
    vuint32_t _unused_3:27;            /* Reserved */
    vuint32_t DPA:1;                   /* Disable Preempt Ability */
    vuint32_t ECP:1;                   /* Enable Channel Preemption */
  } B;
} RESULT_DMA_TCD_TCD_CH_PRI_tag;

typedef union RESULT_DMA_TCD_TCD_SADDR_union_tag { /* TCD Source Address */
  vuint32_t R;
  struct {
    vuint32_t SADDR:32;                /* Source Address */
  } B;
} RESULT_DMA_TCD_TCD_SADDR_tag;

typedef union RESULT_DMA_TCD_TCD_SOFF_union_tag { /* TCD Signed Source Address Offset */
  vuint16_t R;
  struct {
    vuint16_t SOFF:16;                 /* Source Address Signed Offset */
  } B;
} RESULT_DMA_TCD_TCD_SOFF_tag;

typedef union RESULT_DMA_TCD_TCD_ATTR_union_tag { /* TCD Transfer Attributes */
  vuint16_t R;
  struct {
    vuint16_t DSIZE:3;                 /* Destination Data Transfer Size */
    vuint16_t DMOD:5;                  /* Destination Address Modulo */
    vuint16_t SSIZE:3;                 /* Source Data Transfer Size */
    vuint16_t SMOD:5;                  /* Source Address Modulo */
  } B;
} RESULT_DMA_TCD_TCD_ATTR_tag;

typedef union RESULT_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFNO_union_tag { /* TCD Transfer Size Without Minor Loop Offsets */
  vuint32_t R;
  struct {
    vuint32_t NBYTES:30;               /* Number of Bytes To Transfer Per Service Request */
    vuint32_t DMLOE:1;                 /* Destination Minor Loop Offset Enable */
    vuint32_t SMLOE:1;                 /* Source Minor Loop Offset Enable */
  } B;
} RESULT_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFNO_tag;

typedef union RESULT_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFYES_union_tag { /* TCD Transfer Size with Minor Loop Offsets */
  vuint32_t R;
  struct {
    vuint32_t NBYTES:10;               /* Number of Bytes To Transfer Per Service Request */
    vuint32_t MLOFF:20;                /* Minor Loop Offset */
    vuint32_t DMLOE:1;                 /* Destination Minor Loop Offset Enable */
    vuint32_t SMLOE:1;                 /* Source Minor Loop Offset Enable */
  } B;
} RESULT_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFYES_tag;

typedef union RESULT_DMA_TCD_TCD_NBYTES_union_tag {
  RESULT_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFNO_tag NBYTES_MLOFFNO; /* TCD Transfer Size Without Minor Loop Offsets */
  RESULT_DMA_TCD_TCD_NBYTES_NBYTES_MLOFFYES_tag NBYTES_MLOFFYES; /* TCD Transfer Size with Minor Loop Offsets */
} RESULT_DMA_TCD_TCD_NBYTES_tag;

typedef union RESULT_DMA_TCD_TCD_SLAST_SDA_union_tag { /* TCD Last Source Address Adjustment / Store DADDR Address */
  vuint32_t R;
  struct {
    vuint32_t SLAST_SDA:32;            /* Last Source Address Adjustment / Store DADDR Address */
  } B;
} RESULT_DMA_TCD_TCD_SLAST_SDA_tag;

typedef union RESULT_DMA_TCD_TCD_DADDR_union_tag { /* TCD Destination Address */
  vuint32_t R;
  struct {
    vuint32_t DADDR:32;                /* Destination Address */
  } B;
} RESULT_DMA_TCD_TCD_DADDR_tag;

typedef union RESULT_DMA_TCD_TCD_DOFF_union_tag { /* TCD Signed Destination Address Offset */
  vuint16_t R;
  struct {
    vuint16_t DOFF:16;                 /* Destination Address Signed Offset */
  } B;
} RESULT_DMA_TCD_TCD_DOFF_tag;

typedef union RESULT_DMA_TCD_TCD_CITER_CITER_ELINKNO_union_tag { /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
  vuint16_t R;
  struct {
    vuint16_t CITER:15;                /* Current Major Iteration Count */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} RESULT_DMA_TCD_TCD_CITER_CITER_ELINKNO_tag;

typedef union RESULT_DMA_TCD_TCD_CITER_CITER_ELINKYES_union_tag { /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
  vuint16_t R;
  struct {
    vuint16_t CITER:9;                 /* Current Major Iteration Count */
    vuint16_t LINKCH:5;                /* Minor Loop Link Channel Number */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} RESULT_DMA_TCD_TCD_CITER_CITER_ELINKYES_tag;

typedef union RESULT_DMA_TCD_TCD_CITER_union_tag {
  RESULT_DMA_TCD_TCD_CITER_CITER_ELINKNO_tag CITER_ELINKNO; /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
  RESULT_DMA_TCD_TCD_CITER_CITER_ELINKYES_tag CITER_ELINKYES; /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
} RESULT_DMA_TCD_TCD_CITER_tag;

typedef union RESULT_DMA_TCD_TCD_DLAST_SGA_union_tag { /* TCD Last Destination Address Adjustment / Scatter Gather Address */
  vuint32_t R;
  struct {
    vuint32_t DLAST_SGA:32;            /* Last Destination Address Adjustment / Scatter Gather Address */
  } B;
} RESULT_DMA_TCD_TCD_DLAST_SGA_tag;

typedef union RESULT_DMA_TCD_TCD_CSR_union_tag { /* TCD Control and Status */
  vuint16_t R;
  struct {
    vuint16_t START:1;                 /* Channel Start */
    vuint16_t INTMAJOR:1;              /* Enable Interrupt If Major count complete */
    vuint16_t INTHALF:1;               /* Enable Interrupt If Major Counter Half-complete */
    vuint16_t DREQ:1;                  /* Disable Request */
    vuint16_t ESG:1;                   /* Enable Scatter/Gather Processing */
    vuint16_t MAJORELINK:1;            /* Enable Link When Major Loop Complete */
    vuint16_t _unused_6:1;
    vuint16_t ESDA:1;                  /* Enable Store Destination Address */
    vuint16_t MAJORLINKCH:5;           /* Major Loop Link Channel Number */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t BWC:2;                   /* Bandwidth Control */
  } B;
} RESULT_DMA_TCD_TCD_CSR_tag;

typedef union RESULT_DMA_TCD_TCD_BITER_BITER_ELINKNO_union_tag { /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
  vuint16_t R;
  struct {
    vuint16_t BITER:15;                /* Starting Major Iteration Count */
    vuint16_t ELINK:1;                 /* Enables Link */
  } B;
} RESULT_DMA_TCD_TCD_BITER_BITER_ELINKNO_tag;

typedef union RESULT_DMA_TCD_TCD_BITER_BITER_ELINKYES_union_tag { /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
  vuint16_t R;
  struct {
    vuint16_t BITER:9;                 /* Starting Major Iteration Count */
    vuint16_t LINKCH:5;                /* Link Channel Number */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t ELINK:1;                 /* Enable Link */
  } B;
} RESULT_DMA_TCD_TCD_BITER_BITER_ELINKYES_tag;

typedef union RESULT_DMA_TCD_TCD_BITER_union_tag {
  RESULT_DMA_TCD_TCD_BITER_BITER_ELINKNO_tag BITER_ELINKNO; /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
  RESULT_DMA_TCD_TCD_BITER_BITER_ELINKYES_tag BITER_ELINKYES; /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
} RESULT_DMA_TCD_TCD_BITER_tag;

typedef struct RESULT_DMA_TCD_TCD_struct_tag {
  RESULT_DMA_TCD_TCD_CH_CSR_tag CH_CSR; /* Channel Control and Status */
  RESULT_DMA_TCD_TCD_CH_ES_tag CH_ES;  /* Channel Error Status */
  RESULT_DMA_TCD_TCD_CH_INT_tag CH_INT; /* Channel Interrupt Status */
  RESULT_DMA_TCD_TCD_CH_SBR_tag CH_SBR; /* Channel System Bus */
  RESULT_DMA_TCD_TCD_CH_PRI_tag CH_PRI; /* Channel Priority */
  uint8_t TCD_reserved0[12];
  RESULT_DMA_TCD_TCD_SADDR_tag SADDR;  /* TCD Source Address */
  RESULT_DMA_TCD_TCD_SOFF_tag SOFF;    /* TCD Signed Source Address Offset */
  RESULT_DMA_TCD_TCD_ATTR_tag ATTR;    /* TCD Transfer Attributes */
  RESULT_DMA_TCD_TCD_NBYTES_tag NBYTES;
  RESULT_DMA_TCD_TCD_SLAST_SDA_tag SLAST_SDA; /* TCD Last Source Address Adjustment / Store DADDR Address */
  RESULT_DMA_TCD_TCD_DADDR_tag DADDR;  /* TCD Destination Address */
  RESULT_DMA_TCD_TCD_DOFF_tag DOFF;    /* TCD Signed Destination Address Offset */
  RESULT_DMA_TCD_TCD_CITER_tag CITER;
  RESULT_DMA_TCD_TCD_DLAST_SGA_tag DLAST_SGA; /* TCD Last Destination Address Adjustment / Scatter Gather Address */
  RESULT_DMA_TCD_TCD_CSR_tag CSR;      /* TCD Control and Status */
  RESULT_DMA_TCD_TCD_BITER_tag BITER;
  uint8_t TCD_reserved1[4032];
} RESULT_DMA_TCD_TCD_tag;

struct RESULT_DMA_TCD_tag {
  RESULT_DMA_TCD_TCD_tag TCD[24];
};


/* ============================================================================
   =============================== Module: ROUND_ROBIN_ARBITER ================
   ============================================================================ */

typedef union ROUND_ROBIN_ARBITER_RRA_SLT_LO_union_tag { /* RRA Slot LO Register */
  vuint32_t R;
  struct {
    vuint32_t SLOT0_REQM:4;
    vuint32_t SLOT1_REQM:4;
    vuint32_t SLOT2_REQM:4;
    vuint32_t SLOT3_REQM:4;
    vuint32_t SLOT4_REQM:4;
    vuint32_t SLOT5_REQM:4;
    vuint32_t SLOT6_REQM:4;
    vuint32_t SLOT7_REQM:4;
  } B;
} ROUND_ROBIN_ARBITER_RRA_SLT_LO_tag;

typedef union ROUND_ROBIN_ARBITER_RRA_SLT_HI_union_tag { /* RRA Slot HI Register */
  vuint32_t R;
  struct {
    vuint32_t SLOT8_REQM:4;
    vuint32_t SLOT9_REQM:4;
    vuint32_t SLOT10_REQM:4;
    vuint32_t SLOT11_REQM:4;
    vuint32_t _unused_16:16;
  } B;
} ROUND_ROBIN_ARBITER_RRA_SLT_HI_tag;

typedef union ROUND_ROBIN_ARBITER_RRA_SLT_PRI_union_tag { /* RRA Slot Priority Register */
  vuint32_t R;
  struct {
    vuint32_t RRA_RGION1_FIRST_SLOT:4;
    vuint32_t RRA_RGION2_FIRST_SLOT:4;
    vuint32_t RRA_RGION3_FIRST_SLOT:4;
    vuint32_t _unused_12:20;
  } B;
} ROUND_ROBIN_ARBITER_RRA_SLT_PRI_tag;

typedef union ROUND_ROBIN_ARBITER_RRA_WEIGHT_LO_union_tag { /* RRA Weight LO Register */
  vuint32_t R;
  struct {
    vuint32_t DDMA_RD_WEIGHT:3;
    vuint32_t _unused_3:1;
    vuint32_t EDAP_RD_WEIGHT:3;
    vuint32_t _unused_7:1;
    vuint32_t QMAN_WEIGHT:3;
    vuint32_t _unused_11:21;
  } B;
} ROUND_ROBIN_ARBITER_RRA_WEIGHT_LO_tag;

typedef union ROUND_ROBIN_ARBITER_RRA_WEIGHT_HI_union_tag { /* RRA Weight HI Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t DDMA_WR_WEIGHT:3;
    vuint32_t _unused_7:5;
    vuint32_t EDAP_WR_WEIGHT:3;
    vuint32_t _unused_15:17;
  } B;
} ROUND_ROBIN_ARBITER_RRA_WEIGHT_HI_tag;

struct ROUND_ROBIN_ARBITER_tag {
  ROUND_ROBIN_ARBITER_RRA_SLT_LO_tag RRA_SLT_LO; /* RRA Slot LO Register */
  ROUND_ROBIN_ARBITER_RRA_SLT_HI_tag RRA_SLT_HI; /* RRA Slot HI Register */
  ROUND_ROBIN_ARBITER_RRA_SLT_PRI_tag RRA_SLT_PRI; /* RRA Slot Priority Register */
  ROUND_ROBIN_ARBITER_RRA_WEIGHT_LO_tag RRA_WEIGHT_LO; /* RRA Weight LO Register */
  ROUND_ROBIN_ARBITER_RRA_WEIGHT_HI_tag RRA_WEIGHT_HI; /* RRA Weight HI Register */
};


/* ============================================================================
   =============================== Module: RTT ================================
   ============================================================================ */

typedef union RTT_MODE_union_tag {     /* MODE Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:6;
    vuint32_t WR_EN:1;
    vuint32_t WR_RESET:1;
    vuint32_t _unused_8:2;
    vuint32_t BR_ALL:1;
    vuint32_t _unused_11:21;
  } B;
} RTT_MODE_tag;

struct RTT_tag {
  RTT_MODE_tag MODE;                   /* MODE Register */
};


/* ============================================================================
   =============================== Module: RTUE_NIC_D =========================
   ============================================================================ */

typedef union RTUE_NIC_D_REMAP_union_tag { /* Remap */
  vuint32_t R;
  struct {
    vuint32_t REMAP:2;                 /* Memory Map Configuration */
    vuint32_t _unused_2:30;
  } B;
} RTUE_NIC_D_REMAP_tag;

struct RTUE_NIC_D_tag {
  RTUE_NIC_D_REMAP_tag REMAP;          /* Remap */
};


/* ============================================================================
   =============================== Module: RTUF_NIC_D =========================
   ============================================================================ */

typedef union RTUF_NIC_D_REMAP_union_tag { /* Remap */
  vuint32_t R;
  struct {
    vuint32_t REMAP:2;                 /* Memory Map Configuration */
    vuint32_t _unused_2:30;
  } B;
} RTUF_NIC_D_REMAP_tag;

struct RTUF_NIC_D_tag {
  RTUF_NIC_D_REMAP_tag REMAP;          /* Remap */
};


/* ============================================================================
   =============================== Module: RTUM_NIC_D =========================
   ============================================================================ */

typedef union RTUM_NIC_D_REMAP_union_tag { /* Remap */
  vuint32_t R;
  struct {
    vuint32_t REMAP:2;                 /* Memory Map Configuration */
    vuint32_t _unused_2:30;
  } B;
} RTUM_NIC_D_REMAP_tag;

struct RTUM_NIC_D_tag {
  RTUM_NIC_D_REMAP_tag REMAP;          /* Remap */
};


/* ============================================================================
   =============================== Module: RTUP_NIC_B =========================
   ============================================================================ */

typedef union RTUP_NIC_B_REMAP_union_tag { /* Remap */
  vuint32_t R;
  struct {
    vuint32_t REMAP:2;                 /* Memory Map Configuration */
    vuint32_t _unused_2:30;
  } B;
} RTUP_NIC_B_REMAP_tag;

struct RTUP_NIC_B_tag {
  RTUP_NIC_B_REMAP_tag REMAP;          /* Remap */
};


/* ============================================================================
   =============================== Module: RTU_EIM ============================
   ============================================================================ */

typedef union RTU_EIM_EIMCR_union_tag { /* Error Injection Module Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t GEIEN:1;                 /* Global Error Injection Enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} RTU_EIM_EIMCR_tag;

typedef union RTU_EIM_EICHEN_union_tag { /* Error Injection Channel Enable register */
  vuint32_t R;
  struct {
    vuint32_t EICH31EN:1;              /* Error Injection Channel 31 Enable */
    vuint32_t EICH30EN:1;              /* Error Injection Channel 30 Enable */
    vuint32_t EICH29EN:1;              /* Error Injection Channel 29 Enable */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t EICH4EN:1;               /* Error Injection Channel 4 Enable */
    vuint32_t EICH3EN:1;               /* Error Injection Channel 3 Enable */
    vuint32_t EICH2EN:1;               /* Error Injection Channel 2 Enable */
    vuint32_t EICH1EN:1;               /* Error Injection Channel 1 Enable */
    vuint32_t EICH0EN:1;               /* Error Injection Channel 0 Enable */
  } B;
} RTU_EIM_EICHEN_tag;

typedef union RTU_EIM_EICHD0_WORD1_union_tag { /* Error Injection Channel Descriptor 0, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:12;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_12:20;
  } B;
} RTU_EIM_EICHD0_WORD1_tag;

typedef union RTU_EIM_EICHD0_WORD2_union_tag { /* Error Injection Channel Descriptor 0, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} RTU_EIM_EICHD0_WORD2_tag;

typedef union RTU_EIM_EICHD1_WORD1_union_tag { /* Error Injection Channel Descriptor 1, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:22;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_22:10;
  } B;
} RTU_EIM_EICHD1_WORD1_tag;

typedef union RTU_EIM_EICHD1_WORD2_union_tag { /* Error Injection Channel Descriptor 1, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} RTU_EIM_EICHD1_WORD2_tag;

typedef union RTU_EIM_EICHD2_WORD1_union_tag { /* Error Injection Channel Descriptor 2, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:16;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_16:16;
  } B;
} RTU_EIM_EICHD2_WORD1_tag;

typedef union RTU_EIM_EICHD3_WORD1_union_tag { /* Error Injection Channel Descriptor 3, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:26;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_26:6;
  } B;
} RTU_EIM_EICHD3_WORD1_tag;

typedef union RTU_EIM_EICHD4_WORD1_union_tag { /* Error Injection Channel Descriptor 4, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:23;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_23:9;
  } B;
} RTU_EIM_EICHD4_WORD1_tag;

typedef union RTU_EIM_EICHD29_WORD1_union_tag { /* Error Injection Channel Descriptor 29, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:12;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_12:20;
  } B;
} RTU_EIM_EICHD29_WORD1_tag;

typedef union RTU_EIM_EICHD30_WORD1_union_tag { /* Error Injection Channel Descriptor 30, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:22;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_22:10;
  } B;
} RTU_EIM_EICHD30_WORD1_tag;

typedef union RTU_EIM_EICHD30_WORD2_union_tag { /* Error Injection Channel Descriptor 30, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} RTU_EIM_EICHD30_WORD2_tag;

typedef union RTU_EIM_EICHD31_WORD1_union_tag { /* Error Injection Channel Descriptor 31, Word1 */
  vuint32_t R;
  struct {
    vuint32_t B0_3DATA_MASK:22;        /* Data Mask Bytes 0-3 */
    vuint32_t _unused_22:10;
  } B;
} RTU_EIM_EICHD31_WORD1_tag;

typedef union RTU_EIM_EICHD31_WORD2_union_tag { /* Error Injection Channel Descriptor 31, Word2 */
  vuint32_t R;
  struct {
    vuint32_t B4_7DATA_MASK:32;        /* Data Mask Bytes 4-7 */
  } B;
} RTU_EIM_EICHD31_WORD2_tag;

struct RTU_EIM_tag {
  RTU_EIM_EIMCR_tag EIMCR;             /* Error Injection Module Configuration Register */
  RTU_EIM_EICHEN_tag EICHEN;           /* Error Injection Channel Enable register */
  uint8_t RTU_EIM_reserved0[252];
  RTU_EIM_EICHD0_WORD1_tag EICHD0_WORD1; /* Error Injection Channel Descriptor 0, Word1 */
  RTU_EIM_EICHD0_WORD2_tag EICHD0_WORD2; /* Error Injection Channel Descriptor 0, Word2 */
  uint8_t RTU_EIM_reserved1[56];
  RTU_EIM_EICHD1_WORD1_tag EICHD1_WORD1; /* Error Injection Channel Descriptor 1, Word1 */
  RTU_EIM_EICHD1_WORD2_tag EICHD1_WORD2; /* Error Injection Channel Descriptor 1, Word2 */
  uint8_t RTU_EIM_reserved2[56];
  RTU_EIM_EICHD2_WORD1_tag EICHD2_WORD1; /* Error Injection Channel Descriptor 2, Word1 */
  uint8_t RTU_EIM_reserved3[60];
  RTU_EIM_EICHD3_WORD1_tag EICHD3_WORD1; /* Error Injection Channel Descriptor 3, Word1 */
  uint8_t RTU_EIM_reserved4[60];
  RTU_EIM_EICHD4_WORD1_tag EICHD4_WORD1; /* Error Injection Channel Descriptor 4, Word1 */
  uint8_t RTU_EIM_reserved5[1596];
  RTU_EIM_EICHD29_WORD1_tag EICHD29_WORD1; /* Error Injection Channel Descriptor 29, Word1 */
  uint8_t RTU_EIM_reserved6[60];
  RTU_EIM_EICHD30_WORD1_tag EICHD30_WORD1; /* Error Injection Channel Descriptor 30, Word1 */
  RTU_EIM_EICHD30_WORD2_tag EICHD30_WORD2; /* Error Injection Channel Descriptor 30, Word2 */
  uint8_t RTU_EIM_reserved7[56];
  RTU_EIM_EICHD31_WORD1_tag EICHD31_WORD1; /* Error Injection Channel Descriptor 31, Word1 */
  RTU_EIM_EICHD31_WORD2_tag EICHD31_WORD2; /* Error Injection Channel Descriptor 31, Word2 */
};


/* ============================================================================
   =============================== Module: RTU_GPR ============================
   ============================================================================ */

typedef union RTU_GPR_CFG_CORE_union_tag { /* Core Configuration */
  vuint32_t R;
  struct {
    vuint32_t SPLT_LCK:1;              /* Split Lock */
    vuint32_t TCM_BT:1;                /* TCM Boot */
    vuint32_t THUMB:1;                 /* THUMB Instruction Set */
    vuint32_t CAINV:1;                 /* Disable Cache Invalidation */
    vuint32_t INITREG:1;               /* Initialize Registers */
    vuint32_t _unused_5:3;
    vuint32_t PMC_DIS:1;               /* PMC Test Enable */
    vuint32_t _unused_9:23;
  } B;
} RTU_GPR_CFG_CORE_tag;

typedef union RTU_GPR_CFG_BUPERR_DIS0_union_tag { /* Bus Parity Error Disable Group0 */
  vuint32_t R;
  struct {
    vuint32_t BUPERR0:1;               /* Bus Parity Error Checking Disable Control 0 */
    vuint32_t BUPERR1:1;               /* Bus Parity Error Checking Disable Control 1 */
    vuint32_t BUPERR2:1;               /* Bus Parity Error Checking Disable Control 2 */
    vuint32_t BUPERR3:1;               /* Bus Parity Error Checking Disable Control 3 */
    vuint32_t BUPERR4:1;               /* Bus Parity Error Checking Disable Control 4 */
    vuint32_t BUPERR5:1;               /* Bus Parity Error Checking Disable Control 5 */
    vuint32_t BUPERR6:1;               /* Bus Parity Error Checking Disable Control 6 */
    vuint32_t BUPERR7:1;               /* Bus Parity Error Checking Disable Control 7 */
    vuint32_t BUPERR8:1;               /* Bus Parity Error Checking Disable Control 8 */
    vuint32_t BUPERR9:1;               /* Bus Parity Error Checking Disable Control 9 */
    vuint32_t BUPERR10:1;              /* Bus Parity Error Checking Disable Control 10 */
    vuint32_t BUPERR11:1;              /* Bus Parity Error Checking Disable Control 11 */
    vuint32_t BUPERR12:1;              /* Bus Parity Error Checking Disable Control 12 */
    vuint32_t BUPERR13:1;              /* Bus Parity Error Checking Disable Control 13 */
    vuint32_t BUPERR14:1;              /* Bus Parity Error Checking Disable Control 14 */
    vuint32_t BUPERR15:1;              /* Bus Parity Error Checking Disable Control 15 */
    vuint32_t BUPERR16:1;              /* Bus Parity Error Checking Disable Control 16 */
    vuint32_t BUPERR17:1;              /* Bus Parity Error Checking Disable Control 17 */
    vuint32_t BUPERR18:1;              /* Bus Parity Error Checking Disable Control 18 */
    vuint32_t BUPERR19:1;              /* Bus Parity Error Checking Disable Control 19 */
    vuint32_t BUPERR20:1;              /* Bus Parity Error Checking Disable Control 20 */
    vuint32_t BUPERR21:1;              /* Bus Parity Error Checking Disable Control 21 */
    vuint32_t BUPERR22:1;              /* Bus Parity Error Checking Disable Control 22 */
    vuint32_t BUPERR23:1;              /* Bus Parity Error Checking Disable Control 23 */
    vuint32_t BUPERR24:1;              /* Bus Parity Error Checking Disable Control 24 */
    vuint32_t BUPERR25:1;              /* Bus Parity Error Checking Disable Control 25 */
    vuint32_t _unused_26:6;
  } B;
} RTU_GPR_CFG_BUPERR_DIS0_tag;

typedef union RTU_GPR_CFG_BUPERR_DIS1_union_tag { /* Bus Parity Error Disable Group0 */
  vuint32_t R;
  struct {
    vuint32_t BUPERR0:1;               /* Bus Parity Error Checking Disable Control 0 */
    vuint32_t BUPERR1:1;               /* Bus Parity Error Checking Disable Control 1 */
    vuint32_t BUPERR2:1;               /* Bus Parity Error Checking Disable Control 2 */
    vuint32_t BUPERR3:1;               /* Bus Parity Error Checking Disable Control 3 */
    vuint32_t BUPERR4:1;               /* Bus Parity Error Checking Disable Control 4 */
    vuint32_t BUPERR5:1;               /* Bus Parity Error Checking Disable Control 5 */
    vuint32_t BUPERR6:1;               /* Bus Parity Error Checking Disable Control 6 */
    vuint32_t BUPERR7:1;               /* Bus Parity Error Checking Disable Control 7 */
    vuint32_t BUPERR8:1;               /* Bus Parity Error Checking Disable Control 8 */
    vuint32_t BUPERR9:1;               /* Bus Parity Error Checking Disable Control 9 */
    vuint32_t BUPERR10:1;              /* Bus Parity Error Checking Disable Control 10 */
    vuint32_t BUPERR11:1;              /* Bus Parity Error Checking Disable Control 11 */
    vuint32_t BUPERR12:1;              /* Bus Parity Error Checking Disable Control 12 */
    vuint32_t BUPERR13:1;              /* Bus Parity Error Checking Disable Control 13 */
    vuint32_t BUPERR14:1;              /* Bus Parity Error Checking Disable Control 14 */
    vuint32_t BUPERR15:1;              /* Bus Parity Error Checking Disable Control 15 */
    vuint32_t BUPERR16:1;              /* Bus Parity Error Checking Disable Control 16 */
    vuint32_t BUPERR17:1;              /* Bus Parity Error Checking Disable Control 17 */
    vuint32_t BUPERR18:1;              /* Bus Parity Error Checking Disable Control 18 */
    vuint32_t BUPERR19:1;              /* Bus Parity Error Checking Disable Control 19 */
    vuint32_t BUPERR20:1;              /* Bus Parity Error Checking Disable Control 20 */
    vuint32_t BUPERR21:1;              /* Bus Parity Error Checking Disable Control 21 */
    vuint32_t BUPERR22:1;              /* Bus Parity Error Checking Disable Control 22 */
    vuint32_t _unused_23:9;
  } B;
} RTU_GPR_CFG_BUPERR_DIS1_tag;

typedef union RTU_GPR_CFG_FD_union_tag { /* Core Fence and Drain Configuration */
  vuint32_t R;
  struct {
    vuint32_t MFDEN0:1;                /* Core0 AXIM Fence and Drain Enable */
    vuint32_t MFDEN1:1;                /* Core1 AXIM Fence and Drain Enable */
    vuint32_t MFDEN2:1;                /* Core2 AXIM Fence and Drain Enable */
    vuint32_t MFDEN3:1;                /* Core3 AXIM Fence and Drain Enable */
    vuint32_t FFDEN0:1;                /* Core0 AXIF Fence and Drain Enable */
    vuint32_t FFDEN1:1;                /* Core1 AXIF Fence and Drain Enable */
    vuint32_t FFDEN2:1;                /* Core2 AXIF Fence and Drain Enable */
    vuint32_t FFDEN3:1;                /* Core3 AXIF Fence and Drain Enable */
    vuint32_t LFDEN0:1;                /* Core0 LLPP Fence and Drain Enable */
    vuint32_t LFDEN1:1;                /* Core1 LLPP Fence and Drain Enable */
    vuint32_t LFDEN2:1;                /* Core2 LLPP Fence and Drain Enable */
    vuint32_t LFDEN3:1;                /* Core3 LLPP Fence and Drain Enable */
    vuint32_t _unused_12:4;
    vuint32_t SFDEN:1;                 /* AXIS Fence Enable */
    vuint32_t _unused_17:11;
    vuint32_t ISOLEN0:1;               /* Isolation Enable - Cortex-R52 Core0 */
    vuint32_t ISOLEN1:1;               /* Isolation Enable - Cortex-R52 Core1 */
    vuint32_t ISOLEN2:1;               /* Isolation Enable - Cortex-R52 Core2 */
    vuint32_t ISOLEN3:1;               /* Isolation Enable - Cortex-R52 Core3 */
  } B;
} RTU_GPR_CFG_FD_tag;

typedef union RTU_GPR_CFG_CNTDV_union_tag { /* Generic Timer Count Divider Control */
  vuint32_t R;
  struct {
    vuint32_t CNTDV:3;                 /* System Counter Divider for Cortex-R52 processor Generic Timer */
    vuint32_t _unused_3:29;
  } B;
} RTU_GPR_CFG_CNTDV_tag;

typedef union RTU_GPR_CFG_HALT_union_tag { /* Core Halt Control */
  vuint32_t R;
  struct {
    vuint32_t HALT0:1;                 /* Drive CPUHALT0 */
    vuint32_t HALT1:1;                 /* Drive CPUHALT1 */
    vuint32_t HALT2:1;                 /* Drive CPUHALT2 */
    vuint32_t HALT3:1;                 /* Drive CPUHALT3 */
    vuint32_t _unused_4:28;
  } B;
} RTU_GPR_CFG_HALT_tag;

typedef union RTU_GPR_STAT_FD_union_tag { /* Core Fence and Drain Status */
  vuint32_t R;
  struct {
    vuint32_t MFDID0:1;                /* Core0 AXIM Fence and Drain Idle Status */
    vuint32_t MFDID1:1;                /* Core1 AXIM Fence and Drain Idle Status */
    vuint32_t MFDID2:1;                /* Core2 AXIM Fence and Drain Idle Status */
    vuint32_t MFDID3:1;                /* Core3 AXIM Fence and Drain Idle Status */
    vuint32_t FFDID0:1;                /* Core0 AXIF Fence and Drain Idle Status */
    vuint32_t FFDID1:1;                /* Core1 AXIF Fence and Drain Idle Status */
    vuint32_t FFDID2:1;                /* Core2 AXIF Fence and Drain Idle Status */
    vuint32_t FFDID3:1;                /* Core3 AXIF Fence and Drain Idle Status */
    vuint32_t LFDID0:1;                /* Core0 LLPP Fence and Drain Idle Status */
    vuint32_t LFDID1:1;                /* Core1 LLPP Fence and Drain Idle Status */
    vuint32_t LFDID2:1;                /* Core2 LLPP Fence and Drain Idle Status */
    vuint32_t LFDID3:1;                /* Core3 LLPP Fence and Drain Idle Status */
    vuint32_t _unused_12:4;
    vuint32_t SFDID:1;                 /* AXIS Fence and Drain Idle Status */
    vuint32_t _unused_17:15;
  } B;
} RTU_GPR_STAT_FD_tag;

typedef union RTU_GPR_STAT_WFI_union_tag { /* Core STANDBYWFI Status */
  vuint32_t R;
  struct {
    vuint32_t WFI0:1;                  /* Standby WFI Core0 */
    vuint32_t WFI1:1;                  /* Standby WFI Core1 */
    vuint32_t WFI2:1;                  /* Standby WFI Core2 */
    vuint32_t WFI3:1;                  /* Standby WFI Core3 */
    vuint32_t _unused_4:28;
  } B;
} RTU_GPR_STAT_WFI_tag;

typedef union RTU_GPR_STAT_WFE_union_tag { /* Core STANDBYWFE Status */
  vuint32_t R;
  struct {
    vuint32_t WFE0:1;                  /* Standby WFE Core0 */
    vuint32_t WFE1:1;                  /* Standby WFE Core1 */
    vuint32_t WFE2:1;                  /* Standby WFE Core2 */
    vuint32_t WFE3:1;                  /* Standby WFE Core3 */
    vuint32_t _unused_4:28;
  } B;
} RTU_GPR_STAT_WFE_tag;

typedef union RTU_GPR_STAT_WRSTREQ_union_tag { /* Core WARMRSTREQ Status */
  vuint32_t R;
  struct {
    vuint32_t WARMRSTREQ0:1;           /* Cortex-R52 Core0 warm core reset request */
    vuint32_t WARMRSTREQ1:1;           /* Cortex-R52 Core1 warm core reset request */
    vuint32_t WARMRSTREQ2:1;           /* Cortex-R52 Core2 warm core reset request */
    vuint32_t WARMRSTREQ3:1;           /* Cortex-R52 Core3 warm core reset request */
    vuint32_t _unused_4:28;
  } B;
} RTU_GPR_STAT_WRSTREQ_tag;

typedef union RTU_GPR_STAT_EVC0_union_tag { /* Core0 External Event Status */
  vuint32_t R;
  struct {
    vuint32_t C0_EV0:1;                /* Core0 Error Event0 */
    vuint32_t C0_EV1:1;                /* Core0 Error Event1 */
    vuint32_t C0_EV2:1;                /* Core0 Error Event2 */
    vuint32_t C0_EV3:1;                /* Core0 Error Event3 */
    vuint32_t C0_EV4:1;                /* Core0 Error Event4 */
    vuint32_t C0_EV5:1;                /* Core0 Error Event5 */
    vuint32_t C0_EV6:1;                /* Core0 Error Event6 */
    vuint32_t C0_EV7:1;                /* Core0 Error Event7 */
    vuint32_t C0_EV8:1;                /* Core0 Error Event8 */
    vuint32_t C0_EV9:1;                /* Core0 Error Event9 */
    vuint32_t C0_EV10:1;               /* Core0 Error Event10 */
    vuint32_t C0_EV11:1;               /* Core0 Error Event11 */
    vuint32_t C0_EV12:1;               /* Core0 Error Event12 */
    vuint32_t C0_EV13:1;               /* Core0 Error Event13 */
    vuint32_t C0_EV14:1;               /* Core0 Error Event14 */
    vuint32_t C0_EV15:1;               /* Core0 Error Event15 */
    vuint32_t C0_EV16:1;               /* Core0 Error Event16 */
    vuint32_t C0_EV17:1;               /* Core0 Error Event17 */
    vuint32_t C0_EV18:1;               /* Core0 Error Event18 */
    vuint32_t C0_EV19:1;               /* Core0 Error Event19 */
    vuint32_t C0_EV20:1;               /* Core0 Error Event20 */
    vuint32_t C0_EV21:1;               /* Core0 Error Event21 */
    vuint32_t C0_EV22:1;               /* Core0 Error Event22 */
    vuint32_t C0_EV23:1;               /* Core0 Error Event23 */
    vuint32_t C0_EV24:1;               /* Core0 Error Event24 */
    vuint32_t C0_EV25:1;               /* Core0 Error Event25 */
    vuint32_t _unused_26:6;
  } B;
} RTU_GPR_STAT_EVC0_tag;

typedef union RTU_GPR_STAT_EVC1_union_tag { /* Core1 External Event Status */
  vuint32_t R;
  struct {
    vuint32_t C1_EV0:1;                /* Core1 Error Event0 */
    vuint32_t C1_EV1:1;                /* Core1 Error Event1 */
    vuint32_t C1_EV2:1;                /* Core1 Error Event2 */
    vuint32_t C1_EV3:1;                /* Core1 Error Event3 */
    vuint32_t C1_EV4:1;                /* Core1 Event4 */
    vuint32_t C1_EV5:1;                /* Core1 Error Event5 */
    vuint32_t C1_EV6:1;                /* Core1 Error Event6 */
    vuint32_t C1_EV7:1;                /* Core1 Error Event7 */
    vuint32_t C1_EV8:1;                /* Core1 Error Event8 */
    vuint32_t C1_EV9:1;                /* Core1 Error Event9 */
    vuint32_t C1_EV10:1;               /* Core1 Error Event10 */
    vuint32_t C1_EV11:1;               /* Core1 Error Event11 */
    vuint32_t C1_EV12:1;               /* Core1 Error Event12 */
    vuint32_t C1_EV13:1;               /* Core1 Error Event13 */
    vuint32_t C1_EV14:1;               /* Core1 Error Event14 */
    vuint32_t C1_EV15:1;               /* Core1 Error Event15 */
    vuint32_t C1_EV16:1;               /* Core1 Error Event16 */
    vuint32_t C1_EV17:1;               /* Core1 Error Event17 */
    vuint32_t C1_EV18:1;               /* Core1 Error Event18 */
    vuint32_t C1_EV19:1;               /* Core1 Error Event19 */
    vuint32_t C1_EV20:1;               /* Core1 Error Event20 */
    vuint32_t C1_EV21:1;               /* Core1 Error Event21 */
    vuint32_t C1_EV22:1;               /* Core1 Error Event22 */
    vuint32_t C1_EV23:1;               /* Core1 Error Event23 */
    vuint32_t C1_EV24:1;               /* Core1 Error Event24 */
    vuint32_t C1_EV25:1;               /* Core1 Error Event25 */
    vuint32_t _unused_26:6;
  } B;
} RTU_GPR_STAT_EVC1_tag;

typedef union RTU_GPR_STAT_EVC2_union_tag { /* Core2 External Event Status */
  vuint32_t R;
  struct {
    vuint32_t C2_EV0:1;                /* Core2 Error Event0 */
    vuint32_t C2_EV1:1;                /* Core2 Error Event1 */
    vuint32_t C2_EV2:1;                /* Core2 Error Event2 */
    vuint32_t C2_EV3:1;                /* Core2 Error Event3 */
    vuint32_t C2_EV4:1;                /* Core2 Event4 */
    vuint32_t C2_EV5:1;                /* Core2 Error Event5 */
    vuint32_t C2_EV6:1;                /* Core2 Error Event6 */
    vuint32_t C2_EV7:1;                /* Core2 Error Event7 */
    vuint32_t C2_EV8:1;                /* Core2 Error Event8 */
    vuint32_t C2_EV9:1;                /* Core2 Error Event9 */
    vuint32_t C2_EV10:1;               /* Core2 Error Event10 */
    vuint32_t C2_EV11:1;               /* Core2 Error Event11 */
    vuint32_t C2_EV12:1;               /* Core2 Error Event12 */
    vuint32_t C2_EV13:1;               /* Core2 Error Event13 */
    vuint32_t C2_EV14:1;               /* Core2 Error Event14 */
    vuint32_t C2_EV15:1;               /* Core2 Error Event15 */
    vuint32_t C2_EV16:1;               /* Core2 Error Event16 */
    vuint32_t C2_EV17:1;               /* Core2 Error Event17 */
    vuint32_t C2_EV18:1;               /* Core2 Error Event18 */
    vuint32_t C2_EV19:1;               /* Core2 Error Event19 */
    vuint32_t C2_EV20:1;               /* Core2 Error Event20 */
    vuint32_t C2_EV21:1;               /* Core2 Error Event21 */
    vuint32_t C2_EV22:1;               /* Core2 Error Event22 */
    vuint32_t C2_EV23:1;               /* Core2 Error Event23 */
    vuint32_t C2_EV24:1;               /* Core2 Error Event24 */
    vuint32_t C2_EV25:1;               /* Core2 Error Event25 */
    vuint32_t _unused_26:6;
  } B;
} RTU_GPR_STAT_EVC2_tag;

typedef union RTU_GPR_STAT_EVC3_union_tag { /* Core3 External Event Status */
  vuint32_t R;
  struct {
    vuint32_t C3_EV0:1;                /* Core3 Error Event0 */
    vuint32_t C3_EV1:1;                /* Core3 Error Event1 */
    vuint32_t C3_EV2:1;                /* Core3 Error Event2 */
    vuint32_t C3_EV3:1;                /* Core3 Error Event3 */
    vuint32_t C3_EV4:1;                /* Core3 Error Event4 */
    vuint32_t C3_EV5:1;                /* Core3 Error Event5 */
    vuint32_t C3_EV6:1;                /* Core3 Error Event6 */
    vuint32_t C3_EV7:1;                /* Core3 Error Event7 */
    vuint32_t C3_EV8:1;                /* Core3 Error Event8 */
    vuint32_t C3_EV9:1;                /* Core3 Error Event9 */
    vuint32_t C3_EV10:1;               /* Core3 Error Event10 */
    vuint32_t C3_EV11:1;               /* Core3 Error Event11 */
    vuint32_t C3_EV12:1;               /* Core3 Error Event12 */
    vuint32_t C3_EV13:1;               /* Core3 Error Event13 */
    vuint32_t C3_EV14:1;               /* Core3 Error Event14 */
    vuint32_t C3_EV15:1;               /* Core3 Error Event15 */
    vuint32_t C3_EV16:1;               /* Core3 Error Event16 */
    vuint32_t C3_EV17:1;               /* Core3 Error Event17 */
    vuint32_t C3_EV18:1;               /* Core3 Error Event18 */
    vuint32_t C3_EV19:1;               /* Core3 Error Event19 */
    vuint32_t C3_EV20:1;               /* Core3 Error Event20 */
    vuint32_t C3_EV21:1;               /* Core3 Error Event21 */
    vuint32_t C3_EV22:1;               /* Core3 Error Event22 */
    vuint32_t C3_EV23:1;               /* Core3 Error Event23 */
    vuint32_t C3_EV24:1;               /* Core3 Error Event24 */
    vuint32_t C3_EV25:1;               /* Core3 Error Event25 */
    vuint32_t _unused_26:6;
  } B;
} RTU_GPR_STAT_EVC3_tag;

typedef union RTU_GPR_STAT_MERRPM0_union_tag { /* Primary Error Interface Memory Identifier Core0 */
  vuint32_t R;
  struct {
    vuint32_t PRIMEMERRMEM0:15;        /* Primary error reporting interface memory identifier for Cortex-R52 Core0 */
    vuint32_t _unused_15:16;
    vuint32_t PRIMEMERRMEMV0:1;        /* Primary error reporting interface valid for Cortex-R52 Core0 */
  } B;
} RTU_GPR_STAT_MERRPM0_tag;

typedef union RTU_GPR_STAT_MERRPX0_union_tag { /* Primary Error Interface Index Core0 */
  vuint32_t R;
  struct {
    vuint32_t PRIMEMERRMEIDX0:25;      /* Primary error reporting interface index for Cortex-R52 Core0 */
    vuint32_t _unused_25:7;
  } B;
} RTU_GPR_STAT_MERRPX0_tag;

typedef union RTU_GPR_STAT_MERRSM0_union_tag { /* Secondary Error Interface Memory Identifier Core0 */
  vuint32_t R;
  struct {
    vuint32_t SECMEMERRMEM0:15;        /* Secondary error reporting interface memory identifier for Cortex-R52 Core0 */
    vuint32_t _unused_15:16;
    vuint32_t SECMEMERRMEMV0:1;        /* Secondary error reporting interface valid for Cortex-R52 Core0 */
  } B;
} RTU_GPR_STAT_MERRSM0_tag;

typedef union RTU_GPR_STAT_MERRSX0_union_tag { /* Secondary Error Interface Index Core0 */
  vuint32_t R;
  struct {
    vuint32_t SECMEMERRMEIDX0:25;      /* Secondary error reporting interface index for Cortex-R52 Core0 */
    vuint32_t _unused_25:7;
  } B;
} RTU_GPR_STAT_MERRSX0_tag;

typedef union RTU_GPR_STAT_MERRPM1_union_tag { /* Primary Error Interface Memory Identifier Core1 */
  vuint32_t R;
  struct {
    vuint32_t PRIMEMERRMEM1:15;        /* Primary error reporting interface memory identifier for Cortex-R52 Core1 */
    vuint32_t _unused_15:16;
    vuint32_t PRIMEMERRMEMV1:1;        /* Primary error reporting interface valid for Cortex-R52 Core1 */
  } B;
} RTU_GPR_STAT_MERRPM1_tag;

typedef union RTU_GPR_STAT_MERRPX1_union_tag { /* Primary Error Interface Index Core1 */
  vuint32_t R;
  struct {
    vuint32_t PRIMEMERRMEIDX1:25;      /* Primary error reporting interface index for Cortex-R52 Core1 */
    vuint32_t _unused_25:7;
  } B;
} RTU_GPR_STAT_MERRPX1_tag;

typedef union RTU_GPR_STAT_MERRSM1_union_tag { /* Secondary Error Interface Memory Identifier Core1 */
  vuint32_t R;
  struct {
    vuint32_t SECMEMERRMEM1:15;        /* Secondary error reporting interface memory identifier for Cortex-R52 Core1 */
    vuint32_t _unused_15:16;
    vuint32_t SECMEMERRMEMV1:1;        /* Secondary error reporting interface valid for Cortex-R52 Core1 */
  } B;
} RTU_GPR_STAT_MERRSM1_tag;

typedef union RTU_GPR_STAT_MERRSX1_union_tag { /* Secondary Error Interface Index Core1 */
  vuint32_t R;
  struct {
    vuint32_t SECMEMERRMEIDX1:25;      /* Secondary error reporting interface index for Cortex-R52 Core1 */
    vuint32_t _unused_25:7;
  } B;
} RTU_GPR_STAT_MERRSX1_tag;

typedef union RTU_GPR_STAT_MERRPM2_union_tag { /* Primary Error Interface Memory Identifier Core2 */
  vuint32_t R;
  struct {
    vuint32_t PRIMEMERRMEM2:15;        /* Primary error reporting interface memory identifier for Cortex-R52 Core2 */
    vuint32_t _unused_15:16;
    vuint32_t PRIMEMERRMEMV2:1;        /* Primary error reporting interface valid for Cortex-R52 Core2 */
  } B;
} RTU_GPR_STAT_MERRPM2_tag;

typedef union RTU_GPR_STAT_MERRPX2_union_tag { /* Primary Error Interface Index Core2 */
  vuint32_t R;
  struct {
    vuint32_t PRIMEMERRMEIDX2:25;      /* Primary error reporting interface index for Cortex-R52 Core2 */
    vuint32_t _unused_25:7;
  } B;
} RTU_GPR_STAT_MERRPX2_tag;

typedef union RTU_GPR_STAT_MERRSM2_union_tag { /* Secondary Error Interface Memory Identifier Core2 */
  vuint32_t R;
  struct {
    vuint32_t SECMEMERRMEM2:15;        /* Secondary error reporting interface memory identifier for Cortex-R52 Core2 */
    vuint32_t _unused_15:16;
    vuint32_t SECMEMERRMEMV2:1;        /* Secondary error reporting interface valid for Cortex-R52 Core2 */
  } B;
} RTU_GPR_STAT_MERRSM2_tag;

typedef union RTU_GPR_STAT_MERRSX2_union_tag { /* Secondary Error Interface Index Core2 */
  vuint32_t R;
  struct {
    vuint32_t SECMEMERRMEIDX2:25;      /* Secondary error reporting interface index for Cortex-R52 Core2 */
    vuint32_t _unused_25:7;
  } B;
} RTU_GPR_STAT_MERRSX2_tag;

typedef union RTU_GPR_STAT_MERRPM3_union_tag { /* Primary Error Interface Memory Identifier Core3 */
  vuint32_t R;
  struct {
    vuint32_t PRIMEMERRMEM3:15;        /* Primary error reporting interface memory identifier for Cortex-R52 Core3 */
    vuint32_t _unused_15:16;
    vuint32_t PRIMEMERRMEMV3:1;        /* Primary error reporting interface valid for Cortex-R52 Core3 */
  } B;
} RTU_GPR_STAT_MERRPM3_tag;

typedef union RTU_GPR_STAT_MERRPX3_union_tag { /* Primary Error Interface Index Core3 */
  vuint32_t R;
  struct {
    vuint32_t PRIMEMERRMEIDX3:25;      /* Primary error reporting interface index for Cortex-R52 Core3 */
    vuint32_t _unused_25:7;
  } B;
} RTU_GPR_STAT_MERRPX3_tag;

typedef union RTU_GPR_STAT_MERRSM3_union_tag { /* Secondary Error Interface Memory Identifier Core3 */
  vuint32_t R;
  struct {
    vuint32_t SECMEMERRMEM3:15;        /* Secondary error reporting interface memory identifier for Cortex-R52 Core3 */
    vuint32_t _unused_15:16;
    vuint32_t SECMEMERRMEMV3:1;        /* Secondary error reporting interface valid for Cortex-R52 Core3 */
  } B;
} RTU_GPR_STAT_MERRSM3_tag;

typedef union RTU_GPR_STAT_MERRSX3_union_tag { /* Secondary Error Interface Index Core3 */
  vuint32_t R;
  struct {
    vuint32_t SECMEMERRMEIDX3:25;      /* Secondary error reporting interface index for Cortex-R52 Core3 */
    vuint32_t _unused_25:7;
  } B;
} RTU_GPR_STAT_MERRSX3_tag;

typedef union RTU_GPR_STAT_PMU0EV0_union_tag { /* PMU Event Core0 Group0 */
  vuint32_t R;
  struct {
    vuint32_t PMUEVENT0:32;            /* PMU Events 0-31 for Core0 */
  } B;
} RTU_GPR_STAT_PMU0EV0_tag;

typedef union RTU_GPR_STAT_PMU1EV0_union_tag { /* PMU Event Core0 Group1 */
  vuint32_t R;
  struct {
    vuint32_t PMUEVENT0:4;             /* PMU Events 32-35 for Core0 */
    vuint32_t _unused_4:28;
  } B;
} RTU_GPR_STAT_PMU1EV0_tag;

typedef union RTU_GPR_STAT_PMU0EV1_union_tag { /* PMU Event Core1 Group0 */
  vuint32_t R;
  struct {
    vuint32_t PMUEVENT1:32;            /* PMU Events 0-31 for Core1 */
  } B;
} RTU_GPR_STAT_PMU0EV1_tag;

typedef union RTU_GPR_STAT_PMU1EV1_union_tag { /* PMU Event Core1 Group1 */
  vuint32_t R;
  struct {
    vuint32_t PMUEVENT1:4;             /* PMU Events 32-35 for Core1 */
    vuint32_t _unused_4:28;
  } B;
} RTU_GPR_STAT_PMU1EV1_tag;

typedef union RTU_GPR_STAT_PMU0EV2_union_tag { /* PMU Event Core2 Group0 */
  vuint32_t R;
  struct {
    vuint32_t PMUEVENT2:32;            /* PMU Events 0-31 for Core2 */
  } B;
} RTU_GPR_STAT_PMU0EV2_tag;

typedef union RTU_GPR_STAT_PMU1EV2_union_tag { /* PMU Event Core2 Group1 */
  vuint32_t R;
  struct {
    vuint32_t PMUEVENT2:4;             /* PMU Events 32-35 for Core2 */
    vuint32_t _unused_4:28;
  } B;
} RTU_GPR_STAT_PMU1EV2_tag;

typedef union RTU_GPR_STAT_PMU0EV3_union_tag { /* PMU Event Core3 Group0 */
  vuint32_t R;
  struct {
    vuint32_t PMUEVENT3:32;            /* PMU Events 0-31 for Core3 */
  } B;
} RTU_GPR_STAT_PMU0EV3_tag;

typedef union RTU_GPR_STAT_PMU1EV3_union_tag { /* PMU Event Core3 Group1 */
  vuint32_t R;
  struct {
    vuint32_t PMUEVENT3:4;             /* PMU Events 32-35 for Core3 */
    vuint32_t _unused_4:28;
  } B;
} RTU_GPR_STAT_PMU1EV3_tag;

struct RTU_GPR_tag {
  RTU_GPR_CFG_CORE_tag CFG_CORE;       /* Core Configuration */
  RTU_GPR_CFG_BUPERR_DIS0_tag CFG_BUPERR_DIS0; /* Bus Parity Error Disable Group0 */
  RTU_GPR_CFG_BUPERR_DIS1_tag CFG_BUPERR_DIS1; /* Bus Parity Error Disable Group0 */
  RTU_GPR_CFG_FD_tag CFG_FD;           /* Core Fence and Drain Configuration */
  RTU_GPR_CFG_CNTDV_tag CFG_CNTDV;     /* Generic Timer Count Divider Control */
  RTU_GPR_CFG_HALT_tag CFG_HALT;       /* Core Halt Control */
  uint8_t RTU_GPR_reserved0[4];
  RTU_GPR_STAT_FD_tag STAT_FD;         /* Core Fence and Drain Status */
  uint8_t RTU_GPR_reserved1[16];
  RTU_GPR_STAT_WFI_tag STAT_WFI;       /* Core STANDBYWFI Status */
  RTU_GPR_STAT_WFE_tag STAT_WFE;       /* Core STANDBYWFE Status */
  RTU_GPR_STAT_WRSTREQ_tag STAT_WRSTREQ; /* Core WARMRSTREQ Status */
  uint8_t RTU_GPR_reserved2[4];
  RTU_GPR_STAT_EVC0_tag STAT_EVC0;     /* Core0 External Event Status */
  RTU_GPR_STAT_EVC1_tag STAT_EVC1;     /* Core1 External Event Status */
  RTU_GPR_STAT_EVC2_tag STAT_EVC2;     /* Core2 External Event Status */
  RTU_GPR_STAT_EVC3_tag STAT_EVC3;     /* Core3 External Event Status */
  RTU_GPR_STAT_MERRPM0_tag STAT_MERRPM0; /* Primary Error Interface Memory Identifier Core0 */
  RTU_GPR_STAT_MERRPX0_tag STAT_MERRPX0; /* Primary Error Interface Index Core0 */
  RTU_GPR_STAT_MERRSM0_tag STAT_MERRSM0; /* Secondary Error Interface Memory Identifier Core0 */
  RTU_GPR_STAT_MERRSX0_tag STAT_MERRSX0; /* Secondary Error Interface Index Core0 */
  RTU_GPR_STAT_MERRPM1_tag STAT_MERRPM1; /* Primary Error Interface Memory Identifier Core1 */
  RTU_GPR_STAT_MERRPX1_tag STAT_MERRPX1; /* Primary Error Interface Index Core1 */
  RTU_GPR_STAT_MERRSM1_tag STAT_MERRSM1; /* Secondary Error Interface Memory Identifier Core1 */
  RTU_GPR_STAT_MERRSX1_tag STAT_MERRSX1; /* Secondary Error Interface Index Core1 */
  RTU_GPR_STAT_MERRPM2_tag STAT_MERRPM2; /* Primary Error Interface Memory Identifier Core2 */
  RTU_GPR_STAT_MERRPX2_tag STAT_MERRPX2; /* Primary Error Interface Index Core2 */
  RTU_GPR_STAT_MERRSM2_tag STAT_MERRSM2; /* Secondary Error Interface Memory Identifier Core2 */
  RTU_GPR_STAT_MERRSX2_tag STAT_MERRSX2; /* Secondary Error Interface Index Core2 */
  RTU_GPR_STAT_MERRPM3_tag STAT_MERRPM3; /* Primary Error Interface Memory Identifier Core3 */
  RTU_GPR_STAT_MERRPX3_tag STAT_MERRPX3; /* Primary Error Interface Index Core3 */
  RTU_GPR_STAT_MERRSM3_tag STAT_MERRSM3; /* Secondary Error Interface Memory Identifier Core3 */
  RTU_GPR_STAT_MERRSX3_tag STAT_MERRSX3; /* Secondary Error Interface Index Core3 */
  RTU_GPR_STAT_PMU0EV0_tag STAT_PMU0EV0; /* PMU Event Core0 Group0 */
  RTU_GPR_STAT_PMU1EV0_tag STAT_PMU1EV0; /* PMU Event Core0 Group1 */
  RTU_GPR_STAT_PMU0EV1_tag STAT_PMU0EV1; /* PMU Event Core1 Group0 */
  RTU_GPR_STAT_PMU1EV1_tag STAT_PMU1EV1; /* PMU Event Core1 Group1 */
  RTU_GPR_STAT_PMU0EV2_tag STAT_PMU0EV2; /* PMU Event Core2 Group0 */
  RTU_GPR_STAT_PMU1EV2_tag STAT_PMU1EV2; /* PMU Event Core2 Group1 */
  RTU_GPR_STAT_PMU0EV3_tag STAT_PMU0EV3; /* PMU Event Core3 Group0 */
  RTU_GPR_STAT_PMU1EV3_tag STAT_PMU1EV3; /* PMU Event Core3 Group1 */
};


/* ============================================================================
   =============================== Module: RTU_L_VFCCU ========================
   ============================================================================ */

typedef union RTU_L_VFCCU_GDFHID_C0_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID0:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID1:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID2:3;                /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID3:3;                /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID4:3;                /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID5:3;                /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID6:3;                /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID7:3;                /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} RTU_L_VFCCU_GDFHID_C0_tag;

typedef union RTU_L_VFCCU_GDFHID_C1_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID8:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID9:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID10:3;               /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID11:3;               /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID12:3;               /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID13:3;               /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID14:3;               /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID15:3;               /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} RTU_L_VFCCU_GDFHID_C1_tag;

typedef union RTU_L_VFCCU_GFLTPO_C0_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS0:1;                   /* Polarity Selection */
    vuint32_t PS1:1;                   /* Polarity Selection */
    vuint32_t PS2:1;                   /* Polarity Selection */
    vuint32_t PS3:1;                   /* Polarity Selection */
    vuint32_t PS4:1;                   /* Polarity Selection */
    vuint32_t PS5:1;                   /* Polarity Selection */
    vuint32_t PS6:1;                   /* Polarity Selection */
    vuint32_t PS7:1;                   /* Polarity Selection */
    vuint32_t PS8:1;                   /* Polarity Selection */
    vuint32_t PS9:1;                   /* Polarity Selection */
    vuint32_t PS10:1;                  /* Polarity Selection */
    vuint32_t PS11:1;                  /* Polarity Selection */
    vuint32_t PS12:1;                  /* Polarity Selection */
    vuint32_t PS13:1;                  /* Polarity Selection */
    vuint32_t PS14:1;                  /* Polarity Selection */
    vuint32_t PS15:1;                  /* Polarity Selection */
    vuint32_t PS16:1;                  /* Polarity Selection */
    vuint32_t PS17:1;                  /* Polarity Selection */
    vuint32_t PS18:1;                  /* Polarity Selection */
    vuint32_t PS19:1;                  /* Polarity Selection */
    vuint32_t PS20:1;                  /* Polarity Selection */
    vuint32_t PS21:1;                  /* Polarity Selection */
    vuint32_t PS22:1;                  /* Polarity Selection */
    vuint32_t PS23:1;                  /* Polarity Selection */
    vuint32_t PS24:1;                  /* Polarity Selection */
    vuint32_t PS25:1;                  /* Polarity Selection */
    vuint32_t PS26:1;                  /* Polarity Selection */
    vuint32_t PS27:1;                  /* Polarity Selection */
    vuint32_t PS28:1;                  /* Polarity Selection */
    vuint32_t PS29:1;                  /* Polarity Selection */
    vuint32_t PS30:1;                  /* Polarity Selection */
    vuint32_t PS31:1;                  /* Polarity Selection */
  } B;
} RTU_L_VFCCU_GFLTPO_C0_tag;

typedef union RTU_L_VFCCU_GFLTPO_C1_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS32:1;                  /* Polarity Selection */
    vuint32_t PS33:1;                  /* Polarity Selection */
    vuint32_t PS34:1;                  /* Polarity Selection */
    vuint32_t PS35:1;                  /* Polarity Selection */
    vuint32_t PS36:1;                  /* Polarity Selection */
    vuint32_t PS37:1;                  /* Polarity Selection */
    vuint32_t PS38:1;                  /* Polarity Selection */
    vuint32_t PS39:1;                  /* Polarity Selection */
    vuint32_t PS40:1;                  /* Polarity Selection */
    vuint32_t PS41:1;                  /* Polarity Selection */
    vuint32_t PS42:1;                  /* Polarity Selection */
    vuint32_t PS43:1;                  /* Polarity Selection */
    vuint32_t PS44:1;                  /* Polarity Selection */
    vuint32_t PS45:1;                  /* Polarity Selection */
    vuint32_t PS46:1;                  /* Polarity Selection */
    vuint32_t PS47:1;                  /* Polarity Selection */
    vuint32_t PS48:1;                  /* Polarity Selection */
    vuint32_t PS49:1;                  /* Polarity Selection */
    vuint32_t _unused_18:14;
  } B;
} RTU_L_VFCCU_GFLTPO_C1_tag;

typedef union RTU_L_VFCCU_GFLTRC_C0_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW0:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW1:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW2:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW3:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW4:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW5:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW6:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW7:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW8:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW9:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW10:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW11:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW12:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW13:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW14:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW15:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW16:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW17:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW18:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW19:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW20:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW21:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW22:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW23:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW24:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW25:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW26:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW27:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW28:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW29:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW30:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW31:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
  } B;
} RTU_L_VFCCU_GFLTRC_C0_tag;

typedef union RTU_L_VFCCU_GFLTRC_C1_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW32:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW33:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW34:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW35:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW36:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW37:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW38:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW39:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW40:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW41:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW42:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW43:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW44:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW45:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW46:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW47:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW48:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW49:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t _unused_18:14;
  } B;
} RTU_L_VFCCU_GFLTRC_C1_tag;

typedef union RTU_L_VFCCU_GFLTOVDC0_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS0:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS1:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS2:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS3:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS4:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS5:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS6:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS7:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS8:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS9:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS10:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS11:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS12:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS13:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS14:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS15:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS16:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS17:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS18:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS19:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS20:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS21:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS22:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS23:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS24:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS25:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS26:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS27:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS28:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS29:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS30:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS31:1;             /* Fault Overflow Detection Disable */
  } B;
} RTU_L_VFCCU_GFLTOVDC0_tag;

typedef union RTU_L_VFCCU_GFLTOVDC1_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS32:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS33:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS34:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS35:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS36:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS37:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS38:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS39:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS40:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS41:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS42:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS43:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS44:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS45:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS46:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS47:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS48:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS49:1;             /* Fault Overflow Detection Disable */
    vuint32_t _unused_18:14;
  } B;
} RTU_L_VFCCU_GFLTOVDC1_tag;

typedef union RTU_L_VFCCU_GSWFLODC_union_tag { /* Global Software Fault Overflow Detection Disable */
  vuint32_t R;
  struct {
    vuint32_t SWOVF_DIS0:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS1:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS2:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS3:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS4:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS5:1;            /* Software Overflow Disable Detection */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_GSWFLODC_tag;

typedef union RTU_L_VFCCU_GCTRL_union_tag { /* Global Space Control */
  vuint32_t R;
  struct {
    vuint32_t OVF_EN:1;                /* Overflow Enable */
    vuint32_t _unused_1:31;
  } B;
} RTU_L_VFCCU_GCTRL_tag;

typedef union RTU_L_VFCCU_GINTOVFS_union_tag { /* Global DID FSM Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t FLTSERV:1;               /* Fault Serve */
    vuint32_t OVF_DET:1;               /* Overflow Detect */
    vuint32_t _unused_9:7;
    vuint32_t SERV_DID:4;              /* Domain Being Serviced */
    vuint32_t _unused_20:4;
    vuint32_t OVF_DID:4;               /* Overflow DID */
    vuint32_t _unused_28:4;
  } B;
} RTU_L_VFCCU_GINTOVFS_tag;

typedef union RTU_L_VFCCU_GDBGCFG_union_tag { /* Global Debug */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t FRZ:1;                   /* Freeze On First Fault */
    vuint32_t _unused_17:15;
  } B;
} RTU_L_VFCCU_GDBGCFG_tag;

typedef union RTU_L_VFCCU_GDBGSTAT_union_tag { /* Global Debug Status */
  vuint32_t R;
  struct {
    vuint32_t FLTIND:8;                /* Fault Index */
    vuint32_t _unused_8:24;
  } B;
} RTU_L_VFCCU_GDBGSTAT_tag;

typedef union RTU_L_VFCCU_SW_RKN_SWRPTDID_union_tag { /* Software Fault Reported DID */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Software Fault DID */
    vuint32_t _unused_4:28;
  } B;
} RTU_L_VFCCU_SW_RKN_SWRPTDID_tag;

typedef union RTU_L_VFCCU_SW_RKN_SWRKSET_0_union_tag { /* Software Reaction Set */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_SW_RKN_SWRKSET_0_tag;

typedef union RTU_L_VFCCU_SW_RKN_SWRKCLR_0_union_tag { /* Software Reaction Clear */
  vuint32_t R;
  struct {
    vuint32_t RKNCLR0:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR1:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR2:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR3:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR4:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR5:1;               /* Reaction Line Clear */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_SW_RKN_SWRKCLR_0_tag;

typedef struct RTU_L_VFCCU_SW_RKN_struct_tag {
  RTU_L_VFCCU_SW_RKN_SWRPTDID_tag SWRPTDID; /* Software Fault Reported DID */
  RTU_L_VFCCU_SW_RKN_SWRKSET_0_tag SWRKSET_0; /* Software Reaction Set */
  uint8_t SW_RKN_reserved0[4];
  RTU_L_VFCCU_SW_RKN_SWRKCLR_0_tag SWRKCLR_0; /* Software Reaction Clear */
  uint8_t SW_RKN_reserved1[16];
} RTU_L_VFCCU_SW_RKN_tag;

typedef union RTU_L_VFCCU_FHCFG0_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} RTU_L_VFCCU_FHCFG0_tag;

typedef union RTU_L_VFCCU_FHSRVDS0_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} RTU_L_VFCCU_FHSRVDS0_tag;

typedef union RTU_L_VFCCU_FHFLTENC00_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} RTU_L_VFCCU_FHFLTENC00_tag;

typedef union RTU_L_VFCCU_FHFLTENC01_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t _unused_18:14;
  } B;
} RTU_L_VFCCU_FHFLTENC01_tag;

typedef union RTU_L_VFCCU_FHFLTS00_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} RTU_L_VFCCU_FHFLTS00_tag;

typedef union RTU_L_VFCCU_FHFLTS01_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t _unused_18:14;
  } B;
} RTU_L_VFCCU_FHFLTS01_tag;

typedef union RTU_L_VFCCU_FHFLTRKC00_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} RTU_L_VFCCU_FHFLTRKC00_tag;

typedef union RTU_L_VFCCU_FHFLTRKC01_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} RTU_L_VFCCU_FHFLTRKC01_tag;

typedef union RTU_L_VFCCU_FHFLTRKC02_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} RTU_L_VFCCU_FHFLTRKC02_tag;

typedef union RTU_L_VFCCU_FHFLTRKC03_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} RTU_L_VFCCU_FHFLTRKC03_tag;

typedef union RTU_L_VFCCU_FHFLTRKC04_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} RTU_L_VFCCU_FHFLTRKC04_tag;

typedef union RTU_L_VFCCU_FHFLTRKC05_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} RTU_L_VFCCU_FHFLTRKC05_tag;

typedef union RTU_L_VFCCU_FHFLTRKC06_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:24;
  } B;
} RTU_L_VFCCU_FHFLTRKC06_tag;

typedef union RTU_L_VFCCU_FHIMRKC0_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_FHIMRKC0_00_tag;

typedef union RTU_L_VFCCU_FHIMRKC0_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_FHIMRKC0_10_tag;

typedef union RTU_L_VFCCU_FHIMRKC0_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_FHIMRKC0_20_tag;

typedef union RTU_L_VFCCU_FHIMRKC0_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_FHIMRKC0_30_tag;

typedef union RTU_L_VFCCU_FHIMRKC0_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_FHIMRKC0_40_tag;

typedef union RTU_L_VFCCU_FHIMRKC0_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} RTU_L_VFCCU_FHIMRKC0_50_tag;

struct RTU_L_VFCCU_tag {
  RTU_L_VFCCU_GDFHID_C0_tag GDFHID_C0; /* Global DID-FHID Map */
  RTU_L_VFCCU_GDFHID_C1_tag GDFHID_C1; /* Global DID-FHID Map */
  uint8_t RTU_L_VFCCU_reserved0[24];
  RTU_L_VFCCU_GFLTPO_C0_tag GFLTPO_C0; /* Global Fault Polarity */
  RTU_L_VFCCU_GFLTPO_C1_tag GFLTPO_C1; /* Global Fault Polarity */
  uint8_t RTU_L_VFCCU_reserved1[56];
  RTU_L_VFCCU_GFLTRC_C0_tag GFLTRC_C0; /* Global Fault Recovery */
  RTU_L_VFCCU_GFLTRC_C1_tag GFLTRC_C1; /* Global Fault Recovery */
  uint8_t RTU_L_VFCCU_reserved2[56];
  RTU_L_VFCCU_GFLTOVDC0_tag GFLTOVDC0; /* Global Fault Overflow Detection */
  RTU_L_VFCCU_GFLTOVDC1_tag GFLTOVDC1; /* Global Fault Overflow Detection */
  uint8_t RTU_L_VFCCU_reserved3[104];
  RTU_L_VFCCU_GSWFLODC_tag GSWFLODC;   /* Global Software Fault Overflow Detection Disable */
  uint8_t RTU_L_VFCCU_reserved4[28];
  RTU_L_VFCCU_GCTRL_tag GCTRL;         /* Global Space Control */
  RTU_L_VFCCU_GINTOVFS_tag GINTOVFS;   /* Global DID FSM Status */
  uint8_t RTU_L_VFCCU_reserved5[428];
  RTU_L_VFCCU_GDBGCFG_tag GDBGCFG;     /* Global Debug */
  RTU_L_VFCCU_GDBGSTAT_tag GDBGSTAT;   /* Global Debug Status */
  uint8_t RTU_L_VFCCU_reserved6[15636];
  RTU_L_VFCCU_SW_RKN_tag SW_RKN[6];
  uint8_t RTU_L_VFCCU_reserved7[16192];
  RTU_L_VFCCU_FHCFG0_tag FHCFG0;       /* Fault Handler */
  RTU_L_VFCCU_FHSRVDS0_tag FHSRVDS0;   /* Fault Handler Status */
  uint8_t RTU_L_VFCCU_reserved8[8];
  RTU_L_VFCCU_FHFLTENC00_tag FHFLTENC00; /* Fault Enable */
  RTU_L_VFCCU_FHFLTENC01_tag FHFLTENC01; /* Fault Enable */
  uint8_t RTU_L_VFCCU_reserved9[56];
  RTU_L_VFCCU_FHFLTS00_tag FHFLTS00;   /* Fault Status */
  RTU_L_VFCCU_FHFLTS01_tag FHFLTS01;   /* Fault Status */
  uint8_t RTU_L_VFCCU_reserved10[56];
  RTU_L_VFCCU_FHFLTRKC00_tag FHFLTRKC00; /* Fault Reaction Set Configuration */
  RTU_L_VFCCU_FHFLTRKC01_tag FHFLTRKC01; /* Fault Reaction Set Configuration */
  RTU_L_VFCCU_FHFLTRKC02_tag FHFLTRKC02; /* Fault Reaction Set Configuration */
  RTU_L_VFCCU_FHFLTRKC03_tag FHFLTRKC03; /* Fault Reaction Set Configuration */
  RTU_L_VFCCU_FHFLTRKC04_tag FHFLTRKC04; /* Fault Reaction Set Configuration */
  RTU_L_VFCCU_FHFLTRKC05_tag FHFLTRKC05; /* Fault Reaction Set Configuration */
  RTU_L_VFCCU_FHFLTRKC06_tag FHFLTRKC06; /* Fault Reaction Set Configuration */
  uint8_t RTU_L_VFCCU_reserved11[228];
  RTU_L_VFCCU_FHIMRKC0_00_tag FHIMRKC0_00; /* Immediate Reaction Configuration */
  uint8_t RTU_L_VFCCU_reserved12[12];
  RTU_L_VFCCU_FHIMRKC0_10_tag FHIMRKC0_10; /* Immediate Reaction Configuration */
  uint8_t RTU_L_VFCCU_reserved13[12];
  RTU_L_VFCCU_FHIMRKC0_20_tag FHIMRKC0_20; /* Immediate Reaction Configuration */
  uint8_t RTU_L_VFCCU_reserved14[12];
  RTU_L_VFCCU_FHIMRKC0_30_tag FHIMRKC0_30; /* Immediate Reaction Configuration */
  uint8_t RTU_L_VFCCU_reserved15[12];
  RTU_L_VFCCU_FHIMRKC0_40_tag FHIMRKC0_40; /* Immediate Reaction Configuration */
  uint8_t RTU_L_VFCCU_reserved16[12];
  RTU_L_VFCCU_FHIMRKC0_50_tag FHIMRKC0_50; /* Immediate Reaction Configuration */
};


/* ============================================================================
   =============================== Module: RTU_MC_CGM =========================
   ============================================================================ */

typedef union RTU_MC_CGM_PCFS_SDUR_union_tag { /* PCFS Step Duration */
  vuint32_t R;
  struct {
    vuint32_t SDUR:16;                 /* Step duration */
    vuint32_t _unused_16:16;
  } B;
} RTU_MC_CGM_PCFS_SDUR_tag;

typedef union RTU_MC_CGM_PCFS_DIVC10_union_tag { /* PCFS Divider Change 10 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} RTU_MC_CGM_PCFS_DIVC10_tag;

typedef union RTU_MC_CGM_PCFS_DIVE10_union_tag { /* PCFS Divider End 10 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} RTU_MC_CGM_PCFS_DIVE10_tag;

typedef union RTU_MC_CGM_PCFS_DIVS10_union_tag { /* PCFS Divider Start 10 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} RTU_MC_CGM_PCFS_DIVS10_tag;

typedef union RTU_MC_CGM_PCFS_DIVC11_union_tag { /* PCFS Divider Change 11 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} RTU_MC_CGM_PCFS_DIVC11_tag;

typedef union RTU_MC_CGM_PCFS_DIVE11_union_tag { /* PCFS Divider End 11 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} RTU_MC_CGM_PCFS_DIVE11_tag;

typedef union RTU_MC_CGM_PCFS_DIVS11_union_tag { /* PCFS Divider Start 11 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} RTU_MC_CGM_PCFS_DIVS11_tag;

typedef union RTU_MC_CGM_PCFS_DIVC12_union_tag { /* PCFS Divider Change 12 Register */
  vuint32_t R;
  struct {
    vuint32_t RATE:8;                  /* Divider change rate */
    vuint32_t _unused_8:8;
    vuint32_t INIT:16;                 /* Divider change initial value */
  } B;
} RTU_MC_CGM_PCFS_DIVC12_tag;

typedef union RTU_MC_CGM_PCFS_DIVE12_union_tag { /* PCFS Divider End 12 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVE:20;                 /* Divider end value */
    vuint32_t _unused_20:12;
  } B;
} RTU_MC_CGM_PCFS_DIVE12_tag;

typedef union RTU_MC_CGM_PCFS_DIVS12_union_tag { /* PCFS Divider Start 12 Register */
  vuint32_t R;
  struct {
    vuint32_t DIVS:20;                 /* Divider start value */
    vuint32_t _unused_20:12;
  } B;
} RTU_MC_CGM_PCFS_DIVS12_tag;

typedef union RTU_MC_CGM_MUX_0_CSC_union_tag { /* Clock Mux 0 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} RTU_MC_CGM_MUX_0_CSC_tag;

typedef union RTU_MC_CGM_MUX_0_CSS_union_tag { /* Clock Mux 0 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t RAMPUP:1;                /* PCFS ramp-up */
    vuint32_t RAMPDOWN:1;              /* PCFS ramp-down */
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} RTU_MC_CGM_MUX_0_CSS_tag;

typedef union RTU_MC_CGM_MUX_0_DC_0_union_tag { /* Clock Mux 0 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} RTU_MC_CGM_MUX_0_DC_0_tag;

typedef union RTU_MC_CGM_MUX_0_DIV_UPD_STAT_union_tag { /* Clock Mux 0 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 0 */
    vuint32_t _unused_1:31;
  } B;
} RTU_MC_CGM_MUX_0_DIV_UPD_STAT_tag;

typedef union RTU_MC_CGM_MUX_1_CSC_union_tag { /* Clock Mux 1 Select Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:20;
    vuint32_t SELCTL:4;                /* Clock source selection control */
    vuint32_t _unused_28:4;
  } B;
} RTU_MC_CGM_MUX_1_CSC_tag;

typedef union RTU_MC_CGM_MUX_1_CSS_union_tag { /* Clock Mux 1 Select Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;
    vuint32_t CLK_SW:1;                /* Clock switch */
    vuint32_t SAFE_SW:1;               /* Safe clock request */
    vuint32_t _unused_4:12;
    vuint32_t SWIP:1;                  /* Switch in progress */
    vuint32_t SWTRG:3;                 /* Switch trigger cause */
    vuint32_t _unused_20:4;
    vuint32_t SELSTAT:4;               /* Clock source selection status */
    vuint32_t _unused_28:4;
  } B;
} RTU_MC_CGM_MUX_1_CSS_tag;

typedef union RTU_MC_CGM_MUX_1_DC_0_union_tag { /* Clock Mux 1 Divider 0 Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t DIV:8;                   /* Division value */
    vuint32_t _unused_24:7;
    vuint32_t DE:1;                    /* Divider enable */
  } B;
} RTU_MC_CGM_MUX_1_DC_0_tag;

typedef union RTU_MC_CGM_MUX_1_DIV_UPD_STAT_union_tag { /* Clock Mux 1 Divider Update Status Register */
  vuint32_t R;
  struct {
    vuint32_t DIV_STAT:1;              /* Divider status for clock mux 1 */
    vuint32_t _unused_1:31;
  } B;
} RTU_MC_CGM_MUX_1_DIV_UPD_STAT_tag;

struct RTU_MC_CGM_tag {
  RTU_MC_CGM_PCFS_SDUR_tag PCFS_SDUR;  /* PCFS Step Duration */
  uint8_t RTU_MC_CGM_reserved0[108];
  RTU_MC_CGM_PCFS_DIVC10_tag PCFS_DIVC10; /* PCFS Divider Change 10 Register */
  RTU_MC_CGM_PCFS_DIVE10_tag PCFS_DIVE10; /* PCFS Divider End 10 Register */
  RTU_MC_CGM_PCFS_DIVS10_tag PCFS_DIVS10; /* PCFS Divider Start 10 Register */
  RTU_MC_CGM_PCFS_DIVC11_tag PCFS_DIVC11; /* PCFS Divider Change 11 Register */
  RTU_MC_CGM_PCFS_DIVE11_tag PCFS_DIVE11; /* PCFS Divider End 11 Register */
  RTU_MC_CGM_PCFS_DIVS11_tag PCFS_DIVS11; /* PCFS Divider Start 11 Register */
  RTU_MC_CGM_PCFS_DIVC12_tag PCFS_DIVC12; /* PCFS Divider Change 12 Register */
  RTU_MC_CGM_PCFS_DIVE12_tag PCFS_DIVE12; /* PCFS Divider End 12 Register */
  RTU_MC_CGM_PCFS_DIVS12_tag PCFS_DIVS12; /* PCFS Divider Start 12 Register */
  uint8_t RTU_MC_CGM_reserved1[620];
  RTU_MC_CGM_MUX_0_CSC_tag MUX_0_CSC;  /* Clock Mux 0 Select Control Register */
  RTU_MC_CGM_MUX_0_CSS_tag MUX_0_CSS;  /* Clock Mux 0 Select Status Register */
  RTU_MC_CGM_MUX_0_DC_0_tag MUX_0_DC_0; /* Clock Mux 0 Divider 0 Control Register */
  uint8_t RTU_MC_CGM_reserved2[48];
  RTU_MC_CGM_MUX_0_DIV_UPD_STAT_tag MUX_0_DIV_UPD_STAT; /* Clock Mux 0 Divider Update Status Register */
  RTU_MC_CGM_MUX_1_CSC_tag MUX_1_CSC;  /* Clock Mux 1 Select Control Register */
  RTU_MC_CGM_MUX_1_CSS_tag MUX_1_CSS;  /* Clock Mux 1 Select Status Register */
  RTU_MC_CGM_MUX_1_DC_0_tag MUX_1_DC_0; /* Clock Mux 1 Divider 0 Control Register */
  uint8_t RTU_MC_CGM_reserved3[48];
  RTU_MC_CGM_MUX_1_DIV_UPD_STAT_tag MUX_1_DIV_UPD_STAT; /* Clock Mux 1 Divider Update Status Register */
};


/* ============================================================================
   =============================== Module: RTU_MRU ============================
   ============================================================================ */

typedef union RTU_MRU_CHXCONFIG_CH_CFG0_union_tag { /* Channel (x) Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t CHE:1;                   /* Channel Enable */
    vuint32_t CHR:1;                   /* Channel Reset */
    vuint32_t IE:1;                    /* Interrupt Enable */
    vuint32_t _unused_3:13;
    vuint32_t MBE0:1;                  /* Mailbox Enable */
    vuint32_t MBE1:1;                  /* Mailbox Enable */
    vuint32_t _unused_18:14;
  } B;
} RTU_MRU_CHXCONFIG_CH_CFG0_tag;

typedef union RTU_MRU_CHXCONFIG_CH_CFG1_union_tag { /* Channel (x) Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MBIC0:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC1:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t _unused_18:14;
  } B;
} RTU_MRU_CHXCONFIG_CH_CFG1_tag;

typedef union RTU_MRU_CHXCONFIG_CH_MBSTAT_union_tag { /* Channel (x) Mailbox Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MBS0:1;                  /* Mailbox Status */
    vuint32_t MBS1:1;                  /* Mailbox Status */
    vuint32_t _unused_18:14;
  } B;
} RTU_MRU_CHXCONFIG_CH_MBSTAT_tag;

typedef struct RTU_MRU_CHXCONFIG_struct_tag {
  RTU_MRU_CHXCONFIG_CH_CFG0_tag CH_CFG0; /* Channel (x) Configuration 0 */
  RTU_MRU_CHXCONFIG_CH_CFG1_tag CH_CFG1; /* Channel (x) Configuration 1 */
  RTU_MRU_CHXCONFIG_CH_MBSTAT_tag CH_MBSTAT; /* Channel (x) Mailbox Status */
  uint8_t CHXCONFIG_reserved0[4];
} RTU_MRU_CHXCONFIG_tag;

typedef union RTU_MRU_NOTIFY0_union_tag { /* Notification 0 Status */
  vuint32_t R;
  struct {
    vuint32_t CH1_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH2_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH3_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH4_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH5_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH6_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH7_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH8_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH9_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH10_IS0:1;              /* Interrupt 0 Status */
    vuint32_t CH11_IS0:1;              /* Interrupt 0 Status */
    vuint32_t CH12_IS0:1;              /* Interrupt 0 Status */
    vuint32_t _unused_12:20;
  } B;
} RTU_MRU_NOTIFY0_tag;

typedef union RTU_MRU_NOTIFY1_union_tag { /* Notification 1 Status */
  vuint32_t R;
  struct {
    vuint32_t CH1_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH2_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH3_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH4_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH5_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH6_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH7_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH8_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH9_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH10_IS1:1;              /* Interrupt 1 Status */
    vuint32_t CH11_IS1:1;              /* Interrupt 1 Status */
    vuint32_t CH12_IS1:1;              /* Interrupt 1 Status */
    vuint32_t _unused_12:20;
  } B;
} RTU_MRU_NOTIFY1_tag;

typedef union RTU_MRU_CH1_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH1_MB0_tag;

typedef union RTU_MRU_CH1_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH1_MB1_tag;

typedef union RTU_MRU_CH2_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH2_MB0_tag;

typedef union RTU_MRU_CH2_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH2_MB1_tag;

typedef union RTU_MRU_CH3_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH3_MB0_tag;

typedef union RTU_MRU_CH3_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH3_MB1_tag;

typedef union RTU_MRU_CH4_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH4_MB0_tag;

typedef union RTU_MRU_CH4_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH4_MB1_tag;

typedef union RTU_MRU_CH5_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH5_MB0_tag;

typedef union RTU_MRU_CH5_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH5_MB1_tag;

typedef union RTU_MRU_CH6_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH6_MB0_tag;

typedef union RTU_MRU_CH6_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH6_MB1_tag;

typedef union RTU_MRU_CH7_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH7_MB0_tag;

typedef union RTU_MRU_CH7_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH7_MB1_tag;

typedef union RTU_MRU_CH8_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH8_MB0_tag;

typedef union RTU_MRU_CH8_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH8_MB1_tag;

typedef union RTU_MRU_CH8_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH8_MB2_tag;

typedef union RTU_MRU_CH8_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH8_MB3_tag;

typedef union RTU_MRU_CH8_MB4_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH8_MB4_tag;

typedef union RTU_MRU_CH8_MB5_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH8_MB5_tag;

typedef union RTU_MRU_CH8_MB6_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH8_MB6_tag;

typedef union RTU_MRU_CH8_MB7_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH8_MB7_tag;

typedef union RTU_MRU_CH9_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH9_MB0_tag;

typedef union RTU_MRU_CH9_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH9_MB1_tag;

typedef union RTU_MRU_CH9_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH9_MB2_tag;

typedef union RTU_MRU_CH9_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH9_MB3_tag;

typedef union RTU_MRU_CH10_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH10_MB0_tag;

typedef union RTU_MRU_CH10_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH10_MB1_tag;

typedef union RTU_MRU_CH10_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH10_MB2_tag;

typedef union RTU_MRU_CH10_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH10_MB3_tag;

typedef union RTU_MRU_CH11_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH11_MB0_tag;

typedef union RTU_MRU_CH11_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH11_MB1_tag;

typedef union RTU_MRU_CH11_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH11_MB2_tag;

typedef union RTU_MRU_CH11_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH11_MB3_tag;

typedef union RTU_MRU_CH12_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH12_MB0_tag;

typedef union RTU_MRU_CH12_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH12_MB1_tag;

typedef union RTU_MRU_CH12_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH12_MB2_tag;

typedef union RTU_MRU_CH12_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} RTU_MRU_CH12_MB3_tag;

struct RTU_MRU_tag {
  RTU_MRU_CHXCONFIG_tag CHXCONFIG[12];
  uint8_t RTU_MRU_reserved0[320];
  RTU_MRU_NOTIFY0_tag NOTIFY0;         /* Notification 0 Status */
  RTU_MRU_NOTIFY1_tag NOTIFY1;         /* Notification 1 Status */
  uint8_t RTU_MRU_reserved1[3576];
  RTU_MRU_CH1_MB0_tag CH1_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH1_MB1_tag CH1_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved2[4088];
  RTU_MRU_CH2_MB0_tag CH2_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH2_MB1_tag CH2_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved3[4088];
  RTU_MRU_CH3_MB0_tag CH3_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH3_MB1_tag CH3_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved4[4088];
  RTU_MRU_CH4_MB0_tag CH4_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH4_MB1_tag CH4_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved5[4088];
  RTU_MRU_CH5_MB0_tag CH5_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH5_MB1_tag CH5_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved6[4088];
  RTU_MRU_CH6_MB0_tag CH6_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH6_MB1_tag CH6_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved7[4088];
  RTU_MRU_CH7_MB0_tag CH7_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH7_MB1_tag CH7_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved8[4088];
  RTU_MRU_CH8_MB0_tag CH8_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH8_MB1_tag CH8_MB1;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH8_MB2_tag CH8_MB2;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH8_MB3_tag CH8_MB3;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH8_MB4_tag CH8_MB4;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH8_MB5_tag CH8_MB5;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH8_MB6_tag CH8_MB6;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH8_MB7_tag CH8_MB7;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved9[4064];
  RTU_MRU_CH9_MB0_tag CH9_MB0;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH9_MB1_tag CH9_MB1;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH9_MB2_tag CH9_MB2;         /* Channel (x) Mailbox (n) */
  RTU_MRU_CH9_MB3_tag CH9_MB3;         /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved10[4080];
  RTU_MRU_CH10_MB0_tag CH10_MB0;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH10_MB1_tag CH10_MB1;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH10_MB2_tag CH10_MB2;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH10_MB3_tag CH10_MB3;       /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved11[4080];
  RTU_MRU_CH11_MB0_tag CH11_MB0;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH11_MB1_tag CH11_MB1;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH11_MB2_tag CH11_MB2;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH11_MB3_tag CH11_MB3;       /* Channel (x) Mailbox (n) */
  uint8_t RTU_MRU_reserved12[4080];
  RTU_MRU_CH12_MB0_tag CH12_MB0;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH12_MB1_tag CH12_MB1;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH12_MB2_tag CH12_MB2;       /* Channel (x) Mailbox (n) */
  RTU_MRU_CH12_MB3_tag CH12_MB3;       /* Channel (x) Mailbox (n) */
};


/* ============================================================================
   =============================== Module: RTU_PMC ============================
   ============================================================================ */

typedef union RTU_PMC_CTRL_union_tag { /* Main Control Register */
  vuint32_t R;
  struct {
    vuint32_t PEEN:1;                  /* Program execution enable */
    vuint32_t PES:1;                   /* Program execution suspended */
    vuint32_t TCSEN:1;                 /* Test continue signal enable */
    vuint32_t EXECO:1;                 /* Execute once */
    vuint32_t STOPF:1;                 /* Stop on failure */
    vuint32_t TE:1;                    /* Test ended status bit */
    vuint32_t TESEN:1;                 /* Test ended signal enable */
    vuint32_t TF:1;                    /* Test failed status bit */
    vuint32_t TFSEN:1;                 /* Test failed signal enable */
    vuint32_t ADDRCD:1;                /* Address change direction */
    vuint32_t ADDRID:1;                /* Address increment or decrement control */
    vuint32_t TEN:1;                   /* Test enable */
    vuint32_t STATE:2;                 /* PMC-R52 state machine status */
    vuint32_t MBISTACK:1;              /* MBISTACK signal value */
    vuint32_t _unused_15:1;
    vuint32_t MBISTCFG:16;             /* MBISTCFG signal value */
  } B;
} RTU_PMC_CTRL_tag;

typedef union RTU_PMC_MCR_union_tag {  /* Memory Control Register */
  vuint32_t R;
  struct {
    vuint32_t ARRAY:8;                 /* MBISTARRAY signal value */
    vuint32_t PD:5;                    /* Pipeline depth */
    vuint32_t RCO:4;                   /* RAM cycles per operation */
    vuint32_t CCW:2;                   /* Column counter width */
    vuint32_t _unused_19:2;
    vuint32_t RCW:5;                   /* Row counter width */
    vuint32_t _unused_26:6;
  } B;
} RTU_PMC_MCR_tag;

typedef union RTU_PMC_BER_union_tag {  /* Byte Enable Register */
  vuint32_t R;
  struct {
    vuint32_t BE:32;                   /* MBISTBE signal value */
  } B;
} RTU_PMC_BER_tag;

typedef union RTU_PMC_PCR_union_tag {  /* Program Control Register */
  vuint32_t R;
  struct {
    vuint32_t PC:5;                    /* Program Counter */
    vuint32_t _unused_5:27;
  } B;
} RTU_PMC_PCR_tag;

typedef union RTU_PMC_RPR_union_tag {  /* Read Pipeline Register */
  vuint32_t R;
  struct {
    vuint32_t R:32;                    /* Current value of the RPR in the execution unit */
  } B;
} RTU_PMC_RPR_tag;

typedef union RTU_PMC_STOPADDR_union_tag { /* Stop Address Register */
  vuint32_t R;
  struct {
    vuint32_t SA:32;                   /* Stop address */
  } B;
} RTU_PMC_STOPADDR_tag;

typedef union RTU_PMC_CADDR_union_tag { /* Column Address Register */
  vuint32_t R;
  struct {
    vuint32_t CA:5;                    /* Column Address */
    vuint32_t _unused_5:27;
  } B;
} RTU_PMC_CADDR_tag;

typedef union RTU_PMC_RADDR_union_tag { /* Row Address Register */
  vuint32_t R;
  struct {
    vuint32_t RA:30;                   /* Row Address */
    vuint32_t _unused_30:2;
  } B;
} RTU_PMC_RADDR_tag;

typedef union RTU_PMC_X0_union_tag {   /* X Data Register 0 */
  vuint32_t R;
  struct {
    vuint32_t X0:32;                   /* X data register bits [31:0] */
  } B;
} RTU_PMC_X0_tag;

typedef union RTU_PMC_X1_union_tag {   /* X Data Register 1 */
  vuint32_t R;
  struct {
    vuint32_t X1:32;                   /* X data register bits [63:32] */
  } B;
} RTU_PMC_X1_tag;

typedef union RTU_PMC_X2_union_tag {   /* X Data Register 2 */
  vuint32_t R;
  struct {
    vuint32_t X2:32;                   /* X data register bits [95:64] */
  } B;
} RTU_PMC_X2_tag;

typedef union RTU_PMC_X3_union_tag {   /* X Data Register 3 */
  vuint32_t R;
  struct {
    vuint32_t X3:32;                   /* X data register bits [127:96] */
  } B;
} RTU_PMC_X3_tag;

typedef union RTU_PMC_X4_union_tag {   /* X Data Register 4 */
  vuint32_t R;
  struct {
    vuint32_t X4:32;                   /* X data register bits [159:128] */
  } B;
} RTU_PMC_X4_tag;

typedef union RTU_PMC_X5_union_tag {   /* X Data Register 5 */
  vuint32_t R;
  struct {
    vuint32_t X5:32;                   /* X data register bits [191:160] */
  } B;
} RTU_PMC_X5_tag;

typedef union RTU_PMC_X6_union_tag {   /* X Data Register 6 */
  vuint32_t R;
  struct {
    vuint32_t X6:32;                   /* X data register bits [223:192] */
  } B;
} RTU_PMC_X6_tag;

typedef union RTU_PMC_X7_union_tag {   /* X Data Register 7 */
  vuint32_t R;
  struct {
    vuint32_t X7:32;                   /* X data register bits [255:224] */
  } B;
} RTU_PMC_X7_tag;

typedef union RTU_PMC_Y0_union_tag {   /* Y Data Register 0 */
  vuint32_t R;
  struct {
    vuint32_t Y0:32;                   /* Y data register bits [31:0] */
  } B;
} RTU_PMC_Y0_tag;

typedef union RTU_PMC_Y1_union_tag {   /* Y Data Register 1 */
  vuint32_t R;
  struct {
    vuint32_t Y1:32;                   /* Y data register bits [63:32] */
  } B;
} RTU_PMC_Y1_tag;

typedef union RTU_PMC_Y2_union_tag {   /* Y Data Register 2 */
  vuint32_t R;
  struct {
    vuint32_t Y2:32;                   /* Y data register bits [95:64] */
  } B;
} RTU_PMC_Y2_tag;

typedef union RTU_PMC_Y3_union_tag {   /* Y Data Register 3 */
  vuint32_t R;
  struct {
    vuint32_t Y3:32;                   /* Y data register bits [127:96] */
  } B;
} RTU_PMC_Y3_tag;

typedef union RTU_PMC_Y4_union_tag {   /* Y Data Register 4 */
  vuint32_t R;
  struct {
    vuint32_t Y4:32;                   /* Y data register bits [159:128] */
  } B;
} RTU_PMC_Y4_tag;

typedef union RTU_PMC_Y5_union_tag {   /* Y Data Register 5 */
  vuint32_t R;
  struct {
    vuint32_t Y5:32;                   /* Y data register bits [191:160] */
  } B;
} RTU_PMC_Y5_tag;

typedef union RTU_PMC_Y6_union_tag {   /* Y Data Register 6 */
  vuint32_t R;
  struct {
    vuint32_t Y6:32;                   /* Y data register bits [223:192] */
  } B;
} RTU_PMC_Y6_tag;

typedef union RTU_PMC_Y7_union_tag {   /* Y Data Register 7 */
  vuint32_t R;
  struct {
    vuint32_t Y7:32;                   /* Y data register bits [255:224] */
  } B;
} RTU_PMC_Y7_tag;

typedef union RTU_PMC_AIR_union_tag {  /* Auxiliary Input Register */
  vuint32_t R;
  struct {
    vuint32_t AIR:32;                  /* AUXIN signal value */
  } B;
} RTU_PMC_AIR_tag;

typedef union RTU_PMC_AOR_union_tag {  /* Auxiliary Output Register */
  vuint32_t R;
  struct {
    vuint32_t AOR:32;                  /* AUXOUT signal value */
  } B;
} RTU_PMC_AOR_tag;

typedef union RTU_PMC_DM0_union_tag {  /* Data Mask Register 0 */
  vuint32_t R;
  struct {
    vuint32_t DM0:32;                  /* Data mask bits [31:0] */
  } B;
} RTU_PMC_DM0_tag;

typedef union RTU_PMC_DM1_union_tag {  /* Data Mask Register 1 */
  vuint32_t R;
  struct {
    vuint32_t DM1:32;                  /* Data mask bits [63:32] */
  } B;
} RTU_PMC_DM1_tag;

typedef union RTU_PMC_DM2_union_tag {  /* Data Mask Register 2 */
  vuint32_t R;
  struct {
    vuint32_t DM2:32;                  /* Data mask bits [95:64] */
  } B;
} RTU_PMC_DM2_tag;

typedef union RTU_PMC_DM3_union_tag {  /* Data Mask Register 3 */
  vuint32_t R;
  struct {
    vuint32_t DM3:32;                  /* Data mask bits [127:96] */
  } B;
} RTU_PMC_DM3_tag;

typedef union RTU_PMC_DM4_union_tag {  /* Data Mask Register 4 */
  vuint32_t R;
  struct {
    vuint32_t DM4:32;                  /* Data mask bits [159:128] */
  } B;
} RTU_PMC_DM4_tag;

typedef union RTU_PMC_DM5_union_tag {  /* Data Mask Register 5 */
  vuint32_t R;
  struct {
    vuint32_t DM5:32;                  /* Data mask bits [191:160] */
  } B;
} RTU_PMC_DM5_tag;

typedef union RTU_PMC_DM6_union_tag {  /* Data Mask Register 6 */
  vuint32_t R;
  struct {
    vuint32_t DM6:32;                  /* Data mask bits [223:192] */
  } B;
} RTU_PMC_DM6_tag;

typedef union RTU_PMC_DM7_union_tag {  /* Data Mask Register 7 */
  vuint32_t R;
  struct {
    vuint32_t DM7:32;                  /* Data mask bits [255:224] */
  } B;
} RTU_PMC_DM7_tag;

typedef union RTU_PMC_P0_union_tag {   /* Program Register 0 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P0_tag;

typedef union RTU_PMC_P1_union_tag {   /* Program Register 1 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P1_tag;

typedef union RTU_PMC_P2_union_tag {   /* Program Register 2 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P2_tag;

typedef union RTU_PMC_P3_union_tag {   /* Program Register 3 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P3_tag;

typedef union RTU_PMC_P4_union_tag {   /* Program Register 4 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P4_tag;

typedef union RTU_PMC_P5_union_tag {   /* Program Register 5 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P5_tag;

typedef union RTU_PMC_P6_union_tag {   /* Program Register 6 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P6_tag;

typedef union RTU_PMC_P7_union_tag {   /* Program Register 7 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P7_tag;

typedef union RTU_PMC_P8_union_tag {   /* Program Register 8 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P8_tag;

typedef union RTU_PMC_P9_union_tag {   /* Program Register 9 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P9_tag;

typedef union RTU_PMC_P10_union_tag {  /* Program Register 10 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P10_tag;

typedef union RTU_PMC_P11_union_tag {  /* Program Register 11 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P11_tag;

typedef union RTU_PMC_P12_union_tag {  /* Program Register 12 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P12_tag;

typedef union RTU_PMC_P13_union_tag {  /* Program Register 13 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P13_tag;

typedef union RTU_PMC_P14_union_tag {  /* Program Register 14 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P14_tag;

typedef union RTU_PMC_P15_union_tag {  /* Program Register 15 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P15_tag;

typedef union RTU_PMC_P16_union_tag {  /* Program Register 16 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P16_tag;

typedef union RTU_PMC_P17_union_tag {  /* Program Register 17 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P17_tag;

typedef union RTU_PMC_P18_union_tag {  /* Program Register 18 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P18_tag;

typedef union RTU_PMC_P19_union_tag {  /* Program Register 19 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P19_tag;

typedef union RTU_PMC_P20_union_tag {  /* Program Register 20 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P20_tag;

typedef union RTU_PMC_P21_union_tag {  /* Program Register 21 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P21_tag;

typedef union RTU_PMC_P22_union_tag {  /* Program Register 22 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P22_tag;

typedef union RTU_PMC_P23_union_tag {  /* Program Register 23 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P23_tag;

typedef union RTU_PMC_P24_union_tag {  /* Program Register 24 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P24_tag;

typedef union RTU_PMC_P25_union_tag {  /* Program Register 25 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P25_tag;

typedef union RTU_PMC_P26_union_tag {  /* Program Register 26 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P26_tag;

typedef union RTU_PMC_P27_union_tag {  /* Program Register 27 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P27_tag;

typedef union RTU_PMC_P28_union_tag {  /* Program Register 28 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P28_tag;

typedef union RTU_PMC_P29_union_tag {  /* Program Register 29 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P29_tag;

typedef union RTU_PMC_P30_union_tag {  /* Program Register 30 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P30_tag;

typedef union RTU_PMC_P31_union_tag {  /* Program Register 31 */
  vuint32_t R;
  struct {
    vuint32_t OP:2;                    /* Operation field */
    vuint32_t _unused_2:2;
    vuint32_t TRANS:2;                 /* MBIST transaction generated */
    vuint32_t DREG:1;                  /* Data register that the instruction uses */
    vuint32_t DPOL:1;                  /* Data polarity */
    vuint32_t UA:1;                    /* Update address */
    vuint32_t AO:1;                    /* Address output */
    vuint32_t _unused_10:22;
  } B;
} RTU_PMC_P31_tag;

typedef union RTU_PMC_ITCTRL_union_tag { /* Integration Mode Control Register */
  vuint32_t R;
  struct {
    vuint32_t IME:1;                   /* Row counter width */
    vuint32_t _unused_1:31;
  } B;
} RTU_PMC_ITCTRL_tag;

typedef union RTU_PMC_CLAIMSET_union_tag { /* Claim Tag Set Register */
  vuint32_t R;
  struct {
    vuint32_t SET:4;                   /* Claim set */
    vuint32_t _unused_4:28;
  } B;
} RTU_PMC_CLAIMSET_tag;

typedef union RTU_PMC_CLAIMCLR_union_tag { /* Claim Tag Clear Register */
  vuint32_t R;
  struct {
    vuint32_t CLR:4;                   /* Claim clear */
    vuint32_t _unused_4:28;
  } B;
} RTU_PMC_CLAIMCLR_tag;

typedef union RTU_PMC_DEVAFF0_union_tag { /* Device Affinity 0 Register */
  vuint32_t R;
  struct {
    vuint32_t AFF0:8;                  /* Affinity level 0 */
    vuint32_t AFF1:8;                  /* Affinity level 1 */
    vuint32_t AFF2:8;                  /* Affinity level 2 */
    vuint32_t MT:1;                    /* Lowest level of affinity consists of logical cores implemented using a multi-threading approach */
    vuint32_t _unused_25:5;
    vuint32_t U:1;                     /* Single core system being distinct from core 0 in a processor */
    vuint32_t M:1;                     /* Unused */
  } B;
} RTU_PMC_DEVAFF0_tag;

typedef union RTU_PMC_DEVAFF1_union_tag { /* Device Affinity 1 Register */
  vuint32_t R;
  struct {
    vuint32_t DEVAFF1:32;              /* revisit empty displayName */
  } B;
} RTU_PMC_DEVAFF1_tag;

typedef union RTU_PMC_LAR_union_tag {  /* Lock Access Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_LAR_tag;

typedef union RTU_PMC_LSR_union_tag {  /* Lock Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_LSR_tag;

typedef union RTU_PMC_AUTHSTATUS_union_tag { /* Authentication Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_AUTHSTATUS_tag;

typedef union RTU_PMC_DEVARCH_union_tag { /* Device Architecture Register */
  vuint32_t R;
  struct {
    vuint32_t ARCHID:16;               /* Architecture ID */
    vuint32_t REVISION:4;              /* Architecture revision */
    vuint32_t PRESENT:1;               /* Indicates the presence of this register. */
    vuint32_t ARCHITECT:11;            /* Defines the architect of the component */
  } B;
} RTU_PMC_DEVARCH_tag;

typedef union RTU_PMC_DEVID2_union_tag { /* Device ID Register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_DEVID2_tag;

typedef union RTU_PMC_DEVID1_union_tag { /* Device ID Register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_DEVID1_tag;

typedef union RTU_PMC_DEVID_union_tag { /* Device ID Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_DEVID_tag;

typedef union RTU_PMC_DEVTYPE_union_tag { /* Device Type Identifier Register */
  vuint32_t R;
  struct {
    vuint32_t MAJOR:4;                 /* The main type of the component */
    vuint32_t SUB:4;                   /* The subtype of the component */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_DEVTYPE_tag;

typedef union RTU_PMC_PIDR4_union_tag { /* Peripheral Identification Register 4 */
  vuint32_t R;
  struct {
    vuint32_t DES_2:4;                 /* JEP106 continuation code */
    vuint32_t SIZE:4;                  /* Indicates the memory size that this component uses */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_PIDR4_tag;

typedef union RTU_PMC_PIDR5_union_tag { /* Peripheral Identification Register 5 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_PIDR5_tag;

typedef union RTU_PMC_PIDR6_union_tag { /* Peripheral Identification Register 6 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_PIDR6_tag;

typedef union RTU_PMC_PIDR7_union_tag { /* Peripheral Identification Register 7 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:32;
  } B;
} RTU_PMC_PIDR7_tag;

typedef union RTU_PMC_PIDR0_union_tag { /* Peripheral Identification Register 0 */
  vuint32_t R;
  struct {
    vuint32_t PART_0:8;                /* Part number bits [7:0] */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_PIDR0_tag;

typedef union RTU_PMC_PIDR1_union_tag { /* Peripheral Identification Register 1 */
  vuint32_t R;
  struct {
    vuint32_t PART_1:4;                /* Part number bits [11:8] */
    vuint32_t DES_0:4;                 /* JEP106 identification code [3:0] */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_PIDR1_tag;

typedef union RTU_PMC_PIDR2_union_tag { /* Peripheral Identification Register 2 */
  vuint32_t R;
  struct {
    vuint32_t DES_1:3;                 /* JEP106 identification bits [6:4] */
    vuint32_t JEDEC:1;                 /* JEDEC */
    vuint32_t REVISION:4;              /* Revision number of PMC-R52 */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_PIDR2_tag;

typedef union RTU_PMC_PIDR3_union_tag { /* Peripheral Identification Register 3 */
  vuint32_t R;
  struct {
    vuint32_t CMOD:4;                  /* Customer modified */
    vuint32_t REVAND:4;                /* Part minor revision */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_PIDR3_tag;

typedef union RTU_PMC_CIDR0_union_tag { /* Component Identification Register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_0:8;               /* Preamble, segment 0 */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_CIDR0_tag;

typedef union RTU_PMC_CIDR1_union_tag { /* Component Identification Register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_1:4;               /* Preamble, segment 1 */
    vuint32_t CLASS:4;                 /* The component class */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_CIDR1_tag;

typedef union RTU_PMC_CIDR2_union_tag { /* Component Identification Register 2 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_2:8;               /* Preamble, segment 2 */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_CIDR2_tag;

typedef union RTU_PMC_CIDR3_union_tag { /* Component Identification Register 3 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_3:8;               /* Preamble, segment 3 */
    vuint32_t _unused_8:24;
  } B;
} RTU_PMC_CIDR3_tag;

struct RTU_PMC_tag {
  RTU_PMC_CTRL_tag CTRL;               /* Main Control Register */
  RTU_PMC_MCR_tag MCR;                 /* Memory Control Register */
  RTU_PMC_BER_tag BER;                 /* Byte Enable Register */
  RTU_PMC_PCR_tag PCR;                 /* Program Control Register */
  RTU_PMC_RPR_tag RPR;                 /* Read Pipeline Register */
  RTU_PMC_STOPADDR_tag STOPADDR;       /* Stop Address Register */
  RTU_PMC_CADDR_tag CADDR;             /* Column Address Register */
  RTU_PMC_RADDR_tag RADDR;             /* Row Address Register */
  RTU_PMC_X0_tag X0;                   /* X Data Register 0 */
  RTU_PMC_X1_tag X1;                   /* X Data Register 1 */
  RTU_PMC_X2_tag X2;                   /* X Data Register 2 */
  RTU_PMC_X3_tag X3;                   /* X Data Register 3 */
  RTU_PMC_X4_tag X4;                   /* X Data Register 4 */
  RTU_PMC_X5_tag X5;                   /* X Data Register 5 */
  RTU_PMC_X6_tag X6;                   /* X Data Register 6 */
  RTU_PMC_X7_tag X7;                   /* X Data Register 7 */
  RTU_PMC_Y0_tag Y0;                   /* Y Data Register 0 */
  RTU_PMC_Y1_tag Y1;                   /* Y Data Register 1 */
  RTU_PMC_Y2_tag Y2;                   /* Y Data Register 2 */
  RTU_PMC_Y3_tag Y3;                   /* Y Data Register 3 */
  RTU_PMC_Y4_tag Y4;                   /* Y Data Register 4 */
  RTU_PMC_Y5_tag Y5;                   /* Y Data Register 5 */
  RTU_PMC_Y6_tag Y6;                   /* Y Data Register 6 */
  RTU_PMC_Y7_tag Y7;                   /* Y Data Register 7 */
  RTU_PMC_AIR_tag AIR;                 /* Auxiliary Input Register */
  RTU_PMC_AOR_tag AOR;                 /* Auxiliary Output Register */
  uint8_t RTU_PMC_reserved0[24];
  RTU_PMC_DM0_tag DM0;                 /* Data Mask Register 0 */
  RTU_PMC_DM1_tag DM1;                 /* Data Mask Register 1 */
  RTU_PMC_DM2_tag DM2;                 /* Data Mask Register 2 */
  RTU_PMC_DM3_tag DM3;                 /* Data Mask Register 3 */
  RTU_PMC_DM4_tag DM4;                 /* Data Mask Register 4 */
  RTU_PMC_DM5_tag DM5;                 /* Data Mask Register 5 */
  RTU_PMC_DM6_tag DM6;                 /* Data Mask Register 6 */
  RTU_PMC_DM7_tag DM7;                 /* Data Mask Register 7 */
  uint8_t RTU_PMC_reserved1[96];
  RTU_PMC_P0_tag P0;                   /* Program Register 0 */
  RTU_PMC_P1_tag P1;                   /* Program Register 1 */
  RTU_PMC_P2_tag P2;                   /* Program Register 2 */
  RTU_PMC_P3_tag P3;                   /* Program Register 3 */
  RTU_PMC_P4_tag P4;                   /* Program Register 4 */
  RTU_PMC_P5_tag P5;                   /* Program Register 5 */
  RTU_PMC_P6_tag P6;                   /* Program Register 6 */
  RTU_PMC_P7_tag P7;                   /* Program Register 7 */
  RTU_PMC_P8_tag P8;                   /* Program Register 8 */
  RTU_PMC_P9_tag P9;                   /* Program Register 9 */
  RTU_PMC_P10_tag P10;                 /* Program Register 10 */
  RTU_PMC_P11_tag P11;                 /* Program Register 11 */
  RTU_PMC_P12_tag P12;                 /* Program Register 12 */
  RTU_PMC_P13_tag P13;                 /* Program Register 13 */
  RTU_PMC_P14_tag P14;                 /* Program Register 14 */
  RTU_PMC_P15_tag P15;                 /* Program Register 15 */
  RTU_PMC_P16_tag P16;                 /* Program Register 16 */
  RTU_PMC_P17_tag P17;                 /* Program Register 17 */
  RTU_PMC_P18_tag P18;                 /* Program Register 18 */
  RTU_PMC_P19_tag P19;                 /* Program Register 19 */
  RTU_PMC_P20_tag P20;                 /* Program Register 20 */
  RTU_PMC_P21_tag P21;                 /* Program Register 21 */
  RTU_PMC_P22_tag P22;                 /* Program Register 22 */
  RTU_PMC_P23_tag P23;                 /* Program Register 23 */
  RTU_PMC_P24_tag P24;                 /* Program Register 24 */
  RTU_PMC_P25_tag P25;                 /* Program Register 25 */
  RTU_PMC_P26_tag P26;                 /* Program Register 26 */
  RTU_PMC_P27_tag P27;                 /* Program Register 27 */
  RTU_PMC_P28_tag P28;                 /* Program Register 28 */
  RTU_PMC_P29_tag P29;                 /* Program Register 29 */
  RTU_PMC_P30_tag P30;                 /* Program Register 30 */
  RTU_PMC_P31_tag P31;                 /* Program Register 31 */
  uint8_t RTU_PMC_reserved2[3456];
  RTU_PMC_ITCTRL_tag ITCTRL;           /* Integration Mode Control Register */
  uint8_t RTU_PMC_reserved3[156];
  RTU_PMC_CLAIMSET_tag CLAIMSET;       /* Claim Tag Set Register */
  RTU_PMC_CLAIMCLR_tag CLAIMCLR;       /* Claim Tag Clear Register */
  RTU_PMC_DEVAFF0_tag DEVAFF0;         /* Device Affinity 0 Register */
  RTU_PMC_DEVAFF1_tag DEVAFF1;         /* Device Affinity 1 Register */
  RTU_PMC_LAR_tag LAR;                 /* Lock Access Register */
  RTU_PMC_LSR_tag LSR;                 /* Lock Status Register */
  RTU_PMC_AUTHSTATUS_tag AUTHSTATUS;   /* Authentication Status Register */
  RTU_PMC_DEVARCH_tag DEVARCH;         /* Device Architecture Register */
  RTU_PMC_DEVID2_tag DEVID2;           /* Device ID Register 2 */
  RTU_PMC_DEVID1_tag DEVID1;           /* Device ID Register 1 */
  RTU_PMC_DEVID_tag DEVID;             /* Device ID Register */
  RTU_PMC_DEVTYPE_tag DEVTYPE;         /* Device Type Identifier Register */
  RTU_PMC_PIDR4_tag PIDR4;             /* Peripheral Identification Register 4 */
  RTU_PMC_PIDR5_tag PIDR5;             /* Peripheral Identification Register 5 */
  RTU_PMC_PIDR6_tag PIDR6;             /* Peripheral Identification Register 6 */
  RTU_PMC_PIDR7_tag PIDR7;             /* Peripheral Identification Register 7 */
  RTU_PMC_PIDR0_tag PIDR0;             /* Peripheral Identification Register 0 */
  RTU_PMC_PIDR1_tag PIDR1;             /* Peripheral Identification Register 1 */
  RTU_PMC_PIDR2_tag PIDR2;             /* Peripheral Identification Register 2 */
  RTU_PMC_PIDR3_tag PIDR3;             /* Peripheral Identification Register 3 */
  RTU_PMC_CIDR0_tag CIDR0;             /* Component Identification Register 0 */
  RTU_PMC_CIDR1_tag CIDR1;             /* Component Identification Register 1 */
  RTU_PMC_CIDR2_tag CIDR2;             /* Component Identification Register 2 */
  RTU_PMC_CIDR3_tag CIDR3;             /* Component Identification Register 3 */
};


/* ============================================================================
   =============================== Module: RTU_SEMA42 =========================
   ============================================================================ */

typedef union RTU_SEMA42_GATE_union_tag { /* Gate */
  vuint8_t R;
  struct {
    vuint8_t GTFSM:4;                  /* Gate Finite State Machine */
    vuint8_t _unused_4:4;              /* Reserved */
  } B;
} RTU_SEMA42_GATE_tag;

typedef union RTU_SEMA42_RSTGT_R_union_tag { /* Reset Gate Read */
  vuint16_t R;
  struct {
    vuint16_t RSTGTN:8;                /* Reset Gate Number */
    vuint16_t RSTGMS:4;                /* Reset Gate Domain */
    vuint16_t RSTGSM:2;                /* Reset Gate Finite State Machine */
    vuint16_t _unused_14:2;
  } B;
} RTU_SEMA42_RSTGT_R_tag;

typedef union RTU_SEMA42_RSTGT_W_union_tag { /* Reset Gate Write */
  vuint16_t R;
  struct {
    vuint16_t RSTGTN:8;                /* Reset Gate Number */
    vuint16_t RSTGDP:8;                /* Reset Gate Data Pattern */
  } B;
} RTU_SEMA42_RSTGT_W_tag;

typedef union RTU_SEMA42_RSTGT_union_tag {
  RTU_SEMA42_RSTGT_R_tag R;            /* Reset Gate Read */
  RTU_SEMA42_RSTGT_W_tag W;            /* Reset Gate Write */
} RTU_SEMA42_RSTGT_tag;

struct RTU_SEMA42_tag {
  RTU_SEMA42_GATE_tag GATE[16];        /* Gate */
  uint8_t RTU_SEMA42_reserved0[50];
  RTU_SEMA42_RSTGT_tag RSTGT;
};


/* ============================================================================
   =============================== Module: RTU_XRDC ===========================
   ============================================================================ */

typedef union RTU_XRDC_CR_union_tag {  /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} RTU_XRDC_CR_tag;

typedef union RTU_XRDC_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} RTU_XRDC_HWCFG0_tag;

typedef union RTU_XRDC_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} RTU_XRDC_HWCFG1_tag;

typedef union RTU_XRDC_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} RTU_XRDC_MDACFG_tag;

typedef union RTU_XRDC_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} RTU_XRDC_MRCFG_tag;

typedef union RTU_XRDC_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} RTU_XRDC_DERRLOC_tag;

typedef union RTU_XRDC_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} RTU_XRDC_DERRW0_DERR_W0_tag;

typedef union RTU_XRDC_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} RTU_XRDC_DERRW0_DERR_W1_tag;

typedef union RTU_XRDC_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} RTU_XRDC_DERRW0_DERR_W3_tag;

typedef struct RTU_XRDC_DERRW0_struct_tag {
  RTU_XRDC_DERRW0_DERR_W0_tag DERR_W0; /* Domain Error Word 0 */
  RTU_XRDC_DERRW0_DERR_W1_tag DERR_W1; /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  RTU_XRDC_DERRW0_DERR_W3_tag DERR_W3; /* Domain Error Word 3 */
} RTU_XRDC_DERRW0_tag;

typedef union RTU_XRDC_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_0_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_1_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_2_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_3_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_3_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_4_DFMT1_tag;

typedef union RTU_XRDC_MDA_W1_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W1_4_DFMT1_tag;

typedef union RTU_XRDC_MDA_W2_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W2_4_DFMT1_tag;

typedef union RTU_XRDC_MDA_W3_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W3_4_DFMT1_tag;

typedef union RTU_XRDC_MDA_W4_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W4_4_DFMT1_tag;

typedef union RTU_XRDC_MDA_W5_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W5_4_DFMT1_tag;

typedef union RTU_XRDC_MDA_W6_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W6_4_DFMT1_tag;

typedef union RTU_XRDC_MDA_W7_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W7_4_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_5_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_5_DFMT1_tag;

typedef union RTU_XRDC_MDA_W1_5_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W1_5_DFMT1_tag;

typedef union RTU_XRDC_MDA_W2_5_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W2_5_DFMT1_tag;

typedef union RTU_XRDC_MDA_W3_5_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W3_5_DFMT1_tag;

typedef union RTU_XRDC_MDA_W4_5_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W4_5_DFMT1_tag;

typedef union RTU_XRDC_MDA_W5_5_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W5_5_DFMT1_tag;

typedef union RTU_XRDC_MDA_W6_5_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W6_5_DFMT1_tag;

typedef union RTU_XRDC_MDA_W7_5_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W7_5_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_6_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_6_DFMT1_tag;

typedef union RTU_XRDC_MDA_W1_6_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W1_6_DFMT1_tag;

typedef union RTU_XRDC_MDA_W2_6_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W2_6_DFMT1_tag;

typedef union RTU_XRDC_MDA_W3_6_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W3_6_DFMT1_tag;

typedef union RTU_XRDC_MDA_W4_6_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W4_6_DFMT1_tag;

typedef union RTU_XRDC_MDA_W5_6_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W5_6_DFMT1_tag;

typedef union RTU_XRDC_MDA_W6_6_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W6_6_DFMT1_tag;

typedef union RTU_XRDC_MDA_W7_6_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W7_6_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_7_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_7_DFMT1_tag;

typedef union RTU_XRDC_MDA_W1_7_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W1_7_DFMT1_tag;

typedef union RTU_XRDC_MDA_W2_7_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W2_7_DFMT1_tag;

typedef union RTU_XRDC_MDA_W3_7_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W3_7_DFMT1_tag;

typedef union RTU_XRDC_MDA_W4_7_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W4_7_DFMT1_tag;

typedef union RTU_XRDC_MDA_W5_7_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W5_7_DFMT1_tag;

typedef union RTU_XRDC_MDA_W6_7_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W6_7_DFMT1_tag;

typedef union RTU_XRDC_MDA_W7_7_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W7_7_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_8_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_8_DFMT1_tag;

typedef union RTU_XRDC_MDA_W1_8_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W1_8_DFMT1_tag;

typedef union RTU_XRDC_MDA_W2_8_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W2_8_DFMT1_tag;

typedef union RTU_XRDC_MDA_W3_8_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W3_8_DFMT1_tag;

typedef union RTU_XRDC_MDA_W4_8_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W4_8_DFMT1_tag;

typedef union RTU_XRDC_MDA_W5_8_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W5_8_DFMT1_tag;

typedef union RTU_XRDC_MDA_W6_8_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W6_8_DFMT1_tag;

typedef union RTU_XRDC_MDA_W7_8_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W7_8_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_9_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_9_DFMT1_tag;

typedef union RTU_XRDC_MDA_W1_9_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W1_9_DFMT1_tag;

typedef union RTU_XRDC_MDA_W2_9_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W2_9_DFMT1_tag;

typedef union RTU_XRDC_MDA_W3_9_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W3_9_DFMT1_tag;

typedef union RTU_XRDC_MDA_W4_9_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W4_9_DFMT1_tag;

typedef union RTU_XRDC_MDA_W5_9_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W5_9_DFMT1_tag;

typedef union RTU_XRDC_MDA_W6_9_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W6_9_DFMT1_tag;

typedef union RTU_XRDC_MDA_W7_9_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W7_9_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_10_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_10_DFMT1_tag;

typedef union RTU_XRDC_MDA_W1_10_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W1_10_DFMT1_tag;

typedef union RTU_XRDC_MDA_W2_10_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W2_10_DFMT1_tag;

typedef union RTU_XRDC_MDA_W3_10_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W3_10_DFMT1_tag;

typedef union RTU_XRDC_MDA_W4_10_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W4_10_DFMT1_tag;

typedef union RTU_XRDC_MDA_W5_10_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W5_10_DFMT1_tag;

typedef union RTU_XRDC_MDA_W6_10_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W6_10_DFMT1_tag;

typedef union RTU_XRDC_MDA_W7_10_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W7_10_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_11_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_11_DFMT1_tag;

typedef union RTU_XRDC_MDA_W1_11_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W1_11_DFMT1_tag;

typedef union RTU_XRDC_MDA_W2_11_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W2_11_DFMT1_tag;

typedef union RTU_XRDC_MDA_W3_11_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W3_11_DFMT1_tag;

typedef union RTU_XRDC_MDA_W4_11_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W4_11_DFMT1_tag;

typedef union RTU_XRDC_MDA_W5_11_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W5_11_DFMT1_tag;

typedef union RTU_XRDC_MDA_W6_11_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W6_11_DFMT1_tag;

typedef union RTU_XRDC_MDA_W7_11_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W7_11_DFMT1_tag;

typedef union RTU_XRDC_MDA_W0_12_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MDA_W0_12_DFMT1_tag;

typedef union RTU_XRDC_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} RTU_XRDC_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union RTU_XRDC_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct RTU_XRDC_PDAC_SLOT_PDACN_struct_tag {
  RTU_XRDC_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  RTU_XRDC_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} RTU_XRDC_PDAC_SLOT_PDACN_tag;

typedef struct RTU_XRDC_PDAC_SLOT_struct_tag {
  RTU_XRDC_PDAC_SLOT_PDACN_tag PDACN[43]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[680];
} RTU_XRDC_PDAC_SLOT_tag;

typedef union RTU_XRDC_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} RTU_XRDC_MRGDN_XRDC_MRGD_W0_tag;

typedef union RTU_XRDC_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} RTU_XRDC_MRGDN_XRDC_MRGD_W1_tag;

typedef union RTU_XRDC_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} RTU_XRDC_MRGDN_XRDC_MRGD_W2_tag;

typedef union RTU_XRDC_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} RTU_XRDC_MRGDN_XRDC_MRGD_W3_tag;

typedef struct RTU_XRDC_MRGDN_struct_tag {
  RTU_XRDC_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  RTU_XRDC_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  RTU_XRDC_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  RTU_XRDC_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} RTU_XRDC_MRGDN_tag;

struct RTU_XRDC_tag {
  RTU_XRDC_CR_tag CR;                  /* Control */
  uint8_t RTU_XRDC_reserved0[236];
  RTU_XRDC_HWCFG0_tag HWCFG0;          /* Hardware Configuration 0 */
  RTU_XRDC_HWCFG1_tag HWCFG1;          /* Hardware Configuration 1 */
  uint8_t RTU_XRDC_reserved1[8];
  RTU_XRDC_MDACFG_tag MDACFG[13];      /* Master Domain Assignment Configuration */
  uint8_t RTU_XRDC_reserved2[51];
  RTU_XRDC_MRCFG_tag MRCFG[15];        /* Memory Region Configuration */
  uint8_t RTU_XRDC_reserved3[177];
  RTU_XRDC_DERRLOC_tag DERRLOC[16];    /* Domain Error Location */
  uint8_t RTU_XRDC_reserved4[448];
  RTU_XRDC_DERRW0_tag DERRW0[19];      /* Valid array indices: 0-14, 16-18 */
  uint8_t RTU_XRDC_reserved5[720];
  RTU_XRDC_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  uint8_t RTU_XRDC_reserved6[28];
  RTU_XRDC_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  uint8_t RTU_XRDC_reserved7[28];
  RTU_XRDC_MDA_W0_2_DFMT1_tag MDA_W0_2_DFMT1; /* Master Domain Assignment */
  uint8_t RTU_XRDC_reserved8[28];
  RTU_XRDC_MDA_W0_3_DFMT1_tag MDA_W0_3_DFMT1; /* Master Domain Assignment */
  uint8_t RTU_XRDC_reserved9[28];
  RTU_XRDC_MDA_W0_4_DFMT1_tag MDA_W0_4_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W1_4_DFMT1_tag MDA_W1_4_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W2_4_DFMT1_tag MDA_W2_4_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W3_4_DFMT1_tag MDA_W3_4_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W4_4_DFMT1_tag MDA_W4_4_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W5_4_DFMT1_tag MDA_W5_4_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W6_4_DFMT1_tag MDA_W6_4_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W7_4_DFMT1_tag MDA_W7_4_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W0_5_DFMT1_tag MDA_W0_5_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W1_5_DFMT1_tag MDA_W1_5_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W2_5_DFMT1_tag MDA_W2_5_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W3_5_DFMT1_tag MDA_W3_5_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W4_5_DFMT1_tag MDA_W4_5_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W5_5_DFMT1_tag MDA_W5_5_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W6_5_DFMT1_tag MDA_W6_5_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W7_5_DFMT1_tag MDA_W7_5_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W0_6_DFMT1_tag MDA_W0_6_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W1_6_DFMT1_tag MDA_W1_6_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W2_6_DFMT1_tag MDA_W2_6_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W3_6_DFMT1_tag MDA_W3_6_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W4_6_DFMT1_tag MDA_W4_6_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W5_6_DFMT1_tag MDA_W5_6_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W6_6_DFMT1_tag MDA_W6_6_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W7_6_DFMT1_tag MDA_W7_6_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W0_7_DFMT1_tag MDA_W0_7_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W1_7_DFMT1_tag MDA_W1_7_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W2_7_DFMT1_tag MDA_W2_7_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W3_7_DFMT1_tag MDA_W3_7_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W4_7_DFMT1_tag MDA_W4_7_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W5_7_DFMT1_tag MDA_W5_7_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W6_7_DFMT1_tag MDA_W6_7_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W7_7_DFMT1_tag MDA_W7_7_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W0_8_DFMT1_tag MDA_W0_8_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W1_8_DFMT1_tag MDA_W1_8_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W2_8_DFMT1_tag MDA_W2_8_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W3_8_DFMT1_tag MDA_W3_8_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W4_8_DFMT1_tag MDA_W4_8_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W5_8_DFMT1_tag MDA_W5_8_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W6_8_DFMT1_tag MDA_W6_8_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W7_8_DFMT1_tag MDA_W7_8_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W0_9_DFMT1_tag MDA_W0_9_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W1_9_DFMT1_tag MDA_W1_9_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W2_9_DFMT1_tag MDA_W2_9_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W3_9_DFMT1_tag MDA_W3_9_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W4_9_DFMT1_tag MDA_W4_9_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W5_9_DFMT1_tag MDA_W5_9_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W6_9_DFMT1_tag MDA_W6_9_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W7_9_DFMT1_tag MDA_W7_9_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W0_10_DFMT1_tag MDA_W0_10_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W1_10_DFMT1_tag MDA_W1_10_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W2_10_DFMT1_tag MDA_W2_10_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W3_10_DFMT1_tag MDA_W3_10_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W4_10_DFMT1_tag MDA_W4_10_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W5_10_DFMT1_tag MDA_W5_10_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W6_10_DFMT1_tag MDA_W6_10_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W7_10_DFMT1_tag MDA_W7_10_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W0_11_DFMT1_tag MDA_W0_11_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W1_11_DFMT1_tag MDA_W1_11_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W2_11_DFMT1_tag MDA_W2_11_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W3_11_DFMT1_tag MDA_W3_11_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W4_11_DFMT1_tag MDA_W4_11_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W5_11_DFMT1_tag MDA_W5_11_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W6_11_DFMT1_tag MDA_W6_11_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W7_11_DFMT1_tag MDA_W7_11_DFMT1; /* Master Domain Assignment */
  RTU_XRDC_MDA_W0_12_DFMT1_tag MDA_W0_12_DFMT1; /* Master Domain Assignment */
  uint8_t RTU_XRDC_reserved10[1660];
  RTU_XRDC_PDAC_SLOT_tag PDAC_SLOT[3];
  uint8_t RTU_XRDC_reserved11[1024];
  RTU_XRDC_MRGDN_tag MRGDN[15][16];
};


/* ============================================================================
   =============================== Module: RXLUT ==============================
   ============================================================================ */

typedef union RXLUT_RLCR_union_tag {   /* RXLUT Configuration */
  vuint32_t R;
  struct {
    vuint32_t RXLEN:1;                 /* RXLUT Module Enable */
    vuint32_t SRCH_EN:1;               /* Search Enable */
    vuint32_t PS_EN:1;                 /* Priority Search Enable */
    vuint32_t SOVR:1;                  /* Search Result Overwrite */
    vuint32_t FLUSH:1;                 /* Flush */
    vuint32_t ECC_EN:1;                /* Memory ECC Enable */
    vuint32_t FID_RAR_EN:1;            /* CAN Frame ID Rearrangement Enable */
    vuint32_t _unused_7:25;
  } B;
} RXLUT_RLCR_tag;

typedef union RXLUT_RLSR_union_tag {   /* RXLUT Status */
  vuint32_t R;
  struct {
    vuint32_t BUSY:1;                  /* Busy Status */
    vuint32_t FLUSH:1;                 /* Flush */
    vuint32_t PENDS:1;                 /* Pending Search Status */
    vuint32_t MS:1;                    /* Match Status */
    vuint32_t CIE:1;                   /* Channel ID Error */
    vuint32_t _unused_5:27;
  } B;
} RXLUT_RLSR_tag;

typedef union RXLUT_RLSRIER_union_tag { /* RXLUT Search Result Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t AVLIE:12;                /* Normal Search Result Available Interrupt Enable Per Channel */
    vuint32_t _unused_12:4;
    vuint32_t _unused_16:12;
    vuint32_t PS_AVLIE:1;              /* Priority Search Result Available Interrupt Enable */
    vuint32_t _unused_29:3;
  } B;
} RXLUT_RLSRIER_tag;

typedef union RXLUT_RLSRSR_union_tag { /* RXLUT Search Result Status */
  vuint32_t R;
  struct {
    vuint32_t AVL:12;                  /* Normal Search Result Status Per Channel */
    vuint32_t _unused_12:4;
    vuint32_t _unused_16:12;
    vuint32_t PS_AVL:1;                /* Priority Search Result Status */
    vuint32_t _unused_29:3;
  } B;
} RXLUT_RLSRSR_tag;

typedef union RXLUT_RLFIMER_union_tag { /* RXLUT Frame ID End */
  vuint32_t R;
  struct {
    vuint32_t FIME:32;                 /* Frame ID End */
  } B;
} RXLUT_RLFIMER_tag;

typedef union RXLUT_RLFISR_union_tag { /* RXLUT Frame ID Start */
  vuint32_t R;
  struct {
    vuint32_t FIS:32;                  /* Frame ID Start */
  } B;
} RXLUT_RLFISR_tag;

typedef union RXLUT_RLCIXR_union_tag { /* RXLUT Channel ID AND Index */
  vuint32_t R;
  struct {
    vuint32_t SRCH:1;                  /* Search */
    vuint32_t ENABLE:1;                /* Enable */
    vuint32_t CID:6;                   /* Channel ID */
    vuint32_t CTRL:2;                  /* Control */
    vuint32_t _unused_10:5;
    vuint32_t _unused_15:1;
    vuint32_t INDX:16;                 /* Index */
  } B;
} RXLUT_RLCIXR_tag;

typedef union RXLUT_RLADRCR_union_tag { /* RXLUT Address and Command */
  vuint32_t R;
  struct {
    vuint32_t ADDR:16;                 /* Entry Address */
    vuint32_t CMD:1;                   /* Command */
    vuint32_t _unused_17:15;
  } B;
} RXLUT_RLADRCR_tag;

typedef union RXLUT_RLFIMESR_union_tag { /* RXLUT Frame ID End Status */
  vuint32_t R;
  struct {
    vuint32_t FIMES:32;                /* Frame ID Mask OR Frame ID End Status */
  } B;
} RXLUT_RLFIMESR_tag;

typedef union RXLUT_RLFISSR_union_tag { /* RXLUT Frame ID Start Status */
  vuint32_t R;
  struct {
    vuint32_t FISS:32;                 /* Frame ID Start Status */
  } B;
} RXLUT_RLFISSR_tag;

typedef union RXLUT_RLCIXSR_union_tag { /* RXLUT Channel ID AND Index Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t ENABLE:1;                /* Enable Status */
    vuint32_t CIDS:6;                  /* Channel ID Status */
    vuint32_t CTRLS:2;                 /* Control Status */
    vuint32_t _unused_10:5;
    vuint32_t _unused_15:1;
    vuint32_t INDXS:16;                /* Index Status */
  } B;
} RXLUT_RLCIXSR_tag;

typedef union RXLUT_RLADRSR_union_tag { /* RXLUT Address Status */
  vuint32_t R;
  struct {
    vuint32_t ADDRS:16;                /* Entry Address Status */
    vuint32_t _unused_16:16;
  } B;
} RXLUT_RLADRSR_tag;

typedef union RXLUT_RLSICR_union_tag { /* RXLUT Search Input Command */
  vuint32_t R;
  struct {
    vuint32_t CID:6;                   /* Channel ID */
    vuint32_t _unused_6:10;
    vuint32_t SYNC_ID:14;              /* Synchronization Identity */
    vuint32_t _unused_30:2;
  } B;
} RXLUT_RLSICR_tag;

typedef union RXLUT_RLSICFIR_union_tag { /* RXLUT Search Input Command Frame ID */
  vuint32_t R;
  struct {
    vuint32_t FID:32;                  /* Frame ID */
  } B;
} RXLUT_RLSICFIR_tag;

typedef union RXLUT_RLSOIXR_RLSOIXR_union_tag { /* RXLUT Search Output */
  vuint32_t R;
  struct {
    vuint32_t MS:1;                    /* Match Status */
    vuint32_t ENABLE:1;                /* Entry Enable */
    vuint32_t SYNC_ID:14;              /* Synchronization Identity */
    vuint32_t IDX:16;                  /* Index */
  } B;
} RXLUT_RLSOIXR_RLSOIXR_tag;

typedef union RXLUT_RLSOIXR_RLSOADR_union_tag { /* RXLUT Search Output Address */
  vuint32_t R;
  struct {
    vuint32_t ADR:16;                  /* Address */
    vuint32_t _unused_16:16;
  } B;
} RXLUT_RLSOIXR_RLSOADR_tag;

typedef struct RXLUT_RLSOIXR_struct_tag {
  RXLUT_RLSOIXR_RLSOIXR_tag RLSOIXR;   /* RXLUT Search Output */
  RXLUT_RLSOIXR_RLSOADR_tag RLSOADR;   /* RXLUT Search Output Address */
} RXLUT_RLSOIXR_tag;

typedef union RXLUT_RLSICFSR_union_tag { /* RXLUT Search Input Command FIFO Status */
  vuint32_t R;
  struct {
    vuint32_t EMPTY:1;                 /* FIFO Empty */
    vuint32_t FULL:1;                  /* FIFO Full */
    vuint32_t _unused_2:30;
  } B;
} RXLUT_RLSICFSR_tag;

typedef union RXLUT_RLSOFSR0_union_tag { /* RXLUT Search Output FIFO Status */
  vuint32_t R;
  struct {
    vuint32_t EMPTY0:1;                /* FIFO Empty for channel 0 */
    vuint32_t FULL0:1;                 /* FIFO Full for channel 0 */
    vuint32_t _unused_2:2;
    vuint32_t EMPTY1:1;                /* FIFO Empty for channel 1 */
    vuint32_t FULL1:1;                 /* FIFO Full for channel 1 */
    vuint32_t _unused_6:2;
    vuint32_t EMPTY2:1;                /* FIFO Empty for channel 2 */
    vuint32_t FULL2:1;                 /* FIFO Full for channel 2 */
    vuint32_t _unused_10:2;
    vuint32_t EMPTY3:1;                /* FIFO Empty for channel 3 */
    vuint32_t FULL3:1;                 /* FIFO Full for channel 3 */
    vuint32_t _unused_14:2;
    vuint32_t EMPTY4:1;                /* FIFO Empty for channel 4 */
    vuint32_t FULL4:1;                 /* FIFO Full for channel 4 */
    vuint32_t _unused_18:2;
    vuint32_t EMPTY5:1;                /* FIFO Empty for channel 5 */
    vuint32_t FULL5:1;                 /* FIFO Full for channel 5 */
    vuint32_t _unused_22:2;
    vuint32_t EMPTY6:1;                /* FIFO Empty for channel 6 */
    vuint32_t FULL6:1;                 /* FIFO Full for channel 6 */
    vuint32_t _unused_26:2;
    vuint32_t EMPTY7:1;                /* FIFO Empty for channel 7 */
    vuint32_t FULL7:1;                 /* FIFO Full for channel 7 */
    vuint32_t _unused_30:2;
  } B;
} RXLUT_RLSOFSR0_tag;

typedef union RXLUT_RLSOFSR1_union_tag { /* RXLUT Search Output FIFO Status */
  vuint32_t R;
  struct {
    vuint32_t EMPTY8:1;                /* FIFO Empty for channel 8 */
    vuint32_t FULL8:1;                 /* FIFO Full for channel 8 */
    vuint32_t _unused_2:2;
    vuint32_t EMPTY9:1;                /* FIFO Empty for channel 9 */
    vuint32_t FULL9:1;                 /* FIFO Full for channel 9 */
    vuint32_t _unused_6:2;
    vuint32_t EMPTY10:1;               /* FIFO Empty for channel 10 */
    vuint32_t FULL10:1;                /* FIFO Full for channel 10 */
    vuint32_t _unused_10:2;
    vuint32_t EMPTY11:1;               /* FIFO Empty for channel 11 */
    vuint32_t FULL11:1;                /* FIFO Full for channel 11 */
    vuint32_t _unused_14:2;
    vuint32_t _unused_16:16;
  } B;
} RXLUT_RLSOFSR1_tag;

typedef union RXLUT_RLICEC0_union_tag { /* RXLUT Input Command Entry Counter */
  vuint32_t R;
  struct {
    vuint32_t CNTR0:4;                 /* Input command entry counter for channel 0 */
    vuint32_t CNTR1:4;                 /* Input command entry counter for channel 1 */
    vuint32_t CNTR2:4;                 /* Input command entry counter for channel 2 */
    vuint32_t CNTR3:4;                 /* Input command entry counter for channel 3 */
    vuint32_t CNTR4:4;                 /* Input command entry counter for channel 4 */
    vuint32_t CNTR5:4;                 /* Input command entry counter for channel 5 */
    vuint32_t CNTR6:4;                 /* Input command entry counter for channel 6 */
    vuint32_t CNTR7:4;                 /* Input command entry counter for channel 7 */
  } B;
} RXLUT_RLICEC0_tag;

typedef union RXLUT_RLICEC1_union_tag { /* RXLUT Input Command Entry Counter */
  vuint32_t R;
  struct {
    vuint32_t CNTR8:4;                 /* Input command entry counter for channel 8 */
    vuint32_t CNTR9:4;                 /* Input command entry counter for channel 9 */
    vuint32_t CNTR10:4;                /* Input command entry counter for channel 10 */
    vuint32_t CNTR11:4;                /* Input command entry counter for channel 11 */
    vuint32_t _unused_16:16;
  } B;
} RXLUT_RLICEC1_tag;

typedef union RXLUT_RLER_union_tag {   /* RXLUT Error */
  vuint32_t R;
  struct {
    vuint32_t SBIT:1;                  /* Single bit ECC error */
    vuint32_t _unused_1:15;
    vuint32_t MBIT:1;                  /* Multi bit ECC error */
    vuint32_t _unused_17:15;
  } B;
} RXLUT_RLER_tag;

typedef union RXLUT_RLOOR_union_tag {  /* RXLUT Output Overflow */
  vuint32_t R;
  struct {
    vuint32_t LOR:12;                  /* Lookup overrun */
    vuint32_t _unused_12:4;
    vuint32_t _unused_16:16;
  } B;
} RXLUT_RLOOR_tag;

struct RXLUT_tag {
  RXLUT_RLCR_tag RLCR;                 /* RXLUT Configuration */
  RXLUT_RLSR_tag RLSR;                 /* RXLUT Status */
  RXLUT_RLSRIER_tag RLSRIER;           /* RXLUT Search Result Interrupt Enable */
  RXLUT_RLSRSR_tag RLSRSR;             /* RXLUT Search Result Status */
  RXLUT_RLFIMER_tag RLFIMER;           /* RXLUT Frame ID End */
  RXLUT_RLFISR_tag RLFISR;             /* RXLUT Frame ID Start */
  RXLUT_RLCIXR_tag RLCIXR;             /* RXLUT Channel ID AND Index */
  RXLUT_RLADRCR_tag RLADRCR;           /* RXLUT Address and Command */
  RXLUT_RLFIMESR_tag RLFIMESR;         /* RXLUT Frame ID End Status */
  RXLUT_RLFISSR_tag RLFISSR;           /* RXLUT Frame ID Start Status */
  RXLUT_RLCIXSR_tag RLCIXSR;           /* RXLUT Channel ID AND Index Status */
  RXLUT_RLADRSR_tag RLADRSR;           /* RXLUT Address Status */
  uint8_t RXLUT_reserved0[16];
  RXLUT_RLSICR_tag RLSICR;             /* RXLUT Search Input Command */
  RXLUT_RLSICFIR_tag RLSICFIR;         /* RXLUT Search Input Command Frame ID */
  RXLUT_RLSOIXR_tag RLSOIXR[12];
  uint8_t RXLUT_reserved1[168];
  RXLUT_RLSICFSR_tag RLSICFSR;         /* RXLUT Search Input Command FIFO Status */
  RXLUT_RLSOFSR0_tag RLSOFSR0;         /* RXLUT Search Output FIFO Status */
  RXLUT_RLSOFSR1_tag RLSOFSR1;         /* RXLUT Search Output FIFO Status */
  RXLUT_RLICEC0_tag RLICEC0;           /* RXLUT Input Command Entry Counter */
  RXLUT_RLICEC1_tag RLICEC1;           /* RXLUT Input Command Entry Counter */
  RXLUT_RLER_tag RLER;                 /* RXLUT Error */
  RXLUT_RLOOR_tag RLOOR;               /* RXLUT Output Overflow */
};


/* ============================================================================
   =============================== Module: SBSW ===============================
   ============================================================================ */

typedef union SBSW_TMC_CONFIG_UNLOCK_union_tag { /* TMC Configuration Unlock */
  vuint32_t R;
  struct {
    vuint32_t KEY:32;                  /* Key */
  } B;
} SBSW_TMC_CONFIG_UNLOCK_tag;

typedef union SBSW_TMWDP_CONFIG_UNLOCK_union_tag { /* TMWDP Configuration Unlock */
  vuint32_t R;
  struct {
    vuint32_t KEY:32;                  /* Key */
  } B;
} SBSW_TMWDP_CONFIG_UNLOCK_tag;

typedef union SBSW_TMC_CONFIG_STATUS_union_tag { /* TMC Configuration Status */
  vuint32_t R;
  struct {
    vuint32_t STATUS:1;                /* Status */
    vuint32_t _unused_1:31;
  } B;
} SBSW_TMC_CONFIG_STATUS_tag;

typedef union SBSW_TMWDP_CONFIG_STATUS_union_tag { /* TMWDP Configuration Status */
  vuint32_t R;
  struct {
    vuint32_t STATUS:1;                /* Status */
    vuint32_t _unused_1:31;
  } B;
} SBSW_TMWDP_CONFIG_STATUS_tag;

typedef union SBSW_DEBUG_MODE_union_tag { /* Debug Mode */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t MODE:1;                  /* Mode */
  } B;
} SBSW_DEBUG_MODE_tag;

typedef union SBSW_TMC_FAULT_STATUS_union_tag { /* TMC Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STATUS0:1;               /* Status n */
    vuint32_t STATUS1:1;               /* Status n */
    vuint32_t STATUS2:1;               /* Status n */
    vuint32_t STATUS3:1;               /* Status n */
    vuint32_t STATUS4:1;               /* Status n */
    vuint32_t STATUS5:1;               /* Status n */
    vuint32_t STATUS6:1;               /* Status n */
    vuint32_t STATUS7:1;               /* Status n */
    vuint32_t STATUS8:1;               /* Status n */
    vuint32_t STATUS9:1;               /* Status n */
    vuint32_t STATUS10:1;              /* Status n */
    vuint32_t STATUS11:1;              /* Status n */
    vuint32_t STATUS12:1;              /* Status n */
    vuint32_t STATUS13:1;              /* Status n */
    vuint32_t STATUS14:1;              /* Status n */
    vuint32_t STATUS15:1;              /* Status n */
    vuint32_t STATUS16:1;              /* Status n */
    vuint32_t STATUS17:1;              /* Status n */
    vuint32_t STATUS18:1;              /* Status n */
    vuint32_t STATUS19:1;              /* Status n */
    vuint32_t STATUS20:1;              /* Status n */
    vuint32_t STATUS21:1;              /* Status n */
    vuint32_t STATUS22:1;              /* Status n */
    vuint32_t STATUS23:1;              /* Status n */
    vuint32_t STATUS24:1;              /* Status n */
    vuint32_t STATUS25:1;              /* Status n */
    vuint32_t STATUS26:1;              /* Status n */
    vuint32_t STATUS27:1;              /* Status n */
    vuint32_t STATUS28:1;              /* Status n */
    vuint32_t STATUS29:1;              /* Status n */
    vuint32_t STATUS30:1;              /* Status n */
    vuint32_t STATUS31:1;              /* Status n */
  } B;
} SBSW_TMC_FAULT_STATUS_tag;

typedef union SBSW_TMWDP_FAULT_STATUS_union_tag { /* TMWDP Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STATUS0:1;               /* Status n */
    vuint32_t STATUS1:1;               /* Status n */
    vuint32_t STATUS2:1;               /* Status n */
    vuint32_t STATUS3:1;               /* Status n */
    vuint32_t STATUS4:1;               /* Status n */
    vuint32_t STATUS5:1;               /* Status n */
    vuint32_t STATUS6:1;               /* Status n */
    vuint32_t STATUS7:1;               /* Status n */
    vuint32_t STATUS8:1;               /* Status n */
    vuint32_t STATUS9:1;               /* Status n */
    vuint32_t STATUS10:1;              /* Status n */
    vuint32_t STATUS11:1;              /* Status n */
    vuint32_t STATUS12:1;              /* Status n */
    vuint32_t STATUS13:1;              /* Status n */
    vuint32_t STATUS14:1;              /* Status n */
    vuint32_t STATUS15:1;              /* Status n */
    vuint32_t STATUS16:1;              /* Status n */
    vuint32_t STATUS17:1;              /* Status n */
    vuint32_t STATUS18:1;              /* Status n */
    vuint32_t STATUS19:1;              /* Status n */
    vuint32_t STATUS20:1;              /* Status n */
    vuint32_t STATUS21:1;              /* Status n */
    vuint32_t STATUS22:1;              /* Status n */
    vuint32_t STATUS23:1;              /* Status n */
    vuint32_t STATUS24:1;              /* Status n */
    vuint32_t STATUS25:1;              /* Status n */
    vuint32_t STATUS26:1;              /* Status n */
    vuint32_t STATUS27:1;              /* Status n */
    vuint32_t STATUS28:1;              /* Status n */
    vuint32_t STATUS29:1;              /* Status n */
    vuint32_t STATUS30:1;              /* Status n */
    vuint32_t STATUS31:1;              /* Status n */
  } B;
} SBSW_TMWDP_FAULT_STATUS_tag;

typedef union SBSW_TMC_CONFIG_union_tag { /* TMC Configuration */
  vuint32_t R;
  struct {
    vuint32_t TIMING_MODE:1;           /* Timing mode */
    vuint32_t COMPARE_MODE:2;          /* Compare mode */
    vuint32_t _unused_3:29;
  } B;
} SBSW_TMC_CONFIG_tag;

typedef union SBSW_TMC_DISTANCE_union_tag { /* TMC Distance */
  vuint32_t R;
  struct {
    vuint32_t DISTANCE:32;             /* Distance value */
  } B;
} SBSW_TMC_DISTANCE_tag;

typedef union SBSW_TMC_TIMEOUT_union_tag { /* TMC Timeout */
  vuint32_t R;
  struct {
    vuint32_t TIMEOUT:32;              /* Timeout interval */
  } B;
} SBSW_TMC_TIMEOUT_tag;

typedef union SBSW_TMC_CONTROL_union_tag { /* TMC Control */
  vuint32_t R;
  struct {
    vuint32_t ENABLE:1;                /* Enable */
    vuint32_t _unused_1:31;
  } B;
} SBSW_TMC_CONTROL_tag;

typedef union SBSW_TMC_R0_union_tag {  /* TMC R0 Data */
  vuint32_t R;
  struct {
    vuint32_t R0:32;                   /* R0 data value */
  } B;
} SBSW_TMC_R0_tag;

typedef union SBSW_TMC_R1_union_tag {  /* TMC R1 Data */
  vuint32_t R;
  struct {
    vuint32_t R1:32;                   /* R1 data value */
  } B;
} SBSW_TMC_R1_tag;

typedef union SBSW_TMC_STATUS_union_tag { /* TMC Status */
  vuint32_t R;
  struct {
    vuint32_t R0_WRITTEN:1;            /* R0 written */
    vuint32_t R1_WRITTEN:1;            /* R1 written */
    vuint32_t FAULT:2;                 /* Fault indicator */
    vuint32_t OVERWRITTEN:1;           /* Overwritten */
    vuint32_t _unused_5:27;
  } B;
} SBSW_TMC_STATUS_tag;

typedef union SBSW_TMC_TIMER_union_tag { /* TMC Timer */
  vuint32_t R;
  struct {
    vuint32_t TIMER:32;                /* Timer value */
  } B;
} SBSW_TMC_TIMER_tag;

typedef struct SBSW_TMC_struct_tag {
  SBSW_TMC_CONFIG_tag CONFIG;          /* TMC Configuration */
  SBSW_TMC_DISTANCE_tag DISTANCE;      /* TMC Distance */
  SBSW_TMC_TIMEOUT_tag TIMEOUT;        /* TMC Timeout */
  SBSW_TMC_CONTROL_tag CONTROL;        /* TMC Control */
  SBSW_TMC_R0_tag R0;                  /* TMC R0 Data */
  SBSW_TMC_R1_tag R1;                  /* TMC R1 Data */
  SBSW_TMC_STATUS_tag STATUS;          /* TMC Status */
  SBSW_TMC_TIMER_tag TIMER;            /* TMC Timer */
} SBSW_TMC_tag;

typedef union SBSW_TMWDP_CONFIG_ADDR_union_tag { /* TMWDP Configuration Address */
  vuint32_t R;
  struct {
    vuint32_t ADDRESS:32;              /* Configuration address */
  } B;
} SBSW_TMWDP_CONFIG_ADDR_tag;

typedef union SBSW_TMWDP_CONTROL_union_tag { /* TMWDP Control */
  vuint32_t R;
  struct {
    vuint32_t ENABLE:1;                /* Enable */
    vuint32_t _unused_1:31;
  } B;
} SBSW_TMWDP_CONTROL_tag;

typedef union SBSW_TMWDP_STATUS_union_tag { /* TMWDP Status */
  vuint32_t R;
  struct {
    vuint32_t INTERNAL_ERROR:1;        /* TMWDP internal error */
    vuint32_t CONFIG_ERROR:1;          /* Configuration error */
    vuint32_t RUNNING:1;               /* TMWDP running */
    vuint32_t _unused_3:29;
  } B;
} SBSW_TMWDP_STATUS_tag;

typedef union SBSW_TMWDP_AUTOMATA_STATUS_union_tag { /* TMWDP Automata Status */
  vuint32_t R;
  struct {
    vuint32_t STATUS0:1;               /* Status n */
    vuint32_t STATUS1:1;               /* Status n */
    vuint32_t STATUS2:1;               /* Status n */
    vuint32_t STATUS3:1;               /* Status n */
    vuint32_t STATUS4:1;               /* Status n */
    vuint32_t STATUS5:1;               /* Status n */
    vuint32_t STATUS6:1;               /* Status n */
    vuint32_t STATUS7:1;               /* Status n */
    vuint32_t STATUS8:1;               /* Status n */
    vuint32_t STATUS9:1;               /* Status n */
    vuint32_t STATUS10:1;              /* Status n */
    vuint32_t STATUS11:1;              /* Status n */
    vuint32_t STATUS12:1;              /* Status n */
    vuint32_t STATUS13:1;              /* Status n */
    vuint32_t STATUS14:1;              /* Status n */
    vuint32_t STATUS15:1;              /* Status n */
    vuint32_t STATUS16:1;              /* Status n */
    vuint32_t STATUS17:1;              /* Status n */
    vuint32_t STATUS18:1;              /* Status n */
    vuint32_t STATUS19:1;              /* Status n */
    vuint32_t STATUS20:1;              /* Status n */
    vuint32_t STATUS21:1;              /* Status n */
    vuint32_t STATUS22:1;              /* Status n */
    vuint32_t STATUS23:1;              /* Status n */
    vuint32_t STATUS24:1;              /* Status n */
    vuint32_t STATUS25:1;              /* Status n */
    vuint32_t STATUS26:1;              /* Status n */
    vuint32_t STATUS27:1;              /* Status n */
    vuint32_t STATUS28:1;              /* Status n */
    vuint32_t STATUS29:1;              /* Status n */
    vuint32_t STATUS30:1;              /* Status n */
    vuint32_t STATUS31:1;              /* Status n */
  } B;
} SBSW_TMWDP_AUTOMATA_STATUS_tag;

typedef union SBSW_TMWDP_AUTOMATA_ILLGL_TRANS_union_tag { /* TMWDP Automata Illegal Transition */
  vuint32_t R;
  struct {
    vuint32_t STATUS0:1;               /* Status n */
    vuint32_t STATUS1:1;               /* Status n */
    vuint32_t STATUS2:1;               /* Status n */
    vuint32_t STATUS3:1;               /* Status n */
    vuint32_t STATUS4:1;               /* Status n */
    vuint32_t STATUS5:1;               /* Status n */
    vuint32_t STATUS6:1;               /* Status n */
    vuint32_t STATUS7:1;               /* Status n */
    vuint32_t STATUS8:1;               /* Status n */
    vuint32_t STATUS9:1;               /* Status n */
    vuint32_t STATUS10:1;              /* Status n */
    vuint32_t STATUS11:1;              /* Status n */
    vuint32_t STATUS12:1;              /* Status n */
    vuint32_t STATUS13:1;              /* Status n */
    vuint32_t STATUS14:1;              /* Status n */
    vuint32_t STATUS15:1;              /* Status n */
    vuint32_t STATUS16:1;              /* Status n */
    vuint32_t STATUS17:1;              /* Status n */
    vuint32_t STATUS18:1;              /* Status n */
    vuint32_t STATUS19:1;              /* Status n */
    vuint32_t STATUS20:1;              /* Status n */
    vuint32_t STATUS21:1;              /* Status n */
    vuint32_t STATUS22:1;              /* Status n */
    vuint32_t STATUS23:1;              /* Status n */
    vuint32_t STATUS24:1;              /* Status n */
    vuint32_t STATUS25:1;              /* Status n */
    vuint32_t STATUS26:1;              /* Status n */
    vuint32_t STATUS27:1;              /* Status n */
    vuint32_t STATUS28:1;              /* Status n */
    vuint32_t STATUS29:1;              /* Status n */
    vuint32_t STATUS30:1;              /* Status n */
    vuint32_t STATUS31:1;              /* Status n */
  } B;
} SBSW_TMWDP_AUTOMATA_ILLGL_TRANS_tag;

typedef union SBSW_TMWDP_AUTOMATA_TIME_VIOLATION_union_tag { /* TMWDP Automata Time Violation */
  vuint32_t R;
  struct {
    vuint32_t STATUS0:1;               /* Status n */
    vuint32_t STATUS1:1;               /* Status n */
    vuint32_t STATUS2:1;               /* Status n */
    vuint32_t STATUS3:1;               /* Status n */
    vuint32_t STATUS4:1;               /* Status n */
    vuint32_t STATUS5:1;               /* Status n */
    vuint32_t STATUS6:1;               /* Status n */
    vuint32_t STATUS7:1;               /* Status n */
    vuint32_t STATUS8:1;               /* Status n */
    vuint32_t STATUS9:1;               /* Status n */
    vuint32_t STATUS10:1;              /* Status n */
    vuint32_t STATUS11:1;              /* Status n */
    vuint32_t STATUS12:1;              /* Status n */
    vuint32_t STATUS13:1;              /* Status n */
    vuint32_t STATUS14:1;              /* Status n */
    vuint32_t STATUS15:1;              /* Status n */
    vuint32_t STATUS16:1;              /* Status n */
    vuint32_t STATUS17:1;              /* Status n */
    vuint32_t STATUS18:1;              /* Status n */
    vuint32_t STATUS19:1;              /* Status n */
    vuint32_t STATUS20:1;              /* Status n */
    vuint32_t STATUS21:1;              /* Status n */
    vuint32_t STATUS22:1;              /* Status n */
    vuint32_t STATUS23:1;              /* Status n */
    vuint32_t STATUS24:1;              /* Status n */
    vuint32_t STATUS25:1;              /* Status n */
    vuint32_t STATUS26:1;              /* Status n */
    vuint32_t STATUS27:1;              /* Status n */
    vuint32_t STATUS28:1;              /* Status n */
    vuint32_t STATUS29:1;              /* Status n */
    vuint32_t STATUS30:1;              /* Status n */
    vuint32_t STATUS31:1;              /* Status n */
  } B;
} SBSW_TMWDP_AUTOMATA_TIME_VIOLATION_tag;

typedef union SBSW_AUT_STATUS_union_tag { /* TMWDP Automaton Status */
  vuint32_t R;
  struct {
    vuint32_t CURRENT_STATE:16;        /* Automaton current state */
    vuint32_t STATUS:1;                /* Automaton status */
    vuint32_t ILLEGAL_TRANSITION:1;    /* Illegal transition */
    vuint32_t TIME_VIOLATION:1;        /* Time violation */
    vuint32_t _unused_19:13;
  } B;
} SBSW_AUT_STATUS_tag;

typedef union SBSW_AUT_PRGS_REQ_union_tag { /* TMWDP Automaton Progress Request */
  vuint32_t R;
  struct {
    vuint32_t PROGRESS_REQ:16;         /* Application progress request */
    vuint32_t WRITTEN:1;               /* Written */
    vuint32_t OVERWRITTEN:1;           /* Overwritten */
    vuint32_t _unused_18:14;
  } B;
} SBSW_AUT_PRGS_REQ_tag;

typedef struct SBSW_AUT_struct_tag {
  SBSW_AUT_STATUS_tag STATUS;          /* TMWDP Automaton Status */
  SBSW_AUT_PRGS_REQ_tag PRGS_REQ;      /* TMWDP Automaton Progress Request */
} SBSW_AUT_tag;

typedef union SBSW_TC_ID_union_tag {   /* TMWDP Core Domain ID */
  vuint32_t R;
  struct {
    vuint32_t ID:4;                    /* TC domain ID value */
    vuint32_t _unused_4:4;
    vuint32_t WRITTEN:1;               /* TC ID status */
    vuint32_t _unused_9:23;
  } B;
} SBSW_TC_ID_tag;

struct SBSW_tag {
  SBSW_TMC_CONFIG_UNLOCK_tag TMC_CONFIG_UNLOCK; /* TMC Configuration Unlock */
  SBSW_TMWDP_CONFIG_UNLOCK_tag TMWDP_CONFIG_UNLOCK; /* TMWDP Configuration Unlock */
  SBSW_TMC_CONFIG_STATUS_tag TMC_CONFIG_STATUS; /* TMC Configuration Status */
  SBSW_TMWDP_CONFIG_STATUS_tag TMWDP_CONFIG_STATUS; /* TMWDP Configuration Status */
  SBSW_DEBUG_MODE_tag DEBUG_MODE;      /* Debug Mode */
  SBSW_TMC_FAULT_STATUS_tag TMC_FAULT_STATUS; /* TMC Fault Status */
  SBSW_TMWDP_FAULT_STATUS_tag TMWDP_FAULT_STATUS; /* TMWDP Fault Status */
  uint8_t SBSW_reserved0[4];
  SBSW_TMC_tag TMC[32];
  SBSW_TMWDP_CONFIG_ADDR_tag TMWDP_CONFIG_ADDR; /* TMWDP Configuration Address */
  SBSW_TMWDP_CONTROL_tag TMWDP_CONTROL; /* TMWDP Control */
  SBSW_TMWDP_STATUS_tag TMWDP_STATUS;  /* TMWDP Status */
  SBSW_TMWDP_AUTOMATA_STATUS_tag TMWDP_AUTOMATA_STATUS; /* TMWDP Automata Status */
  SBSW_TMWDP_AUTOMATA_ILLGL_TRANS_tag TMWDP_AUTOMATA_ILLGL_TRANS; /* TMWDP Automata Illegal Transition */
  SBSW_TMWDP_AUTOMATA_TIME_VIOLATION_tag TMWDP_AUTOMATA_TIME_VIOLATION; /* TMWDP Automata Time Violation */
  uint8_t SBSW_reserved1[24];
  SBSW_AUT_tag AUT[32];
  uint8_t SBSW_reserved2[2732];
  SBSW_TC_ID_tag TC_ID;                /* TMWDP Core Domain ID */
};


/* ============================================================================
   =============================== Module: SDA_AP =============================
   ============================================================================ */

typedef union SDA_AP_AUTHSTTS_union_tag { /* Authentication status register */
  vuint32_t R;
  struct {
    vuint32_t CHALRDY:1;               /* Challenge ready */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t UIDRDY:1;                /* UID Ready */
    vuint32_t _unused_3:27;
    vuint32_t APPDBGEN:1;              /* Application Debug enabled */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SDA_AP_AUTHSTTS_tag;

typedef union SDA_AP_AUTHCTL_union_tag { /* Authentication Control Register */
  vuint32_t R;
  struct {
    vuint32_t HSEAUTHREQ:1;            /* HSE authentication request */
    vuint32_t _unused_1:31;
  } B;
} SDA_AP_AUTHCTL_tag;

typedef union SDA_AP_KEYCHAL0_union_tag { /* Key Challenge 0 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL0:32;              /* HSE Challenge 0 register */
  } B;
} SDA_AP_KEYCHAL0_tag;

typedef union SDA_AP_KEYCHAL1_union_tag { /* Key Challenge 1 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL1:32;              /* HSE Challenge 1 register */
  } B;
} SDA_AP_KEYCHAL1_tag;

typedef union SDA_AP_KEYCHAL2_union_tag { /* Key Challenge 2 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL2:32;              /* HSE Challenge 2 register */
  } B;
} SDA_AP_KEYCHAL2_tag;

typedef union SDA_AP_KEYCHAL3_union_tag { /* Key Challenge 3 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL3:32;              /* HSE Challenge 3 register */
  } B;
} SDA_AP_KEYCHAL3_tag;

typedef union SDA_AP_KEYCHAL4_union_tag { /* Key Challenge 4 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL4:32;              /* HSE Challenge 4 register */
  } B;
} SDA_AP_KEYCHAL4_tag;

typedef union SDA_AP_KEYCHAL5_union_tag { /* Key Challenge 5 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL5:32;              /* HSE Challenge 5 register */
  } B;
} SDA_AP_KEYCHAL5_tag;

typedef union SDA_AP_KEYCHAL6_union_tag { /* Key Challenge 6 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL6:32;              /* HSE Challenge 6 register */
  } B;
} SDA_AP_KEYCHAL6_tag;

typedef union SDA_AP_KEYCHAL7_union_tag { /* Key Challenge 7 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL7:32;              /* HSE Challenge 7 register */
  } B;
} SDA_AP_KEYCHAL7_tag;

typedef union SDA_AP_KEYRESP0_union_tag { /* Key Response 0 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL0:32;              /* HSE Response 0 register */
  } B;
} SDA_AP_KEYRESP0_tag;

typedef union SDA_AP_KEYRESP1_union_tag { /* Key Response 1 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL1:32;              /* HSE Response 1 register */
  } B;
} SDA_AP_KEYRESP1_tag;

typedef union SDA_AP_KEYRESP2_union_tag { /* Key Response 2 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL2:32;              /* HSE Response 2 register */
  } B;
} SDA_AP_KEYRESP2_tag;

typedef union SDA_AP_KEYRESP3_union_tag { /* Key Response 3 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL3:32;              /* HSE Response 3 register */
  } B;
} SDA_AP_KEYRESP3_tag;

typedef union SDA_AP_KEYRESP4_union_tag { /* Key Response 4 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL4:32;              /* HSE Response 4 register */
  } B;
} SDA_AP_KEYRESP4_tag;

typedef union SDA_AP_KEYRESP5_union_tag { /* Key Response 5 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL5:32;              /* HSE Response 5 register */
  } B;
} SDA_AP_KEYRESP5_tag;

typedef union SDA_AP_KEYRESP6_union_tag { /* Key Response 6 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL6:32;              /* HSE Response 6 register */
  } B;
} SDA_AP_KEYRESP6_tag;

typedef union SDA_AP_KEYRESP7_union_tag { /* Key Response 7 register */
  vuint32_t R;
  struct {
    vuint32_t KEYCAL7:32;              /* HSE Response 7 register */
  } B;
} SDA_AP_KEYRESP7_tag;

typedef union SDA_AP_SDAUIDL_union_tag { /* UID Low Register */
  vuint32_t R;
  struct {
    vuint32_t SDAUIDL:32;              /* UID Low */
  } B;
} SDA_AP_SDAUIDL_tag;

typedef union SDA_AP_SDAUIDH_union_tag { /* UID High Register */
  vuint32_t R;
  struct {
    vuint32_t SDAUIDH:32;              /* UID High */
  } B;
} SDA_AP_SDAUIDH_tag;

typedef union SDA_AP_SDASYSRSTS_union_tag { /* System reset status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t DESTRUCT_RESET:1;        /* Indicates destructive reset state */
    vuint32_t SYSTEM_RESET:1;          /* Indicates system (functional) reset state */
    vuint32_t _unused_3:29;
  } B;
} SDA_AP_SDASYSRSTS_tag;

typedef union SDA_AP_SDASYSRSTC_union_tag { /* System Reset Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t SYSRESTREQ:1;            /* System destructive reset */
    vuint32_t SYSFUNCREQ:1;            /* System functional reset */
    vuint32_t _unused_6:26;
  } B;
} SDA_AP_SDASYSRSTC_tag;

typedef union SDA_AP_SDARSTCTRL_union_tag { /* Reset Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t RSTWAITCEB:1;            /* Reset release for CE Cortex-M33 Core 1 */
    vuint32_t RSTWAITCEA:1;            /* Reset release for CE Cortex-M33 Core 0 */
    vuint32_t _unused_13:2;
    vuint32_t RSTWAITSMU:1;            /* Reset release for SMU Cortex-M33 core */
    vuint32_t RSTRELSPF2_SYS:1;        /* Reset release for Ceva-SensPro Float 2 (CEVA_SPF2) system */
    vuint32_t RSTRELSPF2_CORE:1;       /* Reset release for Ceva-SensPro Float 2 (CEVA_SPF2) core */
    vuint32_t _unused_18:2;
    vuint32_t RSTRELRTU1_C3:1;         /* Reset release for RTU1 core 3 */
    vuint32_t RSTRELRTU1_C2:1;         /* Reset release for RTU1 Core 2 */
    vuint32_t RSTRELRTU1_C1:1;         /* Reset release for RTU1 core 1 */
    vuint32_t RSTREL_RTU1_C0:1;        /* Reset release for RTU1 Core 0 */
    vuint32_t _unused_24:4;
    vuint32_t RSTRELRTU0_C3:1;         /* Reset release for RTU0 Core 3 */
    vuint32_t RSTRELRTU0_C2:1;         /* Reset release for RTU0 Core 2 */
    vuint32_t RSTRELRTU0_C1:1;         /* Reset release for RTU0 Core 1 */
    vuint32_t RSTRELRTU0_C0:1;         /* Reset release for RTU0 Core 0 */
  } B;
} SDA_AP_SDARSTCTRL_tag;

typedef union SDA_AP_SDARSTMASK_union_tag { /* Reset Control Mask Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:19;
    vuint32_t RSTMSKRTU1_POR:1;        /* Reset Mask for RTU1 Subsystem */
    vuint32_t RSTMSKRTU1_C3:1;         /* Reset Mask for RTU1 Core 3 */
    vuint32_t RSTMSKRTU1_C2:1;         /* Reset Mask for RTU1 Core 2 */
    vuint32_t RSTMSKRTU1_C1:1;         /* Reset Mask for RTU1 Core 1 */
    vuint32_t RSTMSK_RTU1_C0:1;        /* Reset Mask for RTU1 Core 0 */
    vuint32_t _unused_24:3;
    vuint32_t RSTMSKRTU0_POR:1;        /* Reset Mask for RTU0 Subsystem */
    vuint32_t RSTMSKRTU0_C3:1;         /* Reset Mask for RTU0 Core 3 */
    vuint32_t RSTMSKRTU0_C2:1;         /* Reset Mask for RTU0 Core 2 */
    vuint32_t RSTMSKRTU0_C1:1;         /* Reset Mask for RTU0 Core 1 */
    vuint32_t RSTMSKRTU0_C0:1;         /* Reset Mask for RTU0 Core 0 */
  } B;
} SDA_AP_SDARSTMASK_tag;

typedef union SDA_AP_IDR_union_tag {   /* Identification Register */
  vuint32_t R;
  struct {
    vuint32_t TYPE:4;                  /* Indicates Type of component */
    vuint32_t VARIANT:4;               /* Variation from base type specified by IDR TYPE */
    vuint32_t _unused_8:5;
    vuint32_t CLASS:4;                 /* Class */
    vuint32_t JEDEC_CODE:7;            /* The JEP106 identification code */
    vuint32_t JEDEC_BANK:4;            /* The JEP106 continuation code */
    vuint32_t REVISION:4;              /* Revision */
  } B;
} SDA_AP_IDR_tag;

typedef union SDA_AP_DEVARCH_union_tag { /* CoreSight Device Architecture Register */
  vuint32_t R;
  struct {
    vuint32_t ARCHID:16;               /* Architecture ID */
    vuint32_t REVISION:4;              /* Architecture revision */
    vuint32_t PRESENT:1;               /* DEVARCH register is present */
    vuint32_t ARCHITECT:11;            /* Architect of the component - JEP106 code */
  } B;
} SDA_AP_DEVARCH_tag;

typedef union SDA_AP_DEVTYPE_union_tag { /* CoreSight Device Type Identifier Register */
  vuint32_t R;
  struct {
    vuint32_t MAJOR:4;                 /* Major Classification */
    vuint32_t SUB:4;                   /* Minor Classification */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_DEVTYPE_tag;

typedef union SDA_AP_PIDR4_union_tag { /* CoreSight Peripheral Identification Register 4 */
  vuint32_t R;
  struct {
    vuint32_t DES_2:4;                 /* DES_2 */
    vuint32_t SIZE:4;                  /* SIZE */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_PIDR4_tag;

typedef union SDA_AP_PIDR0_union_tag { /* CoreSight Peripheral Identification Register 0 */
  vuint32_t R;
  struct {
    vuint32_t PART_0:8;                /* PART_0 */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_PIDR0_tag;

typedef union SDA_AP_PIDR1_union_tag { /* CoreSight Peripheral Identification Register 1 */
  vuint32_t R;
  struct {
    vuint32_t PART_1:4;                /* PART_1 */
    vuint32_t DES_0:4;                 /* DES_0 */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_PIDR1_tag;

typedef union SDA_AP_PIDR2_union_tag { /* CoreSight Peripheral Identification Register 2 */
  vuint32_t R;
  struct {
    vuint32_t DES_1:3;                 /* DES_1 */
    vuint32_t JEDEC:1;                 /* JEDEC */
    vuint32_t REVISION:4;              /* Revision */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_PIDR2_tag;

typedef union SDA_AP_PIDR3_union_tag { /* CoreSight Peripheral Identification Register 3 */
  vuint32_t R;
  struct {
    vuint32_t CMOD:4;                  /* Customer Modified */
    vuint32_t REVAND:4;                /* REVAND */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_PIDR3_tag;

typedef union SDA_AP_CIDR0_union_tag { /* CoreSight Component Identification Register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_0:8;               /* Preamble */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_CIDR0_tag;

typedef union SDA_AP_CIDR1_union_tag { /* CoreSight Component Identification Register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_1:4;               /* PRMBL_1 */
    vuint32_t CLASS:4;                 /* Component Class */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_CIDR1_tag;

typedef union SDA_AP_CIDR2_union_tag { /* CoreSight Component Identification Register 2 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_2:8;               /* PRMBL_2 */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_CIDR2_tag;

typedef union SDA_AP_CIDR3_union_tag { /* CoreSight Component Identification Register 3 */
  vuint32_t R;
  struct {
    vuint32_t PRMBL_3:8;               /* PRMBL_3 */
    vuint32_t _unused_8:24;
  } B;
} SDA_AP_CIDR3_tag;

struct SDA_AP_tag {
  SDA_AP_AUTHSTTS_tag AUTHSTTS;        /* Authentication status register */
  SDA_AP_AUTHCTL_tag AUTHCTL;          /* Authentication Control Register */
  uint8_t SDA_AP_reserved0[8];
  SDA_AP_KEYCHAL0_tag KEYCHAL0;        /* Key Challenge 0 register */
  SDA_AP_KEYCHAL1_tag KEYCHAL1;        /* Key Challenge 1 register */
  SDA_AP_KEYCHAL2_tag KEYCHAL2;        /* Key Challenge 2 register */
  SDA_AP_KEYCHAL3_tag KEYCHAL3;        /* Key Challenge 3 register */
  SDA_AP_KEYCHAL4_tag KEYCHAL4;        /* Key Challenge 4 register */
  SDA_AP_KEYCHAL5_tag KEYCHAL5;        /* Key Challenge 5 register */
  SDA_AP_KEYCHAL6_tag KEYCHAL6;        /* Key Challenge 6 register */
  SDA_AP_KEYCHAL7_tag KEYCHAL7;        /* Key Challenge 7 register */
  SDA_AP_KEYRESP0_tag KEYRESP0;        /* Key Response 0 register */
  SDA_AP_KEYRESP1_tag KEYRESP1;        /* Key Response 1 register */
  SDA_AP_KEYRESP2_tag KEYRESP2;        /* Key Response 2 register */
  SDA_AP_KEYRESP3_tag KEYRESP3;        /* Key Response 3 register */
  SDA_AP_KEYRESP4_tag KEYRESP4;        /* Key Response 4 register */
  SDA_AP_KEYRESP5_tag KEYRESP5;        /* Key Response 5 register */
  SDA_AP_KEYRESP6_tag KEYRESP6;        /* Key Response 6 register */
  SDA_AP_KEYRESP7_tag KEYRESP7;        /* Key Response 7 register */
  uint8_t SDA_AP_reserved1[32];
  SDA_AP_SDAUIDL_tag SDAUIDL;          /* UID Low Register */
  SDA_AP_SDAUIDH_tag SDAUIDH;          /* UID High Register */
  uint8_t SDA_AP_reserved2[8];
  SDA_AP_SDASYSRSTS_tag SDASYSRSTS;    /* System reset status register */
  uint8_t SDA_AP_reserved3[4];
  SDA_AP_SDASYSRSTC_tag SDASYSRSTC;    /* System Reset Control Register */
  uint8_t SDA_AP_reserved4[4];
  SDA_AP_SDARSTCTRL_tag SDARSTCTRL;    /* Reset Control Register */
  SDA_AP_SDARSTMASK_tag SDARSTMASK;    /* Reset Control Mask Register */
  uint8_t SDA_AP_reserved5[3428];
  SDA_AP_IDR_tag IDR;                  /* Identification Register */
  uint8_t SDA_AP_reserved6[444];
  SDA_AP_DEVARCH_tag DEVARCH;          /* CoreSight Device Architecture Register */
  uint8_t SDA_AP_reserved7[12];
  SDA_AP_DEVTYPE_tag DEVTYPE;          /* CoreSight Device Type Identifier Register */
  SDA_AP_PIDR4_tag PIDR4;              /* CoreSight Peripheral Identification Register 4 */
  uint8_t SDA_AP_reserved8[12];
  SDA_AP_PIDR0_tag PIDR0;              /* CoreSight Peripheral Identification Register 0 */
  SDA_AP_PIDR1_tag PIDR1;              /* CoreSight Peripheral Identification Register 1 */
  SDA_AP_PIDR2_tag PIDR2;              /* CoreSight Peripheral Identification Register 2 */
  SDA_AP_PIDR3_tag PIDR3;              /* CoreSight Peripheral Identification Register 3 */
  SDA_AP_CIDR0_tag CIDR0;              /* CoreSight Component Identification Register 0 */
  SDA_AP_CIDR1_tag CIDR1;              /* CoreSight Component Identification Register 1 */
  SDA_AP_CIDR2_tag CIDR2;              /* CoreSight Component Identification Register 2 */
  SDA_AP_CIDR3_tag CIDR3;              /* CoreSight Component Identification Register 3 */
};


/* ============================================================================
   =============================== Module: SEC_S250 ===========================
   ============================================================================ */

typedef union SEC_S250_MISCDAT0_union_tag { /* Miscellaneous Data 0 */
  vuint32_t R;
  struct {
    vuint32_t EDB:1;                   /* External Debugger Indication */
    vuint32_t _unused_1:31;
  } B;
} SEC_S250_MISCDAT0_tag;

typedef union SEC_S250_HSE_DAT0_union_tag { /* HSE_H Data 0 */
  vuint32_t R;
  struct {
    vuint32_t DATA0:6;                 /* Unused */
    vuint32_t EDB_NEW:1;               /* External Debugger or Application Core Debug and Trace Indication */
    vuint32_t DATA1:25;                /* Unused */
  } B;
} SEC_S250_HSE_DAT0_tag;

typedef union SEC_S250_HSE_DAT_union_tag { /* HSE_H Data n */
  vuint32_t R;
  struct {
    vuint32_t DATA:32;                 /* General-Purpose Data */
  } B;
} SEC_S250_HSE_DAT_tag;

typedef union SEC_S250_HSE_QSPI0_DAT0_union_tag { /* QuadSPI_0 Data 0 */
  vuint32_t R;
  struct {
    vuint32_t DATA0:6;                 /* Safe MID Value for Secure Domain */
    vuint32_t _unused_6:2;
    vuint32_t DATA1:1;                 /* Safe MID Value Valid */
    vuint32_t _unused_9:23;
  } B;
} SEC_S250_HSE_QSPI0_DAT0_tag;

typedef union SEC_S250_HSE_QSPI0_DAT1_union_tag { /* QuadSPI_0 Data 1 */
  vuint32_t R;
  struct {
    vuint32_t DATA0:11;                /* Safe Flash Memory Value for Secure Domain */
    vuint32_t _unused_11:5;
    vuint32_t DATA1:1;                 /* Safe Flash Memory Values Valid */
    vuint32_t _unused_17:15;
  } B;
} SEC_S250_HSE_QSPI0_DAT1_tag;

typedef union SEC_S250_HSE_QSPI1_DAT0_union_tag { /* QuadSPI_1 Data 0 */
  vuint32_t R;
  struct {
    vuint32_t DATA0:6;                 /* Safe MID Value for Secure Domain */
    vuint32_t _unused_6:2;
    vuint32_t DATA1:1;                 /* Safe MID Value Valid */
    vuint32_t _unused_9:23;
  } B;
} SEC_S250_HSE_QSPI1_DAT0_tag;

typedef union SEC_S250_HSE_QSPI1_DAT1_union_tag { /* QuadSPI_1 Data 1 */
  vuint32_t R;
  struct {
    vuint32_t DATA0:11;                /* Safe Flash Memory Value for Secure Domain */
    vuint32_t _unused_11:5;
    vuint32_t DATA1:1;                 /* Safe Flash Memory Values Valid */
    vuint32_t _unused_17:15;
  } B;
} SEC_S250_HSE_QSPI1_DAT1_tag;

struct SEC_S250_tag {
  uint8_t SEC_S250_reserved0[8];
  SEC_S250_MISCDAT0_tag MISCDAT0;      /* Miscellaneous Data 0 */
  uint8_t SEC_S250_reserved1[16];
  SEC_S250_HSE_DAT0_tag HSE_DAT0;      /* HSE_H Data 0 */
  uint8_t SEC_S250_reserved2[4];
  SEC_S250_HSE_DAT_tag HSE_DAT[10];    /* HSE_H Data n */
  SEC_S250_HSE_QSPI0_DAT0_tag HSE_QSPI0_DAT0; /* QuadSPI_0 Data 0 */
  SEC_S250_HSE_QSPI0_DAT1_tag HSE_QSPI0_DAT1; /* QuadSPI_0 Data 1 */
  SEC_S250_HSE_QSPI1_DAT0_tag HSE_QSPI1_DAT0; /* QuadSPI_1 Data 0 */
  SEC_S250_HSE_QSPI1_DAT1_tag HSE_QSPI1_DAT1; /* QuadSPI_1 Data 1 */
};


/* ============================================================================
   =============================== Module: SEMA42 =============================
   ============================================================================ */

typedef union SEMA42_GATE_union_tag {  /* Gate */
  vuint8_t R;
  struct {
    vuint8_t GTFSM:4;                  /* Gate Finite State Machine */
    vuint8_t _unused_4:4;              /* Reserved */
  } B;
} SEMA42_GATE_tag;

typedef union SEMA42_RSTGT_R_union_tag { /* Reset Gate Read */
  vuint16_t R;
  struct {
    vuint16_t RSTGTN:8;                /* Reset Gate Number */
    vuint16_t RSTGMS:4;                /* Reset Gate Domain */
    vuint16_t RSTGSM:2;                /* Reset Gate Finite State Machine */
    vuint16_t _unused_14:2;
  } B;
} SEMA42_RSTGT_R_tag;

typedef union SEMA42_RSTGT_W_union_tag { /* Reset Gate Write */
  vuint16_t R;
  struct {
    vuint16_t RSTGTN:8;                /* Reset Gate Number */
    vuint16_t RSTGDP:8;                /* Reset Gate Data Pattern */
  } B;
} SEMA42_RSTGT_W_tag;

typedef union SEMA42_RSTGT_union_tag {
  SEMA42_RSTGT_R_tag R;                /* Reset Gate Read */
  SEMA42_RSTGT_W_tag W;                /* Reset Gate Write */
} SEMA42_RSTGT_tag;

struct SEMA42_tag {
  SEMA42_GATE_tag GATE[16];            /* Gate */
  uint8_t SEMA42_reserved0[50];
  SEMA42_RSTGT_tag RSTGT;
};


/* ============================================================================
   =============================== Module: SINC ===============================
   ============================================================================ */

typedef union SINC_VERID_union_tag {   /* Version ID */
  vuint32_t R;
  struct {
    vuint32_t FEATURE:16;              /* Feature Specification Code */
    vuint32_t MINOR:8;                 /* Minor Version Number */
    vuint32_t MAJOR:8;                 /* Major Version Number */
  } B;
} SINC_VERID_tag;

typedef union SINC_PARAMETER_union_tag { /* Parameters */
  vuint32_t R;
  struct {
    vuint32_t FIFO_DEPTH:5;            /* FIFO Depth */
    vuint32_t _unused_5:3;
    vuint32_t FLT_NUM:4;               /* Filter Channel Number */
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:2;
    vuint32_t _unused_16:3;
    vuint32_t PF_ORD_SEL:2;            /* PF Order Select */
    vuint32_t _unused_21:2;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:4;
  } B;
} SINC_PARAMETER_tag;

typedef union SINC_MCR_union_tag {     /* Main Control */
  vuint32_t R;
  struct {
    vuint32_t STRIG0:1;                /* Software Trigger For Channel 0 */
    vuint32_t STRIG1:1;                /* Software Trigger For Channel 1 */
    vuint32_t STRIG2:1;                /* Software Trigger For Channel 2 */
    vuint32_t STRIG3:1;                /* Software Trigger For Channel 3 */
    vuint32_t _unused_4:4;
    vuint32_t _unused_8:2;
    vuint32_t DOZEN:1;                 /* Doze Or Stop Enable */
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t RST:1;                   /* Software Reset */
    vuint32_t _unused_14:1;
    vuint32_t MEN:1;                   /* Master Enable */
    vuint32_t MCLKDIV:8;               /* Modulator Clock Divider */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t PRESCALE:2;              /* Prescale Before Clock Divider */
    vuint32_t MCLK0DIS:1;              /* Disable Modulator Clock 0 Output */
    vuint32_t MCLK1DIS:1;              /* Disable Modulator Clock 1 Output */
    vuint32_t MCLK2DIS:1;              /* Disable Modulator Clock 2 Output */
    vuint32_t _unused_30:2;
  } B;
} SINC_MCR_tag;

typedef union SINC_NIE_union_tag {     /* Normal Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t COCIE0:1;                /* Conversion Complete Interrupt Enable */
    vuint32_t COCIE1:1;                /* Conversion Complete Interrupt Enable */
    vuint32_t COCIE2:1;                /* Conversion Complete Interrupt Enable */
    vuint32_t COCIE3:1;                /* Conversion Complete Interrupt Enable */
    vuint32_t _unused_4:4;
    vuint32_t CHFIE0:1;                /* Data Output Ready Interrupt Enable */
    vuint32_t CHFIE1:1;                /* Data Output Ready Interrupt Enable */
    vuint32_t CHFIE2:1;                /* Data Output Ready Interrupt Enable */
    vuint32_t CHFIE3:1;                /* Data Output Ready Interrupt Enable */
    vuint32_t _unused_12:4;
    vuint32_t ZCDIE0:1;                /* Zero Cross Detected Interrupt Enable */
    vuint32_t ZCDIE1:1;                /* Zero Cross Detected Interrupt Enable */
    vuint32_t ZCDIE2:1;                /* Zero Cross Detected Interrupt Enable */
    vuint32_t ZCDIE3:1;                /* Zero Cross Detected Interrupt Enable */
    vuint32_t _unused_20:4;
    vuint32_t _unused_24:8;
  } B;
} SINC_NIE_tag;

typedef union SINC_EIE_union_tag {     /* Error Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t SCDIE0:1;                /* Short Circuit Detected Interrupt Enable */
    vuint32_t SCDIE1:1;                /* Short Circuit Detected Interrupt Enable */
    vuint32_t SCDIE2:1;                /* Short Circuit Detected Interrupt Enable */
    vuint32_t SCDIE3:1;                /* Short Circuit Detected Interrupt Enable */
    vuint32_t _unused_4:4;
    vuint32_t WLMTIE0:1;               /* Window Limit Interrupt Enable */
    vuint32_t WLMTIE1:1;               /* Window Limit Interrupt Enable */
    vuint32_t WLMTIE2:1;               /* Window Limit Interrupt Enable */
    vuint32_t WLMTIE3:1;               /* Window Limit Interrupt Enable */
    vuint32_t _unused_12:4;
    vuint32_t LLMTIE0:1;               /* Low Limit Interrupt Enable */
    vuint32_t LLMTIE1:1;               /* Low Limit Interrupt Enable */
    vuint32_t LLMTIE2:1;               /* Low Limit Interrupt Enable */
    vuint32_t LLMTIE3:1;               /* Low Limit Interrupt Enable */
    vuint32_t _unused_20:4;
    vuint32_t HLMTIE0:1;               /* High Limit Interrupt Enable */
    vuint32_t HLMTIE1:1;               /* High Limit Interrupt Enable */
    vuint32_t HLMTIE2:1;               /* High Limit Interrupt Enable */
    vuint32_t HLMTIE3:1;               /* High Limit Interrupt Enable */
    vuint32_t _unused_28:4;
  } B;
} SINC_EIE_tag;

typedef union SINC_FIFOIE_union_tag {  /* FIFO And CAD Error Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t FUNFIE0:1;               /* FIFO Underflow Interrupt Enable */
    vuint32_t FUNFIE1:1;               /* FIFO Underflow Interrupt Enable */
    vuint32_t FUNFIE2:1;               /* FIFO Underflow Interrupt Enable */
    vuint32_t FUNFIE3:1;               /* FIFO Underflow Interrupt Enable */
    vuint32_t _unused_4:4;
    vuint32_t FOVFIE0:1;               /* FIFO Overflow Interrupt Enable */
    vuint32_t FOVFIE1:1;               /* FIFO Overflow Interrupt Enable */
    vuint32_t FOVFIE2:1;               /* FIFO Overflow Interrupt Enable */
    vuint32_t FOVFIE3:1;               /* FIFO Overflow Interrupt Enable */
    vuint32_t _unused_12:4;
    vuint32_t CADIE0:1;                /* Clock Absence Interrupt Enable */
    vuint32_t CADIE1:1;                /* Clock Absence Interrupt Enable */
    vuint32_t CADIE2:1;                /* Clock Absence Interrupt Enable */
    vuint32_t CADIE3:1;                /* Clock Absence Interrupt Enable */
    vuint32_t _unused_20:4;
    vuint32_t SATIE0:1;                /* Saturation Interrupt Enable */
    vuint32_t SATIE1:1;                /* Saturation Interrupt Enable */
    vuint32_t SATIE2:1;                /* Saturation Interrupt Enable */
    vuint32_t SATIE3:1;                /* Saturation Interrupt Enable */
    vuint32_t _unused_28:4;
  } B;
} SINC_FIFOIE_tag;

typedef union SINC_NIS_union_tag {     /* Normal Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t COC0:1;                  /* Conversion Complete Flag */
    vuint32_t COC1:1;                  /* Conversion Complete Flag */
    vuint32_t COC2:1;                  /* Conversion Complete Flag */
    vuint32_t COC3:1;                  /* Conversion Complete Flag */
    vuint32_t _unused_4:4;
    vuint32_t CHF0:1;                  /* Data Output Ready Flag */
    vuint32_t CHF1:1;                  /* Data Output Ready Flag */
    vuint32_t CHF2:1;                  /* Data Output Ready Flag */
    vuint32_t CHF3:1;                  /* Data Output Ready Flag */
    vuint32_t _unused_12:4;
    vuint32_t ZCD0:1;                  /* Zero Cross Detected Flag */
    vuint32_t ZCD1:1;                  /* Zero Cross Detected Flag */
    vuint32_t ZCD2:1;                  /* Zero Cross Detected Flag */
    vuint32_t ZCD3:1;                  /* Zero Cross Detected Flag */
    vuint32_t _unused_20:4;
    vuint32_t _unused_24:8;
  } B;
} SINC_NIS_tag;

typedef union SINC_EIS_union_tag {     /* Error Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t SCD0:1;                  /* Short Circuit Detected Flag */
    vuint32_t SCD1:1;                  /* Short Circuit Detected Flag */
    vuint32_t SCD2:1;                  /* Short Circuit Detected Flag */
    vuint32_t SCD3:1;                  /* Short Circuit Detected Flag */
    vuint32_t _unused_4:4;
    vuint32_t WLMT0:1;                 /* Window Limit Flag */
    vuint32_t WLMT1:1;                 /* Window Limit Flag */
    vuint32_t WLMT2:1;                 /* Window Limit Flag */
    vuint32_t WLMT3:1;                 /* Window Limit Flag */
    vuint32_t _unused_12:4;
    vuint32_t LLMT0:1;                 /* Low Limit Flag */
    vuint32_t LLMT1:1;                 /* Low Limit Flag */
    vuint32_t LLMT2:1;                 /* Low Limit Flag */
    vuint32_t LLMT3:1;                 /* Low Limit Flag */
    vuint32_t _unused_20:4;
    vuint32_t HLMT0:1;                 /* High Limit Flag */
    vuint32_t HLMT1:1;                 /* High Limit Flag */
    vuint32_t HLMT2:1;                 /* High Limit Flag */
    vuint32_t HLMT3:1;                 /* High Limit Flag */
    vuint32_t _unused_28:4;
  } B;
} SINC_EIS_tag;

typedef union SINC_FIFOIS_union_tag {  /* FIFO And CAD Error Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t FUNF0:1;                 /* FIFO Underflow Flag */
    vuint32_t FUNF1:1;                 /* FIFO Underflow Flag */
    vuint32_t FUNF2:1;                 /* FIFO Underflow Flag */
    vuint32_t FUNF3:1;                 /* FIFO Underflow Flag */
    vuint32_t _unused_4:4;
    vuint32_t FOVF0:1;                 /* FIFO Overflow Flag */
    vuint32_t FOVF1:1;                 /* FIFO Overflow Flag */
    vuint32_t FOVF2:1;                 /* FIFO Overflow Flag */
    vuint32_t FOVF3:1;                 /* FIFO Overflow Flag */
    vuint32_t _unused_12:4;
    vuint32_t CAD0:1;                  /* Clock Absence Flag */
    vuint32_t CAD1:1;                  /* Clock Absence Flag */
    vuint32_t CAD2:1;                  /* Clock Absence Flag */
    vuint32_t CAD3:1;                  /* Clock Absence Flag */
    vuint32_t _unused_20:4;
    vuint32_t SAT0:1;                  /* Saturation Flag */
    vuint32_t SAT1:1;                  /* Saturation Flag */
    vuint32_t SAT2:1;                  /* Saturation Flag */
    vuint32_t SAT3:1;                  /* Saturation Flag */
    vuint32_t _unused_28:4;
  } B;
} SINC_FIFOIS_tag;

typedef union SINC_SR_union_tag {      /* Status */
  vuint32_t R;
  struct {
    vuint32_t CIP0:1;                  /* Conversion In Progress */
    vuint32_t CIP1:1;                  /* Conversion In Progress */
    vuint32_t CIP2:1;                  /* Conversion In Progress */
    vuint32_t CIP3:1;                  /* Conversion In Progress */
    vuint32_t _unused_4:4;
    vuint32_t CHRDY0:1;                /* Channel Ready For Conversion */
    vuint32_t CHRDY1:1;                /* Channel Ready For Conversion */
    vuint32_t CHRDY2:1;                /* Channel Ready For Conversion */
    vuint32_t CHRDY3:1;                /* Channel Ready For Conversion */
    vuint32_t _unused_12:4;
    vuint32_t FIFOEMPTY0:1;            /* FIFO Empty */
    vuint32_t FIFOEMPTY1:1;            /* FIFO Empty */
    vuint32_t FIFOEMPTY2:1;            /* FIFO Empty */
    vuint32_t FIFOEMPTY3:1;            /* FIFO Empty */
    vuint32_t _unused_20:4;
    vuint32_t MCLKRDY0:1;              /* Modulator Clock 0 Ready */
    vuint32_t MCLKRDY1:1;              /* Modulator Clock 1 Ready */
    vuint32_t MCLKRDY2:1;              /* Modulator Clock 2 Ready */
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:4;
  } B;
} SINC_SR_tag;

typedef union SINC_CHANNEL_CCR_union_tag { /* Channel 0 Control */
  vuint32_t R;
  struct {
    vuint32_t CHEN:1;                  /* Channel Enable */
    vuint32_t PFEN:1;                  /* PF Enable */
    vuint32_t _unused_2:1;
    vuint32_t DMAEN:1;                 /* DMA Enable */
    vuint32_t _unused_4:4;
    vuint32_t SCDEN:1;                 /* Short Circuit Detect Enable */
    vuint32_t CADEN:1;                 /* Clock Absence Detect Enable */
    vuint32_t _unused_10:2;
    vuint32_t ZCDEN:1;                 /* Zero Cross Detect Enable */
    vuint32_t LMTEN:1;                 /* Limit Enable */
    vuint32_t FIFOEN:1;                /* FIFO Enable */
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:4;
    vuint32_t DBGSEL:4;                /* Debug Output Selection */
    vuint32_t _unused_24:2;
    vuint32_t _unused_26:2;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} SINC_CHANNEL_CCR_tag;

typedef union SINC_CHANNEL_CDR_union_tag { /* Channel 0 Data Rate */
  vuint32_t R;
  struct {
    vuint32_t PFOSR:11;                /* PF OSR */
    vuint32_t PFORD:2;                 /* PF Order */
    vuint32_t _unused_13:1;
    vuint32_t PFCM:2;                  /* PF Conversion Mode */
    vuint32_t _unused_16:11;
    vuint32_t _unused_27:2;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} SINC_CHANNEL_CDR_tag;

typedef union SINC_CHANNEL_CCFR_union_tag { /* Channel 0 Configuration */
  vuint32_t R;
  struct {
    vuint32_t PFSFT:5;                 /* PF Shift */
    vuint32_t _unused_5:1;
    vuint32_t RDFMT:1;                 /* Result Data Format */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:2;
    vuint32_t FIFOWMK:4;               /* FIFO Watermark */
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t IBFMT:2;                 /* Input Bit Format */
    vuint32_t ICSEL:3;                 /* Input Clock Select */
    vuint32_t ICESEL:3;                /* Input Clock Edge Select */
    vuint32_t ITSEL:2;                 /* Input Trigger Select */
    vuint32_t IBSEL:2;                 /* Input Bit Select */
    vuint32_t ITLVL:1;                 /* Input Trigger Level Type */
    vuint32_t _unused_29:1;
    vuint32_t ZCOP:2;                  /* Zero Cross Option */
  } B;
} SINC_CHANNEL_CCFR_tag;

typedef union SINC_CHANNEL_CPROT_union_tag { /* Channel 0 Protection */
  vuint32_t R;
  struct {
    vuint32_t SCDLMT:8;                /* SCD Limit Threshold */
    vuint32_t _unused_8:2;
    vuint32_t _unused_10:1;
    vuint32_t SCDCM:1;                 /* SCD Conversion Mode */
    vuint32_t SCDOP:2;                 /* SCD Option */
    vuint32_t LMTOP:2;                 /* Limit Detection Option */
    vuint32_t CADLMT:4;                /* CAD Limit Threshold */
    vuint32_t _unused_20:2;
    vuint32_t _unused_22:4;
    vuint32_t CADBK:1;                 /* CAD Break Signal */
    vuint32_t SCDBK:1;                 /* SCD Break Signal */
    vuint32_t _unused_28:1;
    vuint32_t LLMTBK:1;                /* Low Limit Break Signal */
    vuint32_t WLMTBK:1;                /* Window Limit Break Signal */
    vuint32_t HLMTBK:1;                /* High Limit Break Signal */
  } B;
} SINC_CHANNEL_CPROT_tag;

typedef union SINC_CHANNEL_CBIAS_union_tag { /* Channel 0 Bias */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t BIAS:24;                 /* Bias Value */
  } B;
} SINC_CHANNEL_CBIAS_tag;

typedef union SINC_CHANNEL_CLOLMT_union_tag { /* Channel 0 Low Limit */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t LOLMT:24;                /* Low Limit Threshold */
  } B;
} SINC_CHANNEL_CLOLMT_tag;

typedef union SINC_CHANNEL_CHILMT_union_tag { /* Channel 0 High Limit */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t HILMT:24;                /* High Limit Threshold */
  } B;
} SINC_CHANNEL_CHILMT_tag;

typedef union SINC_CHANNEL_CRDATA_union_tag { /* Channel 0 Result Data */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:8;
    vuint32_t RDATA:24;                /* Result Data */
  } B;
} SINC_CHANNEL_CRDATA_tag;

typedef union SINC_CHANNEL_CMPDATA_union_tag { /* Channel 0 Multipurpose Data */
  vuint32_t R;
  struct {
    vuint32_t MPDATA:32;               /* Multipurpose Data */
  } B;
} SINC_CHANNEL_CMPDATA_tag;

typedef union SINC_CHANNEL_CACFR_union_tag { /* Channel 0 Advanced Configuration */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:11;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:4;            /* Reserved */
    vuint32_t HPFA:4;                  /* HPF DC Remover Alpha Coefficient */
    vuint32_t IBDLY:4;                 /* Input Modulator Bitstream Delay */
    vuint32_t _unused_24:5;
    vuint32_t _unused_29:3;
  } B;
} SINC_CHANNEL_CACFR_tag;

typedef union SINC_CHANNEL_CSR_union_tag { /* Channel 0 Status */
  vuint32_t R;
  struct {
    vuint32_t FIFOAVIL:5;              /* FIFO Available Data */
    vuint32_t _unused_5:2;
    vuint32_t PSRDY:1;                 /* Parallel or Serial Data Ready */
    vuint32_t PFSAT:1;                 /* Primary CIC Saturation Flag */
    vuint32_t HPFSAT:1;                /* HPF Saturation Flag */
    vuint32_t SFTSAT:1;                /* Shift Saturation Flag */
    vuint32_t BIASSAT:1;               /* Bias Saturation Flag */
    vuint32_t RDRS:1;                  /* Result Data Direct Read Status */
    vuint32_t SRDS:1;                  /* Start Read Debug Data Sync */
    vuint32_t DBGRS:2;                 /* Debug Data Read Status */
    vuint32_t CNUM:7;                  /* Number Of Conversions */
    vuint32_t CNUM_OV:1;               /* Overflow In Number Of Conversions */
    vuint32_t _unused_24:2;
    vuint32_t _unused_26:2;
    vuint32_t _unused_28:4;
  } B;
} SINC_CHANNEL_CSR_tag;

typedef union SINC_CHANNEL_CDBGR_union_tag { /* Channel 0 Debug */
  vuint32_t R;
  struct {
    vuint32_t DBGDATA:32;              /* Debug Data */
  } B;
} SINC_CHANNEL_CDBGR_tag;

typedef struct SINC_CHANNEL_struct_tag {
  SINC_CHANNEL_CCR_tag CCR;            /* Channel 0 Control */
  SINC_CHANNEL_CDR_tag CDR;            /* Channel 0 Data Rate */
  SINC_CHANNEL_CCFR_tag CCFR;          /* Channel 0 Configuration */
  SINC_CHANNEL_CPROT_tag CPROT;        /* Channel 0 Protection */
  SINC_CHANNEL_CBIAS_tag CBIAS;        /* Channel 0 Bias */
  SINC_CHANNEL_CLOLMT_tag CLOLMT;      /* Channel 0 Low Limit */
  SINC_CHANNEL_CHILMT_tag CHILMT;      /* Channel 0 High Limit */
  SINC_CHANNEL_CRDATA_tag CRDATA;      /* Channel 0 Result Data */
  SINC_CHANNEL_CMPDATA_tag CMPDATA;    /* Channel 0 Multipurpose Data */
  SINC_CHANNEL_CACFR_tag CACFR;        /* Channel 0 Advanced Configuration */
  SINC_CHANNEL_CSR_tag CSR;            /* Channel 0 Status */
  SINC_CHANNEL_CDBGR_tag CDBGR;        /* Channel 0 Debug */
} SINC_CHANNEL_tag;

struct SINC_tag {
  SINC_VERID_tag VERID;                /* Version ID */
  SINC_PARAMETER_tag PARAMETER;        /* Parameters */
  SINC_MCR_tag MCR;                    /* Main Control */
  SINC_NIE_tag NIE;                    /* Normal Interrupt Enable */
  SINC_EIE_tag EIE;                    /* Error Interrupt Enable */
  SINC_FIFOIE_tag FIFOIE;              /* FIFO And CAD Error Interrupt Enable */
  SINC_NIS_tag NIS;                    /* Normal Interrupt Status */
  SINC_EIS_tag EIS;                    /* Error Interrupt Status */
  SINC_FIFOIS_tag FIFOIS;              /* FIFO And CAD Error Interrupt Status */
  SINC_SR_tag SR;                      /* Status */
  uint8_t SINC_reserved0[16];
  SINC_CHANNEL_tag CHANNEL[4];
};


/* ============================================================================
   =============================== Module: SIPI ===============================
   ============================================================================ */

typedef union SIPI_CCR0_union_tag {    /* SIPI Channel Control Register 0 */
  vuint32_t R;
  struct {
    vuint32_t DEN:1;                   /* DMA Enable. */
    vuint32_t WRT:1;                   /* Write Request Transfer. */
    vuint32_t RRT:1;                   /* Read Request Transfer. */
    vuint32_t IDT:1;                   /* ID Read Request Transfer. */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t CHEN:1;                  /* Channel Enable. */
    vuint32_t WL:2;                    /* Word Length Transfer. */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t TC:1;                    /* Send Trigger Command. */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} SIPI_CCR0_tag;

typedef union SIPI_CSR0_union_tag {    /* SIPI Channel Status Register 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t CB:1;                    /* Channel Busy. */
    vuint32_t ACKR:1;                  /* ACKR */
    vuint32_t TID:3;                   /* Transaction ID of transmitted frame. */
    vuint32_t RAR:1;                   /* RAR */
    vuint32_t READ_RX:1;               /* READ_RX */
    vuint32_t WRITE_RX:1;              /* WRITE_RX */
    vuint32_t _unused_10:22;           /* Reserved */
  } B;
} SIPI_CSR0_tag;

typedef union SIPI_CIR0_union_tag {    /* SIPI Channel Interrupt Register 0 */
  vuint32_t R;
  struct {
    vuint32_t ACKIE:1;                 /* Acknowledge Error Interrupt Enable. */
    vuint32_t TIDIE:1;                 /* Transaction ID Error Interrupt Enable. */
    vuint32_t TOIE:1;                  /* Timeout Error Interrupt Enabled. */
    vuint32_t TCIE:1;                  /* Trigger Command Interrupt Enable. */
    vuint32_t RAIE:1;                  /* Read Answer Interrupt Enable. */
    vuint32_t WAIE:1;                  /* Write Acknowledge Interrupt Enable. */
    vuint32_t _unused_6:26;            /* Reserved */
  } B;
} SIPI_CIR0_tag;

typedef union SIPI_CTOR0_union_tag {   /* SIPI Channel Timeout Register 0 */
  vuint32_t R;
  struct {
    vuint32_t TOR:32;                  /* Timeout value for transmitted requests. */
  } B;
} SIPI_CTOR0_tag;

typedef union SIPI_CCRC0_union_tag {   /* SIPI Channel CRC Register 0 */
  vuint32_t R;
  struct {
    vuint32_t CRCT:16;                 /* CRCT */
    vuint32_t CRCI:16;                 /* CRCI */
  } B;
} SIPI_CCRC0_tag;

typedef union SIPI_CAR0_union_tag {    /* SIPI Channel Address Register 0 */
  vuint32_t R;
  struct {
    vuint32_t CAR:32;                  /* CAR */
  } B;
} SIPI_CAR0_tag;

typedef union SIPI_CDR0_union_tag {    /* SIPI Channel Data Register 0 */
  vuint32_t R;
  struct {
    vuint32_t CDR:32;                  /* CDR */
  } B;
} SIPI_CDR0_tag;

typedef union SIPI_CCR1_union_tag {    /* SIPI Channel Control Register 1 */
  vuint32_t R;
  struct {
    vuint32_t DEN:1;                   /* DMA Enable. */
    vuint32_t WRT:1;                   /* Write Request Transfer. */
    vuint32_t RRT:1;                   /* RRT */
    vuint32_t IDT:1;                   /* ID Read Request Transfer. */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t CHEN:1;                  /* Channel Enable. */
    vuint32_t WL:2;                    /* Word Length Transfer. */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t TC:1;                    /* Send Trigger Command. */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} SIPI_CCR1_tag;

typedef union SIPI_CSR1_union_tag {    /* SIPI Channel Status Register 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t CB:1;                    /* Channel Busy. */
    vuint32_t ACKR:1;                  /* ACKR */
    vuint32_t TID:3;                   /* TID */
    vuint32_t RAR:1;                   /* RAR */
    vuint32_t READ_RX:1;               /* READ_RX */
    vuint32_t WRITE_RX:1;              /* WRITE_RX */
    vuint32_t _unused_10:22;           /* Reserved */
  } B;
} SIPI_CSR1_tag;

typedef union SIPI_CIR1_union_tag {    /* SIPI Channel Interrupt Register 1 */
  vuint32_t R;
  struct {
    vuint32_t ACKIE:1;                 /* ACKIE */
    vuint32_t TIDIE:1;                 /* TIDIE */
    vuint32_t TOIE:1;                  /* TOIE */
    vuint32_t TCIE:1;                  /* TCIE */
    vuint32_t RAIE:1;                  /* RAIE */
    vuint32_t WAIE:1;                  /* WAIE */
    vuint32_t _unused_6:26;            /* Reserved */
  } B;
} SIPI_CIR1_tag;

typedef union SIPI_CTOR1_union_tag {   /* SIPI Channel Timeout Register 1 */
  vuint32_t R;
  struct {
    vuint32_t TOR:32;                  /* Timeout value for transmitted requests. */
  } B;
} SIPI_CTOR1_tag;

typedef union SIPI_CCRC1_union_tag {   /* SIPI Channel CRC Register 1 */
  vuint32_t R;
  struct {
    vuint32_t CRCT:16;                 /* CRCT */
    vuint32_t CRCI:16;                 /* CRCI */
  } B;
} SIPI_CCRC1_tag;

typedef union SIPI_CAR1_union_tag {    /* SIPI Channel Address Register 1 */
  vuint32_t R;
  struct {
    vuint32_t CAR:32;                  /* CAR */
  } B;
} SIPI_CAR1_tag;

typedef union SIPI_CDR1_union_tag {    /* SIPI Channel Data Register 1 */
  vuint32_t R;
  struct {
    vuint32_t CDR:32;                  /* CDR */
  } B;
} SIPI_CDR1_tag;

typedef union SIPI_CCR2_union_tag {    /* SIPI Channel Control Register 2 */
  vuint32_t R;
  struct {
    vuint32_t DEN:1;                   /* DMA Enable. */
    vuint32_t WRT:1;                   /* Write Request Transfer. */
    vuint32_t RRT:1;                   /* Read Request Transfer. */
    vuint32_t IDT:1;                   /* ID Read Request Transfer. */
    vuint32_t ST:1;                    /* ST */
    vuint32_t CHEN:1;                  /* Channel Enable. */
    vuint32_t WL:2;                    /* Word Length Transfer. */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t TC:1;                    /* Send Trigger Command. */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} SIPI_CCR2_tag;

typedef union SIPI_CSR2_union_tag {    /* SIPI Channel Status Register 2 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t CB:1;                    /* Channel Busy. */
    vuint32_t ACKR:1;                  /* ACKR */
    vuint32_t TID:3;                   /* Transaction ID of transmitted frame. */
    vuint32_t RAR:1;                   /* RAR */
    vuint32_t READ_RX:1;               /* READ_RX */
    vuint32_t WRITE_RX:1;              /* WRITE_RX */
    vuint32_t WRITE_STR_RX:1;          /* WRITE_STR_RX */
    vuint32_t _unused_11:21;           /* Reserved */
  } B;
} SIPI_CSR2_tag;

typedef union SIPI_CIR2_union_tag {    /* SIPI Channel Interrupt Register 2 */
  vuint32_t R;
  struct {
    vuint32_t ACKIE:1;                 /* ACKIE */
    vuint32_t TIDIE:1;                 /* TIDIE */
    vuint32_t TOIE:1;                  /* TOIE */
    vuint32_t TCIE:1;                  /* TCIE */
    vuint32_t RAIE:1;                  /* RAIE */
    vuint32_t WAIE:1;                  /* WAIE */
    vuint32_t _unused_6:26;            /* Reserved */
  } B;
} SIPI_CIR2_tag;

typedef union SIPI_CTOR2_union_tag {   /* SIPI Channel Timeout Register 2 */
  vuint32_t R;
  struct {
    vuint32_t TOR:32;                  /* Timeout value for transmitted requests. */
  } B;
} SIPI_CTOR2_tag;

typedef union SIPI_CCRC2_union_tag {   /* SIPI Channel CRC Register 2 */
  vuint32_t R;
  struct {
    vuint32_t CRCT:16;                 /* CRCT */
    vuint32_t CRCI:16;                 /* CRCI */
  } B;
} SIPI_CCRC2_tag;

typedef union SIPI_CAR2_union_tag {    /* SIPI Channel Address Register 2 */
  vuint32_t R;
  struct {
    vuint32_t CAR:32;                  /* CAR */
  } B;
} SIPI_CAR2_tag;

typedef union SIPI_CDR2_union_tag {    /* SIPI Channel Data Register 2 */
  vuint32_t R;
  struct {
    vuint32_t CDR2:32;                 /* CDR2 */
  } B;
} SIPI_CDR2_tag;

typedef union SIPI_CCR3_union_tag {    /* SIPI Channel Control Register 3 */
  vuint32_t R;
  struct {
    vuint32_t DEN:1;                   /* DMA Enable. */
    vuint32_t WRT:1;                   /* Write Request Transfer. */
    vuint32_t RRT:1;                   /* Read Request Transfer. */
    vuint32_t IDT:1;                   /* ID Read Request Transfer. */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t CHEN:1;                  /* Channel Enable. */
    vuint32_t WL:2;                    /* Word Length Transfer. */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t TC:1;                    /* Send Trigger Command. */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} SIPI_CCR3_tag;

typedef union SIPI_CSR3_union_tag {    /* SIPI Channel Status Register 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t CB:1;                    /* Channel Busy. */
    vuint32_t ACKR:1;                  /* ACKR */
    vuint32_t TID:3;                   /* Transaction ID of transmitted frame. */
    vuint32_t RAR:1;                   /* RAR */
    vuint32_t READ_RX:1;               /* READ_RX */
    vuint32_t WRITE_RX:1;              /* WRITE_RX */
    vuint32_t _unused_10:22;           /* Reserved */
  } B;
} SIPI_CSR3_tag;

typedef union SIPI_CIR3_union_tag {    /* SIPI Channel Interrupt Register 3 */
  vuint32_t R;
  struct {
    vuint32_t ACKIE:1;                 /* ACKIE */
    vuint32_t TIDIE:1;                 /* TIDIE */
    vuint32_t TOIE:1;                  /* TOIE */
    vuint32_t TCIE:1;                  /* TCIE */
    vuint32_t RAIE:1;                  /* RAIE */
    vuint32_t WAIE:1;                  /* WAIE */
    vuint32_t _unused_6:26;            /* Reserved */
  } B;
} SIPI_CIR3_tag;

typedef union SIPI_CTOR3_union_tag {   /* SIPI Channel Timeout Register 3 */
  vuint32_t R;
  struct {
    vuint32_t TOR:32;                  /* Timeout value for transmitted requests. */
  } B;
} SIPI_CTOR3_tag;

typedef union SIPI_CCRC3_union_tag {   /* SIPI Channel CRC Register 3 */
  vuint32_t R;
  struct {
    vuint32_t CRCT:16;                 /* CRCT */
    vuint32_t CRCI:16;                 /* CRCI */
  } B;
} SIPI_CCRC3_tag;

typedef union SIPI_CAR3_union_tag {    /* SIPI Channel Address Register 3 */
  vuint32_t R;
  struct {
    vuint32_t CAR:32;                  /* CAR */
  } B;
} SIPI_CAR3_tag;

typedef union SIPI_CDR3_union_tag {    /* SIPI Channel Data Register 3 */
  vuint32_t R;
  struct {
    vuint32_t CDR:32;                  /* CDR */
  } B;
} SIPI_CDR3_tag;

typedef union SIPI_MCR_union_tag {     /* SIPI Module Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SR:1;                    /* Soft Reset */
    vuint32_t MOEN:1;                  /* Module Enable */
    vuint32_t INIT:1;                  /* Initialization Mode */
    vuint32_t TEN:1;                   /* Target Enable */
    vuint32_t CHNSB:1;                 /* Channel coding select bit. */
    vuint32_t LEGACY_MODE_BIT:1;       /* Legacy_mode select bit. */
    vuint32_t _unused_6:3;             /* Reserved */
    vuint32_t MCRIE:1;                 /* Max Count Reached Interrupt Enable */
    vuint32_t CRCIE:1;                 /* CRC Error Interrupt Enable */
    vuint32_t _unused_11:3;            /* Reserved */
    vuint32_t AID:2;                   /* Address Increment/Decrement */
    vuint32_t PRSCLR:11;               /* Timeout counter prescaler */
    vuint32_t _unused_27:2;            /* Reserved */
    vuint32_t HALT:1;                  /* Halt Mode Enable */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t FRZ:1;                   /* Freeze Enable */
  } B;
} SIPI_MCR_tag;

typedef union SIPI_SR_union_tag {      /* SIPI Status Register */
  vuint32_t R;
  struct {
    vuint32_t STATE:4;                 /* STATE */
    vuint32_t TE:4;                    /* TE */
    vuint32_t FRAME_TX:1;              /* FRAME_TX . */
    vuint32_t MCR:1;                   /* Maximum Count Reached. */
    vuint32_t GCRCE:1;                 /* GCRCE */
    vuint32_t ACK_TX:1;                /* ACK_TX */
    vuint32_t READ_OK_TX:1;            /* READ_OK_TX */
    vuint32_t INVALID_FRAME_RX:1;      /* INVALID_FRAME_RX */
    vuint32_t _unused_14:16;           /* Reserved */
    vuint32_t LPMACK:1;                /* Low Power Mode Acknowledge. */
    vuint32_t FRZACK:1;                /* FRZACK */
  } B;
} SIPI_SR_tag;

typedef union SIPI_MAXCR_union_tag {   /* SIPI Max Count Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t MXCNT:30;                /* MXCNT */
  } B;
} SIPI_MAXCR_tag;

typedef union SIPI_ARR_union_tag {     /* SIPI Address Reload Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADRLD:30;                /* ADRLD */
  } B;
} SIPI_ARR_tag;

typedef union SIPI_ACR_union_tag {     /* SIPI Address Count Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADCNT:30;                /* Feflects the count value of address counter at target node. */
  } B;
} SIPI_ACR_tag;

typedef union SIPI_ERR_union_tag {     /* SIPI Error Register */
  vuint32_t R;
  struct {
    vuint32_t ACKE0:1;                 /* ACKE0 */
    vuint32_t TIDE0:1;                 /* TIDE0 */
    vuint32_t TOE0:1;                  /* TOE0 */
    vuint32_t _unused_3:5;             /* Reserved */
    vuint32_t ACKE1:1;                 /* ACKE1 */
    vuint32_t TIDE1:1;                 /* TIDE1 */
    vuint32_t TOE1:1;                  /* TOE1 */
    vuint32_t _unused_11:5;            /* Reserved */
    vuint32_t ACKE2:1;                 /* ACKE2 */
    vuint32_t TIDE2:1;                 /* TIDE2 */
    vuint32_t TOE2:1;                  /* TOE2 */
    vuint32_t _unused_19:5;            /* Reserved */
    vuint32_t ACKE3:1;                 /* ACKE3 */
    vuint32_t TIDE3:1;                 /* TIDE3 */
    vuint32_t TOE3:1;                  /* TOE3 */
    vuint32_t _unused_27:5;            /* Reserved */
  } B;
} SIPI_ERR_tag;

typedef union SIPI_DEBUG_union_tag {   /* SIPI Debug Register */
  vuint32_t R;
  struct {
    vuint32_t STATE:16;                /* STATE */
    vuint32_t OUTSTANDING:4;           /* OUTSTANDING FRAMES */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} SIPI_DEBUG_tag;

typedef union SIPI_PMR_union_tag {     /* SIPI Performance Monitor register */
  vuint32_t R;
  struct {
    vuint32_t BIN0:8;                  /* BIN0 */
    vuint32_t BIN1:8;                  /* BIN1 */
    vuint32_t BIN2:8;                  /* BIN2 */
    vuint32_t BIN3:8;                  /* BIN3 */
  } B;
} SIPI_PMR_tag;

struct SIPI_tag {
  SIPI_CCR0_tag CCR0;                  /* SIPI Channel Control Register 0 */
  SIPI_CSR0_tag CSR0;                  /* SIPI Channel Status Register 0 */
  uint8_t SIPI_reserved0[4];
  SIPI_CIR0_tag CIR0;                  /* SIPI Channel Interrupt Register 0 */
  SIPI_CTOR0_tag CTOR0;                /* SIPI Channel Timeout Register 0 */
  SIPI_CCRC0_tag CCRC0;                /* SIPI Channel CRC Register 0 */
  SIPI_CAR0_tag CAR0;                  /* SIPI Channel Address Register 0 */
  SIPI_CDR0_tag CDR0;                  /* SIPI Channel Data Register 0 */
  SIPI_CCR1_tag CCR1;                  /* SIPI Channel Control Register 1 */
  SIPI_CSR1_tag CSR1;                  /* SIPI Channel Status Register 1 */
  uint8_t SIPI_reserved1[4];
  SIPI_CIR1_tag CIR1;                  /* SIPI Channel Interrupt Register 1 */
  SIPI_CTOR1_tag CTOR1;                /* SIPI Channel Timeout Register 1 */
  SIPI_CCRC1_tag CCRC1;                /* SIPI Channel CRC Register 1 */
  SIPI_CAR1_tag CAR1;                  /* SIPI Channel Address Register 1 */
  SIPI_CDR1_tag CDR1;                  /* SIPI Channel Data Register 1 */
  SIPI_CCR2_tag CCR2;                  /* SIPI Channel Control Register 2 */
  SIPI_CSR2_tag CSR2;                  /* SIPI Channel Status Register 2 */
  uint8_t SIPI_reserved2[4];
  SIPI_CIR2_tag CIR2;                  /* SIPI Channel Interrupt Register 2 */
  SIPI_CTOR2_tag CTOR2;                /* SIPI Channel Timeout Register 2 */
  SIPI_CCRC2_tag CCRC2;                /* SIPI Channel CRC Register 2 */
  SIPI_CAR2_tag CAR2;                  /* SIPI Channel Address Register 2 */
  SIPI_CDR2_tag CDR2[8];               /* SIPI Channel Data Register 2 */
  SIPI_CCR3_tag CCR3;                  /* SIPI Channel Control Register 3 */
  SIPI_CSR3_tag CSR3;                  /* SIPI Channel Status Register 3 */
  uint8_t SIPI_reserved3[4];
  SIPI_CIR3_tag CIR3;                  /* SIPI Channel Interrupt Register 3 */
  SIPI_CTOR3_tag CTOR3;                /* SIPI Channel Timeout Register 3 */
  SIPI_CCRC3_tag CCRC3;                /* SIPI Channel CRC Register 3 */
  SIPI_CAR3_tag CAR3;                  /* SIPI Channel Address Register 3 */
  SIPI_CDR3_tag CDR3;                  /* SIPI Channel Data Register 3 */
  SIPI_MCR_tag MCR;                    /* SIPI Module Configuration Register */
  SIPI_SR_tag SR;                      /* SIPI Status Register */
  SIPI_MAXCR_tag MAXCR;                /* SIPI Max Count Register */
  SIPI_ARR_tag ARR;                    /* SIPI Address Reload Register */
  SIPI_ACR_tag ACR;                    /* SIPI Address Count Register */
  SIPI_ERR_tag ERR;                    /* SIPI Error Register */
  SIPI_DEBUG_tag DEBUG;                /* SIPI Debug Register */
  SIPI_PMR_tag PMR;                    /* SIPI Performance Monitor register */
};


/* ============================================================================
   =============================== Module: SIUL2_0 ============================
   ============================================================================ */

typedef union SIUL2_0_DISR0_union_tag { /* SIUL2 DMA/Interrupt Status Flag 0 */
  vuint32_t R;
  struct {
    vuint32_t EIF0:1;                  /* External Interrupt Status Flag 0 */
    vuint32_t EIF1:1;                  /* External Interrupt Status Flag 1 */
    vuint32_t EIF2:1;                  /* External Interrupt Status Flag 2 */
    vuint32_t EIF3:1;                  /* External Interrupt Status Flag 3 */
    vuint32_t EIF4:1;                  /* External Interrupt Status Flag 4 */
    vuint32_t EIF5:1;                  /* External Interrupt Status Flag 5 */
    vuint32_t EIF6:1;                  /* External Interrupt Status Flag 6 */
    vuint32_t EIF7:1;                  /* External Interrupt Status Flag 7 */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_0_DISR0_tag;

typedef union SIUL2_0_DIRER0_union_tag { /* SIUL2 DMA/Interrupt Request Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t EIRE0:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE1:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE2:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE3:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE4:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE5:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE6:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE7:1;                 /* External Interrupt Request Enable */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_0_DIRER0_tag;

typedef union SIUL2_0_DIRSR0_union_tag { /* SIUL2 DMA/Interrupt Request Select 0 */
  vuint32_t R;
  struct {
    vuint32_t DIRSR0:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR1:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR2:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR3:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR4:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR5:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR6:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR7:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_0_DIRSR0_tag;

typedef union SIUL2_0_IREER0_union_tag { /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IREE0:1;                 /* Enables rising-edge events to set DISR0[EIF0]. */
    vuint32_t IREE1:1;                 /* Enables rising-edge events to set DISR0[EIF1]. */
    vuint32_t IREE2:1;                 /* Enables rising-edge events to set DISR0[EIF2]. */
    vuint32_t IREE3:1;                 /* Enables rising-edge events to set DISR0[EIF3]. */
    vuint32_t IREE4:1;                 /* Enables rising-edge events to set DISR0[EIF4]. */
    vuint32_t IREE5:1;                 /* Enables rising-edge events to set DISR0[EIF5]. */
    vuint32_t IREE6:1;                 /* Enables rising-edge events to set DISR0[EIF6]. */
    vuint32_t IREE7:1;                 /* Enables rising-edge events to set DISR0[EIF7]. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_0_IREER0_tag;

typedef union SIUL2_0_IFEER0_union_tag { /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IFEE0:1;                 /* Enables falling-edge events to set DISR0[EIF0]. */
    vuint32_t IFEE1:1;                 /* Enables falling-edge events to set DISR0[EIF1]. */
    vuint32_t IFEE2:1;                 /* Enables falling-edge events to set DISR0[EIF2]. */
    vuint32_t IFEE3:1;                 /* Enables falling-edge events to set DISR0[EIF3]. */
    vuint32_t IFEE4:1;                 /* Enables falling-edge events to set DISR0[EIF4]. */
    vuint32_t IFEE5:1;                 /* Enables falling-edge events to set DISR0[EIF5]. */
    vuint32_t IFEE6:1;                 /* Enables falling-edge events to set DISR0[EIF6]. */
    vuint32_t IFEE7:1;                 /* Enables falling-edge events to set DISR0[EIF7]. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_0_IFEER0_tag;

typedef union SIUL2_0_IFER0_union_tag { /* SIUL2 Interrupt Filter Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IFE0:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE1:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE2:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE3:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE4:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE5:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE6:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE7:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_0_IFER0_tag;

typedef union SIUL2_0_IFMCR_union_tag { /* SIUL2 Interrupt Filter Maximum Counter */
  vuint32_t R;
  struct {
    vuint32_t MAXCNT:4;                /* Maximum Interrupt Filter Counter setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_0_IFMCR_tag;

typedef union SIUL2_0_IFCPR_union_tag { /* SIUL2 Interrupt Filter Clock Prescaler */
  vuint32_t R;
  struct {
    vuint32_t IFCP:4;                  /* Interrupt Filter Clock Prescaler setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_0_IFCPR_tag;

typedef union SIUL2_0_MSCR_union_tag { /* SIUL2 Multiplexed Signal Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:1;
    vuint32_t SMC:4;                   /* Safe Mode Control */
    vuint32_t TRC:1;                   /* Termination Resistor Control */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t RCVR:1;                  /* Receiver Select */
    vuint32_t CREF:1;                  /* Current Reference Control */
    vuint32_t PUS:1;                   /* Pull Select */
    vuint32_t PUE:1;                   /* Pull Enable */
    vuint32_t SRE:3;                   /* Slew Rate Control */
    vuint32_t RXCB:1;                  /* RX LVDS Current Boost */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t IBE:1;                   /* Input Buffer Enable */
    vuint32_t ODE:1;                   /* Open Drain Enable */
    vuint32_t OBE:1;                   /* GPIO Output Buffer Enable */
    vuint32_t _unused_22:10;           /* Reserved */
  } B;
} SIUL2_0_MSCR_tag;

typedef union SIUL2_0_IMCR_union_tag { /* SIUL2 Input Multiplexed Signal Configuration */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} SIUL2_0_IMCR_tag;

typedef union SIUL2_0_GPDO3_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO3_tag;

typedef union SIUL2_0_GPDO2_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO2_tag;

typedef union SIUL2_0_GPDO1_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO1_tag;

typedef union SIUL2_0_GPDO0_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO0_tag;

typedef union SIUL2_0_GPDO7_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO7_tag;

typedef union SIUL2_0_GPDO6_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO6_tag;

typedef union SIUL2_0_GPDO5_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO5_tag;

typedef union SIUL2_0_GPDO4_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO4_tag;

typedef union SIUL2_0_GPDO11_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO11_tag;

typedef union SIUL2_0_GPDO10_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO10_tag;

typedef union SIUL2_0_GPDO9_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO9_tag;

typedef union SIUL2_0_GPDO8_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO8_tag;

typedef union SIUL2_0_GPDO15_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO15_tag;

typedef union SIUL2_0_GPDO14_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO14_tag;

typedef union SIUL2_0_GPDO13_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO13_tag;

typedef union SIUL2_0_GPDO12_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO12_tag;

typedef union SIUL2_0_GPDO19_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO19_tag;

typedef union SIUL2_0_GPDO18_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO18_tag;

typedef union SIUL2_0_GPDO17_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO17_tag;

typedef union SIUL2_0_GPDO16_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO16_tag;

typedef union SIUL2_0_GPDO23_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO23_tag;

typedef union SIUL2_0_GPDO22_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO22_tag;

typedef union SIUL2_0_GPDO21_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO21_tag;

typedef union SIUL2_0_GPDO20_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO20_tag;

typedef union SIUL2_0_GPDO27_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO27_tag;

typedef union SIUL2_0_GPDO26_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO26_tag;

typedef union SIUL2_0_GPDO25_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO25_tag;

typedef union SIUL2_0_GPDO24_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO24_tag;

typedef union SIUL2_0_GPDO30_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO30_tag;

typedef union SIUL2_0_GPDO29_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO29_tag;

typedef union SIUL2_0_GPDO28_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO28_tag;

typedef union SIUL2_0_GPDO171_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO171_tag;

typedef union SIUL2_0_GPDO170_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO170_tag;

typedef union SIUL2_0_GPDO173_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO173_tag;

typedef union SIUL2_0_GPDO172_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDO172_tag;

typedef union SIUL2_0_GPDI3_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI3_tag;

typedef union SIUL2_0_GPDI2_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI2_tag;

typedef union SIUL2_0_GPDI1_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI1_tag;

typedef union SIUL2_0_GPDI0_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI0_tag;

typedef union SIUL2_0_GPDI7_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI7_tag;

typedef union SIUL2_0_GPDI6_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI6_tag;

typedef union SIUL2_0_GPDI5_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI5_tag;

typedef union SIUL2_0_GPDI4_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI4_tag;

typedef union SIUL2_0_GPDI11_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI11_tag;

typedef union SIUL2_0_GPDI10_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI10_tag;

typedef union SIUL2_0_GPDI9_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI9_tag;

typedef union SIUL2_0_GPDI8_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI8_tag;

typedef union SIUL2_0_GPDI15_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI15_tag;

typedef union SIUL2_0_GPDI14_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI14_tag;

typedef union SIUL2_0_GPDI13_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI13_tag;

typedef union SIUL2_0_GPDI12_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI12_tag;

typedef union SIUL2_0_GPDI19_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI19_tag;

typedef union SIUL2_0_GPDI18_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI18_tag;

typedef union SIUL2_0_GPDI17_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI17_tag;

typedef union SIUL2_0_GPDI16_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI16_tag;

typedef union SIUL2_0_GPDI23_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI23_tag;

typedef union SIUL2_0_GPDI22_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI22_tag;

typedef union SIUL2_0_GPDI21_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI21_tag;

typedef union SIUL2_0_GPDI20_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI20_tag;

typedef union SIUL2_0_GPDI27_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI27_tag;

typedef union SIUL2_0_GPDI26_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI26_tag;

typedef union SIUL2_0_GPDI25_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI25_tag;

typedef union SIUL2_0_GPDI24_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI24_tag;

typedef union SIUL2_0_GPDI30_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI30_tag;

typedef union SIUL2_0_GPDI29_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI29_tag;

typedef union SIUL2_0_GPDI28_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI28_tag;

typedef union SIUL2_0_GPDI171_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI171_tag;

typedef union SIUL2_0_GPDI170_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI170_tag;

typedef union SIUL2_0_GPDI173_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI173_tag;

typedef union SIUL2_0_GPDI172_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_0_GPDI172_tag;

typedef union SIUL2_0_PGPDO1_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_0_PGPDO1_tag;

typedef union SIUL2_0_PGPDO0_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_0_PGPDO0_tag;

typedef union SIUL2_0_PGPDO10_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} SIUL2_0_PGPDO10_tag;

typedef union SIUL2_0_PGPDI1_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_0_PGPDI1_tag;

typedef union SIUL2_0_PGPDI0_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_0_PGPDI0_tag;

typedef union SIUL2_0_PGPDI10_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} SIUL2_0_PGPDI10_tag;

typedef union SIUL2_0_MPGPDO_union_tag { /* SIUL2 Masked Parallel GPIO Pad Data Out */
  vuint32_t R;
  struct {
    vuint32_t MPPDO0:1;                /* Masked Parallel Pad Data Out 0 */
    vuint32_t MPPDO1:1;                /* Masked Parallel Pad Data Out 1 */
    vuint32_t MPPDO2:1;                /* Masked Parallel Pad Data Out 2 */
    vuint32_t MPPDO3:1;                /* Masked Parallel Pad Data Out 3 */
    vuint32_t MPPDO4:1;                /* Masked Parallel Pad Data Out 4 */
    vuint32_t MPPDO5:1;                /* Masked Parallel Pad Data Out 5 */
    vuint32_t MPPDO6:1;                /* Masked Parallel Pad Data Out 6 */
    vuint32_t MPPDO7:1;                /* Masked Parallel Pad Data Out 7 */
    vuint32_t MPPDO8:1;                /* Masked Parallel Pad Data Out 8 */
    vuint32_t MPPDO9:1;                /* Masked Parallel Pad Data Out 9 */
    vuint32_t MPPDO10:1;               /* Masked Parallel Pad Data Out 10 */
    vuint32_t MPPDO11:1;               /* Masked Parallel Pad Data Out 11 */
    vuint32_t MPPDO12:1;               /* Masked Parallel Pad Data Out 12 */
    vuint32_t MPPDO13:1;               /* Masked Parallel Pad Data Out 13 */
    vuint32_t MPPDO14:1;               /* Masked Parallel Pad Data Out 14 */
    vuint32_t MPPDO15:1;               /* Masked Parallel Pad Data Out 15 */
    vuint32_t MASK0:1;                 /* Mask Field 0 */
    vuint32_t MASK1:1;                 /* Mask Field 1 */
    vuint32_t MASK2:1;                 /* Mask Field 2 */
    vuint32_t MASK3:1;                 /* Mask Field 3 */
    vuint32_t MASK4:1;                 /* Mask Field 4 */
    vuint32_t MASK5:1;                 /* Mask Field 5 */
    vuint32_t MASK6:1;                 /* Mask Field 6 */
    vuint32_t MASK7:1;                 /* Mask Field 7 */
    vuint32_t MASK8:1;                 /* Mask Field 8 */
    vuint32_t MASK9:1;                 /* Mask Field 9 */
    vuint32_t MASK10:1;                /* Mask Field 10 */
    vuint32_t MASK11:1;                /* Mask Field 11 */
    vuint32_t MASK12:1;                /* Mask Field 12 */
    vuint32_t MASK13:1;                /* Mask Field 13 */
    vuint32_t MASK14:1;                /* Mask Field 14 */
    vuint32_t MASK15:1;                /* Mask Field 15 */
  } B;
} SIUL2_0_MPGPDO_tag;

struct SIUL2_0_tag {
  uint8_t SIUL2_0_reserved0[16];
  SIUL2_0_DISR0_tag DISR0;             /* SIUL2 DMA/Interrupt Status Flag 0 */
  uint8_t SIUL2_0_reserved1[4];
  SIUL2_0_DIRER0_tag DIRER0;           /* SIUL2 DMA/Interrupt Request Enable 0 */
  uint8_t SIUL2_0_reserved2[4];
  SIUL2_0_DIRSR0_tag DIRSR0;           /* SIUL2 DMA/Interrupt Request Select 0 */
  uint8_t SIUL2_0_reserved3[4];
  SIUL2_0_IREER0_tag IREER0;           /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  uint8_t SIUL2_0_reserved4[4];
  SIUL2_0_IFEER0_tag IFEER0;           /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  uint8_t SIUL2_0_reserved5[4];
  SIUL2_0_IFER0_tag IFER0;             /* SIUL2 Interrupt Filter Enable 0 */
  uint8_t SIUL2_0_reserved6[4];
  SIUL2_0_IFMCR_tag IFMCR[8];          /* SIUL2 Interrupt Filter Maximum Counter */
  uint8_t SIUL2_0_reserved7[96];
  SIUL2_0_IFCPR_tag IFCPR;             /* SIUL2 Interrupt Filter Clock Prescaler */
  uint8_t SIUL2_0_reserved8[380];
  SIUL2_0_MSCR_tag MSCR[503];          /* SIUL2 Multiplexed Signal Configuration Register; Valid array indices: 0-30, 170-173, 500-502 */
  uint8_t SIUL2_0_reserved9[36];
  SIUL2_0_IMCR_tag IMCR[90];           /* SIUL2 Input Multiplexed Signal Configuration; Valid array indices: 0-51, 55-89 */
  uint8_t SIUL2_0_reserved10[1880];
  SIUL2_0_GPDO3_tag GPDO3;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO2_tag GPDO2;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO1_tag GPDO1;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO0_tag GPDO0;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO7_tag GPDO7;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO6_tag GPDO6;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO5_tag GPDO5;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO4_tag GPDO4;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO11_tag GPDO11;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO10_tag GPDO10;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO9_tag GPDO9;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO8_tag GPDO8;             /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO15_tag GPDO15;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO14_tag GPDO14;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO13_tag GPDO13;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO12_tag GPDO12;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO19_tag GPDO19;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO18_tag GPDO18;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO17_tag GPDO17;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO16_tag GPDO16;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO23_tag GPDO23;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO22_tag GPDO22;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO21_tag GPDO21;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO20_tag GPDO20;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO27_tag GPDO27;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO26_tag GPDO26;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO25_tag GPDO25;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO24_tag GPDO24;           /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_0_reserved11[1];
  SIUL2_0_GPDO30_tag GPDO30;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO29_tag GPDO29;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO28_tag GPDO28;           /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_0_reserved12[136];
  SIUL2_0_GPDO171_tag GPDO171;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO170_tag GPDO170;         /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_0_reserved13[4];
  SIUL2_0_GPDO173_tag GPDO173;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_0_GPDO172_tag GPDO172;         /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_0_reserved14[336];
  SIUL2_0_GPDI3_tag GPDI3;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI2_tag GPDI2;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI1_tag GPDI1;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI0_tag GPDI0;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI7_tag GPDI7;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI6_tag GPDI6;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI5_tag GPDI5;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI4_tag GPDI4;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI11_tag GPDI11;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI10_tag GPDI10;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI9_tag GPDI9;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI8_tag GPDI8;             /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI15_tag GPDI15;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI14_tag GPDI14;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI13_tag GPDI13;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI12_tag GPDI12;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI19_tag GPDI19;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI18_tag GPDI18;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI17_tag GPDI17;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI16_tag GPDI16;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI23_tag GPDI23;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI22_tag GPDI22;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI21_tag GPDI21;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI20_tag GPDI20;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI27_tag GPDI27;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI26_tag GPDI26;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI25_tag GPDI25;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI24_tag GPDI24;           /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_0_reserved15[1];
  SIUL2_0_GPDI30_tag GPDI30;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI29_tag GPDI29;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI28_tag GPDI28;           /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_0_reserved16[136];
  SIUL2_0_GPDI171_tag GPDI171;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI170_tag GPDI170;         /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_0_reserved17[4];
  SIUL2_0_GPDI173_tag GPDI173;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_0_GPDI172_tag GPDI172;         /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_0_reserved18[336];
  SIUL2_0_PGPDO1_tag PGPDO1;           /* SIUL2 Parallel GPIO Pad Data Out */
  SIUL2_0_PGPDO0_tag PGPDO0;           /* SIUL2 Parallel GPIO Pad Data Out */
  uint8_t SIUL2_0_reserved19[18];
  SIUL2_0_PGPDO10_tag PGPDO10;         /* SIUL2 Parallel GPIO Pad Data Out */
  uint8_t SIUL2_0_reserved20[40];
  SIUL2_0_PGPDI1_tag PGPDI1;           /* SIUL2 Parallel GPIO Pad Data In */
  SIUL2_0_PGPDI0_tag PGPDI0;           /* SIUL2 Parallel GPIO Pad Data In */
  uint8_t SIUL2_0_reserved21[18];
  SIUL2_0_PGPDI10_tag PGPDI10;         /* SIUL2 Parallel GPIO Pad Data In */
  uint8_t SIUL2_0_reserved22[40];
  SIUL2_0_MPGPDO_tag MPGPDO[11];       /* SIUL2 Masked Parallel GPIO Pad Data Out; Valid array indices: 0-1, 10 */
};


/* ============================================================================
   =============================== Module: SIUL2_1 ============================
   ============================================================================ */

typedef union SIUL2_1_DISR0_union_tag { /* SIUL2 DMA/Interrupt Status Flag 0 */
  vuint32_t R;
  struct {
    vuint32_t EIF0:1;                  /* External Interrupt Status Flag 0 */
    vuint32_t EIF1:1;                  /* External Interrupt Status Flag 1 */
    vuint32_t EIF2:1;                  /* External Interrupt Status Flag 2 */
    vuint32_t EIF3:1;                  /* External Interrupt Status Flag 3 */
    vuint32_t EIF4:1;                  /* External Interrupt Status Flag 4 */
    vuint32_t EIF5:1;                  /* External Interrupt Status Flag 5 */
    vuint32_t EIF6:1;                  /* External Interrupt Status Flag 6 */
    vuint32_t EIF7:1;                  /* External Interrupt Status Flag 7 */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_1_DISR0_tag;

typedef union SIUL2_1_DIRER0_union_tag { /* SIUL2 DMA/Interrupt Request Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t EIRE0:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE1:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE2:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE3:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE4:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE5:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE6:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE7:1;                 /* External Interrupt Request Enable */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_1_DIRER0_tag;

typedef union SIUL2_1_DIRSR0_union_tag { /* SIUL2 DMA/Interrupt Request Select 0 */
  vuint32_t R;
  struct {
    vuint32_t DIRSR0:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR1:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR2:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR3:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR4:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR5:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR6:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR7:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_1_DIRSR0_tag;

typedef union SIUL2_1_IREER0_union_tag { /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IREE0:1;                 /* Enables rising-edge events to set DISR0[EIF0]. */
    vuint32_t IREE1:1;                 /* Enables rising-edge events to set DISR0[EIF1]. */
    vuint32_t IREE2:1;                 /* Enables rising-edge events to set DISR0[EIF2]. */
    vuint32_t IREE3:1;                 /* Enables rising-edge events to set DISR0[EIF3]. */
    vuint32_t IREE4:1;                 /* Enables rising-edge events to set DISR0[EIF4]. */
    vuint32_t IREE5:1;                 /* Enables rising-edge events to set DISR0[EIF5]. */
    vuint32_t IREE6:1;                 /* Enables rising-edge events to set DISR0[EIF6]. */
    vuint32_t IREE7:1;                 /* Enables rising-edge events to set DISR0[EIF7]. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_1_IREER0_tag;

typedef union SIUL2_1_IFEER0_union_tag { /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IFEE0:1;                 /* Enables falling-edge events to set DISR0[EIF0]. */
    vuint32_t IFEE1:1;                 /* Enables falling-edge events to set DISR0[EIF1]. */
    vuint32_t IFEE2:1;                 /* Enables falling-edge events to set DISR0[EIF2]. */
    vuint32_t IFEE3:1;                 /* Enables falling-edge events to set DISR0[EIF3]. */
    vuint32_t IFEE4:1;                 /* Enables falling-edge events to set DISR0[EIF4]. */
    vuint32_t IFEE5:1;                 /* Enables falling-edge events to set DISR0[EIF5]. */
    vuint32_t IFEE6:1;                 /* Enables falling-edge events to set DISR0[EIF6]. */
    vuint32_t IFEE7:1;                 /* Enables falling-edge events to set DISR0[EIF7]. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_1_IFEER0_tag;

typedef union SIUL2_1_IFER0_union_tag { /* SIUL2 Interrupt Filter Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IFE0:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE1:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE2:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE3:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE4:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE5:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE6:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE7:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_1_IFER0_tag;

typedef union SIUL2_1_IFMCR_union_tag { /* SIUL2 Interrupt Filter Maximum Counter */
  vuint32_t R;
  struct {
    vuint32_t MAXCNT:4;                /* Maximum Interrupt Filter Counter setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_1_IFMCR_tag;

typedef union SIUL2_1_IFCPR_union_tag { /* SIUL2 Interrupt Filter Clock Prescaler */
  vuint32_t R;
  struct {
    vuint32_t IFCP:4;                  /* Interrupt Filter Clock Prescaler setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_1_IFCPR_tag;

typedef union SIUL2_1_MSCR_union_tag { /* SIUL2 Multiplexed Signal Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:1;
    vuint32_t SMC:4;                   /* Safe Mode Control */
    vuint32_t TRC:1;                   /* Termination Resistor Control */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t RCVR:1;                  /* Receiver Select */
    vuint32_t CREF:1;                  /* Current Reference Control */
    vuint32_t PUS:1;                   /* Pull Select */
    vuint32_t PUE:1;                   /* Pull Enable */
    vuint32_t SRE:3;                   /* Slew Rate Control */
    vuint32_t RXCB:1;                  /* RX LVDS Current Boost */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t IBE:1;                   /* Input Buffer Enable */
    vuint32_t ODE:1;                   /* Open Drain Enable */
    vuint32_t OBE:1;                   /* GPIO Output Buffer Enable */
    vuint32_t _unused_22:10;           /* Reserved */
  } B;
} SIUL2_1_MSCR_tag;

typedef union SIUL2_1_IMCR_union_tag { /* SIUL2 Input Multiplexed Signal Configuration */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} SIUL2_1_IMCR_tag;

typedef union SIUL2_1_GPDO31_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO31_tag;

typedef union SIUL2_1_GPDO35_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO35_tag;

typedef union SIUL2_1_GPDO34_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO34_tag;

typedef union SIUL2_1_GPDO33_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO33_tag;

typedef union SIUL2_1_GPDO32_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO32_tag;

typedef union SIUL2_1_GPDO39_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO39_tag;

typedef union SIUL2_1_GPDO38_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO38_tag;

typedef union SIUL2_1_GPDO37_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO37_tag;

typedef union SIUL2_1_GPDO36_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO36_tag;

typedef union SIUL2_1_GPDO43_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO43_tag;

typedef union SIUL2_1_GPDO42_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO42_tag;

typedef union SIUL2_1_GPDO41_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO41_tag;

typedef union SIUL2_1_GPDO40_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO40_tag;

typedef union SIUL2_1_GPDO47_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO47_tag;

typedef union SIUL2_1_GPDO46_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO46_tag;

typedef union SIUL2_1_GPDO45_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO45_tag;

typedef union SIUL2_1_GPDO44_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO44_tag;

typedef union SIUL2_1_GPDO51_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO51_tag;

typedef union SIUL2_1_GPDO50_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO50_tag;

typedef union SIUL2_1_GPDO49_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO49_tag;

typedef union SIUL2_1_GPDO48_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO48_tag;

typedef union SIUL2_1_GPDO55_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO55_tag;

typedef union SIUL2_1_GPDO54_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO54_tag;

typedef union SIUL2_1_GPDO53_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO53_tag;

typedef union SIUL2_1_GPDO52_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO52_tag;

typedef union SIUL2_1_GPDO59_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO59_tag;

typedef union SIUL2_1_GPDO58_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO58_tag;

typedef union SIUL2_1_GPDO57_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO57_tag;

typedef union SIUL2_1_GPDO56_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO56_tag;

typedef union SIUL2_1_GPDO63_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO63_tag;

typedef union SIUL2_1_GPDO62_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO62_tag;

typedef union SIUL2_1_GPDO61_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO61_tag;

typedef union SIUL2_1_GPDO60_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO60_tag;

typedef union SIUL2_1_GPDO67_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO67_tag;

typedef union SIUL2_1_GPDO66_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO66_tag;

typedef union SIUL2_1_GPDO65_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO65_tag;

typedef union SIUL2_1_GPDO64_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO64_tag;

typedef union SIUL2_1_GPDO71_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO71_tag;

typedef union SIUL2_1_GPDO70_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO70_tag;

typedef union SIUL2_1_GPDO69_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO69_tag;

typedef union SIUL2_1_GPDO68_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO68_tag;

typedef union SIUL2_1_GPDO75_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO75_tag;

typedef union SIUL2_1_GPDO74_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO74_tag;

typedef union SIUL2_1_GPDO73_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO73_tag;

typedef union SIUL2_1_GPDO72_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO72_tag;

typedef union SIUL2_1_GPDO79_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO79_tag;

typedef union SIUL2_1_GPDO78_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO78_tag;

typedef union SIUL2_1_GPDO77_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO77_tag;

typedef union SIUL2_1_GPDO76_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO76_tag;

typedef union SIUL2_1_GPDO83_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO83_tag;

typedef union SIUL2_1_GPDO82_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO82_tag;

typedef union SIUL2_1_GPDO81_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO81_tag;

typedef union SIUL2_1_GPDO80_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO80_tag;

typedef union SIUL2_1_GPDO87_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO87_tag;

typedef union SIUL2_1_GPDO86_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO86_tag;

typedef union SIUL2_1_GPDO85_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO85_tag;

typedef union SIUL2_1_GPDO84_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO84_tag;

typedef union SIUL2_1_GPDO91_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO91_tag;

typedef union SIUL2_1_GPDO90_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO90_tag;

typedef union SIUL2_1_GPDO89_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO89_tag;

typedef union SIUL2_1_GPDO88_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDO88_tag;

typedef union SIUL2_1_GPDI31_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI31_tag;

typedef union SIUL2_1_GPDI35_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI35_tag;

typedef union SIUL2_1_GPDI34_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI34_tag;

typedef union SIUL2_1_GPDI33_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI33_tag;

typedef union SIUL2_1_GPDI32_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI32_tag;

typedef union SIUL2_1_GPDI39_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI39_tag;

typedef union SIUL2_1_GPDI38_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI38_tag;

typedef union SIUL2_1_GPDI37_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI37_tag;

typedef union SIUL2_1_GPDI36_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI36_tag;

typedef union SIUL2_1_GPDI43_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI43_tag;

typedef union SIUL2_1_GPDI42_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI42_tag;

typedef union SIUL2_1_GPDI41_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI41_tag;

typedef union SIUL2_1_GPDI40_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI40_tag;

typedef union SIUL2_1_GPDI47_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI47_tag;

typedef union SIUL2_1_GPDI46_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI46_tag;

typedef union SIUL2_1_GPDI45_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI45_tag;

typedef union SIUL2_1_GPDI44_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI44_tag;

typedef union SIUL2_1_GPDI51_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI51_tag;

typedef union SIUL2_1_GPDI50_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI50_tag;

typedef union SIUL2_1_GPDI49_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI49_tag;

typedef union SIUL2_1_GPDI48_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI48_tag;

typedef union SIUL2_1_GPDI55_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI55_tag;

typedef union SIUL2_1_GPDI54_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI54_tag;

typedef union SIUL2_1_GPDI53_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI53_tag;

typedef union SIUL2_1_GPDI52_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI52_tag;

typedef union SIUL2_1_GPDI59_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI59_tag;

typedef union SIUL2_1_GPDI58_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI58_tag;

typedef union SIUL2_1_GPDI57_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI57_tag;

typedef union SIUL2_1_GPDI56_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI56_tag;

typedef union SIUL2_1_GPDI63_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI63_tag;

typedef union SIUL2_1_GPDI62_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI62_tag;

typedef union SIUL2_1_GPDI61_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI61_tag;

typedef union SIUL2_1_GPDI60_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI60_tag;

typedef union SIUL2_1_GPDI67_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI67_tag;

typedef union SIUL2_1_GPDI66_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI66_tag;

typedef union SIUL2_1_GPDI65_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI65_tag;

typedef union SIUL2_1_GPDI64_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI64_tag;

typedef union SIUL2_1_GPDI71_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI71_tag;

typedef union SIUL2_1_GPDI70_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI70_tag;

typedef union SIUL2_1_GPDI69_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI69_tag;

typedef union SIUL2_1_GPDI68_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI68_tag;

typedef union SIUL2_1_GPDI75_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI75_tag;

typedef union SIUL2_1_GPDI74_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI74_tag;

typedef union SIUL2_1_GPDI73_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI73_tag;

typedef union SIUL2_1_GPDI72_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI72_tag;

typedef union SIUL2_1_GPDI79_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI79_tag;

typedef union SIUL2_1_GPDI78_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI78_tag;

typedef union SIUL2_1_GPDI77_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI77_tag;

typedef union SIUL2_1_GPDI76_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI76_tag;

typedef union SIUL2_1_GPDI83_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI83_tag;

typedef union SIUL2_1_GPDI82_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI82_tag;

typedef union SIUL2_1_GPDI81_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI81_tag;

typedef union SIUL2_1_GPDI80_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI80_tag;

typedef union SIUL2_1_GPDI87_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI87_tag;

typedef union SIUL2_1_GPDI86_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI86_tag;

typedef union SIUL2_1_GPDI85_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI85_tag;

typedef union SIUL2_1_GPDI84_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI84_tag;

typedef union SIUL2_1_GPDI91_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI91_tag;

typedef union SIUL2_1_GPDI90_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI90_tag;

typedef union SIUL2_1_GPDI89_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI89_tag;

typedef union SIUL2_1_GPDI88_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_1_GPDI88_tag;

typedef union SIUL2_1_PGPDO1_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t _unused_2:1;             /* Reserved */
    vuint16_t _unused_3:1;             /* Reserved */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t _unused_5:1;             /* Reserved */
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} SIUL2_1_PGPDO1_tag;

typedef union SIUL2_1_PGPDO3_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_1_PGPDO3_tag;

typedef union SIUL2_1_PGPDO2_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_1_PGPDO2_tag;

typedef union SIUL2_1_PGPDO5_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t _unused_2:1;             /* Reserved */
    vuint16_t _unused_3:1;             /* Reserved */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_1_PGPDO5_tag;

typedef union SIUL2_1_PGPDO4_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_1_PGPDO4_tag;

typedef union SIUL2_1_PGPDI1_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t _unused_2:1;             /* Reserved */
    vuint16_t _unused_3:1;             /* Reserved */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t _unused_5:1;             /* Reserved */
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} SIUL2_1_PGPDI1_tag;

typedef union SIUL2_1_PGPDI3_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_1_PGPDI3_tag;

typedef union SIUL2_1_PGPDI2_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_1_PGPDI2_tag;

typedef union SIUL2_1_PGPDI5_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t _unused_2:1;             /* Reserved */
    vuint16_t _unused_3:1;             /* Reserved */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_1_PGPDI5_tag;

typedef union SIUL2_1_PGPDI4_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_1_PGPDI4_tag;

typedef union SIUL2_1_MPGPDO_union_tag { /* SIUL2 Masked Parallel GPIO Pad Data Out */
  vuint32_t R;
  struct {
    vuint32_t MPPDO0:1;                /* Masked Parallel Pad Data Out 0 */
    vuint32_t MPPDO1:1;                /* Masked Parallel Pad Data Out 1 */
    vuint32_t MPPDO2:1;                /* Masked Parallel Pad Data Out 2 */
    vuint32_t MPPDO3:1;                /* Masked Parallel Pad Data Out 3 */
    vuint32_t MPPDO4:1;                /* Masked Parallel Pad Data Out 4 */
    vuint32_t MPPDO5:1;                /* Masked Parallel Pad Data Out 5 */
    vuint32_t MPPDO6:1;                /* Masked Parallel Pad Data Out 6 */
    vuint32_t MPPDO7:1;                /* Masked Parallel Pad Data Out 7 */
    vuint32_t MPPDO8:1;                /* Masked Parallel Pad Data Out 8 */
    vuint32_t MPPDO9:1;                /* Masked Parallel Pad Data Out 9 */
    vuint32_t MPPDO10:1;               /* Masked Parallel Pad Data Out 10 */
    vuint32_t MPPDO11:1;               /* Masked Parallel Pad Data Out 11 */
    vuint32_t MPPDO12:1;               /* Masked Parallel Pad Data Out 12 */
    vuint32_t MPPDO13:1;               /* Masked Parallel Pad Data Out 13 */
    vuint32_t MPPDO14:1;               /* Masked Parallel Pad Data Out 14 */
    vuint32_t MPPDO15:1;               /* Masked Parallel Pad Data Out 15 */
    vuint32_t MASK0:1;                 /* Mask Field 0 */
    vuint32_t MASK1:1;                 /* Mask Field 1 */
    vuint32_t MASK2:1;                 /* Mask Field 2 */
    vuint32_t MASK3:1;                 /* Mask Field 3 */
    vuint32_t MASK4:1;                 /* Mask Field 4 */
    vuint32_t MASK5:1;                 /* Mask Field 5 */
    vuint32_t MASK6:1;                 /* Mask Field 6 */
    vuint32_t MASK7:1;                 /* Mask Field 7 */
    vuint32_t MASK8:1;                 /* Mask Field 8 */
    vuint32_t MASK9:1;                 /* Mask Field 9 */
    vuint32_t MASK10:1;                /* Mask Field 10 */
    vuint32_t MASK11:1;                /* Mask Field 11 */
    vuint32_t MASK12:1;                /* Mask Field 12 */
    vuint32_t MASK13:1;                /* Mask Field 13 */
    vuint32_t MASK14:1;                /* Mask Field 14 */
    vuint32_t MASK15:1;                /* Mask Field 15 */
  } B;
} SIUL2_1_MPGPDO_tag;

struct SIUL2_1_tag {
  uint8_t SIUL2_1_reserved0[16];
  SIUL2_1_DISR0_tag DISR0;             /* SIUL2 DMA/Interrupt Status Flag 0 */
  uint8_t SIUL2_1_reserved1[4];
  SIUL2_1_DIRER0_tag DIRER0;           /* SIUL2 DMA/Interrupt Request Enable 0 */
  uint8_t SIUL2_1_reserved2[4];
  SIUL2_1_DIRSR0_tag DIRSR0;           /* SIUL2 DMA/Interrupt Request Select 0 */
  uint8_t SIUL2_1_reserved3[4];
  SIUL2_1_IREER0_tag IREER0;           /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  uint8_t SIUL2_1_reserved4[4];
  SIUL2_1_IFEER0_tag IFEER0;           /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  uint8_t SIUL2_1_reserved5[4];
  SIUL2_1_IFER0_tag IFER0;             /* SIUL2 Interrupt Filter Enable 0 */
  uint8_t SIUL2_1_reserved6[4];
  SIUL2_1_IFMCR_tag IFMCR[8];          /* SIUL2 Interrupt Filter Maximum Counter */
  uint8_t SIUL2_1_reserved7[96];
  SIUL2_1_IFCPR_tag IFCPR;             /* SIUL2 Interrupt Filter Clock Prescaler */
  uint8_t SIUL2_1_reserved8[380];
  SIUL2_1_MSCR_tag MSCR[504];          /* SIUL2 Multiplexed Signal Configuration Register; Valid array indices: 31-91, 500-503 */
  uint8_t SIUL2_1_reserved9[32];
  SIUL2_1_IMCR_tag IMCR[210];          /* SIUL2 Input Multiplexed Signal Configuration; Valid array indices: 127-181, 185-187, 191-209 */
  uint8_t SIUL2_1_reserved10[1428];
  SIUL2_1_GPDO31_tag GPDO31;           /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_1_reserved11[3];
  SIUL2_1_GPDO35_tag GPDO35;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO34_tag GPDO34;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO33_tag GPDO33;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO32_tag GPDO32;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO39_tag GPDO39;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO38_tag GPDO38;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO37_tag GPDO37;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO36_tag GPDO36;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO43_tag GPDO43;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO42_tag GPDO42;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO41_tag GPDO41;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO40_tag GPDO40;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO47_tag GPDO47;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO46_tag GPDO46;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO45_tag GPDO45;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO44_tag GPDO44;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO51_tag GPDO51;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO50_tag GPDO50;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO49_tag GPDO49;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO48_tag GPDO48;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO55_tag GPDO55;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO54_tag GPDO54;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO53_tag GPDO53;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO52_tag GPDO52;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO59_tag GPDO59;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO58_tag GPDO58;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO57_tag GPDO57;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO56_tag GPDO56;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO63_tag GPDO63;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO62_tag GPDO62;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO61_tag GPDO61;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO60_tag GPDO60;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO67_tag GPDO67;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO66_tag GPDO66;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO65_tag GPDO65;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO64_tag GPDO64;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO71_tag GPDO71;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO70_tag GPDO70;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO69_tag GPDO69;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO68_tag GPDO68;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO75_tag GPDO75;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO74_tag GPDO74;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO73_tag GPDO73;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO72_tag GPDO72;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO79_tag GPDO79;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO78_tag GPDO78;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO77_tag GPDO77;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO76_tag GPDO76;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO83_tag GPDO83;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO82_tag GPDO82;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO81_tag GPDO81;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO80_tag GPDO80;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO87_tag GPDO87;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO86_tag GPDO86;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO85_tag GPDO85;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO84_tag GPDO84;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO91_tag GPDO91;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO90_tag GPDO90;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO89_tag GPDO89;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_1_GPDO88_tag GPDO88;           /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_1_reserved12[448];
  SIUL2_1_GPDI31_tag GPDI31;           /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_1_reserved13[3];
  SIUL2_1_GPDI35_tag GPDI35;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI34_tag GPDI34;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI33_tag GPDI33;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI32_tag GPDI32;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI39_tag GPDI39;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI38_tag GPDI38;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI37_tag GPDI37;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI36_tag GPDI36;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI43_tag GPDI43;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI42_tag GPDI42;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI41_tag GPDI41;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI40_tag GPDI40;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI47_tag GPDI47;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI46_tag GPDI46;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI45_tag GPDI45;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI44_tag GPDI44;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI51_tag GPDI51;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI50_tag GPDI50;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI49_tag GPDI49;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI48_tag GPDI48;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI55_tag GPDI55;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI54_tag GPDI54;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI53_tag GPDI53;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI52_tag GPDI52;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI59_tag GPDI59;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI58_tag GPDI58;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI57_tag GPDI57;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI56_tag GPDI56;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI63_tag GPDI63;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI62_tag GPDI62;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI61_tag GPDI61;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI60_tag GPDI60;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI67_tag GPDI67;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI66_tag GPDI66;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI65_tag GPDI65;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI64_tag GPDI64;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI71_tag GPDI71;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI70_tag GPDI70;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI69_tag GPDI69;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI68_tag GPDI68;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI75_tag GPDI75;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI74_tag GPDI74;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI73_tag GPDI73;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI72_tag GPDI72;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI79_tag GPDI79;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI78_tag GPDI78;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI77_tag GPDI77;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI76_tag GPDI76;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI83_tag GPDI83;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI82_tag GPDI82;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI81_tag GPDI81;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI80_tag GPDI80;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI87_tag GPDI87;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI86_tag GPDI86;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI85_tag GPDI85;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI84_tag GPDI84;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI91_tag GPDI91;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI90_tag GPDI90;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI89_tag GPDI89;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_1_GPDI88_tag GPDI88;           /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_1_reserved14[420];
  SIUL2_1_PGPDO1_tag PGPDO1;           /* SIUL2 Parallel GPIO Pad Data Out */
  uint8_t SIUL2_1_reserved15[2];
  SIUL2_1_PGPDO3_tag PGPDO3;           /* SIUL2 Parallel GPIO Pad Data Out */
  SIUL2_1_PGPDO2_tag PGPDO2;           /* SIUL2 Parallel GPIO Pad Data Out */
  SIUL2_1_PGPDO5_tag PGPDO5;           /* SIUL2 Parallel GPIO Pad Data Out */
  SIUL2_1_PGPDO4_tag PGPDO4;           /* SIUL2 Parallel GPIO Pad Data Out */
  uint8_t SIUL2_1_reserved16[52];
  SIUL2_1_PGPDI1_tag PGPDI1;           /* SIUL2 Parallel GPIO Pad Data In */
  uint8_t SIUL2_1_reserved17[2];
  SIUL2_1_PGPDI3_tag PGPDI3;           /* SIUL2 Parallel GPIO Pad Data In */
  SIUL2_1_PGPDI2_tag PGPDI2;           /* SIUL2 Parallel GPIO Pad Data In */
  SIUL2_1_PGPDI5_tag PGPDI5;           /* SIUL2 Parallel GPIO Pad Data In */
  SIUL2_1_PGPDI4_tag PGPDI4;           /* SIUL2 Parallel GPIO Pad Data In */
  uint8_t SIUL2_1_reserved18[52];
  SIUL2_1_MPGPDO_tag MPGPDO[6];        /* SIUL2 Masked Parallel GPIO Pad Data Out; Valid array indices: 1-5 */
};


/* ============================================================================
   =============================== Module: SIUL2_3 ============================
   ============================================================================ */

typedef union SIUL2_3_DISR0_union_tag { /* SIUL2 DMA/Interrupt Status Flag 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_3_DISR0_tag;

typedef union SIUL2_3_DIRER0_union_tag { /* SIUL2 DMA/Interrupt Request Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_3_DIRER0_tag;

typedef union SIUL2_3_DIRSR0_union_tag { /* SIUL2 DMA/Interrupt Request Select 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_3_DIRSR0_tag;

typedef union SIUL2_3_IREER0_union_tag { /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_3_IREER0_tag;

typedef union SIUL2_3_IFEER0_union_tag { /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_3_IFEER0_tag;

typedef union SIUL2_3_IFER0_union_tag { /* SIUL2 Interrupt Filter Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_3_IFER0_tag;

typedef union SIUL2_3_IFCPR_union_tag { /* SIUL2 Interrupt Filter Clock Prescaler */
  vuint32_t R;
  struct {
    vuint32_t IFCP:4;                  /* Interrupt Filter Clock Prescaler setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_3_IFCPR_tag;

typedef union SIUL2_3_IMCR_union_tag { /* SIUL2 Input Multiplexed Signal Configuration */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} SIUL2_3_IMCR_tag;

struct SIUL2_3_tag {
  uint8_t SIUL2_3_reserved0[16];
  SIUL2_3_DISR0_tag DISR0;             /* SIUL2 DMA/Interrupt Status Flag 0 */
  uint8_t SIUL2_3_reserved1[4];
  SIUL2_3_DIRER0_tag DIRER0;           /* SIUL2 DMA/Interrupt Request Enable 0 */
  uint8_t SIUL2_3_reserved2[4];
  SIUL2_3_DIRSR0_tag DIRSR0;           /* SIUL2 DMA/Interrupt Request Select 0 */
  uint8_t SIUL2_3_reserved3[4];
  SIUL2_3_IREER0_tag IREER0;           /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  uint8_t SIUL2_3_reserved4[4];
  SIUL2_3_IFEER0_tag IFEER0;           /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  uint8_t SIUL2_3_reserved5[4];
  SIUL2_3_IFER0_tag IFER0;             /* SIUL2 Interrupt Filter Enable 0 */
  uint8_t SIUL2_3_reserved6[132];
  SIUL2_3_IFCPR_tag IFCPR;             /* SIUL2 Interrupt Filter Clock Prescaler */
  uint8_t SIUL2_3_reserved7[2428];
  SIUL2_3_IMCR_tag IMCR[24];           /* SIUL2 Input Multiplexed Signal Configuration */
};


/* ============================================================================
   =============================== Module: SIUL2_4 ============================
   ============================================================================ */

typedef union SIUL2_4_MIDR1_union_tag { /* SIUL2 MCU ID 1 */
  vuint32_t R;
  struct {
    vuint32_t MINOR_MASK:4;            /* Minor Mask Revision */
    vuint32_t MAJOR_MASK:4;            /* Major Mask Revision */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t PART_NO:10;              /* Part Number */
    vuint32_t PRODUCT_LINE_LETTER:6;   /* Product Line Letter */
  } B;
} SIUL2_4_MIDR1_tag;

typedef union SIUL2_4_MIDR2_union_tag { /* SIUL2 MCU ID 2 */
  vuint32_t R;
  struct {
    vuint32_t FLASH_SIZE_CODE:8;       /* Flash Size Code */
    vuint32_t FLASH_SIZE_DATA:4;       /* Flash Size Data */
    vuint32_t FLASH_DATA:2;            /* Flash Data */
    vuint32_t FLASH_CODE:2;            /* Flash Code */
    vuint32_t FREQUENCY:4;             /* Frequency */
    vuint32_t PACKAGE:6;               /* Package */
    vuint32_t TEMPERATURE:3;           /* Temperature */
    vuint32_t TECHNOLOGY:3;            /* Technology */
  } B;
} SIUL2_4_MIDR2_tag;

typedef union SIUL2_4_DISR0_union_tag { /* SIUL2 DMA/Interrupt Status Flag 0 */
  vuint32_t R;
  struct {
    vuint32_t EIF0:1;                  /* External Interrupt Status Flag 0 */
    vuint32_t EIF1:1;                  /* External Interrupt Status Flag 1 */
    vuint32_t EIF2:1;                  /* External Interrupt Status Flag 2 */
    vuint32_t EIF3:1;                  /* External Interrupt Status Flag 3 */
    vuint32_t EIF4:1;                  /* External Interrupt Status Flag 4 */
    vuint32_t EIF5:1;                  /* External Interrupt Status Flag 5 */
    vuint32_t EIF6:1;                  /* External Interrupt Status Flag 6 */
    vuint32_t EIF7:1;                  /* External Interrupt Status Flag 7 */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_4_DISR0_tag;

typedef union SIUL2_4_DIRER0_union_tag { /* SIUL2 DMA/Interrupt Request Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t EIRE0:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE1:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE2:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE3:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE4:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE5:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE6:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE7:1;                 /* External Interrupt Request Enable */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_4_DIRER0_tag;

typedef union SIUL2_4_DIRSR0_union_tag { /* SIUL2 DMA/Interrupt Request Select 0 */
  vuint32_t R;
  struct {
    vuint32_t DIRSR0:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR1:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR2:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR3:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR4:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR5:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR6:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR7:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_4_DIRSR0_tag;

typedef union SIUL2_4_IREER0_union_tag { /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IREE0:1;                 /* Enables rising-edge events to set DISR0[EIF0]. */
    vuint32_t IREE1:1;                 /* Enables rising-edge events to set DISR0[EIF1]. */
    vuint32_t IREE2:1;                 /* Enables rising-edge events to set DISR0[EIF2]. */
    vuint32_t IREE3:1;                 /* Enables rising-edge events to set DISR0[EIF3]. */
    vuint32_t IREE4:1;                 /* Enables rising-edge events to set DISR0[EIF4]. */
    vuint32_t IREE5:1;                 /* Enables rising-edge events to set DISR0[EIF5]. */
    vuint32_t IREE6:1;                 /* Enables rising-edge events to set DISR0[EIF6]. */
    vuint32_t IREE7:1;                 /* Enables rising-edge events to set DISR0[EIF7]. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_4_IREER0_tag;

typedef union SIUL2_4_IFEER0_union_tag { /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IFEE0:1;                 /* Enables falling-edge events to set DISR0[EIF0]. */
    vuint32_t IFEE1:1;                 /* Enables falling-edge events to set DISR0[EIF1]. */
    vuint32_t IFEE2:1;                 /* Enables falling-edge events to set DISR0[EIF2]. */
    vuint32_t IFEE3:1;                 /* Enables falling-edge events to set DISR0[EIF3]. */
    vuint32_t IFEE4:1;                 /* Enables falling-edge events to set DISR0[EIF4]. */
    vuint32_t IFEE5:1;                 /* Enables falling-edge events to set DISR0[EIF5]. */
    vuint32_t IFEE6:1;                 /* Enables falling-edge events to set DISR0[EIF6]. */
    vuint32_t IFEE7:1;                 /* Enables falling-edge events to set DISR0[EIF7]. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_4_IFEER0_tag;

typedef union SIUL2_4_IFER0_union_tag { /* SIUL2 Interrupt Filter Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IFE0:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE1:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE2:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE3:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE4:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE5:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE6:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE7:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t _unused_8:1;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t _unused_16:1;            /* Reserved */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_4_IFER0_tag;

typedef union SIUL2_4_IFMCR_union_tag { /* SIUL2 Interrupt Filter Maximum Counter */
  vuint32_t R;
  struct {
    vuint32_t MAXCNT:4;                /* Maximum Interrupt Filter Counter setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_4_IFMCR_tag;

typedef union SIUL2_4_IFCPR_union_tag { /* SIUL2 Interrupt Filter Clock Prescaler */
  vuint32_t R;
  struct {
    vuint32_t IFCP:4;                  /* Interrupt Filter Clock Prescaler setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_4_IFCPR_tag;

typedef union SIUL2_4_MIDR3_union_tag { /* SIUL2 MCU ID 3 */
  vuint32_t R;
  struct {
    vuint32_t SYSTEM_RAM_SIZE:6;       /* System RAM Size */
    vuint32_t _unused_6:4;             /* Reserved */
    vuint32_t PART_NO_LETTER:6;        /* Part Number Letter */
    vuint32_t PRODUCT_FAMILY_NO:10;    /* Product Family Number */
    vuint32_t PRODUCT_FAMILY_LETTER:6; /* Product Family Letter */
  } B;
} SIUL2_4_MIDR3_tag;

typedef union SIUL2_4_MIDR4_union_tag { /* SIUL2 MCU ID 4 */
  vuint32_t R;
  struct {
    vuint32_t AE0:1;                   /* Application Extension Subsystem 0 */
    vuint32_t AE_RC:1;                 /* Application Extension Subsystem Reset */
    vuint32_t DIP:1;                   /* diPortSD */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:4;             /* Reserved */
    vuint32_t GTM:1;                   /* Generic Timer IP module */
    vuint32_t CANXL_FUNC:1;            /* CANXL Function */
    vuint32_t CANXL_INST:1;            /* CANXL Instances */
    vuint32_t MCP:1;                   /* Math Coprocessor */
    vuint32_t HPBF:1;                  /* High Performance Buffered Flash */
    vuint32_t OTA:1;                   /* Over The Air */
    vuint32_t SERDES:1;                /* SerDes subsystem */
    vuint32_t COMPUTE_DIE_PART_NUMBER:10; /* Compute Die Part Number */
    vuint32_t _unused_26:2;            /* Reserved */
    vuint32_t FLASH_CONFIG:2;          /* Flash Configuration */
    vuint32_t FLASH_VENDOR:2;          /* Flash Vendor */
  } B;
} SIUL2_4_MIDR4_tag;

typedef union SIUL2_4_MSCR_union_tag { /* SIUL2 Multiplexed Signal Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:1;
    vuint32_t SMC:4;                   /* Safe Mode Control */
    vuint32_t TRC:1;                   /* Termination Resistor Control */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t RCVR:1;                  /* Receiver Select */
    vuint32_t CREF:1;                  /* Current Reference Control */
    vuint32_t PUS:1;                   /* Pull Select */
    vuint32_t PUE:1;                   /* Pull Enable */
    vuint32_t SRE:3;                   /* Slew Rate Control */
    vuint32_t RXCB:1;                  /* RX LVDS Current Boost */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t IBE:1;                   /* Input Buffer Enable */
    vuint32_t ODE:1;                   /* Open Drain Enable */
    vuint32_t OBE:1;                   /* GPIO Output Buffer Enable */
    vuint32_t _unused_22:10;           /* Reserved */
  } B;
} SIUL2_4_MSCR_tag;

typedef union SIUL2_4_IMCR_union_tag { /* SIUL2 Input Multiplexed Signal Configuration */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} SIUL2_4_IMCR_tag;

typedef union SIUL2_4_GPDO95_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO95_tag;

typedef union SIUL2_4_GPDO94_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO94_tag;

typedef union SIUL2_4_GPDO93_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO93_tag;

typedef union SIUL2_4_GPDO92_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO92_tag;

typedef union SIUL2_4_GPDO99_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO99_tag;

typedef union SIUL2_4_GPDO98_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO98_tag;

typedef union SIUL2_4_GPDO97_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO97_tag;

typedef union SIUL2_4_GPDO96_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO96_tag;

typedef union SIUL2_4_GPDO103_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO103_tag;

typedef union SIUL2_4_GPDO102_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO102_tag;

typedef union SIUL2_4_GPDO101_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO101_tag;

typedef union SIUL2_4_GPDO100_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO100_tag;

typedef union SIUL2_4_GPDO107_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO107_tag;

typedef union SIUL2_4_GPDO106_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO106_tag;

typedef union SIUL2_4_GPDO105_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO105_tag;

typedef union SIUL2_4_GPDO104_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO104_tag;

typedef union SIUL2_4_GPDO111_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO111_tag;

typedef union SIUL2_4_GPDO110_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO110_tag;

typedef union SIUL2_4_GPDO109_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO109_tag;

typedef union SIUL2_4_GPDO108_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO108_tag;

typedef union SIUL2_4_GPDO115_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO115_tag;

typedef union SIUL2_4_GPDO114_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO114_tag;

typedef union SIUL2_4_GPDO113_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO113_tag;

typedef union SIUL2_4_GPDO112_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO112_tag;

typedef union SIUL2_4_GPDO119_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO119_tag;

typedef union SIUL2_4_GPDO118_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO118_tag;

typedef union SIUL2_4_GPDO117_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO117_tag;

typedef union SIUL2_4_GPDO116_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO116_tag;

typedef union SIUL2_4_GPDO123_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO123_tag;

typedef union SIUL2_4_GPDO122_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO122_tag;

typedef union SIUL2_4_GPDO121_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO121_tag;

typedef union SIUL2_4_GPDO120_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO120_tag;

typedef union SIUL2_4_GPDO127_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO127_tag;

typedef union SIUL2_4_GPDO126_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO126_tag;

typedef union SIUL2_4_GPDO125_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO125_tag;

typedef union SIUL2_4_GPDO124_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO124_tag;

typedef union SIUL2_4_GPDO131_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO131_tag;

typedef union SIUL2_4_GPDO130_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO130_tag;

typedef union SIUL2_4_GPDO129_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO129_tag;

typedef union SIUL2_4_GPDO128_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO128_tag;

typedef union SIUL2_4_GPDO135_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO135_tag;

typedef union SIUL2_4_GPDO134_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO134_tag;

typedef union SIUL2_4_GPDO133_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO133_tag;

typedef union SIUL2_4_GPDO132_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO132_tag;

typedef union SIUL2_4_GPDO139_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO139_tag;

typedef union SIUL2_4_GPDO138_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO138_tag;

typedef union SIUL2_4_GPDO137_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO137_tag;

typedef union SIUL2_4_GPDO136_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO136_tag;

typedef union SIUL2_4_GPDO143_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO143_tag;

typedef union SIUL2_4_GPDO142_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO142_tag;

typedef union SIUL2_4_GPDO141_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO141_tag;

typedef union SIUL2_4_GPDO140_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO140_tag;

typedef union SIUL2_4_GPDO145_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO145_tag;

typedef union SIUL2_4_GPDO144_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDO144_tag;

typedef union SIUL2_4_GPDI95_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI95_tag;

typedef union SIUL2_4_GPDI94_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI94_tag;

typedef union SIUL2_4_GPDI93_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI93_tag;

typedef union SIUL2_4_GPDI92_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI92_tag;

typedef union SIUL2_4_GPDI99_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI99_tag;

typedef union SIUL2_4_GPDI98_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI98_tag;

typedef union SIUL2_4_GPDI97_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI97_tag;

typedef union SIUL2_4_GPDI96_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI96_tag;

typedef union SIUL2_4_GPDI103_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI103_tag;

typedef union SIUL2_4_GPDI102_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI102_tag;

typedef union SIUL2_4_GPDI101_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI101_tag;

typedef union SIUL2_4_GPDI100_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI100_tag;

typedef union SIUL2_4_GPDI107_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI107_tag;

typedef union SIUL2_4_GPDI106_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI106_tag;

typedef union SIUL2_4_GPDI105_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI105_tag;

typedef union SIUL2_4_GPDI104_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI104_tag;

typedef union SIUL2_4_GPDI111_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI111_tag;

typedef union SIUL2_4_GPDI110_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI110_tag;

typedef union SIUL2_4_GPDI109_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI109_tag;

typedef union SIUL2_4_GPDI108_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI108_tag;

typedef union SIUL2_4_GPDI115_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI115_tag;

typedef union SIUL2_4_GPDI114_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI114_tag;

typedef union SIUL2_4_GPDI113_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI113_tag;

typedef union SIUL2_4_GPDI112_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI112_tag;

typedef union SIUL2_4_GPDI119_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI119_tag;

typedef union SIUL2_4_GPDI118_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI118_tag;

typedef union SIUL2_4_GPDI117_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI117_tag;

typedef union SIUL2_4_GPDI116_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI116_tag;

typedef union SIUL2_4_GPDI123_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI123_tag;

typedef union SIUL2_4_GPDI122_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI122_tag;

typedef union SIUL2_4_GPDI121_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI121_tag;

typedef union SIUL2_4_GPDI120_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI120_tag;

typedef union SIUL2_4_GPDI127_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI127_tag;

typedef union SIUL2_4_GPDI126_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI126_tag;

typedef union SIUL2_4_GPDI125_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI125_tag;

typedef union SIUL2_4_GPDI124_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI124_tag;

typedef union SIUL2_4_GPDI131_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI131_tag;

typedef union SIUL2_4_GPDI130_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI130_tag;

typedef union SIUL2_4_GPDI129_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI129_tag;

typedef union SIUL2_4_GPDI128_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI128_tag;

typedef union SIUL2_4_GPDI135_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI135_tag;

typedef union SIUL2_4_GPDI134_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI134_tag;

typedef union SIUL2_4_GPDI133_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI133_tag;

typedef union SIUL2_4_GPDI132_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI132_tag;

typedef union SIUL2_4_GPDI139_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI139_tag;

typedef union SIUL2_4_GPDI138_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI138_tag;

typedef union SIUL2_4_GPDI137_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI137_tag;

typedef union SIUL2_4_GPDI136_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI136_tag;

typedef union SIUL2_4_GPDI143_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI143_tag;

typedef union SIUL2_4_GPDI142_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI142_tag;

typedef union SIUL2_4_GPDI141_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI141_tag;

typedef union SIUL2_4_GPDI140_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI140_tag;

typedef union SIUL2_4_GPDI145_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI145_tag;

typedef union SIUL2_4_GPDI144_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_4_GPDI144_tag;

typedef union SIUL2_4_PGPDO5_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t _unused_5:1;             /* Reserved */
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} SIUL2_4_PGPDO5_tag;

typedef union SIUL2_4_PGPDO7_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_4_PGPDO7_tag;

typedef union SIUL2_4_PGPDO6_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_4_PGPDO6_tag;

typedef union SIUL2_4_PGPDO9_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t _unused_2:1;             /* Reserved */
    vuint16_t _unused_3:1;             /* Reserved */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t _unused_5:1;             /* Reserved */
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_4_PGPDO9_tag;

typedef union SIUL2_4_PGPDO8_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_4_PGPDO8_tag;

typedef union SIUL2_4_PGPDI5_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t _unused_5:1;             /* Reserved */
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} SIUL2_4_PGPDI5_tag;

typedef union SIUL2_4_PGPDI7_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_4_PGPDI7_tag;

typedef union SIUL2_4_PGPDI6_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_4_PGPDI6_tag;

typedef union SIUL2_4_PGPDI9_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t _unused_2:1;             /* Reserved */
    vuint16_t _unused_3:1;             /* Reserved */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t _unused_5:1;             /* Reserved */
    vuint16_t _unused_6:1;             /* Reserved */
    vuint16_t _unused_7:1;             /* Reserved */
    vuint16_t _unused_8:1;             /* Reserved */
    vuint16_t _unused_9:1;             /* Reserved */
    vuint16_t _unused_10:1;            /* Reserved */
    vuint16_t _unused_11:1;            /* Reserved */
    vuint16_t _unused_12:1;            /* Reserved */
    vuint16_t _unused_13:1;            /* Reserved */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_4_PGPDI9_tag;

typedef union SIUL2_4_PGPDI8_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_4_PGPDI8_tag;

typedef union SIUL2_4_MPGPDO_union_tag { /* SIUL2 Masked Parallel GPIO Pad Data Out */
  vuint32_t R;
  struct {
    vuint32_t MPPDO0:1;                /* Masked Parallel Pad Data Out 0 */
    vuint32_t MPPDO1:1;                /* Masked Parallel Pad Data Out 1 */
    vuint32_t MPPDO2:1;                /* Masked Parallel Pad Data Out 2 */
    vuint32_t MPPDO3:1;                /* Masked Parallel Pad Data Out 3 */
    vuint32_t MPPDO4:1;                /* Masked Parallel Pad Data Out 4 */
    vuint32_t MPPDO5:1;                /* Masked Parallel Pad Data Out 5 */
    vuint32_t MPPDO6:1;                /* Masked Parallel Pad Data Out 6 */
    vuint32_t MPPDO7:1;                /* Masked Parallel Pad Data Out 7 */
    vuint32_t MPPDO8:1;                /* Masked Parallel Pad Data Out 8 */
    vuint32_t MPPDO9:1;                /* Masked Parallel Pad Data Out 9 */
    vuint32_t MPPDO10:1;               /* Masked Parallel Pad Data Out 10 */
    vuint32_t MPPDO11:1;               /* Masked Parallel Pad Data Out 11 */
    vuint32_t MPPDO12:1;               /* Masked Parallel Pad Data Out 12 */
    vuint32_t MPPDO13:1;               /* Masked Parallel Pad Data Out 13 */
    vuint32_t MPPDO14:1;               /* Masked Parallel Pad Data Out 14 */
    vuint32_t MPPDO15:1;               /* Masked Parallel Pad Data Out 15 */
    vuint32_t MASK0:1;                 /* Mask Field 0 */
    vuint32_t MASK1:1;                 /* Mask Field 1 */
    vuint32_t MASK2:1;                 /* Mask Field 2 */
    vuint32_t MASK3:1;                 /* Mask Field 3 */
    vuint32_t MASK4:1;                 /* Mask Field 4 */
    vuint32_t MASK5:1;                 /* Mask Field 5 */
    vuint32_t MASK6:1;                 /* Mask Field 6 */
    vuint32_t MASK7:1;                 /* Mask Field 7 */
    vuint32_t MASK8:1;                 /* Mask Field 8 */
    vuint32_t MASK9:1;                 /* Mask Field 9 */
    vuint32_t MASK10:1;                /* Mask Field 10 */
    vuint32_t MASK11:1;                /* Mask Field 11 */
    vuint32_t MASK12:1;                /* Mask Field 12 */
    vuint32_t MASK13:1;                /* Mask Field 13 */
    vuint32_t MASK14:1;                /* Mask Field 14 */
    vuint32_t MASK15:1;                /* Mask Field 15 */
  } B;
} SIUL2_4_MPGPDO_tag;

struct SIUL2_4_tag {
  uint8_t SIUL2_4_reserved0[4];
  SIUL2_4_MIDR1_tag MIDR1;             /* SIUL2 MCU ID 1 */
  SIUL2_4_MIDR2_tag MIDR2;             /* SIUL2 MCU ID 2 */
  uint8_t SIUL2_4_reserved1[4];
  SIUL2_4_DISR0_tag DISR0;             /* SIUL2 DMA/Interrupt Status Flag 0 */
  uint8_t SIUL2_4_reserved2[4];
  SIUL2_4_DIRER0_tag DIRER0;           /* SIUL2 DMA/Interrupt Request Enable 0 */
  uint8_t SIUL2_4_reserved3[4];
  SIUL2_4_DIRSR0_tag DIRSR0;           /* SIUL2 DMA/Interrupt Request Select 0 */
  uint8_t SIUL2_4_reserved4[4];
  SIUL2_4_IREER0_tag IREER0;           /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  uint8_t SIUL2_4_reserved5[4];
  SIUL2_4_IFEER0_tag IFEER0;           /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  uint8_t SIUL2_4_reserved6[4];
  SIUL2_4_IFER0_tag IFER0;             /* SIUL2 Interrupt Filter Enable 0 */
  uint8_t SIUL2_4_reserved7[4];
  SIUL2_4_IFMCR_tag IFMCR[8];          /* SIUL2 Interrupt Filter Maximum Counter */
  uint8_t SIUL2_4_reserved8[96];
  SIUL2_4_IFCPR_tag IFCPR;             /* SIUL2 Interrupt Filter Clock Prescaler */
  uint8_t SIUL2_4_reserved9[316];
  SIUL2_4_MIDR3_tag MIDR3;             /* SIUL2 MCU ID 3 */
  SIUL2_4_MIDR4_tag MIDR4;             /* SIUL2 MCU ID 4 */
  uint8_t SIUL2_4_reserved10[56];
  SIUL2_4_MSCR_tag MSCR[146];          /* SIUL2 Multiplexed Signal Configuration Register; Valid array indices: 92-145 */
  uint8_t SIUL2_4_reserved11[1464];
  SIUL2_4_IMCR_tag IMCR[372];          /* SIUL2 Input Multiplexed Signal Configuration; Valid array indices: 256-258, 292-295, 302-312, 316-371 */
  uint8_t SIUL2_4_reserved12[844];
  SIUL2_4_GPDO95_tag GPDO95;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO94_tag GPDO94;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO93_tag GPDO93;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO92_tag GPDO92;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO99_tag GPDO99;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO98_tag GPDO98;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO97_tag GPDO97;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO96_tag GPDO96;           /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO103_tag GPDO103;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO102_tag GPDO102;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO101_tag GPDO101;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO100_tag GPDO100;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO107_tag GPDO107;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO106_tag GPDO106;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO105_tag GPDO105;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO104_tag GPDO104;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO111_tag GPDO111;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO110_tag GPDO110;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO109_tag GPDO109;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO108_tag GPDO108;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO115_tag GPDO115;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO114_tag GPDO114;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO113_tag GPDO113;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO112_tag GPDO112;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO119_tag GPDO119;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO118_tag GPDO118;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO117_tag GPDO117;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO116_tag GPDO116;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO123_tag GPDO123;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO122_tag GPDO122;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO121_tag GPDO121;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO120_tag GPDO120;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO127_tag GPDO127;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO126_tag GPDO126;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO125_tag GPDO125;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO124_tag GPDO124;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO131_tag GPDO131;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO130_tag GPDO130;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO129_tag GPDO129;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO128_tag GPDO128;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO135_tag GPDO135;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO134_tag GPDO134;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO133_tag GPDO133;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO132_tag GPDO132;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO139_tag GPDO139;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO138_tag GPDO138;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO137_tag GPDO137;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO136_tag GPDO136;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO143_tag GPDO143;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO142_tag GPDO142;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO141_tag GPDO141;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO140_tag GPDO140;         /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_4_reserved13[2];
  SIUL2_4_GPDO145_tag GPDO145;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_4_GPDO144_tag GPDO144;         /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_4_reserved14[456];
  SIUL2_4_GPDI95_tag GPDI95;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI94_tag GPDI94;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI93_tag GPDI93;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI92_tag GPDI92;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI99_tag GPDI99;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI98_tag GPDI98;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI97_tag GPDI97;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI96_tag GPDI96;           /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI103_tag GPDI103;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI102_tag GPDI102;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI101_tag GPDI101;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI100_tag GPDI100;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI107_tag GPDI107;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI106_tag GPDI106;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI105_tag GPDI105;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI104_tag GPDI104;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI111_tag GPDI111;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI110_tag GPDI110;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI109_tag GPDI109;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI108_tag GPDI108;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI115_tag GPDI115;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI114_tag GPDI114;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI113_tag GPDI113;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI112_tag GPDI112;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI119_tag GPDI119;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI118_tag GPDI118;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI117_tag GPDI117;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI116_tag GPDI116;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI123_tag GPDI123;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI122_tag GPDI122;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI121_tag GPDI121;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI120_tag GPDI120;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI127_tag GPDI127;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI126_tag GPDI126;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI125_tag GPDI125;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI124_tag GPDI124;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI131_tag GPDI131;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI130_tag GPDI130;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI129_tag GPDI129;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI128_tag GPDI128;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI135_tag GPDI135;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI134_tag GPDI134;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI133_tag GPDI133;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI132_tag GPDI132;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI139_tag GPDI139;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI138_tag GPDI138;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI137_tag GPDI137;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI136_tag GPDI136;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI143_tag GPDI143;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI142_tag GPDI142;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI141_tag GPDI141;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI140_tag GPDI140;         /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_4_reserved15[2];
  SIUL2_4_GPDI145_tag GPDI145;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_4_GPDI144_tag GPDI144;         /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_4_reserved16[372];
  SIUL2_4_PGPDO5_tag PGPDO5;           /* SIUL2 Parallel GPIO Pad Data Out */
  uint8_t SIUL2_4_reserved17[2];
  SIUL2_4_PGPDO7_tag PGPDO7;           /* SIUL2 Parallel GPIO Pad Data Out */
  SIUL2_4_PGPDO6_tag PGPDO6;           /* SIUL2 Parallel GPIO Pad Data Out */
  SIUL2_4_PGPDO9_tag PGPDO9;           /* SIUL2 Parallel GPIO Pad Data Out */
  SIUL2_4_PGPDO8_tag PGPDO8;           /* SIUL2 Parallel GPIO Pad Data Out */
  uint8_t SIUL2_4_reserved18[52];
  SIUL2_4_PGPDI5_tag PGPDI5;           /* SIUL2 Parallel GPIO Pad Data In */
  uint8_t SIUL2_4_reserved19[2];
  SIUL2_4_PGPDI7_tag PGPDI7;           /* SIUL2 Parallel GPIO Pad Data In */
  SIUL2_4_PGPDI6_tag PGPDI6;           /* SIUL2 Parallel GPIO Pad Data In */
  SIUL2_4_PGPDI9_tag PGPDI9;           /* SIUL2 Parallel GPIO Pad Data In */
  SIUL2_4_PGPDI8_tag PGPDI8;           /* SIUL2 Parallel GPIO Pad Data In */
  uint8_t SIUL2_4_reserved20[44];
  SIUL2_4_MPGPDO_tag MPGPDO[10];       /* SIUL2 Masked Parallel GPIO Pad Data Out; Valid array indices: 5-9 */
};


/* ============================================================================
   =============================== Module: SIUL2_5 ============================
   ============================================================================ */

typedef union SIUL2_5_DISR0_union_tag { /* SIUL2 DMA/Interrupt Status Flag 0 */
  vuint32_t R;
  struct {
    vuint32_t EIF0:1;                  /* External Interrupt Status Flag 0 */
    vuint32_t EIF1:1;                  /* External Interrupt Status Flag 1 */
    vuint32_t EIF2:1;                  /* External Interrupt Status Flag 2 */
    vuint32_t EIF3:1;                  /* External Interrupt Status Flag 3 */
    vuint32_t EIF4:1;                  /* External Interrupt Status Flag 4 */
    vuint32_t EIF5:1;                  /* External Interrupt Status Flag 5 */
    vuint32_t EIF6:1;                  /* External Interrupt Status Flag 6 */
    vuint32_t EIF7:1;                  /* External Interrupt Status Flag 7 */
    vuint32_t EIF8:1;                  /* External Interrupt Status Flag 8 */
    vuint32_t EIF9:1;                  /* External Interrupt Status Flag 9 */
    vuint32_t EIF10:1;                 /* External Interrupt Status Flag 10 */
    vuint32_t EIF11:1;                 /* External Interrupt Status Flag 11 */
    vuint32_t EIF12:1;                 /* External Interrupt Status Flag 12 */
    vuint32_t EIF13:1;                 /* External Interrupt Status Flag 13 */
    vuint32_t EIF14:1;                 /* External Interrupt Status Flag 14 */
    vuint32_t EIF15:1;                 /* External Interrupt Status Flag 15 */
    vuint32_t EIF16:1;                 /* External Interrupt Status Flag 16 */
    vuint32_t EIF17:1;                 /* External Interrupt Status Flag 17 */
    vuint32_t EIF18:1;                 /* External Interrupt Status Flag 18 */
    vuint32_t EIF19:1;                 /* External Interrupt Status Flag 19 */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_5_DISR0_tag;

typedef union SIUL2_5_DIRER0_union_tag { /* SIUL2 DMA/Interrupt Request Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t EIRE0:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE1:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE2:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE3:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE4:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE5:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE6:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE7:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE8:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE9:1;                 /* External Interrupt Request Enable */
    vuint32_t EIRE10:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE11:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE12:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE13:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE14:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE15:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE16:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE17:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE18:1;                /* External Interrupt Request Enable */
    vuint32_t EIRE19:1;                /* External Interrupt Request Enable */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_5_DIRER0_tag;

typedef union SIUL2_5_DIRSR0_union_tag { /* SIUL2 DMA/Interrupt Request Select 0 */
  vuint32_t R;
  struct {
    vuint32_t DIRSR0:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR1:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR2:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR3:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR4:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR5:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR6:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR7:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR8:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR9:1;                /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR10:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR11:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR12:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR13:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR14:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR15:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR16:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR17:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR18:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t DIRSR19:1;               /* DMA/Interrupt Request Select Register */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_5_DIRSR0_tag;

typedef union SIUL2_5_IREER0_union_tag { /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IREE0:1;                 /* Enables rising-edge events to set DISR0[EIF0]. */
    vuint32_t IREE1:1;                 /* Enables rising-edge events to set DISR0[EIF1]. */
    vuint32_t IREE2:1;                 /* Enables rising-edge events to set DISR0[EIF2]. */
    vuint32_t IREE3:1;                 /* Enables rising-edge events to set DISR0[EIF3]. */
    vuint32_t IREE4:1;                 /* Enables rising-edge events to set DISR0[EIF4]. */
    vuint32_t IREE5:1;                 /* Enables rising-edge events to set DISR0[EIF5]. */
    vuint32_t IREE6:1;                 /* Enables rising-edge events to set DISR0[EIF6]. */
    vuint32_t IREE7:1;                 /* Enables rising-edge events to set DISR0[EIF7]. */
    vuint32_t IREE8:1;                 /* Enables rising-edge events to set DISR0[EIF8]. */
    vuint32_t IREE9:1;                 /* Enables rising-edge events to set DISR0[EIF9]. */
    vuint32_t IREE10:1;                /* Enables rising-edge events to set DISR0[EIF10]. */
    vuint32_t IREE11:1;                /* Enables rising-edge events to set DISR0[EIF11]. */
    vuint32_t IREE12:1;                /* Enables rising-edge events to set DISR0[EIF12]. */
    vuint32_t IREE13:1;                /* Enables rising-edge events to set DISR0[EIF13]. */
    vuint32_t IREE14:1;                /* Enables rising-edge events to set DISR0[EIF14]. */
    vuint32_t IREE15:1;                /* Enables rising-edge events to set DISR0[EIF15]. */
    vuint32_t IREE16:1;                /* Enables rising-edge events to set DISR0[EIF16]. */
    vuint32_t IREE17:1;                /* Enables rising-edge events to set DISR0[EIF17]. */
    vuint32_t IREE18:1;                /* Enables rising-edge events to set DISR0[EIF18]. */
    vuint32_t IREE19:1;                /* Enables rising-edge events to set DISR0[EIF19]. */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_5_IREER0_tag;

typedef union SIUL2_5_IFEER0_union_tag { /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IFEE0:1;                 /* Enables falling-edge events to set DISR0[EIF0]. */
    vuint32_t IFEE1:1;                 /* Enables falling-edge events to set DISR0[EIF1]. */
    vuint32_t IFEE2:1;                 /* Enables falling-edge events to set DISR0[EIF2]. */
    vuint32_t IFEE3:1;                 /* Enables falling-edge events to set DISR0[EIF3]. */
    vuint32_t IFEE4:1;                 /* Enables falling-edge events to set DISR0[EIF4]. */
    vuint32_t IFEE5:1;                 /* Enables falling-edge events to set DISR0[EIF5]. */
    vuint32_t IFEE6:1;                 /* Enables falling-edge events to set DISR0[EIF6]. */
    vuint32_t IFEE7:1;                 /* Enables falling-edge events to set DISR0[EIF7]. */
    vuint32_t IFEE8:1;                 /* Enables falling-edge events to set DISR0[EIF8]. */
    vuint32_t IFEE9:1;                 /* Enables falling-edge events to set DISR0[EIF9]. */
    vuint32_t IFEE10:1;                /* Enables falling-edge events to set DISR0[EIF10]. */
    vuint32_t IFEE11:1;                /* Enables falling-edge events to set DISR0[EIF11]. */
    vuint32_t IFEE12:1;                /* Enables falling-edge events to set DISR0[EIF12]. */
    vuint32_t IFEE13:1;                /* Enables falling-edge events to set DISR0[EIF13]. */
    vuint32_t IFEE14:1;                /* Enables falling-edge events to set DISR0[EIF14]. */
    vuint32_t IFEE15:1;                /* Enables falling-edge events to set DISR0[EIF15]. */
    vuint32_t IFEE16:1;                /* Enables falling-edge events to set DISR0[EIF16]. */
    vuint32_t IFEE17:1;                /* Enables falling-edge events to set DISR0[EIF17]. */
    vuint32_t IFEE18:1;                /* Enables falling-edge events to set DISR0[EIF18]. */
    vuint32_t IFEE19:1;                /* Enables falling-edge events to set DISR0[EIF19]. */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_5_IFEER0_tag;

typedef union SIUL2_5_IFER0_union_tag { /* SIUL2 Interrupt Filter Enable 0 */
  vuint32_t R;
  struct {
    vuint32_t IFE0:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE1:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE2:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE3:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE4:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE5:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE6:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE7:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE8:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE9:1;                  /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE10:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE11:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE12:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE13:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE14:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE15:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE16:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE17:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE18:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t IFE19:1;                 /* Enables digital glitch filter on the interrupt pad input. */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t _unused_24:1;            /* Reserved */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SIUL2_5_IFER0_tag;

typedef union SIUL2_5_IFMCR_union_tag { /* SIUL2 Interrupt Filter Maximum Counter */
  vuint32_t R;
  struct {
    vuint32_t MAXCNT:4;                /* Maximum Interrupt Filter Counter setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_5_IFMCR_tag;

typedef union SIUL2_5_IFCPR_union_tag { /* SIUL2 Interrupt Filter Clock Prescaler */
  vuint32_t R;
  struct {
    vuint32_t IFCP:4;                  /* Interrupt Filter Clock Prescaler setting */
    vuint32_t _unused_4:28;
  } B;
} SIUL2_5_IFCPR_tag;

typedef union SIUL2_5_MSCR_union_tag { /* SIUL2 Multiplexed Signal Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:1;
    vuint32_t SMC:4;                   /* Safe Mode Control */
    vuint32_t TRC:1;                   /* Termination Resistor Control */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t RCVR:1;                  /* Receiver Select */
    vuint32_t CREF:1;                  /* Current Reference Control */
    vuint32_t PUS:1;                   /* Pull Select */
    vuint32_t PUE:1;                   /* Pull Enable */
    vuint32_t SRE:3;                   /* Slew Rate Control */
    vuint32_t RXCB:1;                  /* RX LVDS Current Boost */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t IBE:1;                   /* Input Buffer Enable */
    vuint32_t ODE:1;                   /* Open Drain Enable */
    vuint32_t OBE:1;                   /* GPIO Output Buffer Enable */
    vuint32_t _unused_22:10;           /* Reserved */
  } B;
} SIUL2_5_MSCR_tag;

typedef union SIUL2_5_IMCR_union_tag { /* SIUL2 Input Multiplexed Signal Configuration */
  vuint32_t R;
  struct {
    vuint32_t SSS:3;                   /* Source Signal Select */
    vuint32_t _unused_3:29;            /* Reserved */
  } B;
} SIUL2_5_IMCR_tag;

typedef union SIUL2_5_GPDO147_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO147_tag;

typedef union SIUL2_5_GPDO146_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO146_tag;

typedef union SIUL2_5_GPDO151_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO151_tag;

typedef union SIUL2_5_GPDO150_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO150_tag;

typedef union SIUL2_5_GPDO149_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO149_tag;

typedef union SIUL2_5_GPDO148_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO148_tag;

typedef union SIUL2_5_GPDO155_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO155_tag;

typedef union SIUL2_5_GPDO154_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO154_tag;

typedef union SIUL2_5_GPDO153_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO153_tag;

typedef union SIUL2_5_GPDO152_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO152_tag;

typedef union SIUL2_5_GPDO159_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO159_tag;

typedef union SIUL2_5_GPDO158_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO158_tag;

typedef union SIUL2_5_GPDO157_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO157_tag;

typedef union SIUL2_5_GPDO156_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO156_tag;

typedef union SIUL2_5_GPDO163_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO163_tag;

typedef union SIUL2_5_GPDO162_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO162_tag;

typedef union SIUL2_5_GPDO161_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO161_tag;

typedef union SIUL2_5_GPDO160_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO160_tag;

typedef union SIUL2_5_GPDO167_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO167_tag;

typedef union SIUL2_5_GPDO166_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO166_tag;

typedef union SIUL2_5_GPDO165_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO165_tag;

typedef union SIUL2_5_GPDO164_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO164_tag;

typedef union SIUL2_5_GPDO169_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO169_tag;

typedef union SIUL2_5_GPDO168_union_tag { /* SIUL2 GPIO Pad Data Output */
  vuint8_t R;
  struct {
    vuint8_t PDO_N:1;                  /* Pad Data Out */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDO168_tag;

typedef union SIUL2_5_GPDI147_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI147_tag;

typedef union SIUL2_5_GPDI146_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI146_tag;

typedef union SIUL2_5_GPDI151_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI151_tag;

typedef union SIUL2_5_GPDI150_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI150_tag;

typedef union SIUL2_5_GPDI149_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI149_tag;

typedef union SIUL2_5_GPDI148_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI148_tag;

typedef union SIUL2_5_GPDI155_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI155_tag;

typedef union SIUL2_5_GPDI154_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI154_tag;

typedef union SIUL2_5_GPDI153_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI153_tag;

typedef union SIUL2_5_GPDI152_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI152_tag;

typedef union SIUL2_5_GPDI159_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI159_tag;

typedef union SIUL2_5_GPDI158_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI158_tag;

typedef union SIUL2_5_GPDI157_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI157_tag;

typedef union SIUL2_5_GPDI156_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI156_tag;

typedef union SIUL2_5_GPDI163_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI163_tag;

typedef union SIUL2_5_GPDI162_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI162_tag;

typedef union SIUL2_5_GPDI161_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI161_tag;

typedef union SIUL2_5_GPDI160_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI160_tag;

typedef union SIUL2_5_GPDI167_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI167_tag;

typedef union SIUL2_5_GPDI166_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI166_tag;

typedef union SIUL2_5_GPDI165_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI165_tag;

typedef union SIUL2_5_GPDI164_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI164_tag;

typedef union SIUL2_5_GPDI169_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI169_tag;

typedef union SIUL2_5_GPDI168_union_tag { /* SIUL2 GPIO Pad Data Input */
  vuint8_t R;
  struct {
    vuint8_t PDI_N:1;                  /* Pad Data In */
    vuint8_t _unused_1:7;              /* Reserved */
  } B;
} SIUL2_5_GPDI168_tag;

typedef union SIUL2_5_PGPDO9_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t PPDO0:1;                 /* Parallel Pad Data Out 0 */
    vuint16_t PPDO1:1;                 /* Parallel Pad Data Out 1 */
    vuint16_t PPDO2:1;                 /* Parallel Pad Data Out 2 */
    vuint16_t PPDO3:1;                 /* Parallel Pad Data Out 3 */
    vuint16_t PPDO4:1;                 /* Parallel Pad Data Out 4 */
    vuint16_t PPDO5:1;                 /* Parallel Pad Data Out 5 */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} SIUL2_5_PGPDO9_tag;

typedef union SIUL2_5_PGPDO10_union_tag { /* SIUL2 Parallel GPIO Pad Data Out */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t _unused_2:1;             /* Reserved */
    vuint16_t _unused_3:1;             /* Reserved */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t _unused_5:1;             /* Reserved */
    vuint16_t PPDO6:1;                 /* Parallel Pad Data Out 6 */
    vuint16_t PPDO7:1;                 /* Parallel Pad Data Out 7 */
    vuint16_t PPDO8:1;                 /* Parallel Pad Data Out 8 */
    vuint16_t PPDO9:1;                 /* Parallel Pad Data Out 9 */
    vuint16_t PPDO10:1;                /* Parallel Pad Data Out 10 */
    vuint16_t PPDO11:1;                /* Parallel Pad Data Out 11 */
    vuint16_t PPDO12:1;                /* Parallel Pad Data Out 12 */
    vuint16_t PPDO13:1;                /* Parallel Pad Data Out 13 */
    vuint16_t PPDO14:1;                /* Parallel Pad Data Out 14 */
    vuint16_t PPDO15:1;                /* Parallel Pad Data Out 15 */
  } B;
} SIUL2_5_PGPDO10_tag;

typedef union SIUL2_5_PGPDI9_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t PPDI0:1;                 /* Parallel Pad Data In 0 */
    vuint16_t PPDI1:1;                 /* Parallel Pad Data In 1 */
    vuint16_t PPDI2:1;                 /* Parallel Pad Data In 2 */
    vuint16_t PPDI3:1;                 /* Parallel Pad Data In 3 */
    vuint16_t PPDI4:1;                 /* Parallel Pad Data In 4 */
    vuint16_t PPDI5:1;                 /* Parallel Pad Data In 5 */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t _unused_14:1;            /* Reserved */
    vuint16_t _unused_15:1;            /* Reserved */
  } B;
} SIUL2_5_PGPDI9_tag;

typedef union SIUL2_5_PGPDI10_union_tag { /* SIUL2 Parallel GPIO Pad Data In */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:1;             /* Reserved */
    vuint16_t _unused_1:1;             /* Reserved */
    vuint16_t _unused_2:1;             /* Reserved */
    vuint16_t _unused_3:1;             /* Reserved */
    vuint16_t _unused_4:1;             /* Reserved */
    vuint16_t _unused_5:1;             /* Reserved */
    vuint16_t PPDI6:1;                 /* Parallel Pad Data In 6 */
    vuint16_t PPDI7:1;                 /* Parallel Pad Data In 7 */
    vuint16_t PPDI8:1;                 /* Parallel Pad Data In 8 */
    vuint16_t PPDI9:1;                 /* Parallel Pad Data In 9 */
    vuint16_t PPDI10:1;                /* Parallel Pad Data In 10 */
    vuint16_t PPDI11:1;                /* Parallel Pad Data In 11 */
    vuint16_t PPDI12:1;                /* Parallel Pad Data In 12 */
    vuint16_t PPDI13:1;                /* Parallel Pad Data In 13 */
    vuint16_t PPDI14:1;                /* Parallel Pad Data In 14 */
    vuint16_t PPDI15:1;                /* Parallel Pad Data In 15 */
  } B;
} SIUL2_5_PGPDI10_tag;

typedef union SIUL2_5_MPGPDO_union_tag { /* SIUL2 Masked Parallel GPIO Pad Data Out */
  vuint32_t R;
  struct {
    vuint32_t MPPDO0:1;                /* Masked Parallel Pad Data Out 0 */
    vuint32_t MPPDO1:1;                /* Masked Parallel Pad Data Out 1 */
    vuint32_t MPPDO2:1;                /* Masked Parallel Pad Data Out 2 */
    vuint32_t MPPDO3:1;                /* Masked Parallel Pad Data Out 3 */
    vuint32_t MPPDO4:1;                /* Masked Parallel Pad Data Out 4 */
    vuint32_t MPPDO5:1;                /* Masked Parallel Pad Data Out 5 */
    vuint32_t MPPDO6:1;                /* Masked Parallel Pad Data Out 6 */
    vuint32_t MPPDO7:1;                /* Masked Parallel Pad Data Out 7 */
    vuint32_t MPPDO8:1;                /* Masked Parallel Pad Data Out 8 */
    vuint32_t MPPDO9:1;                /* Masked Parallel Pad Data Out 9 */
    vuint32_t MPPDO10:1;               /* Masked Parallel Pad Data Out 10 */
    vuint32_t MPPDO11:1;               /* Masked Parallel Pad Data Out 11 */
    vuint32_t MPPDO12:1;               /* Masked Parallel Pad Data Out 12 */
    vuint32_t MPPDO13:1;               /* Masked Parallel Pad Data Out 13 */
    vuint32_t MPPDO14:1;               /* Masked Parallel Pad Data Out 14 */
    vuint32_t MPPDO15:1;               /* Masked Parallel Pad Data Out 15 */
    vuint32_t MASK0:1;                 /* Mask Field 0 */
    vuint32_t MASK1:1;                 /* Mask Field 1 */
    vuint32_t MASK2:1;                 /* Mask Field 2 */
    vuint32_t MASK3:1;                 /* Mask Field 3 */
    vuint32_t MASK4:1;                 /* Mask Field 4 */
    vuint32_t MASK5:1;                 /* Mask Field 5 */
    vuint32_t MASK6:1;                 /* Mask Field 6 */
    vuint32_t MASK7:1;                 /* Mask Field 7 */
    vuint32_t MASK8:1;                 /* Mask Field 8 */
    vuint32_t MASK9:1;                 /* Mask Field 9 */
    vuint32_t MASK10:1;                /* Mask Field 10 */
    vuint32_t MASK11:1;                /* Mask Field 11 */
    vuint32_t MASK12:1;                /* Mask Field 12 */
    vuint32_t MASK13:1;                /* Mask Field 13 */
    vuint32_t MASK14:1;                /* Mask Field 14 */
    vuint32_t MASK15:1;                /* Mask Field 15 */
  } B;
} SIUL2_5_MPGPDO_tag;

struct SIUL2_5_tag {
  uint8_t SIUL2_5_reserved0[16];
  SIUL2_5_DISR0_tag DISR0;             /* SIUL2 DMA/Interrupt Status Flag 0 */
  uint8_t SIUL2_5_reserved1[4];
  SIUL2_5_DIRER0_tag DIRER0;           /* SIUL2 DMA/Interrupt Request Enable 0 */
  uint8_t SIUL2_5_reserved2[4];
  SIUL2_5_DIRSR0_tag DIRSR0;           /* SIUL2 DMA/Interrupt Request Select 0 */
  uint8_t SIUL2_5_reserved3[4];
  SIUL2_5_IREER0_tag IREER0;           /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
  uint8_t SIUL2_5_reserved4[4];
  SIUL2_5_IFEER0_tag IFEER0;           /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
  uint8_t SIUL2_5_reserved5[4];
  SIUL2_5_IFER0_tag IFER0;             /* SIUL2 Interrupt Filter Enable 0 */
  uint8_t SIUL2_5_reserved6[4];
  SIUL2_5_IFMCR_tag IFMCR[20];         /* SIUL2 Interrupt Filter Maximum Counter */
  uint8_t SIUL2_5_reserved7[48];
  SIUL2_5_IFCPR_tag IFCPR;             /* SIUL2 Interrupt Filter Clock Prescaler */
  uint8_t SIUL2_5_reserved8[380];
  SIUL2_5_MSCR_tag MSCR[170];          /* SIUL2 Multiplexed Signal Configuration Register; Valid array indices: 146-169 */
  uint8_t SIUL2_5_reserved9[1368];
  SIUL2_5_IMCR_tag IMCR[496];          /* SIUL2 Input Multiplexed Signal Configuration; Valid array indices: 449-454, 462-474, 488-495 */
  uint8_t SIUL2_5_reserved10[400];
  SIUL2_5_GPDO147_tag GPDO147;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO146_tag GPDO146;         /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_5_reserved11[2];
  SIUL2_5_GPDO151_tag GPDO151;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO150_tag GPDO150;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO149_tag GPDO149;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO148_tag GPDO148;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO155_tag GPDO155;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO154_tag GPDO154;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO153_tag GPDO153;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO152_tag GPDO152;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO159_tag GPDO159;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO158_tag GPDO158;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO157_tag GPDO157;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO156_tag GPDO156;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO163_tag GPDO163;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO162_tag GPDO162;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO161_tag GPDO161;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO160_tag GPDO160;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO167_tag GPDO167;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO166_tag GPDO166;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO165_tag GPDO165;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO164_tag GPDO164;         /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_5_reserved12[2];
  SIUL2_5_GPDO169_tag GPDO169;         /* SIUL2 GPIO Pad Data Output */
  SIUL2_5_GPDO168_tag GPDO168;         /* SIUL2 GPIO Pad Data Output */
  uint8_t SIUL2_5_reserved13[484];
  SIUL2_5_GPDI147_tag GPDI147;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI146_tag GPDI146;         /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_5_reserved14[2];
  SIUL2_5_GPDI151_tag GPDI151;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI150_tag GPDI150;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI149_tag GPDI149;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI148_tag GPDI148;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI155_tag GPDI155;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI154_tag GPDI154;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI153_tag GPDI153;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI152_tag GPDI152;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI159_tag GPDI159;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI158_tag GPDI158;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI157_tag GPDI157;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI156_tag GPDI156;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI163_tag GPDI163;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI162_tag GPDI162;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI161_tag GPDI161;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI160_tag GPDI160;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI167_tag GPDI167;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI166_tag GPDI166;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI165_tag GPDI165;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI164_tag GPDI164;         /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_5_reserved15[2];
  SIUL2_5_GPDI169_tag GPDI169;         /* SIUL2 GPIO Pad Data Input */
  SIUL2_5_GPDI168_tag GPDI168;         /* SIUL2 GPIO Pad Data Input */
  uint8_t SIUL2_5_reserved16[356];
  SIUL2_5_PGPDO9_tag PGPDO9;           /* SIUL2 Parallel GPIO Pad Data Out */
  uint8_t SIUL2_5_reserved17[4];
  SIUL2_5_PGPDO10_tag PGPDO10;         /* SIUL2 Parallel GPIO Pad Data Out */
  uint8_t SIUL2_5_reserved18[56];
  SIUL2_5_PGPDI9_tag PGPDI9;           /* SIUL2 Parallel GPIO Pad Data In */
  uint8_t SIUL2_5_reserved19[4];
  SIUL2_5_PGPDI10_tag PGPDI10;         /* SIUL2 Parallel GPIO Pad Data In */
  uint8_t SIUL2_5_reserved20[40];
  SIUL2_5_MPGPDO_tag MPGPDO[11];       /* SIUL2 Masked Parallel GPIO Pad Data Out; Valid array indices: 9-10 */
};


/* ============================================================================
   =============================== Module: SMU_L_VFCCU ========================
   ============================================================================ */

typedef union SMU_L_VFCCU_GDFHID_C0_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID0:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID1:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID2:3;                /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID3:3;                /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID4:3;                /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID5:3;                /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID6:3;                /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID7:3;                /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} SMU_L_VFCCU_GDFHID_C0_tag;

typedef union SMU_L_VFCCU_GDFHID_C1_union_tag { /* Global DID-FHID Map */
  vuint32_t R;
  struct {
    vuint32_t FHDID8:3;                /* DID to FHID */
    vuint32_t _unused_3:1;
    vuint32_t FHDID9:3;                /* DID to FHID */
    vuint32_t _unused_7:1;
    vuint32_t FHDID10:3;               /* DID to FHID */
    vuint32_t _unused_11:1;
    vuint32_t FHDID11:3;               /* DID to FHID */
    vuint32_t _unused_15:1;
    vuint32_t FHDID12:3;               /* DID to FHID */
    vuint32_t _unused_19:1;
    vuint32_t FHDID13:3;               /* DID to FHID */
    vuint32_t _unused_23:1;
    vuint32_t FHDID14:3;               /* DID to FHID */
    vuint32_t _unused_27:1;
    vuint32_t FHDID15:3;               /* DID to FHID */
    vuint32_t _unused_31:1;
  } B;
} SMU_L_VFCCU_GDFHID_C1_tag;

typedef union SMU_L_VFCCU_GFLTPO_C0_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS0:1;                   /* Polarity Selection */
    vuint32_t PS1:1;                   /* Polarity Selection */
    vuint32_t PS2:1;                   /* Polarity Selection */
    vuint32_t PS3:1;                   /* Polarity Selection */
    vuint32_t PS4:1;                   /* Polarity Selection */
    vuint32_t PS5:1;                   /* Polarity Selection */
    vuint32_t PS6:1;                   /* Polarity Selection */
    vuint32_t PS7:1;                   /* Polarity Selection */
    vuint32_t PS8:1;                   /* Polarity Selection */
    vuint32_t PS9:1;                   /* Polarity Selection */
    vuint32_t PS10:1;                  /* Polarity Selection */
    vuint32_t PS11:1;                  /* Polarity Selection */
    vuint32_t PS12:1;                  /* Polarity Selection */
    vuint32_t PS13:1;                  /* Polarity Selection */
    vuint32_t PS14:1;                  /* Polarity Selection */
    vuint32_t PS15:1;                  /* Polarity Selection */
    vuint32_t PS16:1;                  /* Polarity Selection */
    vuint32_t PS17:1;                  /* Polarity Selection */
    vuint32_t PS18:1;                  /* Polarity Selection */
    vuint32_t PS19:1;                  /* Polarity Selection */
    vuint32_t PS20:1;                  /* Polarity Selection */
    vuint32_t PS21:1;                  /* Polarity Selection */
    vuint32_t PS22:1;                  /* Polarity Selection */
    vuint32_t PS23:1;                  /* Polarity Selection */
    vuint32_t PS24:1;                  /* Polarity Selection */
    vuint32_t PS25:1;                  /* Polarity Selection */
    vuint32_t PS26:1;                  /* Polarity Selection */
    vuint32_t PS27:1;                  /* Polarity Selection */
    vuint32_t PS28:1;                  /* Polarity Selection */
    vuint32_t PS29:1;                  /* Polarity Selection */
    vuint32_t PS30:1;                  /* Polarity Selection */
    vuint32_t PS31:1;                  /* Polarity Selection */
  } B;
} SMU_L_VFCCU_GFLTPO_C0_tag;

typedef union SMU_L_VFCCU_GFLTPO_C1_union_tag { /* Global Fault Polarity */
  vuint32_t R;
  struct {
    vuint32_t PS32:1;                  /* Polarity Selection */
    vuint32_t PS33:1;                  /* Polarity Selection */
    vuint32_t PS34:1;                  /* Polarity Selection */
    vuint32_t PS35:1;                  /* Polarity Selection */
    vuint32_t PS36:1;                  /* Polarity Selection */
    vuint32_t PS37:1;                  /* Polarity Selection */
    vuint32_t PS38:1;                  /* Polarity Selection */
    vuint32_t PS39:1;                  /* Polarity Selection */
    vuint32_t PS40:1;                  /* Polarity Selection */
    vuint32_t PS41:1;                  /* Polarity Selection */
    vuint32_t PS42:1;                  /* Polarity Selection */
    vuint32_t PS43:1;                  /* Polarity Selection */
    vuint32_t PS44:1;                  /* Polarity Selection */
    vuint32_t PS45:1;                  /* Polarity Selection */
    vuint32_t PS46:1;                  /* Polarity Selection */
    vuint32_t PS47:1;                  /* Polarity Selection */
    vuint32_t PS48:1;                  /* Polarity Selection */
    vuint32_t PS49:1;                  /* Polarity Selection */
    vuint32_t PS50:1;                  /* Polarity Selection */
    vuint32_t PS51:1;                  /* Polarity Selection */
    vuint32_t PS52:1;                  /* Polarity Selection */
    vuint32_t _unused_21:11;
  } B;
} SMU_L_VFCCU_GFLTPO_C1_tag;

typedef union SMU_L_VFCCU_GFLTRC_C0_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW0:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW1:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW2:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW3:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW4:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW5:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW6:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW7:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW8:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW9:1;                /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW10:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW11:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW12:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW13:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW14:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW15:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW16:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW17:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW18:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW19:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW20:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW21:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW22:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW23:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW24:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW25:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW26:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW27:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW28:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW29:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW30:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW31:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
  } B;
} SMU_L_VFCCU_GFLTRC_C0_tag;

typedef union SMU_L_VFCCU_GFLTRC_C1_union_tag { /* Global Fault Recovery */
  vuint32_t R;
  struct {
    vuint32_t RHWSW32:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW33:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW34:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW35:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW36:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW37:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW38:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW39:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW40:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW41:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW42:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW43:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW44:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW45:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW46:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW47:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW48:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW49:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW50:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW51:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t RHWSW52:1;               /* Recovery/Clearing Mechanism Hardware Or Software */
    vuint32_t _unused_21:11;
  } B;
} SMU_L_VFCCU_GFLTRC_C1_tag;

typedef union SMU_L_VFCCU_GFLTOVDC0_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS0:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS1:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS2:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS3:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS4:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS5:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS6:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS7:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS8:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS9:1;              /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS10:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS11:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS12:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS13:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS14:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS15:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS16:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS17:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS18:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS19:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS20:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS21:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS22:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS23:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS24:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS25:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS26:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS27:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS28:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS29:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS30:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS31:1;             /* Fault Overflow Detection Disable */
  } B;
} SMU_L_VFCCU_GFLTOVDC0_tag;

typedef union SMU_L_VFCCU_GFLTOVDC1_union_tag { /* Global Fault Overflow Detection */
  vuint32_t R;
  struct {
    vuint32_t OVF_DIS32:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS33:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS34:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS35:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS36:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS37:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS38:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS39:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS40:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS41:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS42:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS43:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS44:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS45:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS46:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS47:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS48:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS49:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS50:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS51:1;             /* Fault Overflow Detection Disable */
    vuint32_t OVF_DIS52:1;             /* Fault Overflow Detection Disable */
    vuint32_t _unused_21:11;
  } B;
} SMU_L_VFCCU_GFLTOVDC1_tag;

typedef union SMU_L_VFCCU_GSWFLODC_union_tag { /* Global Software Fault Overflow Detection Disable */
  vuint32_t R;
  struct {
    vuint32_t SWOVF_DIS0:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS1:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS2:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS3:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS4:1;            /* Software Overflow Disable Detection */
    vuint32_t SWOVF_DIS5:1;            /* Software Overflow Disable Detection */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_GSWFLODC_tag;

typedef union SMU_L_VFCCU_GCTRL_union_tag { /* Global Space Control */
  vuint32_t R;
  struct {
    vuint32_t OVF_EN:1;                /* Overflow Enable */
    vuint32_t _unused_1:31;
  } B;
} SMU_L_VFCCU_GCTRL_tag;

typedef union SMU_L_VFCCU_GINTOVFS_union_tag { /* Global DID FSM Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t FLTSERV:1;               /* Fault Serve */
    vuint32_t OVF_DET:1;               /* Overflow Detect */
    vuint32_t _unused_9:7;
    vuint32_t SERV_DID:4;              /* Domain Being Serviced */
    vuint32_t _unused_20:4;
    vuint32_t OVF_DID:4;               /* Overflow DID */
    vuint32_t _unused_28:4;
  } B;
} SMU_L_VFCCU_GINTOVFS_tag;

typedef union SMU_L_VFCCU_GDBGCFG_union_tag { /* Global Debug */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t FRZ:1;                   /* Freeze On First Fault */
    vuint32_t _unused_17:15;
  } B;
} SMU_L_VFCCU_GDBGCFG_tag;

typedef union SMU_L_VFCCU_GDBGSTAT_union_tag { /* Global Debug Status */
  vuint32_t R;
  struct {
    vuint32_t FLTIND:8;                /* Fault Index */
    vuint32_t _unused_8:24;
  } B;
} SMU_L_VFCCU_GDBGSTAT_tag;

typedef union SMU_L_VFCCU_SW_RKN_SWRPTDID_union_tag { /* Software Fault Reported DID */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Software Fault DID */
    vuint32_t _unused_4:28;
  } B;
} SMU_L_VFCCU_SW_RKN_SWRPTDID_tag;

typedef union SMU_L_VFCCU_SW_RKN_SWRKSET_0_union_tag { /* Software Reaction Set */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_SW_RKN_SWRKSET_0_tag;

typedef union SMU_L_VFCCU_SW_RKN_SWRKCLR_0_union_tag { /* Software Reaction Clear */
  vuint32_t R;
  struct {
    vuint32_t RKNCLR0:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR1:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR2:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR3:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR4:1;               /* Reaction Line Clear */
    vuint32_t RKNCLR5:1;               /* Reaction Line Clear */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_SW_RKN_SWRKCLR_0_tag;

typedef struct SMU_L_VFCCU_SW_RKN_struct_tag {
  SMU_L_VFCCU_SW_RKN_SWRPTDID_tag SWRPTDID; /* Software Fault Reported DID */
  SMU_L_VFCCU_SW_RKN_SWRKSET_0_tag SWRKSET_0; /* Software Reaction Set */
  uint8_t SW_RKN_reserved0[4];
  SMU_L_VFCCU_SW_RKN_SWRKCLR_0_tag SWRKCLR_0; /* Software Reaction Clear */
  uint8_t SW_RKN_reserved1[16];
} SMU_L_VFCCU_SW_RKN_tag;

typedef union SMU_L_VFCCU_FHCFG0_union_tag { /* Fault Handler */
  vuint32_t R;
  struct {
    vuint32_t FHIDEN:1;                /* Fault Handler Enable */
    vuint32_t _unused_1:31;
  } B;
} SMU_L_VFCCU_FHCFG0_tag;

typedef union SMU_L_VFCCU_FHSRVDS0_union_tag { /* Fault Handler Status */
  vuint32_t R;
  struct {
    vuint32_t SERV_DID:4;              /* DID Being Serviced */
    vuint32_t AGGFLTS:1;               /* Aggregated Fault Status */
    vuint32_t _unused_5:27;
  } B;
} SMU_L_VFCCU_FHSRVDS0_tag;

typedef union SMU_L_VFCCU_FHFLTENC00_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN0:1;                   /* Fault Enable */
    vuint32_t EN1:1;                   /* Fault Enable */
    vuint32_t EN2:1;                   /* Fault Enable */
    vuint32_t EN3:1;                   /* Fault Enable */
    vuint32_t EN4:1;                   /* Fault Enable */
    vuint32_t EN5:1;                   /* Fault Enable */
    vuint32_t EN6:1;                   /* Fault Enable */
    vuint32_t EN7:1;                   /* Fault Enable */
    vuint32_t EN8:1;                   /* Fault Enable */
    vuint32_t EN9:1;                   /* Fault Enable */
    vuint32_t EN10:1;                  /* Fault Enable */
    vuint32_t EN11:1;                  /* Fault Enable */
    vuint32_t EN12:1;                  /* Fault Enable */
    vuint32_t EN13:1;                  /* Fault Enable */
    vuint32_t EN14:1;                  /* Fault Enable */
    vuint32_t EN15:1;                  /* Fault Enable */
    vuint32_t EN16:1;                  /* Fault Enable */
    vuint32_t EN17:1;                  /* Fault Enable */
    vuint32_t EN18:1;                  /* Fault Enable */
    vuint32_t EN19:1;                  /* Fault Enable */
    vuint32_t EN20:1;                  /* Fault Enable */
    vuint32_t EN21:1;                  /* Fault Enable */
    vuint32_t EN22:1;                  /* Fault Enable */
    vuint32_t EN23:1;                  /* Fault Enable */
    vuint32_t EN24:1;                  /* Fault Enable */
    vuint32_t EN25:1;                  /* Fault Enable */
    vuint32_t EN26:1;                  /* Fault Enable */
    vuint32_t EN27:1;                  /* Fault Enable */
    vuint32_t EN28:1;                  /* Fault Enable */
    vuint32_t EN29:1;                  /* Fault Enable */
    vuint32_t EN30:1;                  /* Fault Enable */
    vuint32_t EN31:1;                  /* Fault Enable */
  } B;
} SMU_L_VFCCU_FHFLTENC00_tag;

typedef union SMU_L_VFCCU_FHFLTENC01_union_tag { /* Fault Enable */
  vuint32_t R;
  struct {
    vuint32_t EN32:1;                  /* Fault Enable */
    vuint32_t EN33:1;                  /* Fault Enable */
    vuint32_t EN34:1;                  /* Fault Enable */
    vuint32_t EN35:1;                  /* Fault Enable */
    vuint32_t EN36:1;                  /* Fault Enable */
    vuint32_t EN37:1;                  /* Fault Enable */
    vuint32_t EN38:1;                  /* Fault Enable */
    vuint32_t EN39:1;                  /* Fault Enable */
    vuint32_t EN40:1;                  /* Fault Enable */
    vuint32_t EN41:1;                  /* Fault Enable */
    vuint32_t EN42:1;                  /* Fault Enable */
    vuint32_t EN43:1;                  /* Fault Enable */
    vuint32_t EN44:1;                  /* Fault Enable */
    vuint32_t EN45:1;                  /* Fault Enable */
    vuint32_t EN46:1;                  /* Fault Enable */
    vuint32_t EN47:1;                  /* Fault Enable */
    vuint32_t EN48:1;                  /* Fault Enable */
    vuint32_t EN49:1;                  /* Fault Enable */
    vuint32_t EN50:1;                  /* Fault Enable */
    vuint32_t EN51:1;                  /* Fault Enable */
    vuint32_t EN52:1;                  /* Fault Enable */
    vuint32_t _unused_21:11;
  } B;
} SMU_L_VFCCU_FHFLTENC01_tag;

typedef union SMU_L_VFCCU_FHFLTS00_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT0:1;                 /* Fault Status */
    vuint32_t STAT1:1;                 /* Fault Status */
    vuint32_t STAT2:1;                 /* Fault Status */
    vuint32_t STAT3:1;                 /* Fault Status */
    vuint32_t STAT4:1;                 /* Fault Status */
    vuint32_t STAT5:1;                 /* Fault Status */
    vuint32_t STAT6:1;                 /* Fault Status */
    vuint32_t STAT7:1;                 /* Fault Status */
    vuint32_t STAT8:1;                 /* Fault Status */
    vuint32_t STAT9:1;                 /* Fault Status */
    vuint32_t STAT10:1;                /* Fault Status */
    vuint32_t STAT11:1;                /* Fault Status */
    vuint32_t STAT12:1;                /* Fault Status */
    vuint32_t STAT13:1;                /* Fault Status */
    vuint32_t STAT14:1;                /* Fault Status */
    vuint32_t STAT15:1;                /* Fault Status */
    vuint32_t STAT16:1;                /* Fault Status */
    vuint32_t STAT17:1;                /* Fault Status */
    vuint32_t STAT18:1;                /* Fault Status */
    vuint32_t STAT19:1;                /* Fault Status */
    vuint32_t STAT20:1;                /* Fault Status */
    vuint32_t STAT21:1;                /* Fault Status */
    vuint32_t STAT22:1;                /* Fault Status */
    vuint32_t STAT23:1;                /* Fault Status */
    vuint32_t STAT24:1;                /* Fault Status */
    vuint32_t STAT25:1;                /* Fault Status */
    vuint32_t STAT26:1;                /* Fault Status */
    vuint32_t STAT27:1;                /* Fault Status */
    vuint32_t STAT28:1;                /* Fault Status */
    vuint32_t STAT29:1;                /* Fault Status */
    vuint32_t STAT30:1;                /* Fault Status */
    vuint32_t STAT31:1;                /* Fault Status */
  } B;
} SMU_L_VFCCU_FHFLTS00_tag;

typedef union SMU_L_VFCCU_FHFLTS01_union_tag { /* Fault Status */
  vuint32_t R;
  struct {
    vuint32_t STAT32:1;                /* Fault Status */
    vuint32_t STAT33:1;                /* Fault Status */
    vuint32_t STAT34:1;                /* Fault Status */
    vuint32_t STAT35:1;                /* Fault Status */
    vuint32_t STAT36:1;                /* Fault Status */
    vuint32_t STAT37:1;                /* Fault Status */
    vuint32_t STAT38:1;                /* Fault Status */
    vuint32_t STAT39:1;                /* Fault Status */
    vuint32_t STAT40:1;                /* Fault Status */
    vuint32_t STAT41:1;                /* Fault Status */
    vuint32_t STAT42:1;                /* Fault Status */
    vuint32_t STAT43:1;                /* Fault Status */
    vuint32_t STAT44:1;                /* Fault Status */
    vuint32_t STAT45:1;                /* Fault Status */
    vuint32_t STAT46:1;                /* Fault Status */
    vuint32_t STAT47:1;                /* Fault Status */
    vuint32_t STAT48:1;                /* Fault Status */
    vuint32_t STAT49:1;                /* Fault Status */
    vuint32_t STAT50:1;                /* Fault Status */
    vuint32_t STAT51:1;                /* Fault Status */
    vuint32_t STAT52:1;                /* Fault Status */
    vuint32_t _unused_21:11;
  } B;
} SMU_L_VFCCU_FHFLTS01_tag;

typedef union SMU_L_VFCCU_FHFLTRKC00_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL0:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL1:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL2:3;               /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL3:3;               /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL4:3;               /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL5:3;               /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL6:3;               /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL7:3;               /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} SMU_L_VFCCU_FHFLTRKC00_tag;

typedef union SMU_L_VFCCU_FHFLTRKC01_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL8:3;               /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL9:3;               /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL10:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL11:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL12:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL13:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL14:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL15:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} SMU_L_VFCCU_FHFLTRKC01_tag;

typedef union SMU_L_VFCCU_FHFLTRKC02_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL16:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL17:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL18:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL19:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL20:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL21:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL22:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL23:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} SMU_L_VFCCU_FHFLTRKC02_tag;

typedef union SMU_L_VFCCU_FHFLTRKC03_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL24:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL25:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL26:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL27:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL28:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL29:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL30:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL31:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} SMU_L_VFCCU_FHFLTRKC03_tag;

typedef union SMU_L_VFCCU_FHFLTRKC04_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL32:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL33:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL34:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL35:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL36:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL37:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL38:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL39:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} SMU_L_VFCCU_FHFLTRKC04_tag;

typedef union SMU_L_VFCCU_FHFLTRKC05_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL40:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL41:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL42:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL43:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL44:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t RKNSEL45:3;              /* Reaction Selection */
    vuint32_t _unused_23:1;
    vuint32_t RKNSEL46:3;              /* Reaction Selection */
    vuint32_t _unused_27:1;
    vuint32_t RKNSEL47:3;              /* Reaction Selection */
    vuint32_t _unused_31:1;
  } B;
} SMU_L_VFCCU_FHFLTRKC05_tag;

typedef union SMU_L_VFCCU_FHFLTRKC06_union_tag { /* Fault Reaction Set Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNSEL48:3;              /* Reaction Selection */
    vuint32_t _unused_3:1;
    vuint32_t RKNSEL49:3;              /* Reaction Selection */
    vuint32_t _unused_7:1;
    vuint32_t RKNSEL50:3;              /* Reaction Selection */
    vuint32_t _unused_11:1;
    vuint32_t RKNSEL51:3;              /* Reaction Selection */
    vuint32_t _unused_15:1;
    vuint32_t RKNSEL52:3;              /* Reaction Selection */
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:12;
  } B;
} SMU_L_VFCCU_FHFLTRKC06_tag;

typedef union SMU_L_VFCCU_FHIMRKC0_00_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_FHIMRKC0_00_tag;

typedef union SMU_L_VFCCU_FHIMRKC0_10_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_FHIMRKC0_10_tag;

typedef union SMU_L_VFCCU_FHIMRKC0_20_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_FHIMRKC0_20_tag;

typedef union SMU_L_VFCCU_FHIMRKC0_30_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_FHIMRKC0_30_tag;

typedef union SMU_L_VFCCU_FHIMRKC0_40_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_FHIMRKC0_40_tag;

typedef union SMU_L_VFCCU_FHIMRKC0_50_union_tag { /* Immediate Reaction Configuration */
  vuint32_t R;
  struct {
    vuint32_t RKNEN0:1;                /* Reaction Line Enable */
    vuint32_t RKNEN1:1;                /* Reaction Line Enable */
    vuint32_t RKNEN2:1;                /* Reaction Line Enable */
    vuint32_t RKNEN3:1;                /* Reaction Line Enable */
    vuint32_t RKNEN4:1;                /* Reaction Line Enable */
    vuint32_t RKNEN5:1;                /* Reaction Line Enable */
    vuint32_t _unused_6:26;
  } B;
} SMU_L_VFCCU_FHIMRKC0_50_tag;

struct SMU_L_VFCCU_tag {
  SMU_L_VFCCU_GDFHID_C0_tag GDFHID_C0; /* Global DID-FHID Map */
  SMU_L_VFCCU_GDFHID_C1_tag GDFHID_C1; /* Global DID-FHID Map */
  uint8_t SMU_L_VFCCU_reserved0[24];
  SMU_L_VFCCU_GFLTPO_C0_tag GFLTPO_C0; /* Global Fault Polarity */
  SMU_L_VFCCU_GFLTPO_C1_tag GFLTPO_C1; /* Global Fault Polarity */
  uint8_t SMU_L_VFCCU_reserved1[56];
  SMU_L_VFCCU_GFLTRC_C0_tag GFLTRC_C0; /* Global Fault Recovery */
  SMU_L_VFCCU_GFLTRC_C1_tag GFLTRC_C1; /* Global Fault Recovery */
  uint8_t SMU_L_VFCCU_reserved2[56];
  SMU_L_VFCCU_GFLTOVDC0_tag GFLTOVDC0; /* Global Fault Overflow Detection */
  SMU_L_VFCCU_GFLTOVDC1_tag GFLTOVDC1; /* Global Fault Overflow Detection */
  uint8_t SMU_L_VFCCU_reserved3[104];
  SMU_L_VFCCU_GSWFLODC_tag GSWFLODC;   /* Global Software Fault Overflow Detection Disable */
  uint8_t SMU_L_VFCCU_reserved4[28];
  SMU_L_VFCCU_GCTRL_tag GCTRL;         /* Global Space Control */
  SMU_L_VFCCU_GINTOVFS_tag GINTOVFS;   /* Global DID FSM Status */
  uint8_t SMU_L_VFCCU_reserved5[428];
  SMU_L_VFCCU_GDBGCFG_tag GDBGCFG;     /* Global Debug */
  SMU_L_VFCCU_GDBGSTAT_tag GDBGSTAT;   /* Global Debug Status */
  uint8_t SMU_L_VFCCU_reserved6[15636];
  SMU_L_VFCCU_SW_RKN_tag SW_RKN[6];
  uint8_t SMU_L_VFCCU_reserved7[16192];
  SMU_L_VFCCU_FHCFG0_tag FHCFG0;       /* Fault Handler */
  SMU_L_VFCCU_FHSRVDS0_tag FHSRVDS0;   /* Fault Handler Status */
  uint8_t SMU_L_VFCCU_reserved8[8];
  SMU_L_VFCCU_FHFLTENC00_tag FHFLTENC00; /* Fault Enable */
  SMU_L_VFCCU_FHFLTENC01_tag FHFLTENC01; /* Fault Enable */
  uint8_t SMU_L_VFCCU_reserved9[56];
  SMU_L_VFCCU_FHFLTS00_tag FHFLTS00;   /* Fault Status */
  SMU_L_VFCCU_FHFLTS01_tag FHFLTS01;   /* Fault Status */
  uint8_t SMU_L_VFCCU_reserved10[56];
  SMU_L_VFCCU_FHFLTRKC00_tag FHFLTRKC00; /* Fault Reaction Set Configuration */
  SMU_L_VFCCU_FHFLTRKC01_tag FHFLTRKC01; /* Fault Reaction Set Configuration */
  SMU_L_VFCCU_FHFLTRKC02_tag FHFLTRKC02; /* Fault Reaction Set Configuration */
  SMU_L_VFCCU_FHFLTRKC03_tag FHFLTRKC03; /* Fault Reaction Set Configuration */
  SMU_L_VFCCU_FHFLTRKC04_tag FHFLTRKC04; /* Fault Reaction Set Configuration */
  SMU_L_VFCCU_FHFLTRKC05_tag FHFLTRKC05; /* Fault Reaction Set Configuration */
  SMU_L_VFCCU_FHFLTRKC06_tag FHFLTRKC06; /* Fault Reaction Set Configuration */
  uint8_t SMU_L_VFCCU_reserved11[228];
  SMU_L_VFCCU_FHIMRKC0_00_tag FHIMRKC0_00; /* Immediate Reaction Configuration */
  uint8_t SMU_L_VFCCU_reserved12[12];
  SMU_L_VFCCU_FHIMRKC0_10_tag FHIMRKC0_10; /* Immediate Reaction Configuration */
  uint8_t SMU_L_VFCCU_reserved13[12];
  SMU_L_VFCCU_FHIMRKC0_20_tag FHIMRKC0_20; /* Immediate Reaction Configuration */
  uint8_t SMU_L_VFCCU_reserved14[12];
  SMU_L_VFCCU_FHIMRKC0_30_tag FHIMRKC0_30; /* Immediate Reaction Configuration */
  uint8_t SMU_L_VFCCU_reserved15[12];
  SMU_L_VFCCU_FHIMRKC0_40_tag FHIMRKC0_40; /* Immediate Reaction Configuration */
  uint8_t SMU_L_VFCCU_reserved16[12];
  SMU_L_VFCCU_FHIMRKC0_50_tag FHIMRKC0_50; /* Immediate Reaction Configuration */
};


/* ============================================================================
   =============================== Module: SMU_MRU ============================
   ============================================================================ */

typedef union SMU_MRU_CHXCONFIG_CH_CFG0_union_tag { /* Channel (x) Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t CHE:1;                   /* Channel Enable */
    vuint32_t CHR:1;                   /* Channel Reset */
    vuint32_t IE:1;                    /* Interrupt Enable */
    vuint32_t _unused_3:13;
    vuint32_t MBE0:1;                  /* Mailbox Enable */
    vuint32_t MBE1:1;                  /* Mailbox Enable */
    vuint32_t MBE2:1;                  /* Mailbox Enable */
    vuint32_t MBE3:1;                  /* Mailbox Enable */
    vuint32_t _unused_20:12;
  } B;
} SMU_MRU_CHXCONFIG_CH_CFG0_tag;

typedef union SMU_MRU_CHXCONFIG_CH_CFG1_union_tag { /* Channel (x) Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MBIC0:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC1:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC2:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t MBIC3:1;                 /* Mailbox Interrupt Configuration */
    vuint32_t _unused_20:12;
  } B;
} SMU_MRU_CHXCONFIG_CH_CFG1_tag;

typedef union SMU_MRU_CHXCONFIG_CH_MBSTAT_union_tag { /* Channel (x) Mailbox Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;
    vuint32_t MBS0:1;                  /* Mailbox Status */
    vuint32_t MBS1:1;                  /* Mailbox Status */
    vuint32_t MBS2:1;                  /* Mailbox Status */
    vuint32_t MBS3:1;                  /* Mailbox Status */
    vuint32_t _unused_20:12;
  } B;
} SMU_MRU_CHXCONFIG_CH_MBSTAT_tag;

typedef struct SMU_MRU_CHXCONFIG_struct_tag {
  SMU_MRU_CHXCONFIG_CH_CFG0_tag CH_CFG0; /* Channel (x) Configuration 0 */
  SMU_MRU_CHXCONFIG_CH_CFG1_tag CH_CFG1; /* Channel (x) Configuration 1 */
  SMU_MRU_CHXCONFIG_CH_MBSTAT_tag CH_MBSTAT; /* Channel (x) Mailbox Status */
  uint8_t CHXCONFIG_reserved0[4];
} SMU_MRU_CHXCONFIG_tag;

typedef union SMU_MRU_NOTIFY0_union_tag { /* Notification 0 Status */
  vuint32_t R;
  struct {
    vuint32_t CH1_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH2_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH3_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH4_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH5_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH6_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH7_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH8_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH9_IS0:1;               /* Interrupt 0 Status */
    vuint32_t CH10_IS0:1;              /* Interrupt 0 Status */
    vuint32_t CH11_IS0:1;              /* Interrupt 0 Status */
    vuint32_t CH12_IS0:1;              /* Interrupt 0 Status */
    vuint32_t _unused_12:20;
  } B;
} SMU_MRU_NOTIFY0_tag;

typedef union SMU_MRU_NOTIFY1_union_tag { /* Notification 1 Status */
  vuint32_t R;
  struct {
    vuint32_t CH1_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH2_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH3_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH4_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH5_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH6_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH7_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH8_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH9_IS1:1;               /* Interrupt 1 Status */
    vuint32_t CH10_IS1:1;              /* Interrupt 1 Status */
    vuint32_t CH11_IS1:1;              /* Interrupt 1 Status */
    vuint32_t CH12_IS1:1;              /* Interrupt 1 Status */
    vuint32_t _unused_12:20;
  } B;
} SMU_MRU_NOTIFY1_tag;

typedef union SMU_MRU_CH1_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH1_MB0_tag;

typedef union SMU_MRU_CH1_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH1_MB1_tag;

typedef union SMU_MRU_CH1_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH1_MB2_tag;

typedef union SMU_MRU_CH1_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH1_MB3_tag;

typedef union SMU_MRU_CH2_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH2_MB0_tag;

typedef union SMU_MRU_CH2_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH2_MB1_tag;

typedef union SMU_MRU_CH2_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH2_MB2_tag;

typedef union SMU_MRU_CH2_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH2_MB3_tag;

typedef union SMU_MRU_CH3_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH3_MB0_tag;

typedef union SMU_MRU_CH3_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH3_MB1_tag;

typedef union SMU_MRU_CH3_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH3_MB2_tag;

typedef union SMU_MRU_CH3_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH3_MB3_tag;

typedef union SMU_MRU_CH4_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH4_MB0_tag;

typedef union SMU_MRU_CH4_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH4_MB1_tag;

typedef union SMU_MRU_CH4_MB2_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH4_MB2_tag;

typedef union SMU_MRU_CH4_MB3_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH4_MB3_tag;

typedef union SMU_MRU_CH5_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH5_MB0_tag;

typedef union SMU_MRU_CH5_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH5_MB1_tag;

typedef union SMU_MRU_CH6_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH6_MB0_tag;

typedef union SMU_MRU_CH6_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH6_MB1_tag;

typedef union SMU_MRU_CH7_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH7_MB0_tag;

typedef union SMU_MRU_CH7_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH7_MB1_tag;

typedef union SMU_MRU_CH8_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH8_MB0_tag;

typedef union SMU_MRU_CH8_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH8_MB1_tag;

typedef union SMU_MRU_CH9_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH9_MB0_tag;

typedef union SMU_MRU_CH9_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH9_MB1_tag;

typedef union SMU_MRU_CH10_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH10_MB0_tag;

typedef union SMU_MRU_CH10_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH10_MB1_tag;

typedef union SMU_MRU_CH11_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH11_MB0_tag;

typedef union SMU_MRU_CH11_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH11_MB1_tag;

typedef union SMU_MRU_CH12_MB0_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH12_MB0_tag;

typedef union SMU_MRU_CH12_MB1_union_tag { /* Channel (x) Mailbox (n) */
  vuint32_t R;
  struct {
    vuint32_t MBD:32;                  /* Mailbox Data */
  } B;
} SMU_MRU_CH12_MB1_tag;

struct SMU_MRU_tag {
  SMU_MRU_CHXCONFIG_tag CHXCONFIG[12];
  uint8_t SMU_MRU_reserved0[320];
  SMU_MRU_NOTIFY0_tag NOTIFY0;         /* Notification 0 Status */
  SMU_MRU_NOTIFY1_tag NOTIFY1;         /* Notification 1 Status */
  uint8_t SMU_MRU_reserved1[65016];
  SMU_MRU_CH1_MB0_tag CH1_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH1_MB1_tag CH1_MB1;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH1_MB2_tag CH1_MB2;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH1_MB3_tag CH1_MB3;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved2[65520];
  SMU_MRU_CH2_MB0_tag CH2_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH2_MB1_tag CH2_MB1;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH2_MB2_tag CH2_MB2;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH2_MB3_tag CH2_MB3;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved3[65520];
  SMU_MRU_CH3_MB0_tag CH3_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH3_MB1_tag CH3_MB1;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH3_MB2_tag CH3_MB2;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH3_MB3_tag CH3_MB3;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved4[65520];
  SMU_MRU_CH4_MB0_tag CH4_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH4_MB1_tag CH4_MB1;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH4_MB2_tag CH4_MB2;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH4_MB3_tag CH4_MB3;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved5[65520];
  SMU_MRU_CH5_MB0_tag CH5_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH5_MB1_tag CH5_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved6[65528];
  SMU_MRU_CH6_MB0_tag CH6_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH6_MB1_tag CH6_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved7[65528];
  SMU_MRU_CH7_MB0_tag CH7_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH7_MB1_tag CH7_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved8[65528];
  SMU_MRU_CH8_MB0_tag CH8_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH8_MB1_tag CH8_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved9[65528];
  SMU_MRU_CH9_MB0_tag CH9_MB0;         /* Channel (x) Mailbox (n) */
  SMU_MRU_CH9_MB1_tag CH9_MB1;         /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved10[65528];
  SMU_MRU_CH10_MB0_tag CH10_MB0;       /* Channel (x) Mailbox (n) */
  SMU_MRU_CH10_MB1_tag CH10_MB1;       /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved11[65528];
  SMU_MRU_CH11_MB0_tag CH11_MB0;       /* Channel (x) Mailbox (n) */
  SMU_MRU_CH11_MB1_tag CH11_MB1;       /* Channel (x) Mailbox (n) */
  uint8_t SMU_MRU_reserved12[65528];
  SMU_MRU_CH12_MB0_tag CH12_MB0;       /* Channel (x) Mailbox (n) */
  SMU_MRU_CH12_MB1_tag CH12_MB1;       /* Channel (x) Mailbox (n) */
};


/* ============================================================================
   =============================== Module: SMU_SEMA42 =========================
   ============================================================================ */

typedef union SMU_SEMA42_GATE_union_tag { /* Gate */
  vuint8_t R;
  struct {
    vuint8_t GTFSM:4;                  /* Gate Finite State Machine */
    vuint8_t _unused_4:4;              /* Reserved */
  } B;
} SMU_SEMA42_GATE_tag;

typedef union SMU_SEMA42_RSTGT_R_union_tag { /* Reset Gate Read */
  vuint16_t R;
  struct {
    vuint16_t RSTGTN:8;                /* Reset Gate Number */
    vuint16_t RSTGMS:4;                /* Reset Gate Domain */
    vuint16_t RSTGSM:2;                /* Reset Gate Finite State Machine */
    vuint16_t _unused_14:2;
  } B;
} SMU_SEMA42_RSTGT_R_tag;

typedef union SMU_SEMA42_RSTGT_W_union_tag { /* Reset Gate Write */
  vuint16_t R;
  struct {
    vuint16_t RSTGTN:8;                /* Reset Gate Number */
    vuint16_t RSTGDP:8;                /* Reset Gate Data Pattern */
  } B;
} SMU_SEMA42_RSTGT_W_tag;

typedef union SMU_SEMA42_RSTGT_union_tag {
  SMU_SEMA42_RSTGT_R_tag R;            /* Reset Gate Read */
  SMU_SEMA42_RSTGT_W_tag W;            /* Reset Gate Write */
} SMU_SEMA42_RSTGT_tag;

struct SMU_SEMA42_tag {
  SMU_SEMA42_GATE_tag GATE[16];        /* Gate */
  uint8_t SMU_SEMA42_reserved0[50];
  SMU_SEMA42_RSTGT_tag RSTGT;
};


/* ============================================================================
   =============================== Module: SMU_SRG_S ==========================
   ============================================================================ */

typedef union SMU_SRG_S_CR0_union_tag { /* Control 0 */
  vuint32_t R;
  struct {
    vuint32_t MSTR0DOW:1;              /* MSTR0 Disable Optimized Write */
    vuint32_t CCMDOW:1;                /* CCM Disable Optimized Write */
    vuint32_t CSMDOW:1;                /* CSM Disable Optimized Write */
    vuint32_t DEDOW:1;                 /* DMA ETR Disable Optimized Write */
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t MSTR0HPE:1;              /* MSTR High Priority Elevation */
    vuint32_t CCMHPE:1;                /* CCM High Priority Elevation */
    vuint32_t CSMHPE:1;                /* CSM High Priority Elevation */
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t SLV0SBE:1;               /* SLV0 Suppress Bus Error */
    vuint32_t AIPS0SBE:1;              /* SMU.AIPS_0 Suppress Bus Error */
    vuint32_t AIPS1SBE:1;              /* SMU.AIPS_1 Suppress Bus Error */
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} SMU_SRG_S_CR0_tag;

typedef union SMU_SRG_S_LCR0_union_tag { /* Lock Control 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t LNSMPU:1;                /* Lock Non-Secure MPU */
    vuint32_t _unused_17:1;
    vuint32_t LNSVTOR:1;               /* Lock Non-Secure Vector Table */
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} SMU_SRG_S_LCR0_tag;

typedef union SMU_SRG_S_LCR2_union_tag { /* Lock Control 2 */
  vuint32_t R;
  struct {
    vuint32_t REMAP_EN:1;              /* Remap Enable */
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t REMAP0:1;                /* Remap RAM0 */
    vuint32_t REMAP1:1;                /* Remap RAM1 */
    vuint32_t REMAP2:1;                /* Remap RAM2 */
    vuint32_t REMAP3:1;                /* Remap RAM3 */
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} SMU_SRG_S_LCR2_tag;

typedef union SMU_SRG_S_LCR3_union_tag { /* Lock Control 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t R0RO:5;                  /* RAM0 Remap Offset */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t R1RO:5;                  /* RAM1 Remap Offset */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} SMU_SRG_S_LCR3_tag;

typedef union SMU_SRG_S_LCR4_union_tag { /* Lock Control 4 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t R2RO:5;                  /* RAM2 Remap Offset */
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t R3RO:5;                  /* RAM3 Remap Offset */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} SMU_SRG_S_LCR4_tag;

typedef union SMU_SRG_S_LCR7_union_tag { /* Lock Control 7 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:1;
    vuint32_t _unused_5:1;
    vuint32_t _unused_6:1;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:1;
    vuint32_t _unused_9:1;
    vuint32_t _unused_10:1;
    vuint32_t _unused_11:1;
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t DBGSA:1;                 /* Debug Secure Access */
    vuint32_t _unused_17:1;
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t _unused_20:1;
    vuint32_t _unused_21:1;
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t DID:4;                   /* Domain ID */
    vuint32_t _unused_28:2;
    vuint32_t LKR:2;                   /* Lock Register */
  } B;
} SMU_SRG_S_LCR7_tag;

typedef union SMU_SRG_S_SR0_union_tag { /* Status 0 */
  vuint32_t R;
  struct {
    vuint32_t R0EA:1;                  /* RAM0 Error Alarm */
    vuint32_t R1EA:1;                  /* RAM1 Error Alarm */
    vuint32_t R2EA:1;                  /* RAM2 Error Alarm */
    vuint32_t R3EA:1;                  /* RAM3 Error Alarm */
    vuint32_t R0WEA:1;                 /* RAM0 Write Error Alarm */
    vuint32_t R1WEA:1;                 /* RAM1 Write Error Alarm */
    vuint32_t R2WEA:1;                 /* RAM2 Write Error Alarm */
    vuint32_t R3WEA:1;                 /* RAM3 Write Error Alarm */
    vuint32_t CTRE:1;                  /* Code Bus TAG Array Recoverable Error */
    vuint32_t CDRE:1;                  /* Code Bus DATA Array Recoverable Error */
    vuint32_t STRE:1;                  /* System Bus TAG Array Recoverable Error */
    vuint32_t SDRE:1;                  /* System Bus DATA Array Recoverable Error */
    vuint32_t _unused_12:1;
    vuint32_t _unused_13:1;
    vuint32_t _unused_14:1;
    vuint32_t _unused_15:1;
    vuint32_t CCCMCRE:1;               /* Code Bus LMEM Cache Array Cache Maintenance Command Recoverable Error */
    vuint32_t SCCMCRE:1;               /* System Bus LMEM Cache Array Cache Maintenance Command Recoverable Error */
    vuint32_t _unused_18:1;
    vuint32_t _unused_19:1;
    vuint32_t COREH:1;                 /* Core Halted */
    vuint32_t CORETU:1;                /* Core Trace Units Enabled */
    vuint32_t _unused_22:1;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:1;
    vuint32_t _unused_28:1;
    vuint32_t _unused_29:1;
    vuint32_t _unused_30:1;
    vuint32_t _unused_31:1;
  } B;
} SMU_SRG_S_SR0_tag;

struct SMU_SRG_S_tag {
  SMU_SRG_S_CR0_tag CR0;               /* Control 0 */
  uint8_t SMU_SRG_S_reserved0[252];
  SMU_SRG_S_LCR0_tag LCR0;             /* Lock Control 0 */
  uint8_t SMU_SRG_S_reserved1[4];
  SMU_SRG_S_LCR2_tag LCR2;             /* Lock Control 2 */
  SMU_SRG_S_LCR3_tag LCR3;             /* Lock Control 3 */
  SMU_SRG_S_LCR4_tag LCR4;             /* Lock Control 4 */
  uint8_t SMU_SRG_S_reserved2[8];
  SMU_SRG_S_LCR7_tag LCR7;             /* Lock Control 7 */
  uint8_t SMU_SRG_S_reserved3[224];
  SMU_SRG_S_SR0_tag SR0;               /* Status 0 */
};


/* ============================================================================
   =============================== Module: SMU_XRDC ===========================
   ============================================================================ */

typedef union SMU_XRDC_CR_union_tag {  /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} SMU_XRDC_CR_tag;

typedef union SMU_XRDC_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} SMU_XRDC_HWCFG0_tag;

typedef union SMU_XRDC_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} SMU_XRDC_HWCFG1_tag;

typedef union SMU_XRDC_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} SMU_XRDC_MDACFG_tag;

typedef union SMU_XRDC_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} SMU_XRDC_MRCFG_tag;

typedef union SMU_XRDC_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} SMU_XRDC_DERRLOC_tag;

typedef union SMU_XRDC_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} SMU_XRDC_DERRW0_DERR_W0_tag;

typedef union SMU_XRDC_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} SMU_XRDC_DERRW0_DERR_W1_tag;

typedef union SMU_XRDC_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} SMU_XRDC_DERRW0_DERR_W3_tag;

typedef struct SMU_XRDC_DERRW0_struct_tag {
  SMU_XRDC_DERRW0_DERR_W0_tag DERR_W0; /* Domain Error Word 0 */
  SMU_XRDC_DERRW0_DERR_W1_tag DERR_W1; /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  SMU_XRDC_DERRW0_DERR_W3_tag DERR_W3; /* Domain Error Word 3 */
} SMU_XRDC_DERRW0_tag;

typedef union SMU_XRDC_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W0_0_DFMT1_tag;

typedef union SMU_XRDC_MDA_W1_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W1_0_DFMT1_tag;

typedef union SMU_XRDC_MDA_W2_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W2_0_DFMT1_tag;

typedef union SMU_XRDC_MDA_W3_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W3_0_DFMT1_tag;

typedef union SMU_XRDC_MDA_W4_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W4_0_DFMT1_tag;

typedef union SMU_XRDC_MDA_W5_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W5_0_DFMT1_tag;

typedef union SMU_XRDC_MDA_W6_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W6_0_DFMT1_tag;

typedef union SMU_XRDC_MDA_W7_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W7_0_DFMT1_tag;

typedef union SMU_XRDC_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MDA_W0_1_DFMT1_tag;

typedef union SMU_XRDC_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} SMU_XRDC_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union SMU_XRDC_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct SMU_XRDC_PDAC_SLOT_PDACN_struct_tag {
  SMU_XRDC_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  SMU_XRDC_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} SMU_XRDC_PDAC_SLOT_PDACN_tag;

typedef struct SMU_XRDC_PDAC_SLOT_struct_tag {
  SMU_XRDC_PDAC_SLOT_PDACN_tag PDACN[113]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[120];
} SMU_XRDC_PDAC_SLOT_tag;

typedef union SMU_XRDC_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} SMU_XRDC_MRGDN_XRDC_MRGD_W0_tag;

typedef union SMU_XRDC_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} SMU_XRDC_MRGDN_XRDC_MRGD_W1_tag;

typedef union SMU_XRDC_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} SMU_XRDC_MRGDN_XRDC_MRGD_W2_tag;

typedef union SMU_XRDC_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} SMU_XRDC_MRGDN_XRDC_MRGD_W3_tag;

typedef struct SMU_XRDC_MRGDN_struct_tag {
  SMU_XRDC_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  SMU_XRDC_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  SMU_XRDC_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  SMU_XRDC_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} SMU_XRDC_MRGDN_tag;

struct SMU_XRDC_tag {
  SMU_XRDC_CR_tag CR;                  /* Control */
  uint8_t SMU_XRDC_reserved0[236];
  SMU_XRDC_HWCFG0_tag HWCFG0;          /* Hardware Configuration 0 */
  SMU_XRDC_HWCFG1_tag HWCFG1;          /* Hardware Configuration 1 */
  uint8_t SMU_XRDC_reserved1[8];
  SMU_XRDC_MDACFG_tag MDACFG[2];       /* Master Domain Assignment Configuration */
  uint8_t SMU_XRDC_reserved2[62];
  SMU_XRDC_MRCFG_tag MRCFG[4];         /* Memory Region Configuration */
  uint8_t SMU_XRDC_reserved3[188];
  SMU_XRDC_DERRLOC_tag DERRLOC[16];    /* Domain Error Location */
  uint8_t SMU_XRDC_reserved4[448];
  SMU_XRDC_DERRW0_tag DERRW0[18];      /* Valid array indices: 0-3, 16-17 */
  uint8_t SMU_XRDC_reserved5[736];
  SMU_XRDC_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  SMU_XRDC_MDA_W1_0_DFMT1_tag MDA_W1_0_DFMT1; /* Master Domain Assignment */
  SMU_XRDC_MDA_W2_0_DFMT1_tag MDA_W2_0_DFMT1; /* Master Domain Assignment */
  SMU_XRDC_MDA_W3_0_DFMT1_tag MDA_W3_0_DFMT1; /* Master Domain Assignment */
  SMU_XRDC_MDA_W4_0_DFMT1_tag MDA_W4_0_DFMT1; /* Master Domain Assignment */
  SMU_XRDC_MDA_W5_0_DFMT1_tag MDA_W5_0_DFMT1; /* Master Domain Assignment */
  SMU_XRDC_MDA_W6_0_DFMT1_tag MDA_W6_0_DFMT1; /* Master Domain Assignment */
  SMU_XRDC_MDA_W7_0_DFMT1_tag MDA_W7_0_DFMT1; /* Master Domain Assignment */
  SMU_XRDC_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  uint8_t SMU_XRDC_reserved6[2012];
  SMU_XRDC_PDAC_SLOT_tag PDAC_SLOT[2];
  uint8_t SMU_XRDC_reserved7[2048];
  SMU_XRDC_MRGDN_tag MRGDN[4][16];
};


/* ============================================================================
   =============================== Module: SPFU ===============================
   ============================================================================ */

typedef union SPFU_PFN_PF_CFG_union_tag { /* Prefetch Configuration */
  vuint32_t R;
  struct {
    vuint32_t PFEN:1;                  /* Prefetch Enable */
    vuint32_t _unused_1:7;
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_12:4;
    vuint32_t THRSHLD:4;               /* Prefetch Threshold */
    vuint32_t _unused_20:4;
    vuint32_t PF_SIZE:4;               /* Prefetch Size */
    vuint32_t _unused_28:4;
  } B;
} SPFU_PFN_PF_CFG_tag;

typedef struct SPFU_PFN_struct_tag {
  SPFU_PFN_PF_CFG_tag PF_CFG;          /* Prefetch Configuration */
  uint8_t PFN_reserved0[252];
} SPFU_PFN_tag;

struct SPFU_tag {
  SPFU_PFN_tag PFN[5];
};


/* ============================================================================
   =============================== Module: SPI ================================
   ============================================================================ */

typedef union SPI_MCR_union_tag {      /* Module Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t HALT:1;                  /* Halt */
    vuint32_t PES:1;                   /* Parity Error Stop */
    vuint32_t FCPCS:1;                 /* Fast Continuous PCS Mode */
    vuint32_t XSPI:1;                  /* Extended SPI Mode */
    vuint32_t _unused_4:4;             /* Reserved */
    vuint32_t SMPL_PT:2;               /* Sample Point */
    vuint32_t CLR_RXF:1;               /* Clear RX FIFO */
    vuint32_t CLR_TXF:1;               /* Clear TX FIFO */
    vuint32_t DIS_RXF:1;               /* Disable Receive FIFO */
    vuint32_t DIS_TXF:1;               /* Disable Transmit FIFO */
    vuint32_t MDIS:1;                  /* Module Disable */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t PCSIS:5;                 /* Peripheral Chip Select x Inactive State */
    vuint32_t _unused_21:3;            /* Always write the reset value to this field. */
    vuint32_t ROOE:1;                  /* Receive FIFO Overflow Overwrite Enable */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t MTFE:1;                  /* Modified Transfer Format Enable */
    vuint32_t FRZ:1;                   /* Freeze */
    vuint32_t DCONF:2;                 /* SPI Configuration */
    vuint32_t CONT_SCKE:1;             /* Continuous SCK Enable */
    vuint32_t MSTR:1;                  /* Master/Slave Mode Select */
  } B;
} SPI_MCR_tag;

typedef union SPI_TCR_union_tag {      /* Transfer Count Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t SPI_TCNT:16;             /* SPI Transfer Counter */
  } B;
} SPI_TCR_tag;

typedef union SPI_MODE_CTAR_union_tag { /* Clock and Transfer Attributes Register (in Master mode) */
  vuint32_t R;
  struct {
    vuint32_t BR:4;                    /* Baud Rate Scaler */
    vuint32_t DT:4;                    /* Delay After Transfer Scaler */
    vuint32_t ASC:4;                   /* After SCK Delay Scaler */
    vuint32_t CSSCK:4;                 /* PCS to SCK Delay Scaler */
    vuint32_t PBR:2;                   /* Baud Rate Prescaler */
    vuint32_t PDT:2;                   /* Delay after Transfer Prescaler */
    vuint32_t PASC:2;                  /* After SCK Delay Prescaler */
    vuint32_t PCSSCK:2;                /* PCS to SCK Delay Prescaler */
    vuint32_t LSBFE:1;                 /* LSB First */
    vuint32_t CPHA:1;                  /* Clock Phase */
    vuint32_t CPOL:1;                  /* Clock Polarity */
    vuint32_t FMSZ:4;                  /* Frame Size */
    vuint32_t DBR:1;                   /* Double Baud Rate */
  } B;
} SPI_MODE_CTAR_tag;

typedef union SPI_MODE_CTAR_SLAVE_union_tag { /* Clock and Transfer Attributes Register (in Slave mode) */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:22;            /* Reserved */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t PP:1;                    /* Parity Polarity */
    vuint32_t PE:1;                    /* Parity Enable */
    vuint32_t CPHA:1;                  /* Clock Phase */
    vuint32_t CPOL:1;                  /* Clock Polarity */
    vuint32_t FMSZ:5;                  /* Frame Size */
  } B;
} SPI_MODE_CTAR_SLAVE_tag;

typedef union SPI_MODE_union_tag {
  SPI_MODE_CTAR_tag CTAR[6];           /* Clock and Transfer Attributes Register (in Master mode) */
  SPI_MODE_CTAR_SLAVE_tag CTAR_SLAVE[1]; /* Clock and Transfer Attributes Register (in Slave mode) */
} SPI_MODE_tag;

typedef union SPI_SR_union_tag {       /* Status Register */
  vuint32_t R;
  struct {
    vuint32_t POPNXTPTR:4;             /* Pop Next Pointer */
    vuint32_t RXCTR:4;                 /* RX FIFO Counter */
    vuint32_t TXNXTPTR:4;              /* Transmit Next Pointer */
    vuint32_t TXCTR:4;                 /* TX FIFO Counter */
    vuint32_t CMDFFF:1;                /* Command FIFO Fill Flag */
    vuint32_t RFDF:1;                  /* Receive FIFO Drain Flag */
    vuint32_t TFIWF:1;                 /* Transmit FIFO Invalid Write Flag */
    vuint32_t RFOF:1;                  /* Receive FIFO Overflow Flag */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t SPEF:1;                  /* SPI Parity Error Flag */
    vuint32_t _unused_22:1;            /* Reserved */
    vuint32_t CMDTCF:1;                /* Command Transfer Complete Flag */
    vuint32_t BSYF:1;                  /* Busy Flag */
    vuint32_t TFFF:1;                  /* Transmit FIFO Fill Flag */
    vuint32_t _unused_26:1;            /* Reserved */
    vuint32_t TFUF:1;                  /* Transmit FIFO Underflow Flag */
    vuint32_t EOQF:1;                  /* End of Queue Flag */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t TXRXS:1;                 /* TX and RX Status */
    vuint32_t TCF:1;                   /* Transfer Complete Flag */
  } B;
} SPI_SR_tag;

typedef union SPI_RSER_union_tag {     /* DMA/Interrupt Request Select and Enable Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:14;            /* Reserved */
    vuint32_t _unused_14:1;            /* Always write the reset value to this field. */
    vuint32_t CMDFFF_DIRS:1;           /* Command FIFO Fill DMA or Interrupt Request Select */
    vuint32_t RFDF_DIRS:1;             /* Receive FIFO Drain DMA or Interrupt Request Select */
    vuint32_t RFDF_RE:1;               /* Receive FIFO Drain Request Enable */
    vuint32_t TFIWF_RE:1;              /* Transmit FIFO Invalid Write Request Enable */
    vuint32_t RFOF_RE:1;               /* Receive FIFO Overflow Request Enable */
    vuint32_t _unused_20:1;            /* Always write the reset value to this field. */
    vuint32_t SPEF_RE:1;               /* SPI Parity Error Request Enable */
    vuint32_t _unused_22:1;            /* Always write the reset value to this field. */
    vuint32_t CMDTCF_RE:1;             /* Command Transmission Complete Request Enable */
    vuint32_t TFFF_DIRS:1;             /* Transmit FIFO Fill DMA or Interrupt Request Select */
    vuint32_t TFFF_RE:1;               /* Transmit FIFO Fill Request Enable */
    vuint32_t _unused_26:1;            /* Always write the reset value to this field. */
    vuint32_t TFUF_RE:1;               /* Transmit FIFO Underflow Request Enable */
    vuint32_t EOQF_RE:1;               /* Finished Request Enable */
    vuint32_t _unused_29:1;            /* Always write the reset value to this field. */
    vuint32_t CMDFFF_RE:1;             /* Command FIFO Fill Flag Request Enable */
    vuint32_t TCF_RE:1;                /* Transmission Complete Request Enable */
  } B;
} SPI_RSER_tag;

typedef union SPI_PUSHR_PUSHR_union_tag { /* PUSH TX FIFO Register In Master Mode */
  vuint32_t R;
  struct {
    vuint32_t TXDATA:16;               /* Transmit Data */
    vuint32_t PCS:5;                   /* PCS */
    vuint32_t _unused_21:3;            /* Always write the reset value to this field. */
    vuint32_t PP_MCSC:1;               /* Parity Polarity or Mask TCSC Delay in Next Frame */
    vuint32_t PE_MASC:1;               /* Parity Enable or Mask TASC Delay in Current Frame */
    vuint32_t CTCNT:1;                 /* Clear Transfer Counter */
    vuint32_t EOQ:1;                   /* End of Queue */
    vuint32_t CTAS:3;                  /* Clock and Transfer Attributes Select */
    vuint32_t CONT:1;                  /* Continuous Peripheral Chip Select Enable */
  } B;
} SPI_PUSHR_PUSHR_tag;

typedef union SPI_PUSHR_PUSHR_SLAVE_union_tag { /* PUSH TX FIFO Register In Slave Mode */
  vuint32_t R;
  struct {
    vuint32_t TXDATA:16;               /* Transmit Data */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} SPI_PUSHR_PUSHR_SLAVE_tag;

typedef union SPI_PUSHR_union_tag {
  SPI_PUSHR_PUSHR_tag PUSHR;           /* PUSH TX FIFO Register In Master Mode */
  SPI_PUSHR_PUSHR_SLAVE_tag PUSHR_SLAVE; /* PUSH TX FIFO Register In Slave Mode */
} SPI_PUSHR_tag;

typedef union SPI_POPR_union_tag {     /* POP RX FIFO Register */
  vuint32_t R;
  struct {
    vuint32_t RXDATA:32;               /* Received Data */
  } B;
} SPI_POPR_tag;

typedef union SPI_TXFR_union_tag {     /* Transmit FIFO Registers */
  vuint32_t R;
  struct {
    vuint32_t TXDATA:16;               /* Transmit Data */
    vuint32_t TXCMD_TXDATA:16;         /* Transmit Command or Transmit Data */
  } B;
} SPI_TXFR_tag;

typedef union SPI_RXFR_union_tag {     /* Receive FIFO Registers */
  vuint32_t R;
  struct {
    vuint32_t RXDATA:32;               /* Receive Data */
  } B;
} SPI_RXFR_tag;

typedef union SPI_CTARE_union_tag {    /* Clock and Transfer Attributes Register Extended */
  vuint32_t R;
  struct {
    vuint32_t DTCP:11;                 /* Data Transfer Count Preload */
    vuint32_t _unused_11:5;            /* Reserved */
    vuint32_t FMSZE:1;                 /* Frame Size Extended */
    vuint32_t _unused_17:15;           /* Reserved */
  } B;
} SPI_CTARE_tag;

typedef union SPI_SREX_union_tag {     /* Status Register Extended */
  vuint32_t R;
  struct {
    vuint32_t CMDNXTPTR:4;             /* Command Next Pointer */
    vuint32_t CMDCTR:5;                /* CMD FIFO Counter */
    vuint32_t _unused_9:2;             /* Reserved */
    vuint32_t RXCTR4:1;                /* RX FIFO Counter[4] */
    vuint32_t _unused_12:2;            /* Reserved */
    vuint32_t TXCTR4:1;                /* TX FIFO Counter[4] */
    vuint32_t _unused_15:17;           /* Reserved */
  } B;
} SPI_SREX_tag;

struct SPI_tag {
  SPI_MCR_tag MCR;                     /* Module Configuration Register */
  uint8_t SPI_reserved0[4];
  SPI_TCR_tag TCR;                     /* Transfer Count Register */
  SPI_MODE_tag MODE;
  uint8_t SPI_reserved1[8];
  SPI_SR_tag SR;                       /* Status Register */
  SPI_RSER_tag RSER;                   /* DMA/Interrupt Request Select and Enable Register */
  SPI_PUSHR_tag PUSHR;
  SPI_POPR_tag POPR;                   /* POP RX FIFO Register */
  SPI_TXFR_tag TXFR[5];                /* Transmit FIFO Registers */
  uint8_t SPI_reserved2[44];
  SPI_RXFR_tag RXFR[5];                /* Receive FIFO Registers */
  uint8_t SPI_reserved3[140];
  SPI_CTARE_tag CTARE[6];              /* Clock and Transfer Attributes Register Extended */
  uint8_t SPI_reserved4[8];
  SPI_SREX_tag SREX;                   /* Status Register Extended */
};


/* ============================================================================
   =============================== Module: SRAMCTL ============================
   ============================================================================ */

typedef union SRAMCTL_RAMCR_union_tag { /* RAM Control */
  vuint32_t R;
  struct {
    vuint32_t INIT:1;                  /* Initialization Request */
    vuint32_t IWS:2;                   /* Initialization Wait States */
    vuint32_t _unused_3:5;
    vuint32_t INIT_SYSA:1;             /* Initialize With System Address */
    vuint32_t _unused_9:23;
  } B;
} SRAMCTL_RAMCR_tag;

typedef union SRAMCTL_RAMIAS_union_tag { /* RAM Initialization Address Start */
  vuint32_t R;
  struct {
    vuint32_t IAS:32;                  /* Initialization Address Start */
  } B;
} SRAMCTL_RAMIAS_tag;

typedef union SRAMCTL_RAMIAE_union_tag { /* RAM Initialization Address End */
  vuint32_t R;
  struct {
    vuint32_t IAE:32;                  /* Initialization Address End */
  } B;
} SRAMCTL_RAMIAE_tag;

typedef union SRAMCTL_RAMSR_union_tag { /* RAM Status */
  vuint32_t R;
  struct {
    vuint32_t IDONE:1;                 /* Initialization Done */
    vuint32_t BUSERR:1;                /* Bus Error */
    vuint32_t IPEND:1;                 /* Initialization Pending */
    vuint32_t AVALID:1;                /* Addresses Valid */
    vuint32_t _unused_4:1;
    vuint32_t AERR:1;                  /* ECC Address Error */
    vuint32_t MLTERR:1;                /* ECC Multi-Bit Error */
    vuint32_t SGLERR:1;                /* ECC Single-Bit Error */
    vuint32_t SYND:8;                  /* ECC Syndrome Value */
    vuint32_t EINFO:8;                 /* Event Information */
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:7;
  } B;
} SRAMCTL_RAMSR_tag;

typedef union SRAMCTL_RAMMEMA_union_tag { /* RAM ECC Address */
  vuint32_t R;
  struct {
    vuint32_t MEMA:17;                 /* RAM Bank Address */
    vuint32_t _unused_17:3;
    vuint32_t BANK:5;                  /* RAM Bank ID */
    vuint32_t _unused_25:7;
  } B;
} SRAMCTL_RAMMEMA_tag;

typedef union SRAMCTL_RAMSYSA_union_tag { /* RAM System Address */
  vuint32_t R;
  struct {
    vuint32_t SYSA:32;                 /* System Address */
  } B;
} SRAMCTL_RAMSYSA_tag;

typedef union SRAMCTL_RAMECCNT_union_tag { /* RAM Correctable Error Count */
  vuint32_t R;
  struct {
    vuint32_t ECCNT:8;                 /* ECC Correctable Error Count */
    vuint32_t _unused_8:24;
  } B;
} SRAMCTL_RAMECCNT_tag;

typedef union SRAMCTL_RAMEID0_union_tag { /* RAM Error Injection Data 0 */
  vuint32_t R;
  struct {
    vuint32_t EID_W0:32;               /* Error Injection Data Word 0 */
  } B;
} SRAMCTL_RAMEID0_tag;

typedef union SRAMCTL_RAMEID1_union_tag { /* RAM Error Injection Data 1 */
  vuint32_t R;
  struct {
    vuint32_t EID_W1:32;               /* Error Injection Data Word 1 */
  } B;
} SRAMCTL_RAMEID1_tag;

typedef union SRAMCTL_RAMEIDC_union_tag { /* RAM Error Injection Data Control */
  vuint32_t R;
  struct {
    vuint32_t EID_CKB:8;               /* Error Injection Data Checkbits */
    vuint32_t _unused_8:16;
    vuint32_t EIP_EN:1;                /* Error Injection Into Pipeline Enable */
    vuint32_t _unused_25:5;
    vuint32_t EIA_EN:1;                /* Error Injection Address Enable */
    vuint32_t EID_EN:1;                /* Error Injection Data Enable */
  } B;
} SRAMCTL_RAMEIDC_tag;

typedef union SRAMCTL_RAMEIA_union_tag { /* RAM Error Injection Base Address */
  vuint32_t R;
  struct {
    vuint32_t EIA:32;                  /* Error Injection Base Address */
  } B;
} SRAMCTL_RAMEIA_tag;

typedef union SRAMCTL_RAMEIAM_union_tag { /* RAM Error Injection Address Mask */
  vuint32_t R;
  struct {
    vuint32_t EIAM:32;                 /* Error Injection Address Mask */
  } B;
} SRAMCTL_RAMEIAM_tag;

typedef union SRAMCTL_RAMMAXA_union_tag { /* RAM Maximum-Value Address */
  vuint32_t R;
  struct {
    vuint32_t MAXA:32;                 /* Maximum Address */
  } B;
} SRAMCTL_RAMMAXA_tag;

typedef union SRAMCTL_RAMCR2_union_tag { /* RAM Control 2 */
  vuint32_t R;
  struct {
    vuint32_t PREF:1;                  /* Prefetch for Read Bursts */
    vuint32_t WBUF:2;                  /* Write Buffer Control */
    vuint32_t DEM:1;                   /* Disable Exclusive Monitor */
    vuint32_t _unused_4:28;
  } B;
} SRAMCTL_RAMCR2_tag;

typedef union SRAMCTL_RAMPC_union_tag { /* RAM Power Control */
  vuint32_t R;
  struct {
    vuint32_t PCUT:8;                  /* Power Control Memory Cuts, merged from more bitFields with different positions or sizes */
    vuint32_t _unused_8:8;
    vuint32_t PCNT:4;                  /* Power Control Counter */
    vuint32_t _unused_20:12;
  } B;
} SRAMCTL_RAMPC_tag;

struct SRAMCTL_tag {
  SRAMCTL_RAMCR_tag RAMCR;             /* RAM Control */
  SRAMCTL_RAMIAS_tag RAMIAS;           /* RAM Initialization Address Start */
  SRAMCTL_RAMIAE_tag RAMIAE;           /* RAM Initialization Address End */
  SRAMCTL_RAMSR_tag RAMSR;             /* RAM Status */
  SRAMCTL_RAMMEMA_tag RAMMEMA;         /* RAM ECC Address */
  uint8_t SRAMCTL_reserved0[4];
  SRAMCTL_RAMSYSA_tag RAMSYSA;         /* RAM System Address */
  SRAMCTL_RAMECCNT_tag RAMECCNT;       /* RAM Correctable Error Count */
  SRAMCTL_RAMEID0_tag RAMEID0;         /* RAM Error Injection Data 0 */
  SRAMCTL_RAMEID1_tag RAMEID1;         /* RAM Error Injection Data 1 */
  SRAMCTL_RAMEIDC_tag RAMEIDC;         /* RAM Error Injection Data Control */
  uint8_t SRAMCTL_reserved1[4];
  SRAMCTL_RAMEIA_tag RAMEIA;           /* RAM Error Injection Base Address */
  SRAMCTL_RAMEIAM_tag RAMEIAM;         /* RAM Error Injection Address Mask */
  uint8_t SRAMCTL_reserved2[8];
  SRAMCTL_RAMMAXA_tag RAMMAXA;         /* RAM Maximum-Value Address */
  uint8_t SRAMCTL_reserved3[60];
  SRAMCTL_RAMCR2_tag RAMCR2;           /* RAM Control 2 */
  uint8_t SRAMCTL_reserved4[124];
  SRAMCTL_RAMPC_tag RAMPC;             /* RAM Power Control */
};


/* ============================================================================
   =============================== Module: SRX ================================
   ============================================================================ */

typedef union SRX_GBL_CTRL_union_tag { /* Global Control Register */
  vuint32_t R;
  struct {
    vuint32_t SENT_EN:1;               /* SENT_EN */
    vuint32_t _unused_1:1;             /* Reserved */
    vuint32_t DBG_FRZ:1;               /* DBG_FRZ */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t FAST_CLR:1;              /* FAST_CLR */
    vuint32_t _unused_5:3;             /* Reserved */
    vuint32_t SMDUIE:1;                /* SMDUIE */
    vuint32_t FMDUIE:1;                /* FMDUIE */
    vuint32_t FMFOIE:1;                /* FMFOIE */
    vuint32_t _unused_11:4;            /* Reserved */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t FIFOWM:5;                /* FIFOWM */
    vuint32_t _unused_21:3;            /* Reserved */
    vuint32_t TSPRSC:8;                /* TSPRSC */
  } B;
} SRX_GBL_CTRL_tag;

typedef union SRX_CHNL_EN_union_tag {  /* Channel Enable Register */
  vuint32_t R;
  struct {
    vuint32_t EN_CH:8;                 /* EN_CH */
    vuint32_t _unused_8:24;
  } B;
} SRX_CHNL_EN_tag;

typedef union SRX_SLOW_MSG_UVF_union_tag { /* Slow Channel Status */
  vuint32_t R;
  struct {
    vuint32_t CH0_SMDU:1;              /* Slow Serial Message DMA Underflow */
    vuint32_t CH1_SMDU:1;              /* Slow Serial Message DMA Underflow */
    vuint32_t CH2_SMDU:1;              /* Slow Serial Message DMA Underflow */
    vuint32_t CH3_SMDU:1;              /* Slow Serial Message DMA Underflow */
    vuint32_t CH4_SMDU:1;              /* Slow Serial Message DMA Underflow */
    vuint32_t CH5_SMDU:1;              /* Slow Serial Message DMA Underflow */
    vuint32_t CH6_SMDU:1;              /* Slow Serial Message DMA Underflow */
    vuint32_t CH7_SMDU:1;              /* Slow Serial Message DMA Underflow */
    vuint32_t _unused_8:24;
  } B;
} SRX_SLOW_MSG_UVF_tag;

typedef union SRX_FMSG_RDY_union_tag { /* Fast Message Ready Status Register */
  vuint32_t R;
  struct {
    vuint32_t F_RDY0:1;                /* F_RDY */
    vuint32_t F_RDY1:1;                /* F_RDY */
    vuint32_t F_RDY2:1;                /* F_RDY */
    vuint32_t F_RDY3:1;                /* F_RDY */
    vuint32_t F_RDY4:1;                /* F_RDY */
    vuint32_t F_RDY5:1;                /* F_RDY */
    vuint32_t F_RDY6:1;                /* F_RDY */
    vuint32_t F_RDY7:1;                /* F_RDY */
    vuint32_t _unused_8:24;
  } B;
} SRX_FMSG_RDY_tag;

typedef union SRX_SMSG_RDY_union_tag { /* Slow Serial Message Ready Status Register */
  vuint32_t R;
  struct {
    vuint32_t S_RDY0:1;                /* S_RDY */
    vuint32_t S_RDY1:1;                /* S_RDY */
    vuint32_t S_RDY2:1;                /* S_RDY */
    vuint32_t S_RDY3:1;                /* S_RDY */
    vuint32_t S_RDY4:1;                /* S_RDY */
    vuint32_t S_RDY5:1;                /* S_RDY */
    vuint32_t S_RDY6:1;                /* S_RDY */
    vuint32_t S_RDY7:1;                /* S_RDY */
    vuint32_t _unused_8:24;
  } B;
} SRX_SMSG_RDY_tag;

typedef union SRX_CHNL_FIFO_EN_union_tag { /* Channel FIFO Enable */
  vuint32_t R;
  struct {
    vuint32_t CH0FIFO_EN:1;            /* FIFO Enable */
    vuint32_t CH1FIFO_EN:1;            /* FIFO Enable */
    vuint32_t CH2FIFO_EN:1;            /* FIFO Enable */
    vuint32_t CH3FIFO_EN:1;            /* FIFO Enable */
    vuint32_t CH4FIFO_EN:1;            /* FIFO Enable */
    vuint32_t CH5FIFO_EN:1;            /* FIFO Enable */
    vuint32_t CH6FIFO_EN:1;            /* FIFO Enable */
    vuint32_t CH7FIFO_EN:1;            /* FIFO Enable */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} SRX_CHNL_FIFO_EN_tag;

typedef union SRX_DATA_CTRL1_union_tag { /* Data Control Register 1 */
  vuint32_t R;
  struct {
    vuint32_t NIBBCH7:3;               /* NIBBCH7 */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t NIBBCH6:3;               /* NIBBCH6 */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t NIBBCH5:3;               /* NIBBCH5 */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t NIBBCH4:3;               /* NIBBCH4 */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t NIBBCH3:3;               /* NIBBCH3 */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t NIBBCH2:3;               /* NIBBCH2 */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t NIBBCH1:3;               /* NIBBCH1 */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t NIBBCH0:3;               /* NIBBCH0 */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SRX_DATA_CTRL1_tag;

typedef union SRX_CHNL_OVF_UVF_union_tag { /* Fast Channel Status */
  vuint32_t R;
  struct {
    vuint32_t FDMU_CH0:1;              /* Channel n Fast Message DMA Underflow */
    vuint32_t FDMU_CH1:1;              /* Channel n Fast Message DMA Underflow */
    vuint32_t FDMU_CH2:1;              /* Channel n Fast Message DMA Underflow */
    vuint32_t FDMU_CH3:1;              /* Channel n Fast Message DMA Underflow */
    vuint32_t FDMU_CH4:1;              /* Channel n Fast Message DMA Underflow */
    vuint32_t FDMU_CH5:1;              /* Channel n Fast Message DMA Underflow */
    vuint32_t FDMU_CH6:1;              /* Channel n Fast Message DMA Underflow */
    vuint32_t FDMU_CH7:1;              /* Channel n Fast Message DMA Underflow */
    vuint32_t _unused_8:8;
    vuint32_t FMFO_CH0:1;              /* Channel n Fast Message FIFO Overflow */
    vuint32_t FMFO_CH1:1;              /* Channel n Fast Message FIFO Overflow */
    vuint32_t FMFO_CH2:1;              /* Channel n Fast Message FIFO Overflow */
    vuint32_t FMFO_CH3:1;              /* Channel n Fast Message FIFO Overflow */
    vuint32_t FMFO_CH4:1;              /* Channel n Fast Message FIFO Overflow */
    vuint32_t FMFO_CH5:1;              /* Channel n Fast Message FIFO Overflow */
    vuint32_t FMFO_CH6:1;              /* Channel n Fast Message FIFO Overflow */
    vuint32_t FMFO_CH7:1;              /* Channel n Fast Message FIFO Overflow */
    vuint32_t _unused_24:8;
  } B;
} SRX_CHNL_OVF_UVF_tag;

typedef union SRX_FDMA_CTRL_union_tag { /* Fast Message DMA Control Register */
  vuint32_t R;
  struct {
    vuint32_t FDMA_EN:8;               /* FDMA_EN */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} SRX_FDMA_CTRL_tag;

typedef union SRX_SDMA_CTRL_union_tag { /* Slow Serial Message DMA Control Register */
  vuint32_t R;
  struct {
    vuint32_t SDMA_EN:8;               /* SDMA_EN */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} SRX_SDMA_CTRL_tag;

typedef union SRX_CHNL_NB_REORD1_union_tag { /* Channel Nibble Reorder 1 */
  vuint32_t R;
  struct {
    vuint32_t DATAORD7:4;              /* Data Reorder n */
    vuint32_t DATAORD6:4;              /* Data Reorder n */
    vuint32_t DATAORD5:4;              /* Data Reorder n */
    vuint32_t DATAORD4:4;              /* Data Reorder n */
    vuint32_t DATAORD3:4;              /* Data Reorder n */
    vuint32_t DATAORD2:4;              /* Data Reorder n */
    vuint32_t DATAORD1:4;              /* Data Reorder n */
    vuint32_t DATAORD0:4;              /* Data Reorder n */
  } B;
} SRX_CHNL_NB_REORD1_tag;

typedef union SRX_FMSG_RDY_IE_union_tag { /* Fast Message Ready Interrupt Control Register */
  vuint32_t R;
  struct {
    vuint32_t FRDY_IE0:1;              /* Fast message ready interrupt */
    vuint32_t FRDY_IE1:1;              /* Fast message ready interrupt */
    vuint32_t FRDY_IE2:1;              /* Fast message ready interrupt */
    vuint32_t FRDY_IE3:1;              /* Fast message ready interrupt */
    vuint32_t FRDY_IE4:1;              /* Fast message ready interrupt */
    vuint32_t FRDY_IE5:1;              /* Fast message ready interrupt */
    vuint32_t FRDY_IE6:1;              /* Fast message ready interrupt */
    vuint32_t FRDY_IE7:1;              /* Fast message ready interrupt */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} SRX_FMSG_RDY_IE_tag;

typedef union SRX_SMSG_RDY_IE_union_tag { /* Slow Serial Message Ready Interrupt Enable Register */
  vuint32_t R;
  struct {
    vuint32_t SRDY_IE0:1;              /* Slow Serial Message Ready Interrupt */
    vuint32_t SRDY_IE1:1;              /* Slow Serial Message Ready Interrupt */
    vuint32_t SRDY_IE2:1;              /* Slow Serial Message Ready Interrupt */
    vuint32_t SRDY_IE3:1;              /* Slow Serial Message Ready Interrupt */
    vuint32_t SRDY_IE4:1;              /* Slow Serial Message Ready Interrupt */
    vuint32_t SRDY_IE5:1;              /* Slow Serial Message Ready Interrupt */
    vuint32_t SRDY_IE6:1;              /* Slow Serial Message Ready Interrupt */
    vuint32_t SRDY_IE7:1;              /* Slow Serial Message Ready Interrupt */
    vuint32_t _unused_8:24;            /* Reserved */
  } B;
} SRX_SMSG_RDY_IE_tag;

typedef union SRX_CNL_CH_CLK_CTRL_union_tag { /* Channel '0' Clock Control Register */
  vuint32_t R;
  struct {
    vuint32_t PRSC:14;                 /* PRSC */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t COMP_EN:1;               /* COMP_EN */
    vuint32_t CM_PRSC:15;              /* CM_PRSC */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} SRX_CNL_CH_CLK_CTRL_tag;

typedef union SRX_CNL_CH_STATUS_union_tag { /* Channel '0' Status Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:16;            /* Reserved */
    vuint32_t NUM_EDGES_ERR:1;         /* NUM_EDGES_ERR */
    vuint32_t FMSG_CRC_ERR:1;          /* FMSG_CRC_ERR */
    vuint32_t SMSG_CRC_ERR:1;          /* SMSG_CRC_ERR */
    vuint32_t NIB_VAL_ERR_HIGH:1;      /* Nibble Value High Error */
    vuint32_t CAL_DIAG_ERR:1;          /* CAL_DIAG_ERR */
    vuint32_t CAL_LEN_ERR:1;           /* CAL_LEN_ERR */
    vuint32_t PP_DIAG_ERR:1;           /* PP_DIAG_ERR */
    vuint32_t NIB_VAL_ERR_LOW:1;       /* Nibble Value Low Error */
    vuint32_t FMSG_OFLW:1;             /* FMSG_OFLW */
    vuint32_t SMSG_OFLW:1;             /* SMSG_OFLW */
    vuint32_t CAL_20_25:1;             /* CAL_20_25 */
    vuint32_t CAL_RESYNC:1;            /* CAL_RESYNC */
    vuint32_t _unused_28:3;            /* Reserved */
    vuint32_t BUS_IDLE:1;              /* BUS_IDLE */
  } B;
} SRX_CNL_CH_STATUS_tag;

typedef union SRX_CNL_CH_CONFIG_union_tag { /* Channel '0' Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t FIL_CNT:8;               /* FIL_CNT */
    vuint32_t SUCC_CAL_CHK:1;          /* SUCC_CAL_CHK */
    vuint32_t PAUSE_EN:1;              /* PAUSE_EN */
    vuint32_t SCRC_TYPE:1;             /* SCRC_TYPE */
    vuint32_t FCRC_SC_EN:1;            /* FCRC_SC_EN */
    vuint32_t FCRC_TYPE:1;             /* FCRC_TYPE */
    vuint32_t PP_CHKSEL:1;             /* PP_CHKSEL */
    vuint32_t CAL_RNG:1;               /* CAL_RNG */
    vuint32_t DCHNG_INT:1;             /* DCHNG_INT */
    vuint32_t IE_NUM_EDGES_ERR:1;      /* IE_NUM_EDGES_ERR */
    vuint32_t IE_FMSG_CRC_ERR:1;       /* IE_FMSG_CRC_ERR */
    vuint32_t IE_SMSG_CRC_ERR:1;       /* IE_SMSG_CRC_ERR */
    vuint32_t IE_NIB_VAL_ERR:1;        /* IE_NIB_VAL_ERR */
    vuint32_t IE_CAL_DIAG_ERR:1;       /* IE_CAL_DIAG_ERR */
    vuint32_t IE_CAL_LEN_ERR:1;        /* IE_CAL_LEN_ERR */
    vuint32_t IE_PP_DIAG_ERR:1;        /* IE_PP_DIAG_ERR */
    vuint32_t FCRC_CHK_OFF:1;          /* FCRC_CHK_OFF */
    vuint32_t IE_FMSG_OFLW:1;          /* IE_FMSG_OFLW */
    vuint32_t IE_SMSG_OFLW:1;          /* IE_SMSG_OFLW */
    vuint32_t IE_CAL_20_25:1;          /* IE_CAL_20_25 */
    vuint32_t IE_CAL_RESYNC:1;         /* IE_CAL_RESYNC */
    vuint32_t BUS_IDLE_CNT:4;          /* BUS_IDLE_CNT */
  } B;
} SRX_CNL_CH_CONFIG_tag;

typedef union SRX_CNL_CH_FMSG_DATA_union_tag { /* Channel '0' Fast Message Data Read Register */
  vuint32_t R;
  struct {
    vuint32_t DNIB8:4;                 /* DNIB8 */
    vuint32_t DNIB7:4;                 /* DNIB7 */
    vuint32_t DNIB6:4;                 /* DNIB6 */
    vuint32_t DNIB5:4;                 /* DNIB5 */
    vuint32_t DNIB4:4;                 /* DNIB4 */
    vuint32_t DNIB3:4;                 /* DNIB3 */
    vuint32_t DNIB2:4;                 /* DNIB2 */
    vuint32_t DNIB1:4;                 /* DNIB1 */
  } B;
} SRX_CNL_CH_FMSG_DATA_tag;

typedef union SRX_CNL_CH_FMSG_CRC_union_tag { /* Channel '0' Fast Message CRC Read Register */
  vuint32_t R;
  struct {
    vuint32_t SCNIB:4;                 /* Status And Communication Nibble of message */
    vuint32_t CHNUM:4;                 /* Channel Number */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t CRC4B:4;                 /* CRC4b */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} SRX_CNL_CH_FMSG_CRC_tag;

typedef union SRX_CNL_CH_FMSG_TS_union_tag { /* Channel '0' Fast Message Time Stamp Read Register */
  vuint32_t R;
  struct {
    vuint32_t TS:32;                   /* TS */
  } B;
} SRX_CNL_CH_FMSG_TS_tag;

typedef union SRX_CNL_CH_SMSG_BIT3_union_tag { /* Channel '0' Serial Message Read Register (Bit 3) */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t ID3_0_DATA15_12:4;       /* ID3_0_DATA15_12 */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t ID7_4_ID3_0:4;           /* ID7_4_ID3_0 */
    vuint32_t CFG:1;                   /* CFG */
    vuint32_t _unused_11:16;           /* Reserved */
    vuint32_t TYPE:1;                  /* TYPE */
    vuint32_t CHNUM:4;                 /* Channel Number */
  } B;
} SRX_CNL_CH_SMSG_BIT3_tag;

typedef union SRX_CNL_CH_SMSG_BIT2_union_tag { /* Channel '0' Serial Message Read Register (Bit 2) */
  vuint32_t R;
  struct {
    vuint32_t DATA:12;                 /* DATA */
    vuint32_t _unused_12:4;
    vuint32_t SMCRC:6;                 /* SMCRC */
    vuint32_t _unused_22:10;
  } B;
} SRX_CNL_CH_SMSG_BIT2_tag;

typedef union SRX_CNL_CH_SMSG_TS_union_tag { /* Channel '0' Serial Message Time Stamp Read */
  vuint32_t R;
  struct {
    vuint32_t TS:32;                   /* TS */
  } B;
} SRX_CNL_CH_SMSG_TS_tag;

typedef union SRX_CNL_CH_F_DMA_RD_union_tag { /* Channel '0' DMA Fast Message Data Read */
  vuint32_t R;
  struct {
    vuint32_t DNIB8:4;                 /* Data Nibble 8 */
    vuint32_t DNIB7:4;                 /* Data Nibble 7 */
    vuint32_t DNIB6:4;                 /* Data Nibble 6 */
    vuint32_t DNIB5:4;                 /* Data Nibble 5 */
    vuint32_t DNIB4:4;                 /* Data Nibble 4 */
    vuint32_t DNIB3:4;                 /* Data Nibble 3 */
    vuint32_t DNIB2:4;                 /* Data Nibble 2 */
    vuint32_t DNIB1:4;                 /* Data Nibble 1 */
  } B;
} SRX_CNL_CH_F_DMA_RD_tag;

typedef union SRX_CNL_CH_F_DMA_CRC_union_tag { /* Channel '0' DMA Fast Message CRC Read */
  vuint32_t R;
  struct {
    vuint32_t SCNIB:4;                 /* Status And Communication Nibble */
    vuint32_t CHNUM:4;                 /* Channel Number */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t CRC4B:4;                 /* CRC Value */
    vuint32_t _unused_20:12;           /* Reserved */
  } B;
} SRX_CNL_CH_F_DMA_CRC_tag;

typedef union SRX_CNL_CH_F_DMA_TS_union_tag { /* Channel '0' DMA Fast Message Time Stamp Read */
  vuint32_t R;
  struct {
    vuint32_t TS:32;                   /* Time Stamp Of Received Message */
  } B;
} SRX_CNL_CH_F_DMA_TS_tag;

typedef union SRX_CNL_CH_S_DMA_RD_union_tag { /* Channel '0' DMA Slow Serial Message Bit3 Read */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t ID3_0_DATA15_12:4;       /* ID or Data Field */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t ID7_4_ID3_0:4;           /* ID Field */
    vuint32_t CFG:1;                   /* Configuration */
    vuint32_t _unused_11:16;           /* Reserved */
    vuint32_t TYPE:1;                  /* Serial Message Type */
    vuint32_t CHNUM:4;                 /* Channel Number */
  } B;
} SRX_CNL_CH_S_DMA_RD_tag;

typedef union SRX_CNL_CH_S_DMA_CRC_union_tag { /* Channel '0' DMA Slow Serial Message Bit2 Read */
  vuint32_t R;
  struct {
    vuint32_t DATA:12;                 /* Data on Slow Channel */
    vuint32_t _unused_12:4;
    vuint32_t SMCRC:6;                 /* Short Message CRC */
    vuint32_t _unused_22:10;
  } B;
} SRX_CNL_CH_S_DMA_CRC_tag;

typedef union SRX_CNL_CH_S_DMA_TS_union_tag { /* Channel '0' DMA Slow Serial Message Time Stamp Read */
  vuint32_t R;
  struct {
    vuint32_t TS:32;                   /* Time Stamp Of Received Message */
  } B;
} SRX_CNL_CH_S_DMA_TS_tag;

typedef union SRX_CNL_CHNL_TIMESTAMP_union_tag { /* Channel '0' Time Stamp */
  vuint32_t R;
  struct {
    vuint32_t CHN_TS:32;               /* Time Stamp Counter */
  } B;
} SRX_CNL_CHNL_TIMESTAMP_tag;

typedef union SRX_CNL_CHNL_COUNTER_union_tag { /* Channel '0' Counter */
  vuint32_t R;
  struct {
    vuint32_t EDGE_CNT:8;              /* Edge Counter */
    vuint32_t _unused_8:7;
    vuint32_t CLR_EDGE_CNT:1;          /* Clear Edge Counter */
    vuint32_t FRAME_CNT:8;             /* Frame Counter */
    vuint32_t _unused_24:7;
    vuint32_t CLR_FRAME_CNT:1;         /* Clear Frame Counter */
  } B;
} SRX_CNL_CHNL_COUNTER_tag;

typedef struct SRX_CNL_struct_tag {
  SRX_CNL_CH_CLK_CTRL_tag CH_CLK_CTRL; /* Channel '0' Clock Control Register */
  SRX_CNL_CH_STATUS_tag CH_STATUS;     /* Channel '0' Status Register */
  SRX_CNL_CH_CONFIG_tag CH_CONFIG;     /* Channel '0' Configuration Register */
  SRX_CNL_CH_FMSG_DATA_tag CH_FMSG_DATA; /* Channel '0' Fast Message Data Read Register */
  SRX_CNL_CH_FMSG_CRC_tag CH_FMSG_CRC; /* Channel '0' Fast Message CRC Read Register */
  SRX_CNL_CH_FMSG_TS_tag CH_FMSG_TS;   /* Channel '0' Fast Message Time Stamp Read Register */
  SRX_CNL_CH_SMSG_BIT3_tag CH_SMSG_BIT3; /* Channel '0' Serial Message Read Register (Bit 3) */
  SRX_CNL_CH_SMSG_BIT2_tag CH_SMSG_BIT2; /* Channel '0' Serial Message Read Register (Bit 2) */
  SRX_CNL_CH_SMSG_TS_tag CH_SMSG_TS;   /* Channel '0' Serial Message Time Stamp Read */
  SRX_CNL_CH_F_DMA_RD_tag CH_F_DMA_RD; /* Channel '0' DMA Fast Message Data Read */
  SRX_CNL_CH_F_DMA_CRC_tag CH_F_DMA_CRC; /* Channel '0' DMA Fast Message CRC Read */
  SRX_CNL_CH_F_DMA_TS_tag CH_F_DMA_TS; /* Channel '0' DMA Fast Message Time Stamp Read */
  SRX_CNL_CH_S_DMA_RD_tag CH_S_DMA_RD; /* Channel '0' DMA Slow Serial Message Bit3 Read */
  SRX_CNL_CH_S_DMA_CRC_tag CH_S_DMA_CRC; /* Channel '0' DMA Slow Serial Message Bit2 Read */
  SRX_CNL_CH_S_DMA_TS_tag CH_S_DMA_TS; /* Channel '0' DMA Slow Serial Message Time Stamp Read */
  SRX_CNL_CHNL_TIMESTAMP_tag CHNL_TIMESTAMP; /* Channel '0' Time Stamp */
  SRX_CNL_CHNL_COUNTER_tag CHNL_COUNTER; /* Channel '0' Counter */
} SRX_CNL_tag;

struct SRX_tag {
  SRX_GBL_CTRL_tag GBL_CTRL;           /* Global Control Register */
  SRX_CHNL_EN_tag CHNL_EN;             /* Channel Enable Register */
  SRX_SLOW_MSG_UVF_tag SLOW_MSG_UVF;   /* Slow Channel Status */
  SRX_FMSG_RDY_tag FMSG_RDY;           /* Fast Message Ready Status Register */
  SRX_SMSG_RDY_tag SMSG_RDY;           /* Slow Serial Message Ready Status Register */
  SRX_CHNL_FIFO_EN_tag CHNL_FIFO_EN;   /* Channel FIFO Enable */
  SRX_DATA_CTRL1_tag DATA_CTRL1;       /* Data Control Register 1 */
  uint8_t SRX_reserved0[4];
  SRX_CHNL_OVF_UVF_tag CHNL_OVF_UVF;   /* Fast Channel Status */
  SRX_FDMA_CTRL_tag FDMA_CTRL;         /* Fast Message DMA Control Register */
  SRX_SDMA_CTRL_tag SDMA_CTRL;         /* Slow Serial Message DMA Control Register */
  SRX_CHNL_NB_REORD1_tag CHNL_NB_REORD1; /* Channel Nibble Reorder 1 */
  uint8_t SRX_reserved1[4];
  SRX_FMSG_RDY_IE_tag FMSG_RDY_IE;     /* Fast Message Ready Interrupt Control Register */
  SRX_SMSG_RDY_IE_tag SMSG_RDY_IE;     /* Slow Serial Message Ready Interrupt Enable Register */
  uint8_t SRX_reserved2[36];
  SRX_CNL_tag CNL[8];
};


/* ============================================================================
   =============================== Module: STM ================================
   ============================================================================ */

typedef union STM_CR_union_tag {       /* Control */
  vuint32_t R;
  struct {
    vuint32_t TEN:1;                   /* Timer Enable */
    vuint32_t FRZ:1;                   /* Freeze */
    vuint32_t _unused_2:6;
    vuint32_t CPS:8;                   /* Counter Prescaler */
    vuint32_t _unused_16:16;
  } B;
} STM_CR_tag;

typedef union STM_CNT_union_tag {      /* Count */
  vuint32_t R;
  struct {
    vuint32_t CNT:32;                  /* Timer Count */
  } B;
} STM_CNT_tag;

typedef union STM_CHANNEL_CCR_union_tag { /* Channel Control */
  vuint32_t R;
  struct {
    vuint32_t CEN:1;                   /* Channel Enable */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} STM_CHANNEL_CCR_tag;

typedef union STM_CHANNEL_CIR_union_tag { /* Channel Interrupt */
  vuint32_t R;
  struct {
    vuint32_t CIF:1;                   /* Channel Interrupt Flag */
    vuint32_t _unused_1:31;
  } B;
} STM_CHANNEL_CIR_tag;

typedef union STM_CHANNEL_CMP_union_tag { /* Channel Compare */
  vuint32_t R;
  struct {
    vuint32_t CMP:32;                  /* Channel Compare */
  } B;
} STM_CHANNEL_CMP_tag;

typedef struct STM_CHANNEL_struct_tag {
  STM_CHANNEL_CCR_tag CCR;             /* Channel Control */
  STM_CHANNEL_CIR_tag CIR;             /* Channel Interrupt */
  STM_CHANNEL_CMP_tag CMP;             /* Channel Compare */
  uint8_t CHANNEL_reserved0[4];
} STM_CHANNEL_tag;

struct STM_tag {
  STM_CR_tag CR;                       /* Control */
  STM_CNT_tag CNT;                     /* Count */
  uint8_t STM_reserved0[8];
  STM_CHANNEL_tag CHANNEL[4];
};


/* ============================================================================
   =============================== Module: SWT ================================
   ============================================================================ */

typedef union SWT_CR_union_tag {       /* Control */
  vuint32_t R;
  struct {
    vuint32_t WEN:1;                   /* Watchdog Enable */
    vuint32_t FRZ:1;                   /* Debug Mode Control */
    vuint32_t STP:1;                   /* Stop Mode Control */
    vuint32_t _unused_3:1;             /* Reserved */
    vuint32_t SLK:1;                   /* Soft Lock */
    vuint32_t HLK:1;                   /* Hard Lock */
    vuint32_t ITR:1;                   /* Interrupt Then Reset Request */
    vuint32_t WND:1;                   /* Window Mode */
    vuint32_t RIA:1;                   /* Reset on Invalid Access */
    vuint32_t SMD:2;                   /* Service Mode */
    vuint32_t _unused_11:13;           /* Reserved */
    vuint32_t MAP7:1;                  /* Master Access Protection For Master 7 */
    vuint32_t MAP6:1;                  /* Master Access Protection For Master 6 */
    vuint32_t MAP5:1;                  /* Master Access Protection For Master 5 */
    vuint32_t MAP4:1;                  /* Master Access Protection For Master 4 */
    vuint32_t MAP3:1;                  /* Master Access Protection For Master 3 */
    vuint32_t MAP2:1;                  /* Master Access Protection For Master 2 */
    vuint32_t MAP1:1;                  /* Master Access Protection For Master 1 */
    vuint32_t MAP0:1;                  /* Master Access Protection For Master 0 */
  } B;
} SWT_CR_tag;

typedef union SWT_IR_union_tag {       /* Interrupt */
  vuint32_t R;
  struct {
    vuint32_t TIF:1;                   /* Timeout Interrupt Flag */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} SWT_IR_tag;

typedef union SWT_TO_union_tag {       /* Timeout */
  vuint32_t R;
  struct {
    vuint32_t WTO:32;                  /* Watchdog Timeout */
  } B;
} SWT_TO_tag;

typedef union SWT_WN_union_tag {       /* Window */
  vuint32_t R;
  struct {
    vuint32_t WST:32;                  /* Window Start Value */
  } B;
} SWT_WN_tag;

typedef union SWT_SR_union_tag {       /* Service */
  vuint32_t R;
  struct {
    vuint32_t WSC:16;                  /* Watchdog Service Code */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} SWT_SR_tag;

typedef union SWT_CO_union_tag {       /* Counter Output */
  vuint32_t R;
  struct {
    vuint32_t CNT:32;                  /* Watchdog Count */
  } B;
} SWT_CO_tag;

typedef union SWT_SK_union_tag {       /* Service Key */
  vuint32_t R;
  struct {
    vuint32_t SK:16;                   /* Service Key */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} SWT_SK_tag;

typedef union SWT_RRR_union_tag {      /* Event Request */
  vuint32_t R;
  struct {
    vuint32_t RRF:1;                   /* Reset Request Flag */
    vuint32_t _unused_1:31;            /* Reserved */
  } B;
} SWT_RRR_tag;

struct SWT_tag {
  SWT_CR_tag CR;                       /* Control */
  SWT_IR_tag IR;                       /* Interrupt */
  SWT_TO_tag TO;                       /* Timeout */
  SWT_WN_tag WN;                       /* Window */
  SWT_SR_tag SR;                       /* Service */
  SWT_CO_tag CO;                       /* Counter Output */
  SWT_SK_tag SK;                       /* Service Key */
  SWT_RRR_tag RRR;                     /* Event Request */
};


/* ============================================================================
   =============================== Module: SW_ETH_MAC_PORT0 ===================
   ============================================================================ */

typedef union SW_ETH_MAC_PORT0_PM0_COMMAND_CONFIG_union_tag { /* Port MAC 0 Command and Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t TX_EN:1;                 /* MAC transmit path enable */
    vuint32_t RX_EN:1;                 /* MAC receive path enable */
    vuint32_t _unused_2:5;
    vuint32_t PAUSE_FWD:1;             /* Terminate/forward received PAUSE frames */
    vuint32_t PAUSE_IGN:1;             /* Ignore PAUSE frame quanta */
    vuint32_t TX_ADDR_INS:1;           /* Transmit source MAC address insertion */
    vuint32_t LOOP_ENA:1;              /* Loopback enable */
    vuint32_t _unused_11:2;
    vuint32_t CNT_FRM_EN:1;            /* Control frame reception enable */
    vuint32_t _unused_14:1;
    vuint32_t TXP:1;                   /* Enable padding of frames in transmit direction (1, default). */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:3;
    vuint32_t _unused_20:2;
    vuint32_t TX_FLUSH:1;              /* Tx flush */
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t SWR:1;                   /* Software Reset. Self clearing bit. */
    vuint32_t _unused_27:3;
    vuint32_t TS_MODE:1;               /* Transmit timestamp mode */
    vuint32_t MG:1;                    /* Magic Packet detection enable. */
  } B;
} SW_ETH_MAC_PORT0_PM0_COMMAND_CONFIG_tag;

typedef union SW_ETH_MAC_PORT0_PM0_MAC_ADDR_0_union_tag { /* Port MAC 0 MAC Address Register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR_0:32;           /* MAC address 0 */
  } B;
} SW_ETH_MAC_PORT0_PM0_MAC_ADDR_0_tag;

typedef union SW_ETH_MAC_PORT0_PM0_MAC_ADDR_1_union_tag { /* Port MAC 0 MAC Address Register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR_1:16;           /* MAC address 1 */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PM0_MAC_ADDR_1_tag;

typedef union SW_ETH_MAC_PORT0_PM0_MAXFRM_union_tag { /* Port MAC 0 Maximum Frame Length Register */
  vuint32_t R;
  struct {
    vuint32_t MAXFRM:16;               /* Maximum supported received frame length. */
    vuint32_t TX_MTU:16;               /* Maximum transmit frame length */
  } B;
} SW_ETH_MAC_PORT0_PM0_MAXFRM_tag;

typedef union SW_ETH_MAC_PORT0_PM0_IEVENT_union_tag { /* Port MAC 0 Interrupt Event Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t _unused_4:1;
    vuint32_t TX_EMPTY:1;              /* Transmit FIFO empty event */
    vuint32_t RX_EMPTY:1;              /* Receive idle event */
    vuint32_t _unused_7:3;
    vuint32_t TX_OVFL:1;               /* Transmit FIFO overflow event. */
    vuint32_t TX_UNFL:1;               /* Transmit FIFO underflow event. */
    vuint32_t RX_OVFL:1;               /* Receive FIFO overflow event. */
    vuint32_t _unused_13:1;
    vuint32_t MGI:1;                   /* Magic packet detection indication event */
    vuint32_t _unused_15:12;
    vuint32_t _unused_27:2;
    vuint32_t _unused_29:3;
  } B;
} SW_ETH_MAC_PORT0_PM0_IEVENT_tag;

typedef union SW_ETH_MAC_PORT0_PM0_IMASK_union_tag { /* Port MAC 0 Interrupt Mask Register(INT_MASK) */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:4;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:6;
    vuint32_t MGI:1;                   /* Magic packet detection indication event mask. */
    vuint32_t _unused_15:9;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:5;
  } B;
} SW_ETH_MAC_PORT0_PM0_IMASK_tag;

typedef union SW_ETH_MAC_PORT0_PM0_PAUSE_QUANTA_union_tag { /* Port MAC 0 Pause Quanta Register */
  vuint32_t R;
  struct {
    vuint32_t PQNT:16;                 /* Value to be used for the quanta value when XOFF is triggered. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PM0_PAUSE_QUANTA_tag;

typedef union SW_ETH_MAC_PORT0_PM0_PAUSE_THRESH_union_tag { /* Port MAC 0 Pause Quanta Threshold Register */
  vuint32_t R;
  struct {
    vuint32_t QTH:16;                  /* Quanta threshold. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PM0_PAUSE_THRESH_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RX_PAUSE_STATUS_union_tag { /* Port MAC 0 Receive Pause Status Register */
  vuint32_t R;
  struct {
    vuint32_t PSTAT:1;                 /* Pause status. */
    vuint32_t _unused_1:31;
  } B;
} SW_ETH_MAC_PORT0_PM0_RX_PAUSE_STATUS_tag;

typedef union SW_ETH_MAC_PORT0_PM0_SINGLE_STEP_union_tag { /* Port MAC 0 IEEE1588 Single-Step Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t OFFSET:8;
    vuint32_t _unused_15:16;
    vuint32_t EN:1;                    /* IEEE-1588 Single-Step enable. */
  } B;
} SW_ETH_MAC_PORT0_PM0_SINGLE_STEP_tag;

typedef union SW_ETH_MAC_PORT0_PM0_HD_BACKOFF_ENTROPY_union_tag { /* Port MAC 0 half-duplex backoff entropy register */
  vuint32_t R;
  struct {
    vuint32_t HD_BACKOFF_ENTROPY:10;   /* Half duplex backoff entropy */
    vuint32_t _unused_10:21;
    vuint32_t SW_ENTROPY_VALID:1;      /* Software programmable entropy valid */
  } B;
} SW_ETH_MAC_PORT0_PM0_HD_BACKOFF_ENTROPY_tag;

typedef union SW_ETH_MAC_PORT0_PM0_STATN_CONFIG_union_tag { /* Port MAC 0 Statistics Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SAT:1;
    vuint32_t COD:1;
    vuint32_t CLR:1;
    vuint32_t _unused_3:29;
  } B;
} SW_ETH_MAC_PORT0_PM0_STATN_CONFIG_tag;

typedef union SW_ETH_MAC_PORT0_PM0_REOCTN_union_tag { /* Port MAC 0 Receive Ethernet Octets Counter(etherStatsOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t REOCTN:64;               /* Incremented for each octet received in both good and bad packets. */
  } B;
} SW_ETH_MAC_PORT0_PM0_REOCTN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_ROCTN_union_tag { /* Port MAC 0 Receive Octets Counter(iflnOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t ROCTN:64;                /* Incremented for each octet received except preamble (that is, Header, Payload, Pad and FCS) for all valid frames and valid PAUSE frames received. */
  } B;
} SW_ETH_MAC_PORT0_PM0_ROCTN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RXPFN_union_tag { /* Port MAC 0 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  vuint64_t R;
  struct {
    vuint64_t RXPFN:64;                /* Incremented for each valid PAUSE frame received . */
  } B;
} SW_ETH_MAC_PORT0_PM0_RXPFN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RFRMN_union_tag { /* Port MAC 0 Receive Frame Counter Register(aFramesReceivedOKn) */
  vuint64_t R;
  struct {
    vuint64_t RFRMN:64;                /* Incremented for each frame received without error, including PAUSE frames. */
  } B;
} SW_ETH_MAC_PORT0_PM0_RFRMN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RFCSN_union_tag { /* Port MAC 0 Receive Frame Check Sequence Error Counter Register() */
  vuint64_t R;
  struct {
    vuint64_t RFCSN:64;                /* Incremented for each frame received with a CRC-32 error but the frame is otherwise of correct length. */
  } B;
} SW_ETH_MAC_PORT0_PM0_RFCSN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RVLANN_union_tag { /* Port MAC 0 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
  vuint64_t R;
  struct {
    vuint64_t RVLANN:64;               /* Incremented for each valid VLAN tagged frame received with ethertype 0x8100 */
  } B;
} SW_ETH_MAC_PORT0_PM0_RVLANN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RERRN_union_tag { /* Port MAC 0 Receive Frame Error Counter Register(ifInErrorsn) */
  vuint64_t R;
  struct {
    vuint64_t RERRN:64;                /* Incremented for each frame received with an error (except for undersized/fragment frame): */
  } B;
} SW_ETH_MAC_PORT0_PM0_RERRN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RUCAN_union_tag { /* Port MAC 0 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RUCAN:64;                /* Incremented for each valid frame received (on the receive FIFO interface) in which bit 0 of the destination address was 0 . */
  } B;
} SW_ETH_MAC_PORT0_PM0_RUCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RMCAN_union_tag { /* Port MAC 0 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RMCAN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_RMCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RBCAN_union_tag { /* Port MAC 0 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RBCAN:64;                /* Incremented for each valid frame received (on the receive FIFO interface) in which all bits of the destination address were 1 . */
  } B;
} SW_ETH_MAC_PORT0_PM0_RBCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RDRPN_union_tag { /* Port MAC 0 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
  vuint64_t R;
  struct {
    vuint64_t RDRPN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_RDRPN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RPKTN_union_tag { /* Port MAC 0 Receive Packets Counter Register(etherStatsPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RPKTN:64;                /* Incremented for each good or bad packet received. */
  } B;
} SW_ETH_MAC_PORT0_PM0_RPKTN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RUNDN_union_tag { /* Port MAC 0 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t RUNDN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_RUNDN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_R64N_union_tag { /* Port MAC 0 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R64N:64;                 /* Incremented for each 64-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_R64N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_R127N_union_tag { /* Port MAC 0 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R127N:64;                /* Incremented for each 65- to 127-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_R127N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_R255N_union_tag { /* Port MAC 0 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R255N:64;                /* Incremented for each 128- to 255-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_R255N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_R511N_union_tag { /* Port MAC 0 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R511N:64;                /* Incremented for each 256- to 511-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_R511N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_R1023N_union_tag { /* Port MAC 0 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1023N:64;               /* Incremented for each 512- to 1023-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_R1023N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_R1522N_union_tag { /* Port MAC 0 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1522N:64;               /* Incremented for each 1024- to 1522-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_R1522N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_R1523XN_union_tag { /* Port MAC 0 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1523XN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_R1523XN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_ROVRN_union_tag { /* Port MAC 0 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t ROVRN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_ROVRN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RJBRN_union_tag { /* Port MAC 0 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
  vuint64_t R;
  struct {
    vuint64_t RJBRN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_RJBRN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RFRGN_union_tag { /* Port MAC 0 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
  vuint64_t R;
  struct {
    vuint64_t RFRGN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_RFRGN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RCNPN_union_tag { /* Port MAC 0 Receive Control Packet Counter Register */
  vuint64_t R;
  struct {
    vuint64_t RCNPN:64;                /* Incremented for each valid control packet (type 0x8808) but not for PAUSE packets */
  } B;
} SW_ETH_MAC_PORT0_PM0_RCNPN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_RDRNTPN_union_tag { /* Port MAC 0 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
  vuint64_t R;
  struct {
    vuint64_t RDRNTPN:64;              /* Incremented for each fully dropped packet (not truncated) due to internal errors of the MAC client. Occurs when a receive FIFO overflows. */
  } B;
} SW_ETH_MAC_PORT0_PM0_RDRNTPN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TEOCTN_union_tag { /* Port MAC 0 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t TEOCTN:64;               /* Incremented for each octet transmitted in both good and bad packets. */
  } B;
} SW_ETH_MAC_PORT0_PM0_TEOCTN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TOCTN_union_tag { /* Port MAC 0 Transmit Octets Counter Register(ifOutOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t TOCTN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_TOCTN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TXPFN_union_tag { /* Port MAC 0 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  vuint64_t R;
  struct {
    vuint64_t TXPFN:64;                /* Incremented for each valid PAUSE frame transmitted . Note: Pause frames forwarded to the MAC from MAC Client are not counted by TXPFn. */
  } B;
} SW_ETH_MAC_PORT0_PM0_TXPFN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TFRMN_union_tag { /* Port MAC 0 Transmit Frame Counter Register(aFramesTransmittedOKn) */
  vuint64_t R;
  struct {
    vuint64_t TFRMN:64;                /* Incremented for each frame transmitted without error, including PAUSE frames. */
  } B;
} SW_ETH_MAC_PORT0_PM0_TFRMN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TFCSN_union_tag { /* Port MAC 0 Transmit Frame Check Sequence Error Counter Register() */
  vuint64_t R;
  struct {
    vuint64_t TFCSN:64;                /* Incremented for each frame transmitted with a CRC-32 error except for underflows. */
  } B;
} SW_ETH_MAC_PORT0_PM0_TFCSN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TVLANN_union_tag { /* Port MAC 0 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
  vuint64_t R;
  struct {
    vuint64_t TVLANN:64;               /* Incremented for each valid VLAN tagged frame transmitted with ethertype 0x8100. */
  } B;
} SW_ETH_MAC_PORT0_PM0_TVLANN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TERRN_union_tag { /* Port MAC 0 Transmit Frame Error Counter Register(ifOutErrorsn) */
  vuint64_t R;
  struct {
    vuint64_t TERRN:64;                /* Transmit frame error count */
  } B;
} SW_ETH_MAC_PORT0_PM0_TERRN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TUCAN_union_tag { /* Port MAC 0 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TUCAN:64;                /* Incremented for each valid frame transmitted (to the FIFO interface) in which bit 0 of the destination address was 0. */
  } B;
} SW_ETH_MAC_PORT0_PM0_TUCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TMCAN_union_tag { /* Port MAC 0 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TMCAN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_TMCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TBCAN_union_tag { /* Port MAC 0 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TBCAN:64;                /* Incremented for each valid frame transmitted (to the FIFO interface) in which all bits of the destination address were 1 . */
  } B;
} SW_ETH_MAC_PORT0_PM0_TBCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TPKTN_union_tag { /* Port MAC 0 Transmit Packets Counter Register(etherStatsPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TPKTN:64;                /* Incremented for each good or bad packet transmitted. */
  } B;
} SW_ETH_MAC_PORT0_PM0_TPKTN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TUNDN_union_tag { /* Port MAC 0 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t TUNDN:64;                /* Incremented for each packet transmitted that was less than 64 octets long with a good CRC. */
  } B;
} SW_ETH_MAC_PORT0_PM0_TUNDN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_T64N_union_tag { /* Port MAC 0 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T64N:64;                 /* Incremented for each 64-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_T64N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_T127N_union_tag { /* Port MAC 0 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T127N:64;                /* Incremented for each 65 to 127-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_T127N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_T255N_union_tag { /* Port MAC 0 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T255N:64;                /* Incremented for each 128 to 255-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_T255N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_T511N_union_tag { /* Port MAC 0 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T511N:64;                /* Incremented for each 256 to 511-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_T511N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_T1023N_union_tag { /* Port MAC 0 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1023N:64;               /* Incremented for each 512 to 1023-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_T1023N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_T1522N_union_tag { /* Port MAC 0 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1522N:64;               /* Incremented for each 1024- to 1522-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM0_T1522N_tag;

typedef union SW_ETH_MAC_PORT0_PM0_T1523XN_union_tag { /* Port MAC 0 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1523XN:64;
  } B;
} SW_ETH_MAC_PORT0_PM0_T1523XN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TCNPN_union_tag { /* Port MAC 0 Transmit Control Packet Counter Register */
  vuint64_t R;
  struct {
    vuint64_t TCNPN:64;                /* Incremented for each valid control packet transmitted (type 0x8808) but not for PAUSE packets */
  } B;
} SW_ETH_MAC_PORT0_PM0_TCNPN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TDFRN_union_tag { /* Port MAC 0 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
  vuint64_t R;
  struct {
    vuint64_t TDFRN:64;                /* Increments for successful transmissions, without retransmits, that were deferred (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT0_PM0_TDFRN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TMCOLN_union_tag { /* Port MAC 0 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
  vuint64_t R;
  struct {
    vuint64_t TMCOLN:64;               /* Increments for successful transmission after more than one retransmission (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT0_PM0_TMCOLN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TSCOLN_union_tag { /* Port MAC 0 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
  vuint64_t R;
  struct {
    vuint64_t TSCOLN:64;               /* Increments for successful transmission after one retransmission (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT0_PM0_TSCOLN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TLCOLN_union_tag { /* Port MAC 0 Transmit Late Collision Counter(aLateCollisions) Register */
  vuint64_t R;
  struct {
    vuint64_t TLCOLN:64;               /* Late collision occurred. Frame corrupted / discarded (half-duplex only) */
  } B;
} SW_ETH_MAC_PORT0_PM0_TLCOLN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_TECOLN_union_tag { /* Port MAC 0 Transmit Excessive Collisions Counter Register */
  vuint64_t R;
  struct {
    vuint64_t TECOLN:64;               /* Excessive collisions occurred. Frame was discarded (half-duplex only) */
  } B;
} SW_ETH_MAC_PORT0_PM0_TECOLN_tag;

typedef union SW_ETH_MAC_PORT0_PM0_IF_MODE_union_tag { /* Port MAC 0 Interface Mode Control Register */
  vuint32_t R;
  struct {
    vuint32_t IFMODE:3;                /* Interface mode */
    vuint32_t REVMII:1;                /* Reverse Mode */
    vuint32_t M10:1;
    vuint32_t _unused_5:1;
    vuint32_t HD:1;                    /* Half-duplex */
    vuint32_t _unused_7:5;
    vuint32_t _unused_12:1;
    vuint32_t SSP:2;                   /* Set Speed */
    vuint32_t _unused_15:17;
  } B;
} SW_ETH_MAC_PORT0_PM0_IF_MODE_tag;

typedef union SW_ETH_MAC_PORT0_PM1_COMMAND_CONFIG_union_tag { /* Port MAC 1 Command and Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t TX_EN:1;                 /* MAC transmit path enable */
    vuint32_t RX_EN:1;                 /* MAC receive path enable */
    vuint32_t _unused_2:5;
    vuint32_t PAUSE_FWD:1;             /* Terminate/forward received PAUSE frames */
    vuint32_t PAUSE_IGN:1;             /* Ignore PAUSE frame quanta */
    vuint32_t TX_ADDR_INS:1;           /* Transmit source MAC address insertion */
    vuint32_t LOOP_ENA:1;              /* Loopback enable */
    vuint32_t _unused_11:2;
    vuint32_t CNT_FRM_EN:1;            /* Control frame reception enable */
    vuint32_t _unused_14:1;
    vuint32_t TXP:1;                   /* Enable padding of frames in transmit direction (1, default). */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:3;
    vuint32_t _unused_20:2;
    vuint32_t TX_FLUSH:1;              /* Tx flush */
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t SWR:1;                   /* Software Reset. Self clearing bit. */
    vuint32_t _unused_27:3;
    vuint32_t TS_MODE:1;               /* Transmit timestamp mode */
    vuint32_t MG:1;                    /* Magic Packet detection enable. */
  } B;
} SW_ETH_MAC_PORT0_PM1_COMMAND_CONFIG_tag;

typedef union SW_ETH_MAC_PORT0_PM1_MAC_ADDR_0_union_tag { /* Port MAC 1 MAC Address Register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR_0:32;           /* MAC address 0 */
  } B;
} SW_ETH_MAC_PORT0_PM1_MAC_ADDR_0_tag;

typedef union SW_ETH_MAC_PORT0_PM1_MAC_ADDR_1_union_tag { /* Port MAC 1 MAC Address Register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR_1:16;           /* MAC address 1 */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PM1_MAC_ADDR_1_tag;

typedef union SW_ETH_MAC_PORT0_PM1_MAXFRM_union_tag { /* Port MAC 1 Maximum Frame Length Register */
  vuint32_t R;
  struct {
    vuint32_t MAXFRM:16;               /* Maximum supported received frame length. */
    vuint32_t TX_MTU:16;               /* Maximum transmit frame length */
  } B;
} SW_ETH_MAC_PORT0_PM1_MAXFRM_tag;

typedef union SW_ETH_MAC_PORT0_PM1_IEVENT_union_tag { /* Port MAC 1 Interrupt Event Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t _unused_4:1;
    vuint32_t TX_EMPTY:1;              /* Transmit FIFO empty event */
    vuint32_t RX_EMPTY:1;              /* Receive idle event */
    vuint32_t _unused_7:3;
    vuint32_t TX_OVFL:1;               /* Transmit FIFO overflow event. */
    vuint32_t TX_UNFL:1;               /* Transmit FIFO underflow event. */
    vuint32_t RX_OVFL:1;               /* Receive FIFO overflow event. */
    vuint32_t _unused_13:1;
    vuint32_t MGI:1;                   /* Magic packet detection indication event */
    vuint32_t _unused_15:12;
    vuint32_t _unused_27:2;
    vuint32_t _unused_29:3;
  } B;
} SW_ETH_MAC_PORT0_PM1_IEVENT_tag;

typedef union SW_ETH_MAC_PORT0_PM1_IMASK_union_tag { /* Port MAC 1 Interrupt Mask Register(INT_MASK) */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:4;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:6;
    vuint32_t MGI:1;                   /* Magic packet detection indication event mask. */
    vuint32_t _unused_15:9;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:5;
  } B;
} SW_ETH_MAC_PORT0_PM1_IMASK_tag;

typedef union SW_ETH_MAC_PORT0_PM1_PAUSE_QUANTA_union_tag { /* Port MAC 1 Pause Quanta Register */
  vuint32_t R;
  struct {
    vuint32_t PQNT:16;                 /* Value to be used for the quanta value when XOFF is triggered. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PM1_PAUSE_QUANTA_tag;

typedef union SW_ETH_MAC_PORT0_PM1_PAUSE_THRESH_union_tag { /* Port MAC 1 Pause Quanta Threshold Register */
  vuint32_t R;
  struct {
    vuint32_t QTH:16;                  /* Quanta threshold. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PM1_PAUSE_THRESH_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RX_PAUSE_STATUS_union_tag { /* Port MAC 1 Receive Pause Status Register */
  vuint32_t R;
  struct {
    vuint32_t PSTAT:1;                 /* Pause status. */
    vuint32_t _unused_1:31;
  } B;
} SW_ETH_MAC_PORT0_PM1_RX_PAUSE_STATUS_tag;

typedef union SW_ETH_MAC_PORT0_PM1_SINGLE_STEP_union_tag { /* Port MAC 1 IEEE1588 Single-Step Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t OFFSET:8;
    vuint32_t _unused_15:16;
    vuint32_t EN:1;                    /* IEEE-1588 Single-Step enable. */
  } B;
} SW_ETH_MAC_PORT0_PM1_SINGLE_STEP_tag;

typedef union SW_ETH_MAC_PORT0_PM1_HD_BACKOFF_ENTROPY_union_tag { /* Port MAC 1 half-duplex backoff entropy register */
  vuint32_t R;
  struct {
    vuint32_t HD_BACKOFF_ENTROPY:10;   /* Half duplex backoff entropy */
    vuint32_t _unused_10:21;
    vuint32_t SW_ENTROPY_VALID:1;      /* Software programmable entropy valid */
  } B;
} SW_ETH_MAC_PORT0_PM1_HD_BACKOFF_ENTROPY_tag;

typedef union SW_ETH_MAC_PORT0_PM1_STATN_CONFIG_union_tag { /* Port MAC 1 Statistics Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SAT:1;
    vuint32_t COD:1;
    vuint32_t CLR:1;
    vuint32_t _unused_3:29;
  } B;
} SW_ETH_MAC_PORT0_PM1_STATN_CONFIG_tag;

typedef union SW_ETH_MAC_PORT0_PM1_REOCTN_union_tag { /* Port MAC 1 Receive Ethernet Octets Counter(etherStatsOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t REOCTN:64;               /* Incremented for each octet received in both good and bad packets. */
  } B;
} SW_ETH_MAC_PORT0_PM1_REOCTN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_ROCTN_union_tag { /* Port MAC 1 Receive Octets Counter(iflnOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t ROCTN:64;                /* Incremented for each octet received except preamble (that is, Header, Payload, Pad and FCS) for all valid frames and valid PAUSE frames received. */
  } B;
} SW_ETH_MAC_PORT0_PM1_ROCTN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RXPFN_union_tag { /* Port MAC 1 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  vuint64_t R;
  struct {
    vuint64_t RXPFN:64;                /* Incremented for each valid PAUSE frame received . */
  } B;
} SW_ETH_MAC_PORT0_PM1_RXPFN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RFRMN_union_tag { /* Port MAC 1 Receive Frame Counter Register(aFramesReceivedOKn) */
  vuint64_t R;
  struct {
    vuint64_t RFRMN:64;                /* Incremented for each frame received without error, including PAUSE frames. */
  } B;
} SW_ETH_MAC_PORT0_PM1_RFRMN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RFCSN_union_tag { /* Port MAC 1 Receive Frame Check Sequence Error Counter Register() */
  vuint64_t R;
  struct {
    vuint64_t RFCSN:64;                /* Incremented for each frame received with a CRC-32 error but the frame is otherwise of correct length. */
  } B;
} SW_ETH_MAC_PORT0_PM1_RFCSN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RVLANN_union_tag { /* Port MAC 1 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
  vuint64_t R;
  struct {
    vuint64_t RVLANN:64;               /* Incremented for each valid VLAN tagged frame received with ethertype 0x8100 */
  } B;
} SW_ETH_MAC_PORT0_PM1_RVLANN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RERRN_union_tag { /* Port MAC 1 Receive Frame Error Counter Register(ifInErrorsn) */
  vuint64_t R;
  struct {
    vuint64_t RERRN:64;                /* Incremented for each frame received with an error (except for undersized/fragment frame): */
  } B;
} SW_ETH_MAC_PORT0_PM1_RERRN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RUCAN_union_tag { /* Port MAC 1 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RUCAN:64;                /* Incremented for each valid frame received (on the receive FIFO interface) in which bit 0 of the destination address was 0 . */
  } B;
} SW_ETH_MAC_PORT0_PM1_RUCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RMCAN_union_tag { /* Port MAC 1 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RMCAN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_RMCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RBCAN_union_tag { /* Port MAC 1 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RBCAN:64;                /* Incremented for each valid frame received (on the receive FIFO interface) in which all bits of the destination address were 1 . */
  } B;
} SW_ETH_MAC_PORT0_PM1_RBCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RDRPN_union_tag { /* Port MAC 1 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
  vuint64_t R;
  struct {
    vuint64_t RDRPN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_RDRPN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RPKTN_union_tag { /* Port MAC 1 Receive Packets Counter Register(etherStatsPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RPKTN:64;                /* Incremented for each good or bad packet received. */
  } B;
} SW_ETH_MAC_PORT0_PM1_RPKTN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RUNDN_union_tag { /* Port MAC 1 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t RUNDN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_RUNDN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_R64N_union_tag { /* Port MAC 1 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R64N:64;                 /* Incremented for each 64-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_R64N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_R127N_union_tag { /* Port MAC 1 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R127N:64;                /* Incremented for each 65- to 127-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_R127N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_R255N_union_tag { /* Port MAC 1 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R255N:64;                /* Incremented for each 128- to 255-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_R255N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_R511N_union_tag { /* Port MAC 1 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R511N:64;                /* Incremented for each 256- to 511-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_R511N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_R1023N_union_tag { /* Port MAC 1 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1023N:64;               /* Incremented for each 512- to 1023-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_R1023N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_R1522N_union_tag { /* Port MAC 1 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1522N:64;               /* Incremented for each 1024- to 1522-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_R1522N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_R1523XN_union_tag { /* Port MAC 1 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1523XN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_R1523XN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_ROVRN_union_tag { /* Port MAC 1 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t ROVRN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_ROVRN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RJBRN_union_tag { /* Port MAC 1 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
  vuint64_t R;
  struct {
    vuint64_t RJBRN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_RJBRN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RFRGN_union_tag { /* Port MAC 1 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
  vuint64_t R;
  struct {
    vuint64_t RFRGN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_RFRGN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RCNPN_union_tag { /* Port MAC 1 Receive Control Packet Counter Register */
  vuint64_t R;
  struct {
    vuint64_t RCNPN:64;                /* Incremented for each valid control packet (type 0x8808) but not for PAUSE packets */
  } B;
} SW_ETH_MAC_PORT0_PM1_RCNPN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_RDRNTPN_union_tag { /* Port MAC 1 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
  vuint64_t R;
  struct {
    vuint64_t RDRNTPN:64;              /* Incremented for each fully dropped packet (not truncated) due to internal errors of the MAC client. Occurs when a receive FIFO overflows. */
  } B;
} SW_ETH_MAC_PORT0_PM1_RDRNTPN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TEOCTN_union_tag { /* Port MAC 1 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t TEOCTN:64;               /* Incremented for each octet transmitted in both good and bad packets. */
  } B;
} SW_ETH_MAC_PORT0_PM1_TEOCTN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TOCTN_union_tag { /* Port MAC 1 Transmit Octets Counter Register(ifOutOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t TOCTN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_TOCTN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TXPFN_union_tag { /* Port MAC 1 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  vuint64_t R;
  struct {
    vuint64_t TXPFN:64;                /* Incremented for each valid PAUSE frame transmitted . Note: Pause frames forwarded to the MAC from MAC Client are not counted by TXPFn. */
  } B;
} SW_ETH_MAC_PORT0_PM1_TXPFN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TFRMN_union_tag { /* Port MAC 1 Transmit Frame Counter Register(aFramesTransmittedOKn) */
  vuint64_t R;
  struct {
    vuint64_t TFRMN:64;                /* Incremented for each frame transmitted without error, including PAUSE frames. */
  } B;
} SW_ETH_MAC_PORT0_PM1_TFRMN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TFCSN_union_tag { /* Port MAC 1 Transmit Frame Check Sequence Error Counter Register() */
  vuint64_t R;
  struct {
    vuint64_t TFCSN:64;                /* Incremented for each frame transmitted with a CRC-32 error except for underflows. */
  } B;
} SW_ETH_MAC_PORT0_PM1_TFCSN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TVLANN_union_tag { /* Port MAC 1 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
  vuint64_t R;
  struct {
    vuint64_t TVLANN:64;               /* Incremented for each valid VLAN tagged frame transmitted with ethertype 0x8100. */
  } B;
} SW_ETH_MAC_PORT0_PM1_TVLANN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TERRN_union_tag { /* Port MAC 1 Transmit Frame Error Counter Register(ifOutErrorsn) */
  vuint64_t R;
  struct {
    vuint64_t TERRN:64;                /* Transmit frame error count */
  } B;
} SW_ETH_MAC_PORT0_PM1_TERRN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TUCAN_union_tag { /* Port MAC 1 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TUCAN:64;                /* Incremented for each valid frame transmitted (to the FIFO interface) in which bit 0 of the destination address was 0. */
  } B;
} SW_ETH_MAC_PORT0_PM1_TUCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TMCAN_union_tag { /* Port MAC 1 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TMCAN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_TMCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TBCAN_union_tag { /* Port MAC 1 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TBCAN:64;                /* Incremented for each valid frame transmitted (to the FIFO interface) in which all bits of the destination address were 1 . */
  } B;
} SW_ETH_MAC_PORT0_PM1_TBCAN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TPKTN_union_tag { /* Port MAC 1 Transmit Packets Counter Register(etherStatsPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TPKTN:64;                /* Incremented for each good or bad packet transmitted. */
  } B;
} SW_ETH_MAC_PORT0_PM1_TPKTN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TUNDN_union_tag { /* Port MAC 1 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t TUNDN:64;                /* Incremented for each packet transmitted that was less than 64 octets long with a good CRC. */
  } B;
} SW_ETH_MAC_PORT0_PM1_TUNDN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_T64N_union_tag { /* Port MAC 1 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T64N:64;                 /* Incremented for each 64-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_T64N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_T127N_union_tag { /* Port MAC 1 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T127N:64;                /* Incremented for each 65 to 127-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_T127N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_T255N_union_tag { /* Port MAC 1 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T255N:64;                /* Incremented for each 128 to 255-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_T255N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_T511N_union_tag { /* Port MAC 1 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T511N:64;                /* Incremented for each 256 to 511-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_T511N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_T1023N_union_tag { /* Port MAC 1 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1023N:64;               /* Incremented for each 512 to 1023-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_T1023N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_T1522N_union_tag { /* Port MAC 1 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1522N:64;               /* Incremented for each 1024- to 1522-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT0_PM1_T1522N_tag;

typedef union SW_ETH_MAC_PORT0_PM1_T1523XN_union_tag { /* Port MAC 1 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1523XN:64;
  } B;
} SW_ETH_MAC_PORT0_PM1_T1523XN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TCNPN_union_tag { /* Port MAC 1 Transmit Control Packet Counter Register */
  vuint64_t R;
  struct {
    vuint64_t TCNPN:64;                /* Incremented for each valid control packet transmitted (type 0x8808) but not for PAUSE packets */
  } B;
} SW_ETH_MAC_PORT0_PM1_TCNPN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TDFRN_union_tag { /* Port MAC 1 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
  vuint64_t R;
  struct {
    vuint64_t TDFRN:64;                /* Increments for successful transmissions, without retransmits, that were deferred (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT0_PM1_TDFRN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TMCOLN_union_tag { /* Port MAC 1 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
  vuint64_t R;
  struct {
    vuint64_t TMCOLN:64;               /* Increments for successful transmission after more than one retransmission (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT0_PM1_TMCOLN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TSCOLN_union_tag { /* Port MAC 1 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
  vuint64_t R;
  struct {
    vuint64_t TSCOLN:64;               /* Increments for successful transmission after one retransmission (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT0_PM1_TSCOLN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TLCOLN_union_tag { /* Port MAC 1 Transmit Late Collision Counter(aLateCollisions) Register */
  vuint64_t R;
  struct {
    vuint64_t TLCOLN:64;               /* Late collision occurred. Frame corrupted / discarded (half-duplex only) */
  } B;
} SW_ETH_MAC_PORT0_PM1_TLCOLN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_TECOLN_union_tag { /* Port MAC 1 Transmit Excessive Collisions Counter Register */
  vuint64_t R;
  struct {
    vuint64_t TECOLN:64;               /* Excessive collisions occurred. Frame was discarded (half-duplex only) */
  } B;
} SW_ETH_MAC_PORT0_PM1_TECOLN_tag;

typedef union SW_ETH_MAC_PORT0_PM1_IF_MODE_union_tag { /* Port MAC 1 Interface Mode Control Register */
  vuint32_t R;
  struct {
    vuint32_t IFMODE:3;                /* Interface mode */
    vuint32_t REVMII:1;                /* Reverse Mode */
    vuint32_t M10:1;
    vuint32_t _unused_5:1;
    vuint32_t HD:1;                    /* Half-duplex */
    vuint32_t _unused_7:5;
    vuint32_t _unused_12:1;
    vuint32_t SSP:2;                   /* Set Speed */
    vuint32_t _unused_15:17;
  } B;
} SW_ETH_MAC_PORT0_PM1_IF_MODE_tag;

typedef union SW_ETH_MAC_PORT0_MAC_MERGE_MMCSR_union_tag { /* Port MAC Merge Control and Status Register */
  vuint32_t R;
  struct {
    vuint32_t LPS:1;                   /* Local preemption supported */
    vuint32_t LPE:1;                   /* Local preemption enabled */
    vuint32_t LPA:1;                   /* Local preemption active */
    vuint32_t LAFS:2;                  /* Local additional fragment size */
    vuint32_t RPS:1;                   /* Remote preemption supported */
    vuint32_t RPE:1;                   /* Remote preemption enabled */
    vuint32_t RPA:1;                   /* Remote preemption active */
    vuint32_t RAFS:2;                  /* Remote additional fragment size */
    vuint32_t _unused_10:5;
    vuint32_t ME:2;                    /* Merge enabled */
    vuint32_t VDIS:1;                  /* Verify disabled */
    vuint32_t VSTS:3;                  /* Verify status */
    vuint32_t TXSTS:2;                 /* Merge status */
    vuint32_t VT:7;                    /* Verify Time */
    vuint32_t _unused_30:1;
    vuint32_t LINK_FAIL:1;             /* Link Fail */
  } B;
} SW_ETH_MAC_PORT0_MAC_MERGE_MMCSR_tag;

typedef union SW_ETH_MAC_PORT0_MAC_MERGE_MMFAECR_union_tag { /* Port MAC Merge Frame Assembly Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t MMFAEC:32;
  } B;
} SW_ETH_MAC_PORT0_MAC_MERGE_MMFAECR_tag;

typedef union SW_ETH_MAC_PORT0_MAC_MERGE_MMFSECR_union_tag { /* Port MAC Merge Frame SMD Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t MMFSEC:32;
  } B;
} SW_ETH_MAC_PORT0_MAC_MERGE_MMFSECR_tag;

typedef union SW_ETH_MAC_PORT0_MAC_MERGE_MMFAOCR_union_tag { /* Port MAC Merge Frame Assembly OK Count Register */
  vuint32_t R;
  struct {
    vuint32_t MMFAOC:32;
  } B;
} SW_ETH_MAC_PORT0_MAC_MERGE_MMFAOCR_tag;

typedef union SW_ETH_MAC_PORT0_MAC_MERGE_MMFCRXR_union_tag { /* Port MAC Merge Fragment Count RX Register */
  vuint32_t R;
  struct {
    vuint32_t MMFCRX:32;
  } B;
} SW_ETH_MAC_PORT0_MAC_MERGE_MMFCRXR_tag;

typedef union SW_ETH_MAC_PORT0_MAC_MERGE_MMFCTXR_union_tag { /* Port MAC Merge Fragment Count TX Register */
  vuint32_t R;
  struct {
    vuint32_t MMFCTX:32;
  } B;
} SW_ETH_MAC_PORT0_MAC_MERGE_MMFCTXR_tag;

typedef union SW_ETH_MAC_PORT0_MAC_MERGE_MMHCR_union_tag { /* Port MAC Merge Hold Count Register */
  vuint32_t R;
  struct {
    vuint32_t MMHC:32;
  } B;
} SW_ETH_MAC_PORT0_MAC_MERGE_MMHCR_tag;

typedef union SW_ETH_MAC_PORT0_PEMDIOCR_union_tag { /* Port external MDIO configuration register */
  vuint32_t R;
  struct {
    vuint32_t BSY2:1;                  /* Busy 2 (same as bit 31) */
    vuint32_t MDIO_RD_ER:1;
    vuint32_t MDIO_HOLD:3;             /* MDIO Hold Time */
    vuint32_t PRE_DIS:1;               /* MDIO Preamble Disable */
    vuint32_t ENC45:1;                 /* Enable Clause 45 Support */
    vuint32_t MDIO_CLK_DIV:9;          /* MDIO Clock Divisor */
    vuint32_t WHOAMI:3;                /* Returns the link ID */
    vuint32_t _unused_19:3;
    vuint32_t EHOLD:1;                 /* Extended HOLD */
    vuint32_t NEG:1;
    vuint32_t _unused_24:4;
    vuint32_t ADDR_ERR:1;              /* Address Error */
    vuint32_t CIM:1;
    vuint32_t CMP:1;                   /* MDIO Command Completion */
    vuint32_t BSY1:1;                  /* Busy 1 */
  } B;
} SW_ETH_MAC_PORT0_PEMDIOCR_tag;

typedef union SW_ETH_MAC_PORT0_PEMDIOICR_union_tag { /* Port external MDIO interface control register */
  vuint32_t R;
  struct {
    vuint32_t DEV_ADDR:5;              /* 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
    vuint32_t PORT_ADDR:5;             /* 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
    vuint32_t _unused_10:4;
    vuint32_t POST_INC:1;              /* MDIO read with address post-increment initiation. Self-clearing once transaction is complete. */
    vuint32_t READ:1;                  /* MDIO read initiation. */
    vuint32_t _unused_16:15;
    vuint32_t BSY:1;                   /* MDIO busy */
  } B;
} SW_ETH_MAC_PORT0_PEMDIOICR_tag;

typedef union SW_ETH_MAC_PORT0_PEMDIOIDR_union_tag { /* Port external MDIO interface data register */
  vuint32_t R;
  struct {
    vuint32_t MDIO_DATA:16;            /* 16-bit MDIO data. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PEMDIOIDR_tag;

typedef union SW_ETH_MAC_PORT0_PEMDIORAR_union_tag { /* Port external MDIO register address register */
  vuint32_t R;
  struct {
    vuint32_t REGADDR:16;              /* MDIO PHY register address. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PEMDIORAR_tag;

typedef union SW_ETH_MAC_PORT0_PEMDIOSR_union_tag { /* Port external MDIO status register */
  vuint32_t R;
  struct {
    vuint32_t BSY:1;                   /* Global MDIO busy */
    vuint32_t _unused_1:7;
    vuint32_t WHT_LIST:5;              /* PHY white list */
    vuint32_t _unused_13:2;
    vuint32_t WHT_LIST_ENA:1;          /* PHY white list enable */
    vuint32_t PORT_ID:3;               /* Port ID */
    vuint32_t REQ_TYPE:1;              /* Port ID */
    vuint32_t _unused_20:12;
  } B;
} SW_ETH_MAC_PORT0_PEMDIOSR_tag;

typedef union SW_ETH_MAC_PORT0_PPSCR_union_tag { /* PHY status configuration register */
  vuint32_t R;
  struct {
    vuint32_t BSY:1;                   /* MDIO busy */
    vuint32_t MDIO_RD_ER:1;            /* MDIO read error */
    vuint32_t _unused_2:14;
    vuint32_t STATUS_INTERVAL:16;      /* PHY status read interval */
  } B;
} SW_ETH_MAC_PORT0_PPSCR_tag;

typedef union SW_ETH_MAC_PORT0_PPSCTRLR_union_tag { /* Port PHY status control register */
  vuint32_t R;
  struct {
    vuint32_t DEV_ADDR:5;              /* 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
    vuint32_t PORT_ADDR:5;             /* 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
    vuint32_t _unused_10:22;
  } B;
} SW_ETH_MAC_PORT0_PPSCTRLR_tag;

typedef union SW_ETH_MAC_PORT0_PPSDR_union_tag { /* Port PHY status data register */
  vuint32_t R;
  struct {
    vuint32_t MDIO_DATA:16;            /* 16-bit MDIO data */
    vuint32_t CURR_CNT:16;             /* Current count */
  } B;
} SW_ETH_MAC_PORT0_PPSDR_tag;

typedef union SW_ETH_MAC_PORT0_PPSRAR_union_tag { /* Port PHY status register address register */
  vuint32_t R;
  struct {
    vuint32_t REGADDR:16;              /* MDIO PHY register address. Address of the register within the Clause 45 PHY device from which data is to be read. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT0_PPSRAR_tag;

typedef union SW_ETH_MAC_PORT0_PPSER_union_tag { /* Port PHY status event register */
  vuint32_t R;
  struct {
    vuint32_t STATUS_EVENT_HL:16;      /* Status event high-to-low. Set to 1 if a 1->0 transition on a corresponding data bit has occurred. Write 1 to clear. */
    vuint32_t STATUS_EVENT_LH:16;      /* Status event low-to-high. Set to 1 if a 0->1 transition on a corresponding data bit has occurred. Write 1 to clear. */
  } B;
} SW_ETH_MAC_PORT0_PPSER_tag;

typedef union SW_ETH_MAC_PORT0_PPSMR_union_tag { /* Port PHY status mask register */
  vuint32_t R;
  struct {
    vuint32_t STATUS_MASK_HL:16;       /* Status high-to-low mask. If set to 1, assert an interrupt if the corresponding event bit is set. */
    vuint32_t STATUS_MASK_LH:16;       /* Status mask low-to-high. If set to 1, assert an interrupt if the corresponding event bit is set. */
  } B;
} SW_ETH_MAC_PORT0_PPSMR_tag;

struct SW_ETH_MAC_PORT0_tag {
  uint8_t SW_ETH_MAC_PORT0_reserved0[8];
  SW_ETH_MAC_PORT0_PM0_COMMAND_CONFIG_tag PM0_COMMAND_CONFIG; /* Port MAC 0 Command and Configuration Register */
  SW_ETH_MAC_PORT0_PM0_MAC_ADDR_0_tag PM0_MAC_ADDR_0; /* Port MAC 0 MAC Address Register 0 */
  SW_ETH_MAC_PORT0_PM0_MAC_ADDR_1_tag PM0_MAC_ADDR_1; /* Port MAC 0 MAC Address Register 1 */
  SW_ETH_MAC_PORT0_PM0_MAXFRM_tag PM0_MAXFRM; /* Port MAC 0 Maximum Frame Length Register */
  uint8_t SW_ETH_MAC_PORT0_reserved1[40];
  SW_ETH_MAC_PORT0_PM0_IEVENT_tag PM0_IEVENT; /* Port MAC 0 Interrupt Event Register */
  uint8_t SW_ETH_MAC_PORT0_reserved2[8];
  SW_ETH_MAC_PORT0_PM0_IMASK_tag PM0_IMASK; /* Port MAC 0 Interrupt Mask Register(INT_MASK) */
  uint8_t SW_ETH_MAC_PORT0_reserved3[4];
  SW_ETH_MAC_PORT0_PM0_PAUSE_QUANTA_tag PM0_PAUSE_QUANTA; /* Port MAC 0 Pause Quanta Register */
  uint8_t SW_ETH_MAC_PORT0_reserved4[12];
  SW_ETH_MAC_PORT0_PM0_PAUSE_THRESH_tag PM0_PAUSE_THRESH; /* Port MAC 0 Pause Quanta Threshold Register */
  uint8_t SW_ETH_MAC_PORT0_reserved5[12];
  SW_ETH_MAC_PORT0_PM0_RX_PAUSE_STATUS_tag PM0_RX_PAUSE_STATUS; /* Port MAC 0 Receive Pause Status Register */
  uint8_t SW_ETH_MAC_PORT0_reserved6[72];
  SW_ETH_MAC_PORT0_PM0_SINGLE_STEP_tag PM0_SINGLE_STEP; /* Port MAC 0 IEEE1588 Single-Step Control Register */
  uint8_t SW_ETH_MAC_PORT0_reserved7[12];
  SW_ETH_MAC_PORT0_PM0_HD_BACKOFF_ENTROPY_tag PM0_HD_BACKOFF_ENTROPY; /* Port MAC 0 half-duplex backoff entropy register */
  uint8_t SW_ETH_MAC_PORT0_reserved8[12];
  SW_ETH_MAC_PORT0_PM0_STATN_CONFIG_tag PM0_STATN_CONFIG; /* Port MAC 0 Statistics Configuration Register */
  uint8_t SW_ETH_MAC_PORT0_reserved9[28];
  SW_ETH_MAC_PORT0_PM0_REOCTN_tag PM0_REOCTN; /* Port MAC 0 Receive Ethernet Octets Counter(etherStatsOctetsn) */
  SW_ETH_MAC_PORT0_PM0_ROCTN_tag PM0_ROCTN; /* Port MAC 0 Receive Octets Counter(iflnOctetsn) */
  uint8_t SW_ETH_MAC_PORT0_reserved10[8];
  SW_ETH_MAC_PORT0_PM0_RXPFN_tag PM0_RXPFN; /* Port MAC 0 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  SW_ETH_MAC_PORT0_PM0_RFRMN_tag PM0_RFRMN; /* Port MAC 0 Receive Frame Counter Register(aFramesReceivedOKn) */
  SW_ETH_MAC_PORT0_PM0_RFCSN_tag PM0_RFCSN; /* Port MAC 0 Receive Frame Check Sequence Error Counter Register() */
  SW_ETH_MAC_PORT0_PM0_RVLANN_tag PM0_RVLANN; /* Port MAC 0 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
  SW_ETH_MAC_PORT0_PM0_RERRN_tag PM0_RERRN; /* Port MAC 0 Receive Frame Error Counter Register(ifInErrorsn) */
  SW_ETH_MAC_PORT0_PM0_RUCAN_tag PM0_RUCAN; /* Port MAC 0 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
  SW_ETH_MAC_PORT0_PM0_RMCAN_tag PM0_RMCAN; /* Port MAC 0 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
  SW_ETH_MAC_PORT0_PM0_RBCAN_tag PM0_RBCAN; /* Port MAC 0 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
  SW_ETH_MAC_PORT0_PM0_RDRPN_tag PM0_RDRPN; /* Port MAC 0 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
  SW_ETH_MAC_PORT0_PM0_RPKTN_tag PM0_RPKTN; /* Port MAC 0 Receive Packets Counter Register(etherStatsPktsn) */
  SW_ETH_MAC_PORT0_PM0_RUNDN_tag PM0_RUNDN; /* Port MAC 0 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  SW_ETH_MAC_PORT0_PM0_R64N_tag PM0_R64N; /* Port MAC 0 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
  SW_ETH_MAC_PORT0_PM0_R127N_tag PM0_R127N; /* Port MAC 0 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
  SW_ETH_MAC_PORT0_PM0_R255N_tag PM0_R255N; /* Port MAC 0 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
  SW_ETH_MAC_PORT0_PM0_R511N_tag PM0_R511N; /* Port MAC 0 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
  SW_ETH_MAC_PORT0_PM0_R1023N_tag PM0_R1023N; /* Port MAC 0 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
  SW_ETH_MAC_PORT0_PM0_R1522N_tag PM0_R1522N; /* Port MAC 0 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
  SW_ETH_MAC_PORT0_PM0_R1523XN_tag PM0_R1523XN; /* Port MAC 0 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
  SW_ETH_MAC_PORT0_PM0_ROVRN_tag PM0_ROVRN; /* Port MAC 0 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
  SW_ETH_MAC_PORT0_PM0_RJBRN_tag PM0_RJBRN; /* Port MAC 0 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
  SW_ETH_MAC_PORT0_PM0_RFRGN_tag PM0_RFRGN; /* Port MAC 0 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
  SW_ETH_MAC_PORT0_PM0_RCNPN_tag PM0_RCNPN; /* Port MAC 0 Receive Control Packet Counter Register */
  SW_ETH_MAC_PORT0_PM0_RDRNTPN_tag PM0_RDRNTPN; /* Port MAC 0 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
  uint8_t SW_ETH_MAC_PORT0_reserved11[48];
  SW_ETH_MAC_PORT0_PM0_TEOCTN_tag PM0_TEOCTN; /* Port MAC 0 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
  SW_ETH_MAC_PORT0_PM0_TOCTN_tag PM0_TOCTN; /* Port MAC 0 Transmit Octets Counter Register(ifOutOctetsn) */
  uint8_t SW_ETH_MAC_PORT0_reserved12[8];
  SW_ETH_MAC_PORT0_PM0_TXPFN_tag PM0_TXPFN; /* Port MAC 0 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  SW_ETH_MAC_PORT0_PM0_TFRMN_tag PM0_TFRMN; /* Port MAC 0 Transmit Frame Counter Register(aFramesTransmittedOKn) */
  SW_ETH_MAC_PORT0_PM0_TFCSN_tag PM0_TFCSN; /* Port MAC 0 Transmit Frame Check Sequence Error Counter Register() */
  SW_ETH_MAC_PORT0_PM0_TVLANN_tag PM0_TVLANN; /* Port MAC 0 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
  SW_ETH_MAC_PORT0_PM0_TERRN_tag PM0_TERRN; /* Port MAC 0 Transmit Frame Error Counter Register(ifOutErrorsn) */
  SW_ETH_MAC_PORT0_PM0_TUCAN_tag PM0_TUCAN; /* Port MAC 0 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
  SW_ETH_MAC_PORT0_PM0_TMCAN_tag PM0_TMCAN; /* Port MAC 0 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
  SW_ETH_MAC_PORT0_PM0_TBCAN_tag PM0_TBCAN; /* Port MAC 0 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
  uint8_t SW_ETH_MAC_PORT0_reserved13[8];
  SW_ETH_MAC_PORT0_PM0_TPKTN_tag PM0_TPKTN; /* Port MAC 0 Transmit Packets Counter Register(etherStatsPktsn) */
  SW_ETH_MAC_PORT0_PM0_TUNDN_tag PM0_TUNDN; /* Port MAC 0 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  SW_ETH_MAC_PORT0_PM0_T64N_tag PM0_T64N; /* Port MAC 0 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
  SW_ETH_MAC_PORT0_PM0_T127N_tag PM0_T127N; /* Port MAC 0 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
  SW_ETH_MAC_PORT0_PM0_T255N_tag PM0_T255N; /* Port MAC 0 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
  SW_ETH_MAC_PORT0_PM0_T511N_tag PM0_T511N; /* Port MAC 0 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
  SW_ETH_MAC_PORT0_PM0_T1023N_tag PM0_T1023N; /* Port MAC 0 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
  SW_ETH_MAC_PORT0_PM0_T1522N_tag PM0_T1522N; /* Port MAC 0 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
  SW_ETH_MAC_PORT0_PM0_T1523XN_tag PM0_T1523XN; /* Port MAC 0 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
  uint8_t SW_ETH_MAC_PORT0_reserved14[24];
  SW_ETH_MAC_PORT0_PM0_TCNPN_tag PM0_TCNPN; /* Port MAC 0 Transmit Control Packet Counter Register */
  uint8_t SW_ETH_MAC_PORT0_reserved15[8];
  SW_ETH_MAC_PORT0_PM0_TDFRN_tag PM0_TDFRN; /* Port MAC 0 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
  SW_ETH_MAC_PORT0_PM0_TMCOLN_tag PM0_TMCOLN; /* Port MAC 0 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
  SW_ETH_MAC_PORT0_PM0_TSCOLN_tag PM0_TSCOLN; /* Port MAC 0 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
  SW_ETH_MAC_PORT0_PM0_TLCOLN_tag PM0_TLCOLN; /* Port MAC 0 Transmit Late Collision Counter(aLateCollisions) Register */
  SW_ETH_MAC_PORT0_PM0_TECOLN_tag PM0_TECOLN; /* Port MAC 0 Transmit Excessive Collisions Counter Register */
  uint8_t SW_ETH_MAC_PORT0_reserved16[8];
  SW_ETH_MAC_PORT0_PM0_IF_MODE_tag PM0_IF_MODE; /* Port MAC 0 Interface Mode Control Register */
  uint8_t SW_ETH_MAC_PORT0_reserved17[260];
  SW_ETH_MAC_PORT0_PM1_COMMAND_CONFIG_tag PM1_COMMAND_CONFIG; /* Port MAC 1 Command and Configuration Register */
  SW_ETH_MAC_PORT0_PM1_MAC_ADDR_0_tag PM1_MAC_ADDR_0; /* Port MAC 1 MAC Address Register 0 */
  SW_ETH_MAC_PORT0_PM1_MAC_ADDR_1_tag PM1_MAC_ADDR_1; /* Port MAC 1 MAC Address Register 1 */
  SW_ETH_MAC_PORT0_PM1_MAXFRM_tag PM1_MAXFRM; /* Port MAC 1 Maximum Frame Length Register */
  uint8_t SW_ETH_MAC_PORT0_reserved18[40];
  SW_ETH_MAC_PORT0_PM1_IEVENT_tag PM1_IEVENT; /* Port MAC 1 Interrupt Event Register */
  uint8_t SW_ETH_MAC_PORT0_reserved19[8];
  SW_ETH_MAC_PORT0_PM1_IMASK_tag PM1_IMASK; /* Port MAC 1 Interrupt Mask Register(INT_MASK) */
  uint8_t SW_ETH_MAC_PORT0_reserved20[4];
  SW_ETH_MAC_PORT0_PM1_PAUSE_QUANTA_tag PM1_PAUSE_QUANTA; /* Port MAC 1 Pause Quanta Register */
  uint8_t SW_ETH_MAC_PORT0_reserved21[12];
  SW_ETH_MAC_PORT0_PM1_PAUSE_THRESH_tag PM1_PAUSE_THRESH; /* Port MAC 1 Pause Quanta Threshold Register */
  uint8_t SW_ETH_MAC_PORT0_reserved22[12];
  SW_ETH_MAC_PORT0_PM1_RX_PAUSE_STATUS_tag PM1_RX_PAUSE_STATUS; /* Port MAC 1 Receive Pause Status Register */
  uint8_t SW_ETH_MAC_PORT0_reserved23[72];
  SW_ETH_MAC_PORT0_PM1_SINGLE_STEP_tag PM1_SINGLE_STEP; /* Port MAC 1 IEEE1588 Single-Step Control Register */
  uint8_t SW_ETH_MAC_PORT0_reserved24[12];
  SW_ETH_MAC_PORT0_PM1_HD_BACKOFF_ENTROPY_tag PM1_HD_BACKOFF_ENTROPY; /* Port MAC 1 half-duplex backoff entropy register */
  uint8_t SW_ETH_MAC_PORT0_reserved25[12];
  SW_ETH_MAC_PORT0_PM1_STATN_CONFIG_tag PM1_STATN_CONFIG; /* Port MAC 1 Statistics Configuration Register */
  uint8_t SW_ETH_MAC_PORT0_reserved26[28];
  SW_ETH_MAC_PORT0_PM1_REOCTN_tag PM1_REOCTN; /* Port MAC 1 Receive Ethernet Octets Counter(etherStatsOctetsn) */
  SW_ETH_MAC_PORT0_PM1_ROCTN_tag PM1_ROCTN; /* Port MAC 1 Receive Octets Counter(iflnOctetsn) */
  uint8_t SW_ETH_MAC_PORT0_reserved27[8];
  SW_ETH_MAC_PORT0_PM1_RXPFN_tag PM1_RXPFN; /* Port MAC 1 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  SW_ETH_MAC_PORT0_PM1_RFRMN_tag PM1_RFRMN; /* Port MAC 1 Receive Frame Counter Register(aFramesReceivedOKn) */
  SW_ETH_MAC_PORT0_PM1_RFCSN_tag PM1_RFCSN; /* Port MAC 1 Receive Frame Check Sequence Error Counter Register() */
  SW_ETH_MAC_PORT0_PM1_RVLANN_tag PM1_RVLANN; /* Port MAC 1 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
  SW_ETH_MAC_PORT0_PM1_RERRN_tag PM1_RERRN; /* Port MAC 1 Receive Frame Error Counter Register(ifInErrorsn) */
  SW_ETH_MAC_PORT0_PM1_RUCAN_tag PM1_RUCAN; /* Port MAC 1 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
  SW_ETH_MAC_PORT0_PM1_RMCAN_tag PM1_RMCAN; /* Port MAC 1 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
  SW_ETH_MAC_PORT0_PM1_RBCAN_tag PM1_RBCAN; /* Port MAC 1 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
  SW_ETH_MAC_PORT0_PM1_RDRPN_tag PM1_RDRPN; /* Port MAC 1 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
  SW_ETH_MAC_PORT0_PM1_RPKTN_tag PM1_RPKTN; /* Port MAC 1 Receive Packets Counter Register(etherStatsPktsn) */
  SW_ETH_MAC_PORT0_PM1_RUNDN_tag PM1_RUNDN; /* Port MAC 1 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  SW_ETH_MAC_PORT0_PM1_R64N_tag PM1_R64N; /* Port MAC 1 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
  SW_ETH_MAC_PORT0_PM1_R127N_tag PM1_R127N; /* Port MAC 1 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
  SW_ETH_MAC_PORT0_PM1_R255N_tag PM1_R255N; /* Port MAC 1 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
  SW_ETH_MAC_PORT0_PM1_R511N_tag PM1_R511N; /* Port MAC 1 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
  SW_ETH_MAC_PORT0_PM1_R1023N_tag PM1_R1023N; /* Port MAC 1 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
  SW_ETH_MAC_PORT0_PM1_R1522N_tag PM1_R1522N; /* Port MAC 1 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
  SW_ETH_MAC_PORT0_PM1_R1523XN_tag PM1_R1523XN; /* Port MAC 1 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
  SW_ETH_MAC_PORT0_PM1_ROVRN_tag PM1_ROVRN; /* Port MAC 1 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
  SW_ETH_MAC_PORT0_PM1_RJBRN_tag PM1_RJBRN; /* Port MAC 1 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
  SW_ETH_MAC_PORT0_PM1_RFRGN_tag PM1_RFRGN; /* Port MAC 1 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
  SW_ETH_MAC_PORT0_PM1_RCNPN_tag PM1_RCNPN; /* Port MAC 1 Receive Control Packet Counter Register */
  SW_ETH_MAC_PORT0_PM1_RDRNTPN_tag PM1_RDRNTPN; /* Port MAC 1 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
  uint8_t SW_ETH_MAC_PORT0_reserved28[48];
  SW_ETH_MAC_PORT0_PM1_TEOCTN_tag PM1_TEOCTN; /* Port MAC 1 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
  SW_ETH_MAC_PORT0_PM1_TOCTN_tag PM1_TOCTN; /* Port MAC 1 Transmit Octets Counter Register(ifOutOctetsn) */
  uint8_t SW_ETH_MAC_PORT0_reserved29[8];
  SW_ETH_MAC_PORT0_PM1_TXPFN_tag PM1_TXPFN; /* Port MAC 1 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  SW_ETH_MAC_PORT0_PM1_TFRMN_tag PM1_TFRMN; /* Port MAC 1 Transmit Frame Counter Register(aFramesTransmittedOKn) */
  SW_ETH_MAC_PORT0_PM1_TFCSN_tag PM1_TFCSN; /* Port MAC 1 Transmit Frame Check Sequence Error Counter Register() */
  SW_ETH_MAC_PORT0_PM1_TVLANN_tag PM1_TVLANN; /* Port MAC 1 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
  SW_ETH_MAC_PORT0_PM1_TERRN_tag PM1_TERRN; /* Port MAC 1 Transmit Frame Error Counter Register(ifOutErrorsn) */
  SW_ETH_MAC_PORT0_PM1_TUCAN_tag PM1_TUCAN; /* Port MAC 1 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
  SW_ETH_MAC_PORT0_PM1_TMCAN_tag PM1_TMCAN; /* Port MAC 1 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
  SW_ETH_MAC_PORT0_PM1_TBCAN_tag PM1_TBCAN; /* Port MAC 1 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
  uint8_t SW_ETH_MAC_PORT0_reserved30[8];
  SW_ETH_MAC_PORT0_PM1_TPKTN_tag PM1_TPKTN; /* Port MAC 1 Transmit Packets Counter Register(etherStatsPktsn) */
  SW_ETH_MAC_PORT0_PM1_TUNDN_tag PM1_TUNDN; /* Port MAC 1 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  SW_ETH_MAC_PORT0_PM1_T64N_tag PM1_T64N; /* Port MAC 1 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
  SW_ETH_MAC_PORT0_PM1_T127N_tag PM1_T127N; /* Port MAC 1 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
  SW_ETH_MAC_PORT0_PM1_T255N_tag PM1_T255N; /* Port MAC 1 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
  SW_ETH_MAC_PORT0_PM1_T511N_tag PM1_T511N; /* Port MAC 1 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
  SW_ETH_MAC_PORT0_PM1_T1023N_tag PM1_T1023N; /* Port MAC 1 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
  SW_ETH_MAC_PORT0_PM1_T1522N_tag PM1_T1522N; /* Port MAC 1 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
  SW_ETH_MAC_PORT0_PM1_T1523XN_tag PM1_T1523XN; /* Port MAC 1 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
  uint8_t SW_ETH_MAC_PORT0_reserved31[24];
  SW_ETH_MAC_PORT0_PM1_TCNPN_tag PM1_TCNPN; /* Port MAC 1 Transmit Control Packet Counter Register */
  uint8_t SW_ETH_MAC_PORT0_reserved32[8];
  SW_ETH_MAC_PORT0_PM1_TDFRN_tag PM1_TDFRN; /* Port MAC 1 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
  SW_ETH_MAC_PORT0_PM1_TMCOLN_tag PM1_TMCOLN; /* Port MAC 1 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
  SW_ETH_MAC_PORT0_PM1_TSCOLN_tag PM1_TSCOLN; /* Port MAC 1 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
  SW_ETH_MAC_PORT0_PM1_TLCOLN_tag PM1_TLCOLN; /* Port MAC 1 Transmit Late Collision Counter(aLateCollisions) Register */
  SW_ETH_MAC_PORT0_PM1_TECOLN_tag PM1_TECOLN; /* Port MAC 1 Transmit Excessive Collisions Counter Register */
  uint8_t SW_ETH_MAC_PORT0_reserved33[8];
  SW_ETH_MAC_PORT0_PM1_IF_MODE_tag PM1_IF_MODE; /* Port MAC 1 Interface Mode Control Register */
  uint8_t SW_ETH_MAC_PORT0_reserved34[252];
  SW_ETH_MAC_PORT0_MAC_MERGE_MMCSR_tag MAC_MERGE_MMCSR; /* Port MAC Merge Control and Status Register */
  uint8_t SW_ETH_MAC_PORT0_reserved35[4];
  SW_ETH_MAC_PORT0_MAC_MERGE_MMFAECR_tag MAC_MERGE_MMFAECR; /* Port MAC Merge Frame Assembly Error Count Register */
  SW_ETH_MAC_PORT0_MAC_MERGE_MMFSECR_tag MAC_MERGE_MMFSECR; /* Port MAC Merge Frame SMD Error Count Register */
  SW_ETH_MAC_PORT0_MAC_MERGE_MMFAOCR_tag MAC_MERGE_MMFAOCR; /* Port MAC Merge Frame Assembly OK Count Register */
  SW_ETH_MAC_PORT0_MAC_MERGE_MMFCRXR_tag MAC_MERGE_MMFCRXR; /* Port MAC Merge Fragment Count RX Register */
  SW_ETH_MAC_PORT0_MAC_MERGE_MMFCTXR_tag MAC_MERGE_MMFCTXR; /* Port MAC Merge Fragment Count TX Register */
  SW_ETH_MAC_PORT0_MAC_MERGE_MMHCR_tag MAC_MERGE_MMHCR; /* Port MAC Merge Hold Count Register */
  uint8_t SW_ETH_MAC_PORT0_reserved36[992];
  SW_ETH_MAC_PORT0_PEMDIOCR_tag PEMDIOCR; /* Port external MDIO configuration register */
  SW_ETH_MAC_PORT0_PEMDIOICR_tag PEMDIOICR; /* Port external MDIO interface control register */
  SW_ETH_MAC_PORT0_PEMDIOIDR_tag PEMDIOIDR; /* Port external MDIO interface data register */
  SW_ETH_MAC_PORT0_PEMDIORAR_tag PEMDIORAR; /* Port external MDIO register address register */
  SW_ETH_MAC_PORT0_PEMDIOSR_tag PEMDIOSR; /* Port external MDIO status register */
  uint8_t SW_ETH_MAC_PORT0_reserved37[12];
  SW_ETH_MAC_PORT0_PPSCR_tag PPSCR;    /* PHY status configuration register */
  SW_ETH_MAC_PORT0_PPSCTRLR_tag PPSCTRLR; /* Port PHY status control register */
  SW_ETH_MAC_PORT0_PPSDR_tag PPSDR;    /* Port PHY status data register */
  SW_ETH_MAC_PORT0_PPSRAR_tag PPSRAR;  /* Port PHY status register address register */
  SW_ETH_MAC_PORT0_PPSER_tag PPSER;    /* Port PHY status event register */
  SW_ETH_MAC_PORT0_PPSMR_tag PPSMR;    /* Port PHY status mask register */
};


/* ============================================================================
   =============================== Module: SW_ETH_MAC_PORT1 ===================
   ============================================================================ */

typedef union SW_ETH_MAC_PORT1_PM0_COMMAND_CONFIG_union_tag { /* Port MAC 0 Command and Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t TX_EN:1;                 /* MAC transmit path enable */
    vuint32_t RX_EN:1;                 /* MAC receive path enable */
    vuint32_t _unused_2:5;
    vuint32_t PAUSE_FWD:1;             /* Terminate/forward received PAUSE frames */
    vuint32_t PAUSE_IGN:1;             /* Ignore PAUSE frame quanta */
    vuint32_t TX_ADDR_INS:1;           /* Transmit source MAC address insertion */
    vuint32_t LOOP_ENA:1;              /* Loopback enable */
    vuint32_t _unused_11:2;
    vuint32_t CNT_FRM_EN:1;            /* Control frame reception enable */
    vuint32_t _unused_14:1;
    vuint32_t TXP:1;                   /* Enable padding of frames in transmit direction (1, default). */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:3;
    vuint32_t _unused_20:2;
    vuint32_t TX_FLUSH:1;              /* Tx flush */
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t SWR:1;                   /* Software Reset. Self clearing bit. */
    vuint32_t _unused_27:3;
    vuint32_t TS_MODE:1;               /* Transmit timestamp mode */
    vuint32_t MG:1;                    /* Magic Packet detection enable. */
  } B;
} SW_ETH_MAC_PORT1_PM0_COMMAND_CONFIG_tag;

typedef union SW_ETH_MAC_PORT1_PM0_MAC_ADDR_0_union_tag { /* Port MAC 0 MAC Address Register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR_0:32;           /* MAC address 0 */
  } B;
} SW_ETH_MAC_PORT1_PM0_MAC_ADDR_0_tag;

typedef union SW_ETH_MAC_PORT1_PM0_MAC_ADDR_1_union_tag { /* Port MAC 0 MAC Address Register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR_1:16;           /* MAC address 1 */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PM0_MAC_ADDR_1_tag;

typedef union SW_ETH_MAC_PORT1_PM0_MAXFRM_union_tag { /* Port MAC 0 Maximum Frame Length Register */
  vuint32_t R;
  struct {
    vuint32_t MAXFRM:16;               /* Maximum supported received frame length. */
    vuint32_t TX_MTU:16;               /* Maximum transmit frame length */
  } B;
} SW_ETH_MAC_PORT1_PM0_MAXFRM_tag;

typedef union SW_ETH_MAC_PORT1_PM0_IEVENT_union_tag { /* Port MAC 0 Interrupt Event Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t _unused_4:1;
    vuint32_t TX_EMPTY:1;              /* Transmit FIFO empty event */
    vuint32_t RX_EMPTY:1;              /* Receive idle event */
    vuint32_t _unused_7:3;
    vuint32_t TX_OVFL:1;               /* Transmit FIFO overflow event. */
    vuint32_t TX_UNFL:1;               /* Transmit FIFO underflow event. */
    vuint32_t RX_OVFL:1;               /* Receive FIFO overflow event. */
    vuint32_t _unused_13:1;
    vuint32_t MGI:1;                   /* Magic packet detection indication event */
    vuint32_t _unused_15:12;
    vuint32_t _unused_27:2;
    vuint32_t _unused_29:3;
  } B;
} SW_ETH_MAC_PORT1_PM0_IEVENT_tag;

typedef union SW_ETH_MAC_PORT1_PM0_IMASK_union_tag { /* Port MAC 0 Interrupt Mask Register(INT_MASK) */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:4;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:6;
    vuint32_t MGI:1;                   /* Magic packet detection indication event mask. */
    vuint32_t _unused_15:9;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:5;
  } B;
} SW_ETH_MAC_PORT1_PM0_IMASK_tag;

typedef union SW_ETH_MAC_PORT1_PM0_PAUSE_QUANTA_union_tag { /* Port MAC 0 Pause Quanta Register */
  vuint32_t R;
  struct {
    vuint32_t PQNT:16;                 /* Value to be used for the quanta value when XOFF is triggered. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PM0_PAUSE_QUANTA_tag;

typedef union SW_ETH_MAC_PORT1_PM0_PAUSE_THRESH_union_tag { /* Port MAC 0 Pause Quanta Threshold Register */
  vuint32_t R;
  struct {
    vuint32_t QTH:16;                  /* Quanta threshold. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PM0_PAUSE_THRESH_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RX_PAUSE_STATUS_union_tag { /* Port MAC 0 Receive Pause Status Register */
  vuint32_t R;
  struct {
    vuint32_t PSTAT:1;                 /* Pause status. */
    vuint32_t _unused_1:31;
  } B;
} SW_ETH_MAC_PORT1_PM0_RX_PAUSE_STATUS_tag;

typedef union SW_ETH_MAC_PORT1_PM0_SINGLE_STEP_union_tag { /* Port MAC 0 IEEE1588 Single-Step Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t OFFSET:8;
    vuint32_t _unused_15:16;
    vuint32_t EN:1;                    /* IEEE-1588 Single-Step enable. */
  } B;
} SW_ETH_MAC_PORT1_PM0_SINGLE_STEP_tag;

typedef union SW_ETH_MAC_PORT1_PM0_HD_BACKOFF_ENTROPY_union_tag { /* Port MAC 0 half-duplex backoff entropy register */
  vuint32_t R;
  struct {
    vuint32_t HD_BACKOFF_ENTROPY:10;   /* Half duplex backoff entropy */
    vuint32_t _unused_10:21;
    vuint32_t SW_ENTROPY_VALID:1;      /* Software programmable entropy valid */
  } B;
} SW_ETH_MAC_PORT1_PM0_HD_BACKOFF_ENTROPY_tag;

typedef union SW_ETH_MAC_PORT1_PM0_STATN_CONFIG_union_tag { /* Port MAC 0 Statistics Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SAT:1;
    vuint32_t COD:1;
    vuint32_t CLR:1;
    vuint32_t _unused_3:29;
  } B;
} SW_ETH_MAC_PORT1_PM0_STATN_CONFIG_tag;

typedef union SW_ETH_MAC_PORT1_PM0_REOCTN_union_tag { /* Port MAC 0 Receive Ethernet Octets Counter(etherStatsOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t REOCTN:64;               /* Incremented for each octet received in both good and bad packets. */
  } B;
} SW_ETH_MAC_PORT1_PM0_REOCTN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_ROCTN_union_tag { /* Port MAC 0 Receive Octets Counter(iflnOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t ROCTN:64;                /* Incremented for each octet received except preamble (that is, Header, Payload, Pad and FCS) for all valid frames and valid PAUSE frames received. */
  } B;
} SW_ETH_MAC_PORT1_PM0_ROCTN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RXPFN_union_tag { /* Port MAC 0 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  vuint64_t R;
  struct {
    vuint64_t RXPFN:64;                /* Incremented for each valid PAUSE frame received . */
  } B;
} SW_ETH_MAC_PORT1_PM0_RXPFN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RFRMN_union_tag { /* Port MAC 0 Receive Frame Counter Register(aFramesReceivedOKn) */
  vuint64_t R;
  struct {
    vuint64_t RFRMN:64;                /* Incremented for each frame received without error, including PAUSE frames. */
  } B;
} SW_ETH_MAC_PORT1_PM0_RFRMN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RFCSN_union_tag { /* Port MAC 0 Receive Frame Check Sequence Error Counter Register() */
  vuint64_t R;
  struct {
    vuint64_t RFCSN:64;                /* Incremented for each frame received with a CRC-32 error but the frame is otherwise of correct length. */
  } B;
} SW_ETH_MAC_PORT1_PM0_RFCSN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RVLANN_union_tag { /* Port MAC 0 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
  vuint64_t R;
  struct {
    vuint64_t RVLANN:64;               /* Incremented for each valid VLAN tagged frame received with ethertype 0x8100 */
  } B;
} SW_ETH_MAC_PORT1_PM0_RVLANN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RERRN_union_tag { /* Port MAC 0 Receive Frame Error Counter Register(ifInErrorsn) */
  vuint64_t R;
  struct {
    vuint64_t RERRN:64;                /* Incremented for each frame received with an error (except for undersized/fragment frame): */
  } B;
} SW_ETH_MAC_PORT1_PM0_RERRN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RUCAN_union_tag { /* Port MAC 0 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RUCAN:64;                /* Incremented for each valid frame received (on the receive FIFO interface) in which bit 0 of the destination address was 0 . */
  } B;
} SW_ETH_MAC_PORT1_PM0_RUCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RMCAN_union_tag { /* Port MAC 0 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RMCAN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_RMCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RBCAN_union_tag { /* Port MAC 0 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RBCAN:64;                /* Incremented for each valid frame received (on the receive FIFO interface) in which all bits of the destination address were 1 . */
  } B;
} SW_ETH_MAC_PORT1_PM0_RBCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RDRPN_union_tag { /* Port MAC 0 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
  vuint64_t R;
  struct {
    vuint64_t RDRPN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_RDRPN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RPKTN_union_tag { /* Port MAC 0 Receive Packets Counter Register(etherStatsPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RPKTN:64;                /* Incremented for each good or bad packet received. */
  } B;
} SW_ETH_MAC_PORT1_PM0_RPKTN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RUNDN_union_tag { /* Port MAC 0 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t RUNDN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_RUNDN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_R64N_union_tag { /* Port MAC 0 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R64N:64;                 /* Incremented for each 64-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_R64N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_R127N_union_tag { /* Port MAC 0 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R127N:64;                /* Incremented for each 65- to 127-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_R127N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_R255N_union_tag { /* Port MAC 0 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R255N:64;                /* Incremented for each 128- to 255-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_R255N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_R511N_union_tag { /* Port MAC 0 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R511N:64;                /* Incremented for each 256- to 511-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_R511N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_R1023N_union_tag { /* Port MAC 0 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1023N:64;               /* Incremented for each 512- to 1023-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_R1023N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_R1522N_union_tag { /* Port MAC 0 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1522N:64;               /* Incremented for each 1024- to 1522-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_R1522N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_R1523XN_union_tag { /* Port MAC 0 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1523XN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_R1523XN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_ROVRN_union_tag { /* Port MAC 0 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t ROVRN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_ROVRN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RJBRN_union_tag { /* Port MAC 0 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
  vuint64_t R;
  struct {
    vuint64_t RJBRN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_RJBRN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RFRGN_union_tag { /* Port MAC 0 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
  vuint64_t R;
  struct {
    vuint64_t RFRGN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_RFRGN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RCNPN_union_tag { /* Port MAC 0 Receive Control Packet Counter Register */
  vuint64_t R;
  struct {
    vuint64_t RCNPN:64;                /* Incremented for each valid control packet (type 0x8808) but not for PAUSE packets */
  } B;
} SW_ETH_MAC_PORT1_PM0_RCNPN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_RDRNTPN_union_tag { /* Port MAC 0 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
  vuint64_t R;
  struct {
    vuint64_t RDRNTPN:64;              /* Incremented for each fully dropped packet (not truncated) due to internal errors of the MAC client. Occurs when a receive FIFO overflows. */
  } B;
} SW_ETH_MAC_PORT1_PM0_RDRNTPN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TEOCTN_union_tag { /* Port MAC 0 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t TEOCTN:64;               /* Incremented for each octet transmitted in both good and bad packets. */
  } B;
} SW_ETH_MAC_PORT1_PM0_TEOCTN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TOCTN_union_tag { /* Port MAC 0 Transmit Octets Counter Register(ifOutOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t TOCTN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_TOCTN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TXPFN_union_tag { /* Port MAC 0 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  vuint64_t R;
  struct {
    vuint64_t TXPFN:64;                /* Incremented for each valid PAUSE frame transmitted . Note: Pause frames forwarded to the MAC from MAC Client are not counted by TXPFn. */
  } B;
} SW_ETH_MAC_PORT1_PM0_TXPFN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TFRMN_union_tag { /* Port MAC 0 Transmit Frame Counter Register(aFramesTransmittedOKn) */
  vuint64_t R;
  struct {
    vuint64_t TFRMN:64;                /* Incremented for each frame transmitted without error, including PAUSE frames. */
  } B;
} SW_ETH_MAC_PORT1_PM0_TFRMN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TFCSN_union_tag { /* Port MAC 0 Transmit Frame Check Sequence Error Counter Register() */
  vuint64_t R;
  struct {
    vuint64_t TFCSN:64;                /* Incremented for each frame transmitted with a CRC-32 error except for underflows. */
  } B;
} SW_ETH_MAC_PORT1_PM0_TFCSN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TVLANN_union_tag { /* Port MAC 0 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
  vuint64_t R;
  struct {
    vuint64_t TVLANN:64;               /* Incremented for each valid VLAN tagged frame transmitted with ethertype 0x8100. */
  } B;
} SW_ETH_MAC_PORT1_PM0_TVLANN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TERRN_union_tag { /* Port MAC 0 Transmit Frame Error Counter Register(ifOutErrorsn) */
  vuint64_t R;
  struct {
    vuint64_t TERRN:64;                /* Transmit frame error count */
  } B;
} SW_ETH_MAC_PORT1_PM0_TERRN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TUCAN_union_tag { /* Port MAC 0 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TUCAN:64;                /* Incremented for each valid frame transmitted (to the FIFO interface) in which bit 0 of the destination address was 0. */
  } B;
} SW_ETH_MAC_PORT1_PM0_TUCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TMCAN_union_tag { /* Port MAC 0 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TMCAN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_TMCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TBCAN_union_tag { /* Port MAC 0 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TBCAN:64;                /* Incremented for each valid frame transmitted (to the FIFO interface) in which all bits of the destination address were 1 . */
  } B;
} SW_ETH_MAC_PORT1_PM0_TBCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TPKTN_union_tag { /* Port MAC 0 Transmit Packets Counter Register(etherStatsPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TPKTN:64;                /* Incremented for each good or bad packet transmitted. */
  } B;
} SW_ETH_MAC_PORT1_PM0_TPKTN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TUNDN_union_tag { /* Port MAC 0 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t TUNDN:64;                /* Incremented for each packet transmitted that was less than 64 octets long with a good CRC. */
  } B;
} SW_ETH_MAC_PORT1_PM0_TUNDN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_T64N_union_tag { /* Port MAC 0 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T64N:64;                 /* Incremented for each 64-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_T64N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_T127N_union_tag { /* Port MAC 0 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T127N:64;                /* Incremented for each 65 to 127-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_T127N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_T255N_union_tag { /* Port MAC 0 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T255N:64;                /* Incremented for each 128 to 255-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_T255N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_T511N_union_tag { /* Port MAC 0 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T511N:64;                /* Incremented for each 256 to 511-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_T511N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_T1023N_union_tag { /* Port MAC 0 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1023N:64;               /* Incremented for each 512 to 1023-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_T1023N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_T1522N_union_tag { /* Port MAC 0 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1522N:64;               /* Incremented for each 1024- to 1522-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM0_T1522N_tag;

typedef union SW_ETH_MAC_PORT1_PM0_T1523XN_union_tag { /* Port MAC 0 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1523XN:64;
  } B;
} SW_ETH_MAC_PORT1_PM0_T1523XN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TCNPN_union_tag { /* Port MAC 0 Transmit Control Packet Counter Register */
  vuint64_t R;
  struct {
    vuint64_t TCNPN:64;                /* Incremented for each valid control packet transmitted (type 0x8808) but not for PAUSE packets */
  } B;
} SW_ETH_MAC_PORT1_PM0_TCNPN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TDFRN_union_tag { /* Port MAC 0 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
  vuint64_t R;
  struct {
    vuint64_t TDFRN:64;                /* Increments for successful transmissions, without retransmits, that were deferred (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT1_PM0_TDFRN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TMCOLN_union_tag { /* Port MAC 0 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
  vuint64_t R;
  struct {
    vuint64_t TMCOLN:64;               /* Increments for successful transmission after more than one retransmission (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT1_PM0_TMCOLN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TSCOLN_union_tag { /* Port MAC 0 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
  vuint64_t R;
  struct {
    vuint64_t TSCOLN:64;               /* Increments for successful transmission after one retransmission (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT1_PM0_TSCOLN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TLCOLN_union_tag { /* Port MAC 0 Transmit Late Collision Counter(aLateCollisions) Register */
  vuint64_t R;
  struct {
    vuint64_t TLCOLN:64;               /* Late collision occurred. Frame corrupted / discarded (half-duplex only) */
  } B;
} SW_ETH_MAC_PORT1_PM0_TLCOLN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_TECOLN_union_tag { /* Port MAC 0 Transmit Excessive Collisions Counter Register */
  vuint64_t R;
  struct {
    vuint64_t TECOLN:64;               /* Excessive collisions occurred. Frame was discarded (half-duplex only) */
  } B;
} SW_ETH_MAC_PORT1_PM0_TECOLN_tag;

typedef union SW_ETH_MAC_PORT1_PM0_IF_MODE_union_tag { /* Port MAC 0 Interface Mode Control Register */
  vuint32_t R;
  struct {
    vuint32_t IFMODE:3;                /* Interface mode */
    vuint32_t REVMII:1;                /* Reverse Mode */
    vuint32_t M10:1;
    vuint32_t _unused_5:1;
    vuint32_t HD:1;                    /* Half-duplex */
    vuint32_t _unused_7:5;
    vuint32_t _unused_12:1;
    vuint32_t SSP:2;                   /* Set Speed */
    vuint32_t _unused_15:17;
  } B;
} SW_ETH_MAC_PORT1_PM0_IF_MODE_tag;

typedef union SW_ETH_MAC_PORT1_PM1_COMMAND_CONFIG_union_tag { /* Port MAC 1 Command and Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t TX_EN:1;                 /* MAC transmit path enable */
    vuint32_t RX_EN:1;                 /* MAC receive path enable */
    vuint32_t _unused_2:5;
    vuint32_t PAUSE_FWD:1;             /* Terminate/forward received PAUSE frames */
    vuint32_t PAUSE_IGN:1;             /* Ignore PAUSE frame quanta */
    vuint32_t TX_ADDR_INS:1;           /* Transmit source MAC address insertion */
    vuint32_t LOOP_ENA:1;              /* Loopback enable */
    vuint32_t _unused_11:2;
    vuint32_t CNT_FRM_EN:1;            /* Control frame reception enable */
    vuint32_t _unused_14:1;
    vuint32_t TXP:1;                   /* Enable padding of frames in transmit direction (1, default). */
    vuint32_t _unused_16:1;
    vuint32_t _unused_17:3;
    vuint32_t _unused_20:2;
    vuint32_t TX_FLUSH:1;              /* Tx flush */
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t SWR:1;                   /* Software Reset. Self clearing bit. */
    vuint32_t _unused_27:3;
    vuint32_t TS_MODE:1;               /* Transmit timestamp mode */
    vuint32_t MG:1;                    /* Magic Packet detection enable. */
  } B;
} SW_ETH_MAC_PORT1_PM1_COMMAND_CONFIG_tag;

typedef union SW_ETH_MAC_PORT1_PM1_MAC_ADDR_0_union_tag { /* Port MAC 1 MAC Address Register 0 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR_0:32;           /* MAC address 0 */
  } B;
} SW_ETH_MAC_PORT1_PM1_MAC_ADDR_0_tag;

typedef union SW_ETH_MAC_PORT1_PM1_MAC_ADDR_1_union_tag { /* Port MAC 1 MAC Address Register 1 */
  vuint32_t R;
  struct {
    vuint32_t MAC_ADDR_1:16;           /* MAC address 1 */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PM1_MAC_ADDR_1_tag;

typedef union SW_ETH_MAC_PORT1_PM1_MAXFRM_union_tag { /* Port MAC 1 Maximum Frame Length Register */
  vuint32_t R;
  struct {
    vuint32_t MAXFRM:16;               /* Maximum supported received frame length. */
    vuint32_t TX_MTU:16;               /* Maximum transmit frame length */
  } B;
} SW_ETH_MAC_PORT1_PM1_MAXFRM_tag;

typedef union SW_ETH_MAC_PORT1_PM1_IEVENT_union_tag { /* Port MAC 1 Interrupt Event Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t _unused_4:1;
    vuint32_t TX_EMPTY:1;              /* Transmit FIFO empty event */
    vuint32_t RX_EMPTY:1;              /* Receive idle event */
    vuint32_t _unused_7:3;
    vuint32_t TX_OVFL:1;               /* Transmit FIFO overflow event. */
    vuint32_t TX_UNFL:1;               /* Transmit FIFO underflow event. */
    vuint32_t RX_OVFL:1;               /* Receive FIFO overflow event. */
    vuint32_t _unused_13:1;
    vuint32_t MGI:1;                   /* Magic packet detection indication event */
    vuint32_t _unused_15:12;
    vuint32_t _unused_27:2;
    vuint32_t _unused_29:3;
  } B;
} SW_ETH_MAC_PORT1_PM1_IEVENT_tag;

typedef union SW_ETH_MAC_PORT1_PM1_IMASK_union_tag { /* Port MAC 1 Interrupt Mask Register(INT_MASK) */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:1;
    vuint32_t _unused_2:1;
    vuint32_t _unused_3:4;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:6;
    vuint32_t MGI:1;                   /* Magic packet detection indication event mask. */
    vuint32_t _unused_15:9;
    vuint32_t _unused_24:1;
    vuint32_t _unused_25:1;
    vuint32_t _unused_26:1;
    vuint32_t _unused_27:5;
  } B;
} SW_ETH_MAC_PORT1_PM1_IMASK_tag;

typedef union SW_ETH_MAC_PORT1_PM1_PAUSE_QUANTA_union_tag { /* Port MAC 1 Pause Quanta Register */
  vuint32_t R;
  struct {
    vuint32_t PQNT:16;                 /* Value to be used for the quanta value when XOFF is triggered. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PM1_PAUSE_QUANTA_tag;

typedef union SW_ETH_MAC_PORT1_PM1_PAUSE_THRESH_union_tag { /* Port MAC 1 Pause Quanta Threshold Register */
  vuint32_t R;
  struct {
    vuint32_t QTH:16;                  /* Quanta threshold. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PM1_PAUSE_THRESH_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RX_PAUSE_STATUS_union_tag { /* Port MAC 1 Receive Pause Status Register */
  vuint32_t R;
  struct {
    vuint32_t PSTAT:1;                 /* Pause status. */
    vuint32_t _unused_1:31;
  } B;
} SW_ETH_MAC_PORT1_PM1_RX_PAUSE_STATUS_tag;

typedef union SW_ETH_MAC_PORT1_PM1_SINGLE_STEP_union_tag { /* Port MAC 1 IEEE1588 Single-Step Control Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:7;
    vuint32_t OFFSET:8;
    vuint32_t _unused_15:16;
    vuint32_t EN:1;                    /* IEEE-1588 Single-Step enable. */
  } B;
} SW_ETH_MAC_PORT1_PM1_SINGLE_STEP_tag;

typedef union SW_ETH_MAC_PORT1_PM1_HD_BACKOFF_ENTROPY_union_tag { /* Port MAC 1 half-duplex backoff entropy register */
  vuint32_t R;
  struct {
    vuint32_t HD_BACKOFF_ENTROPY:10;   /* Half duplex backoff entropy */
    vuint32_t _unused_10:21;
    vuint32_t SW_ENTROPY_VALID:1;      /* Software programmable entropy valid */
  } B;
} SW_ETH_MAC_PORT1_PM1_HD_BACKOFF_ENTROPY_tag;

typedef union SW_ETH_MAC_PORT1_PM1_STATN_CONFIG_union_tag { /* Port MAC 1 Statistics Configuration Register */
  vuint32_t R;
  struct {
    vuint32_t SAT:1;
    vuint32_t COD:1;
    vuint32_t CLR:1;
    vuint32_t _unused_3:29;
  } B;
} SW_ETH_MAC_PORT1_PM1_STATN_CONFIG_tag;

typedef union SW_ETH_MAC_PORT1_PM1_REOCTN_union_tag { /* Port MAC 1 Receive Ethernet Octets Counter(etherStatsOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t REOCTN:64;               /* Incremented for each octet received in both good and bad packets. */
  } B;
} SW_ETH_MAC_PORT1_PM1_REOCTN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_ROCTN_union_tag { /* Port MAC 1 Receive Octets Counter(iflnOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t ROCTN:64;                /* Incremented for each octet received except preamble (that is, Header, Payload, Pad and FCS) for all valid frames and valid PAUSE frames received. */
  } B;
} SW_ETH_MAC_PORT1_PM1_ROCTN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RXPFN_union_tag { /* Port MAC 1 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  vuint64_t R;
  struct {
    vuint64_t RXPFN:64;                /* Incremented for each valid PAUSE frame received . */
  } B;
} SW_ETH_MAC_PORT1_PM1_RXPFN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RFRMN_union_tag { /* Port MAC 1 Receive Frame Counter Register(aFramesReceivedOKn) */
  vuint64_t R;
  struct {
    vuint64_t RFRMN:64;                /* Incremented for each frame received without error, including PAUSE frames. */
  } B;
} SW_ETH_MAC_PORT1_PM1_RFRMN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RFCSN_union_tag { /* Port MAC 1 Receive Frame Check Sequence Error Counter Register() */
  vuint64_t R;
  struct {
    vuint64_t RFCSN:64;                /* Incremented for each frame received with a CRC-32 error but the frame is otherwise of correct length. */
  } B;
} SW_ETH_MAC_PORT1_PM1_RFCSN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RVLANN_union_tag { /* Port MAC 1 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
  vuint64_t R;
  struct {
    vuint64_t RVLANN:64;               /* Incremented for each valid VLAN tagged frame received with ethertype 0x8100 */
  } B;
} SW_ETH_MAC_PORT1_PM1_RVLANN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RERRN_union_tag { /* Port MAC 1 Receive Frame Error Counter Register(ifInErrorsn) */
  vuint64_t R;
  struct {
    vuint64_t RERRN:64;                /* Incremented for each frame received with an error (except for undersized/fragment frame): */
  } B;
} SW_ETH_MAC_PORT1_PM1_RERRN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RUCAN_union_tag { /* Port MAC 1 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RUCAN:64;                /* Incremented for each valid frame received (on the receive FIFO interface) in which bit 0 of the destination address was 0 . */
  } B;
} SW_ETH_MAC_PORT1_PM1_RUCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RMCAN_union_tag { /* Port MAC 1 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RMCAN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_RMCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RBCAN_union_tag { /* Port MAC 1 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RBCAN:64;                /* Incremented for each valid frame received (on the receive FIFO interface) in which all bits of the destination address were 1 . */
  } B;
} SW_ETH_MAC_PORT1_PM1_RBCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RDRPN_union_tag { /* Port MAC 1 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
  vuint64_t R;
  struct {
    vuint64_t RDRPN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_RDRPN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RPKTN_union_tag { /* Port MAC 1 Receive Packets Counter Register(etherStatsPktsn) */
  vuint64_t R;
  struct {
    vuint64_t RPKTN:64;                /* Incremented for each good or bad packet received. */
  } B;
} SW_ETH_MAC_PORT1_PM1_RPKTN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RUNDN_union_tag { /* Port MAC 1 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t RUNDN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_RUNDN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_R64N_union_tag { /* Port MAC 1 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R64N:64;                 /* Incremented for each 64-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_R64N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_R127N_union_tag { /* Port MAC 1 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R127N:64;                /* Incremented for each 65- to 127-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_R127N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_R255N_union_tag { /* Port MAC 1 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R255N:64;                /* Incremented for each 128- to 255-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_R255N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_R511N_union_tag { /* Port MAC 1 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R511N:64;                /* Incremented for each 256- to 511-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_R511N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_R1023N_union_tag { /* Port MAC 1 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1023N:64;               /* Incremented for each 512- to 1023-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_R1023N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_R1522N_union_tag { /* Port MAC 1 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1522N:64;               /* Incremented for each 1024- to 1522-octet frame received, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_R1522N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_R1523XN_union_tag { /* Port MAC 1 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
  vuint64_t R;
  struct {
    vuint64_t R1523XN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_R1523XN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_ROVRN_union_tag { /* Port MAC 1 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t ROVRN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_ROVRN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RJBRN_union_tag { /* Port MAC 1 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
  vuint64_t R;
  struct {
    vuint64_t RJBRN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_RJBRN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RFRGN_union_tag { /* Port MAC 1 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
  vuint64_t R;
  struct {
    vuint64_t RFRGN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_RFRGN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RCNPN_union_tag { /* Port MAC 1 Receive Control Packet Counter Register */
  vuint64_t R;
  struct {
    vuint64_t RCNPN:64;                /* Incremented for each valid control packet (type 0x8808) but not for PAUSE packets */
  } B;
} SW_ETH_MAC_PORT1_PM1_RCNPN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_RDRNTPN_union_tag { /* Port MAC 1 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
  vuint64_t R;
  struct {
    vuint64_t RDRNTPN:64;              /* Incremented for each fully dropped packet (not truncated) due to internal errors of the MAC client. Occurs when a receive FIFO overflows. */
  } B;
} SW_ETH_MAC_PORT1_PM1_RDRNTPN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TEOCTN_union_tag { /* Port MAC 1 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t TEOCTN:64;               /* Incremented for each octet transmitted in both good and bad packets. */
  } B;
} SW_ETH_MAC_PORT1_PM1_TEOCTN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TOCTN_union_tag { /* Port MAC 1 Transmit Octets Counter Register(ifOutOctetsn) */
  vuint64_t R;
  struct {
    vuint64_t TOCTN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_TOCTN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TXPFN_union_tag { /* Port MAC 1 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  vuint64_t R;
  struct {
    vuint64_t TXPFN:64;                /* Incremented for each valid PAUSE frame transmitted . Note: Pause frames forwarded to the MAC from MAC Client are not counted by TXPFn. */
  } B;
} SW_ETH_MAC_PORT1_PM1_TXPFN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TFRMN_union_tag { /* Port MAC 1 Transmit Frame Counter Register(aFramesTransmittedOKn) */
  vuint64_t R;
  struct {
    vuint64_t TFRMN:64;                /* Incremented for each frame transmitted without error, including PAUSE frames. */
  } B;
} SW_ETH_MAC_PORT1_PM1_TFRMN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TFCSN_union_tag { /* Port MAC 1 Transmit Frame Check Sequence Error Counter Register() */
  vuint64_t R;
  struct {
    vuint64_t TFCSN:64;                /* Incremented for each frame transmitted with a CRC-32 error except for underflows. */
  } B;
} SW_ETH_MAC_PORT1_PM1_TFCSN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TVLANN_union_tag { /* Port MAC 1 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
  vuint64_t R;
  struct {
    vuint64_t TVLANN:64;               /* Incremented for each valid VLAN tagged frame transmitted with ethertype 0x8100. */
  } B;
} SW_ETH_MAC_PORT1_PM1_TVLANN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TERRN_union_tag { /* Port MAC 1 Transmit Frame Error Counter Register(ifOutErrorsn) */
  vuint64_t R;
  struct {
    vuint64_t TERRN:64;                /* Transmit frame error count */
  } B;
} SW_ETH_MAC_PORT1_PM1_TERRN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TUCAN_union_tag { /* Port MAC 1 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TUCAN:64;                /* Incremented for each valid frame transmitted (to the FIFO interface) in which bit 0 of the destination address was 0. */
  } B;
} SW_ETH_MAC_PORT1_PM1_TUCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TMCAN_union_tag { /* Port MAC 1 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TMCAN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_TMCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TBCAN_union_tag { /* Port MAC 1 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TBCAN:64;                /* Incremented for each valid frame transmitted (to the FIFO interface) in which all bits of the destination address were 1 . */
  } B;
} SW_ETH_MAC_PORT1_PM1_TBCAN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TPKTN_union_tag { /* Port MAC 1 Transmit Packets Counter Register(etherStatsPktsn) */
  vuint64_t R;
  struct {
    vuint64_t TPKTN:64;                /* Incremented for each good or bad packet transmitted. */
  } B;
} SW_ETH_MAC_PORT1_PM1_TPKTN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TUNDN_union_tag { /* Port MAC 1 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  vuint64_t R;
  struct {
    vuint64_t TUNDN:64;                /* Incremented for each packet transmitted that was less than 64 octets long with a good CRC. */
  } B;
} SW_ETH_MAC_PORT1_PM1_TUNDN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_T64N_union_tag { /* Port MAC 1 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T64N:64;                 /* Incremented for each 64-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_T64N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_T127N_union_tag { /* Port MAC 1 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T127N:64;                /* Incremented for each 65 to 127-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_T127N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_T255N_union_tag { /* Port MAC 1 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T255N:64;                /* Incremented for each 128 to 255-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_T255N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_T511N_union_tag { /* Port MAC 1 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T511N:64;                /* Incremented for each 256 to 511-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_T511N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_T1023N_union_tag { /* Port MAC 1 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1023N:64;               /* Incremented for each 512 to 1023-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_T1023N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_T1522N_union_tag { /* Port MAC 1 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1522N:64;               /* Incremented for each 1024- to 1522-octet frame transmitted, good or bad. */
  } B;
} SW_ETH_MAC_PORT1_PM1_T1522N_tag;

typedef union SW_ETH_MAC_PORT1_PM1_T1523XN_union_tag { /* Port MAC 1 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
  vuint64_t R;
  struct {
    vuint64_t T1523XN:64;
  } B;
} SW_ETH_MAC_PORT1_PM1_T1523XN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TCNPN_union_tag { /* Port MAC 1 Transmit Control Packet Counter Register */
  vuint64_t R;
  struct {
    vuint64_t TCNPN:64;                /* Incremented for each valid control packet transmitted (type 0x8808) but not for PAUSE packets */
  } B;
} SW_ETH_MAC_PORT1_PM1_TCNPN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TDFRN_union_tag { /* Port MAC 1 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
  vuint64_t R;
  struct {
    vuint64_t TDFRN:64;                /* Increments for successful transmissions, without retransmits, that were deferred (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT1_PM1_TDFRN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TMCOLN_union_tag { /* Port MAC 1 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
  vuint64_t R;
  struct {
    vuint64_t TMCOLN:64;               /* Increments for successful transmission after more than one retransmission (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT1_PM1_TMCOLN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TSCOLN_union_tag { /* Port MAC 1 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
  vuint64_t R;
  struct {
    vuint64_t TSCOLN:64;               /* Increments for successful transmission after one retransmission (half-duplex only). */
  } B;
} SW_ETH_MAC_PORT1_PM1_TSCOLN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TLCOLN_union_tag { /* Port MAC 1 Transmit Late Collision Counter(aLateCollisions) Register */
  vuint64_t R;
  struct {
    vuint64_t TLCOLN:64;               /* Late collision occurred. Frame corrupted / discarded (half-duplex only) */
  } B;
} SW_ETH_MAC_PORT1_PM1_TLCOLN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_TECOLN_union_tag { /* Port MAC 1 Transmit Excessive Collisions Counter Register */
  vuint64_t R;
  struct {
    vuint64_t TECOLN:64;               /* Excessive collisions occurred. Frame was discarded (half-duplex only) */
  } B;
} SW_ETH_MAC_PORT1_PM1_TECOLN_tag;

typedef union SW_ETH_MAC_PORT1_PM1_IF_MODE_union_tag { /* Port MAC 1 Interface Mode Control Register */
  vuint32_t R;
  struct {
    vuint32_t IFMODE:3;                /* Interface mode */
    vuint32_t REVMII:1;                /* Reverse Mode */
    vuint32_t M10:1;
    vuint32_t _unused_5:1;
    vuint32_t HD:1;                    /* Half-duplex */
    vuint32_t _unused_7:5;
    vuint32_t _unused_12:1;
    vuint32_t SSP:2;                   /* Set Speed */
    vuint32_t _unused_15:17;
  } B;
} SW_ETH_MAC_PORT1_PM1_IF_MODE_tag;

typedef union SW_ETH_MAC_PORT1_MAC_MERGE_MMCSR_union_tag { /* Port MAC Merge Control and Status Register */
  vuint32_t R;
  struct {
    vuint32_t LPS:1;                   /* Local preemption supported */
    vuint32_t LPE:1;                   /* Local preemption enabled */
    vuint32_t LPA:1;                   /* Local preemption active */
    vuint32_t LAFS:2;                  /* Local additional fragment size */
    vuint32_t RPS:1;                   /* Remote preemption supported */
    vuint32_t RPE:1;                   /* Remote preemption enabled */
    vuint32_t RPA:1;                   /* Remote preemption active */
    vuint32_t RAFS:2;                  /* Remote additional fragment size */
    vuint32_t _unused_10:5;
    vuint32_t ME:2;                    /* Merge enabled */
    vuint32_t VDIS:1;                  /* Verify disabled */
    vuint32_t VSTS:3;                  /* Verify status */
    vuint32_t TXSTS:2;                 /* Merge status */
    vuint32_t VT:7;                    /* Verify Time */
    vuint32_t _unused_30:1;
    vuint32_t LINK_FAIL:1;             /* Link Fail */
  } B;
} SW_ETH_MAC_PORT1_MAC_MERGE_MMCSR_tag;

typedef union SW_ETH_MAC_PORT1_MAC_MERGE_MMFAECR_union_tag { /* Port MAC Merge Frame Assembly Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t MMFAEC:32;
  } B;
} SW_ETH_MAC_PORT1_MAC_MERGE_MMFAECR_tag;

typedef union SW_ETH_MAC_PORT1_MAC_MERGE_MMFSECR_union_tag { /* Port MAC Merge Frame SMD Error Count Register */
  vuint32_t R;
  struct {
    vuint32_t MMFSEC:32;
  } B;
} SW_ETH_MAC_PORT1_MAC_MERGE_MMFSECR_tag;

typedef union SW_ETH_MAC_PORT1_MAC_MERGE_MMFAOCR_union_tag { /* Port MAC Merge Frame Assembly OK Count Register */
  vuint32_t R;
  struct {
    vuint32_t MMFAOC:32;
  } B;
} SW_ETH_MAC_PORT1_MAC_MERGE_MMFAOCR_tag;

typedef union SW_ETH_MAC_PORT1_MAC_MERGE_MMFCRXR_union_tag { /* Port MAC Merge Fragment Count RX Register */
  vuint32_t R;
  struct {
    vuint32_t MMFCRX:32;
  } B;
} SW_ETH_MAC_PORT1_MAC_MERGE_MMFCRXR_tag;

typedef union SW_ETH_MAC_PORT1_MAC_MERGE_MMFCTXR_union_tag { /* Port MAC Merge Fragment Count TX Register */
  vuint32_t R;
  struct {
    vuint32_t MMFCTX:32;
  } B;
} SW_ETH_MAC_PORT1_MAC_MERGE_MMFCTXR_tag;

typedef union SW_ETH_MAC_PORT1_MAC_MERGE_MMHCR_union_tag { /* Port MAC Merge Hold Count Register */
  vuint32_t R;
  struct {
    vuint32_t MMHC:32;
  } B;
} SW_ETH_MAC_PORT1_MAC_MERGE_MMHCR_tag;

typedef union SW_ETH_MAC_PORT1_PEMDIOCR_union_tag { /* Port external MDIO configuration register */
  vuint32_t R;
  struct {
    vuint32_t BSY2:1;                  /* Busy 2 (same as bit 31) */
    vuint32_t MDIO_RD_ER:1;
    vuint32_t MDIO_HOLD:3;             /* MDIO Hold Time */
    vuint32_t PRE_DIS:1;               /* MDIO Preamble Disable */
    vuint32_t ENC45:1;                 /* Enable Clause 45 Support */
    vuint32_t MDIO_CLK_DIV:9;          /* MDIO Clock Divisor */
    vuint32_t WHOAMI:3;                /* Returns the link ID */
    vuint32_t _unused_19:3;
    vuint32_t EHOLD:1;                 /* Extended HOLD */
    vuint32_t NEG:1;
    vuint32_t _unused_24:4;
    vuint32_t ADDR_ERR:1;              /* Address Error */
    vuint32_t CIM:1;
    vuint32_t CMP:1;                   /* MDIO Command Completion */
    vuint32_t BSY1:1;                  /* Busy 1 */
  } B;
} SW_ETH_MAC_PORT1_PEMDIOCR_tag;

typedef union SW_ETH_MAC_PORT1_PEMDIOICR_union_tag { /* Port external MDIO interface control register */
  vuint32_t R;
  struct {
    vuint32_t DEV_ADDR:5;              /* 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
    vuint32_t PORT_ADDR:5;             /* 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
    vuint32_t _unused_10:4;
    vuint32_t POST_INC:1;              /* MDIO read with address post-increment initiation. Self-clearing once transaction is complete. */
    vuint32_t READ:1;                  /* MDIO read initiation. */
    vuint32_t _unused_16:15;
    vuint32_t BSY:1;                   /* MDIO busy */
  } B;
} SW_ETH_MAC_PORT1_PEMDIOICR_tag;

typedef union SW_ETH_MAC_PORT1_PEMDIOIDR_union_tag { /* Port external MDIO interface data register */
  vuint32_t R;
  struct {
    vuint32_t MDIO_DATA:16;            /* 16-bit MDIO data. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PEMDIOIDR_tag;

typedef union SW_ETH_MAC_PORT1_PEMDIORAR_union_tag { /* Port external MDIO register address register */
  vuint32_t R;
  struct {
    vuint32_t REGADDR:16;              /* MDIO PHY register address. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PEMDIORAR_tag;

typedef union SW_ETH_MAC_PORT1_PEMDIOSR_union_tag { /* Port external MDIO status register */
  vuint32_t R;
  struct {
    vuint32_t BSY:1;                   /* Global MDIO busy */
    vuint32_t _unused_1:7;
    vuint32_t WHT_LIST:5;              /* PHY white list */
    vuint32_t _unused_13:2;
    vuint32_t WHT_LIST_ENA:1;          /* PHY white list enable */
    vuint32_t PORT_ID:3;               /* Port ID */
    vuint32_t REQ_TYPE:1;              /* Port ID */
    vuint32_t _unused_20:12;
  } B;
} SW_ETH_MAC_PORT1_PEMDIOSR_tag;

typedef union SW_ETH_MAC_PORT1_PPSCR_union_tag { /* PHY status configuration register */
  vuint32_t R;
  struct {
    vuint32_t BSY:1;                   /* MDIO busy */
    vuint32_t MDIO_RD_ER:1;            /* MDIO read error */
    vuint32_t _unused_2:14;
    vuint32_t STATUS_INTERVAL:16;      /* PHY status read interval */
  } B;
} SW_ETH_MAC_PORT1_PPSCR_tag;

typedef union SW_ETH_MAC_PORT1_PPSCTRLR_union_tag { /* Port PHY status control register */
  vuint32_t R;
  struct {
    vuint32_t DEV_ADDR:5;              /* 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
    vuint32_t PORT_ADDR:5;             /* 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
    vuint32_t _unused_10:22;
  } B;
} SW_ETH_MAC_PORT1_PPSCTRLR_tag;

typedef union SW_ETH_MAC_PORT1_PPSDR_union_tag { /* Port PHY status data register */
  vuint32_t R;
  struct {
    vuint32_t MDIO_DATA:16;            /* 16-bit MDIO data */
    vuint32_t CURR_CNT:16;             /* Current count */
  } B;
} SW_ETH_MAC_PORT1_PPSDR_tag;

typedef union SW_ETH_MAC_PORT1_PPSRAR_union_tag { /* Port PHY status register address register */
  vuint32_t R;
  struct {
    vuint32_t REGADDR:16;              /* MDIO PHY register address. Address of the register within the Clause 45 PHY device from which data is to be read. */
    vuint32_t _unused_16:16;
  } B;
} SW_ETH_MAC_PORT1_PPSRAR_tag;

typedef union SW_ETH_MAC_PORT1_PPSER_union_tag { /* Port PHY status event register */
  vuint32_t R;
  struct {
    vuint32_t STATUS_EVENT_HL:16;      /* Status event high-to-low. Set to 1 if a 1->0 transition on a corresponding data bit has occurred. Write 1 to clear. */
    vuint32_t STATUS_EVENT_LH:16;      /* Status event low-to-high. Set to 1 if a 0->1 transition on a corresponding data bit has occurred. Write 1 to clear. */
  } B;
} SW_ETH_MAC_PORT1_PPSER_tag;

typedef union SW_ETH_MAC_PORT1_PPSMR_union_tag { /* Port PHY status mask register */
  vuint32_t R;
  struct {
    vuint32_t STATUS_MASK_HL:16;       /* Status high-to-low mask. If set to 1, assert an interrupt if the corresponding event bit is set. */
    vuint32_t STATUS_MASK_LH:16;       /* Status mask low-to-high. If set to 1, assert an interrupt if the corresponding event bit is set. */
  } B;
} SW_ETH_MAC_PORT1_PPSMR_tag;

struct SW_ETH_MAC_PORT1_tag {
  uint8_t SW_ETH_MAC_PORT1_reserved0[8];
  SW_ETH_MAC_PORT1_PM0_COMMAND_CONFIG_tag PM0_COMMAND_CONFIG; /* Port MAC 0 Command and Configuration Register */
  SW_ETH_MAC_PORT1_PM0_MAC_ADDR_0_tag PM0_MAC_ADDR_0; /* Port MAC 0 MAC Address Register 0 */
  SW_ETH_MAC_PORT1_PM0_MAC_ADDR_1_tag PM0_MAC_ADDR_1; /* Port MAC 0 MAC Address Register 1 */
  SW_ETH_MAC_PORT1_PM0_MAXFRM_tag PM0_MAXFRM; /* Port MAC 0 Maximum Frame Length Register */
  uint8_t SW_ETH_MAC_PORT1_reserved1[40];
  SW_ETH_MAC_PORT1_PM0_IEVENT_tag PM0_IEVENT; /* Port MAC 0 Interrupt Event Register */
  uint8_t SW_ETH_MAC_PORT1_reserved2[8];
  SW_ETH_MAC_PORT1_PM0_IMASK_tag PM0_IMASK; /* Port MAC 0 Interrupt Mask Register(INT_MASK) */
  uint8_t SW_ETH_MAC_PORT1_reserved3[4];
  SW_ETH_MAC_PORT1_PM0_PAUSE_QUANTA_tag PM0_PAUSE_QUANTA; /* Port MAC 0 Pause Quanta Register */
  uint8_t SW_ETH_MAC_PORT1_reserved4[12];
  SW_ETH_MAC_PORT1_PM0_PAUSE_THRESH_tag PM0_PAUSE_THRESH; /* Port MAC 0 Pause Quanta Threshold Register */
  uint8_t SW_ETH_MAC_PORT1_reserved5[12];
  SW_ETH_MAC_PORT1_PM0_RX_PAUSE_STATUS_tag PM0_RX_PAUSE_STATUS; /* Port MAC 0 Receive Pause Status Register */
  uint8_t SW_ETH_MAC_PORT1_reserved6[72];
  SW_ETH_MAC_PORT1_PM0_SINGLE_STEP_tag PM0_SINGLE_STEP; /* Port MAC 0 IEEE1588 Single-Step Control Register */
  uint8_t SW_ETH_MAC_PORT1_reserved7[12];
  SW_ETH_MAC_PORT1_PM0_HD_BACKOFF_ENTROPY_tag PM0_HD_BACKOFF_ENTROPY; /* Port MAC 0 half-duplex backoff entropy register */
  uint8_t SW_ETH_MAC_PORT1_reserved8[12];
  SW_ETH_MAC_PORT1_PM0_STATN_CONFIG_tag PM0_STATN_CONFIG; /* Port MAC 0 Statistics Configuration Register */
  uint8_t SW_ETH_MAC_PORT1_reserved9[28];
  SW_ETH_MAC_PORT1_PM0_REOCTN_tag PM0_REOCTN; /* Port MAC 0 Receive Ethernet Octets Counter(etherStatsOctetsn) */
  SW_ETH_MAC_PORT1_PM0_ROCTN_tag PM0_ROCTN; /* Port MAC 0 Receive Octets Counter(iflnOctetsn) */
  uint8_t SW_ETH_MAC_PORT1_reserved10[8];
  SW_ETH_MAC_PORT1_PM0_RXPFN_tag PM0_RXPFN; /* Port MAC 0 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  SW_ETH_MAC_PORT1_PM0_RFRMN_tag PM0_RFRMN; /* Port MAC 0 Receive Frame Counter Register(aFramesReceivedOKn) */
  SW_ETH_MAC_PORT1_PM0_RFCSN_tag PM0_RFCSN; /* Port MAC 0 Receive Frame Check Sequence Error Counter Register() */
  SW_ETH_MAC_PORT1_PM0_RVLANN_tag PM0_RVLANN; /* Port MAC 0 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
  SW_ETH_MAC_PORT1_PM0_RERRN_tag PM0_RERRN; /* Port MAC 0 Receive Frame Error Counter Register(ifInErrorsn) */
  SW_ETH_MAC_PORT1_PM0_RUCAN_tag PM0_RUCAN; /* Port MAC 0 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
  SW_ETH_MAC_PORT1_PM0_RMCAN_tag PM0_RMCAN; /* Port MAC 0 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
  SW_ETH_MAC_PORT1_PM0_RBCAN_tag PM0_RBCAN; /* Port MAC 0 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
  SW_ETH_MAC_PORT1_PM0_RDRPN_tag PM0_RDRPN; /* Port MAC 0 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
  SW_ETH_MAC_PORT1_PM0_RPKTN_tag PM0_RPKTN; /* Port MAC 0 Receive Packets Counter Register(etherStatsPktsn) */
  SW_ETH_MAC_PORT1_PM0_RUNDN_tag PM0_RUNDN; /* Port MAC 0 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  SW_ETH_MAC_PORT1_PM0_R64N_tag PM0_R64N; /* Port MAC 0 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
  SW_ETH_MAC_PORT1_PM0_R127N_tag PM0_R127N; /* Port MAC 0 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
  SW_ETH_MAC_PORT1_PM0_R255N_tag PM0_R255N; /* Port MAC 0 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
  SW_ETH_MAC_PORT1_PM0_R511N_tag PM0_R511N; /* Port MAC 0 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
  SW_ETH_MAC_PORT1_PM0_R1023N_tag PM0_R1023N; /* Port MAC 0 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
  SW_ETH_MAC_PORT1_PM0_R1522N_tag PM0_R1522N; /* Port MAC 0 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
  SW_ETH_MAC_PORT1_PM0_R1523XN_tag PM0_R1523XN; /* Port MAC 0 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
  SW_ETH_MAC_PORT1_PM0_ROVRN_tag PM0_ROVRN; /* Port MAC 0 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
  SW_ETH_MAC_PORT1_PM0_RJBRN_tag PM0_RJBRN; /* Port MAC 0 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
  SW_ETH_MAC_PORT1_PM0_RFRGN_tag PM0_RFRGN; /* Port MAC 0 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
  SW_ETH_MAC_PORT1_PM0_RCNPN_tag PM0_RCNPN; /* Port MAC 0 Receive Control Packet Counter Register */
  SW_ETH_MAC_PORT1_PM0_RDRNTPN_tag PM0_RDRNTPN; /* Port MAC 0 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
  uint8_t SW_ETH_MAC_PORT1_reserved11[48];
  SW_ETH_MAC_PORT1_PM0_TEOCTN_tag PM0_TEOCTN; /* Port MAC 0 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
  SW_ETH_MAC_PORT1_PM0_TOCTN_tag PM0_TOCTN; /* Port MAC 0 Transmit Octets Counter Register(ifOutOctetsn) */
  uint8_t SW_ETH_MAC_PORT1_reserved12[8];
  SW_ETH_MAC_PORT1_PM0_TXPFN_tag PM0_TXPFN; /* Port MAC 0 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  SW_ETH_MAC_PORT1_PM0_TFRMN_tag PM0_TFRMN; /* Port MAC 0 Transmit Frame Counter Register(aFramesTransmittedOKn) */
  SW_ETH_MAC_PORT1_PM0_TFCSN_tag PM0_TFCSN; /* Port MAC 0 Transmit Frame Check Sequence Error Counter Register() */
  SW_ETH_MAC_PORT1_PM0_TVLANN_tag PM0_TVLANN; /* Port MAC 0 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
  SW_ETH_MAC_PORT1_PM0_TERRN_tag PM0_TERRN; /* Port MAC 0 Transmit Frame Error Counter Register(ifOutErrorsn) */
  SW_ETH_MAC_PORT1_PM0_TUCAN_tag PM0_TUCAN; /* Port MAC 0 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
  SW_ETH_MAC_PORT1_PM0_TMCAN_tag PM0_TMCAN; /* Port MAC 0 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
  SW_ETH_MAC_PORT1_PM0_TBCAN_tag PM0_TBCAN; /* Port MAC 0 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
  uint8_t SW_ETH_MAC_PORT1_reserved13[8];
  SW_ETH_MAC_PORT1_PM0_TPKTN_tag PM0_TPKTN; /* Port MAC 0 Transmit Packets Counter Register(etherStatsPktsn) */
  SW_ETH_MAC_PORT1_PM0_TUNDN_tag PM0_TUNDN; /* Port MAC 0 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  SW_ETH_MAC_PORT1_PM0_T64N_tag PM0_T64N; /* Port MAC 0 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
  SW_ETH_MAC_PORT1_PM0_T127N_tag PM0_T127N; /* Port MAC 0 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
  SW_ETH_MAC_PORT1_PM0_T255N_tag PM0_T255N; /* Port MAC 0 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
  SW_ETH_MAC_PORT1_PM0_T511N_tag PM0_T511N; /* Port MAC 0 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
  SW_ETH_MAC_PORT1_PM0_T1023N_tag PM0_T1023N; /* Port MAC 0 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
  SW_ETH_MAC_PORT1_PM0_T1522N_tag PM0_T1522N; /* Port MAC 0 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
  SW_ETH_MAC_PORT1_PM0_T1523XN_tag PM0_T1523XN; /* Port MAC 0 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
  uint8_t SW_ETH_MAC_PORT1_reserved14[24];
  SW_ETH_MAC_PORT1_PM0_TCNPN_tag PM0_TCNPN; /* Port MAC 0 Transmit Control Packet Counter Register */
  uint8_t SW_ETH_MAC_PORT1_reserved15[8];
  SW_ETH_MAC_PORT1_PM0_TDFRN_tag PM0_TDFRN; /* Port MAC 0 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
  SW_ETH_MAC_PORT1_PM0_TMCOLN_tag PM0_TMCOLN; /* Port MAC 0 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
  SW_ETH_MAC_PORT1_PM0_TSCOLN_tag PM0_TSCOLN; /* Port MAC 0 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
  SW_ETH_MAC_PORT1_PM0_TLCOLN_tag PM0_TLCOLN; /* Port MAC 0 Transmit Late Collision Counter(aLateCollisions) Register */
  SW_ETH_MAC_PORT1_PM0_TECOLN_tag PM0_TECOLN; /* Port MAC 0 Transmit Excessive Collisions Counter Register */
  uint8_t SW_ETH_MAC_PORT1_reserved16[8];
  SW_ETH_MAC_PORT1_PM0_IF_MODE_tag PM0_IF_MODE; /* Port MAC 0 Interface Mode Control Register */
  uint8_t SW_ETH_MAC_PORT1_reserved17[260];
  SW_ETH_MAC_PORT1_PM1_COMMAND_CONFIG_tag PM1_COMMAND_CONFIG; /* Port MAC 1 Command and Configuration Register */
  SW_ETH_MAC_PORT1_PM1_MAC_ADDR_0_tag PM1_MAC_ADDR_0; /* Port MAC 1 MAC Address Register 0 */
  SW_ETH_MAC_PORT1_PM1_MAC_ADDR_1_tag PM1_MAC_ADDR_1; /* Port MAC 1 MAC Address Register 1 */
  SW_ETH_MAC_PORT1_PM1_MAXFRM_tag PM1_MAXFRM; /* Port MAC 1 Maximum Frame Length Register */
  uint8_t SW_ETH_MAC_PORT1_reserved18[40];
  SW_ETH_MAC_PORT1_PM1_IEVENT_tag PM1_IEVENT; /* Port MAC 1 Interrupt Event Register */
  uint8_t SW_ETH_MAC_PORT1_reserved19[8];
  SW_ETH_MAC_PORT1_PM1_IMASK_tag PM1_IMASK; /* Port MAC 1 Interrupt Mask Register(INT_MASK) */
  uint8_t SW_ETH_MAC_PORT1_reserved20[4];
  SW_ETH_MAC_PORT1_PM1_PAUSE_QUANTA_tag PM1_PAUSE_QUANTA; /* Port MAC 1 Pause Quanta Register */
  uint8_t SW_ETH_MAC_PORT1_reserved21[12];
  SW_ETH_MAC_PORT1_PM1_PAUSE_THRESH_tag PM1_PAUSE_THRESH; /* Port MAC 1 Pause Quanta Threshold Register */
  uint8_t SW_ETH_MAC_PORT1_reserved22[12];
  SW_ETH_MAC_PORT1_PM1_RX_PAUSE_STATUS_tag PM1_RX_PAUSE_STATUS; /* Port MAC 1 Receive Pause Status Register */
  uint8_t SW_ETH_MAC_PORT1_reserved23[72];
  SW_ETH_MAC_PORT1_PM1_SINGLE_STEP_tag PM1_SINGLE_STEP; /* Port MAC 1 IEEE1588 Single-Step Control Register */
  uint8_t SW_ETH_MAC_PORT1_reserved24[12];
  SW_ETH_MAC_PORT1_PM1_HD_BACKOFF_ENTROPY_tag PM1_HD_BACKOFF_ENTROPY; /* Port MAC 1 half-duplex backoff entropy register */
  uint8_t SW_ETH_MAC_PORT1_reserved25[12];
  SW_ETH_MAC_PORT1_PM1_STATN_CONFIG_tag PM1_STATN_CONFIG; /* Port MAC 1 Statistics Configuration Register */
  uint8_t SW_ETH_MAC_PORT1_reserved26[28];
  SW_ETH_MAC_PORT1_PM1_REOCTN_tag PM1_REOCTN; /* Port MAC 1 Receive Ethernet Octets Counter(etherStatsOctetsn) */
  SW_ETH_MAC_PORT1_PM1_ROCTN_tag PM1_ROCTN; /* Port MAC 1 Receive Octets Counter(iflnOctetsn) */
  uint8_t SW_ETH_MAC_PORT1_reserved27[8];
  SW_ETH_MAC_PORT1_PM1_RXPFN_tag PM1_RXPFN; /* Port MAC 1 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  SW_ETH_MAC_PORT1_PM1_RFRMN_tag PM1_RFRMN; /* Port MAC 1 Receive Frame Counter Register(aFramesReceivedOKn) */
  SW_ETH_MAC_PORT1_PM1_RFCSN_tag PM1_RFCSN; /* Port MAC 1 Receive Frame Check Sequence Error Counter Register() */
  SW_ETH_MAC_PORT1_PM1_RVLANN_tag PM1_RVLANN; /* Port MAC 1 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
  SW_ETH_MAC_PORT1_PM1_RERRN_tag PM1_RERRN; /* Port MAC 1 Receive Frame Error Counter Register(ifInErrorsn) */
  SW_ETH_MAC_PORT1_PM1_RUCAN_tag PM1_RUCAN; /* Port MAC 1 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
  SW_ETH_MAC_PORT1_PM1_RMCAN_tag PM1_RMCAN; /* Port MAC 1 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
  SW_ETH_MAC_PORT1_PM1_RBCAN_tag PM1_RBCAN; /* Port MAC 1 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
  SW_ETH_MAC_PORT1_PM1_RDRPN_tag PM1_RDRPN; /* Port MAC 1 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
  SW_ETH_MAC_PORT1_PM1_RPKTN_tag PM1_RPKTN; /* Port MAC 1 Receive Packets Counter Register(etherStatsPktsn) */
  SW_ETH_MAC_PORT1_PM1_RUNDN_tag PM1_RUNDN; /* Port MAC 1 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  SW_ETH_MAC_PORT1_PM1_R64N_tag PM1_R64N; /* Port MAC 1 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
  SW_ETH_MAC_PORT1_PM1_R127N_tag PM1_R127N; /* Port MAC 1 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
  SW_ETH_MAC_PORT1_PM1_R255N_tag PM1_R255N; /* Port MAC 1 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
  SW_ETH_MAC_PORT1_PM1_R511N_tag PM1_R511N; /* Port MAC 1 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
  SW_ETH_MAC_PORT1_PM1_R1023N_tag PM1_R1023N; /* Port MAC 1 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
  SW_ETH_MAC_PORT1_PM1_R1522N_tag PM1_R1522N; /* Port MAC 1 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
  SW_ETH_MAC_PORT1_PM1_R1523XN_tag PM1_R1523XN; /* Port MAC 1 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
  SW_ETH_MAC_PORT1_PM1_ROVRN_tag PM1_ROVRN; /* Port MAC 1 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
  SW_ETH_MAC_PORT1_PM1_RJBRN_tag PM1_RJBRN; /* Port MAC 1 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
  SW_ETH_MAC_PORT1_PM1_RFRGN_tag PM1_RFRGN; /* Port MAC 1 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
  SW_ETH_MAC_PORT1_PM1_RCNPN_tag PM1_RCNPN; /* Port MAC 1 Receive Control Packet Counter Register */
  SW_ETH_MAC_PORT1_PM1_RDRNTPN_tag PM1_RDRNTPN; /* Port MAC 1 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
  uint8_t SW_ETH_MAC_PORT1_reserved28[48];
  SW_ETH_MAC_PORT1_PM1_TEOCTN_tag PM1_TEOCTN; /* Port MAC 1 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
  SW_ETH_MAC_PORT1_PM1_TOCTN_tag PM1_TOCTN; /* Port MAC 1 Transmit Octets Counter Register(ifOutOctetsn) */
  uint8_t SW_ETH_MAC_PORT1_reserved29[8];
  SW_ETH_MAC_PORT1_PM1_TXPFN_tag PM1_TXPFN; /* Port MAC 1 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
  SW_ETH_MAC_PORT1_PM1_TFRMN_tag PM1_TFRMN; /* Port MAC 1 Transmit Frame Counter Register(aFramesTransmittedOKn) */
  SW_ETH_MAC_PORT1_PM1_TFCSN_tag PM1_TFCSN; /* Port MAC 1 Transmit Frame Check Sequence Error Counter Register() */
  SW_ETH_MAC_PORT1_PM1_TVLANN_tag PM1_TVLANN; /* Port MAC 1 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
  SW_ETH_MAC_PORT1_PM1_TERRN_tag PM1_TERRN; /* Port MAC 1 Transmit Frame Error Counter Register(ifOutErrorsn) */
  SW_ETH_MAC_PORT1_PM1_TUCAN_tag PM1_TUCAN; /* Port MAC 1 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
  SW_ETH_MAC_PORT1_PM1_TMCAN_tag PM1_TMCAN; /* Port MAC 1 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
  SW_ETH_MAC_PORT1_PM1_TBCAN_tag PM1_TBCAN; /* Port MAC 1 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
  uint8_t SW_ETH_MAC_PORT1_reserved30[8];
  SW_ETH_MAC_PORT1_PM1_TPKTN_tag PM1_TPKTN; /* Port MAC 1 Transmit Packets Counter Register(etherStatsPktsn) */
  SW_ETH_MAC_PORT1_PM1_TUNDN_tag PM1_TUNDN; /* Port MAC 1 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
  SW_ETH_MAC_PORT1_PM1_T64N_tag PM1_T64N; /* Port MAC 1 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
  SW_ETH_MAC_PORT1_PM1_T127N_tag PM1_T127N; /* Port MAC 1 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
  SW_ETH_MAC_PORT1_PM1_T255N_tag PM1_T255N; /* Port MAC 1 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
  SW_ETH_MAC_PORT1_PM1_T511N_tag PM1_T511N; /* Port MAC 1 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
  SW_ETH_MAC_PORT1_PM1_T1023N_tag PM1_T1023N; /* Port MAC 1 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
  SW_ETH_MAC_PORT1_PM1_T1522N_tag PM1_T1522N; /* Port MAC 1 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
  SW_ETH_MAC_PORT1_PM1_T1523XN_tag PM1_T1523XN; /* Port MAC 1 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
  uint8_t SW_ETH_MAC_PORT1_reserved31[24];
  SW_ETH_MAC_PORT1_PM1_TCNPN_tag PM1_TCNPN; /* Port MAC 1 Transmit Control Packet Counter Register */
  uint8_t SW_ETH_MAC_PORT1_reserved32[8];
  SW_ETH_MAC_PORT1_PM1_TDFRN_tag PM1_TDFRN; /* Port MAC 1 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
  SW_ETH_MAC_PORT1_PM1_TMCOLN_tag PM1_TMCOLN; /* Port MAC 1 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
  SW_ETH_MAC_PORT1_PM1_TSCOLN_tag PM1_TSCOLN; /* Port MAC 1 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
  SW_ETH_MAC_PORT1_PM1_TLCOLN_tag PM1_TLCOLN; /* Port MAC 1 Transmit Late Collision Counter(aLateCollisions) Register */
  SW_ETH_MAC_PORT1_PM1_TECOLN_tag PM1_TECOLN; /* Port MAC 1 Transmit Excessive Collisions Counter Register */
  uint8_t SW_ETH_MAC_PORT1_reserved33[8];
  SW_ETH_MAC_PORT1_PM1_IF_MODE_tag PM1_IF_MODE; /* Port MAC 1 Interface Mode Control Register */
  uint8_t SW_ETH_MAC_PORT1_reserved34[252];
  SW_ETH_MAC_PORT1_MAC_MERGE_MMCSR_tag MAC_MERGE_MMCSR; /* Port MAC Merge Control and Status Register */
  uint8_t SW_ETH_MAC_PORT1_reserved35[4];
  SW_ETH_MAC_PORT1_MAC_MERGE_MMFAECR_tag MAC_MERGE_MMFAECR; /* Port MAC Merge Frame Assembly Error Count Register */
  SW_ETH_MAC_PORT1_MAC_MERGE_MMFSECR_tag MAC_MERGE_MMFSECR; /* Port MAC Merge Frame SMD Error Count Register */
  SW_ETH_MAC_PORT1_MAC_MERGE_MMFAOCR_tag MAC_MERGE_MMFAOCR; /* Port MAC Merge Frame Assembly OK Count Register */
  SW_ETH_MAC_PORT1_MAC_MERGE_MMFCRXR_tag MAC_MERGE_MMFCRXR; /* Port MAC Merge Fragment Count RX Register */
  SW_ETH_MAC_PORT1_MAC_MERGE_MMFCTXR_tag MAC_MERGE_MMFCTXR; /* Port MAC Merge Fragment Count TX Register */
  SW_ETH_MAC_PORT1_MAC_MERGE_MMHCR_tag MAC_MERGE_MMHCR; /* Port MAC Merge Hold Count Register */
  uint8_t SW_ETH_MAC_PORT1_reserved36[992];
  SW_ETH_MAC_PORT1_PEMDIOCR_tag PEMDIOCR; /* Port external MDIO configuration register */
  SW_ETH_MAC_PORT1_PEMDIOICR_tag PEMDIOICR; /* Port external MDIO interface control register */
  SW_ETH_MAC_PORT1_PEMDIOIDR_tag PEMDIOIDR; /* Port external MDIO interface data register */
  SW_ETH_MAC_PORT1_PEMDIORAR_tag PEMDIORAR; /* Port external MDIO register address register */
  SW_ETH_MAC_PORT1_PEMDIOSR_tag PEMDIOSR; /* Port external MDIO status register */
  uint8_t SW_ETH_MAC_PORT1_reserved37[12];
  SW_ETH_MAC_PORT1_PPSCR_tag PPSCR;    /* PHY status configuration register */
  SW_ETH_MAC_PORT1_PPSCTRLR_tag PPSCTRLR; /* Port PHY status control register */
  SW_ETH_MAC_PORT1_PPSDR_tag PPSDR;    /* Port PHY status data register */
  SW_ETH_MAC_PORT1_PPSRAR_tag PPSRAR;  /* Port PHY status register address register */
  SW_ETH_MAC_PORT1_PPSER_tag PPSER;    /* Port PHY status event register */
  SW_ETH_MAC_PORT1_PPSMR_tag PPSMR;    /* Port PHY status mask register */
};


/* ============================================================================
   =============================== Module: SW_PORT0 ===========================
   ============================================================================ */

typedef union SW_PORT0_PCAPR_union_tag { /* Port capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t LINK_TYPE:1;
    vuint32_t _unused_5:7;
    vuint32_t NUM_TC:4;
    vuint32_t NUM_Q:4;
    vuint32_t _unused_20:4;
    vuint32_t NUM_CG:4;
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t CBS:1;                   /* Credit Based Shaping */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT0_PCAPR_tag;

typedef union SW_PORT0_PMCAPR_union_tag { /* Port MAC capability register */
  vuint32_t R;
  struct {
    vuint32_t MAC_VAR:3;               /* MAC Variant */
    vuint32_t _unused_3:1;
    vuint32_t EFPAD:2;
    vuint32_t _unused_6:2;
    vuint32_t HD:1;
    vuint32_t FP:2;                    /* Indicates if frame preemption is supported */
    vuint32_t _unused_11:13;
    vuint32_t MII_PROT:4;              /* Indicates the MII protocol supported */
    vuint32_t _unused_28:4;
  } B;
} SW_PORT0_PMCAPR_tag;

typedef union SW_PORT0_PIOCAPR_union_tag { /* Port I/O capability register */
  vuint32_t R;
  struct {
    vuint32_t PCS_PROT:16;             /* PCS protocols supported */
    vuint32_t _unused_16:7;
    vuint32_t _unused_23:1;
    vuint32_t IO_VAR:4;                /* IO Variants supported */
    vuint32_t EMDIO:1;
    vuint32_t _unused_29:1;
    vuint32_t REVMII_RATE:1;           /* RevMII MII rate */
    vuint32_t REVMII:1;                /* Reverse Mode Device Configuration */
  } B;
} SW_PORT0_PIOCAPR_tag;

typedef union SW_PORT0_PCR_union_tag { /* Port configuration register */
  vuint32_t R;
  struct {
    vuint32_t HDR_FMT:1;
    vuint32_t _unused_1:3;
    vuint32_t L2DOSE:1;
    vuint32_t _unused_5:3;
    vuint32_t TIMER_CS:1;
    vuint32_t _unused_9:3;
    vuint32_t FCSEA:1;                 /* FCS Error Action */
    vuint32_t _unused_13:3;
    vuint32_t PSPEED:14;
    vuint32_t _unused_30:2;
  } B;
} SW_PORT0_PCR_tag;

typedef union SW_PORT0_PMAR0_union_tag { /* Port MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} SW_PORT0_PMAR0_tag;

typedef union SW_PORT0_PMAR1_union_tag { /* Port MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT0_PMAR1_tag;

typedef union SW_PORT0_PTAR_union_tag { /* Port TPID acceptance register */
  vuint32_t R;
  struct {
    vuint32_t OVTPIDL:4;
    vuint32_t IVTPIDL:4;
    vuint32_t _unused_8:24;
  } B;
} SW_PORT0_PTAR_tag;

typedef union SW_PORT0_PQOSMR_union_tag { /* Port QoS mode register */
  vuint32_t R;
  struct {
    vuint32_t VS:1;
    vuint32_t VE:1;
    vuint32_t DDRf:2;
    vuint32_t DIPV:3;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:8;
    vuint32_t VQMP:4;                  /* Mapping profile index */
    vuint32_t QVMP:4;                  /* Mapping profile index */
    vuint32_t _unused_24:8;
  } B;
} SW_PORT0_PQOSMR_tag;

typedef union SW_PORT0_PQOR_union_tag { /* Port Queue Operational register */
  vuint32_t R;
  struct {
    vuint32_t Q0S:1;
    vuint32_t Q1S:1;
    vuint32_t Q2S:1;
    vuint32_t Q3S:1;
    vuint32_t Q4S:1;
    vuint32_t Q5S:1;
    vuint32_t Q6S:1;
    vuint32_t Q7S:1;
    vuint32_t _unused_8:24;
  } B;
} SW_PORT0_PQOR_tag;

typedef union SW_PORT0_PPCR_union_tag { /* Port parser configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t L1PFS:5;
    vuint32_t _unused_6:3;
    vuint32_t L2PFS:5;
    vuint32_t _unused_14:2;
    vuint32_t L3HFP:1;                 /* L3 header fields present */
    vuint32_t L3PFS:5;                 /* L3 payload fields size in bytes */
    vuint32_t _unused_22:2;
    vuint32_t L4HFP:1;                 /* L4 Header fields present */
    vuint32_t L4PFS:5;                 /* L4 payload fields size in bytes */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT0_PPCR_tag;

typedef union SW_PORT0_PIPFCR_union_tag { /* Port ingress port filter configuration register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t _unused_1:31;
  } B;
} SW_PORT0_PIPFCR_tag;

typedef union SW_PORT0_PSGCR_union_tag { /* Port stream gate configuration register */
  vuint32_t R;
  struct {
    vuint32_t PDELAY:24;
    vuint32_t _unused_24:7;
    vuint32_t OGC:1;                   /* Stream Gate Open Gate Check mode */
  } B;
} SW_PORT0_PSGCR_tag;

typedef union SW_PORT0_POR_union_tag { /* Port operational register */
  vuint32_t R;
  struct {
    vuint32_t TXDIS:1;                 /* Tx Disable. */
    vuint32_t RXDIS:1;                 /* Rx Disable. */
    vuint32_t _unused_2:30;
  } B;
} SW_PORT0_POR_tag;

typedef union SW_PORT0_PSR_union_tag { /* Port status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;               /* Transmit busy. */
    vuint32_t RX_BUSY:1;               /* Receive busy. */
    vuint32_t _unused_2:30;
  } B;
} SW_PORT0_PSR_tag;

typedef union SW_PORT0_PRXSDUOR_union_tag { /* Port receive SDU overhead register */
  vuint32_t R;
  struct {
    vuint32_t PPDU_BCO:5;              /* PPDU Byte count overhead */
    vuint32_t _unused_5:3;
    vuint32_t MACSEC_BCO:5;            /* MACsec byte count overhead */
    vuint32_t _unused_13:19;
  } B;
} SW_PORT0_PRXSDUOR_tag;

typedef union SW_PORT0_PTXSDUOR_union_tag { /* Port transmit SDU overhead register */
  vuint32_t R;
  struct {
    vuint32_t PPDU_BCO:5;              /* PPDU Byte count overhead */
    vuint32_t _unused_5:3;
    vuint32_t MACSEC_BCO:5;            /* MACsec byte count overhead */
    vuint32_t _unused_13:19;
  } B;
} SW_PORT0_PTXSDUOR_tag;

typedef union SW_PORT0_PTGSCR_union_tag { /* Port time gate scheduling control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t TGE:1;                   /* Time Gating Enable */
  } B;
} SW_PORT0_PTGSCR_tag;

typedef union SW_PORT0_PTGAGLSR_union_tag { /* Port time gate scheduling admin gate list status register */
  vuint32_t R;
  struct {
    vuint32_t TG:1;
    vuint32_t CFG_PEND:1;
    vuint32_t _unused_2:30;
  } B;
} SW_PORT0_PTGAGLSR_tag;

typedef union SW_PORT0_PTGAGLLR_union_tag { /* Port time gate scheduling admin gate list length register */
  vuint32_t R;
  struct {
    vuint32_t ADMIN_GATE_LIST_LENGTH:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT0_PTGAGLLR_tag;

typedef union SW_PORT0_PTGOGLLR_union_tag { /* Port time gating operational gate list length register */
  vuint32_t R;
  struct {
    vuint32_t OPER_GATE_LIST_LENGTH:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT0_PTGOGLLR_tag;

typedef union SW_PORT0_PTGSHAR_union_tag { /* Port time gate scheduling hold advance register */
  vuint32_t R;
  struct {
    vuint32_t HOLDADVANCE:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT0_PTGSHAR_tag;

typedef union SW_PORT0_PTGSRAR_union_tag { /* Port time gate scheduling release advance register */
  vuint32_t R;
  struct {
    vuint32_t RELEASEADVANCE:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT0_PTGSRAR_tag;

typedef union SW_PORT0_PTGSHCR_union_tag { /* Port time gate scheduling hold configuration register */
  vuint32_t R;
  struct {
    vuint32_t HOLD_SKEW:20;
    vuint32_t _unused_20:12;
  } B;
} SW_PORT0_PTGSHCR_tag;

typedef union SW_PORT0_PFPCR_union_tag { /* Port frame preemption configuration register */
  vuint32_t R;
  struct {
    vuint32_t FPE_TC0:1;
    vuint32_t FPE_TC1:1;
    vuint32_t FPE_TC2:1;
    vuint32_t FPE_TC3:1;
    vuint32_t FPE_TC4:1;
    vuint32_t FPE_TC5:1;
    vuint32_t FPE_TC6:1;
    vuint32_t FPE_TC7:1;
    vuint32_t _unused_8:24;
  } B;
} SW_PORT0_PFPCR_tag;

typedef union SW_PORT0_PRXDCR_union_tag { /* Port Rx discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT0_PRXDCR_tag;

typedef union SW_PORT0_PRXDCRR0_union_tag { /* Port Rx discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCDR:1;
    vuint32_t SMREDR:1;
    vuint32_t RXDISDR:1;
    vuint32_t IPFDR:1;
    vuint32_t RPDR:1;
    vuint32_t ISFDR:1;
    vuint32_t SGCDR:1;
    vuint32_t SGOEDR:1;
    vuint32_t MSDUEDR:1;
    vuint32_t FMMEDR:1;
    vuint32_t CMDR:1;
    vuint32_t ITEDR:1;
    vuint32_t ECCEDR:1;
    vuint32_t _unused_13:1;
    vuint32_t L2DOSDR:1;
    vuint32_t _unused_15:1;
    vuint32_t PEDR:1;
    vuint32_t NODESTDR:1;              /* No Destination Discard Reason */
    vuint32_t _unused_18:14;
  } B;
} SW_PORT0_PRXDCRR0_tag;

typedef union SW_PORT0_PRXDCRR1_union_tag { /* Port Rx discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:16;
    vuint32_t _unused_16:12;
    vuint32_t TT:4;
  } B;
} SW_PORT0_PRXDCRR1_tag;

typedef union SW_PORT0_PTXDCR_union_tag { /* Port Tx discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT0_PTXDCR_tag;

typedef union SW_PORT0_PTXDCRR0_union_tag { /* Port Tx discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXDISDR:1;
    vuint32_t ECCEDR:1;                /* ECC Error Discard Reason */
    vuint32_t PEDR:1;                  /* Parity Error Discard Reason */
    vuint32_t _unused_3:1;
    vuint32_t TGSFTLDR:1;
    vuint32_t FMMDR:1;
    vuint32_t TXDISEDR:1;
    vuint32_t MSDUEDR:1;
    vuint32_t QCONGDR:1;
    vuint32_t ITEDR:1;
    vuint32_t INVEQDR:1;
    vuint32_t SQRTNSQDR:1;
    vuint32_t _unused_12:1;
    vuint32_t SQRRDR:1;
    vuint32_t SQRDDR:1;
    vuint32_t SMREDR:1;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT0_PTXDCRR0_tag;

typedef union SW_PORT0_PTXDCRR1_union_tag { /* Port Tx discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:16;
    vuint32_t _unused_16:12;
    vuint32_t TT:4;
  } B;
} SW_PORT0_PTXDCRR1_tag;

typedef union SW_PORT0_TCT_NUM_PTGSTCSR_union_tag { /* Port time gate scheduling traffic class 0 status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:15;
    vuint32_t LH_STATE:1;
    vuint32_t _unused_17:15;
  } B;
} SW_PORT0_TCT_NUM_PTGSTCSR_tag;

typedef union SW_PORT0_TCT_NUM_PTCTMSDUR_union_tag { /* Port traffic class 0 transmit maximum SDU register */
  vuint32_t R;
  struct {
    vuint32_t MAXSDU:16;               /* Tx Maximum Service Data Unit Size */
    vuint32_t SDU_TYPE:2;              /* SDU type */
    vuint32_t _unused_18:6;
    vuint32_t SF_MAXSDU_DIS:1;
    vuint32_t _unused_25:7;
  } B;
} SW_PORT0_TCT_NUM_PTCTMSDUR_tag;

typedef union SW_PORT0_TCT_NUM_PTCCBSR0_union_tag { /* Port transmit traffic class 0 credit based shaper register 0 */
  vuint32_t R;
  struct {
    vuint32_t BW:7;                    /* Bandwidth */
    vuint32_t _unused_7:24;
    vuint32_t CBSE:1;                  /* Credit Based Shaper Enable */
  } B;
} SW_PORT0_TCT_NUM_PTCCBSR0_tag;

typedef union SW_PORT0_TCT_NUM_PTCCBSR1_union_tag { /* Port traffic class 0 credit based shaper register 1 */
  vuint32_t R;
  struct {
    vuint32_t HI_CREDIT:32;            /* hicredit (in credit units) */
  } B;
} SW_PORT0_TCT_NUM_PTCCBSR1_tag;

typedef struct SW_PORT0_TCT_NUM_struct_tag {
  SW_PORT0_TCT_NUM_PTGSTCSR_tag PTGSTCSR; /* Port time gate scheduling traffic class 0 status register */
  uint8_t TCT_NUM_reserved0[4];
  SW_PORT0_TCT_NUM_PTCTMSDUR_tag PTCTMSDUR; /* Port traffic class 0 transmit maximum SDU register */
  uint8_t TCT_NUM_reserved1[4];
  SW_PORT0_TCT_NUM_PTCCBSR0_tag PTCCBSR0; /* Port transmit traffic class 0 credit based shaper register 0 */
  SW_PORT0_TCT_NUM_PTCCBSR1_tag PTCCBSR1; /* Port traffic class 0 credit based shaper register 1 */
  uint8_t TCT_NUM_reserved2[8];
} SW_PORT0_TCT_NUM_tag;

typedef union SW_PORT0_PBPMCR0_union_tag { /* Port buffer pool mapping configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0_INDEX:8;
    vuint32_t IPV1_INDEX:8;
    vuint32_t IPV2_INDEX:8;
    vuint32_t IPV3_INDEX:8;
  } B;
} SW_PORT0_PBPMCR0_tag;

typedef union SW_PORT0_PBPMCR1_union_tag { /* Port buffer pool mapping configuration register 1 */
  vuint32_t R;
  struct {
    vuint32_t IPV4_INDEX:8;
    vuint32_t IPV5_INDEX:8;
    vuint32_t IPV6_INDEX:8;
    vuint32_t IPV7_INDEX:8;
  } B;
} SW_PORT0_PBPMCR1_tag;

typedef union SW_PORT0_PPCPDEIMR_union_tag { /* Port PCP DEI mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPCPMP:4;
    vuint32_t _unused_4:3;
    vuint32_t IPCPMPV:1;
    vuint32_t EPCPMP:4;
    vuint32_t _unused_12:3;
    vuint32_t EPCPMPV:1;
    vuint32_t DR0DEI:1;
    vuint32_t DR1DEI:1;
    vuint32_t DR2DEI:1;
    vuint32_t DR3DEI:1;
    vuint32_t DRME:1;
    vuint32_t _unused_21:11;
  } B;
} SW_PORT0_PPCPDEIMR_tag;

typedef union SW_PORT0_PMCR_union_tag { /* Port mirror configuration register */
  vuint32_t R;
  struct {
    vuint32_t IMIRE:1;
    vuint32_t _unused_1:31;
  } B;
} SW_PORT0_PMCR_tag;

typedef union SW_PORT0_PCTFCR_union_tag { /* Port cut through forwarding configuration register */
  vuint32_t R;
  struct {
    vuint32_t ICTS:1;
    vuint32_t ECTS:1;
    vuint32_t _unused_2:2;
    vuint32_t BSQS:2;
    vuint32_t _unused_6:26;
  } B;
} SW_PORT0_PCTFCR_tag;

typedef union SW_PORT0_PLANIDCR_union_tag { /* Port LANID configuration register */
  vuint32_t R;
  struct {
    vuint32_t LANID:4;                 /* LAN Identifier */
    vuint32_t _unused_4:28;
  } B;
} SW_PORT0_PLANIDCR_tag;

typedef union SW_PORT0_PISIDCR_union_tag { /* Port ingress stream identification configuration register */
  vuint32_t R;
  struct {
    vuint32_t KCPAIR:1;
    vuint32_t KC0EN:1;                 /* Key Construction 0 Enable */
    vuint32_t KC1EN:1;                 /* Key Construction 1 Enable */
    vuint32_t _unused_3:13;
    vuint32_t ISEID:16;
  } B;
} SW_PORT0_PISIDCR_tag;

typedef union SW_PORT0_PFMCR_union_tag { /* Port frame modification configuration register */
  vuint32_t R;
  struct {
    vuint32_t FMMA:1;                  /* Frame Modification Misconfiguration Action */
    vuint32_t _unused_1:31;
  } B;
} SW_PORT0_PFMCR_tag;

typedef union SW_PORT0_PIPV2QMR0_union_tag { /* Port IPV to queue mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0_Q:4;
    vuint32_t IPV1_Q:4;
    vuint32_t IPV2_Q:4;
    vuint32_t IPV3_Q:4;
    vuint32_t IPV4_Q:4;
    vuint32_t IPV5_Q:4;
    vuint32_t IPV6_Q:4;
    vuint32_t IPV7_Q:4;
  } B;
} SW_PORT0_PIPV2QMR0_tag;

typedef union SW_PORT0_PTCMINLR_union_tag { /* Port time capture minimum latency register */
  vuint32_t R;
  struct {
    vuint32_t LATENCY:30;
    vuint32_t COUNT:2;
  } B;
} SW_PORT0_PTCMINLR_tag;

typedef union SW_PORT0_PTCMAXLR_union_tag { /* Port time capture maximum latency register */
  vuint32_t R;
  struct {
    vuint32_t LATENCY:30;
    vuint32_t _unused_30:2;
  } B;
} SW_PORT0_PTCMAXLR_tag;

typedef union SW_PORT0_BPCR_union_tag { /* Bridge port configuration register */
  vuint32_t R;
  struct {
    vuint32_t DYN_LIMIT:16;            /* Dynamic Limit */
    vuint32_t _unused_16:8;
    vuint32_t UUCASTE:1;               /* Unknown Unicast Storm Control Enable */
    vuint32_t UMCASTE:1;               /* Unknown Multicast Storm Control Enable */
    vuint32_t MCASTE:1;                /* Multicast Storm Control Enable */
    vuint32_t BCASTE:1;                /* Broadcast Storm Control Enable */
    vuint32_t STAMVD:1;                /* Station Move Disallow */
    vuint32_t SRCPRND:1;               /* Source Port Pruning Disable */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT0_BPCR_tag;

typedef union SW_PORT0_BPDVR_union_tag { /* Bridge port default VLAN register */
  vuint32_t R;
  struct {
    vuint32_t VID:12;
    vuint32_t DEI:1;
    vuint32_t PCP:3;
    vuint32_t TPID:1;
    vuint32_t _unused_17:3;
    vuint32_t RXTAGA:4;
    vuint32_t RXVAM:1;
    vuint32_t TXTAGA:2;
    vuint32_t _unused_27:5;
  } B;
} SW_PORT0_BPDVR_tag;

typedef union SW_PORT0_BPSTGSR_union_tag { /* Bridge port spanning tree group state register */
  vuint32_t R;
  struct {
    vuint32_t STG_STATE0:2;
    vuint32_t STG_STATE1:2;
    vuint32_t STG_STATE2:2;
    vuint32_t STG_STATE3:2;
    vuint32_t STG_STATE4:2;
    vuint32_t STG_STATE5:2;
    vuint32_t STG_STATE6:2;
    vuint32_t STG_STATE7:2;
    vuint32_t STG_STATE8:2;
    vuint32_t STG_STATE9:2;
    vuint32_t STG_STATE10:2;
    vuint32_t STG_STATE11:2;
    vuint32_t STG_STATE12:2;
    vuint32_t STG_STATE13:2;
    vuint32_t STG_STATE14:2;
    vuint32_t STG_STATE15:2;
  } B;
} SW_PORT0_BPSTGSR_tag;

typedef union SW_PORT0_BPSCR0_union_tag { /* Bridge port storm control register 0 */
  vuint32_t R;
  struct {
    vuint32_t UUCASTRPEID:12;
    vuint32_t _unused_12:4;
    vuint32_t BCASTRPEID:12;
    vuint32_t _unused_28:4;
  } B;
} SW_PORT0_BPSCR0_tag;

typedef union SW_PORT0_BPSCR1_union_tag { /* Bridge port storm control register 1 */
  vuint32_t R;
  struct {
    vuint32_t MCASTRPEID:12;
    vuint32_t _unused_12:4;
    vuint32_t UMCASTRPEID:12;
    vuint32_t _unused_28:4;
  } B;
} SW_PORT0_BPSCR1_tag;

typedef union SW_PORT0_BPOR_union_tag { /* Bridge port operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_DYN:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT0_BPOR_tag;

typedef union SW_PORT0_BPDCR_union_tag { /* Bridge port discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT0_BPDCR_tag;

typedef union SW_PORT0_BPDCRR0_union_tag { /* Bridge port discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t BPACDR:1;
    vuint32_t ISTGSDR:1;
    vuint32_t BPVFLTDR:1;
    vuint32_t MACLNFDR:1;
    vuint32_t _unused_4:3;
    vuint32_t STAMVDDR:1;
    vuint32_t MACFDDDR:1;
    vuint32_t NODESTDR:1;
    vuint32_t IPMFDR:1;
    vuint32_t UFMMDR:1;
    vuint32_t MISCDR:1;
    vuint32_t STRMCTRLDR:1;            /* Discard due to Storm Control Policer Discard Reason */
    vuint32_t _unused_14:18;
  } B;
} SW_PORT0_BPDCRR0_tag;

typedef union SW_PORT0_BPDCRR1_union_tag { /* Bridge port discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:27;
    vuint32_t _unused_27:1;
    vuint32_t TT:4;
  } B;
} SW_PORT0_BPDCRR1_tag;

typedef union SW_PORT0_BPMLFSR_union_tag { /* Bridge port MAC learning failure status register */
  vuint32_t R;
  struct {
    vuint32_t BPMLLRFR:1;              /* Bridge Port MAC Learn Limit Reached Failure Reason */
    vuint32_t FFDBTRFR:1;              /* Full FDB Table Reached Failure Reason */
    vuint32_t HCCLRFR:1;               /* Hash Collision chain limit Reached Failure Reason */
    vuint32_t _unused_3:29;
  } B;
} SW_PORT0_BPMLFSR_tag;

struct SW_PORT0_tag {
  SW_PORT0_PCAPR_tag PCAPR;            /* Port capability register */
  SW_PORT0_PMCAPR_tag PMCAPR;          /* Port MAC capability register */
  SW_PORT0_PIOCAPR_tag PIOCAPR;        /* Port I/O capability register */
  uint8_t SW_PORT0_reserved0[4];
  SW_PORT0_PCR_tag PCR;                /* Port configuration register */
  uint8_t SW_PORT0_reserved1[12];
  SW_PORT0_PMAR0_tag PMAR0;            /* Port MAC address register 0 */
  SW_PORT0_PMAR1_tag PMAR1;            /* Port MAC address register 1 */
  uint8_t SW_PORT0_reserved2[40];
  SW_PORT0_PTAR_tag PTAR;              /* Port TPID acceptance register */
  SW_PORT0_PQOSMR_tag PQOSMR;          /* Port QoS mode register */
  uint8_t SW_PORT0_reserved3[8];
  SW_PORT0_PQOR_tag PQOR;              /* Port Queue Operational register */
  uint8_t SW_PORT0_reserved4[28];
  SW_PORT0_PPCR_tag PPCR;              /* Port parser configuration register */
  SW_PORT0_PIPFCR_tag PIPFCR;          /* Port ingress port filter configuration register */
  uint8_t SW_PORT0_reserved5[24];
  SW_PORT0_PSGCR_tag PSGCR;            /* Port stream gate configuration register */
  uint8_t SW_PORT0_reserved6[92];
  SW_PORT0_POR_tag PORr;               /* Port operational register */
  SW_PORT0_PSR_tag PSR;                /* Port status register */
  SW_PORT0_PRXSDUOR_tag PRXSDUOR;      /* Port receive SDU overhead register */
  SW_PORT0_PTXSDUOR_tag PTXSDUOR;      /* Port transmit SDU overhead register */
  SW_PORT0_PTGSCR_tag PTGSCR;          /* Port time gate scheduling control register */
  SW_PORT0_PTGAGLSR_tag PTGAGLSR;      /* Port time gate scheduling admin gate list status register */
  SW_PORT0_PTGAGLLR_tag PTGAGLLR;      /* Port time gate scheduling admin gate list length register */
  SW_PORT0_PTGOGLLR_tag PTGOGLLR;      /* Port time gating operational gate list length register */
  uint8_t SW_PORT0_reserved7[4];
  SW_PORT0_PTGSHAR_tag PTGSHAR;        /* Port time gate scheduling hold advance register */
  SW_PORT0_PTGSRAR_tag PTGSRAR;        /* Port time gate scheduling release advance register */
  SW_PORT0_PTGSHCR_tag PTGSHCR;        /* Port time gate scheduling hold configuration register */
  uint8_t SW_PORT0_reserved8[4];
  SW_PORT0_PFPCR_tag PFPCR;            /* Port frame preemption configuration register */
  uint8_t SW_PORT0_reserved9[136];
  SW_PORT0_PRXDCR_tag PRXDCR;          /* Port Rx discard count register */
  uint8_t SW_PORT0_reserved10[4];
  SW_PORT0_PRXDCRR0_tag PRXDCRR0;      /* Port Rx discard count reason register 0 */
  SW_PORT0_PRXDCRR1_tag PRXDCRR1;      /* Port Rx discard count reason register 1 */
  uint8_t SW_PORT0_reserved11[16];
  SW_PORT0_PTXDCR_tag PTXDCR;          /* Port Tx discard count register */
  uint8_t SW_PORT0_reserved12[4];
  SW_PORT0_PTXDCRR0_tag PTXDCRR0;      /* Port Tx discard count reason register 0 */
  SW_PORT0_PTXDCRR1_tag PTXDCRR1;      /* Port Tx discard count reason register 1 */
  uint8_t SW_PORT0_reserved13[16];
  SW_PORT0_TCT_NUM_tag TCT_NUM[8];
  uint8_t SW_PORT0_reserved14[256];
  SW_PORT0_PBPMCR0_tag PBPMCR0;        /* Port buffer pool mapping configuration register 0 */
  SW_PORT0_PBPMCR1_tag PBPMCR1;        /* Port buffer pool mapping configuration register 1 */
  uint8_t SW_PORT0_reserved15[48];
  SW_PORT0_PPCPDEIMR_tag PPCPDEIMR;    /* Port PCP DEI mapping register */
  uint8_t SW_PORT0_reserved16[4];
  SW_PORT0_PMCR_tag PMCR;              /* Port mirror configuration register */
  uint8_t SW_PORT0_reserved17[12];
  SW_PORT0_PCTFCR_tag PCTFCR;          /* Port cut through forwarding configuration register */
  uint8_t SW_PORT0_reserved18[4];
  SW_PORT0_PLANIDCR_tag PLANIDCR;      /* Port LANID configuration register */
  uint8_t SW_PORT0_reserved19[4];
  SW_PORT0_PISIDCR_tag PISIDCR;        /* Port ingress stream identification configuration register */
  SW_PORT0_PFMCR_tag PFMCR;            /* Port frame modification configuration register */
  uint8_t SW_PORT0_reserved20[8];
  SW_PORT0_PIPV2QMR0_tag PIPV2QMR0;    /* Port IPV to queue mapping register 0 */
  uint8_t SW_PORT0_reserved21[60];
  SW_PORT0_PTCMINLR_tag PTCMINLR;      /* Port time capture minimum latency register */
  SW_PORT0_PTCMAXLR_tag PTCMAXLR;      /* Port time capture maximum latency register */
  uint8_t SW_PORT0_reserved22[72];
  SW_PORT0_BPCR_tag BPCR;              /* Bridge port configuration register */
  uint8_t SW_PORT0_reserved23[12];
  SW_PORT0_BPDVR_tag BPDVR;            /* Bridge port default VLAN register */
  uint8_t SW_PORT0_reserved24[12];
  SW_PORT0_BPSTGSR_tag BPSTGSR;        /* Bridge port spanning tree group state register */
  uint8_t SW_PORT0_reserved25[4];
  SW_PORT0_BPSCR0_tag BPSCR0;          /* Bridge port storm control register 0 */
  SW_PORT0_BPSCR1_tag BPSCR1;          /* Bridge port storm control register 1 */
  SW_PORT0_BPOR_tag BPOR;              /* Bridge port operational register */
  uint8_t SW_PORT0_reserved26[76];
  SW_PORT0_BPDCR_tag BPDCR;            /* Bridge port discard count register */
  uint8_t SW_PORT0_reserved27[4];
  SW_PORT0_BPDCRR0_tag BPDCRR0;        /* Bridge port discard count reason register 0 */
  SW_PORT0_BPDCRR1_tag BPDCRR1;        /* Bridge port discard count reason register 1 */
  SW_PORT0_BPMLFSR_tag BPMLFSR;        /* Bridge port MAC learning failure status register */
};


/* ============================================================================
   =============================== Module: SW_PORT1 ===========================
   ============================================================================ */

typedef union SW_PORT1_PCAPR_union_tag { /* Port capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t LINK_TYPE:1;
    vuint32_t _unused_5:7;
    vuint32_t NUM_TC:4;
    vuint32_t NUM_Q:4;
    vuint32_t _unused_20:4;
    vuint32_t NUM_CG:4;
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t CBS:1;                   /* Credit Based Shaping */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT1_PCAPR_tag;

typedef union SW_PORT1_PMCAPR_union_tag { /* Port MAC capability register */
  vuint32_t R;
  struct {
    vuint32_t MAC_VAR:3;               /* MAC Variant */
    vuint32_t _unused_3:1;
    vuint32_t EFPAD:2;
    vuint32_t _unused_6:2;
    vuint32_t HD:1;
    vuint32_t FP:2;                    /* Indicates if frame preemption is supported */
    vuint32_t _unused_11:13;
    vuint32_t MII_PROT:4;              /* Indicates the MII protocol supported */
    vuint32_t _unused_28:4;
  } B;
} SW_PORT1_PMCAPR_tag;

typedef union SW_PORT1_PIOCAPR_union_tag { /* Port I/O capability register */
  vuint32_t R;
  struct {
    vuint32_t PCS_PROT:16;             /* PCS protocols supported */
    vuint32_t _unused_16:7;
    vuint32_t _unused_23:1;
    vuint32_t IO_VAR:4;                /* IO Variants supported */
    vuint32_t EMDIO:1;
    vuint32_t _unused_29:1;
    vuint32_t REVMII_RATE:1;           /* RevMII MII rate */
    vuint32_t REVMII:1;                /* Reverse Mode Device Configuration */
  } B;
} SW_PORT1_PIOCAPR_tag;

typedef union SW_PORT1_PCR_union_tag { /* Port configuration register */
  vuint32_t R;
  struct {
    vuint32_t HDR_FMT:1;
    vuint32_t _unused_1:3;
    vuint32_t L2DOSE:1;
    vuint32_t _unused_5:3;
    vuint32_t TIMER_CS:1;
    vuint32_t _unused_9:3;
    vuint32_t FCSEA:1;                 /* FCS Error Action */
    vuint32_t _unused_13:3;
    vuint32_t PSPEED:14;
    vuint32_t _unused_30:2;
  } B;
} SW_PORT1_PCR_tag;

typedef union SW_PORT1_PMAR0_union_tag { /* Port MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} SW_PORT1_PMAR0_tag;

typedef union SW_PORT1_PMAR1_union_tag { /* Port MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT1_PMAR1_tag;

typedef union SW_PORT1_PTAR_union_tag { /* Port TPID acceptance register */
  vuint32_t R;
  struct {
    vuint32_t OVTPIDL:4;
    vuint32_t IVTPIDL:4;
    vuint32_t _unused_8:24;
  } B;
} SW_PORT1_PTAR_tag;

typedef union SW_PORT1_PQOSMR_union_tag { /* Port QoS mode register */
  vuint32_t R;
  struct {
    vuint32_t VS:1;
    vuint32_t VE:1;
    vuint32_t DDRf:2;
    vuint32_t DIPV:3;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:8;
    vuint32_t VQMP:4;                  /* Mapping profile index */
    vuint32_t QVMP:4;                  /* Mapping profile index */
    vuint32_t _unused_24:8;
  } B;
} SW_PORT1_PQOSMR_tag;

typedef union SW_PORT1_PQOR_union_tag { /* Port Queue Operational register */
  vuint32_t R;
  struct {
    vuint32_t Q0S:1;
    vuint32_t Q1S:1;
    vuint32_t Q2S:1;
    vuint32_t Q3S:1;
    vuint32_t Q4S:1;
    vuint32_t Q5S:1;
    vuint32_t Q6S:1;
    vuint32_t Q7S:1;
    vuint32_t _unused_8:24;
  } B;
} SW_PORT1_PQOR_tag;

typedef union SW_PORT1_PPCR_union_tag { /* Port parser configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t L1PFS:5;
    vuint32_t _unused_6:3;
    vuint32_t L2PFS:5;
    vuint32_t _unused_14:2;
    vuint32_t L3HFP:1;                 /* L3 header fields present */
    vuint32_t L3PFS:5;                 /* L3 payload fields size in bytes */
    vuint32_t _unused_22:2;
    vuint32_t L4HFP:1;                 /* L4 Header fields present */
    vuint32_t L4PFS:5;                 /* L4 payload fields size in bytes */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT1_PPCR_tag;

typedef union SW_PORT1_PIPFCR_union_tag { /* Port ingress port filter configuration register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t _unused_1:31;
  } B;
} SW_PORT1_PIPFCR_tag;

typedef union SW_PORT1_PSGCR_union_tag { /* Port stream gate configuration register */
  vuint32_t R;
  struct {
    vuint32_t PDELAY:24;
    vuint32_t _unused_24:7;
    vuint32_t OGC:1;                   /* Stream Gate Open Gate Check mode */
  } B;
} SW_PORT1_PSGCR_tag;

typedef union SW_PORT1_POR_union_tag { /* Port operational register */
  vuint32_t R;
  struct {
    vuint32_t TXDIS:1;                 /* Tx Disable. */
    vuint32_t RXDIS:1;                 /* Rx Disable. */
    vuint32_t _unused_2:30;
  } B;
} SW_PORT1_POR_tag;

typedef union SW_PORT1_PSR_union_tag { /* Port status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;               /* Transmit busy. */
    vuint32_t RX_BUSY:1;               /* Receive busy. */
    vuint32_t _unused_2:30;
  } B;
} SW_PORT1_PSR_tag;

typedef union SW_PORT1_PRXSDUOR_union_tag { /* Port receive SDU overhead register */
  vuint32_t R;
  struct {
    vuint32_t PPDU_BCO:5;              /* PPDU Byte count overhead */
    vuint32_t _unused_5:3;
    vuint32_t MACSEC_BCO:5;            /* MACsec byte count overhead */
    vuint32_t _unused_13:19;
  } B;
} SW_PORT1_PRXSDUOR_tag;

typedef union SW_PORT1_PTXSDUOR_union_tag { /* Port transmit SDU overhead register */
  vuint32_t R;
  struct {
    vuint32_t PPDU_BCO:5;              /* PPDU Byte count overhead */
    vuint32_t _unused_5:3;
    vuint32_t MACSEC_BCO:5;            /* MACsec byte count overhead */
    vuint32_t _unused_13:19;
  } B;
} SW_PORT1_PTXSDUOR_tag;

typedef union SW_PORT1_PTGSCR_union_tag { /* Port time gate scheduling control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t TGE:1;                   /* Time Gating Enable */
  } B;
} SW_PORT1_PTGSCR_tag;

typedef union SW_PORT1_PTGAGLSR_union_tag { /* Port time gate scheduling admin gate list status register */
  vuint32_t R;
  struct {
    vuint32_t TG:1;
    vuint32_t CFG_PEND:1;
    vuint32_t _unused_2:30;
  } B;
} SW_PORT1_PTGAGLSR_tag;

typedef union SW_PORT1_PTGAGLLR_union_tag { /* Port time gate scheduling admin gate list length register */
  vuint32_t R;
  struct {
    vuint32_t ADMIN_GATE_LIST_LENGTH:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT1_PTGAGLLR_tag;

typedef union SW_PORT1_PTGOGLLR_union_tag { /* Port time gating operational gate list length register */
  vuint32_t R;
  struct {
    vuint32_t OPER_GATE_LIST_LENGTH:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT1_PTGOGLLR_tag;

typedef union SW_PORT1_PTGSHAR_union_tag { /* Port time gate scheduling hold advance register */
  vuint32_t R;
  struct {
    vuint32_t HOLDADVANCE:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT1_PTGSHAR_tag;

typedef union SW_PORT1_PTGSRAR_union_tag { /* Port time gate scheduling release advance register */
  vuint32_t R;
  struct {
    vuint32_t RELEASEADVANCE:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT1_PTGSRAR_tag;

typedef union SW_PORT1_PTGSHCR_union_tag { /* Port time gate scheduling hold configuration register */
  vuint32_t R;
  struct {
    vuint32_t HOLD_SKEW:20;
    vuint32_t _unused_20:12;
  } B;
} SW_PORT1_PTGSHCR_tag;

typedef union SW_PORT1_PFPCR_union_tag { /* Port frame preemption configuration register */
  vuint32_t R;
  struct {
    vuint32_t FPE_TC0:1;
    vuint32_t FPE_TC1:1;
    vuint32_t FPE_TC2:1;
    vuint32_t FPE_TC3:1;
    vuint32_t FPE_TC4:1;
    vuint32_t FPE_TC5:1;
    vuint32_t FPE_TC6:1;
    vuint32_t FPE_TC7:1;
    vuint32_t _unused_8:24;
  } B;
} SW_PORT1_PFPCR_tag;

typedef union SW_PORT1_PRXDCR_union_tag { /* Port Rx discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT1_PRXDCR_tag;

typedef union SW_PORT1_PRXDCRR0_union_tag { /* Port Rx discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCDR:1;
    vuint32_t SMREDR:1;
    vuint32_t RXDISDR:1;
    vuint32_t IPFDR:1;
    vuint32_t RPDR:1;
    vuint32_t ISFDR:1;
    vuint32_t SGCDR:1;
    vuint32_t SGOEDR:1;
    vuint32_t MSDUEDR:1;
    vuint32_t FMMEDR:1;
    vuint32_t CMDR:1;
    vuint32_t ITEDR:1;
    vuint32_t ECCEDR:1;
    vuint32_t _unused_13:1;
    vuint32_t L2DOSDR:1;
    vuint32_t _unused_15:1;
    vuint32_t PEDR:1;
    vuint32_t NODESTDR:1;              /* No Destination Discard Reason */
    vuint32_t _unused_18:14;
  } B;
} SW_PORT1_PRXDCRR0_tag;

typedef union SW_PORT1_PRXDCRR1_union_tag { /* Port Rx discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:16;
    vuint32_t _unused_16:12;
    vuint32_t TT:4;
  } B;
} SW_PORT1_PRXDCRR1_tag;

typedef union SW_PORT1_PTXDCR_union_tag { /* Port Tx discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT1_PTXDCR_tag;

typedef union SW_PORT1_PTXDCRR0_union_tag { /* Port Tx discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXDISDR:1;
    vuint32_t ECCEDR:1;                /* ECC Error Discard Reason */
    vuint32_t PEDR:1;                  /* Parity Error Discard Reason */
    vuint32_t _unused_3:1;
    vuint32_t TGSFTLDR:1;
    vuint32_t FMMDR:1;
    vuint32_t TXDISEDR:1;
    vuint32_t MSDUEDR:1;
    vuint32_t QCONGDR:1;
    vuint32_t ITEDR:1;
    vuint32_t INVEQDR:1;
    vuint32_t SQRTNSQDR:1;
    vuint32_t _unused_12:1;
    vuint32_t SQRRDR:1;
    vuint32_t SQRDDR:1;
    vuint32_t SMREDR:1;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT1_PTXDCRR0_tag;

typedef union SW_PORT1_PTXDCRR1_union_tag { /* Port Tx discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:16;
    vuint32_t _unused_16:12;
    vuint32_t TT:4;
  } B;
} SW_PORT1_PTXDCRR1_tag;

typedef union SW_PORT1_TCT_NUM_PTGSTCSR_union_tag { /* Port time gate scheduling traffic class 0 status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:15;
    vuint32_t LH_STATE:1;
    vuint32_t _unused_17:15;
  } B;
} SW_PORT1_TCT_NUM_PTGSTCSR_tag;

typedef union SW_PORT1_TCT_NUM_PTCTMSDUR_union_tag { /* Port traffic class 0 transmit maximum SDU register */
  vuint32_t R;
  struct {
    vuint32_t MAXSDU:16;               /* Tx Maximum Service Data Unit Size */
    vuint32_t SDU_TYPE:2;              /* SDU type */
    vuint32_t _unused_18:6;
    vuint32_t SF_MAXSDU_DIS:1;
    vuint32_t _unused_25:7;
  } B;
} SW_PORT1_TCT_NUM_PTCTMSDUR_tag;

typedef union SW_PORT1_TCT_NUM_PTCCBSR0_union_tag { /* Port transmit traffic class 0 credit based shaper register 0 */
  vuint32_t R;
  struct {
    vuint32_t BW:7;                    /* Bandwidth */
    vuint32_t _unused_7:24;
    vuint32_t CBSE:1;                  /* Credit Based Shaper Enable */
  } B;
} SW_PORT1_TCT_NUM_PTCCBSR0_tag;

typedef union SW_PORT1_TCT_NUM_PTCCBSR1_union_tag { /* Port traffic class 0 credit based shaper register 1 */
  vuint32_t R;
  struct {
    vuint32_t HI_CREDIT:32;            /* hicredit (in credit units) */
  } B;
} SW_PORT1_TCT_NUM_PTCCBSR1_tag;

typedef struct SW_PORT1_TCT_NUM_struct_tag {
  SW_PORT1_TCT_NUM_PTGSTCSR_tag PTGSTCSR; /* Port time gate scheduling traffic class 0 status register */
  uint8_t TCT_NUM_reserved0[4];
  SW_PORT1_TCT_NUM_PTCTMSDUR_tag PTCTMSDUR; /* Port traffic class 0 transmit maximum SDU register */
  uint8_t TCT_NUM_reserved1[4];
  SW_PORT1_TCT_NUM_PTCCBSR0_tag PTCCBSR0; /* Port transmit traffic class 0 credit based shaper register 0 */
  SW_PORT1_TCT_NUM_PTCCBSR1_tag PTCCBSR1; /* Port traffic class 0 credit based shaper register 1 */
  uint8_t TCT_NUM_reserved2[8];
} SW_PORT1_TCT_NUM_tag;

typedef union SW_PORT1_PBPMCR0_union_tag { /* Port buffer pool mapping configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0_INDEX:8;
    vuint32_t IPV1_INDEX:8;
    vuint32_t IPV2_INDEX:8;
    vuint32_t IPV3_INDEX:8;
  } B;
} SW_PORT1_PBPMCR0_tag;

typedef union SW_PORT1_PBPMCR1_union_tag { /* Port buffer pool mapping configuration register 1 */
  vuint32_t R;
  struct {
    vuint32_t IPV4_INDEX:8;
    vuint32_t IPV5_INDEX:8;
    vuint32_t IPV6_INDEX:8;
    vuint32_t IPV7_INDEX:8;
  } B;
} SW_PORT1_PBPMCR1_tag;

typedef union SW_PORT1_PPCPDEIMR_union_tag { /* Port PCP DEI mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPCPMP:4;
    vuint32_t _unused_4:3;
    vuint32_t IPCPMPV:1;
    vuint32_t EPCPMP:4;
    vuint32_t _unused_12:3;
    vuint32_t EPCPMPV:1;
    vuint32_t DR0DEI:1;
    vuint32_t DR1DEI:1;
    vuint32_t DR2DEI:1;
    vuint32_t DR3DEI:1;
    vuint32_t DRME:1;
    vuint32_t _unused_21:11;
  } B;
} SW_PORT1_PPCPDEIMR_tag;

typedef union SW_PORT1_PMCR_union_tag { /* Port mirror configuration register */
  vuint32_t R;
  struct {
    vuint32_t IMIRE:1;
    vuint32_t _unused_1:31;
  } B;
} SW_PORT1_PMCR_tag;

typedef union SW_PORT1_PCTFCR_union_tag { /* Port cut through forwarding configuration register */
  vuint32_t R;
  struct {
    vuint32_t ICTS:1;
    vuint32_t ECTS:1;
    vuint32_t _unused_2:2;
    vuint32_t BSQS:2;
    vuint32_t _unused_6:26;
  } B;
} SW_PORT1_PCTFCR_tag;

typedef union SW_PORT1_PLANIDCR_union_tag { /* Port LANID configuration register */
  vuint32_t R;
  struct {
    vuint32_t LANID:4;                 /* LAN Identifier */
    vuint32_t _unused_4:28;
  } B;
} SW_PORT1_PLANIDCR_tag;

typedef union SW_PORT1_PISIDCR_union_tag { /* Port ingress stream identification configuration register */
  vuint32_t R;
  struct {
    vuint32_t KCPAIR:1;
    vuint32_t KC0EN:1;                 /* Key Construction 0 Enable */
    vuint32_t KC1EN:1;                 /* Key Construction 1 Enable */
    vuint32_t _unused_3:13;
    vuint32_t ISEID:16;
  } B;
} SW_PORT1_PISIDCR_tag;

typedef union SW_PORT1_PFMCR_union_tag { /* Port frame modification configuration register */
  vuint32_t R;
  struct {
    vuint32_t FMMA:1;                  /* Frame Modification Misconfiguration Action */
    vuint32_t _unused_1:31;
  } B;
} SW_PORT1_PFMCR_tag;

typedef union SW_PORT1_PIPV2QMR0_union_tag { /* Port IPV to queue mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0_Q:4;
    vuint32_t IPV1_Q:4;
    vuint32_t IPV2_Q:4;
    vuint32_t IPV3_Q:4;
    vuint32_t IPV4_Q:4;
    vuint32_t IPV5_Q:4;
    vuint32_t IPV6_Q:4;
    vuint32_t IPV7_Q:4;
  } B;
} SW_PORT1_PIPV2QMR0_tag;

typedef union SW_PORT1_PTCMINLR_union_tag { /* Port time capture minimum latency register */
  vuint32_t R;
  struct {
    vuint32_t LATENCY:30;
    vuint32_t COUNT:2;
  } B;
} SW_PORT1_PTCMINLR_tag;

typedef union SW_PORT1_PTCMAXLR_union_tag { /* Port time capture maximum latency register */
  vuint32_t R;
  struct {
    vuint32_t LATENCY:30;
    vuint32_t _unused_30:2;
  } B;
} SW_PORT1_PTCMAXLR_tag;

typedef union SW_PORT1_BPCR_union_tag { /* Bridge port configuration register */
  vuint32_t R;
  struct {
    vuint32_t DYN_LIMIT:16;            /* Dynamic Limit */
    vuint32_t _unused_16:8;
    vuint32_t UUCASTE:1;               /* Unknown Unicast Storm Control Enable */
    vuint32_t UMCASTE:1;               /* Unknown Multicast Storm Control Enable */
    vuint32_t MCASTE:1;                /* Multicast Storm Control Enable */
    vuint32_t BCASTE:1;                /* Broadcast Storm Control Enable */
    vuint32_t STAMVD:1;                /* Station Move Disallow */
    vuint32_t SRCPRND:1;               /* Source Port Pruning Disable */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT1_BPCR_tag;

typedef union SW_PORT1_BPDVR_union_tag { /* Bridge port default VLAN register */
  vuint32_t R;
  struct {
    vuint32_t VID:12;
    vuint32_t DEI:1;
    vuint32_t PCP:3;
    vuint32_t TPID:1;
    vuint32_t _unused_17:3;
    vuint32_t RXTAGA:4;
    vuint32_t RXVAM:1;
    vuint32_t TXTAGA:2;
    vuint32_t _unused_27:5;
  } B;
} SW_PORT1_BPDVR_tag;

typedef union SW_PORT1_BPSTGSR_union_tag { /* Bridge port spanning tree group state register */
  vuint32_t R;
  struct {
    vuint32_t STG_STATE0:2;
    vuint32_t STG_STATE1:2;
    vuint32_t STG_STATE2:2;
    vuint32_t STG_STATE3:2;
    vuint32_t STG_STATE4:2;
    vuint32_t STG_STATE5:2;
    vuint32_t STG_STATE6:2;
    vuint32_t STG_STATE7:2;
    vuint32_t STG_STATE8:2;
    vuint32_t STG_STATE9:2;
    vuint32_t STG_STATE10:2;
    vuint32_t STG_STATE11:2;
    vuint32_t STG_STATE12:2;
    vuint32_t STG_STATE13:2;
    vuint32_t STG_STATE14:2;
    vuint32_t STG_STATE15:2;
  } B;
} SW_PORT1_BPSTGSR_tag;

typedef union SW_PORT1_BPSCR0_union_tag { /* Bridge port storm control register 0 */
  vuint32_t R;
  struct {
    vuint32_t UUCASTRPEID:12;
    vuint32_t _unused_12:4;
    vuint32_t BCASTRPEID:12;
    vuint32_t _unused_28:4;
  } B;
} SW_PORT1_BPSCR0_tag;

typedef union SW_PORT1_BPSCR1_union_tag { /* Bridge port storm control register 1 */
  vuint32_t R;
  struct {
    vuint32_t MCASTRPEID:12;
    vuint32_t _unused_12:4;
    vuint32_t UMCASTRPEID:12;
    vuint32_t _unused_28:4;
  } B;
} SW_PORT1_BPSCR1_tag;

typedef union SW_PORT1_BPOR_union_tag { /* Bridge port operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_DYN:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT1_BPOR_tag;

typedef union SW_PORT1_BPDCR_union_tag { /* Bridge port discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT1_BPDCR_tag;

typedef union SW_PORT1_BPDCRR0_union_tag { /* Bridge port discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t BPACDR:1;
    vuint32_t ISTGSDR:1;
    vuint32_t BPVFLTDR:1;
    vuint32_t MACLNFDR:1;
    vuint32_t _unused_4:3;
    vuint32_t STAMVDDR:1;
    vuint32_t MACFDDDR:1;
    vuint32_t NODESTDR:1;
    vuint32_t IPMFDR:1;
    vuint32_t UFMMDR:1;
    vuint32_t MISCDR:1;
    vuint32_t STRMCTRLDR:1;            /* Discard due to Storm Control Policer Discard Reason */
    vuint32_t _unused_14:18;
  } B;
} SW_PORT1_BPDCRR0_tag;

typedef union SW_PORT1_BPDCRR1_union_tag { /* Bridge port discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:27;
    vuint32_t _unused_27:1;
    vuint32_t TT:4;
  } B;
} SW_PORT1_BPDCRR1_tag;

typedef union SW_PORT1_BPMLFSR_union_tag { /* Bridge port MAC learning failure status register */
  vuint32_t R;
  struct {
    vuint32_t BPMLLRFR:1;              /* Bridge Port MAC Learn Limit Reached Failure Reason */
    vuint32_t FFDBTRFR:1;              /* Full FDB Table Reached Failure Reason */
    vuint32_t HCCLRFR:1;               /* Hash Collision chain limit Reached Failure Reason */
    vuint32_t _unused_3:29;
  } B;
} SW_PORT1_BPMLFSR_tag;

struct SW_PORT1_tag {
  SW_PORT1_PCAPR_tag PCAPR;            /* Port capability register */
  SW_PORT1_PMCAPR_tag PMCAPR;          /* Port MAC capability register */
  SW_PORT1_PIOCAPR_tag PIOCAPR;        /* Port I/O capability register */
  uint8_t SW_PORT1_reserved0[4];
  SW_PORT1_PCR_tag PCR;                /* Port configuration register */
  uint8_t SW_PORT1_reserved1[12];
  SW_PORT1_PMAR0_tag PMAR0;            /* Port MAC address register 0 */
  SW_PORT1_PMAR1_tag PMAR1;            /* Port MAC address register 1 */
  uint8_t SW_PORT1_reserved2[40];
  SW_PORT1_PTAR_tag PTAR;              /* Port TPID acceptance register */
  SW_PORT1_PQOSMR_tag PQOSMR;          /* Port QoS mode register */
  uint8_t SW_PORT1_reserved3[8];
  SW_PORT1_PQOR_tag PQOR;              /* Port Queue Operational register */
  uint8_t SW_PORT1_reserved4[28];
  SW_PORT1_PPCR_tag PPCR;              /* Port parser configuration register */
  SW_PORT1_PIPFCR_tag PIPFCR;          /* Port ingress port filter configuration register */
  uint8_t SW_PORT1_reserved5[24];
  SW_PORT1_PSGCR_tag PSGCR;            /* Port stream gate configuration register */
  uint8_t SW_PORT1_reserved6[92];
  SW_PORT1_POR_tag PORr;               /* Port operational register */
  SW_PORT1_PSR_tag PSR;                /* Port status register */
  SW_PORT1_PRXSDUOR_tag PRXSDUOR;      /* Port receive SDU overhead register */
  SW_PORT1_PTXSDUOR_tag PTXSDUOR;      /* Port transmit SDU overhead register */
  SW_PORT1_PTGSCR_tag PTGSCR;          /* Port time gate scheduling control register */
  SW_PORT1_PTGAGLSR_tag PTGAGLSR;      /* Port time gate scheduling admin gate list status register */
  SW_PORT1_PTGAGLLR_tag PTGAGLLR;      /* Port time gate scheduling admin gate list length register */
  SW_PORT1_PTGOGLLR_tag PTGOGLLR;      /* Port time gating operational gate list length register */
  uint8_t SW_PORT1_reserved7[4];
  SW_PORT1_PTGSHAR_tag PTGSHAR;        /* Port time gate scheduling hold advance register */
  SW_PORT1_PTGSRAR_tag PTGSRAR;        /* Port time gate scheduling release advance register */
  SW_PORT1_PTGSHCR_tag PTGSHCR;        /* Port time gate scheduling hold configuration register */
  uint8_t SW_PORT1_reserved8[4];
  SW_PORT1_PFPCR_tag PFPCR;            /* Port frame preemption configuration register */
  uint8_t SW_PORT1_reserved9[136];
  SW_PORT1_PRXDCR_tag PRXDCR;          /* Port Rx discard count register */
  uint8_t SW_PORT1_reserved10[4];
  SW_PORT1_PRXDCRR0_tag PRXDCRR0;      /* Port Rx discard count reason register 0 */
  SW_PORT1_PRXDCRR1_tag PRXDCRR1;      /* Port Rx discard count reason register 1 */
  uint8_t SW_PORT1_reserved11[16];
  SW_PORT1_PTXDCR_tag PTXDCR;          /* Port Tx discard count register */
  uint8_t SW_PORT1_reserved12[4];
  SW_PORT1_PTXDCRR0_tag PTXDCRR0;      /* Port Tx discard count reason register 0 */
  SW_PORT1_PTXDCRR1_tag PTXDCRR1;      /* Port Tx discard count reason register 1 */
  uint8_t SW_PORT1_reserved13[16];
  SW_PORT1_TCT_NUM_tag TCT_NUM[8];
  uint8_t SW_PORT1_reserved14[256];
  SW_PORT1_PBPMCR0_tag PBPMCR0;        /* Port buffer pool mapping configuration register 0 */
  SW_PORT1_PBPMCR1_tag PBPMCR1;        /* Port buffer pool mapping configuration register 1 */
  uint8_t SW_PORT1_reserved15[48];
  SW_PORT1_PPCPDEIMR_tag PPCPDEIMR;    /* Port PCP DEI mapping register */
  uint8_t SW_PORT1_reserved16[4];
  SW_PORT1_PMCR_tag PMCR;              /* Port mirror configuration register */
  uint8_t SW_PORT1_reserved17[12];
  SW_PORT1_PCTFCR_tag PCTFCR;          /* Port cut through forwarding configuration register */
  uint8_t SW_PORT1_reserved18[4];
  SW_PORT1_PLANIDCR_tag PLANIDCR;      /* Port LANID configuration register */
  uint8_t SW_PORT1_reserved19[4];
  SW_PORT1_PISIDCR_tag PISIDCR;        /* Port ingress stream identification configuration register */
  SW_PORT1_PFMCR_tag PFMCR;            /* Port frame modification configuration register */
  uint8_t SW_PORT1_reserved20[8];
  SW_PORT1_PIPV2QMR0_tag PIPV2QMR0;    /* Port IPV to queue mapping register 0 */
  uint8_t SW_PORT1_reserved21[60];
  SW_PORT1_PTCMINLR_tag PTCMINLR;      /* Port time capture minimum latency register */
  SW_PORT1_PTCMAXLR_tag PTCMAXLR;      /* Port time capture maximum latency register */
  uint8_t SW_PORT1_reserved22[72];
  SW_PORT1_BPCR_tag BPCR;              /* Bridge port configuration register */
  uint8_t SW_PORT1_reserved23[12];
  SW_PORT1_BPDVR_tag BPDVR;            /* Bridge port default VLAN register */
  uint8_t SW_PORT1_reserved24[12];
  SW_PORT1_BPSTGSR_tag BPSTGSR;        /* Bridge port spanning tree group state register */
  uint8_t SW_PORT1_reserved25[4];
  SW_PORT1_BPSCR0_tag BPSCR0;          /* Bridge port storm control register 0 */
  SW_PORT1_BPSCR1_tag BPSCR1;          /* Bridge port storm control register 1 */
  SW_PORT1_BPOR_tag BPOR;              /* Bridge port operational register */
  uint8_t SW_PORT1_reserved26[76];
  SW_PORT1_BPDCR_tag BPDCR;            /* Bridge port discard count register */
  uint8_t SW_PORT1_reserved27[4];
  SW_PORT1_BPDCRR0_tag BPDCRR0;        /* Bridge port discard count reason register 0 */
  SW_PORT1_BPDCRR1_tag BPDCRR1;        /* Bridge port discard count reason register 1 */
  SW_PORT1_BPMLFSR_tag BPMLFSR;        /* Bridge port MAC learning failure status register */
};


/* ============================================================================
   =============================== Module: SW_PORT2 ===========================
   ============================================================================ */

typedef union SW_PORT2_PCAPR_union_tag { /* Port capability register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;
    vuint32_t LINK_TYPE:1;
    vuint32_t _unused_5:7;
    vuint32_t NUM_TC:4;
    vuint32_t NUM_Q:4;
    vuint32_t _unused_20:4;
    vuint32_t NUM_CG:4;
    vuint32_t TGS:1;                   /* Time Gate Scheduling */
    vuint32_t CBS:1;                   /* Credit Based Shaping */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT2_PCAPR_tag;

typedef union SW_PORT2_PMCAPR_union_tag { /* Port MAC capability register */
  vuint32_t R;
  struct {
    vuint32_t MAC_VAR:3;               /* MAC Variant */
    vuint32_t _unused_3:1;
    vuint32_t EFPAD:2;
    vuint32_t _unused_6:2;
    vuint32_t HD:1;
    vuint32_t FP:2;                    /* Indicates if frame preemption is supported */
    vuint32_t _unused_11:13;
    vuint32_t MII_PROT:4;              /* Indicates the MII protocol supported */
    vuint32_t _unused_28:4;
  } B;
} SW_PORT2_PMCAPR_tag;

typedef union SW_PORT2_PIOCAPR_union_tag { /* Port I/O capability register */
  vuint32_t R;
  struct {
    vuint32_t PCS_PROT:16;             /* PCS protocols supported */
    vuint32_t _unused_16:7;
    vuint32_t _unused_23:1;
    vuint32_t IO_VAR:4;                /* IO Variants supported */
    vuint32_t EMDIO:1;
    vuint32_t _unused_29:1;
    vuint32_t REVMII_RATE:1;           /* RevMII MII rate */
    vuint32_t REVMII:1;                /* Reverse Mode Device Configuration */
  } B;
} SW_PORT2_PIOCAPR_tag;

typedef union SW_PORT2_PCR_union_tag { /* Port configuration register */
  vuint32_t R;
  struct {
    vuint32_t HDR_FMT:1;
    vuint32_t _unused_1:3;
    vuint32_t L2DOSE:1;
    vuint32_t _unused_5:3;
    vuint32_t TIMER_CS:1;
    vuint32_t _unused_9:3;
    vuint32_t FCSEA:1;                 /* FCS Error Action */
    vuint32_t _unused_13:3;
    vuint32_t PSPEED:14;
    vuint32_t _unused_30:2;
  } B;
} SW_PORT2_PCR_tag;

typedef union SW_PORT2_PMAR0_union_tag { /* Port MAC address register 0 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:32;
  } B;
} SW_PORT2_PMAR0_tag;

typedef union SW_PORT2_PMAR1_union_tag { /* Port MAC address register 1 */
  vuint32_t R;
  struct {
    vuint32_t PRIM_MAC_ADDR:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT2_PMAR1_tag;

typedef union SW_PORT2_PTAR_union_tag { /* Port TPID acceptance register */
  vuint32_t R;
  struct {
    vuint32_t OVTPIDL:4;
    vuint32_t IVTPIDL:4;
    vuint32_t _unused_8:24;
  } B;
} SW_PORT2_PTAR_tag;

typedef union SW_PORT2_PQOSMR_union_tag { /* Port QoS mode register */
  vuint32_t R;
  struct {
    vuint32_t VS:1;
    vuint32_t VE:1;
    vuint32_t DDRf:2;
    vuint32_t DIPV:3;
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:8;
    vuint32_t VQMP:4;                  /* Mapping profile index */
    vuint32_t QVMP:4;                  /* Mapping profile index */
    vuint32_t _unused_24:8;
  } B;
} SW_PORT2_PQOSMR_tag;

typedef union SW_PORT2_PQOR_union_tag { /* Port Queue Operational register */
  vuint32_t R;
  struct {
    vuint32_t Q0S:1;
    vuint32_t Q1S:1;
    vuint32_t Q2S:1;
    vuint32_t Q3S:1;
    vuint32_t Q4S:1;
    vuint32_t Q5S:1;
    vuint32_t Q6S:1;
    vuint32_t Q7S:1;
    vuint32_t _unused_8:24;
  } B;
} SW_PORT2_PQOR_tag;

typedef union SW_PORT2_PPCR_union_tag { /* Port parser configuration register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t L1PFS:5;
    vuint32_t _unused_6:3;
    vuint32_t L2PFS:5;
    vuint32_t _unused_14:2;
    vuint32_t L3HFP:1;                 /* L3 header fields present */
    vuint32_t L3PFS:5;                 /* L3 payload fields size in bytes */
    vuint32_t _unused_22:2;
    vuint32_t L4HFP:1;                 /* L4 Header fields present */
    vuint32_t L4PFS:5;                 /* L4 payload fields size in bytes */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT2_PPCR_tag;

typedef union SW_PORT2_PIPFCR_union_tag { /* Port ingress port filter configuration register */
  vuint32_t R;
  struct {
    vuint32_t EN:1;
    vuint32_t _unused_1:31;
  } B;
} SW_PORT2_PIPFCR_tag;

typedef union SW_PORT2_PSGCR_union_tag { /* Port stream gate configuration register */
  vuint32_t R;
  struct {
    vuint32_t PDELAY:24;
    vuint32_t _unused_24:7;
    vuint32_t OGC:1;                   /* Stream Gate Open Gate Check mode */
  } B;
} SW_PORT2_PSGCR_tag;

typedef union SW_PORT2_POR_union_tag { /* Port operational register */
  vuint32_t R;
  struct {
    vuint32_t TXDIS:1;                 /* Tx Disable. */
    vuint32_t RXDIS:1;                 /* Rx Disable. */
    vuint32_t _unused_2:30;
  } B;
} SW_PORT2_POR_tag;

typedef union SW_PORT2_PSR_union_tag { /* Port status register */
  vuint32_t R;
  struct {
    vuint32_t TX_BUSY:1;               /* Transmit busy. */
    vuint32_t RX_BUSY:1;               /* Receive busy. */
    vuint32_t _unused_2:30;
  } B;
} SW_PORT2_PSR_tag;

typedef union SW_PORT2_PRXSDUOR_union_tag { /* Port receive SDU overhead register */
  vuint32_t R;
  struct {
    vuint32_t PPDU_BCO:5;              /* PPDU Byte count overhead */
    vuint32_t _unused_5:3;
    vuint32_t MACSEC_BCO:5;            /* MACsec byte count overhead */
    vuint32_t _unused_13:19;
  } B;
} SW_PORT2_PRXSDUOR_tag;

typedef union SW_PORT2_PTXSDUOR_union_tag { /* Port transmit SDU overhead register */
  vuint32_t R;
  struct {
    vuint32_t PPDU_BCO:5;              /* PPDU Byte count overhead */
    vuint32_t _unused_5:3;
    vuint32_t MACSEC_BCO:5;            /* MACsec byte count overhead */
    vuint32_t _unused_13:19;
  } B;
} SW_PORT2_PTXSDUOR_tag;

typedef union SW_PORT2_PTGSCR_union_tag { /* Port time gate scheduling control register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:31;
    vuint32_t TGE:1;                   /* Time Gating Enable */
  } B;
} SW_PORT2_PTGSCR_tag;

typedef union SW_PORT2_PTGAGLSR_union_tag { /* Port time gate scheduling admin gate list status register */
  vuint32_t R;
  struct {
    vuint32_t TG:1;
    vuint32_t CFG_PEND:1;
    vuint32_t _unused_2:30;
  } B;
} SW_PORT2_PTGAGLSR_tag;

typedef union SW_PORT2_PTGAGLLR_union_tag { /* Port time gate scheduling admin gate list length register */
  vuint32_t R;
  struct {
    vuint32_t ADMIN_GATE_LIST_LENGTH:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT2_PTGAGLLR_tag;

typedef union SW_PORT2_PTGOGLLR_union_tag { /* Port time gating operational gate list length register */
  vuint32_t R;
  struct {
    vuint32_t OPER_GATE_LIST_LENGTH:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT2_PTGOGLLR_tag;

typedef union SW_PORT2_PRXDCR_union_tag { /* Port Rx discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT2_PRXDCR_tag;

typedef union SW_PORT2_PRXDCRR0_union_tag { /* Port Rx discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t PCDR:1;
    vuint32_t SMREDR:1;
    vuint32_t RXDISDR:1;
    vuint32_t IPFDR:1;
    vuint32_t RPDR:1;
    vuint32_t ISFDR:1;
    vuint32_t SGCDR:1;
    vuint32_t SGOEDR:1;
    vuint32_t MSDUEDR:1;
    vuint32_t FMMEDR:1;
    vuint32_t CMDR:1;
    vuint32_t ITEDR:1;
    vuint32_t ECCEDR:1;
    vuint32_t _unused_13:1;
    vuint32_t L2DOSDR:1;
    vuint32_t _unused_15:1;
    vuint32_t PEDR:1;
    vuint32_t NODESTDR:1;              /* No Destination Discard Reason */
    vuint32_t _unused_18:14;
  } B;
} SW_PORT2_PRXDCRR0_tag;

typedef union SW_PORT2_PRXDCRR1_union_tag { /* Port Rx discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:16;
    vuint32_t _unused_16:12;
    vuint32_t TT:4;
  } B;
} SW_PORT2_PRXDCRR1_tag;

typedef union SW_PORT2_PTXDCR_union_tag { /* Port Tx discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT2_PTXDCR_tag;

typedef union SW_PORT2_PTXDCRR0_union_tag { /* Port Tx discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t TXDISDR:1;
    vuint32_t ECCEDR:1;                /* ECC Error Discard Reason */
    vuint32_t PEDR:1;                  /* Parity Error Discard Reason */
    vuint32_t _unused_3:1;
    vuint32_t TGSFTLDR:1;
    vuint32_t FMMDR:1;
    vuint32_t TXDISEDR:1;
    vuint32_t MSDUEDR:1;
    vuint32_t QCONGDR:1;
    vuint32_t ITEDR:1;
    vuint32_t INVEQDR:1;
    vuint32_t SQRTNSQDR:1;
    vuint32_t _unused_12:1;
    vuint32_t SQRRDR:1;
    vuint32_t SQRDDR:1;
    vuint32_t SMREDR:1;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT2_PTXDCRR0_tag;

typedef union SW_PORT2_PTXDCRR1_union_tag { /* Port Tx discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:16;
    vuint32_t _unused_16:12;
    vuint32_t TT:4;
  } B;
} SW_PORT2_PTXDCRR1_tag;

typedef union SW_PORT2_TCT_NUM_PTGSTCSR_union_tag { /* Port time gate scheduling traffic class 0 status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t _unused_1:15;
    vuint32_t LH_STATE:1;
    vuint32_t _unused_17:15;
  } B;
} SW_PORT2_TCT_NUM_PTGSTCSR_tag;

typedef union SW_PORT2_TCT_NUM_PTCTMSDUR_union_tag { /* Port traffic class 0 transmit maximum SDU register */
  vuint32_t R;
  struct {
    vuint32_t MAXSDU:16;               /* Tx Maximum Service Data Unit Size */
    vuint32_t SDU_TYPE:2;              /* SDU type */
    vuint32_t _unused_18:6;
    vuint32_t SF_MAXSDU_DIS:1;
    vuint32_t _unused_25:7;
  } B;
} SW_PORT2_TCT_NUM_PTCTMSDUR_tag;

typedef union SW_PORT2_TCT_NUM_PTCCBSR0_union_tag { /* Port transmit traffic class 0 credit based shaper register 0 */
  vuint32_t R;
  struct {
    vuint32_t BW:7;                    /* Bandwidth */
    vuint32_t _unused_7:24;
    vuint32_t CBSE:1;                  /* Credit Based Shaper Enable */
  } B;
} SW_PORT2_TCT_NUM_PTCCBSR0_tag;

typedef union SW_PORT2_TCT_NUM_PTCCBSR1_union_tag { /* Port traffic class 0 credit based shaper register 1 */
  vuint32_t R;
  struct {
    vuint32_t HI_CREDIT:32;            /* hicredit (in credit units) */
  } B;
} SW_PORT2_TCT_NUM_PTCCBSR1_tag;

typedef struct SW_PORT2_TCT_NUM_struct_tag {
  SW_PORT2_TCT_NUM_PTGSTCSR_tag PTGSTCSR; /* Port time gate scheduling traffic class 0 status register */
  uint8_t TCT_NUM_reserved0[4];
  SW_PORT2_TCT_NUM_PTCTMSDUR_tag PTCTMSDUR; /* Port traffic class 0 transmit maximum SDU register */
  uint8_t TCT_NUM_reserved1[4];
  SW_PORT2_TCT_NUM_PTCCBSR0_tag PTCCBSR0; /* Port transmit traffic class 0 credit based shaper register 0 */
  SW_PORT2_TCT_NUM_PTCCBSR1_tag PTCCBSR1; /* Port traffic class 0 credit based shaper register 1 */
  uint8_t TCT_NUM_reserved2[8];
} SW_PORT2_TCT_NUM_tag;

typedef union SW_PORT2_PBPMCR0_union_tag { /* Port buffer pool mapping configuration register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0_INDEX:8;
    vuint32_t IPV1_INDEX:8;
    vuint32_t IPV2_INDEX:8;
    vuint32_t IPV3_INDEX:8;
  } B;
} SW_PORT2_PBPMCR0_tag;

typedef union SW_PORT2_PBPMCR1_union_tag { /* Port buffer pool mapping configuration register 1 */
  vuint32_t R;
  struct {
    vuint32_t IPV4_INDEX:8;
    vuint32_t IPV5_INDEX:8;
    vuint32_t IPV6_INDEX:8;
    vuint32_t IPV7_INDEX:8;
  } B;
} SW_PORT2_PBPMCR1_tag;

typedef union SW_PORT2_PPCPDEIMR_union_tag { /* Port PCP DEI mapping register */
  vuint32_t R;
  struct {
    vuint32_t IPCPMP:4;
    vuint32_t _unused_4:3;
    vuint32_t IPCPMPV:1;
    vuint32_t EPCPMP:4;
    vuint32_t _unused_12:3;
    vuint32_t EPCPMPV:1;
    vuint32_t DR0DEI:1;
    vuint32_t DR1DEI:1;
    vuint32_t DR2DEI:1;
    vuint32_t DR3DEI:1;
    vuint32_t DRME:1;
    vuint32_t _unused_21:11;
  } B;
} SW_PORT2_PPCPDEIMR_tag;

typedef union SW_PORT2_PMCR_union_tag { /* Port mirror configuration register */
  vuint32_t R;
  struct {
    vuint32_t IMIRE:1;
    vuint32_t _unused_1:31;
  } B;
} SW_PORT2_PMCR_tag;

typedef union SW_PORT2_PLANIDCR_union_tag { /* Port LANID configuration register */
  vuint32_t R;
  struct {
    vuint32_t LANID:4;                 /* LAN Identifier */
    vuint32_t _unused_4:28;
  } B;
} SW_PORT2_PLANIDCR_tag;

typedef union SW_PORT2_PISIDCR_union_tag { /* Port ingress stream identification configuration register */
  vuint32_t R;
  struct {
    vuint32_t KCPAIR:1;
    vuint32_t KC0EN:1;                 /* Key Construction 0 Enable */
    vuint32_t KC1EN:1;                 /* Key Construction 1 Enable */
    vuint32_t _unused_3:13;
    vuint32_t ISEID:16;
  } B;
} SW_PORT2_PISIDCR_tag;

typedef union SW_PORT2_PFMCR_union_tag { /* Port frame modification configuration register */
  vuint32_t R;
  struct {
    vuint32_t FMMA:1;                  /* Frame Modification Misconfiguration Action */
    vuint32_t _unused_1:31;
  } B;
} SW_PORT2_PFMCR_tag;

typedef union SW_PORT2_PIPV2QMR0_union_tag { /* Port IPV to queue mapping register 0 */
  vuint32_t R;
  struct {
    vuint32_t IPV0_Q:4;
    vuint32_t IPV1_Q:4;
    vuint32_t IPV2_Q:4;
    vuint32_t IPV3_Q:4;
    vuint32_t IPV4_Q:4;
    vuint32_t IPV5_Q:4;
    vuint32_t IPV6_Q:4;
    vuint32_t IPV7_Q:4;
  } B;
} SW_PORT2_PIPV2QMR0_tag;

typedef union SW_PORT2_PTCMINLR_union_tag { /* Port time capture minimum latency register */
  vuint32_t R;
  struct {
    vuint32_t LATENCY:30;
    vuint32_t COUNT:2;
  } B;
} SW_PORT2_PTCMINLR_tag;

typedef union SW_PORT2_PTCMAXLR_union_tag { /* Port time capture maximum latency register */
  vuint32_t R;
  struct {
    vuint32_t LATENCY:30;
    vuint32_t _unused_30:2;
  } B;
} SW_PORT2_PTCMAXLR_tag;

typedef union SW_PORT2_BPCR_union_tag { /* Bridge port configuration register */
  vuint32_t R;
  struct {
    vuint32_t DYN_LIMIT:16;            /* Dynamic Limit */
    vuint32_t _unused_16:8;
    vuint32_t UUCASTE:1;               /* Unknown Unicast Storm Control Enable */
    vuint32_t UMCASTE:1;               /* Unknown Multicast Storm Control Enable */
    vuint32_t MCASTE:1;                /* Multicast Storm Control Enable */
    vuint32_t BCASTE:1;                /* Broadcast Storm Control Enable */
    vuint32_t STAMVD:1;                /* Station Move Disallow */
    vuint32_t SRCPRND:1;               /* Source Port Pruning Disable */
    vuint32_t _unused_30:2;
  } B;
} SW_PORT2_BPCR_tag;

typedef union SW_PORT2_BPDVR_union_tag { /* Bridge port default VLAN register */
  vuint32_t R;
  struct {
    vuint32_t VID:12;
    vuint32_t DEI:1;
    vuint32_t PCP:3;
    vuint32_t TPID:1;
    vuint32_t _unused_17:3;
    vuint32_t RXTAGA:4;
    vuint32_t RXVAM:1;
    vuint32_t TXTAGA:2;
    vuint32_t _unused_27:5;
  } B;
} SW_PORT2_BPDVR_tag;

typedef union SW_PORT2_BPSTGSR_union_tag { /* Bridge port spanning tree group state register */
  vuint32_t R;
  struct {
    vuint32_t STG_STATE0:2;
    vuint32_t STG_STATE1:2;
    vuint32_t STG_STATE2:2;
    vuint32_t STG_STATE3:2;
    vuint32_t STG_STATE4:2;
    vuint32_t STG_STATE5:2;
    vuint32_t STG_STATE6:2;
    vuint32_t STG_STATE7:2;
    vuint32_t STG_STATE8:2;
    vuint32_t STG_STATE9:2;
    vuint32_t STG_STATE10:2;
    vuint32_t STG_STATE11:2;
    vuint32_t STG_STATE12:2;
    vuint32_t STG_STATE13:2;
    vuint32_t STG_STATE14:2;
    vuint32_t STG_STATE15:2;
  } B;
} SW_PORT2_BPSTGSR_tag;

typedef union SW_PORT2_BPSCR0_union_tag { /* Bridge port storm control register 0 */
  vuint32_t R;
  struct {
    vuint32_t UUCASTRPEID:12;
    vuint32_t _unused_12:4;
    vuint32_t BCASTRPEID:12;
    vuint32_t _unused_28:4;
  } B;
} SW_PORT2_BPSCR0_tag;

typedef union SW_PORT2_BPSCR1_union_tag { /* Bridge port storm control register 1 */
  vuint32_t R;
  struct {
    vuint32_t MCASTRPEID:12;
    vuint32_t _unused_12:4;
    vuint32_t UMCASTRPEID:12;
    vuint32_t _unused_28:4;
  } B;
} SW_PORT2_BPSCR1_tag;

typedef union SW_PORT2_BPOR_union_tag { /* Bridge port operational register */
  vuint32_t R;
  struct {
    vuint32_t NUM_DYN:16;
    vuint32_t _unused_16:16;
  } B;
} SW_PORT2_BPOR_tag;

typedef union SW_PORT2_BPDCR_union_tag { /* Bridge port discard count register */
  vuint32_t R;
  struct {
    vuint32_t COUNT:32;
  } B;
} SW_PORT2_BPDCR_tag;

typedef union SW_PORT2_BPDCRR0_union_tag { /* Bridge port discard count reason register 0 */
  vuint32_t R;
  struct {
    vuint32_t BPACDR:1;
    vuint32_t ISTGSDR:1;
    vuint32_t BPVFLTDR:1;
    vuint32_t MACLNFDR:1;
    vuint32_t _unused_4:3;
    vuint32_t STAMVDDR:1;
    vuint32_t MACFDDDR:1;
    vuint32_t NODESTDR:1;
    vuint32_t IPMFDR:1;
    vuint32_t UFMMDR:1;
    vuint32_t MISCDR:1;
    vuint32_t STRMCTRLDR:1;            /* Discard due to Storm Control Policer Discard Reason */
    vuint32_t _unused_14:18;
  } B;
} SW_PORT2_BPDCRR0_tag;

typedef union SW_PORT2_BPDCRR1_union_tag { /* Bridge port discard count reason register 1 */
  vuint32_t R;
  struct {
    vuint32_t ENTRYID:27;
    vuint32_t _unused_27:1;
    vuint32_t TT:4;
  } B;
} SW_PORT2_BPDCRR1_tag;

typedef union SW_PORT2_BPMLFSR_union_tag { /* Bridge port MAC learning failure status register */
  vuint32_t R;
  struct {
    vuint32_t BPMLLRFR:1;              /* Bridge Port MAC Learn Limit Reached Failure Reason */
    vuint32_t FFDBTRFR:1;              /* Full FDB Table Reached Failure Reason */
    vuint32_t HCCLRFR:1;               /* Hash Collision chain limit Reached Failure Reason */
    vuint32_t _unused_3:29;
  } B;
} SW_PORT2_BPMLFSR_tag;

struct SW_PORT2_tag {
  SW_PORT2_PCAPR_tag PCAPR;            /* Port capability register */
  SW_PORT2_PMCAPR_tag PMCAPR;          /* Port MAC capability register */
  SW_PORT2_PIOCAPR_tag PIOCAPR;        /* Port I/O capability register */
  uint8_t SW_PORT2_reserved0[4];
  SW_PORT2_PCR_tag PCR;                /* Port configuration register */
  uint8_t SW_PORT2_reserved1[12];
  SW_PORT2_PMAR0_tag PMAR0;            /* Port MAC address register 0 */
  SW_PORT2_PMAR1_tag PMAR1;            /* Port MAC address register 1 */
  uint8_t SW_PORT2_reserved2[40];
  SW_PORT2_PTAR_tag PTAR;              /* Port TPID acceptance register */
  SW_PORT2_PQOSMR_tag PQOSMR;          /* Port QoS mode register */
  uint8_t SW_PORT2_reserved3[8];
  SW_PORT2_PQOR_tag PQOR;              /* Port Queue Operational register */
  uint8_t SW_PORT2_reserved4[28];
  SW_PORT2_PPCR_tag PPCR;              /* Port parser configuration register */
  SW_PORT2_PIPFCR_tag PIPFCR;          /* Port ingress port filter configuration register */
  uint8_t SW_PORT2_reserved5[24];
  SW_PORT2_PSGCR_tag PSGCR;            /* Port stream gate configuration register */
  uint8_t SW_PORT2_reserved6[92];
  SW_PORT2_POR_tag PORr;               /* Port operational register */
  SW_PORT2_PSR_tag PSR;                /* Port status register */
  SW_PORT2_PRXSDUOR_tag PRXSDUOR;      /* Port receive SDU overhead register */
  SW_PORT2_PTXSDUOR_tag PTXSDUOR;      /* Port transmit SDU overhead register */
  SW_PORT2_PTGSCR_tag PTGSCR;          /* Port time gate scheduling control register */
  SW_PORT2_PTGAGLSR_tag PTGAGLSR;      /* Port time gate scheduling admin gate list status register */
  SW_PORT2_PTGAGLLR_tag PTGAGLLR;      /* Port time gate scheduling admin gate list length register */
  SW_PORT2_PTGOGLLR_tag PTGOGLLR;      /* Port time gating operational gate list length register */
  uint8_t SW_PORT2_reserved7[160];
  SW_PORT2_PRXDCR_tag PRXDCR;          /* Port Rx discard count register */
  uint8_t SW_PORT2_reserved8[4];
  SW_PORT2_PRXDCRR0_tag PRXDCRR0;      /* Port Rx discard count reason register 0 */
  SW_PORT2_PRXDCRR1_tag PRXDCRR1;      /* Port Rx discard count reason register 1 */
  uint8_t SW_PORT2_reserved9[16];
  SW_PORT2_PTXDCR_tag PTXDCR;          /* Port Tx discard count register */
  uint8_t SW_PORT2_reserved10[4];
  SW_PORT2_PTXDCRR0_tag PTXDCRR0;      /* Port Tx discard count reason register 0 */
  SW_PORT2_PTXDCRR1_tag PTXDCRR1;      /* Port Tx discard count reason register 1 */
  uint8_t SW_PORT2_reserved11[16];
  SW_PORT2_TCT_NUM_tag TCT_NUM[8];
  uint8_t SW_PORT2_reserved12[256];
  SW_PORT2_PBPMCR0_tag PBPMCR0;        /* Port buffer pool mapping configuration register 0 */
  SW_PORT2_PBPMCR1_tag PBPMCR1;        /* Port buffer pool mapping configuration register 1 */
  uint8_t SW_PORT2_reserved13[48];
  SW_PORT2_PPCPDEIMR_tag PPCPDEIMR;    /* Port PCP DEI mapping register */
  uint8_t SW_PORT2_reserved14[4];
  SW_PORT2_PMCR_tag PMCR;              /* Port mirror configuration register */
  uint8_t SW_PORT2_reserved15[20];
  SW_PORT2_PLANIDCR_tag PLANIDCR;      /* Port LANID configuration register */
  uint8_t SW_PORT2_reserved16[4];
  SW_PORT2_PISIDCR_tag PISIDCR;        /* Port ingress stream identification configuration register */
  SW_PORT2_PFMCR_tag PFMCR;            /* Port frame modification configuration register */
  uint8_t SW_PORT2_reserved17[8];
  SW_PORT2_PIPV2QMR0_tag PIPV2QMR0;    /* Port IPV to queue mapping register 0 */
  uint8_t SW_PORT2_reserved18[60];
  SW_PORT2_PTCMINLR_tag PTCMINLR;      /* Port time capture minimum latency register */
  SW_PORT2_PTCMAXLR_tag PTCMAXLR;      /* Port time capture maximum latency register */
  uint8_t SW_PORT2_reserved19[72];
  SW_PORT2_BPCR_tag BPCR;              /* Bridge port configuration register */
  uint8_t SW_PORT2_reserved20[12];
  SW_PORT2_BPDVR_tag BPDVR;            /* Bridge port default VLAN register */
  uint8_t SW_PORT2_reserved21[12];
  SW_PORT2_BPSTGSR_tag BPSTGSR;        /* Bridge port spanning tree group state register */
  uint8_t SW_PORT2_reserved22[4];
  SW_PORT2_BPSCR0_tag BPSCR0;          /* Bridge port storm control register 0 */
  SW_PORT2_BPSCR1_tag BPSCR1;          /* Bridge port storm control register 1 */
  SW_PORT2_BPOR_tag BPOR;              /* Bridge port operational register */
  uint8_t SW_PORT2_reserved23[76];
  SW_PORT2_BPDCR_tag BPDCR;            /* Bridge port discard count register */
  uint8_t SW_PORT2_reserved24[4];
  SW_PORT2_BPDCRR0_tag BPDCRR0;        /* Bridge port discard count reason register 0 */
  SW_PORT2_BPDCRR1_tag BPDCRR1;        /* Bridge port discard count reason register 1 */
  SW_PORT2_BPMLFSR_tag BPMLFSR;        /* Bridge port MAC learning failure status register */
};


/* ============================================================================
   =============================== Module: SW_PSEUDO_MAC_PORT2 ================
   ============================================================================ */

typedef union SW_PSEUDO_MAC_PORT2_PPMSR_union_tag { /* Port pseudo MAC status register */
  vuint32_t R;
  struct {
    vuint32_t LSTATE:1;
    vuint32_t _unused_1:7;
    vuint32_t RSTATE:1;
    vuint32_t _unused_9:23;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMSR_tag;

typedef union SW_PSEUDO_MAC_PORT2_PPMROCR_union_tag { /* Port pseudo MAC receive octets counter */
  vuint32_t R;
  struct {
    vuint32_t ROCT:32;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMROCR_tag;

typedef union SW_PSEUDO_MAC_PORT2_PPMRUFCR_union_tag { /* Port pseudo MAC receive unicast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t RUCA:32;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMRUFCR_tag;

typedef union SW_PSEUDO_MAC_PORT2_PPMRMFCR_union_tag { /* Port pseudo MAC receive multicast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t RMCA:32;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMRMFCR_tag;

typedef union SW_PSEUDO_MAC_PORT2_PPMRBFCR_union_tag { /* Port pseudo MAC receive broadcast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t RBCA:32;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMRBFCR_tag;

typedef union SW_PSEUDO_MAC_PORT2_PPMTOCR_union_tag { /* Port pseudo MAC transmit octets counter */
  vuint32_t R;
  struct {
    vuint32_t TOCT:32;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMTOCR_tag;

typedef union SW_PSEUDO_MAC_PORT2_PPMTUFCR_union_tag { /* Port pseudo MAC transmit unicast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t TUCA:32;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMTUFCR_tag;

typedef union SW_PSEUDO_MAC_PORT2_PPMTMFCR_union_tag { /* Port pseudo MAC transmit multicast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t TMCA:32;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMTMFCR_tag;

typedef union SW_PSEUDO_MAC_PORT2_PPMTBFCR_union_tag { /* Port pseudo MAC transmit broadcast frame counter register */
  vuint32_t R;
  struct {
    vuint32_t TBCA:32;
  } B;
} SW_PSEUDO_MAC_PORT2_PPMTBFCR_tag;

struct SW_PSEUDO_MAC_PORT2_tag {
  SW_PSEUDO_MAC_PORT2_PPMSR_tag PPMSR; /* Port pseudo MAC status register */
  uint8_t SW_PSEUDO_MAC_PORT2_reserved0[124];
  SW_PSEUDO_MAC_PORT2_PPMROCR_tag PPMROCR[2]; /* Port pseudo MAC receive octets counter */
  SW_PSEUDO_MAC_PORT2_PPMRUFCR_tag PPMRUFCR[2]; /* Port pseudo MAC receive unicast frame counter register */
  SW_PSEUDO_MAC_PORT2_PPMRMFCR_tag PPMRMFCR[2]; /* Port pseudo MAC receive multicast frame counter register */
  SW_PSEUDO_MAC_PORT2_PPMRBFCR_tag PPMRBFCR[2]; /* Port pseudo MAC receive broadcast frame counter register */
  uint8_t SW_PSEUDO_MAC_PORT2_reserved1[32];
  SW_PSEUDO_MAC_PORT2_PPMTOCR_tag PPMTOCR[2]; /* Port pseudo MAC transmit octets counter */
  SW_PSEUDO_MAC_PORT2_PPMTUFCR_tag PPMTUFCR[2]; /* Port pseudo MAC transmit unicast frame counter register */
  SW_PSEUDO_MAC_PORT2_PPMTMFCR_tag PPMTMFCR[2]; /* Port pseudo MAC transmit multicast frame counter register */
  SW_PSEUDO_MAC_PORT2_PPMTBFCR_tag PPMTBFCR[2]; /* Port pseudo MAC transmit broadcast frame counter register */
};


/* ============================================================================
   =============================== Module: TIMERS =============================
   ============================================================================ */

typedef union TIMERS_TIMER0_CFG_union_tag { /* TIMER0 CFG */
  vuint32_t R;
  struct {
    vuint32_t TS:2;
    vuint32_t CM:3;
    vuint32_t ES:1;
    vuint32_t TP:1;
    vuint32_t TM:2;
    vuint32_t _unused_9:1;
    vuint32_t CAS:1;
    vuint32_t _unused_11:5;
    vuint32_t RES:1;
    vuint32_t PC:1;
    vuint32_t CT:1;
    vuint32_t RU:1;
    vuint32_t BPEN:1;
    vuint32_t _unused_21:11;
  } B;
} TIMERS_TIMER0_CFG_tag;

typedef union TIMERS_TIMER0_EW_union_tag { /* TIMER0 EW */
  vuint32_t R;
  struct {
    vuint32_t EW:1;
    vuint32_t _unused_1:31;
  } B;
} TIMERS_TIMER0_EW_tag;

typedef union TIMERS_TIMER0_CC_union_tag { /* TIMER0 CC */
  vuint32_t R;
  struct {
    vuint32_t CC:32;
  } B;
} TIMERS_TIMER0_CC_tag;

typedef union TIMERS_TIMER0_SC_union_tag { /* TIMER0 SC */
  vuint32_t R;
  struct {
    vuint32_t SC:32;
  } B;
} TIMERS_TIMER0_SC_tag;

typedef union TIMERS_TIMER1_CFG_union_tag { /* TIMER1 CFG */
  vuint32_t R;
  struct {
    vuint32_t TS:2;
    vuint32_t CM:3;
    vuint32_t ES:1;
    vuint32_t _unused_6:10;
    vuint32_t RES:1;
    vuint32_t PC:1;
    vuint32_t _unused_18:1;
    vuint32_t RU:1;
    vuint32_t BPEN:1;
    vuint32_t _unused_21:11;
  } B;
} TIMERS_TIMER1_CFG_tag;

typedef union TIMERS_TIMER1_EW_union_tag { /* TIMER1 EW */
  vuint32_t R;
  struct {
    vuint32_t EW:1;
    vuint32_t _unused_1:31;
  } B;
} TIMERS_TIMER1_EW_tag;

typedef union TIMERS_TIMER1_CC_union_tag { /* TIMER1 CC */
  vuint32_t R;
  struct {
    vuint32_t CC:32;
  } B;
} TIMERS_TIMER1_CC_tag;

typedef union TIMERS_TIMER1_SC_union_tag { /* TIMER1 SC */
  vuint32_t R;
  struct {
    vuint32_t SC:32;
  } B;
} TIMERS_TIMER1_SC_tag;

typedef union TIMERS_TIMER2_CFG_union_tag { /* TIMER2 CFG */
  vuint32_t R;
  struct {
    vuint32_t TS:2;
    vuint32_t CM:3;
    vuint32_t ES:1;
    vuint32_t _unused_6:4;
    vuint32_t CAS:1;
    vuint32_t _unused_11:5;
    vuint32_t RES:1;
    vuint32_t PC:1;
    vuint32_t _unused_18:1;
    vuint32_t RU:1;
    vuint32_t BPEN:1;
    vuint32_t _unused_21:11;
  } B;
} TIMERS_TIMER2_CFG_tag;

typedef union TIMERS_TIMER2_EW_union_tag { /* TIMER2 EW */
  vuint32_t R;
  struct {
    vuint32_t EW:1;
    vuint32_t _unused_1:31;
  } B;
} TIMERS_TIMER2_EW_tag;

typedef union TIMERS_TIMER2_CC_union_tag { /* TIMER2 CC */
  vuint32_t R;
  struct {
    vuint32_t CC:32;
  } B;
} TIMERS_TIMER2_CC_tag;

typedef union TIMERS_TIMER2_SC_union_tag { /* TIMER2 SC */
  vuint32_t R;
  struct {
    vuint32_t SC:32;
  } B;
} TIMERS_TIMER2_SC_tag;

typedef union TIMERS_TIMER3_CFG_union_tag { /* TIMER3 CFG */
  vuint32_t R;
  struct {
    vuint32_t TS:2;
    vuint32_t CM:3;
    vuint32_t ES:1;
    vuint32_t _unused_6:10;
    vuint32_t RES:1;
    vuint32_t PC:1;
    vuint32_t _unused_18:1;
    vuint32_t RU:1;
    vuint32_t BPEN:1;
    vuint32_t _unused_21:11;
  } B;
} TIMERS_TIMER3_CFG_tag;

typedef union TIMERS_TIMER3_EW_union_tag { /* TIMER3 EW */
  vuint32_t R;
  struct {
    vuint32_t EW:1;
    vuint32_t _unused_1:31;
  } B;
} TIMERS_TIMER3_EW_tag;

typedef union TIMERS_TIMER3_CC_union_tag { /* TIMER3 CC */
  vuint32_t R;
  struct {
    vuint32_t CC:32;
  } B;
} TIMERS_TIMER3_CC_tag;

typedef union TIMERS_TIMER3_SC_union_tag { /* TIMER3 SC */
  vuint32_t R;
  struct {
    vuint32_t SC:32;
  } B;
} TIMERS_TIMER3_SC_tag;

struct TIMERS_tag {
  TIMERS_TIMER0_CFG_tag TIMER0_CFG;    /* TIMER0 CFG */
  TIMERS_TIMER0_EW_tag TIMER0_EW;      /* TIMER0 EW */
  TIMERS_TIMER0_CC_tag TIMER0_CC;      /* TIMER0 CC */
  TIMERS_TIMER0_SC_tag TIMER0_SC;      /* TIMER0 SC */
  TIMERS_TIMER1_CFG_tag TIMER1_CFG;    /* TIMER1 CFG */
  TIMERS_TIMER1_EW_tag TIMER1_EW;      /* TIMER1 EW */
  TIMERS_TIMER1_CC_tag TIMER1_CC;      /* TIMER1 CC */
  TIMERS_TIMER1_SC_tag TIMER1_SC;      /* TIMER1 SC */
  TIMERS_TIMER2_CFG_tag TIMER2_CFG;    /* TIMER2 CFG */
  TIMERS_TIMER2_EW_tag TIMER2_EW;      /* TIMER2 EW */
  TIMERS_TIMER2_CC_tag TIMER2_CC;      /* TIMER2 CC */
  TIMERS_TIMER2_SC_tag TIMER2_SC;      /* TIMER2 SC */
  TIMERS_TIMER3_CFG_tag TIMER3_CFG;    /* TIMER3 CFG */
  TIMERS_TIMER3_EW_tag TIMER3_EW;      /* TIMER3 EW */
  TIMERS_TIMER3_CC_tag TIMER3_CC;      /* TIMER3 CC */
  TIMERS_TIMER3_SC_tag TIMER3_SC;      /* TIMER3 SC */
};


/* ============================================================================
   =============================== Module: TMR0_BASE ==========================
   ============================================================================ */

typedef union TMR0_BASE_TMR_ID_union_tag { /* Module ID Register */
  vuint32_t R;
  struct {
    vuint32_t REV_MN:8;                /* Minor revision */
    vuint32_t REV_MJ:8;                /* Major revision */
    vuint32_t TMR_ID:16;               /* Timer IP ID */
  } B;
} TMR0_BASE_TMR_ID_tag;

typedef union TMR0_BASE_TMR_CAPR_union_tag { /* Timer Capability Register */
  vuint32_t R;
  struct {
    vuint32_t IEEE_1722:1;
    vuint32_t ECADJ:1;
    vuint32_t IEEE_8021AS_REV:1;
    vuint32_t _unused_3:13;
    vuint32_t NUM_MSIX:1;
    vuint32_t _unused_17:15;
  } B;
} TMR0_BASE_TMR_CAPR_tag;

typedef union TMR0_BASE_TMR_FRT_L_union_tag { /* Timer free running time low register */
  vuint32_t R;
  struct {
    vuint32_t TMR_FRT_L:32;
  } B;
} TMR0_BASE_TMR_FRT_L_tag;

typedef union TMR0_BASE_TMR_FRT_H_union_tag { /* Timer free running time high register */
  vuint32_t R;
  struct {
    vuint32_t TMR_FRT_H:32;
  } B;
} TMR0_BASE_TMR_FRT_H_tag;

typedef union TMR0_BASE_TMR_SRT_L_union_tag { /* Timer synchronous time low register */
  vuint32_t R;
  struct {
    vuint32_t TMR_SRT_L:32;
  } B;
} TMR0_BASE_TMR_SRT_L_tag;

typedef union TMR0_BASE_TMR_SRT_H_union_tag { /* Timer synchronous time high register. */
  vuint32_t R;
  struct {
    vuint32_t TMR_SRT_H:32;
  } B;
} TMR0_BASE_TMR_SRT_H_tag;

typedef union TMR0_BASE_TMR_DEF_CNT_L_union_tag { /* Default ns timer counter low register */
  vuint32_t R;
  struct {
    vuint32_t TMR_DEF_CNT_L:32;
  } B;
} TMR0_BASE_TMR_DEF_CNT_L_tag;

typedef union TMR0_BASE_TMR_DEF_CNT_H_union_tag { /* Default ns timer counter high register */
  vuint32_t R;
  struct {
    vuint32_t TMR_DEF_CNT_H:32;
  } B;
} TMR0_BASE_TMR_DEF_CNT_H_tag;

typedef union TMR0_BASE_TMR_CTRL_union_tag { /* Timer Control Register */
  vuint32_t R;
  struct {
    vuint32_t CK_SEL:2;                /* 1588 timer reference clock source select */
    vuint32_t TE:1;                    /* 1588 timer enable */
    vuint32_t _unused_3:1;
    vuint32_t _unused_4:2;
    vuint32_t CIPH:1;                  /* External oscillator input clock phase */
    vuint32_t COPH:1;                  /* Generated clock (TMR_GCLK) output phase. */
    vuint32_t ETEP1:1;                 /* External trigger 1 edge polarity */
    vuint32_t ETEP2:1;                 /* External trigger 2 edge polarity */
    vuint32_t _unused_10:4;
    vuint32_t _unused_14:2;
    vuint32_t TCLK_PERIOD:10;          /* 1588 timer reference clock period */
    vuint32_t PP2L:1;                  /* Fiper2 pulse loopback mode enabled */
    vuint32_t PP1L:1;                  /* Fiper1 pulse loopback mode enabled */
    vuint32_t FS:1;                    /* FIPER start indication */
    vuint32_t _unused_29:1;
    vuint32_t ALM2P:1;                 /* Alarm2 output polarity */
    vuint32_t ALM1P:1;                 /* Alarm1 output polarity */
  } B;
} TMR0_BASE_TMR_CTRL_tag;

typedef union TMR0_BASE_TMR_TEVENT_union_tag { /* Timer Event Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t PP3EN:1;
    vuint32_t PP2EN:1;
    vuint32_t PP1EN:1;
    vuint32_t _unused_8:8;
    vuint32_t ALM1EN:1;
    vuint32_t ALM2EN:1;
    vuint32_t _unused_18:2;
    vuint32_t ETS1_THREN:1;
    vuint32_t ETS2_THREN:1;
    vuint32_t _unused_22:2;
    vuint32_t ETS1EN:1;                /* External trigger 1 new timestamp sample event available */
    vuint32_t ETS2EN:1;                /* External trigger 2 new timestamp sample event available */
    vuint32_t _unused_26:2;
    vuint32_t ETS1_OVEN:1;             /* External trigger 1 timestamp FIFO Overflow event occurred */
    vuint32_t ETS2_OVEN:1;             /* External trigger 2 timestamp FIFO Overflow event occurred */
    vuint32_t _unused_30:2;
  } B;
} TMR0_BASE_TMR_TEVENT_tag;

typedef union TMR0_BASE_TMR_TEMASK_union_tag { /* Timer event mask register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t PP3EN:1;
    vuint32_t PP2EN:1;
    vuint32_t PP1EN:1;
    vuint32_t _unused_8:8;
    vuint32_t ALM1EN:1;
    vuint32_t ALM2EN:1;
    vuint32_t _unused_18:2;
    vuint32_t ETS1_THREN:1;
    vuint32_t ETS2_THREN:1;
    vuint32_t _unused_22:2;
    vuint32_t ETS1EN:1;
    vuint32_t ETS2EN:1;
    vuint32_t _unused_26:2;
    vuint32_t ETS1_OVEN:1;
    vuint32_t ETS2_OVEN:1;
    vuint32_t _unused_30:2;
  } B;
} TMR0_BASE_TMR_TEMASK_tag;

typedef union TMR0_BASE_TMR_STAT_union_tag { /* Timer status register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t ETS1_VLD:1;
    vuint32_t ETS2_VLD:1;
    vuint32_t _unused_26:5;
    vuint32_t RCD:1;                   /* Timer Reference Clock Detected */
  } B;
} TMR0_BASE_TMR_STAT_tag;

typedef union TMR0_BASE_TMR_CNT_L_union_tag { /* Timer counter low register */
  vuint32_t R;
  struct {
    vuint32_t TMR_CNT_L:32;            /* Timer counter register. */
  } B;
} TMR0_BASE_TMR_CNT_L_tag;

typedef union TMR0_BASE_TMR_CNT_H_union_tag { /* Timer counter high register */
  vuint32_t R;
  struct {
    vuint32_t TMR_CNT_H:32;            /* Timer counter register. */
  } B;
} TMR0_BASE_TMR_CNT_H_tag;

typedef union TMR0_BASE_TMR_ADD_union_tag { /* Timer addend register */
  vuint32_t R;
  struct {
    vuint32_t ADDEND:32;               /* Timer addend. */
  } B;
} TMR0_BASE_TMR_ADD_tag;

typedef union TMR0_BASE_TMR_ACC_union_tag { /* Timer accumulator register */
  vuint32_t R;
  struct {
    vuint32_t TMR_ACC:32;              /* 32-bit timer accumulator register */
  } B;
} TMR0_BASE_TMR_ACC_tag;

typedef union TMR0_BASE_TMR_PRSC_union_tag { /* Timer prescale register */
  vuint32_t R;
  struct {
    vuint32_t PRSC_OCK:16;             /* Output clock division prescale factor. */
    vuint32_t _unused_16:16;
  } B;
} TMR0_BASE_TMR_PRSC_tag;

typedef union TMR0_BASE_TMR_ECTRL_union_tag { /* Extended timer control register */
  vuint32_t R;
  struct {
    vuint32_t ETFF_THR:4;              /* External trigger FIFO threshold. */
    vuint32_t _unused_4:28;
  } B;
} TMR0_BASE_TMR_ECTRL_tag;

typedef union TMR0_BASE_TMROFF_L_union_tag { /* Timer offset low register */
  vuint32_t R;
  struct {
    vuint32_t TMROFF_L:32;             /* Offset value of the clock counter. */
  } B;
} TMR0_BASE_TMROFF_L_tag;

typedef union TMR0_BASE_TMROFF_H_union_tag { /* Timer offset high register */
  vuint32_t R;
  struct {
    vuint32_t TMROFF_H:32;             /* Offset value of the clock counter. */
  } B;
} TMR0_BASE_TMROFF_H_tag;

typedef union TMR0_BASE_TMR_ALARM1_L_union_tag { /* Alarm 1 time comparator low register */
  vuint32_t R;
  struct {
    vuint32_t ALARM_L:32;              /* Alarm time comparator register. */
  } B;
} TMR0_BASE_TMR_ALARM1_L_tag;

typedef union TMR0_BASE_TMR_ALARM1_H_union_tag { /* Alarm 1 time comparator high register */
  vuint32_t R;
  struct {
    vuint32_t ALARM_H:32;              /* Alarm time comparator register. */
  } B;
} TMR0_BASE_TMR_ALARM1_H_tag;

typedef union TMR0_BASE_TMR_ALARM2_L_union_tag { /* Alarm 2 time comparator low register */
  vuint32_t R;
  struct {
    vuint32_t ALARM_L:32;              /* Alarm time comparator register. */
  } B;
} TMR0_BASE_TMR_ALARM2_L_tag;

typedef union TMR0_BASE_TMR_ALARM2_H_union_tag { /* Alarm 2 time comparator high register */
  vuint32_t R;
  struct {
    vuint32_t ALARM_H:32;              /* Alarm time comparator register. */
  } B;
} TMR0_BASE_TMR_ALARM2_H_tag;

typedef union TMR0_BASE_TMR_ALARM_CTRL_union_tag { /* Timer Alarm Control Register */
  vuint32_t R;
  struct {
    vuint32_t ALARM1_PW:5;             /* ALARM 1 pulse width selector */
    vuint32_t _unused_5:2;
    vuint32_t PG1:1;                   /* Alarm1 pulse generation time */
    vuint32_t ALARM2_PW:5;             /* ALARM 2 pulse width selector */
    vuint32_t _unused_13:2;
    vuint32_t PG2:1;                   /* Alarm2 pulse generation time */
    vuint32_t _unused_16:16;
  } B;
} TMR0_BASE_TMR_ALARM_CTRL_tag;

typedef union TMR0_BASE_TMR_FIPER_union_tag { /* Timer 1 fixed interval period register */
  vuint32_t R;
  struct {
    vuint32_t FIPER:32;                /* Fixed interval pulse period register. */
  } B;
} TMR0_BASE_TMR_FIPER_tag;

typedef union TMR0_BASE_TMR_FIPER_CTRL_union_tag { /* Timer FIPER Control Register */
  vuint32_t R;
  struct {
    vuint32_t FIPER1_PW:5;             /* FIPER 1 pulse width selection */
    vuint32_t _unused_5:1;
    vuint32_t PG1:1;                   /* FIPER1 pulse generation select */
    vuint32_t FIPER1_DIS:1;            /* FIPER1 disable */
    vuint32_t FIPER2_PW:5;             /* FIPER 2 pulse width selection */
    vuint32_t _unused_13:1;
    vuint32_t PG2:1;                   /* FIPER2 pulse generation time */
    vuint32_t FIPER2_DIS:1;            /* FIPER2 disable */
    vuint32_t FIPER3_PW:5;             /* FIPER 3 pulse width selection */
    vuint32_t _unused_21:1;
    vuint32_t PG3:1;                   /* FIPER3 pulse generation time */
    vuint32_t FIPER3_DIS:1;            /* FIPER3 disable */
    vuint32_t _unused_24:8;
  } B;
} TMR0_BASE_TMR_FIPER_CTRL_tag;

typedef union TMR0_BASE_TMR_ETTS1_L_union_tag { /* External trigger stamp register */
  vuint32_t R;
  struct {
    vuint32_t ETTS_L:32;
  } B;
} TMR0_BASE_TMR_ETTS1_L_tag;

typedef union TMR0_BASE_TMR_ETTS1_H_union_tag { /* External trigger stamp register */
  vuint32_t R;
  struct {
    vuint32_t ETTS_H:32;
  } B;
} TMR0_BASE_TMR_ETTS1_H_tag;

typedef union TMR0_BASE_TMR_ETTS2_L_union_tag { /* External trigger stamp register */
  vuint32_t R;
  struct {
    vuint32_t ETTS_L:32;
  } B;
} TMR0_BASE_TMR_ETTS2_L_tag;

typedef union TMR0_BASE_TMR_ETTS2_H_union_tag { /* External trigger stamp register */
  vuint32_t R;
  struct {
    vuint32_t ETTS_H:32;
  } B;
} TMR0_BASE_TMR_ETTS2_H_tag;

typedef union TMR0_BASE_TMR_CUR_TIME_L_union_tag { /* Timer current time low register */
  vuint32_t R;
  struct {
    vuint32_t TMR_CUR_TIME_L:32;
  } B;
} TMR0_BASE_TMR_CUR_TIME_L_tag;

typedef union TMR0_BASE_TMR_CUR_TIME_H_union_tag { /* Timer current time high register */
  vuint32_t R;
  struct {
    vuint32_t TMR_CUR_TIME_H:32;
  } B;
} TMR0_BASE_TMR_CUR_TIME_H_tag;

typedef union TMR0_BASE_TMR_PARAM_union_tag { /* Timer parameter register */
  vuint32_t R;
  struct {
    vuint32_t SYNC:1;                  /* Timer synchronization */
    vuint32_t PARAM_VAL:31;            /* User specific parameter values */
  } B;
} TMR0_BASE_TMR_PARAM_tag;

struct TMR0_BASE_tag {
  TMR0_BASE_TMR_ID_tag TMR_ID;         /* Module ID Register */
  uint8_t TMR0_BASE_reserved0[4];
  TMR0_BASE_TMR_CAPR_tag TMR_CAPR;     /* Timer Capability Register */
  uint8_t TMR0_BASE_reserved1[20];
  TMR0_BASE_TMR_FRT_L_tag TMR_FRT_L;   /* Timer free running time low register */
  TMR0_BASE_TMR_FRT_H_tag TMR_FRT_H;   /* Timer free running time high register */
  TMR0_BASE_TMR_SRT_L_tag TMR_SRT_L;   /* Timer synchronous time low register */
  TMR0_BASE_TMR_SRT_H_tag TMR_SRT_H;   /* Timer synchronous time high register. */
  TMR0_BASE_TMR_DEF_CNT_L_tag TMR_DEF_CNT_L; /* Default ns timer counter low register */
  TMR0_BASE_TMR_DEF_CNT_H_tag TMR_DEF_CNT_H; /* Default ns timer counter high register */
  uint8_t TMR0_BASE_reserved2[72];
  TMR0_BASE_TMR_CTRL_tag TMR_CTRL;     /* Timer Control Register */
  TMR0_BASE_TMR_TEVENT_tag TMR_TEVENT; /* Timer Event Register */
  TMR0_BASE_TMR_TEMASK_tag TMR_TEMASK; /* Timer event mask register */
  uint8_t TMR0_BASE_reserved3[8];
  TMR0_BASE_TMR_STAT_tag TMR_STAT;     /* Timer status register */
  TMR0_BASE_TMR_CNT_L_tag TMR_CNT_L;   /* Timer counter low register */
  TMR0_BASE_TMR_CNT_H_tag TMR_CNT_H;   /* Timer counter high register */
  TMR0_BASE_TMR_ADD_tag TMR_ADD;       /* Timer addend register */
  TMR0_BASE_TMR_ACC_tag TMR_ACC;       /* Timer accumulator register */
  TMR0_BASE_TMR_PRSC_tag TMR_PRSC;     /* Timer prescale register */
  TMR0_BASE_TMR_ECTRL_tag TMR_ECTRL;   /* Extended timer control register */
  TMR0_BASE_TMROFF_L_tag TMROFF_L;     /* Timer offset low register */
  TMR0_BASE_TMROFF_H_tag TMROFF_H;     /* Timer offset high register */
  TMR0_BASE_TMR_ALARM1_L_tag TMR_ALARM1_L; /* Alarm 1 time comparator low register */
  TMR0_BASE_TMR_ALARM1_H_tag TMR_ALARM1_H; /* Alarm 1 time comparator high register */
  TMR0_BASE_TMR_ALARM2_L_tag TMR_ALARM2_L; /* Alarm 2 time comparator low register */
  TMR0_BASE_TMR_ALARM2_H_tag TMR_ALARM2_H; /* Alarm 2 time comparator high register */
  uint8_t TMR0_BASE_reserved4[4];
  TMR0_BASE_TMR_ALARM_CTRL_tag TMR_ALARM_CTRL; /* Timer Alarm Control Register */
  TMR0_BASE_TMR_FIPER_tag TMR_FIPER[3]; /* Timer 1 fixed interval period register */
  TMR0_BASE_TMR_FIPER_CTRL_tag TMR_FIPER_CTRL; /* Timer FIPER Control Register */
  TMR0_BASE_TMR_ETTS1_L_tag TMR_ETTS1_L; /* External trigger stamp register */
  TMR0_BASE_TMR_ETTS1_H_tag TMR_ETTS1_H; /* External trigger stamp register */
  TMR0_BASE_TMR_ETTS2_L_tag TMR_ETTS2_L; /* External trigger stamp register */
  TMR0_BASE_TMR_ETTS2_H_tag TMR_ETTS2_H; /* External trigger stamp register */
  TMR0_BASE_TMR_CUR_TIME_L_tag TMR_CUR_TIME_L; /* Timer current time low register */
  TMR0_BASE_TMR_CUR_TIME_H_tag TMR_CUR_TIME_H; /* Timer current time high register */
  TMR0_BASE_TMR_PARAM_tag TMR_PARAM;   /* Timer parameter register */
};


/* ============================================================================
   =============================== Module: TMU ================================
   ============================================================================ */

typedef union TMU_TMR_union_tag {      /* Mode */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t ALPF:2;                  /* Average Low Pass Filter Setting */
    vuint32_t _unused_26:3;
    vuint32_t CMD:1;                   /* Central Module Disable */
    vuint32_t MODE:2;                  /* Mode */
  } B;
} TMU_TMR_tag;

typedef union TMU_TSR_union_tag {      /* Status */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t ORH:1;                   /* Out-of-Range High Temperature Measurement */
    vuint32_t ORL:1;                   /* Out-of-Range Low Temperature Measurement */
    vuint32_t MIE:1;                   /* Monitoring Interval Exceeded */
    vuint32_t TB:1;                    /* TMU Busy */
  } B;
} TMU_TSR_tag;

typedef union TMU_TMSR_union_tag {     /* Monitor Site */
  vuint32_t R;
  struct {
    vuint32_t SITE:5;                  /* Monitoring Site Select 4-0 */
    vuint32_t _unused_5:11;
    vuint32_t _unused_16:16;
  } B;
} TMU_TMSR_tag;

typedef union TMU_TMTMIR_union_tag {   /* Monitor Temperature Measurement Interval */
  vuint32_t R;
  struct {
    vuint32_t TMI:4;                   /* Temperature Monitoring Interval in Seconds */
    vuint32_t _unused_4:28;
  } B;
} TMU_TMTMIR_tag;

typedef union TMU_TIER_union_tag {     /* Interrupt Enable */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t FTRCTIE:1;               /* Falling Temperature Rate Critical Threshold Interrupt Enable */
    vuint32_t RTRCTIE:1;               /* Rising Temperature Rate Critical Threshold Interrupt Enable */
    vuint32_t ALTCTIE:1;               /* Average Low Temperature Critical Threshold Interrupt Enable */
    vuint32_t ALTTIE:1;                /* Average Low Temperature Threshold Interrupt Enable */
    vuint32_t ILTTIE:1;                /* Immediate Low Temperature Threshold Interrupt Enable */
    vuint32_t AHTCTIE:1;               /* Average High Temperature Critical Threshold Interrupt Enable */
    vuint32_t AHTTIE:1;                /* Average High Temperature Threshold Interrupt Enable */
    vuint32_t IHTTIE:1;                /* Immediate High Temperature Threshold Interrupt Enable */
  } B;
} TMU_TIER_tag;

typedef union TMU_TIDR_union_tag {     /* Interrupt Detect */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:24;
    vuint32_t FTRCT:1;                 /* Falling Temperature Rate Critical Threshold */
    vuint32_t RTRCT:1;                 /* Rising Temperature Rate Critical Threshold */
    vuint32_t ALTCT:1;                 /* Average Low Temperature Critical Threshold */
    vuint32_t ALTT:1;                  /* Average Low Temperature Threshold */
    vuint32_t ILTT:1;                  /* Immediate Low Temperature Threshold */
    vuint32_t AHTCT:1;                 /* Average High Temperature Critical Threshold Exceeded */
    vuint32_t AHTT:1;                  /* Average High Temperature Threshold Exceeded */
    vuint32_t IHTT:1;                  /* Immediate High Temperature Threshold Exceeded */
  } B;
} TMU_TIDR_tag;

typedef union TMU_TIISCR_union_tag {   /* Interrupt Immediate Site Capture */
  vuint32_t R;
  struct {
    vuint32_t SITE:5;                  /* Temperature Sensor Site */
    vuint32_t _unused_5:11;
    vuint32_t _unused_16:16;
  } B;
} TMU_TIISCR_tag;

typedef union TMU_TIASCR_union_tag {   /* Interrupt Average Site Capture */
  vuint32_t R;
  struct {
    vuint32_t SITE:5;                  /* Temperature Sensor Site */
    vuint32_t _unused_5:11;
    vuint32_t _unused_16:16;
  } B;
} TMU_TIASCR_tag;

typedef union TMU_TICSCR_union_tag {   /* Interrupt Critical Site Capture */
  vuint32_t R;
  struct {
    vuint32_t SITE:5;                  /* Temperature Sensor Site */
    vuint32_t _unused_5:11;
    vuint32_t _unused_16:16;
  } B;
} TMU_TICSCR_tag;

typedef union TMU_TMHTCR_union_tag {   /* Monitor High Temperature Capture */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* Highest Temperature Recorded in Kelvin by Any Enabled Monitored Site */
    vuint32_t TP5:1;                   /* Highest Temperature Recorded in Kelvin by Any Enabled Monitored Site */
    vuint32_t _unused_10:21;
    vuint32_t V:1;                     /* Valid Reading */
  } B;
} TMU_TMHTCR_tag;

typedef union TMU_TMLTCR_union_tag {   /* Monitor Low Temperature Capture */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* Lowest Temperature in Kelvin that Any Enabled Monitored Site Records */
    vuint32_t TP5:1;                   /* Lowest Temperature in Kelvin that Any Enabled Monitored Site Records */
    vuint32_t _unused_10:21;
    vuint32_t V:1;                     /* Valid Reading */
  } B;
} TMU_TMLTCR_tag;

typedef union TMU_TMRTRCR_union_tag {  /* Monitor Rising Temperature Rate Capture */
  vuint32_t R;
  struct {
    vuint32_t TEMP:8;                  /* Highest Rising Temperature Rate Change in Kelvin that Any Enabled Monitored Site Records */
    vuint32_t _unused_8:23;
    vuint32_t V:1;                     /* Valid Reading */
  } B;
} TMU_TMRTRCR_tag;

typedef union TMU_TMFTRCR_union_tag {  /* Monitor Falling Temperature Rate Capture */
  vuint32_t R;
  struct {
    vuint32_t TEMP:8;                  /* Highest Falling Temperature Rate Change in Kelvin that Any Enabled Monitored Site Records */
    vuint32_t _unused_8:23;
    vuint32_t V:1;                     /* Valid Reading */
  } B;
} TMU_TMFTRCR_tag;

typedef union TMU_TMHTITR_union_tag {  /* Monitor High Temperature Immediate Threshold */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* High Temperature Immediate Threshold Value */
    vuint32_t _unused_9:22;
    vuint32_t EN:1;                    /* Enable Threshold */
  } B;
} TMU_TMHTITR_tag;

typedef union TMU_TMHTATR_union_tag {  /* Monitor High Temperature Average Threshold */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* High Temperature Average Threshold Value */
    vuint32_t _unused_9:22;
    vuint32_t EN:1;                    /* Enable Threshold */
  } B;
} TMU_TMHTATR_tag;

typedef union TMU_TMHTACTR_union_tag { /* Monitor High Temperature Average Critical Threshold */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* High Temperature Average Critical Threshold Value */
    vuint32_t _unused_9:22;
    vuint32_t EN:1;                    /* Enable Threshold */
  } B;
} TMU_TMHTACTR_tag;

typedef union TMU_TMLTITR_union_tag {  /* Monitor Low Temperature Immediate Threshold */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* Low Temperature Immediate Threshold Value */
    vuint32_t _unused_9:22;
    vuint32_t EN:1;                    /* Enable Threshold */
  } B;
} TMU_TMLTITR_tag;

typedef union TMU_TMLTATR_union_tag {  /* Monitor Low Temperature Average Threshold */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* Low Temperature Average Threshold Value */
    vuint32_t _unused_9:22;
    vuint32_t EN:1;                    /* Enable Threshold */
  } B;
} TMU_TMLTATR_tag;

typedef union TMU_TMLTACTR_union_tag { /* Monitor Low Temperature Average Critical Threshold */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* Low Temperature Average Critical Threshold Value */
    vuint32_t _unused_9:22;
    vuint32_t EN:1;                    /* Enable Threshold */
  } B;
} TMU_TMLTACTR_tag;

typedef union TMU_TMRTRCTR_union_tag { /* Monitor Rising Temperature Rate Critical Threshold */
  vuint32_t R;
  struct {
    vuint32_t TEMP:8;                  /* Temperature Difference Between Two Measurements From the Same Site */
    vuint32_t _unused_8:23;
    vuint32_t EN:1;                    /* Enable Threshold */
  } B;
} TMU_TMRTRCTR_tag;

typedef union TMU_TMFTRCTR_union_tag { /* Monitor Falling Temperature Rate Critical Threshold */
  vuint32_t R;
  struct {
    vuint32_t TEMP:8;                  /* Temperature Difference Between Two Measurements From the Same Site */
    vuint32_t _unused_8:23;
    vuint32_t EN:1;                    /* Enable threshold */
  } B;
} TMU_TMFTRCTR_tag;

typedef union TMU_TTCFGR_union_tag {   /* Temperature Configuration */
  vuint32_t R;
  struct {
    vuint32_t CAL_PT:4;                /* Calibration Point */
    vuint32_t _unused_4:28;
  } B;
} TMU_TTCFGR_tag;

typedef union TMU_TSCFGR_union_tag {   /* Sensor Configuration */
  vuint32_t R;
  struct {
    vuint32_t SENSOR:9;                /* Sensor Value */
    vuint32_t _unused_9:23;
  } B;
} TMU_TSCFGR_tag;

typedef union TMU_TRITRATSR_TRITSR_union_tag { /* Report Immediate Temperature at Site */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* Last Temperature Reading in Kelvin at the Site */
    vuint32_t TP5:1;                   /* Last Temperature Reading in Kelvin at the Site */
    vuint32_t _unused_10:21;
    vuint32_t V:1;                     /* Valid Measured Temperature */
  } B;
} TMU_TRITRATSR_TRITSR_tag;

typedef union TMU_TRITRATSR_TRATSR_union_tag { /* Report Average Temperature at Site */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* Average Temperature Reading in Kelvin at the Site */
    vuint32_t _unused_9:22;
    vuint32_t V:1;                     /* Valid Measured Temperature */
  } B;
} TMU_TRITRATSR_TRATSR_tag;

typedef struct TMU_TRITRATSR_struct_tag {
  TMU_TRITRATSR_TRITSR_tag TRITSR;     /* Report Immediate Temperature at Site */
  TMU_TRITRATSR_TRATSR_tag TRATSR;     /* Report Average Temperature at Site */
  uint8_t TRITRATSR_reserved0[8];
} TMU_TRITRATSR_tag;

typedef union TMU_TCMCFG_union_tag {   /* Central Module Configuration */
  vuint32_t R;
  struct {
    vuint32_t DAC_OFFSET:7;            /* DAC Offset */
    vuint32_t _unused_7:1;
    vuint32_t CMET:2;                  /* Central Module Enable Time */
    vuint32_t DFD:2;                   /* Digital Filter Depth */
    vuint32_t CLK_DIV:4;               /* Clock Divider */
    vuint32_t SAR_RDY:1;               /* SAR Ready */
    vuint32_t _unused_17:7;
    vuint32_t RCTC:3;                  /* RC Time Constant Setting */
    vuint32_t _unused_27:1;
    vuint32_t DEMA:1;                  /* Dynamic Element Match Averaging Mode */
    vuint32_t OCS:1;                   /* Offset Cancellation Manual Setting */
    vuint32_t OCM:1;                   /* Offset Cancellation Mode */
    vuint32_t DPM:1;                   /* Dynamic Power Management */
  } B;
} TMU_TCMCFG_tag;

typedef union TMU_TTRCR_union_tag {    /* Temperature Range Control 0 */
  vuint32_t R;
  struct {
    vuint32_t TEMP:9;                  /* Temperature in Kelvin for the Calibration Point */
    vuint32_t _unused_9:22;
    vuint32_t V:1;                     /* Calibration Point is Valid */
  } B;
} TMU_TTRCR_tag;

struct TMU_tag {
  TMU_TMR_tag TMR;                     /* Mode */
  TMU_TSR_tag TSR;                     /* Status */
  TMU_TMSR_tag TMSR;                   /* Monitor Site */
  TMU_TMTMIR_tag TMTMIR;               /* Monitor Temperature Measurement Interval */
  uint8_t TMU_reserved0[16];
  TMU_TIER_tag TIER;                   /* Interrupt Enable */
  TMU_TIDR_tag TIDR;                   /* Interrupt Detect */
  uint8_t TMU_reserved1[8];
  TMU_TIISCR_tag TIISCR;               /* Interrupt Immediate Site Capture */
  TMU_TIASCR_tag TIASCR;               /* Interrupt Average Site Capture */
  TMU_TICSCR_tag TICSCR;               /* Interrupt Critical Site Capture */
  uint8_t TMU_reserved2[4];
  TMU_TMHTCR_tag TMHTCR;               /* Monitor High Temperature Capture */
  TMU_TMLTCR_tag TMLTCR;               /* Monitor Low Temperature Capture */
  TMU_TMRTRCR_tag TMRTRCR;             /* Monitor Rising Temperature Rate Capture */
  TMU_TMFTRCR_tag TMFTRCR;             /* Monitor Falling Temperature Rate Capture */
  TMU_TMHTITR_tag TMHTITR;             /* Monitor High Temperature Immediate Threshold */
  TMU_TMHTATR_tag TMHTATR;             /* Monitor High Temperature Average Threshold */
  TMU_TMHTACTR_tag TMHTACTR;           /* Monitor High Temperature Average Critical Threshold */
  uint8_t TMU_reserved3[4];
  TMU_TMLTITR_tag TMLTITR;             /* Monitor Low Temperature Immediate Threshold */
  TMU_TMLTATR_tag TMLTATR;             /* Monitor Low Temperature Average Threshold */
  TMU_TMLTACTR_tag TMLTACTR;           /* Monitor Low Temperature Average Critical Threshold */
  uint8_t TMU_reserved4[4];
  TMU_TMRTRCTR_tag TMRTRCTR;           /* Monitor Rising Temperature Rate Critical Threshold */
  TMU_TMFTRCTR_tag TMFTRCTR;           /* Monitor Falling Temperature Rate Critical Threshold */
  uint8_t TMU_reserved5[8];
  TMU_TTCFGR_tag TTCFGR;               /* Temperature Configuration */
  TMU_TSCFGR_tag TSCFGR;               /* Sensor Configuration */
  uint8_t TMU_reserved6[120];
  TMU_TRITRATSR_tag TRITRATSR[5];
  uint8_t TMU_reserved7[3504];
  TMU_TCMCFG_tag TCMCFG;               /* Central Module Configuration */
  uint8_t TMU_reserved8[12];
  TMU_TTRCR_tag TTRCR[16];             /* Temperature Range Control 0 */
};


/* ============================================================================
   =============================== Module: TRGMUX_0 ===========================
   ============================================================================ */

typedef union TRGMUX_0_REG_0_union_tag { /* TRGMUX REG_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t SEL2:6;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_22:2;
    vuint32_t SEL3:6;                  /* TRGMUX Source Select 3 */
    vuint32_t _unused_30:1;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_0_REG_0_tag;

typedef union TRGMUX_0_REG_1_union_tag { /* TRGMUX REG_1 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t SEL2:6;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_22:2;
    vuint32_t SEL3:6;                  /* TRGMUX Source Select 3 */
    vuint32_t _unused_30:1;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_0_REG_1_tag;

typedef union TRGMUX_0_REG_2_union_tag { /* TRGMUX REG_2 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t SEL2:6;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_22:2;
    vuint32_t SEL3:6;                  /* TRGMUX Source Select 3 */
    vuint32_t _unused_30:1;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_0_REG_2_tag;

typedef union TRGMUX_0_REG_3_union_tag { /* TRGMUX REG_3 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t SEL2:6;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_22:2;
    vuint32_t SEL3:6;                  /* TRGMUX Source Select 3 */
    vuint32_t _unused_30:1;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_0_REG_3_tag;

typedef union TRGMUX_0_REG_4_union_tag { /* TRGMUX REG_4 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_0_REG_4_tag;

struct TRGMUX_0_tag {
  TRGMUX_0_REG_0_tag REG_0;            /* TRGMUX REG_0 */
  TRGMUX_0_REG_1_tag REG_1;            /* TRGMUX REG_1 */
  TRGMUX_0_REG_2_tag REG_2;            /* TRGMUX REG_2 */
  TRGMUX_0_REG_3_tag REG_3;            /* TRGMUX REG_3 */
  TRGMUX_0_REG_4_tag REG_4;            /* TRGMUX REG_4 */
};


/* ============================================================================
   =============================== Module: TRGMUX_1 ===========================
   ============================================================================ */

typedef union TRGMUX_1_REG_0_union_tag { /* TRGMUX REG_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t SEL2:6;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_22:2;
    vuint32_t SEL3:6;                  /* TRGMUX Source Select 3 */
    vuint32_t _unused_30:1;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_1_REG_0_tag;

typedef union TRGMUX_1_REG_1_union_tag { /* TRGMUX REG_1 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t SEL2:6;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_22:2;
    vuint32_t SEL3:6;                  /* TRGMUX Source Select 3 */
    vuint32_t _unused_30:1;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_1_REG_1_tag;

typedef union TRGMUX_1_REG_2_union_tag { /* TRGMUX REG_2 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t SEL2:6;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_22:2;
    vuint32_t SEL3:6;                  /* TRGMUX Source Select 3 */
    vuint32_t _unused_30:1;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_1_REG_2_tag;

typedef union TRGMUX_1_REG_3_union_tag { /* TRGMUX REG_3 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t SEL2:6;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_22:2;
    vuint32_t SEL3:6;                  /* TRGMUX Source Select 3 */
    vuint32_t _unused_30:1;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_1_REG_3_tag;

typedef union TRGMUX_1_REG_4_union_tag { /* TRGMUX REG_4 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:6;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_6:2;
    vuint32_t SEL1:6;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_14:2;
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_1_REG_4_tag;

struct TRGMUX_1_tag {
  TRGMUX_1_REG_0_tag REG_0;            /* TRGMUX REG_0 */
  TRGMUX_1_REG_1_tag REG_1;            /* TRGMUX REG_1 */
  TRGMUX_1_REG_2_tag REG_2;            /* TRGMUX REG_2 */
  TRGMUX_1_REG_3_tag REG_3;            /* TRGMUX REG_3 */
  TRGMUX_1_REG_4_tag REG_4;            /* TRGMUX REG_4 */
};


/* ============================================================================
   =============================== Module: TRGMUX_2 ===========================
   ============================================================================ */

typedef union TRGMUX_2_TIM0_1_0_union_tag { /* TRGMUX TIM0_1_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM0_1_0_tag;

typedef union TRGMUX_2_TIM0_3_2_union_tag { /* TRGMUX TIM0_3_2 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM0_3_2_tag;

typedef union TRGMUX_2_TIM0_5_4_union_tag { /* TRGMUX TIM0_5_4 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM0_5_4_tag;

typedef union TRGMUX_2_TIM0_7_6_union_tag { /* TRGMUX TIM0_7_6 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM0_7_6_tag;

typedef union TRGMUX_2_TIM1_1_0_union_tag { /* TRGMUX TIM1_1_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM1_1_0_tag;

typedef union TRGMUX_2_TIM1_3_2_union_tag { /* TRGMUX TIM1_3_2 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM1_3_2_tag;

typedef union TRGMUX_2_TIM1_5_4_union_tag { /* TRGMUX TIM1_5_4 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM1_5_4_tag;

typedef union TRGMUX_2_TIM1_7_6_union_tag { /* TRGMUX TIM1_7_6 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM1_7_6_tag;

typedef union TRGMUX_2_TIM2_1_0_union_tag { /* TRGMUX TIM2_1_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM2_1_0_tag;

typedef union TRGMUX_2_TIM2_3_2_union_tag { /* TRGMUX TIM2_3_2 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM2_3_2_tag;

typedef union TRGMUX_2_TIM2_5_4_union_tag { /* TRGMUX TIM2_5_4 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM2_5_4_tag;

typedef union TRGMUX_2_TIM2_7_6_union_tag { /* TRGMUX TIM2_7_6 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIM2_7_6_tag;

typedef union TRGMUX_2_TIO1_1_0_union_tag { /* TRGMUX TIO1_1_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO1_1_0_tag;

typedef union TRGMUX_2_TIO1_3_2_union_tag { /* TRGMUX TIO1_3_2 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO1_3_2_tag;

typedef union TRGMUX_2_TIO1_5_4_union_tag { /* TRGMUX TIO1_5_4 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO1_5_4_tag;

typedef union TRGMUX_2_TIO1_7_6_union_tag { /* TRGMUX TIO1_7_6 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO1_7_6_tag;

typedef union TRGMUX_2_TIO2_1_0_union_tag { /* TRGMUX TIO2_1_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO2_1_0_tag;

typedef union TRGMUX_2_TIO2_3_2_union_tag { /* TRGMUX TIO2_3_2 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO2_3_2_tag;

typedef union TRGMUX_2_TIO2_5_4_union_tag { /* TRGMUX TIO2_5_4 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO2_5_4_tag;

typedef union TRGMUX_2_TIO2_7_6_union_tag { /* TRGMUX TIO2_7_6 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO2_7_6_tag;

typedef union TRGMUX_2_TIO3_1_0_union_tag { /* TRGMUX TIO3_1_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO3_1_0_tag;

typedef union TRGMUX_2_TIO3_3_2_union_tag { /* TRGMUX TIO3_3_2 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO3_3_2_tag;

typedef union TRGMUX_2_TIO3_5_4_union_tag { /* TRGMUX TIO3_5_4 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO3_5_4_tag;

typedef union TRGMUX_2_TIO3_7_6_union_tag { /* TRGMUX TIO3_7_6 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:8;                  /* TRGMUX Input Source Select 0 */
    vuint32_t SEL1:8;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_16:8;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_2_TIO3_7_6_tag;

struct TRGMUX_2_tag {
  TRGMUX_2_TIM0_1_0_tag TIM0_1_0;      /* TRGMUX TIM0_1_0 */
  TRGMUX_2_TIM0_3_2_tag TIM0_3_2;      /* TRGMUX TIM0_3_2 */
  TRGMUX_2_TIM0_5_4_tag TIM0_5_4;      /* TRGMUX TIM0_5_4 */
  TRGMUX_2_TIM0_7_6_tag TIM0_7_6;      /* TRGMUX TIM0_7_6 */
  TRGMUX_2_TIM1_1_0_tag TIM1_1_0;      /* TRGMUX TIM1_1_0 */
  TRGMUX_2_TIM1_3_2_tag TIM1_3_2;      /* TRGMUX TIM1_3_2 */
  TRGMUX_2_TIM1_5_4_tag TIM1_5_4;      /* TRGMUX TIM1_5_4 */
  TRGMUX_2_TIM1_7_6_tag TIM1_7_6;      /* TRGMUX TIM1_7_6 */
  TRGMUX_2_TIM2_1_0_tag TIM2_1_0;      /* TRGMUX TIM2_1_0 */
  TRGMUX_2_TIM2_3_2_tag TIM2_3_2;      /* TRGMUX TIM2_3_2 */
  TRGMUX_2_TIM2_5_4_tag TIM2_5_4;      /* TRGMUX TIM2_5_4 */
  TRGMUX_2_TIM2_7_6_tag TIM2_7_6;      /* TRGMUX TIM2_7_6 */
  TRGMUX_2_TIO1_1_0_tag TIO1_1_0;      /* TRGMUX TIO1_1_0 */
  TRGMUX_2_TIO1_3_2_tag TIO1_3_2;      /* TRGMUX TIO1_3_2 */
  TRGMUX_2_TIO1_5_4_tag TIO1_5_4;      /* TRGMUX TIO1_5_4 */
  TRGMUX_2_TIO1_7_6_tag TIO1_7_6;      /* TRGMUX TIO1_7_6 */
  TRGMUX_2_TIO2_1_0_tag TIO2_1_0;      /* TRGMUX TIO2_1_0 */
  TRGMUX_2_TIO2_3_2_tag TIO2_3_2;      /* TRGMUX TIO2_3_2 */
  TRGMUX_2_TIO2_5_4_tag TIO2_5_4;      /* TRGMUX TIO2_5_4 */
  TRGMUX_2_TIO2_7_6_tag TIO2_7_6;      /* TRGMUX TIO2_7_6 */
  TRGMUX_2_TIO3_1_0_tag TIO3_1_0;      /* TRGMUX TIO3_1_0 */
  TRGMUX_2_TIO3_3_2_tag TIO3_3_2;      /* TRGMUX TIO3_3_2 */
  TRGMUX_2_TIO3_5_4_tag TIO3_5_4;      /* TRGMUX TIO3_5_4 */
  TRGMUX_2_TIO3_7_6_tag TIO3_7_6;      /* TRGMUX TIO3_7_6 */
};


/* ============================================================================
   =============================== Module: TRGMUX_3 ===========================
   ============================================================================ */

typedef union TRGMUX_3_GTM_003_000_union_tag { /* TRGMUX GTM_003_000 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_003_000_tag;

typedef union TRGMUX_3_GTM_007_004_union_tag { /* TRGMUX GTM_007_004 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_007_004_tag;

typedef union TRGMUX_3_GTM_011_008_union_tag { /* TRGMUX GTM_011_008 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_011_008_tag;

typedef union TRGMUX_3_GTM_015_012_union_tag { /* TRGMUX GTM_015_012 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_015_012_tag;

typedef union TRGMUX_3_GTM_019_016_union_tag { /* TRGMUX GTM_019_016 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_019_016_tag;

typedef union TRGMUX_3_GTM_023_020_union_tag { /* TRGMUX GTM_023_020 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_023_020_tag;

typedef union TRGMUX_3_GTM_027_024_union_tag { /* TRGMUX GTM_027_024 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_027_024_tag;

typedef union TRGMUX_3_GTM_031_028_union_tag { /* TRGMUX GTM_031_028 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_031_028_tag;

typedef union TRGMUX_3_GTM_035_032_union_tag { /* TRGMUX GTM_035_032 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_035_032_tag;

typedef union TRGMUX_3_GTM_039_036_union_tag { /* TRGMUX GTM_039_036 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_039_036_tag;

typedef union TRGMUX_3_GTM_043_040_union_tag { /* TRGMUX GTM_043_040 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_043_040_tag;

typedef union TRGMUX_3_GTM_047_044_union_tag { /* TRGMUX GTM_047_044 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_047_044_tag;

typedef union TRGMUX_3_GTM_051_048_union_tag { /* TRGMUX GTM_051_048 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_051_048_tag;

typedef union TRGMUX_3_GTM_055_052_union_tag { /* TRGMUX GTM_055_052 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_055_052_tag;

typedef union TRGMUX_3_GTM_059_056_union_tag { /* TRGMUX GTM_059_056 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_059_056_tag;

typedef union TRGMUX_3_GTM_063_060_union_tag { /* TRGMUX GTM_063_060 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_063_060_tag;

typedef union TRGMUX_3_GTM_067_064_union_tag { /* TRGMUX GTM_067_064 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_067_064_tag;

typedef union TRGMUX_3_GTM_071_068_union_tag { /* TRGMUX GTM_071_068 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_071_068_tag;

typedef union TRGMUX_3_GTM_075_072_union_tag { /* TRGMUX GTM_075_072 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_075_072_tag;

typedef union TRGMUX_3_GTM_079_076_union_tag { /* TRGMUX GTM_079_076 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_079_076_tag;

typedef union TRGMUX_3_GTM_083_080_union_tag { /* TRGMUX GTM_083_080 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_083_080_tag;

typedef union TRGMUX_3_GTM_087_084_union_tag { /* TRGMUX GTM_087_084 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_087_084_tag;

typedef union TRGMUX_3_GTM_091_088_union_tag { /* TRGMUX GTM_091_088 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_091_088_tag;

typedef union TRGMUX_3_GTM_095_092_union_tag { /* TRGMUX GTM_095_092 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_095_092_tag;

typedef union TRGMUX_3_GTM_099_096_union_tag { /* TRGMUX GTM_099_096 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_099_096_tag;

typedef union TRGMUX_3_GTM_103_100_union_tag { /* TRGMUX GTM_103_100 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_103_100_tag;

typedef union TRGMUX_3_GTM_107_104_union_tag { /* TRGMUX GTM_107_104 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_107_104_tag;

typedef union TRGMUX_3_GTM_111_108_union_tag { /* TRGMUX GTM_111_108 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_111_108_tag;

typedef union TRGMUX_3_GTM_115_112_union_tag { /* TRGMUX GTM_115_112 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_115_112_tag;

typedef union TRGMUX_3_GTM_119_116_union_tag { /* TRGMUX GTM_119_116 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_119_116_tag;

typedef union TRGMUX_3_GTM_123_120_union_tag { /* TRGMUX GTM_123_120 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_123_120_tag;

typedef union TRGMUX_3_GTM_127_124_union_tag { /* TRGMUX GTM_127_124 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_127_124_tag;

typedef union TRGMUX_3_GTM_131_128_union_tag { /* TRGMUX GTM_131_128 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_131_128_tag;

typedef union TRGMUX_3_GTM_135_132_union_tag { /* TRGMUX GTM_135_132 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_135_132_tag;

typedef union TRGMUX_3_GTM_139_136_union_tag { /* TRGMUX GTM_139_136 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_139_136_tag;

typedef union TRGMUX_3_GTM_143_140_union_tag { /* TRGMUX GTM_143_140 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_143_140_tag;

typedef union TRGMUX_3_GTM_147_144_union_tag { /* TRGMUX GTM_147_144 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_147_144_tag;

typedef union TRGMUX_3_GTM_151_148_union_tag { /* TRGMUX GTM_151_148 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_151_148_tag;

typedef union TRGMUX_3_GTM_155_152_union_tag { /* TRGMUX GTM_155_152 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_155_152_tag;

typedef union TRGMUX_3_GTM_159_156_union_tag { /* TRGMUX GTM_159_156 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_159_156_tag;

typedef union TRGMUX_3_GTM_163_160_union_tag { /* TRGMUX GTM_163_160 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_163_160_tag;

typedef union TRGMUX_3_GTM_167_164_union_tag { /* TRGMUX GTM_167_164 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_167_164_tag;

typedef union TRGMUX_3_GTM_168_union_tag { /* TRGMUX GTM_168 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t _unused_8:7;
    vuint32_t _unused_15:1;
    vuint32_t _unused_16:7;
    vuint32_t _unused_23:1;
    vuint32_t _unused_24:7;
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_GTM_168_tag;

typedef union TRGMUX_3_MSC_03_00_union_tag { /* TRGMUX MSC_03_00 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_03_00_tag;

typedef union TRGMUX_3_MSC_07_04_union_tag { /* TRGMUX MSC_07_04 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_07_04_tag;

typedef union TRGMUX_3_MSC_11_08_union_tag { /* TRGMUX MSC_11_08 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_11_08_tag;

typedef union TRGMUX_3_MSC_15_12_union_tag { /* TRGMUX MSC_15_12 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_15_12_tag;

typedef union TRGMUX_3_MSC_19_16_union_tag { /* TRGMUX MSC_19_16 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_19_16_tag;

typedef union TRGMUX_3_MSC_23_20_union_tag { /* TRGMUX MSC_23_20 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_23_20_tag;

typedef union TRGMUX_3_MSC_27_24_union_tag { /* TRGMUX MSC_27_24 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_27_24_tag;

typedef union TRGMUX_3_MSC_31_28_union_tag { /* TRGMUX MSC_31_28 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_31_28_tag;

typedef union TRGMUX_3_MSC_35_32_union_tag { /* TRGMUX MSC_35_32 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_35_32_tag;

typedef union TRGMUX_3_MSC_39_36_union_tag { /* TRGMUX MSC_39_36 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_39_36_tag;

typedef union TRGMUX_3_MSC_43_40_union_tag { /* TRGMUX MSC_43_40 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_43_40_tag;

typedef union TRGMUX_3_MSC_47_44_union_tag { /* TRGMUX MSC_47_44 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_47_44_tag;

typedef union TRGMUX_3_MSC_51_48_union_tag { /* TRGMUX MSC_51_48 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_51_48_tag;

typedef union TRGMUX_3_MSC_55_52_union_tag { /* TRGMUX MSC_55_52 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_55_52_tag;

typedef union TRGMUX_3_MSC_59_56_union_tag { /* TRGMUX MSC_59_56 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_59_56_tag;

typedef union TRGMUX_3_MSC_63_60_union_tag { /* TRGMUX MSC_63_60 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_MSC_63_60_tag;

typedef union TRGMUX_3_CTU_3_0_union_tag { /* TRGMUX CTU_3_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_CTU_3_0_tag;

typedef union TRGMUX_3_SINC_3_0_union_tag { /* TRGMUX SINC_3_0 */
  vuint32_t R;
  struct {
    vuint32_t SEL0:7;                  /* TRGMUX Source Select 0 */
    vuint32_t _unused_7:1;
    vuint32_t SEL1:7;                  /* TRGMUX Source Select 1 */
    vuint32_t _unused_15:1;
    vuint32_t SEL2:7;                  /* TRGMUX Source Select 2 */
    vuint32_t _unused_23:1;
    vuint32_t SEL3:7;                  /* TRGMUX Source Select 3 */
    vuint32_t LK:1;                    /* TRGMUX Register Lock */
  } B;
} TRGMUX_3_SINC_3_0_tag;

struct TRGMUX_3_tag {
  TRGMUX_3_GTM_003_000_tag GTM_003_000; /* TRGMUX GTM_003_000 */
  TRGMUX_3_GTM_007_004_tag GTM_007_004; /* TRGMUX GTM_007_004 */
  TRGMUX_3_GTM_011_008_tag GTM_011_008; /* TRGMUX GTM_011_008 */
  TRGMUX_3_GTM_015_012_tag GTM_015_012; /* TRGMUX GTM_015_012 */
  TRGMUX_3_GTM_019_016_tag GTM_019_016; /* TRGMUX GTM_019_016 */
  TRGMUX_3_GTM_023_020_tag GTM_023_020; /* TRGMUX GTM_023_020 */
  TRGMUX_3_GTM_027_024_tag GTM_027_024; /* TRGMUX GTM_027_024 */
  TRGMUX_3_GTM_031_028_tag GTM_031_028; /* TRGMUX GTM_031_028 */
  TRGMUX_3_GTM_035_032_tag GTM_035_032; /* TRGMUX GTM_035_032 */
  TRGMUX_3_GTM_039_036_tag GTM_039_036; /* TRGMUX GTM_039_036 */
  TRGMUX_3_GTM_043_040_tag GTM_043_040; /* TRGMUX GTM_043_040 */
  TRGMUX_3_GTM_047_044_tag GTM_047_044; /* TRGMUX GTM_047_044 */
  TRGMUX_3_GTM_051_048_tag GTM_051_048; /* TRGMUX GTM_051_048 */
  TRGMUX_3_GTM_055_052_tag GTM_055_052; /* TRGMUX GTM_055_052 */
  TRGMUX_3_GTM_059_056_tag GTM_059_056; /* TRGMUX GTM_059_056 */
  TRGMUX_3_GTM_063_060_tag GTM_063_060; /* TRGMUX GTM_063_060 */
  TRGMUX_3_GTM_067_064_tag GTM_067_064; /* TRGMUX GTM_067_064 */
  TRGMUX_3_GTM_071_068_tag GTM_071_068; /* TRGMUX GTM_071_068 */
  TRGMUX_3_GTM_075_072_tag GTM_075_072; /* TRGMUX GTM_075_072 */
  TRGMUX_3_GTM_079_076_tag GTM_079_076; /* TRGMUX GTM_079_076 */
  TRGMUX_3_GTM_083_080_tag GTM_083_080; /* TRGMUX GTM_083_080 */
  TRGMUX_3_GTM_087_084_tag GTM_087_084; /* TRGMUX GTM_087_084 */
  TRGMUX_3_GTM_091_088_tag GTM_091_088; /* TRGMUX GTM_091_088 */
  TRGMUX_3_GTM_095_092_tag GTM_095_092; /* TRGMUX GTM_095_092 */
  TRGMUX_3_GTM_099_096_tag GTM_099_096; /* TRGMUX GTM_099_096 */
  TRGMUX_3_GTM_103_100_tag GTM_103_100; /* TRGMUX GTM_103_100 */
  TRGMUX_3_GTM_107_104_tag GTM_107_104; /* TRGMUX GTM_107_104 */
  TRGMUX_3_GTM_111_108_tag GTM_111_108; /* TRGMUX GTM_111_108 */
  TRGMUX_3_GTM_115_112_tag GTM_115_112; /* TRGMUX GTM_115_112 */
  TRGMUX_3_GTM_119_116_tag GTM_119_116; /* TRGMUX GTM_119_116 */
  TRGMUX_3_GTM_123_120_tag GTM_123_120; /* TRGMUX GTM_123_120 */
  TRGMUX_3_GTM_127_124_tag GTM_127_124; /* TRGMUX GTM_127_124 */
  TRGMUX_3_GTM_131_128_tag GTM_131_128; /* TRGMUX GTM_131_128 */
  TRGMUX_3_GTM_135_132_tag GTM_135_132; /* TRGMUX GTM_135_132 */
  TRGMUX_3_GTM_139_136_tag GTM_139_136; /* TRGMUX GTM_139_136 */
  TRGMUX_3_GTM_143_140_tag GTM_143_140; /* TRGMUX GTM_143_140 */
  TRGMUX_3_GTM_147_144_tag GTM_147_144; /* TRGMUX GTM_147_144 */
  TRGMUX_3_GTM_151_148_tag GTM_151_148; /* TRGMUX GTM_151_148 */
  TRGMUX_3_GTM_155_152_tag GTM_155_152; /* TRGMUX GTM_155_152 */
  TRGMUX_3_GTM_159_156_tag GTM_159_156; /* TRGMUX GTM_159_156 */
  TRGMUX_3_GTM_163_160_tag GTM_163_160; /* TRGMUX GTM_163_160 */
  TRGMUX_3_GTM_167_164_tag GTM_167_164; /* TRGMUX GTM_167_164 */
  TRGMUX_3_GTM_168_tag GTM_168;        /* TRGMUX GTM_168 */
  TRGMUX_3_MSC_03_00_tag MSC_03_00;    /* TRGMUX MSC_03_00 */
  TRGMUX_3_MSC_07_04_tag MSC_07_04;    /* TRGMUX MSC_07_04 */
  TRGMUX_3_MSC_11_08_tag MSC_11_08;    /* TRGMUX MSC_11_08 */
  TRGMUX_3_MSC_15_12_tag MSC_15_12;    /* TRGMUX MSC_15_12 */
  TRGMUX_3_MSC_19_16_tag MSC_19_16;    /* TRGMUX MSC_19_16 */
  TRGMUX_3_MSC_23_20_tag MSC_23_20;    /* TRGMUX MSC_23_20 */
  TRGMUX_3_MSC_27_24_tag MSC_27_24;    /* TRGMUX MSC_27_24 */
  TRGMUX_3_MSC_31_28_tag MSC_31_28;    /* TRGMUX MSC_31_28 */
  TRGMUX_3_MSC_35_32_tag MSC_35_32;    /* TRGMUX MSC_35_32 */
  TRGMUX_3_MSC_39_36_tag MSC_39_36;    /* TRGMUX MSC_39_36 */
  TRGMUX_3_MSC_43_40_tag MSC_43_40;    /* TRGMUX MSC_43_40 */
  TRGMUX_3_MSC_47_44_tag MSC_47_44;    /* TRGMUX MSC_47_44 */
  TRGMUX_3_MSC_51_48_tag MSC_51_48;    /* TRGMUX MSC_51_48 */
  TRGMUX_3_MSC_55_52_tag MSC_55_52;    /* TRGMUX MSC_55_52 */
  TRGMUX_3_MSC_59_56_tag MSC_59_56;    /* TRGMUX MSC_59_56 */
  TRGMUX_3_MSC_63_60_tag MSC_63_60;    /* TRGMUX MSC_63_60 */
  TRGMUX_3_CTU_3_0_tag CTU_3_0;        /* TRGMUX CTU_3_0 */
  TRGMUX_3_SINC_3_0_tag SINC_3_0;      /* TRGMUX SINC_3_0 */
};


/* ============================================================================
   =============================== Module: USDHC ==============================
   ============================================================================ */

typedef union USDHC_DS_ADDR_union_tag { /* DMA System Address */
  vuint32_t R;
  struct {
    vuint32_t DS_ADDR:32;              /* System address */
  } B;
} USDHC_DS_ADDR_tag;

typedef union USDHC_BLK_ATT_union_tag { /* Block Attributes */
  vuint32_t R;
  struct {
    vuint32_t BLKSIZE:13;              /* Transfer block size */
    vuint32_t _unused_13:3;            /* Reserved */
    vuint32_t BLKCNT:16;               /* Blocks count for current transfer */
  } B;
} USDHC_BLK_ATT_tag;

typedef union USDHC_CMD_ARG_union_tag { /* Command Argument */
  vuint32_t R;
  struct {
    vuint32_t CMDARG:32;               /* Command argument */
  } B;
} USDHC_CMD_ARG_tag;

typedef union USDHC_CMD_XFR_TYP_union_tag { /* Command Transfer Type */
  vuint32_t R;
  struct {
    vuint32_t DMAEN:1;                 /* DMAEN */
    vuint32_t BCEN:1;                  /* BCEN */
    vuint32_t AC12EN:1;                /* AC12EN */
    vuint32_t DDR_EN:1;                /* DDR_EN */
    vuint32_t DTDSEL:1;                /* DTDSEL */
    vuint32_t MSBSEL:1;                /* MSBSEL */
    vuint32_t NIBBLE_POS:1;            /* NIBBLE_POS */
    vuint32_t AC23EN:1;                /* AC23EN */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t RSPTYP:2;                /* Response type select */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t CCCEN:1;                 /* Command CRC check enable */
    vuint32_t CICEN:1;                 /* Command index check enable */
    vuint32_t DPSEL:1;                 /* Data present select */
    vuint32_t CMDTYP:2;                /* Command type */
    vuint32_t CMDINX:6;                /* Command index */
    vuint32_t _unused_30:2;            /* Reserved */
  } B;
} USDHC_CMD_XFR_TYP_tag;

typedef union USDHC_CMD_RSP0_union_tag { /* Command Response0 */
  vuint32_t R;
  struct {
    vuint32_t CMDRSP0:32;              /* Command response 0 */
  } B;
} USDHC_CMD_RSP0_tag;

typedef union USDHC_CMD_RSP1_union_tag { /* Command Response1 */
  vuint32_t R;
  struct {
    vuint32_t CMDRSP1:32;              /* Command response 1 */
  } B;
} USDHC_CMD_RSP1_tag;

typedef union USDHC_CMD_RSP2_union_tag { /* Command Response2 */
  vuint32_t R;
  struct {
    vuint32_t CMDRSP2:32;              /* Command response 2 */
  } B;
} USDHC_CMD_RSP2_tag;

typedef union USDHC_CMD_RSP3_union_tag { /* Command Response3 */
  vuint32_t R;
  struct {
    vuint32_t CMDRSP3:32;              /* Command response 3 */
  } B;
} USDHC_CMD_RSP3_tag;

typedef union USDHC_DATA_BUFF_ACC_PORT_union_tag { /* Data Buffer Access Port */
  vuint32_t R;
  struct {
    vuint32_t DATCONT:32;              /* Data content */
  } B;
} USDHC_DATA_BUFF_ACC_PORT_tag;

typedef union USDHC_PRES_STATE_union_tag { /* Present State */
  vuint32_t R;
  struct {
    vuint32_t CIHB:1;                  /* Command inhibit (CMD) */
    vuint32_t CDIHB:1;                 /* Command Inhibit Data (DATA) */
    vuint32_t DLA:1;                   /* Data line active */
    vuint32_t SDSTB:1;                 /* SD clock stable */
    vuint32_t _unused_4:4;
    vuint32_t WTA:1;                   /* Write transfer active */
    vuint32_t RTA:1;                   /* Read transfer active */
    vuint32_t BWEN:1;                  /* Buffer write enable */
    vuint32_t BREN:1;                  /* Buffer read enable */
    vuint32_t RTR:1;                   /* Re-Tuning Request (only for SD3.0 SDR104 mode, and eMMC HS200 mode) */
    vuint32_t _unused_13:2;            /* Reserved */
    vuint32_t TSCD:1;                  /* Tap select change done */
    vuint32_t CINST:1;                 /* Card inserted */
    vuint32_t _unused_17:1;            /* Reserved */
    vuint32_t _unused_18:1;            /* Reserved */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t _unused_20:3;            /* Reserved */
    vuint32_t CLSL:1;                  /* CMD line signal level */
    vuint32_t DLSL:8;                  /* DATA[7:0] line signal level */
  } B;
} USDHC_PRES_STATE_tag;

typedef union USDHC_PROT_CTRL_union_tag { /* Protocol Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t DTW:2;                   /* Data transfer width */
    vuint32_t D3CD:1;                  /* DATA3 as card detection pin */
    vuint32_t EMODE:2;                 /* Endian mode */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t DMASEL:2;                /* DMA select */
    vuint32_t _unused_10:6;            /* Reserved */
    vuint32_t SABGREQ:1;               /* Stop at block gap request */
    vuint32_t CREQ:1;                  /* Continue request */
    vuint32_t RWCTL:1;                 /* Read wait control */
    vuint32_t IABG:1;                  /* Interrupt at block gap */
    vuint32_t RD_DONE_NO_8CLK:1;       /* Read performed number 8 clock */
    vuint32_t _unused_21:3;            /* Reserved */
    vuint32_t WECINT:1;                /* Wakeup event enable on card interrupt */
    vuint32_t WECINS:1;                /* Wakeup event enable on SD card insertion */
    vuint32_t WECRM:1;                 /* Wakeup event enable on SD card removal */
    vuint32_t _unused_27:3;            /* Reserved */
    vuint32_t NON_EXACT_BLK_RD:1;      /* Non-exact block read */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} USDHC_PROT_CTRL_tag;

typedef union USDHC_SYS_CTRL_union_tag { /* System Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:4;             /* Reserved */
    vuint32_t DVS:4;                   /* Divisor */
    vuint32_t SDCLKFS:8;               /* SDCLK frequency select */
    vuint32_t DTOCV:4;                 /* Data timeout counter value */
    vuint32_t _unused_20:1;            /* Reserved */
    vuint32_t _unused_21:1;            /* Reserved */
    vuint32_t RST_FIFO:1;              /* Reset the async FIFO */
    vuint32_t IPP_RST_N:1;             /* Hardware reset */
    vuint32_t RSTA:1;                  /* Software reset for all */
    vuint32_t RSTC:1;                  /* Software reset for CMD line */
    vuint32_t RSTD:1;                  /* Software reset for data line */
    vuint32_t INITA:1;                 /* Initialization active */
    vuint32_t RSTT:1;                  /* Reset tuning */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} USDHC_SYS_CTRL_tag;

typedef union USDHC_INT_STATUS_union_tag { /* Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t CC:1;                    /* Command complete */
    vuint32_t TC:1;                    /* Transfer complete */
    vuint32_t BGE:1;                   /* Block gap event */
    vuint32_t DINT:1;                  /* DMA interrupt */
    vuint32_t BWR:1;                   /* Buffer write ready */
    vuint32_t BRR:1;                   /* Buffer read ready */
    vuint32_t CINS:1;                  /* Card insertion */
    vuint32_t CRM:1;                   /* Card removal */
    vuint32_t CINT:1;                  /* Card interrupt */
    vuint32_t _unused_9:2;             /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t RTE:1;                   /* Re-tuning event: (only for SD3.0 SDR104 mode and eMMC HS200 mode) */
    vuint32_t TP:1;                    /* Tuning pass:(only for SD3.0 SDR104 mode and eMMC HS200 mode) */
    vuint32_t CQI:1;                   /* Command queuing interrupt */
    vuint32_t ERR_INT_STATUS:1;        /* Error Interrupt Status */
    vuint32_t CTOE:1;                  /* Command timeout error */
    vuint32_t CCE:1;                   /* Command CRC error */
    vuint32_t CEBE:1;                  /* Command end bit error */
    vuint32_t CIE:1;                   /* Command index error */
    vuint32_t DTOE:1;                  /* Data timeout error */
    vuint32_t DCE:1;                   /* Data CRC error */
    vuint32_t DEBE:1;                  /* Data end bit error */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t AC12E:1;                 /* Auto CMD12 error */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t TNE:1;                   /* Tuning error: (only for SD3.0 SDR104 mode and eMMC HS200 mode) */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t DMAE:1;                  /* DMA error */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} USDHC_INT_STATUS_tag;

typedef union USDHC_INT_STATUS_EN_union_tag { /* Interrupt Status Enable */
  vuint32_t R;
  struct {
    vuint32_t CCSEN:1;                 /* Command complete status enable */
    vuint32_t TCSEN:1;                 /* Transfer complete status enable */
    vuint32_t BGESEN:1;                /* Block gap event status enable */
    vuint32_t DINTSEN:1;               /* DMA interrupt status enable */
    vuint32_t BWRSEN:1;                /* Buffer write ready status enable */
    vuint32_t BRRSEN:1;                /* Buffer read ready status enable */
    vuint32_t CINSSEN:1;               /* Card insertion status enable */
    vuint32_t CRMSEN:1;                /* Card removal status enable */
    vuint32_t CINTSEN:1;               /* Card interrupt status enable */
    vuint32_t _unused_9:2;             /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t RTESEN:1;                /* Re-tuning event status enable */
    vuint32_t TPSEN:1;                 /* Tuning pass status enable */
    vuint32_t CQISEN:1;                /* Command queuing status enable */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t CTOESEN:1;               /* Command timeout error status enable */
    vuint32_t CCESEN:1;                /* Command CRC error status enable */
    vuint32_t CEBESEN:1;               /* Command end bit error status enable */
    vuint32_t CIESEN:1;                /* Command index error status enable */
    vuint32_t DTOESEN:1;               /* Data timeout error status enable */
    vuint32_t DCESEN:1;                /* Data CRC error status enable */
    vuint32_t DEBESEN:1;               /* Data end bit error status enable */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t AC12ESEN:1;              /* Auto CMD12 error status enable */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t TNESEN:1;                /* Tuning error status enable */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t DMAESEN:1;               /* DMA error status enable */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} USDHC_INT_STATUS_EN_tag;

typedef union USDHC_INT_SIGNAL_EN_union_tag { /* Interrupt Signal Enable */
  vuint32_t R;
  struct {
    vuint32_t CCIEN:1;                 /* Command complete interrupt enable */
    vuint32_t TCIEN:1;                 /* Transfer complete interrupt enable */
    vuint32_t BGEIEN:1;                /* Block gap event interrupt enable */
    vuint32_t DINTIEN:1;               /* DMA interrupt enable */
    vuint32_t BWRIEN:1;                /* Buffer write ready interrupt enable */
    vuint32_t BRRIEN:1;                /* Buffer read ready interrupt enable */
    vuint32_t CINSIEN:1;               /* Card insertion interrupt enable */
    vuint32_t CRMIEN:1;                /* Card removal interrupt enable */
    vuint32_t CINTIEN:1;               /* Card interrupt enable */
    vuint32_t _unused_9:2;             /* Reserved */
    vuint32_t _unused_11:1;            /* Reserved */
    vuint32_t RTEIEN:1;                /* Re-tuning event interrupt enable */
    vuint32_t TPIEN:1;                 /* Tuning pass interrupt enable */
    vuint32_t CQIIEN:1;                /* Command queuing signal enable */
    vuint32_t _unused_15:1;            /* Reserved */
    vuint32_t CTOEIEN:1;               /* Command timeout error interrupt enable */
    vuint32_t CCEIEN:1;                /* Command CRC error interrupt enable */
    vuint32_t CEBEIEN:1;               /* Command end bit error interrupt enable */
    vuint32_t CIEIEN:1;                /* Command index error interrupt enable */
    vuint32_t DTOEIEN:1;               /* Data timeout error interrupt enable */
    vuint32_t DCEIEN:1;                /* Data CRC error interrupt enable */
    vuint32_t DEBEIEN:1;               /* Data end bit error interrupt enable */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t AC12EIEN:1;              /* Auto CMD12 error interrupt enable */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t TNEIEN:1;                /* Tuning error interrupt enable */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t DMAEIEN:1;               /* DMA error interrupt enable */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} USDHC_INT_SIGNAL_EN_tag;

typedef union USDHC_AUTOCMD12_ERR_STATUS_union_tag { /* Auto CMD12 Error Status */
  vuint32_t R;
  struct {
    vuint32_t AC12NE:1;                /* Auto CMD12 not executed */
    vuint32_t AC12TOE:1;               /* Auto CMD12 / 23 timeout error */
    vuint32_t AC12CE:1;                /* Auto CMD12 / 23 CRC error */
    vuint32_t AC12EBE:1;               /* Auto CMD12 / 23 end bit error */
    vuint32_t AC12IE:1;                /* Auto CMD12 / 23 index error */
    vuint32_t _unused_5:2;             /* Reserved */
    vuint32_t CNIBAC12E:1;             /* Command not issued by Auto CMD12 error */
    vuint32_t _unused_8:14;            /* Reserved */
    vuint32_t EXECUTE_TUNING:1;        /* Execute tuning */
    vuint32_t SMP_CLK_SEL:1;           /* Sample clock select */
    vuint32_t _unused_24:8;            /* Reserved */
  } B;
} USDHC_AUTOCMD12_ERR_STATUS_tag;

typedef union USDHC_HOST_CTRL_CAP_union_tag { /* Host Controller Capabilities */
  vuint32_t R;
  struct {
    vuint32_t SDR50_SUPPORT:1;         /* SDR50 support */
    vuint32_t SDR104_SUPPORT:1;        /* SDR104 support */
    vuint32_t DDR50_SUPPORT:1;         /* DDR50 support */
    vuint32_t _unused_3:5;             /* Reserved */
    vuint32_t _unused_8:4;
    vuint32_t _unused_12:1;            /* Reserved */
    vuint32_t USE_TUNING_SDR50:1;      /* Use Tuning for SDR50 */
    vuint32_t _unused_14:2;
    vuint32_t MBL:3;                   /* Max block length */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t ADMAS:1;                 /* ADMA support */
    vuint32_t HSS:1;                   /* High speed support */
    vuint32_t DMAS:1;                  /* DMA support */
    vuint32_t SRS:1;                   /* Suspend / resume support */
    vuint32_t VS33:1;                  /* Voltage support 3.3 V */
    vuint32_t VS30:1;                  /* Voltage support 3.0 V */
    vuint32_t VS18:1;                  /* Voltage support 1.8 V */
    vuint32_t _unused_27:5;            /* Reserved */
  } B;
} USDHC_HOST_CTRL_CAP_tag;

typedef union USDHC_WTMK_LVL_union_tag { /* Watermark Level */
  vuint32_t R;
  struct {
    vuint32_t RD_WML:8;                /* Read watermark level */
    vuint32_t _unused_8:5;             /* Reserved */
    vuint32_t _unused_13:3;            /* Reserved */
    vuint32_t WR_WML:8;                /* Write watermark level */
    vuint32_t _unused_24:5;            /* Reserved */
    vuint32_t _unused_29:3;            /* Reserved */
  } B;
} USDHC_WTMK_LVL_tag;

typedef union USDHC_MIX_CTRL_union_tag { /* Mixer Control */
  vuint32_t R;
  struct {
    vuint32_t DMAEN:1;                 /* DMA enable */
    vuint32_t BCEN:1;                  /* Block count enable */
    vuint32_t AC12EN:1;                /* Auto CMD12 enable */
    vuint32_t DDR_EN:1;                /* Dual data rate mode selection */
    vuint32_t DTDSEL:1;                /* Data transfer direction select */
    vuint32_t MSBSEL:1;                /* Multi / Single block select */
    vuint32_t NIBBLE_POS:1;            /* Nibble position indication */
    vuint32_t AC23EN:1;                /* Auto CMD23 enable */
    vuint32_t _unused_8:14;            /* Reserved */
    vuint32_t EXE_TUNE:1;              /* Execute tuning: (Only used for SD3.0, SDR104 mode and eMMC HS200 mode) */
    vuint32_t SMP_CLK_SEL:1;           /* Clock selection */
    vuint32_t AUTO_TUNE_EN:1;          /* Auto tuning enable (Only used for SD3.0, SDR104 mode and eMMC HS200 mode) */
    vuint32_t FBCLK_SEL:1;             /* Feedback clock source selection (Only used for SD3.0, SDR104 mode and eMMC HS200 mode) */
    vuint32_t HS400_MODE:1;            /* Enable HS400 mode */
    vuint32_t EN_HS400_MODE:1;         /* Enable enhance HS400 mode */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t _unused_31:1;            /* Reserved */
  } B;
} USDHC_MIX_CTRL_tag;

typedef union USDHC_FORCE_EVENT_union_tag { /* Force Event */
  vuint32_t R;
  struct {
    vuint32_t FEVTAC12NE:1;            /* Force event auto command 12 not executed */
    vuint32_t FEVTAC12TOE:1;           /* Force event auto command 12 time out error */
    vuint32_t FEVTAC12CE:1;            /* Force event auto command 12 CRC error */
    vuint32_t FEVTAC12EBE:1;           /* Force event Auto Command 12 end bit error */
    vuint32_t FEVTAC12IE:1;            /* Force event Auto Command 12 index error */
    vuint32_t _unused_5:2;             /* Reserved */
    vuint32_t FEVTCNIBAC12E:1;         /* Force event command not executed by Auto Command 12 error */
    vuint32_t _unused_8:8;             /* Reserved */
    vuint32_t FEVTCTOE:1;              /* Force event command time out error */
    vuint32_t FEVTCCE:1;               /* Force event command CRC error */
    vuint32_t FEVTCEBE:1;              /* Force event command end bit error */
    vuint32_t FEVTCIE:1;               /* Force event command index error */
    vuint32_t FEVTDTOE:1;              /* Force event data time out error */
    vuint32_t FEVTDCE:1;               /* Force event data CRC error */
    vuint32_t FEVTDEBE:1;              /* Force event data end bit error */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t FEVTAC12E:1;             /* Force event Auto Command 12 error */
    vuint32_t _unused_25:1;            /* Reserved */
    vuint32_t FEVTTNE:1;               /* Force tuning error */
    vuint32_t _unused_27:1;            /* Reserved */
    vuint32_t FEVTDMAE:1;              /* Force event DMA error */
    vuint32_t _unused_29:2;            /* Reserved */
    vuint32_t FEVTCINT:1;              /* Force event card interrupt */
  } B;
} USDHC_FORCE_EVENT_tag;

typedef union USDHC_ADMA_ERR_STATUS_union_tag { /* ADMA Error Status */
  vuint32_t R;
  struct {
    vuint32_t ADMAES:2;                /* ADMA error state (when ADMA error is occurred) */
    vuint32_t ADMALME:1;               /* ADMA length mismatch error */
    vuint32_t ADMADCE:1;               /* ADMA descriptor error */
    vuint32_t _unused_4:28;            /* Reserved */
  } B;
} USDHC_ADMA_ERR_STATUS_tag;

typedef union USDHC_ADMA_SYS_ADDR_union_tag { /* ADMA System Address */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:2;             /* Reserved */
    vuint32_t ADS_ADDR:30;             /* ADMA system address */
  } B;
} USDHC_ADMA_SYS_ADDR_tag;

typedef union USDHC_DLL_CTRL_union_tag { /* DLL (Delay Line) Control */
  vuint32_t R;
  struct {
    vuint32_t DLL_CTRL_ENABLE:1;       /* DLL and delay chain */
    vuint32_t DLL_CTRL_RESET:1;        /* DLL reset */
    vuint32_t DLL_CTRL_SLV_FORCE_UPD:1; /* DLL slave delay line */
    vuint32_t DLL_CTRL_SLV_DLY_TARGET0:4; /* DLL slave delay target0 */
    vuint32_t DLL_CTRL_GATE_UPDATE:1;  /* DLL gate update */
    vuint32_t DLL_CTRL_SLV_OVERRIDE:1; /* DLL slave override */
    vuint32_t DLL_CTRL_SLV_OVERRIDE_VAL:7; /* DLL slave override val */
    vuint32_t DLL_CTRL_SLV_DLY_TARGET1:3; /* DLL slave delay target1 */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t DLL_CTRL_SLV_UPDATE_INT:8; /* Slave delay line update interval */
    vuint32_t DLL_CTRL_REF_UPDATE_INT:4; /* DLL control loop update interval */
  } B;
} USDHC_DLL_CTRL_tag;

typedef union USDHC_DLL_STATUS_union_tag { /* DLL Status */
  vuint32_t R;
  struct {
    vuint32_t DLL_STS_SLV_LOCK:1;      /* Slave delay-line lock status */
    vuint32_t DLL_STS_REF_LOCK:1;      /* Reference DLL lock status */
    vuint32_t DLL_STS_SLV_SEL:7;       /* Slave delay line select status */
    vuint32_t DLL_STS_REF_SEL:7;       /* Reference delay line select taps */
    vuint32_t _unused_16:16;           /* Reserved */
  } B;
} USDHC_DLL_STATUS_tag;

typedef union USDHC_CLK_TUNE_CTRL_STATUS_union_tag { /* CLK Tuning Control and Status */
  vuint32_t R;
  struct {
    vuint32_t DLY_CELL_SET_POST:4;     /* Delay cells on the feedback clock between CLK_OUT and CLK_POST */
    vuint32_t DLY_CELL_SET_OUT:4;      /* Delay cells on the feedback clock between CLK_PRE and CLK_OUT */
    vuint32_t DLY_CELL_SET_PRE:7;      /* delay cells on the feedback clock between the feedback clock and CLK_PRE */
    vuint32_t NXT_ERR:1;               /* NXT error */
    vuint32_t TAP_SEL_POST:4;          /* Delay cells added on the feedback clock between CLK_OUT and CLK_POST */
    vuint32_t TAP_SEL_OUT:4;           /* Delay cells added on the feedback clock between CLK_PRE and CLK_OUT */
    vuint32_t TAP_SEL_PRE:7;           /* TAP_SEL_PRE */
    vuint32_t PRE_ERR:1;               /* PRE error */
  } B;
} USDHC_CLK_TUNE_CTRL_STATUS_tag;

typedef union USDHC_STROBE_DLL_CTRL_union_tag { /* Strobe DLL control */
  vuint32_t R;
  struct {
    vuint32_t STROBE_DLL_CTRL_ENABLE:1; /* Strobe DLL control enable */
    vuint32_t STROBE_DLL_CTRL_RESET:1; /* Strobe DLL control reset */
    vuint32_t STROBE_DLL_CTRL_SLV_FORCE_UPD:1; /* Strobe DLL control slave force updated */
    vuint32_t STROBE_DLL_CTRL_SLV_DLY_TARGET:4; /* Strobe DLL Control Slave Delay Target */
    vuint32_t STROBE_DLL_CTRL_GATE_UPDATE:1; /* Strobe DLL control gate update */
    vuint32_t STROBE_DLL_CTRL_SLV_OVERRIDE:1; /* Strobe DLL control slave override */
    vuint32_t STROBE_DLL_CTRL_SLV_OVERRIDE_VAL:7; /* Strobe DLL control slave Override value */
    vuint32_t _unused_16:4;            /* This field is reserved. */
    vuint32_t STROBE_DLL_CTRL_SLV_UPDATE_INT:8; /* Strobe DLL control slave update interval */
    vuint32_t STROBE_DLL_CTRL_REF_UPDATE_INT:4; /* Strobe DLL control reference update interval */
  } B;
} USDHC_STROBE_DLL_CTRL_tag;

typedef union USDHC_STROBE_DLL_STATUS_union_tag { /* Strobe DLL status */
  vuint32_t R;
  struct {
    vuint32_t STROBE_DLL_STS_SLV_LOCK:1; /* Strobe DLL status slave lock */
    vuint32_t STROBE_DLL_STS_REF_LOCK:1; /* Strobe DLL status reference lock */
    vuint32_t STROBE_DLL_STS_SLV_SEL:7; /* Strobe DLL status slave select */
    vuint32_t STROBE_DLL_STS_REF_SEL:7; /* Strobe DLL status reference select */
    vuint32_t _unused_16:16;           /* This field is reserved. */
  } B;
} USDHC_STROBE_DLL_STATUS_tag;

typedef union USDHC_VEND_SPEC_union_tag { /* Vendor Specific Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;             /* Reserved */
    vuint32_t VSELECT:1;               /* Voltage selection */
    vuint32_t _unused_2:1;             /* Reserved */
    vuint32_t AC12_WR_CHKBUSY_EN:1;    /* Check busy enable */
    vuint32_t _unused_4:1;             /* Reserved */
    vuint32_t _unused_5:1;             /* Reserved */
    vuint32_t _unused_6:1;             /* Reserved */
    vuint32_t _unused_7:1;             /* Reserved */
    vuint32_t FRC_SDCLK_ON:1;          /* Force CLK */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t _unused_10:1;            /* Reserved */
    vuint32_t _unused_11:4;            /* Reserved */
    vuint32_t CRC_CHK_DIS:1;           /* CRC Check Disable */
    vuint32_t _unused_16:8;            /* Reserved */
    vuint32_t _unused_24:4;            /* Reserved */
    vuint32_t _unused_28:1;            /* Reserved */
    vuint32_t _unused_29:1;            /* Reserved */
    vuint32_t _unused_30:1;            /* Reserved */
    vuint32_t CMD_BYTE_EN:1;           /* Register byte access for CMD_XFR_TYP */
  } B;
} USDHC_VEND_SPEC_tag;

typedef union USDHC_MMC_BOOT_union_tag { /* eMMC Boot */
  vuint32_t R;
  struct {
    vuint32_t DTOCV_ACK:4;             /* DTOCV_ACK */
    vuint32_t BOOT_ACK:1;              /* BOOT ACK */
    vuint32_t BOOT_MODE:1;             /* Boot mode */
    vuint32_t BOOT_EN:1;               /* Boot enable */
    vuint32_t AUTO_SABG_EN:1;          /* Auto stop at block gap */
    vuint32_t DISABLE_TIME_OUT:1;      /* Time out */
    vuint32_t _unused_9:7;             /* Reserved */
    vuint32_t BOOT_BLK_CNT:16;         /* Stop At Block Gap value of automatic mode */
  } B;
} USDHC_MMC_BOOT_tag;

typedef union USDHC_VEND_SPEC2_union_tag { /* Vendor Specific 2 Register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:3;             /* Reserved */
    vuint32_t CARD_INT_D3_TEST:1;      /* Card interrupt detection test */
    vuint32_t TUNING_BIT_EN:2;         /* Tuning bit enable */
    vuint32_t TUNING_CMD_EN:1;         /* Tuning command enable */
    vuint32_t _unused_7:2;             /* Reserved */
    vuint32_t _unused_9:1;             /* Reserved */
    vuint32_t HS400_WR_CLK_STOP_EN:1;  /* HS400 write clock stop enable */
    vuint32_t HS400_RD_CLK_STOP_EN:1;  /* HS400 read clock stop enable */
    vuint32_t ACMD23_ARGU2_EN:1;       /* Argument2 register enable for ACMD23 */
    vuint32_t _unused_13:1;            /* Reserved */
    vuint32_t _unused_14:1;            /* Reserved */
    vuint32_t EN_32K_CLK:1;            /* Select the clock source for host card detection. */
    vuint32_t FBCLK_TAP_SEL:16;        /* Enable extra delay on internal feedback clock */
  } B;
} USDHC_VEND_SPEC2_tag;

typedef union USDHC_TUNING_CTRL_union_tag { /* Tuning Control */
  vuint32_t R;
  struct {
    vuint32_t TUNING_START_TAP:7;      /* Tuning start */
    vuint32_t DIS_CMD_CHK_FOR_STD_TUNING:1; /* Disable command check for standard tuning */
    vuint32_t TUNING_COUNTER:8;        /* Tuning counter */
    vuint32_t TUNING_STEP:3;           /* TUNING_STEP */
    vuint32_t _unused_19:1;            /* Reserved */
    vuint32_t TUNING_WINDOW:3;         /* Data window */
    vuint32_t _unused_23:1;            /* Reserved */
    vuint32_t STD_TUNING_EN:1;         /* Standard tuning circuit and procedure enable */
    vuint32_t _unused_25:7;            /* Reserved */
  } B;
} USDHC_TUNING_CTRL_tag;

typedef union USDHC_CQVER_union_tag {  /* Command Queuing Version */
  vuint32_t R;
  struct {
    vuint32_t VERSION_SUFFIX:4;        /* eMMC version suffix */
    vuint32_t MINOR_VN:4;              /* eMMC minor version number */
    vuint32_t MAJOR_VN:4;              /* eMMC major version number */
    vuint32_t _unused_12:20;
  } B;
} USDHC_CQVER_tag;

typedef union USDHC_CQCAP_union_tag {  /* Command Queuing Capabilities */
  vuint32_t R;
  struct {
    vuint32_t ITCFVAL:10;              /* Internal timer clock frequency value */
    vuint32_t _unused_10:2;
    vuint32_t ITCFMUL:4;               /* Internal timer clock frequency multiplier */
    vuint32_t _unused_16:16;
  } B;
} USDHC_CQCAP_tag;

typedef union USDHC_CQCFG_union_tag {  /* Command Queuing Configuration */
  vuint32_t R;
  struct {
    vuint32_t CQUE:1;                  /* Command queuing enable */
    vuint32_t _unused_1:7;
    vuint32_t TDS:1;                   /* Task descriptor size */
    vuint32_t _unused_9:3;
    vuint32_t DCMDE:1;                 /* Direct command (DCMD) enable */
    vuint32_t _unused_13:19;
  } B;
} USDHC_CQCFG_tag;

typedef union USDHC_CQCTL_union_tag {  /* Command Queuing Control */
  vuint32_t R;
  struct {
    vuint32_t HALT:1;                  /* Halt */
    vuint32_t _unused_1:7;
    vuint32_t CLEAR:1;                 /* Clear all tasks */
    vuint32_t _unused_9:23;
  } B;
} USDHC_CQCTL_tag;

typedef union USDHC_CQIS_union_tag {   /* Command Queuing Interrupt Status */
  vuint32_t R;
  struct {
    vuint32_t HAC:1;                   /* Halt complete interrupt */
    vuint32_t TCC:1;                   /* Task complete interrupt */
    vuint32_t RED:1;                   /* Response error detected interrupt */
    vuint32_t TCL:1;                   /* Task cleared */
    vuint32_t _unused_4:28;
  } B;
} USDHC_CQIS_tag;

typedef union USDHC_CQISTE_union_tag { /* Command Queuing Interrupt Status Enable */
  vuint32_t R;
  struct {
    vuint32_t HAC_STE:1;               /* Halt complete status enable */
    vuint32_t TCC_STE:1;               /* Task complete status enable */
    vuint32_t RED_STE:1;               /* Response error detected status enable */
    vuint32_t TCL_STE:1;               /* Task cleared status enable */
    vuint32_t _unused_4:28;
  } B;
} USDHC_CQISTE_tag;

typedef union USDHC_CQISGE_union_tag { /* Command Queuing Interrupt Signal Enable */
  vuint32_t R;
  struct {
    vuint32_t HAC_SGE:1;               /* Halt complete signal enable */
    vuint32_t TCC_SGE:1;               /* Task complete signal enable */
    vuint32_t RED_SGE:1;               /* Response error detected signal enable */
    vuint32_t TCL_SGE:1;               /* Task cleared signal enable */
    vuint32_t _unused_4:28;
  } B;
} USDHC_CQISGE_tag;

typedef union USDHC_CQIC_union_tag {   /* Command Queuing Interrupt Coalescing */
  vuint32_t R;
  struct {
    vuint32_t ICTOVAL:7;               /* Interrupt coalescing timeout value */
    vuint32_t ICTOVALWEN:1;            /* Interrupt coalescing timeout value write enable */
    vuint32_t ICCTH:5;                 /* Interrupt coalescing counter threshold */
    vuint32_t _unused_13:2;
    vuint32_t ICCTHWEN:1;              /* Interrupt coalescing counter threshold write enable */
    vuint32_t ICCTR:1;                 /* Counter and timer reset */
    vuint32_t _unused_17:3;
    vuint32_t ICSB:1;                  /* Interrupt coalescing status */
    vuint32_t _unused_21:10;
    vuint32_t ICENDIS:1;               /* Interrupt coalescing enable/disable */
  } B;
} USDHC_CQIC_tag;

typedef union USDHC_CQTDLBA_union_tag { /* Command Queuing Task Descriptor List Base Address */
  vuint32_t R;
  struct {
    vuint32_t TDLBA:32;                /* Task descriptor list base address */
  } B;
} USDHC_CQTDLBA_tag;

typedef union USDHC_CQTDLBAU_union_tag { /* Command Queuing Task Descriptor List Base Address Upper 32 Bits */
  vuint32_t R;
  struct {
    vuint32_t TDLBAU:32;               /* Task descriptor list base address */
  } B;
} USDHC_CQTDLBAU_tag;

typedef union USDHC_CQTDBR_union_tag { /* Command Queuing Task Doorbell */
  vuint32_t R;
  struct {
    vuint32_t TDBR:32;                 /* Task doorbell */
  } B;
} USDHC_CQTDBR_tag;

typedef union USDHC_CQTCN_union_tag {  /* Command Queuing Task Completion Notification */
  vuint32_t R;
  struct {
    vuint32_t TCN:32;                  /* Task complete notification */
  } B;
} USDHC_CQTCN_tag;

typedef union USDHC_CQDQS_union_tag {  /* Command Queuing Device Queue Status */
  vuint32_t R;
  struct {
    vuint32_t DQS:32;                  /* Device queue status */
  } B;
} USDHC_CQDQS_tag;

typedef union USDHC_CQDPT_union_tag {  /* Command Queuing Device Pending Tasks */
  vuint32_t R;
  struct {
    vuint32_t DPT:32;                  /* Device pending tasks */
  } B;
} USDHC_CQDPT_tag;

typedef union USDHC_CQTCLR_union_tag { /* Command Queuing Task Clear */
  vuint32_t R;
  struct {
    vuint32_t TCLR:32;                 /* Task clear */
  } B;
} USDHC_CQTCLR_tag;

typedef union USDHC_CQSSC1_union_tag { /* Command Queuing Send Status Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t CIT:16;                  /* Send status command idle timer */
    vuint32_t CBC:4;                   /* Send status command block counter */
    vuint32_t _unused_20:12;
  } B;
} USDHC_CQSSC1_tag;

typedef union USDHC_CQSSC2_union_tag { /* Command Queuing Send Status Configuration 2 */
  vuint32_t R;
  struct {
    vuint32_t SSC2:16;                 /* Send queue status RCA */
    vuint32_t _unused_16:16;
  } B;
} USDHC_CQSSC2_tag;

typedef union USDHC_CQCRDCT_union_tag { /* Command Queuing Command Response for Direct-Command Task */
  vuint32_t R;
  struct {
    vuint32_t CRDCT:32;                /* Direct command last response */
  } B;
} USDHC_CQCRDCT_tag;

typedef union USDHC_CQRMEM_union_tag { /* Command Queuing Response Mode Error Mask */
  vuint32_t R;
  struct {
    vuint32_t RMEM:32;                 /* Response mode error mask */
  } B;
} USDHC_CQRMEM_tag;

typedef union USDHC_CQTERRI_union_tag { /* Command Queuing Task Error Information */
  vuint32_t R;
  struct {
    vuint32_t RMECI:6;                 /* Response mode error command index */
    vuint32_t _unused_6:2;
    vuint32_t RMETID:5;                /* Response mode error task ID */
    vuint32_t _unused_13:2;
    vuint32_t RMEFV:1;                 /* Response mode error fields valid */
    vuint32_t DTECI:6;                 /* Data transfer error command index */
    vuint32_t _unused_22:2;
    vuint32_t DTETID:5;                /* Data transfer error task ID */
    vuint32_t _unused_29:2;
    vuint32_t DTEFV:1;                 /* Data transfer error fields valid */
  } B;
} USDHC_CQTERRI_tag;

typedef union USDHC_CQCRI_union_tag {  /* Command Queuing Command Response Index */
  vuint32_t R;
  struct {
    vuint32_t LCMDRI:6;                /* Last command response index */
    vuint32_t _unused_6:26;
  } B;
} USDHC_CQCRI_tag;

typedef union USDHC_CQCRA_union_tag {  /* Command Queuing Command Response Argument */
  vuint32_t R;
  struct {
    vuint32_t LCMDRA:32;               /* Last command response argument */
  } B;
} USDHC_CQCRA_tag;

struct USDHC_tag {
  USDHC_DS_ADDR_tag DS_ADDR;           /* DMA System Address */
  USDHC_BLK_ATT_tag BLK_ATT;           /* Block Attributes */
  USDHC_CMD_ARG_tag CMD_ARG;           /* Command Argument */
  USDHC_CMD_XFR_TYP_tag CMD_XFR_TYP;   /* Command Transfer Type */
  USDHC_CMD_RSP0_tag CMD_RSP0;         /* Command Response0 */
  USDHC_CMD_RSP1_tag CMD_RSP1;         /* Command Response1 */
  USDHC_CMD_RSP2_tag CMD_RSP2;         /* Command Response2 */
  USDHC_CMD_RSP3_tag CMD_RSP3;         /* Command Response3 */
  USDHC_DATA_BUFF_ACC_PORT_tag DATA_BUFF_ACC_PORT; /* Data Buffer Access Port */
  USDHC_PRES_STATE_tag PRES_STATE;     /* Present State */
  USDHC_PROT_CTRL_tag PROT_CTRL;       /* Protocol Control */
  USDHC_SYS_CTRL_tag SYS_CTRL;         /* System Control */
  USDHC_INT_STATUS_tag INT_STATUS;     /* Interrupt Status */
  USDHC_INT_STATUS_EN_tag INT_STATUS_EN; /* Interrupt Status Enable */
  USDHC_INT_SIGNAL_EN_tag INT_SIGNAL_EN; /* Interrupt Signal Enable */
  USDHC_AUTOCMD12_ERR_STATUS_tag AUTOCMD12_ERR_STATUS; /* Auto CMD12 Error Status */
  USDHC_HOST_CTRL_CAP_tag HOST_CTRL_CAP; /* Host Controller Capabilities */
  USDHC_WTMK_LVL_tag WTMK_LVL;         /* Watermark Level */
  USDHC_MIX_CTRL_tag MIX_CTRL;         /* Mixer Control */
  uint8_t USDHC_reserved0[4];
  USDHC_FORCE_EVENT_tag FORCE_EVENT;   /* Force Event */
  USDHC_ADMA_ERR_STATUS_tag ADMA_ERR_STATUS; /* ADMA Error Status */
  USDHC_ADMA_SYS_ADDR_tag ADMA_SYS_ADDR; /* ADMA System Address */
  uint8_t USDHC_reserved1[4];
  USDHC_DLL_CTRL_tag DLL_CTRL;         /* DLL (Delay Line) Control */
  USDHC_DLL_STATUS_tag DLL_STATUS;     /* DLL Status */
  USDHC_CLK_TUNE_CTRL_STATUS_tag CLK_TUNE_CTRL_STATUS; /* CLK Tuning Control and Status */
  uint8_t USDHC_reserved2[4];
  USDHC_STROBE_DLL_CTRL_tag STROBE_DLL_CTRL; /* Strobe DLL control */
  USDHC_STROBE_DLL_STATUS_tag STROBE_DLL_STATUS; /* Strobe DLL status */
  uint8_t USDHC_reserved3[72];
  USDHC_VEND_SPEC_tag VEND_SPEC;       /* Vendor Specific Register */
  USDHC_MMC_BOOT_tag MMC_BOOT;         /* eMMC Boot */
  USDHC_VEND_SPEC2_tag VEND_SPEC2;     /* Vendor Specific 2 Register */
  USDHC_TUNING_CTRL_tag TUNING_CTRL;   /* Tuning Control */
  uint8_t USDHC_reserved4[48];
  USDHC_CQVER_tag CQVER;               /* Command Queuing Version */
  USDHC_CQCAP_tag CQCAP;               /* Command Queuing Capabilities */
  USDHC_CQCFG_tag CQCFG;               /* Command Queuing Configuration */
  USDHC_CQCTL_tag CQCTL;               /* Command Queuing Control */
  USDHC_CQIS_tag CQIS;                 /* Command Queuing Interrupt Status */
  USDHC_CQISTE_tag CQISTE;             /* Command Queuing Interrupt Status Enable */
  USDHC_CQISGE_tag CQISGE;             /* Command Queuing Interrupt Signal Enable */
  USDHC_CQIC_tag CQIC;                 /* Command Queuing Interrupt Coalescing */
  USDHC_CQTDLBA_tag CQTDLBA;           /* Command Queuing Task Descriptor List Base Address */
  USDHC_CQTDLBAU_tag CQTDLBAU;         /* Command Queuing Task Descriptor List Base Address Upper 32 Bits */
  USDHC_CQTDBR_tag CQTDBR;             /* Command Queuing Task Doorbell */
  USDHC_CQTCN_tag CQTCN;               /* Command Queuing Task Completion Notification */
  USDHC_CQDQS_tag CQDQS;               /* Command Queuing Device Queue Status */
  USDHC_CQDPT_tag CQDPT;               /* Command Queuing Device Pending Tasks */
  USDHC_CQTCLR_tag CQTCLR;             /* Command Queuing Task Clear */
  uint8_t USDHC_reserved5[4];
  USDHC_CQSSC1_tag CQSSC1;             /* Command Queuing Send Status Configuration 1 */
  USDHC_CQSSC2_tag CQSSC2;             /* Command Queuing Send Status Configuration 2 */
  USDHC_CQCRDCT_tag CQCRDCT;           /* Command Queuing Command Response for Direct-Command Task */
  uint8_t USDHC_reserved6[4];
  USDHC_CQRMEM_tag CQRMEM;             /* Command Queuing Response Mode Error Mask */
  USDHC_CQTERRI_tag CQTERRI;           /* Command Queuing Task Error Information */
  USDHC_CQCRI_tag CQCRI;               /* Command Queuing Command Response Index */
  USDHC_CQCRA_tag CQCRA;               /* Command Queuing Command Response Argument */
};


/* ============================================================================
   =============================== Module: VIRT_WRAP ==========================
   ============================================================================ */

typedef union VIRT_WRAP_REG_A_union_tag { /* Parameter_n Register */
  vuint32_t R;
  struct {
    vuint32_t PAD_0:2;                 /* PAD_0 */
    vuint32_t PAD_1:2;                 /* PAD_1 */
    vuint32_t PAD_2:2;                 /* PAD_2 */
    vuint32_t PAD_3:2;                 /* PAD_3 */
    vuint32_t PAD_4:2;                 /* PAD_4 */
    vuint32_t PAD_5:2;                 /* PAD_5 */
    vuint32_t PAD_6:2;                 /* PAD_6 */
    vuint32_t PAD_7:2;                 /* PAD_7 */
    vuint32_t PAD_8:2;                 /* PAD_8 */
    vuint32_t PAD_9:2;                 /* PAD_9 */
    vuint32_t PAD_10:2;                /* PAD_10 */
    vuint32_t PAD_11:2;                /* PAD_11 */
    vuint32_t PAD_12:2;                /* PAD_12 */
    vuint32_t PAD_13:2;                /* PAD_13 */
    vuint32_t PAD_14:2;                /* PAD_14 */
    vuint32_t PAD_15:2;                /* PAD_15 */
  } B;
} VIRT_WRAP_REG_A_tag;

typedef union VIRT_WRAP_REG_B_union_tag { /* Parameter_n Register */
  vuint32_t R;
  struct {
    vuint32_t INMUX_0:2;               /* INMUX_0 */
    vuint32_t INMUX_1:2;               /* INMUX_1 */
    vuint32_t INMUX_2:2;               /* INMUX_2 */
    vuint32_t INMUX_3:2;               /* INMUX_3 */
    vuint32_t INMUX_4:2;               /* INMUX_4 */
    vuint32_t INMUX_5:2;               /* INMUX_5 */
    vuint32_t INMUX_6:2;               /* INMUX_6 */
    vuint32_t INMUX_7:2;               /* INMUX_7 */
    vuint32_t INMUX_8:2;               /* INMUX_8 */
    vuint32_t INMUX_9:2;               /* INMUX_9 */
    vuint32_t INMUX_10:2;              /* INMUX_10 */
    vuint32_t INMUX_11:2;              /* INMUX_11 */
    vuint32_t INMUX_12:2;              /* INMUX_12 */
    vuint32_t INMUX_13:2;              /* INMUX_13 */
    vuint32_t INMUX_14:2;              /* INMUX_14 */
    vuint32_t INMUX_15:2;              /* INMUX_15 */
  } B;
} VIRT_WRAP_REG_B_tag;

typedef union VIRT_WRAP_REG_C_union_tag { /* Parameter_n Register */
  vuint32_t R;
  struct {
    vuint32_t INTC_CTRL:2;             /* Interrupt register control */
    vuint32_t _unused_2:30;            /* Reserved */
  } B;
} VIRT_WRAP_REG_C_tag;

struct VIRT_WRAP_tag {
  VIRT_WRAP_REG_A_tag REG_A[32];       /* Parameter_n Register */
  VIRT_WRAP_REG_B_tag REG_B[32];       /* Parameter_n Register */
  VIRT_WRAP_REG_C_tag REG_C[1];        /* Parameter_n Register */
};


/* ============================================================================
   =============================== Module: WATCHDOG ===========================
   ============================================================================ */

typedef union WATCHDOG_WDOGACS_union_tag { /* Watchdog Access Control */
  vuint32_t R;
  struct {
    vuint32_t WDOG_ACS:16;
    vuint32_t _unused_16:16;
  } B;
} WATCHDOG_WDOGACS_tag;

typedef union WATCHDOG_WDOGCFG_union_tag { /* System Watchdog Configuration */
  vuint32_t R;
  struct {
    vuint32_t WDOG_EN:1;
    vuint32_t _unused_1:31;
  } B;
} WATCHDOG_WDOGCFG_tag;

typedef union WATCHDOG_WDOGMIN_union_tag { /* System Watchdog Minimum Threshold Configuration */
  vuint32_t R;
  struct {
    vuint32_t WDOGMIN:32;
  } B;
} WATCHDOG_WDOGMIN_tag;

typedef union WATCHDOG_WDOGMAX_union_tag { /* System Watchdog Maximum Threshold Configuration */
  vuint32_t R;
  struct {
    vuint32_t WDOGMAX:32;
  } B;
} WATCHDOG_WDOGMAX_tag;

typedef union WATCHDOG_WDOGCNT_union_tag { /* System Watchdog Count Value */
  vuint32_t R;
  struct {
    vuint32_t WDOGCNT:32;
  } B;
} WATCHDOG_WDOGCNT_tag;

typedef union WATCHDOG_MPWDOGCFG_union_tag { /* Master Port Watchdog Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:1;
    vuint32_t EPPWDOG_EN:1;
    vuint32_t EDPWDOG_EN:1;
    vuint32_t _unused_3:29;
  } B;
} WATCHDOG_MPWDOGCFG_tag;

typedef union WATCHDOG_MPWDOGT_union_tag { /* DMSS Master Port Watchdog Threshold */
  vuint32_t R;
  struct {
    vuint32_t MPWDOGT:32;
  } B;
} WATCHDOG_MPWDOGT_tag;

typedef union WATCHDOG_EPPWDOGC_union_tag { /* PMSS Master Port Watchdog Threshold */
  vuint32_t R;
  struct {
    vuint32_t EPPWDOGC:32;
  } B;
} WATCHDOG_EPPWDOGC_tag;

typedef union WATCHDOG_EDPWDOGC_union_tag { /* EDP Master Port Count Value */
  vuint32_t R;
  struct {
    vuint32_t EDPWDOGC:32;
  } B;
} WATCHDOG_EDPWDOGC_tag;

typedef union WATCHDOG_ICUWDOGCFG_union_tag { /* ICU Watchdog Configuration */
  vuint32_t R;
  struct {
    vuint32_t ICUWDOG_EN:1;
    vuint32_t _unused_1:31;
  } B;
} WATCHDOG_ICUWDOGCFG_tag;

typedef union WATCHDOG_ICUWDOGT_union_tag { /* ICU Watchdog Threshold Configuration */
  vuint32_t R;
  struct {
    vuint32_t ICUWDOGT:32;
  } B;
} WATCHDOG_ICUWDOGT_tag;

typedef union WATCHDOG_SYSWDOGSIG_union_tag { /* System Watchdog Signal */
  vuint32_t R;
  struct {
    vuint32_t TASK0_STATE:2;
    vuint32_t TASK1_STATE:2;
    vuint32_t TASK2_STATE:2;
    vuint32_t TASK3_STATE:2;
    vuint32_t TASK4_STATE:2;
    vuint32_t TASK5_STATE:2;
    vuint32_t TASK6_STATE:2;
    vuint32_t TASK7_STATE:2;
    vuint32_t _unused_16:16;
  } B;
} WATCHDOG_SYSWDOGSIG_tag;

typedef union WATCHDOG_ICUWDOGC_union_tag { /* ICU Watchdog Count Value */
  vuint32_t R;
  struct {
    vuint32_t ICUWDOGC:32;
  } B;
} WATCHDOG_ICUWDOGC_tag;

struct WATCHDOG_tag {
  WATCHDOG_WDOGACS_tag WDOGACS;        /* Watchdog Access Control */
  WATCHDOG_WDOGCFG_tag WDOGCFG;        /* System Watchdog Configuration */
  WATCHDOG_WDOGMIN_tag WDOGMIN;        /* System Watchdog Minimum Threshold Configuration */
  WATCHDOG_WDOGMAX_tag WDOGMAX;        /* System Watchdog Maximum Threshold Configuration */
  uint8_t WATCHDOG_reserved0[8];
  WATCHDOG_WDOGCNT_tag WDOGCNT;        /* System Watchdog Count Value */
  WATCHDOG_MPWDOGCFG_tag MPWDOGCFG;    /* Master Port Watchdog Control */
  WATCHDOG_MPWDOGT_tag MPWDOGT;        /* DMSS Master Port Watchdog Threshold */
  WATCHDOG_EPPWDOGC_tag EPPWDOGC;      /* PMSS Master Port Watchdog Threshold */
  WATCHDOG_EDPWDOGC_tag EDPWDOGC;      /* EDP Master Port Count Value */
  uint8_t WATCHDOG_reserved1[20];
  WATCHDOG_ICUWDOGCFG_tag ICUWDOGCFG;  /* ICU Watchdog Configuration */
  WATCHDOG_ICUWDOGT_tag ICUWDOGT;      /* ICU Watchdog Threshold Configuration */
  WATCHDOG_SYSWDOGSIG_tag SYSWDOGSIG;  /* System Watchdog Signal */
  WATCHDOG_ICUWDOGC_tag ICUWDOGC;      /* ICU Watchdog Count Value */
};


/* ============================================================================
   =============================== Module: XBIC ===============================
   ============================================================================ */

typedef union XBIC_MCR_union_tag {     /* XBIC Module Control */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:18;            /* Reserved */
    vuint32_t ME5:1;
    vuint32_t ME4:1;
    vuint32_t ME3:1;
    vuint32_t ME2:1;
    vuint32_t ME1:1;
    vuint32_t ME0:1;
    vuint32_t SE7:1;
    vuint32_t SE6:1;
    vuint32_t SE5:1;
    vuint32_t SE4:1;
    vuint32_t SE3:1;
    vuint32_t SE2:1;
    vuint32_t SE1:1;
    vuint32_t SE0:1;
  } B;
} XBIC_MCR_tag;

typedef union XBIC_EIR_union_tag {     /* XBIC Error Injection Attributes */
  vuint32_t R;
  struct {
    vuint32_t SYN:8;                   /* Syndrome */
    vuint32_t MST:4;                   /* Target Master ID */
    vuint32_t SLV:3;                   /* Target Slave Port */
    vuint32_t _unused_15:16;           /* Reserved */
    vuint32_t EIE:1;                   /* Error Injection Enable */
  } B;
} XBIC_EIR_tag;

typedef union XBIC_ESR_union_tag {     /* XBIC Error Status Attributes */
  vuint32_t R;
  struct {
    vuint32_t SYN:8;                   /* Syndrome */
    vuint32_t MST:4;                   /* Master ID */
    vuint32_t SLV:3;                   /* Slave Port */
    vuint32_t DPME7:1;
    vuint32_t DPME6:1;
    vuint32_t DPME5:1;
    vuint32_t DPME4:1;
    vuint32_t DPME3:1;
    vuint32_t DPME2:1;
    vuint32_t DPME1:1;
    vuint32_t DPME0:1;
    vuint32_t DPSE7:1;
    vuint32_t DPSE6:1;
    vuint32_t DPSE5:1;
    vuint32_t DPSE4:1;
    vuint32_t DPSE3:1;
    vuint32_t DPSE2:1;
    vuint32_t DPSE1:1;
    vuint32_t DPSE0:1;
    vuint32_t VLD:1;
  } B;
} XBIC_ESR_tag;

typedef union XBIC_EAR_union_tag {     /* XBIC Error Address */
  vuint32_t R;
  struct {
    vuint32_t ADDR:32;                 /* Error Address */
  } B;
} XBIC_EAR_tag;

struct XBIC_tag {
  XBIC_MCR_tag MCR;                    /* XBIC Module Control */
  XBIC_EIR_tag EIR;                    /* XBIC Error Injection Attributes */
  XBIC_ESR_tag ESR;                    /* XBIC Error Status Attributes */
  XBIC_EAR_tag EAR;                    /* XBIC Error Address */
};


/* ============================================================================
   =============================== Module: XRDC_0 =============================
   ============================================================================ */

typedef union XRDC_0_CR_union_tag {    /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} XRDC_0_CR_tag;

typedef union XRDC_0_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} XRDC_0_HWCFG0_tag;

typedef union XRDC_0_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} XRDC_0_HWCFG1_tag;

typedef union XRDC_0_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} XRDC_0_MDACFG_tag;

typedef union XRDC_0_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} XRDC_0_MRCFG_tag;

typedef union XRDC_0_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} XRDC_0_DERRLOC_tag;

typedef union XRDC_0_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} XRDC_0_DERRW0_DERR_W0_tag;

typedef union XRDC_0_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} XRDC_0_DERRW0_DERR_W1_tag;

typedef union XRDC_0_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} XRDC_0_DERRW0_DERR_W3_tag;

typedef struct XRDC_0_DERRW0_struct_tag {
  XRDC_0_DERRW0_DERR_W0_tag DERR_W0;   /* Domain Error Word 0 */
  XRDC_0_DERRW0_DERR_W1_tag DERR_W1;   /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  XRDC_0_DERRW0_DERR_W3_tag DERR_W3;   /* Domain Error Word 3 */
} XRDC_0_DERRW0_tag;

typedef union XRDC_0_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MDA_W0_0_DFMT1_tag;

typedef union XRDC_0_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MDA_W0_1_DFMT1_tag;

typedef union XRDC_0_MDA_W0_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MDA_W0_2_DFMT1_tag;

typedef union XRDC_0_MDA_W1_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MDA_W1_2_DFMT1_tag;

typedef union XRDC_0_MDA_W2_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MDA_W2_2_DFMT1_tag;

typedef union XRDC_0_MDA_W3_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MDA_W3_2_DFMT1_tag;

typedef union XRDC_0_MDA_W0_3_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MDA_W0_3_DFMT1_tag;

typedef union XRDC_0_MDA_W0_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MDA_W0_4_DFMT1_tag;

typedef union XRDC_0_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_0_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union XRDC_0_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct XRDC_0_PDAC_SLOT_PDACN_struct_tag {
  XRDC_0_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  XRDC_0_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} XRDC_0_PDAC_SLOT_PDACN_tag;

typedef struct XRDC_0_PDAC_SLOT_struct_tag {
  XRDC_0_PDAC_SLOT_PDACN_tag PDACN[64]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[512];
} XRDC_0_PDAC_SLOT_tag;

typedef union XRDC_0_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} XRDC_0_MRGDN_XRDC_MRGD_W0_tag;

typedef union XRDC_0_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} XRDC_0_MRGDN_XRDC_MRGD_W1_tag;

typedef union XRDC_0_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_0_MRGDN_XRDC_MRGD_W2_tag;

typedef union XRDC_0_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_0_MRGDN_XRDC_MRGD_W3_tag;

typedef struct XRDC_0_MRGDN_struct_tag {
  XRDC_0_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  XRDC_0_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  XRDC_0_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  XRDC_0_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} XRDC_0_MRGDN_tag;

struct XRDC_0_tag {
  XRDC_0_CR_tag CR;                    /* Control */
  uint8_t XRDC_0_reserved0[236];
  XRDC_0_HWCFG0_tag HWCFG0;            /* Hardware Configuration 0 */
  XRDC_0_HWCFG1_tag HWCFG1;            /* Hardware Configuration 1 */
  uint8_t XRDC_0_reserved1[8];
  XRDC_0_MDACFG_tag MDACFG[5];         /* Master Domain Assignment Configuration */
  uint8_t XRDC_0_reserved2[59];
  XRDC_0_MRCFG_tag MRCFG[1];           /* Memory Region Configuration */
  uint8_t XRDC_0_reserved3[191];
  XRDC_0_DERRLOC_tag DERRLOC[16];      /* Domain Error Location */
  uint8_t XRDC_0_reserved4[448];
  XRDC_0_DERRW0_tag DERRW0[20];        /* Valid array indices: 0, 16-19 */
  uint8_t XRDC_0_reserved5[704];
  XRDC_0_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_0_reserved6[28];
  XRDC_0_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_0_reserved7[28];
  XRDC_0_MDA_W0_2_DFMT1_tag MDA_W0_2_DFMT1; /* Master Domain Assignment */
  XRDC_0_MDA_W1_2_DFMT1_tag MDA_W1_2_DFMT1; /* Master Domain Assignment */
  XRDC_0_MDA_W2_2_DFMT1_tag MDA_W2_2_DFMT1; /* Master Domain Assignment */
  XRDC_0_MDA_W3_2_DFMT1_tag MDA_W3_2_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_0_reserved8[16];
  XRDC_0_MDA_W0_3_DFMT1_tag MDA_W0_3_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_0_reserved9[28];
  XRDC_0_MDA_W0_4_DFMT1_tag MDA_W0_4_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_0_reserved10[1916];
  XRDC_0_PDAC_SLOT_tag PDAC_SLOT[4];
  XRDC_0_MRGDN_tag MRGDN[1][16];
};


/* ============================================================================
   =============================== Module: XRDC_1 =============================
   ============================================================================ */

typedef union XRDC_1_CR_union_tag {    /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} XRDC_1_CR_tag;

typedef union XRDC_1_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} XRDC_1_HWCFG0_tag;

typedef union XRDC_1_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} XRDC_1_HWCFG1_tag;

typedef union XRDC_1_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} XRDC_1_MDACFG_tag;

typedef union XRDC_1_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} XRDC_1_MRCFG_tag;

typedef union XRDC_1_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} XRDC_1_DERRLOC_tag;

typedef union XRDC_1_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} XRDC_1_DERRW0_DERR_W0_tag;

typedef union XRDC_1_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} XRDC_1_DERRW0_DERR_W1_tag;

typedef union XRDC_1_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} XRDC_1_DERRW0_DERR_W3_tag;

typedef struct XRDC_1_DERRW0_struct_tag {
  XRDC_1_DERRW0_DERR_W0_tag DERR_W0;   /* Domain Error Word 0 */
  XRDC_1_DERRW0_DERR_W1_tag DERR_W1;   /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  XRDC_1_DERRW0_DERR_W3_tag DERR_W3;   /* Domain Error Word 3 */
} XRDC_1_DERRW0_tag;

typedef union XRDC_1_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W0_0_DFMT1_tag;

typedef union XRDC_1_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W0_1_DFMT1_tag;

typedef union XRDC_1_MDA_W1_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W1_1_DFMT1_tag;

typedef union XRDC_1_MDA_W2_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W2_1_DFMT1_tag;

typedef union XRDC_1_MDA_W3_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W3_1_DFMT1_tag;

typedef union XRDC_1_MDA_W4_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W4_1_DFMT1_tag;

typedef union XRDC_1_MDA_W5_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W5_1_DFMT1_tag;

typedef union XRDC_1_MDA_W6_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W6_1_DFMT1_tag;

typedef union XRDC_1_MDA_W7_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W7_1_DFMT1_tag;

typedef union XRDC_1_MDA_W0_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W0_2_DFMT1_tag;

typedef union XRDC_1_MDA_W0_3_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W0_3_DFMT1_tag;

typedef union XRDC_1_MDA_W0_4_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MDA_W0_4_DFMT1_tag;

typedef union XRDC_1_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_1_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union XRDC_1_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct XRDC_1_PDAC_SLOT_PDACN_struct_tag {
  XRDC_1_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  XRDC_1_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} XRDC_1_PDAC_SLOT_PDACN_tag;

typedef struct XRDC_1_PDAC_SLOT_struct_tag {
  XRDC_1_PDAC_SLOT_PDACN_tag PDACN[48]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[640];
} XRDC_1_PDAC_SLOT_tag;

typedef union XRDC_1_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} XRDC_1_MRGDN_XRDC_MRGD_W0_tag;

typedef union XRDC_1_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} XRDC_1_MRGDN_XRDC_MRGD_W1_tag;

typedef union XRDC_1_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_1_MRGDN_XRDC_MRGD_W2_tag;

typedef union XRDC_1_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_1_MRGDN_XRDC_MRGD_W3_tag;

typedef struct XRDC_1_MRGDN_struct_tag {
  XRDC_1_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  XRDC_1_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  XRDC_1_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  XRDC_1_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} XRDC_1_MRGDN_tag;

struct XRDC_1_tag {
  XRDC_1_CR_tag CR;                    /* Control */
  uint8_t XRDC_1_reserved0[236];
  XRDC_1_HWCFG0_tag HWCFG0;            /* Hardware Configuration 0 */
  XRDC_1_HWCFG1_tag HWCFG1;            /* Hardware Configuration 1 */
  uint8_t XRDC_1_reserved1[8];
  XRDC_1_MDACFG_tag MDACFG[5];         /* Master Domain Assignment Configuration */
  uint8_t XRDC_1_reserved2[59];
  XRDC_1_MRCFG_tag MRCFG[1];           /* Memory Region Configuration */
  uint8_t XRDC_1_reserved3[191];
  XRDC_1_DERRLOC_tag DERRLOC[16];      /* Domain Error Location */
  uint8_t XRDC_1_reserved4[448];
  XRDC_1_DERRW0_tag DERRW0[20];        /* Valid array indices: 0, 16-19 */
  uint8_t XRDC_1_reserved5[704];
  XRDC_1_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_1_reserved6[28];
  XRDC_1_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  XRDC_1_MDA_W1_1_DFMT1_tag MDA_W1_1_DFMT1; /* Master Domain Assignment */
  XRDC_1_MDA_W2_1_DFMT1_tag MDA_W2_1_DFMT1; /* Master Domain Assignment */
  XRDC_1_MDA_W3_1_DFMT1_tag MDA_W3_1_DFMT1; /* Master Domain Assignment */
  XRDC_1_MDA_W4_1_DFMT1_tag MDA_W4_1_DFMT1; /* Master Domain Assignment */
  XRDC_1_MDA_W5_1_DFMT1_tag MDA_W5_1_DFMT1; /* Master Domain Assignment */
  XRDC_1_MDA_W6_1_DFMT1_tag MDA_W6_1_DFMT1; /* Master Domain Assignment */
  XRDC_1_MDA_W7_1_DFMT1_tag MDA_W7_1_DFMT1; /* Master Domain Assignment */
  XRDC_1_MDA_W0_2_DFMT1_tag MDA_W0_2_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_1_reserved7[28];
  XRDC_1_MDA_W0_3_DFMT1_tag MDA_W0_3_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_1_reserved8[28];
  XRDC_1_MDA_W0_4_DFMT1_tag MDA_W0_4_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_1_reserved9[1916];
  XRDC_1_PDAC_SLOT_tag PDAC_SLOT[4];
  XRDC_1_MRGDN_tag MRGDN[1][16];
};


/* ============================================================================
   =============================== Module: XRDC_2 =============================
   ============================================================================ */

typedef union XRDC_2_CR_union_tag {    /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} XRDC_2_CR_tag;

typedef union XRDC_2_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} XRDC_2_HWCFG0_tag;

typedef union XRDC_2_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} XRDC_2_HWCFG1_tag;

typedef union XRDC_2_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} XRDC_2_MDACFG_tag;

typedef union XRDC_2_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} XRDC_2_MRCFG_tag;

typedef union XRDC_2_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} XRDC_2_DERRLOC_tag;

typedef union XRDC_2_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} XRDC_2_DERRW0_DERR_W0_tag;

typedef union XRDC_2_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} XRDC_2_DERRW0_DERR_W1_tag;

typedef union XRDC_2_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} XRDC_2_DERRW0_DERR_W3_tag;

typedef struct XRDC_2_DERRW0_struct_tag {
  XRDC_2_DERRW0_DERR_W0_tag DERR_W0;   /* Domain Error Word 0 */
  XRDC_2_DERRW0_DERR_W1_tag DERR_W1;   /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  XRDC_2_DERRW0_DERR_W3_tag DERR_W3;   /* Domain Error Word 3 */
} XRDC_2_DERRW0_tag;

typedef union XRDC_2_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_2_MDA_W0_0_DFMT1_tag;

typedef union XRDC_2_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_2_MDA_W0_1_DFMT1_tag;

typedef union XRDC_2_MDA_W0_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_2_MDA_W0_2_DFMT1_tag;

typedef union XRDC_2_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_2_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union XRDC_2_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_2_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct XRDC_2_PDAC_SLOT_PDACN_struct_tag {
  XRDC_2_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  XRDC_2_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} XRDC_2_PDAC_SLOT_PDACN_tag;

typedef struct XRDC_2_PDAC_SLOT_struct_tag {
  XRDC_2_PDAC_SLOT_PDACN_tag PDACN[16]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[896];
} XRDC_2_PDAC_SLOT_tag;

typedef union XRDC_2_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} XRDC_2_MRGDN_XRDC_MRGD_W0_tag;

typedef union XRDC_2_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} XRDC_2_MRGDN_XRDC_MRGD_W1_tag;

typedef union XRDC_2_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_2_MRGDN_XRDC_MRGD_W2_tag;

typedef union XRDC_2_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_2_MRGDN_XRDC_MRGD_W3_tag;

typedef struct XRDC_2_MRGDN_struct_tag {
  XRDC_2_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  XRDC_2_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  XRDC_2_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  XRDC_2_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} XRDC_2_MRGDN_tag;

struct XRDC_2_tag {
  XRDC_2_CR_tag CR;                    /* Control */
  uint8_t XRDC_2_reserved0[236];
  XRDC_2_HWCFG0_tag HWCFG0;            /* Hardware Configuration 0 */
  XRDC_2_HWCFG1_tag HWCFG1;            /* Hardware Configuration 1 */
  uint8_t XRDC_2_reserved1[8];
  XRDC_2_MDACFG_tag MDACFG[3];         /* Master Domain Assignment Configuration */
  uint8_t XRDC_2_reserved2[61];
  XRDC_2_MRCFG_tag MRCFG[3];           /* Memory Region Configuration */
  uint8_t XRDC_2_reserved3[189];
  XRDC_2_DERRLOC_tag DERRLOC[16];      /* Domain Error Location */
  uint8_t XRDC_2_reserved4[448];
  XRDC_2_DERRW0_tag DERRW0[18];        /* Valid array indices: 0-2, 16-17 */
  uint8_t XRDC_2_reserved5[736];
  XRDC_2_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_2_reserved6[28];
  XRDC_2_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_2_reserved7[28];
  XRDC_2_MDA_W0_2_DFMT1_tag MDA_W0_2_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_2_reserved8[1980];
  XRDC_2_PDAC_SLOT_tag PDAC_SLOT[2];
  uint8_t XRDC_2_reserved9[2048];
  XRDC_2_MRGDN_tag MRGDN[3][16];
};


/* ============================================================================
   =============================== Module: XRDC_3A ============================
   ============================================================================ */

typedef union XRDC_3A_CR_union_tag {   /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} XRDC_3A_CR_tag;

typedef union XRDC_3A_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} XRDC_3A_HWCFG0_tag;

typedef union XRDC_3A_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} XRDC_3A_HWCFG1_tag;

typedef union XRDC_3A_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} XRDC_3A_MDACFG_tag;

typedef union XRDC_3A_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} XRDC_3A_MRCFG_tag;

typedef union XRDC_3A_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} XRDC_3A_DERRLOC_tag;

typedef union XRDC_3A_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} XRDC_3A_DERRW0_DERR_W0_tag;

typedef union XRDC_3A_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} XRDC_3A_DERRW0_DERR_W1_tag;

typedef union XRDC_3A_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} XRDC_3A_DERRW0_DERR_W3_tag;

typedef struct XRDC_3A_DERRW0_struct_tag {
  XRDC_3A_DERRW0_DERR_W0_tag DERR_W0;  /* Domain Error Word 0 */
  XRDC_3A_DERRW0_DERR_W1_tag DERR_W1;  /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  XRDC_3A_DERRW0_DERR_W3_tag DERR_W3;  /* Domain Error Word 3 */
} XRDC_3A_DERRW0_tag;

typedef union XRDC_3A_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W0_0_DFMT1_tag;

typedef union XRDC_3A_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W0_1_DFMT1_tag;

typedef union XRDC_3A_MDA_W1_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W1_1_DFMT1_tag;

typedef union XRDC_3A_MDA_W2_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W2_1_DFMT1_tag;

typedef union XRDC_3A_MDA_W3_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W3_1_DFMT1_tag;

typedef union XRDC_3A_MDA_W4_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W4_1_DFMT1_tag;

typedef union XRDC_3A_MDA_W5_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W5_1_DFMT1_tag;

typedef union XRDC_3A_MDA_W6_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W6_1_DFMT1_tag;

typedef union XRDC_3A_MDA_W7_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W7_1_DFMT1_tag;

typedef union XRDC_3A_MDA_W0_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W0_2_DFMT1_tag;

typedef union XRDC_3A_MDA_W1_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W1_2_DFMT1_tag;

typedef union XRDC_3A_MDA_W2_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W2_2_DFMT1_tag;

typedef union XRDC_3A_MDA_W3_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W3_2_DFMT1_tag;

typedef union XRDC_3A_MDA_W4_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W4_2_DFMT1_tag;

typedef union XRDC_3A_MDA_W5_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W5_2_DFMT1_tag;

typedef union XRDC_3A_MDA_W6_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W6_2_DFMT1_tag;

typedef union XRDC_3A_MDA_W7_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MDA_W7_2_DFMT1_tag;

typedef union XRDC_3A_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_3A_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union XRDC_3A_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct XRDC_3A_PDAC_SLOT_PDACN_struct_tag {
  XRDC_3A_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  XRDC_3A_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} XRDC_3A_PDAC_SLOT_PDACN_tag;

typedef struct XRDC_3A_PDAC_SLOT_struct_tag {
  XRDC_3A_PDAC_SLOT_PDACN_tag PDACN[89]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[312];
} XRDC_3A_PDAC_SLOT_tag;

typedef union XRDC_3A_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} XRDC_3A_MRGDN_XRDC_MRGD_W0_tag;

typedef union XRDC_3A_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} XRDC_3A_MRGDN_XRDC_MRGD_W1_tag;

typedef union XRDC_3A_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_3A_MRGDN_XRDC_MRGD_W2_tag;

typedef union XRDC_3A_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3A_MRGDN_XRDC_MRGD_W3_tag;

typedef struct XRDC_3A_MRGDN_struct_tag {
  XRDC_3A_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  XRDC_3A_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  XRDC_3A_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  XRDC_3A_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} XRDC_3A_MRGDN_tag;

struct XRDC_3A_tag {
  XRDC_3A_CR_tag CR;                   /* Control */
  uint8_t XRDC_3A_reserved0[236];
  XRDC_3A_HWCFG0_tag HWCFG0;           /* Hardware Configuration 0 */
  XRDC_3A_HWCFG1_tag HWCFG1;           /* Hardware Configuration 1 */
  uint8_t XRDC_3A_reserved1[8];
  XRDC_3A_MDACFG_tag MDACFG[3];        /* Master Domain Assignment Configuration */
  uint8_t XRDC_3A_reserved2[61];
  XRDC_3A_MRCFG_tag MRCFG[1];          /* Memory Region Configuration */
  uint8_t XRDC_3A_reserved3[191];
  XRDC_3A_DERRLOC_tag DERRLOC[16];     /* Domain Error Location */
  uint8_t XRDC_3A_reserved4[448];
  XRDC_3A_DERRW0_tag DERRW0[20];       /* Valid array indices: 0, 16-19 */
  uint8_t XRDC_3A_reserved5[704];
  XRDC_3A_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_3A_reserved6[28];
  XRDC_3A_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W1_1_DFMT1_tag MDA_W1_1_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W2_1_DFMT1_tag MDA_W2_1_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W3_1_DFMT1_tag MDA_W3_1_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W4_1_DFMT1_tag MDA_W4_1_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W5_1_DFMT1_tag MDA_W5_1_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W6_1_DFMT1_tag MDA_W6_1_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W7_1_DFMT1_tag MDA_W7_1_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W0_2_DFMT1_tag MDA_W0_2_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W1_2_DFMT1_tag MDA_W1_2_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W2_2_DFMT1_tag MDA_W2_2_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W3_2_DFMT1_tag MDA_W3_2_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W4_2_DFMT1_tag MDA_W4_2_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W5_2_DFMT1_tag MDA_W5_2_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W6_2_DFMT1_tag MDA_W6_2_DFMT1; /* Master Domain Assignment */
  XRDC_3A_MDA_W7_2_DFMT1_tag MDA_W7_2_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_3A_reserved7[1952];
  XRDC_3A_PDAC_SLOT_tag PDAC_SLOT[4];
  XRDC_3A_MRGDN_tag MRGDN[1][16];
};


/* ============================================================================
   =============================== Module: XRDC_3B ============================
   ============================================================================ */

typedef union XRDC_3B_CR_union_tag {   /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} XRDC_3B_CR_tag;

typedef union XRDC_3B_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} XRDC_3B_HWCFG0_tag;

typedef union XRDC_3B_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} XRDC_3B_HWCFG1_tag;

typedef union XRDC_3B_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} XRDC_3B_DERRLOC_tag;

typedef union XRDC_3B_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} XRDC_3B_DERRW0_DERR_W0_tag;

typedef union XRDC_3B_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} XRDC_3B_DERRW0_DERR_W1_tag;

typedef union XRDC_3B_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} XRDC_3B_DERRW0_DERR_W3_tag;

typedef struct XRDC_3B_DERRW0_struct_tag {
  XRDC_3B_DERRW0_DERR_W0_tag DERR_W0;  /* Domain Error Word 0 */
  XRDC_3B_DERRW0_DERR_W1_tag DERR_W1;  /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  XRDC_3B_DERRW0_DERR_W3_tag DERR_W3;  /* Domain Error Word 3 */
} XRDC_3B_DERRW0_tag;

typedef union XRDC_3B_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_3B_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union XRDC_3B_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_3B_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct XRDC_3B_PDAC_SLOT_PDACN_struct_tag {
  XRDC_3B_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  XRDC_3B_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} XRDC_3B_PDAC_SLOT_PDACN_tag;

typedef struct XRDC_3B_PDAC_SLOT_struct_tag {
  XRDC_3B_PDAC_SLOT_PDACN_tag PDACN[16]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[896];
} XRDC_3B_PDAC_SLOT_tag;

struct XRDC_3B_tag {
  XRDC_3B_CR_tag CR;                   /* Control */
  uint8_t XRDC_3B_reserved0[236];
  XRDC_3B_HWCFG0_tag HWCFG0;           /* Hardware Configuration 0 */
  XRDC_3B_HWCFG1_tag HWCFG1;           /* Hardware Configuration 1 */
  uint8_t XRDC_3B_reserved1[264];
  XRDC_3B_DERRLOC_tag DERRLOC[16];     /* Domain Error Location */
  uint8_t XRDC_3B_reserved2[448];
  XRDC_3B_DERRW0_tag DERRW0[19];       /* Valid array indices: 16-18 */
  uint8_t XRDC_3B_reserved3[2768];
  XRDC_3B_PDAC_SLOT_tag PDAC_SLOT[3];
};


/* ============================================================================
   =============================== Module: XRDC_4 =============================
   ============================================================================ */

typedef union XRDC_4_CR_union_tag {    /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} XRDC_4_CR_tag;

typedef union XRDC_4_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} XRDC_4_HWCFG0_tag;

typedef union XRDC_4_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} XRDC_4_HWCFG1_tag;

typedef union XRDC_4_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} XRDC_4_MDACFG_tag;

typedef union XRDC_4_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} XRDC_4_MRCFG_tag;

typedef union XRDC_4_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} XRDC_4_DERRLOC_tag;

typedef union XRDC_4_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} XRDC_4_DERRW0_DERR_W0_tag;

typedef union XRDC_4_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} XRDC_4_DERRW0_DERR_W1_tag;

typedef union XRDC_4_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} XRDC_4_DERRW0_DERR_W3_tag;

typedef struct XRDC_4_DERRW0_struct_tag {
  XRDC_4_DERRW0_DERR_W0_tag DERR_W0;   /* Domain Error Word 0 */
  XRDC_4_DERRW0_DERR_W1_tag DERR_W1;   /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  XRDC_4_DERRW0_DERR_W3_tag DERR_W3;   /* Domain Error Word 3 */
} XRDC_4_DERRW0_tag;

typedef union XRDC_4_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W0_0_DFMT1_tag;

typedef union XRDC_4_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W0_1_DFMT1_tag;

typedef union XRDC_4_MDA_W0_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W0_2_DFMT1_tag;

typedef union XRDC_4_MDA_W1_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W1_2_DFMT1_tag;

typedef union XRDC_4_MDA_W2_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W2_2_DFMT1_tag;

typedef union XRDC_4_MDA_W3_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W3_2_DFMT1_tag;

typedef union XRDC_4_MDA_W4_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W4_2_DFMT1_tag;

typedef union XRDC_4_MDA_W5_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W5_2_DFMT1_tag;

typedef union XRDC_4_MDA_W6_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W6_2_DFMT1_tag;

typedef union XRDC_4_MDA_W7_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t LPID:4;                  /* Logical Partition Identifier */
    vuint32_t LPE:1;                   /* Logical Partition Enable */
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MDA_W7_2_DFMT1_tag;

typedef union XRDC_4_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_4_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union XRDC_4_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct XRDC_4_PDAC_SLOT_PDACN_struct_tag {
  XRDC_4_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  XRDC_4_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} XRDC_4_PDAC_SLOT_PDACN_tag;

typedef struct XRDC_4_PDAC_SLOT_struct_tag {
  XRDC_4_PDAC_SLOT_PDACN_tag PDACN[48]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[640];
} XRDC_4_PDAC_SLOT_tag;

typedef union XRDC_4_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} XRDC_4_MRGDN_XRDC_MRGD_W0_tag;

typedef union XRDC_4_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} XRDC_4_MRGDN_XRDC_MRGD_W1_tag;

typedef union XRDC_4_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_4_MRGDN_XRDC_MRGD_W2_tag;

typedef union XRDC_4_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_4_MRGDN_XRDC_MRGD_W3_tag;

typedef struct XRDC_4_MRGDN_struct_tag {
  XRDC_4_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  XRDC_4_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  XRDC_4_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  XRDC_4_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} XRDC_4_MRGDN_tag;

struct XRDC_4_tag {
  XRDC_4_CR_tag CR;                    /* Control */
  uint8_t XRDC_4_reserved0[236];
  XRDC_4_HWCFG0_tag HWCFG0;            /* Hardware Configuration 0 */
  XRDC_4_HWCFG1_tag HWCFG1;            /* Hardware Configuration 1 */
  uint8_t XRDC_4_reserved1[8];
  XRDC_4_MDACFG_tag MDACFG[3];         /* Master Domain Assignment Configuration */
  uint8_t XRDC_4_reserved2[61];
  XRDC_4_MRCFG_tag MRCFG[2];           /* Memory Region Configuration */
  uint8_t XRDC_4_reserved3[190];
  XRDC_4_DERRLOC_tag DERRLOC[16];      /* Domain Error Location */
  uint8_t XRDC_4_reserved4[448];
  XRDC_4_DERRW0_tag DERRW0[20];        /* Valid array indices: 0-1, 16-19 */
  uint8_t XRDC_4_reserved5[704];
  XRDC_4_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_4_reserved6[28];
  XRDC_4_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_4_reserved7[28];
  XRDC_4_MDA_W0_2_DFMT1_tag MDA_W0_2_DFMT1; /* Master Domain Assignment */
  XRDC_4_MDA_W1_2_DFMT1_tag MDA_W1_2_DFMT1; /* Master Domain Assignment */
  XRDC_4_MDA_W2_2_DFMT1_tag MDA_W2_2_DFMT1; /* Master Domain Assignment */
  XRDC_4_MDA_W3_2_DFMT1_tag MDA_W3_2_DFMT1; /* Master Domain Assignment */
  XRDC_4_MDA_W4_2_DFMT1_tag MDA_W4_2_DFMT1; /* Master Domain Assignment */
  XRDC_4_MDA_W5_2_DFMT1_tag MDA_W5_2_DFMT1; /* Master Domain Assignment */
  XRDC_4_MDA_W6_2_DFMT1_tag MDA_W6_2_DFMT1; /* Master Domain Assignment */
  XRDC_4_MDA_W7_2_DFMT1_tag MDA_W7_2_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_4_reserved8[1952];
  XRDC_4_PDAC_SLOT_tag PDAC_SLOT[4];
  XRDC_4_MRGDN_tag MRGDN[2][16];
};


/* ============================================================================
   =============================== Module: XRDC_5 =============================
   ============================================================================ */

typedef union XRDC_5_CR_union_tag {    /* Control */
  vuint32_t R;
  struct {
    vuint32_t GVLD:1;                  /* Global Valid (XRDC Global Enable/Disable) */
    vuint32_t HRL:4;                   /* Hardware Revision Level */
    vuint32_t _unused_5:2;
    vuint32_t MRF:1;                   /* Memory Region Format */
    vuint32_t VAW:1;                   /* Virtualization Aware */
    vuint32_t _unused_9:21;
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t _unused_31:1;
  } B;
} XRDC_5_CR_tag;

typedef union XRDC_5_HWCFG0_union_tag { /* Hardware Configuration 0 */
  vuint32_t R;
  struct {
    vuint32_t NDID:8;                  /* Number Of DIDs */
    vuint32_t NMSTR:8;                 /* Number Of Bus Masters */
    vuint32_t NMRC:8;                  /* Number of MRCs */
    vuint32_t NPAC:4;                  /* Number Of PACs */
    vuint32_t MID:4;                   /* Module ID */
  } B;
} XRDC_5_HWCFG0_tag;

typedef union XRDC_5_HWCFG1_union_tag { /* Hardware Configuration 1 */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t _unused_4:28;
  } B;
} XRDC_5_HWCFG1_tag;

typedef union XRDC_5_MDACFG_union_tag { /* Master Domain Assignment Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMDAR:4;                  /* Number Of Master Domain Assignment Registers */
    vuint8_t _unused_4:3;
    vuint8_t NCM:1;                    /* Noncore Master */
  } B;
} XRDC_5_MDACFG_tag;

typedef union XRDC_5_MRCFG_union_tag { /* Memory Region Configuration */
  vuint8_t R;
  struct {
    vuint8_t NMRGD:5;                  /* Number Of Memory Region Descriptors */
    vuint8_t _unused_5:3;
  } B;
} XRDC_5_MRCFG_tag;

typedef union XRDC_5_DERRLOC_union_tag { /* Domain Error Location */
  vuint32_t R;
  struct {
    vuint32_t MRCINST:16;              /* MRC Instance */
    vuint32_t PACINST:4;               /* PAC Instance */
    vuint32_t _unused_20:12;
  } B;
} XRDC_5_DERRLOC_tag;

typedef union XRDC_5_DERRW0_DERR_W0_union_tag { /* Domain Error Word 0 */
  vuint32_t R;
  struct {
    vuint32_t EADDR:32;                /* Error Address */
  } B;
} XRDC_5_DERRW0_DERR_W0_tag;

typedef union XRDC_5_DERRW0_DERR_W1_union_tag { /* Domain Error Word 1 */
  vuint32_t R;
  struct {
    vuint32_t EDID:4;                  /* Error Domain Identifier */
    vuint32_t _unused_4:4;
    vuint32_t EATR:3;                  /* Error Attributes */
    vuint32_t ERW:1;                   /* Error Read Or Write */
    vuint32_t _unused_12:12;
    vuint32_t EPORT:3;                 /* Error Port */
    vuint32_t _unused_27:3;
    vuint32_t EST:2;                   /* Error State */
  } B;
} XRDC_5_DERRW0_DERR_W1_tag;

typedef union XRDC_5_DERRW0_DERR_W3_union_tag { /* Domain Error Word 3 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:30;
    vuint32_t RECR:2;                  /* Rearm Error Capture Registers */
  } B;
} XRDC_5_DERRW0_DERR_W3_tag;

typedef struct XRDC_5_DERRW0_struct_tag {
  XRDC_5_DERRW0_DERR_W0_tag DERR_W0;   /* Domain Error Word 0 */
  XRDC_5_DERRW0_DERR_W1_tag DERR_W1;   /* Domain Error Word 1 */
  uint8_t DERRW0_reserved0[4];
  XRDC_5_DERRW0_DERR_W3_tag DERR_W3;   /* Domain Error Word 3 */
} XRDC_5_DERRW0_tag;

typedef union XRDC_5_MDA_W0_0_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_5_MDA_W0_0_DFMT1_tag;

typedef union XRDC_5_MDA_W0_1_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_5_MDA_W0_1_DFMT1_tag;

typedef union XRDC_5_MDA_W0_2_DFMT1_union_tag { /* Master Domain Assignment */
  vuint32_t R;
  struct {
    vuint32_t DID:4;                   /* Domain Identifier */
    vuint32_t PA:2;                    /* Privileged Attribute */
    vuint32_t SA:2;                    /* Secure Attribute */
    vuint32_t DIDB:1;                  /* DID Bypass */
    vuint32_t _unused_9:15;
    vuint32_t _unused_24:4;
    vuint32_t _unused_28:1;
    vuint32_t DFMT:1;                  /* Domain Format */
    vuint32_t LK1:1;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_5_MDA_W0_2_DFMT1_tag;

typedef union XRDC_5_PDAC_SLOT_PDACN_PDAC_W0_union_tag { /* Peripheral Domain Access Control Word 0 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_5_PDAC_SLOT_PDACN_PDAC_W0_tag;

typedef union XRDC_5_PDAC_SLOT_PDACN_PDAC_W1_union_tag { /* Peripheral Domain Access Control Word 1 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain ACP */
    vuint32_t D9ACP:3;                 /* Domain ACP */
    vuint32_t D10ACP:3;                /* Domain ACP */
    vuint32_t D11ACP:3;                /* Domain ACP */
    vuint32_t D12ACP:3;                /* Domain ACP */
    vuint32_t D13ACP:3;                /* Domain ACP */
    vuint32_t D14ACP:3;                /* Domain ACP */
    vuint32_t D15ACP:3;                /* Domain ACP */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_5_PDAC_SLOT_PDACN_PDAC_W1_tag;

typedef struct XRDC_5_PDAC_SLOT_PDACN_struct_tag {
  XRDC_5_PDAC_SLOT_PDACN_PDAC_W0_tag PDAC_W0; /* Peripheral Domain Access Control Word 0 */
  XRDC_5_PDAC_SLOT_PDACN_PDAC_W1_tag PDAC_W1; /* Peripheral Domain Access Control Word 1 */
} XRDC_5_PDAC_SLOT_PDACN_tag;

typedef struct XRDC_5_PDAC_SLOT_struct_tag {
  XRDC_5_PDAC_SLOT_PDACN_tag PDACN[48]; /* Not every combination of array indices is valid */
  uint8_t PDAC_SLOT_reserved0[640];
} XRDC_5_PDAC_SLOT_tag;

typedef union XRDC_5_MRGDN_XRDC_MRGD_W0_union_tag { /* Memory Region Descriptor Word 0 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t SRTADDR:27;              /* Start Address */
  } B;
} XRDC_5_MRGDN_XRDC_MRGD_W0_tag;

typedef union XRDC_5_MRGDN_XRDC_MRGD_W1_union_tag { /* Memory Region Descriptor Word 1 */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:5;
    vuint32_t ENDADDR:27;              /* End Address */
  } B;
} XRDC_5_MRGDN_XRDC_MRGD_W1_tag;

typedef union XRDC_5_MRGDN_XRDC_MRGD_W2_union_tag { /* Memory Region Descriptor Word 2 */
  vuint32_t R;
  struct {
    vuint32_t D0ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D1ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D2ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D3ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D4ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D5ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D6ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D7ACP:3;                 /* Domain Access Control Policy */
    vuint32_t SNUM:4;                  /* Semaphore Number */
    vuint32_t _unused_28:2;
    vuint32_t SE:1;                    /* Semaphore Enable */
    vuint32_t _unused_31:1;
  } B;
} XRDC_5_MRGDN_XRDC_MRGD_W2_tag;

typedef union XRDC_5_MRGDN_XRDC_MRGD_W3_union_tag { /* Memory Region Descriptor Word 3 */
  vuint32_t R;
  struct {
    vuint32_t D8ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D9ACP:3;                 /* Domain Access Control Policy */
    vuint32_t D10ACP:3;                /* Domain Access Control Policy */
    vuint32_t D11ACP:3;                /* Domain Access Control Policy */
    vuint32_t D12ACP:3;                /* Domain Access Control Policy */
    vuint32_t D13ACP:3;                /* Domain Access Control Policy */
    vuint32_t D14ACP:3;                /* Domain Access Control Policy */
    vuint32_t D15ACP:3;                /* Domain Access Control Policy */
    vuint32_t _unused_24:5;
    vuint32_t LK2:2;                   /* Lock */
    vuint32_t VLD:1;                   /* Valid */
  } B;
} XRDC_5_MRGDN_XRDC_MRGD_W3_tag;

typedef struct XRDC_5_MRGDN_struct_tag {
  XRDC_5_MRGDN_XRDC_MRGD_W0_tag XRDC_MRGD_W0; /* Memory Region Descriptor Word 0 */
  XRDC_5_MRGDN_XRDC_MRGD_W1_tag XRDC_MRGD_W1; /* Memory Region Descriptor Word 1 */
  XRDC_5_MRGDN_XRDC_MRGD_W2_tag XRDC_MRGD_W2; /* Memory Region Descriptor Word 2 */
  XRDC_5_MRGDN_XRDC_MRGD_W3_tag XRDC_MRGD_W3; /* Memory Region Descriptor Word 3 */
  uint8_t MRGDN_reserved0[16];
} XRDC_5_MRGDN_tag;

struct XRDC_5_tag {
  XRDC_5_CR_tag CR;                    /* Control */
  uint8_t XRDC_5_reserved0[236];
  XRDC_5_HWCFG0_tag HWCFG0;            /* Hardware Configuration 0 */
  XRDC_5_HWCFG1_tag HWCFG1;            /* Hardware Configuration 1 */
  uint8_t XRDC_5_reserved1[8];
  XRDC_5_MDACFG_tag MDACFG[3];         /* Master Domain Assignment Configuration */
  uint8_t XRDC_5_reserved2[61];
  XRDC_5_MRCFG_tag MRCFG[3];           /* Memory Region Configuration */
  uint8_t XRDC_5_reserved3[189];
  XRDC_5_DERRLOC_tag DERRLOC[16];      /* Domain Error Location */
  uint8_t XRDC_5_reserved4[448];
  XRDC_5_DERRW0_tag DERRW0[19];        /* Valid array indices: 0-2, 16-18 */
  uint8_t XRDC_5_reserved5[720];
  XRDC_5_MDA_W0_0_DFMT1_tag MDA_W0_0_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_5_reserved6[28];
  XRDC_5_MDA_W0_1_DFMT1_tag MDA_W0_1_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_5_reserved7[28];
  XRDC_5_MDA_W0_2_DFMT1_tag MDA_W0_2_DFMT1; /* Master Domain Assignment */
  uint8_t XRDC_5_reserved8[1980];
  XRDC_5_PDAC_SLOT_tag PDAC_SLOT[3];
  uint8_t XRDC_5_reserved9[1024];
  XRDC_5_MRGDN_tag MRGDN[3][16];
};


/* ============================================================================
   =============================== Module: ierc_ierb ==========================
   ============================================================================ */

typedef union ierc_ierb_F0_EC_CFH_DIDVID_union_tag { /* Function 0 EC config header device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} ierc_ierb_F0_EC_CFH_DIDVID_tag;

typedef union ierc_ierb_F0_EC_CFH_SIDSVID_union_tag { /* Function 0 EC config header subsystem ID and subsystem vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t SUBSYSTEM_VENDOR_ID:16;
    vuint32_t SUBSYSTEM_DEVICE_ID:16;
  } B;
} ierc_ierb_F0_EC_CFH_SIDSVID_tag;

struct ierc_ierb_tag {
  ierc_ierb_F0_EC_CFH_DIDVID_tag F0_EC_CFH_DIDVID; /* Function 0 EC config header device ID and vendor ID register */
  ierc_ierb_F0_EC_CFH_SIDSVID_tag F0_EC_CFH_SIDSVID; /* Function 0 EC config header subsystem ID and subsystem vendor ID register */
};


/* ============================================================================
   =============================== Module: ierc_pci ===========================
   ============================================================================ */

typedef union ierc_pci_PCI_CFH_DID_VID_union_tag { /* PCI device ID and vendor ID register */
  vuint32_t R;
  struct {
    vuint32_t VENDOR_ID:16;
    vuint32_t DEVICE_ID:16;
  } B;
} ierc_pci_PCI_CFH_DID_VID_tag;

typedef union ierc_pci_PCI_CFH_CMD_union_tag { /* PCI command register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:10;
    vuint16_t INTR_DISABLE:1;
    vuint16_t _unused_11:5;
  } B;
} ierc_pci_PCI_CFH_CMD_tag;

typedef union ierc_pci_PCI_CFH_STAT_union_tag { /* PCI status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:3;
    vuint16_t INTR_STATUS:1;
    vuint16_t CAP_LIST:1;
    vuint16_t _unused_5:11;
  } B;
} ierc_pci_PCI_CFH_STAT_tag;

typedef union ierc_pci_PCI_CFH_REVID_CLASSCODE_union_tag { /* PCI revision ID and classcode register */
  vuint32_t R;
  struct {
    vuint32_t REV_ID:8;
    vuint32_t CLASS_CODE:24;
  } B;
} ierc_pci_PCI_CFH_REVID_CLASSCODE_tag;

typedef union ierc_pci_PCI_CFH_CL_SIZE_union_tag { /* PCI cache line size register */
  vuint8_t R;
  struct {
    vuint8_t CACHE_LINE_SIZE:8;
  } B;
} ierc_pci_PCI_CFH_CL_SIZE_tag;

typedef union ierc_pci_PCI_CFH_HDR_TYPE_union_tag { /* PCI header type register */
  vuint8_t R;
  struct {
    vuint8_t HDR_TYPE:7;
    vuint8_t MULT_FUNC_DEV:1;
  } B;
} ierc_pci_PCI_CFH_HDR_TYPE_tag;

typedef union ierc_pci_PCI_CFH_SUBSYS_VID_union_tag { /* PCI subsystem vendor ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_VENDOR_ID:16;
  } B;
} ierc_pci_PCI_CFH_SUBSYS_VID_tag;

typedef union ierc_pci_PCI_CFH_SUBSYS_ID_union_tag { /* PCI subsystem ID register */
  vuint16_t R;
  struct {
    vuint16_t SUBSYSTEM_ID:16;
  } B;
} ierc_pci_PCI_CFH_SUBSYS_ID_tag;

typedef union ierc_pci_PCI_CFH_CAP_PTR_union_tag { /* PCI capabilities pointer register */
  vuint8_t R;
  struct {
    vuint8_t CAP_PTR:8;
  } B;
} ierc_pci_PCI_CFH_CAP_PTR_tag;

typedef union ierc_pci_PCI_CFH_INT_LINE_union_tag { /* PCI interrupt line register */
  vuint8_t R;
  struct {
    vuint8_t INT_LINE:8;
  } B;
} ierc_pci_PCI_CFH_INT_LINE_tag;

typedef union ierc_pci_PCI_CFH_INT_PIN_union_tag { /* PCI interrupt pin register */
  vuint8_t R;
  struct {
    vuint8_t INT_PIN:8;
  } B;
} ierc_pci_PCI_CFH_INT_PIN_tag;

typedef union ierc_pci_PCI_CFC_PCIE_CAP_LIST_union_tag { /* PCI PCIe capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_ID:8;
  } B;
} ierc_pci_PCI_CFC_PCIE_CAP_LIST_tag;

typedef union ierc_pci_PCI_CFC_PCIE_CAP_union_tag { /* PCI PCIe capabilities register */
  vuint16_t R;
  struct {
    vuint16_t CAP_VER:4;
    vuint16_t DEV_PORT_TYPE:4;
    vuint16_t _unused_8:1;
    vuint16_t INT_MSG_NUM:5;
    vuint16_t _unused_14:2;
  } B;
} ierc_pci_PCI_CFC_PCIE_CAP_tag;

typedef union ierc_pci_PCI_CFC_PCIE_DEV_CAP_union_tag { /* PCI PCIe device capabilities register */
  vuint32_t R;
  struct {
    vuint32_t _unused_0:28;
    vuint32_t FLR_CAP:1;
    vuint32_t _unused_29:3;
  } B;
} ierc_pci_PCI_CFC_PCIE_DEV_CAP_tag;

typedef union ierc_pci_PCI_CFC_PCIE_DEV_STAT_union_tag { /* PCI PCIe device status register */
  vuint16_t R;
  struct {
    vuint16_t _unused_0:5;
    vuint16_t TRANS_PEND:1;
    vuint16_t _unused_6:10;
  } B;
} ierc_pci_PCI_CFC_PCIE_DEV_STAT_tag;

typedef union ierc_pci_PCI_CFC_PCIPM_CAP_LIST_union_tag { /* PCI PCI-PM capabilities list register */
  vuint16_t R;
  struct {
    vuint16_t CAP_ID:8;
    vuint16_t NEXT_CAP_ID:8;
  } B;
} ierc_pci_PCI_CFC_PCIPM_CAP_LIST_tag;

typedef union ierc_pci_PCI_CFC_PCIPM_CAP_union_tag { /* PCI PCI-PM capabilities register */
  vuint16_t R;
  struct {
    vuint16_t VERSION:3;
    vuint16_t _unused_3:8;
    vuint16_t PME_SUPPORT:5;
  } B;
} ierc_pci_PCI_CFC_PCIPM_CAP_tag;

typedef union ierc_pci_PCI_CFC_PCIPM_CTL_STAT_union_tag { /* PCI PCI-PM control and status register */
  vuint16_t R;
  struct {
    vuint16_t PWR_STATE:2;
    vuint16_t _unused_2:1;
    vuint16_t NO_SOFT_RST:1;
    vuint16_t _unused_4:12;
  } B;
} ierc_pci_PCI_CFC_PCIPM_CTL_STAT_tag;

typedef union ierc_pci_PCI_CFC_PCIPM_DATA_union_tag { /* PCI PCI-PM capabilities data register */
  vuint8_t R;
  struct {
    vuint8_t _unused_0:8;
  } B;
} ierc_pci_PCI_CFC_PCIPM_DATA_tag;

typedef union ierc_pci_PCIE_CFC_AER_EXT_CAP_HDR_union_tag { /* PCIe AER extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} ierc_pci_PCIE_CFC_AER_EXT_CAP_HDR_tag;

typedef union ierc_pci_PCIE_CFC_AER_ROOT_ERR_CMD_union_tag { /* PCIe AER root error command register */
  vuint32_t R;
  struct {
    vuint32_t CORR_ERR_RPT_EN:1;
    vuint32_t NON_FATAL_ERR_RPT_EN:1;
    vuint32_t FATAL_ERR_RPT_EN:1;
    vuint32_t _unused_3:29;
  } B;
} ierc_pci_PCIE_CFC_AER_ROOT_ERR_CMD_tag;

typedef union ierc_pci_PCIE_CFC_AER_ROOT_ERR_STAT_union_tag { /* PCIe AER root error status register */
  vuint32_t R;
  struct {
    vuint32_t ERR_CORR:1;
    vuint32_t MULT_ERR_CORR:1;
    vuint32_t ERR_FATAL_NON_FATAL:1;
    vuint32_t MULT_ERR_FATAL_NON_FATAL:1;
    vuint32_t FIRST_UCORR_FATAL:1;
    vuint32_t ERR_NON_FATAL:1;
    vuint32_t ERR_FATAL:1;
    vuint32_t _unused_7:25;
  } B;
} ierc_pci_PCIE_CFC_AER_ROOT_ERR_STAT_tag;

typedef union ierc_pci_PCIE_CFC_AER_ERR_SRC_ID_union_tag { /* PCIe AER error source identification register */
  vuint32_t R;
  struct {
    vuint32_t ERR_CORR_SRC_ID:16;
    vuint32_t ERR_FATAL_NON_FATAL_SRC_ID:16;
  } B;
} ierc_pci_PCIE_CFC_AER_ERR_SRC_ID_tag;

typedef union ierc_pci_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_union_tag { /* PCIe RCEC Endpoint association extended capability header */
  vuint32_t R;
  struct {
    vuint32_t PCIE_EXT_CAP_ID:16;
    vuint32_t CAP_VER:4;
    vuint32_t NEXT_CAP_OFF:12;
  } B;
} ierc_pci_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_tag;

typedef union ierc_pci_PCIE_CFC_RCEC_EPA_BITMAP_union_tag { /* PCIe RCEC Endpoint association bitmap registerr */
  vuint32_t R;
  struct {
    vuint32_t DEV_BITMAP:32;
  } B;
} ierc_pci_PCIE_CFC_RCEC_EPA_BITMAP_tag;

struct ierc_pci_tag {
  ierc_pci_PCI_CFH_DID_VID_tag PCI_CFH_DID_VID; /* PCI device ID and vendor ID register */
  ierc_pci_PCI_CFH_CMD_tag PCI_CFH_CMD; /* PCI command register */
  ierc_pci_PCI_CFH_STAT_tag PCI_CFH_STAT; /* PCI status register */
  ierc_pci_PCI_CFH_REVID_CLASSCODE_tag PCI_CFH_REVID_CLASSCODE; /* PCI revision ID and classcode register */
  ierc_pci_PCI_CFH_CL_SIZE_tag PCI_CFH_CL_SIZE; /* PCI cache line size register */
  uint8_t ierc_pci_reserved0[1];
  ierc_pci_PCI_CFH_HDR_TYPE_tag PCI_CFH_HDR_TYPE; /* PCI header type register */
  uint8_t ierc_pci_reserved1[29];
  ierc_pci_PCI_CFH_SUBSYS_VID_tag PCI_CFH_SUBSYS_VID; /* PCI subsystem vendor ID register */
  ierc_pci_PCI_CFH_SUBSYS_ID_tag PCI_CFH_SUBSYS_ID; /* PCI subsystem ID register */
  uint8_t ierc_pci_reserved2[4];
  ierc_pci_PCI_CFH_CAP_PTR_tag PCI_CFH_CAP_PTR; /* PCI capabilities pointer register */
  uint8_t ierc_pci_reserved3[7];
  ierc_pci_PCI_CFH_INT_LINE_tag PCI_CFH_INT_LINE; /* PCI interrupt line register */
  ierc_pci_PCI_CFH_INT_PIN_tag PCI_CFH_INT_PIN; /* PCI interrupt pin register */
  uint8_t ierc_pci_reserved4[2];
  ierc_pci_PCI_CFC_PCIE_CAP_LIST_tag PCI_CFC_PCIE_CAP_LIST; /* PCI PCIe capabilities list register */
  ierc_pci_PCI_CFC_PCIE_CAP_tag PCI_CFC_PCIE_CAP; /* PCI PCIe capabilities register */
  ierc_pci_PCI_CFC_PCIE_DEV_CAP_tag PCI_CFC_PCIE_DEV_CAP; /* PCI PCIe device capabilities register */
  uint8_t ierc_pci_reserved5[2];
  ierc_pci_PCI_CFC_PCIE_DEV_STAT_tag PCI_CFC_PCIE_DEV_STAT; /* PCI PCIe device status register */
  uint8_t ierc_pci_reserved6[52];
  ierc_pci_PCI_CFC_PCIPM_CAP_LIST_tag PCI_CFC_PCIPM_CAP_LIST; /* PCI PCI-PM capabilities list register */
  ierc_pci_PCI_CFC_PCIPM_CAP_tag PCI_CFC_PCIPM_CAP; /* PCI PCI-PM capabilities register */
  ierc_pci_PCI_CFC_PCIPM_CTL_STAT_tag PCI_CFC_PCIPM_CTL_STAT; /* PCI PCI-PM control and status register */
  uint8_t ierc_pci_reserved7[1];
  ierc_pci_PCI_CFC_PCIPM_DATA_tag PCI_CFC_PCIPM_DATA; /* PCI PCI-PM capabilities data register */
  uint8_t ierc_pci_reserved8[120];
  ierc_pci_PCIE_CFC_AER_EXT_CAP_HDR_tag PCIE_CFC_AER_EXT_CAP_HDR; /* PCIe AER extended capability header */
  uint8_t ierc_pci_reserved9[40];
  ierc_pci_PCIE_CFC_AER_ROOT_ERR_CMD_tag PCIE_CFC_AER_ROOT_ERR_CMD; /* PCIe AER root error command register */
  ierc_pci_PCIE_CFC_AER_ROOT_ERR_STAT_tag PCIE_CFC_AER_ROOT_ERR_STAT; /* PCIe AER root error status register */
  ierc_pci_PCIE_CFC_AER_ERR_SRC_ID_tag PCIE_CFC_AER_ERR_SRC_ID; /* PCIe AER error source identification register */
  ierc_pci_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_tag PCIE_CFC_RCEC_EPA_EXT_CAP_HDR; /* PCIe RCEC Endpoint association extended capability header */
  ierc_pci_PCIE_CFC_RCEC_EPA_BITMAP_tag PCIE_CFC_RCEC_EPA_BITMAP; /* PCIe RCEC Endpoint association bitmap registerr */
};

#define CEVA_SPF2__ACCESS_PROTECTION (*(volatile struct ACCESS_PROTECTION_tag *) 0x24400C80UL)
#define AES__ACE__AHB (*(volatile struct ACE_tag *) 0x47000000UL)
#define AES__ACE__AHB2 (*(volatile struct ACE_tag *) 0x47000000UL)
#define ADC_0 (*(volatile struct ADC_tag *) 0x402C0000UL)
#define ADC_1 (*(volatile struct ADC_tag *) 0x402E0000UL)
#define AES__AES_ACCEL (*(volatile struct AES_ACCEL_tag *) 0x472A0000UL)
#define ATP (*(volatile struct ATP_tag *) 0x4D190000UL)
#define CE_AXBS (*(volatile struct AXBS_tag *) 0x44860000UL)
#define SMU__AXBS (*(volatile struct AXBS_tag *) 0x45060000UL)
#define BMR (*(volatile struct Boot_tag *) 0x42280000UL)
#define POR (*(volatile struct Boot_tag *) 0x42060000UL)
#define CANXL_0__DSC_CONTROL (*(volatile struct CANXL_DSC_CONTROL_tag *) 0x47424000UL)
#define CANXL_1__DSC_CONTROL (*(volatile struct CANXL_DSC_CONTROL_tag *) 0x47524000UL)
#define CANXL_0__FILTER_BANK (*(volatile struct CANXL_FILTER_BANK_tag *) 0x47422000UL)
#define CANXL_1__FILTER_BANK (*(volatile struct CANXL_FILTER_BANK_tag *) 0x47522000UL)
#define CANXL_0__GRP_CONTROL (*(volatile struct CANXL_GRP_CONTROL_tag *) 0x47426000UL)
#define CANXL_1__GRP_CONTROL (*(volatile struct CANXL_GRP_CONTROL_tag *) 0x47526000UL)
#define CANXL_0__MRU (*(volatile struct CANXL_MRU_tag *) 0x47427000UL)
#define CANXL_1__MRU (*(volatile struct CANXL_MRU_tag *) 0x47527000UL)
#define CANXL_0__MSG_DESCRIPTORS (*(volatile struct CANXL_MSG_DESCRIPTORS_tag *) 0x47420000UL)
#define CANXL_1__MSG_DESCRIPTORS (*(volatile struct CANXL_MSG_DESCRIPTORS_tag *) 0x47520000UL)
#define CANXL_0__RAMECC (*(volatile struct CANXL_RAMECC_tag *) 0x47460000UL)
#define CANXL_1__RAMECC (*(volatile struct CANXL_RAMECC_tag *) 0x47560000UL)
#define CANXL_0__RXFIFO (*(volatile struct CANXL_RXFIFO_tag *) 0x47423000UL)
#define CANXL_1__RXFIFO (*(volatile struct CANXL_RXFIFO_tag *) 0x47523000UL)
#define CANXL_0__RXFIFO_CONTROL (*(volatile struct CANXL_RXFIFO_CONTROL_tag *) 0x47425000UL)
#define CANXL_1__RXFIFO_CONTROL (*(volatile struct CANXL_RXFIFO_CONTROL_tag *) 0x47525000UL)
#define CANXL_0__SIC (*(volatile struct CANXL_SIC_tag *) 0x4741B000UL)
#define CANXL_1__SIC (*(volatile struct CANXL_SIC_tag *) 0x4751B000UL)
#define CE_CAN_HUB (*(volatile struct CAN_HUB_tag *) 0x448BC000UL)
#define CE_CAN_TBS_0 (*(volatile struct CAN_TBS_tag *) 0x449A8000UL)
#define CE_CAN_TBS_10 (*(volatile struct CAN_TBS_tag *) 0x44BE8000UL)
#define CE_CAN_TBS_11 (*(volatile struct CAN_TBS_tag *) 0x44BF8000UL)
#define CE_CAN_TBS_12 (*(volatile struct CAN_TBS_tag *) 0x44DA8000UL)
#define CE_CAN_TBS_13 (*(volatile struct CAN_TBS_tag *) 0x44DB8000UL)
#define CE_CAN_TBS_14 (*(volatile struct CAN_TBS_tag *) 0x44DC8000UL)
#define CE_CAN_TBS_15 (*(volatile struct CAN_TBS_tag *) 0x44DD8000UL)
#define CE_CAN_TBS_16 (*(volatile struct CAN_TBS_tag *) 0x44DE8000UL)
#define CE_CAN_TBS_17 (*(volatile struct CAN_TBS_tag *) 0x44DF8000UL)
#define CE_CAN_TBS_18 (*(volatile struct CAN_TBS_tag *) 0x44FA8000UL)
#define CE_CAN_TBS_19 (*(volatile struct CAN_TBS_tag *) 0x44FB8000UL)
#define CE_CAN_TBS_1 (*(volatile struct CAN_TBS_tag *) 0x449B8000UL)
#define CE_CAN_TBS_20 (*(volatile struct CAN_TBS_tag *) 0x44FC8000UL)
#define CE_CAN_TBS_21 (*(volatile struct CAN_TBS_tag *) 0x44FD8000UL)
#define CE_CAN_TBS_22 (*(volatile struct CAN_TBS_tag *) 0x44FE8000UL)
#define CE_CAN_TBS_23 (*(volatile struct CAN_TBS_tag *) 0x44FF8000UL)
#define CE_CAN_TBS_2 (*(volatile struct CAN_TBS_tag *) 0x449C8000UL)
#define CE_CAN_TBS_3 (*(volatile struct CAN_TBS_tag *) 0x449D8000UL)
#define CE_CAN_TBS_4 (*(volatile struct CAN_TBS_tag *) 0x449E8000UL)
#define CE_CAN_TBS_5 (*(volatile struct CAN_TBS_tag *) 0x449F8000UL)
#define CE_CAN_TBS_6 (*(volatile struct CAN_TBS_tag *) 0x44BA8000UL)
#define CE_CAN_TBS_7 (*(volatile struct CAN_TBS_tag *) 0x44BB8000UL)
#define CE_CAN_TBS_8 (*(volatile struct CAN_TBS_tag *) 0x44BC8000UL)
#define CE_CAN_TBS_9 (*(volatile struct CAN_TBS_tag *) 0x44BD8000UL)
#define CANXL_0_TBS (*(volatile struct CAN_TBS_tag *) 0x419B0000UL)
#define CANXL_1_TBS (*(volatile struct CAN_TBS_tag *) 0x419C0000UL)
#define CE_DMAMUX (*(volatile struct CE_DMAMUX_tag *) 0x448D8000UL)
#define CE_L_VFCCU (*(volatile struct CE_L_VFCCU_tag *) 0x44840000UL)
#define CE_SEMA42 (*(volatile struct CE_SEMA42_tag *) 0x44A30000UL)
#define CE_SRG_S (*(volatile struct CE_SRG_S_tag *) 0x44864000UL)
#define CE_XRDC (*(volatile struct CE_XRDC_tag *) 0x44800000UL)
#define CEVA_SPF2_APB__CLASS_0X9_CORESIGHT_COMPONENT (*(volatile struct CLASS_0X9_CORESIGHT_COMPONENT_tag *) 0x4D180FCCUL)
#define CE_CMU_FC_2 (*(volatile struct CMU_FC_tag *) 0x44E10000UL)
#define CE_CMU_FC_1 (*(volatile struct CMU_FC_tag *) 0x44A54000UL)
#define CE_CMU_FC_0 (*(volatile struct CMU_FC_tag *) 0x44854000UL)
#define AES__CMU_FC (*(volatile struct CMU_FC_tag *) 0x472B0000UL)
#define CMU_FC_3 (*(volatile struct CMU_FC_tag *) 0x41840000UL)
#define CMU_FC_6 (*(volatile struct CMU_FC_tag *) 0x44040000UL)
#define CMU_FC_1 (*(volatile struct CMU_FC_tag *) 0x40840000UL)
#define CMU_FC_DEBUG_2 (*(volatile struct CMU_FC_tag *) 0x40850000UL)
#define SMU__CMU_FC (*(volatile struct CMU_FC_tag *) 0x45054000UL)
#define CMU_FC_2D (*(volatile struct CMU_FC_tag *) 0x410C0000UL)
#define CMU_FC_2C (*(volatile struct CMU_FC_tag *) 0x410B0000UL)
#define CMU_FC_2A (*(volatile struct CMU_FC_tag *) 0x41040000UL)
#define CMU_FC_2B (*(volatile struct CMU_FC_tag *) 0x410A0000UL)
#define CMU_FC_0 (*(volatile struct CMU_FC_tag *) 0x40040000UL)
#define CMU_FC_DEBUG_1 (*(volatile struct CMU_FC_tag *) 0x40050000UL)
#define CMU_FC_4 (*(volatile struct CMU_FC_tag *) 0x42040000UL)
#define RTU0__CMU_FC_0 (*(volatile struct CMU_FC_tag *) 0x76080000UL)
#define RTU1__CMU_FC_0 (*(volatile struct CMU_FC_tag *) 0x76880000UL)
#define RTU0__CMU_FC_3 (*(volatile struct CMU_FC_tag *) 0x76160000UL)
#define RTU1__CMU_FC_3 (*(volatile struct CMU_FC_tag *) 0x76960000UL)
#define RTU0__CMU_FC_4 (*(volatile struct CMU_FC_tag *) 0x76170000UL)
#define RTU1__CMU_FC_4 (*(volatile struct CMU_FC_tag *) 0x76970000UL)
#define RTU0__CMU_FC_1 (*(volatile struct CMU_FC_tag *) 0x760A0000UL)
#define RTU1__CMU_FC_1 (*(volatile struct CMU_FC_tag *) 0x768A0000UL)
#define RTU0__CMU_FC_2 (*(volatile struct CMU_FC_tag *) 0x760B0000UL)
#define RTU1__CMU_FC_2 (*(volatile struct CMU_FC_tag *) 0x768B0000UL)
#define CMU_FC_5 (*(volatile struct CMU_FC_tag *) 0x42840000UL)
#define CEVA_SPF2_APB__CORESIGHT_OCEM (*(volatile struct CORESIGHT_OCEM_tag *) 0x4D180CFCUL)
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL (*(volatile struct CORESIGHT_PROGRAMMING_MODEL_tag *) 0x4D180FD0UL)
#define CEVA_SPF2__CORE_DEBUGGER_INTERFACE (*(volatile struct CORE_DEBUGGER_INTERFACE_tag *) 0x244001D0UL)
#define CORE_PLL (*(volatile struct CORE_PLL_tag *) 0x40210000UL)
#define CEVA_SPF2__CORE_SAFETY (*(volatile struct CORE_SAFETY_tag *) 0x24400750UL)
#define CE_CRC_0 (*(volatile struct CRC_tag *) 0x44A88000UL)
#define CE_CRC_1 (*(volatile struct CRC_tag *) 0x44A98000UL)
#define CE_CRC_2 (*(volatile struct CRC_tag *) 0x44AA8000UL)
#define CE_CRC_3 (*(volatile struct CRC_tag *) 0x44AB8000UL)
#define CE_CRC_4 (*(volatile struct CRC_tag *) 0x44AC8000UL)
#define CE_CRC_5 (*(volatile struct CRC_tag *) 0x44AD8000UL)
#define RTU0__CRC (*(volatile struct CRC_tag *) 0x761C0000UL)
#define RTU1__CRC (*(volatile struct CRC_tag *) 0x769C0000UL)
#define CSTCU (*(volatile struct CSTCU_tag *) 0x41910000UL)
#define CTU (*(volatile struct CTU_tag *) 0x40390000UL)
#define C_VFCCU (*(volatile struct C_VFCCU_tag *) 0x41C60000UL)
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM (*(volatile struct DATA_MEMORY_SUBSYSTEM_tag *) 0x24400600UL)
#define CEVA_SPF2__DBG (*(volatile struct DBG_tag *) 0x24400D00UL)
#define CEVA_SPF2__DCU (*(volatile struct DCU_tag *) 0x24400110UL)
#define DDR (*(volatile struct DDRC_tag *) 0x72000000UL)
#define DDR_PLL (*(volatile struct DDR_PLL_tag *) 0x440A0000UL)
#define CORE_DFS (*(volatile struct DFS_tag *) 0x40260000UL)
#define PERIPH_DFS (*(volatile struct DFS_tag *) 0x40270000UL)
#define DIPORTSD (*(volatile struct DIPORTSD_tag *) 0x42960000UL)
#define DMAMUX_1 (*(volatile struct DMAMUX_tag *) 0x40DA0000UL)
#define DMAMUX_0A (*(volatile struct DMAMUX_tag *) 0x405A0000UL)
#define DMAMUX_0B (*(volatile struct DMAMUX_tag *) 0x405B0000UL)
#define DMAMUX_4 (*(volatile struct DMAMUX_tag *) 0x425A0000UL)
#define DMAMUX_5 (*(volatile struct DMAMUX_tag *) 0x42DA0000UL)
#define CE_DMA_CRC (*(volatile struct DMA_CRC_tag *) 0x44E08000UL)
#define DMA_CRC_3 (*(volatile struct DMA_CRC_tag *) 0x41DE0000UL)
#define DMA_CRC_1 (*(volatile struct DMA_CRC_tag *) 0x40DE0000UL)
#define DMA_CRC_0 (*(volatile struct DMA_CRC_tag *) 0x405E0000UL)
#define DMA_CRC_4 (*(volatile struct DMA_CRC_tag *) 0x425E0000UL)
#define DMA_CRC_5 (*(volatile struct DMA_CRC_tag *) 0x42DE0000UL)
#define CEVA_SPF2__DMSS_SAFETY (*(volatile struct DMSS_SAFETY_tag *) 0x244007B0UL)
#define MSC_0_DSPI (*(volatile struct DSPI_10_tag *) 0x40340000UL)
#define EDMA_0_MP (*(volatile struct EDMA3_MP_tag *) 0x405D0000UL)
#define EDMA_1_MP (*(volatile struct EDMA3_MP_tag *) 0x40DD0000UL)
#define EDMA_4_MP (*(volatile struct EDMA3_MP_tag *) 0x425D0000UL)
#define EDMA_5_MP (*(volatile struct EDMA3_MP_tag *) 0x42DD0000UL)
#define CE_EDMA_MP (*(volatile struct EDMA3_MP_tag *) 0x44E00000UL)
#define EDMA_0_TCD (*(volatile struct EDMA3_TCD_tag *) 0x40600000UL)
#define EDMA_1_TCD (*(volatile struct EDMA3_TCD_tag *) 0x40E00000UL)
#define EDMA_4_TCD (*(volatile struct EDMA3_TCD_tag *) 0x42600000UL)
#define EDMA_5_TCD (*(volatile struct EDMA3_TCD_tag *) 0x42E00000UL)
#define CE_EDMA_TCD (*(volatile struct EDMA3_TCD_tag *) 0x44E04000UL)
#define EDMA_3_MP (*(volatile struct EDMA4_MP_tag *) 0x41DD0000UL)
#define EDMA_3_TCD (*(volatile struct EDMA4_TCD_tag *) 0x41E00000UL)
#define CE_EIM_0 (*(volatile struct EIM_tag *) 0x44814000UL)
#define CE_EIM_1 (*(volatile struct EIM_tag *) 0x4481C000UL)
#define EIM_3 (*(volatile struct EIM_tag *) 0x41880000UL)
#define EIM_1 (*(volatile struct EIM_tag *) 0x40880000UL)
#define SMU__EIM (*(volatile struct EIM_tag *) 0x45014000UL)
#define EIM_2 (*(volatile struct EIM_tag *) 0x41080000UL)
#define EIM_GTM (*(volatile struct EIM_GTM_tag *) 0x40378000UL)
#define EMIOS_1 (*(volatile struct EMIOS_tag *) 0x400B0000UL)
#define EMIOS_0 (*(volatile struct EMIOS_tag *) 0x420B0000UL)
#define NETC__ENETC0_PORT (*(volatile struct ENETC_PORT_tag *) 0x74B14000UL)
#define NETC__ENETC0_PSEUDO_MAC_PORT (*(volatile struct ENETC_PSEUDO_MAC_PORT2_tag *) 0x74B15000UL)
#define CE_ERM_2 (*(volatile struct ERM_tag *) 0x44830000UL)
#define CE_ERM_0 (*(volatile struct ERM_tag *) 0x44810000UL)
#define CE_ERM_1 (*(volatile struct ERM_tag *) 0x44818000UL)
#define ERM_3 (*(volatile struct ERM_tag *) 0x41870000UL)
#define ERM_1 (*(volatile struct ERM_tag *) 0x40870000UL)
#define SMU__ERM (*(volatile struct ERM_tag *) 0x45010000UL)
#define ERM_2 (*(volatile struct ERM_tag *) 0x41070000UL)
#define ERM_0 (*(volatile struct ERM_tag *) 0x40070000UL)
#define ERM_4 (*(volatile struct ERM_tag *) 0x42070000UL)
#define ERM_5 (*(volatile struct ERM_tag *) 0x42870000UL)
#define ERM_GTM (*(volatile struct ERM_GTM_tag *) 0x40374000UL)
#define CEVA_SPF2__ERROR_CORRECTION_CODES (*(volatile struct ERROR_CORRECTION_CODES_tag *) 0x24400780UL)
#define AES__FEED_DMACRC (*(volatile struct FEED_DMACRC_tag *) 0x47230000UL)
#define AES__FEED_DMA_MP (*(volatile struct FEED_DMA_MP_tag *) 0x47200000UL)
#define AES__FEED_DMA_TCD (*(volatile struct FEED_DMA_TCD_tag *) 0x47210000UL)
#define CE_CAN_0 (*(volatile struct FLEXCAN_tag *) 0x449A0000UL)
#define CE_CAN_1 (*(volatile struct FLEXCAN_tag *) 0x449B0000UL)
#define CE_CAN_10 (*(volatile struct FLEXCAN_tag *) 0x44BE0000UL)
#define CE_CAN_11 (*(volatile struct FLEXCAN_tag *) 0x44BF0000UL)
#define CE_CAN_12 (*(volatile struct FLEXCAN_tag *) 0x44DA0000UL)
#define CE_CAN_13 (*(volatile struct FLEXCAN_tag *) 0x44DB0000UL)
#define CE_CAN_14 (*(volatile struct FLEXCAN_tag *) 0x44DC0000UL)
#define CE_CAN_15 (*(volatile struct FLEXCAN_tag *) 0x44DD0000UL)
#define CE_CAN_16 (*(volatile struct FLEXCAN_tag *) 0x44DE0000UL)
#define CE_CAN_17 (*(volatile struct FLEXCAN_tag *) 0x44DF0000UL)
#define CE_CAN_18 (*(volatile struct FLEXCAN_tag *) 0x44FA0000UL)
#define CE_CAN_19 (*(volatile struct FLEXCAN_tag *) 0x44FB0000UL)
#define CE_CAN_2 (*(volatile struct FLEXCAN_tag *) 0x449C0000UL)
#define CE_CAN_20 (*(volatile struct FLEXCAN_tag *) 0x44FC0000UL)
#define CE_CAN_21 (*(volatile struct FLEXCAN_tag *) 0x44FD0000UL)
#define CE_CAN_22 (*(volatile struct FLEXCAN_tag *) 0x44FE0000UL)
#define CE_CAN_23 (*(volatile struct FLEXCAN_tag *) 0x44FF0000UL)
#define CE_CAN_3 (*(volatile struct FLEXCAN_tag *) 0x449D0000UL)
#define CE_CAN_4 (*(volatile struct FLEXCAN_tag *) 0x449E0000UL)
#define CE_CAN_5 (*(volatile struct FLEXCAN_tag *) 0x449F0000UL)
#define CE_CAN_6 (*(volatile struct FLEXCAN_tag *) 0x44BA0000UL)
#define CE_CAN_7 (*(volatile struct FLEXCAN_tag *) 0x44BB0000UL)
#define CE_CAN_8 (*(volatile struct FLEXCAN_tag *) 0x44BC0000UL)
#define CE_CAN_9 (*(volatile struct FLEXCAN_tag *) 0x44BD0000UL)
#define FR_0 (*(volatile struct FLEXRAY_tag *) 0x401B0000UL)
#define FR_1 (*(volatile struct FLEXRAY_tag *) 0x401C0000UL)
#define FXOSC (*(volatile struct FXOSC_tag *) 0x40200000UL)
#define GPR0 (*(volatile struct GPR0_tag *) 0x40060000UL)
#define GPR0_PCTL (*(volatile struct GPR0_PCTL_tag *) 0x40010000UL)
#define GPR1 (*(volatile struct GPR1_tag *) 0x40860000UL)
#define GPR1_PCTL (*(volatile struct GPR1_PCTL_tag *) 0x40810000UL)
#define GPR2 (*(volatile struct GPR2_tag *) 0x41060000UL)
#define GPR3 (*(volatile struct GPR3_tag *) 0x41860000UL)
#define GPR3_PCTL (*(volatile struct GPR3_PCTL_tag *) 0x41810000UL)
#define GPR4 (*(volatile struct GPR4_tag *) 0x42060000UL)
#define GPR4_PCTL (*(volatile struct GPR4_PCTL_tag *) 0x42010000UL)
#define GPR5 (*(volatile struct GPR5_tag *) 0x42860000UL)
#define GPR5_PCTL (*(volatile struct GPR5_PCTL_tag *) 0x42810000UL)
#define GPR6 (*(volatile struct GPR6_tag *) 0x44060000UL)
#define GPR6_PCTL (*(volatile struct GPR6_PCTL_tag *) 0x44010000UL)
#define GTMDI (*(volatile struct GTMDI_tag *) 0x4D480000UL)
#define GTMSS (*(volatile struct GTMSS_tag *) 0x4037C000UL)
#define GTM_gtm_cls0 (*(volatile struct GTM_gtm_cls0_tag *) 0x73000000UL)
#define GTM_gtm_cls1 (*(volatile struct GTM_gtm_cls1_tag *) 0x73020000UL)
#define GTM_gtm_cls2 (*(volatile struct GTM_gtm_cls2_tag *) 0x73040000UL)
#define GTM_gtm_cls3 (*(volatile struct GTM_gtm_cls3_tag *) 0x73060000UL)
#define I3C_0 (*(volatile struct I3C_tag *) 0x401D0000UL)
#define CEVA_SPF2__INTERFACE_CONFIGURATION (*(volatile struct INTERFACE_CONFIGURATION_tag *) 0x24400900UL)
#define CEVA_SPF2__INTERRUPT_CONTROL (*(volatile struct INTERRUPT_CONTROL_tag *) 0x24402100UL)
#define LCU_1 (*(volatile struct LCU_tag *) 0x400A0000UL)
#define LCU_0 (*(volatile struct LCU_tag *) 0x420A0000UL)
#define LFAST_0 (*(volatile struct LFAST_tag *) 0x40A10000UL)
#define LFAST_1 (*(volatile struct LFAST_tag *) 0x40A30000UL)
#define LINFLEX_3 (*(volatile struct LINFLEXD_tag *) 0x40970000UL)
#define LINFLEX_4 (*(volatile struct LINFLEXD_tag *) 0x40980000UL)
#define LINFLEX_5 (*(volatile struct LINFLEXD_tag *) 0x40990000UL)
#define LINFLEX_0 (*(volatile struct LINFLEXD_tag *) 0x40170000UL)
#define LINFLEX_1 (*(volatile struct LINFLEXD_tag *) 0x40180000UL)
#define LINFLEX_2 (*(volatile struct LINFLEXD_tag *) 0x40190000UL)
#define MSC_0_LIN (*(volatile struct LINFLEXD_tag *) 0x40330000UL)
#define LINFLEX_6 (*(volatile struct LINFLEXD_tag *) 0x42170000UL)
#define LINFLEX_7 (*(volatile struct LINFLEXD_tag *) 0x42180000UL)
#define LINFLEX_8 (*(volatile struct LINFLEXD_tag *) 0x42190000UL)
#define LINFLEX_10 (*(volatile struct LINFLEXD_tag *) 0x42990000UL)
#define LINFLEX_11 (*(volatile struct LINFLEXD_tag *) 0x429A0000UL)
#define LINFLEX_9 (*(volatile struct LINFLEXD_tag *) 0x42980000UL)
#define RTU0__LLC_CSR (*(volatile struct LLC_CSR_tag *) 0x76050000UL)
#define RTU1__LLC_CSR (*(volatile struct LLC_CSR_tag *) 0x76850000UL)
#define RTU0__LLC_FSC (*(volatile struct LLC_FSC_tag *) 0x76040000UL)
#define RTU1__LLC_FSC (*(volatile struct LLC_FSC_tag *) 0x76840000UL)
#define CE_LMEM64 (*(volatile struct LMEM64_tag *) 0xE0082000UL)
#define SMU__LMEM64 (*(volatile struct LMEM64_tag *) 0xE0082000UL)
#define LPI2C_1 (*(volatile struct LPI2C_tag *) 0x409D0000UL)
#define LPI2C_2 (*(volatile struct LPI2C_tag *) 0x421D0000UL)
#define LSTCU_6 (*(volatile struct LSTCU_tag *) 0x4D008000UL)
#define LSTCU_1 (*(volatile struct LSTCU_tag *) 0x4D208000UL)
#define LSTCU_11 (*(volatile struct LSTCU_tag *) 0x4D10A000UL)
#define LSTCU_10 (*(volatile struct LSTCU_tag *) 0x4D109000UL)
#define LSTCU_2 (*(volatile struct LSTCU_tag *) 0x4D108000UL)
#define LSTCU_9 (*(volatile struct LSTCU_tag *) 0x4D409000UL)
#define LSTCU_0 (*(volatile struct LSTCU_tag *) 0x4D408000UL)
#define LSTCU_12 (*(volatile struct LSTCU_tag *) 0x4D309000UL)
#define LSTCU_4 (*(volatile struct LSTCU_tag *) 0x4D308000UL)
#define LSTCU_5 (*(volatile struct LSTCU_tag *) 0x4D388000UL)
#define LSTCU_14 (*(volatile struct LSTCU_14_15_17_18_tag *) 0x4C009000UL)
#define LSTCU_17 (*(volatile struct LSTCU_14_15_17_18_tag *) 0x4C809000UL)
#define LSTCU_15 (*(volatile struct LSTCU_14_15_17_18_tag *) 0x4C00A000UL)
#define LSTCU_18 (*(volatile struct LSTCU_14_15_17_18_tag *) 0x4C80A000UL)
#define LSTCU_19 (*(volatile struct LSTCU_19_tag *) 0x4D389000UL)
#define LSTCU_3 (*(volatile struct LSTCU_3_7_tag *) 0x4D288000UL)
#define LSTCU_7 (*(volatile struct LSTCU_3_7_tag *) 0x4D209000UL)
#define LSTCU_8 (*(volatile struct LSTCU_8_tag *) 0x4D289000UL)
#define L_VFCCU_3 (*(volatile struct L_VFCCU_tag *) 0x41C10000UL)
#define L_VFCCU_6 (*(volatile struct L_VFCCU_tag *) 0x440E0000UL)
#define L_VFCCU_1 (*(volatile struct L_VFCCU_tag *) 0x40C10000UL)
#define L_VFCCU_2 (*(volatile struct L_VFCCU_tag *) 0x410E0000UL)
#define L_VFCCU_0 (*(volatile struct L_VFCCU_tag *) 0x40410000UL)
#define L_VFCCU_4 (*(volatile struct L_VFCCU_tag *) 0x42410000UL)
#define L_VFCCU_5 (*(volatile struct L_VFCCU_tag *) 0x42C10000UL)
#define CE_MCM (*(volatile struct MCM_tag *) 0xE0080000UL)
#define SMU__MCM (*(volatile struct MCM_tag *) 0xE0080000UL)
#define MC_CGM_0 (*(volatile struct MC_CGM_0_tag *) 0x40030000UL)
#define MC_CGM_1 (*(volatile struct MC_CGM_1_tag *) 0x40830000UL)
#define MC_CGM_2 (*(volatile struct MC_CGM_2_tag *) 0x41030000UL)
#define MC_CGM_3 (*(volatile struct MC_CGM_3_tag *) 0x41830000UL)
#define MC_CGM_4 (*(volatile struct MC_CGM_4_tag *) 0x42030000UL)
#define MC_CGM_5 (*(volatile struct MC_CGM_5_tag *) 0x42830000UL)
#define MC_CGM_6 (*(volatile struct MC_CGM_6_tag *) 0x44030000UL)
#define MC_ME (*(volatile struct MC_ME_tag *) 0x41900000UL)
#define MC_RGM (*(volatile struct MC_RGM_tag *) 0x41850000UL)
#define MDM_AP (*(volatile struct MDM_AP_tag *) 0x4DC11000UL)
#define MEW_0_AXI (*(volatile struct MEW_tag *) 0x42380000UL)
#define MEW_1_AXI (*(volatile struct MEW_tag *) 0x42390000UL)
#define CE_MRU_0 (*(volatile struct MRU_tag *) 0x44AEC000UL)
#define CE_MRU_1 (*(volatile struct MRU_tag *) 0x44B0C000UL)
#define CE_MRU_10 (*(volatile struct MRU_tag *) 0x44CCC000UL)
#define CE_MRU_11 (*(volatile struct MRU_tag *) 0x44CFC000UL)
#define CE_MRU_12 (*(volatile struct MRU_tag *) 0x448EC000UL)
#define CE_MRU_13 (*(volatile struct MRU_tag *) 0x4490C000UL)
#define CE_MRU_14 (*(volatile struct MRU_tag *) 0x4492C000UL)
#define CE_MRU_15 (*(volatile struct MRU_tag *) 0x4494C000UL)
#define CE_MRU_16 (*(volatile struct MRU_tag *) 0x4496C000UL)
#define CE_MRU_17 (*(volatile struct MRU_tag *) 0x4498C000UL)
#define CE_MRU_18 (*(volatile struct MRU_tag *) 0x44D2C000UL)
#define CE_MRU_19 (*(volatile struct MRU_tag *) 0x44D4C000UL)
#define CE_MRU_2 (*(volatile struct MRU_tag *) 0x44B2C000UL)
#define CE_MRU_3 (*(volatile struct MRU_tag *) 0x44B4C000UL)
#define CE_MRU_4 (*(volatile struct MRU_tag *) 0x44B6C000UL)
#define CE_MRU_5 (*(volatile struct MRU_tag *) 0x44B8C000UL)
#define CE_MRU_6 (*(volatile struct MRU_tag *) 0x44C0C000UL)
#define CE_MRU_7 (*(volatile struct MRU_tag *) 0x44C3C000UL)
#define CE_MRU_8 (*(volatile struct MRU_tag *) 0x44C6C000UL)
#define CE_MRU_9 (*(volatile struct MRU_tag *) 0x44C9C000UL)
#define MSCM (*(volatile struct MSCM_tag *) 0x419A0000UL)
#define MU0__MUA (*(volatile struct MU_tag *) 0x23258000UL)
#define MU1__MUA (*(volatile struct MU_tag *) 0x23259000UL)
#define MU2__MUA (*(volatile struct MU_tag *) 0x2325A000UL)
#define MU3__MUA (*(volatile struct MU_tag *) 0x2325B000UL)
#define MU4__MUA (*(volatile struct MU_tag *) 0x2325C000UL)
#define MU5__MUA (*(volatile struct MU_tag *) 0x2325D000UL)
#define MU6__MUA (*(volatile struct MU_tag *) 0x2325E000UL)
#define MU7__MUA (*(volatile struct MU_tag *) 0x2325F000UL)
#define MU0__MUB (*(volatile struct MU_tag *) 0x42200000UL)
#define MU1__MUB (*(volatile struct MU_tag *) 0x42210000UL)
#define MU2__MUB (*(volatile struct MU_tag *) 0x42220000UL)
#define MU3__MUB (*(volatile struct MU_tag *) 0x42230000UL)
#define MU4__MUB (*(volatile struct MU_tag *) 0x42240000UL)
#define MU5__MUB (*(volatile struct MU_tag *) 0x42250000UL)
#define MU6__MUB (*(volatile struct MU_tag *) 0x42260000UL)
#define MU7__MUB (*(volatile struct MU_tag *) 0x42270000UL)
#define CEVA_SPF2__MULTICORE_CONFIGURATION (*(volatile struct MULTICORE_CONFIGURATION_tag *) 0x24400000UL)
#define NETC__TMR0_GLOBAL (*(volatile struct NETC_F0_GLOBAL_tag *) 0x74B50000UL)
#define NETC__NETC_F0_PCI_HDR_TYPE0 (*(volatile struct NETC_F0_PCI_HDR_TYPE0_tag *) 0x74000000UL)
#define NETC__EMDIO_BASE (*(volatile struct NETC_F1_tag *) 0x74B60000UL)
#define NETC__EMDIO_GLOBAL (*(volatile struct NETC_F1_GLOBAL_tag *) 0x74B70000UL)
#define NETC__NETC_F1_PCI_HDR_TYPE0 (*(volatile struct NETC_F1_PCI_HDR_TYPE0_tag *) 0x74001000UL)
#define NETC__SW0_BASE (*(volatile struct NETC_F2_tag *) 0x74A00000UL)
#define NETC__SW0_COMMON (*(volatile struct NETC_F2_COMMON_tag *) 0x74A00000UL)
#define NETC__SW0_GLOBAL (*(volatile struct NETC_F2_GLOBAL_tag *) 0x74A80000UL)
#define NETC__NETC_F2_PCI_HDR_TYPE0 (*(volatile struct NETC_F2_PCI_HDR_TYPE0_tag *) 0x74002000UL)
#define NETC__ENETC0_BASE (*(volatile struct NETC_F3_tag *) 0x74B10000UL)
#define NETC__ENETC0_COMMON (*(volatile struct NETC_F3_COMMON_tag *) 0x74B10000UL)
#define NETC__ENETC0_GLOBAL (*(volatile struct NETC_F3_GLOBAL_tag *) 0x74B20000UL)
#define NETC__NETC_F3_PCI_HDR_TYPE0 (*(volatile struct NETC_F3_PCI_HDR_TYPE0_tag *) 0x74003000UL)
#define NETC__ENETC0_SI0 (*(volatile struct NETC_F3_SI0_tag *) 0x74B00000UL)
#define NETC__ENETC0_SI1 (*(volatile struct NETC_F3_SI1_tag *) 0x74BC0000UL)
#define NETC__ENETC0_SI2 (*(volatile struct NETC_F3_SI2_tag *) 0x74BD0000UL)
#define NETC__ENETC0_SI3 (*(volatile struct NETC_F3_SI3_tag *) 0x74BE0000UL)
#define NETC__ENETC0_SI4 (*(volatile struct NETC_F3_SI4_tag *) 0x74BF0000UL)
#define NETC__ENETC0_SI5 (*(volatile struct NETC_F3_SI5_tag *) 0x74C00000UL)
#define NETC__ENETC0_SI6 (*(volatile struct NETC_F3_SI6_tag *) 0x74C10000UL)
#define NETC__ENETC0_SI7 (*(volatile struct NETC_F3_SI7_tag *) 0x74C20000UL)
#define NETC__NETC_IERB (*(volatile struct NETC_IERB_tag *) 0x74800000UL)
#define NETC__NETC_PRIV (*(volatile struct NETC_PRIV_tag *) 0x74900000UL)
#define NETC__NETC_VF1_PCI_HDR_TYPE0 (*(volatile struct NETC_VF1_PCI_HDR_TYPE0_tag *) 0x74100000UL)
#define NETC__NETC_VF2_PCI_HDR_TYPE0 (*(volatile struct NETC_VF2_PCI_HDR_TYPE0_tag *) 0x74110000UL)
#define NETC__NETC_VF3_PCI_HDR_TYPE0 (*(volatile struct NETC_VF3_PCI_HDR_TYPE0_tag *) 0x74120000UL)
#define NETC__NETC_VF4_PCI_HDR_TYPE0 (*(volatile struct NETC_VF4_PCI_HDR_TYPE0_tag *) 0x74130000UL)
#define NETC__NETC_VF5_PCI_HDR_TYPE0 (*(volatile struct NETC_VF5_PCI_HDR_TYPE0_tag *) 0x74140000UL)
#define NETC__NETC_VF6_PCI_HDR_TYPE0 (*(volatile struct NETC_VF6_PCI_HDR_TYPE0_tag *) 0x74150000UL)
#define NETC__NETC_VF7_PCI_HDR_TYPE0 (*(volatile struct NETC_VF7_PCI_HDR_TYPE0_tag *) 0x74160000UL)
#define OCOTP (*(volatile struct OCOTP_tag *) 0x423B0000UL)
#define RTU0__OMU (*(volatile struct OMU_tag *) 0x761A0000UL)
#define RTU1__OMU (*(volatile struct OMU_tag *) 0x769A0000UL)
#define PERIPH_PLL (*(volatile struct PERIPH_PLL_tag *) 0x40220000UL)
#define CE_PIT_0 (*(volatile struct PIT_tag *) 0x44A80000UL)
#define CE_PIT_1 (*(volatile struct PIT_tag *) 0x44A90000UL)
#define CE_PIT_2 (*(volatile struct PIT_tag *) 0x44AA0000UL)
#define CE_PIT_3 (*(volatile struct PIT_tag *) 0x44AB0000UL)
#define CE_PIT_4 (*(volatile struct PIT_tag *) 0x44AC0000UL)
#define CE_PIT_5 (*(volatile struct PIT_tag *) 0x44AD0000UL)
#define PIT_1 (*(volatile struct PIT_tag *) 0x40920000UL)
#define PIT_0 (*(volatile struct PIT_tag *) 0x40120000UL)
#define PIT_4 (*(volatile struct PIT_tag *) 0x42120000UL)
#define RTU0__PIT (*(volatile struct PIT_tag *) 0x76150000UL)
#define RTU1__PIT (*(volatile struct PIT_tag *) 0x76950000UL)
#define PIT_5 (*(volatile struct PIT_tag *) 0x42920000UL)
#define PMC (*(volatile struct PMC_tag *) 0x40250000UL)
#define CEVA_SPF2__PMSS_SAFETY (*(volatile struct PMSS_SAFETY_tag *) 0x24400580UL)
#define CEVA_SPF2__POWER_SCALING_UNIT (*(volatile struct POWER_SCALING_UNIT_tag *) 0x24400E50UL)
#define CEVA_SPF2__PROFILER (*(volatile struct PROFILER_tag *) 0x24400300UL)
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM (*(volatile struct PROGRAM_MEMORY_SUBSYSTEM_tag *) 0x24400400UL)
#define PSI5_0 (*(volatile struct PSI5_tag *) 0x401E0000UL)
#define PSI5_1 (*(volatile struct PSI5_tag *) 0x421E0000UL)
#define PSI5_S_0 (*(volatile struct PSI5_S_tag *) 0x401F0000UL)
#define PSI5_S_1 (*(volatile struct PSI5_S_tag *) 0x421F0000UL)
#define CEVA_SPF2__QMAN_CNTRL (*(volatile struct QMAN_CNTRL_tag *) 0x24401180UL)
#define CEVA_SPF2__QUEUE_DESCRIPTOR (*(volatile struct QUEUE_DESCRIPTOR_tag *) 0x24401100UL)
#define CEVA_SPF2__QUEUE_MANAGER (*(volatile struct QUEUE_MANAGER_tag *) 0x24401200UL)
#define CEVA_SPF2__QUEUE_MANAGER2 (*(volatile struct QUEUE_MANAGER2_tag *) 0x24401300UL)
#define CEVA_SPF2__QUEUE_MANAGER3 (*(volatile struct QUEUE_MANAGER3_tag *) 0x24401380UL)
#define QUADSPI_0 (*(volatile struct QuadSPI_tag *) 0x42320000UL)
#define QUADSPI_1 (*(volatile struct QuadSPI_tag *) 0x42340000UL)
#define QUADSPI_0_ARDB (*(volatile struct QuadSPI_ARDB_tag *) 0x70000000UL)
#define QUADSPI_1_ARDB (*(volatile struct QuadSPI_ARDB_tag *) 0x71000000UL)
#define CEVA_SPF2__QUEUE_MANAGER1 (*(volatile struct Queue_Manager1_tag *) 0x24401280UL)
#define RDC_0 (*(volatile struct RDC_tag *) 0x41890000UL)
#define RDC_1 (*(volatile struct RDC_tag *) 0x418A0000UL)
#define AES__RESULT_DMACRC (*(volatile struct RESULT_DMACRC_tag *) 0x47280000UL)
#define AES__RESULT_DMA_MP (*(volatile struct RESULT_DMA_MP_tag *) 0x47250000UL)
#define AES__RESULT_DMA_TCD (*(volatile struct RESULT_DMA_TCD_tag *) 0x47260000UL)
#define CEVA_SPF2__ROUND_ROBIN_ARBITER (*(volatile struct ROUND_ROBIN_ARBITER_tag *) 0x24400FA0UL)
#define CEVA_SPF2__RTT (*(volatile struct RTT_tag *) 0x24400180UL)
#define RTU0__RTUE_NIC_D (*(volatile struct RTUE_NIC_D_tag *) 0x75300000UL)
#define RTU1__RTUE_NIC_D (*(volatile struct RTUE_NIC_D_tag *) 0x75700000UL)
#define RTU0__RTUF_NIC_D (*(volatile struct RTUF_NIC_D_tag *) 0x75100000UL)
#define RTU1__RTUF_NIC_D (*(volatile struct RTUF_NIC_D_tag *) 0x75500000UL)
#define RTU0__RTUM_NIC_D (*(volatile struct RTUM_NIC_D_tag *) 0x75000000UL)
#define RTU1__RTUM_NIC_D (*(volatile struct RTUM_NIC_D_tag *) 0x75400000UL)
#define RTU0__RTUP_NIC_B (*(volatile struct RTUP_NIC_B_tag *) 0x75200000UL)
#define RTU1__RTUP_NIC_B (*(volatile struct RTUP_NIC_B_tag *) 0x75600000UL)
#define RTU0__EIM (*(volatile struct RTU_EIM_tag *) 0x76180000UL)
#define RTU1__EIM (*(volatile struct RTU_EIM_tag *) 0x76980000UL)
#define RTU0__GPR (*(volatile struct RTU_GPR_tag *) 0x76120000UL)
#define RTU1__GPR (*(volatile struct RTU_GPR_tag *) 0x76920000UL)
#define RTU0__L_VFCCU (*(volatile struct RTU_L_VFCCU_tag *) 0x76130000UL)
#define RTU1__L_VFCCU (*(volatile struct RTU_L_VFCCU_tag *) 0x76930000UL)
#define RTU0__MC_CGM (*(volatile struct RTU_MC_CGM_tag *) 0x762C0000UL)
#define RTU1__MC_CGM (*(volatile struct RTU_MC_CGM_tag *) 0x76AC0000UL)
#define RTU0__MRU_0 (*(volatile struct RTU_MRU_tag *) 0x76070000UL)
#define RTU1__MRU_0 (*(volatile struct RTU_MRU_tag *) 0x76870000UL)
#define RTU0__MRU_1 (*(volatile struct RTU_MRU_tag *) 0x76090000UL)
#define RTU1__MRU_1 (*(volatile struct RTU_MRU_tag *) 0x76890000UL)
#define RTU0__MRU_2 (*(volatile struct RTU_MRU_tag *) 0x76270000UL)
#define RTU1__MRU_2 (*(volatile struct RTU_MRU_tag *) 0x76A70000UL)
#define RTU0__MRU_3 (*(volatile struct RTU_MRU_tag *) 0x76290000UL)
#define RTU1__MRU_3 (*(volatile struct RTU_MRU_tag *) 0x76A90000UL)
#define RTU0__PMC (*(volatile struct RTU_PMC_tag *) 0x761B0000UL)
#define RTU1__PMC (*(volatile struct RTU_PMC_tag *) 0x769B0000UL)
#define RTU0__SEMA42 (*(volatile struct RTU_SEMA42_tag *) 0x76110000UL)
#define RTU1__SEMA42 (*(volatile struct RTU_SEMA42_tag *) 0x76910000UL)
#define RTU0__XRDC (*(volatile struct RTU_XRDC_tag *) 0x76100000UL)
#define RTU1__XRDC (*(volatile struct RTU_XRDC_tag *) 0x76900000UL)
#define CE_RXLUT (*(volatile struct RXLUT_tag *) 0x4488C000UL)
#define SMU__SBSW (*(volatile struct SBSW_tag *) 0x45290000UL)
#define SDA_AP (*(volatile struct SDA_AP_tag *) 0x4DC71000UL)
#define SEC (*(volatile struct SEC_S250_tag *) 0x42280000UL)
#define SEMA42_3 (*(volatile struct SEMA42_tag *) 0x418D0000UL)
#define SEMA42_1 (*(volatile struct SEMA42_tag *) 0x408D0000UL)
#define SEMA42_2 (*(volatile struct SEMA42_tag *) 0x410D0000UL)
#define SEMA42_0 (*(volatile struct SEMA42_tag *) 0x400D0000UL)
#define SEMA42_4 (*(volatile struct SEMA42_tag *) 0x420D0000UL)
#define SEMA42_5 (*(volatile struct SEMA42_tag *) 0x428D0000UL)
#define SINC (*(volatile struct SINC_tag *) 0x403A0000UL)
#define SIPI_0 (*(volatile struct SIPI_tag *) 0x40A20000UL)
#define SIPI_1 (*(volatile struct SIPI_tag *) 0x40A40000UL)
#define SIUL2_0 (*(volatile struct SIUL2_0_tag *) 0x40520000UL)
#define SIUL2_1 (*(volatile struct SIUL2_1_tag *) 0x40D20000UL)
#define SIUL2_3 (*(volatile struct SIUL2_3_tag *) 0x41D20000UL)
#define SIUL2_4 (*(volatile struct SIUL2_4_tag *) 0x42520000UL)
#define SIUL2_5 (*(volatile struct SIUL2_5_tag *) 0x42D20000UL)
#define SMU__L_VFCCU (*(volatile struct SMU_L_VFCCU_tag *) 0x45040000UL)
#define SMU__MRU (*(volatile struct SMU_MRU_tag *) 0x45300000UL)
#define SMU__SEMA42 (*(volatile struct SMU_SEMA42_tag *) 0x45230000UL)
#define SMU__SRG_S (*(volatile struct SMU_SRG_S_tag *) 0x45064000UL)
#define SMU__XRDC (*(volatile struct SMU_XRDC_tag *) 0x45000000UL)
#define RTU0__SPFU (*(volatile struct SPFU_tag *) 0x76060000UL)
#define RTU1__SPFU (*(volatile struct SPFU_tag *) 0x76860000UL)
#define SPI_2 (*(volatile struct SPI_tag *) 0x40930000UL)
#define SPI_3 (*(volatile struct SPI_tag *) 0x40940000UL)
#define SPI_4 (*(volatile struct SPI_tag *) 0x40950000UL)
#define SPI_0 (*(volatile struct SPI_tag *) 0x40130000UL)
#define SPI_1 (*(volatile struct SPI_tag *) 0x40140000UL)
#define SPI_5 (*(volatile struct SPI_tag *) 0x42130000UL)
#define SPI_6 (*(volatile struct SPI_tag *) 0x42140000UL)
#define SPI_7 (*(volatile struct SPI_tag *) 0x42150000UL)
#define SPI_8 (*(volatile struct SPI_tag *) 0x42930000UL)
#define SPI_9 (*(volatile struct SPI_tag *) 0x42940000UL)
#define CE_SRAMCTL_0 (*(volatile struct SRAMCTL_tag *) 0x44A60000UL)
#define CE_SRAMCTL_1 (*(volatile struct SRAMCTL_tag *) 0x44A64000UL)
#define CE_SRAMCTL_2 (*(volatile struct SRAMCTL_tag *) 0x44A68000UL)
#define SMU__SRAMCTL_0 (*(volatile struct SRAMCTL_tag *) 0x45260000UL)
#define SMU__SRAMCTL_1 (*(volatile struct SRAMCTL_tag *) 0x45264000UL)
#define SMU__SRAMCTL_2 (*(volatile struct SRAMCTL_tag *) 0x45268000UL)
#define SMU__SRAMCTL_3 (*(volatile struct SRAMCTL_tag *) 0x4526C000UL)
#define RTU0__SRAMCTL_D0 (*(volatile struct SRAMCTL_tag *) 0x761D0000UL)
#define RTU1__SRAMCTL_D0 (*(volatile struct SRAMCTL_tag *) 0x769D0000UL)
#define RTU0__SRAMCTL_D1 (*(volatile struct SRAMCTL_tag *) 0x761E0000UL)
#define RTU1__SRAMCTL_D1 (*(volatile struct SRAMCTL_tag *) 0x769E0000UL)
#define RTU0__SRAMCTL_D2 (*(volatile struct SRAMCTL_tag *) 0x761F0000UL)
#define RTU1__SRAMCTL_D2 (*(volatile struct SRAMCTL_tag *) 0x769F0000UL)
#define RTU0__SRAMCTL_C0 (*(volatile struct SRAMCTL_tag *) 0x760C0000UL)
#define RTU1__SRAMCTL_C0 (*(volatile struct SRAMCTL_tag *) 0x768C0000UL)
#define RTU0__SRAMCTL_C1 (*(volatile struct SRAMCTL_tag *) 0x760D0000UL)
#define RTU1__SRAMCTL_C1 (*(volatile struct SRAMCTL_tag *) 0x768D0000UL)
#define RTU0__SRAMCTL_C2 (*(volatile struct SRAMCTL_tag *) 0x760E0000UL)
#define RTU1__SRAMCTL_C2 (*(volatile struct SRAMCTL_tag *) 0x768E0000UL)
#define RTU0__SRAMCTL_C3 (*(volatile struct SRAMCTL_tag *) 0x760F0000UL)
#define RTU1__SRAMCTL_C3 (*(volatile struct SRAMCTL_tag *) 0x768F0000UL)
#define RTU0__SRAMCTL_C4 (*(volatile struct SRAMCTL_tag *) 0x76240000UL)
#define RTU1__SRAMCTL_C4 (*(volatile struct SRAMCTL_tag *) 0x76A40000UL)
#define RTU0__SRAMCTL_C5 (*(volatile struct SRAMCTL_tag *) 0x76250000UL)
#define RTU1__SRAMCTL_C5 (*(volatile struct SRAMCTL_tag *) 0x76A50000UL)
#define RTU0__SRAMCTL_C6 (*(volatile struct SRAMCTL_tag *) 0x76260000UL)
#define RTU1__SRAMCTL_C6 (*(volatile struct SRAMCTL_tag *) 0x76A60000UL)
#define SRX_0 (*(volatile struct SRX_tag *) 0x40A50000UL)
#define SRX_1 (*(volatile struct SRX_tag *) 0x42050000UL)
#define CE_STM_0 (*(volatile struct STM_tag *) 0x44820000UL)
#define CE_STM_1 (*(volatile struct STM_tag *) 0x44A20000UL)
#define CE_STM_2 (*(volatile struct STM_tag *) 0x44A40000UL)
#define SMU__STM_0 (*(volatile struct STM_tag *) 0x45020000UL)
#define SMU__STM_2 (*(volatile struct STM_tag *) 0x45240000UL)
#define RTU0__STM_0 (*(volatile struct STM_tag *) 0x76200000UL)
#define RTU1__STM_0 (*(volatile struct STM_tag *) 0x76A00000UL)
#define RTU0__STM_1 (*(volatile struct STM_tag *) 0x76210000UL)
#define RTU1__STM_1 (*(volatile struct STM_tag *) 0x76A10000UL)
#define RTU0__STM_2 (*(volatile struct STM_tag *) 0x76020000UL)
#define RTU1__STM_2 (*(volatile struct STM_tag *) 0x76820000UL)
#define RTU0__STM_3 (*(volatile struct STM_tag *) 0x76030000UL)
#define RTU1__STM_3 (*(volatile struct STM_tag *) 0x76830000UL)
#define CE_SWT_0 (*(volatile struct SWT_tag *) 0x44824000UL)
#define CE_SWT_1 (*(volatile struct SWT_tag *) 0x44A24000UL)
#define SMU__SWT (*(volatile struct SWT_tag *) 0x45024000UL)
#define RTU0__SWT_0 (*(volatile struct SWT_tag *) 0x76000000UL)
#define RTU1__SWT_0 (*(volatile struct SWT_tag *) 0x76800000UL)
#define RTU0__SWT_1 (*(volatile struct SWT_tag *) 0x76010000UL)
#define RTU1__SWT_1 (*(volatile struct SWT_tag *) 0x76810000UL)
#define RTU0__SWT_2 (*(volatile struct SWT_tag *) 0x76220000UL)
#define RTU1__SWT_2 (*(volatile struct SWT_tag *) 0x76A20000UL)
#define RTU0__SWT_3 (*(volatile struct SWT_tag *) 0x76230000UL)
#define RTU1__SWT_3 (*(volatile struct SWT_tag *) 0x76A30000UL)
#define RTU0__SWT_4 (*(volatile struct SWT_tag *) 0x76140000UL)
#define RTU1__SWT_4 (*(volatile struct SWT_tag *) 0x76940000UL)
#define NETC__SW0_ETH_MAC_PORT0 (*(volatile struct SW_ETH_MAC_PORT0_tag *) 0x74A05000UL)
#define NETC__SW0_ETH_MAC_PORT1 (*(volatile struct SW_ETH_MAC_PORT1_tag *) 0x74A09000UL)
#define NETC__SW0_PORT0 (*(volatile struct SW_PORT0_tag *) 0x74A04000UL)
#define NETC__SW0_PORT1 (*(volatile struct SW_PORT1_tag *) 0x74A08000UL)
#define NETC__SW0_PORT2 (*(volatile struct SW_PORT2_tag *) 0x74A0C000UL)
#define NETC__SW0_PSEUDO_MAC_PORT2 (*(volatile struct SW_PSEUDO_MAC_PORT2_tag *) 0x74A0D000UL)
#define CEVA_SPF2__TIMERS (*(volatile struct TIMERS_tag *) 0x24402000UL)
#define NETC__TMR0_BASE (*(volatile struct TMR0_BASE_tag *) 0x74B40000UL)
#define TMU (*(volatile struct TMU_tag *) 0x402B0000UL)
#define TRGMUX_0 (*(volatile struct TRGMUX_0_tag *) 0x42090000UL)
#define TRGMUX_1 (*(volatile struct TRGMUX_1_tag *) 0x40090000UL)
#define TRGMUX_2 (*(volatile struct TRGMUX_2_tag *) 0x40370000UL)
#define TRGMUX_3 (*(volatile struct TRGMUX_3_tag *) 0x40372000UL)
#define USDHC (*(volatile struct USDHC_tag *) 0x42300000UL)
#define VIRT_WRAP_3 (*(volatile struct VIRT_WRAP_tag *) 0x41D60000UL)
#define VIRT_WRAP_1 (*(volatile struct VIRT_WRAP_tag *) 0x40D60000UL)
#define VIRT_WRAP_0 (*(volatile struct VIRT_WRAP_tag *) 0x40560000UL)
#define VIRT_WRAP_4 (*(volatile struct VIRT_WRAP_tag *) 0x42560000UL)
#define VIRT_WRAP_5 (*(volatile struct VIRT_WRAP_tag *) 0x42D60000UL)
#define CEVA_SPF2__WATCHDOG (*(volatile struct WATCHDOG_tag *) 0x24400700UL)
#define CE_XBIC (*(volatile struct XBIC_tag *) 0x44850000UL)
#define AES__FEED_XBIC_AHB (*(volatile struct XBIC_tag *) 0x472C0000UL)
#define AES__RESULT_XBIC_AHB (*(volatile struct XBIC_tag *) 0x472D0000UL)
#define SMU__XBIC (*(volatile struct XBIC_tag *) 0x45050000UL)
#define XRDC_0 (*(volatile struct XRDC_0_tag *) 0x40000000UL)
#define XRDC_1 (*(volatile struct XRDC_1_tag *) 0x40800000UL)
#define XRDC_2 (*(volatile struct XRDC_2_tag *) 0x41000000UL)
#define XRDC_3A (*(volatile struct XRDC_3A_tag *) 0x41800000UL)
#define XRDC_3B (*(volatile struct XRDC_3B_tag *) 0x419D0000UL)
#define XRDC_4 (*(volatile struct XRDC_4_tag *) 0x42000000UL)
#define XRDC_5 (*(volatile struct XRDC_5_tag *) 0x42800000UL)
#define NETC__IERC_IERB (*(volatile struct ierc_ierb_tag *) 0x74810000UL)
#define NETC__IERC_F0_PCI_HDR_TYPE0 (*(volatile struct ierc_pci_tag *) 0x740F8000UL)

/* ACCESS_PROTECTION */
#define CEVA_SPF2__ACCESS_PROTECTION_MAPAR CEVA_SPF2__ACCESS_PROTECTION.MAPAR.R /* Memory Access Protection Address Register */
#define CEVA_SPF2__ACCESS_PROTECTION_D_DTAP CEVA_SPF2__ACCESS_PROTECTION.D_DTAP.R /* IDM Access Configuration */

/* ACE */
#define AES__ACE__AHB_INIT_VEC AES__ACE__AHB.INIT_VEC.R    /* Initial Value */
#define AES__ACE__AHB_DATA_IN AES__ACE__AHB.DATA_IN.R      /* Data Input */

#define AES__ACE__AHB2_STATUS AES__ACE__AHB2.STATUS.R      /* Status */
#define AES__ACE__AHB2_CONTROL AES__ACE__AHB2.CONTROL.R    /* Control */
#define AES__ACE__AHB2_RESULT AES__ACE__AHB2.RESULT.R      /* Result */
#define AES__ACE__AHB2_AUTH_TAG AES__ACE__AHB2.AUTH_TAG.R  /* Auth Tag Output */
#define AES__ACE__AHB2_VERSION AES__ACE__AHB2.VERSION.R    /* ACE Version */

/* ADC */
#define ADC_0_MCR            ADC_0.MCR.R                   /* Main Configuration */
#define ADC_0_MSR            ADC_0.MSR.R                   /* Main Status */
#define ADC_0_ISR            ADC_0.ISR.R                   /* Interrupt Status */
#define ADC_0_CEOCFR0        ADC_0.CEOCFR0.R               /* Channel Pending 0 */
#define ADC_0_CEOCFR1        ADC_0.CEOCFR1.R               /* Channel Pending 1 */
#define ADC_0_IMR            ADC_0.IMR.R                   /* Interrupt Mask */
#define ADC_0_CIMR0          ADC_0.CIMR0.R                 /* Channel Interrupt Mask 0 */
#define ADC_0_CIMR1          ADC_0.CIMR1.R                 /* Channel Interrupt Mask 1 */
#define ADC_0_WTISR          ADC_0.WTISR.R                 /* Watchdog Threshold Interrupt Status */
#define ADC_0_WTIMR          ADC_0.WTIMR.R                 /* Watchdog Threshold Interrupt Mask */
#define ADC_0_DMAE           ADC_0.DMAE.R                  /* DMAE */
#define ADC_0_DMAR0          ADC_0.DMAR0.R                 /* DMA 0 */
#define ADC_0_DMAR1          ADC_0.DMAR1.R                 /* DMA 1 */
#define ADC_0_THRHLR0        ADC_0.THRHLR0.R               /* Analog Watchdog Threshold 0 */
#define ADC_0_THRHLR1        ADC_0.THRHLR1.R               /* Analog Watchdog Threshold 1 */
#define ADC_0_THRHLR2        ADC_0.THRHLR2.R               /* Analog Watchdog Threshold 2 */
#define ADC_0_THRHLR3        ADC_0.THRHLR3.R               /* Analog Watchdog Threshold 3 */
#define ADC_0_PSCR           ADC_0.PSCR.R                  /* Presampling Control */
#define ADC_0_PSR0           ADC_0.PSR0.R                  /* Presampling 0 */
#define ADC_0_PSR1           ADC_0.PSR1.R                  /* Presampling 1 */
#define ADC_0_CTR0           ADC_0.CTR0.R                  /* Conversion Timing 0 */
#define ADC_0_CTR1           ADC_0.CTR1.R                  /* Conversion Timing 1 */
#define ADC_0_NCMR0          ADC_0.NCMR0.R                 /* Normal Conversion Mask 0 */
#define ADC_0_NCMR1          ADC_0.NCMR1.R                 /* Normal Conversion Mask 1 */
#define ADC_0_JCMR0          ADC_0.JCMR0.R                 /* Injected Conversion Mask 0 */
#define ADC_0_JCMR1          ADC_0.JCMR1.R                 /* Injected Conversion Mask 1 */
#define ADC_0_USROFSGN       ADC_0.USROFSGN.R              /* User OFFSET and Gain */
#define ADC_0_PDEDR          ADC_0.PDEDR.R                 /* Power Down Exit Delay */
#define ADC_0_PCDR0          ADC_0.PCDR[0].R               /* Precision Channel n Data */
#define ADC_0_PCDR1          ADC_0.PCDR[1].R               /* Precision Channel n Data */
#define ADC_0_PCDR2          ADC_0.PCDR[2].R               /* Precision Channel n Data */
#define ADC_0_PCDR3          ADC_0.PCDR[3].R               /* Precision Channel n Data */
#define ADC_0_PCDR4          ADC_0.PCDR[4].R               /* Precision Channel n Data */
#define ADC_0_PCDR5          ADC_0.PCDR[5].R               /* Precision Channel n Data */
#define ADC_0_PCDR6          ADC_0.PCDR[6].R               /* Precision Channel n Data */
#define ADC_0_PCDR7          ADC_0.PCDR[7].R               /* Precision Channel n Data */
#define ADC_0_ICDR32         ADC_0.ICDR[0].R               /* Internal Channel n Data */
#define ADC_0_ICDR33         ADC_0.ICDR[1].R               /* Internal Channel n Data */
#define ADC_0_ICDR34         ADC_0.ICDR[2].R               /* Internal Channel n Data */
#define ADC_0_ICDR35         ADC_0.ICDR[3].R               /* Internal Channel n Data */
#define ADC_0_ICDR36         ADC_0.ICDR[4].R               /* Internal Channel n Data */
#define ADC_0_ICDR37         ADC_0.ICDR[5].R               /* Internal Channel n Data */
#define ADC_0_ICDR38         ADC_0.ICDR[6].R               /* Internal Channel n Data */
#define ADC_0_ICDR39         ADC_0.ICDR[7].R               /* Internal Channel n Data */
#define ADC_0_THRHLR4        ADC_0.THRHLR4.R               /* Analog Watchdog Threshold 4 */
#define ADC_0_THRHLR5        ADC_0.THRHLR5.R               /* Analog Watchdog Threshold 5 */
#define ADC_0_THRHLR6        ADC_0.THRHLR6.R               /* Analog Watchdog Threshold 6 */
#define ADC_0_THRHLR7        ADC_0.THRHLR7.R               /* Analog Watchdog Threshold 7 */
#define ADC_0_CWSELR0        ADC_0.CWSELR0.R               /* Channel Watchdog Select 0 */
#define ADC_0_CWSELR4        ADC_0.CWSELR4.R               /* Channel Watchdog Select 4 */
#define ADC_0_CWENR0         ADC_0.CWENR0.R                /* Channel Watchdog Enable 0 */
#define ADC_0_CWENR1         ADC_0.CWENR1.R                /* Channel Watchdog Enable 1 */
#define ADC_0_AWORR0         ADC_0.AWORR0.R                /* Analog Watchdog Out of Range 0 */
#define ADC_0_AWORR1         ADC_0.AWORR1.R                /* Analog Watchdog Out of Range 1 */
#define ADC_0_STCR1          ADC_0.STCR1.R                 /* Self-Test Configuration 1 */
#define ADC_0_STCR2          ADC_0.STCR2.R                 /* Self-Test Configuration 2 */
#define ADC_0_STCR3          ADC_0.STCR3.R                 /* Self-Test Configuration 3 */
#define ADC_0_STBRR          ADC_0.STBRR.R                 /* Self-Test Baud Rate */
#define ADC_0_STSR1          ADC_0.STSR1.R                 /* Self-Test Status 1 */
#define ADC_0_STSR2          ADC_0.STSR2.R                 /* Self-Test Status 2 */
#define ADC_0_STSR3          ADC_0.STSR3.R                 /* Self-Test Status 3 */
#define ADC_0_STSR4          ADC_0.STSR4.R                 /* Self-Test Status 4 */
#define ADC_0_STDR1          ADC_0.STDR1.R                 /* Self-Test Data 1 */
#define ADC_0_STDR2          ADC_0.STDR2.R                 /* Self-Test Data 2 */
#define ADC_0_STAW0R         ADC_0.STAW0R.R                /* Self-Test Analog Watchdog 0 */
#define ADC_0_STAW1AR        ADC_0.STAW1AR.R               /* Self-Test Analog Watchdog 1A */
#define ADC_0_STAW1BR        ADC_0.STAW1BR.R               /* Self-Test Analog Watchdog 1B */
#define ADC_0_STAW2R         ADC_0.STAW2R.R                /* Self-Test Analog Watchdog 2 */
#define ADC_0_STAW3R         ADC_0.STAW3R.R                /* Self-Test Analog Watchdog 3 */
#define ADC_0_STAW4R         ADC_0.STAW4R.R                /* Self-Test Analog Watchdog 4 */
#define ADC_0_STAW5R         ADC_0.STAW5R.R                /* Self-Test Analog Watchdog 5 */
#define ADC_0_CALSTAT        ADC_0.CALSTAT.R               /* Calibration Status */

#define ADC_1_MCR            ADC_1.MCR.R                   /* Main Configuration */
#define ADC_1_MSR            ADC_1.MSR.R                   /* Main Status */
#define ADC_1_ISR            ADC_1.ISR.R                   /* Interrupt Status */
#define ADC_1_CEOCFR0        ADC_1.CEOCFR0.R               /* Channel Pending 0 */
#define ADC_1_CEOCFR1        ADC_1.CEOCFR1.R               /* Channel Pending 1 */
#define ADC_1_IMR            ADC_1.IMR.R                   /* Interrupt Mask */
#define ADC_1_CIMR0          ADC_1.CIMR0.R                 /* Channel Interrupt Mask 0 */
#define ADC_1_CIMR1          ADC_1.CIMR1.R                 /* Channel Interrupt Mask 1 */
#define ADC_1_WTISR          ADC_1.WTISR.R                 /* Watchdog Threshold Interrupt Status */
#define ADC_1_WTIMR          ADC_1.WTIMR.R                 /* Watchdog Threshold Interrupt Mask */
#define ADC_1_DMAE           ADC_1.DMAE.R                  /* DMAE */
#define ADC_1_DMAR0          ADC_1.DMAR0.R                 /* DMA 0 */
#define ADC_1_DMAR1          ADC_1.DMAR1.R                 /* DMA 1 */
#define ADC_1_THRHLR0        ADC_1.THRHLR0.R               /* Analog Watchdog Threshold 0 */
#define ADC_1_THRHLR1        ADC_1.THRHLR1.R               /* Analog Watchdog Threshold 1 */
#define ADC_1_THRHLR2        ADC_1.THRHLR2.R               /* Analog Watchdog Threshold 2 */
#define ADC_1_THRHLR3        ADC_1.THRHLR3.R               /* Analog Watchdog Threshold 3 */
#define ADC_1_PSCR           ADC_1.PSCR.R                  /* Presampling Control */
#define ADC_1_PSR0           ADC_1.PSR0.R                  /* Presampling 0 */
#define ADC_1_PSR1           ADC_1.PSR1.R                  /* Presampling 1 */
#define ADC_1_CTR0           ADC_1.CTR0.R                  /* Conversion Timing 0 */
#define ADC_1_CTR1           ADC_1.CTR1.R                  /* Conversion Timing 1 */
#define ADC_1_NCMR0          ADC_1.NCMR0.R                 /* Normal Conversion Mask 0 */
#define ADC_1_NCMR1          ADC_1.NCMR1.R                 /* Normal Conversion Mask 1 */
#define ADC_1_JCMR0          ADC_1.JCMR0.R                 /* Injected Conversion Mask 0 */
#define ADC_1_JCMR1          ADC_1.JCMR1.R                 /* Injected Conversion Mask 1 */
#define ADC_1_USROFSGN       ADC_1.USROFSGN.R              /* User OFFSET and Gain */
#define ADC_1_PDEDR          ADC_1.PDEDR.R                 /* Power Down Exit Delay */
#define ADC_1_PCDR0          ADC_1.PCDR[0].R               /* Precision Channel n Data */
#define ADC_1_PCDR1          ADC_1.PCDR[1].R               /* Precision Channel n Data */
#define ADC_1_PCDR2          ADC_1.PCDR[2].R               /* Precision Channel n Data */
#define ADC_1_PCDR3          ADC_1.PCDR[3].R               /* Precision Channel n Data */
#define ADC_1_PCDR4          ADC_1.PCDR[4].R               /* Precision Channel n Data */
#define ADC_1_PCDR5          ADC_1.PCDR[5].R               /* Precision Channel n Data */
#define ADC_1_PCDR6          ADC_1.PCDR[6].R               /* Precision Channel n Data */
#define ADC_1_PCDR7          ADC_1.PCDR[7].R               /* Precision Channel n Data */
#define ADC_1_ICDR32         ADC_1.ICDR[0].R               /* Internal Channel n Data */
#define ADC_1_ICDR33         ADC_1.ICDR[1].R               /* Internal Channel n Data */
#define ADC_1_ICDR34         ADC_1.ICDR[2].R               /* Internal Channel n Data */
#define ADC_1_ICDR35         ADC_1.ICDR[3].R               /* Internal Channel n Data */
#define ADC_1_ICDR36         ADC_1.ICDR[4].R               /* Internal Channel n Data */
#define ADC_1_ICDR37         ADC_1.ICDR[5].R               /* Internal Channel n Data */
#define ADC_1_ICDR38         ADC_1.ICDR[6].R               /* Internal Channel n Data */
#define ADC_1_ICDR39         ADC_1.ICDR[7].R               /* Internal Channel n Data */
#define ADC_1_THRHLR4        ADC_1.THRHLR4.R               /* Analog Watchdog Threshold 4 */
#define ADC_1_THRHLR5        ADC_1.THRHLR5.R               /* Analog Watchdog Threshold 5 */
#define ADC_1_THRHLR6        ADC_1.THRHLR6.R               /* Analog Watchdog Threshold 6 */
#define ADC_1_THRHLR7        ADC_1.THRHLR7.R               /* Analog Watchdog Threshold 7 */
#define ADC_1_CWSELR0        ADC_1.CWSELR0.R               /* Channel Watchdog Select 0 */
#define ADC_1_CWSELR4        ADC_1.CWSELR4.R               /* Channel Watchdog Select 4 */
#define ADC_1_CWENR0         ADC_1.CWENR0.R                /* Channel Watchdog Enable 0 */
#define ADC_1_CWENR1         ADC_1.CWENR1.R                /* Channel Watchdog Enable 1 */
#define ADC_1_AWORR0         ADC_1.AWORR0.R                /* Analog Watchdog Out of Range 0 */
#define ADC_1_AWORR1         ADC_1.AWORR1.R                /* Analog Watchdog Out of Range 1 */
#define ADC_1_STCR1          ADC_1.STCR1.R                 /* Self-Test Configuration 1 */
#define ADC_1_STCR2          ADC_1.STCR2.R                 /* Self-Test Configuration 2 */
#define ADC_1_STCR3          ADC_1.STCR3.R                 /* Self-Test Configuration 3 */
#define ADC_1_STBRR          ADC_1.STBRR.R                 /* Self-Test Baud Rate */
#define ADC_1_STSR1          ADC_1.STSR1.R                 /* Self-Test Status 1 */
#define ADC_1_STSR2          ADC_1.STSR2.R                 /* Self-Test Status 2 */
#define ADC_1_STSR3          ADC_1.STSR3.R                 /* Self-Test Status 3 */
#define ADC_1_STSR4          ADC_1.STSR4.R                 /* Self-Test Status 4 */
#define ADC_1_STDR1          ADC_1.STDR1.R                 /* Self-Test Data 1 */
#define ADC_1_STDR2          ADC_1.STDR2.R                 /* Self-Test Data 2 */
#define ADC_1_STAW0R         ADC_1.STAW0R.R                /* Self-Test Analog Watchdog 0 */
#define ADC_1_STAW1AR        ADC_1.STAW1AR.R               /* Self-Test Analog Watchdog 1A */
#define ADC_1_STAW1BR        ADC_1.STAW1BR.R               /* Self-Test Analog Watchdog 1B */
#define ADC_1_STAW2R         ADC_1.STAW2R.R                /* Self-Test Analog Watchdog 2 */
#define ADC_1_STAW3R         ADC_1.STAW3R.R                /* Self-Test Analog Watchdog 3 */
#define ADC_1_STAW4R         ADC_1.STAW4R.R                /* Self-Test Analog Watchdog 4 */
#define ADC_1_STAW5R         ADC_1.STAW5R.R                /* Self-Test Analog Watchdog 5 */
#define ADC_1_CALSTAT        ADC_1.CALSTAT.R               /* Calibration Status */

/* AES_ACCEL */
#define AES__AES_ACCEL_LEN0  AES__AES_ACCEL.ACCEL[0].LEN.R /* LEN */
#define AES__AES_ACCEL_AILEN0 AES__AES_ACCEL.ACCEL[0].AILEN.R /* AAD/IV length */
#define AES__AES_ACCEL_CRYPT0 AES__AES_ACCEL.ACCEL[0].CRYPT.R /* CRYPT */
#define AES__AES_ACCEL_OWNSTAT0 AES__AES_ACCEL.ACCEL[0].OWNSTAT.R /* OWN_Status */
#define AES__AES_ACCEL_TLVAL0 AES__AES_ACCEL.ACCEL[0].TLVAL.R /* Timer Load Value */
#define AES__AES_ACCEL_CVAL0 AES__AES_ACCEL.ACCEL[0].CVAL.R /* Current Timer Value */
#define AES__AES_ACCEL_TCTRL0 AES__AES_ACCEL.ACCEL[0].TCTRL.R /* Timer Control */
#define AES__AES_ACCEL_TFLG0 AES__AES_ACCEL.ACCEL[0].TFLG.R /* Timer Flag */
#define AES__AES_ACCEL_FEEDINTMAP0 AES__AES_ACCEL.ACCEL[0].FEEDINTMAP.R /* FEEDINTMAP */
#define AES__AES_ACCEL_RESULTINTMAP0 AES__AES_ACCEL.ACCEL[0].RESULTINTMAP.R /* RESULTINTMAP */
#define AES__AES_ACCEL_LEN1  AES__AES_ACCEL.ACCEL[1].LEN.R /* LEN */
#define AES__AES_ACCEL_AILEN1 AES__AES_ACCEL.ACCEL[1].AILEN.R /* AAD/IV length */
#define AES__AES_ACCEL_CRYPT1 AES__AES_ACCEL.ACCEL[1].CRYPT.R /* CRYPT */
#define AES__AES_ACCEL_OWNSTAT1 AES__AES_ACCEL.ACCEL[1].OWNSTAT.R /* OWN_Status */
#define AES__AES_ACCEL_TLVAL1 AES__AES_ACCEL.ACCEL[1].TLVAL.R /* Timer Load Value */
#define AES__AES_ACCEL_CVAL1 AES__AES_ACCEL.ACCEL[1].CVAL.R /* Current Timer Value */
#define AES__AES_ACCEL_TCTRL1 AES__AES_ACCEL.ACCEL[1].TCTRL.R /* Timer Control */
#define AES__AES_ACCEL_TFLG1 AES__AES_ACCEL.ACCEL[1].TFLG.R /* Timer Flag */
#define AES__AES_ACCEL_FEEDINTMAP1 AES__AES_ACCEL.ACCEL[1].FEEDINTMAP.R /* FEEDINTMAP */
#define AES__AES_ACCEL_RESULTINTMAP1 AES__AES_ACCEL.ACCEL[1].RESULTINTMAP.R /* RESULTINTMAP */
#define AES__AES_ACCEL_LEN2  AES__AES_ACCEL.ACCEL[2].LEN.R /* LEN */
#define AES__AES_ACCEL_AILEN2 AES__AES_ACCEL.ACCEL[2].AILEN.R /* AAD/IV length */
#define AES__AES_ACCEL_CRYPT2 AES__AES_ACCEL.ACCEL[2].CRYPT.R /* CRYPT */
#define AES__AES_ACCEL_OWNSTAT2 AES__AES_ACCEL.ACCEL[2].OWNSTAT.R /* OWN_Status */
#define AES__AES_ACCEL_TLVAL2 AES__AES_ACCEL.ACCEL[2].TLVAL.R /* Timer Load Value */
#define AES__AES_ACCEL_CVAL2 AES__AES_ACCEL.ACCEL[2].CVAL.R /* Current Timer Value */
#define AES__AES_ACCEL_TCTRL2 AES__AES_ACCEL.ACCEL[2].TCTRL.R /* Timer Control */
#define AES__AES_ACCEL_TFLG2 AES__AES_ACCEL.ACCEL[2].TFLG.R /* Timer Flag */
#define AES__AES_ACCEL_FEEDINTMAP2 AES__AES_ACCEL.ACCEL[2].FEEDINTMAP.R /* FEEDINTMAP */
#define AES__AES_ACCEL_RESULTINTMAP2 AES__AES_ACCEL.ACCEL[2].RESULTINTMAP.R /* RESULTINTMAP */
#define AES__AES_ACCEL_LEN3  AES__AES_ACCEL.ACCEL[3].LEN.R /* LEN */
#define AES__AES_ACCEL_AILEN3 AES__AES_ACCEL.ACCEL[3].AILEN.R /* AAD/IV length */
#define AES__AES_ACCEL_CRYPT3 AES__AES_ACCEL.ACCEL[3].CRYPT.R /* CRYPT */
#define AES__AES_ACCEL_OWNSTAT3 AES__AES_ACCEL.ACCEL[3].OWNSTAT.R /* OWN_Status */
#define AES__AES_ACCEL_TLVAL3 AES__AES_ACCEL.ACCEL[3].TLVAL.R /* Timer Load Value */
#define AES__AES_ACCEL_CVAL3 AES__AES_ACCEL.ACCEL[3].CVAL.R /* Current Timer Value */
#define AES__AES_ACCEL_TCTRL3 AES__AES_ACCEL.ACCEL[3].TCTRL.R /* Timer Control */
#define AES__AES_ACCEL_TFLG3 AES__AES_ACCEL.ACCEL[3].TFLG.R /* Timer Flag */
#define AES__AES_ACCEL_FEEDINTMAP3 AES__AES_ACCEL.ACCEL[3].FEEDINTMAP.R /* FEEDINTMAP */
#define AES__AES_ACCEL_RESULTINTMAP3 AES__AES_ACCEL.ACCEL[3].RESULTINTMAP.R /* RESULTINTMAP */
#define AES__AES_ACCEL_LEN4  AES__AES_ACCEL.ACCEL[4].LEN.R /* LEN */
#define AES__AES_ACCEL_AILEN4 AES__AES_ACCEL.ACCEL[4].AILEN.R /* AAD/IV length */
#define AES__AES_ACCEL_CRYPT4 AES__AES_ACCEL.ACCEL[4].CRYPT.R /* CRYPT */
#define AES__AES_ACCEL_OWNSTAT4 AES__AES_ACCEL.ACCEL[4].OWNSTAT.R /* OWN_Status */
#define AES__AES_ACCEL_TLVAL4 AES__AES_ACCEL.ACCEL[4].TLVAL.R /* Timer Load Value */
#define AES__AES_ACCEL_CVAL4 AES__AES_ACCEL.ACCEL[4].CVAL.R /* Current Timer Value */
#define AES__AES_ACCEL_TCTRL4 AES__AES_ACCEL.ACCEL[4].TCTRL.R /* Timer Control */
#define AES__AES_ACCEL_TFLG4 AES__AES_ACCEL.ACCEL[4].TFLG.R /* Timer Flag */
#define AES__AES_ACCEL_FEEDINTMAP4 AES__AES_ACCEL.ACCEL[4].FEEDINTMAP.R /* FEEDINTMAP */
#define AES__AES_ACCEL_RESULTINTMAP4 AES__AES_ACCEL.ACCEL[4].RESULTINTMAP.R /* RESULTINTMAP */
#define AES__AES_ACCEL_LEN5  AES__AES_ACCEL.ACCEL[5].LEN.R /* LEN */
#define AES__AES_ACCEL_AILEN5 AES__AES_ACCEL.ACCEL[5].AILEN.R /* AAD/IV length */
#define AES__AES_ACCEL_CRYPT5 AES__AES_ACCEL.ACCEL[5].CRYPT.R /* CRYPT */
#define AES__AES_ACCEL_OWNSTAT5 AES__AES_ACCEL.ACCEL[5].OWNSTAT.R /* OWN_Status */
#define AES__AES_ACCEL_TLVAL5 AES__AES_ACCEL.ACCEL[5].TLVAL.R /* Timer Load Value */
#define AES__AES_ACCEL_CVAL5 AES__AES_ACCEL.ACCEL[5].CVAL.R /* Current Timer Value */
#define AES__AES_ACCEL_TCTRL5 AES__AES_ACCEL.ACCEL[5].TCTRL.R /* Timer Control */
#define AES__AES_ACCEL_TFLG5 AES__AES_ACCEL.ACCEL[5].TFLG.R /* Timer Flag */
#define AES__AES_ACCEL_FEEDINTMAP5 AES__AES_ACCEL.ACCEL[5].FEEDINTMAP.R /* FEEDINTMAP */
#define AES__AES_ACCEL_RESULTINTMAP5 AES__AES_ACCEL.ACCEL[5].RESULTINTMAP.R /* RESULTINTMAP */
#define AES__AES_ACCEL_LEN6  AES__AES_ACCEL.ACCEL[6].LEN.R /* LEN */
#define AES__AES_ACCEL_AILEN6 AES__AES_ACCEL.ACCEL[6].AILEN.R /* AAD/IV length */
#define AES__AES_ACCEL_CRYPT6 AES__AES_ACCEL.ACCEL[6].CRYPT.R /* CRYPT */
#define AES__AES_ACCEL_OWNSTAT6 AES__AES_ACCEL.ACCEL[6].OWNSTAT.R /* OWN_Status */
#define AES__AES_ACCEL_TLVAL6 AES__AES_ACCEL.ACCEL[6].TLVAL.R /* Timer Load Value */
#define AES__AES_ACCEL_CVAL6 AES__AES_ACCEL.ACCEL[6].CVAL.R /* Current Timer Value */
#define AES__AES_ACCEL_TCTRL6 AES__AES_ACCEL.ACCEL[6].TCTRL.R /* Timer Control */
#define AES__AES_ACCEL_TFLG6 AES__AES_ACCEL.ACCEL[6].TFLG.R /* Timer Flag */
#define AES__AES_ACCEL_FEEDINTMAP6 AES__AES_ACCEL.ACCEL[6].FEEDINTMAP.R /* FEEDINTMAP */
#define AES__AES_ACCEL_RESULTINTMAP6 AES__AES_ACCEL.ACCEL[6].RESULTINTMAP.R /* RESULTINTMAP */
#define AES__AES_ACCEL_LEN7  AES__AES_ACCEL.ACCEL[7].LEN.R /* LEN */
#define AES__AES_ACCEL_AILEN7 AES__AES_ACCEL.ACCEL[7].AILEN.R /* AAD/IV length */
#define AES__AES_ACCEL_CRYPT7 AES__AES_ACCEL.ACCEL[7].CRYPT.R /* CRYPT */
#define AES__AES_ACCEL_OWNSTAT7 AES__AES_ACCEL.ACCEL[7].OWNSTAT.R /* OWN_Status */
#define AES__AES_ACCEL_TLVAL7 AES__AES_ACCEL.ACCEL[7].TLVAL.R /* Timer Load Value */
#define AES__AES_ACCEL_CVAL7 AES__AES_ACCEL.ACCEL[7].CVAL.R /* Current Timer Value */
#define AES__AES_ACCEL_TCTRL7 AES__AES_ACCEL.ACCEL[7].TCTRL.R /* Timer Control */
#define AES__AES_ACCEL_TFLG7 AES__AES_ACCEL.ACCEL[7].TFLG.R /* Timer Flag */
#define AES__AES_ACCEL_FEEDINTMAP7 AES__AES_ACCEL.ACCEL[7].FEEDINTMAP.R /* FEEDINTMAP */
#define AES__AES_ACCEL_RESULTINTMAP7 AES__AES_ACCEL.ACCEL[7].RESULTINTMAP.R /* RESULTINTMAP */

/* ATP */
#define ATP_PLLC             ATP.PLLC.R                    /* PLL Control */
#define ATP_PLLS             ATP.PLLS.R                    /* PLL Status */
#define ATP_PLLDIV           ATP.PLLDIV.R                  /* PLL Divider */
#define ATP_PLLCAL_2         ATP.PLLCAL_2.R                /* PLL Calibration 2 */
#define ATP_PLLCKMUX         ATP.PLLCKMUX.R                /* PLL Clock MUX */
#define ATP_ALS              ATP.ALS.R                     /* AL Status */
#define ATP_ALGC             ATP.ALGC.R                    /* AL General Control */
#define ATP_ALTC             ATP.ALTC.R                    /* AL Training Control */
#define ATP_LVDSTX           ATP.LVDSTX.R                  /* LVDS TX IO Configuration */
#define ATP_LVDSRX           ATP.LVDSRX.R                  /* LVDS RX IO Configuration */
#define ATP_LVDSTXOBE        ATP.LVDSTXOBE.R               /* LVDS TX OBE Configuration */
#define ATP_CIAC             ATP.CIAC.R                    /* CIA Control */
#define ATP_ATPE             ATP.ATPE.R                    /* Aurora Trace Port Enable */
#define ATP_ITCTRL           ATP.ITCTRL.R                  /* Integration Mode Control */
#define ATP_CLAIMSET         ATP.CLAIMSET.R                /* Claim Tag Set */
#define ATP_CLAIMCLR         ATP.CLAIMCLR.R                /* Claim Tag Clear */
#define ATP_DEVAFF0          ATP.DEVAFF0.R                 /* Device Affinity 0 */
#define ATP_DEVAFF1          ATP.DEVAFF1.R                 /* Device Affinity 1 */
#define ATP_LAR              ATP.LAR.R                     /* Lock Access Register */
#define ATP_LSR              ATP.LSR.R                     /* Lock Status Register */
#define ATP_AUTHSTATUS       ATP.AUTHSTATUS.R              /* Authentication Status */
#define ATP_DEVARCH          ATP.DEVARCH.R                 /* Device Architecture */
#define ATP_DEVID2           ATP.DEVID2.R                  /* Device Configuration 2 */
#define ATP_DEVID1           ATP.DEVID1.R                  /* Device Configuration 1 */
#define ATP_DEVID            ATP.DEVID.R                   /* Device Configuration */
#define ATP_DEVTYPE          ATP.DEVTYPE.R                 /* Device Type Identifier */
#define ATP_PIDR4            ATP.PIDR4.R                   /* Peripheral Identification Register 4 */
#define ATP_PIDR0            ATP.PIDR0.R                   /* Peripheral Identification Register 0 */
#define ATP_PIDR1            ATP.PIDR1.R                   /* Peripheral Identification Register 1 */
#define ATP_PIDR2            ATP.PIDR2.R                   /* Peripheral Identification Register 2 */
#define ATP_PIDR3            ATP.PIDR3.R                   /* Peripheral Identification Register 3 */
#define ATP_CIDR0            ATP.CIDR0.R                   /* Component Identification Register 0 */
#define ATP_CIDR1            ATP.CIDR1.R                   /* Component Identification Register 1 */
#define ATP_CIDR2            ATP.CIDR2.R                   /* Component Identification Register 2 */
#define ATP_CIDR3            ATP.CIDR3.R                   /* Component Identification Register 3 */

/* AXBS */
#define CE_AXBS_PRS0         CE_AXBS.PORT[0].PRS.R         /* Priority Slave Registers */
#define CE_AXBS_CRS0         CE_AXBS.PORT[0].CRS.R         /* Control Register */
#define CE_AXBS_PRS1         CE_AXBS.PORT[1].PRS.R         /* Priority Slave Registers */
#define CE_AXBS_CRS1         CE_AXBS.PORT[1].CRS.R         /* Control Register */
#define CE_AXBS_PRS2         CE_AXBS.PORT[2].PRS.R         /* Priority Slave Registers */
#define CE_AXBS_CRS2         CE_AXBS.PORT[2].CRS.R         /* Control Register */
#define CE_AXBS_PRS3         CE_AXBS.PORT[3].PRS.R         /* Priority Slave Registers */
#define CE_AXBS_CRS3         CE_AXBS.PORT[3].CRS.R         /* Control Register */
#define CE_AXBS_PRS4         CE_AXBS.PORT[4].PRS.R         /* Priority Slave Registers */
#define CE_AXBS_CRS4         CE_AXBS.PORT[4].CRS.R         /* Control Register */
#define CE_AXBS_PRS5         CE_AXBS.PORT[5].PRS.R         /* Priority Slave Registers */
#define CE_AXBS_CRS5         CE_AXBS.PORT[5].CRS.R         /* Control Register */
#define CE_AXBS_PRS6         CE_AXBS.PORT[6].PRS.R         /* Priority Slave Registers */
#define CE_AXBS_CRS6         CE_AXBS.PORT[6].CRS.R         /* Control Register */
#define CE_AXBS_PRS7         CE_AXBS.PORT[7].PRS.R         /* Priority Slave Registers */
#define CE_AXBS_CRS7         CE_AXBS.PORT[7].CRS.R         /* Control Register */
#define CE_AXBS_MGPCR0       CE_AXBS.MGPCR0.R              /* Master General Purpose Control Register */
#define CE_AXBS_MGPCR1       CE_AXBS.MGPCR1.R              /* Master General Purpose Control Register */
#define CE_AXBS_MGPCR2       CE_AXBS.MGPCR2.R              /* Master General Purpose Control Register */
#define CE_AXBS_MGPCR3       CE_AXBS.MGPCR3.R              /* Master General Purpose Control Register */
#define CE_AXBS_MGPCR4       CE_AXBS.MGPCR4.R              /* Master General Purpose Control Register */
#define CE_AXBS_MGPCR5       CE_AXBS.MGPCR5.R              /* Master General Purpose Control Register */

#define SMU__AXBS_PRS0       SMU__AXBS.PORT[0].PRS.R       /* Priority Slave Registers */
#define SMU__AXBS_CRS0       SMU__AXBS.PORT[0].CRS.R       /* Control Register */
#define SMU__AXBS_PRS1       SMU__AXBS.PORT[1].PRS.R       /* Priority Slave Registers */
#define SMU__AXBS_CRS1       SMU__AXBS.PORT[1].CRS.R       /* Control Register */
#define SMU__AXBS_PRS2       SMU__AXBS.PORT[2].PRS.R       /* Priority Slave Registers */
#define SMU__AXBS_CRS2       SMU__AXBS.PORT[2].CRS.R       /* Control Register */
#define SMU__AXBS_PRS3       SMU__AXBS.PORT[3].PRS.R       /* Priority Slave Registers */
#define SMU__AXBS_CRS3       SMU__AXBS.PORT[3].CRS.R       /* Control Register */
#define SMU__AXBS_PRS4       SMU__AXBS.PORT[4].PRS.R       /* Priority Slave Registers */
#define SMU__AXBS_CRS4       SMU__AXBS.PORT[4].CRS.R       /* Control Register */
#define SMU__AXBS_PRS5       SMU__AXBS.PORT[5].PRS.R       /* Priority Slave Registers */
#define SMU__AXBS_CRS5       SMU__AXBS.PORT[5].CRS.R       /* Control Register */
#define SMU__AXBS_PRS6       SMU__AXBS.PORT[6].PRS.R       /* Priority Slave Registers */
#define SMU__AXBS_CRS6       SMU__AXBS.PORT[6].CRS.R       /* Control Register */
#define SMU__AXBS_MGPCR0     SMU__AXBS.MGPCR0.R            /* Master General Purpose Control Register */
#define SMU__AXBS_MGPCR1     SMU__AXBS.MGPCR1.R            /* Master General Purpose Control Register */
#define SMU__AXBS_MGPCR2     SMU__AXBS.MGPCR2.R            /* Master General Purpose Control Register */
#define SMU__AXBS_MGPCR3     SMU__AXBS.MGPCR3.R            /* Master General Purpose Control Register */

/* Boot */
#define BMR_BOOT_GPR_BMR1    BMR.BOOT_GPR_BMR1.R           /* Boot Configuration 1 */
#define BMR_BOOT_GPR_BMR2    BMR.BOOT_GPR_BMR2.R           /* Boot Configuration 2 */

#define POR_BOOT_POR_CTRL_REG POR.BOOT_POR_CTRL_REG.R      /* Boot POR Control */
#define POR_BOOT_POR_C1_REG  POR.BOOT_POR_C1_REG.R         /* BOOT POR Control 1 Register */
#define POR_BOOT_DEST_C0_REG POR.BOOT_DEST_C0_REG.R        /* BOOT DEST Control 0 Register */
#define POR_BOOT_FUNC_C0_REG POR.BOOT_FUNC_C0_REG.R        /* BOOT FUNC Control 0 Register */

/* CANXL_DSC_CONTROL */
#define CANXL_0__DSC_CONTROL_DCSTA0 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[0].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH0 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[0].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK0 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[0].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT0 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[0].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA1 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[1].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH1 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[1].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK1 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[1].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT1 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[1].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA2 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[2].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH2 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[2].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK2 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[2].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT2 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[2].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA3 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[3].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH3 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[3].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK3 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[3].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT3 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[3].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA4 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[4].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH4 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[4].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK4 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[4].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT4 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[4].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA5 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[5].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH5 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[5].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK5 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[5].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT5 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[5].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA6 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[6].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH6 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[6].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK6 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[6].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT6 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[6].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA7 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[7].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH7 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[7].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK7 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[7].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT7 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[7].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA8 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[8].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH8 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[8].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK8 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[8].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT8 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[8].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA9 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[9].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH9 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[9].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK9 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[9].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT9 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[9].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA10 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[10].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH10 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[10].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK10 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[10].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT10 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[10].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA11 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[11].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH11 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[11].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK11 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[11].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT11 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[11].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA12 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[12].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH12 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[12].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK12 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[12].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT12 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[12].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA13 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[13].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH13 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[13].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK13 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[13].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT13 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[13].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA14 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[14].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH14 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[14].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK14 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[14].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT14 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[14].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA15 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[15].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH15 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[15].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK15 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[15].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT15 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[15].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA16 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[16].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH16 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[16].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK16 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[16].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT16 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[16].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA17 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[17].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH17 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[17].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK17 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[17].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT17 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[17].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA18 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[18].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH18 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[18].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK18 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[18].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT18 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[18].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA19 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[19].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH19 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[19].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK19 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[19].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT19 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[19].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA20 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[20].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH20 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[20].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK20 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[20].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT20 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[20].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA21 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[21].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH21 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[21].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK21 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[21].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT21 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[21].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA22 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[22].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH22 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[22].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK22 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[22].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT22 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[22].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA23 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[23].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH23 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[23].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK23 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[23].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT23 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[23].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA24 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[24].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH24 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[24].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK24 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[24].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT24 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[24].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA25 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[25].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH25 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[25].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK25 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[25].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT25 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[25].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA26 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[26].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH26 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[26].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK26 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[26].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT26 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[26].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA27 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[27].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH27 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[27].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK27 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[27].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT27 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[27].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA28 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[28].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH28 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[28].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK28 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[28].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT28 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[28].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA29 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[29].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH29 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[29].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK29 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[29].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT29 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[29].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA30 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[30].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH30 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[30].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK30 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[30].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT30 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[30].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA31 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[31].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH31 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[31].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK31 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[31].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT31 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[31].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA32 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[32].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH32 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[32].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK32 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[32].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT32 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[32].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA33 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[33].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH33 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[33].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK33 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[33].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT33 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[33].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA34 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[34].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH34 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[34].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK34 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[34].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT34 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[34].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA35 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[35].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH35 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[35].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK35 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[35].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT35 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[35].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA36 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[36].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH36 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[36].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK36 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[36].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT36 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[36].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA37 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[37].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH37 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[37].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK37 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[37].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT37 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[37].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA38 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[38].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH38 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[38].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK38 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[38].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT38 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[38].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA39 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[39].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH39 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[39].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK39 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[39].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT39 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[39].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA40 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[40].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH40 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[40].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK40 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[40].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT40 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[40].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA41 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[41].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH41 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[41].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK41 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[41].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT41 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[41].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA42 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[42].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH42 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[42].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK42 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[42].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT42 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[42].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA43 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[43].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH43 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[43].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK43 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[43].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT43 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[43].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA44 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[44].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH44 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[44].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK44 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[44].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT44 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[44].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA45 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[45].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH45 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[45].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK45 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[45].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT45 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[45].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA46 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[46].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH46 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[46].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK46 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[46].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT46 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[46].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA47 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[47].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH47 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[47].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK47 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[47].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT47 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[47].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA48 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[48].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH48 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[48].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK48 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[48].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT48 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[48].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA49 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[49].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH49 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[49].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK49 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[49].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT49 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[49].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA50 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[50].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH50 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[50].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK50 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[50].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT50 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[50].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA51 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[51].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH51 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[51].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK51 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[51].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT51 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[51].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA52 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[52].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH52 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[52].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK52 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[52].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT52 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[52].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA53 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[53].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH53 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[53].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK53 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[53].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT53 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[53].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA54 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[54].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH54 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[54].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK54 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[54].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT54 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[54].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA55 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[55].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH55 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[55].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK55 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[55].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT55 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[55].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA56 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[56].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH56 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[56].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK56 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[56].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT56 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[56].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA57 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[57].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH57 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[57].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK57 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[57].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT57 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[57].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA58 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[58].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH58 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[58].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK58 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[58].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT58 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[58].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA59 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[59].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH59 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[59].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK59 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[59].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT59 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[59].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA60 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[60].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH60 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[60].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK60 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[60].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT60 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[60].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA61 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[61].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH61 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[61].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK61 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[61].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT61 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[61].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA62 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[62].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH62 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[62].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK62 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[62].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT62 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[62].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA63 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[63].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH63 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[63].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK63 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[63].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT63 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[63].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA64 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[64].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH64 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[64].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK64 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[64].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT64 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[64].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA65 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[65].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH65 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[65].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK65 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[65].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT65 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[65].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA66 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[66].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH66 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[66].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK66 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[66].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT66 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[66].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA67 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[67].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH67 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[67].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK67 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[67].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT67 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[67].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA68 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[68].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH68 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[68].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK68 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[68].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT68 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[68].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA69 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[69].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH69 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[69].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK69 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[69].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT69 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[69].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA70 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[70].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH70 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[70].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK70 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[70].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT70 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[70].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA71 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[71].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH71 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[71].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK71 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[71].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT71 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[71].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA72 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[72].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH72 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[72].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK72 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[72].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT72 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[72].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA73 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[73].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH73 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[73].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK73 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[73].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT73 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[73].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA74 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[74].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH74 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[74].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK74 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[74].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT74 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[74].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA75 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[75].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH75 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[75].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK75 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[75].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT75 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[75].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA76 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[76].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH76 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[76].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK76 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[76].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT76 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[76].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA77 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[77].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH77 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[77].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK77 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[77].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT77 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[77].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA78 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[78].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH78 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[78].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK78 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[78].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT78 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[78].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA79 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[79].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH79 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[79].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK79 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[79].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT79 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[79].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA80 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[80].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH80 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[80].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK80 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[80].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT80 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[80].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA81 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[81].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH81 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[81].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK81 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[81].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT81 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[81].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA82 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[82].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH82 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[82].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK82 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[82].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT82 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[82].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA83 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[83].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH83 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[83].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK83 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[83].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT83 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[83].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA84 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[84].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH84 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[84].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK84 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[84].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT84 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[84].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA85 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[85].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH85 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[85].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK85 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[85].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT85 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[85].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA86 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[86].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH86 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[86].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK86 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[86].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT86 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[86].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA87 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[87].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH87 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[87].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK87 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[87].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT87 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[87].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA88 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[88].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH88 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[88].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK88 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[88].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT88 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[88].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA89 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[89].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH89 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[89].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK89 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[89].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT89 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[89].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA90 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[90].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH90 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[90].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK90 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[90].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT90 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[90].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA91 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[91].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH91 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[91].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK91 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[91].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT91 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[91].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA92 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[92].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH92 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[92].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK92 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[92].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT92 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[92].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA93 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[93].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH93 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[93].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK93 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[93].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT93 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[93].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA94 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[94].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH94 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[94].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK94 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[94].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT94 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[94].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA95 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[95].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH95 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[95].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK95 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[95].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT95 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[95].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA96 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[96].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH96 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[96].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK96 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[96].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT96 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[96].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA97 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[97].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH97 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[97].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK97 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[97].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT97 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[97].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA98 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[98].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH98 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[98].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK98 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[98].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT98 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[98].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA99 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[99].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH99 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[99].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK99 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[99].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT99 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[99].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA100 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[100].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH100 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[100].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK100 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[100].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT100 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[100].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA101 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[101].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH101 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[101].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK101 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[101].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT101 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[101].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA102 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[102].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH102 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[102].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK102 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[102].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT102 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[102].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA103 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[103].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH103 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[103].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK103 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[103].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT103 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[103].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA104 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[104].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH104 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[104].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK104 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[104].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT104 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[104].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA105 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[105].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH105 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[105].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK105 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[105].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT105 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[105].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA106 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[106].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH106 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[106].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK106 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[106].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT106 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[106].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA107 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[107].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH107 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[107].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK107 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[107].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT107 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[107].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA108 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[108].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH108 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[108].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK108 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[108].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT108 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[108].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA109 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[109].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH109 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[109].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK109 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[109].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT109 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[109].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA110 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[110].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH110 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[110].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK110 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[110].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT110 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[110].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA111 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[111].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH111 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[111].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK111 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[111].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT111 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[111].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA112 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[112].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH112 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[112].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK112 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[112].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT112 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[112].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA113 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[113].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH113 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[113].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK113 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[113].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT113 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[113].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA114 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[114].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH114 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[114].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK114 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[114].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT114 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[114].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA115 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[115].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH115 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[115].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK115 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[115].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT115 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[115].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA116 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[116].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH116 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[116].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK116 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[116].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT116 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[116].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA117 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[117].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH117 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[117].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK117 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[117].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT117 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[117].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA118 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[118].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH118 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[118].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK118 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[118].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT118 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[118].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA119 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[119].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH119 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[119].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK119 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[119].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT119 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[119].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA120 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[120].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH120 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[120].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK120 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[120].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT120 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[120].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA121 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[121].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH121 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[121].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK121 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[121].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT121 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[121].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA122 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[122].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH122 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[122].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK122 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[122].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT122 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[122].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA123 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[123].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH123 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[123].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK123 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[123].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT123 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[123].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA124 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[124].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH124 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[124].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK124 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[124].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT124 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[124].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA125 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[125].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH125 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[125].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK125 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[125].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT125 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[125].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA126 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[126].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH126 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[126].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK126 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[126].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT126 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[126].DCACT.R /* Descriptor Control Activation */
#define CANXL_0__DSC_CONTROL_DCSTA127 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[127].DCSTA.R /* Descriptor Control Status */
#define CANXL_0__DSC_CONTROL_DCSYSPUSH127 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[127].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_0__DSC_CONTROL_DCSYSLOCK127 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[127].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_0__DSC_CONTROL_DCACT127 CANXL_0__DSC_CONTROL.DSCMBCTRLAR[127].DCACT.R /* Descriptor Control Activation */

#define CANXL_1__DSC_CONTROL_DCSTA0 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[0].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH0 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[0].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK0 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[0].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT0 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[0].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA1 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[1].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH1 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[1].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK1 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[1].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT1 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[1].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA2 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[2].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH2 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[2].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK2 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[2].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT2 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[2].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA3 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[3].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH3 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[3].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK3 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[3].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT3 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[3].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA4 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[4].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH4 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[4].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK4 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[4].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT4 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[4].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA5 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[5].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH5 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[5].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK5 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[5].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT5 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[5].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA6 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[6].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH6 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[6].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK6 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[6].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT6 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[6].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA7 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[7].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH7 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[7].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK7 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[7].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT7 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[7].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA8 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[8].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH8 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[8].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK8 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[8].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT8 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[8].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA9 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[9].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH9 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[9].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK9 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[9].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT9 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[9].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA10 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[10].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH10 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[10].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK10 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[10].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT10 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[10].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA11 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[11].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH11 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[11].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK11 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[11].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT11 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[11].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA12 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[12].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH12 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[12].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK12 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[12].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT12 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[12].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA13 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[13].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH13 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[13].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK13 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[13].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT13 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[13].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA14 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[14].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH14 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[14].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK14 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[14].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT14 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[14].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA15 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[15].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH15 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[15].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK15 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[15].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT15 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[15].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA16 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[16].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH16 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[16].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK16 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[16].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT16 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[16].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA17 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[17].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH17 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[17].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK17 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[17].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT17 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[17].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA18 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[18].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH18 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[18].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK18 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[18].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT18 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[18].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA19 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[19].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH19 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[19].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK19 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[19].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT19 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[19].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA20 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[20].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH20 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[20].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK20 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[20].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT20 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[20].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA21 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[21].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH21 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[21].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK21 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[21].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT21 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[21].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA22 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[22].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH22 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[22].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK22 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[22].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT22 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[22].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA23 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[23].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH23 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[23].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK23 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[23].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT23 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[23].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA24 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[24].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH24 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[24].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK24 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[24].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT24 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[24].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA25 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[25].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH25 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[25].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK25 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[25].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT25 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[25].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA26 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[26].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH26 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[26].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK26 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[26].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT26 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[26].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA27 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[27].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH27 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[27].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK27 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[27].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT27 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[27].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA28 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[28].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH28 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[28].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK28 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[28].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT28 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[28].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA29 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[29].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH29 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[29].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK29 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[29].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT29 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[29].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA30 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[30].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH30 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[30].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK30 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[30].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT30 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[30].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA31 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[31].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH31 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[31].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK31 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[31].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT31 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[31].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA32 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[32].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH32 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[32].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK32 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[32].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT32 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[32].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA33 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[33].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH33 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[33].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK33 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[33].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT33 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[33].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA34 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[34].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH34 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[34].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK34 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[34].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT34 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[34].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA35 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[35].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH35 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[35].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK35 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[35].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT35 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[35].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA36 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[36].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH36 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[36].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK36 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[36].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT36 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[36].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA37 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[37].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH37 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[37].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK37 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[37].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT37 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[37].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA38 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[38].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH38 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[38].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK38 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[38].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT38 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[38].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA39 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[39].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH39 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[39].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK39 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[39].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT39 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[39].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA40 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[40].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH40 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[40].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK40 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[40].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT40 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[40].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA41 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[41].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH41 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[41].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK41 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[41].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT41 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[41].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA42 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[42].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH42 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[42].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK42 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[42].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT42 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[42].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA43 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[43].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH43 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[43].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK43 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[43].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT43 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[43].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA44 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[44].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH44 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[44].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK44 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[44].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT44 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[44].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA45 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[45].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH45 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[45].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK45 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[45].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT45 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[45].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA46 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[46].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH46 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[46].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK46 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[46].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT46 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[46].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA47 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[47].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH47 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[47].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK47 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[47].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT47 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[47].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA48 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[48].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH48 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[48].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK48 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[48].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT48 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[48].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA49 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[49].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH49 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[49].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK49 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[49].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT49 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[49].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA50 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[50].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH50 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[50].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK50 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[50].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT50 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[50].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA51 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[51].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH51 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[51].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK51 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[51].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT51 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[51].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA52 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[52].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH52 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[52].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK52 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[52].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT52 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[52].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA53 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[53].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH53 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[53].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK53 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[53].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT53 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[53].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA54 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[54].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH54 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[54].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK54 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[54].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT54 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[54].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA55 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[55].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH55 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[55].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK55 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[55].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT55 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[55].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA56 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[56].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH56 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[56].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK56 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[56].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT56 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[56].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA57 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[57].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH57 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[57].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK57 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[57].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT57 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[57].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA58 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[58].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH58 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[58].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK58 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[58].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT58 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[58].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA59 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[59].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH59 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[59].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK59 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[59].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT59 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[59].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA60 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[60].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH60 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[60].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK60 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[60].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT60 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[60].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA61 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[61].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH61 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[61].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK61 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[61].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT61 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[61].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA62 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[62].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH62 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[62].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK62 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[62].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT62 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[62].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA63 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[63].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH63 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[63].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK63 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[63].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT63 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[63].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA64 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[64].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH64 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[64].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK64 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[64].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT64 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[64].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA65 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[65].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH65 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[65].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK65 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[65].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT65 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[65].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA66 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[66].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH66 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[66].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK66 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[66].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT66 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[66].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA67 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[67].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH67 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[67].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK67 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[67].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT67 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[67].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA68 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[68].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH68 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[68].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK68 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[68].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT68 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[68].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA69 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[69].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH69 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[69].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK69 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[69].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT69 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[69].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA70 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[70].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH70 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[70].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK70 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[70].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT70 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[70].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA71 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[71].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH71 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[71].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK71 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[71].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT71 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[71].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA72 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[72].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH72 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[72].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK72 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[72].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT72 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[72].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA73 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[73].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH73 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[73].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK73 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[73].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT73 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[73].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA74 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[74].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH74 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[74].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK74 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[74].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT74 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[74].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA75 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[75].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH75 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[75].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK75 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[75].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT75 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[75].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA76 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[76].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH76 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[76].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK76 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[76].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT76 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[76].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA77 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[77].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH77 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[77].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK77 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[77].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT77 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[77].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA78 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[78].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH78 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[78].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK78 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[78].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT78 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[78].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA79 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[79].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH79 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[79].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK79 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[79].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT79 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[79].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA80 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[80].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH80 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[80].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK80 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[80].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT80 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[80].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA81 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[81].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH81 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[81].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK81 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[81].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT81 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[81].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA82 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[82].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH82 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[82].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK82 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[82].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT82 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[82].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA83 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[83].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH83 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[83].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK83 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[83].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT83 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[83].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA84 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[84].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH84 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[84].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK84 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[84].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT84 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[84].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA85 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[85].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH85 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[85].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK85 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[85].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT85 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[85].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA86 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[86].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH86 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[86].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK86 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[86].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT86 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[86].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA87 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[87].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH87 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[87].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK87 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[87].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT87 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[87].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA88 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[88].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH88 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[88].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK88 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[88].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT88 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[88].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA89 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[89].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH89 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[89].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK89 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[89].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT89 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[89].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA90 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[90].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH90 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[90].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK90 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[90].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT90 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[90].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA91 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[91].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH91 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[91].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK91 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[91].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT91 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[91].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA92 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[92].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH92 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[92].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK92 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[92].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT92 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[92].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA93 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[93].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH93 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[93].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK93 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[93].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT93 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[93].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA94 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[94].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH94 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[94].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK94 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[94].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT94 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[94].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA95 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[95].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH95 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[95].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK95 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[95].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT95 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[95].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA96 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[96].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH96 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[96].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK96 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[96].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT96 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[96].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA97 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[97].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH97 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[97].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK97 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[97].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT97 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[97].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA98 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[98].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH98 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[98].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK98 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[98].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT98 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[98].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA99 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[99].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH99 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[99].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK99 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[99].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT99 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[99].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA100 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[100].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH100 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[100].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK100 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[100].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT100 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[100].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA101 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[101].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH101 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[101].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK101 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[101].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT101 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[101].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA102 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[102].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH102 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[102].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK102 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[102].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT102 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[102].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA103 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[103].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH103 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[103].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK103 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[103].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT103 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[103].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA104 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[104].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH104 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[104].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK104 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[104].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT104 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[104].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA105 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[105].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH105 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[105].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK105 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[105].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT105 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[105].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA106 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[106].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH106 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[106].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK106 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[106].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT106 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[106].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA107 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[107].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH107 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[107].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK107 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[107].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT107 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[107].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA108 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[108].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH108 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[108].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK108 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[108].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT108 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[108].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA109 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[109].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH109 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[109].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK109 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[109].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT109 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[109].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA110 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[110].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH110 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[110].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK110 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[110].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT110 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[110].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA111 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[111].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH111 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[111].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK111 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[111].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT111 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[111].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA112 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[112].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH112 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[112].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK112 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[112].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT112 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[112].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA113 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[113].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH113 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[113].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK113 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[113].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT113 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[113].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA114 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[114].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH114 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[114].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK114 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[114].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT114 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[114].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA115 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[115].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH115 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[115].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK115 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[115].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT115 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[115].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA116 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[116].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH116 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[116].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK116 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[116].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT116 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[116].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA117 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[117].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH117 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[117].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK117 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[117].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT117 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[117].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA118 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[118].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH118 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[118].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK118 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[118].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT118 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[118].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA119 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[119].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH119 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[119].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK119 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[119].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT119 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[119].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA120 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[120].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH120 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[120].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK120 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[120].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT120 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[120].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA121 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[121].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH121 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[121].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK121 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[121].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT121 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[121].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA122 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[122].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH122 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[122].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK122 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[122].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT122 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[122].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA123 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[123].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH123 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[123].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK123 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[123].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT123 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[123].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA124 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[124].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH124 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[124].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK124 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[124].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT124 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[124].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA125 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[125].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH125 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[125].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK125 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[125].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT125 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[125].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA126 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[126].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH126 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[126].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK126 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[126].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT126 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[126].DCACT.R /* Descriptor Control Activation */
#define CANXL_1__DSC_CONTROL_DCSTA127 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[127].DCSTA.R /* Descriptor Control Status */
#define CANXL_1__DSC_CONTROL_DCSYSPUSH127 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[127].DCSYSPUSH.R /* Descriptor Control System Push */
#define CANXL_1__DSC_CONTROL_DCSYSLOCK127 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[127].DCSYSLOCK.R /* Descriptor Control System Lock */
#define CANXL_1__DSC_CONTROL_DCACT127 CANXL_1__DSC_CONTROL.DSCMBCTRLAR[127].DCACT.R /* Descriptor Control Activation */

/* CANXL_FILTER_BANK */
#define CANXL_0__FILTER_BANK_AFCFG0 CANXL_0__FILTER_BANK.AFCFG0.R /* Acceptance Filter Configuration */
#define CANXL_0__FILTER_BANK_VAMRCFG0 CANXL_0__FILTER_BANK.VAMRCFG0.R /* VCAN Acceptance Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_SAMRCFG0 CANXL_0__FILTER_BANK.SAMRCFG0.R /* SDU Acceptance Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_AAMRCFG0 CANXL_0__FILTER_BANK.AAMRCFG0.R /* ADDR Acceptance Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_VAFLT0_0 CANXL_0__FILTER_BANK.VAFLT0_0.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_2 CANXL_0__FILTER_BANK.VAFLT0_2.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_4 CANXL_0__FILTER_BANK.VAFLT0_4.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_6 CANXL_0__FILTER_BANK.VAFLT0_6.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_8 CANXL_0__FILTER_BANK.VAFLT0_8.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_10 CANXL_0__FILTER_BANK.VAFLT0_10.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_12 CANXL_0__FILTER_BANK.VAFLT0_12.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_14 CANXL_0__FILTER_BANK.VAFLT0_14.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_16 CANXL_0__FILTER_BANK.VAFLT0_16.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_18 CANXL_0__FILTER_BANK.VAFLT0_18.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_20 CANXL_0__FILTER_BANK.VAFLT0_20.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_22 CANXL_0__FILTER_BANK.VAFLT0_22.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_24 CANXL_0__FILTER_BANK.VAFLT0_24.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_26 CANXL_0__FILTER_BANK.VAFLT0_26.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_28 CANXL_0__FILTER_BANK.VAFLT0_28.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT0_30 CANXL_0__FILTER_BANK.VAFLT0_30.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_0 CANXL_0__FILTER_BANK.SAFLT0_0.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_2 CANXL_0__FILTER_BANK.SAFLT0_2.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_4 CANXL_0__FILTER_BANK.SAFLT0_4.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_6 CANXL_0__FILTER_BANK.SAFLT0_6.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_8 CANXL_0__FILTER_BANK.SAFLT0_8.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_10 CANXL_0__FILTER_BANK.SAFLT0_10.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_12 CANXL_0__FILTER_BANK.SAFLT0_12.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_14 CANXL_0__FILTER_BANK.SAFLT0_14.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_16 CANXL_0__FILTER_BANK.SAFLT0_16.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_18 CANXL_0__FILTER_BANK.SAFLT0_18.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_20 CANXL_0__FILTER_BANK.SAFLT0_20.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_22 CANXL_0__FILTER_BANK.SAFLT0_22.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_24 CANXL_0__FILTER_BANK.SAFLT0_24.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_26 CANXL_0__FILTER_BANK.SAFLT0_26.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_28 CANXL_0__FILTER_BANK.SAFLT0_28.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT0_30 CANXL_0__FILTER_BANK.SAFLT0_30.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_AAFLT0_0L CANXL_0__FILTER_BANK.AAFLT0_0L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_0H CANXL_0__FILTER_BANK.AAFLT0_0H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_1L CANXL_0__FILTER_BANK.AAFLT0_1L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_1H CANXL_0__FILTER_BANK.AAFLT0_1H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_2L CANXL_0__FILTER_BANK.AAFLT0_2L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_2H CANXL_0__FILTER_BANK.AAFLT0_2H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_3L CANXL_0__FILTER_BANK.AAFLT0_3L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_3H CANXL_0__FILTER_BANK.AAFLT0_3H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_4L CANXL_0__FILTER_BANK.AAFLT0_4L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_4H CANXL_0__FILTER_BANK.AAFLT0_4H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_5L CANXL_0__FILTER_BANK.AAFLT0_5L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_5H CANXL_0__FILTER_BANK.AAFLT0_5H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_6L CANXL_0__FILTER_BANK.AAFLT0_6L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_6H CANXL_0__FILTER_BANK.AAFLT0_6H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_7L CANXL_0__FILTER_BANK.AAFLT0_7L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_7H CANXL_0__FILTER_BANK.AAFLT0_7H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_8L CANXL_0__FILTER_BANK.AAFLT0_8L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_8H CANXL_0__FILTER_BANK.AAFLT0_8H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_9L CANXL_0__FILTER_BANK.AAFLT0_9L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_9H CANXL_0__FILTER_BANK.AAFLT0_9H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_10L CANXL_0__FILTER_BANK.AAFLT0_10L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_10H CANXL_0__FILTER_BANK.AAFLT0_10H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_11L CANXL_0__FILTER_BANK.AAFLT0_11L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_11H CANXL_0__FILTER_BANK.AAFLT0_11H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_12L CANXL_0__FILTER_BANK.AAFLT0_12L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_12H CANXL_0__FILTER_BANK.AAFLT0_12H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_13L CANXL_0__FILTER_BANK.AAFLT0_13L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_13H CANXL_0__FILTER_BANK.AAFLT0_13H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_14L CANXL_0__FILTER_BANK.AAFLT0_14L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_14H CANXL_0__FILTER_BANK.AAFLT0_14H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_15L CANXL_0__FILTER_BANK.AAFLT0_15L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_15H CANXL_0__FILTER_BANK.AAFLT0_15H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_16L CANXL_0__FILTER_BANK.AAFLT0_16L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_16H CANXL_0__FILTER_BANK.AAFLT0_16H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_17L CANXL_0__FILTER_BANK.AAFLT0_17L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_17H CANXL_0__FILTER_BANK.AAFLT0_17H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_18L CANXL_0__FILTER_BANK.AAFLT0_18L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_18H CANXL_0__FILTER_BANK.AAFLT0_18H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_19L CANXL_0__FILTER_BANK.AAFLT0_19L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_19H CANXL_0__FILTER_BANK.AAFLT0_19H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_20L CANXL_0__FILTER_BANK.AAFLT0_20L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_20H CANXL_0__FILTER_BANK.AAFLT0_20H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_21L CANXL_0__FILTER_BANK.AAFLT0_21L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_21H CANXL_0__FILTER_BANK.AAFLT0_21H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_22L CANXL_0__FILTER_BANK.AAFLT0_22L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_22H CANXL_0__FILTER_BANK.AAFLT0_22H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_23L CANXL_0__FILTER_BANK.AAFLT0_23L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_23H CANXL_0__FILTER_BANK.AAFLT0_23H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_24L CANXL_0__FILTER_BANK.AAFLT0_24L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_24H CANXL_0__FILTER_BANK.AAFLT0_24H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_25L CANXL_0__FILTER_BANK.AAFLT0_25L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_25H CANXL_0__FILTER_BANK.AAFLT0_25H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_26L CANXL_0__FILTER_BANK.AAFLT0_26L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_26H CANXL_0__FILTER_BANK.AAFLT0_26H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_27L CANXL_0__FILTER_BANK.AAFLT0_27L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_27H CANXL_0__FILTER_BANK.AAFLT0_27H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_28L CANXL_0__FILTER_BANK.AAFLT0_28L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_28H CANXL_0__FILTER_BANK.AAFLT0_28H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_29L CANXL_0__FILTER_BANK.AAFLT0_29L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_29H CANXL_0__FILTER_BANK.AAFLT0_29H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_30L CANXL_0__FILTER_BANK.AAFLT0_30L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_30H CANXL_0__FILTER_BANK.AAFLT0_30H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT0_31L CANXL_0__FILTER_BANK.AAFLT0_31L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT0_31H CANXL_0__FILTER_BANK.AAFLT0_31H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_RFCFG0 CANXL_0__FILTER_BANK.RFCFG0.R /* Rejection Filter Configuration */
#define CANXL_0__FILTER_BANK_VRMRCFG0 CANXL_0__FILTER_BANK.VRMRCFG0.R /* VCAN Rejection Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_SRMRCFG0 CANXL_0__FILTER_BANK.SRMRCFG0.R /* SDU Rejection Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_ARMRCFG0 CANXL_0__FILTER_BANK.ARMRCFG0.R /* ADDR Rejection Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_VRFLT0_0 CANXL_0__FILTER_BANK.VRFLT0_0.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_2 CANXL_0__FILTER_BANK.VRFLT0_2.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_4 CANXL_0__FILTER_BANK.VRFLT0_4.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_6 CANXL_0__FILTER_BANK.VRFLT0_6.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_8 CANXL_0__FILTER_BANK.VRFLT0_8.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_10 CANXL_0__FILTER_BANK.VRFLT0_10.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_12 CANXL_0__FILTER_BANK.VRFLT0_12.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_14 CANXL_0__FILTER_BANK.VRFLT0_14.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_16 CANXL_0__FILTER_BANK.VRFLT0_16.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_18 CANXL_0__FILTER_BANK.VRFLT0_18.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_20 CANXL_0__FILTER_BANK.VRFLT0_20.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_22 CANXL_0__FILTER_BANK.VRFLT0_22.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_24 CANXL_0__FILTER_BANK.VRFLT0_24.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_26 CANXL_0__FILTER_BANK.VRFLT0_26.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_28 CANXL_0__FILTER_BANK.VRFLT0_28.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT0_30 CANXL_0__FILTER_BANK.VRFLT0_30.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_0 CANXL_0__FILTER_BANK.SRFLT0_0.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_2 CANXL_0__FILTER_BANK.SRFLT0_2.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_4 CANXL_0__FILTER_BANK.SRFLT0_4.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_6 CANXL_0__FILTER_BANK.SRFLT0_6.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_8 CANXL_0__FILTER_BANK.SRFLT0_8.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_10 CANXL_0__FILTER_BANK.SRFLT0_10.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_12 CANXL_0__FILTER_BANK.SRFLT0_12.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_14 CANXL_0__FILTER_BANK.SRFLT0_14.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_16 CANXL_0__FILTER_BANK.SRFLT0_16.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_18 CANXL_0__FILTER_BANK.SRFLT0_18.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_20 CANXL_0__FILTER_BANK.SRFLT0_20.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_22 CANXL_0__FILTER_BANK.SRFLT0_22.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_24 CANXL_0__FILTER_BANK.SRFLT0_24.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_26 CANXL_0__FILTER_BANK.SRFLT0_26.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_28 CANXL_0__FILTER_BANK.SRFLT0_28.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT0_30 CANXL_0__FILTER_BANK.SRFLT0_30.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_ARFLT0_0L CANXL_0__FILTER_BANK.ARFLT0_0L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_0H CANXL_0__FILTER_BANK.ARFLT0_0H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_1L CANXL_0__FILTER_BANK.ARFLT0_1L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_1H CANXL_0__FILTER_BANK.ARFLT0_1H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_2L CANXL_0__FILTER_BANK.ARFLT0_2L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_2H CANXL_0__FILTER_BANK.ARFLT0_2H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_3L CANXL_0__FILTER_BANK.ARFLT0_3L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_3H CANXL_0__FILTER_BANK.ARFLT0_3H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_4L CANXL_0__FILTER_BANK.ARFLT0_4L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_4H CANXL_0__FILTER_BANK.ARFLT0_4H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_5L CANXL_0__FILTER_BANK.ARFLT0_5L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_5H CANXL_0__FILTER_BANK.ARFLT0_5H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_6L CANXL_0__FILTER_BANK.ARFLT0_6L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_6H CANXL_0__FILTER_BANK.ARFLT0_6H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_7L CANXL_0__FILTER_BANK.ARFLT0_7L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_7H CANXL_0__FILTER_BANK.ARFLT0_7H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_8L CANXL_0__FILTER_BANK.ARFLT0_8L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_8H CANXL_0__FILTER_BANK.ARFLT0_8H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_9L CANXL_0__FILTER_BANK.ARFLT0_9L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_9H CANXL_0__FILTER_BANK.ARFLT0_9H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_10L CANXL_0__FILTER_BANK.ARFLT0_10L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_10H CANXL_0__FILTER_BANK.ARFLT0_10H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_11L CANXL_0__FILTER_BANK.ARFLT0_11L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_11H CANXL_0__FILTER_BANK.ARFLT0_11H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_12L CANXL_0__FILTER_BANK.ARFLT0_12L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_12H CANXL_0__FILTER_BANK.ARFLT0_12H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_13L CANXL_0__FILTER_BANK.ARFLT0_13L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_13H CANXL_0__FILTER_BANK.ARFLT0_13H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_14L CANXL_0__FILTER_BANK.ARFLT0_14L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_14H CANXL_0__FILTER_BANK.ARFLT0_14H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_15L CANXL_0__FILTER_BANK.ARFLT0_15L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_15H CANXL_0__FILTER_BANK.ARFLT0_15H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_16L CANXL_0__FILTER_BANK.ARFLT0_16L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_16H CANXL_0__FILTER_BANK.ARFLT0_16H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_17L CANXL_0__FILTER_BANK.ARFLT0_17L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_17H CANXL_0__FILTER_BANK.ARFLT0_17H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_18L CANXL_0__FILTER_BANK.ARFLT0_18L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_18H CANXL_0__FILTER_BANK.ARFLT0_18H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_19L CANXL_0__FILTER_BANK.ARFLT0_19L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_19H CANXL_0__FILTER_BANK.ARFLT0_19H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_20L CANXL_0__FILTER_BANK.ARFLT0_20L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_20H CANXL_0__FILTER_BANK.ARFLT0_20H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_21L CANXL_0__FILTER_BANK.ARFLT0_21L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_21H CANXL_0__FILTER_BANK.ARFLT0_21H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_22L CANXL_0__FILTER_BANK.ARFLT0_22L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_22H CANXL_0__FILTER_BANK.ARFLT0_22H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_23L CANXL_0__FILTER_BANK.ARFLT0_23L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_23H CANXL_0__FILTER_BANK.ARFLT0_23H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_24L CANXL_0__FILTER_BANK.ARFLT0_24L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_24H CANXL_0__FILTER_BANK.ARFLT0_24H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_25L CANXL_0__FILTER_BANK.ARFLT0_25L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_25H CANXL_0__FILTER_BANK.ARFLT0_25H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_26L CANXL_0__FILTER_BANK.ARFLT0_26L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_26H CANXL_0__FILTER_BANK.ARFLT0_26H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_27L CANXL_0__FILTER_BANK.ARFLT0_27L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_27H CANXL_0__FILTER_BANK.ARFLT0_27H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_28L CANXL_0__FILTER_BANK.ARFLT0_28L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_28H CANXL_0__FILTER_BANK.ARFLT0_28H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_29L CANXL_0__FILTER_BANK.ARFLT0_29L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_29H CANXL_0__FILTER_BANK.ARFLT0_29H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_30L CANXL_0__FILTER_BANK.ARFLT0_30L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_30H CANXL_0__FILTER_BANK.ARFLT0_30H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT0_31L CANXL_0__FILTER_BANK.ARFLT0_31L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT0_31H CANXL_0__FILTER_BANK.ARFLT0_31H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_AFCFG1 CANXL_0__FILTER_BANK.AFCFG1.R /* Acceptance Filter Configuration */
#define CANXL_0__FILTER_BANK_VAMRCFG1 CANXL_0__FILTER_BANK.VAMRCFG1.R /* VCAN Acceptance Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_SAMRCFG1 CANXL_0__FILTER_BANK.SAMRCFG1.R /* SDU Acceptance Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_AAMRCFG1 CANXL_0__FILTER_BANK.AAMRCFG1.R /* ADDR Acceptance Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_VAFLT1_0 CANXL_0__FILTER_BANK.VAFLT1_0.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_2 CANXL_0__FILTER_BANK.VAFLT1_2.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_4 CANXL_0__FILTER_BANK.VAFLT1_4.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_6 CANXL_0__FILTER_BANK.VAFLT1_6.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_8 CANXL_0__FILTER_BANK.VAFLT1_8.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_10 CANXL_0__FILTER_BANK.VAFLT1_10.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_12 CANXL_0__FILTER_BANK.VAFLT1_12.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_14 CANXL_0__FILTER_BANK.VAFLT1_14.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_16 CANXL_0__FILTER_BANK.VAFLT1_16.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_18 CANXL_0__FILTER_BANK.VAFLT1_18.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_20 CANXL_0__FILTER_BANK.VAFLT1_20.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_22 CANXL_0__FILTER_BANK.VAFLT1_22.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_24 CANXL_0__FILTER_BANK.VAFLT1_24.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_26 CANXL_0__FILTER_BANK.VAFLT1_26.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_28 CANXL_0__FILTER_BANK.VAFLT1_28.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_VAFLT1_30 CANXL_0__FILTER_BANK.VAFLT1_30.R /* VCAN Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_0 CANXL_0__FILTER_BANK.SAFLT1_0.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_2 CANXL_0__FILTER_BANK.SAFLT1_2.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_4 CANXL_0__FILTER_BANK.SAFLT1_4.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_6 CANXL_0__FILTER_BANK.SAFLT1_6.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_8 CANXL_0__FILTER_BANK.SAFLT1_8.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_10 CANXL_0__FILTER_BANK.SAFLT1_10.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_12 CANXL_0__FILTER_BANK.SAFLT1_12.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_14 CANXL_0__FILTER_BANK.SAFLT1_14.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_16 CANXL_0__FILTER_BANK.SAFLT1_16.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_18 CANXL_0__FILTER_BANK.SAFLT1_18.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_20 CANXL_0__FILTER_BANK.SAFLT1_20.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_22 CANXL_0__FILTER_BANK.SAFLT1_22.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_24 CANXL_0__FILTER_BANK.SAFLT1_24.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_26 CANXL_0__FILTER_BANK.SAFLT1_26.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_28 CANXL_0__FILTER_BANK.SAFLT1_28.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_SAFLT1_30 CANXL_0__FILTER_BANK.SAFLT1_30.R /* SDU Acceptance Filter */
#define CANXL_0__FILTER_BANK_AAFLT1_0L CANXL_0__FILTER_BANK.AAFLT1_0L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_0H CANXL_0__FILTER_BANK.AAFLT1_0H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_1L CANXL_0__FILTER_BANK.AAFLT1_1L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_1H CANXL_0__FILTER_BANK.AAFLT1_1H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_2L CANXL_0__FILTER_BANK.AAFLT1_2L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_2H CANXL_0__FILTER_BANK.AAFLT1_2H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_3L CANXL_0__FILTER_BANK.AAFLT1_3L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_3H CANXL_0__FILTER_BANK.AAFLT1_3H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_4L CANXL_0__FILTER_BANK.AAFLT1_4L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_4H CANXL_0__FILTER_BANK.AAFLT1_4H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_5L CANXL_0__FILTER_BANK.AAFLT1_5L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_5H CANXL_0__FILTER_BANK.AAFLT1_5H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_6L CANXL_0__FILTER_BANK.AAFLT1_6L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_6H CANXL_0__FILTER_BANK.AAFLT1_6H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_7L CANXL_0__FILTER_BANK.AAFLT1_7L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_7H CANXL_0__FILTER_BANK.AAFLT1_7H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_8L CANXL_0__FILTER_BANK.AAFLT1_8L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_8H CANXL_0__FILTER_BANK.AAFLT1_8H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_9L CANXL_0__FILTER_BANK.AAFLT1_9L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_9H CANXL_0__FILTER_BANK.AAFLT1_9H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_10L CANXL_0__FILTER_BANK.AAFLT1_10L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_10H CANXL_0__FILTER_BANK.AAFLT1_10H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_11L CANXL_0__FILTER_BANK.AAFLT1_11L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_11H CANXL_0__FILTER_BANK.AAFLT1_11H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_12L CANXL_0__FILTER_BANK.AAFLT1_12L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_12H CANXL_0__FILTER_BANK.AAFLT1_12H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_13L CANXL_0__FILTER_BANK.AAFLT1_13L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_13H CANXL_0__FILTER_BANK.AAFLT1_13H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_14L CANXL_0__FILTER_BANK.AAFLT1_14L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_14H CANXL_0__FILTER_BANK.AAFLT1_14H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_15L CANXL_0__FILTER_BANK.AAFLT1_15L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_15H CANXL_0__FILTER_BANK.AAFLT1_15H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_16L CANXL_0__FILTER_BANK.AAFLT1_16L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_16H CANXL_0__FILTER_BANK.AAFLT1_16H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_17L CANXL_0__FILTER_BANK.AAFLT1_17L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_17H CANXL_0__FILTER_BANK.AAFLT1_17H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_18L CANXL_0__FILTER_BANK.AAFLT1_18L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_18H CANXL_0__FILTER_BANK.AAFLT1_18H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_19L CANXL_0__FILTER_BANK.AAFLT1_19L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_19H CANXL_0__FILTER_BANK.AAFLT1_19H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_20L CANXL_0__FILTER_BANK.AAFLT1_20L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_20H CANXL_0__FILTER_BANK.AAFLT1_20H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_21L CANXL_0__FILTER_BANK.AAFLT1_21L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_21H CANXL_0__FILTER_BANK.AAFLT1_21H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_22L CANXL_0__FILTER_BANK.AAFLT1_22L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_22H CANXL_0__FILTER_BANK.AAFLT1_22H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_23L CANXL_0__FILTER_BANK.AAFLT1_23L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_23H CANXL_0__FILTER_BANK.AAFLT1_23H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_24L CANXL_0__FILTER_BANK.AAFLT1_24L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_24H CANXL_0__FILTER_BANK.AAFLT1_24H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_25L CANXL_0__FILTER_BANK.AAFLT1_25L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_25H CANXL_0__FILTER_BANK.AAFLT1_25H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_26L CANXL_0__FILTER_BANK.AAFLT1_26L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_26H CANXL_0__FILTER_BANK.AAFLT1_26H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_27L CANXL_0__FILTER_BANK.AAFLT1_27L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_27H CANXL_0__FILTER_BANK.AAFLT1_27H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_28L CANXL_0__FILTER_BANK.AAFLT1_28L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_28H CANXL_0__FILTER_BANK.AAFLT1_28H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_29L CANXL_0__FILTER_BANK.AAFLT1_29L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_29H CANXL_0__FILTER_BANK.AAFLT1_29H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_30L CANXL_0__FILTER_BANK.AAFLT1_30L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_30H CANXL_0__FILTER_BANK.AAFLT1_30H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_AAFLT1_31L CANXL_0__FILTER_BANK.AAFLT1_31L.R /* ADDR Acceptance Filter Low */
#define CANXL_0__FILTER_BANK_AAFLT1_31H CANXL_0__FILTER_BANK.AAFLT1_31H.R /* ADDR Acceptance Filter High */
#define CANXL_0__FILTER_BANK_RFCFG1 CANXL_0__FILTER_BANK.RFCFG1.R /* Rejection Filter Configuration */
#define CANXL_0__FILTER_BANK_VRMRCFG1 CANXL_0__FILTER_BANK.VRMRCFG1.R /* VCAN Rejection Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_SRMRCFG1 CANXL_0__FILTER_BANK.SRMRCFG1.R /* SDU Rejection Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_ARMRCFG1 CANXL_0__FILTER_BANK.ARMRCFG1.R /* ADDR Rejection Mask or Range Configuration */
#define CANXL_0__FILTER_BANK_VRFLT1_0 CANXL_0__FILTER_BANK.VRFLT1_0.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_2 CANXL_0__FILTER_BANK.VRFLT1_2.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_4 CANXL_0__FILTER_BANK.VRFLT1_4.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_6 CANXL_0__FILTER_BANK.VRFLT1_6.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_8 CANXL_0__FILTER_BANK.VRFLT1_8.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_10 CANXL_0__FILTER_BANK.VRFLT1_10.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_12 CANXL_0__FILTER_BANK.VRFLT1_12.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_14 CANXL_0__FILTER_BANK.VRFLT1_14.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_16 CANXL_0__FILTER_BANK.VRFLT1_16.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_18 CANXL_0__FILTER_BANK.VRFLT1_18.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_20 CANXL_0__FILTER_BANK.VRFLT1_20.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_22 CANXL_0__FILTER_BANK.VRFLT1_22.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_24 CANXL_0__FILTER_BANK.VRFLT1_24.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_26 CANXL_0__FILTER_BANK.VRFLT1_26.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_28 CANXL_0__FILTER_BANK.VRFLT1_28.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_VRFLT1_30 CANXL_0__FILTER_BANK.VRFLT1_30.R /* VCAN Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_0 CANXL_0__FILTER_BANK.SRFLT1_0.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_2 CANXL_0__FILTER_BANK.SRFLT1_2.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_4 CANXL_0__FILTER_BANK.SRFLT1_4.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_6 CANXL_0__FILTER_BANK.SRFLT1_6.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_8 CANXL_0__FILTER_BANK.SRFLT1_8.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_10 CANXL_0__FILTER_BANK.SRFLT1_10.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_12 CANXL_0__FILTER_BANK.SRFLT1_12.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_14 CANXL_0__FILTER_BANK.SRFLT1_14.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_16 CANXL_0__FILTER_BANK.SRFLT1_16.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_18 CANXL_0__FILTER_BANK.SRFLT1_18.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_20 CANXL_0__FILTER_BANK.SRFLT1_20.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_22 CANXL_0__FILTER_BANK.SRFLT1_22.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_24 CANXL_0__FILTER_BANK.SRFLT1_24.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_26 CANXL_0__FILTER_BANK.SRFLT1_26.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_28 CANXL_0__FILTER_BANK.SRFLT1_28.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_SRFLT1_30 CANXL_0__FILTER_BANK.SRFLT1_30.R /* SDU Rejection Filter */
#define CANXL_0__FILTER_BANK_ARFLT1_0L CANXL_0__FILTER_BANK.ARFLT1_0L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_0H CANXL_0__FILTER_BANK.ARFLT1_0H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_1L CANXL_0__FILTER_BANK.ARFLT1_1L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_1H CANXL_0__FILTER_BANK.ARFLT1_1H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_2L CANXL_0__FILTER_BANK.ARFLT1_2L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_2H CANXL_0__FILTER_BANK.ARFLT1_2H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_3L CANXL_0__FILTER_BANK.ARFLT1_3L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_3H CANXL_0__FILTER_BANK.ARFLT1_3H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_4L CANXL_0__FILTER_BANK.ARFLT1_4L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_4H CANXL_0__FILTER_BANK.ARFLT1_4H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_5L CANXL_0__FILTER_BANK.ARFLT1_5L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_5H CANXL_0__FILTER_BANK.ARFLT1_5H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_6L CANXL_0__FILTER_BANK.ARFLT1_6L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_6H CANXL_0__FILTER_BANK.ARFLT1_6H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_7L CANXL_0__FILTER_BANK.ARFLT1_7L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_7H CANXL_0__FILTER_BANK.ARFLT1_7H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_8L CANXL_0__FILTER_BANK.ARFLT1_8L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_8H CANXL_0__FILTER_BANK.ARFLT1_8H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_9L CANXL_0__FILTER_BANK.ARFLT1_9L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_9H CANXL_0__FILTER_BANK.ARFLT1_9H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_10L CANXL_0__FILTER_BANK.ARFLT1_10L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_10H CANXL_0__FILTER_BANK.ARFLT1_10H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_11L CANXL_0__FILTER_BANK.ARFLT1_11L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_11H CANXL_0__FILTER_BANK.ARFLT1_11H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_12L CANXL_0__FILTER_BANK.ARFLT1_12L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_12H CANXL_0__FILTER_BANK.ARFLT1_12H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_13L CANXL_0__FILTER_BANK.ARFLT1_13L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_13H CANXL_0__FILTER_BANK.ARFLT1_13H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_14L CANXL_0__FILTER_BANK.ARFLT1_14L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_14H CANXL_0__FILTER_BANK.ARFLT1_14H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_15L CANXL_0__FILTER_BANK.ARFLT1_15L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_15H CANXL_0__FILTER_BANK.ARFLT1_15H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_16L CANXL_0__FILTER_BANK.ARFLT1_16L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_16H CANXL_0__FILTER_BANK.ARFLT1_16H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_17L CANXL_0__FILTER_BANK.ARFLT1_17L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_17H CANXL_0__FILTER_BANK.ARFLT1_17H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_18L CANXL_0__FILTER_BANK.ARFLT1_18L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_18H CANXL_0__FILTER_BANK.ARFLT1_18H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_19L CANXL_0__FILTER_BANK.ARFLT1_19L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_19H CANXL_0__FILTER_BANK.ARFLT1_19H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_20L CANXL_0__FILTER_BANK.ARFLT1_20L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_20H CANXL_0__FILTER_BANK.ARFLT1_20H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_21L CANXL_0__FILTER_BANK.ARFLT1_21L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_21H CANXL_0__FILTER_BANK.ARFLT1_21H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_22L CANXL_0__FILTER_BANK.ARFLT1_22L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_22H CANXL_0__FILTER_BANK.ARFLT1_22H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_23L CANXL_0__FILTER_BANK.ARFLT1_23L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_23H CANXL_0__FILTER_BANK.ARFLT1_23H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_24L CANXL_0__FILTER_BANK.ARFLT1_24L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_24H CANXL_0__FILTER_BANK.ARFLT1_24H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_25L CANXL_0__FILTER_BANK.ARFLT1_25L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_25H CANXL_0__FILTER_BANK.ARFLT1_25H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_26L CANXL_0__FILTER_BANK.ARFLT1_26L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_26H CANXL_0__FILTER_BANK.ARFLT1_26H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_27L CANXL_0__FILTER_BANK.ARFLT1_27L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_27H CANXL_0__FILTER_BANK.ARFLT1_27H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_28L CANXL_0__FILTER_BANK.ARFLT1_28L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_28H CANXL_0__FILTER_BANK.ARFLT1_28H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_29L CANXL_0__FILTER_BANK.ARFLT1_29L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_29H CANXL_0__FILTER_BANK.ARFLT1_29H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_30L CANXL_0__FILTER_BANK.ARFLT1_30L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_30H CANXL_0__FILTER_BANK.ARFLT1_30H.R /* ADDR Rejection Filter High */
#define CANXL_0__FILTER_BANK_ARFLT1_31L CANXL_0__FILTER_BANK.ARFLT1_31L.R /* ADDR Rejection Filter Low */
#define CANXL_0__FILTER_BANK_ARFLT1_31H CANXL_0__FILTER_BANK.ARFLT1_31H.R /* ADDR Rejection Filter High */

#define CANXL_1__FILTER_BANK_AFCFG0 CANXL_1__FILTER_BANK.AFCFG0.R /* Acceptance Filter Configuration */
#define CANXL_1__FILTER_BANK_VAMRCFG0 CANXL_1__FILTER_BANK.VAMRCFG0.R /* VCAN Acceptance Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_SAMRCFG0 CANXL_1__FILTER_BANK.SAMRCFG0.R /* SDU Acceptance Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_AAMRCFG0 CANXL_1__FILTER_BANK.AAMRCFG0.R /* ADDR Acceptance Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_VAFLT0_0 CANXL_1__FILTER_BANK.VAFLT0_0.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_2 CANXL_1__FILTER_BANK.VAFLT0_2.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_4 CANXL_1__FILTER_BANK.VAFLT0_4.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_6 CANXL_1__FILTER_BANK.VAFLT0_6.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_8 CANXL_1__FILTER_BANK.VAFLT0_8.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_10 CANXL_1__FILTER_BANK.VAFLT0_10.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_12 CANXL_1__FILTER_BANK.VAFLT0_12.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_14 CANXL_1__FILTER_BANK.VAFLT0_14.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_16 CANXL_1__FILTER_BANK.VAFLT0_16.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_18 CANXL_1__FILTER_BANK.VAFLT0_18.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_20 CANXL_1__FILTER_BANK.VAFLT0_20.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_22 CANXL_1__FILTER_BANK.VAFLT0_22.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_24 CANXL_1__FILTER_BANK.VAFLT0_24.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_26 CANXL_1__FILTER_BANK.VAFLT0_26.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_28 CANXL_1__FILTER_BANK.VAFLT0_28.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT0_30 CANXL_1__FILTER_BANK.VAFLT0_30.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_0 CANXL_1__FILTER_BANK.SAFLT0_0.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_2 CANXL_1__FILTER_BANK.SAFLT0_2.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_4 CANXL_1__FILTER_BANK.SAFLT0_4.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_6 CANXL_1__FILTER_BANK.SAFLT0_6.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_8 CANXL_1__FILTER_BANK.SAFLT0_8.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_10 CANXL_1__FILTER_BANK.SAFLT0_10.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_12 CANXL_1__FILTER_BANK.SAFLT0_12.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_14 CANXL_1__FILTER_BANK.SAFLT0_14.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_16 CANXL_1__FILTER_BANK.SAFLT0_16.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_18 CANXL_1__FILTER_BANK.SAFLT0_18.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_20 CANXL_1__FILTER_BANK.SAFLT0_20.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_22 CANXL_1__FILTER_BANK.SAFLT0_22.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_24 CANXL_1__FILTER_BANK.SAFLT0_24.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_26 CANXL_1__FILTER_BANK.SAFLT0_26.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_28 CANXL_1__FILTER_BANK.SAFLT0_28.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT0_30 CANXL_1__FILTER_BANK.SAFLT0_30.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_AAFLT0_0L CANXL_1__FILTER_BANK.AAFLT0_0L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_0H CANXL_1__FILTER_BANK.AAFLT0_0H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_1L CANXL_1__FILTER_BANK.AAFLT0_1L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_1H CANXL_1__FILTER_BANK.AAFLT0_1H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_2L CANXL_1__FILTER_BANK.AAFLT0_2L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_2H CANXL_1__FILTER_BANK.AAFLT0_2H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_3L CANXL_1__FILTER_BANK.AAFLT0_3L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_3H CANXL_1__FILTER_BANK.AAFLT0_3H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_4L CANXL_1__FILTER_BANK.AAFLT0_4L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_4H CANXL_1__FILTER_BANK.AAFLT0_4H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_5L CANXL_1__FILTER_BANK.AAFLT0_5L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_5H CANXL_1__FILTER_BANK.AAFLT0_5H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_6L CANXL_1__FILTER_BANK.AAFLT0_6L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_6H CANXL_1__FILTER_BANK.AAFLT0_6H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_7L CANXL_1__FILTER_BANK.AAFLT0_7L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_7H CANXL_1__FILTER_BANK.AAFLT0_7H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_8L CANXL_1__FILTER_BANK.AAFLT0_8L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_8H CANXL_1__FILTER_BANK.AAFLT0_8H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_9L CANXL_1__FILTER_BANK.AAFLT0_9L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_9H CANXL_1__FILTER_BANK.AAFLT0_9H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_10L CANXL_1__FILTER_BANK.AAFLT0_10L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_10H CANXL_1__FILTER_BANK.AAFLT0_10H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_11L CANXL_1__FILTER_BANK.AAFLT0_11L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_11H CANXL_1__FILTER_BANK.AAFLT0_11H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_12L CANXL_1__FILTER_BANK.AAFLT0_12L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_12H CANXL_1__FILTER_BANK.AAFLT0_12H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_13L CANXL_1__FILTER_BANK.AAFLT0_13L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_13H CANXL_1__FILTER_BANK.AAFLT0_13H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_14L CANXL_1__FILTER_BANK.AAFLT0_14L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_14H CANXL_1__FILTER_BANK.AAFLT0_14H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_15L CANXL_1__FILTER_BANK.AAFLT0_15L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_15H CANXL_1__FILTER_BANK.AAFLT0_15H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_16L CANXL_1__FILTER_BANK.AAFLT0_16L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_16H CANXL_1__FILTER_BANK.AAFLT0_16H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_17L CANXL_1__FILTER_BANK.AAFLT0_17L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_17H CANXL_1__FILTER_BANK.AAFLT0_17H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_18L CANXL_1__FILTER_BANK.AAFLT0_18L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_18H CANXL_1__FILTER_BANK.AAFLT0_18H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_19L CANXL_1__FILTER_BANK.AAFLT0_19L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_19H CANXL_1__FILTER_BANK.AAFLT0_19H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_20L CANXL_1__FILTER_BANK.AAFLT0_20L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_20H CANXL_1__FILTER_BANK.AAFLT0_20H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_21L CANXL_1__FILTER_BANK.AAFLT0_21L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_21H CANXL_1__FILTER_BANK.AAFLT0_21H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_22L CANXL_1__FILTER_BANK.AAFLT0_22L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_22H CANXL_1__FILTER_BANK.AAFLT0_22H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_23L CANXL_1__FILTER_BANK.AAFLT0_23L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_23H CANXL_1__FILTER_BANK.AAFLT0_23H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_24L CANXL_1__FILTER_BANK.AAFLT0_24L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_24H CANXL_1__FILTER_BANK.AAFLT0_24H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_25L CANXL_1__FILTER_BANK.AAFLT0_25L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_25H CANXL_1__FILTER_BANK.AAFLT0_25H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_26L CANXL_1__FILTER_BANK.AAFLT0_26L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_26H CANXL_1__FILTER_BANK.AAFLT0_26H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_27L CANXL_1__FILTER_BANK.AAFLT0_27L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_27H CANXL_1__FILTER_BANK.AAFLT0_27H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_28L CANXL_1__FILTER_BANK.AAFLT0_28L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_28H CANXL_1__FILTER_BANK.AAFLT0_28H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_29L CANXL_1__FILTER_BANK.AAFLT0_29L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_29H CANXL_1__FILTER_BANK.AAFLT0_29H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_30L CANXL_1__FILTER_BANK.AAFLT0_30L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_30H CANXL_1__FILTER_BANK.AAFLT0_30H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT0_31L CANXL_1__FILTER_BANK.AAFLT0_31L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT0_31H CANXL_1__FILTER_BANK.AAFLT0_31H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_RFCFG0 CANXL_1__FILTER_BANK.RFCFG0.R /* Rejection Filter Configuration */
#define CANXL_1__FILTER_BANK_VRMRCFG0 CANXL_1__FILTER_BANK.VRMRCFG0.R /* VCAN Rejection Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_SRMRCFG0 CANXL_1__FILTER_BANK.SRMRCFG0.R /* SDU Rejection Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_ARMRCFG0 CANXL_1__FILTER_BANK.ARMRCFG0.R /* ADDR Rejection Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_VRFLT0_0 CANXL_1__FILTER_BANK.VRFLT0_0.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_2 CANXL_1__FILTER_BANK.VRFLT0_2.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_4 CANXL_1__FILTER_BANK.VRFLT0_4.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_6 CANXL_1__FILTER_BANK.VRFLT0_6.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_8 CANXL_1__FILTER_BANK.VRFLT0_8.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_10 CANXL_1__FILTER_BANK.VRFLT0_10.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_12 CANXL_1__FILTER_BANK.VRFLT0_12.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_14 CANXL_1__FILTER_BANK.VRFLT0_14.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_16 CANXL_1__FILTER_BANK.VRFLT0_16.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_18 CANXL_1__FILTER_BANK.VRFLT0_18.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_20 CANXL_1__FILTER_BANK.VRFLT0_20.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_22 CANXL_1__FILTER_BANK.VRFLT0_22.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_24 CANXL_1__FILTER_BANK.VRFLT0_24.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_26 CANXL_1__FILTER_BANK.VRFLT0_26.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_28 CANXL_1__FILTER_BANK.VRFLT0_28.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT0_30 CANXL_1__FILTER_BANK.VRFLT0_30.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_0 CANXL_1__FILTER_BANK.SRFLT0_0.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_2 CANXL_1__FILTER_BANK.SRFLT0_2.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_4 CANXL_1__FILTER_BANK.SRFLT0_4.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_6 CANXL_1__FILTER_BANK.SRFLT0_6.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_8 CANXL_1__FILTER_BANK.SRFLT0_8.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_10 CANXL_1__FILTER_BANK.SRFLT0_10.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_12 CANXL_1__FILTER_BANK.SRFLT0_12.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_14 CANXL_1__FILTER_BANK.SRFLT0_14.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_16 CANXL_1__FILTER_BANK.SRFLT0_16.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_18 CANXL_1__FILTER_BANK.SRFLT0_18.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_20 CANXL_1__FILTER_BANK.SRFLT0_20.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_22 CANXL_1__FILTER_BANK.SRFLT0_22.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_24 CANXL_1__FILTER_BANK.SRFLT0_24.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_26 CANXL_1__FILTER_BANK.SRFLT0_26.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_28 CANXL_1__FILTER_BANK.SRFLT0_28.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT0_30 CANXL_1__FILTER_BANK.SRFLT0_30.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_ARFLT0_0L CANXL_1__FILTER_BANK.ARFLT0_0L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_0H CANXL_1__FILTER_BANK.ARFLT0_0H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_1L CANXL_1__FILTER_BANK.ARFLT0_1L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_1H CANXL_1__FILTER_BANK.ARFLT0_1H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_2L CANXL_1__FILTER_BANK.ARFLT0_2L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_2H CANXL_1__FILTER_BANK.ARFLT0_2H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_3L CANXL_1__FILTER_BANK.ARFLT0_3L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_3H CANXL_1__FILTER_BANK.ARFLT0_3H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_4L CANXL_1__FILTER_BANK.ARFLT0_4L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_4H CANXL_1__FILTER_BANK.ARFLT0_4H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_5L CANXL_1__FILTER_BANK.ARFLT0_5L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_5H CANXL_1__FILTER_BANK.ARFLT0_5H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_6L CANXL_1__FILTER_BANK.ARFLT0_6L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_6H CANXL_1__FILTER_BANK.ARFLT0_6H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_7L CANXL_1__FILTER_BANK.ARFLT0_7L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_7H CANXL_1__FILTER_BANK.ARFLT0_7H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_8L CANXL_1__FILTER_BANK.ARFLT0_8L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_8H CANXL_1__FILTER_BANK.ARFLT0_8H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_9L CANXL_1__FILTER_BANK.ARFLT0_9L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_9H CANXL_1__FILTER_BANK.ARFLT0_9H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_10L CANXL_1__FILTER_BANK.ARFLT0_10L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_10H CANXL_1__FILTER_BANK.ARFLT0_10H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_11L CANXL_1__FILTER_BANK.ARFLT0_11L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_11H CANXL_1__FILTER_BANK.ARFLT0_11H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_12L CANXL_1__FILTER_BANK.ARFLT0_12L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_12H CANXL_1__FILTER_BANK.ARFLT0_12H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_13L CANXL_1__FILTER_BANK.ARFLT0_13L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_13H CANXL_1__FILTER_BANK.ARFLT0_13H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_14L CANXL_1__FILTER_BANK.ARFLT0_14L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_14H CANXL_1__FILTER_BANK.ARFLT0_14H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_15L CANXL_1__FILTER_BANK.ARFLT0_15L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_15H CANXL_1__FILTER_BANK.ARFLT0_15H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_16L CANXL_1__FILTER_BANK.ARFLT0_16L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_16H CANXL_1__FILTER_BANK.ARFLT0_16H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_17L CANXL_1__FILTER_BANK.ARFLT0_17L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_17H CANXL_1__FILTER_BANK.ARFLT0_17H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_18L CANXL_1__FILTER_BANK.ARFLT0_18L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_18H CANXL_1__FILTER_BANK.ARFLT0_18H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_19L CANXL_1__FILTER_BANK.ARFLT0_19L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_19H CANXL_1__FILTER_BANK.ARFLT0_19H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_20L CANXL_1__FILTER_BANK.ARFLT0_20L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_20H CANXL_1__FILTER_BANK.ARFLT0_20H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_21L CANXL_1__FILTER_BANK.ARFLT0_21L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_21H CANXL_1__FILTER_BANK.ARFLT0_21H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_22L CANXL_1__FILTER_BANK.ARFLT0_22L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_22H CANXL_1__FILTER_BANK.ARFLT0_22H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_23L CANXL_1__FILTER_BANK.ARFLT0_23L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_23H CANXL_1__FILTER_BANK.ARFLT0_23H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_24L CANXL_1__FILTER_BANK.ARFLT0_24L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_24H CANXL_1__FILTER_BANK.ARFLT0_24H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_25L CANXL_1__FILTER_BANK.ARFLT0_25L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_25H CANXL_1__FILTER_BANK.ARFLT0_25H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_26L CANXL_1__FILTER_BANK.ARFLT0_26L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_26H CANXL_1__FILTER_BANK.ARFLT0_26H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_27L CANXL_1__FILTER_BANK.ARFLT0_27L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_27H CANXL_1__FILTER_BANK.ARFLT0_27H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_28L CANXL_1__FILTER_BANK.ARFLT0_28L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_28H CANXL_1__FILTER_BANK.ARFLT0_28H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_29L CANXL_1__FILTER_BANK.ARFLT0_29L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_29H CANXL_1__FILTER_BANK.ARFLT0_29H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_30L CANXL_1__FILTER_BANK.ARFLT0_30L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_30H CANXL_1__FILTER_BANK.ARFLT0_30H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT0_31L CANXL_1__FILTER_BANK.ARFLT0_31L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT0_31H CANXL_1__FILTER_BANK.ARFLT0_31H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_AFCFG1 CANXL_1__FILTER_BANK.AFCFG1.R /* Acceptance Filter Configuration */
#define CANXL_1__FILTER_BANK_VAMRCFG1 CANXL_1__FILTER_BANK.VAMRCFG1.R /* VCAN Acceptance Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_SAMRCFG1 CANXL_1__FILTER_BANK.SAMRCFG1.R /* SDU Acceptance Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_AAMRCFG1 CANXL_1__FILTER_BANK.AAMRCFG1.R /* ADDR Acceptance Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_VAFLT1_0 CANXL_1__FILTER_BANK.VAFLT1_0.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_2 CANXL_1__FILTER_BANK.VAFLT1_2.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_4 CANXL_1__FILTER_BANK.VAFLT1_4.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_6 CANXL_1__FILTER_BANK.VAFLT1_6.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_8 CANXL_1__FILTER_BANK.VAFLT1_8.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_10 CANXL_1__FILTER_BANK.VAFLT1_10.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_12 CANXL_1__FILTER_BANK.VAFLT1_12.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_14 CANXL_1__FILTER_BANK.VAFLT1_14.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_16 CANXL_1__FILTER_BANK.VAFLT1_16.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_18 CANXL_1__FILTER_BANK.VAFLT1_18.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_20 CANXL_1__FILTER_BANK.VAFLT1_20.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_22 CANXL_1__FILTER_BANK.VAFLT1_22.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_24 CANXL_1__FILTER_BANK.VAFLT1_24.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_26 CANXL_1__FILTER_BANK.VAFLT1_26.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_28 CANXL_1__FILTER_BANK.VAFLT1_28.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_VAFLT1_30 CANXL_1__FILTER_BANK.VAFLT1_30.R /* VCAN Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_0 CANXL_1__FILTER_BANK.SAFLT1_0.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_2 CANXL_1__FILTER_BANK.SAFLT1_2.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_4 CANXL_1__FILTER_BANK.SAFLT1_4.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_6 CANXL_1__FILTER_BANK.SAFLT1_6.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_8 CANXL_1__FILTER_BANK.SAFLT1_8.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_10 CANXL_1__FILTER_BANK.SAFLT1_10.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_12 CANXL_1__FILTER_BANK.SAFLT1_12.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_14 CANXL_1__FILTER_BANK.SAFLT1_14.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_16 CANXL_1__FILTER_BANK.SAFLT1_16.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_18 CANXL_1__FILTER_BANK.SAFLT1_18.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_20 CANXL_1__FILTER_BANK.SAFLT1_20.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_22 CANXL_1__FILTER_BANK.SAFLT1_22.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_24 CANXL_1__FILTER_BANK.SAFLT1_24.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_26 CANXL_1__FILTER_BANK.SAFLT1_26.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_28 CANXL_1__FILTER_BANK.SAFLT1_28.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_SAFLT1_30 CANXL_1__FILTER_BANK.SAFLT1_30.R /* SDU Acceptance Filter */
#define CANXL_1__FILTER_BANK_AAFLT1_0L CANXL_1__FILTER_BANK.AAFLT1_0L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_0H CANXL_1__FILTER_BANK.AAFLT1_0H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_1L CANXL_1__FILTER_BANK.AAFLT1_1L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_1H CANXL_1__FILTER_BANK.AAFLT1_1H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_2L CANXL_1__FILTER_BANK.AAFLT1_2L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_2H CANXL_1__FILTER_BANK.AAFLT1_2H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_3L CANXL_1__FILTER_BANK.AAFLT1_3L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_3H CANXL_1__FILTER_BANK.AAFLT1_3H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_4L CANXL_1__FILTER_BANK.AAFLT1_4L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_4H CANXL_1__FILTER_BANK.AAFLT1_4H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_5L CANXL_1__FILTER_BANK.AAFLT1_5L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_5H CANXL_1__FILTER_BANK.AAFLT1_5H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_6L CANXL_1__FILTER_BANK.AAFLT1_6L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_6H CANXL_1__FILTER_BANK.AAFLT1_6H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_7L CANXL_1__FILTER_BANK.AAFLT1_7L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_7H CANXL_1__FILTER_BANK.AAFLT1_7H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_8L CANXL_1__FILTER_BANK.AAFLT1_8L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_8H CANXL_1__FILTER_BANK.AAFLT1_8H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_9L CANXL_1__FILTER_BANK.AAFLT1_9L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_9H CANXL_1__FILTER_BANK.AAFLT1_9H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_10L CANXL_1__FILTER_BANK.AAFLT1_10L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_10H CANXL_1__FILTER_BANK.AAFLT1_10H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_11L CANXL_1__FILTER_BANK.AAFLT1_11L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_11H CANXL_1__FILTER_BANK.AAFLT1_11H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_12L CANXL_1__FILTER_BANK.AAFLT1_12L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_12H CANXL_1__FILTER_BANK.AAFLT1_12H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_13L CANXL_1__FILTER_BANK.AAFLT1_13L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_13H CANXL_1__FILTER_BANK.AAFLT1_13H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_14L CANXL_1__FILTER_BANK.AAFLT1_14L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_14H CANXL_1__FILTER_BANK.AAFLT1_14H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_15L CANXL_1__FILTER_BANK.AAFLT1_15L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_15H CANXL_1__FILTER_BANK.AAFLT1_15H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_16L CANXL_1__FILTER_BANK.AAFLT1_16L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_16H CANXL_1__FILTER_BANK.AAFLT1_16H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_17L CANXL_1__FILTER_BANK.AAFLT1_17L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_17H CANXL_1__FILTER_BANK.AAFLT1_17H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_18L CANXL_1__FILTER_BANK.AAFLT1_18L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_18H CANXL_1__FILTER_BANK.AAFLT1_18H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_19L CANXL_1__FILTER_BANK.AAFLT1_19L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_19H CANXL_1__FILTER_BANK.AAFLT1_19H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_20L CANXL_1__FILTER_BANK.AAFLT1_20L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_20H CANXL_1__FILTER_BANK.AAFLT1_20H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_21L CANXL_1__FILTER_BANK.AAFLT1_21L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_21H CANXL_1__FILTER_BANK.AAFLT1_21H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_22L CANXL_1__FILTER_BANK.AAFLT1_22L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_22H CANXL_1__FILTER_BANK.AAFLT1_22H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_23L CANXL_1__FILTER_BANK.AAFLT1_23L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_23H CANXL_1__FILTER_BANK.AAFLT1_23H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_24L CANXL_1__FILTER_BANK.AAFLT1_24L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_24H CANXL_1__FILTER_BANK.AAFLT1_24H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_25L CANXL_1__FILTER_BANK.AAFLT1_25L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_25H CANXL_1__FILTER_BANK.AAFLT1_25H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_26L CANXL_1__FILTER_BANK.AAFLT1_26L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_26H CANXL_1__FILTER_BANK.AAFLT1_26H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_27L CANXL_1__FILTER_BANK.AAFLT1_27L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_27H CANXL_1__FILTER_BANK.AAFLT1_27H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_28L CANXL_1__FILTER_BANK.AAFLT1_28L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_28H CANXL_1__FILTER_BANK.AAFLT1_28H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_29L CANXL_1__FILTER_BANK.AAFLT1_29L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_29H CANXL_1__FILTER_BANK.AAFLT1_29H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_30L CANXL_1__FILTER_BANK.AAFLT1_30L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_30H CANXL_1__FILTER_BANK.AAFLT1_30H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_AAFLT1_31L CANXL_1__FILTER_BANK.AAFLT1_31L.R /* ADDR Acceptance Filter Low */
#define CANXL_1__FILTER_BANK_AAFLT1_31H CANXL_1__FILTER_BANK.AAFLT1_31H.R /* ADDR Acceptance Filter High */
#define CANXL_1__FILTER_BANK_RFCFG1 CANXL_1__FILTER_BANK.RFCFG1.R /* Rejection Filter Configuration */
#define CANXL_1__FILTER_BANK_VRMRCFG1 CANXL_1__FILTER_BANK.VRMRCFG1.R /* VCAN Rejection Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_SRMRCFG1 CANXL_1__FILTER_BANK.SRMRCFG1.R /* SDU Rejection Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_ARMRCFG1 CANXL_1__FILTER_BANK.ARMRCFG1.R /* ADDR Rejection Mask or Range Configuration */
#define CANXL_1__FILTER_BANK_VRFLT1_0 CANXL_1__FILTER_BANK.VRFLT1_0.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_2 CANXL_1__FILTER_BANK.VRFLT1_2.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_4 CANXL_1__FILTER_BANK.VRFLT1_4.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_6 CANXL_1__FILTER_BANK.VRFLT1_6.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_8 CANXL_1__FILTER_BANK.VRFLT1_8.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_10 CANXL_1__FILTER_BANK.VRFLT1_10.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_12 CANXL_1__FILTER_BANK.VRFLT1_12.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_14 CANXL_1__FILTER_BANK.VRFLT1_14.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_16 CANXL_1__FILTER_BANK.VRFLT1_16.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_18 CANXL_1__FILTER_BANK.VRFLT1_18.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_20 CANXL_1__FILTER_BANK.VRFLT1_20.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_22 CANXL_1__FILTER_BANK.VRFLT1_22.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_24 CANXL_1__FILTER_BANK.VRFLT1_24.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_26 CANXL_1__FILTER_BANK.VRFLT1_26.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_28 CANXL_1__FILTER_BANK.VRFLT1_28.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_VRFLT1_30 CANXL_1__FILTER_BANK.VRFLT1_30.R /* VCAN Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_0 CANXL_1__FILTER_BANK.SRFLT1_0.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_2 CANXL_1__FILTER_BANK.SRFLT1_2.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_4 CANXL_1__FILTER_BANK.SRFLT1_4.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_6 CANXL_1__FILTER_BANK.SRFLT1_6.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_8 CANXL_1__FILTER_BANK.SRFLT1_8.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_10 CANXL_1__FILTER_BANK.SRFLT1_10.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_12 CANXL_1__FILTER_BANK.SRFLT1_12.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_14 CANXL_1__FILTER_BANK.SRFLT1_14.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_16 CANXL_1__FILTER_BANK.SRFLT1_16.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_18 CANXL_1__FILTER_BANK.SRFLT1_18.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_20 CANXL_1__FILTER_BANK.SRFLT1_20.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_22 CANXL_1__FILTER_BANK.SRFLT1_22.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_24 CANXL_1__FILTER_BANK.SRFLT1_24.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_26 CANXL_1__FILTER_BANK.SRFLT1_26.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_28 CANXL_1__FILTER_BANK.SRFLT1_28.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_SRFLT1_30 CANXL_1__FILTER_BANK.SRFLT1_30.R /* SDU Rejection Filter */
#define CANXL_1__FILTER_BANK_ARFLT1_0L CANXL_1__FILTER_BANK.ARFLT1_0L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_0H CANXL_1__FILTER_BANK.ARFLT1_0H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_1L CANXL_1__FILTER_BANK.ARFLT1_1L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_1H CANXL_1__FILTER_BANK.ARFLT1_1H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_2L CANXL_1__FILTER_BANK.ARFLT1_2L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_2H CANXL_1__FILTER_BANK.ARFLT1_2H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_3L CANXL_1__FILTER_BANK.ARFLT1_3L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_3H CANXL_1__FILTER_BANK.ARFLT1_3H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_4L CANXL_1__FILTER_BANK.ARFLT1_4L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_4H CANXL_1__FILTER_BANK.ARFLT1_4H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_5L CANXL_1__FILTER_BANK.ARFLT1_5L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_5H CANXL_1__FILTER_BANK.ARFLT1_5H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_6L CANXL_1__FILTER_BANK.ARFLT1_6L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_6H CANXL_1__FILTER_BANK.ARFLT1_6H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_7L CANXL_1__FILTER_BANK.ARFLT1_7L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_7H CANXL_1__FILTER_BANK.ARFLT1_7H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_8L CANXL_1__FILTER_BANK.ARFLT1_8L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_8H CANXL_1__FILTER_BANK.ARFLT1_8H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_9L CANXL_1__FILTER_BANK.ARFLT1_9L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_9H CANXL_1__FILTER_BANK.ARFLT1_9H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_10L CANXL_1__FILTER_BANK.ARFLT1_10L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_10H CANXL_1__FILTER_BANK.ARFLT1_10H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_11L CANXL_1__FILTER_BANK.ARFLT1_11L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_11H CANXL_1__FILTER_BANK.ARFLT1_11H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_12L CANXL_1__FILTER_BANK.ARFLT1_12L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_12H CANXL_1__FILTER_BANK.ARFLT1_12H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_13L CANXL_1__FILTER_BANK.ARFLT1_13L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_13H CANXL_1__FILTER_BANK.ARFLT1_13H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_14L CANXL_1__FILTER_BANK.ARFLT1_14L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_14H CANXL_1__FILTER_BANK.ARFLT1_14H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_15L CANXL_1__FILTER_BANK.ARFLT1_15L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_15H CANXL_1__FILTER_BANK.ARFLT1_15H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_16L CANXL_1__FILTER_BANK.ARFLT1_16L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_16H CANXL_1__FILTER_BANK.ARFLT1_16H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_17L CANXL_1__FILTER_BANK.ARFLT1_17L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_17H CANXL_1__FILTER_BANK.ARFLT1_17H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_18L CANXL_1__FILTER_BANK.ARFLT1_18L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_18H CANXL_1__FILTER_BANK.ARFLT1_18H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_19L CANXL_1__FILTER_BANK.ARFLT1_19L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_19H CANXL_1__FILTER_BANK.ARFLT1_19H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_20L CANXL_1__FILTER_BANK.ARFLT1_20L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_20H CANXL_1__FILTER_BANK.ARFLT1_20H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_21L CANXL_1__FILTER_BANK.ARFLT1_21L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_21H CANXL_1__FILTER_BANK.ARFLT1_21H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_22L CANXL_1__FILTER_BANK.ARFLT1_22L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_22H CANXL_1__FILTER_BANK.ARFLT1_22H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_23L CANXL_1__FILTER_BANK.ARFLT1_23L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_23H CANXL_1__FILTER_BANK.ARFLT1_23H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_24L CANXL_1__FILTER_BANK.ARFLT1_24L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_24H CANXL_1__FILTER_BANK.ARFLT1_24H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_25L CANXL_1__FILTER_BANK.ARFLT1_25L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_25H CANXL_1__FILTER_BANK.ARFLT1_25H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_26L CANXL_1__FILTER_BANK.ARFLT1_26L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_26H CANXL_1__FILTER_BANK.ARFLT1_26H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_27L CANXL_1__FILTER_BANK.ARFLT1_27L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_27H CANXL_1__FILTER_BANK.ARFLT1_27H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_28L CANXL_1__FILTER_BANK.ARFLT1_28L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_28H CANXL_1__FILTER_BANK.ARFLT1_28H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_29L CANXL_1__FILTER_BANK.ARFLT1_29L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_29H CANXL_1__FILTER_BANK.ARFLT1_29H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_30L CANXL_1__FILTER_BANK.ARFLT1_30L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_30H CANXL_1__FILTER_BANK.ARFLT1_30H.R /* ADDR Rejection Filter High */
#define CANXL_1__FILTER_BANK_ARFLT1_31L CANXL_1__FILTER_BANK.ARFLT1_31L.R /* ADDR Rejection Filter Low */
#define CANXL_1__FILTER_BANK_ARFLT1_31H CANXL_1__FILTER_BANK.ARFLT1_31H.R /* ADDR Rejection Filter High */

/* CANXL_GRP_CONTROL */
#define CANXL_0__GRP_CONTROL_DSCCTRL CANXL_0__GRP_CONTROL.DSCCTRL.R /* Descriptors Control */
#define CANXL_0__GRP_CONTROL_DSCACTIVE1 CANXL_0__GRP_CONTROL.DSCACTIVE1.R /* Descriptor Activation 1 */
#define CANXL_0__GRP_CONTROL_DSCACTIVE2 CANXL_0__GRP_CONTROL.DSCACTIVE2.R /* Descriptor Activation 2 */
#define CANXL_0__GRP_CONTROL_DSCACTIVE3 CANXL_0__GRP_CONTROL.DSCACTIVE3.R /* Descriptor Activation 3 */
#define CANXL_0__GRP_CONTROL_DSCACTIVE4 CANXL_0__GRP_CONTROL.DSCACTIVE4.R /* Descriptor Activation 4 */
#define CANXL_0__GRP_CONTROL_FIFOCTRL1 CANXL_0__GRP_CONTROL.FIFOCTRL1.R /* Message FIFO Control 1 */
#define CANXL_0__GRP_CONTROL_FIFOCTRL2 CANXL_0__GRP_CONTROL.FIFOCTRL2.R /* Message FIFO Control 2 */
#define CANXL_0__GRP_CONTROL_FIFOCTRL3 CANXL_0__GRP_CONTROL.FIFOCTRL3.R /* Message FIFO Control 3 */
#define CANXL_0__GRP_CONTROL_FIFOCTRL4 CANXL_0__GRP_CONTROL.FIFOCTRL4.R /* Message FIFO Control 4 */
#define CANXL_0__GRP_CONTROL_MSGIFLAG1 CANXL_0__GRP_CONTROL.MSGIFLAG1.R /* Message Interrupt FLAG 1 */
#define CANXL_0__GRP_CONTROL_MSGIFLAG2 CANXL_0__GRP_CONTROL.MSGIFLAG2.R /* Message Interrupt FLAG 2 */
#define CANXL_0__GRP_CONTROL_MSGIFLAG3 CANXL_0__GRP_CONTROL.MSGIFLAG3.R /* Message Interrupt FLAG 3 */
#define CANXL_0__GRP_CONTROL_MSGIFLAG4 CANXL_0__GRP_CONTROL.MSGIFLAG4.R /* Message Interrupt FLAG 4 */
#define CANXL_0__GRP_CONTROL_MSGIMASK1 CANXL_0__GRP_CONTROL.MSGIMASK1.R /* Message Interrupt Mask 1 */
#define CANXL_0__GRP_CONTROL_MSGIMASK2 CANXL_0__GRP_CONTROL.MSGIMASK2.R /* Message Interrupt Mask 2 */
#define CANXL_0__GRP_CONTROL_MSGIMASK3 CANXL_0__GRP_CONTROL.MSGIMASK3.R /* Message Interrupt Mask 3 */
#define CANXL_0__GRP_CONTROL_MSGIMASK4 CANXL_0__GRP_CONTROL.MSGIMASK4.R /* Message Interrupt Mask 4 */
#define CANXL_0__GRP_CONTROL_FREEDSC0 CANXL_0__GRP_CONTROL.FREEDSC0.R /* Free Descriptor Index 0 */
#define CANXL_0__GRP_CONTROL_OVERRUNF1 CANXL_0__GRP_CONTROL.OVERRUNF1.R /* Descriptor Overrun Flag 1 */
#define CANXL_0__GRP_CONTROL_OVERRUNF2 CANXL_0__GRP_CONTROL.OVERRUNF2.R /* Descriptor Overrun Flag 2 */
#define CANXL_0__GRP_CONTROL_OVERRUNF3 CANXL_0__GRP_CONTROL.OVERRUNF3.R /* Descriptor Overrun Flag 3 */
#define CANXL_0__GRP_CONTROL_OVERRUNF4 CANXL_0__GRP_CONTROL.OVERRUNF4.R /* Descriptor Overrun Flag 4 */
#define CANXL_0__GRP_CONTROL_URUNF1 CANXL_0__GRP_CONTROL.URUNF1.R /* Under Run Flag 1 */
#define CANXL_0__GRP_CONTROL_URUNF2 CANXL_0__GRP_CONTROL.URUNF2.R /* Under Run Flag 2 */
#define CANXL_0__GRP_CONTROL_URUNF3 CANXL_0__GRP_CONTROL.URUNF3.R /* Under Run Flag 3 */
#define CANXL_0__GRP_CONTROL_URUNF4 CANXL_0__GRP_CONTROL.URUNF4.R /* Under Run Flag 4 */

#define CANXL_1__GRP_CONTROL_DSCCTRL CANXL_1__GRP_CONTROL.DSCCTRL.R /* Descriptors Control */
#define CANXL_1__GRP_CONTROL_DSCACTIVE1 CANXL_1__GRP_CONTROL.DSCACTIVE1.R /* Descriptor Activation 1 */
#define CANXL_1__GRP_CONTROL_DSCACTIVE2 CANXL_1__GRP_CONTROL.DSCACTIVE2.R /* Descriptor Activation 2 */
#define CANXL_1__GRP_CONTROL_DSCACTIVE3 CANXL_1__GRP_CONTROL.DSCACTIVE3.R /* Descriptor Activation 3 */
#define CANXL_1__GRP_CONTROL_DSCACTIVE4 CANXL_1__GRP_CONTROL.DSCACTIVE4.R /* Descriptor Activation 4 */
#define CANXL_1__GRP_CONTROL_FIFOCTRL1 CANXL_1__GRP_CONTROL.FIFOCTRL1.R /* Message FIFO Control 1 */
#define CANXL_1__GRP_CONTROL_FIFOCTRL2 CANXL_1__GRP_CONTROL.FIFOCTRL2.R /* Message FIFO Control 2 */
#define CANXL_1__GRP_CONTROL_FIFOCTRL3 CANXL_1__GRP_CONTROL.FIFOCTRL3.R /* Message FIFO Control 3 */
#define CANXL_1__GRP_CONTROL_FIFOCTRL4 CANXL_1__GRP_CONTROL.FIFOCTRL4.R /* Message FIFO Control 4 */
#define CANXL_1__GRP_CONTROL_MSGIFLAG1 CANXL_1__GRP_CONTROL.MSGIFLAG1.R /* Message Interrupt FLAG 1 */
#define CANXL_1__GRP_CONTROL_MSGIFLAG2 CANXL_1__GRP_CONTROL.MSGIFLAG2.R /* Message Interrupt FLAG 2 */
#define CANXL_1__GRP_CONTROL_MSGIFLAG3 CANXL_1__GRP_CONTROL.MSGIFLAG3.R /* Message Interrupt FLAG 3 */
#define CANXL_1__GRP_CONTROL_MSGIFLAG4 CANXL_1__GRP_CONTROL.MSGIFLAG4.R /* Message Interrupt FLAG 4 */
#define CANXL_1__GRP_CONTROL_MSGIMASK1 CANXL_1__GRP_CONTROL.MSGIMASK1.R /* Message Interrupt Mask 1 */
#define CANXL_1__GRP_CONTROL_MSGIMASK2 CANXL_1__GRP_CONTROL.MSGIMASK2.R /* Message Interrupt Mask 2 */
#define CANXL_1__GRP_CONTROL_MSGIMASK3 CANXL_1__GRP_CONTROL.MSGIMASK3.R /* Message Interrupt Mask 3 */
#define CANXL_1__GRP_CONTROL_MSGIMASK4 CANXL_1__GRP_CONTROL.MSGIMASK4.R /* Message Interrupt Mask 4 */
#define CANXL_1__GRP_CONTROL_FREEDSC0 CANXL_1__GRP_CONTROL.FREEDSC0.R /* Free Descriptor Index 0 */
#define CANXL_1__GRP_CONTROL_OVERRUNF1 CANXL_1__GRP_CONTROL.OVERRUNF1.R /* Descriptor Overrun Flag 1 */
#define CANXL_1__GRP_CONTROL_OVERRUNF2 CANXL_1__GRP_CONTROL.OVERRUNF2.R /* Descriptor Overrun Flag 2 */
#define CANXL_1__GRP_CONTROL_OVERRUNF3 CANXL_1__GRP_CONTROL.OVERRUNF3.R /* Descriptor Overrun Flag 3 */
#define CANXL_1__GRP_CONTROL_OVERRUNF4 CANXL_1__GRP_CONTROL.OVERRUNF4.R /* Descriptor Overrun Flag 4 */
#define CANXL_1__GRP_CONTROL_URUNF1 CANXL_1__GRP_CONTROL.URUNF1.R /* Under Run Flag 1 */
#define CANXL_1__GRP_CONTROL_URUNF2 CANXL_1__GRP_CONTROL.URUNF2.R /* Under Run Flag 2 */
#define CANXL_1__GRP_CONTROL_URUNF3 CANXL_1__GRP_CONTROL.URUNF3.R /* Under Run Flag 3 */
#define CANXL_1__GRP_CONTROL_URUNF4 CANXL_1__GRP_CONTROL.URUNF4.R /* Under Run Flag 4 */

/* CANXL_MRU */
#define CANXL_0__MRU_CH1_CFG0 CANXL_0__MRU.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CANXL_0__MRU_CH1_CFG1 CANXL_0__MRU.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CANXL_0__MRU_CH1_MBSTAT CANXL_0__MRU.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CANXL_0__MRU_NOTIFY0 CANXL_0__MRU.NOTIFY0.R        /* Notification 0 Status */
#define CANXL_0__MRU_NOTIFY1 CANXL_0__MRU.NOTIFY1.R        /* Notification 1 Status */
#define CANXL_0__MRU_CH1_MB0 CANXL_0__MRU.CH1_MB0.R        /* Channel (x) Mailbox (n) */
#define CANXL_0__MRU_CH1_MB1 CANXL_0__MRU.CH1_MB1.R        /* Channel (x) Mailbox (n) */
#define CANXL_0__MRU_CH1_MB2 CANXL_0__MRU.CH1_MB2.R        /* Channel (x) Mailbox (n) */
#define CANXL_0__MRU_CH1_MB3 CANXL_0__MRU.CH1_MB3.R        /* Channel (x) Mailbox (n) */

#define CANXL_1__MRU_CH1_CFG0 CANXL_1__MRU.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CANXL_1__MRU_CH1_CFG1 CANXL_1__MRU.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CANXL_1__MRU_CH1_MBSTAT CANXL_1__MRU.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CANXL_1__MRU_NOTIFY0 CANXL_1__MRU.NOTIFY0.R        /* Notification 0 Status */
#define CANXL_1__MRU_NOTIFY1 CANXL_1__MRU.NOTIFY1.R        /* Notification 1 Status */
#define CANXL_1__MRU_CH1_MB0 CANXL_1__MRU.CH1_MB0.R        /* Channel (x) Mailbox (n) */
#define CANXL_1__MRU_CH1_MB1 CANXL_1__MRU.CH1_MB1.R        /* Channel (x) Mailbox (n) */
#define CANXL_1__MRU_CH1_MB2 CANXL_1__MRU.CH1_MB2.R        /* Channel (x) Mailbox (n) */
#define CANXL_1__MRU_CH1_MB3 CANXL_1__MRU.CH1_MB3.R        /* Channel (x) Mailbox (n) */

/* CANXL_MSG_DESCRIPTORS */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_0 CANXL_0__MSG_DESCRIPTORS.MSGDSC[0].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_1 CANXL_0__MSG_DESCRIPTORS.MSGDSC[1].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_1 CANXL_0__MSG_DESCRIPTORS.MSGDSC[1].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_1 CANXL_0__MSG_DESCRIPTORS.MSGDSC[1].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_1 CANXL_0__MSG_DESCRIPTORS.MSGDSC[1].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_1 CANXL_0__MSG_DESCRIPTORS.MSGDSC[1].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_1 CANXL_0__MSG_DESCRIPTORS.MSGDSC[1].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_1 CANXL_0__MSG_DESCRIPTORS.MSGDSC[1].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_2 CANXL_0__MSG_DESCRIPTORS.MSGDSC[2].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_2 CANXL_0__MSG_DESCRIPTORS.MSGDSC[2].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_2 CANXL_0__MSG_DESCRIPTORS.MSGDSC[2].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_2 CANXL_0__MSG_DESCRIPTORS.MSGDSC[2].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_2 CANXL_0__MSG_DESCRIPTORS.MSGDSC[2].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_2 CANXL_0__MSG_DESCRIPTORS.MSGDSC[2].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_2 CANXL_0__MSG_DESCRIPTORS.MSGDSC[2].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_3 CANXL_0__MSG_DESCRIPTORS.MSGDSC[3].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_3 CANXL_0__MSG_DESCRIPTORS.MSGDSC[3].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_3 CANXL_0__MSG_DESCRIPTORS.MSGDSC[3].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_3 CANXL_0__MSG_DESCRIPTORS.MSGDSC[3].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_3 CANXL_0__MSG_DESCRIPTORS.MSGDSC[3].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_3 CANXL_0__MSG_DESCRIPTORS.MSGDSC[3].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_3 CANXL_0__MSG_DESCRIPTORS.MSGDSC[3].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_4 CANXL_0__MSG_DESCRIPTORS.MSGDSC[4].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_4 CANXL_0__MSG_DESCRIPTORS.MSGDSC[4].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_4 CANXL_0__MSG_DESCRIPTORS.MSGDSC[4].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_4 CANXL_0__MSG_DESCRIPTORS.MSGDSC[4].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_4 CANXL_0__MSG_DESCRIPTORS.MSGDSC[4].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_4 CANXL_0__MSG_DESCRIPTORS.MSGDSC[4].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_4 CANXL_0__MSG_DESCRIPTORS.MSGDSC[4].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_5 CANXL_0__MSG_DESCRIPTORS.MSGDSC[5].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_5 CANXL_0__MSG_DESCRIPTORS.MSGDSC[5].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_5 CANXL_0__MSG_DESCRIPTORS.MSGDSC[5].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_5 CANXL_0__MSG_DESCRIPTORS.MSGDSC[5].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_5 CANXL_0__MSG_DESCRIPTORS.MSGDSC[5].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_5 CANXL_0__MSG_DESCRIPTORS.MSGDSC[5].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_5 CANXL_0__MSG_DESCRIPTORS.MSGDSC[5].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_6 CANXL_0__MSG_DESCRIPTORS.MSGDSC[6].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_6 CANXL_0__MSG_DESCRIPTORS.MSGDSC[6].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_6 CANXL_0__MSG_DESCRIPTORS.MSGDSC[6].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_6 CANXL_0__MSG_DESCRIPTORS.MSGDSC[6].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_6 CANXL_0__MSG_DESCRIPTORS.MSGDSC[6].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_6 CANXL_0__MSG_DESCRIPTORS.MSGDSC[6].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_6 CANXL_0__MSG_DESCRIPTORS.MSGDSC[6].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_7 CANXL_0__MSG_DESCRIPTORS.MSGDSC[7].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_7 CANXL_0__MSG_DESCRIPTORS.MSGDSC[7].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_7 CANXL_0__MSG_DESCRIPTORS.MSGDSC[7].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_7 CANXL_0__MSG_DESCRIPTORS.MSGDSC[7].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_7 CANXL_0__MSG_DESCRIPTORS.MSGDSC[7].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_7 CANXL_0__MSG_DESCRIPTORS.MSGDSC[7].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_7 CANXL_0__MSG_DESCRIPTORS.MSGDSC[7].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_8 CANXL_0__MSG_DESCRIPTORS.MSGDSC[8].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_8 CANXL_0__MSG_DESCRIPTORS.MSGDSC[8].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_8 CANXL_0__MSG_DESCRIPTORS.MSGDSC[8].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_8 CANXL_0__MSG_DESCRIPTORS.MSGDSC[8].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_8 CANXL_0__MSG_DESCRIPTORS.MSGDSC[8].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_8 CANXL_0__MSG_DESCRIPTORS.MSGDSC[8].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_8 CANXL_0__MSG_DESCRIPTORS.MSGDSC[8].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_9 CANXL_0__MSG_DESCRIPTORS.MSGDSC[9].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_9 CANXL_0__MSG_DESCRIPTORS.MSGDSC[9].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_9 CANXL_0__MSG_DESCRIPTORS.MSGDSC[9].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_9 CANXL_0__MSG_DESCRIPTORS.MSGDSC[9].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_9 CANXL_0__MSG_DESCRIPTORS.MSGDSC[9].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_9 CANXL_0__MSG_DESCRIPTORS.MSGDSC[9].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_9 CANXL_0__MSG_DESCRIPTORS.MSGDSC[9].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_10 CANXL_0__MSG_DESCRIPTORS.MSGDSC[10].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_10 CANXL_0__MSG_DESCRIPTORS.MSGDSC[10].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_10 CANXL_0__MSG_DESCRIPTORS.MSGDSC[10].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_10 CANXL_0__MSG_DESCRIPTORS.MSGDSC[10].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_10 CANXL_0__MSG_DESCRIPTORS.MSGDSC[10].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_10 CANXL_0__MSG_DESCRIPTORS.MSGDSC[10].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_10 CANXL_0__MSG_DESCRIPTORS.MSGDSC[10].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_11 CANXL_0__MSG_DESCRIPTORS.MSGDSC[11].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_11 CANXL_0__MSG_DESCRIPTORS.MSGDSC[11].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_11 CANXL_0__MSG_DESCRIPTORS.MSGDSC[11].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_11 CANXL_0__MSG_DESCRIPTORS.MSGDSC[11].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_11 CANXL_0__MSG_DESCRIPTORS.MSGDSC[11].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_11 CANXL_0__MSG_DESCRIPTORS.MSGDSC[11].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_11 CANXL_0__MSG_DESCRIPTORS.MSGDSC[11].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_12 CANXL_0__MSG_DESCRIPTORS.MSGDSC[12].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_12 CANXL_0__MSG_DESCRIPTORS.MSGDSC[12].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_12 CANXL_0__MSG_DESCRIPTORS.MSGDSC[12].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_12 CANXL_0__MSG_DESCRIPTORS.MSGDSC[12].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_12 CANXL_0__MSG_DESCRIPTORS.MSGDSC[12].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_12 CANXL_0__MSG_DESCRIPTORS.MSGDSC[12].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_12 CANXL_0__MSG_DESCRIPTORS.MSGDSC[12].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_13 CANXL_0__MSG_DESCRIPTORS.MSGDSC[13].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_13 CANXL_0__MSG_DESCRIPTORS.MSGDSC[13].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_13 CANXL_0__MSG_DESCRIPTORS.MSGDSC[13].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_13 CANXL_0__MSG_DESCRIPTORS.MSGDSC[13].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_13 CANXL_0__MSG_DESCRIPTORS.MSGDSC[13].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_13 CANXL_0__MSG_DESCRIPTORS.MSGDSC[13].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_13 CANXL_0__MSG_DESCRIPTORS.MSGDSC[13].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_14 CANXL_0__MSG_DESCRIPTORS.MSGDSC[14].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_14 CANXL_0__MSG_DESCRIPTORS.MSGDSC[14].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_14 CANXL_0__MSG_DESCRIPTORS.MSGDSC[14].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_14 CANXL_0__MSG_DESCRIPTORS.MSGDSC[14].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_14 CANXL_0__MSG_DESCRIPTORS.MSGDSC[14].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_14 CANXL_0__MSG_DESCRIPTORS.MSGDSC[14].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_14 CANXL_0__MSG_DESCRIPTORS.MSGDSC[14].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_15 CANXL_0__MSG_DESCRIPTORS.MSGDSC[15].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_15 CANXL_0__MSG_DESCRIPTORS.MSGDSC[15].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_15 CANXL_0__MSG_DESCRIPTORS.MSGDSC[15].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_15 CANXL_0__MSG_DESCRIPTORS.MSGDSC[15].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_15 CANXL_0__MSG_DESCRIPTORS.MSGDSC[15].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_15 CANXL_0__MSG_DESCRIPTORS.MSGDSC[15].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_15 CANXL_0__MSG_DESCRIPTORS.MSGDSC[15].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_16 CANXL_0__MSG_DESCRIPTORS.MSGDSC[16].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_16 CANXL_0__MSG_DESCRIPTORS.MSGDSC[16].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_16 CANXL_0__MSG_DESCRIPTORS.MSGDSC[16].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_16 CANXL_0__MSG_DESCRIPTORS.MSGDSC[16].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_16 CANXL_0__MSG_DESCRIPTORS.MSGDSC[16].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_16 CANXL_0__MSG_DESCRIPTORS.MSGDSC[16].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_16 CANXL_0__MSG_DESCRIPTORS.MSGDSC[16].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_17 CANXL_0__MSG_DESCRIPTORS.MSGDSC[17].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_17 CANXL_0__MSG_DESCRIPTORS.MSGDSC[17].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_17 CANXL_0__MSG_DESCRIPTORS.MSGDSC[17].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_17 CANXL_0__MSG_DESCRIPTORS.MSGDSC[17].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_17 CANXL_0__MSG_DESCRIPTORS.MSGDSC[17].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_17 CANXL_0__MSG_DESCRIPTORS.MSGDSC[17].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_17 CANXL_0__MSG_DESCRIPTORS.MSGDSC[17].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_18 CANXL_0__MSG_DESCRIPTORS.MSGDSC[18].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_18 CANXL_0__MSG_DESCRIPTORS.MSGDSC[18].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_18 CANXL_0__MSG_DESCRIPTORS.MSGDSC[18].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_18 CANXL_0__MSG_DESCRIPTORS.MSGDSC[18].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_18 CANXL_0__MSG_DESCRIPTORS.MSGDSC[18].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_18 CANXL_0__MSG_DESCRIPTORS.MSGDSC[18].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_18 CANXL_0__MSG_DESCRIPTORS.MSGDSC[18].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_19 CANXL_0__MSG_DESCRIPTORS.MSGDSC[19].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_19 CANXL_0__MSG_DESCRIPTORS.MSGDSC[19].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_19 CANXL_0__MSG_DESCRIPTORS.MSGDSC[19].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_19 CANXL_0__MSG_DESCRIPTORS.MSGDSC[19].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_19 CANXL_0__MSG_DESCRIPTORS.MSGDSC[19].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_19 CANXL_0__MSG_DESCRIPTORS.MSGDSC[19].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_19 CANXL_0__MSG_DESCRIPTORS.MSGDSC[19].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_20 CANXL_0__MSG_DESCRIPTORS.MSGDSC[20].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_20 CANXL_0__MSG_DESCRIPTORS.MSGDSC[20].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_20 CANXL_0__MSG_DESCRIPTORS.MSGDSC[20].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_20 CANXL_0__MSG_DESCRIPTORS.MSGDSC[20].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_20 CANXL_0__MSG_DESCRIPTORS.MSGDSC[20].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_20 CANXL_0__MSG_DESCRIPTORS.MSGDSC[20].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_20 CANXL_0__MSG_DESCRIPTORS.MSGDSC[20].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_21 CANXL_0__MSG_DESCRIPTORS.MSGDSC[21].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_21 CANXL_0__MSG_DESCRIPTORS.MSGDSC[21].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_21 CANXL_0__MSG_DESCRIPTORS.MSGDSC[21].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_21 CANXL_0__MSG_DESCRIPTORS.MSGDSC[21].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_21 CANXL_0__MSG_DESCRIPTORS.MSGDSC[21].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_21 CANXL_0__MSG_DESCRIPTORS.MSGDSC[21].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_21 CANXL_0__MSG_DESCRIPTORS.MSGDSC[21].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_22 CANXL_0__MSG_DESCRIPTORS.MSGDSC[22].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_22 CANXL_0__MSG_DESCRIPTORS.MSGDSC[22].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_22 CANXL_0__MSG_DESCRIPTORS.MSGDSC[22].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_22 CANXL_0__MSG_DESCRIPTORS.MSGDSC[22].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_22 CANXL_0__MSG_DESCRIPTORS.MSGDSC[22].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_22 CANXL_0__MSG_DESCRIPTORS.MSGDSC[22].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_22 CANXL_0__MSG_DESCRIPTORS.MSGDSC[22].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_23 CANXL_0__MSG_DESCRIPTORS.MSGDSC[23].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_23 CANXL_0__MSG_DESCRIPTORS.MSGDSC[23].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_23 CANXL_0__MSG_DESCRIPTORS.MSGDSC[23].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_23 CANXL_0__MSG_DESCRIPTORS.MSGDSC[23].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_23 CANXL_0__MSG_DESCRIPTORS.MSGDSC[23].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_23 CANXL_0__MSG_DESCRIPTORS.MSGDSC[23].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_23 CANXL_0__MSG_DESCRIPTORS.MSGDSC[23].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_24 CANXL_0__MSG_DESCRIPTORS.MSGDSC[24].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_24 CANXL_0__MSG_DESCRIPTORS.MSGDSC[24].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_24 CANXL_0__MSG_DESCRIPTORS.MSGDSC[24].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_24 CANXL_0__MSG_DESCRIPTORS.MSGDSC[24].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_24 CANXL_0__MSG_DESCRIPTORS.MSGDSC[24].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_24 CANXL_0__MSG_DESCRIPTORS.MSGDSC[24].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_24 CANXL_0__MSG_DESCRIPTORS.MSGDSC[24].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_25 CANXL_0__MSG_DESCRIPTORS.MSGDSC[25].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_25 CANXL_0__MSG_DESCRIPTORS.MSGDSC[25].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_25 CANXL_0__MSG_DESCRIPTORS.MSGDSC[25].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_25 CANXL_0__MSG_DESCRIPTORS.MSGDSC[25].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_25 CANXL_0__MSG_DESCRIPTORS.MSGDSC[25].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_25 CANXL_0__MSG_DESCRIPTORS.MSGDSC[25].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_25 CANXL_0__MSG_DESCRIPTORS.MSGDSC[25].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_26 CANXL_0__MSG_DESCRIPTORS.MSGDSC[26].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_26 CANXL_0__MSG_DESCRIPTORS.MSGDSC[26].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_26 CANXL_0__MSG_DESCRIPTORS.MSGDSC[26].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_26 CANXL_0__MSG_DESCRIPTORS.MSGDSC[26].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_26 CANXL_0__MSG_DESCRIPTORS.MSGDSC[26].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_26 CANXL_0__MSG_DESCRIPTORS.MSGDSC[26].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_26 CANXL_0__MSG_DESCRIPTORS.MSGDSC[26].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_27 CANXL_0__MSG_DESCRIPTORS.MSGDSC[27].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_27 CANXL_0__MSG_DESCRIPTORS.MSGDSC[27].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_27 CANXL_0__MSG_DESCRIPTORS.MSGDSC[27].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_27 CANXL_0__MSG_DESCRIPTORS.MSGDSC[27].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_27 CANXL_0__MSG_DESCRIPTORS.MSGDSC[27].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_27 CANXL_0__MSG_DESCRIPTORS.MSGDSC[27].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_27 CANXL_0__MSG_DESCRIPTORS.MSGDSC[27].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_28 CANXL_0__MSG_DESCRIPTORS.MSGDSC[28].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_28 CANXL_0__MSG_DESCRIPTORS.MSGDSC[28].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_28 CANXL_0__MSG_DESCRIPTORS.MSGDSC[28].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_28 CANXL_0__MSG_DESCRIPTORS.MSGDSC[28].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_28 CANXL_0__MSG_DESCRIPTORS.MSGDSC[28].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_28 CANXL_0__MSG_DESCRIPTORS.MSGDSC[28].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_28 CANXL_0__MSG_DESCRIPTORS.MSGDSC[28].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_29 CANXL_0__MSG_DESCRIPTORS.MSGDSC[29].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_29 CANXL_0__MSG_DESCRIPTORS.MSGDSC[29].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_29 CANXL_0__MSG_DESCRIPTORS.MSGDSC[29].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_29 CANXL_0__MSG_DESCRIPTORS.MSGDSC[29].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_29 CANXL_0__MSG_DESCRIPTORS.MSGDSC[29].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_29 CANXL_0__MSG_DESCRIPTORS.MSGDSC[29].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_29 CANXL_0__MSG_DESCRIPTORS.MSGDSC[29].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_30 CANXL_0__MSG_DESCRIPTORS.MSGDSC[30].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_30 CANXL_0__MSG_DESCRIPTORS.MSGDSC[30].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_30 CANXL_0__MSG_DESCRIPTORS.MSGDSC[30].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_30 CANXL_0__MSG_DESCRIPTORS.MSGDSC[30].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_30 CANXL_0__MSG_DESCRIPTORS.MSGDSC[30].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_30 CANXL_0__MSG_DESCRIPTORS.MSGDSC[30].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_30 CANXL_0__MSG_DESCRIPTORS.MSGDSC[30].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_31 CANXL_0__MSG_DESCRIPTORS.MSGDSC[31].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_31 CANXL_0__MSG_DESCRIPTORS.MSGDSC[31].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_31 CANXL_0__MSG_DESCRIPTORS.MSGDSC[31].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_31 CANXL_0__MSG_DESCRIPTORS.MSGDSC[31].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_31 CANXL_0__MSG_DESCRIPTORS.MSGDSC[31].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_31 CANXL_0__MSG_DESCRIPTORS.MSGDSC[31].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_31 CANXL_0__MSG_DESCRIPTORS.MSGDSC[31].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_32 CANXL_0__MSG_DESCRIPTORS.MSGDSC[32].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_32 CANXL_0__MSG_DESCRIPTORS.MSGDSC[32].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_32 CANXL_0__MSG_DESCRIPTORS.MSGDSC[32].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_32 CANXL_0__MSG_DESCRIPTORS.MSGDSC[32].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_32 CANXL_0__MSG_DESCRIPTORS.MSGDSC[32].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_32 CANXL_0__MSG_DESCRIPTORS.MSGDSC[32].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_32 CANXL_0__MSG_DESCRIPTORS.MSGDSC[32].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_33 CANXL_0__MSG_DESCRIPTORS.MSGDSC[33].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_33 CANXL_0__MSG_DESCRIPTORS.MSGDSC[33].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_33 CANXL_0__MSG_DESCRIPTORS.MSGDSC[33].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_33 CANXL_0__MSG_DESCRIPTORS.MSGDSC[33].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_33 CANXL_0__MSG_DESCRIPTORS.MSGDSC[33].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_33 CANXL_0__MSG_DESCRIPTORS.MSGDSC[33].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_33 CANXL_0__MSG_DESCRIPTORS.MSGDSC[33].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_34 CANXL_0__MSG_DESCRIPTORS.MSGDSC[34].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_34 CANXL_0__MSG_DESCRIPTORS.MSGDSC[34].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_34 CANXL_0__MSG_DESCRIPTORS.MSGDSC[34].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_34 CANXL_0__MSG_DESCRIPTORS.MSGDSC[34].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_34 CANXL_0__MSG_DESCRIPTORS.MSGDSC[34].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_34 CANXL_0__MSG_DESCRIPTORS.MSGDSC[34].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_34 CANXL_0__MSG_DESCRIPTORS.MSGDSC[34].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_35 CANXL_0__MSG_DESCRIPTORS.MSGDSC[35].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_35 CANXL_0__MSG_DESCRIPTORS.MSGDSC[35].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_35 CANXL_0__MSG_DESCRIPTORS.MSGDSC[35].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_35 CANXL_0__MSG_DESCRIPTORS.MSGDSC[35].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_35 CANXL_0__MSG_DESCRIPTORS.MSGDSC[35].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_35 CANXL_0__MSG_DESCRIPTORS.MSGDSC[35].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_35 CANXL_0__MSG_DESCRIPTORS.MSGDSC[35].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_36 CANXL_0__MSG_DESCRIPTORS.MSGDSC[36].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_36 CANXL_0__MSG_DESCRIPTORS.MSGDSC[36].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_36 CANXL_0__MSG_DESCRIPTORS.MSGDSC[36].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_36 CANXL_0__MSG_DESCRIPTORS.MSGDSC[36].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_36 CANXL_0__MSG_DESCRIPTORS.MSGDSC[36].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_36 CANXL_0__MSG_DESCRIPTORS.MSGDSC[36].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_36 CANXL_0__MSG_DESCRIPTORS.MSGDSC[36].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_37 CANXL_0__MSG_DESCRIPTORS.MSGDSC[37].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_37 CANXL_0__MSG_DESCRIPTORS.MSGDSC[37].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_37 CANXL_0__MSG_DESCRIPTORS.MSGDSC[37].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_37 CANXL_0__MSG_DESCRIPTORS.MSGDSC[37].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_37 CANXL_0__MSG_DESCRIPTORS.MSGDSC[37].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_37 CANXL_0__MSG_DESCRIPTORS.MSGDSC[37].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_37 CANXL_0__MSG_DESCRIPTORS.MSGDSC[37].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_38 CANXL_0__MSG_DESCRIPTORS.MSGDSC[38].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_38 CANXL_0__MSG_DESCRIPTORS.MSGDSC[38].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_38 CANXL_0__MSG_DESCRIPTORS.MSGDSC[38].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_38 CANXL_0__MSG_DESCRIPTORS.MSGDSC[38].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_38 CANXL_0__MSG_DESCRIPTORS.MSGDSC[38].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_38 CANXL_0__MSG_DESCRIPTORS.MSGDSC[38].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_38 CANXL_0__MSG_DESCRIPTORS.MSGDSC[38].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_39 CANXL_0__MSG_DESCRIPTORS.MSGDSC[39].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_39 CANXL_0__MSG_DESCRIPTORS.MSGDSC[39].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_39 CANXL_0__MSG_DESCRIPTORS.MSGDSC[39].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_39 CANXL_0__MSG_DESCRIPTORS.MSGDSC[39].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_39 CANXL_0__MSG_DESCRIPTORS.MSGDSC[39].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_39 CANXL_0__MSG_DESCRIPTORS.MSGDSC[39].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_39 CANXL_0__MSG_DESCRIPTORS.MSGDSC[39].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_40 CANXL_0__MSG_DESCRIPTORS.MSGDSC[40].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_40 CANXL_0__MSG_DESCRIPTORS.MSGDSC[40].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_40 CANXL_0__MSG_DESCRIPTORS.MSGDSC[40].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_40 CANXL_0__MSG_DESCRIPTORS.MSGDSC[40].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_40 CANXL_0__MSG_DESCRIPTORS.MSGDSC[40].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_40 CANXL_0__MSG_DESCRIPTORS.MSGDSC[40].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_40 CANXL_0__MSG_DESCRIPTORS.MSGDSC[40].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_41 CANXL_0__MSG_DESCRIPTORS.MSGDSC[41].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_41 CANXL_0__MSG_DESCRIPTORS.MSGDSC[41].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_41 CANXL_0__MSG_DESCRIPTORS.MSGDSC[41].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_41 CANXL_0__MSG_DESCRIPTORS.MSGDSC[41].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_41 CANXL_0__MSG_DESCRIPTORS.MSGDSC[41].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_41 CANXL_0__MSG_DESCRIPTORS.MSGDSC[41].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_41 CANXL_0__MSG_DESCRIPTORS.MSGDSC[41].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_42 CANXL_0__MSG_DESCRIPTORS.MSGDSC[42].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_42 CANXL_0__MSG_DESCRIPTORS.MSGDSC[42].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_42 CANXL_0__MSG_DESCRIPTORS.MSGDSC[42].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_42 CANXL_0__MSG_DESCRIPTORS.MSGDSC[42].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_42 CANXL_0__MSG_DESCRIPTORS.MSGDSC[42].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_42 CANXL_0__MSG_DESCRIPTORS.MSGDSC[42].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_42 CANXL_0__MSG_DESCRIPTORS.MSGDSC[42].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_43 CANXL_0__MSG_DESCRIPTORS.MSGDSC[43].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_43 CANXL_0__MSG_DESCRIPTORS.MSGDSC[43].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_43 CANXL_0__MSG_DESCRIPTORS.MSGDSC[43].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_43 CANXL_0__MSG_DESCRIPTORS.MSGDSC[43].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_43 CANXL_0__MSG_DESCRIPTORS.MSGDSC[43].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_43 CANXL_0__MSG_DESCRIPTORS.MSGDSC[43].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_43 CANXL_0__MSG_DESCRIPTORS.MSGDSC[43].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_44 CANXL_0__MSG_DESCRIPTORS.MSGDSC[44].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_44 CANXL_0__MSG_DESCRIPTORS.MSGDSC[44].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_44 CANXL_0__MSG_DESCRIPTORS.MSGDSC[44].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_44 CANXL_0__MSG_DESCRIPTORS.MSGDSC[44].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_44 CANXL_0__MSG_DESCRIPTORS.MSGDSC[44].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_44 CANXL_0__MSG_DESCRIPTORS.MSGDSC[44].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_44 CANXL_0__MSG_DESCRIPTORS.MSGDSC[44].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_45 CANXL_0__MSG_DESCRIPTORS.MSGDSC[45].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_45 CANXL_0__MSG_DESCRIPTORS.MSGDSC[45].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_45 CANXL_0__MSG_DESCRIPTORS.MSGDSC[45].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_45 CANXL_0__MSG_DESCRIPTORS.MSGDSC[45].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_45 CANXL_0__MSG_DESCRIPTORS.MSGDSC[45].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_45 CANXL_0__MSG_DESCRIPTORS.MSGDSC[45].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_45 CANXL_0__MSG_DESCRIPTORS.MSGDSC[45].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_46 CANXL_0__MSG_DESCRIPTORS.MSGDSC[46].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_46 CANXL_0__MSG_DESCRIPTORS.MSGDSC[46].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_46 CANXL_0__MSG_DESCRIPTORS.MSGDSC[46].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_46 CANXL_0__MSG_DESCRIPTORS.MSGDSC[46].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_46 CANXL_0__MSG_DESCRIPTORS.MSGDSC[46].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_46 CANXL_0__MSG_DESCRIPTORS.MSGDSC[46].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_46 CANXL_0__MSG_DESCRIPTORS.MSGDSC[46].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_47 CANXL_0__MSG_DESCRIPTORS.MSGDSC[47].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_47 CANXL_0__MSG_DESCRIPTORS.MSGDSC[47].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_47 CANXL_0__MSG_DESCRIPTORS.MSGDSC[47].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_47 CANXL_0__MSG_DESCRIPTORS.MSGDSC[47].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_47 CANXL_0__MSG_DESCRIPTORS.MSGDSC[47].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_47 CANXL_0__MSG_DESCRIPTORS.MSGDSC[47].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_47 CANXL_0__MSG_DESCRIPTORS.MSGDSC[47].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_48 CANXL_0__MSG_DESCRIPTORS.MSGDSC[48].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_48 CANXL_0__MSG_DESCRIPTORS.MSGDSC[48].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_48 CANXL_0__MSG_DESCRIPTORS.MSGDSC[48].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_48 CANXL_0__MSG_DESCRIPTORS.MSGDSC[48].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_48 CANXL_0__MSG_DESCRIPTORS.MSGDSC[48].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_48 CANXL_0__MSG_DESCRIPTORS.MSGDSC[48].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_48 CANXL_0__MSG_DESCRIPTORS.MSGDSC[48].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_49 CANXL_0__MSG_DESCRIPTORS.MSGDSC[49].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_49 CANXL_0__MSG_DESCRIPTORS.MSGDSC[49].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_49 CANXL_0__MSG_DESCRIPTORS.MSGDSC[49].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_49 CANXL_0__MSG_DESCRIPTORS.MSGDSC[49].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_49 CANXL_0__MSG_DESCRIPTORS.MSGDSC[49].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_49 CANXL_0__MSG_DESCRIPTORS.MSGDSC[49].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_49 CANXL_0__MSG_DESCRIPTORS.MSGDSC[49].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_50 CANXL_0__MSG_DESCRIPTORS.MSGDSC[50].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_50 CANXL_0__MSG_DESCRIPTORS.MSGDSC[50].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_50 CANXL_0__MSG_DESCRIPTORS.MSGDSC[50].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_50 CANXL_0__MSG_DESCRIPTORS.MSGDSC[50].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_50 CANXL_0__MSG_DESCRIPTORS.MSGDSC[50].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_50 CANXL_0__MSG_DESCRIPTORS.MSGDSC[50].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_50 CANXL_0__MSG_DESCRIPTORS.MSGDSC[50].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_51 CANXL_0__MSG_DESCRIPTORS.MSGDSC[51].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_51 CANXL_0__MSG_DESCRIPTORS.MSGDSC[51].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_51 CANXL_0__MSG_DESCRIPTORS.MSGDSC[51].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_51 CANXL_0__MSG_DESCRIPTORS.MSGDSC[51].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_51 CANXL_0__MSG_DESCRIPTORS.MSGDSC[51].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_51 CANXL_0__MSG_DESCRIPTORS.MSGDSC[51].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_51 CANXL_0__MSG_DESCRIPTORS.MSGDSC[51].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_52 CANXL_0__MSG_DESCRIPTORS.MSGDSC[52].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_52 CANXL_0__MSG_DESCRIPTORS.MSGDSC[52].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_52 CANXL_0__MSG_DESCRIPTORS.MSGDSC[52].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_52 CANXL_0__MSG_DESCRIPTORS.MSGDSC[52].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_52 CANXL_0__MSG_DESCRIPTORS.MSGDSC[52].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_52 CANXL_0__MSG_DESCRIPTORS.MSGDSC[52].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_52 CANXL_0__MSG_DESCRIPTORS.MSGDSC[52].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_53 CANXL_0__MSG_DESCRIPTORS.MSGDSC[53].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_53 CANXL_0__MSG_DESCRIPTORS.MSGDSC[53].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_53 CANXL_0__MSG_DESCRIPTORS.MSGDSC[53].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_53 CANXL_0__MSG_DESCRIPTORS.MSGDSC[53].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_53 CANXL_0__MSG_DESCRIPTORS.MSGDSC[53].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_53 CANXL_0__MSG_DESCRIPTORS.MSGDSC[53].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_53 CANXL_0__MSG_DESCRIPTORS.MSGDSC[53].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_54 CANXL_0__MSG_DESCRIPTORS.MSGDSC[54].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_54 CANXL_0__MSG_DESCRIPTORS.MSGDSC[54].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_54 CANXL_0__MSG_DESCRIPTORS.MSGDSC[54].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_54 CANXL_0__MSG_DESCRIPTORS.MSGDSC[54].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_54 CANXL_0__MSG_DESCRIPTORS.MSGDSC[54].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_54 CANXL_0__MSG_DESCRIPTORS.MSGDSC[54].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_54 CANXL_0__MSG_DESCRIPTORS.MSGDSC[54].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_55 CANXL_0__MSG_DESCRIPTORS.MSGDSC[55].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_55 CANXL_0__MSG_DESCRIPTORS.MSGDSC[55].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_55 CANXL_0__MSG_DESCRIPTORS.MSGDSC[55].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_55 CANXL_0__MSG_DESCRIPTORS.MSGDSC[55].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_55 CANXL_0__MSG_DESCRIPTORS.MSGDSC[55].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_55 CANXL_0__MSG_DESCRIPTORS.MSGDSC[55].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_55 CANXL_0__MSG_DESCRIPTORS.MSGDSC[55].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_56 CANXL_0__MSG_DESCRIPTORS.MSGDSC[56].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_56 CANXL_0__MSG_DESCRIPTORS.MSGDSC[56].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_56 CANXL_0__MSG_DESCRIPTORS.MSGDSC[56].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_56 CANXL_0__MSG_DESCRIPTORS.MSGDSC[56].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_56 CANXL_0__MSG_DESCRIPTORS.MSGDSC[56].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_56 CANXL_0__MSG_DESCRIPTORS.MSGDSC[56].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_56 CANXL_0__MSG_DESCRIPTORS.MSGDSC[56].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_57 CANXL_0__MSG_DESCRIPTORS.MSGDSC[57].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_57 CANXL_0__MSG_DESCRIPTORS.MSGDSC[57].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_57 CANXL_0__MSG_DESCRIPTORS.MSGDSC[57].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_57 CANXL_0__MSG_DESCRIPTORS.MSGDSC[57].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_57 CANXL_0__MSG_DESCRIPTORS.MSGDSC[57].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_57 CANXL_0__MSG_DESCRIPTORS.MSGDSC[57].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_57 CANXL_0__MSG_DESCRIPTORS.MSGDSC[57].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_58 CANXL_0__MSG_DESCRIPTORS.MSGDSC[58].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_58 CANXL_0__MSG_DESCRIPTORS.MSGDSC[58].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_58 CANXL_0__MSG_DESCRIPTORS.MSGDSC[58].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_58 CANXL_0__MSG_DESCRIPTORS.MSGDSC[58].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_58 CANXL_0__MSG_DESCRIPTORS.MSGDSC[58].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_58 CANXL_0__MSG_DESCRIPTORS.MSGDSC[58].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_58 CANXL_0__MSG_DESCRIPTORS.MSGDSC[58].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_59 CANXL_0__MSG_DESCRIPTORS.MSGDSC[59].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_59 CANXL_0__MSG_DESCRIPTORS.MSGDSC[59].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_59 CANXL_0__MSG_DESCRIPTORS.MSGDSC[59].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_59 CANXL_0__MSG_DESCRIPTORS.MSGDSC[59].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_59 CANXL_0__MSG_DESCRIPTORS.MSGDSC[59].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_59 CANXL_0__MSG_DESCRIPTORS.MSGDSC[59].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_59 CANXL_0__MSG_DESCRIPTORS.MSGDSC[59].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_60 CANXL_0__MSG_DESCRIPTORS.MSGDSC[60].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_60 CANXL_0__MSG_DESCRIPTORS.MSGDSC[60].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_60 CANXL_0__MSG_DESCRIPTORS.MSGDSC[60].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_60 CANXL_0__MSG_DESCRIPTORS.MSGDSC[60].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_60 CANXL_0__MSG_DESCRIPTORS.MSGDSC[60].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_60 CANXL_0__MSG_DESCRIPTORS.MSGDSC[60].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_60 CANXL_0__MSG_DESCRIPTORS.MSGDSC[60].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_61 CANXL_0__MSG_DESCRIPTORS.MSGDSC[61].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_61 CANXL_0__MSG_DESCRIPTORS.MSGDSC[61].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_61 CANXL_0__MSG_DESCRIPTORS.MSGDSC[61].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_61 CANXL_0__MSG_DESCRIPTORS.MSGDSC[61].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_61 CANXL_0__MSG_DESCRIPTORS.MSGDSC[61].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_61 CANXL_0__MSG_DESCRIPTORS.MSGDSC[61].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_61 CANXL_0__MSG_DESCRIPTORS.MSGDSC[61].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_62 CANXL_0__MSG_DESCRIPTORS.MSGDSC[62].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_62 CANXL_0__MSG_DESCRIPTORS.MSGDSC[62].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_62 CANXL_0__MSG_DESCRIPTORS.MSGDSC[62].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_62 CANXL_0__MSG_DESCRIPTORS.MSGDSC[62].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_62 CANXL_0__MSG_DESCRIPTORS.MSGDSC[62].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_62 CANXL_0__MSG_DESCRIPTORS.MSGDSC[62].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_62 CANXL_0__MSG_DESCRIPTORS.MSGDSC[62].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_63 CANXL_0__MSG_DESCRIPTORS.MSGDSC[63].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_63 CANXL_0__MSG_DESCRIPTORS.MSGDSC[63].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_63 CANXL_0__MSG_DESCRIPTORS.MSGDSC[63].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_63 CANXL_0__MSG_DESCRIPTORS.MSGDSC[63].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_63 CANXL_0__MSG_DESCRIPTORS.MSGDSC[63].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_63 CANXL_0__MSG_DESCRIPTORS.MSGDSC[63].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_63 CANXL_0__MSG_DESCRIPTORS.MSGDSC[63].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_64 CANXL_0__MSG_DESCRIPTORS.MSGDSC[64].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_64 CANXL_0__MSG_DESCRIPTORS.MSGDSC[64].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_64 CANXL_0__MSG_DESCRIPTORS.MSGDSC[64].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_64 CANXL_0__MSG_DESCRIPTORS.MSGDSC[64].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_64 CANXL_0__MSG_DESCRIPTORS.MSGDSC[64].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_64 CANXL_0__MSG_DESCRIPTORS.MSGDSC[64].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_64 CANXL_0__MSG_DESCRIPTORS.MSGDSC[64].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_65 CANXL_0__MSG_DESCRIPTORS.MSGDSC[65].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_65 CANXL_0__MSG_DESCRIPTORS.MSGDSC[65].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_65 CANXL_0__MSG_DESCRIPTORS.MSGDSC[65].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_65 CANXL_0__MSG_DESCRIPTORS.MSGDSC[65].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_65 CANXL_0__MSG_DESCRIPTORS.MSGDSC[65].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_65 CANXL_0__MSG_DESCRIPTORS.MSGDSC[65].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_65 CANXL_0__MSG_DESCRIPTORS.MSGDSC[65].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_66 CANXL_0__MSG_DESCRIPTORS.MSGDSC[66].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_66 CANXL_0__MSG_DESCRIPTORS.MSGDSC[66].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_66 CANXL_0__MSG_DESCRIPTORS.MSGDSC[66].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_66 CANXL_0__MSG_DESCRIPTORS.MSGDSC[66].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_66 CANXL_0__MSG_DESCRIPTORS.MSGDSC[66].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_66 CANXL_0__MSG_DESCRIPTORS.MSGDSC[66].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_66 CANXL_0__MSG_DESCRIPTORS.MSGDSC[66].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_67 CANXL_0__MSG_DESCRIPTORS.MSGDSC[67].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_67 CANXL_0__MSG_DESCRIPTORS.MSGDSC[67].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_67 CANXL_0__MSG_DESCRIPTORS.MSGDSC[67].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_67 CANXL_0__MSG_DESCRIPTORS.MSGDSC[67].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_67 CANXL_0__MSG_DESCRIPTORS.MSGDSC[67].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_67 CANXL_0__MSG_DESCRIPTORS.MSGDSC[67].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_67 CANXL_0__MSG_DESCRIPTORS.MSGDSC[67].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_68 CANXL_0__MSG_DESCRIPTORS.MSGDSC[68].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_68 CANXL_0__MSG_DESCRIPTORS.MSGDSC[68].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_68 CANXL_0__MSG_DESCRIPTORS.MSGDSC[68].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_68 CANXL_0__MSG_DESCRIPTORS.MSGDSC[68].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_68 CANXL_0__MSG_DESCRIPTORS.MSGDSC[68].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_68 CANXL_0__MSG_DESCRIPTORS.MSGDSC[68].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_68 CANXL_0__MSG_DESCRIPTORS.MSGDSC[68].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_69 CANXL_0__MSG_DESCRIPTORS.MSGDSC[69].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_69 CANXL_0__MSG_DESCRIPTORS.MSGDSC[69].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_69 CANXL_0__MSG_DESCRIPTORS.MSGDSC[69].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_69 CANXL_0__MSG_DESCRIPTORS.MSGDSC[69].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_69 CANXL_0__MSG_DESCRIPTORS.MSGDSC[69].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_69 CANXL_0__MSG_DESCRIPTORS.MSGDSC[69].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_69 CANXL_0__MSG_DESCRIPTORS.MSGDSC[69].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_70 CANXL_0__MSG_DESCRIPTORS.MSGDSC[70].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_70 CANXL_0__MSG_DESCRIPTORS.MSGDSC[70].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_70 CANXL_0__MSG_DESCRIPTORS.MSGDSC[70].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_70 CANXL_0__MSG_DESCRIPTORS.MSGDSC[70].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_70 CANXL_0__MSG_DESCRIPTORS.MSGDSC[70].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_70 CANXL_0__MSG_DESCRIPTORS.MSGDSC[70].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_70 CANXL_0__MSG_DESCRIPTORS.MSGDSC[70].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_71 CANXL_0__MSG_DESCRIPTORS.MSGDSC[71].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_71 CANXL_0__MSG_DESCRIPTORS.MSGDSC[71].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_71 CANXL_0__MSG_DESCRIPTORS.MSGDSC[71].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_71 CANXL_0__MSG_DESCRIPTORS.MSGDSC[71].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_71 CANXL_0__MSG_DESCRIPTORS.MSGDSC[71].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_71 CANXL_0__MSG_DESCRIPTORS.MSGDSC[71].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_71 CANXL_0__MSG_DESCRIPTORS.MSGDSC[71].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_72 CANXL_0__MSG_DESCRIPTORS.MSGDSC[72].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_72 CANXL_0__MSG_DESCRIPTORS.MSGDSC[72].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_72 CANXL_0__MSG_DESCRIPTORS.MSGDSC[72].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_72 CANXL_0__MSG_DESCRIPTORS.MSGDSC[72].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_72 CANXL_0__MSG_DESCRIPTORS.MSGDSC[72].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_72 CANXL_0__MSG_DESCRIPTORS.MSGDSC[72].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_72 CANXL_0__MSG_DESCRIPTORS.MSGDSC[72].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_73 CANXL_0__MSG_DESCRIPTORS.MSGDSC[73].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_73 CANXL_0__MSG_DESCRIPTORS.MSGDSC[73].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_73 CANXL_0__MSG_DESCRIPTORS.MSGDSC[73].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_73 CANXL_0__MSG_DESCRIPTORS.MSGDSC[73].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_73 CANXL_0__MSG_DESCRIPTORS.MSGDSC[73].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_73 CANXL_0__MSG_DESCRIPTORS.MSGDSC[73].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_73 CANXL_0__MSG_DESCRIPTORS.MSGDSC[73].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_74 CANXL_0__MSG_DESCRIPTORS.MSGDSC[74].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_74 CANXL_0__MSG_DESCRIPTORS.MSGDSC[74].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_74 CANXL_0__MSG_DESCRIPTORS.MSGDSC[74].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_74 CANXL_0__MSG_DESCRIPTORS.MSGDSC[74].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_74 CANXL_0__MSG_DESCRIPTORS.MSGDSC[74].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_74 CANXL_0__MSG_DESCRIPTORS.MSGDSC[74].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_74 CANXL_0__MSG_DESCRIPTORS.MSGDSC[74].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_75 CANXL_0__MSG_DESCRIPTORS.MSGDSC[75].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_75 CANXL_0__MSG_DESCRIPTORS.MSGDSC[75].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_75 CANXL_0__MSG_DESCRIPTORS.MSGDSC[75].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_75 CANXL_0__MSG_DESCRIPTORS.MSGDSC[75].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_75 CANXL_0__MSG_DESCRIPTORS.MSGDSC[75].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_75 CANXL_0__MSG_DESCRIPTORS.MSGDSC[75].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_75 CANXL_0__MSG_DESCRIPTORS.MSGDSC[75].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_76 CANXL_0__MSG_DESCRIPTORS.MSGDSC[76].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_76 CANXL_0__MSG_DESCRIPTORS.MSGDSC[76].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_76 CANXL_0__MSG_DESCRIPTORS.MSGDSC[76].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_76 CANXL_0__MSG_DESCRIPTORS.MSGDSC[76].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_76 CANXL_0__MSG_DESCRIPTORS.MSGDSC[76].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_76 CANXL_0__MSG_DESCRIPTORS.MSGDSC[76].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_76 CANXL_0__MSG_DESCRIPTORS.MSGDSC[76].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_77 CANXL_0__MSG_DESCRIPTORS.MSGDSC[77].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_77 CANXL_0__MSG_DESCRIPTORS.MSGDSC[77].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_77 CANXL_0__MSG_DESCRIPTORS.MSGDSC[77].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_77 CANXL_0__MSG_DESCRIPTORS.MSGDSC[77].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_77 CANXL_0__MSG_DESCRIPTORS.MSGDSC[77].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_77 CANXL_0__MSG_DESCRIPTORS.MSGDSC[77].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_77 CANXL_0__MSG_DESCRIPTORS.MSGDSC[77].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_78 CANXL_0__MSG_DESCRIPTORS.MSGDSC[78].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_78 CANXL_0__MSG_DESCRIPTORS.MSGDSC[78].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_78 CANXL_0__MSG_DESCRIPTORS.MSGDSC[78].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_78 CANXL_0__MSG_DESCRIPTORS.MSGDSC[78].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_78 CANXL_0__MSG_DESCRIPTORS.MSGDSC[78].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_78 CANXL_0__MSG_DESCRIPTORS.MSGDSC[78].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_78 CANXL_0__MSG_DESCRIPTORS.MSGDSC[78].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_79 CANXL_0__MSG_DESCRIPTORS.MSGDSC[79].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_79 CANXL_0__MSG_DESCRIPTORS.MSGDSC[79].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_79 CANXL_0__MSG_DESCRIPTORS.MSGDSC[79].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_79 CANXL_0__MSG_DESCRIPTORS.MSGDSC[79].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_79 CANXL_0__MSG_DESCRIPTORS.MSGDSC[79].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_79 CANXL_0__MSG_DESCRIPTORS.MSGDSC[79].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_79 CANXL_0__MSG_DESCRIPTORS.MSGDSC[79].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_80 CANXL_0__MSG_DESCRIPTORS.MSGDSC[80].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_80 CANXL_0__MSG_DESCRIPTORS.MSGDSC[80].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_80 CANXL_0__MSG_DESCRIPTORS.MSGDSC[80].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_80 CANXL_0__MSG_DESCRIPTORS.MSGDSC[80].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_80 CANXL_0__MSG_DESCRIPTORS.MSGDSC[80].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_80 CANXL_0__MSG_DESCRIPTORS.MSGDSC[80].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_80 CANXL_0__MSG_DESCRIPTORS.MSGDSC[80].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_81 CANXL_0__MSG_DESCRIPTORS.MSGDSC[81].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_81 CANXL_0__MSG_DESCRIPTORS.MSGDSC[81].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_81 CANXL_0__MSG_DESCRIPTORS.MSGDSC[81].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_81 CANXL_0__MSG_DESCRIPTORS.MSGDSC[81].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_81 CANXL_0__MSG_DESCRIPTORS.MSGDSC[81].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_81 CANXL_0__MSG_DESCRIPTORS.MSGDSC[81].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_81 CANXL_0__MSG_DESCRIPTORS.MSGDSC[81].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_82 CANXL_0__MSG_DESCRIPTORS.MSGDSC[82].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_82 CANXL_0__MSG_DESCRIPTORS.MSGDSC[82].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_82 CANXL_0__MSG_DESCRIPTORS.MSGDSC[82].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_82 CANXL_0__MSG_DESCRIPTORS.MSGDSC[82].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_82 CANXL_0__MSG_DESCRIPTORS.MSGDSC[82].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_82 CANXL_0__MSG_DESCRIPTORS.MSGDSC[82].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_82 CANXL_0__MSG_DESCRIPTORS.MSGDSC[82].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_83 CANXL_0__MSG_DESCRIPTORS.MSGDSC[83].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_83 CANXL_0__MSG_DESCRIPTORS.MSGDSC[83].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_83 CANXL_0__MSG_DESCRIPTORS.MSGDSC[83].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_83 CANXL_0__MSG_DESCRIPTORS.MSGDSC[83].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_83 CANXL_0__MSG_DESCRIPTORS.MSGDSC[83].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_83 CANXL_0__MSG_DESCRIPTORS.MSGDSC[83].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_83 CANXL_0__MSG_DESCRIPTORS.MSGDSC[83].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_84 CANXL_0__MSG_DESCRIPTORS.MSGDSC[84].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_84 CANXL_0__MSG_DESCRIPTORS.MSGDSC[84].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_84 CANXL_0__MSG_DESCRIPTORS.MSGDSC[84].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_84 CANXL_0__MSG_DESCRIPTORS.MSGDSC[84].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_84 CANXL_0__MSG_DESCRIPTORS.MSGDSC[84].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_84 CANXL_0__MSG_DESCRIPTORS.MSGDSC[84].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_84 CANXL_0__MSG_DESCRIPTORS.MSGDSC[84].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_85 CANXL_0__MSG_DESCRIPTORS.MSGDSC[85].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_85 CANXL_0__MSG_DESCRIPTORS.MSGDSC[85].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_85 CANXL_0__MSG_DESCRIPTORS.MSGDSC[85].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_85 CANXL_0__MSG_DESCRIPTORS.MSGDSC[85].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_85 CANXL_0__MSG_DESCRIPTORS.MSGDSC[85].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_85 CANXL_0__MSG_DESCRIPTORS.MSGDSC[85].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_85 CANXL_0__MSG_DESCRIPTORS.MSGDSC[85].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_86 CANXL_0__MSG_DESCRIPTORS.MSGDSC[86].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_86 CANXL_0__MSG_DESCRIPTORS.MSGDSC[86].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_86 CANXL_0__MSG_DESCRIPTORS.MSGDSC[86].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_86 CANXL_0__MSG_DESCRIPTORS.MSGDSC[86].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_86 CANXL_0__MSG_DESCRIPTORS.MSGDSC[86].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_86 CANXL_0__MSG_DESCRIPTORS.MSGDSC[86].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_86 CANXL_0__MSG_DESCRIPTORS.MSGDSC[86].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_87 CANXL_0__MSG_DESCRIPTORS.MSGDSC[87].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_87 CANXL_0__MSG_DESCRIPTORS.MSGDSC[87].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_87 CANXL_0__MSG_DESCRIPTORS.MSGDSC[87].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_87 CANXL_0__MSG_DESCRIPTORS.MSGDSC[87].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_87 CANXL_0__MSG_DESCRIPTORS.MSGDSC[87].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_87 CANXL_0__MSG_DESCRIPTORS.MSGDSC[87].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_87 CANXL_0__MSG_DESCRIPTORS.MSGDSC[87].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_88 CANXL_0__MSG_DESCRIPTORS.MSGDSC[88].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_88 CANXL_0__MSG_DESCRIPTORS.MSGDSC[88].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_88 CANXL_0__MSG_DESCRIPTORS.MSGDSC[88].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_88 CANXL_0__MSG_DESCRIPTORS.MSGDSC[88].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_88 CANXL_0__MSG_DESCRIPTORS.MSGDSC[88].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_88 CANXL_0__MSG_DESCRIPTORS.MSGDSC[88].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_88 CANXL_0__MSG_DESCRIPTORS.MSGDSC[88].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_89 CANXL_0__MSG_DESCRIPTORS.MSGDSC[89].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_89 CANXL_0__MSG_DESCRIPTORS.MSGDSC[89].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_89 CANXL_0__MSG_DESCRIPTORS.MSGDSC[89].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_89 CANXL_0__MSG_DESCRIPTORS.MSGDSC[89].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_89 CANXL_0__MSG_DESCRIPTORS.MSGDSC[89].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_89 CANXL_0__MSG_DESCRIPTORS.MSGDSC[89].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_89 CANXL_0__MSG_DESCRIPTORS.MSGDSC[89].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_90 CANXL_0__MSG_DESCRIPTORS.MSGDSC[90].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_90 CANXL_0__MSG_DESCRIPTORS.MSGDSC[90].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_90 CANXL_0__MSG_DESCRIPTORS.MSGDSC[90].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_90 CANXL_0__MSG_DESCRIPTORS.MSGDSC[90].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_90 CANXL_0__MSG_DESCRIPTORS.MSGDSC[90].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_90 CANXL_0__MSG_DESCRIPTORS.MSGDSC[90].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_90 CANXL_0__MSG_DESCRIPTORS.MSGDSC[90].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_91 CANXL_0__MSG_DESCRIPTORS.MSGDSC[91].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_91 CANXL_0__MSG_DESCRIPTORS.MSGDSC[91].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_91 CANXL_0__MSG_DESCRIPTORS.MSGDSC[91].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_91 CANXL_0__MSG_DESCRIPTORS.MSGDSC[91].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_91 CANXL_0__MSG_DESCRIPTORS.MSGDSC[91].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_91 CANXL_0__MSG_DESCRIPTORS.MSGDSC[91].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_91 CANXL_0__MSG_DESCRIPTORS.MSGDSC[91].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_92 CANXL_0__MSG_DESCRIPTORS.MSGDSC[92].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_92 CANXL_0__MSG_DESCRIPTORS.MSGDSC[92].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_92 CANXL_0__MSG_DESCRIPTORS.MSGDSC[92].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_92 CANXL_0__MSG_DESCRIPTORS.MSGDSC[92].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_92 CANXL_0__MSG_DESCRIPTORS.MSGDSC[92].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_92 CANXL_0__MSG_DESCRIPTORS.MSGDSC[92].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_92 CANXL_0__MSG_DESCRIPTORS.MSGDSC[92].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_93 CANXL_0__MSG_DESCRIPTORS.MSGDSC[93].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_93 CANXL_0__MSG_DESCRIPTORS.MSGDSC[93].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_93 CANXL_0__MSG_DESCRIPTORS.MSGDSC[93].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_93 CANXL_0__MSG_DESCRIPTORS.MSGDSC[93].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_93 CANXL_0__MSG_DESCRIPTORS.MSGDSC[93].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_93 CANXL_0__MSG_DESCRIPTORS.MSGDSC[93].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_93 CANXL_0__MSG_DESCRIPTORS.MSGDSC[93].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_94 CANXL_0__MSG_DESCRIPTORS.MSGDSC[94].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_94 CANXL_0__MSG_DESCRIPTORS.MSGDSC[94].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_94 CANXL_0__MSG_DESCRIPTORS.MSGDSC[94].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_94 CANXL_0__MSG_DESCRIPTORS.MSGDSC[94].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_94 CANXL_0__MSG_DESCRIPTORS.MSGDSC[94].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_94 CANXL_0__MSG_DESCRIPTORS.MSGDSC[94].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_94 CANXL_0__MSG_DESCRIPTORS.MSGDSC[94].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_95 CANXL_0__MSG_DESCRIPTORS.MSGDSC[95].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_95 CANXL_0__MSG_DESCRIPTORS.MSGDSC[95].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_95 CANXL_0__MSG_DESCRIPTORS.MSGDSC[95].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_95 CANXL_0__MSG_DESCRIPTORS.MSGDSC[95].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_95 CANXL_0__MSG_DESCRIPTORS.MSGDSC[95].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_95 CANXL_0__MSG_DESCRIPTORS.MSGDSC[95].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_95 CANXL_0__MSG_DESCRIPTORS.MSGDSC[95].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_96 CANXL_0__MSG_DESCRIPTORS.MSGDSC[96].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_96 CANXL_0__MSG_DESCRIPTORS.MSGDSC[96].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_96 CANXL_0__MSG_DESCRIPTORS.MSGDSC[96].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_96 CANXL_0__MSG_DESCRIPTORS.MSGDSC[96].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_96 CANXL_0__MSG_DESCRIPTORS.MSGDSC[96].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_96 CANXL_0__MSG_DESCRIPTORS.MSGDSC[96].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_96 CANXL_0__MSG_DESCRIPTORS.MSGDSC[96].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_97 CANXL_0__MSG_DESCRIPTORS.MSGDSC[97].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_97 CANXL_0__MSG_DESCRIPTORS.MSGDSC[97].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_97 CANXL_0__MSG_DESCRIPTORS.MSGDSC[97].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_97 CANXL_0__MSG_DESCRIPTORS.MSGDSC[97].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_97 CANXL_0__MSG_DESCRIPTORS.MSGDSC[97].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_97 CANXL_0__MSG_DESCRIPTORS.MSGDSC[97].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_97 CANXL_0__MSG_DESCRIPTORS.MSGDSC[97].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_98 CANXL_0__MSG_DESCRIPTORS.MSGDSC[98].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_98 CANXL_0__MSG_DESCRIPTORS.MSGDSC[98].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_98 CANXL_0__MSG_DESCRIPTORS.MSGDSC[98].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_98 CANXL_0__MSG_DESCRIPTORS.MSGDSC[98].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_98 CANXL_0__MSG_DESCRIPTORS.MSGDSC[98].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_98 CANXL_0__MSG_DESCRIPTORS.MSGDSC[98].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_98 CANXL_0__MSG_DESCRIPTORS.MSGDSC[98].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_99 CANXL_0__MSG_DESCRIPTORS.MSGDSC[99].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_99 CANXL_0__MSG_DESCRIPTORS.MSGDSC[99].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_99 CANXL_0__MSG_DESCRIPTORS.MSGDSC[99].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_99 CANXL_0__MSG_DESCRIPTORS.MSGDSC[99].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_99 CANXL_0__MSG_DESCRIPTORS.MSGDSC[99].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_99 CANXL_0__MSG_DESCRIPTORS.MSGDSC[99].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_99 CANXL_0__MSG_DESCRIPTORS.MSGDSC[99].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_100 CANXL_0__MSG_DESCRIPTORS.MSGDSC[100].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_100 CANXL_0__MSG_DESCRIPTORS.MSGDSC[100].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_100 CANXL_0__MSG_DESCRIPTORS.MSGDSC[100].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_100 CANXL_0__MSG_DESCRIPTORS.MSGDSC[100].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_100 CANXL_0__MSG_DESCRIPTORS.MSGDSC[100].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_100 CANXL_0__MSG_DESCRIPTORS.MSGDSC[100].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_100 CANXL_0__MSG_DESCRIPTORS.MSGDSC[100].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_101 CANXL_0__MSG_DESCRIPTORS.MSGDSC[101].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_101 CANXL_0__MSG_DESCRIPTORS.MSGDSC[101].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_101 CANXL_0__MSG_DESCRIPTORS.MSGDSC[101].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_101 CANXL_0__MSG_DESCRIPTORS.MSGDSC[101].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_101 CANXL_0__MSG_DESCRIPTORS.MSGDSC[101].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_101 CANXL_0__MSG_DESCRIPTORS.MSGDSC[101].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_101 CANXL_0__MSG_DESCRIPTORS.MSGDSC[101].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_102 CANXL_0__MSG_DESCRIPTORS.MSGDSC[102].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_102 CANXL_0__MSG_DESCRIPTORS.MSGDSC[102].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_102 CANXL_0__MSG_DESCRIPTORS.MSGDSC[102].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_102 CANXL_0__MSG_DESCRIPTORS.MSGDSC[102].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_102 CANXL_0__MSG_DESCRIPTORS.MSGDSC[102].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_102 CANXL_0__MSG_DESCRIPTORS.MSGDSC[102].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_102 CANXL_0__MSG_DESCRIPTORS.MSGDSC[102].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_103 CANXL_0__MSG_DESCRIPTORS.MSGDSC[103].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_103 CANXL_0__MSG_DESCRIPTORS.MSGDSC[103].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_103 CANXL_0__MSG_DESCRIPTORS.MSGDSC[103].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_103 CANXL_0__MSG_DESCRIPTORS.MSGDSC[103].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_103 CANXL_0__MSG_DESCRIPTORS.MSGDSC[103].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_103 CANXL_0__MSG_DESCRIPTORS.MSGDSC[103].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_103 CANXL_0__MSG_DESCRIPTORS.MSGDSC[103].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_104 CANXL_0__MSG_DESCRIPTORS.MSGDSC[104].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_104 CANXL_0__MSG_DESCRIPTORS.MSGDSC[104].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_104 CANXL_0__MSG_DESCRIPTORS.MSGDSC[104].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_104 CANXL_0__MSG_DESCRIPTORS.MSGDSC[104].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_104 CANXL_0__MSG_DESCRIPTORS.MSGDSC[104].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_104 CANXL_0__MSG_DESCRIPTORS.MSGDSC[104].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_104 CANXL_0__MSG_DESCRIPTORS.MSGDSC[104].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_105 CANXL_0__MSG_DESCRIPTORS.MSGDSC[105].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_105 CANXL_0__MSG_DESCRIPTORS.MSGDSC[105].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_105 CANXL_0__MSG_DESCRIPTORS.MSGDSC[105].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_105 CANXL_0__MSG_DESCRIPTORS.MSGDSC[105].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_105 CANXL_0__MSG_DESCRIPTORS.MSGDSC[105].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_105 CANXL_0__MSG_DESCRIPTORS.MSGDSC[105].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_105 CANXL_0__MSG_DESCRIPTORS.MSGDSC[105].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_106 CANXL_0__MSG_DESCRIPTORS.MSGDSC[106].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_106 CANXL_0__MSG_DESCRIPTORS.MSGDSC[106].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_106 CANXL_0__MSG_DESCRIPTORS.MSGDSC[106].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_106 CANXL_0__MSG_DESCRIPTORS.MSGDSC[106].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_106 CANXL_0__MSG_DESCRIPTORS.MSGDSC[106].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_106 CANXL_0__MSG_DESCRIPTORS.MSGDSC[106].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_106 CANXL_0__MSG_DESCRIPTORS.MSGDSC[106].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_107 CANXL_0__MSG_DESCRIPTORS.MSGDSC[107].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_107 CANXL_0__MSG_DESCRIPTORS.MSGDSC[107].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_107 CANXL_0__MSG_DESCRIPTORS.MSGDSC[107].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_107 CANXL_0__MSG_DESCRIPTORS.MSGDSC[107].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_107 CANXL_0__MSG_DESCRIPTORS.MSGDSC[107].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_107 CANXL_0__MSG_DESCRIPTORS.MSGDSC[107].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_107 CANXL_0__MSG_DESCRIPTORS.MSGDSC[107].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_108 CANXL_0__MSG_DESCRIPTORS.MSGDSC[108].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_108 CANXL_0__MSG_DESCRIPTORS.MSGDSC[108].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_108 CANXL_0__MSG_DESCRIPTORS.MSGDSC[108].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_108 CANXL_0__MSG_DESCRIPTORS.MSGDSC[108].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_108 CANXL_0__MSG_DESCRIPTORS.MSGDSC[108].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_108 CANXL_0__MSG_DESCRIPTORS.MSGDSC[108].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_108 CANXL_0__MSG_DESCRIPTORS.MSGDSC[108].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_109 CANXL_0__MSG_DESCRIPTORS.MSGDSC[109].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_109 CANXL_0__MSG_DESCRIPTORS.MSGDSC[109].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_109 CANXL_0__MSG_DESCRIPTORS.MSGDSC[109].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_109 CANXL_0__MSG_DESCRIPTORS.MSGDSC[109].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_109 CANXL_0__MSG_DESCRIPTORS.MSGDSC[109].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_109 CANXL_0__MSG_DESCRIPTORS.MSGDSC[109].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_109 CANXL_0__MSG_DESCRIPTORS.MSGDSC[109].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_110 CANXL_0__MSG_DESCRIPTORS.MSGDSC[110].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_110 CANXL_0__MSG_DESCRIPTORS.MSGDSC[110].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_110 CANXL_0__MSG_DESCRIPTORS.MSGDSC[110].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_110 CANXL_0__MSG_DESCRIPTORS.MSGDSC[110].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_110 CANXL_0__MSG_DESCRIPTORS.MSGDSC[110].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_110 CANXL_0__MSG_DESCRIPTORS.MSGDSC[110].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_110 CANXL_0__MSG_DESCRIPTORS.MSGDSC[110].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_111 CANXL_0__MSG_DESCRIPTORS.MSGDSC[111].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_111 CANXL_0__MSG_DESCRIPTORS.MSGDSC[111].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_111 CANXL_0__MSG_DESCRIPTORS.MSGDSC[111].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_111 CANXL_0__MSG_DESCRIPTORS.MSGDSC[111].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_111 CANXL_0__MSG_DESCRIPTORS.MSGDSC[111].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_111 CANXL_0__MSG_DESCRIPTORS.MSGDSC[111].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_111 CANXL_0__MSG_DESCRIPTORS.MSGDSC[111].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_112 CANXL_0__MSG_DESCRIPTORS.MSGDSC[112].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_112 CANXL_0__MSG_DESCRIPTORS.MSGDSC[112].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_112 CANXL_0__MSG_DESCRIPTORS.MSGDSC[112].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_112 CANXL_0__MSG_DESCRIPTORS.MSGDSC[112].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_112 CANXL_0__MSG_DESCRIPTORS.MSGDSC[112].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_112 CANXL_0__MSG_DESCRIPTORS.MSGDSC[112].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_112 CANXL_0__MSG_DESCRIPTORS.MSGDSC[112].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_113 CANXL_0__MSG_DESCRIPTORS.MSGDSC[113].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_113 CANXL_0__MSG_DESCRIPTORS.MSGDSC[113].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_113 CANXL_0__MSG_DESCRIPTORS.MSGDSC[113].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_113 CANXL_0__MSG_DESCRIPTORS.MSGDSC[113].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_113 CANXL_0__MSG_DESCRIPTORS.MSGDSC[113].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_113 CANXL_0__MSG_DESCRIPTORS.MSGDSC[113].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_113 CANXL_0__MSG_DESCRIPTORS.MSGDSC[113].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_114 CANXL_0__MSG_DESCRIPTORS.MSGDSC[114].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_114 CANXL_0__MSG_DESCRIPTORS.MSGDSC[114].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_114 CANXL_0__MSG_DESCRIPTORS.MSGDSC[114].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_114 CANXL_0__MSG_DESCRIPTORS.MSGDSC[114].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_114 CANXL_0__MSG_DESCRIPTORS.MSGDSC[114].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_114 CANXL_0__MSG_DESCRIPTORS.MSGDSC[114].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_114 CANXL_0__MSG_DESCRIPTORS.MSGDSC[114].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_115 CANXL_0__MSG_DESCRIPTORS.MSGDSC[115].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_115 CANXL_0__MSG_DESCRIPTORS.MSGDSC[115].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_115 CANXL_0__MSG_DESCRIPTORS.MSGDSC[115].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_115 CANXL_0__MSG_DESCRIPTORS.MSGDSC[115].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_115 CANXL_0__MSG_DESCRIPTORS.MSGDSC[115].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_115 CANXL_0__MSG_DESCRIPTORS.MSGDSC[115].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_115 CANXL_0__MSG_DESCRIPTORS.MSGDSC[115].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_116 CANXL_0__MSG_DESCRIPTORS.MSGDSC[116].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_116 CANXL_0__MSG_DESCRIPTORS.MSGDSC[116].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_116 CANXL_0__MSG_DESCRIPTORS.MSGDSC[116].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_116 CANXL_0__MSG_DESCRIPTORS.MSGDSC[116].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_116 CANXL_0__MSG_DESCRIPTORS.MSGDSC[116].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_116 CANXL_0__MSG_DESCRIPTORS.MSGDSC[116].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_116 CANXL_0__MSG_DESCRIPTORS.MSGDSC[116].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_117 CANXL_0__MSG_DESCRIPTORS.MSGDSC[117].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_117 CANXL_0__MSG_DESCRIPTORS.MSGDSC[117].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_117 CANXL_0__MSG_DESCRIPTORS.MSGDSC[117].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_117 CANXL_0__MSG_DESCRIPTORS.MSGDSC[117].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_117 CANXL_0__MSG_DESCRIPTORS.MSGDSC[117].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_117 CANXL_0__MSG_DESCRIPTORS.MSGDSC[117].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_117 CANXL_0__MSG_DESCRIPTORS.MSGDSC[117].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_118 CANXL_0__MSG_DESCRIPTORS.MSGDSC[118].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_118 CANXL_0__MSG_DESCRIPTORS.MSGDSC[118].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_118 CANXL_0__MSG_DESCRIPTORS.MSGDSC[118].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_118 CANXL_0__MSG_DESCRIPTORS.MSGDSC[118].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_118 CANXL_0__MSG_DESCRIPTORS.MSGDSC[118].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_118 CANXL_0__MSG_DESCRIPTORS.MSGDSC[118].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_118 CANXL_0__MSG_DESCRIPTORS.MSGDSC[118].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_119 CANXL_0__MSG_DESCRIPTORS.MSGDSC[119].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_119 CANXL_0__MSG_DESCRIPTORS.MSGDSC[119].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_119 CANXL_0__MSG_DESCRIPTORS.MSGDSC[119].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_119 CANXL_0__MSG_DESCRIPTORS.MSGDSC[119].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_119 CANXL_0__MSG_DESCRIPTORS.MSGDSC[119].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_119 CANXL_0__MSG_DESCRIPTORS.MSGDSC[119].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_119 CANXL_0__MSG_DESCRIPTORS.MSGDSC[119].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_120 CANXL_0__MSG_DESCRIPTORS.MSGDSC[120].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_120 CANXL_0__MSG_DESCRIPTORS.MSGDSC[120].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_120 CANXL_0__MSG_DESCRIPTORS.MSGDSC[120].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_120 CANXL_0__MSG_DESCRIPTORS.MSGDSC[120].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_120 CANXL_0__MSG_DESCRIPTORS.MSGDSC[120].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_120 CANXL_0__MSG_DESCRIPTORS.MSGDSC[120].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_120 CANXL_0__MSG_DESCRIPTORS.MSGDSC[120].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_121 CANXL_0__MSG_DESCRIPTORS.MSGDSC[121].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_121 CANXL_0__MSG_DESCRIPTORS.MSGDSC[121].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_121 CANXL_0__MSG_DESCRIPTORS.MSGDSC[121].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_121 CANXL_0__MSG_DESCRIPTORS.MSGDSC[121].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_121 CANXL_0__MSG_DESCRIPTORS.MSGDSC[121].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_121 CANXL_0__MSG_DESCRIPTORS.MSGDSC[121].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_121 CANXL_0__MSG_DESCRIPTORS.MSGDSC[121].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_122 CANXL_0__MSG_DESCRIPTORS.MSGDSC[122].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_122 CANXL_0__MSG_DESCRIPTORS.MSGDSC[122].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_122 CANXL_0__MSG_DESCRIPTORS.MSGDSC[122].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_122 CANXL_0__MSG_DESCRIPTORS.MSGDSC[122].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_122 CANXL_0__MSG_DESCRIPTORS.MSGDSC[122].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_122 CANXL_0__MSG_DESCRIPTORS.MSGDSC[122].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_122 CANXL_0__MSG_DESCRIPTORS.MSGDSC[122].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_123 CANXL_0__MSG_DESCRIPTORS.MSGDSC[123].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_123 CANXL_0__MSG_DESCRIPTORS.MSGDSC[123].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_123 CANXL_0__MSG_DESCRIPTORS.MSGDSC[123].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_123 CANXL_0__MSG_DESCRIPTORS.MSGDSC[123].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_123 CANXL_0__MSG_DESCRIPTORS.MSGDSC[123].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_123 CANXL_0__MSG_DESCRIPTORS.MSGDSC[123].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_123 CANXL_0__MSG_DESCRIPTORS.MSGDSC[123].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_124 CANXL_0__MSG_DESCRIPTORS.MSGDSC[124].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_124 CANXL_0__MSG_DESCRIPTORS.MSGDSC[124].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_124 CANXL_0__MSG_DESCRIPTORS.MSGDSC[124].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_124 CANXL_0__MSG_DESCRIPTORS.MSGDSC[124].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_124 CANXL_0__MSG_DESCRIPTORS.MSGDSC[124].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_124 CANXL_0__MSG_DESCRIPTORS.MSGDSC[124].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_124 CANXL_0__MSG_DESCRIPTORS.MSGDSC[124].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_125 CANXL_0__MSG_DESCRIPTORS.MSGDSC[125].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_125 CANXL_0__MSG_DESCRIPTORS.MSGDSC[125].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_125 CANXL_0__MSG_DESCRIPTORS.MSGDSC[125].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_125 CANXL_0__MSG_DESCRIPTORS.MSGDSC[125].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_125 CANXL_0__MSG_DESCRIPTORS.MSGDSC[125].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_125 CANXL_0__MSG_DESCRIPTORS.MSGDSC[125].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_125 CANXL_0__MSG_DESCRIPTORS.MSGDSC[125].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_126 CANXL_0__MSG_DESCRIPTORS.MSGDSC[126].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_126 CANXL_0__MSG_DESCRIPTORS.MSGDSC[126].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_126 CANXL_0__MSG_DESCRIPTORS.MSGDSC[126].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_126 CANXL_0__MSG_DESCRIPTORS.MSGDSC[126].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_126 CANXL_0__MSG_DESCRIPTORS.MSGDSC[126].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_126 CANXL_0__MSG_DESCRIPTORS.MSGDSC[126].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_126 CANXL_0__MSG_DESCRIPTORS.MSGDSC[126].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXCTRL_127 CANXL_0__MSG_DESCRIPTORS.MSGDSC[127].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1FD_127 CANXL_0__MSG_DESCRIPTORS.MSGDSC[127].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT1XL_127 CANXL_0__MSG_DESCRIPTORS.MSGDSC[127].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2FD_127 CANXL_0__MSG_DESCRIPTORS.MSGDSC[127].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_0__MSG_DESCRIPTORS_MDFLT2XL_127 CANXL_0__MSG_DESCRIPTORS.MSGDSC[127].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_0__MSG_DESCRIPTORS_RXLSTPTR_127 CANXL_0__MSG_DESCRIPTORS.MSGDSC[127].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_0__MSG_DESCRIPTORS_TXLSTPTR_127 CANXL_0__MSG_DESCRIPTORS.MSGDSC[127].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */

#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_0 CANXL_1__MSG_DESCRIPTORS.MSGDSC[0].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_1 CANXL_1__MSG_DESCRIPTORS.MSGDSC[1].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_1 CANXL_1__MSG_DESCRIPTORS.MSGDSC[1].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_1 CANXL_1__MSG_DESCRIPTORS.MSGDSC[1].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_1 CANXL_1__MSG_DESCRIPTORS.MSGDSC[1].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_1 CANXL_1__MSG_DESCRIPTORS.MSGDSC[1].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_1 CANXL_1__MSG_DESCRIPTORS.MSGDSC[1].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_1 CANXL_1__MSG_DESCRIPTORS.MSGDSC[1].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_2 CANXL_1__MSG_DESCRIPTORS.MSGDSC[2].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_2 CANXL_1__MSG_DESCRIPTORS.MSGDSC[2].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_2 CANXL_1__MSG_DESCRIPTORS.MSGDSC[2].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_2 CANXL_1__MSG_DESCRIPTORS.MSGDSC[2].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_2 CANXL_1__MSG_DESCRIPTORS.MSGDSC[2].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_2 CANXL_1__MSG_DESCRIPTORS.MSGDSC[2].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_2 CANXL_1__MSG_DESCRIPTORS.MSGDSC[2].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_3 CANXL_1__MSG_DESCRIPTORS.MSGDSC[3].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_3 CANXL_1__MSG_DESCRIPTORS.MSGDSC[3].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_3 CANXL_1__MSG_DESCRIPTORS.MSGDSC[3].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_3 CANXL_1__MSG_DESCRIPTORS.MSGDSC[3].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_3 CANXL_1__MSG_DESCRIPTORS.MSGDSC[3].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_3 CANXL_1__MSG_DESCRIPTORS.MSGDSC[3].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_3 CANXL_1__MSG_DESCRIPTORS.MSGDSC[3].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_4 CANXL_1__MSG_DESCRIPTORS.MSGDSC[4].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_4 CANXL_1__MSG_DESCRIPTORS.MSGDSC[4].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_4 CANXL_1__MSG_DESCRIPTORS.MSGDSC[4].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_4 CANXL_1__MSG_DESCRIPTORS.MSGDSC[4].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_4 CANXL_1__MSG_DESCRIPTORS.MSGDSC[4].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_4 CANXL_1__MSG_DESCRIPTORS.MSGDSC[4].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_4 CANXL_1__MSG_DESCRIPTORS.MSGDSC[4].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_5 CANXL_1__MSG_DESCRIPTORS.MSGDSC[5].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_5 CANXL_1__MSG_DESCRIPTORS.MSGDSC[5].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_5 CANXL_1__MSG_DESCRIPTORS.MSGDSC[5].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_5 CANXL_1__MSG_DESCRIPTORS.MSGDSC[5].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_5 CANXL_1__MSG_DESCRIPTORS.MSGDSC[5].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_5 CANXL_1__MSG_DESCRIPTORS.MSGDSC[5].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_5 CANXL_1__MSG_DESCRIPTORS.MSGDSC[5].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_6 CANXL_1__MSG_DESCRIPTORS.MSGDSC[6].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_6 CANXL_1__MSG_DESCRIPTORS.MSGDSC[6].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_6 CANXL_1__MSG_DESCRIPTORS.MSGDSC[6].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_6 CANXL_1__MSG_DESCRIPTORS.MSGDSC[6].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_6 CANXL_1__MSG_DESCRIPTORS.MSGDSC[6].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_6 CANXL_1__MSG_DESCRIPTORS.MSGDSC[6].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_6 CANXL_1__MSG_DESCRIPTORS.MSGDSC[6].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_7 CANXL_1__MSG_DESCRIPTORS.MSGDSC[7].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_7 CANXL_1__MSG_DESCRIPTORS.MSGDSC[7].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_7 CANXL_1__MSG_DESCRIPTORS.MSGDSC[7].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_7 CANXL_1__MSG_DESCRIPTORS.MSGDSC[7].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_7 CANXL_1__MSG_DESCRIPTORS.MSGDSC[7].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_7 CANXL_1__MSG_DESCRIPTORS.MSGDSC[7].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_7 CANXL_1__MSG_DESCRIPTORS.MSGDSC[7].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_8 CANXL_1__MSG_DESCRIPTORS.MSGDSC[8].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_8 CANXL_1__MSG_DESCRIPTORS.MSGDSC[8].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_8 CANXL_1__MSG_DESCRIPTORS.MSGDSC[8].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_8 CANXL_1__MSG_DESCRIPTORS.MSGDSC[8].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_8 CANXL_1__MSG_DESCRIPTORS.MSGDSC[8].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_8 CANXL_1__MSG_DESCRIPTORS.MSGDSC[8].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_8 CANXL_1__MSG_DESCRIPTORS.MSGDSC[8].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_9 CANXL_1__MSG_DESCRIPTORS.MSGDSC[9].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_9 CANXL_1__MSG_DESCRIPTORS.MSGDSC[9].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_9 CANXL_1__MSG_DESCRIPTORS.MSGDSC[9].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_9 CANXL_1__MSG_DESCRIPTORS.MSGDSC[9].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_9 CANXL_1__MSG_DESCRIPTORS.MSGDSC[9].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_9 CANXL_1__MSG_DESCRIPTORS.MSGDSC[9].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_9 CANXL_1__MSG_DESCRIPTORS.MSGDSC[9].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_10 CANXL_1__MSG_DESCRIPTORS.MSGDSC[10].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_10 CANXL_1__MSG_DESCRIPTORS.MSGDSC[10].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_10 CANXL_1__MSG_DESCRIPTORS.MSGDSC[10].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_10 CANXL_1__MSG_DESCRIPTORS.MSGDSC[10].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_10 CANXL_1__MSG_DESCRIPTORS.MSGDSC[10].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_10 CANXL_1__MSG_DESCRIPTORS.MSGDSC[10].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_10 CANXL_1__MSG_DESCRIPTORS.MSGDSC[10].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_11 CANXL_1__MSG_DESCRIPTORS.MSGDSC[11].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_11 CANXL_1__MSG_DESCRIPTORS.MSGDSC[11].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_11 CANXL_1__MSG_DESCRIPTORS.MSGDSC[11].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_11 CANXL_1__MSG_DESCRIPTORS.MSGDSC[11].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_11 CANXL_1__MSG_DESCRIPTORS.MSGDSC[11].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_11 CANXL_1__MSG_DESCRIPTORS.MSGDSC[11].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_11 CANXL_1__MSG_DESCRIPTORS.MSGDSC[11].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_12 CANXL_1__MSG_DESCRIPTORS.MSGDSC[12].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_12 CANXL_1__MSG_DESCRIPTORS.MSGDSC[12].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_12 CANXL_1__MSG_DESCRIPTORS.MSGDSC[12].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_12 CANXL_1__MSG_DESCRIPTORS.MSGDSC[12].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_12 CANXL_1__MSG_DESCRIPTORS.MSGDSC[12].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_12 CANXL_1__MSG_DESCRIPTORS.MSGDSC[12].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_12 CANXL_1__MSG_DESCRIPTORS.MSGDSC[12].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_13 CANXL_1__MSG_DESCRIPTORS.MSGDSC[13].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_13 CANXL_1__MSG_DESCRIPTORS.MSGDSC[13].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_13 CANXL_1__MSG_DESCRIPTORS.MSGDSC[13].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_13 CANXL_1__MSG_DESCRIPTORS.MSGDSC[13].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_13 CANXL_1__MSG_DESCRIPTORS.MSGDSC[13].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_13 CANXL_1__MSG_DESCRIPTORS.MSGDSC[13].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_13 CANXL_1__MSG_DESCRIPTORS.MSGDSC[13].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_14 CANXL_1__MSG_DESCRIPTORS.MSGDSC[14].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_14 CANXL_1__MSG_DESCRIPTORS.MSGDSC[14].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_14 CANXL_1__MSG_DESCRIPTORS.MSGDSC[14].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_14 CANXL_1__MSG_DESCRIPTORS.MSGDSC[14].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_14 CANXL_1__MSG_DESCRIPTORS.MSGDSC[14].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_14 CANXL_1__MSG_DESCRIPTORS.MSGDSC[14].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_14 CANXL_1__MSG_DESCRIPTORS.MSGDSC[14].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_15 CANXL_1__MSG_DESCRIPTORS.MSGDSC[15].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_15 CANXL_1__MSG_DESCRIPTORS.MSGDSC[15].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_15 CANXL_1__MSG_DESCRIPTORS.MSGDSC[15].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_15 CANXL_1__MSG_DESCRIPTORS.MSGDSC[15].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_15 CANXL_1__MSG_DESCRIPTORS.MSGDSC[15].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_15 CANXL_1__MSG_DESCRIPTORS.MSGDSC[15].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_15 CANXL_1__MSG_DESCRIPTORS.MSGDSC[15].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_16 CANXL_1__MSG_DESCRIPTORS.MSGDSC[16].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_16 CANXL_1__MSG_DESCRIPTORS.MSGDSC[16].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_16 CANXL_1__MSG_DESCRIPTORS.MSGDSC[16].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_16 CANXL_1__MSG_DESCRIPTORS.MSGDSC[16].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_16 CANXL_1__MSG_DESCRIPTORS.MSGDSC[16].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_16 CANXL_1__MSG_DESCRIPTORS.MSGDSC[16].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_16 CANXL_1__MSG_DESCRIPTORS.MSGDSC[16].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_17 CANXL_1__MSG_DESCRIPTORS.MSGDSC[17].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_17 CANXL_1__MSG_DESCRIPTORS.MSGDSC[17].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_17 CANXL_1__MSG_DESCRIPTORS.MSGDSC[17].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_17 CANXL_1__MSG_DESCRIPTORS.MSGDSC[17].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_17 CANXL_1__MSG_DESCRIPTORS.MSGDSC[17].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_17 CANXL_1__MSG_DESCRIPTORS.MSGDSC[17].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_17 CANXL_1__MSG_DESCRIPTORS.MSGDSC[17].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_18 CANXL_1__MSG_DESCRIPTORS.MSGDSC[18].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_18 CANXL_1__MSG_DESCRIPTORS.MSGDSC[18].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_18 CANXL_1__MSG_DESCRIPTORS.MSGDSC[18].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_18 CANXL_1__MSG_DESCRIPTORS.MSGDSC[18].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_18 CANXL_1__MSG_DESCRIPTORS.MSGDSC[18].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_18 CANXL_1__MSG_DESCRIPTORS.MSGDSC[18].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_18 CANXL_1__MSG_DESCRIPTORS.MSGDSC[18].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_19 CANXL_1__MSG_DESCRIPTORS.MSGDSC[19].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_19 CANXL_1__MSG_DESCRIPTORS.MSGDSC[19].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_19 CANXL_1__MSG_DESCRIPTORS.MSGDSC[19].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_19 CANXL_1__MSG_DESCRIPTORS.MSGDSC[19].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_19 CANXL_1__MSG_DESCRIPTORS.MSGDSC[19].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_19 CANXL_1__MSG_DESCRIPTORS.MSGDSC[19].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_19 CANXL_1__MSG_DESCRIPTORS.MSGDSC[19].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_20 CANXL_1__MSG_DESCRIPTORS.MSGDSC[20].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_20 CANXL_1__MSG_DESCRIPTORS.MSGDSC[20].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_20 CANXL_1__MSG_DESCRIPTORS.MSGDSC[20].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_20 CANXL_1__MSG_DESCRIPTORS.MSGDSC[20].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_20 CANXL_1__MSG_DESCRIPTORS.MSGDSC[20].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_20 CANXL_1__MSG_DESCRIPTORS.MSGDSC[20].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_20 CANXL_1__MSG_DESCRIPTORS.MSGDSC[20].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_21 CANXL_1__MSG_DESCRIPTORS.MSGDSC[21].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_21 CANXL_1__MSG_DESCRIPTORS.MSGDSC[21].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_21 CANXL_1__MSG_DESCRIPTORS.MSGDSC[21].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_21 CANXL_1__MSG_DESCRIPTORS.MSGDSC[21].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_21 CANXL_1__MSG_DESCRIPTORS.MSGDSC[21].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_21 CANXL_1__MSG_DESCRIPTORS.MSGDSC[21].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_21 CANXL_1__MSG_DESCRIPTORS.MSGDSC[21].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_22 CANXL_1__MSG_DESCRIPTORS.MSGDSC[22].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_22 CANXL_1__MSG_DESCRIPTORS.MSGDSC[22].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_22 CANXL_1__MSG_DESCRIPTORS.MSGDSC[22].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_22 CANXL_1__MSG_DESCRIPTORS.MSGDSC[22].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_22 CANXL_1__MSG_DESCRIPTORS.MSGDSC[22].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_22 CANXL_1__MSG_DESCRIPTORS.MSGDSC[22].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_22 CANXL_1__MSG_DESCRIPTORS.MSGDSC[22].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_23 CANXL_1__MSG_DESCRIPTORS.MSGDSC[23].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_23 CANXL_1__MSG_DESCRIPTORS.MSGDSC[23].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_23 CANXL_1__MSG_DESCRIPTORS.MSGDSC[23].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_23 CANXL_1__MSG_DESCRIPTORS.MSGDSC[23].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_23 CANXL_1__MSG_DESCRIPTORS.MSGDSC[23].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_23 CANXL_1__MSG_DESCRIPTORS.MSGDSC[23].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_23 CANXL_1__MSG_DESCRIPTORS.MSGDSC[23].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_24 CANXL_1__MSG_DESCRIPTORS.MSGDSC[24].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_24 CANXL_1__MSG_DESCRIPTORS.MSGDSC[24].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_24 CANXL_1__MSG_DESCRIPTORS.MSGDSC[24].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_24 CANXL_1__MSG_DESCRIPTORS.MSGDSC[24].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_24 CANXL_1__MSG_DESCRIPTORS.MSGDSC[24].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_24 CANXL_1__MSG_DESCRIPTORS.MSGDSC[24].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_24 CANXL_1__MSG_DESCRIPTORS.MSGDSC[24].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_25 CANXL_1__MSG_DESCRIPTORS.MSGDSC[25].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_25 CANXL_1__MSG_DESCRIPTORS.MSGDSC[25].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_25 CANXL_1__MSG_DESCRIPTORS.MSGDSC[25].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_25 CANXL_1__MSG_DESCRIPTORS.MSGDSC[25].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_25 CANXL_1__MSG_DESCRIPTORS.MSGDSC[25].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_25 CANXL_1__MSG_DESCRIPTORS.MSGDSC[25].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_25 CANXL_1__MSG_DESCRIPTORS.MSGDSC[25].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_26 CANXL_1__MSG_DESCRIPTORS.MSGDSC[26].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_26 CANXL_1__MSG_DESCRIPTORS.MSGDSC[26].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_26 CANXL_1__MSG_DESCRIPTORS.MSGDSC[26].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_26 CANXL_1__MSG_DESCRIPTORS.MSGDSC[26].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_26 CANXL_1__MSG_DESCRIPTORS.MSGDSC[26].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_26 CANXL_1__MSG_DESCRIPTORS.MSGDSC[26].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_26 CANXL_1__MSG_DESCRIPTORS.MSGDSC[26].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_27 CANXL_1__MSG_DESCRIPTORS.MSGDSC[27].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_27 CANXL_1__MSG_DESCRIPTORS.MSGDSC[27].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_27 CANXL_1__MSG_DESCRIPTORS.MSGDSC[27].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_27 CANXL_1__MSG_DESCRIPTORS.MSGDSC[27].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_27 CANXL_1__MSG_DESCRIPTORS.MSGDSC[27].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_27 CANXL_1__MSG_DESCRIPTORS.MSGDSC[27].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_27 CANXL_1__MSG_DESCRIPTORS.MSGDSC[27].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_28 CANXL_1__MSG_DESCRIPTORS.MSGDSC[28].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_28 CANXL_1__MSG_DESCRIPTORS.MSGDSC[28].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_28 CANXL_1__MSG_DESCRIPTORS.MSGDSC[28].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_28 CANXL_1__MSG_DESCRIPTORS.MSGDSC[28].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_28 CANXL_1__MSG_DESCRIPTORS.MSGDSC[28].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_28 CANXL_1__MSG_DESCRIPTORS.MSGDSC[28].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_28 CANXL_1__MSG_DESCRIPTORS.MSGDSC[28].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_29 CANXL_1__MSG_DESCRIPTORS.MSGDSC[29].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_29 CANXL_1__MSG_DESCRIPTORS.MSGDSC[29].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_29 CANXL_1__MSG_DESCRIPTORS.MSGDSC[29].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_29 CANXL_1__MSG_DESCRIPTORS.MSGDSC[29].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_29 CANXL_1__MSG_DESCRIPTORS.MSGDSC[29].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_29 CANXL_1__MSG_DESCRIPTORS.MSGDSC[29].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_29 CANXL_1__MSG_DESCRIPTORS.MSGDSC[29].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_30 CANXL_1__MSG_DESCRIPTORS.MSGDSC[30].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_30 CANXL_1__MSG_DESCRIPTORS.MSGDSC[30].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_30 CANXL_1__MSG_DESCRIPTORS.MSGDSC[30].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_30 CANXL_1__MSG_DESCRIPTORS.MSGDSC[30].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_30 CANXL_1__MSG_DESCRIPTORS.MSGDSC[30].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_30 CANXL_1__MSG_DESCRIPTORS.MSGDSC[30].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_30 CANXL_1__MSG_DESCRIPTORS.MSGDSC[30].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_31 CANXL_1__MSG_DESCRIPTORS.MSGDSC[31].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_31 CANXL_1__MSG_DESCRIPTORS.MSGDSC[31].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_31 CANXL_1__MSG_DESCRIPTORS.MSGDSC[31].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_31 CANXL_1__MSG_DESCRIPTORS.MSGDSC[31].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_31 CANXL_1__MSG_DESCRIPTORS.MSGDSC[31].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_31 CANXL_1__MSG_DESCRIPTORS.MSGDSC[31].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_31 CANXL_1__MSG_DESCRIPTORS.MSGDSC[31].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_32 CANXL_1__MSG_DESCRIPTORS.MSGDSC[32].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_32 CANXL_1__MSG_DESCRIPTORS.MSGDSC[32].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_32 CANXL_1__MSG_DESCRIPTORS.MSGDSC[32].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_32 CANXL_1__MSG_DESCRIPTORS.MSGDSC[32].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_32 CANXL_1__MSG_DESCRIPTORS.MSGDSC[32].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_32 CANXL_1__MSG_DESCRIPTORS.MSGDSC[32].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_32 CANXL_1__MSG_DESCRIPTORS.MSGDSC[32].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_33 CANXL_1__MSG_DESCRIPTORS.MSGDSC[33].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_33 CANXL_1__MSG_DESCRIPTORS.MSGDSC[33].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_33 CANXL_1__MSG_DESCRIPTORS.MSGDSC[33].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_33 CANXL_1__MSG_DESCRIPTORS.MSGDSC[33].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_33 CANXL_1__MSG_DESCRIPTORS.MSGDSC[33].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_33 CANXL_1__MSG_DESCRIPTORS.MSGDSC[33].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_33 CANXL_1__MSG_DESCRIPTORS.MSGDSC[33].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_34 CANXL_1__MSG_DESCRIPTORS.MSGDSC[34].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_34 CANXL_1__MSG_DESCRIPTORS.MSGDSC[34].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_34 CANXL_1__MSG_DESCRIPTORS.MSGDSC[34].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_34 CANXL_1__MSG_DESCRIPTORS.MSGDSC[34].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_34 CANXL_1__MSG_DESCRIPTORS.MSGDSC[34].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_34 CANXL_1__MSG_DESCRIPTORS.MSGDSC[34].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_34 CANXL_1__MSG_DESCRIPTORS.MSGDSC[34].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_35 CANXL_1__MSG_DESCRIPTORS.MSGDSC[35].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_35 CANXL_1__MSG_DESCRIPTORS.MSGDSC[35].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_35 CANXL_1__MSG_DESCRIPTORS.MSGDSC[35].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_35 CANXL_1__MSG_DESCRIPTORS.MSGDSC[35].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_35 CANXL_1__MSG_DESCRIPTORS.MSGDSC[35].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_35 CANXL_1__MSG_DESCRIPTORS.MSGDSC[35].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_35 CANXL_1__MSG_DESCRIPTORS.MSGDSC[35].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_36 CANXL_1__MSG_DESCRIPTORS.MSGDSC[36].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_36 CANXL_1__MSG_DESCRIPTORS.MSGDSC[36].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_36 CANXL_1__MSG_DESCRIPTORS.MSGDSC[36].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_36 CANXL_1__MSG_DESCRIPTORS.MSGDSC[36].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_36 CANXL_1__MSG_DESCRIPTORS.MSGDSC[36].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_36 CANXL_1__MSG_DESCRIPTORS.MSGDSC[36].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_36 CANXL_1__MSG_DESCRIPTORS.MSGDSC[36].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_37 CANXL_1__MSG_DESCRIPTORS.MSGDSC[37].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_37 CANXL_1__MSG_DESCRIPTORS.MSGDSC[37].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_37 CANXL_1__MSG_DESCRIPTORS.MSGDSC[37].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_37 CANXL_1__MSG_DESCRIPTORS.MSGDSC[37].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_37 CANXL_1__MSG_DESCRIPTORS.MSGDSC[37].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_37 CANXL_1__MSG_DESCRIPTORS.MSGDSC[37].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_37 CANXL_1__MSG_DESCRIPTORS.MSGDSC[37].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_38 CANXL_1__MSG_DESCRIPTORS.MSGDSC[38].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_38 CANXL_1__MSG_DESCRIPTORS.MSGDSC[38].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_38 CANXL_1__MSG_DESCRIPTORS.MSGDSC[38].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_38 CANXL_1__MSG_DESCRIPTORS.MSGDSC[38].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_38 CANXL_1__MSG_DESCRIPTORS.MSGDSC[38].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_38 CANXL_1__MSG_DESCRIPTORS.MSGDSC[38].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_38 CANXL_1__MSG_DESCRIPTORS.MSGDSC[38].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_39 CANXL_1__MSG_DESCRIPTORS.MSGDSC[39].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_39 CANXL_1__MSG_DESCRIPTORS.MSGDSC[39].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_39 CANXL_1__MSG_DESCRIPTORS.MSGDSC[39].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_39 CANXL_1__MSG_DESCRIPTORS.MSGDSC[39].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_39 CANXL_1__MSG_DESCRIPTORS.MSGDSC[39].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_39 CANXL_1__MSG_DESCRIPTORS.MSGDSC[39].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_39 CANXL_1__MSG_DESCRIPTORS.MSGDSC[39].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_40 CANXL_1__MSG_DESCRIPTORS.MSGDSC[40].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_40 CANXL_1__MSG_DESCRIPTORS.MSGDSC[40].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_40 CANXL_1__MSG_DESCRIPTORS.MSGDSC[40].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_40 CANXL_1__MSG_DESCRIPTORS.MSGDSC[40].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_40 CANXL_1__MSG_DESCRIPTORS.MSGDSC[40].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_40 CANXL_1__MSG_DESCRIPTORS.MSGDSC[40].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_40 CANXL_1__MSG_DESCRIPTORS.MSGDSC[40].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_41 CANXL_1__MSG_DESCRIPTORS.MSGDSC[41].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_41 CANXL_1__MSG_DESCRIPTORS.MSGDSC[41].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_41 CANXL_1__MSG_DESCRIPTORS.MSGDSC[41].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_41 CANXL_1__MSG_DESCRIPTORS.MSGDSC[41].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_41 CANXL_1__MSG_DESCRIPTORS.MSGDSC[41].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_41 CANXL_1__MSG_DESCRIPTORS.MSGDSC[41].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_41 CANXL_1__MSG_DESCRIPTORS.MSGDSC[41].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_42 CANXL_1__MSG_DESCRIPTORS.MSGDSC[42].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_42 CANXL_1__MSG_DESCRIPTORS.MSGDSC[42].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_42 CANXL_1__MSG_DESCRIPTORS.MSGDSC[42].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_42 CANXL_1__MSG_DESCRIPTORS.MSGDSC[42].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_42 CANXL_1__MSG_DESCRIPTORS.MSGDSC[42].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_42 CANXL_1__MSG_DESCRIPTORS.MSGDSC[42].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_42 CANXL_1__MSG_DESCRIPTORS.MSGDSC[42].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_43 CANXL_1__MSG_DESCRIPTORS.MSGDSC[43].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_43 CANXL_1__MSG_DESCRIPTORS.MSGDSC[43].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_43 CANXL_1__MSG_DESCRIPTORS.MSGDSC[43].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_43 CANXL_1__MSG_DESCRIPTORS.MSGDSC[43].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_43 CANXL_1__MSG_DESCRIPTORS.MSGDSC[43].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_43 CANXL_1__MSG_DESCRIPTORS.MSGDSC[43].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_43 CANXL_1__MSG_DESCRIPTORS.MSGDSC[43].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_44 CANXL_1__MSG_DESCRIPTORS.MSGDSC[44].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_44 CANXL_1__MSG_DESCRIPTORS.MSGDSC[44].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_44 CANXL_1__MSG_DESCRIPTORS.MSGDSC[44].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_44 CANXL_1__MSG_DESCRIPTORS.MSGDSC[44].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_44 CANXL_1__MSG_DESCRIPTORS.MSGDSC[44].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_44 CANXL_1__MSG_DESCRIPTORS.MSGDSC[44].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_44 CANXL_1__MSG_DESCRIPTORS.MSGDSC[44].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_45 CANXL_1__MSG_DESCRIPTORS.MSGDSC[45].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_45 CANXL_1__MSG_DESCRIPTORS.MSGDSC[45].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_45 CANXL_1__MSG_DESCRIPTORS.MSGDSC[45].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_45 CANXL_1__MSG_DESCRIPTORS.MSGDSC[45].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_45 CANXL_1__MSG_DESCRIPTORS.MSGDSC[45].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_45 CANXL_1__MSG_DESCRIPTORS.MSGDSC[45].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_45 CANXL_1__MSG_DESCRIPTORS.MSGDSC[45].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_46 CANXL_1__MSG_DESCRIPTORS.MSGDSC[46].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_46 CANXL_1__MSG_DESCRIPTORS.MSGDSC[46].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_46 CANXL_1__MSG_DESCRIPTORS.MSGDSC[46].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_46 CANXL_1__MSG_DESCRIPTORS.MSGDSC[46].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_46 CANXL_1__MSG_DESCRIPTORS.MSGDSC[46].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_46 CANXL_1__MSG_DESCRIPTORS.MSGDSC[46].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_46 CANXL_1__MSG_DESCRIPTORS.MSGDSC[46].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_47 CANXL_1__MSG_DESCRIPTORS.MSGDSC[47].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_47 CANXL_1__MSG_DESCRIPTORS.MSGDSC[47].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_47 CANXL_1__MSG_DESCRIPTORS.MSGDSC[47].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_47 CANXL_1__MSG_DESCRIPTORS.MSGDSC[47].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_47 CANXL_1__MSG_DESCRIPTORS.MSGDSC[47].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_47 CANXL_1__MSG_DESCRIPTORS.MSGDSC[47].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_47 CANXL_1__MSG_DESCRIPTORS.MSGDSC[47].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_48 CANXL_1__MSG_DESCRIPTORS.MSGDSC[48].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_48 CANXL_1__MSG_DESCRIPTORS.MSGDSC[48].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_48 CANXL_1__MSG_DESCRIPTORS.MSGDSC[48].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_48 CANXL_1__MSG_DESCRIPTORS.MSGDSC[48].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_48 CANXL_1__MSG_DESCRIPTORS.MSGDSC[48].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_48 CANXL_1__MSG_DESCRIPTORS.MSGDSC[48].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_48 CANXL_1__MSG_DESCRIPTORS.MSGDSC[48].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_49 CANXL_1__MSG_DESCRIPTORS.MSGDSC[49].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_49 CANXL_1__MSG_DESCRIPTORS.MSGDSC[49].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_49 CANXL_1__MSG_DESCRIPTORS.MSGDSC[49].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_49 CANXL_1__MSG_DESCRIPTORS.MSGDSC[49].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_49 CANXL_1__MSG_DESCRIPTORS.MSGDSC[49].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_49 CANXL_1__MSG_DESCRIPTORS.MSGDSC[49].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_49 CANXL_1__MSG_DESCRIPTORS.MSGDSC[49].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_50 CANXL_1__MSG_DESCRIPTORS.MSGDSC[50].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_50 CANXL_1__MSG_DESCRIPTORS.MSGDSC[50].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_50 CANXL_1__MSG_DESCRIPTORS.MSGDSC[50].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_50 CANXL_1__MSG_DESCRIPTORS.MSGDSC[50].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_50 CANXL_1__MSG_DESCRIPTORS.MSGDSC[50].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_50 CANXL_1__MSG_DESCRIPTORS.MSGDSC[50].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_50 CANXL_1__MSG_DESCRIPTORS.MSGDSC[50].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_51 CANXL_1__MSG_DESCRIPTORS.MSGDSC[51].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_51 CANXL_1__MSG_DESCRIPTORS.MSGDSC[51].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_51 CANXL_1__MSG_DESCRIPTORS.MSGDSC[51].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_51 CANXL_1__MSG_DESCRIPTORS.MSGDSC[51].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_51 CANXL_1__MSG_DESCRIPTORS.MSGDSC[51].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_51 CANXL_1__MSG_DESCRIPTORS.MSGDSC[51].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_51 CANXL_1__MSG_DESCRIPTORS.MSGDSC[51].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_52 CANXL_1__MSG_DESCRIPTORS.MSGDSC[52].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_52 CANXL_1__MSG_DESCRIPTORS.MSGDSC[52].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_52 CANXL_1__MSG_DESCRIPTORS.MSGDSC[52].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_52 CANXL_1__MSG_DESCRIPTORS.MSGDSC[52].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_52 CANXL_1__MSG_DESCRIPTORS.MSGDSC[52].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_52 CANXL_1__MSG_DESCRIPTORS.MSGDSC[52].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_52 CANXL_1__MSG_DESCRIPTORS.MSGDSC[52].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_53 CANXL_1__MSG_DESCRIPTORS.MSGDSC[53].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_53 CANXL_1__MSG_DESCRIPTORS.MSGDSC[53].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_53 CANXL_1__MSG_DESCRIPTORS.MSGDSC[53].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_53 CANXL_1__MSG_DESCRIPTORS.MSGDSC[53].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_53 CANXL_1__MSG_DESCRIPTORS.MSGDSC[53].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_53 CANXL_1__MSG_DESCRIPTORS.MSGDSC[53].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_53 CANXL_1__MSG_DESCRIPTORS.MSGDSC[53].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_54 CANXL_1__MSG_DESCRIPTORS.MSGDSC[54].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_54 CANXL_1__MSG_DESCRIPTORS.MSGDSC[54].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_54 CANXL_1__MSG_DESCRIPTORS.MSGDSC[54].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_54 CANXL_1__MSG_DESCRIPTORS.MSGDSC[54].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_54 CANXL_1__MSG_DESCRIPTORS.MSGDSC[54].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_54 CANXL_1__MSG_DESCRIPTORS.MSGDSC[54].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_54 CANXL_1__MSG_DESCRIPTORS.MSGDSC[54].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_55 CANXL_1__MSG_DESCRIPTORS.MSGDSC[55].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_55 CANXL_1__MSG_DESCRIPTORS.MSGDSC[55].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_55 CANXL_1__MSG_DESCRIPTORS.MSGDSC[55].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_55 CANXL_1__MSG_DESCRIPTORS.MSGDSC[55].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_55 CANXL_1__MSG_DESCRIPTORS.MSGDSC[55].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_55 CANXL_1__MSG_DESCRIPTORS.MSGDSC[55].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_55 CANXL_1__MSG_DESCRIPTORS.MSGDSC[55].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_56 CANXL_1__MSG_DESCRIPTORS.MSGDSC[56].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_56 CANXL_1__MSG_DESCRIPTORS.MSGDSC[56].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_56 CANXL_1__MSG_DESCRIPTORS.MSGDSC[56].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_56 CANXL_1__MSG_DESCRIPTORS.MSGDSC[56].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_56 CANXL_1__MSG_DESCRIPTORS.MSGDSC[56].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_56 CANXL_1__MSG_DESCRIPTORS.MSGDSC[56].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_56 CANXL_1__MSG_DESCRIPTORS.MSGDSC[56].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_57 CANXL_1__MSG_DESCRIPTORS.MSGDSC[57].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_57 CANXL_1__MSG_DESCRIPTORS.MSGDSC[57].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_57 CANXL_1__MSG_DESCRIPTORS.MSGDSC[57].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_57 CANXL_1__MSG_DESCRIPTORS.MSGDSC[57].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_57 CANXL_1__MSG_DESCRIPTORS.MSGDSC[57].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_57 CANXL_1__MSG_DESCRIPTORS.MSGDSC[57].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_57 CANXL_1__MSG_DESCRIPTORS.MSGDSC[57].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_58 CANXL_1__MSG_DESCRIPTORS.MSGDSC[58].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_58 CANXL_1__MSG_DESCRIPTORS.MSGDSC[58].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_58 CANXL_1__MSG_DESCRIPTORS.MSGDSC[58].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_58 CANXL_1__MSG_DESCRIPTORS.MSGDSC[58].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_58 CANXL_1__MSG_DESCRIPTORS.MSGDSC[58].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_58 CANXL_1__MSG_DESCRIPTORS.MSGDSC[58].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_58 CANXL_1__MSG_DESCRIPTORS.MSGDSC[58].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_59 CANXL_1__MSG_DESCRIPTORS.MSGDSC[59].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_59 CANXL_1__MSG_DESCRIPTORS.MSGDSC[59].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_59 CANXL_1__MSG_DESCRIPTORS.MSGDSC[59].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_59 CANXL_1__MSG_DESCRIPTORS.MSGDSC[59].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_59 CANXL_1__MSG_DESCRIPTORS.MSGDSC[59].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_59 CANXL_1__MSG_DESCRIPTORS.MSGDSC[59].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_59 CANXL_1__MSG_DESCRIPTORS.MSGDSC[59].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_60 CANXL_1__MSG_DESCRIPTORS.MSGDSC[60].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_60 CANXL_1__MSG_DESCRIPTORS.MSGDSC[60].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_60 CANXL_1__MSG_DESCRIPTORS.MSGDSC[60].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_60 CANXL_1__MSG_DESCRIPTORS.MSGDSC[60].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_60 CANXL_1__MSG_DESCRIPTORS.MSGDSC[60].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_60 CANXL_1__MSG_DESCRIPTORS.MSGDSC[60].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_60 CANXL_1__MSG_DESCRIPTORS.MSGDSC[60].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_61 CANXL_1__MSG_DESCRIPTORS.MSGDSC[61].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_61 CANXL_1__MSG_DESCRIPTORS.MSGDSC[61].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_61 CANXL_1__MSG_DESCRIPTORS.MSGDSC[61].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_61 CANXL_1__MSG_DESCRIPTORS.MSGDSC[61].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_61 CANXL_1__MSG_DESCRIPTORS.MSGDSC[61].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_61 CANXL_1__MSG_DESCRIPTORS.MSGDSC[61].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_61 CANXL_1__MSG_DESCRIPTORS.MSGDSC[61].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_62 CANXL_1__MSG_DESCRIPTORS.MSGDSC[62].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_62 CANXL_1__MSG_DESCRIPTORS.MSGDSC[62].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_62 CANXL_1__MSG_DESCRIPTORS.MSGDSC[62].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_62 CANXL_1__MSG_DESCRIPTORS.MSGDSC[62].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_62 CANXL_1__MSG_DESCRIPTORS.MSGDSC[62].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_62 CANXL_1__MSG_DESCRIPTORS.MSGDSC[62].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_62 CANXL_1__MSG_DESCRIPTORS.MSGDSC[62].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_63 CANXL_1__MSG_DESCRIPTORS.MSGDSC[63].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_63 CANXL_1__MSG_DESCRIPTORS.MSGDSC[63].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_63 CANXL_1__MSG_DESCRIPTORS.MSGDSC[63].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_63 CANXL_1__MSG_DESCRIPTORS.MSGDSC[63].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_63 CANXL_1__MSG_DESCRIPTORS.MSGDSC[63].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_63 CANXL_1__MSG_DESCRIPTORS.MSGDSC[63].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_63 CANXL_1__MSG_DESCRIPTORS.MSGDSC[63].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_64 CANXL_1__MSG_DESCRIPTORS.MSGDSC[64].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_64 CANXL_1__MSG_DESCRIPTORS.MSGDSC[64].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_64 CANXL_1__MSG_DESCRIPTORS.MSGDSC[64].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_64 CANXL_1__MSG_DESCRIPTORS.MSGDSC[64].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_64 CANXL_1__MSG_DESCRIPTORS.MSGDSC[64].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_64 CANXL_1__MSG_DESCRIPTORS.MSGDSC[64].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_64 CANXL_1__MSG_DESCRIPTORS.MSGDSC[64].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_65 CANXL_1__MSG_DESCRIPTORS.MSGDSC[65].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_65 CANXL_1__MSG_DESCRIPTORS.MSGDSC[65].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_65 CANXL_1__MSG_DESCRIPTORS.MSGDSC[65].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_65 CANXL_1__MSG_DESCRIPTORS.MSGDSC[65].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_65 CANXL_1__MSG_DESCRIPTORS.MSGDSC[65].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_65 CANXL_1__MSG_DESCRIPTORS.MSGDSC[65].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_65 CANXL_1__MSG_DESCRIPTORS.MSGDSC[65].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_66 CANXL_1__MSG_DESCRIPTORS.MSGDSC[66].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_66 CANXL_1__MSG_DESCRIPTORS.MSGDSC[66].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_66 CANXL_1__MSG_DESCRIPTORS.MSGDSC[66].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_66 CANXL_1__MSG_DESCRIPTORS.MSGDSC[66].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_66 CANXL_1__MSG_DESCRIPTORS.MSGDSC[66].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_66 CANXL_1__MSG_DESCRIPTORS.MSGDSC[66].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_66 CANXL_1__MSG_DESCRIPTORS.MSGDSC[66].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_67 CANXL_1__MSG_DESCRIPTORS.MSGDSC[67].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_67 CANXL_1__MSG_DESCRIPTORS.MSGDSC[67].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_67 CANXL_1__MSG_DESCRIPTORS.MSGDSC[67].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_67 CANXL_1__MSG_DESCRIPTORS.MSGDSC[67].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_67 CANXL_1__MSG_DESCRIPTORS.MSGDSC[67].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_67 CANXL_1__MSG_DESCRIPTORS.MSGDSC[67].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_67 CANXL_1__MSG_DESCRIPTORS.MSGDSC[67].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_68 CANXL_1__MSG_DESCRIPTORS.MSGDSC[68].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_68 CANXL_1__MSG_DESCRIPTORS.MSGDSC[68].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_68 CANXL_1__MSG_DESCRIPTORS.MSGDSC[68].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_68 CANXL_1__MSG_DESCRIPTORS.MSGDSC[68].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_68 CANXL_1__MSG_DESCRIPTORS.MSGDSC[68].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_68 CANXL_1__MSG_DESCRIPTORS.MSGDSC[68].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_68 CANXL_1__MSG_DESCRIPTORS.MSGDSC[68].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_69 CANXL_1__MSG_DESCRIPTORS.MSGDSC[69].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_69 CANXL_1__MSG_DESCRIPTORS.MSGDSC[69].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_69 CANXL_1__MSG_DESCRIPTORS.MSGDSC[69].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_69 CANXL_1__MSG_DESCRIPTORS.MSGDSC[69].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_69 CANXL_1__MSG_DESCRIPTORS.MSGDSC[69].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_69 CANXL_1__MSG_DESCRIPTORS.MSGDSC[69].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_69 CANXL_1__MSG_DESCRIPTORS.MSGDSC[69].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_70 CANXL_1__MSG_DESCRIPTORS.MSGDSC[70].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_70 CANXL_1__MSG_DESCRIPTORS.MSGDSC[70].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_70 CANXL_1__MSG_DESCRIPTORS.MSGDSC[70].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_70 CANXL_1__MSG_DESCRIPTORS.MSGDSC[70].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_70 CANXL_1__MSG_DESCRIPTORS.MSGDSC[70].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_70 CANXL_1__MSG_DESCRIPTORS.MSGDSC[70].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_70 CANXL_1__MSG_DESCRIPTORS.MSGDSC[70].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_71 CANXL_1__MSG_DESCRIPTORS.MSGDSC[71].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_71 CANXL_1__MSG_DESCRIPTORS.MSGDSC[71].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_71 CANXL_1__MSG_DESCRIPTORS.MSGDSC[71].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_71 CANXL_1__MSG_DESCRIPTORS.MSGDSC[71].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_71 CANXL_1__MSG_DESCRIPTORS.MSGDSC[71].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_71 CANXL_1__MSG_DESCRIPTORS.MSGDSC[71].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_71 CANXL_1__MSG_DESCRIPTORS.MSGDSC[71].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_72 CANXL_1__MSG_DESCRIPTORS.MSGDSC[72].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_72 CANXL_1__MSG_DESCRIPTORS.MSGDSC[72].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_72 CANXL_1__MSG_DESCRIPTORS.MSGDSC[72].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_72 CANXL_1__MSG_DESCRIPTORS.MSGDSC[72].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_72 CANXL_1__MSG_DESCRIPTORS.MSGDSC[72].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_72 CANXL_1__MSG_DESCRIPTORS.MSGDSC[72].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_72 CANXL_1__MSG_DESCRIPTORS.MSGDSC[72].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_73 CANXL_1__MSG_DESCRIPTORS.MSGDSC[73].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_73 CANXL_1__MSG_DESCRIPTORS.MSGDSC[73].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_73 CANXL_1__MSG_DESCRIPTORS.MSGDSC[73].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_73 CANXL_1__MSG_DESCRIPTORS.MSGDSC[73].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_73 CANXL_1__MSG_DESCRIPTORS.MSGDSC[73].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_73 CANXL_1__MSG_DESCRIPTORS.MSGDSC[73].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_73 CANXL_1__MSG_DESCRIPTORS.MSGDSC[73].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_74 CANXL_1__MSG_DESCRIPTORS.MSGDSC[74].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_74 CANXL_1__MSG_DESCRIPTORS.MSGDSC[74].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_74 CANXL_1__MSG_DESCRIPTORS.MSGDSC[74].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_74 CANXL_1__MSG_DESCRIPTORS.MSGDSC[74].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_74 CANXL_1__MSG_DESCRIPTORS.MSGDSC[74].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_74 CANXL_1__MSG_DESCRIPTORS.MSGDSC[74].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_74 CANXL_1__MSG_DESCRIPTORS.MSGDSC[74].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_75 CANXL_1__MSG_DESCRIPTORS.MSGDSC[75].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_75 CANXL_1__MSG_DESCRIPTORS.MSGDSC[75].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_75 CANXL_1__MSG_DESCRIPTORS.MSGDSC[75].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_75 CANXL_1__MSG_DESCRIPTORS.MSGDSC[75].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_75 CANXL_1__MSG_DESCRIPTORS.MSGDSC[75].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_75 CANXL_1__MSG_DESCRIPTORS.MSGDSC[75].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_75 CANXL_1__MSG_DESCRIPTORS.MSGDSC[75].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_76 CANXL_1__MSG_DESCRIPTORS.MSGDSC[76].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_76 CANXL_1__MSG_DESCRIPTORS.MSGDSC[76].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_76 CANXL_1__MSG_DESCRIPTORS.MSGDSC[76].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_76 CANXL_1__MSG_DESCRIPTORS.MSGDSC[76].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_76 CANXL_1__MSG_DESCRIPTORS.MSGDSC[76].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_76 CANXL_1__MSG_DESCRIPTORS.MSGDSC[76].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_76 CANXL_1__MSG_DESCRIPTORS.MSGDSC[76].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_77 CANXL_1__MSG_DESCRIPTORS.MSGDSC[77].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_77 CANXL_1__MSG_DESCRIPTORS.MSGDSC[77].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_77 CANXL_1__MSG_DESCRIPTORS.MSGDSC[77].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_77 CANXL_1__MSG_DESCRIPTORS.MSGDSC[77].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_77 CANXL_1__MSG_DESCRIPTORS.MSGDSC[77].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_77 CANXL_1__MSG_DESCRIPTORS.MSGDSC[77].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_77 CANXL_1__MSG_DESCRIPTORS.MSGDSC[77].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_78 CANXL_1__MSG_DESCRIPTORS.MSGDSC[78].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_78 CANXL_1__MSG_DESCRIPTORS.MSGDSC[78].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_78 CANXL_1__MSG_DESCRIPTORS.MSGDSC[78].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_78 CANXL_1__MSG_DESCRIPTORS.MSGDSC[78].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_78 CANXL_1__MSG_DESCRIPTORS.MSGDSC[78].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_78 CANXL_1__MSG_DESCRIPTORS.MSGDSC[78].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_78 CANXL_1__MSG_DESCRIPTORS.MSGDSC[78].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_79 CANXL_1__MSG_DESCRIPTORS.MSGDSC[79].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_79 CANXL_1__MSG_DESCRIPTORS.MSGDSC[79].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_79 CANXL_1__MSG_DESCRIPTORS.MSGDSC[79].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_79 CANXL_1__MSG_DESCRIPTORS.MSGDSC[79].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_79 CANXL_1__MSG_DESCRIPTORS.MSGDSC[79].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_79 CANXL_1__MSG_DESCRIPTORS.MSGDSC[79].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_79 CANXL_1__MSG_DESCRIPTORS.MSGDSC[79].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_80 CANXL_1__MSG_DESCRIPTORS.MSGDSC[80].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_80 CANXL_1__MSG_DESCRIPTORS.MSGDSC[80].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_80 CANXL_1__MSG_DESCRIPTORS.MSGDSC[80].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_80 CANXL_1__MSG_DESCRIPTORS.MSGDSC[80].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_80 CANXL_1__MSG_DESCRIPTORS.MSGDSC[80].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_80 CANXL_1__MSG_DESCRIPTORS.MSGDSC[80].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_80 CANXL_1__MSG_DESCRIPTORS.MSGDSC[80].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_81 CANXL_1__MSG_DESCRIPTORS.MSGDSC[81].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_81 CANXL_1__MSG_DESCRIPTORS.MSGDSC[81].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_81 CANXL_1__MSG_DESCRIPTORS.MSGDSC[81].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_81 CANXL_1__MSG_DESCRIPTORS.MSGDSC[81].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_81 CANXL_1__MSG_DESCRIPTORS.MSGDSC[81].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_81 CANXL_1__MSG_DESCRIPTORS.MSGDSC[81].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_81 CANXL_1__MSG_DESCRIPTORS.MSGDSC[81].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_82 CANXL_1__MSG_DESCRIPTORS.MSGDSC[82].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_82 CANXL_1__MSG_DESCRIPTORS.MSGDSC[82].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_82 CANXL_1__MSG_DESCRIPTORS.MSGDSC[82].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_82 CANXL_1__MSG_DESCRIPTORS.MSGDSC[82].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_82 CANXL_1__MSG_DESCRIPTORS.MSGDSC[82].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_82 CANXL_1__MSG_DESCRIPTORS.MSGDSC[82].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_82 CANXL_1__MSG_DESCRIPTORS.MSGDSC[82].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_83 CANXL_1__MSG_DESCRIPTORS.MSGDSC[83].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_83 CANXL_1__MSG_DESCRIPTORS.MSGDSC[83].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_83 CANXL_1__MSG_DESCRIPTORS.MSGDSC[83].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_83 CANXL_1__MSG_DESCRIPTORS.MSGDSC[83].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_83 CANXL_1__MSG_DESCRIPTORS.MSGDSC[83].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_83 CANXL_1__MSG_DESCRIPTORS.MSGDSC[83].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_83 CANXL_1__MSG_DESCRIPTORS.MSGDSC[83].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_84 CANXL_1__MSG_DESCRIPTORS.MSGDSC[84].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_84 CANXL_1__MSG_DESCRIPTORS.MSGDSC[84].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_84 CANXL_1__MSG_DESCRIPTORS.MSGDSC[84].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_84 CANXL_1__MSG_DESCRIPTORS.MSGDSC[84].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_84 CANXL_1__MSG_DESCRIPTORS.MSGDSC[84].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_84 CANXL_1__MSG_DESCRIPTORS.MSGDSC[84].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_84 CANXL_1__MSG_DESCRIPTORS.MSGDSC[84].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_85 CANXL_1__MSG_DESCRIPTORS.MSGDSC[85].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_85 CANXL_1__MSG_DESCRIPTORS.MSGDSC[85].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_85 CANXL_1__MSG_DESCRIPTORS.MSGDSC[85].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_85 CANXL_1__MSG_DESCRIPTORS.MSGDSC[85].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_85 CANXL_1__MSG_DESCRIPTORS.MSGDSC[85].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_85 CANXL_1__MSG_DESCRIPTORS.MSGDSC[85].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_85 CANXL_1__MSG_DESCRIPTORS.MSGDSC[85].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_86 CANXL_1__MSG_DESCRIPTORS.MSGDSC[86].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_86 CANXL_1__MSG_DESCRIPTORS.MSGDSC[86].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_86 CANXL_1__MSG_DESCRIPTORS.MSGDSC[86].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_86 CANXL_1__MSG_DESCRIPTORS.MSGDSC[86].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_86 CANXL_1__MSG_DESCRIPTORS.MSGDSC[86].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_86 CANXL_1__MSG_DESCRIPTORS.MSGDSC[86].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_86 CANXL_1__MSG_DESCRIPTORS.MSGDSC[86].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_87 CANXL_1__MSG_DESCRIPTORS.MSGDSC[87].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_87 CANXL_1__MSG_DESCRIPTORS.MSGDSC[87].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_87 CANXL_1__MSG_DESCRIPTORS.MSGDSC[87].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_87 CANXL_1__MSG_DESCRIPTORS.MSGDSC[87].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_87 CANXL_1__MSG_DESCRIPTORS.MSGDSC[87].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_87 CANXL_1__MSG_DESCRIPTORS.MSGDSC[87].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_87 CANXL_1__MSG_DESCRIPTORS.MSGDSC[87].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_88 CANXL_1__MSG_DESCRIPTORS.MSGDSC[88].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_88 CANXL_1__MSG_DESCRIPTORS.MSGDSC[88].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_88 CANXL_1__MSG_DESCRIPTORS.MSGDSC[88].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_88 CANXL_1__MSG_DESCRIPTORS.MSGDSC[88].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_88 CANXL_1__MSG_DESCRIPTORS.MSGDSC[88].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_88 CANXL_1__MSG_DESCRIPTORS.MSGDSC[88].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_88 CANXL_1__MSG_DESCRIPTORS.MSGDSC[88].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_89 CANXL_1__MSG_DESCRIPTORS.MSGDSC[89].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_89 CANXL_1__MSG_DESCRIPTORS.MSGDSC[89].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_89 CANXL_1__MSG_DESCRIPTORS.MSGDSC[89].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_89 CANXL_1__MSG_DESCRIPTORS.MSGDSC[89].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_89 CANXL_1__MSG_DESCRIPTORS.MSGDSC[89].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_89 CANXL_1__MSG_DESCRIPTORS.MSGDSC[89].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_89 CANXL_1__MSG_DESCRIPTORS.MSGDSC[89].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_90 CANXL_1__MSG_DESCRIPTORS.MSGDSC[90].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_90 CANXL_1__MSG_DESCRIPTORS.MSGDSC[90].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_90 CANXL_1__MSG_DESCRIPTORS.MSGDSC[90].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_90 CANXL_1__MSG_DESCRIPTORS.MSGDSC[90].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_90 CANXL_1__MSG_DESCRIPTORS.MSGDSC[90].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_90 CANXL_1__MSG_DESCRIPTORS.MSGDSC[90].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_90 CANXL_1__MSG_DESCRIPTORS.MSGDSC[90].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_91 CANXL_1__MSG_DESCRIPTORS.MSGDSC[91].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_91 CANXL_1__MSG_DESCRIPTORS.MSGDSC[91].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_91 CANXL_1__MSG_DESCRIPTORS.MSGDSC[91].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_91 CANXL_1__MSG_DESCRIPTORS.MSGDSC[91].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_91 CANXL_1__MSG_DESCRIPTORS.MSGDSC[91].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_91 CANXL_1__MSG_DESCRIPTORS.MSGDSC[91].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_91 CANXL_1__MSG_DESCRIPTORS.MSGDSC[91].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_92 CANXL_1__MSG_DESCRIPTORS.MSGDSC[92].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_92 CANXL_1__MSG_DESCRIPTORS.MSGDSC[92].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_92 CANXL_1__MSG_DESCRIPTORS.MSGDSC[92].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_92 CANXL_1__MSG_DESCRIPTORS.MSGDSC[92].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_92 CANXL_1__MSG_DESCRIPTORS.MSGDSC[92].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_92 CANXL_1__MSG_DESCRIPTORS.MSGDSC[92].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_92 CANXL_1__MSG_DESCRIPTORS.MSGDSC[92].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_93 CANXL_1__MSG_DESCRIPTORS.MSGDSC[93].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_93 CANXL_1__MSG_DESCRIPTORS.MSGDSC[93].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_93 CANXL_1__MSG_DESCRIPTORS.MSGDSC[93].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_93 CANXL_1__MSG_DESCRIPTORS.MSGDSC[93].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_93 CANXL_1__MSG_DESCRIPTORS.MSGDSC[93].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_93 CANXL_1__MSG_DESCRIPTORS.MSGDSC[93].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_93 CANXL_1__MSG_DESCRIPTORS.MSGDSC[93].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_94 CANXL_1__MSG_DESCRIPTORS.MSGDSC[94].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_94 CANXL_1__MSG_DESCRIPTORS.MSGDSC[94].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_94 CANXL_1__MSG_DESCRIPTORS.MSGDSC[94].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_94 CANXL_1__MSG_DESCRIPTORS.MSGDSC[94].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_94 CANXL_1__MSG_DESCRIPTORS.MSGDSC[94].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_94 CANXL_1__MSG_DESCRIPTORS.MSGDSC[94].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_94 CANXL_1__MSG_DESCRIPTORS.MSGDSC[94].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_95 CANXL_1__MSG_DESCRIPTORS.MSGDSC[95].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_95 CANXL_1__MSG_DESCRIPTORS.MSGDSC[95].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_95 CANXL_1__MSG_DESCRIPTORS.MSGDSC[95].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_95 CANXL_1__MSG_DESCRIPTORS.MSGDSC[95].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_95 CANXL_1__MSG_DESCRIPTORS.MSGDSC[95].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_95 CANXL_1__MSG_DESCRIPTORS.MSGDSC[95].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_95 CANXL_1__MSG_DESCRIPTORS.MSGDSC[95].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_96 CANXL_1__MSG_DESCRIPTORS.MSGDSC[96].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_96 CANXL_1__MSG_DESCRIPTORS.MSGDSC[96].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_96 CANXL_1__MSG_DESCRIPTORS.MSGDSC[96].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_96 CANXL_1__MSG_DESCRIPTORS.MSGDSC[96].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_96 CANXL_1__MSG_DESCRIPTORS.MSGDSC[96].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_96 CANXL_1__MSG_DESCRIPTORS.MSGDSC[96].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_96 CANXL_1__MSG_DESCRIPTORS.MSGDSC[96].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_97 CANXL_1__MSG_DESCRIPTORS.MSGDSC[97].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_97 CANXL_1__MSG_DESCRIPTORS.MSGDSC[97].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_97 CANXL_1__MSG_DESCRIPTORS.MSGDSC[97].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_97 CANXL_1__MSG_DESCRIPTORS.MSGDSC[97].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_97 CANXL_1__MSG_DESCRIPTORS.MSGDSC[97].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_97 CANXL_1__MSG_DESCRIPTORS.MSGDSC[97].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_97 CANXL_1__MSG_DESCRIPTORS.MSGDSC[97].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_98 CANXL_1__MSG_DESCRIPTORS.MSGDSC[98].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_98 CANXL_1__MSG_DESCRIPTORS.MSGDSC[98].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_98 CANXL_1__MSG_DESCRIPTORS.MSGDSC[98].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_98 CANXL_1__MSG_DESCRIPTORS.MSGDSC[98].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_98 CANXL_1__MSG_DESCRIPTORS.MSGDSC[98].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_98 CANXL_1__MSG_DESCRIPTORS.MSGDSC[98].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_98 CANXL_1__MSG_DESCRIPTORS.MSGDSC[98].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_99 CANXL_1__MSG_DESCRIPTORS.MSGDSC[99].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_99 CANXL_1__MSG_DESCRIPTORS.MSGDSC[99].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_99 CANXL_1__MSG_DESCRIPTORS.MSGDSC[99].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_99 CANXL_1__MSG_DESCRIPTORS.MSGDSC[99].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_99 CANXL_1__MSG_DESCRIPTORS.MSGDSC[99].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_99 CANXL_1__MSG_DESCRIPTORS.MSGDSC[99].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_99 CANXL_1__MSG_DESCRIPTORS.MSGDSC[99].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_100 CANXL_1__MSG_DESCRIPTORS.MSGDSC[100].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_100 CANXL_1__MSG_DESCRIPTORS.MSGDSC[100].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_100 CANXL_1__MSG_DESCRIPTORS.MSGDSC[100].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_100 CANXL_1__MSG_DESCRIPTORS.MSGDSC[100].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_100 CANXL_1__MSG_DESCRIPTORS.MSGDSC[100].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_100 CANXL_1__MSG_DESCRIPTORS.MSGDSC[100].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_100 CANXL_1__MSG_DESCRIPTORS.MSGDSC[100].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_101 CANXL_1__MSG_DESCRIPTORS.MSGDSC[101].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_101 CANXL_1__MSG_DESCRIPTORS.MSGDSC[101].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_101 CANXL_1__MSG_DESCRIPTORS.MSGDSC[101].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_101 CANXL_1__MSG_DESCRIPTORS.MSGDSC[101].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_101 CANXL_1__MSG_DESCRIPTORS.MSGDSC[101].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_101 CANXL_1__MSG_DESCRIPTORS.MSGDSC[101].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_101 CANXL_1__MSG_DESCRIPTORS.MSGDSC[101].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_102 CANXL_1__MSG_DESCRIPTORS.MSGDSC[102].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_102 CANXL_1__MSG_DESCRIPTORS.MSGDSC[102].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_102 CANXL_1__MSG_DESCRIPTORS.MSGDSC[102].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_102 CANXL_1__MSG_DESCRIPTORS.MSGDSC[102].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_102 CANXL_1__MSG_DESCRIPTORS.MSGDSC[102].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_102 CANXL_1__MSG_DESCRIPTORS.MSGDSC[102].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_102 CANXL_1__MSG_DESCRIPTORS.MSGDSC[102].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_103 CANXL_1__MSG_DESCRIPTORS.MSGDSC[103].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_103 CANXL_1__MSG_DESCRIPTORS.MSGDSC[103].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_103 CANXL_1__MSG_DESCRIPTORS.MSGDSC[103].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_103 CANXL_1__MSG_DESCRIPTORS.MSGDSC[103].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_103 CANXL_1__MSG_DESCRIPTORS.MSGDSC[103].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_103 CANXL_1__MSG_DESCRIPTORS.MSGDSC[103].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_103 CANXL_1__MSG_DESCRIPTORS.MSGDSC[103].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_104 CANXL_1__MSG_DESCRIPTORS.MSGDSC[104].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_104 CANXL_1__MSG_DESCRIPTORS.MSGDSC[104].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_104 CANXL_1__MSG_DESCRIPTORS.MSGDSC[104].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_104 CANXL_1__MSG_DESCRIPTORS.MSGDSC[104].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_104 CANXL_1__MSG_DESCRIPTORS.MSGDSC[104].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_104 CANXL_1__MSG_DESCRIPTORS.MSGDSC[104].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_104 CANXL_1__MSG_DESCRIPTORS.MSGDSC[104].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_105 CANXL_1__MSG_DESCRIPTORS.MSGDSC[105].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_105 CANXL_1__MSG_DESCRIPTORS.MSGDSC[105].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_105 CANXL_1__MSG_DESCRIPTORS.MSGDSC[105].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_105 CANXL_1__MSG_DESCRIPTORS.MSGDSC[105].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_105 CANXL_1__MSG_DESCRIPTORS.MSGDSC[105].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_105 CANXL_1__MSG_DESCRIPTORS.MSGDSC[105].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_105 CANXL_1__MSG_DESCRIPTORS.MSGDSC[105].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_106 CANXL_1__MSG_DESCRIPTORS.MSGDSC[106].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_106 CANXL_1__MSG_DESCRIPTORS.MSGDSC[106].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_106 CANXL_1__MSG_DESCRIPTORS.MSGDSC[106].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_106 CANXL_1__MSG_DESCRIPTORS.MSGDSC[106].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_106 CANXL_1__MSG_DESCRIPTORS.MSGDSC[106].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_106 CANXL_1__MSG_DESCRIPTORS.MSGDSC[106].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_106 CANXL_1__MSG_DESCRIPTORS.MSGDSC[106].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_107 CANXL_1__MSG_DESCRIPTORS.MSGDSC[107].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_107 CANXL_1__MSG_DESCRIPTORS.MSGDSC[107].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_107 CANXL_1__MSG_DESCRIPTORS.MSGDSC[107].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_107 CANXL_1__MSG_DESCRIPTORS.MSGDSC[107].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_107 CANXL_1__MSG_DESCRIPTORS.MSGDSC[107].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_107 CANXL_1__MSG_DESCRIPTORS.MSGDSC[107].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_107 CANXL_1__MSG_DESCRIPTORS.MSGDSC[107].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_108 CANXL_1__MSG_DESCRIPTORS.MSGDSC[108].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_108 CANXL_1__MSG_DESCRIPTORS.MSGDSC[108].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_108 CANXL_1__MSG_DESCRIPTORS.MSGDSC[108].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_108 CANXL_1__MSG_DESCRIPTORS.MSGDSC[108].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_108 CANXL_1__MSG_DESCRIPTORS.MSGDSC[108].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_108 CANXL_1__MSG_DESCRIPTORS.MSGDSC[108].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_108 CANXL_1__MSG_DESCRIPTORS.MSGDSC[108].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_109 CANXL_1__MSG_DESCRIPTORS.MSGDSC[109].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_109 CANXL_1__MSG_DESCRIPTORS.MSGDSC[109].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_109 CANXL_1__MSG_DESCRIPTORS.MSGDSC[109].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_109 CANXL_1__MSG_DESCRIPTORS.MSGDSC[109].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_109 CANXL_1__MSG_DESCRIPTORS.MSGDSC[109].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_109 CANXL_1__MSG_DESCRIPTORS.MSGDSC[109].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_109 CANXL_1__MSG_DESCRIPTORS.MSGDSC[109].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_110 CANXL_1__MSG_DESCRIPTORS.MSGDSC[110].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_110 CANXL_1__MSG_DESCRIPTORS.MSGDSC[110].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_110 CANXL_1__MSG_DESCRIPTORS.MSGDSC[110].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_110 CANXL_1__MSG_DESCRIPTORS.MSGDSC[110].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_110 CANXL_1__MSG_DESCRIPTORS.MSGDSC[110].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_110 CANXL_1__MSG_DESCRIPTORS.MSGDSC[110].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_110 CANXL_1__MSG_DESCRIPTORS.MSGDSC[110].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_111 CANXL_1__MSG_DESCRIPTORS.MSGDSC[111].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_111 CANXL_1__MSG_DESCRIPTORS.MSGDSC[111].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_111 CANXL_1__MSG_DESCRIPTORS.MSGDSC[111].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_111 CANXL_1__MSG_DESCRIPTORS.MSGDSC[111].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_111 CANXL_1__MSG_DESCRIPTORS.MSGDSC[111].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_111 CANXL_1__MSG_DESCRIPTORS.MSGDSC[111].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_111 CANXL_1__MSG_DESCRIPTORS.MSGDSC[111].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_112 CANXL_1__MSG_DESCRIPTORS.MSGDSC[112].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_112 CANXL_1__MSG_DESCRIPTORS.MSGDSC[112].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_112 CANXL_1__MSG_DESCRIPTORS.MSGDSC[112].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_112 CANXL_1__MSG_DESCRIPTORS.MSGDSC[112].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_112 CANXL_1__MSG_DESCRIPTORS.MSGDSC[112].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_112 CANXL_1__MSG_DESCRIPTORS.MSGDSC[112].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_112 CANXL_1__MSG_DESCRIPTORS.MSGDSC[112].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_113 CANXL_1__MSG_DESCRIPTORS.MSGDSC[113].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_113 CANXL_1__MSG_DESCRIPTORS.MSGDSC[113].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_113 CANXL_1__MSG_DESCRIPTORS.MSGDSC[113].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_113 CANXL_1__MSG_DESCRIPTORS.MSGDSC[113].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_113 CANXL_1__MSG_DESCRIPTORS.MSGDSC[113].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_113 CANXL_1__MSG_DESCRIPTORS.MSGDSC[113].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_113 CANXL_1__MSG_DESCRIPTORS.MSGDSC[113].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_114 CANXL_1__MSG_DESCRIPTORS.MSGDSC[114].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_114 CANXL_1__MSG_DESCRIPTORS.MSGDSC[114].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_114 CANXL_1__MSG_DESCRIPTORS.MSGDSC[114].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_114 CANXL_1__MSG_DESCRIPTORS.MSGDSC[114].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_114 CANXL_1__MSG_DESCRIPTORS.MSGDSC[114].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_114 CANXL_1__MSG_DESCRIPTORS.MSGDSC[114].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_114 CANXL_1__MSG_DESCRIPTORS.MSGDSC[114].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_115 CANXL_1__MSG_DESCRIPTORS.MSGDSC[115].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_115 CANXL_1__MSG_DESCRIPTORS.MSGDSC[115].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_115 CANXL_1__MSG_DESCRIPTORS.MSGDSC[115].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_115 CANXL_1__MSG_DESCRIPTORS.MSGDSC[115].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_115 CANXL_1__MSG_DESCRIPTORS.MSGDSC[115].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_115 CANXL_1__MSG_DESCRIPTORS.MSGDSC[115].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_115 CANXL_1__MSG_DESCRIPTORS.MSGDSC[115].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_116 CANXL_1__MSG_DESCRIPTORS.MSGDSC[116].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_116 CANXL_1__MSG_DESCRIPTORS.MSGDSC[116].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_116 CANXL_1__MSG_DESCRIPTORS.MSGDSC[116].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_116 CANXL_1__MSG_DESCRIPTORS.MSGDSC[116].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_116 CANXL_1__MSG_DESCRIPTORS.MSGDSC[116].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_116 CANXL_1__MSG_DESCRIPTORS.MSGDSC[116].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_116 CANXL_1__MSG_DESCRIPTORS.MSGDSC[116].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_117 CANXL_1__MSG_DESCRIPTORS.MSGDSC[117].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_117 CANXL_1__MSG_DESCRIPTORS.MSGDSC[117].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_117 CANXL_1__MSG_DESCRIPTORS.MSGDSC[117].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_117 CANXL_1__MSG_DESCRIPTORS.MSGDSC[117].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_117 CANXL_1__MSG_DESCRIPTORS.MSGDSC[117].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_117 CANXL_1__MSG_DESCRIPTORS.MSGDSC[117].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_117 CANXL_1__MSG_DESCRIPTORS.MSGDSC[117].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_118 CANXL_1__MSG_DESCRIPTORS.MSGDSC[118].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_118 CANXL_1__MSG_DESCRIPTORS.MSGDSC[118].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_118 CANXL_1__MSG_DESCRIPTORS.MSGDSC[118].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_118 CANXL_1__MSG_DESCRIPTORS.MSGDSC[118].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_118 CANXL_1__MSG_DESCRIPTORS.MSGDSC[118].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_118 CANXL_1__MSG_DESCRIPTORS.MSGDSC[118].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_118 CANXL_1__MSG_DESCRIPTORS.MSGDSC[118].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_119 CANXL_1__MSG_DESCRIPTORS.MSGDSC[119].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_119 CANXL_1__MSG_DESCRIPTORS.MSGDSC[119].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_119 CANXL_1__MSG_DESCRIPTORS.MSGDSC[119].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_119 CANXL_1__MSG_DESCRIPTORS.MSGDSC[119].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_119 CANXL_1__MSG_DESCRIPTORS.MSGDSC[119].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_119 CANXL_1__MSG_DESCRIPTORS.MSGDSC[119].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_119 CANXL_1__MSG_DESCRIPTORS.MSGDSC[119].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_120 CANXL_1__MSG_DESCRIPTORS.MSGDSC[120].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_120 CANXL_1__MSG_DESCRIPTORS.MSGDSC[120].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_120 CANXL_1__MSG_DESCRIPTORS.MSGDSC[120].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_120 CANXL_1__MSG_DESCRIPTORS.MSGDSC[120].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_120 CANXL_1__MSG_DESCRIPTORS.MSGDSC[120].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_120 CANXL_1__MSG_DESCRIPTORS.MSGDSC[120].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_120 CANXL_1__MSG_DESCRIPTORS.MSGDSC[120].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_121 CANXL_1__MSG_DESCRIPTORS.MSGDSC[121].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_121 CANXL_1__MSG_DESCRIPTORS.MSGDSC[121].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_121 CANXL_1__MSG_DESCRIPTORS.MSGDSC[121].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_121 CANXL_1__MSG_DESCRIPTORS.MSGDSC[121].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_121 CANXL_1__MSG_DESCRIPTORS.MSGDSC[121].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_121 CANXL_1__MSG_DESCRIPTORS.MSGDSC[121].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_121 CANXL_1__MSG_DESCRIPTORS.MSGDSC[121].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_122 CANXL_1__MSG_DESCRIPTORS.MSGDSC[122].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_122 CANXL_1__MSG_DESCRIPTORS.MSGDSC[122].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_122 CANXL_1__MSG_DESCRIPTORS.MSGDSC[122].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_122 CANXL_1__MSG_DESCRIPTORS.MSGDSC[122].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_122 CANXL_1__MSG_DESCRIPTORS.MSGDSC[122].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_122 CANXL_1__MSG_DESCRIPTORS.MSGDSC[122].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_122 CANXL_1__MSG_DESCRIPTORS.MSGDSC[122].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_123 CANXL_1__MSG_DESCRIPTORS.MSGDSC[123].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_123 CANXL_1__MSG_DESCRIPTORS.MSGDSC[123].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_123 CANXL_1__MSG_DESCRIPTORS.MSGDSC[123].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_123 CANXL_1__MSG_DESCRIPTORS.MSGDSC[123].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_123 CANXL_1__MSG_DESCRIPTORS.MSGDSC[123].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_123 CANXL_1__MSG_DESCRIPTORS.MSGDSC[123].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_123 CANXL_1__MSG_DESCRIPTORS.MSGDSC[123].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_124 CANXL_1__MSG_DESCRIPTORS.MSGDSC[124].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_124 CANXL_1__MSG_DESCRIPTORS.MSGDSC[124].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_124 CANXL_1__MSG_DESCRIPTORS.MSGDSC[124].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_124 CANXL_1__MSG_DESCRIPTORS.MSGDSC[124].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_124 CANXL_1__MSG_DESCRIPTORS.MSGDSC[124].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_124 CANXL_1__MSG_DESCRIPTORS.MSGDSC[124].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_124 CANXL_1__MSG_DESCRIPTORS.MSGDSC[124].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_125 CANXL_1__MSG_DESCRIPTORS.MSGDSC[125].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_125 CANXL_1__MSG_DESCRIPTORS.MSGDSC[125].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_125 CANXL_1__MSG_DESCRIPTORS.MSGDSC[125].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_125 CANXL_1__MSG_DESCRIPTORS.MSGDSC[125].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_125 CANXL_1__MSG_DESCRIPTORS.MSGDSC[125].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_125 CANXL_1__MSG_DESCRIPTORS.MSGDSC[125].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_125 CANXL_1__MSG_DESCRIPTORS.MSGDSC[125].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_126 CANXL_1__MSG_DESCRIPTORS.MSGDSC[126].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_126 CANXL_1__MSG_DESCRIPTORS.MSGDSC[126].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_126 CANXL_1__MSG_DESCRIPTORS.MSGDSC[126].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_126 CANXL_1__MSG_DESCRIPTORS.MSGDSC[126].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_126 CANXL_1__MSG_DESCRIPTORS.MSGDSC[126].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_126 CANXL_1__MSG_DESCRIPTORS.MSGDSC[126].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_126 CANXL_1__MSG_DESCRIPTORS.MSGDSC[126].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXCTRL_127 CANXL_1__MSG_DESCRIPTORS.MSGDSC[127].RXCTRL.R /* Message Descriptor Control (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1FD_127 CANXL_1__MSG_DESCRIPTORS.MSGDSC[127].CFG1.MDFLT1FD.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT1XL_127 CANXL_1__MSG_DESCRIPTORS.MSGDSC[127].CFG1.MDFLT1XL.R /* Message Descriptor Filter Configuration 1 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2FD_127 CANXL_1__MSG_DESCRIPTORS.MSGDSC[127].CFG2.MDFLT2FD.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In FD Mode) */
#define CANXL_1__MSG_DESCRIPTORS_MDFLT2XL_127 CANXL_1__MSG_DESCRIPTORS.MSGDSC[127].CFG2.MDFLT2XL.R /* Message Descriptor Filter Configuration 2 (In Rx Mode/In XL Mode) */
#define CANXL_1__MSG_DESCRIPTORS_RXLSTPTR_127 CANXL_1__MSG_DESCRIPTORS.MSGDSC[127].LSTPNT.RXLSTPTR.R /* Message List Pointer (In Rx Mode) */
#define CANXL_1__MSG_DESCRIPTORS_TXLSTPTR_127 CANXL_1__MSG_DESCRIPTORS.MSGDSC[127].LSTPNT.TXLSTPTR.R /* Message List Pointer (In Tx Mode) */

/* CANXL_RAMECC */
#define CANXL_0__RAMECC_DRMEC CANXL_0__RAMECC.DRMEC.R      /* Data RAM Error Control */
#define CANXL_0__RAMECC_DRERRIA CANXL_0__RAMECC.DRERRIA.R  /* Data RAM Error Injection Address */
#define CANXL_0__RAMECC_DRERRIDP CANXL_0__RAMECC.DRERRIDP.R /* Data RAM Error Injection Data Pattern */
#define CANXL_0__RAMECC_DRERRIPP CANXL_0__RAMECC.DRERRIPP.R /* Data RAM Error Injection Parity Pattern */
#define CANXL_0__RAMECC_DRRERRA CANXL_0__RAMECC.DRRERRA.R  /* Data RAM Error Report Address */
#define CANXL_0__RAMECC_DRRERRD CANXL_0__RAMECC.DRRERRD.R  /* Data RAM Error Report Data */
#define CANXL_0__RAMECC_DRRERRSYN CANXL_0__RAMECC.DRRERRSYN.R /* Data RAM Error Report Syndrome */
#define CANXL_0__RAMECC_DRERRS CANXL_0__RAMECC.DRERRS.R    /* Data RAM Error Status */

#define CANXL_1__RAMECC_DRMEC CANXL_1__RAMECC.DRMEC.R      /* Data RAM Error Control */
#define CANXL_1__RAMECC_DRERRIA CANXL_1__RAMECC.DRERRIA.R  /* Data RAM Error Injection Address */
#define CANXL_1__RAMECC_DRERRIDP CANXL_1__RAMECC.DRERRIDP.R /* Data RAM Error Injection Data Pattern */
#define CANXL_1__RAMECC_DRERRIPP CANXL_1__RAMECC.DRERRIPP.R /* Data RAM Error Injection Parity Pattern */
#define CANXL_1__RAMECC_DRRERRA CANXL_1__RAMECC.DRRERRA.R  /* Data RAM Error Report Address */
#define CANXL_1__RAMECC_DRRERRD CANXL_1__RAMECC.DRRERRD.R  /* Data RAM Error Report Data */
#define CANXL_1__RAMECC_DRRERRSYN CANXL_1__RAMECC.DRRERRSYN.R /* Data RAM Error Report Syndrome */
#define CANXL_1__RAMECC_DRERRS CANXL_1__RAMECC.DRERRS.R    /* Data RAM Error Status */

/* CANXL_RXFIFO */
#define CANXL_0__RXFIFO_RXFFCTR CANXL_0__RXFIFO.RXFFCTR.R  /* Receive FIFO Message Buffer Control */
#define CANXL_0__RXFIFO_RXFMBP0 CANXL_0__RXFIFO.RXFMBP[0].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP1 CANXL_0__RXFIFO.RXFMBP[1].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP2 CANXL_0__RXFIFO.RXFMBP[2].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP3 CANXL_0__RXFIFO.RXFMBP[3].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP4 CANXL_0__RXFIFO.RXFMBP[4].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP5 CANXL_0__RXFIFO.RXFMBP[5].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP6 CANXL_0__RXFIFO.RXFMBP[6].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP7 CANXL_0__RXFIFO.RXFMBP[7].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP8 CANXL_0__RXFIFO.RXFMBP[8].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP9 CANXL_0__RXFIFO.RXFMBP[9].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP10 CANXL_0__RXFIFO.RXFMBP[10].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP11 CANXL_0__RXFIFO.RXFMBP[11].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP12 CANXL_0__RXFIFO.RXFMBP[12].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP13 CANXL_0__RXFIFO.RXFMBP[13].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP14 CANXL_0__RXFIFO.RXFMBP[14].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP15 CANXL_0__RXFIFO.RXFMBP[15].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP16 CANXL_0__RXFIFO.RXFMBP[16].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP17 CANXL_0__RXFIFO.RXFMBP[17].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP18 CANXL_0__RXFIFO.RXFMBP[18].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP19 CANXL_0__RXFIFO.RXFMBP[19].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP20 CANXL_0__RXFIFO.RXFMBP[20].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP21 CANXL_0__RXFIFO.RXFMBP[21].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP22 CANXL_0__RXFIFO.RXFMBP[22].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP23 CANXL_0__RXFIFO.RXFMBP[23].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP24 CANXL_0__RXFIFO.RXFMBP[24].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP25 CANXL_0__RXFIFO.RXFMBP[25].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP26 CANXL_0__RXFIFO.RXFMBP[26].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP27 CANXL_0__RXFIFO.RXFMBP[27].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP28 CANXL_0__RXFIFO.RXFMBP[28].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP29 CANXL_0__RXFIFO.RXFMBP[29].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP30 CANXL_0__RXFIFO.RXFMBP[30].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_RXFMBP31 CANXL_0__RXFIFO.RXFMBP[31].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_0__RXFIFO_SECAM CANXL_0__RXFIFO.SECAM.R      /* SEC Acceptance Mask */
#define CANXL_0__RXFIFO_SECAV CANXL_0__RXFIFO.SECAV.R      /* SEC Acceptance Value */
#define CANXL_0__RXFIFO_AFCFG CANXL_0__RXFIFO.AFCFG.R      /* Acceptance Filter Configuration */
#define CANXL_0__RXFIFO_VAMRCFG CANXL_0__RXFIFO.VAMRCFG.R  /* VCAN Acceptance Mask or Range Configuration */
#define CANXL_0__RXFIFO_SAMRCFG CANXL_0__RXFIFO.SAMRCFG.R  /* SDU Acceptance Mask or Range Configuration */
#define CANXL_0__RXFIFO_AAMRCFG CANXL_0__RXFIFO.AAMRCFG.R  /* ADDR Acceptance Mask or Range Configuration */
#define CANXL_0__RXFIFO_ACPTIDMR CANXL_0__RXFIFO.ACPTIDMR.R /* ID Acceptance Mask or Range Configuration */
#define CANXL_0__RXFIFO_VAFLT0 CANXL_0__RXFIFO.VAFLT[0].R  /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT2 CANXL_0__RXFIFO.VAFLT[1].R  /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT4 CANXL_0__RXFIFO.VAFLT[2].R  /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT6 CANXL_0__RXFIFO.VAFLT[3].R  /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT8 CANXL_0__RXFIFO.VAFLT[4].R  /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT10 CANXL_0__RXFIFO.VAFLT[5].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT12 CANXL_0__RXFIFO.VAFLT[6].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT14 CANXL_0__RXFIFO.VAFLT[7].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT16 CANXL_0__RXFIFO.VAFLT[8].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT18 CANXL_0__RXFIFO.VAFLT[9].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT20 CANXL_0__RXFIFO.VAFLT[10].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT22 CANXL_0__RXFIFO.VAFLT[11].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT24 CANXL_0__RXFIFO.VAFLT[12].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT26 CANXL_0__RXFIFO.VAFLT[13].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT28 CANXL_0__RXFIFO.VAFLT[14].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_VAFLT30 CANXL_0__RXFIFO.VAFLT[15].R /* VCAN Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT0 CANXL_0__RXFIFO.SAFLT[0].R  /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT2 CANXL_0__RXFIFO.SAFLT[1].R  /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT4 CANXL_0__RXFIFO.SAFLT[2].R  /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT6 CANXL_0__RXFIFO.SAFLT[3].R  /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT8 CANXL_0__RXFIFO.SAFLT[4].R  /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT10 CANXL_0__RXFIFO.SAFLT[5].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT12 CANXL_0__RXFIFO.SAFLT[6].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT14 CANXL_0__RXFIFO.SAFLT[7].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT16 CANXL_0__RXFIFO.SAFLT[8].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT18 CANXL_0__RXFIFO.SAFLT[9].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT20 CANXL_0__RXFIFO.SAFLT[10].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT22 CANXL_0__RXFIFO.SAFLT[11].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT24 CANXL_0__RXFIFO.SAFLT[12].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT26 CANXL_0__RXFIFO.SAFLT[13].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT28 CANXL_0__RXFIFO.SAFLT[14].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_SAFLT30 CANXL_0__RXFIFO.SAFLT[15].R /* SDU Acceptance Filter */
#define CANXL_0__RXFIFO_AAFLT0L CANXL_0__RXFIFO.ADDRACPTFLTAR[0].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT0H CANXL_0__RXFIFO.ADDRACPTFLTAR[0].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT1L CANXL_0__RXFIFO.ADDRACPTFLTAR[1].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT1H CANXL_0__RXFIFO.ADDRACPTFLTAR[1].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT2L CANXL_0__RXFIFO.ADDRACPTFLTAR[2].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT2H CANXL_0__RXFIFO.ADDRACPTFLTAR[2].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT3L CANXL_0__RXFIFO.ADDRACPTFLTAR[3].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT3H CANXL_0__RXFIFO.ADDRACPTFLTAR[3].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT4L CANXL_0__RXFIFO.ADDRACPTFLTAR[4].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT4H CANXL_0__RXFIFO.ADDRACPTFLTAR[4].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT5L CANXL_0__RXFIFO.ADDRACPTFLTAR[5].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT5H CANXL_0__RXFIFO.ADDRACPTFLTAR[5].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT6L CANXL_0__RXFIFO.ADDRACPTFLTAR[6].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT6H CANXL_0__RXFIFO.ADDRACPTFLTAR[6].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT7L CANXL_0__RXFIFO.ADDRACPTFLTAR[7].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT7H CANXL_0__RXFIFO.ADDRACPTFLTAR[7].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT8L CANXL_0__RXFIFO.ADDRACPTFLTAR[8].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT8H CANXL_0__RXFIFO.ADDRACPTFLTAR[8].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT9L CANXL_0__RXFIFO.ADDRACPTFLTAR[9].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT9H CANXL_0__RXFIFO.ADDRACPTFLTAR[9].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT10L CANXL_0__RXFIFO.ADDRACPTFLTAR[10].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT10H CANXL_0__RXFIFO.ADDRACPTFLTAR[10].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT11L CANXL_0__RXFIFO.ADDRACPTFLTAR[11].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT11H CANXL_0__RXFIFO.ADDRACPTFLTAR[11].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT12L CANXL_0__RXFIFO.ADDRACPTFLTAR[12].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT12H CANXL_0__RXFIFO.ADDRACPTFLTAR[12].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT13L CANXL_0__RXFIFO.ADDRACPTFLTAR[13].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT13H CANXL_0__RXFIFO.ADDRACPTFLTAR[13].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT14L CANXL_0__RXFIFO.ADDRACPTFLTAR[14].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT14H CANXL_0__RXFIFO.ADDRACPTFLTAR[14].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT15L CANXL_0__RXFIFO.ADDRACPTFLTAR[15].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT15H CANXL_0__RXFIFO.ADDRACPTFLTAR[15].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT16L CANXL_0__RXFIFO.ADDRACPTFLTAR[16].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT16H CANXL_0__RXFIFO.ADDRACPTFLTAR[16].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT17L CANXL_0__RXFIFO.ADDRACPTFLTAR[17].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT17H CANXL_0__RXFIFO.ADDRACPTFLTAR[17].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT18L CANXL_0__RXFIFO.ADDRACPTFLTAR[18].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT18H CANXL_0__RXFIFO.ADDRACPTFLTAR[18].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT19L CANXL_0__RXFIFO.ADDRACPTFLTAR[19].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT19H CANXL_0__RXFIFO.ADDRACPTFLTAR[19].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT20L CANXL_0__RXFIFO.ADDRACPTFLTAR[20].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT20H CANXL_0__RXFIFO.ADDRACPTFLTAR[20].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT21L CANXL_0__RXFIFO.ADDRACPTFLTAR[21].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT21H CANXL_0__RXFIFO.ADDRACPTFLTAR[21].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT22L CANXL_0__RXFIFO.ADDRACPTFLTAR[22].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT22H CANXL_0__RXFIFO.ADDRACPTFLTAR[22].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT23L CANXL_0__RXFIFO.ADDRACPTFLTAR[23].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT23H CANXL_0__RXFIFO.ADDRACPTFLTAR[23].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT24L CANXL_0__RXFIFO.ADDRACPTFLTAR[24].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT24H CANXL_0__RXFIFO.ADDRACPTFLTAR[24].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT25L CANXL_0__RXFIFO.ADDRACPTFLTAR[25].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT25H CANXL_0__RXFIFO.ADDRACPTFLTAR[25].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT26L CANXL_0__RXFIFO.ADDRACPTFLTAR[26].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT26H CANXL_0__RXFIFO.ADDRACPTFLTAR[26].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT27L CANXL_0__RXFIFO.ADDRACPTFLTAR[27].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT27H CANXL_0__RXFIFO.ADDRACPTFLTAR[27].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT28L CANXL_0__RXFIFO.ADDRACPTFLTAR[28].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT28H CANXL_0__RXFIFO.ADDRACPTFLTAR[28].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT29L CANXL_0__RXFIFO.ADDRACPTFLTAR[29].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT29H CANXL_0__RXFIFO.ADDRACPTFLTAR[29].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT30L CANXL_0__RXFIFO.ADDRACPTFLTAR[30].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT30H CANXL_0__RXFIFO.ADDRACPTFLTAR[30].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_AAFLT31L CANXL_0__RXFIFO.ADDRACPTFLTAR[31].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_AAFLT31H CANXL_0__RXFIFO.ADDRACPTFLTAR[31].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT0L CANXL_0__RXFIFO.IDACPTFLTAR[0].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT0H CANXL_0__RXFIFO.IDACPTFLTAR[0].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT1L CANXL_0__RXFIFO.IDACPTFLTAR[1].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT1H CANXL_0__RXFIFO.IDACPTFLTAR[1].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT2L CANXL_0__RXFIFO.IDACPTFLTAR[2].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT2H CANXL_0__RXFIFO.IDACPTFLTAR[2].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT3L CANXL_0__RXFIFO.IDACPTFLTAR[3].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT3H CANXL_0__RXFIFO.IDACPTFLTAR[3].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT4L CANXL_0__RXFIFO.IDACPTFLTAR[4].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT4H CANXL_0__RXFIFO.IDACPTFLTAR[4].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT5L CANXL_0__RXFIFO.IDACPTFLTAR[5].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT5H CANXL_0__RXFIFO.IDACPTFLTAR[5].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT6L CANXL_0__RXFIFO.IDACPTFLTAR[6].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT6H CANXL_0__RXFIFO.IDACPTFLTAR[6].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT7L CANXL_0__RXFIFO.IDACPTFLTAR[7].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT7H CANXL_0__RXFIFO.IDACPTFLTAR[7].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT8L CANXL_0__RXFIFO.IDACPTFLTAR[8].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT8H CANXL_0__RXFIFO.IDACPTFLTAR[8].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT9L CANXL_0__RXFIFO.IDACPTFLTAR[9].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT9H CANXL_0__RXFIFO.IDACPTFLTAR[9].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT10L CANXL_0__RXFIFO.IDACPTFLTAR[10].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT10H CANXL_0__RXFIFO.IDACPTFLTAR[10].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT11L CANXL_0__RXFIFO.IDACPTFLTAR[11].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT11H CANXL_0__RXFIFO.IDACPTFLTAR[11].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT12L CANXL_0__RXFIFO.IDACPTFLTAR[12].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT12H CANXL_0__RXFIFO.IDACPTFLTAR[12].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT13L CANXL_0__RXFIFO.IDACPTFLTAR[13].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT13H CANXL_0__RXFIFO.IDACPTFLTAR[13].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT14L CANXL_0__RXFIFO.IDACPTFLTAR[14].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT14H CANXL_0__RXFIFO.IDACPTFLTAR[14].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT15L CANXL_0__RXFIFO.IDACPTFLTAR[15].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT15H CANXL_0__RXFIFO.IDACPTFLTAR[15].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT16L CANXL_0__RXFIFO.IDACPTFLTAR[16].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT16H CANXL_0__RXFIFO.IDACPTFLTAR[16].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT17L CANXL_0__RXFIFO.IDACPTFLTAR[17].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT17H CANXL_0__RXFIFO.IDACPTFLTAR[17].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT18L CANXL_0__RXFIFO.IDACPTFLTAR[18].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT18H CANXL_0__RXFIFO.IDACPTFLTAR[18].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT19L CANXL_0__RXFIFO.IDACPTFLTAR[19].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT19H CANXL_0__RXFIFO.IDACPTFLTAR[19].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT20L CANXL_0__RXFIFO.IDACPTFLTAR[20].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT20H CANXL_0__RXFIFO.IDACPTFLTAR[20].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT21L CANXL_0__RXFIFO.IDACPTFLTAR[21].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT21H CANXL_0__RXFIFO.IDACPTFLTAR[21].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT22L CANXL_0__RXFIFO.IDACPTFLTAR[22].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT22H CANXL_0__RXFIFO.IDACPTFLTAR[22].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT23L CANXL_0__RXFIFO.IDACPTFLTAR[23].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT23H CANXL_0__RXFIFO.IDACPTFLTAR[23].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT24L CANXL_0__RXFIFO.IDACPTFLTAR[24].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT24H CANXL_0__RXFIFO.IDACPTFLTAR[24].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT25L CANXL_0__RXFIFO.IDACPTFLTAR[25].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT25H CANXL_0__RXFIFO.IDACPTFLTAR[25].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT26L CANXL_0__RXFIFO.IDACPTFLTAR[26].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT26H CANXL_0__RXFIFO.IDACPTFLTAR[26].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT27L CANXL_0__RXFIFO.IDACPTFLTAR[27].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT27H CANXL_0__RXFIFO.IDACPTFLTAR[27].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT28L CANXL_0__RXFIFO.IDACPTFLTAR[28].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT28H CANXL_0__RXFIFO.IDACPTFLTAR[28].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT29L CANXL_0__RXFIFO.IDACPTFLTAR[29].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT29H CANXL_0__RXFIFO.IDACPTFLTAR[29].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT30L CANXL_0__RXFIFO.IDACPTFLTAR[30].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT30H CANXL_0__RXFIFO.IDACPTFLTAR[30].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_0__RXFIFO_IDAFLT31L CANXL_0__RXFIFO.IDACPTFLTAR[31].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_0__RXFIFO_IDAFLT31H CANXL_0__RXFIFO.IDACPTFLTAR[31].IDAFLTH.R /* ID Acceptance Filter Element High */

#define CANXL_1__RXFIFO_RXFFCTR CANXL_1__RXFIFO.RXFFCTR.R  /* Receive FIFO Message Buffer Control */
#define CANXL_1__RXFIFO_RXFMBP0 CANXL_1__RXFIFO.RXFMBP[0].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP1 CANXL_1__RXFIFO.RXFMBP[1].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP2 CANXL_1__RXFIFO.RXFMBP[2].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP3 CANXL_1__RXFIFO.RXFMBP[3].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP4 CANXL_1__RXFIFO.RXFMBP[4].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP5 CANXL_1__RXFIFO.RXFMBP[5].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP6 CANXL_1__RXFIFO.RXFMBP[6].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP7 CANXL_1__RXFIFO.RXFMBP[7].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP8 CANXL_1__RXFIFO.RXFMBP[8].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP9 CANXL_1__RXFIFO.RXFMBP[9].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP10 CANXL_1__RXFIFO.RXFMBP[10].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP11 CANXL_1__RXFIFO.RXFMBP[11].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP12 CANXL_1__RXFIFO.RXFMBP[12].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP13 CANXL_1__RXFIFO.RXFMBP[13].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP14 CANXL_1__RXFIFO.RXFMBP[14].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP15 CANXL_1__RXFIFO.RXFMBP[15].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP16 CANXL_1__RXFIFO.RXFMBP[16].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP17 CANXL_1__RXFIFO.RXFMBP[17].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP18 CANXL_1__RXFIFO.RXFMBP[18].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP19 CANXL_1__RXFIFO.RXFMBP[19].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP20 CANXL_1__RXFIFO.RXFMBP[20].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP21 CANXL_1__RXFIFO.RXFMBP[21].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP22 CANXL_1__RXFIFO.RXFMBP[22].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP23 CANXL_1__RXFIFO.RXFMBP[23].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP24 CANXL_1__RXFIFO.RXFMBP[24].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP25 CANXL_1__RXFIFO.RXFMBP[25].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP26 CANXL_1__RXFIFO.RXFMBP[26].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP27 CANXL_1__RXFIFO.RXFMBP[27].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP28 CANXL_1__RXFIFO.RXFMBP[28].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP29 CANXL_1__RXFIFO.RXFMBP[29].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP30 CANXL_1__RXFIFO.RXFMBP[30].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_RXFMBP31 CANXL_1__RXFIFO.RXFMBP[31].R /* Receive FIFO Message Buffer Pointer */
#define CANXL_1__RXFIFO_SECAM CANXL_1__RXFIFO.SECAM.R      /* SEC Acceptance Mask */
#define CANXL_1__RXFIFO_SECAV CANXL_1__RXFIFO.SECAV.R      /* SEC Acceptance Value */
#define CANXL_1__RXFIFO_AFCFG CANXL_1__RXFIFO.AFCFG.R      /* Acceptance Filter Configuration */
#define CANXL_1__RXFIFO_VAMRCFG CANXL_1__RXFIFO.VAMRCFG.R  /* VCAN Acceptance Mask or Range Configuration */
#define CANXL_1__RXFIFO_SAMRCFG CANXL_1__RXFIFO.SAMRCFG.R  /* SDU Acceptance Mask or Range Configuration */
#define CANXL_1__RXFIFO_AAMRCFG CANXL_1__RXFIFO.AAMRCFG.R  /* ADDR Acceptance Mask or Range Configuration */
#define CANXL_1__RXFIFO_ACPTIDMR CANXL_1__RXFIFO.ACPTIDMR.R /* ID Acceptance Mask or Range Configuration */
#define CANXL_1__RXFIFO_VAFLT0 CANXL_1__RXFIFO.VAFLT[0].R  /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT2 CANXL_1__RXFIFO.VAFLT[1].R  /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT4 CANXL_1__RXFIFO.VAFLT[2].R  /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT6 CANXL_1__RXFIFO.VAFLT[3].R  /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT8 CANXL_1__RXFIFO.VAFLT[4].R  /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT10 CANXL_1__RXFIFO.VAFLT[5].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT12 CANXL_1__RXFIFO.VAFLT[6].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT14 CANXL_1__RXFIFO.VAFLT[7].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT16 CANXL_1__RXFIFO.VAFLT[8].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT18 CANXL_1__RXFIFO.VAFLT[9].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT20 CANXL_1__RXFIFO.VAFLT[10].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT22 CANXL_1__RXFIFO.VAFLT[11].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT24 CANXL_1__RXFIFO.VAFLT[12].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT26 CANXL_1__RXFIFO.VAFLT[13].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT28 CANXL_1__RXFIFO.VAFLT[14].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_VAFLT30 CANXL_1__RXFIFO.VAFLT[15].R /* VCAN Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT0 CANXL_1__RXFIFO.SAFLT[0].R  /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT2 CANXL_1__RXFIFO.SAFLT[1].R  /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT4 CANXL_1__RXFIFO.SAFLT[2].R  /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT6 CANXL_1__RXFIFO.SAFLT[3].R  /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT8 CANXL_1__RXFIFO.SAFLT[4].R  /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT10 CANXL_1__RXFIFO.SAFLT[5].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT12 CANXL_1__RXFIFO.SAFLT[6].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT14 CANXL_1__RXFIFO.SAFLT[7].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT16 CANXL_1__RXFIFO.SAFLT[8].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT18 CANXL_1__RXFIFO.SAFLT[9].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT20 CANXL_1__RXFIFO.SAFLT[10].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT22 CANXL_1__RXFIFO.SAFLT[11].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT24 CANXL_1__RXFIFO.SAFLT[12].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT26 CANXL_1__RXFIFO.SAFLT[13].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT28 CANXL_1__RXFIFO.SAFLT[14].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_SAFLT30 CANXL_1__RXFIFO.SAFLT[15].R /* SDU Acceptance Filter */
#define CANXL_1__RXFIFO_AAFLT0L CANXL_1__RXFIFO.ADDRACPTFLTAR[0].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT0H CANXL_1__RXFIFO.ADDRACPTFLTAR[0].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT1L CANXL_1__RXFIFO.ADDRACPTFLTAR[1].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT1H CANXL_1__RXFIFO.ADDRACPTFLTAR[1].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT2L CANXL_1__RXFIFO.ADDRACPTFLTAR[2].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT2H CANXL_1__RXFIFO.ADDRACPTFLTAR[2].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT3L CANXL_1__RXFIFO.ADDRACPTFLTAR[3].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT3H CANXL_1__RXFIFO.ADDRACPTFLTAR[3].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT4L CANXL_1__RXFIFO.ADDRACPTFLTAR[4].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT4H CANXL_1__RXFIFO.ADDRACPTFLTAR[4].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT5L CANXL_1__RXFIFO.ADDRACPTFLTAR[5].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT5H CANXL_1__RXFIFO.ADDRACPTFLTAR[5].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT6L CANXL_1__RXFIFO.ADDRACPTFLTAR[6].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT6H CANXL_1__RXFIFO.ADDRACPTFLTAR[6].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT7L CANXL_1__RXFIFO.ADDRACPTFLTAR[7].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT7H CANXL_1__RXFIFO.ADDRACPTFLTAR[7].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT8L CANXL_1__RXFIFO.ADDRACPTFLTAR[8].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT8H CANXL_1__RXFIFO.ADDRACPTFLTAR[8].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT9L CANXL_1__RXFIFO.ADDRACPTFLTAR[9].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT9H CANXL_1__RXFIFO.ADDRACPTFLTAR[9].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT10L CANXL_1__RXFIFO.ADDRACPTFLTAR[10].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT10H CANXL_1__RXFIFO.ADDRACPTFLTAR[10].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT11L CANXL_1__RXFIFO.ADDRACPTFLTAR[11].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT11H CANXL_1__RXFIFO.ADDRACPTFLTAR[11].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT12L CANXL_1__RXFIFO.ADDRACPTFLTAR[12].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT12H CANXL_1__RXFIFO.ADDRACPTFLTAR[12].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT13L CANXL_1__RXFIFO.ADDRACPTFLTAR[13].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT13H CANXL_1__RXFIFO.ADDRACPTFLTAR[13].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT14L CANXL_1__RXFIFO.ADDRACPTFLTAR[14].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT14H CANXL_1__RXFIFO.ADDRACPTFLTAR[14].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT15L CANXL_1__RXFIFO.ADDRACPTFLTAR[15].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT15H CANXL_1__RXFIFO.ADDRACPTFLTAR[15].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT16L CANXL_1__RXFIFO.ADDRACPTFLTAR[16].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT16H CANXL_1__RXFIFO.ADDRACPTFLTAR[16].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT17L CANXL_1__RXFIFO.ADDRACPTFLTAR[17].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT17H CANXL_1__RXFIFO.ADDRACPTFLTAR[17].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT18L CANXL_1__RXFIFO.ADDRACPTFLTAR[18].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT18H CANXL_1__RXFIFO.ADDRACPTFLTAR[18].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT19L CANXL_1__RXFIFO.ADDRACPTFLTAR[19].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT19H CANXL_1__RXFIFO.ADDRACPTFLTAR[19].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT20L CANXL_1__RXFIFO.ADDRACPTFLTAR[20].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT20H CANXL_1__RXFIFO.ADDRACPTFLTAR[20].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT21L CANXL_1__RXFIFO.ADDRACPTFLTAR[21].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT21H CANXL_1__RXFIFO.ADDRACPTFLTAR[21].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT22L CANXL_1__RXFIFO.ADDRACPTFLTAR[22].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT22H CANXL_1__RXFIFO.ADDRACPTFLTAR[22].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT23L CANXL_1__RXFIFO.ADDRACPTFLTAR[23].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT23H CANXL_1__RXFIFO.ADDRACPTFLTAR[23].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT24L CANXL_1__RXFIFO.ADDRACPTFLTAR[24].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT24H CANXL_1__RXFIFO.ADDRACPTFLTAR[24].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT25L CANXL_1__RXFIFO.ADDRACPTFLTAR[25].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT25H CANXL_1__RXFIFO.ADDRACPTFLTAR[25].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT26L CANXL_1__RXFIFO.ADDRACPTFLTAR[26].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT26H CANXL_1__RXFIFO.ADDRACPTFLTAR[26].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT27L CANXL_1__RXFIFO.ADDRACPTFLTAR[27].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT27H CANXL_1__RXFIFO.ADDRACPTFLTAR[27].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT28L CANXL_1__RXFIFO.ADDRACPTFLTAR[28].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT28H CANXL_1__RXFIFO.ADDRACPTFLTAR[28].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT29L CANXL_1__RXFIFO.ADDRACPTFLTAR[29].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT29H CANXL_1__RXFIFO.ADDRACPTFLTAR[29].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT30L CANXL_1__RXFIFO.ADDRACPTFLTAR[30].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT30H CANXL_1__RXFIFO.ADDRACPTFLTAR[30].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_AAFLT31L CANXL_1__RXFIFO.ADDRACPTFLTAR[31].AAFLTL.R /* ADDR Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_AAFLT31H CANXL_1__RXFIFO.ADDRACPTFLTAR[31].AAFLTH.R /* ADDR Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT0L CANXL_1__RXFIFO.IDACPTFLTAR[0].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT0H CANXL_1__RXFIFO.IDACPTFLTAR[0].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT1L CANXL_1__RXFIFO.IDACPTFLTAR[1].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT1H CANXL_1__RXFIFO.IDACPTFLTAR[1].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT2L CANXL_1__RXFIFO.IDACPTFLTAR[2].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT2H CANXL_1__RXFIFO.IDACPTFLTAR[2].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT3L CANXL_1__RXFIFO.IDACPTFLTAR[3].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT3H CANXL_1__RXFIFO.IDACPTFLTAR[3].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT4L CANXL_1__RXFIFO.IDACPTFLTAR[4].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT4H CANXL_1__RXFIFO.IDACPTFLTAR[4].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT5L CANXL_1__RXFIFO.IDACPTFLTAR[5].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT5H CANXL_1__RXFIFO.IDACPTFLTAR[5].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT6L CANXL_1__RXFIFO.IDACPTFLTAR[6].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT6H CANXL_1__RXFIFO.IDACPTFLTAR[6].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT7L CANXL_1__RXFIFO.IDACPTFLTAR[7].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT7H CANXL_1__RXFIFO.IDACPTFLTAR[7].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT8L CANXL_1__RXFIFO.IDACPTFLTAR[8].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT8H CANXL_1__RXFIFO.IDACPTFLTAR[8].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT9L CANXL_1__RXFIFO.IDACPTFLTAR[9].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT9H CANXL_1__RXFIFO.IDACPTFLTAR[9].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT10L CANXL_1__RXFIFO.IDACPTFLTAR[10].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT10H CANXL_1__RXFIFO.IDACPTFLTAR[10].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT11L CANXL_1__RXFIFO.IDACPTFLTAR[11].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT11H CANXL_1__RXFIFO.IDACPTFLTAR[11].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT12L CANXL_1__RXFIFO.IDACPTFLTAR[12].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT12H CANXL_1__RXFIFO.IDACPTFLTAR[12].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT13L CANXL_1__RXFIFO.IDACPTFLTAR[13].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT13H CANXL_1__RXFIFO.IDACPTFLTAR[13].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT14L CANXL_1__RXFIFO.IDACPTFLTAR[14].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT14H CANXL_1__RXFIFO.IDACPTFLTAR[14].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT15L CANXL_1__RXFIFO.IDACPTFLTAR[15].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT15H CANXL_1__RXFIFO.IDACPTFLTAR[15].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT16L CANXL_1__RXFIFO.IDACPTFLTAR[16].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT16H CANXL_1__RXFIFO.IDACPTFLTAR[16].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT17L CANXL_1__RXFIFO.IDACPTFLTAR[17].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT17H CANXL_1__RXFIFO.IDACPTFLTAR[17].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT18L CANXL_1__RXFIFO.IDACPTFLTAR[18].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT18H CANXL_1__RXFIFO.IDACPTFLTAR[18].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT19L CANXL_1__RXFIFO.IDACPTFLTAR[19].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT19H CANXL_1__RXFIFO.IDACPTFLTAR[19].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT20L CANXL_1__RXFIFO.IDACPTFLTAR[20].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT20H CANXL_1__RXFIFO.IDACPTFLTAR[20].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT21L CANXL_1__RXFIFO.IDACPTFLTAR[21].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT21H CANXL_1__RXFIFO.IDACPTFLTAR[21].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT22L CANXL_1__RXFIFO.IDACPTFLTAR[22].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT22H CANXL_1__RXFIFO.IDACPTFLTAR[22].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT23L CANXL_1__RXFIFO.IDACPTFLTAR[23].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT23H CANXL_1__RXFIFO.IDACPTFLTAR[23].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT24L CANXL_1__RXFIFO.IDACPTFLTAR[24].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT24H CANXL_1__RXFIFO.IDACPTFLTAR[24].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT25L CANXL_1__RXFIFO.IDACPTFLTAR[25].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT25H CANXL_1__RXFIFO.IDACPTFLTAR[25].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT26L CANXL_1__RXFIFO.IDACPTFLTAR[26].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT26H CANXL_1__RXFIFO.IDACPTFLTAR[26].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT27L CANXL_1__RXFIFO.IDACPTFLTAR[27].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT27H CANXL_1__RXFIFO.IDACPTFLTAR[27].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT28L CANXL_1__RXFIFO.IDACPTFLTAR[28].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT28H CANXL_1__RXFIFO.IDACPTFLTAR[28].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT29L CANXL_1__RXFIFO.IDACPTFLTAR[29].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT29H CANXL_1__RXFIFO.IDACPTFLTAR[29].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT30L CANXL_1__RXFIFO.IDACPTFLTAR[30].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT30H CANXL_1__RXFIFO.IDACPTFLTAR[30].IDAFLTH.R /* ID Acceptance Filter Element High */
#define CANXL_1__RXFIFO_IDAFLT31L CANXL_1__RXFIFO.IDACPTFLTAR[31].IDAFLTL.R /* ID Acceptance Filter Element Low */
#define CANXL_1__RXFIFO_IDAFLT31H CANXL_1__RXFIFO.IDACPTFLTAR[31].IDAFLTH.R /* ID Acceptance Filter Element High */

/* CANXL_RXFIFO_CONTROL */
#define CANXL_0__RXFIFO_CONTROL_RXFCSTA CANXL_0__RXFIFO_CONTROL.RXFCSTA.R /* RxFIFO Control Status */
#define CANXL_0__RXFIFO_CONTROL_RXFSYSPOP CANXL_0__RXFIFO_CONTROL.RXFSYSPOP.R /* RxFIFO POP Control */
#define CANXL_0__RXFIFO_CONTROL_RXFSYSLOCK CANXL_0__RXFIFO_CONTROL.RXFSYSLOCK.R /* RxFIFO System Lock Control */
#define CANXL_0__RXFIFO_CONTROL_RXFSYSACT CANXL_0__RXFIFO_CONTROL.RXFSYSACT.R /* RxFIFO Activation */
#define CANXL_0__RXFIFO_CONTROL_RXFIEN CANXL_0__RXFIFO_CONTROL.RXFIEN.R /* RxFIFO Interrupt Enable */
#define CANXL_0__RXFIFO_CONTROL_RXFS CANXL_0__RXFIFO_CONTROL.RXFS.R /* RxFIFO Status */
#define CANXL_0__RXFIFO_CONTROL_RXFC CANXL_0__RXFIFO_CONTROL.RXFC.R /* RxFIFO Configuration */

#define CANXL_1__RXFIFO_CONTROL_RXFCSTA CANXL_1__RXFIFO_CONTROL.RXFCSTA.R /* RxFIFO Control Status */
#define CANXL_1__RXFIFO_CONTROL_RXFSYSPOP CANXL_1__RXFIFO_CONTROL.RXFSYSPOP.R /* RxFIFO POP Control */
#define CANXL_1__RXFIFO_CONTROL_RXFSYSLOCK CANXL_1__RXFIFO_CONTROL.RXFSYSLOCK.R /* RxFIFO System Lock Control */
#define CANXL_1__RXFIFO_CONTROL_RXFSYSACT CANXL_1__RXFIFO_CONTROL.RXFSYSACT.R /* RxFIFO Activation */
#define CANXL_1__RXFIFO_CONTROL_RXFIEN CANXL_1__RXFIFO_CONTROL.RXFIEN.R /* RxFIFO Interrupt Enable */
#define CANXL_1__RXFIFO_CONTROL_RXFS CANXL_1__RXFIFO_CONTROL.RXFS.R /* RxFIFO Status */
#define CANXL_1__RXFIFO_CONTROL_RXFC CANXL_1__RXFIFO_CONTROL.RXFC.R /* RxFIFO Configuration */

/* CANXL_SIC */
#define CANXL_0__SIC_SYSMC   CANXL_0__SIC.SYSMC.R          /* System Module Control */
#define CANXL_0__SIC_SYSMCFG CANXL_0__SIC.SYSMCFG.R        /* System Module Configuration */
#define CANXL_0__SIC_SYSIE   CANXL_0__SIC.SYSIE.R          /* System Interrupt Enable */
#define CANXL_0__SIC_SYSS    CANXL_0__SIC.SYSS.R           /* System Status */
#define CANXL_0__SIC_SYSFRTL CANXL_0__SIC.SYSFRTL.R        /* System Free Running Timer Low */
#define CANXL_0__SIC_SYSFRTH CANXL_0__SIC.SYSFRTH.R        /* System Free Running Timer High */
#define CANXL_0__SIC_SYSMDHICFG CANXL_0__SIC.SYSMDHICFG.R  /* System Message Descriptor Host Interrupt Configuration */
#define CANXL_0__SIC_BCFG1   CANXL_0__SIC.BCFG1.R          /* BCANXL Configuration 1 */
#define CANXL_0__SIC_BCFG2   CANXL_0__SIC.BCFG2.R          /* BCANXL Configuration 2 */
#define CANXL_0__SIC_BBPRS   CANXL_0__SIC.BBPRS.R          /* BCANXL Bit Time Quanta Prescaler */
#define CANXL_0__SIC_BNCBT   CANXL_0__SIC.BNCBT.R          /* BCANXL Nominal Bit Timing */
#define CANXL_0__SIC_BFDCBT  CANXL_0__SIC.BFDCBT.R         /* BCANXL FD Data Phase Bit Timing */
#define CANXL_0__SIC_BXDCBT  CANXL_0__SIC.BXDCBT.R         /* BCANXL XL Data Phase Bit Timing */
#define CANXL_0__SIC_BTDCC   CANXL_0__SIC.BTDCC.R          /* BCANXL Transceiver Delay Compensation Control */
#define CANXL_0__SIC_BMICI   CANXL_0__SIC.BMICI.R          /* BCANXL PWM Configuration */

#define CANXL_1__SIC_SYSMC   CANXL_1__SIC.SYSMC.R          /* System Module Control */
#define CANXL_1__SIC_SYSMCFG CANXL_1__SIC.SYSMCFG.R        /* System Module Configuration */
#define CANXL_1__SIC_SYSIE   CANXL_1__SIC.SYSIE.R          /* System Interrupt Enable */
#define CANXL_1__SIC_SYSS    CANXL_1__SIC.SYSS.R           /* System Status */
#define CANXL_1__SIC_SYSFRTL CANXL_1__SIC.SYSFRTL.R        /* System Free Running Timer Low */
#define CANXL_1__SIC_SYSFRTH CANXL_1__SIC.SYSFRTH.R        /* System Free Running Timer High */
#define CANXL_1__SIC_SYSMDHICFG CANXL_1__SIC.SYSMDHICFG.R  /* System Message Descriptor Host Interrupt Configuration */
#define CANXL_1__SIC_BCFG1   CANXL_1__SIC.BCFG1.R          /* BCANXL Configuration 1 */
#define CANXL_1__SIC_BCFG2   CANXL_1__SIC.BCFG2.R          /* BCANXL Configuration 2 */
#define CANXL_1__SIC_BBPRS   CANXL_1__SIC.BBPRS.R          /* BCANXL Bit Time Quanta Prescaler */
#define CANXL_1__SIC_BNCBT   CANXL_1__SIC.BNCBT.R          /* BCANXL Nominal Bit Timing */
#define CANXL_1__SIC_BFDCBT  CANXL_1__SIC.BFDCBT.R         /* BCANXL FD Data Phase Bit Timing */
#define CANXL_1__SIC_BXDCBT  CANXL_1__SIC.BXDCBT.R         /* BCANXL XL Data Phase Bit Timing */
#define CANXL_1__SIC_BTDCC   CANXL_1__SIC.BTDCC.R          /* BCANXL Transceiver Delay Compensation Control */
#define CANXL_1__SIC_BMICI   CANXL_1__SIC.BMICI.R          /* BCANXL PWM Configuration */

/* CAN_HUB */
#define CE_CAN_HUB_CANNETW   CE_CAN_HUB.CANNETW.R          /* CAN Module Connected to Bus Flag */
#define CE_CAN_HUB_CAN0ALWGRP CE_CAN_HUB.CAN0ALWGRP.R      /* CAN0 Allowable Groups */
#define CE_CAN_HUB_CAN1ALWGRP CE_CAN_HUB.CAN1ALWGRP.R      /* CAN1 Allowable Groups */
#define CE_CAN_HUB_CAN2ALWGRP CE_CAN_HUB.CAN2ALWGRP.R      /* CAN2 Allowable Groups */
#define CE_CAN_HUB_CAN3ALWGRP CE_CAN_HUB.CAN3ALWGRP.R      /* CAN3 Allowable Groups */
#define CE_CAN_HUB_CAN4ALWGRP CE_CAN_HUB.CAN4ALWGRP.R      /* CAN4 Allowable Groups */
#define CE_CAN_HUB_CAN5ALWGRP CE_CAN_HUB.CAN5ALWGRP.R      /* CAN5 Allowable Groups */
#define CE_CAN_HUB_CAN6ALWGRP CE_CAN_HUB.CAN6ALWGRP.R      /* CAN6 Allowable Groups */
#define CE_CAN_HUB_CAN7ALWGRP CE_CAN_HUB.CAN7ALWGRP.R      /* CAN7 Allowable Groups */
#define CE_CAN_HUB_CAN8ALWGRP CE_CAN_HUB.CAN8ALWGRP.R      /* CAN8 Allowable Groups */
#define CE_CAN_HUB_CAN9ALWGRP CE_CAN_HUB.CAN9ALWGRP.R      /* CAN9 Allowable Groups */
#define CE_CAN_HUB_CAN10ALWGRP CE_CAN_HUB.CAN10ALWGRP.R    /* CAN10 Allowable Groups */
#define CE_CAN_HUB_CAN11ALWGRP CE_CAN_HUB.CAN11ALWGRP.R    /* CAN11 Allowable Groups */
#define CE_CAN_HUB_CAN12ALWGRP CE_CAN_HUB.CAN12ALWGRP.R    /* CAN12 Allowable Groups */
#define CE_CAN_HUB_CAN13ALWGRP CE_CAN_HUB.CAN13ALWGRP.R    /* CAN13 Allowable Groups */
#define CE_CAN_HUB_CAN14ALWGRP CE_CAN_HUB.CAN14ALWGRP.R    /* CAN14 Allowable Groups */
#define CE_CAN_HUB_CAN15ALWGRP CE_CAN_HUB.CAN15ALWGRP.R    /* CAN15 Allowable Groups */
#define CE_CAN_HUB_CAN16ALWGRP CE_CAN_HUB.CAN16ALWGRP.R    /* CAN16 Allowable Groups */
#define CE_CAN_HUB_CAN17ALWGRP CE_CAN_HUB.CAN17ALWGRP.R    /* CAN17 Allowable Groups */
#define CE_CAN_HUB_CAN18ALWGRP CE_CAN_HUB.CAN18ALWGRP.R    /* CAN18 Allowable Groups */
#define CE_CAN_HUB_CAN19ALWGRP CE_CAN_HUB.CAN19ALWGRP.R    /* CAN19 Allowable Groups */
#define CE_CAN_HUB_CAN20ALWGRP CE_CAN_HUB.CAN20ALWGRP.R    /* CAN20 Allowable Groups */
#define CE_CAN_HUB_CAN21ALWGRP CE_CAN_HUB.CAN21ALWGRP.R    /* CAN21 Allowable Groups */
#define CE_CAN_HUB_CAN22ALWGRP CE_CAN_HUB.CAN22ALWGRP.R    /* CAN22 Allowable Groups */
#define CE_CAN_HUB_CAN23ALWGRP CE_CAN_HUB.CAN23ALWGRP.R    /* CAN23 Allowable Groups */
#define CE_CAN_HUB_CAN0GRPCTL CE_CAN_HUB.CAN0GRPCTL.R      /* CAN0 Module Group Control Register */
#define CE_CAN_HUB_CAN1GRPCTL CE_CAN_HUB.CAN1GRPCTL.R      /* CAN1 Module Group Control Register */
#define CE_CAN_HUB_CAN2GRPCTL CE_CAN_HUB.CAN2GRPCTL.R      /* CAN2 Module Group Control Register */
#define CE_CAN_HUB_CAN3GRPCTL CE_CAN_HUB.CAN3GRPCTL.R      /* CAN3 Module Group Control Register */
#define CE_CAN_HUB_CAN4GRPCTL CE_CAN_HUB.CAN4GRPCTL.R      /* CAN4 Module Group Control Register */
#define CE_CAN_HUB_CAN5GRPCTL CE_CAN_HUB.CAN5GRPCTL.R      /* CAN5 Module Group Control Register */
#define CE_CAN_HUB_CAN6GRPCTL CE_CAN_HUB.CAN6GRPCTL.R      /* CAN6 Module Group Control Register */
#define CE_CAN_HUB_CAN7GRPCTL CE_CAN_HUB.CAN7GRPCTL.R      /* CAN7 Module Group Control Register */
#define CE_CAN_HUB_CAN8GRPCTL CE_CAN_HUB.CAN8GRPCTL.R      /* CAN8 Module Group Control Register */
#define CE_CAN_HUB_CAN9GRPCTL CE_CAN_HUB.CAN9GRPCTL.R      /* CAN9 Module Group Control Register */
#define CE_CAN_HUB_CAN10GRPCTL CE_CAN_HUB.CAN10GRPCTL.R    /* CAN10 Module Group Control Register */
#define CE_CAN_HUB_CAN11GRPCTL CE_CAN_HUB.CAN11GRPCTL.R    /* CAN11 Module Group Control Register */
#define CE_CAN_HUB_CAN12GRPCTL CE_CAN_HUB.CAN12GRPCTL.R    /* CAN12 Module Group Control Register */
#define CE_CAN_HUB_CAN13GRPCTL CE_CAN_HUB.CAN13GRPCTL.R    /* CAN13 Module Group Control Register */
#define CE_CAN_HUB_CAN14GRPCTL CE_CAN_HUB.CAN14GRPCTL.R    /* CAN14 Module Group Control Register */
#define CE_CAN_HUB_CAN15GRPCTL CE_CAN_HUB.CAN15GRPCTL.R    /* CAN15 Module Group Control Register */
#define CE_CAN_HUB_CAN16GRPCTL CE_CAN_HUB.CAN16GRPCTL.R    /* CAN16 Module Group Control Register */
#define CE_CAN_HUB_CAN17GRPCTL CE_CAN_HUB.CAN17GRPCTL.R    /* CAN17 Module Group Control Register */
#define CE_CAN_HUB_CAN18GRPCTL CE_CAN_HUB.CAN18GRPCTL.R    /* CAN18 Module Group Control Register */
#define CE_CAN_HUB_CAN19GRPCTL CE_CAN_HUB.CAN19GRPCTL.R    /* CAN19 Module Group Control Register */
#define CE_CAN_HUB_CAN20GRPCTL CE_CAN_HUB.CAN20GRPCTL.R    /* CAN20 Module Group Control Register */
#define CE_CAN_HUB_CAN21GRPCTL CE_CAN_HUB.CAN21GRPCTL.R    /* CAN21 Module Group Control Register */
#define CE_CAN_HUB_CAN22GRPCTL CE_CAN_HUB.CAN22GRPCTL.R    /* CAN22 Module Group Control Register */
#define CE_CAN_HUB_CAN23GRPCTL CE_CAN_HUB.CAN23GRPCTL.R    /* CAN23 Module Group Control Register */

/* CAN_TBS */
#define CE_CAN_TBS_0_CAN_TB_VALUE CE_CAN_TBS_0.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_0_CAN_TS_SEL CE_CAN_TBS_0.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_10_CAN_TB_VALUE CE_CAN_TBS_10.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_10_CAN_TS_SEL CE_CAN_TBS_10.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_11_CAN_TB_VALUE CE_CAN_TBS_11.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_11_CAN_TS_SEL CE_CAN_TBS_11.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_12_CAN_TB_VALUE CE_CAN_TBS_12.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_12_CAN_TS_SEL CE_CAN_TBS_12.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_13_CAN_TB_VALUE CE_CAN_TBS_13.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_13_CAN_TS_SEL CE_CAN_TBS_13.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_14_CAN_TB_VALUE CE_CAN_TBS_14.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_14_CAN_TS_SEL CE_CAN_TBS_14.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_15_CAN_TB_VALUE CE_CAN_TBS_15.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_15_CAN_TS_SEL CE_CAN_TBS_15.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_16_CAN_TB_VALUE CE_CAN_TBS_16.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_16_CAN_TS_SEL CE_CAN_TBS_16.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_17_CAN_TB_VALUE CE_CAN_TBS_17.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_17_CAN_TS_SEL CE_CAN_TBS_17.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_18_CAN_TB_VALUE CE_CAN_TBS_18.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_18_CAN_TS_SEL CE_CAN_TBS_18.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_19_CAN_TB_VALUE CE_CAN_TBS_19.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_19_CAN_TS_SEL CE_CAN_TBS_19.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_1_CAN_TB_VALUE CE_CAN_TBS_1.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_1_CAN_TS_SEL CE_CAN_TBS_1.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_20_CAN_TB_VALUE CE_CAN_TBS_20.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_20_CAN_TS_SEL CE_CAN_TBS_20.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_21_CAN_TB_VALUE CE_CAN_TBS_21.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_21_CAN_TS_SEL CE_CAN_TBS_21.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_22_CAN_TB_VALUE CE_CAN_TBS_22.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_22_CAN_TS_SEL CE_CAN_TBS_22.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_23_CAN_TB_VALUE CE_CAN_TBS_23.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_23_CAN_TS_SEL CE_CAN_TBS_23.CAN_TS_SEL.R /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_2_CAN_TB_VALUE CE_CAN_TBS_2.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_2_CAN_TS_SEL CE_CAN_TBS_2.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_3_CAN_TB_VALUE CE_CAN_TBS_3.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_3_CAN_TS_SEL CE_CAN_TBS_3.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_4_CAN_TB_VALUE CE_CAN_TBS_4.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_4_CAN_TS_SEL CE_CAN_TBS_4.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_5_CAN_TB_VALUE CE_CAN_TBS_5.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_5_CAN_TS_SEL CE_CAN_TBS_5.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_6_CAN_TB_VALUE CE_CAN_TBS_6.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_6_CAN_TS_SEL CE_CAN_TBS_6.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_7_CAN_TB_VALUE CE_CAN_TBS_7.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_7_CAN_TS_SEL CE_CAN_TBS_7.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_8_CAN_TB_VALUE CE_CAN_TBS_8.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_8_CAN_TS_SEL CE_CAN_TBS_8.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CE_CAN_TBS_9_CAN_TB_VALUE CE_CAN_TBS_9.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CE_CAN_TBS_9_CAN_TS_SEL CE_CAN_TBS_9.CAN_TS_SEL.R  /* CAN Timestamp Mux Selector */

#define CANXL_0_TBS_CAN_TB_VALUE CANXL_0_TBS.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CANXL_0_TBS_CAN_TS_SEL CANXL_0_TBS.CAN_TS_SEL.R    /* CAN Timestamp Mux Selector */

#define CANXL_1_TBS_CAN_TB_VALUE CANXL_1_TBS.CAN_TB_VALUE.R /* CAN Time Base Output Value */
#define CANXL_1_TBS_CAN_TS_SEL CANXL_1_TBS.CAN_TS_SEL.R    /* CAN Timestamp Mux Selector */

/* CE_DMAMUX */
#define CE_DMAMUX_CHCFG3     CE_DMAMUX.CHCONF[0].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG2     CE_DMAMUX.CHCONF[1].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG1     CE_DMAMUX.CHCONF[2].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG0     CE_DMAMUX.CHCONF[3].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG7     CE_DMAMUX.CHCONF[4].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG6     CE_DMAMUX.CHCONF[5].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG5     CE_DMAMUX.CHCONF[6].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG4     CE_DMAMUX.CHCONF[7].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG11    CE_DMAMUX.CHCONF[8].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG10    CE_DMAMUX.CHCONF[9].R         /* Channel Configuration */
#define CE_DMAMUX_CHCFG9     CE_DMAMUX.CHCONF[10].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG8     CE_DMAMUX.CHCONF[11].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG15    CE_DMAMUX.CHCONF[12].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG14    CE_DMAMUX.CHCONF[13].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG13    CE_DMAMUX.CHCONF[14].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG12    CE_DMAMUX.CHCONF[15].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG19    CE_DMAMUX.CHCONF[16].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG18    CE_DMAMUX.CHCONF[17].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG17    CE_DMAMUX.CHCONF[18].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG16    CE_DMAMUX.CHCONF[19].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG23    CE_DMAMUX.CHCONF[20].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG22    CE_DMAMUX.CHCONF[21].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG21    CE_DMAMUX.CHCONF[22].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG20    CE_DMAMUX.CHCONF[23].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG27    CE_DMAMUX.CHCONF[24].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG26    CE_DMAMUX.CHCONF[25].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG25    CE_DMAMUX.CHCONF[26].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG24    CE_DMAMUX.CHCONF[27].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG31    CE_DMAMUX.CHCONF[28].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG30    CE_DMAMUX.CHCONF[29].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG29    CE_DMAMUX.CHCONF[30].R        /* Channel Configuration */
#define CE_DMAMUX_CHCFG28    CE_DMAMUX.CHCONF[31].R        /* Channel Configuration */

/* CE_L_VFCCU */
#define CE_L_VFCCU_GDFHID_C0 CE_L_VFCCU.GDFHID_C0.R        /* Global DID-FHID Map */
#define CE_L_VFCCU_GDFHID_C1 CE_L_VFCCU.GDFHID_C1.R        /* Global DID-FHID Map */
#define CE_L_VFCCU_GFLTPO_C0 CE_L_VFCCU.GFLTPO_C0.R        /* Global Fault Polarity */
#define CE_L_VFCCU_GFLTPO_C1 CE_L_VFCCU.GFLTPO_C1.R        /* Global Fault Polarity */
#define CE_L_VFCCU_GFLTRC_C0 CE_L_VFCCU.GFLTRC_C0.R        /* Global Fault Recovery */
#define CE_L_VFCCU_GFLTRC_C1 CE_L_VFCCU.GFLTRC_C1.R        /* Global Fault Recovery */
#define CE_L_VFCCU_GFLTOVDC0 CE_L_VFCCU.GFLTOVDC0.R        /* Global Fault Overflow Detection */
#define CE_L_VFCCU_GFLTOVDC1 CE_L_VFCCU.GFLTOVDC1.R        /* Global Fault Overflow Detection */
#define CE_L_VFCCU_GSWFLODC  CE_L_VFCCU.GSWFLODC.R         /* Global Software Fault Overflow Detection Disable */
#define CE_L_VFCCU_GCTRL     CE_L_VFCCU.GCTRL.R            /* Global Space Control */
#define CE_L_VFCCU_GINTOVFS  CE_L_VFCCU.GINTOVFS.R         /* Global DID FSM Status */
#define CE_L_VFCCU_GDBGCFG   CE_L_VFCCU.GDBGCFG.R          /* Global Debug */
#define CE_L_VFCCU_GDBGSTAT  CE_L_VFCCU.GDBGSTAT.R         /* Global Debug Status */
#define CE_L_VFCCU_SWRPTDID0 CE_L_VFCCU.SW_RKN[0].SWRPTDID.R /* Software Fault Reported DID */
#define CE_L_VFCCU_SWRKSET_00 CE_L_VFCCU.SW_RKN[0].SWRKSET_0.R /* Software Reaction Set */
#define CE_L_VFCCU_SWRKCLR_00 CE_L_VFCCU.SW_RKN[0].SWRKCLR_0.R /* Software Reaction Clear */
#define CE_L_VFCCU_SWRPTDID1 CE_L_VFCCU.SW_RKN[1].SWRPTDID.R /* Software Fault Reported DID */
#define CE_L_VFCCU_SWRKSET_10 CE_L_VFCCU.SW_RKN[1].SWRKSET_0.R /* Software Reaction Set */
#define CE_L_VFCCU_SWRKCLR_10 CE_L_VFCCU.SW_RKN[1].SWRKCLR_0.R /* Software Reaction Clear */
#define CE_L_VFCCU_SWRPTDID2 CE_L_VFCCU.SW_RKN[2].SWRPTDID.R /* Software Fault Reported DID */
#define CE_L_VFCCU_SWRKSET_20 CE_L_VFCCU.SW_RKN[2].SWRKSET_0.R /* Software Reaction Set */
#define CE_L_VFCCU_SWRKCLR_20 CE_L_VFCCU.SW_RKN[2].SWRKCLR_0.R /* Software Reaction Clear */
#define CE_L_VFCCU_SWRPTDID3 CE_L_VFCCU.SW_RKN[3].SWRPTDID.R /* Software Fault Reported DID */
#define CE_L_VFCCU_SWRKSET_30 CE_L_VFCCU.SW_RKN[3].SWRKSET_0.R /* Software Reaction Set */
#define CE_L_VFCCU_SWRKCLR_30 CE_L_VFCCU.SW_RKN[3].SWRKCLR_0.R /* Software Reaction Clear */
#define CE_L_VFCCU_SWRPTDID4 CE_L_VFCCU.SW_RKN[4].SWRPTDID.R /* Software Fault Reported DID */
#define CE_L_VFCCU_SWRKSET_40 CE_L_VFCCU.SW_RKN[4].SWRKSET_0.R /* Software Reaction Set */
#define CE_L_VFCCU_SWRKCLR_40 CE_L_VFCCU.SW_RKN[4].SWRKCLR_0.R /* Software Reaction Clear */
#define CE_L_VFCCU_SWRPTDID5 CE_L_VFCCU.SW_RKN[5].SWRPTDID.R /* Software Fault Reported DID */
#define CE_L_VFCCU_SWRKSET_50 CE_L_VFCCU.SW_RKN[5].SWRKSET_0.R /* Software Reaction Set */
#define CE_L_VFCCU_SWRKCLR_50 CE_L_VFCCU.SW_RKN[5].SWRKCLR_0.R /* Software Reaction Clear */
#define CE_L_VFCCU_FHCFG0    CE_L_VFCCU.FHCFG0.R           /* Fault Handler */
#define CE_L_VFCCU_FHSRVDS0  CE_L_VFCCU.FHSRVDS0.R         /* Fault Handler Status */
#define CE_L_VFCCU_FHFLTENC00 CE_L_VFCCU.FHFLTENC00.R      /* Fault Enable */
#define CE_L_VFCCU_FHFLTENC01 CE_L_VFCCU.FHFLTENC01.R      /* Fault Enable */
#define CE_L_VFCCU_FHFLTS00  CE_L_VFCCU.FHFLTS00.R         /* Fault Status */
#define CE_L_VFCCU_FHFLTS01  CE_L_VFCCU.FHFLTS01.R         /* Fault Status */
#define CE_L_VFCCU_FHFLTRKC00 CE_L_VFCCU.FHFLTRKC00.R      /* Fault Reaction Set Configuration */
#define CE_L_VFCCU_FHFLTRKC01 CE_L_VFCCU.FHFLTRKC01.R      /* Fault Reaction Set Configuration */
#define CE_L_VFCCU_FHFLTRKC02 CE_L_VFCCU.FHFLTRKC02.R      /* Fault Reaction Set Configuration */
#define CE_L_VFCCU_FHFLTRKC03 CE_L_VFCCU.FHFLTRKC03.R      /* Fault Reaction Set Configuration */
#define CE_L_VFCCU_FHFLTRKC04 CE_L_VFCCU.FHFLTRKC04.R      /* Fault Reaction Set Configuration */
#define CE_L_VFCCU_FHFLTRKC05 CE_L_VFCCU.FHFLTRKC05.R      /* Fault Reaction Set Configuration */
#define CE_L_VFCCU_FHFLTRKC06 CE_L_VFCCU.FHFLTRKC06.R      /* Fault Reaction Set Configuration */
#define CE_L_VFCCU_FHIMRKC0_00 CE_L_VFCCU.FHIMRKC0_00.R    /* Immediate Reaction Configuration */
#define CE_L_VFCCU_FHIMRKC0_10 CE_L_VFCCU.FHIMRKC0_10.R    /* Immediate Reaction Configuration */
#define CE_L_VFCCU_FHIMRKC0_20 CE_L_VFCCU.FHIMRKC0_20.R    /* Immediate Reaction Configuration */
#define CE_L_VFCCU_FHIMRKC0_30 CE_L_VFCCU.FHIMRKC0_30.R    /* Immediate Reaction Configuration */
#define CE_L_VFCCU_FHIMRKC0_40 CE_L_VFCCU.FHIMRKC0_40.R    /* Immediate Reaction Configuration */
#define CE_L_VFCCU_FHIMRKC0_50 CE_L_VFCCU.FHIMRKC0_50.R    /* Immediate Reaction Configuration */

/* CE_SEMA42 */
#define CE_SEMA42_GATE3      CE_SEMA42.GATE[0].R           /* Gate */
#define CE_SEMA42_GATE2      CE_SEMA42.GATE[1].R           /* Gate */
#define CE_SEMA42_GATE1      CE_SEMA42.GATE[2].R           /* Gate */
#define CE_SEMA42_GATE0      CE_SEMA42.GATE[3].R           /* Gate */
#define CE_SEMA42_GATE7      CE_SEMA42.GATE[4].R           /* Gate */
#define CE_SEMA42_GATE6      CE_SEMA42.GATE[5].R           /* Gate */
#define CE_SEMA42_GATE5      CE_SEMA42.GATE[6].R           /* Gate */
#define CE_SEMA42_GATE4      CE_SEMA42.GATE[7].R           /* Gate */
#define CE_SEMA42_GATE11     CE_SEMA42.GATE[8].R           /* Gate */
#define CE_SEMA42_GATE10     CE_SEMA42.GATE[9].R           /* Gate */
#define CE_SEMA42_GATE9      CE_SEMA42.GATE[10].R          /* Gate */
#define CE_SEMA42_GATE8      CE_SEMA42.GATE[11].R          /* Gate */
#define CE_SEMA42_GATE15     CE_SEMA42.GATE[12].R          /* Gate */
#define CE_SEMA42_GATE14     CE_SEMA42.GATE[13].R          /* Gate */
#define CE_SEMA42_GATE13     CE_SEMA42.GATE[14].R          /* Gate */
#define CE_SEMA42_GATE12     CE_SEMA42.GATE[15].R          /* Gate */
#define CE_SEMA42_RSTGT_R    CE_SEMA42.RSTGT.R.R           /* Reset Gate Read */
#define CE_SEMA42_RSTGT_W    CE_SEMA42.RSTGT.W.R           /* Reset Gate Write */

/* CE_SRG_S */
#define CE_SRG_S_CR0         CE_SRG_S.CR0.R                /* Control 0 */
#define CE_SRG_S_LCR0        CE_SRG_S.LCR0.R               /* Lock Control 0 */
#define CE_SRG_S_LCR1        CE_SRG_S.LCR1.R               /* Lock Control 1 */
#define CE_SRG_S_LCR2        CE_SRG_S.LCR2.R               /* Lock Control 2 */
#define CE_SRG_S_LCR3        CE_SRG_S.LCR3.R               /* Lock Control 3 */
#define CE_SRG_S_LCR4        CE_SRG_S.LCR4.R               /* Lock Control 4 */
#define CE_SRG_S_LCR7        CE_SRG_S.LCR7.R               /* Lock Control 7 */
#define CE_SRG_S_SR0         CE_SRG_S.SR0.R                /* Status 0 */

/* CE_XRDC */
#define CE_XRDC_CR           CE_XRDC.CR.R                  /* Control */
#define CE_XRDC_HWCFG0       CE_XRDC.HWCFG0.R              /* Hardware Configuration 0 */
#define CE_XRDC_HWCFG1       CE_XRDC.HWCFG1.R              /* Hardware Configuration 1 */
#define CE_XRDC_MDACFG0      CE_XRDC.MDACFG[0].R           /* Master Domain Assignment Configuration */
#define CE_XRDC_MDACFG1      CE_XRDC.MDACFG[1].R           /* Master Domain Assignment Configuration */
#define CE_XRDC_MDACFG2      CE_XRDC.MDACFG[2].R           /* Master Domain Assignment Configuration */
#define CE_XRDC_MRCFG0       CE_XRDC.MRCFG[0].R            /* Memory Region Configuration */
#define CE_XRDC_MRCFG1       CE_XRDC.MRCFG[1].R            /* Memory Region Configuration */
#define CE_XRDC_MRCFG2       CE_XRDC.MRCFG[2].R            /* Memory Region Configuration */
#define CE_XRDC_DERRLOC0     CE_XRDC.DERRLOC[0].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC1     CE_XRDC.DERRLOC[1].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC2     CE_XRDC.DERRLOC[2].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC3     CE_XRDC.DERRLOC[3].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC4     CE_XRDC.DERRLOC[4].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC5     CE_XRDC.DERRLOC[5].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC6     CE_XRDC.DERRLOC[6].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC7     CE_XRDC.DERRLOC[7].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC8     CE_XRDC.DERRLOC[8].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC9     CE_XRDC.DERRLOC[9].R          /* Domain Error Location */
#define CE_XRDC_DERRLOC10    CE_XRDC.DERRLOC[10].R         /* Domain Error Location */
#define CE_XRDC_DERRLOC11    CE_XRDC.DERRLOC[11].R         /* Domain Error Location */
#define CE_XRDC_DERRLOC12    CE_XRDC.DERRLOC[12].R         /* Domain Error Location */
#define CE_XRDC_DERRLOC13    CE_XRDC.DERRLOC[13].R         /* Domain Error Location */
#define CE_XRDC_DERRLOC14    CE_XRDC.DERRLOC[14].R         /* Domain Error Location */
#define CE_XRDC_DERRLOC15    CE_XRDC.DERRLOC[15].R         /* Domain Error Location */
#define CE_XRDC_DERR_W0_0    CE_XRDC.DERRW0[0].DERR_W0.R   /* Domain Error Word 0 */
#define CE_XRDC_DERR_W1_0    CE_XRDC.DERRW0[0].DERR_W1.R   /* Domain Error Word 1 */
#define CE_XRDC_DERR_W3_0    CE_XRDC.DERRW0[0].DERR_W3.R   /* Domain Error Word 3 */
#define CE_XRDC_DERR_W0_1    CE_XRDC.DERRW0[1].DERR_W0.R   /* Domain Error Word 0 */
#define CE_XRDC_DERR_W1_1    CE_XRDC.DERRW0[1].DERR_W1.R   /* Domain Error Word 1 */
#define CE_XRDC_DERR_W3_1    CE_XRDC.DERRW0[1].DERR_W3.R   /* Domain Error Word 3 */
#define CE_XRDC_DERR_W0_2    CE_XRDC.DERRW0[2].DERR_W0.R   /* Domain Error Word 0 */
#define CE_XRDC_DERR_W1_2    CE_XRDC.DERRW0[2].DERR_W1.R   /* Domain Error Word 1 */
#define CE_XRDC_DERR_W3_2    CE_XRDC.DERRW0[2].DERR_W3.R   /* Domain Error Word 3 */
#define CE_XRDC_DERR_W0_16   CE_XRDC.DERRW0[16].DERR_W0.R  /* Domain Error Word 0 */
#define CE_XRDC_DERR_W1_16   CE_XRDC.DERRW0[16].DERR_W1.R  /* Domain Error Word 1 */
#define CE_XRDC_DERR_W3_16   CE_XRDC.DERRW0[16].DERR_W3.R  /* Domain Error Word 3 */
#define CE_XRDC_DERR_W0_17   CE_XRDC.DERRW0[17].DERR_W0.R  /* Domain Error Word 0 */
#define CE_XRDC_DERR_W1_17   CE_XRDC.DERRW0[17].DERR_W1.R  /* Domain Error Word 1 */
#define CE_XRDC_DERR_W3_17   CE_XRDC.DERRW0[17].DERR_W3.R  /* Domain Error Word 3 */
#define CE_XRDC_DERR_W0_18   CE_XRDC.DERRW0[18].DERR_W0.R  /* Domain Error Word 0 */
#define CE_XRDC_DERR_W1_18   CE_XRDC.DERRW0[18].DERR_W1.R  /* Domain Error Word 1 */
#define CE_XRDC_DERR_W3_18   CE_XRDC.DERRW0[18].DERR_W3.R  /* Domain Error Word 3 */
#define CE_XRDC_DERR_W0_19   CE_XRDC.DERRW0[19].DERR_W0.R  /* Domain Error Word 0 */
#define CE_XRDC_DERR_W1_19   CE_XRDC.DERRW0[19].DERR_W1.R  /* Domain Error Word 1 */
#define CE_XRDC_DERR_W3_19   CE_XRDC.DERRW0[19].DERR_W3.R  /* Domain Error Word 3 */
#define CE_XRDC_MDA_W0_0_DFMT1 CE_XRDC.MDA_W0_0_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W1_0_DFMT1 CE_XRDC.MDA_W1_0_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W2_0_DFMT1 CE_XRDC.MDA_W2_0_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W3_0_DFMT1 CE_XRDC.MDA_W3_0_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W4_0_DFMT1 CE_XRDC.MDA_W4_0_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W5_0_DFMT1 CE_XRDC.MDA_W5_0_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W6_0_DFMT1 CE_XRDC.MDA_W6_0_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W7_0_DFMT1 CE_XRDC.MDA_W7_0_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W0_1_DFMT1 CE_XRDC.MDA_W0_1_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W0_2_DFMT1 CE_XRDC.MDA_W0_2_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W1_2_DFMT1 CE_XRDC.MDA_W1_2_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W2_2_DFMT1 CE_XRDC.MDA_W2_2_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W3_2_DFMT1 CE_XRDC.MDA_W3_2_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W4_2_DFMT1 CE_XRDC.MDA_W4_2_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W5_2_DFMT1 CE_XRDC.MDA_W5_2_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W6_2_DFMT1 CE_XRDC.MDA_W6_2_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_MDA_W7_2_DFMT1 CE_XRDC.MDA_W7_2_DFMT1.R    /* Master Domain Assignment */
#define CE_XRDC_PDAC_W0_0    CE_XRDC.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_0    CE_XRDC.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_4    CE_XRDC.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_4    CE_XRDC.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_5    CE_XRDC.PDAC_SLOT[0].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_5    CE_XRDC.PDAC_SLOT[0].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_6    CE_XRDC.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_6    CE_XRDC.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_7    CE_XRDC.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_7    CE_XRDC.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_8    CE_XRDC.PDAC_SLOT[0].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_8    CE_XRDC.PDAC_SLOT[0].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_9    CE_XRDC.PDAC_SLOT[0].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_9    CE_XRDC.PDAC_SLOT[0].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_12   CE_XRDC.PDAC_SLOT[0].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_12   CE_XRDC.PDAC_SLOT[0].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_16   CE_XRDC.PDAC_SLOT[0].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_16   CE_XRDC.PDAC_SLOT[0].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_17   CE_XRDC.PDAC_SLOT[0].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_17   CE_XRDC.PDAC_SLOT[0].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_18   CE_XRDC.PDAC_SLOT[0].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_18   CE_XRDC.PDAC_SLOT[0].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_19   CE_XRDC.PDAC_SLOT[0].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_19   CE_XRDC.PDAC_SLOT[0].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_20   CE_XRDC.PDAC_SLOT[0].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_20   CE_XRDC.PDAC_SLOT[0].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_21   CE_XRDC.PDAC_SLOT[0].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_21   CE_XRDC.PDAC_SLOT[0].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_24   CE_XRDC.PDAC_SLOT[0].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_24   CE_XRDC.PDAC_SLOT[0].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_25   CE_XRDC.PDAC_SLOT[0].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_25   CE_XRDC.PDAC_SLOT[0].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_35   CE_XRDC.PDAC_SLOT[0].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_35   CE_XRDC.PDAC_SLOT[0].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_47   CE_XRDC.PDAC_SLOT[0].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_47   CE_XRDC.PDAC_SLOT[0].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_54   CE_XRDC.PDAC_SLOT[0].PDACN[54].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_54   CE_XRDC.PDAC_SLOT[0].PDACN[54].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_55   CE_XRDC.PDAC_SLOT[0].PDACN[55].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_55   CE_XRDC.PDAC_SLOT[0].PDACN[55].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_59   CE_XRDC.PDAC_SLOT[0].PDACN[59].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_59   CE_XRDC.PDAC_SLOT[0].PDACN[59].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_60   CE_XRDC.PDAC_SLOT[0].PDACN[60].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_60   CE_XRDC.PDAC_SLOT[0].PDACN[60].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_67   CE_XRDC.PDAC_SLOT[0].PDACN[67].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_67   CE_XRDC.PDAC_SLOT[0].PDACN[67].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_68   CE_XRDC.PDAC_SLOT[0].PDACN[68].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_68   CE_XRDC.PDAC_SLOT[0].PDACN[68].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_75   CE_XRDC.PDAC_SLOT[0].PDACN[75].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_75   CE_XRDC.PDAC_SLOT[0].PDACN[75].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_76   CE_XRDC.PDAC_SLOT[0].PDACN[76].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_76   CE_XRDC.PDAC_SLOT[0].PDACN[76].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_83   CE_XRDC.PDAC_SLOT[0].PDACN[83].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_83   CE_XRDC.PDAC_SLOT[0].PDACN[83].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_84   CE_XRDC.PDAC_SLOT[0].PDACN[84].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_84   CE_XRDC.PDAC_SLOT[0].PDACN[84].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_91   CE_XRDC.PDAC_SLOT[0].PDACN[91].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_91   CE_XRDC.PDAC_SLOT[0].PDACN[91].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_92   CE_XRDC.PDAC_SLOT[0].PDACN[92].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_92   CE_XRDC.PDAC_SLOT[0].PDACN[92].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_99   CE_XRDC.PDAC_SLOT[0].PDACN[99].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_99   CE_XRDC.PDAC_SLOT[0].PDACN[99].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_100  CE_XRDC.PDAC_SLOT[0].PDACN[100].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_100  CE_XRDC.PDAC_SLOT[0].PDACN[100].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_104  CE_XRDC.PDAC_SLOT[0].PDACN[104].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_104  CE_XRDC.PDAC_SLOT[0].PDACN[104].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_106  CE_XRDC.PDAC_SLOT[0].PDACN[106].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_106  CE_XRDC.PDAC_SLOT[0].PDACN[106].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_108  CE_XRDC.PDAC_SLOT[0].PDACN[108].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_108  CE_XRDC.PDAC_SLOT[0].PDACN[108].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_110  CE_XRDC.PDAC_SLOT[0].PDACN[110].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_110  CE_XRDC.PDAC_SLOT[0].PDACN[110].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_112  CE_XRDC.PDAC_SLOT[0].PDACN[112].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_112  CE_XRDC.PDAC_SLOT[0].PDACN[112].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_114  CE_XRDC.PDAC_SLOT[0].PDACN[114].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_114  CE_XRDC.PDAC_SLOT[0].PDACN[114].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_116  CE_XRDC.PDAC_SLOT[0].PDACN[116].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_116  CE_XRDC.PDAC_SLOT[0].PDACN[116].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_118  CE_XRDC.PDAC_SLOT[0].PDACN[118].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_118  CE_XRDC.PDAC_SLOT[0].PDACN[118].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_120  CE_XRDC.PDAC_SLOT[0].PDACN[120].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_120  CE_XRDC.PDAC_SLOT[0].PDACN[120].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_122  CE_XRDC.PDAC_SLOT[0].PDACN[122].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_122  CE_XRDC.PDAC_SLOT[0].PDACN[122].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_124  CE_XRDC.PDAC_SLOT[0].PDACN[124].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_124  CE_XRDC.PDAC_SLOT[0].PDACN[124].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_126  CE_XRDC.PDAC_SLOT[0].PDACN[126].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_126  CE_XRDC.PDAC_SLOT[0].PDACN[126].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_132  CE_XRDC.PDAC_SLOT[1].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_132  CE_XRDC.PDAC_SLOT[1].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_136  CE_XRDC.PDAC_SLOT[1].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_136  CE_XRDC.PDAC_SLOT[1].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_137  CE_XRDC.PDAC_SLOT[1].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_137  CE_XRDC.PDAC_SLOT[1].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_140  CE_XRDC.PDAC_SLOT[1].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_140  CE_XRDC.PDAC_SLOT[1].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_144  CE_XRDC.PDAC_SLOT[1].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_144  CE_XRDC.PDAC_SLOT[1].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_149  CE_XRDC.PDAC_SLOT[1].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_149  CE_XRDC.PDAC_SLOT[1].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_152  CE_XRDC.PDAC_SLOT[1].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_152  CE_XRDC.PDAC_SLOT[1].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_153  CE_XRDC.PDAC_SLOT[1].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_153  CE_XRDC.PDAC_SLOT[1].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_154  CE_XRDC.PDAC_SLOT[1].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_154  CE_XRDC.PDAC_SLOT[1].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_160  CE_XRDC.PDAC_SLOT[1].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_160  CE_XRDC.PDAC_SLOT[1].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_162  CE_XRDC.PDAC_SLOT[1].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_162  CE_XRDC.PDAC_SLOT[1].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_164  CE_XRDC.PDAC_SLOT[1].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_164  CE_XRDC.PDAC_SLOT[1].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_166  CE_XRDC.PDAC_SLOT[1].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_166  CE_XRDC.PDAC_SLOT[1].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_168  CE_XRDC.PDAC_SLOT[1].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_168  CE_XRDC.PDAC_SLOT[1].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_170  CE_XRDC.PDAC_SLOT[1].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_170  CE_XRDC.PDAC_SLOT[1].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_172  CE_XRDC.PDAC_SLOT[1].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_172  CE_XRDC.PDAC_SLOT[1].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_174  CE_XRDC.PDAC_SLOT[1].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_174  CE_XRDC.PDAC_SLOT[1].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_176  CE_XRDC.PDAC_SLOT[1].PDACN[48].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_176  CE_XRDC.PDAC_SLOT[1].PDACN[48].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_178  CE_XRDC.PDAC_SLOT[1].PDACN[50].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_178  CE_XRDC.PDAC_SLOT[1].PDACN[50].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_180  CE_XRDC.PDAC_SLOT[1].PDACN[52].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_180  CE_XRDC.PDAC_SLOT[1].PDACN[52].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_182  CE_XRDC.PDAC_SLOT[1].PDACN[54].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_182  CE_XRDC.PDAC_SLOT[1].PDACN[54].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_187  CE_XRDC.PDAC_SLOT[1].PDACN[59].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_187  CE_XRDC.PDAC_SLOT[1].PDACN[59].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_188  CE_XRDC.PDAC_SLOT[1].PDACN[60].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_188  CE_XRDC.PDAC_SLOT[1].PDACN[60].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_189  CE_XRDC.PDAC_SLOT[1].PDACN[61].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_189  CE_XRDC.PDAC_SLOT[1].PDACN[61].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_190  CE_XRDC.PDAC_SLOT[1].PDACN[62].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_190  CE_XRDC.PDAC_SLOT[1].PDACN[62].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_191  CE_XRDC.PDAC_SLOT[1].PDACN[63].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_191  CE_XRDC.PDAC_SLOT[1].PDACN[63].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_195  CE_XRDC.PDAC_SLOT[1].PDACN[67].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_195  CE_XRDC.PDAC_SLOT[1].PDACN[67].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_196  CE_XRDC.PDAC_SLOT[1].PDACN[68].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_196  CE_XRDC.PDAC_SLOT[1].PDACN[68].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_197  CE_XRDC.PDAC_SLOT[1].PDACN[69].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_197  CE_XRDC.PDAC_SLOT[1].PDACN[69].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_198  CE_XRDC.PDAC_SLOT[1].PDACN[70].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_198  CE_XRDC.PDAC_SLOT[1].PDACN[70].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_199  CE_XRDC.PDAC_SLOT[1].PDACN[71].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_199  CE_XRDC.PDAC_SLOT[1].PDACN[71].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_203  CE_XRDC.PDAC_SLOT[1].PDACN[75].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_203  CE_XRDC.PDAC_SLOT[1].PDACN[75].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_204  CE_XRDC.PDAC_SLOT[1].PDACN[76].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_204  CE_XRDC.PDAC_SLOT[1].PDACN[76].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_205  CE_XRDC.PDAC_SLOT[1].PDACN[77].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_205  CE_XRDC.PDAC_SLOT[1].PDACN[77].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_206  CE_XRDC.PDAC_SLOT[1].PDACN[78].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_206  CE_XRDC.PDAC_SLOT[1].PDACN[78].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_207  CE_XRDC.PDAC_SLOT[1].PDACN[79].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_207  CE_XRDC.PDAC_SLOT[1].PDACN[79].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_211  CE_XRDC.PDAC_SLOT[1].PDACN[83].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_211  CE_XRDC.PDAC_SLOT[1].PDACN[83].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_212  CE_XRDC.PDAC_SLOT[1].PDACN[84].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_212  CE_XRDC.PDAC_SLOT[1].PDACN[84].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_213  CE_XRDC.PDAC_SLOT[1].PDACN[85].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_213  CE_XRDC.PDAC_SLOT[1].PDACN[85].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_214  CE_XRDC.PDAC_SLOT[1].PDACN[86].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_214  CE_XRDC.PDAC_SLOT[1].PDACN[86].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_215  CE_XRDC.PDAC_SLOT[1].PDACN[87].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_215  CE_XRDC.PDAC_SLOT[1].PDACN[87].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_219  CE_XRDC.PDAC_SLOT[1].PDACN[91].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_219  CE_XRDC.PDAC_SLOT[1].PDACN[91].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_220  CE_XRDC.PDAC_SLOT[1].PDACN[92].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_220  CE_XRDC.PDAC_SLOT[1].PDACN[92].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_221  CE_XRDC.PDAC_SLOT[1].PDACN[93].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_221  CE_XRDC.PDAC_SLOT[1].PDACN[93].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_222  CE_XRDC.PDAC_SLOT[1].PDACN[94].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_222  CE_XRDC.PDAC_SLOT[1].PDACN[94].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_223  CE_XRDC.PDAC_SLOT[1].PDACN[95].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_223  CE_XRDC.PDAC_SLOT[1].PDACN[95].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_227  CE_XRDC.PDAC_SLOT[1].PDACN[99].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_227  CE_XRDC.PDAC_SLOT[1].PDACN[99].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_228  CE_XRDC.PDAC_SLOT[1].PDACN[100].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_228  CE_XRDC.PDAC_SLOT[1].PDACN[100].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_229  CE_XRDC.PDAC_SLOT[1].PDACN[101].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_229  CE_XRDC.PDAC_SLOT[1].PDACN[101].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_230  CE_XRDC.PDAC_SLOT[1].PDACN[102].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_230  CE_XRDC.PDAC_SLOT[1].PDACN[102].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_231  CE_XRDC.PDAC_SLOT[1].PDACN[103].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_231  CE_XRDC.PDAC_SLOT[1].PDACN[103].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_232  CE_XRDC.PDAC_SLOT[1].PDACN[104].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_232  CE_XRDC.PDAC_SLOT[1].PDACN[104].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_234  CE_XRDC.PDAC_SLOT[1].PDACN[106].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_234  CE_XRDC.PDAC_SLOT[1].PDACN[106].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_236  CE_XRDC.PDAC_SLOT[1].PDACN[108].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_236  CE_XRDC.PDAC_SLOT[1].PDACN[108].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_238  CE_XRDC.PDAC_SLOT[1].PDACN[110].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_238  CE_XRDC.PDAC_SLOT[1].PDACN[110].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_240  CE_XRDC.PDAC_SLOT[1].PDACN[112].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_240  CE_XRDC.PDAC_SLOT[1].PDACN[112].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_242  CE_XRDC.PDAC_SLOT[1].PDACN[114].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_242  CE_XRDC.PDAC_SLOT[1].PDACN[114].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_244  CE_XRDC.PDAC_SLOT[1].PDACN[116].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_244  CE_XRDC.PDAC_SLOT[1].PDACN[116].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_246  CE_XRDC.PDAC_SLOT[1].PDACN[118].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_246  CE_XRDC.PDAC_SLOT[1].PDACN[118].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_248  CE_XRDC.PDAC_SLOT[1].PDACN[120].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_248  CE_XRDC.PDAC_SLOT[1].PDACN[120].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_250  CE_XRDC.PDAC_SLOT[1].PDACN[122].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_250  CE_XRDC.PDAC_SLOT[1].PDACN[122].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_252  CE_XRDC.PDAC_SLOT[1].PDACN[124].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_252  CE_XRDC.PDAC_SLOT[1].PDACN[124].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_254  CE_XRDC.PDAC_SLOT[1].PDACN[126].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_254  CE_XRDC.PDAC_SLOT[1].PDACN[126].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_259  CE_XRDC.PDAC_SLOT[2].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_259  CE_XRDC.PDAC_SLOT[2].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_260  CE_XRDC.PDAC_SLOT[2].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_260  CE_XRDC.PDAC_SLOT[2].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_261  CE_XRDC.PDAC_SLOT[2].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_261  CE_XRDC.PDAC_SLOT[2].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_262  CE_XRDC.PDAC_SLOT[2].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_262  CE_XRDC.PDAC_SLOT[2].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_263  CE_XRDC.PDAC_SLOT[2].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_263  CE_XRDC.PDAC_SLOT[2].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_271  CE_XRDC.PDAC_SLOT[2].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_271  CE_XRDC.PDAC_SLOT[2].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_272  CE_XRDC.PDAC_SLOT[2].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_272  CE_XRDC.PDAC_SLOT[2].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_273  CE_XRDC.PDAC_SLOT[2].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_273  CE_XRDC.PDAC_SLOT[2].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_274  CE_XRDC.PDAC_SLOT[2].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_274  CE_XRDC.PDAC_SLOT[2].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_275  CE_XRDC.PDAC_SLOT[2].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_275  CE_XRDC.PDAC_SLOT[2].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_283  CE_XRDC.PDAC_SLOT[2].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_283  CE_XRDC.PDAC_SLOT[2].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_284  CE_XRDC.PDAC_SLOT[2].PDACN[28].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_284  CE_XRDC.PDAC_SLOT[2].PDACN[28].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_285  CE_XRDC.PDAC_SLOT[2].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_285  CE_XRDC.PDAC_SLOT[2].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_286  CE_XRDC.PDAC_SLOT[2].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_286  CE_XRDC.PDAC_SLOT[2].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_287  CE_XRDC.PDAC_SLOT[2].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_287  CE_XRDC.PDAC_SLOT[2].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_295  CE_XRDC.PDAC_SLOT[2].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_295  CE_XRDC.PDAC_SLOT[2].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_296  CE_XRDC.PDAC_SLOT[2].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_296  CE_XRDC.PDAC_SLOT[2].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_297  CE_XRDC.PDAC_SLOT[2].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_297  CE_XRDC.PDAC_SLOT[2].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_298  CE_XRDC.PDAC_SLOT[2].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_298  CE_XRDC.PDAC_SLOT[2].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_299  CE_XRDC.PDAC_SLOT[2].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_299  CE_XRDC.PDAC_SLOT[2].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_307  CE_XRDC.PDAC_SLOT[2].PDACN[51].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_307  CE_XRDC.PDAC_SLOT[2].PDACN[51].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_308  CE_XRDC.PDAC_SLOT[2].PDACN[52].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_308  CE_XRDC.PDAC_SLOT[2].PDACN[52].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_309  CE_XRDC.PDAC_SLOT[2].PDACN[53].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_309  CE_XRDC.PDAC_SLOT[2].PDACN[53].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_310  CE_XRDC.PDAC_SLOT[2].PDACN[54].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_310  CE_XRDC.PDAC_SLOT[2].PDACN[54].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_311  CE_XRDC.PDAC_SLOT[2].PDACN[55].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_311  CE_XRDC.PDAC_SLOT[2].PDACN[55].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_319  CE_XRDC.PDAC_SLOT[2].PDACN[63].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_319  CE_XRDC.PDAC_SLOT[2].PDACN[63].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_320  CE_XRDC.PDAC_SLOT[2].PDACN[64].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_320  CE_XRDC.PDAC_SLOT[2].PDACN[64].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_321  CE_XRDC.PDAC_SLOT[2].PDACN[65].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_321  CE_XRDC.PDAC_SLOT[2].PDACN[65].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_322  CE_XRDC.PDAC_SLOT[2].PDACN[66].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_322  CE_XRDC.PDAC_SLOT[2].PDACN[66].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_323  CE_XRDC.PDAC_SLOT[2].PDACN[67].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_323  CE_XRDC.PDAC_SLOT[2].PDACN[67].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_331  CE_XRDC.PDAC_SLOT[2].PDACN[75].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_331  CE_XRDC.PDAC_SLOT[2].PDACN[75].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_332  CE_XRDC.PDAC_SLOT[2].PDACN[76].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_332  CE_XRDC.PDAC_SLOT[2].PDACN[76].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_339  CE_XRDC.PDAC_SLOT[2].PDACN[83].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_339  CE_XRDC.PDAC_SLOT[2].PDACN[83].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_340  CE_XRDC.PDAC_SLOT[2].PDACN[84].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_340  CE_XRDC.PDAC_SLOT[2].PDACN[84].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_360  CE_XRDC.PDAC_SLOT[2].PDACN[104].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_360  CE_XRDC.PDAC_SLOT[2].PDACN[104].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_362  CE_XRDC.PDAC_SLOT[2].PDACN[106].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_362  CE_XRDC.PDAC_SLOT[2].PDACN[106].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_364  CE_XRDC.PDAC_SLOT[2].PDACN[108].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_364  CE_XRDC.PDAC_SLOT[2].PDACN[108].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_366  CE_XRDC.PDAC_SLOT[2].PDACN[110].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_366  CE_XRDC.PDAC_SLOT[2].PDACN[110].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_368  CE_XRDC.PDAC_SLOT[2].PDACN[112].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_368  CE_XRDC.PDAC_SLOT[2].PDACN[112].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_370  CE_XRDC.PDAC_SLOT[2].PDACN[114].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_370  CE_XRDC.PDAC_SLOT[2].PDACN[114].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_372  CE_XRDC.PDAC_SLOT[2].PDACN[116].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_372  CE_XRDC.PDAC_SLOT[2].PDACN[116].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_374  CE_XRDC.PDAC_SLOT[2].PDACN[118].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_374  CE_XRDC.PDAC_SLOT[2].PDACN[118].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_376  CE_XRDC.PDAC_SLOT[2].PDACN[120].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_376  CE_XRDC.PDAC_SLOT[2].PDACN[120].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_378  CE_XRDC.PDAC_SLOT[2].PDACN[122].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_378  CE_XRDC.PDAC_SLOT[2].PDACN[122].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_380  CE_XRDC.PDAC_SLOT[2].PDACN[124].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_380  CE_XRDC.PDAC_SLOT[2].PDACN[124].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_382  CE_XRDC.PDAC_SLOT[2].PDACN[126].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_382  CE_XRDC.PDAC_SLOT[2].PDACN[126].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_384  CE_XRDC.PDAC_SLOT[3].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_384  CE_XRDC.PDAC_SLOT[3].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_385  CE_XRDC.PDAC_SLOT[3].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_385  CE_XRDC.PDAC_SLOT[3].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_386  CE_XRDC.PDAC_SLOT[3].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_386  CE_XRDC.PDAC_SLOT[3].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_388  CE_XRDC.PDAC_SLOT[3].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_388  CE_XRDC.PDAC_SLOT[3].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_389  CE_XRDC.PDAC_SLOT[3].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_389  CE_XRDC.PDAC_SLOT[3].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_393  CE_XRDC.PDAC_SLOT[3].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_393  CE_XRDC.PDAC_SLOT[3].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_397  CE_XRDC.PDAC_SLOT[3].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_397  CE_XRDC.PDAC_SLOT[3].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_401  CE_XRDC.PDAC_SLOT[3].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_401  CE_XRDC.PDAC_SLOT[3].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_405  CE_XRDC.PDAC_SLOT[3].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_405  CE_XRDC.PDAC_SLOT[3].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_409  CE_XRDC.PDAC_SLOT[3].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_409  CE_XRDC.PDAC_SLOT[3].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_413  CE_XRDC.PDAC_SLOT[3].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_413  CE_XRDC.PDAC_SLOT[3].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_417  CE_XRDC.PDAC_SLOT[3].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_417  CE_XRDC.PDAC_SLOT[3].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_421  CE_XRDC.PDAC_SLOT[3].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_421  CE_XRDC.PDAC_SLOT[3].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_425  CE_XRDC.PDAC_SLOT[3].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_425  CE_XRDC.PDAC_SLOT[3].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_429  CE_XRDC.PDAC_SLOT[3].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_429  CE_XRDC.PDAC_SLOT[3].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_433  CE_XRDC.PDAC_SLOT[3].PDACN[49].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_433  CE_XRDC.PDAC_SLOT[3].PDACN[49].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_437  CE_XRDC.PDAC_SLOT[3].PDACN[53].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_437  CE_XRDC.PDAC_SLOT[3].PDACN[53].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_441  CE_XRDC.PDAC_SLOT[3].PDACN[57].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_441  CE_XRDC.PDAC_SLOT[3].PDACN[57].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_445  CE_XRDC.PDAC_SLOT[3].PDACN[61].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_445  CE_XRDC.PDAC_SLOT[3].PDACN[61].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_449  CE_XRDC.PDAC_SLOT[3].PDACN[65].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_449  CE_XRDC.PDAC_SLOT[3].PDACN[65].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_453  CE_XRDC.PDAC_SLOT[3].PDACN[69].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_453  CE_XRDC.PDAC_SLOT[3].PDACN[69].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_457  CE_XRDC.PDAC_SLOT[3].PDACN[73].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_457  CE_XRDC.PDAC_SLOT[3].PDACN[73].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_461  CE_XRDC.PDAC_SLOT[3].PDACN[77].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_461  CE_XRDC.PDAC_SLOT[3].PDACN[77].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_465  CE_XRDC.PDAC_SLOT[3].PDACN[81].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_465  CE_XRDC.PDAC_SLOT[3].PDACN[81].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_469  CE_XRDC.PDAC_SLOT[3].PDACN[85].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_469  CE_XRDC.PDAC_SLOT[3].PDACN[85].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_473  CE_XRDC.PDAC_SLOT[3].PDACN[89].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_473  CE_XRDC.PDAC_SLOT[3].PDACN[89].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_477  CE_XRDC.PDAC_SLOT[3].PDACN[93].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_477  CE_XRDC.PDAC_SLOT[3].PDACN[93].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_481  CE_XRDC.PDAC_SLOT[3].PDACN[97].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_481  CE_XRDC.PDAC_SLOT[3].PDACN[97].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_485  CE_XRDC.PDAC_SLOT[3].PDACN[101].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_485  CE_XRDC.PDAC_SLOT[3].PDACN[101].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_488  CE_XRDC.PDAC_SLOT[3].PDACN[104].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_488  CE_XRDC.PDAC_SLOT[3].PDACN[104].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_489  CE_XRDC.PDAC_SLOT[3].PDACN[105].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_489  CE_XRDC.PDAC_SLOT[3].PDACN[105].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_490  CE_XRDC.PDAC_SLOT[3].PDACN[106].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_490  CE_XRDC.PDAC_SLOT[3].PDACN[106].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_492  CE_XRDC.PDAC_SLOT[3].PDACN[108].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_492  CE_XRDC.PDAC_SLOT[3].PDACN[108].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_493  CE_XRDC.PDAC_SLOT[3].PDACN[109].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_493  CE_XRDC.PDAC_SLOT[3].PDACN[109].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_494  CE_XRDC.PDAC_SLOT[3].PDACN[110].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_494  CE_XRDC.PDAC_SLOT[3].PDACN[110].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_496  CE_XRDC.PDAC_SLOT[3].PDACN[112].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_496  CE_XRDC.PDAC_SLOT[3].PDACN[112].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_497  CE_XRDC.PDAC_SLOT[3].PDACN[113].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_497  CE_XRDC.PDAC_SLOT[3].PDACN[113].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_498  CE_XRDC.PDAC_SLOT[3].PDACN[114].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_498  CE_XRDC.PDAC_SLOT[3].PDACN[114].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_500  CE_XRDC.PDAC_SLOT[3].PDACN[116].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_500  CE_XRDC.PDAC_SLOT[3].PDACN[116].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_501  CE_XRDC.PDAC_SLOT[3].PDACN[117].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_501  CE_XRDC.PDAC_SLOT[3].PDACN[117].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_502  CE_XRDC.PDAC_SLOT[3].PDACN[118].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_502  CE_XRDC.PDAC_SLOT[3].PDACN[118].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_504  CE_XRDC.PDAC_SLOT[3].PDACN[120].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_504  CE_XRDC.PDAC_SLOT[3].PDACN[120].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_505  CE_XRDC.PDAC_SLOT[3].PDACN[121].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_505  CE_XRDC.PDAC_SLOT[3].PDACN[121].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_506  CE_XRDC.PDAC_SLOT[3].PDACN[122].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_506  CE_XRDC.PDAC_SLOT[3].PDACN[122].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_508  CE_XRDC.PDAC_SLOT[3].PDACN[124].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_508  CE_XRDC.PDAC_SLOT[3].PDACN[124].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_509  CE_XRDC.PDAC_SLOT[3].PDACN[125].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_509  CE_XRDC.PDAC_SLOT[3].PDACN[125].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_PDAC_W0_510  CE_XRDC.PDAC_SLOT[3].PDACN[126].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define CE_XRDC_PDAC_W1_510  CE_XRDC.PDAC_SLOT[3].PDACN[126].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define CE_XRDC_MRGD_W0_0    CE_XRDC.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_0    CE_XRDC.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_0    CE_XRDC.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_0    CE_XRDC.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_1    CE_XRDC.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_1    CE_XRDC.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_1    CE_XRDC.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_1    CE_XRDC.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_2    CE_XRDC.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_2    CE_XRDC.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_2    CE_XRDC.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_2    CE_XRDC.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_3    CE_XRDC.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_3    CE_XRDC.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_3    CE_XRDC.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_3    CE_XRDC.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_4    CE_XRDC.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_4    CE_XRDC.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_4    CE_XRDC.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_4    CE_XRDC.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_5    CE_XRDC.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_5    CE_XRDC.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_5    CE_XRDC.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_5    CE_XRDC.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_6    CE_XRDC.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_6    CE_XRDC.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_6    CE_XRDC.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_6    CE_XRDC.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_7    CE_XRDC.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_7    CE_XRDC.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_7    CE_XRDC.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_7    CE_XRDC.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_8    CE_XRDC.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_8    CE_XRDC.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_8    CE_XRDC.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_8    CE_XRDC.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_9    CE_XRDC.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_9    CE_XRDC.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_9    CE_XRDC.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_9    CE_XRDC.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_10   CE_XRDC.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_10   CE_XRDC.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_10   CE_XRDC.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_10   CE_XRDC.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_11   CE_XRDC.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_11   CE_XRDC.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_11   CE_XRDC.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_11   CE_XRDC.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_12   CE_XRDC.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_12   CE_XRDC.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_12   CE_XRDC.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_12   CE_XRDC.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_13   CE_XRDC.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_13   CE_XRDC.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_13   CE_XRDC.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_13   CE_XRDC.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_14   CE_XRDC.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_14   CE_XRDC.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_14   CE_XRDC.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_14   CE_XRDC.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_15   CE_XRDC.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_15   CE_XRDC.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_15   CE_XRDC.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_15   CE_XRDC.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_16   CE_XRDC.MRGDN[1][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_16   CE_XRDC.MRGDN[1][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_16   CE_XRDC.MRGDN[1][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_16   CE_XRDC.MRGDN[1][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_17   CE_XRDC.MRGDN[1][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_17   CE_XRDC.MRGDN[1][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_17   CE_XRDC.MRGDN[1][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_17   CE_XRDC.MRGDN[1][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_18   CE_XRDC.MRGDN[1][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_18   CE_XRDC.MRGDN[1][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_18   CE_XRDC.MRGDN[1][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_18   CE_XRDC.MRGDN[1][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_19   CE_XRDC.MRGDN[1][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_19   CE_XRDC.MRGDN[1][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_19   CE_XRDC.MRGDN[1][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_19   CE_XRDC.MRGDN[1][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_20   CE_XRDC.MRGDN[1][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_20   CE_XRDC.MRGDN[1][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_20   CE_XRDC.MRGDN[1][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_20   CE_XRDC.MRGDN[1][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_21   CE_XRDC.MRGDN[1][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_21   CE_XRDC.MRGDN[1][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_21   CE_XRDC.MRGDN[1][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_21   CE_XRDC.MRGDN[1][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_22   CE_XRDC.MRGDN[1][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_22   CE_XRDC.MRGDN[1][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_22   CE_XRDC.MRGDN[1][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_22   CE_XRDC.MRGDN[1][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_23   CE_XRDC.MRGDN[1][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_23   CE_XRDC.MRGDN[1][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_23   CE_XRDC.MRGDN[1][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_23   CE_XRDC.MRGDN[1][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_24   CE_XRDC.MRGDN[1][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_24   CE_XRDC.MRGDN[1][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_24   CE_XRDC.MRGDN[1][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_24   CE_XRDC.MRGDN[1][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_25   CE_XRDC.MRGDN[1][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_25   CE_XRDC.MRGDN[1][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_25   CE_XRDC.MRGDN[1][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_25   CE_XRDC.MRGDN[1][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_26   CE_XRDC.MRGDN[1][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_26   CE_XRDC.MRGDN[1][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_26   CE_XRDC.MRGDN[1][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_26   CE_XRDC.MRGDN[1][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_27   CE_XRDC.MRGDN[1][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_27   CE_XRDC.MRGDN[1][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_27   CE_XRDC.MRGDN[1][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_27   CE_XRDC.MRGDN[1][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_28   CE_XRDC.MRGDN[1][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_28   CE_XRDC.MRGDN[1][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_28   CE_XRDC.MRGDN[1][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_28   CE_XRDC.MRGDN[1][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_29   CE_XRDC.MRGDN[1][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_29   CE_XRDC.MRGDN[1][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_29   CE_XRDC.MRGDN[1][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_29   CE_XRDC.MRGDN[1][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_30   CE_XRDC.MRGDN[1][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_30   CE_XRDC.MRGDN[1][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_30   CE_XRDC.MRGDN[1][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_30   CE_XRDC.MRGDN[1][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_31   CE_XRDC.MRGDN[1][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_31   CE_XRDC.MRGDN[1][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_31   CE_XRDC.MRGDN[1][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_31   CE_XRDC.MRGDN[1][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_32   CE_XRDC.MRGDN[2][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_32   CE_XRDC.MRGDN[2][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_32   CE_XRDC.MRGDN[2][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_32   CE_XRDC.MRGDN[2][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_33   CE_XRDC.MRGDN[2][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_33   CE_XRDC.MRGDN[2][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_33   CE_XRDC.MRGDN[2][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_33   CE_XRDC.MRGDN[2][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_34   CE_XRDC.MRGDN[2][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_34   CE_XRDC.MRGDN[2][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_34   CE_XRDC.MRGDN[2][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_34   CE_XRDC.MRGDN[2][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_35   CE_XRDC.MRGDN[2][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_35   CE_XRDC.MRGDN[2][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_35   CE_XRDC.MRGDN[2][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_35   CE_XRDC.MRGDN[2][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_36   CE_XRDC.MRGDN[2][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_36   CE_XRDC.MRGDN[2][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_36   CE_XRDC.MRGDN[2][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_36   CE_XRDC.MRGDN[2][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_37   CE_XRDC.MRGDN[2][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_37   CE_XRDC.MRGDN[2][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_37   CE_XRDC.MRGDN[2][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_37   CE_XRDC.MRGDN[2][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_38   CE_XRDC.MRGDN[2][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_38   CE_XRDC.MRGDN[2][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_38   CE_XRDC.MRGDN[2][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_38   CE_XRDC.MRGDN[2][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_39   CE_XRDC.MRGDN[2][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_39   CE_XRDC.MRGDN[2][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_39   CE_XRDC.MRGDN[2][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_39   CE_XRDC.MRGDN[2][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_40   CE_XRDC.MRGDN[2][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_40   CE_XRDC.MRGDN[2][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_40   CE_XRDC.MRGDN[2][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_40   CE_XRDC.MRGDN[2][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_41   CE_XRDC.MRGDN[2][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_41   CE_XRDC.MRGDN[2][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_41   CE_XRDC.MRGDN[2][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_41   CE_XRDC.MRGDN[2][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_42   CE_XRDC.MRGDN[2][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_42   CE_XRDC.MRGDN[2][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_42   CE_XRDC.MRGDN[2][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_42   CE_XRDC.MRGDN[2][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_43   CE_XRDC.MRGDN[2][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_43   CE_XRDC.MRGDN[2][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_43   CE_XRDC.MRGDN[2][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_43   CE_XRDC.MRGDN[2][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_44   CE_XRDC.MRGDN[2][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_44   CE_XRDC.MRGDN[2][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_44   CE_XRDC.MRGDN[2][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_44   CE_XRDC.MRGDN[2][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_45   CE_XRDC.MRGDN[2][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_45   CE_XRDC.MRGDN[2][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_45   CE_XRDC.MRGDN[2][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_45   CE_XRDC.MRGDN[2][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_46   CE_XRDC.MRGDN[2][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_46   CE_XRDC.MRGDN[2][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_46   CE_XRDC.MRGDN[2][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_46   CE_XRDC.MRGDN[2][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define CE_XRDC_MRGD_W0_47   CE_XRDC.MRGDN[2][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define CE_XRDC_MRGD_W1_47   CE_XRDC.MRGDN[2][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define CE_XRDC_MRGD_W2_47   CE_XRDC.MRGDN[2][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define CE_XRDC_MRGD_W3_47   CE_XRDC.MRGDN[2][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* CLASS_0X9_CORESIGHT_COMPONENT */
#define CEVA_SPF2_APB__CLASS_0X9_CORESIGHT_COMPONENT_DTIR CEVA_SPF2_APB__CLASS_0X9_CORESIGHT_COMPONENT.DTIR.R /* DTIR Register */

/* CMU_FC */
#define CE_CMU_FC_2_GCR      CE_CMU_FC_2.GCR.R             /* Global Configuration Register */
#define CE_CMU_FC_2_RCCR     CE_CMU_FC_2.RCCR.R            /* Reference Count Configuration Register */
#define CE_CMU_FC_2_HTCR     CE_CMU_FC_2.HTCR.R            /* High Threshold Configuration Register */
#define CE_CMU_FC_2_LTCR     CE_CMU_FC_2.LTCR.R            /* Low Threshold Configuration Register */
#define CE_CMU_FC_2_SR       CE_CMU_FC_2.SR.R              /* Status Register */
#define CE_CMU_FC_2_IER      CE_CMU_FC_2.IER.R             /* Interrupt Enable Register */

#define CE_CMU_FC_1_GCR      CE_CMU_FC_1.GCR.R             /* Global Configuration Register */
#define CE_CMU_FC_1_RCCR     CE_CMU_FC_1.RCCR.R            /* Reference Count Configuration Register */
#define CE_CMU_FC_1_HTCR     CE_CMU_FC_1.HTCR.R            /* High Threshold Configuration Register */
#define CE_CMU_FC_1_LTCR     CE_CMU_FC_1.LTCR.R            /* Low Threshold Configuration Register */
#define CE_CMU_FC_1_SR       CE_CMU_FC_1.SR.R              /* Status Register */
#define CE_CMU_FC_1_IER      CE_CMU_FC_1.IER.R             /* Interrupt Enable Register */

#define CE_CMU_FC_0_GCR      CE_CMU_FC_0.GCR.R             /* Global Configuration Register */
#define CE_CMU_FC_0_RCCR     CE_CMU_FC_0.RCCR.R            /* Reference Count Configuration Register */
#define CE_CMU_FC_0_HTCR     CE_CMU_FC_0.HTCR.R            /* High Threshold Configuration Register */
#define CE_CMU_FC_0_LTCR     CE_CMU_FC_0.LTCR.R            /* Low Threshold Configuration Register */
#define CE_CMU_FC_0_SR       CE_CMU_FC_0.SR.R              /* Status Register */
#define CE_CMU_FC_0_IER      CE_CMU_FC_0.IER.R             /* Interrupt Enable Register */

#define AES__CMU_FC_GCR      AES__CMU_FC.GCR.R             /* Global Configuration Register */
#define AES__CMU_FC_RCCR     AES__CMU_FC.RCCR.R            /* Reference Count Configuration Register */
#define AES__CMU_FC_HTCR     AES__CMU_FC.HTCR.R            /* High Threshold Configuration Register */
#define AES__CMU_FC_LTCR     AES__CMU_FC.LTCR.R            /* Low Threshold Configuration Register */
#define AES__CMU_FC_SR       AES__CMU_FC.SR.R              /* Status Register */
#define AES__CMU_FC_IER      AES__CMU_FC.IER.R             /* Interrupt Enable Register */

#define CMU_FC_3_GCR         CMU_FC_3.GCR.R                /* Global Configuration Register */
#define CMU_FC_3_RCCR        CMU_FC_3.RCCR.R               /* Reference Count Configuration Register */
#define CMU_FC_3_HTCR        CMU_FC_3.HTCR.R               /* High Threshold Configuration Register */
#define CMU_FC_3_LTCR        CMU_FC_3.LTCR.R               /* Low Threshold Configuration Register */
#define CMU_FC_3_SR          CMU_FC_3.SR.R                 /* Status Register */
#define CMU_FC_3_IER         CMU_FC_3.IER.R                /* Interrupt Enable Register */

#define CMU_FC_6_GCR         CMU_FC_6.GCR.R                /* Global Configuration Register */
#define CMU_FC_6_RCCR        CMU_FC_6.RCCR.R               /* Reference Count Configuration Register */
#define CMU_FC_6_HTCR        CMU_FC_6.HTCR.R               /* High Threshold Configuration Register */
#define CMU_FC_6_LTCR        CMU_FC_6.LTCR.R               /* Low Threshold Configuration Register */
#define CMU_FC_6_SR          CMU_FC_6.SR.R                 /* Status Register */
#define CMU_FC_6_IER         CMU_FC_6.IER.R                /* Interrupt Enable Register */

#define CMU_FC_1_GCR         CMU_FC_1.GCR.R                /* Global Configuration Register */
#define CMU_FC_1_RCCR        CMU_FC_1.RCCR.R               /* Reference Count Configuration Register */
#define CMU_FC_1_HTCR        CMU_FC_1.HTCR.R               /* High Threshold Configuration Register */
#define CMU_FC_1_LTCR        CMU_FC_1.LTCR.R               /* Low Threshold Configuration Register */
#define CMU_FC_1_SR          CMU_FC_1.SR.R                 /* Status Register */
#define CMU_FC_1_IER         CMU_FC_1.IER.R                /* Interrupt Enable Register */

#define CMU_FC_DEBUG_2_GCR   CMU_FC_DEBUG_2.GCR.R          /* Global Configuration Register */
#define CMU_FC_DEBUG_2_RCCR  CMU_FC_DEBUG_2.RCCR.R         /* Reference Count Configuration Register */
#define CMU_FC_DEBUG_2_HTCR  CMU_FC_DEBUG_2.HTCR.R         /* High Threshold Configuration Register */
#define CMU_FC_DEBUG_2_LTCR  CMU_FC_DEBUG_2.LTCR.R         /* Low Threshold Configuration Register */
#define CMU_FC_DEBUG_2_SR    CMU_FC_DEBUG_2.SR.R           /* Status Register */
#define CMU_FC_DEBUG_2_IER   CMU_FC_DEBUG_2.IER.R          /* Interrupt Enable Register */

#define SMU__CMU_FC_GCR      SMU__CMU_FC.GCR.R             /* Global Configuration Register */
#define SMU__CMU_FC_RCCR     SMU__CMU_FC.RCCR.R            /* Reference Count Configuration Register */
#define SMU__CMU_FC_HTCR     SMU__CMU_FC.HTCR.R            /* High Threshold Configuration Register */
#define SMU__CMU_FC_LTCR     SMU__CMU_FC.LTCR.R            /* Low Threshold Configuration Register */
#define SMU__CMU_FC_SR       SMU__CMU_FC.SR.R              /* Status Register */
#define SMU__CMU_FC_IER      SMU__CMU_FC.IER.R             /* Interrupt Enable Register */

#define CMU_FC_2D_GCR        CMU_FC_2D.GCR.R               /* Global Configuration Register */
#define CMU_FC_2D_RCCR       CMU_FC_2D.RCCR.R              /* Reference Count Configuration Register */
#define CMU_FC_2D_HTCR       CMU_FC_2D.HTCR.R              /* High Threshold Configuration Register */
#define CMU_FC_2D_LTCR       CMU_FC_2D.LTCR.R              /* Low Threshold Configuration Register */
#define CMU_FC_2D_SR         CMU_FC_2D.SR.R                /* Status Register */
#define CMU_FC_2D_IER        CMU_FC_2D.IER.R               /* Interrupt Enable Register */

#define CMU_FC_2C_GCR        CMU_FC_2C.GCR.R               /* Global Configuration Register */
#define CMU_FC_2C_RCCR       CMU_FC_2C.RCCR.R              /* Reference Count Configuration Register */
#define CMU_FC_2C_HTCR       CMU_FC_2C.HTCR.R              /* High Threshold Configuration Register */
#define CMU_FC_2C_LTCR       CMU_FC_2C.LTCR.R              /* Low Threshold Configuration Register */
#define CMU_FC_2C_SR         CMU_FC_2C.SR.R                /* Status Register */
#define CMU_FC_2C_IER        CMU_FC_2C.IER.R               /* Interrupt Enable Register */

#define CMU_FC_2A_GCR        CMU_FC_2A.GCR.R               /* Global Configuration Register */
#define CMU_FC_2A_RCCR       CMU_FC_2A.RCCR.R              /* Reference Count Configuration Register */
#define CMU_FC_2A_HTCR       CMU_FC_2A.HTCR.R              /* High Threshold Configuration Register */
#define CMU_FC_2A_LTCR       CMU_FC_2A.LTCR.R              /* Low Threshold Configuration Register */
#define CMU_FC_2A_SR         CMU_FC_2A.SR.R                /* Status Register */
#define CMU_FC_2A_IER        CMU_FC_2A.IER.R               /* Interrupt Enable Register */

#define CMU_FC_2B_GCR        CMU_FC_2B.GCR.R               /* Global Configuration Register */
#define CMU_FC_2B_RCCR       CMU_FC_2B.RCCR.R              /* Reference Count Configuration Register */
#define CMU_FC_2B_HTCR       CMU_FC_2B.HTCR.R              /* High Threshold Configuration Register */
#define CMU_FC_2B_LTCR       CMU_FC_2B.LTCR.R              /* Low Threshold Configuration Register */
#define CMU_FC_2B_SR         CMU_FC_2B.SR.R                /* Status Register */
#define CMU_FC_2B_IER        CMU_FC_2B.IER.R               /* Interrupt Enable Register */

#define CMU_FC_0_GCR         CMU_FC_0.GCR.R                /* Global Configuration Register */
#define CMU_FC_0_RCCR        CMU_FC_0.RCCR.R               /* Reference Count Configuration Register */
#define CMU_FC_0_HTCR        CMU_FC_0.HTCR.R               /* High Threshold Configuration Register */
#define CMU_FC_0_LTCR        CMU_FC_0.LTCR.R               /* Low Threshold Configuration Register */
#define CMU_FC_0_SR          CMU_FC_0.SR.R                 /* Status Register */
#define CMU_FC_0_IER         CMU_FC_0.IER.R                /* Interrupt Enable Register */

#define CMU_FC_DEBUG_1_GCR   CMU_FC_DEBUG_1.GCR.R          /* Global Configuration Register */
#define CMU_FC_DEBUG_1_RCCR  CMU_FC_DEBUG_1.RCCR.R         /* Reference Count Configuration Register */
#define CMU_FC_DEBUG_1_HTCR  CMU_FC_DEBUG_1.HTCR.R         /* High Threshold Configuration Register */
#define CMU_FC_DEBUG_1_LTCR  CMU_FC_DEBUG_1.LTCR.R         /* Low Threshold Configuration Register */
#define CMU_FC_DEBUG_1_SR    CMU_FC_DEBUG_1.SR.R           /* Status Register */
#define CMU_FC_DEBUG_1_IER   CMU_FC_DEBUG_1.IER.R          /* Interrupt Enable Register */

#define CMU_FC_4_GCR         CMU_FC_4.GCR.R                /* Global Configuration Register */
#define CMU_FC_4_RCCR        CMU_FC_4.RCCR.R               /* Reference Count Configuration Register */
#define CMU_FC_4_HTCR        CMU_FC_4.HTCR.R               /* High Threshold Configuration Register */
#define CMU_FC_4_LTCR        CMU_FC_4.LTCR.R               /* Low Threshold Configuration Register */
#define CMU_FC_4_SR          CMU_FC_4.SR.R                 /* Status Register */
#define CMU_FC_4_IER         CMU_FC_4.IER.R                /* Interrupt Enable Register */

#define RTU0__CMU_FC_0_GCR   RTU0__CMU_FC_0.GCR.R          /* Global Configuration Register */
#define RTU0__CMU_FC_0_RCCR  RTU0__CMU_FC_0.RCCR.R         /* Reference Count Configuration Register */
#define RTU0__CMU_FC_0_HTCR  RTU0__CMU_FC_0.HTCR.R         /* High Threshold Configuration Register */
#define RTU0__CMU_FC_0_LTCR  RTU0__CMU_FC_0.LTCR.R         /* Low Threshold Configuration Register */
#define RTU0__CMU_FC_0_SR    RTU0__CMU_FC_0.SR.R           /* Status Register */
#define RTU0__CMU_FC_0_IER   RTU0__CMU_FC_0.IER.R          /* Interrupt Enable Register */

#define RTU1__CMU_FC_0_GCR   RTU1__CMU_FC_0.GCR.R          /* Global Configuration Register */
#define RTU1__CMU_FC_0_RCCR  RTU1__CMU_FC_0.RCCR.R         /* Reference Count Configuration Register */
#define RTU1__CMU_FC_0_HTCR  RTU1__CMU_FC_0.HTCR.R         /* High Threshold Configuration Register */
#define RTU1__CMU_FC_0_LTCR  RTU1__CMU_FC_0.LTCR.R         /* Low Threshold Configuration Register */
#define RTU1__CMU_FC_0_SR    RTU1__CMU_FC_0.SR.R           /* Status Register */
#define RTU1__CMU_FC_0_IER   RTU1__CMU_FC_0.IER.R          /* Interrupt Enable Register */

#define RTU0__CMU_FC_3_GCR   RTU0__CMU_FC_3.GCR.R          /* Global Configuration Register */
#define RTU0__CMU_FC_3_RCCR  RTU0__CMU_FC_3.RCCR.R         /* Reference Count Configuration Register */
#define RTU0__CMU_FC_3_HTCR  RTU0__CMU_FC_3.HTCR.R         /* High Threshold Configuration Register */
#define RTU0__CMU_FC_3_LTCR  RTU0__CMU_FC_3.LTCR.R         /* Low Threshold Configuration Register */
#define RTU0__CMU_FC_3_SR    RTU0__CMU_FC_3.SR.R           /* Status Register */
#define RTU0__CMU_FC_3_IER   RTU0__CMU_FC_3.IER.R          /* Interrupt Enable Register */

#define RTU1__CMU_FC_3_GCR   RTU1__CMU_FC_3.GCR.R          /* Global Configuration Register */
#define RTU1__CMU_FC_3_RCCR  RTU1__CMU_FC_3.RCCR.R         /* Reference Count Configuration Register */
#define RTU1__CMU_FC_3_HTCR  RTU1__CMU_FC_3.HTCR.R         /* High Threshold Configuration Register */
#define RTU1__CMU_FC_3_LTCR  RTU1__CMU_FC_3.LTCR.R         /* Low Threshold Configuration Register */
#define RTU1__CMU_FC_3_SR    RTU1__CMU_FC_3.SR.R           /* Status Register */
#define RTU1__CMU_FC_3_IER   RTU1__CMU_FC_3.IER.R          /* Interrupt Enable Register */

#define RTU0__CMU_FC_4_GCR   RTU0__CMU_FC_4.GCR.R          /* Global Configuration Register */
#define RTU0__CMU_FC_4_RCCR  RTU0__CMU_FC_4.RCCR.R         /* Reference Count Configuration Register */
#define RTU0__CMU_FC_4_HTCR  RTU0__CMU_FC_4.HTCR.R         /* High Threshold Configuration Register */
#define RTU0__CMU_FC_4_LTCR  RTU0__CMU_FC_4.LTCR.R         /* Low Threshold Configuration Register */
#define RTU0__CMU_FC_4_SR    RTU0__CMU_FC_4.SR.R           /* Status Register */
#define RTU0__CMU_FC_4_IER   RTU0__CMU_FC_4.IER.R          /* Interrupt Enable Register */

#define RTU1__CMU_FC_4_GCR   RTU1__CMU_FC_4.GCR.R          /* Global Configuration Register */
#define RTU1__CMU_FC_4_RCCR  RTU1__CMU_FC_4.RCCR.R         /* Reference Count Configuration Register */
#define RTU1__CMU_FC_4_HTCR  RTU1__CMU_FC_4.HTCR.R         /* High Threshold Configuration Register */
#define RTU1__CMU_FC_4_LTCR  RTU1__CMU_FC_4.LTCR.R         /* Low Threshold Configuration Register */
#define RTU1__CMU_FC_4_SR    RTU1__CMU_FC_4.SR.R           /* Status Register */
#define RTU1__CMU_FC_4_IER   RTU1__CMU_FC_4.IER.R          /* Interrupt Enable Register */

#define RTU0__CMU_FC_1_GCR   RTU0__CMU_FC_1.GCR.R          /* Global Configuration Register */
#define RTU0__CMU_FC_1_RCCR  RTU0__CMU_FC_1.RCCR.R         /* Reference Count Configuration Register */
#define RTU0__CMU_FC_1_HTCR  RTU0__CMU_FC_1.HTCR.R         /* High Threshold Configuration Register */
#define RTU0__CMU_FC_1_LTCR  RTU0__CMU_FC_1.LTCR.R         /* Low Threshold Configuration Register */
#define RTU0__CMU_FC_1_SR    RTU0__CMU_FC_1.SR.R           /* Status Register */
#define RTU0__CMU_FC_1_IER   RTU0__CMU_FC_1.IER.R          /* Interrupt Enable Register */

#define RTU1__CMU_FC_1_GCR   RTU1__CMU_FC_1.GCR.R          /* Global Configuration Register */
#define RTU1__CMU_FC_1_RCCR  RTU1__CMU_FC_1.RCCR.R         /* Reference Count Configuration Register */
#define RTU1__CMU_FC_1_HTCR  RTU1__CMU_FC_1.HTCR.R         /* High Threshold Configuration Register */
#define RTU1__CMU_FC_1_LTCR  RTU1__CMU_FC_1.LTCR.R         /* Low Threshold Configuration Register */
#define RTU1__CMU_FC_1_SR    RTU1__CMU_FC_1.SR.R           /* Status Register */
#define RTU1__CMU_FC_1_IER   RTU1__CMU_FC_1.IER.R          /* Interrupt Enable Register */

#define RTU0__CMU_FC_2_GCR   RTU0__CMU_FC_2.GCR.R          /* Global Configuration Register */
#define RTU0__CMU_FC_2_RCCR  RTU0__CMU_FC_2.RCCR.R         /* Reference Count Configuration Register */
#define RTU0__CMU_FC_2_HTCR  RTU0__CMU_FC_2.HTCR.R         /* High Threshold Configuration Register */
#define RTU0__CMU_FC_2_LTCR  RTU0__CMU_FC_2.LTCR.R         /* Low Threshold Configuration Register */
#define RTU0__CMU_FC_2_SR    RTU0__CMU_FC_2.SR.R           /* Status Register */
#define RTU0__CMU_FC_2_IER   RTU0__CMU_FC_2.IER.R          /* Interrupt Enable Register */

#define RTU1__CMU_FC_2_GCR   RTU1__CMU_FC_2.GCR.R          /* Global Configuration Register */
#define RTU1__CMU_FC_2_RCCR  RTU1__CMU_FC_2.RCCR.R         /* Reference Count Configuration Register */
#define RTU1__CMU_FC_2_HTCR  RTU1__CMU_FC_2.HTCR.R         /* High Threshold Configuration Register */
#define RTU1__CMU_FC_2_LTCR  RTU1__CMU_FC_2.LTCR.R         /* Low Threshold Configuration Register */
#define RTU1__CMU_FC_2_SR    RTU1__CMU_FC_2.SR.R           /* Status Register */
#define RTU1__CMU_FC_2_IER   RTU1__CMU_FC_2.IER.R          /* Interrupt Enable Register */

#define CMU_FC_5_GCR         CMU_FC_5.GCR.R                /* Global Configuration Register */
#define CMU_FC_5_RCCR        CMU_FC_5.RCCR.R               /* Reference Count Configuration Register */
#define CMU_FC_5_HTCR        CMU_FC_5.HTCR.R               /* High Threshold Configuration Register */
#define CMU_FC_5_LTCR        CMU_FC_5.LTCR.R               /* Low Threshold Configuration Register */
#define CMU_FC_5_SR          CMU_FC_5.SR.R                 /* Status Register */
#define CMU_FC_5_IER         CMU_FC_5.IER.R                /* Interrupt Enable Register */

/* CORESIGHT_OCEM */
#define CEVA_SPF2_APB__CORESIGHT_OCEM_SCCO CEVA_SPF2_APB__CORESIGHT_OCEM.SCCO.R /* SCCO Register */
#define CEVA_SPF2_APB__CORESIGHT_OCEM_SCDA CEVA_SPF2_APB__CORESIGHT_OCEM.SCDA.R /* SCDA Register */

/* CORESIGHT_PROGRAMMING_MODEL */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_P_ID4 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.P_ID4.R /* PMSS ID4 Register */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_P_ID0 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.P_ID0.R /* PMSS ID0 Register */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_P_ID1 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.P_ID1.R /* PMSS ID1 Register */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_P_ID2 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.P_ID2.R /* PMSS ID2 Register */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_P_ID3 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.P_ID3.R /* PMSS ID3 Register */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_C_ID0 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.C_ID0.R /* Core ID0 Register */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_C_ID1 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.C_ID1.R /* Core ID1 Register */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_C_ID2 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.C_ID2.R /* Core ID2 Register */
#define CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL_C_ID3 CEVA_SPF2_APB__CORESIGHT_PROGRAMMING_MODEL.C_ID3.R /* Core ID3 Register */

/* CORE_DEBUGGER_INTERFACE */
#define CEVA_SPF2__CORE_DEBUGGER_INTERFACE_core_drd3_rd CEVA_SPF2__CORE_DEBUGGER_INTERFACE.CORE_DRD3_RD.R /* Core DRD3 RD Register */
#define CEVA_SPF2__CORE_DEBUGGER_INTERFACE_core_drd2_rd CEVA_SPF2__CORE_DEBUGGER_INTERFACE.CORE_DRD2_RD.R /* Core DRD2 RD Register */
#define CEVA_SPF2__CORE_DEBUGGER_INTERFACE_core_drd1_rd CEVA_SPF2__CORE_DEBUGGER_INTERFACE.CORE_DRD1_RD.R /* Core DRD1 RD Register */
#define CEVA_SPF2__CORE_DEBUGGER_INTERFACE_core_drd_rd CEVA_SPF2__CORE_DEBUGGER_INTERFACE.CORE_DRD_RD.R /* Core DRD RD Register */

/* CORE_PLL */
#define CORE_PLL_PLLCR       CORE_PLL.PLLCR.R              /* PLL Control */
#define CORE_PLL_PLLSR       CORE_PLL.PLLSR.R              /* PLL Status */
#define CORE_PLL_PLLDV       CORE_PLL.PLLDV.R              /* PLL Divider */
#define CORE_PLL_PLLFM       CORE_PLL.PLLFM.R              /* PLL Frequency Modulation */
#define CORE_PLL_PLLFD       CORE_PLL.PLLFD.R              /* PLL Fractional Divider */
#define CORE_PLL_PLLCLKMUX   CORE_PLL.PLLCLKMUX.R          /* PLL Clock Multiplexer */
#define CORE_PLL_PLLODIV_0   CORE_PLL.PLLODIV[0].R         /* PLL Output Divider */

/* CORE_SAFETY */
#define CEVA_SPF2__CORE_SAFETY_C_UCPERR CEVA_SPF2__CORE_SAFETY.C_UCPERR.R /* Core UCP ERR Register */
#define CEVA_SPF2__CORE_SAFETY_C_UCPERR_S CEVA_SPF2__CORE_SAFETY.C_UCPERR_S.R /* Core UCP ERR Shadow Register */

/* CRC */
#define CE_CRC_0_CFG1        CE_CRC_0.CHANNEL[0].CFG.R     /* Configuration Register */
#define CE_CRC_0_INP1        CE_CRC_0.CHANNEL[0].INP.R     /* Input Register */
#define CE_CRC_0_CSTAT1      CE_CRC_0.CHANNEL[0].CSTAT.R   /* Current Status Register */
#define CE_CRC_0_OUTP1       CE_CRC_0.CHANNEL[0].OUTP.R    /* Output Register */
#define CE_CRC_0_CFG2        CE_CRC_0.CHANNEL[1].CFG.R     /* Configuration Register */
#define CE_CRC_0_INP2        CE_CRC_0.CHANNEL[1].INP.R     /* Input Register */
#define CE_CRC_0_CSTAT2      CE_CRC_0.CHANNEL[1].CSTAT.R   /* Current Status Register */
#define CE_CRC_0_OUTP2       CE_CRC_0.CHANNEL[1].OUTP.R    /* Output Register */
#define CE_CRC_0_CFG3        CE_CRC_0.CHANNEL[2].CFG.R     /* Configuration Register */
#define CE_CRC_0_INP3        CE_CRC_0.CHANNEL[2].INP.R     /* Input Register */
#define CE_CRC_0_CSTAT3      CE_CRC_0.CHANNEL[2].CSTAT.R   /* Current Status Register */
#define CE_CRC_0_OUTP3       CE_CRC_0.CHANNEL[2].OUTP.R    /* Output Register */

#define CE_CRC_1_CFG1        CE_CRC_1.CHANNEL[0].CFG.R     /* Configuration Register */
#define CE_CRC_1_INP1        CE_CRC_1.CHANNEL[0].INP.R     /* Input Register */
#define CE_CRC_1_CSTAT1      CE_CRC_1.CHANNEL[0].CSTAT.R   /* Current Status Register */
#define CE_CRC_1_OUTP1       CE_CRC_1.CHANNEL[0].OUTP.R    /* Output Register */
#define CE_CRC_1_CFG2        CE_CRC_1.CHANNEL[1].CFG.R     /* Configuration Register */
#define CE_CRC_1_INP2        CE_CRC_1.CHANNEL[1].INP.R     /* Input Register */
#define CE_CRC_1_CSTAT2      CE_CRC_1.CHANNEL[1].CSTAT.R   /* Current Status Register */
#define CE_CRC_1_OUTP2       CE_CRC_1.CHANNEL[1].OUTP.R    /* Output Register */
#define CE_CRC_1_CFG3        CE_CRC_1.CHANNEL[2].CFG.R     /* Configuration Register */
#define CE_CRC_1_INP3        CE_CRC_1.CHANNEL[2].INP.R     /* Input Register */
#define CE_CRC_1_CSTAT3      CE_CRC_1.CHANNEL[2].CSTAT.R   /* Current Status Register */
#define CE_CRC_1_OUTP3       CE_CRC_1.CHANNEL[2].OUTP.R    /* Output Register */

#define CE_CRC_2_CFG1        CE_CRC_2.CHANNEL[0].CFG.R     /* Configuration Register */
#define CE_CRC_2_INP1        CE_CRC_2.CHANNEL[0].INP.R     /* Input Register */
#define CE_CRC_2_CSTAT1      CE_CRC_2.CHANNEL[0].CSTAT.R   /* Current Status Register */
#define CE_CRC_2_OUTP1       CE_CRC_2.CHANNEL[0].OUTP.R    /* Output Register */
#define CE_CRC_2_CFG2        CE_CRC_2.CHANNEL[1].CFG.R     /* Configuration Register */
#define CE_CRC_2_INP2        CE_CRC_2.CHANNEL[1].INP.R     /* Input Register */
#define CE_CRC_2_CSTAT2      CE_CRC_2.CHANNEL[1].CSTAT.R   /* Current Status Register */
#define CE_CRC_2_OUTP2       CE_CRC_2.CHANNEL[1].OUTP.R    /* Output Register */
#define CE_CRC_2_CFG3        CE_CRC_2.CHANNEL[2].CFG.R     /* Configuration Register */
#define CE_CRC_2_INP3        CE_CRC_2.CHANNEL[2].INP.R     /* Input Register */
#define CE_CRC_2_CSTAT3      CE_CRC_2.CHANNEL[2].CSTAT.R   /* Current Status Register */
#define CE_CRC_2_OUTP3       CE_CRC_2.CHANNEL[2].OUTP.R    /* Output Register */

#define CE_CRC_3_CFG1        CE_CRC_3.CHANNEL[0].CFG.R     /* Configuration Register */
#define CE_CRC_3_INP1        CE_CRC_3.CHANNEL[0].INP.R     /* Input Register */
#define CE_CRC_3_CSTAT1      CE_CRC_3.CHANNEL[0].CSTAT.R   /* Current Status Register */
#define CE_CRC_3_OUTP1       CE_CRC_3.CHANNEL[0].OUTP.R    /* Output Register */
#define CE_CRC_3_CFG2        CE_CRC_3.CHANNEL[1].CFG.R     /* Configuration Register */
#define CE_CRC_3_INP2        CE_CRC_3.CHANNEL[1].INP.R     /* Input Register */
#define CE_CRC_3_CSTAT2      CE_CRC_3.CHANNEL[1].CSTAT.R   /* Current Status Register */
#define CE_CRC_3_OUTP2       CE_CRC_3.CHANNEL[1].OUTP.R    /* Output Register */
#define CE_CRC_3_CFG3        CE_CRC_3.CHANNEL[2].CFG.R     /* Configuration Register */
#define CE_CRC_3_INP3        CE_CRC_3.CHANNEL[2].INP.R     /* Input Register */
#define CE_CRC_3_CSTAT3      CE_CRC_3.CHANNEL[2].CSTAT.R   /* Current Status Register */
#define CE_CRC_3_OUTP3       CE_CRC_3.CHANNEL[2].OUTP.R    /* Output Register */

#define CE_CRC_4_CFG1        CE_CRC_4.CHANNEL[0].CFG.R     /* Configuration Register */
#define CE_CRC_4_INP1        CE_CRC_4.CHANNEL[0].INP.R     /* Input Register */
#define CE_CRC_4_CSTAT1      CE_CRC_4.CHANNEL[0].CSTAT.R   /* Current Status Register */
#define CE_CRC_4_OUTP1       CE_CRC_4.CHANNEL[0].OUTP.R    /* Output Register */
#define CE_CRC_4_CFG2        CE_CRC_4.CHANNEL[1].CFG.R     /* Configuration Register */
#define CE_CRC_4_INP2        CE_CRC_4.CHANNEL[1].INP.R     /* Input Register */
#define CE_CRC_4_CSTAT2      CE_CRC_4.CHANNEL[1].CSTAT.R   /* Current Status Register */
#define CE_CRC_4_OUTP2       CE_CRC_4.CHANNEL[1].OUTP.R    /* Output Register */
#define CE_CRC_4_CFG3        CE_CRC_4.CHANNEL[2].CFG.R     /* Configuration Register */
#define CE_CRC_4_INP3        CE_CRC_4.CHANNEL[2].INP.R     /* Input Register */
#define CE_CRC_4_CSTAT3      CE_CRC_4.CHANNEL[2].CSTAT.R   /* Current Status Register */
#define CE_CRC_4_OUTP3       CE_CRC_4.CHANNEL[2].OUTP.R    /* Output Register */

#define CE_CRC_5_CFG1        CE_CRC_5.CHANNEL[0].CFG.R     /* Configuration Register */
#define CE_CRC_5_INP1        CE_CRC_5.CHANNEL[0].INP.R     /* Input Register */
#define CE_CRC_5_CSTAT1      CE_CRC_5.CHANNEL[0].CSTAT.R   /* Current Status Register */
#define CE_CRC_5_OUTP1       CE_CRC_5.CHANNEL[0].OUTP.R    /* Output Register */
#define CE_CRC_5_CFG2        CE_CRC_5.CHANNEL[1].CFG.R     /* Configuration Register */
#define CE_CRC_5_INP2        CE_CRC_5.CHANNEL[1].INP.R     /* Input Register */
#define CE_CRC_5_CSTAT2      CE_CRC_5.CHANNEL[1].CSTAT.R   /* Current Status Register */
#define CE_CRC_5_OUTP2       CE_CRC_5.CHANNEL[1].OUTP.R    /* Output Register */
#define CE_CRC_5_CFG3        CE_CRC_5.CHANNEL[2].CFG.R     /* Configuration Register */
#define CE_CRC_5_INP3        CE_CRC_5.CHANNEL[2].INP.R     /* Input Register */
#define CE_CRC_5_CSTAT3      CE_CRC_5.CHANNEL[2].CSTAT.R   /* Current Status Register */
#define CE_CRC_5_OUTP3       CE_CRC_5.CHANNEL[2].OUTP.R    /* Output Register */

#define RTU0__CRC_CFG1       RTU0__CRC.CHANNEL[0].CFG.R    /* Configuration Register */
#define RTU0__CRC_INP1       RTU0__CRC.CHANNEL[0].INP.R    /* Input Register */
#define RTU0__CRC_CSTAT1     RTU0__CRC.CHANNEL[0].CSTAT.R  /* Current Status Register */
#define RTU0__CRC_OUTP1      RTU0__CRC.CHANNEL[0].OUTP.R   /* Output Register */
#define RTU0__CRC_CFG2       RTU0__CRC.CHANNEL[1].CFG.R    /* Configuration Register */
#define RTU0__CRC_INP2       RTU0__CRC.CHANNEL[1].INP.R    /* Input Register */
#define RTU0__CRC_CSTAT2     RTU0__CRC.CHANNEL[1].CSTAT.R  /* Current Status Register */
#define RTU0__CRC_OUTP2      RTU0__CRC.CHANNEL[1].OUTP.R   /* Output Register */
#define RTU0__CRC_CFG3       RTU0__CRC.CHANNEL[2].CFG.R    /* Configuration Register */
#define RTU0__CRC_INP3       RTU0__CRC.CHANNEL[2].INP.R    /* Input Register */
#define RTU0__CRC_CSTAT3     RTU0__CRC.CHANNEL[2].CSTAT.R  /* Current Status Register */
#define RTU0__CRC_OUTP3      RTU0__CRC.CHANNEL[2].OUTP.R   /* Output Register */

#define RTU1__CRC_CFG1       RTU1__CRC.CHANNEL[0].CFG.R    /* Configuration Register */
#define RTU1__CRC_INP1       RTU1__CRC.CHANNEL[0].INP.R    /* Input Register */
#define RTU1__CRC_CSTAT1     RTU1__CRC.CHANNEL[0].CSTAT.R  /* Current Status Register */
#define RTU1__CRC_OUTP1      RTU1__CRC.CHANNEL[0].OUTP.R   /* Output Register */
#define RTU1__CRC_CFG2       RTU1__CRC.CHANNEL[1].CFG.R    /* Configuration Register */
#define RTU1__CRC_INP2       RTU1__CRC.CHANNEL[1].INP.R    /* Input Register */
#define RTU1__CRC_CSTAT2     RTU1__CRC.CHANNEL[1].CSTAT.R  /* Current Status Register */
#define RTU1__CRC_OUTP2      RTU1__CRC.CHANNEL[1].OUTP.R   /* Output Register */
#define RTU1__CRC_CFG3       RTU1__CRC.CHANNEL[2].CFG.R    /* Configuration Register */
#define RTU1__CRC_INP3       RTU1__CRC.CHANNEL[2].INP.R    /* Input Register */
#define RTU1__CRC_CSTAT3     RTU1__CRC.CHANNEL[2].CSTAT.R  /* Current Status Register */
#define RTU1__CRC_OUTP3      RTU1__CRC.CHANNEL[2].OUTP.R   /* Output Register */

/* CSTCU */
#define CSTCU_RUNSWREG       CSTCU.RUNSWREG.R              /* Run Software */
#define CSTCU_LSCHLVLD       CSTCU.LSCHLVLD.R              /* LSTCU Scheduler List Valid */
#define CSTCU_WDG            CSTCU.WDG.R                   /* Watchdog Timer */
#define CSTCU_IE             CSTCU.IE.R                    /* Interrupt Enable */
#define CSTCU_IF             CSTCU.IF.R                    /* Interrupt Flag */
#define CSTCU_ERR_STAT       CSTCU.ERR_STAT.R              /* Error Status */
#define CSTCU_ERR_FM         CSTCU.ERR_FM.R                /* Error Fault Mapping */
#define CSTCU_LRFSTAT0       CSTCU.LRFSTAT[0].R            /* LSTCU Recoverable Fault Status */
#define CSTCU_LUFSTAT0       CSTCU.LUFSTAT[0].R            /* LSTCU Unrecoverable Fault Status */
#define CSTCU_RDEN           CSTCU.RDEN.R                  /* Reset Domain Self-Test Enable */
#define CSTCU_RDENSTAT       CSTCU.RDENSTAT.R              /* Reset Domain Enable Status */
#define CSTCU_LASTRDEN       CSTCU.LASTRDEN.R              /* Last Run Reset Domain Enable */
#define CSTCU_BYPLSTCU0      CSTCU.BYPLSTCU[0].R           /* Bypass LSTCU */
#define CSTCU_STAG           CSTCU.STAG.R                  /* Stagger */
#define CSTCU_LMBPTR0        CSTCU.LMBPTR[0].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR1        CSTCU.LMBPTR[1].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR2        CSTCU.LMBPTR[2].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR3        CSTCU.LMBPTR[3].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR4        CSTCU.LMBPTR[4].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR5        CSTCU.LMBPTR[5].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR6        CSTCU.LMBPTR[6].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR7        CSTCU.LMBPTR[7].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR8        CSTCU.LMBPTR[8].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR9        CSTCU.LMBPTR[9].R             /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR10       CSTCU.LMBPTR[10].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR11       CSTCU.LMBPTR[11].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR12       CSTCU.LMBPTR[12].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR13       CSTCU.LMBPTR[13].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR14       CSTCU.LMBPTR[14].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR15       CSTCU.LMBPTR[15].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR16       CSTCU.LMBPTR[16].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR17       CSTCU.LMBPTR[17].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR18       CSTCU.LMBPTR[18].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LMBPTR19       CSTCU.LMBPTR[19].R            /* LSTCU MBIST Run Phase Scheduler Pointer */
#define CSTCU_LLBPTR0        CSTCU.LLBPTR[0].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR1        CSTCU.LLBPTR[1].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR2        CSTCU.LLBPTR[2].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR3        CSTCU.LLBPTR[3].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR4        CSTCU.LLBPTR[4].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR5        CSTCU.LLBPTR[5].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR6        CSTCU.LLBPTR[6].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR7        CSTCU.LLBPTR[7].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR8        CSTCU.LLBPTR[8].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR9        CSTCU.LLBPTR[9].R             /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR10       CSTCU.LLBPTR[10].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR11       CSTCU.LLBPTR[11].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR12       CSTCU.LLBPTR[12].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR13       CSTCU.LLBPTR[13].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR14       CSTCU.LLBPTR[14].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR15       CSTCU.LLBPTR[15].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR16       CSTCU.LLBPTR[16].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR17       CSTCU.LLBPTR[17].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR18       CSTCU.LLBPTR[18].R            /* LSTCU LBIST Run phase Scheduler Pointer */
#define CSTCU_LLBPTR19       CSTCU.LLBPTR[19].R            /* LSTCU LBIST Run phase Scheduler Pointer */

/* CTU */
#define CTU_TGSISR           CTU.TGSISR.R                  /* Trigger Generator Subunit Input Selection Register */
#define CTU_TGSCR            CTU.TGSCR.R                   /* Trigger Generator Subunit Control Register */
#define CTU_T0CR             CTU.TCR[0].R                  /* Trigger Compare Register */
#define CTU_T1CR             CTU.TCR[1].R                  /* Trigger Compare Register */
#define CTU_T2CR             CTU.TCR[2].R                  /* Trigger Compare Register */
#define CTU_T3CR             CTU.TCR[3].R                  /* Trigger Compare Register */
#define CTU_T4CR             CTU.TCR[4].R                  /* Trigger Compare Register */
#define CTU_T5CR             CTU.TCR[5].R                  /* Trigger Compare Register */
#define CTU_T6CR             CTU.TCR[6].R                  /* Trigger Compare Register */
#define CTU_T7CR             CTU.TCR[7].R                  /* Trigger Compare Register */
#define CTU_TGSCCR           CTU.TGSCCR.R                  /* TGS Counter Compare Register */
#define CTU_TGSCRR           CTU.TGSCRR.R                  /* TGS Counter Reload Register */
#define CTU_CLCR1            CTU.CLCR1.R                   /* Commands List Control Register 1 */
#define CTU_CLCR2            CTU.CLCR2.R                   /* Commands List Control Register 2 */
#define CTU_THCR1            CTU.THCR1.R                   /* Trigger Handler Control Register 1 */
#define CTU_THCR2            CTU.THCR2.R                   /* Trigger Handler Control Register 2 */
#define CTU_CLR_A_1          CTU.CLR.A[0].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_2          CTU.CLR.A[1].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_3          CTU.CLR.A[2].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_4          CTU.CLR.A[3].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_5          CTU.CLR.A[4].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_6          CTU.CLR.A[5].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_7          CTU.CLR.A[6].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_8          CTU.CLR.A[7].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_9          CTU.CLR.A[8].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_10         CTU.CLR.A[9].R                /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_11         CTU.CLR.A[10].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_12         CTU.CLR.A[11].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_13         CTU.CLR.A[12].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_14         CTU.CLR.A[13].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_15         CTU.CLR.A[14].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_16         CTU.CLR.A[15].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_17         CTU.CLR.A[16].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_18         CTU.CLR.A[17].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_19         CTU.CLR.A[18].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_20         CTU.CLR.A[19].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_21         CTU.CLR.A[20].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_22         CTU.CLR.A[21].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_23         CTU.CLR.A[22].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_A_24         CTU.CLR.A[23].R               /* Commands List Register A for ADC single-conversion mode commands */
#define CTU_CLR_B_1          CTU.CLR.B[0].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_2          CTU.CLR.B[1].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_3          CTU.CLR.B[2].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_4          CTU.CLR.B[3].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_5          CTU.CLR.B[4].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_6          CTU.CLR.B[5].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_7          CTU.CLR.B[6].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_8          CTU.CLR.B[7].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_9          CTU.CLR.B[8].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_10         CTU.CLR.B[9].R                /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_11         CTU.CLR.B[10].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_12         CTU.CLR.B[11].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_13         CTU.CLR.B[12].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_14         CTU.CLR.B[13].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_15         CTU.CLR.B[14].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_16         CTU.CLR.B[15].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_17         CTU.CLR.B[16].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_18         CTU.CLR.B[17].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_19         CTU.CLR.B[18].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_20         CTU.CLR.B[19].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_21         CTU.CLR.B[20].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_22         CTU.CLR.B[21].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_23         CTU.CLR.B[22].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_CLR_B_24         CTU.CLR.B[23].R               /* Command List Register B for ADC dual-conversion mode commands */
#define CTU_FDCR             CTU.FDCR.R                    /* FIFO DMA Control Register */
#define CTU_FCR              CTU.FCR.R                     /* FIFO Control Register */
#define CTU_FTH              CTU.FTH.R                     /* FIFO Threshold Register */
#define CTU_FST              CTU.FST.R                     /* FIFO Status Register */
#define CTU_FR0              CTU.FR[0].R                   /* FIFO Right Aligned Data Register */
#define CTU_FR1              CTU.FR[1].R                   /* FIFO Right Aligned Data Register */
#define CTU_FR2              CTU.FR[2].R                   /* FIFO Right Aligned Data Register */
#define CTU_FR3              CTU.FR[3].R                   /* FIFO Right Aligned Data Register */
#define CTU_FL0              CTU.FL[0].R                   /* FIFO Signed Left Aligned Data Register */
#define CTU_FL1              CTU.FL[1].R                   /* FIFO Signed Left Aligned Data Register */
#define CTU_FL2              CTU.FL[2].R                   /* FIFO Signed Left Aligned Data Register */
#define CTU_FL3              CTU.FL[3].R                   /* FIFO Signed Left Aligned Data Register */
#define CTU_EFR              CTU.EFR.R                     /* Error Flag Register */
#define CTU_IFR              CTU.IFR.R                     /* Interrupt Flag Register */
#define CTU_IR               CTU.IR.R                      /* Interrupt/DMA Register */
#define CTU_COTR             CTU.COTR.R                    /* Control ON Time Register */
#define CTU_CR               CTU.CR.R                      /* Control Register */
#define CTU_DFR              CTU.DFR.R                     /* Digital Filter Register */
#define CTU_EXPAR            CTU.EXPAR.R                   /* Expected Value A Register */
#define CTU_EXPBR            CTU.EXPBR.R                   /* Expected Value B Register */
#define CTU_CNTRNGR          CTU.CNTRNGR.R                 /* Counter Range Register */
#define CTU_LISTCSR          CTU.LISTCSR.R                 /* List Control/Status Register */

/* C_VFCCU */
#define C_VFCCU_GDFHID_C0    C_VFCCU.GDFHID_C0.R           /* Global DID-FHID Map */
#define C_VFCCU_GDFHID_C1    C_VFCCU.GDFHID_C1.R           /* Global DID-FHID Map */
#define C_VFCCU_GFLTPO_C0    C_VFCCU.GFLTPO_C0.R           /* Global Fault Polarity */
#define C_VFCCU_GFLTPO_C1    C_VFCCU.GFLTPO_C1.R           /* Global Fault Polarity */
#define C_VFCCU_GFLTPO_C2    C_VFCCU.GFLTPO_C2.R           /* Global Fault Polarity */
#define C_VFCCU_GFLTRC_C0    C_VFCCU.GFLTRC_C0.R           /* Global Fault Recovery */
#define C_VFCCU_GFLTRC_C1    C_VFCCU.GFLTRC_C1.R           /* Global Fault Recovery */
#define C_VFCCU_GFLTRC_C2    C_VFCCU.GFLTRC_C2.R           /* Global Fault Recovery */
#define C_VFCCU_GFLTOVDC0    C_VFCCU.GFLTOVDC0.R           /* Global Fault Overflow Detection */
#define C_VFCCU_GFLTOVDC1    C_VFCCU.GFLTOVDC1.R           /* Global Fault Overflow Detection */
#define C_VFCCU_GFLTOVDC2    C_VFCCU.GFLTOVDC2.R           /* Global Fault Overflow Detection */
#define C_VFCCU_GCTRL        C_VFCCU.GCTRL.R               /* Global Space Control */
#define C_VFCCU_GINTOVFS     C_VFCCU.GINTOVFS.R            /* Global DID FSM Status */
#define C_VFCCU_GEXTOVFS     C_VFCCU.GEXTOVFS.R            /* Global External Overflow Status */
#define C_VFCCU_GRKNTIMC     C_VFCCU.GRKNTIMC.R            /* Global Reaction Timer Period */
#define C_VFCCU_GRKNTIMS     C_VFCCU.GRKNTIMS.R            /* Global Reaction Timer Status */
#define C_VFCCU_GOVFRKC0     C_VFCCU.GOVFRKC0.R            /* Global Overflow Reaction */
#define C_VFCCU_GOVFRKC1     C_VFCCU.GOVFRKC1.R            /* Global Overflow Reaction */
#define C_VFCCU_GMEOUTDC     C_VFCCU.GMEOUTDC.R            /* Global Minimum EOUT Duration */
#define C_VFCCU_GEOUTTCT     C_VFCCU.GEOUTTCT.R            /* Global EOUT Timer Disable */
#define C_VFCCU_GEOUTPNC0    C_VFCCU.GLB_EOUT[0].GEOUTPNC.R /* Global EOUT Pin */
#define C_VFCCU_GEOUTPMC0    C_VFCCU.GLB_EOUT[0].GEOUTPMC.R /* Global EOUT Pin Map */
#define C_VFCCU_GEOUTMC0     C_VFCCU.GLB_EOUT[0].GEOUTMC.R /* Global EOUT Mode */
#define C_VFCCU_GEOUTTMS0    C_VFCCU.GLB_EOUT[0].GEOUTTMS.R /* Global EOUT Timer */
#define C_VFCCU_GEOUTDIC0    C_VFCCU.GLB_EOUT[0].GEOUTDIC.R /* Global EOUT DID */
#define C_VFCCU_GEOUTPNC1    C_VFCCU.GLB_EOUT[1].GEOUTPNC.R /* Global EOUT Pin */
#define C_VFCCU_GEOUTPMC1    C_VFCCU.GLB_EOUT[1].GEOUTPMC.R /* Global EOUT Pin Map */
#define C_VFCCU_GEOUTMC1     C_VFCCU.GLB_EOUT[1].GEOUTMC.R /* Global EOUT Mode */
#define C_VFCCU_GEOUTTMS1    C_VFCCU.GLB_EOUT[1].GEOUTTMS.R /* Global EOUT Timer */
#define C_VFCCU_GEOUTDIC1    C_VFCCU.GLB_EOUT[1].GEOUTDIC.R /* Global EOUT DID */
#define C_VFCCU_GEOUTPNC2    C_VFCCU.GLB_EOUT[2].GEOUTPNC.R /* Global EOUT Pin */
#define C_VFCCU_GEOUTPMC2    C_VFCCU.GLB_EOUT[2].GEOUTPMC.R /* Global EOUT Pin Map */
#define C_VFCCU_GEOUTMC2     C_VFCCU.GLB_EOUT[2].GEOUTMC.R /* Global EOUT Mode */
#define C_VFCCU_GEOUTTMS2    C_VFCCU.GLB_EOUT[2].GEOUTTMS.R /* Global EOUT Timer */
#define C_VFCCU_GEOUTDIC2    C_VFCCU.GLB_EOUT[2].GEOUTDIC.R /* Global EOUT DID */
#define C_VFCCU_GEOUTPNC3    C_VFCCU.GLB_EOUT[3].GEOUTPNC.R /* Global EOUT Pin */
#define C_VFCCU_GEOUTPMC3    C_VFCCU.GLB_EOUT[3].GEOUTPMC.R /* Global EOUT Pin Map */
#define C_VFCCU_GEOUTMC3     C_VFCCU.GLB_EOUT[3].GEOUTMC.R /* Global EOUT Mode */
#define C_VFCCU_GEOUTTMS3    C_VFCCU.GLB_EOUT[3].GEOUTTMS.R /* Global EOUT Timer */
#define C_VFCCU_GEOUTDIC3    C_VFCCU.GLB_EOUT[3].GEOUTDIC.R /* Global EOUT DID */
#define C_VFCCU_GDBGCFG      C_VFCCU.GDBGCFG.R             /* Global Debug */
#define C_VFCCU_GDBGSTAT     C_VFCCU.GDBGSTAT.R            /* Global Debug Status */
#define C_VFCCU_FHCFG0       C_VFCCU.FHCFG0.R              /* Fault Handler */
#define C_VFCCU_FHSRVDS0     C_VFCCU.FHSRVDS0.R            /* Fault Handler Status */
#define C_VFCCU_FHFLTENC00   C_VFCCU.FHFLTENC00.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC01   C_VFCCU.FHFLTENC01.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC02   C_VFCCU.FHFLTENC02.R          /* Fault Enable */
#define C_VFCCU_FHFLTS00     C_VFCCU.FHFLTS00.R            /* Fault Status */
#define C_VFCCU_FHFLTS01     C_VFCCU.FHFLTS01.R            /* Fault Status */
#define C_VFCCU_FHFLTS02     C_VFCCU.FHFLTS02.R            /* Fault Status */
#define C_VFCCU_FHFLTRKC00   C_VFCCU.FHFLTRKC00.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC01   C_VFCCU.FHFLTRKC01.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC02   C_VFCCU.FHFLTRKC02.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC03   C_VFCCU.FHFLTRKC03.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC04   C_VFCCU.FHFLTRKC04.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC05   C_VFCCU.FHFLTRKC05.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC06   C_VFCCU.FHFLTRKC06.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC07   C_VFCCU.FHFLTRKC07.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC08   C_VFCCU.FHFLTRKC08.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHIMRKC0_00  C_VFCCU.FHIMRKC0_00.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC0_01  C_VFCCU.FHIMRKC0_01.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC0_00  C_VFCCU.FHDLRKC0_00.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC0_01  C_VFCCU.FHDLRKC0_01.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC0_10  C_VFCCU.FHIMRKC0_10.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC0_11  C_VFCCU.FHIMRKC0_11.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC0_10  C_VFCCU.FHDLRKC0_10.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC0_11  C_VFCCU.FHDLRKC0_11.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC0_20  C_VFCCU.FHIMRKC0_20.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC0_21  C_VFCCU.FHIMRKC0_21.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC0_20  C_VFCCU.FHDLRKC0_20.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC0_21  C_VFCCU.FHDLRKC0_21.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC0_30  C_VFCCU.FHIMRKC0_30.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC0_31  C_VFCCU.FHIMRKC0_31.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC0_30  C_VFCCU.FHDLRKC0_30.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC0_31  C_VFCCU.FHDLRKC0_31.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC0_40  C_VFCCU.FHIMRKC0_40.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC0_41  C_VFCCU.FHIMRKC0_41.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC0_40  C_VFCCU.FHDLRKC0_40.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC0_41  C_VFCCU.FHDLRKC0_41.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC0_50  C_VFCCU.FHIMRKC0_50.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC0_51  C_VFCCU.FHIMRKC0_51.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC0_50  C_VFCCU.FHDLRKC0_50.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC0_51  C_VFCCU.FHDLRKC0_51.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHCFG1       C_VFCCU.FHCFG1.R              /* Fault Handler */
#define C_VFCCU_FHSRVDS1     C_VFCCU.FHSRVDS1.R            /* Fault Handler Status */
#define C_VFCCU_FHFLTENC10   C_VFCCU.FHFLTENC10.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC11   C_VFCCU.FHFLTENC11.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC12   C_VFCCU.FHFLTENC12.R          /* Fault Enable */
#define C_VFCCU_FHFLTS10     C_VFCCU.FHFLTS10.R            /* Fault Status */
#define C_VFCCU_FHFLTS11     C_VFCCU.FHFLTS11.R            /* Fault Status */
#define C_VFCCU_FHFLTS12     C_VFCCU.FHFLTS12.R            /* Fault Status */
#define C_VFCCU_FHFLTRKC10   C_VFCCU.FHFLTRKC10.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC11   C_VFCCU.FHFLTRKC11.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC12   C_VFCCU.FHFLTRKC12.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC13   C_VFCCU.FHFLTRKC13.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC14   C_VFCCU.FHFLTRKC14.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC15   C_VFCCU.FHFLTRKC15.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC16   C_VFCCU.FHFLTRKC16.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC17   C_VFCCU.FHFLTRKC17.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC18   C_VFCCU.FHFLTRKC18.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHIMRKC1_00  C_VFCCU.FHIMRKC1_00.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC1_01  C_VFCCU.FHIMRKC1_01.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC1_00  C_VFCCU.FHDLRKC1_00.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC1_01  C_VFCCU.FHDLRKC1_01.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC1_10  C_VFCCU.FHIMRKC1_10.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC1_11  C_VFCCU.FHIMRKC1_11.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC1_10  C_VFCCU.FHDLRKC1_10.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC1_11  C_VFCCU.FHDLRKC1_11.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC1_20  C_VFCCU.FHIMRKC1_20.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC1_21  C_VFCCU.FHIMRKC1_21.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC1_20  C_VFCCU.FHDLRKC1_20.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC1_21  C_VFCCU.FHDLRKC1_21.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC1_30  C_VFCCU.FHIMRKC1_30.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC1_31  C_VFCCU.FHIMRKC1_31.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC1_30  C_VFCCU.FHDLRKC1_30.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC1_31  C_VFCCU.FHDLRKC1_31.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC1_40  C_VFCCU.FHIMRKC1_40.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC1_41  C_VFCCU.FHIMRKC1_41.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC1_40  C_VFCCU.FHDLRKC1_40.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC1_41  C_VFCCU.FHDLRKC1_41.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC1_50  C_VFCCU.FHIMRKC1_50.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC1_51  C_VFCCU.FHIMRKC1_51.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC1_50  C_VFCCU.FHDLRKC1_50.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC1_51  C_VFCCU.FHDLRKC1_51.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHCFG2       C_VFCCU.FHCFG2.R              /* Fault Handler */
#define C_VFCCU_FHSRVDS2     C_VFCCU.FHSRVDS2.R            /* Fault Handler Status */
#define C_VFCCU_FHFLTENC20   C_VFCCU.FHFLTENC20.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC21   C_VFCCU.FHFLTENC21.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC22   C_VFCCU.FHFLTENC22.R          /* Fault Enable */
#define C_VFCCU_FHFLTS20     C_VFCCU.FHFLTS20.R            /* Fault Status */
#define C_VFCCU_FHFLTS21     C_VFCCU.FHFLTS21.R            /* Fault Status */
#define C_VFCCU_FHFLTS22     C_VFCCU.FHFLTS22.R            /* Fault Status */
#define C_VFCCU_FHFLTRKC20   C_VFCCU.FHFLTRKC20.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC21   C_VFCCU.FHFLTRKC21.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC22   C_VFCCU.FHFLTRKC22.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC23   C_VFCCU.FHFLTRKC23.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC24   C_VFCCU.FHFLTRKC24.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC25   C_VFCCU.FHFLTRKC25.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC26   C_VFCCU.FHFLTRKC26.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC27   C_VFCCU.FHFLTRKC27.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC28   C_VFCCU.FHFLTRKC28.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHIMRKC2_00  C_VFCCU.FHIMRKC2_00.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC2_01  C_VFCCU.FHIMRKC2_01.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC2_00  C_VFCCU.FHDLRKC2_00.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC2_01  C_VFCCU.FHDLRKC2_01.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC2_10  C_VFCCU.FHIMRKC2_10.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC2_11  C_VFCCU.FHIMRKC2_11.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC2_10  C_VFCCU.FHDLRKC2_10.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC2_11  C_VFCCU.FHDLRKC2_11.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC2_20  C_VFCCU.FHIMRKC2_20.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC2_21  C_VFCCU.FHIMRKC2_21.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC2_20  C_VFCCU.FHDLRKC2_20.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC2_21  C_VFCCU.FHDLRKC2_21.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC2_30  C_VFCCU.FHIMRKC2_30.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC2_31  C_VFCCU.FHIMRKC2_31.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC2_30  C_VFCCU.FHDLRKC2_30.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC2_31  C_VFCCU.FHDLRKC2_31.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC2_40  C_VFCCU.FHIMRKC2_40.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC2_41  C_VFCCU.FHIMRKC2_41.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC2_40  C_VFCCU.FHDLRKC2_40.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC2_41  C_VFCCU.FHDLRKC2_41.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC2_50  C_VFCCU.FHIMRKC2_50.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC2_51  C_VFCCU.FHIMRKC2_51.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC2_50  C_VFCCU.FHDLRKC2_50.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC2_51  C_VFCCU.FHDLRKC2_51.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHCFG3       C_VFCCU.FHCFG3.R              /* Fault Handler */
#define C_VFCCU_FHSRVDS3     C_VFCCU.FHSRVDS3.R            /* Fault Handler Status */
#define C_VFCCU_FHFLTENC30   C_VFCCU.FHFLTENC30.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC31   C_VFCCU.FHFLTENC31.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC32   C_VFCCU.FHFLTENC32.R          /* Fault Enable */
#define C_VFCCU_FHFLTS30     C_VFCCU.FHFLTS30.R            /* Fault Status */
#define C_VFCCU_FHFLTS31     C_VFCCU.FHFLTS31.R            /* Fault Status */
#define C_VFCCU_FHFLTS32     C_VFCCU.FHFLTS32.R            /* Fault Status */
#define C_VFCCU_FHFLTRKC30   C_VFCCU.FHFLTRKC30.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC31   C_VFCCU.FHFLTRKC31.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC32   C_VFCCU.FHFLTRKC32.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC33   C_VFCCU.FHFLTRKC33.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC34   C_VFCCU.FHFLTRKC34.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC35   C_VFCCU.FHFLTRKC35.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC36   C_VFCCU.FHFLTRKC36.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC37   C_VFCCU.FHFLTRKC37.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC38   C_VFCCU.FHFLTRKC38.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHIMRKC3_00  C_VFCCU.FHIMRKC3_00.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC3_01  C_VFCCU.FHIMRKC3_01.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC3_00  C_VFCCU.FHDLRKC3_00.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC3_01  C_VFCCU.FHDLRKC3_01.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC3_10  C_VFCCU.FHIMRKC3_10.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC3_11  C_VFCCU.FHIMRKC3_11.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC3_10  C_VFCCU.FHDLRKC3_10.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC3_11  C_VFCCU.FHDLRKC3_11.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC3_20  C_VFCCU.FHIMRKC3_20.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC3_21  C_VFCCU.FHIMRKC3_21.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC3_20  C_VFCCU.FHDLRKC3_20.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC3_21  C_VFCCU.FHDLRKC3_21.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC3_30  C_VFCCU.FHIMRKC3_30.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC3_31  C_VFCCU.FHIMRKC3_31.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC3_30  C_VFCCU.FHDLRKC3_30.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC3_31  C_VFCCU.FHDLRKC3_31.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC3_40  C_VFCCU.FHIMRKC3_40.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC3_41  C_VFCCU.FHIMRKC3_41.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC3_40  C_VFCCU.FHDLRKC3_40.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC3_41  C_VFCCU.FHDLRKC3_41.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC3_50  C_VFCCU.FHIMRKC3_50.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC3_51  C_VFCCU.FHIMRKC3_51.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC3_50  C_VFCCU.FHDLRKC3_50.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC3_51  C_VFCCU.FHDLRKC3_51.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHCFG4       C_VFCCU.FHCFG4.R              /* Fault Handler */
#define C_VFCCU_FHSRVDS4     C_VFCCU.FHSRVDS4.R            /* Fault Handler Status */
#define C_VFCCU_FHFLTENC40   C_VFCCU.FHFLTENC40.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC41   C_VFCCU.FHFLTENC41.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC42   C_VFCCU.FHFLTENC42.R          /* Fault Enable */
#define C_VFCCU_FHFLTS40     C_VFCCU.FHFLTS40.R            /* Fault Status */
#define C_VFCCU_FHFLTS41     C_VFCCU.FHFLTS41.R            /* Fault Status */
#define C_VFCCU_FHFLTS42     C_VFCCU.FHFLTS42.R            /* Fault Status */
#define C_VFCCU_FHFLTRKC40   C_VFCCU.FHFLTRKC40.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC41   C_VFCCU.FHFLTRKC41.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC42   C_VFCCU.FHFLTRKC42.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC43   C_VFCCU.FHFLTRKC43.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC44   C_VFCCU.FHFLTRKC44.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC45   C_VFCCU.FHFLTRKC45.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC46   C_VFCCU.FHFLTRKC46.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC47   C_VFCCU.FHFLTRKC47.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC48   C_VFCCU.FHFLTRKC48.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHIMRKC4_00  C_VFCCU.FHIMRKC4_00.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC4_01  C_VFCCU.FHIMRKC4_01.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC4_00  C_VFCCU.FHDLRKC4_00.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC4_01  C_VFCCU.FHDLRKC4_01.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC4_10  C_VFCCU.FHIMRKC4_10.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC4_11  C_VFCCU.FHIMRKC4_11.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC4_10  C_VFCCU.FHDLRKC4_10.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC4_11  C_VFCCU.FHDLRKC4_11.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC4_20  C_VFCCU.FHIMRKC4_20.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC4_21  C_VFCCU.FHIMRKC4_21.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC4_20  C_VFCCU.FHDLRKC4_20.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC4_21  C_VFCCU.FHDLRKC4_21.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC4_30  C_VFCCU.FHIMRKC4_30.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC4_31  C_VFCCU.FHIMRKC4_31.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC4_30  C_VFCCU.FHDLRKC4_30.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC4_31  C_VFCCU.FHDLRKC4_31.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC4_40  C_VFCCU.FHIMRKC4_40.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC4_41  C_VFCCU.FHIMRKC4_41.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC4_40  C_VFCCU.FHDLRKC4_40.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC4_41  C_VFCCU.FHDLRKC4_41.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC4_50  C_VFCCU.FHIMRKC4_50.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC4_51  C_VFCCU.FHIMRKC4_51.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC4_50  C_VFCCU.FHDLRKC4_50.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC4_51  C_VFCCU.FHDLRKC4_51.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHCFG5       C_VFCCU.FHCFG5.R              /* Fault Handler */
#define C_VFCCU_FHSRVDS5     C_VFCCU.FHSRVDS5.R            /* Fault Handler Status */
#define C_VFCCU_FHFLTENC50   C_VFCCU.FHFLTENC50.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC51   C_VFCCU.FHFLTENC51.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC52   C_VFCCU.FHFLTENC52.R          /* Fault Enable */
#define C_VFCCU_FHFLTS50     C_VFCCU.FHFLTS50.R            /* Fault Status */
#define C_VFCCU_FHFLTS51     C_VFCCU.FHFLTS51.R            /* Fault Status */
#define C_VFCCU_FHFLTS52     C_VFCCU.FHFLTS52.R            /* Fault Status */
#define C_VFCCU_FHFLTRKC50   C_VFCCU.FHFLTRKC50.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC51   C_VFCCU.FHFLTRKC51.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC52   C_VFCCU.FHFLTRKC52.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC53   C_VFCCU.FHFLTRKC53.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC54   C_VFCCU.FHFLTRKC54.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC55   C_VFCCU.FHFLTRKC55.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC56   C_VFCCU.FHFLTRKC56.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC57   C_VFCCU.FHFLTRKC57.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC58   C_VFCCU.FHFLTRKC58.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHIMRKC5_00  C_VFCCU.FHIMRKC5_00.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC5_01  C_VFCCU.FHIMRKC5_01.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC5_00  C_VFCCU.FHDLRKC5_00.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC5_01  C_VFCCU.FHDLRKC5_01.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC5_10  C_VFCCU.FHIMRKC5_10.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC5_11  C_VFCCU.FHIMRKC5_11.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC5_10  C_VFCCU.FHDLRKC5_10.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC5_11  C_VFCCU.FHDLRKC5_11.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC5_20  C_VFCCU.FHIMRKC5_20.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC5_21  C_VFCCU.FHIMRKC5_21.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC5_20  C_VFCCU.FHDLRKC5_20.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC5_21  C_VFCCU.FHDLRKC5_21.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC5_30  C_VFCCU.FHIMRKC5_30.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC5_31  C_VFCCU.FHIMRKC5_31.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC5_30  C_VFCCU.FHDLRKC5_30.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC5_31  C_VFCCU.FHDLRKC5_31.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC5_40  C_VFCCU.FHIMRKC5_40.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC5_41  C_VFCCU.FHIMRKC5_41.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC5_40  C_VFCCU.FHDLRKC5_40.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC5_41  C_VFCCU.FHDLRKC5_41.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC5_50  C_VFCCU.FHIMRKC5_50.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC5_51  C_VFCCU.FHIMRKC5_51.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC5_50  C_VFCCU.FHDLRKC5_50.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC5_51  C_VFCCU.FHDLRKC5_51.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHCFG6       C_VFCCU.FHCFG6.R              /* Fault Handler */
#define C_VFCCU_FHSRVDS6     C_VFCCU.FHSRVDS6.R            /* Fault Handler Status */
#define C_VFCCU_FHFLTENC60   C_VFCCU.FHFLTENC60.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC61   C_VFCCU.FHFLTENC61.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC62   C_VFCCU.FHFLTENC62.R          /* Fault Enable */
#define C_VFCCU_FHFLTS60     C_VFCCU.FHFLTS60.R            /* Fault Status */
#define C_VFCCU_FHFLTS61     C_VFCCU.FHFLTS61.R            /* Fault Status */
#define C_VFCCU_FHFLTS62     C_VFCCU.FHFLTS62.R            /* Fault Status */
#define C_VFCCU_FHFLTRKC60   C_VFCCU.FHFLTRKC60.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC61   C_VFCCU.FHFLTRKC61.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC62   C_VFCCU.FHFLTRKC62.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC63   C_VFCCU.FHFLTRKC63.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC64   C_VFCCU.FHFLTRKC64.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC65   C_VFCCU.FHFLTRKC65.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC66   C_VFCCU.FHFLTRKC66.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC67   C_VFCCU.FHFLTRKC67.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC68   C_VFCCU.FHFLTRKC68.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHIMRKC6_00  C_VFCCU.FHIMRKC6_00.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC6_01  C_VFCCU.FHIMRKC6_01.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC6_00  C_VFCCU.FHDLRKC6_00.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC6_01  C_VFCCU.FHDLRKC6_01.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC6_10  C_VFCCU.FHIMRKC6_10.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC6_11  C_VFCCU.FHIMRKC6_11.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC6_10  C_VFCCU.FHDLRKC6_10.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC6_11  C_VFCCU.FHDLRKC6_11.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC6_20  C_VFCCU.FHIMRKC6_20.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC6_21  C_VFCCU.FHIMRKC6_21.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC6_20  C_VFCCU.FHDLRKC6_20.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC6_21  C_VFCCU.FHDLRKC6_21.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC6_30  C_VFCCU.FHIMRKC6_30.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC6_31  C_VFCCU.FHIMRKC6_31.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC6_30  C_VFCCU.FHDLRKC6_30.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC6_31  C_VFCCU.FHDLRKC6_31.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC6_40  C_VFCCU.FHIMRKC6_40.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC6_41  C_VFCCU.FHIMRKC6_41.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC6_40  C_VFCCU.FHDLRKC6_40.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC6_41  C_VFCCU.FHDLRKC6_41.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC6_50  C_VFCCU.FHIMRKC6_50.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC6_51  C_VFCCU.FHIMRKC6_51.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC6_50  C_VFCCU.FHDLRKC6_50.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC6_51  C_VFCCU.FHDLRKC6_51.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHCFG7       C_VFCCU.FHCFG7.R              /* Fault Handler */
#define C_VFCCU_FHSRVDS7     C_VFCCU.FHSRVDS7.R            /* Fault Handler Status */
#define C_VFCCU_FHFLTENC70   C_VFCCU.FHFLTENC70.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC71   C_VFCCU.FHFLTENC71.R          /* Fault Enable */
#define C_VFCCU_FHFLTENC72   C_VFCCU.FHFLTENC72.R          /* Fault Enable */
#define C_VFCCU_FHFLTS70     C_VFCCU.FHFLTS70.R            /* Fault Status */
#define C_VFCCU_FHFLTS71     C_VFCCU.FHFLTS71.R            /* Fault Status */
#define C_VFCCU_FHFLTS72     C_VFCCU.FHFLTS72.R            /* Fault Status */
#define C_VFCCU_FHFLTRKC70   C_VFCCU.FHFLTRKC70.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC71   C_VFCCU.FHFLTRKC71.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC72   C_VFCCU.FHFLTRKC72.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC73   C_VFCCU.FHFLTRKC73.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC74   C_VFCCU.FHFLTRKC74.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC75   C_VFCCU.FHFLTRKC75.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC76   C_VFCCU.FHFLTRKC76.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC77   C_VFCCU.FHFLTRKC77.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHFLTRKC78   C_VFCCU.FHFLTRKC78.R          /* Fault Reaction Set Configuration */
#define C_VFCCU_FHIMRKC7_00  C_VFCCU.FHIMRKC7_00.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC7_01  C_VFCCU.FHIMRKC7_01.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC7_00  C_VFCCU.FHDLRKC7_00.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC7_01  C_VFCCU.FHDLRKC7_01.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC7_10  C_VFCCU.FHIMRKC7_10.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC7_11  C_VFCCU.FHIMRKC7_11.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC7_10  C_VFCCU.FHDLRKC7_10.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC7_11  C_VFCCU.FHDLRKC7_11.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC7_20  C_VFCCU.FHIMRKC7_20.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC7_21  C_VFCCU.FHIMRKC7_21.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC7_20  C_VFCCU.FHDLRKC7_20.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC7_21  C_VFCCU.FHDLRKC7_21.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC7_30  C_VFCCU.FHIMRKC7_30.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC7_31  C_VFCCU.FHIMRKC7_31.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC7_30  C_VFCCU.FHDLRKC7_30.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC7_31  C_VFCCU.FHDLRKC7_31.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC7_40  C_VFCCU.FHIMRKC7_40.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC7_41  C_VFCCU.FHIMRKC7_41.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC7_40  C_VFCCU.FHDLRKC7_40.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC7_41  C_VFCCU.FHDLRKC7_41.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHIMRKC7_50  C_VFCCU.FHIMRKC7_50.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHIMRKC7_51  C_VFCCU.FHIMRKC7_51.R         /* Immediate Reaction Configuration */
#define C_VFCCU_FHDLRKC7_50  C_VFCCU.FHDLRKC7_50.R         /* Delayed Reaction Configuration */
#define C_VFCCU_FHDLRKC7_51  C_VFCCU.FHDLRKC7_51.R         /* Delayed Reaction Configuration */

/* DATA_MEMORY_SUBSYSTEM */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DMBA CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DMBA.R /* IDM Base Address */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DMBE CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DMBE.R /* DMSS Idle Indication */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_HDCFG CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_HDCFG.R /* DMSS Hardware Configuration */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_GPOUT CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_GPOUT.R /* MSS General Purpose 2 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DACC CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DACC.R /* DDMA Access Configuration */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_SDCFG CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_SDCFG.R /* MSS SW Configuration */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_BARRIER CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_BARRIER.R /* MSS Memory Barrier Control */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_D_DMAB CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.D_DMAB.R /* DDMA Status */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_DBG_WRC0 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.DBG_WRC0.R /* EDP Write Response Counter */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DDTC CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DDTC.R /* DDMA Linear Transfer Size */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_2DCFG1 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_2DCFG1.R /* DDMA Configuration 1 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_2DCFG2 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_2DCFG2.R /* DDMA Configuration 2 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_2DCFG3 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_2DCFG3.R /* DDMA Configuration 3 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_2DCFG4 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_2DCFG4.R /* DDMA Configuration 4 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_2DCFG5 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_2DCFG5.R /* DDMA Configuration 5 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_2DCFG6 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_2DCFG6.R /* DDMA Configuration 6 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DDQS CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DDQS.R /* DDMA Status */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DDEA CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DDEA.R /* DDMA Address 1 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DDIA CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DDIA.R /* DDMA Address 2 */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DDCL CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DDCL.R /* DDMA Control */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_DDMA_PAUSE CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.DDMA_PAUSE.R /* DDMA Pause Control */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DDESC_ID CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DDESC_ID.R /* DDMA Interrupt Source */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSS_DDESC_OV CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSS_DDESC_OV.R /* DDMA Task ID Overwrite Indication */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_MSTR_TASK_ID CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.MSTR_TASK_ID.R /* DDMA Task (Master) ID */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_QMAN_TASK_IDx0 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.QMAN_TASK_IDX[0].R /* QMAN Task IDx */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_QMAN_TASK_IDx1 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.QMAN_TASK_IDX[1].R /* QMAN Task IDx */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_QMAN_TASK_IDx2 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.QMAN_TASK_IDX[2].R /* QMAN Task IDx */
#define CEVA_SPF2__DATA_MEMORY_SUBSYSTEM_QMAN_TASK_IDx3 CEVA_SPF2__DATA_MEMORY_SUBSYSTEM.QMAN_TASK_IDX[3].R /* QMAN Task IDx */

/* DBG */
#define CEVA_SPF2__DBG_DBG_DDAM CEVA_SPF2__DBG.DBG_DDAM.R  /* DDMA Breakpoint Register */
#define CEVA_SPF2__DBG_DBGACS CEVA_SPF2__DBG.DBGACS.R      /* Error Access Control Register */
#define CEVA_SPF2__DBG_DBG_DDC CEVA_SPF2__DBG.DBG_DDC.R    /* DDMA Breakpoint Control Register */
#define CEVA_SPF2__DBG_DBG_DDS CEVA_SPF2__DBG.DBG_DDS.R    /* DDMA Breakpoint Status Register */
#define CEVA_SPF2__DBG_CENSIRQ CEVA_SPF2__DBG.CENSIRQ.R    /* Critical Error Interrupt Requests Register */
#define CEVA_SPF2__DBG_CENSIRQ2 CEVA_SPF2__DBG.CENSIRQ2.R  /* Critical Errors Interrupt 2 Register */
#define CEVA_SPF2__DBG_CWDOGIRQ CEVA_SPF2__DBG.CWDOGIRQ.R  /* Critical Error Interrupt Requests Register */
#define CEVA_SPF2__DBG_NENSIRQ CEVA_SPF2__DBG.NENSIRQ.R    /* Normal Error Interrupt Requests Register */
#define CEVA_SPF2__DBG_TIMER_IRQ CEVA_SPF2__DBG.TIMER_IRQ.R /* Timer Interrupt Source Register */
#define CEVA_SPF2__DBG_DMA_IRQ CEVA_SPF2__DBG.DMA_IRQ.R    /* DMA Interrupt Source Register */
#define CEVA_SPF2__DBG_POSCINT CEVA_SPF2__DBG.POSCINT.R    /* Posted Core Interrupt Register */
#define CEVA_SPF2__DBG_NQBIRQ CEVA_SPF2__DBG.NQBIRQ.R      /* Normal QMAN and Buffers Interrupt Register */
#define CEVA_SPF2__DBG_NENSIRQ_M CEVA_SPF2__DBG.NENSIRQ_M.R /* Normal Error Interrupt Request Mask Register */
#define CEVA_SPF2__DBG_TIMER_IRQ_M CEVA_SPF2__DBG.TIMER_IRQ_M.R /* Timer Interrupt Mask Register */
#define CEVA_SPF2__DBG_NQBIRQ_M CEVA_SPF2__DBG.NQBIRQ_M.R  /* Normal QMAN and Buffers Interrupt Mask Register */
#define CEVA_SPF2__DBG_DBG_STACK_START CEVA_SPF2__DBG.DBG_STACK_START.R /* Stack Violation Start Address Register */
#define CEVA_SPF2__DBG_DBG_STACK_END CEVA_SPF2__DBG.DBG_STACK_END.R /* Stack Violation End Address Register */
#define CEVA_SPF2__DBG_DBG_DUNMPD_MSK CEVA_SPF2__DBG.DBG_DUNMPD_MSK.R /* DBG DMSS Unmapped Access Mask Register */
#define CEVA_SPF2__DBG_DBG_DUNMPD CEVA_SPF2__DBG.DBG_DUNMPD.R /* DMSS Unmapped Access Status Register */
#define CEVA_SPF2__DBG_CENSIRQ_S CEVA_SPF2__DBG.CENSIRQ_S.R /* Critical Error Interrupt Request Shadow Register */
#define CEVA_SPF2__DBG_CENSIRQ2_S CEVA_SPF2__DBG.CENSIRQ2_S.R /* Critical Error Interrupt Request 2 Shadow Register */
#define CEVA_SPF2__DBG_CWDOGIRQ_S CEVA_SPF2__DBG.CWDOGIRQ_S.R /* Critical Error Interrupt Request Shadow Register */
#define CEVA_SPF2__DBG_NENSIRQ_S CEVA_SPF2__DBG.NENSIRQ_S.R /* Normal Error Interrupt Requests Shadow Register */
#define CEVA_SPF2__DBG_TIMER_IRQ_S CEVA_SPF2__DBG.TIMER_IRQ_S.R /* Timer Interrupt Shadow Register */
#define CEVA_SPF2__DBG_DMA_IRQ_S CEVA_SPF2__DBG.DMA_IRQ_S.R /* DMA Interrupt Shadow Register */
#define CEVA_SPF2__DBG_DBG_DESC_ID CEVA_SPF2__DBG.DBG_DESC_ID.R /* DBG DESC ID Register */
#define CEVA_SPF2__DBG_DBG_QMAN_ID CEVA_SPF2__DBG.DBG_QMAN_ID.R /* DBG QMAN ID Register */
#define CEVA_SPF2__DBG_XCI_COR CEVA_SPF2__DBG.XCI_COR.R    /* MSS Acknowledge Control Register */

/* DCU */
#define CEVA_SPF2__DCU_OCM_PADD1 CEVA_SPF2__DCU.OCM_PADD1.R /* OCM PADD1 Register */
#define CEVA_SPF2__DCU_OCM_PADD2 CEVA_SPF2__DCU.OCM_PADD2.R /* OCM PADD2 Register */
#define CEVA_SPF2__DCU_OCM_PCOUNT1 CEVA_SPF2__DCU.OCM_PCOUNT1.R /* OCM PCOUNT1 Register */
#define CEVA_SPF2__DCU_OCM_PCOUNT2 CEVA_SPF2__DCU.OCM_PCOUNT2.R /* OCM PCOUNT2 Register */
#define CEVA_SPF2__DCU_CEVAX_A_PC CEVA_SPF2__DCU.CEVAX_A_PC.R /* CEVAX A PC Register */
#define CEVA_SPF2__DCU_OCM_DADD_LOW CEVA_SPF2__DCU.OCM_DADD_LOW.R /* OCM DDD Low Register */
#define CEVA_SPF2__DCU_OCM_DADD_HIGH CEVA_SPF2__DCU.OCM_DADD_HIGH.R /* OCM DADD HIGH Register */
#define CEVA_SPF2__DCU_OCM_ACOUNT CEVA_SPF2__DCU.OCM_ACOUNT.R /* OCM ACOUNT Register */
#define CEVA_SPF2__DCU_OCM_DCOUNT CEVA_SPF2__DCU.OCM_DCOUNT.R /* OCM DCOUNT Register */
#define CEVA_SPF2__DCU_OCM_DVM CEVA_SPF2__DCU.OCM_DVM.R    /* OCM DVM Register */
#define CEVA_SPF2__DCU_OCM_CONTROL CEVA_SPF2__DCU.OCM_CONTROL.R /* OCM CONTROL Register */
#define CEVA_SPF2__DCU_OCM_SA_BP_EN CEVA_SPF2__DCU.OCM_SA_BP_EN.R /* OCM SA BP EN Register */
#define CEVA_SPF2__DCU_MEM_CONFIG CEVA_SPF2__DCU.MEM_CONFIG.R /* MEM CONFIG Register */
#define CEVA_SPF2__DCU_MSS_CONFIG CEVA_SPF2__DCU.MSS_CONFIG.R /* MSS CONFIG Register */
#define CEVA_SPF2__DCU_OCM_STATUS CEVA_SPF2__DCU.OCM_STATUS.R /* OCM STATUS Register */
#define CEVA_SPF2__DCU_OCM_SA_BP_ST CEVA_SPF2__DCU.OCM_SA_BP_ST.R /* OCM SA BP ST Register */
#define CEVA_SPF2__DCU_M_CONFIG CEVA_SPF2__DCU.M_CONFIG.R  /* M CONFIG Register */
#define CEVA_SPF2__DCU_S_CONFIG CEVA_SPF2__DCU.S_CONFIG.R  /* S_CONFIG Register */
#define CEVA_SPF2__DCU_DEBUG_CONFIG CEVA_SPF2__DCU.DEBUG_CONFIG.R /* DEBUG CONFIG Register */
#define CEVA_SPF2__DCU_CORE_VERSION CEVA_SPF2__DCU.CORE_VERSION.R /* CORE VERSION Register */
#define CEVA_SPF2__DCU_CORE_ID CEVA_SPF2__DCU.CORE_ID.R    /* CORE ID Register */
#define CEVA_SPF2__DCU_CORE_CONFIG CEVA_SPF2__DCU.CORE_CONFIG.R /* CORE CONFIG Register */

/* DDRC */
#define DDR_CS0_BNDS         DDR.CS_BNDS[0].CS_BNDS.R      /* Rank 0 Memory Bounds */
#define DDR_CS1_BNDS         DDR.CS_BNDS[1].CS_BNDS.R      /* Rank 1 Memory Bounds */
#define DDR_REMAP_0A         DDR.REMAP_0A.R                /* Remap Region 0A Configuration */
#define DDR_REMAP_0B         DDR.REMAP_0B.R                /* Remap Region 0B Configuration */
#define DDR_REMAP_1A         DDR.REMAP_1A.R                /* Remap Region 1A Configuration */
#define DDR_REMAP_1B         DDR.REMAP_1B.R                /* Remap Region 1B Configuration */
#define DDR_REMAP_2A         DDR.REMAP_2A.R                /* Remap Region 2A Configuration */
#define DDR_REMAP_2B         DDR.REMAP_2B.R                /* Remap Region 2B Configuration */
#define DDR_REMAP_3A         DDR.REMAP_3A.R                /* Remap Region 3A Configuration */
#define DDR_REMAP_3B         DDR.REMAP_3B.R                /* Remap Region 3B Configuration */
#define DDR_DDR_ADDR_DEC_0   DDR.DDR_ADDR_DEC_0.R          /* DDRC Address Decode 0 */
#define DDR_DDR_ADDR_DEC_1   DDR.DDR_ADDR_DEC_1.R          /* DDRC Address Decode 1 */
#define DDR_DDR_ADDR_DEC_2   DDR.DDR_ADDR_DEC_2.R          /* DDRC Address Decode 2 */
#define DDR_DDR_ADDR_DEC_3   DDR.DDR_ADDR_DEC_3.R          /* DDRC Address Decode 3 */
#define DDR_DDR_ADDR_DEC_4   DDR.DDR_ADDR_DEC_4.R          /* DDRC Address Decode 4 */
#define DDR_DDR_ADDR_DEC_5   DDR.DDR_ADDR_DEC_5.R          /* DDRC Address Decode 5 */
#define DDR_DDR_ADDR_DEC_6   DDR.DDR_ADDR_DEC_6.R          /* DDRC Address Decode 6 */
#define DDR_DDR_ADDR_DEC_7   DDR.DDR_ADDR_DEC_7.R          /* DDRC Address Decode 7 */
#define DDR_DDR_ADDR_DEC_8   DDR.DDR_ADDR_DEC_8.R          /* DDRC Address Decode 8 */
#define DDR_DDR_ADDR_DEC_9   DDR.DDR_ADDR_DEC_9.R          /* DDRC Address Decode 9 */
#define DDR_CS0_CONFIG       DDR.CS_CONFIG[0].R            /* Rank 0 Configuration */
#define DDR_CS1_CONFIG       DDR.CS_CONFIG[1].R            /* Rank 1 Configuration */
#define DDR_TIMING_CFG_3     DDR.TIMING_CFG_3.R            /* DDR SDRAM Timing Configuration 3 */
#define DDR_TIMING_CFG_0     DDR.TIMING_CFG_0.R            /* DDR SDRAM Timing Configuration 0 */
#define DDR_TIMING_CFG_1     DDR.TIMING_CFG_1.R            /* DDR SDRAM Timing Configuration 1 */
#define DDR_TIMING_CFG_2     DDR.TIMING_CFG_2.R            /* DDR SDRAM Timing Configuration 2 */
#define DDR_DDR_SDRAM_CFG    DDR.DDR_SDRAM_CFG.R           /* DDR SDRAM Control Configuration */
#define DDR_DDR_SDRAM_CFG_2  DDR.DDR_SDRAM_CFG_2.R         /* DDR SDRAM Control Configuration 2 */
#define DDR_DDR_SDRAM_MD_CNTL DDR.DDR_SDRAM_MD_CNTL.R      /* DDR SDRAM Mode Control */
#define DDR_DDR_SDRAM_INTERVAL DDR.DDR_SDRAM_INTERVAL.R    /* DDR SDRAM Interval Configuration */
#define DDR_DDR_DATA_INIT    DDR.DDR_DATA_INIT.R           /* DDR SDRAM Data Initialization */
#define DDR_TIMING_CFG_4     DDR.TIMING_CFG_4.R            /* DDR SDRAM Timing Configuration 4 */
#define DDR_TIMING_CFG_7     DDR.TIMING_CFG_7.R            /* DDR SDRAM Timing Configuration 7 */
#define DDR_DDR_ZQ_CNTL      DDR.DDR_ZQ_CNTL.R             /* DDR SDRAM ZQ Calibration Control */
#define DDR_DDR_SR_CNTR      DDR.DDR_SR_CNTR.R             /* DDR SDRAM Self-Refresh Counter */
#define DDR_TIMING_CFG_8     DDR.TIMING_CFG_8.R            /* DDR SDRAM Timing Configuration 8 */
#define DDR_TIMING_CFG_9     DDR.TIMING_CFG_9.R            /* DDR SDRAM timing configuration 9 */
#define DDR_TIMING_CFG_11    DDR.TIMING_CFG_11.R           /* DDR SDRAM Timing Configuration 11 */
#define DDR_DDR_SDRAM_CFG_3  DDR.DDR_SDRAM_CFG_3.R         /* DDR SDRAM Control Configuration 3 */
#define DDR_DDR_SDRAM_REF_RATE DDR.DDR_SDRAM_REF_RATE.R    /* DDR Refresh Rate */
#define DDR_TX_CFG_1         DDR.TX_CFG_1.R                /* Transaction Configuration Register 1 */
#define DDR_FFI_CFG          DDR.FFI_CFG.R                 /* Freedom From Interference Configuration */
#define DDR_FFI_CFG2         DDR.FFI_CFG2.R                /* Freedom From Interference Configuration 2 */
#define DDR_DDRDSR_2         DDR.DDRDSR_2.R                /* DDR SDRAM Debug Status 2 */
#define DDR_DDR_IP_REV1      DDR.DDR_IP_REV1.R             /* DDRC Revision 1 */
#define DDR_DDR_MTCR         DDR.DDR_MTCR.R                /* DDR SDRAM Memory Test Control */
#define DDR_DDR_MTP0         DDR.DDR_MTP[0].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP1         DDR.DDR_MTP[1].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP2         DDR.DDR_MTP[2].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP3         DDR.DDR_MTP[3].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP4         DDR.DDR_MTP[4].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP5         DDR.DDR_MTP[5].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP6         DDR.DDR_MTP[6].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP7         DDR.DDR_MTP[7].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP8         DDR.DDR_MTP[8].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MTP9         DDR.DDR_MTP[9].R              /* DDR SDRAM Memory Test Pattern n */
#define DDR_DDR_MT_ST_EXT_ADDR DDR.DDR_MT_ST_EXT_ADDR.R    /* DDR SDRAM Memory Test Start Extended Address */
#define DDR_DDR_MT_ST_ADDR   DDR.DDR_MT_ST_ADDR.R          /* DDR SDRAM Memory Test Start Address */
#define DDR_DDR_MT_END_EXT_ADDR DDR.DDR_MT_END_EXT_ADDR.R  /* DDR SDRAM Memory Test End Extended Address */
#define DDR_DDR_MT_END_ADDR  DDR.DDR_MT_END_ADDR.R         /* DDR SDRAM Memory Test End Address */
#define DDR_PMGC0            DDR.PMGC0.R                   /* Performance Monitor Global Control */
#define DDR_PMLCA0           DDR.PMLCA0.R                  /* Performance Monitor Local Control A0 */
#define DDR_PMLCB0           DDR.PMLCB0.R                  /* Performance Monitor Local Control B0 */
#define DDR_PMC0a            DDR.PMC0A.R                   /* PMC 0a */
#define DDR_PMC0b            DDR.PMC0B.R                   /* PMC 0b */
#define DDR_PMLCA1           DDR.PMLCA1.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB1           DDR.PMLCB1.R                  /* Performance Monitor Local Control B */
#define DDR_PMC1             DDR.PMC1.R                    /* Performance Monitor Counter */
#define DDR_PMLCA2           DDR.PMLCA2.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB2           DDR.PMLCB2.R                  /* Performance Monitor Local Control B */
#define DDR_PMC2             DDR.PMC2.R                    /* Performance Monitor Counter */
#define DDR_PMLCA3           DDR.PMLCA3.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB3           DDR.PMLCB3.R                  /* Performance Monitor Local Control B */
#define DDR_PMC3             DDR.PMC3.R                    /* Performance Monitor Counter */
#define DDR_PMLCA4           DDR.PMLCA4.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB4           DDR.PMLCB4.R                  /* Performance Monitor Local Control B */
#define DDR_PMC4             DDR.PMC4.R                    /* Performance Monitor Counter */
#define DDR_PMLCA5           DDR.PMLCA5.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB5           DDR.PMLCB5.R                  /* Performance Monitor Local Control B */
#define DDR_PMC5             DDR.PMC5.R                    /* Performance Monitor Counter */
#define DDR_PMLCA6           DDR.PMLCA6.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB6           DDR.PMLCB6.R                  /* Performance Monitor Local Control B */
#define DDR_PMC6             DDR.PMC6.R                    /* Performance Monitor Counter */
#define DDR_PMLCA7           DDR.PMLCA7.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB7           DDR.PMLCB7.R                  /* Performance Monitor Local Control B */
#define DDR_PMC7             DDR.PMC7.R                    /* Performance Monitor Counter */
#define DDR_PMLCA8           DDR.PMLCA8.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB8           DDR.PMLCB8.R                  /* Performance Monitor Local Control B */
#define DDR_PMC8             DDR.PMC8.R                    /* Performance Monitor Counter */
#define DDR_PMLCA9           DDR.PMLCA9.R                  /* Performance Monitor Local Control A */
#define DDR_PMLCB9           DDR.PMLCB9.R                  /* Performance Monitor Local Control B */
#define DDR_PMC9             DDR.PMC9.R                    /* Performance Monitor Counter */
#define DDR_PMLCA10          DDR.PMLCA10.R                 /* Performance Monitor Local Control A */
#define DDR_PMLCB10          DDR.PMLCB10.R                 /* Performance Monitor Local Control B */
#define DDR_PMC10            DDR.PMC10.R                   /* Performance Monitor Counter */
#define DDR_ERR_EN           DDR.ERR_EN.R                  /* Error Enable */
#define DDR_DATA_ERR_INJECT_HI DDR.DATA_ERR_INJECT_HI.R    /* Memory Data Path Error Injection Mask High */
#define DDR_DATA_ERR_INJECT_LO DDR.DATA_ERR_INJECT_LO.R    /* Memory Data Path Error Injection Mask Low */
#define DDR_ERR_INJECT       DDR.ERR_INJECT.R              /* Memory Data Path Error Injection Mask ECC */
#define DDR_ADDR_ERR_INJ     DDR.ADDR_ERR_INJ.R            /* Address Error Inject */
#define DDR_CAPTURE_EXT_DATA_HI DDR.CAPTURE_EXT_DATA_HI.R  /* Memory Extended Data Path Read Capture High */
#define DDR_CAPTURE_EXT_DATA_LO DDR.CAPTURE_EXT_DATA_LO.R  /* Memory Extended Data Path Read Capture Low */
#define DDR_CAPTURE_DATA_HI  DDR.CAPTURE_DATA_HI.R         /* Memory Data Path Read Capture High */
#define DDR_CAPTURE_DATA_LO  DDR.CAPTURE_DATA_LO.R         /* Memory Data Path Read Capture Low */
#define DDR_CAPTURE_ECC      DDR.CAPTURE_ECC.R             /* Memory Data Path Read Capture ECC */
#define DDR_ERR_DETECT       DDR.ERR_DETECT.R              /* Memory Error Detect */
#define DDR_ERR_DISABLE      DDR.ERR_DISABLE.R             /* Memory Error Disable */
#define DDR_ERR_INT_EN       DDR.ERR_INT_EN.R              /* Memory Error Interrupt Enable */
#define DDR_CAPTURE_ATTRIBUTES DDR.CAPTURE_ATTRIBUTES.R    /* Memory Error Attributes Capture */
#define DDR_CAPTURE_ADDRESS  DDR.CAPTURE_ADDRESS.R         /* Memory Error Address Capture */
#define DDR_CAPTURE_EXT_ADDRESS DDR.CAPTURE_EXT_ADDRESS.R  /* Memory Error Extended Address Capture */
#define DDR_ERR_SBE          DDR.ERR_SBE.R                 /* Single-Bit ECC Memory Error Management */
#define DDR_REG_LKSTP_CNTL   DDR.REG_LKSTP_CNTL.R          /* Lockstep Register Control */
#define DDR_REG_CRC_GRP_1    DDR.REG_CRC_GRP_1.R           /* Register CRC Code For Group 1 */
#define DDR_REG_CRC_GRP_2    DDR.REG_CRC_GRP_2.R           /* Register CRC Code For Group 2 */
#define DDR_ECC_REG_0        DDR.ECC_REG_0.R               /* ECC Region 0 Configuration */
#define DDR_ECC_REG_1        DDR.ECC_REG_1.R               /* ECC Region 1 Configuration */
#define DDR_ECC_REG_2        DDR.ECC_REG_2.R               /* ECC Region 2 Configuration */
#define DDR_ECC_REG_3        DDR.ECC_REG_3.R               /* ECC Region 3 Configuration */
#define DDR_ECC_REG_4        DDR.ECC_REG_4.R               /* ECC Region 4 Configuration */
#define DDR_ECC_REG_5        DDR.ECC_REG_5.R               /* ECC Region 5 Configuration */
#define DDR_ECC_REG_6        DDR.ECC_REG_6.R               /* ECC Region 6 Configuration */
#define DDR_ECC_REG_7        DDR.ECC_REG_7.R               /* ECC Region 7 Configuration */

/* DDR_PLL */
#define DDR_PLL_PLLCR        DDR_PLL.PLLCR.R               /* PLL Control */
#define DDR_PLL_PLLSR        DDR_PLL.PLLSR.R               /* PLL Status */
#define DDR_PLL_PLLDV        DDR_PLL.PLLDV.R               /* PLL Divider */
#define DDR_PLL_PLLFM        DDR_PLL.PLLFM.R               /* PLL Frequency Modulation */
#define DDR_PLL_PLLFD        DDR_PLL.PLLFD.R               /* PLL Fractional Divider */
#define DDR_PLL_PLLCLKMUX    DDR_PLL.PLLCLKMUX.R           /* PLL Clock Multiplexer */
#define DDR_PLL_PLLODIV_0    DDR_PLL.PLLODIV[0].R          /* PLL Output Divider */

/* DFS */
#define CORE_DFS_PORTSR      CORE_DFS.PORTSR.R             /* Port Status Register */
#define CORE_DFS_PORTLOLSR   CORE_DFS.PORTLOLSR.R          /* Port Loss of Lock Status */
#define CORE_DFS_PORTRESET   CORE_DFS.PORTRESET.R          /* Port Reset */
#define CORE_DFS_CTL         CORE_DFS.CTL.R                /* Control */
#define CORE_DFS_DVPORT0     CORE_DFS.DVPORT[0].R          /* Divider for Port 0 */
#define CORE_DFS_DVPORT1     CORE_DFS.DVPORT[1].R          /* Divider for Port 1 */
#define CORE_DFS_DVPORT2     CORE_DFS.DVPORT[2].R          /* Divider for Port 2 */
#define CORE_DFS_DVPORT3     CORE_DFS.DVPORT[3].R          /* Divider for Port 3 */
#define CORE_DFS_DVPORT4     CORE_DFS.DVPORT[4].R          /* Divider for Port 4 */
#define CORE_DFS_DVPORT5     CORE_DFS.DVPORT[5].R          /* Divider for Port 5 */

#define PERIPH_DFS_PORTSR    PERIPH_DFS.PORTSR.R           /* Port Status Register */
#define PERIPH_DFS_PORTLOLSR PERIPH_DFS.PORTLOLSR.R        /* Port Loss of Lock Status */
#define PERIPH_DFS_PORTRESET PERIPH_DFS.PORTRESET.R        /* Port Reset */
#define PERIPH_DFS_CTL       PERIPH_DFS.CTL.R              /* Control */
#define PERIPH_DFS_DVPORT0   PERIPH_DFS.DVPORT[0].R        /* Divider for Port 0 */
#define PERIPH_DFS_DVPORT1   PERIPH_DFS.DVPORT[1].R        /* Divider for Port 1 */
#define PERIPH_DFS_DVPORT2   PERIPH_DFS.DVPORT[2].R        /* Divider for Port 2 */
#define PERIPH_DFS_DVPORT3   PERIPH_DFS.DVPORT[3].R        /* Divider for Port 3 */
#define PERIPH_DFS_DVPORT4   PERIPH_DFS.DVPORT[4].R        /* Divider for Port 4 */
#define PERIPH_DFS_DVPORT5   PERIPH_DFS.DVPORT[5].R        /* Divider for Port 5 */

/* DIPORTSD */
#define DIPORTSD_ECR         DIPORTSD.ECR.R                /* Error and Control Register */
#define DIPORTSD_SESR        DIPORTSD.SESR.R               /* Error Status Register */
#define DIPORTSD_QOS         DIPORTSD.QOS.R                /* Quality Of Service */
#define DIPORTSD_SIGEN       DIPORTSD.SIGEN.R              /* Signaling Enable */
#define DIPORTSD_RSRV0       DIPORTSD.RSRV0.R              /* Reserved 0 */
#define DIPORTSD_LAR0        DIPORTSD.LAR0.R               /* Logical Address Region 0 */
#define DIPORTSD_PAR0        DIPORTSD.PAR0.R               /* Physical Address Region 0 */
#define DIPORTSD_RS0         DIPORTSD.RS0.R                /* Region Size 0 */
#define DIPORTSD_LAR1        DIPORTSD.LAR1.R               /* Logical Address Region 1 */
#define DIPORTSD_PAR1        DIPORTSD.PAR1.R               /* Physical Address Region 1 */
#define DIPORTSD_RS1         DIPORTSD.RS1.R                /* Region Size 1 */
#define DIPORTSD_LAR2        DIPORTSD.LAR2.R               /* Logical Address Region 2 */
#define DIPORTSD_PAR2        DIPORTSD.PAR2.R               /* Physical Address Region 2 */
#define DIPORTSD_RS2         DIPORTSD.RS2.R                /* Region Size 2 */
#define DIPORTSD_LAR3        DIPORTSD.LAR3.R               /* Logical Address Region 3 */
#define DIPORTSD_PAR3        DIPORTSD.PAR3.R               /* Physical Address Region 3 */
#define DIPORTSD_RS3         DIPORTSD.RS3.R                /* Region Size 3 */
#define DIPORTSD_LAR4        DIPORTSD.LAR4.R               /* Logical Address Region 4 */
#define DIPORTSD_PAR4        DIPORTSD.PAR4.R               /* Physical Address Region 4 */
#define DIPORTSD_RS4         DIPORTSD.RS4.R                /* Region Size 4 */
#define DIPORTSD_LAR5        DIPORTSD.LAR5.R               /* Logical Address Region 5 */
#define DIPORTSD_PAR5        DIPORTSD.PAR5.R               /* Physical Address Region 5 */
#define DIPORTSD_RS5         DIPORTSD.RS5.R                /* Region Size 5 */
#define DIPORTSD_LAR6        DIPORTSD.LAR6.R               /* Logical Address Region 6 */
#define DIPORTSD_PAR6        DIPORTSD.PAR6.R               /* Physical Address Region 6 */
#define DIPORTSD_RS6         DIPORTSD.RS6.R                /* Region Size 6 */
#define DIPORTSD_LAR7        DIPORTSD.LAR7.R               /* Logical Address Region 7 */
#define DIPORTSD_PAR7        DIPORTSD.PAR7.R               /* Physical Address Region 7 */
#define DIPORTSD_RS7         DIPORTSD.RS7.R                /* Region Size 7 */
#define DIPORTSD_RSRV1       DIPORTSD.RSRV1.R              /* Reserved 1 */
#define DIPORTSD_RSRV2       DIPORTSD.RSRV2.R              /* Reserved 2 */
#define DIPORTSD_RSRV3       DIPORTSD.RSRV3.R              /* Reserved 3 */
#define DIPORTSD_RSRV4       DIPORTSD.RSRV4.R              /* Reserved 4 */
#define DIPORTSD_RSRV5       DIPORTSD.RSRV5.R              /* Reserved 5 */

/* DMAMUX */
#define DMAMUX_1_CHCFG3      DMAMUX_1.CHCONF[0].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG2      DMAMUX_1.CHCONF[1].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG1      DMAMUX_1.CHCONF[2].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG0      DMAMUX_1.CHCONF[3].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG7      DMAMUX_1.CHCONF[4].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG6      DMAMUX_1.CHCONF[5].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG5      DMAMUX_1.CHCONF[6].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG4      DMAMUX_1.CHCONF[7].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG11     DMAMUX_1.CHCONF[8].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG10     DMAMUX_1.CHCONF[9].R          /* Channel Configuration */
#define DMAMUX_1_CHCFG9      DMAMUX_1.CHCONF[10].R         /* Channel Configuration */
#define DMAMUX_1_CHCFG8      DMAMUX_1.CHCONF[11].R         /* Channel Configuration */
#define DMAMUX_1_CHCFG15     DMAMUX_1.CHCONF[12].R         /* Channel Configuration */
#define DMAMUX_1_CHCFG14     DMAMUX_1.CHCONF[13].R         /* Channel Configuration */
#define DMAMUX_1_CHCFG13     DMAMUX_1.CHCONF[14].R         /* Channel Configuration */
#define DMAMUX_1_CHCFG12     DMAMUX_1.CHCONF[15].R         /* Channel Configuration */

#define DMAMUX_0A_CHCFG3     DMAMUX_0A.CHCONF[0].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG2     DMAMUX_0A.CHCONF[1].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG1     DMAMUX_0A.CHCONF[2].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG0     DMAMUX_0A.CHCONF[3].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG7     DMAMUX_0A.CHCONF[4].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG6     DMAMUX_0A.CHCONF[5].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG5     DMAMUX_0A.CHCONF[6].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG4     DMAMUX_0A.CHCONF[7].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG11    DMAMUX_0A.CHCONF[8].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG10    DMAMUX_0A.CHCONF[9].R         /* Channel Configuration */
#define DMAMUX_0A_CHCFG9     DMAMUX_0A.CHCONF[10].R        /* Channel Configuration */
#define DMAMUX_0A_CHCFG8     DMAMUX_0A.CHCONF[11].R        /* Channel Configuration */
#define DMAMUX_0A_CHCFG15    DMAMUX_0A.CHCONF[12].R        /* Channel Configuration */
#define DMAMUX_0A_CHCFG14    DMAMUX_0A.CHCONF[13].R        /* Channel Configuration */
#define DMAMUX_0A_CHCFG13    DMAMUX_0A.CHCONF[14].R        /* Channel Configuration */
#define DMAMUX_0A_CHCFG12    DMAMUX_0A.CHCONF[15].R        /* Channel Configuration */

#define DMAMUX_0B_CHCFG3     DMAMUX_0B.CHCONF[0].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG2     DMAMUX_0B.CHCONF[1].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG1     DMAMUX_0B.CHCONF[2].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG0     DMAMUX_0B.CHCONF[3].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG7     DMAMUX_0B.CHCONF[4].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG6     DMAMUX_0B.CHCONF[5].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG5     DMAMUX_0B.CHCONF[6].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG4     DMAMUX_0B.CHCONF[7].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG11    DMAMUX_0B.CHCONF[8].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG10    DMAMUX_0B.CHCONF[9].R         /* Channel Configuration */
#define DMAMUX_0B_CHCFG9     DMAMUX_0B.CHCONF[10].R        /* Channel Configuration */
#define DMAMUX_0B_CHCFG8     DMAMUX_0B.CHCONF[11].R        /* Channel Configuration */
#define DMAMUX_0B_CHCFG15    DMAMUX_0B.CHCONF[12].R        /* Channel Configuration */
#define DMAMUX_0B_CHCFG14    DMAMUX_0B.CHCONF[13].R        /* Channel Configuration */
#define DMAMUX_0B_CHCFG13    DMAMUX_0B.CHCONF[14].R        /* Channel Configuration */
#define DMAMUX_0B_CHCFG12    DMAMUX_0B.CHCONF[15].R        /* Channel Configuration */

#define DMAMUX_4_CHCFG3      DMAMUX_4.CHCONF[0].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG2      DMAMUX_4.CHCONF[1].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG1      DMAMUX_4.CHCONF[2].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG0      DMAMUX_4.CHCONF[3].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG7      DMAMUX_4.CHCONF[4].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG6      DMAMUX_4.CHCONF[5].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG5      DMAMUX_4.CHCONF[6].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG4      DMAMUX_4.CHCONF[7].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG11     DMAMUX_4.CHCONF[8].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG10     DMAMUX_4.CHCONF[9].R          /* Channel Configuration */
#define DMAMUX_4_CHCFG9      DMAMUX_4.CHCONF[10].R         /* Channel Configuration */
#define DMAMUX_4_CHCFG8      DMAMUX_4.CHCONF[11].R         /* Channel Configuration */
#define DMAMUX_4_CHCFG15     DMAMUX_4.CHCONF[12].R         /* Channel Configuration */
#define DMAMUX_4_CHCFG14     DMAMUX_4.CHCONF[13].R         /* Channel Configuration */
#define DMAMUX_4_CHCFG13     DMAMUX_4.CHCONF[14].R         /* Channel Configuration */
#define DMAMUX_4_CHCFG12     DMAMUX_4.CHCONF[15].R         /* Channel Configuration */

#define DMAMUX_5_CHCFG3      DMAMUX_5.CHCONF[0].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG2      DMAMUX_5.CHCONF[1].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG1      DMAMUX_5.CHCONF[2].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG0      DMAMUX_5.CHCONF[3].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG7      DMAMUX_5.CHCONF[4].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG6      DMAMUX_5.CHCONF[5].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG5      DMAMUX_5.CHCONF[6].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG4      DMAMUX_5.CHCONF[7].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG11     DMAMUX_5.CHCONF[8].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG10     DMAMUX_5.CHCONF[9].R          /* Channel Configuration */
#define DMAMUX_5_CHCFG9      DMAMUX_5.CHCONF[10].R         /* Channel Configuration */
#define DMAMUX_5_CHCFG8      DMAMUX_5.CHCONF[11].R         /* Channel Configuration */
#define DMAMUX_5_CHCFG15     DMAMUX_5.CHCONF[12].R         /* Channel Configuration */
#define DMAMUX_5_CHCFG14     DMAMUX_5.CHCONF[13].R         /* Channel Configuration */
#define DMAMUX_5_CHCFG13     DMAMUX_5.CHCONF[14].R         /* Channel Configuration */
#define DMAMUX_5_CHCFG12     DMAMUX_5.CHCONF[15].R         /* Channel Configuration */

/* DMA_CRC */
#define CE_DMA_CRC_GEC       CE_DMA_CRC.GEC.R              /* Global Enable CRC Register */
#define CE_DMA_CRC_CTL0      CE_DMA_CRC.CONTROL_REGISTER[0].CTL.R /* CRC Control Register */
#define CE_DMA_CRC_ICRC0     CE_DMA_CRC.CONTROL_REGISTER[0].ICRC.R /* Initial CRC Value Register */
#define CE_DMA_CRC_FCRC0     CE_DMA_CRC.CONTROL_REGISTER[0].FCRC.R /* Final CRC Value Register */
#define CE_DMA_CRC_CTL1      CE_DMA_CRC.CONTROL_REGISTER[1].CTL.R /* CRC Control Register */
#define CE_DMA_CRC_ICRC1     CE_DMA_CRC.CONTROL_REGISTER[1].ICRC.R /* Initial CRC Value Register */
#define CE_DMA_CRC_FCRC1     CE_DMA_CRC.CONTROL_REGISTER[1].FCRC.R /* Final CRC Value Register */
#define CE_DMA_CRC_CTL2      CE_DMA_CRC.CONTROL_REGISTER[2].CTL.R /* CRC Control Register */
#define CE_DMA_CRC_ICRC2     CE_DMA_CRC.CONTROL_REGISTER[2].ICRC.R /* Initial CRC Value Register */
#define CE_DMA_CRC_FCRC2     CE_DMA_CRC.CONTROL_REGISTER[2].FCRC.R /* Final CRC Value Register */
#define CE_DMA_CRC_CTL3      CE_DMA_CRC.CONTROL_REGISTER[3].CTL.R /* CRC Control Register */
#define CE_DMA_CRC_ICRC3     CE_DMA_CRC.CONTROL_REGISTER[3].ICRC.R /* Initial CRC Value Register */
#define CE_DMA_CRC_FCRC3     CE_DMA_CRC.CONTROL_REGISTER[3].FCRC.R /* Final CRC Value Register */
#define CE_DMA_CRC_CTL4      CE_DMA_CRC.CONTROL_REGISTER[4].CTL.R /* CRC Control Register */
#define CE_DMA_CRC_ICRC4     CE_DMA_CRC.CONTROL_REGISTER[4].ICRC.R /* Initial CRC Value Register */
#define CE_DMA_CRC_FCRC4     CE_DMA_CRC.CONTROL_REGISTER[4].FCRC.R /* Final CRC Value Register */
#define CE_DMA_CRC_CTL5      CE_DMA_CRC.CONTROL_REGISTER[5].CTL.R /* CRC Control Register */
#define CE_DMA_CRC_ICRC5     CE_DMA_CRC.CONTROL_REGISTER[5].ICRC.R /* Initial CRC Value Register */
#define CE_DMA_CRC_FCRC5     CE_DMA_CRC.CONTROL_REGISTER[5].FCRC.R /* Final CRC Value Register */
#define CE_DMA_CRC_CTL6      CE_DMA_CRC.CONTROL_REGISTER[6].CTL.R /* CRC Control Register */
#define CE_DMA_CRC_ICRC6     CE_DMA_CRC.CONTROL_REGISTER[6].ICRC.R /* Initial CRC Value Register */
#define CE_DMA_CRC_FCRC6     CE_DMA_CRC.CONTROL_REGISTER[6].FCRC.R /* Final CRC Value Register */
#define CE_DMA_CRC_CTL7      CE_DMA_CRC.CONTROL_REGISTER[7].CTL.R /* CRC Control Register */
#define CE_DMA_CRC_ICRC7     CE_DMA_CRC.CONTROL_REGISTER[7].ICRC.R /* Initial CRC Value Register */
#define CE_DMA_CRC_FCRC7     CE_DMA_CRC.CONTROL_REGISTER[7].FCRC.R /* Final CRC Value Register */

#define DMA_CRC_3_GEC        DMA_CRC_3.GEC.R               /* Global Enable CRC Register */
#define DMA_CRC_3_CTL0       DMA_CRC_3.CONTROL_REGISTER[0].CTL.R /* CRC Control Register */
#define DMA_CRC_3_ICRC0      DMA_CRC_3.CONTROL_REGISTER[0].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_3_FCRC0      DMA_CRC_3.CONTROL_REGISTER[0].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_3_CTL1       DMA_CRC_3.CONTROL_REGISTER[1].CTL.R /* CRC Control Register */
#define DMA_CRC_3_ICRC1      DMA_CRC_3.CONTROL_REGISTER[1].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_3_FCRC1      DMA_CRC_3.CONTROL_REGISTER[1].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_3_CTL2       DMA_CRC_3.CONTROL_REGISTER[2].CTL.R /* CRC Control Register */
#define DMA_CRC_3_ICRC2      DMA_CRC_3.CONTROL_REGISTER[2].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_3_FCRC2      DMA_CRC_3.CONTROL_REGISTER[2].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_3_CTL3       DMA_CRC_3.CONTROL_REGISTER[3].CTL.R /* CRC Control Register */
#define DMA_CRC_3_ICRC3      DMA_CRC_3.CONTROL_REGISTER[3].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_3_FCRC3      DMA_CRC_3.CONTROL_REGISTER[3].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_3_CTL4       DMA_CRC_3.CONTROL_REGISTER[4].CTL.R /* CRC Control Register */
#define DMA_CRC_3_ICRC4      DMA_CRC_3.CONTROL_REGISTER[4].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_3_FCRC4      DMA_CRC_3.CONTROL_REGISTER[4].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_3_CTL5       DMA_CRC_3.CONTROL_REGISTER[5].CTL.R /* CRC Control Register */
#define DMA_CRC_3_ICRC5      DMA_CRC_3.CONTROL_REGISTER[5].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_3_FCRC5      DMA_CRC_3.CONTROL_REGISTER[5].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_3_CTL6       DMA_CRC_3.CONTROL_REGISTER[6].CTL.R /* CRC Control Register */
#define DMA_CRC_3_ICRC6      DMA_CRC_3.CONTROL_REGISTER[6].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_3_FCRC6      DMA_CRC_3.CONTROL_REGISTER[6].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_3_CTL7       DMA_CRC_3.CONTROL_REGISTER[7].CTL.R /* CRC Control Register */
#define DMA_CRC_3_ICRC7      DMA_CRC_3.CONTROL_REGISTER[7].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_3_FCRC7      DMA_CRC_3.CONTROL_REGISTER[7].FCRC.R /* Final CRC Value Register */

#define DMA_CRC_1_GEC        DMA_CRC_1.GEC.R               /* Global Enable CRC Register */
#define DMA_CRC_1_CTL0       DMA_CRC_1.CONTROL_REGISTER[0].CTL.R /* CRC Control Register */
#define DMA_CRC_1_ICRC0      DMA_CRC_1.CONTROL_REGISTER[0].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_1_FCRC0      DMA_CRC_1.CONTROL_REGISTER[0].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_1_CTL1       DMA_CRC_1.CONTROL_REGISTER[1].CTL.R /* CRC Control Register */
#define DMA_CRC_1_ICRC1      DMA_CRC_1.CONTROL_REGISTER[1].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_1_FCRC1      DMA_CRC_1.CONTROL_REGISTER[1].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_1_CTL2       DMA_CRC_1.CONTROL_REGISTER[2].CTL.R /* CRC Control Register */
#define DMA_CRC_1_ICRC2      DMA_CRC_1.CONTROL_REGISTER[2].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_1_FCRC2      DMA_CRC_1.CONTROL_REGISTER[2].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_1_CTL3       DMA_CRC_1.CONTROL_REGISTER[3].CTL.R /* CRC Control Register */
#define DMA_CRC_1_ICRC3      DMA_CRC_1.CONTROL_REGISTER[3].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_1_FCRC3      DMA_CRC_1.CONTROL_REGISTER[3].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_1_CTL4       DMA_CRC_1.CONTROL_REGISTER[4].CTL.R /* CRC Control Register */
#define DMA_CRC_1_ICRC4      DMA_CRC_1.CONTROL_REGISTER[4].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_1_FCRC4      DMA_CRC_1.CONTROL_REGISTER[4].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_1_CTL5       DMA_CRC_1.CONTROL_REGISTER[5].CTL.R /* CRC Control Register */
#define DMA_CRC_1_ICRC5      DMA_CRC_1.CONTROL_REGISTER[5].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_1_FCRC5      DMA_CRC_1.CONTROL_REGISTER[5].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_1_CTL6       DMA_CRC_1.CONTROL_REGISTER[6].CTL.R /* CRC Control Register */
#define DMA_CRC_1_ICRC6      DMA_CRC_1.CONTROL_REGISTER[6].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_1_FCRC6      DMA_CRC_1.CONTROL_REGISTER[6].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_1_CTL7       DMA_CRC_1.CONTROL_REGISTER[7].CTL.R /* CRC Control Register */
#define DMA_CRC_1_ICRC7      DMA_CRC_1.CONTROL_REGISTER[7].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_1_FCRC7      DMA_CRC_1.CONTROL_REGISTER[7].FCRC.R /* Final CRC Value Register */

#define DMA_CRC_0_GEC        DMA_CRC_0.GEC.R               /* Global Enable CRC Register */
#define DMA_CRC_0_CTL0       DMA_CRC_0.CONTROL_REGISTER[0].CTL.R /* CRC Control Register */
#define DMA_CRC_0_ICRC0      DMA_CRC_0.CONTROL_REGISTER[0].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_0_FCRC0      DMA_CRC_0.CONTROL_REGISTER[0].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_0_CTL1       DMA_CRC_0.CONTROL_REGISTER[1].CTL.R /* CRC Control Register */
#define DMA_CRC_0_ICRC1      DMA_CRC_0.CONTROL_REGISTER[1].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_0_FCRC1      DMA_CRC_0.CONTROL_REGISTER[1].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_0_CTL2       DMA_CRC_0.CONTROL_REGISTER[2].CTL.R /* CRC Control Register */
#define DMA_CRC_0_ICRC2      DMA_CRC_0.CONTROL_REGISTER[2].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_0_FCRC2      DMA_CRC_0.CONTROL_REGISTER[2].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_0_CTL3       DMA_CRC_0.CONTROL_REGISTER[3].CTL.R /* CRC Control Register */
#define DMA_CRC_0_ICRC3      DMA_CRC_0.CONTROL_REGISTER[3].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_0_FCRC3      DMA_CRC_0.CONTROL_REGISTER[3].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_0_CTL4       DMA_CRC_0.CONTROL_REGISTER[4].CTL.R /* CRC Control Register */
#define DMA_CRC_0_ICRC4      DMA_CRC_0.CONTROL_REGISTER[4].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_0_FCRC4      DMA_CRC_0.CONTROL_REGISTER[4].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_0_CTL5       DMA_CRC_0.CONTROL_REGISTER[5].CTL.R /* CRC Control Register */
#define DMA_CRC_0_ICRC5      DMA_CRC_0.CONTROL_REGISTER[5].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_0_FCRC5      DMA_CRC_0.CONTROL_REGISTER[5].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_0_CTL6       DMA_CRC_0.CONTROL_REGISTER[6].CTL.R /* CRC Control Register */
#define DMA_CRC_0_ICRC6      DMA_CRC_0.CONTROL_REGISTER[6].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_0_FCRC6      DMA_CRC_0.CONTROL_REGISTER[6].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_0_CTL7       DMA_CRC_0.CONTROL_REGISTER[7].CTL.R /* CRC Control Register */
#define DMA_CRC_0_ICRC7      DMA_CRC_0.CONTROL_REGISTER[7].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_0_FCRC7      DMA_CRC_0.CONTROL_REGISTER[7].FCRC.R /* Final CRC Value Register */

#define DMA_CRC_4_GEC        DMA_CRC_4.GEC.R               /* Global Enable CRC Register */
#define DMA_CRC_4_CTL0       DMA_CRC_4.CONTROL_REGISTER[0].CTL.R /* CRC Control Register */
#define DMA_CRC_4_ICRC0      DMA_CRC_4.CONTROL_REGISTER[0].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_4_FCRC0      DMA_CRC_4.CONTROL_REGISTER[0].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_4_CTL1       DMA_CRC_4.CONTROL_REGISTER[1].CTL.R /* CRC Control Register */
#define DMA_CRC_4_ICRC1      DMA_CRC_4.CONTROL_REGISTER[1].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_4_FCRC1      DMA_CRC_4.CONTROL_REGISTER[1].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_4_CTL2       DMA_CRC_4.CONTROL_REGISTER[2].CTL.R /* CRC Control Register */
#define DMA_CRC_4_ICRC2      DMA_CRC_4.CONTROL_REGISTER[2].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_4_FCRC2      DMA_CRC_4.CONTROL_REGISTER[2].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_4_CTL3       DMA_CRC_4.CONTROL_REGISTER[3].CTL.R /* CRC Control Register */
#define DMA_CRC_4_ICRC3      DMA_CRC_4.CONTROL_REGISTER[3].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_4_FCRC3      DMA_CRC_4.CONTROL_REGISTER[3].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_4_CTL4       DMA_CRC_4.CONTROL_REGISTER[4].CTL.R /* CRC Control Register */
#define DMA_CRC_4_ICRC4      DMA_CRC_4.CONTROL_REGISTER[4].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_4_FCRC4      DMA_CRC_4.CONTROL_REGISTER[4].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_4_CTL5       DMA_CRC_4.CONTROL_REGISTER[5].CTL.R /* CRC Control Register */
#define DMA_CRC_4_ICRC5      DMA_CRC_4.CONTROL_REGISTER[5].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_4_FCRC5      DMA_CRC_4.CONTROL_REGISTER[5].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_4_CTL6       DMA_CRC_4.CONTROL_REGISTER[6].CTL.R /* CRC Control Register */
#define DMA_CRC_4_ICRC6      DMA_CRC_4.CONTROL_REGISTER[6].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_4_FCRC6      DMA_CRC_4.CONTROL_REGISTER[6].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_4_CTL7       DMA_CRC_4.CONTROL_REGISTER[7].CTL.R /* CRC Control Register */
#define DMA_CRC_4_ICRC7      DMA_CRC_4.CONTROL_REGISTER[7].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_4_FCRC7      DMA_CRC_4.CONTROL_REGISTER[7].FCRC.R /* Final CRC Value Register */

#define DMA_CRC_5_GEC        DMA_CRC_5.GEC.R               /* Global Enable CRC Register */
#define DMA_CRC_5_CTL0       DMA_CRC_5.CONTROL_REGISTER[0].CTL.R /* CRC Control Register */
#define DMA_CRC_5_ICRC0      DMA_CRC_5.CONTROL_REGISTER[0].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_5_FCRC0      DMA_CRC_5.CONTROL_REGISTER[0].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_5_CTL1       DMA_CRC_5.CONTROL_REGISTER[1].CTL.R /* CRC Control Register */
#define DMA_CRC_5_ICRC1      DMA_CRC_5.CONTROL_REGISTER[1].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_5_FCRC1      DMA_CRC_5.CONTROL_REGISTER[1].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_5_CTL2       DMA_CRC_5.CONTROL_REGISTER[2].CTL.R /* CRC Control Register */
#define DMA_CRC_5_ICRC2      DMA_CRC_5.CONTROL_REGISTER[2].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_5_FCRC2      DMA_CRC_5.CONTROL_REGISTER[2].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_5_CTL3       DMA_CRC_5.CONTROL_REGISTER[3].CTL.R /* CRC Control Register */
#define DMA_CRC_5_ICRC3      DMA_CRC_5.CONTROL_REGISTER[3].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_5_FCRC3      DMA_CRC_5.CONTROL_REGISTER[3].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_5_CTL4       DMA_CRC_5.CONTROL_REGISTER[4].CTL.R /* CRC Control Register */
#define DMA_CRC_5_ICRC4      DMA_CRC_5.CONTROL_REGISTER[4].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_5_FCRC4      DMA_CRC_5.CONTROL_REGISTER[4].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_5_CTL5       DMA_CRC_5.CONTROL_REGISTER[5].CTL.R /* CRC Control Register */
#define DMA_CRC_5_ICRC5      DMA_CRC_5.CONTROL_REGISTER[5].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_5_FCRC5      DMA_CRC_5.CONTROL_REGISTER[5].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_5_CTL6       DMA_CRC_5.CONTROL_REGISTER[6].CTL.R /* CRC Control Register */
#define DMA_CRC_5_ICRC6      DMA_CRC_5.CONTROL_REGISTER[6].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_5_FCRC6      DMA_CRC_5.CONTROL_REGISTER[6].FCRC.R /* Final CRC Value Register */
#define DMA_CRC_5_CTL7       DMA_CRC_5.CONTROL_REGISTER[7].CTL.R /* CRC Control Register */
#define DMA_CRC_5_ICRC7      DMA_CRC_5.CONTROL_REGISTER[7].ICRC.R /* Initial CRC Value Register */
#define DMA_CRC_5_FCRC7      DMA_CRC_5.CONTROL_REGISTER[7].FCRC.R /* Final CRC Value Register */

/* DMSS_SAFETY */
#define CEVA_SPF2__DMSS_SAFETY_D_UCSERR CEVA_SPF2__DMSS_SAFETY.D_UCSERR.R /* DMSS UCS ERR Register */
#define CEVA_SPF2__DMSS_SAFETY_D_CSERR CEVA_SPF2__DMSS_SAFETY.D_CSERR.R /* DMSS CS ERR Register */
#define CEVA_SPF2__DMSS_SAFETY_D_UCSERR_S CEVA_SPF2__DMSS_SAFETY.D_UCSERR_S.R /* DMSS UCS ERR Shadow Register */
#define CEVA_SPF2__DMSS_SAFETY_D_CSERR_S CEVA_SPF2__DMSS_SAFETY.D_CSERR_S.R /* DMSS CS ERR Shadow Register */
#define CEVA_SPF2__DMSS_SAFETY_D_CSERR_M CEVA_SPF2__DMSS_SAFETY.D_CSERR_M.R /* DMSS CS ERR Mask Register */
#define CEVA_SPF2__DMSS_SAFETY_D_ECC_TCFG CEVA_SPF2__DMSS_SAFETY.D_ECC_TCFG.R /* DMSS ECC TCFG Register */
#define CEVA_SPF2__DMSS_SAFETY_D_ECC_TCFG_2 CEVA_SPF2__DMSS_SAFETY.D_ECC_TCFG_2.R /* DMSS ECC TCFG 2 Register */
#define CEVA_SPF2__DMSS_SAFETY_D_SFT_SCPD_0 CEVA_SPF2__DMSS_SAFETY.D_SFT_SCPD_0.R /* DMSS SFT SCPD 0 Register */
#define CEVA_SPF2__DMSS_SAFETY_D_SFT_SCPD_1 CEVA_SPF2__DMSS_SAFETY.D_SFT_SCPD_1.R /* DMSS SFT SCPD 1 Register */
#define CEVA_SPF2__DMSS_SAFETY_D_SFT_SCPD_2 CEVA_SPF2__DMSS_SAFETY.D_SFT_SCPD_2.R /* DMSS SFT SCPD 2 Register */
#define CEVA_SPF2__DMSS_SAFETY_D_SFT_SCPD_3 CEVA_SPF2__DMSS_SAFETY.D_SFT_SCPD_3.R /* DMSS SFT SCPD 3 Register */
#define CEVA_SPF2__DMSS_SAFETY_D_UCSERR_CNT CEVA_SPF2__DMSS_SAFETY.D_UCSERR_CNT.R /* DMSS UCS ERR Register */
#define CEVA_SPF2__DMSS_SAFETY_D_CSERR_CNT CEVA_SPF2__DMSS_SAFETY.D_CSERR_CNT.R /* DMSS CS ERR CNT Register */

/* DSPI_10 */
#define MSC_0_DSPI_MCR       MSC_0_DSPI.MCR.R              /* Module Configuration Register */
#define MSC_0_DSPI_TCR       MSC_0_DSPI.TCR.R              /* Transfer Count Register */
#define MSC_0_DSPI_CTAR0     MSC_0_DSPI.CTAR[0].R          /* Clock and Transfer Attributes Register (In Master Mode) */
#define MSC_0_DSPI_CTAR1     MSC_0_DSPI.CTAR[1].R          /* Clock and Transfer Attributes Register (In Master Mode) */
#define MSC_0_DSPI_CTAR2     MSC_0_DSPI.CTAR[2].R          /* Clock and Transfer Attributes Register (In Master Mode) */
#define MSC_0_DSPI_CTAR3     MSC_0_DSPI.CTAR[3].R          /* Clock and Transfer Attributes Register (In Master Mode) */
#define MSC_0_DSPI_CTAR4     MSC_0_DSPI.CTAR[4].R          /* Clock and Transfer Attributes Register (In Master Mode) */
#define MSC_0_DSPI_CTAR5     MSC_0_DSPI.CTAR[5].R          /* Clock and Transfer Attributes Register (In Master Mode) */
#define MSC_0_DSPI_SR        MSC_0_DSPI.SR.R               /* Status Register */
#define MSC_0_DSPI_RSER      MSC_0_DSPI.RSER.R             /* DMA/Interrupt Request Select and Enable Register */
#define MSC_0_DSPI_PUSHR     MSC_0_DSPI.PUSHR.R            /* PUSH TX FIFO Register In Master Mode */
#define MSC_0_DSPI_POPR      MSC_0_DSPI.POPR.R             /* POP RX FIFO Register */
#define MSC_0_DSPI_TXFR0     MSC_0_DSPI.TXFR[0].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR1     MSC_0_DSPI.TXFR[1].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR2     MSC_0_DSPI.TXFR[2].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR3     MSC_0_DSPI.TXFR[3].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR4     MSC_0_DSPI.TXFR[4].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR5     MSC_0_DSPI.TXFR[5].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR6     MSC_0_DSPI.TXFR[6].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR7     MSC_0_DSPI.TXFR[7].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR8     MSC_0_DSPI.TXFR[8].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR9     MSC_0_DSPI.TXFR[9].R          /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR10    MSC_0_DSPI.TXFR[10].R         /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR11    MSC_0_DSPI.TXFR[11].R         /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR12    MSC_0_DSPI.TXFR[12].R         /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR13    MSC_0_DSPI.TXFR[13].R         /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR14    MSC_0_DSPI.TXFR[14].R         /* Transmit FIFO Registers */
#define MSC_0_DSPI_TXFR15    MSC_0_DSPI.TXFR[15].R         /* Transmit FIFO Registers */
#define MSC_0_DSPI_RXFR0     MSC_0_DSPI.RXFR[0].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR1     MSC_0_DSPI.RXFR[1].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR2     MSC_0_DSPI.RXFR[2].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR3     MSC_0_DSPI.RXFR[3].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR4     MSC_0_DSPI.RXFR[4].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR5     MSC_0_DSPI.RXFR[5].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR6     MSC_0_DSPI.RXFR[6].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR7     MSC_0_DSPI.RXFR[7].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR8     MSC_0_DSPI.RXFR[8].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR9     MSC_0_DSPI.RXFR[9].R          /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR10    MSC_0_DSPI.RXFR[10].R         /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR11    MSC_0_DSPI.RXFR[11].R         /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR12    MSC_0_DSPI.RXFR[12].R         /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR13    MSC_0_DSPI.RXFR[13].R         /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR14    MSC_0_DSPI.RXFR[14].R         /* Receive FIFO Registers */
#define MSC_0_DSPI_RXFR15    MSC_0_DSPI.RXFR[15].R         /* Receive FIFO Registers */
#define MSC_0_DSPI_DSICR0    MSC_0_DSPI.DSICR0.R           /* DSI Configuration Register 0 */
#define MSC_0_DSPI_SDR0      MSC_0_DSPI.SDR0.R             /* DSI Serialization Data Register 0 */
#define MSC_0_DSPI_ASDR0     MSC_0_DSPI.ASDR0.R            /* DSI Alternate Serialization Data Register 0 */
#define MSC_0_DSPI_COMPR0    MSC_0_DSPI.COMPR0.R           /* DSI Transmit Comparison Register 0 */
#define MSC_0_DSPI_DDR0      MSC_0_DSPI.DDR0.R             /* DSI Deserialization Data Register 0 */
#define MSC_0_DSPI_DSICR1    MSC_0_DSPI.DSICR1.R           /* DSI Configuration Register 1 */
#define MSC_0_DSPI_SSR0      MSC_0_DSPI.SSR0.R             /* DSI Serialization Source Select Register 0 */
#define MSC_0_DSPI_DIMR0     MSC_0_DSPI.DIMR0.R            /* DSI Deserialized Data Interrupt Mask Register 0 */
#define MSC_0_DSPI_DPIR0     MSC_0_DSPI.DPIR0.R            /* DSI Deserialized Data Polarity Interrupt Register 0 */
#define MSC_0_DSPI_SDR1      MSC_0_DSPI.SDR1.R             /* DSI Serialization Data Register 1 */
#define MSC_0_DSPI_ASDR1     MSC_0_DSPI.ASDR1.R            /* DSI Alternate Serialization Data Register 1 */
#define MSC_0_DSPI_COMPR1    MSC_0_DSPI.COMPR1.R           /* DSI Transmit Comparison Register 1 */
#define MSC_0_DSPI_DDR1      MSC_0_DSPI.DDR1.R             /* DSI Deserialization Data Register 1 */
#define MSC_0_DSPI_SSR1      MSC_0_DSPI.SSR1.R             /* DSI Serialization Source Select Register 1 */
#define MSC_0_DSPI_DIMR1     MSC_0_DSPI.DIMR1.R            /* DSI Deserialized Data Interrupt Mask Register 1 */
#define MSC_0_DSPI_DPIR1     MSC_0_DSPI.DPIR1.R            /* DSI Deserialized Data Polarity Interrupt Register 1 */
#define MSC_0_DSPI_CTARE0    MSC_0_DSPI.CTARE[0].R         /* Clock and Transfer Attributes Register Extended */
#define MSC_0_DSPI_CTARE1    MSC_0_DSPI.CTARE[1].R         /* Clock and Transfer Attributes Register Extended */
#define MSC_0_DSPI_CTARE2    MSC_0_DSPI.CTARE[2].R         /* Clock and Transfer Attributes Register Extended */
#define MSC_0_DSPI_CTARE3    MSC_0_DSPI.CTARE[3].R         /* Clock and Transfer Attributes Register Extended */
#define MSC_0_DSPI_CTARE4    MSC_0_DSPI.CTARE[4].R         /* Clock and Transfer Attributes Register Extended */
#define MSC_0_DSPI_CTARE5    MSC_0_DSPI.CTARE[5].R         /* Clock and Transfer Attributes Register Extended */
#define MSC_0_DSPI_SREX      MSC_0_DSPI.SREX.R             /* Status Register Extended */
#define MSC_0_DSPI_TSL       MSC_0_DSPI.TSL.R              /* Time Slot Length Register */
#define MSC_0_DSPI_TS_CONF   MSC_0_DSPI.TS_CONF.R          /* Time Slot Configuration Register */

/* EDMA3_MP */
#define EDMA_0_MP_CSR        EDMA_0_MP.CSR.R               /* Management Page Control */
#define EDMA_0_MP_ES         EDMA_0_MP.ES.R                /* Management Page Error Status */
#define EDMA_0_MP_INT        EDMA_0_MP.INT.R               /* Management Page Interrupt Request Status */
#define EDMA_0_MP_HRS        EDMA_0_MP.HRS.R               /* Management Page Hardware Request Status */
#define EDMA_0_MP_CH0_GRPRI  EDMA_0_MP.CH_GRPRI[0].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH1_GRPRI  EDMA_0_MP.CH_GRPRI[1].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH2_GRPRI  EDMA_0_MP.CH_GRPRI[2].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH3_GRPRI  EDMA_0_MP.CH_GRPRI[3].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH4_GRPRI  EDMA_0_MP.CH_GRPRI[4].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH5_GRPRI  EDMA_0_MP.CH_GRPRI[5].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH6_GRPRI  EDMA_0_MP.CH_GRPRI[6].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH7_GRPRI  EDMA_0_MP.CH_GRPRI[7].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH8_GRPRI  EDMA_0_MP.CH_GRPRI[8].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH9_GRPRI  EDMA_0_MP.CH_GRPRI[9].R       /* Channel Arbitration Group */
#define EDMA_0_MP_CH10_GRPRI EDMA_0_MP.CH_GRPRI[10].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH11_GRPRI EDMA_0_MP.CH_GRPRI[11].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH12_GRPRI EDMA_0_MP.CH_GRPRI[12].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH13_GRPRI EDMA_0_MP.CH_GRPRI[13].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH14_GRPRI EDMA_0_MP.CH_GRPRI[14].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH15_GRPRI EDMA_0_MP.CH_GRPRI[15].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH16_GRPRI EDMA_0_MP.CH_GRPRI[16].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH17_GRPRI EDMA_0_MP.CH_GRPRI[17].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH18_GRPRI EDMA_0_MP.CH_GRPRI[18].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH19_GRPRI EDMA_0_MP.CH_GRPRI[19].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH20_GRPRI EDMA_0_MP.CH_GRPRI[20].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH21_GRPRI EDMA_0_MP.CH_GRPRI[21].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH22_GRPRI EDMA_0_MP.CH_GRPRI[22].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH23_GRPRI EDMA_0_MP.CH_GRPRI[23].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH24_GRPRI EDMA_0_MP.CH_GRPRI[24].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH25_GRPRI EDMA_0_MP.CH_GRPRI[25].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH26_GRPRI EDMA_0_MP.CH_GRPRI[26].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH27_GRPRI EDMA_0_MP.CH_GRPRI[27].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH28_GRPRI EDMA_0_MP.CH_GRPRI[28].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH29_GRPRI EDMA_0_MP.CH_GRPRI[29].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH30_GRPRI EDMA_0_MP.CH_GRPRI[30].R      /* Channel Arbitration Group */
#define EDMA_0_MP_CH31_GRPRI EDMA_0_MP.CH_GRPRI[31].R      /* Channel Arbitration Group */

#define EDMA_1_MP_CSR        EDMA_1_MP.CSR.R               /* Management Page Control */
#define EDMA_1_MP_ES         EDMA_1_MP.ES.R                /* Management Page Error Status */
#define EDMA_1_MP_INT        EDMA_1_MP.INT.R               /* Management Page Interrupt Request Status */
#define EDMA_1_MP_HRS        EDMA_1_MP.HRS.R               /* Management Page Hardware Request Status */
#define EDMA_1_MP_CH0_GRPRI  EDMA_1_MP.CH_GRPRI[0].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH1_GRPRI  EDMA_1_MP.CH_GRPRI[1].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH2_GRPRI  EDMA_1_MP.CH_GRPRI[2].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH3_GRPRI  EDMA_1_MP.CH_GRPRI[3].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH4_GRPRI  EDMA_1_MP.CH_GRPRI[4].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH5_GRPRI  EDMA_1_MP.CH_GRPRI[5].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH6_GRPRI  EDMA_1_MP.CH_GRPRI[6].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH7_GRPRI  EDMA_1_MP.CH_GRPRI[7].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH8_GRPRI  EDMA_1_MP.CH_GRPRI[8].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH9_GRPRI  EDMA_1_MP.CH_GRPRI[9].R       /* Channel Arbitration Group */
#define EDMA_1_MP_CH10_GRPRI EDMA_1_MP.CH_GRPRI[10].R      /* Channel Arbitration Group */
#define EDMA_1_MP_CH11_GRPRI EDMA_1_MP.CH_GRPRI[11].R      /* Channel Arbitration Group */
#define EDMA_1_MP_CH12_GRPRI EDMA_1_MP.CH_GRPRI[12].R      /* Channel Arbitration Group */
#define EDMA_1_MP_CH13_GRPRI EDMA_1_MP.CH_GRPRI[13].R      /* Channel Arbitration Group */
#define EDMA_1_MP_CH14_GRPRI EDMA_1_MP.CH_GRPRI[14].R      /* Channel Arbitration Group */
#define EDMA_1_MP_CH15_GRPRI EDMA_1_MP.CH_GRPRI[15].R      /* Channel Arbitration Group */

#define EDMA_4_MP_CSR        EDMA_4_MP.CSR.R               /* Management Page Control */
#define EDMA_4_MP_ES         EDMA_4_MP.ES.R                /* Management Page Error Status */
#define EDMA_4_MP_INT        EDMA_4_MP.INT.R               /* Management Page Interrupt Request Status */
#define EDMA_4_MP_HRS        EDMA_4_MP.HRS.R               /* Management Page Hardware Request Status */
#define EDMA_4_MP_CH0_GRPRI  EDMA_4_MP.CH_GRPRI[0].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH1_GRPRI  EDMA_4_MP.CH_GRPRI[1].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH2_GRPRI  EDMA_4_MP.CH_GRPRI[2].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH3_GRPRI  EDMA_4_MP.CH_GRPRI[3].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH4_GRPRI  EDMA_4_MP.CH_GRPRI[4].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH5_GRPRI  EDMA_4_MP.CH_GRPRI[5].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH6_GRPRI  EDMA_4_MP.CH_GRPRI[6].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH7_GRPRI  EDMA_4_MP.CH_GRPRI[7].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH8_GRPRI  EDMA_4_MP.CH_GRPRI[8].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH9_GRPRI  EDMA_4_MP.CH_GRPRI[9].R       /* Channel Arbitration Group */
#define EDMA_4_MP_CH10_GRPRI EDMA_4_MP.CH_GRPRI[10].R      /* Channel Arbitration Group */
#define EDMA_4_MP_CH11_GRPRI EDMA_4_MP.CH_GRPRI[11].R      /* Channel Arbitration Group */
#define EDMA_4_MP_CH12_GRPRI EDMA_4_MP.CH_GRPRI[12].R      /* Channel Arbitration Group */
#define EDMA_4_MP_CH13_GRPRI EDMA_4_MP.CH_GRPRI[13].R      /* Channel Arbitration Group */
#define EDMA_4_MP_CH14_GRPRI EDMA_4_MP.CH_GRPRI[14].R      /* Channel Arbitration Group */
#define EDMA_4_MP_CH15_GRPRI EDMA_4_MP.CH_GRPRI[15].R      /* Channel Arbitration Group */

#define EDMA_5_MP_CSR        EDMA_5_MP.CSR.R               /* Management Page Control */
#define EDMA_5_MP_ES         EDMA_5_MP.ES.R                /* Management Page Error Status */
#define EDMA_5_MP_INT        EDMA_5_MP.INT.R               /* Management Page Interrupt Request Status */
#define EDMA_5_MP_HRS        EDMA_5_MP.HRS.R               /* Management Page Hardware Request Status */
#define EDMA_5_MP_CH0_GRPRI  EDMA_5_MP.CH_GRPRI[0].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH1_GRPRI  EDMA_5_MP.CH_GRPRI[1].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH2_GRPRI  EDMA_5_MP.CH_GRPRI[2].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH3_GRPRI  EDMA_5_MP.CH_GRPRI[3].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH4_GRPRI  EDMA_5_MP.CH_GRPRI[4].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH5_GRPRI  EDMA_5_MP.CH_GRPRI[5].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH6_GRPRI  EDMA_5_MP.CH_GRPRI[6].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH7_GRPRI  EDMA_5_MP.CH_GRPRI[7].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH8_GRPRI  EDMA_5_MP.CH_GRPRI[8].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH9_GRPRI  EDMA_5_MP.CH_GRPRI[9].R       /* Channel Arbitration Group */
#define EDMA_5_MP_CH10_GRPRI EDMA_5_MP.CH_GRPRI[10].R      /* Channel Arbitration Group */
#define EDMA_5_MP_CH11_GRPRI EDMA_5_MP.CH_GRPRI[11].R      /* Channel Arbitration Group */
#define EDMA_5_MP_CH12_GRPRI EDMA_5_MP.CH_GRPRI[12].R      /* Channel Arbitration Group */
#define EDMA_5_MP_CH13_GRPRI EDMA_5_MP.CH_GRPRI[13].R      /* Channel Arbitration Group */
#define EDMA_5_MP_CH14_GRPRI EDMA_5_MP.CH_GRPRI[14].R      /* Channel Arbitration Group */
#define EDMA_5_MP_CH15_GRPRI EDMA_5_MP.CH_GRPRI[15].R      /* Channel Arbitration Group */

#define CE_EDMA_MP_CSR       CE_EDMA_MP.CSR.R              /* Management Page Control */
#define CE_EDMA_MP_ES        CE_EDMA_MP.ES.R               /* Management Page Error Status */
#define CE_EDMA_MP_INT       CE_EDMA_MP.INT.R              /* Management Page Interrupt Request Status */
#define CE_EDMA_MP_HRS       CE_EDMA_MP.HRS.R              /* Management Page Hardware Request Status */
#define CE_EDMA_MP_CH0_GRPRI CE_EDMA_MP.CH_GRPRI[0].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH1_GRPRI CE_EDMA_MP.CH_GRPRI[1].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH2_GRPRI CE_EDMA_MP.CH_GRPRI[2].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH3_GRPRI CE_EDMA_MP.CH_GRPRI[3].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH4_GRPRI CE_EDMA_MP.CH_GRPRI[4].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH5_GRPRI CE_EDMA_MP.CH_GRPRI[5].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH6_GRPRI CE_EDMA_MP.CH_GRPRI[6].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH7_GRPRI CE_EDMA_MP.CH_GRPRI[7].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH8_GRPRI CE_EDMA_MP.CH_GRPRI[8].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH9_GRPRI CE_EDMA_MP.CH_GRPRI[9].R      /* Channel Arbitration Group */
#define CE_EDMA_MP_CH10_GRPRI CE_EDMA_MP.CH_GRPRI[10].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH11_GRPRI CE_EDMA_MP.CH_GRPRI[11].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH12_GRPRI CE_EDMA_MP.CH_GRPRI[12].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH13_GRPRI CE_EDMA_MP.CH_GRPRI[13].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH14_GRPRI CE_EDMA_MP.CH_GRPRI[14].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH15_GRPRI CE_EDMA_MP.CH_GRPRI[15].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH16_GRPRI CE_EDMA_MP.CH_GRPRI[16].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH17_GRPRI CE_EDMA_MP.CH_GRPRI[17].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH18_GRPRI CE_EDMA_MP.CH_GRPRI[18].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH19_GRPRI CE_EDMA_MP.CH_GRPRI[19].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH20_GRPRI CE_EDMA_MP.CH_GRPRI[20].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH21_GRPRI CE_EDMA_MP.CH_GRPRI[21].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH22_GRPRI CE_EDMA_MP.CH_GRPRI[22].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH23_GRPRI CE_EDMA_MP.CH_GRPRI[23].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH24_GRPRI CE_EDMA_MP.CH_GRPRI[24].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH25_GRPRI CE_EDMA_MP.CH_GRPRI[25].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH26_GRPRI CE_EDMA_MP.CH_GRPRI[26].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH27_GRPRI CE_EDMA_MP.CH_GRPRI[27].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH28_GRPRI CE_EDMA_MP.CH_GRPRI[28].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH29_GRPRI CE_EDMA_MP.CH_GRPRI[29].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH30_GRPRI CE_EDMA_MP.CH_GRPRI[30].R    /* Channel Arbitration Group */
#define CE_EDMA_MP_CH31_GRPRI CE_EDMA_MP.CH_GRPRI[31].R    /* Channel Arbitration Group */

/* EDMA3_TCD */
#define EDMA_0_TCD_CH0_CSR   EDMA_0_TCD.TCD[0].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH0_ES    EDMA_0_TCD.TCD[0].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH0_INT   EDMA_0_TCD.TCD[0].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH0_SBR   EDMA_0_TCD.TCD[0].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH0_PRI   EDMA_0_TCD.TCD[0].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD0_SADDR EDMA_0_TCD.TCD[0].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD0_SOFF EDMA_0_TCD.TCD[0].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD0_ATTR EDMA_0_TCD.TCD[0].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD0_NBYTES_MLOFFNO EDMA_0_TCD.TCD[0].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD0_NBYTES_MLOFFYES EDMA_0_TCD.TCD[0].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD0_SLAST_SDA EDMA_0_TCD.TCD[0].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD0_DADDR EDMA_0_TCD.TCD[0].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD0_DOFF EDMA_0_TCD.TCD[0].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD0_CITER_ELINKNO EDMA_0_TCD.TCD[0].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD0_CITER_ELINKYES EDMA_0_TCD.TCD[0].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD0_DLAST_SGA EDMA_0_TCD.TCD[0].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD0_CSR  EDMA_0_TCD.TCD[0].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD0_BITER_ELINKNO EDMA_0_TCD.TCD[0].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD0_BITER_ELINKYES EDMA_0_TCD.TCD[0].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH1_CSR   EDMA_0_TCD.TCD[1].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH1_ES    EDMA_0_TCD.TCD[1].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH1_INT   EDMA_0_TCD.TCD[1].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH1_SBR   EDMA_0_TCD.TCD[1].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH1_PRI   EDMA_0_TCD.TCD[1].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD1_SADDR EDMA_0_TCD.TCD[1].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD1_SOFF EDMA_0_TCD.TCD[1].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD1_ATTR EDMA_0_TCD.TCD[1].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD1_NBYTES_MLOFFNO EDMA_0_TCD.TCD[1].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD1_NBYTES_MLOFFYES EDMA_0_TCD.TCD[1].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD1_SLAST_SDA EDMA_0_TCD.TCD[1].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD1_DADDR EDMA_0_TCD.TCD[1].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD1_DOFF EDMA_0_TCD.TCD[1].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD1_CITER_ELINKNO EDMA_0_TCD.TCD[1].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD1_CITER_ELINKYES EDMA_0_TCD.TCD[1].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD1_DLAST_SGA EDMA_0_TCD.TCD[1].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD1_CSR  EDMA_0_TCD.TCD[1].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD1_BITER_ELINKNO EDMA_0_TCD.TCD[1].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD1_BITER_ELINKYES EDMA_0_TCD.TCD[1].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH2_CSR   EDMA_0_TCD.TCD[2].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH2_ES    EDMA_0_TCD.TCD[2].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH2_INT   EDMA_0_TCD.TCD[2].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH2_SBR   EDMA_0_TCD.TCD[2].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH2_PRI   EDMA_0_TCD.TCD[2].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD2_SADDR EDMA_0_TCD.TCD[2].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD2_SOFF EDMA_0_TCD.TCD[2].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD2_ATTR EDMA_0_TCD.TCD[2].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD2_NBYTES_MLOFFNO EDMA_0_TCD.TCD[2].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD2_NBYTES_MLOFFYES EDMA_0_TCD.TCD[2].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD2_SLAST_SDA EDMA_0_TCD.TCD[2].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD2_DADDR EDMA_0_TCD.TCD[2].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD2_DOFF EDMA_0_TCD.TCD[2].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD2_CITER_ELINKNO EDMA_0_TCD.TCD[2].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD2_CITER_ELINKYES EDMA_0_TCD.TCD[2].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD2_DLAST_SGA EDMA_0_TCD.TCD[2].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD2_CSR  EDMA_0_TCD.TCD[2].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD2_BITER_ELINKNO EDMA_0_TCD.TCD[2].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD2_BITER_ELINKYES EDMA_0_TCD.TCD[2].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH3_CSR   EDMA_0_TCD.TCD[3].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH3_ES    EDMA_0_TCD.TCD[3].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH3_INT   EDMA_0_TCD.TCD[3].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH3_SBR   EDMA_0_TCD.TCD[3].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH3_PRI   EDMA_0_TCD.TCD[3].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD3_SADDR EDMA_0_TCD.TCD[3].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD3_SOFF EDMA_0_TCD.TCD[3].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD3_ATTR EDMA_0_TCD.TCD[3].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD3_NBYTES_MLOFFNO EDMA_0_TCD.TCD[3].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD3_NBYTES_MLOFFYES EDMA_0_TCD.TCD[3].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD3_SLAST_SDA EDMA_0_TCD.TCD[3].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD3_DADDR EDMA_0_TCD.TCD[3].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD3_DOFF EDMA_0_TCD.TCD[3].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD3_CITER_ELINKNO EDMA_0_TCD.TCD[3].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD3_CITER_ELINKYES EDMA_0_TCD.TCD[3].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD3_DLAST_SGA EDMA_0_TCD.TCD[3].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD3_CSR  EDMA_0_TCD.TCD[3].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD3_BITER_ELINKNO EDMA_0_TCD.TCD[3].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD3_BITER_ELINKYES EDMA_0_TCD.TCD[3].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH4_CSR   EDMA_0_TCD.TCD[4].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH4_ES    EDMA_0_TCD.TCD[4].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH4_INT   EDMA_0_TCD.TCD[4].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH4_SBR   EDMA_0_TCD.TCD[4].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH4_PRI   EDMA_0_TCD.TCD[4].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD4_SADDR EDMA_0_TCD.TCD[4].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD4_SOFF EDMA_0_TCD.TCD[4].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD4_ATTR EDMA_0_TCD.TCD[4].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD4_NBYTES_MLOFFNO EDMA_0_TCD.TCD[4].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD4_NBYTES_MLOFFYES EDMA_0_TCD.TCD[4].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD4_SLAST_SDA EDMA_0_TCD.TCD[4].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD4_DADDR EDMA_0_TCD.TCD[4].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD4_DOFF EDMA_0_TCD.TCD[4].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD4_CITER_ELINKNO EDMA_0_TCD.TCD[4].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD4_CITER_ELINKYES EDMA_0_TCD.TCD[4].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD4_DLAST_SGA EDMA_0_TCD.TCD[4].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD4_CSR  EDMA_0_TCD.TCD[4].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD4_BITER_ELINKNO EDMA_0_TCD.TCD[4].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD4_BITER_ELINKYES EDMA_0_TCD.TCD[4].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH5_CSR   EDMA_0_TCD.TCD[5].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH5_ES    EDMA_0_TCD.TCD[5].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH5_INT   EDMA_0_TCD.TCD[5].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH5_SBR   EDMA_0_TCD.TCD[5].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH5_PRI   EDMA_0_TCD.TCD[5].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD5_SADDR EDMA_0_TCD.TCD[5].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD5_SOFF EDMA_0_TCD.TCD[5].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD5_ATTR EDMA_0_TCD.TCD[5].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD5_NBYTES_MLOFFNO EDMA_0_TCD.TCD[5].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD5_NBYTES_MLOFFYES EDMA_0_TCD.TCD[5].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD5_SLAST_SDA EDMA_0_TCD.TCD[5].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD5_DADDR EDMA_0_TCD.TCD[5].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD5_DOFF EDMA_0_TCD.TCD[5].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD5_CITER_ELINKNO EDMA_0_TCD.TCD[5].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD5_CITER_ELINKYES EDMA_0_TCD.TCD[5].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD5_DLAST_SGA EDMA_0_TCD.TCD[5].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD5_CSR  EDMA_0_TCD.TCD[5].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD5_BITER_ELINKNO EDMA_0_TCD.TCD[5].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD5_BITER_ELINKYES EDMA_0_TCD.TCD[5].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH6_CSR   EDMA_0_TCD.TCD[6].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH6_ES    EDMA_0_TCD.TCD[6].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH6_INT   EDMA_0_TCD.TCD[6].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH6_SBR   EDMA_0_TCD.TCD[6].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH6_PRI   EDMA_0_TCD.TCD[6].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD6_SADDR EDMA_0_TCD.TCD[6].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD6_SOFF EDMA_0_TCD.TCD[6].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD6_ATTR EDMA_0_TCD.TCD[6].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD6_NBYTES_MLOFFNO EDMA_0_TCD.TCD[6].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD6_NBYTES_MLOFFYES EDMA_0_TCD.TCD[6].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD6_SLAST_SDA EDMA_0_TCD.TCD[6].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD6_DADDR EDMA_0_TCD.TCD[6].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD6_DOFF EDMA_0_TCD.TCD[6].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD6_CITER_ELINKNO EDMA_0_TCD.TCD[6].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD6_CITER_ELINKYES EDMA_0_TCD.TCD[6].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD6_DLAST_SGA EDMA_0_TCD.TCD[6].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD6_CSR  EDMA_0_TCD.TCD[6].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD6_BITER_ELINKNO EDMA_0_TCD.TCD[6].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD6_BITER_ELINKYES EDMA_0_TCD.TCD[6].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH7_CSR   EDMA_0_TCD.TCD[7].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH7_ES    EDMA_0_TCD.TCD[7].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH7_INT   EDMA_0_TCD.TCD[7].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH7_SBR   EDMA_0_TCD.TCD[7].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH7_PRI   EDMA_0_TCD.TCD[7].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD7_SADDR EDMA_0_TCD.TCD[7].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD7_SOFF EDMA_0_TCD.TCD[7].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD7_ATTR EDMA_0_TCD.TCD[7].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD7_NBYTES_MLOFFNO EDMA_0_TCD.TCD[7].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD7_NBYTES_MLOFFYES EDMA_0_TCD.TCD[7].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD7_SLAST_SDA EDMA_0_TCD.TCD[7].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD7_DADDR EDMA_0_TCD.TCD[7].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD7_DOFF EDMA_0_TCD.TCD[7].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD7_CITER_ELINKNO EDMA_0_TCD.TCD[7].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD7_CITER_ELINKYES EDMA_0_TCD.TCD[7].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD7_DLAST_SGA EDMA_0_TCD.TCD[7].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD7_CSR  EDMA_0_TCD.TCD[7].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD7_BITER_ELINKNO EDMA_0_TCD.TCD[7].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD7_BITER_ELINKYES EDMA_0_TCD.TCD[7].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH8_CSR   EDMA_0_TCD.TCD[8].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH8_ES    EDMA_0_TCD.TCD[8].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH8_INT   EDMA_0_TCD.TCD[8].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH8_SBR   EDMA_0_TCD.TCD[8].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH8_PRI   EDMA_0_TCD.TCD[8].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD8_SADDR EDMA_0_TCD.TCD[8].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD8_SOFF EDMA_0_TCD.TCD[8].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD8_ATTR EDMA_0_TCD.TCD[8].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD8_NBYTES_MLOFFNO EDMA_0_TCD.TCD[8].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD8_NBYTES_MLOFFYES EDMA_0_TCD.TCD[8].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD8_SLAST_SDA EDMA_0_TCD.TCD[8].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD8_DADDR EDMA_0_TCD.TCD[8].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD8_DOFF EDMA_0_TCD.TCD[8].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD8_CITER_ELINKNO EDMA_0_TCD.TCD[8].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD8_CITER_ELINKYES EDMA_0_TCD.TCD[8].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD8_DLAST_SGA EDMA_0_TCD.TCD[8].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD8_CSR  EDMA_0_TCD.TCD[8].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD8_BITER_ELINKNO EDMA_0_TCD.TCD[8].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD8_BITER_ELINKYES EDMA_0_TCD.TCD[8].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH9_CSR   EDMA_0_TCD.TCD[9].CH_CSR.R    /* Channel Control and Status */
#define EDMA_0_TCD_CH9_ES    EDMA_0_TCD.TCD[9].CH_ES.R     /* Channel Error Status */
#define EDMA_0_TCD_CH9_INT   EDMA_0_TCD.TCD[9].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_0_TCD_CH9_SBR   EDMA_0_TCD.TCD[9].CH_SBR.R    /* Channel System Bus */
#define EDMA_0_TCD_CH9_PRI   EDMA_0_TCD.TCD[9].CH_PRI.R    /* Channel Priority */
#define EDMA_0_TCD_TCD9_SADDR EDMA_0_TCD.TCD[9].SADDR.R    /* TCD Source Address */
#define EDMA_0_TCD_TCD9_SOFF EDMA_0_TCD.TCD[9].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD9_ATTR EDMA_0_TCD.TCD[9].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD9_NBYTES_MLOFFNO EDMA_0_TCD.TCD[9].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD9_NBYTES_MLOFFYES EDMA_0_TCD.TCD[9].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD9_SLAST_SDA EDMA_0_TCD.TCD[9].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD9_DADDR EDMA_0_TCD.TCD[9].DADDR.R    /* TCD Destination Address */
#define EDMA_0_TCD_TCD9_DOFF EDMA_0_TCD.TCD[9].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD9_CITER_ELINKNO EDMA_0_TCD.TCD[9].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD9_CITER_ELINKYES EDMA_0_TCD.TCD[9].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD9_DLAST_SGA EDMA_0_TCD.TCD[9].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD9_CSR  EDMA_0_TCD.TCD[9].CSR.R       /* TCD Control and Status */
#define EDMA_0_TCD_TCD9_BITER_ELINKNO EDMA_0_TCD.TCD[9].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD9_BITER_ELINKYES EDMA_0_TCD.TCD[9].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH10_CSR  EDMA_0_TCD.TCD[10].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH10_ES   EDMA_0_TCD.TCD[10].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH10_INT  EDMA_0_TCD.TCD[10].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH10_SBR  EDMA_0_TCD.TCD[10].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH10_PRI  EDMA_0_TCD.TCD[10].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD10_SADDR EDMA_0_TCD.TCD[10].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD10_SOFF EDMA_0_TCD.TCD[10].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD10_ATTR EDMA_0_TCD.TCD[10].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD10_NBYTES_MLOFFNO EDMA_0_TCD.TCD[10].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD10_NBYTES_MLOFFYES EDMA_0_TCD.TCD[10].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD10_SLAST_SDA EDMA_0_TCD.TCD[10].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD10_DADDR EDMA_0_TCD.TCD[10].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD10_DOFF EDMA_0_TCD.TCD[10].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD10_CITER_ELINKNO EDMA_0_TCD.TCD[10].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD10_CITER_ELINKYES EDMA_0_TCD.TCD[10].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD10_DLAST_SGA EDMA_0_TCD.TCD[10].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD10_CSR EDMA_0_TCD.TCD[10].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD10_BITER_ELINKNO EDMA_0_TCD.TCD[10].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD10_BITER_ELINKYES EDMA_0_TCD.TCD[10].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH11_CSR  EDMA_0_TCD.TCD[11].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH11_ES   EDMA_0_TCD.TCD[11].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH11_INT  EDMA_0_TCD.TCD[11].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH11_SBR  EDMA_0_TCD.TCD[11].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH11_PRI  EDMA_0_TCD.TCD[11].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD11_SADDR EDMA_0_TCD.TCD[11].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD11_SOFF EDMA_0_TCD.TCD[11].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD11_ATTR EDMA_0_TCD.TCD[11].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD11_NBYTES_MLOFFNO EDMA_0_TCD.TCD[11].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD11_NBYTES_MLOFFYES EDMA_0_TCD.TCD[11].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD11_SLAST_SDA EDMA_0_TCD.TCD[11].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD11_DADDR EDMA_0_TCD.TCD[11].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD11_DOFF EDMA_0_TCD.TCD[11].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD11_CITER_ELINKNO EDMA_0_TCD.TCD[11].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD11_CITER_ELINKYES EDMA_0_TCD.TCD[11].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD11_DLAST_SGA EDMA_0_TCD.TCD[11].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD11_CSR EDMA_0_TCD.TCD[11].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD11_BITER_ELINKNO EDMA_0_TCD.TCD[11].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD11_BITER_ELINKYES EDMA_0_TCD.TCD[11].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH12_CSR  EDMA_0_TCD.TCD[12].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH12_ES   EDMA_0_TCD.TCD[12].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH12_INT  EDMA_0_TCD.TCD[12].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH12_SBR  EDMA_0_TCD.TCD[12].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH12_PRI  EDMA_0_TCD.TCD[12].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD12_SADDR EDMA_0_TCD.TCD[12].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD12_SOFF EDMA_0_TCD.TCD[12].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD12_ATTR EDMA_0_TCD.TCD[12].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD12_NBYTES_MLOFFNO EDMA_0_TCD.TCD[12].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD12_NBYTES_MLOFFYES EDMA_0_TCD.TCD[12].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD12_SLAST_SDA EDMA_0_TCD.TCD[12].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD12_DADDR EDMA_0_TCD.TCD[12].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD12_DOFF EDMA_0_TCD.TCD[12].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD12_CITER_ELINKNO EDMA_0_TCD.TCD[12].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD12_CITER_ELINKYES EDMA_0_TCD.TCD[12].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD12_DLAST_SGA EDMA_0_TCD.TCD[12].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD12_CSR EDMA_0_TCD.TCD[12].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD12_BITER_ELINKNO EDMA_0_TCD.TCD[12].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD12_BITER_ELINKYES EDMA_0_TCD.TCD[12].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH13_CSR  EDMA_0_TCD.TCD[13].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH13_ES   EDMA_0_TCD.TCD[13].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH13_INT  EDMA_0_TCD.TCD[13].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH13_SBR  EDMA_0_TCD.TCD[13].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH13_PRI  EDMA_0_TCD.TCD[13].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD13_SADDR EDMA_0_TCD.TCD[13].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD13_SOFF EDMA_0_TCD.TCD[13].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD13_ATTR EDMA_0_TCD.TCD[13].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD13_NBYTES_MLOFFNO EDMA_0_TCD.TCD[13].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD13_NBYTES_MLOFFYES EDMA_0_TCD.TCD[13].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD13_SLAST_SDA EDMA_0_TCD.TCD[13].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD13_DADDR EDMA_0_TCD.TCD[13].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD13_DOFF EDMA_0_TCD.TCD[13].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD13_CITER_ELINKNO EDMA_0_TCD.TCD[13].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD13_CITER_ELINKYES EDMA_0_TCD.TCD[13].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD13_DLAST_SGA EDMA_0_TCD.TCD[13].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD13_CSR EDMA_0_TCD.TCD[13].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD13_BITER_ELINKNO EDMA_0_TCD.TCD[13].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD13_BITER_ELINKYES EDMA_0_TCD.TCD[13].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH14_CSR  EDMA_0_TCD.TCD[14].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH14_ES   EDMA_0_TCD.TCD[14].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH14_INT  EDMA_0_TCD.TCD[14].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH14_SBR  EDMA_0_TCD.TCD[14].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH14_PRI  EDMA_0_TCD.TCD[14].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD14_SADDR EDMA_0_TCD.TCD[14].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD14_SOFF EDMA_0_TCD.TCD[14].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD14_ATTR EDMA_0_TCD.TCD[14].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD14_NBYTES_MLOFFNO EDMA_0_TCD.TCD[14].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD14_NBYTES_MLOFFYES EDMA_0_TCD.TCD[14].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD14_SLAST_SDA EDMA_0_TCD.TCD[14].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD14_DADDR EDMA_0_TCD.TCD[14].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD14_DOFF EDMA_0_TCD.TCD[14].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD14_CITER_ELINKNO EDMA_0_TCD.TCD[14].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD14_CITER_ELINKYES EDMA_0_TCD.TCD[14].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD14_DLAST_SGA EDMA_0_TCD.TCD[14].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD14_CSR EDMA_0_TCD.TCD[14].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD14_BITER_ELINKNO EDMA_0_TCD.TCD[14].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD14_BITER_ELINKYES EDMA_0_TCD.TCD[14].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH15_CSR  EDMA_0_TCD.TCD[15].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH15_ES   EDMA_0_TCD.TCD[15].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH15_INT  EDMA_0_TCD.TCD[15].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH15_SBR  EDMA_0_TCD.TCD[15].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH15_PRI  EDMA_0_TCD.TCD[15].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD15_SADDR EDMA_0_TCD.TCD[15].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD15_SOFF EDMA_0_TCD.TCD[15].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD15_ATTR EDMA_0_TCD.TCD[15].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD15_NBYTES_MLOFFNO EDMA_0_TCD.TCD[15].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD15_NBYTES_MLOFFYES EDMA_0_TCD.TCD[15].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD15_SLAST_SDA EDMA_0_TCD.TCD[15].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD15_DADDR EDMA_0_TCD.TCD[15].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD15_DOFF EDMA_0_TCD.TCD[15].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD15_CITER_ELINKNO EDMA_0_TCD.TCD[15].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD15_CITER_ELINKYES EDMA_0_TCD.TCD[15].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD15_DLAST_SGA EDMA_0_TCD.TCD[15].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD15_CSR EDMA_0_TCD.TCD[15].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD15_BITER_ELINKNO EDMA_0_TCD.TCD[15].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD15_BITER_ELINKYES EDMA_0_TCD.TCD[15].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH16_CSR  EDMA_0_TCD.TCD[16].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH16_ES   EDMA_0_TCD.TCD[16].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH16_INT  EDMA_0_TCD.TCD[16].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH16_SBR  EDMA_0_TCD.TCD[16].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH16_PRI  EDMA_0_TCD.TCD[16].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD16_SADDR EDMA_0_TCD.TCD[16].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD16_SOFF EDMA_0_TCD.TCD[16].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD16_ATTR EDMA_0_TCD.TCD[16].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD16_NBYTES_MLOFFNO EDMA_0_TCD.TCD[16].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD16_NBYTES_MLOFFYES EDMA_0_TCD.TCD[16].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD16_SLAST_SDA EDMA_0_TCD.TCD[16].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD16_DADDR EDMA_0_TCD.TCD[16].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD16_DOFF EDMA_0_TCD.TCD[16].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD16_CITER_ELINKNO EDMA_0_TCD.TCD[16].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD16_CITER_ELINKYES EDMA_0_TCD.TCD[16].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD16_DLAST_SGA EDMA_0_TCD.TCD[16].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD16_CSR EDMA_0_TCD.TCD[16].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD16_BITER_ELINKNO EDMA_0_TCD.TCD[16].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD16_BITER_ELINKYES EDMA_0_TCD.TCD[16].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH17_CSR  EDMA_0_TCD.TCD[17].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH17_ES   EDMA_0_TCD.TCD[17].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH17_INT  EDMA_0_TCD.TCD[17].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH17_SBR  EDMA_0_TCD.TCD[17].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH17_PRI  EDMA_0_TCD.TCD[17].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD17_SADDR EDMA_0_TCD.TCD[17].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD17_SOFF EDMA_0_TCD.TCD[17].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD17_ATTR EDMA_0_TCD.TCD[17].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD17_NBYTES_MLOFFNO EDMA_0_TCD.TCD[17].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD17_NBYTES_MLOFFYES EDMA_0_TCD.TCD[17].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD17_SLAST_SDA EDMA_0_TCD.TCD[17].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD17_DADDR EDMA_0_TCD.TCD[17].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD17_DOFF EDMA_0_TCD.TCD[17].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD17_CITER_ELINKNO EDMA_0_TCD.TCD[17].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD17_CITER_ELINKYES EDMA_0_TCD.TCD[17].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD17_DLAST_SGA EDMA_0_TCD.TCD[17].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD17_CSR EDMA_0_TCD.TCD[17].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD17_BITER_ELINKNO EDMA_0_TCD.TCD[17].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD17_BITER_ELINKYES EDMA_0_TCD.TCD[17].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH18_CSR  EDMA_0_TCD.TCD[18].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH18_ES   EDMA_0_TCD.TCD[18].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH18_INT  EDMA_0_TCD.TCD[18].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH18_SBR  EDMA_0_TCD.TCD[18].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH18_PRI  EDMA_0_TCD.TCD[18].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD18_SADDR EDMA_0_TCD.TCD[18].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD18_SOFF EDMA_0_TCD.TCD[18].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD18_ATTR EDMA_0_TCD.TCD[18].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD18_NBYTES_MLOFFNO EDMA_0_TCD.TCD[18].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD18_NBYTES_MLOFFYES EDMA_0_TCD.TCD[18].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD18_SLAST_SDA EDMA_0_TCD.TCD[18].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD18_DADDR EDMA_0_TCD.TCD[18].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD18_DOFF EDMA_0_TCD.TCD[18].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD18_CITER_ELINKNO EDMA_0_TCD.TCD[18].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD18_CITER_ELINKYES EDMA_0_TCD.TCD[18].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD18_DLAST_SGA EDMA_0_TCD.TCD[18].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD18_CSR EDMA_0_TCD.TCD[18].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD18_BITER_ELINKNO EDMA_0_TCD.TCD[18].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD18_BITER_ELINKYES EDMA_0_TCD.TCD[18].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH19_CSR  EDMA_0_TCD.TCD[19].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH19_ES   EDMA_0_TCD.TCD[19].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH19_INT  EDMA_0_TCD.TCD[19].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH19_SBR  EDMA_0_TCD.TCD[19].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH19_PRI  EDMA_0_TCD.TCD[19].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD19_SADDR EDMA_0_TCD.TCD[19].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD19_SOFF EDMA_0_TCD.TCD[19].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD19_ATTR EDMA_0_TCD.TCD[19].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD19_NBYTES_MLOFFNO EDMA_0_TCD.TCD[19].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD19_NBYTES_MLOFFYES EDMA_0_TCD.TCD[19].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD19_SLAST_SDA EDMA_0_TCD.TCD[19].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD19_DADDR EDMA_0_TCD.TCD[19].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD19_DOFF EDMA_0_TCD.TCD[19].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD19_CITER_ELINKNO EDMA_0_TCD.TCD[19].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD19_CITER_ELINKYES EDMA_0_TCD.TCD[19].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD19_DLAST_SGA EDMA_0_TCD.TCD[19].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD19_CSR EDMA_0_TCD.TCD[19].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD19_BITER_ELINKNO EDMA_0_TCD.TCD[19].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD19_BITER_ELINKYES EDMA_0_TCD.TCD[19].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH20_CSR  EDMA_0_TCD.TCD[20].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH20_ES   EDMA_0_TCD.TCD[20].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH20_INT  EDMA_0_TCD.TCD[20].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH20_SBR  EDMA_0_TCD.TCD[20].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH20_PRI  EDMA_0_TCD.TCD[20].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD20_SADDR EDMA_0_TCD.TCD[20].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD20_SOFF EDMA_0_TCD.TCD[20].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD20_ATTR EDMA_0_TCD.TCD[20].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD20_NBYTES_MLOFFNO EDMA_0_TCD.TCD[20].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD20_NBYTES_MLOFFYES EDMA_0_TCD.TCD[20].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD20_SLAST_SDA EDMA_0_TCD.TCD[20].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD20_DADDR EDMA_0_TCD.TCD[20].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD20_DOFF EDMA_0_TCD.TCD[20].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD20_CITER_ELINKNO EDMA_0_TCD.TCD[20].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD20_CITER_ELINKYES EDMA_0_TCD.TCD[20].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD20_DLAST_SGA EDMA_0_TCD.TCD[20].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD20_CSR EDMA_0_TCD.TCD[20].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD20_BITER_ELINKNO EDMA_0_TCD.TCD[20].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD20_BITER_ELINKYES EDMA_0_TCD.TCD[20].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH21_CSR  EDMA_0_TCD.TCD[21].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH21_ES   EDMA_0_TCD.TCD[21].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH21_INT  EDMA_0_TCD.TCD[21].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH21_SBR  EDMA_0_TCD.TCD[21].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH21_PRI  EDMA_0_TCD.TCD[21].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD21_SADDR EDMA_0_TCD.TCD[21].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD21_SOFF EDMA_0_TCD.TCD[21].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD21_ATTR EDMA_0_TCD.TCD[21].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD21_NBYTES_MLOFFNO EDMA_0_TCD.TCD[21].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD21_NBYTES_MLOFFYES EDMA_0_TCD.TCD[21].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD21_SLAST_SDA EDMA_0_TCD.TCD[21].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD21_DADDR EDMA_0_TCD.TCD[21].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD21_DOFF EDMA_0_TCD.TCD[21].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD21_CITER_ELINKNO EDMA_0_TCD.TCD[21].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD21_CITER_ELINKYES EDMA_0_TCD.TCD[21].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD21_DLAST_SGA EDMA_0_TCD.TCD[21].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD21_CSR EDMA_0_TCD.TCD[21].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD21_BITER_ELINKNO EDMA_0_TCD.TCD[21].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD21_BITER_ELINKYES EDMA_0_TCD.TCD[21].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH22_CSR  EDMA_0_TCD.TCD[22].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH22_ES   EDMA_0_TCD.TCD[22].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH22_INT  EDMA_0_TCD.TCD[22].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH22_SBR  EDMA_0_TCD.TCD[22].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH22_PRI  EDMA_0_TCD.TCD[22].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD22_SADDR EDMA_0_TCD.TCD[22].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD22_SOFF EDMA_0_TCD.TCD[22].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD22_ATTR EDMA_0_TCD.TCD[22].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD22_NBYTES_MLOFFNO EDMA_0_TCD.TCD[22].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD22_NBYTES_MLOFFYES EDMA_0_TCD.TCD[22].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD22_SLAST_SDA EDMA_0_TCD.TCD[22].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD22_DADDR EDMA_0_TCD.TCD[22].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD22_DOFF EDMA_0_TCD.TCD[22].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD22_CITER_ELINKNO EDMA_0_TCD.TCD[22].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD22_CITER_ELINKYES EDMA_0_TCD.TCD[22].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD22_DLAST_SGA EDMA_0_TCD.TCD[22].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD22_CSR EDMA_0_TCD.TCD[22].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD22_BITER_ELINKNO EDMA_0_TCD.TCD[22].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD22_BITER_ELINKYES EDMA_0_TCD.TCD[22].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH23_CSR  EDMA_0_TCD.TCD[23].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH23_ES   EDMA_0_TCD.TCD[23].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH23_INT  EDMA_0_TCD.TCD[23].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH23_SBR  EDMA_0_TCD.TCD[23].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH23_PRI  EDMA_0_TCD.TCD[23].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD23_SADDR EDMA_0_TCD.TCD[23].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD23_SOFF EDMA_0_TCD.TCD[23].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD23_ATTR EDMA_0_TCD.TCD[23].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD23_NBYTES_MLOFFNO EDMA_0_TCD.TCD[23].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD23_NBYTES_MLOFFYES EDMA_0_TCD.TCD[23].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD23_SLAST_SDA EDMA_0_TCD.TCD[23].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD23_DADDR EDMA_0_TCD.TCD[23].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD23_DOFF EDMA_0_TCD.TCD[23].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD23_CITER_ELINKNO EDMA_0_TCD.TCD[23].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD23_CITER_ELINKYES EDMA_0_TCD.TCD[23].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD23_DLAST_SGA EDMA_0_TCD.TCD[23].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD23_CSR EDMA_0_TCD.TCD[23].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD23_BITER_ELINKNO EDMA_0_TCD.TCD[23].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD23_BITER_ELINKYES EDMA_0_TCD.TCD[23].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH24_CSR  EDMA_0_TCD.TCD[24].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH24_ES   EDMA_0_TCD.TCD[24].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH24_INT  EDMA_0_TCD.TCD[24].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH24_SBR  EDMA_0_TCD.TCD[24].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH24_PRI  EDMA_0_TCD.TCD[24].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD24_SADDR EDMA_0_TCD.TCD[24].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD24_SOFF EDMA_0_TCD.TCD[24].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD24_ATTR EDMA_0_TCD.TCD[24].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD24_NBYTES_MLOFFNO EDMA_0_TCD.TCD[24].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD24_NBYTES_MLOFFYES EDMA_0_TCD.TCD[24].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD24_SLAST_SDA EDMA_0_TCD.TCD[24].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD24_DADDR EDMA_0_TCD.TCD[24].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD24_DOFF EDMA_0_TCD.TCD[24].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD24_CITER_ELINKNO EDMA_0_TCD.TCD[24].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD24_CITER_ELINKYES EDMA_0_TCD.TCD[24].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD24_DLAST_SGA EDMA_0_TCD.TCD[24].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD24_CSR EDMA_0_TCD.TCD[24].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD24_BITER_ELINKNO EDMA_0_TCD.TCD[24].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD24_BITER_ELINKYES EDMA_0_TCD.TCD[24].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH25_CSR  EDMA_0_TCD.TCD[25].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH25_ES   EDMA_0_TCD.TCD[25].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH25_INT  EDMA_0_TCD.TCD[25].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH25_SBR  EDMA_0_TCD.TCD[25].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH25_PRI  EDMA_0_TCD.TCD[25].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD25_SADDR EDMA_0_TCD.TCD[25].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD25_SOFF EDMA_0_TCD.TCD[25].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD25_ATTR EDMA_0_TCD.TCD[25].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD25_NBYTES_MLOFFNO EDMA_0_TCD.TCD[25].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD25_NBYTES_MLOFFYES EDMA_0_TCD.TCD[25].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD25_SLAST_SDA EDMA_0_TCD.TCD[25].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD25_DADDR EDMA_0_TCD.TCD[25].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD25_DOFF EDMA_0_TCD.TCD[25].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD25_CITER_ELINKNO EDMA_0_TCD.TCD[25].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD25_CITER_ELINKYES EDMA_0_TCD.TCD[25].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD25_DLAST_SGA EDMA_0_TCD.TCD[25].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD25_CSR EDMA_0_TCD.TCD[25].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD25_BITER_ELINKNO EDMA_0_TCD.TCD[25].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD25_BITER_ELINKYES EDMA_0_TCD.TCD[25].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH26_CSR  EDMA_0_TCD.TCD[26].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH26_ES   EDMA_0_TCD.TCD[26].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH26_INT  EDMA_0_TCD.TCD[26].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH26_SBR  EDMA_0_TCD.TCD[26].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH26_PRI  EDMA_0_TCD.TCD[26].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD26_SADDR EDMA_0_TCD.TCD[26].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD26_SOFF EDMA_0_TCD.TCD[26].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD26_ATTR EDMA_0_TCD.TCD[26].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD26_NBYTES_MLOFFNO EDMA_0_TCD.TCD[26].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD26_NBYTES_MLOFFYES EDMA_0_TCD.TCD[26].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD26_SLAST_SDA EDMA_0_TCD.TCD[26].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD26_DADDR EDMA_0_TCD.TCD[26].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD26_DOFF EDMA_0_TCD.TCD[26].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD26_CITER_ELINKNO EDMA_0_TCD.TCD[26].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD26_CITER_ELINKYES EDMA_0_TCD.TCD[26].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD26_DLAST_SGA EDMA_0_TCD.TCD[26].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD26_CSR EDMA_0_TCD.TCD[26].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD26_BITER_ELINKNO EDMA_0_TCD.TCD[26].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD26_BITER_ELINKYES EDMA_0_TCD.TCD[26].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH27_CSR  EDMA_0_TCD.TCD[27].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH27_ES   EDMA_0_TCD.TCD[27].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH27_INT  EDMA_0_TCD.TCD[27].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH27_SBR  EDMA_0_TCD.TCD[27].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH27_PRI  EDMA_0_TCD.TCD[27].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD27_SADDR EDMA_0_TCD.TCD[27].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD27_SOFF EDMA_0_TCD.TCD[27].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD27_ATTR EDMA_0_TCD.TCD[27].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD27_NBYTES_MLOFFNO EDMA_0_TCD.TCD[27].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD27_NBYTES_MLOFFYES EDMA_0_TCD.TCD[27].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD27_SLAST_SDA EDMA_0_TCD.TCD[27].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD27_DADDR EDMA_0_TCD.TCD[27].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD27_DOFF EDMA_0_TCD.TCD[27].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD27_CITER_ELINKNO EDMA_0_TCD.TCD[27].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD27_CITER_ELINKYES EDMA_0_TCD.TCD[27].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD27_DLAST_SGA EDMA_0_TCD.TCD[27].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD27_CSR EDMA_0_TCD.TCD[27].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD27_BITER_ELINKNO EDMA_0_TCD.TCD[27].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD27_BITER_ELINKYES EDMA_0_TCD.TCD[27].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH28_CSR  EDMA_0_TCD.TCD[28].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH28_ES   EDMA_0_TCD.TCD[28].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH28_INT  EDMA_0_TCD.TCD[28].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH28_SBR  EDMA_0_TCD.TCD[28].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH28_PRI  EDMA_0_TCD.TCD[28].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD28_SADDR EDMA_0_TCD.TCD[28].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD28_SOFF EDMA_0_TCD.TCD[28].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD28_ATTR EDMA_0_TCD.TCD[28].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD28_NBYTES_MLOFFNO EDMA_0_TCD.TCD[28].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD28_NBYTES_MLOFFYES EDMA_0_TCD.TCD[28].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD28_SLAST_SDA EDMA_0_TCD.TCD[28].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD28_DADDR EDMA_0_TCD.TCD[28].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD28_DOFF EDMA_0_TCD.TCD[28].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD28_CITER_ELINKNO EDMA_0_TCD.TCD[28].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD28_CITER_ELINKYES EDMA_0_TCD.TCD[28].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD28_DLAST_SGA EDMA_0_TCD.TCD[28].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD28_CSR EDMA_0_TCD.TCD[28].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD28_BITER_ELINKNO EDMA_0_TCD.TCD[28].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD28_BITER_ELINKYES EDMA_0_TCD.TCD[28].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH29_CSR  EDMA_0_TCD.TCD[29].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH29_ES   EDMA_0_TCD.TCD[29].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH29_INT  EDMA_0_TCD.TCD[29].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH29_SBR  EDMA_0_TCD.TCD[29].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH29_PRI  EDMA_0_TCD.TCD[29].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD29_SADDR EDMA_0_TCD.TCD[29].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD29_SOFF EDMA_0_TCD.TCD[29].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD29_ATTR EDMA_0_TCD.TCD[29].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD29_NBYTES_MLOFFNO EDMA_0_TCD.TCD[29].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD29_NBYTES_MLOFFYES EDMA_0_TCD.TCD[29].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD29_SLAST_SDA EDMA_0_TCD.TCD[29].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD29_DADDR EDMA_0_TCD.TCD[29].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD29_DOFF EDMA_0_TCD.TCD[29].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD29_CITER_ELINKNO EDMA_0_TCD.TCD[29].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD29_CITER_ELINKYES EDMA_0_TCD.TCD[29].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD29_DLAST_SGA EDMA_0_TCD.TCD[29].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD29_CSR EDMA_0_TCD.TCD[29].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD29_BITER_ELINKNO EDMA_0_TCD.TCD[29].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD29_BITER_ELINKYES EDMA_0_TCD.TCD[29].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH30_CSR  EDMA_0_TCD.TCD[30].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH30_ES   EDMA_0_TCD.TCD[30].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH30_INT  EDMA_0_TCD.TCD[30].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH30_SBR  EDMA_0_TCD.TCD[30].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH30_PRI  EDMA_0_TCD.TCD[30].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD30_SADDR EDMA_0_TCD.TCD[30].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD30_SOFF EDMA_0_TCD.TCD[30].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD30_ATTR EDMA_0_TCD.TCD[30].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD30_NBYTES_MLOFFNO EDMA_0_TCD.TCD[30].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD30_NBYTES_MLOFFYES EDMA_0_TCD.TCD[30].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD30_SLAST_SDA EDMA_0_TCD.TCD[30].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD30_DADDR EDMA_0_TCD.TCD[30].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD30_DOFF EDMA_0_TCD.TCD[30].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD30_CITER_ELINKNO EDMA_0_TCD.TCD[30].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD30_CITER_ELINKYES EDMA_0_TCD.TCD[30].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD30_DLAST_SGA EDMA_0_TCD.TCD[30].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD30_CSR EDMA_0_TCD.TCD[30].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD30_BITER_ELINKNO EDMA_0_TCD.TCD[30].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD30_BITER_ELINKYES EDMA_0_TCD.TCD[30].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_CH31_CSR  EDMA_0_TCD.TCD[31].CH_CSR.R   /* Channel Control and Status */
#define EDMA_0_TCD_CH31_ES   EDMA_0_TCD.TCD[31].CH_ES.R    /* Channel Error Status */
#define EDMA_0_TCD_CH31_INT  EDMA_0_TCD.TCD[31].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_0_TCD_CH31_SBR  EDMA_0_TCD.TCD[31].CH_SBR.R   /* Channel System Bus */
#define EDMA_0_TCD_CH31_PRI  EDMA_0_TCD.TCD[31].CH_PRI.R   /* Channel Priority */
#define EDMA_0_TCD_TCD31_SADDR EDMA_0_TCD.TCD[31].SADDR.R  /* TCD Source Address */
#define EDMA_0_TCD_TCD31_SOFF EDMA_0_TCD.TCD[31].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_0_TCD_TCD31_ATTR EDMA_0_TCD.TCD[31].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_0_TCD_TCD31_NBYTES_MLOFFNO EDMA_0_TCD.TCD[31].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_0_TCD_TCD31_NBYTES_MLOFFYES EDMA_0_TCD.TCD[31].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_0_TCD_TCD31_SLAST_SDA EDMA_0_TCD.TCD[31].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_0_TCD_TCD31_DADDR EDMA_0_TCD.TCD[31].DADDR.R  /* TCD Destination Address */
#define EDMA_0_TCD_TCD31_DOFF EDMA_0_TCD.TCD[31].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_0_TCD_TCD31_CITER_ELINKNO EDMA_0_TCD.TCD[31].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD31_CITER_ELINKYES EDMA_0_TCD.TCD[31].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_0_TCD_TCD31_DLAST_SGA EDMA_0_TCD.TCD[31].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_0_TCD_TCD31_CSR EDMA_0_TCD.TCD[31].CSR.R      /* TCD Control and Status */
#define EDMA_0_TCD_TCD31_BITER_ELINKNO EDMA_0_TCD.TCD[31].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_0_TCD_TCD31_BITER_ELINKYES EDMA_0_TCD.TCD[31].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */

#define EDMA_1_TCD_CH0_CSR   EDMA_1_TCD.TCD[0].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH0_ES    EDMA_1_TCD.TCD[0].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH0_INT   EDMA_1_TCD.TCD[0].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH0_SBR   EDMA_1_TCD.TCD[0].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH0_PRI   EDMA_1_TCD.TCD[0].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD0_SADDR EDMA_1_TCD.TCD[0].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD0_SOFF EDMA_1_TCD.TCD[0].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD0_ATTR EDMA_1_TCD.TCD[0].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD0_NBYTES_MLOFFNO EDMA_1_TCD.TCD[0].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD0_NBYTES_MLOFFYES EDMA_1_TCD.TCD[0].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD0_SLAST_SDA EDMA_1_TCD.TCD[0].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD0_DADDR EDMA_1_TCD.TCD[0].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD0_DOFF EDMA_1_TCD.TCD[0].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD0_CITER_ELINKNO EDMA_1_TCD.TCD[0].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD0_CITER_ELINKYES EDMA_1_TCD.TCD[0].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD0_DLAST_SGA EDMA_1_TCD.TCD[0].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD0_CSR  EDMA_1_TCD.TCD[0].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD0_BITER_ELINKNO EDMA_1_TCD.TCD[0].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD0_BITER_ELINKYES EDMA_1_TCD.TCD[0].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH1_CSR   EDMA_1_TCD.TCD[1].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH1_ES    EDMA_1_TCD.TCD[1].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH1_INT   EDMA_1_TCD.TCD[1].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH1_SBR   EDMA_1_TCD.TCD[1].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH1_PRI   EDMA_1_TCD.TCD[1].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD1_SADDR EDMA_1_TCD.TCD[1].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD1_SOFF EDMA_1_TCD.TCD[1].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD1_ATTR EDMA_1_TCD.TCD[1].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD1_NBYTES_MLOFFNO EDMA_1_TCD.TCD[1].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD1_NBYTES_MLOFFYES EDMA_1_TCD.TCD[1].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD1_SLAST_SDA EDMA_1_TCD.TCD[1].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD1_DADDR EDMA_1_TCD.TCD[1].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD1_DOFF EDMA_1_TCD.TCD[1].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD1_CITER_ELINKNO EDMA_1_TCD.TCD[1].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD1_CITER_ELINKYES EDMA_1_TCD.TCD[1].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD1_DLAST_SGA EDMA_1_TCD.TCD[1].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD1_CSR  EDMA_1_TCD.TCD[1].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD1_BITER_ELINKNO EDMA_1_TCD.TCD[1].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD1_BITER_ELINKYES EDMA_1_TCD.TCD[1].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH2_CSR   EDMA_1_TCD.TCD[2].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH2_ES    EDMA_1_TCD.TCD[2].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH2_INT   EDMA_1_TCD.TCD[2].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH2_SBR   EDMA_1_TCD.TCD[2].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH2_PRI   EDMA_1_TCD.TCD[2].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD2_SADDR EDMA_1_TCD.TCD[2].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD2_SOFF EDMA_1_TCD.TCD[2].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD2_ATTR EDMA_1_TCD.TCD[2].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD2_NBYTES_MLOFFNO EDMA_1_TCD.TCD[2].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD2_NBYTES_MLOFFYES EDMA_1_TCD.TCD[2].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD2_SLAST_SDA EDMA_1_TCD.TCD[2].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD2_DADDR EDMA_1_TCD.TCD[2].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD2_DOFF EDMA_1_TCD.TCD[2].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD2_CITER_ELINKNO EDMA_1_TCD.TCD[2].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD2_CITER_ELINKYES EDMA_1_TCD.TCD[2].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD2_DLAST_SGA EDMA_1_TCD.TCD[2].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD2_CSR  EDMA_1_TCD.TCD[2].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD2_BITER_ELINKNO EDMA_1_TCD.TCD[2].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD2_BITER_ELINKYES EDMA_1_TCD.TCD[2].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH3_CSR   EDMA_1_TCD.TCD[3].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH3_ES    EDMA_1_TCD.TCD[3].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH3_INT   EDMA_1_TCD.TCD[3].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH3_SBR   EDMA_1_TCD.TCD[3].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH3_PRI   EDMA_1_TCD.TCD[3].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD3_SADDR EDMA_1_TCD.TCD[3].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD3_SOFF EDMA_1_TCD.TCD[3].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD3_ATTR EDMA_1_TCD.TCD[3].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD3_NBYTES_MLOFFNO EDMA_1_TCD.TCD[3].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD3_NBYTES_MLOFFYES EDMA_1_TCD.TCD[3].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD3_SLAST_SDA EDMA_1_TCD.TCD[3].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD3_DADDR EDMA_1_TCD.TCD[3].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD3_DOFF EDMA_1_TCD.TCD[3].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD3_CITER_ELINKNO EDMA_1_TCD.TCD[3].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD3_CITER_ELINKYES EDMA_1_TCD.TCD[3].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD3_DLAST_SGA EDMA_1_TCD.TCD[3].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD3_CSR  EDMA_1_TCD.TCD[3].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD3_BITER_ELINKNO EDMA_1_TCD.TCD[3].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD3_BITER_ELINKYES EDMA_1_TCD.TCD[3].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH4_CSR   EDMA_1_TCD.TCD[4].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH4_ES    EDMA_1_TCD.TCD[4].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH4_INT   EDMA_1_TCD.TCD[4].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH4_SBR   EDMA_1_TCD.TCD[4].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH4_PRI   EDMA_1_TCD.TCD[4].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD4_SADDR EDMA_1_TCD.TCD[4].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD4_SOFF EDMA_1_TCD.TCD[4].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD4_ATTR EDMA_1_TCD.TCD[4].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD4_NBYTES_MLOFFNO EDMA_1_TCD.TCD[4].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD4_NBYTES_MLOFFYES EDMA_1_TCD.TCD[4].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD4_SLAST_SDA EDMA_1_TCD.TCD[4].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD4_DADDR EDMA_1_TCD.TCD[4].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD4_DOFF EDMA_1_TCD.TCD[4].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD4_CITER_ELINKNO EDMA_1_TCD.TCD[4].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD4_CITER_ELINKYES EDMA_1_TCD.TCD[4].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD4_DLAST_SGA EDMA_1_TCD.TCD[4].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD4_CSR  EDMA_1_TCD.TCD[4].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD4_BITER_ELINKNO EDMA_1_TCD.TCD[4].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD4_BITER_ELINKYES EDMA_1_TCD.TCD[4].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH5_CSR   EDMA_1_TCD.TCD[5].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH5_ES    EDMA_1_TCD.TCD[5].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH5_INT   EDMA_1_TCD.TCD[5].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH5_SBR   EDMA_1_TCD.TCD[5].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH5_PRI   EDMA_1_TCD.TCD[5].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD5_SADDR EDMA_1_TCD.TCD[5].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD5_SOFF EDMA_1_TCD.TCD[5].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD5_ATTR EDMA_1_TCD.TCD[5].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD5_NBYTES_MLOFFNO EDMA_1_TCD.TCD[5].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD5_NBYTES_MLOFFYES EDMA_1_TCD.TCD[5].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD5_SLAST_SDA EDMA_1_TCD.TCD[5].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD5_DADDR EDMA_1_TCD.TCD[5].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD5_DOFF EDMA_1_TCD.TCD[5].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD5_CITER_ELINKNO EDMA_1_TCD.TCD[5].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD5_CITER_ELINKYES EDMA_1_TCD.TCD[5].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD5_DLAST_SGA EDMA_1_TCD.TCD[5].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD5_CSR  EDMA_1_TCD.TCD[5].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD5_BITER_ELINKNO EDMA_1_TCD.TCD[5].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD5_BITER_ELINKYES EDMA_1_TCD.TCD[5].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH6_CSR   EDMA_1_TCD.TCD[6].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH6_ES    EDMA_1_TCD.TCD[6].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH6_INT   EDMA_1_TCD.TCD[6].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH6_SBR   EDMA_1_TCD.TCD[6].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH6_PRI   EDMA_1_TCD.TCD[6].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD6_SADDR EDMA_1_TCD.TCD[6].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD6_SOFF EDMA_1_TCD.TCD[6].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD6_ATTR EDMA_1_TCD.TCD[6].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD6_NBYTES_MLOFFNO EDMA_1_TCD.TCD[6].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD6_NBYTES_MLOFFYES EDMA_1_TCD.TCD[6].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD6_SLAST_SDA EDMA_1_TCD.TCD[6].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD6_DADDR EDMA_1_TCD.TCD[6].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD6_DOFF EDMA_1_TCD.TCD[6].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD6_CITER_ELINKNO EDMA_1_TCD.TCD[6].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD6_CITER_ELINKYES EDMA_1_TCD.TCD[6].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD6_DLAST_SGA EDMA_1_TCD.TCD[6].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD6_CSR  EDMA_1_TCD.TCD[6].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD6_BITER_ELINKNO EDMA_1_TCD.TCD[6].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD6_BITER_ELINKYES EDMA_1_TCD.TCD[6].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH7_CSR   EDMA_1_TCD.TCD[7].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH7_ES    EDMA_1_TCD.TCD[7].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH7_INT   EDMA_1_TCD.TCD[7].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH7_SBR   EDMA_1_TCD.TCD[7].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH7_PRI   EDMA_1_TCD.TCD[7].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD7_SADDR EDMA_1_TCD.TCD[7].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD7_SOFF EDMA_1_TCD.TCD[7].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD7_ATTR EDMA_1_TCD.TCD[7].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD7_NBYTES_MLOFFNO EDMA_1_TCD.TCD[7].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD7_NBYTES_MLOFFYES EDMA_1_TCD.TCD[7].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD7_SLAST_SDA EDMA_1_TCD.TCD[7].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD7_DADDR EDMA_1_TCD.TCD[7].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD7_DOFF EDMA_1_TCD.TCD[7].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD7_CITER_ELINKNO EDMA_1_TCD.TCD[7].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD7_CITER_ELINKYES EDMA_1_TCD.TCD[7].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD7_DLAST_SGA EDMA_1_TCD.TCD[7].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD7_CSR  EDMA_1_TCD.TCD[7].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD7_BITER_ELINKNO EDMA_1_TCD.TCD[7].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD7_BITER_ELINKYES EDMA_1_TCD.TCD[7].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH8_CSR   EDMA_1_TCD.TCD[8].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH8_ES    EDMA_1_TCD.TCD[8].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH8_INT   EDMA_1_TCD.TCD[8].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH8_SBR   EDMA_1_TCD.TCD[8].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH8_PRI   EDMA_1_TCD.TCD[8].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD8_SADDR EDMA_1_TCD.TCD[8].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD8_SOFF EDMA_1_TCD.TCD[8].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD8_ATTR EDMA_1_TCD.TCD[8].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD8_NBYTES_MLOFFNO EDMA_1_TCD.TCD[8].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD8_NBYTES_MLOFFYES EDMA_1_TCD.TCD[8].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD8_SLAST_SDA EDMA_1_TCD.TCD[8].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD8_DADDR EDMA_1_TCD.TCD[8].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD8_DOFF EDMA_1_TCD.TCD[8].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD8_CITER_ELINKNO EDMA_1_TCD.TCD[8].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD8_CITER_ELINKYES EDMA_1_TCD.TCD[8].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD8_DLAST_SGA EDMA_1_TCD.TCD[8].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD8_CSR  EDMA_1_TCD.TCD[8].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD8_BITER_ELINKNO EDMA_1_TCD.TCD[8].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD8_BITER_ELINKYES EDMA_1_TCD.TCD[8].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH9_CSR   EDMA_1_TCD.TCD[9].CH_CSR.R    /* Channel Control and Status */
#define EDMA_1_TCD_CH9_ES    EDMA_1_TCD.TCD[9].CH_ES.R     /* Channel Error Status */
#define EDMA_1_TCD_CH9_INT   EDMA_1_TCD.TCD[9].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_1_TCD_CH9_SBR   EDMA_1_TCD.TCD[9].CH_SBR.R    /* Channel System Bus */
#define EDMA_1_TCD_CH9_PRI   EDMA_1_TCD.TCD[9].CH_PRI.R    /* Channel Priority */
#define EDMA_1_TCD_TCD9_SADDR EDMA_1_TCD.TCD[9].SADDR.R    /* TCD Source Address */
#define EDMA_1_TCD_TCD9_SOFF EDMA_1_TCD.TCD[9].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD9_ATTR EDMA_1_TCD.TCD[9].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD9_NBYTES_MLOFFNO EDMA_1_TCD.TCD[9].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD9_NBYTES_MLOFFYES EDMA_1_TCD.TCD[9].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD9_SLAST_SDA EDMA_1_TCD.TCD[9].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD9_DADDR EDMA_1_TCD.TCD[9].DADDR.R    /* TCD Destination Address */
#define EDMA_1_TCD_TCD9_DOFF EDMA_1_TCD.TCD[9].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD9_CITER_ELINKNO EDMA_1_TCD.TCD[9].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD9_CITER_ELINKYES EDMA_1_TCD.TCD[9].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD9_DLAST_SGA EDMA_1_TCD.TCD[9].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD9_CSR  EDMA_1_TCD.TCD[9].CSR.R       /* TCD Control and Status */
#define EDMA_1_TCD_TCD9_BITER_ELINKNO EDMA_1_TCD.TCD[9].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD9_BITER_ELINKYES EDMA_1_TCD.TCD[9].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH10_CSR  EDMA_1_TCD.TCD[10].CH_CSR.R   /* Channel Control and Status */
#define EDMA_1_TCD_CH10_ES   EDMA_1_TCD.TCD[10].CH_ES.R    /* Channel Error Status */
#define EDMA_1_TCD_CH10_INT  EDMA_1_TCD.TCD[10].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_1_TCD_CH10_SBR  EDMA_1_TCD.TCD[10].CH_SBR.R   /* Channel System Bus */
#define EDMA_1_TCD_CH10_PRI  EDMA_1_TCD.TCD[10].CH_PRI.R   /* Channel Priority */
#define EDMA_1_TCD_TCD10_SADDR EDMA_1_TCD.TCD[10].SADDR.R  /* TCD Source Address */
#define EDMA_1_TCD_TCD10_SOFF EDMA_1_TCD.TCD[10].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD10_ATTR EDMA_1_TCD.TCD[10].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD10_NBYTES_MLOFFNO EDMA_1_TCD.TCD[10].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD10_NBYTES_MLOFFYES EDMA_1_TCD.TCD[10].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD10_SLAST_SDA EDMA_1_TCD.TCD[10].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD10_DADDR EDMA_1_TCD.TCD[10].DADDR.R  /* TCD Destination Address */
#define EDMA_1_TCD_TCD10_DOFF EDMA_1_TCD.TCD[10].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD10_CITER_ELINKNO EDMA_1_TCD.TCD[10].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD10_CITER_ELINKYES EDMA_1_TCD.TCD[10].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD10_DLAST_SGA EDMA_1_TCD.TCD[10].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD10_CSR EDMA_1_TCD.TCD[10].CSR.R      /* TCD Control and Status */
#define EDMA_1_TCD_TCD10_BITER_ELINKNO EDMA_1_TCD.TCD[10].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD10_BITER_ELINKYES EDMA_1_TCD.TCD[10].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH11_CSR  EDMA_1_TCD.TCD[11].CH_CSR.R   /* Channel Control and Status */
#define EDMA_1_TCD_CH11_ES   EDMA_1_TCD.TCD[11].CH_ES.R    /* Channel Error Status */
#define EDMA_1_TCD_CH11_INT  EDMA_1_TCD.TCD[11].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_1_TCD_CH11_SBR  EDMA_1_TCD.TCD[11].CH_SBR.R   /* Channel System Bus */
#define EDMA_1_TCD_CH11_PRI  EDMA_1_TCD.TCD[11].CH_PRI.R   /* Channel Priority */
#define EDMA_1_TCD_TCD11_SADDR EDMA_1_TCD.TCD[11].SADDR.R  /* TCD Source Address */
#define EDMA_1_TCD_TCD11_SOFF EDMA_1_TCD.TCD[11].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD11_ATTR EDMA_1_TCD.TCD[11].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD11_NBYTES_MLOFFNO EDMA_1_TCD.TCD[11].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD11_NBYTES_MLOFFYES EDMA_1_TCD.TCD[11].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD11_SLAST_SDA EDMA_1_TCD.TCD[11].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD11_DADDR EDMA_1_TCD.TCD[11].DADDR.R  /* TCD Destination Address */
#define EDMA_1_TCD_TCD11_DOFF EDMA_1_TCD.TCD[11].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD11_CITER_ELINKNO EDMA_1_TCD.TCD[11].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD11_CITER_ELINKYES EDMA_1_TCD.TCD[11].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD11_DLAST_SGA EDMA_1_TCD.TCD[11].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD11_CSR EDMA_1_TCD.TCD[11].CSR.R      /* TCD Control and Status */
#define EDMA_1_TCD_TCD11_BITER_ELINKNO EDMA_1_TCD.TCD[11].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD11_BITER_ELINKYES EDMA_1_TCD.TCD[11].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH12_CSR  EDMA_1_TCD.TCD[12].CH_CSR.R   /* Channel Control and Status */
#define EDMA_1_TCD_CH12_ES   EDMA_1_TCD.TCD[12].CH_ES.R    /* Channel Error Status */
#define EDMA_1_TCD_CH12_INT  EDMA_1_TCD.TCD[12].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_1_TCD_CH12_SBR  EDMA_1_TCD.TCD[12].CH_SBR.R   /* Channel System Bus */
#define EDMA_1_TCD_CH12_PRI  EDMA_1_TCD.TCD[12].CH_PRI.R   /* Channel Priority */
#define EDMA_1_TCD_TCD12_SADDR EDMA_1_TCD.TCD[12].SADDR.R  /* TCD Source Address */
#define EDMA_1_TCD_TCD12_SOFF EDMA_1_TCD.TCD[12].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD12_ATTR EDMA_1_TCD.TCD[12].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD12_NBYTES_MLOFFNO EDMA_1_TCD.TCD[12].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD12_NBYTES_MLOFFYES EDMA_1_TCD.TCD[12].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD12_SLAST_SDA EDMA_1_TCD.TCD[12].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD12_DADDR EDMA_1_TCD.TCD[12].DADDR.R  /* TCD Destination Address */
#define EDMA_1_TCD_TCD12_DOFF EDMA_1_TCD.TCD[12].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD12_CITER_ELINKNO EDMA_1_TCD.TCD[12].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD12_CITER_ELINKYES EDMA_1_TCD.TCD[12].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD12_DLAST_SGA EDMA_1_TCD.TCD[12].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD12_CSR EDMA_1_TCD.TCD[12].CSR.R      /* TCD Control and Status */
#define EDMA_1_TCD_TCD12_BITER_ELINKNO EDMA_1_TCD.TCD[12].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD12_BITER_ELINKYES EDMA_1_TCD.TCD[12].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH13_CSR  EDMA_1_TCD.TCD[13].CH_CSR.R   /* Channel Control and Status */
#define EDMA_1_TCD_CH13_ES   EDMA_1_TCD.TCD[13].CH_ES.R    /* Channel Error Status */
#define EDMA_1_TCD_CH13_INT  EDMA_1_TCD.TCD[13].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_1_TCD_CH13_SBR  EDMA_1_TCD.TCD[13].CH_SBR.R   /* Channel System Bus */
#define EDMA_1_TCD_CH13_PRI  EDMA_1_TCD.TCD[13].CH_PRI.R   /* Channel Priority */
#define EDMA_1_TCD_TCD13_SADDR EDMA_1_TCD.TCD[13].SADDR.R  /* TCD Source Address */
#define EDMA_1_TCD_TCD13_SOFF EDMA_1_TCD.TCD[13].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD13_ATTR EDMA_1_TCD.TCD[13].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD13_NBYTES_MLOFFNO EDMA_1_TCD.TCD[13].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD13_NBYTES_MLOFFYES EDMA_1_TCD.TCD[13].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD13_SLAST_SDA EDMA_1_TCD.TCD[13].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD13_DADDR EDMA_1_TCD.TCD[13].DADDR.R  /* TCD Destination Address */
#define EDMA_1_TCD_TCD13_DOFF EDMA_1_TCD.TCD[13].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD13_CITER_ELINKNO EDMA_1_TCD.TCD[13].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD13_CITER_ELINKYES EDMA_1_TCD.TCD[13].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD13_DLAST_SGA EDMA_1_TCD.TCD[13].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD13_CSR EDMA_1_TCD.TCD[13].CSR.R      /* TCD Control and Status */
#define EDMA_1_TCD_TCD13_BITER_ELINKNO EDMA_1_TCD.TCD[13].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD13_BITER_ELINKYES EDMA_1_TCD.TCD[13].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH14_CSR  EDMA_1_TCD.TCD[14].CH_CSR.R   /* Channel Control and Status */
#define EDMA_1_TCD_CH14_ES   EDMA_1_TCD.TCD[14].CH_ES.R    /* Channel Error Status */
#define EDMA_1_TCD_CH14_INT  EDMA_1_TCD.TCD[14].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_1_TCD_CH14_SBR  EDMA_1_TCD.TCD[14].CH_SBR.R   /* Channel System Bus */
#define EDMA_1_TCD_CH14_PRI  EDMA_1_TCD.TCD[14].CH_PRI.R   /* Channel Priority */
#define EDMA_1_TCD_TCD14_SADDR EDMA_1_TCD.TCD[14].SADDR.R  /* TCD Source Address */
#define EDMA_1_TCD_TCD14_SOFF EDMA_1_TCD.TCD[14].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD14_ATTR EDMA_1_TCD.TCD[14].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD14_NBYTES_MLOFFNO EDMA_1_TCD.TCD[14].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD14_NBYTES_MLOFFYES EDMA_1_TCD.TCD[14].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD14_SLAST_SDA EDMA_1_TCD.TCD[14].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD14_DADDR EDMA_1_TCD.TCD[14].DADDR.R  /* TCD Destination Address */
#define EDMA_1_TCD_TCD14_DOFF EDMA_1_TCD.TCD[14].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD14_CITER_ELINKNO EDMA_1_TCD.TCD[14].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD14_CITER_ELINKYES EDMA_1_TCD.TCD[14].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD14_DLAST_SGA EDMA_1_TCD.TCD[14].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD14_CSR EDMA_1_TCD.TCD[14].CSR.R      /* TCD Control and Status */
#define EDMA_1_TCD_TCD14_BITER_ELINKNO EDMA_1_TCD.TCD[14].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD14_BITER_ELINKYES EDMA_1_TCD.TCD[14].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_CH15_CSR  EDMA_1_TCD.TCD[15].CH_CSR.R   /* Channel Control and Status */
#define EDMA_1_TCD_CH15_ES   EDMA_1_TCD.TCD[15].CH_ES.R    /* Channel Error Status */
#define EDMA_1_TCD_CH15_INT  EDMA_1_TCD.TCD[15].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_1_TCD_CH15_SBR  EDMA_1_TCD.TCD[15].CH_SBR.R   /* Channel System Bus */
#define EDMA_1_TCD_CH15_PRI  EDMA_1_TCD.TCD[15].CH_PRI.R   /* Channel Priority */
#define EDMA_1_TCD_TCD15_SADDR EDMA_1_TCD.TCD[15].SADDR.R  /* TCD Source Address */
#define EDMA_1_TCD_TCD15_SOFF EDMA_1_TCD.TCD[15].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_1_TCD_TCD15_ATTR EDMA_1_TCD.TCD[15].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_1_TCD_TCD15_NBYTES_MLOFFNO EDMA_1_TCD.TCD[15].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_1_TCD_TCD15_NBYTES_MLOFFYES EDMA_1_TCD.TCD[15].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_1_TCD_TCD15_SLAST_SDA EDMA_1_TCD.TCD[15].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_1_TCD_TCD15_DADDR EDMA_1_TCD.TCD[15].DADDR.R  /* TCD Destination Address */
#define EDMA_1_TCD_TCD15_DOFF EDMA_1_TCD.TCD[15].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_1_TCD_TCD15_CITER_ELINKNO EDMA_1_TCD.TCD[15].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD15_CITER_ELINKYES EDMA_1_TCD.TCD[15].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_1_TCD_TCD15_DLAST_SGA EDMA_1_TCD.TCD[15].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_1_TCD_TCD15_CSR EDMA_1_TCD.TCD[15].CSR.R      /* TCD Control and Status */
#define EDMA_1_TCD_TCD15_BITER_ELINKNO EDMA_1_TCD.TCD[15].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_1_TCD_TCD15_BITER_ELINKYES EDMA_1_TCD.TCD[15].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */

#define EDMA_4_TCD_CH0_CSR   EDMA_4_TCD.TCD[0].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH0_ES    EDMA_4_TCD.TCD[0].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH0_INT   EDMA_4_TCD.TCD[0].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH0_SBR   EDMA_4_TCD.TCD[0].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH0_PRI   EDMA_4_TCD.TCD[0].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD0_SADDR EDMA_4_TCD.TCD[0].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD0_SOFF EDMA_4_TCD.TCD[0].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD0_ATTR EDMA_4_TCD.TCD[0].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD0_NBYTES_MLOFFNO EDMA_4_TCD.TCD[0].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD0_NBYTES_MLOFFYES EDMA_4_TCD.TCD[0].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD0_SLAST_SDA EDMA_4_TCD.TCD[0].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD0_DADDR EDMA_4_TCD.TCD[0].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD0_DOFF EDMA_4_TCD.TCD[0].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD0_CITER_ELINKNO EDMA_4_TCD.TCD[0].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD0_CITER_ELINKYES EDMA_4_TCD.TCD[0].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD0_DLAST_SGA EDMA_4_TCD.TCD[0].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD0_CSR  EDMA_4_TCD.TCD[0].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD0_BITER_ELINKNO EDMA_4_TCD.TCD[0].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD0_BITER_ELINKYES EDMA_4_TCD.TCD[0].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH1_CSR   EDMA_4_TCD.TCD[1].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH1_ES    EDMA_4_TCD.TCD[1].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH1_INT   EDMA_4_TCD.TCD[1].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH1_SBR   EDMA_4_TCD.TCD[1].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH1_PRI   EDMA_4_TCD.TCD[1].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD1_SADDR EDMA_4_TCD.TCD[1].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD1_SOFF EDMA_4_TCD.TCD[1].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD1_ATTR EDMA_4_TCD.TCD[1].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD1_NBYTES_MLOFFNO EDMA_4_TCD.TCD[1].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD1_NBYTES_MLOFFYES EDMA_4_TCD.TCD[1].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD1_SLAST_SDA EDMA_4_TCD.TCD[1].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD1_DADDR EDMA_4_TCD.TCD[1].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD1_DOFF EDMA_4_TCD.TCD[1].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD1_CITER_ELINKNO EDMA_4_TCD.TCD[1].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD1_CITER_ELINKYES EDMA_4_TCD.TCD[1].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD1_DLAST_SGA EDMA_4_TCD.TCD[1].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD1_CSR  EDMA_4_TCD.TCD[1].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD1_BITER_ELINKNO EDMA_4_TCD.TCD[1].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD1_BITER_ELINKYES EDMA_4_TCD.TCD[1].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH2_CSR   EDMA_4_TCD.TCD[2].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH2_ES    EDMA_4_TCD.TCD[2].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH2_INT   EDMA_4_TCD.TCD[2].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH2_SBR   EDMA_4_TCD.TCD[2].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH2_PRI   EDMA_4_TCD.TCD[2].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD2_SADDR EDMA_4_TCD.TCD[2].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD2_SOFF EDMA_4_TCD.TCD[2].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD2_ATTR EDMA_4_TCD.TCD[2].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD2_NBYTES_MLOFFNO EDMA_4_TCD.TCD[2].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD2_NBYTES_MLOFFYES EDMA_4_TCD.TCD[2].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD2_SLAST_SDA EDMA_4_TCD.TCD[2].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD2_DADDR EDMA_4_TCD.TCD[2].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD2_DOFF EDMA_4_TCD.TCD[2].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD2_CITER_ELINKNO EDMA_4_TCD.TCD[2].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD2_CITER_ELINKYES EDMA_4_TCD.TCD[2].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD2_DLAST_SGA EDMA_4_TCD.TCD[2].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD2_CSR  EDMA_4_TCD.TCD[2].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD2_BITER_ELINKNO EDMA_4_TCD.TCD[2].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD2_BITER_ELINKYES EDMA_4_TCD.TCD[2].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH3_CSR   EDMA_4_TCD.TCD[3].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH3_ES    EDMA_4_TCD.TCD[3].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH3_INT   EDMA_4_TCD.TCD[3].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH3_SBR   EDMA_4_TCD.TCD[3].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH3_PRI   EDMA_4_TCD.TCD[3].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD3_SADDR EDMA_4_TCD.TCD[3].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD3_SOFF EDMA_4_TCD.TCD[3].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD3_ATTR EDMA_4_TCD.TCD[3].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD3_NBYTES_MLOFFNO EDMA_4_TCD.TCD[3].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD3_NBYTES_MLOFFYES EDMA_4_TCD.TCD[3].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD3_SLAST_SDA EDMA_4_TCD.TCD[3].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD3_DADDR EDMA_4_TCD.TCD[3].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD3_DOFF EDMA_4_TCD.TCD[3].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD3_CITER_ELINKNO EDMA_4_TCD.TCD[3].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD3_CITER_ELINKYES EDMA_4_TCD.TCD[3].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD3_DLAST_SGA EDMA_4_TCD.TCD[3].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD3_CSR  EDMA_4_TCD.TCD[3].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD3_BITER_ELINKNO EDMA_4_TCD.TCD[3].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD3_BITER_ELINKYES EDMA_4_TCD.TCD[3].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH4_CSR   EDMA_4_TCD.TCD[4].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH4_ES    EDMA_4_TCD.TCD[4].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH4_INT   EDMA_4_TCD.TCD[4].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH4_SBR   EDMA_4_TCD.TCD[4].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH4_PRI   EDMA_4_TCD.TCD[4].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD4_SADDR EDMA_4_TCD.TCD[4].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD4_SOFF EDMA_4_TCD.TCD[4].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD4_ATTR EDMA_4_TCD.TCD[4].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD4_NBYTES_MLOFFNO EDMA_4_TCD.TCD[4].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD4_NBYTES_MLOFFYES EDMA_4_TCD.TCD[4].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD4_SLAST_SDA EDMA_4_TCD.TCD[4].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD4_DADDR EDMA_4_TCD.TCD[4].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD4_DOFF EDMA_4_TCD.TCD[4].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD4_CITER_ELINKNO EDMA_4_TCD.TCD[4].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD4_CITER_ELINKYES EDMA_4_TCD.TCD[4].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD4_DLAST_SGA EDMA_4_TCD.TCD[4].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD4_CSR  EDMA_4_TCD.TCD[4].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD4_BITER_ELINKNO EDMA_4_TCD.TCD[4].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD4_BITER_ELINKYES EDMA_4_TCD.TCD[4].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH5_CSR   EDMA_4_TCD.TCD[5].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH5_ES    EDMA_4_TCD.TCD[5].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH5_INT   EDMA_4_TCD.TCD[5].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH5_SBR   EDMA_4_TCD.TCD[5].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH5_PRI   EDMA_4_TCD.TCD[5].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD5_SADDR EDMA_4_TCD.TCD[5].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD5_SOFF EDMA_4_TCD.TCD[5].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD5_ATTR EDMA_4_TCD.TCD[5].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD5_NBYTES_MLOFFNO EDMA_4_TCD.TCD[5].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD5_NBYTES_MLOFFYES EDMA_4_TCD.TCD[5].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD5_SLAST_SDA EDMA_4_TCD.TCD[5].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD5_DADDR EDMA_4_TCD.TCD[5].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD5_DOFF EDMA_4_TCD.TCD[5].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD5_CITER_ELINKNO EDMA_4_TCD.TCD[5].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD5_CITER_ELINKYES EDMA_4_TCD.TCD[5].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD5_DLAST_SGA EDMA_4_TCD.TCD[5].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD5_CSR  EDMA_4_TCD.TCD[5].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD5_BITER_ELINKNO EDMA_4_TCD.TCD[5].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD5_BITER_ELINKYES EDMA_4_TCD.TCD[5].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH6_CSR   EDMA_4_TCD.TCD[6].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH6_ES    EDMA_4_TCD.TCD[6].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH6_INT   EDMA_4_TCD.TCD[6].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH6_SBR   EDMA_4_TCD.TCD[6].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH6_PRI   EDMA_4_TCD.TCD[6].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD6_SADDR EDMA_4_TCD.TCD[6].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD6_SOFF EDMA_4_TCD.TCD[6].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD6_ATTR EDMA_4_TCD.TCD[6].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD6_NBYTES_MLOFFNO EDMA_4_TCD.TCD[6].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD6_NBYTES_MLOFFYES EDMA_4_TCD.TCD[6].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD6_SLAST_SDA EDMA_4_TCD.TCD[6].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD6_DADDR EDMA_4_TCD.TCD[6].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD6_DOFF EDMA_4_TCD.TCD[6].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD6_CITER_ELINKNO EDMA_4_TCD.TCD[6].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD6_CITER_ELINKYES EDMA_4_TCD.TCD[6].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD6_DLAST_SGA EDMA_4_TCD.TCD[6].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD6_CSR  EDMA_4_TCD.TCD[6].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD6_BITER_ELINKNO EDMA_4_TCD.TCD[6].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD6_BITER_ELINKYES EDMA_4_TCD.TCD[6].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH7_CSR   EDMA_4_TCD.TCD[7].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH7_ES    EDMA_4_TCD.TCD[7].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH7_INT   EDMA_4_TCD.TCD[7].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH7_SBR   EDMA_4_TCD.TCD[7].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH7_PRI   EDMA_4_TCD.TCD[7].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD7_SADDR EDMA_4_TCD.TCD[7].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD7_SOFF EDMA_4_TCD.TCD[7].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD7_ATTR EDMA_4_TCD.TCD[7].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD7_NBYTES_MLOFFNO EDMA_4_TCD.TCD[7].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD7_NBYTES_MLOFFYES EDMA_4_TCD.TCD[7].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD7_SLAST_SDA EDMA_4_TCD.TCD[7].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD7_DADDR EDMA_4_TCD.TCD[7].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD7_DOFF EDMA_4_TCD.TCD[7].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD7_CITER_ELINKNO EDMA_4_TCD.TCD[7].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD7_CITER_ELINKYES EDMA_4_TCD.TCD[7].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD7_DLAST_SGA EDMA_4_TCD.TCD[7].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD7_CSR  EDMA_4_TCD.TCD[7].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD7_BITER_ELINKNO EDMA_4_TCD.TCD[7].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD7_BITER_ELINKYES EDMA_4_TCD.TCD[7].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH8_CSR   EDMA_4_TCD.TCD[8].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH8_ES    EDMA_4_TCD.TCD[8].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH8_INT   EDMA_4_TCD.TCD[8].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH8_SBR   EDMA_4_TCD.TCD[8].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH8_PRI   EDMA_4_TCD.TCD[8].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD8_SADDR EDMA_4_TCD.TCD[8].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD8_SOFF EDMA_4_TCD.TCD[8].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD8_ATTR EDMA_4_TCD.TCD[8].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD8_NBYTES_MLOFFNO EDMA_4_TCD.TCD[8].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD8_NBYTES_MLOFFYES EDMA_4_TCD.TCD[8].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD8_SLAST_SDA EDMA_4_TCD.TCD[8].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD8_DADDR EDMA_4_TCD.TCD[8].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD8_DOFF EDMA_4_TCD.TCD[8].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD8_CITER_ELINKNO EDMA_4_TCD.TCD[8].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD8_CITER_ELINKYES EDMA_4_TCD.TCD[8].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD8_DLAST_SGA EDMA_4_TCD.TCD[8].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD8_CSR  EDMA_4_TCD.TCD[8].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD8_BITER_ELINKNO EDMA_4_TCD.TCD[8].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD8_BITER_ELINKYES EDMA_4_TCD.TCD[8].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH9_CSR   EDMA_4_TCD.TCD[9].CH_CSR.R    /* Channel Control and Status */
#define EDMA_4_TCD_CH9_ES    EDMA_4_TCD.TCD[9].CH_ES.R     /* Channel Error Status */
#define EDMA_4_TCD_CH9_INT   EDMA_4_TCD.TCD[9].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_4_TCD_CH9_SBR   EDMA_4_TCD.TCD[9].CH_SBR.R    /* Channel System Bus */
#define EDMA_4_TCD_CH9_PRI   EDMA_4_TCD.TCD[9].CH_PRI.R    /* Channel Priority */
#define EDMA_4_TCD_TCD9_SADDR EDMA_4_TCD.TCD[9].SADDR.R    /* TCD Source Address */
#define EDMA_4_TCD_TCD9_SOFF EDMA_4_TCD.TCD[9].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD9_ATTR EDMA_4_TCD.TCD[9].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD9_NBYTES_MLOFFNO EDMA_4_TCD.TCD[9].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD9_NBYTES_MLOFFYES EDMA_4_TCD.TCD[9].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD9_SLAST_SDA EDMA_4_TCD.TCD[9].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD9_DADDR EDMA_4_TCD.TCD[9].DADDR.R    /* TCD Destination Address */
#define EDMA_4_TCD_TCD9_DOFF EDMA_4_TCD.TCD[9].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD9_CITER_ELINKNO EDMA_4_TCD.TCD[9].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD9_CITER_ELINKYES EDMA_4_TCD.TCD[9].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD9_DLAST_SGA EDMA_4_TCD.TCD[9].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD9_CSR  EDMA_4_TCD.TCD[9].CSR.R       /* TCD Control and Status */
#define EDMA_4_TCD_TCD9_BITER_ELINKNO EDMA_4_TCD.TCD[9].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD9_BITER_ELINKYES EDMA_4_TCD.TCD[9].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH10_CSR  EDMA_4_TCD.TCD[10].CH_CSR.R   /* Channel Control and Status */
#define EDMA_4_TCD_CH10_ES   EDMA_4_TCD.TCD[10].CH_ES.R    /* Channel Error Status */
#define EDMA_4_TCD_CH10_INT  EDMA_4_TCD.TCD[10].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_4_TCD_CH10_SBR  EDMA_4_TCD.TCD[10].CH_SBR.R   /* Channel System Bus */
#define EDMA_4_TCD_CH10_PRI  EDMA_4_TCD.TCD[10].CH_PRI.R   /* Channel Priority */
#define EDMA_4_TCD_TCD10_SADDR EDMA_4_TCD.TCD[10].SADDR.R  /* TCD Source Address */
#define EDMA_4_TCD_TCD10_SOFF EDMA_4_TCD.TCD[10].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD10_ATTR EDMA_4_TCD.TCD[10].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD10_NBYTES_MLOFFNO EDMA_4_TCD.TCD[10].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD10_NBYTES_MLOFFYES EDMA_4_TCD.TCD[10].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD10_SLAST_SDA EDMA_4_TCD.TCD[10].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD10_DADDR EDMA_4_TCD.TCD[10].DADDR.R  /* TCD Destination Address */
#define EDMA_4_TCD_TCD10_DOFF EDMA_4_TCD.TCD[10].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD10_CITER_ELINKNO EDMA_4_TCD.TCD[10].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD10_CITER_ELINKYES EDMA_4_TCD.TCD[10].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD10_DLAST_SGA EDMA_4_TCD.TCD[10].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD10_CSR EDMA_4_TCD.TCD[10].CSR.R      /* TCD Control and Status */
#define EDMA_4_TCD_TCD10_BITER_ELINKNO EDMA_4_TCD.TCD[10].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD10_BITER_ELINKYES EDMA_4_TCD.TCD[10].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH11_CSR  EDMA_4_TCD.TCD[11].CH_CSR.R   /* Channel Control and Status */
#define EDMA_4_TCD_CH11_ES   EDMA_4_TCD.TCD[11].CH_ES.R    /* Channel Error Status */
#define EDMA_4_TCD_CH11_INT  EDMA_4_TCD.TCD[11].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_4_TCD_CH11_SBR  EDMA_4_TCD.TCD[11].CH_SBR.R   /* Channel System Bus */
#define EDMA_4_TCD_CH11_PRI  EDMA_4_TCD.TCD[11].CH_PRI.R   /* Channel Priority */
#define EDMA_4_TCD_TCD11_SADDR EDMA_4_TCD.TCD[11].SADDR.R  /* TCD Source Address */
#define EDMA_4_TCD_TCD11_SOFF EDMA_4_TCD.TCD[11].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD11_ATTR EDMA_4_TCD.TCD[11].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD11_NBYTES_MLOFFNO EDMA_4_TCD.TCD[11].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD11_NBYTES_MLOFFYES EDMA_4_TCD.TCD[11].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD11_SLAST_SDA EDMA_4_TCD.TCD[11].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD11_DADDR EDMA_4_TCD.TCD[11].DADDR.R  /* TCD Destination Address */
#define EDMA_4_TCD_TCD11_DOFF EDMA_4_TCD.TCD[11].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD11_CITER_ELINKNO EDMA_4_TCD.TCD[11].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD11_CITER_ELINKYES EDMA_4_TCD.TCD[11].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD11_DLAST_SGA EDMA_4_TCD.TCD[11].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD11_CSR EDMA_4_TCD.TCD[11].CSR.R      /* TCD Control and Status */
#define EDMA_4_TCD_TCD11_BITER_ELINKNO EDMA_4_TCD.TCD[11].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD11_BITER_ELINKYES EDMA_4_TCD.TCD[11].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH12_CSR  EDMA_4_TCD.TCD[12].CH_CSR.R   /* Channel Control and Status */
#define EDMA_4_TCD_CH12_ES   EDMA_4_TCD.TCD[12].CH_ES.R    /* Channel Error Status */
#define EDMA_4_TCD_CH12_INT  EDMA_4_TCD.TCD[12].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_4_TCD_CH12_SBR  EDMA_4_TCD.TCD[12].CH_SBR.R   /* Channel System Bus */
#define EDMA_4_TCD_CH12_PRI  EDMA_4_TCD.TCD[12].CH_PRI.R   /* Channel Priority */
#define EDMA_4_TCD_TCD12_SADDR EDMA_4_TCD.TCD[12].SADDR.R  /* TCD Source Address */
#define EDMA_4_TCD_TCD12_SOFF EDMA_4_TCD.TCD[12].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD12_ATTR EDMA_4_TCD.TCD[12].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD12_NBYTES_MLOFFNO EDMA_4_TCD.TCD[12].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD12_NBYTES_MLOFFYES EDMA_4_TCD.TCD[12].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD12_SLAST_SDA EDMA_4_TCD.TCD[12].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD12_DADDR EDMA_4_TCD.TCD[12].DADDR.R  /* TCD Destination Address */
#define EDMA_4_TCD_TCD12_DOFF EDMA_4_TCD.TCD[12].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD12_CITER_ELINKNO EDMA_4_TCD.TCD[12].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD12_CITER_ELINKYES EDMA_4_TCD.TCD[12].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD12_DLAST_SGA EDMA_4_TCD.TCD[12].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD12_CSR EDMA_4_TCD.TCD[12].CSR.R      /* TCD Control and Status */
#define EDMA_4_TCD_TCD12_BITER_ELINKNO EDMA_4_TCD.TCD[12].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD12_BITER_ELINKYES EDMA_4_TCD.TCD[12].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH13_CSR  EDMA_4_TCD.TCD[13].CH_CSR.R   /* Channel Control and Status */
#define EDMA_4_TCD_CH13_ES   EDMA_4_TCD.TCD[13].CH_ES.R    /* Channel Error Status */
#define EDMA_4_TCD_CH13_INT  EDMA_4_TCD.TCD[13].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_4_TCD_CH13_SBR  EDMA_4_TCD.TCD[13].CH_SBR.R   /* Channel System Bus */
#define EDMA_4_TCD_CH13_PRI  EDMA_4_TCD.TCD[13].CH_PRI.R   /* Channel Priority */
#define EDMA_4_TCD_TCD13_SADDR EDMA_4_TCD.TCD[13].SADDR.R  /* TCD Source Address */
#define EDMA_4_TCD_TCD13_SOFF EDMA_4_TCD.TCD[13].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD13_ATTR EDMA_4_TCD.TCD[13].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD13_NBYTES_MLOFFNO EDMA_4_TCD.TCD[13].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD13_NBYTES_MLOFFYES EDMA_4_TCD.TCD[13].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD13_SLAST_SDA EDMA_4_TCD.TCD[13].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD13_DADDR EDMA_4_TCD.TCD[13].DADDR.R  /* TCD Destination Address */
#define EDMA_4_TCD_TCD13_DOFF EDMA_4_TCD.TCD[13].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD13_CITER_ELINKNO EDMA_4_TCD.TCD[13].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD13_CITER_ELINKYES EDMA_4_TCD.TCD[13].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD13_DLAST_SGA EDMA_4_TCD.TCD[13].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD13_CSR EDMA_4_TCD.TCD[13].CSR.R      /* TCD Control and Status */
#define EDMA_4_TCD_TCD13_BITER_ELINKNO EDMA_4_TCD.TCD[13].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD13_BITER_ELINKYES EDMA_4_TCD.TCD[13].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH14_CSR  EDMA_4_TCD.TCD[14].CH_CSR.R   /* Channel Control and Status */
#define EDMA_4_TCD_CH14_ES   EDMA_4_TCD.TCD[14].CH_ES.R    /* Channel Error Status */
#define EDMA_4_TCD_CH14_INT  EDMA_4_TCD.TCD[14].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_4_TCD_CH14_SBR  EDMA_4_TCD.TCD[14].CH_SBR.R   /* Channel System Bus */
#define EDMA_4_TCD_CH14_PRI  EDMA_4_TCD.TCD[14].CH_PRI.R   /* Channel Priority */
#define EDMA_4_TCD_TCD14_SADDR EDMA_4_TCD.TCD[14].SADDR.R  /* TCD Source Address */
#define EDMA_4_TCD_TCD14_SOFF EDMA_4_TCD.TCD[14].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD14_ATTR EDMA_4_TCD.TCD[14].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD14_NBYTES_MLOFFNO EDMA_4_TCD.TCD[14].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD14_NBYTES_MLOFFYES EDMA_4_TCD.TCD[14].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD14_SLAST_SDA EDMA_4_TCD.TCD[14].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD14_DADDR EDMA_4_TCD.TCD[14].DADDR.R  /* TCD Destination Address */
#define EDMA_4_TCD_TCD14_DOFF EDMA_4_TCD.TCD[14].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD14_CITER_ELINKNO EDMA_4_TCD.TCD[14].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD14_CITER_ELINKYES EDMA_4_TCD.TCD[14].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD14_DLAST_SGA EDMA_4_TCD.TCD[14].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD14_CSR EDMA_4_TCD.TCD[14].CSR.R      /* TCD Control and Status */
#define EDMA_4_TCD_TCD14_BITER_ELINKNO EDMA_4_TCD.TCD[14].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD14_BITER_ELINKYES EDMA_4_TCD.TCD[14].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_CH15_CSR  EDMA_4_TCD.TCD[15].CH_CSR.R   /* Channel Control and Status */
#define EDMA_4_TCD_CH15_ES   EDMA_4_TCD.TCD[15].CH_ES.R    /* Channel Error Status */
#define EDMA_4_TCD_CH15_INT  EDMA_4_TCD.TCD[15].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_4_TCD_CH15_SBR  EDMA_4_TCD.TCD[15].CH_SBR.R   /* Channel System Bus */
#define EDMA_4_TCD_CH15_PRI  EDMA_4_TCD.TCD[15].CH_PRI.R   /* Channel Priority */
#define EDMA_4_TCD_TCD15_SADDR EDMA_4_TCD.TCD[15].SADDR.R  /* TCD Source Address */
#define EDMA_4_TCD_TCD15_SOFF EDMA_4_TCD.TCD[15].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_4_TCD_TCD15_ATTR EDMA_4_TCD.TCD[15].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_4_TCD_TCD15_NBYTES_MLOFFNO EDMA_4_TCD.TCD[15].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_4_TCD_TCD15_NBYTES_MLOFFYES EDMA_4_TCD.TCD[15].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_4_TCD_TCD15_SLAST_SDA EDMA_4_TCD.TCD[15].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_4_TCD_TCD15_DADDR EDMA_4_TCD.TCD[15].DADDR.R  /* TCD Destination Address */
#define EDMA_4_TCD_TCD15_DOFF EDMA_4_TCD.TCD[15].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_4_TCD_TCD15_CITER_ELINKNO EDMA_4_TCD.TCD[15].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD15_CITER_ELINKYES EDMA_4_TCD.TCD[15].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_4_TCD_TCD15_DLAST_SGA EDMA_4_TCD.TCD[15].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_4_TCD_TCD15_CSR EDMA_4_TCD.TCD[15].CSR.R      /* TCD Control and Status */
#define EDMA_4_TCD_TCD15_BITER_ELINKNO EDMA_4_TCD.TCD[15].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_4_TCD_TCD15_BITER_ELINKYES EDMA_4_TCD.TCD[15].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */

#define EDMA_5_TCD_CH0_CSR   EDMA_5_TCD.TCD[0].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH0_ES    EDMA_5_TCD.TCD[0].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH0_INT   EDMA_5_TCD.TCD[0].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH0_SBR   EDMA_5_TCD.TCD[0].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH0_PRI   EDMA_5_TCD.TCD[0].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD0_SADDR EDMA_5_TCD.TCD[0].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD0_SOFF EDMA_5_TCD.TCD[0].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD0_ATTR EDMA_5_TCD.TCD[0].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD0_NBYTES_MLOFFNO EDMA_5_TCD.TCD[0].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD0_NBYTES_MLOFFYES EDMA_5_TCD.TCD[0].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD0_SLAST_SDA EDMA_5_TCD.TCD[0].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD0_DADDR EDMA_5_TCD.TCD[0].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD0_DOFF EDMA_5_TCD.TCD[0].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD0_CITER_ELINKNO EDMA_5_TCD.TCD[0].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD0_CITER_ELINKYES EDMA_5_TCD.TCD[0].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD0_DLAST_SGA EDMA_5_TCD.TCD[0].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD0_CSR  EDMA_5_TCD.TCD[0].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD0_BITER_ELINKNO EDMA_5_TCD.TCD[0].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD0_BITER_ELINKYES EDMA_5_TCD.TCD[0].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH1_CSR   EDMA_5_TCD.TCD[1].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH1_ES    EDMA_5_TCD.TCD[1].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH1_INT   EDMA_5_TCD.TCD[1].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH1_SBR   EDMA_5_TCD.TCD[1].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH1_PRI   EDMA_5_TCD.TCD[1].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD1_SADDR EDMA_5_TCD.TCD[1].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD1_SOFF EDMA_5_TCD.TCD[1].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD1_ATTR EDMA_5_TCD.TCD[1].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD1_NBYTES_MLOFFNO EDMA_5_TCD.TCD[1].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD1_NBYTES_MLOFFYES EDMA_5_TCD.TCD[1].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD1_SLAST_SDA EDMA_5_TCD.TCD[1].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD1_DADDR EDMA_5_TCD.TCD[1].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD1_DOFF EDMA_5_TCD.TCD[1].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD1_CITER_ELINKNO EDMA_5_TCD.TCD[1].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD1_CITER_ELINKYES EDMA_5_TCD.TCD[1].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD1_DLAST_SGA EDMA_5_TCD.TCD[1].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD1_CSR  EDMA_5_TCD.TCD[1].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD1_BITER_ELINKNO EDMA_5_TCD.TCD[1].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD1_BITER_ELINKYES EDMA_5_TCD.TCD[1].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH2_CSR   EDMA_5_TCD.TCD[2].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH2_ES    EDMA_5_TCD.TCD[2].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH2_INT   EDMA_5_TCD.TCD[2].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH2_SBR   EDMA_5_TCD.TCD[2].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH2_PRI   EDMA_5_TCD.TCD[2].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD2_SADDR EDMA_5_TCD.TCD[2].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD2_SOFF EDMA_5_TCD.TCD[2].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD2_ATTR EDMA_5_TCD.TCD[2].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD2_NBYTES_MLOFFNO EDMA_5_TCD.TCD[2].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD2_NBYTES_MLOFFYES EDMA_5_TCD.TCD[2].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD2_SLAST_SDA EDMA_5_TCD.TCD[2].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD2_DADDR EDMA_5_TCD.TCD[2].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD2_DOFF EDMA_5_TCD.TCD[2].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD2_CITER_ELINKNO EDMA_5_TCD.TCD[2].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD2_CITER_ELINKYES EDMA_5_TCD.TCD[2].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD2_DLAST_SGA EDMA_5_TCD.TCD[2].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD2_CSR  EDMA_5_TCD.TCD[2].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD2_BITER_ELINKNO EDMA_5_TCD.TCD[2].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD2_BITER_ELINKYES EDMA_5_TCD.TCD[2].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH3_CSR   EDMA_5_TCD.TCD[3].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH3_ES    EDMA_5_TCD.TCD[3].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH3_INT   EDMA_5_TCD.TCD[3].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH3_SBR   EDMA_5_TCD.TCD[3].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH3_PRI   EDMA_5_TCD.TCD[3].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD3_SADDR EDMA_5_TCD.TCD[3].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD3_SOFF EDMA_5_TCD.TCD[3].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD3_ATTR EDMA_5_TCD.TCD[3].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD3_NBYTES_MLOFFNO EDMA_5_TCD.TCD[3].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD3_NBYTES_MLOFFYES EDMA_5_TCD.TCD[3].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD3_SLAST_SDA EDMA_5_TCD.TCD[3].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD3_DADDR EDMA_5_TCD.TCD[3].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD3_DOFF EDMA_5_TCD.TCD[3].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD3_CITER_ELINKNO EDMA_5_TCD.TCD[3].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD3_CITER_ELINKYES EDMA_5_TCD.TCD[3].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD3_DLAST_SGA EDMA_5_TCD.TCD[3].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD3_CSR  EDMA_5_TCD.TCD[3].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD3_BITER_ELINKNO EDMA_5_TCD.TCD[3].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD3_BITER_ELINKYES EDMA_5_TCD.TCD[3].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH4_CSR   EDMA_5_TCD.TCD[4].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH4_ES    EDMA_5_TCD.TCD[4].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH4_INT   EDMA_5_TCD.TCD[4].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH4_SBR   EDMA_5_TCD.TCD[4].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH4_PRI   EDMA_5_TCD.TCD[4].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD4_SADDR EDMA_5_TCD.TCD[4].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD4_SOFF EDMA_5_TCD.TCD[4].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD4_ATTR EDMA_5_TCD.TCD[4].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD4_NBYTES_MLOFFNO EDMA_5_TCD.TCD[4].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD4_NBYTES_MLOFFYES EDMA_5_TCD.TCD[4].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD4_SLAST_SDA EDMA_5_TCD.TCD[4].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD4_DADDR EDMA_5_TCD.TCD[4].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD4_DOFF EDMA_5_TCD.TCD[4].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD4_CITER_ELINKNO EDMA_5_TCD.TCD[4].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD4_CITER_ELINKYES EDMA_5_TCD.TCD[4].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD4_DLAST_SGA EDMA_5_TCD.TCD[4].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD4_CSR  EDMA_5_TCD.TCD[4].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD4_BITER_ELINKNO EDMA_5_TCD.TCD[4].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD4_BITER_ELINKYES EDMA_5_TCD.TCD[4].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH5_CSR   EDMA_5_TCD.TCD[5].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH5_ES    EDMA_5_TCD.TCD[5].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH5_INT   EDMA_5_TCD.TCD[5].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH5_SBR   EDMA_5_TCD.TCD[5].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH5_PRI   EDMA_5_TCD.TCD[5].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD5_SADDR EDMA_5_TCD.TCD[5].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD5_SOFF EDMA_5_TCD.TCD[5].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD5_ATTR EDMA_5_TCD.TCD[5].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD5_NBYTES_MLOFFNO EDMA_5_TCD.TCD[5].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD5_NBYTES_MLOFFYES EDMA_5_TCD.TCD[5].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD5_SLAST_SDA EDMA_5_TCD.TCD[5].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD5_DADDR EDMA_5_TCD.TCD[5].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD5_DOFF EDMA_5_TCD.TCD[5].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD5_CITER_ELINKNO EDMA_5_TCD.TCD[5].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD5_CITER_ELINKYES EDMA_5_TCD.TCD[5].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD5_DLAST_SGA EDMA_5_TCD.TCD[5].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD5_CSR  EDMA_5_TCD.TCD[5].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD5_BITER_ELINKNO EDMA_5_TCD.TCD[5].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD5_BITER_ELINKYES EDMA_5_TCD.TCD[5].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH6_CSR   EDMA_5_TCD.TCD[6].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH6_ES    EDMA_5_TCD.TCD[6].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH6_INT   EDMA_5_TCD.TCD[6].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH6_SBR   EDMA_5_TCD.TCD[6].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH6_PRI   EDMA_5_TCD.TCD[6].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD6_SADDR EDMA_5_TCD.TCD[6].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD6_SOFF EDMA_5_TCD.TCD[6].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD6_ATTR EDMA_5_TCD.TCD[6].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD6_NBYTES_MLOFFNO EDMA_5_TCD.TCD[6].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD6_NBYTES_MLOFFYES EDMA_5_TCD.TCD[6].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD6_SLAST_SDA EDMA_5_TCD.TCD[6].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD6_DADDR EDMA_5_TCD.TCD[6].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD6_DOFF EDMA_5_TCD.TCD[6].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD6_CITER_ELINKNO EDMA_5_TCD.TCD[6].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD6_CITER_ELINKYES EDMA_5_TCD.TCD[6].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD6_DLAST_SGA EDMA_5_TCD.TCD[6].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD6_CSR  EDMA_5_TCD.TCD[6].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD6_BITER_ELINKNO EDMA_5_TCD.TCD[6].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD6_BITER_ELINKYES EDMA_5_TCD.TCD[6].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH7_CSR   EDMA_5_TCD.TCD[7].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH7_ES    EDMA_5_TCD.TCD[7].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH7_INT   EDMA_5_TCD.TCD[7].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH7_SBR   EDMA_5_TCD.TCD[7].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH7_PRI   EDMA_5_TCD.TCD[7].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD7_SADDR EDMA_5_TCD.TCD[7].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD7_SOFF EDMA_5_TCD.TCD[7].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD7_ATTR EDMA_5_TCD.TCD[7].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD7_NBYTES_MLOFFNO EDMA_5_TCD.TCD[7].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD7_NBYTES_MLOFFYES EDMA_5_TCD.TCD[7].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD7_SLAST_SDA EDMA_5_TCD.TCD[7].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD7_DADDR EDMA_5_TCD.TCD[7].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD7_DOFF EDMA_5_TCD.TCD[7].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD7_CITER_ELINKNO EDMA_5_TCD.TCD[7].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD7_CITER_ELINKYES EDMA_5_TCD.TCD[7].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD7_DLAST_SGA EDMA_5_TCD.TCD[7].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD7_CSR  EDMA_5_TCD.TCD[7].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD7_BITER_ELINKNO EDMA_5_TCD.TCD[7].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD7_BITER_ELINKYES EDMA_5_TCD.TCD[7].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH8_CSR   EDMA_5_TCD.TCD[8].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH8_ES    EDMA_5_TCD.TCD[8].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH8_INT   EDMA_5_TCD.TCD[8].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH8_SBR   EDMA_5_TCD.TCD[8].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH8_PRI   EDMA_5_TCD.TCD[8].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD8_SADDR EDMA_5_TCD.TCD[8].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD8_SOFF EDMA_5_TCD.TCD[8].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD8_ATTR EDMA_5_TCD.TCD[8].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD8_NBYTES_MLOFFNO EDMA_5_TCD.TCD[8].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD8_NBYTES_MLOFFYES EDMA_5_TCD.TCD[8].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD8_SLAST_SDA EDMA_5_TCD.TCD[8].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD8_DADDR EDMA_5_TCD.TCD[8].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD8_DOFF EDMA_5_TCD.TCD[8].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD8_CITER_ELINKNO EDMA_5_TCD.TCD[8].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD8_CITER_ELINKYES EDMA_5_TCD.TCD[8].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD8_DLAST_SGA EDMA_5_TCD.TCD[8].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD8_CSR  EDMA_5_TCD.TCD[8].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD8_BITER_ELINKNO EDMA_5_TCD.TCD[8].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD8_BITER_ELINKYES EDMA_5_TCD.TCD[8].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH9_CSR   EDMA_5_TCD.TCD[9].CH_CSR.R    /* Channel Control and Status */
#define EDMA_5_TCD_CH9_ES    EDMA_5_TCD.TCD[9].CH_ES.R     /* Channel Error Status */
#define EDMA_5_TCD_CH9_INT   EDMA_5_TCD.TCD[9].CH_INT.R    /* Channel Interrupt Status */
#define EDMA_5_TCD_CH9_SBR   EDMA_5_TCD.TCD[9].CH_SBR.R    /* Channel System Bus */
#define EDMA_5_TCD_CH9_PRI   EDMA_5_TCD.TCD[9].CH_PRI.R    /* Channel Priority */
#define EDMA_5_TCD_TCD9_SADDR EDMA_5_TCD.TCD[9].SADDR.R    /* TCD Source Address */
#define EDMA_5_TCD_TCD9_SOFF EDMA_5_TCD.TCD[9].SOFF.R      /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD9_ATTR EDMA_5_TCD.TCD[9].ATTR.R      /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD9_NBYTES_MLOFFNO EDMA_5_TCD.TCD[9].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD9_NBYTES_MLOFFYES EDMA_5_TCD.TCD[9].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD9_SLAST_SDA EDMA_5_TCD.TCD[9].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD9_DADDR EDMA_5_TCD.TCD[9].DADDR.R    /* TCD Destination Address */
#define EDMA_5_TCD_TCD9_DOFF EDMA_5_TCD.TCD[9].DOFF.R      /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD9_CITER_ELINKNO EDMA_5_TCD.TCD[9].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD9_CITER_ELINKYES EDMA_5_TCD.TCD[9].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD9_DLAST_SGA EDMA_5_TCD.TCD[9].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD9_CSR  EDMA_5_TCD.TCD[9].CSR.R       /* TCD Control and Status */
#define EDMA_5_TCD_TCD9_BITER_ELINKNO EDMA_5_TCD.TCD[9].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD9_BITER_ELINKYES EDMA_5_TCD.TCD[9].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH10_CSR  EDMA_5_TCD.TCD[10].CH_CSR.R   /* Channel Control and Status */
#define EDMA_5_TCD_CH10_ES   EDMA_5_TCD.TCD[10].CH_ES.R    /* Channel Error Status */
#define EDMA_5_TCD_CH10_INT  EDMA_5_TCD.TCD[10].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_5_TCD_CH10_SBR  EDMA_5_TCD.TCD[10].CH_SBR.R   /* Channel System Bus */
#define EDMA_5_TCD_CH10_PRI  EDMA_5_TCD.TCD[10].CH_PRI.R   /* Channel Priority */
#define EDMA_5_TCD_TCD10_SADDR EDMA_5_TCD.TCD[10].SADDR.R  /* TCD Source Address */
#define EDMA_5_TCD_TCD10_SOFF EDMA_5_TCD.TCD[10].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD10_ATTR EDMA_5_TCD.TCD[10].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD10_NBYTES_MLOFFNO EDMA_5_TCD.TCD[10].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD10_NBYTES_MLOFFYES EDMA_5_TCD.TCD[10].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD10_SLAST_SDA EDMA_5_TCD.TCD[10].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD10_DADDR EDMA_5_TCD.TCD[10].DADDR.R  /* TCD Destination Address */
#define EDMA_5_TCD_TCD10_DOFF EDMA_5_TCD.TCD[10].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD10_CITER_ELINKNO EDMA_5_TCD.TCD[10].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD10_CITER_ELINKYES EDMA_5_TCD.TCD[10].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD10_DLAST_SGA EDMA_5_TCD.TCD[10].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD10_CSR EDMA_5_TCD.TCD[10].CSR.R      /* TCD Control and Status */
#define EDMA_5_TCD_TCD10_BITER_ELINKNO EDMA_5_TCD.TCD[10].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD10_BITER_ELINKYES EDMA_5_TCD.TCD[10].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH11_CSR  EDMA_5_TCD.TCD[11].CH_CSR.R   /* Channel Control and Status */
#define EDMA_5_TCD_CH11_ES   EDMA_5_TCD.TCD[11].CH_ES.R    /* Channel Error Status */
#define EDMA_5_TCD_CH11_INT  EDMA_5_TCD.TCD[11].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_5_TCD_CH11_SBR  EDMA_5_TCD.TCD[11].CH_SBR.R   /* Channel System Bus */
#define EDMA_5_TCD_CH11_PRI  EDMA_5_TCD.TCD[11].CH_PRI.R   /* Channel Priority */
#define EDMA_5_TCD_TCD11_SADDR EDMA_5_TCD.TCD[11].SADDR.R  /* TCD Source Address */
#define EDMA_5_TCD_TCD11_SOFF EDMA_5_TCD.TCD[11].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD11_ATTR EDMA_5_TCD.TCD[11].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD11_NBYTES_MLOFFNO EDMA_5_TCD.TCD[11].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD11_NBYTES_MLOFFYES EDMA_5_TCD.TCD[11].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD11_SLAST_SDA EDMA_5_TCD.TCD[11].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD11_DADDR EDMA_5_TCD.TCD[11].DADDR.R  /* TCD Destination Address */
#define EDMA_5_TCD_TCD11_DOFF EDMA_5_TCD.TCD[11].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD11_CITER_ELINKNO EDMA_5_TCD.TCD[11].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD11_CITER_ELINKYES EDMA_5_TCD.TCD[11].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD11_DLAST_SGA EDMA_5_TCD.TCD[11].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD11_CSR EDMA_5_TCD.TCD[11].CSR.R      /* TCD Control and Status */
#define EDMA_5_TCD_TCD11_BITER_ELINKNO EDMA_5_TCD.TCD[11].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD11_BITER_ELINKYES EDMA_5_TCD.TCD[11].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH12_CSR  EDMA_5_TCD.TCD[12].CH_CSR.R   /* Channel Control and Status */
#define EDMA_5_TCD_CH12_ES   EDMA_5_TCD.TCD[12].CH_ES.R    /* Channel Error Status */
#define EDMA_5_TCD_CH12_INT  EDMA_5_TCD.TCD[12].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_5_TCD_CH12_SBR  EDMA_5_TCD.TCD[12].CH_SBR.R   /* Channel System Bus */
#define EDMA_5_TCD_CH12_PRI  EDMA_5_TCD.TCD[12].CH_PRI.R   /* Channel Priority */
#define EDMA_5_TCD_TCD12_SADDR EDMA_5_TCD.TCD[12].SADDR.R  /* TCD Source Address */
#define EDMA_5_TCD_TCD12_SOFF EDMA_5_TCD.TCD[12].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD12_ATTR EDMA_5_TCD.TCD[12].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD12_NBYTES_MLOFFNO EDMA_5_TCD.TCD[12].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD12_NBYTES_MLOFFYES EDMA_5_TCD.TCD[12].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD12_SLAST_SDA EDMA_5_TCD.TCD[12].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD12_DADDR EDMA_5_TCD.TCD[12].DADDR.R  /* TCD Destination Address */
#define EDMA_5_TCD_TCD12_DOFF EDMA_5_TCD.TCD[12].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD12_CITER_ELINKNO EDMA_5_TCD.TCD[12].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD12_CITER_ELINKYES EDMA_5_TCD.TCD[12].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD12_DLAST_SGA EDMA_5_TCD.TCD[12].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD12_CSR EDMA_5_TCD.TCD[12].CSR.R      /* TCD Control and Status */
#define EDMA_5_TCD_TCD12_BITER_ELINKNO EDMA_5_TCD.TCD[12].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD12_BITER_ELINKYES EDMA_5_TCD.TCD[12].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH13_CSR  EDMA_5_TCD.TCD[13].CH_CSR.R   /* Channel Control and Status */
#define EDMA_5_TCD_CH13_ES   EDMA_5_TCD.TCD[13].CH_ES.R    /* Channel Error Status */
#define EDMA_5_TCD_CH13_INT  EDMA_5_TCD.TCD[13].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_5_TCD_CH13_SBR  EDMA_5_TCD.TCD[13].CH_SBR.R   /* Channel System Bus */
#define EDMA_5_TCD_CH13_PRI  EDMA_5_TCD.TCD[13].CH_PRI.R   /* Channel Priority */
#define EDMA_5_TCD_TCD13_SADDR EDMA_5_TCD.TCD[13].SADDR.R  /* TCD Source Address */
#define EDMA_5_TCD_TCD13_SOFF EDMA_5_TCD.TCD[13].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD13_ATTR EDMA_5_TCD.TCD[13].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD13_NBYTES_MLOFFNO EDMA_5_TCD.TCD[13].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD13_NBYTES_MLOFFYES EDMA_5_TCD.TCD[13].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD13_SLAST_SDA EDMA_5_TCD.TCD[13].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD13_DADDR EDMA_5_TCD.TCD[13].DADDR.R  /* TCD Destination Address */
#define EDMA_5_TCD_TCD13_DOFF EDMA_5_TCD.TCD[13].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD13_CITER_ELINKNO EDMA_5_TCD.TCD[13].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD13_CITER_ELINKYES EDMA_5_TCD.TCD[13].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD13_DLAST_SGA EDMA_5_TCD.TCD[13].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD13_CSR EDMA_5_TCD.TCD[13].CSR.R      /* TCD Control and Status */
#define EDMA_5_TCD_TCD13_BITER_ELINKNO EDMA_5_TCD.TCD[13].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD13_BITER_ELINKYES EDMA_5_TCD.TCD[13].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH14_CSR  EDMA_5_TCD.TCD[14].CH_CSR.R   /* Channel Control and Status */
#define EDMA_5_TCD_CH14_ES   EDMA_5_TCD.TCD[14].CH_ES.R    /* Channel Error Status */
#define EDMA_5_TCD_CH14_INT  EDMA_5_TCD.TCD[14].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_5_TCD_CH14_SBR  EDMA_5_TCD.TCD[14].CH_SBR.R   /* Channel System Bus */
#define EDMA_5_TCD_CH14_PRI  EDMA_5_TCD.TCD[14].CH_PRI.R   /* Channel Priority */
#define EDMA_5_TCD_TCD14_SADDR EDMA_5_TCD.TCD[14].SADDR.R  /* TCD Source Address */
#define EDMA_5_TCD_TCD14_SOFF EDMA_5_TCD.TCD[14].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD14_ATTR EDMA_5_TCD.TCD[14].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD14_NBYTES_MLOFFNO EDMA_5_TCD.TCD[14].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD14_NBYTES_MLOFFYES EDMA_5_TCD.TCD[14].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD14_SLAST_SDA EDMA_5_TCD.TCD[14].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD14_DADDR EDMA_5_TCD.TCD[14].DADDR.R  /* TCD Destination Address */
#define EDMA_5_TCD_TCD14_DOFF EDMA_5_TCD.TCD[14].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD14_CITER_ELINKNO EDMA_5_TCD.TCD[14].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD14_CITER_ELINKYES EDMA_5_TCD.TCD[14].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD14_DLAST_SGA EDMA_5_TCD.TCD[14].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD14_CSR EDMA_5_TCD.TCD[14].CSR.R      /* TCD Control and Status */
#define EDMA_5_TCD_TCD14_BITER_ELINKNO EDMA_5_TCD.TCD[14].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD14_BITER_ELINKYES EDMA_5_TCD.TCD[14].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_CH15_CSR  EDMA_5_TCD.TCD[15].CH_CSR.R   /* Channel Control and Status */
#define EDMA_5_TCD_CH15_ES   EDMA_5_TCD.TCD[15].CH_ES.R    /* Channel Error Status */
#define EDMA_5_TCD_CH15_INT  EDMA_5_TCD.TCD[15].CH_INT.R   /* Channel Interrupt Status */
#define EDMA_5_TCD_CH15_SBR  EDMA_5_TCD.TCD[15].CH_SBR.R   /* Channel System Bus */
#define EDMA_5_TCD_CH15_PRI  EDMA_5_TCD.TCD[15].CH_PRI.R   /* Channel Priority */
#define EDMA_5_TCD_TCD15_SADDR EDMA_5_TCD.TCD[15].SADDR.R  /* TCD Source Address */
#define EDMA_5_TCD_TCD15_SOFF EDMA_5_TCD.TCD[15].SOFF.R    /* TCD Signed Source Address Offset */
#define EDMA_5_TCD_TCD15_ATTR EDMA_5_TCD.TCD[15].ATTR.R    /* TCD Transfer Attributes */
#define EDMA_5_TCD_TCD15_NBYTES_MLOFFNO EDMA_5_TCD.TCD[15].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define EDMA_5_TCD_TCD15_NBYTES_MLOFFYES EDMA_5_TCD.TCD[15].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define EDMA_5_TCD_TCD15_SLAST_SDA EDMA_5_TCD.TCD[15].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define EDMA_5_TCD_TCD15_DADDR EDMA_5_TCD.TCD[15].DADDR.R  /* TCD Destination Address */
#define EDMA_5_TCD_TCD15_DOFF EDMA_5_TCD.TCD[15].DOFF.R    /* TCD Signed Destination Address Offset */
#define EDMA_5_TCD_TCD15_CITER_ELINKNO EDMA_5_TCD.TCD[15].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD15_CITER_ELINKYES EDMA_5_TCD.TCD[15].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define EDMA_5_TCD_TCD15_DLAST_SGA EDMA_5_TCD.TCD[15].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define EDMA_5_TCD_TCD15_CSR EDMA_5_TCD.TCD[15].CSR.R      /* TCD Control and Status */
#define EDMA_5_TCD_TCD15_BITER_ELINKNO EDMA_5_TCD.TCD[15].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define EDMA_5_TCD_TCD15_BITER_ELINKYES EDMA_5_TCD.TCD[15].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */

#define CE_EDMA_TCD_CH0_CSR  CE_EDMA_TCD.TCD[0].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH0_ES   CE_EDMA_TCD.TCD[0].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH0_INT  CE_EDMA_TCD.TCD[0].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH0_SBR  CE_EDMA_TCD.TCD[0].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH0_PRI  CE_EDMA_TCD.TCD[0].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD0_SADDR CE_EDMA_TCD.TCD[0].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD0_SOFF CE_EDMA_TCD.TCD[0].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD0_ATTR CE_EDMA_TCD.TCD[0].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD0_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[0].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD0_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[0].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD0_SLAST_SDA CE_EDMA_TCD.TCD[0].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD0_DADDR CE_EDMA_TCD.TCD[0].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD0_DOFF CE_EDMA_TCD.TCD[0].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD0_CITER_ELINKNO CE_EDMA_TCD.TCD[0].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD0_CITER_ELINKYES CE_EDMA_TCD.TCD[0].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD0_DLAST_SGA CE_EDMA_TCD.TCD[0].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD0_CSR CE_EDMA_TCD.TCD[0].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD0_BITER_ELINKNO CE_EDMA_TCD.TCD[0].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD0_BITER_ELINKYES CE_EDMA_TCD.TCD[0].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH1_CSR  CE_EDMA_TCD.TCD[1].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH1_ES   CE_EDMA_TCD.TCD[1].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH1_INT  CE_EDMA_TCD.TCD[1].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH1_SBR  CE_EDMA_TCD.TCD[1].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH1_PRI  CE_EDMA_TCD.TCD[1].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD1_SADDR CE_EDMA_TCD.TCD[1].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD1_SOFF CE_EDMA_TCD.TCD[1].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD1_ATTR CE_EDMA_TCD.TCD[1].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD1_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[1].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD1_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[1].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD1_SLAST_SDA CE_EDMA_TCD.TCD[1].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD1_DADDR CE_EDMA_TCD.TCD[1].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD1_DOFF CE_EDMA_TCD.TCD[1].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD1_CITER_ELINKNO CE_EDMA_TCD.TCD[1].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD1_CITER_ELINKYES CE_EDMA_TCD.TCD[1].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD1_DLAST_SGA CE_EDMA_TCD.TCD[1].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD1_CSR CE_EDMA_TCD.TCD[1].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD1_BITER_ELINKNO CE_EDMA_TCD.TCD[1].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD1_BITER_ELINKYES CE_EDMA_TCD.TCD[1].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH2_CSR  CE_EDMA_TCD.TCD[2].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH2_ES   CE_EDMA_TCD.TCD[2].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH2_INT  CE_EDMA_TCD.TCD[2].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH2_SBR  CE_EDMA_TCD.TCD[2].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH2_PRI  CE_EDMA_TCD.TCD[2].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD2_SADDR CE_EDMA_TCD.TCD[2].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD2_SOFF CE_EDMA_TCD.TCD[2].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD2_ATTR CE_EDMA_TCD.TCD[2].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD2_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[2].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD2_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[2].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD2_SLAST_SDA CE_EDMA_TCD.TCD[2].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD2_DADDR CE_EDMA_TCD.TCD[2].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD2_DOFF CE_EDMA_TCD.TCD[2].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD2_CITER_ELINKNO CE_EDMA_TCD.TCD[2].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD2_CITER_ELINKYES CE_EDMA_TCD.TCD[2].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD2_DLAST_SGA CE_EDMA_TCD.TCD[2].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD2_CSR CE_EDMA_TCD.TCD[2].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD2_BITER_ELINKNO CE_EDMA_TCD.TCD[2].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD2_BITER_ELINKYES CE_EDMA_TCD.TCD[2].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH3_CSR  CE_EDMA_TCD.TCD[3].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH3_ES   CE_EDMA_TCD.TCD[3].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH3_INT  CE_EDMA_TCD.TCD[3].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH3_SBR  CE_EDMA_TCD.TCD[3].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH3_PRI  CE_EDMA_TCD.TCD[3].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD3_SADDR CE_EDMA_TCD.TCD[3].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD3_SOFF CE_EDMA_TCD.TCD[3].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD3_ATTR CE_EDMA_TCD.TCD[3].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD3_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[3].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD3_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[3].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD3_SLAST_SDA CE_EDMA_TCD.TCD[3].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD3_DADDR CE_EDMA_TCD.TCD[3].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD3_DOFF CE_EDMA_TCD.TCD[3].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD3_CITER_ELINKNO CE_EDMA_TCD.TCD[3].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD3_CITER_ELINKYES CE_EDMA_TCD.TCD[3].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD3_DLAST_SGA CE_EDMA_TCD.TCD[3].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD3_CSR CE_EDMA_TCD.TCD[3].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD3_BITER_ELINKNO CE_EDMA_TCD.TCD[3].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD3_BITER_ELINKYES CE_EDMA_TCD.TCD[3].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH4_CSR  CE_EDMA_TCD.TCD[4].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH4_ES   CE_EDMA_TCD.TCD[4].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH4_INT  CE_EDMA_TCD.TCD[4].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH4_SBR  CE_EDMA_TCD.TCD[4].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH4_PRI  CE_EDMA_TCD.TCD[4].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD4_SADDR CE_EDMA_TCD.TCD[4].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD4_SOFF CE_EDMA_TCD.TCD[4].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD4_ATTR CE_EDMA_TCD.TCD[4].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD4_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[4].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD4_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[4].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD4_SLAST_SDA CE_EDMA_TCD.TCD[4].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD4_DADDR CE_EDMA_TCD.TCD[4].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD4_DOFF CE_EDMA_TCD.TCD[4].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD4_CITER_ELINKNO CE_EDMA_TCD.TCD[4].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD4_CITER_ELINKYES CE_EDMA_TCD.TCD[4].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD4_DLAST_SGA CE_EDMA_TCD.TCD[4].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD4_CSR CE_EDMA_TCD.TCD[4].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD4_BITER_ELINKNO CE_EDMA_TCD.TCD[4].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD4_BITER_ELINKYES CE_EDMA_TCD.TCD[4].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH5_CSR  CE_EDMA_TCD.TCD[5].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH5_ES   CE_EDMA_TCD.TCD[5].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH5_INT  CE_EDMA_TCD.TCD[5].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH5_SBR  CE_EDMA_TCD.TCD[5].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH5_PRI  CE_EDMA_TCD.TCD[5].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD5_SADDR CE_EDMA_TCD.TCD[5].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD5_SOFF CE_EDMA_TCD.TCD[5].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD5_ATTR CE_EDMA_TCD.TCD[5].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD5_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[5].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD5_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[5].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD5_SLAST_SDA CE_EDMA_TCD.TCD[5].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD5_DADDR CE_EDMA_TCD.TCD[5].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD5_DOFF CE_EDMA_TCD.TCD[5].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD5_CITER_ELINKNO CE_EDMA_TCD.TCD[5].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD5_CITER_ELINKYES CE_EDMA_TCD.TCD[5].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD5_DLAST_SGA CE_EDMA_TCD.TCD[5].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD5_CSR CE_EDMA_TCD.TCD[5].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD5_BITER_ELINKNO CE_EDMA_TCD.TCD[5].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD5_BITER_ELINKYES CE_EDMA_TCD.TCD[5].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH6_CSR  CE_EDMA_TCD.TCD[6].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH6_ES   CE_EDMA_TCD.TCD[6].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH6_INT  CE_EDMA_TCD.TCD[6].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH6_SBR  CE_EDMA_TCD.TCD[6].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH6_PRI  CE_EDMA_TCD.TCD[6].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD6_SADDR CE_EDMA_TCD.TCD[6].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD6_SOFF CE_EDMA_TCD.TCD[6].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD6_ATTR CE_EDMA_TCD.TCD[6].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD6_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[6].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD6_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[6].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD6_SLAST_SDA CE_EDMA_TCD.TCD[6].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD6_DADDR CE_EDMA_TCD.TCD[6].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD6_DOFF CE_EDMA_TCD.TCD[6].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD6_CITER_ELINKNO CE_EDMA_TCD.TCD[6].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD6_CITER_ELINKYES CE_EDMA_TCD.TCD[6].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD6_DLAST_SGA CE_EDMA_TCD.TCD[6].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD6_CSR CE_EDMA_TCD.TCD[6].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD6_BITER_ELINKNO CE_EDMA_TCD.TCD[6].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD6_BITER_ELINKYES CE_EDMA_TCD.TCD[6].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH7_CSR  CE_EDMA_TCD.TCD[7].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH7_ES   CE_EDMA_TCD.TCD[7].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH7_INT  CE_EDMA_TCD.TCD[7].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH7_SBR  CE_EDMA_TCD.TCD[7].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH7_PRI  CE_EDMA_TCD.TCD[7].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD7_SADDR CE_EDMA_TCD.TCD[7].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD7_SOFF CE_EDMA_TCD.TCD[7].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD7_ATTR CE_EDMA_TCD.TCD[7].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD7_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[7].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD7_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[7].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD7_SLAST_SDA CE_EDMA_TCD.TCD[7].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD7_DADDR CE_EDMA_TCD.TCD[7].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD7_DOFF CE_EDMA_TCD.TCD[7].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD7_CITER_ELINKNO CE_EDMA_TCD.TCD[7].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD7_CITER_ELINKYES CE_EDMA_TCD.TCD[7].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD7_DLAST_SGA CE_EDMA_TCD.TCD[7].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD7_CSR CE_EDMA_TCD.TCD[7].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD7_BITER_ELINKNO CE_EDMA_TCD.TCD[7].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD7_BITER_ELINKYES CE_EDMA_TCD.TCD[7].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH8_CSR  CE_EDMA_TCD.TCD[8].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH8_ES   CE_EDMA_TCD.TCD[8].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH8_INT  CE_EDMA_TCD.TCD[8].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH8_SBR  CE_EDMA_TCD.TCD[8].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH8_PRI  CE_EDMA_TCD.TCD[8].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD8_SADDR CE_EDMA_TCD.TCD[8].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD8_SOFF CE_EDMA_TCD.TCD[8].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD8_ATTR CE_EDMA_TCD.TCD[8].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD8_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[8].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD8_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[8].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD8_SLAST_SDA CE_EDMA_TCD.TCD[8].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD8_DADDR CE_EDMA_TCD.TCD[8].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD8_DOFF CE_EDMA_TCD.TCD[8].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD8_CITER_ELINKNO CE_EDMA_TCD.TCD[8].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD8_CITER_ELINKYES CE_EDMA_TCD.TCD[8].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD8_DLAST_SGA CE_EDMA_TCD.TCD[8].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD8_CSR CE_EDMA_TCD.TCD[8].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD8_BITER_ELINKNO CE_EDMA_TCD.TCD[8].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD8_BITER_ELINKYES CE_EDMA_TCD.TCD[8].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH9_CSR  CE_EDMA_TCD.TCD[9].CH_CSR.R   /* Channel Control and Status */
#define CE_EDMA_TCD_CH9_ES   CE_EDMA_TCD.TCD[9].CH_ES.R    /* Channel Error Status */
#define CE_EDMA_TCD_CH9_INT  CE_EDMA_TCD.TCD[9].CH_INT.R   /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH9_SBR  CE_EDMA_TCD.TCD[9].CH_SBR.R   /* Channel System Bus */
#define CE_EDMA_TCD_CH9_PRI  CE_EDMA_TCD.TCD[9].CH_PRI.R   /* Channel Priority */
#define CE_EDMA_TCD_TCD9_SADDR CE_EDMA_TCD.TCD[9].SADDR.R  /* TCD Source Address */
#define CE_EDMA_TCD_TCD9_SOFF CE_EDMA_TCD.TCD[9].SOFF.R    /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD9_ATTR CE_EDMA_TCD.TCD[9].ATTR.R    /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD9_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[9].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD9_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[9].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD9_SLAST_SDA CE_EDMA_TCD.TCD[9].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD9_DADDR CE_EDMA_TCD.TCD[9].DADDR.R  /* TCD Destination Address */
#define CE_EDMA_TCD_TCD9_DOFF CE_EDMA_TCD.TCD[9].DOFF.R    /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD9_CITER_ELINKNO CE_EDMA_TCD.TCD[9].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD9_CITER_ELINKYES CE_EDMA_TCD.TCD[9].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD9_DLAST_SGA CE_EDMA_TCD.TCD[9].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD9_CSR CE_EDMA_TCD.TCD[9].CSR.R      /* TCD Control and Status */
#define CE_EDMA_TCD_TCD9_BITER_ELINKNO CE_EDMA_TCD.TCD[9].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD9_BITER_ELINKYES CE_EDMA_TCD.TCD[9].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH10_CSR CE_EDMA_TCD.TCD[10].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH10_ES  CE_EDMA_TCD.TCD[10].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH10_INT CE_EDMA_TCD.TCD[10].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH10_SBR CE_EDMA_TCD.TCD[10].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH10_PRI CE_EDMA_TCD.TCD[10].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD10_SADDR CE_EDMA_TCD.TCD[10].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD10_SOFF CE_EDMA_TCD.TCD[10].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD10_ATTR CE_EDMA_TCD.TCD[10].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD10_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[10].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD10_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[10].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD10_SLAST_SDA CE_EDMA_TCD.TCD[10].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD10_DADDR CE_EDMA_TCD.TCD[10].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD10_DOFF CE_EDMA_TCD.TCD[10].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD10_CITER_ELINKNO CE_EDMA_TCD.TCD[10].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD10_CITER_ELINKYES CE_EDMA_TCD.TCD[10].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD10_DLAST_SGA CE_EDMA_TCD.TCD[10].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD10_CSR CE_EDMA_TCD.TCD[10].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD10_BITER_ELINKNO CE_EDMA_TCD.TCD[10].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD10_BITER_ELINKYES CE_EDMA_TCD.TCD[10].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH11_CSR CE_EDMA_TCD.TCD[11].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH11_ES  CE_EDMA_TCD.TCD[11].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH11_INT CE_EDMA_TCD.TCD[11].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH11_SBR CE_EDMA_TCD.TCD[11].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH11_PRI CE_EDMA_TCD.TCD[11].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD11_SADDR CE_EDMA_TCD.TCD[11].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD11_SOFF CE_EDMA_TCD.TCD[11].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD11_ATTR CE_EDMA_TCD.TCD[11].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD11_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[11].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD11_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[11].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD11_SLAST_SDA CE_EDMA_TCD.TCD[11].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD11_DADDR CE_EDMA_TCD.TCD[11].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD11_DOFF CE_EDMA_TCD.TCD[11].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD11_CITER_ELINKNO CE_EDMA_TCD.TCD[11].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD11_CITER_ELINKYES CE_EDMA_TCD.TCD[11].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD11_DLAST_SGA CE_EDMA_TCD.TCD[11].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD11_CSR CE_EDMA_TCD.TCD[11].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD11_BITER_ELINKNO CE_EDMA_TCD.TCD[11].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD11_BITER_ELINKYES CE_EDMA_TCD.TCD[11].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH12_CSR CE_EDMA_TCD.TCD[12].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH12_ES  CE_EDMA_TCD.TCD[12].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH12_INT CE_EDMA_TCD.TCD[12].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH12_SBR CE_EDMA_TCD.TCD[12].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH12_PRI CE_EDMA_TCD.TCD[12].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD12_SADDR CE_EDMA_TCD.TCD[12].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD12_SOFF CE_EDMA_TCD.TCD[12].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD12_ATTR CE_EDMA_TCD.TCD[12].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD12_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[12].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD12_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[12].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD12_SLAST_SDA CE_EDMA_TCD.TCD[12].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD12_DADDR CE_EDMA_TCD.TCD[12].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD12_DOFF CE_EDMA_TCD.TCD[12].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD12_CITER_ELINKNO CE_EDMA_TCD.TCD[12].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD12_CITER_ELINKYES CE_EDMA_TCD.TCD[12].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD12_DLAST_SGA CE_EDMA_TCD.TCD[12].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD12_CSR CE_EDMA_TCD.TCD[12].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD12_BITER_ELINKNO CE_EDMA_TCD.TCD[12].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD12_BITER_ELINKYES CE_EDMA_TCD.TCD[12].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH13_CSR CE_EDMA_TCD.TCD[13].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH13_ES  CE_EDMA_TCD.TCD[13].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH13_INT CE_EDMA_TCD.TCD[13].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH13_SBR CE_EDMA_TCD.TCD[13].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH13_PRI CE_EDMA_TCD.TCD[13].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD13_SADDR CE_EDMA_TCD.TCD[13].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD13_SOFF CE_EDMA_TCD.TCD[13].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD13_ATTR CE_EDMA_TCD.TCD[13].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD13_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[13].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD13_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[13].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD13_SLAST_SDA CE_EDMA_TCD.TCD[13].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD13_DADDR CE_EDMA_TCD.TCD[13].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD13_DOFF CE_EDMA_TCD.TCD[13].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD13_CITER_ELINKNO CE_EDMA_TCD.TCD[13].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD13_CITER_ELINKYES CE_EDMA_TCD.TCD[13].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD13_DLAST_SGA CE_EDMA_TCD.TCD[13].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD13_CSR CE_EDMA_TCD.TCD[13].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD13_BITER_ELINKNO CE_EDMA_TCD.TCD[13].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD13_BITER_ELINKYES CE_EDMA_TCD.TCD[13].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH14_CSR CE_EDMA_TCD.TCD[14].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH14_ES  CE_EDMA_TCD.TCD[14].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH14_INT CE_EDMA_TCD.TCD[14].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH14_SBR CE_EDMA_TCD.TCD[14].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH14_PRI CE_EDMA_TCD.TCD[14].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD14_SADDR CE_EDMA_TCD.TCD[14].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD14_SOFF CE_EDMA_TCD.TCD[14].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD14_ATTR CE_EDMA_TCD.TCD[14].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD14_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[14].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD14_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[14].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD14_SLAST_SDA CE_EDMA_TCD.TCD[14].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD14_DADDR CE_EDMA_TCD.TCD[14].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD14_DOFF CE_EDMA_TCD.TCD[14].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD14_CITER_ELINKNO CE_EDMA_TCD.TCD[14].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD14_CITER_ELINKYES CE_EDMA_TCD.TCD[14].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD14_DLAST_SGA CE_EDMA_TCD.TCD[14].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD14_CSR CE_EDMA_TCD.TCD[14].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD14_BITER_ELINKNO CE_EDMA_TCD.TCD[14].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD14_BITER_ELINKYES CE_EDMA_TCD.TCD[14].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH15_CSR CE_EDMA_TCD.TCD[15].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH15_ES  CE_EDMA_TCD.TCD[15].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH15_INT CE_EDMA_TCD.TCD[15].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH15_SBR CE_EDMA_TCD.TCD[15].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH15_PRI CE_EDMA_TCD.TCD[15].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD15_SADDR CE_EDMA_TCD.TCD[15].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD15_SOFF CE_EDMA_TCD.TCD[15].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD15_ATTR CE_EDMA_TCD.TCD[15].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD15_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[15].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD15_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[15].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD15_SLAST_SDA CE_EDMA_TCD.TCD[15].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD15_DADDR CE_EDMA_TCD.TCD[15].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD15_DOFF CE_EDMA_TCD.TCD[15].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD15_CITER_ELINKNO CE_EDMA_TCD.TCD[15].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD15_CITER_ELINKYES CE_EDMA_TCD.TCD[15].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD15_DLAST_SGA CE_EDMA_TCD.TCD[15].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD15_CSR CE_EDMA_TCD.TCD[15].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD15_BITER_ELINKNO CE_EDMA_TCD.TCD[15].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD15_BITER_ELINKYES CE_EDMA_TCD.TCD[15].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH16_CSR CE_EDMA_TCD.TCD[16].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH16_ES  CE_EDMA_TCD.TCD[16].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH16_INT CE_EDMA_TCD.TCD[16].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH16_SBR CE_EDMA_TCD.TCD[16].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH16_PRI CE_EDMA_TCD.TCD[16].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD16_SADDR CE_EDMA_TCD.TCD[16].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD16_SOFF CE_EDMA_TCD.TCD[16].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD16_ATTR CE_EDMA_TCD.TCD[16].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD16_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[16].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD16_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[16].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD16_SLAST_SDA CE_EDMA_TCD.TCD[16].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD16_DADDR CE_EDMA_TCD.TCD[16].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD16_DOFF CE_EDMA_TCD.TCD[16].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD16_CITER_ELINKNO CE_EDMA_TCD.TCD[16].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD16_CITER_ELINKYES CE_EDMA_TCD.TCD[16].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD16_DLAST_SGA CE_EDMA_TCD.TCD[16].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD16_CSR CE_EDMA_TCD.TCD[16].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD16_BITER_ELINKNO CE_EDMA_TCD.TCD[16].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD16_BITER_ELINKYES CE_EDMA_TCD.TCD[16].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH17_CSR CE_EDMA_TCD.TCD[17].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH17_ES  CE_EDMA_TCD.TCD[17].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH17_INT CE_EDMA_TCD.TCD[17].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH17_SBR CE_EDMA_TCD.TCD[17].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH17_PRI CE_EDMA_TCD.TCD[17].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD17_SADDR CE_EDMA_TCD.TCD[17].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD17_SOFF CE_EDMA_TCD.TCD[17].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD17_ATTR CE_EDMA_TCD.TCD[17].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD17_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[17].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD17_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[17].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD17_SLAST_SDA CE_EDMA_TCD.TCD[17].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD17_DADDR CE_EDMA_TCD.TCD[17].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD17_DOFF CE_EDMA_TCD.TCD[17].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD17_CITER_ELINKNO CE_EDMA_TCD.TCD[17].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD17_CITER_ELINKYES CE_EDMA_TCD.TCD[17].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD17_DLAST_SGA CE_EDMA_TCD.TCD[17].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD17_CSR CE_EDMA_TCD.TCD[17].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD17_BITER_ELINKNO CE_EDMA_TCD.TCD[17].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD17_BITER_ELINKYES CE_EDMA_TCD.TCD[17].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH18_CSR CE_EDMA_TCD.TCD[18].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH18_ES  CE_EDMA_TCD.TCD[18].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH18_INT CE_EDMA_TCD.TCD[18].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH18_SBR CE_EDMA_TCD.TCD[18].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH18_PRI CE_EDMA_TCD.TCD[18].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD18_SADDR CE_EDMA_TCD.TCD[18].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD18_SOFF CE_EDMA_TCD.TCD[18].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD18_ATTR CE_EDMA_TCD.TCD[18].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD18_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[18].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD18_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[18].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD18_SLAST_SDA CE_EDMA_TCD.TCD[18].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD18_DADDR CE_EDMA_TCD.TCD[18].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD18_DOFF CE_EDMA_TCD.TCD[18].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD18_CITER_ELINKNO CE_EDMA_TCD.TCD[18].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD18_CITER_ELINKYES CE_EDMA_TCD.TCD[18].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD18_DLAST_SGA CE_EDMA_TCD.TCD[18].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD18_CSR CE_EDMA_TCD.TCD[18].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD18_BITER_ELINKNO CE_EDMA_TCD.TCD[18].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD18_BITER_ELINKYES CE_EDMA_TCD.TCD[18].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH19_CSR CE_EDMA_TCD.TCD[19].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH19_ES  CE_EDMA_TCD.TCD[19].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH19_INT CE_EDMA_TCD.TCD[19].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH19_SBR CE_EDMA_TCD.TCD[19].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH19_PRI CE_EDMA_TCD.TCD[19].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD19_SADDR CE_EDMA_TCD.TCD[19].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD19_SOFF CE_EDMA_TCD.TCD[19].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD19_ATTR CE_EDMA_TCD.TCD[19].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD19_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[19].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD19_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[19].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD19_SLAST_SDA CE_EDMA_TCD.TCD[19].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD19_DADDR CE_EDMA_TCD.TCD[19].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD19_DOFF CE_EDMA_TCD.TCD[19].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD19_CITER_ELINKNO CE_EDMA_TCD.TCD[19].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD19_CITER_ELINKYES CE_EDMA_TCD.TCD[19].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD19_DLAST_SGA CE_EDMA_TCD.TCD[19].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD19_CSR CE_EDMA_TCD.TCD[19].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD19_BITER_ELINKNO CE_EDMA_TCD.TCD[19].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD19_BITER_ELINKYES CE_EDMA_TCD.TCD[19].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH20_CSR CE_EDMA_TCD.TCD[20].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH20_ES  CE_EDMA_TCD.TCD[20].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH20_INT CE_EDMA_TCD.TCD[20].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH20_SBR CE_EDMA_TCD.TCD[20].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH20_PRI CE_EDMA_TCD.TCD[20].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD20_SADDR CE_EDMA_TCD.TCD[20].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD20_SOFF CE_EDMA_TCD.TCD[20].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD20_ATTR CE_EDMA_TCD.TCD[20].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD20_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[20].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD20_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[20].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD20_SLAST_SDA CE_EDMA_TCD.TCD[20].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD20_DADDR CE_EDMA_TCD.TCD[20].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD20_DOFF CE_EDMA_TCD.TCD[20].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD20_CITER_ELINKNO CE_EDMA_TCD.TCD[20].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD20_CITER_ELINKYES CE_EDMA_TCD.TCD[20].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD20_DLAST_SGA CE_EDMA_TCD.TCD[20].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD20_CSR CE_EDMA_TCD.TCD[20].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD20_BITER_ELINKNO CE_EDMA_TCD.TCD[20].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD20_BITER_ELINKYES CE_EDMA_TCD.TCD[20].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH21_CSR CE_EDMA_TCD.TCD[21].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH21_ES  CE_EDMA_TCD.TCD[21].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH21_INT CE_EDMA_TCD.TCD[21].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH21_SBR CE_EDMA_TCD.TCD[21].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH21_PRI CE_EDMA_TCD.TCD[21].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD21_SADDR CE_EDMA_TCD.TCD[21].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD21_SOFF CE_EDMA_TCD.TCD[21].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD21_ATTR CE_EDMA_TCD.TCD[21].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD21_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[21].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD21_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[21].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD21_SLAST_SDA CE_EDMA_TCD.TCD[21].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD21_DADDR CE_EDMA_TCD.TCD[21].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD21_DOFF CE_EDMA_TCD.TCD[21].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD21_CITER_ELINKNO CE_EDMA_TCD.TCD[21].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD21_CITER_ELINKYES CE_EDMA_TCD.TCD[21].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD21_DLAST_SGA CE_EDMA_TCD.TCD[21].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD21_CSR CE_EDMA_TCD.TCD[21].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD21_BITER_ELINKNO CE_EDMA_TCD.TCD[21].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD21_BITER_ELINKYES CE_EDMA_TCD.TCD[21].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH22_CSR CE_EDMA_TCD.TCD[22].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH22_ES  CE_EDMA_TCD.TCD[22].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH22_INT CE_EDMA_TCD.TCD[22].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH22_SBR CE_EDMA_TCD.TCD[22].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH22_PRI CE_EDMA_TCD.TCD[22].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD22_SADDR CE_EDMA_TCD.TCD[22].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD22_SOFF CE_EDMA_TCD.TCD[22].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD22_ATTR CE_EDMA_TCD.TCD[22].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD22_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[22].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD22_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[22].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD22_SLAST_SDA CE_EDMA_TCD.TCD[22].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD22_DADDR CE_EDMA_TCD.TCD[22].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD22_DOFF CE_EDMA_TCD.TCD[22].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD22_CITER_ELINKNO CE_EDMA_TCD.TCD[22].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD22_CITER_ELINKYES CE_EDMA_TCD.TCD[22].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD22_DLAST_SGA CE_EDMA_TCD.TCD[22].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD22_CSR CE_EDMA_TCD.TCD[22].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD22_BITER_ELINKNO CE_EDMA_TCD.TCD[22].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD22_BITER_ELINKYES CE_EDMA_TCD.TCD[22].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH23_CSR CE_EDMA_TCD.TCD[23].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH23_ES  CE_EDMA_TCD.TCD[23].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH23_INT CE_EDMA_TCD.TCD[23].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH23_SBR CE_EDMA_TCD.TCD[23].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH23_PRI CE_EDMA_TCD.TCD[23].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD23_SADDR CE_EDMA_TCD.TCD[23].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD23_SOFF CE_EDMA_TCD.TCD[23].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD23_ATTR CE_EDMA_TCD.TCD[23].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD23_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[23].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD23_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[23].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD23_SLAST_SDA CE_EDMA_TCD.TCD[23].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD23_DADDR CE_EDMA_TCD.TCD[23].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD23_DOFF CE_EDMA_TCD.TCD[23].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD23_CITER_ELINKNO CE_EDMA_TCD.TCD[23].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD23_CITER_ELINKYES CE_EDMA_TCD.TCD[23].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD23_DLAST_SGA CE_EDMA_TCD.TCD[23].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD23_CSR CE_EDMA_TCD.TCD[23].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD23_BITER_ELINKNO CE_EDMA_TCD.TCD[23].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD23_BITER_ELINKYES CE_EDMA_TCD.TCD[23].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH24_CSR CE_EDMA_TCD.TCD[24].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH24_ES  CE_EDMA_TCD.TCD[24].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH24_INT CE_EDMA_TCD.TCD[24].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH24_SBR CE_EDMA_TCD.TCD[24].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH24_PRI CE_EDMA_TCD.TCD[24].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD24_SADDR CE_EDMA_TCD.TCD[24].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD24_SOFF CE_EDMA_TCD.TCD[24].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD24_ATTR CE_EDMA_TCD.TCD[24].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD24_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[24].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD24_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[24].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD24_SLAST_SDA CE_EDMA_TCD.TCD[24].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD24_DADDR CE_EDMA_TCD.TCD[24].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD24_DOFF CE_EDMA_TCD.TCD[24].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD24_CITER_ELINKNO CE_EDMA_TCD.TCD[24].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD24_CITER_ELINKYES CE_EDMA_TCD.TCD[24].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD24_DLAST_SGA CE_EDMA_TCD.TCD[24].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD24_CSR CE_EDMA_TCD.TCD[24].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD24_BITER_ELINKNO CE_EDMA_TCD.TCD[24].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD24_BITER_ELINKYES CE_EDMA_TCD.TCD[24].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH25_CSR CE_EDMA_TCD.TCD[25].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH25_ES  CE_EDMA_TCD.TCD[25].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH25_INT CE_EDMA_TCD.TCD[25].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH25_SBR CE_EDMA_TCD.TCD[25].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH25_PRI CE_EDMA_TCD.TCD[25].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD25_SADDR CE_EDMA_TCD.TCD[25].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD25_SOFF CE_EDMA_TCD.TCD[25].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD25_ATTR CE_EDMA_TCD.TCD[25].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD25_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[25].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD25_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[25].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD25_SLAST_SDA CE_EDMA_TCD.TCD[25].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD25_DADDR CE_EDMA_TCD.TCD[25].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD25_DOFF CE_EDMA_TCD.TCD[25].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD25_CITER_ELINKNO CE_EDMA_TCD.TCD[25].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD25_CITER_ELINKYES CE_EDMA_TCD.TCD[25].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD25_DLAST_SGA CE_EDMA_TCD.TCD[25].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD25_CSR CE_EDMA_TCD.TCD[25].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD25_BITER_ELINKNO CE_EDMA_TCD.TCD[25].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD25_BITER_ELINKYES CE_EDMA_TCD.TCD[25].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH26_CSR CE_EDMA_TCD.TCD[26].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH26_ES  CE_EDMA_TCD.TCD[26].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH26_INT CE_EDMA_TCD.TCD[26].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH26_SBR CE_EDMA_TCD.TCD[26].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH26_PRI CE_EDMA_TCD.TCD[26].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD26_SADDR CE_EDMA_TCD.TCD[26].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD26_SOFF CE_EDMA_TCD.TCD[26].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD26_ATTR CE_EDMA_TCD.TCD[26].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD26_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[26].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD26_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[26].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD26_SLAST_SDA CE_EDMA_TCD.TCD[26].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD26_DADDR CE_EDMA_TCD.TCD[26].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD26_DOFF CE_EDMA_TCD.TCD[26].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD26_CITER_ELINKNO CE_EDMA_TCD.TCD[26].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD26_CITER_ELINKYES CE_EDMA_TCD.TCD[26].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD26_DLAST_SGA CE_EDMA_TCD.TCD[26].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD26_CSR CE_EDMA_TCD.TCD[26].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD26_BITER_ELINKNO CE_EDMA_TCD.TCD[26].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD26_BITER_ELINKYES CE_EDMA_TCD.TCD[26].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH27_CSR CE_EDMA_TCD.TCD[27].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH27_ES  CE_EDMA_TCD.TCD[27].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH27_INT CE_EDMA_TCD.TCD[27].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH27_SBR CE_EDMA_TCD.TCD[27].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH27_PRI CE_EDMA_TCD.TCD[27].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD27_SADDR CE_EDMA_TCD.TCD[27].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD27_SOFF CE_EDMA_TCD.TCD[27].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD27_ATTR CE_EDMA_TCD.TCD[27].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD27_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[27].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD27_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[27].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD27_SLAST_SDA CE_EDMA_TCD.TCD[27].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD27_DADDR CE_EDMA_TCD.TCD[27].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD27_DOFF CE_EDMA_TCD.TCD[27].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD27_CITER_ELINKNO CE_EDMA_TCD.TCD[27].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD27_CITER_ELINKYES CE_EDMA_TCD.TCD[27].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD27_DLAST_SGA CE_EDMA_TCD.TCD[27].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD27_CSR CE_EDMA_TCD.TCD[27].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD27_BITER_ELINKNO CE_EDMA_TCD.TCD[27].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD27_BITER_ELINKYES CE_EDMA_TCD.TCD[27].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH28_CSR CE_EDMA_TCD.TCD[28].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH28_ES  CE_EDMA_TCD.TCD[28].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH28_INT CE_EDMA_TCD.TCD[28].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH28_SBR CE_EDMA_TCD.TCD[28].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH28_PRI CE_EDMA_TCD.TCD[28].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD28_SADDR CE_EDMA_TCD.TCD[28].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD28_SOFF CE_EDMA_TCD.TCD[28].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD28_ATTR CE_EDMA_TCD.TCD[28].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD28_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[28].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD28_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[28].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD28_SLAST_SDA CE_EDMA_TCD.TCD[28].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD28_DADDR CE_EDMA_TCD.TCD[28].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD28_DOFF CE_EDMA_TCD.TCD[28].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD28_CITER_ELINKNO CE_EDMA_TCD.TCD[28].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD28_CITER_ELINKYES CE_EDMA_TCD.TCD[28].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD28_DLAST_SGA CE_EDMA_TCD.TCD[28].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD28_CSR CE_EDMA_TCD.TCD[28].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD28_BITER_ELINKNO CE_EDMA_TCD.TCD[28].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD28_BITER_ELINKYES CE_EDMA_TCD.TCD[28].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH29_CSR CE_EDMA_TCD.TCD[29].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH29_ES  CE_EDMA_TCD.TCD[29].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH29_INT CE_EDMA_TCD.TCD[29].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH29_SBR CE_EDMA_TCD.TCD[29].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH29_PRI CE_EDMA_TCD.TCD[29].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD29_SADDR CE_EDMA_TCD.TCD[29].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD29_SOFF CE_EDMA_TCD.TCD[29].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD29_ATTR CE_EDMA_TCD.TCD[29].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD29_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[29].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD29_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[29].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD29_SLAST_SDA CE_EDMA_TCD.TCD[29].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD29_DADDR CE_EDMA_TCD.TCD[29].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD29_DOFF CE_EDMA_TCD.TCD[29].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD29_CITER_ELINKNO CE_EDMA_TCD.TCD[29].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD29_CITER_ELINKYES CE_EDMA_TCD.TCD[29].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD29_DLAST_SGA CE_EDMA_TCD.TCD[29].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD29_CSR CE_EDMA_TCD.TCD[29].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD29_BITER_ELINKNO CE_EDMA_TCD.TCD[29].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD29_BITER_ELINKYES CE_EDMA_TCD.TCD[29].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH30_CSR CE_EDMA_TCD.TCD[30].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH30_ES  CE_EDMA_TCD.TCD[30].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH30_INT CE_EDMA_TCD.TCD[30].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH30_SBR CE_EDMA_TCD.TCD[30].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH30_PRI CE_EDMA_TCD.TCD[30].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD30_SADDR CE_EDMA_TCD.TCD[30].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD30_SOFF CE_EDMA_TCD.TCD[30].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD30_ATTR CE_EDMA_TCD.TCD[30].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD30_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[30].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD30_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[30].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD30_SLAST_SDA CE_EDMA_TCD.TCD[30].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD30_DADDR CE_EDMA_TCD.TCD[30].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD30_DOFF CE_EDMA_TCD.TCD[30].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD30_CITER_ELINKNO CE_EDMA_TCD.TCD[30].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD30_CITER_ELINKYES CE_EDMA_TCD.TCD[30].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD30_DLAST_SGA CE_EDMA_TCD.TCD[30].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD30_CSR CE_EDMA_TCD.TCD[30].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD30_BITER_ELINKNO CE_EDMA_TCD.TCD[30].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD30_BITER_ELINKYES CE_EDMA_TCD.TCD[30].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_CH31_CSR CE_EDMA_TCD.TCD[31].CH_CSR.R  /* Channel Control and Status */
#define CE_EDMA_TCD_CH31_ES  CE_EDMA_TCD.TCD[31].CH_ES.R   /* Channel Error Status */
#define CE_EDMA_TCD_CH31_INT CE_EDMA_TCD.TCD[31].CH_INT.R  /* Channel Interrupt Status */
#define CE_EDMA_TCD_CH31_SBR CE_EDMA_TCD.TCD[31].CH_SBR.R  /* Channel System Bus */
#define CE_EDMA_TCD_CH31_PRI CE_EDMA_TCD.TCD[31].CH_PRI.R  /* Channel Priority */
#define CE_EDMA_TCD_TCD31_SADDR CE_EDMA_TCD.TCD[31].SADDR.R /* TCD Source Address */
#define CE_EDMA_TCD_TCD31_SOFF CE_EDMA_TCD.TCD[31].SOFF.R  /* TCD Signed Source Address Offset */
#define CE_EDMA_TCD_TCD31_ATTR CE_EDMA_TCD.TCD[31].ATTR.R  /* TCD Transfer Attributes */
#define CE_EDMA_TCD_TCD31_NBYTES_MLOFFNO CE_EDMA_TCD.TCD[31].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define CE_EDMA_TCD_TCD31_NBYTES_MLOFFYES CE_EDMA_TCD.TCD[31].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define CE_EDMA_TCD_TCD31_SLAST_SDA CE_EDMA_TCD.TCD[31].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define CE_EDMA_TCD_TCD31_DADDR CE_EDMA_TCD.TCD[31].DADDR.R /* TCD Destination Address */
#define CE_EDMA_TCD_TCD31_DOFF CE_EDMA_TCD.TCD[31].DOFF.R  /* TCD Signed Destination Address Offset */
#define CE_EDMA_TCD_TCD31_CITER_ELINKNO CE_EDMA_TCD.TCD[31].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD31_CITER_ELINKYES CE_EDMA_TCD.TCD[31].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define CE_EDMA_TCD_TCD31_DLAST_SGA CE_EDMA_TCD.TCD[31].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define CE_EDMA_TCD_TCD31_CSR CE_EDMA_TCD.TCD[31].CSR.R    /* TCD Control and Status */
#define CE_EDMA_TCD_TCD31_BITER_ELINKNO CE_EDMA_TCD.TCD[31].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define CE_EDMA_TCD_TCD31_BITER_ELINKYES CE_EDMA_TCD.TCD[31].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */

/* EDMA4_MP */
#define EDMA_3_MP_CSR        EDMA_3_MP.CSR.R               /* Management Page Control Register */
#define EDMA_3_MP_ES         EDMA_3_MP.ES.R                /* Management Page Error Status Register */
#define EDMA_3_MP_INT        EDMA_3_MP.INT.R               /* Management Page Interrupt Request Status Register */
#define EDMA_3_MP_HRS        EDMA_3_MP.HRS.R               /* Management Page Hardware Request Status Register */
#define EDMA_3_MP_CH0_GRPRI  EDMA_3_MP.CH_GRPRI[0].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH1_GRPRI  EDMA_3_MP.CH_GRPRI[1].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH2_GRPRI  EDMA_3_MP.CH_GRPRI[2].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH3_GRPRI  EDMA_3_MP.CH_GRPRI[3].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH4_GRPRI  EDMA_3_MP.CH_GRPRI[4].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH5_GRPRI  EDMA_3_MP.CH_GRPRI[5].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH6_GRPRI  EDMA_3_MP.CH_GRPRI[6].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH7_GRPRI  EDMA_3_MP.CH_GRPRI[7].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH8_GRPRI  EDMA_3_MP.CH_GRPRI[8].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH9_GRPRI  EDMA_3_MP.CH_GRPRI[9].R       /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH10_GRPRI EDMA_3_MP.CH_GRPRI[10].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH11_GRPRI EDMA_3_MP.CH_GRPRI[11].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH12_GRPRI EDMA_3_MP.CH_GRPRI[12].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH13_GRPRI EDMA_3_MP.CH_GRPRI[13].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH14_GRPRI EDMA_3_MP.CH_GRPRI[14].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH15_GRPRI EDMA_3_MP.CH_GRPRI[15].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH16_GRPRI EDMA_3_MP.CH_GRPRI[16].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH17_GRPRI EDMA_3_MP.CH_GRPRI[17].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH18_GRPRI EDMA_3_MP.CH_GRPRI[18].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH19_GRPRI EDMA_3_MP.CH_GRPRI[19].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH20_GRPRI EDMA_3_MP.CH_GRPRI[20].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH21_GRPRI EDMA_3_MP.CH_GRPRI[21].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH22_GRPRI EDMA_3_MP.CH_GRPRI[22].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH23_GRPRI EDMA_3_MP.CH_GRPRI[23].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH24_GRPRI EDMA_3_MP.CH_GRPRI[24].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH25_GRPRI EDMA_3_MP.CH_GRPRI[25].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH26_GRPRI EDMA_3_MP.CH_GRPRI[26].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH27_GRPRI EDMA_3_MP.CH_GRPRI[27].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH28_GRPRI EDMA_3_MP.CH_GRPRI[28].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH29_GRPRI EDMA_3_MP.CH_GRPRI[29].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH30_GRPRI EDMA_3_MP.CH_GRPRI[30].R      /* Channel Arbitration Group Register */
#define EDMA_3_MP_CH31_GRPRI EDMA_3_MP.CH_GRPRI[31].R      /* Channel Arbitration Group Register */

/* EDMA4_TCD */
#define EDMA_3_TCD_CH0_CSR   EDMA_3_TCD.TCD[0].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH0_ES    EDMA_3_TCD.TCD[0].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH0_INT   EDMA_3_TCD.TCD[0].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH0_SBR   EDMA_3_TCD.TCD[0].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH0_PRI   EDMA_3_TCD.TCD[0].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH0_MATTR EDMA_3_TCD.TCD[0].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD0_SADDR EDMA_3_TCD.TCD[0].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD0_SOFF EDMA_3_TCD.TCD[0].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD0_ATTR EDMA_3_TCD.TCD[0].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD0_NBYTES_MLOFFNO EDMA_3_TCD.TCD[0].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD0_NBYTES_MLOFFYES EDMA_3_TCD.TCD[0].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD0_SLAST_SDA EDMA_3_TCD.TCD[0].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD0_DADDR EDMA_3_TCD.TCD[0].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD0_DOFF EDMA_3_TCD.TCD[0].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD0_CITER_ELINKNO EDMA_3_TCD.TCD[0].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD0_CITER_ELINKYES EDMA_3_TCD.TCD[0].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD0_DLAST_SGA EDMA_3_TCD.TCD[0].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD0_CSR  EDMA_3_TCD.TCD[0].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD0_BITER_ELINKNO EDMA_3_TCD.TCD[0].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD0_BITER_ELINKYES EDMA_3_TCD.TCD[0].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH1_CSR   EDMA_3_TCD.TCD[1].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH1_ES    EDMA_3_TCD.TCD[1].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH1_INT   EDMA_3_TCD.TCD[1].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH1_SBR   EDMA_3_TCD.TCD[1].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH1_PRI   EDMA_3_TCD.TCD[1].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH1_MATTR EDMA_3_TCD.TCD[1].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD1_SADDR EDMA_3_TCD.TCD[1].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD1_SOFF EDMA_3_TCD.TCD[1].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD1_ATTR EDMA_3_TCD.TCD[1].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD1_NBYTES_MLOFFNO EDMA_3_TCD.TCD[1].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD1_NBYTES_MLOFFYES EDMA_3_TCD.TCD[1].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD1_SLAST_SDA EDMA_3_TCD.TCD[1].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD1_DADDR EDMA_3_TCD.TCD[1].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD1_DOFF EDMA_3_TCD.TCD[1].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD1_CITER_ELINKNO EDMA_3_TCD.TCD[1].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD1_CITER_ELINKYES EDMA_3_TCD.TCD[1].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD1_DLAST_SGA EDMA_3_TCD.TCD[1].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD1_CSR  EDMA_3_TCD.TCD[1].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD1_BITER_ELINKNO EDMA_3_TCD.TCD[1].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD1_BITER_ELINKYES EDMA_3_TCD.TCD[1].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH2_CSR   EDMA_3_TCD.TCD[2].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH2_ES    EDMA_3_TCD.TCD[2].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH2_INT   EDMA_3_TCD.TCD[2].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH2_SBR   EDMA_3_TCD.TCD[2].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH2_PRI   EDMA_3_TCD.TCD[2].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH2_MATTR EDMA_3_TCD.TCD[2].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD2_SADDR EDMA_3_TCD.TCD[2].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD2_SOFF EDMA_3_TCD.TCD[2].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD2_ATTR EDMA_3_TCD.TCD[2].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD2_NBYTES_MLOFFNO EDMA_3_TCD.TCD[2].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD2_NBYTES_MLOFFYES EDMA_3_TCD.TCD[2].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD2_SLAST_SDA EDMA_3_TCD.TCD[2].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD2_DADDR EDMA_3_TCD.TCD[2].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD2_DOFF EDMA_3_TCD.TCD[2].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD2_CITER_ELINKNO EDMA_3_TCD.TCD[2].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD2_CITER_ELINKYES EDMA_3_TCD.TCD[2].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD2_DLAST_SGA EDMA_3_TCD.TCD[2].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD2_CSR  EDMA_3_TCD.TCD[2].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD2_BITER_ELINKNO EDMA_3_TCD.TCD[2].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD2_BITER_ELINKYES EDMA_3_TCD.TCD[2].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH3_CSR   EDMA_3_TCD.TCD[3].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH3_ES    EDMA_3_TCD.TCD[3].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH3_INT   EDMA_3_TCD.TCD[3].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH3_SBR   EDMA_3_TCD.TCD[3].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH3_PRI   EDMA_3_TCD.TCD[3].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH3_MATTR EDMA_3_TCD.TCD[3].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD3_SADDR EDMA_3_TCD.TCD[3].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD3_SOFF EDMA_3_TCD.TCD[3].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD3_ATTR EDMA_3_TCD.TCD[3].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD3_NBYTES_MLOFFNO EDMA_3_TCD.TCD[3].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD3_NBYTES_MLOFFYES EDMA_3_TCD.TCD[3].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD3_SLAST_SDA EDMA_3_TCD.TCD[3].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD3_DADDR EDMA_3_TCD.TCD[3].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD3_DOFF EDMA_3_TCD.TCD[3].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD3_CITER_ELINKNO EDMA_3_TCD.TCD[3].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD3_CITER_ELINKYES EDMA_3_TCD.TCD[3].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD3_DLAST_SGA EDMA_3_TCD.TCD[3].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD3_CSR  EDMA_3_TCD.TCD[3].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD3_BITER_ELINKNO EDMA_3_TCD.TCD[3].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD3_BITER_ELINKYES EDMA_3_TCD.TCD[3].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH4_CSR   EDMA_3_TCD.TCD[4].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH4_ES    EDMA_3_TCD.TCD[4].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH4_INT   EDMA_3_TCD.TCD[4].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH4_SBR   EDMA_3_TCD.TCD[4].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH4_PRI   EDMA_3_TCD.TCD[4].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH4_MATTR EDMA_3_TCD.TCD[4].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD4_SADDR EDMA_3_TCD.TCD[4].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD4_SOFF EDMA_3_TCD.TCD[4].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD4_ATTR EDMA_3_TCD.TCD[4].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD4_NBYTES_MLOFFNO EDMA_3_TCD.TCD[4].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD4_NBYTES_MLOFFYES EDMA_3_TCD.TCD[4].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD4_SLAST_SDA EDMA_3_TCD.TCD[4].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD4_DADDR EDMA_3_TCD.TCD[4].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD4_DOFF EDMA_3_TCD.TCD[4].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD4_CITER_ELINKNO EDMA_3_TCD.TCD[4].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD4_CITER_ELINKYES EDMA_3_TCD.TCD[4].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD4_DLAST_SGA EDMA_3_TCD.TCD[4].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD4_CSR  EDMA_3_TCD.TCD[4].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD4_BITER_ELINKNO EDMA_3_TCD.TCD[4].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD4_BITER_ELINKYES EDMA_3_TCD.TCD[4].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH5_CSR   EDMA_3_TCD.TCD[5].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH5_ES    EDMA_3_TCD.TCD[5].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH5_INT   EDMA_3_TCD.TCD[5].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH5_SBR   EDMA_3_TCD.TCD[5].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH5_PRI   EDMA_3_TCD.TCD[5].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH5_MATTR EDMA_3_TCD.TCD[5].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD5_SADDR EDMA_3_TCD.TCD[5].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD5_SOFF EDMA_3_TCD.TCD[5].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD5_ATTR EDMA_3_TCD.TCD[5].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD5_NBYTES_MLOFFNO EDMA_3_TCD.TCD[5].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD5_NBYTES_MLOFFYES EDMA_3_TCD.TCD[5].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD5_SLAST_SDA EDMA_3_TCD.TCD[5].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD5_DADDR EDMA_3_TCD.TCD[5].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD5_DOFF EDMA_3_TCD.TCD[5].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD5_CITER_ELINKNO EDMA_3_TCD.TCD[5].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD5_CITER_ELINKYES EDMA_3_TCD.TCD[5].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD5_DLAST_SGA EDMA_3_TCD.TCD[5].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD5_CSR  EDMA_3_TCD.TCD[5].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD5_BITER_ELINKNO EDMA_3_TCD.TCD[5].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD5_BITER_ELINKYES EDMA_3_TCD.TCD[5].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH6_CSR   EDMA_3_TCD.TCD[6].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH6_ES    EDMA_3_TCD.TCD[6].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH6_INT   EDMA_3_TCD.TCD[6].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH6_SBR   EDMA_3_TCD.TCD[6].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH6_PRI   EDMA_3_TCD.TCD[6].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH6_MATTR EDMA_3_TCD.TCD[6].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD6_SADDR EDMA_3_TCD.TCD[6].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD6_SOFF EDMA_3_TCD.TCD[6].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD6_ATTR EDMA_3_TCD.TCD[6].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD6_NBYTES_MLOFFNO EDMA_3_TCD.TCD[6].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD6_NBYTES_MLOFFYES EDMA_3_TCD.TCD[6].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD6_SLAST_SDA EDMA_3_TCD.TCD[6].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD6_DADDR EDMA_3_TCD.TCD[6].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD6_DOFF EDMA_3_TCD.TCD[6].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD6_CITER_ELINKNO EDMA_3_TCD.TCD[6].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD6_CITER_ELINKYES EDMA_3_TCD.TCD[6].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD6_DLAST_SGA EDMA_3_TCD.TCD[6].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD6_CSR  EDMA_3_TCD.TCD[6].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD6_BITER_ELINKNO EDMA_3_TCD.TCD[6].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD6_BITER_ELINKYES EDMA_3_TCD.TCD[6].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH7_CSR   EDMA_3_TCD.TCD[7].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH7_ES    EDMA_3_TCD.TCD[7].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH7_INT   EDMA_3_TCD.TCD[7].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH7_SBR   EDMA_3_TCD.TCD[7].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH7_PRI   EDMA_3_TCD.TCD[7].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH7_MATTR EDMA_3_TCD.TCD[7].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD7_SADDR EDMA_3_TCD.TCD[7].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD7_SOFF EDMA_3_TCD.TCD[7].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD7_ATTR EDMA_3_TCD.TCD[7].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD7_NBYTES_MLOFFNO EDMA_3_TCD.TCD[7].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD7_NBYTES_MLOFFYES EDMA_3_TCD.TCD[7].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD7_SLAST_SDA EDMA_3_TCD.TCD[7].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD7_DADDR EDMA_3_TCD.TCD[7].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD7_DOFF EDMA_3_TCD.TCD[7].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD7_CITER_ELINKNO EDMA_3_TCD.TCD[7].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD7_CITER_ELINKYES EDMA_3_TCD.TCD[7].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD7_DLAST_SGA EDMA_3_TCD.TCD[7].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD7_CSR  EDMA_3_TCD.TCD[7].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD7_BITER_ELINKNO EDMA_3_TCD.TCD[7].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD7_BITER_ELINKYES EDMA_3_TCD.TCD[7].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH8_CSR   EDMA_3_TCD.TCD[8].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH8_ES    EDMA_3_TCD.TCD[8].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH8_INT   EDMA_3_TCD.TCD[8].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH8_SBR   EDMA_3_TCD.TCD[8].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH8_PRI   EDMA_3_TCD.TCD[8].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH8_MATTR EDMA_3_TCD.TCD[8].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD8_SADDR EDMA_3_TCD.TCD[8].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD8_SOFF EDMA_3_TCD.TCD[8].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD8_ATTR EDMA_3_TCD.TCD[8].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD8_NBYTES_MLOFFNO EDMA_3_TCD.TCD[8].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD8_NBYTES_MLOFFYES EDMA_3_TCD.TCD[8].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD8_SLAST_SDA EDMA_3_TCD.TCD[8].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD8_DADDR EDMA_3_TCD.TCD[8].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD8_DOFF EDMA_3_TCD.TCD[8].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD8_CITER_ELINKNO EDMA_3_TCD.TCD[8].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD8_CITER_ELINKYES EDMA_3_TCD.TCD[8].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD8_DLAST_SGA EDMA_3_TCD.TCD[8].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD8_CSR  EDMA_3_TCD.TCD[8].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD8_BITER_ELINKNO EDMA_3_TCD.TCD[8].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD8_BITER_ELINKYES EDMA_3_TCD.TCD[8].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH9_CSR   EDMA_3_TCD.TCD[9].CH_CSR.R    /* Channel Control and Status Register */
#define EDMA_3_TCD_CH9_ES    EDMA_3_TCD.TCD[9].CH_ES.R     /* Channel Error Status Register */
#define EDMA_3_TCD_CH9_INT   EDMA_3_TCD.TCD[9].CH_INT.R    /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH9_SBR   EDMA_3_TCD.TCD[9].CH_SBR.R    /* Channel System Bus Register */
#define EDMA_3_TCD_CH9_PRI   EDMA_3_TCD.TCD[9].CH_PRI.R    /* Channel Priority Register */
#define EDMA_3_TCD_CH9_MATTR EDMA_3_TCD.TCD[9].CH_MATTR.R  /* Memory Attributes Register */
#define EDMA_3_TCD_TCD9_SADDR EDMA_3_TCD.TCD[9].SADDR.R    /* TCD Source Address Register */
#define EDMA_3_TCD_TCD9_SOFF EDMA_3_TCD.TCD[9].SOFF.R      /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD9_ATTR EDMA_3_TCD.TCD[9].ATTR.R      /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD9_NBYTES_MLOFFNO EDMA_3_TCD.TCD[9].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD9_NBYTES_MLOFFYES EDMA_3_TCD.TCD[9].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD9_SLAST_SDA EDMA_3_TCD.TCD[9].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD9_DADDR EDMA_3_TCD.TCD[9].DADDR.R    /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD9_DOFF EDMA_3_TCD.TCD[9].DOFF.R      /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD9_CITER_ELINKNO EDMA_3_TCD.TCD[9].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD9_CITER_ELINKYES EDMA_3_TCD.TCD[9].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD9_DLAST_SGA EDMA_3_TCD.TCD[9].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD9_CSR  EDMA_3_TCD.TCD[9].CSR.R       /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD9_BITER_ELINKNO EDMA_3_TCD.TCD[9].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD9_BITER_ELINKYES EDMA_3_TCD.TCD[9].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH10_CSR  EDMA_3_TCD.TCD[10].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH10_ES   EDMA_3_TCD.TCD[10].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH10_INT  EDMA_3_TCD.TCD[10].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH10_SBR  EDMA_3_TCD.TCD[10].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH10_PRI  EDMA_3_TCD.TCD[10].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH10_MATTR EDMA_3_TCD.TCD[10].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD10_SADDR EDMA_3_TCD.TCD[10].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD10_SOFF EDMA_3_TCD.TCD[10].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD10_ATTR EDMA_3_TCD.TCD[10].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD10_NBYTES_MLOFFNO EDMA_3_TCD.TCD[10].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD10_NBYTES_MLOFFYES EDMA_3_TCD.TCD[10].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD10_SLAST_SDA EDMA_3_TCD.TCD[10].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD10_DADDR EDMA_3_TCD.TCD[10].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD10_DOFF EDMA_3_TCD.TCD[10].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD10_CITER_ELINKNO EDMA_3_TCD.TCD[10].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD10_CITER_ELINKYES EDMA_3_TCD.TCD[10].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD10_DLAST_SGA EDMA_3_TCD.TCD[10].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD10_CSR EDMA_3_TCD.TCD[10].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD10_BITER_ELINKNO EDMA_3_TCD.TCD[10].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD10_BITER_ELINKYES EDMA_3_TCD.TCD[10].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH11_CSR  EDMA_3_TCD.TCD[11].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH11_ES   EDMA_3_TCD.TCD[11].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH11_INT  EDMA_3_TCD.TCD[11].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH11_SBR  EDMA_3_TCD.TCD[11].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH11_PRI  EDMA_3_TCD.TCD[11].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH11_MATTR EDMA_3_TCD.TCD[11].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD11_SADDR EDMA_3_TCD.TCD[11].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD11_SOFF EDMA_3_TCD.TCD[11].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD11_ATTR EDMA_3_TCD.TCD[11].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD11_NBYTES_MLOFFNO EDMA_3_TCD.TCD[11].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD11_NBYTES_MLOFFYES EDMA_3_TCD.TCD[11].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD11_SLAST_SDA EDMA_3_TCD.TCD[11].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD11_DADDR EDMA_3_TCD.TCD[11].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD11_DOFF EDMA_3_TCD.TCD[11].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD11_CITER_ELINKNO EDMA_3_TCD.TCD[11].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD11_CITER_ELINKYES EDMA_3_TCD.TCD[11].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD11_DLAST_SGA EDMA_3_TCD.TCD[11].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD11_CSR EDMA_3_TCD.TCD[11].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD11_BITER_ELINKNO EDMA_3_TCD.TCD[11].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD11_BITER_ELINKYES EDMA_3_TCD.TCD[11].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH12_CSR  EDMA_3_TCD.TCD[12].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH12_ES   EDMA_3_TCD.TCD[12].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH12_INT  EDMA_3_TCD.TCD[12].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH12_SBR  EDMA_3_TCD.TCD[12].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH12_PRI  EDMA_3_TCD.TCD[12].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH12_MATTR EDMA_3_TCD.TCD[12].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD12_SADDR EDMA_3_TCD.TCD[12].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD12_SOFF EDMA_3_TCD.TCD[12].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD12_ATTR EDMA_3_TCD.TCD[12].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD12_NBYTES_MLOFFNO EDMA_3_TCD.TCD[12].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD12_NBYTES_MLOFFYES EDMA_3_TCD.TCD[12].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD12_SLAST_SDA EDMA_3_TCD.TCD[12].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD12_DADDR EDMA_3_TCD.TCD[12].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD12_DOFF EDMA_3_TCD.TCD[12].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD12_CITER_ELINKNO EDMA_3_TCD.TCD[12].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD12_CITER_ELINKYES EDMA_3_TCD.TCD[12].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD12_DLAST_SGA EDMA_3_TCD.TCD[12].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD12_CSR EDMA_3_TCD.TCD[12].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD12_BITER_ELINKNO EDMA_3_TCD.TCD[12].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD12_BITER_ELINKYES EDMA_3_TCD.TCD[12].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH13_CSR  EDMA_3_TCD.TCD[13].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH13_ES   EDMA_3_TCD.TCD[13].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH13_INT  EDMA_3_TCD.TCD[13].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH13_SBR  EDMA_3_TCD.TCD[13].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH13_PRI  EDMA_3_TCD.TCD[13].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH13_MATTR EDMA_3_TCD.TCD[13].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD13_SADDR EDMA_3_TCD.TCD[13].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD13_SOFF EDMA_3_TCD.TCD[13].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD13_ATTR EDMA_3_TCD.TCD[13].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD13_NBYTES_MLOFFNO EDMA_3_TCD.TCD[13].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD13_NBYTES_MLOFFYES EDMA_3_TCD.TCD[13].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD13_SLAST_SDA EDMA_3_TCD.TCD[13].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD13_DADDR EDMA_3_TCD.TCD[13].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD13_DOFF EDMA_3_TCD.TCD[13].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD13_CITER_ELINKNO EDMA_3_TCD.TCD[13].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD13_CITER_ELINKYES EDMA_3_TCD.TCD[13].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD13_DLAST_SGA EDMA_3_TCD.TCD[13].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD13_CSR EDMA_3_TCD.TCD[13].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD13_BITER_ELINKNO EDMA_3_TCD.TCD[13].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD13_BITER_ELINKYES EDMA_3_TCD.TCD[13].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH14_CSR  EDMA_3_TCD.TCD[14].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH14_ES   EDMA_3_TCD.TCD[14].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH14_INT  EDMA_3_TCD.TCD[14].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH14_SBR  EDMA_3_TCD.TCD[14].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH14_PRI  EDMA_3_TCD.TCD[14].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH14_MATTR EDMA_3_TCD.TCD[14].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD14_SADDR EDMA_3_TCD.TCD[14].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD14_SOFF EDMA_3_TCD.TCD[14].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD14_ATTR EDMA_3_TCD.TCD[14].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD14_NBYTES_MLOFFNO EDMA_3_TCD.TCD[14].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD14_NBYTES_MLOFFYES EDMA_3_TCD.TCD[14].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD14_SLAST_SDA EDMA_3_TCD.TCD[14].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD14_DADDR EDMA_3_TCD.TCD[14].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD14_DOFF EDMA_3_TCD.TCD[14].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD14_CITER_ELINKNO EDMA_3_TCD.TCD[14].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD14_CITER_ELINKYES EDMA_3_TCD.TCD[14].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD14_DLAST_SGA EDMA_3_TCD.TCD[14].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD14_CSR EDMA_3_TCD.TCD[14].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD14_BITER_ELINKNO EDMA_3_TCD.TCD[14].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD14_BITER_ELINKYES EDMA_3_TCD.TCD[14].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH15_CSR  EDMA_3_TCD.TCD[15].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH15_ES   EDMA_3_TCD.TCD[15].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH15_INT  EDMA_3_TCD.TCD[15].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH15_SBR  EDMA_3_TCD.TCD[15].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH15_PRI  EDMA_3_TCD.TCD[15].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH15_MATTR EDMA_3_TCD.TCD[15].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD15_SADDR EDMA_3_TCD.TCD[15].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD15_SOFF EDMA_3_TCD.TCD[15].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD15_ATTR EDMA_3_TCD.TCD[15].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD15_NBYTES_MLOFFNO EDMA_3_TCD.TCD[15].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD15_NBYTES_MLOFFYES EDMA_3_TCD.TCD[15].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD15_SLAST_SDA EDMA_3_TCD.TCD[15].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD15_DADDR EDMA_3_TCD.TCD[15].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD15_DOFF EDMA_3_TCD.TCD[15].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD15_CITER_ELINKNO EDMA_3_TCD.TCD[15].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD15_CITER_ELINKYES EDMA_3_TCD.TCD[15].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD15_DLAST_SGA EDMA_3_TCD.TCD[15].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD15_CSR EDMA_3_TCD.TCD[15].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD15_BITER_ELINKNO EDMA_3_TCD.TCD[15].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD15_BITER_ELINKYES EDMA_3_TCD.TCD[15].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH16_CSR  EDMA_3_TCD.TCD[16].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH16_ES   EDMA_3_TCD.TCD[16].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH16_INT  EDMA_3_TCD.TCD[16].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH16_SBR  EDMA_3_TCD.TCD[16].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH16_PRI  EDMA_3_TCD.TCD[16].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH16_MATTR EDMA_3_TCD.TCD[16].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD16_SADDR EDMA_3_TCD.TCD[16].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD16_SOFF EDMA_3_TCD.TCD[16].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD16_ATTR EDMA_3_TCD.TCD[16].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD16_NBYTES_MLOFFNO EDMA_3_TCD.TCD[16].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD16_NBYTES_MLOFFYES EDMA_3_TCD.TCD[16].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD16_SLAST_SDA EDMA_3_TCD.TCD[16].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD16_DADDR EDMA_3_TCD.TCD[16].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD16_DOFF EDMA_3_TCD.TCD[16].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD16_CITER_ELINKNO EDMA_3_TCD.TCD[16].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD16_CITER_ELINKYES EDMA_3_TCD.TCD[16].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD16_DLAST_SGA EDMA_3_TCD.TCD[16].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD16_CSR EDMA_3_TCD.TCD[16].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD16_BITER_ELINKNO EDMA_3_TCD.TCD[16].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD16_BITER_ELINKYES EDMA_3_TCD.TCD[16].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH17_CSR  EDMA_3_TCD.TCD[17].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH17_ES   EDMA_3_TCD.TCD[17].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH17_INT  EDMA_3_TCD.TCD[17].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH17_SBR  EDMA_3_TCD.TCD[17].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH17_PRI  EDMA_3_TCD.TCD[17].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH17_MATTR EDMA_3_TCD.TCD[17].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD17_SADDR EDMA_3_TCD.TCD[17].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD17_SOFF EDMA_3_TCD.TCD[17].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD17_ATTR EDMA_3_TCD.TCD[17].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD17_NBYTES_MLOFFNO EDMA_3_TCD.TCD[17].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD17_NBYTES_MLOFFYES EDMA_3_TCD.TCD[17].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD17_SLAST_SDA EDMA_3_TCD.TCD[17].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD17_DADDR EDMA_3_TCD.TCD[17].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD17_DOFF EDMA_3_TCD.TCD[17].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD17_CITER_ELINKNO EDMA_3_TCD.TCD[17].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD17_CITER_ELINKYES EDMA_3_TCD.TCD[17].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD17_DLAST_SGA EDMA_3_TCD.TCD[17].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD17_CSR EDMA_3_TCD.TCD[17].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD17_BITER_ELINKNO EDMA_3_TCD.TCD[17].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD17_BITER_ELINKYES EDMA_3_TCD.TCD[17].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH18_CSR  EDMA_3_TCD.TCD[18].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH18_ES   EDMA_3_TCD.TCD[18].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH18_INT  EDMA_3_TCD.TCD[18].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH18_SBR  EDMA_3_TCD.TCD[18].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH18_PRI  EDMA_3_TCD.TCD[18].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH18_MATTR EDMA_3_TCD.TCD[18].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD18_SADDR EDMA_3_TCD.TCD[18].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD18_SOFF EDMA_3_TCD.TCD[18].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD18_ATTR EDMA_3_TCD.TCD[18].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD18_NBYTES_MLOFFNO EDMA_3_TCD.TCD[18].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD18_NBYTES_MLOFFYES EDMA_3_TCD.TCD[18].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD18_SLAST_SDA EDMA_3_TCD.TCD[18].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD18_DADDR EDMA_3_TCD.TCD[18].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD18_DOFF EDMA_3_TCD.TCD[18].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD18_CITER_ELINKNO EDMA_3_TCD.TCD[18].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD18_CITER_ELINKYES EDMA_3_TCD.TCD[18].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD18_DLAST_SGA EDMA_3_TCD.TCD[18].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD18_CSR EDMA_3_TCD.TCD[18].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD18_BITER_ELINKNO EDMA_3_TCD.TCD[18].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD18_BITER_ELINKYES EDMA_3_TCD.TCD[18].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH19_CSR  EDMA_3_TCD.TCD[19].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH19_ES   EDMA_3_TCD.TCD[19].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH19_INT  EDMA_3_TCD.TCD[19].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH19_SBR  EDMA_3_TCD.TCD[19].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH19_PRI  EDMA_3_TCD.TCD[19].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH19_MATTR EDMA_3_TCD.TCD[19].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD19_SADDR EDMA_3_TCD.TCD[19].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD19_SOFF EDMA_3_TCD.TCD[19].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD19_ATTR EDMA_3_TCD.TCD[19].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD19_NBYTES_MLOFFNO EDMA_3_TCD.TCD[19].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD19_NBYTES_MLOFFYES EDMA_3_TCD.TCD[19].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD19_SLAST_SDA EDMA_3_TCD.TCD[19].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD19_DADDR EDMA_3_TCD.TCD[19].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD19_DOFF EDMA_3_TCD.TCD[19].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD19_CITER_ELINKNO EDMA_3_TCD.TCD[19].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD19_CITER_ELINKYES EDMA_3_TCD.TCD[19].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD19_DLAST_SGA EDMA_3_TCD.TCD[19].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD19_CSR EDMA_3_TCD.TCD[19].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD19_BITER_ELINKNO EDMA_3_TCD.TCD[19].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD19_BITER_ELINKYES EDMA_3_TCD.TCD[19].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH20_CSR  EDMA_3_TCD.TCD[20].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH20_ES   EDMA_3_TCD.TCD[20].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH20_INT  EDMA_3_TCD.TCD[20].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH20_SBR  EDMA_3_TCD.TCD[20].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH20_PRI  EDMA_3_TCD.TCD[20].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH20_MATTR EDMA_3_TCD.TCD[20].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD20_SADDR EDMA_3_TCD.TCD[20].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD20_SOFF EDMA_3_TCD.TCD[20].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD20_ATTR EDMA_3_TCD.TCD[20].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD20_NBYTES_MLOFFNO EDMA_3_TCD.TCD[20].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD20_NBYTES_MLOFFYES EDMA_3_TCD.TCD[20].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD20_SLAST_SDA EDMA_3_TCD.TCD[20].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD20_DADDR EDMA_3_TCD.TCD[20].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD20_DOFF EDMA_3_TCD.TCD[20].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD20_CITER_ELINKNO EDMA_3_TCD.TCD[20].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD20_CITER_ELINKYES EDMA_3_TCD.TCD[20].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD20_DLAST_SGA EDMA_3_TCD.TCD[20].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD20_CSR EDMA_3_TCD.TCD[20].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD20_BITER_ELINKNO EDMA_3_TCD.TCD[20].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD20_BITER_ELINKYES EDMA_3_TCD.TCD[20].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH21_CSR  EDMA_3_TCD.TCD[21].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH21_ES   EDMA_3_TCD.TCD[21].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH21_INT  EDMA_3_TCD.TCD[21].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH21_SBR  EDMA_3_TCD.TCD[21].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH21_PRI  EDMA_3_TCD.TCD[21].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH21_MATTR EDMA_3_TCD.TCD[21].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD21_SADDR EDMA_3_TCD.TCD[21].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD21_SOFF EDMA_3_TCD.TCD[21].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD21_ATTR EDMA_3_TCD.TCD[21].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD21_NBYTES_MLOFFNO EDMA_3_TCD.TCD[21].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD21_NBYTES_MLOFFYES EDMA_3_TCD.TCD[21].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD21_SLAST_SDA EDMA_3_TCD.TCD[21].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD21_DADDR EDMA_3_TCD.TCD[21].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD21_DOFF EDMA_3_TCD.TCD[21].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD21_CITER_ELINKNO EDMA_3_TCD.TCD[21].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD21_CITER_ELINKYES EDMA_3_TCD.TCD[21].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD21_DLAST_SGA EDMA_3_TCD.TCD[21].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD21_CSR EDMA_3_TCD.TCD[21].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD21_BITER_ELINKNO EDMA_3_TCD.TCD[21].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD21_BITER_ELINKYES EDMA_3_TCD.TCD[21].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH22_CSR  EDMA_3_TCD.TCD[22].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH22_ES   EDMA_3_TCD.TCD[22].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH22_INT  EDMA_3_TCD.TCD[22].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH22_SBR  EDMA_3_TCD.TCD[22].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH22_PRI  EDMA_3_TCD.TCD[22].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH22_MATTR EDMA_3_TCD.TCD[22].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD22_SADDR EDMA_3_TCD.TCD[22].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD22_SOFF EDMA_3_TCD.TCD[22].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD22_ATTR EDMA_3_TCD.TCD[22].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD22_NBYTES_MLOFFNO EDMA_3_TCD.TCD[22].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD22_NBYTES_MLOFFYES EDMA_3_TCD.TCD[22].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD22_SLAST_SDA EDMA_3_TCD.TCD[22].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD22_DADDR EDMA_3_TCD.TCD[22].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD22_DOFF EDMA_3_TCD.TCD[22].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD22_CITER_ELINKNO EDMA_3_TCD.TCD[22].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD22_CITER_ELINKYES EDMA_3_TCD.TCD[22].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD22_DLAST_SGA EDMA_3_TCD.TCD[22].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD22_CSR EDMA_3_TCD.TCD[22].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD22_BITER_ELINKNO EDMA_3_TCD.TCD[22].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD22_BITER_ELINKYES EDMA_3_TCD.TCD[22].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH23_CSR  EDMA_3_TCD.TCD[23].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH23_ES   EDMA_3_TCD.TCD[23].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH23_INT  EDMA_3_TCD.TCD[23].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH23_SBR  EDMA_3_TCD.TCD[23].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH23_PRI  EDMA_3_TCD.TCD[23].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH23_MATTR EDMA_3_TCD.TCD[23].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD23_SADDR EDMA_3_TCD.TCD[23].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD23_SOFF EDMA_3_TCD.TCD[23].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD23_ATTR EDMA_3_TCD.TCD[23].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD23_NBYTES_MLOFFNO EDMA_3_TCD.TCD[23].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD23_NBYTES_MLOFFYES EDMA_3_TCD.TCD[23].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD23_SLAST_SDA EDMA_3_TCD.TCD[23].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD23_DADDR EDMA_3_TCD.TCD[23].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD23_DOFF EDMA_3_TCD.TCD[23].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD23_CITER_ELINKNO EDMA_3_TCD.TCD[23].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD23_CITER_ELINKYES EDMA_3_TCD.TCD[23].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD23_DLAST_SGA EDMA_3_TCD.TCD[23].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD23_CSR EDMA_3_TCD.TCD[23].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD23_BITER_ELINKNO EDMA_3_TCD.TCD[23].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD23_BITER_ELINKYES EDMA_3_TCD.TCD[23].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH24_CSR  EDMA_3_TCD.TCD[24].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH24_ES   EDMA_3_TCD.TCD[24].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH24_INT  EDMA_3_TCD.TCD[24].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH24_SBR  EDMA_3_TCD.TCD[24].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH24_PRI  EDMA_3_TCD.TCD[24].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH24_MATTR EDMA_3_TCD.TCD[24].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD24_SADDR EDMA_3_TCD.TCD[24].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD24_SOFF EDMA_3_TCD.TCD[24].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD24_ATTR EDMA_3_TCD.TCD[24].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD24_NBYTES_MLOFFNO EDMA_3_TCD.TCD[24].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD24_NBYTES_MLOFFYES EDMA_3_TCD.TCD[24].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD24_SLAST_SDA EDMA_3_TCD.TCD[24].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD24_DADDR EDMA_3_TCD.TCD[24].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD24_DOFF EDMA_3_TCD.TCD[24].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD24_CITER_ELINKNO EDMA_3_TCD.TCD[24].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD24_CITER_ELINKYES EDMA_3_TCD.TCD[24].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD24_DLAST_SGA EDMA_3_TCD.TCD[24].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD24_CSR EDMA_3_TCD.TCD[24].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD24_BITER_ELINKNO EDMA_3_TCD.TCD[24].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD24_BITER_ELINKYES EDMA_3_TCD.TCD[24].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH25_CSR  EDMA_3_TCD.TCD[25].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH25_ES   EDMA_3_TCD.TCD[25].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH25_INT  EDMA_3_TCD.TCD[25].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH25_SBR  EDMA_3_TCD.TCD[25].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH25_PRI  EDMA_3_TCD.TCD[25].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH25_MATTR EDMA_3_TCD.TCD[25].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD25_SADDR EDMA_3_TCD.TCD[25].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD25_SOFF EDMA_3_TCD.TCD[25].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD25_ATTR EDMA_3_TCD.TCD[25].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD25_NBYTES_MLOFFNO EDMA_3_TCD.TCD[25].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD25_NBYTES_MLOFFYES EDMA_3_TCD.TCD[25].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD25_SLAST_SDA EDMA_3_TCD.TCD[25].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD25_DADDR EDMA_3_TCD.TCD[25].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD25_DOFF EDMA_3_TCD.TCD[25].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD25_CITER_ELINKNO EDMA_3_TCD.TCD[25].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD25_CITER_ELINKYES EDMA_3_TCD.TCD[25].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD25_DLAST_SGA EDMA_3_TCD.TCD[25].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD25_CSR EDMA_3_TCD.TCD[25].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD25_BITER_ELINKNO EDMA_3_TCD.TCD[25].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD25_BITER_ELINKYES EDMA_3_TCD.TCD[25].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH26_CSR  EDMA_3_TCD.TCD[26].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH26_ES   EDMA_3_TCD.TCD[26].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH26_INT  EDMA_3_TCD.TCD[26].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH26_SBR  EDMA_3_TCD.TCD[26].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH26_PRI  EDMA_3_TCD.TCD[26].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH26_MATTR EDMA_3_TCD.TCD[26].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD26_SADDR EDMA_3_TCD.TCD[26].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD26_SOFF EDMA_3_TCD.TCD[26].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD26_ATTR EDMA_3_TCD.TCD[26].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD26_NBYTES_MLOFFNO EDMA_3_TCD.TCD[26].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD26_NBYTES_MLOFFYES EDMA_3_TCD.TCD[26].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD26_SLAST_SDA EDMA_3_TCD.TCD[26].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD26_DADDR EDMA_3_TCD.TCD[26].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD26_DOFF EDMA_3_TCD.TCD[26].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD26_CITER_ELINKNO EDMA_3_TCD.TCD[26].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD26_CITER_ELINKYES EDMA_3_TCD.TCD[26].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD26_DLAST_SGA EDMA_3_TCD.TCD[26].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD26_CSR EDMA_3_TCD.TCD[26].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD26_BITER_ELINKNO EDMA_3_TCD.TCD[26].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD26_BITER_ELINKYES EDMA_3_TCD.TCD[26].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH27_CSR  EDMA_3_TCD.TCD[27].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH27_ES   EDMA_3_TCD.TCD[27].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH27_INT  EDMA_3_TCD.TCD[27].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH27_SBR  EDMA_3_TCD.TCD[27].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH27_PRI  EDMA_3_TCD.TCD[27].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH27_MATTR EDMA_3_TCD.TCD[27].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD27_SADDR EDMA_3_TCD.TCD[27].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD27_SOFF EDMA_3_TCD.TCD[27].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD27_ATTR EDMA_3_TCD.TCD[27].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD27_NBYTES_MLOFFNO EDMA_3_TCD.TCD[27].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD27_NBYTES_MLOFFYES EDMA_3_TCD.TCD[27].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD27_SLAST_SDA EDMA_3_TCD.TCD[27].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD27_DADDR EDMA_3_TCD.TCD[27].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD27_DOFF EDMA_3_TCD.TCD[27].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD27_CITER_ELINKNO EDMA_3_TCD.TCD[27].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD27_CITER_ELINKYES EDMA_3_TCD.TCD[27].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD27_DLAST_SGA EDMA_3_TCD.TCD[27].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD27_CSR EDMA_3_TCD.TCD[27].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD27_BITER_ELINKNO EDMA_3_TCD.TCD[27].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD27_BITER_ELINKYES EDMA_3_TCD.TCD[27].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH28_CSR  EDMA_3_TCD.TCD[28].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH28_ES   EDMA_3_TCD.TCD[28].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH28_INT  EDMA_3_TCD.TCD[28].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH28_SBR  EDMA_3_TCD.TCD[28].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH28_PRI  EDMA_3_TCD.TCD[28].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH28_MATTR EDMA_3_TCD.TCD[28].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD28_SADDR EDMA_3_TCD.TCD[28].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD28_SOFF EDMA_3_TCD.TCD[28].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD28_ATTR EDMA_3_TCD.TCD[28].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD28_NBYTES_MLOFFNO EDMA_3_TCD.TCD[28].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD28_NBYTES_MLOFFYES EDMA_3_TCD.TCD[28].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD28_SLAST_SDA EDMA_3_TCD.TCD[28].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD28_DADDR EDMA_3_TCD.TCD[28].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD28_DOFF EDMA_3_TCD.TCD[28].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD28_CITER_ELINKNO EDMA_3_TCD.TCD[28].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD28_CITER_ELINKYES EDMA_3_TCD.TCD[28].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD28_DLAST_SGA EDMA_3_TCD.TCD[28].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD28_CSR EDMA_3_TCD.TCD[28].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD28_BITER_ELINKNO EDMA_3_TCD.TCD[28].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD28_BITER_ELINKYES EDMA_3_TCD.TCD[28].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH29_CSR  EDMA_3_TCD.TCD[29].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH29_ES   EDMA_3_TCD.TCD[29].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH29_INT  EDMA_3_TCD.TCD[29].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH29_SBR  EDMA_3_TCD.TCD[29].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH29_PRI  EDMA_3_TCD.TCD[29].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH29_MATTR EDMA_3_TCD.TCD[29].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD29_SADDR EDMA_3_TCD.TCD[29].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD29_SOFF EDMA_3_TCD.TCD[29].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD29_ATTR EDMA_3_TCD.TCD[29].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD29_NBYTES_MLOFFNO EDMA_3_TCD.TCD[29].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD29_NBYTES_MLOFFYES EDMA_3_TCD.TCD[29].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD29_SLAST_SDA EDMA_3_TCD.TCD[29].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD29_DADDR EDMA_3_TCD.TCD[29].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD29_DOFF EDMA_3_TCD.TCD[29].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD29_CITER_ELINKNO EDMA_3_TCD.TCD[29].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD29_CITER_ELINKYES EDMA_3_TCD.TCD[29].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD29_DLAST_SGA EDMA_3_TCD.TCD[29].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD29_CSR EDMA_3_TCD.TCD[29].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD29_BITER_ELINKNO EDMA_3_TCD.TCD[29].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD29_BITER_ELINKYES EDMA_3_TCD.TCD[29].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH30_CSR  EDMA_3_TCD.TCD[30].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH30_ES   EDMA_3_TCD.TCD[30].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH30_INT  EDMA_3_TCD.TCD[30].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH30_SBR  EDMA_3_TCD.TCD[30].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH30_PRI  EDMA_3_TCD.TCD[30].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH30_MATTR EDMA_3_TCD.TCD[30].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD30_SADDR EDMA_3_TCD.TCD[30].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD30_SOFF EDMA_3_TCD.TCD[30].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD30_ATTR EDMA_3_TCD.TCD[30].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD30_NBYTES_MLOFFNO EDMA_3_TCD.TCD[30].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD30_NBYTES_MLOFFYES EDMA_3_TCD.TCD[30].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD30_SLAST_SDA EDMA_3_TCD.TCD[30].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD30_DADDR EDMA_3_TCD.TCD[30].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD30_DOFF EDMA_3_TCD.TCD[30].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD30_CITER_ELINKNO EDMA_3_TCD.TCD[30].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD30_CITER_ELINKYES EDMA_3_TCD.TCD[30].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD30_DLAST_SGA EDMA_3_TCD.TCD[30].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD30_CSR EDMA_3_TCD.TCD[30].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD30_BITER_ELINKNO EDMA_3_TCD.TCD[30].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD30_BITER_ELINKYES EDMA_3_TCD.TCD[30].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_CH31_CSR  EDMA_3_TCD.TCD[31].CH_CSR.R   /* Channel Control and Status Register */
#define EDMA_3_TCD_CH31_ES   EDMA_3_TCD.TCD[31].CH_ES.R    /* Channel Error Status Register */
#define EDMA_3_TCD_CH31_INT  EDMA_3_TCD.TCD[31].CH_INT.R   /* Channel Interrupt Status Register */
#define EDMA_3_TCD_CH31_SBR  EDMA_3_TCD.TCD[31].CH_SBR.R   /* Channel System Bus Register */
#define EDMA_3_TCD_CH31_PRI  EDMA_3_TCD.TCD[31].CH_PRI.R   /* Channel Priority Register */
#define EDMA_3_TCD_CH31_MATTR EDMA_3_TCD.TCD[31].CH_MATTR.R /* Memory Attributes Register */
#define EDMA_3_TCD_TCD31_SADDR EDMA_3_TCD.TCD[31].SADDR.R  /* TCD Source Address Register */
#define EDMA_3_TCD_TCD31_SOFF EDMA_3_TCD.TCD[31].SOFF.R    /* TCD Signed Source Address Offset Register */
#define EDMA_3_TCD_TCD31_ATTR EDMA_3_TCD.TCD[31].ATTR.R    /* TCD Transfer Attributes Register */
#define EDMA_3_TCD_TCD31_NBYTES_MLOFFNO EDMA_3_TCD.TCD[31].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size without Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD31_NBYTES_MLOFFYES EDMA_3_TCD.TCD[31].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets Register */
#define EDMA_3_TCD_TCD31_SLAST_SDA EDMA_3_TCD.TCD[31].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address Register */
#define EDMA_3_TCD_TCD31_DADDR EDMA_3_TCD.TCD[31].DADDR.R  /* TCD Destination Address Register */
#define EDMA_3_TCD_TCD31_DOFF EDMA_3_TCD.TCD[31].DOFF.R    /* TCD Signed Destination Address Offset Register */
#define EDMA_3_TCD_TCD31_CITER_ELINKNO EDMA_3_TCD.TCD[31].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD31_CITER_ELINKYES EDMA_3_TCD.TCD[31].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
#define EDMA_3_TCD_TCD31_DLAST_SGA EDMA_3_TCD.TCD[31].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address Register */
#define EDMA_3_TCD_TCD31_CSR EDMA_3_TCD.TCD[31].CSR.R      /* TCD Control and Status Register */
#define EDMA_3_TCD_TCD31_BITER_ELINKNO EDMA_3_TCD.TCD[31].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
#define EDMA_3_TCD_TCD31_BITER_ELINKYES EDMA_3_TCD.TCD[31].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */

/* EIM */
#define CE_EIM_0_EIMCR       CE_EIM_0.EIMCR.R              /* Error Injection Module Configuration Register */
#define CE_EIM_0_EICHEN      CE_EIM_0.EICHEN.R             /* Error Injection Channel Enable register */
#define CE_EIM_0_EICHD0_WORD0 CE_EIM_0.EICHD0_WORD0.R      /* Error Injection Channel Descriptor 0, Word0 */
#define CE_EIM_0_EICHD0_WORD1 CE_EIM_0.EICHD0_WORD1.R      /* Error Injection Channel Descriptor 0, Word1 */
#define CE_EIM_0_EICHD0_WORD2 CE_EIM_0.EICHD0_WORD2.R      /* Error Injection Channel Descriptor 0, Word2 */
#define CE_EIM_0_EICHD1_WORD0 CE_EIM_0.EICHD1_WORD0.R      /* Error Injection Channel Descriptor 1, Word0 */
#define CE_EIM_0_EICHD1_WORD1 CE_EIM_0.EICHD1_WORD1.R      /* Error Injection Channel Descriptor 1, Word1 */
#define CE_EIM_0_EICHD1_WORD2 CE_EIM_0.EICHD1_WORD2.R      /* Error Injection Channel Descriptor 1, Word2 */
#define CE_EIM_0_EICHD2_WORD0 CE_EIM_0.EICHD2_WORD0.R      /* Error Injection Channel Descriptor 2, Word0 */
#define CE_EIM_0_EICHD2_WORD1 CE_EIM_0.EICHD2_WORD1.R      /* Error Injection Channel Descriptor 2, Word1 */
#define CE_EIM_0_EICHD2_WORD2 CE_EIM_0.EICHD2_WORD2.R      /* Error Injection Channel Descriptor 2, Word2 */
#define CE_EIM_0_EICHD3_WORD0 CE_EIM_0.EICHD3_WORD0.R      /* Error Injection Channel Descriptor 3, Word0 */
#define CE_EIM_0_EICHD3_WORD1 CE_EIM_0.EICHD3_WORD1.R      /* Error Injection Channel Descriptor 3, Word1 */
#define CE_EIM_0_EICHD3_WORD2 CE_EIM_0.EICHD3_WORD2.R      /* Error Injection Channel Descriptor 3, Word2 */
#define CE_EIM_0_EICHD7_WORD0 CE_EIM_0.EICHD7_WORD0.R      /* Error Injection Channel Descriptor 7, Word0 */
#define CE_EIM_0_EICHD7_WORD1 CE_EIM_0.EICHD7_WORD1.R      /* Error Injection Channel Descriptor 7, Word1 */
#define CE_EIM_0_EICHD7_WORD2 CE_EIM_0.EICHD7_WORD2.R      /* Error Injection Channel Descriptor 7, Word2 */
#define CE_EIM_0_EICHD8_WORD0 CE_EIM_0.EICHD8_WORD0.R      /* Error Injection Channel Descriptor 8, Word0 */
#define CE_EIM_0_EICHD8_WORD1 CE_EIM_0.EICHD8_WORD1.R      /* Error Injection Channel Descriptor 8, Word1 */
#define CE_EIM_0_EICHD8_WORD2 CE_EIM_0.EICHD8_WORD2.R      /* Error Injection Channel Descriptor 8, Word2 */
#define CE_EIM_0_EICHD8_WORD3 CE_EIM_0.EICHD8_WORD3.R      /* Error Injection Channel Descriptor 8, Word3 */
#define CE_EIM_0_EICHD8_WORD4 CE_EIM_0.EICHD8_WORD4.R      /* Error Injection Channel Descriptor 8, Word4 */
#define CE_EIM_0_EICHD9_WORD0 CE_EIM_0.EICHD9_WORD0.R      /* Error Injection Channel Descriptor 9, Word0 */
#define CE_EIM_0_EICHD9_WORD1 CE_EIM_0.EICHD9_WORD1.R      /* Error Injection Channel Descriptor 9, Word1 */
#define CE_EIM_0_EICHD9_WORD2 CE_EIM_0.EICHD9_WORD2.R      /* Error Injection Channel Descriptor 9, Word2 */
#define CE_EIM_0_EICHD10_WORD0 CE_EIM_0.EICHD10_WORD0.R    /* Error Injection Channel Descriptor 10, Word0 */
#define CE_EIM_0_EICHD10_WORD1 CE_EIM_0.EICHD10_WORD1.R    /* Error Injection Channel Descriptor 10, Word1 */
#define CE_EIM_0_EICHD10_WORD2 CE_EIM_0.EICHD10_WORD2.R    /* Error Injection Channel Descriptor 10, Word2 */
#define CE_EIM_0_EICHD10_WORD3 CE_EIM_0.EICHD10_WORD3.R    /* Error Injection Channel Descriptor 10, Word3 */
#define CE_EIM_0_EICHD10_WORD4 CE_EIM_0.EICHD10_WORD4.R    /* Error Injection Channel Descriptor 10, Word4 */
#define CE_EIM_0_EICHD11_WORD0 CE_EIM_0.EICHD11_WORD0.R    /* Error Injection Channel Descriptor 11, Word0 */
#define CE_EIM_0_EICHD11_WORD1 CE_EIM_0.EICHD11_WORD1.R    /* Error Injection Channel Descriptor 11, Word1 */
#define CE_EIM_0_EICHD11_WORD2 CE_EIM_0.EICHD11_WORD2.R    /* Error Injection Channel Descriptor 11, Word2 */
#define CE_EIM_0_EICHD12_WORD0 CE_EIM_0.EICHD12_WORD0.R    /* Error Injection Channel Descriptor 12, Word0 */
#define CE_EIM_0_EICHD12_WORD1 CE_EIM_0.EICHD12_WORD1.R    /* Error Injection Channel Descriptor 12, Word1 */
#define CE_EIM_0_EICHD12_WORD2 CE_EIM_0.EICHD12_WORD2.R    /* Error Injection Channel Descriptor 12, Word2 */
#define CE_EIM_0_EICHD13_WORD0 CE_EIM_0.EICHD13_WORD0.R    /* Error Injection Channel Descriptor 13, Word0 */
#define CE_EIM_0_EICHD13_WORD1 CE_EIM_0.EICHD13_WORD1.R    /* Error Injection Channel Descriptor 13, Word1 */
#define CE_EIM_0_EICHD13_WORD2 CE_EIM_0.EICHD13_WORD2.R    /* Error Injection Channel Descriptor 13, Word2 */
#define CE_EIM_0_EICHD14_WORD0 CE_EIM_0.EICHD14_WORD0.R    /* Error Injection Channel Descriptor 14, Word0 */
#define CE_EIM_0_EICHD14_WORD1 CE_EIM_0.EICHD14_WORD1.R    /* Error Injection Channel Descriptor 14, Word1 */
#define CE_EIM_0_EICHD14_WORD2 CE_EIM_0.EICHD14_WORD2.R    /* Error Injection Channel Descriptor 14, Word2 */
#define CE_EIM_0_EICHD15_WORD0 CE_EIM_0.EICHD15_WORD0.R    /* Error Injection Channel Descriptor 15, Word0 */
#define CE_EIM_0_EICHD15_WORD1 CE_EIM_0.EICHD15_WORD1.R    /* Error Injection Channel Descriptor 15, Word1 */
#define CE_EIM_0_EICHD15_WORD2 CE_EIM_0.EICHD15_WORD2.R    /* Error Injection Channel Descriptor 15, Word2 */
#define CE_EIM_0_EICHD16_WORD0 CE_EIM_0.EICHD16_WORD0.R    /* Error Injection Channel Descriptor 16, Word0 */
#define CE_EIM_0_EICHD16_WORD1 CE_EIM_0.EICHD16_WORD1.R    /* Error Injection Channel Descriptor 16, Word1 */
#define CE_EIM_0_EICHD16_WORD2 CE_EIM_0.EICHD16_WORD2.R    /* Error Injection Channel Descriptor 16, Word2 */
#define CE_EIM_0_EICHD17_WORD0 CE_EIM_0.EICHD17_WORD0.R    /* Error Injection Channel Descriptor 17, Word0 */
#define CE_EIM_0_EICHD17_WORD1 CE_EIM_0.EICHD17_WORD1.R    /* Error Injection Channel Descriptor 17, Word1 */
#define CE_EIM_0_EICHD17_WORD2 CE_EIM_0.EICHD17_WORD2.R    /* Error Injection Channel Descriptor 17, Word2 */
#define CE_EIM_0_EICHD18_WORD1 CE_EIM_0.EICHD18_WORD1.R    /* Error Injection Channel Descriptor 18, Word1 */
#define CE_EIM_0_EICHD18_WORD2 CE_EIM_0.EICHD18_WORD2.R    /* Error Injection Channel Descriptor 18, Word2 */
#define CE_EIM_0_EICHD19_WORD1 CE_EIM_0.EICHD19_WORD1.R    /* Error Injection Channel Descriptor 19, Word1 */
#define CE_EIM_0_EICHD19_WORD2 CE_EIM_0.EICHD19_WORD2.R    /* Error Injection Channel Descriptor 19, Word2 */
#define CE_EIM_0_EICHD20_WORD1 CE_EIM_0.EICHD20_WORD1.R    /* Error Injection Channel Descriptor 20, Word1 */
#define CE_EIM_0_EICHD20_WORD2 CE_EIM_0.EICHD20_WORD2.R    /* Error Injection Channel Descriptor 20, Word2 */
#define CE_EIM_0_EICHD21_WORD1 CE_EIM_0.EICHD21_WORD1.R    /* Error Injection Channel Descriptor 21, Word1 */
#define CE_EIM_0_EICHD21_WORD2 CE_EIM_0.EICHD21_WORD2.R    /* Error Injection Channel Descriptor 21, Word2 */
#define CE_EIM_0_EICHD22_WORD1 CE_EIM_0.EICHD22_WORD1.R    /* Error Injection Channel Descriptor 22, Word1 */
#define CE_EIM_0_EICHD23_WORD1 CE_EIM_0.EICHD23_WORD1.R    /* Error Injection Channel Descriptor 23, Word1 */
#define CE_EIM_0_EICHD24_WORD1 CE_EIM_0.EICHD24_WORD1.R    /* Error Injection Channel Descriptor 24, Word1 */
#define CE_EIM_0_EICHD26_WORD0 CE_EIM_0.EICHD26_WORD0.R    /* Error Injection Channel Descriptor 26, Word0 */
#define CE_EIM_0_EICHD26_WORD1 CE_EIM_0.EICHD26_WORD1.R    /* Error Injection Channel Descriptor 26, Word1 */
#define CE_EIM_0_EICHD26_WORD2 CE_EIM_0.EICHD26_WORD2.R    /* Error Injection Channel Descriptor 26, Word2 */
#define CE_EIM_0_EICHD27_WORD1 CE_EIM_0.EICHD27_WORD1.R    /* Error Injection Channel Descriptor 27, Word1 */

#define CE_EIM_1_EIMCR       CE_EIM_1.EIMCR.R              /* Error Injection Module Configuration Register */
#define CE_EIM_1_EICHEN      CE_EIM_1.EICHEN.R             /* Error Injection Channel Enable register */
#define CE_EIM_1_EICHD7_WORD0 CE_EIM_1.EICHD7_WORD0.R      /* Error Injection Channel Descriptor 7, Word0 */
#define CE_EIM_1_EICHD7_WORD1 CE_EIM_1.EICHD7_WORD1.R      /* Error Injection Channel Descriptor 7, Word1 */
#define CE_EIM_1_EICHD7_WORD2 CE_EIM_1.EICHD7_WORD2.R      /* Error Injection Channel Descriptor 7, Word2 */
#define CE_EIM_1_EICHD8_WORD0 CE_EIM_1.EICHD8_WORD0.R      /* Error Injection Channel Descriptor 8, Word0 */
#define CE_EIM_1_EICHD8_WORD1 CE_EIM_1.EICHD8_WORD1.R      /* Error Injection Channel Descriptor 8, Word1 */
#define CE_EIM_1_EICHD8_WORD2 CE_EIM_1.EICHD8_WORD2.R      /* Error Injection Channel Descriptor 8, Word2 */
#define CE_EIM_1_EICHD8_WORD3 CE_EIM_1.EICHD8_WORD3.R      /* Error Injection Channel Descriptor 8, Word3 */
#define CE_EIM_1_EICHD8_WORD4 CE_EIM_1.EICHD8_WORD4.R      /* Error Injection Channel Descriptor 8, Word4 */
#define CE_EIM_1_EICHD9_WORD0 CE_EIM_1.EICHD9_WORD0.R      /* Error Injection Channel Descriptor 9, Word0 */
#define CE_EIM_1_EICHD9_WORD1 CE_EIM_1.EICHD9_WORD1.R      /* Error Injection Channel Descriptor 9, Word1 */
#define CE_EIM_1_EICHD9_WORD2 CE_EIM_1.EICHD9_WORD2.R      /* Error Injection Channel Descriptor 9, Word2 */
#define CE_EIM_1_EICHD10_WORD0 CE_EIM_1.EICHD10_WORD0.R    /* Error Injection Channel Descriptor 10, Word0 */
#define CE_EIM_1_EICHD10_WORD1 CE_EIM_1.EICHD10_WORD1.R    /* Error Injection Channel Descriptor 10, Word1 */
#define CE_EIM_1_EICHD10_WORD2 CE_EIM_1.EICHD10_WORD2.R    /* Error Injection Channel Descriptor 10, Word2 */
#define CE_EIM_1_EICHD10_WORD3 CE_EIM_1.EICHD10_WORD3.R    /* Error Injection Channel Descriptor 10, Word3 */
#define CE_EIM_1_EICHD10_WORD4 CE_EIM_1.EICHD10_WORD4.R    /* Error Injection Channel Descriptor 10, Word4 */
#define CE_EIM_1_EICHD11_WORD0 CE_EIM_1.EICHD11_WORD0.R    /* Error Injection Channel Descriptor 11, Word0 */
#define CE_EIM_1_EICHD11_WORD1 CE_EIM_1.EICHD11_WORD1.R    /* Error Injection Channel Descriptor 11, Word1 */
#define CE_EIM_1_EICHD11_WORD2 CE_EIM_1.EICHD11_WORD2.R    /* Error Injection Channel Descriptor 11, Word2 */
#define CE_EIM_1_EICHD12_WORD0 CE_EIM_1.EICHD12_WORD0.R    /* Error Injection Channel Descriptor 12, Word0 */
#define CE_EIM_1_EICHD12_WORD1 CE_EIM_1.EICHD12_WORD1.R    /* Error Injection Channel Descriptor 12, Word1 */
#define CE_EIM_1_EICHD12_WORD2 CE_EIM_1.EICHD12_WORD2.R    /* Error Injection Channel Descriptor 12, Word2 */
#define CE_EIM_1_EICHD16_WORD0 CE_EIM_1.EICHD16_WORD0.R    /* Error Injection Channel Descriptor 16, Word0 */
#define CE_EIM_1_EICHD16_WORD1 CE_EIM_1.EICHD16_WORD1.R    /* Error Injection Channel Descriptor 16, Word1 */
#define CE_EIM_1_EICHD16_WORD2 CE_EIM_1.EICHD16_WORD2.R    /* Error Injection Channel Descriptor 16, Word2 */
#define CE_EIM_1_EICHD18_WORD1 CE_EIM_1.EICHD18_WORD1.R    /* Error Injection Channel Descriptor 18, Word1 */
#define CE_EIM_1_EICHD18_WORD2 CE_EIM_1.EICHD18_WORD2.R    /* Error Injection Channel Descriptor 18, Word2 */
#define CE_EIM_1_EICHD19_WORD1 CE_EIM_1.EICHD19_WORD1.R    /* Error Injection Channel Descriptor 19, Word1 */
#define CE_EIM_1_EICHD19_WORD2 CE_EIM_1.EICHD19_WORD2.R    /* Error Injection Channel Descriptor 19, Word2 */
#define CE_EIM_1_EICHD24_WORD1 CE_EIM_1.EICHD24_WORD1.R    /* Error Injection Channel Descriptor 24, Word1 */

#define EIM_3_EIMCR          EIM_3.EIMCR.R                 /* Error Injection Module Configuration Register */
#define EIM_3_EICHEN         EIM_3.EICHEN.R                /* Error Injection Channel Enable register */
#define EIM_3_EICHD0_WORD0   EIM_3.EICHD0_WORD0.R          /* Error Injection Channel Descriptor 0, Word0 */
#define EIM_3_EICHD0_WORD1   EIM_3.EICHD0_WORD1.R          /* Error Injection Channel Descriptor 0, Word1 */
#define EIM_3_EICHD0_WORD2   EIM_3.EICHD0_WORD2.R          /* Error Injection Channel Descriptor 0, Word2 */
#define EIM_3_EICHD1_WORD1   EIM_3.EICHD1_WORD1.R          /* Error Injection Channel Descriptor 1, Word1 */
#define EIM_3_EICHD1_WORD2   EIM_3.EICHD1_WORD2.R          /* Error Injection Channel Descriptor 1, Word2 */
#define EIM_3_EICHD1_WORD3   EIM_3.EICHD1_WORD3.R          /* Error Injection Channel Descriptor 1, Word3 */
#define EIM_3_EICHD1_WORD4   EIM_3.EICHD1_WORD4.R          /* Error Injection Channel Descriptor 1, Word4 */
#define EIM_3_EICHD1_WORD5   EIM_3.EICHD1_WORD5.R          /* Error Injection Channel Descriptor 1, Word5 */
#define EIM_3_EICHD1_WORD6   EIM_3.EICHD1_WORD6.R          /* Error Injection Channel Descriptor 1, Word6 */
#define EIM_3_EICHD1_WORD7   EIM_3.EICHD1_WORD7.R          /* Error Injection Channel Descriptor 1, Word7 */
#define EIM_3_EICHD1_WORD8   EIM_3.EICHD1_WORD8.R          /* Error Injection Channel Descriptor 1, Word8 */
#define EIM_3_EICHD2_WORD0   EIM_3.EICHD2_WORD0.R          /* Error Injection Channel Descriptor 2, Word0 */
#define EIM_3_EICHD2_WORD1   EIM_3.EICHD2_WORD1.R          /* Error Injection Channel Descriptor 2, Word1 */
#define EIM_3_EICHD2_WORD2   EIM_3.EICHD2_WORD2.R          /* Error Injection Channel Descriptor 2, Word2 */
#define EIM_3_EICHD3_WORD0   EIM_3.EICHD3_WORD0.R          /* Error Injection Channel Descriptor 3, Word0 */
#define EIM_3_EICHD3_WORD1   EIM_3.EICHD3_WORD1.R          /* Error Injection Channel Descriptor 3, Word1 */
#define EIM_3_EICHD3_WORD2   EIM_3.EICHD3_WORD2.R          /* Error Injection Channel Descriptor 3, Word2 */
#define EIM_3_EICHD4_WORD0   EIM_3.EICHD4_WORD0.R          /* Error Injection Channel Descriptor 4, Word0 */
#define EIM_3_EICHD4_WORD1   EIM_3.EICHD4_WORD1.R          /* Error Injection Channel Descriptor 4, Word1 */
#define EIM_3_EICHD4_WORD2   EIM_3.EICHD4_WORD2.R          /* Error Injection Channel Descriptor 4, Word2 */
#define EIM_3_EICHD5_WORD0   EIM_3.EICHD5_WORD0.R          /* Error Injection Channel Descriptor 5, Word0 */
#define EIM_3_EICHD5_WORD1   EIM_3.EICHD5_WORD1.R          /* Error Injection Channel Descriptor 5, Word1 */
#define EIM_3_EICHD5_WORD2   EIM_3.EICHD5_WORD2.R          /* Error Injection Channel Descriptor 5, Word2 */
#define EIM_3_EICHD6_WORD0   EIM_3.EICHD6_WORD0.R          /* Error Injection Channel Descriptor 6, Word0 */
#define EIM_3_EICHD6_WORD1   EIM_3.EICHD6_WORD1.R          /* Error Injection Channel Descriptor 6, Word1 */
#define EIM_3_EICHD6_WORD2   EIM_3.EICHD6_WORD2.R          /* Error Injection Channel Descriptor 6, Word2 */
#define EIM_3_EICHD7_WORD0   EIM_3.EICHD7_WORD0.R          /* Error Injection Channel Descriptor 7, Word0 */
#define EIM_3_EICHD7_WORD1   EIM_3.EICHD7_WORD1.R          /* Error Injection Channel Descriptor 7, Word1 */
#define EIM_3_EICHD7_WORD2   EIM_3.EICHD7_WORD2.R          /* Error Injection Channel Descriptor 7, Word2 */
#define EIM_3_EICHD8_WORD1   EIM_3.EICHD8_WORD1.R          /* Error Injection Channel Descriptor 8, Word1 */
#define EIM_3_EICHD8_WORD2   EIM_3.EICHD8_WORD2.R          /* Error Injection Channel Descriptor 8, Word2 */
#define EIM_3_EICHD8_WORD3   EIM_3.EICHD8_WORD3.R          /* Error Injection Channel Descriptor 8, Word3 */
#define EIM_3_EICHD8_WORD4   EIM_3.EICHD8_WORD4.R          /* Error Injection Channel Descriptor 8, Word4 */
#define EIM_3_EICHD8_WORD5   EIM_3.EICHD8_WORD5.R          /* Error Injection Channel Descriptor 8, Word5 */
#define EIM_3_EICHD8_WORD6   EIM_3.EICHD8_WORD6.R          /* Error Injection Channel Descriptor 8, Word6 */
#define EIM_3_EICHD8_WORD7   EIM_3.EICHD8_WORD7.R          /* Error Injection Channel Descriptor 8, Word7 */
#define EIM_3_EICHD8_WORD8   EIM_3.EICHD8_WORD8.R          /* Error Injection Channel Descriptor 8, Word8 */
#define EIM_3_EICHD9_WORD1   EIM_3.EICHD9_WORD1.R          /* Error Injection Channel Descriptor 9, Word1 */
#define EIM_3_EICHD9_WORD2   EIM_3.EICHD9_WORD2.R          /* Error Injection Channel Descriptor 9, Word2 */
#define EIM_3_EICHD9_WORD3   EIM_3.EICHD9_WORD3.R          /* Error Injection Channel Descriptor 9, Word3 */
#define EIM_3_EICHD9_WORD4   EIM_3.EICHD9_WORD4.R          /* Error Injection Channel Descriptor 9, Word4 */
#define EIM_3_EICHD9_WORD5   EIM_3.EICHD9_WORD5.R          /* Error Injection Channel Descriptor 9, Word5 */
#define EIM_3_EICHD9_WORD6   EIM_3.EICHD9_WORD6.R          /* Error Injection Channel Descriptor 9, Word6 */
#define EIM_3_EICHD9_WORD7   EIM_3.EICHD9_WORD7.R          /* Error Injection Channel Descriptor 9, Word7 */
#define EIM_3_EICHD9_WORD8   EIM_3.EICHD9_WORD8.R          /* Error Injection Channel Descriptor 9, Word8 */
#define EIM_3_EICHD10_WORD1  EIM_3.EICHD10_WORD1.R         /* Error Injection Channel Descriptor 10, Word1 */
#define EIM_3_EICHD10_WORD2  EIM_3.EICHD10_WORD2.R         /* Error Injection Channel Descriptor 10, Word2 */
#define EIM_3_EICHD10_WORD3  EIM_3.EICHD10_WORD3.R         /* Error Injection Channel Descriptor 10, Word3 */
#define EIM_3_EICHD11_WORD1  EIM_3.EICHD11_WORD1.R         /* Error Injection Channel Descriptor 11, Word1 */
#define EIM_3_EICHD12_WORD1  EIM_3.EICHD12_WORD1.R         /* Error Injection Channel Descriptor 12, Word1 */
#define EIM_3_EICHD13_WORD0  EIM_3.EICHD13_WORD0.R         /* Error Injection Channel Descriptor 13, Word0 */
#define EIM_3_EICHD13_WORD1  EIM_3.EICHD13_WORD1.R         /* Error Injection Channel Descriptor 13, Word1 */
#define EIM_3_EICHD13_WORD2  EIM_3.EICHD13_WORD2.R         /* Error Injection Channel Descriptor 13, Word2 */
#define EIM_3_EICHD14_WORD0  EIM_3.EICHD14_WORD0.R         /* Error Injection Channel Descriptor 14, Word0 */
#define EIM_3_EICHD14_WORD1  EIM_3.EICHD14_WORD1.R         /* Error Injection Channel Descriptor 14, Word1 */
#define EIM_3_EICHD14_WORD2  EIM_3.EICHD14_WORD2.R         /* Error Injection Channel Descriptor 14, Word2 */
#define EIM_3_EICHD15_WORD0  EIM_3.EICHD15_WORD0.R         /* Error Injection Channel Descriptor 15, Word0 */
#define EIM_3_EICHD15_WORD1  EIM_3.EICHD15_WORD1.R         /* Error Injection Channel Descriptor 15, Word1 */
#define EIM_3_EICHD15_WORD2  EIM_3.EICHD15_WORD2.R         /* Error Injection Channel Descriptor 15, Word2 */
#define EIM_3_EICHD16_WORD1  EIM_3.EICHD16_WORD1.R         /* Error Injection Channel Descriptor 16, Word1 */
#define EIM_3_EICHD17_WORD1  EIM_3.EICHD17_WORD1.R         /* Error Injection Channel Descriptor 17, Word1 */
#define EIM_3_EICHD18_WORD1  EIM_3.EICHD18_WORD1.R         /* Error Injection Channel Descriptor 18, Word1 */
#define EIM_3_EICHD19_WORD0  EIM_3.EICHD19_WORD0.R         /* Error Injection Channel Descriptor 19, Word0 */
#define EIM_3_EICHD19_WORD1  EIM_3.EICHD19_WORD1.R         /* Error Injection Channel Descriptor 19, Word1 */
#define EIM_3_EICHD19_WORD2  EIM_3.EICHD19_WORD2.R         /* Error Injection Channel Descriptor 19, Word2 */
#define EIM_3_EICHD20_WORD0  EIM_3.EICHD20_WORD0.R         /* Error Injection Channel Descriptor 20, Word0 */
#define EIM_3_EICHD20_WORD1  EIM_3.EICHD20_WORD1.R         /* Error Injection Channel Descriptor 20, Word1 */
#define EIM_3_EICHD20_WORD2  EIM_3.EICHD20_WORD2.R         /* Error Injection Channel Descriptor 20, Word2 */
#define EIM_3_EICHD21_WORD0  EIM_3.EICHD21_WORD0.R         /* Error Injection Channel Descriptor 21, Word0 */
#define EIM_3_EICHD21_WORD1  EIM_3.EICHD21_WORD1.R         /* Error Injection Channel Descriptor 21, Word1 */
#define EIM_3_EICHD21_WORD2  EIM_3.EICHD21_WORD2.R         /* Error Injection Channel Descriptor 21, Word2 */
#define EIM_3_EICHD22_WORD0  EIM_3.EICHD22_WORD0.R         /* Error Injection Channel Descriptor 22, Word0 */
#define EIM_3_EICHD22_WORD1  EIM_3.EICHD22_WORD1.R         /* Error Injection Channel Descriptor 22, Word1 */
#define EIM_3_EICHD22_WORD2  EIM_3.EICHD22_WORD2.R         /* Error Injection Channel Descriptor 22, Word2 */
#define EIM_3_EICHD23_WORD1  EIM_3.EICHD23_WORD1.R         /* Error Injection Channel Descriptor 23, Word1 */
#define EIM_3_EICHD24_WORD1  EIM_3.EICHD24_WORD1.R         /* Error Injection Channel Descriptor 24, Word1 */
#define EIM_3_EICHD25_WORD1  EIM_3.EICHD25_WORD1.R         /* Error Injection Channel Descriptor 25, Word1 */
#define EIM_3_EICHD26_WORD1  EIM_3.EICHD26_WORD1.R         /* Error Injection Channel Descriptor 26, Word1 */
#define EIM_3_EICHD27_WORD0  EIM_3.EICHD27_WORD0.R         /* Error Injection Channel Descriptor 27, Word0 */
#define EIM_3_EICHD27_WORD1  EIM_3.EICHD27_WORD1.R         /* Error Injection Channel Descriptor 27, Word1 */
#define EIM_3_EICHD27_WORD2  EIM_3.EICHD27_WORD2.R         /* Error Injection Channel Descriptor 27, Word2 */
#define EIM_3_EICHD28_WORD0  EIM_3.EICHD28_WORD0.R         /* Error Injection Channel Descriptor 28, Word0 */
#define EIM_3_EICHD28_WORD1  EIM_3.EICHD28_WORD1.R         /* Error Injection Channel Descriptor 28, Word1 */
#define EIM_3_EICHD28_WORD2  EIM_3.EICHD28_WORD2.R         /* Error Injection Channel Descriptor 28, Word2 */
#define EIM_3_EICHD29_WORD0  EIM_3.EICHD29_WORD0.R         /* Error Injection Channel Descriptor 29, Word0 */
#define EIM_3_EICHD29_WORD1  EIM_3.EICHD29_WORD1.R         /* Error Injection Channel Descriptor 29, Word1 */
#define EIM_3_EICHD29_WORD2  EIM_3.EICHD29_WORD2.R         /* Error Injection Channel Descriptor 29, Word2 */
#define EIM_3_EICHD30_WORD0  EIM_3.EICHD30_WORD0.R         /* Error Injection Channel Descriptor 30, Word0 */
#define EIM_3_EICHD30_WORD1  EIM_3.EICHD30_WORD1.R         /* Error Injection Channel Descriptor 30, Word1 */
#define EIM_3_EICHD30_WORD2  EIM_3.EICHD30_WORD2.R         /* Error Injection Channel Descriptor 30, Word2 */
#define EIM_3_EICHD31_WORD1  EIM_3.EICHD31_WORD1.R         /* Error Injection Channel Descriptor 31, Word1 */

#define EIM_1_EIMCR          EIM_1.EIMCR.R                 /* Error Injection Module Configuration Register */
#define EIM_1_EICHEN         EIM_1.EICHEN.R                /* Error Injection Channel Enable register */
#define EIM_1_EICHD0_WORD1   EIM_1.EICHD0_WORD1.R          /* Error Injection Channel Descriptor 0, Word1 */
#define EIM_1_EICHD0_WORD2   EIM_1.EICHD0_WORD2.R          /* Error Injection Channel Descriptor 0, Word2 */

#define SMU__EIM_EIMCR       SMU__EIM.EIMCR.R              /* Error Injection Module Configuration Register */
#define SMU__EIM_EICHEN      SMU__EIM.EICHEN.R             /* Error Injection Channel Enable register */
#define SMU__EIM_EICHD0_WORD0 SMU__EIM.EICHD0_WORD0.R      /* Error Injection Channel Descriptor 0, Word0 */
#define SMU__EIM_EICHD0_WORD1 SMU__EIM.EICHD0_WORD1.R      /* Error Injection Channel Descriptor 0, Word1 */
#define SMU__EIM_EICHD0_WORD2 SMU__EIM.EICHD0_WORD2.R      /* Error Injection Channel Descriptor 0, Word2 */
#define SMU__EIM_EICHD1_WORD0 SMU__EIM.EICHD1_WORD0.R      /* Error Injection Channel Descriptor 1, Word0 */
#define SMU__EIM_EICHD1_WORD1 SMU__EIM.EICHD1_WORD1.R      /* Error Injection Channel Descriptor 1, Word1 */
#define SMU__EIM_EICHD1_WORD2 SMU__EIM.EICHD1_WORD2.R      /* Error Injection Channel Descriptor 1, Word2 */
#define SMU__EIM_EICHD2_WORD0 SMU__EIM.EICHD2_WORD0.R      /* Error Injection Channel Descriptor 2, Word0 */
#define SMU__EIM_EICHD2_WORD1 SMU__EIM.EICHD2_WORD1.R      /* Error Injection Channel Descriptor 2, Word1 */
#define SMU__EIM_EICHD2_WORD2 SMU__EIM.EICHD2_WORD2.R      /* Error Injection Channel Descriptor 2, Word2 */
#define SMU__EIM_EICHD3_WORD0 SMU__EIM.EICHD3_WORD0.R      /* Error Injection Channel Descriptor 3, Word0 */
#define SMU__EIM_EICHD3_WORD1 SMU__EIM.EICHD3_WORD1.R      /* Error Injection Channel Descriptor 3, Word1 */
#define SMU__EIM_EICHD3_WORD2 SMU__EIM.EICHD3_WORD2.R      /* Error Injection Channel Descriptor 3, Word2 */
#define SMU__EIM_EICHD7_WORD0 SMU__EIM.EICHD7_WORD0.R      /* Error Injection Channel Descriptor 7, Word0 */
#define SMU__EIM_EICHD7_WORD1 SMU__EIM.EICHD7_WORD1.R      /* Error Injection Channel Descriptor 7, Word1 */
#define SMU__EIM_EICHD7_WORD2 SMU__EIM.EICHD7_WORD2.R      /* Error Injection Channel Descriptor 7, Word2 */
#define SMU__EIM_EICHD8_WORD0 SMU__EIM.EICHD8_WORD0.R      /* Error Injection Channel Descriptor 8, Word0 */
#define SMU__EIM_EICHD8_WORD1 SMU__EIM.EICHD8_WORD1.R      /* Error Injection Channel Descriptor 8, Word1 */
#define SMU__EIM_EICHD8_WORD2 SMU__EIM.EICHD8_WORD2.R      /* Error Injection Channel Descriptor 8, Word2 */
#define SMU__EIM_EICHD8_WORD3 SMU__EIM.EICHD8_WORD3.R      /* Error Injection Channel Descriptor 8, Word3 */
#define SMU__EIM_EICHD8_WORD4 SMU__EIM.EICHD8_WORD4.R      /* Error Injection Channel Descriptor 8, Word4 */
#define SMU__EIM_EICHD9_WORD0 SMU__EIM.EICHD9_WORD0.R      /* Error Injection Channel Descriptor 9, Word0 */
#define SMU__EIM_EICHD9_WORD1 SMU__EIM.EICHD9_WORD1.R      /* Error Injection Channel Descriptor 9, Word1 */
#define SMU__EIM_EICHD9_WORD2 SMU__EIM.EICHD9_WORD2.R      /* Error Injection Channel Descriptor 9, Word2 */
#define SMU__EIM_EICHD10_WORD0 SMU__EIM.EICHD10_WORD0.R    /* Error Injection Channel Descriptor 10, Word0 */
#define SMU__EIM_EICHD10_WORD1 SMU__EIM.EICHD10_WORD1.R    /* Error Injection Channel Descriptor 10, Word1 */
#define SMU__EIM_EICHD10_WORD2 SMU__EIM.EICHD10_WORD2.R    /* Error Injection Channel Descriptor 10, Word2 */
#define SMU__EIM_EICHD10_WORD3 SMU__EIM.EICHD10_WORD3.R    /* Error Injection Channel Descriptor 10, Word3 */
#define SMU__EIM_EICHD10_WORD4 SMU__EIM.EICHD10_WORD4.R    /* Error Injection Channel Descriptor 10, Word4 */
#define SMU__EIM_EICHD11_WORD0 SMU__EIM.EICHD11_WORD0.R    /* Error Injection Channel Descriptor 11, Word0 */
#define SMU__EIM_EICHD11_WORD1 SMU__EIM.EICHD11_WORD1.R    /* Error Injection Channel Descriptor 11, Word1 */
#define SMU__EIM_EICHD11_WORD2 SMU__EIM.EICHD11_WORD2.R    /* Error Injection Channel Descriptor 11, Word2 */
#define SMU__EIM_EICHD12_WORD0 SMU__EIM.EICHD12_WORD0.R    /* Error Injection Channel Descriptor 12, Word0 */
#define SMU__EIM_EICHD12_WORD1 SMU__EIM.EICHD12_WORD1.R    /* Error Injection Channel Descriptor 12, Word1 */
#define SMU__EIM_EICHD12_WORD2 SMU__EIM.EICHD12_WORD2.R    /* Error Injection Channel Descriptor 12, Word2 */
#define SMU__EIM_EICHD13_WORD0 SMU__EIM.EICHD13_WORD0.R    /* Error Injection Channel Descriptor 13, Word0 */
#define SMU__EIM_EICHD13_WORD1 SMU__EIM.EICHD13_WORD1.R    /* Error Injection Channel Descriptor 13, Word1 */
#define SMU__EIM_EICHD13_WORD2 SMU__EIM.EICHD13_WORD2.R    /* Error Injection Channel Descriptor 13, Word2 */
#define SMU__EIM_EICHD14_WORD0 SMU__EIM.EICHD14_WORD0.R    /* Error Injection Channel Descriptor 14, Word0 */
#define SMU__EIM_EICHD14_WORD1 SMU__EIM.EICHD14_WORD1.R    /* Error Injection Channel Descriptor 14, Word1 */
#define SMU__EIM_EICHD14_WORD2 SMU__EIM.EICHD14_WORD2.R    /* Error Injection Channel Descriptor 14, Word2 */
#define SMU__EIM_EICHD15_WORD0 SMU__EIM.EICHD15_WORD0.R    /* Error Injection Channel Descriptor 15, Word0 */
#define SMU__EIM_EICHD15_WORD1 SMU__EIM.EICHD15_WORD1.R    /* Error Injection Channel Descriptor 15, Word1 */
#define SMU__EIM_EICHD15_WORD2 SMU__EIM.EICHD15_WORD2.R    /* Error Injection Channel Descriptor 15, Word2 */
#define SMU__EIM_EICHD16_WORD0 SMU__EIM.EICHD16_WORD0.R    /* Error Injection Channel Descriptor 16, Word0 */
#define SMU__EIM_EICHD16_WORD1 SMU__EIM.EICHD16_WORD1.R    /* Error Injection Channel Descriptor 16, Word1 */
#define SMU__EIM_EICHD16_WORD2 SMU__EIM.EICHD16_WORD2.R    /* Error Injection Channel Descriptor 16, Word2 */
#define SMU__EIM_EICHD17_WORD0 SMU__EIM.EICHD17_WORD0.R    /* Error Injection Channel Descriptor 17, Word0 */
#define SMU__EIM_EICHD17_WORD1 SMU__EIM.EICHD17_WORD1.R    /* Error Injection Channel Descriptor 17, Word1 */
#define SMU__EIM_EICHD17_WORD2 SMU__EIM.EICHD17_WORD2.R    /* Error Injection Channel Descriptor 17, Word2 */
#define SMU__EIM_EICHD18_WORD1 SMU__EIM.EICHD18_WORD1.R    /* Error Injection Channel Descriptor 18, Word1 */
#define SMU__EIM_EICHD18_WORD2 SMU__EIM.EICHD18_WORD2.R    /* Error Injection Channel Descriptor 18, Word2 */
#define SMU__EIM_EICHD19_WORD1 SMU__EIM.EICHD19_WORD1.R    /* Error Injection Channel Descriptor 19, Word1 */
#define SMU__EIM_EICHD19_WORD2 SMU__EIM.EICHD19_WORD2.R    /* Error Injection Channel Descriptor 19, Word2 */
#define SMU__EIM_EICHD20_WORD1 SMU__EIM.EICHD20_WORD1.R    /* Error Injection Channel Descriptor 20, Word1 */
#define SMU__EIM_EICHD20_WORD2 SMU__EIM.EICHD20_WORD2.R    /* Error Injection Channel Descriptor 20, Word2 */
#define SMU__EIM_EICHD21_WORD1 SMU__EIM.EICHD21_WORD1.R    /* Error Injection Channel Descriptor 21, Word1 */
#define SMU__EIM_EICHD21_WORD2 SMU__EIM.EICHD21_WORD2.R    /* Error Injection Channel Descriptor 21, Word2 */
#define SMU__EIM_EICHD22_WORD1 SMU__EIM.EICHD22_WORD1.R    /* Error Injection Channel Descriptor 22, Word1 */
#define SMU__EIM_EICHD23_WORD1 SMU__EIM.EICHD23_WORD1.R    /* Error Injection Channel Descriptor 23, Word1 */
#define SMU__EIM_EICHD24_WORD1 SMU__EIM.EICHD24_WORD1.R    /* Error Injection Channel Descriptor 24, Word1 */
#define SMU__EIM_EICHD26_WORD0 SMU__EIM.EICHD26_WORD0.R    /* Error Injection Channel Descriptor 26, Word0 */
#define SMU__EIM_EICHD26_WORD1 SMU__EIM.EICHD26_WORD1.R    /* Error Injection Channel Descriptor 26, Word1 */
#define SMU__EIM_EICHD26_WORD2 SMU__EIM.EICHD26_WORD2.R    /* Error Injection Channel Descriptor 26, Word2 */
#define SMU__EIM_EICHD27_WORD1 SMU__EIM.EICHD27_WORD1.R    /* Error Injection Channel Descriptor 27, Word1 */

#define EIM_2_EIMCR          EIM_2.EIMCR.R                 /* Error Injection Module Configuration Register */
#define EIM_2_EICHEN         EIM_2.EICHEN.R                /* Error Injection Channel Enable register */
#define EIM_2_EICHD0_WORD1   EIM_2.EICHD0_WORD1.R          /* Error Injection Channel Descriptor 0, Word1 */
#define EIM_2_EICHD0_WORD2   EIM_2.EICHD0_WORD2.R          /* Error Injection Channel Descriptor 0, Word2 */
#define EIM_2_EICHD0_WORD3   EIM_2.EICHD0_WORD3.R          /* Error Injection Channel Descriptor 0, Word3 */
#define EIM_2_EICHD1_WORD1   EIM_2.EICHD1_WORD1.R          /* Error Injection Channel Descriptor 1, Word1 */
#define EIM_2_EICHD2_WORD1   EIM_2.EICHD2_WORD1.R          /* Error Injection Channel Descriptor 2, Word1 */
#define EIM_2_EICHD3_WORD1   EIM_2.EICHD3_WORD1.R          /* Error Injection Channel Descriptor 3, Word1 */

/* EIM_GTM */
#define EIM_GTM_EIMCR        EIM_GTM.EIMCR.R               /* Error Injection Module Configuration Register */
#define EIM_GTM_EICHEN       EIM_GTM.EICHEN.R              /* Error Injection Channel Enable register */
#define EIM_GTM_EICHD0_WORD0 EIM_GTM.EICHD0_WORD0.R        /* Error Injection Channel Descriptor 0, Word0 */
#define EIM_GTM_EICHD0_WORD1 EIM_GTM.EICHD0_WORD1.R        /* Error Injection Channel Descriptor 0, Word1 */
#define EIM_GTM_EICHD1_WORD0 EIM_GTM.EICHD1_WORD0.R        /* Error Injection Channel Descriptor 1, Word0 */
#define EIM_GTM_EICHD1_WORD1 EIM_GTM.EICHD1_WORD1.R        /* Error Injection Channel Descriptor 1, Word1 */
#define EIM_GTM_EICHD2_WORD0 EIM_GTM.EICHD2_WORD0.R        /* Error Injection Channel Descriptor 2, Word0 */
#define EIM_GTM_EICHD2_WORD1 EIM_GTM.EICHD2_WORD1.R        /* Error Injection Channel Descriptor 2, Word1 */
#define EIM_GTM_EICHD3_WORD0 EIM_GTM.EICHD3_WORD0.R        /* Error Injection Channel Descriptor 3, Word0 */
#define EIM_GTM_EICHD3_WORD1 EIM_GTM.EICHD3_WORD1.R        /* Error Injection Channel Descriptor 3, Word1 */
#define EIM_GTM_EICHD4_WORD0 EIM_GTM.EICHD4_WORD0.R        /* Error Injection Channel Descriptor 4, Word0 */
#define EIM_GTM_EICHD4_WORD1 EIM_GTM.EICHD4_WORD1.R        /* Error Injection Channel Descriptor 4, Word1 */
#define EIM_GTM_EICHD5_WORD0 EIM_GTM.EICHD5_WORD0.R        /* Error Injection Channel Descriptor 5, Word0 */
#define EIM_GTM_EICHD5_WORD1 EIM_GTM.EICHD5_WORD1.R        /* Error Injection Channel Descriptor 5, Word1 */
#define EIM_GTM_EICHD6_WORD0 EIM_GTM.EICHD6_WORD0.R        /* Error Injection Channel Descriptor 6, Word0 */
#define EIM_GTM_EICHD6_WORD1 EIM_GTM.EICHD6_WORD1.R        /* Error Injection Channel Descriptor 6, Word1 */
#define EIM_GTM_EICHD7_WORD0 EIM_GTM.EICHD7_WORD0.R        /* Error Injection Channel Descriptor 7, Word0 */
#define EIM_GTM_EICHD7_WORD1 EIM_GTM.EICHD7_WORD1.R        /* Error Injection Channel Descriptor 7, Word1 */
#define EIM_GTM_EICHD8_WORD0 EIM_GTM.EICHD8_WORD0.R        /* Error Injection Channel Descriptor 8, Word0 */
#define EIM_GTM_EICHD8_WORD1 EIM_GTM.EICHD8_WORD1.R        /* Error Injection Channel Descriptor 8, Word1 */
#define EIM_GTM_EICHD9_WORD0 EIM_GTM.EICHD9_WORD0.R        /* Error Injection Channel Descriptor 9, Word0 */
#define EIM_GTM_EICHD9_WORD1 EIM_GTM.EICHD9_WORD1.R        /* Error Injection Channel Descriptor 9, Word1 */
#define EIM_GTM_EICHD10_WORD0 EIM_GTM.EICHD10_WORD0.R      /* Error Injection Channel Descriptor 10, Word0 */
#define EIM_GTM_EICHD10_WORD1 EIM_GTM.EICHD10_WORD1.R      /* Error Injection Channel Descriptor 10, Word1 */
#define EIM_GTM_EICHD11_WORD0 EIM_GTM.EICHD11_WORD0.R      /* Error Injection Channel Descriptor 11, Word0 */
#define EIM_GTM_EICHD11_WORD1 EIM_GTM.EICHD11_WORD1.R      /* Error Injection Channel Descriptor 11, Word1 */

/* EMIOS */
#define EMIOS_1_MCR          EMIOS_1.MCR.R                 /* Module Configuration */
#define EMIOS_1_GFLAG        EMIOS_1.GFLAG.R               /* Global Flag */
#define EMIOS_1_OUDIS        EMIOS_1.OUDIS.R               /* Output Update Disable */
#define EMIOS_1_A0           EMIOS_1.CH.UC[0].A.R          /* UC A 0 */
#define EMIOS_1_B0           EMIOS_1.CH.UC[0].B.R          /* UC B 0 */
#define EMIOS_1_CNT0         EMIOS_1.CH.UC[0].CNT.R        /* UC Counter 0 */
#define EMIOS_1_C0           EMIOS_1.CH.UC[0].C.R          /* UC Control 0 */
#define EMIOS_1_S0           EMIOS_1.CH.UC[0].S.R          /* UC Status 0 */
#define EMIOS_1_ALTA0        EMIOS_1.CH.UC[0].ALTA.R       /* Alternate Address 0 */
#define EMIOS_1_C2_0         EMIOS_1.CH.UC[0].C2.R         /* UC Control 2 0 */
#define EMIOS_1_A1           EMIOS_1.CH.UC[1].A.R          /* UC A 1 */
#define EMIOS_1_B1           EMIOS_1.CH.UC[1].B.R          /* UC B 1 */
#define EMIOS_1_CNT1         EMIOS_1.CH.UC[1].CNT.R        /* UC Counter 1 */
#define EMIOS_1_C1           EMIOS_1.CH.UC[1].C.R          /* UC Control 1 */
#define EMIOS_1_S1           EMIOS_1.CH.UC[1].S.R          /* UC Status 1 */
#define EMIOS_1_ALTA1        EMIOS_1.CH.UC[1].ALTA.R       /* Alternate Address 1 */
#define EMIOS_1_C2_1         EMIOS_1.CH.UC[1].C2.R         /* UC Control 2 1 */
#define EMIOS_1_A2           EMIOS_1.CH.UC[2].A.R          /* UC A 2 */
#define EMIOS_1_B2           EMIOS_1.CH.UC[2].B.R          /* UC B 2 */
#define EMIOS_1_CNT2         EMIOS_1.CH.UC[2].CNT.R        /* UC Counter 2 */
#define EMIOS_1_C2           EMIOS_1.CH.UC[2].C.R          /* UC Control 2 */
#define EMIOS_1_S2           EMIOS_1.CH.UC[2].S.R          /* UC Status 2 */
#define EMIOS_1_ALTA2        EMIOS_1.CH.UC[2].ALTA.R       /* Alternate Address 2 */
#define EMIOS_1_C2_2         EMIOS_1.CH.UC[2].C2.R         /* UC Control 2 2 */
#define EMIOS_1_A3           EMIOS_1.CH.UC[3].A.R          /* UC A 3 */
#define EMIOS_1_B3           EMIOS_1.CH.UC[3].B.R          /* UC B 3 */
#define EMIOS_1_CNT3         EMIOS_1.CH.UC[3].CNT.R        /* UC Counter 3 */
#define EMIOS_1_C3           EMIOS_1.CH.UC[3].C.R          /* UC Control 3 */
#define EMIOS_1_S3           EMIOS_1.CH.UC[3].S.R          /* UC Status 3 */
#define EMIOS_1_ALTA3        EMIOS_1.CH.UC[3].ALTA.R       /* Alternate Address 3 */
#define EMIOS_1_C2_3         EMIOS_1.CH.UC[3].C2.R         /* UC Control 2 3 */
#define EMIOS_1_A4           EMIOS_1.CH.UC[4].A.R          /* UC A 4 */
#define EMIOS_1_B4           EMIOS_1.CH.UC[4].B.R          /* UC B 4 */
#define EMIOS_1_CNT4         EMIOS_1.CH.UC[4].CNT.R        /* UC Counter 4 */
#define EMIOS_1_C4           EMIOS_1.CH.UC[4].C.R          /* UC Control 4 */
#define EMIOS_1_S4           EMIOS_1.CH.UC[4].S.R          /* UC Status 4 */
#define EMIOS_1_ALTA4        EMIOS_1.CH.UC[4].ALTA.R       /* Alternate Address 4 */
#define EMIOS_1_C2_4         EMIOS_1.CH.UC[4].C2.R         /* UC Control 2 4 */
#define EMIOS_1_A5           EMIOS_1.CH.UC[5].A.R          /* UC A 5 */
#define EMIOS_1_B5           EMIOS_1.CH.UC[5].B.R          /* UC B 5 */
#define EMIOS_1_CNT5         EMIOS_1.CH.UC[5].CNT.R        /* UC Counter 5 */
#define EMIOS_1_C5           EMIOS_1.CH.UC[5].C.R          /* UC Control 5 */
#define EMIOS_1_S5           EMIOS_1.CH.UC[5].S.R          /* UC Status 5 */
#define EMIOS_1_ALTA5        EMIOS_1.CH.UC[5].ALTA.R       /* Alternate Address 5 */
#define EMIOS_1_C2_5         EMIOS_1.CH.UC[5].C2.R         /* UC Control 2 5 */
#define EMIOS_1_A6           EMIOS_1.CH.UC[6].A.R          /* UC A 6 */
#define EMIOS_1_B6           EMIOS_1.CH.UC[6].B.R          /* UC B 6 */
#define EMIOS_1_CNT6         EMIOS_1.CH.UC[6].CNT.R        /* UC Counter 6 */
#define EMIOS_1_C6           EMIOS_1.CH.UC[6].C.R          /* UC Control 6 */
#define EMIOS_1_S6           EMIOS_1.CH.UC[6].S.R          /* UC Status 6 */
#define EMIOS_1_ALTA6        EMIOS_1.CH.UC[6].ALTA.R       /* Alternate Address 6 */
#define EMIOS_1_C2_6         EMIOS_1.CH.UC[6].C2.R         /* UC Control 2 6 */
#define EMIOS_1_A7           EMIOS_1.CH.UC[7].A.R          /* UC A 7 */
#define EMIOS_1_B7           EMIOS_1.CH.UC[7].B.R          /* UC B 7 */
#define EMIOS_1_CNT7         EMIOS_1.CH.UC[7].CNT.R        /* UC Counter 7 */
#define EMIOS_1_C7           EMIOS_1.CH.UC[7].C.R          /* UC Control 7 */
#define EMIOS_1_S7           EMIOS_1.CH.UC[7].S.R          /* UC Status 7 */
#define EMIOS_1_ALTA7        EMIOS_1.CH.UC[7].ALTA.R       /* Alternate Address 7 */
#define EMIOS_1_C2_7         EMIOS_1.CH.UC[7].C2.R         /* UC Control 2 7 */
#define EMIOS_1_A16          EMIOS_1.CH.UC[16].A.R         /* UC A 16 */
#define EMIOS_1_B16          EMIOS_1.CH.UC[16].B.R         /* UC B 16 */
#define EMIOS_1_CNT16        EMIOS_1.CH.UC[16].CNT.R       /* UC Counter 16 */
#define EMIOS_1_C16          EMIOS_1.CH.UC[16].C.R         /* UC Control 16 */
#define EMIOS_1_S16          EMIOS_1.CH.UC[16].S.R         /* UC Status 16 */
#define EMIOS_1_ALTA16       EMIOS_1.CH.UC[16].ALTA.R      /* Alternate Address 16 */
#define EMIOS_1_C2_16        EMIOS_1.CH.UC[16].C2.R        /* UC Control 2 16 */
#define EMIOS_1_A17          EMIOS_1.CH.UC[17].A.R         /* UC A 17 */
#define EMIOS_1_B17          EMIOS_1.CH.UC[17].B.R         /* UC B 17 */
#define EMIOS_1_CNT17        EMIOS_1.CH.UC[17].CNT.R       /* UC Counter 17 */
#define EMIOS_1_C17          EMIOS_1.CH.UC[17].C.R         /* UC Control 17 */
#define EMIOS_1_S17          EMIOS_1.CH.UC[17].S.R         /* UC Status 17 */
#define EMIOS_1_ALTA17       EMIOS_1.CH.UC[17].ALTA.R      /* Alternate Address 17 */
#define EMIOS_1_C2_17        EMIOS_1.CH.UC[17].C2.R        /* UC Control 2 17 */
#define EMIOS_1_A18          EMIOS_1.CH.UC[18].A.R         /* UC A 18 */
#define EMIOS_1_B18          EMIOS_1.CH.UC[18].B.R         /* UC B 18 */
#define EMIOS_1_CNT18        EMIOS_1.CH.UC[18].CNT.R       /* UC Counter 18 */
#define EMIOS_1_C18          EMIOS_1.CH.UC[18].C.R         /* UC Control 18 */
#define EMIOS_1_S18          EMIOS_1.CH.UC[18].S.R         /* UC Status 18 */
#define EMIOS_1_ALTA18       EMIOS_1.CH.UC[18].ALTA.R      /* Alternate Address 18 */
#define EMIOS_1_C2_18        EMIOS_1.CH.UC[18].C2.R        /* UC Control 2 18 */
#define EMIOS_1_A19          EMIOS_1.CH.UC[19].A.R         /* UC A 19 */
#define EMIOS_1_B19          EMIOS_1.CH.UC[19].B.R         /* UC B 19 */
#define EMIOS_1_CNT19        EMIOS_1.CH.UC[19].CNT.R       /* UC Counter 19 */
#define EMIOS_1_C19          EMIOS_1.CH.UC[19].C.R         /* UC Control 19 */
#define EMIOS_1_S19          EMIOS_1.CH.UC[19].S.R         /* UC Status 19 */
#define EMIOS_1_ALTA19       EMIOS_1.CH.UC[19].ALTA.R      /* Alternate Address 19 */
#define EMIOS_1_C2_19        EMIOS_1.CH.UC[19].C2.R        /* UC Control 2 19 */
#define EMIOS_1_A20          EMIOS_1.CH.UC[20].A.R         /* UC A 20 */
#define EMIOS_1_B20          EMIOS_1.CH.UC[20].B.R         /* UC B 20 */
#define EMIOS_1_CNT20        EMIOS_1.CH.UC[20].CNT.R       /* UC Counter 20 */
#define EMIOS_1_C20          EMIOS_1.CH.UC[20].C.R         /* UC Control 20 */
#define EMIOS_1_S20          EMIOS_1.CH.UC[20].S.R         /* UC Status 20 */
#define EMIOS_1_ALTA20       EMIOS_1.CH.UC[20].ALTA.R      /* Alternate Address 20 */
#define EMIOS_1_C2_20        EMIOS_1.CH.UC[20].C2.R        /* UC Control 2 20 */
#define EMIOS_1_A21          EMIOS_1.CH.UC[21].A.R         /* UC A 21 */
#define EMIOS_1_B21          EMIOS_1.CH.UC[21].B.R         /* UC B 21 */
#define EMIOS_1_CNT21        EMIOS_1.CH.UC[21].CNT.R       /* UC Counter 21 */
#define EMIOS_1_C21          EMIOS_1.CH.UC[21].C.R         /* UC Control 21 */
#define EMIOS_1_S21          EMIOS_1.CH.UC[21].S.R         /* UC Status 21 */
#define EMIOS_1_ALTA21       EMIOS_1.CH.UC[21].ALTA.R      /* Alternate Address 21 */
#define EMIOS_1_C2_21        EMIOS_1.CH.UC[21].C2.R        /* UC Control 2 21 */
#define EMIOS_1_A22          EMIOS_1.CH.UC[22].A.R         /* UC A 22 */
#define EMIOS_1_B22          EMIOS_1.CH.UC[22].B.R         /* UC B 22 */
#define EMIOS_1_CNT22        EMIOS_1.CH.UC[22].CNT.R       /* UC Counter 22 */
#define EMIOS_1_C22          EMIOS_1.CH.UC[22].C.R         /* UC Control 22 */
#define EMIOS_1_S22          EMIOS_1.CH.UC[22].S.R         /* UC Status 22 */
#define EMIOS_1_ALTA22       EMIOS_1.CH.UC[22].ALTA.R      /* Alternate Address 22 */
#define EMIOS_1_C2_22        EMIOS_1.CH.UC[22].C2.R        /* UC Control 2 22 */
#define EMIOS_1_A23          EMIOS_1.CH.UC[23].A.R         /* UC A 23 */
#define EMIOS_1_B23          EMIOS_1.CH.UC[23].B.R         /* UC B 23 */
#define EMIOS_1_CNT23        EMIOS_1.CH.UC[23].CNT.R       /* UC Counter 23 */
#define EMIOS_1_C23          EMIOS_1.CH.UC[23].C.R         /* UC Control 23 */
#define EMIOS_1_S23          EMIOS_1.CH.UC[23].S.R         /* UC Status 23 */
#define EMIOS_1_ALTA23       EMIOS_1.CH.UC[23].ALTA.R      /* Alternate Address 23 */
#define EMIOS_1_C2_23        EMIOS_1.CH.UC[23].C2.R        /* UC Control 2 23 */
#define EMIOS_1_A24          EMIOS_1.CH.UC[24].A.R         /* UC A 24 */
#define EMIOS_1_B24          EMIOS_1.CH.UC[24].B.R         /* UC B 24 */
#define EMIOS_1_CNT24        EMIOS_1.CH.UC[24].CNT.R       /* UC Counter 24 */
#define EMIOS_1_C24          EMIOS_1.CH.UC[24].C.R         /* UC Control 24 */
#define EMIOS_1_S24          EMIOS_1.CH.UC[24].S.R         /* UC Status 24 */
#define EMIOS_1_ALTA24       EMIOS_1.CH.UC[24].ALTA.R      /* Alternate Address 24 */
#define EMIOS_1_C2_24        EMIOS_1.CH.UC[24].C2.R        /* UC Control 2 24 */
#define EMIOS_1_A25          EMIOS_1.CH.UC[25].A.R         /* UC A 25 */
#define EMIOS_1_B25          EMIOS_1.CH.UC[25].B.R         /* UC B 25 */
#define EMIOS_1_CNT25        EMIOS_1.CH.UC[25].CNT.R       /* UC Counter 25 */
#define EMIOS_1_C25          EMIOS_1.CH.UC[25].C.R         /* UC Control 25 */
#define EMIOS_1_S25          EMIOS_1.CH.UC[25].S.R         /* UC Status 25 */
#define EMIOS_1_ALTA25       EMIOS_1.CH.UC[25].ALTA.R      /* Alternate Address 25 */
#define EMIOS_1_C2_25        EMIOS_1.CH.UC[25].C2.R        /* UC Control 2 25 */
#define EMIOS_1_A26          EMIOS_1.CH.UC[26].A.R         /* UC A 26 */
#define EMIOS_1_B26          EMIOS_1.CH.UC[26].B.R         /* UC B 26 */
#define EMIOS_1_CNT26        EMIOS_1.CH.UC[26].CNT.R       /* UC Counter 26 */
#define EMIOS_1_C26          EMIOS_1.CH.UC[26].C.R         /* UC Control 26 */
#define EMIOS_1_S26          EMIOS_1.CH.UC[26].S.R         /* UC Status 26 */
#define EMIOS_1_ALTA26       EMIOS_1.CH.UC[26].ALTA.R      /* Alternate Address 26 */
#define EMIOS_1_C2_26        EMIOS_1.CH.UC[26].C2.R        /* UC Control 2 26 */
#define EMIOS_1_A27          EMIOS_1.CH.UC[27].A.R         /* UC A 27 */
#define EMIOS_1_B27          EMIOS_1.CH.UC[27].B.R         /* UC B 27 */
#define EMIOS_1_CNT27        EMIOS_1.CH.UC[27].CNT.R       /* UC Counter 27 */
#define EMIOS_1_C27          EMIOS_1.CH.UC[27].C.R         /* UC Control 27 */
#define EMIOS_1_S27          EMIOS_1.CH.UC[27].S.R         /* UC Status 27 */
#define EMIOS_1_ALTA27       EMIOS_1.CH.UC[27].ALTA.R      /* Alternate Address 27 */
#define EMIOS_1_C2_27        EMIOS_1.CH.UC[27].C2.R        /* UC Control 2 27 */
#define EMIOS_1_A28          EMIOS_1.CH.UC[28].A.R         /* UC A 28 */
#define EMIOS_1_B28          EMIOS_1.CH.UC[28].B.R         /* UC B 28 */
#define EMIOS_1_CNT28        EMIOS_1.CH.UC[28].CNT.R       /* UC Counter 28 */
#define EMIOS_1_C28          EMIOS_1.CH.UC[28].C.R         /* UC Control 28 */
#define EMIOS_1_S28          EMIOS_1.CH.UC[28].S.R         /* UC Status 28 */
#define EMIOS_1_ALTA28       EMIOS_1.CH.UC[28].ALTA.R      /* Alternate Address 28 */
#define EMIOS_1_C2_28        EMIOS_1.CH.UC[28].C2.R        /* UC Control 2 28 */
#define EMIOS_1_A29          EMIOS_1.CH.UC[29].A.R         /* UC A 29 */
#define EMIOS_1_B29          EMIOS_1.CH.UC[29].B.R         /* UC B 29 */
#define EMIOS_1_CNT29        EMIOS_1.CH.UC[29].CNT.R       /* UC Counter 29 */
#define EMIOS_1_C29          EMIOS_1.CH.UC[29].C.R         /* UC Control 29 */
#define EMIOS_1_S29          EMIOS_1.CH.UC[29].S.R         /* UC Status 29 */
#define EMIOS_1_ALTA29       EMIOS_1.CH.UC[29].ALTA.R      /* Alternate Address 29 */
#define EMIOS_1_C2_29        EMIOS_1.CH.UC[29].C2.R        /* UC Control 2 29 */
#define EMIOS_1_A30          EMIOS_1.CH.UC[30].A.R         /* UC A 30 */
#define EMIOS_1_B30          EMIOS_1.CH.UC[30].B.R         /* UC B 30 */
#define EMIOS_1_CNT30        EMIOS_1.CH.UC[30].CNT.R       /* UC Counter 30 */
#define EMIOS_1_C30          EMIOS_1.CH.UC[30].C.R         /* UC Control 30 */
#define EMIOS_1_S30          EMIOS_1.CH.UC[30].S.R         /* UC Status 30 */
#define EMIOS_1_ALTA30       EMIOS_1.CH.UC[30].ALTA.R      /* Alternate Address 30 */
#define EMIOS_1_C2_30        EMIOS_1.CH.UC[30].C2.R        /* UC Control 2 30 */
#define EMIOS_1_A31          EMIOS_1.CH.UC[31].A.R         /* UC A 31 */
#define EMIOS_1_B31          EMIOS_1.CH.UC[31].B.R         /* UC B 31 */
#define EMIOS_1_CNT31        EMIOS_1.CH.UC[31].CNT.R       /* UC Counter 31 */
#define EMIOS_1_C31          EMIOS_1.CH.UC[31].C.R         /* UC Control 31 */
#define EMIOS_1_S31          EMIOS_1.CH.UC[31].S.R         /* UC Status 31 */
#define EMIOS_1_ALTA31       EMIOS_1.CH.UC[31].ALTA.R      /* Alternate Address 31 */
#define EMIOS_1_C2_31        EMIOS_1.CH.UC[31].C2.R        /* UC Control 2 31 */
#define EMIOS_1_WSCAEC_8     EMIOS_1.CH.WSC[4].WSCAEC.R    /* WSC Capture A and Event Counter 8 */
#define EMIOS_1_WSCAPB_8     EMIOS_1.CH.WSC[4].WSCAPB.R    /* WSC Capture B 8 */
#define EMIOS_1_WSC1_8       EMIOS_1.CH.WSC[4].WSC1.R      /* WSC Control 1 8 */
#define EMIOS_1_WSC2_8       EMIOS_1.CH.WSC[4].WSC2.R      /* WSC Control 2 8 */
#define EMIOS_1_WSS_8        EMIOS_1.CH.WSC[4].WSS.R       /* WSC Status 8 */
#define EMIOS_1_WSEV_8       EMIOS_1.CH.WSC[4].WSEV.R      /* WSC Event 8 */
#define EMIOS_1_WSCEV_8      EMIOS_1.CH.WSC[4].WSCEV.R     /* WSC Capture Event 8 */
#define EMIOS_1_WSPW_8       EMIOS_1.CH.WSC[4].WSPW.R      /* WSC Pulse Width 8 */
#define EMIOS_1_WSPWCNT_8    EMIOS_1.CH.WSC[4].WSPWCNT.R   /* WSC Pulse Width Counter 8 */
#define EMIOS_1_WSFC_8       EMIOS_1.CH.WSC[4].WSFC.R      /* WSC FIFO Control 8 */
#define EMIOS_1_WSFR_8       EMIOS_1.CH.WSC[4].WSFR.R      /* WSC FIFO Read 8 */
#define EMIOS_1_WSFCNT_8     EMIOS_1.CH.WSC[4].WSFCNT.R    /* WSC FIFO Counter 8 */
#define EMIOS_1_WSFPNT_8     EMIOS_1.CH.WSC[4].WSFPNT.R    /* WSC FIFO Pointer 8 */
#define EMIOS_1_WSCAEC_10    EMIOS_1.CH.WSC[5].WSCAEC.R    /* WSC Capture A and Event Counter 10 */
#define EMIOS_1_WSCAPB_10    EMIOS_1.CH.WSC[5].WSCAPB.R    /* WSC Capture B 10 */
#define EMIOS_1_WSC1_10      EMIOS_1.CH.WSC[5].WSC1.R      /* WSC Control 1 10 */
#define EMIOS_1_WSC2_10      EMIOS_1.CH.WSC[5].WSC2.R      /* WSC Control 2 10 */
#define EMIOS_1_WSS_10       EMIOS_1.CH.WSC[5].WSS.R       /* WSC Status 10 */
#define EMIOS_1_WSEV_10      EMIOS_1.CH.WSC[5].WSEV.R      /* WSC Event 10 */
#define EMIOS_1_WSCEV_10     EMIOS_1.CH.WSC[5].WSCEV.R     /* WSC Capture Event 10 */
#define EMIOS_1_WSPW_10      EMIOS_1.CH.WSC[5].WSPW.R      /* WSC Pulse Width 10 */
#define EMIOS_1_WSPWCNT_10   EMIOS_1.CH.WSC[5].WSPWCNT.R   /* WSC Pulse Width Counter 10 */
#define EMIOS_1_WSFC_10      EMIOS_1.CH.WSC[5].WSFC.R      /* WSC FIFO Control 10 */
#define EMIOS_1_WSFR_10      EMIOS_1.CH.WSC[5].WSFR.R      /* WSC FIFO Read 10 */
#define EMIOS_1_WSFCNT_10    EMIOS_1.CH.WSC[5].WSFCNT.R    /* WSC FIFO Counter 10 */
#define EMIOS_1_WSFPNT_10    EMIOS_1.CH.WSC[5].WSFPNT.R    /* WSC FIFO Pointer 10 */
#define EMIOS_1_WSCAEC_12    EMIOS_1.CH.WSC[6].WSCAEC.R    /* WSC Capture A and Event Counter 12 */
#define EMIOS_1_WSCAPB_12    EMIOS_1.CH.WSC[6].WSCAPB.R    /* WSC Capture B 12 */
#define EMIOS_1_WSC1_12      EMIOS_1.CH.WSC[6].WSC1.R      /* WSC Control 1 12 */
#define EMIOS_1_WSC2_12      EMIOS_1.CH.WSC[6].WSC2.R      /* WSC Control 2 12 */
#define EMIOS_1_WSS_12       EMIOS_1.CH.WSC[6].WSS.R       /* WSC Status 12 */
#define EMIOS_1_WSEV_12      EMIOS_1.CH.WSC[6].WSEV.R      /* WSC Event 12 */
#define EMIOS_1_WSCEV_12     EMIOS_1.CH.WSC[6].WSCEV.R     /* WSC Capture Event 12 */
#define EMIOS_1_WSPW_12      EMIOS_1.CH.WSC[6].WSPW.R      /* WSC Pulse Width 12 */
#define EMIOS_1_WSPWCNT_12   EMIOS_1.CH.WSC[6].WSPWCNT.R   /* WSC Pulse Width Counter 12 */
#define EMIOS_1_WSFC_12      EMIOS_1.CH.WSC[6].WSFC.R      /* WSC FIFO Control 12 */
#define EMIOS_1_WSFR_12      EMIOS_1.CH.WSC[6].WSFR.R      /* WSC FIFO Read 12 */
#define EMIOS_1_WSFCNT_12    EMIOS_1.CH.WSC[6].WSFCNT.R    /* WSC FIFO Counter 12 */
#define EMIOS_1_WSFPNT_12    EMIOS_1.CH.WSC[6].WSFPNT.R    /* WSC FIFO Pointer 12 */
#define EMIOS_1_WSCAEC_14    EMIOS_1.CH.WSC[7].WSCAEC.R    /* WSC Capture A and Event Counter 14 */
#define EMIOS_1_WSCAPB_14    EMIOS_1.CH.WSC[7].WSCAPB.R    /* WSC Capture B 14 */
#define EMIOS_1_WSC1_14      EMIOS_1.CH.WSC[7].WSC1.R      /* WSC Control 1 14 */
#define EMIOS_1_WSC2_14      EMIOS_1.CH.WSC[7].WSC2.R      /* WSC Control 2 14 */
#define EMIOS_1_WSS_14       EMIOS_1.CH.WSC[7].WSS.R       /* WSC Status 14 */
#define EMIOS_1_WSEV_14      EMIOS_1.CH.WSC[7].WSEV.R      /* WSC Event 14 */
#define EMIOS_1_WSCEV_14     EMIOS_1.CH.WSC[7].WSCEV.R     /* WSC Capture Event 14 */
#define EMIOS_1_WSPW_14      EMIOS_1.CH.WSC[7].WSPW.R      /* WSC Pulse Width 14 */
#define EMIOS_1_WSPWCNT_14   EMIOS_1.CH.WSC[7].WSPWCNT.R   /* WSC Pulse Width Counter 14 */
#define EMIOS_1_WSFC_14      EMIOS_1.CH.WSC[7].WSFC.R      /* WSC FIFO Control 14 */
#define EMIOS_1_WSFR_14      EMIOS_1.CH.WSC[7].WSFR.R      /* WSC FIFO Read 14 */
#define EMIOS_1_WSFCNT_14    EMIOS_1.CH.WSC[7].WSFCNT.R    /* WSC FIFO Counter 14 */
#define EMIOS_1_WSFPNT_14    EMIOS_1.CH.WSC[7].WSFPNT.R    /* WSC FIFO Pointer 14 */

#define EMIOS_0_MCR          EMIOS_0.MCR.R                 /* Module Configuration */
#define EMIOS_0_GFLAG        EMIOS_0.GFLAG.R               /* Global Flag */
#define EMIOS_0_OUDIS        EMIOS_0.OUDIS.R               /* Output Update Disable */
#define EMIOS_0_A0           EMIOS_0.CH.UC[0].A.R          /* UC A 0 */
#define EMIOS_0_B0           EMIOS_0.CH.UC[0].B.R          /* UC B 0 */
#define EMIOS_0_CNT0         EMIOS_0.CH.UC[0].CNT.R        /* UC Counter 0 */
#define EMIOS_0_C0           EMIOS_0.CH.UC[0].C.R          /* UC Control 0 */
#define EMIOS_0_S0           EMIOS_0.CH.UC[0].S.R          /* UC Status 0 */
#define EMIOS_0_ALTA0        EMIOS_0.CH.UC[0].ALTA.R       /* Alternate Address 0 */
#define EMIOS_0_C2_0         EMIOS_0.CH.UC[0].C2.R         /* UC Control 2 0 */
#define EMIOS_0_A1           EMIOS_0.CH.UC[1].A.R          /* UC A 1 */
#define EMIOS_0_B1           EMIOS_0.CH.UC[1].B.R          /* UC B 1 */
#define EMIOS_0_CNT1         EMIOS_0.CH.UC[1].CNT.R        /* UC Counter 1 */
#define EMIOS_0_C1           EMIOS_0.CH.UC[1].C.R          /* UC Control 1 */
#define EMIOS_0_S1           EMIOS_0.CH.UC[1].S.R          /* UC Status 1 */
#define EMIOS_0_ALTA1        EMIOS_0.CH.UC[1].ALTA.R       /* Alternate Address 1 */
#define EMIOS_0_C2_1         EMIOS_0.CH.UC[1].C2.R         /* UC Control 2 1 */
#define EMIOS_0_A2           EMIOS_0.CH.UC[2].A.R          /* UC A 2 */
#define EMIOS_0_B2           EMIOS_0.CH.UC[2].B.R          /* UC B 2 */
#define EMIOS_0_CNT2         EMIOS_0.CH.UC[2].CNT.R        /* UC Counter 2 */
#define EMIOS_0_C2           EMIOS_0.CH.UC[2].C.R          /* UC Control 2 */
#define EMIOS_0_S2           EMIOS_0.CH.UC[2].S.R          /* UC Status 2 */
#define EMIOS_0_ALTA2        EMIOS_0.CH.UC[2].ALTA.R       /* Alternate Address 2 */
#define EMIOS_0_C2_2         EMIOS_0.CH.UC[2].C2.R         /* UC Control 2 2 */
#define EMIOS_0_A3           EMIOS_0.CH.UC[3].A.R          /* UC A 3 */
#define EMIOS_0_B3           EMIOS_0.CH.UC[3].B.R          /* UC B 3 */
#define EMIOS_0_CNT3         EMIOS_0.CH.UC[3].CNT.R        /* UC Counter 3 */
#define EMIOS_0_C3           EMIOS_0.CH.UC[3].C.R          /* UC Control 3 */
#define EMIOS_0_S3           EMIOS_0.CH.UC[3].S.R          /* UC Status 3 */
#define EMIOS_0_ALTA3        EMIOS_0.CH.UC[3].ALTA.R       /* Alternate Address 3 */
#define EMIOS_0_C2_3         EMIOS_0.CH.UC[3].C2.R         /* UC Control 2 3 */
#define EMIOS_0_A4           EMIOS_0.CH.UC[4].A.R          /* UC A 4 */
#define EMIOS_0_B4           EMIOS_0.CH.UC[4].B.R          /* UC B 4 */
#define EMIOS_0_CNT4         EMIOS_0.CH.UC[4].CNT.R        /* UC Counter 4 */
#define EMIOS_0_C4           EMIOS_0.CH.UC[4].C.R          /* UC Control 4 */
#define EMIOS_0_S4           EMIOS_0.CH.UC[4].S.R          /* UC Status 4 */
#define EMIOS_0_ALTA4        EMIOS_0.CH.UC[4].ALTA.R       /* Alternate Address 4 */
#define EMIOS_0_C2_4         EMIOS_0.CH.UC[4].C2.R         /* UC Control 2 4 */
#define EMIOS_0_A5           EMIOS_0.CH.UC[5].A.R          /* UC A 5 */
#define EMIOS_0_B5           EMIOS_0.CH.UC[5].B.R          /* UC B 5 */
#define EMIOS_0_CNT5         EMIOS_0.CH.UC[5].CNT.R        /* UC Counter 5 */
#define EMIOS_0_C5           EMIOS_0.CH.UC[5].C.R          /* UC Control 5 */
#define EMIOS_0_S5           EMIOS_0.CH.UC[5].S.R          /* UC Status 5 */
#define EMIOS_0_ALTA5        EMIOS_0.CH.UC[5].ALTA.R       /* Alternate Address 5 */
#define EMIOS_0_C2_5         EMIOS_0.CH.UC[5].C2.R         /* UC Control 2 5 */
#define EMIOS_0_A6           EMIOS_0.CH.UC[6].A.R          /* UC A 6 */
#define EMIOS_0_B6           EMIOS_0.CH.UC[6].B.R          /* UC B 6 */
#define EMIOS_0_CNT6         EMIOS_0.CH.UC[6].CNT.R        /* UC Counter 6 */
#define EMIOS_0_C6           EMIOS_0.CH.UC[6].C.R          /* UC Control 6 */
#define EMIOS_0_S6           EMIOS_0.CH.UC[6].S.R          /* UC Status 6 */
#define EMIOS_0_ALTA6        EMIOS_0.CH.UC[6].ALTA.R       /* Alternate Address 6 */
#define EMIOS_0_C2_6         EMIOS_0.CH.UC[6].C2.R         /* UC Control 2 6 */
#define EMIOS_0_A7           EMIOS_0.CH.UC[7].A.R          /* UC A 7 */
#define EMIOS_0_B7           EMIOS_0.CH.UC[7].B.R          /* UC B 7 */
#define EMIOS_0_CNT7         EMIOS_0.CH.UC[7].CNT.R        /* UC Counter 7 */
#define EMIOS_0_C7           EMIOS_0.CH.UC[7].C.R          /* UC Control 7 */
#define EMIOS_0_S7           EMIOS_0.CH.UC[7].S.R          /* UC Status 7 */
#define EMIOS_0_ALTA7        EMIOS_0.CH.UC[7].ALTA.R       /* Alternate Address 7 */
#define EMIOS_0_C2_7         EMIOS_0.CH.UC[7].C2.R         /* UC Control 2 7 */
#define EMIOS_0_A16          EMIOS_0.CH.UC[16].A.R         /* UC A 16 */
#define EMIOS_0_B16          EMIOS_0.CH.UC[16].B.R         /* UC B 16 */
#define EMIOS_0_CNT16        EMIOS_0.CH.UC[16].CNT.R       /* UC Counter 16 */
#define EMIOS_0_C16          EMIOS_0.CH.UC[16].C.R         /* UC Control 16 */
#define EMIOS_0_S16          EMIOS_0.CH.UC[16].S.R         /* UC Status 16 */
#define EMIOS_0_ALTA16       EMIOS_0.CH.UC[16].ALTA.R      /* Alternate Address 16 */
#define EMIOS_0_C2_16        EMIOS_0.CH.UC[16].C2.R        /* UC Control 2 16 */
#define EMIOS_0_A17          EMIOS_0.CH.UC[17].A.R         /* UC A 17 */
#define EMIOS_0_B17          EMIOS_0.CH.UC[17].B.R         /* UC B 17 */
#define EMIOS_0_CNT17        EMIOS_0.CH.UC[17].CNT.R       /* UC Counter 17 */
#define EMIOS_0_C17          EMIOS_0.CH.UC[17].C.R         /* UC Control 17 */
#define EMIOS_0_S17          EMIOS_0.CH.UC[17].S.R         /* UC Status 17 */
#define EMIOS_0_ALTA17       EMIOS_0.CH.UC[17].ALTA.R      /* Alternate Address 17 */
#define EMIOS_0_C2_17        EMIOS_0.CH.UC[17].C2.R        /* UC Control 2 17 */
#define EMIOS_0_A18          EMIOS_0.CH.UC[18].A.R         /* UC A 18 */
#define EMIOS_0_B18          EMIOS_0.CH.UC[18].B.R         /* UC B 18 */
#define EMIOS_0_CNT18        EMIOS_0.CH.UC[18].CNT.R       /* UC Counter 18 */
#define EMIOS_0_C18          EMIOS_0.CH.UC[18].C.R         /* UC Control 18 */
#define EMIOS_0_S18          EMIOS_0.CH.UC[18].S.R         /* UC Status 18 */
#define EMIOS_0_ALTA18       EMIOS_0.CH.UC[18].ALTA.R      /* Alternate Address 18 */
#define EMIOS_0_C2_18        EMIOS_0.CH.UC[18].C2.R        /* UC Control 2 18 */
#define EMIOS_0_A19          EMIOS_0.CH.UC[19].A.R         /* UC A 19 */
#define EMIOS_0_B19          EMIOS_0.CH.UC[19].B.R         /* UC B 19 */
#define EMIOS_0_CNT19        EMIOS_0.CH.UC[19].CNT.R       /* UC Counter 19 */
#define EMIOS_0_C19          EMIOS_0.CH.UC[19].C.R         /* UC Control 19 */
#define EMIOS_0_S19          EMIOS_0.CH.UC[19].S.R         /* UC Status 19 */
#define EMIOS_0_ALTA19       EMIOS_0.CH.UC[19].ALTA.R      /* Alternate Address 19 */
#define EMIOS_0_C2_19        EMIOS_0.CH.UC[19].C2.R        /* UC Control 2 19 */
#define EMIOS_0_A20          EMIOS_0.CH.UC[20].A.R         /* UC A 20 */
#define EMIOS_0_B20          EMIOS_0.CH.UC[20].B.R         /* UC B 20 */
#define EMIOS_0_CNT20        EMIOS_0.CH.UC[20].CNT.R       /* UC Counter 20 */
#define EMIOS_0_C20          EMIOS_0.CH.UC[20].C.R         /* UC Control 20 */
#define EMIOS_0_S20          EMIOS_0.CH.UC[20].S.R         /* UC Status 20 */
#define EMIOS_0_ALTA20       EMIOS_0.CH.UC[20].ALTA.R      /* Alternate Address 20 */
#define EMIOS_0_C2_20        EMIOS_0.CH.UC[20].C2.R        /* UC Control 2 20 */
#define EMIOS_0_A21          EMIOS_0.CH.UC[21].A.R         /* UC A 21 */
#define EMIOS_0_B21          EMIOS_0.CH.UC[21].B.R         /* UC B 21 */
#define EMIOS_0_CNT21        EMIOS_0.CH.UC[21].CNT.R       /* UC Counter 21 */
#define EMIOS_0_C21          EMIOS_0.CH.UC[21].C.R         /* UC Control 21 */
#define EMIOS_0_S21          EMIOS_0.CH.UC[21].S.R         /* UC Status 21 */
#define EMIOS_0_ALTA21       EMIOS_0.CH.UC[21].ALTA.R      /* Alternate Address 21 */
#define EMIOS_0_C2_21        EMIOS_0.CH.UC[21].C2.R        /* UC Control 2 21 */
#define EMIOS_0_A22          EMIOS_0.CH.UC[22].A.R         /* UC A 22 */
#define EMIOS_0_B22          EMIOS_0.CH.UC[22].B.R         /* UC B 22 */
#define EMIOS_0_CNT22        EMIOS_0.CH.UC[22].CNT.R       /* UC Counter 22 */
#define EMIOS_0_C22          EMIOS_0.CH.UC[22].C.R         /* UC Control 22 */
#define EMIOS_0_S22          EMIOS_0.CH.UC[22].S.R         /* UC Status 22 */
#define EMIOS_0_ALTA22       EMIOS_0.CH.UC[22].ALTA.R      /* Alternate Address 22 */
#define EMIOS_0_C2_22        EMIOS_0.CH.UC[22].C2.R        /* UC Control 2 22 */
#define EMIOS_0_A23          EMIOS_0.CH.UC[23].A.R         /* UC A 23 */
#define EMIOS_0_B23          EMIOS_0.CH.UC[23].B.R         /* UC B 23 */
#define EMIOS_0_CNT23        EMIOS_0.CH.UC[23].CNT.R       /* UC Counter 23 */
#define EMIOS_0_C23          EMIOS_0.CH.UC[23].C.R         /* UC Control 23 */
#define EMIOS_0_S23          EMIOS_0.CH.UC[23].S.R         /* UC Status 23 */
#define EMIOS_0_ALTA23       EMIOS_0.CH.UC[23].ALTA.R      /* Alternate Address 23 */
#define EMIOS_0_C2_23        EMIOS_0.CH.UC[23].C2.R        /* UC Control 2 23 */
#define EMIOS_0_A24          EMIOS_0.CH.UC[24].A.R         /* UC A 24 */
#define EMIOS_0_B24          EMIOS_0.CH.UC[24].B.R         /* UC B 24 */
#define EMIOS_0_CNT24        EMIOS_0.CH.UC[24].CNT.R       /* UC Counter 24 */
#define EMIOS_0_C24          EMIOS_0.CH.UC[24].C.R         /* UC Control 24 */
#define EMIOS_0_S24          EMIOS_0.CH.UC[24].S.R         /* UC Status 24 */
#define EMIOS_0_ALTA24       EMIOS_0.CH.UC[24].ALTA.R      /* Alternate Address 24 */
#define EMIOS_0_C2_24        EMIOS_0.CH.UC[24].C2.R        /* UC Control 2 24 */
#define EMIOS_0_A25          EMIOS_0.CH.UC[25].A.R         /* UC A 25 */
#define EMIOS_0_B25          EMIOS_0.CH.UC[25].B.R         /* UC B 25 */
#define EMIOS_0_CNT25        EMIOS_0.CH.UC[25].CNT.R       /* UC Counter 25 */
#define EMIOS_0_C25          EMIOS_0.CH.UC[25].C.R         /* UC Control 25 */
#define EMIOS_0_S25          EMIOS_0.CH.UC[25].S.R         /* UC Status 25 */
#define EMIOS_0_ALTA25       EMIOS_0.CH.UC[25].ALTA.R      /* Alternate Address 25 */
#define EMIOS_0_C2_25        EMIOS_0.CH.UC[25].C2.R        /* UC Control 2 25 */
#define EMIOS_0_A26          EMIOS_0.CH.UC[26].A.R         /* UC A 26 */
#define EMIOS_0_B26          EMIOS_0.CH.UC[26].B.R         /* UC B 26 */
#define EMIOS_0_CNT26        EMIOS_0.CH.UC[26].CNT.R       /* UC Counter 26 */
#define EMIOS_0_C26          EMIOS_0.CH.UC[26].C.R         /* UC Control 26 */
#define EMIOS_0_S26          EMIOS_0.CH.UC[26].S.R         /* UC Status 26 */
#define EMIOS_0_ALTA26       EMIOS_0.CH.UC[26].ALTA.R      /* Alternate Address 26 */
#define EMIOS_0_C2_26        EMIOS_0.CH.UC[26].C2.R        /* UC Control 2 26 */
#define EMIOS_0_A27          EMIOS_0.CH.UC[27].A.R         /* UC A 27 */
#define EMIOS_0_B27          EMIOS_0.CH.UC[27].B.R         /* UC B 27 */
#define EMIOS_0_CNT27        EMIOS_0.CH.UC[27].CNT.R       /* UC Counter 27 */
#define EMIOS_0_C27          EMIOS_0.CH.UC[27].C.R         /* UC Control 27 */
#define EMIOS_0_S27          EMIOS_0.CH.UC[27].S.R         /* UC Status 27 */
#define EMIOS_0_ALTA27       EMIOS_0.CH.UC[27].ALTA.R      /* Alternate Address 27 */
#define EMIOS_0_C2_27        EMIOS_0.CH.UC[27].C2.R        /* UC Control 2 27 */
#define EMIOS_0_A28          EMIOS_0.CH.UC[28].A.R         /* UC A 28 */
#define EMIOS_0_B28          EMIOS_0.CH.UC[28].B.R         /* UC B 28 */
#define EMIOS_0_CNT28        EMIOS_0.CH.UC[28].CNT.R       /* UC Counter 28 */
#define EMIOS_0_C28          EMIOS_0.CH.UC[28].C.R         /* UC Control 28 */
#define EMIOS_0_S28          EMIOS_0.CH.UC[28].S.R         /* UC Status 28 */
#define EMIOS_0_ALTA28       EMIOS_0.CH.UC[28].ALTA.R      /* Alternate Address 28 */
#define EMIOS_0_C2_28        EMIOS_0.CH.UC[28].C2.R        /* UC Control 2 28 */
#define EMIOS_0_A29          EMIOS_0.CH.UC[29].A.R         /* UC A 29 */
#define EMIOS_0_B29          EMIOS_0.CH.UC[29].B.R         /* UC B 29 */
#define EMIOS_0_CNT29        EMIOS_0.CH.UC[29].CNT.R       /* UC Counter 29 */
#define EMIOS_0_C29          EMIOS_0.CH.UC[29].C.R         /* UC Control 29 */
#define EMIOS_0_S29          EMIOS_0.CH.UC[29].S.R         /* UC Status 29 */
#define EMIOS_0_ALTA29       EMIOS_0.CH.UC[29].ALTA.R      /* Alternate Address 29 */
#define EMIOS_0_C2_29        EMIOS_0.CH.UC[29].C2.R        /* UC Control 2 29 */
#define EMIOS_0_A30          EMIOS_0.CH.UC[30].A.R         /* UC A 30 */
#define EMIOS_0_B30          EMIOS_0.CH.UC[30].B.R         /* UC B 30 */
#define EMIOS_0_CNT30        EMIOS_0.CH.UC[30].CNT.R       /* UC Counter 30 */
#define EMIOS_0_C30          EMIOS_0.CH.UC[30].C.R         /* UC Control 30 */
#define EMIOS_0_S30          EMIOS_0.CH.UC[30].S.R         /* UC Status 30 */
#define EMIOS_0_ALTA30       EMIOS_0.CH.UC[30].ALTA.R      /* Alternate Address 30 */
#define EMIOS_0_C2_30        EMIOS_0.CH.UC[30].C2.R        /* UC Control 2 30 */
#define EMIOS_0_A31          EMIOS_0.CH.UC[31].A.R         /* UC A 31 */
#define EMIOS_0_B31          EMIOS_0.CH.UC[31].B.R         /* UC B 31 */
#define EMIOS_0_CNT31        EMIOS_0.CH.UC[31].CNT.R       /* UC Counter 31 */
#define EMIOS_0_C31          EMIOS_0.CH.UC[31].C.R         /* UC Control 31 */
#define EMIOS_0_S31          EMIOS_0.CH.UC[31].S.R         /* UC Status 31 */
#define EMIOS_0_ALTA31       EMIOS_0.CH.UC[31].ALTA.R      /* Alternate Address 31 */
#define EMIOS_0_C2_31        EMIOS_0.CH.UC[31].C2.R        /* UC Control 2 31 */
#define EMIOS_0_WSCAEC_8     EMIOS_0.CH.WSC[4].WSCAEC.R    /* WSC Capture A and Event Counter 8 */
#define EMIOS_0_WSCAPB_8     EMIOS_0.CH.WSC[4].WSCAPB.R    /* WSC Capture B 8 */
#define EMIOS_0_WSC1_8       EMIOS_0.CH.WSC[4].WSC1.R      /* WSC Control 1 8 */
#define EMIOS_0_WSC2_8       EMIOS_0.CH.WSC[4].WSC2.R      /* WSC Control 2 8 */
#define EMIOS_0_WSS_8        EMIOS_0.CH.WSC[4].WSS.R       /* WSC Status 8 */
#define EMIOS_0_WSEV_8       EMIOS_0.CH.WSC[4].WSEV.R      /* WSC Event 8 */
#define EMIOS_0_WSCEV_8      EMIOS_0.CH.WSC[4].WSCEV.R     /* WSC Capture Event 8 */
#define EMIOS_0_WSPW_8       EMIOS_0.CH.WSC[4].WSPW.R      /* WSC Pulse Width 8 */
#define EMIOS_0_WSPWCNT_8    EMIOS_0.CH.WSC[4].WSPWCNT.R   /* WSC Pulse Width Counter 8 */
#define EMIOS_0_WSFC_8       EMIOS_0.CH.WSC[4].WSFC.R      /* WSC FIFO Control 8 */
#define EMIOS_0_WSFR_8       EMIOS_0.CH.WSC[4].WSFR.R      /* WSC FIFO Read 8 */
#define EMIOS_0_WSFCNT_8     EMIOS_0.CH.WSC[4].WSFCNT.R    /* WSC FIFO Counter 8 */
#define EMIOS_0_WSFPNT_8     EMIOS_0.CH.WSC[4].WSFPNT.R    /* WSC FIFO Pointer 8 */
#define EMIOS_0_WSCAEC_10    EMIOS_0.CH.WSC[5].WSCAEC.R    /* WSC Capture A and Event Counter 10 */
#define EMIOS_0_WSCAPB_10    EMIOS_0.CH.WSC[5].WSCAPB.R    /* WSC Capture B 10 */
#define EMIOS_0_WSC1_10      EMIOS_0.CH.WSC[5].WSC1.R      /* WSC Control 1 10 */
#define EMIOS_0_WSC2_10      EMIOS_0.CH.WSC[5].WSC2.R      /* WSC Control 2 10 */
#define EMIOS_0_WSS_10       EMIOS_0.CH.WSC[5].WSS.R       /* WSC Status 10 */
#define EMIOS_0_WSEV_10      EMIOS_0.CH.WSC[5].WSEV.R      /* WSC Event 10 */
#define EMIOS_0_WSCEV_10     EMIOS_0.CH.WSC[5].WSCEV.R     /* WSC Capture Event 10 */
#define EMIOS_0_WSPW_10      EMIOS_0.CH.WSC[5].WSPW.R      /* WSC Pulse Width 10 */
#define EMIOS_0_WSPWCNT_10   EMIOS_0.CH.WSC[5].WSPWCNT.R   /* WSC Pulse Width Counter 10 */
#define EMIOS_0_WSFC_10      EMIOS_0.CH.WSC[5].WSFC.R      /* WSC FIFO Control 10 */
#define EMIOS_0_WSFR_10      EMIOS_0.CH.WSC[5].WSFR.R      /* WSC FIFO Read 10 */
#define EMIOS_0_WSFCNT_10    EMIOS_0.CH.WSC[5].WSFCNT.R    /* WSC FIFO Counter 10 */
#define EMIOS_0_WSFPNT_10    EMIOS_0.CH.WSC[5].WSFPNT.R    /* WSC FIFO Pointer 10 */
#define EMIOS_0_WSCAEC_12    EMIOS_0.CH.WSC[6].WSCAEC.R    /* WSC Capture A and Event Counter 12 */
#define EMIOS_0_WSCAPB_12    EMIOS_0.CH.WSC[6].WSCAPB.R    /* WSC Capture B 12 */
#define EMIOS_0_WSC1_12      EMIOS_0.CH.WSC[6].WSC1.R      /* WSC Control 1 12 */
#define EMIOS_0_WSC2_12      EMIOS_0.CH.WSC[6].WSC2.R      /* WSC Control 2 12 */
#define EMIOS_0_WSS_12       EMIOS_0.CH.WSC[6].WSS.R       /* WSC Status 12 */
#define EMIOS_0_WSEV_12      EMIOS_0.CH.WSC[6].WSEV.R      /* WSC Event 12 */
#define EMIOS_0_WSCEV_12     EMIOS_0.CH.WSC[6].WSCEV.R     /* WSC Capture Event 12 */
#define EMIOS_0_WSPW_12      EMIOS_0.CH.WSC[6].WSPW.R      /* WSC Pulse Width 12 */
#define EMIOS_0_WSPWCNT_12   EMIOS_0.CH.WSC[6].WSPWCNT.R   /* WSC Pulse Width Counter 12 */
#define EMIOS_0_WSFC_12      EMIOS_0.CH.WSC[6].WSFC.R      /* WSC FIFO Control 12 */
#define EMIOS_0_WSFR_12      EMIOS_0.CH.WSC[6].WSFR.R      /* WSC FIFO Read 12 */
#define EMIOS_0_WSFCNT_12    EMIOS_0.CH.WSC[6].WSFCNT.R    /* WSC FIFO Counter 12 */
#define EMIOS_0_WSFPNT_12    EMIOS_0.CH.WSC[6].WSFPNT.R    /* WSC FIFO Pointer 12 */
#define EMIOS_0_WSCAEC_14    EMIOS_0.CH.WSC[7].WSCAEC.R    /* WSC Capture A and Event Counter 14 */
#define EMIOS_0_WSCAPB_14    EMIOS_0.CH.WSC[7].WSCAPB.R    /* WSC Capture B 14 */
#define EMIOS_0_WSC1_14      EMIOS_0.CH.WSC[7].WSC1.R      /* WSC Control 1 14 */
#define EMIOS_0_WSC2_14      EMIOS_0.CH.WSC[7].WSC2.R      /* WSC Control 2 14 */
#define EMIOS_0_WSS_14       EMIOS_0.CH.WSC[7].WSS.R       /* WSC Status 14 */
#define EMIOS_0_WSEV_14      EMIOS_0.CH.WSC[7].WSEV.R      /* WSC Event 14 */
#define EMIOS_0_WSCEV_14     EMIOS_0.CH.WSC[7].WSCEV.R     /* WSC Capture Event 14 */
#define EMIOS_0_WSPW_14      EMIOS_0.CH.WSC[7].WSPW.R      /* WSC Pulse Width 14 */
#define EMIOS_0_WSPWCNT_14   EMIOS_0.CH.WSC[7].WSPWCNT.R   /* WSC Pulse Width Counter 14 */
#define EMIOS_0_WSFC_14      EMIOS_0.CH.WSC[7].WSFC.R      /* WSC FIFO Control 14 */
#define EMIOS_0_WSFR_14      EMIOS_0.CH.WSC[7].WSFR.R      /* WSC FIFO Read 14 */
#define EMIOS_0_WSFCNT_14    EMIOS_0.CH.WSC[7].WSFCNT.R    /* WSC FIFO Counter 14 */
#define EMIOS_0_WSFPNT_14    EMIOS_0.CH.WSC[7].WSFPNT.R    /* WSC FIFO Pointer 14 */

/* ENETC_PORT */
#define NETC__ENETC0_PORT_PCAPR NETC__ENETC0_PORT.PCAPR.R  /* Port capability register */
#define NETC__ENETC0_PORT_PMCAPR NETC__ENETC0_PORT.PMCAPR.R /* Port MAC capability register */
#define NETC__ENETC0_PORT_PIOCAPR NETC__ENETC0_PORT.PIOCAPR.R /* Port I/O capability register */
#define NETC__ENETC0_PORT_PCR NETC__ENETC0_PORT.PCR.R      /* Port configuration register */
#define NETC__ENETC0_PORT_PMAR0 NETC__ENETC0_PORT.PMAR0.R  /* Port MAC address register 0 */
#define NETC__ENETC0_PORT_PMAR1 NETC__ENETC0_PORT.PMAR1.R  /* Port MAC address register 1 */
#define NETC__ENETC0_PORT_PTAR NETC__ENETC0_PORT.PTAR.R    /* Port TPID acceptance register */
#define NETC__ENETC0_PORT_PQOSMR NETC__ENETC0_PORT.PQOSMR.R /* Port QoS mode register */
#define NETC__ENETC0_PORT_PPCR NETC__ENETC0_PORT.PPCR.R    /* Port parser configuration register */
#define NETC__ENETC0_PORT_PIPFCR NETC__ENETC0_PORT.PIPFCR.R /* Port ingress port filter configuration register */
#define NETC__ENETC0_PORT_PSGCR NETC__ENETC0_PORT.PSGCR.R  /* Port stream gate configuration register */
#define NETC__ENETC0_PORT_POR NETC__ENETC0_PORT.PORr.R     /* Port operational register */
#define NETC__ENETC0_PORT_PSR NETC__ENETC0_PORT.PSR.R      /* Port status register */
#define NETC__ENETC0_PORT_PRXSDUOR NETC__ENETC0_PORT.PRXSDUOR.R /* Port receive SDU overhead register */
#define NETC__ENETC0_PORT_PTXSDUOR NETC__ENETC0_PORT.PTXSDUOR.R /* Port transmit SDU overhead register */
#define NETC__ENETC0_PORT_PTGSCR NETC__ENETC0_PORT.PTGSCR.R /* Port time gate scheduling control register */
#define NETC__ENETC0_PORT_PTGAGLSR NETC__ENETC0_PORT.PTGAGLSR.R /* Port time gate scheduling admin gate list status register */
#define NETC__ENETC0_PORT_PTGAGLLR NETC__ENETC0_PORT.PTGAGLLR.R /* Port time gate scheduling admin gate list length register */
#define NETC__ENETC0_PORT_PTGOGLLR NETC__ENETC0_PORT.PTGOGLLR.R /* Port time gating operational gate list length register */
#define NETC__ENETC0_PORT_PTGSATOR NETC__ENETC0_PORT.PTGSATOR.R /* Port time gate scheduling advance time offset register */
#define NETC__ENETC0_PORT_PRXDCR NETC__ENETC0_PORT.PRXDCR.R /* Port Rx discard count register */
#define NETC__ENETC0_PORT_PRXDCRR0 NETC__ENETC0_PORT.PRXDCRR0.R /* Port Rx discard count reason register 0 */
#define NETC__ENETC0_PORT_PRXDCRR1 NETC__ENETC0_PORT.PRXDCRR1.R /* Port Rx discard count reason register 1 */
#define NETC__ENETC0_PORT_PTGSTC0SR NETC__ENETC0_PORT.TCT_NUM[0].PTGSTCSR.R /* Port time gate scheduling traffic class 0 status register */
#define NETC__ENETC0_PORT_PTC0TMSDUR NETC__ENETC0_PORT.TCT_NUM[0].PTCTMSDUR.R /* Port traffic class 0 transmit maximum SDU register */
#define NETC__ENETC0_PORT_PTC0CBSR0 NETC__ENETC0_PORT.TCT_NUM[0].PTCCBSR0.R /* Port transmit traffic class 0 credit based shaper register 0 */
#define NETC__ENETC0_PORT_PTC0CBSR1 NETC__ENETC0_PORT.TCT_NUM[0].PTCCBSR1.R /* Port traffic class 0 credit based shaper register 1 */
#define NETC__ENETC0_PORT_PTGSTC1SR NETC__ENETC0_PORT.TCT_NUM[1].PTGSTCSR.R /* Port time gate scheduling traffic class 1 status register */
#define NETC__ENETC0_PORT_PTC1TMSDUR NETC__ENETC0_PORT.TCT_NUM[1].PTCTMSDUR.R /* Port traffic class 1 transmit maximum SDU register */
#define NETC__ENETC0_PORT_PTC1CBSR0 NETC__ENETC0_PORT.TCT_NUM[1].PTCCBSR0.R /* Port transmit traffic class 1 credit based shaper register 0 */
#define NETC__ENETC0_PORT_PTC1CBSR1 NETC__ENETC0_PORT.TCT_NUM[1].PTCCBSR1.R /* Port traffic class 1 credit based shaper register 1 */
#define NETC__ENETC0_PORT_PTGSTC2SR NETC__ENETC0_PORT.TCT_NUM[2].PTGSTCSR.R /* Port time gate scheduling traffic class 2 status register */
#define NETC__ENETC0_PORT_PTC2TMSDUR NETC__ENETC0_PORT.TCT_NUM[2].PTCTMSDUR.R /* Port traffic class 2 transmit maximum SDU register */
#define NETC__ENETC0_PORT_PTC2CBSR0 NETC__ENETC0_PORT.TCT_NUM[2].PTCCBSR0.R /* Port transmit traffic class 2 credit based shaper register 0 */
#define NETC__ENETC0_PORT_PTC2CBSR1 NETC__ENETC0_PORT.TCT_NUM[2].PTCCBSR1.R /* Port traffic class 2 credit based shaper register 1 */
#define NETC__ENETC0_PORT_PTGSTC3SR NETC__ENETC0_PORT.TCT_NUM[3].PTGSTCSR.R /* Port time gate scheduling traffic class 3 status register */
#define NETC__ENETC0_PORT_PTC3TMSDUR NETC__ENETC0_PORT.TCT_NUM[3].PTCTMSDUR.R /* Port traffic class 3 transmit maximum SDU register */
#define NETC__ENETC0_PORT_PTC3CBSR0 NETC__ENETC0_PORT.TCT_NUM[3].PTCCBSR0.R /* Port transmit traffic class 3 credit based shaper register 0 */
#define NETC__ENETC0_PORT_PTC3CBSR1 NETC__ENETC0_PORT.TCT_NUM[3].PTCCBSR1.R /* Port traffic class 3 credit based shaper register 1 */
#define NETC__ENETC0_PORT_PTGSTC4SR NETC__ENETC0_PORT.TCT_NUM[4].PTGSTCSR.R /* Port time gate scheduling traffic class 4 status register */
#define NETC__ENETC0_PORT_PTC4TMSDUR NETC__ENETC0_PORT.TCT_NUM[4].PTCTMSDUR.R /* Port traffic class 4 transmit maximum SDU register */
#define NETC__ENETC0_PORT_PTC4CBSR0 NETC__ENETC0_PORT.TCT_NUM[4].PTCCBSR0.R /* Port transmit traffic class 4 credit based shaper register 0 */
#define NETC__ENETC0_PORT_PTC4CBSR1 NETC__ENETC0_PORT.TCT_NUM[4].PTCCBSR1.R /* Port traffic class 4 credit based shaper register 1 */
#define NETC__ENETC0_PORT_PTGSTC5SR NETC__ENETC0_PORT.TCT_NUM[5].PTGSTCSR.R /* Port time gate scheduling traffic class 5 status register */
#define NETC__ENETC0_PORT_PTC5TMSDUR NETC__ENETC0_PORT.TCT_NUM[5].PTCTMSDUR.R /* Port traffic class 5 transmit maximum SDU register */
#define NETC__ENETC0_PORT_PTC5CBSR0 NETC__ENETC0_PORT.TCT_NUM[5].PTCCBSR0.R /* Port transmit traffic class 5 credit based shaper register 0 */
#define NETC__ENETC0_PORT_PTC5CBSR1 NETC__ENETC0_PORT.TCT_NUM[5].PTCCBSR1.R /* Port traffic class 5 credit based shaper register 1 */
#define NETC__ENETC0_PORT_PTGSTC6SR NETC__ENETC0_PORT.TCT_NUM[6].PTGSTCSR.R /* Port time gate scheduling traffic class 6 status register */
#define NETC__ENETC0_PORT_PTC6TMSDUR NETC__ENETC0_PORT.TCT_NUM[6].PTCTMSDUR.R /* Port traffic class 6 transmit maximum SDU register */
#define NETC__ENETC0_PORT_PTC6CBSR0 NETC__ENETC0_PORT.TCT_NUM[6].PTCCBSR0.R /* Port transmit traffic class 6 credit based shaper register 0 */
#define NETC__ENETC0_PORT_PTC6CBSR1 NETC__ENETC0_PORT.TCT_NUM[6].PTCCBSR1.R /* Port traffic class 6 credit based shaper register 1 */
#define NETC__ENETC0_PORT_PTGSTC7SR NETC__ENETC0_PORT.TCT_NUM[7].PTGSTCSR.R /* Port time gate scheduling traffic class 7 status register */
#define NETC__ENETC0_PORT_PTC7TMSDUR NETC__ENETC0_PORT.TCT_NUM[7].PTCTMSDUR.R /* Port traffic class 7 transmit maximum SDU register */
#define NETC__ENETC0_PORT_PTC7CBSR0 NETC__ENETC0_PORT.TCT_NUM[7].PTCCBSR0.R /* Port transmit traffic class 7 credit based shaper register 0 */
#define NETC__ENETC0_PORT_PTC7CBSR1 NETC__ENETC0_PORT.TCT_NUM[7].PTCCBSR1.R /* Port traffic class 7 credit based shaper register 1 */
#define NETC__ENETC0_PORT_PPCPDEIMR NETC__ENETC0_PORT.PPCPDEIMR.R /* Port PCP DEI mapping register */
#define NETC__ENETC0_PORT_PISIDCR NETC__ENETC0_PORT.PISIDCR.R /* Port ingress stream identification configuration register */

/* ENETC_PSEUDO_MAC_PORT2 */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMSR NETC__ENETC0_PSEUDO_MAC_PORT.PPMSR.R /* Port pseudo MAC status register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMCR NETC__ENETC0_PSEUDO_MAC_PORT.PPMCR.R /* Port pseudo MAC configuration register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMROCR0 NETC__ENETC0_PSEUDO_MAC_PORT.PPMROCR[0].R /* Port pseudo MAC receive octets counter */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMROCR1 NETC__ENETC0_PSEUDO_MAC_PORT.PPMROCR[1].R /* Port pseudo MAC receive octets counter */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMRUFCR0 NETC__ENETC0_PSEUDO_MAC_PORT.PPMRUFCR[0].R /* Port pseudo MAC receive unicast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMRUFCR1 NETC__ENETC0_PSEUDO_MAC_PORT.PPMRUFCR[1].R /* Port pseudo MAC receive unicast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMRMFCR0 NETC__ENETC0_PSEUDO_MAC_PORT.PPMRMFCR[0].R /* Port pseudo MAC receive multicast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMRMFCR1 NETC__ENETC0_PSEUDO_MAC_PORT.PPMRMFCR[1].R /* Port pseudo MAC receive multicast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMRBFCR0 NETC__ENETC0_PSEUDO_MAC_PORT.PPMRBFCR[0].R /* Port pseudo MAC receive broadcast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMRBFCR1 NETC__ENETC0_PSEUDO_MAC_PORT.PPMRBFCR[1].R /* Port pseudo MAC receive broadcast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMTOCR0 NETC__ENETC0_PSEUDO_MAC_PORT.PPMTOCR[0].R /* Port pseudo MAC transmit octets counter */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMTOCR1 NETC__ENETC0_PSEUDO_MAC_PORT.PPMTOCR[1].R /* Port pseudo MAC transmit octets counter */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMTUFCR0 NETC__ENETC0_PSEUDO_MAC_PORT.PPMTUFCR[0].R /* Port pseudo MAC transmit unicast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMTUFCR1 NETC__ENETC0_PSEUDO_MAC_PORT.PPMTUFCR[1].R /* Port pseudo MAC transmit unicast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMTMFCR0 NETC__ENETC0_PSEUDO_MAC_PORT.PPMTMFCR[0].R /* Port pseudo MAC transmit multicast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMTMFCR1 NETC__ENETC0_PSEUDO_MAC_PORT.PPMTMFCR[1].R /* Port pseudo MAC transmit multicast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMTBFCR0 NETC__ENETC0_PSEUDO_MAC_PORT.PPMTBFCR[0].R /* Port pseudo MAC transmit broadcast frame counter register */
#define NETC__ENETC0_PSEUDO_MAC_PORT_PPMTBFCR1 NETC__ENETC0_PSEUDO_MAC_PORT.PPMTBFCR[1].R /* Port pseudo MAC transmit broadcast frame counter register */

/* ERM */
#define CE_ERM_2_CR0         CE_ERM_2.CR0.R                /* ERM Configuration Register 0 */
#define CE_ERM_2_CR1         CE_ERM_2.CR1.R                /* ERM Configuration Register 1 */
#define CE_ERM_2_CR2         CE_ERM_2.CR2.R                /* ERM Configuration Register 2 */
#define CE_ERM_2_SR0         CE_ERM_2.SR0.R                /* ERM Status Register 0 */
#define CE_ERM_2_SR1         CE_ERM_2.SR1.R                /* ERM Status Register 1 */
#define CE_ERM_2_SR2         CE_ERM_2.SR2.R                /* ERM Status Register 2 */
#define CE_ERM_2_EAR0        CE_ERM_2.EAR0.R               /* ERM Memory 0 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT0 CE_ERM_2.CORR_ERR_CNT0.R    /* ERM Memory 0 Correctable Error Count Register */
#define CE_ERM_2_EAR1        CE_ERM_2.EAR1.R               /* ERM Memory 1 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT1 CE_ERM_2.CORR_ERR_CNT1.R    /* ERM Memory 1 Correctable Error Count Register */
#define CE_ERM_2_EAR2        CE_ERM_2.EAR2.R               /* ERM Memory 2 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT2 CE_ERM_2.CORR_ERR_CNT2.R    /* ERM Memory 2 Correctable Error Count Register */
#define CE_ERM_2_EAR3        CE_ERM_2.EAR3.R               /* ERM Memory 3 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT3 CE_ERM_2.CORR_ERR_CNT3.R    /* ERM Memory 3 Correctable Error Count Register */
#define CE_ERM_2_EAR4        CE_ERM_2.EAR4.R               /* ERM Memory 4 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT4 CE_ERM_2.CORR_ERR_CNT4.R    /* ERM Memory 4 Correctable Error Count Register */
#define CE_ERM_2_EAR5        CE_ERM_2.EAR5.R               /* ERM Memory 5 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT5 CE_ERM_2.CORR_ERR_CNT5.R    /* ERM Memory 5 Correctable Error Count Register */
#define CE_ERM_2_EAR6        CE_ERM_2.EAR6.R               /* ERM Memory 6 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT6 CE_ERM_2.CORR_ERR_CNT6.R    /* ERM Memory 6 Correctable Error Count Register */
#define CE_ERM_2_EAR7        CE_ERM_2.EAR7.R               /* ERM Memory 7 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT7 CE_ERM_2.CORR_ERR_CNT7.R    /* ERM Memory 7 Correctable Error Count Register */
#define CE_ERM_2_EAR8        CE_ERM_2.EAR8.R               /* ERM Memory 8 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT8 CE_ERM_2.CORR_ERR_CNT8.R    /* ERM Memory 8 Correctable Error Count Register */
#define CE_ERM_2_EAR9        CE_ERM_2.EAR9.R               /* ERM Memory 9 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT9 CE_ERM_2.CORR_ERR_CNT9.R    /* ERM Memory 9 Correctable Error Count Register */
#define CE_ERM_2_EAR10       CE_ERM_2.EAR10.R              /* ERM Memory 10 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT10 CE_ERM_2.CORR_ERR_CNT10.R  /* ERM Memory 10 Correctable Error Count Register */
#define CE_ERM_2_EAR11       CE_ERM_2.EAR11.R              /* ERM Memory 11 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT11 CE_ERM_2.CORR_ERR_CNT11.R  /* ERM Memory 11 Correctable Error Count Register */
#define CE_ERM_2_EAR12       CE_ERM_2.EAR12.R              /* ERM Memory 12 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT12 CE_ERM_2.CORR_ERR_CNT12.R  /* ERM Memory 12 Correctable Error Count Register */
#define CE_ERM_2_EAR13       CE_ERM_2.EAR13.R              /* ERM Memory 13 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT13 CE_ERM_2.CORR_ERR_CNT13.R  /* ERM Memory 13 Correctable Error Count Register */
#define CE_ERM_2_EAR14       CE_ERM_2.EAR14.R              /* ERM Memory 14 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT14 CE_ERM_2.CORR_ERR_CNT14.R  /* ERM Memory 14 Correctable Error Count Register */
#define CE_ERM_2_EAR15       CE_ERM_2.EAR15.R              /* ERM Memory 15 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT15 CE_ERM_2.CORR_ERR_CNT15.R  /* ERM Memory 15 Correctable Error Count Register */
#define CE_ERM_2_EAR16       CE_ERM_2.EAR16.R              /* ERM Memory 16 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT16 CE_ERM_2.CORR_ERR_CNT16.R  /* ERM Memory 16 Correctable Error Count Register */
#define CE_ERM_2_EAR17       CE_ERM_2.EAR17.R              /* ERM Memory 17 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT17 CE_ERM_2.CORR_ERR_CNT17.R  /* ERM Memory 17 Correctable Error Count Register */
#define CE_ERM_2_EAR18       CE_ERM_2.EAR18.R              /* ERM Memory 18 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT18 CE_ERM_2.CORR_ERR_CNT18.R  /* ERM Memory 18 Correctable Error Count Register */
#define CE_ERM_2_EAR19       CE_ERM_2.EAR19.R              /* ERM Memory 19 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT19 CE_ERM_2.CORR_ERR_CNT19.R  /* ERM Memory 19 Correctable Error Count Register */
#define CE_ERM_2_EAR20       CE_ERM_2.EAR20.R              /* ERM Memory 20 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT20 CE_ERM_2.CORR_ERR_CNT20.R  /* ERM Memory 20 Correctable Error Count Register */
#define CE_ERM_2_EAR21       CE_ERM_2.EAR21.R              /* ERM Memory 21 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT21 CE_ERM_2.CORR_ERR_CNT21.R  /* ERM Memory 21 Correctable Error Count Register */
#define CE_ERM_2_EAR22       CE_ERM_2.EAR22.R              /* ERM Memory 22 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT22 CE_ERM_2.CORR_ERR_CNT22.R  /* ERM Memory 22 Correctable Error Count Register */
#define CE_ERM_2_EAR23       CE_ERM_2.EAR23.R              /* ERM Memory 23 Error Address Register */
#define CE_ERM_2_CORR_ERR_CNT23 CE_ERM_2.CORR_ERR_CNT23.R  /* ERM Memory 23 Correctable Error Count Register */

#define CE_ERM_0_CR0         CE_ERM_0.CR0.R                /* ERM Configuration Register 0 */
#define CE_ERM_0_CR1         CE_ERM_0.CR1.R                /* ERM Configuration Register 1 */
#define CE_ERM_0_CR2         CE_ERM_0.CR2.R                /* ERM Configuration Register 2 */
#define CE_ERM_0_SR0         CE_ERM_0.SR0.R                /* ERM Status Register 0 */
#define CE_ERM_0_SR1         CE_ERM_0.SR1.R                /* ERM Status Register 1 */
#define CE_ERM_0_SR2         CE_ERM_0.SR2.R                /* ERM Status Register 2 */
#define CE_ERM_0_EAR0        CE_ERM_0.EAR0.R               /* ERM Memory 0 Error Address Register */
#define CE_ERM_0_SYN0        CE_ERM_0.SYN0.R               /* ERM Memory 0 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT0 CE_ERM_0.CORR_ERR_CNT0.R    /* ERM Memory 0 Correctable Error Count Register */
#define CE_ERM_0_EAR1        CE_ERM_0.EAR1.R               /* ERM Memory 1 Error Address Register */
#define CE_ERM_0_SYN1        CE_ERM_0.SYN1.R               /* ERM Memory 1 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT1 CE_ERM_0.CORR_ERR_CNT1.R    /* ERM Memory 1 Correctable Error Count Register */
#define CE_ERM_0_EAR2        CE_ERM_0.EAR2.R               /* ERM Memory 2 Error Address Register */
#define CE_ERM_0_SYN2        CE_ERM_0.SYN2.R               /* ERM Memory 2 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT2 CE_ERM_0.CORR_ERR_CNT2.R    /* ERM Memory 2 Correctable Error Count Register */
#define CE_ERM_0_EAR3        CE_ERM_0.EAR3.R               /* ERM Memory 3 Error Address Register */
#define CE_ERM_0_CORR_ERR_CNT3 CE_ERM_0.CORR_ERR_CNT3.R    /* ERM Memory 3 Correctable Error Count Register */
#define CE_ERM_0_EAR5        CE_ERM_0.EAR5.R               /* ERM Memory 5 Error Address Register */
#define CE_ERM_0_SYN5        CE_ERM_0.SYN5.R               /* ERM Memory 5 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT5 CE_ERM_0.CORR_ERR_CNT5.R    /* ERM Memory 5 Correctable Error Count Register */
#define CE_ERM_0_EAR6        CE_ERM_0.EAR6.R               /* ERM Memory 6 Error Address Register */
#define CE_ERM_0_SYN6        CE_ERM_0.SYN6.R               /* ERM Memory 6 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT6 CE_ERM_0.CORR_ERR_CNT6.R    /* ERM Memory 6 Correctable Error Count Register */
#define CE_ERM_0_EAR7        CE_ERM_0.EAR7.R               /* ERM Memory 7 Error Address Register */
#define CE_ERM_0_SYN7        CE_ERM_0.SYN7.R               /* ERM Memory 7 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT7 CE_ERM_0.CORR_ERR_CNT7.R    /* ERM Memory 7 Correctable Error Count Register */
#define CE_ERM_0_EAR8        CE_ERM_0.EAR8.R               /* ERM Memory 8 Error Address Register */
#define CE_ERM_0_SYN8        CE_ERM_0.SYN8.R               /* ERM Memory 8 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT8 CE_ERM_0.CORR_ERR_CNT8.R    /* ERM Memory 8 Correctable Error Count Register */
#define CE_ERM_0_EAR9        CE_ERM_0.EAR9.R               /* ERM Memory 9 Error Address Register */
#define CE_ERM_0_SYN9        CE_ERM_0.SYN9.R               /* ERM Memory 9 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT9 CE_ERM_0.CORR_ERR_CNT9.R    /* ERM Memory 9 Correctable Error Count Register */
#define CE_ERM_0_EAR10       CE_ERM_0.EAR10.R              /* ERM Memory 10 Error Address Register */
#define CE_ERM_0_SYN10       CE_ERM_0.SYN10.R              /* ERM Memory 10 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT10 CE_ERM_0.CORR_ERR_CNT10.R  /* ERM Memory 10 Correctable Error Count Register */
#define CE_ERM_0_EAR11       CE_ERM_0.EAR11.R              /* ERM Memory 11 Error Address Register */
#define CE_ERM_0_SYN11       CE_ERM_0.SYN11.R              /* ERM Memory 11 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT11 CE_ERM_0.CORR_ERR_CNT11.R  /* ERM Memory 11 Correctable Error Count Register */
#define CE_ERM_0_EAR12       CE_ERM_0.EAR12.R              /* ERM Memory 12 Error Address Register */
#define CE_ERM_0_SYN12       CE_ERM_0.SYN12.R              /* ERM Memory 12 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT12 CE_ERM_0.CORR_ERR_CNT12.R  /* ERM Memory 12 Correctable Error Count Register */
#define CE_ERM_0_EAR13       CE_ERM_0.EAR13.R              /* ERM Memory 13 Error Address Register */
#define CE_ERM_0_SYN13       CE_ERM_0.SYN13.R              /* ERM Memory 13 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT13 CE_ERM_0.CORR_ERR_CNT13.R  /* ERM Memory 13 Correctable Error Count Register */
#define CE_ERM_0_EAR14       CE_ERM_0.EAR14.R              /* ERM Memory 14 Error Address Register */
#define CE_ERM_0_SYN14       CE_ERM_0.SYN14.R              /* ERM Memory 14 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT14 CE_ERM_0.CORR_ERR_CNT14.R  /* ERM Memory 14 Correctable Error Count Register */
#define CE_ERM_0_EAR15       CE_ERM_0.EAR15.R              /* ERM Memory 15 Error Address Register */
#define CE_ERM_0_CORR_ERR_CNT15 CE_ERM_0.CORR_ERR_CNT15.R  /* ERM Memory 15 Correctable Error Count Register */
#define CE_ERM_0_EAR16       CE_ERM_0.EAR16.R              /* ERM Memory 16 Error Address Register */
#define CE_ERM_0_CORR_ERR_CNT16 CE_ERM_0.CORR_ERR_CNT16.R  /* ERM Memory 16 Correctable Error Count Register */
#define CE_ERM_0_EAR17       CE_ERM_0.EAR17.R              /* ERM Memory 17 Error Address Register */
#define CE_ERM_0_CORR_ERR_CNT17 CE_ERM_0.CORR_ERR_CNT17.R  /* ERM Memory 17 Correctable Error Count Register */
#define CE_ERM_0_EAR18       CE_ERM_0.EAR18.R              /* ERM Memory 18 Error Address Register */
#define CE_ERM_0_SYN18       CE_ERM_0.SYN18.R              /* ERM Memory 18 Syndrome Register */
#define CE_ERM_0_CORR_ERR_CNT18 CE_ERM_0.CORR_ERR_CNT18.R  /* ERM Memory 18 Correctable Error Count Register */

#define CE_ERM_1_CR0         CE_ERM_1.CR0.R                /* ERM Configuration Register 0 */
#define CE_ERM_1_CR1         CE_ERM_1.CR1.R                /* ERM Configuration Register 1 */
#define CE_ERM_1_CR2         CE_ERM_1.CR2.R                /* ERM Configuration Register 2 */
#define CE_ERM_1_SR0         CE_ERM_1.SR0.R                /* ERM Status Register 0 */
#define CE_ERM_1_SR1         CE_ERM_1.SR1.R                /* ERM Status Register 1 */
#define CE_ERM_1_SR2         CE_ERM_1.SR2.R                /* ERM Status Register 2 */
#define CE_ERM_1_EAR5        CE_ERM_1.EAR5.R               /* ERM Memory 5 Error Address Register */
#define CE_ERM_1_CORR_ERR_CNT5 CE_ERM_1.CORR_ERR_CNT5.R    /* ERM Memory 5 Correctable Error Count Register */
#define CE_ERM_1_EAR6        CE_ERM_1.EAR6.R               /* ERM Memory 6 Error Address Register */
#define CE_ERM_1_SYN6        CE_ERM_1.SYN6.R               /* ERM Memory 6 Syndrome Register */
#define CE_ERM_1_CORR_ERR_CNT6 CE_ERM_1.CORR_ERR_CNT6.R    /* ERM Memory 6 Correctable Error Count Register */
#define CE_ERM_1_EAR7        CE_ERM_1.EAR7.R               /* ERM Memory 7 Error Address Register */
#define CE_ERM_1_SYN7        CE_ERM_1.SYN7.R               /* ERM Memory 7 Syndrome Register */
#define CE_ERM_1_CORR_ERR_CNT7 CE_ERM_1.CORR_ERR_CNT7.R    /* ERM Memory 7 Correctable Error Count Register */
#define CE_ERM_1_EAR8        CE_ERM_1.EAR8.R               /* ERM Memory 8 Error Address Register */
#define CE_ERM_1_SYN8        CE_ERM_1.SYN8.R               /* ERM Memory 8 Syndrome Register */
#define CE_ERM_1_CORR_ERR_CNT8 CE_ERM_1.CORR_ERR_CNT8.R    /* ERM Memory 8 Correctable Error Count Register */
#define CE_ERM_1_EAR9        CE_ERM_1.EAR9.R               /* ERM Memory 9 Error Address Register */
#define CE_ERM_1_SYN9        CE_ERM_1.SYN9.R               /* ERM Memory 9 Syndrome Register */
#define CE_ERM_1_CORR_ERR_CNT9 CE_ERM_1.CORR_ERR_CNT9.R    /* ERM Memory 9 Correctable Error Count Register */
#define CE_ERM_1_EAR10       CE_ERM_1.EAR10.R              /* ERM Memory 10 Error Address Register */
#define CE_ERM_1_CORR_ERR_CNT10 CE_ERM_1.CORR_ERR_CNT10.R  /* ERM Memory 10 Correctable Error Count Register */
#define CE_ERM_1_EAR11       CE_ERM_1.EAR11.R              /* ERM Memory 11 Error Address Register */
#define CE_ERM_1_CORR_ERR_CNT11 CE_ERM_1.CORR_ERR_CNT11.R  /* ERM Memory 11 Correctable Error Count Register */
#define CE_ERM_1_EAR12       CE_ERM_1.EAR12.R              /* ERM Memory 12 Error Address Register */
#define CE_ERM_1_CORR_ERR_CNT12 CE_ERM_1.CORR_ERR_CNT12.R  /* ERM Memory 12 Correctable Error Count Register */
#define CE_ERM_1_EAR16       CE_ERM_1.EAR16.R              /* ERM Memory 16 Error Address Register */
#define CE_ERM_1_CORR_ERR_CNT16 CE_ERM_1.CORR_ERR_CNT16.R  /* ERM Memory 16 Correctable Error Count Register */
#define CE_ERM_1_EAR20       CE_ERM_1.EAR20.R              /* ERM Memory 20 Error Address Register */
#define CE_ERM_1_SYN20       CE_ERM_1.SYN20.R              /* ERM Memory 20 Syndrome Register */
#define CE_ERM_1_CORR_ERR_CNT20 CE_ERM_1.CORR_ERR_CNT20.R  /* ERM Memory 20 Correctable Error Count Register */

#define ERM_3_CR0            ERM_3.CR0.R                   /* ERM Configuration Register 0 */
#define ERM_3_CR1            ERM_3.CR1.R                   /* ERM Configuration Register 1 */
#define ERM_3_CR2            ERM_3.CR2.R                   /* ERM Configuration Register 2 */
#define ERM_3_SR0            ERM_3.SR0.R                   /* ERM Status Register 0 */
#define ERM_3_SR1            ERM_3.SR1.R                   /* ERM Status Register 1 */
#define ERM_3_SR2            ERM_3.SR2.R                   /* ERM Status Register 2 */
#define ERM_3_EAR0           ERM_3.EAR0.R                  /* ERM Memory 0 Error Address Register */
#define ERM_3_SYN0           ERM_3.SYN0.R                  /* ERM Memory 0 Syndrome Register */
#define ERM_3_CORR_ERR_CNT0  ERM_3.CORR_ERR_CNT0.R         /* ERM Memory 0 Correctable Error Count Register */
#define ERM_3_EAR1           ERM_3.EAR1.R                  /* ERM Memory 1 Error Address Register */
#define ERM_3_SYN1           ERM_3.SYN1.R                  /* ERM Memory 1 Syndrome Register */
#define ERM_3_CORR_ERR_CNT1  ERM_3.CORR_ERR_CNT1.R         /* ERM Memory 1 Correctable Error Count Register */
#define ERM_3_EAR2           ERM_3.EAR2.R                  /* ERM Memory 2 Error Address Register */
#define ERM_3_SYN2           ERM_3.SYN2.R                  /* ERM Memory 2 Syndrome Register */
#define ERM_3_CORR_ERR_CNT2  ERM_3.CORR_ERR_CNT2.R         /* ERM Memory 2 Correctable Error Count Register */
#define ERM_3_EAR3           ERM_3.EAR3.R                  /* ERM Memory 3 Error Address Register */
#define ERM_3_SYN3           ERM_3.SYN3.R                  /* ERM Memory 3 Syndrome Register */
#define ERM_3_CORR_ERR_CNT3  ERM_3.CORR_ERR_CNT3.R         /* ERM Memory 3 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT4  ERM_3.CORR_ERR_CNT4.R         /* ERM Memory 4 Correctable Error Count Register */
#define ERM_3_EAR5           ERM_3.EAR5.R                  /* ERM Memory 5 Error Address Register */
#define ERM_3_SYN5           ERM_3.SYN5.R                  /* ERM Memory 5 Syndrome Register */
#define ERM_3_CORR_ERR_CNT5  ERM_3.CORR_ERR_CNT5.R         /* ERM Memory 5 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT6  ERM_3.CORR_ERR_CNT6.R         /* ERM Memory 6 Correctable Error Count Register */
#define ERM_3_EAR7           ERM_3.EAR7.R                  /* ERM Memory 7 Error Address Register */
#define ERM_3_CORR_ERR_CNT7  ERM_3.CORR_ERR_CNT7.R         /* ERM Memory 7 Correctable Error Count Register */
#define ERM_3_EAR8           ERM_3.EAR8.R                  /* ERM Memory 8 Error Address Register */
#define ERM_3_CORR_ERR_CNT8  ERM_3.CORR_ERR_CNT8.R         /* ERM Memory 8 Correctable Error Count Register */
#define ERM_3_EAR9           ERM_3.EAR9.R                  /* ERM Memory 9 Error Address Register */
#define ERM_3_SYN9           ERM_3.SYN9.R                  /* ERM Memory 9 Syndrome Register */
#define ERM_3_CORR_ERR_CNT9  ERM_3.CORR_ERR_CNT9.R         /* ERM Memory 9 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT10 ERM_3.CORR_ERR_CNT10.R        /* ERM Memory 10 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT11 ERM_3.CORR_ERR_CNT11.R        /* ERM Memory 11 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT12 ERM_3.CORR_ERR_CNT12.R        /* ERM Memory 12 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT13 ERM_3.CORR_ERR_CNT13.R        /* ERM Memory 13 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT14 ERM_3.CORR_ERR_CNT14.R        /* ERM Memory 14 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT15 ERM_3.CORR_ERR_CNT15.R        /* ERM Memory 15 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT16 ERM_3.CORR_ERR_CNT16.R        /* ERM Memory 16 Correctable Error Count Register */
#define ERM_3_CORR_ERR_CNT17 ERM_3.CORR_ERR_CNT17.R        /* ERM Memory 17 Correctable Error Count Register */

#define ERM_1_CR0            ERM_1.CR0.R                   /* ERM Configuration Register 0 */
#define ERM_1_SR0            ERM_1.SR0.R                   /* ERM Status Register 0 */
#define ERM_1_EAR0           ERM_1.EAR0.R                  /* ERM Memory 0 Error Address Register */
#define ERM_1_SYN0           ERM_1.SYN0.R                  /* ERM Memory 0 Syndrome Register */
#define ERM_1_CORR_ERR_CNT0  ERM_1.CORR_ERR_CNT0.R         /* ERM Memory 0 Correctable Error Count Register */

#define SMU__ERM_CR0         SMU__ERM.CR0.R                /* ERM Configuration Register 0 */
#define SMU__ERM_CR1         SMU__ERM.CR1.R                /* ERM Configuration Register 1 */
#define SMU__ERM_CR2         SMU__ERM.CR2.R                /* ERM Configuration Register 2 */
#define SMU__ERM_SR0         SMU__ERM.SR0.R                /* ERM Status Register 0 */
#define SMU__ERM_SR1         SMU__ERM.SR1.R                /* ERM Status Register 1 */
#define SMU__ERM_SR2         SMU__ERM.SR2.R                /* ERM Status Register 2 */
#define SMU__ERM_EAR0        SMU__ERM.EAR0.R               /* ERM Memory 0 Error Address Register */
#define SMU__ERM_SYN0        SMU__ERM.SYN0.R               /* ERM Memory 0 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT0 SMU__ERM.CORR_ERR_CNT0.R    /* ERM Memory 0 Correctable Error Count Register */
#define SMU__ERM_EAR1        SMU__ERM.EAR1.R               /* ERM Memory 1 Error Address Register */
#define SMU__ERM_SYN1        SMU__ERM.SYN1.R               /* ERM Memory 1 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT1 SMU__ERM.CORR_ERR_CNT1.R    /* ERM Memory 1 Correctable Error Count Register */
#define SMU__ERM_EAR2        SMU__ERM.EAR2.R               /* ERM Memory 2 Error Address Register */
#define SMU__ERM_SYN2        SMU__ERM.SYN2.R               /* ERM Memory 2 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT2 SMU__ERM.CORR_ERR_CNT2.R    /* ERM Memory 2 Correctable Error Count Register */
#define SMU__ERM_EAR3        SMU__ERM.EAR3.R               /* ERM Memory 3 Error Address Register */
#define SMU__ERM_SYN3        SMU__ERM.SYN3.R               /* ERM Memory 3 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT3 SMU__ERM.CORR_ERR_CNT3.R    /* ERM Memory 3 Correctable Error Count Register */
#define SMU__ERM_EAR5        SMU__ERM.EAR5.R               /* ERM Memory 5 Error Address Register */
#define SMU__ERM_SYN5        SMU__ERM.SYN5.R               /* ERM Memory 5 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT5 SMU__ERM.CORR_ERR_CNT5.R    /* ERM Memory 5 Correctable Error Count Register */
#define SMU__ERM_EAR6        SMU__ERM.EAR6.R               /* ERM Memory 6 Error Address Register */
#define SMU__ERM_SYN6        SMU__ERM.SYN6.R               /* ERM Memory 6 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT6 SMU__ERM.CORR_ERR_CNT6.R    /* ERM Memory 6 Correctable Error Count Register */
#define SMU__ERM_EAR7        SMU__ERM.EAR7.R               /* ERM Memory 7 Error Address Register */
#define SMU__ERM_SYN7        SMU__ERM.SYN7.R               /* ERM Memory 7 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT7 SMU__ERM.CORR_ERR_CNT7.R    /* ERM Memory 7 Correctable Error Count Register */
#define SMU__ERM_EAR8        SMU__ERM.EAR8.R               /* ERM Memory 8 Error Address Register */
#define SMU__ERM_SYN8        SMU__ERM.SYN8.R               /* ERM Memory 8 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT8 SMU__ERM.CORR_ERR_CNT8.R    /* ERM Memory 8 Correctable Error Count Register */
#define SMU__ERM_EAR9        SMU__ERM.EAR9.R               /* ERM Memory 9 Error Address Register */
#define SMU__ERM_SYN9        SMU__ERM.SYN9.R               /* ERM Memory 9 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT9 SMU__ERM.CORR_ERR_CNT9.R    /* ERM Memory 9 Correctable Error Count Register */
#define SMU__ERM_EAR10       SMU__ERM.EAR10.R              /* ERM Memory 10 Error Address Register */
#define SMU__ERM_SYN10       SMU__ERM.SYN10.R              /* ERM Memory 10 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT10 SMU__ERM.CORR_ERR_CNT10.R  /* ERM Memory 10 Correctable Error Count Register */
#define SMU__ERM_EAR11       SMU__ERM.EAR11.R              /* ERM Memory 11 Error Address Register */
#define SMU__ERM_SYN11       SMU__ERM.SYN11.R              /* ERM Memory 11 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT11 SMU__ERM.CORR_ERR_CNT11.R  /* ERM Memory 11 Correctable Error Count Register */
#define SMU__ERM_EAR12       SMU__ERM.EAR12.R              /* ERM Memory 12 Error Address Register */
#define SMU__ERM_SYN12       SMU__ERM.SYN12.R              /* ERM Memory 12 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT12 SMU__ERM.CORR_ERR_CNT12.R  /* ERM Memory 12 Correctable Error Count Register */
#define SMU__ERM_EAR13       SMU__ERM.EAR13.R              /* ERM Memory 13 Error Address Register */
#define SMU__ERM_SYN13       SMU__ERM.SYN13.R              /* ERM Memory 13 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT13 SMU__ERM.CORR_ERR_CNT13.R  /* ERM Memory 13 Correctable Error Count Register */
#define SMU__ERM_EAR14       SMU__ERM.EAR14.R              /* ERM Memory 14 Error Address Register */
#define SMU__ERM_SYN14       SMU__ERM.SYN14.R              /* ERM Memory 14 Syndrome Register */
#define SMU__ERM_CORR_ERR_CNT14 SMU__ERM.CORR_ERR_CNT14.R  /* ERM Memory 14 Correctable Error Count Register */
#define SMU__ERM_EAR15       SMU__ERM.EAR15.R              /* ERM Memory 15 Error Address Register */
#define SMU__ERM_CORR_ERR_CNT15 SMU__ERM.CORR_ERR_CNT15.R  /* ERM Memory 15 Correctable Error Count Register */
#define SMU__ERM_EAR16       SMU__ERM.EAR16.R              /* ERM Memory 16 Error Address Register */
#define SMU__ERM_CORR_ERR_CNT16 SMU__ERM.CORR_ERR_CNT16.R  /* ERM Memory 16 Correctable Error Count Register */
#define SMU__ERM_EAR17       SMU__ERM.EAR17.R              /* ERM Memory 17 Error Address Register */
#define SMU__ERM_CORR_ERR_CNT17 SMU__ERM.CORR_ERR_CNT17.R  /* ERM Memory 17 Correctable Error Count Register */
#define SMU__ERM_CORR_ERR_CNT18 SMU__ERM.CORR_ERR_CNT18.R  /* ERM Memory 18 Correctable Error Count Register */

#define ERM_2_CR0            ERM_2.CR0.R                   /* ERM Configuration Register 0 */
#define ERM_2_SR0            ERM_2.SR0.R                   /* ERM Status Register 0 */
#define ERM_2_CORR_ERR_CNT0  ERM_2.CORR_ERR_CNT0.R         /* ERM Memory 0 Correctable Error Count Register */
#define ERM_2_CORR_ERR_CNT1  ERM_2.CORR_ERR_CNT1.R         /* ERM Memory 1 Correctable Error Count Register */
#define ERM_2_CORR_ERR_CNT2  ERM_2.CORR_ERR_CNT2.R         /* ERM Memory 2 Correctable Error Count Register */
#define ERM_2_CORR_ERR_CNT3  ERM_2.CORR_ERR_CNT3.R         /* ERM Memory 3 Correctable Error Count Register */
#define ERM_2_CORR_ERR_CNT4  ERM_2.CORR_ERR_CNT4.R         /* ERM Memory 4 Correctable Error Count Register */
#define ERM_2_CORR_ERR_CNT5  ERM_2.CORR_ERR_CNT5.R         /* ERM Memory 5 Correctable Error Count Register */
#define ERM_2_CORR_ERR_CNT6  ERM_2.CORR_ERR_CNT6.R         /* ERM Memory 6 Correctable Error Count Register */
#define ERM_2_CORR_ERR_CNT7  ERM_2.CORR_ERR_CNT7.R         /* ERM Memory 7 Correctable Error Count Register */

#define ERM_0_CR0            ERM_0.CR0.R                   /* ERM Configuration Register 0 */
#define ERM_0_SR0            ERM_0.SR0.R                   /* ERM Status Register 0 */
#define ERM_0_EAR0           ERM_0.EAR0.R                  /* ERM Memory 0 Error Address Register */
#define ERM_0_SYN0           ERM_0.SYN0.R                  /* ERM Memory 0 Syndrome Register */
#define ERM_0_CORR_ERR_CNT0  ERM_0.CORR_ERR_CNT0.R         /* ERM Memory 0 Correctable Error Count Register */

#define ERM_4_CR0            ERM_4.CR0.R                   /* ERM Configuration Register 0 */
#define ERM_4_SR0            ERM_4.SR0.R                   /* ERM Status Register 0 */
#define ERM_4_EAR0           ERM_4.EAR0.R                  /* ERM Memory 0 Error Address Register */
#define ERM_4_SYN0           ERM_4.SYN0.R                  /* ERM Memory 0 Syndrome Register */
#define ERM_4_CORR_ERR_CNT0  ERM_4.CORR_ERR_CNT0.R         /* ERM Memory 0 Correctable Error Count Register */
#define ERM_4_CORR_ERR_CNT1  ERM_4.CORR_ERR_CNT1.R         /* ERM Memory 1 Correctable Error Count Register */
#define ERM_4_CORR_ERR_CNT2  ERM_4.CORR_ERR_CNT2.R         /* ERM Memory 2 Correctable Error Count Register */
#define ERM_4_CORR_ERR_CNT3  ERM_4.CORR_ERR_CNT3.R         /* ERM Memory 3 Correctable Error Count Register */
#define ERM_4_CORR_ERR_CNT4  ERM_4.CORR_ERR_CNT4.R         /* ERM Memory 4 Correctable Error Count Register */

#define ERM_5_CR0            ERM_5.CR0.R                   /* ERM Configuration Register 0 */
#define ERM_5_SR0            ERM_5.SR0.R                   /* ERM Status Register 0 */
#define ERM_5_EAR0           ERM_5.EAR0.R                  /* ERM Memory 0 Error Address Register */
#define ERM_5_SYN0           ERM_5.SYN0.R                  /* ERM Memory 0 Syndrome Register */
#define ERM_5_CORR_ERR_CNT0  ERM_5.CORR_ERR_CNT0.R         /* ERM Memory 0 Correctable Error Count Register */
#define ERM_5_EAR1           ERM_5.EAR1.R                  /* ERM Memory 1 Error Address Register */
#define ERM_5_CORR_ERR_CNT1  ERM_5.CORR_ERR_CNT1.R         /* ERM Memory 1 Correctable Error Count Register */
#define ERM_5_EAR2           ERM_5.EAR2.R                  /* ERM Memory 2 Error Address Register */
#define ERM_5_CORR_ERR_CNT2  ERM_5.CORR_ERR_CNT2.R         /* ERM Memory 2 Correctable Error Count Register */
#define ERM_5_EAR3           ERM_5.EAR3.R                  /* ERM Memory 3 Error Address Register */
#define ERM_5_CORR_ERR_CNT3  ERM_5.CORR_ERR_CNT3.R         /* ERM Memory 3 Correctable Error Count Register */
#define ERM_5_EAR4           ERM_5.EAR4.R                  /* ERM Memory 4 Error Address Register */
#define ERM_5_CORR_ERR_CNT4  ERM_5.CORR_ERR_CNT4.R         /* ERM Memory 4 Correctable Error Count Register */

/* ERM_GTM */
#define ERM_GTM_CR0          ERM_GTM.CR0.R                 /* ERM Configuration Register 0 */
#define ERM_GTM_CR1          ERM_GTM.CR1.R                 /* ERM Configuration Register 1 */
#define ERM_GTM_SR0          ERM_GTM.SR0.R                 /* ERM Status Register 0 */
#define ERM_GTM_SR1          ERM_GTM.SR1.R                 /* ERM Status Register 1 */
#define ERM_GTM_EAR0         ERM_GTM.EAR0.R                /* ERM Memory 0 Error Address Register */
#define ERM_GTM_SYN0         ERM_GTM.SYN0.R                /* ERM Memory 0 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT0 ERM_GTM.CORR_ERR_CNT0.R      /* ERM Memory 0 Correctable Error Count Register */
#define ERM_GTM_EAR1         ERM_GTM.EAR1.R                /* ERM Memory 1 Error Address Register */
#define ERM_GTM_SYN1         ERM_GTM.SYN1.R                /* ERM Memory 1 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT1 ERM_GTM.CORR_ERR_CNT1.R      /* ERM Memory 1 Correctable Error Count Register */
#define ERM_GTM_EAR2         ERM_GTM.EAR2.R                /* ERM Memory 2 Error Address Register */
#define ERM_GTM_SYN2         ERM_GTM.SYN2.R                /* ERM Memory 2 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT2 ERM_GTM.CORR_ERR_CNT2.R      /* ERM Memory 2 Correctable Error Count Register */
#define ERM_GTM_EAR3         ERM_GTM.EAR3.R                /* ERM Memory 3 Error Address Register */
#define ERM_GTM_SYN3         ERM_GTM.SYN3.R                /* ERM Memory 3 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT3 ERM_GTM.CORR_ERR_CNT3.R      /* ERM Memory 3 Correctable Error Count Register */
#define ERM_GTM_EAR4         ERM_GTM.EAR4.R                /* ERM Memory 4 Error Address Register */
#define ERM_GTM_SYN4         ERM_GTM.SYN4.R                /* ERM Memory 4 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT4 ERM_GTM.CORR_ERR_CNT4.R      /* ERM Memory 4 Correctable Error Count Register */
#define ERM_GTM_EAR5         ERM_GTM.EAR5.R                /* ERM Memory 5 Error Address Register */
#define ERM_GTM_SYN5         ERM_GTM.SYN5.R                /* ERM Memory 5 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT5 ERM_GTM.CORR_ERR_CNT5.R      /* ERM Memory 5 Correctable Error Count Register */
#define ERM_GTM_EAR6         ERM_GTM.EAR6.R                /* ERM Memory 6 Error Address Register */
#define ERM_GTM_SYN6         ERM_GTM.SYN6.R                /* ERM Memory 6 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT6 ERM_GTM.CORR_ERR_CNT6.R      /* ERM Memory 6 Correctable Error Count Register */
#define ERM_GTM_EAR7         ERM_GTM.EAR7.R                /* ERM Memory 7 Error Address Register */
#define ERM_GTM_SYN7         ERM_GTM.SYN7.R                /* ERM Memory 7 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT7 ERM_GTM.CORR_ERR_CNT7.R      /* ERM Memory 7 Correctable Error Count Register */
#define ERM_GTM_EAR8         ERM_GTM.EAR8.R                /* ERM Memory 8 Error Address Register */
#define ERM_GTM_SYN8         ERM_GTM.SYN8.R                /* ERM Memory 8 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT8 ERM_GTM.CORR_ERR_CNT8.R      /* ERM Memory 8 Correctable Error Count Register */
#define ERM_GTM_EAR9         ERM_GTM.EAR9.R                /* ERM Memory 9 Error Address Register */
#define ERM_GTM_SYN9         ERM_GTM.SYN9.R                /* ERM Memory 9 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT9 ERM_GTM.CORR_ERR_CNT9.R      /* ERM Memory 9 Correctable Error Count Register */
#define ERM_GTM_EAR10        ERM_GTM.EAR10.R               /* ERM Memory 10 Error Address Register */
#define ERM_GTM_SYN10        ERM_GTM.SYN10.R               /* ERM Memory 10 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT10 ERM_GTM.CORR_ERR_CNT10.R    /* ERM Memory 10 Correctable Error Count Register */
#define ERM_GTM_EAR11        ERM_GTM.EAR11.R               /* ERM Memory 11 Error Address Register */
#define ERM_GTM_SYN11        ERM_GTM.SYN11.R               /* ERM Memory 11 Syndrome Register */
#define ERM_GTM_CORR_ERR_CNT11 ERM_GTM.CORR_ERR_CNT11.R    /* ERM Memory 11 Correctable Error Count Register */

/* ERROR_CORRECTION_CODES */
#define CEVA_SPF2__ERROR_CORRECTION_CODES_D_ECCCOR CEVA_SPF2__ERROR_CORRECTION_CODES.D_ECCCOR.R /* ECC Correctable Error Address */
#define CEVA_SPF2__ERROR_CORRECTION_CODES_D_ECCERR CEVA_SPF2__ERROR_CORRECTION_CODES.D_ECCERR.R /* ECC Uncorrectable Error Address */

/* FEED_DMACRC */
#define AES__FEED_DMACRC_GEC AES__FEED_DMACRC.GEC.R        /* Global Enable CRC Register */
#define AES__FEED_DMACRC_CTL0 AES__FEED_DMACRC.CONTROL_REGISTER[0].CTL.R /* CRC Control Register */
#define AES__FEED_DMACRC_ICRC0 AES__FEED_DMACRC.CONTROL_REGISTER[0].ICRC.R /* Initial CRC Value Register */
#define AES__FEED_DMACRC_FCRC0 AES__FEED_DMACRC.CONTROL_REGISTER[0].FCRC.R /* Final CRC Value Register */
#define AES__FEED_DMACRC_CTL1 AES__FEED_DMACRC.CONTROL_REGISTER[1].CTL.R /* CRC Control Register */
#define AES__FEED_DMACRC_ICRC1 AES__FEED_DMACRC.CONTROL_REGISTER[1].ICRC.R /* Initial CRC Value Register */
#define AES__FEED_DMACRC_FCRC1 AES__FEED_DMACRC.CONTROL_REGISTER[1].FCRC.R /* Final CRC Value Register */
#define AES__FEED_DMACRC_CTL2 AES__FEED_DMACRC.CONTROL_REGISTER[2].CTL.R /* CRC Control Register */
#define AES__FEED_DMACRC_ICRC2 AES__FEED_DMACRC.CONTROL_REGISTER[2].ICRC.R /* Initial CRC Value Register */
#define AES__FEED_DMACRC_FCRC2 AES__FEED_DMACRC.CONTROL_REGISTER[2].FCRC.R /* Final CRC Value Register */
#define AES__FEED_DMACRC_CTL3 AES__FEED_DMACRC.CONTROL_REGISTER[3].CTL.R /* CRC Control Register */
#define AES__FEED_DMACRC_ICRC3 AES__FEED_DMACRC.CONTROL_REGISTER[3].ICRC.R /* Initial CRC Value Register */
#define AES__FEED_DMACRC_FCRC3 AES__FEED_DMACRC.CONTROL_REGISTER[3].FCRC.R /* Final CRC Value Register */
#define AES__FEED_DMACRC_CTL4 AES__FEED_DMACRC.CONTROL_REGISTER[4].CTL.R /* CRC Control Register */
#define AES__FEED_DMACRC_ICRC4 AES__FEED_DMACRC.CONTROL_REGISTER[4].ICRC.R /* Initial CRC Value Register */
#define AES__FEED_DMACRC_FCRC4 AES__FEED_DMACRC.CONTROL_REGISTER[4].FCRC.R /* Final CRC Value Register */
#define AES__FEED_DMACRC_CTL5 AES__FEED_DMACRC.CONTROL_REGISTER[5].CTL.R /* CRC Control Register */
#define AES__FEED_DMACRC_ICRC5 AES__FEED_DMACRC.CONTROL_REGISTER[5].ICRC.R /* Initial CRC Value Register */
#define AES__FEED_DMACRC_FCRC5 AES__FEED_DMACRC.CONTROL_REGISTER[5].FCRC.R /* Final CRC Value Register */
#define AES__FEED_DMACRC_CTL6 AES__FEED_DMACRC.CONTROL_REGISTER[6].CTL.R /* CRC Control Register */
#define AES__FEED_DMACRC_ICRC6 AES__FEED_DMACRC.CONTROL_REGISTER[6].ICRC.R /* Initial CRC Value Register */
#define AES__FEED_DMACRC_FCRC6 AES__FEED_DMACRC.CONTROL_REGISTER[6].FCRC.R /* Final CRC Value Register */
#define AES__FEED_DMACRC_CTL7 AES__FEED_DMACRC.CONTROL_REGISTER[7].CTL.R /* CRC Control Register */
#define AES__FEED_DMACRC_ICRC7 AES__FEED_DMACRC.CONTROL_REGISTER[7].ICRC.R /* Initial CRC Value Register */
#define AES__FEED_DMACRC_FCRC7 AES__FEED_DMACRC.CONTROL_REGISTER[7].FCRC.R /* Final CRC Value Register */

/* FEED_DMA_MP */
#define AES__FEED_DMA_MP_CSR AES__FEED_DMA_MP.CSR.R        /* Management Page Control */
#define AES__FEED_DMA_MP_ES  AES__FEED_DMA_MP.ES.R         /* Management Page Error Status */
#define AES__FEED_DMA_MP_INT AES__FEED_DMA_MP.INT.R        /* Management Page Interrupt Request Status */
#define AES__FEED_DMA_MP_HRS AES__FEED_DMA_MP.HRS.R        /* Management Page Hardware Request Status */
#define AES__FEED_DMA_MP_CH0_GRPRI AES__FEED_DMA_MP.CH_GRPRI[0].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH1_GRPRI AES__FEED_DMA_MP.CH_GRPRI[1].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH2_GRPRI AES__FEED_DMA_MP.CH_GRPRI[2].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH3_GRPRI AES__FEED_DMA_MP.CH_GRPRI[3].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH4_GRPRI AES__FEED_DMA_MP.CH_GRPRI[4].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH5_GRPRI AES__FEED_DMA_MP.CH_GRPRI[5].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH6_GRPRI AES__FEED_DMA_MP.CH_GRPRI[6].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH7_GRPRI AES__FEED_DMA_MP.CH_GRPRI[7].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH8_GRPRI AES__FEED_DMA_MP.CH_GRPRI[8].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH9_GRPRI AES__FEED_DMA_MP.CH_GRPRI[9].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH10_GRPRI AES__FEED_DMA_MP.CH_GRPRI[10].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH11_GRPRI AES__FEED_DMA_MP.CH_GRPRI[11].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH12_GRPRI AES__FEED_DMA_MP.CH_GRPRI[12].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH13_GRPRI AES__FEED_DMA_MP.CH_GRPRI[13].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH14_GRPRI AES__FEED_DMA_MP.CH_GRPRI[14].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH15_GRPRI AES__FEED_DMA_MP.CH_GRPRI[15].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH16_GRPRI AES__FEED_DMA_MP.CH_GRPRI[16].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH17_GRPRI AES__FEED_DMA_MP.CH_GRPRI[17].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH18_GRPRI AES__FEED_DMA_MP.CH_GRPRI[18].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH19_GRPRI AES__FEED_DMA_MP.CH_GRPRI[19].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH20_GRPRI AES__FEED_DMA_MP.CH_GRPRI[20].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH21_GRPRI AES__FEED_DMA_MP.CH_GRPRI[21].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH22_GRPRI AES__FEED_DMA_MP.CH_GRPRI[22].R /* Channel Arbitration Group */
#define AES__FEED_DMA_MP_CH23_GRPRI AES__FEED_DMA_MP.CH_GRPRI[23].R /* Channel Arbitration Group */

/* FEED_DMA_TCD */
#define AES__FEED_DMA_TCD_CH0_CSR AES__FEED_DMA_TCD.TCD[0].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH0_ES AES__FEED_DMA_TCD.TCD[0].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH0_INT AES__FEED_DMA_TCD.TCD[0].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH0_SBR AES__FEED_DMA_TCD.TCD[0].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH0_PRI AES__FEED_DMA_TCD.TCD[0].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD0_SADDR AES__FEED_DMA_TCD.TCD[0].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD0_SOFF AES__FEED_DMA_TCD.TCD[0].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD0_ATTR AES__FEED_DMA_TCD.TCD[0].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD0_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[0].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD0_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[0].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD0_SLAST_SDA AES__FEED_DMA_TCD.TCD[0].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD0_DADDR AES__FEED_DMA_TCD.TCD[0].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD0_DOFF AES__FEED_DMA_TCD.TCD[0].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD0_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[0].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD0_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[0].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD0_DLAST_SGA AES__FEED_DMA_TCD.TCD[0].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD0_CSR AES__FEED_DMA_TCD.TCD[0].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD0_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[0].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD0_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[0].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH1_CSR AES__FEED_DMA_TCD.TCD[1].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH1_ES AES__FEED_DMA_TCD.TCD[1].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH1_INT AES__FEED_DMA_TCD.TCD[1].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH1_SBR AES__FEED_DMA_TCD.TCD[1].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH1_PRI AES__FEED_DMA_TCD.TCD[1].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD1_SADDR AES__FEED_DMA_TCD.TCD[1].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD1_SOFF AES__FEED_DMA_TCD.TCD[1].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD1_ATTR AES__FEED_DMA_TCD.TCD[1].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD1_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[1].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD1_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[1].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD1_SLAST_SDA AES__FEED_DMA_TCD.TCD[1].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD1_DADDR AES__FEED_DMA_TCD.TCD[1].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD1_DOFF AES__FEED_DMA_TCD.TCD[1].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD1_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[1].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD1_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[1].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD1_DLAST_SGA AES__FEED_DMA_TCD.TCD[1].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD1_CSR AES__FEED_DMA_TCD.TCD[1].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD1_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[1].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD1_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[1].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH2_CSR AES__FEED_DMA_TCD.TCD[2].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH2_ES AES__FEED_DMA_TCD.TCD[2].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH2_INT AES__FEED_DMA_TCD.TCD[2].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH2_SBR AES__FEED_DMA_TCD.TCD[2].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH2_PRI AES__FEED_DMA_TCD.TCD[2].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD2_SADDR AES__FEED_DMA_TCD.TCD[2].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD2_SOFF AES__FEED_DMA_TCD.TCD[2].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD2_ATTR AES__FEED_DMA_TCD.TCD[2].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD2_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[2].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD2_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[2].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD2_SLAST_SDA AES__FEED_DMA_TCD.TCD[2].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD2_DADDR AES__FEED_DMA_TCD.TCD[2].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD2_DOFF AES__FEED_DMA_TCD.TCD[2].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD2_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[2].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD2_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[2].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD2_DLAST_SGA AES__FEED_DMA_TCD.TCD[2].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD2_CSR AES__FEED_DMA_TCD.TCD[2].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD2_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[2].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD2_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[2].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH3_CSR AES__FEED_DMA_TCD.TCD[3].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH3_ES AES__FEED_DMA_TCD.TCD[3].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH3_INT AES__FEED_DMA_TCD.TCD[3].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH3_SBR AES__FEED_DMA_TCD.TCD[3].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH3_PRI AES__FEED_DMA_TCD.TCD[3].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD3_SADDR AES__FEED_DMA_TCD.TCD[3].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD3_SOFF AES__FEED_DMA_TCD.TCD[3].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD3_ATTR AES__FEED_DMA_TCD.TCD[3].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD3_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[3].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD3_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[3].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD3_SLAST_SDA AES__FEED_DMA_TCD.TCD[3].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD3_DADDR AES__FEED_DMA_TCD.TCD[3].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD3_DOFF AES__FEED_DMA_TCD.TCD[3].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD3_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[3].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD3_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[3].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD3_DLAST_SGA AES__FEED_DMA_TCD.TCD[3].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD3_CSR AES__FEED_DMA_TCD.TCD[3].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD3_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[3].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD3_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[3].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH4_CSR AES__FEED_DMA_TCD.TCD[4].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH4_ES AES__FEED_DMA_TCD.TCD[4].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH4_INT AES__FEED_DMA_TCD.TCD[4].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH4_SBR AES__FEED_DMA_TCD.TCD[4].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH4_PRI AES__FEED_DMA_TCD.TCD[4].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD4_SADDR AES__FEED_DMA_TCD.TCD[4].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD4_SOFF AES__FEED_DMA_TCD.TCD[4].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD4_ATTR AES__FEED_DMA_TCD.TCD[4].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD4_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[4].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD4_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[4].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD4_SLAST_SDA AES__FEED_DMA_TCD.TCD[4].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD4_DADDR AES__FEED_DMA_TCD.TCD[4].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD4_DOFF AES__FEED_DMA_TCD.TCD[4].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD4_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[4].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD4_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[4].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD4_DLAST_SGA AES__FEED_DMA_TCD.TCD[4].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD4_CSR AES__FEED_DMA_TCD.TCD[4].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD4_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[4].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD4_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[4].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH5_CSR AES__FEED_DMA_TCD.TCD[5].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH5_ES AES__FEED_DMA_TCD.TCD[5].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH5_INT AES__FEED_DMA_TCD.TCD[5].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH5_SBR AES__FEED_DMA_TCD.TCD[5].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH5_PRI AES__FEED_DMA_TCD.TCD[5].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD5_SADDR AES__FEED_DMA_TCD.TCD[5].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD5_SOFF AES__FEED_DMA_TCD.TCD[5].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD5_ATTR AES__FEED_DMA_TCD.TCD[5].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD5_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[5].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD5_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[5].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD5_SLAST_SDA AES__FEED_DMA_TCD.TCD[5].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD5_DADDR AES__FEED_DMA_TCD.TCD[5].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD5_DOFF AES__FEED_DMA_TCD.TCD[5].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD5_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[5].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD5_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[5].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD5_DLAST_SGA AES__FEED_DMA_TCD.TCD[5].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD5_CSR AES__FEED_DMA_TCD.TCD[5].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD5_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[5].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD5_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[5].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH6_CSR AES__FEED_DMA_TCD.TCD[6].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH6_ES AES__FEED_DMA_TCD.TCD[6].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH6_INT AES__FEED_DMA_TCD.TCD[6].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH6_SBR AES__FEED_DMA_TCD.TCD[6].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH6_PRI AES__FEED_DMA_TCD.TCD[6].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD6_SADDR AES__FEED_DMA_TCD.TCD[6].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD6_SOFF AES__FEED_DMA_TCD.TCD[6].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD6_ATTR AES__FEED_DMA_TCD.TCD[6].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD6_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[6].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD6_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[6].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD6_SLAST_SDA AES__FEED_DMA_TCD.TCD[6].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD6_DADDR AES__FEED_DMA_TCD.TCD[6].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD6_DOFF AES__FEED_DMA_TCD.TCD[6].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD6_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[6].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD6_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[6].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD6_DLAST_SGA AES__FEED_DMA_TCD.TCD[6].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD6_CSR AES__FEED_DMA_TCD.TCD[6].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD6_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[6].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD6_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[6].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH7_CSR AES__FEED_DMA_TCD.TCD[7].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH7_ES AES__FEED_DMA_TCD.TCD[7].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH7_INT AES__FEED_DMA_TCD.TCD[7].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH7_SBR AES__FEED_DMA_TCD.TCD[7].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH7_PRI AES__FEED_DMA_TCD.TCD[7].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD7_SADDR AES__FEED_DMA_TCD.TCD[7].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD7_SOFF AES__FEED_DMA_TCD.TCD[7].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD7_ATTR AES__FEED_DMA_TCD.TCD[7].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD7_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[7].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD7_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[7].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD7_SLAST_SDA AES__FEED_DMA_TCD.TCD[7].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD7_DADDR AES__FEED_DMA_TCD.TCD[7].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD7_DOFF AES__FEED_DMA_TCD.TCD[7].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD7_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[7].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD7_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[7].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD7_DLAST_SGA AES__FEED_DMA_TCD.TCD[7].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD7_CSR AES__FEED_DMA_TCD.TCD[7].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD7_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[7].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD7_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[7].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH8_CSR AES__FEED_DMA_TCD.TCD[8].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH8_ES AES__FEED_DMA_TCD.TCD[8].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH8_INT AES__FEED_DMA_TCD.TCD[8].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH8_SBR AES__FEED_DMA_TCD.TCD[8].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH8_PRI AES__FEED_DMA_TCD.TCD[8].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD8_SADDR AES__FEED_DMA_TCD.TCD[8].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD8_SOFF AES__FEED_DMA_TCD.TCD[8].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD8_ATTR AES__FEED_DMA_TCD.TCD[8].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD8_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[8].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD8_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[8].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD8_SLAST_SDA AES__FEED_DMA_TCD.TCD[8].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD8_DADDR AES__FEED_DMA_TCD.TCD[8].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD8_DOFF AES__FEED_DMA_TCD.TCD[8].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD8_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[8].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD8_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[8].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD8_DLAST_SGA AES__FEED_DMA_TCD.TCD[8].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD8_CSR AES__FEED_DMA_TCD.TCD[8].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD8_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[8].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD8_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[8].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH9_CSR AES__FEED_DMA_TCD.TCD[9].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH9_ES AES__FEED_DMA_TCD.TCD[9].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH9_INT AES__FEED_DMA_TCD.TCD[9].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH9_SBR AES__FEED_DMA_TCD.TCD[9].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH9_PRI AES__FEED_DMA_TCD.TCD[9].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD9_SADDR AES__FEED_DMA_TCD.TCD[9].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD9_SOFF AES__FEED_DMA_TCD.TCD[9].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD9_ATTR AES__FEED_DMA_TCD.TCD[9].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD9_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[9].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD9_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[9].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD9_SLAST_SDA AES__FEED_DMA_TCD.TCD[9].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD9_DADDR AES__FEED_DMA_TCD.TCD[9].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD9_DOFF AES__FEED_DMA_TCD.TCD[9].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD9_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[9].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD9_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[9].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD9_DLAST_SGA AES__FEED_DMA_TCD.TCD[9].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD9_CSR AES__FEED_DMA_TCD.TCD[9].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD9_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[9].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD9_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[9].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH10_CSR AES__FEED_DMA_TCD.TCD[10].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH10_ES AES__FEED_DMA_TCD.TCD[10].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH10_INT AES__FEED_DMA_TCD.TCD[10].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH10_SBR AES__FEED_DMA_TCD.TCD[10].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH10_PRI AES__FEED_DMA_TCD.TCD[10].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD10_SADDR AES__FEED_DMA_TCD.TCD[10].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD10_SOFF AES__FEED_DMA_TCD.TCD[10].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD10_ATTR AES__FEED_DMA_TCD.TCD[10].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD10_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[10].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD10_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[10].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD10_SLAST_SDA AES__FEED_DMA_TCD.TCD[10].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD10_DADDR AES__FEED_DMA_TCD.TCD[10].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD10_DOFF AES__FEED_DMA_TCD.TCD[10].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD10_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[10].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD10_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[10].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD10_DLAST_SGA AES__FEED_DMA_TCD.TCD[10].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD10_CSR AES__FEED_DMA_TCD.TCD[10].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD10_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[10].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD10_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[10].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH11_CSR AES__FEED_DMA_TCD.TCD[11].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH11_ES AES__FEED_DMA_TCD.TCD[11].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH11_INT AES__FEED_DMA_TCD.TCD[11].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH11_SBR AES__FEED_DMA_TCD.TCD[11].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH11_PRI AES__FEED_DMA_TCD.TCD[11].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD11_SADDR AES__FEED_DMA_TCD.TCD[11].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD11_SOFF AES__FEED_DMA_TCD.TCD[11].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD11_ATTR AES__FEED_DMA_TCD.TCD[11].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD11_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[11].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD11_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[11].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD11_SLAST_SDA AES__FEED_DMA_TCD.TCD[11].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD11_DADDR AES__FEED_DMA_TCD.TCD[11].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD11_DOFF AES__FEED_DMA_TCD.TCD[11].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD11_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[11].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD11_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[11].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD11_DLAST_SGA AES__FEED_DMA_TCD.TCD[11].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD11_CSR AES__FEED_DMA_TCD.TCD[11].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD11_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[11].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD11_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[11].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH12_CSR AES__FEED_DMA_TCD.TCD[12].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH12_ES AES__FEED_DMA_TCD.TCD[12].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH12_INT AES__FEED_DMA_TCD.TCD[12].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH12_SBR AES__FEED_DMA_TCD.TCD[12].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH12_PRI AES__FEED_DMA_TCD.TCD[12].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD12_SADDR AES__FEED_DMA_TCD.TCD[12].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD12_SOFF AES__FEED_DMA_TCD.TCD[12].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD12_ATTR AES__FEED_DMA_TCD.TCD[12].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD12_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[12].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD12_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[12].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD12_SLAST_SDA AES__FEED_DMA_TCD.TCD[12].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD12_DADDR AES__FEED_DMA_TCD.TCD[12].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD12_DOFF AES__FEED_DMA_TCD.TCD[12].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD12_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[12].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD12_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[12].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD12_DLAST_SGA AES__FEED_DMA_TCD.TCD[12].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD12_CSR AES__FEED_DMA_TCD.TCD[12].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD12_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[12].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD12_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[12].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH13_CSR AES__FEED_DMA_TCD.TCD[13].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH13_ES AES__FEED_DMA_TCD.TCD[13].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH13_INT AES__FEED_DMA_TCD.TCD[13].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH13_SBR AES__FEED_DMA_TCD.TCD[13].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH13_PRI AES__FEED_DMA_TCD.TCD[13].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD13_SADDR AES__FEED_DMA_TCD.TCD[13].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD13_SOFF AES__FEED_DMA_TCD.TCD[13].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD13_ATTR AES__FEED_DMA_TCD.TCD[13].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD13_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[13].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD13_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[13].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD13_SLAST_SDA AES__FEED_DMA_TCD.TCD[13].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD13_DADDR AES__FEED_DMA_TCD.TCD[13].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD13_DOFF AES__FEED_DMA_TCD.TCD[13].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD13_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[13].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD13_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[13].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD13_DLAST_SGA AES__FEED_DMA_TCD.TCD[13].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD13_CSR AES__FEED_DMA_TCD.TCD[13].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD13_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[13].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD13_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[13].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH14_CSR AES__FEED_DMA_TCD.TCD[14].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH14_ES AES__FEED_DMA_TCD.TCD[14].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH14_INT AES__FEED_DMA_TCD.TCD[14].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH14_SBR AES__FEED_DMA_TCD.TCD[14].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH14_PRI AES__FEED_DMA_TCD.TCD[14].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD14_SADDR AES__FEED_DMA_TCD.TCD[14].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD14_SOFF AES__FEED_DMA_TCD.TCD[14].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD14_ATTR AES__FEED_DMA_TCD.TCD[14].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD14_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[14].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD14_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[14].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD14_SLAST_SDA AES__FEED_DMA_TCD.TCD[14].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD14_DADDR AES__FEED_DMA_TCD.TCD[14].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD14_DOFF AES__FEED_DMA_TCD.TCD[14].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD14_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[14].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD14_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[14].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD14_DLAST_SGA AES__FEED_DMA_TCD.TCD[14].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD14_CSR AES__FEED_DMA_TCD.TCD[14].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD14_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[14].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD14_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[14].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH15_CSR AES__FEED_DMA_TCD.TCD[15].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH15_ES AES__FEED_DMA_TCD.TCD[15].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH15_INT AES__FEED_DMA_TCD.TCD[15].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH15_SBR AES__FEED_DMA_TCD.TCD[15].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH15_PRI AES__FEED_DMA_TCD.TCD[15].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD15_SADDR AES__FEED_DMA_TCD.TCD[15].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD15_SOFF AES__FEED_DMA_TCD.TCD[15].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD15_ATTR AES__FEED_DMA_TCD.TCD[15].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD15_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[15].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD15_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[15].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD15_SLAST_SDA AES__FEED_DMA_TCD.TCD[15].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD15_DADDR AES__FEED_DMA_TCD.TCD[15].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD15_DOFF AES__FEED_DMA_TCD.TCD[15].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD15_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[15].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD15_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[15].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD15_DLAST_SGA AES__FEED_DMA_TCD.TCD[15].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD15_CSR AES__FEED_DMA_TCD.TCD[15].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD15_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[15].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD15_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[15].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH16_CSR AES__FEED_DMA_TCD.TCD[16].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH16_ES AES__FEED_DMA_TCD.TCD[16].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH16_INT AES__FEED_DMA_TCD.TCD[16].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH16_SBR AES__FEED_DMA_TCD.TCD[16].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH16_PRI AES__FEED_DMA_TCD.TCD[16].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD16_SADDR AES__FEED_DMA_TCD.TCD[16].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD16_SOFF AES__FEED_DMA_TCD.TCD[16].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD16_ATTR AES__FEED_DMA_TCD.TCD[16].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD16_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[16].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD16_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[16].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD16_SLAST_SDA AES__FEED_DMA_TCD.TCD[16].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD16_DADDR AES__FEED_DMA_TCD.TCD[16].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD16_DOFF AES__FEED_DMA_TCD.TCD[16].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD16_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[16].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD16_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[16].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD16_DLAST_SGA AES__FEED_DMA_TCD.TCD[16].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD16_CSR AES__FEED_DMA_TCD.TCD[16].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD16_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[16].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD16_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[16].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH17_CSR AES__FEED_DMA_TCD.TCD[17].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH17_ES AES__FEED_DMA_TCD.TCD[17].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH17_INT AES__FEED_DMA_TCD.TCD[17].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH17_SBR AES__FEED_DMA_TCD.TCD[17].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH17_PRI AES__FEED_DMA_TCD.TCD[17].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD17_SADDR AES__FEED_DMA_TCD.TCD[17].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD17_SOFF AES__FEED_DMA_TCD.TCD[17].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD17_ATTR AES__FEED_DMA_TCD.TCD[17].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD17_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[17].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD17_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[17].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD17_SLAST_SDA AES__FEED_DMA_TCD.TCD[17].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD17_DADDR AES__FEED_DMA_TCD.TCD[17].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD17_DOFF AES__FEED_DMA_TCD.TCD[17].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD17_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[17].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD17_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[17].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD17_DLAST_SGA AES__FEED_DMA_TCD.TCD[17].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD17_CSR AES__FEED_DMA_TCD.TCD[17].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD17_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[17].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD17_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[17].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH18_CSR AES__FEED_DMA_TCD.TCD[18].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH18_ES AES__FEED_DMA_TCD.TCD[18].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH18_INT AES__FEED_DMA_TCD.TCD[18].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH18_SBR AES__FEED_DMA_TCD.TCD[18].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH18_PRI AES__FEED_DMA_TCD.TCD[18].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD18_SADDR AES__FEED_DMA_TCD.TCD[18].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD18_SOFF AES__FEED_DMA_TCD.TCD[18].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD18_ATTR AES__FEED_DMA_TCD.TCD[18].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD18_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[18].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD18_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[18].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD18_SLAST_SDA AES__FEED_DMA_TCD.TCD[18].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD18_DADDR AES__FEED_DMA_TCD.TCD[18].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD18_DOFF AES__FEED_DMA_TCD.TCD[18].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD18_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[18].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD18_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[18].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD18_DLAST_SGA AES__FEED_DMA_TCD.TCD[18].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD18_CSR AES__FEED_DMA_TCD.TCD[18].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD18_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[18].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD18_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[18].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH19_CSR AES__FEED_DMA_TCD.TCD[19].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH19_ES AES__FEED_DMA_TCD.TCD[19].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH19_INT AES__FEED_DMA_TCD.TCD[19].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH19_SBR AES__FEED_DMA_TCD.TCD[19].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH19_PRI AES__FEED_DMA_TCD.TCD[19].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD19_SADDR AES__FEED_DMA_TCD.TCD[19].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD19_SOFF AES__FEED_DMA_TCD.TCD[19].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD19_ATTR AES__FEED_DMA_TCD.TCD[19].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD19_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[19].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD19_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[19].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD19_SLAST_SDA AES__FEED_DMA_TCD.TCD[19].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD19_DADDR AES__FEED_DMA_TCD.TCD[19].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD19_DOFF AES__FEED_DMA_TCD.TCD[19].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD19_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[19].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD19_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[19].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD19_DLAST_SGA AES__FEED_DMA_TCD.TCD[19].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD19_CSR AES__FEED_DMA_TCD.TCD[19].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD19_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[19].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD19_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[19].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH20_CSR AES__FEED_DMA_TCD.TCD[20].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH20_ES AES__FEED_DMA_TCD.TCD[20].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH20_INT AES__FEED_DMA_TCD.TCD[20].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH20_SBR AES__FEED_DMA_TCD.TCD[20].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH20_PRI AES__FEED_DMA_TCD.TCD[20].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD20_SADDR AES__FEED_DMA_TCD.TCD[20].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD20_SOFF AES__FEED_DMA_TCD.TCD[20].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD20_ATTR AES__FEED_DMA_TCD.TCD[20].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD20_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[20].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD20_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[20].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD20_SLAST_SDA AES__FEED_DMA_TCD.TCD[20].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD20_DADDR AES__FEED_DMA_TCD.TCD[20].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD20_DOFF AES__FEED_DMA_TCD.TCD[20].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD20_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[20].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD20_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[20].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD20_DLAST_SGA AES__FEED_DMA_TCD.TCD[20].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD20_CSR AES__FEED_DMA_TCD.TCD[20].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD20_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[20].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD20_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[20].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH21_CSR AES__FEED_DMA_TCD.TCD[21].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH21_ES AES__FEED_DMA_TCD.TCD[21].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH21_INT AES__FEED_DMA_TCD.TCD[21].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH21_SBR AES__FEED_DMA_TCD.TCD[21].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH21_PRI AES__FEED_DMA_TCD.TCD[21].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD21_SADDR AES__FEED_DMA_TCD.TCD[21].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD21_SOFF AES__FEED_DMA_TCD.TCD[21].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD21_ATTR AES__FEED_DMA_TCD.TCD[21].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD21_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[21].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD21_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[21].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD21_SLAST_SDA AES__FEED_DMA_TCD.TCD[21].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD21_DADDR AES__FEED_DMA_TCD.TCD[21].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD21_DOFF AES__FEED_DMA_TCD.TCD[21].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD21_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[21].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD21_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[21].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD21_DLAST_SGA AES__FEED_DMA_TCD.TCD[21].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD21_CSR AES__FEED_DMA_TCD.TCD[21].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD21_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[21].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD21_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[21].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH22_CSR AES__FEED_DMA_TCD.TCD[22].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH22_ES AES__FEED_DMA_TCD.TCD[22].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH22_INT AES__FEED_DMA_TCD.TCD[22].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH22_SBR AES__FEED_DMA_TCD.TCD[22].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH22_PRI AES__FEED_DMA_TCD.TCD[22].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD22_SADDR AES__FEED_DMA_TCD.TCD[22].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD22_SOFF AES__FEED_DMA_TCD.TCD[22].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD22_ATTR AES__FEED_DMA_TCD.TCD[22].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD22_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[22].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD22_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[22].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD22_SLAST_SDA AES__FEED_DMA_TCD.TCD[22].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD22_DADDR AES__FEED_DMA_TCD.TCD[22].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD22_DOFF AES__FEED_DMA_TCD.TCD[22].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD22_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[22].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD22_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[22].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD22_DLAST_SGA AES__FEED_DMA_TCD.TCD[22].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD22_CSR AES__FEED_DMA_TCD.TCD[22].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD22_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[22].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD22_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[22].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_CH23_CSR AES__FEED_DMA_TCD.TCD[23].CH_CSR.R /* Channel Control and Status */
#define AES__FEED_DMA_TCD_CH23_ES AES__FEED_DMA_TCD.TCD[23].CH_ES.R /* Channel Error Status */
#define AES__FEED_DMA_TCD_CH23_INT AES__FEED_DMA_TCD.TCD[23].CH_INT.R /* Channel Interrupt Status */
#define AES__FEED_DMA_TCD_CH23_SBR AES__FEED_DMA_TCD.TCD[23].CH_SBR.R /* Channel System Bus */
#define AES__FEED_DMA_TCD_CH23_PRI AES__FEED_DMA_TCD.TCD[23].CH_PRI.R /* Channel Priority */
#define AES__FEED_DMA_TCD_TCD23_SADDR AES__FEED_DMA_TCD.TCD[23].SADDR.R /* TCD Source Address */
#define AES__FEED_DMA_TCD_TCD23_SOFF AES__FEED_DMA_TCD.TCD[23].SOFF.R /* TCD Signed Source Address Offset */
#define AES__FEED_DMA_TCD_TCD23_ATTR AES__FEED_DMA_TCD.TCD[23].ATTR.R /* TCD Transfer Attributes */
#define AES__FEED_DMA_TCD_TCD23_NBYTES_MLOFFNO AES__FEED_DMA_TCD.TCD[23].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD23_NBYTES_MLOFFYES AES__FEED_DMA_TCD.TCD[23].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__FEED_DMA_TCD_TCD23_SLAST_SDA AES__FEED_DMA_TCD.TCD[23].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__FEED_DMA_TCD_TCD23_DADDR AES__FEED_DMA_TCD.TCD[23].DADDR.R /* TCD Destination Address */
#define AES__FEED_DMA_TCD_TCD23_DOFF AES__FEED_DMA_TCD.TCD[23].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__FEED_DMA_TCD_TCD23_CITER_ELINKNO AES__FEED_DMA_TCD.TCD[23].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD23_CITER_ELINKYES AES__FEED_DMA_TCD.TCD[23].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__FEED_DMA_TCD_TCD23_DLAST_SGA AES__FEED_DMA_TCD.TCD[23].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__FEED_DMA_TCD_TCD23_CSR AES__FEED_DMA_TCD.TCD[23].CSR.R /* TCD Control and Status */
#define AES__FEED_DMA_TCD_TCD23_BITER_ELINKNO AES__FEED_DMA_TCD.TCD[23].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__FEED_DMA_TCD_TCD23_BITER_ELINKYES AES__FEED_DMA_TCD.TCD[23].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */

/* FLEXCAN */
#define CE_CAN_0_MCR         CE_CAN_0.MCR.R                /* Module Configuration */
#define CE_CAN_0_CTRL1       CE_CAN_0.CTRL1.R              /* Control 1 */
#define CE_CAN_0_TIMER       CE_CAN_0.TIMER.R              /* Free-Running Timer */
#define CE_CAN_0_RXMGMASK    CE_CAN_0.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_0_RX14MASK    CE_CAN_0.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_0_RX15MASK    CE_CAN_0.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_0_ECR         CE_CAN_0.ECR.R                /* Error Counter */
#define CE_CAN_0_ESR1        CE_CAN_0.ESR1.R               /* Error and Status 1 */
#define CE_CAN_0_IMASK2      CE_CAN_0.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_0_IMASK1      CE_CAN_0.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_0_IFLAG2      CE_CAN_0.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_0_IFLAG1      CE_CAN_0.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_0_CTRL2       CE_CAN_0.CTRL2.R              /* Control 2 */
#define CE_CAN_0_ESR2        CE_CAN_0.ESR2.R               /* Error and Status 2 */
#define CE_CAN_0_CRCR        CE_CAN_0.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_0_RXFGMASK    CE_CAN_0.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_0_RXFIR       CE_CAN_0.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_0_CBT         CE_CAN_0.CBT.R                /* CAN Bit Timing */
#define CE_CAN_0_IMASK4      CE_CAN_0.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_0_IMASK3      CE_CAN_0.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_0_IFLAG4      CE_CAN_0.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_0_IFLAG3      CE_CAN_0.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_0_CS0            CE_CAN_0.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_0_ID0            CE_CAN_0.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_0_B0             CE_CAN_0.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_0_H0             CE_CAN_0.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_0_W0             CE_CAN_0.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_0_CS1            CE_CAN_0.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_0_ID1            CE_CAN_0.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_0_B1             CE_CAN_0.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_0_H1             CE_CAN_0.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_0_W1             CE_CAN_0.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_0_CS2            CE_CAN_0.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_0_ID2            CE_CAN_0.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_0_B2             CE_CAN_0.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_0_H2             CE_CAN_0.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_0_W2             CE_CAN_0.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_0_CS3            CE_CAN_0.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_0_ID3            CE_CAN_0.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_0_B3             CE_CAN_0.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_0_H3             CE_CAN_0.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_0_W3             CE_CAN_0.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_0_CS4            CE_CAN_0.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_0_ID4            CE_CAN_0.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_0_B4             CE_CAN_0.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_0_H4             CE_CAN_0.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_0_W4             CE_CAN_0.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_0_CS5            CE_CAN_0.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_0_ID5            CE_CAN_0.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_0_B5             CE_CAN_0.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_0_H5             CE_CAN_0.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_0_W5             CE_CAN_0.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_0_CS6            CE_CAN_0.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_0_ID6            CE_CAN_0.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_0_B6             CE_CAN_0.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_0_H6             CE_CAN_0.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_0_W6             CE_CAN_0.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_0_CS7            CE_CAN_0.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_0_ID7            CE_CAN_0.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_0_B7             CE_CAN_0.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_0_H7             CE_CAN_0.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_0_W7             CE_CAN_0.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_0_CS8            CE_CAN_0.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_0_ID8            CE_CAN_0.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_0_B8             CE_CAN_0.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_0_H8             CE_CAN_0.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_0_W8             CE_CAN_0.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_0_CS9            CE_CAN_0.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_0_ID9            CE_CAN_0.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_0_B9             CE_CAN_0.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_0_H9             CE_CAN_0.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_0_W9             CE_CAN_0.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_0_CS10           CE_CAN_0.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_0_ID10           CE_CAN_0.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_0_B10            CE_CAN_0.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_0_H10            CE_CAN_0.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_0_W10            CE_CAN_0.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_0_CS11           CE_CAN_0.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_0_ID11           CE_CAN_0.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_0_B11            CE_CAN_0.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_0_H11            CE_CAN_0.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_0_W11            CE_CAN_0.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_0_CS12           CE_CAN_0.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_0_ID12           CE_CAN_0.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_0_B12            CE_CAN_0.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_0_H12            CE_CAN_0.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_0_W12            CE_CAN_0.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_0_CS13           CE_CAN_0.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_0_ID13           CE_CAN_0.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_0_B13            CE_CAN_0.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_0_H13            CE_CAN_0.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_0_W13            CE_CAN_0.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_0_CS14           CE_CAN_0.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_0_ID14           CE_CAN_0.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_0_B14            CE_CAN_0.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_0_H14            CE_CAN_0.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_0_W14            CE_CAN_0.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_0_CS15           CE_CAN_0.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_0_ID15           CE_CAN_0.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_0_B15            CE_CAN_0.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_0_H15            CE_CAN_0.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_0_W15            CE_CAN_0.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_0_CS16           CE_CAN_0.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_0_ID16           CE_CAN_0.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_0_B16            CE_CAN_0.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_0_H16            CE_CAN_0.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_0_W16            CE_CAN_0.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_0_CS17           CE_CAN_0.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_0_ID17           CE_CAN_0.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_0_B17            CE_CAN_0.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_0_H17            CE_CAN_0.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_0_W17            CE_CAN_0.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_0_CS18           CE_CAN_0.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_0_ID18           CE_CAN_0.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_0_B18            CE_CAN_0.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_0_H18            CE_CAN_0.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_0_W18            CE_CAN_0.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_0_CS19           CE_CAN_0.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_0_ID19           CE_CAN_0.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_0_B19            CE_CAN_0.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_0_H19            CE_CAN_0.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_0_W19            CE_CAN_0.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_0_CS20           CE_CAN_0.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_0_ID20           CE_CAN_0.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_0_B20            CE_CAN_0.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_0_H20            CE_CAN_0.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_0_W20            CE_CAN_0.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_0_CS21           CE_CAN_0.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_0_ID21           CE_CAN_0.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_0_B21            CE_CAN_0.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_0_H21            CE_CAN_0.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_0_W21            CE_CAN_0.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_0_CS22           CE_CAN_0.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_0_ID22           CE_CAN_0.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_0_B22            CE_CAN_0.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_0_H22            CE_CAN_0.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_0_W22            CE_CAN_0.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_0_CS23           CE_CAN_0.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_0_ID23           CE_CAN_0.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_0_B23            CE_CAN_0.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_0_H23            CE_CAN_0.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_0_W23            CE_CAN_0.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_0_CS24           CE_CAN_0.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_0_ID24           CE_CAN_0.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_0_B24            CE_CAN_0.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_0_H24            CE_CAN_0.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_0_W24            CE_CAN_0.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_0_CS25           CE_CAN_0.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_0_ID25           CE_CAN_0.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_0_B25            CE_CAN_0.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_0_H25            CE_CAN_0.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_0_W25            CE_CAN_0.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_0_CS26           CE_CAN_0.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_0_ID26           CE_CAN_0.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_0_B26            CE_CAN_0.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_0_H26            CE_CAN_0.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_0_W26            CE_CAN_0.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_0_CS27           CE_CAN_0.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_0_ID27           CE_CAN_0.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_0_B27            CE_CAN_0.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_0_H27            CE_CAN_0.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_0_W27            CE_CAN_0.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_0_CS28           CE_CAN_0.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_0_ID28           CE_CAN_0.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_0_B28            CE_CAN_0.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_0_H28            CE_CAN_0.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_0_W28            CE_CAN_0.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_0_CS29           CE_CAN_0.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_0_ID29           CE_CAN_0.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_0_B29            CE_CAN_0.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_0_H29            CE_CAN_0.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_0_W29            CE_CAN_0.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_0_CS30           CE_CAN_0.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_0_ID30           CE_CAN_0.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_0_B30            CE_CAN_0.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_0_H30            CE_CAN_0.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_0_W30            CE_CAN_0.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_0_CS31           CE_CAN_0.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_0_ID31           CE_CAN_0.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_0_B31            CE_CAN_0.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_0_H31            CE_CAN_0.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_0_W31            CE_CAN_0.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_0_CS32           CE_CAN_0.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_0_ID32           CE_CAN_0.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_0_B32            CE_CAN_0.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_0_H32            CE_CAN_0.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_0_W32            CE_CAN_0.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_0_CS33           CE_CAN_0.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_0_ID33           CE_CAN_0.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_0_B33            CE_CAN_0.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_0_H33            CE_CAN_0.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_0_W33            CE_CAN_0.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_0_CS34           CE_CAN_0.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_0_ID34           CE_CAN_0.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_0_B34            CE_CAN_0.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_0_H34            CE_CAN_0.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_0_W34            CE_CAN_0.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_0_CS35           CE_CAN_0.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_0_ID35           CE_CAN_0.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_0_B35            CE_CAN_0.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_0_H35            CE_CAN_0.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_0_W35            CE_CAN_0.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_0_CS36           CE_CAN_0.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_0_ID36           CE_CAN_0.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_0_B36            CE_CAN_0.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_0_H36            CE_CAN_0.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_0_W36            CE_CAN_0.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_0_CS37           CE_CAN_0.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_0_ID37           CE_CAN_0.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_0_B37            CE_CAN_0.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_0_H37            CE_CAN_0.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_0_W37            CE_CAN_0.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_0_CS38           CE_CAN_0.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_0_ID38           CE_CAN_0.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_0_B38            CE_CAN_0.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_0_H38            CE_CAN_0.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_0_W38            CE_CAN_0.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_0_CS39           CE_CAN_0.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_0_ID39           CE_CAN_0.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_0_B39            CE_CAN_0.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_0_H39            CE_CAN_0.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_0_W39            CE_CAN_0.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_0_CS40           CE_CAN_0.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_0_ID40           CE_CAN_0.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_0_B40            CE_CAN_0.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_0_H40            CE_CAN_0.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_0_W40            CE_CAN_0.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_0_CS41           CE_CAN_0.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_0_ID41           CE_CAN_0.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_0_B41            CE_CAN_0.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_0_H41            CE_CAN_0.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_0_W41            CE_CAN_0.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_0_CS42           CE_CAN_0.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_0_ID42           CE_CAN_0.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_0_B42            CE_CAN_0.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_0_H42            CE_CAN_0.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_0_W42            CE_CAN_0.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_0_CS43           CE_CAN_0.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_0_ID43           CE_CAN_0.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_0_B43            CE_CAN_0.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_0_H43            CE_CAN_0.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_0_W43            CE_CAN_0.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_0_CS44           CE_CAN_0.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_0_ID44           CE_CAN_0.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_0_B44            CE_CAN_0.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_0_H44            CE_CAN_0.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_0_W44            CE_CAN_0.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_0_CS45           CE_CAN_0.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_0_ID45           CE_CAN_0.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_0_B45            CE_CAN_0.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_0_H45            CE_CAN_0.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_0_W45            CE_CAN_0.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_0_CS46           CE_CAN_0.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_0_ID46           CE_CAN_0.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_0_B46            CE_CAN_0.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_0_H46            CE_CAN_0.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_0_W46            CE_CAN_0.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_0_CS47           CE_CAN_0.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_0_ID47           CE_CAN_0.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_0_B47            CE_CAN_0.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_0_H47            CE_CAN_0.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_0_W47            CE_CAN_0.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_0_CS48           CE_CAN_0.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_0_ID48           CE_CAN_0.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_0_B48            CE_CAN_0.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_0_H48            CE_CAN_0.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_0_W48            CE_CAN_0.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_0_CS49           CE_CAN_0.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_0_ID49           CE_CAN_0.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_0_B49            CE_CAN_0.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_0_H49            CE_CAN_0.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_0_W49            CE_CAN_0.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_0_CS50           CE_CAN_0.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_0_ID50           CE_CAN_0.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_0_B50            CE_CAN_0.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_0_H50            CE_CAN_0.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_0_W50            CE_CAN_0.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_0_CS51           CE_CAN_0.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_0_ID51           CE_CAN_0.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_0_B51            CE_CAN_0.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_0_H51            CE_CAN_0.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_0_W51            CE_CAN_0.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_0_CS52           CE_CAN_0.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_0_ID52           CE_CAN_0.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_0_B52            CE_CAN_0.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_0_H52            CE_CAN_0.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_0_W52            CE_CAN_0.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_0_CS53           CE_CAN_0.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_0_ID53           CE_CAN_0.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_0_B53            CE_CAN_0.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_0_H53            CE_CAN_0.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_0_W53            CE_CAN_0.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_0_CS54           CE_CAN_0.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_0_ID54           CE_CAN_0.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_0_B54            CE_CAN_0.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_0_H54            CE_CAN_0.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_0_W54            CE_CAN_0.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_0_CS55           CE_CAN_0.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_0_ID55           CE_CAN_0.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_0_B55            CE_CAN_0.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_0_H55            CE_CAN_0.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_0_W55            CE_CAN_0.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_0_CS56           CE_CAN_0.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_0_ID56           CE_CAN_0.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_0_B56            CE_CAN_0.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_0_H56            CE_CAN_0.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_0_W56            CE_CAN_0.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_0_CS57           CE_CAN_0.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_0_ID57           CE_CAN_0.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_0_B57            CE_CAN_0.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_0_H57            CE_CAN_0.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_0_W57            CE_CAN_0.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_0_CS58           CE_CAN_0.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_0_ID58           CE_CAN_0.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_0_B58            CE_CAN_0.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_0_H58            CE_CAN_0.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_0_W58            CE_CAN_0.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_0_CS59           CE_CAN_0.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_0_ID59           CE_CAN_0.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_0_B59            CE_CAN_0.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_0_H59            CE_CAN_0.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_0_W59            CE_CAN_0.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_0_CS60           CE_CAN_0.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_0_ID60           CE_CAN_0.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_0_B60            CE_CAN_0.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_0_H60            CE_CAN_0.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_0_W60            CE_CAN_0.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_0_CS61           CE_CAN_0.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_0_ID61           CE_CAN_0.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_0_B61            CE_CAN_0.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_0_H61            CE_CAN_0.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_0_W61            CE_CAN_0.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_0_CS62           CE_CAN_0.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_0_ID62           CE_CAN_0.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_0_B62            CE_CAN_0.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_0_H62            CE_CAN_0.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_0_W62            CE_CAN_0.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_0_CS63           CE_CAN_0.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_0_ID63           CE_CAN_0.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_0_B63            CE_CAN_0.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_0_H63            CE_CAN_0.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_0_W63            CE_CAN_0.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_0_CS64           CE_CAN_0.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_0_ID64           CE_CAN_0.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_0_B64            CE_CAN_0.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_0_H64            CE_CAN_0.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_0_W64            CE_CAN_0.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_0_CS65           CE_CAN_0.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_0_ID65           CE_CAN_0.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_0_B65            CE_CAN_0.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_0_H65            CE_CAN_0.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_0_W65            CE_CAN_0.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_0_CS66           CE_CAN_0.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_0_ID66           CE_CAN_0.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_0_B66            CE_CAN_0.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_0_H66            CE_CAN_0.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_0_W66            CE_CAN_0.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_0_CS67           CE_CAN_0.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_0_ID67           CE_CAN_0.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_0_B67            CE_CAN_0.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_0_H67            CE_CAN_0.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_0_W67            CE_CAN_0.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_0_CS68           CE_CAN_0.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_0_ID68           CE_CAN_0.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_0_B68            CE_CAN_0.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_0_H68            CE_CAN_0.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_0_W68            CE_CAN_0.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_0_CS69           CE_CAN_0.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_0_ID69           CE_CAN_0.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_0_B69            CE_CAN_0.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_0_H69            CE_CAN_0.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_0_W69            CE_CAN_0.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_0_CS70           CE_CAN_0.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_0_ID70           CE_CAN_0.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_0_B70            CE_CAN_0.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_0_H70            CE_CAN_0.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_0_W70            CE_CAN_0.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_0_CS71           CE_CAN_0.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_0_ID71           CE_CAN_0.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_0_B71            CE_CAN_0.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_0_H71            CE_CAN_0.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_0_W71            CE_CAN_0.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_0_CS72           CE_CAN_0.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_0_ID72           CE_CAN_0.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_0_B72            CE_CAN_0.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_0_H72            CE_CAN_0.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_0_W72            CE_CAN_0.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_0_CS73           CE_CAN_0.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_0_ID73           CE_CAN_0.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_0_B73            CE_CAN_0.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_0_H73            CE_CAN_0.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_0_W73            CE_CAN_0.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_0_CS74           CE_CAN_0.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_0_ID74           CE_CAN_0.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_0_B74            CE_CAN_0.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_0_H74            CE_CAN_0.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_0_W74            CE_CAN_0.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_0_CS75           CE_CAN_0.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_0_ID75           CE_CAN_0.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_0_B75            CE_CAN_0.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_0_H75            CE_CAN_0.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_0_W75            CE_CAN_0.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_0_CS76           CE_CAN_0.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_0_ID76           CE_CAN_0.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_0_B76            CE_CAN_0.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_0_H76            CE_CAN_0.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_0_W76            CE_CAN_0.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_0_CS77           CE_CAN_0.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_0_ID77           CE_CAN_0.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_0_B77            CE_CAN_0.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_0_H77            CE_CAN_0.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_0_W77            CE_CAN_0.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_0_CS78           CE_CAN_0.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_0_ID78           CE_CAN_0.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_0_B78            CE_CAN_0.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_0_H78            CE_CAN_0.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_0_W78            CE_CAN_0.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_0_CS79           CE_CAN_0.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_0_ID79           CE_CAN_0.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_0_B79            CE_CAN_0.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_0_H79            CE_CAN_0.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_0_W79            CE_CAN_0.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_0_CS80           CE_CAN_0.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_0_ID80           CE_CAN_0.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_0_B80            CE_CAN_0.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_0_H80            CE_CAN_0.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_0_W80            CE_CAN_0.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_0_CS81           CE_CAN_0.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_0_ID81           CE_CAN_0.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_0_B81            CE_CAN_0.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_0_H81            CE_CAN_0.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_0_W81            CE_CAN_0.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_0_CS82           CE_CAN_0.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_0_ID82           CE_CAN_0.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_0_B82            CE_CAN_0.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_0_H82            CE_CAN_0.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_0_W82            CE_CAN_0.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_0_CS83           CE_CAN_0.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_0_ID83           CE_CAN_0.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_0_B83            CE_CAN_0.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_0_H83            CE_CAN_0.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_0_W83            CE_CAN_0.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_0_CS84           CE_CAN_0.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_0_ID84           CE_CAN_0.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_0_B84            CE_CAN_0.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_0_H84            CE_CAN_0.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_0_W84            CE_CAN_0.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_0_CS85           CE_CAN_0.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_0_ID85           CE_CAN_0.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_0_B85            CE_CAN_0.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_0_H85            CE_CAN_0.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_0_W85            CE_CAN_0.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_0_CS86           CE_CAN_0.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_0_ID86           CE_CAN_0.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_0_B86            CE_CAN_0.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_0_H86            CE_CAN_0.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_0_W86            CE_CAN_0.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_0_CS87           CE_CAN_0.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_0_ID87           CE_CAN_0.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_0_B87            CE_CAN_0.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_0_H87            CE_CAN_0.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_0_W87            CE_CAN_0.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_0_CS88           CE_CAN_0.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_0_ID88           CE_CAN_0.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_0_B88            CE_CAN_0.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_0_H88            CE_CAN_0.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_0_W848           CE_CAN_0.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_0_CS89           CE_CAN_0.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_0_ID89           CE_CAN_0.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_0_B89            CE_CAN_0.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_0_H89            CE_CAN_0.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_0_W89            CE_CAN_0.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_0_CS90           CE_CAN_0.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_0_ID90           CE_CAN_0.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_0_B90            CE_CAN_0.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_0_H90            CE_CAN_0.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_0_W90            CE_CAN_0.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_0_CS91           CE_CAN_0.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_0_ID91           CE_CAN_0.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_0_B91            CE_CAN_0.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_0_H91            CE_CAN_0.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_0_W91            CE_CAN_0.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_0_CS92           CE_CAN_0.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_0_ID92           CE_CAN_0.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_0_B92            CE_CAN_0.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_0_H92            CE_CAN_0.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_0_W92            CE_CAN_0.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_0_CS93           CE_CAN_0.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_0_ID93           CE_CAN_0.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_0_B93            CE_CAN_0.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_0_H93            CE_CAN_0.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_0_W93            CE_CAN_0.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_0_CS94           CE_CAN_0.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_0_ID94           CE_CAN_0.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_0_B94            CE_CAN_0.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_0_H94            CE_CAN_0.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_0_W94            CE_CAN_0.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_0_CS95           CE_CAN_0.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_0_ID95           CE_CAN_0.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_0_B95            CE_CAN_0.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_0_H95            CE_CAN_0.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_0_W95            CE_CAN_0.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_0_CS96           CE_CAN_0.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_0_ID96           CE_CAN_0.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_0_B96            CE_CAN_0.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_0_H96            CE_CAN_0.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_0_W96            CE_CAN_0.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_0_CS97           CE_CAN_0.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_0_ID97           CE_CAN_0.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_0_B97            CE_CAN_0.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_0_H97            CE_CAN_0.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_0_W97            CE_CAN_0.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_0_CS98           CE_CAN_0.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_0_ID98           CE_CAN_0.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_0_B98            CE_CAN_0.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_0_H98            CE_CAN_0.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_0_W98            CE_CAN_0.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_0_CS99           CE_CAN_0.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_0_ID99           CE_CAN_0.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_0_B99            CE_CAN_0.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_0_H99            CE_CAN_0.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_0_W99            CE_CAN_0.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_0_CS100          CE_CAN_0.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_0_ID100          CE_CAN_0.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_0_B100           CE_CAN_0.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_0_H100           CE_CAN_0.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_0_W100           CE_CAN_0.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_0_CS101          CE_CAN_0.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_0_ID101          CE_CAN_0.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_0_B101           CE_CAN_0.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_0_H101           CE_CAN_0.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_0_W101           CE_CAN_0.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_0_CS102          CE_CAN_0.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_0_ID102          CE_CAN_0.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_0_B102           CE_CAN_0.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_0_H102           CE_CAN_0.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_0_W102           CE_CAN_0.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_0_CS103          CE_CAN_0.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_0_ID103          CE_CAN_0.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_0_B103           CE_CAN_0.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_0_H103           CE_CAN_0.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_0_W103           CE_CAN_0.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_0_CS104          CE_CAN_0.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_0_ID104          CE_CAN_0.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_0_B104           CE_CAN_0.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_0_H104           CE_CAN_0.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_0_W104           CE_CAN_0.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_0_CS105          CE_CAN_0.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_0_ID105          CE_CAN_0.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_0_B105           CE_CAN_0.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_0_H105           CE_CAN_0.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_0_W105           CE_CAN_0.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_0_CS106          CE_CAN_0.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_0_ID106          CE_CAN_0.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_0_B106           CE_CAN_0.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_0_H106           CE_CAN_0.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_0_W106           CE_CAN_0.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_0_CS107          CE_CAN_0.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_0_ID107          CE_CAN_0.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_0_B107           CE_CAN_0.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_0_H107           CE_CAN_0.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_0_W107           CE_CAN_0.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_0_CS108          CE_CAN_0.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_0_ID108          CE_CAN_0.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_0_B108           CE_CAN_0.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_0_H108           CE_CAN_0.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_0_W108           CE_CAN_0.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_0_CS109          CE_CAN_0.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_0_ID109          CE_CAN_0.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_0_B109           CE_CAN_0.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_0_H109           CE_CAN_0.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_0_W109           CE_CAN_0.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_0_CS110          CE_CAN_0.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_0_ID110          CE_CAN_0.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_0_B110           CE_CAN_0.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_0_H110           CE_CAN_0.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_0_W110           CE_CAN_0.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_0_CS111          CE_CAN_0.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_0_ID111          CE_CAN_0.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_0_B111           CE_CAN_0.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_0_H111           CE_CAN_0.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_0_W111           CE_CAN_0.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_0_CS112          CE_CAN_0.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_0_ID112          CE_CAN_0.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_0_B112           CE_CAN_0.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_0_H112           CE_CAN_0.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_0_W112           CE_CAN_0.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_0_CS113          CE_CAN_0.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_0_ID113          CE_CAN_0.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_0_B113           CE_CAN_0.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_0_H113           CE_CAN_0.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_0_W113           CE_CAN_0.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_0_CS114          CE_CAN_0.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_0_ID114          CE_CAN_0.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_0_B114           CE_CAN_0.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_0_H114           CE_CAN_0.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_0_W114           CE_CAN_0.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_0_CS115          CE_CAN_0.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_0_ID115          CE_CAN_0.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_0_B115           CE_CAN_0.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_0_H115           CE_CAN_0.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_0_W115           CE_CAN_0.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_0_CS116          CE_CAN_0.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_0_ID116          CE_CAN_0.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_0_B116           CE_CAN_0.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_0_H116           CE_CAN_0.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_0_W116           CE_CAN_0.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_0_CS117          CE_CAN_0.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_0_ID117          CE_CAN_0.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_0_B117           CE_CAN_0.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_0_H117           CE_CAN_0.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_0_W117           CE_CAN_0.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_0_CS118          CE_CAN_0.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_0_ID118          CE_CAN_0.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_0_B118           CE_CAN_0.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_0_H118           CE_CAN_0.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_0_W118           CE_CAN_0.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_0_CS119          CE_CAN_0.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_0_ID119          CE_CAN_0.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_0_B119           CE_CAN_0.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_0_H119           CE_CAN_0.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_0_W119           CE_CAN_0.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_0_CS120          CE_CAN_0.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_0_ID120          CE_CAN_0.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_0_B120           CE_CAN_0.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_0_H120           CE_CAN_0.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_0_W120           CE_CAN_0.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_0_CS121          CE_CAN_0.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_0_ID121          CE_CAN_0.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_0_B121           CE_CAN_0.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_0_H121           CE_CAN_0.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_0_W121           CE_CAN_0.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_0_CS122          CE_CAN_0.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_0_ID122          CE_CAN_0.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_0_B122           CE_CAN_0.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_0_H122           CE_CAN_0.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_0_W122           CE_CAN_0.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_0_CS123          CE_CAN_0.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_0_ID123          CE_CAN_0.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_0_B123           CE_CAN_0.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_0_H123           CE_CAN_0.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_0_W123           CE_CAN_0.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_0_CS124          CE_CAN_0.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_0_ID124          CE_CAN_0.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_0_B124           CE_CAN_0.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_0_H124           CE_CAN_0.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_0_W124           CE_CAN_0.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_0_CS125          CE_CAN_0.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_0_ID125          CE_CAN_0.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_0_B125           CE_CAN_0.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_0_H125           CE_CAN_0.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_0_W125           CE_CAN_0.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_0_CS126          CE_CAN_0.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_0_ID126          CE_CAN_0.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_0_B126           CE_CAN_0.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_0_H126           CE_CAN_0.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_0_W126           CE_CAN_0.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_0_CS127          CE_CAN_0.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_0_ID127          CE_CAN_0.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_0_B127           CE_CAN_0.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_0_H127           CE_CAN_0.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_0_W127           CE_CAN_0.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_0_RXIMR0      CE_CAN_0.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR1      CE_CAN_0.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR2      CE_CAN_0.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR3      CE_CAN_0.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR4      CE_CAN_0.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR5      CE_CAN_0.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR6      CE_CAN_0.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR7      CE_CAN_0.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR8      CE_CAN_0.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR9      CE_CAN_0.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_0_RXIMR10     CE_CAN_0.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR11     CE_CAN_0.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR12     CE_CAN_0.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR13     CE_CAN_0.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR14     CE_CAN_0.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR15     CE_CAN_0.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR16     CE_CAN_0.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR17     CE_CAN_0.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR18     CE_CAN_0.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR19     CE_CAN_0.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR20     CE_CAN_0.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR21     CE_CAN_0.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR22     CE_CAN_0.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR23     CE_CAN_0.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR24     CE_CAN_0.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR25     CE_CAN_0.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR26     CE_CAN_0.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR27     CE_CAN_0.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR28     CE_CAN_0.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR29     CE_CAN_0.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR30     CE_CAN_0.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR31     CE_CAN_0.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR32     CE_CAN_0.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR33     CE_CAN_0.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR34     CE_CAN_0.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR35     CE_CAN_0.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR36     CE_CAN_0.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR37     CE_CAN_0.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR38     CE_CAN_0.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR39     CE_CAN_0.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR40     CE_CAN_0.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR41     CE_CAN_0.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR42     CE_CAN_0.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR43     CE_CAN_0.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR44     CE_CAN_0.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR45     CE_CAN_0.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR46     CE_CAN_0.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR47     CE_CAN_0.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR48     CE_CAN_0.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR49     CE_CAN_0.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR50     CE_CAN_0.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR51     CE_CAN_0.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR52     CE_CAN_0.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR53     CE_CAN_0.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR54     CE_CAN_0.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR55     CE_CAN_0.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR56     CE_CAN_0.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR57     CE_CAN_0.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR58     CE_CAN_0.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR59     CE_CAN_0.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR60     CE_CAN_0.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR61     CE_CAN_0.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR62     CE_CAN_0.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR63     CE_CAN_0.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR64     CE_CAN_0.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR65     CE_CAN_0.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR66     CE_CAN_0.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR67     CE_CAN_0.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR68     CE_CAN_0.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR69     CE_CAN_0.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR70     CE_CAN_0.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR71     CE_CAN_0.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR72     CE_CAN_0.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR73     CE_CAN_0.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR74     CE_CAN_0.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR75     CE_CAN_0.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR76     CE_CAN_0.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR77     CE_CAN_0.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR78     CE_CAN_0.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR79     CE_CAN_0.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR80     CE_CAN_0.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR81     CE_CAN_0.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR82     CE_CAN_0.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR83     CE_CAN_0.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR84     CE_CAN_0.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR85     CE_CAN_0.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR86     CE_CAN_0.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR87     CE_CAN_0.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR88     CE_CAN_0.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR89     CE_CAN_0.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR90     CE_CAN_0.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR91     CE_CAN_0.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR92     CE_CAN_0.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR93     CE_CAN_0.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR94     CE_CAN_0.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR95     CE_CAN_0.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR96     CE_CAN_0.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR97     CE_CAN_0.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR98     CE_CAN_0.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR99     CE_CAN_0.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_0_RXIMR100    CE_CAN_0.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR101    CE_CAN_0.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR102    CE_CAN_0.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR103    CE_CAN_0.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR104    CE_CAN_0.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR105    CE_CAN_0.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR106    CE_CAN_0.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR107    CE_CAN_0.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR108    CE_CAN_0.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR109    CE_CAN_0.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR110    CE_CAN_0.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR111    CE_CAN_0.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR112    CE_CAN_0.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR113    CE_CAN_0.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR114    CE_CAN_0.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR115    CE_CAN_0.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR116    CE_CAN_0.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR117    CE_CAN_0.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR118    CE_CAN_0.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR119    CE_CAN_0.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR120    CE_CAN_0.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR121    CE_CAN_0.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR122    CE_CAN_0.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR123    CE_CAN_0.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR124    CE_CAN_0.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR125    CE_CAN_0.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR126    CE_CAN_0.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_0_RXIMR127    CE_CAN_0.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_0_MECR        CE_CAN_0.MECR.R               /* Memory Error Control */
#define CE_CAN_0_ERRIAR      CE_CAN_0.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_0_ERRIDPR     CE_CAN_0.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_0_ERRIPPR     CE_CAN_0.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_0_RERRAR      CE_CAN_0.RERRAR.R             /* Error Report Address */
#define CE_CAN_0_RERRDR      CE_CAN_0.RERRDR.R             /* Error Report Data */
#define CE_CAN_0_RERRSYNR    CE_CAN_0.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_0_ERRSR       CE_CAN_0.ERRSR.R              /* Error Status */
#define CE_CAN_0_EPRS        CE_CAN_0.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_0_ENCBT       CE_CAN_0.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_0_EDCBT       CE_CAN_0.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_0_ETDC        CE_CAN_0.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_0_FDCTRL      CE_CAN_0.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_0_FDCBT       CE_CAN_0.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_0_FDCRC       CE_CAN_0.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_0_ERFCR       CE_CAN_0.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_0_ERFIER      CE_CAN_0.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_0_ERFSR       CE_CAN_0.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_0_HR_TIME_STAMP0 CE_CAN_0.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP1 CE_CAN_0.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP2 CE_CAN_0.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP3 CE_CAN_0.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP4 CE_CAN_0.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP5 CE_CAN_0.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP6 CE_CAN_0.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP7 CE_CAN_0.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP8 CE_CAN_0.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP9 CE_CAN_0.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP10 CE_CAN_0.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP11 CE_CAN_0.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP12 CE_CAN_0.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP13 CE_CAN_0.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP14 CE_CAN_0.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP15 CE_CAN_0.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP16 CE_CAN_0.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP17 CE_CAN_0.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP18 CE_CAN_0.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP19 CE_CAN_0.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP20 CE_CAN_0.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP21 CE_CAN_0.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP22 CE_CAN_0.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP23 CE_CAN_0.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP24 CE_CAN_0.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP25 CE_CAN_0.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP26 CE_CAN_0.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP27 CE_CAN_0.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP28 CE_CAN_0.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP29 CE_CAN_0.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP30 CE_CAN_0.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP31 CE_CAN_0.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP32 CE_CAN_0.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP33 CE_CAN_0.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP34 CE_CAN_0.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP35 CE_CAN_0.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP36 CE_CAN_0.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP37 CE_CAN_0.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP38 CE_CAN_0.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP39 CE_CAN_0.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP40 CE_CAN_0.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP41 CE_CAN_0.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP42 CE_CAN_0.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP43 CE_CAN_0.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP44 CE_CAN_0.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP45 CE_CAN_0.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP46 CE_CAN_0.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP47 CE_CAN_0.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP48 CE_CAN_0.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP49 CE_CAN_0.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP50 CE_CAN_0.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP51 CE_CAN_0.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP52 CE_CAN_0.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP53 CE_CAN_0.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP54 CE_CAN_0.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP55 CE_CAN_0.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP56 CE_CAN_0.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP57 CE_CAN_0.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP58 CE_CAN_0.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP59 CE_CAN_0.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP60 CE_CAN_0.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP61 CE_CAN_0.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP62 CE_CAN_0.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP63 CE_CAN_0.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP64 CE_CAN_0.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP65 CE_CAN_0.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP66 CE_CAN_0.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP67 CE_CAN_0.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP68 CE_CAN_0.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP69 CE_CAN_0.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP70 CE_CAN_0.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP71 CE_CAN_0.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP72 CE_CAN_0.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP73 CE_CAN_0.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP74 CE_CAN_0.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP75 CE_CAN_0.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP76 CE_CAN_0.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP77 CE_CAN_0.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP78 CE_CAN_0.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP79 CE_CAN_0.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP80 CE_CAN_0.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP81 CE_CAN_0.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP82 CE_CAN_0.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP83 CE_CAN_0.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP84 CE_CAN_0.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP85 CE_CAN_0.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP86 CE_CAN_0.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP87 CE_CAN_0.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP88 CE_CAN_0.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP89 CE_CAN_0.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP90 CE_CAN_0.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP91 CE_CAN_0.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP92 CE_CAN_0.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP93 CE_CAN_0.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP94 CE_CAN_0.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP95 CE_CAN_0.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP96 CE_CAN_0.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP97 CE_CAN_0.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP98 CE_CAN_0.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP99 CE_CAN_0.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP100 CE_CAN_0.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP101 CE_CAN_0.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP102 CE_CAN_0.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP103 CE_CAN_0.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP104 CE_CAN_0.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP105 CE_CAN_0.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP106 CE_CAN_0.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP107 CE_CAN_0.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP108 CE_CAN_0.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP109 CE_CAN_0.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP110 CE_CAN_0.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP111 CE_CAN_0.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP112 CE_CAN_0.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP113 CE_CAN_0.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP114 CE_CAN_0.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP115 CE_CAN_0.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP116 CE_CAN_0.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP117 CE_CAN_0.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP118 CE_CAN_0.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP119 CE_CAN_0.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP120 CE_CAN_0.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP121 CE_CAN_0.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP122 CE_CAN_0.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP123 CE_CAN_0.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP124 CE_CAN_0.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP125 CE_CAN_0.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP126 CE_CAN_0.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_0_HR_TIME_STAMP127 CE_CAN_0.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_0_ERFFEL0     CE_CAN_0.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL1     CE_CAN_0.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL2     CE_CAN_0.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL3     CE_CAN_0.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL4     CE_CAN_0.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL5     CE_CAN_0.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL6     CE_CAN_0.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL7     CE_CAN_0.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL8     CE_CAN_0.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL9     CE_CAN_0.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL10    CE_CAN_0.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL11    CE_CAN_0.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL12    CE_CAN_0.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL13    CE_CAN_0.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL14    CE_CAN_0.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL15    CE_CAN_0.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL16    CE_CAN_0.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL17    CE_CAN_0.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL18    CE_CAN_0.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL19    CE_CAN_0.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL20    CE_CAN_0.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL21    CE_CAN_0.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL22    CE_CAN_0.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL23    CE_CAN_0.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL24    CE_CAN_0.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL25    CE_CAN_0.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL26    CE_CAN_0.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL27    CE_CAN_0.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL28    CE_CAN_0.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL29    CE_CAN_0.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL30    CE_CAN_0.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL31    CE_CAN_0.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL32    CE_CAN_0.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL33    CE_CAN_0.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL34    CE_CAN_0.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL35    CE_CAN_0.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL36    CE_CAN_0.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL37    CE_CAN_0.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL38    CE_CAN_0.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL39    CE_CAN_0.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL40    CE_CAN_0.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL41    CE_CAN_0.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL42    CE_CAN_0.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL43    CE_CAN_0.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL44    CE_CAN_0.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL45    CE_CAN_0.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL46    CE_CAN_0.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL47    CE_CAN_0.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL48    CE_CAN_0.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL49    CE_CAN_0.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL50    CE_CAN_0.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL51    CE_CAN_0.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL52    CE_CAN_0.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL53    CE_CAN_0.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL54    CE_CAN_0.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL55    CE_CAN_0.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL56    CE_CAN_0.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL57    CE_CAN_0.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL58    CE_CAN_0.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL59    CE_CAN_0.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL60    CE_CAN_0.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL61    CE_CAN_0.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL62    CE_CAN_0.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL63    CE_CAN_0.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL64    CE_CAN_0.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL65    CE_CAN_0.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL66    CE_CAN_0.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL67    CE_CAN_0.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL68    CE_CAN_0.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL69    CE_CAN_0.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL70    CE_CAN_0.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL71    CE_CAN_0.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL72    CE_CAN_0.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL73    CE_CAN_0.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL74    CE_CAN_0.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL75    CE_CAN_0.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL76    CE_CAN_0.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL77    CE_CAN_0.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL78    CE_CAN_0.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL79    CE_CAN_0.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL80    CE_CAN_0.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL81    CE_CAN_0.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL82    CE_CAN_0.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL83    CE_CAN_0.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL84    CE_CAN_0.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL85    CE_CAN_0.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL86    CE_CAN_0.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL87    CE_CAN_0.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL88    CE_CAN_0.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL89    CE_CAN_0.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL90    CE_CAN_0.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL91    CE_CAN_0.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL92    CE_CAN_0.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL93    CE_CAN_0.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL94    CE_CAN_0.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL95    CE_CAN_0.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL96    CE_CAN_0.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL97    CE_CAN_0.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL98    CE_CAN_0.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL99    CE_CAN_0.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL100   CE_CAN_0.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL101   CE_CAN_0.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL102   CE_CAN_0.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL103   CE_CAN_0.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL104   CE_CAN_0.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL105   CE_CAN_0.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL106   CE_CAN_0.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL107   CE_CAN_0.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL108   CE_CAN_0.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL109   CE_CAN_0.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL110   CE_CAN_0.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL111   CE_CAN_0.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL112   CE_CAN_0.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL113   CE_CAN_0.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL114   CE_CAN_0.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL115   CE_CAN_0.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL116   CE_CAN_0.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL117   CE_CAN_0.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL118   CE_CAN_0.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL119   CE_CAN_0.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL120   CE_CAN_0.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL121   CE_CAN_0.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL122   CE_CAN_0.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL123   CE_CAN_0.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL124   CE_CAN_0.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL125   CE_CAN_0.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL126   CE_CAN_0.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_0_ERFFEL127   CE_CAN_0.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_1_MCR         CE_CAN_1.MCR.R                /* Module Configuration */
#define CE_CAN_1_CTRL1       CE_CAN_1.CTRL1.R              /* Control 1 */
#define CE_CAN_1_TIMER       CE_CAN_1.TIMER.R              /* Free-Running Timer */
#define CE_CAN_1_RXMGMASK    CE_CAN_1.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_1_RX14MASK    CE_CAN_1.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_1_RX15MASK    CE_CAN_1.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_1_ECR         CE_CAN_1.ECR.R                /* Error Counter */
#define CE_CAN_1_ESR1        CE_CAN_1.ESR1.R               /* Error and Status 1 */
#define CE_CAN_1_IMASK2      CE_CAN_1.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_1_IMASK1      CE_CAN_1.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_1_IFLAG2      CE_CAN_1.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_1_IFLAG1      CE_CAN_1.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_1_CTRL2       CE_CAN_1.CTRL2.R              /* Control 2 */
#define CE_CAN_1_ESR2        CE_CAN_1.ESR2.R               /* Error and Status 2 */
#define CE_CAN_1_CRCR        CE_CAN_1.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_1_RXFGMASK    CE_CAN_1.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_1_RXFIR       CE_CAN_1.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_1_CBT         CE_CAN_1.CBT.R                /* CAN Bit Timing */
#define CE_CAN_1_IMASK4      CE_CAN_1.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_1_IMASK3      CE_CAN_1.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_1_IFLAG4      CE_CAN_1.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_1_IFLAG3      CE_CAN_1.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_1_CS0            CE_CAN_1.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_1_ID0            CE_CAN_1.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_1_B0             CE_CAN_1.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_1_H0             CE_CAN_1.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_1_W0             CE_CAN_1.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_1_CS1            CE_CAN_1.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_1_ID1            CE_CAN_1.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_1_B1             CE_CAN_1.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_1_H1             CE_CAN_1.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_1_W1             CE_CAN_1.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_1_CS2            CE_CAN_1.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_1_ID2            CE_CAN_1.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_1_B2             CE_CAN_1.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_1_H2             CE_CAN_1.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_1_W2             CE_CAN_1.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_1_CS3            CE_CAN_1.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_1_ID3            CE_CAN_1.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_1_B3             CE_CAN_1.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_1_H3             CE_CAN_1.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_1_W3             CE_CAN_1.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_1_CS4            CE_CAN_1.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_1_ID4            CE_CAN_1.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_1_B4             CE_CAN_1.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_1_H4             CE_CAN_1.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_1_W4             CE_CAN_1.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_1_CS5            CE_CAN_1.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_1_ID5            CE_CAN_1.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_1_B5             CE_CAN_1.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_1_H5             CE_CAN_1.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_1_W5             CE_CAN_1.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_1_CS6            CE_CAN_1.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_1_ID6            CE_CAN_1.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_1_B6             CE_CAN_1.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_1_H6             CE_CAN_1.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_1_W6             CE_CAN_1.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_1_CS7            CE_CAN_1.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_1_ID7            CE_CAN_1.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_1_B7             CE_CAN_1.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_1_H7             CE_CAN_1.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_1_W7             CE_CAN_1.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_1_CS8            CE_CAN_1.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_1_ID8            CE_CAN_1.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_1_B8             CE_CAN_1.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_1_H8             CE_CAN_1.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_1_W8             CE_CAN_1.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_1_CS9            CE_CAN_1.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_1_ID9            CE_CAN_1.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_1_B9             CE_CAN_1.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_1_H9             CE_CAN_1.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_1_W9             CE_CAN_1.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_1_CS10           CE_CAN_1.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_1_ID10           CE_CAN_1.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_1_B10            CE_CAN_1.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_1_H10            CE_CAN_1.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_1_W10            CE_CAN_1.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_1_CS11           CE_CAN_1.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_1_ID11           CE_CAN_1.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_1_B11            CE_CAN_1.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_1_H11            CE_CAN_1.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_1_W11            CE_CAN_1.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_1_CS12           CE_CAN_1.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_1_ID12           CE_CAN_1.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_1_B12            CE_CAN_1.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_1_H12            CE_CAN_1.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_1_W12            CE_CAN_1.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_1_CS13           CE_CAN_1.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_1_ID13           CE_CAN_1.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_1_B13            CE_CAN_1.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_1_H13            CE_CAN_1.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_1_W13            CE_CAN_1.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_1_CS14           CE_CAN_1.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_1_ID14           CE_CAN_1.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_1_B14            CE_CAN_1.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_1_H14            CE_CAN_1.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_1_W14            CE_CAN_1.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_1_CS15           CE_CAN_1.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_1_ID15           CE_CAN_1.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_1_B15            CE_CAN_1.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_1_H15            CE_CAN_1.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_1_W15            CE_CAN_1.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_1_CS16           CE_CAN_1.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_1_ID16           CE_CAN_1.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_1_B16            CE_CAN_1.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_1_H16            CE_CAN_1.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_1_W16            CE_CAN_1.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_1_CS17           CE_CAN_1.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_1_ID17           CE_CAN_1.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_1_B17            CE_CAN_1.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_1_H17            CE_CAN_1.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_1_W17            CE_CAN_1.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_1_CS18           CE_CAN_1.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_1_ID18           CE_CAN_1.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_1_B18            CE_CAN_1.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_1_H18            CE_CAN_1.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_1_W18            CE_CAN_1.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_1_CS19           CE_CAN_1.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_1_ID19           CE_CAN_1.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_1_B19            CE_CAN_1.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_1_H19            CE_CAN_1.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_1_W19            CE_CAN_1.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_1_CS20           CE_CAN_1.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_1_ID20           CE_CAN_1.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_1_B20            CE_CAN_1.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_1_H20            CE_CAN_1.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_1_W20            CE_CAN_1.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_1_CS21           CE_CAN_1.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_1_ID21           CE_CAN_1.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_1_B21            CE_CAN_1.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_1_H21            CE_CAN_1.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_1_W21            CE_CAN_1.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_1_CS22           CE_CAN_1.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_1_ID22           CE_CAN_1.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_1_B22            CE_CAN_1.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_1_H22            CE_CAN_1.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_1_W22            CE_CAN_1.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_1_CS23           CE_CAN_1.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_1_ID23           CE_CAN_1.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_1_B23            CE_CAN_1.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_1_H23            CE_CAN_1.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_1_W23            CE_CAN_1.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_1_CS24           CE_CAN_1.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_1_ID24           CE_CAN_1.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_1_B24            CE_CAN_1.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_1_H24            CE_CAN_1.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_1_W24            CE_CAN_1.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_1_CS25           CE_CAN_1.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_1_ID25           CE_CAN_1.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_1_B25            CE_CAN_1.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_1_H25            CE_CAN_1.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_1_W25            CE_CAN_1.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_1_CS26           CE_CAN_1.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_1_ID26           CE_CAN_1.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_1_B26            CE_CAN_1.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_1_H26            CE_CAN_1.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_1_W26            CE_CAN_1.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_1_CS27           CE_CAN_1.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_1_ID27           CE_CAN_1.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_1_B27            CE_CAN_1.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_1_H27            CE_CAN_1.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_1_W27            CE_CAN_1.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_1_CS28           CE_CAN_1.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_1_ID28           CE_CAN_1.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_1_B28            CE_CAN_1.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_1_H28            CE_CAN_1.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_1_W28            CE_CAN_1.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_1_CS29           CE_CAN_1.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_1_ID29           CE_CAN_1.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_1_B29            CE_CAN_1.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_1_H29            CE_CAN_1.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_1_W29            CE_CAN_1.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_1_CS30           CE_CAN_1.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_1_ID30           CE_CAN_1.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_1_B30            CE_CAN_1.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_1_H30            CE_CAN_1.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_1_W30            CE_CAN_1.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_1_CS31           CE_CAN_1.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_1_ID31           CE_CAN_1.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_1_B31            CE_CAN_1.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_1_H31            CE_CAN_1.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_1_W31            CE_CAN_1.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_1_CS32           CE_CAN_1.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_1_ID32           CE_CAN_1.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_1_B32            CE_CAN_1.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_1_H32            CE_CAN_1.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_1_W32            CE_CAN_1.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_1_CS33           CE_CAN_1.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_1_ID33           CE_CAN_1.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_1_B33            CE_CAN_1.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_1_H33            CE_CAN_1.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_1_W33            CE_CAN_1.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_1_CS34           CE_CAN_1.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_1_ID34           CE_CAN_1.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_1_B34            CE_CAN_1.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_1_H34            CE_CAN_1.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_1_W34            CE_CAN_1.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_1_CS35           CE_CAN_1.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_1_ID35           CE_CAN_1.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_1_B35            CE_CAN_1.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_1_H35            CE_CAN_1.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_1_W35            CE_CAN_1.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_1_CS36           CE_CAN_1.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_1_ID36           CE_CAN_1.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_1_B36            CE_CAN_1.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_1_H36            CE_CAN_1.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_1_W36            CE_CAN_1.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_1_CS37           CE_CAN_1.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_1_ID37           CE_CAN_1.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_1_B37            CE_CAN_1.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_1_H37            CE_CAN_1.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_1_W37            CE_CAN_1.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_1_CS38           CE_CAN_1.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_1_ID38           CE_CAN_1.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_1_B38            CE_CAN_1.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_1_H38            CE_CAN_1.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_1_W38            CE_CAN_1.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_1_CS39           CE_CAN_1.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_1_ID39           CE_CAN_1.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_1_B39            CE_CAN_1.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_1_H39            CE_CAN_1.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_1_W39            CE_CAN_1.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_1_CS40           CE_CAN_1.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_1_ID40           CE_CAN_1.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_1_B40            CE_CAN_1.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_1_H40            CE_CAN_1.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_1_W40            CE_CAN_1.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_1_CS41           CE_CAN_1.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_1_ID41           CE_CAN_1.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_1_B41            CE_CAN_1.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_1_H41            CE_CAN_1.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_1_W41            CE_CAN_1.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_1_CS42           CE_CAN_1.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_1_ID42           CE_CAN_1.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_1_B42            CE_CAN_1.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_1_H42            CE_CAN_1.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_1_W42            CE_CAN_1.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_1_CS43           CE_CAN_1.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_1_ID43           CE_CAN_1.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_1_B43            CE_CAN_1.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_1_H43            CE_CAN_1.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_1_W43            CE_CAN_1.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_1_CS44           CE_CAN_1.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_1_ID44           CE_CAN_1.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_1_B44            CE_CAN_1.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_1_H44            CE_CAN_1.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_1_W44            CE_CAN_1.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_1_CS45           CE_CAN_1.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_1_ID45           CE_CAN_1.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_1_B45            CE_CAN_1.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_1_H45            CE_CAN_1.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_1_W45            CE_CAN_1.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_1_CS46           CE_CAN_1.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_1_ID46           CE_CAN_1.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_1_B46            CE_CAN_1.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_1_H46            CE_CAN_1.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_1_W46            CE_CAN_1.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_1_CS47           CE_CAN_1.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_1_ID47           CE_CAN_1.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_1_B47            CE_CAN_1.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_1_H47            CE_CAN_1.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_1_W47            CE_CAN_1.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_1_CS48           CE_CAN_1.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_1_ID48           CE_CAN_1.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_1_B48            CE_CAN_1.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_1_H48            CE_CAN_1.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_1_W48            CE_CAN_1.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_1_CS49           CE_CAN_1.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_1_ID49           CE_CAN_1.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_1_B49            CE_CAN_1.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_1_H49            CE_CAN_1.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_1_W49            CE_CAN_1.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_1_CS50           CE_CAN_1.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_1_ID50           CE_CAN_1.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_1_B50            CE_CAN_1.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_1_H50            CE_CAN_1.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_1_W50            CE_CAN_1.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_1_CS51           CE_CAN_1.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_1_ID51           CE_CAN_1.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_1_B51            CE_CAN_1.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_1_H51            CE_CAN_1.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_1_W51            CE_CAN_1.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_1_CS52           CE_CAN_1.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_1_ID52           CE_CAN_1.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_1_B52            CE_CAN_1.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_1_H52            CE_CAN_1.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_1_W52            CE_CAN_1.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_1_CS53           CE_CAN_1.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_1_ID53           CE_CAN_1.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_1_B53            CE_CAN_1.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_1_H53            CE_CAN_1.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_1_W53            CE_CAN_1.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_1_CS54           CE_CAN_1.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_1_ID54           CE_CAN_1.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_1_B54            CE_CAN_1.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_1_H54            CE_CAN_1.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_1_W54            CE_CAN_1.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_1_CS55           CE_CAN_1.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_1_ID55           CE_CAN_1.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_1_B55            CE_CAN_1.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_1_H55            CE_CAN_1.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_1_W55            CE_CAN_1.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_1_CS56           CE_CAN_1.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_1_ID56           CE_CAN_1.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_1_B56            CE_CAN_1.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_1_H56            CE_CAN_1.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_1_W56            CE_CAN_1.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_1_CS57           CE_CAN_1.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_1_ID57           CE_CAN_1.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_1_B57            CE_CAN_1.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_1_H57            CE_CAN_1.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_1_W57            CE_CAN_1.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_1_CS58           CE_CAN_1.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_1_ID58           CE_CAN_1.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_1_B58            CE_CAN_1.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_1_H58            CE_CAN_1.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_1_W58            CE_CAN_1.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_1_CS59           CE_CAN_1.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_1_ID59           CE_CAN_1.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_1_B59            CE_CAN_1.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_1_H59            CE_CAN_1.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_1_W59            CE_CAN_1.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_1_CS60           CE_CAN_1.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_1_ID60           CE_CAN_1.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_1_B60            CE_CAN_1.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_1_H60            CE_CAN_1.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_1_W60            CE_CAN_1.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_1_CS61           CE_CAN_1.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_1_ID61           CE_CAN_1.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_1_B61            CE_CAN_1.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_1_H61            CE_CAN_1.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_1_W61            CE_CAN_1.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_1_CS62           CE_CAN_1.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_1_ID62           CE_CAN_1.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_1_B62            CE_CAN_1.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_1_H62            CE_CAN_1.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_1_W62            CE_CAN_1.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_1_CS63           CE_CAN_1.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_1_ID63           CE_CAN_1.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_1_B63            CE_CAN_1.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_1_H63            CE_CAN_1.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_1_W63            CE_CAN_1.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_1_CS64           CE_CAN_1.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_1_ID64           CE_CAN_1.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_1_B64            CE_CAN_1.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_1_H64            CE_CAN_1.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_1_W64            CE_CAN_1.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_1_CS65           CE_CAN_1.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_1_ID65           CE_CAN_1.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_1_B65            CE_CAN_1.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_1_H65            CE_CAN_1.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_1_W65            CE_CAN_1.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_1_CS66           CE_CAN_1.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_1_ID66           CE_CAN_1.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_1_B66            CE_CAN_1.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_1_H66            CE_CAN_1.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_1_W66            CE_CAN_1.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_1_CS67           CE_CAN_1.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_1_ID67           CE_CAN_1.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_1_B67            CE_CAN_1.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_1_H67            CE_CAN_1.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_1_W67            CE_CAN_1.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_1_CS68           CE_CAN_1.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_1_ID68           CE_CAN_1.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_1_B68            CE_CAN_1.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_1_H68            CE_CAN_1.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_1_W68            CE_CAN_1.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_1_CS69           CE_CAN_1.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_1_ID69           CE_CAN_1.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_1_B69            CE_CAN_1.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_1_H69            CE_CAN_1.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_1_W69            CE_CAN_1.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_1_CS70           CE_CAN_1.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_1_ID70           CE_CAN_1.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_1_B70            CE_CAN_1.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_1_H70            CE_CAN_1.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_1_W70            CE_CAN_1.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_1_CS71           CE_CAN_1.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_1_ID71           CE_CAN_1.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_1_B71            CE_CAN_1.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_1_H71            CE_CAN_1.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_1_W71            CE_CAN_1.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_1_CS72           CE_CAN_1.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_1_ID72           CE_CAN_1.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_1_B72            CE_CAN_1.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_1_H72            CE_CAN_1.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_1_W72            CE_CAN_1.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_1_CS73           CE_CAN_1.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_1_ID73           CE_CAN_1.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_1_B73            CE_CAN_1.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_1_H73            CE_CAN_1.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_1_W73            CE_CAN_1.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_1_CS74           CE_CAN_1.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_1_ID74           CE_CAN_1.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_1_B74            CE_CAN_1.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_1_H74            CE_CAN_1.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_1_W74            CE_CAN_1.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_1_CS75           CE_CAN_1.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_1_ID75           CE_CAN_1.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_1_B75            CE_CAN_1.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_1_H75            CE_CAN_1.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_1_W75            CE_CAN_1.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_1_CS76           CE_CAN_1.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_1_ID76           CE_CAN_1.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_1_B76            CE_CAN_1.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_1_H76            CE_CAN_1.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_1_W76            CE_CAN_1.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_1_CS77           CE_CAN_1.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_1_ID77           CE_CAN_1.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_1_B77            CE_CAN_1.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_1_H77            CE_CAN_1.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_1_W77            CE_CAN_1.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_1_CS78           CE_CAN_1.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_1_ID78           CE_CAN_1.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_1_B78            CE_CAN_1.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_1_H78            CE_CAN_1.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_1_W78            CE_CAN_1.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_1_CS79           CE_CAN_1.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_1_ID79           CE_CAN_1.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_1_B79            CE_CAN_1.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_1_H79            CE_CAN_1.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_1_W79            CE_CAN_1.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_1_CS80           CE_CAN_1.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_1_ID80           CE_CAN_1.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_1_B80            CE_CAN_1.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_1_H80            CE_CAN_1.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_1_W80            CE_CAN_1.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_1_CS81           CE_CAN_1.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_1_ID81           CE_CAN_1.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_1_B81            CE_CAN_1.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_1_H81            CE_CAN_1.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_1_W81            CE_CAN_1.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_1_CS82           CE_CAN_1.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_1_ID82           CE_CAN_1.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_1_B82            CE_CAN_1.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_1_H82            CE_CAN_1.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_1_W82            CE_CAN_1.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_1_CS83           CE_CAN_1.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_1_ID83           CE_CAN_1.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_1_B83            CE_CAN_1.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_1_H83            CE_CAN_1.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_1_W83            CE_CAN_1.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_1_CS84           CE_CAN_1.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_1_ID84           CE_CAN_1.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_1_B84            CE_CAN_1.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_1_H84            CE_CAN_1.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_1_W84            CE_CAN_1.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_1_CS85           CE_CAN_1.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_1_ID85           CE_CAN_1.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_1_B85            CE_CAN_1.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_1_H85            CE_CAN_1.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_1_W85            CE_CAN_1.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_1_CS86           CE_CAN_1.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_1_ID86           CE_CAN_1.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_1_B86            CE_CAN_1.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_1_H86            CE_CAN_1.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_1_W86            CE_CAN_1.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_1_CS87           CE_CAN_1.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_1_ID87           CE_CAN_1.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_1_B87            CE_CAN_1.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_1_H87            CE_CAN_1.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_1_W87            CE_CAN_1.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_1_CS88           CE_CAN_1.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_1_ID88           CE_CAN_1.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_1_B88            CE_CAN_1.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_1_H88            CE_CAN_1.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_1_W848           CE_CAN_1.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_1_CS89           CE_CAN_1.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_1_ID89           CE_CAN_1.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_1_B89            CE_CAN_1.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_1_H89            CE_CAN_1.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_1_W89            CE_CAN_1.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_1_CS90           CE_CAN_1.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_1_ID90           CE_CAN_1.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_1_B90            CE_CAN_1.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_1_H90            CE_CAN_1.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_1_W90            CE_CAN_1.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_1_CS91           CE_CAN_1.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_1_ID91           CE_CAN_1.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_1_B91            CE_CAN_1.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_1_H91            CE_CAN_1.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_1_W91            CE_CAN_1.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_1_CS92           CE_CAN_1.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_1_ID92           CE_CAN_1.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_1_B92            CE_CAN_1.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_1_H92            CE_CAN_1.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_1_W92            CE_CAN_1.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_1_CS93           CE_CAN_1.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_1_ID93           CE_CAN_1.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_1_B93            CE_CAN_1.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_1_H93            CE_CAN_1.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_1_W93            CE_CAN_1.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_1_CS94           CE_CAN_1.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_1_ID94           CE_CAN_1.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_1_B94            CE_CAN_1.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_1_H94            CE_CAN_1.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_1_W94            CE_CAN_1.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_1_CS95           CE_CAN_1.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_1_ID95           CE_CAN_1.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_1_B95            CE_CAN_1.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_1_H95            CE_CAN_1.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_1_W95            CE_CAN_1.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_1_CS96           CE_CAN_1.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_1_ID96           CE_CAN_1.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_1_B96            CE_CAN_1.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_1_H96            CE_CAN_1.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_1_W96            CE_CAN_1.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_1_CS97           CE_CAN_1.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_1_ID97           CE_CAN_1.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_1_B97            CE_CAN_1.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_1_H97            CE_CAN_1.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_1_W97            CE_CAN_1.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_1_CS98           CE_CAN_1.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_1_ID98           CE_CAN_1.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_1_B98            CE_CAN_1.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_1_H98            CE_CAN_1.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_1_W98            CE_CAN_1.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_1_CS99           CE_CAN_1.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_1_ID99           CE_CAN_1.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_1_B99            CE_CAN_1.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_1_H99            CE_CAN_1.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_1_W99            CE_CAN_1.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_1_CS100          CE_CAN_1.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_1_ID100          CE_CAN_1.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_1_B100           CE_CAN_1.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_1_H100           CE_CAN_1.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_1_W100           CE_CAN_1.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_1_CS101          CE_CAN_1.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_1_ID101          CE_CAN_1.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_1_B101           CE_CAN_1.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_1_H101           CE_CAN_1.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_1_W101           CE_CAN_1.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_1_CS102          CE_CAN_1.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_1_ID102          CE_CAN_1.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_1_B102           CE_CAN_1.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_1_H102           CE_CAN_1.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_1_W102           CE_CAN_1.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_1_CS103          CE_CAN_1.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_1_ID103          CE_CAN_1.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_1_B103           CE_CAN_1.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_1_H103           CE_CAN_1.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_1_W103           CE_CAN_1.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_1_CS104          CE_CAN_1.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_1_ID104          CE_CAN_1.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_1_B104           CE_CAN_1.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_1_H104           CE_CAN_1.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_1_W104           CE_CAN_1.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_1_CS105          CE_CAN_1.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_1_ID105          CE_CAN_1.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_1_B105           CE_CAN_1.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_1_H105           CE_CAN_1.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_1_W105           CE_CAN_1.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_1_CS106          CE_CAN_1.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_1_ID106          CE_CAN_1.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_1_B106           CE_CAN_1.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_1_H106           CE_CAN_1.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_1_W106           CE_CAN_1.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_1_CS107          CE_CAN_1.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_1_ID107          CE_CAN_1.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_1_B107           CE_CAN_1.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_1_H107           CE_CAN_1.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_1_W107           CE_CAN_1.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_1_CS108          CE_CAN_1.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_1_ID108          CE_CAN_1.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_1_B108           CE_CAN_1.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_1_H108           CE_CAN_1.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_1_W108           CE_CAN_1.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_1_CS109          CE_CAN_1.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_1_ID109          CE_CAN_1.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_1_B109           CE_CAN_1.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_1_H109           CE_CAN_1.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_1_W109           CE_CAN_1.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_1_CS110          CE_CAN_1.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_1_ID110          CE_CAN_1.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_1_B110           CE_CAN_1.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_1_H110           CE_CAN_1.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_1_W110           CE_CAN_1.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_1_CS111          CE_CAN_1.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_1_ID111          CE_CAN_1.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_1_B111           CE_CAN_1.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_1_H111           CE_CAN_1.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_1_W111           CE_CAN_1.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_1_CS112          CE_CAN_1.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_1_ID112          CE_CAN_1.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_1_B112           CE_CAN_1.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_1_H112           CE_CAN_1.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_1_W112           CE_CAN_1.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_1_CS113          CE_CAN_1.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_1_ID113          CE_CAN_1.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_1_B113           CE_CAN_1.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_1_H113           CE_CAN_1.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_1_W113           CE_CAN_1.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_1_CS114          CE_CAN_1.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_1_ID114          CE_CAN_1.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_1_B114           CE_CAN_1.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_1_H114           CE_CAN_1.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_1_W114           CE_CAN_1.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_1_CS115          CE_CAN_1.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_1_ID115          CE_CAN_1.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_1_B115           CE_CAN_1.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_1_H115           CE_CAN_1.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_1_W115           CE_CAN_1.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_1_CS116          CE_CAN_1.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_1_ID116          CE_CAN_1.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_1_B116           CE_CAN_1.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_1_H116           CE_CAN_1.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_1_W116           CE_CAN_1.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_1_CS117          CE_CAN_1.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_1_ID117          CE_CAN_1.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_1_B117           CE_CAN_1.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_1_H117           CE_CAN_1.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_1_W117           CE_CAN_1.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_1_CS118          CE_CAN_1.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_1_ID118          CE_CAN_1.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_1_B118           CE_CAN_1.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_1_H118           CE_CAN_1.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_1_W118           CE_CAN_1.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_1_CS119          CE_CAN_1.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_1_ID119          CE_CAN_1.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_1_B119           CE_CAN_1.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_1_H119           CE_CAN_1.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_1_W119           CE_CAN_1.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_1_CS120          CE_CAN_1.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_1_ID120          CE_CAN_1.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_1_B120           CE_CAN_1.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_1_H120           CE_CAN_1.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_1_W120           CE_CAN_1.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_1_CS121          CE_CAN_1.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_1_ID121          CE_CAN_1.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_1_B121           CE_CAN_1.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_1_H121           CE_CAN_1.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_1_W121           CE_CAN_1.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_1_CS122          CE_CAN_1.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_1_ID122          CE_CAN_1.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_1_B122           CE_CAN_1.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_1_H122           CE_CAN_1.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_1_W122           CE_CAN_1.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_1_CS123          CE_CAN_1.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_1_ID123          CE_CAN_1.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_1_B123           CE_CAN_1.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_1_H123           CE_CAN_1.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_1_W123           CE_CAN_1.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_1_CS124          CE_CAN_1.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_1_ID124          CE_CAN_1.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_1_B124           CE_CAN_1.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_1_H124           CE_CAN_1.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_1_W124           CE_CAN_1.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_1_CS125          CE_CAN_1.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_1_ID125          CE_CAN_1.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_1_B125           CE_CAN_1.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_1_H125           CE_CAN_1.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_1_W125           CE_CAN_1.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_1_CS126          CE_CAN_1.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_1_ID126          CE_CAN_1.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_1_B126           CE_CAN_1.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_1_H126           CE_CAN_1.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_1_W126           CE_CAN_1.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_1_CS127          CE_CAN_1.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_1_ID127          CE_CAN_1.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_1_B127           CE_CAN_1.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_1_H127           CE_CAN_1.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_1_W127           CE_CAN_1.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_1_RXIMR0      CE_CAN_1.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR1      CE_CAN_1.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR2      CE_CAN_1.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR3      CE_CAN_1.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR4      CE_CAN_1.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR5      CE_CAN_1.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR6      CE_CAN_1.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR7      CE_CAN_1.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR8      CE_CAN_1.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR9      CE_CAN_1.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_1_RXIMR10     CE_CAN_1.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR11     CE_CAN_1.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR12     CE_CAN_1.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR13     CE_CAN_1.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR14     CE_CAN_1.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR15     CE_CAN_1.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR16     CE_CAN_1.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR17     CE_CAN_1.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR18     CE_CAN_1.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR19     CE_CAN_1.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR20     CE_CAN_1.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR21     CE_CAN_1.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR22     CE_CAN_1.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR23     CE_CAN_1.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR24     CE_CAN_1.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR25     CE_CAN_1.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR26     CE_CAN_1.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR27     CE_CAN_1.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR28     CE_CAN_1.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR29     CE_CAN_1.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR30     CE_CAN_1.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR31     CE_CAN_1.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR32     CE_CAN_1.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR33     CE_CAN_1.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR34     CE_CAN_1.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR35     CE_CAN_1.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR36     CE_CAN_1.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR37     CE_CAN_1.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR38     CE_CAN_1.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR39     CE_CAN_1.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR40     CE_CAN_1.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR41     CE_CAN_1.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR42     CE_CAN_1.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR43     CE_CAN_1.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR44     CE_CAN_1.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR45     CE_CAN_1.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR46     CE_CAN_1.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR47     CE_CAN_1.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR48     CE_CAN_1.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR49     CE_CAN_1.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR50     CE_CAN_1.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR51     CE_CAN_1.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR52     CE_CAN_1.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR53     CE_CAN_1.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR54     CE_CAN_1.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR55     CE_CAN_1.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR56     CE_CAN_1.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR57     CE_CAN_1.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR58     CE_CAN_1.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR59     CE_CAN_1.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR60     CE_CAN_1.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR61     CE_CAN_1.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR62     CE_CAN_1.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR63     CE_CAN_1.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR64     CE_CAN_1.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR65     CE_CAN_1.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR66     CE_CAN_1.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR67     CE_CAN_1.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR68     CE_CAN_1.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR69     CE_CAN_1.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR70     CE_CAN_1.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR71     CE_CAN_1.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR72     CE_CAN_1.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR73     CE_CAN_1.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR74     CE_CAN_1.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR75     CE_CAN_1.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR76     CE_CAN_1.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR77     CE_CAN_1.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR78     CE_CAN_1.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR79     CE_CAN_1.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR80     CE_CAN_1.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR81     CE_CAN_1.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR82     CE_CAN_1.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR83     CE_CAN_1.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR84     CE_CAN_1.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR85     CE_CAN_1.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR86     CE_CAN_1.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR87     CE_CAN_1.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR88     CE_CAN_1.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR89     CE_CAN_1.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR90     CE_CAN_1.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR91     CE_CAN_1.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR92     CE_CAN_1.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR93     CE_CAN_1.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR94     CE_CAN_1.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR95     CE_CAN_1.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR96     CE_CAN_1.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR97     CE_CAN_1.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR98     CE_CAN_1.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR99     CE_CAN_1.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_1_RXIMR100    CE_CAN_1.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR101    CE_CAN_1.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR102    CE_CAN_1.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR103    CE_CAN_1.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR104    CE_CAN_1.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR105    CE_CAN_1.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR106    CE_CAN_1.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR107    CE_CAN_1.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR108    CE_CAN_1.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR109    CE_CAN_1.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR110    CE_CAN_1.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR111    CE_CAN_1.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR112    CE_CAN_1.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR113    CE_CAN_1.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR114    CE_CAN_1.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR115    CE_CAN_1.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR116    CE_CAN_1.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR117    CE_CAN_1.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR118    CE_CAN_1.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR119    CE_CAN_1.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR120    CE_CAN_1.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR121    CE_CAN_1.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR122    CE_CAN_1.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR123    CE_CAN_1.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR124    CE_CAN_1.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR125    CE_CAN_1.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR126    CE_CAN_1.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_1_RXIMR127    CE_CAN_1.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_1_MECR        CE_CAN_1.MECR.R               /* Memory Error Control */
#define CE_CAN_1_ERRIAR      CE_CAN_1.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_1_ERRIDPR     CE_CAN_1.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_1_ERRIPPR     CE_CAN_1.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_1_RERRAR      CE_CAN_1.RERRAR.R             /* Error Report Address */
#define CE_CAN_1_RERRDR      CE_CAN_1.RERRDR.R             /* Error Report Data */
#define CE_CAN_1_RERRSYNR    CE_CAN_1.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_1_ERRSR       CE_CAN_1.ERRSR.R              /* Error Status */
#define CE_CAN_1_EPRS        CE_CAN_1.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_1_ENCBT       CE_CAN_1.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_1_EDCBT       CE_CAN_1.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_1_ETDC        CE_CAN_1.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_1_FDCTRL      CE_CAN_1.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_1_FDCBT       CE_CAN_1.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_1_FDCRC       CE_CAN_1.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_1_ERFCR       CE_CAN_1.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_1_ERFIER      CE_CAN_1.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_1_ERFSR       CE_CAN_1.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_1_HR_TIME_STAMP0 CE_CAN_1.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP1 CE_CAN_1.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP2 CE_CAN_1.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP3 CE_CAN_1.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP4 CE_CAN_1.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP5 CE_CAN_1.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP6 CE_CAN_1.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP7 CE_CAN_1.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP8 CE_CAN_1.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP9 CE_CAN_1.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP10 CE_CAN_1.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP11 CE_CAN_1.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP12 CE_CAN_1.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP13 CE_CAN_1.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP14 CE_CAN_1.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP15 CE_CAN_1.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP16 CE_CAN_1.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP17 CE_CAN_1.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP18 CE_CAN_1.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP19 CE_CAN_1.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP20 CE_CAN_1.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP21 CE_CAN_1.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP22 CE_CAN_1.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP23 CE_CAN_1.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP24 CE_CAN_1.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP25 CE_CAN_1.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP26 CE_CAN_1.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP27 CE_CAN_1.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP28 CE_CAN_1.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP29 CE_CAN_1.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP30 CE_CAN_1.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP31 CE_CAN_1.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP32 CE_CAN_1.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP33 CE_CAN_1.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP34 CE_CAN_1.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP35 CE_CAN_1.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP36 CE_CAN_1.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP37 CE_CAN_1.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP38 CE_CAN_1.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP39 CE_CAN_1.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP40 CE_CAN_1.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP41 CE_CAN_1.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP42 CE_CAN_1.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP43 CE_CAN_1.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP44 CE_CAN_1.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP45 CE_CAN_1.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP46 CE_CAN_1.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP47 CE_CAN_1.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP48 CE_CAN_1.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP49 CE_CAN_1.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP50 CE_CAN_1.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP51 CE_CAN_1.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP52 CE_CAN_1.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP53 CE_CAN_1.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP54 CE_CAN_1.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP55 CE_CAN_1.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP56 CE_CAN_1.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP57 CE_CAN_1.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP58 CE_CAN_1.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP59 CE_CAN_1.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP60 CE_CAN_1.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP61 CE_CAN_1.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP62 CE_CAN_1.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP63 CE_CAN_1.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP64 CE_CAN_1.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP65 CE_CAN_1.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP66 CE_CAN_1.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP67 CE_CAN_1.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP68 CE_CAN_1.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP69 CE_CAN_1.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP70 CE_CAN_1.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP71 CE_CAN_1.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP72 CE_CAN_1.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP73 CE_CAN_1.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP74 CE_CAN_1.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP75 CE_CAN_1.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP76 CE_CAN_1.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP77 CE_CAN_1.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP78 CE_CAN_1.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP79 CE_CAN_1.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP80 CE_CAN_1.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP81 CE_CAN_1.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP82 CE_CAN_1.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP83 CE_CAN_1.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP84 CE_CAN_1.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP85 CE_CAN_1.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP86 CE_CAN_1.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP87 CE_CAN_1.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP88 CE_CAN_1.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP89 CE_CAN_1.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP90 CE_CAN_1.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP91 CE_CAN_1.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP92 CE_CAN_1.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP93 CE_CAN_1.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP94 CE_CAN_1.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP95 CE_CAN_1.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP96 CE_CAN_1.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP97 CE_CAN_1.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP98 CE_CAN_1.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP99 CE_CAN_1.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP100 CE_CAN_1.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP101 CE_CAN_1.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP102 CE_CAN_1.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP103 CE_CAN_1.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP104 CE_CAN_1.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP105 CE_CAN_1.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP106 CE_CAN_1.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP107 CE_CAN_1.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP108 CE_CAN_1.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP109 CE_CAN_1.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP110 CE_CAN_1.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP111 CE_CAN_1.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP112 CE_CAN_1.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP113 CE_CAN_1.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP114 CE_CAN_1.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP115 CE_CAN_1.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP116 CE_CAN_1.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP117 CE_CAN_1.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP118 CE_CAN_1.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP119 CE_CAN_1.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP120 CE_CAN_1.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP121 CE_CAN_1.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP122 CE_CAN_1.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP123 CE_CAN_1.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP124 CE_CAN_1.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP125 CE_CAN_1.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP126 CE_CAN_1.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_1_HR_TIME_STAMP127 CE_CAN_1.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_1_ERFFEL0     CE_CAN_1.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL1     CE_CAN_1.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL2     CE_CAN_1.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL3     CE_CAN_1.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL4     CE_CAN_1.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL5     CE_CAN_1.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL6     CE_CAN_1.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL7     CE_CAN_1.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL8     CE_CAN_1.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL9     CE_CAN_1.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL10    CE_CAN_1.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL11    CE_CAN_1.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL12    CE_CAN_1.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL13    CE_CAN_1.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL14    CE_CAN_1.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL15    CE_CAN_1.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL16    CE_CAN_1.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL17    CE_CAN_1.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL18    CE_CAN_1.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL19    CE_CAN_1.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL20    CE_CAN_1.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL21    CE_CAN_1.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL22    CE_CAN_1.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL23    CE_CAN_1.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL24    CE_CAN_1.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL25    CE_CAN_1.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL26    CE_CAN_1.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL27    CE_CAN_1.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL28    CE_CAN_1.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL29    CE_CAN_1.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL30    CE_CAN_1.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL31    CE_CAN_1.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL32    CE_CAN_1.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL33    CE_CAN_1.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL34    CE_CAN_1.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL35    CE_CAN_1.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL36    CE_CAN_1.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL37    CE_CAN_1.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL38    CE_CAN_1.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL39    CE_CAN_1.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL40    CE_CAN_1.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL41    CE_CAN_1.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL42    CE_CAN_1.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL43    CE_CAN_1.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL44    CE_CAN_1.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL45    CE_CAN_1.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL46    CE_CAN_1.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL47    CE_CAN_1.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL48    CE_CAN_1.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL49    CE_CAN_1.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL50    CE_CAN_1.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL51    CE_CAN_1.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL52    CE_CAN_1.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL53    CE_CAN_1.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL54    CE_CAN_1.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL55    CE_CAN_1.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL56    CE_CAN_1.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL57    CE_CAN_1.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL58    CE_CAN_1.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL59    CE_CAN_1.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL60    CE_CAN_1.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL61    CE_CAN_1.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL62    CE_CAN_1.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL63    CE_CAN_1.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL64    CE_CAN_1.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL65    CE_CAN_1.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL66    CE_CAN_1.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL67    CE_CAN_1.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL68    CE_CAN_1.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL69    CE_CAN_1.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL70    CE_CAN_1.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL71    CE_CAN_1.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL72    CE_CAN_1.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL73    CE_CAN_1.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL74    CE_CAN_1.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL75    CE_CAN_1.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL76    CE_CAN_1.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL77    CE_CAN_1.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL78    CE_CAN_1.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL79    CE_CAN_1.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL80    CE_CAN_1.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL81    CE_CAN_1.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL82    CE_CAN_1.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL83    CE_CAN_1.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL84    CE_CAN_1.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL85    CE_CAN_1.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL86    CE_CAN_1.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL87    CE_CAN_1.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL88    CE_CAN_1.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL89    CE_CAN_1.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL90    CE_CAN_1.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL91    CE_CAN_1.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL92    CE_CAN_1.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL93    CE_CAN_1.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL94    CE_CAN_1.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL95    CE_CAN_1.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL96    CE_CAN_1.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL97    CE_CAN_1.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL98    CE_CAN_1.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL99    CE_CAN_1.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL100   CE_CAN_1.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL101   CE_CAN_1.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL102   CE_CAN_1.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL103   CE_CAN_1.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL104   CE_CAN_1.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL105   CE_CAN_1.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL106   CE_CAN_1.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL107   CE_CAN_1.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL108   CE_CAN_1.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL109   CE_CAN_1.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL110   CE_CAN_1.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL111   CE_CAN_1.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL112   CE_CAN_1.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL113   CE_CAN_1.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL114   CE_CAN_1.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL115   CE_CAN_1.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL116   CE_CAN_1.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL117   CE_CAN_1.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL118   CE_CAN_1.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL119   CE_CAN_1.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL120   CE_CAN_1.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL121   CE_CAN_1.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL122   CE_CAN_1.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL123   CE_CAN_1.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL124   CE_CAN_1.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL125   CE_CAN_1.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL126   CE_CAN_1.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_1_ERFFEL127   CE_CAN_1.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_10_MCR        CE_CAN_10.MCR.R               /* Module Configuration */
#define CE_CAN_10_CTRL1      CE_CAN_10.CTRL1.R             /* Control 1 */
#define CE_CAN_10_TIMER      CE_CAN_10.TIMER.R             /* Free-Running Timer */
#define CE_CAN_10_RXMGMASK   CE_CAN_10.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_10_RX14MASK   CE_CAN_10.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_10_RX15MASK   CE_CAN_10.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_10_ECR        CE_CAN_10.ECR.R               /* Error Counter */
#define CE_CAN_10_ESR1       CE_CAN_10.ESR1.R              /* Error and Status 1 */
#define CE_CAN_10_IMASK2     CE_CAN_10.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_10_IMASK1     CE_CAN_10.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_10_IFLAG2     CE_CAN_10.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_10_IFLAG1     CE_CAN_10.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_10_CTRL2      CE_CAN_10.CTRL2.R             /* Control 2 */
#define CE_CAN_10_ESR2       CE_CAN_10.ESR2.R              /* Error and Status 2 */
#define CE_CAN_10_CRCR       CE_CAN_10.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_10_RXFGMASK   CE_CAN_10.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_10_RXFIR      CE_CAN_10.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_10_CBT        CE_CAN_10.CBT.R               /* CAN Bit Timing */
#define CE_CAN_10_IMASK4     CE_CAN_10.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_10_IMASK3     CE_CAN_10.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_10_IFLAG4     CE_CAN_10.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_10_IFLAG3     CE_CAN_10.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_10_CS0            CE_CAN_10.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_10_ID0            CE_CAN_10.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_10_B0             CE_CAN_10.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_10_H0             CE_CAN_10.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_10_W0             CE_CAN_10.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_10_CS1            CE_CAN_10.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_10_ID1            CE_CAN_10.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_10_B1             CE_CAN_10.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_10_H1             CE_CAN_10.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_10_W1             CE_CAN_10.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_10_CS2            CE_CAN_10.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_10_ID2            CE_CAN_10.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_10_B2             CE_CAN_10.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_10_H2             CE_CAN_10.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_10_W2             CE_CAN_10.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_10_CS3            CE_CAN_10.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_10_ID3            CE_CAN_10.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_10_B3             CE_CAN_10.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_10_H3             CE_CAN_10.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_10_W3             CE_CAN_10.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_10_CS4            CE_CAN_10.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_10_ID4            CE_CAN_10.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_10_B4             CE_CAN_10.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_10_H4             CE_CAN_10.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_10_W4             CE_CAN_10.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_10_CS5            CE_CAN_10.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_10_ID5            CE_CAN_10.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_10_B5             CE_CAN_10.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_10_H5             CE_CAN_10.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_10_W5             CE_CAN_10.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_10_CS6            CE_CAN_10.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_10_ID6            CE_CAN_10.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_10_B6             CE_CAN_10.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_10_H6             CE_CAN_10.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_10_W6             CE_CAN_10.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_10_CS7            CE_CAN_10.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_10_ID7            CE_CAN_10.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_10_B7             CE_CAN_10.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_10_H7             CE_CAN_10.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_10_W7             CE_CAN_10.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_10_CS8            CE_CAN_10.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_10_ID8            CE_CAN_10.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_10_B8             CE_CAN_10.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_10_H8             CE_CAN_10.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_10_W8             CE_CAN_10.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_10_CS9            CE_CAN_10.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_10_ID9            CE_CAN_10.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_10_B9             CE_CAN_10.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_10_H9             CE_CAN_10.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_10_W9             CE_CAN_10.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_10_CS10           CE_CAN_10.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_10_ID10           CE_CAN_10.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_10_B10            CE_CAN_10.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_10_H10            CE_CAN_10.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_10_W10            CE_CAN_10.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_10_CS11           CE_CAN_10.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_10_ID11           CE_CAN_10.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_10_B11            CE_CAN_10.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_10_H11            CE_CAN_10.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_10_W11            CE_CAN_10.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_10_CS12           CE_CAN_10.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_10_ID12           CE_CAN_10.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_10_B12            CE_CAN_10.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_10_H12            CE_CAN_10.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_10_W12            CE_CAN_10.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_10_CS13           CE_CAN_10.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_10_ID13           CE_CAN_10.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_10_B13            CE_CAN_10.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_10_H13            CE_CAN_10.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_10_W13            CE_CAN_10.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_10_CS14           CE_CAN_10.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_10_ID14           CE_CAN_10.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_10_B14            CE_CAN_10.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_10_H14            CE_CAN_10.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_10_W14            CE_CAN_10.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_10_CS15           CE_CAN_10.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_10_ID15           CE_CAN_10.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_10_B15            CE_CAN_10.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_10_H15            CE_CAN_10.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_10_W15            CE_CAN_10.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_10_CS16           CE_CAN_10.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_10_ID16           CE_CAN_10.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_10_B16            CE_CAN_10.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_10_H16            CE_CAN_10.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_10_W16            CE_CAN_10.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_10_CS17           CE_CAN_10.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_10_ID17           CE_CAN_10.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_10_B17            CE_CAN_10.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_10_H17            CE_CAN_10.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_10_W17            CE_CAN_10.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_10_CS18           CE_CAN_10.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_10_ID18           CE_CAN_10.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_10_B18            CE_CAN_10.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_10_H18            CE_CAN_10.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_10_W18            CE_CAN_10.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_10_CS19           CE_CAN_10.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_10_ID19           CE_CAN_10.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_10_B19            CE_CAN_10.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_10_H19            CE_CAN_10.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_10_W19            CE_CAN_10.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_10_CS20           CE_CAN_10.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_10_ID20           CE_CAN_10.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_10_B20            CE_CAN_10.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_10_H20            CE_CAN_10.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_10_W20            CE_CAN_10.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_10_CS21           CE_CAN_10.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_10_ID21           CE_CAN_10.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_10_B21            CE_CAN_10.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_10_H21            CE_CAN_10.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_10_W21            CE_CAN_10.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_10_CS22           CE_CAN_10.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_10_ID22           CE_CAN_10.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_10_B22            CE_CAN_10.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_10_H22            CE_CAN_10.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_10_W22            CE_CAN_10.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_10_CS23           CE_CAN_10.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_10_ID23           CE_CAN_10.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_10_B23            CE_CAN_10.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_10_H23            CE_CAN_10.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_10_W23            CE_CAN_10.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_10_CS24           CE_CAN_10.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_10_ID24           CE_CAN_10.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_10_B24            CE_CAN_10.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_10_H24            CE_CAN_10.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_10_W24            CE_CAN_10.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_10_CS25           CE_CAN_10.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_10_ID25           CE_CAN_10.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_10_B25            CE_CAN_10.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_10_H25            CE_CAN_10.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_10_W25            CE_CAN_10.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_10_CS26           CE_CAN_10.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_10_ID26           CE_CAN_10.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_10_B26            CE_CAN_10.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_10_H26            CE_CAN_10.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_10_W26            CE_CAN_10.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_10_CS27           CE_CAN_10.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_10_ID27           CE_CAN_10.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_10_B27            CE_CAN_10.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_10_H27            CE_CAN_10.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_10_W27            CE_CAN_10.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_10_CS28           CE_CAN_10.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_10_ID28           CE_CAN_10.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_10_B28            CE_CAN_10.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_10_H28            CE_CAN_10.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_10_W28            CE_CAN_10.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_10_CS29           CE_CAN_10.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_10_ID29           CE_CAN_10.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_10_B29            CE_CAN_10.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_10_H29            CE_CAN_10.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_10_W29            CE_CAN_10.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_10_CS30           CE_CAN_10.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_10_ID30           CE_CAN_10.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_10_B30            CE_CAN_10.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_10_H30            CE_CAN_10.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_10_W30            CE_CAN_10.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_10_CS31           CE_CAN_10.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_10_ID31           CE_CAN_10.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_10_B31            CE_CAN_10.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_10_H31            CE_CAN_10.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_10_W31            CE_CAN_10.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_10_CS32           CE_CAN_10.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_10_ID32           CE_CAN_10.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_10_B32            CE_CAN_10.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_10_H32            CE_CAN_10.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_10_W32            CE_CAN_10.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_10_CS33           CE_CAN_10.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_10_ID33           CE_CAN_10.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_10_B33            CE_CAN_10.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_10_H33            CE_CAN_10.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_10_W33            CE_CAN_10.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_10_CS34           CE_CAN_10.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_10_ID34           CE_CAN_10.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_10_B34            CE_CAN_10.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_10_H34            CE_CAN_10.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_10_W34            CE_CAN_10.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_10_CS35           CE_CAN_10.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_10_ID35           CE_CAN_10.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_10_B35            CE_CAN_10.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_10_H35            CE_CAN_10.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_10_W35            CE_CAN_10.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_10_CS36           CE_CAN_10.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_10_ID36           CE_CAN_10.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_10_B36            CE_CAN_10.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_10_H36            CE_CAN_10.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_10_W36            CE_CAN_10.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_10_CS37           CE_CAN_10.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_10_ID37           CE_CAN_10.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_10_B37            CE_CAN_10.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_10_H37            CE_CAN_10.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_10_W37            CE_CAN_10.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_10_CS38           CE_CAN_10.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_10_ID38           CE_CAN_10.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_10_B38            CE_CAN_10.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_10_H38            CE_CAN_10.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_10_W38            CE_CAN_10.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_10_CS39           CE_CAN_10.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_10_ID39           CE_CAN_10.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_10_B39            CE_CAN_10.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_10_H39            CE_CAN_10.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_10_W39            CE_CAN_10.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_10_CS40           CE_CAN_10.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_10_ID40           CE_CAN_10.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_10_B40            CE_CAN_10.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_10_H40            CE_CAN_10.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_10_W40            CE_CAN_10.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_10_CS41           CE_CAN_10.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_10_ID41           CE_CAN_10.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_10_B41            CE_CAN_10.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_10_H41            CE_CAN_10.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_10_W41            CE_CAN_10.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_10_CS42           CE_CAN_10.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_10_ID42           CE_CAN_10.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_10_B42            CE_CAN_10.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_10_H42            CE_CAN_10.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_10_W42            CE_CAN_10.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_10_CS43           CE_CAN_10.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_10_ID43           CE_CAN_10.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_10_B43            CE_CAN_10.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_10_H43            CE_CAN_10.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_10_W43            CE_CAN_10.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_10_CS44           CE_CAN_10.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_10_ID44           CE_CAN_10.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_10_B44            CE_CAN_10.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_10_H44            CE_CAN_10.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_10_W44            CE_CAN_10.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_10_CS45           CE_CAN_10.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_10_ID45           CE_CAN_10.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_10_B45            CE_CAN_10.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_10_H45            CE_CAN_10.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_10_W45            CE_CAN_10.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_10_CS46           CE_CAN_10.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_10_ID46           CE_CAN_10.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_10_B46            CE_CAN_10.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_10_H46            CE_CAN_10.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_10_W46            CE_CAN_10.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_10_CS47           CE_CAN_10.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_10_ID47           CE_CAN_10.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_10_B47            CE_CAN_10.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_10_H47            CE_CAN_10.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_10_W47            CE_CAN_10.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_10_CS48           CE_CAN_10.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_10_ID48           CE_CAN_10.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_10_B48            CE_CAN_10.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_10_H48            CE_CAN_10.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_10_W48            CE_CAN_10.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_10_CS49           CE_CAN_10.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_10_ID49           CE_CAN_10.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_10_B49            CE_CAN_10.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_10_H49            CE_CAN_10.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_10_W49            CE_CAN_10.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_10_CS50           CE_CAN_10.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_10_ID50           CE_CAN_10.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_10_B50            CE_CAN_10.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_10_H50            CE_CAN_10.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_10_W50            CE_CAN_10.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_10_CS51           CE_CAN_10.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_10_ID51           CE_CAN_10.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_10_B51            CE_CAN_10.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_10_H51            CE_CAN_10.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_10_W51            CE_CAN_10.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_10_CS52           CE_CAN_10.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_10_ID52           CE_CAN_10.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_10_B52            CE_CAN_10.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_10_H52            CE_CAN_10.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_10_W52            CE_CAN_10.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_10_CS53           CE_CAN_10.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_10_ID53           CE_CAN_10.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_10_B53            CE_CAN_10.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_10_H53            CE_CAN_10.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_10_W53            CE_CAN_10.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_10_CS54           CE_CAN_10.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_10_ID54           CE_CAN_10.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_10_B54            CE_CAN_10.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_10_H54            CE_CAN_10.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_10_W54            CE_CAN_10.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_10_CS55           CE_CAN_10.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_10_ID55           CE_CAN_10.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_10_B55            CE_CAN_10.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_10_H55            CE_CAN_10.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_10_W55            CE_CAN_10.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_10_CS56           CE_CAN_10.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_10_ID56           CE_CAN_10.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_10_B56            CE_CAN_10.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_10_H56            CE_CAN_10.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_10_W56            CE_CAN_10.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_10_CS57           CE_CAN_10.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_10_ID57           CE_CAN_10.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_10_B57            CE_CAN_10.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_10_H57            CE_CAN_10.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_10_W57            CE_CAN_10.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_10_CS58           CE_CAN_10.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_10_ID58           CE_CAN_10.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_10_B58            CE_CAN_10.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_10_H58            CE_CAN_10.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_10_W58            CE_CAN_10.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_10_CS59           CE_CAN_10.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_10_ID59           CE_CAN_10.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_10_B59            CE_CAN_10.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_10_H59            CE_CAN_10.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_10_W59            CE_CAN_10.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_10_CS60           CE_CAN_10.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_10_ID60           CE_CAN_10.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_10_B60            CE_CAN_10.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_10_H60            CE_CAN_10.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_10_W60            CE_CAN_10.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_10_CS61           CE_CAN_10.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_10_ID61           CE_CAN_10.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_10_B61            CE_CAN_10.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_10_H61            CE_CAN_10.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_10_W61            CE_CAN_10.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_10_CS62           CE_CAN_10.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_10_ID62           CE_CAN_10.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_10_B62            CE_CAN_10.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_10_H62            CE_CAN_10.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_10_W62            CE_CAN_10.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_10_CS63           CE_CAN_10.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_10_ID63           CE_CAN_10.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_10_B63            CE_CAN_10.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_10_H63            CE_CAN_10.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_10_W63            CE_CAN_10.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_10_CS64           CE_CAN_10.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_10_ID64           CE_CAN_10.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_10_B64            CE_CAN_10.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_10_H64            CE_CAN_10.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_10_W64            CE_CAN_10.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_10_CS65           CE_CAN_10.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_10_ID65           CE_CAN_10.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_10_B65            CE_CAN_10.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_10_H65            CE_CAN_10.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_10_W65            CE_CAN_10.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_10_CS66           CE_CAN_10.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_10_ID66           CE_CAN_10.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_10_B66            CE_CAN_10.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_10_H66            CE_CAN_10.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_10_W66            CE_CAN_10.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_10_CS67           CE_CAN_10.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_10_ID67           CE_CAN_10.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_10_B67            CE_CAN_10.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_10_H67            CE_CAN_10.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_10_W67            CE_CAN_10.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_10_CS68           CE_CAN_10.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_10_ID68           CE_CAN_10.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_10_B68            CE_CAN_10.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_10_H68            CE_CAN_10.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_10_W68            CE_CAN_10.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_10_CS69           CE_CAN_10.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_10_ID69           CE_CAN_10.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_10_B69            CE_CAN_10.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_10_H69            CE_CAN_10.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_10_W69            CE_CAN_10.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_10_CS70           CE_CAN_10.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_10_ID70           CE_CAN_10.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_10_B70            CE_CAN_10.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_10_H70            CE_CAN_10.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_10_W70            CE_CAN_10.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_10_CS71           CE_CAN_10.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_10_ID71           CE_CAN_10.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_10_B71            CE_CAN_10.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_10_H71            CE_CAN_10.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_10_W71            CE_CAN_10.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_10_CS72           CE_CAN_10.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_10_ID72           CE_CAN_10.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_10_B72            CE_CAN_10.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_10_H72            CE_CAN_10.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_10_W72            CE_CAN_10.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_10_CS73           CE_CAN_10.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_10_ID73           CE_CAN_10.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_10_B73            CE_CAN_10.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_10_H73            CE_CAN_10.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_10_W73            CE_CAN_10.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_10_CS74           CE_CAN_10.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_10_ID74           CE_CAN_10.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_10_B74            CE_CAN_10.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_10_H74            CE_CAN_10.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_10_W74            CE_CAN_10.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_10_CS75           CE_CAN_10.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_10_ID75           CE_CAN_10.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_10_B75            CE_CAN_10.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_10_H75            CE_CAN_10.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_10_W75            CE_CAN_10.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_10_CS76           CE_CAN_10.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_10_ID76           CE_CAN_10.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_10_B76            CE_CAN_10.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_10_H76            CE_CAN_10.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_10_W76            CE_CAN_10.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_10_CS77           CE_CAN_10.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_10_ID77           CE_CAN_10.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_10_B77            CE_CAN_10.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_10_H77            CE_CAN_10.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_10_W77            CE_CAN_10.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_10_CS78           CE_CAN_10.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_10_ID78           CE_CAN_10.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_10_B78            CE_CAN_10.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_10_H78            CE_CAN_10.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_10_W78            CE_CAN_10.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_10_CS79           CE_CAN_10.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_10_ID79           CE_CAN_10.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_10_B79            CE_CAN_10.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_10_H79            CE_CAN_10.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_10_W79            CE_CAN_10.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_10_CS80           CE_CAN_10.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_10_ID80           CE_CAN_10.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_10_B80            CE_CAN_10.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_10_H80            CE_CAN_10.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_10_W80            CE_CAN_10.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_10_CS81           CE_CAN_10.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_10_ID81           CE_CAN_10.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_10_B81            CE_CAN_10.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_10_H81            CE_CAN_10.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_10_W81            CE_CAN_10.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_10_CS82           CE_CAN_10.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_10_ID82           CE_CAN_10.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_10_B82            CE_CAN_10.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_10_H82            CE_CAN_10.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_10_W82            CE_CAN_10.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_10_CS83           CE_CAN_10.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_10_ID83           CE_CAN_10.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_10_B83            CE_CAN_10.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_10_H83            CE_CAN_10.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_10_W83            CE_CAN_10.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_10_CS84           CE_CAN_10.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_10_ID84           CE_CAN_10.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_10_B84            CE_CAN_10.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_10_H84            CE_CAN_10.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_10_W84            CE_CAN_10.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_10_CS85           CE_CAN_10.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_10_ID85           CE_CAN_10.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_10_B85            CE_CAN_10.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_10_H85            CE_CAN_10.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_10_W85            CE_CAN_10.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_10_CS86           CE_CAN_10.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_10_ID86           CE_CAN_10.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_10_B86            CE_CAN_10.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_10_H86            CE_CAN_10.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_10_W86            CE_CAN_10.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_10_CS87           CE_CAN_10.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_10_ID87           CE_CAN_10.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_10_B87            CE_CAN_10.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_10_H87            CE_CAN_10.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_10_W87            CE_CAN_10.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_10_CS88           CE_CAN_10.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_10_ID88           CE_CAN_10.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_10_B88            CE_CAN_10.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_10_H88            CE_CAN_10.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_10_W848           CE_CAN_10.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_10_CS89           CE_CAN_10.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_10_ID89           CE_CAN_10.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_10_B89            CE_CAN_10.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_10_H89            CE_CAN_10.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_10_W89            CE_CAN_10.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_10_CS90           CE_CAN_10.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_10_ID90           CE_CAN_10.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_10_B90            CE_CAN_10.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_10_H90            CE_CAN_10.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_10_W90            CE_CAN_10.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_10_CS91           CE_CAN_10.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_10_ID91           CE_CAN_10.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_10_B91            CE_CAN_10.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_10_H91            CE_CAN_10.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_10_W91            CE_CAN_10.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_10_CS92           CE_CAN_10.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_10_ID92           CE_CAN_10.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_10_B92            CE_CAN_10.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_10_H92            CE_CAN_10.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_10_W92            CE_CAN_10.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_10_CS93           CE_CAN_10.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_10_ID93           CE_CAN_10.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_10_B93            CE_CAN_10.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_10_H93            CE_CAN_10.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_10_W93            CE_CAN_10.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_10_CS94           CE_CAN_10.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_10_ID94           CE_CAN_10.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_10_B94            CE_CAN_10.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_10_H94            CE_CAN_10.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_10_W94            CE_CAN_10.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_10_CS95           CE_CAN_10.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_10_ID95           CE_CAN_10.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_10_B95            CE_CAN_10.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_10_H95            CE_CAN_10.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_10_W95            CE_CAN_10.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_10_CS96           CE_CAN_10.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_10_ID96           CE_CAN_10.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_10_B96            CE_CAN_10.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_10_H96            CE_CAN_10.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_10_W96            CE_CAN_10.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_10_CS97           CE_CAN_10.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_10_ID97           CE_CAN_10.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_10_B97            CE_CAN_10.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_10_H97            CE_CAN_10.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_10_W97            CE_CAN_10.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_10_CS98           CE_CAN_10.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_10_ID98           CE_CAN_10.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_10_B98            CE_CAN_10.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_10_H98            CE_CAN_10.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_10_W98            CE_CAN_10.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_10_CS99           CE_CAN_10.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_10_ID99           CE_CAN_10.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_10_B99            CE_CAN_10.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_10_H99            CE_CAN_10.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_10_W99            CE_CAN_10.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_10_CS100          CE_CAN_10.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_10_ID100          CE_CAN_10.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_10_B100           CE_CAN_10.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_10_H100           CE_CAN_10.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_10_W100           CE_CAN_10.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_10_CS101          CE_CAN_10.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_10_ID101          CE_CAN_10.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_10_B101           CE_CAN_10.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_10_H101           CE_CAN_10.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_10_W101           CE_CAN_10.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_10_CS102          CE_CAN_10.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_10_ID102          CE_CAN_10.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_10_B102           CE_CAN_10.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_10_H102           CE_CAN_10.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_10_W102           CE_CAN_10.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_10_CS103          CE_CAN_10.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_10_ID103          CE_CAN_10.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_10_B103           CE_CAN_10.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_10_H103           CE_CAN_10.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_10_W103           CE_CAN_10.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_10_CS104          CE_CAN_10.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_10_ID104          CE_CAN_10.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_10_B104           CE_CAN_10.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_10_H104           CE_CAN_10.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_10_W104           CE_CAN_10.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_10_CS105          CE_CAN_10.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_10_ID105          CE_CAN_10.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_10_B105           CE_CAN_10.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_10_H105           CE_CAN_10.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_10_W105           CE_CAN_10.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_10_CS106          CE_CAN_10.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_10_ID106          CE_CAN_10.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_10_B106           CE_CAN_10.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_10_H106           CE_CAN_10.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_10_W106           CE_CAN_10.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_10_CS107          CE_CAN_10.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_10_ID107          CE_CAN_10.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_10_B107           CE_CAN_10.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_10_H107           CE_CAN_10.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_10_W107           CE_CAN_10.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_10_CS108          CE_CAN_10.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_10_ID108          CE_CAN_10.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_10_B108           CE_CAN_10.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_10_H108           CE_CAN_10.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_10_W108           CE_CAN_10.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_10_CS109          CE_CAN_10.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_10_ID109          CE_CAN_10.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_10_B109           CE_CAN_10.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_10_H109           CE_CAN_10.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_10_W109           CE_CAN_10.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_10_CS110          CE_CAN_10.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_10_ID110          CE_CAN_10.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_10_B110           CE_CAN_10.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_10_H110           CE_CAN_10.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_10_W110           CE_CAN_10.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_10_CS111          CE_CAN_10.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_10_ID111          CE_CAN_10.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_10_B111           CE_CAN_10.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_10_H111           CE_CAN_10.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_10_W111           CE_CAN_10.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_10_CS112          CE_CAN_10.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_10_ID112          CE_CAN_10.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_10_B112           CE_CAN_10.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_10_H112           CE_CAN_10.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_10_W112           CE_CAN_10.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_10_CS113          CE_CAN_10.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_10_ID113          CE_CAN_10.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_10_B113           CE_CAN_10.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_10_H113           CE_CAN_10.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_10_W113           CE_CAN_10.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_10_CS114          CE_CAN_10.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_10_ID114          CE_CAN_10.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_10_B114           CE_CAN_10.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_10_H114           CE_CAN_10.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_10_W114           CE_CAN_10.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_10_CS115          CE_CAN_10.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_10_ID115          CE_CAN_10.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_10_B115           CE_CAN_10.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_10_H115           CE_CAN_10.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_10_W115           CE_CAN_10.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_10_CS116          CE_CAN_10.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_10_ID116          CE_CAN_10.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_10_B116           CE_CAN_10.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_10_H116           CE_CAN_10.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_10_W116           CE_CAN_10.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_10_CS117          CE_CAN_10.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_10_ID117          CE_CAN_10.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_10_B117           CE_CAN_10.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_10_H117           CE_CAN_10.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_10_W117           CE_CAN_10.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_10_CS118          CE_CAN_10.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_10_ID118          CE_CAN_10.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_10_B118           CE_CAN_10.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_10_H118           CE_CAN_10.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_10_W118           CE_CAN_10.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_10_CS119          CE_CAN_10.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_10_ID119          CE_CAN_10.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_10_B119           CE_CAN_10.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_10_H119           CE_CAN_10.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_10_W119           CE_CAN_10.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_10_CS120          CE_CAN_10.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_10_ID120          CE_CAN_10.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_10_B120           CE_CAN_10.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_10_H120           CE_CAN_10.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_10_W120           CE_CAN_10.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_10_CS121          CE_CAN_10.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_10_ID121          CE_CAN_10.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_10_B121           CE_CAN_10.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_10_H121           CE_CAN_10.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_10_W121           CE_CAN_10.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_10_CS122          CE_CAN_10.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_10_ID122          CE_CAN_10.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_10_B122           CE_CAN_10.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_10_H122           CE_CAN_10.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_10_W122           CE_CAN_10.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_10_CS123          CE_CAN_10.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_10_ID123          CE_CAN_10.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_10_B123           CE_CAN_10.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_10_H123           CE_CAN_10.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_10_W123           CE_CAN_10.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_10_CS124          CE_CAN_10.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_10_ID124          CE_CAN_10.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_10_B124           CE_CAN_10.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_10_H124           CE_CAN_10.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_10_W124           CE_CAN_10.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_10_CS125          CE_CAN_10.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_10_ID125          CE_CAN_10.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_10_B125           CE_CAN_10.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_10_H125           CE_CAN_10.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_10_W125           CE_CAN_10.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_10_CS126          CE_CAN_10.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_10_ID126          CE_CAN_10.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_10_B126           CE_CAN_10.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_10_H126           CE_CAN_10.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_10_W126           CE_CAN_10.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_10_CS127          CE_CAN_10.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_10_ID127          CE_CAN_10.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_10_B127           CE_CAN_10.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_10_H127           CE_CAN_10.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_10_W127           CE_CAN_10.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_10_RXIMR0     CE_CAN_10.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR1     CE_CAN_10.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR2     CE_CAN_10.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR3     CE_CAN_10.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR4     CE_CAN_10.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR5     CE_CAN_10.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR6     CE_CAN_10.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR7     CE_CAN_10.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR8     CE_CAN_10.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR9     CE_CAN_10.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_10_RXIMR10    CE_CAN_10.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR11    CE_CAN_10.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR12    CE_CAN_10.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR13    CE_CAN_10.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR14    CE_CAN_10.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR15    CE_CAN_10.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR16    CE_CAN_10.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR17    CE_CAN_10.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR18    CE_CAN_10.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR19    CE_CAN_10.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR20    CE_CAN_10.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR21    CE_CAN_10.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR22    CE_CAN_10.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR23    CE_CAN_10.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR24    CE_CAN_10.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR25    CE_CAN_10.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR26    CE_CAN_10.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR27    CE_CAN_10.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR28    CE_CAN_10.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR29    CE_CAN_10.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR30    CE_CAN_10.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR31    CE_CAN_10.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR32    CE_CAN_10.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR33    CE_CAN_10.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR34    CE_CAN_10.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR35    CE_CAN_10.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR36    CE_CAN_10.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR37    CE_CAN_10.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR38    CE_CAN_10.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR39    CE_CAN_10.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR40    CE_CAN_10.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR41    CE_CAN_10.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR42    CE_CAN_10.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR43    CE_CAN_10.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR44    CE_CAN_10.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR45    CE_CAN_10.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR46    CE_CAN_10.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR47    CE_CAN_10.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR48    CE_CAN_10.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR49    CE_CAN_10.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR50    CE_CAN_10.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR51    CE_CAN_10.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR52    CE_CAN_10.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR53    CE_CAN_10.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR54    CE_CAN_10.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR55    CE_CAN_10.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR56    CE_CAN_10.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR57    CE_CAN_10.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR58    CE_CAN_10.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR59    CE_CAN_10.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR60    CE_CAN_10.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR61    CE_CAN_10.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR62    CE_CAN_10.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR63    CE_CAN_10.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR64    CE_CAN_10.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR65    CE_CAN_10.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR66    CE_CAN_10.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR67    CE_CAN_10.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR68    CE_CAN_10.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR69    CE_CAN_10.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR70    CE_CAN_10.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR71    CE_CAN_10.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR72    CE_CAN_10.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR73    CE_CAN_10.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR74    CE_CAN_10.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR75    CE_CAN_10.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR76    CE_CAN_10.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR77    CE_CAN_10.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR78    CE_CAN_10.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR79    CE_CAN_10.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR80    CE_CAN_10.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR81    CE_CAN_10.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR82    CE_CAN_10.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR83    CE_CAN_10.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR84    CE_CAN_10.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR85    CE_CAN_10.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR86    CE_CAN_10.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR87    CE_CAN_10.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR88    CE_CAN_10.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR89    CE_CAN_10.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR90    CE_CAN_10.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR91    CE_CAN_10.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR92    CE_CAN_10.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR93    CE_CAN_10.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR94    CE_CAN_10.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR95    CE_CAN_10.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR96    CE_CAN_10.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR97    CE_CAN_10.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR98    CE_CAN_10.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR99    CE_CAN_10.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_10_RXIMR100   CE_CAN_10.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR101   CE_CAN_10.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR102   CE_CAN_10.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR103   CE_CAN_10.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR104   CE_CAN_10.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR105   CE_CAN_10.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR106   CE_CAN_10.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR107   CE_CAN_10.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR108   CE_CAN_10.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR109   CE_CAN_10.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR110   CE_CAN_10.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR111   CE_CAN_10.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR112   CE_CAN_10.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR113   CE_CAN_10.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR114   CE_CAN_10.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR115   CE_CAN_10.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR116   CE_CAN_10.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR117   CE_CAN_10.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR118   CE_CAN_10.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR119   CE_CAN_10.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR120   CE_CAN_10.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR121   CE_CAN_10.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR122   CE_CAN_10.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR123   CE_CAN_10.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR124   CE_CAN_10.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR125   CE_CAN_10.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR126   CE_CAN_10.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_10_RXIMR127   CE_CAN_10.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_10_MECR       CE_CAN_10.MECR.R              /* Memory Error Control */
#define CE_CAN_10_ERRIAR     CE_CAN_10.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_10_ERRIDPR    CE_CAN_10.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_10_ERRIPPR    CE_CAN_10.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_10_RERRAR     CE_CAN_10.RERRAR.R            /* Error Report Address */
#define CE_CAN_10_RERRDR     CE_CAN_10.RERRDR.R            /* Error Report Data */
#define CE_CAN_10_RERRSYNR   CE_CAN_10.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_10_ERRSR      CE_CAN_10.ERRSR.R             /* Error Status */
#define CE_CAN_10_EPRS       CE_CAN_10.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_10_ENCBT      CE_CAN_10.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_10_EDCBT      CE_CAN_10.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_10_ETDC       CE_CAN_10.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_10_FDCTRL     CE_CAN_10.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_10_FDCBT      CE_CAN_10.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_10_FDCRC      CE_CAN_10.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_10_ERFCR      CE_CAN_10.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_10_ERFIER     CE_CAN_10.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_10_ERFSR      CE_CAN_10.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_10_HR_TIME_STAMP0 CE_CAN_10.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP1 CE_CAN_10.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP2 CE_CAN_10.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP3 CE_CAN_10.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP4 CE_CAN_10.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP5 CE_CAN_10.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP6 CE_CAN_10.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP7 CE_CAN_10.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP8 CE_CAN_10.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP9 CE_CAN_10.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP10 CE_CAN_10.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP11 CE_CAN_10.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP12 CE_CAN_10.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP13 CE_CAN_10.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP14 CE_CAN_10.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP15 CE_CAN_10.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP16 CE_CAN_10.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP17 CE_CAN_10.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP18 CE_CAN_10.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP19 CE_CAN_10.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP20 CE_CAN_10.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP21 CE_CAN_10.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP22 CE_CAN_10.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP23 CE_CAN_10.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP24 CE_CAN_10.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP25 CE_CAN_10.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP26 CE_CAN_10.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP27 CE_CAN_10.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP28 CE_CAN_10.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP29 CE_CAN_10.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP30 CE_CAN_10.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP31 CE_CAN_10.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP32 CE_CAN_10.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP33 CE_CAN_10.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP34 CE_CAN_10.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP35 CE_CAN_10.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP36 CE_CAN_10.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP37 CE_CAN_10.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP38 CE_CAN_10.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP39 CE_CAN_10.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP40 CE_CAN_10.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP41 CE_CAN_10.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP42 CE_CAN_10.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP43 CE_CAN_10.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP44 CE_CAN_10.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP45 CE_CAN_10.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP46 CE_CAN_10.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP47 CE_CAN_10.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP48 CE_CAN_10.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP49 CE_CAN_10.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP50 CE_CAN_10.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP51 CE_CAN_10.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP52 CE_CAN_10.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP53 CE_CAN_10.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP54 CE_CAN_10.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP55 CE_CAN_10.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP56 CE_CAN_10.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP57 CE_CAN_10.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP58 CE_CAN_10.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP59 CE_CAN_10.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP60 CE_CAN_10.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP61 CE_CAN_10.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP62 CE_CAN_10.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP63 CE_CAN_10.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP64 CE_CAN_10.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP65 CE_CAN_10.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP66 CE_CAN_10.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP67 CE_CAN_10.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP68 CE_CAN_10.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP69 CE_CAN_10.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP70 CE_CAN_10.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP71 CE_CAN_10.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP72 CE_CAN_10.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP73 CE_CAN_10.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP74 CE_CAN_10.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP75 CE_CAN_10.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP76 CE_CAN_10.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP77 CE_CAN_10.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP78 CE_CAN_10.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP79 CE_CAN_10.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP80 CE_CAN_10.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP81 CE_CAN_10.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP82 CE_CAN_10.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP83 CE_CAN_10.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP84 CE_CAN_10.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP85 CE_CAN_10.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP86 CE_CAN_10.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP87 CE_CAN_10.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP88 CE_CAN_10.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP89 CE_CAN_10.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP90 CE_CAN_10.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP91 CE_CAN_10.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP92 CE_CAN_10.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP93 CE_CAN_10.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP94 CE_CAN_10.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP95 CE_CAN_10.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP96 CE_CAN_10.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP97 CE_CAN_10.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP98 CE_CAN_10.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP99 CE_CAN_10.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP100 CE_CAN_10.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP101 CE_CAN_10.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP102 CE_CAN_10.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP103 CE_CAN_10.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP104 CE_CAN_10.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP105 CE_CAN_10.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP106 CE_CAN_10.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP107 CE_CAN_10.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP108 CE_CAN_10.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP109 CE_CAN_10.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP110 CE_CAN_10.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP111 CE_CAN_10.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP112 CE_CAN_10.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP113 CE_CAN_10.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP114 CE_CAN_10.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP115 CE_CAN_10.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP116 CE_CAN_10.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP117 CE_CAN_10.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP118 CE_CAN_10.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP119 CE_CAN_10.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP120 CE_CAN_10.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP121 CE_CAN_10.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP122 CE_CAN_10.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP123 CE_CAN_10.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP124 CE_CAN_10.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP125 CE_CAN_10.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP126 CE_CAN_10.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_10_HR_TIME_STAMP127 CE_CAN_10.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_10_ERFFEL0    CE_CAN_10.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL1    CE_CAN_10.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL2    CE_CAN_10.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL3    CE_CAN_10.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL4    CE_CAN_10.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL5    CE_CAN_10.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL6    CE_CAN_10.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL7    CE_CAN_10.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL8    CE_CAN_10.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL9    CE_CAN_10.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL10   CE_CAN_10.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL11   CE_CAN_10.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL12   CE_CAN_10.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL13   CE_CAN_10.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL14   CE_CAN_10.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL15   CE_CAN_10.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL16   CE_CAN_10.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL17   CE_CAN_10.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL18   CE_CAN_10.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL19   CE_CAN_10.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL20   CE_CAN_10.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL21   CE_CAN_10.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL22   CE_CAN_10.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL23   CE_CAN_10.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL24   CE_CAN_10.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL25   CE_CAN_10.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL26   CE_CAN_10.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL27   CE_CAN_10.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL28   CE_CAN_10.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL29   CE_CAN_10.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL30   CE_CAN_10.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL31   CE_CAN_10.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL32   CE_CAN_10.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL33   CE_CAN_10.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL34   CE_CAN_10.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL35   CE_CAN_10.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL36   CE_CAN_10.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL37   CE_CAN_10.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL38   CE_CAN_10.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL39   CE_CAN_10.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL40   CE_CAN_10.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL41   CE_CAN_10.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL42   CE_CAN_10.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL43   CE_CAN_10.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL44   CE_CAN_10.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL45   CE_CAN_10.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL46   CE_CAN_10.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL47   CE_CAN_10.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL48   CE_CAN_10.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL49   CE_CAN_10.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL50   CE_CAN_10.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL51   CE_CAN_10.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL52   CE_CAN_10.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL53   CE_CAN_10.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL54   CE_CAN_10.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL55   CE_CAN_10.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL56   CE_CAN_10.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL57   CE_CAN_10.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL58   CE_CAN_10.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL59   CE_CAN_10.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL60   CE_CAN_10.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL61   CE_CAN_10.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL62   CE_CAN_10.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL63   CE_CAN_10.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL64   CE_CAN_10.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL65   CE_CAN_10.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL66   CE_CAN_10.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL67   CE_CAN_10.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL68   CE_CAN_10.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL69   CE_CAN_10.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL70   CE_CAN_10.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL71   CE_CAN_10.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL72   CE_CAN_10.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL73   CE_CAN_10.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL74   CE_CAN_10.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL75   CE_CAN_10.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL76   CE_CAN_10.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL77   CE_CAN_10.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL78   CE_CAN_10.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL79   CE_CAN_10.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL80   CE_CAN_10.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL81   CE_CAN_10.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL82   CE_CAN_10.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL83   CE_CAN_10.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL84   CE_CAN_10.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL85   CE_CAN_10.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL86   CE_CAN_10.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL87   CE_CAN_10.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL88   CE_CAN_10.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL89   CE_CAN_10.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL90   CE_CAN_10.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL91   CE_CAN_10.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL92   CE_CAN_10.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL93   CE_CAN_10.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL94   CE_CAN_10.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL95   CE_CAN_10.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL96   CE_CAN_10.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL97   CE_CAN_10.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL98   CE_CAN_10.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL99   CE_CAN_10.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL100  CE_CAN_10.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL101  CE_CAN_10.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL102  CE_CAN_10.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL103  CE_CAN_10.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL104  CE_CAN_10.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL105  CE_CAN_10.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL106  CE_CAN_10.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL107  CE_CAN_10.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL108  CE_CAN_10.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL109  CE_CAN_10.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL110  CE_CAN_10.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL111  CE_CAN_10.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL112  CE_CAN_10.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL113  CE_CAN_10.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL114  CE_CAN_10.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL115  CE_CAN_10.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL116  CE_CAN_10.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL117  CE_CAN_10.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL118  CE_CAN_10.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL119  CE_CAN_10.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL120  CE_CAN_10.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL121  CE_CAN_10.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL122  CE_CAN_10.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL123  CE_CAN_10.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL124  CE_CAN_10.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL125  CE_CAN_10.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL126  CE_CAN_10.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_10_ERFFEL127  CE_CAN_10.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_11_MCR        CE_CAN_11.MCR.R               /* Module Configuration */
#define CE_CAN_11_CTRL1      CE_CAN_11.CTRL1.R             /* Control 1 */
#define CE_CAN_11_TIMER      CE_CAN_11.TIMER.R             /* Free-Running Timer */
#define CE_CAN_11_RXMGMASK   CE_CAN_11.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_11_RX14MASK   CE_CAN_11.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_11_RX15MASK   CE_CAN_11.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_11_ECR        CE_CAN_11.ECR.R               /* Error Counter */
#define CE_CAN_11_ESR1       CE_CAN_11.ESR1.R              /* Error and Status 1 */
#define CE_CAN_11_IMASK2     CE_CAN_11.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_11_IMASK1     CE_CAN_11.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_11_IFLAG2     CE_CAN_11.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_11_IFLAG1     CE_CAN_11.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_11_CTRL2      CE_CAN_11.CTRL2.R             /* Control 2 */
#define CE_CAN_11_ESR2       CE_CAN_11.ESR2.R              /* Error and Status 2 */
#define CE_CAN_11_CRCR       CE_CAN_11.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_11_RXFGMASK   CE_CAN_11.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_11_RXFIR      CE_CAN_11.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_11_CBT        CE_CAN_11.CBT.R               /* CAN Bit Timing */
#define CE_CAN_11_IMASK4     CE_CAN_11.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_11_IMASK3     CE_CAN_11.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_11_IFLAG4     CE_CAN_11.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_11_IFLAG3     CE_CAN_11.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_11_CS0            CE_CAN_11.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_11_ID0            CE_CAN_11.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_11_B0             CE_CAN_11.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_11_H0             CE_CAN_11.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_11_W0             CE_CAN_11.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_11_CS1            CE_CAN_11.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_11_ID1            CE_CAN_11.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_11_B1             CE_CAN_11.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_11_H1             CE_CAN_11.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_11_W1             CE_CAN_11.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_11_CS2            CE_CAN_11.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_11_ID2            CE_CAN_11.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_11_B2             CE_CAN_11.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_11_H2             CE_CAN_11.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_11_W2             CE_CAN_11.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_11_CS3            CE_CAN_11.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_11_ID3            CE_CAN_11.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_11_B3             CE_CAN_11.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_11_H3             CE_CAN_11.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_11_W3             CE_CAN_11.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_11_CS4            CE_CAN_11.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_11_ID4            CE_CAN_11.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_11_B4             CE_CAN_11.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_11_H4             CE_CAN_11.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_11_W4             CE_CAN_11.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_11_CS5            CE_CAN_11.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_11_ID5            CE_CAN_11.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_11_B5             CE_CAN_11.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_11_H5             CE_CAN_11.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_11_W5             CE_CAN_11.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_11_CS6            CE_CAN_11.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_11_ID6            CE_CAN_11.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_11_B6             CE_CAN_11.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_11_H6             CE_CAN_11.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_11_W6             CE_CAN_11.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_11_CS7            CE_CAN_11.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_11_ID7            CE_CAN_11.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_11_B7             CE_CAN_11.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_11_H7             CE_CAN_11.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_11_W7             CE_CAN_11.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_11_CS8            CE_CAN_11.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_11_ID8            CE_CAN_11.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_11_B8             CE_CAN_11.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_11_H8             CE_CAN_11.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_11_W8             CE_CAN_11.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_11_CS9            CE_CAN_11.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_11_ID9            CE_CAN_11.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_11_B9             CE_CAN_11.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_11_H9             CE_CAN_11.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_11_W9             CE_CAN_11.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_11_CS10           CE_CAN_11.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_11_ID10           CE_CAN_11.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_11_B10            CE_CAN_11.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_11_H10            CE_CAN_11.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_11_W10            CE_CAN_11.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_11_CS11           CE_CAN_11.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_11_ID11           CE_CAN_11.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_11_B11            CE_CAN_11.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_11_H11            CE_CAN_11.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_11_W11            CE_CAN_11.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_11_CS12           CE_CAN_11.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_11_ID12           CE_CAN_11.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_11_B12            CE_CAN_11.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_11_H12            CE_CAN_11.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_11_W12            CE_CAN_11.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_11_CS13           CE_CAN_11.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_11_ID13           CE_CAN_11.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_11_B13            CE_CAN_11.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_11_H13            CE_CAN_11.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_11_W13            CE_CAN_11.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_11_CS14           CE_CAN_11.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_11_ID14           CE_CAN_11.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_11_B14            CE_CAN_11.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_11_H14            CE_CAN_11.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_11_W14            CE_CAN_11.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_11_CS15           CE_CAN_11.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_11_ID15           CE_CAN_11.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_11_B15            CE_CAN_11.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_11_H15            CE_CAN_11.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_11_W15            CE_CAN_11.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_11_CS16           CE_CAN_11.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_11_ID16           CE_CAN_11.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_11_B16            CE_CAN_11.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_11_H16            CE_CAN_11.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_11_W16            CE_CAN_11.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_11_CS17           CE_CAN_11.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_11_ID17           CE_CAN_11.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_11_B17            CE_CAN_11.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_11_H17            CE_CAN_11.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_11_W17            CE_CAN_11.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_11_CS18           CE_CAN_11.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_11_ID18           CE_CAN_11.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_11_B18            CE_CAN_11.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_11_H18            CE_CAN_11.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_11_W18            CE_CAN_11.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_11_CS19           CE_CAN_11.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_11_ID19           CE_CAN_11.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_11_B19            CE_CAN_11.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_11_H19            CE_CAN_11.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_11_W19            CE_CAN_11.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_11_CS20           CE_CAN_11.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_11_ID20           CE_CAN_11.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_11_B20            CE_CAN_11.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_11_H20            CE_CAN_11.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_11_W20            CE_CAN_11.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_11_CS21           CE_CAN_11.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_11_ID21           CE_CAN_11.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_11_B21            CE_CAN_11.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_11_H21            CE_CAN_11.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_11_W21            CE_CAN_11.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_11_CS22           CE_CAN_11.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_11_ID22           CE_CAN_11.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_11_B22            CE_CAN_11.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_11_H22            CE_CAN_11.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_11_W22            CE_CAN_11.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_11_CS23           CE_CAN_11.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_11_ID23           CE_CAN_11.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_11_B23            CE_CAN_11.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_11_H23            CE_CAN_11.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_11_W23            CE_CAN_11.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_11_CS24           CE_CAN_11.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_11_ID24           CE_CAN_11.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_11_B24            CE_CAN_11.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_11_H24            CE_CAN_11.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_11_W24            CE_CAN_11.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_11_CS25           CE_CAN_11.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_11_ID25           CE_CAN_11.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_11_B25            CE_CAN_11.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_11_H25            CE_CAN_11.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_11_W25            CE_CAN_11.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_11_CS26           CE_CAN_11.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_11_ID26           CE_CAN_11.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_11_B26            CE_CAN_11.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_11_H26            CE_CAN_11.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_11_W26            CE_CAN_11.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_11_CS27           CE_CAN_11.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_11_ID27           CE_CAN_11.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_11_B27            CE_CAN_11.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_11_H27            CE_CAN_11.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_11_W27            CE_CAN_11.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_11_CS28           CE_CAN_11.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_11_ID28           CE_CAN_11.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_11_B28            CE_CAN_11.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_11_H28            CE_CAN_11.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_11_W28            CE_CAN_11.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_11_CS29           CE_CAN_11.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_11_ID29           CE_CAN_11.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_11_B29            CE_CAN_11.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_11_H29            CE_CAN_11.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_11_W29            CE_CAN_11.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_11_CS30           CE_CAN_11.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_11_ID30           CE_CAN_11.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_11_B30            CE_CAN_11.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_11_H30            CE_CAN_11.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_11_W30            CE_CAN_11.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_11_CS31           CE_CAN_11.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_11_ID31           CE_CAN_11.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_11_B31            CE_CAN_11.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_11_H31            CE_CAN_11.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_11_W31            CE_CAN_11.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_11_CS32           CE_CAN_11.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_11_ID32           CE_CAN_11.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_11_B32            CE_CAN_11.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_11_H32            CE_CAN_11.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_11_W32            CE_CAN_11.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_11_CS33           CE_CAN_11.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_11_ID33           CE_CAN_11.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_11_B33            CE_CAN_11.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_11_H33            CE_CAN_11.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_11_W33            CE_CAN_11.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_11_CS34           CE_CAN_11.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_11_ID34           CE_CAN_11.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_11_B34            CE_CAN_11.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_11_H34            CE_CAN_11.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_11_W34            CE_CAN_11.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_11_CS35           CE_CAN_11.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_11_ID35           CE_CAN_11.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_11_B35            CE_CAN_11.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_11_H35            CE_CAN_11.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_11_W35            CE_CAN_11.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_11_CS36           CE_CAN_11.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_11_ID36           CE_CAN_11.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_11_B36            CE_CAN_11.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_11_H36            CE_CAN_11.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_11_W36            CE_CAN_11.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_11_CS37           CE_CAN_11.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_11_ID37           CE_CAN_11.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_11_B37            CE_CAN_11.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_11_H37            CE_CAN_11.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_11_W37            CE_CAN_11.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_11_CS38           CE_CAN_11.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_11_ID38           CE_CAN_11.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_11_B38            CE_CAN_11.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_11_H38            CE_CAN_11.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_11_W38            CE_CAN_11.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_11_CS39           CE_CAN_11.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_11_ID39           CE_CAN_11.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_11_B39            CE_CAN_11.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_11_H39            CE_CAN_11.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_11_W39            CE_CAN_11.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_11_CS40           CE_CAN_11.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_11_ID40           CE_CAN_11.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_11_B40            CE_CAN_11.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_11_H40            CE_CAN_11.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_11_W40            CE_CAN_11.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_11_CS41           CE_CAN_11.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_11_ID41           CE_CAN_11.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_11_B41            CE_CAN_11.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_11_H41            CE_CAN_11.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_11_W41            CE_CAN_11.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_11_CS42           CE_CAN_11.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_11_ID42           CE_CAN_11.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_11_B42            CE_CAN_11.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_11_H42            CE_CAN_11.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_11_W42            CE_CAN_11.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_11_CS43           CE_CAN_11.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_11_ID43           CE_CAN_11.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_11_B43            CE_CAN_11.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_11_H43            CE_CAN_11.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_11_W43            CE_CAN_11.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_11_CS44           CE_CAN_11.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_11_ID44           CE_CAN_11.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_11_B44            CE_CAN_11.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_11_H44            CE_CAN_11.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_11_W44            CE_CAN_11.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_11_CS45           CE_CAN_11.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_11_ID45           CE_CAN_11.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_11_B45            CE_CAN_11.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_11_H45            CE_CAN_11.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_11_W45            CE_CAN_11.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_11_CS46           CE_CAN_11.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_11_ID46           CE_CAN_11.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_11_B46            CE_CAN_11.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_11_H46            CE_CAN_11.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_11_W46            CE_CAN_11.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_11_CS47           CE_CAN_11.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_11_ID47           CE_CAN_11.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_11_B47            CE_CAN_11.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_11_H47            CE_CAN_11.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_11_W47            CE_CAN_11.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_11_CS48           CE_CAN_11.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_11_ID48           CE_CAN_11.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_11_B48            CE_CAN_11.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_11_H48            CE_CAN_11.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_11_W48            CE_CAN_11.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_11_CS49           CE_CAN_11.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_11_ID49           CE_CAN_11.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_11_B49            CE_CAN_11.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_11_H49            CE_CAN_11.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_11_W49            CE_CAN_11.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_11_CS50           CE_CAN_11.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_11_ID50           CE_CAN_11.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_11_B50            CE_CAN_11.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_11_H50            CE_CAN_11.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_11_W50            CE_CAN_11.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_11_CS51           CE_CAN_11.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_11_ID51           CE_CAN_11.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_11_B51            CE_CAN_11.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_11_H51            CE_CAN_11.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_11_W51            CE_CAN_11.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_11_CS52           CE_CAN_11.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_11_ID52           CE_CAN_11.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_11_B52            CE_CAN_11.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_11_H52            CE_CAN_11.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_11_W52            CE_CAN_11.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_11_CS53           CE_CAN_11.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_11_ID53           CE_CAN_11.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_11_B53            CE_CAN_11.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_11_H53            CE_CAN_11.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_11_W53            CE_CAN_11.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_11_CS54           CE_CAN_11.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_11_ID54           CE_CAN_11.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_11_B54            CE_CAN_11.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_11_H54            CE_CAN_11.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_11_W54            CE_CAN_11.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_11_CS55           CE_CAN_11.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_11_ID55           CE_CAN_11.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_11_B55            CE_CAN_11.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_11_H55            CE_CAN_11.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_11_W55            CE_CAN_11.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_11_CS56           CE_CAN_11.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_11_ID56           CE_CAN_11.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_11_B56            CE_CAN_11.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_11_H56            CE_CAN_11.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_11_W56            CE_CAN_11.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_11_CS57           CE_CAN_11.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_11_ID57           CE_CAN_11.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_11_B57            CE_CAN_11.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_11_H57            CE_CAN_11.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_11_W57            CE_CAN_11.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_11_CS58           CE_CAN_11.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_11_ID58           CE_CAN_11.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_11_B58            CE_CAN_11.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_11_H58            CE_CAN_11.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_11_W58            CE_CAN_11.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_11_CS59           CE_CAN_11.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_11_ID59           CE_CAN_11.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_11_B59            CE_CAN_11.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_11_H59            CE_CAN_11.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_11_W59            CE_CAN_11.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_11_CS60           CE_CAN_11.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_11_ID60           CE_CAN_11.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_11_B60            CE_CAN_11.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_11_H60            CE_CAN_11.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_11_W60            CE_CAN_11.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_11_CS61           CE_CAN_11.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_11_ID61           CE_CAN_11.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_11_B61            CE_CAN_11.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_11_H61            CE_CAN_11.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_11_W61            CE_CAN_11.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_11_CS62           CE_CAN_11.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_11_ID62           CE_CAN_11.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_11_B62            CE_CAN_11.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_11_H62            CE_CAN_11.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_11_W62            CE_CAN_11.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_11_CS63           CE_CAN_11.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_11_ID63           CE_CAN_11.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_11_B63            CE_CAN_11.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_11_H63            CE_CAN_11.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_11_W63            CE_CAN_11.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_11_CS64           CE_CAN_11.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_11_ID64           CE_CAN_11.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_11_B64            CE_CAN_11.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_11_H64            CE_CAN_11.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_11_W64            CE_CAN_11.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_11_CS65           CE_CAN_11.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_11_ID65           CE_CAN_11.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_11_B65            CE_CAN_11.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_11_H65            CE_CAN_11.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_11_W65            CE_CAN_11.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_11_CS66           CE_CAN_11.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_11_ID66           CE_CAN_11.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_11_B66            CE_CAN_11.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_11_H66            CE_CAN_11.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_11_W66            CE_CAN_11.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_11_CS67           CE_CAN_11.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_11_ID67           CE_CAN_11.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_11_B67            CE_CAN_11.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_11_H67            CE_CAN_11.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_11_W67            CE_CAN_11.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_11_CS68           CE_CAN_11.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_11_ID68           CE_CAN_11.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_11_B68            CE_CAN_11.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_11_H68            CE_CAN_11.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_11_W68            CE_CAN_11.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_11_CS69           CE_CAN_11.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_11_ID69           CE_CAN_11.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_11_B69            CE_CAN_11.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_11_H69            CE_CAN_11.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_11_W69            CE_CAN_11.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_11_CS70           CE_CAN_11.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_11_ID70           CE_CAN_11.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_11_B70            CE_CAN_11.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_11_H70            CE_CAN_11.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_11_W70            CE_CAN_11.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_11_CS71           CE_CAN_11.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_11_ID71           CE_CAN_11.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_11_B71            CE_CAN_11.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_11_H71            CE_CAN_11.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_11_W71            CE_CAN_11.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_11_CS72           CE_CAN_11.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_11_ID72           CE_CAN_11.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_11_B72            CE_CAN_11.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_11_H72            CE_CAN_11.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_11_W72            CE_CAN_11.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_11_CS73           CE_CAN_11.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_11_ID73           CE_CAN_11.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_11_B73            CE_CAN_11.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_11_H73            CE_CAN_11.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_11_W73            CE_CAN_11.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_11_CS74           CE_CAN_11.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_11_ID74           CE_CAN_11.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_11_B74            CE_CAN_11.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_11_H74            CE_CAN_11.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_11_W74            CE_CAN_11.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_11_CS75           CE_CAN_11.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_11_ID75           CE_CAN_11.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_11_B75            CE_CAN_11.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_11_H75            CE_CAN_11.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_11_W75            CE_CAN_11.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_11_CS76           CE_CAN_11.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_11_ID76           CE_CAN_11.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_11_B76            CE_CAN_11.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_11_H76            CE_CAN_11.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_11_W76            CE_CAN_11.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_11_CS77           CE_CAN_11.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_11_ID77           CE_CAN_11.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_11_B77            CE_CAN_11.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_11_H77            CE_CAN_11.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_11_W77            CE_CAN_11.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_11_CS78           CE_CAN_11.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_11_ID78           CE_CAN_11.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_11_B78            CE_CAN_11.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_11_H78            CE_CAN_11.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_11_W78            CE_CAN_11.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_11_CS79           CE_CAN_11.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_11_ID79           CE_CAN_11.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_11_B79            CE_CAN_11.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_11_H79            CE_CAN_11.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_11_W79            CE_CAN_11.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_11_CS80           CE_CAN_11.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_11_ID80           CE_CAN_11.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_11_B80            CE_CAN_11.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_11_H80            CE_CAN_11.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_11_W80            CE_CAN_11.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_11_CS81           CE_CAN_11.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_11_ID81           CE_CAN_11.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_11_B81            CE_CAN_11.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_11_H81            CE_CAN_11.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_11_W81            CE_CAN_11.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_11_CS82           CE_CAN_11.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_11_ID82           CE_CAN_11.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_11_B82            CE_CAN_11.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_11_H82            CE_CAN_11.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_11_W82            CE_CAN_11.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_11_CS83           CE_CAN_11.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_11_ID83           CE_CAN_11.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_11_B83            CE_CAN_11.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_11_H83            CE_CAN_11.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_11_W83            CE_CAN_11.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_11_CS84           CE_CAN_11.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_11_ID84           CE_CAN_11.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_11_B84            CE_CAN_11.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_11_H84            CE_CAN_11.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_11_W84            CE_CAN_11.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_11_CS85           CE_CAN_11.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_11_ID85           CE_CAN_11.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_11_B85            CE_CAN_11.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_11_H85            CE_CAN_11.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_11_W85            CE_CAN_11.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_11_CS86           CE_CAN_11.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_11_ID86           CE_CAN_11.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_11_B86            CE_CAN_11.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_11_H86            CE_CAN_11.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_11_W86            CE_CAN_11.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_11_CS87           CE_CAN_11.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_11_ID87           CE_CAN_11.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_11_B87            CE_CAN_11.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_11_H87            CE_CAN_11.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_11_W87            CE_CAN_11.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_11_CS88           CE_CAN_11.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_11_ID88           CE_CAN_11.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_11_B88            CE_CAN_11.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_11_H88            CE_CAN_11.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_11_W848           CE_CAN_11.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_11_CS89           CE_CAN_11.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_11_ID89           CE_CAN_11.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_11_B89            CE_CAN_11.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_11_H89            CE_CAN_11.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_11_W89            CE_CAN_11.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_11_CS90           CE_CAN_11.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_11_ID90           CE_CAN_11.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_11_B90            CE_CAN_11.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_11_H90            CE_CAN_11.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_11_W90            CE_CAN_11.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_11_CS91           CE_CAN_11.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_11_ID91           CE_CAN_11.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_11_B91            CE_CAN_11.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_11_H91            CE_CAN_11.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_11_W91            CE_CAN_11.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_11_CS92           CE_CAN_11.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_11_ID92           CE_CAN_11.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_11_B92            CE_CAN_11.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_11_H92            CE_CAN_11.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_11_W92            CE_CAN_11.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_11_CS93           CE_CAN_11.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_11_ID93           CE_CAN_11.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_11_B93            CE_CAN_11.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_11_H93            CE_CAN_11.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_11_W93            CE_CAN_11.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_11_CS94           CE_CAN_11.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_11_ID94           CE_CAN_11.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_11_B94            CE_CAN_11.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_11_H94            CE_CAN_11.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_11_W94            CE_CAN_11.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_11_CS95           CE_CAN_11.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_11_ID95           CE_CAN_11.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_11_B95            CE_CAN_11.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_11_H95            CE_CAN_11.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_11_W95            CE_CAN_11.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_11_CS96           CE_CAN_11.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_11_ID96           CE_CAN_11.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_11_B96            CE_CAN_11.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_11_H96            CE_CAN_11.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_11_W96            CE_CAN_11.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_11_CS97           CE_CAN_11.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_11_ID97           CE_CAN_11.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_11_B97            CE_CAN_11.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_11_H97            CE_CAN_11.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_11_W97            CE_CAN_11.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_11_CS98           CE_CAN_11.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_11_ID98           CE_CAN_11.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_11_B98            CE_CAN_11.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_11_H98            CE_CAN_11.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_11_W98            CE_CAN_11.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_11_CS99           CE_CAN_11.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_11_ID99           CE_CAN_11.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_11_B99            CE_CAN_11.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_11_H99            CE_CAN_11.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_11_W99            CE_CAN_11.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_11_CS100          CE_CAN_11.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_11_ID100          CE_CAN_11.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_11_B100           CE_CAN_11.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_11_H100           CE_CAN_11.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_11_W100           CE_CAN_11.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_11_CS101          CE_CAN_11.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_11_ID101          CE_CAN_11.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_11_B101           CE_CAN_11.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_11_H101           CE_CAN_11.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_11_W101           CE_CAN_11.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_11_CS102          CE_CAN_11.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_11_ID102          CE_CAN_11.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_11_B102           CE_CAN_11.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_11_H102           CE_CAN_11.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_11_W102           CE_CAN_11.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_11_CS103          CE_CAN_11.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_11_ID103          CE_CAN_11.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_11_B103           CE_CAN_11.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_11_H103           CE_CAN_11.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_11_W103           CE_CAN_11.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_11_CS104          CE_CAN_11.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_11_ID104          CE_CAN_11.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_11_B104           CE_CAN_11.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_11_H104           CE_CAN_11.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_11_W104           CE_CAN_11.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_11_CS105          CE_CAN_11.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_11_ID105          CE_CAN_11.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_11_B105           CE_CAN_11.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_11_H105           CE_CAN_11.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_11_W105           CE_CAN_11.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_11_CS106          CE_CAN_11.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_11_ID106          CE_CAN_11.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_11_B106           CE_CAN_11.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_11_H106           CE_CAN_11.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_11_W106           CE_CAN_11.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_11_CS107          CE_CAN_11.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_11_ID107          CE_CAN_11.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_11_B107           CE_CAN_11.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_11_H107           CE_CAN_11.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_11_W107           CE_CAN_11.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_11_CS108          CE_CAN_11.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_11_ID108          CE_CAN_11.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_11_B108           CE_CAN_11.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_11_H108           CE_CAN_11.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_11_W108           CE_CAN_11.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_11_CS109          CE_CAN_11.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_11_ID109          CE_CAN_11.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_11_B109           CE_CAN_11.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_11_H109           CE_CAN_11.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_11_W109           CE_CAN_11.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_11_CS110          CE_CAN_11.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_11_ID110          CE_CAN_11.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_11_B110           CE_CAN_11.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_11_H110           CE_CAN_11.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_11_W110           CE_CAN_11.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_11_CS111          CE_CAN_11.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_11_ID111          CE_CAN_11.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_11_B111           CE_CAN_11.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_11_H111           CE_CAN_11.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_11_W111           CE_CAN_11.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_11_CS112          CE_CAN_11.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_11_ID112          CE_CAN_11.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_11_B112           CE_CAN_11.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_11_H112           CE_CAN_11.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_11_W112           CE_CAN_11.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_11_CS113          CE_CAN_11.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_11_ID113          CE_CAN_11.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_11_B113           CE_CAN_11.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_11_H113           CE_CAN_11.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_11_W113           CE_CAN_11.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_11_CS114          CE_CAN_11.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_11_ID114          CE_CAN_11.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_11_B114           CE_CAN_11.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_11_H114           CE_CAN_11.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_11_W114           CE_CAN_11.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_11_CS115          CE_CAN_11.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_11_ID115          CE_CAN_11.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_11_B115           CE_CAN_11.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_11_H115           CE_CAN_11.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_11_W115           CE_CAN_11.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_11_CS116          CE_CAN_11.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_11_ID116          CE_CAN_11.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_11_B116           CE_CAN_11.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_11_H116           CE_CAN_11.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_11_W116           CE_CAN_11.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_11_CS117          CE_CAN_11.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_11_ID117          CE_CAN_11.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_11_B117           CE_CAN_11.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_11_H117           CE_CAN_11.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_11_W117           CE_CAN_11.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_11_CS118          CE_CAN_11.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_11_ID118          CE_CAN_11.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_11_B118           CE_CAN_11.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_11_H118           CE_CAN_11.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_11_W118           CE_CAN_11.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_11_CS119          CE_CAN_11.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_11_ID119          CE_CAN_11.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_11_B119           CE_CAN_11.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_11_H119           CE_CAN_11.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_11_W119           CE_CAN_11.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_11_CS120          CE_CAN_11.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_11_ID120          CE_CAN_11.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_11_B120           CE_CAN_11.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_11_H120           CE_CAN_11.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_11_W120           CE_CAN_11.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_11_CS121          CE_CAN_11.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_11_ID121          CE_CAN_11.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_11_B121           CE_CAN_11.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_11_H121           CE_CAN_11.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_11_W121           CE_CAN_11.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_11_CS122          CE_CAN_11.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_11_ID122          CE_CAN_11.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_11_B122           CE_CAN_11.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_11_H122           CE_CAN_11.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_11_W122           CE_CAN_11.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_11_CS123          CE_CAN_11.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_11_ID123          CE_CAN_11.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_11_B123           CE_CAN_11.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_11_H123           CE_CAN_11.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_11_W123           CE_CAN_11.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_11_CS124          CE_CAN_11.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_11_ID124          CE_CAN_11.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_11_B124           CE_CAN_11.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_11_H124           CE_CAN_11.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_11_W124           CE_CAN_11.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_11_CS125          CE_CAN_11.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_11_ID125          CE_CAN_11.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_11_B125           CE_CAN_11.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_11_H125           CE_CAN_11.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_11_W125           CE_CAN_11.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_11_CS126          CE_CAN_11.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_11_ID126          CE_CAN_11.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_11_B126           CE_CAN_11.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_11_H126           CE_CAN_11.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_11_W126           CE_CAN_11.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_11_CS127          CE_CAN_11.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_11_ID127          CE_CAN_11.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_11_B127           CE_CAN_11.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_11_H127           CE_CAN_11.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_11_W127           CE_CAN_11.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_11_RXIMR0     CE_CAN_11.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR1     CE_CAN_11.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR2     CE_CAN_11.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR3     CE_CAN_11.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR4     CE_CAN_11.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR5     CE_CAN_11.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR6     CE_CAN_11.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR7     CE_CAN_11.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR8     CE_CAN_11.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR9     CE_CAN_11.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_11_RXIMR10    CE_CAN_11.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR11    CE_CAN_11.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR12    CE_CAN_11.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR13    CE_CAN_11.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR14    CE_CAN_11.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR15    CE_CAN_11.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR16    CE_CAN_11.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR17    CE_CAN_11.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR18    CE_CAN_11.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR19    CE_CAN_11.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR20    CE_CAN_11.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR21    CE_CAN_11.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR22    CE_CAN_11.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR23    CE_CAN_11.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR24    CE_CAN_11.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR25    CE_CAN_11.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR26    CE_CAN_11.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR27    CE_CAN_11.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR28    CE_CAN_11.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR29    CE_CAN_11.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR30    CE_CAN_11.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR31    CE_CAN_11.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR32    CE_CAN_11.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR33    CE_CAN_11.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR34    CE_CAN_11.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR35    CE_CAN_11.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR36    CE_CAN_11.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR37    CE_CAN_11.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR38    CE_CAN_11.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR39    CE_CAN_11.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR40    CE_CAN_11.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR41    CE_CAN_11.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR42    CE_CAN_11.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR43    CE_CAN_11.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR44    CE_CAN_11.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR45    CE_CAN_11.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR46    CE_CAN_11.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR47    CE_CAN_11.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR48    CE_CAN_11.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR49    CE_CAN_11.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR50    CE_CAN_11.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR51    CE_CAN_11.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR52    CE_CAN_11.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR53    CE_CAN_11.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR54    CE_CAN_11.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR55    CE_CAN_11.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR56    CE_CAN_11.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR57    CE_CAN_11.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR58    CE_CAN_11.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR59    CE_CAN_11.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR60    CE_CAN_11.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR61    CE_CAN_11.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR62    CE_CAN_11.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR63    CE_CAN_11.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR64    CE_CAN_11.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR65    CE_CAN_11.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR66    CE_CAN_11.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR67    CE_CAN_11.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR68    CE_CAN_11.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR69    CE_CAN_11.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR70    CE_CAN_11.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR71    CE_CAN_11.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR72    CE_CAN_11.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR73    CE_CAN_11.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR74    CE_CAN_11.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR75    CE_CAN_11.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR76    CE_CAN_11.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR77    CE_CAN_11.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR78    CE_CAN_11.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR79    CE_CAN_11.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR80    CE_CAN_11.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR81    CE_CAN_11.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR82    CE_CAN_11.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR83    CE_CAN_11.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR84    CE_CAN_11.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR85    CE_CAN_11.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR86    CE_CAN_11.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR87    CE_CAN_11.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR88    CE_CAN_11.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR89    CE_CAN_11.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR90    CE_CAN_11.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR91    CE_CAN_11.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR92    CE_CAN_11.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR93    CE_CAN_11.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR94    CE_CAN_11.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR95    CE_CAN_11.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR96    CE_CAN_11.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR97    CE_CAN_11.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR98    CE_CAN_11.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR99    CE_CAN_11.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_11_RXIMR100   CE_CAN_11.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR101   CE_CAN_11.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR102   CE_CAN_11.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR103   CE_CAN_11.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR104   CE_CAN_11.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR105   CE_CAN_11.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR106   CE_CAN_11.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR107   CE_CAN_11.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR108   CE_CAN_11.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR109   CE_CAN_11.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR110   CE_CAN_11.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR111   CE_CAN_11.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR112   CE_CAN_11.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR113   CE_CAN_11.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR114   CE_CAN_11.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR115   CE_CAN_11.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR116   CE_CAN_11.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR117   CE_CAN_11.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR118   CE_CAN_11.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR119   CE_CAN_11.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR120   CE_CAN_11.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR121   CE_CAN_11.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR122   CE_CAN_11.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR123   CE_CAN_11.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR124   CE_CAN_11.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR125   CE_CAN_11.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR126   CE_CAN_11.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_11_RXIMR127   CE_CAN_11.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_11_MECR       CE_CAN_11.MECR.R              /* Memory Error Control */
#define CE_CAN_11_ERRIAR     CE_CAN_11.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_11_ERRIDPR    CE_CAN_11.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_11_ERRIPPR    CE_CAN_11.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_11_RERRAR     CE_CAN_11.RERRAR.R            /* Error Report Address */
#define CE_CAN_11_RERRDR     CE_CAN_11.RERRDR.R            /* Error Report Data */
#define CE_CAN_11_RERRSYNR   CE_CAN_11.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_11_ERRSR      CE_CAN_11.ERRSR.R             /* Error Status */
#define CE_CAN_11_EPRS       CE_CAN_11.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_11_ENCBT      CE_CAN_11.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_11_EDCBT      CE_CAN_11.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_11_ETDC       CE_CAN_11.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_11_FDCTRL     CE_CAN_11.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_11_FDCBT      CE_CAN_11.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_11_FDCRC      CE_CAN_11.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_11_ERFCR      CE_CAN_11.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_11_ERFIER     CE_CAN_11.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_11_ERFSR      CE_CAN_11.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_11_HR_TIME_STAMP0 CE_CAN_11.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP1 CE_CAN_11.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP2 CE_CAN_11.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP3 CE_CAN_11.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP4 CE_CAN_11.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP5 CE_CAN_11.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP6 CE_CAN_11.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP7 CE_CAN_11.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP8 CE_CAN_11.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP9 CE_CAN_11.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP10 CE_CAN_11.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP11 CE_CAN_11.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP12 CE_CAN_11.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP13 CE_CAN_11.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP14 CE_CAN_11.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP15 CE_CAN_11.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP16 CE_CAN_11.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP17 CE_CAN_11.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP18 CE_CAN_11.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP19 CE_CAN_11.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP20 CE_CAN_11.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP21 CE_CAN_11.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP22 CE_CAN_11.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP23 CE_CAN_11.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP24 CE_CAN_11.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP25 CE_CAN_11.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP26 CE_CAN_11.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP27 CE_CAN_11.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP28 CE_CAN_11.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP29 CE_CAN_11.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP30 CE_CAN_11.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP31 CE_CAN_11.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP32 CE_CAN_11.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP33 CE_CAN_11.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP34 CE_CAN_11.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP35 CE_CAN_11.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP36 CE_CAN_11.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP37 CE_CAN_11.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP38 CE_CAN_11.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP39 CE_CAN_11.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP40 CE_CAN_11.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP41 CE_CAN_11.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP42 CE_CAN_11.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP43 CE_CAN_11.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP44 CE_CAN_11.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP45 CE_CAN_11.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP46 CE_CAN_11.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP47 CE_CAN_11.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP48 CE_CAN_11.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP49 CE_CAN_11.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP50 CE_CAN_11.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP51 CE_CAN_11.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP52 CE_CAN_11.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP53 CE_CAN_11.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP54 CE_CAN_11.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP55 CE_CAN_11.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP56 CE_CAN_11.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP57 CE_CAN_11.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP58 CE_CAN_11.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP59 CE_CAN_11.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP60 CE_CAN_11.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP61 CE_CAN_11.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP62 CE_CAN_11.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP63 CE_CAN_11.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP64 CE_CAN_11.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP65 CE_CAN_11.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP66 CE_CAN_11.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP67 CE_CAN_11.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP68 CE_CAN_11.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP69 CE_CAN_11.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP70 CE_CAN_11.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP71 CE_CAN_11.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP72 CE_CAN_11.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP73 CE_CAN_11.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP74 CE_CAN_11.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP75 CE_CAN_11.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP76 CE_CAN_11.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP77 CE_CAN_11.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP78 CE_CAN_11.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP79 CE_CAN_11.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP80 CE_CAN_11.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP81 CE_CAN_11.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP82 CE_CAN_11.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP83 CE_CAN_11.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP84 CE_CAN_11.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP85 CE_CAN_11.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP86 CE_CAN_11.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP87 CE_CAN_11.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP88 CE_CAN_11.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP89 CE_CAN_11.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP90 CE_CAN_11.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP91 CE_CAN_11.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP92 CE_CAN_11.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP93 CE_CAN_11.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP94 CE_CAN_11.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP95 CE_CAN_11.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP96 CE_CAN_11.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP97 CE_CAN_11.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP98 CE_CAN_11.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP99 CE_CAN_11.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP100 CE_CAN_11.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP101 CE_CAN_11.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP102 CE_CAN_11.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP103 CE_CAN_11.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP104 CE_CAN_11.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP105 CE_CAN_11.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP106 CE_CAN_11.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP107 CE_CAN_11.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP108 CE_CAN_11.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP109 CE_CAN_11.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP110 CE_CAN_11.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP111 CE_CAN_11.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP112 CE_CAN_11.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP113 CE_CAN_11.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP114 CE_CAN_11.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP115 CE_CAN_11.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP116 CE_CAN_11.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP117 CE_CAN_11.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP118 CE_CAN_11.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP119 CE_CAN_11.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP120 CE_CAN_11.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP121 CE_CAN_11.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP122 CE_CAN_11.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP123 CE_CAN_11.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP124 CE_CAN_11.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP125 CE_CAN_11.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP126 CE_CAN_11.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_11_HR_TIME_STAMP127 CE_CAN_11.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_11_ERFFEL0    CE_CAN_11.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL1    CE_CAN_11.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL2    CE_CAN_11.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL3    CE_CAN_11.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL4    CE_CAN_11.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL5    CE_CAN_11.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL6    CE_CAN_11.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL7    CE_CAN_11.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL8    CE_CAN_11.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL9    CE_CAN_11.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL10   CE_CAN_11.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL11   CE_CAN_11.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL12   CE_CAN_11.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL13   CE_CAN_11.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL14   CE_CAN_11.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL15   CE_CAN_11.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL16   CE_CAN_11.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL17   CE_CAN_11.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL18   CE_CAN_11.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL19   CE_CAN_11.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL20   CE_CAN_11.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL21   CE_CAN_11.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL22   CE_CAN_11.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL23   CE_CAN_11.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL24   CE_CAN_11.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL25   CE_CAN_11.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL26   CE_CAN_11.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL27   CE_CAN_11.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL28   CE_CAN_11.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL29   CE_CAN_11.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL30   CE_CAN_11.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL31   CE_CAN_11.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL32   CE_CAN_11.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL33   CE_CAN_11.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL34   CE_CAN_11.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL35   CE_CAN_11.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL36   CE_CAN_11.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL37   CE_CAN_11.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL38   CE_CAN_11.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL39   CE_CAN_11.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL40   CE_CAN_11.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL41   CE_CAN_11.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL42   CE_CAN_11.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL43   CE_CAN_11.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL44   CE_CAN_11.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL45   CE_CAN_11.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL46   CE_CAN_11.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL47   CE_CAN_11.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL48   CE_CAN_11.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL49   CE_CAN_11.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL50   CE_CAN_11.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL51   CE_CAN_11.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL52   CE_CAN_11.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL53   CE_CAN_11.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL54   CE_CAN_11.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL55   CE_CAN_11.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL56   CE_CAN_11.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL57   CE_CAN_11.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL58   CE_CAN_11.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL59   CE_CAN_11.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL60   CE_CAN_11.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL61   CE_CAN_11.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL62   CE_CAN_11.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL63   CE_CAN_11.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL64   CE_CAN_11.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL65   CE_CAN_11.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL66   CE_CAN_11.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL67   CE_CAN_11.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL68   CE_CAN_11.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL69   CE_CAN_11.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL70   CE_CAN_11.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL71   CE_CAN_11.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL72   CE_CAN_11.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL73   CE_CAN_11.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL74   CE_CAN_11.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL75   CE_CAN_11.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL76   CE_CAN_11.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL77   CE_CAN_11.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL78   CE_CAN_11.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL79   CE_CAN_11.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL80   CE_CAN_11.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL81   CE_CAN_11.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL82   CE_CAN_11.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL83   CE_CAN_11.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL84   CE_CAN_11.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL85   CE_CAN_11.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL86   CE_CAN_11.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL87   CE_CAN_11.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL88   CE_CAN_11.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL89   CE_CAN_11.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL90   CE_CAN_11.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL91   CE_CAN_11.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL92   CE_CAN_11.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL93   CE_CAN_11.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL94   CE_CAN_11.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL95   CE_CAN_11.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL96   CE_CAN_11.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL97   CE_CAN_11.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL98   CE_CAN_11.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL99   CE_CAN_11.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL100  CE_CAN_11.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL101  CE_CAN_11.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL102  CE_CAN_11.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL103  CE_CAN_11.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL104  CE_CAN_11.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL105  CE_CAN_11.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL106  CE_CAN_11.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL107  CE_CAN_11.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL108  CE_CAN_11.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL109  CE_CAN_11.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL110  CE_CAN_11.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL111  CE_CAN_11.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL112  CE_CAN_11.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL113  CE_CAN_11.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL114  CE_CAN_11.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL115  CE_CAN_11.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL116  CE_CAN_11.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL117  CE_CAN_11.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL118  CE_CAN_11.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL119  CE_CAN_11.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL120  CE_CAN_11.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL121  CE_CAN_11.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL122  CE_CAN_11.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL123  CE_CAN_11.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL124  CE_CAN_11.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL125  CE_CAN_11.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL126  CE_CAN_11.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_11_ERFFEL127  CE_CAN_11.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_12_MCR        CE_CAN_12.MCR.R               /* Module Configuration */
#define CE_CAN_12_CTRL1      CE_CAN_12.CTRL1.R             /* Control 1 */
#define CE_CAN_12_TIMER      CE_CAN_12.TIMER.R             /* Free-Running Timer */
#define CE_CAN_12_RXMGMASK   CE_CAN_12.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_12_RX14MASK   CE_CAN_12.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_12_RX15MASK   CE_CAN_12.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_12_ECR        CE_CAN_12.ECR.R               /* Error Counter */
#define CE_CAN_12_ESR1       CE_CAN_12.ESR1.R              /* Error and Status 1 */
#define CE_CAN_12_IMASK2     CE_CAN_12.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_12_IMASK1     CE_CAN_12.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_12_IFLAG2     CE_CAN_12.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_12_IFLAG1     CE_CAN_12.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_12_CTRL2      CE_CAN_12.CTRL2.R             /* Control 2 */
#define CE_CAN_12_ESR2       CE_CAN_12.ESR2.R              /* Error and Status 2 */
#define CE_CAN_12_CRCR       CE_CAN_12.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_12_RXFGMASK   CE_CAN_12.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_12_RXFIR      CE_CAN_12.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_12_CBT        CE_CAN_12.CBT.R               /* CAN Bit Timing */
#define CE_CAN_12_IMASK4     CE_CAN_12.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_12_IMASK3     CE_CAN_12.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_12_IFLAG4     CE_CAN_12.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_12_IFLAG3     CE_CAN_12.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_12_CS0            CE_CAN_12.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_12_ID0            CE_CAN_12.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_12_B0             CE_CAN_12.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_12_H0             CE_CAN_12.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_12_W0             CE_CAN_12.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_12_CS1            CE_CAN_12.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_12_ID1            CE_CAN_12.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_12_B1             CE_CAN_12.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_12_H1             CE_CAN_12.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_12_W1             CE_CAN_12.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_12_CS2            CE_CAN_12.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_12_ID2            CE_CAN_12.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_12_B2             CE_CAN_12.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_12_H2             CE_CAN_12.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_12_W2             CE_CAN_12.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_12_CS3            CE_CAN_12.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_12_ID3            CE_CAN_12.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_12_B3             CE_CAN_12.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_12_H3             CE_CAN_12.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_12_W3             CE_CAN_12.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_12_CS4            CE_CAN_12.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_12_ID4            CE_CAN_12.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_12_B4             CE_CAN_12.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_12_H4             CE_CAN_12.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_12_W4             CE_CAN_12.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_12_CS5            CE_CAN_12.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_12_ID5            CE_CAN_12.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_12_B5             CE_CAN_12.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_12_H5             CE_CAN_12.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_12_W5             CE_CAN_12.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_12_CS6            CE_CAN_12.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_12_ID6            CE_CAN_12.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_12_B6             CE_CAN_12.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_12_H6             CE_CAN_12.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_12_W6             CE_CAN_12.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_12_CS7            CE_CAN_12.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_12_ID7            CE_CAN_12.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_12_B7             CE_CAN_12.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_12_H7             CE_CAN_12.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_12_W7             CE_CAN_12.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_12_CS8            CE_CAN_12.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_12_ID8            CE_CAN_12.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_12_B8             CE_CAN_12.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_12_H8             CE_CAN_12.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_12_W8             CE_CAN_12.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_12_CS9            CE_CAN_12.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_12_ID9            CE_CAN_12.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_12_B9             CE_CAN_12.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_12_H9             CE_CAN_12.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_12_W9             CE_CAN_12.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_12_CS10           CE_CAN_12.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_12_ID10           CE_CAN_12.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_12_B10            CE_CAN_12.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_12_H10            CE_CAN_12.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_12_W10            CE_CAN_12.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_12_CS11           CE_CAN_12.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_12_ID11           CE_CAN_12.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_12_B11            CE_CAN_12.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_12_H11            CE_CAN_12.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_12_W11            CE_CAN_12.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_12_CS12           CE_CAN_12.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_12_ID12           CE_CAN_12.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_12_B12            CE_CAN_12.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_12_H12            CE_CAN_12.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_12_W12            CE_CAN_12.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_12_CS13           CE_CAN_12.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_12_ID13           CE_CAN_12.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_12_B13            CE_CAN_12.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_12_H13            CE_CAN_12.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_12_W13            CE_CAN_12.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_12_CS14           CE_CAN_12.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_12_ID14           CE_CAN_12.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_12_B14            CE_CAN_12.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_12_H14            CE_CAN_12.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_12_W14            CE_CAN_12.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_12_CS15           CE_CAN_12.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_12_ID15           CE_CAN_12.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_12_B15            CE_CAN_12.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_12_H15            CE_CAN_12.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_12_W15            CE_CAN_12.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_12_CS16           CE_CAN_12.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_12_ID16           CE_CAN_12.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_12_B16            CE_CAN_12.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_12_H16            CE_CAN_12.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_12_W16            CE_CAN_12.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_12_CS17           CE_CAN_12.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_12_ID17           CE_CAN_12.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_12_B17            CE_CAN_12.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_12_H17            CE_CAN_12.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_12_W17            CE_CAN_12.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_12_CS18           CE_CAN_12.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_12_ID18           CE_CAN_12.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_12_B18            CE_CAN_12.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_12_H18            CE_CAN_12.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_12_W18            CE_CAN_12.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_12_CS19           CE_CAN_12.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_12_ID19           CE_CAN_12.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_12_B19            CE_CAN_12.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_12_H19            CE_CAN_12.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_12_W19            CE_CAN_12.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_12_CS20           CE_CAN_12.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_12_ID20           CE_CAN_12.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_12_B20            CE_CAN_12.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_12_H20            CE_CAN_12.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_12_W20            CE_CAN_12.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_12_CS21           CE_CAN_12.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_12_ID21           CE_CAN_12.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_12_B21            CE_CAN_12.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_12_H21            CE_CAN_12.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_12_W21            CE_CAN_12.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_12_CS22           CE_CAN_12.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_12_ID22           CE_CAN_12.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_12_B22            CE_CAN_12.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_12_H22            CE_CAN_12.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_12_W22            CE_CAN_12.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_12_CS23           CE_CAN_12.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_12_ID23           CE_CAN_12.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_12_B23            CE_CAN_12.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_12_H23            CE_CAN_12.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_12_W23            CE_CAN_12.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_12_CS24           CE_CAN_12.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_12_ID24           CE_CAN_12.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_12_B24            CE_CAN_12.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_12_H24            CE_CAN_12.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_12_W24            CE_CAN_12.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_12_CS25           CE_CAN_12.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_12_ID25           CE_CAN_12.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_12_B25            CE_CAN_12.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_12_H25            CE_CAN_12.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_12_W25            CE_CAN_12.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_12_CS26           CE_CAN_12.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_12_ID26           CE_CAN_12.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_12_B26            CE_CAN_12.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_12_H26            CE_CAN_12.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_12_W26            CE_CAN_12.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_12_CS27           CE_CAN_12.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_12_ID27           CE_CAN_12.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_12_B27            CE_CAN_12.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_12_H27            CE_CAN_12.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_12_W27            CE_CAN_12.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_12_CS28           CE_CAN_12.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_12_ID28           CE_CAN_12.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_12_B28            CE_CAN_12.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_12_H28            CE_CAN_12.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_12_W28            CE_CAN_12.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_12_CS29           CE_CAN_12.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_12_ID29           CE_CAN_12.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_12_B29            CE_CAN_12.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_12_H29            CE_CAN_12.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_12_W29            CE_CAN_12.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_12_CS30           CE_CAN_12.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_12_ID30           CE_CAN_12.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_12_B30            CE_CAN_12.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_12_H30            CE_CAN_12.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_12_W30            CE_CAN_12.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_12_CS31           CE_CAN_12.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_12_ID31           CE_CAN_12.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_12_B31            CE_CAN_12.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_12_H31            CE_CAN_12.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_12_W31            CE_CAN_12.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_12_CS32           CE_CAN_12.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_12_ID32           CE_CAN_12.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_12_B32            CE_CAN_12.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_12_H32            CE_CAN_12.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_12_W32            CE_CAN_12.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_12_CS33           CE_CAN_12.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_12_ID33           CE_CAN_12.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_12_B33            CE_CAN_12.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_12_H33            CE_CAN_12.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_12_W33            CE_CAN_12.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_12_CS34           CE_CAN_12.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_12_ID34           CE_CAN_12.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_12_B34            CE_CAN_12.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_12_H34            CE_CAN_12.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_12_W34            CE_CAN_12.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_12_CS35           CE_CAN_12.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_12_ID35           CE_CAN_12.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_12_B35            CE_CAN_12.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_12_H35            CE_CAN_12.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_12_W35            CE_CAN_12.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_12_CS36           CE_CAN_12.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_12_ID36           CE_CAN_12.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_12_B36            CE_CAN_12.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_12_H36            CE_CAN_12.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_12_W36            CE_CAN_12.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_12_CS37           CE_CAN_12.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_12_ID37           CE_CAN_12.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_12_B37            CE_CAN_12.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_12_H37            CE_CAN_12.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_12_W37            CE_CAN_12.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_12_CS38           CE_CAN_12.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_12_ID38           CE_CAN_12.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_12_B38            CE_CAN_12.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_12_H38            CE_CAN_12.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_12_W38            CE_CAN_12.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_12_CS39           CE_CAN_12.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_12_ID39           CE_CAN_12.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_12_B39            CE_CAN_12.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_12_H39            CE_CAN_12.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_12_W39            CE_CAN_12.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_12_CS40           CE_CAN_12.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_12_ID40           CE_CAN_12.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_12_B40            CE_CAN_12.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_12_H40            CE_CAN_12.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_12_W40            CE_CAN_12.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_12_CS41           CE_CAN_12.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_12_ID41           CE_CAN_12.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_12_B41            CE_CAN_12.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_12_H41            CE_CAN_12.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_12_W41            CE_CAN_12.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_12_CS42           CE_CAN_12.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_12_ID42           CE_CAN_12.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_12_B42            CE_CAN_12.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_12_H42            CE_CAN_12.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_12_W42            CE_CAN_12.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_12_CS43           CE_CAN_12.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_12_ID43           CE_CAN_12.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_12_B43            CE_CAN_12.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_12_H43            CE_CAN_12.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_12_W43            CE_CAN_12.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_12_CS44           CE_CAN_12.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_12_ID44           CE_CAN_12.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_12_B44            CE_CAN_12.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_12_H44            CE_CAN_12.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_12_W44            CE_CAN_12.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_12_CS45           CE_CAN_12.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_12_ID45           CE_CAN_12.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_12_B45            CE_CAN_12.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_12_H45            CE_CAN_12.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_12_W45            CE_CAN_12.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_12_CS46           CE_CAN_12.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_12_ID46           CE_CAN_12.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_12_B46            CE_CAN_12.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_12_H46            CE_CAN_12.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_12_W46            CE_CAN_12.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_12_CS47           CE_CAN_12.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_12_ID47           CE_CAN_12.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_12_B47            CE_CAN_12.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_12_H47            CE_CAN_12.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_12_W47            CE_CAN_12.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_12_CS48           CE_CAN_12.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_12_ID48           CE_CAN_12.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_12_B48            CE_CAN_12.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_12_H48            CE_CAN_12.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_12_W48            CE_CAN_12.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_12_CS49           CE_CAN_12.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_12_ID49           CE_CAN_12.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_12_B49            CE_CAN_12.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_12_H49            CE_CAN_12.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_12_W49            CE_CAN_12.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_12_CS50           CE_CAN_12.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_12_ID50           CE_CAN_12.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_12_B50            CE_CAN_12.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_12_H50            CE_CAN_12.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_12_W50            CE_CAN_12.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_12_CS51           CE_CAN_12.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_12_ID51           CE_CAN_12.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_12_B51            CE_CAN_12.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_12_H51            CE_CAN_12.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_12_W51            CE_CAN_12.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_12_CS52           CE_CAN_12.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_12_ID52           CE_CAN_12.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_12_B52            CE_CAN_12.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_12_H52            CE_CAN_12.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_12_W52            CE_CAN_12.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_12_CS53           CE_CAN_12.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_12_ID53           CE_CAN_12.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_12_B53            CE_CAN_12.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_12_H53            CE_CAN_12.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_12_W53            CE_CAN_12.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_12_CS54           CE_CAN_12.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_12_ID54           CE_CAN_12.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_12_B54            CE_CAN_12.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_12_H54            CE_CAN_12.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_12_W54            CE_CAN_12.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_12_CS55           CE_CAN_12.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_12_ID55           CE_CAN_12.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_12_B55            CE_CAN_12.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_12_H55            CE_CAN_12.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_12_W55            CE_CAN_12.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_12_CS56           CE_CAN_12.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_12_ID56           CE_CAN_12.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_12_B56            CE_CAN_12.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_12_H56            CE_CAN_12.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_12_W56            CE_CAN_12.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_12_CS57           CE_CAN_12.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_12_ID57           CE_CAN_12.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_12_B57            CE_CAN_12.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_12_H57            CE_CAN_12.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_12_W57            CE_CAN_12.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_12_CS58           CE_CAN_12.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_12_ID58           CE_CAN_12.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_12_B58            CE_CAN_12.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_12_H58            CE_CAN_12.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_12_W58            CE_CAN_12.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_12_CS59           CE_CAN_12.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_12_ID59           CE_CAN_12.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_12_B59            CE_CAN_12.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_12_H59            CE_CAN_12.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_12_W59            CE_CAN_12.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_12_CS60           CE_CAN_12.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_12_ID60           CE_CAN_12.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_12_B60            CE_CAN_12.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_12_H60            CE_CAN_12.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_12_W60            CE_CAN_12.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_12_CS61           CE_CAN_12.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_12_ID61           CE_CAN_12.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_12_B61            CE_CAN_12.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_12_H61            CE_CAN_12.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_12_W61            CE_CAN_12.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_12_CS62           CE_CAN_12.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_12_ID62           CE_CAN_12.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_12_B62            CE_CAN_12.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_12_H62            CE_CAN_12.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_12_W62            CE_CAN_12.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_12_CS63           CE_CAN_12.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_12_ID63           CE_CAN_12.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_12_B63            CE_CAN_12.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_12_H63            CE_CAN_12.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_12_W63            CE_CAN_12.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_12_CS64           CE_CAN_12.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_12_ID64           CE_CAN_12.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_12_B64            CE_CAN_12.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_12_H64            CE_CAN_12.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_12_W64            CE_CAN_12.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_12_CS65           CE_CAN_12.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_12_ID65           CE_CAN_12.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_12_B65            CE_CAN_12.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_12_H65            CE_CAN_12.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_12_W65            CE_CAN_12.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_12_CS66           CE_CAN_12.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_12_ID66           CE_CAN_12.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_12_B66            CE_CAN_12.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_12_H66            CE_CAN_12.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_12_W66            CE_CAN_12.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_12_CS67           CE_CAN_12.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_12_ID67           CE_CAN_12.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_12_B67            CE_CAN_12.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_12_H67            CE_CAN_12.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_12_W67            CE_CAN_12.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_12_CS68           CE_CAN_12.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_12_ID68           CE_CAN_12.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_12_B68            CE_CAN_12.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_12_H68            CE_CAN_12.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_12_W68            CE_CAN_12.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_12_CS69           CE_CAN_12.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_12_ID69           CE_CAN_12.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_12_B69            CE_CAN_12.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_12_H69            CE_CAN_12.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_12_W69            CE_CAN_12.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_12_CS70           CE_CAN_12.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_12_ID70           CE_CAN_12.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_12_B70            CE_CAN_12.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_12_H70            CE_CAN_12.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_12_W70            CE_CAN_12.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_12_CS71           CE_CAN_12.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_12_ID71           CE_CAN_12.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_12_B71            CE_CAN_12.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_12_H71            CE_CAN_12.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_12_W71            CE_CAN_12.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_12_CS72           CE_CAN_12.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_12_ID72           CE_CAN_12.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_12_B72            CE_CAN_12.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_12_H72            CE_CAN_12.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_12_W72            CE_CAN_12.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_12_CS73           CE_CAN_12.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_12_ID73           CE_CAN_12.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_12_B73            CE_CAN_12.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_12_H73            CE_CAN_12.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_12_W73            CE_CAN_12.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_12_CS74           CE_CAN_12.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_12_ID74           CE_CAN_12.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_12_B74            CE_CAN_12.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_12_H74            CE_CAN_12.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_12_W74            CE_CAN_12.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_12_CS75           CE_CAN_12.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_12_ID75           CE_CAN_12.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_12_B75            CE_CAN_12.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_12_H75            CE_CAN_12.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_12_W75            CE_CAN_12.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_12_CS76           CE_CAN_12.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_12_ID76           CE_CAN_12.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_12_B76            CE_CAN_12.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_12_H76            CE_CAN_12.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_12_W76            CE_CAN_12.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_12_CS77           CE_CAN_12.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_12_ID77           CE_CAN_12.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_12_B77            CE_CAN_12.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_12_H77            CE_CAN_12.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_12_W77            CE_CAN_12.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_12_CS78           CE_CAN_12.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_12_ID78           CE_CAN_12.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_12_B78            CE_CAN_12.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_12_H78            CE_CAN_12.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_12_W78            CE_CAN_12.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_12_CS79           CE_CAN_12.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_12_ID79           CE_CAN_12.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_12_B79            CE_CAN_12.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_12_H79            CE_CAN_12.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_12_W79            CE_CAN_12.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_12_CS80           CE_CAN_12.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_12_ID80           CE_CAN_12.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_12_B80            CE_CAN_12.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_12_H80            CE_CAN_12.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_12_W80            CE_CAN_12.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_12_CS81           CE_CAN_12.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_12_ID81           CE_CAN_12.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_12_B81            CE_CAN_12.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_12_H81            CE_CAN_12.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_12_W81            CE_CAN_12.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_12_CS82           CE_CAN_12.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_12_ID82           CE_CAN_12.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_12_B82            CE_CAN_12.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_12_H82            CE_CAN_12.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_12_W82            CE_CAN_12.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_12_CS83           CE_CAN_12.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_12_ID83           CE_CAN_12.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_12_B83            CE_CAN_12.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_12_H83            CE_CAN_12.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_12_W83            CE_CAN_12.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_12_CS84           CE_CAN_12.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_12_ID84           CE_CAN_12.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_12_B84            CE_CAN_12.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_12_H84            CE_CAN_12.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_12_W84            CE_CAN_12.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_12_CS85           CE_CAN_12.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_12_ID85           CE_CAN_12.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_12_B85            CE_CAN_12.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_12_H85            CE_CAN_12.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_12_W85            CE_CAN_12.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_12_CS86           CE_CAN_12.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_12_ID86           CE_CAN_12.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_12_B86            CE_CAN_12.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_12_H86            CE_CAN_12.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_12_W86            CE_CAN_12.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_12_CS87           CE_CAN_12.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_12_ID87           CE_CAN_12.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_12_B87            CE_CAN_12.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_12_H87            CE_CAN_12.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_12_W87            CE_CAN_12.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_12_CS88           CE_CAN_12.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_12_ID88           CE_CAN_12.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_12_B88            CE_CAN_12.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_12_H88            CE_CAN_12.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_12_W848           CE_CAN_12.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_12_CS89           CE_CAN_12.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_12_ID89           CE_CAN_12.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_12_B89            CE_CAN_12.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_12_H89            CE_CAN_12.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_12_W89            CE_CAN_12.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_12_CS90           CE_CAN_12.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_12_ID90           CE_CAN_12.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_12_B90            CE_CAN_12.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_12_H90            CE_CAN_12.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_12_W90            CE_CAN_12.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_12_CS91           CE_CAN_12.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_12_ID91           CE_CAN_12.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_12_B91            CE_CAN_12.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_12_H91            CE_CAN_12.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_12_W91            CE_CAN_12.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_12_CS92           CE_CAN_12.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_12_ID92           CE_CAN_12.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_12_B92            CE_CAN_12.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_12_H92            CE_CAN_12.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_12_W92            CE_CAN_12.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_12_CS93           CE_CAN_12.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_12_ID93           CE_CAN_12.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_12_B93            CE_CAN_12.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_12_H93            CE_CAN_12.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_12_W93            CE_CAN_12.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_12_CS94           CE_CAN_12.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_12_ID94           CE_CAN_12.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_12_B94            CE_CAN_12.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_12_H94            CE_CAN_12.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_12_W94            CE_CAN_12.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_12_CS95           CE_CAN_12.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_12_ID95           CE_CAN_12.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_12_B95            CE_CAN_12.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_12_H95            CE_CAN_12.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_12_W95            CE_CAN_12.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_12_CS96           CE_CAN_12.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_12_ID96           CE_CAN_12.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_12_B96            CE_CAN_12.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_12_H96            CE_CAN_12.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_12_W96            CE_CAN_12.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_12_CS97           CE_CAN_12.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_12_ID97           CE_CAN_12.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_12_B97            CE_CAN_12.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_12_H97            CE_CAN_12.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_12_W97            CE_CAN_12.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_12_CS98           CE_CAN_12.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_12_ID98           CE_CAN_12.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_12_B98            CE_CAN_12.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_12_H98            CE_CAN_12.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_12_W98            CE_CAN_12.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_12_CS99           CE_CAN_12.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_12_ID99           CE_CAN_12.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_12_B99            CE_CAN_12.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_12_H99            CE_CAN_12.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_12_W99            CE_CAN_12.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_12_CS100          CE_CAN_12.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_12_ID100          CE_CAN_12.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_12_B100           CE_CAN_12.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_12_H100           CE_CAN_12.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_12_W100           CE_CAN_12.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_12_CS101          CE_CAN_12.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_12_ID101          CE_CAN_12.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_12_B101           CE_CAN_12.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_12_H101           CE_CAN_12.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_12_W101           CE_CAN_12.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_12_CS102          CE_CAN_12.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_12_ID102          CE_CAN_12.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_12_B102           CE_CAN_12.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_12_H102           CE_CAN_12.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_12_W102           CE_CAN_12.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_12_CS103          CE_CAN_12.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_12_ID103          CE_CAN_12.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_12_B103           CE_CAN_12.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_12_H103           CE_CAN_12.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_12_W103           CE_CAN_12.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_12_CS104          CE_CAN_12.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_12_ID104          CE_CAN_12.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_12_B104           CE_CAN_12.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_12_H104           CE_CAN_12.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_12_W104           CE_CAN_12.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_12_CS105          CE_CAN_12.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_12_ID105          CE_CAN_12.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_12_B105           CE_CAN_12.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_12_H105           CE_CAN_12.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_12_W105           CE_CAN_12.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_12_CS106          CE_CAN_12.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_12_ID106          CE_CAN_12.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_12_B106           CE_CAN_12.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_12_H106           CE_CAN_12.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_12_W106           CE_CAN_12.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_12_CS107          CE_CAN_12.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_12_ID107          CE_CAN_12.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_12_B107           CE_CAN_12.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_12_H107           CE_CAN_12.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_12_W107           CE_CAN_12.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_12_CS108          CE_CAN_12.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_12_ID108          CE_CAN_12.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_12_B108           CE_CAN_12.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_12_H108           CE_CAN_12.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_12_W108           CE_CAN_12.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_12_CS109          CE_CAN_12.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_12_ID109          CE_CAN_12.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_12_B109           CE_CAN_12.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_12_H109           CE_CAN_12.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_12_W109           CE_CAN_12.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_12_CS110          CE_CAN_12.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_12_ID110          CE_CAN_12.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_12_B110           CE_CAN_12.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_12_H110           CE_CAN_12.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_12_W110           CE_CAN_12.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_12_CS111          CE_CAN_12.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_12_ID111          CE_CAN_12.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_12_B111           CE_CAN_12.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_12_H111           CE_CAN_12.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_12_W111           CE_CAN_12.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_12_CS112          CE_CAN_12.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_12_ID112          CE_CAN_12.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_12_B112           CE_CAN_12.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_12_H112           CE_CAN_12.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_12_W112           CE_CAN_12.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_12_CS113          CE_CAN_12.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_12_ID113          CE_CAN_12.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_12_B113           CE_CAN_12.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_12_H113           CE_CAN_12.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_12_W113           CE_CAN_12.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_12_CS114          CE_CAN_12.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_12_ID114          CE_CAN_12.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_12_B114           CE_CAN_12.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_12_H114           CE_CAN_12.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_12_W114           CE_CAN_12.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_12_CS115          CE_CAN_12.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_12_ID115          CE_CAN_12.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_12_B115           CE_CAN_12.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_12_H115           CE_CAN_12.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_12_W115           CE_CAN_12.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_12_CS116          CE_CAN_12.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_12_ID116          CE_CAN_12.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_12_B116           CE_CAN_12.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_12_H116           CE_CAN_12.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_12_W116           CE_CAN_12.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_12_CS117          CE_CAN_12.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_12_ID117          CE_CAN_12.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_12_B117           CE_CAN_12.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_12_H117           CE_CAN_12.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_12_W117           CE_CAN_12.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_12_CS118          CE_CAN_12.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_12_ID118          CE_CAN_12.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_12_B118           CE_CAN_12.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_12_H118           CE_CAN_12.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_12_W118           CE_CAN_12.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_12_CS119          CE_CAN_12.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_12_ID119          CE_CAN_12.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_12_B119           CE_CAN_12.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_12_H119           CE_CAN_12.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_12_W119           CE_CAN_12.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_12_CS120          CE_CAN_12.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_12_ID120          CE_CAN_12.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_12_B120           CE_CAN_12.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_12_H120           CE_CAN_12.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_12_W120           CE_CAN_12.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_12_CS121          CE_CAN_12.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_12_ID121          CE_CAN_12.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_12_B121           CE_CAN_12.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_12_H121           CE_CAN_12.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_12_W121           CE_CAN_12.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_12_CS122          CE_CAN_12.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_12_ID122          CE_CAN_12.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_12_B122           CE_CAN_12.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_12_H122           CE_CAN_12.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_12_W122           CE_CAN_12.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_12_CS123          CE_CAN_12.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_12_ID123          CE_CAN_12.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_12_B123           CE_CAN_12.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_12_H123           CE_CAN_12.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_12_W123           CE_CAN_12.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_12_CS124          CE_CAN_12.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_12_ID124          CE_CAN_12.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_12_B124           CE_CAN_12.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_12_H124           CE_CAN_12.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_12_W124           CE_CAN_12.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_12_CS125          CE_CAN_12.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_12_ID125          CE_CAN_12.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_12_B125           CE_CAN_12.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_12_H125           CE_CAN_12.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_12_W125           CE_CAN_12.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_12_CS126          CE_CAN_12.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_12_ID126          CE_CAN_12.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_12_B126           CE_CAN_12.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_12_H126           CE_CAN_12.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_12_W126           CE_CAN_12.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_12_CS127          CE_CAN_12.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_12_ID127          CE_CAN_12.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_12_B127           CE_CAN_12.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_12_H127           CE_CAN_12.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_12_W127           CE_CAN_12.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_12_RXIMR0     CE_CAN_12.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR1     CE_CAN_12.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR2     CE_CAN_12.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR3     CE_CAN_12.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR4     CE_CAN_12.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR5     CE_CAN_12.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR6     CE_CAN_12.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR7     CE_CAN_12.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR8     CE_CAN_12.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR9     CE_CAN_12.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_12_RXIMR10    CE_CAN_12.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR11    CE_CAN_12.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR12    CE_CAN_12.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR13    CE_CAN_12.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR14    CE_CAN_12.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR15    CE_CAN_12.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR16    CE_CAN_12.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR17    CE_CAN_12.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR18    CE_CAN_12.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR19    CE_CAN_12.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR20    CE_CAN_12.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR21    CE_CAN_12.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR22    CE_CAN_12.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR23    CE_CAN_12.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR24    CE_CAN_12.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR25    CE_CAN_12.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR26    CE_CAN_12.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR27    CE_CAN_12.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR28    CE_CAN_12.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR29    CE_CAN_12.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR30    CE_CAN_12.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR31    CE_CAN_12.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR32    CE_CAN_12.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR33    CE_CAN_12.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR34    CE_CAN_12.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR35    CE_CAN_12.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR36    CE_CAN_12.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR37    CE_CAN_12.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR38    CE_CAN_12.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR39    CE_CAN_12.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR40    CE_CAN_12.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR41    CE_CAN_12.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR42    CE_CAN_12.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR43    CE_CAN_12.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR44    CE_CAN_12.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR45    CE_CAN_12.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR46    CE_CAN_12.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR47    CE_CAN_12.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR48    CE_CAN_12.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR49    CE_CAN_12.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR50    CE_CAN_12.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR51    CE_CAN_12.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR52    CE_CAN_12.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR53    CE_CAN_12.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR54    CE_CAN_12.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR55    CE_CAN_12.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR56    CE_CAN_12.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR57    CE_CAN_12.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR58    CE_CAN_12.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR59    CE_CAN_12.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR60    CE_CAN_12.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR61    CE_CAN_12.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR62    CE_CAN_12.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR63    CE_CAN_12.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR64    CE_CAN_12.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR65    CE_CAN_12.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR66    CE_CAN_12.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR67    CE_CAN_12.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR68    CE_CAN_12.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR69    CE_CAN_12.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR70    CE_CAN_12.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR71    CE_CAN_12.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR72    CE_CAN_12.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR73    CE_CAN_12.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR74    CE_CAN_12.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR75    CE_CAN_12.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR76    CE_CAN_12.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR77    CE_CAN_12.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR78    CE_CAN_12.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR79    CE_CAN_12.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR80    CE_CAN_12.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR81    CE_CAN_12.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR82    CE_CAN_12.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR83    CE_CAN_12.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR84    CE_CAN_12.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR85    CE_CAN_12.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR86    CE_CAN_12.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR87    CE_CAN_12.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR88    CE_CAN_12.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR89    CE_CAN_12.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR90    CE_CAN_12.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR91    CE_CAN_12.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR92    CE_CAN_12.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR93    CE_CAN_12.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR94    CE_CAN_12.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR95    CE_CAN_12.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR96    CE_CAN_12.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR97    CE_CAN_12.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR98    CE_CAN_12.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR99    CE_CAN_12.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_12_RXIMR100   CE_CAN_12.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR101   CE_CAN_12.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR102   CE_CAN_12.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR103   CE_CAN_12.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR104   CE_CAN_12.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR105   CE_CAN_12.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR106   CE_CAN_12.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR107   CE_CAN_12.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR108   CE_CAN_12.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR109   CE_CAN_12.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR110   CE_CAN_12.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR111   CE_CAN_12.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR112   CE_CAN_12.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR113   CE_CAN_12.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR114   CE_CAN_12.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR115   CE_CAN_12.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR116   CE_CAN_12.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR117   CE_CAN_12.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR118   CE_CAN_12.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR119   CE_CAN_12.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR120   CE_CAN_12.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR121   CE_CAN_12.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR122   CE_CAN_12.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR123   CE_CAN_12.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR124   CE_CAN_12.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR125   CE_CAN_12.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR126   CE_CAN_12.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_12_RXIMR127   CE_CAN_12.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_12_MECR       CE_CAN_12.MECR.R              /* Memory Error Control */
#define CE_CAN_12_ERRIAR     CE_CAN_12.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_12_ERRIDPR    CE_CAN_12.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_12_ERRIPPR    CE_CAN_12.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_12_RERRAR     CE_CAN_12.RERRAR.R            /* Error Report Address */
#define CE_CAN_12_RERRDR     CE_CAN_12.RERRDR.R            /* Error Report Data */
#define CE_CAN_12_RERRSYNR   CE_CAN_12.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_12_ERRSR      CE_CAN_12.ERRSR.R             /* Error Status */
#define CE_CAN_12_EPRS       CE_CAN_12.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_12_ENCBT      CE_CAN_12.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_12_EDCBT      CE_CAN_12.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_12_ETDC       CE_CAN_12.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_12_FDCTRL     CE_CAN_12.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_12_FDCBT      CE_CAN_12.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_12_FDCRC      CE_CAN_12.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_12_ERFCR      CE_CAN_12.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_12_ERFIER     CE_CAN_12.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_12_ERFSR      CE_CAN_12.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_12_HR_TIME_STAMP0 CE_CAN_12.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP1 CE_CAN_12.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP2 CE_CAN_12.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP3 CE_CAN_12.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP4 CE_CAN_12.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP5 CE_CAN_12.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP6 CE_CAN_12.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP7 CE_CAN_12.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP8 CE_CAN_12.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP9 CE_CAN_12.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP10 CE_CAN_12.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP11 CE_CAN_12.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP12 CE_CAN_12.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP13 CE_CAN_12.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP14 CE_CAN_12.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP15 CE_CAN_12.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP16 CE_CAN_12.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP17 CE_CAN_12.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP18 CE_CAN_12.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP19 CE_CAN_12.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP20 CE_CAN_12.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP21 CE_CAN_12.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP22 CE_CAN_12.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP23 CE_CAN_12.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP24 CE_CAN_12.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP25 CE_CAN_12.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP26 CE_CAN_12.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP27 CE_CAN_12.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP28 CE_CAN_12.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP29 CE_CAN_12.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP30 CE_CAN_12.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP31 CE_CAN_12.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP32 CE_CAN_12.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP33 CE_CAN_12.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP34 CE_CAN_12.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP35 CE_CAN_12.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP36 CE_CAN_12.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP37 CE_CAN_12.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP38 CE_CAN_12.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP39 CE_CAN_12.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP40 CE_CAN_12.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP41 CE_CAN_12.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP42 CE_CAN_12.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP43 CE_CAN_12.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP44 CE_CAN_12.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP45 CE_CAN_12.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP46 CE_CAN_12.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP47 CE_CAN_12.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP48 CE_CAN_12.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP49 CE_CAN_12.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP50 CE_CAN_12.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP51 CE_CAN_12.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP52 CE_CAN_12.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP53 CE_CAN_12.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP54 CE_CAN_12.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP55 CE_CAN_12.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP56 CE_CAN_12.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP57 CE_CAN_12.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP58 CE_CAN_12.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP59 CE_CAN_12.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP60 CE_CAN_12.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP61 CE_CAN_12.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP62 CE_CAN_12.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP63 CE_CAN_12.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP64 CE_CAN_12.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP65 CE_CAN_12.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP66 CE_CAN_12.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP67 CE_CAN_12.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP68 CE_CAN_12.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP69 CE_CAN_12.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP70 CE_CAN_12.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP71 CE_CAN_12.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP72 CE_CAN_12.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP73 CE_CAN_12.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP74 CE_CAN_12.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP75 CE_CAN_12.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP76 CE_CAN_12.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP77 CE_CAN_12.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP78 CE_CAN_12.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP79 CE_CAN_12.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP80 CE_CAN_12.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP81 CE_CAN_12.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP82 CE_CAN_12.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP83 CE_CAN_12.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP84 CE_CAN_12.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP85 CE_CAN_12.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP86 CE_CAN_12.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP87 CE_CAN_12.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP88 CE_CAN_12.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP89 CE_CAN_12.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP90 CE_CAN_12.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP91 CE_CAN_12.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP92 CE_CAN_12.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP93 CE_CAN_12.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP94 CE_CAN_12.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP95 CE_CAN_12.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP96 CE_CAN_12.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP97 CE_CAN_12.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP98 CE_CAN_12.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP99 CE_CAN_12.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP100 CE_CAN_12.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP101 CE_CAN_12.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP102 CE_CAN_12.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP103 CE_CAN_12.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP104 CE_CAN_12.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP105 CE_CAN_12.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP106 CE_CAN_12.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP107 CE_CAN_12.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP108 CE_CAN_12.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP109 CE_CAN_12.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP110 CE_CAN_12.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP111 CE_CAN_12.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP112 CE_CAN_12.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP113 CE_CAN_12.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP114 CE_CAN_12.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP115 CE_CAN_12.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP116 CE_CAN_12.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP117 CE_CAN_12.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP118 CE_CAN_12.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP119 CE_CAN_12.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP120 CE_CAN_12.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP121 CE_CAN_12.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP122 CE_CAN_12.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP123 CE_CAN_12.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP124 CE_CAN_12.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP125 CE_CAN_12.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP126 CE_CAN_12.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_12_HR_TIME_STAMP127 CE_CAN_12.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_12_ERFFEL0    CE_CAN_12.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL1    CE_CAN_12.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL2    CE_CAN_12.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL3    CE_CAN_12.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL4    CE_CAN_12.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL5    CE_CAN_12.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL6    CE_CAN_12.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL7    CE_CAN_12.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL8    CE_CAN_12.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL9    CE_CAN_12.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL10   CE_CAN_12.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL11   CE_CAN_12.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL12   CE_CAN_12.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL13   CE_CAN_12.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL14   CE_CAN_12.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL15   CE_CAN_12.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL16   CE_CAN_12.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL17   CE_CAN_12.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL18   CE_CAN_12.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL19   CE_CAN_12.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL20   CE_CAN_12.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL21   CE_CAN_12.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL22   CE_CAN_12.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL23   CE_CAN_12.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL24   CE_CAN_12.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL25   CE_CAN_12.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL26   CE_CAN_12.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL27   CE_CAN_12.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL28   CE_CAN_12.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL29   CE_CAN_12.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL30   CE_CAN_12.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL31   CE_CAN_12.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL32   CE_CAN_12.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL33   CE_CAN_12.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL34   CE_CAN_12.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL35   CE_CAN_12.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL36   CE_CAN_12.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL37   CE_CAN_12.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL38   CE_CAN_12.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL39   CE_CAN_12.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL40   CE_CAN_12.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL41   CE_CAN_12.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL42   CE_CAN_12.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL43   CE_CAN_12.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL44   CE_CAN_12.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL45   CE_CAN_12.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL46   CE_CAN_12.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL47   CE_CAN_12.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL48   CE_CAN_12.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL49   CE_CAN_12.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL50   CE_CAN_12.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL51   CE_CAN_12.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL52   CE_CAN_12.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL53   CE_CAN_12.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL54   CE_CAN_12.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL55   CE_CAN_12.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL56   CE_CAN_12.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL57   CE_CAN_12.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL58   CE_CAN_12.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL59   CE_CAN_12.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL60   CE_CAN_12.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL61   CE_CAN_12.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL62   CE_CAN_12.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL63   CE_CAN_12.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL64   CE_CAN_12.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL65   CE_CAN_12.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL66   CE_CAN_12.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL67   CE_CAN_12.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL68   CE_CAN_12.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL69   CE_CAN_12.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL70   CE_CAN_12.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL71   CE_CAN_12.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL72   CE_CAN_12.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL73   CE_CAN_12.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL74   CE_CAN_12.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL75   CE_CAN_12.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL76   CE_CAN_12.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL77   CE_CAN_12.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL78   CE_CAN_12.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL79   CE_CAN_12.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL80   CE_CAN_12.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL81   CE_CAN_12.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL82   CE_CAN_12.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL83   CE_CAN_12.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL84   CE_CAN_12.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL85   CE_CAN_12.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL86   CE_CAN_12.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL87   CE_CAN_12.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL88   CE_CAN_12.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL89   CE_CAN_12.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL90   CE_CAN_12.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL91   CE_CAN_12.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL92   CE_CAN_12.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL93   CE_CAN_12.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL94   CE_CAN_12.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL95   CE_CAN_12.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL96   CE_CAN_12.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL97   CE_CAN_12.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL98   CE_CAN_12.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL99   CE_CAN_12.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL100  CE_CAN_12.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL101  CE_CAN_12.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL102  CE_CAN_12.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL103  CE_CAN_12.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL104  CE_CAN_12.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL105  CE_CAN_12.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL106  CE_CAN_12.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL107  CE_CAN_12.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL108  CE_CAN_12.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL109  CE_CAN_12.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL110  CE_CAN_12.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL111  CE_CAN_12.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL112  CE_CAN_12.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL113  CE_CAN_12.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL114  CE_CAN_12.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL115  CE_CAN_12.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL116  CE_CAN_12.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL117  CE_CAN_12.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL118  CE_CAN_12.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL119  CE_CAN_12.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL120  CE_CAN_12.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL121  CE_CAN_12.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL122  CE_CAN_12.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL123  CE_CAN_12.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL124  CE_CAN_12.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL125  CE_CAN_12.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL126  CE_CAN_12.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_12_ERFFEL127  CE_CAN_12.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_13_MCR        CE_CAN_13.MCR.R               /* Module Configuration */
#define CE_CAN_13_CTRL1      CE_CAN_13.CTRL1.R             /* Control 1 */
#define CE_CAN_13_TIMER      CE_CAN_13.TIMER.R             /* Free-Running Timer */
#define CE_CAN_13_RXMGMASK   CE_CAN_13.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_13_RX14MASK   CE_CAN_13.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_13_RX15MASK   CE_CAN_13.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_13_ECR        CE_CAN_13.ECR.R               /* Error Counter */
#define CE_CAN_13_ESR1       CE_CAN_13.ESR1.R              /* Error and Status 1 */
#define CE_CAN_13_IMASK2     CE_CAN_13.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_13_IMASK1     CE_CAN_13.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_13_IFLAG2     CE_CAN_13.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_13_IFLAG1     CE_CAN_13.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_13_CTRL2      CE_CAN_13.CTRL2.R             /* Control 2 */
#define CE_CAN_13_ESR2       CE_CAN_13.ESR2.R              /* Error and Status 2 */
#define CE_CAN_13_CRCR       CE_CAN_13.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_13_RXFGMASK   CE_CAN_13.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_13_RXFIR      CE_CAN_13.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_13_CBT        CE_CAN_13.CBT.R               /* CAN Bit Timing */
#define CE_CAN_13_IMASK4     CE_CAN_13.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_13_IMASK3     CE_CAN_13.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_13_IFLAG4     CE_CAN_13.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_13_IFLAG3     CE_CAN_13.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_13_CS0            CE_CAN_13.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_13_ID0            CE_CAN_13.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_13_B0             CE_CAN_13.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_13_H0             CE_CAN_13.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_13_W0             CE_CAN_13.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_13_CS1            CE_CAN_13.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_13_ID1            CE_CAN_13.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_13_B1             CE_CAN_13.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_13_H1             CE_CAN_13.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_13_W1             CE_CAN_13.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_13_CS2            CE_CAN_13.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_13_ID2            CE_CAN_13.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_13_B2             CE_CAN_13.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_13_H2             CE_CAN_13.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_13_W2             CE_CAN_13.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_13_CS3            CE_CAN_13.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_13_ID3            CE_CAN_13.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_13_B3             CE_CAN_13.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_13_H3             CE_CAN_13.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_13_W3             CE_CAN_13.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_13_CS4            CE_CAN_13.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_13_ID4            CE_CAN_13.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_13_B4             CE_CAN_13.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_13_H4             CE_CAN_13.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_13_W4             CE_CAN_13.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_13_CS5            CE_CAN_13.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_13_ID5            CE_CAN_13.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_13_B5             CE_CAN_13.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_13_H5             CE_CAN_13.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_13_W5             CE_CAN_13.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_13_CS6            CE_CAN_13.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_13_ID6            CE_CAN_13.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_13_B6             CE_CAN_13.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_13_H6             CE_CAN_13.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_13_W6             CE_CAN_13.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_13_CS7            CE_CAN_13.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_13_ID7            CE_CAN_13.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_13_B7             CE_CAN_13.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_13_H7             CE_CAN_13.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_13_W7             CE_CAN_13.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_13_CS8            CE_CAN_13.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_13_ID8            CE_CAN_13.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_13_B8             CE_CAN_13.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_13_H8             CE_CAN_13.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_13_W8             CE_CAN_13.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_13_CS9            CE_CAN_13.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_13_ID9            CE_CAN_13.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_13_B9             CE_CAN_13.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_13_H9             CE_CAN_13.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_13_W9             CE_CAN_13.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_13_CS10           CE_CAN_13.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_13_ID10           CE_CAN_13.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_13_B10            CE_CAN_13.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_13_H10            CE_CAN_13.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_13_W10            CE_CAN_13.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_13_CS11           CE_CAN_13.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_13_ID11           CE_CAN_13.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_13_B11            CE_CAN_13.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_13_H11            CE_CAN_13.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_13_W11            CE_CAN_13.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_13_CS12           CE_CAN_13.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_13_ID12           CE_CAN_13.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_13_B12            CE_CAN_13.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_13_H12            CE_CAN_13.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_13_W12            CE_CAN_13.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_13_CS13           CE_CAN_13.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_13_ID13           CE_CAN_13.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_13_B13            CE_CAN_13.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_13_H13            CE_CAN_13.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_13_W13            CE_CAN_13.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_13_CS14           CE_CAN_13.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_13_ID14           CE_CAN_13.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_13_B14            CE_CAN_13.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_13_H14            CE_CAN_13.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_13_W14            CE_CAN_13.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_13_CS15           CE_CAN_13.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_13_ID15           CE_CAN_13.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_13_B15            CE_CAN_13.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_13_H15            CE_CAN_13.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_13_W15            CE_CAN_13.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_13_CS16           CE_CAN_13.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_13_ID16           CE_CAN_13.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_13_B16            CE_CAN_13.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_13_H16            CE_CAN_13.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_13_W16            CE_CAN_13.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_13_CS17           CE_CAN_13.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_13_ID17           CE_CAN_13.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_13_B17            CE_CAN_13.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_13_H17            CE_CAN_13.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_13_W17            CE_CAN_13.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_13_CS18           CE_CAN_13.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_13_ID18           CE_CAN_13.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_13_B18            CE_CAN_13.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_13_H18            CE_CAN_13.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_13_W18            CE_CAN_13.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_13_CS19           CE_CAN_13.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_13_ID19           CE_CAN_13.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_13_B19            CE_CAN_13.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_13_H19            CE_CAN_13.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_13_W19            CE_CAN_13.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_13_CS20           CE_CAN_13.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_13_ID20           CE_CAN_13.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_13_B20            CE_CAN_13.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_13_H20            CE_CAN_13.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_13_W20            CE_CAN_13.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_13_CS21           CE_CAN_13.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_13_ID21           CE_CAN_13.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_13_B21            CE_CAN_13.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_13_H21            CE_CAN_13.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_13_W21            CE_CAN_13.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_13_CS22           CE_CAN_13.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_13_ID22           CE_CAN_13.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_13_B22            CE_CAN_13.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_13_H22            CE_CAN_13.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_13_W22            CE_CAN_13.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_13_CS23           CE_CAN_13.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_13_ID23           CE_CAN_13.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_13_B23            CE_CAN_13.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_13_H23            CE_CAN_13.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_13_W23            CE_CAN_13.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_13_CS24           CE_CAN_13.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_13_ID24           CE_CAN_13.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_13_B24            CE_CAN_13.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_13_H24            CE_CAN_13.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_13_W24            CE_CAN_13.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_13_CS25           CE_CAN_13.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_13_ID25           CE_CAN_13.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_13_B25            CE_CAN_13.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_13_H25            CE_CAN_13.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_13_W25            CE_CAN_13.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_13_CS26           CE_CAN_13.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_13_ID26           CE_CAN_13.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_13_B26            CE_CAN_13.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_13_H26            CE_CAN_13.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_13_W26            CE_CAN_13.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_13_CS27           CE_CAN_13.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_13_ID27           CE_CAN_13.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_13_B27            CE_CAN_13.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_13_H27            CE_CAN_13.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_13_W27            CE_CAN_13.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_13_CS28           CE_CAN_13.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_13_ID28           CE_CAN_13.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_13_B28            CE_CAN_13.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_13_H28            CE_CAN_13.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_13_W28            CE_CAN_13.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_13_CS29           CE_CAN_13.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_13_ID29           CE_CAN_13.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_13_B29            CE_CAN_13.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_13_H29            CE_CAN_13.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_13_W29            CE_CAN_13.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_13_CS30           CE_CAN_13.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_13_ID30           CE_CAN_13.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_13_B30            CE_CAN_13.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_13_H30            CE_CAN_13.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_13_W30            CE_CAN_13.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_13_CS31           CE_CAN_13.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_13_ID31           CE_CAN_13.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_13_B31            CE_CAN_13.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_13_H31            CE_CAN_13.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_13_W31            CE_CAN_13.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_13_CS32           CE_CAN_13.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_13_ID32           CE_CAN_13.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_13_B32            CE_CAN_13.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_13_H32            CE_CAN_13.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_13_W32            CE_CAN_13.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_13_CS33           CE_CAN_13.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_13_ID33           CE_CAN_13.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_13_B33            CE_CAN_13.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_13_H33            CE_CAN_13.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_13_W33            CE_CAN_13.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_13_CS34           CE_CAN_13.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_13_ID34           CE_CAN_13.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_13_B34            CE_CAN_13.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_13_H34            CE_CAN_13.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_13_W34            CE_CAN_13.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_13_CS35           CE_CAN_13.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_13_ID35           CE_CAN_13.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_13_B35            CE_CAN_13.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_13_H35            CE_CAN_13.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_13_W35            CE_CAN_13.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_13_CS36           CE_CAN_13.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_13_ID36           CE_CAN_13.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_13_B36            CE_CAN_13.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_13_H36            CE_CAN_13.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_13_W36            CE_CAN_13.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_13_CS37           CE_CAN_13.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_13_ID37           CE_CAN_13.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_13_B37            CE_CAN_13.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_13_H37            CE_CAN_13.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_13_W37            CE_CAN_13.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_13_CS38           CE_CAN_13.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_13_ID38           CE_CAN_13.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_13_B38            CE_CAN_13.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_13_H38            CE_CAN_13.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_13_W38            CE_CAN_13.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_13_CS39           CE_CAN_13.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_13_ID39           CE_CAN_13.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_13_B39            CE_CAN_13.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_13_H39            CE_CAN_13.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_13_W39            CE_CAN_13.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_13_CS40           CE_CAN_13.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_13_ID40           CE_CAN_13.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_13_B40            CE_CAN_13.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_13_H40            CE_CAN_13.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_13_W40            CE_CAN_13.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_13_CS41           CE_CAN_13.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_13_ID41           CE_CAN_13.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_13_B41            CE_CAN_13.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_13_H41            CE_CAN_13.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_13_W41            CE_CAN_13.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_13_CS42           CE_CAN_13.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_13_ID42           CE_CAN_13.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_13_B42            CE_CAN_13.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_13_H42            CE_CAN_13.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_13_W42            CE_CAN_13.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_13_CS43           CE_CAN_13.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_13_ID43           CE_CAN_13.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_13_B43            CE_CAN_13.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_13_H43            CE_CAN_13.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_13_W43            CE_CAN_13.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_13_CS44           CE_CAN_13.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_13_ID44           CE_CAN_13.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_13_B44            CE_CAN_13.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_13_H44            CE_CAN_13.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_13_W44            CE_CAN_13.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_13_CS45           CE_CAN_13.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_13_ID45           CE_CAN_13.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_13_B45            CE_CAN_13.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_13_H45            CE_CAN_13.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_13_W45            CE_CAN_13.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_13_CS46           CE_CAN_13.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_13_ID46           CE_CAN_13.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_13_B46            CE_CAN_13.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_13_H46            CE_CAN_13.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_13_W46            CE_CAN_13.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_13_CS47           CE_CAN_13.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_13_ID47           CE_CAN_13.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_13_B47            CE_CAN_13.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_13_H47            CE_CAN_13.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_13_W47            CE_CAN_13.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_13_CS48           CE_CAN_13.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_13_ID48           CE_CAN_13.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_13_B48            CE_CAN_13.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_13_H48            CE_CAN_13.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_13_W48            CE_CAN_13.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_13_CS49           CE_CAN_13.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_13_ID49           CE_CAN_13.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_13_B49            CE_CAN_13.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_13_H49            CE_CAN_13.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_13_W49            CE_CAN_13.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_13_CS50           CE_CAN_13.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_13_ID50           CE_CAN_13.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_13_B50            CE_CAN_13.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_13_H50            CE_CAN_13.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_13_W50            CE_CAN_13.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_13_CS51           CE_CAN_13.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_13_ID51           CE_CAN_13.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_13_B51            CE_CAN_13.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_13_H51            CE_CAN_13.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_13_W51            CE_CAN_13.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_13_CS52           CE_CAN_13.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_13_ID52           CE_CAN_13.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_13_B52            CE_CAN_13.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_13_H52            CE_CAN_13.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_13_W52            CE_CAN_13.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_13_CS53           CE_CAN_13.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_13_ID53           CE_CAN_13.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_13_B53            CE_CAN_13.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_13_H53            CE_CAN_13.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_13_W53            CE_CAN_13.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_13_CS54           CE_CAN_13.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_13_ID54           CE_CAN_13.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_13_B54            CE_CAN_13.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_13_H54            CE_CAN_13.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_13_W54            CE_CAN_13.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_13_CS55           CE_CAN_13.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_13_ID55           CE_CAN_13.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_13_B55            CE_CAN_13.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_13_H55            CE_CAN_13.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_13_W55            CE_CAN_13.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_13_CS56           CE_CAN_13.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_13_ID56           CE_CAN_13.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_13_B56            CE_CAN_13.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_13_H56            CE_CAN_13.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_13_W56            CE_CAN_13.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_13_CS57           CE_CAN_13.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_13_ID57           CE_CAN_13.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_13_B57            CE_CAN_13.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_13_H57            CE_CAN_13.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_13_W57            CE_CAN_13.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_13_CS58           CE_CAN_13.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_13_ID58           CE_CAN_13.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_13_B58            CE_CAN_13.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_13_H58            CE_CAN_13.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_13_W58            CE_CAN_13.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_13_CS59           CE_CAN_13.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_13_ID59           CE_CAN_13.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_13_B59            CE_CAN_13.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_13_H59            CE_CAN_13.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_13_W59            CE_CAN_13.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_13_CS60           CE_CAN_13.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_13_ID60           CE_CAN_13.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_13_B60            CE_CAN_13.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_13_H60            CE_CAN_13.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_13_W60            CE_CAN_13.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_13_CS61           CE_CAN_13.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_13_ID61           CE_CAN_13.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_13_B61            CE_CAN_13.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_13_H61            CE_CAN_13.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_13_W61            CE_CAN_13.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_13_CS62           CE_CAN_13.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_13_ID62           CE_CAN_13.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_13_B62            CE_CAN_13.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_13_H62            CE_CAN_13.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_13_W62            CE_CAN_13.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_13_CS63           CE_CAN_13.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_13_ID63           CE_CAN_13.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_13_B63            CE_CAN_13.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_13_H63            CE_CAN_13.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_13_W63            CE_CAN_13.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_13_CS64           CE_CAN_13.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_13_ID64           CE_CAN_13.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_13_B64            CE_CAN_13.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_13_H64            CE_CAN_13.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_13_W64            CE_CAN_13.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_13_CS65           CE_CAN_13.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_13_ID65           CE_CAN_13.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_13_B65            CE_CAN_13.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_13_H65            CE_CAN_13.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_13_W65            CE_CAN_13.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_13_CS66           CE_CAN_13.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_13_ID66           CE_CAN_13.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_13_B66            CE_CAN_13.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_13_H66            CE_CAN_13.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_13_W66            CE_CAN_13.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_13_CS67           CE_CAN_13.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_13_ID67           CE_CAN_13.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_13_B67            CE_CAN_13.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_13_H67            CE_CAN_13.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_13_W67            CE_CAN_13.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_13_CS68           CE_CAN_13.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_13_ID68           CE_CAN_13.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_13_B68            CE_CAN_13.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_13_H68            CE_CAN_13.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_13_W68            CE_CAN_13.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_13_CS69           CE_CAN_13.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_13_ID69           CE_CAN_13.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_13_B69            CE_CAN_13.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_13_H69            CE_CAN_13.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_13_W69            CE_CAN_13.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_13_CS70           CE_CAN_13.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_13_ID70           CE_CAN_13.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_13_B70            CE_CAN_13.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_13_H70            CE_CAN_13.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_13_W70            CE_CAN_13.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_13_CS71           CE_CAN_13.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_13_ID71           CE_CAN_13.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_13_B71            CE_CAN_13.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_13_H71            CE_CAN_13.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_13_W71            CE_CAN_13.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_13_CS72           CE_CAN_13.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_13_ID72           CE_CAN_13.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_13_B72            CE_CAN_13.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_13_H72            CE_CAN_13.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_13_W72            CE_CAN_13.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_13_CS73           CE_CAN_13.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_13_ID73           CE_CAN_13.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_13_B73            CE_CAN_13.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_13_H73            CE_CAN_13.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_13_W73            CE_CAN_13.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_13_CS74           CE_CAN_13.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_13_ID74           CE_CAN_13.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_13_B74            CE_CAN_13.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_13_H74            CE_CAN_13.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_13_W74            CE_CAN_13.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_13_CS75           CE_CAN_13.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_13_ID75           CE_CAN_13.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_13_B75            CE_CAN_13.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_13_H75            CE_CAN_13.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_13_W75            CE_CAN_13.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_13_CS76           CE_CAN_13.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_13_ID76           CE_CAN_13.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_13_B76            CE_CAN_13.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_13_H76            CE_CAN_13.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_13_W76            CE_CAN_13.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_13_CS77           CE_CAN_13.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_13_ID77           CE_CAN_13.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_13_B77            CE_CAN_13.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_13_H77            CE_CAN_13.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_13_W77            CE_CAN_13.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_13_CS78           CE_CAN_13.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_13_ID78           CE_CAN_13.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_13_B78            CE_CAN_13.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_13_H78            CE_CAN_13.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_13_W78            CE_CAN_13.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_13_CS79           CE_CAN_13.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_13_ID79           CE_CAN_13.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_13_B79            CE_CAN_13.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_13_H79            CE_CAN_13.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_13_W79            CE_CAN_13.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_13_CS80           CE_CAN_13.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_13_ID80           CE_CAN_13.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_13_B80            CE_CAN_13.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_13_H80            CE_CAN_13.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_13_W80            CE_CAN_13.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_13_CS81           CE_CAN_13.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_13_ID81           CE_CAN_13.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_13_B81            CE_CAN_13.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_13_H81            CE_CAN_13.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_13_W81            CE_CAN_13.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_13_CS82           CE_CAN_13.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_13_ID82           CE_CAN_13.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_13_B82            CE_CAN_13.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_13_H82            CE_CAN_13.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_13_W82            CE_CAN_13.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_13_CS83           CE_CAN_13.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_13_ID83           CE_CAN_13.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_13_B83            CE_CAN_13.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_13_H83            CE_CAN_13.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_13_W83            CE_CAN_13.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_13_CS84           CE_CAN_13.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_13_ID84           CE_CAN_13.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_13_B84            CE_CAN_13.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_13_H84            CE_CAN_13.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_13_W84            CE_CAN_13.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_13_CS85           CE_CAN_13.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_13_ID85           CE_CAN_13.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_13_B85            CE_CAN_13.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_13_H85            CE_CAN_13.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_13_W85            CE_CAN_13.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_13_CS86           CE_CAN_13.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_13_ID86           CE_CAN_13.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_13_B86            CE_CAN_13.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_13_H86            CE_CAN_13.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_13_W86            CE_CAN_13.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_13_CS87           CE_CAN_13.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_13_ID87           CE_CAN_13.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_13_B87            CE_CAN_13.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_13_H87            CE_CAN_13.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_13_W87            CE_CAN_13.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_13_CS88           CE_CAN_13.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_13_ID88           CE_CAN_13.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_13_B88            CE_CAN_13.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_13_H88            CE_CAN_13.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_13_W848           CE_CAN_13.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_13_CS89           CE_CAN_13.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_13_ID89           CE_CAN_13.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_13_B89            CE_CAN_13.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_13_H89            CE_CAN_13.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_13_W89            CE_CAN_13.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_13_CS90           CE_CAN_13.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_13_ID90           CE_CAN_13.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_13_B90            CE_CAN_13.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_13_H90            CE_CAN_13.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_13_W90            CE_CAN_13.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_13_CS91           CE_CAN_13.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_13_ID91           CE_CAN_13.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_13_B91            CE_CAN_13.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_13_H91            CE_CAN_13.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_13_W91            CE_CAN_13.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_13_CS92           CE_CAN_13.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_13_ID92           CE_CAN_13.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_13_B92            CE_CAN_13.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_13_H92            CE_CAN_13.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_13_W92            CE_CAN_13.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_13_CS93           CE_CAN_13.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_13_ID93           CE_CAN_13.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_13_B93            CE_CAN_13.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_13_H93            CE_CAN_13.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_13_W93            CE_CAN_13.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_13_CS94           CE_CAN_13.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_13_ID94           CE_CAN_13.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_13_B94            CE_CAN_13.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_13_H94            CE_CAN_13.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_13_W94            CE_CAN_13.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_13_CS95           CE_CAN_13.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_13_ID95           CE_CAN_13.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_13_B95            CE_CAN_13.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_13_H95            CE_CAN_13.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_13_W95            CE_CAN_13.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_13_CS96           CE_CAN_13.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_13_ID96           CE_CAN_13.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_13_B96            CE_CAN_13.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_13_H96            CE_CAN_13.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_13_W96            CE_CAN_13.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_13_CS97           CE_CAN_13.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_13_ID97           CE_CAN_13.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_13_B97            CE_CAN_13.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_13_H97            CE_CAN_13.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_13_W97            CE_CAN_13.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_13_CS98           CE_CAN_13.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_13_ID98           CE_CAN_13.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_13_B98            CE_CAN_13.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_13_H98            CE_CAN_13.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_13_W98            CE_CAN_13.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_13_CS99           CE_CAN_13.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_13_ID99           CE_CAN_13.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_13_B99            CE_CAN_13.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_13_H99            CE_CAN_13.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_13_W99            CE_CAN_13.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_13_CS100          CE_CAN_13.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_13_ID100          CE_CAN_13.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_13_B100           CE_CAN_13.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_13_H100           CE_CAN_13.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_13_W100           CE_CAN_13.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_13_CS101          CE_CAN_13.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_13_ID101          CE_CAN_13.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_13_B101           CE_CAN_13.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_13_H101           CE_CAN_13.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_13_W101           CE_CAN_13.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_13_CS102          CE_CAN_13.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_13_ID102          CE_CAN_13.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_13_B102           CE_CAN_13.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_13_H102           CE_CAN_13.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_13_W102           CE_CAN_13.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_13_CS103          CE_CAN_13.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_13_ID103          CE_CAN_13.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_13_B103           CE_CAN_13.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_13_H103           CE_CAN_13.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_13_W103           CE_CAN_13.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_13_CS104          CE_CAN_13.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_13_ID104          CE_CAN_13.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_13_B104           CE_CAN_13.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_13_H104           CE_CAN_13.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_13_W104           CE_CAN_13.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_13_CS105          CE_CAN_13.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_13_ID105          CE_CAN_13.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_13_B105           CE_CAN_13.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_13_H105           CE_CAN_13.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_13_W105           CE_CAN_13.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_13_CS106          CE_CAN_13.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_13_ID106          CE_CAN_13.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_13_B106           CE_CAN_13.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_13_H106           CE_CAN_13.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_13_W106           CE_CAN_13.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_13_CS107          CE_CAN_13.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_13_ID107          CE_CAN_13.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_13_B107           CE_CAN_13.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_13_H107           CE_CAN_13.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_13_W107           CE_CAN_13.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_13_CS108          CE_CAN_13.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_13_ID108          CE_CAN_13.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_13_B108           CE_CAN_13.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_13_H108           CE_CAN_13.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_13_W108           CE_CAN_13.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_13_CS109          CE_CAN_13.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_13_ID109          CE_CAN_13.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_13_B109           CE_CAN_13.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_13_H109           CE_CAN_13.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_13_W109           CE_CAN_13.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_13_CS110          CE_CAN_13.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_13_ID110          CE_CAN_13.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_13_B110           CE_CAN_13.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_13_H110           CE_CAN_13.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_13_W110           CE_CAN_13.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_13_CS111          CE_CAN_13.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_13_ID111          CE_CAN_13.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_13_B111           CE_CAN_13.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_13_H111           CE_CAN_13.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_13_W111           CE_CAN_13.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_13_CS112          CE_CAN_13.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_13_ID112          CE_CAN_13.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_13_B112           CE_CAN_13.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_13_H112           CE_CAN_13.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_13_W112           CE_CAN_13.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_13_CS113          CE_CAN_13.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_13_ID113          CE_CAN_13.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_13_B113           CE_CAN_13.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_13_H113           CE_CAN_13.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_13_W113           CE_CAN_13.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_13_CS114          CE_CAN_13.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_13_ID114          CE_CAN_13.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_13_B114           CE_CAN_13.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_13_H114           CE_CAN_13.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_13_W114           CE_CAN_13.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_13_CS115          CE_CAN_13.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_13_ID115          CE_CAN_13.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_13_B115           CE_CAN_13.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_13_H115           CE_CAN_13.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_13_W115           CE_CAN_13.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_13_CS116          CE_CAN_13.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_13_ID116          CE_CAN_13.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_13_B116           CE_CAN_13.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_13_H116           CE_CAN_13.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_13_W116           CE_CAN_13.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_13_CS117          CE_CAN_13.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_13_ID117          CE_CAN_13.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_13_B117           CE_CAN_13.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_13_H117           CE_CAN_13.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_13_W117           CE_CAN_13.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_13_CS118          CE_CAN_13.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_13_ID118          CE_CAN_13.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_13_B118           CE_CAN_13.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_13_H118           CE_CAN_13.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_13_W118           CE_CAN_13.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_13_CS119          CE_CAN_13.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_13_ID119          CE_CAN_13.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_13_B119           CE_CAN_13.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_13_H119           CE_CAN_13.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_13_W119           CE_CAN_13.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_13_CS120          CE_CAN_13.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_13_ID120          CE_CAN_13.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_13_B120           CE_CAN_13.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_13_H120           CE_CAN_13.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_13_W120           CE_CAN_13.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_13_CS121          CE_CAN_13.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_13_ID121          CE_CAN_13.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_13_B121           CE_CAN_13.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_13_H121           CE_CAN_13.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_13_W121           CE_CAN_13.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_13_CS122          CE_CAN_13.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_13_ID122          CE_CAN_13.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_13_B122           CE_CAN_13.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_13_H122           CE_CAN_13.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_13_W122           CE_CAN_13.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_13_CS123          CE_CAN_13.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_13_ID123          CE_CAN_13.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_13_B123           CE_CAN_13.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_13_H123           CE_CAN_13.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_13_W123           CE_CAN_13.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_13_CS124          CE_CAN_13.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_13_ID124          CE_CAN_13.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_13_B124           CE_CAN_13.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_13_H124           CE_CAN_13.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_13_W124           CE_CAN_13.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_13_CS125          CE_CAN_13.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_13_ID125          CE_CAN_13.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_13_B125           CE_CAN_13.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_13_H125           CE_CAN_13.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_13_W125           CE_CAN_13.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_13_CS126          CE_CAN_13.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_13_ID126          CE_CAN_13.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_13_B126           CE_CAN_13.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_13_H126           CE_CAN_13.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_13_W126           CE_CAN_13.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_13_CS127          CE_CAN_13.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_13_ID127          CE_CAN_13.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_13_B127           CE_CAN_13.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_13_H127           CE_CAN_13.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_13_W127           CE_CAN_13.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_13_RXIMR0     CE_CAN_13.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR1     CE_CAN_13.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR2     CE_CAN_13.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR3     CE_CAN_13.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR4     CE_CAN_13.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR5     CE_CAN_13.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR6     CE_CAN_13.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR7     CE_CAN_13.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR8     CE_CAN_13.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR9     CE_CAN_13.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_13_RXIMR10    CE_CAN_13.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR11    CE_CAN_13.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR12    CE_CAN_13.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR13    CE_CAN_13.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR14    CE_CAN_13.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR15    CE_CAN_13.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR16    CE_CAN_13.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR17    CE_CAN_13.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR18    CE_CAN_13.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR19    CE_CAN_13.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR20    CE_CAN_13.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR21    CE_CAN_13.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR22    CE_CAN_13.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR23    CE_CAN_13.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR24    CE_CAN_13.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR25    CE_CAN_13.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR26    CE_CAN_13.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR27    CE_CAN_13.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR28    CE_CAN_13.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR29    CE_CAN_13.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR30    CE_CAN_13.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR31    CE_CAN_13.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR32    CE_CAN_13.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR33    CE_CAN_13.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR34    CE_CAN_13.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR35    CE_CAN_13.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR36    CE_CAN_13.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR37    CE_CAN_13.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR38    CE_CAN_13.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR39    CE_CAN_13.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR40    CE_CAN_13.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR41    CE_CAN_13.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR42    CE_CAN_13.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR43    CE_CAN_13.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR44    CE_CAN_13.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR45    CE_CAN_13.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR46    CE_CAN_13.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR47    CE_CAN_13.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR48    CE_CAN_13.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR49    CE_CAN_13.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR50    CE_CAN_13.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR51    CE_CAN_13.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR52    CE_CAN_13.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR53    CE_CAN_13.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR54    CE_CAN_13.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR55    CE_CAN_13.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR56    CE_CAN_13.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR57    CE_CAN_13.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR58    CE_CAN_13.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR59    CE_CAN_13.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR60    CE_CAN_13.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR61    CE_CAN_13.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR62    CE_CAN_13.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR63    CE_CAN_13.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR64    CE_CAN_13.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR65    CE_CAN_13.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR66    CE_CAN_13.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR67    CE_CAN_13.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR68    CE_CAN_13.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR69    CE_CAN_13.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR70    CE_CAN_13.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR71    CE_CAN_13.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR72    CE_CAN_13.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR73    CE_CAN_13.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR74    CE_CAN_13.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR75    CE_CAN_13.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR76    CE_CAN_13.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR77    CE_CAN_13.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR78    CE_CAN_13.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR79    CE_CAN_13.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR80    CE_CAN_13.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR81    CE_CAN_13.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR82    CE_CAN_13.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR83    CE_CAN_13.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR84    CE_CAN_13.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR85    CE_CAN_13.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR86    CE_CAN_13.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR87    CE_CAN_13.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR88    CE_CAN_13.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR89    CE_CAN_13.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR90    CE_CAN_13.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR91    CE_CAN_13.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR92    CE_CAN_13.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR93    CE_CAN_13.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR94    CE_CAN_13.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR95    CE_CAN_13.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR96    CE_CAN_13.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR97    CE_CAN_13.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR98    CE_CAN_13.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR99    CE_CAN_13.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_13_RXIMR100   CE_CAN_13.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR101   CE_CAN_13.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR102   CE_CAN_13.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR103   CE_CAN_13.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR104   CE_CAN_13.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR105   CE_CAN_13.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR106   CE_CAN_13.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR107   CE_CAN_13.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR108   CE_CAN_13.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR109   CE_CAN_13.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR110   CE_CAN_13.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR111   CE_CAN_13.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR112   CE_CAN_13.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR113   CE_CAN_13.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR114   CE_CAN_13.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR115   CE_CAN_13.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR116   CE_CAN_13.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR117   CE_CAN_13.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR118   CE_CAN_13.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR119   CE_CAN_13.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR120   CE_CAN_13.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR121   CE_CAN_13.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR122   CE_CAN_13.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR123   CE_CAN_13.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR124   CE_CAN_13.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR125   CE_CAN_13.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR126   CE_CAN_13.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_13_RXIMR127   CE_CAN_13.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_13_MECR       CE_CAN_13.MECR.R              /* Memory Error Control */
#define CE_CAN_13_ERRIAR     CE_CAN_13.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_13_ERRIDPR    CE_CAN_13.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_13_ERRIPPR    CE_CAN_13.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_13_RERRAR     CE_CAN_13.RERRAR.R            /* Error Report Address */
#define CE_CAN_13_RERRDR     CE_CAN_13.RERRDR.R            /* Error Report Data */
#define CE_CAN_13_RERRSYNR   CE_CAN_13.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_13_ERRSR      CE_CAN_13.ERRSR.R             /* Error Status */
#define CE_CAN_13_EPRS       CE_CAN_13.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_13_ENCBT      CE_CAN_13.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_13_EDCBT      CE_CAN_13.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_13_ETDC       CE_CAN_13.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_13_FDCTRL     CE_CAN_13.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_13_FDCBT      CE_CAN_13.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_13_FDCRC      CE_CAN_13.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_13_ERFCR      CE_CAN_13.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_13_ERFIER     CE_CAN_13.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_13_ERFSR      CE_CAN_13.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_13_HR_TIME_STAMP0 CE_CAN_13.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP1 CE_CAN_13.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP2 CE_CAN_13.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP3 CE_CAN_13.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP4 CE_CAN_13.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP5 CE_CAN_13.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP6 CE_CAN_13.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP7 CE_CAN_13.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP8 CE_CAN_13.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP9 CE_CAN_13.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP10 CE_CAN_13.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP11 CE_CAN_13.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP12 CE_CAN_13.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP13 CE_CAN_13.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP14 CE_CAN_13.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP15 CE_CAN_13.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP16 CE_CAN_13.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP17 CE_CAN_13.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP18 CE_CAN_13.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP19 CE_CAN_13.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP20 CE_CAN_13.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP21 CE_CAN_13.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP22 CE_CAN_13.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP23 CE_CAN_13.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP24 CE_CAN_13.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP25 CE_CAN_13.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP26 CE_CAN_13.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP27 CE_CAN_13.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP28 CE_CAN_13.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP29 CE_CAN_13.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP30 CE_CAN_13.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP31 CE_CAN_13.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP32 CE_CAN_13.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP33 CE_CAN_13.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP34 CE_CAN_13.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP35 CE_CAN_13.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP36 CE_CAN_13.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP37 CE_CAN_13.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP38 CE_CAN_13.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP39 CE_CAN_13.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP40 CE_CAN_13.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP41 CE_CAN_13.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP42 CE_CAN_13.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP43 CE_CAN_13.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP44 CE_CAN_13.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP45 CE_CAN_13.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP46 CE_CAN_13.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP47 CE_CAN_13.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP48 CE_CAN_13.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP49 CE_CAN_13.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP50 CE_CAN_13.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP51 CE_CAN_13.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP52 CE_CAN_13.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP53 CE_CAN_13.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP54 CE_CAN_13.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP55 CE_CAN_13.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP56 CE_CAN_13.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP57 CE_CAN_13.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP58 CE_CAN_13.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP59 CE_CAN_13.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP60 CE_CAN_13.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP61 CE_CAN_13.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP62 CE_CAN_13.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP63 CE_CAN_13.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP64 CE_CAN_13.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP65 CE_CAN_13.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP66 CE_CAN_13.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP67 CE_CAN_13.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP68 CE_CAN_13.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP69 CE_CAN_13.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP70 CE_CAN_13.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP71 CE_CAN_13.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP72 CE_CAN_13.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP73 CE_CAN_13.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP74 CE_CAN_13.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP75 CE_CAN_13.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP76 CE_CAN_13.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP77 CE_CAN_13.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP78 CE_CAN_13.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP79 CE_CAN_13.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP80 CE_CAN_13.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP81 CE_CAN_13.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP82 CE_CAN_13.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP83 CE_CAN_13.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP84 CE_CAN_13.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP85 CE_CAN_13.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP86 CE_CAN_13.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP87 CE_CAN_13.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP88 CE_CAN_13.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP89 CE_CAN_13.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP90 CE_CAN_13.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP91 CE_CAN_13.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP92 CE_CAN_13.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP93 CE_CAN_13.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP94 CE_CAN_13.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP95 CE_CAN_13.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP96 CE_CAN_13.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP97 CE_CAN_13.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP98 CE_CAN_13.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP99 CE_CAN_13.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP100 CE_CAN_13.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP101 CE_CAN_13.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP102 CE_CAN_13.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP103 CE_CAN_13.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP104 CE_CAN_13.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP105 CE_CAN_13.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP106 CE_CAN_13.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP107 CE_CAN_13.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP108 CE_CAN_13.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP109 CE_CAN_13.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP110 CE_CAN_13.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP111 CE_CAN_13.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP112 CE_CAN_13.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP113 CE_CAN_13.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP114 CE_CAN_13.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP115 CE_CAN_13.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP116 CE_CAN_13.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP117 CE_CAN_13.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP118 CE_CAN_13.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP119 CE_CAN_13.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP120 CE_CAN_13.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP121 CE_CAN_13.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP122 CE_CAN_13.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP123 CE_CAN_13.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP124 CE_CAN_13.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP125 CE_CAN_13.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP126 CE_CAN_13.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_13_HR_TIME_STAMP127 CE_CAN_13.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_13_ERFFEL0    CE_CAN_13.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL1    CE_CAN_13.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL2    CE_CAN_13.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL3    CE_CAN_13.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL4    CE_CAN_13.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL5    CE_CAN_13.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL6    CE_CAN_13.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL7    CE_CAN_13.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL8    CE_CAN_13.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL9    CE_CAN_13.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL10   CE_CAN_13.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL11   CE_CAN_13.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL12   CE_CAN_13.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL13   CE_CAN_13.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL14   CE_CAN_13.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL15   CE_CAN_13.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL16   CE_CAN_13.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL17   CE_CAN_13.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL18   CE_CAN_13.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL19   CE_CAN_13.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL20   CE_CAN_13.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL21   CE_CAN_13.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL22   CE_CAN_13.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL23   CE_CAN_13.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL24   CE_CAN_13.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL25   CE_CAN_13.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL26   CE_CAN_13.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL27   CE_CAN_13.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL28   CE_CAN_13.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL29   CE_CAN_13.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL30   CE_CAN_13.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL31   CE_CAN_13.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL32   CE_CAN_13.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL33   CE_CAN_13.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL34   CE_CAN_13.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL35   CE_CAN_13.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL36   CE_CAN_13.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL37   CE_CAN_13.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL38   CE_CAN_13.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL39   CE_CAN_13.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL40   CE_CAN_13.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL41   CE_CAN_13.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL42   CE_CAN_13.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL43   CE_CAN_13.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL44   CE_CAN_13.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL45   CE_CAN_13.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL46   CE_CAN_13.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL47   CE_CAN_13.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL48   CE_CAN_13.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL49   CE_CAN_13.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL50   CE_CAN_13.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL51   CE_CAN_13.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL52   CE_CAN_13.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL53   CE_CAN_13.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL54   CE_CAN_13.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL55   CE_CAN_13.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL56   CE_CAN_13.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL57   CE_CAN_13.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL58   CE_CAN_13.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL59   CE_CAN_13.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL60   CE_CAN_13.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL61   CE_CAN_13.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL62   CE_CAN_13.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL63   CE_CAN_13.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL64   CE_CAN_13.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL65   CE_CAN_13.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL66   CE_CAN_13.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL67   CE_CAN_13.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL68   CE_CAN_13.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL69   CE_CAN_13.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL70   CE_CAN_13.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL71   CE_CAN_13.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL72   CE_CAN_13.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL73   CE_CAN_13.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL74   CE_CAN_13.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL75   CE_CAN_13.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL76   CE_CAN_13.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL77   CE_CAN_13.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL78   CE_CAN_13.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL79   CE_CAN_13.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL80   CE_CAN_13.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL81   CE_CAN_13.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL82   CE_CAN_13.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL83   CE_CAN_13.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL84   CE_CAN_13.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL85   CE_CAN_13.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL86   CE_CAN_13.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL87   CE_CAN_13.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL88   CE_CAN_13.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL89   CE_CAN_13.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL90   CE_CAN_13.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL91   CE_CAN_13.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL92   CE_CAN_13.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL93   CE_CAN_13.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL94   CE_CAN_13.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL95   CE_CAN_13.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL96   CE_CAN_13.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL97   CE_CAN_13.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL98   CE_CAN_13.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL99   CE_CAN_13.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL100  CE_CAN_13.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL101  CE_CAN_13.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL102  CE_CAN_13.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL103  CE_CAN_13.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL104  CE_CAN_13.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL105  CE_CAN_13.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL106  CE_CAN_13.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL107  CE_CAN_13.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL108  CE_CAN_13.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL109  CE_CAN_13.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL110  CE_CAN_13.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL111  CE_CAN_13.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL112  CE_CAN_13.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL113  CE_CAN_13.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL114  CE_CAN_13.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL115  CE_CAN_13.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL116  CE_CAN_13.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL117  CE_CAN_13.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL118  CE_CAN_13.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL119  CE_CAN_13.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL120  CE_CAN_13.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL121  CE_CAN_13.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL122  CE_CAN_13.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL123  CE_CAN_13.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL124  CE_CAN_13.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL125  CE_CAN_13.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL126  CE_CAN_13.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_13_ERFFEL127  CE_CAN_13.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_14_MCR        CE_CAN_14.MCR.R               /* Module Configuration */
#define CE_CAN_14_CTRL1      CE_CAN_14.CTRL1.R             /* Control 1 */
#define CE_CAN_14_TIMER      CE_CAN_14.TIMER.R             /* Free-Running Timer */
#define CE_CAN_14_RXMGMASK   CE_CAN_14.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_14_RX14MASK   CE_CAN_14.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_14_RX15MASK   CE_CAN_14.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_14_ECR        CE_CAN_14.ECR.R               /* Error Counter */
#define CE_CAN_14_ESR1       CE_CAN_14.ESR1.R              /* Error and Status 1 */
#define CE_CAN_14_IMASK2     CE_CAN_14.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_14_IMASK1     CE_CAN_14.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_14_IFLAG2     CE_CAN_14.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_14_IFLAG1     CE_CAN_14.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_14_CTRL2      CE_CAN_14.CTRL2.R             /* Control 2 */
#define CE_CAN_14_ESR2       CE_CAN_14.ESR2.R              /* Error and Status 2 */
#define CE_CAN_14_CRCR       CE_CAN_14.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_14_RXFGMASK   CE_CAN_14.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_14_RXFIR      CE_CAN_14.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_14_CBT        CE_CAN_14.CBT.R               /* CAN Bit Timing */
#define CE_CAN_14_IMASK4     CE_CAN_14.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_14_IMASK3     CE_CAN_14.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_14_IFLAG4     CE_CAN_14.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_14_IFLAG3     CE_CAN_14.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_14_CS0            CE_CAN_14.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_14_ID0            CE_CAN_14.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_14_B0             CE_CAN_14.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_14_H0             CE_CAN_14.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_14_W0             CE_CAN_14.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_14_CS1            CE_CAN_14.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_14_ID1            CE_CAN_14.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_14_B1             CE_CAN_14.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_14_H1             CE_CAN_14.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_14_W1             CE_CAN_14.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_14_CS2            CE_CAN_14.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_14_ID2            CE_CAN_14.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_14_B2             CE_CAN_14.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_14_H2             CE_CAN_14.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_14_W2             CE_CAN_14.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_14_CS3            CE_CAN_14.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_14_ID3            CE_CAN_14.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_14_B3             CE_CAN_14.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_14_H3             CE_CAN_14.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_14_W3             CE_CAN_14.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_14_CS4            CE_CAN_14.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_14_ID4            CE_CAN_14.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_14_B4             CE_CAN_14.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_14_H4             CE_CAN_14.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_14_W4             CE_CAN_14.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_14_CS5            CE_CAN_14.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_14_ID5            CE_CAN_14.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_14_B5             CE_CAN_14.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_14_H5             CE_CAN_14.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_14_W5             CE_CAN_14.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_14_CS6            CE_CAN_14.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_14_ID6            CE_CAN_14.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_14_B6             CE_CAN_14.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_14_H6             CE_CAN_14.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_14_W6             CE_CAN_14.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_14_CS7            CE_CAN_14.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_14_ID7            CE_CAN_14.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_14_B7             CE_CAN_14.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_14_H7             CE_CAN_14.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_14_W7             CE_CAN_14.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_14_CS8            CE_CAN_14.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_14_ID8            CE_CAN_14.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_14_B8             CE_CAN_14.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_14_H8             CE_CAN_14.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_14_W8             CE_CAN_14.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_14_CS9            CE_CAN_14.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_14_ID9            CE_CAN_14.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_14_B9             CE_CAN_14.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_14_H9             CE_CAN_14.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_14_W9             CE_CAN_14.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_14_CS10           CE_CAN_14.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_14_ID10           CE_CAN_14.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_14_B10            CE_CAN_14.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_14_H10            CE_CAN_14.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_14_W10            CE_CAN_14.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_14_CS11           CE_CAN_14.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_14_ID11           CE_CAN_14.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_14_B11            CE_CAN_14.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_14_H11            CE_CAN_14.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_14_W11            CE_CAN_14.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_14_CS12           CE_CAN_14.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_14_ID12           CE_CAN_14.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_14_B12            CE_CAN_14.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_14_H12            CE_CAN_14.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_14_W12            CE_CAN_14.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_14_CS13           CE_CAN_14.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_14_ID13           CE_CAN_14.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_14_B13            CE_CAN_14.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_14_H13            CE_CAN_14.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_14_W13            CE_CAN_14.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_14_CS14           CE_CAN_14.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_14_ID14           CE_CAN_14.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_14_B14            CE_CAN_14.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_14_H14            CE_CAN_14.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_14_W14            CE_CAN_14.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_14_CS15           CE_CAN_14.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_14_ID15           CE_CAN_14.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_14_B15            CE_CAN_14.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_14_H15            CE_CAN_14.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_14_W15            CE_CAN_14.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_14_CS16           CE_CAN_14.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_14_ID16           CE_CAN_14.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_14_B16            CE_CAN_14.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_14_H16            CE_CAN_14.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_14_W16            CE_CAN_14.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_14_CS17           CE_CAN_14.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_14_ID17           CE_CAN_14.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_14_B17            CE_CAN_14.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_14_H17            CE_CAN_14.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_14_W17            CE_CAN_14.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_14_CS18           CE_CAN_14.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_14_ID18           CE_CAN_14.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_14_B18            CE_CAN_14.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_14_H18            CE_CAN_14.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_14_W18            CE_CAN_14.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_14_CS19           CE_CAN_14.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_14_ID19           CE_CAN_14.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_14_B19            CE_CAN_14.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_14_H19            CE_CAN_14.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_14_W19            CE_CAN_14.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_14_CS20           CE_CAN_14.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_14_ID20           CE_CAN_14.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_14_B20            CE_CAN_14.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_14_H20            CE_CAN_14.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_14_W20            CE_CAN_14.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_14_CS21           CE_CAN_14.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_14_ID21           CE_CAN_14.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_14_B21            CE_CAN_14.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_14_H21            CE_CAN_14.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_14_W21            CE_CAN_14.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_14_CS22           CE_CAN_14.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_14_ID22           CE_CAN_14.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_14_B22            CE_CAN_14.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_14_H22            CE_CAN_14.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_14_W22            CE_CAN_14.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_14_CS23           CE_CAN_14.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_14_ID23           CE_CAN_14.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_14_B23            CE_CAN_14.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_14_H23            CE_CAN_14.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_14_W23            CE_CAN_14.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_14_CS24           CE_CAN_14.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_14_ID24           CE_CAN_14.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_14_B24            CE_CAN_14.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_14_H24            CE_CAN_14.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_14_W24            CE_CAN_14.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_14_CS25           CE_CAN_14.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_14_ID25           CE_CAN_14.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_14_B25            CE_CAN_14.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_14_H25            CE_CAN_14.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_14_W25            CE_CAN_14.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_14_CS26           CE_CAN_14.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_14_ID26           CE_CAN_14.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_14_B26            CE_CAN_14.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_14_H26            CE_CAN_14.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_14_W26            CE_CAN_14.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_14_CS27           CE_CAN_14.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_14_ID27           CE_CAN_14.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_14_B27            CE_CAN_14.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_14_H27            CE_CAN_14.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_14_W27            CE_CAN_14.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_14_CS28           CE_CAN_14.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_14_ID28           CE_CAN_14.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_14_B28            CE_CAN_14.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_14_H28            CE_CAN_14.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_14_W28            CE_CAN_14.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_14_CS29           CE_CAN_14.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_14_ID29           CE_CAN_14.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_14_B29            CE_CAN_14.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_14_H29            CE_CAN_14.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_14_W29            CE_CAN_14.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_14_CS30           CE_CAN_14.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_14_ID30           CE_CAN_14.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_14_B30            CE_CAN_14.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_14_H30            CE_CAN_14.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_14_W30            CE_CAN_14.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_14_CS31           CE_CAN_14.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_14_ID31           CE_CAN_14.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_14_B31            CE_CAN_14.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_14_H31            CE_CAN_14.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_14_W31            CE_CAN_14.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_14_CS32           CE_CAN_14.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_14_ID32           CE_CAN_14.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_14_B32            CE_CAN_14.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_14_H32            CE_CAN_14.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_14_W32            CE_CAN_14.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_14_CS33           CE_CAN_14.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_14_ID33           CE_CAN_14.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_14_B33            CE_CAN_14.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_14_H33            CE_CAN_14.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_14_W33            CE_CAN_14.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_14_CS34           CE_CAN_14.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_14_ID34           CE_CAN_14.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_14_B34            CE_CAN_14.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_14_H34            CE_CAN_14.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_14_W34            CE_CAN_14.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_14_CS35           CE_CAN_14.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_14_ID35           CE_CAN_14.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_14_B35            CE_CAN_14.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_14_H35            CE_CAN_14.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_14_W35            CE_CAN_14.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_14_CS36           CE_CAN_14.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_14_ID36           CE_CAN_14.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_14_B36            CE_CAN_14.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_14_H36            CE_CAN_14.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_14_W36            CE_CAN_14.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_14_CS37           CE_CAN_14.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_14_ID37           CE_CAN_14.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_14_B37            CE_CAN_14.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_14_H37            CE_CAN_14.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_14_W37            CE_CAN_14.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_14_CS38           CE_CAN_14.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_14_ID38           CE_CAN_14.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_14_B38            CE_CAN_14.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_14_H38            CE_CAN_14.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_14_W38            CE_CAN_14.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_14_CS39           CE_CAN_14.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_14_ID39           CE_CAN_14.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_14_B39            CE_CAN_14.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_14_H39            CE_CAN_14.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_14_W39            CE_CAN_14.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_14_CS40           CE_CAN_14.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_14_ID40           CE_CAN_14.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_14_B40            CE_CAN_14.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_14_H40            CE_CAN_14.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_14_W40            CE_CAN_14.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_14_CS41           CE_CAN_14.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_14_ID41           CE_CAN_14.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_14_B41            CE_CAN_14.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_14_H41            CE_CAN_14.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_14_W41            CE_CAN_14.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_14_CS42           CE_CAN_14.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_14_ID42           CE_CAN_14.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_14_B42            CE_CAN_14.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_14_H42            CE_CAN_14.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_14_W42            CE_CAN_14.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_14_CS43           CE_CAN_14.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_14_ID43           CE_CAN_14.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_14_B43            CE_CAN_14.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_14_H43            CE_CAN_14.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_14_W43            CE_CAN_14.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_14_CS44           CE_CAN_14.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_14_ID44           CE_CAN_14.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_14_B44            CE_CAN_14.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_14_H44            CE_CAN_14.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_14_W44            CE_CAN_14.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_14_CS45           CE_CAN_14.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_14_ID45           CE_CAN_14.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_14_B45            CE_CAN_14.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_14_H45            CE_CAN_14.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_14_W45            CE_CAN_14.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_14_CS46           CE_CAN_14.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_14_ID46           CE_CAN_14.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_14_B46            CE_CAN_14.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_14_H46            CE_CAN_14.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_14_W46            CE_CAN_14.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_14_CS47           CE_CAN_14.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_14_ID47           CE_CAN_14.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_14_B47            CE_CAN_14.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_14_H47            CE_CAN_14.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_14_W47            CE_CAN_14.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_14_CS48           CE_CAN_14.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_14_ID48           CE_CAN_14.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_14_B48            CE_CAN_14.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_14_H48            CE_CAN_14.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_14_W48            CE_CAN_14.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_14_CS49           CE_CAN_14.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_14_ID49           CE_CAN_14.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_14_B49            CE_CAN_14.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_14_H49            CE_CAN_14.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_14_W49            CE_CAN_14.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_14_CS50           CE_CAN_14.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_14_ID50           CE_CAN_14.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_14_B50            CE_CAN_14.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_14_H50            CE_CAN_14.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_14_W50            CE_CAN_14.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_14_CS51           CE_CAN_14.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_14_ID51           CE_CAN_14.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_14_B51            CE_CAN_14.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_14_H51            CE_CAN_14.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_14_W51            CE_CAN_14.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_14_CS52           CE_CAN_14.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_14_ID52           CE_CAN_14.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_14_B52            CE_CAN_14.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_14_H52            CE_CAN_14.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_14_W52            CE_CAN_14.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_14_CS53           CE_CAN_14.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_14_ID53           CE_CAN_14.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_14_B53            CE_CAN_14.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_14_H53            CE_CAN_14.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_14_W53            CE_CAN_14.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_14_CS54           CE_CAN_14.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_14_ID54           CE_CAN_14.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_14_B54            CE_CAN_14.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_14_H54            CE_CAN_14.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_14_W54            CE_CAN_14.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_14_CS55           CE_CAN_14.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_14_ID55           CE_CAN_14.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_14_B55            CE_CAN_14.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_14_H55            CE_CAN_14.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_14_W55            CE_CAN_14.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_14_CS56           CE_CAN_14.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_14_ID56           CE_CAN_14.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_14_B56            CE_CAN_14.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_14_H56            CE_CAN_14.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_14_W56            CE_CAN_14.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_14_CS57           CE_CAN_14.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_14_ID57           CE_CAN_14.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_14_B57            CE_CAN_14.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_14_H57            CE_CAN_14.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_14_W57            CE_CAN_14.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_14_CS58           CE_CAN_14.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_14_ID58           CE_CAN_14.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_14_B58            CE_CAN_14.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_14_H58            CE_CAN_14.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_14_W58            CE_CAN_14.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_14_CS59           CE_CAN_14.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_14_ID59           CE_CAN_14.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_14_B59            CE_CAN_14.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_14_H59            CE_CAN_14.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_14_W59            CE_CAN_14.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_14_CS60           CE_CAN_14.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_14_ID60           CE_CAN_14.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_14_B60            CE_CAN_14.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_14_H60            CE_CAN_14.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_14_W60            CE_CAN_14.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_14_CS61           CE_CAN_14.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_14_ID61           CE_CAN_14.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_14_B61            CE_CAN_14.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_14_H61            CE_CAN_14.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_14_W61            CE_CAN_14.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_14_CS62           CE_CAN_14.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_14_ID62           CE_CAN_14.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_14_B62            CE_CAN_14.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_14_H62            CE_CAN_14.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_14_W62            CE_CAN_14.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_14_CS63           CE_CAN_14.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_14_ID63           CE_CAN_14.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_14_B63            CE_CAN_14.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_14_H63            CE_CAN_14.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_14_W63            CE_CAN_14.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_14_CS64           CE_CAN_14.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_14_ID64           CE_CAN_14.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_14_B64            CE_CAN_14.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_14_H64            CE_CAN_14.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_14_W64            CE_CAN_14.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_14_CS65           CE_CAN_14.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_14_ID65           CE_CAN_14.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_14_B65            CE_CAN_14.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_14_H65            CE_CAN_14.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_14_W65            CE_CAN_14.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_14_CS66           CE_CAN_14.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_14_ID66           CE_CAN_14.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_14_B66            CE_CAN_14.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_14_H66            CE_CAN_14.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_14_W66            CE_CAN_14.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_14_CS67           CE_CAN_14.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_14_ID67           CE_CAN_14.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_14_B67            CE_CAN_14.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_14_H67            CE_CAN_14.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_14_W67            CE_CAN_14.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_14_CS68           CE_CAN_14.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_14_ID68           CE_CAN_14.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_14_B68            CE_CAN_14.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_14_H68            CE_CAN_14.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_14_W68            CE_CAN_14.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_14_CS69           CE_CAN_14.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_14_ID69           CE_CAN_14.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_14_B69            CE_CAN_14.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_14_H69            CE_CAN_14.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_14_W69            CE_CAN_14.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_14_CS70           CE_CAN_14.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_14_ID70           CE_CAN_14.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_14_B70            CE_CAN_14.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_14_H70            CE_CAN_14.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_14_W70            CE_CAN_14.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_14_CS71           CE_CAN_14.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_14_ID71           CE_CAN_14.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_14_B71            CE_CAN_14.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_14_H71            CE_CAN_14.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_14_W71            CE_CAN_14.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_14_CS72           CE_CAN_14.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_14_ID72           CE_CAN_14.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_14_B72            CE_CAN_14.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_14_H72            CE_CAN_14.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_14_W72            CE_CAN_14.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_14_CS73           CE_CAN_14.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_14_ID73           CE_CAN_14.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_14_B73            CE_CAN_14.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_14_H73            CE_CAN_14.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_14_W73            CE_CAN_14.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_14_CS74           CE_CAN_14.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_14_ID74           CE_CAN_14.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_14_B74            CE_CAN_14.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_14_H74            CE_CAN_14.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_14_W74            CE_CAN_14.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_14_CS75           CE_CAN_14.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_14_ID75           CE_CAN_14.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_14_B75            CE_CAN_14.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_14_H75            CE_CAN_14.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_14_W75            CE_CAN_14.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_14_CS76           CE_CAN_14.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_14_ID76           CE_CAN_14.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_14_B76            CE_CAN_14.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_14_H76            CE_CAN_14.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_14_W76            CE_CAN_14.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_14_CS77           CE_CAN_14.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_14_ID77           CE_CAN_14.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_14_B77            CE_CAN_14.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_14_H77            CE_CAN_14.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_14_W77            CE_CAN_14.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_14_CS78           CE_CAN_14.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_14_ID78           CE_CAN_14.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_14_B78            CE_CAN_14.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_14_H78            CE_CAN_14.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_14_W78            CE_CAN_14.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_14_CS79           CE_CAN_14.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_14_ID79           CE_CAN_14.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_14_B79            CE_CAN_14.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_14_H79            CE_CAN_14.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_14_W79            CE_CAN_14.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_14_CS80           CE_CAN_14.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_14_ID80           CE_CAN_14.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_14_B80            CE_CAN_14.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_14_H80            CE_CAN_14.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_14_W80            CE_CAN_14.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_14_CS81           CE_CAN_14.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_14_ID81           CE_CAN_14.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_14_B81            CE_CAN_14.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_14_H81            CE_CAN_14.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_14_W81            CE_CAN_14.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_14_CS82           CE_CAN_14.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_14_ID82           CE_CAN_14.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_14_B82            CE_CAN_14.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_14_H82            CE_CAN_14.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_14_W82            CE_CAN_14.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_14_CS83           CE_CAN_14.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_14_ID83           CE_CAN_14.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_14_B83            CE_CAN_14.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_14_H83            CE_CAN_14.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_14_W83            CE_CAN_14.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_14_CS84           CE_CAN_14.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_14_ID84           CE_CAN_14.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_14_B84            CE_CAN_14.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_14_H84            CE_CAN_14.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_14_W84            CE_CAN_14.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_14_CS85           CE_CAN_14.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_14_ID85           CE_CAN_14.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_14_B85            CE_CAN_14.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_14_H85            CE_CAN_14.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_14_W85            CE_CAN_14.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_14_CS86           CE_CAN_14.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_14_ID86           CE_CAN_14.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_14_B86            CE_CAN_14.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_14_H86            CE_CAN_14.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_14_W86            CE_CAN_14.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_14_CS87           CE_CAN_14.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_14_ID87           CE_CAN_14.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_14_B87            CE_CAN_14.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_14_H87            CE_CAN_14.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_14_W87            CE_CAN_14.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_14_CS88           CE_CAN_14.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_14_ID88           CE_CAN_14.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_14_B88            CE_CAN_14.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_14_H88            CE_CAN_14.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_14_W848           CE_CAN_14.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_14_CS89           CE_CAN_14.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_14_ID89           CE_CAN_14.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_14_B89            CE_CAN_14.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_14_H89            CE_CAN_14.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_14_W89            CE_CAN_14.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_14_CS90           CE_CAN_14.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_14_ID90           CE_CAN_14.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_14_B90            CE_CAN_14.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_14_H90            CE_CAN_14.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_14_W90            CE_CAN_14.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_14_CS91           CE_CAN_14.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_14_ID91           CE_CAN_14.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_14_B91            CE_CAN_14.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_14_H91            CE_CAN_14.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_14_W91            CE_CAN_14.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_14_CS92           CE_CAN_14.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_14_ID92           CE_CAN_14.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_14_B92            CE_CAN_14.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_14_H92            CE_CAN_14.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_14_W92            CE_CAN_14.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_14_CS93           CE_CAN_14.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_14_ID93           CE_CAN_14.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_14_B93            CE_CAN_14.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_14_H93            CE_CAN_14.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_14_W93            CE_CAN_14.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_14_CS94           CE_CAN_14.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_14_ID94           CE_CAN_14.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_14_B94            CE_CAN_14.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_14_H94            CE_CAN_14.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_14_W94            CE_CAN_14.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_14_CS95           CE_CAN_14.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_14_ID95           CE_CAN_14.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_14_B95            CE_CAN_14.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_14_H95            CE_CAN_14.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_14_W95            CE_CAN_14.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_14_CS96           CE_CAN_14.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_14_ID96           CE_CAN_14.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_14_B96            CE_CAN_14.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_14_H96            CE_CAN_14.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_14_W96            CE_CAN_14.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_14_CS97           CE_CAN_14.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_14_ID97           CE_CAN_14.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_14_B97            CE_CAN_14.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_14_H97            CE_CAN_14.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_14_W97            CE_CAN_14.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_14_CS98           CE_CAN_14.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_14_ID98           CE_CAN_14.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_14_B98            CE_CAN_14.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_14_H98            CE_CAN_14.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_14_W98            CE_CAN_14.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_14_CS99           CE_CAN_14.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_14_ID99           CE_CAN_14.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_14_B99            CE_CAN_14.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_14_H99            CE_CAN_14.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_14_W99            CE_CAN_14.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_14_CS100          CE_CAN_14.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_14_ID100          CE_CAN_14.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_14_B100           CE_CAN_14.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_14_H100           CE_CAN_14.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_14_W100           CE_CAN_14.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_14_CS101          CE_CAN_14.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_14_ID101          CE_CAN_14.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_14_B101           CE_CAN_14.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_14_H101           CE_CAN_14.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_14_W101           CE_CAN_14.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_14_CS102          CE_CAN_14.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_14_ID102          CE_CAN_14.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_14_B102           CE_CAN_14.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_14_H102           CE_CAN_14.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_14_W102           CE_CAN_14.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_14_CS103          CE_CAN_14.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_14_ID103          CE_CAN_14.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_14_B103           CE_CAN_14.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_14_H103           CE_CAN_14.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_14_W103           CE_CAN_14.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_14_CS104          CE_CAN_14.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_14_ID104          CE_CAN_14.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_14_B104           CE_CAN_14.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_14_H104           CE_CAN_14.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_14_W104           CE_CAN_14.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_14_CS105          CE_CAN_14.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_14_ID105          CE_CAN_14.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_14_B105           CE_CAN_14.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_14_H105           CE_CAN_14.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_14_W105           CE_CAN_14.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_14_CS106          CE_CAN_14.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_14_ID106          CE_CAN_14.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_14_B106           CE_CAN_14.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_14_H106           CE_CAN_14.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_14_W106           CE_CAN_14.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_14_CS107          CE_CAN_14.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_14_ID107          CE_CAN_14.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_14_B107           CE_CAN_14.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_14_H107           CE_CAN_14.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_14_W107           CE_CAN_14.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_14_CS108          CE_CAN_14.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_14_ID108          CE_CAN_14.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_14_B108           CE_CAN_14.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_14_H108           CE_CAN_14.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_14_W108           CE_CAN_14.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_14_CS109          CE_CAN_14.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_14_ID109          CE_CAN_14.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_14_B109           CE_CAN_14.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_14_H109           CE_CAN_14.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_14_W109           CE_CAN_14.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_14_CS110          CE_CAN_14.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_14_ID110          CE_CAN_14.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_14_B110           CE_CAN_14.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_14_H110           CE_CAN_14.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_14_W110           CE_CAN_14.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_14_CS111          CE_CAN_14.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_14_ID111          CE_CAN_14.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_14_B111           CE_CAN_14.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_14_H111           CE_CAN_14.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_14_W111           CE_CAN_14.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_14_CS112          CE_CAN_14.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_14_ID112          CE_CAN_14.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_14_B112           CE_CAN_14.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_14_H112           CE_CAN_14.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_14_W112           CE_CAN_14.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_14_CS113          CE_CAN_14.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_14_ID113          CE_CAN_14.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_14_B113           CE_CAN_14.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_14_H113           CE_CAN_14.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_14_W113           CE_CAN_14.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_14_CS114          CE_CAN_14.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_14_ID114          CE_CAN_14.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_14_B114           CE_CAN_14.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_14_H114           CE_CAN_14.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_14_W114           CE_CAN_14.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_14_CS115          CE_CAN_14.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_14_ID115          CE_CAN_14.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_14_B115           CE_CAN_14.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_14_H115           CE_CAN_14.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_14_W115           CE_CAN_14.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_14_CS116          CE_CAN_14.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_14_ID116          CE_CAN_14.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_14_B116           CE_CAN_14.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_14_H116           CE_CAN_14.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_14_W116           CE_CAN_14.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_14_CS117          CE_CAN_14.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_14_ID117          CE_CAN_14.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_14_B117           CE_CAN_14.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_14_H117           CE_CAN_14.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_14_W117           CE_CAN_14.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_14_CS118          CE_CAN_14.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_14_ID118          CE_CAN_14.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_14_B118           CE_CAN_14.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_14_H118           CE_CAN_14.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_14_W118           CE_CAN_14.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_14_CS119          CE_CAN_14.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_14_ID119          CE_CAN_14.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_14_B119           CE_CAN_14.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_14_H119           CE_CAN_14.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_14_W119           CE_CAN_14.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_14_CS120          CE_CAN_14.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_14_ID120          CE_CAN_14.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_14_B120           CE_CAN_14.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_14_H120           CE_CAN_14.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_14_W120           CE_CAN_14.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_14_CS121          CE_CAN_14.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_14_ID121          CE_CAN_14.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_14_B121           CE_CAN_14.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_14_H121           CE_CAN_14.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_14_W121           CE_CAN_14.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_14_CS122          CE_CAN_14.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_14_ID122          CE_CAN_14.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_14_B122           CE_CAN_14.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_14_H122           CE_CAN_14.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_14_W122           CE_CAN_14.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_14_CS123          CE_CAN_14.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_14_ID123          CE_CAN_14.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_14_B123           CE_CAN_14.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_14_H123           CE_CAN_14.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_14_W123           CE_CAN_14.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_14_CS124          CE_CAN_14.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_14_ID124          CE_CAN_14.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_14_B124           CE_CAN_14.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_14_H124           CE_CAN_14.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_14_W124           CE_CAN_14.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_14_CS125          CE_CAN_14.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_14_ID125          CE_CAN_14.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_14_B125           CE_CAN_14.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_14_H125           CE_CAN_14.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_14_W125           CE_CAN_14.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_14_CS126          CE_CAN_14.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_14_ID126          CE_CAN_14.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_14_B126           CE_CAN_14.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_14_H126           CE_CAN_14.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_14_W126           CE_CAN_14.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_14_CS127          CE_CAN_14.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_14_ID127          CE_CAN_14.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_14_B127           CE_CAN_14.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_14_H127           CE_CAN_14.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_14_W127           CE_CAN_14.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_14_RXIMR0     CE_CAN_14.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR1     CE_CAN_14.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR2     CE_CAN_14.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR3     CE_CAN_14.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR4     CE_CAN_14.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR5     CE_CAN_14.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR6     CE_CAN_14.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR7     CE_CAN_14.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR8     CE_CAN_14.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR9     CE_CAN_14.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_14_RXIMR10    CE_CAN_14.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR11    CE_CAN_14.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR12    CE_CAN_14.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR13    CE_CAN_14.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR14    CE_CAN_14.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR15    CE_CAN_14.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR16    CE_CAN_14.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR17    CE_CAN_14.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR18    CE_CAN_14.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR19    CE_CAN_14.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR20    CE_CAN_14.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR21    CE_CAN_14.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR22    CE_CAN_14.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR23    CE_CAN_14.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR24    CE_CAN_14.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR25    CE_CAN_14.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR26    CE_CAN_14.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR27    CE_CAN_14.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR28    CE_CAN_14.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR29    CE_CAN_14.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR30    CE_CAN_14.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR31    CE_CAN_14.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR32    CE_CAN_14.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR33    CE_CAN_14.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR34    CE_CAN_14.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR35    CE_CAN_14.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR36    CE_CAN_14.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR37    CE_CAN_14.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR38    CE_CAN_14.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR39    CE_CAN_14.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR40    CE_CAN_14.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR41    CE_CAN_14.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR42    CE_CAN_14.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR43    CE_CAN_14.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR44    CE_CAN_14.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR45    CE_CAN_14.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR46    CE_CAN_14.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR47    CE_CAN_14.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR48    CE_CAN_14.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR49    CE_CAN_14.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR50    CE_CAN_14.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR51    CE_CAN_14.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR52    CE_CAN_14.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR53    CE_CAN_14.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR54    CE_CAN_14.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR55    CE_CAN_14.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR56    CE_CAN_14.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR57    CE_CAN_14.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR58    CE_CAN_14.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR59    CE_CAN_14.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR60    CE_CAN_14.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR61    CE_CAN_14.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR62    CE_CAN_14.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR63    CE_CAN_14.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR64    CE_CAN_14.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR65    CE_CAN_14.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR66    CE_CAN_14.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR67    CE_CAN_14.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR68    CE_CAN_14.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR69    CE_CAN_14.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR70    CE_CAN_14.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR71    CE_CAN_14.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR72    CE_CAN_14.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR73    CE_CAN_14.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR74    CE_CAN_14.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR75    CE_CAN_14.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR76    CE_CAN_14.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR77    CE_CAN_14.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR78    CE_CAN_14.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR79    CE_CAN_14.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR80    CE_CAN_14.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR81    CE_CAN_14.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR82    CE_CAN_14.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR83    CE_CAN_14.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR84    CE_CAN_14.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR85    CE_CAN_14.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR86    CE_CAN_14.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR87    CE_CAN_14.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR88    CE_CAN_14.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR89    CE_CAN_14.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR90    CE_CAN_14.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR91    CE_CAN_14.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR92    CE_CAN_14.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR93    CE_CAN_14.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR94    CE_CAN_14.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR95    CE_CAN_14.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR96    CE_CAN_14.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR97    CE_CAN_14.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR98    CE_CAN_14.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR99    CE_CAN_14.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_14_RXIMR100   CE_CAN_14.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR101   CE_CAN_14.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR102   CE_CAN_14.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR103   CE_CAN_14.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR104   CE_CAN_14.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR105   CE_CAN_14.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR106   CE_CAN_14.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR107   CE_CAN_14.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR108   CE_CAN_14.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR109   CE_CAN_14.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR110   CE_CAN_14.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR111   CE_CAN_14.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR112   CE_CAN_14.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR113   CE_CAN_14.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR114   CE_CAN_14.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR115   CE_CAN_14.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR116   CE_CAN_14.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR117   CE_CAN_14.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR118   CE_CAN_14.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR119   CE_CAN_14.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR120   CE_CAN_14.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR121   CE_CAN_14.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR122   CE_CAN_14.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR123   CE_CAN_14.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR124   CE_CAN_14.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR125   CE_CAN_14.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR126   CE_CAN_14.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_14_RXIMR127   CE_CAN_14.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_14_MECR       CE_CAN_14.MECR.R              /* Memory Error Control */
#define CE_CAN_14_ERRIAR     CE_CAN_14.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_14_ERRIDPR    CE_CAN_14.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_14_ERRIPPR    CE_CAN_14.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_14_RERRAR     CE_CAN_14.RERRAR.R            /* Error Report Address */
#define CE_CAN_14_RERRDR     CE_CAN_14.RERRDR.R            /* Error Report Data */
#define CE_CAN_14_RERRSYNR   CE_CAN_14.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_14_ERRSR      CE_CAN_14.ERRSR.R             /* Error Status */
#define CE_CAN_14_EPRS       CE_CAN_14.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_14_ENCBT      CE_CAN_14.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_14_EDCBT      CE_CAN_14.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_14_ETDC       CE_CAN_14.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_14_FDCTRL     CE_CAN_14.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_14_FDCBT      CE_CAN_14.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_14_FDCRC      CE_CAN_14.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_14_ERFCR      CE_CAN_14.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_14_ERFIER     CE_CAN_14.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_14_ERFSR      CE_CAN_14.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_14_HR_TIME_STAMP0 CE_CAN_14.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP1 CE_CAN_14.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP2 CE_CAN_14.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP3 CE_CAN_14.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP4 CE_CAN_14.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP5 CE_CAN_14.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP6 CE_CAN_14.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP7 CE_CAN_14.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP8 CE_CAN_14.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP9 CE_CAN_14.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP10 CE_CAN_14.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP11 CE_CAN_14.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP12 CE_CAN_14.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP13 CE_CAN_14.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP14 CE_CAN_14.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP15 CE_CAN_14.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP16 CE_CAN_14.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP17 CE_CAN_14.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP18 CE_CAN_14.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP19 CE_CAN_14.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP20 CE_CAN_14.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP21 CE_CAN_14.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP22 CE_CAN_14.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP23 CE_CAN_14.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP24 CE_CAN_14.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP25 CE_CAN_14.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP26 CE_CAN_14.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP27 CE_CAN_14.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP28 CE_CAN_14.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP29 CE_CAN_14.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP30 CE_CAN_14.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP31 CE_CAN_14.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP32 CE_CAN_14.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP33 CE_CAN_14.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP34 CE_CAN_14.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP35 CE_CAN_14.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP36 CE_CAN_14.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP37 CE_CAN_14.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP38 CE_CAN_14.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP39 CE_CAN_14.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP40 CE_CAN_14.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP41 CE_CAN_14.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP42 CE_CAN_14.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP43 CE_CAN_14.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP44 CE_CAN_14.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP45 CE_CAN_14.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP46 CE_CAN_14.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP47 CE_CAN_14.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP48 CE_CAN_14.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP49 CE_CAN_14.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP50 CE_CAN_14.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP51 CE_CAN_14.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP52 CE_CAN_14.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP53 CE_CAN_14.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP54 CE_CAN_14.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP55 CE_CAN_14.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP56 CE_CAN_14.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP57 CE_CAN_14.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP58 CE_CAN_14.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP59 CE_CAN_14.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP60 CE_CAN_14.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP61 CE_CAN_14.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP62 CE_CAN_14.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP63 CE_CAN_14.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP64 CE_CAN_14.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP65 CE_CAN_14.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP66 CE_CAN_14.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP67 CE_CAN_14.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP68 CE_CAN_14.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP69 CE_CAN_14.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP70 CE_CAN_14.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP71 CE_CAN_14.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP72 CE_CAN_14.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP73 CE_CAN_14.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP74 CE_CAN_14.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP75 CE_CAN_14.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP76 CE_CAN_14.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP77 CE_CAN_14.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP78 CE_CAN_14.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP79 CE_CAN_14.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP80 CE_CAN_14.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP81 CE_CAN_14.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP82 CE_CAN_14.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP83 CE_CAN_14.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP84 CE_CAN_14.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP85 CE_CAN_14.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP86 CE_CAN_14.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP87 CE_CAN_14.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP88 CE_CAN_14.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP89 CE_CAN_14.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP90 CE_CAN_14.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP91 CE_CAN_14.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP92 CE_CAN_14.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP93 CE_CAN_14.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP94 CE_CAN_14.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP95 CE_CAN_14.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP96 CE_CAN_14.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP97 CE_CAN_14.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP98 CE_CAN_14.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP99 CE_CAN_14.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP100 CE_CAN_14.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP101 CE_CAN_14.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP102 CE_CAN_14.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP103 CE_CAN_14.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP104 CE_CAN_14.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP105 CE_CAN_14.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP106 CE_CAN_14.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP107 CE_CAN_14.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP108 CE_CAN_14.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP109 CE_CAN_14.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP110 CE_CAN_14.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP111 CE_CAN_14.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP112 CE_CAN_14.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP113 CE_CAN_14.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP114 CE_CAN_14.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP115 CE_CAN_14.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP116 CE_CAN_14.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP117 CE_CAN_14.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP118 CE_CAN_14.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP119 CE_CAN_14.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP120 CE_CAN_14.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP121 CE_CAN_14.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP122 CE_CAN_14.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP123 CE_CAN_14.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP124 CE_CAN_14.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP125 CE_CAN_14.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP126 CE_CAN_14.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_14_HR_TIME_STAMP127 CE_CAN_14.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_14_ERFFEL0    CE_CAN_14.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL1    CE_CAN_14.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL2    CE_CAN_14.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL3    CE_CAN_14.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL4    CE_CAN_14.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL5    CE_CAN_14.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL6    CE_CAN_14.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL7    CE_CAN_14.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL8    CE_CAN_14.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL9    CE_CAN_14.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL10   CE_CAN_14.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL11   CE_CAN_14.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL12   CE_CAN_14.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL13   CE_CAN_14.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL14   CE_CAN_14.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL15   CE_CAN_14.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL16   CE_CAN_14.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL17   CE_CAN_14.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL18   CE_CAN_14.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL19   CE_CAN_14.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL20   CE_CAN_14.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL21   CE_CAN_14.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL22   CE_CAN_14.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL23   CE_CAN_14.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL24   CE_CAN_14.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL25   CE_CAN_14.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL26   CE_CAN_14.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL27   CE_CAN_14.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL28   CE_CAN_14.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL29   CE_CAN_14.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL30   CE_CAN_14.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL31   CE_CAN_14.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL32   CE_CAN_14.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL33   CE_CAN_14.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL34   CE_CAN_14.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL35   CE_CAN_14.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL36   CE_CAN_14.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL37   CE_CAN_14.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL38   CE_CAN_14.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL39   CE_CAN_14.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL40   CE_CAN_14.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL41   CE_CAN_14.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL42   CE_CAN_14.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL43   CE_CAN_14.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL44   CE_CAN_14.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL45   CE_CAN_14.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL46   CE_CAN_14.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL47   CE_CAN_14.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL48   CE_CAN_14.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL49   CE_CAN_14.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL50   CE_CAN_14.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL51   CE_CAN_14.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL52   CE_CAN_14.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL53   CE_CAN_14.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL54   CE_CAN_14.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL55   CE_CAN_14.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL56   CE_CAN_14.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL57   CE_CAN_14.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL58   CE_CAN_14.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL59   CE_CAN_14.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL60   CE_CAN_14.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL61   CE_CAN_14.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL62   CE_CAN_14.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL63   CE_CAN_14.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL64   CE_CAN_14.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL65   CE_CAN_14.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL66   CE_CAN_14.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL67   CE_CAN_14.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL68   CE_CAN_14.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL69   CE_CAN_14.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL70   CE_CAN_14.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL71   CE_CAN_14.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL72   CE_CAN_14.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL73   CE_CAN_14.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL74   CE_CAN_14.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL75   CE_CAN_14.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL76   CE_CAN_14.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL77   CE_CAN_14.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL78   CE_CAN_14.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL79   CE_CAN_14.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL80   CE_CAN_14.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL81   CE_CAN_14.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL82   CE_CAN_14.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL83   CE_CAN_14.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL84   CE_CAN_14.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL85   CE_CAN_14.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL86   CE_CAN_14.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL87   CE_CAN_14.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL88   CE_CAN_14.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL89   CE_CAN_14.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL90   CE_CAN_14.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL91   CE_CAN_14.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL92   CE_CAN_14.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL93   CE_CAN_14.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL94   CE_CAN_14.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL95   CE_CAN_14.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL96   CE_CAN_14.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL97   CE_CAN_14.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL98   CE_CAN_14.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL99   CE_CAN_14.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL100  CE_CAN_14.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL101  CE_CAN_14.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL102  CE_CAN_14.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL103  CE_CAN_14.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL104  CE_CAN_14.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL105  CE_CAN_14.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL106  CE_CAN_14.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL107  CE_CAN_14.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL108  CE_CAN_14.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL109  CE_CAN_14.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL110  CE_CAN_14.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL111  CE_CAN_14.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL112  CE_CAN_14.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL113  CE_CAN_14.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL114  CE_CAN_14.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL115  CE_CAN_14.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL116  CE_CAN_14.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL117  CE_CAN_14.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL118  CE_CAN_14.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL119  CE_CAN_14.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL120  CE_CAN_14.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL121  CE_CAN_14.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL122  CE_CAN_14.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL123  CE_CAN_14.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL124  CE_CAN_14.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL125  CE_CAN_14.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL126  CE_CAN_14.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_14_ERFFEL127  CE_CAN_14.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_15_MCR        CE_CAN_15.MCR.R               /* Module Configuration */
#define CE_CAN_15_CTRL1      CE_CAN_15.CTRL1.R             /* Control 1 */
#define CE_CAN_15_TIMER      CE_CAN_15.TIMER.R             /* Free-Running Timer */
#define CE_CAN_15_RXMGMASK   CE_CAN_15.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_15_RX14MASK   CE_CAN_15.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_15_RX15MASK   CE_CAN_15.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_15_ECR        CE_CAN_15.ECR.R               /* Error Counter */
#define CE_CAN_15_ESR1       CE_CAN_15.ESR1.R              /* Error and Status 1 */
#define CE_CAN_15_IMASK2     CE_CAN_15.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_15_IMASK1     CE_CAN_15.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_15_IFLAG2     CE_CAN_15.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_15_IFLAG1     CE_CAN_15.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_15_CTRL2      CE_CAN_15.CTRL2.R             /* Control 2 */
#define CE_CAN_15_ESR2       CE_CAN_15.ESR2.R              /* Error and Status 2 */
#define CE_CAN_15_CRCR       CE_CAN_15.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_15_RXFGMASK   CE_CAN_15.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_15_RXFIR      CE_CAN_15.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_15_CBT        CE_CAN_15.CBT.R               /* CAN Bit Timing */
#define CE_CAN_15_IMASK4     CE_CAN_15.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_15_IMASK3     CE_CAN_15.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_15_IFLAG4     CE_CAN_15.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_15_IFLAG3     CE_CAN_15.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_15_CS0            CE_CAN_15.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_15_ID0            CE_CAN_15.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_15_B0             CE_CAN_15.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_15_H0             CE_CAN_15.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_15_W0             CE_CAN_15.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_15_CS1            CE_CAN_15.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_15_ID1            CE_CAN_15.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_15_B1             CE_CAN_15.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_15_H1             CE_CAN_15.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_15_W1             CE_CAN_15.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_15_CS2            CE_CAN_15.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_15_ID2            CE_CAN_15.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_15_B2             CE_CAN_15.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_15_H2             CE_CAN_15.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_15_W2             CE_CAN_15.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_15_CS3            CE_CAN_15.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_15_ID3            CE_CAN_15.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_15_B3             CE_CAN_15.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_15_H3             CE_CAN_15.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_15_W3             CE_CAN_15.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_15_CS4            CE_CAN_15.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_15_ID4            CE_CAN_15.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_15_B4             CE_CAN_15.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_15_H4             CE_CAN_15.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_15_W4             CE_CAN_15.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_15_CS5            CE_CAN_15.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_15_ID5            CE_CAN_15.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_15_B5             CE_CAN_15.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_15_H5             CE_CAN_15.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_15_W5             CE_CAN_15.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_15_CS6            CE_CAN_15.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_15_ID6            CE_CAN_15.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_15_B6             CE_CAN_15.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_15_H6             CE_CAN_15.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_15_W6             CE_CAN_15.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_15_CS7            CE_CAN_15.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_15_ID7            CE_CAN_15.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_15_B7             CE_CAN_15.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_15_H7             CE_CAN_15.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_15_W7             CE_CAN_15.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_15_CS8            CE_CAN_15.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_15_ID8            CE_CAN_15.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_15_B8             CE_CAN_15.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_15_H8             CE_CAN_15.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_15_W8             CE_CAN_15.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_15_CS9            CE_CAN_15.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_15_ID9            CE_CAN_15.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_15_B9             CE_CAN_15.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_15_H9             CE_CAN_15.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_15_W9             CE_CAN_15.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_15_CS10           CE_CAN_15.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_15_ID10           CE_CAN_15.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_15_B10            CE_CAN_15.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_15_H10            CE_CAN_15.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_15_W10            CE_CAN_15.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_15_CS11           CE_CAN_15.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_15_ID11           CE_CAN_15.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_15_B11            CE_CAN_15.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_15_H11            CE_CAN_15.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_15_W11            CE_CAN_15.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_15_CS12           CE_CAN_15.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_15_ID12           CE_CAN_15.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_15_B12            CE_CAN_15.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_15_H12            CE_CAN_15.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_15_W12            CE_CAN_15.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_15_CS13           CE_CAN_15.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_15_ID13           CE_CAN_15.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_15_B13            CE_CAN_15.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_15_H13            CE_CAN_15.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_15_W13            CE_CAN_15.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_15_CS14           CE_CAN_15.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_15_ID14           CE_CAN_15.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_15_B14            CE_CAN_15.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_15_H14            CE_CAN_15.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_15_W14            CE_CAN_15.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_15_CS15           CE_CAN_15.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_15_ID15           CE_CAN_15.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_15_B15            CE_CAN_15.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_15_H15            CE_CAN_15.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_15_W15            CE_CAN_15.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_15_CS16           CE_CAN_15.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_15_ID16           CE_CAN_15.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_15_B16            CE_CAN_15.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_15_H16            CE_CAN_15.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_15_W16            CE_CAN_15.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_15_CS17           CE_CAN_15.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_15_ID17           CE_CAN_15.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_15_B17            CE_CAN_15.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_15_H17            CE_CAN_15.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_15_W17            CE_CAN_15.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_15_CS18           CE_CAN_15.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_15_ID18           CE_CAN_15.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_15_B18            CE_CAN_15.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_15_H18            CE_CAN_15.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_15_W18            CE_CAN_15.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_15_CS19           CE_CAN_15.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_15_ID19           CE_CAN_15.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_15_B19            CE_CAN_15.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_15_H19            CE_CAN_15.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_15_W19            CE_CAN_15.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_15_CS20           CE_CAN_15.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_15_ID20           CE_CAN_15.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_15_B20            CE_CAN_15.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_15_H20            CE_CAN_15.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_15_W20            CE_CAN_15.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_15_CS21           CE_CAN_15.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_15_ID21           CE_CAN_15.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_15_B21            CE_CAN_15.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_15_H21            CE_CAN_15.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_15_W21            CE_CAN_15.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_15_CS22           CE_CAN_15.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_15_ID22           CE_CAN_15.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_15_B22            CE_CAN_15.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_15_H22            CE_CAN_15.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_15_W22            CE_CAN_15.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_15_CS23           CE_CAN_15.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_15_ID23           CE_CAN_15.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_15_B23            CE_CAN_15.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_15_H23            CE_CAN_15.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_15_W23            CE_CAN_15.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_15_CS24           CE_CAN_15.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_15_ID24           CE_CAN_15.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_15_B24            CE_CAN_15.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_15_H24            CE_CAN_15.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_15_W24            CE_CAN_15.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_15_CS25           CE_CAN_15.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_15_ID25           CE_CAN_15.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_15_B25            CE_CAN_15.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_15_H25            CE_CAN_15.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_15_W25            CE_CAN_15.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_15_CS26           CE_CAN_15.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_15_ID26           CE_CAN_15.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_15_B26            CE_CAN_15.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_15_H26            CE_CAN_15.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_15_W26            CE_CAN_15.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_15_CS27           CE_CAN_15.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_15_ID27           CE_CAN_15.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_15_B27            CE_CAN_15.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_15_H27            CE_CAN_15.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_15_W27            CE_CAN_15.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_15_CS28           CE_CAN_15.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_15_ID28           CE_CAN_15.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_15_B28            CE_CAN_15.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_15_H28            CE_CAN_15.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_15_W28            CE_CAN_15.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_15_CS29           CE_CAN_15.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_15_ID29           CE_CAN_15.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_15_B29            CE_CAN_15.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_15_H29            CE_CAN_15.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_15_W29            CE_CAN_15.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_15_CS30           CE_CAN_15.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_15_ID30           CE_CAN_15.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_15_B30            CE_CAN_15.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_15_H30            CE_CAN_15.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_15_W30            CE_CAN_15.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_15_CS31           CE_CAN_15.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_15_ID31           CE_CAN_15.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_15_B31            CE_CAN_15.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_15_H31            CE_CAN_15.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_15_W31            CE_CAN_15.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_15_CS32           CE_CAN_15.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_15_ID32           CE_CAN_15.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_15_B32            CE_CAN_15.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_15_H32            CE_CAN_15.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_15_W32            CE_CAN_15.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_15_CS33           CE_CAN_15.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_15_ID33           CE_CAN_15.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_15_B33            CE_CAN_15.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_15_H33            CE_CAN_15.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_15_W33            CE_CAN_15.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_15_CS34           CE_CAN_15.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_15_ID34           CE_CAN_15.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_15_B34            CE_CAN_15.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_15_H34            CE_CAN_15.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_15_W34            CE_CAN_15.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_15_CS35           CE_CAN_15.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_15_ID35           CE_CAN_15.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_15_B35            CE_CAN_15.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_15_H35            CE_CAN_15.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_15_W35            CE_CAN_15.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_15_CS36           CE_CAN_15.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_15_ID36           CE_CAN_15.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_15_B36            CE_CAN_15.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_15_H36            CE_CAN_15.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_15_W36            CE_CAN_15.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_15_CS37           CE_CAN_15.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_15_ID37           CE_CAN_15.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_15_B37            CE_CAN_15.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_15_H37            CE_CAN_15.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_15_W37            CE_CAN_15.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_15_CS38           CE_CAN_15.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_15_ID38           CE_CAN_15.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_15_B38            CE_CAN_15.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_15_H38            CE_CAN_15.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_15_W38            CE_CAN_15.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_15_CS39           CE_CAN_15.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_15_ID39           CE_CAN_15.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_15_B39            CE_CAN_15.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_15_H39            CE_CAN_15.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_15_W39            CE_CAN_15.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_15_CS40           CE_CAN_15.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_15_ID40           CE_CAN_15.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_15_B40            CE_CAN_15.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_15_H40            CE_CAN_15.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_15_W40            CE_CAN_15.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_15_CS41           CE_CAN_15.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_15_ID41           CE_CAN_15.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_15_B41            CE_CAN_15.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_15_H41            CE_CAN_15.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_15_W41            CE_CAN_15.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_15_CS42           CE_CAN_15.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_15_ID42           CE_CAN_15.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_15_B42            CE_CAN_15.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_15_H42            CE_CAN_15.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_15_W42            CE_CAN_15.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_15_CS43           CE_CAN_15.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_15_ID43           CE_CAN_15.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_15_B43            CE_CAN_15.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_15_H43            CE_CAN_15.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_15_W43            CE_CAN_15.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_15_CS44           CE_CAN_15.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_15_ID44           CE_CAN_15.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_15_B44            CE_CAN_15.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_15_H44            CE_CAN_15.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_15_W44            CE_CAN_15.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_15_CS45           CE_CAN_15.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_15_ID45           CE_CAN_15.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_15_B45            CE_CAN_15.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_15_H45            CE_CAN_15.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_15_W45            CE_CAN_15.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_15_CS46           CE_CAN_15.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_15_ID46           CE_CAN_15.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_15_B46            CE_CAN_15.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_15_H46            CE_CAN_15.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_15_W46            CE_CAN_15.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_15_CS47           CE_CAN_15.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_15_ID47           CE_CAN_15.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_15_B47            CE_CAN_15.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_15_H47            CE_CAN_15.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_15_W47            CE_CAN_15.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_15_CS48           CE_CAN_15.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_15_ID48           CE_CAN_15.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_15_B48            CE_CAN_15.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_15_H48            CE_CAN_15.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_15_W48            CE_CAN_15.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_15_CS49           CE_CAN_15.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_15_ID49           CE_CAN_15.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_15_B49            CE_CAN_15.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_15_H49            CE_CAN_15.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_15_W49            CE_CAN_15.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_15_CS50           CE_CAN_15.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_15_ID50           CE_CAN_15.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_15_B50            CE_CAN_15.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_15_H50            CE_CAN_15.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_15_W50            CE_CAN_15.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_15_CS51           CE_CAN_15.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_15_ID51           CE_CAN_15.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_15_B51            CE_CAN_15.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_15_H51            CE_CAN_15.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_15_W51            CE_CAN_15.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_15_CS52           CE_CAN_15.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_15_ID52           CE_CAN_15.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_15_B52            CE_CAN_15.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_15_H52            CE_CAN_15.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_15_W52            CE_CAN_15.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_15_CS53           CE_CAN_15.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_15_ID53           CE_CAN_15.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_15_B53            CE_CAN_15.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_15_H53            CE_CAN_15.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_15_W53            CE_CAN_15.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_15_CS54           CE_CAN_15.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_15_ID54           CE_CAN_15.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_15_B54            CE_CAN_15.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_15_H54            CE_CAN_15.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_15_W54            CE_CAN_15.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_15_CS55           CE_CAN_15.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_15_ID55           CE_CAN_15.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_15_B55            CE_CAN_15.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_15_H55            CE_CAN_15.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_15_W55            CE_CAN_15.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_15_CS56           CE_CAN_15.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_15_ID56           CE_CAN_15.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_15_B56            CE_CAN_15.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_15_H56            CE_CAN_15.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_15_W56            CE_CAN_15.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_15_CS57           CE_CAN_15.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_15_ID57           CE_CAN_15.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_15_B57            CE_CAN_15.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_15_H57            CE_CAN_15.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_15_W57            CE_CAN_15.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_15_CS58           CE_CAN_15.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_15_ID58           CE_CAN_15.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_15_B58            CE_CAN_15.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_15_H58            CE_CAN_15.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_15_W58            CE_CAN_15.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_15_CS59           CE_CAN_15.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_15_ID59           CE_CAN_15.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_15_B59            CE_CAN_15.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_15_H59            CE_CAN_15.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_15_W59            CE_CAN_15.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_15_CS60           CE_CAN_15.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_15_ID60           CE_CAN_15.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_15_B60            CE_CAN_15.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_15_H60            CE_CAN_15.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_15_W60            CE_CAN_15.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_15_CS61           CE_CAN_15.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_15_ID61           CE_CAN_15.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_15_B61            CE_CAN_15.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_15_H61            CE_CAN_15.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_15_W61            CE_CAN_15.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_15_CS62           CE_CAN_15.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_15_ID62           CE_CAN_15.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_15_B62            CE_CAN_15.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_15_H62            CE_CAN_15.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_15_W62            CE_CAN_15.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_15_CS63           CE_CAN_15.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_15_ID63           CE_CAN_15.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_15_B63            CE_CAN_15.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_15_H63            CE_CAN_15.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_15_W63            CE_CAN_15.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_15_CS64           CE_CAN_15.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_15_ID64           CE_CAN_15.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_15_B64            CE_CAN_15.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_15_H64            CE_CAN_15.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_15_W64            CE_CAN_15.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_15_CS65           CE_CAN_15.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_15_ID65           CE_CAN_15.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_15_B65            CE_CAN_15.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_15_H65            CE_CAN_15.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_15_W65            CE_CAN_15.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_15_CS66           CE_CAN_15.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_15_ID66           CE_CAN_15.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_15_B66            CE_CAN_15.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_15_H66            CE_CAN_15.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_15_W66            CE_CAN_15.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_15_CS67           CE_CAN_15.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_15_ID67           CE_CAN_15.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_15_B67            CE_CAN_15.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_15_H67            CE_CAN_15.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_15_W67            CE_CAN_15.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_15_CS68           CE_CAN_15.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_15_ID68           CE_CAN_15.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_15_B68            CE_CAN_15.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_15_H68            CE_CAN_15.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_15_W68            CE_CAN_15.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_15_CS69           CE_CAN_15.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_15_ID69           CE_CAN_15.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_15_B69            CE_CAN_15.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_15_H69            CE_CAN_15.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_15_W69            CE_CAN_15.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_15_CS70           CE_CAN_15.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_15_ID70           CE_CAN_15.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_15_B70            CE_CAN_15.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_15_H70            CE_CAN_15.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_15_W70            CE_CAN_15.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_15_CS71           CE_CAN_15.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_15_ID71           CE_CAN_15.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_15_B71            CE_CAN_15.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_15_H71            CE_CAN_15.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_15_W71            CE_CAN_15.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_15_CS72           CE_CAN_15.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_15_ID72           CE_CAN_15.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_15_B72            CE_CAN_15.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_15_H72            CE_CAN_15.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_15_W72            CE_CAN_15.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_15_CS73           CE_CAN_15.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_15_ID73           CE_CAN_15.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_15_B73            CE_CAN_15.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_15_H73            CE_CAN_15.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_15_W73            CE_CAN_15.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_15_CS74           CE_CAN_15.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_15_ID74           CE_CAN_15.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_15_B74            CE_CAN_15.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_15_H74            CE_CAN_15.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_15_W74            CE_CAN_15.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_15_CS75           CE_CAN_15.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_15_ID75           CE_CAN_15.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_15_B75            CE_CAN_15.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_15_H75            CE_CAN_15.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_15_W75            CE_CAN_15.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_15_CS76           CE_CAN_15.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_15_ID76           CE_CAN_15.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_15_B76            CE_CAN_15.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_15_H76            CE_CAN_15.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_15_W76            CE_CAN_15.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_15_CS77           CE_CAN_15.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_15_ID77           CE_CAN_15.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_15_B77            CE_CAN_15.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_15_H77            CE_CAN_15.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_15_W77            CE_CAN_15.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_15_CS78           CE_CAN_15.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_15_ID78           CE_CAN_15.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_15_B78            CE_CAN_15.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_15_H78            CE_CAN_15.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_15_W78            CE_CAN_15.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_15_CS79           CE_CAN_15.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_15_ID79           CE_CAN_15.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_15_B79            CE_CAN_15.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_15_H79            CE_CAN_15.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_15_W79            CE_CAN_15.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_15_CS80           CE_CAN_15.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_15_ID80           CE_CAN_15.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_15_B80            CE_CAN_15.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_15_H80            CE_CAN_15.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_15_W80            CE_CAN_15.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_15_CS81           CE_CAN_15.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_15_ID81           CE_CAN_15.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_15_B81            CE_CAN_15.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_15_H81            CE_CAN_15.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_15_W81            CE_CAN_15.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_15_CS82           CE_CAN_15.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_15_ID82           CE_CAN_15.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_15_B82            CE_CAN_15.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_15_H82            CE_CAN_15.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_15_W82            CE_CAN_15.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_15_CS83           CE_CAN_15.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_15_ID83           CE_CAN_15.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_15_B83            CE_CAN_15.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_15_H83            CE_CAN_15.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_15_W83            CE_CAN_15.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_15_CS84           CE_CAN_15.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_15_ID84           CE_CAN_15.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_15_B84            CE_CAN_15.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_15_H84            CE_CAN_15.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_15_W84            CE_CAN_15.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_15_CS85           CE_CAN_15.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_15_ID85           CE_CAN_15.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_15_B85            CE_CAN_15.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_15_H85            CE_CAN_15.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_15_W85            CE_CAN_15.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_15_CS86           CE_CAN_15.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_15_ID86           CE_CAN_15.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_15_B86            CE_CAN_15.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_15_H86            CE_CAN_15.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_15_W86            CE_CAN_15.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_15_CS87           CE_CAN_15.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_15_ID87           CE_CAN_15.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_15_B87            CE_CAN_15.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_15_H87            CE_CAN_15.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_15_W87            CE_CAN_15.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_15_CS88           CE_CAN_15.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_15_ID88           CE_CAN_15.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_15_B88            CE_CAN_15.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_15_H88            CE_CAN_15.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_15_W848           CE_CAN_15.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_15_CS89           CE_CAN_15.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_15_ID89           CE_CAN_15.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_15_B89            CE_CAN_15.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_15_H89            CE_CAN_15.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_15_W89            CE_CAN_15.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_15_CS90           CE_CAN_15.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_15_ID90           CE_CAN_15.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_15_B90            CE_CAN_15.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_15_H90            CE_CAN_15.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_15_W90            CE_CAN_15.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_15_CS91           CE_CAN_15.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_15_ID91           CE_CAN_15.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_15_B91            CE_CAN_15.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_15_H91            CE_CAN_15.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_15_W91            CE_CAN_15.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_15_CS92           CE_CAN_15.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_15_ID92           CE_CAN_15.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_15_B92            CE_CAN_15.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_15_H92            CE_CAN_15.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_15_W92            CE_CAN_15.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_15_CS93           CE_CAN_15.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_15_ID93           CE_CAN_15.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_15_B93            CE_CAN_15.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_15_H93            CE_CAN_15.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_15_W93            CE_CAN_15.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_15_CS94           CE_CAN_15.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_15_ID94           CE_CAN_15.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_15_B94            CE_CAN_15.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_15_H94            CE_CAN_15.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_15_W94            CE_CAN_15.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_15_CS95           CE_CAN_15.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_15_ID95           CE_CAN_15.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_15_B95            CE_CAN_15.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_15_H95            CE_CAN_15.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_15_W95            CE_CAN_15.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_15_CS96           CE_CAN_15.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_15_ID96           CE_CAN_15.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_15_B96            CE_CAN_15.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_15_H96            CE_CAN_15.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_15_W96            CE_CAN_15.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_15_CS97           CE_CAN_15.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_15_ID97           CE_CAN_15.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_15_B97            CE_CAN_15.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_15_H97            CE_CAN_15.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_15_W97            CE_CAN_15.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_15_CS98           CE_CAN_15.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_15_ID98           CE_CAN_15.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_15_B98            CE_CAN_15.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_15_H98            CE_CAN_15.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_15_W98            CE_CAN_15.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_15_CS99           CE_CAN_15.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_15_ID99           CE_CAN_15.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_15_B99            CE_CAN_15.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_15_H99            CE_CAN_15.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_15_W99            CE_CAN_15.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_15_CS100          CE_CAN_15.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_15_ID100          CE_CAN_15.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_15_B100           CE_CAN_15.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_15_H100           CE_CAN_15.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_15_W100           CE_CAN_15.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_15_CS101          CE_CAN_15.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_15_ID101          CE_CAN_15.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_15_B101           CE_CAN_15.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_15_H101           CE_CAN_15.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_15_W101           CE_CAN_15.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_15_CS102          CE_CAN_15.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_15_ID102          CE_CAN_15.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_15_B102           CE_CAN_15.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_15_H102           CE_CAN_15.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_15_W102           CE_CAN_15.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_15_CS103          CE_CAN_15.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_15_ID103          CE_CAN_15.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_15_B103           CE_CAN_15.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_15_H103           CE_CAN_15.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_15_W103           CE_CAN_15.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_15_CS104          CE_CAN_15.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_15_ID104          CE_CAN_15.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_15_B104           CE_CAN_15.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_15_H104           CE_CAN_15.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_15_W104           CE_CAN_15.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_15_CS105          CE_CAN_15.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_15_ID105          CE_CAN_15.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_15_B105           CE_CAN_15.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_15_H105           CE_CAN_15.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_15_W105           CE_CAN_15.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_15_CS106          CE_CAN_15.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_15_ID106          CE_CAN_15.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_15_B106           CE_CAN_15.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_15_H106           CE_CAN_15.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_15_W106           CE_CAN_15.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_15_CS107          CE_CAN_15.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_15_ID107          CE_CAN_15.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_15_B107           CE_CAN_15.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_15_H107           CE_CAN_15.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_15_W107           CE_CAN_15.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_15_CS108          CE_CAN_15.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_15_ID108          CE_CAN_15.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_15_B108           CE_CAN_15.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_15_H108           CE_CAN_15.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_15_W108           CE_CAN_15.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_15_CS109          CE_CAN_15.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_15_ID109          CE_CAN_15.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_15_B109           CE_CAN_15.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_15_H109           CE_CAN_15.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_15_W109           CE_CAN_15.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_15_CS110          CE_CAN_15.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_15_ID110          CE_CAN_15.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_15_B110           CE_CAN_15.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_15_H110           CE_CAN_15.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_15_W110           CE_CAN_15.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_15_CS111          CE_CAN_15.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_15_ID111          CE_CAN_15.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_15_B111           CE_CAN_15.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_15_H111           CE_CAN_15.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_15_W111           CE_CAN_15.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_15_CS112          CE_CAN_15.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_15_ID112          CE_CAN_15.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_15_B112           CE_CAN_15.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_15_H112           CE_CAN_15.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_15_W112           CE_CAN_15.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_15_CS113          CE_CAN_15.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_15_ID113          CE_CAN_15.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_15_B113           CE_CAN_15.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_15_H113           CE_CAN_15.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_15_W113           CE_CAN_15.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_15_CS114          CE_CAN_15.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_15_ID114          CE_CAN_15.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_15_B114           CE_CAN_15.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_15_H114           CE_CAN_15.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_15_W114           CE_CAN_15.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_15_CS115          CE_CAN_15.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_15_ID115          CE_CAN_15.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_15_B115           CE_CAN_15.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_15_H115           CE_CAN_15.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_15_W115           CE_CAN_15.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_15_CS116          CE_CAN_15.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_15_ID116          CE_CAN_15.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_15_B116           CE_CAN_15.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_15_H116           CE_CAN_15.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_15_W116           CE_CAN_15.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_15_CS117          CE_CAN_15.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_15_ID117          CE_CAN_15.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_15_B117           CE_CAN_15.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_15_H117           CE_CAN_15.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_15_W117           CE_CAN_15.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_15_CS118          CE_CAN_15.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_15_ID118          CE_CAN_15.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_15_B118           CE_CAN_15.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_15_H118           CE_CAN_15.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_15_W118           CE_CAN_15.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_15_CS119          CE_CAN_15.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_15_ID119          CE_CAN_15.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_15_B119           CE_CAN_15.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_15_H119           CE_CAN_15.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_15_W119           CE_CAN_15.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_15_CS120          CE_CAN_15.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_15_ID120          CE_CAN_15.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_15_B120           CE_CAN_15.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_15_H120           CE_CAN_15.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_15_W120           CE_CAN_15.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_15_CS121          CE_CAN_15.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_15_ID121          CE_CAN_15.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_15_B121           CE_CAN_15.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_15_H121           CE_CAN_15.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_15_W121           CE_CAN_15.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_15_CS122          CE_CAN_15.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_15_ID122          CE_CAN_15.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_15_B122           CE_CAN_15.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_15_H122           CE_CAN_15.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_15_W122           CE_CAN_15.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_15_CS123          CE_CAN_15.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_15_ID123          CE_CAN_15.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_15_B123           CE_CAN_15.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_15_H123           CE_CAN_15.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_15_W123           CE_CAN_15.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_15_CS124          CE_CAN_15.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_15_ID124          CE_CAN_15.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_15_B124           CE_CAN_15.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_15_H124           CE_CAN_15.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_15_W124           CE_CAN_15.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_15_CS125          CE_CAN_15.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_15_ID125          CE_CAN_15.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_15_B125           CE_CAN_15.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_15_H125           CE_CAN_15.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_15_W125           CE_CAN_15.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_15_CS126          CE_CAN_15.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_15_ID126          CE_CAN_15.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_15_B126           CE_CAN_15.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_15_H126           CE_CAN_15.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_15_W126           CE_CAN_15.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_15_CS127          CE_CAN_15.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_15_ID127          CE_CAN_15.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_15_B127           CE_CAN_15.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_15_H127           CE_CAN_15.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_15_W127           CE_CAN_15.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_15_RXIMR0     CE_CAN_15.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR1     CE_CAN_15.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR2     CE_CAN_15.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR3     CE_CAN_15.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR4     CE_CAN_15.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR5     CE_CAN_15.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR6     CE_CAN_15.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR7     CE_CAN_15.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR8     CE_CAN_15.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR9     CE_CAN_15.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_15_RXIMR10    CE_CAN_15.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR11    CE_CAN_15.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR12    CE_CAN_15.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR13    CE_CAN_15.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR14    CE_CAN_15.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR15    CE_CAN_15.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR16    CE_CAN_15.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR17    CE_CAN_15.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR18    CE_CAN_15.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR19    CE_CAN_15.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR20    CE_CAN_15.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR21    CE_CAN_15.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR22    CE_CAN_15.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR23    CE_CAN_15.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR24    CE_CAN_15.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR25    CE_CAN_15.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR26    CE_CAN_15.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR27    CE_CAN_15.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR28    CE_CAN_15.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR29    CE_CAN_15.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR30    CE_CAN_15.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR31    CE_CAN_15.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR32    CE_CAN_15.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR33    CE_CAN_15.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR34    CE_CAN_15.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR35    CE_CAN_15.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR36    CE_CAN_15.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR37    CE_CAN_15.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR38    CE_CAN_15.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR39    CE_CAN_15.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR40    CE_CAN_15.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR41    CE_CAN_15.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR42    CE_CAN_15.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR43    CE_CAN_15.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR44    CE_CAN_15.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR45    CE_CAN_15.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR46    CE_CAN_15.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR47    CE_CAN_15.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR48    CE_CAN_15.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR49    CE_CAN_15.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR50    CE_CAN_15.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR51    CE_CAN_15.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR52    CE_CAN_15.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR53    CE_CAN_15.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR54    CE_CAN_15.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR55    CE_CAN_15.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR56    CE_CAN_15.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR57    CE_CAN_15.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR58    CE_CAN_15.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR59    CE_CAN_15.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR60    CE_CAN_15.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR61    CE_CAN_15.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR62    CE_CAN_15.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR63    CE_CAN_15.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR64    CE_CAN_15.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR65    CE_CAN_15.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR66    CE_CAN_15.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR67    CE_CAN_15.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR68    CE_CAN_15.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR69    CE_CAN_15.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR70    CE_CAN_15.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR71    CE_CAN_15.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR72    CE_CAN_15.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR73    CE_CAN_15.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR74    CE_CAN_15.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR75    CE_CAN_15.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR76    CE_CAN_15.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR77    CE_CAN_15.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR78    CE_CAN_15.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR79    CE_CAN_15.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR80    CE_CAN_15.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR81    CE_CAN_15.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR82    CE_CAN_15.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR83    CE_CAN_15.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR84    CE_CAN_15.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR85    CE_CAN_15.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR86    CE_CAN_15.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR87    CE_CAN_15.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR88    CE_CAN_15.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR89    CE_CAN_15.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR90    CE_CAN_15.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR91    CE_CAN_15.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR92    CE_CAN_15.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR93    CE_CAN_15.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR94    CE_CAN_15.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR95    CE_CAN_15.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR96    CE_CAN_15.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR97    CE_CAN_15.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR98    CE_CAN_15.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR99    CE_CAN_15.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_15_RXIMR100   CE_CAN_15.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR101   CE_CAN_15.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR102   CE_CAN_15.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR103   CE_CAN_15.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR104   CE_CAN_15.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR105   CE_CAN_15.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR106   CE_CAN_15.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR107   CE_CAN_15.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR108   CE_CAN_15.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR109   CE_CAN_15.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR110   CE_CAN_15.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR111   CE_CAN_15.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR112   CE_CAN_15.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR113   CE_CAN_15.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR114   CE_CAN_15.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR115   CE_CAN_15.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR116   CE_CAN_15.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR117   CE_CAN_15.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR118   CE_CAN_15.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR119   CE_CAN_15.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR120   CE_CAN_15.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR121   CE_CAN_15.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR122   CE_CAN_15.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR123   CE_CAN_15.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR124   CE_CAN_15.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR125   CE_CAN_15.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR126   CE_CAN_15.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_15_RXIMR127   CE_CAN_15.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_15_MECR       CE_CAN_15.MECR.R              /* Memory Error Control */
#define CE_CAN_15_ERRIAR     CE_CAN_15.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_15_ERRIDPR    CE_CAN_15.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_15_ERRIPPR    CE_CAN_15.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_15_RERRAR     CE_CAN_15.RERRAR.R            /* Error Report Address */
#define CE_CAN_15_RERRDR     CE_CAN_15.RERRDR.R            /* Error Report Data */
#define CE_CAN_15_RERRSYNR   CE_CAN_15.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_15_ERRSR      CE_CAN_15.ERRSR.R             /* Error Status */
#define CE_CAN_15_EPRS       CE_CAN_15.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_15_ENCBT      CE_CAN_15.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_15_EDCBT      CE_CAN_15.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_15_ETDC       CE_CAN_15.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_15_FDCTRL     CE_CAN_15.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_15_FDCBT      CE_CAN_15.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_15_FDCRC      CE_CAN_15.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_15_ERFCR      CE_CAN_15.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_15_ERFIER     CE_CAN_15.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_15_ERFSR      CE_CAN_15.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_15_HR_TIME_STAMP0 CE_CAN_15.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP1 CE_CAN_15.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP2 CE_CAN_15.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP3 CE_CAN_15.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP4 CE_CAN_15.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP5 CE_CAN_15.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP6 CE_CAN_15.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP7 CE_CAN_15.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP8 CE_CAN_15.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP9 CE_CAN_15.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP10 CE_CAN_15.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP11 CE_CAN_15.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP12 CE_CAN_15.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP13 CE_CAN_15.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP14 CE_CAN_15.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP15 CE_CAN_15.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP16 CE_CAN_15.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP17 CE_CAN_15.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP18 CE_CAN_15.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP19 CE_CAN_15.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP20 CE_CAN_15.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP21 CE_CAN_15.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP22 CE_CAN_15.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP23 CE_CAN_15.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP24 CE_CAN_15.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP25 CE_CAN_15.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP26 CE_CAN_15.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP27 CE_CAN_15.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP28 CE_CAN_15.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP29 CE_CAN_15.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP30 CE_CAN_15.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP31 CE_CAN_15.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP32 CE_CAN_15.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP33 CE_CAN_15.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP34 CE_CAN_15.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP35 CE_CAN_15.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP36 CE_CAN_15.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP37 CE_CAN_15.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP38 CE_CAN_15.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP39 CE_CAN_15.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP40 CE_CAN_15.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP41 CE_CAN_15.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP42 CE_CAN_15.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP43 CE_CAN_15.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP44 CE_CAN_15.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP45 CE_CAN_15.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP46 CE_CAN_15.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP47 CE_CAN_15.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP48 CE_CAN_15.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP49 CE_CAN_15.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP50 CE_CAN_15.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP51 CE_CAN_15.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP52 CE_CAN_15.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP53 CE_CAN_15.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP54 CE_CAN_15.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP55 CE_CAN_15.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP56 CE_CAN_15.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP57 CE_CAN_15.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP58 CE_CAN_15.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP59 CE_CAN_15.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP60 CE_CAN_15.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP61 CE_CAN_15.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP62 CE_CAN_15.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP63 CE_CAN_15.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP64 CE_CAN_15.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP65 CE_CAN_15.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP66 CE_CAN_15.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP67 CE_CAN_15.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP68 CE_CAN_15.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP69 CE_CAN_15.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP70 CE_CAN_15.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP71 CE_CAN_15.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP72 CE_CAN_15.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP73 CE_CAN_15.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP74 CE_CAN_15.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP75 CE_CAN_15.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP76 CE_CAN_15.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP77 CE_CAN_15.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP78 CE_CAN_15.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP79 CE_CAN_15.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP80 CE_CAN_15.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP81 CE_CAN_15.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP82 CE_CAN_15.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP83 CE_CAN_15.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP84 CE_CAN_15.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP85 CE_CAN_15.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP86 CE_CAN_15.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP87 CE_CAN_15.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP88 CE_CAN_15.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP89 CE_CAN_15.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP90 CE_CAN_15.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP91 CE_CAN_15.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP92 CE_CAN_15.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP93 CE_CAN_15.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP94 CE_CAN_15.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP95 CE_CAN_15.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP96 CE_CAN_15.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP97 CE_CAN_15.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP98 CE_CAN_15.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP99 CE_CAN_15.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP100 CE_CAN_15.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP101 CE_CAN_15.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP102 CE_CAN_15.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP103 CE_CAN_15.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP104 CE_CAN_15.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP105 CE_CAN_15.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP106 CE_CAN_15.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP107 CE_CAN_15.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP108 CE_CAN_15.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP109 CE_CAN_15.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP110 CE_CAN_15.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP111 CE_CAN_15.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP112 CE_CAN_15.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP113 CE_CAN_15.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP114 CE_CAN_15.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP115 CE_CAN_15.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP116 CE_CAN_15.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP117 CE_CAN_15.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP118 CE_CAN_15.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP119 CE_CAN_15.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP120 CE_CAN_15.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP121 CE_CAN_15.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP122 CE_CAN_15.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP123 CE_CAN_15.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP124 CE_CAN_15.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP125 CE_CAN_15.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP126 CE_CAN_15.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_15_HR_TIME_STAMP127 CE_CAN_15.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_15_ERFFEL0    CE_CAN_15.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL1    CE_CAN_15.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL2    CE_CAN_15.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL3    CE_CAN_15.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL4    CE_CAN_15.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL5    CE_CAN_15.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL6    CE_CAN_15.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL7    CE_CAN_15.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL8    CE_CAN_15.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL9    CE_CAN_15.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL10   CE_CAN_15.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL11   CE_CAN_15.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL12   CE_CAN_15.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL13   CE_CAN_15.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL14   CE_CAN_15.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL15   CE_CAN_15.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL16   CE_CAN_15.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL17   CE_CAN_15.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL18   CE_CAN_15.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL19   CE_CAN_15.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL20   CE_CAN_15.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL21   CE_CAN_15.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL22   CE_CAN_15.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL23   CE_CAN_15.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL24   CE_CAN_15.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL25   CE_CAN_15.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL26   CE_CAN_15.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL27   CE_CAN_15.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL28   CE_CAN_15.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL29   CE_CAN_15.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL30   CE_CAN_15.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL31   CE_CAN_15.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL32   CE_CAN_15.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL33   CE_CAN_15.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL34   CE_CAN_15.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL35   CE_CAN_15.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL36   CE_CAN_15.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL37   CE_CAN_15.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL38   CE_CAN_15.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL39   CE_CAN_15.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL40   CE_CAN_15.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL41   CE_CAN_15.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL42   CE_CAN_15.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL43   CE_CAN_15.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL44   CE_CAN_15.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL45   CE_CAN_15.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL46   CE_CAN_15.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL47   CE_CAN_15.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL48   CE_CAN_15.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL49   CE_CAN_15.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL50   CE_CAN_15.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL51   CE_CAN_15.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL52   CE_CAN_15.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL53   CE_CAN_15.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL54   CE_CAN_15.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL55   CE_CAN_15.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL56   CE_CAN_15.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL57   CE_CAN_15.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL58   CE_CAN_15.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL59   CE_CAN_15.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL60   CE_CAN_15.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL61   CE_CAN_15.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL62   CE_CAN_15.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL63   CE_CAN_15.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL64   CE_CAN_15.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL65   CE_CAN_15.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL66   CE_CAN_15.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL67   CE_CAN_15.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL68   CE_CAN_15.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL69   CE_CAN_15.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL70   CE_CAN_15.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL71   CE_CAN_15.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL72   CE_CAN_15.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL73   CE_CAN_15.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL74   CE_CAN_15.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL75   CE_CAN_15.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL76   CE_CAN_15.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL77   CE_CAN_15.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL78   CE_CAN_15.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL79   CE_CAN_15.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL80   CE_CAN_15.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL81   CE_CAN_15.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL82   CE_CAN_15.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL83   CE_CAN_15.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL84   CE_CAN_15.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL85   CE_CAN_15.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL86   CE_CAN_15.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL87   CE_CAN_15.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL88   CE_CAN_15.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL89   CE_CAN_15.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL90   CE_CAN_15.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL91   CE_CAN_15.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL92   CE_CAN_15.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL93   CE_CAN_15.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL94   CE_CAN_15.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL95   CE_CAN_15.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL96   CE_CAN_15.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL97   CE_CAN_15.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL98   CE_CAN_15.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL99   CE_CAN_15.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL100  CE_CAN_15.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL101  CE_CAN_15.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL102  CE_CAN_15.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL103  CE_CAN_15.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL104  CE_CAN_15.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL105  CE_CAN_15.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL106  CE_CAN_15.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL107  CE_CAN_15.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL108  CE_CAN_15.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL109  CE_CAN_15.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL110  CE_CAN_15.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL111  CE_CAN_15.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL112  CE_CAN_15.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL113  CE_CAN_15.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL114  CE_CAN_15.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL115  CE_CAN_15.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL116  CE_CAN_15.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL117  CE_CAN_15.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL118  CE_CAN_15.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL119  CE_CAN_15.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL120  CE_CAN_15.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL121  CE_CAN_15.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL122  CE_CAN_15.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL123  CE_CAN_15.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL124  CE_CAN_15.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL125  CE_CAN_15.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL126  CE_CAN_15.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_15_ERFFEL127  CE_CAN_15.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_16_MCR        CE_CAN_16.MCR.R               /* Module Configuration */
#define CE_CAN_16_CTRL1      CE_CAN_16.CTRL1.R             /* Control 1 */
#define CE_CAN_16_TIMER      CE_CAN_16.TIMER.R             /* Free-Running Timer */
#define CE_CAN_16_RXMGMASK   CE_CAN_16.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_16_RX14MASK   CE_CAN_16.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_16_RX15MASK   CE_CAN_16.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_16_ECR        CE_CAN_16.ECR.R               /* Error Counter */
#define CE_CAN_16_ESR1       CE_CAN_16.ESR1.R              /* Error and Status 1 */
#define CE_CAN_16_IMASK2     CE_CAN_16.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_16_IMASK1     CE_CAN_16.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_16_IFLAG2     CE_CAN_16.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_16_IFLAG1     CE_CAN_16.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_16_CTRL2      CE_CAN_16.CTRL2.R             /* Control 2 */
#define CE_CAN_16_ESR2       CE_CAN_16.ESR2.R              /* Error and Status 2 */
#define CE_CAN_16_CRCR       CE_CAN_16.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_16_RXFGMASK   CE_CAN_16.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_16_RXFIR      CE_CAN_16.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_16_CBT        CE_CAN_16.CBT.R               /* CAN Bit Timing */
#define CE_CAN_16_IMASK4     CE_CAN_16.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_16_IMASK3     CE_CAN_16.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_16_IFLAG4     CE_CAN_16.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_16_IFLAG3     CE_CAN_16.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_16_CS0            CE_CAN_16.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_16_ID0            CE_CAN_16.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_16_B0             CE_CAN_16.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_16_H0             CE_CAN_16.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_16_W0             CE_CAN_16.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_16_CS1            CE_CAN_16.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_16_ID1            CE_CAN_16.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_16_B1             CE_CAN_16.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_16_H1             CE_CAN_16.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_16_W1             CE_CAN_16.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_16_CS2            CE_CAN_16.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_16_ID2            CE_CAN_16.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_16_B2             CE_CAN_16.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_16_H2             CE_CAN_16.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_16_W2             CE_CAN_16.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_16_CS3            CE_CAN_16.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_16_ID3            CE_CAN_16.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_16_B3             CE_CAN_16.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_16_H3             CE_CAN_16.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_16_W3             CE_CAN_16.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_16_CS4            CE_CAN_16.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_16_ID4            CE_CAN_16.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_16_B4             CE_CAN_16.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_16_H4             CE_CAN_16.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_16_W4             CE_CAN_16.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_16_CS5            CE_CAN_16.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_16_ID5            CE_CAN_16.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_16_B5             CE_CAN_16.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_16_H5             CE_CAN_16.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_16_W5             CE_CAN_16.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_16_CS6            CE_CAN_16.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_16_ID6            CE_CAN_16.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_16_B6             CE_CAN_16.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_16_H6             CE_CAN_16.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_16_W6             CE_CAN_16.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_16_CS7            CE_CAN_16.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_16_ID7            CE_CAN_16.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_16_B7             CE_CAN_16.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_16_H7             CE_CAN_16.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_16_W7             CE_CAN_16.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_16_CS8            CE_CAN_16.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_16_ID8            CE_CAN_16.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_16_B8             CE_CAN_16.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_16_H8             CE_CAN_16.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_16_W8             CE_CAN_16.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_16_CS9            CE_CAN_16.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_16_ID9            CE_CAN_16.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_16_B9             CE_CAN_16.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_16_H9             CE_CAN_16.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_16_W9             CE_CAN_16.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_16_CS10           CE_CAN_16.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_16_ID10           CE_CAN_16.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_16_B10            CE_CAN_16.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_16_H10            CE_CAN_16.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_16_W10            CE_CAN_16.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_16_CS11           CE_CAN_16.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_16_ID11           CE_CAN_16.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_16_B11            CE_CAN_16.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_16_H11            CE_CAN_16.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_16_W11            CE_CAN_16.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_16_CS12           CE_CAN_16.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_16_ID12           CE_CAN_16.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_16_B12            CE_CAN_16.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_16_H12            CE_CAN_16.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_16_W12            CE_CAN_16.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_16_CS13           CE_CAN_16.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_16_ID13           CE_CAN_16.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_16_B13            CE_CAN_16.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_16_H13            CE_CAN_16.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_16_W13            CE_CAN_16.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_16_CS14           CE_CAN_16.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_16_ID14           CE_CAN_16.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_16_B14            CE_CAN_16.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_16_H14            CE_CAN_16.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_16_W14            CE_CAN_16.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_16_CS15           CE_CAN_16.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_16_ID15           CE_CAN_16.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_16_B15            CE_CAN_16.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_16_H15            CE_CAN_16.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_16_W15            CE_CAN_16.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_16_CS16           CE_CAN_16.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_16_ID16           CE_CAN_16.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_16_B16            CE_CAN_16.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_16_H16            CE_CAN_16.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_16_W16            CE_CAN_16.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_16_CS17           CE_CAN_16.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_16_ID17           CE_CAN_16.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_16_B17            CE_CAN_16.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_16_H17            CE_CAN_16.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_16_W17            CE_CAN_16.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_16_CS18           CE_CAN_16.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_16_ID18           CE_CAN_16.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_16_B18            CE_CAN_16.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_16_H18            CE_CAN_16.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_16_W18            CE_CAN_16.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_16_CS19           CE_CAN_16.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_16_ID19           CE_CAN_16.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_16_B19            CE_CAN_16.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_16_H19            CE_CAN_16.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_16_W19            CE_CAN_16.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_16_CS20           CE_CAN_16.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_16_ID20           CE_CAN_16.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_16_B20            CE_CAN_16.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_16_H20            CE_CAN_16.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_16_W20            CE_CAN_16.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_16_CS21           CE_CAN_16.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_16_ID21           CE_CAN_16.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_16_B21            CE_CAN_16.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_16_H21            CE_CAN_16.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_16_W21            CE_CAN_16.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_16_CS22           CE_CAN_16.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_16_ID22           CE_CAN_16.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_16_B22            CE_CAN_16.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_16_H22            CE_CAN_16.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_16_W22            CE_CAN_16.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_16_CS23           CE_CAN_16.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_16_ID23           CE_CAN_16.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_16_B23            CE_CAN_16.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_16_H23            CE_CAN_16.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_16_W23            CE_CAN_16.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_16_CS24           CE_CAN_16.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_16_ID24           CE_CAN_16.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_16_B24            CE_CAN_16.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_16_H24            CE_CAN_16.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_16_W24            CE_CAN_16.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_16_CS25           CE_CAN_16.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_16_ID25           CE_CAN_16.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_16_B25            CE_CAN_16.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_16_H25            CE_CAN_16.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_16_W25            CE_CAN_16.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_16_CS26           CE_CAN_16.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_16_ID26           CE_CAN_16.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_16_B26            CE_CAN_16.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_16_H26            CE_CAN_16.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_16_W26            CE_CAN_16.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_16_CS27           CE_CAN_16.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_16_ID27           CE_CAN_16.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_16_B27            CE_CAN_16.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_16_H27            CE_CAN_16.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_16_W27            CE_CAN_16.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_16_CS28           CE_CAN_16.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_16_ID28           CE_CAN_16.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_16_B28            CE_CAN_16.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_16_H28            CE_CAN_16.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_16_W28            CE_CAN_16.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_16_CS29           CE_CAN_16.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_16_ID29           CE_CAN_16.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_16_B29            CE_CAN_16.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_16_H29            CE_CAN_16.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_16_W29            CE_CAN_16.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_16_CS30           CE_CAN_16.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_16_ID30           CE_CAN_16.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_16_B30            CE_CAN_16.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_16_H30            CE_CAN_16.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_16_W30            CE_CAN_16.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_16_CS31           CE_CAN_16.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_16_ID31           CE_CAN_16.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_16_B31            CE_CAN_16.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_16_H31            CE_CAN_16.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_16_W31            CE_CAN_16.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_16_CS32           CE_CAN_16.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_16_ID32           CE_CAN_16.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_16_B32            CE_CAN_16.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_16_H32            CE_CAN_16.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_16_W32            CE_CAN_16.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_16_CS33           CE_CAN_16.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_16_ID33           CE_CAN_16.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_16_B33            CE_CAN_16.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_16_H33            CE_CAN_16.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_16_W33            CE_CAN_16.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_16_CS34           CE_CAN_16.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_16_ID34           CE_CAN_16.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_16_B34            CE_CAN_16.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_16_H34            CE_CAN_16.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_16_W34            CE_CAN_16.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_16_CS35           CE_CAN_16.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_16_ID35           CE_CAN_16.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_16_B35            CE_CAN_16.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_16_H35            CE_CAN_16.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_16_W35            CE_CAN_16.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_16_CS36           CE_CAN_16.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_16_ID36           CE_CAN_16.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_16_B36            CE_CAN_16.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_16_H36            CE_CAN_16.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_16_W36            CE_CAN_16.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_16_CS37           CE_CAN_16.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_16_ID37           CE_CAN_16.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_16_B37            CE_CAN_16.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_16_H37            CE_CAN_16.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_16_W37            CE_CAN_16.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_16_CS38           CE_CAN_16.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_16_ID38           CE_CAN_16.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_16_B38            CE_CAN_16.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_16_H38            CE_CAN_16.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_16_W38            CE_CAN_16.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_16_CS39           CE_CAN_16.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_16_ID39           CE_CAN_16.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_16_B39            CE_CAN_16.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_16_H39            CE_CAN_16.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_16_W39            CE_CAN_16.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_16_CS40           CE_CAN_16.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_16_ID40           CE_CAN_16.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_16_B40            CE_CAN_16.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_16_H40            CE_CAN_16.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_16_W40            CE_CAN_16.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_16_CS41           CE_CAN_16.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_16_ID41           CE_CAN_16.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_16_B41            CE_CAN_16.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_16_H41            CE_CAN_16.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_16_W41            CE_CAN_16.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_16_CS42           CE_CAN_16.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_16_ID42           CE_CAN_16.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_16_B42            CE_CAN_16.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_16_H42            CE_CAN_16.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_16_W42            CE_CAN_16.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_16_CS43           CE_CAN_16.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_16_ID43           CE_CAN_16.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_16_B43            CE_CAN_16.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_16_H43            CE_CAN_16.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_16_W43            CE_CAN_16.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_16_CS44           CE_CAN_16.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_16_ID44           CE_CAN_16.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_16_B44            CE_CAN_16.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_16_H44            CE_CAN_16.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_16_W44            CE_CAN_16.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_16_CS45           CE_CAN_16.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_16_ID45           CE_CAN_16.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_16_B45            CE_CAN_16.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_16_H45            CE_CAN_16.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_16_W45            CE_CAN_16.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_16_CS46           CE_CAN_16.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_16_ID46           CE_CAN_16.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_16_B46            CE_CAN_16.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_16_H46            CE_CAN_16.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_16_W46            CE_CAN_16.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_16_CS47           CE_CAN_16.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_16_ID47           CE_CAN_16.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_16_B47            CE_CAN_16.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_16_H47            CE_CAN_16.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_16_W47            CE_CAN_16.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_16_CS48           CE_CAN_16.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_16_ID48           CE_CAN_16.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_16_B48            CE_CAN_16.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_16_H48            CE_CAN_16.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_16_W48            CE_CAN_16.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_16_CS49           CE_CAN_16.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_16_ID49           CE_CAN_16.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_16_B49            CE_CAN_16.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_16_H49            CE_CAN_16.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_16_W49            CE_CAN_16.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_16_CS50           CE_CAN_16.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_16_ID50           CE_CAN_16.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_16_B50            CE_CAN_16.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_16_H50            CE_CAN_16.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_16_W50            CE_CAN_16.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_16_CS51           CE_CAN_16.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_16_ID51           CE_CAN_16.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_16_B51            CE_CAN_16.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_16_H51            CE_CAN_16.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_16_W51            CE_CAN_16.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_16_CS52           CE_CAN_16.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_16_ID52           CE_CAN_16.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_16_B52            CE_CAN_16.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_16_H52            CE_CAN_16.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_16_W52            CE_CAN_16.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_16_CS53           CE_CAN_16.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_16_ID53           CE_CAN_16.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_16_B53            CE_CAN_16.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_16_H53            CE_CAN_16.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_16_W53            CE_CAN_16.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_16_CS54           CE_CAN_16.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_16_ID54           CE_CAN_16.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_16_B54            CE_CAN_16.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_16_H54            CE_CAN_16.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_16_W54            CE_CAN_16.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_16_CS55           CE_CAN_16.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_16_ID55           CE_CAN_16.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_16_B55            CE_CAN_16.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_16_H55            CE_CAN_16.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_16_W55            CE_CAN_16.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_16_CS56           CE_CAN_16.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_16_ID56           CE_CAN_16.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_16_B56            CE_CAN_16.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_16_H56            CE_CAN_16.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_16_W56            CE_CAN_16.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_16_CS57           CE_CAN_16.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_16_ID57           CE_CAN_16.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_16_B57            CE_CAN_16.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_16_H57            CE_CAN_16.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_16_W57            CE_CAN_16.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_16_CS58           CE_CAN_16.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_16_ID58           CE_CAN_16.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_16_B58            CE_CAN_16.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_16_H58            CE_CAN_16.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_16_W58            CE_CAN_16.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_16_CS59           CE_CAN_16.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_16_ID59           CE_CAN_16.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_16_B59            CE_CAN_16.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_16_H59            CE_CAN_16.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_16_W59            CE_CAN_16.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_16_CS60           CE_CAN_16.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_16_ID60           CE_CAN_16.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_16_B60            CE_CAN_16.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_16_H60            CE_CAN_16.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_16_W60            CE_CAN_16.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_16_CS61           CE_CAN_16.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_16_ID61           CE_CAN_16.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_16_B61            CE_CAN_16.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_16_H61            CE_CAN_16.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_16_W61            CE_CAN_16.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_16_CS62           CE_CAN_16.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_16_ID62           CE_CAN_16.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_16_B62            CE_CAN_16.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_16_H62            CE_CAN_16.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_16_W62            CE_CAN_16.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_16_CS63           CE_CAN_16.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_16_ID63           CE_CAN_16.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_16_B63            CE_CAN_16.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_16_H63            CE_CAN_16.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_16_W63            CE_CAN_16.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_16_CS64           CE_CAN_16.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_16_ID64           CE_CAN_16.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_16_B64            CE_CAN_16.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_16_H64            CE_CAN_16.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_16_W64            CE_CAN_16.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_16_CS65           CE_CAN_16.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_16_ID65           CE_CAN_16.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_16_B65            CE_CAN_16.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_16_H65            CE_CAN_16.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_16_W65            CE_CAN_16.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_16_CS66           CE_CAN_16.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_16_ID66           CE_CAN_16.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_16_B66            CE_CAN_16.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_16_H66            CE_CAN_16.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_16_W66            CE_CAN_16.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_16_CS67           CE_CAN_16.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_16_ID67           CE_CAN_16.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_16_B67            CE_CAN_16.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_16_H67            CE_CAN_16.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_16_W67            CE_CAN_16.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_16_CS68           CE_CAN_16.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_16_ID68           CE_CAN_16.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_16_B68            CE_CAN_16.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_16_H68            CE_CAN_16.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_16_W68            CE_CAN_16.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_16_CS69           CE_CAN_16.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_16_ID69           CE_CAN_16.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_16_B69            CE_CAN_16.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_16_H69            CE_CAN_16.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_16_W69            CE_CAN_16.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_16_CS70           CE_CAN_16.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_16_ID70           CE_CAN_16.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_16_B70            CE_CAN_16.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_16_H70            CE_CAN_16.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_16_W70            CE_CAN_16.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_16_CS71           CE_CAN_16.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_16_ID71           CE_CAN_16.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_16_B71            CE_CAN_16.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_16_H71            CE_CAN_16.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_16_W71            CE_CAN_16.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_16_CS72           CE_CAN_16.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_16_ID72           CE_CAN_16.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_16_B72            CE_CAN_16.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_16_H72            CE_CAN_16.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_16_W72            CE_CAN_16.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_16_CS73           CE_CAN_16.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_16_ID73           CE_CAN_16.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_16_B73            CE_CAN_16.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_16_H73            CE_CAN_16.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_16_W73            CE_CAN_16.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_16_CS74           CE_CAN_16.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_16_ID74           CE_CAN_16.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_16_B74            CE_CAN_16.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_16_H74            CE_CAN_16.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_16_W74            CE_CAN_16.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_16_CS75           CE_CAN_16.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_16_ID75           CE_CAN_16.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_16_B75            CE_CAN_16.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_16_H75            CE_CAN_16.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_16_W75            CE_CAN_16.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_16_CS76           CE_CAN_16.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_16_ID76           CE_CAN_16.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_16_B76            CE_CAN_16.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_16_H76            CE_CAN_16.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_16_W76            CE_CAN_16.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_16_CS77           CE_CAN_16.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_16_ID77           CE_CAN_16.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_16_B77            CE_CAN_16.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_16_H77            CE_CAN_16.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_16_W77            CE_CAN_16.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_16_CS78           CE_CAN_16.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_16_ID78           CE_CAN_16.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_16_B78            CE_CAN_16.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_16_H78            CE_CAN_16.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_16_W78            CE_CAN_16.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_16_CS79           CE_CAN_16.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_16_ID79           CE_CAN_16.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_16_B79            CE_CAN_16.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_16_H79            CE_CAN_16.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_16_W79            CE_CAN_16.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_16_CS80           CE_CAN_16.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_16_ID80           CE_CAN_16.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_16_B80            CE_CAN_16.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_16_H80            CE_CAN_16.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_16_W80            CE_CAN_16.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_16_CS81           CE_CAN_16.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_16_ID81           CE_CAN_16.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_16_B81            CE_CAN_16.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_16_H81            CE_CAN_16.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_16_W81            CE_CAN_16.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_16_CS82           CE_CAN_16.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_16_ID82           CE_CAN_16.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_16_B82            CE_CAN_16.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_16_H82            CE_CAN_16.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_16_W82            CE_CAN_16.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_16_CS83           CE_CAN_16.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_16_ID83           CE_CAN_16.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_16_B83            CE_CAN_16.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_16_H83            CE_CAN_16.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_16_W83            CE_CAN_16.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_16_CS84           CE_CAN_16.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_16_ID84           CE_CAN_16.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_16_B84            CE_CAN_16.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_16_H84            CE_CAN_16.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_16_W84            CE_CAN_16.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_16_CS85           CE_CAN_16.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_16_ID85           CE_CAN_16.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_16_B85            CE_CAN_16.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_16_H85            CE_CAN_16.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_16_W85            CE_CAN_16.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_16_CS86           CE_CAN_16.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_16_ID86           CE_CAN_16.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_16_B86            CE_CAN_16.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_16_H86            CE_CAN_16.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_16_W86            CE_CAN_16.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_16_CS87           CE_CAN_16.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_16_ID87           CE_CAN_16.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_16_B87            CE_CAN_16.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_16_H87            CE_CAN_16.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_16_W87            CE_CAN_16.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_16_CS88           CE_CAN_16.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_16_ID88           CE_CAN_16.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_16_B88            CE_CAN_16.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_16_H88            CE_CAN_16.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_16_W848           CE_CAN_16.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_16_CS89           CE_CAN_16.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_16_ID89           CE_CAN_16.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_16_B89            CE_CAN_16.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_16_H89            CE_CAN_16.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_16_W89            CE_CAN_16.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_16_CS90           CE_CAN_16.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_16_ID90           CE_CAN_16.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_16_B90            CE_CAN_16.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_16_H90            CE_CAN_16.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_16_W90            CE_CAN_16.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_16_CS91           CE_CAN_16.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_16_ID91           CE_CAN_16.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_16_B91            CE_CAN_16.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_16_H91            CE_CAN_16.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_16_W91            CE_CAN_16.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_16_CS92           CE_CAN_16.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_16_ID92           CE_CAN_16.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_16_B92            CE_CAN_16.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_16_H92            CE_CAN_16.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_16_W92            CE_CAN_16.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_16_CS93           CE_CAN_16.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_16_ID93           CE_CAN_16.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_16_B93            CE_CAN_16.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_16_H93            CE_CAN_16.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_16_W93            CE_CAN_16.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_16_CS94           CE_CAN_16.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_16_ID94           CE_CAN_16.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_16_B94            CE_CAN_16.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_16_H94            CE_CAN_16.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_16_W94            CE_CAN_16.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_16_CS95           CE_CAN_16.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_16_ID95           CE_CAN_16.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_16_B95            CE_CAN_16.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_16_H95            CE_CAN_16.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_16_W95            CE_CAN_16.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_16_CS96           CE_CAN_16.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_16_ID96           CE_CAN_16.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_16_B96            CE_CAN_16.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_16_H96            CE_CAN_16.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_16_W96            CE_CAN_16.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_16_CS97           CE_CAN_16.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_16_ID97           CE_CAN_16.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_16_B97            CE_CAN_16.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_16_H97            CE_CAN_16.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_16_W97            CE_CAN_16.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_16_CS98           CE_CAN_16.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_16_ID98           CE_CAN_16.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_16_B98            CE_CAN_16.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_16_H98            CE_CAN_16.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_16_W98            CE_CAN_16.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_16_CS99           CE_CAN_16.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_16_ID99           CE_CAN_16.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_16_B99            CE_CAN_16.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_16_H99            CE_CAN_16.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_16_W99            CE_CAN_16.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_16_CS100          CE_CAN_16.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_16_ID100          CE_CAN_16.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_16_B100           CE_CAN_16.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_16_H100           CE_CAN_16.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_16_W100           CE_CAN_16.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_16_CS101          CE_CAN_16.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_16_ID101          CE_CAN_16.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_16_B101           CE_CAN_16.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_16_H101           CE_CAN_16.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_16_W101           CE_CAN_16.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_16_CS102          CE_CAN_16.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_16_ID102          CE_CAN_16.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_16_B102           CE_CAN_16.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_16_H102           CE_CAN_16.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_16_W102           CE_CAN_16.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_16_CS103          CE_CAN_16.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_16_ID103          CE_CAN_16.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_16_B103           CE_CAN_16.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_16_H103           CE_CAN_16.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_16_W103           CE_CAN_16.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_16_CS104          CE_CAN_16.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_16_ID104          CE_CAN_16.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_16_B104           CE_CAN_16.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_16_H104           CE_CAN_16.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_16_W104           CE_CAN_16.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_16_CS105          CE_CAN_16.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_16_ID105          CE_CAN_16.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_16_B105           CE_CAN_16.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_16_H105           CE_CAN_16.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_16_W105           CE_CAN_16.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_16_CS106          CE_CAN_16.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_16_ID106          CE_CAN_16.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_16_B106           CE_CAN_16.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_16_H106           CE_CAN_16.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_16_W106           CE_CAN_16.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_16_CS107          CE_CAN_16.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_16_ID107          CE_CAN_16.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_16_B107           CE_CAN_16.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_16_H107           CE_CAN_16.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_16_W107           CE_CAN_16.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_16_CS108          CE_CAN_16.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_16_ID108          CE_CAN_16.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_16_B108           CE_CAN_16.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_16_H108           CE_CAN_16.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_16_W108           CE_CAN_16.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_16_CS109          CE_CAN_16.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_16_ID109          CE_CAN_16.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_16_B109           CE_CAN_16.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_16_H109           CE_CAN_16.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_16_W109           CE_CAN_16.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_16_CS110          CE_CAN_16.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_16_ID110          CE_CAN_16.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_16_B110           CE_CAN_16.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_16_H110           CE_CAN_16.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_16_W110           CE_CAN_16.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_16_CS111          CE_CAN_16.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_16_ID111          CE_CAN_16.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_16_B111           CE_CAN_16.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_16_H111           CE_CAN_16.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_16_W111           CE_CAN_16.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_16_CS112          CE_CAN_16.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_16_ID112          CE_CAN_16.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_16_B112           CE_CAN_16.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_16_H112           CE_CAN_16.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_16_W112           CE_CAN_16.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_16_CS113          CE_CAN_16.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_16_ID113          CE_CAN_16.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_16_B113           CE_CAN_16.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_16_H113           CE_CAN_16.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_16_W113           CE_CAN_16.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_16_CS114          CE_CAN_16.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_16_ID114          CE_CAN_16.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_16_B114           CE_CAN_16.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_16_H114           CE_CAN_16.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_16_W114           CE_CAN_16.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_16_CS115          CE_CAN_16.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_16_ID115          CE_CAN_16.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_16_B115           CE_CAN_16.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_16_H115           CE_CAN_16.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_16_W115           CE_CAN_16.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_16_CS116          CE_CAN_16.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_16_ID116          CE_CAN_16.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_16_B116           CE_CAN_16.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_16_H116           CE_CAN_16.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_16_W116           CE_CAN_16.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_16_CS117          CE_CAN_16.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_16_ID117          CE_CAN_16.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_16_B117           CE_CAN_16.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_16_H117           CE_CAN_16.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_16_W117           CE_CAN_16.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_16_CS118          CE_CAN_16.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_16_ID118          CE_CAN_16.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_16_B118           CE_CAN_16.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_16_H118           CE_CAN_16.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_16_W118           CE_CAN_16.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_16_CS119          CE_CAN_16.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_16_ID119          CE_CAN_16.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_16_B119           CE_CAN_16.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_16_H119           CE_CAN_16.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_16_W119           CE_CAN_16.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_16_CS120          CE_CAN_16.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_16_ID120          CE_CAN_16.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_16_B120           CE_CAN_16.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_16_H120           CE_CAN_16.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_16_W120           CE_CAN_16.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_16_CS121          CE_CAN_16.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_16_ID121          CE_CAN_16.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_16_B121           CE_CAN_16.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_16_H121           CE_CAN_16.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_16_W121           CE_CAN_16.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_16_CS122          CE_CAN_16.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_16_ID122          CE_CAN_16.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_16_B122           CE_CAN_16.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_16_H122           CE_CAN_16.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_16_W122           CE_CAN_16.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_16_CS123          CE_CAN_16.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_16_ID123          CE_CAN_16.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_16_B123           CE_CAN_16.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_16_H123           CE_CAN_16.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_16_W123           CE_CAN_16.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_16_CS124          CE_CAN_16.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_16_ID124          CE_CAN_16.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_16_B124           CE_CAN_16.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_16_H124           CE_CAN_16.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_16_W124           CE_CAN_16.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_16_CS125          CE_CAN_16.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_16_ID125          CE_CAN_16.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_16_B125           CE_CAN_16.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_16_H125           CE_CAN_16.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_16_W125           CE_CAN_16.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_16_CS126          CE_CAN_16.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_16_ID126          CE_CAN_16.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_16_B126           CE_CAN_16.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_16_H126           CE_CAN_16.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_16_W126           CE_CAN_16.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_16_CS127          CE_CAN_16.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_16_ID127          CE_CAN_16.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_16_B127           CE_CAN_16.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_16_H127           CE_CAN_16.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_16_W127           CE_CAN_16.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_16_RXIMR0     CE_CAN_16.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR1     CE_CAN_16.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR2     CE_CAN_16.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR3     CE_CAN_16.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR4     CE_CAN_16.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR5     CE_CAN_16.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR6     CE_CAN_16.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR7     CE_CAN_16.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR8     CE_CAN_16.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR9     CE_CAN_16.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_16_RXIMR10    CE_CAN_16.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR11    CE_CAN_16.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR12    CE_CAN_16.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR13    CE_CAN_16.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR14    CE_CAN_16.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR15    CE_CAN_16.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR16    CE_CAN_16.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR17    CE_CAN_16.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR18    CE_CAN_16.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR19    CE_CAN_16.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR20    CE_CAN_16.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR21    CE_CAN_16.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR22    CE_CAN_16.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR23    CE_CAN_16.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR24    CE_CAN_16.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR25    CE_CAN_16.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR26    CE_CAN_16.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR27    CE_CAN_16.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR28    CE_CAN_16.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR29    CE_CAN_16.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR30    CE_CAN_16.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR31    CE_CAN_16.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR32    CE_CAN_16.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR33    CE_CAN_16.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR34    CE_CAN_16.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR35    CE_CAN_16.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR36    CE_CAN_16.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR37    CE_CAN_16.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR38    CE_CAN_16.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR39    CE_CAN_16.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR40    CE_CAN_16.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR41    CE_CAN_16.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR42    CE_CAN_16.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR43    CE_CAN_16.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR44    CE_CAN_16.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR45    CE_CAN_16.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR46    CE_CAN_16.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR47    CE_CAN_16.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR48    CE_CAN_16.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR49    CE_CAN_16.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR50    CE_CAN_16.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR51    CE_CAN_16.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR52    CE_CAN_16.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR53    CE_CAN_16.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR54    CE_CAN_16.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR55    CE_CAN_16.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR56    CE_CAN_16.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR57    CE_CAN_16.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR58    CE_CAN_16.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR59    CE_CAN_16.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR60    CE_CAN_16.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR61    CE_CAN_16.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR62    CE_CAN_16.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR63    CE_CAN_16.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR64    CE_CAN_16.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR65    CE_CAN_16.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR66    CE_CAN_16.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR67    CE_CAN_16.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR68    CE_CAN_16.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR69    CE_CAN_16.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR70    CE_CAN_16.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR71    CE_CAN_16.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR72    CE_CAN_16.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR73    CE_CAN_16.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR74    CE_CAN_16.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR75    CE_CAN_16.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR76    CE_CAN_16.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR77    CE_CAN_16.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR78    CE_CAN_16.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR79    CE_CAN_16.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR80    CE_CAN_16.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR81    CE_CAN_16.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR82    CE_CAN_16.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR83    CE_CAN_16.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR84    CE_CAN_16.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR85    CE_CAN_16.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR86    CE_CAN_16.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR87    CE_CAN_16.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR88    CE_CAN_16.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR89    CE_CAN_16.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR90    CE_CAN_16.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR91    CE_CAN_16.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR92    CE_CAN_16.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR93    CE_CAN_16.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR94    CE_CAN_16.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR95    CE_CAN_16.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR96    CE_CAN_16.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR97    CE_CAN_16.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR98    CE_CAN_16.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR99    CE_CAN_16.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_16_RXIMR100   CE_CAN_16.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR101   CE_CAN_16.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR102   CE_CAN_16.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR103   CE_CAN_16.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR104   CE_CAN_16.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR105   CE_CAN_16.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR106   CE_CAN_16.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR107   CE_CAN_16.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR108   CE_CAN_16.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR109   CE_CAN_16.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR110   CE_CAN_16.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR111   CE_CAN_16.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR112   CE_CAN_16.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR113   CE_CAN_16.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR114   CE_CAN_16.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR115   CE_CAN_16.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR116   CE_CAN_16.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR117   CE_CAN_16.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR118   CE_CAN_16.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR119   CE_CAN_16.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR120   CE_CAN_16.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR121   CE_CAN_16.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR122   CE_CAN_16.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR123   CE_CAN_16.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR124   CE_CAN_16.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR125   CE_CAN_16.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR126   CE_CAN_16.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_16_RXIMR127   CE_CAN_16.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_16_MECR       CE_CAN_16.MECR.R              /* Memory Error Control */
#define CE_CAN_16_ERRIAR     CE_CAN_16.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_16_ERRIDPR    CE_CAN_16.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_16_ERRIPPR    CE_CAN_16.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_16_RERRAR     CE_CAN_16.RERRAR.R            /* Error Report Address */
#define CE_CAN_16_RERRDR     CE_CAN_16.RERRDR.R            /* Error Report Data */
#define CE_CAN_16_RERRSYNR   CE_CAN_16.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_16_ERRSR      CE_CAN_16.ERRSR.R             /* Error Status */
#define CE_CAN_16_EPRS       CE_CAN_16.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_16_ENCBT      CE_CAN_16.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_16_EDCBT      CE_CAN_16.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_16_ETDC       CE_CAN_16.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_16_FDCTRL     CE_CAN_16.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_16_FDCBT      CE_CAN_16.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_16_FDCRC      CE_CAN_16.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_16_ERFCR      CE_CAN_16.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_16_ERFIER     CE_CAN_16.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_16_ERFSR      CE_CAN_16.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_16_HR_TIME_STAMP0 CE_CAN_16.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP1 CE_CAN_16.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP2 CE_CAN_16.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP3 CE_CAN_16.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP4 CE_CAN_16.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP5 CE_CAN_16.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP6 CE_CAN_16.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP7 CE_CAN_16.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP8 CE_CAN_16.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP9 CE_CAN_16.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP10 CE_CAN_16.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP11 CE_CAN_16.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP12 CE_CAN_16.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP13 CE_CAN_16.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP14 CE_CAN_16.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP15 CE_CAN_16.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP16 CE_CAN_16.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP17 CE_CAN_16.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP18 CE_CAN_16.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP19 CE_CAN_16.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP20 CE_CAN_16.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP21 CE_CAN_16.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP22 CE_CAN_16.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP23 CE_CAN_16.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP24 CE_CAN_16.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP25 CE_CAN_16.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP26 CE_CAN_16.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP27 CE_CAN_16.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP28 CE_CAN_16.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP29 CE_CAN_16.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP30 CE_CAN_16.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP31 CE_CAN_16.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP32 CE_CAN_16.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP33 CE_CAN_16.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP34 CE_CAN_16.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP35 CE_CAN_16.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP36 CE_CAN_16.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP37 CE_CAN_16.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP38 CE_CAN_16.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP39 CE_CAN_16.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP40 CE_CAN_16.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP41 CE_CAN_16.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP42 CE_CAN_16.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP43 CE_CAN_16.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP44 CE_CAN_16.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP45 CE_CAN_16.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP46 CE_CAN_16.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP47 CE_CAN_16.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP48 CE_CAN_16.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP49 CE_CAN_16.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP50 CE_CAN_16.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP51 CE_CAN_16.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP52 CE_CAN_16.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP53 CE_CAN_16.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP54 CE_CAN_16.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP55 CE_CAN_16.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP56 CE_CAN_16.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP57 CE_CAN_16.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP58 CE_CAN_16.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP59 CE_CAN_16.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP60 CE_CAN_16.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP61 CE_CAN_16.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP62 CE_CAN_16.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP63 CE_CAN_16.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP64 CE_CAN_16.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP65 CE_CAN_16.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP66 CE_CAN_16.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP67 CE_CAN_16.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP68 CE_CAN_16.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP69 CE_CAN_16.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP70 CE_CAN_16.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP71 CE_CAN_16.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP72 CE_CAN_16.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP73 CE_CAN_16.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP74 CE_CAN_16.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP75 CE_CAN_16.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP76 CE_CAN_16.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP77 CE_CAN_16.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP78 CE_CAN_16.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP79 CE_CAN_16.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP80 CE_CAN_16.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP81 CE_CAN_16.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP82 CE_CAN_16.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP83 CE_CAN_16.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP84 CE_CAN_16.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP85 CE_CAN_16.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP86 CE_CAN_16.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP87 CE_CAN_16.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP88 CE_CAN_16.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP89 CE_CAN_16.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP90 CE_CAN_16.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP91 CE_CAN_16.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP92 CE_CAN_16.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP93 CE_CAN_16.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP94 CE_CAN_16.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP95 CE_CAN_16.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP96 CE_CAN_16.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP97 CE_CAN_16.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP98 CE_CAN_16.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP99 CE_CAN_16.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP100 CE_CAN_16.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP101 CE_CAN_16.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP102 CE_CAN_16.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP103 CE_CAN_16.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP104 CE_CAN_16.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP105 CE_CAN_16.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP106 CE_CAN_16.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP107 CE_CAN_16.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP108 CE_CAN_16.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP109 CE_CAN_16.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP110 CE_CAN_16.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP111 CE_CAN_16.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP112 CE_CAN_16.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP113 CE_CAN_16.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP114 CE_CAN_16.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP115 CE_CAN_16.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP116 CE_CAN_16.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP117 CE_CAN_16.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP118 CE_CAN_16.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP119 CE_CAN_16.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP120 CE_CAN_16.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP121 CE_CAN_16.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP122 CE_CAN_16.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP123 CE_CAN_16.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP124 CE_CAN_16.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP125 CE_CAN_16.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP126 CE_CAN_16.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_16_HR_TIME_STAMP127 CE_CAN_16.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_16_ERFFEL0    CE_CAN_16.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL1    CE_CAN_16.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL2    CE_CAN_16.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL3    CE_CAN_16.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL4    CE_CAN_16.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL5    CE_CAN_16.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL6    CE_CAN_16.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL7    CE_CAN_16.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL8    CE_CAN_16.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL9    CE_CAN_16.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL10   CE_CAN_16.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL11   CE_CAN_16.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL12   CE_CAN_16.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL13   CE_CAN_16.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL14   CE_CAN_16.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL15   CE_CAN_16.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL16   CE_CAN_16.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL17   CE_CAN_16.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL18   CE_CAN_16.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL19   CE_CAN_16.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL20   CE_CAN_16.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL21   CE_CAN_16.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL22   CE_CAN_16.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL23   CE_CAN_16.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL24   CE_CAN_16.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL25   CE_CAN_16.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL26   CE_CAN_16.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL27   CE_CAN_16.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL28   CE_CAN_16.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL29   CE_CAN_16.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL30   CE_CAN_16.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL31   CE_CAN_16.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL32   CE_CAN_16.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL33   CE_CAN_16.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL34   CE_CAN_16.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL35   CE_CAN_16.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL36   CE_CAN_16.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL37   CE_CAN_16.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL38   CE_CAN_16.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL39   CE_CAN_16.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL40   CE_CAN_16.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL41   CE_CAN_16.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL42   CE_CAN_16.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL43   CE_CAN_16.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL44   CE_CAN_16.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL45   CE_CAN_16.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL46   CE_CAN_16.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL47   CE_CAN_16.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL48   CE_CAN_16.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL49   CE_CAN_16.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL50   CE_CAN_16.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL51   CE_CAN_16.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL52   CE_CAN_16.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL53   CE_CAN_16.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL54   CE_CAN_16.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL55   CE_CAN_16.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL56   CE_CAN_16.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL57   CE_CAN_16.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL58   CE_CAN_16.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL59   CE_CAN_16.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL60   CE_CAN_16.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL61   CE_CAN_16.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL62   CE_CAN_16.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL63   CE_CAN_16.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL64   CE_CAN_16.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL65   CE_CAN_16.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL66   CE_CAN_16.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL67   CE_CAN_16.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL68   CE_CAN_16.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL69   CE_CAN_16.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL70   CE_CAN_16.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL71   CE_CAN_16.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL72   CE_CAN_16.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL73   CE_CAN_16.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL74   CE_CAN_16.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL75   CE_CAN_16.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL76   CE_CAN_16.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL77   CE_CAN_16.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL78   CE_CAN_16.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL79   CE_CAN_16.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL80   CE_CAN_16.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL81   CE_CAN_16.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL82   CE_CAN_16.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL83   CE_CAN_16.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL84   CE_CAN_16.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL85   CE_CAN_16.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL86   CE_CAN_16.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL87   CE_CAN_16.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL88   CE_CAN_16.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL89   CE_CAN_16.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL90   CE_CAN_16.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL91   CE_CAN_16.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL92   CE_CAN_16.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL93   CE_CAN_16.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL94   CE_CAN_16.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL95   CE_CAN_16.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL96   CE_CAN_16.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL97   CE_CAN_16.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL98   CE_CAN_16.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL99   CE_CAN_16.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL100  CE_CAN_16.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL101  CE_CAN_16.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL102  CE_CAN_16.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL103  CE_CAN_16.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL104  CE_CAN_16.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL105  CE_CAN_16.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL106  CE_CAN_16.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL107  CE_CAN_16.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL108  CE_CAN_16.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL109  CE_CAN_16.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL110  CE_CAN_16.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL111  CE_CAN_16.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL112  CE_CAN_16.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL113  CE_CAN_16.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL114  CE_CAN_16.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL115  CE_CAN_16.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL116  CE_CAN_16.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL117  CE_CAN_16.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL118  CE_CAN_16.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL119  CE_CAN_16.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL120  CE_CAN_16.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL121  CE_CAN_16.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL122  CE_CAN_16.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL123  CE_CAN_16.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL124  CE_CAN_16.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL125  CE_CAN_16.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL126  CE_CAN_16.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_16_ERFFEL127  CE_CAN_16.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_17_MCR        CE_CAN_17.MCR.R               /* Module Configuration */
#define CE_CAN_17_CTRL1      CE_CAN_17.CTRL1.R             /* Control 1 */
#define CE_CAN_17_TIMER      CE_CAN_17.TIMER.R             /* Free-Running Timer */
#define CE_CAN_17_RXMGMASK   CE_CAN_17.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_17_RX14MASK   CE_CAN_17.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_17_RX15MASK   CE_CAN_17.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_17_ECR        CE_CAN_17.ECR.R               /* Error Counter */
#define CE_CAN_17_ESR1       CE_CAN_17.ESR1.R              /* Error and Status 1 */
#define CE_CAN_17_IMASK2     CE_CAN_17.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_17_IMASK1     CE_CAN_17.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_17_IFLAG2     CE_CAN_17.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_17_IFLAG1     CE_CAN_17.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_17_CTRL2      CE_CAN_17.CTRL2.R             /* Control 2 */
#define CE_CAN_17_ESR2       CE_CAN_17.ESR2.R              /* Error and Status 2 */
#define CE_CAN_17_CRCR       CE_CAN_17.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_17_RXFGMASK   CE_CAN_17.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_17_RXFIR      CE_CAN_17.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_17_CBT        CE_CAN_17.CBT.R               /* CAN Bit Timing */
#define CE_CAN_17_IMASK4     CE_CAN_17.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_17_IMASK3     CE_CAN_17.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_17_IFLAG4     CE_CAN_17.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_17_IFLAG3     CE_CAN_17.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_17_CS0            CE_CAN_17.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_17_ID0            CE_CAN_17.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_17_B0             CE_CAN_17.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_17_H0             CE_CAN_17.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_17_W0             CE_CAN_17.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_17_CS1            CE_CAN_17.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_17_ID1            CE_CAN_17.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_17_B1             CE_CAN_17.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_17_H1             CE_CAN_17.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_17_W1             CE_CAN_17.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_17_CS2            CE_CAN_17.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_17_ID2            CE_CAN_17.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_17_B2             CE_CAN_17.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_17_H2             CE_CAN_17.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_17_W2             CE_CAN_17.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_17_CS3            CE_CAN_17.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_17_ID3            CE_CAN_17.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_17_B3             CE_CAN_17.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_17_H3             CE_CAN_17.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_17_W3             CE_CAN_17.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_17_CS4            CE_CAN_17.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_17_ID4            CE_CAN_17.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_17_B4             CE_CAN_17.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_17_H4             CE_CAN_17.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_17_W4             CE_CAN_17.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_17_CS5            CE_CAN_17.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_17_ID5            CE_CAN_17.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_17_B5             CE_CAN_17.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_17_H5             CE_CAN_17.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_17_W5             CE_CAN_17.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_17_CS6            CE_CAN_17.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_17_ID6            CE_CAN_17.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_17_B6             CE_CAN_17.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_17_H6             CE_CAN_17.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_17_W6             CE_CAN_17.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_17_CS7            CE_CAN_17.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_17_ID7            CE_CAN_17.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_17_B7             CE_CAN_17.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_17_H7             CE_CAN_17.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_17_W7             CE_CAN_17.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_17_CS8            CE_CAN_17.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_17_ID8            CE_CAN_17.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_17_B8             CE_CAN_17.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_17_H8             CE_CAN_17.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_17_W8             CE_CAN_17.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_17_CS9            CE_CAN_17.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_17_ID9            CE_CAN_17.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_17_B9             CE_CAN_17.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_17_H9             CE_CAN_17.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_17_W9             CE_CAN_17.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_17_CS10           CE_CAN_17.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_17_ID10           CE_CAN_17.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_17_B10            CE_CAN_17.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_17_H10            CE_CAN_17.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_17_W10            CE_CAN_17.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_17_CS11           CE_CAN_17.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_17_ID11           CE_CAN_17.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_17_B11            CE_CAN_17.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_17_H11            CE_CAN_17.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_17_W11            CE_CAN_17.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_17_CS12           CE_CAN_17.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_17_ID12           CE_CAN_17.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_17_B12            CE_CAN_17.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_17_H12            CE_CAN_17.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_17_W12            CE_CAN_17.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_17_CS13           CE_CAN_17.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_17_ID13           CE_CAN_17.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_17_B13            CE_CAN_17.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_17_H13            CE_CAN_17.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_17_W13            CE_CAN_17.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_17_CS14           CE_CAN_17.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_17_ID14           CE_CAN_17.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_17_B14            CE_CAN_17.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_17_H14            CE_CAN_17.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_17_W14            CE_CAN_17.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_17_CS15           CE_CAN_17.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_17_ID15           CE_CAN_17.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_17_B15            CE_CAN_17.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_17_H15            CE_CAN_17.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_17_W15            CE_CAN_17.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_17_CS16           CE_CAN_17.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_17_ID16           CE_CAN_17.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_17_B16            CE_CAN_17.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_17_H16            CE_CAN_17.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_17_W16            CE_CAN_17.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_17_CS17           CE_CAN_17.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_17_ID17           CE_CAN_17.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_17_B17            CE_CAN_17.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_17_H17            CE_CAN_17.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_17_W17            CE_CAN_17.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_17_CS18           CE_CAN_17.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_17_ID18           CE_CAN_17.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_17_B18            CE_CAN_17.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_17_H18            CE_CAN_17.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_17_W18            CE_CAN_17.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_17_CS19           CE_CAN_17.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_17_ID19           CE_CAN_17.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_17_B19            CE_CAN_17.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_17_H19            CE_CAN_17.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_17_W19            CE_CAN_17.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_17_CS20           CE_CAN_17.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_17_ID20           CE_CAN_17.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_17_B20            CE_CAN_17.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_17_H20            CE_CAN_17.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_17_W20            CE_CAN_17.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_17_CS21           CE_CAN_17.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_17_ID21           CE_CAN_17.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_17_B21            CE_CAN_17.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_17_H21            CE_CAN_17.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_17_W21            CE_CAN_17.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_17_CS22           CE_CAN_17.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_17_ID22           CE_CAN_17.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_17_B22            CE_CAN_17.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_17_H22            CE_CAN_17.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_17_W22            CE_CAN_17.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_17_CS23           CE_CAN_17.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_17_ID23           CE_CAN_17.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_17_B23            CE_CAN_17.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_17_H23            CE_CAN_17.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_17_W23            CE_CAN_17.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_17_CS24           CE_CAN_17.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_17_ID24           CE_CAN_17.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_17_B24            CE_CAN_17.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_17_H24            CE_CAN_17.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_17_W24            CE_CAN_17.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_17_CS25           CE_CAN_17.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_17_ID25           CE_CAN_17.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_17_B25            CE_CAN_17.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_17_H25            CE_CAN_17.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_17_W25            CE_CAN_17.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_17_CS26           CE_CAN_17.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_17_ID26           CE_CAN_17.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_17_B26            CE_CAN_17.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_17_H26            CE_CAN_17.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_17_W26            CE_CAN_17.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_17_CS27           CE_CAN_17.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_17_ID27           CE_CAN_17.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_17_B27            CE_CAN_17.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_17_H27            CE_CAN_17.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_17_W27            CE_CAN_17.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_17_CS28           CE_CAN_17.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_17_ID28           CE_CAN_17.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_17_B28            CE_CAN_17.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_17_H28            CE_CAN_17.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_17_W28            CE_CAN_17.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_17_CS29           CE_CAN_17.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_17_ID29           CE_CAN_17.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_17_B29            CE_CAN_17.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_17_H29            CE_CAN_17.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_17_W29            CE_CAN_17.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_17_CS30           CE_CAN_17.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_17_ID30           CE_CAN_17.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_17_B30            CE_CAN_17.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_17_H30            CE_CAN_17.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_17_W30            CE_CAN_17.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_17_CS31           CE_CAN_17.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_17_ID31           CE_CAN_17.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_17_B31            CE_CAN_17.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_17_H31            CE_CAN_17.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_17_W31            CE_CAN_17.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_17_CS32           CE_CAN_17.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_17_ID32           CE_CAN_17.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_17_B32            CE_CAN_17.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_17_H32            CE_CAN_17.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_17_W32            CE_CAN_17.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_17_CS33           CE_CAN_17.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_17_ID33           CE_CAN_17.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_17_B33            CE_CAN_17.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_17_H33            CE_CAN_17.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_17_W33            CE_CAN_17.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_17_CS34           CE_CAN_17.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_17_ID34           CE_CAN_17.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_17_B34            CE_CAN_17.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_17_H34            CE_CAN_17.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_17_W34            CE_CAN_17.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_17_CS35           CE_CAN_17.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_17_ID35           CE_CAN_17.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_17_B35            CE_CAN_17.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_17_H35            CE_CAN_17.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_17_W35            CE_CAN_17.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_17_CS36           CE_CAN_17.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_17_ID36           CE_CAN_17.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_17_B36            CE_CAN_17.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_17_H36            CE_CAN_17.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_17_W36            CE_CAN_17.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_17_CS37           CE_CAN_17.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_17_ID37           CE_CAN_17.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_17_B37            CE_CAN_17.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_17_H37            CE_CAN_17.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_17_W37            CE_CAN_17.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_17_CS38           CE_CAN_17.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_17_ID38           CE_CAN_17.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_17_B38            CE_CAN_17.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_17_H38            CE_CAN_17.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_17_W38            CE_CAN_17.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_17_CS39           CE_CAN_17.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_17_ID39           CE_CAN_17.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_17_B39            CE_CAN_17.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_17_H39            CE_CAN_17.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_17_W39            CE_CAN_17.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_17_CS40           CE_CAN_17.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_17_ID40           CE_CAN_17.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_17_B40            CE_CAN_17.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_17_H40            CE_CAN_17.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_17_W40            CE_CAN_17.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_17_CS41           CE_CAN_17.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_17_ID41           CE_CAN_17.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_17_B41            CE_CAN_17.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_17_H41            CE_CAN_17.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_17_W41            CE_CAN_17.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_17_CS42           CE_CAN_17.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_17_ID42           CE_CAN_17.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_17_B42            CE_CAN_17.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_17_H42            CE_CAN_17.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_17_W42            CE_CAN_17.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_17_CS43           CE_CAN_17.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_17_ID43           CE_CAN_17.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_17_B43            CE_CAN_17.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_17_H43            CE_CAN_17.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_17_W43            CE_CAN_17.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_17_CS44           CE_CAN_17.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_17_ID44           CE_CAN_17.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_17_B44            CE_CAN_17.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_17_H44            CE_CAN_17.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_17_W44            CE_CAN_17.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_17_CS45           CE_CAN_17.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_17_ID45           CE_CAN_17.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_17_B45            CE_CAN_17.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_17_H45            CE_CAN_17.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_17_W45            CE_CAN_17.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_17_CS46           CE_CAN_17.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_17_ID46           CE_CAN_17.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_17_B46            CE_CAN_17.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_17_H46            CE_CAN_17.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_17_W46            CE_CAN_17.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_17_CS47           CE_CAN_17.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_17_ID47           CE_CAN_17.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_17_B47            CE_CAN_17.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_17_H47            CE_CAN_17.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_17_W47            CE_CAN_17.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_17_CS48           CE_CAN_17.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_17_ID48           CE_CAN_17.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_17_B48            CE_CAN_17.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_17_H48            CE_CAN_17.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_17_W48            CE_CAN_17.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_17_CS49           CE_CAN_17.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_17_ID49           CE_CAN_17.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_17_B49            CE_CAN_17.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_17_H49            CE_CAN_17.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_17_W49            CE_CAN_17.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_17_CS50           CE_CAN_17.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_17_ID50           CE_CAN_17.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_17_B50            CE_CAN_17.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_17_H50            CE_CAN_17.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_17_W50            CE_CAN_17.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_17_CS51           CE_CAN_17.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_17_ID51           CE_CAN_17.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_17_B51            CE_CAN_17.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_17_H51            CE_CAN_17.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_17_W51            CE_CAN_17.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_17_CS52           CE_CAN_17.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_17_ID52           CE_CAN_17.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_17_B52            CE_CAN_17.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_17_H52            CE_CAN_17.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_17_W52            CE_CAN_17.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_17_CS53           CE_CAN_17.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_17_ID53           CE_CAN_17.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_17_B53            CE_CAN_17.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_17_H53            CE_CAN_17.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_17_W53            CE_CAN_17.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_17_CS54           CE_CAN_17.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_17_ID54           CE_CAN_17.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_17_B54            CE_CAN_17.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_17_H54            CE_CAN_17.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_17_W54            CE_CAN_17.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_17_CS55           CE_CAN_17.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_17_ID55           CE_CAN_17.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_17_B55            CE_CAN_17.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_17_H55            CE_CAN_17.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_17_W55            CE_CAN_17.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_17_CS56           CE_CAN_17.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_17_ID56           CE_CAN_17.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_17_B56            CE_CAN_17.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_17_H56            CE_CAN_17.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_17_W56            CE_CAN_17.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_17_CS57           CE_CAN_17.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_17_ID57           CE_CAN_17.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_17_B57            CE_CAN_17.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_17_H57            CE_CAN_17.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_17_W57            CE_CAN_17.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_17_CS58           CE_CAN_17.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_17_ID58           CE_CAN_17.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_17_B58            CE_CAN_17.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_17_H58            CE_CAN_17.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_17_W58            CE_CAN_17.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_17_CS59           CE_CAN_17.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_17_ID59           CE_CAN_17.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_17_B59            CE_CAN_17.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_17_H59            CE_CAN_17.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_17_W59            CE_CAN_17.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_17_CS60           CE_CAN_17.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_17_ID60           CE_CAN_17.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_17_B60            CE_CAN_17.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_17_H60            CE_CAN_17.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_17_W60            CE_CAN_17.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_17_CS61           CE_CAN_17.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_17_ID61           CE_CAN_17.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_17_B61            CE_CAN_17.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_17_H61            CE_CAN_17.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_17_W61            CE_CAN_17.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_17_CS62           CE_CAN_17.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_17_ID62           CE_CAN_17.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_17_B62            CE_CAN_17.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_17_H62            CE_CAN_17.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_17_W62            CE_CAN_17.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_17_CS63           CE_CAN_17.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_17_ID63           CE_CAN_17.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_17_B63            CE_CAN_17.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_17_H63            CE_CAN_17.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_17_W63            CE_CAN_17.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_17_CS64           CE_CAN_17.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_17_ID64           CE_CAN_17.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_17_B64            CE_CAN_17.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_17_H64            CE_CAN_17.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_17_W64            CE_CAN_17.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_17_CS65           CE_CAN_17.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_17_ID65           CE_CAN_17.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_17_B65            CE_CAN_17.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_17_H65            CE_CAN_17.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_17_W65            CE_CAN_17.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_17_CS66           CE_CAN_17.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_17_ID66           CE_CAN_17.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_17_B66            CE_CAN_17.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_17_H66            CE_CAN_17.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_17_W66            CE_CAN_17.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_17_CS67           CE_CAN_17.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_17_ID67           CE_CAN_17.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_17_B67            CE_CAN_17.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_17_H67            CE_CAN_17.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_17_W67            CE_CAN_17.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_17_CS68           CE_CAN_17.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_17_ID68           CE_CAN_17.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_17_B68            CE_CAN_17.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_17_H68            CE_CAN_17.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_17_W68            CE_CAN_17.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_17_CS69           CE_CAN_17.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_17_ID69           CE_CAN_17.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_17_B69            CE_CAN_17.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_17_H69            CE_CAN_17.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_17_W69            CE_CAN_17.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_17_CS70           CE_CAN_17.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_17_ID70           CE_CAN_17.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_17_B70            CE_CAN_17.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_17_H70            CE_CAN_17.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_17_W70            CE_CAN_17.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_17_CS71           CE_CAN_17.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_17_ID71           CE_CAN_17.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_17_B71            CE_CAN_17.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_17_H71            CE_CAN_17.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_17_W71            CE_CAN_17.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_17_CS72           CE_CAN_17.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_17_ID72           CE_CAN_17.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_17_B72            CE_CAN_17.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_17_H72            CE_CAN_17.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_17_W72            CE_CAN_17.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_17_CS73           CE_CAN_17.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_17_ID73           CE_CAN_17.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_17_B73            CE_CAN_17.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_17_H73            CE_CAN_17.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_17_W73            CE_CAN_17.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_17_CS74           CE_CAN_17.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_17_ID74           CE_CAN_17.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_17_B74            CE_CAN_17.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_17_H74            CE_CAN_17.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_17_W74            CE_CAN_17.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_17_CS75           CE_CAN_17.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_17_ID75           CE_CAN_17.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_17_B75            CE_CAN_17.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_17_H75            CE_CAN_17.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_17_W75            CE_CAN_17.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_17_CS76           CE_CAN_17.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_17_ID76           CE_CAN_17.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_17_B76            CE_CAN_17.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_17_H76            CE_CAN_17.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_17_W76            CE_CAN_17.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_17_CS77           CE_CAN_17.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_17_ID77           CE_CAN_17.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_17_B77            CE_CAN_17.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_17_H77            CE_CAN_17.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_17_W77            CE_CAN_17.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_17_CS78           CE_CAN_17.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_17_ID78           CE_CAN_17.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_17_B78            CE_CAN_17.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_17_H78            CE_CAN_17.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_17_W78            CE_CAN_17.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_17_CS79           CE_CAN_17.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_17_ID79           CE_CAN_17.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_17_B79            CE_CAN_17.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_17_H79            CE_CAN_17.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_17_W79            CE_CAN_17.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_17_CS80           CE_CAN_17.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_17_ID80           CE_CAN_17.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_17_B80            CE_CAN_17.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_17_H80            CE_CAN_17.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_17_W80            CE_CAN_17.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_17_CS81           CE_CAN_17.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_17_ID81           CE_CAN_17.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_17_B81            CE_CAN_17.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_17_H81            CE_CAN_17.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_17_W81            CE_CAN_17.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_17_CS82           CE_CAN_17.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_17_ID82           CE_CAN_17.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_17_B82            CE_CAN_17.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_17_H82            CE_CAN_17.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_17_W82            CE_CAN_17.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_17_CS83           CE_CAN_17.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_17_ID83           CE_CAN_17.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_17_B83            CE_CAN_17.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_17_H83            CE_CAN_17.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_17_W83            CE_CAN_17.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_17_CS84           CE_CAN_17.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_17_ID84           CE_CAN_17.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_17_B84            CE_CAN_17.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_17_H84            CE_CAN_17.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_17_W84            CE_CAN_17.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_17_CS85           CE_CAN_17.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_17_ID85           CE_CAN_17.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_17_B85            CE_CAN_17.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_17_H85            CE_CAN_17.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_17_W85            CE_CAN_17.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_17_CS86           CE_CAN_17.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_17_ID86           CE_CAN_17.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_17_B86            CE_CAN_17.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_17_H86            CE_CAN_17.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_17_W86            CE_CAN_17.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_17_CS87           CE_CAN_17.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_17_ID87           CE_CAN_17.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_17_B87            CE_CAN_17.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_17_H87            CE_CAN_17.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_17_W87            CE_CAN_17.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_17_CS88           CE_CAN_17.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_17_ID88           CE_CAN_17.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_17_B88            CE_CAN_17.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_17_H88            CE_CAN_17.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_17_W848           CE_CAN_17.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_17_CS89           CE_CAN_17.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_17_ID89           CE_CAN_17.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_17_B89            CE_CAN_17.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_17_H89            CE_CAN_17.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_17_W89            CE_CAN_17.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_17_CS90           CE_CAN_17.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_17_ID90           CE_CAN_17.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_17_B90            CE_CAN_17.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_17_H90            CE_CAN_17.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_17_W90            CE_CAN_17.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_17_CS91           CE_CAN_17.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_17_ID91           CE_CAN_17.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_17_B91            CE_CAN_17.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_17_H91            CE_CAN_17.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_17_W91            CE_CAN_17.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_17_CS92           CE_CAN_17.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_17_ID92           CE_CAN_17.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_17_B92            CE_CAN_17.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_17_H92            CE_CAN_17.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_17_W92            CE_CAN_17.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_17_CS93           CE_CAN_17.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_17_ID93           CE_CAN_17.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_17_B93            CE_CAN_17.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_17_H93            CE_CAN_17.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_17_W93            CE_CAN_17.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_17_CS94           CE_CAN_17.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_17_ID94           CE_CAN_17.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_17_B94            CE_CAN_17.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_17_H94            CE_CAN_17.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_17_W94            CE_CAN_17.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_17_CS95           CE_CAN_17.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_17_ID95           CE_CAN_17.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_17_B95            CE_CAN_17.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_17_H95            CE_CAN_17.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_17_W95            CE_CAN_17.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_17_CS96           CE_CAN_17.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_17_ID96           CE_CAN_17.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_17_B96            CE_CAN_17.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_17_H96            CE_CAN_17.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_17_W96            CE_CAN_17.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_17_CS97           CE_CAN_17.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_17_ID97           CE_CAN_17.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_17_B97            CE_CAN_17.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_17_H97            CE_CAN_17.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_17_W97            CE_CAN_17.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_17_CS98           CE_CAN_17.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_17_ID98           CE_CAN_17.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_17_B98            CE_CAN_17.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_17_H98            CE_CAN_17.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_17_W98            CE_CAN_17.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_17_CS99           CE_CAN_17.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_17_ID99           CE_CAN_17.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_17_B99            CE_CAN_17.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_17_H99            CE_CAN_17.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_17_W99            CE_CAN_17.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_17_CS100          CE_CAN_17.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_17_ID100          CE_CAN_17.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_17_B100           CE_CAN_17.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_17_H100           CE_CAN_17.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_17_W100           CE_CAN_17.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_17_CS101          CE_CAN_17.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_17_ID101          CE_CAN_17.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_17_B101           CE_CAN_17.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_17_H101           CE_CAN_17.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_17_W101           CE_CAN_17.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_17_CS102          CE_CAN_17.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_17_ID102          CE_CAN_17.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_17_B102           CE_CAN_17.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_17_H102           CE_CAN_17.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_17_W102           CE_CAN_17.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_17_CS103          CE_CAN_17.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_17_ID103          CE_CAN_17.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_17_B103           CE_CAN_17.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_17_H103           CE_CAN_17.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_17_W103           CE_CAN_17.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_17_CS104          CE_CAN_17.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_17_ID104          CE_CAN_17.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_17_B104           CE_CAN_17.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_17_H104           CE_CAN_17.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_17_W104           CE_CAN_17.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_17_CS105          CE_CAN_17.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_17_ID105          CE_CAN_17.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_17_B105           CE_CAN_17.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_17_H105           CE_CAN_17.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_17_W105           CE_CAN_17.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_17_CS106          CE_CAN_17.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_17_ID106          CE_CAN_17.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_17_B106           CE_CAN_17.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_17_H106           CE_CAN_17.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_17_W106           CE_CAN_17.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_17_CS107          CE_CAN_17.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_17_ID107          CE_CAN_17.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_17_B107           CE_CAN_17.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_17_H107           CE_CAN_17.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_17_W107           CE_CAN_17.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_17_CS108          CE_CAN_17.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_17_ID108          CE_CAN_17.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_17_B108           CE_CAN_17.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_17_H108           CE_CAN_17.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_17_W108           CE_CAN_17.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_17_CS109          CE_CAN_17.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_17_ID109          CE_CAN_17.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_17_B109           CE_CAN_17.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_17_H109           CE_CAN_17.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_17_W109           CE_CAN_17.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_17_CS110          CE_CAN_17.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_17_ID110          CE_CAN_17.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_17_B110           CE_CAN_17.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_17_H110           CE_CAN_17.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_17_W110           CE_CAN_17.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_17_CS111          CE_CAN_17.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_17_ID111          CE_CAN_17.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_17_B111           CE_CAN_17.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_17_H111           CE_CAN_17.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_17_W111           CE_CAN_17.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_17_CS112          CE_CAN_17.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_17_ID112          CE_CAN_17.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_17_B112           CE_CAN_17.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_17_H112           CE_CAN_17.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_17_W112           CE_CAN_17.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_17_CS113          CE_CAN_17.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_17_ID113          CE_CAN_17.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_17_B113           CE_CAN_17.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_17_H113           CE_CAN_17.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_17_W113           CE_CAN_17.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_17_CS114          CE_CAN_17.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_17_ID114          CE_CAN_17.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_17_B114           CE_CAN_17.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_17_H114           CE_CAN_17.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_17_W114           CE_CAN_17.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_17_CS115          CE_CAN_17.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_17_ID115          CE_CAN_17.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_17_B115           CE_CAN_17.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_17_H115           CE_CAN_17.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_17_W115           CE_CAN_17.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_17_CS116          CE_CAN_17.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_17_ID116          CE_CAN_17.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_17_B116           CE_CAN_17.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_17_H116           CE_CAN_17.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_17_W116           CE_CAN_17.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_17_CS117          CE_CAN_17.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_17_ID117          CE_CAN_17.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_17_B117           CE_CAN_17.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_17_H117           CE_CAN_17.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_17_W117           CE_CAN_17.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_17_CS118          CE_CAN_17.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_17_ID118          CE_CAN_17.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_17_B118           CE_CAN_17.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_17_H118           CE_CAN_17.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_17_W118           CE_CAN_17.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_17_CS119          CE_CAN_17.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_17_ID119          CE_CAN_17.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_17_B119           CE_CAN_17.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_17_H119           CE_CAN_17.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_17_W119           CE_CAN_17.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_17_CS120          CE_CAN_17.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_17_ID120          CE_CAN_17.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_17_B120           CE_CAN_17.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_17_H120           CE_CAN_17.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_17_W120           CE_CAN_17.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_17_CS121          CE_CAN_17.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_17_ID121          CE_CAN_17.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_17_B121           CE_CAN_17.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_17_H121           CE_CAN_17.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_17_W121           CE_CAN_17.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_17_CS122          CE_CAN_17.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_17_ID122          CE_CAN_17.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_17_B122           CE_CAN_17.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_17_H122           CE_CAN_17.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_17_W122           CE_CAN_17.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_17_CS123          CE_CAN_17.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_17_ID123          CE_CAN_17.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_17_B123           CE_CAN_17.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_17_H123           CE_CAN_17.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_17_W123           CE_CAN_17.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_17_CS124          CE_CAN_17.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_17_ID124          CE_CAN_17.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_17_B124           CE_CAN_17.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_17_H124           CE_CAN_17.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_17_W124           CE_CAN_17.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_17_CS125          CE_CAN_17.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_17_ID125          CE_CAN_17.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_17_B125           CE_CAN_17.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_17_H125           CE_CAN_17.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_17_W125           CE_CAN_17.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_17_CS126          CE_CAN_17.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_17_ID126          CE_CAN_17.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_17_B126           CE_CAN_17.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_17_H126           CE_CAN_17.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_17_W126           CE_CAN_17.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_17_CS127          CE_CAN_17.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_17_ID127          CE_CAN_17.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_17_B127           CE_CAN_17.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_17_H127           CE_CAN_17.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_17_W127           CE_CAN_17.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_17_RXIMR0     CE_CAN_17.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR1     CE_CAN_17.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR2     CE_CAN_17.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR3     CE_CAN_17.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR4     CE_CAN_17.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR5     CE_CAN_17.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR6     CE_CAN_17.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR7     CE_CAN_17.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR8     CE_CAN_17.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR9     CE_CAN_17.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_17_RXIMR10    CE_CAN_17.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR11    CE_CAN_17.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR12    CE_CAN_17.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR13    CE_CAN_17.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR14    CE_CAN_17.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR15    CE_CAN_17.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR16    CE_CAN_17.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR17    CE_CAN_17.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR18    CE_CAN_17.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR19    CE_CAN_17.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR20    CE_CAN_17.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR21    CE_CAN_17.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR22    CE_CAN_17.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR23    CE_CAN_17.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR24    CE_CAN_17.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR25    CE_CAN_17.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR26    CE_CAN_17.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR27    CE_CAN_17.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR28    CE_CAN_17.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR29    CE_CAN_17.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR30    CE_CAN_17.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR31    CE_CAN_17.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR32    CE_CAN_17.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR33    CE_CAN_17.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR34    CE_CAN_17.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR35    CE_CAN_17.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR36    CE_CAN_17.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR37    CE_CAN_17.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR38    CE_CAN_17.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR39    CE_CAN_17.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR40    CE_CAN_17.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR41    CE_CAN_17.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR42    CE_CAN_17.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR43    CE_CAN_17.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR44    CE_CAN_17.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR45    CE_CAN_17.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR46    CE_CAN_17.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR47    CE_CAN_17.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR48    CE_CAN_17.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR49    CE_CAN_17.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR50    CE_CAN_17.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR51    CE_CAN_17.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR52    CE_CAN_17.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR53    CE_CAN_17.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR54    CE_CAN_17.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR55    CE_CAN_17.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR56    CE_CAN_17.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR57    CE_CAN_17.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR58    CE_CAN_17.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR59    CE_CAN_17.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR60    CE_CAN_17.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR61    CE_CAN_17.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR62    CE_CAN_17.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR63    CE_CAN_17.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR64    CE_CAN_17.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR65    CE_CAN_17.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR66    CE_CAN_17.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR67    CE_CAN_17.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR68    CE_CAN_17.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR69    CE_CAN_17.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR70    CE_CAN_17.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR71    CE_CAN_17.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR72    CE_CAN_17.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR73    CE_CAN_17.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR74    CE_CAN_17.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR75    CE_CAN_17.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR76    CE_CAN_17.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR77    CE_CAN_17.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR78    CE_CAN_17.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR79    CE_CAN_17.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR80    CE_CAN_17.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR81    CE_CAN_17.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR82    CE_CAN_17.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR83    CE_CAN_17.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR84    CE_CAN_17.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR85    CE_CAN_17.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR86    CE_CAN_17.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR87    CE_CAN_17.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR88    CE_CAN_17.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR89    CE_CAN_17.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR90    CE_CAN_17.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR91    CE_CAN_17.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR92    CE_CAN_17.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR93    CE_CAN_17.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR94    CE_CAN_17.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR95    CE_CAN_17.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR96    CE_CAN_17.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR97    CE_CAN_17.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR98    CE_CAN_17.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR99    CE_CAN_17.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_17_RXIMR100   CE_CAN_17.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR101   CE_CAN_17.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR102   CE_CAN_17.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR103   CE_CAN_17.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR104   CE_CAN_17.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR105   CE_CAN_17.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR106   CE_CAN_17.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR107   CE_CAN_17.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR108   CE_CAN_17.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR109   CE_CAN_17.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR110   CE_CAN_17.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR111   CE_CAN_17.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR112   CE_CAN_17.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR113   CE_CAN_17.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR114   CE_CAN_17.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR115   CE_CAN_17.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR116   CE_CAN_17.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR117   CE_CAN_17.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR118   CE_CAN_17.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR119   CE_CAN_17.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR120   CE_CAN_17.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR121   CE_CAN_17.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR122   CE_CAN_17.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR123   CE_CAN_17.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR124   CE_CAN_17.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR125   CE_CAN_17.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR126   CE_CAN_17.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_17_RXIMR127   CE_CAN_17.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_17_MECR       CE_CAN_17.MECR.R              /* Memory Error Control */
#define CE_CAN_17_ERRIAR     CE_CAN_17.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_17_ERRIDPR    CE_CAN_17.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_17_ERRIPPR    CE_CAN_17.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_17_RERRAR     CE_CAN_17.RERRAR.R            /* Error Report Address */
#define CE_CAN_17_RERRDR     CE_CAN_17.RERRDR.R            /* Error Report Data */
#define CE_CAN_17_RERRSYNR   CE_CAN_17.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_17_ERRSR      CE_CAN_17.ERRSR.R             /* Error Status */
#define CE_CAN_17_EPRS       CE_CAN_17.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_17_ENCBT      CE_CAN_17.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_17_EDCBT      CE_CAN_17.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_17_ETDC       CE_CAN_17.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_17_FDCTRL     CE_CAN_17.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_17_FDCBT      CE_CAN_17.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_17_FDCRC      CE_CAN_17.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_17_ERFCR      CE_CAN_17.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_17_ERFIER     CE_CAN_17.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_17_ERFSR      CE_CAN_17.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_17_HR_TIME_STAMP0 CE_CAN_17.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP1 CE_CAN_17.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP2 CE_CAN_17.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP3 CE_CAN_17.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP4 CE_CAN_17.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP5 CE_CAN_17.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP6 CE_CAN_17.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP7 CE_CAN_17.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP8 CE_CAN_17.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP9 CE_CAN_17.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP10 CE_CAN_17.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP11 CE_CAN_17.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP12 CE_CAN_17.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP13 CE_CAN_17.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP14 CE_CAN_17.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP15 CE_CAN_17.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP16 CE_CAN_17.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP17 CE_CAN_17.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP18 CE_CAN_17.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP19 CE_CAN_17.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP20 CE_CAN_17.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP21 CE_CAN_17.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP22 CE_CAN_17.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP23 CE_CAN_17.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP24 CE_CAN_17.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP25 CE_CAN_17.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP26 CE_CAN_17.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP27 CE_CAN_17.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP28 CE_CAN_17.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP29 CE_CAN_17.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP30 CE_CAN_17.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP31 CE_CAN_17.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP32 CE_CAN_17.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP33 CE_CAN_17.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP34 CE_CAN_17.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP35 CE_CAN_17.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP36 CE_CAN_17.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP37 CE_CAN_17.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP38 CE_CAN_17.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP39 CE_CAN_17.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP40 CE_CAN_17.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP41 CE_CAN_17.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP42 CE_CAN_17.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP43 CE_CAN_17.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP44 CE_CAN_17.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP45 CE_CAN_17.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP46 CE_CAN_17.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP47 CE_CAN_17.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP48 CE_CAN_17.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP49 CE_CAN_17.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP50 CE_CAN_17.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP51 CE_CAN_17.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP52 CE_CAN_17.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP53 CE_CAN_17.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP54 CE_CAN_17.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP55 CE_CAN_17.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP56 CE_CAN_17.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP57 CE_CAN_17.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP58 CE_CAN_17.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP59 CE_CAN_17.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP60 CE_CAN_17.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP61 CE_CAN_17.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP62 CE_CAN_17.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP63 CE_CAN_17.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP64 CE_CAN_17.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP65 CE_CAN_17.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP66 CE_CAN_17.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP67 CE_CAN_17.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP68 CE_CAN_17.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP69 CE_CAN_17.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP70 CE_CAN_17.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP71 CE_CAN_17.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP72 CE_CAN_17.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP73 CE_CAN_17.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP74 CE_CAN_17.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP75 CE_CAN_17.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP76 CE_CAN_17.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP77 CE_CAN_17.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP78 CE_CAN_17.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP79 CE_CAN_17.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP80 CE_CAN_17.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP81 CE_CAN_17.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP82 CE_CAN_17.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP83 CE_CAN_17.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP84 CE_CAN_17.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP85 CE_CAN_17.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP86 CE_CAN_17.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP87 CE_CAN_17.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP88 CE_CAN_17.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP89 CE_CAN_17.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP90 CE_CAN_17.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP91 CE_CAN_17.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP92 CE_CAN_17.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP93 CE_CAN_17.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP94 CE_CAN_17.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP95 CE_CAN_17.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP96 CE_CAN_17.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP97 CE_CAN_17.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP98 CE_CAN_17.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP99 CE_CAN_17.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP100 CE_CAN_17.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP101 CE_CAN_17.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP102 CE_CAN_17.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP103 CE_CAN_17.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP104 CE_CAN_17.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP105 CE_CAN_17.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP106 CE_CAN_17.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP107 CE_CAN_17.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP108 CE_CAN_17.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP109 CE_CAN_17.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP110 CE_CAN_17.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP111 CE_CAN_17.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP112 CE_CAN_17.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP113 CE_CAN_17.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP114 CE_CAN_17.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP115 CE_CAN_17.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP116 CE_CAN_17.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP117 CE_CAN_17.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP118 CE_CAN_17.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP119 CE_CAN_17.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP120 CE_CAN_17.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP121 CE_CAN_17.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP122 CE_CAN_17.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP123 CE_CAN_17.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP124 CE_CAN_17.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP125 CE_CAN_17.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP126 CE_CAN_17.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_17_HR_TIME_STAMP127 CE_CAN_17.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_17_ERFFEL0    CE_CAN_17.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL1    CE_CAN_17.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL2    CE_CAN_17.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL3    CE_CAN_17.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL4    CE_CAN_17.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL5    CE_CAN_17.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL6    CE_CAN_17.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL7    CE_CAN_17.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL8    CE_CAN_17.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL9    CE_CAN_17.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL10   CE_CAN_17.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL11   CE_CAN_17.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL12   CE_CAN_17.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL13   CE_CAN_17.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL14   CE_CAN_17.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL15   CE_CAN_17.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL16   CE_CAN_17.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL17   CE_CAN_17.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL18   CE_CAN_17.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL19   CE_CAN_17.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL20   CE_CAN_17.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL21   CE_CAN_17.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL22   CE_CAN_17.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL23   CE_CAN_17.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL24   CE_CAN_17.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL25   CE_CAN_17.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL26   CE_CAN_17.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL27   CE_CAN_17.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL28   CE_CAN_17.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL29   CE_CAN_17.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL30   CE_CAN_17.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL31   CE_CAN_17.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL32   CE_CAN_17.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL33   CE_CAN_17.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL34   CE_CAN_17.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL35   CE_CAN_17.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL36   CE_CAN_17.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL37   CE_CAN_17.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL38   CE_CAN_17.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL39   CE_CAN_17.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL40   CE_CAN_17.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL41   CE_CAN_17.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL42   CE_CAN_17.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL43   CE_CAN_17.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL44   CE_CAN_17.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL45   CE_CAN_17.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL46   CE_CAN_17.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL47   CE_CAN_17.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL48   CE_CAN_17.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL49   CE_CAN_17.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL50   CE_CAN_17.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL51   CE_CAN_17.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL52   CE_CAN_17.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL53   CE_CAN_17.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL54   CE_CAN_17.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL55   CE_CAN_17.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL56   CE_CAN_17.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL57   CE_CAN_17.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL58   CE_CAN_17.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL59   CE_CAN_17.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL60   CE_CAN_17.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL61   CE_CAN_17.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL62   CE_CAN_17.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL63   CE_CAN_17.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL64   CE_CAN_17.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL65   CE_CAN_17.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL66   CE_CAN_17.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL67   CE_CAN_17.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL68   CE_CAN_17.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL69   CE_CAN_17.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL70   CE_CAN_17.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL71   CE_CAN_17.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL72   CE_CAN_17.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL73   CE_CAN_17.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL74   CE_CAN_17.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL75   CE_CAN_17.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL76   CE_CAN_17.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL77   CE_CAN_17.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL78   CE_CAN_17.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL79   CE_CAN_17.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL80   CE_CAN_17.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL81   CE_CAN_17.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL82   CE_CAN_17.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL83   CE_CAN_17.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL84   CE_CAN_17.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL85   CE_CAN_17.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL86   CE_CAN_17.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL87   CE_CAN_17.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL88   CE_CAN_17.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL89   CE_CAN_17.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL90   CE_CAN_17.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL91   CE_CAN_17.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL92   CE_CAN_17.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL93   CE_CAN_17.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL94   CE_CAN_17.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL95   CE_CAN_17.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL96   CE_CAN_17.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL97   CE_CAN_17.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL98   CE_CAN_17.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL99   CE_CAN_17.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL100  CE_CAN_17.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL101  CE_CAN_17.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL102  CE_CAN_17.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL103  CE_CAN_17.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL104  CE_CAN_17.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL105  CE_CAN_17.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL106  CE_CAN_17.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL107  CE_CAN_17.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL108  CE_CAN_17.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL109  CE_CAN_17.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL110  CE_CAN_17.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL111  CE_CAN_17.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL112  CE_CAN_17.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL113  CE_CAN_17.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL114  CE_CAN_17.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL115  CE_CAN_17.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL116  CE_CAN_17.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL117  CE_CAN_17.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL118  CE_CAN_17.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL119  CE_CAN_17.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL120  CE_CAN_17.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL121  CE_CAN_17.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL122  CE_CAN_17.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL123  CE_CAN_17.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL124  CE_CAN_17.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL125  CE_CAN_17.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL126  CE_CAN_17.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_17_ERFFEL127  CE_CAN_17.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_18_MCR        CE_CAN_18.MCR.R               /* Module Configuration */
#define CE_CAN_18_CTRL1      CE_CAN_18.CTRL1.R             /* Control 1 */
#define CE_CAN_18_TIMER      CE_CAN_18.TIMER.R             /* Free-Running Timer */
#define CE_CAN_18_RXMGMASK   CE_CAN_18.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_18_RX14MASK   CE_CAN_18.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_18_RX15MASK   CE_CAN_18.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_18_ECR        CE_CAN_18.ECR.R               /* Error Counter */
#define CE_CAN_18_ESR1       CE_CAN_18.ESR1.R              /* Error and Status 1 */
#define CE_CAN_18_IMASK2     CE_CAN_18.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_18_IMASK1     CE_CAN_18.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_18_IFLAG2     CE_CAN_18.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_18_IFLAG1     CE_CAN_18.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_18_CTRL2      CE_CAN_18.CTRL2.R             /* Control 2 */
#define CE_CAN_18_ESR2       CE_CAN_18.ESR2.R              /* Error and Status 2 */
#define CE_CAN_18_CRCR       CE_CAN_18.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_18_RXFGMASK   CE_CAN_18.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_18_RXFIR      CE_CAN_18.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_18_CBT        CE_CAN_18.CBT.R               /* CAN Bit Timing */
#define CE_CAN_18_IMASK4     CE_CAN_18.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_18_IMASK3     CE_CAN_18.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_18_IFLAG4     CE_CAN_18.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_18_IFLAG3     CE_CAN_18.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_18_CS0            CE_CAN_18.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_18_ID0            CE_CAN_18.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_18_B0             CE_CAN_18.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_18_H0             CE_CAN_18.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_18_W0             CE_CAN_18.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_18_CS1            CE_CAN_18.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_18_ID1            CE_CAN_18.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_18_B1             CE_CAN_18.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_18_H1             CE_CAN_18.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_18_W1             CE_CAN_18.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_18_CS2            CE_CAN_18.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_18_ID2            CE_CAN_18.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_18_B2             CE_CAN_18.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_18_H2             CE_CAN_18.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_18_W2             CE_CAN_18.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_18_CS3            CE_CAN_18.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_18_ID3            CE_CAN_18.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_18_B3             CE_CAN_18.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_18_H3             CE_CAN_18.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_18_W3             CE_CAN_18.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_18_CS4            CE_CAN_18.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_18_ID4            CE_CAN_18.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_18_B4             CE_CAN_18.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_18_H4             CE_CAN_18.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_18_W4             CE_CAN_18.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_18_CS5            CE_CAN_18.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_18_ID5            CE_CAN_18.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_18_B5             CE_CAN_18.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_18_H5             CE_CAN_18.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_18_W5             CE_CAN_18.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_18_CS6            CE_CAN_18.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_18_ID6            CE_CAN_18.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_18_B6             CE_CAN_18.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_18_H6             CE_CAN_18.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_18_W6             CE_CAN_18.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_18_CS7            CE_CAN_18.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_18_ID7            CE_CAN_18.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_18_B7             CE_CAN_18.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_18_H7             CE_CAN_18.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_18_W7             CE_CAN_18.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_18_CS8            CE_CAN_18.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_18_ID8            CE_CAN_18.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_18_B8             CE_CAN_18.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_18_H8             CE_CAN_18.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_18_W8             CE_CAN_18.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_18_CS9            CE_CAN_18.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_18_ID9            CE_CAN_18.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_18_B9             CE_CAN_18.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_18_H9             CE_CAN_18.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_18_W9             CE_CAN_18.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_18_CS10           CE_CAN_18.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_18_ID10           CE_CAN_18.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_18_B10            CE_CAN_18.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_18_H10            CE_CAN_18.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_18_W10            CE_CAN_18.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_18_CS11           CE_CAN_18.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_18_ID11           CE_CAN_18.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_18_B11            CE_CAN_18.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_18_H11            CE_CAN_18.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_18_W11            CE_CAN_18.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_18_CS12           CE_CAN_18.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_18_ID12           CE_CAN_18.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_18_B12            CE_CAN_18.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_18_H12            CE_CAN_18.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_18_W12            CE_CAN_18.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_18_CS13           CE_CAN_18.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_18_ID13           CE_CAN_18.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_18_B13            CE_CAN_18.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_18_H13            CE_CAN_18.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_18_W13            CE_CAN_18.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_18_CS14           CE_CAN_18.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_18_ID14           CE_CAN_18.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_18_B14            CE_CAN_18.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_18_H14            CE_CAN_18.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_18_W14            CE_CAN_18.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_18_CS15           CE_CAN_18.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_18_ID15           CE_CAN_18.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_18_B15            CE_CAN_18.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_18_H15            CE_CAN_18.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_18_W15            CE_CAN_18.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_18_CS16           CE_CAN_18.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_18_ID16           CE_CAN_18.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_18_B16            CE_CAN_18.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_18_H16            CE_CAN_18.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_18_W16            CE_CAN_18.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_18_CS17           CE_CAN_18.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_18_ID17           CE_CAN_18.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_18_B17            CE_CAN_18.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_18_H17            CE_CAN_18.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_18_W17            CE_CAN_18.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_18_CS18           CE_CAN_18.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_18_ID18           CE_CAN_18.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_18_B18            CE_CAN_18.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_18_H18            CE_CAN_18.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_18_W18            CE_CAN_18.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_18_CS19           CE_CAN_18.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_18_ID19           CE_CAN_18.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_18_B19            CE_CAN_18.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_18_H19            CE_CAN_18.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_18_W19            CE_CAN_18.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_18_CS20           CE_CAN_18.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_18_ID20           CE_CAN_18.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_18_B20            CE_CAN_18.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_18_H20            CE_CAN_18.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_18_W20            CE_CAN_18.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_18_CS21           CE_CAN_18.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_18_ID21           CE_CAN_18.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_18_B21            CE_CAN_18.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_18_H21            CE_CAN_18.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_18_W21            CE_CAN_18.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_18_CS22           CE_CAN_18.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_18_ID22           CE_CAN_18.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_18_B22            CE_CAN_18.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_18_H22            CE_CAN_18.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_18_W22            CE_CAN_18.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_18_CS23           CE_CAN_18.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_18_ID23           CE_CAN_18.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_18_B23            CE_CAN_18.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_18_H23            CE_CAN_18.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_18_W23            CE_CAN_18.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_18_CS24           CE_CAN_18.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_18_ID24           CE_CAN_18.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_18_B24            CE_CAN_18.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_18_H24            CE_CAN_18.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_18_W24            CE_CAN_18.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_18_CS25           CE_CAN_18.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_18_ID25           CE_CAN_18.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_18_B25            CE_CAN_18.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_18_H25            CE_CAN_18.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_18_W25            CE_CAN_18.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_18_CS26           CE_CAN_18.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_18_ID26           CE_CAN_18.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_18_B26            CE_CAN_18.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_18_H26            CE_CAN_18.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_18_W26            CE_CAN_18.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_18_CS27           CE_CAN_18.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_18_ID27           CE_CAN_18.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_18_B27            CE_CAN_18.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_18_H27            CE_CAN_18.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_18_W27            CE_CAN_18.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_18_CS28           CE_CAN_18.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_18_ID28           CE_CAN_18.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_18_B28            CE_CAN_18.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_18_H28            CE_CAN_18.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_18_W28            CE_CAN_18.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_18_CS29           CE_CAN_18.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_18_ID29           CE_CAN_18.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_18_B29            CE_CAN_18.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_18_H29            CE_CAN_18.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_18_W29            CE_CAN_18.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_18_CS30           CE_CAN_18.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_18_ID30           CE_CAN_18.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_18_B30            CE_CAN_18.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_18_H30            CE_CAN_18.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_18_W30            CE_CAN_18.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_18_CS31           CE_CAN_18.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_18_ID31           CE_CAN_18.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_18_B31            CE_CAN_18.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_18_H31            CE_CAN_18.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_18_W31            CE_CAN_18.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_18_CS32           CE_CAN_18.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_18_ID32           CE_CAN_18.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_18_B32            CE_CAN_18.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_18_H32            CE_CAN_18.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_18_W32            CE_CAN_18.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_18_CS33           CE_CAN_18.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_18_ID33           CE_CAN_18.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_18_B33            CE_CAN_18.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_18_H33            CE_CAN_18.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_18_W33            CE_CAN_18.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_18_CS34           CE_CAN_18.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_18_ID34           CE_CAN_18.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_18_B34            CE_CAN_18.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_18_H34            CE_CAN_18.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_18_W34            CE_CAN_18.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_18_CS35           CE_CAN_18.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_18_ID35           CE_CAN_18.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_18_B35            CE_CAN_18.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_18_H35            CE_CAN_18.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_18_W35            CE_CAN_18.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_18_CS36           CE_CAN_18.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_18_ID36           CE_CAN_18.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_18_B36            CE_CAN_18.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_18_H36            CE_CAN_18.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_18_W36            CE_CAN_18.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_18_CS37           CE_CAN_18.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_18_ID37           CE_CAN_18.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_18_B37            CE_CAN_18.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_18_H37            CE_CAN_18.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_18_W37            CE_CAN_18.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_18_CS38           CE_CAN_18.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_18_ID38           CE_CAN_18.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_18_B38            CE_CAN_18.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_18_H38            CE_CAN_18.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_18_W38            CE_CAN_18.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_18_CS39           CE_CAN_18.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_18_ID39           CE_CAN_18.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_18_B39            CE_CAN_18.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_18_H39            CE_CAN_18.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_18_W39            CE_CAN_18.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_18_CS40           CE_CAN_18.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_18_ID40           CE_CAN_18.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_18_B40            CE_CAN_18.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_18_H40            CE_CAN_18.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_18_W40            CE_CAN_18.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_18_CS41           CE_CAN_18.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_18_ID41           CE_CAN_18.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_18_B41            CE_CAN_18.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_18_H41            CE_CAN_18.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_18_W41            CE_CAN_18.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_18_CS42           CE_CAN_18.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_18_ID42           CE_CAN_18.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_18_B42            CE_CAN_18.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_18_H42            CE_CAN_18.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_18_W42            CE_CAN_18.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_18_CS43           CE_CAN_18.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_18_ID43           CE_CAN_18.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_18_B43            CE_CAN_18.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_18_H43            CE_CAN_18.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_18_W43            CE_CAN_18.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_18_CS44           CE_CAN_18.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_18_ID44           CE_CAN_18.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_18_B44            CE_CAN_18.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_18_H44            CE_CAN_18.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_18_W44            CE_CAN_18.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_18_CS45           CE_CAN_18.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_18_ID45           CE_CAN_18.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_18_B45            CE_CAN_18.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_18_H45            CE_CAN_18.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_18_W45            CE_CAN_18.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_18_CS46           CE_CAN_18.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_18_ID46           CE_CAN_18.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_18_B46            CE_CAN_18.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_18_H46            CE_CAN_18.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_18_W46            CE_CAN_18.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_18_CS47           CE_CAN_18.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_18_ID47           CE_CAN_18.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_18_B47            CE_CAN_18.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_18_H47            CE_CAN_18.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_18_W47            CE_CAN_18.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_18_CS48           CE_CAN_18.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_18_ID48           CE_CAN_18.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_18_B48            CE_CAN_18.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_18_H48            CE_CAN_18.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_18_W48            CE_CAN_18.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_18_CS49           CE_CAN_18.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_18_ID49           CE_CAN_18.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_18_B49            CE_CAN_18.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_18_H49            CE_CAN_18.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_18_W49            CE_CAN_18.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_18_CS50           CE_CAN_18.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_18_ID50           CE_CAN_18.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_18_B50            CE_CAN_18.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_18_H50            CE_CAN_18.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_18_W50            CE_CAN_18.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_18_CS51           CE_CAN_18.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_18_ID51           CE_CAN_18.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_18_B51            CE_CAN_18.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_18_H51            CE_CAN_18.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_18_W51            CE_CAN_18.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_18_CS52           CE_CAN_18.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_18_ID52           CE_CAN_18.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_18_B52            CE_CAN_18.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_18_H52            CE_CAN_18.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_18_W52            CE_CAN_18.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_18_CS53           CE_CAN_18.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_18_ID53           CE_CAN_18.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_18_B53            CE_CAN_18.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_18_H53            CE_CAN_18.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_18_W53            CE_CAN_18.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_18_CS54           CE_CAN_18.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_18_ID54           CE_CAN_18.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_18_B54            CE_CAN_18.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_18_H54            CE_CAN_18.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_18_W54            CE_CAN_18.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_18_CS55           CE_CAN_18.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_18_ID55           CE_CAN_18.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_18_B55            CE_CAN_18.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_18_H55            CE_CAN_18.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_18_W55            CE_CAN_18.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_18_CS56           CE_CAN_18.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_18_ID56           CE_CAN_18.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_18_B56            CE_CAN_18.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_18_H56            CE_CAN_18.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_18_W56            CE_CAN_18.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_18_CS57           CE_CAN_18.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_18_ID57           CE_CAN_18.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_18_B57            CE_CAN_18.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_18_H57            CE_CAN_18.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_18_W57            CE_CAN_18.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_18_CS58           CE_CAN_18.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_18_ID58           CE_CAN_18.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_18_B58            CE_CAN_18.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_18_H58            CE_CAN_18.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_18_W58            CE_CAN_18.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_18_CS59           CE_CAN_18.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_18_ID59           CE_CAN_18.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_18_B59            CE_CAN_18.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_18_H59            CE_CAN_18.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_18_W59            CE_CAN_18.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_18_CS60           CE_CAN_18.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_18_ID60           CE_CAN_18.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_18_B60            CE_CAN_18.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_18_H60            CE_CAN_18.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_18_W60            CE_CAN_18.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_18_CS61           CE_CAN_18.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_18_ID61           CE_CAN_18.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_18_B61            CE_CAN_18.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_18_H61            CE_CAN_18.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_18_W61            CE_CAN_18.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_18_CS62           CE_CAN_18.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_18_ID62           CE_CAN_18.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_18_B62            CE_CAN_18.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_18_H62            CE_CAN_18.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_18_W62            CE_CAN_18.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_18_CS63           CE_CAN_18.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_18_ID63           CE_CAN_18.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_18_B63            CE_CAN_18.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_18_H63            CE_CAN_18.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_18_W63            CE_CAN_18.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_18_CS64           CE_CAN_18.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_18_ID64           CE_CAN_18.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_18_B64            CE_CAN_18.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_18_H64            CE_CAN_18.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_18_W64            CE_CAN_18.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_18_CS65           CE_CAN_18.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_18_ID65           CE_CAN_18.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_18_B65            CE_CAN_18.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_18_H65            CE_CAN_18.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_18_W65            CE_CAN_18.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_18_CS66           CE_CAN_18.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_18_ID66           CE_CAN_18.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_18_B66            CE_CAN_18.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_18_H66            CE_CAN_18.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_18_W66            CE_CAN_18.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_18_CS67           CE_CAN_18.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_18_ID67           CE_CAN_18.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_18_B67            CE_CAN_18.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_18_H67            CE_CAN_18.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_18_W67            CE_CAN_18.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_18_CS68           CE_CAN_18.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_18_ID68           CE_CAN_18.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_18_B68            CE_CAN_18.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_18_H68            CE_CAN_18.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_18_W68            CE_CAN_18.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_18_CS69           CE_CAN_18.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_18_ID69           CE_CAN_18.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_18_B69            CE_CAN_18.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_18_H69            CE_CAN_18.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_18_W69            CE_CAN_18.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_18_CS70           CE_CAN_18.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_18_ID70           CE_CAN_18.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_18_B70            CE_CAN_18.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_18_H70            CE_CAN_18.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_18_W70            CE_CAN_18.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_18_CS71           CE_CAN_18.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_18_ID71           CE_CAN_18.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_18_B71            CE_CAN_18.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_18_H71            CE_CAN_18.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_18_W71            CE_CAN_18.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_18_CS72           CE_CAN_18.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_18_ID72           CE_CAN_18.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_18_B72            CE_CAN_18.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_18_H72            CE_CAN_18.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_18_W72            CE_CAN_18.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_18_CS73           CE_CAN_18.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_18_ID73           CE_CAN_18.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_18_B73            CE_CAN_18.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_18_H73            CE_CAN_18.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_18_W73            CE_CAN_18.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_18_CS74           CE_CAN_18.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_18_ID74           CE_CAN_18.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_18_B74            CE_CAN_18.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_18_H74            CE_CAN_18.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_18_W74            CE_CAN_18.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_18_CS75           CE_CAN_18.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_18_ID75           CE_CAN_18.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_18_B75            CE_CAN_18.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_18_H75            CE_CAN_18.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_18_W75            CE_CAN_18.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_18_CS76           CE_CAN_18.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_18_ID76           CE_CAN_18.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_18_B76            CE_CAN_18.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_18_H76            CE_CAN_18.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_18_W76            CE_CAN_18.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_18_CS77           CE_CAN_18.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_18_ID77           CE_CAN_18.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_18_B77            CE_CAN_18.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_18_H77            CE_CAN_18.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_18_W77            CE_CAN_18.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_18_CS78           CE_CAN_18.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_18_ID78           CE_CAN_18.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_18_B78            CE_CAN_18.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_18_H78            CE_CAN_18.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_18_W78            CE_CAN_18.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_18_CS79           CE_CAN_18.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_18_ID79           CE_CAN_18.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_18_B79            CE_CAN_18.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_18_H79            CE_CAN_18.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_18_W79            CE_CAN_18.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_18_CS80           CE_CAN_18.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_18_ID80           CE_CAN_18.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_18_B80            CE_CAN_18.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_18_H80            CE_CAN_18.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_18_W80            CE_CAN_18.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_18_CS81           CE_CAN_18.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_18_ID81           CE_CAN_18.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_18_B81            CE_CAN_18.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_18_H81            CE_CAN_18.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_18_W81            CE_CAN_18.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_18_CS82           CE_CAN_18.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_18_ID82           CE_CAN_18.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_18_B82            CE_CAN_18.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_18_H82            CE_CAN_18.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_18_W82            CE_CAN_18.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_18_CS83           CE_CAN_18.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_18_ID83           CE_CAN_18.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_18_B83            CE_CAN_18.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_18_H83            CE_CAN_18.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_18_W83            CE_CAN_18.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_18_CS84           CE_CAN_18.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_18_ID84           CE_CAN_18.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_18_B84            CE_CAN_18.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_18_H84            CE_CAN_18.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_18_W84            CE_CAN_18.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_18_CS85           CE_CAN_18.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_18_ID85           CE_CAN_18.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_18_B85            CE_CAN_18.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_18_H85            CE_CAN_18.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_18_W85            CE_CAN_18.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_18_CS86           CE_CAN_18.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_18_ID86           CE_CAN_18.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_18_B86            CE_CAN_18.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_18_H86            CE_CAN_18.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_18_W86            CE_CAN_18.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_18_CS87           CE_CAN_18.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_18_ID87           CE_CAN_18.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_18_B87            CE_CAN_18.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_18_H87            CE_CAN_18.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_18_W87            CE_CAN_18.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_18_CS88           CE_CAN_18.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_18_ID88           CE_CAN_18.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_18_B88            CE_CAN_18.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_18_H88            CE_CAN_18.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_18_W848           CE_CAN_18.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_18_CS89           CE_CAN_18.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_18_ID89           CE_CAN_18.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_18_B89            CE_CAN_18.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_18_H89            CE_CAN_18.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_18_W89            CE_CAN_18.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_18_CS90           CE_CAN_18.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_18_ID90           CE_CAN_18.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_18_B90            CE_CAN_18.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_18_H90            CE_CAN_18.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_18_W90            CE_CAN_18.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_18_CS91           CE_CAN_18.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_18_ID91           CE_CAN_18.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_18_B91            CE_CAN_18.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_18_H91            CE_CAN_18.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_18_W91            CE_CAN_18.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_18_CS92           CE_CAN_18.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_18_ID92           CE_CAN_18.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_18_B92            CE_CAN_18.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_18_H92            CE_CAN_18.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_18_W92            CE_CAN_18.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_18_CS93           CE_CAN_18.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_18_ID93           CE_CAN_18.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_18_B93            CE_CAN_18.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_18_H93            CE_CAN_18.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_18_W93            CE_CAN_18.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_18_CS94           CE_CAN_18.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_18_ID94           CE_CAN_18.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_18_B94            CE_CAN_18.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_18_H94            CE_CAN_18.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_18_W94            CE_CAN_18.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_18_CS95           CE_CAN_18.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_18_ID95           CE_CAN_18.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_18_B95            CE_CAN_18.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_18_H95            CE_CAN_18.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_18_W95            CE_CAN_18.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_18_CS96           CE_CAN_18.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_18_ID96           CE_CAN_18.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_18_B96            CE_CAN_18.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_18_H96            CE_CAN_18.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_18_W96            CE_CAN_18.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_18_CS97           CE_CAN_18.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_18_ID97           CE_CAN_18.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_18_B97            CE_CAN_18.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_18_H97            CE_CAN_18.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_18_W97            CE_CAN_18.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_18_CS98           CE_CAN_18.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_18_ID98           CE_CAN_18.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_18_B98            CE_CAN_18.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_18_H98            CE_CAN_18.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_18_W98            CE_CAN_18.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_18_CS99           CE_CAN_18.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_18_ID99           CE_CAN_18.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_18_B99            CE_CAN_18.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_18_H99            CE_CAN_18.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_18_W99            CE_CAN_18.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_18_CS100          CE_CAN_18.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_18_ID100          CE_CAN_18.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_18_B100           CE_CAN_18.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_18_H100           CE_CAN_18.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_18_W100           CE_CAN_18.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_18_CS101          CE_CAN_18.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_18_ID101          CE_CAN_18.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_18_B101           CE_CAN_18.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_18_H101           CE_CAN_18.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_18_W101           CE_CAN_18.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_18_CS102          CE_CAN_18.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_18_ID102          CE_CAN_18.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_18_B102           CE_CAN_18.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_18_H102           CE_CAN_18.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_18_W102           CE_CAN_18.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_18_CS103          CE_CAN_18.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_18_ID103          CE_CAN_18.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_18_B103           CE_CAN_18.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_18_H103           CE_CAN_18.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_18_W103           CE_CAN_18.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_18_CS104          CE_CAN_18.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_18_ID104          CE_CAN_18.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_18_B104           CE_CAN_18.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_18_H104           CE_CAN_18.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_18_W104           CE_CAN_18.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_18_CS105          CE_CAN_18.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_18_ID105          CE_CAN_18.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_18_B105           CE_CAN_18.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_18_H105           CE_CAN_18.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_18_W105           CE_CAN_18.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_18_CS106          CE_CAN_18.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_18_ID106          CE_CAN_18.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_18_B106           CE_CAN_18.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_18_H106           CE_CAN_18.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_18_W106           CE_CAN_18.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_18_CS107          CE_CAN_18.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_18_ID107          CE_CAN_18.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_18_B107           CE_CAN_18.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_18_H107           CE_CAN_18.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_18_W107           CE_CAN_18.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_18_CS108          CE_CAN_18.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_18_ID108          CE_CAN_18.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_18_B108           CE_CAN_18.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_18_H108           CE_CAN_18.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_18_W108           CE_CAN_18.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_18_CS109          CE_CAN_18.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_18_ID109          CE_CAN_18.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_18_B109           CE_CAN_18.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_18_H109           CE_CAN_18.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_18_W109           CE_CAN_18.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_18_CS110          CE_CAN_18.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_18_ID110          CE_CAN_18.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_18_B110           CE_CAN_18.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_18_H110           CE_CAN_18.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_18_W110           CE_CAN_18.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_18_CS111          CE_CAN_18.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_18_ID111          CE_CAN_18.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_18_B111           CE_CAN_18.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_18_H111           CE_CAN_18.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_18_W111           CE_CAN_18.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_18_CS112          CE_CAN_18.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_18_ID112          CE_CAN_18.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_18_B112           CE_CAN_18.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_18_H112           CE_CAN_18.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_18_W112           CE_CAN_18.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_18_CS113          CE_CAN_18.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_18_ID113          CE_CAN_18.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_18_B113           CE_CAN_18.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_18_H113           CE_CAN_18.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_18_W113           CE_CAN_18.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_18_CS114          CE_CAN_18.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_18_ID114          CE_CAN_18.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_18_B114           CE_CAN_18.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_18_H114           CE_CAN_18.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_18_W114           CE_CAN_18.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_18_CS115          CE_CAN_18.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_18_ID115          CE_CAN_18.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_18_B115           CE_CAN_18.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_18_H115           CE_CAN_18.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_18_W115           CE_CAN_18.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_18_CS116          CE_CAN_18.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_18_ID116          CE_CAN_18.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_18_B116           CE_CAN_18.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_18_H116           CE_CAN_18.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_18_W116           CE_CAN_18.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_18_CS117          CE_CAN_18.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_18_ID117          CE_CAN_18.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_18_B117           CE_CAN_18.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_18_H117           CE_CAN_18.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_18_W117           CE_CAN_18.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_18_CS118          CE_CAN_18.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_18_ID118          CE_CAN_18.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_18_B118           CE_CAN_18.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_18_H118           CE_CAN_18.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_18_W118           CE_CAN_18.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_18_CS119          CE_CAN_18.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_18_ID119          CE_CAN_18.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_18_B119           CE_CAN_18.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_18_H119           CE_CAN_18.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_18_W119           CE_CAN_18.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_18_CS120          CE_CAN_18.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_18_ID120          CE_CAN_18.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_18_B120           CE_CAN_18.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_18_H120           CE_CAN_18.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_18_W120           CE_CAN_18.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_18_CS121          CE_CAN_18.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_18_ID121          CE_CAN_18.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_18_B121           CE_CAN_18.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_18_H121           CE_CAN_18.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_18_W121           CE_CAN_18.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_18_CS122          CE_CAN_18.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_18_ID122          CE_CAN_18.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_18_B122           CE_CAN_18.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_18_H122           CE_CAN_18.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_18_W122           CE_CAN_18.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_18_CS123          CE_CAN_18.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_18_ID123          CE_CAN_18.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_18_B123           CE_CAN_18.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_18_H123           CE_CAN_18.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_18_W123           CE_CAN_18.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_18_CS124          CE_CAN_18.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_18_ID124          CE_CAN_18.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_18_B124           CE_CAN_18.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_18_H124           CE_CAN_18.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_18_W124           CE_CAN_18.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_18_CS125          CE_CAN_18.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_18_ID125          CE_CAN_18.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_18_B125           CE_CAN_18.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_18_H125           CE_CAN_18.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_18_W125           CE_CAN_18.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_18_CS126          CE_CAN_18.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_18_ID126          CE_CAN_18.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_18_B126           CE_CAN_18.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_18_H126           CE_CAN_18.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_18_W126           CE_CAN_18.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_18_CS127          CE_CAN_18.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_18_ID127          CE_CAN_18.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_18_B127           CE_CAN_18.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_18_H127           CE_CAN_18.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_18_W127           CE_CAN_18.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_18_RXIMR0     CE_CAN_18.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR1     CE_CAN_18.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR2     CE_CAN_18.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR3     CE_CAN_18.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR4     CE_CAN_18.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR5     CE_CAN_18.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR6     CE_CAN_18.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR7     CE_CAN_18.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR8     CE_CAN_18.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR9     CE_CAN_18.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_18_RXIMR10    CE_CAN_18.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR11    CE_CAN_18.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR12    CE_CAN_18.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR13    CE_CAN_18.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR14    CE_CAN_18.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR15    CE_CAN_18.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR16    CE_CAN_18.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR17    CE_CAN_18.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR18    CE_CAN_18.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR19    CE_CAN_18.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR20    CE_CAN_18.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR21    CE_CAN_18.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR22    CE_CAN_18.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR23    CE_CAN_18.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR24    CE_CAN_18.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR25    CE_CAN_18.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR26    CE_CAN_18.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR27    CE_CAN_18.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR28    CE_CAN_18.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR29    CE_CAN_18.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR30    CE_CAN_18.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR31    CE_CAN_18.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR32    CE_CAN_18.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR33    CE_CAN_18.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR34    CE_CAN_18.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR35    CE_CAN_18.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR36    CE_CAN_18.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR37    CE_CAN_18.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR38    CE_CAN_18.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR39    CE_CAN_18.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR40    CE_CAN_18.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR41    CE_CAN_18.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR42    CE_CAN_18.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR43    CE_CAN_18.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR44    CE_CAN_18.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR45    CE_CAN_18.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR46    CE_CAN_18.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR47    CE_CAN_18.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR48    CE_CAN_18.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR49    CE_CAN_18.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR50    CE_CAN_18.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR51    CE_CAN_18.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR52    CE_CAN_18.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR53    CE_CAN_18.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR54    CE_CAN_18.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR55    CE_CAN_18.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR56    CE_CAN_18.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR57    CE_CAN_18.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR58    CE_CAN_18.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR59    CE_CAN_18.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR60    CE_CAN_18.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR61    CE_CAN_18.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR62    CE_CAN_18.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR63    CE_CAN_18.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR64    CE_CAN_18.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR65    CE_CAN_18.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR66    CE_CAN_18.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR67    CE_CAN_18.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR68    CE_CAN_18.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR69    CE_CAN_18.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR70    CE_CAN_18.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR71    CE_CAN_18.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR72    CE_CAN_18.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR73    CE_CAN_18.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR74    CE_CAN_18.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR75    CE_CAN_18.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR76    CE_CAN_18.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR77    CE_CAN_18.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR78    CE_CAN_18.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR79    CE_CAN_18.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR80    CE_CAN_18.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR81    CE_CAN_18.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR82    CE_CAN_18.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR83    CE_CAN_18.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR84    CE_CAN_18.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR85    CE_CAN_18.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR86    CE_CAN_18.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR87    CE_CAN_18.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR88    CE_CAN_18.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR89    CE_CAN_18.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR90    CE_CAN_18.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR91    CE_CAN_18.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR92    CE_CAN_18.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR93    CE_CAN_18.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR94    CE_CAN_18.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR95    CE_CAN_18.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR96    CE_CAN_18.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR97    CE_CAN_18.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR98    CE_CAN_18.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR99    CE_CAN_18.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_18_RXIMR100   CE_CAN_18.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR101   CE_CAN_18.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR102   CE_CAN_18.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR103   CE_CAN_18.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR104   CE_CAN_18.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR105   CE_CAN_18.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR106   CE_CAN_18.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR107   CE_CAN_18.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR108   CE_CAN_18.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR109   CE_CAN_18.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR110   CE_CAN_18.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR111   CE_CAN_18.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR112   CE_CAN_18.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR113   CE_CAN_18.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR114   CE_CAN_18.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR115   CE_CAN_18.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR116   CE_CAN_18.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR117   CE_CAN_18.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR118   CE_CAN_18.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR119   CE_CAN_18.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR120   CE_CAN_18.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR121   CE_CAN_18.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR122   CE_CAN_18.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR123   CE_CAN_18.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR124   CE_CAN_18.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR125   CE_CAN_18.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR126   CE_CAN_18.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_18_RXIMR127   CE_CAN_18.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_18_MECR       CE_CAN_18.MECR.R              /* Memory Error Control */
#define CE_CAN_18_ERRIAR     CE_CAN_18.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_18_ERRIDPR    CE_CAN_18.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_18_ERRIPPR    CE_CAN_18.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_18_RERRAR     CE_CAN_18.RERRAR.R            /* Error Report Address */
#define CE_CAN_18_RERRDR     CE_CAN_18.RERRDR.R            /* Error Report Data */
#define CE_CAN_18_RERRSYNR   CE_CAN_18.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_18_ERRSR      CE_CAN_18.ERRSR.R             /* Error Status */
#define CE_CAN_18_EPRS       CE_CAN_18.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_18_ENCBT      CE_CAN_18.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_18_EDCBT      CE_CAN_18.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_18_ETDC       CE_CAN_18.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_18_FDCTRL     CE_CAN_18.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_18_FDCBT      CE_CAN_18.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_18_FDCRC      CE_CAN_18.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_18_ERFCR      CE_CAN_18.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_18_ERFIER     CE_CAN_18.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_18_ERFSR      CE_CAN_18.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_18_HR_TIME_STAMP0 CE_CAN_18.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP1 CE_CAN_18.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP2 CE_CAN_18.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP3 CE_CAN_18.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP4 CE_CAN_18.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP5 CE_CAN_18.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP6 CE_CAN_18.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP7 CE_CAN_18.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP8 CE_CAN_18.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP9 CE_CAN_18.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP10 CE_CAN_18.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP11 CE_CAN_18.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP12 CE_CAN_18.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP13 CE_CAN_18.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP14 CE_CAN_18.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP15 CE_CAN_18.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP16 CE_CAN_18.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP17 CE_CAN_18.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP18 CE_CAN_18.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP19 CE_CAN_18.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP20 CE_CAN_18.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP21 CE_CAN_18.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP22 CE_CAN_18.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP23 CE_CAN_18.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP24 CE_CAN_18.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP25 CE_CAN_18.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP26 CE_CAN_18.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP27 CE_CAN_18.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP28 CE_CAN_18.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP29 CE_CAN_18.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP30 CE_CAN_18.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP31 CE_CAN_18.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP32 CE_CAN_18.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP33 CE_CAN_18.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP34 CE_CAN_18.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP35 CE_CAN_18.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP36 CE_CAN_18.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP37 CE_CAN_18.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP38 CE_CAN_18.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP39 CE_CAN_18.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP40 CE_CAN_18.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP41 CE_CAN_18.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP42 CE_CAN_18.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP43 CE_CAN_18.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP44 CE_CAN_18.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP45 CE_CAN_18.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP46 CE_CAN_18.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP47 CE_CAN_18.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP48 CE_CAN_18.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP49 CE_CAN_18.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP50 CE_CAN_18.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP51 CE_CAN_18.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP52 CE_CAN_18.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP53 CE_CAN_18.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP54 CE_CAN_18.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP55 CE_CAN_18.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP56 CE_CAN_18.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP57 CE_CAN_18.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP58 CE_CAN_18.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP59 CE_CAN_18.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP60 CE_CAN_18.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP61 CE_CAN_18.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP62 CE_CAN_18.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP63 CE_CAN_18.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP64 CE_CAN_18.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP65 CE_CAN_18.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP66 CE_CAN_18.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP67 CE_CAN_18.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP68 CE_CAN_18.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP69 CE_CAN_18.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP70 CE_CAN_18.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP71 CE_CAN_18.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP72 CE_CAN_18.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP73 CE_CAN_18.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP74 CE_CAN_18.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP75 CE_CAN_18.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP76 CE_CAN_18.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP77 CE_CAN_18.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP78 CE_CAN_18.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP79 CE_CAN_18.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP80 CE_CAN_18.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP81 CE_CAN_18.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP82 CE_CAN_18.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP83 CE_CAN_18.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP84 CE_CAN_18.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP85 CE_CAN_18.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP86 CE_CAN_18.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP87 CE_CAN_18.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP88 CE_CAN_18.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP89 CE_CAN_18.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP90 CE_CAN_18.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP91 CE_CAN_18.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP92 CE_CAN_18.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP93 CE_CAN_18.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP94 CE_CAN_18.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP95 CE_CAN_18.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP96 CE_CAN_18.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP97 CE_CAN_18.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP98 CE_CAN_18.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP99 CE_CAN_18.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP100 CE_CAN_18.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP101 CE_CAN_18.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP102 CE_CAN_18.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP103 CE_CAN_18.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP104 CE_CAN_18.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP105 CE_CAN_18.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP106 CE_CAN_18.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP107 CE_CAN_18.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP108 CE_CAN_18.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP109 CE_CAN_18.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP110 CE_CAN_18.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP111 CE_CAN_18.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP112 CE_CAN_18.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP113 CE_CAN_18.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP114 CE_CAN_18.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP115 CE_CAN_18.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP116 CE_CAN_18.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP117 CE_CAN_18.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP118 CE_CAN_18.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP119 CE_CAN_18.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP120 CE_CAN_18.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP121 CE_CAN_18.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP122 CE_CAN_18.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP123 CE_CAN_18.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP124 CE_CAN_18.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP125 CE_CAN_18.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP126 CE_CAN_18.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_18_HR_TIME_STAMP127 CE_CAN_18.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_18_ERFFEL0    CE_CAN_18.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL1    CE_CAN_18.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL2    CE_CAN_18.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL3    CE_CAN_18.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL4    CE_CAN_18.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL5    CE_CAN_18.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL6    CE_CAN_18.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL7    CE_CAN_18.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL8    CE_CAN_18.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL9    CE_CAN_18.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL10   CE_CAN_18.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL11   CE_CAN_18.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL12   CE_CAN_18.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL13   CE_CAN_18.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL14   CE_CAN_18.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL15   CE_CAN_18.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL16   CE_CAN_18.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL17   CE_CAN_18.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL18   CE_CAN_18.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL19   CE_CAN_18.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL20   CE_CAN_18.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL21   CE_CAN_18.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL22   CE_CAN_18.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL23   CE_CAN_18.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL24   CE_CAN_18.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL25   CE_CAN_18.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL26   CE_CAN_18.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL27   CE_CAN_18.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL28   CE_CAN_18.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL29   CE_CAN_18.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL30   CE_CAN_18.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL31   CE_CAN_18.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL32   CE_CAN_18.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL33   CE_CAN_18.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL34   CE_CAN_18.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL35   CE_CAN_18.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL36   CE_CAN_18.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL37   CE_CAN_18.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL38   CE_CAN_18.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL39   CE_CAN_18.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL40   CE_CAN_18.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL41   CE_CAN_18.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL42   CE_CAN_18.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL43   CE_CAN_18.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL44   CE_CAN_18.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL45   CE_CAN_18.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL46   CE_CAN_18.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL47   CE_CAN_18.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL48   CE_CAN_18.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL49   CE_CAN_18.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL50   CE_CAN_18.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL51   CE_CAN_18.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL52   CE_CAN_18.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL53   CE_CAN_18.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL54   CE_CAN_18.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL55   CE_CAN_18.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL56   CE_CAN_18.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL57   CE_CAN_18.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL58   CE_CAN_18.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL59   CE_CAN_18.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL60   CE_CAN_18.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL61   CE_CAN_18.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL62   CE_CAN_18.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL63   CE_CAN_18.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL64   CE_CAN_18.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL65   CE_CAN_18.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL66   CE_CAN_18.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL67   CE_CAN_18.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL68   CE_CAN_18.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL69   CE_CAN_18.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL70   CE_CAN_18.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL71   CE_CAN_18.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL72   CE_CAN_18.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL73   CE_CAN_18.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL74   CE_CAN_18.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL75   CE_CAN_18.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL76   CE_CAN_18.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL77   CE_CAN_18.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL78   CE_CAN_18.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL79   CE_CAN_18.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL80   CE_CAN_18.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL81   CE_CAN_18.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL82   CE_CAN_18.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL83   CE_CAN_18.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL84   CE_CAN_18.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL85   CE_CAN_18.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL86   CE_CAN_18.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL87   CE_CAN_18.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL88   CE_CAN_18.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL89   CE_CAN_18.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL90   CE_CAN_18.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL91   CE_CAN_18.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL92   CE_CAN_18.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL93   CE_CAN_18.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL94   CE_CAN_18.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL95   CE_CAN_18.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL96   CE_CAN_18.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL97   CE_CAN_18.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL98   CE_CAN_18.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL99   CE_CAN_18.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL100  CE_CAN_18.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL101  CE_CAN_18.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL102  CE_CAN_18.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL103  CE_CAN_18.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL104  CE_CAN_18.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL105  CE_CAN_18.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL106  CE_CAN_18.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL107  CE_CAN_18.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL108  CE_CAN_18.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL109  CE_CAN_18.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL110  CE_CAN_18.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL111  CE_CAN_18.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL112  CE_CAN_18.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL113  CE_CAN_18.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL114  CE_CAN_18.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL115  CE_CAN_18.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL116  CE_CAN_18.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL117  CE_CAN_18.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL118  CE_CAN_18.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL119  CE_CAN_18.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL120  CE_CAN_18.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL121  CE_CAN_18.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL122  CE_CAN_18.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL123  CE_CAN_18.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL124  CE_CAN_18.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL125  CE_CAN_18.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL126  CE_CAN_18.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_18_ERFFEL127  CE_CAN_18.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_19_MCR        CE_CAN_19.MCR.R               /* Module Configuration */
#define CE_CAN_19_CTRL1      CE_CAN_19.CTRL1.R             /* Control 1 */
#define CE_CAN_19_TIMER      CE_CAN_19.TIMER.R             /* Free-Running Timer */
#define CE_CAN_19_RXMGMASK   CE_CAN_19.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_19_RX14MASK   CE_CAN_19.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_19_RX15MASK   CE_CAN_19.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_19_ECR        CE_CAN_19.ECR.R               /* Error Counter */
#define CE_CAN_19_ESR1       CE_CAN_19.ESR1.R              /* Error and Status 1 */
#define CE_CAN_19_IMASK2     CE_CAN_19.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_19_IMASK1     CE_CAN_19.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_19_IFLAG2     CE_CAN_19.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_19_IFLAG1     CE_CAN_19.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_19_CTRL2      CE_CAN_19.CTRL2.R             /* Control 2 */
#define CE_CAN_19_ESR2       CE_CAN_19.ESR2.R              /* Error and Status 2 */
#define CE_CAN_19_CRCR       CE_CAN_19.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_19_RXFGMASK   CE_CAN_19.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_19_RXFIR      CE_CAN_19.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_19_CBT        CE_CAN_19.CBT.R               /* CAN Bit Timing */
#define CE_CAN_19_IMASK4     CE_CAN_19.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_19_IMASK3     CE_CAN_19.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_19_IFLAG4     CE_CAN_19.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_19_IFLAG3     CE_CAN_19.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_19_CS0            CE_CAN_19.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_19_ID0            CE_CAN_19.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_19_B0             CE_CAN_19.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_19_H0             CE_CAN_19.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_19_W0             CE_CAN_19.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_19_CS1            CE_CAN_19.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_19_ID1            CE_CAN_19.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_19_B1             CE_CAN_19.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_19_H1             CE_CAN_19.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_19_W1             CE_CAN_19.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_19_CS2            CE_CAN_19.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_19_ID2            CE_CAN_19.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_19_B2             CE_CAN_19.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_19_H2             CE_CAN_19.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_19_W2             CE_CAN_19.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_19_CS3            CE_CAN_19.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_19_ID3            CE_CAN_19.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_19_B3             CE_CAN_19.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_19_H3             CE_CAN_19.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_19_W3             CE_CAN_19.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_19_CS4            CE_CAN_19.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_19_ID4            CE_CAN_19.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_19_B4             CE_CAN_19.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_19_H4             CE_CAN_19.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_19_W4             CE_CAN_19.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_19_CS5            CE_CAN_19.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_19_ID5            CE_CAN_19.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_19_B5             CE_CAN_19.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_19_H5             CE_CAN_19.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_19_W5             CE_CAN_19.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_19_CS6            CE_CAN_19.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_19_ID6            CE_CAN_19.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_19_B6             CE_CAN_19.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_19_H6             CE_CAN_19.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_19_W6             CE_CAN_19.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_19_CS7            CE_CAN_19.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_19_ID7            CE_CAN_19.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_19_B7             CE_CAN_19.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_19_H7             CE_CAN_19.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_19_W7             CE_CAN_19.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_19_CS8            CE_CAN_19.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_19_ID8            CE_CAN_19.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_19_B8             CE_CAN_19.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_19_H8             CE_CAN_19.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_19_W8             CE_CAN_19.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_19_CS9            CE_CAN_19.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_19_ID9            CE_CAN_19.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_19_B9             CE_CAN_19.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_19_H9             CE_CAN_19.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_19_W9             CE_CAN_19.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_19_CS10           CE_CAN_19.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_19_ID10           CE_CAN_19.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_19_B10            CE_CAN_19.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_19_H10            CE_CAN_19.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_19_W10            CE_CAN_19.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_19_CS11           CE_CAN_19.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_19_ID11           CE_CAN_19.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_19_B11            CE_CAN_19.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_19_H11            CE_CAN_19.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_19_W11            CE_CAN_19.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_19_CS12           CE_CAN_19.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_19_ID12           CE_CAN_19.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_19_B12            CE_CAN_19.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_19_H12            CE_CAN_19.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_19_W12            CE_CAN_19.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_19_CS13           CE_CAN_19.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_19_ID13           CE_CAN_19.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_19_B13            CE_CAN_19.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_19_H13            CE_CAN_19.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_19_W13            CE_CAN_19.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_19_CS14           CE_CAN_19.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_19_ID14           CE_CAN_19.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_19_B14            CE_CAN_19.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_19_H14            CE_CAN_19.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_19_W14            CE_CAN_19.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_19_CS15           CE_CAN_19.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_19_ID15           CE_CAN_19.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_19_B15            CE_CAN_19.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_19_H15            CE_CAN_19.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_19_W15            CE_CAN_19.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_19_CS16           CE_CAN_19.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_19_ID16           CE_CAN_19.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_19_B16            CE_CAN_19.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_19_H16            CE_CAN_19.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_19_W16            CE_CAN_19.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_19_CS17           CE_CAN_19.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_19_ID17           CE_CAN_19.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_19_B17            CE_CAN_19.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_19_H17            CE_CAN_19.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_19_W17            CE_CAN_19.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_19_CS18           CE_CAN_19.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_19_ID18           CE_CAN_19.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_19_B18            CE_CAN_19.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_19_H18            CE_CAN_19.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_19_W18            CE_CAN_19.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_19_CS19           CE_CAN_19.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_19_ID19           CE_CAN_19.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_19_B19            CE_CAN_19.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_19_H19            CE_CAN_19.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_19_W19            CE_CAN_19.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_19_CS20           CE_CAN_19.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_19_ID20           CE_CAN_19.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_19_B20            CE_CAN_19.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_19_H20            CE_CAN_19.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_19_W20            CE_CAN_19.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_19_CS21           CE_CAN_19.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_19_ID21           CE_CAN_19.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_19_B21            CE_CAN_19.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_19_H21            CE_CAN_19.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_19_W21            CE_CAN_19.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_19_CS22           CE_CAN_19.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_19_ID22           CE_CAN_19.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_19_B22            CE_CAN_19.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_19_H22            CE_CAN_19.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_19_W22            CE_CAN_19.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_19_CS23           CE_CAN_19.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_19_ID23           CE_CAN_19.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_19_B23            CE_CAN_19.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_19_H23            CE_CAN_19.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_19_W23            CE_CAN_19.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_19_CS24           CE_CAN_19.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_19_ID24           CE_CAN_19.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_19_B24            CE_CAN_19.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_19_H24            CE_CAN_19.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_19_W24            CE_CAN_19.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_19_CS25           CE_CAN_19.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_19_ID25           CE_CAN_19.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_19_B25            CE_CAN_19.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_19_H25            CE_CAN_19.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_19_W25            CE_CAN_19.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_19_CS26           CE_CAN_19.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_19_ID26           CE_CAN_19.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_19_B26            CE_CAN_19.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_19_H26            CE_CAN_19.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_19_W26            CE_CAN_19.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_19_CS27           CE_CAN_19.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_19_ID27           CE_CAN_19.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_19_B27            CE_CAN_19.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_19_H27            CE_CAN_19.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_19_W27            CE_CAN_19.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_19_CS28           CE_CAN_19.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_19_ID28           CE_CAN_19.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_19_B28            CE_CAN_19.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_19_H28            CE_CAN_19.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_19_W28            CE_CAN_19.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_19_CS29           CE_CAN_19.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_19_ID29           CE_CAN_19.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_19_B29            CE_CAN_19.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_19_H29            CE_CAN_19.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_19_W29            CE_CAN_19.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_19_CS30           CE_CAN_19.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_19_ID30           CE_CAN_19.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_19_B30            CE_CAN_19.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_19_H30            CE_CAN_19.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_19_W30            CE_CAN_19.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_19_CS31           CE_CAN_19.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_19_ID31           CE_CAN_19.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_19_B31            CE_CAN_19.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_19_H31            CE_CAN_19.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_19_W31            CE_CAN_19.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_19_CS32           CE_CAN_19.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_19_ID32           CE_CAN_19.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_19_B32            CE_CAN_19.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_19_H32            CE_CAN_19.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_19_W32            CE_CAN_19.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_19_CS33           CE_CAN_19.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_19_ID33           CE_CAN_19.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_19_B33            CE_CAN_19.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_19_H33            CE_CAN_19.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_19_W33            CE_CAN_19.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_19_CS34           CE_CAN_19.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_19_ID34           CE_CAN_19.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_19_B34            CE_CAN_19.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_19_H34            CE_CAN_19.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_19_W34            CE_CAN_19.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_19_CS35           CE_CAN_19.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_19_ID35           CE_CAN_19.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_19_B35            CE_CAN_19.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_19_H35            CE_CAN_19.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_19_W35            CE_CAN_19.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_19_CS36           CE_CAN_19.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_19_ID36           CE_CAN_19.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_19_B36            CE_CAN_19.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_19_H36            CE_CAN_19.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_19_W36            CE_CAN_19.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_19_CS37           CE_CAN_19.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_19_ID37           CE_CAN_19.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_19_B37            CE_CAN_19.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_19_H37            CE_CAN_19.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_19_W37            CE_CAN_19.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_19_CS38           CE_CAN_19.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_19_ID38           CE_CAN_19.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_19_B38            CE_CAN_19.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_19_H38            CE_CAN_19.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_19_W38            CE_CAN_19.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_19_CS39           CE_CAN_19.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_19_ID39           CE_CAN_19.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_19_B39            CE_CAN_19.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_19_H39            CE_CAN_19.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_19_W39            CE_CAN_19.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_19_CS40           CE_CAN_19.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_19_ID40           CE_CAN_19.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_19_B40            CE_CAN_19.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_19_H40            CE_CAN_19.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_19_W40            CE_CAN_19.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_19_CS41           CE_CAN_19.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_19_ID41           CE_CAN_19.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_19_B41            CE_CAN_19.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_19_H41            CE_CAN_19.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_19_W41            CE_CAN_19.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_19_CS42           CE_CAN_19.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_19_ID42           CE_CAN_19.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_19_B42            CE_CAN_19.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_19_H42            CE_CAN_19.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_19_W42            CE_CAN_19.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_19_CS43           CE_CAN_19.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_19_ID43           CE_CAN_19.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_19_B43            CE_CAN_19.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_19_H43            CE_CAN_19.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_19_W43            CE_CAN_19.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_19_CS44           CE_CAN_19.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_19_ID44           CE_CAN_19.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_19_B44            CE_CAN_19.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_19_H44            CE_CAN_19.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_19_W44            CE_CAN_19.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_19_CS45           CE_CAN_19.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_19_ID45           CE_CAN_19.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_19_B45            CE_CAN_19.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_19_H45            CE_CAN_19.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_19_W45            CE_CAN_19.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_19_CS46           CE_CAN_19.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_19_ID46           CE_CAN_19.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_19_B46            CE_CAN_19.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_19_H46            CE_CAN_19.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_19_W46            CE_CAN_19.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_19_CS47           CE_CAN_19.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_19_ID47           CE_CAN_19.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_19_B47            CE_CAN_19.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_19_H47            CE_CAN_19.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_19_W47            CE_CAN_19.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_19_CS48           CE_CAN_19.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_19_ID48           CE_CAN_19.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_19_B48            CE_CAN_19.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_19_H48            CE_CAN_19.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_19_W48            CE_CAN_19.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_19_CS49           CE_CAN_19.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_19_ID49           CE_CAN_19.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_19_B49            CE_CAN_19.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_19_H49            CE_CAN_19.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_19_W49            CE_CAN_19.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_19_CS50           CE_CAN_19.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_19_ID50           CE_CAN_19.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_19_B50            CE_CAN_19.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_19_H50            CE_CAN_19.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_19_W50            CE_CAN_19.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_19_CS51           CE_CAN_19.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_19_ID51           CE_CAN_19.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_19_B51            CE_CAN_19.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_19_H51            CE_CAN_19.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_19_W51            CE_CAN_19.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_19_CS52           CE_CAN_19.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_19_ID52           CE_CAN_19.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_19_B52            CE_CAN_19.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_19_H52            CE_CAN_19.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_19_W52            CE_CAN_19.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_19_CS53           CE_CAN_19.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_19_ID53           CE_CAN_19.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_19_B53            CE_CAN_19.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_19_H53            CE_CAN_19.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_19_W53            CE_CAN_19.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_19_CS54           CE_CAN_19.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_19_ID54           CE_CAN_19.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_19_B54            CE_CAN_19.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_19_H54            CE_CAN_19.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_19_W54            CE_CAN_19.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_19_CS55           CE_CAN_19.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_19_ID55           CE_CAN_19.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_19_B55            CE_CAN_19.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_19_H55            CE_CAN_19.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_19_W55            CE_CAN_19.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_19_CS56           CE_CAN_19.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_19_ID56           CE_CAN_19.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_19_B56            CE_CAN_19.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_19_H56            CE_CAN_19.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_19_W56            CE_CAN_19.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_19_CS57           CE_CAN_19.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_19_ID57           CE_CAN_19.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_19_B57            CE_CAN_19.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_19_H57            CE_CAN_19.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_19_W57            CE_CAN_19.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_19_CS58           CE_CAN_19.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_19_ID58           CE_CAN_19.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_19_B58            CE_CAN_19.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_19_H58            CE_CAN_19.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_19_W58            CE_CAN_19.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_19_CS59           CE_CAN_19.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_19_ID59           CE_CAN_19.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_19_B59            CE_CAN_19.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_19_H59            CE_CAN_19.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_19_W59            CE_CAN_19.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_19_CS60           CE_CAN_19.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_19_ID60           CE_CAN_19.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_19_B60            CE_CAN_19.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_19_H60            CE_CAN_19.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_19_W60            CE_CAN_19.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_19_CS61           CE_CAN_19.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_19_ID61           CE_CAN_19.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_19_B61            CE_CAN_19.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_19_H61            CE_CAN_19.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_19_W61            CE_CAN_19.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_19_CS62           CE_CAN_19.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_19_ID62           CE_CAN_19.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_19_B62            CE_CAN_19.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_19_H62            CE_CAN_19.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_19_W62            CE_CAN_19.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_19_CS63           CE_CAN_19.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_19_ID63           CE_CAN_19.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_19_B63            CE_CAN_19.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_19_H63            CE_CAN_19.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_19_W63            CE_CAN_19.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_19_CS64           CE_CAN_19.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_19_ID64           CE_CAN_19.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_19_B64            CE_CAN_19.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_19_H64            CE_CAN_19.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_19_W64            CE_CAN_19.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_19_CS65           CE_CAN_19.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_19_ID65           CE_CAN_19.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_19_B65            CE_CAN_19.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_19_H65            CE_CAN_19.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_19_W65            CE_CAN_19.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_19_CS66           CE_CAN_19.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_19_ID66           CE_CAN_19.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_19_B66            CE_CAN_19.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_19_H66            CE_CAN_19.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_19_W66            CE_CAN_19.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_19_CS67           CE_CAN_19.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_19_ID67           CE_CAN_19.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_19_B67            CE_CAN_19.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_19_H67            CE_CAN_19.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_19_W67            CE_CAN_19.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_19_CS68           CE_CAN_19.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_19_ID68           CE_CAN_19.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_19_B68            CE_CAN_19.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_19_H68            CE_CAN_19.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_19_W68            CE_CAN_19.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_19_CS69           CE_CAN_19.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_19_ID69           CE_CAN_19.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_19_B69            CE_CAN_19.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_19_H69            CE_CAN_19.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_19_W69            CE_CAN_19.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_19_CS70           CE_CAN_19.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_19_ID70           CE_CAN_19.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_19_B70            CE_CAN_19.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_19_H70            CE_CAN_19.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_19_W70            CE_CAN_19.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_19_CS71           CE_CAN_19.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_19_ID71           CE_CAN_19.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_19_B71            CE_CAN_19.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_19_H71            CE_CAN_19.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_19_W71            CE_CAN_19.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_19_CS72           CE_CAN_19.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_19_ID72           CE_CAN_19.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_19_B72            CE_CAN_19.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_19_H72            CE_CAN_19.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_19_W72            CE_CAN_19.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_19_CS73           CE_CAN_19.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_19_ID73           CE_CAN_19.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_19_B73            CE_CAN_19.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_19_H73            CE_CAN_19.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_19_W73            CE_CAN_19.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_19_CS74           CE_CAN_19.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_19_ID74           CE_CAN_19.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_19_B74            CE_CAN_19.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_19_H74            CE_CAN_19.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_19_W74            CE_CAN_19.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_19_CS75           CE_CAN_19.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_19_ID75           CE_CAN_19.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_19_B75            CE_CAN_19.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_19_H75            CE_CAN_19.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_19_W75            CE_CAN_19.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_19_CS76           CE_CAN_19.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_19_ID76           CE_CAN_19.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_19_B76            CE_CAN_19.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_19_H76            CE_CAN_19.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_19_W76            CE_CAN_19.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_19_CS77           CE_CAN_19.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_19_ID77           CE_CAN_19.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_19_B77            CE_CAN_19.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_19_H77            CE_CAN_19.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_19_W77            CE_CAN_19.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_19_CS78           CE_CAN_19.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_19_ID78           CE_CAN_19.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_19_B78            CE_CAN_19.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_19_H78            CE_CAN_19.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_19_W78            CE_CAN_19.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_19_CS79           CE_CAN_19.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_19_ID79           CE_CAN_19.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_19_B79            CE_CAN_19.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_19_H79            CE_CAN_19.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_19_W79            CE_CAN_19.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_19_CS80           CE_CAN_19.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_19_ID80           CE_CAN_19.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_19_B80            CE_CAN_19.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_19_H80            CE_CAN_19.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_19_W80            CE_CAN_19.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_19_CS81           CE_CAN_19.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_19_ID81           CE_CAN_19.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_19_B81            CE_CAN_19.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_19_H81            CE_CAN_19.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_19_W81            CE_CAN_19.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_19_CS82           CE_CAN_19.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_19_ID82           CE_CAN_19.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_19_B82            CE_CAN_19.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_19_H82            CE_CAN_19.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_19_W82            CE_CAN_19.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_19_CS83           CE_CAN_19.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_19_ID83           CE_CAN_19.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_19_B83            CE_CAN_19.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_19_H83            CE_CAN_19.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_19_W83            CE_CAN_19.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_19_CS84           CE_CAN_19.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_19_ID84           CE_CAN_19.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_19_B84            CE_CAN_19.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_19_H84            CE_CAN_19.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_19_W84            CE_CAN_19.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_19_CS85           CE_CAN_19.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_19_ID85           CE_CAN_19.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_19_B85            CE_CAN_19.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_19_H85            CE_CAN_19.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_19_W85            CE_CAN_19.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_19_CS86           CE_CAN_19.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_19_ID86           CE_CAN_19.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_19_B86            CE_CAN_19.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_19_H86            CE_CAN_19.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_19_W86            CE_CAN_19.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_19_CS87           CE_CAN_19.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_19_ID87           CE_CAN_19.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_19_B87            CE_CAN_19.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_19_H87            CE_CAN_19.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_19_W87            CE_CAN_19.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_19_CS88           CE_CAN_19.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_19_ID88           CE_CAN_19.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_19_B88            CE_CAN_19.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_19_H88            CE_CAN_19.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_19_W848           CE_CAN_19.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_19_CS89           CE_CAN_19.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_19_ID89           CE_CAN_19.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_19_B89            CE_CAN_19.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_19_H89            CE_CAN_19.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_19_W89            CE_CAN_19.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_19_CS90           CE_CAN_19.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_19_ID90           CE_CAN_19.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_19_B90            CE_CAN_19.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_19_H90            CE_CAN_19.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_19_W90            CE_CAN_19.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_19_CS91           CE_CAN_19.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_19_ID91           CE_CAN_19.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_19_B91            CE_CAN_19.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_19_H91            CE_CAN_19.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_19_W91            CE_CAN_19.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_19_CS92           CE_CAN_19.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_19_ID92           CE_CAN_19.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_19_B92            CE_CAN_19.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_19_H92            CE_CAN_19.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_19_W92            CE_CAN_19.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_19_CS93           CE_CAN_19.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_19_ID93           CE_CAN_19.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_19_B93            CE_CAN_19.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_19_H93            CE_CAN_19.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_19_W93            CE_CAN_19.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_19_CS94           CE_CAN_19.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_19_ID94           CE_CAN_19.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_19_B94            CE_CAN_19.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_19_H94            CE_CAN_19.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_19_W94            CE_CAN_19.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_19_CS95           CE_CAN_19.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_19_ID95           CE_CAN_19.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_19_B95            CE_CAN_19.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_19_H95            CE_CAN_19.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_19_W95            CE_CAN_19.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_19_CS96           CE_CAN_19.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_19_ID96           CE_CAN_19.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_19_B96            CE_CAN_19.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_19_H96            CE_CAN_19.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_19_W96            CE_CAN_19.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_19_CS97           CE_CAN_19.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_19_ID97           CE_CAN_19.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_19_B97            CE_CAN_19.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_19_H97            CE_CAN_19.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_19_W97            CE_CAN_19.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_19_CS98           CE_CAN_19.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_19_ID98           CE_CAN_19.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_19_B98            CE_CAN_19.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_19_H98            CE_CAN_19.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_19_W98            CE_CAN_19.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_19_CS99           CE_CAN_19.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_19_ID99           CE_CAN_19.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_19_B99            CE_CAN_19.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_19_H99            CE_CAN_19.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_19_W99            CE_CAN_19.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_19_CS100          CE_CAN_19.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_19_ID100          CE_CAN_19.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_19_B100           CE_CAN_19.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_19_H100           CE_CAN_19.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_19_W100           CE_CAN_19.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_19_CS101          CE_CAN_19.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_19_ID101          CE_CAN_19.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_19_B101           CE_CAN_19.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_19_H101           CE_CAN_19.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_19_W101           CE_CAN_19.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_19_CS102          CE_CAN_19.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_19_ID102          CE_CAN_19.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_19_B102           CE_CAN_19.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_19_H102           CE_CAN_19.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_19_W102           CE_CAN_19.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_19_CS103          CE_CAN_19.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_19_ID103          CE_CAN_19.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_19_B103           CE_CAN_19.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_19_H103           CE_CAN_19.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_19_W103           CE_CAN_19.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_19_CS104          CE_CAN_19.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_19_ID104          CE_CAN_19.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_19_B104           CE_CAN_19.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_19_H104           CE_CAN_19.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_19_W104           CE_CAN_19.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_19_CS105          CE_CAN_19.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_19_ID105          CE_CAN_19.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_19_B105           CE_CAN_19.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_19_H105           CE_CAN_19.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_19_W105           CE_CAN_19.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_19_CS106          CE_CAN_19.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_19_ID106          CE_CAN_19.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_19_B106           CE_CAN_19.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_19_H106           CE_CAN_19.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_19_W106           CE_CAN_19.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_19_CS107          CE_CAN_19.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_19_ID107          CE_CAN_19.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_19_B107           CE_CAN_19.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_19_H107           CE_CAN_19.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_19_W107           CE_CAN_19.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_19_CS108          CE_CAN_19.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_19_ID108          CE_CAN_19.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_19_B108           CE_CAN_19.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_19_H108           CE_CAN_19.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_19_W108           CE_CAN_19.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_19_CS109          CE_CAN_19.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_19_ID109          CE_CAN_19.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_19_B109           CE_CAN_19.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_19_H109           CE_CAN_19.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_19_W109           CE_CAN_19.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_19_CS110          CE_CAN_19.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_19_ID110          CE_CAN_19.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_19_B110           CE_CAN_19.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_19_H110           CE_CAN_19.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_19_W110           CE_CAN_19.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_19_CS111          CE_CAN_19.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_19_ID111          CE_CAN_19.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_19_B111           CE_CAN_19.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_19_H111           CE_CAN_19.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_19_W111           CE_CAN_19.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_19_CS112          CE_CAN_19.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_19_ID112          CE_CAN_19.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_19_B112           CE_CAN_19.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_19_H112           CE_CAN_19.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_19_W112           CE_CAN_19.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_19_CS113          CE_CAN_19.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_19_ID113          CE_CAN_19.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_19_B113           CE_CAN_19.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_19_H113           CE_CAN_19.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_19_W113           CE_CAN_19.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_19_CS114          CE_CAN_19.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_19_ID114          CE_CAN_19.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_19_B114           CE_CAN_19.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_19_H114           CE_CAN_19.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_19_W114           CE_CAN_19.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_19_CS115          CE_CAN_19.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_19_ID115          CE_CAN_19.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_19_B115           CE_CAN_19.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_19_H115           CE_CAN_19.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_19_W115           CE_CAN_19.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_19_CS116          CE_CAN_19.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_19_ID116          CE_CAN_19.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_19_B116           CE_CAN_19.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_19_H116           CE_CAN_19.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_19_W116           CE_CAN_19.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_19_CS117          CE_CAN_19.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_19_ID117          CE_CAN_19.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_19_B117           CE_CAN_19.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_19_H117           CE_CAN_19.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_19_W117           CE_CAN_19.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_19_CS118          CE_CAN_19.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_19_ID118          CE_CAN_19.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_19_B118           CE_CAN_19.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_19_H118           CE_CAN_19.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_19_W118           CE_CAN_19.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_19_CS119          CE_CAN_19.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_19_ID119          CE_CAN_19.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_19_B119           CE_CAN_19.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_19_H119           CE_CAN_19.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_19_W119           CE_CAN_19.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_19_CS120          CE_CAN_19.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_19_ID120          CE_CAN_19.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_19_B120           CE_CAN_19.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_19_H120           CE_CAN_19.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_19_W120           CE_CAN_19.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_19_CS121          CE_CAN_19.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_19_ID121          CE_CAN_19.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_19_B121           CE_CAN_19.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_19_H121           CE_CAN_19.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_19_W121           CE_CAN_19.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_19_CS122          CE_CAN_19.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_19_ID122          CE_CAN_19.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_19_B122           CE_CAN_19.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_19_H122           CE_CAN_19.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_19_W122           CE_CAN_19.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_19_CS123          CE_CAN_19.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_19_ID123          CE_CAN_19.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_19_B123           CE_CAN_19.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_19_H123           CE_CAN_19.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_19_W123           CE_CAN_19.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_19_CS124          CE_CAN_19.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_19_ID124          CE_CAN_19.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_19_B124           CE_CAN_19.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_19_H124           CE_CAN_19.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_19_W124           CE_CAN_19.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_19_CS125          CE_CAN_19.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_19_ID125          CE_CAN_19.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_19_B125           CE_CAN_19.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_19_H125           CE_CAN_19.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_19_W125           CE_CAN_19.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_19_CS126          CE_CAN_19.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_19_ID126          CE_CAN_19.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_19_B126           CE_CAN_19.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_19_H126           CE_CAN_19.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_19_W126           CE_CAN_19.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_19_CS127          CE_CAN_19.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_19_ID127          CE_CAN_19.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_19_B127           CE_CAN_19.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_19_H127           CE_CAN_19.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_19_W127           CE_CAN_19.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_19_RXIMR0     CE_CAN_19.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR1     CE_CAN_19.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR2     CE_CAN_19.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR3     CE_CAN_19.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR4     CE_CAN_19.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR5     CE_CAN_19.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR6     CE_CAN_19.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR7     CE_CAN_19.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR8     CE_CAN_19.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR9     CE_CAN_19.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_19_RXIMR10    CE_CAN_19.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR11    CE_CAN_19.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR12    CE_CAN_19.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR13    CE_CAN_19.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR14    CE_CAN_19.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR15    CE_CAN_19.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR16    CE_CAN_19.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR17    CE_CAN_19.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR18    CE_CAN_19.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR19    CE_CAN_19.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR20    CE_CAN_19.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR21    CE_CAN_19.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR22    CE_CAN_19.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR23    CE_CAN_19.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR24    CE_CAN_19.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR25    CE_CAN_19.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR26    CE_CAN_19.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR27    CE_CAN_19.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR28    CE_CAN_19.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR29    CE_CAN_19.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR30    CE_CAN_19.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR31    CE_CAN_19.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR32    CE_CAN_19.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR33    CE_CAN_19.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR34    CE_CAN_19.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR35    CE_CAN_19.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR36    CE_CAN_19.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR37    CE_CAN_19.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR38    CE_CAN_19.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR39    CE_CAN_19.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR40    CE_CAN_19.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR41    CE_CAN_19.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR42    CE_CAN_19.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR43    CE_CAN_19.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR44    CE_CAN_19.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR45    CE_CAN_19.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR46    CE_CAN_19.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR47    CE_CAN_19.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR48    CE_CAN_19.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR49    CE_CAN_19.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR50    CE_CAN_19.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR51    CE_CAN_19.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR52    CE_CAN_19.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR53    CE_CAN_19.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR54    CE_CAN_19.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR55    CE_CAN_19.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR56    CE_CAN_19.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR57    CE_CAN_19.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR58    CE_CAN_19.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR59    CE_CAN_19.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR60    CE_CAN_19.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR61    CE_CAN_19.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR62    CE_CAN_19.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR63    CE_CAN_19.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR64    CE_CAN_19.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR65    CE_CAN_19.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR66    CE_CAN_19.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR67    CE_CAN_19.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR68    CE_CAN_19.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR69    CE_CAN_19.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR70    CE_CAN_19.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR71    CE_CAN_19.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR72    CE_CAN_19.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR73    CE_CAN_19.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR74    CE_CAN_19.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR75    CE_CAN_19.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR76    CE_CAN_19.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR77    CE_CAN_19.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR78    CE_CAN_19.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR79    CE_CAN_19.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR80    CE_CAN_19.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR81    CE_CAN_19.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR82    CE_CAN_19.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR83    CE_CAN_19.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR84    CE_CAN_19.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR85    CE_CAN_19.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR86    CE_CAN_19.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR87    CE_CAN_19.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR88    CE_CAN_19.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR89    CE_CAN_19.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR90    CE_CAN_19.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR91    CE_CAN_19.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR92    CE_CAN_19.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR93    CE_CAN_19.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR94    CE_CAN_19.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR95    CE_CAN_19.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR96    CE_CAN_19.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR97    CE_CAN_19.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR98    CE_CAN_19.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR99    CE_CAN_19.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_19_RXIMR100   CE_CAN_19.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR101   CE_CAN_19.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR102   CE_CAN_19.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR103   CE_CAN_19.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR104   CE_CAN_19.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR105   CE_CAN_19.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR106   CE_CAN_19.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR107   CE_CAN_19.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR108   CE_CAN_19.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR109   CE_CAN_19.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR110   CE_CAN_19.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR111   CE_CAN_19.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR112   CE_CAN_19.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR113   CE_CAN_19.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR114   CE_CAN_19.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR115   CE_CAN_19.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR116   CE_CAN_19.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR117   CE_CAN_19.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR118   CE_CAN_19.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR119   CE_CAN_19.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR120   CE_CAN_19.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR121   CE_CAN_19.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR122   CE_CAN_19.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR123   CE_CAN_19.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR124   CE_CAN_19.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR125   CE_CAN_19.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR126   CE_CAN_19.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_19_RXIMR127   CE_CAN_19.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_19_MECR       CE_CAN_19.MECR.R              /* Memory Error Control */
#define CE_CAN_19_ERRIAR     CE_CAN_19.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_19_ERRIDPR    CE_CAN_19.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_19_ERRIPPR    CE_CAN_19.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_19_RERRAR     CE_CAN_19.RERRAR.R            /* Error Report Address */
#define CE_CAN_19_RERRDR     CE_CAN_19.RERRDR.R            /* Error Report Data */
#define CE_CAN_19_RERRSYNR   CE_CAN_19.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_19_ERRSR      CE_CAN_19.ERRSR.R             /* Error Status */
#define CE_CAN_19_EPRS       CE_CAN_19.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_19_ENCBT      CE_CAN_19.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_19_EDCBT      CE_CAN_19.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_19_ETDC       CE_CAN_19.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_19_FDCTRL     CE_CAN_19.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_19_FDCBT      CE_CAN_19.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_19_FDCRC      CE_CAN_19.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_19_ERFCR      CE_CAN_19.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_19_ERFIER     CE_CAN_19.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_19_ERFSR      CE_CAN_19.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_19_HR_TIME_STAMP0 CE_CAN_19.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP1 CE_CAN_19.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP2 CE_CAN_19.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP3 CE_CAN_19.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP4 CE_CAN_19.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP5 CE_CAN_19.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP6 CE_CAN_19.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP7 CE_CAN_19.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP8 CE_CAN_19.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP9 CE_CAN_19.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP10 CE_CAN_19.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP11 CE_CAN_19.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP12 CE_CAN_19.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP13 CE_CAN_19.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP14 CE_CAN_19.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP15 CE_CAN_19.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP16 CE_CAN_19.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP17 CE_CAN_19.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP18 CE_CAN_19.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP19 CE_CAN_19.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP20 CE_CAN_19.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP21 CE_CAN_19.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP22 CE_CAN_19.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP23 CE_CAN_19.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP24 CE_CAN_19.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP25 CE_CAN_19.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP26 CE_CAN_19.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP27 CE_CAN_19.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP28 CE_CAN_19.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP29 CE_CAN_19.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP30 CE_CAN_19.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP31 CE_CAN_19.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP32 CE_CAN_19.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP33 CE_CAN_19.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP34 CE_CAN_19.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP35 CE_CAN_19.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP36 CE_CAN_19.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP37 CE_CAN_19.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP38 CE_CAN_19.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP39 CE_CAN_19.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP40 CE_CAN_19.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP41 CE_CAN_19.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP42 CE_CAN_19.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP43 CE_CAN_19.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP44 CE_CAN_19.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP45 CE_CAN_19.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP46 CE_CAN_19.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP47 CE_CAN_19.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP48 CE_CAN_19.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP49 CE_CAN_19.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP50 CE_CAN_19.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP51 CE_CAN_19.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP52 CE_CAN_19.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP53 CE_CAN_19.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP54 CE_CAN_19.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP55 CE_CAN_19.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP56 CE_CAN_19.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP57 CE_CAN_19.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP58 CE_CAN_19.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP59 CE_CAN_19.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP60 CE_CAN_19.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP61 CE_CAN_19.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP62 CE_CAN_19.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP63 CE_CAN_19.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP64 CE_CAN_19.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP65 CE_CAN_19.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP66 CE_CAN_19.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP67 CE_CAN_19.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP68 CE_CAN_19.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP69 CE_CAN_19.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP70 CE_CAN_19.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP71 CE_CAN_19.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP72 CE_CAN_19.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP73 CE_CAN_19.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP74 CE_CAN_19.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP75 CE_CAN_19.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP76 CE_CAN_19.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP77 CE_CAN_19.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP78 CE_CAN_19.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP79 CE_CAN_19.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP80 CE_CAN_19.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP81 CE_CAN_19.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP82 CE_CAN_19.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP83 CE_CAN_19.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP84 CE_CAN_19.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP85 CE_CAN_19.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP86 CE_CAN_19.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP87 CE_CAN_19.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP88 CE_CAN_19.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP89 CE_CAN_19.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP90 CE_CAN_19.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP91 CE_CAN_19.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP92 CE_CAN_19.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP93 CE_CAN_19.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP94 CE_CAN_19.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP95 CE_CAN_19.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP96 CE_CAN_19.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP97 CE_CAN_19.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP98 CE_CAN_19.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP99 CE_CAN_19.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP100 CE_CAN_19.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP101 CE_CAN_19.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP102 CE_CAN_19.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP103 CE_CAN_19.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP104 CE_CAN_19.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP105 CE_CAN_19.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP106 CE_CAN_19.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP107 CE_CAN_19.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP108 CE_CAN_19.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP109 CE_CAN_19.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP110 CE_CAN_19.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP111 CE_CAN_19.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP112 CE_CAN_19.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP113 CE_CAN_19.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP114 CE_CAN_19.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP115 CE_CAN_19.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP116 CE_CAN_19.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP117 CE_CAN_19.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP118 CE_CAN_19.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP119 CE_CAN_19.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP120 CE_CAN_19.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP121 CE_CAN_19.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP122 CE_CAN_19.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP123 CE_CAN_19.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP124 CE_CAN_19.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP125 CE_CAN_19.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP126 CE_CAN_19.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_19_HR_TIME_STAMP127 CE_CAN_19.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_19_ERFFEL0    CE_CAN_19.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL1    CE_CAN_19.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL2    CE_CAN_19.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL3    CE_CAN_19.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL4    CE_CAN_19.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL5    CE_CAN_19.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL6    CE_CAN_19.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL7    CE_CAN_19.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL8    CE_CAN_19.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL9    CE_CAN_19.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL10   CE_CAN_19.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL11   CE_CAN_19.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL12   CE_CAN_19.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL13   CE_CAN_19.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL14   CE_CAN_19.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL15   CE_CAN_19.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL16   CE_CAN_19.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL17   CE_CAN_19.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL18   CE_CAN_19.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL19   CE_CAN_19.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL20   CE_CAN_19.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL21   CE_CAN_19.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL22   CE_CAN_19.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL23   CE_CAN_19.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL24   CE_CAN_19.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL25   CE_CAN_19.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL26   CE_CAN_19.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL27   CE_CAN_19.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL28   CE_CAN_19.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL29   CE_CAN_19.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL30   CE_CAN_19.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL31   CE_CAN_19.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL32   CE_CAN_19.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL33   CE_CAN_19.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL34   CE_CAN_19.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL35   CE_CAN_19.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL36   CE_CAN_19.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL37   CE_CAN_19.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL38   CE_CAN_19.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL39   CE_CAN_19.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL40   CE_CAN_19.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL41   CE_CAN_19.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL42   CE_CAN_19.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL43   CE_CAN_19.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL44   CE_CAN_19.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL45   CE_CAN_19.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL46   CE_CAN_19.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL47   CE_CAN_19.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL48   CE_CAN_19.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL49   CE_CAN_19.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL50   CE_CAN_19.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL51   CE_CAN_19.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL52   CE_CAN_19.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL53   CE_CAN_19.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL54   CE_CAN_19.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL55   CE_CAN_19.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL56   CE_CAN_19.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL57   CE_CAN_19.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL58   CE_CAN_19.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL59   CE_CAN_19.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL60   CE_CAN_19.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL61   CE_CAN_19.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL62   CE_CAN_19.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL63   CE_CAN_19.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL64   CE_CAN_19.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL65   CE_CAN_19.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL66   CE_CAN_19.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL67   CE_CAN_19.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL68   CE_CAN_19.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL69   CE_CAN_19.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL70   CE_CAN_19.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL71   CE_CAN_19.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL72   CE_CAN_19.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL73   CE_CAN_19.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL74   CE_CAN_19.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL75   CE_CAN_19.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL76   CE_CAN_19.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL77   CE_CAN_19.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL78   CE_CAN_19.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL79   CE_CAN_19.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL80   CE_CAN_19.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL81   CE_CAN_19.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL82   CE_CAN_19.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL83   CE_CAN_19.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL84   CE_CAN_19.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL85   CE_CAN_19.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL86   CE_CAN_19.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL87   CE_CAN_19.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL88   CE_CAN_19.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL89   CE_CAN_19.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL90   CE_CAN_19.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL91   CE_CAN_19.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL92   CE_CAN_19.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL93   CE_CAN_19.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL94   CE_CAN_19.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL95   CE_CAN_19.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL96   CE_CAN_19.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL97   CE_CAN_19.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL98   CE_CAN_19.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL99   CE_CAN_19.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL100  CE_CAN_19.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL101  CE_CAN_19.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL102  CE_CAN_19.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL103  CE_CAN_19.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL104  CE_CAN_19.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL105  CE_CAN_19.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL106  CE_CAN_19.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL107  CE_CAN_19.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL108  CE_CAN_19.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL109  CE_CAN_19.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL110  CE_CAN_19.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL111  CE_CAN_19.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL112  CE_CAN_19.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL113  CE_CAN_19.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL114  CE_CAN_19.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL115  CE_CAN_19.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL116  CE_CAN_19.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL117  CE_CAN_19.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL118  CE_CAN_19.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL119  CE_CAN_19.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL120  CE_CAN_19.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL121  CE_CAN_19.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL122  CE_CAN_19.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL123  CE_CAN_19.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL124  CE_CAN_19.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL125  CE_CAN_19.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL126  CE_CAN_19.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_19_ERFFEL127  CE_CAN_19.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_2_MCR         CE_CAN_2.MCR.R                /* Module Configuration */
#define CE_CAN_2_CTRL1       CE_CAN_2.CTRL1.R              /* Control 1 */
#define CE_CAN_2_TIMER       CE_CAN_2.TIMER.R              /* Free-Running Timer */
#define CE_CAN_2_RXMGMASK    CE_CAN_2.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_2_RX14MASK    CE_CAN_2.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_2_RX15MASK    CE_CAN_2.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_2_ECR         CE_CAN_2.ECR.R                /* Error Counter */
#define CE_CAN_2_ESR1        CE_CAN_2.ESR1.R               /* Error and Status 1 */
#define CE_CAN_2_IMASK2      CE_CAN_2.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_2_IMASK1      CE_CAN_2.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_2_IFLAG2      CE_CAN_2.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_2_IFLAG1      CE_CAN_2.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_2_CTRL2       CE_CAN_2.CTRL2.R              /* Control 2 */
#define CE_CAN_2_ESR2        CE_CAN_2.ESR2.R               /* Error and Status 2 */
#define CE_CAN_2_CRCR        CE_CAN_2.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_2_RXFGMASK    CE_CAN_2.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_2_RXFIR       CE_CAN_2.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_2_CBT         CE_CAN_2.CBT.R                /* CAN Bit Timing */
#define CE_CAN_2_IMASK4      CE_CAN_2.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_2_IMASK3      CE_CAN_2.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_2_IFLAG4      CE_CAN_2.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_2_IFLAG3      CE_CAN_2.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_2_CS0            CE_CAN_2.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_2_ID0            CE_CAN_2.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_2_B0             CE_CAN_2.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_2_H0             CE_CAN_2.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_2_W0             CE_CAN_2.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_2_CS1            CE_CAN_2.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_2_ID1            CE_CAN_2.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_2_B1             CE_CAN_2.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_2_H1             CE_CAN_2.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_2_W1             CE_CAN_2.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_2_CS2            CE_CAN_2.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_2_ID2            CE_CAN_2.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_2_B2             CE_CAN_2.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_2_H2             CE_CAN_2.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_2_W2             CE_CAN_2.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_2_CS3            CE_CAN_2.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_2_ID3            CE_CAN_2.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_2_B3             CE_CAN_2.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_2_H3             CE_CAN_2.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_2_W3             CE_CAN_2.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_2_CS4            CE_CAN_2.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_2_ID4            CE_CAN_2.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_2_B4             CE_CAN_2.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_2_H4             CE_CAN_2.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_2_W4             CE_CAN_2.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_2_CS5            CE_CAN_2.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_2_ID5            CE_CAN_2.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_2_B5             CE_CAN_2.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_2_H5             CE_CAN_2.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_2_W5             CE_CAN_2.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_2_CS6            CE_CAN_2.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_2_ID6            CE_CAN_2.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_2_B6             CE_CAN_2.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_2_H6             CE_CAN_2.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_2_W6             CE_CAN_2.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_2_CS7            CE_CAN_2.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_2_ID7            CE_CAN_2.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_2_B7             CE_CAN_2.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_2_H7             CE_CAN_2.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_2_W7             CE_CAN_2.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_2_CS8            CE_CAN_2.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_2_ID8            CE_CAN_2.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_2_B8             CE_CAN_2.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_2_H8             CE_CAN_2.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_2_W8             CE_CAN_2.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_2_CS9            CE_CAN_2.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_2_ID9            CE_CAN_2.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_2_B9             CE_CAN_2.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_2_H9             CE_CAN_2.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_2_W9             CE_CAN_2.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_2_CS10           CE_CAN_2.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_2_ID10           CE_CAN_2.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_2_B10            CE_CAN_2.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_2_H10            CE_CAN_2.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_2_W10            CE_CAN_2.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_2_CS11           CE_CAN_2.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_2_ID11           CE_CAN_2.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_2_B11            CE_CAN_2.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_2_H11            CE_CAN_2.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_2_W11            CE_CAN_2.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_2_CS12           CE_CAN_2.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_2_ID12           CE_CAN_2.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_2_B12            CE_CAN_2.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_2_H12            CE_CAN_2.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_2_W12            CE_CAN_2.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_2_CS13           CE_CAN_2.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_2_ID13           CE_CAN_2.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_2_B13            CE_CAN_2.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_2_H13            CE_CAN_2.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_2_W13            CE_CAN_2.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_2_CS14           CE_CAN_2.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_2_ID14           CE_CAN_2.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_2_B14            CE_CAN_2.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_2_H14            CE_CAN_2.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_2_W14            CE_CAN_2.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_2_CS15           CE_CAN_2.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_2_ID15           CE_CAN_2.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_2_B15            CE_CAN_2.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_2_H15            CE_CAN_2.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_2_W15            CE_CAN_2.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_2_CS16           CE_CAN_2.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_2_ID16           CE_CAN_2.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_2_B16            CE_CAN_2.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_2_H16            CE_CAN_2.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_2_W16            CE_CAN_2.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_2_CS17           CE_CAN_2.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_2_ID17           CE_CAN_2.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_2_B17            CE_CAN_2.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_2_H17            CE_CAN_2.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_2_W17            CE_CAN_2.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_2_CS18           CE_CAN_2.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_2_ID18           CE_CAN_2.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_2_B18            CE_CAN_2.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_2_H18            CE_CAN_2.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_2_W18            CE_CAN_2.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_2_CS19           CE_CAN_2.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_2_ID19           CE_CAN_2.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_2_B19            CE_CAN_2.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_2_H19            CE_CAN_2.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_2_W19            CE_CAN_2.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_2_CS20           CE_CAN_2.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_2_ID20           CE_CAN_2.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_2_B20            CE_CAN_2.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_2_H20            CE_CAN_2.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_2_W20            CE_CAN_2.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_2_CS21           CE_CAN_2.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_2_ID21           CE_CAN_2.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_2_B21            CE_CAN_2.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_2_H21            CE_CAN_2.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_2_W21            CE_CAN_2.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_2_CS22           CE_CAN_2.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_2_ID22           CE_CAN_2.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_2_B22            CE_CAN_2.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_2_H22            CE_CAN_2.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_2_W22            CE_CAN_2.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_2_CS23           CE_CAN_2.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_2_ID23           CE_CAN_2.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_2_B23            CE_CAN_2.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_2_H23            CE_CAN_2.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_2_W23            CE_CAN_2.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_2_CS24           CE_CAN_2.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_2_ID24           CE_CAN_2.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_2_B24            CE_CAN_2.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_2_H24            CE_CAN_2.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_2_W24            CE_CAN_2.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_2_CS25           CE_CAN_2.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_2_ID25           CE_CAN_2.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_2_B25            CE_CAN_2.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_2_H25            CE_CAN_2.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_2_W25            CE_CAN_2.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_2_CS26           CE_CAN_2.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_2_ID26           CE_CAN_2.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_2_B26            CE_CAN_2.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_2_H26            CE_CAN_2.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_2_W26            CE_CAN_2.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_2_CS27           CE_CAN_2.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_2_ID27           CE_CAN_2.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_2_B27            CE_CAN_2.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_2_H27            CE_CAN_2.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_2_W27            CE_CAN_2.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_2_CS28           CE_CAN_2.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_2_ID28           CE_CAN_2.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_2_B28            CE_CAN_2.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_2_H28            CE_CAN_2.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_2_W28            CE_CAN_2.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_2_CS29           CE_CAN_2.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_2_ID29           CE_CAN_2.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_2_B29            CE_CAN_2.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_2_H29            CE_CAN_2.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_2_W29            CE_CAN_2.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_2_CS30           CE_CAN_2.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_2_ID30           CE_CAN_2.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_2_B30            CE_CAN_2.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_2_H30            CE_CAN_2.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_2_W30            CE_CAN_2.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_2_CS31           CE_CAN_2.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_2_ID31           CE_CAN_2.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_2_B31            CE_CAN_2.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_2_H31            CE_CAN_2.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_2_W31            CE_CAN_2.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_2_CS32           CE_CAN_2.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_2_ID32           CE_CAN_2.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_2_B32            CE_CAN_2.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_2_H32            CE_CAN_2.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_2_W32            CE_CAN_2.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_2_CS33           CE_CAN_2.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_2_ID33           CE_CAN_2.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_2_B33            CE_CAN_2.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_2_H33            CE_CAN_2.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_2_W33            CE_CAN_2.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_2_CS34           CE_CAN_2.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_2_ID34           CE_CAN_2.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_2_B34            CE_CAN_2.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_2_H34            CE_CAN_2.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_2_W34            CE_CAN_2.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_2_CS35           CE_CAN_2.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_2_ID35           CE_CAN_2.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_2_B35            CE_CAN_2.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_2_H35            CE_CAN_2.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_2_W35            CE_CAN_2.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_2_CS36           CE_CAN_2.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_2_ID36           CE_CAN_2.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_2_B36            CE_CAN_2.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_2_H36            CE_CAN_2.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_2_W36            CE_CAN_2.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_2_CS37           CE_CAN_2.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_2_ID37           CE_CAN_2.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_2_B37            CE_CAN_2.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_2_H37            CE_CAN_2.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_2_W37            CE_CAN_2.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_2_CS38           CE_CAN_2.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_2_ID38           CE_CAN_2.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_2_B38            CE_CAN_2.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_2_H38            CE_CAN_2.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_2_W38            CE_CAN_2.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_2_CS39           CE_CAN_2.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_2_ID39           CE_CAN_2.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_2_B39            CE_CAN_2.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_2_H39            CE_CAN_2.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_2_W39            CE_CAN_2.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_2_CS40           CE_CAN_2.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_2_ID40           CE_CAN_2.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_2_B40            CE_CAN_2.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_2_H40            CE_CAN_2.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_2_W40            CE_CAN_2.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_2_CS41           CE_CAN_2.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_2_ID41           CE_CAN_2.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_2_B41            CE_CAN_2.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_2_H41            CE_CAN_2.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_2_W41            CE_CAN_2.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_2_CS42           CE_CAN_2.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_2_ID42           CE_CAN_2.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_2_B42            CE_CAN_2.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_2_H42            CE_CAN_2.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_2_W42            CE_CAN_2.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_2_CS43           CE_CAN_2.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_2_ID43           CE_CAN_2.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_2_B43            CE_CAN_2.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_2_H43            CE_CAN_2.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_2_W43            CE_CAN_2.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_2_CS44           CE_CAN_2.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_2_ID44           CE_CAN_2.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_2_B44            CE_CAN_2.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_2_H44            CE_CAN_2.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_2_W44            CE_CAN_2.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_2_CS45           CE_CAN_2.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_2_ID45           CE_CAN_2.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_2_B45            CE_CAN_2.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_2_H45            CE_CAN_2.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_2_W45            CE_CAN_2.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_2_CS46           CE_CAN_2.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_2_ID46           CE_CAN_2.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_2_B46            CE_CAN_2.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_2_H46            CE_CAN_2.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_2_W46            CE_CAN_2.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_2_CS47           CE_CAN_2.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_2_ID47           CE_CAN_2.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_2_B47            CE_CAN_2.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_2_H47            CE_CAN_2.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_2_W47            CE_CAN_2.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_2_CS48           CE_CAN_2.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_2_ID48           CE_CAN_2.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_2_B48            CE_CAN_2.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_2_H48            CE_CAN_2.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_2_W48            CE_CAN_2.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_2_CS49           CE_CAN_2.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_2_ID49           CE_CAN_2.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_2_B49            CE_CAN_2.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_2_H49            CE_CAN_2.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_2_W49            CE_CAN_2.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_2_CS50           CE_CAN_2.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_2_ID50           CE_CAN_2.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_2_B50            CE_CAN_2.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_2_H50            CE_CAN_2.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_2_W50            CE_CAN_2.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_2_CS51           CE_CAN_2.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_2_ID51           CE_CAN_2.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_2_B51            CE_CAN_2.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_2_H51            CE_CAN_2.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_2_W51            CE_CAN_2.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_2_CS52           CE_CAN_2.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_2_ID52           CE_CAN_2.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_2_B52            CE_CAN_2.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_2_H52            CE_CAN_2.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_2_W52            CE_CAN_2.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_2_CS53           CE_CAN_2.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_2_ID53           CE_CAN_2.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_2_B53            CE_CAN_2.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_2_H53            CE_CAN_2.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_2_W53            CE_CAN_2.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_2_CS54           CE_CAN_2.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_2_ID54           CE_CAN_2.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_2_B54            CE_CAN_2.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_2_H54            CE_CAN_2.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_2_W54            CE_CAN_2.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_2_CS55           CE_CAN_2.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_2_ID55           CE_CAN_2.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_2_B55            CE_CAN_2.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_2_H55            CE_CAN_2.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_2_W55            CE_CAN_2.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_2_CS56           CE_CAN_2.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_2_ID56           CE_CAN_2.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_2_B56            CE_CAN_2.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_2_H56            CE_CAN_2.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_2_W56            CE_CAN_2.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_2_CS57           CE_CAN_2.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_2_ID57           CE_CAN_2.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_2_B57            CE_CAN_2.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_2_H57            CE_CAN_2.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_2_W57            CE_CAN_2.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_2_CS58           CE_CAN_2.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_2_ID58           CE_CAN_2.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_2_B58            CE_CAN_2.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_2_H58            CE_CAN_2.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_2_W58            CE_CAN_2.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_2_CS59           CE_CAN_2.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_2_ID59           CE_CAN_2.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_2_B59            CE_CAN_2.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_2_H59            CE_CAN_2.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_2_W59            CE_CAN_2.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_2_CS60           CE_CAN_2.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_2_ID60           CE_CAN_2.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_2_B60            CE_CAN_2.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_2_H60            CE_CAN_2.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_2_W60            CE_CAN_2.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_2_CS61           CE_CAN_2.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_2_ID61           CE_CAN_2.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_2_B61            CE_CAN_2.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_2_H61            CE_CAN_2.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_2_W61            CE_CAN_2.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_2_CS62           CE_CAN_2.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_2_ID62           CE_CAN_2.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_2_B62            CE_CAN_2.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_2_H62            CE_CAN_2.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_2_W62            CE_CAN_2.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_2_CS63           CE_CAN_2.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_2_ID63           CE_CAN_2.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_2_B63            CE_CAN_2.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_2_H63            CE_CAN_2.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_2_W63            CE_CAN_2.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_2_CS64           CE_CAN_2.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_2_ID64           CE_CAN_2.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_2_B64            CE_CAN_2.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_2_H64            CE_CAN_2.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_2_W64            CE_CAN_2.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_2_CS65           CE_CAN_2.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_2_ID65           CE_CAN_2.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_2_B65            CE_CAN_2.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_2_H65            CE_CAN_2.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_2_W65            CE_CAN_2.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_2_CS66           CE_CAN_2.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_2_ID66           CE_CAN_2.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_2_B66            CE_CAN_2.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_2_H66            CE_CAN_2.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_2_W66            CE_CAN_2.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_2_CS67           CE_CAN_2.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_2_ID67           CE_CAN_2.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_2_B67            CE_CAN_2.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_2_H67            CE_CAN_2.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_2_W67            CE_CAN_2.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_2_CS68           CE_CAN_2.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_2_ID68           CE_CAN_2.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_2_B68            CE_CAN_2.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_2_H68            CE_CAN_2.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_2_W68            CE_CAN_2.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_2_CS69           CE_CAN_2.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_2_ID69           CE_CAN_2.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_2_B69            CE_CAN_2.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_2_H69            CE_CAN_2.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_2_W69            CE_CAN_2.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_2_CS70           CE_CAN_2.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_2_ID70           CE_CAN_2.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_2_B70            CE_CAN_2.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_2_H70            CE_CAN_2.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_2_W70            CE_CAN_2.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_2_CS71           CE_CAN_2.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_2_ID71           CE_CAN_2.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_2_B71            CE_CAN_2.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_2_H71            CE_CAN_2.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_2_W71            CE_CAN_2.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_2_CS72           CE_CAN_2.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_2_ID72           CE_CAN_2.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_2_B72            CE_CAN_2.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_2_H72            CE_CAN_2.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_2_W72            CE_CAN_2.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_2_CS73           CE_CAN_2.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_2_ID73           CE_CAN_2.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_2_B73            CE_CAN_2.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_2_H73            CE_CAN_2.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_2_W73            CE_CAN_2.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_2_CS74           CE_CAN_2.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_2_ID74           CE_CAN_2.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_2_B74            CE_CAN_2.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_2_H74            CE_CAN_2.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_2_W74            CE_CAN_2.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_2_CS75           CE_CAN_2.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_2_ID75           CE_CAN_2.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_2_B75            CE_CAN_2.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_2_H75            CE_CAN_2.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_2_W75            CE_CAN_2.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_2_CS76           CE_CAN_2.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_2_ID76           CE_CAN_2.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_2_B76            CE_CAN_2.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_2_H76            CE_CAN_2.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_2_W76            CE_CAN_2.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_2_CS77           CE_CAN_2.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_2_ID77           CE_CAN_2.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_2_B77            CE_CAN_2.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_2_H77            CE_CAN_2.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_2_W77            CE_CAN_2.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_2_CS78           CE_CAN_2.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_2_ID78           CE_CAN_2.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_2_B78            CE_CAN_2.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_2_H78            CE_CAN_2.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_2_W78            CE_CAN_2.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_2_CS79           CE_CAN_2.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_2_ID79           CE_CAN_2.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_2_B79            CE_CAN_2.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_2_H79            CE_CAN_2.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_2_W79            CE_CAN_2.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_2_CS80           CE_CAN_2.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_2_ID80           CE_CAN_2.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_2_B80            CE_CAN_2.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_2_H80            CE_CAN_2.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_2_W80            CE_CAN_2.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_2_CS81           CE_CAN_2.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_2_ID81           CE_CAN_2.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_2_B81            CE_CAN_2.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_2_H81            CE_CAN_2.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_2_W81            CE_CAN_2.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_2_CS82           CE_CAN_2.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_2_ID82           CE_CAN_2.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_2_B82            CE_CAN_2.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_2_H82            CE_CAN_2.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_2_W82            CE_CAN_2.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_2_CS83           CE_CAN_2.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_2_ID83           CE_CAN_2.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_2_B83            CE_CAN_2.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_2_H83            CE_CAN_2.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_2_W83            CE_CAN_2.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_2_CS84           CE_CAN_2.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_2_ID84           CE_CAN_2.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_2_B84            CE_CAN_2.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_2_H84            CE_CAN_2.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_2_W84            CE_CAN_2.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_2_CS85           CE_CAN_2.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_2_ID85           CE_CAN_2.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_2_B85            CE_CAN_2.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_2_H85            CE_CAN_2.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_2_W85            CE_CAN_2.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_2_CS86           CE_CAN_2.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_2_ID86           CE_CAN_2.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_2_B86            CE_CAN_2.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_2_H86            CE_CAN_2.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_2_W86            CE_CAN_2.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_2_CS87           CE_CAN_2.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_2_ID87           CE_CAN_2.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_2_B87            CE_CAN_2.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_2_H87            CE_CAN_2.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_2_W87            CE_CAN_2.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_2_CS88           CE_CAN_2.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_2_ID88           CE_CAN_2.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_2_B88            CE_CAN_2.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_2_H88            CE_CAN_2.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_2_W848           CE_CAN_2.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_2_CS89           CE_CAN_2.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_2_ID89           CE_CAN_2.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_2_B89            CE_CAN_2.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_2_H89            CE_CAN_2.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_2_W89            CE_CAN_2.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_2_CS90           CE_CAN_2.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_2_ID90           CE_CAN_2.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_2_B90            CE_CAN_2.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_2_H90            CE_CAN_2.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_2_W90            CE_CAN_2.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_2_CS91           CE_CAN_2.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_2_ID91           CE_CAN_2.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_2_B91            CE_CAN_2.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_2_H91            CE_CAN_2.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_2_W91            CE_CAN_2.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_2_CS92           CE_CAN_2.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_2_ID92           CE_CAN_2.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_2_B92            CE_CAN_2.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_2_H92            CE_CAN_2.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_2_W92            CE_CAN_2.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_2_CS93           CE_CAN_2.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_2_ID93           CE_CAN_2.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_2_B93            CE_CAN_2.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_2_H93            CE_CAN_2.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_2_W93            CE_CAN_2.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_2_CS94           CE_CAN_2.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_2_ID94           CE_CAN_2.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_2_B94            CE_CAN_2.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_2_H94            CE_CAN_2.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_2_W94            CE_CAN_2.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_2_CS95           CE_CAN_2.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_2_ID95           CE_CAN_2.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_2_B95            CE_CAN_2.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_2_H95            CE_CAN_2.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_2_W95            CE_CAN_2.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_2_CS96           CE_CAN_2.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_2_ID96           CE_CAN_2.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_2_B96            CE_CAN_2.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_2_H96            CE_CAN_2.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_2_W96            CE_CAN_2.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_2_CS97           CE_CAN_2.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_2_ID97           CE_CAN_2.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_2_B97            CE_CAN_2.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_2_H97            CE_CAN_2.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_2_W97            CE_CAN_2.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_2_CS98           CE_CAN_2.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_2_ID98           CE_CAN_2.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_2_B98            CE_CAN_2.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_2_H98            CE_CAN_2.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_2_W98            CE_CAN_2.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_2_CS99           CE_CAN_2.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_2_ID99           CE_CAN_2.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_2_B99            CE_CAN_2.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_2_H99            CE_CAN_2.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_2_W99            CE_CAN_2.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_2_CS100          CE_CAN_2.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_2_ID100          CE_CAN_2.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_2_B100           CE_CAN_2.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_2_H100           CE_CAN_2.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_2_W100           CE_CAN_2.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_2_CS101          CE_CAN_2.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_2_ID101          CE_CAN_2.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_2_B101           CE_CAN_2.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_2_H101           CE_CAN_2.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_2_W101           CE_CAN_2.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_2_CS102          CE_CAN_2.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_2_ID102          CE_CAN_2.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_2_B102           CE_CAN_2.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_2_H102           CE_CAN_2.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_2_W102           CE_CAN_2.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_2_CS103          CE_CAN_2.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_2_ID103          CE_CAN_2.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_2_B103           CE_CAN_2.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_2_H103           CE_CAN_2.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_2_W103           CE_CAN_2.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_2_CS104          CE_CAN_2.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_2_ID104          CE_CAN_2.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_2_B104           CE_CAN_2.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_2_H104           CE_CAN_2.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_2_W104           CE_CAN_2.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_2_CS105          CE_CAN_2.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_2_ID105          CE_CAN_2.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_2_B105           CE_CAN_2.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_2_H105           CE_CAN_2.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_2_W105           CE_CAN_2.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_2_CS106          CE_CAN_2.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_2_ID106          CE_CAN_2.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_2_B106           CE_CAN_2.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_2_H106           CE_CAN_2.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_2_W106           CE_CAN_2.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_2_CS107          CE_CAN_2.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_2_ID107          CE_CAN_2.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_2_B107           CE_CAN_2.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_2_H107           CE_CAN_2.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_2_W107           CE_CAN_2.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_2_CS108          CE_CAN_2.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_2_ID108          CE_CAN_2.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_2_B108           CE_CAN_2.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_2_H108           CE_CAN_2.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_2_W108           CE_CAN_2.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_2_CS109          CE_CAN_2.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_2_ID109          CE_CAN_2.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_2_B109           CE_CAN_2.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_2_H109           CE_CAN_2.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_2_W109           CE_CAN_2.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_2_CS110          CE_CAN_2.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_2_ID110          CE_CAN_2.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_2_B110           CE_CAN_2.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_2_H110           CE_CAN_2.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_2_W110           CE_CAN_2.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_2_CS111          CE_CAN_2.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_2_ID111          CE_CAN_2.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_2_B111           CE_CAN_2.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_2_H111           CE_CAN_2.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_2_W111           CE_CAN_2.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_2_CS112          CE_CAN_2.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_2_ID112          CE_CAN_2.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_2_B112           CE_CAN_2.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_2_H112           CE_CAN_2.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_2_W112           CE_CAN_2.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_2_CS113          CE_CAN_2.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_2_ID113          CE_CAN_2.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_2_B113           CE_CAN_2.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_2_H113           CE_CAN_2.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_2_W113           CE_CAN_2.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_2_CS114          CE_CAN_2.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_2_ID114          CE_CAN_2.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_2_B114           CE_CAN_2.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_2_H114           CE_CAN_2.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_2_W114           CE_CAN_2.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_2_CS115          CE_CAN_2.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_2_ID115          CE_CAN_2.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_2_B115           CE_CAN_2.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_2_H115           CE_CAN_2.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_2_W115           CE_CAN_2.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_2_CS116          CE_CAN_2.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_2_ID116          CE_CAN_2.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_2_B116           CE_CAN_2.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_2_H116           CE_CAN_2.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_2_W116           CE_CAN_2.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_2_CS117          CE_CAN_2.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_2_ID117          CE_CAN_2.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_2_B117           CE_CAN_2.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_2_H117           CE_CAN_2.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_2_W117           CE_CAN_2.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_2_CS118          CE_CAN_2.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_2_ID118          CE_CAN_2.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_2_B118           CE_CAN_2.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_2_H118           CE_CAN_2.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_2_W118           CE_CAN_2.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_2_CS119          CE_CAN_2.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_2_ID119          CE_CAN_2.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_2_B119           CE_CAN_2.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_2_H119           CE_CAN_2.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_2_W119           CE_CAN_2.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_2_CS120          CE_CAN_2.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_2_ID120          CE_CAN_2.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_2_B120           CE_CAN_2.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_2_H120           CE_CAN_2.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_2_W120           CE_CAN_2.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_2_CS121          CE_CAN_2.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_2_ID121          CE_CAN_2.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_2_B121           CE_CAN_2.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_2_H121           CE_CAN_2.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_2_W121           CE_CAN_2.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_2_CS122          CE_CAN_2.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_2_ID122          CE_CAN_2.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_2_B122           CE_CAN_2.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_2_H122           CE_CAN_2.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_2_W122           CE_CAN_2.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_2_CS123          CE_CAN_2.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_2_ID123          CE_CAN_2.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_2_B123           CE_CAN_2.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_2_H123           CE_CAN_2.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_2_W123           CE_CAN_2.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_2_CS124          CE_CAN_2.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_2_ID124          CE_CAN_2.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_2_B124           CE_CAN_2.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_2_H124           CE_CAN_2.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_2_W124           CE_CAN_2.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_2_CS125          CE_CAN_2.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_2_ID125          CE_CAN_2.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_2_B125           CE_CAN_2.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_2_H125           CE_CAN_2.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_2_W125           CE_CAN_2.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_2_CS126          CE_CAN_2.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_2_ID126          CE_CAN_2.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_2_B126           CE_CAN_2.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_2_H126           CE_CAN_2.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_2_W126           CE_CAN_2.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_2_CS127          CE_CAN_2.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_2_ID127          CE_CAN_2.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_2_B127           CE_CAN_2.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_2_H127           CE_CAN_2.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_2_W127           CE_CAN_2.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_2_RXIMR0      CE_CAN_2.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR1      CE_CAN_2.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR2      CE_CAN_2.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR3      CE_CAN_2.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR4      CE_CAN_2.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR5      CE_CAN_2.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR6      CE_CAN_2.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR7      CE_CAN_2.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR8      CE_CAN_2.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR9      CE_CAN_2.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_2_RXIMR10     CE_CAN_2.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR11     CE_CAN_2.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR12     CE_CAN_2.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR13     CE_CAN_2.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR14     CE_CAN_2.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR15     CE_CAN_2.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR16     CE_CAN_2.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR17     CE_CAN_2.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR18     CE_CAN_2.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR19     CE_CAN_2.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR20     CE_CAN_2.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR21     CE_CAN_2.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR22     CE_CAN_2.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR23     CE_CAN_2.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR24     CE_CAN_2.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR25     CE_CAN_2.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR26     CE_CAN_2.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR27     CE_CAN_2.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR28     CE_CAN_2.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR29     CE_CAN_2.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR30     CE_CAN_2.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR31     CE_CAN_2.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR32     CE_CAN_2.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR33     CE_CAN_2.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR34     CE_CAN_2.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR35     CE_CAN_2.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR36     CE_CAN_2.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR37     CE_CAN_2.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR38     CE_CAN_2.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR39     CE_CAN_2.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR40     CE_CAN_2.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR41     CE_CAN_2.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR42     CE_CAN_2.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR43     CE_CAN_2.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR44     CE_CAN_2.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR45     CE_CAN_2.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR46     CE_CAN_2.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR47     CE_CAN_2.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR48     CE_CAN_2.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR49     CE_CAN_2.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR50     CE_CAN_2.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR51     CE_CAN_2.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR52     CE_CAN_2.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR53     CE_CAN_2.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR54     CE_CAN_2.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR55     CE_CAN_2.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR56     CE_CAN_2.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR57     CE_CAN_2.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR58     CE_CAN_2.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR59     CE_CAN_2.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR60     CE_CAN_2.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR61     CE_CAN_2.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR62     CE_CAN_2.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR63     CE_CAN_2.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR64     CE_CAN_2.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR65     CE_CAN_2.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR66     CE_CAN_2.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR67     CE_CAN_2.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR68     CE_CAN_2.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR69     CE_CAN_2.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR70     CE_CAN_2.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR71     CE_CAN_2.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR72     CE_CAN_2.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR73     CE_CAN_2.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR74     CE_CAN_2.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR75     CE_CAN_2.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR76     CE_CAN_2.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR77     CE_CAN_2.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR78     CE_CAN_2.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR79     CE_CAN_2.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR80     CE_CAN_2.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR81     CE_CAN_2.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR82     CE_CAN_2.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR83     CE_CAN_2.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR84     CE_CAN_2.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR85     CE_CAN_2.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR86     CE_CAN_2.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR87     CE_CAN_2.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR88     CE_CAN_2.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR89     CE_CAN_2.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR90     CE_CAN_2.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR91     CE_CAN_2.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR92     CE_CAN_2.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR93     CE_CAN_2.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR94     CE_CAN_2.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR95     CE_CAN_2.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR96     CE_CAN_2.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR97     CE_CAN_2.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR98     CE_CAN_2.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR99     CE_CAN_2.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_2_RXIMR100    CE_CAN_2.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR101    CE_CAN_2.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR102    CE_CAN_2.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR103    CE_CAN_2.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR104    CE_CAN_2.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR105    CE_CAN_2.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR106    CE_CAN_2.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR107    CE_CAN_2.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR108    CE_CAN_2.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR109    CE_CAN_2.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR110    CE_CAN_2.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR111    CE_CAN_2.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR112    CE_CAN_2.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR113    CE_CAN_2.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR114    CE_CAN_2.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR115    CE_CAN_2.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR116    CE_CAN_2.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR117    CE_CAN_2.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR118    CE_CAN_2.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR119    CE_CAN_2.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR120    CE_CAN_2.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR121    CE_CAN_2.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR122    CE_CAN_2.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR123    CE_CAN_2.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR124    CE_CAN_2.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR125    CE_CAN_2.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR126    CE_CAN_2.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_2_RXIMR127    CE_CAN_2.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_2_MECR        CE_CAN_2.MECR.R               /* Memory Error Control */
#define CE_CAN_2_ERRIAR      CE_CAN_2.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_2_ERRIDPR     CE_CAN_2.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_2_ERRIPPR     CE_CAN_2.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_2_RERRAR      CE_CAN_2.RERRAR.R             /* Error Report Address */
#define CE_CAN_2_RERRDR      CE_CAN_2.RERRDR.R             /* Error Report Data */
#define CE_CAN_2_RERRSYNR    CE_CAN_2.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_2_ERRSR       CE_CAN_2.ERRSR.R              /* Error Status */
#define CE_CAN_2_EPRS        CE_CAN_2.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_2_ENCBT       CE_CAN_2.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_2_EDCBT       CE_CAN_2.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_2_ETDC        CE_CAN_2.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_2_FDCTRL      CE_CAN_2.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_2_FDCBT       CE_CAN_2.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_2_FDCRC       CE_CAN_2.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_2_ERFCR       CE_CAN_2.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_2_ERFIER      CE_CAN_2.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_2_ERFSR       CE_CAN_2.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_2_HR_TIME_STAMP0 CE_CAN_2.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP1 CE_CAN_2.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP2 CE_CAN_2.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP3 CE_CAN_2.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP4 CE_CAN_2.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP5 CE_CAN_2.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP6 CE_CAN_2.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP7 CE_CAN_2.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP8 CE_CAN_2.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP9 CE_CAN_2.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP10 CE_CAN_2.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP11 CE_CAN_2.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP12 CE_CAN_2.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP13 CE_CAN_2.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP14 CE_CAN_2.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP15 CE_CAN_2.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP16 CE_CAN_2.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP17 CE_CAN_2.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP18 CE_CAN_2.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP19 CE_CAN_2.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP20 CE_CAN_2.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP21 CE_CAN_2.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP22 CE_CAN_2.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP23 CE_CAN_2.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP24 CE_CAN_2.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP25 CE_CAN_2.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP26 CE_CAN_2.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP27 CE_CAN_2.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP28 CE_CAN_2.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP29 CE_CAN_2.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP30 CE_CAN_2.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP31 CE_CAN_2.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP32 CE_CAN_2.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP33 CE_CAN_2.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP34 CE_CAN_2.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP35 CE_CAN_2.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP36 CE_CAN_2.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP37 CE_CAN_2.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP38 CE_CAN_2.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP39 CE_CAN_2.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP40 CE_CAN_2.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP41 CE_CAN_2.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP42 CE_CAN_2.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP43 CE_CAN_2.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP44 CE_CAN_2.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP45 CE_CAN_2.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP46 CE_CAN_2.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP47 CE_CAN_2.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP48 CE_CAN_2.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP49 CE_CAN_2.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP50 CE_CAN_2.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP51 CE_CAN_2.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP52 CE_CAN_2.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP53 CE_CAN_2.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP54 CE_CAN_2.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP55 CE_CAN_2.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP56 CE_CAN_2.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP57 CE_CAN_2.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP58 CE_CAN_2.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP59 CE_CAN_2.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP60 CE_CAN_2.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP61 CE_CAN_2.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP62 CE_CAN_2.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP63 CE_CAN_2.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP64 CE_CAN_2.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP65 CE_CAN_2.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP66 CE_CAN_2.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP67 CE_CAN_2.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP68 CE_CAN_2.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP69 CE_CAN_2.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP70 CE_CAN_2.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP71 CE_CAN_2.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP72 CE_CAN_2.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP73 CE_CAN_2.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP74 CE_CAN_2.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP75 CE_CAN_2.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP76 CE_CAN_2.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP77 CE_CAN_2.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP78 CE_CAN_2.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP79 CE_CAN_2.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP80 CE_CAN_2.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP81 CE_CAN_2.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP82 CE_CAN_2.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP83 CE_CAN_2.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP84 CE_CAN_2.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP85 CE_CAN_2.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP86 CE_CAN_2.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP87 CE_CAN_2.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP88 CE_CAN_2.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP89 CE_CAN_2.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP90 CE_CAN_2.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP91 CE_CAN_2.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP92 CE_CAN_2.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP93 CE_CAN_2.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP94 CE_CAN_2.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP95 CE_CAN_2.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP96 CE_CAN_2.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP97 CE_CAN_2.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP98 CE_CAN_2.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP99 CE_CAN_2.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP100 CE_CAN_2.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP101 CE_CAN_2.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP102 CE_CAN_2.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP103 CE_CAN_2.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP104 CE_CAN_2.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP105 CE_CAN_2.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP106 CE_CAN_2.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP107 CE_CAN_2.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP108 CE_CAN_2.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP109 CE_CAN_2.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP110 CE_CAN_2.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP111 CE_CAN_2.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP112 CE_CAN_2.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP113 CE_CAN_2.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP114 CE_CAN_2.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP115 CE_CAN_2.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP116 CE_CAN_2.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP117 CE_CAN_2.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP118 CE_CAN_2.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP119 CE_CAN_2.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP120 CE_CAN_2.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP121 CE_CAN_2.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP122 CE_CAN_2.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP123 CE_CAN_2.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP124 CE_CAN_2.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP125 CE_CAN_2.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP126 CE_CAN_2.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_2_HR_TIME_STAMP127 CE_CAN_2.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_2_ERFFEL0     CE_CAN_2.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL1     CE_CAN_2.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL2     CE_CAN_2.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL3     CE_CAN_2.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL4     CE_CAN_2.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL5     CE_CAN_2.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL6     CE_CAN_2.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL7     CE_CAN_2.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL8     CE_CAN_2.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL9     CE_CAN_2.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL10    CE_CAN_2.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL11    CE_CAN_2.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL12    CE_CAN_2.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL13    CE_CAN_2.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL14    CE_CAN_2.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL15    CE_CAN_2.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL16    CE_CAN_2.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL17    CE_CAN_2.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL18    CE_CAN_2.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL19    CE_CAN_2.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL20    CE_CAN_2.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL21    CE_CAN_2.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL22    CE_CAN_2.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL23    CE_CAN_2.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL24    CE_CAN_2.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL25    CE_CAN_2.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL26    CE_CAN_2.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL27    CE_CAN_2.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL28    CE_CAN_2.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL29    CE_CAN_2.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL30    CE_CAN_2.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL31    CE_CAN_2.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL32    CE_CAN_2.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL33    CE_CAN_2.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL34    CE_CAN_2.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL35    CE_CAN_2.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL36    CE_CAN_2.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL37    CE_CAN_2.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL38    CE_CAN_2.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL39    CE_CAN_2.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL40    CE_CAN_2.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL41    CE_CAN_2.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL42    CE_CAN_2.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL43    CE_CAN_2.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL44    CE_CAN_2.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL45    CE_CAN_2.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL46    CE_CAN_2.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL47    CE_CAN_2.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL48    CE_CAN_2.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL49    CE_CAN_2.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL50    CE_CAN_2.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL51    CE_CAN_2.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL52    CE_CAN_2.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL53    CE_CAN_2.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL54    CE_CAN_2.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL55    CE_CAN_2.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL56    CE_CAN_2.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL57    CE_CAN_2.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL58    CE_CAN_2.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL59    CE_CAN_2.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL60    CE_CAN_2.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL61    CE_CAN_2.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL62    CE_CAN_2.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL63    CE_CAN_2.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL64    CE_CAN_2.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL65    CE_CAN_2.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL66    CE_CAN_2.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL67    CE_CAN_2.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL68    CE_CAN_2.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL69    CE_CAN_2.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL70    CE_CAN_2.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL71    CE_CAN_2.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL72    CE_CAN_2.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL73    CE_CAN_2.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL74    CE_CAN_2.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL75    CE_CAN_2.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL76    CE_CAN_2.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL77    CE_CAN_2.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL78    CE_CAN_2.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL79    CE_CAN_2.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL80    CE_CAN_2.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL81    CE_CAN_2.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL82    CE_CAN_2.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL83    CE_CAN_2.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL84    CE_CAN_2.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL85    CE_CAN_2.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL86    CE_CAN_2.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL87    CE_CAN_2.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL88    CE_CAN_2.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL89    CE_CAN_2.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL90    CE_CAN_2.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL91    CE_CAN_2.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL92    CE_CAN_2.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL93    CE_CAN_2.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL94    CE_CAN_2.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL95    CE_CAN_2.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL96    CE_CAN_2.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL97    CE_CAN_2.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL98    CE_CAN_2.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL99    CE_CAN_2.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL100   CE_CAN_2.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL101   CE_CAN_2.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL102   CE_CAN_2.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL103   CE_CAN_2.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL104   CE_CAN_2.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL105   CE_CAN_2.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL106   CE_CAN_2.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL107   CE_CAN_2.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL108   CE_CAN_2.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL109   CE_CAN_2.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL110   CE_CAN_2.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL111   CE_CAN_2.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL112   CE_CAN_2.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL113   CE_CAN_2.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL114   CE_CAN_2.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL115   CE_CAN_2.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL116   CE_CAN_2.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL117   CE_CAN_2.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL118   CE_CAN_2.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL119   CE_CAN_2.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL120   CE_CAN_2.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL121   CE_CAN_2.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL122   CE_CAN_2.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL123   CE_CAN_2.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL124   CE_CAN_2.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL125   CE_CAN_2.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL126   CE_CAN_2.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_2_ERFFEL127   CE_CAN_2.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_20_MCR        CE_CAN_20.MCR.R               /* Module Configuration */
#define CE_CAN_20_CTRL1      CE_CAN_20.CTRL1.R             /* Control 1 */
#define CE_CAN_20_TIMER      CE_CAN_20.TIMER.R             /* Free-Running Timer */
#define CE_CAN_20_RXMGMASK   CE_CAN_20.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_20_RX14MASK   CE_CAN_20.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_20_RX15MASK   CE_CAN_20.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_20_ECR        CE_CAN_20.ECR.R               /* Error Counter */
#define CE_CAN_20_ESR1       CE_CAN_20.ESR1.R              /* Error and Status 1 */
#define CE_CAN_20_IMASK2     CE_CAN_20.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_20_IMASK1     CE_CAN_20.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_20_IFLAG2     CE_CAN_20.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_20_IFLAG1     CE_CAN_20.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_20_CTRL2      CE_CAN_20.CTRL2.R             /* Control 2 */
#define CE_CAN_20_ESR2       CE_CAN_20.ESR2.R              /* Error and Status 2 */
#define CE_CAN_20_CRCR       CE_CAN_20.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_20_RXFGMASK   CE_CAN_20.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_20_RXFIR      CE_CAN_20.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_20_CBT        CE_CAN_20.CBT.R               /* CAN Bit Timing */
#define CE_CAN_20_IMASK4     CE_CAN_20.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_20_IMASK3     CE_CAN_20.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_20_IFLAG4     CE_CAN_20.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_20_IFLAG3     CE_CAN_20.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_20_CS0            CE_CAN_20.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_20_ID0            CE_CAN_20.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_20_B0             CE_CAN_20.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_20_H0             CE_CAN_20.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_20_W0             CE_CAN_20.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_20_CS1            CE_CAN_20.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_20_ID1            CE_CAN_20.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_20_B1             CE_CAN_20.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_20_H1             CE_CAN_20.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_20_W1             CE_CAN_20.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_20_CS2            CE_CAN_20.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_20_ID2            CE_CAN_20.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_20_B2             CE_CAN_20.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_20_H2             CE_CAN_20.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_20_W2             CE_CAN_20.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_20_CS3            CE_CAN_20.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_20_ID3            CE_CAN_20.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_20_B3             CE_CAN_20.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_20_H3             CE_CAN_20.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_20_W3             CE_CAN_20.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_20_CS4            CE_CAN_20.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_20_ID4            CE_CAN_20.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_20_B4             CE_CAN_20.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_20_H4             CE_CAN_20.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_20_W4             CE_CAN_20.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_20_CS5            CE_CAN_20.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_20_ID5            CE_CAN_20.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_20_B5             CE_CAN_20.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_20_H5             CE_CAN_20.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_20_W5             CE_CAN_20.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_20_CS6            CE_CAN_20.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_20_ID6            CE_CAN_20.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_20_B6             CE_CAN_20.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_20_H6             CE_CAN_20.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_20_W6             CE_CAN_20.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_20_CS7            CE_CAN_20.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_20_ID7            CE_CAN_20.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_20_B7             CE_CAN_20.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_20_H7             CE_CAN_20.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_20_W7             CE_CAN_20.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_20_CS8            CE_CAN_20.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_20_ID8            CE_CAN_20.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_20_B8             CE_CAN_20.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_20_H8             CE_CAN_20.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_20_W8             CE_CAN_20.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_20_CS9            CE_CAN_20.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_20_ID9            CE_CAN_20.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_20_B9             CE_CAN_20.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_20_H9             CE_CAN_20.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_20_W9             CE_CAN_20.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_20_CS10           CE_CAN_20.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_20_ID10           CE_CAN_20.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_20_B10            CE_CAN_20.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_20_H10            CE_CAN_20.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_20_W10            CE_CAN_20.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_20_CS11           CE_CAN_20.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_20_ID11           CE_CAN_20.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_20_B11            CE_CAN_20.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_20_H11            CE_CAN_20.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_20_W11            CE_CAN_20.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_20_CS12           CE_CAN_20.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_20_ID12           CE_CAN_20.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_20_B12            CE_CAN_20.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_20_H12            CE_CAN_20.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_20_W12            CE_CAN_20.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_20_CS13           CE_CAN_20.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_20_ID13           CE_CAN_20.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_20_B13            CE_CAN_20.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_20_H13            CE_CAN_20.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_20_W13            CE_CAN_20.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_20_CS14           CE_CAN_20.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_20_ID14           CE_CAN_20.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_20_B14            CE_CAN_20.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_20_H14            CE_CAN_20.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_20_W14            CE_CAN_20.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_20_CS15           CE_CAN_20.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_20_ID15           CE_CAN_20.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_20_B15            CE_CAN_20.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_20_H15            CE_CAN_20.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_20_W15            CE_CAN_20.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_20_CS16           CE_CAN_20.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_20_ID16           CE_CAN_20.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_20_B16            CE_CAN_20.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_20_H16            CE_CAN_20.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_20_W16            CE_CAN_20.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_20_CS17           CE_CAN_20.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_20_ID17           CE_CAN_20.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_20_B17            CE_CAN_20.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_20_H17            CE_CAN_20.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_20_W17            CE_CAN_20.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_20_CS18           CE_CAN_20.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_20_ID18           CE_CAN_20.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_20_B18            CE_CAN_20.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_20_H18            CE_CAN_20.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_20_W18            CE_CAN_20.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_20_CS19           CE_CAN_20.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_20_ID19           CE_CAN_20.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_20_B19            CE_CAN_20.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_20_H19            CE_CAN_20.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_20_W19            CE_CAN_20.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_20_CS20           CE_CAN_20.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_20_ID20           CE_CAN_20.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_20_B20            CE_CAN_20.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_20_H20            CE_CAN_20.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_20_W20            CE_CAN_20.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_20_CS21           CE_CAN_20.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_20_ID21           CE_CAN_20.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_20_B21            CE_CAN_20.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_20_H21            CE_CAN_20.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_20_W21            CE_CAN_20.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_20_CS22           CE_CAN_20.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_20_ID22           CE_CAN_20.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_20_B22            CE_CAN_20.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_20_H22            CE_CAN_20.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_20_W22            CE_CAN_20.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_20_CS23           CE_CAN_20.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_20_ID23           CE_CAN_20.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_20_B23            CE_CAN_20.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_20_H23            CE_CAN_20.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_20_W23            CE_CAN_20.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_20_CS24           CE_CAN_20.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_20_ID24           CE_CAN_20.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_20_B24            CE_CAN_20.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_20_H24            CE_CAN_20.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_20_W24            CE_CAN_20.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_20_CS25           CE_CAN_20.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_20_ID25           CE_CAN_20.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_20_B25            CE_CAN_20.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_20_H25            CE_CAN_20.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_20_W25            CE_CAN_20.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_20_CS26           CE_CAN_20.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_20_ID26           CE_CAN_20.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_20_B26            CE_CAN_20.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_20_H26            CE_CAN_20.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_20_W26            CE_CAN_20.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_20_CS27           CE_CAN_20.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_20_ID27           CE_CAN_20.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_20_B27            CE_CAN_20.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_20_H27            CE_CAN_20.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_20_W27            CE_CAN_20.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_20_CS28           CE_CAN_20.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_20_ID28           CE_CAN_20.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_20_B28            CE_CAN_20.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_20_H28            CE_CAN_20.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_20_W28            CE_CAN_20.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_20_CS29           CE_CAN_20.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_20_ID29           CE_CAN_20.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_20_B29            CE_CAN_20.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_20_H29            CE_CAN_20.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_20_W29            CE_CAN_20.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_20_CS30           CE_CAN_20.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_20_ID30           CE_CAN_20.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_20_B30            CE_CAN_20.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_20_H30            CE_CAN_20.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_20_W30            CE_CAN_20.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_20_CS31           CE_CAN_20.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_20_ID31           CE_CAN_20.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_20_B31            CE_CAN_20.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_20_H31            CE_CAN_20.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_20_W31            CE_CAN_20.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_20_CS32           CE_CAN_20.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_20_ID32           CE_CAN_20.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_20_B32            CE_CAN_20.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_20_H32            CE_CAN_20.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_20_W32            CE_CAN_20.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_20_CS33           CE_CAN_20.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_20_ID33           CE_CAN_20.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_20_B33            CE_CAN_20.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_20_H33            CE_CAN_20.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_20_W33            CE_CAN_20.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_20_CS34           CE_CAN_20.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_20_ID34           CE_CAN_20.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_20_B34            CE_CAN_20.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_20_H34            CE_CAN_20.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_20_W34            CE_CAN_20.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_20_CS35           CE_CAN_20.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_20_ID35           CE_CAN_20.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_20_B35            CE_CAN_20.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_20_H35            CE_CAN_20.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_20_W35            CE_CAN_20.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_20_CS36           CE_CAN_20.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_20_ID36           CE_CAN_20.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_20_B36            CE_CAN_20.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_20_H36            CE_CAN_20.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_20_W36            CE_CAN_20.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_20_CS37           CE_CAN_20.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_20_ID37           CE_CAN_20.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_20_B37            CE_CAN_20.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_20_H37            CE_CAN_20.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_20_W37            CE_CAN_20.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_20_CS38           CE_CAN_20.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_20_ID38           CE_CAN_20.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_20_B38            CE_CAN_20.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_20_H38            CE_CAN_20.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_20_W38            CE_CAN_20.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_20_CS39           CE_CAN_20.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_20_ID39           CE_CAN_20.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_20_B39            CE_CAN_20.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_20_H39            CE_CAN_20.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_20_W39            CE_CAN_20.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_20_CS40           CE_CAN_20.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_20_ID40           CE_CAN_20.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_20_B40            CE_CAN_20.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_20_H40            CE_CAN_20.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_20_W40            CE_CAN_20.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_20_CS41           CE_CAN_20.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_20_ID41           CE_CAN_20.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_20_B41            CE_CAN_20.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_20_H41            CE_CAN_20.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_20_W41            CE_CAN_20.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_20_CS42           CE_CAN_20.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_20_ID42           CE_CAN_20.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_20_B42            CE_CAN_20.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_20_H42            CE_CAN_20.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_20_W42            CE_CAN_20.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_20_CS43           CE_CAN_20.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_20_ID43           CE_CAN_20.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_20_B43            CE_CAN_20.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_20_H43            CE_CAN_20.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_20_W43            CE_CAN_20.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_20_CS44           CE_CAN_20.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_20_ID44           CE_CAN_20.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_20_B44            CE_CAN_20.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_20_H44            CE_CAN_20.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_20_W44            CE_CAN_20.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_20_CS45           CE_CAN_20.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_20_ID45           CE_CAN_20.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_20_B45            CE_CAN_20.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_20_H45            CE_CAN_20.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_20_W45            CE_CAN_20.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_20_CS46           CE_CAN_20.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_20_ID46           CE_CAN_20.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_20_B46            CE_CAN_20.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_20_H46            CE_CAN_20.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_20_W46            CE_CAN_20.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_20_CS47           CE_CAN_20.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_20_ID47           CE_CAN_20.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_20_B47            CE_CAN_20.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_20_H47            CE_CAN_20.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_20_W47            CE_CAN_20.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_20_CS48           CE_CAN_20.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_20_ID48           CE_CAN_20.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_20_B48            CE_CAN_20.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_20_H48            CE_CAN_20.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_20_W48            CE_CAN_20.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_20_CS49           CE_CAN_20.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_20_ID49           CE_CAN_20.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_20_B49            CE_CAN_20.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_20_H49            CE_CAN_20.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_20_W49            CE_CAN_20.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_20_CS50           CE_CAN_20.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_20_ID50           CE_CAN_20.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_20_B50            CE_CAN_20.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_20_H50            CE_CAN_20.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_20_W50            CE_CAN_20.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_20_CS51           CE_CAN_20.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_20_ID51           CE_CAN_20.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_20_B51            CE_CAN_20.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_20_H51            CE_CAN_20.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_20_W51            CE_CAN_20.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_20_CS52           CE_CAN_20.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_20_ID52           CE_CAN_20.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_20_B52            CE_CAN_20.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_20_H52            CE_CAN_20.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_20_W52            CE_CAN_20.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_20_CS53           CE_CAN_20.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_20_ID53           CE_CAN_20.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_20_B53            CE_CAN_20.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_20_H53            CE_CAN_20.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_20_W53            CE_CAN_20.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_20_CS54           CE_CAN_20.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_20_ID54           CE_CAN_20.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_20_B54            CE_CAN_20.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_20_H54            CE_CAN_20.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_20_W54            CE_CAN_20.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_20_CS55           CE_CAN_20.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_20_ID55           CE_CAN_20.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_20_B55            CE_CAN_20.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_20_H55            CE_CAN_20.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_20_W55            CE_CAN_20.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_20_CS56           CE_CAN_20.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_20_ID56           CE_CAN_20.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_20_B56            CE_CAN_20.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_20_H56            CE_CAN_20.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_20_W56            CE_CAN_20.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_20_CS57           CE_CAN_20.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_20_ID57           CE_CAN_20.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_20_B57            CE_CAN_20.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_20_H57            CE_CAN_20.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_20_W57            CE_CAN_20.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_20_CS58           CE_CAN_20.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_20_ID58           CE_CAN_20.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_20_B58            CE_CAN_20.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_20_H58            CE_CAN_20.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_20_W58            CE_CAN_20.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_20_CS59           CE_CAN_20.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_20_ID59           CE_CAN_20.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_20_B59            CE_CAN_20.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_20_H59            CE_CAN_20.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_20_W59            CE_CAN_20.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_20_CS60           CE_CAN_20.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_20_ID60           CE_CAN_20.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_20_B60            CE_CAN_20.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_20_H60            CE_CAN_20.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_20_W60            CE_CAN_20.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_20_CS61           CE_CAN_20.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_20_ID61           CE_CAN_20.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_20_B61            CE_CAN_20.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_20_H61            CE_CAN_20.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_20_W61            CE_CAN_20.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_20_CS62           CE_CAN_20.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_20_ID62           CE_CAN_20.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_20_B62            CE_CAN_20.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_20_H62            CE_CAN_20.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_20_W62            CE_CAN_20.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_20_CS63           CE_CAN_20.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_20_ID63           CE_CAN_20.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_20_B63            CE_CAN_20.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_20_H63            CE_CAN_20.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_20_W63            CE_CAN_20.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_20_CS64           CE_CAN_20.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_20_ID64           CE_CAN_20.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_20_B64            CE_CAN_20.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_20_H64            CE_CAN_20.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_20_W64            CE_CAN_20.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_20_CS65           CE_CAN_20.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_20_ID65           CE_CAN_20.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_20_B65            CE_CAN_20.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_20_H65            CE_CAN_20.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_20_W65            CE_CAN_20.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_20_CS66           CE_CAN_20.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_20_ID66           CE_CAN_20.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_20_B66            CE_CAN_20.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_20_H66            CE_CAN_20.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_20_W66            CE_CAN_20.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_20_CS67           CE_CAN_20.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_20_ID67           CE_CAN_20.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_20_B67            CE_CAN_20.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_20_H67            CE_CAN_20.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_20_W67            CE_CAN_20.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_20_CS68           CE_CAN_20.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_20_ID68           CE_CAN_20.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_20_B68            CE_CAN_20.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_20_H68            CE_CAN_20.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_20_W68            CE_CAN_20.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_20_CS69           CE_CAN_20.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_20_ID69           CE_CAN_20.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_20_B69            CE_CAN_20.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_20_H69            CE_CAN_20.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_20_W69            CE_CAN_20.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_20_CS70           CE_CAN_20.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_20_ID70           CE_CAN_20.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_20_B70            CE_CAN_20.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_20_H70            CE_CAN_20.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_20_W70            CE_CAN_20.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_20_CS71           CE_CAN_20.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_20_ID71           CE_CAN_20.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_20_B71            CE_CAN_20.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_20_H71            CE_CAN_20.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_20_W71            CE_CAN_20.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_20_CS72           CE_CAN_20.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_20_ID72           CE_CAN_20.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_20_B72            CE_CAN_20.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_20_H72            CE_CAN_20.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_20_W72            CE_CAN_20.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_20_CS73           CE_CAN_20.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_20_ID73           CE_CAN_20.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_20_B73            CE_CAN_20.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_20_H73            CE_CAN_20.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_20_W73            CE_CAN_20.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_20_CS74           CE_CAN_20.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_20_ID74           CE_CAN_20.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_20_B74            CE_CAN_20.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_20_H74            CE_CAN_20.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_20_W74            CE_CAN_20.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_20_CS75           CE_CAN_20.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_20_ID75           CE_CAN_20.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_20_B75            CE_CAN_20.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_20_H75            CE_CAN_20.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_20_W75            CE_CAN_20.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_20_CS76           CE_CAN_20.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_20_ID76           CE_CAN_20.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_20_B76            CE_CAN_20.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_20_H76            CE_CAN_20.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_20_W76            CE_CAN_20.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_20_CS77           CE_CAN_20.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_20_ID77           CE_CAN_20.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_20_B77            CE_CAN_20.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_20_H77            CE_CAN_20.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_20_W77            CE_CAN_20.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_20_CS78           CE_CAN_20.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_20_ID78           CE_CAN_20.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_20_B78            CE_CAN_20.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_20_H78            CE_CAN_20.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_20_W78            CE_CAN_20.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_20_CS79           CE_CAN_20.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_20_ID79           CE_CAN_20.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_20_B79            CE_CAN_20.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_20_H79            CE_CAN_20.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_20_W79            CE_CAN_20.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_20_CS80           CE_CAN_20.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_20_ID80           CE_CAN_20.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_20_B80            CE_CAN_20.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_20_H80            CE_CAN_20.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_20_W80            CE_CAN_20.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_20_CS81           CE_CAN_20.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_20_ID81           CE_CAN_20.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_20_B81            CE_CAN_20.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_20_H81            CE_CAN_20.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_20_W81            CE_CAN_20.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_20_CS82           CE_CAN_20.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_20_ID82           CE_CAN_20.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_20_B82            CE_CAN_20.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_20_H82            CE_CAN_20.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_20_W82            CE_CAN_20.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_20_CS83           CE_CAN_20.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_20_ID83           CE_CAN_20.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_20_B83            CE_CAN_20.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_20_H83            CE_CAN_20.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_20_W83            CE_CAN_20.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_20_CS84           CE_CAN_20.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_20_ID84           CE_CAN_20.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_20_B84            CE_CAN_20.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_20_H84            CE_CAN_20.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_20_W84            CE_CAN_20.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_20_CS85           CE_CAN_20.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_20_ID85           CE_CAN_20.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_20_B85            CE_CAN_20.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_20_H85            CE_CAN_20.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_20_W85            CE_CAN_20.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_20_CS86           CE_CAN_20.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_20_ID86           CE_CAN_20.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_20_B86            CE_CAN_20.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_20_H86            CE_CAN_20.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_20_W86            CE_CAN_20.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_20_CS87           CE_CAN_20.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_20_ID87           CE_CAN_20.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_20_B87            CE_CAN_20.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_20_H87            CE_CAN_20.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_20_W87            CE_CAN_20.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_20_CS88           CE_CAN_20.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_20_ID88           CE_CAN_20.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_20_B88            CE_CAN_20.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_20_H88            CE_CAN_20.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_20_W848           CE_CAN_20.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_20_CS89           CE_CAN_20.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_20_ID89           CE_CAN_20.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_20_B89            CE_CAN_20.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_20_H89            CE_CAN_20.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_20_W89            CE_CAN_20.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_20_CS90           CE_CAN_20.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_20_ID90           CE_CAN_20.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_20_B90            CE_CAN_20.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_20_H90            CE_CAN_20.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_20_W90            CE_CAN_20.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_20_CS91           CE_CAN_20.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_20_ID91           CE_CAN_20.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_20_B91            CE_CAN_20.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_20_H91            CE_CAN_20.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_20_W91            CE_CAN_20.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_20_CS92           CE_CAN_20.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_20_ID92           CE_CAN_20.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_20_B92            CE_CAN_20.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_20_H92            CE_CAN_20.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_20_W92            CE_CAN_20.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_20_CS93           CE_CAN_20.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_20_ID93           CE_CAN_20.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_20_B93            CE_CAN_20.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_20_H93            CE_CAN_20.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_20_W93            CE_CAN_20.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_20_CS94           CE_CAN_20.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_20_ID94           CE_CAN_20.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_20_B94            CE_CAN_20.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_20_H94            CE_CAN_20.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_20_W94            CE_CAN_20.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_20_CS95           CE_CAN_20.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_20_ID95           CE_CAN_20.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_20_B95            CE_CAN_20.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_20_H95            CE_CAN_20.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_20_W95            CE_CAN_20.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_20_CS96           CE_CAN_20.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_20_ID96           CE_CAN_20.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_20_B96            CE_CAN_20.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_20_H96            CE_CAN_20.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_20_W96            CE_CAN_20.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_20_CS97           CE_CAN_20.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_20_ID97           CE_CAN_20.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_20_B97            CE_CAN_20.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_20_H97            CE_CAN_20.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_20_W97            CE_CAN_20.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_20_CS98           CE_CAN_20.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_20_ID98           CE_CAN_20.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_20_B98            CE_CAN_20.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_20_H98            CE_CAN_20.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_20_W98            CE_CAN_20.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_20_CS99           CE_CAN_20.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_20_ID99           CE_CAN_20.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_20_B99            CE_CAN_20.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_20_H99            CE_CAN_20.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_20_W99            CE_CAN_20.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_20_CS100          CE_CAN_20.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_20_ID100          CE_CAN_20.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_20_B100           CE_CAN_20.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_20_H100           CE_CAN_20.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_20_W100           CE_CAN_20.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_20_CS101          CE_CAN_20.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_20_ID101          CE_CAN_20.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_20_B101           CE_CAN_20.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_20_H101           CE_CAN_20.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_20_W101           CE_CAN_20.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_20_CS102          CE_CAN_20.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_20_ID102          CE_CAN_20.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_20_B102           CE_CAN_20.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_20_H102           CE_CAN_20.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_20_W102           CE_CAN_20.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_20_CS103          CE_CAN_20.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_20_ID103          CE_CAN_20.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_20_B103           CE_CAN_20.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_20_H103           CE_CAN_20.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_20_W103           CE_CAN_20.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_20_CS104          CE_CAN_20.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_20_ID104          CE_CAN_20.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_20_B104           CE_CAN_20.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_20_H104           CE_CAN_20.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_20_W104           CE_CAN_20.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_20_CS105          CE_CAN_20.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_20_ID105          CE_CAN_20.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_20_B105           CE_CAN_20.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_20_H105           CE_CAN_20.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_20_W105           CE_CAN_20.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_20_CS106          CE_CAN_20.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_20_ID106          CE_CAN_20.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_20_B106           CE_CAN_20.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_20_H106           CE_CAN_20.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_20_W106           CE_CAN_20.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_20_CS107          CE_CAN_20.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_20_ID107          CE_CAN_20.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_20_B107           CE_CAN_20.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_20_H107           CE_CAN_20.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_20_W107           CE_CAN_20.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_20_CS108          CE_CAN_20.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_20_ID108          CE_CAN_20.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_20_B108           CE_CAN_20.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_20_H108           CE_CAN_20.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_20_W108           CE_CAN_20.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_20_CS109          CE_CAN_20.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_20_ID109          CE_CAN_20.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_20_B109           CE_CAN_20.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_20_H109           CE_CAN_20.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_20_W109           CE_CAN_20.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_20_CS110          CE_CAN_20.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_20_ID110          CE_CAN_20.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_20_B110           CE_CAN_20.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_20_H110           CE_CAN_20.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_20_W110           CE_CAN_20.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_20_CS111          CE_CAN_20.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_20_ID111          CE_CAN_20.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_20_B111           CE_CAN_20.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_20_H111           CE_CAN_20.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_20_W111           CE_CAN_20.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_20_CS112          CE_CAN_20.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_20_ID112          CE_CAN_20.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_20_B112           CE_CAN_20.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_20_H112           CE_CAN_20.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_20_W112           CE_CAN_20.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_20_CS113          CE_CAN_20.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_20_ID113          CE_CAN_20.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_20_B113           CE_CAN_20.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_20_H113           CE_CAN_20.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_20_W113           CE_CAN_20.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_20_CS114          CE_CAN_20.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_20_ID114          CE_CAN_20.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_20_B114           CE_CAN_20.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_20_H114           CE_CAN_20.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_20_W114           CE_CAN_20.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_20_CS115          CE_CAN_20.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_20_ID115          CE_CAN_20.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_20_B115           CE_CAN_20.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_20_H115           CE_CAN_20.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_20_W115           CE_CAN_20.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_20_CS116          CE_CAN_20.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_20_ID116          CE_CAN_20.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_20_B116           CE_CAN_20.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_20_H116           CE_CAN_20.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_20_W116           CE_CAN_20.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_20_CS117          CE_CAN_20.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_20_ID117          CE_CAN_20.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_20_B117           CE_CAN_20.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_20_H117           CE_CAN_20.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_20_W117           CE_CAN_20.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_20_CS118          CE_CAN_20.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_20_ID118          CE_CAN_20.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_20_B118           CE_CAN_20.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_20_H118           CE_CAN_20.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_20_W118           CE_CAN_20.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_20_CS119          CE_CAN_20.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_20_ID119          CE_CAN_20.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_20_B119           CE_CAN_20.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_20_H119           CE_CAN_20.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_20_W119           CE_CAN_20.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_20_CS120          CE_CAN_20.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_20_ID120          CE_CAN_20.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_20_B120           CE_CAN_20.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_20_H120           CE_CAN_20.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_20_W120           CE_CAN_20.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_20_CS121          CE_CAN_20.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_20_ID121          CE_CAN_20.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_20_B121           CE_CAN_20.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_20_H121           CE_CAN_20.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_20_W121           CE_CAN_20.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_20_CS122          CE_CAN_20.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_20_ID122          CE_CAN_20.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_20_B122           CE_CAN_20.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_20_H122           CE_CAN_20.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_20_W122           CE_CAN_20.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_20_CS123          CE_CAN_20.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_20_ID123          CE_CAN_20.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_20_B123           CE_CAN_20.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_20_H123           CE_CAN_20.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_20_W123           CE_CAN_20.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_20_CS124          CE_CAN_20.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_20_ID124          CE_CAN_20.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_20_B124           CE_CAN_20.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_20_H124           CE_CAN_20.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_20_W124           CE_CAN_20.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_20_CS125          CE_CAN_20.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_20_ID125          CE_CAN_20.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_20_B125           CE_CAN_20.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_20_H125           CE_CAN_20.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_20_W125           CE_CAN_20.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_20_CS126          CE_CAN_20.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_20_ID126          CE_CAN_20.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_20_B126           CE_CAN_20.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_20_H126           CE_CAN_20.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_20_W126           CE_CAN_20.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_20_CS127          CE_CAN_20.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_20_ID127          CE_CAN_20.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_20_B127           CE_CAN_20.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_20_H127           CE_CAN_20.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_20_W127           CE_CAN_20.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_20_RXIMR0     CE_CAN_20.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR1     CE_CAN_20.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR2     CE_CAN_20.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR3     CE_CAN_20.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR4     CE_CAN_20.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR5     CE_CAN_20.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR6     CE_CAN_20.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR7     CE_CAN_20.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR8     CE_CAN_20.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR9     CE_CAN_20.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_20_RXIMR10    CE_CAN_20.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR11    CE_CAN_20.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR12    CE_CAN_20.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR13    CE_CAN_20.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR14    CE_CAN_20.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR15    CE_CAN_20.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR16    CE_CAN_20.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR17    CE_CAN_20.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR18    CE_CAN_20.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR19    CE_CAN_20.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR20    CE_CAN_20.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR21    CE_CAN_20.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR22    CE_CAN_20.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR23    CE_CAN_20.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR24    CE_CAN_20.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR25    CE_CAN_20.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR26    CE_CAN_20.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR27    CE_CAN_20.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR28    CE_CAN_20.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR29    CE_CAN_20.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR30    CE_CAN_20.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR31    CE_CAN_20.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR32    CE_CAN_20.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR33    CE_CAN_20.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR34    CE_CAN_20.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR35    CE_CAN_20.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR36    CE_CAN_20.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR37    CE_CAN_20.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR38    CE_CAN_20.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR39    CE_CAN_20.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR40    CE_CAN_20.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR41    CE_CAN_20.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR42    CE_CAN_20.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR43    CE_CAN_20.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR44    CE_CAN_20.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR45    CE_CAN_20.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR46    CE_CAN_20.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR47    CE_CAN_20.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR48    CE_CAN_20.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR49    CE_CAN_20.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR50    CE_CAN_20.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR51    CE_CAN_20.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR52    CE_CAN_20.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR53    CE_CAN_20.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR54    CE_CAN_20.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR55    CE_CAN_20.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR56    CE_CAN_20.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR57    CE_CAN_20.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR58    CE_CAN_20.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR59    CE_CAN_20.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR60    CE_CAN_20.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR61    CE_CAN_20.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR62    CE_CAN_20.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR63    CE_CAN_20.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR64    CE_CAN_20.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR65    CE_CAN_20.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR66    CE_CAN_20.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR67    CE_CAN_20.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR68    CE_CAN_20.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR69    CE_CAN_20.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR70    CE_CAN_20.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR71    CE_CAN_20.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR72    CE_CAN_20.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR73    CE_CAN_20.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR74    CE_CAN_20.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR75    CE_CAN_20.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR76    CE_CAN_20.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR77    CE_CAN_20.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR78    CE_CAN_20.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR79    CE_CAN_20.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR80    CE_CAN_20.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR81    CE_CAN_20.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR82    CE_CAN_20.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR83    CE_CAN_20.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR84    CE_CAN_20.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR85    CE_CAN_20.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR86    CE_CAN_20.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR87    CE_CAN_20.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR88    CE_CAN_20.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR89    CE_CAN_20.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR90    CE_CAN_20.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR91    CE_CAN_20.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR92    CE_CAN_20.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR93    CE_CAN_20.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR94    CE_CAN_20.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR95    CE_CAN_20.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR96    CE_CAN_20.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR97    CE_CAN_20.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR98    CE_CAN_20.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR99    CE_CAN_20.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_20_RXIMR100   CE_CAN_20.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR101   CE_CAN_20.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR102   CE_CAN_20.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR103   CE_CAN_20.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR104   CE_CAN_20.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR105   CE_CAN_20.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR106   CE_CAN_20.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR107   CE_CAN_20.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR108   CE_CAN_20.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR109   CE_CAN_20.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR110   CE_CAN_20.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR111   CE_CAN_20.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR112   CE_CAN_20.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR113   CE_CAN_20.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR114   CE_CAN_20.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR115   CE_CAN_20.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR116   CE_CAN_20.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR117   CE_CAN_20.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR118   CE_CAN_20.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR119   CE_CAN_20.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR120   CE_CAN_20.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR121   CE_CAN_20.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR122   CE_CAN_20.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR123   CE_CAN_20.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR124   CE_CAN_20.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR125   CE_CAN_20.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR126   CE_CAN_20.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_20_RXIMR127   CE_CAN_20.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_20_MECR       CE_CAN_20.MECR.R              /* Memory Error Control */
#define CE_CAN_20_ERRIAR     CE_CAN_20.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_20_ERRIDPR    CE_CAN_20.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_20_ERRIPPR    CE_CAN_20.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_20_RERRAR     CE_CAN_20.RERRAR.R            /* Error Report Address */
#define CE_CAN_20_RERRDR     CE_CAN_20.RERRDR.R            /* Error Report Data */
#define CE_CAN_20_RERRSYNR   CE_CAN_20.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_20_ERRSR      CE_CAN_20.ERRSR.R             /* Error Status */
#define CE_CAN_20_EPRS       CE_CAN_20.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_20_ENCBT      CE_CAN_20.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_20_EDCBT      CE_CAN_20.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_20_ETDC       CE_CAN_20.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_20_FDCTRL     CE_CAN_20.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_20_FDCBT      CE_CAN_20.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_20_FDCRC      CE_CAN_20.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_20_ERFCR      CE_CAN_20.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_20_ERFIER     CE_CAN_20.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_20_ERFSR      CE_CAN_20.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_20_HR_TIME_STAMP0 CE_CAN_20.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP1 CE_CAN_20.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP2 CE_CAN_20.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP3 CE_CAN_20.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP4 CE_CAN_20.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP5 CE_CAN_20.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP6 CE_CAN_20.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP7 CE_CAN_20.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP8 CE_CAN_20.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP9 CE_CAN_20.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP10 CE_CAN_20.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP11 CE_CAN_20.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP12 CE_CAN_20.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP13 CE_CAN_20.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP14 CE_CAN_20.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP15 CE_CAN_20.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP16 CE_CAN_20.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP17 CE_CAN_20.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP18 CE_CAN_20.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP19 CE_CAN_20.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP20 CE_CAN_20.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP21 CE_CAN_20.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP22 CE_CAN_20.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP23 CE_CAN_20.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP24 CE_CAN_20.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP25 CE_CAN_20.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP26 CE_CAN_20.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP27 CE_CAN_20.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP28 CE_CAN_20.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP29 CE_CAN_20.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP30 CE_CAN_20.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP31 CE_CAN_20.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP32 CE_CAN_20.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP33 CE_CAN_20.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP34 CE_CAN_20.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP35 CE_CAN_20.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP36 CE_CAN_20.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP37 CE_CAN_20.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP38 CE_CAN_20.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP39 CE_CAN_20.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP40 CE_CAN_20.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP41 CE_CAN_20.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP42 CE_CAN_20.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP43 CE_CAN_20.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP44 CE_CAN_20.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP45 CE_CAN_20.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP46 CE_CAN_20.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP47 CE_CAN_20.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP48 CE_CAN_20.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP49 CE_CAN_20.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP50 CE_CAN_20.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP51 CE_CAN_20.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP52 CE_CAN_20.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP53 CE_CAN_20.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP54 CE_CAN_20.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP55 CE_CAN_20.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP56 CE_CAN_20.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP57 CE_CAN_20.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP58 CE_CAN_20.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP59 CE_CAN_20.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP60 CE_CAN_20.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP61 CE_CAN_20.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP62 CE_CAN_20.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP63 CE_CAN_20.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP64 CE_CAN_20.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP65 CE_CAN_20.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP66 CE_CAN_20.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP67 CE_CAN_20.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP68 CE_CAN_20.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP69 CE_CAN_20.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP70 CE_CAN_20.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP71 CE_CAN_20.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP72 CE_CAN_20.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP73 CE_CAN_20.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP74 CE_CAN_20.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP75 CE_CAN_20.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP76 CE_CAN_20.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP77 CE_CAN_20.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP78 CE_CAN_20.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP79 CE_CAN_20.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP80 CE_CAN_20.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP81 CE_CAN_20.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP82 CE_CAN_20.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP83 CE_CAN_20.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP84 CE_CAN_20.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP85 CE_CAN_20.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP86 CE_CAN_20.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP87 CE_CAN_20.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP88 CE_CAN_20.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP89 CE_CAN_20.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP90 CE_CAN_20.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP91 CE_CAN_20.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP92 CE_CAN_20.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP93 CE_CAN_20.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP94 CE_CAN_20.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP95 CE_CAN_20.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP96 CE_CAN_20.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP97 CE_CAN_20.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP98 CE_CAN_20.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP99 CE_CAN_20.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP100 CE_CAN_20.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP101 CE_CAN_20.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP102 CE_CAN_20.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP103 CE_CAN_20.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP104 CE_CAN_20.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP105 CE_CAN_20.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP106 CE_CAN_20.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP107 CE_CAN_20.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP108 CE_CAN_20.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP109 CE_CAN_20.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP110 CE_CAN_20.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP111 CE_CAN_20.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP112 CE_CAN_20.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP113 CE_CAN_20.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP114 CE_CAN_20.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP115 CE_CAN_20.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP116 CE_CAN_20.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP117 CE_CAN_20.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP118 CE_CAN_20.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP119 CE_CAN_20.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP120 CE_CAN_20.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP121 CE_CAN_20.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP122 CE_CAN_20.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP123 CE_CAN_20.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP124 CE_CAN_20.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP125 CE_CAN_20.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP126 CE_CAN_20.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_20_HR_TIME_STAMP127 CE_CAN_20.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_20_ERFFEL0    CE_CAN_20.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL1    CE_CAN_20.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL2    CE_CAN_20.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL3    CE_CAN_20.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL4    CE_CAN_20.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL5    CE_CAN_20.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL6    CE_CAN_20.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL7    CE_CAN_20.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL8    CE_CAN_20.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL9    CE_CAN_20.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL10   CE_CAN_20.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL11   CE_CAN_20.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL12   CE_CAN_20.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL13   CE_CAN_20.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL14   CE_CAN_20.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL15   CE_CAN_20.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL16   CE_CAN_20.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL17   CE_CAN_20.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL18   CE_CAN_20.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL19   CE_CAN_20.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL20   CE_CAN_20.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL21   CE_CAN_20.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL22   CE_CAN_20.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL23   CE_CAN_20.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL24   CE_CAN_20.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL25   CE_CAN_20.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL26   CE_CAN_20.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL27   CE_CAN_20.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL28   CE_CAN_20.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL29   CE_CAN_20.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL30   CE_CAN_20.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL31   CE_CAN_20.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL32   CE_CAN_20.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL33   CE_CAN_20.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL34   CE_CAN_20.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL35   CE_CAN_20.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL36   CE_CAN_20.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL37   CE_CAN_20.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL38   CE_CAN_20.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL39   CE_CAN_20.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL40   CE_CAN_20.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL41   CE_CAN_20.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL42   CE_CAN_20.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL43   CE_CAN_20.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL44   CE_CAN_20.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL45   CE_CAN_20.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL46   CE_CAN_20.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL47   CE_CAN_20.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL48   CE_CAN_20.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL49   CE_CAN_20.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL50   CE_CAN_20.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL51   CE_CAN_20.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL52   CE_CAN_20.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL53   CE_CAN_20.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL54   CE_CAN_20.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL55   CE_CAN_20.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL56   CE_CAN_20.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL57   CE_CAN_20.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL58   CE_CAN_20.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL59   CE_CAN_20.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL60   CE_CAN_20.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL61   CE_CAN_20.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL62   CE_CAN_20.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL63   CE_CAN_20.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL64   CE_CAN_20.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL65   CE_CAN_20.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL66   CE_CAN_20.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL67   CE_CAN_20.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL68   CE_CAN_20.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL69   CE_CAN_20.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL70   CE_CAN_20.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL71   CE_CAN_20.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL72   CE_CAN_20.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL73   CE_CAN_20.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL74   CE_CAN_20.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL75   CE_CAN_20.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL76   CE_CAN_20.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL77   CE_CAN_20.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL78   CE_CAN_20.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL79   CE_CAN_20.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL80   CE_CAN_20.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL81   CE_CAN_20.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL82   CE_CAN_20.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL83   CE_CAN_20.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL84   CE_CAN_20.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL85   CE_CAN_20.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL86   CE_CAN_20.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL87   CE_CAN_20.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL88   CE_CAN_20.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL89   CE_CAN_20.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL90   CE_CAN_20.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL91   CE_CAN_20.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL92   CE_CAN_20.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL93   CE_CAN_20.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL94   CE_CAN_20.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL95   CE_CAN_20.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL96   CE_CAN_20.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL97   CE_CAN_20.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL98   CE_CAN_20.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL99   CE_CAN_20.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL100  CE_CAN_20.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL101  CE_CAN_20.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL102  CE_CAN_20.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL103  CE_CAN_20.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL104  CE_CAN_20.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL105  CE_CAN_20.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL106  CE_CAN_20.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL107  CE_CAN_20.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL108  CE_CAN_20.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL109  CE_CAN_20.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL110  CE_CAN_20.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL111  CE_CAN_20.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL112  CE_CAN_20.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL113  CE_CAN_20.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL114  CE_CAN_20.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL115  CE_CAN_20.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL116  CE_CAN_20.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL117  CE_CAN_20.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL118  CE_CAN_20.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL119  CE_CAN_20.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL120  CE_CAN_20.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL121  CE_CAN_20.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL122  CE_CAN_20.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL123  CE_CAN_20.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL124  CE_CAN_20.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL125  CE_CAN_20.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL126  CE_CAN_20.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_20_ERFFEL127  CE_CAN_20.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_21_MCR        CE_CAN_21.MCR.R               /* Module Configuration */
#define CE_CAN_21_CTRL1      CE_CAN_21.CTRL1.R             /* Control 1 */
#define CE_CAN_21_TIMER      CE_CAN_21.TIMER.R             /* Free-Running Timer */
#define CE_CAN_21_RXMGMASK   CE_CAN_21.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_21_RX14MASK   CE_CAN_21.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_21_RX15MASK   CE_CAN_21.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_21_ECR        CE_CAN_21.ECR.R               /* Error Counter */
#define CE_CAN_21_ESR1       CE_CAN_21.ESR1.R              /* Error and Status 1 */
#define CE_CAN_21_IMASK2     CE_CAN_21.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_21_IMASK1     CE_CAN_21.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_21_IFLAG2     CE_CAN_21.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_21_IFLAG1     CE_CAN_21.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_21_CTRL2      CE_CAN_21.CTRL2.R             /* Control 2 */
#define CE_CAN_21_ESR2       CE_CAN_21.ESR2.R              /* Error and Status 2 */
#define CE_CAN_21_CRCR       CE_CAN_21.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_21_RXFGMASK   CE_CAN_21.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_21_RXFIR      CE_CAN_21.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_21_CBT        CE_CAN_21.CBT.R               /* CAN Bit Timing */
#define CE_CAN_21_IMASK4     CE_CAN_21.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_21_IMASK3     CE_CAN_21.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_21_IFLAG4     CE_CAN_21.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_21_IFLAG3     CE_CAN_21.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_21_CS0            CE_CAN_21.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_21_ID0            CE_CAN_21.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_21_B0             CE_CAN_21.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_21_H0             CE_CAN_21.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_21_W0             CE_CAN_21.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_21_CS1            CE_CAN_21.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_21_ID1            CE_CAN_21.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_21_B1             CE_CAN_21.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_21_H1             CE_CAN_21.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_21_W1             CE_CAN_21.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_21_CS2            CE_CAN_21.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_21_ID2            CE_CAN_21.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_21_B2             CE_CAN_21.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_21_H2             CE_CAN_21.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_21_W2             CE_CAN_21.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_21_CS3            CE_CAN_21.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_21_ID3            CE_CAN_21.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_21_B3             CE_CAN_21.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_21_H3             CE_CAN_21.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_21_W3             CE_CAN_21.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_21_CS4            CE_CAN_21.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_21_ID4            CE_CAN_21.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_21_B4             CE_CAN_21.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_21_H4             CE_CAN_21.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_21_W4             CE_CAN_21.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_21_CS5            CE_CAN_21.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_21_ID5            CE_CAN_21.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_21_B5             CE_CAN_21.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_21_H5             CE_CAN_21.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_21_W5             CE_CAN_21.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_21_CS6            CE_CAN_21.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_21_ID6            CE_CAN_21.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_21_B6             CE_CAN_21.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_21_H6             CE_CAN_21.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_21_W6             CE_CAN_21.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_21_CS7            CE_CAN_21.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_21_ID7            CE_CAN_21.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_21_B7             CE_CAN_21.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_21_H7             CE_CAN_21.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_21_W7             CE_CAN_21.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_21_CS8            CE_CAN_21.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_21_ID8            CE_CAN_21.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_21_B8             CE_CAN_21.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_21_H8             CE_CAN_21.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_21_W8             CE_CAN_21.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_21_CS9            CE_CAN_21.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_21_ID9            CE_CAN_21.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_21_B9             CE_CAN_21.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_21_H9             CE_CAN_21.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_21_W9             CE_CAN_21.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_21_CS10           CE_CAN_21.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_21_ID10           CE_CAN_21.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_21_B10            CE_CAN_21.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_21_H10            CE_CAN_21.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_21_W10            CE_CAN_21.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_21_CS11           CE_CAN_21.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_21_ID11           CE_CAN_21.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_21_B11            CE_CAN_21.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_21_H11            CE_CAN_21.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_21_W11            CE_CAN_21.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_21_CS12           CE_CAN_21.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_21_ID12           CE_CAN_21.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_21_B12            CE_CAN_21.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_21_H12            CE_CAN_21.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_21_W12            CE_CAN_21.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_21_CS13           CE_CAN_21.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_21_ID13           CE_CAN_21.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_21_B13            CE_CAN_21.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_21_H13            CE_CAN_21.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_21_W13            CE_CAN_21.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_21_CS14           CE_CAN_21.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_21_ID14           CE_CAN_21.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_21_B14            CE_CAN_21.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_21_H14            CE_CAN_21.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_21_W14            CE_CAN_21.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_21_CS15           CE_CAN_21.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_21_ID15           CE_CAN_21.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_21_B15            CE_CAN_21.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_21_H15            CE_CAN_21.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_21_W15            CE_CAN_21.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_21_CS16           CE_CAN_21.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_21_ID16           CE_CAN_21.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_21_B16            CE_CAN_21.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_21_H16            CE_CAN_21.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_21_W16            CE_CAN_21.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_21_CS17           CE_CAN_21.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_21_ID17           CE_CAN_21.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_21_B17            CE_CAN_21.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_21_H17            CE_CAN_21.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_21_W17            CE_CAN_21.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_21_CS18           CE_CAN_21.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_21_ID18           CE_CAN_21.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_21_B18            CE_CAN_21.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_21_H18            CE_CAN_21.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_21_W18            CE_CAN_21.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_21_CS19           CE_CAN_21.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_21_ID19           CE_CAN_21.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_21_B19            CE_CAN_21.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_21_H19            CE_CAN_21.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_21_W19            CE_CAN_21.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_21_CS20           CE_CAN_21.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_21_ID20           CE_CAN_21.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_21_B20            CE_CAN_21.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_21_H20            CE_CAN_21.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_21_W20            CE_CAN_21.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_21_CS21           CE_CAN_21.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_21_ID21           CE_CAN_21.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_21_B21            CE_CAN_21.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_21_H21            CE_CAN_21.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_21_W21            CE_CAN_21.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_21_CS22           CE_CAN_21.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_21_ID22           CE_CAN_21.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_21_B22            CE_CAN_21.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_21_H22            CE_CAN_21.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_21_W22            CE_CAN_21.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_21_CS23           CE_CAN_21.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_21_ID23           CE_CAN_21.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_21_B23            CE_CAN_21.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_21_H23            CE_CAN_21.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_21_W23            CE_CAN_21.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_21_CS24           CE_CAN_21.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_21_ID24           CE_CAN_21.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_21_B24            CE_CAN_21.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_21_H24            CE_CAN_21.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_21_W24            CE_CAN_21.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_21_CS25           CE_CAN_21.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_21_ID25           CE_CAN_21.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_21_B25            CE_CAN_21.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_21_H25            CE_CAN_21.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_21_W25            CE_CAN_21.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_21_CS26           CE_CAN_21.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_21_ID26           CE_CAN_21.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_21_B26            CE_CAN_21.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_21_H26            CE_CAN_21.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_21_W26            CE_CAN_21.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_21_CS27           CE_CAN_21.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_21_ID27           CE_CAN_21.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_21_B27            CE_CAN_21.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_21_H27            CE_CAN_21.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_21_W27            CE_CAN_21.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_21_CS28           CE_CAN_21.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_21_ID28           CE_CAN_21.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_21_B28            CE_CAN_21.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_21_H28            CE_CAN_21.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_21_W28            CE_CAN_21.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_21_CS29           CE_CAN_21.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_21_ID29           CE_CAN_21.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_21_B29            CE_CAN_21.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_21_H29            CE_CAN_21.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_21_W29            CE_CAN_21.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_21_CS30           CE_CAN_21.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_21_ID30           CE_CAN_21.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_21_B30            CE_CAN_21.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_21_H30            CE_CAN_21.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_21_W30            CE_CAN_21.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_21_CS31           CE_CAN_21.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_21_ID31           CE_CAN_21.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_21_B31            CE_CAN_21.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_21_H31            CE_CAN_21.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_21_W31            CE_CAN_21.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_21_CS32           CE_CAN_21.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_21_ID32           CE_CAN_21.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_21_B32            CE_CAN_21.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_21_H32            CE_CAN_21.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_21_W32            CE_CAN_21.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_21_CS33           CE_CAN_21.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_21_ID33           CE_CAN_21.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_21_B33            CE_CAN_21.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_21_H33            CE_CAN_21.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_21_W33            CE_CAN_21.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_21_CS34           CE_CAN_21.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_21_ID34           CE_CAN_21.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_21_B34            CE_CAN_21.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_21_H34            CE_CAN_21.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_21_W34            CE_CAN_21.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_21_CS35           CE_CAN_21.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_21_ID35           CE_CAN_21.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_21_B35            CE_CAN_21.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_21_H35            CE_CAN_21.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_21_W35            CE_CAN_21.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_21_CS36           CE_CAN_21.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_21_ID36           CE_CAN_21.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_21_B36            CE_CAN_21.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_21_H36            CE_CAN_21.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_21_W36            CE_CAN_21.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_21_CS37           CE_CAN_21.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_21_ID37           CE_CAN_21.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_21_B37            CE_CAN_21.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_21_H37            CE_CAN_21.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_21_W37            CE_CAN_21.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_21_CS38           CE_CAN_21.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_21_ID38           CE_CAN_21.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_21_B38            CE_CAN_21.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_21_H38            CE_CAN_21.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_21_W38            CE_CAN_21.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_21_CS39           CE_CAN_21.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_21_ID39           CE_CAN_21.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_21_B39            CE_CAN_21.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_21_H39            CE_CAN_21.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_21_W39            CE_CAN_21.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_21_CS40           CE_CAN_21.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_21_ID40           CE_CAN_21.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_21_B40            CE_CAN_21.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_21_H40            CE_CAN_21.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_21_W40            CE_CAN_21.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_21_CS41           CE_CAN_21.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_21_ID41           CE_CAN_21.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_21_B41            CE_CAN_21.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_21_H41            CE_CAN_21.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_21_W41            CE_CAN_21.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_21_CS42           CE_CAN_21.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_21_ID42           CE_CAN_21.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_21_B42            CE_CAN_21.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_21_H42            CE_CAN_21.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_21_W42            CE_CAN_21.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_21_CS43           CE_CAN_21.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_21_ID43           CE_CAN_21.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_21_B43            CE_CAN_21.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_21_H43            CE_CAN_21.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_21_W43            CE_CAN_21.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_21_CS44           CE_CAN_21.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_21_ID44           CE_CAN_21.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_21_B44            CE_CAN_21.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_21_H44            CE_CAN_21.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_21_W44            CE_CAN_21.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_21_CS45           CE_CAN_21.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_21_ID45           CE_CAN_21.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_21_B45            CE_CAN_21.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_21_H45            CE_CAN_21.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_21_W45            CE_CAN_21.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_21_CS46           CE_CAN_21.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_21_ID46           CE_CAN_21.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_21_B46            CE_CAN_21.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_21_H46            CE_CAN_21.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_21_W46            CE_CAN_21.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_21_CS47           CE_CAN_21.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_21_ID47           CE_CAN_21.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_21_B47            CE_CAN_21.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_21_H47            CE_CAN_21.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_21_W47            CE_CAN_21.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_21_CS48           CE_CAN_21.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_21_ID48           CE_CAN_21.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_21_B48            CE_CAN_21.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_21_H48            CE_CAN_21.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_21_W48            CE_CAN_21.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_21_CS49           CE_CAN_21.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_21_ID49           CE_CAN_21.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_21_B49            CE_CAN_21.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_21_H49            CE_CAN_21.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_21_W49            CE_CAN_21.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_21_CS50           CE_CAN_21.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_21_ID50           CE_CAN_21.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_21_B50            CE_CAN_21.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_21_H50            CE_CAN_21.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_21_W50            CE_CAN_21.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_21_CS51           CE_CAN_21.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_21_ID51           CE_CAN_21.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_21_B51            CE_CAN_21.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_21_H51            CE_CAN_21.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_21_W51            CE_CAN_21.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_21_CS52           CE_CAN_21.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_21_ID52           CE_CAN_21.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_21_B52            CE_CAN_21.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_21_H52            CE_CAN_21.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_21_W52            CE_CAN_21.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_21_CS53           CE_CAN_21.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_21_ID53           CE_CAN_21.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_21_B53            CE_CAN_21.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_21_H53            CE_CAN_21.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_21_W53            CE_CAN_21.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_21_CS54           CE_CAN_21.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_21_ID54           CE_CAN_21.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_21_B54            CE_CAN_21.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_21_H54            CE_CAN_21.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_21_W54            CE_CAN_21.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_21_CS55           CE_CAN_21.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_21_ID55           CE_CAN_21.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_21_B55            CE_CAN_21.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_21_H55            CE_CAN_21.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_21_W55            CE_CAN_21.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_21_CS56           CE_CAN_21.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_21_ID56           CE_CAN_21.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_21_B56            CE_CAN_21.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_21_H56            CE_CAN_21.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_21_W56            CE_CAN_21.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_21_CS57           CE_CAN_21.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_21_ID57           CE_CAN_21.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_21_B57            CE_CAN_21.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_21_H57            CE_CAN_21.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_21_W57            CE_CAN_21.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_21_CS58           CE_CAN_21.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_21_ID58           CE_CAN_21.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_21_B58            CE_CAN_21.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_21_H58            CE_CAN_21.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_21_W58            CE_CAN_21.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_21_CS59           CE_CAN_21.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_21_ID59           CE_CAN_21.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_21_B59            CE_CAN_21.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_21_H59            CE_CAN_21.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_21_W59            CE_CAN_21.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_21_CS60           CE_CAN_21.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_21_ID60           CE_CAN_21.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_21_B60            CE_CAN_21.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_21_H60            CE_CAN_21.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_21_W60            CE_CAN_21.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_21_CS61           CE_CAN_21.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_21_ID61           CE_CAN_21.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_21_B61            CE_CAN_21.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_21_H61            CE_CAN_21.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_21_W61            CE_CAN_21.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_21_CS62           CE_CAN_21.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_21_ID62           CE_CAN_21.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_21_B62            CE_CAN_21.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_21_H62            CE_CAN_21.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_21_W62            CE_CAN_21.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_21_CS63           CE_CAN_21.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_21_ID63           CE_CAN_21.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_21_B63            CE_CAN_21.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_21_H63            CE_CAN_21.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_21_W63            CE_CAN_21.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_21_CS64           CE_CAN_21.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_21_ID64           CE_CAN_21.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_21_B64            CE_CAN_21.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_21_H64            CE_CAN_21.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_21_W64            CE_CAN_21.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_21_CS65           CE_CAN_21.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_21_ID65           CE_CAN_21.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_21_B65            CE_CAN_21.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_21_H65            CE_CAN_21.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_21_W65            CE_CAN_21.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_21_CS66           CE_CAN_21.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_21_ID66           CE_CAN_21.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_21_B66            CE_CAN_21.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_21_H66            CE_CAN_21.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_21_W66            CE_CAN_21.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_21_CS67           CE_CAN_21.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_21_ID67           CE_CAN_21.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_21_B67            CE_CAN_21.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_21_H67            CE_CAN_21.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_21_W67            CE_CAN_21.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_21_CS68           CE_CAN_21.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_21_ID68           CE_CAN_21.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_21_B68            CE_CAN_21.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_21_H68            CE_CAN_21.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_21_W68            CE_CAN_21.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_21_CS69           CE_CAN_21.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_21_ID69           CE_CAN_21.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_21_B69            CE_CAN_21.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_21_H69            CE_CAN_21.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_21_W69            CE_CAN_21.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_21_CS70           CE_CAN_21.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_21_ID70           CE_CAN_21.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_21_B70            CE_CAN_21.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_21_H70            CE_CAN_21.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_21_W70            CE_CAN_21.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_21_CS71           CE_CAN_21.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_21_ID71           CE_CAN_21.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_21_B71            CE_CAN_21.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_21_H71            CE_CAN_21.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_21_W71            CE_CAN_21.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_21_CS72           CE_CAN_21.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_21_ID72           CE_CAN_21.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_21_B72            CE_CAN_21.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_21_H72            CE_CAN_21.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_21_W72            CE_CAN_21.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_21_CS73           CE_CAN_21.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_21_ID73           CE_CAN_21.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_21_B73            CE_CAN_21.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_21_H73            CE_CAN_21.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_21_W73            CE_CAN_21.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_21_CS74           CE_CAN_21.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_21_ID74           CE_CAN_21.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_21_B74            CE_CAN_21.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_21_H74            CE_CAN_21.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_21_W74            CE_CAN_21.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_21_CS75           CE_CAN_21.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_21_ID75           CE_CAN_21.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_21_B75            CE_CAN_21.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_21_H75            CE_CAN_21.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_21_W75            CE_CAN_21.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_21_CS76           CE_CAN_21.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_21_ID76           CE_CAN_21.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_21_B76            CE_CAN_21.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_21_H76            CE_CAN_21.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_21_W76            CE_CAN_21.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_21_CS77           CE_CAN_21.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_21_ID77           CE_CAN_21.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_21_B77            CE_CAN_21.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_21_H77            CE_CAN_21.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_21_W77            CE_CAN_21.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_21_CS78           CE_CAN_21.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_21_ID78           CE_CAN_21.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_21_B78            CE_CAN_21.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_21_H78            CE_CAN_21.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_21_W78            CE_CAN_21.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_21_CS79           CE_CAN_21.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_21_ID79           CE_CAN_21.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_21_B79            CE_CAN_21.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_21_H79            CE_CAN_21.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_21_W79            CE_CAN_21.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_21_CS80           CE_CAN_21.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_21_ID80           CE_CAN_21.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_21_B80            CE_CAN_21.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_21_H80            CE_CAN_21.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_21_W80            CE_CAN_21.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_21_CS81           CE_CAN_21.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_21_ID81           CE_CAN_21.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_21_B81            CE_CAN_21.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_21_H81            CE_CAN_21.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_21_W81            CE_CAN_21.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_21_CS82           CE_CAN_21.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_21_ID82           CE_CAN_21.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_21_B82            CE_CAN_21.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_21_H82            CE_CAN_21.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_21_W82            CE_CAN_21.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_21_CS83           CE_CAN_21.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_21_ID83           CE_CAN_21.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_21_B83            CE_CAN_21.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_21_H83            CE_CAN_21.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_21_W83            CE_CAN_21.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_21_CS84           CE_CAN_21.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_21_ID84           CE_CAN_21.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_21_B84            CE_CAN_21.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_21_H84            CE_CAN_21.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_21_W84            CE_CAN_21.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_21_CS85           CE_CAN_21.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_21_ID85           CE_CAN_21.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_21_B85            CE_CAN_21.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_21_H85            CE_CAN_21.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_21_W85            CE_CAN_21.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_21_CS86           CE_CAN_21.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_21_ID86           CE_CAN_21.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_21_B86            CE_CAN_21.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_21_H86            CE_CAN_21.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_21_W86            CE_CAN_21.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_21_CS87           CE_CAN_21.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_21_ID87           CE_CAN_21.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_21_B87            CE_CAN_21.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_21_H87            CE_CAN_21.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_21_W87            CE_CAN_21.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_21_CS88           CE_CAN_21.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_21_ID88           CE_CAN_21.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_21_B88            CE_CAN_21.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_21_H88            CE_CAN_21.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_21_W848           CE_CAN_21.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_21_CS89           CE_CAN_21.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_21_ID89           CE_CAN_21.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_21_B89            CE_CAN_21.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_21_H89            CE_CAN_21.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_21_W89            CE_CAN_21.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_21_CS90           CE_CAN_21.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_21_ID90           CE_CAN_21.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_21_B90            CE_CAN_21.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_21_H90            CE_CAN_21.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_21_W90            CE_CAN_21.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_21_CS91           CE_CAN_21.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_21_ID91           CE_CAN_21.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_21_B91            CE_CAN_21.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_21_H91            CE_CAN_21.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_21_W91            CE_CAN_21.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_21_CS92           CE_CAN_21.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_21_ID92           CE_CAN_21.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_21_B92            CE_CAN_21.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_21_H92            CE_CAN_21.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_21_W92            CE_CAN_21.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_21_CS93           CE_CAN_21.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_21_ID93           CE_CAN_21.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_21_B93            CE_CAN_21.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_21_H93            CE_CAN_21.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_21_W93            CE_CAN_21.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_21_CS94           CE_CAN_21.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_21_ID94           CE_CAN_21.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_21_B94            CE_CAN_21.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_21_H94            CE_CAN_21.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_21_W94            CE_CAN_21.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_21_CS95           CE_CAN_21.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_21_ID95           CE_CAN_21.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_21_B95            CE_CAN_21.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_21_H95            CE_CAN_21.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_21_W95            CE_CAN_21.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_21_CS96           CE_CAN_21.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_21_ID96           CE_CAN_21.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_21_B96            CE_CAN_21.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_21_H96            CE_CAN_21.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_21_W96            CE_CAN_21.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_21_CS97           CE_CAN_21.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_21_ID97           CE_CAN_21.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_21_B97            CE_CAN_21.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_21_H97            CE_CAN_21.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_21_W97            CE_CAN_21.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_21_CS98           CE_CAN_21.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_21_ID98           CE_CAN_21.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_21_B98            CE_CAN_21.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_21_H98            CE_CAN_21.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_21_W98            CE_CAN_21.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_21_CS99           CE_CAN_21.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_21_ID99           CE_CAN_21.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_21_B99            CE_CAN_21.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_21_H99            CE_CAN_21.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_21_W99            CE_CAN_21.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_21_CS100          CE_CAN_21.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_21_ID100          CE_CAN_21.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_21_B100           CE_CAN_21.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_21_H100           CE_CAN_21.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_21_W100           CE_CAN_21.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_21_CS101          CE_CAN_21.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_21_ID101          CE_CAN_21.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_21_B101           CE_CAN_21.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_21_H101           CE_CAN_21.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_21_W101           CE_CAN_21.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_21_CS102          CE_CAN_21.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_21_ID102          CE_CAN_21.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_21_B102           CE_CAN_21.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_21_H102           CE_CAN_21.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_21_W102           CE_CAN_21.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_21_CS103          CE_CAN_21.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_21_ID103          CE_CAN_21.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_21_B103           CE_CAN_21.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_21_H103           CE_CAN_21.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_21_W103           CE_CAN_21.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_21_CS104          CE_CAN_21.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_21_ID104          CE_CAN_21.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_21_B104           CE_CAN_21.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_21_H104           CE_CAN_21.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_21_W104           CE_CAN_21.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_21_CS105          CE_CAN_21.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_21_ID105          CE_CAN_21.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_21_B105           CE_CAN_21.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_21_H105           CE_CAN_21.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_21_W105           CE_CAN_21.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_21_CS106          CE_CAN_21.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_21_ID106          CE_CAN_21.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_21_B106           CE_CAN_21.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_21_H106           CE_CAN_21.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_21_W106           CE_CAN_21.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_21_CS107          CE_CAN_21.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_21_ID107          CE_CAN_21.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_21_B107           CE_CAN_21.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_21_H107           CE_CAN_21.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_21_W107           CE_CAN_21.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_21_CS108          CE_CAN_21.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_21_ID108          CE_CAN_21.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_21_B108           CE_CAN_21.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_21_H108           CE_CAN_21.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_21_W108           CE_CAN_21.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_21_CS109          CE_CAN_21.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_21_ID109          CE_CAN_21.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_21_B109           CE_CAN_21.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_21_H109           CE_CAN_21.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_21_W109           CE_CAN_21.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_21_CS110          CE_CAN_21.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_21_ID110          CE_CAN_21.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_21_B110           CE_CAN_21.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_21_H110           CE_CAN_21.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_21_W110           CE_CAN_21.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_21_CS111          CE_CAN_21.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_21_ID111          CE_CAN_21.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_21_B111           CE_CAN_21.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_21_H111           CE_CAN_21.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_21_W111           CE_CAN_21.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_21_CS112          CE_CAN_21.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_21_ID112          CE_CAN_21.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_21_B112           CE_CAN_21.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_21_H112           CE_CAN_21.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_21_W112           CE_CAN_21.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_21_CS113          CE_CAN_21.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_21_ID113          CE_CAN_21.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_21_B113           CE_CAN_21.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_21_H113           CE_CAN_21.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_21_W113           CE_CAN_21.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_21_CS114          CE_CAN_21.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_21_ID114          CE_CAN_21.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_21_B114           CE_CAN_21.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_21_H114           CE_CAN_21.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_21_W114           CE_CAN_21.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_21_CS115          CE_CAN_21.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_21_ID115          CE_CAN_21.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_21_B115           CE_CAN_21.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_21_H115           CE_CAN_21.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_21_W115           CE_CAN_21.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_21_CS116          CE_CAN_21.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_21_ID116          CE_CAN_21.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_21_B116           CE_CAN_21.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_21_H116           CE_CAN_21.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_21_W116           CE_CAN_21.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_21_CS117          CE_CAN_21.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_21_ID117          CE_CAN_21.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_21_B117           CE_CAN_21.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_21_H117           CE_CAN_21.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_21_W117           CE_CAN_21.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_21_CS118          CE_CAN_21.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_21_ID118          CE_CAN_21.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_21_B118           CE_CAN_21.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_21_H118           CE_CAN_21.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_21_W118           CE_CAN_21.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_21_CS119          CE_CAN_21.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_21_ID119          CE_CAN_21.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_21_B119           CE_CAN_21.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_21_H119           CE_CAN_21.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_21_W119           CE_CAN_21.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_21_CS120          CE_CAN_21.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_21_ID120          CE_CAN_21.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_21_B120           CE_CAN_21.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_21_H120           CE_CAN_21.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_21_W120           CE_CAN_21.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_21_CS121          CE_CAN_21.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_21_ID121          CE_CAN_21.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_21_B121           CE_CAN_21.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_21_H121           CE_CAN_21.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_21_W121           CE_CAN_21.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_21_CS122          CE_CAN_21.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_21_ID122          CE_CAN_21.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_21_B122           CE_CAN_21.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_21_H122           CE_CAN_21.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_21_W122           CE_CAN_21.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_21_CS123          CE_CAN_21.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_21_ID123          CE_CAN_21.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_21_B123           CE_CAN_21.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_21_H123           CE_CAN_21.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_21_W123           CE_CAN_21.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_21_CS124          CE_CAN_21.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_21_ID124          CE_CAN_21.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_21_B124           CE_CAN_21.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_21_H124           CE_CAN_21.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_21_W124           CE_CAN_21.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_21_CS125          CE_CAN_21.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_21_ID125          CE_CAN_21.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_21_B125           CE_CAN_21.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_21_H125           CE_CAN_21.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_21_W125           CE_CAN_21.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_21_CS126          CE_CAN_21.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_21_ID126          CE_CAN_21.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_21_B126           CE_CAN_21.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_21_H126           CE_CAN_21.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_21_W126           CE_CAN_21.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_21_CS127          CE_CAN_21.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_21_ID127          CE_CAN_21.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_21_B127           CE_CAN_21.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_21_H127           CE_CAN_21.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_21_W127           CE_CAN_21.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_21_RXIMR0     CE_CAN_21.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR1     CE_CAN_21.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR2     CE_CAN_21.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR3     CE_CAN_21.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR4     CE_CAN_21.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR5     CE_CAN_21.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR6     CE_CAN_21.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR7     CE_CAN_21.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR8     CE_CAN_21.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR9     CE_CAN_21.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_21_RXIMR10    CE_CAN_21.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR11    CE_CAN_21.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR12    CE_CAN_21.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR13    CE_CAN_21.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR14    CE_CAN_21.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR15    CE_CAN_21.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR16    CE_CAN_21.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR17    CE_CAN_21.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR18    CE_CAN_21.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR19    CE_CAN_21.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR20    CE_CAN_21.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR21    CE_CAN_21.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR22    CE_CAN_21.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR23    CE_CAN_21.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR24    CE_CAN_21.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR25    CE_CAN_21.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR26    CE_CAN_21.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR27    CE_CAN_21.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR28    CE_CAN_21.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR29    CE_CAN_21.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR30    CE_CAN_21.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR31    CE_CAN_21.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR32    CE_CAN_21.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR33    CE_CAN_21.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR34    CE_CAN_21.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR35    CE_CAN_21.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR36    CE_CAN_21.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR37    CE_CAN_21.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR38    CE_CAN_21.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR39    CE_CAN_21.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR40    CE_CAN_21.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR41    CE_CAN_21.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR42    CE_CAN_21.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR43    CE_CAN_21.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR44    CE_CAN_21.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR45    CE_CAN_21.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR46    CE_CAN_21.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR47    CE_CAN_21.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR48    CE_CAN_21.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR49    CE_CAN_21.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR50    CE_CAN_21.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR51    CE_CAN_21.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR52    CE_CAN_21.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR53    CE_CAN_21.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR54    CE_CAN_21.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR55    CE_CAN_21.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR56    CE_CAN_21.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR57    CE_CAN_21.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR58    CE_CAN_21.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR59    CE_CAN_21.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR60    CE_CAN_21.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR61    CE_CAN_21.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR62    CE_CAN_21.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR63    CE_CAN_21.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR64    CE_CAN_21.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR65    CE_CAN_21.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR66    CE_CAN_21.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR67    CE_CAN_21.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR68    CE_CAN_21.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR69    CE_CAN_21.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR70    CE_CAN_21.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR71    CE_CAN_21.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR72    CE_CAN_21.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR73    CE_CAN_21.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR74    CE_CAN_21.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR75    CE_CAN_21.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR76    CE_CAN_21.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR77    CE_CAN_21.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR78    CE_CAN_21.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR79    CE_CAN_21.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR80    CE_CAN_21.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR81    CE_CAN_21.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR82    CE_CAN_21.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR83    CE_CAN_21.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR84    CE_CAN_21.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR85    CE_CAN_21.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR86    CE_CAN_21.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR87    CE_CAN_21.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR88    CE_CAN_21.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR89    CE_CAN_21.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR90    CE_CAN_21.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR91    CE_CAN_21.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR92    CE_CAN_21.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR93    CE_CAN_21.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR94    CE_CAN_21.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR95    CE_CAN_21.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR96    CE_CAN_21.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR97    CE_CAN_21.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR98    CE_CAN_21.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR99    CE_CAN_21.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_21_RXIMR100   CE_CAN_21.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR101   CE_CAN_21.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR102   CE_CAN_21.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR103   CE_CAN_21.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR104   CE_CAN_21.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR105   CE_CAN_21.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR106   CE_CAN_21.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR107   CE_CAN_21.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR108   CE_CAN_21.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR109   CE_CAN_21.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR110   CE_CAN_21.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR111   CE_CAN_21.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR112   CE_CAN_21.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR113   CE_CAN_21.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR114   CE_CAN_21.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR115   CE_CAN_21.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR116   CE_CAN_21.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR117   CE_CAN_21.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR118   CE_CAN_21.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR119   CE_CAN_21.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR120   CE_CAN_21.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR121   CE_CAN_21.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR122   CE_CAN_21.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR123   CE_CAN_21.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR124   CE_CAN_21.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR125   CE_CAN_21.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR126   CE_CAN_21.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_21_RXIMR127   CE_CAN_21.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_21_MECR       CE_CAN_21.MECR.R              /* Memory Error Control */
#define CE_CAN_21_ERRIAR     CE_CAN_21.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_21_ERRIDPR    CE_CAN_21.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_21_ERRIPPR    CE_CAN_21.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_21_RERRAR     CE_CAN_21.RERRAR.R            /* Error Report Address */
#define CE_CAN_21_RERRDR     CE_CAN_21.RERRDR.R            /* Error Report Data */
#define CE_CAN_21_RERRSYNR   CE_CAN_21.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_21_ERRSR      CE_CAN_21.ERRSR.R             /* Error Status */
#define CE_CAN_21_EPRS       CE_CAN_21.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_21_ENCBT      CE_CAN_21.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_21_EDCBT      CE_CAN_21.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_21_ETDC       CE_CAN_21.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_21_FDCTRL     CE_CAN_21.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_21_FDCBT      CE_CAN_21.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_21_FDCRC      CE_CAN_21.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_21_ERFCR      CE_CAN_21.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_21_ERFIER     CE_CAN_21.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_21_ERFSR      CE_CAN_21.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_21_HR_TIME_STAMP0 CE_CAN_21.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP1 CE_CAN_21.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP2 CE_CAN_21.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP3 CE_CAN_21.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP4 CE_CAN_21.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP5 CE_CAN_21.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP6 CE_CAN_21.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP7 CE_CAN_21.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP8 CE_CAN_21.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP9 CE_CAN_21.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP10 CE_CAN_21.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP11 CE_CAN_21.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP12 CE_CAN_21.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP13 CE_CAN_21.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP14 CE_CAN_21.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP15 CE_CAN_21.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP16 CE_CAN_21.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP17 CE_CAN_21.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP18 CE_CAN_21.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP19 CE_CAN_21.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP20 CE_CAN_21.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP21 CE_CAN_21.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP22 CE_CAN_21.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP23 CE_CAN_21.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP24 CE_CAN_21.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP25 CE_CAN_21.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP26 CE_CAN_21.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP27 CE_CAN_21.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP28 CE_CAN_21.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP29 CE_CAN_21.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP30 CE_CAN_21.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP31 CE_CAN_21.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP32 CE_CAN_21.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP33 CE_CAN_21.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP34 CE_CAN_21.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP35 CE_CAN_21.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP36 CE_CAN_21.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP37 CE_CAN_21.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP38 CE_CAN_21.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP39 CE_CAN_21.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP40 CE_CAN_21.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP41 CE_CAN_21.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP42 CE_CAN_21.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP43 CE_CAN_21.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP44 CE_CAN_21.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP45 CE_CAN_21.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP46 CE_CAN_21.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP47 CE_CAN_21.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP48 CE_CAN_21.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP49 CE_CAN_21.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP50 CE_CAN_21.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP51 CE_CAN_21.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP52 CE_CAN_21.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP53 CE_CAN_21.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP54 CE_CAN_21.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP55 CE_CAN_21.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP56 CE_CAN_21.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP57 CE_CAN_21.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP58 CE_CAN_21.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP59 CE_CAN_21.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP60 CE_CAN_21.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP61 CE_CAN_21.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP62 CE_CAN_21.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP63 CE_CAN_21.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP64 CE_CAN_21.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP65 CE_CAN_21.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP66 CE_CAN_21.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP67 CE_CAN_21.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP68 CE_CAN_21.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP69 CE_CAN_21.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP70 CE_CAN_21.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP71 CE_CAN_21.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP72 CE_CAN_21.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP73 CE_CAN_21.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP74 CE_CAN_21.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP75 CE_CAN_21.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP76 CE_CAN_21.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP77 CE_CAN_21.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP78 CE_CAN_21.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP79 CE_CAN_21.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP80 CE_CAN_21.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP81 CE_CAN_21.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP82 CE_CAN_21.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP83 CE_CAN_21.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP84 CE_CAN_21.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP85 CE_CAN_21.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP86 CE_CAN_21.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP87 CE_CAN_21.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP88 CE_CAN_21.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP89 CE_CAN_21.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP90 CE_CAN_21.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP91 CE_CAN_21.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP92 CE_CAN_21.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP93 CE_CAN_21.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP94 CE_CAN_21.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP95 CE_CAN_21.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP96 CE_CAN_21.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP97 CE_CAN_21.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP98 CE_CAN_21.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP99 CE_CAN_21.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP100 CE_CAN_21.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP101 CE_CAN_21.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP102 CE_CAN_21.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP103 CE_CAN_21.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP104 CE_CAN_21.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP105 CE_CAN_21.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP106 CE_CAN_21.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP107 CE_CAN_21.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP108 CE_CAN_21.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP109 CE_CAN_21.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP110 CE_CAN_21.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP111 CE_CAN_21.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP112 CE_CAN_21.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP113 CE_CAN_21.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP114 CE_CAN_21.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP115 CE_CAN_21.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP116 CE_CAN_21.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP117 CE_CAN_21.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP118 CE_CAN_21.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP119 CE_CAN_21.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP120 CE_CAN_21.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP121 CE_CAN_21.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP122 CE_CAN_21.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP123 CE_CAN_21.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP124 CE_CAN_21.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP125 CE_CAN_21.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP126 CE_CAN_21.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_21_HR_TIME_STAMP127 CE_CAN_21.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_21_ERFFEL0    CE_CAN_21.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL1    CE_CAN_21.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL2    CE_CAN_21.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL3    CE_CAN_21.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL4    CE_CAN_21.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL5    CE_CAN_21.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL6    CE_CAN_21.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL7    CE_CAN_21.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL8    CE_CAN_21.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL9    CE_CAN_21.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL10   CE_CAN_21.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL11   CE_CAN_21.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL12   CE_CAN_21.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL13   CE_CAN_21.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL14   CE_CAN_21.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL15   CE_CAN_21.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL16   CE_CAN_21.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL17   CE_CAN_21.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL18   CE_CAN_21.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL19   CE_CAN_21.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL20   CE_CAN_21.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL21   CE_CAN_21.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL22   CE_CAN_21.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL23   CE_CAN_21.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL24   CE_CAN_21.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL25   CE_CAN_21.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL26   CE_CAN_21.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL27   CE_CAN_21.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL28   CE_CAN_21.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL29   CE_CAN_21.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL30   CE_CAN_21.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL31   CE_CAN_21.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL32   CE_CAN_21.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL33   CE_CAN_21.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL34   CE_CAN_21.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL35   CE_CAN_21.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL36   CE_CAN_21.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL37   CE_CAN_21.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL38   CE_CAN_21.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL39   CE_CAN_21.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL40   CE_CAN_21.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL41   CE_CAN_21.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL42   CE_CAN_21.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL43   CE_CAN_21.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL44   CE_CAN_21.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL45   CE_CAN_21.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL46   CE_CAN_21.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL47   CE_CAN_21.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL48   CE_CAN_21.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL49   CE_CAN_21.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL50   CE_CAN_21.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL51   CE_CAN_21.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL52   CE_CAN_21.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL53   CE_CAN_21.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL54   CE_CAN_21.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL55   CE_CAN_21.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL56   CE_CAN_21.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL57   CE_CAN_21.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL58   CE_CAN_21.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL59   CE_CAN_21.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL60   CE_CAN_21.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL61   CE_CAN_21.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL62   CE_CAN_21.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL63   CE_CAN_21.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL64   CE_CAN_21.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL65   CE_CAN_21.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL66   CE_CAN_21.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL67   CE_CAN_21.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL68   CE_CAN_21.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL69   CE_CAN_21.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL70   CE_CAN_21.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL71   CE_CAN_21.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL72   CE_CAN_21.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL73   CE_CAN_21.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL74   CE_CAN_21.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL75   CE_CAN_21.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL76   CE_CAN_21.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL77   CE_CAN_21.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL78   CE_CAN_21.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL79   CE_CAN_21.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL80   CE_CAN_21.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL81   CE_CAN_21.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL82   CE_CAN_21.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL83   CE_CAN_21.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL84   CE_CAN_21.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL85   CE_CAN_21.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL86   CE_CAN_21.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL87   CE_CAN_21.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL88   CE_CAN_21.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL89   CE_CAN_21.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL90   CE_CAN_21.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL91   CE_CAN_21.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL92   CE_CAN_21.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL93   CE_CAN_21.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL94   CE_CAN_21.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL95   CE_CAN_21.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL96   CE_CAN_21.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL97   CE_CAN_21.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL98   CE_CAN_21.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL99   CE_CAN_21.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL100  CE_CAN_21.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL101  CE_CAN_21.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL102  CE_CAN_21.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL103  CE_CAN_21.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL104  CE_CAN_21.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL105  CE_CAN_21.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL106  CE_CAN_21.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL107  CE_CAN_21.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL108  CE_CAN_21.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL109  CE_CAN_21.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL110  CE_CAN_21.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL111  CE_CAN_21.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL112  CE_CAN_21.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL113  CE_CAN_21.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL114  CE_CAN_21.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL115  CE_CAN_21.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL116  CE_CAN_21.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL117  CE_CAN_21.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL118  CE_CAN_21.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL119  CE_CAN_21.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL120  CE_CAN_21.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL121  CE_CAN_21.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL122  CE_CAN_21.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL123  CE_CAN_21.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL124  CE_CAN_21.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL125  CE_CAN_21.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL126  CE_CAN_21.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_21_ERFFEL127  CE_CAN_21.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_22_MCR        CE_CAN_22.MCR.R               /* Module Configuration */
#define CE_CAN_22_CTRL1      CE_CAN_22.CTRL1.R             /* Control 1 */
#define CE_CAN_22_TIMER      CE_CAN_22.TIMER.R             /* Free-Running Timer */
#define CE_CAN_22_RXMGMASK   CE_CAN_22.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_22_RX14MASK   CE_CAN_22.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_22_RX15MASK   CE_CAN_22.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_22_ECR        CE_CAN_22.ECR.R               /* Error Counter */
#define CE_CAN_22_ESR1       CE_CAN_22.ESR1.R              /* Error and Status 1 */
#define CE_CAN_22_IMASK2     CE_CAN_22.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_22_IMASK1     CE_CAN_22.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_22_IFLAG2     CE_CAN_22.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_22_IFLAG1     CE_CAN_22.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_22_CTRL2      CE_CAN_22.CTRL2.R             /* Control 2 */
#define CE_CAN_22_ESR2       CE_CAN_22.ESR2.R              /* Error and Status 2 */
#define CE_CAN_22_CRCR       CE_CAN_22.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_22_RXFGMASK   CE_CAN_22.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_22_RXFIR      CE_CAN_22.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_22_CBT        CE_CAN_22.CBT.R               /* CAN Bit Timing */
#define CE_CAN_22_IMASK4     CE_CAN_22.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_22_IMASK3     CE_CAN_22.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_22_IFLAG4     CE_CAN_22.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_22_IFLAG3     CE_CAN_22.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_22_CS0            CE_CAN_22.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_22_ID0            CE_CAN_22.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_22_B0             CE_CAN_22.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_22_H0             CE_CAN_22.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_22_W0             CE_CAN_22.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_22_CS1            CE_CAN_22.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_22_ID1            CE_CAN_22.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_22_B1             CE_CAN_22.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_22_H1             CE_CAN_22.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_22_W1             CE_CAN_22.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_22_CS2            CE_CAN_22.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_22_ID2            CE_CAN_22.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_22_B2             CE_CAN_22.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_22_H2             CE_CAN_22.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_22_W2             CE_CAN_22.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_22_CS3            CE_CAN_22.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_22_ID3            CE_CAN_22.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_22_B3             CE_CAN_22.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_22_H3             CE_CAN_22.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_22_W3             CE_CAN_22.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_22_CS4            CE_CAN_22.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_22_ID4            CE_CAN_22.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_22_B4             CE_CAN_22.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_22_H4             CE_CAN_22.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_22_W4             CE_CAN_22.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_22_CS5            CE_CAN_22.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_22_ID5            CE_CAN_22.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_22_B5             CE_CAN_22.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_22_H5             CE_CAN_22.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_22_W5             CE_CAN_22.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_22_CS6            CE_CAN_22.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_22_ID6            CE_CAN_22.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_22_B6             CE_CAN_22.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_22_H6             CE_CAN_22.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_22_W6             CE_CAN_22.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_22_CS7            CE_CAN_22.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_22_ID7            CE_CAN_22.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_22_B7             CE_CAN_22.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_22_H7             CE_CAN_22.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_22_W7             CE_CAN_22.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_22_CS8            CE_CAN_22.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_22_ID8            CE_CAN_22.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_22_B8             CE_CAN_22.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_22_H8             CE_CAN_22.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_22_W8             CE_CAN_22.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_22_CS9            CE_CAN_22.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_22_ID9            CE_CAN_22.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_22_B9             CE_CAN_22.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_22_H9             CE_CAN_22.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_22_W9             CE_CAN_22.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_22_CS10           CE_CAN_22.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_22_ID10           CE_CAN_22.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_22_B10            CE_CAN_22.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_22_H10            CE_CAN_22.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_22_W10            CE_CAN_22.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_22_CS11           CE_CAN_22.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_22_ID11           CE_CAN_22.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_22_B11            CE_CAN_22.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_22_H11            CE_CAN_22.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_22_W11            CE_CAN_22.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_22_CS12           CE_CAN_22.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_22_ID12           CE_CAN_22.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_22_B12            CE_CAN_22.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_22_H12            CE_CAN_22.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_22_W12            CE_CAN_22.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_22_CS13           CE_CAN_22.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_22_ID13           CE_CAN_22.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_22_B13            CE_CAN_22.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_22_H13            CE_CAN_22.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_22_W13            CE_CAN_22.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_22_CS14           CE_CAN_22.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_22_ID14           CE_CAN_22.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_22_B14            CE_CAN_22.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_22_H14            CE_CAN_22.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_22_W14            CE_CAN_22.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_22_CS15           CE_CAN_22.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_22_ID15           CE_CAN_22.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_22_B15            CE_CAN_22.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_22_H15            CE_CAN_22.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_22_W15            CE_CAN_22.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_22_CS16           CE_CAN_22.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_22_ID16           CE_CAN_22.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_22_B16            CE_CAN_22.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_22_H16            CE_CAN_22.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_22_W16            CE_CAN_22.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_22_CS17           CE_CAN_22.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_22_ID17           CE_CAN_22.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_22_B17            CE_CAN_22.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_22_H17            CE_CAN_22.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_22_W17            CE_CAN_22.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_22_CS18           CE_CAN_22.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_22_ID18           CE_CAN_22.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_22_B18            CE_CAN_22.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_22_H18            CE_CAN_22.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_22_W18            CE_CAN_22.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_22_CS19           CE_CAN_22.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_22_ID19           CE_CAN_22.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_22_B19            CE_CAN_22.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_22_H19            CE_CAN_22.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_22_W19            CE_CAN_22.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_22_CS20           CE_CAN_22.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_22_ID20           CE_CAN_22.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_22_B20            CE_CAN_22.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_22_H20            CE_CAN_22.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_22_W20            CE_CAN_22.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_22_CS21           CE_CAN_22.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_22_ID21           CE_CAN_22.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_22_B21            CE_CAN_22.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_22_H21            CE_CAN_22.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_22_W21            CE_CAN_22.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_22_CS22           CE_CAN_22.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_22_ID22           CE_CAN_22.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_22_B22            CE_CAN_22.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_22_H22            CE_CAN_22.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_22_W22            CE_CAN_22.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_22_CS23           CE_CAN_22.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_22_ID23           CE_CAN_22.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_22_B23            CE_CAN_22.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_22_H23            CE_CAN_22.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_22_W23            CE_CAN_22.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_22_CS24           CE_CAN_22.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_22_ID24           CE_CAN_22.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_22_B24            CE_CAN_22.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_22_H24            CE_CAN_22.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_22_W24            CE_CAN_22.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_22_CS25           CE_CAN_22.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_22_ID25           CE_CAN_22.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_22_B25            CE_CAN_22.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_22_H25            CE_CAN_22.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_22_W25            CE_CAN_22.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_22_CS26           CE_CAN_22.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_22_ID26           CE_CAN_22.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_22_B26            CE_CAN_22.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_22_H26            CE_CAN_22.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_22_W26            CE_CAN_22.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_22_CS27           CE_CAN_22.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_22_ID27           CE_CAN_22.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_22_B27            CE_CAN_22.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_22_H27            CE_CAN_22.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_22_W27            CE_CAN_22.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_22_CS28           CE_CAN_22.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_22_ID28           CE_CAN_22.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_22_B28            CE_CAN_22.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_22_H28            CE_CAN_22.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_22_W28            CE_CAN_22.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_22_CS29           CE_CAN_22.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_22_ID29           CE_CAN_22.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_22_B29            CE_CAN_22.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_22_H29            CE_CAN_22.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_22_W29            CE_CAN_22.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_22_CS30           CE_CAN_22.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_22_ID30           CE_CAN_22.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_22_B30            CE_CAN_22.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_22_H30            CE_CAN_22.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_22_W30            CE_CAN_22.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_22_CS31           CE_CAN_22.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_22_ID31           CE_CAN_22.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_22_B31            CE_CAN_22.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_22_H31            CE_CAN_22.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_22_W31            CE_CAN_22.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_22_CS32           CE_CAN_22.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_22_ID32           CE_CAN_22.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_22_B32            CE_CAN_22.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_22_H32            CE_CAN_22.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_22_W32            CE_CAN_22.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_22_CS33           CE_CAN_22.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_22_ID33           CE_CAN_22.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_22_B33            CE_CAN_22.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_22_H33            CE_CAN_22.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_22_W33            CE_CAN_22.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_22_CS34           CE_CAN_22.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_22_ID34           CE_CAN_22.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_22_B34            CE_CAN_22.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_22_H34            CE_CAN_22.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_22_W34            CE_CAN_22.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_22_CS35           CE_CAN_22.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_22_ID35           CE_CAN_22.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_22_B35            CE_CAN_22.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_22_H35            CE_CAN_22.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_22_W35            CE_CAN_22.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_22_CS36           CE_CAN_22.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_22_ID36           CE_CAN_22.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_22_B36            CE_CAN_22.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_22_H36            CE_CAN_22.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_22_W36            CE_CAN_22.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_22_CS37           CE_CAN_22.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_22_ID37           CE_CAN_22.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_22_B37            CE_CAN_22.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_22_H37            CE_CAN_22.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_22_W37            CE_CAN_22.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_22_CS38           CE_CAN_22.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_22_ID38           CE_CAN_22.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_22_B38            CE_CAN_22.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_22_H38            CE_CAN_22.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_22_W38            CE_CAN_22.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_22_CS39           CE_CAN_22.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_22_ID39           CE_CAN_22.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_22_B39            CE_CAN_22.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_22_H39            CE_CAN_22.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_22_W39            CE_CAN_22.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_22_CS40           CE_CAN_22.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_22_ID40           CE_CAN_22.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_22_B40            CE_CAN_22.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_22_H40            CE_CAN_22.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_22_W40            CE_CAN_22.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_22_CS41           CE_CAN_22.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_22_ID41           CE_CAN_22.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_22_B41            CE_CAN_22.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_22_H41            CE_CAN_22.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_22_W41            CE_CAN_22.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_22_CS42           CE_CAN_22.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_22_ID42           CE_CAN_22.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_22_B42            CE_CAN_22.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_22_H42            CE_CAN_22.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_22_W42            CE_CAN_22.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_22_CS43           CE_CAN_22.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_22_ID43           CE_CAN_22.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_22_B43            CE_CAN_22.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_22_H43            CE_CAN_22.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_22_W43            CE_CAN_22.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_22_CS44           CE_CAN_22.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_22_ID44           CE_CAN_22.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_22_B44            CE_CAN_22.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_22_H44            CE_CAN_22.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_22_W44            CE_CAN_22.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_22_CS45           CE_CAN_22.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_22_ID45           CE_CAN_22.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_22_B45            CE_CAN_22.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_22_H45            CE_CAN_22.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_22_W45            CE_CAN_22.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_22_CS46           CE_CAN_22.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_22_ID46           CE_CAN_22.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_22_B46            CE_CAN_22.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_22_H46            CE_CAN_22.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_22_W46            CE_CAN_22.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_22_CS47           CE_CAN_22.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_22_ID47           CE_CAN_22.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_22_B47            CE_CAN_22.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_22_H47            CE_CAN_22.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_22_W47            CE_CAN_22.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_22_CS48           CE_CAN_22.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_22_ID48           CE_CAN_22.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_22_B48            CE_CAN_22.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_22_H48            CE_CAN_22.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_22_W48            CE_CAN_22.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_22_CS49           CE_CAN_22.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_22_ID49           CE_CAN_22.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_22_B49            CE_CAN_22.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_22_H49            CE_CAN_22.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_22_W49            CE_CAN_22.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_22_CS50           CE_CAN_22.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_22_ID50           CE_CAN_22.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_22_B50            CE_CAN_22.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_22_H50            CE_CAN_22.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_22_W50            CE_CAN_22.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_22_CS51           CE_CAN_22.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_22_ID51           CE_CAN_22.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_22_B51            CE_CAN_22.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_22_H51            CE_CAN_22.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_22_W51            CE_CAN_22.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_22_CS52           CE_CAN_22.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_22_ID52           CE_CAN_22.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_22_B52            CE_CAN_22.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_22_H52            CE_CAN_22.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_22_W52            CE_CAN_22.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_22_CS53           CE_CAN_22.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_22_ID53           CE_CAN_22.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_22_B53            CE_CAN_22.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_22_H53            CE_CAN_22.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_22_W53            CE_CAN_22.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_22_CS54           CE_CAN_22.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_22_ID54           CE_CAN_22.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_22_B54            CE_CAN_22.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_22_H54            CE_CAN_22.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_22_W54            CE_CAN_22.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_22_CS55           CE_CAN_22.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_22_ID55           CE_CAN_22.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_22_B55            CE_CAN_22.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_22_H55            CE_CAN_22.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_22_W55            CE_CAN_22.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_22_CS56           CE_CAN_22.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_22_ID56           CE_CAN_22.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_22_B56            CE_CAN_22.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_22_H56            CE_CAN_22.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_22_W56            CE_CAN_22.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_22_CS57           CE_CAN_22.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_22_ID57           CE_CAN_22.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_22_B57            CE_CAN_22.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_22_H57            CE_CAN_22.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_22_W57            CE_CAN_22.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_22_CS58           CE_CAN_22.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_22_ID58           CE_CAN_22.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_22_B58            CE_CAN_22.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_22_H58            CE_CAN_22.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_22_W58            CE_CAN_22.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_22_CS59           CE_CAN_22.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_22_ID59           CE_CAN_22.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_22_B59            CE_CAN_22.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_22_H59            CE_CAN_22.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_22_W59            CE_CAN_22.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_22_CS60           CE_CAN_22.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_22_ID60           CE_CAN_22.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_22_B60            CE_CAN_22.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_22_H60            CE_CAN_22.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_22_W60            CE_CAN_22.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_22_CS61           CE_CAN_22.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_22_ID61           CE_CAN_22.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_22_B61            CE_CAN_22.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_22_H61            CE_CAN_22.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_22_W61            CE_CAN_22.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_22_CS62           CE_CAN_22.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_22_ID62           CE_CAN_22.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_22_B62            CE_CAN_22.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_22_H62            CE_CAN_22.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_22_W62            CE_CAN_22.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_22_CS63           CE_CAN_22.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_22_ID63           CE_CAN_22.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_22_B63            CE_CAN_22.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_22_H63            CE_CAN_22.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_22_W63            CE_CAN_22.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_22_CS64           CE_CAN_22.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_22_ID64           CE_CAN_22.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_22_B64            CE_CAN_22.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_22_H64            CE_CAN_22.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_22_W64            CE_CAN_22.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_22_CS65           CE_CAN_22.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_22_ID65           CE_CAN_22.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_22_B65            CE_CAN_22.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_22_H65            CE_CAN_22.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_22_W65            CE_CAN_22.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_22_CS66           CE_CAN_22.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_22_ID66           CE_CAN_22.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_22_B66            CE_CAN_22.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_22_H66            CE_CAN_22.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_22_W66            CE_CAN_22.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_22_CS67           CE_CAN_22.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_22_ID67           CE_CAN_22.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_22_B67            CE_CAN_22.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_22_H67            CE_CAN_22.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_22_W67            CE_CAN_22.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_22_CS68           CE_CAN_22.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_22_ID68           CE_CAN_22.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_22_B68            CE_CAN_22.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_22_H68            CE_CAN_22.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_22_W68            CE_CAN_22.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_22_CS69           CE_CAN_22.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_22_ID69           CE_CAN_22.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_22_B69            CE_CAN_22.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_22_H69            CE_CAN_22.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_22_W69            CE_CAN_22.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_22_CS70           CE_CAN_22.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_22_ID70           CE_CAN_22.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_22_B70            CE_CAN_22.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_22_H70            CE_CAN_22.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_22_W70            CE_CAN_22.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_22_CS71           CE_CAN_22.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_22_ID71           CE_CAN_22.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_22_B71            CE_CAN_22.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_22_H71            CE_CAN_22.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_22_W71            CE_CAN_22.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_22_CS72           CE_CAN_22.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_22_ID72           CE_CAN_22.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_22_B72            CE_CAN_22.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_22_H72            CE_CAN_22.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_22_W72            CE_CAN_22.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_22_CS73           CE_CAN_22.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_22_ID73           CE_CAN_22.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_22_B73            CE_CAN_22.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_22_H73            CE_CAN_22.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_22_W73            CE_CAN_22.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_22_CS74           CE_CAN_22.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_22_ID74           CE_CAN_22.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_22_B74            CE_CAN_22.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_22_H74            CE_CAN_22.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_22_W74            CE_CAN_22.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_22_CS75           CE_CAN_22.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_22_ID75           CE_CAN_22.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_22_B75            CE_CAN_22.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_22_H75            CE_CAN_22.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_22_W75            CE_CAN_22.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_22_CS76           CE_CAN_22.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_22_ID76           CE_CAN_22.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_22_B76            CE_CAN_22.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_22_H76            CE_CAN_22.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_22_W76            CE_CAN_22.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_22_CS77           CE_CAN_22.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_22_ID77           CE_CAN_22.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_22_B77            CE_CAN_22.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_22_H77            CE_CAN_22.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_22_W77            CE_CAN_22.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_22_CS78           CE_CAN_22.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_22_ID78           CE_CAN_22.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_22_B78            CE_CAN_22.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_22_H78            CE_CAN_22.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_22_W78            CE_CAN_22.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_22_CS79           CE_CAN_22.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_22_ID79           CE_CAN_22.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_22_B79            CE_CAN_22.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_22_H79            CE_CAN_22.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_22_W79            CE_CAN_22.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_22_CS80           CE_CAN_22.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_22_ID80           CE_CAN_22.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_22_B80            CE_CAN_22.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_22_H80            CE_CAN_22.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_22_W80            CE_CAN_22.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_22_CS81           CE_CAN_22.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_22_ID81           CE_CAN_22.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_22_B81            CE_CAN_22.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_22_H81            CE_CAN_22.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_22_W81            CE_CAN_22.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_22_CS82           CE_CAN_22.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_22_ID82           CE_CAN_22.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_22_B82            CE_CAN_22.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_22_H82            CE_CAN_22.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_22_W82            CE_CAN_22.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_22_CS83           CE_CAN_22.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_22_ID83           CE_CAN_22.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_22_B83            CE_CAN_22.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_22_H83            CE_CAN_22.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_22_W83            CE_CAN_22.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_22_CS84           CE_CAN_22.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_22_ID84           CE_CAN_22.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_22_B84            CE_CAN_22.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_22_H84            CE_CAN_22.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_22_W84            CE_CAN_22.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_22_CS85           CE_CAN_22.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_22_ID85           CE_CAN_22.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_22_B85            CE_CAN_22.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_22_H85            CE_CAN_22.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_22_W85            CE_CAN_22.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_22_CS86           CE_CAN_22.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_22_ID86           CE_CAN_22.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_22_B86            CE_CAN_22.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_22_H86            CE_CAN_22.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_22_W86            CE_CAN_22.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_22_CS87           CE_CAN_22.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_22_ID87           CE_CAN_22.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_22_B87            CE_CAN_22.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_22_H87            CE_CAN_22.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_22_W87            CE_CAN_22.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_22_CS88           CE_CAN_22.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_22_ID88           CE_CAN_22.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_22_B88            CE_CAN_22.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_22_H88            CE_CAN_22.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_22_W848           CE_CAN_22.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_22_CS89           CE_CAN_22.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_22_ID89           CE_CAN_22.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_22_B89            CE_CAN_22.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_22_H89            CE_CAN_22.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_22_W89            CE_CAN_22.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_22_CS90           CE_CAN_22.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_22_ID90           CE_CAN_22.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_22_B90            CE_CAN_22.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_22_H90            CE_CAN_22.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_22_W90            CE_CAN_22.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_22_CS91           CE_CAN_22.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_22_ID91           CE_CAN_22.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_22_B91            CE_CAN_22.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_22_H91            CE_CAN_22.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_22_W91            CE_CAN_22.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_22_CS92           CE_CAN_22.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_22_ID92           CE_CAN_22.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_22_B92            CE_CAN_22.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_22_H92            CE_CAN_22.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_22_W92            CE_CAN_22.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_22_CS93           CE_CAN_22.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_22_ID93           CE_CAN_22.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_22_B93            CE_CAN_22.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_22_H93            CE_CAN_22.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_22_W93            CE_CAN_22.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_22_CS94           CE_CAN_22.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_22_ID94           CE_CAN_22.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_22_B94            CE_CAN_22.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_22_H94            CE_CAN_22.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_22_W94            CE_CAN_22.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_22_CS95           CE_CAN_22.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_22_ID95           CE_CAN_22.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_22_B95            CE_CAN_22.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_22_H95            CE_CAN_22.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_22_W95            CE_CAN_22.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_22_CS96           CE_CAN_22.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_22_ID96           CE_CAN_22.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_22_B96            CE_CAN_22.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_22_H96            CE_CAN_22.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_22_W96            CE_CAN_22.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_22_CS97           CE_CAN_22.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_22_ID97           CE_CAN_22.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_22_B97            CE_CAN_22.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_22_H97            CE_CAN_22.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_22_W97            CE_CAN_22.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_22_CS98           CE_CAN_22.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_22_ID98           CE_CAN_22.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_22_B98            CE_CAN_22.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_22_H98            CE_CAN_22.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_22_W98            CE_CAN_22.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_22_CS99           CE_CAN_22.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_22_ID99           CE_CAN_22.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_22_B99            CE_CAN_22.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_22_H99            CE_CAN_22.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_22_W99            CE_CAN_22.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_22_CS100          CE_CAN_22.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_22_ID100          CE_CAN_22.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_22_B100           CE_CAN_22.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_22_H100           CE_CAN_22.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_22_W100           CE_CAN_22.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_22_CS101          CE_CAN_22.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_22_ID101          CE_CAN_22.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_22_B101           CE_CAN_22.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_22_H101           CE_CAN_22.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_22_W101           CE_CAN_22.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_22_CS102          CE_CAN_22.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_22_ID102          CE_CAN_22.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_22_B102           CE_CAN_22.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_22_H102           CE_CAN_22.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_22_W102           CE_CAN_22.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_22_CS103          CE_CAN_22.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_22_ID103          CE_CAN_22.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_22_B103           CE_CAN_22.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_22_H103           CE_CAN_22.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_22_W103           CE_CAN_22.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_22_CS104          CE_CAN_22.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_22_ID104          CE_CAN_22.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_22_B104           CE_CAN_22.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_22_H104           CE_CAN_22.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_22_W104           CE_CAN_22.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_22_CS105          CE_CAN_22.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_22_ID105          CE_CAN_22.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_22_B105           CE_CAN_22.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_22_H105           CE_CAN_22.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_22_W105           CE_CAN_22.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_22_CS106          CE_CAN_22.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_22_ID106          CE_CAN_22.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_22_B106           CE_CAN_22.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_22_H106           CE_CAN_22.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_22_W106           CE_CAN_22.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_22_CS107          CE_CAN_22.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_22_ID107          CE_CAN_22.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_22_B107           CE_CAN_22.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_22_H107           CE_CAN_22.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_22_W107           CE_CAN_22.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_22_CS108          CE_CAN_22.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_22_ID108          CE_CAN_22.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_22_B108           CE_CAN_22.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_22_H108           CE_CAN_22.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_22_W108           CE_CAN_22.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_22_CS109          CE_CAN_22.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_22_ID109          CE_CAN_22.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_22_B109           CE_CAN_22.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_22_H109           CE_CAN_22.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_22_W109           CE_CAN_22.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_22_CS110          CE_CAN_22.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_22_ID110          CE_CAN_22.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_22_B110           CE_CAN_22.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_22_H110           CE_CAN_22.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_22_W110           CE_CAN_22.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_22_CS111          CE_CAN_22.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_22_ID111          CE_CAN_22.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_22_B111           CE_CAN_22.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_22_H111           CE_CAN_22.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_22_W111           CE_CAN_22.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_22_CS112          CE_CAN_22.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_22_ID112          CE_CAN_22.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_22_B112           CE_CAN_22.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_22_H112           CE_CAN_22.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_22_W112           CE_CAN_22.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_22_CS113          CE_CAN_22.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_22_ID113          CE_CAN_22.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_22_B113           CE_CAN_22.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_22_H113           CE_CAN_22.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_22_W113           CE_CAN_22.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_22_CS114          CE_CAN_22.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_22_ID114          CE_CAN_22.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_22_B114           CE_CAN_22.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_22_H114           CE_CAN_22.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_22_W114           CE_CAN_22.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_22_CS115          CE_CAN_22.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_22_ID115          CE_CAN_22.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_22_B115           CE_CAN_22.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_22_H115           CE_CAN_22.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_22_W115           CE_CAN_22.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_22_CS116          CE_CAN_22.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_22_ID116          CE_CAN_22.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_22_B116           CE_CAN_22.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_22_H116           CE_CAN_22.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_22_W116           CE_CAN_22.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_22_CS117          CE_CAN_22.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_22_ID117          CE_CAN_22.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_22_B117           CE_CAN_22.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_22_H117           CE_CAN_22.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_22_W117           CE_CAN_22.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_22_CS118          CE_CAN_22.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_22_ID118          CE_CAN_22.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_22_B118           CE_CAN_22.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_22_H118           CE_CAN_22.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_22_W118           CE_CAN_22.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_22_CS119          CE_CAN_22.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_22_ID119          CE_CAN_22.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_22_B119           CE_CAN_22.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_22_H119           CE_CAN_22.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_22_W119           CE_CAN_22.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_22_CS120          CE_CAN_22.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_22_ID120          CE_CAN_22.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_22_B120           CE_CAN_22.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_22_H120           CE_CAN_22.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_22_W120           CE_CAN_22.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_22_CS121          CE_CAN_22.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_22_ID121          CE_CAN_22.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_22_B121           CE_CAN_22.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_22_H121           CE_CAN_22.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_22_W121           CE_CAN_22.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_22_CS122          CE_CAN_22.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_22_ID122          CE_CAN_22.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_22_B122           CE_CAN_22.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_22_H122           CE_CAN_22.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_22_W122           CE_CAN_22.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_22_CS123          CE_CAN_22.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_22_ID123          CE_CAN_22.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_22_B123           CE_CAN_22.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_22_H123           CE_CAN_22.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_22_W123           CE_CAN_22.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_22_CS124          CE_CAN_22.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_22_ID124          CE_CAN_22.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_22_B124           CE_CAN_22.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_22_H124           CE_CAN_22.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_22_W124           CE_CAN_22.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_22_CS125          CE_CAN_22.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_22_ID125          CE_CAN_22.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_22_B125           CE_CAN_22.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_22_H125           CE_CAN_22.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_22_W125           CE_CAN_22.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_22_CS126          CE_CAN_22.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_22_ID126          CE_CAN_22.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_22_B126           CE_CAN_22.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_22_H126           CE_CAN_22.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_22_W126           CE_CAN_22.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_22_CS127          CE_CAN_22.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_22_ID127          CE_CAN_22.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_22_B127           CE_CAN_22.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_22_H127           CE_CAN_22.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_22_W127           CE_CAN_22.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_22_RXIMR0     CE_CAN_22.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR1     CE_CAN_22.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR2     CE_CAN_22.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR3     CE_CAN_22.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR4     CE_CAN_22.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR5     CE_CAN_22.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR6     CE_CAN_22.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR7     CE_CAN_22.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR8     CE_CAN_22.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR9     CE_CAN_22.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_22_RXIMR10    CE_CAN_22.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR11    CE_CAN_22.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR12    CE_CAN_22.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR13    CE_CAN_22.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR14    CE_CAN_22.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR15    CE_CAN_22.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR16    CE_CAN_22.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR17    CE_CAN_22.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR18    CE_CAN_22.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR19    CE_CAN_22.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR20    CE_CAN_22.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR21    CE_CAN_22.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR22    CE_CAN_22.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR23    CE_CAN_22.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR24    CE_CAN_22.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR25    CE_CAN_22.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR26    CE_CAN_22.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR27    CE_CAN_22.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR28    CE_CAN_22.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR29    CE_CAN_22.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR30    CE_CAN_22.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR31    CE_CAN_22.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR32    CE_CAN_22.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR33    CE_CAN_22.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR34    CE_CAN_22.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR35    CE_CAN_22.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR36    CE_CAN_22.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR37    CE_CAN_22.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR38    CE_CAN_22.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR39    CE_CAN_22.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR40    CE_CAN_22.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR41    CE_CAN_22.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR42    CE_CAN_22.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR43    CE_CAN_22.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR44    CE_CAN_22.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR45    CE_CAN_22.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR46    CE_CAN_22.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR47    CE_CAN_22.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR48    CE_CAN_22.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR49    CE_CAN_22.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR50    CE_CAN_22.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR51    CE_CAN_22.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR52    CE_CAN_22.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR53    CE_CAN_22.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR54    CE_CAN_22.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR55    CE_CAN_22.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR56    CE_CAN_22.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR57    CE_CAN_22.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR58    CE_CAN_22.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR59    CE_CAN_22.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR60    CE_CAN_22.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR61    CE_CAN_22.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR62    CE_CAN_22.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR63    CE_CAN_22.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR64    CE_CAN_22.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR65    CE_CAN_22.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR66    CE_CAN_22.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR67    CE_CAN_22.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR68    CE_CAN_22.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR69    CE_CAN_22.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR70    CE_CAN_22.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR71    CE_CAN_22.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR72    CE_CAN_22.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR73    CE_CAN_22.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR74    CE_CAN_22.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR75    CE_CAN_22.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR76    CE_CAN_22.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR77    CE_CAN_22.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR78    CE_CAN_22.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR79    CE_CAN_22.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR80    CE_CAN_22.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR81    CE_CAN_22.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR82    CE_CAN_22.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR83    CE_CAN_22.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR84    CE_CAN_22.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR85    CE_CAN_22.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR86    CE_CAN_22.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR87    CE_CAN_22.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR88    CE_CAN_22.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR89    CE_CAN_22.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR90    CE_CAN_22.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR91    CE_CAN_22.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR92    CE_CAN_22.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR93    CE_CAN_22.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR94    CE_CAN_22.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR95    CE_CAN_22.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR96    CE_CAN_22.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR97    CE_CAN_22.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR98    CE_CAN_22.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR99    CE_CAN_22.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_22_RXIMR100   CE_CAN_22.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR101   CE_CAN_22.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR102   CE_CAN_22.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR103   CE_CAN_22.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR104   CE_CAN_22.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR105   CE_CAN_22.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR106   CE_CAN_22.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR107   CE_CAN_22.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR108   CE_CAN_22.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR109   CE_CAN_22.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR110   CE_CAN_22.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR111   CE_CAN_22.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR112   CE_CAN_22.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR113   CE_CAN_22.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR114   CE_CAN_22.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR115   CE_CAN_22.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR116   CE_CAN_22.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR117   CE_CAN_22.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR118   CE_CAN_22.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR119   CE_CAN_22.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR120   CE_CAN_22.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR121   CE_CAN_22.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR122   CE_CAN_22.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR123   CE_CAN_22.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR124   CE_CAN_22.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR125   CE_CAN_22.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR126   CE_CAN_22.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_22_RXIMR127   CE_CAN_22.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_22_MECR       CE_CAN_22.MECR.R              /* Memory Error Control */
#define CE_CAN_22_ERRIAR     CE_CAN_22.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_22_ERRIDPR    CE_CAN_22.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_22_ERRIPPR    CE_CAN_22.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_22_RERRAR     CE_CAN_22.RERRAR.R            /* Error Report Address */
#define CE_CAN_22_RERRDR     CE_CAN_22.RERRDR.R            /* Error Report Data */
#define CE_CAN_22_RERRSYNR   CE_CAN_22.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_22_ERRSR      CE_CAN_22.ERRSR.R             /* Error Status */
#define CE_CAN_22_EPRS       CE_CAN_22.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_22_ENCBT      CE_CAN_22.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_22_EDCBT      CE_CAN_22.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_22_ETDC       CE_CAN_22.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_22_FDCTRL     CE_CAN_22.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_22_FDCBT      CE_CAN_22.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_22_FDCRC      CE_CAN_22.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_22_ERFCR      CE_CAN_22.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_22_ERFIER     CE_CAN_22.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_22_ERFSR      CE_CAN_22.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_22_HR_TIME_STAMP0 CE_CAN_22.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP1 CE_CAN_22.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP2 CE_CAN_22.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP3 CE_CAN_22.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP4 CE_CAN_22.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP5 CE_CAN_22.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP6 CE_CAN_22.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP7 CE_CAN_22.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP8 CE_CAN_22.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP9 CE_CAN_22.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP10 CE_CAN_22.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP11 CE_CAN_22.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP12 CE_CAN_22.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP13 CE_CAN_22.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP14 CE_CAN_22.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP15 CE_CAN_22.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP16 CE_CAN_22.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP17 CE_CAN_22.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP18 CE_CAN_22.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP19 CE_CAN_22.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP20 CE_CAN_22.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP21 CE_CAN_22.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP22 CE_CAN_22.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP23 CE_CAN_22.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP24 CE_CAN_22.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP25 CE_CAN_22.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP26 CE_CAN_22.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP27 CE_CAN_22.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP28 CE_CAN_22.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP29 CE_CAN_22.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP30 CE_CAN_22.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP31 CE_CAN_22.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP32 CE_CAN_22.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP33 CE_CAN_22.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP34 CE_CAN_22.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP35 CE_CAN_22.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP36 CE_CAN_22.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP37 CE_CAN_22.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP38 CE_CAN_22.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP39 CE_CAN_22.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP40 CE_CAN_22.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP41 CE_CAN_22.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP42 CE_CAN_22.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP43 CE_CAN_22.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP44 CE_CAN_22.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP45 CE_CAN_22.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP46 CE_CAN_22.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP47 CE_CAN_22.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP48 CE_CAN_22.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP49 CE_CAN_22.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP50 CE_CAN_22.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP51 CE_CAN_22.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP52 CE_CAN_22.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP53 CE_CAN_22.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP54 CE_CAN_22.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP55 CE_CAN_22.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP56 CE_CAN_22.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP57 CE_CAN_22.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP58 CE_CAN_22.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP59 CE_CAN_22.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP60 CE_CAN_22.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP61 CE_CAN_22.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP62 CE_CAN_22.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP63 CE_CAN_22.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP64 CE_CAN_22.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP65 CE_CAN_22.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP66 CE_CAN_22.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP67 CE_CAN_22.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP68 CE_CAN_22.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP69 CE_CAN_22.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP70 CE_CAN_22.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP71 CE_CAN_22.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP72 CE_CAN_22.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP73 CE_CAN_22.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP74 CE_CAN_22.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP75 CE_CAN_22.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP76 CE_CAN_22.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP77 CE_CAN_22.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP78 CE_CAN_22.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP79 CE_CAN_22.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP80 CE_CAN_22.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP81 CE_CAN_22.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP82 CE_CAN_22.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP83 CE_CAN_22.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP84 CE_CAN_22.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP85 CE_CAN_22.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP86 CE_CAN_22.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP87 CE_CAN_22.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP88 CE_CAN_22.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP89 CE_CAN_22.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP90 CE_CAN_22.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP91 CE_CAN_22.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP92 CE_CAN_22.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP93 CE_CAN_22.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP94 CE_CAN_22.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP95 CE_CAN_22.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP96 CE_CAN_22.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP97 CE_CAN_22.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP98 CE_CAN_22.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP99 CE_CAN_22.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP100 CE_CAN_22.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP101 CE_CAN_22.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP102 CE_CAN_22.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP103 CE_CAN_22.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP104 CE_CAN_22.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP105 CE_CAN_22.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP106 CE_CAN_22.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP107 CE_CAN_22.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP108 CE_CAN_22.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP109 CE_CAN_22.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP110 CE_CAN_22.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP111 CE_CAN_22.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP112 CE_CAN_22.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP113 CE_CAN_22.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP114 CE_CAN_22.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP115 CE_CAN_22.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP116 CE_CAN_22.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP117 CE_CAN_22.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP118 CE_CAN_22.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP119 CE_CAN_22.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP120 CE_CAN_22.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP121 CE_CAN_22.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP122 CE_CAN_22.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP123 CE_CAN_22.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP124 CE_CAN_22.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP125 CE_CAN_22.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP126 CE_CAN_22.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_22_HR_TIME_STAMP127 CE_CAN_22.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_22_ERFFEL0    CE_CAN_22.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL1    CE_CAN_22.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL2    CE_CAN_22.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL3    CE_CAN_22.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL4    CE_CAN_22.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL5    CE_CAN_22.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL6    CE_CAN_22.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL7    CE_CAN_22.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL8    CE_CAN_22.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL9    CE_CAN_22.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL10   CE_CAN_22.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL11   CE_CAN_22.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL12   CE_CAN_22.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL13   CE_CAN_22.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL14   CE_CAN_22.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL15   CE_CAN_22.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL16   CE_CAN_22.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL17   CE_CAN_22.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL18   CE_CAN_22.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL19   CE_CAN_22.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL20   CE_CAN_22.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL21   CE_CAN_22.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL22   CE_CAN_22.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL23   CE_CAN_22.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL24   CE_CAN_22.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL25   CE_CAN_22.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL26   CE_CAN_22.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL27   CE_CAN_22.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL28   CE_CAN_22.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL29   CE_CAN_22.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL30   CE_CAN_22.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL31   CE_CAN_22.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL32   CE_CAN_22.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL33   CE_CAN_22.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL34   CE_CAN_22.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL35   CE_CAN_22.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL36   CE_CAN_22.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL37   CE_CAN_22.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL38   CE_CAN_22.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL39   CE_CAN_22.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL40   CE_CAN_22.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL41   CE_CAN_22.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL42   CE_CAN_22.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL43   CE_CAN_22.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL44   CE_CAN_22.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL45   CE_CAN_22.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL46   CE_CAN_22.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL47   CE_CAN_22.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL48   CE_CAN_22.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL49   CE_CAN_22.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL50   CE_CAN_22.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL51   CE_CAN_22.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL52   CE_CAN_22.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL53   CE_CAN_22.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL54   CE_CAN_22.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL55   CE_CAN_22.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL56   CE_CAN_22.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL57   CE_CAN_22.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL58   CE_CAN_22.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL59   CE_CAN_22.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL60   CE_CAN_22.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL61   CE_CAN_22.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL62   CE_CAN_22.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL63   CE_CAN_22.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL64   CE_CAN_22.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL65   CE_CAN_22.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL66   CE_CAN_22.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL67   CE_CAN_22.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL68   CE_CAN_22.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL69   CE_CAN_22.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL70   CE_CAN_22.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL71   CE_CAN_22.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL72   CE_CAN_22.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL73   CE_CAN_22.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL74   CE_CAN_22.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL75   CE_CAN_22.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL76   CE_CAN_22.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL77   CE_CAN_22.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL78   CE_CAN_22.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL79   CE_CAN_22.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL80   CE_CAN_22.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL81   CE_CAN_22.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL82   CE_CAN_22.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL83   CE_CAN_22.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL84   CE_CAN_22.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL85   CE_CAN_22.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL86   CE_CAN_22.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL87   CE_CAN_22.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL88   CE_CAN_22.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL89   CE_CAN_22.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL90   CE_CAN_22.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL91   CE_CAN_22.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL92   CE_CAN_22.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL93   CE_CAN_22.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL94   CE_CAN_22.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL95   CE_CAN_22.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL96   CE_CAN_22.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL97   CE_CAN_22.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL98   CE_CAN_22.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL99   CE_CAN_22.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL100  CE_CAN_22.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL101  CE_CAN_22.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL102  CE_CAN_22.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL103  CE_CAN_22.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL104  CE_CAN_22.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL105  CE_CAN_22.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL106  CE_CAN_22.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL107  CE_CAN_22.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL108  CE_CAN_22.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL109  CE_CAN_22.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL110  CE_CAN_22.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL111  CE_CAN_22.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL112  CE_CAN_22.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL113  CE_CAN_22.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL114  CE_CAN_22.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL115  CE_CAN_22.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL116  CE_CAN_22.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL117  CE_CAN_22.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL118  CE_CAN_22.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL119  CE_CAN_22.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL120  CE_CAN_22.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL121  CE_CAN_22.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL122  CE_CAN_22.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL123  CE_CAN_22.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL124  CE_CAN_22.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL125  CE_CAN_22.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL126  CE_CAN_22.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_22_ERFFEL127  CE_CAN_22.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_23_MCR        CE_CAN_23.MCR.R               /* Module Configuration */
#define CE_CAN_23_CTRL1      CE_CAN_23.CTRL1.R             /* Control 1 */
#define CE_CAN_23_TIMER      CE_CAN_23.TIMER.R             /* Free-Running Timer */
#define CE_CAN_23_RXMGMASK   CE_CAN_23.RXMGMASK.R          /* RX Message Buffers Global Mask */
#define CE_CAN_23_RX14MASK   CE_CAN_23.RX14MASK.R          /* Receive 14 Mask */
#define CE_CAN_23_RX15MASK   CE_CAN_23.RX15MASK.R          /* Receive 15 Mask */
#define CE_CAN_23_ECR        CE_CAN_23.ECR.R               /* Error Counter */
#define CE_CAN_23_ESR1       CE_CAN_23.ESR1.R              /* Error and Status 1 */
#define CE_CAN_23_IMASK2     CE_CAN_23.IMASK2.R            /* Interrupt Masks 2 */
#define CE_CAN_23_IMASK1     CE_CAN_23.IMASK1.R            /* Interrupt Masks 1 */
#define CE_CAN_23_IFLAG2     CE_CAN_23.IFLAG2.R            /* Interrupt Flags 2 */
#define CE_CAN_23_IFLAG1     CE_CAN_23.IFLAG1.R            /* Interrupt Flags 1 */
#define CE_CAN_23_CTRL2      CE_CAN_23.CTRL2.R             /* Control 2 */
#define CE_CAN_23_ESR2       CE_CAN_23.ESR2.R              /* Error and Status 2 */
#define CE_CAN_23_CRCR       CE_CAN_23.CRCR.R              /* Cyclic Redundancy Check */
#define CE_CAN_23_RXFGMASK   CE_CAN_23.RXFGMASK.R          /* Legacy RX FIFO Global Mask */
#define CE_CAN_23_RXFIR      CE_CAN_23.RXFIR.R             /* Legacy RX FIFO Information */
#define CE_CAN_23_CBT        CE_CAN_23.CBT.R               /* CAN Bit Timing */
#define CE_CAN_23_IMASK4     CE_CAN_23.IMASK4.R            /* Interrupt Masks 4 */
#define CE_CAN_23_IMASK3     CE_CAN_23.IMASK3.R            /* Interrupt Masks 3 */
#define CE_CAN_23_IFLAG4     CE_CAN_23.IFLAG4.R            /* Interrupt Flags 4 */
#define CE_CAN_23_IFLAG3     CE_CAN_23.IFLAG3.R            /* Interrupt Flags 3 */
#define CE_CAN_23_CS0            CE_CAN_23.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_23_ID0            CE_CAN_23.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_23_B0             CE_CAN_23.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_23_H0             CE_CAN_23.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_23_W0             CE_CAN_23.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_23_CS1            CE_CAN_23.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_23_ID1            CE_CAN_23.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_23_B1             CE_CAN_23.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_23_H1             CE_CAN_23.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_23_W1             CE_CAN_23.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_23_CS2            CE_CAN_23.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_23_ID2            CE_CAN_23.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_23_B2             CE_CAN_23.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_23_H2             CE_CAN_23.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_23_W2             CE_CAN_23.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_23_CS3            CE_CAN_23.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_23_ID3            CE_CAN_23.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_23_B3             CE_CAN_23.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_23_H3             CE_CAN_23.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_23_W3             CE_CAN_23.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_23_CS4            CE_CAN_23.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_23_ID4            CE_CAN_23.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_23_B4             CE_CAN_23.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_23_H4             CE_CAN_23.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_23_W4             CE_CAN_23.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_23_CS5            CE_CAN_23.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_23_ID5            CE_CAN_23.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_23_B5             CE_CAN_23.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_23_H5             CE_CAN_23.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_23_W5             CE_CAN_23.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_23_CS6            CE_CAN_23.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_23_ID6            CE_CAN_23.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_23_B6             CE_CAN_23.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_23_H6             CE_CAN_23.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_23_W6             CE_CAN_23.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_23_CS7            CE_CAN_23.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_23_ID7            CE_CAN_23.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_23_B7             CE_CAN_23.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_23_H7             CE_CAN_23.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_23_W7             CE_CAN_23.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_23_CS8            CE_CAN_23.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_23_ID8            CE_CAN_23.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_23_B8             CE_CAN_23.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_23_H8             CE_CAN_23.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_23_W8             CE_CAN_23.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_23_CS9            CE_CAN_23.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_23_ID9            CE_CAN_23.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_23_B9             CE_CAN_23.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_23_H9             CE_CAN_23.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_23_W9             CE_CAN_23.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_23_CS10           CE_CAN_23.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_23_ID10           CE_CAN_23.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_23_B10            CE_CAN_23.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_23_H10            CE_CAN_23.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_23_W10            CE_CAN_23.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_23_CS11           CE_CAN_23.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_23_ID11           CE_CAN_23.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_23_B11            CE_CAN_23.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_23_H11            CE_CAN_23.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_23_W11            CE_CAN_23.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_23_CS12           CE_CAN_23.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_23_ID12           CE_CAN_23.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_23_B12            CE_CAN_23.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_23_H12            CE_CAN_23.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_23_W12            CE_CAN_23.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_23_CS13           CE_CAN_23.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_23_ID13           CE_CAN_23.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_23_B13            CE_CAN_23.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_23_H13            CE_CAN_23.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_23_W13            CE_CAN_23.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_23_CS14           CE_CAN_23.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_23_ID14           CE_CAN_23.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_23_B14            CE_CAN_23.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_23_H14            CE_CAN_23.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_23_W14            CE_CAN_23.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_23_CS15           CE_CAN_23.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_23_ID15           CE_CAN_23.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_23_B15            CE_CAN_23.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_23_H15            CE_CAN_23.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_23_W15            CE_CAN_23.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_23_CS16           CE_CAN_23.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_23_ID16           CE_CAN_23.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_23_B16            CE_CAN_23.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_23_H16            CE_CAN_23.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_23_W16            CE_CAN_23.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_23_CS17           CE_CAN_23.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_23_ID17           CE_CAN_23.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_23_B17            CE_CAN_23.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_23_H17            CE_CAN_23.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_23_W17            CE_CAN_23.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_23_CS18           CE_CAN_23.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_23_ID18           CE_CAN_23.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_23_B18            CE_CAN_23.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_23_H18            CE_CAN_23.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_23_W18            CE_CAN_23.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_23_CS19           CE_CAN_23.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_23_ID19           CE_CAN_23.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_23_B19            CE_CAN_23.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_23_H19            CE_CAN_23.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_23_W19            CE_CAN_23.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_23_CS20           CE_CAN_23.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_23_ID20           CE_CAN_23.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_23_B20            CE_CAN_23.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_23_H20            CE_CAN_23.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_23_W20            CE_CAN_23.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_23_CS21           CE_CAN_23.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_23_ID21           CE_CAN_23.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_23_B21            CE_CAN_23.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_23_H21            CE_CAN_23.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_23_W21            CE_CAN_23.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_23_CS22           CE_CAN_23.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_23_ID22           CE_CAN_23.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_23_B22            CE_CAN_23.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_23_H22            CE_CAN_23.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_23_W22            CE_CAN_23.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_23_CS23           CE_CAN_23.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_23_ID23           CE_CAN_23.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_23_B23            CE_CAN_23.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_23_H23            CE_CAN_23.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_23_W23            CE_CAN_23.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_23_CS24           CE_CAN_23.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_23_ID24           CE_CAN_23.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_23_B24            CE_CAN_23.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_23_H24            CE_CAN_23.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_23_W24            CE_CAN_23.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_23_CS25           CE_CAN_23.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_23_ID25           CE_CAN_23.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_23_B25            CE_CAN_23.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_23_H25            CE_CAN_23.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_23_W25            CE_CAN_23.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_23_CS26           CE_CAN_23.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_23_ID26           CE_CAN_23.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_23_B26            CE_CAN_23.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_23_H26            CE_CAN_23.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_23_W26            CE_CAN_23.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_23_CS27           CE_CAN_23.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_23_ID27           CE_CAN_23.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_23_B27            CE_CAN_23.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_23_H27            CE_CAN_23.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_23_W27            CE_CAN_23.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_23_CS28           CE_CAN_23.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_23_ID28           CE_CAN_23.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_23_B28            CE_CAN_23.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_23_H28            CE_CAN_23.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_23_W28            CE_CAN_23.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_23_CS29           CE_CAN_23.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_23_ID29           CE_CAN_23.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_23_B29            CE_CAN_23.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_23_H29            CE_CAN_23.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_23_W29            CE_CAN_23.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_23_CS30           CE_CAN_23.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_23_ID30           CE_CAN_23.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_23_B30            CE_CAN_23.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_23_H30            CE_CAN_23.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_23_W30            CE_CAN_23.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_23_CS31           CE_CAN_23.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_23_ID31           CE_CAN_23.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_23_B31            CE_CAN_23.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_23_H31            CE_CAN_23.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_23_W31            CE_CAN_23.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_23_CS32           CE_CAN_23.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_23_ID32           CE_CAN_23.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_23_B32            CE_CAN_23.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_23_H32            CE_CAN_23.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_23_W32            CE_CAN_23.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_23_CS33           CE_CAN_23.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_23_ID33           CE_CAN_23.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_23_B33            CE_CAN_23.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_23_H33            CE_CAN_23.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_23_W33            CE_CAN_23.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_23_CS34           CE_CAN_23.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_23_ID34           CE_CAN_23.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_23_B34            CE_CAN_23.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_23_H34            CE_CAN_23.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_23_W34            CE_CAN_23.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_23_CS35           CE_CAN_23.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_23_ID35           CE_CAN_23.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_23_B35            CE_CAN_23.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_23_H35            CE_CAN_23.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_23_W35            CE_CAN_23.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_23_CS36           CE_CAN_23.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_23_ID36           CE_CAN_23.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_23_B36            CE_CAN_23.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_23_H36            CE_CAN_23.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_23_W36            CE_CAN_23.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_23_CS37           CE_CAN_23.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_23_ID37           CE_CAN_23.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_23_B37            CE_CAN_23.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_23_H37            CE_CAN_23.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_23_W37            CE_CAN_23.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_23_CS38           CE_CAN_23.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_23_ID38           CE_CAN_23.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_23_B38            CE_CAN_23.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_23_H38            CE_CAN_23.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_23_W38            CE_CAN_23.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_23_CS39           CE_CAN_23.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_23_ID39           CE_CAN_23.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_23_B39            CE_CAN_23.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_23_H39            CE_CAN_23.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_23_W39            CE_CAN_23.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_23_CS40           CE_CAN_23.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_23_ID40           CE_CAN_23.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_23_B40            CE_CAN_23.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_23_H40            CE_CAN_23.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_23_W40            CE_CAN_23.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_23_CS41           CE_CAN_23.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_23_ID41           CE_CAN_23.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_23_B41            CE_CAN_23.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_23_H41            CE_CAN_23.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_23_W41            CE_CAN_23.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_23_CS42           CE_CAN_23.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_23_ID42           CE_CAN_23.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_23_B42            CE_CAN_23.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_23_H42            CE_CAN_23.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_23_W42            CE_CAN_23.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_23_CS43           CE_CAN_23.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_23_ID43           CE_CAN_23.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_23_B43            CE_CAN_23.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_23_H43            CE_CAN_23.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_23_W43            CE_CAN_23.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_23_CS44           CE_CAN_23.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_23_ID44           CE_CAN_23.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_23_B44            CE_CAN_23.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_23_H44            CE_CAN_23.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_23_W44            CE_CAN_23.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_23_CS45           CE_CAN_23.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_23_ID45           CE_CAN_23.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_23_B45            CE_CAN_23.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_23_H45            CE_CAN_23.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_23_W45            CE_CAN_23.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_23_CS46           CE_CAN_23.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_23_ID46           CE_CAN_23.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_23_B46            CE_CAN_23.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_23_H46            CE_CAN_23.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_23_W46            CE_CAN_23.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_23_CS47           CE_CAN_23.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_23_ID47           CE_CAN_23.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_23_B47            CE_CAN_23.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_23_H47            CE_CAN_23.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_23_W47            CE_CAN_23.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_23_CS48           CE_CAN_23.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_23_ID48           CE_CAN_23.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_23_B48            CE_CAN_23.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_23_H48            CE_CAN_23.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_23_W48            CE_CAN_23.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_23_CS49           CE_CAN_23.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_23_ID49           CE_CAN_23.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_23_B49            CE_CAN_23.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_23_H49            CE_CAN_23.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_23_W49            CE_CAN_23.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_23_CS50           CE_CAN_23.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_23_ID50           CE_CAN_23.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_23_B50            CE_CAN_23.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_23_H50            CE_CAN_23.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_23_W50            CE_CAN_23.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_23_CS51           CE_CAN_23.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_23_ID51           CE_CAN_23.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_23_B51            CE_CAN_23.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_23_H51            CE_CAN_23.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_23_W51            CE_CAN_23.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_23_CS52           CE_CAN_23.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_23_ID52           CE_CAN_23.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_23_B52            CE_CAN_23.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_23_H52            CE_CAN_23.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_23_W52            CE_CAN_23.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_23_CS53           CE_CAN_23.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_23_ID53           CE_CAN_23.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_23_B53            CE_CAN_23.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_23_H53            CE_CAN_23.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_23_W53            CE_CAN_23.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_23_CS54           CE_CAN_23.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_23_ID54           CE_CAN_23.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_23_B54            CE_CAN_23.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_23_H54            CE_CAN_23.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_23_W54            CE_CAN_23.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_23_CS55           CE_CAN_23.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_23_ID55           CE_CAN_23.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_23_B55            CE_CAN_23.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_23_H55            CE_CAN_23.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_23_W55            CE_CAN_23.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_23_CS56           CE_CAN_23.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_23_ID56           CE_CAN_23.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_23_B56            CE_CAN_23.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_23_H56            CE_CAN_23.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_23_W56            CE_CAN_23.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_23_CS57           CE_CAN_23.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_23_ID57           CE_CAN_23.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_23_B57            CE_CAN_23.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_23_H57            CE_CAN_23.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_23_W57            CE_CAN_23.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_23_CS58           CE_CAN_23.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_23_ID58           CE_CAN_23.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_23_B58            CE_CAN_23.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_23_H58            CE_CAN_23.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_23_W58            CE_CAN_23.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_23_CS59           CE_CAN_23.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_23_ID59           CE_CAN_23.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_23_B59            CE_CAN_23.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_23_H59            CE_CAN_23.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_23_W59            CE_CAN_23.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_23_CS60           CE_CAN_23.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_23_ID60           CE_CAN_23.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_23_B60            CE_CAN_23.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_23_H60            CE_CAN_23.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_23_W60            CE_CAN_23.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_23_CS61           CE_CAN_23.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_23_ID61           CE_CAN_23.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_23_B61            CE_CAN_23.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_23_H61            CE_CAN_23.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_23_W61            CE_CAN_23.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_23_CS62           CE_CAN_23.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_23_ID62           CE_CAN_23.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_23_B62            CE_CAN_23.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_23_H62            CE_CAN_23.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_23_W62            CE_CAN_23.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_23_CS63           CE_CAN_23.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_23_ID63           CE_CAN_23.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_23_B63            CE_CAN_23.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_23_H63            CE_CAN_23.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_23_W63            CE_CAN_23.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_23_CS64           CE_CAN_23.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_23_ID64           CE_CAN_23.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_23_B64            CE_CAN_23.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_23_H64            CE_CAN_23.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_23_W64            CE_CAN_23.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_23_CS65           CE_CAN_23.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_23_ID65           CE_CAN_23.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_23_B65            CE_CAN_23.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_23_H65            CE_CAN_23.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_23_W65            CE_CAN_23.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_23_CS66           CE_CAN_23.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_23_ID66           CE_CAN_23.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_23_B66            CE_CAN_23.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_23_H66            CE_CAN_23.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_23_W66            CE_CAN_23.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_23_CS67           CE_CAN_23.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_23_ID67           CE_CAN_23.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_23_B67            CE_CAN_23.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_23_H67            CE_CAN_23.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_23_W67            CE_CAN_23.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_23_CS68           CE_CAN_23.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_23_ID68           CE_CAN_23.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_23_B68            CE_CAN_23.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_23_H68            CE_CAN_23.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_23_W68            CE_CAN_23.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_23_CS69           CE_CAN_23.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_23_ID69           CE_CAN_23.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_23_B69            CE_CAN_23.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_23_H69            CE_CAN_23.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_23_W69            CE_CAN_23.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_23_CS70           CE_CAN_23.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_23_ID70           CE_CAN_23.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_23_B70            CE_CAN_23.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_23_H70            CE_CAN_23.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_23_W70            CE_CAN_23.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_23_CS71           CE_CAN_23.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_23_ID71           CE_CAN_23.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_23_B71            CE_CAN_23.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_23_H71            CE_CAN_23.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_23_W71            CE_CAN_23.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_23_CS72           CE_CAN_23.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_23_ID72           CE_CAN_23.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_23_B72            CE_CAN_23.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_23_H72            CE_CAN_23.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_23_W72            CE_CAN_23.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_23_CS73           CE_CAN_23.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_23_ID73           CE_CAN_23.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_23_B73            CE_CAN_23.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_23_H73            CE_CAN_23.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_23_W73            CE_CAN_23.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_23_CS74           CE_CAN_23.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_23_ID74           CE_CAN_23.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_23_B74            CE_CAN_23.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_23_H74            CE_CAN_23.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_23_W74            CE_CAN_23.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_23_CS75           CE_CAN_23.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_23_ID75           CE_CAN_23.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_23_B75            CE_CAN_23.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_23_H75            CE_CAN_23.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_23_W75            CE_CAN_23.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_23_CS76           CE_CAN_23.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_23_ID76           CE_CAN_23.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_23_B76            CE_CAN_23.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_23_H76            CE_CAN_23.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_23_W76            CE_CAN_23.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_23_CS77           CE_CAN_23.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_23_ID77           CE_CAN_23.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_23_B77            CE_CAN_23.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_23_H77            CE_CAN_23.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_23_W77            CE_CAN_23.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_23_CS78           CE_CAN_23.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_23_ID78           CE_CAN_23.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_23_B78            CE_CAN_23.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_23_H78            CE_CAN_23.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_23_W78            CE_CAN_23.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_23_CS79           CE_CAN_23.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_23_ID79           CE_CAN_23.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_23_B79            CE_CAN_23.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_23_H79            CE_CAN_23.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_23_W79            CE_CAN_23.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_23_CS80           CE_CAN_23.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_23_ID80           CE_CAN_23.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_23_B80            CE_CAN_23.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_23_H80            CE_CAN_23.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_23_W80            CE_CAN_23.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_23_CS81           CE_CAN_23.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_23_ID81           CE_CAN_23.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_23_B81            CE_CAN_23.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_23_H81            CE_CAN_23.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_23_W81            CE_CAN_23.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_23_CS82           CE_CAN_23.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_23_ID82           CE_CAN_23.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_23_B82            CE_CAN_23.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_23_H82            CE_CAN_23.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_23_W82            CE_CAN_23.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_23_CS83           CE_CAN_23.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_23_ID83           CE_CAN_23.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_23_B83            CE_CAN_23.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_23_H83            CE_CAN_23.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_23_W83            CE_CAN_23.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_23_CS84           CE_CAN_23.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_23_ID84           CE_CAN_23.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_23_B84            CE_CAN_23.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_23_H84            CE_CAN_23.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_23_W84            CE_CAN_23.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_23_CS85           CE_CAN_23.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_23_ID85           CE_CAN_23.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_23_B85            CE_CAN_23.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_23_H85            CE_CAN_23.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_23_W85            CE_CAN_23.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_23_CS86           CE_CAN_23.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_23_ID86           CE_CAN_23.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_23_B86            CE_CAN_23.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_23_H86            CE_CAN_23.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_23_W86            CE_CAN_23.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_23_CS87           CE_CAN_23.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_23_ID87           CE_CAN_23.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_23_B87            CE_CAN_23.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_23_H87            CE_CAN_23.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_23_W87            CE_CAN_23.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_23_CS88           CE_CAN_23.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_23_ID88           CE_CAN_23.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_23_B88            CE_CAN_23.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_23_H88            CE_CAN_23.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_23_W848           CE_CAN_23.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_23_CS89           CE_CAN_23.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_23_ID89           CE_CAN_23.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_23_B89            CE_CAN_23.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_23_H89            CE_CAN_23.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_23_W89            CE_CAN_23.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_23_CS90           CE_CAN_23.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_23_ID90           CE_CAN_23.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_23_B90            CE_CAN_23.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_23_H90            CE_CAN_23.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_23_W90            CE_CAN_23.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_23_CS91           CE_CAN_23.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_23_ID91           CE_CAN_23.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_23_B91            CE_CAN_23.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_23_H91            CE_CAN_23.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_23_W91            CE_CAN_23.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_23_CS92           CE_CAN_23.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_23_ID92           CE_CAN_23.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_23_B92            CE_CAN_23.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_23_H92            CE_CAN_23.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_23_W92            CE_CAN_23.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_23_CS93           CE_CAN_23.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_23_ID93           CE_CAN_23.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_23_B93            CE_CAN_23.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_23_H93            CE_CAN_23.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_23_W93            CE_CAN_23.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_23_CS94           CE_CAN_23.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_23_ID94           CE_CAN_23.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_23_B94            CE_CAN_23.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_23_H94            CE_CAN_23.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_23_W94            CE_CAN_23.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_23_CS95           CE_CAN_23.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_23_ID95           CE_CAN_23.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_23_B95            CE_CAN_23.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_23_H95            CE_CAN_23.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_23_W95            CE_CAN_23.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_23_CS96           CE_CAN_23.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_23_ID96           CE_CAN_23.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_23_B96            CE_CAN_23.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_23_H96            CE_CAN_23.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_23_W96            CE_CAN_23.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_23_CS97           CE_CAN_23.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_23_ID97           CE_CAN_23.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_23_B97            CE_CAN_23.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_23_H97            CE_CAN_23.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_23_W97            CE_CAN_23.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_23_CS98           CE_CAN_23.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_23_ID98           CE_CAN_23.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_23_B98            CE_CAN_23.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_23_H98            CE_CAN_23.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_23_W98            CE_CAN_23.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_23_CS99           CE_CAN_23.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_23_ID99           CE_CAN_23.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_23_B99            CE_CAN_23.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_23_H99            CE_CAN_23.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_23_W99            CE_CAN_23.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_23_CS100          CE_CAN_23.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_23_ID100          CE_CAN_23.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_23_B100           CE_CAN_23.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_23_H100           CE_CAN_23.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_23_W100           CE_CAN_23.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_23_CS101          CE_CAN_23.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_23_ID101          CE_CAN_23.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_23_B101           CE_CAN_23.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_23_H101           CE_CAN_23.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_23_W101           CE_CAN_23.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_23_CS102          CE_CAN_23.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_23_ID102          CE_CAN_23.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_23_B102           CE_CAN_23.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_23_H102           CE_CAN_23.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_23_W102           CE_CAN_23.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_23_CS103          CE_CAN_23.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_23_ID103          CE_CAN_23.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_23_B103           CE_CAN_23.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_23_H103           CE_CAN_23.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_23_W103           CE_CAN_23.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_23_CS104          CE_CAN_23.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_23_ID104          CE_CAN_23.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_23_B104           CE_CAN_23.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_23_H104           CE_CAN_23.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_23_W104           CE_CAN_23.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_23_CS105          CE_CAN_23.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_23_ID105          CE_CAN_23.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_23_B105           CE_CAN_23.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_23_H105           CE_CAN_23.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_23_W105           CE_CAN_23.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_23_CS106          CE_CAN_23.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_23_ID106          CE_CAN_23.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_23_B106           CE_CAN_23.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_23_H106           CE_CAN_23.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_23_W106           CE_CAN_23.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_23_CS107          CE_CAN_23.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_23_ID107          CE_CAN_23.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_23_B107           CE_CAN_23.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_23_H107           CE_CAN_23.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_23_W107           CE_CAN_23.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_23_CS108          CE_CAN_23.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_23_ID108          CE_CAN_23.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_23_B108           CE_CAN_23.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_23_H108           CE_CAN_23.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_23_W108           CE_CAN_23.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_23_CS109          CE_CAN_23.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_23_ID109          CE_CAN_23.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_23_B109           CE_CAN_23.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_23_H109           CE_CAN_23.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_23_W109           CE_CAN_23.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_23_CS110          CE_CAN_23.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_23_ID110          CE_CAN_23.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_23_B110           CE_CAN_23.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_23_H110           CE_CAN_23.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_23_W110           CE_CAN_23.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_23_CS111          CE_CAN_23.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_23_ID111          CE_CAN_23.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_23_B111           CE_CAN_23.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_23_H111           CE_CAN_23.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_23_W111           CE_CAN_23.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_23_CS112          CE_CAN_23.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_23_ID112          CE_CAN_23.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_23_B112           CE_CAN_23.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_23_H112           CE_CAN_23.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_23_W112           CE_CAN_23.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_23_CS113          CE_CAN_23.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_23_ID113          CE_CAN_23.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_23_B113           CE_CAN_23.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_23_H113           CE_CAN_23.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_23_W113           CE_CAN_23.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_23_CS114          CE_CAN_23.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_23_ID114          CE_CAN_23.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_23_B114           CE_CAN_23.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_23_H114           CE_CAN_23.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_23_W114           CE_CAN_23.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_23_CS115          CE_CAN_23.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_23_ID115          CE_CAN_23.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_23_B115           CE_CAN_23.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_23_H115           CE_CAN_23.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_23_W115           CE_CAN_23.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_23_CS116          CE_CAN_23.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_23_ID116          CE_CAN_23.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_23_B116           CE_CAN_23.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_23_H116           CE_CAN_23.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_23_W116           CE_CAN_23.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_23_CS117          CE_CAN_23.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_23_ID117          CE_CAN_23.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_23_B117           CE_CAN_23.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_23_H117           CE_CAN_23.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_23_W117           CE_CAN_23.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_23_CS118          CE_CAN_23.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_23_ID118          CE_CAN_23.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_23_B118           CE_CAN_23.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_23_H118           CE_CAN_23.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_23_W118           CE_CAN_23.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_23_CS119          CE_CAN_23.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_23_ID119          CE_CAN_23.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_23_B119           CE_CAN_23.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_23_H119           CE_CAN_23.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_23_W119           CE_CAN_23.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_23_CS120          CE_CAN_23.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_23_ID120          CE_CAN_23.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_23_B120           CE_CAN_23.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_23_H120           CE_CAN_23.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_23_W120           CE_CAN_23.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_23_CS121          CE_CAN_23.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_23_ID121          CE_CAN_23.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_23_B121           CE_CAN_23.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_23_H121           CE_CAN_23.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_23_W121           CE_CAN_23.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_23_CS122          CE_CAN_23.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_23_ID122          CE_CAN_23.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_23_B122           CE_CAN_23.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_23_H122           CE_CAN_23.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_23_W122           CE_CAN_23.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_23_CS123          CE_CAN_23.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_23_ID123          CE_CAN_23.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_23_B123           CE_CAN_23.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_23_H123           CE_CAN_23.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_23_W123           CE_CAN_23.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_23_CS124          CE_CAN_23.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_23_ID124          CE_CAN_23.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_23_B124           CE_CAN_23.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_23_H124           CE_CAN_23.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_23_W124           CE_CAN_23.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_23_CS125          CE_CAN_23.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_23_ID125          CE_CAN_23.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_23_B125           CE_CAN_23.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_23_H125           CE_CAN_23.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_23_W125           CE_CAN_23.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_23_CS126          CE_CAN_23.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_23_ID126          CE_CAN_23.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_23_B126           CE_CAN_23.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_23_H126           CE_CAN_23.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_23_W126           CE_CAN_23.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_23_CS127          CE_CAN_23.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_23_ID127          CE_CAN_23.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_23_B127           CE_CAN_23.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_23_H127           CE_CAN_23.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_23_W127           CE_CAN_23.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_23_RXIMR0     CE_CAN_23.RXIMR[0].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR1     CE_CAN_23.RXIMR[1].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR2     CE_CAN_23.RXIMR[2].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR3     CE_CAN_23.RXIMR[3].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR4     CE_CAN_23.RXIMR[4].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR5     CE_CAN_23.RXIMR[5].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR6     CE_CAN_23.RXIMR[6].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR7     CE_CAN_23.RXIMR[7].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR8     CE_CAN_23.RXIMR[8].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR9     CE_CAN_23.RXIMR[9].R          /* Receive Individual Mask */
#define CE_CAN_23_RXIMR10    CE_CAN_23.RXIMR[10].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR11    CE_CAN_23.RXIMR[11].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR12    CE_CAN_23.RXIMR[12].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR13    CE_CAN_23.RXIMR[13].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR14    CE_CAN_23.RXIMR[14].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR15    CE_CAN_23.RXIMR[15].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR16    CE_CAN_23.RXIMR[16].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR17    CE_CAN_23.RXIMR[17].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR18    CE_CAN_23.RXIMR[18].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR19    CE_CAN_23.RXIMR[19].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR20    CE_CAN_23.RXIMR[20].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR21    CE_CAN_23.RXIMR[21].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR22    CE_CAN_23.RXIMR[22].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR23    CE_CAN_23.RXIMR[23].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR24    CE_CAN_23.RXIMR[24].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR25    CE_CAN_23.RXIMR[25].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR26    CE_CAN_23.RXIMR[26].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR27    CE_CAN_23.RXIMR[27].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR28    CE_CAN_23.RXIMR[28].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR29    CE_CAN_23.RXIMR[29].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR30    CE_CAN_23.RXIMR[30].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR31    CE_CAN_23.RXIMR[31].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR32    CE_CAN_23.RXIMR[32].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR33    CE_CAN_23.RXIMR[33].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR34    CE_CAN_23.RXIMR[34].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR35    CE_CAN_23.RXIMR[35].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR36    CE_CAN_23.RXIMR[36].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR37    CE_CAN_23.RXIMR[37].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR38    CE_CAN_23.RXIMR[38].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR39    CE_CAN_23.RXIMR[39].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR40    CE_CAN_23.RXIMR[40].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR41    CE_CAN_23.RXIMR[41].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR42    CE_CAN_23.RXIMR[42].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR43    CE_CAN_23.RXIMR[43].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR44    CE_CAN_23.RXIMR[44].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR45    CE_CAN_23.RXIMR[45].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR46    CE_CAN_23.RXIMR[46].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR47    CE_CAN_23.RXIMR[47].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR48    CE_CAN_23.RXIMR[48].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR49    CE_CAN_23.RXIMR[49].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR50    CE_CAN_23.RXIMR[50].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR51    CE_CAN_23.RXIMR[51].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR52    CE_CAN_23.RXIMR[52].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR53    CE_CAN_23.RXIMR[53].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR54    CE_CAN_23.RXIMR[54].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR55    CE_CAN_23.RXIMR[55].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR56    CE_CAN_23.RXIMR[56].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR57    CE_CAN_23.RXIMR[57].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR58    CE_CAN_23.RXIMR[58].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR59    CE_CAN_23.RXIMR[59].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR60    CE_CAN_23.RXIMR[60].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR61    CE_CAN_23.RXIMR[61].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR62    CE_CAN_23.RXIMR[62].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR63    CE_CAN_23.RXIMR[63].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR64    CE_CAN_23.RXIMR[64].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR65    CE_CAN_23.RXIMR[65].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR66    CE_CAN_23.RXIMR[66].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR67    CE_CAN_23.RXIMR[67].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR68    CE_CAN_23.RXIMR[68].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR69    CE_CAN_23.RXIMR[69].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR70    CE_CAN_23.RXIMR[70].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR71    CE_CAN_23.RXIMR[71].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR72    CE_CAN_23.RXIMR[72].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR73    CE_CAN_23.RXIMR[73].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR74    CE_CAN_23.RXIMR[74].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR75    CE_CAN_23.RXIMR[75].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR76    CE_CAN_23.RXIMR[76].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR77    CE_CAN_23.RXIMR[77].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR78    CE_CAN_23.RXIMR[78].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR79    CE_CAN_23.RXIMR[79].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR80    CE_CAN_23.RXIMR[80].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR81    CE_CAN_23.RXIMR[81].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR82    CE_CAN_23.RXIMR[82].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR83    CE_CAN_23.RXIMR[83].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR84    CE_CAN_23.RXIMR[84].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR85    CE_CAN_23.RXIMR[85].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR86    CE_CAN_23.RXIMR[86].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR87    CE_CAN_23.RXIMR[87].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR88    CE_CAN_23.RXIMR[88].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR89    CE_CAN_23.RXIMR[89].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR90    CE_CAN_23.RXIMR[90].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR91    CE_CAN_23.RXIMR[91].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR92    CE_CAN_23.RXIMR[92].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR93    CE_CAN_23.RXIMR[93].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR94    CE_CAN_23.RXIMR[94].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR95    CE_CAN_23.RXIMR[95].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR96    CE_CAN_23.RXIMR[96].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR97    CE_CAN_23.RXIMR[97].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR98    CE_CAN_23.RXIMR[98].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR99    CE_CAN_23.RXIMR[99].R         /* Receive Individual Mask */
#define CE_CAN_23_RXIMR100   CE_CAN_23.RXIMR[100].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR101   CE_CAN_23.RXIMR[101].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR102   CE_CAN_23.RXIMR[102].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR103   CE_CAN_23.RXIMR[103].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR104   CE_CAN_23.RXIMR[104].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR105   CE_CAN_23.RXIMR[105].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR106   CE_CAN_23.RXIMR[106].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR107   CE_CAN_23.RXIMR[107].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR108   CE_CAN_23.RXIMR[108].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR109   CE_CAN_23.RXIMR[109].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR110   CE_CAN_23.RXIMR[110].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR111   CE_CAN_23.RXIMR[111].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR112   CE_CAN_23.RXIMR[112].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR113   CE_CAN_23.RXIMR[113].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR114   CE_CAN_23.RXIMR[114].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR115   CE_CAN_23.RXIMR[115].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR116   CE_CAN_23.RXIMR[116].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR117   CE_CAN_23.RXIMR[117].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR118   CE_CAN_23.RXIMR[118].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR119   CE_CAN_23.RXIMR[119].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR120   CE_CAN_23.RXIMR[120].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR121   CE_CAN_23.RXIMR[121].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR122   CE_CAN_23.RXIMR[122].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR123   CE_CAN_23.RXIMR[123].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR124   CE_CAN_23.RXIMR[124].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR125   CE_CAN_23.RXIMR[125].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR126   CE_CAN_23.RXIMR[126].R        /* Receive Individual Mask */
#define CE_CAN_23_RXIMR127   CE_CAN_23.RXIMR[127].R        /* Receive Individual Mask */
#define CE_CAN_23_MECR       CE_CAN_23.MECR.R              /* Memory Error Control */
#define CE_CAN_23_ERRIAR     CE_CAN_23.ERRIAR.R            /* Error Injection Address */
#define CE_CAN_23_ERRIDPR    CE_CAN_23.ERRIDPR.R           /* Error Injection Data Pattern */
#define CE_CAN_23_ERRIPPR    CE_CAN_23.ERRIPPR.R           /* Error Injection Parity Pattern */
#define CE_CAN_23_RERRAR     CE_CAN_23.RERRAR.R            /* Error Report Address */
#define CE_CAN_23_RERRDR     CE_CAN_23.RERRDR.R            /* Error Report Data */
#define CE_CAN_23_RERRSYNR   CE_CAN_23.RERRSYNR.R          /* Error Report Syndrome */
#define CE_CAN_23_ERRSR      CE_CAN_23.ERRSR.R             /* Error Status */
#define CE_CAN_23_EPRS       CE_CAN_23.EPRS.R              /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_23_ENCBT      CE_CAN_23.ENCBT.R             /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_23_EDCBT      CE_CAN_23.EDCBT.R             /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_23_ETDC       CE_CAN_23.ETDC.R              /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_23_FDCTRL     CE_CAN_23.FDCTRL.R            /* CAN FD Control */
#define CE_CAN_23_FDCBT      CE_CAN_23.FDCBT.R             /* CAN FD Bit Timing */
#define CE_CAN_23_FDCRC      CE_CAN_23.FDCRC.R             /* CAN FD CRC */
#define CE_CAN_23_ERFCR      CE_CAN_23.ERFCR.R             /* Enhanced RX FIFO Control */
#define CE_CAN_23_ERFIER     CE_CAN_23.ERFIER.R            /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_23_ERFSR      CE_CAN_23.ERFSR.R             /* Enhanced RX FIFO Status */
#define CE_CAN_23_HR_TIME_STAMP0 CE_CAN_23.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP1 CE_CAN_23.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP2 CE_CAN_23.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP3 CE_CAN_23.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP4 CE_CAN_23.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP5 CE_CAN_23.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP6 CE_CAN_23.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP7 CE_CAN_23.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP8 CE_CAN_23.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP9 CE_CAN_23.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP10 CE_CAN_23.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP11 CE_CAN_23.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP12 CE_CAN_23.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP13 CE_CAN_23.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP14 CE_CAN_23.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP15 CE_CAN_23.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP16 CE_CAN_23.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP17 CE_CAN_23.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP18 CE_CAN_23.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP19 CE_CAN_23.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP20 CE_CAN_23.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP21 CE_CAN_23.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP22 CE_CAN_23.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP23 CE_CAN_23.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP24 CE_CAN_23.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP25 CE_CAN_23.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP26 CE_CAN_23.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP27 CE_CAN_23.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP28 CE_CAN_23.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP29 CE_CAN_23.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP30 CE_CAN_23.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP31 CE_CAN_23.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP32 CE_CAN_23.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP33 CE_CAN_23.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP34 CE_CAN_23.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP35 CE_CAN_23.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP36 CE_CAN_23.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP37 CE_CAN_23.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP38 CE_CAN_23.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP39 CE_CAN_23.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP40 CE_CAN_23.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP41 CE_CAN_23.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP42 CE_CAN_23.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP43 CE_CAN_23.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP44 CE_CAN_23.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP45 CE_CAN_23.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP46 CE_CAN_23.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP47 CE_CAN_23.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP48 CE_CAN_23.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP49 CE_CAN_23.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP50 CE_CAN_23.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP51 CE_CAN_23.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP52 CE_CAN_23.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP53 CE_CAN_23.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP54 CE_CAN_23.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP55 CE_CAN_23.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP56 CE_CAN_23.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP57 CE_CAN_23.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP58 CE_CAN_23.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP59 CE_CAN_23.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP60 CE_CAN_23.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP61 CE_CAN_23.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP62 CE_CAN_23.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP63 CE_CAN_23.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP64 CE_CAN_23.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP65 CE_CAN_23.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP66 CE_CAN_23.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP67 CE_CAN_23.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP68 CE_CAN_23.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP69 CE_CAN_23.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP70 CE_CAN_23.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP71 CE_CAN_23.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP72 CE_CAN_23.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP73 CE_CAN_23.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP74 CE_CAN_23.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP75 CE_CAN_23.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP76 CE_CAN_23.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP77 CE_CAN_23.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP78 CE_CAN_23.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP79 CE_CAN_23.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP80 CE_CAN_23.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP81 CE_CAN_23.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP82 CE_CAN_23.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP83 CE_CAN_23.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP84 CE_CAN_23.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP85 CE_CAN_23.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP86 CE_CAN_23.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP87 CE_CAN_23.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP88 CE_CAN_23.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP89 CE_CAN_23.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP90 CE_CAN_23.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP91 CE_CAN_23.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP92 CE_CAN_23.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP93 CE_CAN_23.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP94 CE_CAN_23.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP95 CE_CAN_23.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP96 CE_CAN_23.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP97 CE_CAN_23.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP98 CE_CAN_23.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP99 CE_CAN_23.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP100 CE_CAN_23.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP101 CE_CAN_23.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP102 CE_CAN_23.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP103 CE_CAN_23.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP104 CE_CAN_23.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP105 CE_CAN_23.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP106 CE_CAN_23.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP107 CE_CAN_23.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP108 CE_CAN_23.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP109 CE_CAN_23.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP110 CE_CAN_23.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP111 CE_CAN_23.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP112 CE_CAN_23.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP113 CE_CAN_23.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP114 CE_CAN_23.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP115 CE_CAN_23.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP116 CE_CAN_23.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP117 CE_CAN_23.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP118 CE_CAN_23.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP119 CE_CAN_23.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP120 CE_CAN_23.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP121 CE_CAN_23.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP122 CE_CAN_23.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP123 CE_CAN_23.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP124 CE_CAN_23.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP125 CE_CAN_23.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP126 CE_CAN_23.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_23_HR_TIME_STAMP127 CE_CAN_23.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_23_ERFFEL0    CE_CAN_23.ERFFEL[0].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL1    CE_CAN_23.ERFFEL[1].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL2    CE_CAN_23.ERFFEL[2].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL3    CE_CAN_23.ERFFEL[3].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL4    CE_CAN_23.ERFFEL[4].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL5    CE_CAN_23.ERFFEL[5].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL6    CE_CAN_23.ERFFEL[6].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL7    CE_CAN_23.ERFFEL[7].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL8    CE_CAN_23.ERFFEL[8].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL9    CE_CAN_23.ERFFEL[9].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL10   CE_CAN_23.ERFFEL[10].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL11   CE_CAN_23.ERFFEL[11].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL12   CE_CAN_23.ERFFEL[12].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL13   CE_CAN_23.ERFFEL[13].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL14   CE_CAN_23.ERFFEL[14].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL15   CE_CAN_23.ERFFEL[15].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL16   CE_CAN_23.ERFFEL[16].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL17   CE_CAN_23.ERFFEL[17].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL18   CE_CAN_23.ERFFEL[18].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL19   CE_CAN_23.ERFFEL[19].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL20   CE_CAN_23.ERFFEL[20].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL21   CE_CAN_23.ERFFEL[21].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL22   CE_CAN_23.ERFFEL[22].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL23   CE_CAN_23.ERFFEL[23].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL24   CE_CAN_23.ERFFEL[24].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL25   CE_CAN_23.ERFFEL[25].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL26   CE_CAN_23.ERFFEL[26].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL27   CE_CAN_23.ERFFEL[27].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL28   CE_CAN_23.ERFFEL[28].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL29   CE_CAN_23.ERFFEL[29].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL30   CE_CAN_23.ERFFEL[30].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL31   CE_CAN_23.ERFFEL[31].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL32   CE_CAN_23.ERFFEL[32].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL33   CE_CAN_23.ERFFEL[33].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL34   CE_CAN_23.ERFFEL[34].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL35   CE_CAN_23.ERFFEL[35].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL36   CE_CAN_23.ERFFEL[36].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL37   CE_CAN_23.ERFFEL[37].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL38   CE_CAN_23.ERFFEL[38].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL39   CE_CAN_23.ERFFEL[39].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL40   CE_CAN_23.ERFFEL[40].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL41   CE_CAN_23.ERFFEL[41].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL42   CE_CAN_23.ERFFEL[42].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL43   CE_CAN_23.ERFFEL[43].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL44   CE_CAN_23.ERFFEL[44].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL45   CE_CAN_23.ERFFEL[45].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL46   CE_CAN_23.ERFFEL[46].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL47   CE_CAN_23.ERFFEL[47].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL48   CE_CAN_23.ERFFEL[48].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL49   CE_CAN_23.ERFFEL[49].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL50   CE_CAN_23.ERFFEL[50].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL51   CE_CAN_23.ERFFEL[51].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL52   CE_CAN_23.ERFFEL[52].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL53   CE_CAN_23.ERFFEL[53].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL54   CE_CAN_23.ERFFEL[54].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL55   CE_CAN_23.ERFFEL[55].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL56   CE_CAN_23.ERFFEL[56].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL57   CE_CAN_23.ERFFEL[57].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL58   CE_CAN_23.ERFFEL[58].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL59   CE_CAN_23.ERFFEL[59].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL60   CE_CAN_23.ERFFEL[60].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL61   CE_CAN_23.ERFFEL[61].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL62   CE_CAN_23.ERFFEL[62].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL63   CE_CAN_23.ERFFEL[63].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL64   CE_CAN_23.ERFFEL[64].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL65   CE_CAN_23.ERFFEL[65].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL66   CE_CAN_23.ERFFEL[66].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL67   CE_CAN_23.ERFFEL[67].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL68   CE_CAN_23.ERFFEL[68].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL69   CE_CAN_23.ERFFEL[69].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL70   CE_CAN_23.ERFFEL[70].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL71   CE_CAN_23.ERFFEL[71].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL72   CE_CAN_23.ERFFEL[72].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL73   CE_CAN_23.ERFFEL[73].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL74   CE_CAN_23.ERFFEL[74].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL75   CE_CAN_23.ERFFEL[75].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL76   CE_CAN_23.ERFFEL[76].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL77   CE_CAN_23.ERFFEL[77].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL78   CE_CAN_23.ERFFEL[78].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL79   CE_CAN_23.ERFFEL[79].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL80   CE_CAN_23.ERFFEL[80].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL81   CE_CAN_23.ERFFEL[81].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL82   CE_CAN_23.ERFFEL[82].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL83   CE_CAN_23.ERFFEL[83].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL84   CE_CAN_23.ERFFEL[84].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL85   CE_CAN_23.ERFFEL[85].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL86   CE_CAN_23.ERFFEL[86].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL87   CE_CAN_23.ERFFEL[87].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL88   CE_CAN_23.ERFFEL[88].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL89   CE_CAN_23.ERFFEL[89].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL90   CE_CAN_23.ERFFEL[90].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL91   CE_CAN_23.ERFFEL[91].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL92   CE_CAN_23.ERFFEL[92].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL93   CE_CAN_23.ERFFEL[93].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL94   CE_CAN_23.ERFFEL[94].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL95   CE_CAN_23.ERFFEL[95].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL96   CE_CAN_23.ERFFEL[96].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL97   CE_CAN_23.ERFFEL[97].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL98   CE_CAN_23.ERFFEL[98].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL99   CE_CAN_23.ERFFEL[99].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL100  CE_CAN_23.ERFFEL[100].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL101  CE_CAN_23.ERFFEL[101].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL102  CE_CAN_23.ERFFEL[102].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL103  CE_CAN_23.ERFFEL[103].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL104  CE_CAN_23.ERFFEL[104].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL105  CE_CAN_23.ERFFEL[105].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL106  CE_CAN_23.ERFFEL[106].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL107  CE_CAN_23.ERFFEL[107].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL108  CE_CAN_23.ERFFEL[108].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL109  CE_CAN_23.ERFFEL[109].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL110  CE_CAN_23.ERFFEL[110].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL111  CE_CAN_23.ERFFEL[111].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL112  CE_CAN_23.ERFFEL[112].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL113  CE_CAN_23.ERFFEL[113].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL114  CE_CAN_23.ERFFEL[114].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL115  CE_CAN_23.ERFFEL[115].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL116  CE_CAN_23.ERFFEL[116].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL117  CE_CAN_23.ERFFEL[117].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL118  CE_CAN_23.ERFFEL[118].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL119  CE_CAN_23.ERFFEL[119].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL120  CE_CAN_23.ERFFEL[120].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL121  CE_CAN_23.ERFFEL[121].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL122  CE_CAN_23.ERFFEL[122].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL123  CE_CAN_23.ERFFEL[123].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL124  CE_CAN_23.ERFFEL[124].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL125  CE_CAN_23.ERFFEL[125].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL126  CE_CAN_23.ERFFEL[126].R       /* Enhanced RX FIFO Filter Element */
#define CE_CAN_23_ERFFEL127  CE_CAN_23.ERFFEL[127].R       /* Enhanced RX FIFO Filter Element */

#define CE_CAN_3_MCR         CE_CAN_3.MCR.R                /* Module Configuration */
#define CE_CAN_3_CTRL1       CE_CAN_3.CTRL1.R              /* Control 1 */
#define CE_CAN_3_TIMER       CE_CAN_3.TIMER.R              /* Free-Running Timer */
#define CE_CAN_3_RXMGMASK    CE_CAN_3.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_3_RX14MASK    CE_CAN_3.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_3_RX15MASK    CE_CAN_3.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_3_ECR         CE_CAN_3.ECR.R                /* Error Counter */
#define CE_CAN_3_ESR1        CE_CAN_3.ESR1.R               /* Error and Status 1 */
#define CE_CAN_3_IMASK2      CE_CAN_3.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_3_IMASK1      CE_CAN_3.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_3_IFLAG2      CE_CAN_3.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_3_IFLAG1      CE_CAN_3.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_3_CTRL2       CE_CAN_3.CTRL2.R              /* Control 2 */
#define CE_CAN_3_ESR2        CE_CAN_3.ESR2.R               /* Error and Status 2 */
#define CE_CAN_3_CRCR        CE_CAN_3.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_3_RXFGMASK    CE_CAN_3.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_3_RXFIR       CE_CAN_3.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_3_CBT         CE_CAN_3.CBT.R                /* CAN Bit Timing */
#define CE_CAN_3_IMASK4      CE_CAN_3.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_3_IMASK3      CE_CAN_3.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_3_IFLAG4      CE_CAN_3.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_3_IFLAG3      CE_CAN_3.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_3_CS0            CE_CAN_3.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_3_ID0            CE_CAN_3.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_3_B0             CE_CAN_3.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_3_H0             CE_CAN_3.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_3_W0             CE_CAN_3.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_3_CS1            CE_CAN_3.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_3_ID1            CE_CAN_3.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_3_B1             CE_CAN_3.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_3_H1             CE_CAN_3.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_3_W1             CE_CAN_3.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_3_CS2            CE_CAN_3.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_3_ID2            CE_CAN_3.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_3_B2             CE_CAN_3.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_3_H2             CE_CAN_3.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_3_W2             CE_CAN_3.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_3_CS3            CE_CAN_3.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_3_ID3            CE_CAN_3.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_3_B3             CE_CAN_3.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_3_H3             CE_CAN_3.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_3_W3             CE_CAN_3.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_3_CS4            CE_CAN_3.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_3_ID4            CE_CAN_3.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_3_B4             CE_CAN_3.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_3_H4             CE_CAN_3.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_3_W4             CE_CAN_3.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_3_CS5            CE_CAN_3.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_3_ID5            CE_CAN_3.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_3_B5             CE_CAN_3.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_3_H5             CE_CAN_3.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_3_W5             CE_CAN_3.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_3_CS6            CE_CAN_3.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_3_ID6            CE_CAN_3.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_3_B6             CE_CAN_3.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_3_H6             CE_CAN_3.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_3_W6             CE_CAN_3.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_3_CS7            CE_CAN_3.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_3_ID7            CE_CAN_3.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_3_B7             CE_CAN_3.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_3_H7             CE_CAN_3.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_3_W7             CE_CAN_3.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_3_CS8            CE_CAN_3.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_3_ID8            CE_CAN_3.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_3_B8             CE_CAN_3.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_3_H8             CE_CAN_3.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_3_W8             CE_CAN_3.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_3_CS9            CE_CAN_3.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_3_ID9            CE_CAN_3.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_3_B9             CE_CAN_3.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_3_H9             CE_CAN_3.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_3_W9             CE_CAN_3.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_3_CS10           CE_CAN_3.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_3_ID10           CE_CAN_3.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_3_B10            CE_CAN_3.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_3_H10            CE_CAN_3.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_3_W10            CE_CAN_3.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_3_CS11           CE_CAN_3.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_3_ID11           CE_CAN_3.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_3_B11            CE_CAN_3.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_3_H11            CE_CAN_3.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_3_W11            CE_CAN_3.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_3_CS12           CE_CAN_3.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_3_ID12           CE_CAN_3.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_3_B12            CE_CAN_3.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_3_H12            CE_CAN_3.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_3_W12            CE_CAN_3.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_3_CS13           CE_CAN_3.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_3_ID13           CE_CAN_3.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_3_B13            CE_CAN_3.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_3_H13            CE_CAN_3.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_3_W13            CE_CAN_3.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_3_CS14           CE_CAN_3.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_3_ID14           CE_CAN_3.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_3_B14            CE_CAN_3.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_3_H14            CE_CAN_3.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_3_W14            CE_CAN_3.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_3_CS15           CE_CAN_3.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_3_ID15           CE_CAN_3.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_3_B15            CE_CAN_3.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_3_H15            CE_CAN_3.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_3_W15            CE_CAN_3.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_3_CS16           CE_CAN_3.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_3_ID16           CE_CAN_3.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_3_B16            CE_CAN_3.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_3_H16            CE_CAN_3.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_3_W16            CE_CAN_3.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_3_CS17           CE_CAN_3.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_3_ID17           CE_CAN_3.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_3_B17            CE_CAN_3.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_3_H17            CE_CAN_3.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_3_W17            CE_CAN_3.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_3_CS18           CE_CAN_3.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_3_ID18           CE_CAN_3.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_3_B18            CE_CAN_3.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_3_H18            CE_CAN_3.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_3_W18            CE_CAN_3.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_3_CS19           CE_CAN_3.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_3_ID19           CE_CAN_3.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_3_B19            CE_CAN_3.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_3_H19            CE_CAN_3.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_3_W19            CE_CAN_3.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_3_CS20           CE_CAN_3.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_3_ID20           CE_CAN_3.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_3_B20            CE_CAN_3.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_3_H20            CE_CAN_3.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_3_W20            CE_CAN_3.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_3_CS21           CE_CAN_3.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_3_ID21           CE_CAN_3.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_3_B21            CE_CAN_3.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_3_H21            CE_CAN_3.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_3_W21            CE_CAN_3.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_3_CS22           CE_CAN_3.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_3_ID22           CE_CAN_3.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_3_B22            CE_CAN_3.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_3_H22            CE_CAN_3.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_3_W22            CE_CAN_3.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_3_CS23           CE_CAN_3.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_3_ID23           CE_CAN_3.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_3_B23            CE_CAN_3.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_3_H23            CE_CAN_3.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_3_W23            CE_CAN_3.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_3_CS24           CE_CAN_3.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_3_ID24           CE_CAN_3.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_3_B24            CE_CAN_3.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_3_H24            CE_CAN_3.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_3_W24            CE_CAN_3.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_3_CS25           CE_CAN_3.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_3_ID25           CE_CAN_3.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_3_B25            CE_CAN_3.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_3_H25            CE_CAN_3.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_3_W25            CE_CAN_3.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_3_CS26           CE_CAN_3.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_3_ID26           CE_CAN_3.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_3_B26            CE_CAN_3.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_3_H26            CE_CAN_3.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_3_W26            CE_CAN_3.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_3_CS27           CE_CAN_3.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_3_ID27           CE_CAN_3.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_3_B27            CE_CAN_3.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_3_H27            CE_CAN_3.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_3_W27            CE_CAN_3.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_3_CS28           CE_CAN_3.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_3_ID28           CE_CAN_3.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_3_B28            CE_CAN_3.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_3_H28            CE_CAN_3.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_3_W28            CE_CAN_3.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_3_CS29           CE_CAN_3.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_3_ID29           CE_CAN_3.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_3_B29            CE_CAN_3.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_3_H29            CE_CAN_3.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_3_W29            CE_CAN_3.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_3_CS30           CE_CAN_3.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_3_ID30           CE_CAN_3.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_3_B30            CE_CAN_3.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_3_H30            CE_CAN_3.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_3_W30            CE_CAN_3.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_3_CS31           CE_CAN_3.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_3_ID31           CE_CAN_3.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_3_B31            CE_CAN_3.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_3_H31            CE_CAN_3.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_3_W31            CE_CAN_3.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_3_CS32           CE_CAN_3.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_3_ID32           CE_CAN_3.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_3_B32            CE_CAN_3.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_3_H32            CE_CAN_3.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_3_W32            CE_CAN_3.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_3_CS33           CE_CAN_3.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_3_ID33           CE_CAN_3.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_3_B33            CE_CAN_3.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_3_H33            CE_CAN_3.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_3_W33            CE_CAN_3.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_3_CS34           CE_CAN_3.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_3_ID34           CE_CAN_3.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_3_B34            CE_CAN_3.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_3_H34            CE_CAN_3.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_3_W34            CE_CAN_3.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_3_CS35           CE_CAN_3.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_3_ID35           CE_CAN_3.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_3_B35            CE_CAN_3.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_3_H35            CE_CAN_3.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_3_W35            CE_CAN_3.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_3_CS36           CE_CAN_3.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_3_ID36           CE_CAN_3.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_3_B36            CE_CAN_3.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_3_H36            CE_CAN_3.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_3_W36            CE_CAN_3.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_3_CS37           CE_CAN_3.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_3_ID37           CE_CAN_3.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_3_B37            CE_CAN_3.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_3_H37            CE_CAN_3.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_3_W37            CE_CAN_3.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_3_CS38           CE_CAN_3.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_3_ID38           CE_CAN_3.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_3_B38            CE_CAN_3.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_3_H38            CE_CAN_3.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_3_W38            CE_CAN_3.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_3_CS39           CE_CAN_3.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_3_ID39           CE_CAN_3.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_3_B39            CE_CAN_3.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_3_H39            CE_CAN_3.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_3_W39            CE_CAN_3.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_3_CS40           CE_CAN_3.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_3_ID40           CE_CAN_3.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_3_B40            CE_CAN_3.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_3_H40            CE_CAN_3.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_3_W40            CE_CAN_3.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_3_CS41           CE_CAN_3.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_3_ID41           CE_CAN_3.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_3_B41            CE_CAN_3.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_3_H41            CE_CAN_3.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_3_W41            CE_CAN_3.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_3_CS42           CE_CAN_3.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_3_ID42           CE_CAN_3.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_3_B42            CE_CAN_3.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_3_H42            CE_CAN_3.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_3_W42            CE_CAN_3.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_3_CS43           CE_CAN_3.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_3_ID43           CE_CAN_3.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_3_B43            CE_CAN_3.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_3_H43            CE_CAN_3.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_3_W43            CE_CAN_3.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_3_CS44           CE_CAN_3.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_3_ID44           CE_CAN_3.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_3_B44            CE_CAN_3.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_3_H44            CE_CAN_3.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_3_W44            CE_CAN_3.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_3_CS45           CE_CAN_3.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_3_ID45           CE_CAN_3.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_3_B45            CE_CAN_3.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_3_H45            CE_CAN_3.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_3_W45            CE_CAN_3.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_3_CS46           CE_CAN_3.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_3_ID46           CE_CAN_3.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_3_B46            CE_CAN_3.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_3_H46            CE_CAN_3.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_3_W46            CE_CAN_3.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_3_CS47           CE_CAN_3.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_3_ID47           CE_CAN_3.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_3_B47            CE_CAN_3.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_3_H47            CE_CAN_3.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_3_W47            CE_CAN_3.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_3_CS48           CE_CAN_3.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_3_ID48           CE_CAN_3.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_3_B48            CE_CAN_3.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_3_H48            CE_CAN_3.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_3_W48            CE_CAN_3.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_3_CS49           CE_CAN_3.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_3_ID49           CE_CAN_3.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_3_B49            CE_CAN_3.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_3_H49            CE_CAN_3.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_3_W49            CE_CAN_3.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_3_CS50           CE_CAN_3.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_3_ID50           CE_CAN_3.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_3_B50            CE_CAN_3.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_3_H50            CE_CAN_3.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_3_W50            CE_CAN_3.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_3_CS51           CE_CAN_3.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_3_ID51           CE_CAN_3.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_3_B51            CE_CAN_3.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_3_H51            CE_CAN_3.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_3_W51            CE_CAN_3.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_3_CS52           CE_CAN_3.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_3_ID52           CE_CAN_3.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_3_B52            CE_CAN_3.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_3_H52            CE_CAN_3.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_3_W52            CE_CAN_3.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_3_CS53           CE_CAN_3.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_3_ID53           CE_CAN_3.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_3_B53            CE_CAN_3.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_3_H53            CE_CAN_3.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_3_W53            CE_CAN_3.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_3_CS54           CE_CAN_3.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_3_ID54           CE_CAN_3.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_3_B54            CE_CAN_3.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_3_H54            CE_CAN_3.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_3_W54            CE_CAN_3.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_3_CS55           CE_CAN_3.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_3_ID55           CE_CAN_3.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_3_B55            CE_CAN_3.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_3_H55            CE_CAN_3.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_3_W55            CE_CAN_3.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_3_CS56           CE_CAN_3.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_3_ID56           CE_CAN_3.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_3_B56            CE_CAN_3.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_3_H56            CE_CAN_3.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_3_W56            CE_CAN_3.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_3_CS57           CE_CAN_3.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_3_ID57           CE_CAN_3.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_3_B57            CE_CAN_3.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_3_H57            CE_CAN_3.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_3_W57            CE_CAN_3.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_3_CS58           CE_CAN_3.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_3_ID58           CE_CAN_3.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_3_B58            CE_CAN_3.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_3_H58            CE_CAN_3.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_3_W58            CE_CAN_3.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_3_CS59           CE_CAN_3.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_3_ID59           CE_CAN_3.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_3_B59            CE_CAN_3.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_3_H59            CE_CAN_3.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_3_W59            CE_CAN_3.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_3_CS60           CE_CAN_3.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_3_ID60           CE_CAN_3.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_3_B60            CE_CAN_3.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_3_H60            CE_CAN_3.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_3_W60            CE_CAN_3.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_3_CS61           CE_CAN_3.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_3_ID61           CE_CAN_3.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_3_B61            CE_CAN_3.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_3_H61            CE_CAN_3.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_3_W61            CE_CAN_3.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_3_CS62           CE_CAN_3.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_3_ID62           CE_CAN_3.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_3_B62            CE_CAN_3.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_3_H62            CE_CAN_3.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_3_W62            CE_CAN_3.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_3_CS63           CE_CAN_3.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_3_ID63           CE_CAN_3.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_3_B63            CE_CAN_3.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_3_H63            CE_CAN_3.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_3_W63            CE_CAN_3.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_3_CS64           CE_CAN_3.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_3_ID64           CE_CAN_3.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_3_B64            CE_CAN_3.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_3_H64            CE_CAN_3.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_3_W64            CE_CAN_3.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_3_CS65           CE_CAN_3.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_3_ID65           CE_CAN_3.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_3_B65            CE_CAN_3.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_3_H65            CE_CAN_3.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_3_W65            CE_CAN_3.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_3_CS66           CE_CAN_3.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_3_ID66           CE_CAN_3.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_3_B66            CE_CAN_3.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_3_H66            CE_CAN_3.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_3_W66            CE_CAN_3.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_3_CS67           CE_CAN_3.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_3_ID67           CE_CAN_3.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_3_B67            CE_CAN_3.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_3_H67            CE_CAN_3.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_3_W67            CE_CAN_3.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_3_CS68           CE_CAN_3.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_3_ID68           CE_CAN_3.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_3_B68            CE_CAN_3.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_3_H68            CE_CAN_3.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_3_W68            CE_CAN_3.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_3_CS69           CE_CAN_3.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_3_ID69           CE_CAN_3.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_3_B69            CE_CAN_3.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_3_H69            CE_CAN_3.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_3_W69            CE_CAN_3.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_3_CS70           CE_CAN_3.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_3_ID70           CE_CAN_3.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_3_B70            CE_CAN_3.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_3_H70            CE_CAN_3.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_3_W70            CE_CAN_3.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_3_CS71           CE_CAN_3.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_3_ID71           CE_CAN_3.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_3_B71            CE_CAN_3.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_3_H71            CE_CAN_3.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_3_W71            CE_CAN_3.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_3_CS72           CE_CAN_3.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_3_ID72           CE_CAN_3.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_3_B72            CE_CAN_3.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_3_H72            CE_CAN_3.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_3_W72            CE_CAN_3.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_3_CS73           CE_CAN_3.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_3_ID73           CE_CAN_3.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_3_B73            CE_CAN_3.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_3_H73            CE_CAN_3.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_3_W73            CE_CAN_3.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_3_CS74           CE_CAN_3.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_3_ID74           CE_CAN_3.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_3_B74            CE_CAN_3.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_3_H74            CE_CAN_3.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_3_W74            CE_CAN_3.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_3_CS75           CE_CAN_3.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_3_ID75           CE_CAN_3.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_3_B75            CE_CAN_3.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_3_H75            CE_CAN_3.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_3_W75            CE_CAN_3.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_3_CS76           CE_CAN_3.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_3_ID76           CE_CAN_3.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_3_B76            CE_CAN_3.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_3_H76            CE_CAN_3.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_3_W76            CE_CAN_3.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_3_CS77           CE_CAN_3.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_3_ID77           CE_CAN_3.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_3_B77            CE_CAN_3.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_3_H77            CE_CAN_3.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_3_W77            CE_CAN_3.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_3_CS78           CE_CAN_3.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_3_ID78           CE_CAN_3.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_3_B78            CE_CAN_3.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_3_H78            CE_CAN_3.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_3_W78            CE_CAN_3.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_3_CS79           CE_CAN_3.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_3_ID79           CE_CAN_3.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_3_B79            CE_CAN_3.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_3_H79            CE_CAN_3.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_3_W79            CE_CAN_3.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_3_CS80           CE_CAN_3.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_3_ID80           CE_CAN_3.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_3_B80            CE_CAN_3.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_3_H80            CE_CAN_3.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_3_W80            CE_CAN_3.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_3_CS81           CE_CAN_3.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_3_ID81           CE_CAN_3.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_3_B81            CE_CAN_3.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_3_H81            CE_CAN_3.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_3_W81            CE_CAN_3.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_3_CS82           CE_CAN_3.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_3_ID82           CE_CAN_3.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_3_B82            CE_CAN_3.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_3_H82            CE_CAN_3.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_3_W82            CE_CAN_3.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_3_CS83           CE_CAN_3.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_3_ID83           CE_CAN_3.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_3_B83            CE_CAN_3.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_3_H83            CE_CAN_3.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_3_W83            CE_CAN_3.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_3_CS84           CE_CAN_3.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_3_ID84           CE_CAN_3.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_3_B84            CE_CAN_3.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_3_H84            CE_CAN_3.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_3_W84            CE_CAN_3.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_3_CS85           CE_CAN_3.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_3_ID85           CE_CAN_3.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_3_B85            CE_CAN_3.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_3_H85            CE_CAN_3.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_3_W85            CE_CAN_3.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_3_CS86           CE_CAN_3.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_3_ID86           CE_CAN_3.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_3_B86            CE_CAN_3.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_3_H86            CE_CAN_3.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_3_W86            CE_CAN_3.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_3_CS87           CE_CAN_3.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_3_ID87           CE_CAN_3.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_3_B87            CE_CAN_3.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_3_H87            CE_CAN_3.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_3_W87            CE_CAN_3.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_3_CS88           CE_CAN_3.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_3_ID88           CE_CAN_3.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_3_B88            CE_CAN_3.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_3_H88            CE_CAN_3.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_3_W848           CE_CAN_3.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_3_CS89           CE_CAN_3.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_3_ID89           CE_CAN_3.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_3_B89            CE_CAN_3.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_3_H89            CE_CAN_3.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_3_W89            CE_CAN_3.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_3_CS90           CE_CAN_3.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_3_ID90           CE_CAN_3.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_3_B90            CE_CAN_3.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_3_H90            CE_CAN_3.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_3_W90            CE_CAN_3.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_3_CS91           CE_CAN_3.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_3_ID91           CE_CAN_3.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_3_B91            CE_CAN_3.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_3_H91            CE_CAN_3.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_3_W91            CE_CAN_3.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_3_CS92           CE_CAN_3.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_3_ID92           CE_CAN_3.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_3_B92            CE_CAN_3.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_3_H92            CE_CAN_3.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_3_W92            CE_CAN_3.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_3_CS93           CE_CAN_3.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_3_ID93           CE_CAN_3.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_3_B93            CE_CAN_3.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_3_H93            CE_CAN_3.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_3_W93            CE_CAN_3.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_3_CS94           CE_CAN_3.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_3_ID94           CE_CAN_3.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_3_B94            CE_CAN_3.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_3_H94            CE_CAN_3.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_3_W94            CE_CAN_3.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_3_CS95           CE_CAN_3.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_3_ID95           CE_CAN_3.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_3_B95            CE_CAN_3.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_3_H95            CE_CAN_3.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_3_W95            CE_CAN_3.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_3_CS96           CE_CAN_3.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_3_ID96           CE_CAN_3.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_3_B96            CE_CAN_3.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_3_H96            CE_CAN_3.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_3_W96            CE_CAN_3.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_3_CS97           CE_CAN_3.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_3_ID97           CE_CAN_3.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_3_B97            CE_CAN_3.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_3_H97            CE_CAN_3.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_3_W97            CE_CAN_3.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_3_CS98           CE_CAN_3.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_3_ID98           CE_CAN_3.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_3_B98            CE_CAN_3.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_3_H98            CE_CAN_3.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_3_W98            CE_CAN_3.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_3_CS99           CE_CAN_3.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_3_ID99           CE_CAN_3.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_3_B99            CE_CAN_3.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_3_H99            CE_CAN_3.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_3_W99            CE_CAN_3.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_3_CS100          CE_CAN_3.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_3_ID100          CE_CAN_3.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_3_B100           CE_CAN_3.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_3_H100           CE_CAN_3.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_3_W100           CE_CAN_3.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_3_CS101          CE_CAN_3.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_3_ID101          CE_CAN_3.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_3_B101           CE_CAN_3.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_3_H101           CE_CAN_3.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_3_W101           CE_CAN_3.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_3_CS102          CE_CAN_3.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_3_ID102          CE_CAN_3.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_3_B102           CE_CAN_3.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_3_H102           CE_CAN_3.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_3_W102           CE_CAN_3.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_3_CS103          CE_CAN_3.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_3_ID103          CE_CAN_3.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_3_B103           CE_CAN_3.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_3_H103           CE_CAN_3.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_3_W103           CE_CAN_3.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_3_CS104          CE_CAN_3.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_3_ID104          CE_CAN_3.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_3_B104           CE_CAN_3.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_3_H104           CE_CAN_3.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_3_W104           CE_CAN_3.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_3_CS105          CE_CAN_3.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_3_ID105          CE_CAN_3.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_3_B105           CE_CAN_3.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_3_H105           CE_CAN_3.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_3_W105           CE_CAN_3.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_3_CS106          CE_CAN_3.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_3_ID106          CE_CAN_3.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_3_B106           CE_CAN_3.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_3_H106           CE_CAN_3.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_3_W106           CE_CAN_3.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_3_CS107          CE_CAN_3.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_3_ID107          CE_CAN_3.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_3_B107           CE_CAN_3.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_3_H107           CE_CAN_3.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_3_W107           CE_CAN_3.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_3_CS108          CE_CAN_3.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_3_ID108          CE_CAN_3.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_3_B108           CE_CAN_3.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_3_H108           CE_CAN_3.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_3_W108           CE_CAN_3.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_3_CS109          CE_CAN_3.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_3_ID109          CE_CAN_3.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_3_B109           CE_CAN_3.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_3_H109           CE_CAN_3.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_3_W109           CE_CAN_3.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_3_CS110          CE_CAN_3.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_3_ID110          CE_CAN_3.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_3_B110           CE_CAN_3.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_3_H110           CE_CAN_3.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_3_W110           CE_CAN_3.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_3_CS111          CE_CAN_3.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_3_ID111          CE_CAN_3.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_3_B111           CE_CAN_3.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_3_H111           CE_CAN_3.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_3_W111           CE_CAN_3.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_3_CS112          CE_CAN_3.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_3_ID112          CE_CAN_3.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_3_B112           CE_CAN_3.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_3_H112           CE_CAN_3.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_3_W112           CE_CAN_3.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_3_CS113          CE_CAN_3.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_3_ID113          CE_CAN_3.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_3_B113           CE_CAN_3.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_3_H113           CE_CAN_3.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_3_W113           CE_CAN_3.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_3_CS114          CE_CAN_3.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_3_ID114          CE_CAN_3.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_3_B114           CE_CAN_3.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_3_H114           CE_CAN_3.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_3_W114           CE_CAN_3.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_3_CS115          CE_CAN_3.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_3_ID115          CE_CAN_3.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_3_B115           CE_CAN_3.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_3_H115           CE_CAN_3.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_3_W115           CE_CAN_3.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_3_CS116          CE_CAN_3.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_3_ID116          CE_CAN_3.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_3_B116           CE_CAN_3.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_3_H116           CE_CAN_3.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_3_W116           CE_CAN_3.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_3_CS117          CE_CAN_3.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_3_ID117          CE_CAN_3.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_3_B117           CE_CAN_3.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_3_H117           CE_CAN_3.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_3_W117           CE_CAN_3.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_3_CS118          CE_CAN_3.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_3_ID118          CE_CAN_3.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_3_B118           CE_CAN_3.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_3_H118           CE_CAN_3.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_3_W118           CE_CAN_3.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_3_CS119          CE_CAN_3.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_3_ID119          CE_CAN_3.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_3_B119           CE_CAN_3.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_3_H119           CE_CAN_3.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_3_W119           CE_CAN_3.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_3_CS120          CE_CAN_3.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_3_ID120          CE_CAN_3.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_3_B120           CE_CAN_3.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_3_H120           CE_CAN_3.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_3_W120           CE_CAN_3.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_3_CS121          CE_CAN_3.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_3_ID121          CE_CAN_3.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_3_B121           CE_CAN_3.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_3_H121           CE_CAN_3.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_3_W121           CE_CAN_3.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_3_CS122          CE_CAN_3.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_3_ID122          CE_CAN_3.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_3_B122           CE_CAN_3.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_3_H122           CE_CAN_3.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_3_W122           CE_CAN_3.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_3_CS123          CE_CAN_3.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_3_ID123          CE_CAN_3.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_3_B123           CE_CAN_3.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_3_H123           CE_CAN_3.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_3_W123           CE_CAN_3.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_3_CS124          CE_CAN_3.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_3_ID124          CE_CAN_3.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_3_B124           CE_CAN_3.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_3_H124           CE_CAN_3.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_3_W124           CE_CAN_3.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_3_CS125          CE_CAN_3.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_3_ID125          CE_CAN_3.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_3_B125           CE_CAN_3.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_3_H125           CE_CAN_3.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_3_W125           CE_CAN_3.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_3_CS126          CE_CAN_3.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_3_ID126          CE_CAN_3.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_3_B126           CE_CAN_3.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_3_H126           CE_CAN_3.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_3_W126           CE_CAN_3.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_3_CS127          CE_CAN_3.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_3_ID127          CE_CAN_3.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_3_B127           CE_CAN_3.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_3_H127           CE_CAN_3.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_3_W127           CE_CAN_3.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_3_RXIMR0      CE_CAN_3.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR1      CE_CAN_3.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR2      CE_CAN_3.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR3      CE_CAN_3.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR4      CE_CAN_3.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR5      CE_CAN_3.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR6      CE_CAN_3.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR7      CE_CAN_3.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR8      CE_CAN_3.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR9      CE_CAN_3.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_3_RXIMR10     CE_CAN_3.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR11     CE_CAN_3.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR12     CE_CAN_3.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR13     CE_CAN_3.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR14     CE_CAN_3.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR15     CE_CAN_3.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR16     CE_CAN_3.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR17     CE_CAN_3.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR18     CE_CAN_3.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR19     CE_CAN_3.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR20     CE_CAN_3.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR21     CE_CAN_3.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR22     CE_CAN_3.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR23     CE_CAN_3.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR24     CE_CAN_3.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR25     CE_CAN_3.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR26     CE_CAN_3.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR27     CE_CAN_3.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR28     CE_CAN_3.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR29     CE_CAN_3.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR30     CE_CAN_3.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR31     CE_CAN_3.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR32     CE_CAN_3.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR33     CE_CAN_3.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR34     CE_CAN_3.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR35     CE_CAN_3.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR36     CE_CAN_3.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR37     CE_CAN_3.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR38     CE_CAN_3.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR39     CE_CAN_3.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR40     CE_CAN_3.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR41     CE_CAN_3.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR42     CE_CAN_3.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR43     CE_CAN_3.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR44     CE_CAN_3.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR45     CE_CAN_3.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR46     CE_CAN_3.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR47     CE_CAN_3.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR48     CE_CAN_3.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR49     CE_CAN_3.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR50     CE_CAN_3.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR51     CE_CAN_3.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR52     CE_CAN_3.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR53     CE_CAN_3.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR54     CE_CAN_3.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR55     CE_CAN_3.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR56     CE_CAN_3.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR57     CE_CAN_3.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR58     CE_CAN_3.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR59     CE_CAN_3.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR60     CE_CAN_3.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR61     CE_CAN_3.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR62     CE_CAN_3.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR63     CE_CAN_3.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR64     CE_CAN_3.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR65     CE_CAN_3.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR66     CE_CAN_3.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR67     CE_CAN_3.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR68     CE_CAN_3.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR69     CE_CAN_3.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR70     CE_CAN_3.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR71     CE_CAN_3.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR72     CE_CAN_3.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR73     CE_CAN_3.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR74     CE_CAN_3.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR75     CE_CAN_3.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR76     CE_CAN_3.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR77     CE_CAN_3.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR78     CE_CAN_3.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR79     CE_CAN_3.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR80     CE_CAN_3.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR81     CE_CAN_3.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR82     CE_CAN_3.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR83     CE_CAN_3.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR84     CE_CAN_3.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR85     CE_CAN_3.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR86     CE_CAN_3.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR87     CE_CAN_3.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR88     CE_CAN_3.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR89     CE_CAN_3.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR90     CE_CAN_3.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR91     CE_CAN_3.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR92     CE_CAN_3.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR93     CE_CAN_3.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR94     CE_CAN_3.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR95     CE_CAN_3.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR96     CE_CAN_3.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR97     CE_CAN_3.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR98     CE_CAN_3.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR99     CE_CAN_3.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_3_RXIMR100    CE_CAN_3.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR101    CE_CAN_3.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR102    CE_CAN_3.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR103    CE_CAN_3.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR104    CE_CAN_3.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR105    CE_CAN_3.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR106    CE_CAN_3.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR107    CE_CAN_3.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR108    CE_CAN_3.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR109    CE_CAN_3.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR110    CE_CAN_3.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR111    CE_CAN_3.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR112    CE_CAN_3.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR113    CE_CAN_3.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR114    CE_CAN_3.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR115    CE_CAN_3.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR116    CE_CAN_3.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR117    CE_CAN_3.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR118    CE_CAN_3.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR119    CE_CAN_3.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR120    CE_CAN_3.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR121    CE_CAN_3.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR122    CE_CAN_3.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR123    CE_CAN_3.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR124    CE_CAN_3.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR125    CE_CAN_3.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR126    CE_CAN_3.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_3_RXIMR127    CE_CAN_3.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_3_MECR        CE_CAN_3.MECR.R               /* Memory Error Control */
#define CE_CAN_3_ERRIAR      CE_CAN_3.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_3_ERRIDPR     CE_CAN_3.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_3_ERRIPPR     CE_CAN_3.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_3_RERRAR      CE_CAN_3.RERRAR.R             /* Error Report Address */
#define CE_CAN_3_RERRDR      CE_CAN_3.RERRDR.R             /* Error Report Data */
#define CE_CAN_3_RERRSYNR    CE_CAN_3.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_3_ERRSR       CE_CAN_3.ERRSR.R              /* Error Status */
#define CE_CAN_3_EPRS        CE_CAN_3.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_3_ENCBT       CE_CAN_3.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_3_EDCBT       CE_CAN_3.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_3_ETDC        CE_CAN_3.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_3_FDCTRL      CE_CAN_3.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_3_FDCBT       CE_CAN_3.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_3_FDCRC       CE_CAN_3.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_3_ERFCR       CE_CAN_3.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_3_ERFIER      CE_CAN_3.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_3_ERFSR       CE_CAN_3.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_3_HR_TIME_STAMP0 CE_CAN_3.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP1 CE_CAN_3.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP2 CE_CAN_3.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP3 CE_CAN_3.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP4 CE_CAN_3.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP5 CE_CAN_3.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP6 CE_CAN_3.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP7 CE_CAN_3.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP8 CE_CAN_3.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP9 CE_CAN_3.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP10 CE_CAN_3.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP11 CE_CAN_3.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP12 CE_CAN_3.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP13 CE_CAN_3.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP14 CE_CAN_3.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP15 CE_CAN_3.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP16 CE_CAN_3.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP17 CE_CAN_3.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP18 CE_CAN_3.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP19 CE_CAN_3.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP20 CE_CAN_3.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP21 CE_CAN_3.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP22 CE_CAN_3.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP23 CE_CAN_3.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP24 CE_CAN_3.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP25 CE_CAN_3.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP26 CE_CAN_3.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP27 CE_CAN_3.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP28 CE_CAN_3.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP29 CE_CAN_3.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP30 CE_CAN_3.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP31 CE_CAN_3.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP32 CE_CAN_3.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP33 CE_CAN_3.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP34 CE_CAN_3.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP35 CE_CAN_3.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP36 CE_CAN_3.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP37 CE_CAN_3.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP38 CE_CAN_3.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP39 CE_CAN_3.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP40 CE_CAN_3.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP41 CE_CAN_3.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP42 CE_CAN_3.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP43 CE_CAN_3.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP44 CE_CAN_3.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP45 CE_CAN_3.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP46 CE_CAN_3.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP47 CE_CAN_3.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP48 CE_CAN_3.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP49 CE_CAN_3.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP50 CE_CAN_3.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP51 CE_CAN_3.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP52 CE_CAN_3.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP53 CE_CAN_3.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP54 CE_CAN_3.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP55 CE_CAN_3.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP56 CE_CAN_3.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP57 CE_CAN_3.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP58 CE_CAN_3.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP59 CE_CAN_3.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP60 CE_CAN_3.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP61 CE_CAN_3.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP62 CE_CAN_3.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP63 CE_CAN_3.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP64 CE_CAN_3.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP65 CE_CAN_3.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP66 CE_CAN_3.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP67 CE_CAN_3.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP68 CE_CAN_3.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP69 CE_CAN_3.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP70 CE_CAN_3.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP71 CE_CAN_3.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP72 CE_CAN_3.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP73 CE_CAN_3.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP74 CE_CAN_3.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP75 CE_CAN_3.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP76 CE_CAN_3.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP77 CE_CAN_3.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP78 CE_CAN_3.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP79 CE_CAN_3.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP80 CE_CAN_3.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP81 CE_CAN_3.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP82 CE_CAN_3.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP83 CE_CAN_3.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP84 CE_CAN_3.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP85 CE_CAN_3.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP86 CE_CAN_3.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP87 CE_CAN_3.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP88 CE_CAN_3.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP89 CE_CAN_3.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP90 CE_CAN_3.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP91 CE_CAN_3.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP92 CE_CAN_3.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP93 CE_CAN_3.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP94 CE_CAN_3.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP95 CE_CAN_3.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP96 CE_CAN_3.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP97 CE_CAN_3.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP98 CE_CAN_3.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP99 CE_CAN_3.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP100 CE_CAN_3.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP101 CE_CAN_3.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP102 CE_CAN_3.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP103 CE_CAN_3.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP104 CE_CAN_3.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP105 CE_CAN_3.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP106 CE_CAN_3.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP107 CE_CAN_3.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP108 CE_CAN_3.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP109 CE_CAN_3.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP110 CE_CAN_3.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP111 CE_CAN_3.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP112 CE_CAN_3.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP113 CE_CAN_3.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP114 CE_CAN_3.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP115 CE_CAN_3.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP116 CE_CAN_3.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP117 CE_CAN_3.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP118 CE_CAN_3.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP119 CE_CAN_3.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP120 CE_CAN_3.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP121 CE_CAN_3.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP122 CE_CAN_3.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP123 CE_CAN_3.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP124 CE_CAN_3.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP125 CE_CAN_3.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP126 CE_CAN_3.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_3_HR_TIME_STAMP127 CE_CAN_3.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_3_ERFFEL0     CE_CAN_3.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL1     CE_CAN_3.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL2     CE_CAN_3.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL3     CE_CAN_3.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL4     CE_CAN_3.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL5     CE_CAN_3.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL6     CE_CAN_3.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL7     CE_CAN_3.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL8     CE_CAN_3.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL9     CE_CAN_3.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL10    CE_CAN_3.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL11    CE_CAN_3.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL12    CE_CAN_3.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL13    CE_CAN_3.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL14    CE_CAN_3.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL15    CE_CAN_3.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL16    CE_CAN_3.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL17    CE_CAN_3.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL18    CE_CAN_3.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL19    CE_CAN_3.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL20    CE_CAN_3.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL21    CE_CAN_3.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL22    CE_CAN_3.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL23    CE_CAN_3.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL24    CE_CAN_3.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL25    CE_CAN_3.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL26    CE_CAN_3.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL27    CE_CAN_3.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL28    CE_CAN_3.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL29    CE_CAN_3.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL30    CE_CAN_3.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL31    CE_CAN_3.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL32    CE_CAN_3.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL33    CE_CAN_3.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL34    CE_CAN_3.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL35    CE_CAN_3.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL36    CE_CAN_3.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL37    CE_CAN_3.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL38    CE_CAN_3.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL39    CE_CAN_3.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL40    CE_CAN_3.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL41    CE_CAN_3.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL42    CE_CAN_3.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL43    CE_CAN_3.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL44    CE_CAN_3.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL45    CE_CAN_3.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL46    CE_CAN_3.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL47    CE_CAN_3.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL48    CE_CAN_3.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL49    CE_CAN_3.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL50    CE_CAN_3.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL51    CE_CAN_3.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL52    CE_CAN_3.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL53    CE_CAN_3.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL54    CE_CAN_3.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL55    CE_CAN_3.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL56    CE_CAN_3.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL57    CE_CAN_3.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL58    CE_CAN_3.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL59    CE_CAN_3.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL60    CE_CAN_3.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL61    CE_CAN_3.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL62    CE_CAN_3.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL63    CE_CAN_3.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL64    CE_CAN_3.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL65    CE_CAN_3.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL66    CE_CAN_3.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL67    CE_CAN_3.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL68    CE_CAN_3.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL69    CE_CAN_3.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL70    CE_CAN_3.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL71    CE_CAN_3.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL72    CE_CAN_3.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL73    CE_CAN_3.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL74    CE_CAN_3.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL75    CE_CAN_3.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL76    CE_CAN_3.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL77    CE_CAN_3.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL78    CE_CAN_3.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL79    CE_CAN_3.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL80    CE_CAN_3.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL81    CE_CAN_3.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL82    CE_CAN_3.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL83    CE_CAN_3.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL84    CE_CAN_3.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL85    CE_CAN_3.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL86    CE_CAN_3.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL87    CE_CAN_3.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL88    CE_CAN_3.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL89    CE_CAN_3.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL90    CE_CAN_3.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL91    CE_CAN_3.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL92    CE_CAN_3.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL93    CE_CAN_3.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL94    CE_CAN_3.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL95    CE_CAN_3.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL96    CE_CAN_3.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL97    CE_CAN_3.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL98    CE_CAN_3.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL99    CE_CAN_3.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL100   CE_CAN_3.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL101   CE_CAN_3.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL102   CE_CAN_3.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL103   CE_CAN_3.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL104   CE_CAN_3.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL105   CE_CAN_3.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL106   CE_CAN_3.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL107   CE_CAN_3.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL108   CE_CAN_3.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL109   CE_CAN_3.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL110   CE_CAN_3.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL111   CE_CAN_3.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL112   CE_CAN_3.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL113   CE_CAN_3.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL114   CE_CAN_3.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL115   CE_CAN_3.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL116   CE_CAN_3.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL117   CE_CAN_3.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL118   CE_CAN_3.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL119   CE_CAN_3.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL120   CE_CAN_3.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL121   CE_CAN_3.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL122   CE_CAN_3.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL123   CE_CAN_3.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL124   CE_CAN_3.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL125   CE_CAN_3.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL126   CE_CAN_3.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_3_ERFFEL127   CE_CAN_3.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_4_MCR         CE_CAN_4.MCR.R                /* Module Configuration */
#define CE_CAN_4_CTRL1       CE_CAN_4.CTRL1.R              /* Control 1 */
#define CE_CAN_4_TIMER       CE_CAN_4.TIMER.R              /* Free-Running Timer */
#define CE_CAN_4_RXMGMASK    CE_CAN_4.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_4_RX14MASK    CE_CAN_4.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_4_RX15MASK    CE_CAN_4.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_4_ECR         CE_CAN_4.ECR.R                /* Error Counter */
#define CE_CAN_4_ESR1        CE_CAN_4.ESR1.R               /* Error and Status 1 */
#define CE_CAN_4_IMASK2      CE_CAN_4.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_4_IMASK1      CE_CAN_4.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_4_IFLAG2      CE_CAN_4.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_4_IFLAG1      CE_CAN_4.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_4_CTRL2       CE_CAN_4.CTRL2.R              /* Control 2 */
#define CE_CAN_4_ESR2        CE_CAN_4.ESR2.R               /* Error and Status 2 */
#define CE_CAN_4_CRCR        CE_CAN_4.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_4_RXFGMASK    CE_CAN_4.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_4_RXFIR       CE_CAN_4.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_4_CBT         CE_CAN_4.CBT.R                /* CAN Bit Timing */
#define CE_CAN_4_IMASK4      CE_CAN_4.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_4_IMASK3      CE_CAN_4.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_4_IFLAG4      CE_CAN_4.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_4_IFLAG3      CE_CAN_4.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_4_CS0            CE_CAN_4.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_4_ID0            CE_CAN_4.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_4_B0             CE_CAN_4.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_4_H0             CE_CAN_4.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_4_W0             CE_CAN_4.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_4_CS1            CE_CAN_4.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_4_ID1            CE_CAN_4.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_4_B1             CE_CAN_4.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_4_H1             CE_CAN_4.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_4_W1             CE_CAN_4.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_4_CS2            CE_CAN_4.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_4_ID2            CE_CAN_4.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_4_B2             CE_CAN_4.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_4_H2             CE_CAN_4.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_4_W2             CE_CAN_4.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_4_CS3            CE_CAN_4.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_4_ID3            CE_CAN_4.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_4_B3             CE_CAN_4.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_4_H3             CE_CAN_4.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_4_W3             CE_CAN_4.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_4_CS4            CE_CAN_4.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_4_ID4            CE_CAN_4.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_4_B4             CE_CAN_4.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_4_H4             CE_CAN_4.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_4_W4             CE_CAN_4.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_4_CS5            CE_CAN_4.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_4_ID5            CE_CAN_4.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_4_B5             CE_CAN_4.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_4_H5             CE_CAN_4.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_4_W5             CE_CAN_4.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_4_CS6            CE_CAN_4.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_4_ID6            CE_CAN_4.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_4_B6             CE_CAN_4.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_4_H6             CE_CAN_4.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_4_W6             CE_CAN_4.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_4_CS7            CE_CAN_4.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_4_ID7            CE_CAN_4.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_4_B7             CE_CAN_4.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_4_H7             CE_CAN_4.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_4_W7             CE_CAN_4.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_4_CS8            CE_CAN_4.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_4_ID8            CE_CAN_4.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_4_B8             CE_CAN_4.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_4_H8             CE_CAN_4.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_4_W8             CE_CAN_4.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_4_CS9            CE_CAN_4.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_4_ID9            CE_CAN_4.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_4_B9             CE_CAN_4.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_4_H9             CE_CAN_4.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_4_W9             CE_CAN_4.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_4_CS10           CE_CAN_4.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_4_ID10           CE_CAN_4.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_4_B10            CE_CAN_4.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_4_H10            CE_CAN_4.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_4_W10            CE_CAN_4.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_4_CS11           CE_CAN_4.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_4_ID11           CE_CAN_4.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_4_B11            CE_CAN_4.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_4_H11            CE_CAN_4.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_4_W11            CE_CAN_4.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_4_CS12           CE_CAN_4.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_4_ID12           CE_CAN_4.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_4_B12            CE_CAN_4.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_4_H12            CE_CAN_4.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_4_W12            CE_CAN_4.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_4_CS13           CE_CAN_4.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_4_ID13           CE_CAN_4.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_4_B13            CE_CAN_4.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_4_H13            CE_CAN_4.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_4_W13            CE_CAN_4.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_4_CS14           CE_CAN_4.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_4_ID14           CE_CAN_4.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_4_B14            CE_CAN_4.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_4_H14            CE_CAN_4.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_4_W14            CE_CAN_4.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_4_CS15           CE_CAN_4.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_4_ID15           CE_CAN_4.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_4_B15            CE_CAN_4.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_4_H15            CE_CAN_4.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_4_W15            CE_CAN_4.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_4_CS16           CE_CAN_4.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_4_ID16           CE_CAN_4.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_4_B16            CE_CAN_4.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_4_H16            CE_CAN_4.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_4_W16            CE_CAN_4.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_4_CS17           CE_CAN_4.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_4_ID17           CE_CAN_4.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_4_B17            CE_CAN_4.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_4_H17            CE_CAN_4.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_4_W17            CE_CAN_4.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_4_CS18           CE_CAN_4.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_4_ID18           CE_CAN_4.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_4_B18            CE_CAN_4.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_4_H18            CE_CAN_4.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_4_W18            CE_CAN_4.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_4_CS19           CE_CAN_4.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_4_ID19           CE_CAN_4.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_4_B19            CE_CAN_4.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_4_H19            CE_CAN_4.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_4_W19            CE_CAN_4.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_4_CS20           CE_CAN_4.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_4_ID20           CE_CAN_4.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_4_B20            CE_CAN_4.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_4_H20            CE_CAN_4.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_4_W20            CE_CAN_4.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_4_CS21           CE_CAN_4.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_4_ID21           CE_CAN_4.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_4_B21            CE_CAN_4.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_4_H21            CE_CAN_4.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_4_W21            CE_CAN_4.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_4_CS22           CE_CAN_4.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_4_ID22           CE_CAN_4.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_4_B22            CE_CAN_4.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_4_H22            CE_CAN_4.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_4_W22            CE_CAN_4.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_4_CS23           CE_CAN_4.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_4_ID23           CE_CAN_4.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_4_B23            CE_CAN_4.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_4_H23            CE_CAN_4.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_4_W23            CE_CAN_4.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_4_CS24           CE_CAN_4.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_4_ID24           CE_CAN_4.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_4_B24            CE_CAN_4.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_4_H24            CE_CAN_4.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_4_W24            CE_CAN_4.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_4_CS25           CE_CAN_4.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_4_ID25           CE_CAN_4.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_4_B25            CE_CAN_4.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_4_H25            CE_CAN_4.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_4_W25            CE_CAN_4.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_4_CS26           CE_CAN_4.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_4_ID26           CE_CAN_4.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_4_B26            CE_CAN_4.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_4_H26            CE_CAN_4.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_4_W26            CE_CAN_4.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_4_CS27           CE_CAN_4.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_4_ID27           CE_CAN_4.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_4_B27            CE_CAN_4.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_4_H27            CE_CAN_4.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_4_W27            CE_CAN_4.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_4_CS28           CE_CAN_4.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_4_ID28           CE_CAN_4.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_4_B28            CE_CAN_4.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_4_H28            CE_CAN_4.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_4_W28            CE_CAN_4.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_4_CS29           CE_CAN_4.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_4_ID29           CE_CAN_4.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_4_B29            CE_CAN_4.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_4_H29            CE_CAN_4.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_4_W29            CE_CAN_4.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_4_CS30           CE_CAN_4.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_4_ID30           CE_CAN_4.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_4_B30            CE_CAN_4.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_4_H30            CE_CAN_4.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_4_W30            CE_CAN_4.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_4_CS31           CE_CAN_4.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_4_ID31           CE_CAN_4.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_4_B31            CE_CAN_4.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_4_H31            CE_CAN_4.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_4_W31            CE_CAN_4.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_4_CS32           CE_CAN_4.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_4_ID32           CE_CAN_4.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_4_B32            CE_CAN_4.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_4_H32            CE_CAN_4.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_4_W32            CE_CAN_4.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_4_CS33           CE_CAN_4.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_4_ID33           CE_CAN_4.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_4_B33            CE_CAN_4.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_4_H33            CE_CAN_4.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_4_W33            CE_CAN_4.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_4_CS34           CE_CAN_4.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_4_ID34           CE_CAN_4.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_4_B34            CE_CAN_4.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_4_H34            CE_CAN_4.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_4_W34            CE_CAN_4.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_4_CS35           CE_CAN_4.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_4_ID35           CE_CAN_4.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_4_B35            CE_CAN_4.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_4_H35            CE_CAN_4.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_4_W35            CE_CAN_4.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_4_CS36           CE_CAN_4.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_4_ID36           CE_CAN_4.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_4_B36            CE_CAN_4.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_4_H36            CE_CAN_4.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_4_W36            CE_CAN_4.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_4_CS37           CE_CAN_4.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_4_ID37           CE_CAN_4.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_4_B37            CE_CAN_4.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_4_H37            CE_CAN_4.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_4_W37            CE_CAN_4.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_4_CS38           CE_CAN_4.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_4_ID38           CE_CAN_4.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_4_B38            CE_CAN_4.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_4_H38            CE_CAN_4.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_4_W38            CE_CAN_4.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_4_CS39           CE_CAN_4.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_4_ID39           CE_CAN_4.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_4_B39            CE_CAN_4.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_4_H39            CE_CAN_4.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_4_W39            CE_CAN_4.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_4_CS40           CE_CAN_4.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_4_ID40           CE_CAN_4.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_4_B40            CE_CAN_4.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_4_H40            CE_CAN_4.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_4_W40            CE_CAN_4.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_4_CS41           CE_CAN_4.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_4_ID41           CE_CAN_4.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_4_B41            CE_CAN_4.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_4_H41            CE_CAN_4.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_4_W41            CE_CAN_4.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_4_CS42           CE_CAN_4.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_4_ID42           CE_CAN_4.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_4_B42            CE_CAN_4.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_4_H42            CE_CAN_4.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_4_W42            CE_CAN_4.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_4_CS43           CE_CAN_4.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_4_ID43           CE_CAN_4.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_4_B43            CE_CAN_4.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_4_H43            CE_CAN_4.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_4_W43            CE_CAN_4.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_4_CS44           CE_CAN_4.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_4_ID44           CE_CAN_4.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_4_B44            CE_CAN_4.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_4_H44            CE_CAN_4.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_4_W44            CE_CAN_4.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_4_CS45           CE_CAN_4.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_4_ID45           CE_CAN_4.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_4_B45            CE_CAN_4.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_4_H45            CE_CAN_4.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_4_W45            CE_CAN_4.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_4_CS46           CE_CAN_4.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_4_ID46           CE_CAN_4.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_4_B46            CE_CAN_4.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_4_H46            CE_CAN_4.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_4_W46            CE_CAN_4.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_4_CS47           CE_CAN_4.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_4_ID47           CE_CAN_4.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_4_B47            CE_CAN_4.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_4_H47            CE_CAN_4.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_4_W47            CE_CAN_4.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_4_CS48           CE_CAN_4.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_4_ID48           CE_CAN_4.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_4_B48            CE_CAN_4.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_4_H48            CE_CAN_4.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_4_W48            CE_CAN_4.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_4_CS49           CE_CAN_4.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_4_ID49           CE_CAN_4.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_4_B49            CE_CAN_4.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_4_H49            CE_CAN_4.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_4_W49            CE_CAN_4.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_4_CS50           CE_CAN_4.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_4_ID50           CE_CAN_4.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_4_B50            CE_CAN_4.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_4_H50            CE_CAN_4.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_4_W50            CE_CAN_4.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_4_CS51           CE_CAN_4.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_4_ID51           CE_CAN_4.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_4_B51            CE_CAN_4.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_4_H51            CE_CAN_4.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_4_W51            CE_CAN_4.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_4_CS52           CE_CAN_4.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_4_ID52           CE_CAN_4.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_4_B52            CE_CAN_4.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_4_H52            CE_CAN_4.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_4_W52            CE_CAN_4.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_4_CS53           CE_CAN_4.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_4_ID53           CE_CAN_4.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_4_B53            CE_CAN_4.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_4_H53            CE_CAN_4.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_4_W53            CE_CAN_4.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_4_CS54           CE_CAN_4.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_4_ID54           CE_CAN_4.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_4_B54            CE_CAN_4.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_4_H54            CE_CAN_4.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_4_W54            CE_CAN_4.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_4_CS55           CE_CAN_4.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_4_ID55           CE_CAN_4.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_4_B55            CE_CAN_4.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_4_H55            CE_CAN_4.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_4_W55            CE_CAN_4.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_4_CS56           CE_CAN_4.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_4_ID56           CE_CAN_4.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_4_B56            CE_CAN_4.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_4_H56            CE_CAN_4.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_4_W56            CE_CAN_4.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_4_CS57           CE_CAN_4.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_4_ID57           CE_CAN_4.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_4_B57            CE_CAN_4.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_4_H57            CE_CAN_4.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_4_W57            CE_CAN_4.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_4_CS58           CE_CAN_4.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_4_ID58           CE_CAN_4.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_4_B58            CE_CAN_4.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_4_H58            CE_CAN_4.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_4_W58            CE_CAN_4.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_4_CS59           CE_CAN_4.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_4_ID59           CE_CAN_4.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_4_B59            CE_CAN_4.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_4_H59            CE_CAN_4.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_4_W59            CE_CAN_4.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_4_CS60           CE_CAN_4.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_4_ID60           CE_CAN_4.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_4_B60            CE_CAN_4.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_4_H60            CE_CAN_4.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_4_W60            CE_CAN_4.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_4_CS61           CE_CAN_4.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_4_ID61           CE_CAN_4.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_4_B61            CE_CAN_4.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_4_H61            CE_CAN_4.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_4_W61            CE_CAN_4.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_4_CS62           CE_CAN_4.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_4_ID62           CE_CAN_4.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_4_B62            CE_CAN_4.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_4_H62            CE_CAN_4.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_4_W62            CE_CAN_4.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_4_CS63           CE_CAN_4.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_4_ID63           CE_CAN_4.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_4_B63            CE_CAN_4.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_4_H63            CE_CAN_4.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_4_W63            CE_CAN_4.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_4_CS64           CE_CAN_4.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_4_ID64           CE_CAN_4.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_4_B64            CE_CAN_4.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_4_H64            CE_CAN_4.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_4_W64            CE_CAN_4.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_4_CS65           CE_CAN_4.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_4_ID65           CE_CAN_4.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_4_B65            CE_CAN_4.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_4_H65            CE_CAN_4.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_4_W65            CE_CAN_4.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_4_CS66           CE_CAN_4.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_4_ID66           CE_CAN_4.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_4_B66            CE_CAN_4.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_4_H66            CE_CAN_4.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_4_W66            CE_CAN_4.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_4_CS67           CE_CAN_4.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_4_ID67           CE_CAN_4.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_4_B67            CE_CAN_4.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_4_H67            CE_CAN_4.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_4_W67            CE_CAN_4.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_4_CS68           CE_CAN_4.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_4_ID68           CE_CAN_4.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_4_B68            CE_CAN_4.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_4_H68            CE_CAN_4.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_4_W68            CE_CAN_4.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_4_CS69           CE_CAN_4.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_4_ID69           CE_CAN_4.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_4_B69            CE_CAN_4.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_4_H69            CE_CAN_4.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_4_W69            CE_CAN_4.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_4_CS70           CE_CAN_4.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_4_ID70           CE_CAN_4.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_4_B70            CE_CAN_4.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_4_H70            CE_CAN_4.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_4_W70            CE_CAN_4.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_4_CS71           CE_CAN_4.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_4_ID71           CE_CAN_4.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_4_B71            CE_CAN_4.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_4_H71            CE_CAN_4.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_4_W71            CE_CAN_4.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_4_CS72           CE_CAN_4.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_4_ID72           CE_CAN_4.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_4_B72            CE_CAN_4.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_4_H72            CE_CAN_4.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_4_W72            CE_CAN_4.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_4_CS73           CE_CAN_4.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_4_ID73           CE_CAN_4.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_4_B73            CE_CAN_4.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_4_H73            CE_CAN_4.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_4_W73            CE_CAN_4.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_4_CS74           CE_CAN_4.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_4_ID74           CE_CAN_4.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_4_B74            CE_CAN_4.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_4_H74            CE_CAN_4.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_4_W74            CE_CAN_4.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_4_CS75           CE_CAN_4.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_4_ID75           CE_CAN_4.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_4_B75            CE_CAN_4.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_4_H75            CE_CAN_4.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_4_W75            CE_CAN_4.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_4_CS76           CE_CAN_4.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_4_ID76           CE_CAN_4.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_4_B76            CE_CAN_4.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_4_H76            CE_CAN_4.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_4_W76            CE_CAN_4.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_4_CS77           CE_CAN_4.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_4_ID77           CE_CAN_4.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_4_B77            CE_CAN_4.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_4_H77            CE_CAN_4.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_4_W77            CE_CAN_4.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_4_CS78           CE_CAN_4.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_4_ID78           CE_CAN_4.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_4_B78            CE_CAN_4.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_4_H78            CE_CAN_4.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_4_W78            CE_CAN_4.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_4_CS79           CE_CAN_4.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_4_ID79           CE_CAN_4.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_4_B79            CE_CAN_4.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_4_H79            CE_CAN_4.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_4_W79            CE_CAN_4.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_4_CS80           CE_CAN_4.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_4_ID80           CE_CAN_4.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_4_B80            CE_CAN_4.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_4_H80            CE_CAN_4.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_4_W80            CE_CAN_4.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_4_CS81           CE_CAN_4.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_4_ID81           CE_CAN_4.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_4_B81            CE_CAN_4.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_4_H81            CE_CAN_4.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_4_W81            CE_CAN_4.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_4_CS82           CE_CAN_4.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_4_ID82           CE_CAN_4.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_4_B82            CE_CAN_4.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_4_H82            CE_CAN_4.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_4_W82            CE_CAN_4.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_4_CS83           CE_CAN_4.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_4_ID83           CE_CAN_4.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_4_B83            CE_CAN_4.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_4_H83            CE_CAN_4.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_4_W83            CE_CAN_4.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_4_CS84           CE_CAN_4.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_4_ID84           CE_CAN_4.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_4_B84            CE_CAN_4.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_4_H84            CE_CAN_4.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_4_W84            CE_CAN_4.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_4_CS85           CE_CAN_4.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_4_ID85           CE_CAN_4.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_4_B85            CE_CAN_4.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_4_H85            CE_CAN_4.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_4_W85            CE_CAN_4.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_4_CS86           CE_CAN_4.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_4_ID86           CE_CAN_4.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_4_B86            CE_CAN_4.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_4_H86            CE_CAN_4.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_4_W86            CE_CAN_4.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_4_CS87           CE_CAN_4.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_4_ID87           CE_CAN_4.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_4_B87            CE_CAN_4.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_4_H87            CE_CAN_4.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_4_W87            CE_CAN_4.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_4_CS88           CE_CAN_4.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_4_ID88           CE_CAN_4.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_4_B88            CE_CAN_4.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_4_H88            CE_CAN_4.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_4_W848           CE_CAN_4.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_4_CS89           CE_CAN_4.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_4_ID89           CE_CAN_4.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_4_B89            CE_CAN_4.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_4_H89            CE_CAN_4.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_4_W89            CE_CAN_4.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_4_CS90           CE_CAN_4.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_4_ID90           CE_CAN_4.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_4_B90            CE_CAN_4.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_4_H90            CE_CAN_4.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_4_W90            CE_CAN_4.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_4_CS91           CE_CAN_4.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_4_ID91           CE_CAN_4.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_4_B91            CE_CAN_4.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_4_H91            CE_CAN_4.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_4_W91            CE_CAN_4.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_4_CS92           CE_CAN_4.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_4_ID92           CE_CAN_4.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_4_B92            CE_CAN_4.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_4_H92            CE_CAN_4.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_4_W92            CE_CAN_4.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_4_CS93           CE_CAN_4.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_4_ID93           CE_CAN_4.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_4_B93            CE_CAN_4.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_4_H93            CE_CAN_4.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_4_W93            CE_CAN_4.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_4_CS94           CE_CAN_4.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_4_ID94           CE_CAN_4.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_4_B94            CE_CAN_4.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_4_H94            CE_CAN_4.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_4_W94            CE_CAN_4.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_4_CS95           CE_CAN_4.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_4_ID95           CE_CAN_4.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_4_B95            CE_CAN_4.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_4_H95            CE_CAN_4.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_4_W95            CE_CAN_4.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_4_CS96           CE_CAN_4.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_4_ID96           CE_CAN_4.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_4_B96            CE_CAN_4.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_4_H96            CE_CAN_4.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_4_W96            CE_CAN_4.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_4_CS97           CE_CAN_4.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_4_ID97           CE_CAN_4.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_4_B97            CE_CAN_4.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_4_H97            CE_CAN_4.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_4_W97            CE_CAN_4.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_4_CS98           CE_CAN_4.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_4_ID98           CE_CAN_4.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_4_B98            CE_CAN_4.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_4_H98            CE_CAN_4.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_4_W98            CE_CAN_4.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_4_CS99           CE_CAN_4.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_4_ID99           CE_CAN_4.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_4_B99            CE_CAN_4.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_4_H99            CE_CAN_4.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_4_W99            CE_CAN_4.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_4_CS100          CE_CAN_4.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_4_ID100          CE_CAN_4.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_4_B100           CE_CAN_4.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_4_H100           CE_CAN_4.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_4_W100           CE_CAN_4.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_4_CS101          CE_CAN_4.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_4_ID101          CE_CAN_4.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_4_B101           CE_CAN_4.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_4_H101           CE_CAN_4.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_4_W101           CE_CAN_4.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_4_CS102          CE_CAN_4.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_4_ID102          CE_CAN_4.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_4_B102           CE_CAN_4.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_4_H102           CE_CAN_4.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_4_W102           CE_CAN_4.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_4_CS103          CE_CAN_4.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_4_ID103          CE_CAN_4.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_4_B103           CE_CAN_4.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_4_H103           CE_CAN_4.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_4_W103           CE_CAN_4.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_4_CS104          CE_CAN_4.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_4_ID104          CE_CAN_4.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_4_B104           CE_CAN_4.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_4_H104           CE_CAN_4.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_4_W104           CE_CAN_4.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_4_CS105          CE_CAN_4.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_4_ID105          CE_CAN_4.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_4_B105           CE_CAN_4.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_4_H105           CE_CAN_4.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_4_W105           CE_CAN_4.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_4_CS106          CE_CAN_4.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_4_ID106          CE_CAN_4.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_4_B106           CE_CAN_4.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_4_H106           CE_CAN_4.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_4_W106           CE_CAN_4.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_4_CS107          CE_CAN_4.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_4_ID107          CE_CAN_4.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_4_B107           CE_CAN_4.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_4_H107           CE_CAN_4.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_4_W107           CE_CAN_4.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_4_CS108          CE_CAN_4.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_4_ID108          CE_CAN_4.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_4_B108           CE_CAN_4.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_4_H108           CE_CAN_4.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_4_W108           CE_CAN_4.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_4_CS109          CE_CAN_4.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_4_ID109          CE_CAN_4.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_4_B109           CE_CAN_4.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_4_H109           CE_CAN_4.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_4_W109           CE_CAN_4.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_4_CS110          CE_CAN_4.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_4_ID110          CE_CAN_4.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_4_B110           CE_CAN_4.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_4_H110           CE_CAN_4.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_4_W110           CE_CAN_4.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_4_CS111          CE_CAN_4.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_4_ID111          CE_CAN_4.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_4_B111           CE_CAN_4.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_4_H111           CE_CAN_4.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_4_W111           CE_CAN_4.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_4_CS112          CE_CAN_4.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_4_ID112          CE_CAN_4.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_4_B112           CE_CAN_4.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_4_H112           CE_CAN_4.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_4_W112           CE_CAN_4.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_4_CS113          CE_CAN_4.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_4_ID113          CE_CAN_4.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_4_B113           CE_CAN_4.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_4_H113           CE_CAN_4.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_4_W113           CE_CAN_4.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_4_CS114          CE_CAN_4.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_4_ID114          CE_CAN_4.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_4_B114           CE_CAN_4.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_4_H114           CE_CAN_4.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_4_W114           CE_CAN_4.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_4_CS115          CE_CAN_4.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_4_ID115          CE_CAN_4.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_4_B115           CE_CAN_4.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_4_H115           CE_CAN_4.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_4_W115           CE_CAN_4.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_4_CS116          CE_CAN_4.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_4_ID116          CE_CAN_4.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_4_B116           CE_CAN_4.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_4_H116           CE_CAN_4.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_4_W116           CE_CAN_4.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_4_CS117          CE_CAN_4.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_4_ID117          CE_CAN_4.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_4_B117           CE_CAN_4.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_4_H117           CE_CAN_4.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_4_W117           CE_CAN_4.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_4_CS118          CE_CAN_4.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_4_ID118          CE_CAN_4.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_4_B118           CE_CAN_4.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_4_H118           CE_CAN_4.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_4_W118           CE_CAN_4.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_4_CS119          CE_CAN_4.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_4_ID119          CE_CAN_4.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_4_B119           CE_CAN_4.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_4_H119           CE_CAN_4.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_4_W119           CE_CAN_4.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_4_CS120          CE_CAN_4.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_4_ID120          CE_CAN_4.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_4_B120           CE_CAN_4.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_4_H120           CE_CAN_4.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_4_W120           CE_CAN_4.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_4_CS121          CE_CAN_4.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_4_ID121          CE_CAN_4.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_4_B121           CE_CAN_4.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_4_H121           CE_CAN_4.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_4_W121           CE_CAN_4.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_4_CS122          CE_CAN_4.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_4_ID122          CE_CAN_4.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_4_B122           CE_CAN_4.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_4_H122           CE_CAN_4.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_4_W122           CE_CAN_4.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_4_CS123          CE_CAN_4.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_4_ID123          CE_CAN_4.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_4_B123           CE_CAN_4.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_4_H123           CE_CAN_4.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_4_W123           CE_CAN_4.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_4_CS124          CE_CAN_4.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_4_ID124          CE_CAN_4.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_4_B124           CE_CAN_4.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_4_H124           CE_CAN_4.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_4_W124           CE_CAN_4.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_4_CS125          CE_CAN_4.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_4_ID125          CE_CAN_4.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_4_B125           CE_CAN_4.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_4_H125           CE_CAN_4.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_4_W125           CE_CAN_4.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_4_CS126          CE_CAN_4.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_4_ID126          CE_CAN_4.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_4_B126           CE_CAN_4.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_4_H126           CE_CAN_4.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_4_W126           CE_CAN_4.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_4_CS127          CE_CAN_4.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_4_ID127          CE_CAN_4.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_4_B127           CE_CAN_4.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_4_H127           CE_CAN_4.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_4_W127           CE_CAN_4.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_4_RXIMR0      CE_CAN_4.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR1      CE_CAN_4.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR2      CE_CAN_4.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR3      CE_CAN_4.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR4      CE_CAN_4.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR5      CE_CAN_4.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR6      CE_CAN_4.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR7      CE_CAN_4.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR8      CE_CAN_4.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR9      CE_CAN_4.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_4_RXIMR10     CE_CAN_4.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR11     CE_CAN_4.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR12     CE_CAN_4.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR13     CE_CAN_4.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR14     CE_CAN_4.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR15     CE_CAN_4.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR16     CE_CAN_4.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR17     CE_CAN_4.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR18     CE_CAN_4.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR19     CE_CAN_4.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR20     CE_CAN_4.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR21     CE_CAN_4.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR22     CE_CAN_4.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR23     CE_CAN_4.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR24     CE_CAN_4.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR25     CE_CAN_4.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR26     CE_CAN_4.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR27     CE_CAN_4.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR28     CE_CAN_4.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR29     CE_CAN_4.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR30     CE_CAN_4.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR31     CE_CAN_4.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR32     CE_CAN_4.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR33     CE_CAN_4.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR34     CE_CAN_4.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR35     CE_CAN_4.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR36     CE_CAN_4.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR37     CE_CAN_4.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR38     CE_CAN_4.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR39     CE_CAN_4.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR40     CE_CAN_4.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR41     CE_CAN_4.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR42     CE_CAN_4.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR43     CE_CAN_4.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR44     CE_CAN_4.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR45     CE_CAN_4.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR46     CE_CAN_4.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR47     CE_CAN_4.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR48     CE_CAN_4.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR49     CE_CAN_4.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR50     CE_CAN_4.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR51     CE_CAN_4.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR52     CE_CAN_4.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR53     CE_CAN_4.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR54     CE_CAN_4.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR55     CE_CAN_4.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR56     CE_CAN_4.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR57     CE_CAN_4.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR58     CE_CAN_4.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR59     CE_CAN_4.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR60     CE_CAN_4.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR61     CE_CAN_4.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR62     CE_CAN_4.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR63     CE_CAN_4.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR64     CE_CAN_4.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR65     CE_CAN_4.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR66     CE_CAN_4.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR67     CE_CAN_4.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR68     CE_CAN_4.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR69     CE_CAN_4.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR70     CE_CAN_4.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR71     CE_CAN_4.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR72     CE_CAN_4.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR73     CE_CAN_4.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR74     CE_CAN_4.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR75     CE_CAN_4.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR76     CE_CAN_4.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR77     CE_CAN_4.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR78     CE_CAN_4.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR79     CE_CAN_4.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR80     CE_CAN_4.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR81     CE_CAN_4.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR82     CE_CAN_4.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR83     CE_CAN_4.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR84     CE_CAN_4.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR85     CE_CAN_4.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR86     CE_CAN_4.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR87     CE_CAN_4.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR88     CE_CAN_4.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR89     CE_CAN_4.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR90     CE_CAN_4.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR91     CE_CAN_4.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR92     CE_CAN_4.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR93     CE_CAN_4.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR94     CE_CAN_4.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR95     CE_CAN_4.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR96     CE_CAN_4.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR97     CE_CAN_4.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR98     CE_CAN_4.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR99     CE_CAN_4.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_4_RXIMR100    CE_CAN_4.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR101    CE_CAN_4.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR102    CE_CAN_4.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR103    CE_CAN_4.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR104    CE_CAN_4.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR105    CE_CAN_4.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR106    CE_CAN_4.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR107    CE_CAN_4.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR108    CE_CAN_4.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR109    CE_CAN_4.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR110    CE_CAN_4.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR111    CE_CAN_4.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR112    CE_CAN_4.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR113    CE_CAN_4.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR114    CE_CAN_4.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR115    CE_CAN_4.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR116    CE_CAN_4.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR117    CE_CAN_4.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR118    CE_CAN_4.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR119    CE_CAN_4.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR120    CE_CAN_4.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR121    CE_CAN_4.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR122    CE_CAN_4.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR123    CE_CAN_4.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR124    CE_CAN_4.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR125    CE_CAN_4.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR126    CE_CAN_4.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_4_RXIMR127    CE_CAN_4.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_4_MECR        CE_CAN_4.MECR.R               /* Memory Error Control */
#define CE_CAN_4_ERRIAR      CE_CAN_4.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_4_ERRIDPR     CE_CAN_4.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_4_ERRIPPR     CE_CAN_4.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_4_RERRAR      CE_CAN_4.RERRAR.R             /* Error Report Address */
#define CE_CAN_4_RERRDR      CE_CAN_4.RERRDR.R             /* Error Report Data */
#define CE_CAN_4_RERRSYNR    CE_CAN_4.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_4_ERRSR       CE_CAN_4.ERRSR.R              /* Error Status */
#define CE_CAN_4_EPRS        CE_CAN_4.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_4_ENCBT       CE_CAN_4.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_4_EDCBT       CE_CAN_4.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_4_ETDC        CE_CAN_4.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_4_FDCTRL      CE_CAN_4.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_4_FDCBT       CE_CAN_4.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_4_FDCRC       CE_CAN_4.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_4_ERFCR       CE_CAN_4.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_4_ERFIER      CE_CAN_4.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_4_ERFSR       CE_CAN_4.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_4_HR_TIME_STAMP0 CE_CAN_4.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP1 CE_CAN_4.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP2 CE_CAN_4.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP3 CE_CAN_4.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP4 CE_CAN_4.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP5 CE_CAN_4.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP6 CE_CAN_4.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP7 CE_CAN_4.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP8 CE_CAN_4.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP9 CE_CAN_4.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP10 CE_CAN_4.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP11 CE_CAN_4.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP12 CE_CAN_4.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP13 CE_CAN_4.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP14 CE_CAN_4.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP15 CE_CAN_4.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP16 CE_CAN_4.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP17 CE_CAN_4.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP18 CE_CAN_4.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP19 CE_CAN_4.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP20 CE_CAN_4.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP21 CE_CAN_4.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP22 CE_CAN_4.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP23 CE_CAN_4.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP24 CE_CAN_4.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP25 CE_CAN_4.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP26 CE_CAN_4.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP27 CE_CAN_4.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP28 CE_CAN_4.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP29 CE_CAN_4.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP30 CE_CAN_4.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP31 CE_CAN_4.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP32 CE_CAN_4.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP33 CE_CAN_4.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP34 CE_CAN_4.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP35 CE_CAN_4.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP36 CE_CAN_4.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP37 CE_CAN_4.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP38 CE_CAN_4.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP39 CE_CAN_4.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP40 CE_CAN_4.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP41 CE_CAN_4.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP42 CE_CAN_4.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP43 CE_CAN_4.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP44 CE_CAN_4.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP45 CE_CAN_4.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP46 CE_CAN_4.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP47 CE_CAN_4.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP48 CE_CAN_4.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP49 CE_CAN_4.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP50 CE_CAN_4.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP51 CE_CAN_4.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP52 CE_CAN_4.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP53 CE_CAN_4.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP54 CE_CAN_4.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP55 CE_CAN_4.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP56 CE_CAN_4.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP57 CE_CAN_4.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP58 CE_CAN_4.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP59 CE_CAN_4.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP60 CE_CAN_4.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP61 CE_CAN_4.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP62 CE_CAN_4.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP63 CE_CAN_4.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP64 CE_CAN_4.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP65 CE_CAN_4.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP66 CE_CAN_4.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP67 CE_CAN_4.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP68 CE_CAN_4.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP69 CE_CAN_4.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP70 CE_CAN_4.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP71 CE_CAN_4.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP72 CE_CAN_4.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP73 CE_CAN_4.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP74 CE_CAN_4.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP75 CE_CAN_4.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP76 CE_CAN_4.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP77 CE_CAN_4.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP78 CE_CAN_4.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP79 CE_CAN_4.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP80 CE_CAN_4.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP81 CE_CAN_4.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP82 CE_CAN_4.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP83 CE_CAN_4.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP84 CE_CAN_4.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP85 CE_CAN_4.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP86 CE_CAN_4.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP87 CE_CAN_4.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP88 CE_CAN_4.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP89 CE_CAN_4.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP90 CE_CAN_4.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP91 CE_CAN_4.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP92 CE_CAN_4.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP93 CE_CAN_4.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP94 CE_CAN_4.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP95 CE_CAN_4.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP96 CE_CAN_4.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP97 CE_CAN_4.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP98 CE_CAN_4.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP99 CE_CAN_4.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP100 CE_CAN_4.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP101 CE_CAN_4.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP102 CE_CAN_4.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP103 CE_CAN_4.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP104 CE_CAN_4.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP105 CE_CAN_4.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP106 CE_CAN_4.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP107 CE_CAN_4.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP108 CE_CAN_4.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP109 CE_CAN_4.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP110 CE_CAN_4.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP111 CE_CAN_4.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP112 CE_CAN_4.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP113 CE_CAN_4.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP114 CE_CAN_4.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP115 CE_CAN_4.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP116 CE_CAN_4.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP117 CE_CAN_4.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP118 CE_CAN_4.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP119 CE_CAN_4.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP120 CE_CAN_4.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP121 CE_CAN_4.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP122 CE_CAN_4.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP123 CE_CAN_4.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP124 CE_CAN_4.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP125 CE_CAN_4.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP126 CE_CAN_4.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_4_HR_TIME_STAMP127 CE_CAN_4.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_4_ERFFEL0     CE_CAN_4.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL1     CE_CAN_4.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL2     CE_CAN_4.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL3     CE_CAN_4.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL4     CE_CAN_4.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL5     CE_CAN_4.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL6     CE_CAN_4.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL7     CE_CAN_4.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL8     CE_CAN_4.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL9     CE_CAN_4.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL10    CE_CAN_4.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL11    CE_CAN_4.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL12    CE_CAN_4.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL13    CE_CAN_4.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL14    CE_CAN_4.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL15    CE_CAN_4.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL16    CE_CAN_4.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL17    CE_CAN_4.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL18    CE_CAN_4.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL19    CE_CAN_4.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL20    CE_CAN_4.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL21    CE_CAN_4.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL22    CE_CAN_4.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL23    CE_CAN_4.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL24    CE_CAN_4.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL25    CE_CAN_4.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL26    CE_CAN_4.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL27    CE_CAN_4.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL28    CE_CAN_4.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL29    CE_CAN_4.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL30    CE_CAN_4.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL31    CE_CAN_4.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL32    CE_CAN_4.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL33    CE_CAN_4.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL34    CE_CAN_4.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL35    CE_CAN_4.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL36    CE_CAN_4.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL37    CE_CAN_4.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL38    CE_CAN_4.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL39    CE_CAN_4.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL40    CE_CAN_4.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL41    CE_CAN_4.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL42    CE_CAN_4.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL43    CE_CAN_4.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL44    CE_CAN_4.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL45    CE_CAN_4.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL46    CE_CAN_4.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL47    CE_CAN_4.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL48    CE_CAN_4.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL49    CE_CAN_4.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL50    CE_CAN_4.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL51    CE_CAN_4.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL52    CE_CAN_4.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL53    CE_CAN_4.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL54    CE_CAN_4.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL55    CE_CAN_4.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL56    CE_CAN_4.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL57    CE_CAN_4.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL58    CE_CAN_4.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL59    CE_CAN_4.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL60    CE_CAN_4.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL61    CE_CAN_4.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL62    CE_CAN_4.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL63    CE_CAN_4.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL64    CE_CAN_4.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL65    CE_CAN_4.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL66    CE_CAN_4.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL67    CE_CAN_4.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL68    CE_CAN_4.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL69    CE_CAN_4.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL70    CE_CAN_4.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL71    CE_CAN_4.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL72    CE_CAN_4.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL73    CE_CAN_4.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL74    CE_CAN_4.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL75    CE_CAN_4.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL76    CE_CAN_4.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL77    CE_CAN_4.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL78    CE_CAN_4.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL79    CE_CAN_4.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL80    CE_CAN_4.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL81    CE_CAN_4.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL82    CE_CAN_4.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL83    CE_CAN_4.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL84    CE_CAN_4.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL85    CE_CAN_4.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL86    CE_CAN_4.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL87    CE_CAN_4.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL88    CE_CAN_4.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL89    CE_CAN_4.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL90    CE_CAN_4.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL91    CE_CAN_4.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL92    CE_CAN_4.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL93    CE_CAN_4.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL94    CE_CAN_4.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL95    CE_CAN_4.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL96    CE_CAN_4.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL97    CE_CAN_4.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL98    CE_CAN_4.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL99    CE_CAN_4.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL100   CE_CAN_4.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL101   CE_CAN_4.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL102   CE_CAN_4.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL103   CE_CAN_4.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL104   CE_CAN_4.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL105   CE_CAN_4.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL106   CE_CAN_4.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL107   CE_CAN_4.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL108   CE_CAN_4.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL109   CE_CAN_4.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL110   CE_CAN_4.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL111   CE_CAN_4.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL112   CE_CAN_4.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL113   CE_CAN_4.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL114   CE_CAN_4.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL115   CE_CAN_4.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL116   CE_CAN_4.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL117   CE_CAN_4.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL118   CE_CAN_4.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL119   CE_CAN_4.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL120   CE_CAN_4.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL121   CE_CAN_4.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL122   CE_CAN_4.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL123   CE_CAN_4.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL124   CE_CAN_4.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL125   CE_CAN_4.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL126   CE_CAN_4.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_4_ERFFEL127   CE_CAN_4.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_5_MCR         CE_CAN_5.MCR.R                /* Module Configuration */
#define CE_CAN_5_CTRL1       CE_CAN_5.CTRL1.R              /* Control 1 */
#define CE_CAN_5_TIMER       CE_CAN_5.TIMER.R              /* Free-Running Timer */
#define CE_CAN_5_RXMGMASK    CE_CAN_5.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_5_RX14MASK    CE_CAN_5.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_5_RX15MASK    CE_CAN_5.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_5_ECR         CE_CAN_5.ECR.R                /* Error Counter */
#define CE_CAN_5_ESR1        CE_CAN_5.ESR1.R               /* Error and Status 1 */
#define CE_CAN_5_IMASK2      CE_CAN_5.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_5_IMASK1      CE_CAN_5.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_5_IFLAG2      CE_CAN_5.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_5_IFLAG1      CE_CAN_5.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_5_CTRL2       CE_CAN_5.CTRL2.R              /* Control 2 */
#define CE_CAN_5_ESR2        CE_CAN_5.ESR2.R               /* Error and Status 2 */
#define CE_CAN_5_CRCR        CE_CAN_5.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_5_RXFGMASK    CE_CAN_5.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_5_RXFIR       CE_CAN_5.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_5_CBT         CE_CAN_5.CBT.R                /* CAN Bit Timing */
#define CE_CAN_5_IMASK4      CE_CAN_5.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_5_IMASK3      CE_CAN_5.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_5_IFLAG4      CE_CAN_5.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_5_IFLAG3      CE_CAN_5.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_5_CS0            CE_CAN_5.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_5_ID0            CE_CAN_5.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_5_B0             CE_CAN_5.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_5_H0             CE_CAN_5.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_5_W0             CE_CAN_5.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_5_CS1            CE_CAN_5.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_5_ID1            CE_CAN_5.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_5_B1             CE_CAN_5.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_5_H1             CE_CAN_5.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_5_W1             CE_CAN_5.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_5_CS2            CE_CAN_5.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_5_ID2            CE_CAN_5.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_5_B2             CE_CAN_5.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_5_H2             CE_CAN_5.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_5_W2             CE_CAN_5.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_5_CS3            CE_CAN_5.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_5_ID3            CE_CAN_5.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_5_B3             CE_CAN_5.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_5_H3             CE_CAN_5.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_5_W3             CE_CAN_5.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_5_CS4            CE_CAN_5.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_5_ID4            CE_CAN_5.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_5_B4             CE_CAN_5.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_5_H4             CE_CAN_5.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_5_W4             CE_CAN_5.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_5_CS5            CE_CAN_5.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_5_ID5            CE_CAN_5.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_5_B5             CE_CAN_5.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_5_H5             CE_CAN_5.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_5_W5             CE_CAN_5.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_5_CS6            CE_CAN_5.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_5_ID6            CE_CAN_5.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_5_B6             CE_CAN_5.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_5_H6             CE_CAN_5.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_5_W6             CE_CAN_5.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_5_CS7            CE_CAN_5.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_5_ID7            CE_CAN_5.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_5_B7             CE_CAN_5.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_5_H7             CE_CAN_5.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_5_W7             CE_CAN_5.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_5_CS8            CE_CAN_5.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_5_ID8            CE_CAN_5.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_5_B8             CE_CAN_5.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_5_H8             CE_CAN_5.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_5_W8             CE_CAN_5.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_5_CS9            CE_CAN_5.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_5_ID9            CE_CAN_5.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_5_B9             CE_CAN_5.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_5_H9             CE_CAN_5.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_5_W9             CE_CAN_5.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_5_CS10           CE_CAN_5.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_5_ID10           CE_CAN_5.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_5_B10            CE_CAN_5.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_5_H10            CE_CAN_5.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_5_W10            CE_CAN_5.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_5_CS11           CE_CAN_5.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_5_ID11           CE_CAN_5.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_5_B11            CE_CAN_5.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_5_H11            CE_CAN_5.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_5_W11            CE_CAN_5.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_5_CS12           CE_CAN_5.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_5_ID12           CE_CAN_5.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_5_B12            CE_CAN_5.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_5_H12            CE_CAN_5.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_5_W12            CE_CAN_5.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_5_CS13           CE_CAN_5.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_5_ID13           CE_CAN_5.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_5_B13            CE_CAN_5.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_5_H13            CE_CAN_5.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_5_W13            CE_CAN_5.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_5_CS14           CE_CAN_5.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_5_ID14           CE_CAN_5.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_5_B14            CE_CAN_5.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_5_H14            CE_CAN_5.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_5_W14            CE_CAN_5.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_5_CS15           CE_CAN_5.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_5_ID15           CE_CAN_5.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_5_B15            CE_CAN_5.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_5_H15            CE_CAN_5.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_5_W15            CE_CAN_5.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_5_CS16           CE_CAN_5.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_5_ID16           CE_CAN_5.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_5_B16            CE_CAN_5.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_5_H16            CE_CAN_5.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_5_W16            CE_CAN_5.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_5_CS17           CE_CAN_5.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_5_ID17           CE_CAN_5.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_5_B17            CE_CAN_5.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_5_H17            CE_CAN_5.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_5_W17            CE_CAN_5.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_5_CS18           CE_CAN_5.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_5_ID18           CE_CAN_5.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_5_B18            CE_CAN_5.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_5_H18            CE_CAN_5.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_5_W18            CE_CAN_5.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_5_CS19           CE_CAN_5.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_5_ID19           CE_CAN_5.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_5_B19            CE_CAN_5.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_5_H19            CE_CAN_5.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_5_W19            CE_CAN_5.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_5_CS20           CE_CAN_5.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_5_ID20           CE_CAN_5.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_5_B20            CE_CAN_5.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_5_H20            CE_CAN_5.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_5_W20            CE_CAN_5.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_5_CS21           CE_CAN_5.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_5_ID21           CE_CAN_5.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_5_B21            CE_CAN_5.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_5_H21            CE_CAN_5.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_5_W21            CE_CAN_5.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_5_CS22           CE_CAN_5.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_5_ID22           CE_CAN_5.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_5_B22            CE_CAN_5.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_5_H22            CE_CAN_5.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_5_W22            CE_CAN_5.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_5_CS23           CE_CAN_5.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_5_ID23           CE_CAN_5.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_5_B23            CE_CAN_5.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_5_H23            CE_CAN_5.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_5_W23            CE_CAN_5.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_5_CS24           CE_CAN_5.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_5_ID24           CE_CAN_5.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_5_B24            CE_CAN_5.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_5_H24            CE_CAN_5.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_5_W24            CE_CAN_5.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_5_CS25           CE_CAN_5.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_5_ID25           CE_CAN_5.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_5_B25            CE_CAN_5.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_5_H25            CE_CAN_5.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_5_W25            CE_CAN_5.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_5_CS26           CE_CAN_5.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_5_ID26           CE_CAN_5.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_5_B26            CE_CAN_5.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_5_H26            CE_CAN_5.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_5_W26            CE_CAN_5.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_5_CS27           CE_CAN_5.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_5_ID27           CE_CAN_5.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_5_B27            CE_CAN_5.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_5_H27            CE_CAN_5.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_5_W27            CE_CAN_5.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_5_CS28           CE_CAN_5.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_5_ID28           CE_CAN_5.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_5_B28            CE_CAN_5.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_5_H28            CE_CAN_5.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_5_W28            CE_CAN_5.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_5_CS29           CE_CAN_5.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_5_ID29           CE_CAN_5.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_5_B29            CE_CAN_5.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_5_H29            CE_CAN_5.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_5_W29            CE_CAN_5.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_5_CS30           CE_CAN_5.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_5_ID30           CE_CAN_5.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_5_B30            CE_CAN_5.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_5_H30            CE_CAN_5.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_5_W30            CE_CAN_5.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_5_CS31           CE_CAN_5.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_5_ID31           CE_CAN_5.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_5_B31            CE_CAN_5.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_5_H31            CE_CAN_5.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_5_W31            CE_CAN_5.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_5_CS32           CE_CAN_5.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_5_ID32           CE_CAN_5.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_5_B32            CE_CAN_5.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_5_H32            CE_CAN_5.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_5_W32            CE_CAN_5.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_5_CS33           CE_CAN_5.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_5_ID33           CE_CAN_5.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_5_B33            CE_CAN_5.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_5_H33            CE_CAN_5.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_5_W33            CE_CAN_5.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_5_CS34           CE_CAN_5.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_5_ID34           CE_CAN_5.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_5_B34            CE_CAN_5.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_5_H34            CE_CAN_5.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_5_W34            CE_CAN_5.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_5_CS35           CE_CAN_5.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_5_ID35           CE_CAN_5.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_5_B35            CE_CAN_5.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_5_H35            CE_CAN_5.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_5_W35            CE_CAN_5.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_5_CS36           CE_CAN_5.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_5_ID36           CE_CAN_5.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_5_B36            CE_CAN_5.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_5_H36            CE_CAN_5.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_5_W36            CE_CAN_5.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_5_CS37           CE_CAN_5.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_5_ID37           CE_CAN_5.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_5_B37            CE_CAN_5.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_5_H37            CE_CAN_5.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_5_W37            CE_CAN_5.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_5_CS38           CE_CAN_5.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_5_ID38           CE_CAN_5.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_5_B38            CE_CAN_5.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_5_H38            CE_CAN_5.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_5_W38            CE_CAN_5.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_5_CS39           CE_CAN_5.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_5_ID39           CE_CAN_5.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_5_B39            CE_CAN_5.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_5_H39            CE_CAN_5.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_5_W39            CE_CAN_5.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_5_CS40           CE_CAN_5.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_5_ID40           CE_CAN_5.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_5_B40            CE_CAN_5.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_5_H40            CE_CAN_5.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_5_W40            CE_CAN_5.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_5_CS41           CE_CAN_5.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_5_ID41           CE_CAN_5.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_5_B41            CE_CAN_5.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_5_H41            CE_CAN_5.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_5_W41            CE_CAN_5.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_5_CS42           CE_CAN_5.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_5_ID42           CE_CAN_5.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_5_B42            CE_CAN_5.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_5_H42            CE_CAN_5.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_5_W42            CE_CAN_5.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_5_CS43           CE_CAN_5.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_5_ID43           CE_CAN_5.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_5_B43            CE_CAN_5.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_5_H43            CE_CAN_5.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_5_W43            CE_CAN_5.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_5_CS44           CE_CAN_5.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_5_ID44           CE_CAN_5.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_5_B44            CE_CAN_5.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_5_H44            CE_CAN_5.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_5_W44            CE_CAN_5.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_5_CS45           CE_CAN_5.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_5_ID45           CE_CAN_5.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_5_B45            CE_CAN_5.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_5_H45            CE_CAN_5.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_5_W45            CE_CAN_5.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_5_CS46           CE_CAN_5.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_5_ID46           CE_CAN_5.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_5_B46            CE_CAN_5.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_5_H46            CE_CAN_5.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_5_W46            CE_CAN_5.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_5_CS47           CE_CAN_5.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_5_ID47           CE_CAN_5.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_5_B47            CE_CAN_5.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_5_H47            CE_CAN_5.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_5_W47            CE_CAN_5.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_5_CS48           CE_CAN_5.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_5_ID48           CE_CAN_5.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_5_B48            CE_CAN_5.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_5_H48            CE_CAN_5.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_5_W48            CE_CAN_5.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_5_CS49           CE_CAN_5.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_5_ID49           CE_CAN_5.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_5_B49            CE_CAN_5.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_5_H49            CE_CAN_5.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_5_W49            CE_CAN_5.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_5_CS50           CE_CAN_5.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_5_ID50           CE_CAN_5.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_5_B50            CE_CAN_5.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_5_H50            CE_CAN_5.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_5_W50            CE_CAN_5.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_5_CS51           CE_CAN_5.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_5_ID51           CE_CAN_5.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_5_B51            CE_CAN_5.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_5_H51            CE_CAN_5.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_5_W51            CE_CAN_5.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_5_CS52           CE_CAN_5.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_5_ID52           CE_CAN_5.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_5_B52            CE_CAN_5.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_5_H52            CE_CAN_5.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_5_W52            CE_CAN_5.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_5_CS53           CE_CAN_5.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_5_ID53           CE_CAN_5.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_5_B53            CE_CAN_5.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_5_H53            CE_CAN_5.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_5_W53            CE_CAN_5.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_5_CS54           CE_CAN_5.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_5_ID54           CE_CAN_5.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_5_B54            CE_CAN_5.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_5_H54            CE_CAN_5.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_5_W54            CE_CAN_5.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_5_CS55           CE_CAN_5.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_5_ID55           CE_CAN_5.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_5_B55            CE_CAN_5.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_5_H55            CE_CAN_5.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_5_W55            CE_CAN_5.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_5_CS56           CE_CAN_5.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_5_ID56           CE_CAN_5.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_5_B56            CE_CAN_5.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_5_H56            CE_CAN_5.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_5_W56            CE_CAN_5.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_5_CS57           CE_CAN_5.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_5_ID57           CE_CAN_5.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_5_B57            CE_CAN_5.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_5_H57            CE_CAN_5.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_5_W57            CE_CAN_5.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_5_CS58           CE_CAN_5.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_5_ID58           CE_CAN_5.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_5_B58            CE_CAN_5.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_5_H58            CE_CAN_5.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_5_W58            CE_CAN_5.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_5_CS59           CE_CAN_5.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_5_ID59           CE_CAN_5.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_5_B59            CE_CAN_5.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_5_H59            CE_CAN_5.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_5_W59            CE_CAN_5.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_5_CS60           CE_CAN_5.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_5_ID60           CE_CAN_5.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_5_B60            CE_CAN_5.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_5_H60            CE_CAN_5.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_5_W60            CE_CAN_5.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_5_CS61           CE_CAN_5.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_5_ID61           CE_CAN_5.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_5_B61            CE_CAN_5.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_5_H61            CE_CAN_5.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_5_W61            CE_CAN_5.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_5_CS62           CE_CAN_5.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_5_ID62           CE_CAN_5.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_5_B62            CE_CAN_5.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_5_H62            CE_CAN_5.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_5_W62            CE_CAN_5.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_5_CS63           CE_CAN_5.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_5_ID63           CE_CAN_5.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_5_B63            CE_CAN_5.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_5_H63            CE_CAN_5.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_5_W63            CE_CAN_5.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_5_CS64           CE_CAN_5.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_5_ID64           CE_CAN_5.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_5_B64            CE_CAN_5.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_5_H64            CE_CAN_5.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_5_W64            CE_CAN_5.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_5_CS65           CE_CAN_5.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_5_ID65           CE_CAN_5.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_5_B65            CE_CAN_5.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_5_H65            CE_CAN_5.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_5_W65            CE_CAN_5.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_5_CS66           CE_CAN_5.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_5_ID66           CE_CAN_5.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_5_B66            CE_CAN_5.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_5_H66            CE_CAN_5.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_5_W66            CE_CAN_5.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_5_CS67           CE_CAN_5.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_5_ID67           CE_CAN_5.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_5_B67            CE_CAN_5.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_5_H67            CE_CAN_5.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_5_W67            CE_CAN_5.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_5_CS68           CE_CAN_5.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_5_ID68           CE_CAN_5.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_5_B68            CE_CAN_5.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_5_H68            CE_CAN_5.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_5_W68            CE_CAN_5.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_5_CS69           CE_CAN_5.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_5_ID69           CE_CAN_5.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_5_B69            CE_CAN_5.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_5_H69            CE_CAN_5.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_5_W69            CE_CAN_5.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_5_CS70           CE_CAN_5.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_5_ID70           CE_CAN_5.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_5_B70            CE_CAN_5.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_5_H70            CE_CAN_5.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_5_W70            CE_CAN_5.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_5_CS71           CE_CAN_5.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_5_ID71           CE_CAN_5.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_5_B71            CE_CAN_5.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_5_H71            CE_CAN_5.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_5_W71            CE_CAN_5.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_5_CS72           CE_CAN_5.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_5_ID72           CE_CAN_5.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_5_B72            CE_CAN_5.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_5_H72            CE_CAN_5.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_5_W72            CE_CAN_5.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_5_CS73           CE_CAN_5.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_5_ID73           CE_CAN_5.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_5_B73            CE_CAN_5.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_5_H73            CE_CAN_5.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_5_W73            CE_CAN_5.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_5_CS74           CE_CAN_5.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_5_ID74           CE_CAN_5.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_5_B74            CE_CAN_5.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_5_H74            CE_CAN_5.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_5_W74            CE_CAN_5.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_5_CS75           CE_CAN_5.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_5_ID75           CE_CAN_5.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_5_B75            CE_CAN_5.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_5_H75            CE_CAN_5.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_5_W75            CE_CAN_5.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_5_CS76           CE_CAN_5.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_5_ID76           CE_CAN_5.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_5_B76            CE_CAN_5.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_5_H76            CE_CAN_5.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_5_W76            CE_CAN_5.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_5_CS77           CE_CAN_5.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_5_ID77           CE_CAN_5.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_5_B77            CE_CAN_5.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_5_H77            CE_CAN_5.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_5_W77            CE_CAN_5.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_5_CS78           CE_CAN_5.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_5_ID78           CE_CAN_5.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_5_B78            CE_CAN_5.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_5_H78            CE_CAN_5.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_5_W78            CE_CAN_5.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_5_CS79           CE_CAN_5.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_5_ID79           CE_CAN_5.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_5_B79            CE_CAN_5.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_5_H79            CE_CAN_5.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_5_W79            CE_CAN_5.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_5_CS80           CE_CAN_5.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_5_ID80           CE_CAN_5.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_5_B80            CE_CAN_5.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_5_H80            CE_CAN_5.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_5_W80            CE_CAN_5.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_5_CS81           CE_CAN_5.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_5_ID81           CE_CAN_5.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_5_B81            CE_CAN_5.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_5_H81            CE_CAN_5.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_5_W81            CE_CAN_5.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_5_CS82           CE_CAN_5.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_5_ID82           CE_CAN_5.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_5_B82            CE_CAN_5.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_5_H82            CE_CAN_5.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_5_W82            CE_CAN_5.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_5_CS83           CE_CAN_5.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_5_ID83           CE_CAN_5.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_5_B83            CE_CAN_5.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_5_H83            CE_CAN_5.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_5_W83            CE_CAN_5.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_5_CS84           CE_CAN_5.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_5_ID84           CE_CAN_5.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_5_B84            CE_CAN_5.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_5_H84            CE_CAN_5.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_5_W84            CE_CAN_5.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_5_CS85           CE_CAN_5.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_5_ID85           CE_CAN_5.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_5_B85            CE_CAN_5.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_5_H85            CE_CAN_5.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_5_W85            CE_CAN_5.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_5_CS86           CE_CAN_5.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_5_ID86           CE_CAN_5.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_5_B86            CE_CAN_5.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_5_H86            CE_CAN_5.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_5_W86            CE_CAN_5.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_5_CS87           CE_CAN_5.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_5_ID87           CE_CAN_5.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_5_B87            CE_CAN_5.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_5_H87            CE_CAN_5.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_5_W87            CE_CAN_5.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_5_CS88           CE_CAN_5.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_5_ID88           CE_CAN_5.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_5_B88            CE_CAN_5.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_5_H88            CE_CAN_5.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_5_W848           CE_CAN_5.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_5_CS89           CE_CAN_5.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_5_ID89           CE_CAN_5.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_5_B89            CE_CAN_5.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_5_H89            CE_CAN_5.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_5_W89            CE_CAN_5.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_5_CS90           CE_CAN_5.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_5_ID90           CE_CAN_5.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_5_B90            CE_CAN_5.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_5_H90            CE_CAN_5.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_5_W90            CE_CAN_5.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_5_CS91           CE_CAN_5.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_5_ID91           CE_CAN_5.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_5_B91            CE_CAN_5.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_5_H91            CE_CAN_5.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_5_W91            CE_CAN_5.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_5_CS92           CE_CAN_5.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_5_ID92           CE_CAN_5.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_5_B92            CE_CAN_5.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_5_H92            CE_CAN_5.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_5_W92            CE_CAN_5.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_5_CS93           CE_CAN_5.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_5_ID93           CE_CAN_5.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_5_B93            CE_CAN_5.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_5_H93            CE_CAN_5.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_5_W93            CE_CAN_5.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_5_CS94           CE_CAN_5.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_5_ID94           CE_CAN_5.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_5_B94            CE_CAN_5.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_5_H94            CE_CAN_5.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_5_W94            CE_CAN_5.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_5_CS95           CE_CAN_5.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_5_ID95           CE_CAN_5.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_5_B95            CE_CAN_5.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_5_H95            CE_CAN_5.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_5_W95            CE_CAN_5.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_5_CS96           CE_CAN_5.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_5_ID96           CE_CAN_5.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_5_B96            CE_CAN_5.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_5_H96            CE_CAN_5.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_5_W96            CE_CAN_5.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_5_CS97           CE_CAN_5.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_5_ID97           CE_CAN_5.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_5_B97            CE_CAN_5.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_5_H97            CE_CAN_5.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_5_W97            CE_CAN_5.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_5_CS98           CE_CAN_5.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_5_ID98           CE_CAN_5.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_5_B98            CE_CAN_5.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_5_H98            CE_CAN_5.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_5_W98            CE_CAN_5.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_5_CS99           CE_CAN_5.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_5_ID99           CE_CAN_5.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_5_B99            CE_CAN_5.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_5_H99            CE_CAN_5.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_5_W99            CE_CAN_5.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_5_CS100          CE_CAN_5.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_5_ID100          CE_CAN_5.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_5_B100           CE_CAN_5.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_5_H100           CE_CAN_5.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_5_W100           CE_CAN_5.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_5_CS101          CE_CAN_5.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_5_ID101          CE_CAN_5.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_5_B101           CE_CAN_5.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_5_H101           CE_CAN_5.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_5_W101           CE_CAN_5.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_5_CS102          CE_CAN_5.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_5_ID102          CE_CAN_5.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_5_B102           CE_CAN_5.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_5_H102           CE_CAN_5.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_5_W102           CE_CAN_5.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_5_CS103          CE_CAN_5.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_5_ID103          CE_CAN_5.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_5_B103           CE_CAN_5.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_5_H103           CE_CAN_5.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_5_W103           CE_CAN_5.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_5_CS104          CE_CAN_5.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_5_ID104          CE_CAN_5.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_5_B104           CE_CAN_5.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_5_H104           CE_CAN_5.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_5_W104           CE_CAN_5.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_5_CS105          CE_CAN_5.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_5_ID105          CE_CAN_5.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_5_B105           CE_CAN_5.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_5_H105           CE_CAN_5.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_5_W105           CE_CAN_5.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_5_CS106          CE_CAN_5.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_5_ID106          CE_CAN_5.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_5_B106           CE_CAN_5.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_5_H106           CE_CAN_5.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_5_W106           CE_CAN_5.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_5_CS107          CE_CAN_5.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_5_ID107          CE_CAN_5.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_5_B107           CE_CAN_5.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_5_H107           CE_CAN_5.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_5_W107           CE_CAN_5.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_5_CS108          CE_CAN_5.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_5_ID108          CE_CAN_5.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_5_B108           CE_CAN_5.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_5_H108           CE_CAN_5.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_5_W108           CE_CAN_5.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_5_CS109          CE_CAN_5.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_5_ID109          CE_CAN_5.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_5_B109           CE_CAN_5.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_5_H109           CE_CAN_5.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_5_W109           CE_CAN_5.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_5_CS110          CE_CAN_5.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_5_ID110          CE_CAN_5.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_5_B110           CE_CAN_5.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_5_H110           CE_CAN_5.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_5_W110           CE_CAN_5.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_5_CS111          CE_CAN_5.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_5_ID111          CE_CAN_5.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_5_B111           CE_CAN_5.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_5_H111           CE_CAN_5.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_5_W111           CE_CAN_5.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_5_CS112          CE_CAN_5.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_5_ID112          CE_CAN_5.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_5_B112           CE_CAN_5.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_5_H112           CE_CAN_5.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_5_W112           CE_CAN_5.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_5_CS113          CE_CAN_5.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_5_ID113          CE_CAN_5.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_5_B113           CE_CAN_5.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_5_H113           CE_CAN_5.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_5_W113           CE_CAN_5.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_5_CS114          CE_CAN_5.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_5_ID114          CE_CAN_5.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_5_B114           CE_CAN_5.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_5_H114           CE_CAN_5.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_5_W114           CE_CAN_5.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_5_CS115          CE_CAN_5.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_5_ID115          CE_CAN_5.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_5_B115           CE_CAN_5.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_5_H115           CE_CAN_5.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_5_W115           CE_CAN_5.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_5_CS116          CE_CAN_5.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_5_ID116          CE_CAN_5.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_5_B116           CE_CAN_5.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_5_H116           CE_CAN_5.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_5_W116           CE_CAN_5.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_5_CS117          CE_CAN_5.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_5_ID117          CE_CAN_5.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_5_B117           CE_CAN_5.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_5_H117           CE_CAN_5.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_5_W117           CE_CAN_5.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_5_CS118          CE_CAN_5.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_5_ID118          CE_CAN_5.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_5_B118           CE_CAN_5.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_5_H118           CE_CAN_5.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_5_W118           CE_CAN_5.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_5_CS119          CE_CAN_5.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_5_ID119          CE_CAN_5.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_5_B119           CE_CAN_5.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_5_H119           CE_CAN_5.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_5_W119           CE_CAN_5.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_5_CS120          CE_CAN_5.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_5_ID120          CE_CAN_5.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_5_B120           CE_CAN_5.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_5_H120           CE_CAN_5.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_5_W120           CE_CAN_5.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_5_CS121          CE_CAN_5.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_5_ID121          CE_CAN_5.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_5_B121           CE_CAN_5.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_5_H121           CE_CAN_5.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_5_W121           CE_CAN_5.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_5_CS122          CE_CAN_5.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_5_ID122          CE_CAN_5.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_5_B122           CE_CAN_5.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_5_H122           CE_CAN_5.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_5_W122           CE_CAN_5.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_5_CS123          CE_CAN_5.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_5_ID123          CE_CAN_5.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_5_B123           CE_CAN_5.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_5_H123           CE_CAN_5.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_5_W123           CE_CAN_5.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_5_CS124          CE_CAN_5.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_5_ID124          CE_CAN_5.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_5_B124           CE_CAN_5.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_5_H124           CE_CAN_5.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_5_W124           CE_CAN_5.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_5_CS125          CE_CAN_5.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_5_ID125          CE_CAN_5.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_5_B125           CE_CAN_5.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_5_H125           CE_CAN_5.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_5_W125           CE_CAN_5.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_5_CS126          CE_CAN_5.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_5_ID126          CE_CAN_5.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_5_B126           CE_CAN_5.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_5_H126           CE_CAN_5.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_5_W126           CE_CAN_5.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_5_CS127          CE_CAN_5.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_5_ID127          CE_CAN_5.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_5_B127           CE_CAN_5.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_5_H127           CE_CAN_5.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_5_W127           CE_CAN_5.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_5_RXIMR0      CE_CAN_5.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR1      CE_CAN_5.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR2      CE_CAN_5.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR3      CE_CAN_5.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR4      CE_CAN_5.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR5      CE_CAN_5.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR6      CE_CAN_5.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR7      CE_CAN_5.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR8      CE_CAN_5.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR9      CE_CAN_5.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_5_RXIMR10     CE_CAN_5.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR11     CE_CAN_5.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR12     CE_CAN_5.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR13     CE_CAN_5.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR14     CE_CAN_5.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR15     CE_CAN_5.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR16     CE_CAN_5.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR17     CE_CAN_5.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR18     CE_CAN_5.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR19     CE_CAN_5.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR20     CE_CAN_5.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR21     CE_CAN_5.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR22     CE_CAN_5.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR23     CE_CAN_5.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR24     CE_CAN_5.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR25     CE_CAN_5.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR26     CE_CAN_5.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR27     CE_CAN_5.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR28     CE_CAN_5.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR29     CE_CAN_5.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR30     CE_CAN_5.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR31     CE_CAN_5.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR32     CE_CAN_5.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR33     CE_CAN_5.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR34     CE_CAN_5.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR35     CE_CAN_5.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR36     CE_CAN_5.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR37     CE_CAN_5.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR38     CE_CAN_5.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR39     CE_CAN_5.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR40     CE_CAN_5.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR41     CE_CAN_5.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR42     CE_CAN_5.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR43     CE_CAN_5.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR44     CE_CAN_5.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR45     CE_CAN_5.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR46     CE_CAN_5.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR47     CE_CAN_5.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR48     CE_CAN_5.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR49     CE_CAN_5.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR50     CE_CAN_5.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR51     CE_CAN_5.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR52     CE_CAN_5.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR53     CE_CAN_5.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR54     CE_CAN_5.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR55     CE_CAN_5.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR56     CE_CAN_5.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR57     CE_CAN_5.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR58     CE_CAN_5.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR59     CE_CAN_5.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR60     CE_CAN_5.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR61     CE_CAN_5.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR62     CE_CAN_5.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR63     CE_CAN_5.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR64     CE_CAN_5.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR65     CE_CAN_5.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR66     CE_CAN_5.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR67     CE_CAN_5.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR68     CE_CAN_5.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR69     CE_CAN_5.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR70     CE_CAN_5.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR71     CE_CAN_5.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR72     CE_CAN_5.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR73     CE_CAN_5.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR74     CE_CAN_5.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR75     CE_CAN_5.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR76     CE_CAN_5.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR77     CE_CAN_5.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR78     CE_CAN_5.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR79     CE_CAN_5.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR80     CE_CAN_5.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR81     CE_CAN_5.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR82     CE_CAN_5.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR83     CE_CAN_5.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR84     CE_CAN_5.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR85     CE_CAN_5.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR86     CE_CAN_5.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR87     CE_CAN_5.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR88     CE_CAN_5.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR89     CE_CAN_5.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR90     CE_CAN_5.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR91     CE_CAN_5.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR92     CE_CAN_5.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR93     CE_CAN_5.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR94     CE_CAN_5.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR95     CE_CAN_5.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR96     CE_CAN_5.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR97     CE_CAN_5.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR98     CE_CAN_5.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR99     CE_CAN_5.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_5_RXIMR100    CE_CAN_5.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR101    CE_CAN_5.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR102    CE_CAN_5.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR103    CE_CAN_5.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR104    CE_CAN_5.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR105    CE_CAN_5.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR106    CE_CAN_5.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR107    CE_CAN_5.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR108    CE_CAN_5.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR109    CE_CAN_5.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR110    CE_CAN_5.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR111    CE_CAN_5.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR112    CE_CAN_5.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR113    CE_CAN_5.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR114    CE_CAN_5.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR115    CE_CAN_5.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR116    CE_CAN_5.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR117    CE_CAN_5.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR118    CE_CAN_5.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR119    CE_CAN_5.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR120    CE_CAN_5.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR121    CE_CAN_5.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR122    CE_CAN_5.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR123    CE_CAN_5.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR124    CE_CAN_5.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR125    CE_CAN_5.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR126    CE_CAN_5.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_5_RXIMR127    CE_CAN_5.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_5_MECR        CE_CAN_5.MECR.R               /* Memory Error Control */
#define CE_CAN_5_ERRIAR      CE_CAN_5.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_5_ERRIDPR     CE_CAN_5.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_5_ERRIPPR     CE_CAN_5.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_5_RERRAR      CE_CAN_5.RERRAR.R             /* Error Report Address */
#define CE_CAN_5_RERRDR      CE_CAN_5.RERRDR.R             /* Error Report Data */
#define CE_CAN_5_RERRSYNR    CE_CAN_5.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_5_ERRSR       CE_CAN_5.ERRSR.R              /* Error Status */
#define CE_CAN_5_EPRS        CE_CAN_5.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_5_ENCBT       CE_CAN_5.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_5_EDCBT       CE_CAN_5.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_5_ETDC        CE_CAN_5.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_5_FDCTRL      CE_CAN_5.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_5_FDCBT       CE_CAN_5.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_5_FDCRC       CE_CAN_5.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_5_ERFCR       CE_CAN_5.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_5_ERFIER      CE_CAN_5.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_5_ERFSR       CE_CAN_5.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_5_HR_TIME_STAMP0 CE_CAN_5.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP1 CE_CAN_5.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP2 CE_CAN_5.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP3 CE_CAN_5.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP4 CE_CAN_5.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP5 CE_CAN_5.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP6 CE_CAN_5.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP7 CE_CAN_5.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP8 CE_CAN_5.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP9 CE_CAN_5.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP10 CE_CAN_5.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP11 CE_CAN_5.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP12 CE_CAN_5.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP13 CE_CAN_5.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP14 CE_CAN_5.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP15 CE_CAN_5.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP16 CE_CAN_5.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP17 CE_CAN_5.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP18 CE_CAN_5.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP19 CE_CAN_5.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP20 CE_CAN_5.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP21 CE_CAN_5.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP22 CE_CAN_5.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP23 CE_CAN_5.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP24 CE_CAN_5.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP25 CE_CAN_5.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP26 CE_CAN_5.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP27 CE_CAN_5.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP28 CE_CAN_5.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP29 CE_CAN_5.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP30 CE_CAN_5.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP31 CE_CAN_5.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP32 CE_CAN_5.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP33 CE_CAN_5.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP34 CE_CAN_5.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP35 CE_CAN_5.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP36 CE_CAN_5.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP37 CE_CAN_5.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP38 CE_CAN_5.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP39 CE_CAN_5.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP40 CE_CAN_5.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP41 CE_CAN_5.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP42 CE_CAN_5.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP43 CE_CAN_5.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP44 CE_CAN_5.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP45 CE_CAN_5.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP46 CE_CAN_5.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP47 CE_CAN_5.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP48 CE_CAN_5.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP49 CE_CAN_5.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP50 CE_CAN_5.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP51 CE_CAN_5.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP52 CE_CAN_5.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP53 CE_CAN_5.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP54 CE_CAN_5.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP55 CE_CAN_5.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP56 CE_CAN_5.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP57 CE_CAN_5.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP58 CE_CAN_5.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP59 CE_CAN_5.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP60 CE_CAN_5.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP61 CE_CAN_5.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP62 CE_CAN_5.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP63 CE_CAN_5.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP64 CE_CAN_5.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP65 CE_CAN_5.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP66 CE_CAN_5.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP67 CE_CAN_5.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP68 CE_CAN_5.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP69 CE_CAN_5.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP70 CE_CAN_5.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP71 CE_CAN_5.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP72 CE_CAN_5.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP73 CE_CAN_5.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP74 CE_CAN_5.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP75 CE_CAN_5.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP76 CE_CAN_5.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP77 CE_CAN_5.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP78 CE_CAN_5.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP79 CE_CAN_5.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP80 CE_CAN_5.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP81 CE_CAN_5.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP82 CE_CAN_5.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP83 CE_CAN_5.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP84 CE_CAN_5.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP85 CE_CAN_5.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP86 CE_CAN_5.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP87 CE_CAN_5.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP88 CE_CAN_5.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP89 CE_CAN_5.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP90 CE_CAN_5.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP91 CE_CAN_5.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP92 CE_CAN_5.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP93 CE_CAN_5.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP94 CE_CAN_5.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP95 CE_CAN_5.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP96 CE_CAN_5.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP97 CE_CAN_5.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP98 CE_CAN_5.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP99 CE_CAN_5.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP100 CE_CAN_5.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP101 CE_CAN_5.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP102 CE_CAN_5.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP103 CE_CAN_5.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP104 CE_CAN_5.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP105 CE_CAN_5.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP106 CE_CAN_5.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP107 CE_CAN_5.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP108 CE_CAN_5.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP109 CE_CAN_5.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP110 CE_CAN_5.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP111 CE_CAN_5.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP112 CE_CAN_5.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP113 CE_CAN_5.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP114 CE_CAN_5.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP115 CE_CAN_5.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP116 CE_CAN_5.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP117 CE_CAN_5.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP118 CE_CAN_5.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP119 CE_CAN_5.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP120 CE_CAN_5.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP121 CE_CAN_5.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP122 CE_CAN_5.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP123 CE_CAN_5.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP124 CE_CAN_5.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP125 CE_CAN_5.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP126 CE_CAN_5.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_5_HR_TIME_STAMP127 CE_CAN_5.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_5_ERFFEL0     CE_CAN_5.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL1     CE_CAN_5.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL2     CE_CAN_5.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL3     CE_CAN_5.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL4     CE_CAN_5.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL5     CE_CAN_5.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL6     CE_CAN_5.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL7     CE_CAN_5.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL8     CE_CAN_5.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL9     CE_CAN_5.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL10    CE_CAN_5.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL11    CE_CAN_5.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL12    CE_CAN_5.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL13    CE_CAN_5.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL14    CE_CAN_5.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL15    CE_CAN_5.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL16    CE_CAN_5.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL17    CE_CAN_5.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL18    CE_CAN_5.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL19    CE_CAN_5.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL20    CE_CAN_5.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL21    CE_CAN_5.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL22    CE_CAN_5.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL23    CE_CAN_5.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL24    CE_CAN_5.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL25    CE_CAN_5.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL26    CE_CAN_5.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL27    CE_CAN_5.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL28    CE_CAN_5.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL29    CE_CAN_5.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL30    CE_CAN_5.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL31    CE_CAN_5.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL32    CE_CAN_5.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL33    CE_CAN_5.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL34    CE_CAN_5.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL35    CE_CAN_5.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL36    CE_CAN_5.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL37    CE_CAN_5.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL38    CE_CAN_5.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL39    CE_CAN_5.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL40    CE_CAN_5.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL41    CE_CAN_5.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL42    CE_CAN_5.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL43    CE_CAN_5.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL44    CE_CAN_5.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL45    CE_CAN_5.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL46    CE_CAN_5.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL47    CE_CAN_5.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL48    CE_CAN_5.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL49    CE_CAN_5.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL50    CE_CAN_5.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL51    CE_CAN_5.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL52    CE_CAN_5.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL53    CE_CAN_5.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL54    CE_CAN_5.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL55    CE_CAN_5.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL56    CE_CAN_5.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL57    CE_CAN_5.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL58    CE_CAN_5.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL59    CE_CAN_5.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL60    CE_CAN_5.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL61    CE_CAN_5.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL62    CE_CAN_5.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL63    CE_CAN_5.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL64    CE_CAN_5.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL65    CE_CAN_5.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL66    CE_CAN_5.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL67    CE_CAN_5.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL68    CE_CAN_5.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL69    CE_CAN_5.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL70    CE_CAN_5.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL71    CE_CAN_5.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL72    CE_CAN_5.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL73    CE_CAN_5.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL74    CE_CAN_5.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL75    CE_CAN_5.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL76    CE_CAN_5.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL77    CE_CAN_5.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL78    CE_CAN_5.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL79    CE_CAN_5.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL80    CE_CAN_5.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL81    CE_CAN_5.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL82    CE_CAN_5.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL83    CE_CAN_5.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL84    CE_CAN_5.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL85    CE_CAN_5.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL86    CE_CAN_5.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL87    CE_CAN_5.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL88    CE_CAN_5.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL89    CE_CAN_5.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL90    CE_CAN_5.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL91    CE_CAN_5.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL92    CE_CAN_5.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL93    CE_CAN_5.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL94    CE_CAN_5.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL95    CE_CAN_5.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL96    CE_CAN_5.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL97    CE_CAN_5.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL98    CE_CAN_5.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL99    CE_CAN_5.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL100   CE_CAN_5.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL101   CE_CAN_5.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL102   CE_CAN_5.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL103   CE_CAN_5.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL104   CE_CAN_5.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL105   CE_CAN_5.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL106   CE_CAN_5.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL107   CE_CAN_5.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL108   CE_CAN_5.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL109   CE_CAN_5.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL110   CE_CAN_5.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL111   CE_CAN_5.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL112   CE_CAN_5.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL113   CE_CAN_5.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL114   CE_CAN_5.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL115   CE_CAN_5.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL116   CE_CAN_5.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL117   CE_CAN_5.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL118   CE_CAN_5.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL119   CE_CAN_5.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL120   CE_CAN_5.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL121   CE_CAN_5.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL122   CE_CAN_5.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL123   CE_CAN_5.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL124   CE_CAN_5.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL125   CE_CAN_5.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL126   CE_CAN_5.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_5_ERFFEL127   CE_CAN_5.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_6_MCR         CE_CAN_6.MCR.R                /* Module Configuration */
#define CE_CAN_6_CTRL1       CE_CAN_6.CTRL1.R              /* Control 1 */
#define CE_CAN_6_TIMER       CE_CAN_6.TIMER.R              /* Free-Running Timer */
#define CE_CAN_6_RXMGMASK    CE_CAN_6.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_6_RX14MASK    CE_CAN_6.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_6_RX15MASK    CE_CAN_6.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_6_ECR         CE_CAN_6.ECR.R                /* Error Counter */
#define CE_CAN_6_ESR1        CE_CAN_6.ESR1.R               /* Error and Status 1 */
#define CE_CAN_6_IMASK2      CE_CAN_6.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_6_IMASK1      CE_CAN_6.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_6_IFLAG2      CE_CAN_6.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_6_IFLAG1      CE_CAN_6.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_6_CTRL2       CE_CAN_6.CTRL2.R              /* Control 2 */
#define CE_CAN_6_ESR2        CE_CAN_6.ESR2.R               /* Error and Status 2 */
#define CE_CAN_6_CRCR        CE_CAN_6.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_6_RXFGMASK    CE_CAN_6.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_6_RXFIR       CE_CAN_6.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_6_CBT         CE_CAN_6.CBT.R                /* CAN Bit Timing */
#define CE_CAN_6_IMASK4      CE_CAN_6.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_6_IMASK3      CE_CAN_6.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_6_IFLAG4      CE_CAN_6.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_6_IFLAG3      CE_CAN_6.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_6_CS0            CE_CAN_6.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_6_ID0            CE_CAN_6.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_6_B0             CE_CAN_6.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_6_H0             CE_CAN_6.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_6_W0             CE_CAN_6.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_6_CS1            CE_CAN_6.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_6_ID1            CE_CAN_6.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_6_B1             CE_CAN_6.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_6_H1             CE_CAN_6.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_6_W1             CE_CAN_6.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_6_CS2            CE_CAN_6.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_6_ID2            CE_CAN_6.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_6_B2             CE_CAN_6.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_6_H2             CE_CAN_6.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_6_W2             CE_CAN_6.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_6_CS3            CE_CAN_6.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_6_ID3            CE_CAN_6.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_6_B3             CE_CAN_6.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_6_H3             CE_CAN_6.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_6_W3             CE_CAN_6.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_6_CS4            CE_CAN_6.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_6_ID4            CE_CAN_6.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_6_B4             CE_CAN_6.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_6_H4             CE_CAN_6.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_6_W4             CE_CAN_6.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_6_CS5            CE_CAN_6.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_6_ID5            CE_CAN_6.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_6_B5             CE_CAN_6.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_6_H5             CE_CAN_6.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_6_W5             CE_CAN_6.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_6_CS6            CE_CAN_6.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_6_ID6            CE_CAN_6.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_6_B6             CE_CAN_6.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_6_H6             CE_CAN_6.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_6_W6             CE_CAN_6.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_6_CS7            CE_CAN_6.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_6_ID7            CE_CAN_6.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_6_B7             CE_CAN_6.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_6_H7             CE_CAN_6.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_6_W7             CE_CAN_6.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_6_CS8            CE_CAN_6.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_6_ID8            CE_CAN_6.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_6_B8             CE_CAN_6.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_6_H8             CE_CAN_6.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_6_W8             CE_CAN_6.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_6_CS9            CE_CAN_6.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_6_ID9            CE_CAN_6.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_6_B9             CE_CAN_6.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_6_H9             CE_CAN_6.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_6_W9             CE_CAN_6.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_6_CS10           CE_CAN_6.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_6_ID10           CE_CAN_6.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_6_B10            CE_CAN_6.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_6_H10            CE_CAN_6.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_6_W10            CE_CAN_6.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_6_CS11           CE_CAN_6.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_6_ID11           CE_CAN_6.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_6_B11            CE_CAN_6.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_6_H11            CE_CAN_6.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_6_W11            CE_CAN_6.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_6_CS12           CE_CAN_6.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_6_ID12           CE_CAN_6.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_6_B12            CE_CAN_6.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_6_H12            CE_CAN_6.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_6_W12            CE_CAN_6.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_6_CS13           CE_CAN_6.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_6_ID13           CE_CAN_6.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_6_B13            CE_CAN_6.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_6_H13            CE_CAN_6.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_6_W13            CE_CAN_6.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_6_CS14           CE_CAN_6.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_6_ID14           CE_CAN_6.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_6_B14            CE_CAN_6.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_6_H14            CE_CAN_6.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_6_W14            CE_CAN_6.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_6_CS15           CE_CAN_6.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_6_ID15           CE_CAN_6.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_6_B15            CE_CAN_6.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_6_H15            CE_CAN_6.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_6_W15            CE_CAN_6.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_6_CS16           CE_CAN_6.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_6_ID16           CE_CAN_6.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_6_B16            CE_CAN_6.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_6_H16            CE_CAN_6.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_6_W16            CE_CAN_6.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_6_CS17           CE_CAN_6.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_6_ID17           CE_CAN_6.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_6_B17            CE_CAN_6.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_6_H17            CE_CAN_6.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_6_W17            CE_CAN_6.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_6_CS18           CE_CAN_6.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_6_ID18           CE_CAN_6.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_6_B18            CE_CAN_6.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_6_H18            CE_CAN_6.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_6_W18            CE_CAN_6.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_6_CS19           CE_CAN_6.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_6_ID19           CE_CAN_6.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_6_B19            CE_CAN_6.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_6_H19            CE_CAN_6.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_6_W19            CE_CAN_6.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_6_CS20           CE_CAN_6.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_6_ID20           CE_CAN_6.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_6_B20            CE_CAN_6.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_6_H20            CE_CAN_6.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_6_W20            CE_CAN_6.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_6_CS21           CE_CAN_6.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_6_ID21           CE_CAN_6.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_6_B21            CE_CAN_6.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_6_H21            CE_CAN_6.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_6_W21            CE_CAN_6.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_6_CS22           CE_CAN_6.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_6_ID22           CE_CAN_6.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_6_B22            CE_CAN_6.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_6_H22            CE_CAN_6.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_6_W22            CE_CAN_6.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_6_CS23           CE_CAN_6.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_6_ID23           CE_CAN_6.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_6_B23            CE_CAN_6.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_6_H23            CE_CAN_6.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_6_W23            CE_CAN_6.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_6_CS24           CE_CAN_6.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_6_ID24           CE_CAN_6.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_6_B24            CE_CAN_6.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_6_H24            CE_CAN_6.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_6_W24            CE_CAN_6.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_6_CS25           CE_CAN_6.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_6_ID25           CE_CAN_6.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_6_B25            CE_CAN_6.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_6_H25            CE_CAN_6.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_6_W25            CE_CAN_6.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_6_CS26           CE_CAN_6.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_6_ID26           CE_CAN_6.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_6_B26            CE_CAN_6.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_6_H26            CE_CAN_6.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_6_W26            CE_CAN_6.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_6_CS27           CE_CAN_6.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_6_ID27           CE_CAN_6.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_6_B27            CE_CAN_6.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_6_H27            CE_CAN_6.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_6_W27            CE_CAN_6.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_6_CS28           CE_CAN_6.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_6_ID28           CE_CAN_6.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_6_B28            CE_CAN_6.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_6_H28            CE_CAN_6.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_6_W28            CE_CAN_6.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_6_CS29           CE_CAN_6.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_6_ID29           CE_CAN_6.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_6_B29            CE_CAN_6.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_6_H29            CE_CAN_6.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_6_W29            CE_CAN_6.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_6_CS30           CE_CAN_6.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_6_ID30           CE_CAN_6.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_6_B30            CE_CAN_6.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_6_H30            CE_CAN_6.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_6_W30            CE_CAN_6.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_6_CS31           CE_CAN_6.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_6_ID31           CE_CAN_6.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_6_B31            CE_CAN_6.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_6_H31            CE_CAN_6.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_6_W31            CE_CAN_6.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_6_CS32           CE_CAN_6.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_6_ID32           CE_CAN_6.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_6_B32            CE_CAN_6.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_6_H32            CE_CAN_6.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_6_W32            CE_CAN_6.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_6_CS33           CE_CAN_6.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_6_ID33           CE_CAN_6.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_6_B33            CE_CAN_6.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_6_H33            CE_CAN_6.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_6_W33            CE_CAN_6.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_6_CS34           CE_CAN_6.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_6_ID34           CE_CAN_6.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_6_B34            CE_CAN_6.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_6_H34            CE_CAN_6.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_6_W34            CE_CAN_6.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_6_CS35           CE_CAN_6.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_6_ID35           CE_CAN_6.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_6_B35            CE_CAN_6.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_6_H35            CE_CAN_6.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_6_W35            CE_CAN_6.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_6_CS36           CE_CAN_6.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_6_ID36           CE_CAN_6.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_6_B36            CE_CAN_6.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_6_H36            CE_CAN_6.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_6_W36            CE_CAN_6.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_6_CS37           CE_CAN_6.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_6_ID37           CE_CAN_6.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_6_B37            CE_CAN_6.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_6_H37            CE_CAN_6.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_6_W37            CE_CAN_6.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_6_CS38           CE_CAN_6.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_6_ID38           CE_CAN_6.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_6_B38            CE_CAN_6.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_6_H38            CE_CAN_6.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_6_W38            CE_CAN_6.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_6_CS39           CE_CAN_6.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_6_ID39           CE_CAN_6.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_6_B39            CE_CAN_6.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_6_H39            CE_CAN_6.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_6_W39            CE_CAN_6.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_6_CS40           CE_CAN_6.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_6_ID40           CE_CAN_6.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_6_B40            CE_CAN_6.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_6_H40            CE_CAN_6.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_6_W40            CE_CAN_6.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_6_CS41           CE_CAN_6.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_6_ID41           CE_CAN_6.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_6_B41            CE_CAN_6.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_6_H41            CE_CAN_6.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_6_W41            CE_CAN_6.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_6_CS42           CE_CAN_6.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_6_ID42           CE_CAN_6.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_6_B42            CE_CAN_6.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_6_H42            CE_CAN_6.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_6_W42            CE_CAN_6.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_6_CS43           CE_CAN_6.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_6_ID43           CE_CAN_6.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_6_B43            CE_CAN_6.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_6_H43            CE_CAN_6.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_6_W43            CE_CAN_6.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_6_CS44           CE_CAN_6.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_6_ID44           CE_CAN_6.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_6_B44            CE_CAN_6.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_6_H44            CE_CAN_6.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_6_W44            CE_CAN_6.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_6_CS45           CE_CAN_6.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_6_ID45           CE_CAN_6.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_6_B45            CE_CAN_6.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_6_H45            CE_CAN_6.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_6_W45            CE_CAN_6.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_6_CS46           CE_CAN_6.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_6_ID46           CE_CAN_6.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_6_B46            CE_CAN_6.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_6_H46            CE_CAN_6.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_6_W46            CE_CAN_6.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_6_CS47           CE_CAN_6.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_6_ID47           CE_CAN_6.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_6_B47            CE_CAN_6.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_6_H47            CE_CAN_6.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_6_W47            CE_CAN_6.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_6_CS48           CE_CAN_6.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_6_ID48           CE_CAN_6.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_6_B48            CE_CAN_6.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_6_H48            CE_CAN_6.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_6_W48            CE_CAN_6.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_6_CS49           CE_CAN_6.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_6_ID49           CE_CAN_6.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_6_B49            CE_CAN_6.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_6_H49            CE_CAN_6.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_6_W49            CE_CAN_6.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_6_CS50           CE_CAN_6.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_6_ID50           CE_CAN_6.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_6_B50            CE_CAN_6.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_6_H50            CE_CAN_6.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_6_W50            CE_CAN_6.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_6_CS51           CE_CAN_6.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_6_ID51           CE_CAN_6.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_6_B51            CE_CAN_6.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_6_H51            CE_CAN_6.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_6_W51            CE_CAN_6.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_6_CS52           CE_CAN_6.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_6_ID52           CE_CAN_6.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_6_B52            CE_CAN_6.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_6_H52            CE_CAN_6.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_6_W52            CE_CAN_6.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_6_CS53           CE_CAN_6.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_6_ID53           CE_CAN_6.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_6_B53            CE_CAN_6.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_6_H53            CE_CAN_6.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_6_W53            CE_CAN_6.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_6_CS54           CE_CAN_6.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_6_ID54           CE_CAN_6.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_6_B54            CE_CAN_6.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_6_H54            CE_CAN_6.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_6_W54            CE_CAN_6.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_6_CS55           CE_CAN_6.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_6_ID55           CE_CAN_6.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_6_B55            CE_CAN_6.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_6_H55            CE_CAN_6.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_6_W55            CE_CAN_6.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_6_CS56           CE_CAN_6.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_6_ID56           CE_CAN_6.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_6_B56            CE_CAN_6.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_6_H56            CE_CAN_6.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_6_W56            CE_CAN_6.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_6_CS57           CE_CAN_6.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_6_ID57           CE_CAN_6.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_6_B57            CE_CAN_6.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_6_H57            CE_CAN_6.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_6_W57            CE_CAN_6.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_6_CS58           CE_CAN_6.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_6_ID58           CE_CAN_6.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_6_B58            CE_CAN_6.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_6_H58            CE_CAN_6.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_6_W58            CE_CAN_6.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_6_CS59           CE_CAN_6.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_6_ID59           CE_CAN_6.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_6_B59            CE_CAN_6.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_6_H59            CE_CAN_6.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_6_W59            CE_CAN_6.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_6_CS60           CE_CAN_6.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_6_ID60           CE_CAN_6.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_6_B60            CE_CAN_6.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_6_H60            CE_CAN_6.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_6_W60            CE_CAN_6.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_6_CS61           CE_CAN_6.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_6_ID61           CE_CAN_6.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_6_B61            CE_CAN_6.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_6_H61            CE_CAN_6.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_6_W61            CE_CAN_6.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_6_CS62           CE_CAN_6.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_6_ID62           CE_CAN_6.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_6_B62            CE_CAN_6.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_6_H62            CE_CAN_6.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_6_W62            CE_CAN_6.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_6_CS63           CE_CAN_6.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_6_ID63           CE_CAN_6.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_6_B63            CE_CAN_6.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_6_H63            CE_CAN_6.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_6_W63            CE_CAN_6.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_6_CS64           CE_CAN_6.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_6_ID64           CE_CAN_6.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_6_B64            CE_CAN_6.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_6_H64            CE_CAN_6.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_6_W64            CE_CAN_6.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_6_CS65           CE_CAN_6.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_6_ID65           CE_CAN_6.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_6_B65            CE_CAN_6.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_6_H65            CE_CAN_6.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_6_W65            CE_CAN_6.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_6_CS66           CE_CAN_6.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_6_ID66           CE_CAN_6.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_6_B66            CE_CAN_6.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_6_H66            CE_CAN_6.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_6_W66            CE_CAN_6.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_6_CS67           CE_CAN_6.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_6_ID67           CE_CAN_6.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_6_B67            CE_CAN_6.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_6_H67            CE_CAN_6.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_6_W67            CE_CAN_6.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_6_CS68           CE_CAN_6.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_6_ID68           CE_CAN_6.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_6_B68            CE_CAN_6.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_6_H68            CE_CAN_6.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_6_W68            CE_CAN_6.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_6_CS69           CE_CAN_6.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_6_ID69           CE_CAN_6.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_6_B69            CE_CAN_6.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_6_H69            CE_CAN_6.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_6_W69            CE_CAN_6.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_6_CS70           CE_CAN_6.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_6_ID70           CE_CAN_6.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_6_B70            CE_CAN_6.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_6_H70            CE_CAN_6.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_6_W70            CE_CAN_6.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_6_CS71           CE_CAN_6.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_6_ID71           CE_CAN_6.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_6_B71            CE_CAN_6.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_6_H71            CE_CAN_6.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_6_W71            CE_CAN_6.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_6_CS72           CE_CAN_6.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_6_ID72           CE_CAN_6.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_6_B72            CE_CAN_6.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_6_H72            CE_CAN_6.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_6_W72            CE_CAN_6.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_6_CS73           CE_CAN_6.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_6_ID73           CE_CAN_6.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_6_B73            CE_CAN_6.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_6_H73            CE_CAN_6.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_6_W73            CE_CAN_6.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_6_CS74           CE_CAN_6.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_6_ID74           CE_CAN_6.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_6_B74            CE_CAN_6.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_6_H74            CE_CAN_6.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_6_W74            CE_CAN_6.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_6_CS75           CE_CAN_6.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_6_ID75           CE_CAN_6.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_6_B75            CE_CAN_6.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_6_H75            CE_CAN_6.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_6_W75            CE_CAN_6.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_6_CS76           CE_CAN_6.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_6_ID76           CE_CAN_6.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_6_B76            CE_CAN_6.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_6_H76            CE_CAN_6.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_6_W76            CE_CAN_6.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_6_CS77           CE_CAN_6.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_6_ID77           CE_CAN_6.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_6_B77            CE_CAN_6.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_6_H77            CE_CAN_6.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_6_W77            CE_CAN_6.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_6_CS78           CE_CAN_6.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_6_ID78           CE_CAN_6.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_6_B78            CE_CAN_6.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_6_H78            CE_CAN_6.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_6_W78            CE_CAN_6.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_6_CS79           CE_CAN_6.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_6_ID79           CE_CAN_6.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_6_B79            CE_CAN_6.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_6_H79            CE_CAN_6.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_6_W79            CE_CAN_6.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_6_CS80           CE_CAN_6.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_6_ID80           CE_CAN_6.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_6_B80            CE_CAN_6.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_6_H80            CE_CAN_6.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_6_W80            CE_CAN_6.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_6_CS81           CE_CAN_6.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_6_ID81           CE_CAN_6.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_6_B81            CE_CAN_6.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_6_H81            CE_CAN_6.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_6_W81            CE_CAN_6.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_6_CS82           CE_CAN_6.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_6_ID82           CE_CAN_6.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_6_B82            CE_CAN_6.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_6_H82            CE_CAN_6.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_6_W82            CE_CAN_6.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_6_CS83           CE_CAN_6.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_6_ID83           CE_CAN_6.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_6_B83            CE_CAN_6.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_6_H83            CE_CAN_6.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_6_W83            CE_CAN_6.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_6_CS84           CE_CAN_6.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_6_ID84           CE_CAN_6.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_6_B84            CE_CAN_6.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_6_H84            CE_CAN_6.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_6_W84            CE_CAN_6.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_6_CS85           CE_CAN_6.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_6_ID85           CE_CAN_6.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_6_B85            CE_CAN_6.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_6_H85            CE_CAN_6.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_6_W85            CE_CAN_6.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_6_CS86           CE_CAN_6.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_6_ID86           CE_CAN_6.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_6_B86            CE_CAN_6.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_6_H86            CE_CAN_6.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_6_W86            CE_CAN_6.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_6_CS87           CE_CAN_6.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_6_ID87           CE_CAN_6.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_6_B87            CE_CAN_6.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_6_H87            CE_CAN_6.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_6_W87            CE_CAN_6.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_6_CS88           CE_CAN_6.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_6_ID88           CE_CAN_6.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_6_B88            CE_CAN_6.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_6_H88            CE_CAN_6.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_6_W848           CE_CAN_6.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_6_CS89           CE_CAN_6.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_6_ID89           CE_CAN_6.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_6_B89            CE_CAN_6.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_6_H89            CE_CAN_6.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_6_W89            CE_CAN_6.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_6_CS90           CE_CAN_6.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_6_ID90           CE_CAN_6.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_6_B90            CE_CAN_6.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_6_H90            CE_CAN_6.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_6_W90            CE_CAN_6.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_6_CS91           CE_CAN_6.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_6_ID91           CE_CAN_6.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_6_B91            CE_CAN_6.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_6_H91            CE_CAN_6.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_6_W91            CE_CAN_6.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_6_CS92           CE_CAN_6.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_6_ID92           CE_CAN_6.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_6_B92            CE_CAN_6.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_6_H92            CE_CAN_6.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_6_W92            CE_CAN_6.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_6_CS93           CE_CAN_6.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_6_ID93           CE_CAN_6.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_6_B93            CE_CAN_6.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_6_H93            CE_CAN_6.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_6_W93            CE_CAN_6.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_6_CS94           CE_CAN_6.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_6_ID94           CE_CAN_6.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_6_B94            CE_CAN_6.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_6_H94            CE_CAN_6.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_6_W94            CE_CAN_6.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_6_CS95           CE_CAN_6.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_6_ID95           CE_CAN_6.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_6_B95            CE_CAN_6.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_6_H95            CE_CAN_6.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_6_W95            CE_CAN_6.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_6_CS96           CE_CAN_6.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_6_ID96           CE_CAN_6.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_6_B96            CE_CAN_6.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_6_H96            CE_CAN_6.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_6_W96            CE_CAN_6.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_6_CS97           CE_CAN_6.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_6_ID97           CE_CAN_6.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_6_B97            CE_CAN_6.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_6_H97            CE_CAN_6.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_6_W97            CE_CAN_6.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_6_CS98           CE_CAN_6.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_6_ID98           CE_CAN_6.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_6_B98            CE_CAN_6.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_6_H98            CE_CAN_6.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_6_W98            CE_CAN_6.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_6_CS99           CE_CAN_6.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_6_ID99           CE_CAN_6.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_6_B99            CE_CAN_6.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_6_H99            CE_CAN_6.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_6_W99            CE_CAN_6.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_6_CS100          CE_CAN_6.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_6_ID100          CE_CAN_6.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_6_B100           CE_CAN_6.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_6_H100           CE_CAN_6.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_6_W100           CE_CAN_6.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_6_CS101          CE_CAN_6.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_6_ID101          CE_CAN_6.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_6_B101           CE_CAN_6.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_6_H101           CE_CAN_6.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_6_W101           CE_CAN_6.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_6_CS102          CE_CAN_6.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_6_ID102          CE_CAN_6.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_6_B102           CE_CAN_6.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_6_H102           CE_CAN_6.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_6_W102           CE_CAN_6.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_6_CS103          CE_CAN_6.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_6_ID103          CE_CAN_6.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_6_B103           CE_CAN_6.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_6_H103           CE_CAN_6.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_6_W103           CE_CAN_6.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_6_CS104          CE_CAN_6.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_6_ID104          CE_CAN_6.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_6_B104           CE_CAN_6.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_6_H104           CE_CAN_6.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_6_W104           CE_CAN_6.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_6_CS105          CE_CAN_6.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_6_ID105          CE_CAN_6.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_6_B105           CE_CAN_6.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_6_H105           CE_CAN_6.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_6_W105           CE_CAN_6.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_6_CS106          CE_CAN_6.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_6_ID106          CE_CAN_6.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_6_B106           CE_CAN_6.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_6_H106           CE_CAN_6.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_6_W106           CE_CAN_6.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_6_CS107          CE_CAN_6.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_6_ID107          CE_CAN_6.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_6_B107           CE_CAN_6.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_6_H107           CE_CAN_6.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_6_W107           CE_CAN_6.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_6_CS108          CE_CAN_6.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_6_ID108          CE_CAN_6.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_6_B108           CE_CAN_6.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_6_H108           CE_CAN_6.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_6_W108           CE_CAN_6.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_6_CS109          CE_CAN_6.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_6_ID109          CE_CAN_6.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_6_B109           CE_CAN_6.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_6_H109           CE_CAN_6.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_6_W109           CE_CAN_6.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_6_CS110          CE_CAN_6.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_6_ID110          CE_CAN_6.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_6_B110           CE_CAN_6.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_6_H110           CE_CAN_6.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_6_W110           CE_CAN_6.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_6_CS111          CE_CAN_6.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_6_ID111          CE_CAN_6.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_6_B111           CE_CAN_6.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_6_H111           CE_CAN_6.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_6_W111           CE_CAN_6.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_6_CS112          CE_CAN_6.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_6_ID112          CE_CAN_6.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_6_B112           CE_CAN_6.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_6_H112           CE_CAN_6.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_6_W112           CE_CAN_6.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_6_CS113          CE_CAN_6.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_6_ID113          CE_CAN_6.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_6_B113           CE_CAN_6.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_6_H113           CE_CAN_6.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_6_W113           CE_CAN_6.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_6_CS114          CE_CAN_6.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_6_ID114          CE_CAN_6.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_6_B114           CE_CAN_6.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_6_H114           CE_CAN_6.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_6_W114           CE_CAN_6.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_6_CS115          CE_CAN_6.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_6_ID115          CE_CAN_6.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_6_B115           CE_CAN_6.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_6_H115           CE_CAN_6.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_6_W115           CE_CAN_6.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_6_CS116          CE_CAN_6.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_6_ID116          CE_CAN_6.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_6_B116           CE_CAN_6.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_6_H116           CE_CAN_6.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_6_W116           CE_CAN_6.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_6_CS117          CE_CAN_6.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_6_ID117          CE_CAN_6.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_6_B117           CE_CAN_6.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_6_H117           CE_CAN_6.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_6_W117           CE_CAN_6.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_6_CS118          CE_CAN_6.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_6_ID118          CE_CAN_6.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_6_B118           CE_CAN_6.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_6_H118           CE_CAN_6.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_6_W118           CE_CAN_6.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_6_CS119          CE_CAN_6.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_6_ID119          CE_CAN_6.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_6_B119           CE_CAN_6.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_6_H119           CE_CAN_6.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_6_W119           CE_CAN_6.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_6_CS120          CE_CAN_6.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_6_ID120          CE_CAN_6.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_6_B120           CE_CAN_6.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_6_H120           CE_CAN_6.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_6_W120           CE_CAN_6.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_6_CS121          CE_CAN_6.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_6_ID121          CE_CAN_6.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_6_B121           CE_CAN_6.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_6_H121           CE_CAN_6.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_6_W121           CE_CAN_6.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_6_CS122          CE_CAN_6.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_6_ID122          CE_CAN_6.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_6_B122           CE_CAN_6.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_6_H122           CE_CAN_6.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_6_W122           CE_CAN_6.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_6_CS123          CE_CAN_6.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_6_ID123          CE_CAN_6.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_6_B123           CE_CAN_6.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_6_H123           CE_CAN_6.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_6_W123           CE_CAN_6.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_6_CS124          CE_CAN_6.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_6_ID124          CE_CAN_6.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_6_B124           CE_CAN_6.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_6_H124           CE_CAN_6.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_6_W124           CE_CAN_6.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_6_CS125          CE_CAN_6.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_6_ID125          CE_CAN_6.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_6_B125           CE_CAN_6.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_6_H125           CE_CAN_6.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_6_W125           CE_CAN_6.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_6_CS126          CE_CAN_6.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_6_ID126          CE_CAN_6.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_6_B126           CE_CAN_6.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_6_H126           CE_CAN_6.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_6_W126           CE_CAN_6.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_6_CS127          CE_CAN_6.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_6_ID127          CE_CAN_6.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_6_B127           CE_CAN_6.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_6_H127           CE_CAN_6.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_6_W127           CE_CAN_6.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_6_RXIMR0      CE_CAN_6.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR1      CE_CAN_6.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR2      CE_CAN_6.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR3      CE_CAN_6.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR4      CE_CAN_6.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR5      CE_CAN_6.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR6      CE_CAN_6.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR7      CE_CAN_6.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR8      CE_CAN_6.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR9      CE_CAN_6.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_6_RXIMR10     CE_CAN_6.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR11     CE_CAN_6.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR12     CE_CAN_6.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR13     CE_CAN_6.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR14     CE_CAN_6.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR15     CE_CAN_6.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR16     CE_CAN_6.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR17     CE_CAN_6.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR18     CE_CAN_6.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR19     CE_CAN_6.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR20     CE_CAN_6.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR21     CE_CAN_6.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR22     CE_CAN_6.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR23     CE_CAN_6.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR24     CE_CAN_6.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR25     CE_CAN_6.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR26     CE_CAN_6.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR27     CE_CAN_6.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR28     CE_CAN_6.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR29     CE_CAN_6.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR30     CE_CAN_6.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR31     CE_CAN_6.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR32     CE_CAN_6.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR33     CE_CAN_6.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR34     CE_CAN_6.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR35     CE_CAN_6.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR36     CE_CAN_6.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR37     CE_CAN_6.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR38     CE_CAN_6.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR39     CE_CAN_6.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR40     CE_CAN_6.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR41     CE_CAN_6.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR42     CE_CAN_6.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR43     CE_CAN_6.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR44     CE_CAN_6.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR45     CE_CAN_6.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR46     CE_CAN_6.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR47     CE_CAN_6.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR48     CE_CAN_6.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR49     CE_CAN_6.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR50     CE_CAN_6.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR51     CE_CAN_6.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR52     CE_CAN_6.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR53     CE_CAN_6.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR54     CE_CAN_6.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR55     CE_CAN_6.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR56     CE_CAN_6.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR57     CE_CAN_6.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR58     CE_CAN_6.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR59     CE_CAN_6.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR60     CE_CAN_6.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR61     CE_CAN_6.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR62     CE_CAN_6.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR63     CE_CAN_6.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR64     CE_CAN_6.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR65     CE_CAN_6.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR66     CE_CAN_6.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR67     CE_CAN_6.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR68     CE_CAN_6.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR69     CE_CAN_6.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR70     CE_CAN_6.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR71     CE_CAN_6.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR72     CE_CAN_6.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR73     CE_CAN_6.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR74     CE_CAN_6.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR75     CE_CAN_6.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR76     CE_CAN_6.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR77     CE_CAN_6.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR78     CE_CAN_6.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR79     CE_CAN_6.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR80     CE_CAN_6.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR81     CE_CAN_6.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR82     CE_CAN_6.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR83     CE_CAN_6.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR84     CE_CAN_6.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR85     CE_CAN_6.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR86     CE_CAN_6.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR87     CE_CAN_6.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR88     CE_CAN_6.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR89     CE_CAN_6.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR90     CE_CAN_6.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR91     CE_CAN_6.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR92     CE_CAN_6.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR93     CE_CAN_6.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR94     CE_CAN_6.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR95     CE_CAN_6.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR96     CE_CAN_6.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR97     CE_CAN_6.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR98     CE_CAN_6.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR99     CE_CAN_6.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_6_RXIMR100    CE_CAN_6.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR101    CE_CAN_6.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR102    CE_CAN_6.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR103    CE_CAN_6.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR104    CE_CAN_6.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR105    CE_CAN_6.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR106    CE_CAN_6.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR107    CE_CAN_6.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR108    CE_CAN_6.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR109    CE_CAN_6.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR110    CE_CAN_6.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR111    CE_CAN_6.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR112    CE_CAN_6.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR113    CE_CAN_6.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR114    CE_CAN_6.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR115    CE_CAN_6.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR116    CE_CAN_6.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR117    CE_CAN_6.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR118    CE_CAN_6.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR119    CE_CAN_6.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR120    CE_CAN_6.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR121    CE_CAN_6.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR122    CE_CAN_6.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR123    CE_CAN_6.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR124    CE_CAN_6.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR125    CE_CAN_6.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR126    CE_CAN_6.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_6_RXIMR127    CE_CAN_6.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_6_MECR        CE_CAN_6.MECR.R               /* Memory Error Control */
#define CE_CAN_6_ERRIAR      CE_CAN_6.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_6_ERRIDPR     CE_CAN_6.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_6_ERRIPPR     CE_CAN_6.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_6_RERRAR      CE_CAN_6.RERRAR.R             /* Error Report Address */
#define CE_CAN_6_RERRDR      CE_CAN_6.RERRDR.R             /* Error Report Data */
#define CE_CAN_6_RERRSYNR    CE_CAN_6.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_6_ERRSR       CE_CAN_6.ERRSR.R              /* Error Status */
#define CE_CAN_6_EPRS        CE_CAN_6.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_6_ENCBT       CE_CAN_6.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_6_EDCBT       CE_CAN_6.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_6_ETDC        CE_CAN_6.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_6_FDCTRL      CE_CAN_6.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_6_FDCBT       CE_CAN_6.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_6_FDCRC       CE_CAN_6.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_6_ERFCR       CE_CAN_6.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_6_ERFIER      CE_CAN_6.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_6_ERFSR       CE_CAN_6.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_6_HR_TIME_STAMP0 CE_CAN_6.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP1 CE_CAN_6.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP2 CE_CAN_6.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP3 CE_CAN_6.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP4 CE_CAN_6.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP5 CE_CAN_6.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP6 CE_CAN_6.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP7 CE_CAN_6.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP8 CE_CAN_6.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP9 CE_CAN_6.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP10 CE_CAN_6.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP11 CE_CAN_6.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP12 CE_CAN_6.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP13 CE_CAN_6.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP14 CE_CAN_6.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP15 CE_CAN_6.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP16 CE_CAN_6.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP17 CE_CAN_6.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP18 CE_CAN_6.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP19 CE_CAN_6.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP20 CE_CAN_6.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP21 CE_CAN_6.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP22 CE_CAN_6.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP23 CE_CAN_6.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP24 CE_CAN_6.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP25 CE_CAN_6.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP26 CE_CAN_6.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP27 CE_CAN_6.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP28 CE_CAN_6.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP29 CE_CAN_6.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP30 CE_CAN_6.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP31 CE_CAN_6.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP32 CE_CAN_6.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP33 CE_CAN_6.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP34 CE_CAN_6.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP35 CE_CAN_6.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP36 CE_CAN_6.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP37 CE_CAN_6.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP38 CE_CAN_6.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP39 CE_CAN_6.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP40 CE_CAN_6.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP41 CE_CAN_6.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP42 CE_CAN_6.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP43 CE_CAN_6.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP44 CE_CAN_6.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP45 CE_CAN_6.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP46 CE_CAN_6.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP47 CE_CAN_6.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP48 CE_CAN_6.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP49 CE_CAN_6.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP50 CE_CAN_6.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP51 CE_CAN_6.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP52 CE_CAN_6.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP53 CE_CAN_6.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP54 CE_CAN_6.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP55 CE_CAN_6.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP56 CE_CAN_6.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP57 CE_CAN_6.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP58 CE_CAN_6.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP59 CE_CAN_6.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP60 CE_CAN_6.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP61 CE_CAN_6.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP62 CE_CAN_6.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP63 CE_CAN_6.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP64 CE_CAN_6.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP65 CE_CAN_6.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP66 CE_CAN_6.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP67 CE_CAN_6.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP68 CE_CAN_6.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP69 CE_CAN_6.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP70 CE_CAN_6.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP71 CE_CAN_6.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP72 CE_CAN_6.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP73 CE_CAN_6.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP74 CE_CAN_6.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP75 CE_CAN_6.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP76 CE_CAN_6.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP77 CE_CAN_6.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP78 CE_CAN_6.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP79 CE_CAN_6.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP80 CE_CAN_6.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP81 CE_CAN_6.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP82 CE_CAN_6.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP83 CE_CAN_6.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP84 CE_CAN_6.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP85 CE_CAN_6.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP86 CE_CAN_6.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP87 CE_CAN_6.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP88 CE_CAN_6.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP89 CE_CAN_6.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP90 CE_CAN_6.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP91 CE_CAN_6.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP92 CE_CAN_6.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP93 CE_CAN_6.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP94 CE_CAN_6.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP95 CE_CAN_6.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP96 CE_CAN_6.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP97 CE_CAN_6.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP98 CE_CAN_6.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP99 CE_CAN_6.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP100 CE_CAN_6.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP101 CE_CAN_6.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP102 CE_CAN_6.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP103 CE_CAN_6.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP104 CE_CAN_6.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP105 CE_CAN_6.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP106 CE_CAN_6.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP107 CE_CAN_6.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP108 CE_CAN_6.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP109 CE_CAN_6.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP110 CE_CAN_6.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP111 CE_CAN_6.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP112 CE_CAN_6.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP113 CE_CAN_6.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP114 CE_CAN_6.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP115 CE_CAN_6.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP116 CE_CAN_6.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP117 CE_CAN_6.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP118 CE_CAN_6.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP119 CE_CAN_6.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP120 CE_CAN_6.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP121 CE_CAN_6.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP122 CE_CAN_6.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP123 CE_CAN_6.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP124 CE_CAN_6.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP125 CE_CAN_6.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP126 CE_CAN_6.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_6_HR_TIME_STAMP127 CE_CAN_6.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_6_ERFFEL0     CE_CAN_6.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL1     CE_CAN_6.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL2     CE_CAN_6.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL3     CE_CAN_6.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL4     CE_CAN_6.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL5     CE_CAN_6.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL6     CE_CAN_6.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL7     CE_CAN_6.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL8     CE_CAN_6.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL9     CE_CAN_6.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL10    CE_CAN_6.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL11    CE_CAN_6.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL12    CE_CAN_6.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL13    CE_CAN_6.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL14    CE_CAN_6.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL15    CE_CAN_6.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL16    CE_CAN_6.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL17    CE_CAN_6.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL18    CE_CAN_6.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL19    CE_CAN_6.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL20    CE_CAN_6.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL21    CE_CAN_6.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL22    CE_CAN_6.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL23    CE_CAN_6.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL24    CE_CAN_6.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL25    CE_CAN_6.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL26    CE_CAN_6.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL27    CE_CAN_6.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL28    CE_CAN_6.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL29    CE_CAN_6.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL30    CE_CAN_6.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL31    CE_CAN_6.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL32    CE_CAN_6.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL33    CE_CAN_6.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL34    CE_CAN_6.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL35    CE_CAN_6.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL36    CE_CAN_6.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL37    CE_CAN_6.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL38    CE_CAN_6.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL39    CE_CAN_6.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL40    CE_CAN_6.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL41    CE_CAN_6.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL42    CE_CAN_6.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL43    CE_CAN_6.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL44    CE_CAN_6.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL45    CE_CAN_6.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL46    CE_CAN_6.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL47    CE_CAN_6.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL48    CE_CAN_6.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL49    CE_CAN_6.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL50    CE_CAN_6.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL51    CE_CAN_6.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL52    CE_CAN_6.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL53    CE_CAN_6.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL54    CE_CAN_6.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL55    CE_CAN_6.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL56    CE_CAN_6.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL57    CE_CAN_6.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL58    CE_CAN_6.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL59    CE_CAN_6.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL60    CE_CAN_6.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL61    CE_CAN_6.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL62    CE_CAN_6.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL63    CE_CAN_6.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL64    CE_CAN_6.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL65    CE_CAN_6.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL66    CE_CAN_6.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL67    CE_CAN_6.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL68    CE_CAN_6.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL69    CE_CAN_6.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL70    CE_CAN_6.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL71    CE_CAN_6.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL72    CE_CAN_6.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL73    CE_CAN_6.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL74    CE_CAN_6.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL75    CE_CAN_6.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL76    CE_CAN_6.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL77    CE_CAN_6.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL78    CE_CAN_6.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL79    CE_CAN_6.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL80    CE_CAN_6.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL81    CE_CAN_6.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL82    CE_CAN_6.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL83    CE_CAN_6.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL84    CE_CAN_6.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL85    CE_CAN_6.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL86    CE_CAN_6.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL87    CE_CAN_6.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL88    CE_CAN_6.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL89    CE_CAN_6.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL90    CE_CAN_6.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL91    CE_CAN_6.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL92    CE_CAN_6.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL93    CE_CAN_6.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL94    CE_CAN_6.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL95    CE_CAN_6.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL96    CE_CAN_6.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL97    CE_CAN_6.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL98    CE_CAN_6.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL99    CE_CAN_6.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL100   CE_CAN_6.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL101   CE_CAN_6.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL102   CE_CAN_6.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL103   CE_CAN_6.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL104   CE_CAN_6.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL105   CE_CAN_6.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL106   CE_CAN_6.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL107   CE_CAN_6.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL108   CE_CAN_6.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL109   CE_CAN_6.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL110   CE_CAN_6.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL111   CE_CAN_6.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL112   CE_CAN_6.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL113   CE_CAN_6.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL114   CE_CAN_6.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL115   CE_CAN_6.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL116   CE_CAN_6.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL117   CE_CAN_6.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL118   CE_CAN_6.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL119   CE_CAN_6.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL120   CE_CAN_6.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL121   CE_CAN_6.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL122   CE_CAN_6.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL123   CE_CAN_6.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL124   CE_CAN_6.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL125   CE_CAN_6.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL126   CE_CAN_6.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_6_ERFFEL127   CE_CAN_6.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_7_MCR         CE_CAN_7.MCR.R                /* Module Configuration */
#define CE_CAN_7_CTRL1       CE_CAN_7.CTRL1.R              /* Control 1 */
#define CE_CAN_7_TIMER       CE_CAN_7.TIMER.R              /* Free-Running Timer */
#define CE_CAN_7_RXMGMASK    CE_CAN_7.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_7_RX14MASK    CE_CAN_7.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_7_RX15MASK    CE_CAN_7.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_7_ECR         CE_CAN_7.ECR.R                /* Error Counter */
#define CE_CAN_7_ESR1        CE_CAN_7.ESR1.R               /* Error and Status 1 */
#define CE_CAN_7_IMASK2      CE_CAN_7.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_7_IMASK1      CE_CAN_7.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_7_IFLAG2      CE_CAN_7.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_7_IFLAG1      CE_CAN_7.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_7_CTRL2       CE_CAN_7.CTRL2.R              /* Control 2 */
#define CE_CAN_7_ESR2        CE_CAN_7.ESR2.R               /* Error and Status 2 */
#define CE_CAN_7_CRCR        CE_CAN_7.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_7_RXFGMASK    CE_CAN_7.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_7_RXFIR       CE_CAN_7.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_7_CBT         CE_CAN_7.CBT.R                /* CAN Bit Timing */
#define CE_CAN_7_IMASK4      CE_CAN_7.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_7_IMASK3      CE_CAN_7.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_7_IFLAG4      CE_CAN_7.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_7_IFLAG3      CE_CAN_7.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_7_CS0            CE_CAN_7.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_7_ID0            CE_CAN_7.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_7_B0             CE_CAN_7.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_7_H0             CE_CAN_7.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_7_W0             CE_CAN_7.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_7_CS1            CE_CAN_7.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_7_ID1            CE_CAN_7.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_7_B1             CE_CAN_7.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_7_H1             CE_CAN_7.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_7_W1             CE_CAN_7.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_7_CS2            CE_CAN_7.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_7_ID2            CE_CAN_7.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_7_B2             CE_CAN_7.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_7_H2             CE_CAN_7.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_7_W2             CE_CAN_7.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_7_CS3            CE_CAN_7.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_7_ID3            CE_CAN_7.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_7_B3             CE_CAN_7.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_7_H3             CE_CAN_7.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_7_W3             CE_CAN_7.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_7_CS4            CE_CAN_7.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_7_ID4            CE_CAN_7.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_7_B4             CE_CAN_7.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_7_H4             CE_CAN_7.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_7_W4             CE_CAN_7.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_7_CS5            CE_CAN_7.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_7_ID5            CE_CAN_7.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_7_B5             CE_CAN_7.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_7_H5             CE_CAN_7.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_7_W5             CE_CAN_7.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_7_CS6            CE_CAN_7.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_7_ID6            CE_CAN_7.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_7_B6             CE_CAN_7.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_7_H6             CE_CAN_7.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_7_W6             CE_CAN_7.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_7_CS7            CE_CAN_7.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_7_ID7            CE_CAN_7.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_7_B7             CE_CAN_7.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_7_H7             CE_CAN_7.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_7_W7             CE_CAN_7.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_7_CS8            CE_CAN_7.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_7_ID8            CE_CAN_7.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_7_B8             CE_CAN_7.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_7_H8             CE_CAN_7.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_7_W8             CE_CAN_7.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_7_CS9            CE_CAN_7.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_7_ID9            CE_CAN_7.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_7_B9             CE_CAN_7.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_7_H9             CE_CAN_7.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_7_W9             CE_CAN_7.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_7_CS10           CE_CAN_7.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_7_ID10           CE_CAN_7.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_7_B10            CE_CAN_7.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_7_H10            CE_CAN_7.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_7_W10            CE_CAN_7.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_7_CS11           CE_CAN_7.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_7_ID11           CE_CAN_7.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_7_B11            CE_CAN_7.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_7_H11            CE_CAN_7.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_7_W11            CE_CAN_7.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_7_CS12           CE_CAN_7.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_7_ID12           CE_CAN_7.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_7_B12            CE_CAN_7.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_7_H12            CE_CAN_7.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_7_W12            CE_CAN_7.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_7_CS13           CE_CAN_7.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_7_ID13           CE_CAN_7.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_7_B13            CE_CAN_7.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_7_H13            CE_CAN_7.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_7_W13            CE_CAN_7.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_7_CS14           CE_CAN_7.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_7_ID14           CE_CAN_7.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_7_B14            CE_CAN_7.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_7_H14            CE_CAN_7.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_7_W14            CE_CAN_7.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_7_CS15           CE_CAN_7.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_7_ID15           CE_CAN_7.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_7_B15            CE_CAN_7.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_7_H15            CE_CAN_7.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_7_W15            CE_CAN_7.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_7_CS16           CE_CAN_7.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_7_ID16           CE_CAN_7.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_7_B16            CE_CAN_7.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_7_H16            CE_CAN_7.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_7_W16            CE_CAN_7.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_7_CS17           CE_CAN_7.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_7_ID17           CE_CAN_7.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_7_B17            CE_CAN_7.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_7_H17            CE_CAN_7.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_7_W17            CE_CAN_7.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_7_CS18           CE_CAN_7.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_7_ID18           CE_CAN_7.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_7_B18            CE_CAN_7.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_7_H18            CE_CAN_7.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_7_W18            CE_CAN_7.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_7_CS19           CE_CAN_7.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_7_ID19           CE_CAN_7.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_7_B19            CE_CAN_7.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_7_H19            CE_CAN_7.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_7_W19            CE_CAN_7.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_7_CS20           CE_CAN_7.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_7_ID20           CE_CAN_7.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_7_B20            CE_CAN_7.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_7_H20            CE_CAN_7.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_7_W20            CE_CAN_7.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_7_CS21           CE_CAN_7.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_7_ID21           CE_CAN_7.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_7_B21            CE_CAN_7.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_7_H21            CE_CAN_7.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_7_W21            CE_CAN_7.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_7_CS22           CE_CAN_7.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_7_ID22           CE_CAN_7.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_7_B22            CE_CAN_7.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_7_H22            CE_CAN_7.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_7_W22            CE_CAN_7.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_7_CS23           CE_CAN_7.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_7_ID23           CE_CAN_7.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_7_B23            CE_CAN_7.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_7_H23            CE_CAN_7.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_7_W23            CE_CAN_7.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_7_CS24           CE_CAN_7.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_7_ID24           CE_CAN_7.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_7_B24            CE_CAN_7.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_7_H24            CE_CAN_7.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_7_W24            CE_CAN_7.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_7_CS25           CE_CAN_7.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_7_ID25           CE_CAN_7.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_7_B25            CE_CAN_7.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_7_H25            CE_CAN_7.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_7_W25            CE_CAN_7.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_7_CS26           CE_CAN_7.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_7_ID26           CE_CAN_7.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_7_B26            CE_CAN_7.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_7_H26            CE_CAN_7.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_7_W26            CE_CAN_7.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_7_CS27           CE_CAN_7.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_7_ID27           CE_CAN_7.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_7_B27            CE_CAN_7.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_7_H27            CE_CAN_7.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_7_W27            CE_CAN_7.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_7_CS28           CE_CAN_7.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_7_ID28           CE_CAN_7.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_7_B28            CE_CAN_7.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_7_H28            CE_CAN_7.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_7_W28            CE_CAN_7.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_7_CS29           CE_CAN_7.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_7_ID29           CE_CAN_7.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_7_B29            CE_CAN_7.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_7_H29            CE_CAN_7.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_7_W29            CE_CAN_7.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_7_CS30           CE_CAN_7.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_7_ID30           CE_CAN_7.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_7_B30            CE_CAN_7.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_7_H30            CE_CAN_7.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_7_W30            CE_CAN_7.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_7_CS31           CE_CAN_7.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_7_ID31           CE_CAN_7.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_7_B31            CE_CAN_7.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_7_H31            CE_CAN_7.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_7_W31            CE_CAN_7.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_7_CS32           CE_CAN_7.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_7_ID32           CE_CAN_7.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_7_B32            CE_CAN_7.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_7_H32            CE_CAN_7.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_7_W32            CE_CAN_7.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_7_CS33           CE_CAN_7.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_7_ID33           CE_CAN_7.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_7_B33            CE_CAN_7.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_7_H33            CE_CAN_7.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_7_W33            CE_CAN_7.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_7_CS34           CE_CAN_7.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_7_ID34           CE_CAN_7.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_7_B34            CE_CAN_7.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_7_H34            CE_CAN_7.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_7_W34            CE_CAN_7.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_7_CS35           CE_CAN_7.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_7_ID35           CE_CAN_7.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_7_B35            CE_CAN_7.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_7_H35            CE_CAN_7.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_7_W35            CE_CAN_7.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_7_CS36           CE_CAN_7.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_7_ID36           CE_CAN_7.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_7_B36            CE_CAN_7.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_7_H36            CE_CAN_7.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_7_W36            CE_CAN_7.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_7_CS37           CE_CAN_7.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_7_ID37           CE_CAN_7.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_7_B37            CE_CAN_7.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_7_H37            CE_CAN_7.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_7_W37            CE_CAN_7.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_7_CS38           CE_CAN_7.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_7_ID38           CE_CAN_7.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_7_B38            CE_CAN_7.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_7_H38            CE_CAN_7.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_7_W38            CE_CAN_7.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_7_CS39           CE_CAN_7.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_7_ID39           CE_CAN_7.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_7_B39            CE_CAN_7.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_7_H39            CE_CAN_7.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_7_W39            CE_CAN_7.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_7_CS40           CE_CAN_7.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_7_ID40           CE_CAN_7.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_7_B40            CE_CAN_7.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_7_H40            CE_CAN_7.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_7_W40            CE_CAN_7.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_7_CS41           CE_CAN_7.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_7_ID41           CE_CAN_7.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_7_B41            CE_CAN_7.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_7_H41            CE_CAN_7.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_7_W41            CE_CAN_7.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_7_CS42           CE_CAN_7.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_7_ID42           CE_CAN_7.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_7_B42            CE_CAN_7.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_7_H42            CE_CAN_7.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_7_W42            CE_CAN_7.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_7_CS43           CE_CAN_7.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_7_ID43           CE_CAN_7.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_7_B43            CE_CAN_7.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_7_H43            CE_CAN_7.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_7_W43            CE_CAN_7.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_7_CS44           CE_CAN_7.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_7_ID44           CE_CAN_7.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_7_B44            CE_CAN_7.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_7_H44            CE_CAN_7.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_7_W44            CE_CAN_7.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_7_CS45           CE_CAN_7.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_7_ID45           CE_CAN_7.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_7_B45            CE_CAN_7.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_7_H45            CE_CAN_7.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_7_W45            CE_CAN_7.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_7_CS46           CE_CAN_7.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_7_ID46           CE_CAN_7.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_7_B46            CE_CAN_7.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_7_H46            CE_CAN_7.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_7_W46            CE_CAN_7.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_7_CS47           CE_CAN_7.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_7_ID47           CE_CAN_7.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_7_B47            CE_CAN_7.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_7_H47            CE_CAN_7.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_7_W47            CE_CAN_7.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_7_CS48           CE_CAN_7.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_7_ID48           CE_CAN_7.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_7_B48            CE_CAN_7.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_7_H48            CE_CAN_7.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_7_W48            CE_CAN_7.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_7_CS49           CE_CAN_7.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_7_ID49           CE_CAN_7.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_7_B49            CE_CAN_7.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_7_H49            CE_CAN_7.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_7_W49            CE_CAN_7.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_7_CS50           CE_CAN_7.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_7_ID50           CE_CAN_7.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_7_B50            CE_CAN_7.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_7_H50            CE_CAN_7.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_7_W50            CE_CAN_7.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_7_CS51           CE_CAN_7.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_7_ID51           CE_CAN_7.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_7_B51            CE_CAN_7.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_7_H51            CE_CAN_7.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_7_W51            CE_CAN_7.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_7_CS52           CE_CAN_7.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_7_ID52           CE_CAN_7.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_7_B52            CE_CAN_7.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_7_H52            CE_CAN_7.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_7_W52            CE_CAN_7.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_7_CS53           CE_CAN_7.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_7_ID53           CE_CAN_7.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_7_B53            CE_CAN_7.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_7_H53            CE_CAN_7.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_7_W53            CE_CAN_7.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_7_CS54           CE_CAN_7.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_7_ID54           CE_CAN_7.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_7_B54            CE_CAN_7.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_7_H54            CE_CAN_7.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_7_W54            CE_CAN_7.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_7_CS55           CE_CAN_7.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_7_ID55           CE_CAN_7.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_7_B55            CE_CAN_7.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_7_H55            CE_CAN_7.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_7_W55            CE_CAN_7.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_7_CS56           CE_CAN_7.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_7_ID56           CE_CAN_7.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_7_B56            CE_CAN_7.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_7_H56            CE_CAN_7.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_7_W56            CE_CAN_7.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_7_CS57           CE_CAN_7.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_7_ID57           CE_CAN_7.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_7_B57            CE_CAN_7.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_7_H57            CE_CAN_7.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_7_W57            CE_CAN_7.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_7_CS58           CE_CAN_7.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_7_ID58           CE_CAN_7.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_7_B58            CE_CAN_7.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_7_H58            CE_CAN_7.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_7_W58            CE_CAN_7.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_7_CS59           CE_CAN_7.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_7_ID59           CE_CAN_7.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_7_B59            CE_CAN_7.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_7_H59            CE_CAN_7.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_7_W59            CE_CAN_7.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_7_CS60           CE_CAN_7.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_7_ID60           CE_CAN_7.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_7_B60            CE_CAN_7.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_7_H60            CE_CAN_7.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_7_W60            CE_CAN_7.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_7_CS61           CE_CAN_7.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_7_ID61           CE_CAN_7.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_7_B61            CE_CAN_7.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_7_H61            CE_CAN_7.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_7_W61            CE_CAN_7.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_7_CS62           CE_CAN_7.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_7_ID62           CE_CAN_7.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_7_B62            CE_CAN_7.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_7_H62            CE_CAN_7.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_7_W62            CE_CAN_7.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_7_CS63           CE_CAN_7.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_7_ID63           CE_CAN_7.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_7_B63            CE_CAN_7.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_7_H63            CE_CAN_7.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_7_W63            CE_CAN_7.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_7_CS64           CE_CAN_7.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_7_ID64           CE_CAN_7.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_7_B64            CE_CAN_7.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_7_H64            CE_CAN_7.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_7_W64            CE_CAN_7.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_7_CS65           CE_CAN_7.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_7_ID65           CE_CAN_7.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_7_B65            CE_CAN_7.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_7_H65            CE_CAN_7.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_7_W65            CE_CAN_7.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_7_CS66           CE_CAN_7.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_7_ID66           CE_CAN_7.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_7_B66            CE_CAN_7.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_7_H66            CE_CAN_7.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_7_W66            CE_CAN_7.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_7_CS67           CE_CAN_7.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_7_ID67           CE_CAN_7.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_7_B67            CE_CAN_7.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_7_H67            CE_CAN_7.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_7_W67            CE_CAN_7.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_7_CS68           CE_CAN_7.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_7_ID68           CE_CAN_7.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_7_B68            CE_CAN_7.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_7_H68            CE_CAN_7.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_7_W68            CE_CAN_7.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_7_CS69           CE_CAN_7.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_7_ID69           CE_CAN_7.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_7_B69            CE_CAN_7.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_7_H69            CE_CAN_7.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_7_W69            CE_CAN_7.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_7_CS70           CE_CAN_7.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_7_ID70           CE_CAN_7.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_7_B70            CE_CAN_7.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_7_H70            CE_CAN_7.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_7_W70            CE_CAN_7.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_7_CS71           CE_CAN_7.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_7_ID71           CE_CAN_7.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_7_B71            CE_CAN_7.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_7_H71            CE_CAN_7.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_7_W71            CE_CAN_7.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_7_CS72           CE_CAN_7.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_7_ID72           CE_CAN_7.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_7_B72            CE_CAN_7.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_7_H72            CE_CAN_7.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_7_W72            CE_CAN_7.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_7_CS73           CE_CAN_7.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_7_ID73           CE_CAN_7.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_7_B73            CE_CAN_7.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_7_H73            CE_CAN_7.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_7_W73            CE_CAN_7.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_7_CS74           CE_CAN_7.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_7_ID74           CE_CAN_7.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_7_B74            CE_CAN_7.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_7_H74            CE_CAN_7.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_7_W74            CE_CAN_7.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_7_CS75           CE_CAN_7.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_7_ID75           CE_CAN_7.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_7_B75            CE_CAN_7.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_7_H75            CE_CAN_7.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_7_W75            CE_CAN_7.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_7_CS76           CE_CAN_7.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_7_ID76           CE_CAN_7.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_7_B76            CE_CAN_7.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_7_H76            CE_CAN_7.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_7_W76            CE_CAN_7.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_7_CS77           CE_CAN_7.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_7_ID77           CE_CAN_7.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_7_B77            CE_CAN_7.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_7_H77            CE_CAN_7.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_7_W77            CE_CAN_7.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_7_CS78           CE_CAN_7.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_7_ID78           CE_CAN_7.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_7_B78            CE_CAN_7.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_7_H78            CE_CAN_7.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_7_W78            CE_CAN_7.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_7_CS79           CE_CAN_7.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_7_ID79           CE_CAN_7.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_7_B79            CE_CAN_7.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_7_H79            CE_CAN_7.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_7_W79            CE_CAN_7.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_7_CS80           CE_CAN_7.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_7_ID80           CE_CAN_7.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_7_B80            CE_CAN_7.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_7_H80            CE_CAN_7.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_7_W80            CE_CAN_7.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_7_CS81           CE_CAN_7.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_7_ID81           CE_CAN_7.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_7_B81            CE_CAN_7.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_7_H81            CE_CAN_7.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_7_W81            CE_CAN_7.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_7_CS82           CE_CAN_7.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_7_ID82           CE_CAN_7.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_7_B82            CE_CAN_7.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_7_H82            CE_CAN_7.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_7_W82            CE_CAN_7.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_7_CS83           CE_CAN_7.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_7_ID83           CE_CAN_7.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_7_B83            CE_CAN_7.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_7_H83            CE_CAN_7.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_7_W83            CE_CAN_7.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_7_CS84           CE_CAN_7.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_7_ID84           CE_CAN_7.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_7_B84            CE_CAN_7.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_7_H84            CE_CAN_7.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_7_W84            CE_CAN_7.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_7_CS85           CE_CAN_7.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_7_ID85           CE_CAN_7.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_7_B85            CE_CAN_7.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_7_H85            CE_CAN_7.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_7_W85            CE_CAN_7.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_7_CS86           CE_CAN_7.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_7_ID86           CE_CAN_7.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_7_B86            CE_CAN_7.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_7_H86            CE_CAN_7.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_7_W86            CE_CAN_7.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_7_CS87           CE_CAN_7.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_7_ID87           CE_CAN_7.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_7_B87            CE_CAN_7.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_7_H87            CE_CAN_7.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_7_W87            CE_CAN_7.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_7_CS88           CE_CAN_7.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_7_ID88           CE_CAN_7.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_7_B88            CE_CAN_7.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_7_H88            CE_CAN_7.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_7_W848           CE_CAN_7.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_7_CS89           CE_CAN_7.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_7_ID89           CE_CAN_7.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_7_B89            CE_CAN_7.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_7_H89            CE_CAN_7.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_7_W89            CE_CAN_7.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_7_CS90           CE_CAN_7.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_7_ID90           CE_CAN_7.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_7_B90            CE_CAN_7.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_7_H90            CE_CAN_7.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_7_W90            CE_CAN_7.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_7_CS91           CE_CAN_7.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_7_ID91           CE_CAN_7.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_7_B91            CE_CAN_7.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_7_H91            CE_CAN_7.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_7_W91            CE_CAN_7.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_7_CS92           CE_CAN_7.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_7_ID92           CE_CAN_7.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_7_B92            CE_CAN_7.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_7_H92            CE_CAN_7.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_7_W92            CE_CAN_7.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_7_CS93           CE_CAN_7.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_7_ID93           CE_CAN_7.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_7_B93            CE_CAN_7.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_7_H93            CE_CAN_7.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_7_W93            CE_CAN_7.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_7_CS94           CE_CAN_7.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_7_ID94           CE_CAN_7.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_7_B94            CE_CAN_7.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_7_H94            CE_CAN_7.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_7_W94            CE_CAN_7.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_7_CS95           CE_CAN_7.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_7_ID95           CE_CAN_7.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_7_B95            CE_CAN_7.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_7_H95            CE_CAN_7.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_7_W95            CE_CAN_7.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_7_CS96           CE_CAN_7.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_7_ID96           CE_CAN_7.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_7_B96            CE_CAN_7.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_7_H96            CE_CAN_7.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_7_W96            CE_CAN_7.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_7_CS97           CE_CAN_7.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_7_ID97           CE_CAN_7.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_7_B97            CE_CAN_7.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_7_H97            CE_CAN_7.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_7_W97            CE_CAN_7.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_7_CS98           CE_CAN_7.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_7_ID98           CE_CAN_7.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_7_B98            CE_CAN_7.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_7_H98            CE_CAN_7.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_7_W98            CE_CAN_7.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_7_CS99           CE_CAN_7.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_7_ID99           CE_CAN_7.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_7_B99            CE_CAN_7.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_7_H99            CE_CAN_7.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_7_W99            CE_CAN_7.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_7_CS100          CE_CAN_7.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_7_ID100          CE_CAN_7.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_7_B100           CE_CAN_7.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_7_H100           CE_CAN_7.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_7_W100           CE_CAN_7.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_7_CS101          CE_CAN_7.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_7_ID101          CE_CAN_7.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_7_B101           CE_CAN_7.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_7_H101           CE_CAN_7.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_7_W101           CE_CAN_7.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_7_CS102          CE_CAN_7.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_7_ID102          CE_CAN_7.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_7_B102           CE_CAN_7.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_7_H102           CE_CAN_7.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_7_W102           CE_CAN_7.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_7_CS103          CE_CAN_7.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_7_ID103          CE_CAN_7.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_7_B103           CE_CAN_7.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_7_H103           CE_CAN_7.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_7_W103           CE_CAN_7.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_7_CS104          CE_CAN_7.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_7_ID104          CE_CAN_7.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_7_B104           CE_CAN_7.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_7_H104           CE_CAN_7.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_7_W104           CE_CAN_7.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_7_CS105          CE_CAN_7.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_7_ID105          CE_CAN_7.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_7_B105           CE_CAN_7.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_7_H105           CE_CAN_7.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_7_W105           CE_CAN_7.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_7_CS106          CE_CAN_7.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_7_ID106          CE_CAN_7.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_7_B106           CE_CAN_7.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_7_H106           CE_CAN_7.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_7_W106           CE_CAN_7.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_7_CS107          CE_CAN_7.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_7_ID107          CE_CAN_7.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_7_B107           CE_CAN_7.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_7_H107           CE_CAN_7.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_7_W107           CE_CAN_7.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_7_CS108          CE_CAN_7.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_7_ID108          CE_CAN_7.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_7_B108           CE_CAN_7.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_7_H108           CE_CAN_7.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_7_W108           CE_CAN_7.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_7_CS109          CE_CAN_7.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_7_ID109          CE_CAN_7.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_7_B109           CE_CAN_7.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_7_H109           CE_CAN_7.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_7_W109           CE_CAN_7.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_7_CS110          CE_CAN_7.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_7_ID110          CE_CAN_7.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_7_B110           CE_CAN_7.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_7_H110           CE_CAN_7.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_7_W110           CE_CAN_7.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_7_CS111          CE_CAN_7.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_7_ID111          CE_CAN_7.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_7_B111           CE_CAN_7.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_7_H111           CE_CAN_7.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_7_W111           CE_CAN_7.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_7_CS112          CE_CAN_7.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_7_ID112          CE_CAN_7.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_7_B112           CE_CAN_7.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_7_H112           CE_CAN_7.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_7_W112           CE_CAN_7.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_7_CS113          CE_CAN_7.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_7_ID113          CE_CAN_7.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_7_B113           CE_CAN_7.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_7_H113           CE_CAN_7.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_7_W113           CE_CAN_7.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_7_CS114          CE_CAN_7.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_7_ID114          CE_CAN_7.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_7_B114           CE_CAN_7.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_7_H114           CE_CAN_7.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_7_W114           CE_CAN_7.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_7_CS115          CE_CAN_7.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_7_ID115          CE_CAN_7.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_7_B115           CE_CAN_7.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_7_H115           CE_CAN_7.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_7_W115           CE_CAN_7.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_7_CS116          CE_CAN_7.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_7_ID116          CE_CAN_7.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_7_B116           CE_CAN_7.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_7_H116           CE_CAN_7.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_7_W116           CE_CAN_7.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_7_CS117          CE_CAN_7.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_7_ID117          CE_CAN_7.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_7_B117           CE_CAN_7.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_7_H117           CE_CAN_7.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_7_W117           CE_CAN_7.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_7_CS118          CE_CAN_7.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_7_ID118          CE_CAN_7.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_7_B118           CE_CAN_7.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_7_H118           CE_CAN_7.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_7_W118           CE_CAN_7.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_7_CS119          CE_CAN_7.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_7_ID119          CE_CAN_7.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_7_B119           CE_CAN_7.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_7_H119           CE_CAN_7.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_7_W119           CE_CAN_7.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_7_CS120          CE_CAN_7.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_7_ID120          CE_CAN_7.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_7_B120           CE_CAN_7.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_7_H120           CE_CAN_7.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_7_W120           CE_CAN_7.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_7_CS121          CE_CAN_7.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_7_ID121          CE_CAN_7.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_7_B121           CE_CAN_7.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_7_H121           CE_CAN_7.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_7_W121           CE_CAN_7.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_7_CS122          CE_CAN_7.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_7_ID122          CE_CAN_7.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_7_B122           CE_CAN_7.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_7_H122           CE_CAN_7.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_7_W122           CE_CAN_7.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_7_CS123          CE_CAN_7.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_7_ID123          CE_CAN_7.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_7_B123           CE_CAN_7.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_7_H123           CE_CAN_7.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_7_W123           CE_CAN_7.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_7_CS124          CE_CAN_7.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_7_ID124          CE_CAN_7.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_7_B124           CE_CAN_7.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_7_H124           CE_CAN_7.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_7_W124           CE_CAN_7.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_7_CS125          CE_CAN_7.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_7_ID125          CE_CAN_7.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_7_B125           CE_CAN_7.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_7_H125           CE_CAN_7.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_7_W125           CE_CAN_7.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_7_CS126          CE_CAN_7.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_7_ID126          CE_CAN_7.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_7_B126           CE_CAN_7.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_7_H126           CE_CAN_7.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_7_W126           CE_CAN_7.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_7_CS127          CE_CAN_7.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_7_ID127          CE_CAN_7.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_7_B127           CE_CAN_7.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_7_H127           CE_CAN_7.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_7_W127           CE_CAN_7.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_7_RXIMR0      CE_CAN_7.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR1      CE_CAN_7.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR2      CE_CAN_7.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR3      CE_CAN_7.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR4      CE_CAN_7.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR5      CE_CAN_7.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR6      CE_CAN_7.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR7      CE_CAN_7.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR8      CE_CAN_7.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR9      CE_CAN_7.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_7_RXIMR10     CE_CAN_7.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR11     CE_CAN_7.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR12     CE_CAN_7.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR13     CE_CAN_7.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR14     CE_CAN_7.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR15     CE_CAN_7.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR16     CE_CAN_7.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR17     CE_CAN_7.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR18     CE_CAN_7.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR19     CE_CAN_7.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR20     CE_CAN_7.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR21     CE_CAN_7.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR22     CE_CAN_7.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR23     CE_CAN_7.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR24     CE_CAN_7.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR25     CE_CAN_7.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR26     CE_CAN_7.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR27     CE_CAN_7.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR28     CE_CAN_7.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR29     CE_CAN_7.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR30     CE_CAN_7.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR31     CE_CAN_7.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR32     CE_CAN_7.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR33     CE_CAN_7.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR34     CE_CAN_7.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR35     CE_CAN_7.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR36     CE_CAN_7.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR37     CE_CAN_7.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR38     CE_CAN_7.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR39     CE_CAN_7.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR40     CE_CAN_7.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR41     CE_CAN_7.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR42     CE_CAN_7.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR43     CE_CAN_7.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR44     CE_CAN_7.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR45     CE_CAN_7.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR46     CE_CAN_7.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR47     CE_CAN_7.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR48     CE_CAN_7.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR49     CE_CAN_7.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR50     CE_CAN_7.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR51     CE_CAN_7.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR52     CE_CAN_7.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR53     CE_CAN_7.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR54     CE_CAN_7.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR55     CE_CAN_7.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR56     CE_CAN_7.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR57     CE_CAN_7.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR58     CE_CAN_7.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR59     CE_CAN_7.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR60     CE_CAN_7.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR61     CE_CAN_7.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR62     CE_CAN_7.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR63     CE_CAN_7.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR64     CE_CAN_7.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR65     CE_CAN_7.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR66     CE_CAN_7.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR67     CE_CAN_7.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR68     CE_CAN_7.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR69     CE_CAN_7.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR70     CE_CAN_7.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR71     CE_CAN_7.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR72     CE_CAN_7.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR73     CE_CAN_7.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR74     CE_CAN_7.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR75     CE_CAN_7.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR76     CE_CAN_7.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR77     CE_CAN_7.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR78     CE_CAN_7.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR79     CE_CAN_7.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR80     CE_CAN_7.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR81     CE_CAN_7.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR82     CE_CAN_7.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR83     CE_CAN_7.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR84     CE_CAN_7.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR85     CE_CAN_7.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR86     CE_CAN_7.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR87     CE_CAN_7.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR88     CE_CAN_7.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR89     CE_CAN_7.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR90     CE_CAN_7.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR91     CE_CAN_7.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR92     CE_CAN_7.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR93     CE_CAN_7.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR94     CE_CAN_7.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR95     CE_CAN_7.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR96     CE_CAN_7.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR97     CE_CAN_7.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR98     CE_CAN_7.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR99     CE_CAN_7.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_7_RXIMR100    CE_CAN_7.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR101    CE_CAN_7.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR102    CE_CAN_7.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR103    CE_CAN_7.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR104    CE_CAN_7.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR105    CE_CAN_7.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR106    CE_CAN_7.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR107    CE_CAN_7.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR108    CE_CAN_7.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR109    CE_CAN_7.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR110    CE_CAN_7.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR111    CE_CAN_7.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR112    CE_CAN_7.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR113    CE_CAN_7.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR114    CE_CAN_7.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR115    CE_CAN_7.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR116    CE_CAN_7.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR117    CE_CAN_7.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR118    CE_CAN_7.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR119    CE_CAN_7.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR120    CE_CAN_7.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR121    CE_CAN_7.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR122    CE_CAN_7.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR123    CE_CAN_7.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR124    CE_CAN_7.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR125    CE_CAN_7.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR126    CE_CAN_7.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_7_RXIMR127    CE_CAN_7.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_7_MECR        CE_CAN_7.MECR.R               /* Memory Error Control */
#define CE_CAN_7_ERRIAR      CE_CAN_7.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_7_ERRIDPR     CE_CAN_7.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_7_ERRIPPR     CE_CAN_7.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_7_RERRAR      CE_CAN_7.RERRAR.R             /* Error Report Address */
#define CE_CAN_7_RERRDR      CE_CAN_7.RERRDR.R             /* Error Report Data */
#define CE_CAN_7_RERRSYNR    CE_CAN_7.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_7_ERRSR       CE_CAN_7.ERRSR.R              /* Error Status */
#define CE_CAN_7_EPRS        CE_CAN_7.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_7_ENCBT       CE_CAN_7.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_7_EDCBT       CE_CAN_7.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_7_ETDC        CE_CAN_7.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_7_FDCTRL      CE_CAN_7.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_7_FDCBT       CE_CAN_7.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_7_FDCRC       CE_CAN_7.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_7_ERFCR       CE_CAN_7.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_7_ERFIER      CE_CAN_7.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_7_ERFSR       CE_CAN_7.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_7_HR_TIME_STAMP0 CE_CAN_7.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP1 CE_CAN_7.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP2 CE_CAN_7.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP3 CE_CAN_7.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP4 CE_CAN_7.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP5 CE_CAN_7.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP6 CE_CAN_7.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP7 CE_CAN_7.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP8 CE_CAN_7.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP9 CE_CAN_7.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP10 CE_CAN_7.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP11 CE_CAN_7.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP12 CE_CAN_7.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP13 CE_CAN_7.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP14 CE_CAN_7.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP15 CE_CAN_7.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP16 CE_CAN_7.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP17 CE_CAN_7.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP18 CE_CAN_7.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP19 CE_CAN_7.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP20 CE_CAN_7.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP21 CE_CAN_7.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP22 CE_CAN_7.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP23 CE_CAN_7.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP24 CE_CAN_7.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP25 CE_CAN_7.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP26 CE_CAN_7.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP27 CE_CAN_7.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP28 CE_CAN_7.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP29 CE_CAN_7.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP30 CE_CAN_7.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP31 CE_CAN_7.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP32 CE_CAN_7.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP33 CE_CAN_7.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP34 CE_CAN_7.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP35 CE_CAN_7.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP36 CE_CAN_7.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP37 CE_CAN_7.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP38 CE_CAN_7.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP39 CE_CAN_7.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP40 CE_CAN_7.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP41 CE_CAN_7.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP42 CE_CAN_7.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP43 CE_CAN_7.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP44 CE_CAN_7.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP45 CE_CAN_7.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP46 CE_CAN_7.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP47 CE_CAN_7.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP48 CE_CAN_7.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP49 CE_CAN_7.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP50 CE_CAN_7.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP51 CE_CAN_7.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP52 CE_CAN_7.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP53 CE_CAN_7.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP54 CE_CAN_7.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP55 CE_CAN_7.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP56 CE_CAN_7.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP57 CE_CAN_7.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP58 CE_CAN_7.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP59 CE_CAN_7.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP60 CE_CAN_7.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP61 CE_CAN_7.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP62 CE_CAN_7.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP63 CE_CAN_7.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP64 CE_CAN_7.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP65 CE_CAN_7.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP66 CE_CAN_7.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP67 CE_CAN_7.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP68 CE_CAN_7.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP69 CE_CAN_7.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP70 CE_CAN_7.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP71 CE_CAN_7.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP72 CE_CAN_7.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP73 CE_CAN_7.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP74 CE_CAN_7.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP75 CE_CAN_7.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP76 CE_CAN_7.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP77 CE_CAN_7.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP78 CE_CAN_7.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP79 CE_CAN_7.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP80 CE_CAN_7.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP81 CE_CAN_7.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP82 CE_CAN_7.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP83 CE_CAN_7.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP84 CE_CAN_7.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP85 CE_CAN_7.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP86 CE_CAN_7.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP87 CE_CAN_7.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP88 CE_CAN_7.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP89 CE_CAN_7.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP90 CE_CAN_7.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP91 CE_CAN_7.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP92 CE_CAN_7.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP93 CE_CAN_7.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP94 CE_CAN_7.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP95 CE_CAN_7.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP96 CE_CAN_7.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP97 CE_CAN_7.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP98 CE_CAN_7.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP99 CE_CAN_7.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP100 CE_CAN_7.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP101 CE_CAN_7.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP102 CE_CAN_7.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP103 CE_CAN_7.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP104 CE_CAN_7.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP105 CE_CAN_7.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP106 CE_CAN_7.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP107 CE_CAN_7.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP108 CE_CAN_7.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP109 CE_CAN_7.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP110 CE_CAN_7.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP111 CE_CAN_7.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP112 CE_CAN_7.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP113 CE_CAN_7.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP114 CE_CAN_7.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP115 CE_CAN_7.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP116 CE_CAN_7.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP117 CE_CAN_7.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP118 CE_CAN_7.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP119 CE_CAN_7.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP120 CE_CAN_7.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP121 CE_CAN_7.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP122 CE_CAN_7.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP123 CE_CAN_7.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP124 CE_CAN_7.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP125 CE_CAN_7.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP126 CE_CAN_7.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_7_HR_TIME_STAMP127 CE_CAN_7.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_7_ERFFEL0     CE_CAN_7.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL1     CE_CAN_7.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL2     CE_CAN_7.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL3     CE_CAN_7.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL4     CE_CAN_7.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL5     CE_CAN_7.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL6     CE_CAN_7.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL7     CE_CAN_7.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL8     CE_CAN_7.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL9     CE_CAN_7.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL10    CE_CAN_7.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL11    CE_CAN_7.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL12    CE_CAN_7.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL13    CE_CAN_7.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL14    CE_CAN_7.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL15    CE_CAN_7.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL16    CE_CAN_7.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL17    CE_CAN_7.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL18    CE_CAN_7.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL19    CE_CAN_7.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL20    CE_CAN_7.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL21    CE_CAN_7.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL22    CE_CAN_7.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL23    CE_CAN_7.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL24    CE_CAN_7.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL25    CE_CAN_7.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL26    CE_CAN_7.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL27    CE_CAN_7.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL28    CE_CAN_7.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL29    CE_CAN_7.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL30    CE_CAN_7.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL31    CE_CAN_7.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL32    CE_CAN_7.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL33    CE_CAN_7.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL34    CE_CAN_7.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL35    CE_CAN_7.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL36    CE_CAN_7.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL37    CE_CAN_7.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL38    CE_CAN_7.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL39    CE_CAN_7.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL40    CE_CAN_7.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL41    CE_CAN_7.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL42    CE_CAN_7.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL43    CE_CAN_7.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL44    CE_CAN_7.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL45    CE_CAN_7.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL46    CE_CAN_7.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL47    CE_CAN_7.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL48    CE_CAN_7.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL49    CE_CAN_7.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL50    CE_CAN_7.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL51    CE_CAN_7.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL52    CE_CAN_7.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL53    CE_CAN_7.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL54    CE_CAN_7.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL55    CE_CAN_7.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL56    CE_CAN_7.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL57    CE_CAN_7.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL58    CE_CAN_7.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL59    CE_CAN_7.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL60    CE_CAN_7.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL61    CE_CAN_7.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL62    CE_CAN_7.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL63    CE_CAN_7.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL64    CE_CAN_7.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL65    CE_CAN_7.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL66    CE_CAN_7.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL67    CE_CAN_7.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL68    CE_CAN_7.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL69    CE_CAN_7.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL70    CE_CAN_7.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL71    CE_CAN_7.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL72    CE_CAN_7.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL73    CE_CAN_7.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL74    CE_CAN_7.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL75    CE_CAN_7.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL76    CE_CAN_7.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL77    CE_CAN_7.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL78    CE_CAN_7.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL79    CE_CAN_7.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL80    CE_CAN_7.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL81    CE_CAN_7.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL82    CE_CAN_7.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL83    CE_CAN_7.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL84    CE_CAN_7.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL85    CE_CAN_7.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL86    CE_CAN_7.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL87    CE_CAN_7.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL88    CE_CAN_7.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL89    CE_CAN_7.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL90    CE_CAN_7.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL91    CE_CAN_7.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL92    CE_CAN_7.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL93    CE_CAN_7.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL94    CE_CAN_7.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL95    CE_CAN_7.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL96    CE_CAN_7.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL97    CE_CAN_7.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL98    CE_CAN_7.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL99    CE_CAN_7.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL100   CE_CAN_7.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL101   CE_CAN_7.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL102   CE_CAN_7.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL103   CE_CAN_7.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL104   CE_CAN_7.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL105   CE_CAN_7.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL106   CE_CAN_7.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL107   CE_CAN_7.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL108   CE_CAN_7.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL109   CE_CAN_7.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL110   CE_CAN_7.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL111   CE_CAN_7.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL112   CE_CAN_7.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL113   CE_CAN_7.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL114   CE_CAN_7.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL115   CE_CAN_7.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL116   CE_CAN_7.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL117   CE_CAN_7.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL118   CE_CAN_7.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL119   CE_CAN_7.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL120   CE_CAN_7.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL121   CE_CAN_7.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL122   CE_CAN_7.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL123   CE_CAN_7.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL124   CE_CAN_7.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL125   CE_CAN_7.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL126   CE_CAN_7.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_7_ERFFEL127   CE_CAN_7.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_8_MCR         CE_CAN_8.MCR.R                /* Module Configuration */
#define CE_CAN_8_CTRL1       CE_CAN_8.CTRL1.R              /* Control 1 */
#define CE_CAN_8_TIMER       CE_CAN_8.TIMER.R              /* Free-Running Timer */
#define CE_CAN_8_RXMGMASK    CE_CAN_8.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_8_RX14MASK    CE_CAN_8.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_8_RX15MASK    CE_CAN_8.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_8_ECR         CE_CAN_8.ECR.R                /* Error Counter */
#define CE_CAN_8_ESR1        CE_CAN_8.ESR1.R               /* Error and Status 1 */
#define CE_CAN_8_IMASK2      CE_CAN_8.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_8_IMASK1      CE_CAN_8.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_8_IFLAG2      CE_CAN_8.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_8_IFLAG1      CE_CAN_8.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_8_CTRL2       CE_CAN_8.CTRL2.R              /* Control 2 */
#define CE_CAN_8_ESR2        CE_CAN_8.ESR2.R               /* Error and Status 2 */
#define CE_CAN_8_CRCR        CE_CAN_8.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_8_RXFGMASK    CE_CAN_8.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_8_RXFIR       CE_CAN_8.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_8_CBT         CE_CAN_8.CBT.R                /* CAN Bit Timing */
#define CE_CAN_8_IMASK4      CE_CAN_8.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_8_IMASK3      CE_CAN_8.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_8_IFLAG4      CE_CAN_8.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_8_IFLAG3      CE_CAN_8.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_8_CS0            CE_CAN_8.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_8_ID0            CE_CAN_8.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_8_B0             CE_CAN_8.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_8_H0             CE_CAN_8.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_8_W0             CE_CAN_8.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_8_CS1            CE_CAN_8.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_8_ID1            CE_CAN_8.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_8_B1             CE_CAN_8.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_8_H1             CE_CAN_8.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_8_W1             CE_CAN_8.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_8_CS2            CE_CAN_8.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_8_ID2            CE_CAN_8.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_8_B2             CE_CAN_8.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_8_H2             CE_CAN_8.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_8_W2             CE_CAN_8.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_8_CS3            CE_CAN_8.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_8_ID3            CE_CAN_8.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_8_B3             CE_CAN_8.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_8_H3             CE_CAN_8.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_8_W3             CE_CAN_8.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_8_CS4            CE_CAN_8.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_8_ID4            CE_CAN_8.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_8_B4             CE_CAN_8.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_8_H4             CE_CAN_8.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_8_W4             CE_CAN_8.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_8_CS5            CE_CAN_8.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_8_ID5            CE_CAN_8.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_8_B5             CE_CAN_8.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_8_H5             CE_CAN_8.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_8_W5             CE_CAN_8.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_8_CS6            CE_CAN_8.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_8_ID6            CE_CAN_8.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_8_B6             CE_CAN_8.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_8_H6             CE_CAN_8.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_8_W6             CE_CAN_8.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_8_CS7            CE_CAN_8.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_8_ID7            CE_CAN_8.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_8_B7             CE_CAN_8.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_8_H7             CE_CAN_8.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_8_W7             CE_CAN_8.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_8_CS8            CE_CAN_8.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_8_ID8            CE_CAN_8.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_8_B8             CE_CAN_8.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_8_H8             CE_CAN_8.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_8_W8             CE_CAN_8.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_8_CS9            CE_CAN_8.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_8_ID9            CE_CAN_8.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_8_B9             CE_CAN_8.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_8_H9             CE_CAN_8.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_8_W9             CE_CAN_8.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_8_CS10           CE_CAN_8.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_8_ID10           CE_CAN_8.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_8_B10            CE_CAN_8.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_8_H10            CE_CAN_8.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_8_W10            CE_CAN_8.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_8_CS11           CE_CAN_8.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_8_ID11           CE_CAN_8.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_8_B11            CE_CAN_8.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_8_H11            CE_CAN_8.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_8_W11            CE_CAN_8.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_8_CS12           CE_CAN_8.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_8_ID12           CE_CAN_8.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_8_B12            CE_CAN_8.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_8_H12            CE_CAN_8.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_8_W12            CE_CAN_8.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_8_CS13           CE_CAN_8.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_8_ID13           CE_CAN_8.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_8_B13            CE_CAN_8.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_8_H13            CE_CAN_8.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_8_W13            CE_CAN_8.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_8_CS14           CE_CAN_8.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_8_ID14           CE_CAN_8.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_8_B14            CE_CAN_8.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_8_H14            CE_CAN_8.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_8_W14            CE_CAN_8.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_8_CS15           CE_CAN_8.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_8_ID15           CE_CAN_8.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_8_B15            CE_CAN_8.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_8_H15            CE_CAN_8.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_8_W15            CE_CAN_8.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_8_CS16           CE_CAN_8.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_8_ID16           CE_CAN_8.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_8_B16            CE_CAN_8.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_8_H16            CE_CAN_8.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_8_W16            CE_CAN_8.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_8_CS17           CE_CAN_8.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_8_ID17           CE_CAN_8.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_8_B17            CE_CAN_8.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_8_H17            CE_CAN_8.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_8_W17            CE_CAN_8.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_8_CS18           CE_CAN_8.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_8_ID18           CE_CAN_8.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_8_B18            CE_CAN_8.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_8_H18            CE_CAN_8.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_8_W18            CE_CAN_8.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_8_CS19           CE_CAN_8.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_8_ID19           CE_CAN_8.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_8_B19            CE_CAN_8.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_8_H19            CE_CAN_8.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_8_W19            CE_CAN_8.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_8_CS20           CE_CAN_8.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_8_ID20           CE_CAN_8.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_8_B20            CE_CAN_8.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_8_H20            CE_CAN_8.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_8_W20            CE_CAN_8.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_8_CS21           CE_CAN_8.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_8_ID21           CE_CAN_8.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_8_B21            CE_CAN_8.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_8_H21            CE_CAN_8.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_8_W21            CE_CAN_8.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_8_CS22           CE_CAN_8.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_8_ID22           CE_CAN_8.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_8_B22            CE_CAN_8.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_8_H22            CE_CAN_8.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_8_W22            CE_CAN_8.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_8_CS23           CE_CAN_8.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_8_ID23           CE_CAN_8.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_8_B23            CE_CAN_8.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_8_H23            CE_CAN_8.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_8_W23            CE_CAN_8.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_8_CS24           CE_CAN_8.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_8_ID24           CE_CAN_8.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_8_B24            CE_CAN_8.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_8_H24            CE_CAN_8.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_8_W24            CE_CAN_8.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_8_CS25           CE_CAN_8.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_8_ID25           CE_CAN_8.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_8_B25            CE_CAN_8.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_8_H25            CE_CAN_8.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_8_W25            CE_CAN_8.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_8_CS26           CE_CAN_8.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_8_ID26           CE_CAN_8.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_8_B26            CE_CAN_8.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_8_H26            CE_CAN_8.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_8_W26            CE_CAN_8.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_8_CS27           CE_CAN_8.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_8_ID27           CE_CAN_8.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_8_B27            CE_CAN_8.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_8_H27            CE_CAN_8.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_8_W27            CE_CAN_8.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_8_CS28           CE_CAN_8.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_8_ID28           CE_CAN_8.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_8_B28            CE_CAN_8.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_8_H28            CE_CAN_8.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_8_W28            CE_CAN_8.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_8_CS29           CE_CAN_8.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_8_ID29           CE_CAN_8.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_8_B29            CE_CAN_8.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_8_H29            CE_CAN_8.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_8_W29            CE_CAN_8.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_8_CS30           CE_CAN_8.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_8_ID30           CE_CAN_8.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_8_B30            CE_CAN_8.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_8_H30            CE_CAN_8.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_8_W30            CE_CAN_8.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_8_CS31           CE_CAN_8.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_8_ID31           CE_CAN_8.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_8_B31            CE_CAN_8.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_8_H31            CE_CAN_8.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_8_W31            CE_CAN_8.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_8_CS32           CE_CAN_8.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_8_ID32           CE_CAN_8.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_8_B32            CE_CAN_8.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_8_H32            CE_CAN_8.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_8_W32            CE_CAN_8.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_8_CS33           CE_CAN_8.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_8_ID33           CE_CAN_8.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_8_B33            CE_CAN_8.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_8_H33            CE_CAN_8.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_8_W33            CE_CAN_8.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_8_CS34           CE_CAN_8.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_8_ID34           CE_CAN_8.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_8_B34            CE_CAN_8.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_8_H34            CE_CAN_8.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_8_W34            CE_CAN_8.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_8_CS35           CE_CAN_8.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_8_ID35           CE_CAN_8.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_8_B35            CE_CAN_8.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_8_H35            CE_CAN_8.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_8_W35            CE_CAN_8.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_8_CS36           CE_CAN_8.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_8_ID36           CE_CAN_8.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_8_B36            CE_CAN_8.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_8_H36            CE_CAN_8.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_8_W36            CE_CAN_8.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_8_CS37           CE_CAN_8.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_8_ID37           CE_CAN_8.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_8_B37            CE_CAN_8.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_8_H37            CE_CAN_8.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_8_W37            CE_CAN_8.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_8_CS38           CE_CAN_8.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_8_ID38           CE_CAN_8.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_8_B38            CE_CAN_8.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_8_H38            CE_CAN_8.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_8_W38            CE_CAN_8.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_8_CS39           CE_CAN_8.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_8_ID39           CE_CAN_8.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_8_B39            CE_CAN_8.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_8_H39            CE_CAN_8.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_8_W39            CE_CAN_8.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_8_CS40           CE_CAN_8.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_8_ID40           CE_CAN_8.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_8_B40            CE_CAN_8.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_8_H40            CE_CAN_8.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_8_W40            CE_CAN_8.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_8_CS41           CE_CAN_8.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_8_ID41           CE_CAN_8.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_8_B41            CE_CAN_8.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_8_H41            CE_CAN_8.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_8_W41            CE_CAN_8.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_8_CS42           CE_CAN_8.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_8_ID42           CE_CAN_8.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_8_B42            CE_CAN_8.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_8_H42            CE_CAN_8.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_8_W42            CE_CAN_8.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_8_CS43           CE_CAN_8.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_8_ID43           CE_CAN_8.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_8_B43            CE_CAN_8.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_8_H43            CE_CAN_8.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_8_W43            CE_CAN_8.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_8_CS44           CE_CAN_8.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_8_ID44           CE_CAN_8.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_8_B44            CE_CAN_8.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_8_H44            CE_CAN_8.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_8_W44            CE_CAN_8.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_8_CS45           CE_CAN_8.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_8_ID45           CE_CAN_8.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_8_B45            CE_CAN_8.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_8_H45            CE_CAN_8.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_8_W45            CE_CAN_8.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_8_CS46           CE_CAN_8.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_8_ID46           CE_CAN_8.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_8_B46            CE_CAN_8.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_8_H46            CE_CAN_8.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_8_W46            CE_CAN_8.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_8_CS47           CE_CAN_8.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_8_ID47           CE_CAN_8.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_8_B47            CE_CAN_8.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_8_H47            CE_CAN_8.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_8_W47            CE_CAN_8.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_8_CS48           CE_CAN_8.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_8_ID48           CE_CAN_8.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_8_B48            CE_CAN_8.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_8_H48            CE_CAN_8.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_8_W48            CE_CAN_8.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_8_CS49           CE_CAN_8.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_8_ID49           CE_CAN_8.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_8_B49            CE_CAN_8.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_8_H49            CE_CAN_8.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_8_W49            CE_CAN_8.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_8_CS50           CE_CAN_8.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_8_ID50           CE_CAN_8.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_8_B50            CE_CAN_8.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_8_H50            CE_CAN_8.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_8_W50            CE_CAN_8.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_8_CS51           CE_CAN_8.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_8_ID51           CE_CAN_8.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_8_B51            CE_CAN_8.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_8_H51            CE_CAN_8.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_8_W51            CE_CAN_8.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_8_CS52           CE_CAN_8.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_8_ID52           CE_CAN_8.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_8_B52            CE_CAN_8.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_8_H52            CE_CAN_8.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_8_W52            CE_CAN_8.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_8_CS53           CE_CAN_8.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_8_ID53           CE_CAN_8.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_8_B53            CE_CAN_8.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_8_H53            CE_CAN_8.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_8_W53            CE_CAN_8.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_8_CS54           CE_CAN_8.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_8_ID54           CE_CAN_8.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_8_B54            CE_CAN_8.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_8_H54            CE_CAN_8.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_8_W54            CE_CAN_8.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_8_CS55           CE_CAN_8.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_8_ID55           CE_CAN_8.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_8_B55            CE_CAN_8.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_8_H55            CE_CAN_8.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_8_W55            CE_CAN_8.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_8_CS56           CE_CAN_8.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_8_ID56           CE_CAN_8.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_8_B56            CE_CAN_8.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_8_H56            CE_CAN_8.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_8_W56            CE_CAN_8.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_8_CS57           CE_CAN_8.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_8_ID57           CE_CAN_8.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_8_B57            CE_CAN_8.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_8_H57            CE_CAN_8.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_8_W57            CE_CAN_8.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_8_CS58           CE_CAN_8.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_8_ID58           CE_CAN_8.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_8_B58            CE_CAN_8.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_8_H58            CE_CAN_8.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_8_W58            CE_CAN_8.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_8_CS59           CE_CAN_8.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_8_ID59           CE_CAN_8.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_8_B59            CE_CAN_8.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_8_H59            CE_CAN_8.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_8_W59            CE_CAN_8.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_8_CS60           CE_CAN_8.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_8_ID60           CE_CAN_8.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_8_B60            CE_CAN_8.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_8_H60            CE_CAN_8.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_8_W60            CE_CAN_8.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_8_CS61           CE_CAN_8.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_8_ID61           CE_CAN_8.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_8_B61            CE_CAN_8.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_8_H61            CE_CAN_8.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_8_W61            CE_CAN_8.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_8_CS62           CE_CAN_8.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_8_ID62           CE_CAN_8.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_8_B62            CE_CAN_8.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_8_H62            CE_CAN_8.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_8_W62            CE_CAN_8.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_8_CS63           CE_CAN_8.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_8_ID63           CE_CAN_8.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_8_B63            CE_CAN_8.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_8_H63            CE_CAN_8.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_8_W63            CE_CAN_8.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_8_CS64           CE_CAN_8.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_8_ID64           CE_CAN_8.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_8_B64            CE_CAN_8.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_8_H64            CE_CAN_8.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_8_W64            CE_CAN_8.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_8_CS65           CE_CAN_8.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_8_ID65           CE_CAN_8.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_8_B65            CE_CAN_8.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_8_H65            CE_CAN_8.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_8_W65            CE_CAN_8.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_8_CS66           CE_CAN_8.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_8_ID66           CE_CAN_8.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_8_B66            CE_CAN_8.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_8_H66            CE_CAN_8.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_8_W66            CE_CAN_8.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_8_CS67           CE_CAN_8.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_8_ID67           CE_CAN_8.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_8_B67            CE_CAN_8.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_8_H67            CE_CAN_8.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_8_W67            CE_CAN_8.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_8_CS68           CE_CAN_8.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_8_ID68           CE_CAN_8.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_8_B68            CE_CAN_8.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_8_H68            CE_CAN_8.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_8_W68            CE_CAN_8.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_8_CS69           CE_CAN_8.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_8_ID69           CE_CAN_8.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_8_B69            CE_CAN_8.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_8_H69            CE_CAN_8.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_8_W69            CE_CAN_8.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_8_CS70           CE_CAN_8.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_8_ID70           CE_CAN_8.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_8_B70            CE_CAN_8.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_8_H70            CE_CAN_8.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_8_W70            CE_CAN_8.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_8_CS71           CE_CAN_8.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_8_ID71           CE_CAN_8.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_8_B71            CE_CAN_8.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_8_H71            CE_CAN_8.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_8_W71            CE_CAN_8.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_8_CS72           CE_CAN_8.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_8_ID72           CE_CAN_8.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_8_B72            CE_CAN_8.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_8_H72            CE_CAN_8.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_8_W72            CE_CAN_8.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_8_CS73           CE_CAN_8.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_8_ID73           CE_CAN_8.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_8_B73            CE_CAN_8.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_8_H73            CE_CAN_8.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_8_W73            CE_CAN_8.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_8_CS74           CE_CAN_8.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_8_ID74           CE_CAN_8.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_8_B74            CE_CAN_8.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_8_H74            CE_CAN_8.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_8_W74            CE_CAN_8.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_8_CS75           CE_CAN_8.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_8_ID75           CE_CAN_8.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_8_B75            CE_CAN_8.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_8_H75            CE_CAN_8.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_8_W75            CE_CAN_8.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_8_CS76           CE_CAN_8.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_8_ID76           CE_CAN_8.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_8_B76            CE_CAN_8.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_8_H76            CE_CAN_8.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_8_W76            CE_CAN_8.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_8_CS77           CE_CAN_8.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_8_ID77           CE_CAN_8.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_8_B77            CE_CAN_8.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_8_H77            CE_CAN_8.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_8_W77            CE_CAN_8.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_8_CS78           CE_CAN_8.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_8_ID78           CE_CAN_8.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_8_B78            CE_CAN_8.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_8_H78            CE_CAN_8.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_8_W78            CE_CAN_8.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_8_CS79           CE_CAN_8.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_8_ID79           CE_CAN_8.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_8_B79            CE_CAN_8.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_8_H79            CE_CAN_8.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_8_W79            CE_CAN_8.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_8_CS80           CE_CAN_8.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_8_ID80           CE_CAN_8.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_8_B80            CE_CAN_8.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_8_H80            CE_CAN_8.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_8_W80            CE_CAN_8.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_8_CS81           CE_CAN_8.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_8_ID81           CE_CAN_8.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_8_B81            CE_CAN_8.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_8_H81            CE_CAN_8.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_8_W81            CE_CAN_8.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_8_CS82           CE_CAN_8.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_8_ID82           CE_CAN_8.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_8_B82            CE_CAN_8.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_8_H82            CE_CAN_8.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_8_W82            CE_CAN_8.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_8_CS83           CE_CAN_8.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_8_ID83           CE_CAN_8.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_8_B83            CE_CAN_8.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_8_H83            CE_CAN_8.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_8_W83            CE_CAN_8.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_8_CS84           CE_CAN_8.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_8_ID84           CE_CAN_8.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_8_B84            CE_CAN_8.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_8_H84            CE_CAN_8.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_8_W84            CE_CAN_8.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_8_CS85           CE_CAN_8.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_8_ID85           CE_CAN_8.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_8_B85            CE_CAN_8.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_8_H85            CE_CAN_8.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_8_W85            CE_CAN_8.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_8_CS86           CE_CAN_8.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_8_ID86           CE_CAN_8.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_8_B86            CE_CAN_8.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_8_H86            CE_CAN_8.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_8_W86            CE_CAN_8.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_8_CS87           CE_CAN_8.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_8_ID87           CE_CAN_8.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_8_B87            CE_CAN_8.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_8_H87            CE_CAN_8.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_8_W87            CE_CAN_8.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_8_CS88           CE_CAN_8.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_8_ID88           CE_CAN_8.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_8_B88            CE_CAN_8.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_8_H88            CE_CAN_8.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_8_W848           CE_CAN_8.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_8_CS89           CE_CAN_8.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_8_ID89           CE_CAN_8.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_8_B89            CE_CAN_8.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_8_H89            CE_CAN_8.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_8_W89            CE_CAN_8.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_8_CS90           CE_CAN_8.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_8_ID90           CE_CAN_8.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_8_B90            CE_CAN_8.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_8_H90            CE_CAN_8.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_8_W90            CE_CAN_8.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_8_CS91           CE_CAN_8.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_8_ID91           CE_CAN_8.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_8_B91            CE_CAN_8.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_8_H91            CE_CAN_8.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_8_W91            CE_CAN_8.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_8_CS92           CE_CAN_8.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_8_ID92           CE_CAN_8.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_8_B92            CE_CAN_8.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_8_H92            CE_CAN_8.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_8_W92            CE_CAN_8.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_8_CS93           CE_CAN_8.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_8_ID93           CE_CAN_8.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_8_B93            CE_CAN_8.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_8_H93            CE_CAN_8.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_8_W93            CE_CAN_8.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_8_CS94           CE_CAN_8.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_8_ID94           CE_CAN_8.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_8_B94            CE_CAN_8.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_8_H94            CE_CAN_8.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_8_W94            CE_CAN_8.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_8_CS95           CE_CAN_8.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_8_ID95           CE_CAN_8.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_8_B95            CE_CAN_8.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_8_H95            CE_CAN_8.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_8_W95            CE_CAN_8.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_8_CS96           CE_CAN_8.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_8_ID96           CE_CAN_8.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_8_B96            CE_CAN_8.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_8_H96            CE_CAN_8.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_8_W96            CE_CAN_8.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_8_CS97           CE_CAN_8.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_8_ID97           CE_CAN_8.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_8_B97            CE_CAN_8.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_8_H97            CE_CAN_8.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_8_W97            CE_CAN_8.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_8_CS98           CE_CAN_8.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_8_ID98           CE_CAN_8.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_8_B98            CE_CAN_8.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_8_H98            CE_CAN_8.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_8_W98            CE_CAN_8.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_8_CS99           CE_CAN_8.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_8_ID99           CE_CAN_8.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_8_B99            CE_CAN_8.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_8_H99            CE_CAN_8.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_8_W99            CE_CAN_8.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_8_CS100          CE_CAN_8.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_8_ID100          CE_CAN_8.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_8_B100           CE_CAN_8.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_8_H100           CE_CAN_8.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_8_W100           CE_CAN_8.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_8_CS101          CE_CAN_8.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_8_ID101          CE_CAN_8.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_8_B101           CE_CAN_8.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_8_H101           CE_CAN_8.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_8_W101           CE_CAN_8.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_8_CS102          CE_CAN_8.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_8_ID102          CE_CAN_8.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_8_B102           CE_CAN_8.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_8_H102           CE_CAN_8.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_8_W102           CE_CAN_8.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_8_CS103          CE_CAN_8.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_8_ID103          CE_CAN_8.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_8_B103           CE_CAN_8.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_8_H103           CE_CAN_8.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_8_W103           CE_CAN_8.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_8_CS104          CE_CAN_8.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_8_ID104          CE_CAN_8.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_8_B104           CE_CAN_8.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_8_H104           CE_CAN_8.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_8_W104           CE_CAN_8.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_8_CS105          CE_CAN_8.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_8_ID105          CE_CAN_8.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_8_B105           CE_CAN_8.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_8_H105           CE_CAN_8.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_8_W105           CE_CAN_8.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_8_CS106          CE_CAN_8.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_8_ID106          CE_CAN_8.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_8_B106           CE_CAN_8.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_8_H106           CE_CAN_8.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_8_W106           CE_CAN_8.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_8_CS107          CE_CAN_8.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_8_ID107          CE_CAN_8.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_8_B107           CE_CAN_8.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_8_H107           CE_CAN_8.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_8_W107           CE_CAN_8.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_8_CS108          CE_CAN_8.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_8_ID108          CE_CAN_8.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_8_B108           CE_CAN_8.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_8_H108           CE_CAN_8.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_8_W108           CE_CAN_8.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_8_CS109          CE_CAN_8.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_8_ID109          CE_CAN_8.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_8_B109           CE_CAN_8.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_8_H109           CE_CAN_8.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_8_W109           CE_CAN_8.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_8_CS110          CE_CAN_8.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_8_ID110          CE_CAN_8.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_8_B110           CE_CAN_8.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_8_H110           CE_CAN_8.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_8_W110           CE_CAN_8.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_8_CS111          CE_CAN_8.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_8_ID111          CE_CAN_8.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_8_B111           CE_CAN_8.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_8_H111           CE_CAN_8.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_8_W111           CE_CAN_8.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_8_CS112          CE_CAN_8.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_8_ID112          CE_CAN_8.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_8_B112           CE_CAN_8.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_8_H112           CE_CAN_8.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_8_W112           CE_CAN_8.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_8_CS113          CE_CAN_8.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_8_ID113          CE_CAN_8.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_8_B113           CE_CAN_8.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_8_H113           CE_CAN_8.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_8_W113           CE_CAN_8.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_8_CS114          CE_CAN_8.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_8_ID114          CE_CAN_8.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_8_B114           CE_CAN_8.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_8_H114           CE_CAN_8.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_8_W114           CE_CAN_8.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_8_CS115          CE_CAN_8.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_8_ID115          CE_CAN_8.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_8_B115           CE_CAN_8.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_8_H115           CE_CAN_8.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_8_W115           CE_CAN_8.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_8_CS116          CE_CAN_8.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_8_ID116          CE_CAN_8.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_8_B116           CE_CAN_8.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_8_H116           CE_CAN_8.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_8_W116           CE_CAN_8.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_8_CS117          CE_CAN_8.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_8_ID117          CE_CAN_8.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_8_B117           CE_CAN_8.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_8_H117           CE_CAN_8.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_8_W117           CE_CAN_8.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_8_CS118          CE_CAN_8.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_8_ID118          CE_CAN_8.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_8_B118           CE_CAN_8.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_8_H118           CE_CAN_8.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_8_W118           CE_CAN_8.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_8_CS119          CE_CAN_8.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_8_ID119          CE_CAN_8.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_8_B119           CE_CAN_8.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_8_H119           CE_CAN_8.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_8_W119           CE_CAN_8.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_8_CS120          CE_CAN_8.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_8_ID120          CE_CAN_8.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_8_B120           CE_CAN_8.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_8_H120           CE_CAN_8.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_8_W120           CE_CAN_8.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_8_CS121          CE_CAN_8.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_8_ID121          CE_CAN_8.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_8_B121           CE_CAN_8.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_8_H121           CE_CAN_8.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_8_W121           CE_CAN_8.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_8_CS122          CE_CAN_8.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_8_ID122          CE_CAN_8.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_8_B122           CE_CAN_8.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_8_H122           CE_CAN_8.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_8_W122           CE_CAN_8.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_8_CS123          CE_CAN_8.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_8_ID123          CE_CAN_8.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_8_B123           CE_CAN_8.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_8_H123           CE_CAN_8.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_8_W123           CE_CAN_8.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_8_CS124          CE_CAN_8.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_8_ID124          CE_CAN_8.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_8_B124           CE_CAN_8.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_8_H124           CE_CAN_8.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_8_W124           CE_CAN_8.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_8_CS125          CE_CAN_8.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_8_ID125          CE_CAN_8.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_8_B125           CE_CAN_8.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_8_H125           CE_CAN_8.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_8_W125           CE_CAN_8.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_8_CS126          CE_CAN_8.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_8_ID126          CE_CAN_8.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_8_B126           CE_CAN_8.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_8_H126           CE_CAN_8.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_8_W126           CE_CAN_8.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_8_CS127          CE_CAN_8.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_8_ID127          CE_CAN_8.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_8_B127           CE_CAN_8.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_8_H127           CE_CAN_8.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_8_W127           CE_CAN_8.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_8_RXIMR0      CE_CAN_8.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR1      CE_CAN_8.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR2      CE_CAN_8.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR3      CE_CAN_8.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR4      CE_CAN_8.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR5      CE_CAN_8.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR6      CE_CAN_8.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR7      CE_CAN_8.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR8      CE_CAN_8.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR9      CE_CAN_8.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_8_RXIMR10     CE_CAN_8.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR11     CE_CAN_8.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR12     CE_CAN_8.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR13     CE_CAN_8.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR14     CE_CAN_8.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR15     CE_CAN_8.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR16     CE_CAN_8.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR17     CE_CAN_8.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR18     CE_CAN_8.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR19     CE_CAN_8.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR20     CE_CAN_8.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR21     CE_CAN_8.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR22     CE_CAN_8.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR23     CE_CAN_8.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR24     CE_CAN_8.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR25     CE_CAN_8.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR26     CE_CAN_8.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR27     CE_CAN_8.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR28     CE_CAN_8.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR29     CE_CAN_8.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR30     CE_CAN_8.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR31     CE_CAN_8.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR32     CE_CAN_8.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR33     CE_CAN_8.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR34     CE_CAN_8.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR35     CE_CAN_8.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR36     CE_CAN_8.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR37     CE_CAN_8.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR38     CE_CAN_8.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR39     CE_CAN_8.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR40     CE_CAN_8.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR41     CE_CAN_8.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR42     CE_CAN_8.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR43     CE_CAN_8.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR44     CE_CAN_8.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR45     CE_CAN_8.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR46     CE_CAN_8.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR47     CE_CAN_8.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR48     CE_CAN_8.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR49     CE_CAN_8.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR50     CE_CAN_8.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR51     CE_CAN_8.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR52     CE_CAN_8.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR53     CE_CAN_8.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR54     CE_CAN_8.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR55     CE_CAN_8.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR56     CE_CAN_8.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR57     CE_CAN_8.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR58     CE_CAN_8.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR59     CE_CAN_8.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR60     CE_CAN_8.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR61     CE_CAN_8.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR62     CE_CAN_8.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR63     CE_CAN_8.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR64     CE_CAN_8.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR65     CE_CAN_8.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR66     CE_CAN_8.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR67     CE_CAN_8.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR68     CE_CAN_8.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR69     CE_CAN_8.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR70     CE_CAN_8.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR71     CE_CAN_8.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR72     CE_CAN_8.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR73     CE_CAN_8.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR74     CE_CAN_8.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR75     CE_CAN_8.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR76     CE_CAN_8.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR77     CE_CAN_8.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR78     CE_CAN_8.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR79     CE_CAN_8.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR80     CE_CAN_8.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR81     CE_CAN_8.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR82     CE_CAN_8.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR83     CE_CAN_8.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR84     CE_CAN_8.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR85     CE_CAN_8.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR86     CE_CAN_8.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR87     CE_CAN_8.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR88     CE_CAN_8.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR89     CE_CAN_8.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR90     CE_CAN_8.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR91     CE_CAN_8.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR92     CE_CAN_8.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR93     CE_CAN_8.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR94     CE_CAN_8.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR95     CE_CAN_8.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR96     CE_CAN_8.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR97     CE_CAN_8.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR98     CE_CAN_8.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR99     CE_CAN_8.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_8_RXIMR100    CE_CAN_8.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR101    CE_CAN_8.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR102    CE_CAN_8.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR103    CE_CAN_8.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR104    CE_CAN_8.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR105    CE_CAN_8.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR106    CE_CAN_8.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR107    CE_CAN_8.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR108    CE_CAN_8.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR109    CE_CAN_8.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR110    CE_CAN_8.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR111    CE_CAN_8.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR112    CE_CAN_8.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR113    CE_CAN_8.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR114    CE_CAN_8.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR115    CE_CAN_8.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR116    CE_CAN_8.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR117    CE_CAN_8.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR118    CE_CAN_8.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR119    CE_CAN_8.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR120    CE_CAN_8.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR121    CE_CAN_8.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR122    CE_CAN_8.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR123    CE_CAN_8.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR124    CE_CAN_8.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR125    CE_CAN_8.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR126    CE_CAN_8.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_8_RXIMR127    CE_CAN_8.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_8_MECR        CE_CAN_8.MECR.R               /* Memory Error Control */
#define CE_CAN_8_ERRIAR      CE_CAN_8.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_8_ERRIDPR     CE_CAN_8.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_8_ERRIPPR     CE_CAN_8.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_8_RERRAR      CE_CAN_8.RERRAR.R             /* Error Report Address */
#define CE_CAN_8_RERRDR      CE_CAN_8.RERRDR.R             /* Error Report Data */
#define CE_CAN_8_RERRSYNR    CE_CAN_8.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_8_ERRSR       CE_CAN_8.ERRSR.R              /* Error Status */
#define CE_CAN_8_EPRS        CE_CAN_8.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_8_ENCBT       CE_CAN_8.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_8_EDCBT       CE_CAN_8.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_8_ETDC        CE_CAN_8.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_8_FDCTRL      CE_CAN_8.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_8_FDCBT       CE_CAN_8.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_8_FDCRC       CE_CAN_8.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_8_ERFCR       CE_CAN_8.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_8_ERFIER      CE_CAN_8.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_8_ERFSR       CE_CAN_8.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_8_HR_TIME_STAMP0 CE_CAN_8.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP1 CE_CAN_8.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP2 CE_CAN_8.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP3 CE_CAN_8.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP4 CE_CAN_8.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP5 CE_CAN_8.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP6 CE_CAN_8.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP7 CE_CAN_8.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP8 CE_CAN_8.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP9 CE_CAN_8.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP10 CE_CAN_8.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP11 CE_CAN_8.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP12 CE_CAN_8.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP13 CE_CAN_8.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP14 CE_CAN_8.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP15 CE_CAN_8.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP16 CE_CAN_8.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP17 CE_CAN_8.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP18 CE_CAN_8.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP19 CE_CAN_8.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP20 CE_CAN_8.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP21 CE_CAN_8.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP22 CE_CAN_8.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP23 CE_CAN_8.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP24 CE_CAN_8.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP25 CE_CAN_8.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP26 CE_CAN_8.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP27 CE_CAN_8.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP28 CE_CAN_8.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP29 CE_CAN_8.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP30 CE_CAN_8.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP31 CE_CAN_8.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP32 CE_CAN_8.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP33 CE_CAN_8.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP34 CE_CAN_8.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP35 CE_CAN_8.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP36 CE_CAN_8.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP37 CE_CAN_8.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP38 CE_CAN_8.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP39 CE_CAN_8.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP40 CE_CAN_8.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP41 CE_CAN_8.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP42 CE_CAN_8.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP43 CE_CAN_8.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP44 CE_CAN_8.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP45 CE_CAN_8.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP46 CE_CAN_8.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP47 CE_CAN_8.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP48 CE_CAN_8.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP49 CE_CAN_8.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP50 CE_CAN_8.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP51 CE_CAN_8.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP52 CE_CAN_8.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP53 CE_CAN_8.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP54 CE_CAN_8.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP55 CE_CAN_8.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP56 CE_CAN_8.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP57 CE_CAN_8.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP58 CE_CAN_8.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP59 CE_CAN_8.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP60 CE_CAN_8.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP61 CE_CAN_8.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP62 CE_CAN_8.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP63 CE_CAN_8.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP64 CE_CAN_8.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP65 CE_CAN_8.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP66 CE_CAN_8.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP67 CE_CAN_8.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP68 CE_CAN_8.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP69 CE_CAN_8.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP70 CE_CAN_8.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP71 CE_CAN_8.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP72 CE_CAN_8.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP73 CE_CAN_8.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP74 CE_CAN_8.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP75 CE_CAN_8.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP76 CE_CAN_8.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP77 CE_CAN_8.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP78 CE_CAN_8.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP79 CE_CAN_8.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP80 CE_CAN_8.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP81 CE_CAN_8.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP82 CE_CAN_8.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP83 CE_CAN_8.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP84 CE_CAN_8.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP85 CE_CAN_8.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP86 CE_CAN_8.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP87 CE_CAN_8.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP88 CE_CAN_8.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP89 CE_CAN_8.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP90 CE_CAN_8.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP91 CE_CAN_8.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP92 CE_CAN_8.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP93 CE_CAN_8.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP94 CE_CAN_8.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP95 CE_CAN_8.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP96 CE_CAN_8.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP97 CE_CAN_8.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP98 CE_CAN_8.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP99 CE_CAN_8.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP100 CE_CAN_8.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP101 CE_CAN_8.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP102 CE_CAN_8.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP103 CE_CAN_8.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP104 CE_CAN_8.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP105 CE_CAN_8.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP106 CE_CAN_8.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP107 CE_CAN_8.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP108 CE_CAN_8.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP109 CE_CAN_8.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP110 CE_CAN_8.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP111 CE_CAN_8.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP112 CE_CAN_8.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP113 CE_CAN_8.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP114 CE_CAN_8.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP115 CE_CAN_8.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP116 CE_CAN_8.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP117 CE_CAN_8.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP118 CE_CAN_8.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP119 CE_CAN_8.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP120 CE_CAN_8.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP121 CE_CAN_8.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP122 CE_CAN_8.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP123 CE_CAN_8.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP124 CE_CAN_8.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP125 CE_CAN_8.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP126 CE_CAN_8.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_8_HR_TIME_STAMP127 CE_CAN_8.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_8_ERFFEL0     CE_CAN_8.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL1     CE_CAN_8.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL2     CE_CAN_8.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL3     CE_CAN_8.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL4     CE_CAN_8.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL5     CE_CAN_8.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL6     CE_CAN_8.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL7     CE_CAN_8.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL8     CE_CAN_8.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL9     CE_CAN_8.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL10    CE_CAN_8.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL11    CE_CAN_8.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL12    CE_CAN_8.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL13    CE_CAN_8.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL14    CE_CAN_8.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL15    CE_CAN_8.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL16    CE_CAN_8.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL17    CE_CAN_8.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL18    CE_CAN_8.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL19    CE_CAN_8.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL20    CE_CAN_8.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL21    CE_CAN_8.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL22    CE_CAN_8.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL23    CE_CAN_8.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL24    CE_CAN_8.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL25    CE_CAN_8.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL26    CE_CAN_8.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL27    CE_CAN_8.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL28    CE_CAN_8.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL29    CE_CAN_8.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL30    CE_CAN_8.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL31    CE_CAN_8.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL32    CE_CAN_8.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL33    CE_CAN_8.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL34    CE_CAN_8.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL35    CE_CAN_8.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL36    CE_CAN_8.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL37    CE_CAN_8.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL38    CE_CAN_8.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL39    CE_CAN_8.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL40    CE_CAN_8.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL41    CE_CAN_8.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL42    CE_CAN_8.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL43    CE_CAN_8.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL44    CE_CAN_8.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL45    CE_CAN_8.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL46    CE_CAN_8.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL47    CE_CAN_8.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL48    CE_CAN_8.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL49    CE_CAN_8.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL50    CE_CAN_8.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL51    CE_CAN_8.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL52    CE_CAN_8.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL53    CE_CAN_8.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL54    CE_CAN_8.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL55    CE_CAN_8.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL56    CE_CAN_8.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL57    CE_CAN_8.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL58    CE_CAN_8.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL59    CE_CAN_8.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL60    CE_CAN_8.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL61    CE_CAN_8.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL62    CE_CAN_8.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL63    CE_CAN_8.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL64    CE_CAN_8.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL65    CE_CAN_8.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL66    CE_CAN_8.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL67    CE_CAN_8.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL68    CE_CAN_8.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL69    CE_CAN_8.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL70    CE_CAN_8.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL71    CE_CAN_8.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL72    CE_CAN_8.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL73    CE_CAN_8.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL74    CE_CAN_8.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL75    CE_CAN_8.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL76    CE_CAN_8.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL77    CE_CAN_8.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL78    CE_CAN_8.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL79    CE_CAN_8.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL80    CE_CAN_8.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL81    CE_CAN_8.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL82    CE_CAN_8.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL83    CE_CAN_8.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL84    CE_CAN_8.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL85    CE_CAN_8.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL86    CE_CAN_8.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL87    CE_CAN_8.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL88    CE_CAN_8.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL89    CE_CAN_8.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL90    CE_CAN_8.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL91    CE_CAN_8.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL92    CE_CAN_8.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL93    CE_CAN_8.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL94    CE_CAN_8.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL95    CE_CAN_8.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL96    CE_CAN_8.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL97    CE_CAN_8.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL98    CE_CAN_8.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL99    CE_CAN_8.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL100   CE_CAN_8.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL101   CE_CAN_8.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL102   CE_CAN_8.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL103   CE_CAN_8.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL104   CE_CAN_8.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL105   CE_CAN_8.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL106   CE_CAN_8.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL107   CE_CAN_8.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL108   CE_CAN_8.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL109   CE_CAN_8.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL110   CE_CAN_8.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL111   CE_CAN_8.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL112   CE_CAN_8.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL113   CE_CAN_8.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL114   CE_CAN_8.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL115   CE_CAN_8.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL116   CE_CAN_8.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL117   CE_CAN_8.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL118   CE_CAN_8.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL119   CE_CAN_8.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL120   CE_CAN_8.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL121   CE_CAN_8.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL122   CE_CAN_8.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL123   CE_CAN_8.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL124   CE_CAN_8.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL125   CE_CAN_8.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL126   CE_CAN_8.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_8_ERFFEL127   CE_CAN_8.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

#define CE_CAN_9_MCR         CE_CAN_9.MCR.R                /* Module Configuration */
#define CE_CAN_9_CTRL1       CE_CAN_9.CTRL1.R              /* Control 1 */
#define CE_CAN_9_TIMER       CE_CAN_9.TIMER.R              /* Free-Running Timer */
#define CE_CAN_9_RXMGMASK    CE_CAN_9.RXMGMASK.R           /* RX Message Buffers Global Mask */
#define CE_CAN_9_RX14MASK    CE_CAN_9.RX14MASK.R           /* Receive 14 Mask */
#define CE_CAN_9_RX15MASK    CE_CAN_9.RX15MASK.R           /* Receive 15 Mask */
#define CE_CAN_9_ECR         CE_CAN_9.ECR.R                /* Error Counter */
#define CE_CAN_9_ESR1        CE_CAN_9.ESR1.R               /* Error and Status 1 */
#define CE_CAN_9_IMASK2      CE_CAN_9.IMASK2.R             /* Interrupt Masks 2 */
#define CE_CAN_9_IMASK1      CE_CAN_9.IMASK1.R             /* Interrupt Masks 1 */
#define CE_CAN_9_IFLAG2      CE_CAN_9.IFLAG2.R             /* Interrupt Flags 2 */
#define CE_CAN_9_IFLAG1      CE_CAN_9.IFLAG1.R             /* Interrupt Flags 1 */
#define CE_CAN_9_CTRL2       CE_CAN_9.CTRL2.R              /* Control 2 */
#define CE_CAN_9_ESR2        CE_CAN_9.ESR2.R               /* Error and Status 2 */
#define CE_CAN_9_CRCR        CE_CAN_9.CRCR.R               /* Cyclic Redundancy Check */
#define CE_CAN_9_RXFGMASK    CE_CAN_9.RXFGMASK.R           /* Legacy RX FIFO Global Mask */
#define CE_CAN_9_RXFIR       CE_CAN_9.RXFIR.R              /* Legacy RX FIFO Information */
#define CE_CAN_9_CBT         CE_CAN_9.CBT.R                /* CAN Bit Timing */
#define CE_CAN_9_IMASK4      CE_CAN_9.IMASK4.R             /* Interrupt Masks 4 */
#define CE_CAN_9_IMASK3      CE_CAN_9.IMASK3.R             /* Interrupt Masks 3 */
#define CE_CAN_9_IFLAG4      CE_CAN_9.IFLAG4.R             /* Interrupt Flags 4 */
#define CE_CAN_9_IFLAG3      CE_CAN_9.IFLAG3.R             /* Interrupt Flags 3 */
#define CE_CAN_9_CS0            CE_CAN_9.MB[0].CS.R              /* Message Buffer 0 CS Register */
#define CE_CAN_9_ID0            CE_CAN_9.MB[0].ID.R              /* Message Buffer 0 ID Register */
#define CE_CAN_9_B0             CE_CAN_9.MB[0].DATA.B            /* Message Buffer 0 B Register */
#define CE_CAN_9_H0             CE_CAN_9.MB[0].DATA.H            /* Message Buffer 0 H Register */
#define CE_CAN_9_W0             CE_CAN_9.MB[0].DATA.W            /* Message Buffer 0 W Register */
#define CE_CAN_9_CS1            CE_CAN_9.MB[1].CS.R              /* Message Buffer 1 CS Register */
#define CE_CAN_9_ID1            CE_CAN_9.MB[1].ID.R              /* Message Buffer 1 ID Register */
#define CE_CAN_9_B1             CE_CAN_9.MB[1].DATA.B            /* Message Buffer 1 B Register */
#define CE_CAN_9_H1             CE_CAN_9.MB[1].DATA.H            /* Message Buffer 1 H Register */
#define CE_CAN_9_W1             CE_CAN_9.MB[1].DATA.W            /* Message Buffer 1 W Register */
#define CE_CAN_9_CS2            CE_CAN_9.MB[2].CS.R              /* Message Buffer 2 CS Register */
#define CE_CAN_9_ID2            CE_CAN_9.MB[2].ID.R              /* Message Buffer 2 ID Register */
#define CE_CAN_9_B2             CE_CAN_9.MB[2].DATA.B            /* Message Buffer 2 B Register */
#define CE_CAN_9_H2             CE_CAN_9.MB[2].DATA.H            /* Message Buffer 2 H Register */
#define CE_CAN_9_W2             CE_CAN_9.MB[2].DATA.W            /* Message Buffer 2 W Register */
#define CE_CAN_9_CS3            CE_CAN_9.MB[3].CS.R              /* Message Buffer 3 CS Register */
#define CE_CAN_9_ID3            CE_CAN_9.MB[3].ID.R              /* Message Buffer 3 ID Register */
#define CE_CAN_9_B3             CE_CAN_9.MB[3].DATA.B            /* Message Buffer 3 B Register */
#define CE_CAN_9_H3             CE_CAN_9.MB[3].DATA.H            /* Message Buffer 3 H Register */
#define CE_CAN_9_W3             CE_CAN_9.MB[3].DATA.W            /* Message Buffer 3 W Register */
#define CE_CAN_9_CS4            CE_CAN_9.MB[4].CS.R              /* Message Buffer 4 CS Register */
#define CE_CAN_9_ID4            CE_CAN_9.MB[4].ID.R              /* Message Buffer 4 ID Register */
#define CE_CAN_9_B4             CE_CAN_9.MB[4].DATA.B            /* Message Buffer 4 B Register */
#define CE_CAN_9_H4             CE_CAN_9.MB[4].DATA.H            /* Message Buffer 4 H Register */
#define CE_CAN_9_W4             CE_CAN_9.MB[4].DATA.W            /* Message Buffer 4 W Register */
#define CE_CAN_9_CS5            CE_CAN_9.MB[5].CS.R              /* Message Buffer 5 CS Register */
#define CE_CAN_9_ID5            CE_CAN_9.MB[5].ID.R              /* Message Buffer 5 ID Register */
#define CE_CAN_9_B5             CE_CAN_9.MB[5].DATA.B            /* Message Buffer 5 B Register */
#define CE_CAN_9_H5             CE_CAN_9.MB[5].DATA.H            /* Message Buffer 5 H Register */
#define CE_CAN_9_W5             CE_CAN_9.MB[5].DATA.W            /* Message Buffer 5 W Register */
#define CE_CAN_9_CS6            CE_CAN_9.MB[6].CS.R              /* Message Buffer 6 CS Register */
#define CE_CAN_9_ID6            CE_CAN_9.MB[6].ID.R              /* Message Buffer 6 ID Register */
#define CE_CAN_9_B6             CE_CAN_9.MB[6].DATA.B            /* Message Buffer 6 B Register */
#define CE_CAN_9_H6             CE_CAN_9.MB[6].DATA.H            /* Message Buffer 6 H Register */
#define CE_CAN_9_W6             CE_CAN_9.MB[6].DATA.W            /* Message Buffer 6 W Register */
#define CE_CAN_9_CS7            CE_CAN_9.MB[7].CS.R              /* Message Buffer 7 CS Register */
#define CE_CAN_9_ID7            CE_CAN_9.MB[7].ID.R              /* Message Buffer 7 ID Register */
#define CE_CAN_9_B7             CE_CAN_9.MB[7].DATA.B            /* Message Buffer 7 B Register */
#define CE_CAN_9_H7             CE_CAN_9.MB[7].DATA.H            /* Message Buffer 7 H Register */
#define CE_CAN_9_W7             CE_CAN_9.MB[7].DATA.W            /* Message Buffer 7 W Register */
#define CE_CAN_9_CS8            CE_CAN_9.MB[8].CS.R              /* Message Buffer 8 CS Register */
#define CE_CAN_9_ID8            CE_CAN_9.MB[8].ID.R              /* Message Buffer 8 ID Register */
#define CE_CAN_9_B8             CE_CAN_9.MB[8].DATA.B            /* Message Buffer 8 B Register */
#define CE_CAN_9_H8             CE_CAN_9.MB[8].DATA.H            /* Message Buffer 8 H Register */
#define CE_CAN_9_W8             CE_CAN_9.MB[8].DATA.W            /* Message Buffer 8 W Register */
#define CE_CAN_9_CS9            CE_CAN_9.MB[9].CS.R              /* Message Buffer 9 CS Register */
#define CE_CAN_9_ID9            CE_CAN_9.MB[9].ID.R              /* Message Buffer 9 ID Register */
#define CE_CAN_9_B9             CE_CAN_9.MB[9].DATA.B            /* Message Buffer 9 B Register */
#define CE_CAN_9_H9             CE_CAN_9.MB[9].DATA.H            /* Message Buffer 9 H Register */
#define CE_CAN_9_W9             CE_CAN_9.MB[9].DATA.W            /* Message Buffer 9 W Register */
#define CE_CAN_9_CS10           CE_CAN_9.MB[10].CS.R             /* Message Buffer 10 CS Register */
#define CE_CAN_9_ID10           CE_CAN_9.MB[10].ID.R             /* Message Buffer 10 ID Register */
#define CE_CAN_9_B10            CE_CAN_9.MB[10].DATA.B           /* Message Buffer 10 B Register */
#define CE_CAN_9_H10            CE_CAN_9.MB[10].DATA.H           /* Message Buffer 10 H Register */
#define CE_CAN_9_W10            CE_CAN_9.MB[10].DATA.W           /* Message Buffer 10 W Register */
#define CE_CAN_9_CS11           CE_CAN_9.MB[11].CS.R             /* Message Buffer 11 CS Register */
#define CE_CAN_9_ID11           CE_CAN_9.MB[11].ID.R             /* Message Buffer 11 ID Register */
#define CE_CAN_9_B11            CE_CAN_9.MB[11].DATA.B           /* Message Buffer 11 B Register */
#define CE_CAN_9_H11            CE_CAN_9.MB[11].DATA.H           /* Message Buffer 11 H Register */
#define CE_CAN_9_W11            CE_CAN_9.MB[11].DATA.W           /* Message Buffer 11 W Register */
#define CE_CAN_9_CS12           CE_CAN_9.MB[12].CS.R             /* Message Buffer 12 CS Register */
#define CE_CAN_9_ID12           CE_CAN_9.MB[12].ID.R             /* Message Buffer 12 ID Register */
#define CE_CAN_9_B12            CE_CAN_9.MB[12].DATA.B           /* Message Buffer 12 B Register */
#define CE_CAN_9_H12            CE_CAN_9.MB[12].DATA.H           /* Message Buffer 12 H Register */
#define CE_CAN_9_W12            CE_CAN_9.MB[12].DATA.W           /* Message Buffer 12 W Register */
#define CE_CAN_9_CS13           CE_CAN_9.MB[13].CS.R             /* Message Buffer 13 CS Register */
#define CE_CAN_9_ID13           CE_CAN_9.MB[13].ID.R             /* Message Buffer 13 ID Register */
#define CE_CAN_9_B13            CE_CAN_9.MB[13].DATA.B           /* Message Buffer 13 B Register */
#define CE_CAN_9_H13            CE_CAN_9.MB[13].DATA.H           /* Message Buffer 13 H Register */
#define CE_CAN_9_W13            CE_CAN_9.MB[13].DATA.W           /* Message Buffer 13 W Register */
#define CE_CAN_9_CS14           CE_CAN_9.MB[14].CS.R             /* Message Buffer 14 CS Register */
#define CE_CAN_9_ID14           CE_CAN_9.MB[14].ID.R             /* Message Buffer 14 ID Register */
#define CE_CAN_9_B14            CE_CAN_9.MB[14].DATA.B           /* Message Buffer 14 B Register */
#define CE_CAN_9_H14            CE_CAN_9.MB[14].DATA.H           /* Message Buffer 14 H Register */
#define CE_CAN_9_W14            CE_CAN_9.MB[14].DATA.W           /* Message Buffer 14 W Register */
#define CE_CAN_9_CS15           CE_CAN_9.MB[15].CS.R             /* Message Buffer 15 CS Register */
#define CE_CAN_9_ID15           CE_CAN_9.MB[15].ID.R             /* Message Buffer 15 ID Register */
#define CE_CAN_9_B15            CE_CAN_9.MB[15].DATA.B           /* Message Buffer 15 B Register */
#define CE_CAN_9_H15            CE_CAN_9.MB[15].DATA.H           /* Message Buffer 15 H Register */
#define CE_CAN_9_W15            CE_CAN_9.MB[15].DATA.W           /* Message Buffer 15 W Register */
#define CE_CAN_9_CS16           CE_CAN_9.MB[16].CS.R             /* Message Buffer 16 CS Register */
#define CE_CAN_9_ID16           CE_CAN_9.MB[16].ID.R             /* Message Buffer 16 ID Register */
#define CE_CAN_9_B16            CE_CAN_9.MB[16].DATA.B           /* Message Buffer 16 B Register */
#define CE_CAN_9_H16            CE_CAN_9.MB[16].DATA.H           /* Message Buffer 16 H Register */
#define CE_CAN_9_W16            CE_CAN_9.MB[16].DATA.W           /* Message Buffer 16 W Register */
#define CE_CAN_9_CS17           CE_CAN_9.MB[17].CS.R             /* Message Buffer 17 CS Register */
#define CE_CAN_9_ID17           CE_CAN_9.MB[17].ID.R             /* Message Buffer 17 ID Register */
#define CE_CAN_9_B17            CE_CAN_9.MB[17].DATA.B           /* Message Buffer 17 B Register */
#define CE_CAN_9_H17            CE_CAN_9.MB[17].DATA.H           /* Message Buffer 17 H Register */
#define CE_CAN_9_W17            CE_CAN_9.MB[17].DATA.W           /* Message Buffer 17 W Register */
#define CE_CAN_9_CS18           CE_CAN_9.MB[18].CS.R             /* Message Buffer 18 CS Register */
#define CE_CAN_9_ID18           CE_CAN_9.MB[18].ID.R             /* Message Buffer 18 ID Register */
#define CE_CAN_9_B18            CE_CAN_9.MB[18].DATA.B           /* Message Buffer 18 B Register */
#define CE_CAN_9_H18            CE_CAN_9.MB[18].DATA.H           /* Message Buffer 18 H Register */
#define CE_CAN_9_W18            CE_CAN_9.MB[18].DATA.W           /* Message Buffer 18 W Register */
#define CE_CAN_9_CS19           CE_CAN_9.MB[19].CS.R             /* Message Buffer 19 CS Register */
#define CE_CAN_9_ID19           CE_CAN_9.MB[19].ID.R             /* Message Buffer 19 ID Register */
#define CE_CAN_9_B19            CE_CAN_9.MB[19].DATA.B           /* Message Buffer 19 B Register */
#define CE_CAN_9_H19            CE_CAN_9.MB[19].DATA.H           /* Message Buffer 19 H Register */
#define CE_CAN_9_W19            CE_CAN_9.MB[19].DATA.W           /* Message Buffer 19 W Register */
#define CE_CAN_9_CS20           CE_CAN_9.MB[20].CS.R             /* Message Buffer 20 CS Register */
#define CE_CAN_9_ID20           CE_CAN_9.MB[20].ID.R             /* Message Buffer 20 ID Register */
#define CE_CAN_9_B20            CE_CAN_9.MB[20].DATA.B           /* Message Buffer 20 B Register */
#define CE_CAN_9_H20            CE_CAN_9.MB[20].DATA.H           /* Message Buffer 20 H Register */
#define CE_CAN_9_W20            CE_CAN_9.MB[20].DATA.W           /* Message Buffer 20 W Register */
#define CE_CAN_9_CS21           CE_CAN_9.MB[21].CS.R             /* Message Buffer 21 CS Register */
#define CE_CAN_9_ID21           CE_CAN_9.MB[21].ID.R             /* Message Buffer 21 ID Register */
#define CE_CAN_9_B21            CE_CAN_9.MB[21].DATA.B           /* Message Buffer 21 B Register */
#define CE_CAN_9_H21            CE_CAN_9.MB[21].DATA.H           /* Message Buffer 21 H Register */
#define CE_CAN_9_W21            CE_CAN_9.MB[21].DATA.W           /* Message Buffer 21 W Register */
#define CE_CAN_9_CS22           CE_CAN_9.MB[22].CS.R             /* Message Buffer 22 CS Register */
#define CE_CAN_9_ID22           CE_CAN_9.MB[22].ID.R             /* Message Buffer 22 ID Register */
#define CE_CAN_9_B22            CE_CAN_9.MB[22].DATA.B           /* Message Buffer 22 B Register */
#define CE_CAN_9_H22            CE_CAN_9.MB[22].DATA.H           /* Message Buffer 22 H Register */
#define CE_CAN_9_W22            CE_CAN_9.MB[22].DATA.W           /* Message Buffer 22 W Register */
#define CE_CAN_9_CS23           CE_CAN_9.MB[23].CS.R             /* Message Buffer 23 CS Register */
#define CE_CAN_9_ID23           CE_CAN_9.MB[23].ID.R             /* Message Buffer 23 ID Register */
#define CE_CAN_9_B23            CE_CAN_9.MB[23].DATA.B           /* Message Buffer 23 B Register */
#define CE_CAN_9_H23            CE_CAN_9.MB[23].DATA.H           /* Message Buffer 23 H Register */
#define CE_CAN_9_W23            CE_CAN_9.MB[23].DATA.W           /* Message Buffer 23 W Register */
#define CE_CAN_9_CS24           CE_CAN_9.MB[24].CS.R             /* Message Buffer 24 CS Register */
#define CE_CAN_9_ID24           CE_CAN_9.MB[24].ID.R             /* Message Buffer 24 ID Register */
#define CE_CAN_9_B24            CE_CAN_9.MB[24].DATA.B           /* Message Buffer 24 B Register */
#define CE_CAN_9_H24            CE_CAN_9.MB[24].DATA.H           /* Message Buffer 24 H Register */
#define CE_CAN_9_W24            CE_CAN_9.MB[24].DATA.W           /* Message Buffer 24 W Register */
#define CE_CAN_9_CS25           CE_CAN_9.MB[25].CS.R             /* Message Buffer 25 CS Register */
#define CE_CAN_9_ID25           CE_CAN_9.MB[25].ID.R             /* Message Buffer 25 ID Register */
#define CE_CAN_9_B25            CE_CAN_9.MB[25].DATA.B           /* Message Buffer 25 B Register */
#define CE_CAN_9_H25            CE_CAN_9.MB[25].DATA.H           /* Message Buffer 25 H Register */
#define CE_CAN_9_W25            CE_CAN_9.MB[25].DATA.W           /* Message Buffer 25 W Register */
#define CE_CAN_9_CS26           CE_CAN_9.MB[26].CS.R             /* Message Buffer 26 CS Register */
#define CE_CAN_9_ID26           CE_CAN_9.MB[26].ID.R             /* Message Buffer 26 ID Register */
#define CE_CAN_9_B26            CE_CAN_9.MB[26].DATA.B           /* Message Buffer 26 B Register */
#define CE_CAN_9_H26            CE_CAN_9.MB[26].DATA.H           /* Message Buffer 26 H Register */
#define CE_CAN_9_W26            CE_CAN_9.MB[26].DATA.W           /* Message Buffer 26 W Register */
#define CE_CAN_9_CS27           CE_CAN_9.MB[27].CS.R             /* Message Buffer 27 CS Register */
#define CE_CAN_9_ID27           CE_CAN_9.MB[27].ID.R             /* Message Buffer 27 ID Register */
#define CE_CAN_9_B27            CE_CAN_9.MB[27].DATA.B           /* Message Buffer 27 B Register */
#define CE_CAN_9_H27            CE_CAN_9.MB[27].DATA.H           /* Message Buffer 27 H Register */
#define CE_CAN_9_W27            CE_CAN_9.MB[27].DATA.W           /* Message Buffer 27 W Register */
#define CE_CAN_9_CS28           CE_CAN_9.MB[28].CS.R             /* Message Buffer 28 CS Register */
#define CE_CAN_9_ID28           CE_CAN_9.MB[28].ID.R             /* Message Buffer 28 ID Register */
#define CE_CAN_9_B28            CE_CAN_9.MB[28].DATA.B           /* Message Buffer 28 B Register */
#define CE_CAN_9_H28            CE_CAN_9.MB[28].DATA.H           /* Message Buffer 28 H Register */
#define CE_CAN_9_W28            CE_CAN_9.MB[28].DATA.W           /* Message Buffer 28 W Register */
#define CE_CAN_9_CS29           CE_CAN_9.MB[29].CS.R             /* Message Buffer 29 CS Register */
#define CE_CAN_9_ID29           CE_CAN_9.MB[29].ID.R             /* Message Buffer 29 ID Register */
#define CE_CAN_9_B29            CE_CAN_9.MB[29].DATA.B           /* Message Buffer 29 B Register */
#define CE_CAN_9_H29            CE_CAN_9.MB[29].DATA.H           /* Message Buffer 29 H Register */
#define CE_CAN_9_W29            CE_CAN_9.MB[29].DATA.W           /* Message Buffer 29 W Register */
#define CE_CAN_9_CS30           CE_CAN_9.MB[30].CS.R             /* Message Buffer 30 CS Register */
#define CE_CAN_9_ID30           CE_CAN_9.MB[30].ID.R             /* Message Buffer 30 ID Register */
#define CE_CAN_9_B30            CE_CAN_9.MB[30].DATA.B           /* Message Buffer 30 B Register */
#define CE_CAN_9_H30            CE_CAN_9.MB[30].DATA.H           /* Message Buffer 30 H Register */
#define CE_CAN_9_W30            CE_CAN_9.MB[30].DATA.W           /* Message Buffer 30 W Register */
#define CE_CAN_9_CS31           CE_CAN_9.MB[31].CS.R             /* Message Buffer 31 CS Register */
#define CE_CAN_9_ID31           CE_CAN_9.MB[31].ID.R             /* Message Buffer 31 ID Register */
#define CE_CAN_9_B31            CE_CAN_9.MB[31].DATA.B           /* Message Buffer 31 B Register */
#define CE_CAN_9_H31            CE_CAN_9.MB[31].DATA.H           /* Message Buffer 31 H Register */
#define CE_CAN_9_W31            CE_CAN_9.MB[31].DATA.W           /* Message Buffer 31 W Register */
#define CE_CAN_9_CS32           CE_CAN_9.MB[32].CS.R             /* Message Buffer 32 CS Register */
#define CE_CAN_9_ID32           CE_CAN_9.MB[32].ID.R             /* Message Buffer 32 ID Register */
#define CE_CAN_9_B32            CE_CAN_9.MB[32].DATA.B           /* Message Buffer 32 B Register */
#define CE_CAN_9_H32            CE_CAN_9.MB[32].DATA.H           /* Message Buffer 32 H Register */
#define CE_CAN_9_W32            CE_CAN_9.MB[32].DATA.W           /* Message Buffer 32 W Register */
#define CE_CAN_9_CS33           CE_CAN_9.MB[33].CS.R             /* Message Buffer 33 CS Register */
#define CE_CAN_9_ID33           CE_CAN_9.MB[33].ID.R             /* Message Buffer 33 ID Register */
#define CE_CAN_9_B33            CE_CAN_9.MB[33].DATA.B           /* Message Buffer 33 B Register */
#define CE_CAN_9_H33            CE_CAN_9.MB[33].DATA.H           /* Message Buffer 33 H Register */
#define CE_CAN_9_W33            CE_CAN_9.MB[33].DATA.W           /* Message Buffer 33 W Register */
#define CE_CAN_9_CS34           CE_CAN_9.MB[34].CS.R             /* Message Buffer 34 CS Register */
#define CE_CAN_9_ID34           CE_CAN_9.MB[34].ID.R             /* Message Buffer 34 ID Register */
#define CE_CAN_9_B34            CE_CAN_9.MB[34].DATA.B           /* Message Buffer 34 B Register */
#define CE_CAN_9_H34            CE_CAN_9.MB[34].DATA.H           /* Message Buffer 34 H Register */
#define CE_CAN_9_W34            CE_CAN_9.MB[34].DATA.W           /* Message Buffer 34 W Register */
#define CE_CAN_9_CS35           CE_CAN_9.MB[35].CS.R             /* Message Buffer 35 CS Register */
#define CE_CAN_9_ID35           CE_CAN_9.MB[35].ID.R             /* Message Buffer 35 ID Register */
#define CE_CAN_9_B35            CE_CAN_9.MB[35].DATA.B           /* Message Buffer 35 B Register */
#define CE_CAN_9_H35            CE_CAN_9.MB[35].DATA.H           /* Message Buffer 35 H Register */
#define CE_CAN_9_W35            CE_CAN_9.MB[35].DATA.W           /* Message Buffer 35 W Register */
#define CE_CAN_9_CS36           CE_CAN_9.MB[36].CS.R             /* Message Buffer 36 CS Register */
#define CE_CAN_9_ID36           CE_CAN_9.MB[36].ID.R             /* Message Buffer 36 ID Register */
#define CE_CAN_9_B36            CE_CAN_9.MB[36].DATA.B           /* Message Buffer 36 B Register */
#define CE_CAN_9_H36            CE_CAN_9.MB[36].DATA.H           /* Message Buffer 36 H Register */
#define CE_CAN_9_W36            CE_CAN_9.MB[36].DATA.W           /* Message Buffer 36 W Register */
#define CE_CAN_9_CS37           CE_CAN_9.MB[37].CS.R             /* Message Buffer 37 CS Register */
#define CE_CAN_9_ID37           CE_CAN_9.MB[37].ID.R             /* Message Buffer 37 ID Register */
#define CE_CAN_9_B37            CE_CAN_9.MB[37].DATA.B           /* Message Buffer 37 B Register */
#define CE_CAN_9_H37            CE_CAN_9.MB[37].DATA.H           /* Message Buffer 37 H Register */
#define CE_CAN_9_W37            CE_CAN_9.MB[37].DATA.W           /* Message Buffer 37 W Register */
#define CE_CAN_9_CS38           CE_CAN_9.MB[38].CS.R             /* Message Buffer 38 CS Register */
#define CE_CAN_9_ID38           CE_CAN_9.MB[38].ID.R             /* Message Buffer 38 ID Register */
#define CE_CAN_9_B38            CE_CAN_9.MB[38].DATA.B           /* Message Buffer 38 B Register */
#define CE_CAN_9_H38            CE_CAN_9.MB[38].DATA.H           /* Message Buffer 38 H Register */
#define CE_CAN_9_W38            CE_CAN_9.MB[38].DATA.W           /* Message Buffer 38 W Register */
#define CE_CAN_9_CS39           CE_CAN_9.MB[39].CS.R             /* Message Buffer 39 CS Register */
#define CE_CAN_9_ID39           CE_CAN_9.MB[39].ID.R             /* Message Buffer 39 ID Register */
#define CE_CAN_9_B39            CE_CAN_9.MB[39].DATA.B           /* Message Buffer 39 B Register */
#define CE_CAN_9_H39            CE_CAN_9.MB[39].DATA.H           /* Message Buffer 39 H Register */
#define CE_CAN_9_W39            CE_CAN_9.MB[39].DATA.W           /* Message Buffer 39 W Register */
#define CE_CAN_9_CS40           CE_CAN_9.MB[40].CS.R             /* Message Buffer 40 CS Register */
#define CE_CAN_9_ID40           CE_CAN_9.MB[40].ID.R             /* Message Buffer 40 ID Register */
#define CE_CAN_9_B40            CE_CAN_9.MB[40].DATA.B           /* Message Buffer 40 B Register */
#define CE_CAN_9_H40            CE_CAN_9.MB[40].DATA.H           /* Message Buffer 40 H Register */
#define CE_CAN_9_W40            CE_CAN_9.MB[40].DATA.W           /* Message Buffer 40 W Register */
#define CE_CAN_9_CS41           CE_CAN_9.MB[41].CS.R             /* Message Buffer 41 CS Register */
#define CE_CAN_9_ID41           CE_CAN_9.MB[41].ID.R             /* Message Buffer 41 ID Register */
#define CE_CAN_9_B41            CE_CAN_9.MB[41].DATA.B           /* Message Buffer 41 B Register */
#define CE_CAN_9_H41            CE_CAN_9.MB[41].DATA.H           /* Message Buffer 41 H Register */
#define CE_CAN_9_W41            CE_CAN_9.MB[41].DATA.W           /* Message Buffer 41 W Register */
#define CE_CAN_9_CS42           CE_CAN_9.MB[42].CS.R             /* Message Buffer 42 CS Register */
#define CE_CAN_9_ID42           CE_CAN_9.MB[42].ID.R             /* Message Buffer 42 ID Register */
#define CE_CAN_9_B42            CE_CAN_9.MB[42].DATA.B           /* Message Buffer 42 B Register */
#define CE_CAN_9_H42            CE_CAN_9.MB[42].DATA.H           /* Message Buffer 42 H Register */
#define CE_CAN_9_W42            CE_CAN_9.MB[42].DATA.W           /* Message Buffer 42 W Register */
#define CE_CAN_9_CS43           CE_CAN_9.MB[43].CS.R             /* Message Buffer 43 CS Register */
#define CE_CAN_9_ID43           CE_CAN_9.MB[43].ID.R             /* Message Buffer 43 ID Register */
#define CE_CAN_9_B43            CE_CAN_9.MB[43].DATA.B           /* Message Buffer 43 B Register */
#define CE_CAN_9_H43            CE_CAN_9.MB[43].DATA.H           /* Message Buffer 43 H Register */
#define CE_CAN_9_W43            CE_CAN_9.MB[43].DATA.W           /* Message Buffer 43 W Register */
#define CE_CAN_9_CS44           CE_CAN_9.MB[44].CS.R             /* Message Buffer 44 CS Register */
#define CE_CAN_9_ID44           CE_CAN_9.MB[44].ID.R             /* Message Buffer 44 ID Register */
#define CE_CAN_9_B44            CE_CAN_9.MB[44].DATA.B           /* Message Buffer 44 B Register */
#define CE_CAN_9_H44            CE_CAN_9.MB[44].DATA.H           /* Message Buffer 44 H Register */
#define CE_CAN_9_W44            CE_CAN_9.MB[44].DATA.W           /* Message Buffer 44 W Register */
#define CE_CAN_9_CS45           CE_CAN_9.MB[45].CS.R             /* Message Buffer 45 CS Register */
#define CE_CAN_9_ID45           CE_CAN_9.MB[45].ID.R             /* Message Buffer 45 ID Register */
#define CE_CAN_9_B45            CE_CAN_9.MB[45].DATA.B           /* Message Buffer 45 B Register */
#define CE_CAN_9_H45            CE_CAN_9.MB[45].DATA.H           /* Message Buffer 45 H Register */
#define CE_CAN_9_W45            CE_CAN_9.MB[45].DATA.W           /* Message Buffer 45 W Register */
#define CE_CAN_9_CS46           CE_CAN_9.MB[46].CS.R             /* Message Buffer 46 CS Register */
#define CE_CAN_9_ID46           CE_CAN_9.MB[46].ID.R             /* Message Buffer 46 ID Register */
#define CE_CAN_9_B46            CE_CAN_9.MB[46].DATA.B           /* Message Buffer 46 B Register */
#define CE_CAN_9_H46            CE_CAN_9.MB[46].DATA.H           /* Message Buffer 46 H Register */
#define CE_CAN_9_W46            CE_CAN_9.MB[46].DATA.W           /* Message Buffer 46 W Register */
#define CE_CAN_9_CS47           CE_CAN_9.MB[47].CS.R             /* Message Buffer 47 CS Register */
#define CE_CAN_9_ID47           CE_CAN_9.MB[47].ID.R             /* Message Buffer 47 ID Register */
#define CE_CAN_9_B47            CE_CAN_9.MB[47].DATA.B           /* Message Buffer 47 B Register */
#define CE_CAN_9_H47            CE_CAN_9.MB[47].DATA.H           /* Message Buffer 47 H Register */
#define CE_CAN_9_W47            CE_CAN_9.MB[47].DATA.W           /* Message Buffer 47 W Register */
#define CE_CAN_9_CS48           CE_CAN_9.MB[48].CS.R             /* Message Buffer 48 CS Register */
#define CE_CAN_9_ID48           CE_CAN_9.MB[48].ID.R             /* Message Buffer 48 ID Register */
#define CE_CAN_9_B48            CE_CAN_9.MB[48].DATA.B           /* Message Buffer 48 B Register */
#define CE_CAN_9_H48            CE_CAN_9.MB[48].DATA.H           /* Message Buffer 48 H Register */
#define CE_CAN_9_W48            CE_CAN_9.MB[48].DATA.W           /* Message Buffer 48 W Register */
#define CE_CAN_9_CS49           CE_CAN_9.MB[49].CS.R             /* Message Buffer 49 CS Register */
#define CE_CAN_9_ID49           CE_CAN_9.MB[49].ID.R             /* Message Buffer 49 ID Register */
#define CE_CAN_9_B49            CE_CAN_9.MB[49].DATA.B           /* Message Buffer 49 B Register */
#define CE_CAN_9_H49            CE_CAN_9.MB[49].DATA.H           /* Message Buffer 49 H Register */
#define CE_CAN_9_W49            CE_CAN_9.MB[49].DATA.W           /* Message Buffer 49 W Register */
#define CE_CAN_9_CS50           CE_CAN_9.MB[50].CS.R             /* Message Buffer 50 CS Register */
#define CE_CAN_9_ID50           CE_CAN_9.MB[50].ID.R             /* Message Buffer 50 ID Register */
#define CE_CAN_9_B50            CE_CAN_9.MB[50].DATA.B           /* Message Buffer 50 B Register */
#define CE_CAN_9_H50            CE_CAN_9.MB[50].DATA.H           /* Message Buffer 50 H Register */
#define CE_CAN_9_W50            CE_CAN_9.MB[50].DATA.W           /* Message Buffer 50 W Register */
#define CE_CAN_9_CS51           CE_CAN_9.MB[51].CS.R             /* Message Buffer 51 CS Register */
#define CE_CAN_9_ID51           CE_CAN_9.MB[51].ID.R             /* Message Buffer 51 ID Register */
#define CE_CAN_9_B51            CE_CAN_9.MB[51].DATA.B           /* Message Buffer 51 B Register */
#define CE_CAN_9_H51            CE_CAN_9.MB[51].DATA.H           /* Message Buffer 51 H Register */
#define CE_CAN_9_W51            CE_CAN_9.MB[51].DATA.W           /* Message Buffer 51 W Register */
#define CE_CAN_9_CS52           CE_CAN_9.MB[52].CS.R             /* Message Buffer 52 CS Register */
#define CE_CAN_9_ID52           CE_CAN_9.MB[52].ID.R             /* Message Buffer 52 ID Register */
#define CE_CAN_9_B52            CE_CAN_9.MB[52].DATA.B           /* Message Buffer 52 B Register */
#define CE_CAN_9_H52            CE_CAN_9.MB[52].DATA.H           /* Message Buffer 52 H Register */
#define CE_CAN_9_W52            CE_CAN_9.MB[52].DATA.W           /* Message Buffer 52 W Register */
#define CE_CAN_9_CS53           CE_CAN_9.MB[53].CS.R             /* Message Buffer 53 CS Register */
#define CE_CAN_9_ID53           CE_CAN_9.MB[53].ID.R             /* Message Buffer 53 ID Register */
#define CE_CAN_9_B53            CE_CAN_9.MB[53].DATA.B           /* Message Buffer 53 B Register */
#define CE_CAN_9_H53            CE_CAN_9.MB[53].DATA.H           /* Message Buffer 53 H Register */
#define CE_CAN_9_W53            CE_CAN_9.MB[53].DATA.W           /* Message Buffer 53 W Register */
#define CE_CAN_9_CS54           CE_CAN_9.MB[54].CS.R             /* Message Buffer 54 CS Register */
#define CE_CAN_9_ID54           CE_CAN_9.MB[54].ID.R             /* Message Buffer 54 ID Register */
#define CE_CAN_9_B54            CE_CAN_9.MB[54].DATA.B           /* Message Buffer 54 B Register */
#define CE_CAN_9_H54            CE_CAN_9.MB[54].DATA.H           /* Message Buffer 54 H Register */
#define CE_CAN_9_W54            CE_CAN_9.MB[54].DATA.W           /* Message Buffer 54 W Register */
#define CE_CAN_9_CS55           CE_CAN_9.MB[55].CS.R             /* Message Buffer 55 CS Register */
#define CE_CAN_9_ID55           CE_CAN_9.MB[55].ID.R             /* Message Buffer 55 ID Register */
#define CE_CAN_9_B55            CE_CAN_9.MB[55].DATA.B           /* Message Buffer 55 B Register */
#define CE_CAN_9_H55            CE_CAN_9.MB[55].DATA.H           /* Message Buffer 55 H Register */
#define CE_CAN_9_W55            CE_CAN_9.MB[55].DATA.W           /* Message Buffer 55 W Register */
#define CE_CAN_9_CS56           CE_CAN_9.MB[56].CS.R             /* Message Buffer 56 CS Register */
#define CE_CAN_9_ID56           CE_CAN_9.MB[56].ID.R             /* Message Buffer 56 ID Register */
#define CE_CAN_9_B56            CE_CAN_9.MB[56].DATA.B           /* Message Buffer 56 B Register */
#define CE_CAN_9_H56            CE_CAN_9.MB[56].DATA.H           /* Message Buffer 56 H Register */
#define CE_CAN_9_W56            CE_CAN_9.MB[56].DATA.W           /* Message Buffer 56 W Register */
#define CE_CAN_9_CS57           CE_CAN_9.MB[57].CS.R             /* Message Buffer 57 CS Register */
#define CE_CAN_9_ID57           CE_CAN_9.MB[57].ID.R             /* Message Buffer 57 ID Register */
#define CE_CAN_9_B57            CE_CAN_9.MB[57].DATA.B           /* Message Buffer 57 B Register */
#define CE_CAN_9_H57            CE_CAN_9.MB[57].DATA.H           /* Message Buffer 57 H Register */
#define CE_CAN_9_W57            CE_CAN_9.MB[57].DATA.W           /* Message Buffer 57 W Register */
#define CE_CAN_9_CS58           CE_CAN_9.MB[58].CS.R             /* Message Buffer 58 CS Register */
#define CE_CAN_9_ID58           CE_CAN_9.MB[58].ID.R             /* Message Buffer 58 ID Register */
#define CE_CAN_9_B58            CE_CAN_9.MB[58].DATA.B           /* Message Buffer 58 B Register */
#define CE_CAN_9_H58            CE_CAN_9.MB[58].DATA.H           /* Message Buffer 58 H Register */
#define CE_CAN_9_W58            CE_CAN_9.MB[58].DATA.W           /* Message Buffer 58 W Register */
#define CE_CAN_9_CS59           CE_CAN_9.MB[59].CS.R             /* Message Buffer 59 CS Register */
#define CE_CAN_9_ID59           CE_CAN_9.MB[59].ID.R             /* Message Buffer 59 ID Register */
#define CE_CAN_9_B59            CE_CAN_9.MB[59].DATA.B           /* Message Buffer 59 B Register */
#define CE_CAN_9_H59            CE_CAN_9.MB[59].DATA.H           /* Message Buffer 59 H Register */
#define CE_CAN_9_W59            CE_CAN_9.MB[59].DATA.W           /* Message Buffer 59 W Register */
#define CE_CAN_9_CS60           CE_CAN_9.MB[60].CS.R             /* Message Buffer 60 CS Register */
#define CE_CAN_9_ID60           CE_CAN_9.MB[60].ID.R             /* Message Buffer 60 ID Register */
#define CE_CAN_9_B60            CE_CAN_9.MB[60].DATA.B           /* Message Buffer 60 B Register */
#define CE_CAN_9_H60            CE_CAN_9.MB[60].DATA.H           /* Message Buffer 60 H Register */
#define CE_CAN_9_W60            CE_CAN_9.MB[60].DATA.W           /* Message Buffer 60 W Register */
#define CE_CAN_9_CS61           CE_CAN_9.MB[61].CS.R             /* Message Buffer 61 CS Register */
#define CE_CAN_9_ID61           CE_CAN_9.MB[61].ID.R             /* Message Buffer 61 ID Register */
#define CE_CAN_9_B61            CE_CAN_9.MB[61].DATA.B           /* Message Buffer 61 B Register */
#define CE_CAN_9_H61            CE_CAN_9.MB[61].DATA.H           /* Message Buffer 61 H Register */
#define CE_CAN_9_W61            CE_CAN_9.MB[61].DATA.W           /* Message Buffer 61 W Register */
#define CE_CAN_9_CS62           CE_CAN_9.MB[62].CS.R             /* Message Buffer 62 CS Register */
#define CE_CAN_9_ID62           CE_CAN_9.MB[62].ID.R             /* Message Buffer 62 ID Register */
#define CE_CAN_9_B62            CE_CAN_9.MB[62].DATA.B           /* Message Buffer 62 B Register */
#define CE_CAN_9_H62            CE_CAN_9.MB[62].DATA.H           /* Message Buffer 62 H Register */
#define CE_CAN_9_W62            CE_CAN_9.MB[62].DATA.W           /* Message Buffer 62 W Register */
#define CE_CAN_9_CS63           CE_CAN_9.MB[63].CS.R             /* Message Buffer 63 CS Register */
#define CE_CAN_9_ID63           CE_CAN_9.MB[63].ID.R             /* Message Buffer 63 ID Register */
#define CE_CAN_9_B63            CE_CAN_9.MB[63].DATA.B           /* Message Buffer 63 B Register */
#define CE_CAN_9_H63            CE_CAN_9.MB[63].DATA.H           /* Message Buffer 63 H Register */
#define CE_CAN_9_W63            CE_CAN_9.MB[63].DATA.W           /* Message Buffer 63 W Register */
#define CE_CAN_9_CS64           CE_CAN_9.MB[64].CS.R             /* Message Buffer 64 CS Register */
#define CE_CAN_9_ID64           CE_CAN_9.MB[64].ID.R             /* Message Buffer 64 ID Register */
#define CE_CAN_9_B64            CE_CAN_9.MB[64].DATA.B           /* Message Buffer 64 B Register */
#define CE_CAN_9_H64            CE_CAN_9.MB[64].DATA.H           /* Message Buffer 64 H Register */
#define CE_CAN_9_W64            CE_CAN_9.MB[64].DATA.W           /* Message Buffer 64 W Register */
#define CE_CAN_9_CS65           CE_CAN_9.MB[65].CS.R             /* Message Buffer 65 CS Register */
#define CE_CAN_9_ID65           CE_CAN_9.MB[65].ID.R             /* Message Buffer 65 ID Register */
#define CE_CAN_9_B65            CE_CAN_9.MB[65].DATA.B           /* Message Buffer 65 B Register */
#define CE_CAN_9_H65            CE_CAN_9.MB[65].DATA.H           /* Message Buffer 65 H Register */
#define CE_CAN_9_W65            CE_CAN_9.MB[65].DATA.W           /* Message Buffer 65 W Register */
#define CE_CAN_9_CS66           CE_CAN_9.MB[66].CS.R             /* Message Buffer 66 CS Register */
#define CE_CAN_9_ID66           CE_CAN_9.MB[66].ID.R             /* Message Buffer 66 ID Register */
#define CE_CAN_9_B66            CE_CAN_9.MB[66].DATA.B           /* Message Buffer 66 B Register */
#define CE_CAN_9_H66            CE_CAN_9.MB[66].DATA.H           /* Message Buffer 66 H Register */
#define CE_CAN_9_W66            CE_CAN_9.MB[66].DATA.W           /* Message Buffer 66 W Register */
#define CE_CAN_9_CS67           CE_CAN_9.MB[67].CS.R             /* Message Buffer 67 CS Register */
#define CE_CAN_9_ID67           CE_CAN_9.MB[67].ID.R             /* Message Buffer 67 ID Register */
#define CE_CAN_9_B67            CE_CAN_9.MB[67].DATA.B           /* Message Buffer 67 B Register */
#define CE_CAN_9_H67            CE_CAN_9.MB[67].DATA.H           /* Message Buffer 67 H Register */
#define CE_CAN_9_W67            CE_CAN_9.MB[67].DATA.W           /* Message Buffer 67 W Register */
#define CE_CAN_9_CS68           CE_CAN_9.MB[68].CS.R             /* Message Buffer 68 CS Register */
#define CE_CAN_9_ID68           CE_CAN_9.MB[68].ID.R             /* Message Buffer 68 ID Register */
#define CE_CAN_9_B68            CE_CAN_9.MB[68].DATA.B           /* Message Buffer 68 B Register */
#define CE_CAN_9_H68            CE_CAN_9.MB[68].DATA.H           /* Message Buffer 68 H Register */
#define CE_CAN_9_W68            CE_CAN_9.MB[68].DATA.W           /* Message Buffer 68 W Register */
#define CE_CAN_9_CS69           CE_CAN_9.MB[69].CS.R             /* Message Buffer 69 CS Register */
#define CE_CAN_9_ID69           CE_CAN_9.MB[69].ID.R             /* Message Buffer 69 ID Register */
#define CE_CAN_9_B69            CE_CAN_9.MB[69].DATA.B           /* Message Buffer 69 B Register */
#define CE_CAN_9_H69            CE_CAN_9.MB[69].DATA.H           /* Message Buffer 69 H Register */
#define CE_CAN_9_W69            CE_CAN_9.MB[69].DATA.W           /* Message Buffer 69 W Register */
#define CE_CAN_9_CS70           CE_CAN_9.MB[70].CS.R             /* Message Buffer 70 CS Register */
#define CE_CAN_9_ID70           CE_CAN_9.MB[70].ID.R             /* Message Buffer 70 ID Register */
#define CE_CAN_9_B70            CE_CAN_9.MB[70].DATA.B           /* Message Buffer 70 B Register */
#define CE_CAN_9_H70            CE_CAN_9.MB[70].DATA.H           /* Message Buffer 70 H Register */
#define CE_CAN_9_W70            CE_CAN_9.MB[70].DATA.W           /* Message Buffer 70 W Register */
#define CE_CAN_9_CS71           CE_CAN_9.MB[71].CS.R             /* Message Buffer 71 CS Register */
#define CE_CAN_9_ID71           CE_CAN_9.MB[71].ID.R             /* Message Buffer 71 ID Register */
#define CE_CAN_9_B71            CE_CAN_9.MB[71].DATA.B           /* Message Buffer 71 B Register */
#define CE_CAN_9_H71            CE_CAN_9.MB[71].DATA.H           /* Message Buffer 71 H Register */
#define CE_CAN_9_W71            CE_CAN_9.MB[71].DATA.W           /* Message Buffer 71 W Register */
#define CE_CAN_9_CS72           CE_CAN_9.MB[72].CS.R             /* Message Buffer 72 CS Register */
#define CE_CAN_9_ID72           CE_CAN_9.MB[72].ID.R             /* Message Buffer 72 ID Register */
#define CE_CAN_9_B72            CE_CAN_9.MB[72].DATA.B           /* Message Buffer 72 B Register */
#define CE_CAN_9_H72            CE_CAN_9.MB[72].DATA.H           /* Message Buffer 72 H Register */
#define CE_CAN_9_W72            CE_CAN_9.MB[72].DATA.W           /* Message Buffer 72 W Register */
#define CE_CAN_9_CS73           CE_CAN_9.MB[73].CS.R             /* Message Buffer 73 CS Register */
#define CE_CAN_9_ID73           CE_CAN_9.MB[73].ID.R             /* Message Buffer 73 ID Register */
#define CE_CAN_9_B73            CE_CAN_9.MB[73].DATA.B           /* Message Buffer 73 B Register */
#define CE_CAN_9_H73            CE_CAN_9.MB[73].DATA.H           /* Message Buffer 73 H Register */
#define CE_CAN_9_W73            CE_CAN_9.MB[73].DATA.W           /* Message Buffer 73 W Register */
#define CE_CAN_9_CS74           CE_CAN_9.MB[74].CS.R             /* Message Buffer 74 CS Register */
#define CE_CAN_9_ID74           CE_CAN_9.MB[74].ID.R             /* Message Buffer 74 ID Register */
#define CE_CAN_9_B74            CE_CAN_9.MB[74].DATA.B           /* Message Buffer 74 B Register */
#define CE_CAN_9_H74            CE_CAN_9.MB[74].DATA.H           /* Message Buffer 74 H Register */
#define CE_CAN_9_W74            CE_CAN_9.MB[74].DATA.W           /* Message Buffer 74 W Register */
#define CE_CAN_9_CS75           CE_CAN_9.MB[75].CS.R             /* Message Buffer 75 CS Register */
#define CE_CAN_9_ID75           CE_CAN_9.MB[75].ID.R             /* Message Buffer 75 ID Register */
#define CE_CAN_9_B75            CE_CAN_9.MB[75].DATA.B           /* Message Buffer 75 B Register */
#define CE_CAN_9_H75            CE_CAN_9.MB[75].DATA.H           /* Message Buffer 75 H Register */
#define CE_CAN_9_W75            CE_CAN_9.MB[75].DATA.W           /* Message Buffer 75 W Register */
#define CE_CAN_9_CS76           CE_CAN_9.MB[76].CS.R             /* Message Buffer 76 CS Register */
#define CE_CAN_9_ID76           CE_CAN_9.MB[76].ID.R             /* Message Buffer 76 ID Register */
#define CE_CAN_9_B76            CE_CAN_9.MB[76].DATA.B           /* Message Buffer 76 B Register */
#define CE_CAN_9_H76            CE_CAN_9.MB[76].DATA.H           /* Message Buffer 76 H Register */
#define CE_CAN_9_W76            CE_CAN_9.MB[76].DATA.W           /* Message Buffer 76 W Register */
#define CE_CAN_9_CS77           CE_CAN_9.MB[77].CS.R             /* Message Buffer 77 CS Register */
#define CE_CAN_9_ID77           CE_CAN_9.MB[77].ID.R             /* Message Buffer 77 ID Register */
#define CE_CAN_9_B77            CE_CAN_9.MB[77].DATA.B           /* Message Buffer 77 B Register */
#define CE_CAN_9_H77            CE_CAN_9.MB[77].DATA.H           /* Message Buffer 77 H Register */
#define CE_CAN_9_W77            CE_CAN_9.MB[77].DATA.W           /* Message Buffer 77 W Register */
#define CE_CAN_9_CS78           CE_CAN_9.MB[78].CS.R             /* Message Buffer 78 CS Register */
#define CE_CAN_9_ID78           CE_CAN_9.MB[78].ID.R             /* Message Buffer 78 ID Register */
#define CE_CAN_9_B78            CE_CAN_9.MB[78].DATA.B           /* Message Buffer 78 B Register */
#define CE_CAN_9_H78            CE_CAN_9.MB[78].DATA.H           /* Message Buffer 78 H Register */
#define CE_CAN_9_W78            CE_CAN_9.MB[78].DATA.W           /* Message Buffer 78 W Register */
#define CE_CAN_9_CS79           CE_CAN_9.MB[79].CS.R             /* Message Buffer 79 CS Register */
#define CE_CAN_9_ID79           CE_CAN_9.MB[79].ID.R             /* Message Buffer 79 ID Register */
#define CE_CAN_9_B79            CE_CAN_9.MB[79].DATA.B           /* Message Buffer 79 B Register */
#define CE_CAN_9_H79            CE_CAN_9.MB[79].DATA.H           /* Message Buffer 79 H Register */
#define CE_CAN_9_W79            CE_CAN_9.MB[79].DATA.W           /* Message Buffer 79 W Register */
#define CE_CAN_9_CS80           CE_CAN_9.MB[80].CS.R             /* Message Buffer 80 CS Register */
#define CE_CAN_9_ID80           CE_CAN_9.MB[80].ID.R             /* Message Buffer 80 ID Register */
#define CE_CAN_9_B80            CE_CAN_9.MB[80].DATA.B           /* Message Buffer 80 B Register */
#define CE_CAN_9_H80            CE_CAN_9.MB[80].DATA.H           /* Message Buffer 80 H Register */
#define CE_CAN_9_W80            CE_CAN_9.MB[80].DATA.W           /* Message Buffer 80 W Register */
#define CE_CAN_9_CS81           CE_CAN_9.MB[81].CS.R             /* Message Buffer 81 CS Register */
#define CE_CAN_9_ID81           CE_CAN_9.MB[81].ID.R             /* Message Buffer 81 ID Register */
#define CE_CAN_9_B81            CE_CAN_9.MB[81].DATA.B           /* Message Buffer 81 B Register */
#define CE_CAN_9_H81            CE_CAN_9.MB[81].DATA.H           /* Message Buffer 81 H Register */
#define CE_CAN_9_W81            CE_CAN_9.MB[81].DATA.W           /* Message Buffer 81 W Register */
#define CE_CAN_9_CS82           CE_CAN_9.MB[82].CS.R             /* Message Buffer 82 CS Register */
#define CE_CAN_9_ID82           CE_CAN_9.MB[82].ID.R             /* Message Buffer 82 ID Register */
#define CE_CAN_9_B82            CE_CAN_9.MB[82].DATA.B           /* Message Buffer 82 B Register */
#define CE_CAN_9_H82            CE_CAN_9.MB[82].DATA.H           /* Message Buffer 82 H Register */
#define CE_CAN_9_W82            CE_CAN_9.MB[82].DATA.W           /* Message Buffer 82 W Register */
#define CE_CAN_9_CS83           CE_CAN_9.MB[83].CS.R             /* Message Buffer 83 CS Register */
#define CE_CAN_9_ID83           CE_CAN_9.MB[83].ID.R             /* Message Buffer 83 ID Register */
#define CE_CAN_9_B83            CE_CAN_9.MB[83].DATA.B           /* Message Buffer 83 B Register */
#define CE_CAN_9_H83            CE_CAN_9.MB[83].DATA.H           /* Message Buffer 83 H Register */
#define CE_CAN_9_W83            CE_CAN_9.MB[83].DATA.W           /* Message Buffer 83 W Register */
#define CE_CAN_9_CS84           CE_CAN_9.MB[84].CS.R             /* Message Buffer 84 CS Register */
#define CE_CAN_9_ID84           CE_CAN_9.MB[84].ID.R             /* Message Buffer 84 ID Register */
#define CE_CAN_9_B84            CE_CAN_9.MB[84].DATA.B           /* Message Buffer 84 B Register */
#define CE_CAN_9_H84            CE_CAN_9.MB[84].DATA.H           /* Message Buffer 84 H Register */
#define CE_CAN_9_W84            CE_CAN_9.MB[84].DATA.W           /* Message Buffer 84 W Register */
#define CE_CAN_9_CS85           CE_CAN_9.MB[85].CS.R             /* Message Buffer 85 CS Register */
#define CE_CAN_9_ID85           CE_CAN_9.MB[85].ID.R             /* Message Buffer 85 ID Register */
#define CE_CAN_9_B85            CE_CAN_9.MB[85].DATA.B           /* Message Buffer 85 B Register */
#define CE_CAN_9_H85            CE_CAN_9.MB[85].DATA.H           /* Message Buffer 85 H Register */
#define CE_CAN_9_W85            CE_CAN_9.MB[85].DATA.W           /* Message Buffer 85 W Register */
#define CE_CAN_9_CS86           CE_CAN_9.MB[86].CS.R             /* Message Buffer 86 CS Register */
#define CE_CAN_9_ID86           CE_CAN_9.MB[86].ID.R             /* Message Buffer 86 ID Register */
#define CE_CAN_9_B86            CE_CAN_9.MB[86].DATA.B           /* Message Buffer 86 B Register */
#define CE_CAN_9_H86            CE_CAN_9.MB[86].DATA.H           /* Message Buffer 86 H Register */
#define CE_CAN_9_W86            CE_CAN_9.MB[86].DATA.W           /* Message Buffer 86 W Register */
#define CE_CAN_9_CS87           CE_CAN_9.MB[87].CS.R             /* Message Buffer 87 CS Register */
#define CE_CAN_9_ID87           CE_CAN_9.MB[87].ID.R             /* Message Buffer 87 ID Register */
#define CE_CAN_9_B87            CE_CAN_9.MB[87].DATA.B           /* Message Buffer 87 B Register */
#define CE_CAN_9_H87            CE_CAN_9.MB[87].DATA.H           /* Message Buffer 87 H Register */
#define CE_CAN_9_W87            CE_CAN_9.MB[87].DATA.W           /* Message Buffer 87 W Register */
#define CE_CAN_9_CS88           CE_CAN_9.MB[88].CS.R             /* Message Buffer 88 CS Register */
#define CE_CAN_9_ID88           CE_CAN_9.MB[88].ID.R             /* Message Buffer 88 ID Register */
#define CE_CAN_9_B88            CE_CAN_9.MB[88].DATA.B           /* Message Buffer 88 B Register */
#define CE_CAN_9_H88            CE_CAN_9.MB[88].DATA.H           /* Message Buffer 88 H Register */
#define CE_CAN_9_W848           CE_CAN_9.MB[88].DATA.W           /* Message Buffer 88 W Register */
#define CE_CAN_9_CS89           CE_CAN_9.MB[89].CS.R             /* Message Buffer 89 CS Register */
#define CE_CAN_9_ID89           CE_CAN_9.MB[89].ID.R             /* Message Buffer 89 ID Register */
#define CE_CAN_9_B89            CE_CAN_9.MB[89].DATA.B           /* Message Buffer 89 B Register */
#define CE_CAN_9_H89            CE_CAN_9.MB[89].DATA.H           /* Message Buffer 89 H Register */
#define CE_CAN_9_W89            CE_CAN_9.MB[89].DATA.W           /* Message Buffer 89 W Register */
#define CE_CAN_9_CS90           CE_CAN_9.MB[90].CS.R             /* Message Buffer 90 CS Register */
#define CE_CAN_9_ID90           CE_CAN_9.MB[90].ID.R             /* Message Buffer 90 ID Register */
#define CE_CAN_9_B90            CE_CAN_9.MB[90].DATA.B           /* Message Buffer 90 B Register */
#define CE_CAN_9_H90            CE_CAN_9.MB[90].DATA.H           /* Message Buffer 90 H Register */
#define CE_CAN_9_W90            CE_CAN_9.MB[90].DATA.W           /* Message Buffer 90 W Register */
#define CE_CAN_9_CS91           CE_CAN_9.MB[91].CS.R             /* Message Buffer 91 CS Register */
#define CE_CAN_9_ID91           CE_CAN_9.MB[91].ID.R             /* Message Buffer 91 ID Register */
#define CE_CAN_9_B91            CE_CAN_9.MB[91].DATA.B           /* Message Buffer 91 B Register */
#define CE_CAN_9_H91            CE_CAN_9.MB[91].DATA.H           /* Message Buffer 91 H Register */
#define CE_CAN_9_W91            CE_CAN_9.MB[91].DATA.W           /* Message Buffer 91 W Register */
#define CE_CAN_9_CS92           CE_CAN_9.MB[92].CS.R             /* Message Buffer 92 CS Register */
#define CE_CAN_9_ID92           CE_CAN_9.MB[92].ID.R             /* Message Buffer 92 ID Register */
#define CE_CAN_9_B92            CE_CAN_9.MB[92].DATA.B           /* Message Buffer 92 B Register */
#define CE_CAN_9_H92            CE_CAN_9.MB[92].DATA.H           /* Message Buffer 92 H Register */
#define CE_CAN_9_W92            CE_CAN_9.MB[92].DATA.W           /* Message Buffer 92 W Register */
#define CE_CAN_9_CS93           CE_CAN_9.MB[93].CS.R             /* Message Buffer 93 CS Register */
#define CE_CAN_9_ID93           CE_CAN_9.MB[93].ID.R             /* Message Buffer 93 ID Register */
#define CE_CAN_9_B93            CE_CAN_9.MB[93].DATA.B           /* Message Buffer 93 B Register */
#define CE_CAN_9_H93            CE_CAN_9.MB[93].DATA.H           /* Message Buffer 93 H Register */
#define CE_CAN_9_W93            CE_CAN_9.MB[93].DATA.W           /* Message Buffer 93 W Register */
#define CE_CAN_9_CS94           CE_CAN_9.MB[94].CS.R             /* Message Buffer 94 CS Register */
#define CE_CAN_9_ID94           CE_CAN_9.MB[94].ID.R             /* Message Buffer 94 ID Register */
#define CE_CAN_9_B94            CE_CAN_9.MB[94].DATA.B           /* Message Buffer 94 B Register */
#define CE_CAN_9_H94            CE_CAN_9.MB[94].DATA.H           /* Message Buffer 94 H Register */
#define CE_CAN_9_W94            CE_CAN_9.MB[94].DATA.W           /* Message Buffer 94 W Register */
#define CE_CAN_9_CS95           CE_CAN_9.MB[95].CS.R             /* Message Buffer 95 CS Register */
#define CE_CAN_9_ID95           CE_CAN_9.MB[95].ID.R             /* Message Buffer 95 ID Register */
#define CE_CAN_9_B95            CE_CAN_9.MB[95].DATA.B           /* Message Buffer 95 B Register */
#define CE_CAN_9_H95            CE_CAN_9.MB[95].DATA.H           /* Message Buffer 95 H Register */
#define CE_CAN_9_W95            CE_CAN_9.MB[95].DATA.W           /* Message Buffer 95 W Register */
#define CE_CAN_9_CS96           CE_CAN_9.MB[96].CS.R             /* Message Buffer 96 CS Register */
#define CE_CAN_9_ID96           CE_CAN_9.MB[96].ID.R             /* Message Buffer 96 ID Register */
#define CE_CAN_9_B96            CE_CAN_9.MB[96].DATA.B           /* Message Buffer 96 B Register */
#define CE_CAN_9_H96            CE_CAN_9.MB[96].DATA.H           /* Message Buffer 96 H Register */
#define CE_CAN_9_W96            CE_CAN_9.MB[96].DATA.W           /* Message Buffer 96 W Register */
#define CE_CAN_9_CS97           CE_CAN_9.MB[97].CS.R             /* Message Buffer 97 CS Register */
#define CE_CAN_9_ID97           CE_CAN_9.MB[97].ID.R             /* Message Buffer 97 ID Register */
#define CE_CAN_9_B97            CE_CAN_9.MB[97].DATA.B           /* Message Buffer 97 B Register */
#define CE_CAN_9_H97            CE_CAN_9.MB[97].DATA.H           /* Message Buffer 97 H Register */
#define CE_CAN_9_W97            CE_CAN_9.MB[97].DATA.W           /* Message Buffer 97 W Register */
#define CE_CAN_9_CS98           CE_CAN_9.MB[98].CS.R             /* Message Buffer 98 CS Register */
#define CE_CAN_9_ID98           CE_CAN_9.MB[98].ID.R             /* Message Buffer 98 ID Register */
#define CE_CAN_9_B98            CE_CAN_9.MB[98].DATA.B           /* Message Buffer 98 B Register */
#define CE_CAN_9_H98            CE_CAN_9.MB[98].DATA.H           /* Message Buffer 98 H Register */
#define CE_CAN_9_W98            CE_CAN_9.MB[98].DATA.W           /* Message Buffer 98 W Register */
#define CE_CAN_9_CS99           CE_CAN_9.MB[99].CS.R             /* Message Buffer 99 CS Register */
#define CE_CAN_9_ID99           CE_CAN_9.MB[99].ID.R             /* Message Buffer 99 ID Register */
#define CE_CAN_9_B99            CE_CAN_9.MB[99].DATA.B           /* Message Buffer 99 B Register */
#define CE_CAN_9_H99            CE_CAN_9.MB[99].DATA.H           /* Message Buffer 99 H Register */
#define CE_CAN_9_W99            CE_CAN_9.MB[99].DATA.W           /* Message Buffer 99 W Register */
#define CE_CAN_9_CS100          CE_CAN_9.MB[100].CS.R            /* Message Buffer 100 CS Register */
#define CE_CAN_9_ID100          CE_CAN_9.MB[100].ID.R            /* Message Buffer 100 ID Register */
#define CE_CAN_9_B100           CE_CAN_9.MB[100].DATA.B          /* Message Buffer 100 B Register */
#define CE_CAN_9_H100           CE_CAN_9.MB[100].DATA.H          /* Message Buffer 100 H Register */
#define CE_CAN_9_W100           CE_CAN_9.MB[100].DATA.W          /* Message Buffer 100 W Register */
#define CE_CAN_9_CS101          CE_CAN_9.MB[101].CS.R            /* Message Buffer 101 CS Register */
#define CE_CAN_9_ID101          CE_CAN_9.MB[101].ID.R            /* Message Buffer 101 ID Register */
#define CE_CAN_9_B101           CE_CAN_9.MB[101].DATA.B          /* Message Buffer 101 B Register */
#define CE_CAN_9_H101           CE_CAN_9.MB[101].DATA.H          /* Message Buffer 101 H Register */
#define CE_CAN_9_W101           CE_CAN_9.MB[101].DATA.W          /* Message Buffer 101 W Register */
#define CE_CAN_9_CS102          CE_CAN_9.MB[102].CS.R            /* Message Buffer 102 CS Register */
#define CE_CAN_9_ID102          CE_CAN_9.MB[102].ID.R            /* Message Buffer 102 ID Register */
#define CE_CAN_9_B102           CE_CAN_9.MB[102].DATA.B          /* Message Buffer 102 B Register */
#define CE_CAN_9_H102           CE_CAN_9.MB[102].DATA.H          /* Message Buffer 102 H Register */
#define CE_CAN_9_W102           CE_CAN_9.MB[102].DATA.W          /* Message Buffer 102 W Register */
#define CE_CAN_9_CS103          CE_CAN_9.MB[103].CS.R            /* Message Buffer 103 CS Register */
#define CE_CAN_9_ID103          CE_CAN_9.MB[103].ID.R            /* Message Buffer 103 ID Register */
#define CE_CAN_9_B103           CE_CAN_9.MB[103].DATA.B          /* Message Buffer 103 B Register */
#define CE_CAN_9_H103           CE_CAN_9.MB[103].DATA.H          /* Message Buffer 103 H Register */
#define CE_CAN_9_W103           CE_CAN_9.MB[103].DATA.W          /* Message Buffer 103 W Register */
#define CE_CAN_9_CS104          CE_CAN_9.MB[104].CS.R            /* Message Buffer 104 CS Register */
#define CE_CAN_9_ID104          CE_CAN_9.MB[104].ID.R            /* Message Buffer 104 ID Register */
#define CE_CAN_9_B104           CE_CAN_9.MB[104].DATA.B          /* Message Buffer 104 B Register */
#define CE_CAN_9_H104           CE_CAN_9.MB[104].DATA.H          /* Message Buffer 104 H Register */
#define CE_CAN_9_W104           CE_CAN_9.MB[104].DATA.W          /* Message Buffer 104 W Register */
#define CE_CAN_9_CS105          CE_CAN_9.MB[105].CS.R            /* Message Buffer 105 CS Register */
#define CE_CAN_9_ID105          CE_CAN_9.MB[105].ID.R            /* Message Buffer 105 ID Register */
#define CE_CAN_9_B105           CE_CAN_9.MB[105].DATA.B          /* Message Buffer 105 B Register */
#define CE_CAN_9_H105           CE_CAN_9.MB[105].DATA.H          /* Message Buffer 105 H Register */
#define CE_CAN_9_W105           CE_CAN_9.MB[105].DATA.W          /* Message Buffer 105 W Register */
#define CE_CAN_9_CS106          CE_CAN_9.MB[106].CS.R            /* Message Buffer 106 CS Register */
#define CE_CAN_9_ID106          CE_CAN_9.MB[106].ID.R            /* Message Buffer 106 ID Register */
#define CE_CAN_9_B106           CE_CAN_9.MB[106].DATA.B          /* Message Buffer 106 B Register */
#define CE_CAN_9_H106           CE_CAN_9.MB[106].DATA.H          /* Message Buffer 106 H Register */
#define CE_CAN_9_W106           CE_CAN_9.MB[106].DATA.W          /* Message Buffer 106 W Register */
#define CE_CAN_9_CS107          CE_CAN_9.MB[107].CS.R            /* Message Buffer 107 CS Register */
#define CE_CAN_9_ID107          CE_CAN_9.MB[107].ID.R            /* Message Buffer 107 ID Register */
#define CE_CAN_9_B107           CE_CAN_9.MB[107].DATA.B          /* Message Buffer 107 B Register */
#define CE_CAN_9_H107           CE_CAN_9.MB[107].DATA.H          /* Message Buffer 107 H Register */
#define CE_CAN_9_W107           CE_CAN_9.MB[107].DATA.W          /* Message Buffer 107 W Register */
#define CE_CAN_9_CS108          CE_CAN_9.MB[108].CS.R            /* Message Buffer 108 CS Register */
#define CE_CAN_9_ID108          CE_CAN_9.MB[108].ID.R            /* Message Buffer 108 ID Register */
#define CE_CAN_9_B108           CE_CAN_9.MB[108].DATA.B          /* Message Buffer 108 B Register */
#define CE_CAN_9_H108           CE_CAN_9.MB[108].DATA.H          /* Message Buffer 108 H Register */
#define CE_CAN_9_W108           CE_CAN_9.MB[108].DATA.W          /* Message Buffer 108 W Register */
#define CE_CAN_9_CS109          CE_CAN_9.MB[109].CS.R            /* Message Buffer 109 CS Register */
#define CE_CAN_9_ID109          CE_CAN_9.MB[109].ID.R            /* Message Buffer 109 ID Register */
#define CE_CAN_9_B109           CE_CAN_9.MB[109].DATA.B          /* Message Buffer 109 B Register */
#define CE_CAN_9_H109           CE_CAN_9.MB[109].DATA.H          /* Message Buffer 109 H Register */
#define CE_CAN_9_W109           CE_CAN_9.MB[109].DATA.W          /* Message Buffer 109 W Register */
#define CE_CAN_9_CS110          CE_CAN_9.MB[110].CS.R            /* Message Buffer 110 CS Register */
#define CE_CAN_9_ID110          CE_CAN_9.MB[110].ID.R            /* Message Buffer 110 ID Register */
#define CE_CAN_9_B110           CE_CAN_9.MB[110].DATA.B          /* Message Buffer 110 B Register */
#define CE_CAN_9_H110           CE_CAN_9.MB[110].DATA.H          /* Message Buffer 110 H Register */
#define CE_CAN_9_W110           CE_CAN_9.MB[110].DATA.W          /* Message Buffer 110 W Register */
#define CE_CAN_9_CS111          CE_CAN_9.MB[111].CS.R            /* Message Buffer 111 CS Register */
#define CE_CAN_9_ID111          CE_CAN_9.MB[111].ID.R            /* Message Buffer 111 ID Register */
#define CE_CAN_9_B111           CE_CAN_9.MB[111].DATA.B          /* Message Buffer 111 B Register */
#define CE_CAN_9_H111           CE_CAN_9.MB[111].DATA.H          /* Message Buffer 111 H Register */
#define CE_CAN_9_W111           CE_CAN_9.MB[111].DATA.W          /* Message Buffer 111 W Register */
#define CE_CAN_9_CS112          CE_CAN_9.MB[112].CS.R            /* Message Buffer 112 CS Register */
#define CE_CAN_9_ID112          CE_CAN_9.MB[112].ID.R            /* Message Buffer 112 ID Register */
#define CE_CAN_9_B112           CE_CAN_9.MB[112].DATA.B          /* Message Buffer 112 B Register */
#define CE_CAN_9_H112           CE_CAN_9.MB[112].DATA.H          /* Message Buffer 112 H Register */
#define CE_CAN_9_W112           CE_CAN_9.MB[112].DATA.W          /* Message Buffer 112 W Register */
#define CE_CAN_9_CS113          CE_CAN_9.MB[113].CS.R            /* Message Buffer 113 CS Register */
#define CE_CAN_9_ID113          CE_CAN_9.MB[113].ID.R            /* Message Buffer 113 ID Register */
#define CE_CAN_9_B113           CE_CAN_9.MB[113].DATA.B          /* Message Buffer 113 B Register */
#define CE_CAN_9_H113           CE_CAN_9.MB[113].DATA.H          /* Message Buffer 113 H Register */
#define CE_CAN_9_W113           CE_CAN_9.MB[113].DATA.W          /* Message Buffer 113 W Register */
#define CE_CAN_9_CS114          CE_CAN_9.MB[114].CS.R            /* Message Buffer 114 CS Register */
#define CE_CAN_9_ID114          CE_CAN_9.MB[114].ID.R            /* Message Buffer 114 ID Register */
#define CE_CAN_9_B114           CE_CAN_9.MB[114].DATA.B          /* Message Buffer 114 B Register */
#define CE_CAN_9_H114           CE_CAN_9.MB[114].DATA.H          /* Message Buffer 114 H Register */
#define CE_CAN_9_W114           CE_CAN_9.MB[114].DATA.W          /* Message Buffer 114 W Register */
#define CE_CAN_9_CS115          CE_CAN_9.MB[115].CS.R            /* Message Buffer 115 CS Register */
#define CE_CAN_9_ID115          CE_CAN_9.MB[115].ID.R            /* Message Buffer 115 ID Register */
#define CE_CAN_9_B115           CE_CAN_9.MB[115].DATA.B          /* Message Buffer 115 B Register */
#define CE_CAN_9_H115           CE_CAN_9.MB[115].DATA.H          /* Message Buffer 115 H Register */
#define CE_CAN_9_W115           CE_CAN_9.MB[115].DATA.W          /* Message Buffer 115 W Register */
#define CE_CAN_9_CS116          CE_CAN_9.MB[116].CS.R            /* Message Buffer 116 CS Register */
#define CE_CAN_9_ID116          CE_CAN_9.MB[116].ID.R            /* Message Buffer 116 ID Register */
#define CE_CAN_9_B116           CE_CAN_9.MB[116].DATA.B          /* Message Buffer 116 B Register */
#define CE_CAN_9_H116           CE_CAN_9.MB[116].DATA.H          /* Message Buffer 116 H Register */
#define CE_CAN_9_W116           CE_CAN_9.MB[116].DATA.W          /* Message Buffer 116 W Register */
#define CE_CAN_9_CS117          CE_CAN_9.MB[117].CS.R            /* Message Buffer 117 CS Register */
#define CE_CAN_9_ID117          CE_CAN_9.MB[117].ID.R            /* Message Buffer 117 ID Register */
#define CE_CAN_9_B117           CE_CAN_9.MB[117].DATA.B          /* Message Buffer 117 B Register */
#define CE_CAN_9_H117           CE_CAN_9.MB[117].DATA.H          /* Message Buffer 117 H Register */
#define CE_CAN_9_W117           CE_CAN_9.MB[117].DATA.W          /* Message Buffer 117 W Register */
#define CE_CAN_9_CS118          CE_CAN_9.MB[118].CS.R            /* Message Buffer 118 CS Register */
#define CE_CAN_9_ID118          CE_CAN_9.MB[118].ID.R            /* Message Buffer 118 ID Register */
#define CE_CAN_9_B118           CE_CAN_9.MB[118].DATA.B          /* Message Buffer 118 B Register */
#define CE_CAN_9_H118           CE_CAN_9.MB[118].DATA.H          /* Message Buffer 118 H Register */
#define CE_CAN_9_W118           CE_CAN_9.MB[118].DATA.W          /* Message Buffer 118 W Register */
#define CE_CAN_9_CS119          CE_CAN_9.MB[119].CS.R            /* Message Buffer 119 CS Register */
#define CE_CAN_9_ID119          CE_CAN_9.MB[119].ID.R            /* Message Buffer 119 ID Register */
#define CE_CAN_9_B119           CE_CAN_9.MB[119].DATA.B          /* Message Buffer 119 B Register */
#define CE_CAN_9_H119           CE_CAN_9.MB[119].DATA.H          /* Message Buffer 119 H Register */
#define CE_CAN_9_W119           CE_CAN_9.MB[119].DATA.W          /* Message Buffer 119 W Register */
#define CE_CAN_9_CS120          CE_CAN_9.MB[120].CS.R            /* Message Buffer 120 CS Register */
#define CE_CAN_9_ID120          CE_CAN_9.MB[120].ID.R            /* Message Buffer 120 ID Register */
#define CE_CAN_9_B120           CE_CAN_9.MB[120].DATA.B          /* Message Buffer 120 B Register */
#define CE_CAN_9_H120           CE_CAN_9.MB[120].DATA.H          /* Message Buffer 120 H Register */
#define CE_CAN_9_W120           CE_CAN_9.MB[120].DATA.W          /* Message Buffer 120 W Register */
#define CE_CAN_9_CS121          CE_CAN_9.MB[121].CS.R            /* Message Buffer 121 CS Register */
#define CE_CAN_9_ID121          CE_CAN_9.MB[121].ID.R            /* Message Buffer 121 ID Register */
#define CE_CAN_9_B121           CE_CAN_9.MB[121].DATA.B          /* Message Buffer 121 B Register */
#define CE_CAN_9_H121           CE_CAN_9.MB[121].DATA.H          /* Message Buffer 121 H Register */
#define CE_CAN_9_W121           CE_CAN_9.MB[121].DATA.W          /* Message Buffer 121 W Register */
#define CE_CAN_9_CS122          CE_CAN_9.MB[122].CS.R            /* Message Buffer 122 CS Register */
#define CE_CAN_9_ID122          CE_CAN_9.MB[122].ID.R            /* Message Buffer 122 ID Register */
#define CE_CAN_9_B122           CE_CAN_9.MB[122].DATA.B          /* Message Buffer 122 B Register */
#define CE_CAN_9_H122           CE_CAN_9.MB[122].DATA.H          /* Message Buffer 122 H Register */
#define CE_CAN_9_W122           CE_CAN_9.MB[122].DATA.W          /* Message Buffer 122 W Register */
#define CE_CAN_9_CS123          CE_CAN_9.MB[123].CS.R            /* Message Buffer 123 CS Register */
#define CE_CAN_9_ID123          CE_CAN_9.MB[123].ID.R            /* Message Buffer 123 ID Register */
#define CE_CAN_9_B123           CE_CAN_9.MB[123].DATA.B          /* Message Buffer 123 B Register */
#define CE_CAN_9_H123           CE_CAN_9.MB[123].DATA.H          /* Message Buffer 123 H Register */
#define CE_CAN_9_W123           CE_CAN_9.MB[123].DATA.W          /* Message Buffer 123 W Register */
#define CE_CAN_9_CS124          CE_CAN_9.MB[124].CS.R            /* Message Buffer 124 CS Register */
#define CE_CAN_9_ID124          CE_CAN_9.MB[124].ID.R            /* Message Buffer 124 ID Register */
#define CE_CAN_9_B124           CE_CAN_9.MB[124].DATA.B          /* Message Buffer 124 B Register */
#define CE_CAN_9_H124           CE_CAN_9.MB[124].DATA.H          /* Message Buffer 124 H Register */
#define CE_CAN_9_W124           CE_CAN_9.MB[124].DATA.W          /* Message Buffer 124 W Register */
#define CE_CAN_9_CS125          CE_CAN_9.MB[125].CS.R            /* Message Buffer 125 CS Register */
#define CE_CAN_9_ID125          CE_CAN_9.MB[125].ID.R            /* Message Buffer 125 ID Register */
#define CE_CAN_9_B125           CE_CAN_9.MB[125].DATA.B          /* Message Buffer 125 B Register */
#define CE_CAN_9_H125           CE_CAN_9.MB[125].DATA.H          /* Message Buffer 125 H Register */
#define CE_CAN_9_W125           CE_CAN_9.MB[125].DATA.W          /* Message Buffer 125 W Register */
#define CE_CAN_9_CS126          CE_CAN_9.MB[126].CS.R            /* Message Buffer 126 CS Register */
#define CE_CAN_9_ID126          CE_CAN_9.MB[126].ID.R            /* Message Buffer 126 ID Register */
#define CE_CAN_9_B126           CE_CAN_9.MB[126].DATA.B          /* Message Buffer 126 B Register */
#define CE_CAN_9_H126           CE_CAN_9.MB[126].DATA.H          /* Message Buffer 126 H Register */
#define CE_CAN_9_W126           CE_CAN_9.MB[126].DATA.W          /* Message Buffer 126 W Register */
#define CE_CAN_9_CS127          CE_CAN_9.MB[127].CS.R            /* Message Buffer 127 CS Register */
#define CE_CAN_9_ID127          CE_CAN_9.MB[127].ID.R            /* Message Buffer 127 ID Register */
#define CE_CAN_9_B127           CE_CAN_9.MB[127].DATA.B          /* Message Buffer 127 B Register */
#define CE_CAN_9_H127           CE_CAN_9.MB[127].DATA.H          /* Message Buffer 127 H Register */
#define CE_CAN_9_W127           CE_CAN_9.MB[127].DATA.W          /* Message Buffer 127 W Register */

#define CE_CAN_9_RXIMR0      CE_CAN_9.RXIMR[0].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR1      CE_CAN_9.RXIMR[1].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR2      CE_CAN_9.RXIMR[2].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR3      CE_CAN_9.RXIMR[3].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR4      CE_CAN_9.RXIMR[4].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR5      CE_CAN_9.RXIMR[5].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR6      CE_CAN_9.RXIMR[6].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR7      CE_CAN_9.RXIMR[7].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR8      CE_CAN_9.RXIMR[8].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR9      CE_CAN_9.RXIMR[9].R           /* Receive Individual Mask */
#define CE_CAN_9_RXIMR10     CE_CAN_9.RXIMR[10].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR11     CE_CAN_9.RXIMR[11].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR12     CE_CAN_9.RXIMR[12].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR13     CE_CAN_9.RXIMR[13].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR14     CE_CAN_9.RXIMR[14].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR15     CE_CAN_9.RXIMR[15].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR16     CE_CAN_9.RXIMR[16].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR17     CE_CAN_9.RXIMR[17].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR18     CE_CAN_9.RXIMR[18].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR19     CE_CAN_9.RXIMR[19].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR20     CE_CAN_9.RXIMR[20].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR21     CE_CAN_9.RXIMR[21].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR22     CE_CAN_9.RXIMR[22].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR23     CE_CAN_9.RXIMR[23].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR24     CE_CAN_9.RXIMR[24].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR25     CE_CAN_9.RXIMR[25].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR26     CE_CAN_9.RXIMR[26].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR27     CE_CAN_9.RXIMR[27].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR28     CE_CAN_9.RXIMR[28].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR29     CE_CAN_9.RXIMR[29].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR30     CE_CAN_9.RXIMR[30].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR31     CE_CAN_9.RXIMR[31].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR32     CE_CAN_9.RXIMR[32].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR33     CE_CAN_9.RXIMR[33].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR34     CE_CAN_9.RXIMR[34].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR35     CE_CAN_9.RXIMR[35].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR36     CE_CAN_9.RXIMR[36].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR37     CE_CAN_9.RXIMR[37].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR38     CE_CAN_9.RXIMR[38].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR39     CE_CAN_9.RXIMR[39].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR40     CE_CAN_9.RXIMR[40].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR41     CE_CAN_9.RXIMR[41].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR42     CE_CAN_9.RXIMR[42].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR43     CE_CAN_9.RXIMR[43].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR44     CE_CAN_9.RXIMR[44].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR45     CE_CAN_9.RXIMR[45].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR46     CE_CAN_9.RXIMR[46].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR47     CE_CAN_9.RXIMR[47].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR48     CE_CAN_9.RXIMR[48].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR49     CE_CAN_9.RXIMR[49].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR50     CE_CAN_9.RXIMR[50].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR51     CE_CAN_9.RXIMR[51].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR52     CE_CAN_9.RXIMR[52].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR53     CE_CAN_9.RXIMR[53].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR54     CE_CAN_9.RXIMR[54].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR55     CE_CAN_9.RXIMR[55].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR56     CE_CAN_9.RXIMR[56].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR57     CE_CAN_9.RXIMR[57].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR58     CE_CAN_9.RXIMR[58].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR59     CE_CAN_9.RXIMR[59].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR60     CE_CAN_9.RXIMR[60].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR61     CE_CAN_9.RXIMR[61].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR62     CE_CAN_9.RXIMR[62].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR63     CE_CAN_9.RXIMR[63].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR64     CE_CAN_9.RXIMR[64].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR65     CE_CAN_9.RXIMR[65].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR66     CE_CAN_9.RXIMR[66].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR67     CE_CAN_9.RXIMR[67].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR68     CE_CAN_9.RXIMR[68].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR69     CE_CAN_9.RXIMR[69].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR70     CE_CAN_9.RXIMR[70].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR71     CE_CAN_9.RXIMR[71].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR72     CE_CAN_9.RXIMR[72].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR73     CE_CAN_9.RXIMR[73].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR74     CE_CAN_9.RXIMR[74].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR75     CE_CAN_9.RXIMR[75].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR76     CE_CAN_9.RXIMR[76].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR77     CE_CAN_9.RXIMR[77].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR78     CE_CAN_9.RXIMR[78].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR79     CE_CAN_9.RXIMR[79].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR80     CE_CAN_9.RXIMR[80].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR81     CE_CAN_9.RXIMR[81].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR82     CE_CAN_9.RXIMR[82].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR83     CE_CAN_9.RXIMR[83].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR84     CE_CAN_9.RXIMR[84].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR85     CE_CAN_9.RXIMR[85].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR86     CE_CAN_9.RXIMR[86].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR87     CE_CAN_9.RXIMR[87].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR88     CE_CAN_9.RXIMR[88].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR89     CE_CAN_9.RXIMR[89].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR90     CE_CAN_9.RXIMR[90].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR91     CE_CAN_9.RXIMR[91].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR92     CE_CAN_9.RXIMR[92].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR93     CE_CAN_9.RXIMR[93].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR94     CE_CAN_9.RXIMR[94].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR95     CE_CAN_9.RXIMR[95].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR96     CE_CAN_9.RXIMR[96].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR97     CE_CAN_9.RXIMR[97].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR98     CE_CAN_9.RXIMR[98].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR99     CE_CAN_9.RXIMR[99].R          /* Receive Individual Mask */
#define CE_CAN_9_RXIMR100    CE_CAN_9.RXIMR[100].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR101    CE_CAN_9.RXIMR[101].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR102    CE_CAN_9.RXIMR[102].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR103    CE_CAN_9.RXIMR[103].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR104    CE_CAN_9.RXIMR[104].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR105    CE_CAN_9.RXIMR[105].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR106    CE_CAN_9.RXIMR[106].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR107    CE_CAN_9.RXIMR[107].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR108    CE_CAN_9.RXIMR[108].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR109    CE_CAN_9.RXIMR[109].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR110    CE_CAN_9.RXIMR[110].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR111    CE_CAN_9.RXIMR[111].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR112    CE_CAN_9.RXIMR[112].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR113    CE_CAN_9.RXIMR[113].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR114    CE_CAN_9.RXIMR[114].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR115    CE_CAN_9.RXIMR[115].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR116    CE_CAN_9.RXIMR[116].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR117    CE_CAN_9.RXIMR[117].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR118    CE_CAN_9.RXIMR[118].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR119    CE_CAN_9.RXIMR[119].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR120    CE_CAN_9.RXIMR[120].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR121    CE_CAN_9.RXIMR[121].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR122    CE_CAN_9.RXIMR[122].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR123    CE_CAN_9.RXIMR[123].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR124    CE_CAN_9.RXIMR[124].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR125    CE_CAN_9.RXIMR[125].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR126    CE_CAN_9.RXIMR[126].R         /* Receive Individual Mask */
#define CE_CAN_9_RXIMR127    CE_CAN_9.RXIMR[127].R         /* Receive Individual Mask */
#define CE_CAN_9_MECR        CE_CAN_9.MECR.R               /* Memory Error Control */
#define CE_CAN_9_ERRIAR      CE_CAN_9.ERRIAR.R             /* Error Injection Address */
#define CE_CAN_9_ERRIDPR     CE_CAN_9.ERRIDPR.R            /* Error Injection Data Pattern */
#define CE_CAN_9_ERRIPPR     CE_CAN_9.ERRIPPR.R            /* Error Injection Parity Pattern */
#define CE_CAN_9_RERRAR      CE_CAN_9.RERRAR.R             /* Error Report Address */
#define CE_CAN_9_RERRDR      CE_CAN_9.RERRDR.R             /* Error Report Data */
#define CE_CAN_9_RERRSYNR    CE_CAN_9.RERRSYNR.R           /* Error Report Syndrome */
#define CE_CAN_9_ERRSR       CE_CAN_9.ERRSR.R              /* Error Status */
#define CE_CAN_9_EPRS        CE_CAN_9.EPRS.R               /* Enhanced CAN Bit Timing Prescalers */
#define CE_CAN_9_ENCBT       CE_CAN_9.ENCBT.R              /* Enhanced Nominal CAN Bit Timing */
#define CE_CAN_9_EDCBT       CE_CAN_9.EDCBT.R              /* Enhanced Data Phase CAN Bit Timing */
#define CE_CAN_9_ETDC        CE_CAN_9.ETDC.R               /* Enhanced Transceiver Delay Compensation */
#define CE_CAN_9_FDCTRL      CE_CAN_9.FDCTRL.R             /* CAN FD Control */
#define CE_CAN_9_FDCBT       CE_CAN_9.FDCBT.R              /* CAN FD Bit Timing */
#define CE_CAN_9_FDCRC       CE_CAN_9.FDCRC.R              /* CAN FD CRC */
#define CE_CAN_9_ERFCR       CE_CAN_9.ERFCR.R              /* Enhanced RX FIFO Control */
#define CE_CAN_9_ERFIER      CE_CAN_9.ERFIER.R             /* Enhanced RX FIFO Interrupt Enable */
#define CE_CAN_9_ERFSR       CE_CAN_9.ERFSR.R              /* Enhanced RX FIFO Status */
#define CE_CAN_9_HR_TIME_STAMP0 CE_CAN_9.HR_TIME_STAMP[0].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP1 CE_CAN_9.HR_TIME_STAMP[1].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP2 CE_CAN_9.HR_TIME_STAMP[2].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP3 CE_CAN_9.HR_TIME_STAMP[3].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP4 CE_CAN_9.HR_TIME_STAMP[4].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP5 CE_CAN_9.HR_TIME_STAMP[5].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP6 CE_CAN_9.HR_TIME_STAMP[6].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP7 CE_CAN_9.HR_TIME_STAMP[7].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP8 CE_CAN_9.HR_TIME_STAMP[8].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP9 CE_CAN_9.HR_TIME_STAMP[9].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP10 CE_CAN_9.HR_TIME_STAMP[10].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP11 CE_CAN_9.HR_TIME_STAMP[11].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP12 CE_CAN_9.HR_TIME_STAMP[12].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP13 CE_CAN_9.HR_TIME_STAMP[13].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP14 CE_CAN_9.HR_TIME_STAMP[14].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP15 CE_CAN_9.HR_TIME_STAMP[15].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP16 CE_CAN_9.HR_TIME_STAMP[16].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP17 CE_CAN_9.HR_TIME_STAMP[17].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP18 CE_CAN_9.HR_TIME_STAMP[18].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP19 CE_CAN_9.HR_TIME_STAMP[19].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP20 CE_CAN_9.HR_TIME_STAMP[20].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP21 CE_CAN_9.HR_TIME_STAMP[21].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP22 CE_CAN_9.HR_TIME_STAMP[22].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP23 CE_CAN_9.HR_TIME_STAMP[23].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP24 CE_CAN_9.HR_TIME_STAMP[24].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP25 CE_CAN_9.HR_TIME_STAMP[25].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP26 CE_CAN_9.HR_TIME_STAMP[26].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP27 CE_CAN_9.HR_TIME_STAMP[27].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP28 CE_CAN_9.HR_TIME_STAMP[28].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP29 CE_CAN_9.HR_TIME_STAMP[29].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP30 CE_CAN_9.HR_TIME_STAMP[30].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP31 CE_CAN_9.HR_TIME_STAMP[31].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP32 CE_CAN_9.HR_TIME_STAMP[32].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP33 CE_CAN_9.HR_TIME_STAMP[33].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP34 CE_CAN_9.HR_TIME_STAMP[34].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP35 CE_CAN_9.HR_TIME_STAMP[35].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP36 CE_CAN_9.HR_TIME_STAMP[36].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP37 CE_CAN_9.HR_TIME_STAMP[37].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP38 CE_CAN_9.HR_TIME_STAMP[38].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP39 CE_CAN_9.HR_TIME_STAMP[39].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP40 CE_CAN_9.HR_TIME_STAMP[40].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP41 CE_CAN_9.HR_TIME_STAMP[41].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP42 CE_CAN_9.HR_TIME_STAMP[42].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP43 CE_CAN_9.HR_TIME_STAMP[43].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP44 CE_CAN_9.HR_TIME_STAMP[44].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP45 CE_CAN_9.HR_TIME_STAMP[45].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP46 CE_CAN_9.HR_TIME_STAMP[46].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP47 CE_CAN_9.HR_TIME_STAMP[47].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP48 CE_CAN_9.HR_TIME_STAMP[48].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP49 CE_CAN_9.HR_TIME_STAMP[49].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP50 CE_CAN_9.HR_TIME_STAMP[50].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP51 CE_CAN_9.HR_TIME_STAMP[51].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP52 CE_CAN_9.HR_TIME_STAMP[52].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP53 CE_CAN_9.HR_TIME_STAMP[53].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP54 CE_CAN_9.HR_TIME_STAMP[54].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP55 CE_CAN_9.HR_TIME_STAMP[55].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP56 CE_CAN_9.HR_TIME_STAMP[56].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP57 CE_CAN_9.HR_TIME_STAMP[57].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP58 CE_CAN_9.HR_TIME_STAMP[58].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP59 CE_CAN_9.HR_TIME_STAMP[59].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP60 CE_CAN_9.HR_TIME_STAMP[60].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP61 CE_CAN_9.HR_TIME_STAMP[61].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP62 CE_CAN_9.HR_TIME_STAMP[62].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP63 CE_CAN_9.HR_TIME_STAMP[63].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP64 CE_CAN_9.HR_TIME_STAMP[64].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP65 CE_CAN_9.HR_TIME_STAMP[65].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP66 CE_CAN_9.HR_TIME_STAMP[66].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP67 CE_CAN_9.HR_TIME_STAMP[67].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP68 CE_CAN_9.HR_TIME_STAMP[68].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP69 CE_CAN_9.HR_TIME_STAMP[69].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP70 CE_CAN_9.HR_TIME_STAMP[70].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP71 CE_CAN_9.HR_TIME_STAMP[71].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP72 CE_CAN_9.HR_TIME_STAMP[72].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP73 CE_CAN_9.HR_TIME_STAMP[73].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP74 CE_CAN_9.HR_TIME_STAMP[74].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP75 CE_CAN_9.HR_TIME_STAMP[75].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP76 CE_CAN_9.HR_TIME_STAMP[76].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP77 CE_CAN_9.HR_TIME_STAMP[77].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP78 CE_CAN_9.HR_TIME_STAMP[78].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP79 CE_CAN_9.HR_TIME_STAMP[79].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP80 CE_CAN_9.HR_TIME_STAMP[80].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP81 CE_CAN_9.HR_TIME_STAMP[81].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP82 CE_CAN_9.HR_TIME_STAMP[82].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP83 CE_CAN_9.HR_TIME_STAMP[83].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP84 CE_CAN_9.HR_TIME_STAMP[84].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP85 CE_CAN_9.HR_TIME_STAMP[85].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP86 CE_CAN_9.HR_TIME_STAMP[86].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP87 CE_CAN_9.HR_TIME_STAMP[87].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP88 CE_CAN_9.HR_TIME_STAMP[88].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP89 CE_CAN_9.HR_TIME_STAMP[89].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP90 CE_CAN_9.HR_TIME_STAMP[90].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP91 CE_CAN_9.HR_TIME_STAMP[91].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP92 CE_CAN_9.HR_TIME_STAMP[92].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP93 CE_CAN_9.HR_TIME_STAMP[93].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP94 CE_CAN_9.HR_TIME_STAMP[94].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP95 CE_CAN_9.HR_TIME_STAMP[95].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP96 CE_CAN_9.HR_TIME_STAMP[96].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP97 CE_CAN_9.HR_TIME_STAMP[97].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP98 CE_CAN_9.HR_TIME_STAMP[98].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP99 CE_CAN_9.HR_TIME_STAMP[99].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP100 CE_CAN_9.HR_TIME_STAMP[100].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP101 CE_CAN_9.HR_TIME_STAMP[101].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP102 CE_CAN_9.HR_TIME_STAMP[102].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP103 CE_CAN_9.HR_TIME_STAMP[103].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP104 CE_CAN_9.HR_TIME_STAMP[104].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP105 CE_CAN_9.HR_TIME_STAMP[105].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP106 CE_CAN_9.HR_TIME_STAMP[106].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP107 CE_CAN_9.HR_TIME_STAMP[107].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP108 CE_CAN_9.HR_TIME_STAMP[108].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP109 CE_CAN_9.HR_TIME_STAMP[109].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP110 CE_CAN_9.HR_TIME_STAMP[110].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP111 CE_CAN_9.HR_TIME_STAMP[111].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP112 CE_CAN_9.HR_TIME_STAMP[112].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP113 CE_CAN_9.HR_TIME_STAMP[113].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP114 CE_CAN_9.HR_TIME_STAMP[114].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP115 CE_CAN_9.HR_TIME_STAMP[115].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP116 CE_CAN_9.HR_TIME_STAMP[116].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP117 CE_CAN_9.HR_TIME_STAMP[117].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP118 CE_CAN_9.HR_TIME_STAMP[118].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP119 CE_CAN_9.HR_TIME_STAMP[119].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP120 CE_CAN_9.HR_TIME_STAMP[120].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP121 CE_CAN_9.HR_TIME_STAMP[121].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP122 CE_CAN_9.HR_TIME_STAMP[122].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP123 CE_CAN_9.HR_TIME_STAMP[123].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP124 CE_CAN_9.HR_TIME_STAMP[124].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP125 CE_CAN_9.HR_TIME_STAMP[125].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP126 CE_CAN_9.HR_TIME_STAMP[126].R /* High-Resolution Timestamp */
#define CE_CAN_9_HR_TIME_STAMP127 CE_CAN_9.HR_TIME_STAMP[127].R /* High-Resolution Timestamp */
#define CE_CAN_9_ERFFEL0     CE_CAN_9.ERFFEL[0].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL1     CE_CAN_9.ERFFEL[1].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL2     CE_CAN_9.ERFFEL[2].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL3     CE_CAN_9.ERFFEL[3].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL4     CE_CAN_9.ERFFEL[4].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL5     CE_CAN_9.ERFFEL[5].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL6     CE_CAN_9.ERFFEL[6].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL7     CE_CAN_9.ERFFEL[7].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL8     CE_CAN_9.ERFFEL[8].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL9     CE_CAN_9.ERFFEL[9].R          /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL10    CE_CAN_9.ERFFEL[10].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL11    CE_CAN_9.ERFFEL[11].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL12    CE_CAN_9.ERFFEL[12].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL13    CE_CAN_9.ERFFEL[13].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL14    CE_CAN_9.ERFFEL[14].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL15    CE_CAN_9.ERFFEL[15].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL16    CE_CAN_9.ERFFEL[16].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL17    CE_CAN_9.ERFFEL[17].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL18    CE_CAN_9.ERFFEL[18].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL19    CE_CAN_9.ERFFEL[19].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL20    CE_CAN_9.ERFFEL[20].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL21    CE_CAN_9.ERFFEL[21].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL22    CE_CAN_9.ERFFEL[22].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL23    CE_CAN_9.ERFFEL[23].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL24    CE_CAN_9.ERFFEL[24].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL25    CE_CAN_9.ERFFEL[25].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL26    CE_CAN_9.ERFFEL[26].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL27    CE_CAN_9.ERFFEL[27].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL28    CE_CAN_9.ERFFEL[28].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL29    CE_CAN_9.ERFFEL[29].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL30    CE_CAN_9.ERFFEL[30].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL31    CE_CAN_9.ERFFEL[31].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL32    CE_CAN_9.ERFFEL[32].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL33    CE_CAN_9.ERFFEL[33].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL34    CE_CAN_9.ERFFEL[34].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL35    CE_CAN_9.ERFFEL[35].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL36    CE_CAN_9.ERFFEL[36].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL37    CE_CAN_9.ERFFEL[37].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL38    CE_CAN_9.ERFFEL[38].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL39    CE_CAN_9.ERFFEL[39].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL40    CE_CAN_9.ERFFEL[40].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL41    CE_CAN_9.ERFFEL[41].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL42    CE_CAN_9.ERFFEL[42].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL43    CE_CAN_9.ERFFEL[43].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL44    CE_CAN_9.ERFFEL[44].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL45    CE_CAN_9.ERFFEL[45].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL46    CE_CAN_9.ERFFEL[46].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL47    CE_CAN_9.ERFFEL[47].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL48    CE_CAN_9.ERFFEL[48].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL49    CE_CAN_9.ERFFEL[49].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL50    CE_CAN_9.ERFFEL[50].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL51    CE_CAN_9.ERFFEL[51].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL52    CE_CAN_9.ERFFEL[52].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL53    CE_CAN_9.ERFFEL[53].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL54    CE_CAN_9.ERFFEL[54].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL55    CE_CAN_9.ERFFEL[55].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL56    CE_CAN_9.ERFFEL[56].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL57    CE_CAN_9.ERFFEL[57].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL58    CE_CAN_9.ERFFEL[58].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL59    CE_CAN_9.ERFFEL[59].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL60    CE_CAN_9.ERFFEL[60].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL61    CE_CAN_9.ERFFEL[61].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL62    CE_CAN_9.ERFFEL[62].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL63    CE_CAN_9.ERFFEL[63].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL64    CE_CAN_9.ERFFEL[64].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL65    CE_CAN_9.ERFFEL[65].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL66    CE_CAN_9.ERFFEL[66].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL67    CE_CAN_9.ERFFEL[67].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL68    CE_CAN_9.ERFFEL[68].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL69    CE_CAN_9.ERFFEL[69].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL70    CE_CAN_9.ERFFEL[70].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL71    CE_CAN_9.ERFFEL[71].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL72    CE_CAN_9.ERFFEL[72].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL73    CE_CAN_9.ERFFEL[73].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL74    CE_CAN_9.ERFFEL[74].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL75    CE_CAN_9.ERFFEL[75].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL76    CE_CAN_9.ERFFEL[76].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL77    CE_CAN_9.ERFFEL[77].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL78    CE_CAN_9.ERFFEL[78].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL79    CE_CAN_9.ERFFEL[79].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL80    CE_CAN_9.ERFFEL[80].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL81    CE_CAN_9.ERFFEL[81].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL82    CE_CAN_9.ERFFEL[82].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL83    CE_CAN_9.ERFFEL[83].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL84    CE_CAN_9.ERFFEL[84].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL85    CE_CAN_9.ERFFEL[85].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL86    CE_CAN_9.ERFFEL[86].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL87    CE_CAN_9.ERFFEL[87].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL88    CE_CAN_9.ERFFEL[88].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL89    CE_CAN_9.ERFFEL[89].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL90    CE_CAN_9.ERFFEL[90].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL91    CE_CAN_9.ERFFEL[91].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL92    CE_CAN_9.ERFFEL[92].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL93    CE_CAN_9.ERFFEL[93].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL94    CE_CAN_9.ERFFEL[94].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL95    CE_CAN_9.ERFFEL[95].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL96    CE_CAN_9.ERFFEL[96].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL97    CE_CAN_9.ERFFEL[97].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL98    CE_CAN_9.ERFFEL[98].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL99    CE_CAN_9.ERFFEL[99].R         /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL100   CE_CAN_9.ERFFEL[100].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL101   CE_CAN_9.ERFFEL[101].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL102   CE_CAN_9.ERFFEL[102].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL103   CE_CAN_9.ERFFEL[103].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL104   CE_CAN_9.ERFFEL[104].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL105   CE_CAN_9.ERFFEL[105].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL106   CE_CAN_9.ERFFEL[106].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL107   CE_CAN_9.ERFFEL[107].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL108   CE_CAN_9.ERFFEL[108].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL109   CE_CAN_9.ERFFEL[109].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL110   CE_CAN_9.ERFFEL[110].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL111   CE_CAN_9.ERFFEL[111].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL112   CE_CAN_9.ERFFEL[112].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL113   CE_CAN_9.ERFFEL[113].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL114   CE_CAN_9.ERFFEL[114].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL115   CE_CAN_9.ERFFEL[115].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL116   CE_CAN_9.ERFFEL[116].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL117   CE_CAN_9.ERFFEL[117].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL118   CE_CAN_9.ERFFEL[118].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL119   CE_CAN_9.ERFFEL[119].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL120   CE_CAN_9.ERFFEL[120].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL121   CE_CAN_9.ERFFEL[121].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL122   CE_CAN_9.ERFFEL[122].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL123   CE_CAN_9.ERFFEL[123].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL124   CE_CAN_9.ERFFEL[124].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL125   CE_CAN_9.ERFFEL[125].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL126   CE_CAN_9.ERFFEL[126].R        /* Enhanced RX FIFO Filter Element */
#define CE_CAN_9_ERFFEL127   CE_CAN_9.ERFFEL[127].R        /* Enhanced RX FIFO Filter Element */

/* FLEXRAY */
#define FR_0_MVR             FR_0.MVR.R                    /* Module Version */
#define FR_0_MCR             FR_0.MCR.R                    /* Module Configuration */
#define FR_0_SYMBADHR        FR_0.SYMBADHR.R               /* System Memory Base Address High */
#define FR_0_SYMBADLR        FR_0.SYMBADLR.R               /* System Memory Base Address Low */
#define FR_0_STBSCR          FR_0.STBSCR.R                 /* Strobe Signal Control */
#define FR_0_MBDSR           FR_0.MBDSR.R                  /* MB Data Size */
#define FR_0_MBSSUTR         FR_0.MBSSUTR.R                /* MB Segment Size and Utilization */
#define FR_0_PEDRAR          FR_0.PEDRAR.R                 /* PE DRAM Access */
#define FR_0_PEDRDR          FR_0.PEDRDR.R                 /* PE DRAM Data */
#define FR_0_POCR            FR_0.POCR.R                   /* Protocol Operation Control */
#define FR_0_GIFER           FR_0.GIFER.R                  /* Global Interrupt Flag And Enable */
#define FR_0_PIFR0           FR_0.PIFR0.R                  /* Protocol Interrupt Flag 0 */
#define FR_0_PIFR1           FR_0.PIFR1.R                  /* Protocol Interrupt Flag 1 */
#define FR_0_PIER0           FR_0.PIER0.R                  /* Protocol Interrupt Enable 0 */
#define FR_0_PIER1           FR_0.PIER1.R                  /* Protocol Interrupt Enable 1 */
#define FR_0_CHIERFR         FR_0.CHIERFR.R                /* CHI Error Flag */
#define FR_0_MBIVEC          FR_0.MBIVEC.R                 /* MB Interrupt Vector */
#define FR_0_CASERCR         FR_0.CASERCR.R                /* Channel A Status Error Counter Register */
#define FR_0_CBSERCR         FR_0.CBSERCR.R                /* Channel B Status Error Counter */
#define FR_0_PSR0            FR_0.PSR0.R                   /* Protocol Status 0 */
#define FR_0_PSR1            FR_0.PSR1.R                   /* Protocol Status 1 */
#define FR_0_PSR2            FR_0.PSR2.R                   /* Protocol Status 2 */
#define FR_0_PSR3            FR_0.PSR3.R                   /* Protocol Status 3 */
#define FR_0_MTCTR           FR_0.MTCTR.R                  /* MT Counter */
#define FR_0_CYCTR           FR_0.CYCTR.R                  /* Cycle Counter */
#define FR_0_SLTCTAR         FR_0.SLTCTAR.R                /* Slot Counter Channel A */
#define FR_0_SLTCTBR         FR_0.SLTCTBR.R                /* Slot Counter Channel B */
#define FR_0_RTCORVR         FR_0.RTCORVR.R                /* Rate Correction Value */
#define FR_0_OFCORVR         FR_0.OFCORVR.R                /* Offset Correction Value */
#define FR_0_CIFR            FR_0.CIFR.R                   /* Combined Interrupt Flag */
#define FR_0_SYMATOR         FR_0.SYMATOR.R                /* System Memory Access Timeout */
#define FR_0_SFCNTR          FR_0.SFCNTR.R                 /* Sync Frame Counter */
#define FR_0_SFTOR           FR_0.SFTOR.R                  /* Sync Frame Table Offset */
#define FR_0_SFTCCSR         FR_0.SFTCCSR.R                /* Sync Frame Table Configuration Control Status */
#define FR_0_SFIDRFR         FR_0.SFIDRFR.R                /* Sync Frame ID Rejection Filter */
#define FR_0_SFIDAFVR        FR_0.SFIDAFVR.R               /* Sync Frame ID Acceptance Filter Value */
#define FR_0_SFIDAFMR        FR_0.SFIDAFMR.R               /* Sync Frame ID Acceptance Filter Mask */
#define FR_0_NMVR0           FR_0.NMVR[0].R                /* NMV 0 */
#define FR_0_NMVR1           FR_0.NMVR[1].R                /* NMV 1 */
#define FR_0_NMVR2           FR_0.NMVR[2].R                /* NMV 2 */
#define FR_0_NMVR3           FR_0.NMVR[3].R                /* NMV 3 */
#define FR_0_NMVR4           FR_0.NMVR[4].R                /* NMV 4 */
#define FR_0_NMVR5           FR_0.NMVR[5].R                /* NMV 5 */
#define FR_0_NMVLR           FR_0.NMVLR.R                  /* Network Management Vector Length Register */
#define FR_0_TICCR           FR_0.TICCR.R                  /* Timer Configuration And Control */
#define FR_0_TI1CYSR         FR_0.TI1CYSR.R                /* Timer 1 Cycle Set */
#define FR_0_TI1MTOR         FR_0.TI1MTOR.R                /* Timer 1 MT Offset */
#define FR_0_TI2CR0_ABS      FR_0.TI2CR0.ABS.R             /* Timer 2 Configuration 0 (Absolute Timer Configuration) */
#define FR_0_TI2CR0_REL      FR_0.TI2CR0.REL.R             /* Timer 2 Configuration 0 (Relative Timer Configuration) */
#define FR_0_TI2CR1_ABS      FR_0.TI2CR1.ABS.R             /* Timer 2 Configuration 1 (Absolute Timer Configuration) */
#define FR_0_TI2CR1_REL      FR_0.TI2CR1.REL.R             /* Timer 2 Configuration 1 (Relative Timer Configuration) */
#define FR_0_SSSR            FR_0.SSSR.R                   /* Slot Status Selection */
#define FR_0_SSCCR           FR_0.SSCCR.R                  /* Slot Status Counter Condition */
#define FR_0_SSR0            FR_0.SSR[0].R                 /* Slot Status */
#define FR_0_SSR1            FR_0.SSR[1].R                 /* Slot Status */
#define FR_0_SSR2            FR_0.SSR[2].R                 /* Slot Status */
#define FR_0_SSR3            FR_0.SSR[3].R                 /* Slot Status */
#define FR_0_SSR4            FR_0.SSR[4].R                 /* Slot Status */
#define FR_0_SSR5            FR_0.SSR[5].R                 /* Slot Status */
#define FR_0_SSR6            FR_0.SSR[6].R                 /* Slot Status */
#define FR_0_SSR7            FR_0.SSR[7].R                 /* Slot Status */
#define FR_0_SSCR0           FR_0.SSCR[0].R                /* Slot Status Counter */
#define FR_0_SSCR1           FR_0.SSCR[1].R                /* Slot Status Counter */
#define FR_0_SSCR2           FR_0.SSCR[2].R                /* Slot Status Counter */
#define FR_0_SSCR3           FR_0.SSCR[3].R                /* Slot Status Counter */
#define FR_0_MTSACFR         FR_0.MTSACFR.R                /* MTS A Configuration */
#define FR_0_MTSBCFR         FR_0.MTSBCFR.R                /* MTS B Configuration */
#define FR_0_RSBIR           FR_0.RSBIR.R                  /* Receive Shadow Buffer Index */
#define FR_0_RFWMSR          FR_0.RFWMSR.R                 /* Receive FIFO Watermark And Selection */
#define FR_0_RFSIR           FR_0.RFSIR.R                  /* Receive FIFO Start Index */
#define FR_0_RFDSR           FR_0.RFDSR.R                  /* Receive FIFO Depth And Size */
#define FR_0_RFARIR          FR_0.RFARIR.R                 /* Receive FIFO A Read Index */
#define FR_0_RFBRIR          FR_0.RFBRIR.R                 /* Receive FIFO B Read Index */
#define FR_0_RFMIDAFVR       FR_0.RFMIDAFVR.R              /* Receive FIFO Message ID Acceptance Filter Value */
#define FR_0_RFMIDAFMR       FR_0.RFMIDAFMR.R              /* Receive FIFO Message ID Acceptance Filter Mask */
#define FR_0_RFFIDRFVR       FR_0.RFFIDRFVR.R              /* Receive FIFO Frame ID Rejection Filter Value */
#define FR_0_RFFIDRFMR       FR_0.RFFIDRFMR.R              /* Receive FIFO Frame ID Rejection Filter Mask */
#define FR_0_RFRFCFR         FR_0.RFRFCFR.R                /* Receive FIFO Range Filter Configuration */
#define FR_0_RFRFCTR         FR_0.RFRFCTR.R                /* Receive FIFO Range Filter Control */
#define FR_0_LDTXSLAR        FR_0.LDTXSLAR.R               /* Last Dynamic Transmit Slot Channel A */
#define FR_0_LDTXSLBR        FR_0.LDTXSLBR.R               /* Last Dynamic Transmit Slot Channel B */
#define FR_0_PCR0            FR_0.PCR0.R                   /* Protocol Configuration 0 */
#define FR_0_PCR1            FR_0.PCR1.R                   /* Protocol Configuration 1 */
#define FR_0_PCR2            FR_0.PCR2.R                   /* Protocol Configuration 2 */
#define FR_0_PCR3            FR_0.PCR3.R                   /* Protocol Configuration 3 */
#define FR_0_PCR4            FR_0.PCR4.R                   /* Protocol Configuration 4 */
#define FR_0_PCR5            FR_0.PCR5.R                   /* Protocol Configuration 5 */
#define FR_0_PCR6            FR_0.PCR6.R                   /* Protocol Configuration 6 */
#define FR_0_PCR7            FR_0.PCR7.R                   /* Protocol Configuration 7 */
#define FR_0_PCR8            FR_0.PCR8.R                   /* Protocol Configuration 8 */
#define FR_0_PCR9            FR_0.PCR9.R                   /* Protocol Configuration 9 */
#define FR_0_PCR10           FR_0.PCR10.R                  /* Protocol Configuration 10 */
#define FR_0_PCR11           FR_0.PCR11.R                  /* Protocol Configuration 11 */
#define FR_0_PCR12           FR_0.PCR12.R                  /* Protocol Configuration 12 */
#define FR_0_PCR13           FR_0.PCR13.R                  /* Protocol Configuration 13 */
#define FR_0_PCR14           FR_0.PCR14.R                  /* Protocol Configuration 14 */
#define FR_0_PCR15           FR_0.PCR15.R                  /* Protocol Configuration 15 */
#define FR_0_PCR16           FR_0.PCR16.R                  /* Protocol Configuration 16 */
#define FR_0_PCR17           FR_0.PCR17.R                  /* Protocol Configuration 17 */
#define FR_0_PCR18           FR_0.PCR18.R                  /* Protocol Configuration 18 */
#define FR_0_PCR19           FR_0.PCR19.R                  /* Protocol Configuration 19 */
#define FR_0_PCR20           FR_0.PCR20.R                  /* Protocol Configuration 20 */
#define FR_0_PCR21           FR_0.PCR21.R                  /* Protocol Configuration 21 */
#define FR_0_PCR22           FR_0.PCR22.R                  /* Protocol Configuration 22 */
#define FR_0_PCR23           FR_0.PCR23.R                  /* Protocol Configuration 23 */
#define FR_0_PCR24           FR_0.PCR24.R                  /* Protocol Configuration 24 */
#define FR_0_PCR25           FR_0.PCR25.R                  /* Protocol Configuration 25 */
#define FR_0_PCR26           FR_0.PCR26.R                  /* Protocol Configuration 26 */
#define FR_0_PCR27           FR_0.PCR27.R                  /* Protocol Configuration 27 */
#define FR_0_PCR28           FR_0.PCR28.R                  /* Protocol Configuration 28 */
#define FR_0_PCR29           FR_0.PCR29.R                  /* Protocol Configuration 29 */
#define FR_0_PCR30           FR_0.PCR30.R                  /* Protocol Configuration 30 */
#define FR_0_STPWHR          FR_0.STPWHR.R                 /* Stopwatch Count High */
#define FR_0_STPWLR          FR_0.STPWLR.R                 /* Stopwatch Count Low */
#define FR_0_PEOER           FR_0.PEOER.R                  /* Protocol Event Output Enable And Stopwatch Control */
#define FR_0_RFSDOR          FR_0.RFSDOR.R                 /* Receive FIFO Start Data Offset */
#define FR_0_RFSYMBADHR      FR_0.RFSYMBADHR.R             /* Receive FIFO System Memory Base Address High */
#define FR_0_RFSYMBADLR      FR_0.RFSYMBADLR.R             /* Receive FIFO System Memory Base Address Low */
#define FR_0_RFPTR           FR_0.RFPTR.R                  /* Receive FIFO Periodic Timer */
#define FR_0_RFFLPCR         FR_0.RFFLPCR.R                /* Receive FIFO Fill Level and Pop Count */
#define FR_0_EEIFER          FR_0.EEIFER.R                 /* ECC Error Interrupt Flag And Enable */
#define FR_0_EERICR          FR_0.EERICR.R                 /* ECC Error Report And Injection Control */
#define FR_0_EERAR           FR_0.EERAR.R                  /* ECC Error Report Address */
#define FR_0_EERDR           FR_0.EERDR.R                  /* ECC Error Report Data */
#define FR_0_EERCR           FR_0.EERCR.R                  /* ECC Error Report Code */
#define FR_0_EEIAR           FR_0.EEIAR.R                  /* ECC Error Injection Address */
#define FR_0_EEIDR           FR_0.EEIDR.R                  /* ECC Error Injection Data */
#define FR_0_EEICR           FR_0.EEICR.R                  /* ECC Error Injection Code */
#define FR_0_MBCCSR0         FR_0.MB[0].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR0         FR_0.MB[0].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR0         FR_0.MB[0].FIDR.R             /* MB FID */
#define FR_0_MBIDXR0         FR_0.MB[0].IDXR.R             /* MB Index */
#define FR_0_MBCCSR1         FR_0.MB[1].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR1         FR_0.MB[1].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR1         FR_0.MB[1].FIDR.R             /* MB FID */
#define FR_0_MBIDXR1         FR_0.MB[1].IDXR.R             /* MB Index */
#define FR_0_MBCCSR2         FR_0.MB[2].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR2         FR_0.MB[2].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR2         FR_0.MB[2].FIDR.R             /* MB FID */
#define FR_0_MBIDXR2         FR_0.MB[2].IDXR.R             /* MB Index */
#define FR_0_MBCCSR3         FR_0.MB[3].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR3         FR_0.MB[3].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR3         FR_0.MB[3].FIDR.R             /* MB FID */
#define FR_0_MBIDXR3         FR_0.MB[3].IDXR.R             /* MB Index */
#define FR_0_MBCCSR4         FR_0.MB[4].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR4         FR_0.MB[4].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR4         FR_0.MB[4].FIDR.R             /* MB FID */
#define FR_0_MBIDXR4         FR_0.MB[4].IDXR.R             /* MB Index */
#define FR_0_MBCCSR5         FR_0.MB[5].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR5         FR_0.MB[5].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR5         FR_0.MB[5].FIDR.R             /* MB FID */
#define FR_0_MBIDXR5         FR_0.MB[5].IDXR.R             /* MB Index */
#define FR_0_MBCCSR6         FR_0.MB[6].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR6         FR_0.MB[6].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR6         FR_0.MB[6].FIDR.R             /* MB FID */
#define FR_0_MBIDXR6         FR_0.MB[6].IDXR.R             /* MB Index */
#define FR_0_MBCCSR7         FR_0.MB[7].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR7         FR_0.MB[7].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR7         FR_0.MB[7].FIDR.R             /* MB FID */
#define FR_0_MBIDXR7         FR_0.MB[7].IDXR.R             /* MB Index */
#define FR_0_MBCCSR8         FR_0.MB[8].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR8         FR_0.MB[8].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR8         FR_0.MB[8].FIDR.R             /* MB FID */
#define FR_0_MBIDXR8         FR_0.MB[8].IDXR.R             /* MB Index */
#define FR_0_MBCCSR9         FR_0.MB[9].CCSR.R             /* MB Configuration Control Status */
#define FR_0_MBCCFR9         FR_0.MB[9].CCFR.R             /* MB Cycle Counter Filter */
#define FR_0_MBFIDR9         FR_0.MB[9].FIDR.R             /* MB FID */
#define FR_0_MBIDXR9         FR_0.MB[9].IDXR.R             /* MB Index */
#define FR_0_MBCCSR10        FR_0.MB[10].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR10        FR_0.MB[10].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR10        FR_0.MB[10].FIDR.R            /* MB FID */
#define FR_0_MBIDXR10        FR_0.MB[10].IDXR.R            /* MB Index */
#define FR_0_MBCCSR11        FR_0.MB[11].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR11        FR_0.MB[11].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR11        FR_0.MB[11].FIDR.R            /* MB FID */
#define FR_0_MBIDXR11        FR_0.MB[11].IDXR.R            /* MB Index */
#define FR_0_MBCCSR12        FR_0.MB[12].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR12        FR_0.MB[12].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR12        FR_0.MB[12].FIDR.R            /* MB FID */
#define FR_0_MBIDXR12        FR_0.MB[12].IDXR.R            /* MB Index */
#define FR_0_MBCCSR13        FR_0.MB[13].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR13        FR_0.MB[13].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR13        FR_0.MB[13].FIDR.R            /* MB FID */
#define FR_0_MBIDXR13        FR_0.MB[13].IDXR.R            /* MB Index */
#define FR_0_MBCCSR14        FR_0.MB[14].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR14        FR_0.MB[14].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR14        FR_0.MB[14].FIDR.R            /* MB FID */
#define FR_0_MBIDXR14        FR_0.MB[14].IDXR.R            /* MB Index */
#define FR_0_MBCCSR15        FR_0.MB[15].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR15        FR_0.MB[15].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR15        FR_0.MB[15].FIDR.R            /* MB FID */
#define FR_0_MBIDXR15        FR_0.MB[15].IDXR.R            /* MB Index */
#define FR_0_MBCCSR16        FR_0.MB[16].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR16        FR_0.MB[16].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR16        FR_0.MB[16].FIDR.R            /* MB FID */
#define FR_0_MBIDXR16        FR_0.MB[16].IDXR.R            /* MB Index */
#define FR_0_MBCCSR17        FR_0.MB[17].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR17        FR_0.MB[17].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR17        FR_0.MB[17].FIDR.R            /* MB FID */
#define FR_0_MBIDXR17        FR_0.MB[17].IDXR.R            /* MB Index */
#define FR_0_MBCCSR18        FR_0.MB[18].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR18        FR_0.MB[18].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR18        FR_0.MB[18].FIDR.R            /* MB FID */
#define FR_0_MBIDXR18        FR_0.MB[18].IDXR.R            /* MB Index */
#define FR_0_MBCCSR19        FR_0.MB[19].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR19        FR_0.MB[19].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR19        FR_0.MB[19].FIDR.R            /* MB FID */
#define FR_0_MBIDXR19        FR_0.MB[19].IDXR.R            /* MB Index */
#define FR_0_MBCCSR20        FR_0.MB[20].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR20        FR_0.MB[20].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR20        FR_0.MB[20].FIDR.R            /* MB FID */
#define FR_0_MBIDXR20        FR_0.MB[20].IDXR.R            /* MB Index */
#define FR_0_MBCCSR21        FR_0.MB[21].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR21        FR_0.MB[21].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR21        FR_0.MB[21].FIDR.R            /* MB FID */
#define FR_0_MBIDXR21        FR_0.MB[21].IDXR.R            /* MB Index */
#define FR_0_MBCCSR22        FR_0.MB[22].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR22        FR_0.MB[22].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR22        FR_0.MB[22].FIDR.R            /* MB FID */
#define FR_0_MBIDXR22        FR_0.MB[22].IDXR.R            /* MB Index */
#define FR_0_MBCCSR23        FR_0.MB[23].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR23        FR_0.MB[23].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR23        FR_0.MB[23].FIDR.R            /* MB FID */
#define FR_0_MBIDXR23        FR_0.MB[23].IDXR.R            /* MB Index */
#define FR_0_MBCCSR24        FR_0.MB[24].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR24        FR_0.MB[24].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR24        FR_0.MB[24].FIDR.R            /* MB FID */
#define FR_0_MBIDXR24        FR_0.MB[24].IDXR.R            /* MB Index */
#define FR_0_MBCCSR25        FR_0.MB[25].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR25        FR_0.MB[25].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR25        FR_0.MB[25].FIDR.R            /* MB FID */
#define FR_0_MBIDXR25        FR_0.MB[25].IDXR.R            /* MB Index */
#define FR_0_MBCCSR26        FR_0.MB[26].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR26        FR_0.MB[26].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR26        FR_0.MB[26].FIDR.R            /* MB FID */
#define FR_0_MBIDXR26        FR_0.MB[26].IDXR.R            /* MB Index */
#define FR_0_MBCCSR27        FR_0.MB[27].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR27        FR_0.MB[27].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR27        FR_0.MB[27].FIDR.R            /* MB FID */
#define FR_0_MBIDXR27        FR_0.MB[27].IDXR.R            /* MB Index */
#define FR_0_MBCCSR28        FR_0.MB[28].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR28        FR_0.MB[28].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR28        FR_0.MB[28].FIDR.R            /* MB FID */
#define FR_0_MBIDXR28        FR_0.MB[28].IDXR.R            /* MB Index */
#define FR_0_MBCCSR29        FR_0.MB[29].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR29        FR_0.MB[29].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR29        FR_0.MB[29].FIDR.R            /* MB FID */
#define FR_0_MBIDXR29        FR_0.MB[29].IDXR.R            /* MB Index */
#define FR_0_MBCCSR30        FR_0.MB[30].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR30        FR_0.MB[30].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR30        FR_0.MB[30].FIDR.R            /* MB FID */
#define FR_0_MBIDXR30        FR_0.MB[30].IDXR.R            /* MB Index */
#define FR_0_MBCCSR31        FR_0.MB[31].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR31        FR_0.MB[31].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR31        FR_0.MB[31].FIDR.R            /* MB FID */
#define FR_0_MBIDXR31        FR_0.MB[31].IDXR.R            /* MB Index */
#define FR_0_MBCCSR32        FR_0.MB[32].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR32        FR_0.MB[32].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR32        FR_0.MB[32].FIDR.R            /* MB FID */
#define FR_0_MBIDXR32        FR_0.MB[32].IDXR.R            /* MB Index */
#define FR_0_MBCCSR33        FR_0.MB[33].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR33        FR_0.MB[33].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR33        FR_0.MB[33].FIDR.R            /* MB FID */
#define FR_0_MBIDXR33        FR_0.MB[33].IDXR.R            /* MB Index */
#define FR_0_MBCCSR34        FR_0.MB[34].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR34        FR_0.MB[34].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR34        FR_0.MB[34].FIDR.R            /* MB FID */
#define FR_0_MBIDXR34        FR_0.MB[34].IDXR.R            /* MB Index */
#define FR_0_MBCCSR35        FR_0.MB[35].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR35        FR_0.MB[35].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR35        FR_0.MB[35].FIDR.R            /* MB FID */
#define FR_0_MBIDXR35        FR_0.MB[35].IDXR.R            /* MB Index */
#define FR_0_MBCCSR36        FR_0.MB[36].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR36        FR_0.MB[36].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR36        FR_0.MB[36].FIDR.R            /* MB FID */
#define FR_0_MBIDXR36        FR_0.MB[36].IDXR.R            /* MB Index */
#define FR_0_MBCCSR37        FR_0.MB[37].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR37        FR_0.MB[37].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR37        FR_0.MB[37].FIDR.R            /* MB FID */
#define FR_0_MBIDXR37        FR_0.MB[37].IDXR.R            /* MB Index */
#define FR_0_MBCCSR38        FR_0.MB[38].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR38        FR_0.MB[38].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR38        FR_0.MB[38].FIDR.R            /* MB FID */
#define FR_0_MBIDXR38        FR_0.MB[38].IDXR.R            /* MB Index */
#define FR_0_MBCCSR39        FR_0.MB[39].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR39        FR_0.MB[39].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR39        FR_0.MB[39].FIDR.R            /* MB FID */
#define FR_0_MBIDXR39        FR_0.MB[39].IDXR.R            /* MB Index */
#define FR_0_MBCCSR40        FR_0.MB[40].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR40        FR_0.MB[40].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR40        FR_0.MB[40].FIDR.R            /* MB FID */
#define FR_0_MBIDXR40        FR_0.MB[40].IDXR.R            /* MB Index */
#define FR_0_MBCCSR41        FR_0.MB[41].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR41        FR_0.MB[41].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR41        FR_0.MB[41].FIDR.R            /* MB FID */
#define FR_0_MBIDXR41        FR_0.MB[41].IDXR.R            /* MB Index */
#define FR_0_MBCCSR42        FR_0.MB[42].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR42        FR_0.MB[42].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR42        FR_0.MB[42].FIDR.R            /* MB FID */
#define FR_0_MBIDXR42        FR_0.MB[42].IDXR.R            /* MB Index */
#define FR_0_MBCCSR43        FR_0.MB[43].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR43        FR_0.MB[43].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR43        FR_0.MB[43].FIDR.R            /* MB FID */
#define FR_0_MBIDXR43        FR_0.MB[43].IDXR.R            /* MB Index */
#define FR_0_MBCCSR44        FR_0.MB[44].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR44        FR_0.MB[44].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR44        FR_0.MB[44].FIDR.R            /* MB FID */
#define FR_0_MBIDXR44        FR_0.MB[44].IDXR.R            /* MB Index */
#define FR_0_MBCCSR45        FR_0.MB[45].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR45        FR_0.MB[45].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR45        FR_0.MB[45].FIDR.R            /* MB FID */
#define FR_0_MBIDXR45        FR_0.MB[45].IDXR.R            /* MB Index */
#define FR_0_MBCCSR46        FR_0.MB[46].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR46        FR_0.MB[46].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR46        FR_0.MB[46].FIDR.R            /* MB FID */
#define FR_0_MBIDXR46        FR_0.MB[46].IDXR.R            /* MB Index */
#define FR_0_MBCCSR47        FR_0.MB[47].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR47        FR_0.MB[47].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR47        FR_0.MB[47].FIDR.R            /* MB FID */
#define FR_0_MBIDXR47        FR_0.MB[47].IDXR.R            /* MB Index */
#define FR_0_MBCCSR48        FR_0.MB[48].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR48        FR_0.MB[48].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR48        FR_0.MB[48].FIDR.R            /* MB FID */
#define FR_0_MBIDXR48        FR_0.MB[48].IDXR.R            /* MB Index */
#define FR_0_MBCCSR49        FR_0.MB[49].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR49        FR_0.MB[49].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR49        FR_0.MB[49].FIDR.R            /* MB FID */
#define FR_0_MBIDXR49        FR_0.MB[49].IDXR.R            /* MB Index */
#define FR_0_MBCCSR50        FR_0.MB[50].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR50        FR_0.MB[50].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR50        FR_0.MB[50].FIDR.R            /* MB FID */
#define FR_0_MBIDXR50        FR_0.MB[50].IDXR.R            /* MB Index */
#define FR_0_MBCCSR51        FR_0.MB[51].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR51        FR_0.MB[51].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR51        FR_0.MB[51].FIDR.R            /* MB FID */
#define FR_0_MBIDXR51        FR_0.MB[51].IDXR.R            /* MB Index */
#define FR_0_MBCCSR52        FR_0.MB[52].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR52        FR_0.MB[52].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR52        FR_0.MB[52].FIDR.R            /* MB FID */
#define FR_0_MBIDXR52        FR_0.MB[52].IDXR.R            /* MB Index */
#define FR_0_MBCCSR53        FR_0.MB[53].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR53        FR_0.MB[53].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR53        FR_0.MB[53].FIDR.R            /* MB FID */
#define FR_0_MBIDXR53        FR_0.MB[53].IDXR.R            /* MB Index */
#define FR_0_MBCCSR54        FR_0.MB[54].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR54        FR_0.MB[54].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR54        FR_0.MB[54].FIDR.R            /* MB FID */
#define FR_0_MBIDXR54        FR_0.MB[54].IDXR.R            /* MB Index */
#define FR_0_MBCCSR55        FR_0.MB[55].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR55        FR_0.MB[55].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR55        FR_0.MB[55].FIDR.R            /* MB FID */
#define FR_0_MBIDXR55        FR_0.MB[55].IDXR.R            /* MB Index */
#define FR_0_MBCCSR56        FR_0.MB[56].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR56        FR_0.MB[56].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR56        FR_0.MB[56].FIDR.R            /* MB FID */
#define FR_0_MBIDXR56        FR_0.MB[56].IDXR.R            /* MB Index */
#define FR_0_MBCCSR57        FR_0.MB[57].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR57        FR_0.MB[57].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR57        FR_0.MB[57].FIDR.R            /* MB FID */
#define FR_0_MBIDXR57        FR_0.MB[57].IDXR.R            /* MB Index */
#define FR_0_MBCCSR58        FR_0.MB[58].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR58        FR_0.MB[58].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR58        FR_0.MB[58].FIDR.R            /* MB FID */
#define FR_0_MBIDXR58        FR_0.MB[58].IDXR.R            /* MB Index */
#define FR_0_MBCCSR59        FR_0.MB[59].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR59        FR_0.MB[59].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR59        FR_0.MB[59].FIDR.R            /* MB FID */
#define FR_0_MBIDXR59        FR_0.MB[59].IDXR.R            /* MB Index */
#define FR_0_MBCCSR60        FR_0.MB[60].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR60        FR_0.MB[60].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR60        FR_0.MB[60].FIDR.R            /* MB FID */
#define FR_0_MBIDXR60        FR_0.MB[60].IDXR.R            /* MB Index */
#define FR_0_MBCCSR61        FR_0.MB[61].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR61        FR_0.MB[61].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR61        FR_0.MB[61].FIDR.R            /* MB FID */
#define FR_0_MBIDXR61        FR_0.MB[61].IDXR.R            /* MB Index */
#define FR_0_MBCCSR62        FR_0.MB[62].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR62        FR_0.MB[62].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR62        FR_0.MB[62].FIDR.R            /* MB FID */
#define FR_0_MBIDXR62        FR_0.MB[62].IDXR.R            /* MB Index */
#define FR_0_MBCCSR63        FR_0.MB[63].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR63        FR_0.MB[63].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR63        FR_0.MB[63].FIDR.R            /* MB FID */
#define FR_0_MBIDXR63        FR_0.MB[63].IDXR.R            /* MB Index */
#define FR_0_MBCCSR64        FR_0.MB[64].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR64        FR_0.MB[64].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR64        FR_0.MB[64].FIDR.R            /* MB FID */
#define FR_0_MBIDXR64        FR_0.MB[64].IDXR.R            /* MB Index */
#define FR_0_MBCCSR65        FR_0.MB[65].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR65        FR_0.MB[65].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR65        FR_0.MB[65].FIDR.R            /* MB FID */
#define FR_0_MBIDXR65        FR_0.MB[65].IDXR.R            /* MB Index */
#define FR_0_MBCCSR66        FR_0.MB[66].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR66        FR_0.MB[66].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR66        FR_0.MB[66].FIDR.R            /* MB FID */
#define FR_0_MBIDXR66        FR_0.MB[66].IDXR.R            /* MB Index */
#define FR_0_MBCCSR67        FR_0.MB[67].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR67        FR_0.MB[67].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR67        FR_0.MB[67].FIDR.R            /* MB FID */
#define FR_0_MBIDXR67        FR_0.MB[67].IDXR.R            /* MB Index */
#define FR_0_MBCCSR68        FR_0.MB[68].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR68        FR_0.MB[68].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR68        FR_0.MB[68].FIDR.R            /* MB FID */
#define FR_0_MBIDXR68        FR_0.MB[68].IDXR.R            /* MB Index */
#define FR_0_MBCCSR69        FR_0.MB[69].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR69        FR_0.MB[69].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR69        FR_0.MB[69].FIDR.R            /* MB FID */
#define FR_0_MBIDXR69        FR_0.MB[69].IDXR.R            /* MB Index */
#define FR_0_MBCCSR70        FR_0.MB[70].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR70        FR_0.MB[70].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR70        FR_0.MB[70].FIDR.R            /* MB FID */
#define FR_0_MBIDXR70        FR_0.MB[70].IDXR.R            /* MB Index */
#define FR_0_MBCCSR71        FR_0.MB[71].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR71        FR_0.MB[71].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR71        FR_0.MB[71].FIDR.R            /* MB FID */
#define FR_0_MBIDXR71        FR_0.MB[71].IDXR.R            /* MB Index */
#define FR_0_MBCCSR72        FR_0.MB[72].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR72        FR_0.MB[72].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR72        FR_0.MB[72].FIDR.R            /* MB FID */
#define FR_0_MBIDXR72        FR_0.MB[72].IDXR.R            /* MB Index */
#define FR_0_MBCCSR73        FR_0.MB[73].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR73        FR_0.MB[73].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR73        FR_0.MB[73].FIDR.R            /* MB FID */
#define FR_0_MBIDXR73        FR_0.MB[73].IDXR.R            /* MB Index */
#define FR_0_MBCCSR74        FR_0.MB[74].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR74        FR_0.MB[74].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR74        FR_0.MB[74].FIDR.R            /* MB FID */
#define FR_0_MBIDXR74        FR_0.MB[74].IDXR.R            /* MB Index */
#define FR_0_MBCCSR75        FR_0.MB[75].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR75        FR_0.MB[75].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR75        FR_0.MB[75].FIDR.R            /* MB FID */
#define FR_0_MBIDXR75        FR_0.MB[75].IDXR.R            /* MB Index */
#define FR_0_MBCCSR76        FR_0.MB[76].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR76        FR_0.MB[76].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR76        FR_0.MB[76].FIDR.R            /* MB FID */
#define FR_0_MBIDXR76        FR_0.MB[76].IDXR.R            /* MB Index */
#define FR_0_MBCCSR77        FR_0.MB[77].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR77        FR_0.MB[77].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR77        FR_0.MB[77].FIDR.R            /* MB FID */
#define FR_0_MBIDXR77        FR_0.MB[77].IDXR.R            /* MB Index */
#define FR_0_MBCCSR78        FR_0.MB[78].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR78        FR_0.MB[78].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR78        FR_0.MB[78].FIDR.R            /* MB FID */
#define FR_0_MBIDXR78        FR_0.MB[78].IDXR.R            /* MB Index */
#define FR_0_MBCCSR79        FR_0.MB[79].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR79        FR_0.MB[79].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR79        FR_0.MB[79].FIDR.R            /* MB FID */
#define FR_0_MBIDXR79        FR_0.MB[79].IDXR.R            /* MB Index */
#define FR_0_MBCCSR80        FR_0.MB[80].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR80        FR_0.MB[80].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR80        FR_0.MB[80].FIDR.R            /* MB FID */
#define FR_0_MBIDXR80        FR_0.MB[80].IDXR.R            /* MB Index */
#define FR_0_MBCCSR81        FR_0.MB[81].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR81        FR_0.MB[81].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR81        FR_0.MB[81].FIDR.R            /* MB FID */
#define FR_0_MBIDXR81        FR_0.MB[81].IDXR.R            /* MB Index */
#define FR_0_MBCCSR82        FR_0.MB[82].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR82        FR_0.MB[82].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR82        FR_0.MB[82].FIDR.R            /* MB FID */
#define FR_0_MBIDXR82        FR_0.MB[82].IDXR.R            /* MB Index */
#define FR_0_MBCCSR83        FR_0.MB[83].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR83        FR_0.MB[83].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR83        FR_0.MB[83].FIDR.R            /* MB FID */
#define FR_0_MBIDXR83        FR_0.MB[83].IDXR.R            /* MB Index */
#define FR_0_MBCCSR84        FR_0.MB[84].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR84        FR_0.MB[84].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR84        FR_0.MB[84].FIDR.R            /* MB FID */
#define FR_0_MBIDXR84        FR_0.MB[84].IDXR.R            /* MB Index */
#define FR_0_MBCCSR85        FR_0.MB[85].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR85        FR_0.MB[85].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR85        FR_0.MB[85].FIDR.R            /* MB FID */
#define FR_0_MBIDXR85        FR_0.MB[85].IDXR.R            /* MB Index */
#define FR_0_MBCCSR86        FR_0.MB[86].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR86        FR_0.MB[86].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR86        FR_0.MB[86].FIDR.R            /* MB FID */
#define FR_0_MBIDXR86        FR_0.MB[86].IDXR.R            /* MB Index */
#define FR_0_MBCCSR87        FR_0.MB[87].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR87        FR_0.MB[87].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR87        FR_0.MB[87].FIDR.R            /* MB FID */
#define FR_0_MBIDXR87        FR_0.MB[87].IDXR.R            /* MB Index */
#define FR_0_MBCCSR88        FR_0.MB[88].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR88        FR_0.MB[88].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR88        FR_0.MB[88].FIDR.R            /* MB FID */
#define FR_0_MBIDXR88        FR_0.MB[88].IDXR.R            /* MB Index */
#define FR_0_MBCCSR89        FR_0.MB[89].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR89        FR_0.MB[89].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR89        FR_0.MB[89].FIDR.R            /* MB FID */
#define FR_0_MBIDXR89        FR_0.MB[89].IDXR.R            /* MB Index */
#define FR_0_MBCCSR90        FR_0.MB[90].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR90        FR_0.MB[90].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR90        FR_0.MB[90].FIDR.R            /* MB FID */
#define FR_0_MBIDXR90        FR_0.MB[90].IDXR.R            /* MB Index */
#define FR_0_MBCCSR91        FR_0.MB[91].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR91        FR_0.MB[91].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR91        FR_0.MB[91].FIDR.R            /* MB FID */
#define FR_0_MBIDXR91        FR_0.MB[91].IDXR.R            /* MB Index */
#define FR_0_MBCCSR92        FR_0.MB[92].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR92        FR_0.MB[92].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR92        FR_0.MB[92].FIDR.R            /* MB FID */
#define FR_0_MBIDXR92        FR_0.MB[92].IDXR.R            /* MB Index */
#define FR_0_MBCCSR93        FR_0.MB[93].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR93        FR_0.MB[93].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR93        FR_0.MB[93].FIDR.R            /* MB FID */
#define FR_0_MBIDXR93        FR_0.MB[93].IDXR.R            /* MB Index */
#define FR_0_MBCCSR94        FR_0.MB[94].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR94        FR_0.MB[94].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR94        FR_0.MB[94].FIDR.R            /* MB FID */
#define FR_0_MBIDXR94        FR_0.MB[94].IDXR.R            /* MB Index */
#define FR_0_MBCCSR95        FR_0.MB[95].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR95        FR_0.MB[95].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR95        FR_0.MB[95].FIDR.R            /* MB FID */
#define FR_0_MBIDXR95        FR_0.MB[95].IDXR.R            /* MB Index */
#define FR_0_MBCCSR96        FR_0.MB[96].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR96        FR_0.MB[96].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR96        FR_0.MB[96].FIDR.R            /* MB FID */
#define FR_0_MBIDXR96        FR_0.MB[96].IDXR.R            /* MB Index */
#define FR_0_MBCCSR97        FR_0.MB[97].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR97        FR_0.MB[97].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR97        FR_0.MB[97].FIDR.R            /* MB FID */
#define FR_0_MBIDXR97        FR_0.MB[97].IDXR.R            /* MB Index */
#define FR_0_MBCCSR98        FR_0.MB[98].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR98        FR_0.MB[98].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR98        FR_0.MB[98].FIDR.R            /* MB FID */
#define FR_0_MBIDXR98        FR_0.MB[98].IDXR.R            /* MB Index */
#define FR_0_MBCCSR99        FR_0.MB[99].CCSR.R            /* MB Configuration Control Status */
#define FR_0_MBCCFR99        FR_0.MB[99].CCFR.R            /* MB Cycle Counter Filter */
#define FR_0_MBFIDR99        FR_0.MB[99].FIDR.R            /* MB FID */
#define FR_0_MBIDXR99        FR_0.MB[99].IDXR.R            /* MB Index */
#define FR_0_MBCCSR100       FR_0.MB[100].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR100       FR_0.MB[100].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR100       FR_0.MB[100].FIDR.R           /* MB FID */
#define FR_0_MBIDXR100       FR_0.MB[100].IDXR.R           /* MB Index */
#define FR_0_MBCCSR101       FR_0.MB[101].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR101       FR_0.MB[101].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR101       FR_0.MB[101].FIDR.R           /* MB FID */
#define FR_0_MBIDXR101       FR_0.MB[101].IDXR.R           /* MB Index */
#define FR_0_MBCCSR102       FR_0.MB[102].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR102       FR_0.MB[102].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR102       FR_0.MB[102].FIDR.R           /* MB FID */
#define FR_0_MBIDXR102       FR_0.MB[102].IDXR.R           /* MB Index */
#define FR_0_MBCCSR103       FR_0.MB[103].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR103       FR_0.MB[103].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR103       FR_0.MB[103].FIDR.R           /* MB FID */
#define FR_0_MBIDXR103       FR_0.MB[103].IDXR.R           /* MB Index */
#define FR_0_MBCCSR104       FR_0.MB[104].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR104       FR_0.MB[104].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR104       FR_0.MB[104].FIDR.R           /* MB FID */
#define FR_0_MBIDXR104       FR_0.MB[104].IDXR.R           /* MB Index */
#define FR_0_MBCCSR105       FR_0.MB[105].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR105       FR_0.MB[105].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR105       FR_0.MB[105].FIDR.R           /* MB FID */
#define FR_0_MBIDXR105       FR_0.MB[105].IDXR.R           /* MB Index */
#define FR_0_MBCCSR106       FR_0.MB[106].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR106       FR_0.MB[106].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR106       FR_0.MB[106].FIDR.R           /* MB FID */
#define FR_0_MBIDXR106       FR_0.MB[106].IDXR.R           /* MB Index */
#define FR_0_MBCCSR107       FR_0.MB[107].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR107       FR_0.MB[107].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR107       FR_0.MB[107].FIDR.R           /* MB FID */
#define FR_0_MBIDXR107       FR_0.MB[107].IDXR.R           /* MB Index */
#define FR_0_MBCCSR108       FR_0.MB[108].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR108       FR_0.MB[108].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR108       FR_0.MB[108].FIDR.R           /* MB FID */
#define FR_0_MBIDXR108       FR_0.MB[108].IDXR.R           /* MB Index */
#define FR_0_MBCCSR109       FR_0.MB[109].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR109       FR_0.MB[109].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR109       FR_0.MB[109].FIDR.R           /* MB FID */
#define FR_0_MBIDXR109       FR_0.MB[109].IDXR.R           /* MB Index */
#define FR_0_MBCCSR110       FR_0.MB[110].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR110       FR_0.MB[110].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR110       FR_0.MB[110].FIDR.R           /* MB FID */
#define FR_0_MBIDXR110       FR_0.MB[110].IDXR.R           /* MB Index */
#define FR_0_MBCCSR111       FR_0.MB[111].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR111       FR_0.MB[111].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR111       FR_0.MB[111].FIDR.R           /* MB FID */
#define FR_0_MBIDXR111       FR_0.MB[111].IDXR.R           /* MB Index */
#define FR_0_MBCCSR112       FR_0.MB[112].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR112       FR_0.MB[112].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR112       FR_0.MB[112].FIDR.R           /* MB FID */
#define FR_0_MBIDXR112       FR_0.MB[112].IDXR.R           /* MB Index */
#define FR_0_MBCCSR113       FR_0.MB[113].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR113       FR_0.MB[113].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR113       FR_0.MB[113].FIDR.R           /* MB FID */
#define FR_0_MBIDXR113       FR_0.MB[113].IDXR.R           /* MB Index */
#define FR_0_MBCCSR114       FR_0.MB[114].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR114       FR_0.MB[114].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR114       FR_0.MB[114].FIDR.R           /* MB FID */
#define FR_0_MBIDXR114       FR_0.MB[114].IDXR.R           /* MB Index */
#define FR_0_MBCCSR115       FR_0.MB[115].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR115       FR_0.MB[115].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR115       FR_0.MB[115].FIDR.R           /* MB FID */
#define FR_0_MBIDXR115       FR_0.MB[115].IDXR.R           /* MB Index */
#define FR_0_MBCCSR116       FR_0.MB[116].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR116       FR_0.MB[116].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR116       FR_0.MB[116].FIDR.R           /* MB FID */
#define FR_0_MBIDXR116       FR_0.MB[116].IDXR.R           /* MB Index */
#define FR_0_MBCCSR117       FR_0.MB[117].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR117       FR_0.MB[117].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR117       FR_0.MB[117].FIDR.R           /* MB FID */
#define FR_0_MBIDXR117       FR_0.MB[117].IDXR.R           /* MB Index */
#define FR_0_MBCCSR118       FR_0.MB[118].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR118       FR_0.MB[118].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR118       FR_0.MB[118].FIDR.R           /* MB FID */
#define FR_0_MBIDXR118       FR_0.MB[118].IDXR.R           /* MB Index */
#define FR_0_MBCCSR119       FR_0.MB[119].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR119       FR_0.MB[119].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR119       FR_0.MB[119].FIDR.R           /* MB FID */
#define FR_0_MBIDXR119       FR_0.MB[119].IDXR.R           /* MB Index */
#define FR_0_MBCCSR120       FR_0.MB[120].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR120       FR_0.MB[120].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR120       FR_0.MB[120].FIDR.R           /* MB FID */
#define FR_0_MBIDXR120       FR_0.MB[120].IDXR.R           /* MB Index */
#define FR_0_MBCCSR121       FR_0.MB[121].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR121       FR_0.MB[121].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR121       FR_0.MB[121].FIDR.R           /* MB FID */
#define FR_0_MBIDXR121       FR_0.MB[121].IDXR.R           /* MB Index */
#define FR_0_MBCCSR122       FR_0.MB[122].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR122       FR_0.MB[122].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR122       FR_0.MB[122].FIDR.R           /* MB FID */
#define FR_0_MBIDXR122       FR_0.MB[122].IDXR.R           /* MB Index */
#define FR_0_MBCCSR123       FR_0.MB[123].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR123       FR_0.MB[123].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR123       FR_0.MB[123].FIDR.R           /* MB FID */
#define FR_0_MBIDXR123       FR_0.MB[123].IDXR.R           /* MB Index */
#define FR_0_MBCCSR124       FR_0.MB[124].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR124       FR_0.MB[124].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR124       FR_0.MB[124].FIDR.R           /* MB FID */
#define FR_0_MBIDXR124       FR_0.MB[124].IDXR.R           /* MB Index */
#define FR_0_MBCCSR125       FR_0.MB[125].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR125       FR_0.MB[125].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR125       FR_0.MB[125].FIDR.R           /* MB FID */
#define FR_0_MBIDXR125       FR_0.MB[125].IDXR.R           /* MB Index */
#define FR_0_MBCCSR126       FR_0.MB[126].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR126       FR_0.MB[126].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR126       FR_0.MB[126].FIDR.R           /* MB FID */
#define FR_0_MBIDXR126       FR_0.MB[126].IDXR.R           /* MB Index */
#define FR_0_MBCCSR127       FR_0.MB[127].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR127       FR_0.MB[127].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR127       FR_0.MB[127].FIDR.R           /* MB FID */
#define FR_0_MBIDXR127       FR_0.MB[127].IDXR.R           /* MB Index */
#define FR_0_MBCCSR128       FR_0.MB[128].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR128       FR_0.MB[128].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR128       FR_0.MB[128].FIDR.R           /* MB FID */
#define FR_0_MBIDXR128       FR_0.MB[128].IDXR.R           /* MB Index */
#define FR_0_MBCCSR129       FR_0.MB[129].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR129       FR_0.MB[129].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR129       FR_0.MB[129].FIDR.R           /* MB FID */
#define FR_0_MBIDXR129       FR_0.MB[129].IDXR.R           /* MB Index */
#define FR_0_MBCCSR130       FR_0.MB[130].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR130       FR_0.MB[130].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR130       FR_0.MB[130].FIDR.R           /* MB FID */
#define FR_0_MBIDXR130       FR_0.MB[130].IDXR.R           /* MB Index */
#define FR_0_MBCCSR131       FR_0.MB[131].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR131       FR_0.MB[131].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR131       FR_0.MB[131].FIDR.R           /* MB FID */
#define FR_0_MBIDXR131       FR_0.MB[131].IDXR.R           /* MB Index */
#define FR_0_MBCCSR132       FR_0.MB[132].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR132       FR_0.MB[132].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR132       FR_0.MB[132].FIDR.R           /* MB FID */
#define FR_0_MBIDXR132       FR_0.MB[132].IDXR.R           /* MB Index */
#define FR_0_MBCCSR133       FR_0.MB[133].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR133       FR_0.MB[133].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR133       FR_0.MB[133].FIDR.R           /* MB FID */
#define FR_0_MBIDXR133       FR_0.MB[133].IDXR.R           /* MB Index */
#define FR_0_MBCCSR134       FR_0.MB[134].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR134       FR_0.MB[134].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR134       FR_0.MB[134].FIDR.R           /* MB FID */
#define FR_0_MBIDXR134       FR_0.MB[134].IDXR.R           /* MB Index */
#define FR_0_MBCCSR135       FR_0.MB[135].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR135       FR_0.MB[135].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR135       FR_0.MB[135].FIDR.R           /* MB FID */
#define FR_0_MBIDXR135       FR_0.MB[135].IDXR.R           /* MB Index */
#define FR_0_MBCCSR136       FR_0.MB[136].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR136       FR_0.MB[136].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR136       FR_0.MB[136].FIDR.R           /* MB FID */
#define FR_0_MBIDXR136       FR_0.MB[136].IDXR.R           /* MB Index */
#define FR_0_MBCCSR137       FR_0.MB[137].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR137       FR_0.MB[137].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR137       FR_0.MB[137].FIDR.R           /* MB FID */
#define FR_0_MBIDXR137       FR_0.MB[137].IDXR.R           /* MB Index */
#define FR_0_MBCCSR138       FR_0.MB[138].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR138       FR_0.MB[138].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR138       FR_0.MB[138].FIDR.R           /* MB FID */
#define FR_0_MBIDXR138       FR_0.MB[138].IDXR.R           /* MB Index */
#define FR_0_MBCCSR139       FR_0.MB[139].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR139       FR_0.MB[139].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR139       FR_0.MB[139].FIDR.R           /* MB FID */
#define FR_0_MBIDXR139       FR_0.MB[139].IDXR.R           /* MB Index */
#define FR_0_MBCCSR140       FR_0.MB[140].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR140       FR_0.MB[140].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR140       FR_0.MB[140].FIDR.R           /* MB FID */
#define FR_0_MBIDXR140       FR_0.MB[140].IDXR.R           /* MB Index */
#define FR_0_MBCCSR141       FR_0.MB[141].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR141       FR_0.MB[141].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR141       FR_0.MB[141].FIDR.R           /* MB FID */
#define FR_0_MBIDXR141       FR_0.MB[141].IDXR.R           /* MB Index */
#define FR_0_MBCCSR142       FR_0.MB[142].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR142       FR_0.MB[142].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR142       FR_0.MB[142].FIDR.R           /* MB FID */
#define FR_0_MBIDXR142       FR_0.MB[142].IDXR.R           /* MB Index */
#define FR_0_MBCCSR143       FR_0.MB[143].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR143       FR_0.MB[143].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR143       FR_0.MB[143].FIDR.R           /* MB FID */
#define FR_0_MBIDXR143       FR_0.MB[143].IDXR.R           /* MB Index */
#define FR_0_MBCCSR144       FR_0.MB[144].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR144       FR_0.MB[144].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR144       FR_0.MB[144].FIDR.R           /* MB FID */
#define FR_0_MBIDXR144       FR_0.MB[144].IDXR.R           /* MB Index */
#define FR_0_MBCCSR145       FR_0.MB[145].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR145       FR_0.MB[145].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR145       FR_0.MB[145].FIDR.R           /* MB FID */
#define FR_0_MBIDXR145       FR_0.MB[145].IDXR.R           /* MB Index */
#define FR_0_MBCCSR146       FR_0.MB[146].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR146       FR_0.MB[146].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR146       FR_0.MB[146].FIDR.R           /* MB FID */
#define FR_0_MBIDXR146       FR_0.MB[146].IDXR.R           /* MB Index */
#define FR_0_MBCCSR147       FR_0.MB[147].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR147       FR_0.MB[147].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR147       FR_0.MB[147].FIDR.R           /* MB FID */
#define FR_0_MBIDXR147       FR_0.MB[147].IDXR.R           /* MB Index */
#define FR_0_MBCCSR148       FR_0.MB[148].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR148       FR_0.MB[148].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR148       FR_0.MB[148].FIDR.R           /* MB FID */
#define FR_0_MBIDXR148       FR_0.MB[148].IDXR.R           /* MB Index */
#define FR_0_MBCCSR149       FR_0.MB[149].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR149       FR_0.MB[149].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR149       FR_0.MB[149].FIDR.R           /* MB FID */
#define FR_0_MBIDXR149       FR_0.MB[149].IDXR.R           /* MB Index */
#define FR_0_MBCCSR150       FR_0.MB[150].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR150       FR_0.MB[150].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR150       FR_0.MB[150].FIDR.R           /* MB FID */
#define FR_0_MBIDXR150       FR_0.MB[150].IDXR.R           /* MB Index */
#define FR_0_MBCCSR151       FR_0.MB[151].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR151       FR_0.MB[151].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR151       FR_0.MB[151].FIDR.R           /* MB FID */
#define FR_0_MBIDXR151       FR_0.MB[151].IDXR.R           /* MB Index */
#define FR_0_MBCCSR152       FR_0.MB[152].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR152       FR_0.MB[152].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR152       FR_0.MB[152].FIDR.R           /* MB FID */
#define FR_0_MBIDXR152       FR_0.MB[152].IDXR.R           /* MB Index */
#define FR_0_MBCCSR153       FR_0.MB[153].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR153       FR_0.MB[153].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR153       FR_0.MB[153].FIDR.R           /* MB FID */
#define FR_0_MBIDXR153       FR_0.MB[153].IDXR.R           /* MB Index */
#define FR_0_MBCCSR154       FR_0.MB[154].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR154       FR_0.MB[154].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR154       FR_0.MB[154].FIDR.R           /* MB FID */
#define FR_0_MBIDXR154       FR_0.MB[154].IDXR.R           /* MB Index */
#define FR_0_MBCCSR155       FR_0.MB[155].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR155       FR_0.MB[155].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR155       FR_0.MB[155].FIDR.R           /* MB FID */
#define FR_0_MBIDXR155       FR_0.MB[155].IDXR.R           /* MB Index */
#define FR_0_MBCCSR156       FR_0.MB[156].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR156       FR_0.MB[156].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR156       FR_0.MB[156].FIDR.R           /* MB FID */
#define FR_0_MBIDXR156       FR_0.MB[156].IDXR.R           /* MB Index */
#define FR_0_MBCCSR157       FR_0.MB[157].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR157       FR_0.MB[157].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR157       FR_0.MB[157].FIDR.R           /* MB FID */
#define FR_0_MBIDXR157       FR_0.MB[157].IDXR.R           /* MB Index */
#define FR_0_MBCCSR158       FR_0.MB[158].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR158       FR_0.MB[158].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR158       FR_0.MB[158].FIDR.R           /* MB FID */
#define FR_0_MBIDXR158       FR_0.MB[158].IDXR.R           /* MB Index */
#define FR_0_MBCCSR159       FR_0.MB[159].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR159       FR_0.MB[159].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR159       FR_0.MB[159].FIDR.R           /* MB FID */
#define FR_0_MBIDXR159       FR_0.MB[159].IDXR.R           /* MB Index */
#define FR_0_MBCCSR160       FR_0.MB[160].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR160       FR_0.MB[160].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR160       FR_0.MB[160].FIDR.R           /* MB FID */
#define FR_0_MBIDXR160       FR_0.MB[160].IDXR.R           /* MB Index */
#define FR_0_MBCCSR161       FR_0.MB[161].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR161       FR_0.MB[161].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR161       FR_0.MB[161].FIDR.R           /* MB FID */
#define FR_0_MBIDXR161       FR_0.MB[161].IDXR.R           /* MB Index */
#define FR_0_MBCCSR162       FR_0.MB[162].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR162       FR_0.MB[162].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR162       FR_0.MB[162].FIDR.R           /* MB FID */
#define FR_0_MBIDXR162       FR_0.MB[162].IDXR.R           /* MB Index */
#define FR_0_MBCCSR163       FR_0.MB[163].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR163       FR_0.MB[163].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR163       FR_0.MB[163].FIDR.R           /* MB FID */
#define FR_0_MBIDXR163       FR_0.MB[163].IDXR.R           /* MB Index */
#define FR_0_MBCCSR164       FR_0.MB[164].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR164       FR_0.MB[164].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR164       FR_0.MB[164].FIDR.R           /* MB FID */
#define FR_0_MBIDXR164       FR_0.MB[164].IDXR.R           /* MB Index */
#define FR_0_MBCCSR165       FR_0.MB[165].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR165       FR_0.MB[165].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR165       FR_0.MB[165].FIDR.R           /* MB FID */
#define FR_0_MBIDXR165       FR_0.MB[165].IDXR.R           /* MB Index */
#define FR_0_MBCCSR166       FR_0.MB[166].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR166       FR_0.MB[166].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR166       FR_0.MB[166].FIDR.R           /* MB FID */
#define FR_0_MBIDXR166       FR_0.MB[166].IDXR.R           /* MB Index */
#define FR_0_MBCCSR167       FR_0.MB[167].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR167       FR_0.MB[167].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR167       FR_0.MB[167].FIDR.R           /* MB FID */
#define FR_0_MBIDXR167       FR_0.MB[167].IDXR.R           /* MB Index */
#define FR_0_MBCCSR168       FR_0.MB[168].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR168       FR_0.MB[168].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR168       FR_0.MB[168].FIDR.R           /* MB FID */
#define FR_0_MBIDXR168       FR_0.MB[168].IDXR.R           /* MB Index */
#define FR_0_MBCCSR169       FR_0.MB[169].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR169       FR_0.MB[169].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR169       FR_0.MB[169].FIDR.R           /* MB FID */
#define FR_0_MBIDXR169       FR_0.MB[169].IDXR.R           /* MB Index */
#define FR_0_MBCCSR170       FR_0.MB[170].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR170       FR_0.MB[170].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR170       FR_0.MB[170].FIDR.R           /* MB FID */
#define FR_0_MBIDXR170       FR_0.MB[170].IDXR.R           /* MB Index */
#define FR_0_MBCCSR171       FR_0.MB[171].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR171       FR_0.MB[171].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR171       FR_0.MB[171].FIDR.R           /* MB FID */
#define FR_0_MBIDXR171       FR_0.MB[171].IDXR.R           /* MB Index */
#define FR_0_MBCCSR172       FR_0.MB[172].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR172       FR_0.MB[172].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR172       FR_0.MB[172].FIDR.R           /* MB FID */
#define FR_0_MBIDXR172       FR_0.MB[172].IDXR.R           /* MB Index */
#define FR_0_MBCCSR173       FR_0.MB[173].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR173       FR_0.MB[173].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR173       FR_0.MB[173].FIDR.R           /* MB FID */
#define FR_0_MBIDXR173       FR_0.MB[173].IDXR.R           /* MB Index */
#define FR_0_MBCCSR174       FR_0.MB[174].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR174       FR_0.MB[174].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR174       FR_0.MB[174].FIDR.R           /* MB FID */
#define FR_0_MBIDXR174       FR_0.MB[174].IDXR.R           /* MB Index */
#define FR_0_MBCCSR175       FR_0.MB[175].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR175       FR_0.MB[175].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR175       FR_0.MB[175].FIDR.R           /* MB FID */
#define FR_0_MBIDXR175       FR_0.MB[175].IDXR.R           /* MB Index */
#define FR_0_MBCCSR176       FR_0.MB[176].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR176       FR_0.MB[176].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR176       FR_0.MB[176].FIDR.R           /* MB FID */
#define FR_0_MBIDXR176       FR_0.MB[176].IDXR.R           /* MB Index */
#define FR_0_MBCCSR177       FR_0.MB[177].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR177       FR_0.MB[177].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR177       FR_0.MB[177].FIDR.R           /* MB FID */
#define FR_0_MBIDXR177       FR_0.MB[177].IDXR.R           /* MB Index */
#define FR_0_MBCCSR178       FR_0.MB[178].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR178       FR_0.MB[178].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR178       FR_0.MB[178].FIDR.R           /* MB FID */
#define FR_0_MBIDXR178       FR_0.MB[178].IDXR.R           /* MB Index */
#define FR_0_MBCCSR179       FR_0.MB[179].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR179       FR_0.MB[179].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR179       FR_0.MB[179].FIDR.R           /* MB FID */
#define FR_0_MBIDXR179       FR_0.MB[179].IDXR.R           /* MB Index */
#define FR_0_MBCCSR180       FR_0.MB[180].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR180       FR_0.MB[180].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR180       FR_0.MB[180].FIDR.R           /* MB FID */
#define FR_0_MBIDXR180       FR_0.MB[180].IDXR.R           /* MB Index */
#define FR_0_MBCCSR181       FR_0.MB[181].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR181       FR_0.MB[181].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR181       FR_0.MB[181].FIDR.R           /* MB FID */
#define FR_0_MBIDXR181       FR_0.MB[181].IDXR.R           /* MB Index */
#define FR_0_MBCCSR182       FR_0.MB[182].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR182       FR_0.MB[182].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR182       FR_0.MB[182].FIDR.R           /* MB FID */
#define FR_0_MBIDXR182       FR_0.MB[182].IDXR.R           /* MB Index */
#define FR_0_MBCCSR183       FR_0.MB[183].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR183       FR_0.MB[183].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR183       FR_0.MB[183].FIDR.R           /* MB FID */
#define FR_0_MBIDXR183       FR_0.MB[183].IDXR.R           /* MB Index */
#define FR_0_MBCCSR184       FR_0.MB[184].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR184       FR_0.MB[184].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR184       FR_0.MB[184].FIDR.R           /* MB FID */
#define FR_0_MBIDXR184       FR_0.MB[184].IDXR.R           /* MB Index */
#define FR_0_MBCCSR185       FR_0.MB[185].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR185       FR_0.MB[185].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR185       FR_0.MB[185].FIDR.R           /* MB FID */
#define FR_0_MBIDXR185       FR_0.MB[185].IDXR.R           /* MB Index */
#define FR_0_MBCCSR186       FR_0.MB[186].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR186       FR_0.MB[186].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR186       FR_0.MB[186].FIDR.R           /* MB FID */
#define FR_0_MBIDXR186       FR_0.MB[186].IDXR.R           /* MB Index */
#define FR_0_MBCCSR187       FR_0.MB[187].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR187       FR_0.MB[187].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR187       FR_0.MB[187].FIDR.R           /* MB FID */
#define FR_0_MBIDXR187       FR_0.MB[187].IDXR.R           /* MB Index */
#define FR_0_MBCCSR188       FR_0.MB[188].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR188       FR_0.MB[188].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR188       FR_0.MB[188].FIDR.R           /* MB FID */
#define FR_0_MBIDXR188       FR_0.MB[188].IDXR.R           /* MB Index */
#define FR_0_MBCCSR189       FR_0.MB[189].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR189       FR_0.MB[189].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR189       FR_0.MB[189].FIDR.R           /* MB FID */
#define FR_0_MBIDXR189       FR_0.MB[189].IDXR.R           /* MB Index */
#define FR_0_MBCCSR190       FR_0.MB[190].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR190       FR_0.MB[190].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR190       FR_0.MB[190].FIDR.R           /* MB FID */
#define FR_0_MBIDXR190       FR_0.MB[190].IDXR.R           /* MB Index */
#define FR_0_MBCCSR191       FR_0.MB[191].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR191       FR_0.MB[191].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR191       FR_0.MB[191].FIDR.R           /* MB FID */
#define FR_0_MBIDXR191       FR_0.MB[191].IDXR.R           /* MB Index */
#define FR_0_MBCCSR192       FR_0.MB[192].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR192       FR_0.MB[192].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR192       FR_0.MB[192].FIDR.R           /* MB FID */
#define FR_0_MBIDXR192       FR_0.MB[192].IDXR.R           /* MB Index */
#define FR_0_MBCCSR193       FR_0.MB[193].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR193       FR_0.MB[193].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR193       FR_0.MB[193].FIDR.R           /* MB FID */
#define FR_0_MBIDXR193       FR_0.MB[193].IDXR.R           /* MB Index */
#define FR_0_MBCCSR194       FR_0.MB[194].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR194       FR_0.MB[194].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR194       FR_0.MB[194].FIDR.R           /* MB FID */
#define FR_0_MBIDXR194       FR_0.MB[194].IDXR.R           /* MB Index */
#define FR_0_MBCCSR195       FR_0.MB[195].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR195       FR_0.MB[195].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR195       FR_0.MB[195].FIDR.R           /* MB FID */
#define FR_0_MBIDXR195       FR_0.MB[195].IDXR.R           /* MB Index */
#define FR_0_MBCCSR196       FR_0.MB[196].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR196       FR_0.MB[196].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR196       FR_0.MB[196].FIDR.R           /* MB FID */
#define FR_0_MBIDXR196       FR_0.MB[196].IDXR.R           /* MB Index */
#define FR_0_MBCCSR197       FR_0.MB[197].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR197       FR_0.MB[197].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR197       FR_0.MB[197].FIDR.R           /* MB FID */
#define FR_0_MBIDXR197       FR_0.MB[197].IDXR.R           /* MB Index */
#define FR_0_MBCCSR198       FR_0.MB[198].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR198       FR_0.MB[198].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR198       FR_0.MB[198].FIDR.R           /* MB FID */
#define FR_0_MBIDXR198       FR_0.MB[198].IDXR.R           /* MB Index */
#define FR_0_MBCCSR199       FR_0.MB[199].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR199       FR_0.MB[199].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR199       FR_0.MB[199].FIDR.R           /* MB FID */
#define FR_0_MBIDXR199       FR_0.MB[199].IDXR.R           /* MB Index */
#define FR_0_MBCCSR200       FR_0.MB[200].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR200       FR_0.MB[200].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR200       FR_0.MB[200].FIDR.R           /* MB FID */
#define FR_0_MBIDXR200       FR_0.MB[200].IDXR.R           /* MB Index */
#define FR_0_MBCCSR201       FR_0.MB[201].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR201       FR_0.MB[201].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR201       FR_0.MB[201].FIDR.R           /* MB FID */
#define FR_0_MBIDXR201       FR_0.MB[201].IDXR.R           /* MB Index */
#define FR_0_MBCCSR202       FR_0.MB[202].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR202       FR_0.MB[202].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR202       FR_0.MB[202].FIDR.R           /* MB FID */
#define FR_0_MBIDXR202       FR_0.MB[202].IDXR.R           /* MB Index */
#define FR_0_MBCCSR203       FR_0.MB[203].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR203       FR_0.MB[203].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR203       FR_0.MB[203].FIDR.R           /* MB FID */
#define FR_0_MBIDXR203       FR_0.MB[203].IDXR.R           /* MB Index */
#define FR_0_MBCCSR204       FR_0.MB[204].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR204       FR_0.MB[204].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR204       FR_0.MB[204].FIDR.R           /* MB FID */
#define FR_0_MBIDXR204       FR_0.MB[204].IDXR.R           /* MB Index */
#define FR_0_MBCCSR205       FR_0.MB[205].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR205       FR_0.MB[205].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR205       FR_0.MB[205].FIDR.R           /* MB FID */
#define FR_0_MBIDXR205       FR_0.MB[205].IDXR.R           /* MB Index */
#define FR_0_MBCCSR206       FR_0.MB[206].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR206       FR_0.MB[206].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR206       FR_0.MB[206].FIDR.R           /* MB FID */
#define FR_0_MBIDXR206       FR_0.MB[206].IDXR.R           /* MB Index */
#define FR_0_MBCCSR207       FR_0.MB[207].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR207       FR_0.MB[207].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR207       FR_0.MB[207].FIDR.R           /* MB FID */
#define FR_0_MBIDXR207       FR_0.MB[207].IDXR.R           /* MB Index */
#define FR_0_MBCCSR208       FR_0.MB[208].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR208       FR_0.MB[208].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR208       FR_0.MB[208].FIDR.R           /* MB FID */
#define FR_0_MBIDXR208       FR_0.MB[208].IDXR.R           /* MB Index */
#define FR_0_MBCCSR209       FR_0.MB[209].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR209       FR_0.MB[209].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR209       FR_0.MB[209].FIDR.R           /* MB FID */
#define FR_0_MBIDXR209       FR_0.MB[209].IDXR.R           /* MB Index */
#define FR_0_MBCCSR210       FR_0.MB[210].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR210       FR_0.MB[210].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR210       FR_0.MB[210].FIDR.R           /* MB FID */
#define FR_0_MBIDXR210       FR_0.MB[210].IDXR.R           /* MB Index */
#define FR_0_MBCCSR211       FR_0.MB[211].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR211       FR_0.MB[211].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR211       FR_0.MB[211].FIDR.R           /* MB FID */
#define FR_0_MBIDXR211       FR_0.MB[211].IDXR.R           /* MB Index */
#define FR_0_MBCCSR212       FR_0.MB[212].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR212       FR_0.MB[212].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR212       FR_0.MB[212].FIDR.R           /* MB FID */
#define FR_0_MBIDXR212       FR_0.MB[212].IDXR.R           /* MB Index */
#define FR_0_MBCCSR213       FR_0.MB[213].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR213       FR_0.MB[213].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR213       FR_0.MB[213].FIDR.R           /* MB FID */
#define FR_0_MBIDXR213       FR_0.MB[213].IDXR.R           /* MB Index */
#define FR_0_MBCCSR214       FR_0.MB[214].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR214       FR_0.MB[214].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR214       FR_0.MB[214].FIDR.R           /* MB FID */
#define FR_0_MBIDXR214       FR_0.MB[214].IDXR.R           /* MB Index */
#define FR_0_MBCCSR215       FR_0.MB[215].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR215       FR_0.MB[215].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR215       FR_0.MB[215].FIDR.R           /* MB FID */
#define FR_0_MBIDXR215       FR_0.MB[215].IDXR.R           /* MB Index */
#define FR_0_MBCCSR216       FR_0.MB[216].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR216       FR_0.MB[216].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR216       FR_0.MB[216].FIDR.R           /* MB FID */
#define FR_0_MBIDXR216       FR_0.MB[216].IDXR.R           /* MB Index */
#define FR_0_MBCCSR217       FR_0.MB[217].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR217       FR_0.MB[217].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR217       FR_0.MB[217].FIDR.R           /* MB FID */
#define FR_0_MBIDXR217       FR_0.MB[217].IDXR.R           /* MB Index */
#define FR_0_MBCCSR218       FR_0.MB[218].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR218       FR_0.MB[218].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR218       FR_0.MB[218].FIDR.R           /* MB FID */
#define FR_0_MBIDXR218       FR_0.MB[218].IDXR.R           /* MB Index */
#define FR_0_MBCCSR219       FR_0.MB[219].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR219       FR_0.MB[219].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR219       FR_0.MB[219].FIDR.R           /* MB FID */
#define FR_0_MBIDXR219       FR_0.MB[219].IDXR.R           /* MB Index */
#define FR_0_MBCCSR220       FR_0.MB[220].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR220       FR_0.MB[220].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR220       FR_0.MB[220].FIDR.R           /* MB FID */
#define FR_0_MBIDXR220       FR_0.MB[220].IDXR.R           /* MB Index */
#define FR_0_MBCCSR221       FR_0.MB[221].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR221       FR_0.MB[221].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR221       FR_0.MB[221].FIDR.R           /* MB FID */
#define FR_0_MBIDXR221       FR_0.MB[221].IDXR.R           /* MB Index */
#define FR_0_MBCCSR222       FR_0.MB[222].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR222       FR_0.MB[222].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR222       FR_0.MB[222].FIDR.R           /* MB FID */
#define FR_0_MBIDXR222       FR_0.MB[222].IDXR.R           /* MB Index */
#define FR_0_MBCCSR223       FR_0.MB[223].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR223       FR_0.MB[223].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR223       FR_0.MB[223].FIDR.R           /* MB FID */
#define FR_0_MBIDXR223       FR_0.MB[223].IDXR.R           /* MB Index */
#define FR_0_MBCCSR224       FR_0.MB[224].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR224       FR_0.MB[224].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR224       FR_0.MB[224].FIDR.R           /* MB FID */
#define FR_0_MBIDXR224       FR_0.MB[224].IDXR.R           /* MB Index */
#define FR_0_MBCCSR225       FR_0.MB[225].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR225       FR_0.MB[225].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR225       FR_0.MB[225].FIDR.R           /* MB FID */
#define FR_0_MBIDXR225       FR_0.MB[225].IDXR.R           /* MB Index */
#define FR_0_MBCCSR226       FR_0.MB[226].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR226       FR_0.MB[226].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR226       FR_0.MB[226].FIDR.R           /* MB FID */
#define FR_0_MBIDXR226       FR_0.MB[226].IDXR.R           /* MB Index */
#define FR_0_MBCCSR227       FR_0.MB[227].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR227       FR_0.MB[227].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR227       FR_0.MB[227].FIDR.R           /* MB FID */
#define FR_0_MBIDXR227       FR_0.MB[227].IDXR.R           /* MB Index */
#define FR_0_MBCCSR228       FR_0.MB[228].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR228       FR_0.MB[228].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR228       FR_0.MB[228].FIDR.R           /* MB FID */
#define FR_0_MBIDXR228       FR_0.MB[228].IDXR.R           /* MB Index */
#define FR_0_MBCCSR229       FR_0.MB[229].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR229       FR_0.MB[229].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR229       FR_0.MB[229].FIDR.R           /* MB FID */
#define FR_0_MBIDXR229       FR_0.MB[229].IDXR.R           /* MB Index */
#define FR_0_MBCCSR230       FR_0.MB[230].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR230       FR_0.MB[230].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR230       FR_0.MB[230].FIDR.R           /* MB FID */
#define FR_0_MBIDXR230       FR_0.MB[230].IDXR.R           /* MB Index */
#define FR_0_MBCCSR231       FR_0.MB[231].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR231       FR_0.MB[231].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR231       FR_0.MB[231].FIDR.R           /* MB FID */
#define FR_0_MBIDXR231       FR_0.MB[231].IDXR.R           /* MB Index */
#define FR_0_MBCCSR232       FR_0.MB[232].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR232       FR_0.MB[232].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR232       FR_0.MB[232].FIDR.R           /* MB FID */
#define FR_0_MBIDXR232       FR_0.MB[232].IDXR.R           /* MB Index */
#define FR_0_MBCCSR233       FR_0.MB[233].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR233       FR_0.MB[233].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR233       FR_0.MB[233].FIDR.R           /* MB FID */
#define FR_0_MBIDXR233       FR_0.MB[233].IDXR.R           /* MB Index */
#define FR_0_MBCCSR234       FR_0.MB[234].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR234       FR_0.MB[234].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR234       FR_0.MB[234].FIDR.R           /* MB FID */
#define FR_0_MBIDXR234       FR_0.MB[234].IDXR.R           /* MB Index */
#define FR_0_MBCCSR235       FR_0.MB[235].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR235       FR_0.MB[235].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR235       FR_0.MB[235].FIDR.R           /* MB FID */
#define FR_0_MBIDXR235       FR_0.MB[235].IDXR.R           /* MB Index */
#define FR_0_MBCCSR236       FR_0.MB[236].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR236       FR_0.MB[236].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR236       FR_0.MB[236].FIDR.R           /* MB FID */
#define FR_0_MBIDXR236       FR_0.MB[236].IDXR.R           /* MB Index */
#define FR_0_MBCCSR237       FR_0.MB[237].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR237       FR_0.MB[237].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR237       FR_0.MB[237].FIDR.R           /* MB FID */
#define FR_0_MBIDXR237       FR_0.MB[237].IDXR.R           /* MB Index */
#define FR_0_MBCCSR238       FR_0.MB[238].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR238       FR_0.MB[238].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR238       FR_0.MB[238].FIDR.R           /* MB FID */
#define FR_0_MBIDXR238       FR_0.MB[238].IDXR.R           /* MB Index */
#define FR_0_MBCCSR239       FR_0.MB[239].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR239       FR_0.MB[239].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR239       FR_0.MB[239].FIDR.R           /* MB FID */
#define FR_0_MBIDXR239       FR_0.MB[239].IDXR.R           /* MB Index */
#define FR_0_MBCCSR240       FR_0.MB[240].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR240       FR_0.MB[240].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR240       FR_0.MB[240].FIDR.R           /* MB FID */
#define FR_0_MBIDXR240       FR_0.MB[240].IDXR.R           /* MB Index */
#define FR_0_MBCCSR241       FR_0.MB[241].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR241       FR_0.MB[241].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR241       FR_0.MB[241].FIDR.R           /* MB FID */
#define FR_0_MBIDXR241       FR_0.MB[241].IDXR.R           /* MB Index */
#define FR_0_MBCCSR242       FR_0.MB[242].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR242       FR_0.MB[242].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR242       FR_0.MB[242].FIDR.R           /* MB FID */
#define FR_0_MBIDXR242       FR_0.MB[242].IDXR.R           /* MB Index */
#define FR_0_MBCCSR243       FR_0.MB[243].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR243       FR_0.MB[243].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR243       FR_0.MB[243].FIDR.R           /* MB FID */
#define FR_0_MBIDXR243       FR_0.MB[243].IDXR.R           /* MB Index */
#define FR_0_MBCCSR244       FR_0.MB[244].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR244       FR_0.MB[244].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR244       FR_0.MB[244].FIDR.R           /* MB FID */
#define FR_0_MBIDXR244       FR_0.MB[244].IDXR.R           /* MB Index */
#define FR_0_MBCCSR245       FR_0.MB[245].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR245       FR_0.MB[245].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR245       FR_0.MB[245].FIDR.R           /* MB FID */
#define FR_0_MBIDXR245       FR_0.MB[245].IDXR.R           /* MB Index */
#define FR_0_MBCCSR246       FR_0.MB[246].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR246       FR_0.MB[246].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR246       FR_0.MB[246].FIDR.R           /* MB FID */
#define FR_0_MBIDXR246       FR_0.MB[246].IDXR.R           /* MB Index */
#define FR_0_MBCCSR247       FR_0.MB[247].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR247       FR_0.MB[247].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR247       FR_0.MB[247].FIDR.R           /* MB FID */
#define FR_0_MBIDXR247       FR_0.MB[247].IDXR.R           /* MB Index */
#define FR_0_MBCCSR248       FR_0.MB[248].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR248       FR_0.MB[248].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR248       FR_0.MB[248].FIDR.R           /* MB FID */
#define FR_0_MBIDXR248       FR_0.MB[248].IDXR.R           /* MB Index */
#define FR_0_MBCCSR249       FR_0.MB[249].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR249       FR_0.MB[249].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR249       FR_0.MB[249].FIDR.R           /* MB FID */
#define FR_0_MBIDXR249       FR_0.MB[249].IDXR.R           /* MB Index */
#define FR_0_MBCCSR250       FR_0.MB[250].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR250       FR_0.MB[250].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR250       FR_0.MB[250].FIDR.R           /* MB FID */
#define FR_0_MBIDXR250       FR_0.MB[250].IDXR.R           /* MB Index */
#define FR_0_MBCCSR251       FR_0.MB[251].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR251       FR_0.MB[251].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR251       FR_0.MB[251].FIDR.R           /* MB FID */
#define FR_0_MBIDXR251       FR_0.MB[251].IDXR.R           /* MB Index */
#define FR_0_MBCCSR252       FR_0.MB[252].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR252       FR_0.MB[252].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR252       FR_0.MB[252].FIDR.R           /* MB FID */
#define FR_0_MBIDXR252       FR_0.MB[252].IDXR.R           /* MB Index */
#define FR_0_MBCCSR253       FR_0.MB[253].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR253       FR_0.MB[253].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR253       FR_0.MB[253].FIDR.R           /* MB FID */
#define FR_0_MBIDXR253       FR_0.MB[253].IDXR.R           /* MB Index */
#define FR_0_MBCCSR254       FR_0.MB[254].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR254       FR_0.MB[254].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR254       FR_0.MB[254].FIDR.R           /* MB FID */
#define FR_0_MBIDXR254       FR_0.MB[254].IDXR.R           /* MB Index */
#define FR_0_MBCCSR255       FR_0.MB[255].CCSR.R           /* MB Configuration Control Status */
#define FR_0_MBCCFR255       FR_0.MB[255].CCFR.R           /* MB Cycle Counter Filter */
#define FR_0_MBFIDR255       FR_0.MB[255].FIDR.R           /* MB FID */
#define FR_0_MBIDXR255       FR_0.MB[255].IDXR.R           /* MB Index */
#define FR_0_MBDOR0          FR_0.MBDOR[0].R               /* MB Data Field Offset */
#define FR_0_MBDOR1          FR_0.MBDOR[1].R               /* MB Data Field Offset */
#define FR_0_MBDOR2          FR_0.MBDOR[2].R               /* MB Data Field Offset */
#define FR_0_MBDOR3          FR_0.MBDOR[3].R               /* MB Data Field Offset */
#define FR_0_MBDOR4          FR_0.MBDOR[4].R               /* MB Data Field Offset */
#define FR_0_MBDOR5          FR_0.MBDOR[5].R               /* MB Data Field Offset */
#define FR_0_MBDOR6          FR_0.MBDOR[6].R               /* MB Data Field Offset */
#define FR_0_MBDOR7          FR_0.MBDOR[7].R               /* MB Data Field Offset */
#define FR_0_MBDOR8          FR_0.MBDOR[8].R               /* MB Data Field Offset */
#define FR_0_MBDOR9          FR_0.MBDOR[9].R               /* MB Data Field Offset */
#define FR_0_MBDOR10         FR_0.MBDOR[10].R              /* MB Data Field Offset */
#define FR_0_MBDOR11         FR_0.MBDOR[11].R              /* MB Data Field Offset */
#define FR_0_MBDOR12         FR_0.MBDOR[12].R              /* MB Data Field Offset */
#define FR_0_MBDOR13         FR_0.MBDOR[13].R              /* MB Data Field Offset */
#define FR_0_MBDOR14         FR_0.MBDOR[14].R              /* MB Data Field Offset */
#define FR_0_MBDOR15         FR_0.MBDOR[15].R              /* MB Data Field Offset */
#define FR_0_MBDOR16         FR_0.MBDOR[16].R              /* MB Data Field Offset */
#define FR_0_MBDOR17         FR_0.MBDOR[17].R              /* MB Data Field Offset */
#define FR_0_MBDOR18         FR_0.MBDOR[18].R              /* MB Data Field Offset */
#define FR_0_MBDOR19         FR_0.MBDOR[19].R              /* MB Data Field Offset */
#define FR_0_MBDOR20         FR_0.MBDOR[20].R              /* MB Data Field Offset */
#define FR_0_MBDOR21         FR_0.MBDOR[21].R              /* MB Data Field Offset */
#define FR_0_MBDOR22         FR_0.MBDOR[22].R              /* MB Data Field Offset */
#define FR_0_MBDOR23         FR_0.MBDOR[23].R              /* MB Data Field Offset */
#define FR_0_MBDOR24         FR_0.MBDOR[24].R              /* MB Data Field Offset */
#define FR_0_MBDOR25         FR_0.MBDOR[25].R              /* MB Data Field Offset */
#define FR_0_MBDOR26         FR_0.MBDOR[26].R              /* MB Data Field Offset */
#define FR_0_MBDOR27         FR_0.MBDOR[27].R              /* MB Data Field Offset */
#define FR_0_MBDOR28         FR_0.MBDOR[28].R              /* MB Data Field Offset */
#define FR_0_MBDOR29         FR_0.MBDOR[29].R              /* MB Data Field Offset */
#define FR_0_MBDOR30         FR_0.MBDOR[30].R              /* MB Data Field Offset */
#define FR_0_MBDOR31         FR_0.MBDOR[31].R              /* MB Data Field Offset */
#define FR_0_MBDOR32         FR_0.MBDOR[32].R              /* MB Data Field Offset */
#define FR_0_MBDOR33         FR_0.MBDOR[33].R              /* MB Data Field Offset */
#define FR_0_MBDOR34         FR_0.MBDOR[34].R              /* MB Data Field Offset */
#define FR_0_MBDOR35         FR_0.MBDOR[35].R              /* MB Data Field Offset */
#define FR_0_MBDOR36         FR_0.MBDOR[36].R              /* MB Data Field Offset */
#define FR_0_MBDOR37         FR_0.MBDOR[37].R              /* MB Data Field Offset */
#define FR_0_MBDOR38         FR_0.MBDOR[38].R              /* MB Data Field Offset */
#define FR_0_MBDOR39         FR_0.MBDOR[39].R              /* MB Data Field Offset */
#define FR_0_MBDOR40         FR_0.MBDOR[40].R              /* MB Data Field Offset */
#define FR_0_MBDOR41         FR_0.MBDOR[41].R              /* MB Data Field Offset */
#define FR_0_MBDOR42         FR_0.MBDOR[42].R              /* MB Data Field Offset */
#define FR_0_MBDOR43         FR_0.MBDOR[43].R              /* MB Data Field Offset */
#define FR_0_MBDOR44         FR_0.MBDOR[44].R              /* MB Data Field Offset */
#define FR_0_MBDOR45         FR_0.MBDOR[45].R              /* MB Data Field Offset */
#define FR_0_MBDOR46         FR_0.MBDOR[46].R              /* MB Data Field Offset */
#define FR_0_MBDOR47         FR_0.MBDOR[47].R              /* MB Data Field Offset */
#define FR_0_MBDOR48         FR_0.MBDOR[48].R              /* MB Data Field Offset */
#define FR_0_MBDOR49         FR_0.MBDOR[49].R              /* MB Data Field Offset */
#define FR_0_MBDOR50         FR_0.MBDOR[50].R              /* MB Data Field Offset */
#define FR_0_MBDOR51         FR_0.MBDOR[51].R              /* MB Data Field Offset */
#define FR_0_MBDOR52         FR_0.MBDOR[52].R              /* MB Data Field Offset */
#define FR_0_MBDOR53         FR_0.MBDOR[53].R              /* MB Data Field Offset */
#define FR_0_MBDOR54         FR_0.MBDOR[54].R              /* MB Data Field Offset */
#define FR_0_MBDOR55         FR_0.MBDOR[55].R              /* MB Data Field Offset */
#define FR_0_MBDOR56         FR_0.MBDOR[56].R              /* MB Data Field Offset */
#define FR_0_MBDOR57         FR_0.MBDOR[57].R              /* MB Data Field Offset */
#define FR_0_MBDOR58         FR_0.MBDOR[58].R              /* MB Data Field Offset */
#define FR_0_MBDOR59         FR_0.MBDOR[59].R              /* MB Data Field Offset */
#define FR_0_MBDOR60         FR_0.MBDOR[60].R              /* MB Data Field Offset */
#define FR_0_MBDOR61         FR_0.MBDOR[61].R              /* MB Data Field Offset */
#define FR_0_MBDOR62         FR_0.MBDOR[62].R              /* MB Data Field Offset */
#define FR_0_MBDOR63         FR_0.MBDOR[63].R              /* MB Data Field Offset */
#define FR_0_MBDOR64         FR_0.MBDOR[64].R              /* MB Data Field Offset */
#define FR_0_MBDOR65         FR_0.MBDOR[65].R              /* MB Data Field Offset */
#define FR_0_MBDOR66         FR_0.MBDOR[66].R              /* MB Data Field Offset */
#define FR_0_MBDOR67         FR_0.MBDOR[67].R              /* MB Data Field Offset */
#define FR_0_MBDOR68         FR_0.MBDOR[68].R              /* MB Data Field Offset */
#define FR_0_MBDOR69         FR_0.MBDOR[69].R              /* MB Data Field Offset */
#define FR_0_MBDOR70         FR_0.MBDOR[70].R              /* MB Data Field Offset */
#define FR_0_MBDOR71         FR_0.MBDOR[71].R              /* MB Data Field Offset */
#define FR_0_MBDOR72         FR_0.MBDOR[72].R              /* MB Data Field Offset */
#define FR_0_MBDOR73         FR_0.MBDOR[73].R              /* MB Data Field Offset */
#define FR_0_MBDOR74         FR_0.MBDOR[74].R              /* MB Data Field Offset */
#define FR_0_MBDOR75         FR_0.MBDOR[75].R              /* MB Data Field Offset */
#define FR_0_MBDOR76         FR_0.MBDOR[76].R              /* MB Data Field Offset */
#define FR_0_MBDOR77         FR_0.MBDOR[77].R              /* MB Data Field Offset */
#define FR_0_MBDOR78         FR_0.MBDOR[78].R              /* MB Data Field Offset */
#define FR_0_MBDOR79         FR_0.MBDOR[79].R              /* MB Data Field Offset */
#define FR_0_MBDOR80         FR_0.MBDOR[80].R              /* MB Data Field Offset */
#define FR_0_MBDOR81         FR_0.MBDOR[81].R              /* MB Data Field Offset */
#define FR_0_MBDOR82         FR_0.MBDOR[82].R              /* MB Data Field Offset */
#define FR_0_MBDOR83         FR_0.MBDOR[83].R              /* MB Data Field Offset */
#define FR_0_MBDOR84         FR_0.MBDOR[84].R              /* MB Data Field Offset */
#define FR_0_MBDOR85         FR_0.MBDOR[85].R              /* MB Data Field Offset */
#define FR_0_MBDOR86         FR_0.MBDOR[86].R              /* MB Data Field Offset */
#define FR_0_MBDOR87         FR_0.MBDOR[87].R              /* MB Data Field Offset */
#define FR_0_MBDOR88         FR_0.MBDOR[88].R              /* MB Data Field Offset */
#define FR_0_MBDOR89         FR_0.MBDOR[89].R              /* MB Data Field Offset */
#define FR_0_MBDOR90         FR_0.MBDOR[90].R              /* MB Data Field Offset */
#define FR_0_MBDOR91         FR_0.MBDOR[91].R              /* MB Data Field Offset */
#define FR_0_MBDOR92         FR_0.MBDOR[92].R              /* MB Data Field Offset */
#define FR_0_MBDOR93         FR_0.MBDOR[93].R              /* MB Data Field Offset */
#define FR_0_MBDOR94         FR_0.MBDOR[94].R              /* MB Data Field Offset */
#define FR_0_MBDOR95         FR_0.MBDOR[95].R              /* MB Data Field Offset */
#define FR_0_MBDOR96         FR_0.MBDOR[96].R              /* MB Data Field Offset */
#define FR_0_MBDOR97         FR_0.MBDOR[97].R              /* MB Data Field Offset */
#define FR_0_MBDOR98         FR_0.MBDOR[98].R              /* MB Data Field Offset */
#define FR_0_MBDOR99         FR_0.MBDOR[99].R              /* MB Data Field Offset */
#define FR_0_MBDOR100        FR_0.MBDOR[100].R             /* MB Data Field Offset */
#define FR_0_MBDOR101        FR_0.MBDOR[101].R             /* MB Data Field Offset */
#define FR_0_MBDOR102        FR_0.MBDOR[102].R             /* MB Data Field Offset */
#define FR_0_MBDOR103        FR_0.MBDOR[103].R             /* MB Data Field Offset */
#define FR_0_MBDOR104        FR_0.MBDOR[104].R             /* MB Data Field Offset */
#define FR_0_MBDOR105        FR_0.MBDOR[105].R             /* MB Data Field Offset */
#define FR_0_MBDOR106        FR_0.MBDOR[106].R             /* MB Data Field Offset */
#define FR_0_MBDOR107        FR_0.MBDOR[107].R             /* MB Data Field Offset */
#define FR_0_MBDOR108        FR_0.MBDOR[108].R             /* MB Data Field Offset */
#define FR_0_MBDOR109        FR_0.MBDOR[109].R             /* MB Data Field Offset */
#define FR_0_MBDOR110        FR_0.MBDOR[110].R             /* MB Data Field Offset */
#define FR_0_MBDOR111        FR_0.MBDOR[111].R             /* MB Data Field Offset */
#define FR_0_MBDOR112        FR_0.MBDOR[112].R             /* MB Data Field Offset */
#define FR_0_MBDOR113        FR_0.MBDOR[113].R             /* MB Data Field Offset */
#define FR_0_MBDOR114        FR_0.MBDOR[114].R             /* MB Data Field Offset */
#define FR_0_MBDOR115        FR_0.MBDOR[115].R             /* MB Data Field Offset */
#define FR_0_MBDOR116        FR_0.MBDOR[116].R             /* MB Data Field Offset */
#define FR_0_MBDOR117        FR_0.MBDOR[117].R             /* MB Data Field Offset */
#define FR_0_MBDOR118        FR_0.MBDOR[118].R             /* MB Data Field Offset */
#define FR_0_MBDOR119        FR_0.MBDOR[119].R             /* MB Data Field Offset */
#define FR_0_MBDOR120        FR_0.MBDOR[120].R             /* MB Data Field Offset */
#define FR_0_MBDOR121        FR_0.MBDOR[121].R             /* MB Data Field Offset */
#define FR_0_MBDOR122        FR_0.MBDOR[122].R             /* MB Data Field Offset */
#define FR_0_MBDOR123        FR_0.MBDOR[123].R             /* MB Data Field Offset */
#define FR_0_MBDOR124        FR_0.MBDOR[124].R             /* MB Data Field Offset */
#define FR_0_MBDOR125        FR_0.MBDOR[125].R             /* MB Data Field Offset */
#define FR_0_MBDOR126        FR_0.MBDOR[126].R             /* MB Data Field Offset */
#define FR_0_MBDOR127        FR_0.MBDOR[127].R             /* MB Data Field Offset */
#define FR_0_MBDOR128        FR_0.MBDOR[128].R             /* MB Data Field Offset */
#define FR_0_MBDOR129        FR_0.MBDOR[129].R             /* MB Data Field Offset */
#define FR_0_MBDOR130        FR_0.MBDOR[130].R             /* MB Data Field Offset */
#define FR_0_MBDOR131        FR_0.MBDOR[131].R             /* MB Data Field Offset */
#define FR_0_MBDOR132        FR_0.MBDOR[132].R             /* MB Data Field Offset */
#define FR_0_MBDOR133        FR_0.MBDOR[133].R             /* MB Data Field Offset */
#define FR_0_MBDOR134        FR_0.MBDOR[134].R             /* MB Data Field Offset */
#define FR_0_MBDOR135        FR_0.MBDOR[135].R             /* MB Data Field Offset */
#define FR_0_MBDOR136        FR_0.MBDOR[136].R             /* MB Data Field Offset */
#define FR_0_MBDOR137        FR_0.MBDOR[137].R             /* MB Data Field Offset */
#define FR_0_MBDOR138        FR_0.MBDOR[138].R             /* MB Data Field Offset */
#define FR_0_MBDOR139        FR_0.MBDOR[139].R             /* MB Data Field Offset */
#define FR_0_MBDOR140        FR_0.MBDOR[140].R             /* MB Data Field Offset */
#define FR_0_MBDOR141        FR_0.MBDOR[141].R             /* MB Data Field Offset */
#define FR_0_MBDOR142        FR_0.MBDOR[142].R             /* MB Data Field Offset */
#define FR_0_MBDOR143        FR_0.MBDOR[143].R             /* MB Data Field Offset */
#define FR_0_MBDOR144        FR_0.MBDOR[144].R             /* MB Data Field Offset */
#define FR_0_MBDOR145        FR_0.MBDOR[145].R             /* MB Data Field Offset */
#define FR_0_MBDOR146        FR_0.MBDOR[146].R             /* MB Data Field Offset */
#define FR_0_MBDOR147        FR_0.MBDOR[147].R             /* MB Data Field Offset */
#define FR_0_MBDOR148        FR_0.MBDOR[148].R             /* MB Data Field Offset */
#define FR_0_MBDOR149        FR_0.MBDOR[149].R             /* MB Data Field Offset */
#define FR_0_MBDOR150        FR_0.MBDOR[150].R             /* MB Data Field Offset */
#define FR_0_MBDOR151        FR_0.MBDOR[151].R             /* MB Data Field Offset */
#define FR_0_MBDOR152        FR_0.MBDOR[152].R             /* MB Data Field Offset */
#define FR_0_MBDOR153        FR_0.MBDOR[153].R             /* MB Data Field Offset */
#define FR_0_MBDOR154        FR_0.MBDOR[154].R             /* MB Data Field Offset */
#define FR_0_MBDOR155        FR_0.MBDOR[155].R             /* MB Data Field Offset */
#define FR_0_MBDOR156        FR_0.MBDOR[156].R             /* MB Data Field Offset */
#define FR_0_MBDOR157        FR_0.MBDOR[157].R             /* MB Data Field Offset */
#define FR_0_MBDOR158        FR_0.MBDOR[158].R             /* MB Data Field Offset */
#define FR_0_MBDOR159        FR_0.MBDOR[159].R             /* MB Data Field Offset */
#define FR_0_MBDOR160        FR_0.MBDOR[160].R             /* MB Data Field Offset */
#define FR_0_MBDOR161        FR_0.MBDOR[161].R             /* MB Data Field Offset */
#define FR_0_MBDOR162        FR_0.MBDOR[162].R             /* MB Data Field Offset */
#define FR_0_MBDOR163        FR_0.MBDOR[163].R             /* MB Data Field Offset */
#define FR_0_MBDOR164        FR_0.MBDOR[164].R             /* MB Data Field Offset */
#define FR_0_MBDOR165        FR_0.MBDOR[165].R             /* MB Data Field Offset */
#define FR_0_MBDOR166        FR_0.MBDOR[166].R             /* MB Data Field Offset */
#define FR_0_MBDOR167        FR_0.MBDOR[167].R             /* MB Data Field Offset */
#define FR_0_MBDOR168        FR_0.MBDOR[168].R             /* MB Data Field Offset */
#define FR_0_MBDOR169        FR_0.MBDOR[169].R             /* MB Data Field Offset */
#define FR_0_MBDOR170        FR_0.MBDOR[170].R             /* MB Data Field Offset */
#define FR_0_MBDOR171        FR_0.MBDOR[171].R             /* MB Data Field Offset */
#define FR_0_MBDOR172        FR_0.MBDOR[172].R             /* MB Data Field Offset */
#define FR_0_MBDOR173        FR_0.MBDOR[173].R             /* MB Data Field Offset */
#define FR_0_MBDOR174        FR_0.MBDOR[174].R             /* MB Data Field Offset */
#define FR_0_MBDOR175        FR_0.MBDOR[175].R             /* MB Data Field Offset */
#define FR_0_MBDOR176        FR_0.MBDOR[176].R             /* MB Data Field Offset */
#define FR_0_MBDOR177        FR_0.MBDOR[177].R             /* MB Data Field Offset */
#define FR_0_MBDOR178        FR_0.MBDOR[178].R             /* MB Data Field Offset */
#define FR_0_MBDOR179        FR_0.MBDOR[179].R             /* MB Data Field Offset */
#define FR_0_MBDOR180        FR_0.MBDOR[180].R             /* MB Data Field Offset */
#define FR_0_MBDOR181        FR_0.MBDOR[181].R             /* MB Data Field Offset */
#define FR_0_MBDOR182        FR_0.MBDOR[182].R             /* MB Data Field Offset */
#define FR_0_MBDOR183        FR_0.MBDOR[183].R             /* MB Data Field Offset */
#define FR_0_MBDOR184        FR_0.MBDOR[184].R             /* MB Data Field Offset */
#define FR_0_MBDOR185        FR_0.MBDOR[185].R             /* MB Data Field Offset */
#define FR_0_MBDOR186        FR_0.MBDOR[186].R             /* MB Data Field Offset */
#define FR_0_MBDOR187        FR_0.MBDOR[187].R             /* MB Data Field Offset */
#define FR_0_MBDOR188        FR_0.MBDOR[188].R             /* MB Data Field Offset */
#define FR_0_MBDOR189        FR_0.MBDOR[189].R             /* MB Data Field Offset */
#define FR_0_MBDOR190        FR_0.MBDOR[190].R             /* MB Data Field Offset */
#define FR_0_MBDOR191        FR_0.MBDOR[191].R             /* MB Data Field Offset */
#define FR_0_MBDOR192        FR_0.MBDOR[192].R             /* MB Data Field Offset */
#define FR_0_MBDOR193        FR_0.MBDOR[193].R             /* MB Data Field Offset */
#define FR_0_MBDOR194        FR_0.MBDOR[194].R             /* MB Data Field Offset */
#define FR_0_MBDOR195        FR_0.MBDOR[195].R             /* MB Data Field Offset */
#define FR_0_MBDOR196        FR_0.MBDOR[196].R             /* MB Data Field Offset */
#define FR_0_MBDOR197        FR_0.MBDOR[197].R             /* MB Data Field Offset */
#define FR_0_MBDOR198        FR_0.MBDOR[198].R             /* MB Data Field Offset */
#define FR_0_MBDOR199        FR_0.MBDOR[199].R             /* MB Data Field Offset */
#define FR_0_MBDOR200        FR_0.MBDOR[200].R             /* MB Data Field Offset */
#define FR_0_MBDOR201        FR_0.MBDOR[201].R             /* MB Data Field Offset */
#define FR_0_MBDOR202        FR_0.MBDOR[202].R             /* MB Data Field Offset */
#define FR_0_MBDOR203        FR_0.MBDOR[203].R             /* MB Data Field Offset */
#define FR_0_MBDOR204        FR_0.MBDOR[204].R             /* MB Data Field Offset */
#define FR_0_MBDOR205        FR_0.MBDOR[205].R             /* MB Data Field Offset */
#define FR_0_MBDOR206        FR_0.MBDOR[206].R             /* MB Data Field Offset */
#define FR_0_MBDOR207        FR_0.MBDOR[207].R             /* MB Data Field Offset */
#define FR_0_MBDOR208        FR_0.MBDOR[208].R             /* MB Data Field Offset */
#define FR_0_MBDOR209        FR_0.MBDOR[209].R             /* MB Data Field Offset */
#define FR_0_MBDOR210        FR_0.MBDOR[210].R             /* MB Data Field Offset */
#define FR_0_MBDOR211        FR_0.MBDOR[211].R             /* MB Data Field Offset */
#define FR_0_MBDOR212        FR_0.MBDOR[212].R             /* MB Data Field Offset */
#define FR_0_MBDOR213        FR_0.MBDOR[213].R             /* MB Data Field Offset */
#define FR_0_MBDOR214        FR_0.MBDOR[214].R             /* MB Data Field Offset */
#define FR_0_MBDOR215        FR_0.MBDOR[215].R             /* MB Data Field Offset */
#define FR_0_MBDOR216        FR_0.MBDOR[216].R             /* MB Data Field Offset */
#define FR_0_MBDOR217        FR_0.MBDOR[217].R             /* MB Data Field Offset */
#define FR_0_MBDOR218        FR_0.MBDOR[218].R             /* MB Data Field Offset */
#define FR_0_MBDOR219        FR_0.MBDOR[219].R             /* MB Data Field Offset */
#define FR_0_MBDOR220        FR_0.MBDOR[220].R             /* MB Data Field Offset */
#define FR_0_MBDOR221        FR_0.MBDOR[221].R             /* MB Data Field Offset */
#define FR_0_MBDOR222        FR_0.MBDOR[222].R             /* MB Data Field Offset */
#define FR_0_MBDOR223        FR_0.MBDOR[223].R             /* MB Data Field Offset */
#define FR_0_MBDOR224        FR_0.MBDOR[224].R             /* MB Data Field Offset */
#define FR_0_MBDOR225        FR_0.MBDOR[225].R             /* MB Data Field Offset */
#define FR_0_MBDOR226        FR_0.MBDOR[226].R             /* MB Data Field Offset */
#define FR_0_MBDOR227        FR_0.MBDOR[227].R             /* MB Data Field Offset */
#define FR_0_MBDOR228        FR_0.MBDOR[228].R             /* MB Data Field Offset */
#define FR_0_MBDOR229        FR_0.MBDOR[229].R             /* MB Data Field Offset */
#define FR_0_MBDOR230        FR_0.MBDOR[230].R             /* MB Data Field Offset */
#define FR_0_MBDOR231        FR_0.MBDOR[231].R             /* MB Data Field Offset */
#define FR_0_MBDOR232        FR_0.MBDOR[232].R             /* MB Data Field Offset */
#define FR_0_MBDOR233        FR_0.MBDOR[233].R             /* MB Data Field Offset */
#define FR_0_MBDOR234        FR_0.MBDOR[234].R             /* MB Data Field Offset */
#define FR_0_MBDOR235        FR_0.MBDOR[235].R             /* MB Data Field Offset */
#define FR_0_MBDOR236        FR_0.MBDOR[236].R             /* MB Data Field Offset */
#define FR_0_MBDOR237        FR_0.MBDOR[237].R             /* MB Data Field Offset */
#define FR_0_MBDOR238        FR_0.MBDOR[238].R             /* MB Data Field Offset */
#define FR_0_MBDOR239        FR_0.MBDOR[239].R             /* MB Data Field Offset */
#define FR_0_MBDOR240        FR_0.MBDOR[240].R             /* MB Data Field Offset */
#define FR_0_MBDOR241        FR_0.MBDOR[241].R             /* MB Data Field Offset */
#define FR_0_MBDOR242        FR_0.MBDOR[242].R             /* MB Data Field Offset */
#define FR_0_MBDOR243        FR_0.MBDOR[243].R             /* MB Data Field Offset */
#define FR_0_MBDOR244        FR_0.MBDOR[244].R             /* MB Data Field Offset */
#define FR_0_MBDOR245        FR_0.MBDOR[245].R             /* MB Data Field Offset */
#define FR_0_MBDOR246        FR_0.MBDOR[246].R             /* MB Data Field Offset */
#define FR_0_MBDOR247        FR_0.MBDOR[247].R             /* MB Data Field Offset */
#define FR_0_MBDOR248        FR_0.MBDOR[248].R             /* MB Data Field Offset */
#define FR_0_MBDOR249        FR_0.MBDOR[249].R             /* MB Data Field Offset */
#define FR_0_MBDOR250        FR_0.MBDOR[250].R             /* MB Data Field Offset */
#define FR_0_MBDOR251        FR_0.MBDOR[251].R             /* MB Data Field Offset */
#define FR_0_MBDOR252        FR_0.MBDOR[252].R             /* MB Data Field Offset */
#define FR_0_MBDOR253        FR_0.MBDOR[253].R             /* MB Data Field Offset */
#define FR_0_MBDOR254        FR_0.MBDOR[254].R             /* MB Data Field Offset */
#define FR_0_MBDOR255        FR_0.MBDOR[255].R             /* MB Data Field Offset */
#define FR_0_MBDOR256        FR_0.MBDOR[256].R             /* MB Data Field Offset */
#define FR_0_MBDOR257        FR_0.MBDOR[257].R             /* MB Data Field Offset */
#define FR_0_MBDOR258        FR_0.MBDOR[258].R             /* MB Data Field Offset */
#define FR_0_MBDOR259        FR_0.MBDOR[259].R             /* MB Data Field Offset */
#define FR_0_LEETR0          FR_0.LEETR[0].R               /* LRAM ECC Error Test Register */
#define FR_0_LEETR1          FR_0.LEETR[1].R               /* LRAM ECC Error Test Register */
#define FR_0_LEETR2          FR_0.LEETR[2].R               /* LRAM ECC Error Test Register */
#define FR_0_LEETR3          FR_0.LEETR[3].R               /* LRAM ECC Error Test Register */
#define FR_0_LEETR4          FR_0.LEETR[4].R               /* LRAM ECC Error Test Register */
#define FR_0_LEETR5          FR_0.LEETR[5].R               /* LRAM ECC Error Test Register */

#define FR_1_MVR             FR_1.MVR.R                    /* Module Version */
#define FR_1_MCR             FR_1.MCR.R                    /* Module Configuration */
#define FR_1_SYMBADHR        FR_1.SYMBADHR.R               /* System Memory Base Address High */
#define FR_1_SYMBADLR        FR_1.SYMBADLR.R               /* System Memory Base Address Low */
#define FR_1_STBSCR          FR_1.STBSCR.R                 /* Strobe Signal Control */
#define FR_1_MBDSR           FR_1.MBDSR.R                  /* MB Data Size */
#define FR_1_MBSSUTR         FR_1.MBSSUTR.R                /* MB Segment Size and Utilization */
#define FR_1_PEDRAR          FR_1.PEDRAR.R                 /* PE DRAM Access */
#define FR_1_PEDRDR          FR_1.PEDRDR.R                 /* PE DRAM Data */
#define FR_1_POCR            FR_1.POCR.R                   /* Protocol Operation Control */
#define FR_1_GIFER           FR_1.GIFER.R                  /* Global Interrupt Flag And Enable */
#define FR_1_PIFR0           FR_1.PIFR0.R                  /* Protocol Interrupt Flag 0 */
#define FR_1_PIFR1           FR_1.PIFR1.R                  /* Protocol Interrupt Flag 1 */
#define FR_1_PIER0           FR_1.PIER0.R                  /* Protocol Interrupt Enable 0 */
#define FR_1_PIER1           FR_1.PIER1.R                  /* Protocol Interrupt Enable 1 */
#define FR_1_CHIERFR         FR_1.CHIERFR.R                /* CHI Error Flag */
#define FR_1_MBIVEC          FR_1.MBIVEC.R                 /* MB Interrupt Vector */
#define FR_1_CASERCR         FR_1.CASERCR.R                /* Channel A Status Error Counter Register */
#define FR_1_CBSERCR         FR_1.CBSERCR.R                /* Channel B Status Error Counter */
#define FR_1_PSR0            FR_1.PSR0.R                   /* Protocol Status 0 */
#define FR_1_PSR1            FR_1.PSR1.R                   /* Protocol Status 1 */
#define FR_1_PSR2            FR_1.PSR2.R                   /* Protocol Status 2 */
#define FR_1_PSR3            FR_1.PSR3.R                   /* Protocol Status 3 */
#define FR_1_MTCTR           FR_1.MTCTR.R                  /* MT Counter */
#define FR_1_CYCTR           FR_1.CYCTR.R                  /* Cycle Counter */
#define FR_1_SLTCTAR         FR_1.SLTCTAR.R                /* Slot Counter Channel A */
#define FR_1_SLTCTBR         FR_1.SLTCTBR.R                /* Slot Counter Channel B */
#define FR_1_RTCORVR         FR_1.RTCORVR.R                /* Rate Correction Value */
#define FR_1_OFCORVR         FR_1.OFCORVR.R                /* Offset Correction Value */
#define FR_1_CIFR            FR_1.CIFR.R                   /* Combined Interrupt Flag */
#define FR_1_SYMATOR         FR_1.SYMATOR.R                /* System Memory Access Timeout */
#define FR_1_SFCNTR          FR_1.SFCNTR.R                 /* Sync Frame Counter */
#define FR_1_SFTOR           FR_1.SFTOR.R                  /* Sync Frame Table Offset */
#define FR_1_SFTCCSR         FR_1.SFTCCSR.R                /* Sync Frame Table Configuration Control Status */
#define FR_1_SFIDRFR         FR_1.SFIDRFR.R                /* Sync Frame ID Rejection Filter */
#define FR_1_SFIDAFVR        FR_1.SFIDAFVR.R               /* Sync Frame ID Acceptance Filter Value */
#define FR_1_SFIDAFMR        FR_1.SFIDAFMR.R               /* Sync Frame ID Acceptance Filter Mask */
#define FR_1_NMVR0           FR_1.NMVR[0].R                /* NMV 0 */
#define FR_1_NMVR1           FR_1.NMVR[1].R                /* NMV 1 */
#define FR_1_NMVR2           FR_1.NMVR[2].R                /* NMV 2 */
#define FR_1_NMVR3           FR_1.NMVR[3].R                /* NMV 3 */
#define FR_1_NMVR4           FR_1.NMVR[4].R                /* NMV 4 */
#define FR_1_NMVR5           FR_1.NMVR[5].R                /* NMV 5 */
#define FR_1_NMVLR           FR_1.NMVLR.R                  /* Network Management Vector Length Register */
#define FR_1_TICCR           FR_1.TICCR.R                  /* Timer Configuration And Control */
#define FR_1_TI1CYSR         FR_1.TI1CYSR.R                /* Timer 1 Cycle Set */
#define FR_1_TI1MTOR         FR_1.TI1MTOR.R                /* Timer 1 MT Offset */
#define FR_1_TI2CR0_ABS      FR_1.TI2CR0.ABS.R             /* Timer 2 Configuration 0 (Absolute Timer Configuration) */
#define FR_1_TI2CR0_REL      FR_1.TI2CR0.REL.R             /* Timer 2 Configuration 0 (Relative Timer Configuration) */
#define FR_1_TI2CR1_ABS      FR_1.TI2CR1.ABS.R             /* Timer 2 Configuration 1 (Absolute Timer Configuration) */
#define FR_1_TI2CR1_REL      FR_1.TI2CR1.REL.R             /* Timer 2 Configuration 1 (Relative Timer Configuration) */
#define FR_1_SSSR            FR_1.SSSR.R                   /* Slot Status Selection */
#define FR_1_SSCCR           FR_1.SSCCR.R                  /* Slot Status Counter Condition */
#define FR_1_SSR0            FR_1.SSR[0].R                 /* Slot Status */
#define FR_1_SSR1            FR_1.SSR[1].R                 /* Slot Status */
#define FR_1_SSR2            FR_1.SSR[2].R                 /* Slot Status */
#define FR_1_SSR3            FR_1.SSR[3].R                 /* Slot Status */
#define FR_1_SSR4            FR_1.SSR[4].R                 /* Slot Status */
#define FR_1_SSR5            FR_1.SSR[5].R                 /* Slot Status */
#define FR_1_SSR6            FR_1.SSR[6].R                 /* Slot Status */
#define FR_1_SSR7            FR_1.SSR[7].R                 /* Slot Status */
#define FR_1_SSCR0           FR_1.SSCR[0].R                /* Slot Status Counter */
#define FR_1_SSCR1           FR_1.SSCR[1].R                /* Slot Status Counter */
#define FR_1_SSCR2           FR_1.SSCR[2].R                /* Slot Status Counter */
#define FR_1_SSCR3           FR_1.SSCR[3].R                /* Slot Status Counter */
#define FR_1_MTSACFR         FR_1.MTSACFR.R                /* MTS A Configuration */
#define FR_1_MTSBCFR         FR_1.MTSBCFR.R                /* MTS B Configuration */
#define FR_1_RSBIR           FR_1.RSBIR.R                  /* Receive Shadow Buffer Index */
#define FR_1_RFWMSR          FR_1.RFWMSR.R                 /* Receive FIFO Watermark And Selection */
#define FR_1_RFSIR           FR_1.RFSIR.R                  /* Receive FIFO Start Index */
#define FR_1_RFDSR           FR_1.RFDSR.R                  /* Receive FIFO Depth And Size */
#define FR_1_RFARIR          FR_1.RFARIR.R                 /* Receive FIFO A Read Index */
#define FR_1_RFBRIR          FR_1.RFBRIR.R                 /* Receive FIFO B Read Index */
#define FR_1_RFMIDAFVR       FR_1.RFMIDAFVR.R              /* Receive FIFO Message ID Acceptance Filter Value */
#define FR_1_RFMIDAFMR       FR_1.RFMIDAFMR.R              /* Receive FIFO Message ID Acceptance Filter Mask */
#define FR_1_RFFIDRFVR       FR_1.RFFIDRFVR.R              /* Receive FIFO Frame ID Rejection Filter Value */
#define FR_1_RFFIDRFMR       FR_1.RFFIDRFMR.R              /* Receive FIFO Frame ID Rejection Filter Mask */
#define FR_1_RFRFCFR         FR_1.RFRFCFR.R                /* Receive FIFO Range Filter Configuration */
#define FR_1_RFRFCTR         FR_1.RFRFCTR.R                /* Receive FIFO Range Filter Control */
#define FR_1_LDTXSLAR        FR_1.LDTXSLAR.R               /* Last Dynamic Transmit Slot Channel A */
#define FR_1_LDTXSLBR        FR_1.LDTXSLBR.R               /* Last Dynamic Transmit Slot Channel B */
#define FR_1_PCR0            FR_1.PCR0.R                   /* Protocol Configuration 0 */
#define FR_1_PCR1            FR_1.PCR1.R                   /* Protocol Configuration 1 */
#define FR_1_PCR2            FR_1.PCR2.R                   /* Protocol Configuration 2 */
#define FR_1_PCR3            FR_1.PCR3.R                   /* Protocol Configuration 3 */
#define FR_1_PCR4            FR_1.PCR4.R                   /* Protocol Configuration 4 */
#define FR_1_PCR5            FR_1.PCR5.R                   /* Protocol Configuration 5 */
#define FR_1_PCR6            FR_1.PCR6.R                   /* Protocol Configuration 6 */
#define FR_1_PCR7            FR_1.PCR7.R                   /* Protocol Configuration 7 */
#define FR_1_PCR8            FR_1.PCR8.R                   /* Protocol Configuration 8 */
#define FR_1_PCR9            FR_1.PCR9.R                   /* Protocol Configuration 9 */
#define FR_1_PCR10           FR_1.PCR10.R                  /* Protocol Configuration 10 */
#define FR_1_PCR11           FR_1.PCR11.R                  /* Protocol Configuration 11 */
#define FR_1_PCR12           FR_1.PCR12.R                  /* Protocol Configuration 12 */
#define FR_1_PCR13           FR_1.PCR13.R                  /* Protocol Configuration 13 */
#define FR_1_PCR14           FR_1.PCR14.R                  /* Protocol Configuration 14 */
#define FR_1_PCR15           FR_1.PCR15.R                  /* Protocol Configuration 15 */
#define FR_1_PCR16           FR_1.PCR16.R                  /* Protocol Configuration 16 */
#define FR_1_PCR17           FR_1.PCR17.R                  /* Protocol Configuration 17 */
#define FR_1_PCR18           FR_1.PCR18.R                  /* Protocol Configuration 18 */
#define FR_1_PCR19           FR_1.PCR19.R                  /* Protocol Configuration 19 */
#define FR_1_PCR20           FR_1.PCR20.R                  /* Protocol Configuration 20 */
#define FR_1_PCR21           FR_1.PCR21.R                  /* Protocol Configuration 21 */
#define FR_1_PCR22           FR_1.PCR22.R                  /* Protocol Configuration 22 */
#define FR_1_PCR23           FR_1.PCR23.R                  /* Protocol Configuration 23 */
#define FR_1_PCR24           FR_1.PCR24.R                  /* Protocol Configuration 24 */
#define FR_1_PCR25           FR_1.PCR25.R                  /* Protocol Configuration 25 */
#define FR_1_PCR26           FR_1.PCR26.R                  /* Protocol Configuration 26 */
#define FR_1_PCR27           FR_1.PCR27.R                  /* Protocol Configuration 27 */
#define FR_1_PCR28           FR_1.PCR28.R                  /* Protocol Configuration 28 */
#define FR_1_PCR29           FR_1.PCR29.R                  /* Protocol Configuration 29 */
#define FR_1_PCR30           FR_1.PCR30.R                  /* Protocol Configuration 30 */
#define FR_1_STPWHR          FR_1.STPWHR.R                 /* Stopwatch Count High */
#define FR_1_STPWLR          FR_1.STPWLR.R                 /* Stopwatch Count Low */
#define FR_1_PEOER           FR_1.PEOER.R                  /* Protocol Event Output Enable And Stopwatch Control */
#define FR_1_RFSDOR          FR_1.RFSDOR.R                 /* Receive FIFO Start Data Offset */
#define FR_1_RFSYMBADHR      FR_1.RFSYMBADHR.R             /* Receive FIFO System Memory Base Address High */
#define FR_1_RFSYMBADLR      FR_1.RFSYMBADLR.R             /* Receive FIFO System Memory Base Address Low */
#define FR_1_RFPTR           FR_1.RFPTR.R                  /* Receive FIFO Periodic Timer */
#define FR_1_RFFLPCR         FR_1.RFFLPCR.R                /* Receive FIFO Fill Level and Pop Count */
#define FR_1_EEIFER          FR_1.EEIFER.R                 /* ECC Error Interrupt Flag And Enable */
#define FR_1_EERICR          FR_1.EERICR.R                 /* ECC Error Report And Injection Control */
#define FR_1_EERAR           FR_1.EERAR.R                  /* ECC Error Report Address */
#define FR_1_EERDR           FR_1.EERDR.R                  /* ECC Error Report Data */
#define FR_1_EERCR           FR_1.EERCR.R                  /* ECC Error Report Code */
#define FR_1_EEIAR           FR_1.EEIAR.R                  /* ECC Error Injection Address */
#define FR_1_EEIDR           FR_1.EEIDR.R                  /* ECC Error Injection Data */
#define FR_1_EEICR           FR_1.EEICR.R                  /* ECC Error Injection Code */
#define FR_1_MBCCSR0         FR_1.MB[0].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR0         FR_1.MB[0].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR0         FR_1.MB[0].FIDR.R             /* MB FID */
#define FR_1_MBIDXR0         FR_1.MB[0].IDXR.R             /* MB Index */
#define FR_1_MBCCSR1         FR_1.MB[1].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR1         FR_1.MB[1].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR1         FR_1.MB[1].FIDR.R             /* MB FID */
#define FR_1_MBIDXR1         FR_1.MB[1].IDXR.R             /* MB Index */
#define FR_1_MBCCSR2         FR_1.MB[2].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR2         FR_1.MB[2].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR2         FR_1.MB[2].FIDR.R             /* MB FID */
#define FR_1_MBIDXR2         FR_1.MB[2].IDXR.R             /* MB Index */
#define FR_1_MBCCSR3         FR_1.MB[3].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR3         FR_1.MB[3].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR3         FR_1.MB[3].FIDR.R             /* MB FID */
#define FR_1_MBIDXR3         FR_1.MB[3].IDXR.R             /* MB Index */
#define FR_1_MBCCSR4         FR_1.MB[4].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR4         FR_1.MB[4].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR4         FR_1.MB[4].FIDR.R             /* MB FID */
#define FR_1_MBIDXR4         FR_1.MB[4].IDXR.R             /* MB Index */
#define FR_1_MBCCSR5         FR_1.MB[5].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR5         FR_1.MB[5].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR5         FR_1.MB[5].FIDR.R             /* MB FID */
#define FR_1_MBIDXR5         FR_1.MB[5].IDXR.R             /* MB Index */
#define FR_1_MBCCSR6         FR_1.MB[6].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR6         FR_1.MB[6].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR6         FR_1.MB[6].FIDR.R             /* MB FID */
#define FR_1_MBIDXR6         FR_1.MB[6].IDXR.R             /* MB Index */
#define FR_1_MBCCSR7         FR_1.MB[7].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR7         FR_1.MB[7].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR7         FR_1.MB[7].FIDR.R             /* MB FID */
#define FR_1_MBIDXR7         FR_1.MB[7].IDXR.R             /* MB Index */
#define FR_1_MBCCSR8         FR_1.MB[8].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR8         FR_1.MB[8].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR8         FR_1.MB[8].FIDR.R             /* MB FID */
#define FR_1_MBIDXR8         FR_1.MB[8].IDXR.R             /* MB Index */
#define FR_1_MBCCSR9         FR_1.MB[9].CCSR.R             /* MB Configuration Control Status */
#define FR_1_MBCCFR9         FR_1.MB[9].CCFR.R             /* MB Cycle Counter Filter */
#define FR_1_MBFIDR9         FR_1.MB[9].FIDR.R             /* MB FID */
#define FR_1_MBIDXR9         FR_1.MB[9].IDXR.R             /* MB Index */
#define FR_1_MBCCSR10        FR_1.MB[10].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR10        FR_1.MB[10].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR10        FR_1.MB[10].FIDR.R            /* MB FID */
#define FR_1_MBIDXR10        FR_1.MB[10].IDXR.R            /* MB Index */
#define FR_1_MBCCSR11        FR_1.MB[11].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR11        FR_1.MB[11].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR11        FR_1.MB[11].FIDR.R            /* MB FID */
#define FR_1_MBIDXR11        FR_1.MB[11].IDXR.R            /* MB Index */
#define FR_1_MBCCSR12        FR_1.MB[12].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR12        FR_1.MB[12].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR12        FR_1.MB[12].FIDR.R            /* MB FID */
#define FR_1_MBIDXR12        FR_1.MB[12].IDXR.R            /* MB Index */
#define FR_1_MBCCSR13        FR_1.MB[13].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR13        FR_1.MB[13].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR13        FR_1.MB[13].FIDR.R            /* MB FID */
#define FR_1_MBIDXR13        FR_1.MB[13].IDXR.R            /* MB Index */
#define FR_1_MBCCSR14        FR_1.MB[14].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR14        FR_1.MB[14].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR14        FR_1.MB[14].FIDR.R            /* MB FID */
#define FR_1_MBIDXR14        FR_1.MB[14].IDXR.R            /* MB Index */
#define FR_1_MBCCSR15        FR_1.MB[15].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR15        FR_1.MB[15].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR15        FR_1.MB[15].FIDR.R            /* MB FID */
#define FR_1_MBIDXR15        FR_1.MB[15].IDXR.R            /* MB Index */
#define FR_1_MBCCSR16        FR_1.MB[16].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR16        FR_1.MB[16].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR16        FR_1.MB[16].FIDR.R            /* MB FID */
#define FR_1_MBIDXR16        FR_1.MB[16].IDXR.R            /* MB Index */
#define FR_1_MBCCSR17        FR_1.MB[17].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR17        FR_1.MB[17].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR17        FR_1.MB[17].FIDR.R            /* MB FID */
#define FR_1_MBIDXR17        FR_1.MB[17].IDXR.R            /* MB Index */
#define FR_1_MBCCSR18        FR_1.MB[18].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR18        FR_1.MB[18].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR18        FR_1.MB[18].FIDR.R            /* MB FID */
#define FR_1_MBIDXR18        FR_1.MB[18].IDXR.R            /* MB Index */
#define FR_1_MBCCSR19        FR_1.MB[19].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR19        FR_1.MB[19].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR19        FR_1.MB[19].FIDR.R            /* MB FID */
#define FR_1_MBIDXR19        FR_1.MB[19].IDXR.R            /* MB Index */
#define FR_1_MBCCSR20        FR_1.MB[20].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR20        FR_1.MB[20].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR20        FR_1.MB[20].FIDR.R            /* MB FID */
#define FR_1_MBIDXR20        FR_1.MB[20].IDXR.R            /* MB Index */
#define FR_1_MBCCSR21        FR_1.MB[21].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR21        FR_1.MB[21].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR21        FR_1.MB[21].FIDR.R            /* MB FID */
#define FR_1_MBIDXR21        FR_1.MB[21].IDXR.R            /* MB Index */
#define FR_1_MBCCSR22        FR_1.MB[22].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR22        FR_1.MB[22].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR22        FR_1.MB[22].FIDR.R            /* MB FID */
#define FR_1_MBIDXR22        FR_1.MB[22].IDXR.R            /* MB Index */
#define FR_1_MBCCSR23        FR_1.MB[23].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR23        FR_1.MB[23].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR23        FR_1.MB[23].FIDR.R            /* MB FID */
#define FR_1_MBIDXR23        FR_1.MB[23].IDXR.R            /* MB Index */
#define FR_1_MBCCSR24        FR_1.MB[24].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR24        FR_1.MB[24].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR24        FR_1.MB[24].FIDR.R            /* MB FID */
#define FR_1_MBIDXR24        FR_1.MB[24].IDXR.R            /* MB Index */
#define FR_1_MBCCSR25        FR_1.MB[25].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR25        FR_1.MB[25].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR25        FR_1.MB[25].FIDR.R            /* MB FID */
#define FR_1_MBIDXR25        FR_1.MB[25].IDXR.R            /* MB Index */
#define FR_1_MBCCSR26        FR_1.MB[26].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR26        FR_1.MB[26].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR26        FR_1.MB[26].FIDR.R            /* MB FID */
#define FR_1_MBIDXR26        FR_1.MB[26].IDXR.R            /* MB Index */
#define FR_1_MBCCSR27        FR_1.MB[27].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR27        FR_1.MB[27].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR27        FR_1.MB[27].FIDR.R            /* MB FID */
#define FR_1_MBIDXR27        FR_1.MB[27].IDXR.R            /* MB Index */
#define FR_1_MBCCSR28        FR_1.MB[28].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR28        FR_1.MB[28].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR28        FR_1.MB[28].FIDR.R            /* MB FID */
#define FR_1_MBIDXR28        FR_1.MB[28].IDXR.R            /* MB Index */
#define FR_1_MBCCSR29        FR_1.MB[29].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR29        FR_1.MB[29].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR29        FR_1.MB[29].FIDR.R            /* MB FID */
#define FR_1_MBIDXR29        FR_1.MB[29].IDXR.R            /* MB Index */
#define FR_1_MBCCSR30        FR_1.MB[30].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR30        FR_1.MB[30].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR30        FR_1.MB[30].FIDR.R            /* MB FID */
#define FR_1_MBIDXR30        FR_1.MB[30].IDXR.R            /* MB Index */
#define FR_1_MBCCSR31        FR_1.MB[31].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR31        FR_1.MB[31].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR31        FR_1.MB[31].FIDR.R            /* MB FID */
#define FR_1_MBIDXR31        FR_1.MB[31].IDXR.R            /* MB Index */
#define FR_1_MBCCSR32        FR_1.MB[32].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR32        FR_1.MB[32].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR32        FR_1.MB[32].FIDR.R            /* MB FID */
#define FR_1_MBIDXR32        FR_1.MB[32].IDXR.R            /* MB Index */
#define FR_1_MBCCSR33        FR_1.MB[33].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR33        FR_1.MB[33].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR33        FR_1.MB[33].FIDR.R            /* MB FID */
#define FR_1_MBIDXR33        FR_1.MB[33].IDXR.R            /* MB Index */
#define FR_1_MBCCSR34        FR_1.MB[34].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR34        FR_1.MB[34].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR34        FR_1.MB[34].FIDR.R            /* MB FID */
#define FR_1_MBIDXR34        FR_1.MB[34].IDXR.R            /* MB Index */
#define FR_1_MBCCSR35        FR_1.MB[35].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR35        FR_1.MB[35].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR35        FR_1.MB[35].FIDR.R            /* MB FID */
#define FR_1_MBIDXR35        FR_1.MB[35].IDXR.R            /* MB Index */
#define FR_1_MBCCSR36        FR_1.MB[36].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR36        FR_1.MB[36].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR36        FR_1.MB[36].FIDR.R            /* MB FID */
#define FR_1_MBIDXR36        FR_1.MB[36].IDXR.R            /* MB Index */
#define FR_1_MBCCSR37        FR_1.MB[37].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR37        FR_1.MB[37].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR37        FR_1.MB[37].FIDR.R            /* MB FID */
#define FR_1_MBIDXR37        FR_1.MB[37].IDXR.R            /* MB Index */
#define FR_1_MBCCSR38        FR_1.MB[38].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR38        FR_1.MB[38].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR38        FR_1.MB[38].FIDR.R            /* MB FID */
#define FR_1_MBIDXR38        FR_1.MB[38].IDXR.R            /* MB Index */
#define FR_1_MBCCSR39        FR_1.MB[39].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR39        FR_1.MB[39].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR39        FR_1.MB[39].FIDR.R            /* MB FID */
#define FR_1_MBIDXR39        FR_1.MB[39].IDXR.R            /* MB Index */
#define FR_1_MBCCSR40        FR_1.MB[40].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR40        FR_1.MB[40].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR40        FR_1.MB[40].FIDR.R            /* MB FID */
#define FR_1_MBIDXR40        FR_1.MB[40].IDXR.R            /* MB Index */
#define FR_1_MBCCSR41        FR_1.MB[41].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR41        FR_1.MB[41].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR41        FR_1.MB[41].FIDR.R            /* MB FID */
#define FR_1_MBIDXR41        FR_1.MB[41].IDXR.R            /* MB Index */
#define FR_1_MBCCSR42        FR_1.MB[42].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR42        FR_1.MB[42].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR42        FR_1.MB[42].FIDR.R            /* MB FID */
#define FR_1_MBIDXR42        FR_1.MB[42].IDXR.R            /* MB Index */
#define FR_1_MBCCSR43        FR_1.MB[43].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR43        FR_1.MB[43].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR43        FR_1.MB[43].FIDR.R            /* MB FID */
#define FR_1_MBIDXR43        FR_1.MB[43].IDXR.R            /* MB Index */
#define FR_1_MBCCSR44        FR_1.MB[44].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR44        FR_1.MB[44].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR44        FR_1.MB[44].FIDR.R            /* MB FID */
#define FR_1_MBIDXR44        FR_1.MB[44].IDXR.R            /* MB Index */
#define FR_1_MBCCSR45        FR_1.MB[45].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR45        FR_1.MB[45].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR45        FR_1.MB[45].FIDR.R            /* MB FID */
#define FR_1_MBIDXR45        FR_1.MB[45].IDXR.R            /* MB Index */
#define FR_1_MBCCSR46        FR_1.MB[46].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR46        FR_1.MB[46].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR46        FR_1.MB[46].FIDR.R            /* MB FID */
#define FR_1_MBIDXR46        FR_1.MB[46].IDXR.R            /* MB Index */
#define FR_1_MBCCSR47        FR_1.MB[47].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR47        FR_1.MB[47].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR47        FR_1.MB[47].FIDR.R            /* MB FID */
#define FR_1_MBIDXR47        FR_1.MB[47].IDXR.R            /* MB Index */
#define FR_1_MBCCSR48        FR_1.MB[48].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR48        FR_1.MB[48].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR48        FR_1.MB[48].FIDR.R            /* MB FID */
#define FR_1_MBIDXR48        FR_1.MB[48].IDXR.R            /* MB Index */
#define FR_1_MBCCSR49        FR_1.MB[49].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR49        FR_1.MB[49].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR49        FR_1.MB[49].FIDR.R            /* MB FID */
#define FR_1_MBIDXR49        FR_1.MB[49].IDXR.R            /* MB Index */
#define FR_1_MBCCSR50        FR_1.MB[50].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR50        FR_1.MB[50].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR50        FR_1.MB[50].FIDR.R            /* MB FID */
#define FR_1_MBIDXR50        FR_1.MB[50].IDXR.R            /* MB Index */
#define FR_1_MBCCSR51        FR_1.MB[51].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR51        FR_1.MB[51].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR51        FR_1.MB[51].FIDR.R            /* MB FID */
#define FR_1_MBIDXR51        FR_1.MB[51].IDXR.R            /* MB Index */
#define FR_1_MBCCSR52        FR_1.MB[52].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR52        FR_1.MB[52].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR52        FR_1.MB[52].FIDR.R            /* MB FID */
#define FR_1_MBIDXR52        FR_1.MB[52].IDXR.R            /* MB Index */
#define FR_1_MBCCSR53        FR_1.MB[53].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR53        FR_1.MB[53].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR53        FR_1.MB[53].FIDR.R            /* MB FID */
#define FR_1_MBIDXR53        FR_1.MB[53].IDXR.R            /* MB Index */
#define FR_1_MBCCSR54        FR_1.MB[54].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR54        FR_1.MB[54].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR54        FR_1.MB[54].FIDR.R            /* MB FID */
#define FR_1_MBIDXR54        FR_1.MB[54].IDXR.R            /* MB Index */
#define FR_1_MBCCSR55        FR_1.MB[55].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR55        FR_1.MB[55].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR55        FR_1.MB[55].FIDR.R            /* MB FID */
#define FR_1_MBIDXR55        FR_1.MB[55].IDXR.R            /* MB Index */
#define FR_1_MBCCSR56        FR_1.MB[56].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR56        FR_1.MB[56].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR56        FR_1.MB[56].FIDR.R            /* MB FID */
#define FR_1_MBIDXR56        FR_1.MB[56].IDXR.R            /* MB Index */
#define FR_1_MBCCSR57        FR_1.MB[57].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR57        FR_1.MB[57].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR57        FR_1.MB[57].FIDR.R            /* MB FID */
#define FR_1_MBIDXR57        FR_1.MB[57].IDXR.R            /* MB Index */
#define FR_1_MBCCSR58        FR_1.MB[58].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR58        FR_1.MB[58].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR58        FR_1.MB[58].FIDR.R            /* MB FID */
#define FR_1_MBIDXR58        FR_1.MB[58].IDXR.R            /* MB Index */
#define FR_1_MBCCSR59        FR_1.MB[59].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR59        FR_1.MB[59].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR59        FR_1.MB[59].FIDR.R            /* MB FID */
#define FR_1_MBIDXR59        FR_1.MB[59].IDXR.R            /* MB Index */
#define FR_1_MBCCSR60        FR_1.MB[60].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR60        FR_1.MB[60].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR60        FR_1.MB[60].FIDR.R            /* MB FID */
#define FR_1_MBIDXR60        FR_1.MB[60].IDXR.R            /* MB Index */
#define FR_1_MBCCSR61        FR_1.MB[61].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR61        FR_1.MB[61].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR61        FR_1.MB[61].FIDR.R            /* MB FID */
#define FR_1_MBIDXR61        FR_1.MB[61].IDXR.R            /* MB Index */
#define FR_1_MBCCSR62        FR_1.MB[62].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR62        FR_1.MB[62].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR62        FR_1.MB[62].FIDR.R            /* MB FID */
#define FR_1_MBIDXR62        FR_1.MB[62].IDXR.R            /* MB Index */
#define FR_1_MBCCSR63        FR_1.MB[63].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR63        FR_1.MB[63].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR63        FR_1.MB[63].FIDR.R            /* MB FID */
#define FR_1_MBIDXR63        FR_1.MB[63].IDXR.R            /* MB Index */
#define FR_1_MBCCSR64        FR_1.MB[64].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR64        FR_1.MB[64].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR64        FR_1.MB[64].FIDR.R            /* MB FID */
#define FR_1_MBIDXR64        FR_1.MB[64].IDXR.R            /* MB Index */
#define FR_1_MBCCSR65        FR_1.MB[65].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR65        FR_1.MB[65].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR65        FR_1.MB[65].FIDR.R            /* MB FID */
#define FR_1_MBIDXR65        FR_1.MB[65].IDXR.R            /* MB Index */
#define FR_1_MBCCSR66        FR_1.MB[66].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR66        FR_1.MB[66].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR66        FR_1.MB[66].FIDR.R            /* MB FID */
#define FR_1_MBIDXR66        FR_1.MB[66].IDXR.R            /* MB Index */
#define FR_1_MBCCSR67        FR_1.MB[67].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR67        FR_1.MB[67].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR67        FR_1.MB[67].FIDR.R            /* MB FID */
#define FR_1_MBIDXR67        FR_1.MB[67].IDXR.R            /* MB Index */
#define FR_1_MBCCSR68        FR_1.MB[68].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR68        FR_1.MB[68].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR68        FR_1.MB[68].FIDR.R            /* MB FID */
#define FR_1_MBIDXR68        FR_1.MB[68].IDXR.R            /* MB Index */
#define FR_1_MBCCSR69        FR_1.MB[69].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR69        FR_1.MB[69].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR69        FR_1.MB[69].FIDR.R            /* MB FID */
#define FR_1_MBIDXR69        FR_1.MB[69].IDXR.R            /* MB Index */
#define FR_1_MBCCSR70        FR_1.MB[70].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR70        FR_1.MB[70].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR70        FR_1.MB[70].FIDR.R            /* MB FID */
#define FR_1_MBIDXR70        FR_1.MB[70].IDXR.R            /* MB Index */
#define FR_1_MBCCSR71        FR_1.MB[71].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR71        FR_1.MB[71].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR71        FR_1.MB[71].FIDR.R            /* MB FID */
#define FR_1_MBIDXR71        FR_1.MB[71].IDXR.R            /* MB Index */
#define FR_1_MBCCSR72        FR_1.MB[72].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR72        FR_1.MB[72].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR72        FR_1.MB[72].FIDR.R            /* MB FID */
#define FR_1_MBIDXR72        FR_1.MB[72].IDXR.R            /* MB Index */
#define FR_1_MBCCSR73        FR_1.MB[73].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR73        FR_1.MB[73].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR73        FR_1.MB[73].FIDR.R            /* MB FID */
#define FR_1_MBIDXR73        FR_1.MB[73].IDXR.R            /* MB Index */
#define FR_1_MBCCSR74        FR_1.MB[74].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR74        FR_1.MB[74].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR74        FR_1.MB[74].FIDR.R            /* MB FID */
#define FR_1_MBIDXR74        FR_1.MB[74].IDXR.R            /* MB Index */
#define FR_1_MBCCSR75        FR_1.MB[75].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR75        FR_1.MB[75].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR75        FR_1.MB[75].FIDR.R            /* MB FID */
#define FR_1_MBIDXR75        FR_1.MB[75].IDXR.R            /* MB Index */
#define FR_1_MBCCSR76        FR_1.MB[76].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR76        FR_1.MB[76].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR76        FR_1.MB[76].FIDR.R            /* MB FID */
#define FR_1_MBIDXR76        FR_1.MB[76].IDXR.R            /* MB Index */
#define FR_1_MBCCSR77        FR_1.MB[77].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR77        FR_1.MB[77].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR77        FR_1.MB[77].FIDR.R            /* MB FID */
#define FR_1_MBIDXR77        FR_1.MB[77].IDXR.R            /* MB Index */
#define FR_1_MBCCSR78        FR_1.MB[78].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR78        FR_1.MB[78].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR78        FR_1.MB[78].FIDR.R            /* MB FID */
#define FR_1_MBIDXR78        FR_1.MB[78].IDXR.R            /* MB Index */
#define FR_1_MBCCSR79        FR_1.MB[79].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR79        FR_1.MB[79].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR79        FR_1.MB[79].FIDR.R            /* MB FID */
#define FR_1_MBIDXR79        FR_1.MB[79].IDXR.R            /* MB Index */
#define FR_1_MBCCSR80        FR_1.MB[80].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR80        FR_1.MB[80].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR80        FR_1.MB[80].FIDR.R            /* MB FID */
#define FR_1_MBIDXR80        FR_1.MB[80].IDXR.R            /* MB Index */
#define FR_1_MBCCSR81        FR_1.MB[81].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR81        FR_1.MB[81].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR81        FR_1.MB[81].FIDR.R            /* MB FID */
#define FR_1_MBIDXR81        FR_1.MB[81].IDXR.R            /* MB Index */
#define FR_1_MBCCSR82        FR_1.MB[82].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR82        FR_1.MB[82].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR82        FR_1.MB[82].FIDR.R            /* MB FID */
#define FR_1_MBIDXR82        FR_1.MB[82].IDXR.R            /* MB Index */
#define FR_1_MBCCSR83        FR_1.MB[83].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR83        FR_1.MB[83].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR83        FR_1.MB[83].FIDR.R            /* MB FID */
#define FR_1_MBIDXR83        FR_1.MB[83].IDXR.R            /* MB Index */
#define FR_1_MBCCSR84        FR_1.MB[84].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR84        FR_1.MB[84].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR84        FR_1.MB[84].FIDR.R            /* MB FID */
#define FR_1_MBIDXR84        FR_1.MB[84].IDXR.R            /* MB Index */
#define FR_1_MBCCSR85        FR_1.MB[85].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR85        FR_1.MB[85].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR85        FR_1.MB[85].FIDR.R            /* MB FID */
#define FR_1_MBIDXR85        FR_1.MB[85].IDXR.R            /* MB Index */
#define FR_1_MBCCSR86        FR_1.MB[86].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR86        FR_1.MB[86].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR86        FR_1.MB[86].FIDR.R            /* MB FID */
#define FR_1_MBIDXR86        FR_1.MB[86].IDXR.R            /* MB Index */
#define FR_1_MBCCSR87        FR_1.MB[87].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR87        FR_1.MB[87].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR87        FR_1.MB[87].FIDR.R            /* MB FID */
#define FR_1_MBIDXR87        FR_1.MB[87].IDXR.R            /* MB Index */
#define FR_1_MBCCSR88        FR_1.MB[88].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR88        FR_1.MB[88].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR88        FR_1.MB[88].FIDR.R            /* MB FID */
#define FR_1_MBIDXR88        FR_1.MB[88].IDXR.R            /* MB Index */
#define FR_1_MBCCSR89        FR_1.MB[89].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR89        FR_1.MB[89].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR89        FR_1.MB[89].FIDR.R            /* MB FID */
#define FR_1_MBIDXR89        FR_1.MB[89].IDXR.R            /* MB Index */
#define FR_1_MBCCSR90        FR_1.MB[90].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR90        FR_1.MB[90].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR90        FR_1.MB[90].FIDR.R            /* MB FID */
#define FR_1_MBIDXR90        FR_1.MB[90].IDXR.R            /* MB Index */
#define FR_1_MBCCSR91        FR_1.MB[91].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR91        FR_1.MB[91].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR91        FR_1.MB[91].FIDR.R            /* MB FID */
#define FR_1_MBIDXR91        FR_1.MB[91].IDXR.R            /* MB Index */
#define FR_1_MBCCSR92        FR_1.MB[92].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR92        FR_1.MB[92].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR92        FR_1.MB[92].FIDR.R            /* MB FID */
#define FR_1_MBIDXR92        FR_1.MB[92].IDXR.R            /* MB Index */
#define FR_1_MBCCSR93        FR_1.MB[93].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR93        FR_1.MB[93].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR93        FR_1.MB[93].FIDR.R            /* MB FID */
#define FR_1_MBIDXR93        FR_1.MB[93].IDXR.R            /* MB Index */
#define FR_1_MBCCSR94        FR_1.MB[94].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR94        FR_1.MB[94].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR94        FR_1.MB[94].FIDR.R            /* MB FID */
#define FR_1_MBIDXR94        FR_1.MB[94].IDXR.R            /* MB Index */
#define FR_1_MBCCSR95        FR_1.MB[95].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR95        FR_1.MB[95].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR95        FR_1.MB[95].FIDR.R            /* MB FID */
#define FR_1_MBIDXR95        FR_1.MB[95].IDXR.R            /* MB Index */
#define FR_1_MBCCSR96        FR_1.MB[96].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR96        FR_1.MB[96].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR96        FR_1.MB[96].FIDR.R            /* MB FID */
#define FR_1_MBIDXR96        FR_1.MB[96].IDXR.R            /* MB Index */
#define FR_1_MBCCSR97        FR_1.MB[97].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR97        FR_1.MB[97].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR97        FR_1.MB[97].FIDR.R            /* MB FID */
#define FR_1_MBIDXR97        FR_1.MB[97].IDXR.R            /* MB Index */
#define FR_1_MBCCSR98        FR_1.MB[98].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR98        FR_1.MB[98].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR98        FR_1.MB[98].FIDR.R            /* MB FID */
#define FR_1_MBIDXR98        FR_1.MB[98].IDXR.R            /* MB Index */
#define FR_1_MBCCSR99        FR_1.MB[99].CCSR.R            /* MB Configuration Control Status */
#define FR_1_MBCCFR99        FR_1.MB[99].CCFR.R            /* MB Cycle Counter Filter */
#define FR_1_MBFIDR99        FR_1.MB[99].FIDR.R            /* MB FID */
#define FR_1_MBIDXR99        FR_1.MB[99].IDXR.R            /* MB Index */
#define FR_1_MBCCSR100       FR_1.MB[100].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR100       FR_1.MB[100].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR100       FR_1.MB[100].FIDR.R           /* MB FID */
#define FR_1_MBIDXR100       FR_1.MB[100].IDXR.R           /* MB Index */
#define FR_1_MBCCSR101       FR_1.MB[101].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR101       FR_1.MB[101].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR101       FR_1.MB[101].FIDR.R           /* MB FID */
#define FR_1_MBIDXR101       FR_1.MB[101].IDXR.R           /* MB Index */
#define FR_1_MBCCSR102       FR_1.MB[102].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR102       FR_1.MB[102].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR102       FR_1.MB[102].FIDR.R           /* MB FID */
#define FR_1_MBIDXR102       FR_1.MB[102].IDXR.R           /* MB Index */
#define FR_1_MBCCSR103       FR_1.MB[103].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR103       FR_1.MB[103].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR103       FR_1.MB[103].FIDR.R           /* MB FID */
#define FR_1_MBIDXR103       FR_1.MB[103].IDXR.R           /* MB Index */
#define FR_1_MBCCSR104       FR_1.MB[104].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR104       FR_1.MB[104].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR104       FR_1.MB[104].FIDR.R           /* MB FID */
#define FR_1_MBIDXR104       FR_1.MB[104].IDXR.R           /* MB Index */
#define FR_1_MBCCSR105       FR_1.MB[105].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR105       FR_1.MB[105].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR105       FR_1.MB[105].FIDR.R           /* MB FID */
#define FR_1_MBIDXR105       FR_1.MB[105].IDXR.R           /* MB Index */
#define FR_1_MBCCSR106       FR_1.MB[106].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR106       FR_1.MB[106].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR106       FR_1.MB[106].FIDR.R           /* MB FID */
#define FR_1_MBIDXR106       FR_1.MB[106].IDXR.R           /* MB Index */
#define FR_1_MBCCSR107       FR_1.MB[107].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR107       FR_1.MB[107].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR107       FR_1.MB[107].FIDR.R           /* MB FID */
#define FR_1_MBIDXR107       FR_1.MB[107].IDXR.R           /* MB Index */
#define FR_1_MBCCSR108       FR_1.MB[108].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR108       FR_1.MB[108].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR108       FR_1.MB[108].FIDR.R           /* MB FID */
#define FR_1_MBIDXR108       FR_1.MB[108].IDXR.R           /* MB Index */
#define FR_1_MBCCSR109       FR_1.MB[109].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR109       FR_1.MB[109].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR109       FR_1.MB[109].FIDR.R           /* MB FID */
#define FR_1_MBIDXR109       FR_1.MB[109].IDXR.R           /* MB Index */
#define FR_1_MBCCSR110       FR_1.MB[110].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR110       FR_1.MB[110].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR110       FR_1.MB[110].FIDR.R           /* MB FID */
#define FR_1_MBIDXR110       FR_1.MB[110].IDXR.R           /* MB Index */
#define FR_1_MBCCSR111       FR_1.MB[111].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR111       FR_1.MB[111].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR111       FR_1.MB[111].FIDR.R           /* MB FID */
#define FR_1_MBIDXR111       FR_1.MB[111].IDXR.R           /* MB Index */
#define FR_1_MBCCSR112       FR_1.MB[112].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR112       FR_1.MB[112].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR112       FR_1.MB[112].FIDR.R           /* MB FID */
#define FR_1_MBIDXR112       FR_1.MB[112].IDXR.R           /* MB Index */
#define FR_1_MBCCSR113       FR_1.MB[113].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR113       FR_1.MB[113].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR113       FR_1.MB[113].FIDR.R           /* MB FID */
#define FR_1_MBIDXR113       FR_1.MB[113].IDXR.R           /* MB Index */
#define FR_1_MBCCSR114       FR_1.MB[114].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR114       FR_1.MB[114].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR114       FR_1.MB[114].FIDR.R           /* MB FID */
#define FR_1_MBIDXR114       FR_1.MB[114].IDXR.R           /* MB Index */
#define FR_1_MBCCSR115       FR_1.MB[115].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR115       FR_1.MB[115].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR115       FR_1.MB[115].FIDR.R           /* MB FID */
#define FR_1_MBIDXR115       FR_1.MB[115].IDXR.R           /* MB Index */
#define FR_1_MBCCSR116       FR_1.MB[116].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR116       FR_1.MB[116].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR116       FR_1.MB[116].FIDR.R           /* MB FID */
#define FR_1_MBIDXR116       FR_1.MB[116].IDXR.R           /* MB Index */
#define FR_1_MBCCSR117       FR_1.MB[117].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR117       FR_1.MB[117].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR117       FR_1.MB[117].FIDR.R           /* MB FID */
#define FR_1_MBIDXR117       FR_1.MB[117].IDXR.R           /* MB Index */
#define FR_1_MBCCSR118       FR_1.MB[118].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR118       FR_1.MB[118].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR118       FR_1.MB[118].FIDR.R           /* MB FID */
#define FR_1_MBIDXR118       FR_1.MB[118].IDXR.R           /* MB Index */
#define FR_1_MBCCSR119       FR_1.MB[119].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR119       FR_1.MB[119].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR119       FR_1.MB[119].FIDR.R           /* MB FID */
#define FR_1_MBIDXR119       FR_1.MB[119].IDXR.R           /* MB Index */
#define FR_1_MBCCSR120       FR_1.MB[120].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR120       FR_1.MB[120].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR120       FR_1.MB[120].FIDR.R           /* MB FID */
#define FR_1_MBIDXR120       FR_1.MB[120].IDXR.R           /* MB Index */
#define FR_1_MBCCSR121       FR_1.MB[121].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR121       FR_1.MB[121].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR121       FR_1.MB[121].FIDR.R           /* MB FID */
#define FR_1_MBIDXR121       FR_1.MB[121].IDXR.R           /* MB Index */
#define FR_1_MBCCSR122       FR_1.MB[122].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR122       FR_1.MB[122].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR122       FR_1.MB[122].FIDR.R           /* MB FID */
#define FR_1_MBIDXR122       FR_1.MB[122].IDXR.R           /* MB Index */
#define FR_1_MBCCSR123       FR_1.MB[123].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR123       FR_1.MB[123].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR123       FR_1.MB[123].FIDR.R           /* MB FID */
#define FR_1_MBIDXR123       FR_1.MB[123].IDXR.R           /* MB Index */
#define FR_1_MBCCSR124       FR_1.MB[124].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR124       FR_1.MB[124].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR124       FR_1.MB[124].FIDR.R           /* MB FID */
#define FR_1_MBIDXR124       FR_1.MB[124].IDXR.R           /* MB Index */
#define FR_1_MBCCSR125       FR_1.MB[125].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR125       FR_1.MB[125].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR125       FR_1.MB[125].FIDR.R           /* MB FID */
#define FR_1_MBIDXR125       FR_1.MB[125].IDXR.R           /* MB Index */
#define FR_1_MBCCSR126       FR_1.MB[126].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR126       FR_1.MB[126].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR126       FR_1.MB[126].FIDR.R           /* MB FID */
#define FR_1_MBIDXR126       FR_1.MB[126].IDXR.R           /* MB Index */
#define FR_1_MBCCSR127       FR_1.MB[127].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR127       FR_1.MB[127].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR127       FR_1.MB[127].FIDR.R           /* MB FID */
#define FR_1_MBIDXR127       FR_1.MB[127].IDXR.R           /* MB Index */
#define FR_1_MBCCSR128       FR_1.MB[128].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR128       FR_1.MB[128].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR128       FR_1.MB[128].FIDR.R           /* MB FID */
#define FR_1_MBIDXR128       FR_1.MB[128].IDXR.R           /* MB Index */
#define FR_1_MBCCSR129       FR_1.MB[129].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR129       FR_1.MB[129].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR129       FR_1.MB[129].FIDR.R           /* MB FID */
#define FR_1_MBIDXR129       FR_1.MB[129].IDXR.R           /* MB Index */
#define FR_1_MBCCSR130       FR_1.MB[130].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR130       FR_1.MB[130].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR130       FR_1.MB[130].FIDR.R           /* MB FID */
#define FR_1_MBIDXR130       FR_1.MB[130].IDXR.R           /* MB Index */
#define FR_1_MBCCSR131       FR_1.MB[131].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR131       FR_1.MB[131].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR131       FR_1.MB[131].FIDR.R           /* MB FID */
#define FR_1_MBIDXR131       FR_1.MB[131].IDXR.R           /* MB Index */
#define FR_1_MBCCSR132       FR_1.MB[132].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR132       FR_1.MB[132].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR132       FR_1.MB[132].FIDR.R           /* MB FID */
#define FR_1_MBIDXR132       FR_1.MB[132].IDXR.R           /* MB Index */
#define FR_1_MBCCSR133       FR_1.MB[133].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR133       FR_1.MB[133].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR133       FR_1.MB[133].FIDR.R           /* MB FID */
#define FR_1_MBIDXR133       FR_1.MB[133].IDXR.R           /* MB Index */
#define FR_1_MBCCSR134       FR_1.MB[134].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR134       FR_1.MB[134].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR134       FR_1.MB[134].FIDR.R           /* MB FID */
#define FR_1_MBIDXR134       FR_1.MB[134].IDXR.R           /* MB Index */
#define FR_1_MBCCSR135       FR_1.MB[135].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR135       FR_1.MB[135].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR135       FR_1.MB[135].FIDR.R           /* MB FID */
#define FR_1_MBIDXR135       FR_1.MB[135].IDXR.R           /* MB Index */
#define FR_1_MBCCSR136       FR_1.MB[136].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR136       FR_1.MB[136].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR136       FR_1.MB[136].FIDR.R           /* MB FID */
#define FR_1_MBIDXR136       FR_1.MB[136].IDXR.R           /* MB Index */
#define FR_1_MBCCSR137       FR_1.MB[137].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR137       FR_1.MB[137].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR137       FR_1.MB[137].FIDR.R           /* MB FID */
#define FR_1_MBIDXR137       FR_1.MB[137].IDXR.R           /* MB Index */
#define FR_1_MBCCSR138       FR_1.MB[138].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR138       FR_1.MB[138].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR138       FR_1.MB[138].FIDR.R           /* MB FID */
#define FR_1_MBIDXR138       FR_1.MB[138].IDXR.R           /* MB Index */
#define FR_1_MBCCSR139       FR_1.MB[139].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR139       FR_1.MB[139].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR139       FR_1.MB[139].FIDR.R           /* MB FID */
#define FR_1_MBIDXR139       FR_1.MB[139].IDXR.R           /* MB Index */
#define FR_1_MBCCSR140       FR_1.MB[140].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR140       FR_1.MB[140].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR140       FR_1.MB[140].FIDR.R           /* MB FID */
#define FR_1_MBIDXR140       FR_1.MB[140].IDXR.R           /* MB Index */
#define FR_1_MBCCSR141       FR_1.MB[141].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR141       FR_1.MB[141].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR141       FR_1.MB[141].FIDR.R           /* MB FID */
#define FR_1_MBIDXR141       FR_1.MB[141].IDXR.R           /* MB Index */
#define FR_1_MBCCSR142       FR_1.MB[142].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR142       FR_1.MB[142].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR142       FR_1.MB[142].FIDR.R           /* MB FID */
#define FR_1_MBIDXR142       FR_1.MB[142].IDXR.R           /* MB Index */
#define FR_1_MBCCSR143       FR_1.MB[143].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR143       FR_1.MB[143].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR143       FR_1.MB[143].FIDR.R           /* MB FID */
#define FR_1_MBIDXR143       FR_1.MB[143].IDXR.R           /* MB Index */
#define FR_1_MBCCSR144       FR_1.MB[144].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR144       FR_1.MB[144].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR144       FR_1.MB[144].FIDR.R           /* MB FID */
#define FR_1_MBIDXR144       FR_1.MB[144].IDXR.R           /* MB Index */
#define FR_1_MBCCSR145       FR_1.MB[145].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR145       FR_1.MB[145].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR145       FR_1.MB[145].FIDR.R           /* MB FID */
#define FR_1_MBIDXR145       FR_1.MB[145].IDXR.R           /* MB Index */
#define FR_1_MBCCSR146       FR_1.MB[146].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR146       FR_1.MB[146].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR146       FR_1.MB[146].FIDR.R           /* MB FID */
#define FR_1_MBIDXR146       FR_1.MB[146].IDXR.R           /* MB Index */
#define FR_1_MBCCSR147       FR_1.MB[147].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR147       FR_1.MB[147].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR147       FR_1.MB[147].FIDR.R           /* MB FID */
#define FR_1_MBIDXR147       FR_1.MB[147].IDXR.R           /* MB Index */
#define FR_1_MBCCSR148       FR_1.MB[148].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR148       FR_1.MB[148].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR148       FR_1.MB[148].FIDR.R           /* MB FID */
#define FR_1_MBIDXR148       FR_1.MB[148].IDXR.R           /* MB Index */
#define FR_1_MBCCSR149       FR_1.MB[149].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR149       FR_1.MB[149].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR149       FR_1.MB[149].FIDR.R           /* MB FID */
#define FR_1_MBIDXR149       FR_1.MB[149].IDXR.R           /* MB Index */
#define FR_1_MBCCSR150       FR_1.MB[150].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR150       FR_1.MB[150].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR150       FR_1.MB[150].FIDR.R           /* MB FID */
#define FR_1_MBIDXR150       FR_1.MB[150].IDXR.R           /* MB Index */
#define FR_1_MBCCSR151       FR_1.MB[151].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR151       FR_1.MB[151].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR151       FR_1.MB[151].FIDR.R           /* MB FID */
#define FR_1_MBIDXR151       FR_1.MB[151].IDXR.R           /* MB Index */
#define FR_1_MBCCSR152       FR_1.MB[152].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR152       FR_1.MB[152].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR152       FR_1.MB[152].FIDR.R           /* MB FID */
#define FR_1_MBIDXR152       FR_1.MB[152].IDXR.R           /* MB Index */
#define FR_1_MBCCSR153       FR_1.MB[153].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR153       FR_1.MB[153].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR153       FR_1.MB[153].FIDR.R           /* MB FID */
#define FR_1_MBIDXR153       FR_1.MB[153].IDXR.R           /* MB Index */
#define FR_1_MBCCSR154       FR_1.MB[154].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR154       FR_1.MB[154].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR154       FR_1.MB[154].FIDR.R           /* MB FID */
#define FR_1_MBIDXR154       FR_1.MB[154].IDXR.R           /* MB Index */
#define FR_1_MBCCSR155       FR_1.MB[155].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR155       FR_1.MB[155].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR155       FR_1.MB[155].FIDR.R           /* MB FID */
#define FR_1_MBIDXR155       FR_1.MB[155].IDXR.R           /* MB Index */
#define FR_1_MBCCSR156       FR_1.MB[156].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR156       FR_1.MB[156].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR156       FR_1.MB[156].FIDR.R           /* MB FID */
#define FR_1_MBIDXR156       FR_1.MB[156].IDXR.R           /* MB Index */
#define FR_1_MBCCSR157       FR_1.MB[157].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR157       FR_1.MB[157].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR157       FR_1.MB[157].FIDR.R           /* MB FID */
#define FR_1_MBIDXR157       FR_1.MB[157].IDXR.R           /* MB Index */
#define FR_1_MBCCSR158       FR_1.MB[158].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR158       FR_1.MB[158].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR158       FR_1.MB[158].FIDR.R           /* MB FID */
#define FR_1_MBIDXR158       FR_1.MB[158].IDXR.R           /* MB Index */
#define FR_1_MBCCSR159       FR_1.MB[159].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR159       FR_1.MB[159].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR159       FR_1.MB[159].FIDR.R           /* MB FID */
#define FR_1_MBIDXR159       FR_1.MB[159].IDXR.R           /* MB Index */
#define FR_1_MBCCSR160       FR_1.MB[160].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR160       FR_1.MB[160].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR160       FR_1.MB[160].FIDR.R           /* MB FID */
#define FR_1_MBIDXR160       FR_1.MB[160].IDXR.R           /* MB Index */
#define FR_1_MBCCSR161       FR_1.MB[161].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR161       FR_1.MB[161].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR161       FR_1.MB[161].FIDR.R           /* MB FID */
#define FR_1_MBIDXR161       FR_1.MB[161].IDXR.R           /* MB Index */
#define FR_1_MBCCSR162       FR_1.MB[162].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR162       FR_1.MB[162].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR162       FR_1.MB[162].FIDR.R           /* MB FID */
#define FR_1_MBIDXR162       FR_1.MB[162].IDXR.R           /* MB Index */
#define FR_1_MBCCSR163       FR_1.MB[163].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR163       FR_1.MB[163].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR163       FR_1.MB[163].FIDR.R           /* MB FID */
#define FR_1_MBIDXR163       FR_1.MB[163].IDXR.R           /* MB Index */
#define FR_1_MBCCSR164       FR_1.MB[164].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR164       FR_1.MB[164].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR164       FR_1.MB[164].FIDR.R           /* MB FID */
#define FR_1_MBIDXR164       FR_1.MB[164].IDXR.R           /* MB Index */
#define FR_1_MBCCSR165       FR_1.MB[165].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR165       FR_1.MB[165].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR165       FR_1.MB[165].FIDR.R           /* MB FID */
#define FR_1_MBIDXR165       FR_1.MB[165].IDXR.R           /* MB Index */
#define FR_1_MBCCSR166       FR_1.MB[166].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR166       FR_1.MB[166].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR166       FR_1.MB[166].FIDR.R           /* MB FID */
#define FR_1_MBIDXR166       FR_1.MB[166].IDXR.R           /* MB Index */
#define FR_1_MBCCSR167       FR_1.MB[167].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR167       FR_1.MB[167].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR167       FR_1.MB[167].FIDR.R           /* MB FID */
#define FR_1_MBIDXR167       FR_1.MB[167].IDXR.R           /* MB Index */
#define FR_1_MBCCSR168       FR_1.MB[168].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR168       FR_1.MB[168].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR168       FR_1.MB[168].FIDR.R           /* MB FID */
#define FR_1_MBIDXR168       FR_1.MB[168].IDXR.R           /* MB Index */
#define FR_1_MBCCSR169       FR_1.MB[169].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR169       FR_1.MB[169].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR169       FR_1.MB[169].FIDR.R           /* MB FID */
#define FR_1_MBIDXR169       FR_1.MB[169].IDXR.R           /* MB Index */
#define FR_1_MBCCSR170       FR_1.MB[170].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR170       FR_1.MB[170].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR170       FR_1.MB[170].FIDR.R           /* MB FID */
#define FR_1_MBIDXR170       FR_1.MB[170].IDXR.R           /* MB Index */
#define FR_1_MBCCSR171       FR_1.MB[171].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR171       FR_1.MB[171].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR171       FR_1.MB[171].FIDR.R           /* MB FID */
#define FR_1_MBIDXR171       FR_1.MB[171].IDXR.R           /* MB Index */
#define FR_1_MBCCSR172       FR_1.MB[172].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR172       FR_1.MB[172].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR172       FR_1.MB[172].FIDR.R           /* MB FID */
#define FR_1_MBIDXR172       FR_1.MB[172].IDXR.R           /* MB Index */
#define FR_1_MBCCSR173       FR_1.MB[173].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR173       FR_1.MB[173].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR173       FR_1.MB[173].FIDR.R           /* MB FID */
#define FR_1_MBIDXR173       FR_1.MB[173].IDXR.R           /* MB Index */
#define FR_1_MBCCSR174       FR_1.MB[174].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR174       FR_1.MB[174].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR174       FR_1.MB[174].FIDR.R           /* MB FID */
#define FR_1_MBIDXR174       FR_1.MB[174].IDXR.R           /* MB Index */
#define FR_1_MBCCSR175       FR_1.MB[175].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR175       FR_1.MB[175].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR175       FR_1.MB[175].FIDR.R           /* MB FID */
#define FR_1_MBIDXR175       FR_1.MB[175].IDXR.R           /* MB Index */
#define FR_1_MBCCSR176       FR_1.MB[176].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR176       FR_1.MB[176].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR176       FR_1.MB[176].FIDR.R           /* MB FID */
#define FR_1_MBIDXR176       FR_1.MB[176].IDXR.R           /* MB Index */
#define FR_1_MBCCSR177       FR_1.MB[177].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR177       FR_1.MB[177].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR177       FR_1.MB[177].FIDR.R           /* MB FID */
#define FR_1_MBIDXR177       FR_1.MB[177].IDXR.R           /* MB Index */
#define FR_1_MBCCSR178       FR_1.MB[178].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR178       FR_1.MB[178].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR178       FR_1.MB[178].FIDR.R           /* MB FID */
#define FR_1_MBIDXR178       FR_1.MB[178].IDXR.R           /* MB Index */
#define FR_1_MBCCSR179       FR_1.MB[179].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR179       FR_1.MB[179].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR179       FR_1.MB[179].FIDR.R           /* MB FID */
#define FR_1_MBIDXR179       FR_1.MB[179].IDXR.R           /* MB Index */
#define FR_1_MBCCSR180       FR_1.MB[180].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR180       FR_1.MB[180].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR180       FR_1.MB[180].FIDR.R           /* MB FID */
#define FR_1_MBIDXR180       FR_1.MB[180].IDXR.R           /* MB Index */
#define FR_1_MBCCSR181       FR_1.MB[181].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR181       FR_1.MB[181].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR181       FR_1.MB[181].FIDR.R           /* MB FID */
#define FR_1_MBIDXR181       FR_1.MB[181].IDXR.R           /* MB Index */
#define FR_1_MBCCSR182       FR_1.MB[182].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR182       FR_1.MB[182].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR182       FR_1.MB[182].FIDR.R           /* MB FID */
#define FR_1_MBIDXR182       FR_1.MB[182].IDXR.R           /* MB Index */
#define FR_1_MBCCSR183       FR_1.MB[183].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR183       FR_1.MB[183].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR183       FR_1.MB[183].FIDR.R           /* MB FID */
#define FR_1_MBIDXR183       FR_1.MB[183].IDXR.R           /* MB Index */
#define FR_1_MBCCSR184       FR_1.MB[184].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR184       FR_1.MB[184].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR184       FR_1.MB[184].FIDR.R           /* MB FID */
#define FR_1_MBIDXR184       FR_1.MB[184].IDXR.R           /* MB Index */
#define FR_1_MBCCSR185       FR_1.MB[185].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR185       FR_1.MB[185].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR185       FR_1.MB[185].FIDR.R           /* MB FID */
#define FR_1_MBIDXR185       FR_1.MB[185].IDXR.R           /* MB Index */
#define FR_1_MBCCSR186       FR_1.MB[186].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR186       FR_1.MB[186].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR186       FR_1.MB[186].FIDR.R           /* MB FID */
#define FR_1_MBIDXR186       FR_1.MB[186].IDXR.R           /* MB Index */
#define FR_1_MBCCSR187       FR_1.MB[187].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR187       FR_1.MB[187].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR187       FR_1.MB[187].FIDR.R           /* MB FID */
#define FR_1_MBIDXR187       FR_1.MB[187].IDXR.R           /* MB Index */
#define FR_1_MBCCSR188       FR_1.MB[188].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR188       FR_1.MB[188].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR188       FR_1.MB[188].FIDR.R           /* MB FID */
#define FR_1_MBIDXR188       FR_1.MB[188].IDXR.R           /* MB Index */
#define FR_1_MBCCSR189       FR_1.MB[189].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR189       FR_1.MB[189].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR189       FR_1.MB[189].FIDR.R           /* MB FID */
#define FR_1_MBIDXR189       FR_1.MB[189].IDXR.R           /* MB Index */
#define FR_1_MBCCSR190       FR_1.MB[190].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR190       FR_1.MB[190].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR190       FR_1.MB[190].FIDR.R           /* MB FID */
#define FR_1_MBIDXR190       FR_1.MB[190].IDXR.R           /* MB Index */
#define FR_1_MBCCSR191       FR_1.MB[191].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR191       FR_1.MB[191].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR191       FR_1.MB[191].FIDR.R           /* MB FID */
#define FR_1_MBIDXR191       FR_1.MB[191].IDXR.R           /* MB Index */
#define FR_1_MBCCSR192       FR_1.MB[192].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR192       FR_1.MB[192].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR192       FR_1.MB[192].FIDR.R           /* MB FID */
#define FR_1_MBIDXR192       FR_1.MB[192].IDXR.R           /* MB Index */
#define FR_1_MBCCSR193       FR_1.MB[193].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR193       FR_1.MB[193].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR193       FR_1.MB[193].FIDR.R           /* MB FID */
#define FR_1_MBIDXR193       FR_1.MB[193].IDXR.R           /* MB Index */
#define FR_1_MBCCSR194       FR_1.MB[194].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR194       FR_1.MB[194].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR194       FR_1.MB[194].FIDR.R           /* MB FID */
#define FR_1_MBIDXR194       FR_1.MB[194].IDXR.R           /* MB Index */
#define FR_1_MBCCSR195       FR_1.MB[195].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR195       FR_1.MB[195].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR195       FR_1.MB[195].FIDR.R           /* MB FID */
#define FR_1_MBIDXR195       FR_1.MB[195].IDXR.R           /* MB Index */
#define FR_1_MBCCSR196       FR_1.MB[196].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR196       FR_1.MB[196].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR196       FR_1.MB[196].FIDR.R           /* MB FID */
#define FR_1_MBIDXR196       FR_1.MB[196].IDXR.R           /* MB Index */
#define FR_1_MBCCSR197       FR_1.MB[197].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR197       FR_1.MB[197].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR197       FR_1.MB[197].FIDR.R           /* MB FID */
#define FR_1_MBIDXR197       FR_1.MB[197].IDXR.R           /* MB Index */
#define FR_1_MBCCSR198       FR_1.MB[198].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR198       FR_1.MB[198].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR198       FR_1.MB[198].FIDR.R           /* MB FID */
#define FR_1_MBIDXR198       FR_1.MB[198].IDXR.R           /* MB Index */
#define FR_1_MBCCSR199       FR_1.MB[199].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR199       FR_1.MB[199].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR199       FR_1.MB[199].FIDR.R           /* MB FID */
#define FR_1_MBIDXR199       FR_1.MB[199].IDXR.R           /* MB Index */
#define FR_1_MBCCSR200       FR_1.MB[200].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR200       FR_1.MB[200].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR200       FR_1.MB[200].FIDR.R           /* MB FID */
#define FR_1_MBIDXR200       FR_1.MB[200].IDXR.R           /* MB Index */
#define FR_1_MBCCSR201       FR_1.MB[201].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR201       FR_1.MB[201].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR201       FR_1.MB[201].FIDR.R           /* MB FID */
#define FR_1_MBIDXR201       FR_1.MB[201].IDXR.R           /* MB Index */
#define FR_1_MBCCSR202       FR_1.MB[202].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR202       FR_1.MB[202].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR202       FR_1.MB[202].FIDR.R           /* MB FID */
#define FR_1_MBIDXR202       FR_1.MB[202].IDXR.R           /* MB Index */
#define FR_1_MBCCSR203       FR_1.MB[203].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR203       FR_1.MB[203].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR203       FR_1.MB[203].FIDR.R           /* MB FID */
#define FR_1_MBIDXR203       FR_1.MB[203].IDXR.R           /* MB Index */
#define FR_1_MBCCSR204       FR_1.MB[204].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR204       FR_1.MB[204].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR204       FR_1.MB[204].FIDR.R           /* MB FID */
#define FR_1_MBIDXR204       FR_1.MB[204].IDXR.R           /* MB Index */
#define FR_1_MBCCSR205       FR_1.MB[205].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR205       FR_1.MB[205].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR205       FR_1.MB[205].FIDR.R           /* MB FID */
#define FR_1_MBIDXR205       FR_1.MB[205].IDXR.R           /* MB Index */
#define FR_1_MBCCSR206       FR_1.MB[206].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR206       FR_1.MB[206].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR206       FR_1.MB[206].FIDR.R           /* MB FID */
#define FR_1_MBIDXR206       FR_1.MB[206].IDXR.R           /* MB Index */
#define FR_1_MBCCSR207       FR_1.MB[207].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR207       FR_1.MB[207].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR207       FR_1.MB[207].FIDR.R           /* MB FID */
#define FR_1_MBIDXR207       FR_1.MB[207].IDXR.R           /* MB Index */
#define FR_1_MBCCSR208       FR_1.MB[208].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR208       FR_1.MB[208].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR208       FR_1.MB[208].FIDR.R           /* MB FID */
#define FR_1_MBIDXR208       FR_1.MB[208].IDXR.R           /* MB Index */
#define FR_1_MBCCSR209       FR_1.MB[209].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR209       FR_1.MB[209].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR209       FR_1.MB[209].FIDR.R           /* MB FID */
#define FR_1_MBIDXR209       FR_1.MB[209].IDXR.R           /* MB Index */
#define FR_1_MBCCSR210       FR_1.MB[210].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR210       FR_1.MB[210].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR210       FR_1.MB[210].FIDR.R           /* MB FID */
#define FR_1_MBIDXR210       FR_1.MB[210].IDXR.R           /* MB Index */
#define FR_1_MBCCSR211       FR_1.MB[211].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR211       FR_1.MB[211].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR211       FR_1.MB[211].FIDR.R           /* MB FID */
#define FR_1_MBIDXR211       FR_1.MB[211].IDXR.R           /* MB Index */
#define FR_1_MBCCSR212       FR_1.MB[212].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR212       FR_1.MB[212].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR212       FR_1.MB[212].FIDR.R           /* MB FID */
#define FR_1_MBIDXR212       FR_1.MB[212].IDXR.R           /* MB Index */
#define FR_1_MBCCSR213       FR_1.MB[213].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR213       FR_1.MB[213].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR213       FR_1.MB[213].FIDR.R           /* MB FID */
#define FR_1_MBIDXR213       FR_1.MB[213].IDXR.R           /* MB Index */
#define FR_1_MBCCSR214       FR_1.MB[214].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR214       FR_1.MB[214].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR214       FR_1.MB[214].FIDR.R           /* MB FID */
#define FR_1_MBIDXR214       FR_1.MB[214].IDXR.R           /* MB Index */
#define FR_1_MBCCSR215       FR_1.MB[215].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR215       FR_1.MB[215].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR215       FR_1.MB[215].FIDR.R           /* MB FID */
#define FR_1_MBIDXR215       FR_1.MB[215].IDXR.R           /* MB Index */
#define FR_1_MBCCSR216       FR_1.MB[216].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR216       FR_1.MB[216].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR216       FR_1.MB[216].FIDR.R           /* MB FID */
#define FR_1_MBIDXR216       FR_1.MB[216].IDXR.R           /* MB Index */
#define FR_1_MBCCSR217       FR_1.MB[217].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR217       FR_1.MB[217].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR217       FR_1.MB[217].FIDR.R           /* MB FID */
#define FR_1_MBIDXR217       FR_1.MB[217].IDXR.R           /* MB Index */
#define FR_1_MBCCSR218       FR_1.MB[218].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR218       FR_1.MB[218].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR218       FR_1.MB[218].FIDR.R           /* MB FID */
#define FR_1_MBIDXR218       FR_1.MB[218].IDXR.R           /* MB Index */
#define FR_1_MBCCSR219       FR_1.MB[219].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR219       FR_1.MB[219].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR219       FR_1.MB[219].FIDR.R           /* MB FID */
#define FR_1_MBIDXR219       FR_1.MB[219].IDXR.R           /* MB Index */
#define FR_1_MBCCSR220       FR_1.MB[220].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR220       FR_1.MB[220].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR220       FR_1.MB[220].FIDR.R           /* MB FID */
#define FR_1_MBIDXR220       FR_1.MB[220].IDXR.R           /* MB Index */
#define FR_1_MBCCSR221       FR_1.MB[221].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR221       FR_1.MB[221].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR221       FR_1.MB[221].FIDR.R           /* MB FID */
#define FR_1_MBIDXR221       FR_1.MB[221].IDXR.R           /* MB Index */
#define FR_1_MBCCSR222       FR_1.MB[222].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR222       FR_1.MB[222].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR222       FR_1.MB[222].FIDR.R           /* MB FID */
#define FR_1_MBIDXR222       FR_1.MB[222].IDXR.R           /* MB Index */
#define FR_1_MBCCSR223       FR_1.MB[223].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR223       FR_1.MB[223].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR223       FR_1.MB[223].FIDR.R           /* MB FID */
#define FR_1_MBIDXR223       FR_1.MB[223].IDXR.R           /* MB Index */
#define FR_1_MBCCSR224       FR_1.MB[224].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR224       FR_1.MB[224].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR224       FR_1.MB[224].FIDR.R           /* MB FID */
#define FR_1_MBIDXR224       FR_1.MB[224].IDXR.R           /* MB Index */
#define FR_1_MBCCSR225       FR_1.MB[225].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR225       FR_1.MB[225].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR225       FR_1.MB[225].FIDR.R           /* MB FID */
#define FR_1_MBIDXR225       FR_1.MB[225].IDXR.R           /* MB Index */
#define FR_1_MBCCSR226       FR_1.MB[226].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR226       FR_1.MB[226].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR226       FR_1.MB[226].FIDR.R           /* MB FID */
#define FR_1_MBIDXR226       FR_1.MB[226].IDXR.R           /* MB Index */
#define FR_1_MBCCSR227       FR_1.MB[227].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR227       FR_1.MB[227].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR227       FR_1.MB[227].FIDR.R           /* MB FID */
#define FR_1_MBIDXR227       FR_1.MB[227].IDXR.R           /* MB Index */
#define FR_1_MBCCSR228       FR_1.MB[228].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR228       FR_1.MB[228].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR228       FR_1.MB[228].FIDR.R           /* MB FID */
#define FR_1_MBIDXR228       FR_1.MB[228].IDXR.R           /* MB Index */
#define FR_1_MBCCSR229       FR_1.MB[229].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR229       FR_1.MB[229].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR229       FR_1.MB[229].FIDR.R           /* MB FID */
#define FR_1_MBIDXR229       FR_1.MB[229].IDXR.R           /* MB Index */
#define FR_1_MBCCSR230       FR_1.MB[230].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR230       FR_1.MB[230].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR230       FR_1.MB[230].FIDR.R           /* MB FID */
#define FR_1_MBIDXR230       FR_1.MB[230].IDXR.R           /* MB Index */
#define FR_1_MBCCSR231       FR_1.MB[231].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR231       FR_1.MB[231].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR231       FR_1.MB[231].FIDR.R           /* MB FID */
#define FR_1_MBIDXR231       FR_1.MB[231].IDXR.R           /* MB Index */
#define FR_1_MBCCSR232       FR_1.MB[232].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR232       FR_1.MB[232].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR232       FR_1.MB[232].FIDR.R           /* MB FID */
#define FR_1_MBIDXR232       FR_1.MB[232].IDXR.R           /* MB Index */
#define FR_1_MBCCSR233       FR_1.MB[233].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR233       FR_1.MB[233].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR233       FR_1.MB[233].FIDR.R           /* MB FID */
#define FR_1_MBIDXR233       FR_1.MB[233].IDXR.R           /* MB Index */
#define FR_1_MBCCSR234       FR_1.MB[234].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR234       FR_1.MB[234].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR234       FR_1.MB[234].FIDR.R           /* MB FID */
#define FR_1_MBIDXR234       FR_1.MB[234].IDXR.R           /* MB Index */
#define FR_1_MBCCSR235       FR_1.MB[235].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR235       FR_1.MB[235].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR235       FR_1.MB[235].FIDR.R           /* MB FID */
#define FR_1_MBIDXR235       FR_1.MB[235].IDXR.R           /* MB Index */
#define FR_1_MBCCSR236       FR_1.MB[236].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR236       FR_1.MB[236].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR236       FR_1.MB[236].FIDR.R           /* MB FID */
#define FR_1_MBIDXR236       FR_1.MB[236].IDXR.R           /* MB Index */
#define FR_1_MBCCSR237       FR_1.MB[237].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR237       FR_1.MB[237].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR237       FR_1.MB[237].FIDR.R           /* MB FID */
#define FR_1_MBIDXR237       FR_1.MB[237].IDXR.R           /* MB Index */
#define FR_1_MBCCSR238       FR_1.MB[238].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR238       FR_1.MB[238].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR238       FR_1.MB[238].FIDR.R           /* MB FID */
#define FR_1_MBIDXR238       FR_1.MB[238].IDXR.R           /* MB Index */
#define FR_1_MBCCSR239       FR_1.MB[239].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR239       FR_1.MB[239].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR239       FR_1.MB[239].FIDR.R           /* MB FID */
#define FR_1_MBIDXR239       FR_1.MB[239].IDXR.R           /* MB Index */
#define FR_1_MBCCSR240       FR_1.MB[240].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR240       FR_1.MB[240].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR240       FR_1.MB[240].FIDR.R           /* MB FID */
#define FR_1_MBIDXR240       FR_1.MB[240].IDXR.R           /* MB Index */
#define FR_1_MBCCSR241       FR_1.MB[241].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR241       FR_1.MB[241].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR241       FR_1.MB[241].FIDR.R           /* MB FID */
#define FR_1_MBIDXR241       FR_1.MB[241].IDXR.R           /* MB Index */
#define FR_1_MBCCSR242       FR_1.MB[242].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR242       FR_1.MB[242].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR242       FR_1.MB[242].FIDR.R           /* MB FID */
#define FR_1_MBIDXR242       FR_1.MB[242].IDXR.R           /* MB Index */
#define FR_1_MBCCSR243       FR_1.MB[243].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR243       FR_1.MB[243].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR243       FR_1.MB[243].FIDR.R           /* MB FID */
#define FR_1_MBIDXR243       FR_1.MB[243].IDXR.R           /* MB Index */
#define FR_1_MBCCSR244       FR_1.MB[244].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR244       FR_1.MB[244].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR244       FR_1.MB[244].FIDR.R           /* MB FID */
#define FR_1_MBIDXR244       FR_1.MB[244].IDXR.R           /* MB Index */
#define FR_1_MBCCSR245       FR_1.MB[245].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR245       FR_1.MB[245].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR245       FR_1.MB[245].FIDR.R           /* MB FID */
#define FR_1_MBIDXR245       FR_1.MB[245].IDXR.R           /* MB Index */
#define FR_1_MBCCSR246       FR_1.MB[246].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR246       FR_1.MB[246].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR246       FR_1.MB[246].FIDR.R           /* MB FID */
#define FR_1_MBIDXR246       FR_1.MB[246].IDXR.R           /* MB Index */
#define FR_1_MBCCSR247       FR_1.MB[247].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR247       FR_1.MB[247].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR247       FR_1.MB[247].FIDR.R           /* MB FID */
#define FR_1_MBIDXR247       FR_1.MB[247].IDXR.R           /* MB Index */
#define FR_1_MBCCSR248       FR_1.MB[248].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR248       FR_1.MB[248].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR248       FR_1.MB[248].FIDR.R           /* MB FID */
#define FR_1_MBIDXR248       FR_1.MB[248].IDXR.R           /* MB Index */
#define FR_1_MBCCSR249       FR_1.MB[249].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR249       FR_1.MB[249].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR249       FR_1.MB[249].FIDR.R           /* MB FID */
#define FR_1_MBIDXR249       FR_1.MB[249].IDXR.R           /* MB Index */
#define FR_1_MBCCSR250       FR_1.MB[250].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR250       FR_1.MB[250].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR250       FR_1.MB[250].FIDR.R           /* MB FID */
#define FR_1_MBIDXR250       FR_1.MB[250].IDXR.R           /* MB Index */
#define FR_1_MBCCSR251       FR_1.MB[251].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR251       FR_1.MB[251].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR251       FR_1.MB[251].FIDR.R           /* MB FID */
#define FR_1_MBIDXR251       FR_1.MB[251].IDXR.R           /* MB Index */
#define FR_1_MBCCSR252       FR_1.MB[252].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR252       FR_1.MB[252].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR252       FR_1.MB[252].FIDR.R           /* MB FID */
#define FR_1_MBIDXR252       FR_1.MB[252].IDXR.R           /* MB Index */
#define FR_1_MBCCSR253       FR_1.MB[253].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR253       FR_1.MB[253].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR253       FR_1.MB[253].FIDR.R           /* MB FID */
#define FR_1_MBIDXR253       FR_1.MB[253].IDXR.R           /* MB Index */
#define FR_1_MBCCSR254       FR_1.MB[254].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR254       FR_1.MB[254].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR254       FR_1.MB[254].FIDR.R           /* MB FID */
#define FR_1_MBIDXR254       FR_1.MB[254].IDXR.R           /* MB Index */
#define FR_1_MBCCSR255       FR_1.MB[255].CCSR.R           /* MB Configuration Control Status */
#define FR_1_MBCCFR255       FR_1.MB[255].CCFR.R           /* MB Cycle Counter Filter */
#define FR_1_MBFIDR255       FR_1.MB[255].FIDR.R           /* MB FID */
#define FR_1_MBIDXR255       FR_1.MB[255].IDXR.R           /* MB Index */
#define FR_1_MBDOR0          FR_1.MBDOR[0].R               /* MB Data Field Offset */
#define FR_1_MBDOR1          FR_1.MBDOR[1].R               /* MB Data Field Offset */
#define FR_1_MBDOR2          FR_1.MBDOR[2].R               /* MB Data Field Offset */
#define FR_1_MBDOR3          FR_1.MBDOR[3].R               /* MB Data Field Offset */
#define FR_1_MBDOR4          FR_1.MBDOR[4].R               /* MB Data Field Offset */
#define FR_1_MBDOR5          FR_1.MBDOR[5].R               /* MB Data Field Offset */
#define FR_1_MBDOR6          FR_1.MBDOR[6].R               /* MB Data Field Offset */
#define FR_1_MBDOR7          FR_1.MBDOR[7].R               /* MB Data Field Offset */
#define FR_1_MBDOR8          FR_1.MBDOR[8].R               /* MB Data Field Offset */
#define FR_1_MBDOR9          FR_1.MBDOR[9].R               /* MB Data Field Offset */
#define FR_1_MBDOR10         FR_1.MBDOR[10].R              /* MB Data Field Offset */
#define FR_1_MBDOR11         FR_1.MBDOR[11].R              /* MB Data Field Offset */
#define FR_1_MBDOR12         FR_1.MBDOR[12].R              /* MB Data Field Offset */
#define FR_1_MBDOR13         FR_1.MBDOR[13].R              /* MB Data Field Offset */
#define FR_1_MBDOR14         FR_1.MBDOR[14].R              /* MB Data Field Offset */
#define FR_1_MBDOR15         FR_1.MBDOR[15].R              /* MB Data Field Offset */
#define FR_1_MBDOR16         FR_1.MBDOR[16].R              /* MB Data Field Offset */
#define FR_1_MBDOR17         FR_1.MBDOR[17].R              /* MB Data Field Offset */
#define FR_1_MBDOR18         FR_1.MBDOR[18].R              /* MB Data Field Offset */
#define FR_1_MBDOR19         FR_1.MBDOR[19].R              /* MB Data Field Offset */
#define FR_1_MBDOR20         FR_1.MBDOR[20].R              /* MB Data Field Offset */
#define FR_1_MBDOR21         FR_1.MBDOR[21].R              /* MB Data Field Offset */
#define FR_1_MBDOR22         FR_1.MBDOR[22].R              /* MB Data Field Offset */
#define FR_1_MBDOR23         FR_1.MBDOR[23].R              /* MB Data Field Offset */
#define FR_1_MBDOR24         FR_1.MBDOR[24].R              /* MB Data Field Offset */
#define FR_1_MBDOR25         FR_1.MBDOR[25].R              /* MB Data Field Offset */
#define FR_1_MBDOR26         FR_1.MBDOR[26].R              /* MB Data Field Offset */
#define FR_1_MBDOR27         FR_1.MBDOR[27].R              /* MB Data Field Offset */
#define FR_1_MBDOR28         FR_1.MBDOR[28].R              /* MB Data Field Offset */
#define FR_1_MBDOR29         FR_1.MBDOR[29].R              /* MB Data Field Offset */
#define FR_1_MBDOR30         FR_1.MBDOR[30].R              /* MB Data Field Offset */
#define FR_1_MBDOR31         FR_1.MBDOR[31].R              /* MB Data Field Offset */
#define FR_1_MBDOR32         FR_1.MBDOR[32].R              /* MB Data Field Offset */
#define FR_1_MBDOR33         FR_1.MBDOR[33].R              /* MB Data Field Offset */
#define FR_1_MBDOR34         FR_1.MBDOR[34].R              /* MB Data Field Offset */
#define FR_1_MBDOR35         FR_1.MBDOR[35].R              /* MB Data Field Offset */
#define FR_1_MBDOR36         FR_1.MBDOR[36].R              /* MB Data Field Offset */
#define FR_1_MBDOR37         FR_1.MBDOR[37].R              /* MB Data Field Offset */
#define FR_1_MBDOR38         FR_1.MBDOR[38].R              /* MB Data Field Offset */
#define FR_1_MBDOR39         FR_1.MBDOR[39].R              /* MB Data Field Offset */
#define FR_1_MBDOR40         FR_1.MBDOR[40].R              /* MB Data Field Offset */
#define FR_1_MBDOR41         FR_1.MBDOR[41].R              /* MB Data Field Offset */
#define FR_1_MBDOR42         FR_1.MBDOR[42].R              /* MB Data Field Offset */
#define FR_1_MBDOR43         FR_1.MBDOR[43].R              /* MB Data Field Offset */
#define FR_1_MBDOR44         FR_1.MBDOR[44].R              /* MB Data Field Offset */
#define FR_1_MBDOR45         FR_1.MBDOR[45].R              /* MB Data Field Offset */
#define FR_1_MBDOR46         FR_1.MBDOR[46].R              /* MB Data Field Offset */
#define FR_1_MBDOR47         FR_1.MBDOR[47].R              /* MB Data Field Offset */
#define FR_1_MBDOR48         FR_1.MBDOR[48].R              /* MB Data Field Offset */
#define FR_1_MBDOR49         FR_1.MBDOR[49].R              /* MB Data Field Offset */
#define FR_1_MBDOR50         FR_1.MBDOR[50].R              /* MB Data Field Offset */
#define FR_1_MBDOR51         FR_1.MBDOR[51].R              /* MB Data Field Offset */
#define FR_1_MBDOR52         FR_1.MBDOR[52].R              /* MB Data Field Offset */
#define FR_1_MBDOR53         FR_1.MBDOR[53].R              /* MB Data Field Offset */
#define FR_1_MBDOR54         FR_1.MBDOR[54].R              /* MB Data Field Offset */
#define FR_1_MBDOR55         FR_1.MBDOR[55].R              /* MB Data Field Offset */
#define FR_1_MBDOR56         FR_1.MBDOR[56].R              /* MB Data Field Offset */
#define FR_1_MBDOR57         FR_1.MBDOR[57].R              /* MB Data Field Offset */
#define FR_1_MBDOR58         FR_1.MBDOR[58].R              /* MB Data Field Offset */
#define FR_1_MBDOR59         FR_1.MBDOR[59].R              /* MB Data Field Offset */
#define FR_1_MBDOR60         FR_1.MBDOR[60].R              /* MB Data Field Offset */
#define FR_1_MBDOR61         FR_1.MBDOR[61].R              /* MB Data Field Offset */
#define FR_1_MBDOR62         FR_1.MBDOR[62].R              /* MB Data Field Offset */
#define FR_1_MBDOR63         FR_1.MBDOR[63].R              /* MB Data Field Offset */
#define FR_1_MBDOR64         FR_1.MBDOR[64].R              /* MB Data Field Offset */
#define FR_1_MBDOR65         FR_1.MBDOR[65].R              /* MB Data Field Offset */
#define FR_1_MBDOR66         FR_1.MBDOR[66].R              /* MB Data Field Offset */
#define FR_1_MBDOR67         FR_1.MBDOR[67].R              /* MB Data Field Offset */
#define FR_1_MBDOR68         FR_1.MBDOR[68].R              /* MB Data Field Offset */
#define FR_1_MBDOR69         FR_1.MBDOR[69].R              /* MB Data Field Offset */
#define FR_1_MBDOR70         FR_1.MBDOR[70].R              /* MB Data Field Offset */
#define FR_1_MBDOR71         FR_1.MBDOR[71].R              /* MB Data Field Offset */
#define FR_1_MBDOR72         FR_1.MBDOR[72].R              /* MB Data Field Offset */
#define FR_1_MBDOR73         FR_1.MBDOR[73].R              /* MB Data Field Offset */
#define FR_1_MBDOR74         FR_1.MBDOR[74].R              /* MB Data Field Offset */
#define FR_1_MBDOR75         FR_1.MBDOR[75].R              /* MB Data Field Offset */
#define FR_1_MBDOR76         FR_1.MBDOR[76].R              /* MB Data Field Offset */
#define FR_1_MBDOR77         FR_1.MBDOR[77].R              /* MB Data Field Offset */
#define FR_1_MBDOR78         FR_1.MBDOR[78].R              /* MB Data Field Offset */
#define FR_1_MBDOR79         FR_1.MBDOR[79].R              /* MB Data Field Offset */
#define FR_1_MBDOR80         FR_1.MBDOR[80].R              /* MB Data Field Offset */
#define FR_1_MBDOR81         FR_1.MBDOR[81].R              /* MB Data Field Offset */
#define FR_1_MBDOR82         FR_1.MBDOR[82].R              /* MB Data Field Offset */
#define FR_1_MBDOR83         FR_1.MBDOR[83].R              /* MB Data Field Offset */
#define FR_1_MBDOR84         FR_1.MBDOR[84].R              /* MB Data Field Offset */
#define FR_1_MBDOR85         FR_1.MBDOR[85].R              /* MB Data Field Offset */
#define FR_1_MBDOR86         FR_1.MBDOR[86].R              /* MB Data Field Offset */
#define FR_1_MBDOR87         FR_1.MBDOR[87].R              /* MB Data Field Offset */
#define FR_1_MBDOR88         FR_1.MBDOR[88].R              /* MB Data Field Offset */
#define FR_1_MBDOR89         FR_1.MBDOR[89].R              /* MB Data Field Offset */
#define FR_1_MBDOR90         FR_1.MBDOR[90].R              /* MB Data Field Offset */
#define FR_1_MBDOR91         FR_1.MBDOR[91].R              /* MB Data Field Offset */
#define FR_1_MBDOR92         FR_1.MBDOR[92].R              /* MB Data Field Offset */
#define FR_1_MBDOR93         FR_1.MBDOR[93].R              /* MB Data Field Offset */
#define FR_1_MBDOR94         FR_1.MBDOR[94].R              /* MB Data Field Offset */
#define FR_1_MBDOR95         FR_1.MBDOR[95].R              /* MB Data Field Offset */
#define FR_1_MBDOR96         FR_1.MBDOR[96].R              /* MB Data Field Offset */
#define FR_1_MBDOR97         FR_1.MBDOR[97].R              /* MB Data Field Offset */
#define FR_1_MBDOR98         FR_1.MBDOR[98].R              /* MB Data Field Offset */
#define FR_1_MBDOR99         FR_1.MBDOR[99].R              /* MB Data Field Offset */
#define FR_1_MBDOR100        FR_1.MBDOR[100].R             /* MB Data Field Offset */
#define FR_1_MBDOR101        FR_1.MBDOR[101].R             /* MB Data Field Offset */
#define FR_1_MBDOR102        FR_1.MBDOR[102].R             /* MB Data Field Offset */
#define FR_1_MBDOR103        FR_1.MBDOR[103].R             /* MB Data Field Offset */
#define FR_1_MBDOR104        FR_1.MBDOR[104].R             /* MB Data Field Offset */
#define FR_1_MBDOR105        FR_1.MBDOR[105].R             /* MB Data Field Offset */
#define FR_1_MBDOR106        FR_1.MBDOR[106].R             /* MB Data Field Offset */
#define FR_1_MBDOR107        FR_1.MBDOR[107].R             /* MB Data Field Offset */
#define FR_1_MBDOR108        FR_1.MBDOR[108].R             /* MB Data Field Offset */
#define FR_1_MBDOR109        FR_1.MBDOR[109].R             /* MB Data Field Offset */
#define FR_1_MBDOR110        FR_1.MBDOR[110].R             /* MB Data Field Offset */
#define FR_1_MBDOR111        FR_1.MBDOR[111].R             /* MB Data Field Offset */
#define FR_1_MBDOR112        FR_1.MBDOR[112].R             /* MB Data Field Offset */
#define FR_1_MBDOR113        FR_1.MBDOR[113].R             /* MB Data Field Offset */
#define FR_1_MBDOR114        FR_1.MBDOR[114].R             /* MB Data Field Offset */
#define FR_1_MBDOR115        FR_1.MBDOR[115].R             /* MB Data Field Offset */
#define FR_1_MBDOR116        FR_1.MBDOR[116].R             /* MB Data Field Offset */
#define FR_1_MBDOR117        FR_1.MBDOR[117].R             /* MB Data Field Offset */
#define FR_1_MBDOR118        FR_1.MBDOR[118].R             /* MB Data Field Offset */
#define FR_1_MBDOR119        FR_1.MBDOR[119].R             /* MB Data Field Offset */
#define FR_1_MBDOR120        FR_1.MBDOR[120].R             /* MB Data Field Offset */
#define FR_1_MBDOR121        FR_1.MBDOR[121].R             /* MB Data Field Offset */
#define FR_1_MBDOR122        FR_1.MBDOR[122].R             /* MB Data Field Offset */
#define FR_1_MBDOR123        FR_1.MBDOR[123].R             /* MB Data Field Offset */
#define FR_1_MBDOR124        FR_1.MBDOR[124].R             /* MB Data Field Offset */
#define FR_1_MBDOR125        FR_1.MBDOR[125].R             /* MB Data Field Offset */
#define FR_1_MBDOR126        FR_1.MBDOR[126].R             /* MB Data Field Offset */
#define FR_1_MBDOR127        FR_1.MBDOR[127].R             /* MB Data Field Offset */
#define FR_1_MBDOR128        FR_1.MBDOR[128].R             /* MB Data Field Offset */
#define FR_1_MBDOR129        FR_1.MBDOR[129].R             /* MB Data Field Offset */
#define FR_1_MBDOR130        FR_1.MBDOR[130].R             /* MB Data Field Offset */
#define FR_1_MBDOR131        FR_1.MBDOR[131].R             /* MB Data Field Offset */
#define FR_1_MBDOR132        FR_1.MBDOR[132].R             /* MB Data Field Offset */
#define FR_1_MBDOR133        FR_1.MBDOR[133].R             /* MB Data Field Offset */
#define FR_1_MBDOR134        FR_1.MBDOR[134].R             /* MB Data Field Offset */
#define FR_1_MBDOR135        FR_1.MBDOR[135].R             /* MB Data Field Offset */
#define FR_1_MBDOR136        FR_1.MBDOR[136].R             /* MB Data Field Offset */
#define FR_1_MBDOR137        FR_1.MBDOR[137].R             /* MB Data Field Offset */
#define FR_1_MBDOR138        FR_1.MBDOR[138].R             /* MB Data Field Offset */
#define FR_1_MBDOR139        FR_1.MBDOR[139].R             /* MB Data Field Offset */
#define FR_1_MBDOR140        FR_1.MBDOR[140].R             /* MB Data Field Offset */
#define FR_1_MBDOR141        FR_1.MBDOR[141].R             /* MB Data Field Offset */
#define FR_1_MBDOR142        FR_1.MBDOR[142].R             /* MB Data Field Offset */
#define FR_1_MBDOR143        FR_1.MBDOR[143].R             /* MB Data Field Offset */
#define FR_1_MBDOR144        FR_1.MBDOR[144].R             /* MB Data Field Offset */
#define FR_1_MBDOR145        FR_1.MBDOR[145].R             /* MB Data Field Offset */
#define FR_1_MBDOR146        FR_1.MBDOR[146].R             /* MB Data Field Offset */
#define FR_1_MBDOR147        FR_1.MBDOR[147].R             /* MB Data Field Offset */
#define FR_1_MBDOR148        FR_1.MBDOR[148].R             /* MB Data Field Offset */
#define FR_1_MBDOR149        FR_1.MBDOR[149].R             /* MB Data Field Offset */
#define FR_1_MBDOR150        FR_1.MBDOR[150].R             /* MB Data Field Offset */
#define FR_1_MBDOR151        FR_1.MBDOR[151].R             /* MB Data Field Offset */
#define FR_1_MBDOR152        FR_1.MBDOR[152].R             /* MB Data Field Offset */
#define FR_1_MBDOR153        FR_1.MBDOR[153].R             /* MB Data Field Offset */
#define FR_1_MBDOR154        FR_1.MBDOR[154].R             /* MB Data Field Offset */
#define FR_1_MBDOR155        FR_1.MBDOR[155].R             /* MB Data Field Offset */
#define FR_1_MBDOR156        FR_1.MBDOR[156].R             /* MB Data Field Offset */
#define FR_1_MBDOR157        FR_1.MBDOR[157].R             /* MB Data Field Offset */
#define FR_1_MBDOR158        FR_1.MBDOR[158].R             /* MB Data Field Offset */
#define FR_1_MBDOR159        FR_1.MBDOR[159].R             /* MB Data Field Offset */
#define FR_1_MBDOR160        FR_1.MBDOR[160].R             /* MB Data Field Offset */
#define FR_1_MBDOR161        FR_1.MBDOR[161].R             /* MB Data Field Offset */
#define FR_1_MBDOR162        FR_1.MBDOR[162].R             /* MB Data Field Offset */
#define FR_1_MBDOR163        FR_1.MBDOR[163].R             /* MB Data Field Offset */
#define FR_1_MBDOR164        FR_1.MBDOR[164].R             /* MB Data Field Offset */
#define FR_1_MBDOR165        FR_1.MBDOR[165].R             /* MB Data Field Offset */
#define FR_1_MBDOR166        FR_1.MBDOR[166].R             /* MB Data Field Offset */
#define FR_1_MBDOR167        FR_1.MBDOR[167].R             /* MB Data Field Offset */
#define FR_1_MBDOR168        FR_1.MBDOR[168].R             /* MB Data Field Offset */
#define FR_1_MBDOR169        FR_1.MBDOR[169].R             /* MB Data Field Offset */
#define FR_1_MBDOR170        FR_1.MBDOR[170].R             /* MB Data Field Offset */
#define FR_1_MBDOR171        FR_1.MBDOR[171].R             /* MB Data Field Offset */
#define FR_1_MBDOR172        FR_1.MBDOR[172].R             /* MB Data Field Offset */
#define FR_1_MBDOR173        FR_1.MBDOR[173].R             /* MB Data Field Offset */
#define FR_1_MBDOR174        FR_1.MBDOR[174].R             /* MB Data Field Offset */
#define FR_1_MBDOR175        FR_1.MBDOR[175].R             /* MB Data Field Offset */
#define FR_1_MBDOR176        FR_1.MBDOR[176].R             /* MB Data Field Offset */
#define FR_1_MBDOR177        FR_1.MBDOR[177].R             /* MB Data Field Offset */
#define FR_1_MBDOR178        FR_1.MBDOR[178].R             /* MB Data Field Offset */
#define FR_1_MBDOR179        FR_1.MBDOR[179].R             /* MB Data Field Offset */
#define FR_1_MBDOR180        FR_1.MBDOR[180].R             /* MB Data Field Offset */
#define FR_1_MBDOR181        FR_1.MBDOR[181].R             /* MB Data Field Offset */
#define FR_1_MBDOR182        FR_1.MBDOR[182].R             /* MB Data Field Offset */
#define FR_1_MBDOR183        FR_1.MBDOR[183].R             /* MB Data Field Offset */
#define FR_1_MBDOR184        FR_1.MBDOR[184].R             /* MB Data Field Offset */
#define FR_1_MBDOR185        FR_1.MBDOR[185].R             /* MB Data Field Offset */
#define FR_1_MBDOR186        FR_1.MBDOR[186].R             /* MB Data Field Offset */
#define FR_1_MBDOR187        FR_1.MBDOR[187].R             /* MB Data Field Offset */
#define FR_1_MBDOR188        FR_1.MBDOR[188].R             /* MB Data Field Offset */
#define FR_1_MBDOR189        FR_1.MBDOR[189].R             /* MB Data Field Offset */
#define FR_1_MBDOR190        FR_1.MBDOR[190].R             /* MB Data Field Offset */
#define FR_1_MBDOR191        FR_1.MBDOR[191].R             /* MB Data Field Offset */
#define FR_1_MBDOR192        FR_1.MBDOR[192].R             /* MB Data Field Offset */
#define FR_1_MBDOR193        FR_1.MBDOR[193].R             /* MB Data Field Offset */
#define FR_1_MBDOR194        FR_1.MBDOR[194].R             /* MB Data Field Offset */
#define FR_1_MBDOR195        FR_1.MBDOR[195].R             /* MB Data Field Offset */
#define FR_1_MBDOR196        FR_1.MBDOR[196].R             /* MB Data Field Offset */
#define FR_1_MBDOR197        FR_1.MBDOR[197].R             /* MB Data Field Offset */
#define FR_1_MBDOR198        FR_1.MBDOR[198].R             /* MB Data Field Offset */
#define FR_1_MBDOR199        FR_1.MBDOR[199].R             /* MB Data Field Offset */
#define FR_1_MBDOR200        FR_1.MBDOR[200].R             /* MB Data Field Offset */
#define FR_1_MBDOR201        FR_1.MBDOR[201].R             /* MB Data Field Offset */
#define FR_1_MBDOR202        FR_1.MBDOR[202].R             /* MB Data Field Offset */
#define FR_1_MBDOR203        FR_1.MBDOR[203].R             /* MB Data Field Offset */
#define FR_1_MBDOR204        FR_1.MBDOR[204].R             /* MB Data Field Offset */
#define FR_1_MBDOR205        FR_1.MBDOR[205].R             /* MB Data Field Offset */
#define FR_1_MBDOR206        FR_1.MBDOR[206].R             /* MB Data Field Offset */
#define FR_1_MBDOR207        FR_1.MBDOR[207].R             /* MB Data Field Offset */
#define FR_1_MBDOR208        FR_1.MBDOR[208].R             /* MB Data Field Offset */
#define FR_1_MBDOR209        FR_1.MBDOR[209].R             /* MB Data Field Offset */
#define FR_1_MBDOR210        FR_1.MBDOR[210].R             /* MB Data Field Offset */
#define FR_1_MBDOR211        FR_1.MBDOR[211].R             /* MB Data Field Offset */
#define FR_1_MBDOR212        FR_1.MBDOR[212].R             /* MB Data Field Offset */
#define FR_1_MBDOR213        FR_1.MBDOR[213].R             /* MB Data Field Offset */
#define FR_1_MBDOR214        FR_1.MBDOR[214].R             /* MB Data Field Offset */
#define FR_1_MBDOR215        FR_1.MBDOR[215].R             /* MB Data Field Offset */
#define FR_1_MBDOR216        FR_1.MBDOR[216].R             /* MB Data Field Offset */
#define FR_1_MBDOR217        FR_1.MBDOR[217].R             /* MB Data Field Offset */
#define FR_1_MBDOR218        FR_1.MBDOR[218].R             /* MB Data Field Offset */
#define FR_1_MBDOR219        FR_1.MBDOR[219].R             /* MB Data Field Offset */
#define FR_1_MBDOR220        FR_1.MBDOR[220].R             /* MB Data Field Offset */
#define FR_1_MBDOR221        FR_1.MBDOR[221].R             /* MB Data Field Offset */
#define FR_1_MBDOR222        FR_1.MBDOR[222].R             /* MB Data Field Offset */
#define FR_1_MBDOR223        FR_1.MBDOR[223].R             /* MB Data Field Offset */
#define FR_1_MBDOR224        FR_1.MBDOR[224].R             /* MB Data Field Offset */
#define FR_1_MBDOR225        FR_1.MBDOR[225].R             /* MB Data Field Offset */
#define FR_1_MBDOR226        FR_1.MBDOR[226].R             /* MB Data Field Offset */
#define FR_1_MBDOR227        FR_1.MBDOR[227].R             /* MB Data Field Offset */
#define FR_1_MBDOR228        FR_1.MBDOR[228].R             /* MB Data Field Offset */
#define FR_1_MBDOR229        FR_1.MBDOR[229].R             /* MB Data Field Offset */
#define FR_1_MBDOR230        FR_1.MBDOR[230].R             /* MB Data Field Offset */
#define FR_1_MBDOR231        FR_1.MBDOR[231].R             /* MB Data Field Offset */
#define FR_1_MBDOR232        FR_1.MBDOR[232].R             /* MB Data Field Offset */
#define FR_1_MBDOR233        FR_1.MBDOR[233].R             /* MB Data Field Offset */
#define FR_1_MBDOR234        FR_1.MBDOR[234].R             /* MB Data Field Offset */
#define FR_1_MBDOR235        FR_1.MBDOR[235].R             /* MB Data Field Offset */
#define FR_1_MBDOR236        FR_1.MBDOR[236].R             /* MB Data Field Offset */
#define FR_1_MBDOR237        FR_1.MBDOR[237].R             /* MB Data Field Offset */
#define FR_1_MBDOR238        FR_1.MBDOR[238].R             /* MB Data Field Offset */
#define FR_1_MBDOR239        FR_1.MBDOR[239].R             /* MB Data Field Offset */
#define FR_1_MBDOR240        FR_1.MBDOR[240].R             /* MB Data Field Offset */
#define FR_1_MBDOR241        FR_1.MBDOR[241].R             /* MB Data Field Offset */
#define FR_1_MBDOR242        FR_1.MBDOR[242].R             /* MB Data Field Offset */
#define FR_1_MBDOR243        FR_1.MBDOR[243].R             /* MB Data Field Offset */
#define FR_1_MBDOR244        FR_1.MBDOR[244].R             /* MB Data Field Offset */
#define FR_1_MBDOR245        FR_1.MBDOR[245].R             /* MB Data Field Offset */
#define FR_1_MBDOR246        FR_1.MBDOR[246].R             /* MB Data Field Offset */
#define FR_1_MBDOR247        FR_1.MBDOR[247].R             /* MB Data Field Offset */
#define FR_1_MBDOR248        FR_1.MBDOR[248].R             /* MB Data Field Offset */
#define FR_1_MBDOR249        FR_1.MBDOR[249].R             /* MB Data Field Offset */
#define FR_1_MBDOR250        FR_1.MBDOR[250].R             /* MB Data Field Offset */
#define FR_1_MBDOR251        FR_1.MBDOR[251].R             /* MB Data Field Offset */
#define FR_1_MBDOR252        FR_1.MBDOR[252].R             /* MB Data Field Offset */
#define FR_1_MBDOR253        FR_1.MBDOR[253].R             /* MB Data Field Offset */
#define FR_1_MBDOR254        FR_1.MBDOR[254].R             /* MB Data Field Offset */
#define FR_1_MBDOR255        FR_1.MBDOR[255].R             /* MB Data Field Offset */
#define FR_1_MBDOR256        FR_1.MBDOR[256].R             /* MB Data Field Offset */
#define FR_1_MBDOR257        FR_1.MBDOR[257].R             /* MB Data Field Offset */
#define FR_1_MBDOR258        FR_1.MBDOR[258].R             /* MB Data Field Offset */
#define FR_1_MBDOR259        FR_1.MBDOR[259].R             /* MB Data Field Offset */
#define FR_1_LEETR0          FR_1.LEETR[0].R               /* LRAM ECC Error Test Register */
#define FR_1_LEETR1          FR_1.LEETR[1].R               /* LRAM ECC Error Test Register */
#define FR_1_LEETR2          FR_1.LEETR[2].R               /* LRAM ECC Error Test Register */
#define FR_1_LEETR3          FR_1.LEETR[3].R               /* LRAM ECC Error Test Register */
#define FR_1_LEETR4          FR_1.LEETR[4].R               /* LRAM ECC Error Test Register */
#define FR_1_LEETR5          FR_1.LEETR[5].R               /* LRAM ECC Error Test Register */

/* FXOSC */
#define FXOSC_CTRL           FXOSC.CTRL.R                  /* FXOSC Control Register */
#define FXOSC_STAT           FXOSC.STAT.R                  /* Oscillator Status Register */

/* GPR0 */
#define GPR0_LVFCCUGD0       GPR0.LVFCCUGD0.R              /* VFCCU Global DID register 0 */
#define GPR0_LVFCCULD0       GPR0.LVFCCULD0.R              /* VFCCU Local DID register 0 */
#define GPR0_LVFCCULD1       GPR0.LVFCCULD1.R              /* VFCCU Local DID register 1 */
#define GPR0_LVFCCULD2       GPR0.LVFCCULD2.R              /* VFCCU Local DID register 2 */
#define GPR0_FROSTICKC       GPR0.FROSTICKC.R              /* FlexRay OS Tick Mux Selector */
#define GPR0_FUNCC0          GPR0.FUNCC0.R                 /* Source Control for SINC HTRIGs */
#define GPR0_INITM0          GPR0.INITM0.R                 /* NoC Initiator NIU Timeout Status */
#define GPR0_TARGTMS0        GPR0.TARGTMS0.R               /* NoC Target NIU Timeout Status */
#define GPR0_TARGTMC0        GPR0.TARGTMC0.R               /* NoC Target NIU Timeout Control */
#define GPR0_CLKOUT0SEL      GPR0.CLKOUT0SEL.R             /* CLKOUT_0 MUX select */

/* GPR0_PCTL */
#define GPR0_PCTL_DSPI0PCTL  GPR0_PCTL.DSPI0PCTL.R         /* SPI_0 Clock Control Enable */
#define GPR0_PCTL_FR0PCTL    GPR0_PCTL.FR0PCTL.R           /* FlexRay_0 Clock Control Enable */
#define GPR0_PCTL_FR1PCTL    GPR0_PCTL.FR1PCTL.R           /* FlexRay_1 Clock Control Enable */
#define GPR0_PCTL_I3C0PCTL   GPR0_PCTL.I3C0PCTL.R          /* I3C_0 Clock Control Enable */
#define GPR0_PCTL_EDMA0PCTL  GPR0_PCTL.EDMA0PCTL.R         /* eDMA_0 Clock Control Enable */
#define GPR0_PCTL_MSCDSPIPCTL GPR0_PCTL.MSCDSPIPCTL.R      /* MSC_0_DSPI Clock Control Enable */
#define GPR0_PCTL_DSPI1PCTL  GPR0_PCTL.DSPI1PCTL.R         /* SPI_1 Clock Control Enable */
#define GPR0_PCTL_LIN0PCTL   GPR0_PCTL.LIN0PCTL.R          /* LINFlex_0 Clock Control Enable */
#define GPR0_PCTL_LIN1PCTL   GPR0_PCTL.LIN1PCTL.R          /* LINFlex_1 Clock Control Enable */
#define GPR0_PCTL_LIN2PCTL   GPR0_PCTL.LIN2PCTL.R          /* LINFlex_2 Clock Control Enable */
#define GPR0_PCTL_MSCLINPCTL GPR0_PCTL.MSCLINPCTL.R        /* MSC_0_LIN Clock Control Enable */
#define GPR0_PCTL_ADC1PCTL   GPR0_PCTL.ADC1PCTL.R          /* ADC_1 Clock Control Enable */
#define GPR0_PCTL_ADC0PCTL   GPR0_PCTL.ADC0PCTL.R          /* ADC_0 Clock Control Enable */
#define GPR0_PCTL_PSI50PCTL  GPR0_PCTL.PSI50PCTL.R         /* PSI5_0 Clock Control Enable */
#define GPR0_PCTL_CTUPCTL    GPR0_PCTL.CTUPCTL.R           /* CTU Clock Control Enable */
#define GPR0_PCTL_SIUL0PCTL  GPR0_PCTL.SIUL0PCTL.R         /* SIUL2_0 Clock Control Enable */
#define GPR0_PCTL_GTMNANOPCTL GPR0_PCTL.GTMNANOPCTL.R      /* GTM NANO Clock Control Enable */
#define GPR0_PCTL_PSI5S0PCTL GPR0_PCTL.PSI5S0PCTL.R        /* PSI5_S_0 Clock Control Enable */
#define GPR0_PCTL_SINCPCTL   GPR0_PCTL.SINCPCTL.R          /* SINC Clock Control Enable */

/* GPR1 */
#define GPR1_LVFCCUGD1       GPR1.LVFCCUGD1.R              /* VFCCU Global DID register 1 */
#define GPR1_LVFCCULD3       GPR1.LVFCCULD3.R              /* VFCCU Local DID register 3 */
#define GPR1_LVFCCULD4       GPR1.LVFCCULD4.R              /* VFCCU Local DID register 4 */
#define GPR1_LVFCCULD5       GPR1.LVFCCULD5.R              /* VFCCU Local DID register 5 */
#define GPR1_DESTC2          GPR1.DESTC2.R                 /* Source Control for RGMII TX CLK */
#define GPR1_FUNCC2          GPR1.FUNCC2.R                 /* Zipwire Configuration */
#define GPR1_INITM1          GPR1.INITM1.R                 /* NoC Initiator NIU Timeout Status */
#define GPR1_TARGTMS1        GPR1.TARGTMS1.R               /* NoC Target NIU Timeout Status */
#define GPR1_TARGTMC1        GPR1.TARGTMC1.R               /* NoC Target NIU Timeout Control */
#define GPR1_NETCC0          GPR1.NETCC0.R                 /* NETC Control register 0 */
#define GPR1_NETCC1          GPR1.NETCC1.R                 /* NETC Control register 1 */
#define GPR1_CLKOUT1SEL      GPR1.CLKOUT1SEL.R             /* CLKOUT_1 MUX select */
#define GPR1_XPAR1           GPR1.XPAR1.R                  /* Interface parity control and status register */

/* GPR1_PCTL */
#define GPR1_PCTL_I3C1PCTL   GPR1_PCTL.I3C1PCTL.R          /* LPI2C_1 Clock Control Enable */
#define GPR1_PCTL_EDMA1PCTL  GPR1_PCTL.EDMA1PCTL.R         /* eDMA_1 Clock Control Enable */
#define GPR1_PCTL_DSPI2PCTL  GPR1_PCTL.DSPI2PCTL.R         /* SPI_2 Clock Control Enable */
#define GPR1_PCTL_DSPI3PCTL  GPR1_PCTL.DSPI3PCTL.R         /* SPI_3 Clock Control Enable */
#define GPR1_PCTL_DSPI4PCTL  GPR1_PCTL.DSPI4PCTL.R         /* SPI_4 Clock Control Enable */
#define GPR1_PCTL_LIN3PCTL   GPR1_PCTL.LIN3PCTL.R          /* LINFlex_3 Clock Control Enable */
#define GPR1_PCTL_LIN4PCTL   GPR1_PCTL.LIN4PCTL.R          /* LINFlex_4 Clock Control Enable */
#define GPR1_PCTL_LIN5PCTL   GPR1_PCTL.LIN5PCTL.R          /* LINFlex_5 Clock Control Enable */
#define GPR1_PCTL_SIPI0PCTL  GPR1_PCTL.SIPI0PCTL.R         /* SIPI_0 Clock Control Enable */
#define GPR1_PCTL_SIPI1PCTL  GPR1_PCTL.SIPI1PCTL.R         /* SIPI_1 Clock Control Enable */
#define GPR1_PCTL_SRX0PCTL   GPR1_PCTL.SRX0PCTL.R          /* SRX_0 Clock Control Enable */
#define GPR1_PCTL_ENET0PCTL  GPR1_PCTL.ENET0PCTL.R         /* ENET_0 Clock Control Enable */
#define GPR1_PCTL_SIUL1PCTL  GPR1_PCTL.SIUL1PCTL.R         /* SIUL2_1 Clock Control Enable */

/* GPR2 */
#define GPR2_LVFCCUGD2       GPR2.LVFCCUGD2.R              /* VFCCU Global DID register 2 */
#define GPR2_LVFCCULD6       GPR2.LVFCCULD6.R              /* VFCCU Local DID register 6 */
#define GPR2_LVFCCULD7       GPR2.LVFCCULD7.R              /* VFCCU Local DID register 7 */
#define GPR2_LVFCCULD8       GPR2.LVFCCULD8.R              /* VFCCU Local DID register 8 */
#define GPR2_SPF2C           GPR2.SPF2C.R                  /* CEVA_SPF2 Control */
#define GPR2_FUNCC4          GPR2.FUNCC4.R                 /* Miscellaneous Control */
#define GPR2_INITM2          GPR2.INITM2.R                 /* NoC Initiator NIU Timeout Status */
#define GPR2_TARGTMS2        GPR2.TARGTMS2.R               /* NoC Target NIU Timeout Status */
#define GPR2_TARGTMC2        GPR2.TARGTMC2.R               /* NoC Target NIU Timeout Control */
#define GPR2_SMURSTCNT       GPR2.SMURSTCNT.R              /* SMU Cortex-M33 Core Reset Counter */
#define GPR2_SMUS            GPR2.SMUS.R                   /* SMU Cortex-M33 Core Status */
#define GPR2_XPAR2           GPR2.XPAR2.R                  /* Interface parity control and status register */

/* GPR3 */
#define GPR3_LVFCCUGD3       GPR3.LVFCCUGD3.R              /* VFCCU Global DID register 3 */
#define GPR3_LVFCCULD9       GPR3.LVFCCULD9.R              /* VFCCU Local DID register 9 */
#define GPR3_LVFCCULD10      GPR3.LVFCCULD10.R             /* VFCCU Local DID register 10 */
#define GPR3_LVFCCULD11      GPR3.LVFCCULD11.R             /* VFCCU Local DID register 11 */
#define GPR3_DDRCS           GPR3.DDRCS.R                  /* DDRC Status */
#define GPR3_DDRCC           GPR3.DDRCC.R                  /* DDRC Control */
#define GPR3_GENC0           GPR3.GENC0.R                  /* Generic control register 0 */
#define GPR3_FUNCC6          GPR3.FUNCC6.R                 /* Enable for TX Ack Suppress Logic */
#define GPR3_INITM3          GPR3.INITM3.R                 /* NoC Initiator NIU Timeout Status */
#define GPR3_TARGTMS3        GPR3.TARGTMS3.R               /* NoC Target NIU Timeout Status */
#define GPR3_TARGTMC3        GPR3.TARGTMC3.R               /* NoC Target NIU Timeout Control */
#define GPR3_RTU0FDC         GPR3.RTU0FDC.R                /* RTU0 Fence and Drain Control */
#define GPR3_RTU0FDS         GPR3.RTU0FDS.R                /* RTU0 Fence and Drain Status */
#define GPR3_RTU1FDC         GPR3.RTU1FDC.R                /* RTU1 Fence and Drain Control */
#define GPR3_RTU1FDS         GPR3.RTU1FDS.R                /* RTU1 Fence and Drain Status */
#define GPR3_CE0RSTCNT       GPR3.CE0RSTCNT.R              /* FlexLLCE Cortex-M33 Core 0 Reset Counter */
#define GPR3_CE1RSTCNT       GPR3.CE1RSTCNT.R              /* FlexLLCE Cortex-M33 Core 1 Reset Counter */
#define GPR3_CLKOUT4SEL      GPR3.CLKOUT4SEL.R             /* CLKOUT_4 MUX select */
#define GPR3_RGMC            GPR3.RGMC.R                   /* MC_RGM Control */
#define GPR3_XRDC_INT_S      GPR3.XRDC_INT_S.R             /* XRDC Interrupt Source */
#define GPR3_FLEXLLCECM33S   GPR3.FLEXLLCECM33S.R          /* FlexLLCE Cortex-M33 Core 0 Status */
#define GPR3_FLEXLLCECM33CES GPR3.FLEXLLCECM33CES.R        /* FlexLLCE Cortex-M33 Core 1 Status */
#define GPR3_XPAR3           GPR3.XPAR3.R                  /* Interface parity control and status register */
#define GPR3_XPAR4           GPR3.XPAR4.R                  /* Interface parity control and status register */
#define GPR3_RTU0XPAR        GPR3.RTU0XPAR.R               /* RTU0 global parity control register */
#define GPR3_RTU1XPAR        GPR3.RTU1XPAR.R               /* RTU1 global parity control register */

/* GPR3_PCTL */
#define GPR3_PCTL_EDMA3PCTL  GPR3_PCTL.EDMA3PCTL.R         /* eDMA_3 Clock Control Enable */
#define GPR3_PCTL_EDMACEPCTL GPR3_PCTL.EDMACEPCTL.R        /* CE_eDMA Clock Control Enable */
#define GPR3_PCTL_CAN0PCTL   GPR3_PCTL.CAN0PCTL.R          /* CE_CAN_0 Clock Control Enable */
#define GPR3_PCTL_CAN1PCTL   GPR3_PCTL.CAN1PCTL.R          /* CE_CAN_1 Clock Control Enable */
#define GPR3_PCTL_CAN2PCTL   GPR3_PCTL.CAN2PCTL.R          /* CE_CAN_2 Clock Control Enable */
#define GPR3_PCTL_CAN3PCTL   GPR3_PCTL.CAN3PCTL.R          /* CE_CAN_3 Clock Control Enable */
#define GPR3_PCTL_CAN4PCTL   GPR3_PCTL.CAN4PCTL.R          /* CE_CAN_4 Clock Control Enable */
#define GPR3_PCTL_CAN5PCTL   GPR3_PCTL.CAN5PCTL.R          /* CE_CAN_5 Clock Control Enable */
#define GPR3_PCTL_CAN6PCTL   GPR3_PCTL.CAN6PCTL.R          /* CE_CAN_6 Clock Control Enable */
#define GPR3_PCTL_CAN7PCTL   GPR3_PCTL.CAN7PCTL.R          /* CE_CAN_7 Clock Control Enable */
#define GPR3_PCTL_CAN8PCTL   GPR3_PCTL.CAN8PCTL.R          /* CE_CAN_8 Clock Control Enable */
#define GPR3_PCTL_CAN9PCTL   GPR3_PCTL.CAN9PCTL.R          /* CE_CAN_9 Clock Control Enable */
#define GPR3_PCTL_CAN10PCTL  GPR3_PCTL.CAN10PCTL.R         /* CE_CAN_10 Clock Control Enable */
#define GPR3_PCTL_CAN11PCTL  GPR3_PCTL.CAN11PCTL.R         /* CE_CAN_11 Clock Control Enable */
#define GPR3_PCTL_CAN12PCTL  GPR3_PCTL.CAN12PCTL.R         /* CE_CAN_12 Clock Control Enable */
#define GPR3_PCTL_CAN13PCTL  GPR3_PCTL.CAN13PCTL.R         /* CE_CAN_13 Clock Control Enable */
#define GPR3_PCTL_CAN14PCTL  GPR3_PCTL.CAN14PCTL.R         /* CE_CAN_14 Clock Control Enable */
#define GPR3_PCTL_CAN15PCTL  GPR3_PCTL.CAN15PCTL.R         /* CE_CAN_15 Clock Control Enable */
#define GPR3_PCTL_CAN16PCTL  GPR3_PCTL.CAN16PCTL.R         /* CE_CAN_16 Clock Control Enable */
#define GPR3_PCTL_CAN17PCTL  GPR3_PCTL.CAN17PCTL.R         /* CE_CAN_17 Clock Control Enable */
#define GPR3_PCTL_CAN18PCTL  GPR3_PCTL.CAN18PCTL.R         /* CE_CAN_18 Clock Control Enable */
#define GPR3_PCTL_CAN19PCTL  GPR3_PCTL.CAN19PCTL.R         /* CE_CAN_19 Clock Control Enable */
#define GPR3_PCTL_CAN20PCTL  GPR3_PCTL.CAN20PCTL.R         /* CE_CAN_20 Clock Control Enable */
#define GPR3_PCTL_CAN21PCTL  GPR3_PCTL.CAN21PCTL.R         /* CE_CAN_21 Clock Control Enable */
#define GPR3_PCTL_CAN22PCTL  GPR3_PCTL.CAN22PCTL.R         /* CE_CAN_22 Clock Control Enable */
#define GPR3_PCTL_CAN23PCTL  GPR3_PCTL.CAN23PCTL.R         /* CE_CAN_23 Clock Control Enable */
#define GPR3_PCTL_PIT0PCTL   GPR3_PCTL.PIT0PCTL.R          /* CE_PIT_0 Clock Control Enable */
#define GPR3_PCTL_PIT1PCTL   GPR3_PCTL.PIT1PCTL.R          /* CE_PIT_1 Clock Control Enable */
#define GPR3_PCTL_PIT2PCTL   GPR3_PCTL.PIT2PCTL.R          /* CE_PIT_2 Clock Control Enable */
#define GPR3_PCTL_PIT3PCTL   GPR3_PCTL.PIT3PCTL.R          /* CE_PIT_3 Clock Control Enable */
#define GPR3_PCTL_PIT4PCTL   GPR3_PCTL.PIT4PCTL.R          /* CE_PIT_4 Clock Control Enable */
#define GPR3_PCTL_PIT5PCTL   GPR3_PCTL.PIT5PCTL.R          /* CE_PIT_5 Clock Control Enable */
#define GPR3_PCTL_RXLUTPCTL  GPR3_PCTL.RXLUTPCTL.R         /* RXLUT Clock Control Enable */

/* GPR4 */
#define GPR4_LVFCCUGD4       GPR4.LVFCCUGD4.R              /* VFCCU Global DID register 4 */
#define GPR4_LVFCCULD12      GPR4.LVFCCULD12.R             /* VFCCU Local DID register 12 */
#define GPR4_LVFCCULD13      GPR4.LVFCCULD13.R             /* VFCCU Local DID register 13 */
#define GPR4_LVFCCULD14      GPR4.LVFCCULD14.R             /* VFCCU Local DID register 14 */
#define GPR4_INITM4          GPR4.INITM4.R                 /* NoC Initiator NIU Timeout Status */
#define GPR4_TARGTMS4        GPR4.TARGTMS4.R               /* NoC Target NIU Timeout Status */
#define GPR4_TARGTMC4        GPR4.TARGTMC4.R               /* NoC Target NIU Timeout Control */
#define GPR4_CLKOUT2SEL      GPR4.CLKOUT2SEL.R             /* CLKOUT_2 MUX select */
#define GPR4_MEW0MSTDIS      GPR4.MEW0MSTDIS.R             /* MEW_0 Master ID Disable */
#define GPR4_MEW1MSTDIS      GPR4.MEW1MSTDIS.R             /* MEW_1 Master ID Disable */
#define GPR4_HSE_SYS_FUSEC   GPR4.HSE_SYS_FUSEC.R          /* System Fuse Update Disable */

/* GPR4_PCTL */
#define GPR4_PCTL_QSPI0PCTL  GPR4_PCTL.QSPI0PCTL.R         /* QuadSPI_0 Clock Control Enable */
#define GPR4_PCTL_QSPI1PCTL  GPR4_PCTL.QSPI1PCTL.R         /* QuadSPI_1 Clock Control Enable */
#define GPR4_PCTL_EDMA4PCTL  GPR4_PCTL.EDMA4PCTL.R         /* eDMA_4 Clock Control Enable */
#define GPR4_PCTL_DSPI5PCTL  GPR4_PCTL.DSPI5PCTL.R         /* SPI_5 Clock Control Enable */
#define GPR4_PCTL_DSPI6PCTL  GPR4_PCTL.DSPI6PCTL.R         /* SPI_6 Clock Control Enable */
#define GPR4_PCTL_DSPI7PCTL  GPR4_PCTL.DSPI7PCTL.R         /* SPI_7 Clock Control Enable */
#define GPR4_PCTL_LIN6PCTL   GPR4_PCTL.LIN6PCTL.R          /* LINFlex_6 Clock Control Enable */
#define GPR4_PCTL_LIN7PCTL   GPR4_PCTL.LIN7PCTL.R          /* LINFlex_7 Clock Control Enable */
#define GPR4_PCTL_LIN8PCTL   GPR4_PCTL.LIN8PCTL.R          /* LINFlex_8 Clock Control Enable */
#define GPR4_PCTL_SDHCPCTL   GPR4_PCTL.SDHCPCTL.R          /* uSDHC Clock Control Enable */
#define GPR4_PCTL_SRX1PCTL   GPR4_PCTL.SRX1PCTL.R          /* SRX_1 Clock Control Enable */
#define GPR4_PCTL_I3C2PCTL   GPR4_PCTL.I3C2PCTL.R          /* LPI2C_2 Clock Control Enable */
#define GPR4_PCTL_PSI51PCTL  GPR4_PCTL.PSI51PCTL.R         /* PSI5_1 Clock Control Enable */
#define GPR4_PCTL_SIUL4PCTL  GPR4_PCTL.SIUL4PCTL.R         /* SIUL2_4 Clock Control Enable */
#define GPR4_PCTL_PSI5S1PCTL GPR4_PCTL.PSI5S1PCTL.R        /* PSI5_S_1 Clock Control Enable */

/* GPR5 */
#define GPR5_LVFCCUGD5       GPR5.LVFCCUGD5.R              /* VFCCU Global DID register 5 */
#define GPR5_LVFCCULD15      GPR5.LVFCCULD15.R             /* VFCCU Local DID register 15 */
#define GPR5_LVFCCULD16      GPR5.LVFCCULD16.R             /* VFCCU Local DID register 16 */
#define GPR5_LVFCCULD17      GPR5.LVFCCULD17.R             /* VFCCU Local DID register 17 */
#define GPR5_DESTC10         GPR5.DESTC10.R                /* CANXL_PE_CLK Configuration */
#define GPR5_INITM5          GPR5.INITM5.R                 /* NoC Initiator NIU Timeout Status */
#define GPR5_TARGTMS5        GPR5.TARGTMS5.R               /* NoC Target NIU Timeout Status */
#define GPR5_TARGTMC5        GPR5.TARGTMC5.R               /* NoC Target NIU Timeout Control */
#define GPR5_CLKOUT3SEL      GPR5.CLKOUT3SEL.R             /* CLKOUT_3 MUX select */

/* GPR5_PCTL */
#define GPR5_PCTL_EDMA5PCTL  GPR5_PCTL.EDMA5PCTL.R         /* eDMA_5 Clock Control Enable */
#define GPR5_PCTL_DSPI8PCTL  GPR5_PCTL.DSPI8PCTL.R         /* SPI_8 Clock Control Enable */
#define GPR5_PCTL_DSPI9PCTL  GPR5_PCTL.DSPI9PCTL.R         /* SPI_9 Clock Control Enable */
#define GPR5_PCTL_LIN9PCTL   GPR5_PCTL.LIN9PCTL.R          /* LINFlex_9 Clock Control Enable */
#define GPR5_PCTL_LIN10PCTL  GPR5_PCTL.LIN10PCTL.R         /* LINFlex_10 Clock Control Enable */
#define GPR5_PCTL_LIN11PCTL  GPR5_PCTL.LIN11PCTL.R         /* LINFlex_11 Clock Control Enable */
#define GPR5_PCTL_SIUL5PCTL  GPR5_PCTL.SIUL5PCTL.R         /* SIUL2_5 Clock Control Enable */

/* GPR6 */
#define GPR6_LVFCCUGD6       GPR6.LVFCCUGD6.R              /* VFCCU Global DID register 6 */
#define GPR6_LVFCCULD18      GPR6.LVFCCULD18.R             /* VFCCU Local DID register 18 */
#define GPR6_LVFCCULD19      GPR6.LVFCCULD19.R             /* VFCCU Local DID register 19 */
#define GPR6_LVFCCULD20      GPR6.LVFCCULD20.R             /* VFCCU Local DID register 20 */
#define GPR6_TARGTMS6        GPR6.TARGTMS6.R               /* NoC Target NIU Timeout Status */
#define GPR6_TARGTMC6        GPR6.TARGTMC6.R               /* NoC Target NIU Timeout Control */
#define GPR6_XPAR5           GPR6.XPAR5.R                  /* Interface parity control and status register */

/* GPR6_PCTL */
#define GPR6_PCTL_DDRPCTL    GPR6_PCTL.DDRPCTL.R           /* DDRC Clock Control Enable */

/* GTMDI */
#define GTMDI_DID            GTMDI.DID.R                   /* Device Identity */
#define GTMDI_DC             GTMDI.DC.R                    /* GTMDI Development Control */
#define GTMDI_DS             GTMDI.DS.R                    /* GTMDI Development Status */
#define GTMDI_TIM_WPC1       GTMDI.TIM_WPC1.R              /* TIM Watchpoint Control 1 */
#define GTMDI_TIM_WPC2       GTMDI.TIM_WPC2.R              /* TIM Watchpoint Control 2 */
#define GTMDI_TOM_WPC1       GTMDI.TOM_WPC1.R              /* TOM Watchpoint Control 1 */
#define GTMDI_TOM_WPC2       GTMDI.TOM_WPC2.R              /* TOM Watchpoint Control 2 */
#define GTMDI_ATOM_WPC1      GTMDI.ATOM_WPC1.R             /* ATOM Watchpoint Control 1 */
#define GTMDI_ATOM_WPC2      GTMDI.ATOM_WPC2.R             /* ATOM Watchpoint Control 2 */
#define GTMDI_SPEA_WPC1      GTMDI.SPEA_WPC1.R             /* SPEA Watchpoint Control 1 */
#define GTMDI_SPEA_WPC2      GTMDI.SPEA_WPC2.R             /* SPEA Watchpoint Control 2 */
#define GTMDI_SPEB_WPC1      GTMDI.SPEB_WPC1.R             /* SPEB Watchpoint Control 1 */
#define GTMDI_SPEB_WPC2      GTMDI.SPEB_WPC2.R             /* SPEB Watchpoint Control 2 */
#define GTMDI_DPLL_WPC1      GTMDI.DPLL_WPC1.R             /* DPLL Watchpoint Control 1 */
#define GTMDI_DPLL_WPC2      GTMDI.DPLL_WPC2.R             /* DPLL Watchpoint Control 2 */
#define GTMDI_DPLL_WPC3      GTMDI.DPLL_WPC3.R             /* DPLL Watchpoint Control 3 */
#define GTMDI_DPLL_WPC4      GTMDI.DPLL_WPC4.R             /* DPLL Watchpoint Control 4 */
#define GTMDI_DPLL_WPC5      GTMDI.DPLL_WPC5.R             /* DPLL Watchpoint Control 5 */
#define GTMDI_DPLL_DTC       GTMDI.DPLL_DTC.R              /* DPLL Data Trace Control */
#define GTMDI_ARU_WPC1       GTMDI.ARU_WPC1.R              /* ARU Watchpoint Control 1 */
#define GTMDI_ARU_WPC2       GTMDI.ARU_WPC2.R              /* ARU Watchpoint Control 2 */
#define GTMDI_ARU_D0H        GTMDI.ARU_D0H.R               /* ARU Watchpoint DATA0H */
#define GTMDI_ARU_D0L        GTMDI.ARU_D0L.R               /* ARU Watchpoint DATA0L */
#define GTMDI_ARU_D1H        GTMDI.ARU_D1H.R               /* ARU Watchpoint DATA1H */
#define GTMDI_ARU_D1L        GTMDI.ARU_D1L.R               /* ARU Watchpoint ARUD1L */
#define GTMDI_ARU_DTC        GTMDI.ARU_DTC.R               /* ARU Data Trace Control */
#define GTMDI_MCS_HBC        GTMDI.MCS_HBC.R               /* MCS Hardware Breakpoint (HBP) Control */
#define GTMDI_MCS_HLT        GTMDI.MCS_HLT.R               /* MCS Halt Control */
#define GTMDI_MCSA_DC        GTMDI.MCSA_DC.R               /* MCSA Development Control */
#define GTMDI_MCSA_WPC       GTMDI.MCSA_WPC.R              /* MCSA Watchpoint Control */
#define GTMDI_MCSA_PTC       GTMDI.MCSA_PTC.R              /* MCSA Program Fetch Trace Control */
#define GTMDI_MCSA_DTC       GTMDI.MCSA_DTC.R              /* MCSA Data Trace Control */
#define GTMDI_MCSA_WPA1      GTMDI.MCSA_WPA1.R             /* MCSA Watchpoint Address 1 */
#define GTMDI_MCSA_WPA2      GTMDI.MCSA_WPA2.R             /* MCSA Watchpoint Address 2 */
#define GTMDI_MCSA_WPD1      GTMDI.MCSA_WPD1.R             /* MCSA Watchpoint Data 1 */
#define GTMDI_MCSA_WPD2      GTMDI.MCSA_WPD2.R             /* MCSA Watchpoint Data 2 */
#define GTMDI_MCSA_CE        GTMDI.MCSA_CE.R               /* MCSA Channel Enable */
#define GTMDI_MCSA_DTAR1     GTMDI.MCSA_DTAR1.R            /* MCSA Data Trace Address Range 1 */
#define GTMDI_MCSA_DTAR2     GTMDI.MCSA_DTAR2.R            /* MCSA Data Trace Address Range 2 */
#define GTMDI_MCSB_DC        GTMDI.MCSB_DC.R               /* MCSB Development Control */
#define GTMDI_MCSB_WPC       GTMDI.MCSB_WPC.R              /* MCSB Watchpoint Control */
#define GTMDI_MCSB_PTC       GTMDI.MCSB_PTC.R              /* MCSB Program Fetch Trace Control */
#define GTMDI_MCSB_DTC       GTMDI.MCSB_DTC.R              /* MCSB Data Trace Control */
#define GTMDI_MCSB_WPA1      GTMDI.MCSB_WPA1.R             /* MCSB Watchpoint Address 1 */
#define GTMDI_MCSB_WPA2      GTMDI.MCSB_WPA2.R             /* MCSB Watchpoint Address 2 */
#define GTMDI_MCSB_WPD1      GTMDI.MCSB_WPD1.R             /* MCSB Watchpoint Data 1 */
#define GTMDI_MCSB_WPD2      GTMDI.MCSB_WPD2.R             /* MCSB Watchpoint Data 2 */
#define GTMDI_MCSB_CE        GTMDI.MCSB_CE.R               /* MCSB Channel Enable */
#define GTMDI_MCSB_DTAR1     GTMDI.MCSB_DTAR1.R            /* MCSB Data Trace Address Range 1 */
#define GTMDI_MCSB_DTAR2     GTMDI.MCSB_DTAR2.R            /* MCSB Data Trace Address Range 2 */
#define GTMDI_TBU0_WPC1      GTMDI.TBU0_WPC1.R             /* TBU0 Watchpoint Control 1 */
#define GTMDI_TBU0_WPC2      GTMDI.TBU0_WPC2.R             /* TBUn Watchpoint Control 2 */
#define GTMDI_TBU0_D         GTMDI.TBU0_D.R                /* TBU0 Watchpoint DATA */
#define GTMDI_TBU1_WPC1      GTMDI.TBU1_WPC1.R             /* TBUn Watchpoint Control 1 */
#define GTMDI_TBU1_WPC2      GTMDI.TBU1_WPC2.R             /* TBUn Watchpoint Control 2 */
#define GTMDI_TBU1_D         GTMDI.TBU1_D.R                /* TBUn Watchpoint DATA */
#define GTMDI_TBU2_WPC1      GTMDI.TBU2_WPC1.R             /* TBUn Watchpoint Control 1 */
#define GTMDI_TBU2_WPC2      GTMDI.TBU2_WPC2.R             /* TBUn Watchpoint Control 2 */
#define GTMDI_TBU2_D         GTMDI.TBU2_D.R                /* TBUn Watchpoint DATA */
#define GTMDI_TBU3_WPC1      GTMDI.TBU3_WPC1.R             /* TBUn Watchpoint Control 1 */
#define GTMDI_TBU3_WPC2      GTMDI.TBU3_WPC2.R             /* TBUn Watchpoint Control 2 */
#define GTMDI_TBU3_D         GTMDI.TBU3_D.R                /* TBUn Watchpoint DATA */
#define GTMDI_TIO_IN_WPC1    GTMDI.TIO_IN_WPC1.R           /* TIO IN watchpoint control register 1 */
#define GTMDI_TIO_IN_WPC2    GTMDI.TIO_IN_WPC2.R           /* TIO IN watchpoint control register 2 */
#define GTMDI_TIO_OUT_WPC1   GTMDI.TIO_OUT_WPC1.R          /* TIO OUT Watchpoint Control 1 */
#define GTMDI_TIO_OUT_WPC2   GTMDI.TIO_OUT_WPC2.R          /* TIO OUT Watchpoint Control 2 */
#define GTMDI_ATID           GTMDI.ATID.R                  /* ATB Bus ID Control */
#define GTMDI_DEVARCH        GTMDI.DEVARCH.R               /* Device Architecture */
#define GTMDI_DEVTYPE        GTMDI.DEVTYPE.R               /* Device Type Identifier */
#define GTMDI_PIDR4          GTMDI.PIDR4.R                 /* Peripheral Identification 4 */
#define GTMDI_PIDR5          GTMDI.PIDR5.R                 /* Peripheral Identification 5 */
#define GTMDI_PIDR6          GTMDI.PIDR6.R                 /* Peripheral Identification 6 */
#define GTMDI_PIDR7          GTMDI.PIDR7.R                 /* Peripheral Identification 7 */
#define GTMDI_PIDR0          GTMDI.PIDR0.R                 /* Peripheral Identification 0 */
#define GTMDI_PIDR1          GTMDI.PIDR1.R                 /* Peripheral Identification 1 */
#define GTMDI_PIDR2          GTMDI.PIDR2.R                 /* Peripheral Identification 2 */
#define GTMDI_PIDR3          GTMDI.PIDR3.R                 /* Peripheral Identification 3 */
#define GTMDI_CIDR0          GTMDI.CIDR0.R                 /* Component Identification 0 */
#define GTMDI_CIDR1          GTMDI.CIDR1.R                 /* Component Identification 1 */
#define GTMDI_CIDR2          GTMDI.CIDR2.R                 /* Component Identification 2 */
#define GTMDI_CIDR3          GTMDI.CIDR3.R                 /* Component Identification 3 */

/* GTMSS */
#define GTMSS_IRQ_CFG0       GTMSS.IRQ_CFG0.R              /* IRQ Configuration Register for Cluster 0 */
#define GTMSS_IRQ_CFG1       GTMSS.IRQ_CFG1.R              /* IRQ Confoguration Register for Cluster 1 */
#define GTMSS_IRQ_CFG2       GTMSS.IRQ_CFG2.R              /* IRQ Confoguration Register for Cluster 2 */
#define GTMSS_IRQ_CFG3       GTMSS.IRQ_CFG3.R              /* IRQ Confoguration Register for Cluster 3 */
#define GTMSS_DBG_DID        GTMSS.DBG_DID.R               /* Debug domain ID register */

/* GTM_gtm_cls0 */
#define GTM_gtm_cls0_GTM_REV GTM_gtm_cls0.GTM_REV.R        /* GTM version control register */
#define GTM_gtm_cls0_GTM_RST GTM_gtm_cls0.GTM_RST.R        /* GTM global reset register */
#define GTM_gtm_cls0_GTM_CTRL GTM_gtm_cls0.GTM_CTRL.R      /* GTM global control register */
#define GTM_gtm_cls0_GTM_CFG GTM_gtm_cls0.GTM_CFG.R        /* GTM configuration register */
#define GTM_gtm_cls0_GTM_AEI_ADDR_XPT GTM_gtm_cls0.GTM_AEI_ADDR_XPT.R /* GTM AEI timeout exception address register */
#define GTM_gtm_cls0_GTM_AEI_STA_XPT GTM_gtm_cls0.GTM_AEI_STA_XPT.R /* GTM AEI non zero status register */
#define GTM_gtm_cls0_GTM_IRQ_NOTIFY GTM_gtm_cls0.GTM_IRQ_NOTIFY.R /* GTM Interrupt notification register */
#define GTM_gtm_cls0_GTM_IRQ_EN GTM_gtm_cls0.GTM_IRQ_EN.R  /* GTM interrupt enable register */
#define GTM_gtm_cls0_GTM_EIRQ_EN GTM_gtm_cls0.GTM_EIRQ_EN.R /* GTM error interrupt enable register */
#define GTM_gtm_cls0_GTM_IRQ_FORCINT GTM_gtm_cls0.GTM_IRQ_FORCINT.R /* GTM Software interrupt generation register */
#define GTM_gtm_cls0_GTM_IRQ_MODE GTM_gtm_cls0.GTM_IRQ_MODE.R /* GTM top level interrupts mode selection */
#define GTM_gtm_cls0_GTM_CLS_CLK_CFG GTM_gtm_cls0.GTM_CLS_CLK_CFG.R /* GTM Cluster Clock Configuration */
#define GTM_gtm_cls0_GTM_ARU_COM_DIS GTM_gtm_cls0.GTM_ARU_COM_DIS.R /* GTM ARU communication disable */
#define GTM_gtm_cls0_BRIDGE_MODE GTM_gtm_cls0.BRIDGE_MODE.R /* GTM AEI bridge mode register */
#define GTM_gtm_cls0_BRIDGE_PTR1 GTM_gtm_cls0.BRIDGE_PTR1.R /* GTM AEI bridge pointer 1 register */
#define GTM_gtm_cls0_BRIDGE_PTR2 GTM_gtm_cls0.BRIDGE_PTR2.R /* GTM AEI bridge pointer 2 register */
#define GTM_gtm_cls0_MCS_AEM_DIS GTM_gtm_cls0.MCS_AEM_DIS.R /* GTM MCS master port disable register */
#define GTM_gtm_cls0_CMU_CLK_EN GTM_gtm_cls0.CMU_CLK_EN.R  /* CMU clock enable */
#define GTM_gtm_cls0_CMU_GCLK_NUM GTM_gtm_cls0.CMU_GCLK_NUM.R /* The numerator for CMU global clock resolution generator */
#define GTM_gtm_cls0_CMU_GCLK_DEN GTM_gtm_cls0.CMU_GCLK_DEN.R /* The denominator for CMU global clock resolution generator */
#define GTM_gtm_cls0_CMU_CLK_0_CTRL GTM_gtm_cls0.CMU_CLK_0_CTRL.R /* CMU control for clock resolution generator [x] */
#define GTM_gtm_cls0_CMU_CLK_1_CTRL GTM_gtm_cls0.CMU_CLK_1_CTRL.R /* CMU control for clock resolution generator [x] */
#define GTM_gtm_cls0_CMU_CLK_2_CTRL GTM_gtm_cls0.CMU_CLK_2_CTRL.R /* CMU control for clock resolution generator [x] */
#define GTM_gtm_cls0_CMU_CLK_3_CTRL GTM_gtm_cls0.CMU_CLK_3_CTRL.R /* CMU control for clock resolution generator [x] */
#define GTM_gtm_cls0_CMU_CLK_4_CTRL GTM_gtm_cls0.CMU_CLK_4_CTRL.R /* CMU control for clock resolution generator [x] */
#define GTM_gtm_cls0_CMU_CLK_5_CTRL GTM_gtm_cls0.CMU_CLK_5_CTRL.R /* CMU control for clock resolution generator [x] */
#define GTM_gtm_cls0_CMU_CLK_6_CTRL GTM_gtm_cls0.CMU_CLK_6_CTRL.R /* CMU control for clock resolution generator 6 */
#define GTM_gtm_cls0_CMU_CLK_7_CTRL GTM_gtm_cls0.CMU_CLK_7_CTRL.R /* CMU control for clock resolution generator 7 */
#define GTM_gtm_cls0_CMU_ECLK_0_NUM GTM_gtm_cls0.CMU_ECLK_0_NUM.R /* The numerator for the external clock resolution generator [z] */
#define GTM_gtm_cls0_CMU_ECLK_0_DEN GTM_gtm_cls0.CMU_ECLK_0_DEN.R /* The denominator for the external clock resolution generator [z] */
#define GTM_gtm_cls0_CMU_ECLK_1_NUM GTM_gtm_cls0.CMU_ECLK_1_NUM.R /* The numerator for the external clock resolution generator [z] */
#define GTM_gtm_cls0_CMU_ECLK_1_DEN GTM_gtm_cls0.CMU_ECLK_1_DEN.R /* The denominator for the external clock resolution generator [z] */
#define GTM_gtm_cls0_CMU_ECLK_2_NUM GTM_gtm_cls0.CMU_ECLK_2_NUM.R /* The numerator for the external clock resolution generator [z] */
#define GTM_gtm_cls0_CMU_ECLK_2_DEN GTM_gtm_cls0.CMU_ECLK_2_DEN.R /* The denominator for the external clock resolution generator [z] */
#define GTM_gtm_cls0_CMU_FXCLK_CTRL GTM_gtm_cls0.CMU_FXCLK_CTRL.R /* CMU control for selection of FCR subblock input */
#define GTM_gtm_cls0_CMU_GLB_CTRL GTM_gtm_cls0.CMU_GLB_CTRL.R /* CMU synchronizing ARU and clock source */
#define GTM_gtm_cls0_CMU_CLK_CTRL GTM_gtm_cls0.CMU_CLK_CTRL.R /* CMU control for clock resolution generator */
#define GTM_gtm_cls0_TBU_CHEN GTM_gtm_cls0.TBU_CHEN.R      /* TBU global channel enable */
#define GTM_gtm_cls0_TBU_CH0_CTRL GTM_gtm_cls0.TBU_CH0_CTRL.R /* TBU channel 0 control */
#define GTM_gtm_cls0_TBU_CH0_BASE GTM_gtm_cls0.TBU_CH0_BASE.R /* TBU channel 0 base */
#define GTM_gtm_cls0_TBU_CH1_CTRL GTM_gtm_cls0.TBU_CH1_CTRL.R /* TBU channel 1 control */
#define GTM_gtm_cls0_TBU_CH1_BASE GTM_gtm_cls0.TBU_CH1_BASE.R /* TBU channel [x] base */
#define GTM_gtm_cls0_TBU_CH2_CTRL GTM_gtm_cls0.TBU_CH2_CTRL.R /* TBU channel 2 control */
#define GTM_gtm_cls0_TBU_CH2_BASE GTM_gtm_cls0.TBU_CH2_BASE.R /* TBU channel [x] base */
#define GTM_gtm_cls0_TBU_CH3_CTRL GTM_gtm_cls0.TBU_CH3_CTRL.R /* TBU channel 3 control */
#define GTM_gtm_cls0_TBU_CH3_BASE GTM_gtm_cls0.TBU_CH3_BASE.R /* TBU channel 3 base */
#define GTM_gtm_cls0_TBU_CH3_BASE_MARK GTM_gtm_cls0.TBU_CH3_BASE_MARK.R /* TBU channel 3 modulo value */
#define GTM_gtm_cls0_TBU_CH3_BASE_CAPTURE GTM_gtm_cls0.TBU_CH3_BASE_CAPTURE.R /* TBU channel 3 base captured */
#define GTM_gtm_cls0_ARU_ACCESS GTM_gtm_cls0.ARU_ACCESS.R  /* ARU access register */
#define GTM_gtm_cls0_ARU_DATA_H GTM_gtm_cls0.ARU_DATA_H.R  /* ARU access register upper data word */
#define GTM_gtm_cls0_ARU_DATA_L GTM_gtm_cls0.ARU_DATA_L.R  /* ARU access register lower data word */
#define GTM_gtm_cls0_ARU_DBG_ACCESS0 GTM_gtm_cls0.ARU_DBG_ACCESS0.R /* ARU debug access channel 0 */
#define GTM_gtm_cls0_ARU_DBG_DATA0_H GTM_gtm_cls0.ARU_DBG_DATA0_H.R /* ARU debug access 0 transfer register upper data word */
#define GTM_gtm_cls0_ARU_DBG_DATA0_L GTM_gtm_cls0.ARU_DBG_DATA0_L.R /* ARU debug access 0 transfer register lower data word */
#define GTM_gtm_cls0_ARU_DBG_ACCESS1 GTM_gtm_cls0.ARU_DBG_ACCESS1.R /* ARU debug access channel 0 */
#define GTM_gtm_cls0_ARU_DBG_DATA1_H GTM_gtm_cls0.ARU_DBG_DATA1_H.R /* ARU debug access 1 transfer register upper data word */
#define GTM_gtm_cls0_ARU_DBG_DATA1_L GTM_gtm_cls0.ARU_DBG_DATA1_L.R /* ARU debug access 1 transfer register lower data word */
#define GTM_gtm_cls0_ARU_IRQ_NOTIFY GTM_gtm_cls0.ARU_IRQ_NOTIFY.R /* ARU interrupt notification register */
#define GTM_gtm_cls0_ARU_IRQ_EN GTM_gtm_cls0.ARU_IRQ_EN.R  /* ARU interrupt enable register */
#define GTM_gtm_cls0_ARU_IRQ_FORCINT GTM_gtm_cls0.ARU_IRQ_FORCINT.R /* ARU force interrupt register */
#define GTM_gtm_cls0_ARU_IRQ_MODE GTM_gtm_cls0.ARU_IRQ_MODE.R /* ARU interrupt mode register */
#define GTM_gtm_cls0_ARU_CADDR_END GTM_gtm_cls0.ARU_CADDR_END.R /* ARU caddr counter end value */
#define GTM_gtm_cls0_ARU_CTRL GTM_gtm_cls0.ARU_CTRL.R      /* ARU enable dynamic routing */
#define GTM_gtm_cls0_ARU_0_DYN_CTRL GTM_gtm_cls0.ARU_0_DYN_CTRL.R /* ARU [g] dynamic routing control register */
#define GTM_gtm_cls0_ARU_1_DYN_CTRL GTM_gtm_cls0.ARU_1_DYN_CTRL.R /* ARU [g] dynamic routing control register */
#define GTM_gtm_cls0_ARU_0_DYN_ROUTE_LOW GTM_gtm_cls0.ARU_0_DYN_ROUTE_LOW.R /* ARU [g] lower bits of DYN_ROUTE register */
#define GTM_gtm_cls0_ARU_1_DYN_ROUTE_LOW GTM_gtm_cls0.ARU_1_DYN_ROUTE_LOW.R /* ARU [g] lower bits of DYN_ROUTE register */
#define GTM_gtm_cls0_ARU_0_DYN_ROUTE_HIGH GTM_gtm_cls0.ARU_0_DYN_ROUTE_HIGH.R /* ARU [g] higher bits of DYN_ROUTE register */
#define GTM_gtm_cls0_ARU_1_DYN_ROUTE_HIGH GTM_gtm_cls0.ARU_1_DYN_ROUTE_HIGH.R /* ARU [g] higher bits of DYN_ROUTE register */
#define GTM_gtm_cls0_ARU_0_DYN_ROUTE_SR_LOW GTM_gtm_cls0.ARU_0_DYN_ROUTE_SR_LOW.R /* ARU [g] shadow register of DYN_ROUTE register lower bits */
#define GTM_gtm_cls0_ARU_1_DYN_ROUTE_SR_LOW GTM_gtm_cls0.ARU_1_DYN_ROUTE_SR_LOW.R /* ARU [g] shadow register of DYN_ROUTE register lower bits */
#define GTM_gtm_cls0_ARU_0_DYN_ROUTE_SR_HIGH GTM_gtm_cls0.ARU_0_DYN_ROUTE_SR_HIGH.R /* ARU [g] shadow register of DYN_ROUTE register higher bits */
#define GTM_gtm_cls0_ARU_1_DYN_ROUTE_SR_HIGH GTM_gtm_cls0.ARU_1_DYN_ROUTE_SR_HIGH.R /* ARU [g] shadow register of DYN_ROUTE register higher bits */
#define GTM_gtm_cls0_ARU_0_DYN_RDADDR GTM_gtm_cls0.ARU_0_DYN_RDADDR.R /* ARU [g] master ID for dynamic routing */
#define GTM_gtm_cls0_ARU_1_DYN_RDADDR GTM_gtm_cls0.ARU_1_DYN_RDADDR.R /* ARU [g] master ID for dynamic routing */
#define GTM_gtm_cls0_ARU_CADDR GTM_gtm_cls0.ARU_CADDR.R    /* ARU caddr counter value */
#define GTM_gtm_cls0_BRC_SRC_0_ADDR GTM_gtm_cls0.BRC_SRC_0_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_0_DEST GTM_gtm_cls0.BRC_SRC_0_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_1_ADDR GTM_gtm_cls0.BRC_SRC_1_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_1_DEST GTM_gtm_cls0.BRC_SRC_1_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_2_ADDR GTM_gtm_cls0.BRC_SRC_2_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_2_DEST GTM_gtm_cls0.BRC_SRC_2_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_3_ADDR GTM_gtm_cls0.BRC_SRC_3_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_3_DEST GTM_gtm_cls0.BRC_SRC_3_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_4_ADDR GTM_gtm_cls0.BRC_SRC_4_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_4_DEST GTM_gtm_cls0.BRC_SRC_4_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_5_ADDR GTM_gtm_cls0.BRC_SRC_5_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_5_DEST GTM_gtm_cls0.BRC_SRC_5_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_6_ADDR GTM_gtm_cls0.BRC_SRC_6_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_6_DEST GTM_gtm_cls0.BRC_SRC_6_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_7_ADDR GTM_gtm_cls0.BRC_SRC_7_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_7_DEST GTM_gtm_cls0.BRC_SRC_7_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_8_ADDR GTM_gtm_cls0.BRC_SRC_8_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_8_DEST GTM_gtm_cls0.BRC_SRC_8_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_9_ADDR GTM_gtm_cls0.BRC_SRC_9_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_9_DEST GTM_gtm_cls0.BRC_SRC_9_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_10_ADDR GTM_gtm_cls0.BRC_SRC_10_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_10_DEST GTM_gtm_cls0.BRC_SRC_10_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_11_ADDR GTM_gtm_cls0.BRC_SRC_11_ADDR.R /* BRC read address for input channel [x] */
#define GTM_gtm_cls0_BRC_SRC_11_DEST GTM_gtm_cls0.BRC_SRC_11_DEST.R /* BRC destination channels for input channel [x] */
#define GTM_gtm_cls0_BRC_IRQ_NOTIFY GTM_gtm_cls0.BRC_IRQ_NOTIFY.R /* BRC interrupt notification register */
#define GTM_gtm_cls0_BRC_IRQ_EN GTM_gtm_cls0.BRC_IRQ_EN.R  /* BRC interrupt enable register */
#define GTM_gtm_cls0_BRC_IRQ_FORCINT GTM_gtm_cls0.BRC_IRQ_FORCINT.R /* BRC force interrupt register */
#define GTM_gtm_cls0_BRC_IRQ_MODE GTM_gtm_cls0.BRC_IRQ_MODE.R /* BRC interrupt mode configuration register */
#define GTM_gtm_cls0_BRC_RST GTM_gtm_cls0.BRC_RST.R        /* BRC software reset register */
#define GTM_gtm_cls0_BRC_EIRQ_EN GTM_gtm_cls0.BRC_EIRQ_EN.R /* BRC error interrupt enable register */
#define GTM_gtm_cls0_ICM_IRQG_0 GTM_gtm_cls0.ICM_IRQG_0.R  /* ICM Interrupt group register covering infrastructural and safety components (ARU, BRC, AEI, PSM0, PSM1, MAP, CMP, SPE) */
#define GTM_gtm_cls0_ICM_IRQG_1 GTM_gtm_cls0.ICM_IRQG_1.R  /* ICM Interrupt group register covering DPLL */
#define GTM_gtm_cls0_ICM_IRQG_2 GTM_gtm_cls0.ICM_IRQG_2.R  /* ICM Interrupt group register covering TIM0, TIM1, TIM2, TIM3 */
#define GTM_gtm_cls0_ICM_IRQG_4 GTM_gtm_cls0.ICM_IRQG_4.R  /* ICM Interrupt group register covering MCS0 to MCS3 submodules */
#define GTM_gtm_cls0_ICM_IRQG_MEI GTM_gtm_cls0.ICM_IRQG_MEI.R /* ICM Interrupt group register for module error interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_CEI0 GTM_gtm_cls0.ICM_IRQG_CEI0.R /* ICM Interrupt group register 0 for channel error interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_CEI1 GTM_gtm_cls0.ICM_IRQG_CEI1.R /* ICM Interrupt group register 1 for channel error interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_CEI3 GTM_gtm_cls0.ICM_IRQG_CEI3.R /* ICM Interrupt group register 3 for channel error interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_MCS0_CEI GTM_gtm_cls0.ICM_IRQG_MCS0_CEI.R /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_MCS1_CEI GTM_gtm_cls0.ICM_IRQG_MCS1_CEI.R /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_MCS2_CEI GTM_gtm_cls0.ICM_IRQG_MCS2_CEI.R /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_MCS3_CEI GTM_gtm_cls0.ICM_IRQG_MCS3_CEI.R /* ICM Interrupt group MCS[j] for Channel Error Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_PSM_0_CEI GTM_gtm_cls0.ICM_IRQG_PSM_0_CEI.R /* ICM Interrupt group PSM 0 for Channel Error Interrupt information of FIFO0, FIFO1, FIFO2 */
#define GTM_gtm_cls0_ICM_IRQG_SPE_CEI GTM_gtm_cls0.ICM_IRQG_SPE_CEI.R /* ICM Interrupt group SPE for module Error Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_CLS_0_MEI GTM_gtm_cls0.ICM_IRQG_CLS_0_MEI.R /* ICM Interrupt group for module Error Interrupt information for each TIM[j], MCS[j], SPE[j], FIFO[j] (j=4*[g]+k, k∈{0, ... ,3}) */
#define GTM_gtm_cls0_ICM_IRQG_MCS0_CI GTM_gtm_cls0.ICM_IRQG_MCS0_CI.R /* ICM Interrupt group MCS[j] for Channel Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_MCS1_CI GTM_gtm_cls0.ICM_IRQG_MCS1_CI.R /* ICM Interrupt group MCS[j] for Channel Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_MCS2_CI GTM_gtm_cls0.ICM_IRQG_MCS2_CI.R /* ICM Interrupt group MCS[j] for Channel Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_MCS3_CI GTM_gtm_cls0.ICM_IRQG_MCS3_CI.R /* ICM Interrupt group MCS[j] for Channel Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_PSM_0_CI GTM_gtm_cls0.ICM_IRQG_PSM_0_CI.R /* ICM Interrupt group PSM 0 for Channel Interrupt information of FIFO0, FIFO1, FIFO2 */
#define GTM_gtm_cls0_ICM_IRQG_SPE_CI GTM_gtm_cls0.ICM_IRQG_SPE_CI.R /* ICM Interrupt group SPE for module Interrupt information */
#define GTM_gtm_cls0_ICM_IRQG_ATOM_0_CI GTM_gtm_cls0.ICM_IRQG_ATOM_0_CI.R /* ICM Interrupt group ATOM [g] for Channel Interrupt information of ATOM[j] (m=4*[g]+(0..3)) */
#define GTM_gtm_cls0_ICM_IRQG_TOM_0_CI GTM_gtm_cls0.ICM_IRQG_TOM_0_CI.R /* ICM Interrupt group TOM [g] for Channel Interrupt information of TOMm ([j]=2*[g]+(0..1)) */
#define GTM_gtm_cls0_MAP_CTRL GTM_gtm_cls0.MAP_CTRL.R      /* MAP Control register */
#define GTM_gtm_cls0_TIM0_CH0_GPR0 GTM_gtm_cls0.TIM0_CH0_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH0_GPR1 GTM_gtm_cls0.TIM0_CH0_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH0_CNT GTM_gtm_cls0.TIM0_CH0_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls0_TIM0_CH0_ECNT GTM_gtm_cls0.TIM0_CH0_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls0_TIM0_CH0_CNTS GTM_gtm_cls0.TIM0_CH0_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls0_TIM0_CH0_TDUC GTM_gtm_cls0.TIM0_CH0_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls0_TIM0_CH0_TDUV GTM_gtm_cls0.TIM0_CH0_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls0_TIM0_CH0_FLT_RE GTM_gtm_cls0.TIM0_CH0_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls0_TIM0_CH0_FLT_FE GTM_gtm_cls0.TIM0_CH0_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls0_TIM0_CH0_CTRL GTM_gtm_cls0.TIM0_CH0_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls0_TIM0_CH0_ECTRL GTM_gtm_cls0.TIM0_CH0_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls0_TIM0_CH0_IRQ_NOTIFY GTM_gtm_cls0.TIM0_CH0_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TIM0_CH0_IRQ_EN GTM_gtm_cls0.TIM0_CH0_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH0_IRQ_FORCINT GTM_gtm_cls0.TIM0_CH0_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TIM0_CH0_IRQ_MODE GTM_gtm_cls0.TIM0_CH0_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_TIM0_CH0_EIRQ_EN GTM_gtm_cls0.TIM0_CH0_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH1_GPR0 GTM_gtm_cls0.TIM0_CH1_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH1_GPR1 GTM_gtm_cls0.TIM0_CH1_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH1_CNT GTM_gtm_cls0.TIM0_CH1_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls0_TIM0_CH1_ECNT GTM_gtm_cls0.TIM0_CH1_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls0_TIM0_CH1_CNTS GTM_gtm_cls0.TIM0_CH1_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls0_TIM0_CH1_TDUC GTM_gtm_cls0.TIM0_CH1_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls0_TIM0_CH1_TDUV GTM_gtm_cls0.TIM0_CH1_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls0_TIM0_CH1_FLT_RE GTM_gtm_cls0.TIM0_CH1_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls0_TIM0_CH1_FLT_FE GTM_gtm_cls0.TIM0_CH1_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls0_TIM0_CH1_CTRL GTM_gtm_cls0.TIM0_CH1_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls0_TIM0_CH1_ECTRL GTM_gtm_cls0.TIM0_CH1_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls0_TIM0_CH1_IRQ_NOTIFY GTM_gtm_cls0.TIM0_CH1_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TIM0_CH1_IRQ_EN GTM_gtm_cls0.TIM0_CH1_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH1_IRQ_FORCINT GTM_gtm_cls0.TIM0_CH1_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TIM0_CH1_IRQ_MODE GTM_gtm_cls0.TIM0_CH1_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_TIM0_CH1_EIRQ_EN GTM_gtm_cls0.TIM0_CH1_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH2_GPR0 GTM_gtm_cls0.TIM0_CH2_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH2_GPR1 GTM_gtm_cls0.TIM0_CH2_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH2_CNT GTM_gtm_cls0.TIM0_CH2_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls0_TIM0_CH2_ECNT GTM_gtm_cls0.TIM0_CH2_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls0_TIM0_CH2_CNTS GTM_gtm_cls0.TIM0_CH2_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls0_TIM0_CH2_TDUC GTM_gtm_cls0.TIM0_CH2_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls0_TIM0_CH2_TDUV GTM_gtm_cls0.TIM0_CH2_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls0_TIM0_CH2_FLT_RE GTM_gtm_cls0.TIM0_CH2_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls0_TIM0_CH2_FLT_FE GTM_gtm_cls0.TIM0_CH2_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls0_TIM0_CH2_CTRL GTM_gtm_cls0.TIM0_CH2_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls0_TIM0_CH2_ECTRL GTM_gtm_cls0.TIM0_CH2_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls0_TIM0_CH2_IRQ_NOTIFY GTM_gtm_cls0.TIM0_CH2_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TIM0_CH2_IRQ_EN GTM_gtm_cls0.TIM0_CH2_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH2_IRQ_FORCINT GTM_gtm_cls0.TIM0_CH2_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TIM0_CH2_IRQ_MODE GTM_gtm_cls0.TIM0_CH2_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_TIM0_CH2_EIRQ_EN GTM_gtm_cls0.TIM0_CH2_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH3_GPR0 GTM_gtm_cls0.TIM0_CH3_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH3_GPR1 GTM_gtm_cls0.TIM0_CH3_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH3_CNT GTM_gtm_cls0.TIM0_CH3_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls0_TIM0_CH3_ECNT GTM_gtm_cls0.TIM0_CH3_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls0_TIM0_CH3_CNTS GTM_gtm_cls0.TIM0_CH3_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls0_TIM0_CH3_TDUC GTM_gtm_cls0.TIM0_CH3_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls0_TIM0_CH3_TDUV GTM_gtm_cls0.TIM0_CH3_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls0_TIM0_CH3_FLT_RE GTM_gtm_cls0.TIM0_CH3_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls0_TIM0_CH3_FLT_FE GTM_gtm_cls0.TIM0_CH3_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls0_TIM0_CH3_CTRL GTM_gtm_cls0.TIM0_CH3_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls0_TIM0_CH3_ECTRL GTM_gtm_cls0.TIM0_CH3_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls0_TIM0_CH3_IRQ_NOTIFY GTM_gtm_cls0.TIM0_CH3_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TIM0_CH3_IRQ_EN GTM_gtm_cls0.TIM0_CH3_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH3_IRQ_FORCINT GTM_gtm_cls0.TIM0_CH3_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TIM0_CH3_IRQ_MODE GTM_gtm_cls0.TIM0_CH3_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_TIM0_CH3_EIRQ_EN GTM_gtm_cls0.TIM0_CH3_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH4_GPR0 GTM_gtm_cls0.TIM0_CH4_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH4_GPR1 GTM_gtm_cls0.TIM0_CH4_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH4_CNT GTM_gtm_cls0.TIM0_CH4_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls0_TIM0_CH4_ECNT GTM_gtm_cls0.TIM0_CH4_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls0_TIM0_CH4_CNTS GTM_gtm_cls0.TIM0_CH4_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls0_TIM0_CH4_TDUC GTM_gtm_cls0.TIM0_CH4_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls0_TIM0_CH4_TDUV GTM_gtm_cls0.TIM0_CH4_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls0_TIM0_CH4_FLT_RE GTM_gtm_cls0.TIM0_CH4_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls0_TIM0_CH4_FLT_FE GTM_gtm_cls0.TIM0_CH4_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls0_TIM0_CH4_CTRL GTM_gtm_cls0.TIM0_CH4_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls0_TIM0_CH4_ECTRL GTM_gtm_cls0.TIM0_CH4_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls0_TIM0_CH4_IRQ_NOTIFY GTM_gtm_cls0.TIM0_CH4_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TIM0_CH4_IRQ_EN GTM_gtm_cls0.TIM0_CH4_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH4_IRQ_FORCINT GTM_gtm_cls0.TIM0_CH4_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TIM0_CH4_IRQ_MODE GTM_gtm_cls0.TIM0_CH4_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_TIM0_CH4_EIRQ_EN GTM_gtm_cls0.TIM0_CH4_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH5_GPR0 GTM_gtm_cls0.TIM0_CH5_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH5_GPR1 GTM_gtm_cls0.TIM0_CH5_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH5_CNT GTM_gtm_cls0.TIM0_CH5_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls0_TIM0_CH5_ECNT GTM_gtm_cls0.TIM0_CH5_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls0_TIM0_CH5_CNTS GTM_gtm_cls0.TIM0_CH5_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls0_TIM0_CH5_TDUC GTM_gtm_cls0.TIM0_CH5_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls0_TIM0_CH5_TDUV GTM_gtm_cls0.TIM0_CH5_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls0_TIM0_CH5_FLT_RE GTM_gtm_cls0.TIM0_CH5_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls0_TIM0_CH5_FLT_FE GTM_gtm_cls0.TIM0_CH5_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls0_TIM0_CH5_CTRL GTM_gtm_cls0.TIM0_CH5_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls0_TIM0_CH5_ECTRL GTM_gtm_cls0.TIM0_CH5_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls0_TIM0_CH5_IRQ_NOTIFY GTM_gtm_cls0.TIM0_CH5_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TIM0_CH5_IRQ_EN GTM_gtm_cls0.TIM0_CH5_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH5_IRQ_FORCINT GTM_gtm_cls0.TIM0_CH5_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TIM0_CH5_IRQ_MODE GTM_gtm_cls0.TIM0_CH5_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_TIM0_CH5_EIRQ_EN GTM_gtm_cls0.TIM0_CH5_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH6_GPR0 GTM_gtm_cls0.TIM0_CH6_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH6_GPR1 GTM_gtm_cls0.TIM0_CH6_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH6_CNT GTM_gtm_cls0.TIM0_CH6_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls0_TIM0_CH6_ECNT GTM_gtm_cls0.TIM0_CH6_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls0_TIM0_CH6_CNTS GTM_gtm_cls0.TIM0_CH6_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls0_TIM0_CH6_TDUC GTM_gtm_cls0.TIM0_CH6_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls0_TIM0_CH6_TDUV GTM_gtm_cls0.TIM0_CH6_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls0_TIM0_CH6_FLT_RE GTM_gtm_cls0.TIM0_CH6_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls0_TIM0_CH6_FLT_FE GTM_gtm_cls0.TIM0_CH6_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls0_TIM0_CH6_CTRL GTM_gtm_cls0.TIM0_CH6_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls0_TIM0_CH6_ECTRL GTM_gtm_cls0.TIM0_CH6_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls0_TIM0_CH6_IRQ_NOTIFY GTM_gtm_cls0.TIM0_CH6_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TIM0_CH6_IRQ_EN GTM_gtm_cls0.TIM0_CH6_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH6_IRQ_FORCINT GTM_gtm_cls0.TIM0_CH6_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TIM0_CH6_IRQ_MODE GTM_gtm_cls0.TIM0_CH6_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_TIM0_CH6_EIRQ_EN GTM_gtm_cls0.TIM0_CH6_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH7_GPR0 GTM_gtm_cls0.TIM0_CH7_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH7_GPR1 GTM_gtm_cls0.TIM0_CH7_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls0_TIM0_CH7_CNT GTM_gtm_cls0.TIM0_CH7_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls0_TIM0_CH7_ECNT GTM_gtm_cls0.TIM0_CH7_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls0_TIM0_CH7_CNTS GTM_gtm_cls0.TIM0_CH7_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls0_TIM0_CH7_TDUC GTM_gtm_cls0.TIM0_CH7_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls0_TIM0_CH7_TDUV GTM_gtm_cls0.TIM0_CH7_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls0_TIM0_CH7_FLT_RE GTM_gtm_cls0.TIM0_CH7_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls0_TIM0_CH7_FLT_FE GTM_gtm_cls0.TIM0_CH7_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls0_TIM0_CH7_CTRL GTM_gtm_cls0.TIM0_CH7_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls0_TIM0_CH7_ECTRL GTM_gtm_cls0.TIM0_CH7_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls0_TIM0_CH7_IRQ_NOTIFY GTM_gtm_cls0.TIM0_CH7_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TIM0_CH7_IRQ_EN GTM_gtm_cls0.TIM0_CH7_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TIM0_CH7_IRQ_FORCINT GTM_gtm_cls0.TIM0_CH7_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TIM0_CH7_IRQ_MODE GTM_gtm_cls0.TIM0_CH7_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_TIM0_CH7_EIRQ_EN GTM_gtm_cls0.TIM0_CH7_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_TIM0_INP_VAL GTM_gtm_cls0.TIM0_INP_VAL.R /* TIM[i] input value observation register */
#define GTM_gtm_cls0_TIM0_IN_SRC GTM_gtm_cls0.TIM0_IN_SRC.R /* TIM[i] AUX IN source selection register */
#define GTM_gtm_cls0_TIM0_RST GTM_gtm_cls0.TIM0_RST.R      /* TIM[i] global software reset register */
#define GTM_gtm_cls0_TOM0_CH0_CTRL GTM_gtm_cls0.TOM0_CH0_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH0_SR0 GTM_gtm_cls0.TOM0_CH0_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH0_SR1 GTM_gtm_cls0.TOM0_CH0_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH0_CM0 GTM_gtm_cls0.TOM0_CH0_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH0_CM1 GTM_gtm_cls0.TOM0_CH0_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH0_CN0 GTM_gtm_cls0.TOM0_CH0_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH0_STAT GTM_gtm_cls0.TOM0_CH0_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH0_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH0_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH0_IRQ_EN GTM_gtm_cls0.TOM0_CH0_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH0_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH0_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH0_IRQ_MODE GTM_gtm_cls0.TOM0_CH0_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH0_CTRL_SR GTM_gtm_cls0.TOM0_CH0_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH1_CTRL GTM_gtm_cls0.TOM0_CH1_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH1_SR0 GTM_gtm_cls0.TOM0_CH1_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH1_SR1 GTM_gtm_cls0.TOM0_CH1_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH1_CM0 GTM_gtm_cls0.TOM0_CH1_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH1_CM1 GTM_gtm_cls0.TOM0_CH1_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH1_CN0 GTM_gtm_cls0.TOM0_CH1_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH1_STAT GTM_gtm_cls0.TOM0_CH1_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH1_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH1_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH1_IRQ_EN GTM_gtm_cls0.TOM0_CH1_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH1_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH1_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH1_IRQ_MODE GTM_gtm_cls0.TOM0_CH1_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH1_CTRL_SR GTM_gtm_cls0.TOM0_CH1_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH2_CTRL GTM_gtm_cls0.TOM0_CH2_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH2_SR0 GTM_gtm_cls0.TOM0_CH2_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH2_SR1 GTM_gtm_cls0.TOM0_CH2_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH2_CM0 GTM_gtm_cls0.TOM0_CH2_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH2_CM1 GTM_gtm_cls0.TOM0_CH2_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH2_CN0 GTM_gtm_cls0.TOM0_CH2_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH2_STAT GTM_gtm_cls0.TOM0_CH2_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH2_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH2_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH2_IRQ_EN GTM_gtm_cls0.TOM0_CH2_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH2_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH2_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH2_IRQ_MODE GTM_gtm_cls0.TOM0_CH2_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH2_CTRL_SR GTM_gtm_cls0.TOM0_CH2_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH3_CTRL GTM_gtm_cls0.TOM0_CH3_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH3_SR0 GTM_gtm_cls0.TOM0_CH3_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH3_SR1 GTM_gtm_cls0.TOM0_CH3_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH3_CM0 GTM_gtm_cls0.TOM0_CH3_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH3_CM1 GTM_gtm_cls0.TOM0_CH3_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH3_CN0 GTM_gtm_cls0.TOM0_CH3_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH3_STAT GTM_gtm_cls0.TOM0_CH3_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH3_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH3_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH3_IRQ_EN GTM_gtm_cls0.TOM0_CH3_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH3_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH3_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH3_IRQ_MODE GTM_gtm_cls0.TOM0_CH3_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH3_CTRL_SR GTM_gtm_cls0.TOM0_CH3_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH4_CTRL GTM_gtm_cls0.TOM0_CH4_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH4_SR0 GTM_gtm_cls0.TOM0_CH4_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH4_SR1 GTM_gtm_cls0.TOM0_CH4_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH4_CM0 GTM_gtm_cls0.TOM0_CH4_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH4_CM1 GTM_gtm_cls0.TOM0_CH4_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH4_CN0 GTM_gtm_cls0.TOM0_CH4_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH4_STAT GTM_gtm_cls0.TOM0_CH4_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH4_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH4_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH4_IRQ_EN GTM_gtm_cls0.TOM0_CH4_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH4_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH4_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH4_IRQ_MODE GTM_gtm_cls0.TOM0_CH4_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH4_CTRL_SR GTM_gtm_cls0.TOM0_CH4_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH5_CTRL GTM_gtm_cls0.TOM0_CH5_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH5_SR0 GTM_gtm_cls0.TOM0_CH5_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH5_SR1 GTM_gtm_cls0.TOM0_CH5_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH5_CM0 GTM_gtm_cls0.TOM0_CH5_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH5_CM1 GTM_gtm_cls0.TOM0_CH5_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH5_CN0 GTM_gtm_cls0.TOM0_CH5_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH5_STAT GTM_gtm_cls0.TOM0_CH5_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH5_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH5_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH5_IRQ_EN GTM_gtm_cls0.TOM0_CH5_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH5_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH5_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH5_IRQ_MODE GTM_gtm_cls0.TOM0_CH5_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH5_CTRL_SR GTM_gtm_cls0.TOM0_CH5_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH6_CTRL GTM_gtm_cls0.TOM0_CH6_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH6_SR0 GTM_gtm_cls0.TOM0_CH6_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH6_SR1 GTM_gtm_cls0.TOM0_CH6_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH6_CM0 GTM_gtm_cls0.TOM0_CH6_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH6_CM1 GTM_gtm_cls0.TOM0_CH6_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH6_CN0 GTM_gtm_cls0.TOM0_CH6_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH6_STAT GTM_gtm_cls0.TOM0_CH6_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH6_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH6_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH6_IRQ_EN GTM_gtm_cls0.TOM0_CH6_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH6_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH6_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH6_IRQ_MODE GTM_gtm_cls0.TOM0_CH6_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH6_CTRL_SR GTM_gtm_cls0.TOM0_CH6_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH7_CTRL GTM_gtm_cls0.TOM0_CH7_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH7_SR0 GTM_gtm_cls0.TOM0_CH7_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH7_SR1 GTM_gtm_cls0.TOM0_CH7_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH7_CM0 GTM_gtm_cls0.TOM0_CH7_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH7_CM1 GTM_gtm_cls0.TOM0_CH7_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH7_CN0 GTM_gtm_cls0.TOM0_CH7_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH7_STAT GTM_gtm_cls0.TOM0_CH7_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH7_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH7_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH7_IRQ_EN GTM_gtm_cls0.TOM0_CH7_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH7_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH7_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH7_IRQ_MODE GTM_gtm_cls0.TOM0_CH7_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH7_CTRL_SR GTM_gtm_cls0.TOM0_CH7_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH8_CTRL GTM_gtm_cls0.TOM0_CH8_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH8_SR0 GTM_gtm_cls0.TOM0_CH8_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH8_SR1 GTM_gtm_cls0.TOM0_CH8_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH8_CM0 GTM_gtm_cls0.TOM0_CH8_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH8_CM1 GTM_gtm_cls0.TOM0_CH8_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH8_CN0 GTM_gtm_cls0.TOM0_CH8_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH8_STAT GTM_gtm_cls0.TOM0_CH8_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH8_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH8_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH8_IRQ_EN GTM_gtm_cls0.TOM0_CH8_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH8_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH8_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH8_IRQ_MODE GTM_gtm_cls0.TOM0_CH8_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH8_CTRL_SR GTM_gtm_cls0.TOM0_CH8_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH9_CTRL GTM_gtm_cls0.TOM0_CH9_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH9_SR0 GTM_gtm_cls0.TOM0_CH9_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH9_SR1 GTM_gtm_cls0.TOM0_CH9_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH9_CM0 GTM_gtm_cls0.TOM0_CH9_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH9_CM1 GTM_gtm_cls0.TOM0_CH9_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH9_CN0 GTM_gtm_cls0.TOM0_CH9_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH9_STAT GTM_gtm_cls0.TOM0_CH9_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH9_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH9_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH9_IRQ_EN GTM_gtm_cls0.TOM0_CH9_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH9_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH9_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH9_IRQ_MODE GTM_gtm_cls0.TOM0_CH9_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH9_CTRL_SR GTM_gtm_cls0.TOM0_CH9_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH10_CTRL GTM_gtm_cls0.TOM0_CH10_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH10_SR0 GTM_gtm_cls0.TOM0_CH10_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH10_SR1 GTM_gtm_cls0.TOM0_CH10_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH10_CM0 GTM_gtm_cls0.TOM0_CH10_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH10_CM1 GTM_gtm_cls0.TOM0_CH10_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH10_CN0 GTM_gtm_cls0.TOM0_CH10_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH10_STAT GTM_gtm_cls0.TOM0_CH10_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH10_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH10_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH10_IRQ_EN GTM_gtm_cls0.TOM0_CH10_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH10_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH10_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH10_IRQ_MODE GTM_gtm_cls0.TOM0_CH10_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH10_CTRL_SR GTM_gtm_cls0.TOM0_CH10_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH11_CTRL GTM_gtm_cls0.TOM0_CH11_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH11_SR0 GTM_gtm_cls0.TOM0_CH11_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH11_SR1 GTM_gtm_cls0.TOM0_CH11_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH11_CM0 GTM_gtm_cls0.TOM0_CH11_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH11_CM1 GTM_gtm_cls0.TOM0_CH11_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH11_CN0 GTM_gtm_cls0.TOM0_CH11_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH11_STAT GTM_gtm_cls0.TOM0_CH11_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH11_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH11_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH11_IRQ_EN GTM_gtm_cls0.TOM0_CH11_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH11_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH11_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH11_IRQ_MODE GTM_gtm_cls0.TOM0_CH11_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH11_CTRL_SR GTM_gtm_cls0.TOM0_CH11_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH12_CTRL GTM_gtm_cls0.TOM0_CH12_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH12_SR0 GTM_gtm_cls0.TOM0_CH12_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH12_SR1 GTM_gtm_cls0.TOM0_CH12_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH12_CM0 GTM_gtm_cls0.TOM0_CH12_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH12_CM1 GTM_gtm_cls0.TOM0_CH12_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH12_CN0 GTM_gtm_cls0.TOM0_CH12_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH12_STAT GTM_gtm_cls0.TOM0_CH12_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH12_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH12_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH12_IRQ_EN GTM_gtm_cls0.TOM0_CH12_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH12_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH12_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH12_IRQ_MODE GTM_gtm_cls0.TOM0_CH12_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH12_CTRL_SR GTM_gtm_cls0.TOM0_CH12_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH13_CTRL GTM_gtm_cls0.TOM0_CH13_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH13_SR0 GTM_gtm_cls0.TOM0_CH13_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH13_SR1 GTM_gtm_cls0.TOM0_CH13_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH13_CM0 GTM_gtm_cls0.TOM0_CH13_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH13_CM1 GTM_gtm_cls0.TOM0_CH13_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH13_CN0 GTM_gtm_cls0.TOM0_CH13_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH13_STAT GTM_gtm_cls0.TOM0_CH13_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH13_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH13_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH13_IRQ_EN GTM_gtm_cls0.TOM0_CH13_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH13_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH13_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH13_IRQ_MODE GTM_gtm_cls0.TOM0_CH13_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH13_CTRL_SR GTM_gtm_cls0.TOM0_CH13_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH14_CTRL GTM_gtm_cls0.TOM0_CH14_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH14_SR0 GTM_gtm_cls0.TOM0_CH14_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH14_SR1 GTM_gtm_cls0.TOM0_CH14_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH14_CM0 GTM_gtm_cls0.TOM0_CH14_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH14_CM1 GTM_gtm_cls0.TOM0_CH14_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH14_CN0 GTM_gtm_cls0.TOM0_CH14_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH14_STAT GTM_gtm_cls0.TOM0_CH14_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH14_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH14_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH14_IRQ_EN GTM_gtm_cls0.TOM0_CH14_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH14_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH14_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH14_IRQ_MODE GTM_gtm_cls0.TOM0_CH14_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH14_CTRL_SR GTM_gtm_cls0.TOM0_CH14_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_CH15_CTRL GTM_gtm_cls0.TOM0_CH15_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls0_TOM0_CH15_SR0 GTM_gtm_cls0.TOM0_CH15_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH15_SR1 GTM_gtm_cls0.TOM0_CH15_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls0_TOM0_CH15_CM0 GTM_gtm_cls0.TOM0_CH15_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_TOM0_CH15_CM1 GTM_gtm_cls0.TOM0_CH15_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls0_TOM0_CH15_CN0 GTM_gtm_cls0.TOM0_CH15_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls0_TOM0_CH15_STAT GTM_gtm_cls0.TOM0_CH15_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls0_TOM0_CH15_IRQ_NOTIFY GTM_gtm_cls0.TOM0_CH15_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_TOM0_CH15_IRQ_EN GTM_gtm_cls0.TOM0_CH15_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_TOM0_CH15_IRQ_FORCINT GTM_gtm_cls0.TOM0_CH15_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_TOM0_CH15_IRQ_MODE GTM_gtm_cls0.TOM0_CH15_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls0_TOM0_CH15_CTRL_SR GTM_gtm_cls0.TOM0_CH15_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_TOM0_TGC0_GLB_CTRL GTM_gtm_cls0.TOM0_TGC0_GLB_CTRL.R /* TOM[i] TGC [g] global control register */
#define GTM_gtm_cls0_TOM0_TGC0_ACT_TB GTM_gtm_cls0.TOM0_TGC0_ACT_TB.R /* TOM[i] TGC [g] action time base register */
#define GTM_gtm_cls0_TOM0_TGC0_FUPD_CTRL GTM_gtm_cls0.TOM0_TGC0_FUPD_CTRL.R /* TOM[i] TGC [g] force update control register */
#define GTM_gtm_cls0_TOM0_TGC0_INT_TRIG GTM_gtm_cls0.TOM0_TGC0_INT_TRIG.R /* TOM[i] TGC [g] internal trigger control register */
#define GTM_gtm_cls0_TOM0_TGC0_ENDIS_CTRL GTM_gtm_cls0.TOM0_TGC0_ENDIS_CTRL.R /* TOM[i] TGC [g] enable/disable control register */
#define GTM_gtm_cls0_TOM0_TGC0_ENDIS_STAT GTM_gtm_cls0.TOM0_TGC0_ENDIS_STAT.R /* TOM[i] TGC [g] enable/disable status register */
#define GTM_gtm_cls0_TOM0_TGC0_OUTEN_CTRL GTM_gtm_cls0.TOM0_TGC0_OUTEN_CTRL.R /* TOM[i] TGC [g] output enable control register */
#define GTM_gtm_cls0_TOM0_TGC0_OUTEN_STAT GTM_gtm_cls0.TOM0_TGC0_OUTEN_STAT.R /* TOM[i] TGC [g] output enable status register */
#define GTM_gtm_cls0_TOM0_TGC1_GLB_CTRL GTM_gtm_cls0.TOM0_TGC1_GLB_CTRL.R /* TOM[i] TGC [g] global control register */
#define GTM_gtm_cls0_TOM0_TGC1_ACT_TB GTM_gtm_cls0.TOM0_TGC1_ACT_TB.R /* TOM[i] TGC [g] action time base register */
#define GTM_gtm_cls0_TOM0_TGC1_FUPD_CTRL GTM_gtm_cls0.TOM0_TGC1_FUPD_CTRL.R /* TOM[i] TGC [g] force update control register */
#define GTM_gtm_cls0_TOM0_TGC1_INT_TRIG GTM_gtm_cls0.TOM0_TGC1_INT_TRIG.R /* TOM[i] TGC [g] internal trigger control register */
#define GTM_gtm_cls0_TOM0_TGC1_ENDIS_CTRL GTM_gtm_cls0.TOM0_TGC1_ENDIS_CTRL.R /* TOM[i] TGC [g] enable/disable control register */
#define GTM_gtm_cls0_TOM0_TGC1_ENDIS_STAT GTM_gtm_cls0.TOM0_TGC1_ENDIS_STAT.R /* TOM[i] TGC [g] enable/disable status register */
#define GTM_gtm_cls0_TOM0_TGC1_OUTEN_CTRL GTM_gtm_cls0.TOM0_TGC1_OUTEN_CTRL.R /* TOM[i] TGC [g] output enable control register */
#define GTM_gtm_cls0_TOM0_TGC1_OUTEN_STAT GTM_gtm_cls0.TOM0_TGC1_OUTEN_STAT.R /* TOM[i] TGC [g] output enable status register */
#define GTM_gtm_cls0_ATOM0_CH0_RDADDR GTM_gtm_cls0.ATOM0_CH0_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls0_ATOM0_CH0_CTRL GTM_gtm_cls0.ATOM0_CH0_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls0_ATOM0_CH0_SR0 GTM_gtm_cls0.ATOM0_CH0_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH0_SR1 GTM_gtm_cls0.ATOM0_CH0_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH0_CM0 GTM_gtm_cls0.ATOM0_CH0_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH0_CM1 GTM_gtm_cls0.ATOM0_CH0_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH0_CN0 GTM_gtm_cls0.ATOM0_CH0_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls0_ATOM0_CH0_STAT GTM_gtm_cls0.ATOM0_CH0_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls0_ATOM0_CH0_IRQ_NOTIFY GTM_gtm_cls0.ATOM0_CH0_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_ATOM0_CH0_IRQ_EN GTM_gtm_cls0.ATOM0_CH0_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_ATOM0_CH0_IRQ_FORCINT GTM_gtm_cls0.ATOM0_CH0_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls0_ATOM0_CH0_IRQ_MODE GTM_gtm_cls0.ATOM0_CH0_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_ATOM0_CH0_CTRL_SR GTM_gtm_cls0.ATOM0_CH0_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_ATOM0_CH1_RDADDR GTM_gtm_cls0.ATOM0_CH1_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls0_ATOM0_CH1_CTRL GTM_gtm_cls0.ATOM0_CH1_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls0_ATOM0_CH1_SR0 GTM_gtm_cls0.ATOM0_CH1_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH1_SR1 GTM_gtm_cls0.ATOM0_CH1_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH1_CM0 GTM_gtm_cls0.ATOM0_CH1_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH1_CM1 GTM_gtm_cls0.ATOM0_CH1_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH1_CN0 GTM_gtm_cls0.ATOM0_CH1_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls0_ATOM0_CH1_STAT GTM_gtm_cls0.ATOM0_CH1_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls0_ATOM0_CH1_IRQ_NOTIFY GTM_gtm_cls0.ATOM0_CH1_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_ATOM0_CH1_IRQ_EN GTM_gtm_cls0.ATOM0_CH1_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_ATOM0_CH1_IRQ_FORCINT GTM_gtm_cls0.ATOM0_CH1_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls0_ATOM0_CH1_IRQ_MODE GTM_gtm_cls0.ATOM0_CH1_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_ATOM0_CH1_CTRL_SR GTM_gtm_cls0.ATOM0_CH1_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_ATOM0_CH2_RDADDR GTM_gtm_cls0.ATOM0_CH2_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls0_ATOM0_CH2_CTRL GTM_gtm_cls0.ATOM0_CH2_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls0_ATOM0_CH2_SR0 GTM_gtm_cls0.ATOM0_CH2_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH2_SR1 GTM_gtm_cls0.ATOM0_CH2_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH2_CM0 GTM_gtm_cls0.ATOM0_CH2_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH2_CM1 GTM_gtm_cls0.ATOM0_CH2_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH2_CN0 GTM_gtm_cls0.ATOM0_CH2_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls0_ATOM0_CH2_STAT GTM_gtm_cls0.ATOM0_CH2_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls0_ATOM0_CH2_IRQ_NOTIFY GTM_gtm_cls0.ATOM0_CH2_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_ATOM0_CH2_IRQ_EN GTM_gtm_cls0.ATOM0_CH2_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_ATOM0_CH2_IRQ_FORCINT GTM_gtm_cls0.ATOM0_CH2_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls0_ATOM0_CH2_IRQ_MODE GTM_gtm_cls0.ATOM0_CH2_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_ATOM0_CH2_CTRL_SR GTM_gtm_cls0.ATOM0_CH2_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_ATOM0_CH3_RDADDR GTM_gtm_cls0.ATOM0_CH3_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls0_ATOM0_CH3_CTRL GTM_gtm_cls0.ATOM0_CH3_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls0_ATOM0_CH3_SR0 GTM_gtm_cls0.ATOM0_CH3_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH3_SR1 GTM_gtm_cls0.ATOM0_CH3_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH3_CM0 GTM_gtm_cls0.ATOM0_CH3_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH3_CM1 GTM_gtm_cls0.ATOM0_CH3_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH3_CN0 GTM_gtm_cls0.ATOM0_CH3_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls0_ATOM0_CH3_STAT GTM_gtm_cls0.ATOM0_CH3_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls0_ATOM0_CH3_IRQ_NOTIFY GTM_gtm_cls0.ATOM0_CH3_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_ATOM0_CH3_IRQ_EN GTM_gtm_cls0.ATOM0_CH3_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_ATOM0_CH3_IRQ_FORCINT GTM_gtm_cls0.ATOM0_CH3_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls0_ATOM0_CH3_IRQ_MODE GTM_gtm_cls0.ATOM0_CH3_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_ATOM0_CH3_CTRL_SR GTM_gtm_cls0.ATOM0_CH3_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_ATOM0_CH4_RDADDR GTM_gtm_cls0.ATOM0_CH4_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls0_ATOM0_CH4_CTRL GTM_gtm_cls0.ATOM0_CH4_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls0_ATOM0_CH4_SR0 GTM_gtm_cls0.ATOM0_CH4_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH4_SR1 GTM_gtm_cls0.ATOM0_CH4_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH4_CM0 GTM_gtm_cls0.ATOM0_CH4_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH4_CM1 GTM_gtm_cls0.ATOM0_CH4_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH4_CN0 GTM_gtm_cls0.ATOM0_CH4_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls0_ATOM0_CH4_STAT GTM_gtm_cls0.ATOM0_CH4_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls0_ATOM0_CH4_IRQ_NOTIFY GTM_gtm_cls0.ATOM0_CH4_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_ATOM0_CH4_IRQ_EN GTM_gtm_cls0.ATOM0_CH4_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_ATOM0_CH4_IRQ_FORCINT GTM_gtm_cls0.ATOM0_CH4_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls0_ATOM0_CH4_IRQ_MODE GTM_gtm_cls0.ATOM0_CH4_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_ATOM0_CH4_CTRL_SR GTM_gtm_cls0.ATOM0_CH4_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_ATOM0_CH5_RDADDR GTM_gtm_cls0.ATOM0_CH5_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls0_ATOM0_CH5_CTRL GTM_gtm_cls0.ATOM0_CH5_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls0_ATOM0_CH5_SR0 GTM_gtm_cls0.ATOM0_CH5_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH5_SR1 GTM_gtm_cls0.ATOM0_CH5_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH5_CM0 GTM_gtm_cls0.ATOM0_CH5_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH5_CM1 GTM_gtm_cls0.ATOM0_CH5_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH5_CN0 GTM_gtm_cls0.ATOM0_CH5_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls0_ATOM0_CH5_STAT GTM_gtm_cls0.ATOM0_CH5_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls0_ATOM0_CH5_IRQ_NOTIFY GTM_gtm_cls0.ATOM0_CH5_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_ATOM0_CH5_IRQ_EN GTM_gtm_cls0.ATOM0_CH5_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_ATOM0_CH5_IRQ_FORCINT GTM_gtm_cls0.ATOM0_CH5_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls0_ATOM0_CH5_IRQ_MODE GTM_gtm_cls0.ATOM0_CH5_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_ATOM0_CH5_CTRL_SR GTM_gtm_cls0.ATOM0_CH5_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_ATOM0_CH6_RDADDR GTM_gtm_cls0.ATOM0_CH6_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls0_ATOM0_CH6_CTRL GTM_gtm_cls0.ATOM0_CH6_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls0_ATOM0_CH6_SR0 GTM_gtm_cls0.ATOM0_CH6_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH6_SR1 GTM_gtm_cls0.ATOM0_CH6_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH6_CM0 GTM_gtm_cls0.ATOM0_CH6_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH6_CM1 GTM_gtm_cls0.ATOM0_CH6_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH6_CN0 GTM_gtm_cls0.ATOM0_CH6_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls0_ATOM0_CH6_STAT GTM_gtm_cls0.ATOM0_CH6_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls0_ATOM0_CH6_IRQ_NOTIFY GTM_gtm_cls0.ATOM0_CH6_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_ATOM0_CH6_IRQ_EN GTM_gtm_cls0.ATOM0_CH6_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_ATOM0_CH6_IRQ_FORCINT GTM_gtm_cls0.ATOM0_CH6_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls0_ATOM0_CH6_IRQ_MODE GTM_gtm_cls0.ATOM0_CH6_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_ATOM0_CH6_CTRL_SR GTM_gtm_cls0.ATOM0_CH6_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_ATOM0_CH7_RDADDR GTM_gtm_cls0.ATOM0_CH7_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls0_ATOM0_CH7_CTRL GTM_gtm_cls0.ATOM0_CH7_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls0_ATOM0_CH7_SR0 GTM_gtm_cls0.ATOM0_CH7_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH7_SR1 GTM_gtm_cls0.ATOM0_CH7_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls0_ATOM0_CH7_CM0 GTM_gtm_cls0.ATOM0_CH7_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH7_CM1 GTM_gtm_cls0.ATOM0_CH7_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls0_ATOM0_CH7_CN0 GTM_gtm_cls0.ATOM0_CH7_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls0_ATOM0_CH7_STAT GTM_gtm_cls0.ATOM0_CH7_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls0_ATOM0_CH7_IRQ_NOTIFY GTM_gtm_cls0.ATOM0_CH7_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_ATOM0_CH7_IRQ_EN GTM_gtm_cls0.ATOM0_CH7_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_ATOM0_CH7_IRQ_FORCINT GTM_gtm_cls0.ATOM0_CH7_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls0_ATOM0_CH7_IRQ_MODE GTM_gtm_cls0.ATOM0_CH7_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls0_ATOM0_CH7_CTRL_SR GTM_gtm_cls0.ATOM0_CH7_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls0_ATOM0_AGC_GLB_CTRL GTM_gtm_cls0.ATOM0_AGC_GLB_CTRL.R /* ATOM[i] AGC global control register */
#define GTM_gtm_cls0_ATOM0_AGC_ENDIS_CTRL GTM_gtm_cls0.ATOM0_AGC_ENDIS_CTRL.R /* ATOM[i] AGC enable/disable control register */
#define GTM_gtm_cls0_ATOM0_AGC_ENDIS_STAT GTM_gtm_cls0.ATOM0_AGC_ENDIS_STAT.R /* ATOM[i] AGC enable/disable status register */
#define GTM_gtm_cls0_ATOM0_AGC_ACT_TB GTM_gtm_cls0.ATOM0_AGC_ACT_TB.R /* ATOM[i] AGC action time base register */
#define GTM_gtm_cls0_ATOM0_AGC_OUTEN_CTRL GTM_gtm_cls0.ATOM0_AGC_OUTEN_CTRL.R /* ATOM[i] AGC output enable control register */
#define GTM_gtm_cls0_ATOM0_AGC_OUTEN_STAT GTM_gtm_cls0.ATOM0_AGC_OUTEN_STAT.R /* ATOM[i] AGC output enable status register */
#define GTM_gtm_cls0_ATOM0_AGC_FUPD_CTRL GTM_gtm_cls0.ATOM0_AGC_FUPD_CTRL.R /* ATOM[i] AGC force update control register */
#define GTM_gtm_cls0_ATOM0_AGC_INT_TRIG GTM_gtm_cls0.ATOM0_AGC_INT_TRIG.R /* ATOM[i] AGC internal trigger control register */
#define GTM_gtm_cls0_MCS0_CH0_R0 GTM_gtm_cls0.MCS0_CH0_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH0_R1 GTM_gtm_cls0.MCS0_CH0_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH0_R2 GTM_gtm_cls0.MCS0_CH0_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH0_R3 GTM_gtm_cls0.MCS0_CH0_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH0_R4 GTM_gtm_cls0.MCS0_CH0_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH0_R5 GTM_gtm_cls0.MCS0_CH0_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH0_R6 GTM_gtm_cls0.MCS0_CH0_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH0_R7 GTM_gtm_cls0.MCS0_CH0_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH0_CTRL GTM_gtm_cls0.MCS0_CH0_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls0_MCS0_CH0_ACB GTM_gtm_cls0.MCS0_CH0_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls0_MCS0_CH0_MHB GTM_gtm_cls0.MCS0_CH0_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls0_MCS0_CH0_PC GTM_gtm_cls0.MCS0_CH0_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls0_MCS0_CH0_IRQ_NOTIFY GTM_gtm_cls0.MCS0_CH0_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls0_MCS0_CH0_IRQ_EN GTM_gtm_cls0.MCS0_CH0_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH0_IRQ_FORCINT GTM_gtm_cls0.MCS0_CH0_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls0_MCS0_CH0_IRQ_MODE GTM_gtm_cls0.MCS0_CH0_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls0_MCS0_CH0_EIRQ_EN GTM_gtm_cls0.MCS0_CH0_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH1_R0 GTM_gtm_cls0.MCS0_CH1_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH1_R1 GTM_gtm_cls0.MCS0_CH1_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH1_R2 GTM_gtm_cls0.MCS0_CH1_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH1_R3 GTM_gtm_cls0.MCS0_CH1_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH1_R4 GTM_gtm_cls0.MCS0_CH1_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH1_R5 GTM_gtm_cls0.MCS0_CH1_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH1_R6 GTM_gtm_cls0.MCS0_CH1_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH1_R7 GTM_gtm_cls0.MCS0_CH1_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH1_CTRL GTM_gtm_cls0.MCS0_CH1_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls0_MCS0_CH1_ACB GTM_gtm_cls0.MCS0_CH1_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls0_MCS0_CH1_MHB GTM_gtm_cls0.MCS0_CH1_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls0_MCS0_CH1_PC GTM_gtm_cls0.MCS0_CH1_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls0_MCS0_CH1_IRQ_NOTIFY GTM_gtm_cls0.MCS0_CH1_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls0_MCS0_CH1_IRQ_EN GTM_gtm_cls0.MCS0_CH1_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH1_IRQ_FORCINT GTM_gtm_cls0.MCS0_CH1_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls0_MCS0_CH1_IRQ_MODE GTM_gtm_cls0.MCS0_CH1_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls0_MCS0_CH1_EIRQ_EN GTM_gtm_cls0.MCS0_CH1_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH2_R0 GTM_gtm_cls0.MCS0_CH2_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH2_R1 GTM_gtm_cls0.MCS0_CH2_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH2_R2 GTM_gtm_cls0.MCS0_CH2_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH2_R3 GTM_gtm_cls0.MCS0_CH2_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH2_R4 GTM_gtm_cls0.MCS0_CH2_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH2_R5 GTM_gtm_cls0.MCS0_CH2_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH2_R6 GTM_gtm_cls0.MCS0_CH2_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH2_R7 GTM_gtm_cls0.MCS0_CH2_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH2_CTRL GTM_gtm_cls0.MCS0_CH2_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls0_MCS0_CH2_ACB GTM_gtm_cls0.MCS0_CH2_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls0_MCS0_CH2_MHB GTM_gtm_cls0.MCS0_CH2_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls0_MCS0_CH2_PC GTM_gtm_cls0.MCS0_CH2_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls0_MCS0_CH2_IRQ_NOTIFY GTM_gtm_cls0.MCS0_CH2_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls0_MCS0_CH2_IRQ_EN GTM_gtm_cls0.MCS0_CH2_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH2_IRQ_FORCINT GTM_gtm_cls0.MCS0_CH2_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls0_MCS0_CH2_IRQ_MODE GTM_gtm_cls0.MCS0_CH2_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls0_MCS0_CH2_EIRQ_EN GTM_gtm_cls0.MCS0_CH2_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH3_R0 GTM_gtm_cls0.MCS0_CH3_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH3_R1 GTM_gtm_cls0.MCS0_CH3_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH3_R2 GTM_gtm_cls0.MCS0_CH3_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH3_R3 GTM_gtm_cls0.MCS0_CH3_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH3_R4 GTM_gtm_cls0.MCS0_CH3_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH3_R5 GTM_gtm_cls0.MCS0_CH3_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH3_R6 GTM_gtm_cls0.MCS0_CH3_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH3_R7 GTM_gtm_cls0.MCS0_CH3_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH3_CTRL GTM_gtm_cls0.MCS0_CH3_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls0_MCS0_CH3_ACB GTM_gtm_cls0.MCS0_CH3_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls0_MCS0_CH3_MHB GTM_gtm_cls0.MCS0_CH3_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls0_MCS0_CH3_PC GTM_gtm_cls0.MCS0_CH3_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls0_MCS0_CH3_IRQ_NOTIFY GTM_gtm_cls0.MCS0_CH3_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls0_MCS0_CH3_IRQ_EN GTM_gtm_cls0.MCS0_CH3_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH3_IRQ_FORCINT GTM_gtm_cls0.MCS0_CH3_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls0_MCS0_CH3_IRQ_MODE GTM_gtm_cls0.MCS0_CH3_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls0_MCS0_CH3_EIRQ_EN GTM_gtm_cls0.MCS0_CH3_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH4_R0 GTM_gtm_cls0.MCS0_CH4_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH4_R1 GTM_gtm_cls0.MCS0_CH4_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH4_R2 GTM_gtm_cls0.MCS0_CH4_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH4_R3 GTM_gtm_cls0.MCS0_CH4_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH4_R4 GTM_gtm_cls0.MCS0_CH4_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH4_R5 GTM_gtm_cls0.MCS0_CH4_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH4_R6 GTM_gtm_cls0.MCS0_CH4_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH4_R7 GTM_gtm_cls0.MCS0_CH4_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH4_CTRL GTM_gtm_cls0.MCS0_CH4_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls0_MCS0_CH4_ACB GTM_gtm_cls0.MCS0_CH4_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls0_MCS0_CH4_MHB GTM_gtm_cls0.MCS0_CH4_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls0_MCS0_CH4_PC GTM_gtm_cls0.MCS0_CH4_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls0_MCS0_CH4_IRQ_NOTIFY GTM_gtm_cls0.MCS0_CH4_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls0_MCS0_CH4_IRQ_EN GTM_gtm_cls0.MCS0_CH4_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH4_IRQ_FORCINT GTM_gtm_cls0.MCS0_CH4_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls0_MCS0_CH4_IRQ_MODE GTM_gtm_cls0.MCS0_CH4_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls0_MCS0_CH4_EIRQ_EN GTM_gtm_cls0.MCS0_CH4_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH5_R0 GTM_gtm_cls0.MCS0_CH5_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH5_R1 GTM_gtm_cls0.MCS0_CH5_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH5_R2 GTM_gtm_cls0.MCS0_CH5_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH5_R3 GTM_gtm_cls0.MCS0_CH5_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH5_R4 GTM_gtm_cls0.MCS0_CH5_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH5_R5 GTM_gtm_cls0.MCS0_CH5_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH5_R6 GTM_gtm_cls0.MCS0_CH5_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH5_R7 GTM_gtm_cls0.MCS0_CH5_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH5_CTRL GTM_gtm_cls0.MCS0_CH5_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls0_MCS0_CH5_ACB GTM_gtm_cls0.MCS0_CH5_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls0_MCS0_CH5_MHB GTM_gtm_cls0.MCS0_CH5_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls0_MCS0_CH5_PC GTM_gtm_cls0.MCS0_CH5_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls0_MCS0_CH5_IRQ_NOTIFY GTM_gtm_cls0.MCS0_CH5_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls0_MCS0_CH5_IRQ_EN GTM_gtm_cls0.MCS0_CH5_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH5_IRQ_FORCINT GTM_gtm_cls0.MCS0_CH5_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls0_MCS0_CH5_IRQ_MODE GTM_gtm_cls0.MCS0_CH5_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls0_MCS0_CH5_EIRQ_EN GTM_gtm_cls0.MCS0_CH5_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH6_R0 GTM_gtm_cls0.MCS0_CH6_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH6_R1 GTM_gtm_cls0.MCS0_CH6_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH6_R2 GTM_gtm_cls0.MCS0_CH6_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH6_R3 GTM_gtm_cls0.MCS0_CH6_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH6_R4 GTM_gtm_cls0.MCS0_CH6_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH6_R5 GTM_gtm_cls0.MCS0_CH6_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH6_R6 GTM_gtm_cls0.MCS0_CH6_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH6_R7 GTM_gtm_cls0.MCS0_CH6_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH6_CTRL GTM_gtm_cls0.MCS0_CH6_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls0_MCS0_CH6_ACB GTM_gtm_cls0.MCS0_CH6_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls0_MCS0_CH6_MHB GTM_gtm_cls0.MCS0_CH6_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls0_MCS0_CH6_PC GTM_gtm_cls0.MCS0_CH6_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls0_MCS0_CH6_IRQ_NOTIFY GTM_gtm_cls0.MCS0_CH6_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls0_MCS0_CH6_IRQ_EN GTM_gtm_cls0.MCS0_CH6_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH6_IRQ_FORCINT GTM_gtm_cls0.MCS0_CH6_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls0_MCS0_CH6_IRQ_MODE GTM_gtm_cls0.MCS0_CH6_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls0_MCS0_CH6_EIRQ_EN GTM_gtm_cls0.MCS0_CH6_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH7_R0 GTM_gtm_cls0.MCS0_CH7_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH7_R1 GTM_gtm_cls0.MCS0_CH7_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH7_R2 GTM_gtm_cls0.MCS0_CH7_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH7_R3 GTM_gtm_cls0.MCS0_CH7_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH7_R4 GTM_gtm_cls0.MCS0_CH7_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH7_R5 GTM_gtm_cls0.MCS0_CH7_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH7_R6 GTM_gtm_cls0.MCS0_CH7_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH7_R7 GTM_gtm_cls0.MCS0_CH7_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls0_MCS0_CH7_CTRL GTM_gtm_cls0.MCS0_CH7_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls0_MCS0_CH7_ACB GTM_gtm_cls0.MCS0_CH7_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls0_MCS0_CH7_MHB GTM_gtm_cls0.MCS0_CH7_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls0_MCS0_CH7_PC GTM_gtm_cls0.MCS0_CH7_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls0_MCS0_CH7_IRQ_NOTIFY GTM_gtm_cls0.MCS0_CH7_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls0_MCS0_CH7_IRQ_EN GTM_gtm_cls0.MCS0_CH7_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls0_MCS0_CH7_IRQ_FORCINT GTM_gtm_cls0.MCS0_CH7_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls0_MCS0_CH7_IRQ_MODE GTM_gtm_cls0.MCS0_CH7_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls0_MCS0_CH7_EIRQ_EN GTM_gtm_cls0.MCS0_CH7_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls0_MCS0_CTRG GTM_gtm_cls0.MCS0_CTRG.R    /* MCS[i] clear trigger control register */
#define GTM_gtm_cls0_MCS0_STRG GTM_gtm_cls0.MCS0_STRG.R    /* MCS[i] set trigger control register */
#define GTM_gtm_cls0_MCS0_CTRL_STAT GTM_gtm_cls0.MCS0_CTRL_STAT.R /* MCS[i] control and status register */
#define GTM_gtm_cls0_MCS0_RESET GTM_gtm_cls0.MCS0_RESET.R  /* MCS[i] reset register */
#define GTM_gtm_cls0_MCS0_CAT GTM_gtm_cls0.MCS0_CAT.R      /* MCS[i] cancel ARU transfer instruction */
#define GTM_gtm_cls0_MCS0_CWT GTM_gtm_cls0.MCS0_CWT.R      /* MCS[i] cancel waiting instruction */
#define GTM_gtm_cls0_MCS0_ERR GTM_gtm_cls0.MCS0_ERR.R      /* MCS[i] error register */
#define GTM_gtm_cls0_MCS0_REG_PROT GTM_gtm_cls0.MCS0_REG_PROT.R /* MCS[i] write protection register */
#define GTM_gtm_cls0_MCS0_SINT_IRQ_NOTIFY GTM_gtm_cls0.MCS0_SINT_IRQ_NOTIFY.R /* MCS[i] shared interrupt notification register */
#define GTM_gtm_cls0_MCS0_SINT_IRQ_EN GTM_gtm_cls0.MCS0_SINT_IRQ_EN.R /* MCS[i] shared interrupt enable register */
#define GTM_gtm_cls0_MCS0_SINT_IRQ_FORCINT GTM_gtm_cls0.MCS0_SINT_IRQ_FORCINT.R /* MCS[i] force shared interrupt register */
#define GTM_gtm_cls0_MCS0_SINT_IRQ_MODE GTM_gtm_cls0.MCS0_SINT_IRQ_MODE.R /* MCS[i] shared interrupt mode configuration register */
#define GTM_gtm_cls0_MCS0_HBP0_CTRL GTM_gtm_cls0.MCS0_HBP0_CTRL.R /* MCS[i] hardware break point h control register */
#define GTM_gtm_cls0_MCS0_HBP0_PATTERN GTM_gtm_cls0.MCS0_HBP0_PATTERN.R /* MCS[i] hardware break point pattern register */
#define GTM_gtm_cls0_MCS0_HBP0_STATUS GTM_gtm_cls0.MCS0_HBP0_STATUS.R /* MCS[i] hardware break point status register */
#define GTM_gtm_cls0_MCS0_HBP0_IRQ_NOTIFY GTM_gtm_cls0.MCS0_HBP0_IRQ_NOTIFY.R /* MCS[i] hardware break point interrupt notification register */
#define GTM_gtm_cls0_MCS0_HBP0_IRQ_EN GTM_gtm_cls0.MCS0_HBP0_IRQ_EN.R /* MCS[i] hardware break point interrupt enable register */
#define GTM_gtm_cls0_MCS0_HBP0_IRQ_FORCINT GTM_gtm_cls0.MCS0_HBP0_IRQ_FORCINT.R /* MCS[i] force hardware break point interrupt register */
#define GTM_gtm_cls0_MCS0_HBP0_IRQ_MODE GTM_gtm_cls0.MCS0_HBP0_IRQ_MODE.R /* MCS[i] break point h interrupt mode configuration register */
#define GTM_gtm_cls0_MCS0_HBP1_CTRL GTM_gtm_cls0.MCS0_HBP1_CTRL.R /* MCS[i] hardware break point h control register */
#define GTM_gtm_cls0_MCS0_HBP1_PATTERN GTM_gtm_cls0.MCS0_HBP1_PATTERN.R /* MCS[i] hardware break point pattern register */
#define GTM_gtm_cls0_MCS0_HBP1_STATUS GTM_gtm_cls0.MCS0_HBP1_STATUS.R /* MCS[i] hardware break point status register */
#define GTM_gtm_cls0_MCS0_HBP1_IRQ_NOTIFY GTM_gtm_cls0.MCS0_HBP1_IRQ_NOTIFY.R /* MCS[i] hardware break point interrupt notification register */
#define GTM_gtm_cls0_MCS0_HBP1_IRQ_EN GTM_gtm_cls0.MCS0_HBP1_IRQ_EN.R /* MCS[i] hardware break point interrupt enable register */
#define GTM_gtm_cls0_MCS0_HBP1_IRQ_FORCINT GTM_gtm_cls0.MCS0_HBP1_IRQ_FORCINT.R /* MCS[i] force hardware break point interrupt register */
#define GTM_gtm_cls0_MCS0_HBP1_IRQ_MODE GTM_gtm_cls0.MCS0_HBP1_IRQ_MODE.R /* MCS[i] break point h interrupt mode configuration register */
#define GTM_gtm_cls0_CCM0_ARP0_CTRL GTM_gtm_cls0.CCM0_ARP0_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP0_PROT GTM_gtm_cls0.CCM0_ARP0_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP1_CTRL GTM_gtm_cls0.CCM0_ARP1_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP1_PROT GTM_gtm_cls0.CCM0_ARP1_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP2_CTRL GTM_gtm_cls0.CCM0_ARP2_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP2_PROT GTM_gtm_cls0.CCM0_ARP2_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP3_CTRL GTM_gtm_cls0.CCM0_ARP3_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP3_PROT GTM_gtm_cls0.CCM0_ARP3_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP4_CTRL GTM_gtm_cls0.CCM0_ARP4_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP4_PROT GTM_gtm_cls0.CCM0_ARP4_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP5_CTRL GTM_gtm_cls0.CCM0_ARP5_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP5_PROT GTM_gtm_cls0.CCM0_ARP5_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP6_CTRL GTM_gtm_cls0.CCM0_ARP6_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP6_PROT GTM_gtm_cls0.CCM0_ARP6_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP7_CTRL GTM_gtm_cls0.CCM0_ARP7_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP7_PROT GTM_gtm_cls0.CCM0_ARP7_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP8_CTRL GTM_gtm_cls0.CCM0_ARP8_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP8_PROT GTM_gtm_cls0.CCM0_ARP8_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_ARP9_CTRL GTM_gtm_cls0.CCM0_ARP9_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls0_CCM0_ARP9_PROT GTM_gtm_cls0.CCM0_ARP9_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls0_CCM0_HW_CONF2 GTM_gtm_cls0.CCM0_HW_CONF2.R /* CCM[i] 2. Hardware Configuration Register */
#define GTM_gtm_cls0_CCM0_AEIM_STA GTM_gtm_cls0.CCM0_AEIM_STA.R /* CCM[i] MCS Bus Master Status Register */
#define GTM_gtm_cls0_CCM0_HW_CONF GTM_gtm_cls0.CCM0_HW_CONF.R /* CCM[i] Hardware Configuration Register */
#define GTM_gtm_cls0_CCM0_TIM_AUX_IN_SRC GTM_gtm_cls0.CCM0_TIM_AUX_IN_SRC.R /* CCM[i] TIM AUX Input Source Register */
#define GTM_gtm_cls0_CCM0_EXT_CAP_EN GTM_gtm_cls0.CCM0_EXT_CAP_EN.R /* CCM[i] External Capture Enable Register */
#define GTM_gtm_cls0_CCM0_TOM_OUT GTM_gtm_cls0.CCM0_TOM_OUT.R /* CCM[i] TOM Output Register */
#define GTM_gtm_cls0_CCM0_ATOM_OUT GTM_gtm_cls0.CCM0_ATOM_OUT.R /* CCM[i] ATOM Output Register */
#define GTM_gtm_cls0_CCM0_CMU_CLK_CFG GTM_gtm_cls0.CCM0_CMU_CLK_CFG.R /* CCM[i] CMU Clock Configuration Register */
#define GTM_gtm_cls0_CCM0_CMU_FXCLK_CFG GTM_gtm_cls0.CCM0_CMU_FXCLK_CFG.R /* CCM[i] CMU Fixed Clock Configuration Register */
#define GTM_gtm_cls0_CCM0_CFG GTM_gtm_cls0.CCM0_CFG.R      /* CCM[i] Configuration Register */
#define GTM_gtm_cls0_CCM0_PROT GTM_gtm_cls0.CCM0_PROT.R    /* CCM[i] Protection Register */
#define GTM_gtm_cls0_CDTM0_DTM4_CTRL GTM_gtm_cls0.CDTM0_DTM4_CTRL.R /* CDTM[i]_DTM[d] global configuration and control register */
#define GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL1 GTM_gtm_cls0.CDTM0_DTM4_CH_CTRL1.R /* CDTM[i]_DTM[d] channel control register 1 */
#define GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL2 GTM_gtm_cls0.CDTM0_DTM4_CH_CTRL2.R /* CDTM[i]_DTM[d] channel control register 2 */
#define GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL2_SR GTM_gtm_cls0.CDTM0_DTM4_CH_CTRL2_SR.R /* CDTM[i] DTM[j] channel control register 2 shadow */
#define GTM_gtm_cls0_CDTM0_DTM4_PS_CTRL GTM_gtm_cls0.CDTM0_DTM4_PS_CTRL.R /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
#define GTM_gtm_cls0_CDTM0_DTM4_CH0_DTV GTM_gtm_cls0.CDTM0_DTM4_CH_DTV[0].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls0_CDTM0_DTM4_CH1_DTV GTM_gtm_cls0.CDTM0_DTM4_CH_DTV[1].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls0_CDTM0_DTM4_CH2_DTV GTM_gtm_cls0.CDTM0_DTM4_CH_DTV[2].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls0_CDTM0_DTM4_CH3_DTV GTM_gtm_cls0.CDTM0_DTM4_CH_DTV[3].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls0_CDTM0_DTM4_CH_SR GTM_gtm_cls0.CDTM0_DTM4_CH_SR.R /* CDTM[i]_DTM[d] channel shadow register */
#define GTM_gtm_cls0_CDTM0_DTM4_CH_CTRL3 GTM_gtm_cls0.CDTM0_DTM4_CH_CTRL3.R /* CDTM[i]_DTM[d] channel control register 3 */
#define GTM_gtm_cls0_CDTM0_DTM4_CTRL2 GTM_gtm_cls0.CDTM0_DTM4_CTRL2.R /* CDTM[i]_DTM[d] global configuration and control register 2 */
#define GTM_gtm_cls0_CDTM0_DTM4_CH0_DTV_SR GTM_gtm_cls0.CDTM0_DTM4_CH0_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls0_CDTM0_DTM4_CH1_DTV_SR GTM_gtm_cls0.CDTM0_DTM4_CH1_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls0_CDTM0_DTM4_CH2_DTV_SR GTM_gtm_cls0.CDTM0_DTM4_CH2_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls0_CDTM0_DTM4_CH3_DTV_SR GTM_gtm_cls0.CDTM0_DTM4_CH3_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls0_CDTM0_DTM5_CTRL GTM_gtm_cls0.CDTM0_DTM5_CTRL.R /* CDTM[i]_DTM[d] global configuration and control register */
#define GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL1 GTM_gtm_cls0.CDTM0_DTM5_CH_CTRL1.R /* CDTM[i]_DTM[d] channel control register 1 */
#define GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL2 GTM_gtm_cls0.CDTM0_DTM5_CH_CTRL2.R /* CDTM[i]_DTM[d] channel control register 2 */
#define GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL2_SR GTM_gtm_cls0.CDTM0_DTM5_CH_CTRL2_SR.R /* CDTM[i] DTM[j] channel control register 2 shadow */
#define GTM_gtm_cls0_CDTM0_DTM5_PS_CTRL GTM_gtm_cls0.CDTM0_DTM5_PS_CTRL.R /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
#define GTM_gtm_cls0_CDTM0_DTM5_CH0_DTV GTM_gtm_cls0.CDTM0_DTM5_CH_DTV[0].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls0_CDTM0_DTM5_CH1_DTV GTM_gtm_cls0.CDTM0_DTM5_CH_DTV[1].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls0_CDTM0_DTM5_CH2_DTV GTM_gtm_cls0.CDTM0_DTM5_CH_DTV[2].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls0_CDTM0_DTM5_CH3_DTV GTM_gtm_cls0.CDTM0_DTM5_CH_DTV[3].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls0_CDTM0_DTM5_CH_SR GTM_gtm_cls0.CDTM0_DTM5_CH_SR.R /* CDTM[i]_DTM[d] channel shadow register */
#define GTM_gtm_cls0_CDTM0_DTM5_CH_CTRL3 GTM_gtm_cls0.CDTM0_DTM5_CH_CTRL3.R /* CDTM[i]_DTM[d] channel control register 3 */
#define GTM_gtm_cls0_CDTM0_DTM5_CTRL2 GTM_gtm_cls0.CDTM0_DTM5_CTRL2.R /* CDTM[i]_DTM[d] global configuration and control register 2 */
#define GTM_gtm_cls0_CDTM0_DTM5_CH0_DTV_SR GTM_gtm_cls0.CDTM0_DTM5_CH0_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls0_CDTM0_DTM5_CH1_DTV_SR GTM_gtm_cls0.CDTM0_DTM5_CH1_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls0_CDTM0_DTM5_CH2_DTV_SR GTM_gtm_cls0.CDTM0_DTM5_CH2_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls0_CDTM0_DTM5_CH3_DTV_SR GTM_gtm_cls0.CDTM0_DTM5_CH3_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls0_F2A0_CH0_ARU_RD_FIFO GTM_gtm_cls0.F2A0_CH0_ARU_RD_FIFO.R /* F2A[i] stream [x] ARU read register */
#define GTM_gtm_cls0_F2A0_CH1_ARU_RD_FIFO GTM_gtm_cls0.F2A0_CH1_ARU_RD_FIFO.R /* F2A[i] stream [x] ARU read register */
#define GTM_gtm_cls0_F2A0_CH2_ARU_RD_FIFO GTM_gtm_cls0.F2A0_CH2_ARU_RD_FIFO.R /* F2A[i] stream [x] ARU read register */
#define GTM_gtm_cls0_F2A0_CH3_ARU_RD_FIFO GTM_gtm_cls0.F2A0_CH3_ARU_RD_FIFO.R /* F2A[i] stream [x] ARU read register */
#define GTM_gtm_cls0_F2A0_CH4_ARU_RD_FIFO GTM_gtm_cls0.F2A0_CH4_ARU_RD_FIFO.R /* F2A[i] stream [x] ARU read register */
#define GTM_gtm_cls0_F2A0_CH5_ARU_RD_FIFO GTM_gtm_cls0.F2A0_CH5_ARU_RD_FIFO.R /* F2A[i] stream [x] ARU read register */
#define GTM_gtm_cls0_F2A0_CH6_ARU_RD_FIFO GTM_gtm_cls0.F2A0_CH6_ARU_RD_FIFO.R /* F2A[i] stream [x] ARU read register */
#define GTM_gtm_cls0_F2A0_CH7_ARU_RD_FIFO GTM_gtm_cls0.F2A0_CH7_ARU_RD_FIFO.R /* F2A[i] stream [x] ARU read register */
#define GTM_gtm_cls0_F2A0_CH0_STR_CFG GTM_gtm_cls0.F2A0_CH0_STR_CFG.R /* F2A[i] stream [x] configuration register */
#define GTM_gtm_cls0_F2A0_CH1_STR_CFG GTM_gtm_cls0.F2A0_CH1_STR_CFG.R /* F2A[i] stream [x] configuration register */
#define GTM_gtm_cls0_F2A0_CH2_STR_CFG GTM_gtm_cls0.F2A0_CH2_STR_CFG.R /* F2A[i] stream [x] configuration register */
#define GTM_gtm_cls0_F2A0_CH3_STR_CFG GTM_gtm_cls0.F2A0_CH3_STR_CFG.R /* F2A[i] stream [x] configuration register */
#define GTM_gtm_cls0_F2A0_CH4_STR_CFG GTM_gtm_cls0.F2A0_CH4_STR_CFG.R /* F2A[i] stream [x] configuration register */
#define GTM_gtm_cls0_F2A0_CH5_STR_CFG GTM_gtm_cls0.F2A0_CH5_STR_CFG.R /* F2A[i] stream [x] configuration register */
#define GTM_gtm_cls0_F2A0_CH6_STR_CFG GTM_gtm_cls0.F2A0_CH6_STR_CFG.R /* F2A[i] stream [x] configuration register */
#define GTM_gtm_cls0_F2A0_CH7_STR_CFG GTM_gtm_cls0.F2A0_CH7_STR_CFG.R /* F2A[i] stream [x] configuration register */
#define GTM_gtm_cls0_F2A0_ENABLE GTM_gtm_cls0.F2A0_ENABLE.R /* F2A[i] stream activation register */
#define GTM_gtm_cls0_F2A0_CTRL GTM_gtm_cls0.F2A0_CTRL.R    /* F2A[i] stream control register */
#define GTM_gtm_cls0_AFD0_CH0_BUF_ACC GTM_gtm_cls0.AFD0_CH0_BUF_ACC.R /* AFD [i] FIFO [x] buffer access register */
#define GTM_gtm_cls0_AFD0_CH1_BUF_ACC GTM_gtm_cls0.AFD0_CH1_BUF_ACC.R /* AFD [i] FIFO [x] buffer access register */
#define GTM_gtm_cls0_AFD0_CH2_BUF_ACC GTM_gtm_cls0.AFD0_CH2_BUF_ACC.R /* AFD [i] FIFO [x] buffer access register */
#define GTM_gtm_cls0_AFD0_CH3_BUF_ACC GTM_gtm_cls0.AFD0_CH3_BUF_ACC.R /* AFD [i] FIFO [x] buffer access register */
#define GTM_gtm_cls0_AFD0_CH4_BUF_ACC GTM_gtm_cls0.AFD0_CH4_BUF_ACC.R /* AFD [i] FIFO [x] buffer access register */
#define GTM_gtm_cls0_AFD0_CH5_BUF_ACC GTM_gtm_cls0.AFD0_CH5_BUF_ACC.R /* AFD [i] FIFO [x] buffer access register */
#define GTM_gtm_cls0_AFD0_CH6_BUF_ACC GTM_gtm_cls0.AFD0_CH6_BUF_ACC.R /* AFD [i] FIFO [x] buffer access register */
#define GTM_gtm_cls0_AFD0_CH7_BUF_ACC GTM_gtm_cls0.AFD0_CH7_BUF_ACC.R /* AFD [i] FIFO [x] buffer access register */
#define GTM_gtm_cls0_FIFO0_CH0_CTRL GTM_gtm_cls0.FIFO0_CH0_CTRL.R /* FIFO[i] channel [x] control register */
#define GTM_gtm_cls0_FIFO0_CH0_END_ADDR GTM_gtm_cls0.FIFO0_CH0_END_ADDR.R /* FIFO[i] channel [x] end address register */
#define GTM_gtm_cls0_FIFO0_CH0_START_ADDR GTM_gtm_cls0.FIFO0_CH0_START_ADDR.R /* FIFO[i] channel [x] start address register */
#define GTM_gtm_cls0_FIFO0_CH0_UPPER_WM GTM_gtm_cls0.FIFO0_CH0_UPPER_WM.R /* FIFO[i] channel [x] upper watermark register */
#define GTM_gtm_cls0_FIFO0_CH0_LOWER_WM GTM_gtm_cls0.FIFO0_CH0_LOWER_WM.R /* FIFO[i] channel [x] lower watermark register */
#define GTM_gtm_cls0_FIFO0_CH0_STATUS GTM_gtm_cls0.FIFO0_CH0_STATUS.R /* FIFO[i] channel [x] status register */
#define GTM_gtm_cls0_FIFO0_CH0_FILL_LEVEL GTM_gtm_cls0.FIFO0_CH0_FILL_LEVEL.R /* FIFO[i] channel [x] fill level register */
#define GTM_gtm_cls0_FIFO0_CH0_WR_PTR GTM_gtm_cls0.FIFO0_CH0_WR_PTR.R /* FIFO[i] channel [x] write pointer register */
#define GTM_gtm_cls0_FIFO0_CH0_RD_PTR GTM_gtm_cls0.FIFO0_CH0_RD_PTR.R /* FIFO[i] channel [x] read pointer register */
#define GTM_gtm_cls0_FIFO0_CH0_IRQ_NOTIFY GTM_gtm_cls0.FIFO0_CH0_IRQ_NOTIFY.R /* FIFO[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_FIFO0_CH0_IRQ_EN GTM_gtm_cls0.FIFO0_CH0_IRQ_EN.R /* FIFO[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH0_IRQ_FORCINT GTM_gtm_cls0.FIFO0_CH0_IRQ_FORCINT.R /* FIFO[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_FIFO0_CH0_IRQ_MODE GTM_gtm_cls0.FIFO0_CH0_IRQ_MODE.R /* FIFO[i] channel [x] interrupt mode control register */
#define GTM_gtm_cls0_FIFO0_CH0_EIRQ_EN GTM_gtm_cls0.FIFO0_CH0_EIRQ_EN.R /* FIFO[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH1_CTRL GTM_gtm_cls0.FIFO0_CH1_CTRL.R /* FIFO[i] channel [x] control register */
#define GTM_gtm_cls0_FIFO0_CH1_END_ADDR GTM_gtm_cls0.FIFO0_CH1_END_ADDR.R /* FIFO[i] channel [x] end address register */
#define GTM_gtm_cls0_FIFO0_CH1_START_ADDR GTM_gtm_cls0.FIFO0_CH1_START_ADDR.R /* FIFO[i] channel [x] start address register */
#define GTM_gtm_cls0_FIFO0_CH1_UPPER_WM GTM_gtm_cls0.FIFO0_CH1_UPPER_WM.R /* FIFO[i] channel [x] upper watermark register */
#define GTM_gtm_cls0_FIFO0_CH1_LOWER_WM GTM_gtm_cls0.FIFO0_CH1_LOWER_WM.R /* FIFO[i] channel [x] lower watermark register */
#define GTM_gtm_cls0_FIFO0_CH1_STATUS GTM_gtm_cls0.FIFO0_CH1_STATUS.R /* FIFO[i] channel [x] status register */
#define GTM_gtm_cls0_FIFO0_CH1_FILL_LEVEL GTM_gtm_cls0.FIFO0_CH1_FILL_LEVEL.R /* FIFO[i] channel [x] fill level register */
#define GTM_gtm_cls0_FIFO0_CH1_WR_PTR GTM_gtm_cls0.FIFO0_CH1_WR_PTR.R /* FIFO[i] channel [x] write pointer register */
#define GTM_gtm_cls0_FIFO0_CH1_RD_PTR GTM_gtm_cls0.FIFO0_CH1_RD_PTR.R /* FIFO[i] channel [x] read pointer register */
#define GTM_gtm_cls0_FIFO0_CH1_IRQ_NOTIFY GTM_gtm_cls0.FIFO0_CH1_IRQ_NOTIFY.R /* FIFO[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_FIFO0_CH1_IRQ_EN GTM_gtm_cls0.FIFO0_CH1_IRQ_EN.R /* FIFO[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH1_IRQ_FORCINT GTM_gtm_cls0.FIFO0_CH1_IRQ_FORCINT.R /* FIFO[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_FIFO0_CH1_IRQ_MODE GTM_gtm_cls0.FIFO0_CH1_IRQ_MODE.R /* FIFO[i] channel [x] interrupt mode control register */
#define GTM_gtm_cls0_FIFO0_CH1_EIRQ_EN GTM_gtm_cls0.FIFO0_CH1_EIRQ_EN.R /* FIFO[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH2_CTRL GTM_gtm_cls0.FIFO0_CH2_CTRL.R /* FIFO[i] channel [x] control register */
#define GTM_gtm_cls0_FIFO0_CH2_END_ADDR GTM_gtm_cls0.FIFO0_CH2_END_ADDR.R /* FIFO[i] channel [x] end address register */
#define GTM_gtm_cls0_FIFO0_CH2_START_ADDR GTM_gtm_cls0.FIFO0_CH2_START_ADDR.R /* FIFO[i] channel [x] start address register */
#define GTM_gtm_cls0_FIFO0_CH2_UPPER_WM GTM_gtm_cls0.FIFO0_CH2_UPPER_WM.R /* FIFO[i] channel [x] upper watermark register */
#define GTM_gtm_cls0_FIFO0_CH2_LOWER_WM GTM_gtm_cls0.FIFO0_CH2_LOWER_WM.R /* FIFO[i] channel [x] lower watermark register */
#define GTM_gtm_cls0_FIFO0_CH2_STATUS GTM_gtm_cls0.FIFO0_CH2_STATUS.R /* FIFO[i] channel [x] status register */
#define GTM_gtm_cls0_FIFO0_CH2_FILL_LEVEL GTM_gtm_cls0.FIFO0_CH2_FILL_LEVEL.R /* FIFO[i] channel [x] fill level register */
#define GTM_gtm_cls0_FIFO0_CH2_WR_PTR GTM_gtm_cls0.FIFO0_CH2_WR_PTR.R /* FIFO[i] channel [x] write pointer register */
#define GTM_gtm_cls0_FIFO0_CH2_RD_PTR GTM_gtm_cls0.FIFO0_CH2_RD_PTR.R /* FIFO[i] channel [x] read pointer register */
#define GTM_gtm_cls0_FIFO0_CH2_IRQ_NOTIFY GTM_gtm_cls0.FIFO0_CH2_IRQ_NOTIFY.R /* FIFO[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_FIFO0_CH2_IRQ_EN GTM_gtm_cls0.FIFO0_CH2_IRQ_EN.R /* FIFO[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH2_IRQ_FORCINT GTM_gtm_cls0.FIFO0_CH2_IRQ_FORCINT.R /* FIFO[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_FIFO0_CH2_IRQ_MODE GTM_gtm_cls0.FIFO0_CH2_IRQ_MODE.R /* FIFO[i] channel [x] interrupt mode control register */
#define GTM_gtm_cls0_FIFO0_CH2_EIRQ_EN GTM_gtm_cls0.FIFO0_CH2_EIRQ_EN.R /* FIFO[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH3_CTRL GTM_gtm_cls0.FIFO0_CH3_CTRL.R /* FIFO[i] channel [x] control register */
#define GTM_gtm_cls0_FIFO0_CH3_END_ADDR GTM_gtm_cls0.FIFO0_CH3_END_ADDR.R /* FIFO[i] channel [x] end address register */
#define GTM_gtm_cls0_FIFO0_CH3_START_ADDR GTM_gtm_cls0.FIFO0_CH3_START_ADDR.R /* FIFO[i] channel [x] start address register */
#define GTM_gtm_cls0_FIFO0_CH3_UPPER_WM GTM_gtm_cls0.FIFO0_CH3_UPPER_WM.R /* FIFO[i] channel [x] upper watermark register */
#define GTM_gtm_cls0_FIFO0_CH3_LOWER_WM GTM_gtm_cls0.FIFO0_CH3_LOWER_WM.R /* FIFO[i] channel [x] lower watermark register */
#define GTM_gtm_cls0_FIFO0_CH3_STATUS GTM_gtm_cls0.FIFO0_CH3_STATUS.R /* FIFO[i] channel [x] status register */
#define GTM_gtm_cls0_FIFO0_CH3_FILL_LEVEL GTM_gtm_cls0.FIFO0_CH3_FILL_LEVEL.R /* FIFO[i] channel [x] fill level register */
#define GTM_gtm_cls0_FIFO0_CH3_WR_PTR GTM_gtm_cls0.FIFO0_CH3_WR_PTR.R /* FIFO[i] channel [x] write pointer register */
#define GTM_gtm_cls0_FIFO0_CH3_RD_PTR GTM_gtm_cls0.FIFO0_CH3_RD_PTR.R /* FIFO[i] channel [x] read pointer register */
#define GTM_gtm_cls0_FIFO0_CH3_IRQ_NOTIFY GTM_gtm_cls0.FIFO0_CH3_IRQ_NOTIFY.R /* FIFO[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_FIFO0_CH3_IRQ_EN GTM_gtm_cls0.FIFO0_CH3_IRQ_EN.R /* FIFO[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH3_IRQ_FORCINT GTM_gtm_cls0.FIFO0_CH3_IRQ_FORCINT.R /* FIFO[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_FIFO0_CH3_IRQ_MODE GTM_gtm_cls0.FIFO0_CH3_IRQ_MODE.R /* FIFO[i] channel [x] interrupt mode control register */
#define GTM_gtm_cls0_FIFO0_CH3_EIRQ_EN GTM_gtm_cls0.FIFO0_CH3_EIRQ_EN.R /* FIFO[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH4_CTRL GTM_gtm_cls0.FIFO0_CH4_CTRL.R /* FIFO[i] channel [x] control register */
#define GTM_gtm_cls0_FIFO0_CH4_END_ADDR GTM_gtm_cls0.FIFO0_CH4_END_ADDR.R /* FIFO[i] channel [x] end address register */
#define GTM_gtm_cls0_FIFO0_CH4_START_ADDR GTM_gtm_cls0.FIFO0_CH4_START_ADDR.R /* FIFO[i] channel [x] start address register */
#define GTM_gtm_cls0_FIFO0_CH4_UPPER_WM GTM_gtm_cls0.FIFO0_CH4_UPPER_WM.R /* FIFO[i] channel [x] upper watermark register */
#define GTM_gtm_cls0_FIFO0_CH4_LOWER_WM GTM_gtm_cls0.FIFO0_CH4_LOWER_WM.R /* FIFO[i] channel [x] lower watermark register */
#define GTM_gtm_cls0_FIFO0_CH4_STATUS GTM_gtm_cls0.FIFO0_CH4_STATUS.R /* FIFO[i] channel [x] status register */
#define GTM_gtm_cls0_FIFO0_CH4_FILL_LEVEL GTM_gtm_cls0.FIFO0_CH4_FILL_LEVEL.R /* FIFO[i] channel [x] fill level register */
#define GTM_gtm_cls0_FIFO0_CH4_WR_PTR GTM_gtm_cls0.FIFO0_CH4_WR_PTR.R /* FIFO[i] channel [x] write pointer register */
#define GTM_gtm_cls0_FIFO0_CH4_RD_PTR GTM_gtm_cls0.FIFO0_CH4_RD_PTR.R /* FIFO[i] channel [x] read pointer register */
#define GTM_gtm_cls0_FIFO0_CH4_IRQ_NOTIFY GTM_gtm_cls0.FIFO0_CH4_IRQ_NOTIFY.R /* FIFO[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_FIFO0_CH4_IRQ_EN GTM_gtm_cls0.FIFO0_CH4_IRQ_EN.R /* FIFO[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH4_IRQ_FORCINT GTM_gtm_cls0.FIFO0_CH4_IRQ_FORCINT.R /* FIFO[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_FIFO0_CH4_IRQ_MODE GTM_gtm_cls0.FIFO0_CH4_IRQ_MODE.R /* FIFO[i] channel [x] interrupt mode control register */
#define GTM_gtm_cls0_FIFO0_CH4_EIRQ_EN GTM_gtm_cls0.FIFO0_CH4_EIRQ_EN.R /* FIFO[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH5_CTRL GTM_gtm_cls0.FIFO0_CH5_CTRL.R /* FIFO[i] channel [x] control register */
#define GTM_gtm_cls0_FIFO0_CH5_END_ADDR GTM_gtm_cls0.FIFO0_CH5_END_ADDR.R /* FIFO[i] channel [x] end address register */
#define GTM_gtm_cls0_FIFO0_CH5_START_ADDR GTM_gtm_cls0.FIFO0_CH5_START_ADDR.R /* FIFO[i] channel [x] start address register */
#define GTM_gtm_cls0_FIFO0_CH5_UPPER_WM GTM_gtm_cls0.FIFO0_CH5_UPPER_WM.R /* FIFO[i] channel [x] upper watermark register */
#define GTM_gtm_cls0_FIFO0_CH5_LOWER_WM GTM_gtm_cls0.FIFO0_CH5_LOWER_WM.R /* FIFO[i] channel [x] lower watermark register */
#define GTM_gtm_cls0_FIFO0_CH5_STATUS GTM_gtm_cls0.FIFO0_CH5_STATUS.R /* FIFO[i] channel [x] status register */
#define GTM_gtm_cls0_FIFO0_CH5_FILL_LEVEL GTM_gtm_cls0.FIFO0_CH5_FILL_LEVEL.R /* FIFO[i] channel [x] fill level register */
#define GTM_gtm_cls0_FIFO0_CH5_WR_PTR GTM_gtm_cls0.FIFO0_CH5_WR_PTR.R /* FIFO[i] channel [x] write pointer register */
#define GTM_gtm_cls0_FIFO0_CH5_RD_PTR GTM_gtm_cls0.FIFO0_CH5_RD_PTR.R /* FIFO[i] channel [x] read pointer register */
#define GTM_gtm_cls0_FIFO0_CH5_IRQ_NOTIFY GTM_gtm_cls0.FIFO0_CH5_IRQ_NOTIFY.R /* FIFO[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_FIFO0_CH5_IRQ_EN GTM_gtm_cls0.FIFO0_CH5_IRQ_EN.R /* FIFO[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH5_IRQ_FORCINT GTM_gtm_cls0.FIFO0_CH5_IRQ_FORCINT.R /* FIFO[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_FIFO0_CH5_IRQ_MODE GTM_gtm_cls0.FIFO0_CH5_IRQ_MODE.R /* FIFO[i] channel [x] interrupt mode control register */
#define GTM_gtm_cls0_FIFO0_CH5_EIRQ_EN GTM_gtm_cls0.FIFO0_CH5_EIRQ_EN.R /* FIFO[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH6_CTRL GTM_gtm_cls0.FIFO0_CH6_CTRL.R /* FIFO[i] channel [x] control register */
#define GTM_gtm_cls0_FIFO0_CH6_END_ADDR GTM_gtm_cls0.FIFO0_CH6_END_ADDR.R /* FIFO[i] channel [x] end address register */
#define GTM_gtm_cls0_FIFO0_CH6_START_ADDR GTM_gtm_cls0.FIFO0_CH6_START_ADDR.R /* FIFO[i] channel [x] start address register */
#define GTM_gtm_cls0_FIFO0_CH6_UPPER_WM GTM_gtm_cls0.FIFO0_CH6_UPPER_WM.R /* FIFO[i] channel [x] upper watermark register */
#define GTM_gtm_cls0_FIFO0_CH6_LOWER_WM GTM_gtm_cls0.FIFO0_CH6_LOWER_WM.R /* FIFO[i] channel [x] lower watermark register */
#define GTM_gtm_cls0_FIFO0_CH6_STATUS GTM_gtm_cls0.FIFO0_CH6_STATUS.R /* FIFO[i] channel [x] status register */
#define GTM_gtm_cls0_FIFO0_CH6_FILL_LEVEL GTM_gtm_cls0.FIFO0_CH6_FILL_LEVEL.R /* FIFO[i] channel [x] fill level register */
#define GTM_gtm_cls0_FIFO0_CH6_WR_PTR GTM_gtm_cls0.FIFO0_CH6_WR_PTR.R /* FIFO[i] channel [x] write pointer register */
#define GTM_gtm_cls0_FIFO0_CH6_RD_PTR GTM_gtm_cls0.FIFO0_CH6_RD_PTR.R /* FIFO[i] channel [x] read pointer register */
#define GTM_gtm_cls0_FIFO0_CH6_IRQ_NOTIFY GTM_gtm_cls0.FIFO0_CH6_IRQ_NOTIFY.R /* FIFO[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_FIFO0_CH6_IRQ_EN GTM_gtm_cls0.FIFO0_CH6_IRQ_EN.R /* FIFO[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH6_IRQ_FORCINT GTM_gtm_cls0.FIFO0_CH6_IRQ_FORCINT.R /* FIFO[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_FIFO0_CH6_IRQ_MODE GTM_gtm_cls0.FIFO0_CH6_IRQ_MODE.R /* FIFO[i] channel [x] interrupt mode control register */
#define GTM_gtm_cls0_FIFO0_CH6_EIRQ_EN GTM_gtm_cls0.FIFO0_CH6_EIRQ_EN.R /* FIFO[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH7_CTRL GTM_gtm_cls0.FIFO0_CH7_CTRL.R /* FIFO[i] channel [x] control register */
#define GTM_gtm_cls0_FIFO0_CH7_END_ADDR GTM_gtm_cls0.FIFO0_CH7_END_ADDR.R /* FIFO[i] channel [x] end address register */
#define GTM_gtm_cls0_FIFO0_CH7_START_ADDR GTM_gtm_cls0.FIFO0_CH7_START_ADDR.R /* FIFO[i] channel [x] start address register */
#define GTM_gtm_cls0_FIFO0_CH7_UPPER_WM GTM_gtm_cls0.FIFO0_CH7_UPPER_WM.R /* FIFO[i] channel [x] upper watermark register */
#define GTM_gtm_cls0_FIFO0_CH7_LOWER_WM GTM_gtm_cls0.FIFO0_CH7_LOWER_WM.R /* FIFO[i] channel [x] lower watermark register */
#define GTM_gtm_cls0_FIFO0_CH7_STATUS GTM_gtm_cls0.FIFO0_CH7_STATUS.R /* FIFO[i] channel [x] status register */
#define GTM_gtm_cls0_FIFO0_CH7_FILL_LEVEL GTM_gtm_cls0.FIFO0_CH7_FILL_LEVEL.R /* FIFO[i] channel [x] fill level register */
#define GTM_gtm_cls0_FIFO0_CH7_WR_PTR GTM_gtm_cls0.FIFO0_CH7_WR_PTR.R /* FIFO[i] channel [x] write pointer register */
#define GTM_gtm_cls0_FIFO0_CH7_RD_PTR GTM_gtm_cls0.FIFO0_CH7_RD_PTR.R /* FIFO[i] channel [x] read pointer register */
#define GTM_gtm_cls0_FIFO0_CH7_IRQ_NOTIFY GTM_gtm_cls0.FIFO0_CH7_IRQ_NOTIFY.R /* FIFO[i] channel [x] interrupt notification register */
#define GTM_gtm_cls0_FIFO0_CH7_IRQ_EN GTM_gtm_cls0.FIFO0_CH7_IRQ_EN.R /* FIFO[i] channel [x] interrupt enable register */
#define GTM_gtm_cls0_FIFO0_CH7_IRQ_FORCINT GTM_gtm_cls0.FIFO0_CH7_IRQ_FORCINT.R /* FIFO[i] channel [x] force interrupt register */
#define GTM_gtm_cls0_FIFO0_CH7_IRQ_MODE GTM_gtm_cls0.FIFO0_CH7_IRQ_MODE.R /* FIFO[i] channel [x] interrupt mode control register */
#define GTM_gtm_cls0_FIFO0_CH7_EIRQ_EN GTM_gtm_cls0.FIFO0_CH7_EIRQ_EN.R /* FIFO[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls0_SPE0_CTRL_STAT GTM_gtm_cls0.SPE0_CTRL_STAT.R /* SPE[i] Control Status Register */
#define GTM_gtm_cls0_SPE0_PAT GTM_gtm_cls0.SPE0_PAT.R      /* SPE[i] Input Pattern Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_PAT0 GTM_gtm_cls0.SPE0_OUT_PAT0.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_PAT1 GTM_gtm_cls0.SPE0_OUT_PAT1.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_PAT2 GTM_gtm_cls0.SPE0_OUT_PAT2.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_PAT3 GTM_gtm_cls0.SPE0_OUT_PAT3.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_PAT4 GTM_gtm_cls0.SPE0_OUT_PAT4.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_PAT5 GTM_gtm_cls0.SPE0_OUT_PAT5.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_PAT6 GTM_gtm_cls0.SPE0_OUT_PAT6.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_PAT7 GTM_gtm_cls0.SPE0_OUT_PAT7.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls0_SPE0_OUT_CTRL GTM_gtm_cls0.SPE0_OUT_CTRL.R /* SPE[i] Output Control Register */
#define GTM_gtm_cls0_SPE0_IRQ_NOTIFY GTM_gtm_cls0.SPE0_IRQ_NOTIFY.R /* SPE[i] Interrupt Notification Register */
#define GTM_gtm_cls0_SPE0_IRQ_EN GTM_gtm_cls0.SPE0_IRQ_EN.R /* SPE[i] Interrupt Enable Register */
#define GTM_gtm_cls0_SPE0_IRQ_FORCINT GTM_gtm_cls0.SPE0_IRQ_FORCINT.R /* SPE[i] Interrupt Generation By Software */
#define GTM_gtm_cls0_SPE0_IRQ_MODE GTM_gtm_cls0.SPE0_IRQ_MODE.R /* SPE[i] Interrupt Mode Configuration Register */
#define GTM_gtm_cls0_SPE0_EIRQ_EN GTM_gtm_cls0.SPE0_EIRQ_EN.R /* SPE[i] Error Interrupt Enable Register */
#define GTM_gtm_cls0_SPE0_REV_CNT GTM_gtm_cls0.SPE0_REV_CNT.R /* SPE[i] Input Revolution Counter */
#define GTM_gtm_cls0_SPE0_REV_CMP GTM_gtm_cls0.SPE0_REV_CMP.R /* SPE[i] Revolution Counter Compare Value */
#define GTM_gtm_cls0_SPE0_CTRL_STAT2 GTM_gtm_cls0.SPE0_CTRL_STAT2.R /* SPE[i] Control Status Register 2 */
#define GTM_gtm_cls0_SPE0_CMD GTM_gtm_cls0.SPE0_CMD.R      /* SPE[i] Command Register */
#define GTM_gtm_cls0_AXIM0_FREE GTM_gtm_cls0.AXIM0_FREE.R  /* AXIM[i] slot allocation status. */
#define GTM_gtm_cls0_AXIM0_REQUEST GTM_gtm_cls0.AXIM0_REQUEST.R /* AXIM[i] slot request (allocation). */
#define GTM_gtm_cls0_AXIM0_RELEASE GTM_gtm_cls0.AXIM0_RELEASE.R /* AXIM[i] slot release (de-allocation). */
#define GTM_gtm_cls0_AXIM0_SLOT0_ADDR_LOW GTM_gtm_cls0.AXIM0_SLOT0_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls0_AXIM0_SLOT0_DATA_LOW GTM_gtm_cls0.AXIM0_SLOT0_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls0_AXIM0_SLOT0_CFG1 GTM_gtm_cls0.AXIM0_SLOT0_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls0_AXIM0_SLOT0_CFG2 GTM_gtm_cls0.AXIM0_SLOT0_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls0_AXIM0_SLOT0_STATUS GTM_gtm_cls0.AXIM0_SLOT0_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls0_AXIM0_SLOT1_ADDR_LOW GTM_gtm_cls0.AXIM0_SLOT1_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls0_AXIM0_SLOT1_DATA_LOW GTM_gtm_cls0.AXIM0_SLOT1_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls0_AXIM0_SLOT1_CFG1 GTM_gtm_cls0.AXIM0_SLOT1_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls0_AXIM0_SLOT1_CFG2 GTM_gtm_cls0.AXIM0_SLOT1_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls0_AXIM0_SLOT1_STATUS GTM_gtm_cls0.AXIM0_SLOT1_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls0_AXIM0_SLOT2_ADDR_LOW GTM_gtm_cls0.AXIM0_SLOT2_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls0_AXIM0_SLOT2_DATA_LOW GTM_gtm_cls0.AXIM0_SLOT2_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls0_AXIM0_SLOT2_CFG1 GTM_gtm_cls0.AXIM0_SLOT2_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls0_AXIM0_SLOT2_CFG2 GTM_gtm_cls0.AXIM0_SLOT2_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls0_AXIM0_SLOT2_STATUS GTM_gtm_cls0.AXIM0_SLOT2_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls0_AXIM0_SLOT3_ADDR_LOW GTM_gtm_cls0.AXIM0_SLOT3_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls0_AXIM0_SLOT3_DATA_LOW GTM_gtm_cls0.AXIM0_SLOT3_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls0_AXIM0_SLOT3_CFG1 GTM_gtm_cls0.AXIM0_SLOT3_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls0_AXIM0_SLOT3_CFG2 GTM_gtm_cls0.AXIM0_SLOT3_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls0_AXIM0_SLOT3_STATUS GTM_gtm_cls0.AXIM0_SLOT3_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls0_FIFO0_MEMORY0 GTM_gtm_cls0.FIFO0_MEMORY[0].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1 GTM_gtm_cls0.FIFO0_MEMORY[1].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY2 GTM_gtm_cls0.FIFO0_MEMORY[2].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY3 GTM_gtm_cls0.FIFO0_MEMORY[3].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY4 GTM_gtm_cls0.FIFO0_MEMORY[4].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY5 GTM_gtm_cls0.FIFO0_MEMORY[5].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY6 GTM_gtm_cls0.FIFO0_MEMORY[6].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY7 GTM_gtm_cls0.FIFO0_MEMORY[7].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY8 GTM_gtm_cls0.FIFO0_MEMORY[8].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY9 GTM_gtm_cls0.FIFO0_MEMORY[9].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY10 GTM_gtm_cls0.FIFO0_MEMORY[10].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY11 GTM_gtm_cls0.FIFO0_MEMORY[11].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY12 GTM_gtm_cls0.FIFO0_MEMORY[12].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY13 GTM_gtm_cls0.FIFO0_MEMORY[13].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY14 GTM_gtm_cls0.FIFO0_MEMORY[14].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY15 GTM_gtm_cls0.FIFO0_MEMORY[15].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY16 GTM_gtm_cls0.FIFO0_MEMORY[16].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY17 GTM_gtm_cls0.FIFO0_MEMORY[17].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY18 GTM_gtm_cls0.FIFO0_MEMORY[18].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY19 GTM_gtm_cls0.FIFO0_MEMORY[19].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY20 GTM_gtm_cls0.FIFO0_MEMORY[20].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY21 GTM_gtm_cls0.FIFO0_MEMORY[21].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY22 GTM_gtm_cls0.FIFO0_MEMORY[22].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY23 GTM_gtm_cls0.FIFO0_MEMORY[23].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY24 GTM_gtm_cls0.FIFO0_MEMORY[24].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY25 GTM_gtm_cls0.FIFO0_MEMORY[25].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY26 GTM_gtm_cls0.FIFO0_MEMORY[26].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY27 GTM_gtm_cls0.FIFO0_MEMORY[27].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY28 GTM_gtm_cls0.FIFO0_MEMORY[28].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY29 GTM_gtm_cls0.FIFO0_MEMORY[29].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY30 GTM_gtm_cls0.FIFO0_MEMORY[30].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY31 GTM_gtm_cls0.FIFO0_MEMORY[31].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY32 GTM_gtm_cls0.FIFO0_MEMORY[32].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY33 GTM_gtm_cls0.FIFO0_MEMORY[33].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY34 GTM_gtm_cls0.FIFO0_MEMORY[34].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY35 GTM_gtm_cls0.FIFO0_MEMORY[35].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY36 GTM_gtm_cls0.FIFO0_MEMORY[36].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY37 GTM_gtm_cls0.FIFO0_MEMORY[37].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY38 GTM_gtm_cls0.FIFO0_MEMORY[38].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY39 GTM_gtm_cls0.FIFO0_MEMORY[39].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY40 GTM_gtm_cls0.FIFO0_MEMORY[40].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY41 GTM_gtm_cls0.FIFO0_MEMORY[41].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY42 GTM_gtm_cls0.FIFO0_MEMORY[42].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY43 GTM_gtm_cls0.FIFO0_MEMORY[43].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY44 GTM_gtm_cls0.FIFO0_MEMORY[44].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY45 GTM_gtm_cls0.FIFO0_MEMORY[45].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY46 GTM_gtm_cls0.FIFO0_MEMORY[46].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY47 GTM_gtm_cls0.FIFO0_MEMORY[47].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY48 GTM_gtm_cls0.FIFO0_MEMORY[48].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY49 GTM_gtm_cls0.FIFO0_MEMORY[49].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY50 GTM_gtm_cls0.FIFO0_MEMORY[50].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY51 GTM_gtm_cls0.FIFO0_MEMORY[51].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY52 GTM_gtm_cls0.FIFO0_MEMORY[52].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY53 GTM_gtm_cls0.FIFO0_MEMORY[53].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY54 GTM_gtm_cls0.FIFO0_MEMORY[54].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY55 GTM_gtm_cls0.FIFO0_MEMORY[55].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY56 GTM_gtm_cls0.FIFO0_MEMORY[56].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY57 GTM_gtm_cls0.FIFO0_MEMORY[57].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY58 GTM_gtm_cls0.FIFO0_MEMORY[58].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY59 GTM_gtm_cls0.FIFO0_MEMORY[59].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY60 GTM_gtm_cls0.FIFO0_MEMORY[60].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY61 GTM_gtm_cls0.FIFO0_MEMORY[61].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY62 GTM_gtm_cls0.FIFO0_MEMORY[62].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY63 GTM_gtm_cls0.FIFO0_MEMORY[63].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY64 GTM_gtm_cls0.FIFO0_MEMORY[64].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY65 GTM_gtm_cls0.FIFO0_MEMORY[65].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY66 GTM_gtm_cls0.FIFO0_MEMORY[66].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY67 GTM_gtm_cls0.FIFO0_MEMORY[67].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY68 GTM_gtm_cls0.FIFO0_MEMORY[68].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY69 GTM_gtm_cls0.FIFO0_MEMORY[69].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY70 GTM_gtm_cls0.FIFO0_MEMORY[70].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY71 GTM_gtm_cls0.FIFO0_MEMORY[71].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY72 GTM_gtm_cls0.FIFO0_MEMORY[72].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY73 GTM_gtm_cls0.FIFO0_MEMORY[73].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY74 GTM_gtm_cls0.FIFO0_MEMORY[74].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY75 GTM_gtm_cls0.FIFO0_MEMORY[75].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY76 GTM_gtm_cls0.FIFO0_MEMORY[76].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY77 GTM_gtm_cls0.FIFO0_MEMORY[77].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY78 GTM_gtm_cls0.FIFO0_MEMORY[78].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY79 GTM_gtm_cls0.FIFO0_MEMORY[79].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY80 GTM_gtm_cls0.FIFO0_MEMORY[80].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY81 GTM_gtm_cls0.FIFO0_MEMORY[81].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY82 GTM_gtm_cls0.FIFO0_MEMORY[82].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY83 GTM_gtm_cls0.FIFO0_MEMORY[83].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY84 GTM_gtm_cls0.FIFO0_MEMORY[84].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY85 GTM_gtm_cls0.FIFO0_MEMORY[85].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY86 GTM_gtm_cls0.FIFO0_MEMORY[86].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY87 GTM_gtm_cls0.FIFO0_MEMORY[87].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY88 GTM_gtm_cls0.FIFO0_MEMORY[88].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY89 GTM_gtm_cls0.FIFO0_MEMORY[89].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY90 GTM_gtm_cls0.FIFO0_MEMORY[90].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY91 GTM_gtm_cls0.FIFO0_MEMORY[91].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY92 GTM_gtm_cls0.FIFO0_MEMORY[92].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY93 GTM_gtm_cls0.FIFO0_MEMORY[93].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY94 GTM_gtm_cls0.FIFO0_MEMORY[94].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY95 GTM_gtm_cls0.FIFO0_MEMORY[95].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY96 GTM_gtm_cls0.FIFO0_MEMORY[96].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY97 GTM_gtm_cls0.FIFO0_MEMORY[97].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY98 GTM_gtm_cls0.FIFO0_MEMORY[98].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY99 GTM_gtm_cls0.FIFO0_MEMORY[99].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY100 GTM_gtm_cls0.FIFO0_MEMORY[100].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY101 GTM_gtm_cls0.FIFO0_MEMORY[101].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY102 GTM_gtm_cls0.FIFO0_MEMORY[102].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY103 GTM_gtm_cls0.FIFO0_MEMORY[103].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY104 GTM_gtm_cls0.FIFO0_MEMORY[104].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY105 GTM_gtm_cls0.FIFO0_MEMORY[105].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY106 GTM_gtm_cls0.FIFO0_MEMORY[106].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY107 GTM_gtm_cls0.FIFO0_MEMORY[107].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY108 GTM_gtm_cls0.FIFO0_MEMORY[108].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY109 GTM_gtm_cls0.FIFO0_MEMORY[109].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY110 GTM_gtm_cls0.FIFO0_MEMORY[110].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY111 GTM_gtm_cls0.FIFO0_MEMORY[111].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY112 GTM_gtm_cls0.FIFO0_MEMORY[112].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY113 GTM_gtm_cls0.FIFO0_MEMORY[113].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY114 GTM_gtm_cls0.FIFO0_MEMORY[114].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY115 GTM_gtm_cls0.FIFO0_MEMORY[115].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY116 GTM_gtm_cls0.FIFO0_MEMORY[116].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY117 GTM_gtm_cls0.FIFO0_MEMORY[117].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY118 GTM_gtm_cls0.FIFO0_MEMORY[118].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY119 GTM_gtm_cls0.FIFO0_MEMORY[119].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY120 GTM_gtm_cls0.FIFO0_MEMORY[120].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY121 GTM_gtm_cls0.FIFO0_MEMORY[121].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY122 GTM_gtm_cls0.FIFO0_MEMORY[122].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY123 GTM_gtm_cls0.FIFO0_MEMORY[123].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY124 GTM_gtm_cls0.FIFO0_MEMORY[124].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY125 GTM_gtm_cls0.FIFO0_MEMORY[125].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY126 GTM_gtm_cls0.FIFO0_MEMORY[126].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY127 GTM_gtm_cls0.FIFO0_MEMORY[127].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY128 GTM_gtm_cls0.FIFO0_MEMORY[128].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY129 GTM_gtm_cls0.FIFO0_MEMORY[129].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY130 GTM_gtm_cls0.FIFO0_MEMORY[130].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY131 GTM_gtm_cls0.FIFO0_MEMORY[131].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY132 GTM_gtm_cls0.FIFO0_MEMORY[132].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY133 GTM_gtm_cls0.FIFO0_MEMORY[133].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY134 GTM_gtm_cls0.FIFO0_MEMORY[134].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY135 GTM_gtm_cls0.FIFO0_MEMORY[135].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY136 GTM_gtm_cls0.FIFO0_MEMORY[136].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY137 GTM_gtm_cls0.FIFO0_MEMORY[137].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY138 GTM_gtm_cls0.FIFO0_MEMORY[138].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY139 GTM_gtm_cls0.FIFO0_MEMORY[139].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY140 GTM_gtm_cls0.FIFO0_MEMORY[140].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY141 GTM_gtm_cls0.FIFO0_MEMORY[141].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY142 GTM_gtm_cls0.FIFO0_MEMORY[142].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY143 GTM_gtm_cls0.FIFO0_MEMORY[143].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY144 GTM_gtm_cls0.FIFO0_MEMORY[144].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY145 GTM_gtm_cls0.FIFO0_MEMORY[145].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY146 GTM_gtm_cls0.FIFO0_MEMORY[146].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY147 GTM_gtm_cls0.FIFO0_MEMORY[147].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY148 GTM_gtm_cls0.FIFO0_MEMORY[148].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY149 GTM_gtm_cls0.FIFO0_MEMORY[149].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY150 GTM_gtm_cls0.FIFO0_MEMORY[150].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY151 GTM_gtm_cls0.FIFO0_MEMORY[151].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY152 GTM_gtm_cls0.FIFO0_MEMORY[152].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY153 GTM_gtm_cls0.FIFO0_MEMORY[153].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY154 GTM_gtm_cls0.FIFO0_MEMORY[154].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY155 GTM_gtm_cls0.FIFO0_MEMORY[155].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY156 GTM_gtm_cls0.FIFO0_MEMORY[156].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY157 GTM_gtm_cls0.FIFO0_MEMORY[157].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY158 GTM_gtm_cls0.FIFO0_MEMORY[158].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY159 GTM_gtm_cls0.FIFO0_MEMORY[159].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY160 GTM_gtm_cls0.FIFO0_MEMORY[160].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY161 GTM_gtm_cls0.FIFO0_MEMORY[161].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY162 GTM_gtm_cls0.FIFO0_MEMORY[162].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY163 GTM_gtm_cls0.FIFO0_MEMORY[163].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY164 GTM_gtm_cls0.FIFO0_MEMORY[164].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY165 GTM_gtm_cls0.FIFO0_MEMORY[165].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY166 GTM_gtm_cls0.FIFO0_MEMORY[166].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY167 GTM_gtm_cls0.FIFO0_MEMORY[167].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY168 GTM_gtm_cls0.FIFO0_MEMORY[168].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY169 GTM_gtm_cls0.FIFO0_MEMORY[169].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY170 GTM_gtm_cls0.FIFO0_MEMORY[170].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY171 GTM_gtm_cls0.FIFO0_MEMORY[171].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY172 GTM_gtm_cls0.FIFO0_MEMORY[172].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY173 GTM_gtm_cls0.FIFO0_MEMORY[173].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY174 GTM_gtm_cls0.FIFO0_MEMORY[174].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY175 GTM_gtm_cls0.FIFO0_MEMORY[175].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY176 GTM_gtm_cls0.FIFO0_MEMORY[176].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY177 GTM_gtm_cls0.FIFO0_MEMORY[177].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY178 GTM_gtm_cls0.FIFO0_MEMORY[178].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY179 GTM_gtm_cls0.FIFO0_MEMORY[179].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY180 GTM_gtm_cls0.FIFO0_MEMORY[180].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY181 GTM_gtm_cls0.FIFO0_MEMORY[181].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY182 GTM_gtm_cls0.FIFO0_MEMORY[182].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY183 GTM_gtm_cls0.FIFO0_MEMORY[183].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY184 GTM_gtm_cls0.FIFO0_MEMORY[184].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY185 GTM_gtm_cls0.FIFO0_MEMORY[185].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY186 GTM_gtm_cls0.FIFO0_MEMORY[186].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY187 GTM_gtm_cls0.FIFO0_MEMORY[187].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY188 GTM_gtm_cls0.FIFO0_MEMORY[188].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY189 GTM_gtm_cls0.FIFO0_MEMORY[189].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY190 GTM_gtm_cls0.FIFO0_MEMORY[190].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY191 GTM_gtm_cls0.FIFO0_MEMORY[191].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY192 GTM_gtm_cls0.FIFO0_MEMORY[192].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY193 GTM_gtm_cls0.FIFO0_MEMORY[193].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY194 GTM_gtm_cls0.FIFO0_MEMORY[194].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY195 GTM_gtm_cls0.FIFO0_MEMORY[195].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY196 GTM_gtm_cls0.FIFO0_MEMORY[196].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY197 GTM_gtm_cls0.FIFO0_MEMORY[197].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY198 GTM_gtm_cls0.FIFO0_MEMORY[198].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY199 GTM_gtm_cls0.FIFO0_MEMORY[199].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY200 GTM_gtm_cls0.FIFO0_MEMORY[200].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY201 GTM_gtm_cls0.FIFO0_MEMORY[201].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY202 GTM_gtm_cls0.FIFO0_MEMORY[202].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY203 GTM_gtm_cls0.FIFO0_MEMORY[203].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY204 GTM_gtm_cls0.FIFO0_MEMORY[204].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY205 GTM_gtm_cls0.FIFO0_MEMORY[205].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY206 GTM_gtm_cls0.FIFO0_MEMORY[206].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY207 GTM_gtm_cls0.FIFO0_MEMORY[207].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY208 GTM_gtm_cls0.FIFO0_MEMORY[208].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY209 GTM_gtm_cls0.FIFO0_MEMORY[209].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY210 GTM_gtm_cls0.FIFO0_MEMORY[210].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY211 GTM_gtm_cls0.FIFO0_MEMORY[211].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY212 GTM_gtm_cls0.FIFO0_MEMORY[212].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY213 GTM_gtm_cls0.FIFO0_MEMORY[213].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY214 GTM_gtm_cls0.FIFO0_MEMORY[214].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY215 GTM_gtm_cls0.FIFO0_MEMORY[215].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY216 GTM_gtm_cls0.FIFO0_MEMORY[216].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY217 GTM_gtm_cls0.FIFO0_MEMORY[217].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY218 GTM_gtm_cls0.FIFO0_MEMORY[218].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY219 GTM_gtm_cls0.FIFO0_MEMORY[219].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY220 GTM_gtm_cls0.FIFO0_MEMORY[220].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY221 GTM_gtm_cls0.FIFO0_MEMORY[221].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY222 GTM_gtm_cls0.FIFO0_MEMORY[222].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY223 GTM_gtm_cls0.FIFO0_MEMORY[223].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY224 GTM_gtm_cls0.FIFO0_MEMORY[224].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY225 GTM_gtm_cls0.FIFO0_MEMORY[225].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY226 GTM_gtm_cls0.FIFO0_MEMORY[226].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY227 GTM_gtm_cls0.FIFO0_MEMORY[227].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY228 GTM_gtm_cls0.FIFO0_MEMORY[228].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY229 GTM_gtm_cls0.FIFO0_MEMORY[229].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY230 GTM_gtm_cls0.FIFO0_MEMORY[230].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY231 GTM_gtm_cls0.FIFO0_MEMORY[231].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY232 GTM_gtm_cls0.FIFO0_MEMORY[232].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY233 GTM_gtm_cls0.FIFO0_MEMORY[233].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY234 GTM_gtm_cls0.FIFO0_MEMORY[234].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY235 GTM_gtm_cls0.FIFO0_MEMORY[235].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY236 GTM_gtm_cls0.FIFO0_MEMORY[236].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY237 GTM_gtm_cls0.FIFO0_MEMORY[237].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY238 GTM_gtm_cls0.FIFO0_MEMORY[238].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY239 GTM_gtm_cls0.FIFO0_MEMORY[239].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY240 GTM_gtm_cls0.FIFO0_MEMORY[240].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY241 GTM_gtm_cls0.FIFO0_MEMORY[241].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY242 GTM_gtm_cls0.FIFO0_MEMORY[242].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY243 GTM_gtm_cls0.FIFO0_MEMORY[243].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY244 GTM_gtm_cls0.FIFO0_MEMORY[244].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY245 GTM_gtm_cls0.FIFO0_MEMORY[245].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY246 GTM_gtm_cls0.FIFO0_MEMORY[246].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY247 GTM_gtm_cls0.FIFO0_MEMORY[247].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY248 GTM_gtm_cls0.FIFO0_MEMORY[248].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY249 GTM_gtm_cls0.FIFO0_MEMORY[249].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY250 GTM_gtm_cls0.FIFO0_MEMORY[250].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY251 GTM_gtm_cls0.FIFO0_MEMORY[251].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY252 GTM_gtm_cls0.FIFO0_MEMORY[252].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY253 GTM_gtm_cls0.FIFO0_MEMORY[253].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY254 GTM_gtm_cls0.FIFO0_MEMORY[254].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY255 GTM_gtm_cls0.FIFO0_MEMORY[255].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY256 GTM_gtm_cls0.FIFO0_MEMORY[256].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY257 GTM_gtm_cls0.FIFO0_MEMORY[257].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY258 GTM_gtm_cls0.FIFO0_MEMORY[258].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY259 GTM_gtm_cls0.FIFO0_MEMORY[259].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY260 GTM_gtm_cls0.FIFO0_MEMORY[260].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY261 GTM_gtm_cls0.FIFO0_MEMORY[261].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY262 GTM_gtm_cls0.FIFO0_MEMORY[262].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY263 GTM_gtm_cls0.FIFO0_MEMORY[263].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY264 GTM_gtm_cls0.FIFO0_MEMORY[264].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY265 GTM_gtm_cls0.FIFO0_MEMORY[265].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY266 GTM_gtm_cls0.FIFO0_MEMORY[266].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY267 GTM_gtm_cls0.FIFO0_MEMORY[267].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY268 GTM_gtm_cls0.FIFO0_MEMORY[268].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY269 GTM_gtm_cls0.FIFO0_MEMORY[269].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY270 GTM_gtm_cls0.FIFO0_MEMORY[270].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY271 GTM_gtm_cls0.FIFO0_MEMORY[271].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY272 GTM_gtm_cls0.FIFO0_MEMORY[272].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY273 GTM_gtm_cls0.FIFO0_MEMORY[273].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY274 GTM_gtm_cls0.FIFO0_MEMORY[274].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY275 GTM_gtm_cls0.FIFO0_MEMORY[275].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY276 GTM_gtm_cls0.FIFO0_MEMORY[276].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY277 GTM_gtm_cls0.FIFO0_MEMORY[277].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY278 GTM_gtm_cls0.FIFO0_MEMORY[278].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY279 GTM_gtm_cls0.FIFO0_MEMORY[279].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY280 GTM_gtm_cls0.FIFO0_MEMORY[280].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY281 GTM_gtm_cls0.FIFO0_MEMORY[281].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY282 GTM_gtm_cls0.FIFO0_MEMORY[282].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY283 GTM_gtm_cls0.FIFO0_MEMORY[283].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY284 GTM_gtm_cls0.FIFO0_MEMORY[284].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY285 GTM_gtm_cls0.FIFO0_MEMORY[285].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY286 GTM_gtm_cls0.FIFO0_MEMORY[286].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY287 GTM_gtm_cls0.FIFO0_MEMORY[287].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY288 GTM_gtm_cls0.FIFO0_MEMORY[288].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY289 GTM_gtm_cls0.FIFO0_MEMORY[289].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY290 GTM_gtm_cls0.FIFO0_MEMORY[290].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY291 GTM_gtm_cls0.FIFO0_MEMORY[291].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY292 GTM_gtm_cls0.FIFO0_MEMORY[292].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY293 GTM_gtm_cls0.FIFO0_MEMORY[293].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY294 GTM_gtm_cls0.FIFO0_MEMORY[294].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY295 GTM_gtm_cls0.FIFO0_MEMORY[295].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY296 GTM_gtm_cls0.FIFO0_MEMORY[296].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY297 GTM_gtm_cls0.FIFO0_MEMORY[297].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY298 GTM_gtm_cls0.FIFO0_MEMORY[298].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY299 GTM_gtm_cls0.FIFO0_MEMORY[299].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY300 GTM_gtm_cls0.FIFO0_MEMORY[300].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY301 GTM_gtm_cls0.FIFO0_MEMORY[301].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY302 GTM_gtm_cls0.FIFO0_MEMORY[302].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY303 GTM_gtm_cls0.FIFO0_MEMORY[303].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY304 GTM_gtm_cls0.FIFO0_MEMORY[304].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY305 GTM_gtm_cls0.FIFO0_MEMORY[305].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY306 GTM_gtm_cls0.FIFO0_MEMORY[306].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY307 GTM_gtm_cls0.FIFO0_MEMORY[307].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY308 GTM_gtm_cls0.FIFO0_MEMORY[308].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY309 GTM_gtm_cls0.FIFO0_MEMORY[309].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY310 GTM_gtm_cls0.FIFO0_MEMORY[310].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY311 GTM_gtm_cls0.FIFO0_MEMORY[311].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY312 GTM_gtm_cls0.FIFO0_MEMORY[312].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY313 GTM_gtm_cls0.FIFO0_MEMORY[313].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY314 GTM_gtm_cls0.FIFO0_MEMORY[314].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY315 GTM_gtm_cls0.FIFO0_MEMORY[315].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY316 GTM_gtm_cls0.FIFO0_MEMORY[316].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY317 GTM_gtm_cls0.FIFO0_MEMORY[317].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY318 GTM_gtm_cls0.FIFO0_MEMORY[318].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY319 GTM_gtm_cls0.FIFO0_MEMORY[319].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY320 GTM_gtm_cls0.FIFO0_MEMORY[320].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY321 GTM_gtm_cls0.FIFO0_MEMORY[321].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY322 GTM_gtm_cls0.FIFO0_MEMORY[322].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY323 GTM_gtm_cls0.FIFO0_MEMORY[323].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY324 GTM_gtm_cls0.FIFO0_MEMORY[324].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY325 GTM_gtm_cls0.FIFO0_MEMORY[325].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY326 GTM_gtm_cls0.FIFO0_MEMORY[326].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY327 GTM_gtm_cls0.FIFO0_MEMORY[327].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY328 GTM_gtm_cls0.FIFO0_MEMORY[328].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY329 GTM_gtm_cls0.FIFO0_MEMORY[329].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY330 GTM_gtm_cls0.FIFO0_MEMORY[330].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY331 GTM_gtm_cls0.FIFO0_MEMORY[331].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY332 GTM_gtm_cls0.FIFO0_MEMORY[332].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY333 GTM_gtm_cls0.FIFO0_MEMORY[333].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY334 GTM_gtm_cls0.FIFO0_MEMORY[334].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY335 GTM_gtm_cls0.FIFO0_MEMORY[335].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY336 GTM_gtm_cls0.FIFO0_MEMORY[336].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY337 GTM_gtm_cls0.FIFO0_MEMORY[337].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY338 GTM_gtm_cls0.FIFO0_MEMORY[338].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY339 GTM_gtm_cls0.FIFO0_MEMORY[339].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY340 GTM_gtm_cls0.FIFO0_MEMORY[340].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY341 GTM_gtm_cls0.FIFO0_MEMORY[341].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY342 GTM_gtm_cls0.FIFO0_MEMORY[342].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY343 GTM_gtm_cls0.FIFO0_MEMORY[343].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY344 GTM_gtm_cls0.FIFO0_MEMORY[344].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY345 GTM_gtm_cls0.FIFO0_MEMORY[345].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY346 GTM_gtm_cls0.FIFO0_MEMORY[346].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY347 GTM_gtm_cls0.FIFO0_MEMORY[347].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY348 GTM_gtm_cls0.FIFO0_MEMORY[348].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY349 GTM_gtm_cls0.FIFO0_MEMORY[349].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY350 GTM_gtm_cls0.FIFO0_MEMORY[350].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY351 GTM_gtm_cls0.FIFO0_MEMORY[351].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY352 GTM_gtm_cls0.FIFO0_MEMORY[352].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY353 GTM_gtm_cls0.FIFO0_MEMORY[353].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY354 GTM_gtm_cls0.FIFO0_MEMORY[354].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY355 GTM_gtm_cls0.FIFO0_MEMORY[355].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY356 GTM_gtm_cls0.FIFO0_MEMORY[356].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY357 GTM_gtm_cls0.FIFO0_MEMORY[357].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY358 GTM_gtm_cls0.FIFO0_MEMORY[358].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY359 GTM_gtm_cls0.FIFO0_MEMORY[359].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY360 GTM_gtm_cls0.FIFO0_MEMORY[360].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY361 GTM_gtm_cls0.FIFO0_MEMORY[361].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY362 GTM_gtm_cls0.FIFO0_MEMORY[362].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY363 GTM_gtm_cls0.FIFO0_MEMORY[363].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY364 GTM_gtm_cls0.FIFO0_MEMORY[364].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY365 GTM_gtm_cls0.FIFO0_MEMORY[365].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY366 GTM_gtm_cls0.FIFO0_MEMORY[366].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY367 GTM_gtm_cls0.FIFO0_MEMORY[367].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY368 GTM_gtm_cls0.FIFO0_MEMORY[368].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY369 GTM_gtm_cls0.FIFO0_MEMORY[369].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY370 GTM_gtm_cls0.FIFO0_MEMORY[370].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY371 GTM_gtm_cls0.FIFO0_MEMORY[371].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY372 GTM_gtm_cls0.FIFO0_MEMORY[372].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY373 GTM_gtm_cls0.FIFO0_MEMORY[373].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY374 GTM_gtm_cls0.FIFO0_MEMORY[374].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY375 GTM_gtm_cls0.FIFO0_MEMORY[375].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY376 GTM_gtm_cls0.FIFO0_MEMORY[376].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY377 GTM_gtm_cls0.FIFO0_MEMORY[377].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY378 GTM_gtm_cls0.FIFO0_MEMORY[378].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY379 GTM_gtm_cls0.FIFO0_MEMORY[379].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY380 GTM_gtm_cls0.FIFO0_MEMORY[380].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY381 GTM_gtm_cls0.FIFO0_MEMORY[381].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY382 GTM_gtm_cls0.FIFO0_MEMORY[382].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY383 GTM_gtm_cls0.FIFO0_MEMORY[383].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY384 GTM_gtm_cls0.FIFO0_MEMORY[384].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY385 GTM_gtm_cls0.FIFO0_MEMORY[385].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY386 GTM_gtm_cls0.FIFO0_MEMORY[386].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY387 GTM_gtm_cls0.FIFO0_MEMORY[387].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY388 GTM_gtm_cls0.FIFO0_MEMORY[388].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY389 GTM_gtm_cls0.FIFO0_MEMORY[389].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY390 GTM_gtm_cls0.FIFO0_MEMORY[390].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY391 GTM_gtm_cls0.FIFO0_MEMORY[391].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY392 GTM_gtm_cls0.FIFO0_MEMORY[392].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY393 GTM_gtm_cls0.FIFO0_MEMORY[393].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY394 GTM_gtm_cls0.FIFO0_MEMORY[394].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY395 GTM_gtm_cls0.FIFO0_MEMORY[395].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY396 GTM_gtm_cls0.FIFO0_MEMORY[396].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY397 GTM_gtm_cls0.FIFO0_MEMORY[397].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY398 GTM_gtm_cls0.FIFO0_MEMORY[398].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY399 GTM_gtm_cls0.FIFO0_MEMORY[399].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY400 GTM_gtm_cls0.FIFO0_MEMORY[400].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY401 GTM_gtm_cls0.FIFO0_MEMORY[401].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY402 GTM_gtm_cls0.FIFO0_MEMORY[402].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY403 GTM_gtm_cls0.FIFO0_MEMORY[403].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY404 GTM_gtm_cls0.FIFO0_MEMORY[404].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY405 GTM_gtm_cls0.FIFO0_MEMORY[405].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY406 GTM_gtm_cls0.FIFO0_MEMORY[406].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY407 GTM_gtm_cls0.FIFO0_MEMORY[407].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY408 GTM_gtm_cls0.FIFO0_MEMORY[408].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY409 GTM_gtm_cls0.FIFO0_MEMORY[409].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY410 GTM_gtm_cls0.FIFO0_MEMORY[410].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY411 GTM_gtm_cls0.FIFO0_MEMORY[411].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY412 GTM_gtm_cls0.FIFO0_MEMORY[412].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY413 GTM_gtm_cls0.FIFO0_MEMORY[413].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY414 GTM_gtm_cls0.FIFO0_MEMORY[414].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY415 GTM_gtm_cls0.FIFO0_MEMORY[415].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY416 GTM_gtm_cls0.FIFO0_MEMORY[416].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY417 GTM_gtm_cls0.FIFO0_MEMORY[417].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY418 GTM_gtm_cls0.FIFO0_MEMORY[418].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY419 GTM_gtm_cls0.FIFO0_MEMORY[419].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY420 GTM_gtm_cls0.FIFO0_MEMORY[420].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY421 GTM_gtm_cls0.FIFO0_MEMORY[421].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY422 GTM_gtm_cls0.FIFO0_MEMORY[422].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY423 GTM_gtm_cls0.FIFO0_MEMORY[423].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY424 GTM_gtm_cls0.FIFO0_MEMORY[424].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY425 GTM_gtm_cls0.FIFO0_MEMORY[425].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY426 GTM_gtm_cls0.FIFO0_MEMORY[426].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY427 GTM_gtm_cls0.FIFO0_MEMORY[427].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY428 GTM_gtm_cls0.FIFO0_MEMORY[428].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY429 GTM_gtm_cls0.FIFO0_MEMORY[429].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY430 GTM_gtm_cls0.FIFO0_MEMORY[430].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY431 GTM_gtm_cls0.FIFO0_MEMORY[431].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY432 GTM_gtm_cls0.FIFO0_MEMORY[432].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY433 GTM_gtm_cls0.FIFO0_MEMORY[433].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY434 GTM_gtm_cls0.FIFO0_MEMORY[434].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY435 GTM_gtm_cls0.FIFO0_MEMORY[435].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY436 GTM_gtm_cls0.FIFO0_MEMORY[436].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY437 GTM_gtm_cls0.FIFO0_MEMORY[437].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY438 GTM_gtm_cls0.FIFO0_MEMORY[438].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY439 GTM_gtm_cls0.FIFO0_MEMORY[439].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY440 GTM_gtm_cls0.FIFO0_MEMORY[440].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY441 GTM_gtm_cls0.FIFO0_MEMORY[441].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY442 GTM_gtm_cls0.FIFO0_MEMORY[442].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY443 GTM_gtm_cls0.FIFO0_MEMORY[443].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY444 GTM_gtm_cls0.FIFO0_MEMORY[444].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY445 GTM_gtm_cls0.FIFO0_MEMORY[445].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY446 GTM_gtm_cls0.FIFO0_MEMORY[446].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY447 GTM_gtm_cls0.FIFO0_MEMORY[447].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY448 GTM_gtm_cls0.FIFO0_MEMORY[448].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY449 GTM_gtm_cls0.FIFO0_MEMORY[449].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY450 GTM_gtm_cls0.FIFO0_MEMORY[450].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY451 GTM_gtm_cls0.FIFO0_MEMORY[451].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY452 GTM_gtm_cls0.FIFO0_MEMORY[452].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY453 GTM_gtm_cls0.FIFO0_MEMORY[453].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY454 GTM_gtm_cls0.FIFO0_MEMORY[454].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY455 GTM_gtm_cls0.FIFO0_MEMORY[455].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY456 GTM_gtm_cls0.FIFO0_MEMORY[456].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY457 GTM_gtm_cls0.FIFO0_MEMORY[457].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY458 GTM_gtm_cls0.FIFO0_MEMORY[458].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY459 GTM_gtm_cls0.FIFO0_MEMORY[459].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY460 GTM_gtm_cls0.FIFO0_MEMORY[460].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY461 GTM_gtm_cls0.FIFO0_MEMORY[461].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY462 GTM_gtm_cls0.FIFO0_MEMORY[462].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY463 GTM_gtm_cls0.FIFO0_MEMORY[463].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY464 GTM_gtm_cls0.FIFO0_MEMORY[464].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY465 GTM_gtm_cls0.FIFO0_MEMORY[465].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY466 GTM_gtm_cls0.FIFO0_MEMORY[466].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY467 GTM_gtm_cls0.FIFO0_MEMORY[467].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY468 GTM_gtm_cls0.FIFO0_MEMORY[468].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY469 GTM_gtm_cls0.FIFO0_MEMORY[469].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY470 GTM_gtm_cls0.FIFO0_MEMORY[470].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY471 GTM_gtm_cls0.FIFO0_MEMORY[471].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY472 GTM_gtm_cls0.FIFO0_MEMORY[472].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY473 GTM_gtm_cls0.FIFO0_MEMORY[473].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY474 GTM_gtm_cls0.FIFO0_MEMORY[474].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY475 GTM_gtm_cls0.FIFO0_MEMORY[475].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY476 GTM_gtm_cls0.FIFO0_MEMORY[476].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY477 GTM_gtm_cls0.FIFO0_MEMORY[477].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY478 GTM_gtm_cls0.FIFO0_MEMORY[478].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY479 GTM_gtm_cls0.FIFO0_MEMORY[479].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY480 GTM_gtm_cls0.FIFO0_MEMORY[480].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY481 GTM_gtm_cls0.FIFO0_MEMORY[481].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY482 GTM_gtm_cls0.FIFO0_MEMORY[482].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY483 GTM_gtm_cls0.FIFO0_MEMORY[483].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY484 GTM_gtm_cls0.FIFO0_MEMORY[484].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY485 GTM_gtm_cls0.FIFO0_MEMORY[485].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY486 GTM_gtm_cls0.FIFO0_MEMORY[486].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY487 GTM_gtm_cls0.FIFO0_MEMORY[487].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY488 GTM_gtm_cls0.FIFO0_MEMORY[488].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY489 GTM_gtm_cls0.FIFO0_MEMORY[489].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY490 GTM_gtm_cls0.FIFO0_MEMORY[490].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY491 GTM_gtm_cls0.FIFO0_MEMORY[491].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY492 GTM_gtm_cls0.FIFO0_MEMORY[492].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY493 GTM_gtm_cls0.FIFO0_MEMORY[493].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY494 GTM_gtm_cls0.FIFO0_MEMORY[494].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY495 GTM_gtm_cls0.FIFO0_MEMORY[495].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY496 GTM_gtm_cls0.FIFO0_MEMORY[496].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY497 GTM_gtm_cls0.FIFO0_MEMORY[497].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY498 GTM_gtm_cls0.FIFO0_MEMORY[498].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY499 GTM_gtm_cls0.FIFO0_MEMORY[499].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY500 GTM_gtm_cls0.FIFO0_MEMORY[500].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY501 GTM_gtm_cls0.FIFO0_MEMORY[501].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY502 GTM_gtm_cls0.FIFO0_MEMORY[502].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY503 GTM_gtm_cls0.FIFO0_MEMORY[503].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY504 GTM_gtm_cls0.FIFO0_MEMORY[504].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY505 GTM_gtm_cls0.FIFO0_MEMORY[505].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY506 GTM_gtm_cls0.FIFO0_MEMORY[506].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY507 GTM_gtm_cls0.FIFO0_MEMORY[507].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY508 GTM_gtm_cls0.FIFO0_MEMORY[508].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY509 GTM_gtm_cls0.FIFO0_MEMORY[509].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY510 GTM_gtm_cls0.FIFO0_MEMORY[510].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY511 GTM_gtm_cls0.FIFO0_MEMORY[511].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY512 GTM_gtm_cls0.FIFO0_MEMORY[512].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY513 GTM_gtm_cls0.FIFO0_MEMORY[513].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY514 GTM_gtm_cls0.FIFO0_MEMORY[514].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY515 GTM_gtm_cls0.FIFO0_MEMORY[515].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY516 GTM_gtm_cls0.FIFO0_MEMORY[516].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY517 GTM_gtm_cls0.FIFO0_MEMORY[517].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY518 GTM_gtm_cls0.FIFO0_MEMORY[518].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY519 GTM_gtm_cls0.FIFO0_MEMORY[519].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY520 GTM_gtm_cls0.FIFO0_MEMORY[520].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY521 GTM_gtm_cls0.FIFO0_MEMORY[521].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY522 GTM_gtm_cls0.FIFO0_MEMORY[522].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY523 GTM_gtm_cls0.FIFO0_MEMORY[523].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY524 GTM_gtm_cls0.FIFO0_MEMORY[524].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY525 GTM_gtm_cls0.FIFO0_MEMORY[525].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY526 GTM_gtm_cls0.FIFO0_MEMORY[526].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY527 GTM_gtm_cls0.FIFO0_MEMORY[527].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY528 GTM_gtm_cls0.FIFO0_MEMORY[528].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY529 GTM_gtm_cls0.FIFO0_MEMORY[529].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY530 GTM_gtm_cls0.FIFO0_MEMORY[530].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY531 GTM_gtm_cls0.FIFO0_MEMORY[531].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY532 GTM_gtm_cls0.FIFO0_MEMORY[532].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY533 GTM_gtm_cls0.FIFO0_MEMORY[533].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY534 GTM_gtm_cls0.FIFO0_MEMORY[534].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY535 GTM_gtm_cls0.FIFO0_MEMORY[535].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY536 GTM_gtm_cls0.FIFO0_MEMORY[536].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY537 GTM_gtm_cls0.FIFO0_MEMORY[537].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY538 GTM_gtm_cls0.FIFO0_MEMORY[538].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY539 GTM_gtm_cls0.FIFO0_MEMORY[539].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY540 GTM_gtm_cls0.FIFO0_MEMORY[540].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY541 GTM_gtm_cls0.FIFO0_MEMORY[541].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY542 GTM_gtm_cls0.FIFO0_MEMORY[542].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY543 GTM_gtm_cls0.FIFO0_MEMORY[543].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY544 GTM_gtm_cls0.FIFO0_MEMORY[544].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY545 GTM_gtm_cls0.FIFO0_MEMORY[545].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY546 GTM_gtm_cls0.FIFO0_MEMORY[546].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY547 GTM_gtm_cls0.FIFO0_MEMORY[547].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY548 GTM_gtm_cls0.FIFO0_MEMORY[548].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY549 GTM_gtm_cls0.FIFO0_MEMORY[549].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY550 GTM_gtm_cls0.FIFO0_MEMORY[550].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY551 GTM_gtm_cls0.FIFO0_MEMORY[551].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY552 GTM_gtm_cls0.FIFO0_MEMORY[552].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY553 GTM_gtm_cls0.FIFO0_MEMORY[553].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY554 GTM_gtm_cls0.FIFO0_MEMORY[554].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY555 GTM_gtm_cls0.FIFO0_MEMORY[555].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY556 GTM_gtm_cls0.FIFO0_MEMORY[556].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY557 GTM_gtm_cls0.FIFO0_MEMORY[557].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY558 GTM_gtm_cls0.FIFO0_MEMORY[558].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY559 GTM_gtm_cls0.FIFO0_MEMORY[559].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY560 GTM_gtm_cls0.FIFO0_MEMORY[560].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY561 GTM_gtm_cls0.FIFO0_MEMORY[561].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY562 GTM_gtm_cls0.FIFO0_MEMORY[562].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY563 GTM_gtm_cls0.FIFO0_MEMORY[563].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY564 GTM_gtm_cls0.FIFO0_MEMORY[564].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY565 GTM_gtm_cls0.FIFO0_MEMORY[565].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY566 GTM_gtm_cls0.FIFO0_MEMORY[566].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY567 GTM_gtm_cls0.FIFO0_MEMORY[567].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY568 GTM_gtm_cls0.FIFO0_MEMORY[568].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY569 GTM_gtm_cls0.FIFO0_MEMORY[569].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY570 GTM_gtm_cls0.FIFO0_MEMORY[570].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY571 GTM_gtm_cls0.FIFO0_MEMORY[571].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY572 GTM_gtm_cls0.FIFO0_MEMORY[572].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY573 GTM_gtm_cls0.FIFO0_MEMORY[573].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY574 GTM_gtm_cls0.FIFO0_MEMORY[574].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY575 GTM_gtm_cls0.FIFO0_MEMORY[575].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY576 GTM_gtm_cls0.FIFO0_MEMORY[576].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY577 GTM_gtm_cls0.FIFO0_MEMORY[577].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY578 GTM_gtm_cls0.FIFO0_MEMORY[578].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY579 GTM_gtm_cls0.FIFO0_MEMORY[579].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY580 GTM_gtm_cls0.FIFO0_MEMORY[580].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY581 GTM_gtm_cls0.FIFO0_MEMORY[581].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY582 GTM_gtm_cls0.FIFO0_MEMORY[582].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY583 GTM_gtm_cls0.FIFO0_MEMORY[583].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY584 GTM_gtm_cls0.FIFO0_MEMORY[584].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY585 GTM_gtm_cls0.FIFO0_MEMORY[585].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY586 GTM_gtm_cls0.FIFO0_MEMORY[586].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY587 GTM_gtm_cls0.FIFO0_MEMORY[587].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY588 GTM_gtm_cls0.FIFO0_MEMORY[588].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY589 GTM_gtm_cls0.FIFO0_MEMORY[589].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY590 GTM_gtm_cls0.FIFO0_MEMORY[590].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY591 GTM_gtm_cls0.FIFO0_MEMORY[591].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY592 GTM_gtm_cls0.FIFO0_MEMORY[592].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY593 GTM_gtm_cls0.FIFO0_MEMORY[593].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY594 GTM_gtm_cls0.FIFO0_MEMORY[594].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY595 GTM_gtm_cls0.FIFO0_MEMORY[595].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY596 GTM_gtm_cls0.FIFO0_MEMORY[596].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY597 GTM_gtm_cls0.FIFO0_MEMORY[597].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY598 GTM_gtm_cls0.FIFO0_MEMORY[598].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY599 GTM_gtm_cls0.FIFO0_MEMORY[599].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY600 GTM_gtm_cls0.FIFO0_MEMORY[600].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY601 GTM_gtm_cls0.FIFO0_MEMORY[601].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY602 GTM_gtm_cls0.FIFO0_MEMORY[602].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY603 GTM_gtm_cls0.FIFO0_MEMORY[603].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY604 GTM_gtm_cls0.FIFO0_MEMORY[604].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY605 GTM_gtm_cls0.FIFO0_MEMORY[605].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY606 GTM_gtm_cls0.FIFO0_MEMORY[606].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY607 GTM_gtm_cls0.FIFO0_MEMORY[607].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY608 GTM_gtm_cls0.FIFO0_MEMORY[608].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY609 GTM_gtm_cls0.FIFO0_MEMORY[609].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY610 GTM_gtm_cls0.FIFO0_MEMORY[610].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY611 GTM_gtm_cls0.FIFO0_MEMORY[611].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY612 GTM_gtm_cls0.FIFO0_MEMORY[612].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY613 GTM_gtm_cls0.FIFO0_MEMORY[613].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY614 GTM_gtm_cls0.FIFO0_MEMORY[614].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY615 GTM_gtm_cls0.FIFO0_MEMORY[615].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY616 GTM_gtm_cls0.FIFO0_MEMORY[616].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY617 GTM_gtm_cls0.FIFO0_MEMORY[617].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY618 GTM_gtm_cls0.FIFO0_MEMORY[618].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY619 GTM_gtm_cls0.FIFO0_MEMORY[619].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY620 GTM_gtm_cls0.FIFO0_MEMORY[620].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY621 GTM_gtm_cls0.FIFO0_MEMORY[621].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY622 GTM_gtm_cls0.FIFO0_MEMORY[622].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY623 GTM_gtm_cls0.FIFO0_MEMORY[623].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY624 GTM_gtm_cls0.FIFO0_MEMORY[624].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY625 GTM_gtm_cls0.FIFO0_MEMORY[625].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY626 GTM_gtm_cls0.FIFO0_MEMORY[626].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY627 GTM_gtm_cls0.FIFO0_MEMORY[627].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY628 GTM_gtm_cls0.FIFO0_MEMORY[628].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY629 GTM_gtm_cls0.FIFO0_MEMORY[629].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY630 GTM_gtm_cls0.FIFO0_MEMORY[630].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY631 GTM_gtm_cls0.FIFO0_MEMORY[631].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY632 GTM_gtm_cls0.FIFO0_MEMORY[632].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY633 GTM_gtm_cls0.FIFO0_MEMORY[633].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY634 GTM_gtm_cls0.FIFO0_MEMORY[634].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY635 GTM_gtm_cls0.FIFO0_MEMORY[635].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY636 GTM_gtm_cls0.FIFO0_MEMORY[636].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY637 GTM_gtm_cls0.FIFO0_MEMORY[637].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY638 GTM_gtm_cls0.FIFO0_MEMORY[638].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY639 GTM_gtm_cls0.FIFO0_MEMORY[639].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY640 GTM_gtm_cls0.FIFO0_MEMORY[640].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY641 GTM_gtm_cls0.FIFO0_MEMORY[641].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY642 GTM_gtm_cls0.FIFO0_MEMORY[642].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY643 GTM_gtm_cls0.FIFO0_MEMORY[643].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY644 GTM_gtm_cls0.FIFO0_MEMORY[644].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY645 GTM_gtm_cls0.FIFO0_MEMORY[645].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY646 GTM_gtm_cls0.FIFO0_MEMORY[646].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY647 GTM_gtm_cls0.FIFO0_MEMORY[647].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY648 GTM_gtm_cls0.FIFO0_MEMORY[648].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY649 GTM_gtm_cls0.FIFO0_MEMORY[649].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY650 GTM_gtm_cls0.FIFO0_MEMORY[650].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY651 GTM_gtm_cls0.FIFO0_MEMORY[651].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY652 GTM_gtm_cls0.FIFO0_MEMORY[652].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY653 GTM_gtm_cls0.FIFO0_MEMORY[653].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY654 GTM_gtm_cls0.FIFO0_MEMORY[654].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY655 GTM_gtm_cls0.FIFO0_MEMORY[655].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY656 GTM_gtm_cls0.FIFO0_MEMORY[656].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY657 GTM_gtm_cls0.FIFO0_MEMORY[657].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY658 GTM_gtm_cls0.FIFO0_MEMORY[658].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY659 GTM_gtm_cls0.FIFO0_MEMORY[659].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY660 GTM_gtm_cls0.FIFO0_MEMORY[660].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY661 GTM_gtm_cls0.FIFO0_MEMORY[661].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY662 GTM_gtm_cls0.FIFO0_MEMORY[662].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY663 GTM_gtm_cls0.FIFO0_MEMORY[663].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY664 GTM_gtm_cls0.FIFO0_MEMORY[664].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY665 GTM_gtm_cls0.FIFO0_MEMORY[665].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY666 GTM_gtm_cls0.FIFO0_MEMORY[666].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY667 GTM_gtm_cls0.FIFO0_MEMORY[667].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY668 GTM_gtm_cls0.FIFO0_MEMORY[668].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY669 GTM_gtm_cls0.FIFO0_MEMORY[669].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY670 GTM_gtm_cls0.FIFO0_MEMORY[670].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY671 GTM_gtm_cls0.FIFO0_MEMORY[671].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY672 GTM_gtm_cls0.FIFO0_MEMORY[672].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY673 GTM_gtm_cls0.FIFO0_MEMORY[673].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY674 GTM_gtm_cls0.FIFO0_MEMORY[674].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY675 GTM_gtm_cls0.FIFO0_MEMORY[675].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY676 GTM_gtm_cls0.FIFO0_MEMORY[676].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY677 GTM_gtm_cls0.FIFO0_MEMORY[677].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY678 GTM_gtm_cls0.FIFO0_MEMORY[678].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY679 GTM_gtm_cls0.FIFO0_MEMORY[679].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY680 GTM_gtm_cls0.FIFO0_MEMORY[680].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY681 GTM_gtm_cls0.FIFO0_MEMORY[681].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY682 GTM_gtm_cls0.FIFO0_MEMORY[682].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY683 GTM_gtm_cls0.FIFO0_MEMORY[683].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY684 GTM_gtm_cls0.FIFO0_MEMORY[684].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY685 GTM_gtm_cls0.FIFO0_MEMORY[685].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY686 GTM_gtm_cls0.FIFO0_MEMORY[686].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY687 GTM_gtm_cls0.FIFO0_MEMORY[687].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY688 GTM_gtm_cls0.FIFO0_MEMORY[688].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY689 GTM_gtm_cls0.FIFO0_MEMORY[689].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY690 GTM_gtm_cls0.FIFO0_MEMORY[690].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY691 GTM_gtm_cls0.FIFO0_MEMORY[691].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY692 GTM_gtm_cls0.FIFO0_MEMORY[692].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY693 GTM_gtm_cls0.FIFO0_MEMORY[693].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY694 GTM_gtm_cls0.FIFO0_MEMORY[694].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY695 GTM_gtm_cls0.FIFO0_MEMORY[695].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY696 GTM_gtm_cls0.FIFO0_MEMORY[696].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY697 GTM_gtm_cls0.FIFO0_MEMORY[697].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY698 GTM_gtm_cls0.FIFO0_MEMORY[698].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY699 GTM_gtm_cls0.FIFO0_MEMORY[699].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY700 GTM_gtm_cls0.FIFO0_MEMORY[700].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY701 GTM_gtm_cls0.FIFO0_MEMORY[701].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY702 GTM_gtm_cls0.FIFO0_MEMORY[702].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY703 GTM_gtm_cls0.FIFO0_MEMORY[703].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY704 GTM_gtm_cls0.FIFO0_MEMORY[704].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY705 GTM_gtm_cls0.FIFO0_MEMORY[705].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY706 GTM_gtm_cls0.FIFO0_MEMORY[706].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY707 GTM_gtm_cls0.FIFO0_MEMORY[707].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY708 GTM_gtm_cls0.FIFO0_MEMORY[708].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY709 GTM_gtm_cls0.FIFO0_MEMORY[709].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY710 GTM_gtm_cls0.FIFO0_MEMORY[710].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY711 GTM_gtm_cls0.FIFO0_MEMORY[711].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY712 GTM_gtm_cls0.FIFO0_MEMORY[712].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY713 GTM_gtm_cls0.FIFO0_MEMORY[713].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY714 GTM_gtm_cls0.FIFO0_MEMORY[714].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY715 GTM_gtm_cls0.FIFO0_MEMORY[715].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY716 GTM_gtm_cls0.FIFO0_MEMORY[716].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY717 GTM_gtm_cls0.FIFO0_MEMORY[717].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY718 GTM_gtm_cls0.FIFO0_MEMORY[718].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY719 GTM_gtm_cls0.FIFO0_MEMORY[719].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY720 GTM_gtm_cls0.FIFO0_MEMORY[720].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY721 GTM_gtm_cls0.FIFO0_MEMORY[721].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY722 GTM_gtm_cls0.FIFO0_MEMORY[722].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY723 GTM_gtm_cls0.FIFO0_MEMORY[723].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY724 GTM_gtm_cls0.FIFO0_MEMORY[724].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY725 GTM_gtm_cls0.FIFO0_MEMORY[725].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY726 GTM_gtm_cls0.FIFO0_MEMORY[726].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY727 GTM_gtm_cls0.FIFO0_MEMORY[727].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY728 GTM_gtm_cls0.FIFO0_MEMORY[728].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY729 GTM_gtm_cls0.FIFO0_MEMORY[729].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY730 GTM_gtm_cls0.FIFO0_MEMORY[730].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY731 GTM_gtm_cls0.FIFO0_MEMORY[731].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY732 GTM_gtm_cls0.FIFO0_MEMORY[732].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY733 GTM_gtm_cls0.FIFO0_MEMORY[733].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY734 GTM_gtm_cls0.FIFO0_MEMORY[734].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY735 GTM_gtm_cls0.FIFO0_MEMORY[735].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY736 GTM_gtm_cls0.FIFO0_MEMORY[736].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY737 GTM_gtm_cls0.FIFO0_MEMORY[737].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY738 GTM_gtm_cls0.FIFO0_MEMORY[738].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY739 GTM_gtm_cls0.FIFO0_MEMORY[739].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY740 GTM_gtm_cls0.FIFO0_MEMORY[740].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY741 GTM_gtm_cls0.FIFO0_MEMORY[741].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY742 GTM_gtm_cls0.FIFO0_MEMORY[742].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY743 GTM_gtm_cls0.FIFO0_MEMORY[743].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY744 GTM_gtm_cls0.FIFO0_MEMORY[744].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY745 GTM_gtm_cls0.FIFO0_MEMORY[745].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY746 GTM_gtm_cls0.FIFO0_MEMORY[746].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY747 GTM_gtm_cls0.FIFO0_MEMORY[747].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY748 GTM_gtm_cls0.FIFO0_MEMORY[748].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY749 GTM_gtm_cls0.FIFO0_MEMORY[749].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY750 GTM_gtm_cls0.FIFO0_MEMORY[750].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY751 GTM_gtm_cls0.FIFO0_MEMORY[751].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY752 GTM_gtm_cls0.FIFO0_MEMORY[752].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY753 GTM_gtm_cls0.FIFO0_MEMORY[753].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY754 GTM_gtm_cls0.FIFO0_MEMORY[754].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY755 GTM_gtm_cls0.FIFO0_MEMORY[755].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY756 GTM_gtm_cls0.FIFO0_MEMORY[756].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY757 GTM_gtm_cls0.FIFO0_MEMORY[757].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY758 GTM_gtm_cls0.FIFO0_MEMORY[758].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY759 GTM_gtm_cls0.FIFO0_MEMORY[759].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY760 GTM_gtm_cls0.FIFO0_MEMORY[760].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY761 GTM_gtm_cls0.FIFO0_MEMORY[761].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY762 GTM_gtm_cls0.FIFO0_MEMORY[762].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY763 GTM_gtm_cls0.FIFO0_MEMORY[763].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY764 GTM_gtm_cls0.FIFO0_MEMORY[764].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY765 GTM_gtm_cls0.FIFO0_MEMORY[765].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY766 GTM_gtm_cls0.FIFO0_MEMORY[766].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY767 GTM_gtm_cls0.FIFO0_MEMORY[767].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY768 GTM_gtm_cls0.FIFO0_MEMORY[768].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY769 GTM_gtm_cls0.FIFO0_MEMORY[769].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY770 GTM_gtm_cls0.FIFO0_MEMORY[770].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY771 GTM_gtm_cls0.FIFO0_MEMORY[771].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY772 GTM_gtm_cls0.FIFO0_MEMORY[772].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY773 GTM_gtm_cls0.FIFO0_MEMORY[773].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY774 GTM_gtm_cls0.FIFO0_MEMORY[774].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY775 GTM_gtm_cls0.FIFO0_MEMORY[775].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY776 GTM_gtm_cls0.FIFO0_MEMORY[776].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY777 GTM_gtm_cls0.FIFO0_MEMORY[777].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY778 GTM_gtm_cls0.FIFO0_MEMORY[778].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY779 GTM_gtm_cls0.FIFO0_MEMORY[779].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY780 GTM_gtm_cls0.FIFO0_MEMORY[780].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY781 GTM_gtm_cls0.FIFO0_MEMORY[781].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY782 GTM_gtm_cls0.FIFO0_MEMORY[782].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY783 GTM_gtm_cls0.FIFO0_MEMORY[783].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY784 GTM_gtm_cls0.FIFO0_MEMORY[784].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY785 GTM_gtm_cls0.FIFO0_MEMORY[785].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY786 GTM_gtm_cls0.FIFO0_MEMORY[786].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY787 GTM_gtm_cls0.FIFO0_MEMORY[787].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY788 GTM_gtm_cls0.FIFO0_MEMORY[788].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY789 GTM_gtm_cls0.FIFO0_MEMORY[789].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY790 GTM_gtm_cls0.FIFO0_MEMORY[790].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY791 GTM_gtm_cls0.FIFO0_MEMORY[791].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY792 GTM_gtm_cls0.FIFO0_MEMORY[792].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY793 GTM_gtm_cls0.FIFO0_MEMORY[793].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY794 GTM_gtm_cls0.FIFO0_MEMORY[794].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY795 GTM_gtm_cls0.FIFO0_MEMORY[795].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY796 GTM_gtm_cls0.FIFO0_MEMORY[796].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY797 GTM_gtm_cls0.FIFO0_MEMORY[797].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY798 GTM_gtm_cls0.FIFO0_MEMORY[798].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY799 GTM_gtm_cls0.FIFO0_MEMORY[799].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY800 GTM_gtm_cls0.FIFO0_MEMORY[800].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY801 GTM_gtm_cls0.FIFO0_MEMORY[801].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY802 GTM_gtm_cls0.FIFO0_MEMORY[802].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY803 GTM_gtm_cls0.FIFO0_MEMORY[803].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY804 GTM_gtm_cls0.FIFO0_MEMORY[804].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY805 GTM_gtm_cls0.FIFO0_MEMORY[805].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY806 GTM_gtm_cls0.FIFO0_MEMORY[806].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY807 GTM_gtm_cls0.FIFO0_MEMORY[807].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY808 GTM_gtm_cls0.FIFO0_MEMORY[808].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY809 GTM_gtm_cls0.FIFO0_MEMORY[809].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY810 GTM_gtm_cls0.FIFO0_MEMORY[810].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY811 GTM_gtm_cls0.FIFO0_MEMORY[811].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY812 GTM_gtm_cls0.FIFO0_MEMORY[812].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY813 GTM_gtm_cls0.FIFO0_MEMORY[813].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY814 GTM_gtm_cls0.FIFO0_MEMORY[814].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY815 GTM_gtm_cls0.FIFO0_MEMORY[815].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY816 GTM_gtm_cls0.FIFO0_MEMORY[816].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY817 GTM_gtm_cls0.FIFO0_MEMORY[817].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY818 GTM_gtm_cls0.FIFO0_MEMORY[818].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY819 GTM_gtm_cls0.FIFO0_MEMORY[819].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY820 GTM_gtm_cls0.FIFO0_MEMORY[820].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY821 GTM_gtm_cls0.FIFO0_MEMORY[821].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY822 GTM_gtm_cls0.FIFO0_MEMORY[822].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY823 GTM_gtm_cls0.FIFO0_MEMORY[823].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY824 GTM_gtm_cls0.FIFO0_MEMORY[824].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY825 GTM_gtm_cls0.FIFO0_MEMORY[825].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY826 GTM_gtm_cls0.FIFO0_MEMORY[826].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY827 GTM_gtm_cls0.FIFO0_MEMORY[827].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY828 GTM_gtm_cls0.FIFO0_MEMORY[828].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY829 GTM_gtm_cls0.FIFO0_MEMORY[829].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY830 GTM_gtm_cls0.FIFO0_MEMORY[830].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY831 GTM_gtm_cls0.FIFO0_MEMORY[831].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY832 GTM_gtm_cls0.FIFO0_MEMORY[832].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY833 GTM_gtm_cls0.FIFO0_MEMORY[833].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY834 GTM_gtm_cls0.FIFO0_MEMORY[834].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY835 GTM_gtm_cls0.FIFO0_MEMORY[835].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY836 GTM_gtm_cls0.FIFO0_MEMORY[836].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY837 GTM_gtm_cls0.FIFO0_MEMORY[837].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY838 GTM_gtm_cls0.FIFO0_MEMORY[838].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY839 GTM_gtm_cls0.FIFO0_MEMORY[839].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY840 GTM_gtm_cls0.FIFO0_MEMORY[840].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY841 GTM_gtm_cls0.FIFO0_MEMORY[841].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY842 GTM_gtm_cls0.FIFO0_MEMORY[842].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY843 GTM_gtm_cls0.FIFO0_MEMORY[843].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY844 GTM_gtm_cls0.FIFO0_MEMORY[844].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY845 GTM_gtm_cls0.FIFO0_MEMORY[845].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY846 GTM_gtm_cls0.FIFO0_MEMORY[846].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY847 GTM_gtm_cls0.FIFO0_MEMORY[847].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY848 GTM_gtm_cls0.FIFO0_MEMORY[848].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY849 GTM_gtm_cls0.FIFO0_MEMORY[849].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY850 GTM_gtm_cls0.FIFO0_MEMORY[850].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY851 GTM_gtm_cls0.FIFO0_MEMORY[851].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY852 GTM_gtm_cls0.FIFO0_MEMORY[852].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY853 GTM_gtm_cls0.FIFO0_MEMORY[853].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY854 GTM_gtm_cls0.FIFO0_MEMORY[854].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY855 GTM_gtm_cls0.FIFO0_MEMORY[855].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY856 GTM_gtm_cls0.FIFO0_MEMORY[856].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY857 GTM_gtm_cls0.FIFO0_MEMORY[857].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY858 GTM_gtm_cls0.FIFO0_MEMORY[858].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY859 GTM_gtm_cls0.FIFO0_MEMORY[859].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY860 GTM_gtm_cls0.FIFO0_MEMORY[860].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY861 GTM_gtm_cls0.FIFO0_MEMORY[861].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY862 GTM_gtm_cls0.FIFO0_MEMORY[862].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY863 GTM_gtm_cls0.FIFO0_MEMORY[863].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY864 GTM_gtm_cls0.FIFO0_MEMORY[864].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY865 GTM_gtm_cls0.FIFO0_MEMORY[865].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY866 GTM_gtm_cls0.FIFO0_MEMORY[866].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY867 GTM_gtm_cls0.FIFO0_MEMORY[867].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY868 GTM_gtm_cls0.FIFO0_MEMORY[868].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY869 GTM_gtm_cls0.FIFO0_MEMORY[869].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY870 GTM_gtm_cls0.FIFO0_MEMORY[870].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY871 GTM_gtm_cls0.FIFO0_MEMORY[871].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY872 GTM_gtm_cls0.FIFO0_MEMORY[872].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY873 GTM_gtm_cls0.FIFO0_MEMORY[873].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY874 GTM_gtm_cls0.FIFO0_MEMORY[874].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY875 GTM_gtm_cls0.FIFO0_MEMORY[875].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY876 GTM_gtm_cls0.FIFO0_MEMORY[876].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY877 GTM_gtm_cls0.FIFO0_MEMORY[877].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY878 GTM_gtm_cls0.FIFO0_MEMORY[878].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY879 GTM_gtm_cls0.FIFO0_MEMORY[879].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY880 GTM_gtm_cls0.FIFO0_MEMORY[880].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY881 GTM_gtm_cls0.FIFO0_MEMORY[881].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY882 GTM_gtm_cls0.FIFO0_MEMORY[882].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY883 GTM_gtm_cls0.FIFO0_MEMORY[883].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY884 GTM_gtm_cls0.FIFO0_MEMORY[884].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY885 GTM_gtm_cls0.FIFO0_MEMORY[885].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY886 GTM_gtm_cls0.FIFO0_MEMORY[886].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY887 GTM_gtm_cls0.FIFO0_MEMORY[887].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY888 GTM_gtm_cls0.FIFO0_MEMORY[888].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY889 GTM_gtm_cls0.FIFO0_MEMORY[889].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY890 GTM_gtm_cls0.FIFO0_MEMORY[890].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY891 GTM_gtm_cls0.FIFO0_MEMORY[891].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY892 GTM_gtm_cls0.FIFO0_MEMORY[892].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY893 GTM_gtm_cls0.FIFO0_MEMORY[893].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY894 GTM_gtm_cls0.FIFO0_MEMORY[894].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY895 GTM_gtm_cls0.FIFO0_MEMORY[895].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY896 GTM_gtm_cls0.FIFO0_MEMORY[896].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY897 GTM_gtm_cls0.FIFO0_MEMORY[897].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY898 GTM_gtm_cls0.FIFO0_MEMORY[898].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY899 GTM_gtm_cls0.FIFO0_MEMORY[899].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY900 GTM_gtm_cls0.FIFO0_MEMORY[900].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY901 GTM_gtm_cls0.FIFO0_MEMORY[901].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY902 GTM_gtm_cls0.FIFO0_MEMORY[902].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY903 GTM_gtm_cls0.FIFO0_MEMORY[903].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY904 GTM_gtm_cls0.FIFO0_MEMORY[904].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY905 GTM_gtm_cls0.FIFO0_MEMORY[905].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY906 GTM_gtm_cls0.FIFO0_MEMORY[906].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY907 GTM_gtm_cls0.FIFO0_MEMORY[907].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY908 GTM_gtm_cls0.FIFO0_MEMORY[908].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY909 GTM_gtm_cls0.FIFO0_MEMORY[909].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY910 GTM_gtm_cls0.FIFO0_MEMORY[910].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY911 GTM_gtm_cls0.FIFO0_MEMORY[911].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY912 GTM_gtm_cls0.FIFO0_MEMORY[912].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY913 GTM_gtm_cls0.FIFO0_MEMORY[913].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY914 GTM_gtm_cls0.FIFO0_MEMORY[914].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY915 GTM_gtm_cls0.FIFO0_MEMORY[915].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY916 GTM_gtm_cls0.FIFO0_MEMORY[916].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY917 GTM_gtm_cls0.FIFO0_MEMORY[917].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY918 GTM_gtm_cls0.FIFO0_MEMORY[918].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY919 GTM_gtm_cls0.FIFO0_MEMORY[919].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY920 GTM_gtm_cls0.FIFO0_MEMORY[920].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY921 GTM_gtm_cls0.FIFO0_MEMORY[921].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY922 GTM_gtm_cls0.FIFO0_MEMORY[922].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY923 GTM_gtm_cls0.FIFO0_MEMORY[923].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY924 GTM_gtm_cls0.FIFO0_MEMORY[924].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY925 GTM_gtm_cls0.FIFO0_MEMORY[925].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY926 GTM_gtm_cls0.FIFO0_MEMORY[926].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY927 GTM_gtm_cls0.FIFO0_MEMORY[927].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY928 GTM_gtm_cls0.FIFO0_MEMORY[928].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY929 GTM_gtm_cls0.FIFO0_MEMORY[929].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY930 GTM_gtm_cls0.FIFO0_MEMORY[930].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY931 GTM_gtm_cls0.FIFO0_MEMORY[931].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY932 GTM_gtm_cls0.FIFO0_MEMORY[932].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY933 GTM_gtm_cls0.FIFO0_MEMORY[933].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY934 GTM_gtm_cls0.FIFO0_MEMORY[934].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY935 GTM_gtm_cls0.FIFO0_MEMORY[935].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY936 GTM_gtm_cls0.FIFO0_MEMORY[936].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY937 GTM_gtm_cls0.FIFO0_MEMORY[937].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY938 GTM_gtm_cls0.FIFO0_MEMORY[938].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY939 GTM_gtm_cls0.FIFO0_MEMORY[939].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY940 GTM_gtm_cls0.FIFO0_MEMORY[940].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY941 GTM_gtm_cls0.FIFO0_MEMORY[941].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY942 GTM_gtm_cls0.FIFO0_MEMORY[942].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY943 GTM_gtm_cls0.FIFO0_MEMORY[943].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY944 GTM_gtm_cls0.FIFO0_MEMORY[944].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY945 GTM_gtm_cls0.FIFO0_MEMORY[945].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY946 GTM_gtm_cls0.FIFO0_MEMORY[946].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY947 GTM_gtm_cls0.FIFO0_MEMORY[947].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY948 GTM_gtm_cls0.FIFO0_MEMORY[948].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY949 GTM_gtm_cls0.FIFO0_MEMORY[949].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY950 GTM_gtm_cls0.FIFO0_MEMORY[950].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY951 GTM_gtm_cls0.FIFO0_MEMORY[951].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY952 GTM_gtm_cls0.FIFO0_MEMORY[952].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY953 GTM_gtm_cls0.FIFO0_MEMORY[953].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY954 GTM_gtm_cls0.FIFO0_MEMORY[954].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY955 GTM_gtm_cls0.FIFO0_MEMORY[955].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY956 GTM_gtm_cls0.FIFO0_MEMORY[956].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY957 GTM_gtm_cls0.FIFO0_MEMORY[957].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY958 GTM_gtm_cls0.FIFO0_MEMORY[958].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY959 GTM_gtm_cls0.FIFO0_MEMORY[959].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY960 GTM_gtm_cls0.FIFO0_MEMORY[960].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY961 GTM_gtm_cls0.FIFO0_MEMORY[961].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY962 GTM_gtm_cls0.FIFO0_MEMORY[962].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY963 GTM_gtm_cls0.FIFO0_MEMORY[963].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY964 GTM_gtm_cls0.FIFO0_MEMORY[964].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY965 GTM_gtm_cls0.FIFO0_MEMORY[965].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY966 GTM_gtm_cls0.FIFO0_MEMORY[966].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY967 GTM_gtm_cls0.FIFO0_MEMORY[967].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY968 GTM_gtm_cls0.FIFO0_MEMORY[968].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY969 GTM_gtm_cls0.FIFO0_MEMORY[969].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY970 GTM_gtm_cls0.FIFO0_MEMORY[970].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY971 GTM_gtm_cls0.FIFO0_MEMORY[971].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY972 GTM_gtm_cls0.FIFO0_MEMORY[972].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY973 GTM_gtm_cls0.FIFO0_MEMORY[973].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY974 GTM_gtm_cls0.FIFO0_MEMORY[974].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY975 GTM_gtm_cls0.FIFO0_MEMORY[975].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY976 GTM_gtm_cls0.FIFO0_MEMORY[976].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY977 GTM_gtm_cls0.FIFO0_MEMORY[977].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY978 GTM_gtm_cls0.FIFO0_MEMORY[978].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY979 GTM_gtm_cls0.FIFO0_MEMORY[979].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY980 GTM_gtm_cls0.FIFO0_MEMORY[980].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY981 GTM_gtm_cls0.FIFO0_MEMORY[981].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY982 GTM_gtm_cls0.FIFO0_MEMORY[982].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY983 GTM_gtm_cls0.FIFO0_MEMORY[983].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY984 GTM_gtm_cls0.FIFO0_MEMORY[984].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY985 GTM_gtm_cls0.FIFO0_MEMORY[985].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY986 GTM_gtm_cls0.FIFO0_MEMORY[986].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY987 GTM_gtm_cls0.FIFO0_MEMORY[987].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY988 GTM_gtm_cls0.FIFO0_MEMORY[988].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY989 GTM_gtm_cls0.FIFO0_MEMORY[989].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY990 GTM_gtm_cls0.FIFO0_MEMORY[990].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY991 GTM_gtm_cls0.FIFO0_MEMORY[991].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY992 GTM_gtm_cls0.FIFO0_MEMORY[992].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY993 GTM_gtm_cls0.FIFO0_MEMORY[993].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY994 GTM_gtm_cls0.FIFO0_MEMORY[994].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY995 GTM_gtm_cls0.FIFO0_MEMORY[995].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY996 GTM_gtm_cls0.FIFO0_MEMORY[996].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY997 GTM_gtm_cls0.FIFO0_MEMORY[997].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY998 GTM_gtm_cls0.FIFO0_MEMORY[998].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY999 GTM_gtm_cls0.FIFO0_MEMORY[999].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1000 GTM_gtm_cls0.FIFO0_MEMORY[1000].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1001 GTM_gtm_cls0.FIFO0_MEMORY[1001].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1002 GTM_gtm_cls0.FIFO0_MEMORY[1002].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1003 GTM_gtm_cls0.FIFO0_MEMORY[1003].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1004 GTM_gtm_cls0.FIFO0_MEMORY[1004].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1005 GTM_gtm_cls0.FIFO0_MEMORY[1005].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1006 GTM_gtm_cls0.FIFO0_MEMORY[1006].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1007 GTM_gtm_cls0.FIFO0_MEMORY[1007].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1008 GTM_gtm_cls0.FIFO0_MEMORY[1008].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1009 GTM_gtm_cls0.FIFO0_MEMORY[1009].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1010 GTM_gtm_cls0.FIFO0_MEMORY[1010].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1011 GTM_gtm_cls0.FIFO0_MEMORY[1011].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1012 GTM_gtm_cls0.FIFO0_MEMORY[1012].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1013 GTM_gtm_cls0.FIFO0_MEMORY[1013].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1014 GTM_gtm_cls0.FIFO0_MEMORY[1014].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1015 GTM_gtm_cls0.FIFO0_MEMORY[1015].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1016 GTM_gtm_cls0.FIFO0_MEMORY[1016].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1017 GTM_gtm_cls0.FIFO0_MEMORY[1017].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1018 GTM_gtm_cls0.FIFO0_MEMORY[1018].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1019 GTM_gtm_cls0.FIFO0_MEMORY[1019].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1020 GTM_gtm_cls0.FIFO0_MEMORY[1020].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1021 GTM_gtm_cls0.FIFO0_MEMORY[1021].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1022 GTM_gtm_cls0.FIFO0_MEMORY[1022].R /* FIFO data memory */
#define GTM_gtm_cls0_FIFO0_MEMORY1023 GTM_gtm_cls0.FIFO0_MEMORY[1023].R /* FIFO data memory */
#define GTM_gtm_cls0_DPLL_CTRL_0 GTM_gtm_cls0.DPLL_CTRL_0.R /* Control Register 0 */
#define GTM_gtm_cls0_DPLL_CTRL_1 GTM_gtm_cls0.DPLL_CTRL_1.R /* Control Register 1 */
#define GTM_gtm_cls0_DPLL_CTRL_2 GTM_gtm_cls0.DPLL_CTRL_2.R /* Action Enable Register */
#define GTM_gtm_cls0_DPLL_CTRL_3 GTM_gtm_cls0.DPLL_CTRL_3.R /* Action Enable Register */
#define GTM_gtm_cls0_DPLL_CTRL_4 GTM_gtm_cls0.DPLL_CTRL_4.R /* Action Enable Register */
#define GTM_gtm_cls0_DPLL_CTRL_5 GTM_gtm_cls0.DPLL_CTRL_5.R /* Action Enable Register */
#define GTM_gtm_cls0_DPLL_ACT_STA GTM_gtm_cls0.DPLL_ACT_STA.R /* Action Status Register including Shadow Register */
#define GTM_gtm_cls0_DPLL_OSW GTM_gtm_cls0.DPLL_OSW.R      /* Offset and Switch old/new Address Register */
#define GTM_gtm_cls0_DPLL_AOSV_2 GTM_gtm_cls0.DPLL_AOSV_2.R /* Address Offset Register of RAM 2 Regions */
#define GTM_gtm_cls0_DPLL_APT GTM_gtm_cls0.DPLL_APT.R      /* Actual RAM Pointer Address for TRIGGER */
#define GTM_gtm_cls0_DPLL_APS GTM_gtm_cls0.DPLL_APS.R      /* Actual RAM Pointer Address for STATE */
#define GTM_gtm_cls0_DPLL_APT_2C GTM_gtm_cls0.DPLL_APT_2C.R /* Actual RAM Pointer Address for Region 2c */
#define GTM_gtm_cls0_DPLL_APS_1C3 GTM_gtm_cls0.DPLL_APS_1C3.R /* Actual RAM Pointer Address for RAM region 1c3 */
#define GTM_gtm_cls0_DPLL_NUTC GTM_gtm_cls0.DPLL_NUTC.R    /* Number of Recent TRIGGER Events used for Calculations */
#define GTM_gtm_cls0_DPLL_NUSC GTM_gtm_cls0.DPLL_NUSC.R    /* Number of Recent STATE Events used for Calculations */
#define GTM_gtm_cls0_DPLL_NTI_CNT GTM_gtm_cls0.DPLL_NTI_CNT.R /* Number of Active TRIGGER Events to Interrupt */
#define GTM_gtm_cls0_DPLL_IRQ_NOTIFY GTM_gtm_cls0.DPLL_IRQ_NOTIFY.R /* Interrupt Register */
#define GTM_gtm_cls0_DPLL_IRQ_EN GTM_gtm_cls0.DPLL_IRQ_EN.R /* Interrupt Enable Register */
#define GTM_gtm_cls0_DPLL_IRQ_FORCINT GTM_gtm_cls0.DPLL_IRQ_FORCINT.R /* Force Interrupt Register */
#define GTM_gtm_cls0_DPLL_IRQ_MODE GTM_gtm_cls0.DPLL_IRQ_MODE.R /* Interrupt Request Mode */
#define GTM_gtm_cls0_DPLL_EIRQ_EN GTM_gtm_cls0.DPLL_EIRQ_EN.R /* Error Interrupt Enable Register */
#define GTM_gtm_cls0_DPLL_INC_CNT1 GTM_gtm_cls0.DPLL_INC_CNT1.R /* Counter Value of Sent SUB_INC1 Pulses */
#define GTM_gtm_cls0_DPLL_INC_CNT2 GTM_gtm_cls0.DPLL_INC_CNT2.R /* Counter Value of sent SUB_INC2 values (for DPLL_CTRL_1.SMC=1 and DPLL_CTRL_0.RMO=1) */
#define GTM_gtm_cls0_DPLL_APT_SYNC GTM_gtm_cls0.DPLL_APT_SYNC.R /* TRIGGER Time Stamp Field Offset at Synchronization Time */
#define GTM_gtm_cls0_DPLL_APS_SYNC GTM_gtm_cls0.DPLL_APS_SYNC.R /* STATE Time Stamp Field Offset at Synchronization Time */
#define GTM_gtm_cls0_DPLL_TBU_TS0_T GTM_gtm_cls0.DPLL_TBU_TS0_T.R /* Time Stamp Value for the last active TRIGGER */
#define GTM_gtm_cls0_DPLL_TBU_TS0_S GTM_gtm_cls0.DPLL_TBU_TS0_S.R /* Time Stamp Value for the last active STATE */
#define GTM_gtm_cls0_DPLL_ADD_IN_LD1 GTM_gtm_cls0.DPLL_ADD_IN_LD1.R /* ADD_IN Value in Direct Load Mode for TRIGGER */
#define GTM_gtm_cls0_DPLL_ADD_IN_LD2 GTM_gtm_cls0.DPLL_ADD_IN_LD2.R /* ADD_IN Value in Direct Load Mode for STATE */
#define GTM_gtm_cls0_DPLL_STATUS GTM_gtm_cls0.DPLL_STATUS.R /* Status Register */
#define GTM_gtm_cls0_DPLL_ID_PMTR_0 GTM_gtm_cls0.DPLL_ID_PMTR_0.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_1 GTM_gtm_cls0.DPLL_ID_PMTR_1.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_2 GTM_gtm_cls0.DPLL_ID_PMTR_2.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_3 GTM_gtm_cls0.DPLL_ID_PMTR_3.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_4 GTM_gtm_cls0.DPLL_ID_PMTR_4.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_5 GTM_gtm_cls0.DPLL_ID_PMTR_5.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_6 GTM_gtm_cls0.DPLL_ID_PMTR_6.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_7 GTM_gtm_cls0.DPLL_ID_PMTR_7.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_8 GTM_gtm_cls0.DPLL_ID_PMTR_8.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_9 GTM_gtm_cls0.DPLL_ID_PMTR_9.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_10 GTM_gtm_cls0.DPLL_ID_PMTR_10.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_11 GTM_gtm_cls0.DPLL_ID_PMTR_11.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_12 GTM_gtm_cls0.DPLL_ID_PMTR_12.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_13 GTM_gtm_cls0.DPLL_ID_PMTR_13.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_14 GTM_gtm_cls0.DPLL_ID_PMTR_14.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_15 GTM_gtm_cls0.DPLL_ID_PMTR_15.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_16 GTM_gtm_cls0.DPLL_ID_PMTR_16.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_17 GTM_gtm_cls0.DPLL_ID_PMTR_17.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_18 GTM_gtm_cls0.DPLL_ID_PMTR_18.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_19 GTM_gtm_cls0.DPLL_ID_PMTR_19.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_20 GTM_gtm_cls0.DPLL_ID_PMTR_20.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_21 GTM_gtm_cls0.DPLL_ID_PMTR_21.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_22 GTM_gtm_cls0.DPLL_ID_PMTR_22.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_23 GTM_gtm_cls0.DPLL_ID_PMTR_23.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_24 GTM_gtm_cls0.DPLL_ID_PMTR_24.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_25 GTM_gtm_cls0.DPLL_ID_PMTR_25.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_26 GTM_gtm_cls0.DPLL_ID_PMTR_26.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_27 GTM_gtm_cls0.DPLL_ID_PMTR_27.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_28 GTM_gtm_cls0.DPLL_ID_PMTR_28.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_29 GTM_gtm_cls0.DPLL_ID_PMTR_29.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_30 GTM_gtm_cls0.DPLL_ID_PMTR_30.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_ID_PMTR_31 GTM_gtm_cls0.DPLL_ID_PMTR_31.R /* ID Information for Input Signal PMTR[n] (Position minus Time Request) */
#define GTM_gtm_cls0_DPLL_CTRL_0_SHADOW_TRIGGER GTM_gtm_cls0.DPLL_CTRL_0_SHADOW_TRIGGER.R /* Shadow Register of DPLL_CTRL_0 controlled by an active TRIGGER Slope */
#define GTM_gtm_cls0_DPLL_CTRL_0_SHADOW_STATE GTM_gtm_cls0.DPLL_CTRL_0_SHADOW_STATE.R /* Shadow Register of DPLL_CTRL_0 controlled by an active STATE Slope */
#define GTM_gtm_cls0_DPLL_CTRL_1_SHADOW_TRIGGER GTM_gtm_cls0.DPLL_CTRL_1_SHADOW_TRIGGER.R /* Shadow Register of DPLL_CTRL_1 controlled by an active TRIGGER Slope */
#define GTM_gtm_cls0_DPLL_CTRL_1_SHADOW_STATE GTM_gtm_cls0.DPLL_CTRL_1_SHADOW_STATE.R /* DPLL Shadow Register of DPLL_CTRL_1 controlled by an active STATE Slope */
#define GTM_gtm_cls0_DPLL_RAM_INI GTM_gtm_cls0.DPLL_RAM_INI.R /* Register to control the RAM Initialization */
#define GTM_gtm_cls0_DPLL_PSA0 GTM_gtm_cls0.DPLL_PSA0.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA1 GTM_gtm_cls0.DPLL_PSA1.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA2 GTM_gtm_cls0.DPLL_PSA2.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA3 GTM_gtm_cls0.DPLL_PSA3.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA4 GTM_gtm_cls0.DPLL_PSA4.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA5 GTM_gtm_cls0.DPLL_PSA5.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA6 GTM_gtm_cls0.DPLL_PSA6.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA7 GTM_gtm_cls0.DPLL_PSA7.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA8 GTM_gtm_cls0.DPLL_PSA8.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA9 GTM_gtm_cls0.DPLL_PSA9.R    /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA10 GTM_gtm_cls0.DPLL_PSA10.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA11 GTM_gtm_cls0.DPLL_PSA11.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA12 GTM_gtm_cls0.DPLL_PSA12.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA13 GTM_gtm_cls0.DPLL_PSA13.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA14 GTM_gtm_cls0.DPLL_PSA14.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA15 GTM_gtm_cls0.DPLL_PSA15.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA16 GTM_gtm_cls0.DPLL_PSA16.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA17 GTM_gtm_cls0.DPLL_PSA17.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA18 GTM_gtm_cls0.DPLL_PSA18.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA19 GTM_gtm_cls0.DPLL_PSA19.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA20 GTM_gtm_cls0.DPLL_PSA20.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA21 GTM_gtm_cls0.DPLL_PSA21.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA22 GTM_gtm_cls0.DPLL_PSA22.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA23 GTM_gtm_cls0.DPLL_PSA23.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA24 GTM_gtm_cls0.DPLL_PSA24.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA25 GTM_gtm_cls0.DPLL_PSA25.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA26 GTM_gtm_cls0.DPLL_PSA26.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA27 GTM_gtm_cls0.DPLL_PSA27.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA28 GTM_gtm_cls0.DPLL_PSA28.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA29 GTM_gtm_cls0.DPLL_PSA29.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA30 GTM_gtm_cls0.DPLL_PSA30.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_PSA31 GTM_gtm_cls0.DPLL_PSA31.R  /* Position Request for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA0 GTM_gtm_cls0.DPLL_DLA0.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA1 GTM_gtm_cls0.DPLL_DLA1.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA2 GTM_gtm_cls0.DPLL_DLA2.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA3 GTM_gtm_cls0.DPLL_DLA3.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA4 GTM_gtm_cls0.DPLL_DLA4.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA5 GTM_gtm_cls0.DPLL_DLA5.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA6 GTM_gtm_cls0.DPLL_DLA6.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA7 GTM_gtm_cls0.DPLL_DLA7.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA8 GTM_gtm_cls0.DPLL_DLA8.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA9 GTM_gtm_cls0.DPLL_DLA9.R    /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA10 GTM_gtm_cls0.DPLL_DLA10.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA11 GTM_gtm_cls0.DPLL_DLA11.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA12 GTM_gtm_cls0.DPLL_DLA12.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA13 GTM_gtm_cls0.DPLL_DLA13.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA14 GTM_gtm_cls0.DPLL_DLA14.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA15 GTM_gtm_cls0.DPLL_DLA15.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA16 GTM_gtm_cls0.DPLL_DLA16.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA17 GTM_gtm_cls0.DPLL_DLA17.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA18 GTM_gtm_cls0.DPLL_DLA18.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA19 GTM_gtm_cls0.DPLL_DLA19.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA20 GTM_gtm_cls0.DPLL_DLA20.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA21 GTM_gtm_cls0.DPLL_DLA21.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA22 GTM_gtm_cls0.DPLL_DLA22.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA23 GTM_gtm_cls0.DPLL_DLA23.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA24 GTM_gtm_cls0.DPLL_DLA24.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA25 GTM_gtm_cls0.DPLL_DLA25.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA26 GTM_gtm_cls0.DPLL_DLA26.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA27 GTM_gtm_cls0.DPLL_DLA27.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA28 GTM_gtm_cls0.DPLL_DLA28.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA29 GTM_gtm_cls0.DPLL_DLA29.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA30 GTM_gtm_cls0.DPLL_DLA30.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_DLA31 GTM_gtm_cls0.DPLL_DLA31.R  /* Time to React for Action [n] */
#define GTM_gtm_cls0_DPLL_NA0 GTM_gtm_cls0.DPLL_NA0.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA1 GTM_gtm_cls0.DPLL_NA1.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA2 GTM_gtm_cls0.DPLL_NA2.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA3 GTM_gtm_cls0.DPLL_NA3.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA4 GTM_gtm_cls0.DPLL_NA4.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA5 GTM_gtm_cls0.DPLL_NA5.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA6 GTM_gtm_cls0.DPLL_NA6.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA7 GTM_gtm_cls0.DPLL_NA7.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA8 GTM_gtm_cls0.DPLL_NA8.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA9 GTM_gtm_cls0.DPLL_NA9.R      /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA10 GTM_gtm_cls0.DPLL_NA10.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA11 GTM_gtm_cls0.DPLL_NA11.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA12 GTM_gtm_cls0.DPLL_NA12.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA13 GTM_gtm_cls0.DPLL_NA13.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA14 GTM_gtm_cls0.DPLL_NA14.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA15 GTM_gtm_cls0.DPLL_NA15.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA16 GTM_gtm_cls0.DPLL_NA16.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA17 GTM_gtm_cls0.DPLL_NA17.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA18 GTM_gtm_cls0.DPLL_NA18.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA19 GTM_gtm_cls0.DPLL_NA19.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA20 GTM_gtm_cls0.DPLL_NA20.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA21 GTM_gtm_cls0.DPLL_NA21.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA22 GTM_gtm_cls0.DPLL_NA22.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA23 GTM_gtm_cls0.DPLL_NA23.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA24 GTM_gtm_cls0.DPLL_NA24.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA25 GTM_gtm_cls0.DPLL_NA25.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA26 GTM_gtm_cls0.DPLL_NA26.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA27 GTM_gtm_cls0.DPLL_NA27.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA28 GTM_gtm_cls0.DPLL_NA28.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA29 GTM_gtm_cls0.DPLL_NA29.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA30 GTM_gtm_cls0.DPLL_NA30.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_NA31 GTM_gtm_cls0.DPLL_NA31.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA0 GTM_gtm_cls0.DPLL_DTA0.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA1 GTM_gtm_cls0.DPLL_DTA1.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA2 GTM_gtm_cls0.DPLL_DTA2.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA3 GTM_gtm_cls0.DPLL_DTA3.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA4 GTM_gtm_cls0.DPLL_DTA4.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA5 GTM_gtm_cls0.DPLL_DTA5.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA6 GTM_gtm_cls0.DPLL_DTA6.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA7 GTM_gtm_cls0.DPLL_DTA7.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA8 GTM_gtm_cls0.DPLL_DTA8.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA9 GTM_gtm_cls0.DPLL_DTA9.R    /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA10 GTM_gtm_cls0.DPLL_DTA10.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA11 GTM_gtm_cls0.DPLL_DTA11.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA12 GTM_gtm_cls0.DPLL_DTA12.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA13 GTM_gtm_cls0.DPLL_DTA13.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA14 GTM_gtm_cls0.DPLL_DTA14.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA15 GTM_gtm_cls0.DPLL_DTA15.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA16 GTM_gtm_cls0.DPLL_DTA16.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA17 GTM_gtm_cls0.DPLL_DTA17.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA18 GTM_gtm_cls0.DPLL_DTA18.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA19 GTM_gtm_cls0.DPLL_DTA19.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA20 GTM_gtm_cls0.DPLL_DTA20.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA21 GTM_gtm_cls0.DPLL_DTA21.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA22 GTM_gtm_cls0.DPLL_DTA22.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA23 GTM_gtm_cls0.DPLL_DTA23.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA24 GTM_gtm_cls0.DPLL_DTA24.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA25 GTM_gtm_cls0.DPLL_DTA25.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA26 GTM_gtm_cls0.DPLL_DTA26.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA27 GTM_gtm_cls0.DPLL_DTA27.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA28 GTM_gtm_cls0.DPLL_DTA28.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA29 GTM_gtm_cls0.DPLL_DTA29.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA30 GTM_gtm_cls0.DPLL_DTA30.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_DTA31 GTM_gtm_cls0.DPLL_DTA31.R  /* Calculated Relative Time to Action [n] */
#define GTM_gtm_cls0_DPLL_TS_T GTM_gtm_cls0.DPLL_TS_T.R    /* Actual TRIGGER Time Stamp Value */
#define GTM_gtm_cls0_DPLL_TS_T_OLD GTM_gtm_cls0.DPLL_TS_T_OLD.R /* Previous TRIGGER Time Stamp Value */
#define GTM_gtm_cls0_DPLL_FTV_T GTM_gtm_cls0.DPLL_FTV_T.R  /* Actual TRIGGER Filter value */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_0 GTM_gtm_cls0.DPLL_RAM1B_RSVD_0.R /* DPLL RAM1B reserved data */
#define GTM_gtm_cls0_DPLL_TS_S GTM_gtm_cls0.DPLL_TS_S.R    /* Actual STATE Time Stamp Register */
#define GTM_gtm_cls0_DPLL_TS_S_OLD GTM_gtm_cls0.DPLL_TS_S_OLD.R /* Previous STATE Time Stamp Register */
#define GTM_gtm_cls0_DPLL_FTV_S GTM_gtm_cls0.DPLL_FTV_S.R  /* Actual STATE Filter Value */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_1 GTM_gtm_cls0.DPLL_RAM1B_RSVD_1.R /* DPLL RAM1B reserved data */
#define GTM_gtm_cls0_DPLL_THMI GTM_gtm_cls0.DPLL_THMI.R    /* TRIGGER Hold Time Min. Value */
#define GTM_gtm_cls0_DPLL_THMA GTM_gtm_cls0.DPLL_THMA.R    /* TRIGGER Hold Time Max. Value */
#define GTM_gtm_cls0_DPLL_THVAL GTM_gtm_cls0.DPLL_THVAL.R  /* Measured TRIGGER Hold Time Value */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_2 GTM_gtm_cls0.DPLL_RAM1B_RSVD_2.R /* DPLL RAM1B reserved data */
#define GTM_gtm_cls0_DPLL_TOV GTM_gtm_cls0.DPLL_TOV.R      /* Time Out Value of Active TRIGGER Slope (for missing TRIGGER generation) */
#define GTM_gtm_cls0_DPLL_TOV_S GTM_gtm_cls0.DPLL_TOV_S.R  /* Time Out Value of active STATE Slope (for missing STATE generation) */
#define GTM_gtm_cls0_DPLL_ADD_IN_CAL1 GTM_gtm_cls0.DPLL_ADD_IN_CAL1.R /* Calculated ADD_IN Value for SUB_INC1 Generation */
#define GTM_gtm_cls0_DPLL_ADD_IN_CAL2 GTM_gtm_cls0.DPLL_ADD_IN_CAL2.R /* Calculated ADD_IN Value for SUB_INC2 Generation */
#define GTM_gtm_cls0_DPLL_MPVAL1 GTM_gtm_cls0.DPLL_MPVAL1.R /* Missing Pulses to be Added or Subtracted Directly */
#define GTM_gtm_cls0_DPLL_MPVAL2 GTM_gtm_cls0.DPLL_MPVAL2.R /* Missing Pulses to be Added or Subtracted Directly */
#define GTM_gtm_cls0_DPLL_NMB_T_TAR GTM_gtm_cls0.DPLL_NMB_T_TAR.R /* Target Number of Pulses to be sent in Normal Mode */
#define GTM_gtm_cls0_DPLL_NMB_T_TAR_OLD GTM_gtm_cls0.DPLL_NMB_T_TAR_OLD.R /* Last but one Target Number of Pulses to be sent in Normal Mode */
#define GTM_gtm_cls0_DPLL_NMB_S_TAR GTM_gtm_cls0.DPLL_NMB_S_TAR.R /* Target Number of Pulses to be sent in Emergency Mode */
#define GTM_gtm_cls0_DPLL_NMB_S_TAR_OLD GTM_gtm_cls0.DPLL_NMB_S_TAR_OLD.R /* Last but one Target Number of Pulses to be sent in Emergency Mode */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_3_0 GTM_gtm_cls0.DPLL_RAM1B_RSVD_3_0.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_3_1 GTM_gtm_cls0.DPLL_RAM1B_RSVD_3_1.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RCDT_TX GTM_gtm_cls0.DPLL_RCDT_TX.R /* Reciprocal Value of the Expected Increment Duration of TRIGGER */
#define GTM_gtm_cls0_DPLL_RCDT_SX GTM_gtm_cls0.DPLL_RCDT_SX.R /* Reciprocal Value of the Expected Increment Duration of STATE */
#define GTM_gtm_cls0_DPLL_RCDT_TX_NOM GTM_gtm_cls0.DPLL_RCDT_TX_NOM.R /* Reciprocal Value of the Expected Nominal Increment Duration of TRIGGER */
#define GTM_gtm_cls0_DPLL_RCDT_SX_NOM GTM_gtm_cls0.DPLL_RCDT_SX_NOM.R /* Reciprocal Value of the Expected Nominal Increment Duration of STATE */
#define GTM_gtm_cls0_DPLL_RDT_T_ACT GTM_gtm_cls0.DPLL_RDT_T_ACT.R /* Reciprocal Value of the Last Increment of TRIGGER */
#define GTM_gtm_cls0_DPLL_RDT_S_ACT GTM_gtm_cls0.DPLL_RDT_S_ACT.R /* Reciprocal Value of the Last Increment of STATE */
#define GTM_gtm_cls0_DPLL_DT_T_ACT GTM_gtm_cls0.DPLL_DT_T_ACT.R /* Duration of the Last TRIGGER Increment */
#define GTM_gtm_cls0_DPLL_DT_S_ACT GTM_gtm_cls0.DPLL_DT_S_ACT.R /* Duration of the Last STATE Increment */
#define GTM_gtm_cls0_DPLL_EDT_T GTM_gtm_cls0.DPLL_EDT_T.R  /* Difference of Prediction to Actual Value of the Last TRIGGER Increment */
#define GTM_gtm_cls0_DPLL_MEDT_T GTM_gtm_cls0.DPLL_MEDT_T.R /* Weighted Difference of Prediction Errors of TRIGGER */
#define GTM_gtm_cls0_DPLL_EDT_S GTM_gtm_cls0.DPLL_EDT_S.R  /* Difference of Prediction to Actual Value of the Last STATE Increment */
#define GTM_gtm_cls0_DPLL_MEDT_S GTM_gtm_cls0.DPLL_MEDT_S.R /* Weighted Difference of Prediction Errors of STATE */
#define GTM_gtm_cls0_DPLL_CDT_TX GTM_gtm_cls0.DPLL_CDT_TX.R /* Prediction of the Actual TRIGGER Increment Duration */
#define GTM_gtm_cls0_DPLL_CDT_SX GTM_gtm_cls0.DPLL_CDT_SX.R /* Prediction of the Actual STATE Increment Duration */
#define GTM_gtm_cls0_DPLL_CDT_TX_NOM GTM_gtm_cls0.DPLL_CDT_TX_NOM.R /* Prediction of the Nominal TRIGGER Increment Duration */
#define GTM_gtm_cls0_DPLL_CDT_SX_NOM GTM_gtm_cls0.DPLL_CDT_SX_NOM.R /* Prediction of the Nominal STATE Increment Duration */
#define GTM_gtm_cls0_DPLL_TLR GTM_gtm_cls0.DPLL_TLR.R      /* TRIGGER Locking Range */
#define GTM_gtm_cls0_DPLL_SLR GTM_gtm_cls0.DPLL_SLR.R      /* STATE Locking Range */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_0 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_0.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_1 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_1.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_2 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_2.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_3 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_3.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_4 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_4.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_5 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_5.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_6 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_6.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_7 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_7.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_8 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_8.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_9 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_9.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_10 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_10.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_11 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_11.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_12 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_12.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_13 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_13.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_14 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_14.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_15 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_15.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_16 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_16.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_17 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_17.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_18 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_18.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_19 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_19.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_20 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_20.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_4_21 GTM_gtm_cls0.DPLL_RAM1B_RSVD_4_21.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_PDT_0 GTM_gtm_cls0.DPLL_PDT_0.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_1 GTM_gtm_cls0.DPLL_PDT_1.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_2 GTM_gtm_cls0.DPLL_PDT_2.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_3 GTM_gtm_cls0.DPLL_PDT_3.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_4 GTM_gtm_cls0.DPLL_PDT_4.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_5 GTM_gtm_cls0.DPLL_PDT_5.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_6 GTM_gtm_cls0.DPLL_PDT_6.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_7 GTM_gtm_cls0.DPLL_PDT_7.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_8 GTM_gtm_cls0.DPLL_PDT_8.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_9 GTM_gtm_cls0.DPLL_PDT_9.R  /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_10 GTM_gtm_cls0.DPLL_PDT_10.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_11 GTM_gtm_cls0.DPLL_PDT_11.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_12 GTM_gtm_cls0.DPLL_PDT_12.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_13 GTM_gtm_cls0.DPLL_PDT_13.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_14 GTM_gtm_cls0.DPLL_PDT_14.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_15 GTM_gtm_cls0.DPLL_PDT_15.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_16 GTM_gtm_cls0.DPLL_PDT_16.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_17 GTM_gtm_cls0.DPLL_PDT_17.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_18 GTM_gtm_cls0.DPLL_PDT_18.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_19 GTM_gtm_cls0.DPLL_PDT_19.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_20 GTM_gtm_cls0.DPLL_PDT_20.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_21 GTM_gtm_cls0.DPLL_PDT_21.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_22 GTM_gtm_cls0.DPLL_PDT_22.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_23 GTM_gtm_cls0.DPLL_PDT_23.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_24 GTM_gtm_cls0.DPLL_PDT_24.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_25 GTM_gtm_cls0.DPLL_PDT_25.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_26 GTM_gtm_cls0.DPLL_PDT_26.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_27 GTM_gtm_cls0.DPLL_PDT_27.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_28 GTM_gtm_cls0.DPLL_PDT_28.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_29 GTM_gtm_cls0.DPLL_PDT_29.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_30 GTM_gtm_cls0.DPLL_PDT_30.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_PDT_31 GTM_gtm_cls0.DPLL_PDT_31.R /* Projected Increment Sum Relations for Action [n] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_0 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_0.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_1 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_1.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_2 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_2.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_3 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_3.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_4 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_4.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_5 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_5.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_6 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_6.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_7 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_7.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_8 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_8.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_9 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_9.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_10 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_10.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_11 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_11.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_12 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_12.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_13 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_13.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_14 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_14.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_5_15 GTM_gtm_cls0.DPLL_RAM1B_RSVD_5_15.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_MLS1 GTM_gtm_cls0.DPLL_MLS1.R    /* Calculated Number of Sub-Pulses between two nominal STATE Events for DPLL_CTRL_1.SMC = 0 */
#define GTM_gtm_cls0_DPLL_MLS2 GTM_gtm_cls0.DPLL_MLS2.R    /* Calculated Number of Sub-Pulses between two nominal STATE Events for DPLL_CTRL_1.SMC=1 and RMO=1 */
#define GTM_gtm_cls0_DPLL_CNT_NUM_1 GTM_gtm_cls0.DPLL_CNT_NUM_1.R /* Number of SUB_INC1 pulses in continuous mode */
#define GTM_gtm_cls0_DPLL_CNT_NUM_2 GTM_gtm_cls0.DPLL_CNT_NUM_2.R /* Number of SUB_INC2 pulses in continuous mode */
#define GTM_gtm_cls0_DPLL_PVT GTM_gtm_cls0.DPLL_PVT.R      /* Plausibility Value of Next TRIGGER Slope */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_0 GTM_gtm_cls0.DPLL_RAM1B_RSVD_6_0.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_1 GTM_gtm_cls0.DPLL_RAM1B_RSVD_6_1.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_RAM1B_RSVD_6_2 GTM_gtm_cls0.DPLL_RAM1B_RSVD_6_2.R /* DPLL RAM1B reserved data [k] */
#define GTM_gtm_cls0_DPLL_PSTC GTM_gtm_cls0.DPLL_PSTC.R    /* Actual Calculated Position Stamp of TRIGGER */
#define GTM_gtm_cls0_DPLL_PSSC GTM_gtm_cls0.DPLL_PSSC.R    /* Actual Calculated Position Stamp of STATE */
#define GTM_gtm_cls0_DPLL_PSTM GTM_gtm_cls0.DPLL_PSTM.R    /* Measured Position Stamp at Last TRIGGER Input */
#define GTM_gtm_cls0_DPLL_PSTM_OLD GTM_gtm_cls0.DPLL_PSTM_OLD.R /* Measured Position Stamp at Last but one TRIGGER Input */
#define GTM_gtm_cls0_DPLL_PSSM GTM_gtm_cls0.DPLL_PSSM.R    /* Measured Position Stamp at Last STATE Input */
#define GTM_gtm_cls0_DPLL_PSSM_OLD GTM_gtm_cls0.DPLL_PSSM_OLD.R /* Measured Position Stamp at Last but one STATE Input */
#define GTM_gtm_cls0_DPLL_NMB_T GTM_gtm_cls0.DPLL_NMB_T.R  /* Number of Pulses to be sent in Normal Mode */
#define GTM_gtm_cls0_DPLL_NMB_S GTM_gtm_cls0.DPLL_NMB_S.R  /* Number of Pulses to be sent in Emergency Mode */
#define GTM_gtm_cls0_DPLL_RDT_S0 GTM_gtm_cls0.DPLL_RDT_S0.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S1 GTM_gtm_cls0.DPLL_RDT_S1.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S2 GTM_gtm_cls0.DPLL_RDT_S2.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S3 GTM_gtm_cls0.DPLL_RDT_S3.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S4 GTM_gtm_cls0.DPLL_RDT_S4.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S5 GTM_gtm_cls0.DPLL_RDT_S5.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S6 GTM_gtm_cls0.DPLL_RDT_S6.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S7 GTM_gtm_cls0.DPLL_RDT_S7.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S8 GTM_gtm_cls0.DPLL_RDT_S8.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S9 GTM_gtm_cls0.DPLL_RDT_S9.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S10 GTM_gtm_cls0.DPLL_RDT_S10.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S11 GTM_gtm_cls0.DPLL_RDT_S11.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S12 GTM_gtm_cls0.DPLL_RDT_S12.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S13 GTM_gtm_cls0.DPLL_RDT_S13.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S14 GTM_gtm_cls0.DPLL_RDT_S14.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S15 GTM_gtm_cls0.DPLL_RDT_S15.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S16 GTM_gtm_cls0.DPLL_RDT_S16.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S17 GTM_gtm_cls0.DPLL_RDT_S17.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S18 GTM_gtm_cls0.DPLL_RDT_S18.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S19 GTM_gtm_cls0.DPLL_RDT_S19.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S20 GTM_gtm_cls0.DPLL_RDT_S20.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S21 GTM_gtm_cls0.DPLL_RDT_S21.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S22 GTM_gtm_cls0.DPLL_RDT_S22.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S23 GTM_gtm_cls0.DPLL_RDT_S23.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S24 GTM_gtm_cls0.DPLL_RDT_S24.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S25 GTM_gtm_cls0.DPLL_RDT_S25.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S26 GTM_gtm_cls0.DPLL_RDT_S26.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S27 GTM_gtm_cls0.DPLL_RDT_S27.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S28 GTM_gtm_cls0.DPLL_RDT_S28.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S29 GTM_gtm_cls0.DPLL_RDT_S29.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S30 GTM_gtm_cls0.DPLL_RDT_S30.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S31 GTM_gtm_cls0.DPLL_RDT_S31.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S32 GTM_gtm_cls0.DPLL_RDT_S32.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S33 GTM_gtm_cls0.DPLL_RDT_S33.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S34 GTM_gtm_cls0.DPLL_RDT_S34.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S35 GTM_gtm_cls0.DPLL_RDT_S35.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S36 GTM_gtm_cls0.DPLL_RDT_S36.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S37 GTM_gtm_cls0.DPLL_RDT_S37.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S38 GTM_gtm_cls0.DPLL_RDT_S38.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S39 GTM_gtm_cls0.DPLL_RDT_S39.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S40 GTM_gtm_cls0.DPLL_RDT_S40.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S41 GTM_gtm_cls0.DPLL_RDT_S41.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S42 GTM_gtm_cls0.DPLL_RDT_S42.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S43 GTM_gtm_cls0.DPLL_RDT_S43.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S44 GTM_gtm_cls0.DPLL_RDT_S44.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S45 GTM_gtm_cls0.DPLL_RDT_S45.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S46 GTM_gtm_cls0.DPLL_RDT_S46.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S47 GTM_gtm_cls0.DPLL_RDT_S47.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S48 GTM_gtm_cls0.DPLL_RDT_S48.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S49 GTM_gtm_cls0.DPLL_RDT_S49.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S50 GTM_gtm_cls0.DPLL_RDT_S50.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S51 GTM_gtm_cls0.DPLL_RDT_S51.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S52 GTM_gtm_cls0.DPLL_RDT_S52.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S53 GTM_gtm_cls0.DPLL_RDT_S53.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S54 GTM_gtm_cls0.DPLL_RDT_S54.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S55 GTM_gtm_cls0.DPLL_RDT_S55.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S56 GTM_gtm_cls0.DPLL_RDT_S56.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S57 GTM_gtm_cls0.DPLL_RDT_S57.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S58 GTM_gtm_cls0.DPLL_RDT_S58.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S59 GTM_gtm_cls0.DPLL_RDT_S59.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S60 GTM_gtm_cls0.DPLL_RDT_S60.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S61 GTM_gtm_cls0.DPLL_RDT_S61.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S62 GTM_gtm_cls0.DPLL_RDT_S62.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_RDT_S63 GTM_gtm_cls0.DPLL_RDT_S63.R /* Reciprocal Values of the Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S0 GTM_gtm_cls0.DPLL_TSF_S0.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S1 GTM_gtm_cls0.DPLL_TSF_S1.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S2 GTM_gtm_cls0.DPLL_TSF_S2.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S3 GTM_gtm_cls0.DPLL_TSF_S3.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S4 GTM_gtm_cls0.DPLL_TSF_S4.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S5 GTM_gtm_cls0.DPLL_TSF_S5.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S6 GTM_gtm_cls0.DPLL_TSF_S6.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S7 GTM_gtm_cls0.DPLL_TSF_S7.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S8 GTM_gtm_cls0.DPLL_TSF_S8.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S9 GTM_gtm_cls0.DPLL_TSF_S9.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S10 GTM_gtm_cls0.DPLL_TSF_S10.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S11 GTM_gtm_cls0.DPLL_TSF_S11.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S12 GTM_gtm_cls0.DPLL_TSF_S12.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S13 GTM_gtm_cls0.DPLL_TSF_S13.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S14 GTM_gtm_cls0.DPLL_TSF_S14.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S15 GTM_gtm_cls0.DPLL_TSF_S15.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S16 GTM_gtm_cls0.DPLL_TSF_S16.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S17 GTM_gtm_cls0.DPLL_TSF_S17.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S18 GTM_gtm_cls0.DPLL_TSF_S18.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S19 GTM_gtm_cls0.DPLL_TSF_S19.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S20 GTM_gtm_cls0.DPLL_TSF_S20.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S21 GTM_gtm_cls0.DPLL_TSF_S21.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S22 GTM_gtm_cls0.DPLL_TSF_S22.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S23 GTM_gtm_cls0.DPLL_TSF_S23.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S24 GTM_gtm_cls0.DPLL_TSF_S24.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S25 GTM_gtm_cls0.DPLL_TSF_S25.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S26 GTM_gtm_cls0.DPLL_TSF_S26.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S27 GTM_gtm_cls0.DPLL_TSF_S27.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S28 GTM_gtm_cls0.DPLL_TSF_S28.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S29 GTM_gtm_cls0.DPLL_TSF_S29.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S30 GTM_gtm_cls0.DPLL_TSF_S30.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S31 GTM_gtm_cls0.DPLL_TSF_S31.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S32 GTM_gtm_cls0.DPLL_TSF_S32.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S33 GTM_gtm_cls0.DPLL_TSF_S33.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S34 GTM_gtm_cls0.DPLL_TSF_S34.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S35 GTM_gtm_cls0.DPLL_TSF_S35.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S36 GTM_gtm_cls0.DPLL_TSF_S36.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S37 GTM_gtm_cls0.DPLL_TSF_S37.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S38 GTM_gtm_cls0.DPLL_TSF_S38.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S39 GTM_gtm_cls0.DPLL_TSF_S39.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S40 GTM_gtm_cls0.DPLL_TSF_S40.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S41 GTM_gtm_cls0.DPLL_TSF_S41.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S42 GTM_gtm_cls0.DPLL_TSF_S42.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S43 GTM_gtm_cls0.DPLL_TSF_S43.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S44 GTM_gtm_cls0.DPLL_TSF_S44.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S45 GTM_gtm_cls0.DPLL_TSF_S45.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S46 GTM_gtm_cls0.DPLL_TSF_S46.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S47 GTM_gtm_cls0.DPLL_TSF_S47.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S48 GTM_gtm_cls0.DPLL_TSF_S48.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S49 GTM_gtm_cls0.DPLL_TSF_S49.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S50 GTM_gtm_cls0.DPLL_TSF_S50.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S51 GTM_gtm_cls0.DPLL_TSF_S51.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S52 GTM_gtm_cls0.DPLL_TSF_S52.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S53 GTM_gtm_cls0.DPLL_TSF_S53.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S54 GTM_gtm_cls0.DPLL_TSF_S54.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S55 GTM_gtm_cls0.DPLL_TSF_S55.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S56 GTM_gtm_cls0.DPLL_TSF_S56.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S57 GTM_gtm_cls0.DPLL_TSF_S57.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S58 GTM_gtm_cls0.DPLL_TSF_S58.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S59 GTM_gtm_cls0.DPLL_TSF_S59.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S60 GTM_gtm_cls0.DPLL_TSF_S60.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S61 GTM_gtm_cls0.DPLL_TSF_S61.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S62 GTM_gtm_cls0.DPLL_TSF_S62.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSF_S63 GTM_gtm_cls0.DPLL_TSF_S63.R /* Time Stamp Values of the Nominal STATE Events in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_ADT_S0 GTM_gtm_cls0.DPLL_ADT_S0.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S1 GTM_gtm_cls0.DPLL_ADT_S1.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S2 GTM_gtm_cls0.DPLL_ADT_S2.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S3 GTM_gtm_cls0.DPLL_ADT_S3.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S4 GTM_gtm_cls0.DPLL_ADT_S4.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S5 GTM_gtm_cls0.DPLL_ADT_S5.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S6 GTM_gtm_cls0.DPLL_ADT_S6.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S7 GTM_gtm_cls0.DPLL_ADT_S7.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S8 GTM_gtm_cls0.DPLL_ADT_S8.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S9 GTM_gtm_cls0.DPLL_ADT_S9.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S10 GTM_gtm_cls0.DPLL_ADT_S10.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S11 GTM_gtm_cls0.DPLL_ADT_S11.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S12 GTM_gtm_cls0.DPLL_ADT_S12.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S13 GTM_gtm_cls0.DPLL_ADT_S13.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S14 GTM_gtm_cls0.DPLL_ADT_S14.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S15 GTM_gtm_cls0.DPLL_ADT_S15.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S16 GTM_gtm_cls0.DPLL_ADT_S16.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S17 GTM_gtm_cls0.DPLL_ADT_S17.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S18 GTM_gtm_cls0.DPLL_ADT_S18.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S19 GTM_gtm_cls0.DPLL_ADT_S19.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S20 GTM_gtm_cls0.DPLL_ADT_S20.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S21 GTM_gtm_cls0.DPLL_ADT_S21.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S22 GTM_gtm_cls0.DPLL_ADT_S22.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S23 GTM_gtm_cls0.DPLL_ADT_S23.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S24 GTM_gtm_cls0.DPLL_ADT_S24.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S25 GTM_gtm_cls0.DPLL_ADT_S25.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S26 GTM_gtm_cls0.DPLL_ADT_S26.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S27 GTM_gtm_cls0.DPLL_ADT_S27.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S28 GTM_gtm_cls0.DPLL_ADT_S28.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S29 GTM_gtm_cls0.DPLL_ADT_S29.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S30 GTM_gtm_cls0.DPLL_ADT_S30.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S31 GTM_gtm_cls0.DPLL_ADT_S31.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S32 GTM_gtm_cls0.DPLL_ADT_S32.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S33 GTM_gtm_cls0.DPLL_ADT_S33.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S34 GTM_gtm_cls0.DPLL_ADT_S34.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S35 GTM_gtm_cls0.DPLL_ADT_S35.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S36 GTM_gtm_cls0.DPLL_ADT_S36.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S37 GTM_gtm_cls0.DPLL_ADT_S37.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S38 GTM_gtm_cls0.DPLL_ADT_S38.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S39 GTM_gtm_cls0.DPLL_ADT_S39.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S40 GTM_gtm_cls0.DPLL_ADT_S40.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S41 GTM_gtm_cls0.DPLL_ADT_S41.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S42 GTM_gtm_cls0.DPLL_ADT_S42.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S43 GTM_gtm_cls0.DPLL_ADT_S43.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S44 GTM_gtm_cls0.DPLL_ADT_S44.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S45 GTM_gtm_cls0.DPLL_ADT_S45.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S46 GTM_gtm_cls0.DPLL_ADT_S46.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S47 GTM_gtm_cls0.DPLL_ADT_S47.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S48 GTM_gtm_cls0.DPLL_ADT_S48.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S49 GTM_gtm_cls0.DPLL_ADT_S49.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S50 GTM_gtm_cls0.DPLL_ADT_S50.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S51 GTM_gtm_cls0.DPLL_ADT_S51.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S52 GTM_gtm_cls0.DPLL_ADT_S52.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S53 GTM_gtm_cls0.DPLL_ADT_S53.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S54 GTM_gtm_cls0.DPLL_ADT_S54.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S55 GTM_gtm_cls0.DPLL_ADT_S55.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S56 GTM_gtm_cls0.DPLL_ADT_S56.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S57 GTM_gtm_cls0.DPLL_ADT_S57.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S58 GTM_gtm_cls0.DPLL_ADT_S58.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S59 GTM_gtm_cls0.DPLL_ADT_S59.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S60 GTM_gtm_cls0.DPLL_ADT_S60.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S61 GTM_gtm_cls0.DPLL_ADT_S61.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S62 GTM_gtm_cls0.DPLL_ADT_S62.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_ADT_S63 GTM_gtm_cls0.DPLL_ADT_S63.R /* Adapt and Profile Values of the STATE Increments in FULL_SCALE for maximum [p] entries */
#define GTM_gtm_cls0_DPLL_DT_S0 GTM_gtm_cls0.DPLL_DT_S0.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S1 GTM_gtm_cls0.DPLL_DT_S1.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S2 GTM_gtm_cls0.DPLL_DT_S2.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S3 GTM_gtm_cls0.DPLL_DT_S3.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S4 GTM_gtm_cls0.DPLL_DT_S4.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S5 GTM_gtm_cls0.DPLL_DT_S5.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S6 GTM_gtm_cls0.DPLL_DT_S6.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S7 GTM_gtm_cls0.DPLL_DT_S7.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S8 GTM_gtm_cls0.DPLL_DT_S8.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S9 GTM_gtm_cls0.DPLL_DT_S9.R  /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S10 GTM_gtm_cls0.DPLL_DT_S10.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S11 GTM_gtm_cls0.DPLL_DT_S11.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S12 GTM_gtm_cls0.DPLL_DT_S12.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S13 GTM_gtm_cls0.DPLL_DT_S13.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S14 GTM_gtm_cls0.DPLL_DT_S14.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S15 GTM_gtm_cls0.DPLL_DT_S15.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S16 GTM_gtm_cls0.DPLL_DT_S16.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S17 GTM_gtm_cls0.DPLL_DT_S17.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S18 GTM_gtm_cls0.DPLL_DT_S18.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S19 GTM_gtm_cls0.DPLL_DT_S19.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S20 GTM_gtm_cls0.DPLL_DT_S20.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S21 GTM_gtm_cls0.DPLL_DT_S21.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S22 GTM_gtm_cls0.DPLL_DT_S22.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S23 GTM_gtm_cls0.DPLL_DT_S23.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S24 GTM_gtm_cls0.DPLL_DT_S24.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S25 GTM_gtm_cls0.DPLL_DT_S25.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S26 GTM_gtm_cls0.DPLL_DT_S26.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S27 GTM_gtm_cls0.DPLL_DT_S27.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S28 GTM_gtm_cls0.DPLL_DT_S28.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S29 GTM_gtm_cls0.DPLL_DT_S29.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S30 GTM_gtm_cls0.DPLL_DT_S30.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S31 GTM_gtm_cls0.DPLL_DT_S31.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S32 GTM_gtm_cls0.DPLL_DT_S32.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S33 GTM_gtm_cls0.DPLL_DT_S33.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S34 GTM_gtm_cls0.DPLL_DT_S34.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S35 GTM_gtm_cls0.DPLL_DT_S35.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S36 GTM_gtm_cls0.DPLL_DT_S36.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S37 GTM_gtm_cls0.DPLL_DT_S37.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S38 GTM_gtm_cls0.DPLL_DT_S38.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S39 GTM_gtm_cls0.DPLL_DT_S39.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S40 GTM_gtm_cls0.DPLL_DT_S40.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S41 GTM_gtm_cls0.DPLL_DT_S41.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S42 GTM_gtm_cls0.DPLL_DT_S42.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S43 GTM_gtm_cls0.DPLL_DT_S43.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S44 GTM_gtm_cls0.DPLL_DT_S44.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S45 GTM_gtm_cls0.DPLL_DT_S45.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S46 GTM_gtm_cls0.DPLL_DT_S46.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S47 GTM_gtm_cls0.DPLL_DT_S47.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S48 GTM_gtm_cls0.DPLL_DT_S48.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S49 GTM_gtm_cls0.DPLL_DT_S49.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S50 GTM_gtm_cls0.DPLL_DT_S50.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S51 GTM_gtm_cls0.DPLL_DT_S51.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S52 GTM_gtm_cls0.DPLL_DT_S52.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S53 GTM_gtm_cls0.DPLL_DT_S53.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S54 GTM_gtm_cls0.DPLL_DT_S54.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S55 GTM_gtm_cls0.DPLL_DT_S55.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S56 GTM_gtm_cls0.DPLL_DT_S56.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S57 GTM_gtm_cls0.DPLL_DT_S57.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S58 GTM_gtm_cls0.DPLL_DT_S58.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S59 GTM_gtm_cls0.DPLL_DT_S59.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S60 GTM_gtm_cls0.DPLL_DT_S60.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S61 GTM_gtm_cls0.DPLL_DT_S61.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S62 GTM_gtm_cls0.DPLL_DT_S62.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_DT_S63 GTM_gtm_cls0.DPLL_DT_S63.R /* Nominal STATE Increment Duration in FULL_SCALE for maximum [p] entries in profile for STATE (DPLL_ADT_S[p]) */
#define GTM_gtm_cls0_DPLL_TSAC0 GTM_gtm_cls0.DPLL_TSAC0.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC1 GTM_gtm_cls0.DPLL_TSAC1.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC2 GTM_gtm_cls0.DPLL_TSAC2.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC3 GTM_gtm_cls0.DPLL_TSAC3.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC4 GTM_gtm_cls0.DPLL_TSAC4.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC5 GTM_gtm_cls0.DPLL_TSAC5.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC6 GTM_gtm_cls0.DPLL_TSAC6.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC7 GTM_gtm_cls0.DPLL_TSAC7.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC8 GTM_gtm_cls0.DPLL_TSAC8.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC9 GTM_gtm_cls0.DPLL_TSAC9.R  /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC10 GTM_gtm_cls0.DPLL_TSAC10.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC11 GTM_gtm_cls0.DPLL_TSAC11.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC12 GTM_gtm_cls0.DPLL_TSAC12.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC13 GTM_gtm_cls0.DPLL_TSAC13.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC14 GTM_gtm_cls0.DPLL_TSAC14.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC15 GTM_gtm_cls0.DPLL_TSAC15.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC16 GTM_gtm_cls0.DPLL_TSAC16.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC17 GTM_gtm_cls0.DPLL_TSAC17.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC18 GTM_gtm_cls0.DPLL_TSAC18.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC19 GTM_gtm_cls0.DPLL_TSAC19.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC20 GTM_gtm_cls0.DPLL_TSAC20.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC21 GTM_gtm_cls0.DPLL_TSAC21.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC22 GTM_gtm_cls0.DPLL_TSAC22.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC23 GTM_gtm_cls0.DPLL_TSAC23.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC24 GTM_gtm_cls0.DPLL_TSAC24.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC25 GTM_gtm_cls0.DPLL_TSAC25.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC26 GTM_gtm_cls0.DPLL_TSAC26.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC27 GTM_gtm_cls0.DPLL_TSAC27.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC28 GTM_gtm_cls0.DPLL_TSAC28.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC29 GTM_gtm_cls0.DPLL_TSAC29.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC30 GTM_gtm_cls0.DPLL_TSAC30.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_TSAC31 GTM_gtm_cls0.DPLL_TSAC31.R /* Calculated Time Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC0 GTM_gtm_cls0.DPLL_PSAC0.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC1 GTM_gtm_cls0.DPLL_PSAC1.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC2 GTM_gtm_cls0.DPLL_PSAC2.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC3 GTM_gtm_cls0.DPLL_PSAC3.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC4 GTM_gtm_cls0.DPLL_PSAC4.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC5 GTM_gtm_cls0.DPLL_PSAC5.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC6 GTM_gtm_cls0.DPLL_PSAC6.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC7 GTM_gtm_cls0.DPLL_PSAC7.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC8 GTM_gtm_cls0.DPLL_PSAC8.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC9 GTM_gtm_cls0.DPLL_PSAC9.R  /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC10 GTM_gtm_cls0.DPLL_PSAC10.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC11 GTM_gtm_cls0.DPLL_PSAC11.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC12 GTM_gtm_cls0.DPLL_PSAC12.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC13 GTM_gtm_cls0.DPLL_PSAC13.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC14 GTM_gtm_cls0.DPLL_PSAC14.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC15 GTM_gtm_cls0.DPLL_PSAC15.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC16 GTM_gtm_cls0.DPLL_PSAC16.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC17 GTM_gtm_cls0.DPLL_PSAC17.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC18 GTM_gtm_cls0.DPLL_PSAC18.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC19 GTM_gtm_cls0.DPLL_PSAC19.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC20 GTM_gtm_cls0.DPLL_PSAC20.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC21 GTM_gtm_cls0.DPLL_PSAC21.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC22 GTM_gtm_cls0.DPLL_PSAC22.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC23 GTM_gtm_cls0.DPLL_PSAC23.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC24 GTM_gtm_cls0.DPLL_PSAC24.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC25 GTM_gtm_cls0.DPLL_PSAC25.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC26 GTM_gtm_cls0.DPLL_PSAC26.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC27 GTM_gtm_cls0.DPLL_PSAC27.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC28 GTM_gtm_cls0.DPLL_PSAC28.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC29 GTM_gtm_cls0.DPLL_PSAC29.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC30 GTM_gtm_cls0.DPLL_PSAC30.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_PSAC31 GTM_gtm_cls0.DPLL_PSAC31.R /* Calculated Position Value to start Action [n] */
#define GTM_gtm_cls0_DPLL_ACB_0 GTM_gtm_cls0.DPLL_ACB_0.R  /* Control Bits for NOAC Actions */
#define GTM_gtm_cls0_DPLL_ACB_1 GTM_gtm_cls0.DPLL_ACB_1.R  /* Control Bits for NOAC Actions */
#define GTM_gtm_cls0_DPLL_ACB_2 GTM_gtm_cls0.DPLL_ACB_2.R  /* Control Bits for NOAC Actions */
#define GTM_gtm_cls0_DPLL_ACB_3 GTM_gtm_cls0.DPLL_ACB_3.R  /* Control Bits for NOAC Actions */
#define GTM_gtm_cls0_DPLL_ACB_4 GTM_gtm_cls0.DPLL_ACB_4.R  /* Control Bits for NOAC Actions */
#define GTM_gtm_cls0_DPLL_ACB_5 GTM_gtm_cls0.DPLL_ACB_5.R  /* Control Bits for NOAC Actions */
#define GTM_gtm_cls0_DPLL_ACB_6 GTM_gtm_cls0.DPLL_ACB_6.R  /* Control Bits for NOAC Actions */
#define GTM_gtm_cls0_DPLL_ACB_7 GTM_gtm_cls0.DPLL_ACB_7.R  /* Control Bits for NOAC Actions */
#define GTM_gtm_cls0_DPLL_CTRL_11 GTM_gtm_cls0.DPLL_CTRL_11.R /* Control Register 11 */
#define GTM_gtm_cls0_DPLL_THVAL2 GTM_gtm_cls0.DPLL_THVAL2.R /* Measured TRIGGER Hold Time Value 2 */
#define GTM_gtm_cls0_DPLL_TIDEL GTM_gtm_cls0.DPLL_TIDEL.R  /* TRIGGER input delay */
#define GTM_gtm_cls0_DPLL_SIDEL GTM_gtm_cls0.DPLL_SIDEL.R  /* STATE input delay */
#define GTM_gtm_cls0_DPLL_APS_SYNC_EXT GTM_gtm_cls0.DPLL_APS_SYNC_EXT.R /* STATE Time Stamp Field Offset at Synchronization Time */
#define GTM_gtm_cls0_DPLL_CTRL_EXT GTM_gtm_cls0.DPLL_CTRL_EXT.R /* STATE Time Stamp Field Offset at Synchronization Time */
#define GTM_gtm_cls0_DPLL_APS_EXT GTM_gtm_cls0.DPLL_APS_EXT.R /* Actual RAM Pointer Address for STATE */
#define GTM_gtm_cls0_DPLL_APS_1C3_EXT GTM_gtm_cls0.DPLL_APS_1C3_EXT.R /* Actual RAM Pointer Address for RAM region 1c3 (DPLL_CTRL_11.STATE_EXT=1) */
#define GTM_gtm_cls0_DPLL_STA GTM_gtm_cls0.DPLL_STA.R      /* Status of the state machine states */
#define GTM_gtm_cls0_DPLL_INCF1_OFFSET GTM_gtm_cls0.DPLL_INCF1_OFFSET.R /* Start value of ADD_IN_ADDER1 */
#define GTM_gtm_cls0_DPLL_INCF2_OFFSET GTM_gtm_cls0.DPLL_INCF2_OFFSET.R /* Start value of the ADD_IN_ADDER2 */
#define GTM_gtm_cls0_DPLL_DT_T_START GTM_gtm_cls0.DPLL_DT_T_START.R /* Start value of DT_T_ACT */
#define GTM_gtm_cls0_DPLL_DT_S_START GTM_gtm_cls0.DPLL_DT_S_START.R /* Start value of DT_S_ACT */
#define GTM_gtm_cls0_DPLL_STA_MASK GTM_gtm_cls0.DPLL_STA_MASK.R /* Notify values for DPLL_STA */
#define GTM_gtm_cls0_DPLL_STA_FLAG GTM_gtm_cls0.DPLL_STA_FLAG.R /* DPLL STA Flags */
#define GTM_gtm_cls0_DPLL_INC_CNT1_MASK GTM_gtm_cls0.DPLL_INC_CNT1_MASK.R /* Notify value of DPLL_INC_CNT1 */
#define GTM_gtm_cls0_DPLL_INC_CNT2_MASK GTM_gtm_cls0.DPLL_INC_CNT2_MASK.R /* Notify value of DPLL_INC_CNT2 */
#define GTM_gtm_cls0_DPLL_NUSC_EXT1 GTM_gtm_cls0.DPLL_NUSC_EXT1.R /* Number of Recent STATE Events used for Calculations */
#define GTM_gtm_cls0_DPLL_NUSC_EXT2 GTM_gtm_cls0.DPLL_NUSC_EXT2.R /* Number of Recent STATE Events used for Calculations */
#define GTM_gtm_cls0_DPLL_CTN_MIN GTM_gtm_cls0.DPLL_CTN_MIN.R /* Minimum value of predicted nominal increment of TRIGGER */
#define GTM_gtm_cls0_DPLL_CTN_MAX GTM_gtm_cls0.DPLL_CTN_MAX.R /* Maximum value of predicted nominal increment of TRIGGER */
#define GTM_gtm_cls0_DPLL_CSN_MIN GTM_gtm_cls0.DPLL_CSN_MIN.R /* Minimum value of predicted nominal increment of STATE */
#define GTM_gtm_cls0_DPLL_CSN_MAX GTM_gtm_cls0.DPLL_CSN_MAX.R /* Maximum value of predicted nominal increment of STATE */
#define GTM_gtm_cls0_DPLL_SW_TRIG GTM_gtm_cls0.DPLL_SW_TRIG.R /* Software triggered input events */
#define GTM_gtm_cls0_DPLL_MP_T GTM_gtm_cls0.DPLL_MP_T.R    /* Missing pulses of TRIGGER */
#define GTM_gtm_cls0_DPLL_MP_S GTM_gtm_cls0.DPLL_MP_S.R    /* Missing pulses of STATE */
#define GTM_gtm_cls0_DPLL_CTRL_12 GTM_gtm_cls0.DPLL_CTRL_12.R /* DPLL control register 12 */
#define GTM_gtm_cls0_DPLL_RR20 GTM_gtm_cls0.DPLL_RR2[0].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21 GTM_gtm_cls0.DPLL_RR2[1].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22 GTM_gtm_cls0.DPLL_RR2[2].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23 GTM_gtm_cls0.DPLL_RR2[3].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24 GTM_gtm_cls0.DPLL_RR2[4].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR25 GTM_gtm_cls0.DPLL_RR2[5].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR26 GTM_gtm_cls0.DPLL_RR2[6].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR27 GTM_gtm_cls0.DPLL_RR2[7].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR28 GTM_gtm_cls0.DPLL_RR2[8].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR29 GTM_gtm_cls0.DPLL_RR2[9].R  /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR210 GTM_gtm_cls0.DPLL_RR2[10].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR211 GTM_gtm_cls0.DPLL_RR2[11].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR212 GTM_gtm_cls0.DPLL_RR2[12].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR213 GTM_gtm_cls0.DPLL_RR2[13].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR214 GTM_gtm_cls0.DPLL_RR2[14].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR215 GTM_gtm_cls0.DPLL_RR2[15].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR216 GTM_gtm_cls0.DPLL_RR2[16].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR217 GTM_gtm_cls0.DPLL_RR2[17].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR218 GTM_gtm_cls0.DPLL_RR2[18].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR219 GTM_gtm_cls0.DPLL_RR2[19].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR220 GTM_gtm_cls0.DPLL_RR2[20].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR221 GTM_gtm_cls0.DPLL_RR2[21].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR222 GTM_gtm_cls0.DPLL_RR2[22].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR223 GTM_gtm_cls0.DPLL_RR2[23].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR224 GTM_gtm_cls0.DPLL_RR2[24].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR225 GTM_gtm_cls0.DPLL_RR2[25].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR226 GTM_gtm_cls0.DPLL_RR2[26].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR227 GTM_gtm_cls0.DPLL_RR2[27].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR228 GTM_gtm_cls0.DPLL_RR2[28].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR229 GTM_gtm_cls0.DPLL_RR2[29].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR230 GTM_gtm_cls0.DPLL_RR2[30].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR231 GTM_gtm_cls0.DPLL_RR2[31].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR232 GTM_gtm_cls0.DPLL_RR2[32].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR233 GTM_gtm_cls0.DPLL_RR2[33].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR234 GTM_gtm_cls0.DPLL_RR2[34].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR235 GTM_gtm_cls0.DPLL_RR2[35].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR236 GTM_gtm_cls0.DPLL_RR2[36].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR237 GTM_gtm_cls0.DPLL_RR2[37].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR238 GTM_gtm_cls0.DPLL_RR2[38].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR239 GTM_gtm_cls0.DPLL_RR2[39].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR240 GTM_gtm_cls0.DPLL_RR2[40].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR241 GTM_gtm_cls0.DPLL_RR2[41].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR242 GTM_gtm_cls0.DPLL_RR2[42].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR243 GTM_gtm_cls0.DPLL_RR2[43].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR244 GTM_gtm_cls0.DPLL_RR2[44].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR245 GTM_gtm_cls0.DPLL_RR2[45].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR246 GTM_gtm_cls0.DPLL_RR2[46].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR247 GTM_gtm_cls0.DPLL_RR2[47].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR248 GTM_gtm_cls0.DPLL_RR2[48].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR249 GTM_gtm_cls0.DPLL_RR2[49].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR250 GTM_gtm_cls0.DPLL_RR2[50].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR251 GTM_gtm_cls0.DPLL_RR2[51].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR252 GTM_gtm_cls0.DPLL_RR2[52].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR253 GTM_gtm_cls0.DPLL_RR2[53].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR254 GTM_gtm_cls0.DPLL_RR2[54].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR255 GTM_gtm_cls0.DPLL_RR2[55].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR256 GTM_gtm_cls0.DPLL_RR2[56].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR257 GTM_gtm_cls0.DPLL_RR2[57].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR258 GTM_gtm_cls0.DPLL_RR2[58].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR259 GTM_gtm_cls0.DPLL_RR2[59].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR260 GTM_gtm_cls0.DPLL_RR2[60].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR261 GTM_gtm_cls0.DPLL_RR2[61].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR262 GTM_gtm_cls0.DPLL_RR2[62].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR263 GTM_gtm_cls0.DPLL_RR2[63].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR264 GTM_gtm_cls0.DPLL_RR2[64].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR265 GTM_gtm_cls0.DPLL_RR2[65].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR266 GTM_gtm_cls0.DPLL_RR2[66].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR267 GTM_gtm_cls0.DPLL_RR2[67].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR268 GTM_gtm_cls0.DPLL_RR2[68].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR269 GTM_gtm_cls0.DPLL_RR2[69].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR270 GTM_gtm_cls0.DPLL_RR2[70].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR271 GTM_gtm_cls0.DPLL_RR2[71].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR272 GTM_gtm_cls0.DPLL_RR2[72].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR273 GTM_gtm_cls0.DPLL_RR2[73].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR274 GTM_gtm_cls0.DPLL_RR2[74].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR275 GTM_gtm_cls0.DPLL_RR2[75].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR276 GTM_gtm_cls0.DPLL_RR2[76].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR277 GTM_gtm_cls0.DPLL_RR2[77].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR278 GTM_gtm_cls0.DPLL_RR2[78].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR279 GTM_gtm_cls0.DPLL_RR2[79].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR280 GTM_gtm_cls0.DPLL_RR2[80].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR281 GTM_gtm_cls0.DPLL_RR2[81].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR282 GTM_gtm_cls0.DPLL_RR2[82].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR283 GTM_gtm_cls0.DPLL_RR2[83].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR284 GTM_gtm_cls0.DPLL_RR2[84].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR285 GTM_gtm_cls0.DPLL_RR2[85].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR286 GTM_gtm_cls0.DPLL_RR2[86].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR287 GTM_gtm_cls0.DPLL_RR2[87].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR288 GTM_gtm_cls0.DPLL_RR2[88].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR289 GTM_gtm_cls0.DPLL_RR2[89].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR290 GTM_gtm_cls0.DPLL_RR2[90].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR291 GTM_gtm_cls0.DPLL_RR2[91].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR292 GTM_gtm_cls0.DPLL_RR2[92].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR293 GTM_gtm_cls0.DPLL_RR2[93].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR294 GTM_gtm_cls0.DPLL_RR2[94].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR295 GTM_gtm_cls0.DPLL_RR2[95].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR296 GTM_gtm_cls0.DPLL_RR2[96].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR297 GTM_gtm_cls0.DPLL_RR2[97].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR298 GTM_gtm_cls0.DPLL_RR2[98].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR299 GTM_gtm_cls0.DPLL_RR2[99].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2100 GTM_gtm_cls0.DPLL_RR2[100].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2101 GTM_gtm_cls0.DPLL_RR2[101].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2102 GTM_gtm_cls0.DPLL_RR2[102].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2103 GTM_gtm_cls0.DPLL_RR2[103].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2104 GTM_gtm_cls0.DPLL_RR2[104].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2105 GTM_gtm_cls0.DPLL_RR2[105].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2106 GTM_gtm_cls0.DPLL_RR2[106].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2107 GTM_gtm_cls0.DPLL_RR2[107].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2108 GTM_gtm_cls0.DPLL_RR2[108].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2109 GTM_gtm_cls0.DPLL_RR2[109].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2110 GTM_gtm_cls0.DPLL_RR2[110].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2111 GTM_gtm_cls0.DPLL_RR2[111].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2112 GTM_gtm_cls0.DPLL_RR2[112].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2113 GTM_gtm_cls0.DPLL_RR2[113].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2114 GTM_gtm_cls0.DPLL_RR2[114].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2115 GTM_gtm_cls0.DPLL_RR2[115].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2116 GTM_gtm_cls0.DPLL_RR2[116].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2117 GTM_gtm_cls0.DPLL_RR2[117].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2118 GTM_gtm_cls0.DPLL_RR2[118].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2119 GTM_gtm_cls0.DPLL_RR2[119].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2120 GTM_gtm_cls0.DPLL_RR2[120].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2121 GTM_gtm_cls0.DPLL_RR2[121].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2122 GTM_gtm_cls0.DPLL_RR2[122].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2123 GTM_gtm_cls0.DPLL_RR2[123].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2124 GTM_gtm_cls0.DPLL_RR2[124].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2125 GTM_gtm_cls0.DPLL_RR2[125].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2126 GTM_gtm_cls0.DPLL_RR2[126].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2127 GTM_gtm_cls0.DPLL_RR2[127].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2128 GTM_gtm_cls0.DPLL_RR2[128].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2129 GTM_gtm_cls0.DPLL_RR2[129].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2130 GTM_gtm_cls0.DPLL_RR2[130].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2131 GTM_gtm_cls0.DPLL_RR2[131].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2132 GTM_gtm_cls0.DPLL_RR2[132].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2133 GTM_gtm_cls0.DPLL_RR2[133].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2134 GTM_gtm_cls0.DPLL_RR2[134].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2135 GTM_gtm_cls0.DPLL_RR2[135].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2136 GTM_gtm_cls0.DPLL_RR2[136].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2137 GTM_gtm_cls0.DPLL_RR2[137].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2138 GTM_gtm_cls0.DPLL_RR2[138].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2139 GTM_gtm_cls0.DPLL_RR2[139].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2140 GTM_gtm_cls0.DPLL_RR2[140].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2141 GTM_gtm_cls0.DPLL_RR2[141].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2142 GTM_gtm_cls0.DPLL_RR2[142].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2143 GTM_gtm_cls0.DPLL_RR2[143].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2144 GTM_gtm_cls0.DPLL_RR2[144].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2145 GTM_gtm_cls0.DPLL_RR2[145].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2146 GTM_gtm_cls0.DPLL_RR2[146].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2147 GTM_gtm_cls0.DPLL_RR2[147].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2148 GTM_gtm_cls0.DPLL_RR2[148].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2149 GTM_gtm_cls0.DPLL_RR2[149].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2150 GTM_gtm_cls0.DPLL_RR2[150].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2151 GTM_gtm_cls0.DPLL_RR2[151].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2152 GTM_gtm_cls0.DPLL_RR2[152].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2153 GTM_gtm_cls0.DPLL_RR2[153].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2154 GTM_gtm_cls0.DPLL_RR2[154].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2155 GTM_gtm_cls0.DPLL_RR2[155].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2156 GTM_gtm_cls0.DPLL_RR2[156].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2157 GTM_gtm_cls0.DPLL_RR2[157].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2158 GTM_gtm_cls0.DPLL_RR2[158].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2159 GTM_gtm_cls0.DPLL_RR2[159].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2160 GTM_gtm_cls0.DPLL_RR2[160].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2161 GTM_gtm_cls0.DPLL_RR2[161].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2162 GTM_gtm_cls0.DPLL_RR2[162].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2163 GTM_gtm_cls0.DPLL_RR2[163].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2164 GTM_gtm_cls0.DPLL_RR2[164].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2165 GTM_gtm_cls0.DPLL_RR2[165].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2166 GTM_gtm_cls0.DPLL_RR2[166].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2167 GTM_gtm_cls0.DPLL_RR2[167].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2168 GTM_gtm_cls0.DPLL_RR2[168].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2169 GTM_gtm_cls0.DPLL_RR2[169].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2170 GTM_gtm_cls0.DPLL_RR2[170].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2171 GTM_gtm_cls0.DPLL_RR2[171].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2172 GTM_gtm_cls0.DPLL_RR2[172].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2173 GTM_gtm_cls0.DPLL_RR2[173].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2174 GTM_gtm_cls0.DPLL_RR2[174].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2175 GTM_gtm_cls0.DPLL_RR2[175].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2176 GTM_gtm_cls0.DPLL_RR2[176].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2177 GTM_gtm_cls0.DPLL_RR2[177].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2178 GTM_gtm_cls0.DPLL_RR2[178].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2179 GTM_gtm_cls0.DPLL_RR2[179].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2180 GTM_gtm_cls0.DPLL_RR2[180].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2181 GTM_gtm_cls0.DPLL_RR2[181].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2182 GTM_gtm_cls0.DPLL_RR2[182].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2183 GTM_gtm_cls0.DPLL_RR2[183].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2184 GTM_gtm_cls0.DPLL_RR2[184].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2185 GTM_gtm_cls0.DPLL_RR2[185].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2186 GTM_gtm_cls0.DPLL_RR2[186].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2187 GTM_gtm_cls0.DPLL_RR2[187].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2188 GTM_gtm_cls0.DPLL_RR2[188].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2189 GTM_gtm_cls0.DPLL_RR2[189].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2190 GTM_gtm_cls0.DPLL_RR2[190].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2191 GTM_gtm_cls0.DPLL_RR2[191].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2192 GTM_gtm_cls0.DPLL_RR2[192].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2193 GTM_gtm_cls0.DPLL_RR2[193].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2194 GTM_gtm_cls0.DPLL_RR2[194].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2195 GTM_gtm_cls0.DPLL_RR2[195].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2196 GTM_gtm_cls0.DPLL_RR2[196].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2197 GTM_gtm_cls0.DPLL_RR2[197].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2198 GTM_gtm_cls0.DPLL_RR2[198].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2199 GTM_gtm_cls0.DPLL_RR2[199].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2200 GTM_gtm_cls0.DPLL_RR2[200].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2201 GTM_gtm_cls0.DPLL_RR2[201].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2202 GTM_gtm_cls0.DPLL_RR2[202].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2203 GTM_gtm_cls0.DPLL_RR2[203].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2204 GTM_gtm_cls0.DPLL_RR2[204].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2205 GTM_gtm_cls0.DPLL_RR2[205].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2206 GTM_gtm_cls0.DPLL_RR2[206].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2207 GTM_gtm_cls0.DPLL_RR2[207].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2208 GTM_gtm_cls0.DPLL_RR2[208].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2209 GTM_gtm_cls0.DPLL_RR2[209].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2210 GTM_gtm_cls0.DPLL_RR2[210].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2211 GTM_gtm_cls0.DPLL_RR2[211].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2212 GTM_gtm_cls0.DPLL_RR2[212].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2213 GTM_gtm_cls0.DPLL_RR2[213].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2214 GTM_gtm_cls0.DPLL_RR2[214].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2215 GTM_gtm_cls0.DPLL_RR2[215].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2216 GTM_gtm_cls0.DPLL_RR2[216].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2217 GTM_gtm_cls0.DPLL_RR2[217].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2218 GTM_gtm_cls0.DPLL_RR2[218].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2219 GTM_gtm_cls0.DPLL_RR2[219].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2220 GTM_gtm_cls0.DPLL_RR2[220].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2221 GTM_gtm_cls0.DPLL_RR2[221].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2222 GTM_gtm_cls0.DPLL_RR2[222].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2223 GTM_gtm_cls0.DPLL_RR2[223].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2224 GTM_gtm_cls0.DPLL_RR2[224].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2225 GTM_gtm_cls0.DPLL_RR2[225].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2226 GTM_gtm_cls0.DPLL_RR2[226].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2227 GTM_gtm_cls0.DPLL_RR2[227].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2228 GTM_gtm_cls0.DPLL_RR2[228].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2229 GTM_gtm_cls0.DPLL_RR2[229].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2230 GTM_gtm_cls0.DPLL_RR2[230].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2231 GTM_gtm_cls0.DPLL_RR2[231].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2232 GTM_gtm_cls0.DPLL_RR2[232].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2233 GTM_gtm_cls0.DPLL_RR2[233].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2234 GTM_gtm_cls0.DPLL_RR2[234].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2235 GTM_gtm_cls0.DPLL_RR2[235].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2236 GTM_gtm_cls0.DPLL_RR2[236].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2237 GTM_gtm_cls0.DPLL_RR2[237].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2238 GTM_gtm_cls0.DPLL_RR2[238].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2239 GTM_gtm_cls0.DPLL_RR2[239].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2240 GTM_gtm_cls0.DPLL_RR2[240].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2241 GTM_gtm_cls0.DPLL_RR2[241].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2242 GTM_gtm_cls0.DPLL_RR2[242].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2243 GTM_gtm_cls0.DPLL_RR2[243].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2244 GTM_gtm_cls0.DPLL_RR2[244].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2245 GTM_gtm_cls0.DPLL_RR2[245].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2246 GTM_gtm_cls0.DPLL_RR2[246].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2247 GTM_gtm_cls0.DPLL_RR2[247].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2248 GTM_gtm_cls0.DPLL_RR2[248].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2249 GTM_gtm_cls0.DPLL_RR2[249].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2250 GTM_gtm_cls0.DPLL_RR2[250].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2251 GTM_gtm_cls0.DPLL_RR2[251].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2252 GTM_gtm_cls0.DPLL_RR2[252].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2253 GTM_gtm_cls0.DPLL_RR2[253].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2254 GTM_gtm_cls0.DPLL_RR2[254].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2255 GTM_gtm_cls0.DPLL_RR2[255].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2256 GTM_gtm_cls0.DPLL_RR2[256].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2257 GTM_gtm_cls0.DPLL_RR2[257].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2258 GTM_gtm_cls0.DPLL_RR2[258].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2259 GTM_gtm_cls0.DPLL_RR2[259].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2260 GTM_gtm_cls0.DPLL_RR2[260].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2261 GTM_gtm_cls0.DPLL_RR2[261].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2262 GTM_gtm_cls0.DPLL_RR2[262].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2263 GTM_gtm_cls0.DPLL_RR2[263].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2264 GTM_gtm_cls0.DPLL_RR2[264].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2265 GTM_gtm_cls0.DPLL_RR2[265].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2266 GTM_gtm_cls0.DPLL_RR2[266].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2267 GTM_gtm_cls0.DPLL_RR2[267].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2268 GTM_gtm_cls0.DPLL_RR2[268].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2269 GTM_gtm_cls0.DPLL_RR2[269].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2270 GTM_gtm_cls0.DPLL_RR2[270].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2271 GTM_gtm_cls0.DPLL_RR2[271].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2272 GTM_gtm_cls0.DPLL_RR2[272].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2273 GTM_gtm_cls0.DPLL_RR2[273].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2274 GTM_gtm_cls0.DPLL_RR2[274].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2275 GTM_gtm_cls0.DPLL_RR2[275].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2276 GTM_gtm_cls0.DPLL_RR2[276].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2277 GTM_gtm_cls0.DPLL_RR2[277].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2278 GTM_gtm_cls0.DPLL_RR2[278].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2279 GTM_gtm_cls0.DPLL_RR2[279].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2280 GTM_gtm_cls0.DPLL_RR2[280].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2281 GTM_gtm_cls0.DPLL_RR2[281].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2282 GTM_gtm_cls0.DPLL_RR2[282].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2283 GTM_gtm_cls0.DPLL_RR2[283].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2284 GTM_gtm_cls0.DPLL_RR2[284].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2285 GTM_gtm_cls0.DPLL_RR2[285].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2286 GTM_gtm_cls0.DPLL_RR2[286].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2287 GTM_gtm_cls0.DPLL_RR2[287].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2288 GTM_gtm_cls0.DPLL_RR2[288].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2289 GTM_gtm_cls0.DPLL_RR2[289].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2290 GTM_gtm_cls0.DPLL_RR2[290].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2291 GTM_gtm_cls0.DPLL_RR2[291].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2292 GTM_gtm_cls0.DPLL_RR2[292].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2293 GTM_gtm_cls0.DPLL_RR2[293].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2294 GTM_gtm_cls0.DPLL_RR2[294].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2295 GTM_gtm_cls0.DPLL_RR2[295].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2296 GTM_gtm_cls0.DPLL_RR2[296].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2297 GTM_gtm_cls0.DPLL_RR2[297].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2298 GTM_gtm_cls0.DPLL_RR2[298].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2299 GTM_gtm_cls0.DPLL_RR2[299].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2300 GTM_gtm_cls0.DPLL_RR2[300].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2301 GTM_gtm_cls0.DPLL_RR2[301].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2302 GTM_gtm_cls0.DPLL_RR2[302].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2303 GTM_gtm_cls0.DPLL_RR2[303].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2304 GTM_gtm_cls0.DPLL_RR2[304].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2305 GTM_gtm_cls0.DPLL_RR2[305].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2306 GTM_gtm_cls0.DPLL_RR2[306].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2307 GTM_gtm_cls0.DPLL_RR2[307].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2308 GTM_gtm_cls0.DPLL_RR2[308].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2309 GTM_gtm_cls0.DPLL_RR2[309].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2310 GTM_gtm_cls0.DPLL_RR2[310].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2311 GTM_gtm_cls0.DPLL_RR2[311].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2312 GTM_gtm_cls0.DPLL_RR2[312].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2313 GTM_gtm_cls0.DPLL_RR2[313].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2314 GTM_gtm_cls0.DPLL_RR2[314].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2315 GTM_gtm_cls0.DPLL_RR2[315].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2316 GTM_gtm_cls0.DPLL_RR2[316].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2317 GTM_gtm_cls0.DPLL_RR2[317].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2318 GTM_gtm_cls0.DPLL_RR2[318].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2319 GTM_gtm_cls0.DPLL_RR2[319].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2320 GTM_gtm_cls0.DPLL_RR2[320].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2321 GTM_gtm_cls0.DPLL_RR2[321].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2322 GTM_gtm_cls0.DPLL_RR2[322].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2323 GTM_gtm_cls0.DPLL_RR2[323].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2324 GTM_gtm_cls0.DPLL_RR2[324].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2325 GTM_gtm_cls0.DPLL_RR2[325].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2326 GTM_gtm_cls0.DPLL_RR2[326].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2327 GTM_gtm_cls0.DPLL_RR2[327].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2328 GTM_gtm_cls0.DPLL_RR2[328].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2329 GTM_gtm_cls0.DPLL_RR2[329].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2330 GTM_gtm_cls0.DPLL_RR2[330].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2331 GTM_gtm_cls0.DPLL_RR2[331].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2332 GTM_gtm_cls0.DPLL_RR2[332].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2333 GTM_gtm_cls0.DPLL_RR2[333].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2334 GTM_gtm_cls0.DPLL_RR2[334].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2335 GTM_gtm_cls0.DPLL_RR2[335].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2336 GTM_gtm_cls0.DPLL_RR2[336].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2337 GTM_gtm_cls0.DPLL_RR2[337].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2338 GTM_gtm_cls0.DPLL_RR2[338].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2339 GTM_gtm_cls0.DPLL_RR2[339].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2340 GTM_gtm_cls0.DPLL_RR2[340].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2341 GTM_gtm_cls0.DPLL_RR2[341].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2342 GTM_gtm_cls0.DPLL_RR2[342].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2343 GTM_gtm_cls0.DPLL_RR2[343].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2344 GTM_gtm_cls0.DPLL_RR2[344].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2345 GTM_gtm_cls0.DPLL_RR2[345].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2346 GTM_gtm_cls0.DPLL_RR2[346].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2347 GTM_gtm_cls0.DPLL_RR2[347].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2348 GTM_gtm_cls0.DPLL_RR2[348].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2349 GTM_gtm_cls0.DPLL_RR2[349].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2350 GTM_gtm_cls0.DPLL_RR2[350].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2351 GTM_gtm_cls0.DPLL_RR2[351].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2352 GTM_gtm_cls0.DPLL_RR2[352].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2353 GTM_gtm_cls0.DPLL_RR2[353].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2354 GTM_gtm_cls0.DPLL_RR2[354].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2355 GTM_gtm_cls0.DPLL_RR2[355].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2356 GTM_gtm_cls0.DPLL_RR2[356].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2357 GTM_gtm_cls0.DPLL_RR2[357].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2358 GTM_gtm_cls0.DPLL_RR2[358].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2359 GTM_gtm_cls0.DPLL_RR2[359].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2360 GTM_gtm_cls0.DPLL_RR2[360].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2361 GTM_gtm_cls0.DPLL_RR2[361].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2362 GTM_gtm_cls0.DPLL_RR2[362].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2363 GTM_gtm_cls0.DPLL_RR2[363].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2364 GTM_gtm_cls0.DPLL_RR2[364].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2365 GTM_gtm_cls0.DPLL_RR2[365].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2366 GTM_gtm_cls0.DPLL_RR2[366].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2367 GTM_gtm_cls0.DPLL_RR2[367].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2368 GTM_gtm_cls0.DPLL_RR2[368].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2369 GTM_gtm_cls0.DPLL_RR2[369].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2370 GTM_gtm_cls0.DPLL_RR2[370].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2371 GTM_gtm_cls0.DPLL_RR2[371].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2372 GTM_gtm_cls0.DPLL_RR2[372].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2373 GTM_gtm_cls0.DPLL_RR2[373].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2374 GTM_gtm_cls0.DPLL_RR2[374].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2375 GTM_gtm_cls0.DPLL_RR2[375].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2376 GTM_gtm_cls0.DPLL_RR2[376].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2377 GTM_gtm_cls0.DPLL_RR2[377].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2378 GTM_gtm_cls0.DPLL_RR2[378].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2379 GTM_gtm_cls0.DPLL_RR2[379].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2380 GTM_gtm_cls0.DPLL_RR2[380].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2381 GTM_gtm_cls0.DPLL_RR2[381].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2382 GTM_gtm_cls0.DPLL_RR2[382].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2383 GTM_gtm_cls0.DPLL_RR2[383].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2384 GTM_gtm_cls0.DPLL_RR2[384].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2385 GTM_gtm_cls0.DPLL_RR2[385].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2386 GTM_gtm_cls0.DPLL_RR2[386].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2387 GTM_gtm_cls0.DPLL_RR2[387].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2388 GTM_gtm_cls0.DPLL_RR2[388].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2389 GTM_gtm_cls0.DPLL_RR2[389].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2390 GTM_gtm_cls0.DPLL_RR2[390].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2391 GTM_gtm_cls0.DPLL_RR2[391].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2392 GTM_gtm_cls0.DPLL_RR2[392].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2393 GTM_gtm_cls0.DPLL_RR2[393].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2394 GTM_gtm_cls0.DPLL_RR2[394].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2395 GTM_gtm_cls0.DPLL_RR2[395].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2396 GTM_gtm_cls0.DPLL_RR2[396].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2397 GTM_gtm_cls0.DPLL_RR2[397].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2398 GTM_gtm_cls0.DPLL_RR2[398].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2399 GTM_gtm_cls0.DPLL_RR2[399].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2400 GTM_gtm_cls0.DPLL_RR2[400].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2401 GTM_gtm_cls0.DPLL_RR2[401].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2402 GTM_gtm_cls0.DPLL_RR2[402].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2403 GTM_gtm_cls0.DPLL_RR2[403].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2404 GTM_gtm_cls0.DPLL_RR2[404].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2405 GTM_gtm_cls0.DPLL_RR2[405].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2406 GTM_gtm_cls0.DPLL_RR2[406].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2407 GTM_gtm_cls0.DPLL_RR2[407].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2408 GTM_gtm_cls0.DPLL_RR2[408].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2409 GTM_gtm_cls0.DPLL_RR2[409].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2410 GTM_gtm_cls0.DPLL_RR2[410].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2411 GTM_gtm_cls0.DPLL_RR2[411].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2412 GTM_gtm_cls0.DPLL_RR2[412].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2413 GTM_gtm_cls0.DPLL_RR2[413].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2414 GTM_gtm_cls0.DPLL_RR2[414].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2415 GTM_gtm_cls0.DPLL_RR2[415].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2416 GTM_gtm_cls0.DPLL_RR2[416].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2417 GTM_gtm_cls0.DPLL_RR2[417].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2418 GTM_gtm_cls0.DPLL_RR2[418].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2419 GTM_gtm_cls0.DPLL_RR2[419].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2420 GTM_gtm_cls0.DPLL_RR2[420].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2421 GTM_gtm_cls0.DPLL_RR2[421].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2422 GTM_gtm_cls0.DPLL_RR2[422].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2423 GTM_gtm_cls0.DPLL_RR2[423].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2424 GTM_gtm_cls0.DPLL_RR2[424].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2425 GTM_gtm_cls0.DPLL_RR2[425].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2426 GTM_gtm_cls0.DPLL_RR2[426].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2427 GTM_gtm_cls0.DPLL_RR2[427].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2428 GTM_gtm_cls0.DPLL_RR2[428].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2429 GTM_gtm_cls0.DPLL_RR2[429].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2430 GTM_gtm_cls0.DPLL_RR2[430].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2431 GTM_gtm_cls0.DPLL_RR2[431].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2432 GTM_gtm_cls0.DPLL_RR2[432].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2433 GTM_gtm_cls0.DPLL_RR2[433].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2434 GTM_gtm_cls0.DPLL_RR2[434].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2435 GTM_gtm_cls0.DPLL_RR2[435].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2436 GTM_gtm_cls0.DPLL_RR2[436].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2437 GTM_gtm_cls0.DPLL_RR2[437].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2438 GTM_gtm_cls0.DPLL_RR2[438].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2439 GTM_gtm_cls0.DPLL_RR2[439].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2440 GTM_gtm_cls0.DPLL_RR2[440].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2441 GTM_gtm_cls0.DPLL_RR2[441].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2442 GTM_gtm_cls0.DPLL_RR2[442].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2443 GTM_gtm_cls0.DPLL_RR2[443].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2444 GTM_gtm_cls0.DPLL_RR2[444].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2445 GTM_gtm_cls0.DPLL_RR2[445].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2446 GTM_gtm_cls0.DPLL_RR2[446].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2447 GTM_gtm_cls0.DPLL_RR2[447].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2448 GTM_gtm_cls0.DPLL_RR2[448].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2449 GTM_gtm_cls0.DPLL_RR2[449].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2450 GTM_gtm_cls0.DPLL_RR2[450].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2451 GTM_gtm_cls0.DPLL_RR2[451].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2452 GTM_gtm_cls0.DPLL_RR2[452].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2453 GTM_gtm_cls0.DPLL_RR2[453].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2454 GTM_gtm_cls0.DPLL_RR2[454].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2455 GTM_gtm_cls0.DPLL_RR2[455].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2456 GTM_gtm_cls0.DPLL_RR2[456].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2457 GTM_gtm_cls0.DPLL_RR2[457].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2458 GTM_gtm_cls0.DPLL_RR2[458].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2459 GTM_gtm_cls0.DPLL_RR2[459].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2460 GTM_gtm_cls0.DPLL_RR2[460].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2461 GTM_gtm_cls0.DPLL_RR2[461].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2462 GTM_gtm_cls0.DPLL_RR2[462].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2463 GTM_gtm_cls0.DPLL_RR2[463].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2464 GTM_gtm_cls0.DPLL_RR2[464].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2465 GTM_gtm_cls0.DPLL_RR2[465].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2466 GTM_gtm_cls0.DPLL_RR2[466].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2467 GTM_gtm_cls0.DPLL_RR2[467].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2468 GTM_gtm_cls0.DPLL_RR2[468].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2469 GTM_gtm_cls0.DPLL_RR2[469].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2470 GTM_gtm_cls0.DPLL_RR2[470].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2471 GTM_gtm_cls0.DPLL_RR2[471].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2472 GTM_gtm_cls0.DPLL_RR2[472].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2473 GTM_gtm_cls0.DPLL_RR2[473].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2474 GTM_gtm_cls0.DPLL_RR2[474].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2475 GTM_gtm_cls0.DPLL_RR2[475].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2476 GTM_gtm_cls0.DPLL_RR2[476].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2477 GTM_gtm_cls0.DPLL_RR2[477].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2478 GTM_gtm_cls0.DPLL_RR2[478].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2479 GTM_gtm_cls0.DPLL_RR2[479].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2480 GTM_gtm_cls0.DPLL_RR2[480].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2481 GTM_gtm_cls0.DPLL_RR2[481].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2482 GTM_gtm_cls0.DPLL_RR2[482].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2483 GTM_gtm_cls0.DPLL_RR2[483].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2484 GTM_gtm_cls0.DPLL_RR2[484].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2485 GTM_gtm_cls0.DPLL_RR2[485].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2486 GTM_gtm_cls0.DPLL_RR2[486].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2487 GTM_gtm_cls0.DPLL_RR2[487].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2488 GTM_gtm_cls0.DPLL_RR2[488].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2489 GTM_gtm_cls0.DPLL_RR2[489].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2490 GTM_gtm_cls0.DPLL_RR2[490].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2491 GTM_gtm_cls0.DPLL_RR2[491].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2492 GTM_gtm_cls0.DPLL_RR2[492].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2493 GTM_gtm_cls0.DPLL_RR2[493].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2494 GTM_gtm_cls0.DPLL_RR2[494].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2495 GTM_gtm_cls0.DPLL_RR2[495].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2496 GTM_gtm_cls0.DPLL_RR2[496].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2497 GTM_gtm_cls0.DPLL_RR2[497].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2498 GTM_gtm_cls0.DPLL_RR2[498].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2499 GTM_gtm_cls0.DPLL_RR2[499].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2500 GTM_gtm_cls0.DPLL_RR2[500].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2501 GTM_gtm_cls0.DPLL_RR2[501].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2502 GTM_gtm_cls0.DPLL_RR2[502].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2503 GTM_gtm_cls0.DPLL_RR2[503].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2504 GTM_gtm_cls0.DPLL_RR2[504].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2505 GTM_gtm_cls0.DPLL_RR2[505].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2506 GTM_gtm_cls0.DPLL_RR2[506].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2507 GTM_gtm_cls0.DPLL_RR2[507].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2508 GTM_gtm_cls0.DPLL_RR2[508].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2509 GTM_gtm_cls0.DPLL_RR2[509].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2510 GTM_gtm_cls0.DPLL_RR2[510].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2511 GTM_gtm_cls0.DPLL_RR2[511].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2512 GTM_gtm_cls0.DPLL_RR2[512].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2513 GTM_gtm_cls0.DPLL_RR2[513].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2514 GTM_gtm_cls0.DPLL_RR2[514].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2515 GTM_gtm_cls0.DPLL_RR2[515].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2516 GTM_gtm_cls0.DPLL_RR2[516].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2517 GTM_gtm_cls0.DPLL_RR2[517].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2518 GTM_gtm_cls0.DPLL_RR2[518].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2519 GTM_gtm_cls0.DPLL_RR2[519].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2520 GTM_gtm_cls0.DPLL_RR2[520].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2521 GTM_gtm_cls0.DPLL_RR2[521].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2522 GTM_gtm_cls0.DPLL_RR2[522].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2523 GTM_gtm_cls0.DPLL_RR2[523].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2524 GTM_gtm_cls0.DPLL_RR2[524].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2525 GTM_gtm_cls0.DPLL_RR2[525].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2526 GTM_gtm_cls0.DPLL_RR2[526].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2527 GTM_gtm_cls0.DPLL_RR2[527].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2528 GTM_gtm_cls0.DPLL_RR2[528].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2529 GTM_gtm_cls0.DPLL_RR2[529].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2530 GTM_gtm_cls0.DPLL_RR2[530].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2531 GTM_gtm_cls0.DPLL_RR2[531].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2532 GTM_gtm_cls0.DPLL_RR2[532].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2533 GTM_gtm_cls0.DPLL_RR2[533].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2534 GTM_gtm_cls0.DPLL_RR2[534].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2535 GTM_gtm_cls0.DPLL_RR2[535].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2536 GTM_gtm_cls0.DPLL_RR2[536].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2537 GTM_gtm_cls0.DPLL_RR2[537].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2538 GTM_gtm_cls0.DPLL_RR2[538].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2539 GTM_gtm_cls0.DPLL_RR2[539].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2540 GTM_gtm_cls0.DPLL_RR2[540].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2541 GTM_gtm_cls0.DPLL_RR2[541].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2542 GTM_gtm_cls0.DPLL_RR2[542].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2543 GTM_gtm_cls0.DPLL_RR2[543].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2544 GTM_gtm_cls0.DPLL_RR2[544].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2545 GTM_gtm_cls0.DPLL_RR2[545].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2546 GTM_gtm_cls0.DPLL_RR2[546].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2547 GTM_gtm_cls0.DPLL_RR2[547].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2548 GTM_gtm_cls0.DPLL_RR2[548].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2549 GTM_gtm_cls0.DPLL_RR2[549].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2550 GTM_gtm_cls0.DPLL_RR2[550].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2551 GTM_gtm_cls0.DPLL_RR2[551].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2552 GTM_gtm_cls0.DPLL_RR2[552].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2553 GTM_gtm_cls0.DPLL_RR2[553].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2554 GTM_gtm_cls0.DPLL_RR2[554].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2555 GTM_gtm_cls0.DPLL_RR2[555].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2556 GTM_gtm_cls0.DPLL_RR2[556].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2557 GTM_gtm_cls0.DPLL_RR2[557].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2558 GTM_gtm_cls0.DPLL_RR2[558].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2559 GTM_gtm_cls0.DPLL_RR2[559].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2560 GTM_gtm_cls0.DPLL_RR2[560].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2561 GTM_gtm_cls0.DPLL_RR2[561].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2562 GTM_gtm_cls0.DPLL_RR2[562].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2563 GTM_gtm_cls0.DPLL_RR2[563].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2564 GTM_gtm_cls0.DPLL_RR2[564].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2565 GTM_gtm_cls0.DPLL_RR2[565].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2566 GTM_gtm_cls0.DPLL_RR2[566].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2567 GTM_gtm_cls0.DPLL_RR2[567].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2568 GTM_gtm_cls0.DPLL_RR2[568].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2569 GTM_gtm_cls0.DPLL_RR2[569].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2570 GTM_gtm_cls0.DPLL_RR2[570].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2571 GTM_gtm_cls0.DPLL_RR2[571].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2572 GTM_gtm_cls0.DPLL_RR2[572].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2573 GTM_gtm_cls0.DPLL_RR2[573].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2574 GTM_gtm_cls0.DPLL_RR2[574].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2575 GTM_gtm_cls0.DPLL_RR2[575].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2576 GTM_gtm_cls0.DPLL_RR2[576].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2577 GTM_gtm_cls0.DPLL_RR2[577].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2578 GTM_gtm_cls0.DPLL_RR2[578].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2579 GTM_gtm_cls0.DPLL_RR2[579].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2580 GTM_gtm_cls0.DPLL_RR2[580].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2581 GTM_gtm_cls0.DPLL_RR2[581].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2582 GTM_gtm_cls0.DPLL_RR2[582].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2583 GTM_gtm_cls0.DPLL_RR2[583].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2584 GTM_gtm_cls0.DPLL_RR2[584].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2585 GTM_gtm_cls0.DPLL_RR2[585].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2586 GTM_gtm_cls0.DPLL_RR2[586].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2587 GTM_gtm_cls0.DPLL_RR2[587].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2588 GTM_gtm_cls0.DPLL_RR2[588].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2589 GTM_gtm_cls0.DPLL_RR2[589].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2590 GTM_gtm_cls0.DPLL_RR2[590].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2591 GTM_gtm_cls0.DPLL_RR2[591].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2592 GTM_gtm_cls0.DPLL_RR2[592].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2593 GTM_gtm_cls0.DPLL_RR2[593].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2594 GTM_gtm_cls0.DPLL_RR2[594].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2595 GTM_gtm_cls0.DPLL_RR2[595].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2596 GTM_gtm_cls0.DPLL_RR2[596].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2597 GTM_gtm_cls0.DPLL_RR2[597].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2598 GTM_gtm_cls0.DPLL_RR2[598].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2599 GTM_gtm_cls0.DPLL_RR2[599].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2600 GTM_gtm_cls0.DPLL_RR2[600].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2601 GTM_gtm_cls0.DPLL_RR2[601].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2602 GTM_gtm_cls0.DPLL_RR2[602].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2603 GTM_gtm_cls0.DPLL_RR2[603].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2604 GTM_gtm_cls0.DPLL_RR2[604].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2605 GTM_gtm_cls0.DPLL_RR2[605].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2606 GTM_gtm_cls0.DPLL_RR2[606].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2607 GTM_gtm_cls0.DPLL_RR2[607].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2608 GTM_gtm_cls0.DPLL_RR2[608].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2609 GTM_gtm_cls0.DPLL_RR2[609].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2610 GTM_gtm_cls0.DPLL_RR2[610].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2611 GTM_gtm_cls0.DPLL_RR2[611].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2612 GTM_gtm_cls0.DPLL_RR2[612].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2613 GTM_gtm_cls0.DPLL_RR2[613].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2614 GTM_gtm_cls0.DPLL_RR2[614].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2615 GTM_gtm_cls0.DPLL_RR2[615].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2616 GTM_gtm_cls0.DPLL_RR2[616].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2617 GTM_gtm_cls0.DPLL_RR2[617].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2618 GTM_gtm_cls0.DPLL_RR2[618].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2619 GTM_gtm_cls0.DPLL_RR2[619].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2620 GTM_gtm_cls0.DPLL_RR2[620].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2621 GTM_gtm_cls0.DPLL_RR2[621].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2622 GTM_gtm_cls0.DPLL_RR2[622].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2623 GTM_gtm_cls0.DPLL_RR2[623].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2624 GTM_gtm_cls0.DPLL_RR2[624].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2625 GTM_gtm_cls0.DPLL_RR2[625].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2626 GTM_gtm_cls0.DPLL_RR2[626].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2627 GTM_gtm_cls0.DPLL_RR2[627].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2628 GTM_gtm_cls0.DPLL_RR2[628].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2629 GTM_gtm_cls0.DPLL_RR2[629].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2630 GTM_gtm_cls0.DPLL_RR2[630].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2631 GTM_gtm_cls0.DPLL_RR2[631].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2632 GTM_gtm_cls0.DPLL_RR2[632].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2633 GTM_gtm_cls0.DPLL_RR2[633].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2634 GTM_gtm_cls0.DPLL_RR2[634].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2635 GTM_gtm_cls0.DPLL_RR2[635].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2636 GTM_gtm_cls0.DPLL_RR2[636].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2637 GTM_gtm_cls0.DPLL_RR2[637].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2638 GTM_gtm_cls0.DPLL_RR2[638].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2639 GTM_gtm_cls0.DPLL_RR2[639].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2640 GTM_gtm_cls0.DPLL_RR2[640].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2641 GTM_gtm_cls0.DPLL_RR2[641].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2642 GTM_gtm_cls0.DPLL_RR2[642].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2643 GTM_gtm_cls0.DPLL_RR2[643].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2644 GTM_gtm_cls0.DPLL_RR2[644].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2645 GTM_gtm_cls0.DPLL_RR2[645].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2646 GTM_gtm_cls0.DPLL_RR2[646].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2647 GTM_gtm_cls0.DPLL_RR2[647].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2648 GTM_gtm_cls0.DPLL_RR2[648].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2649 GTM_gtm_cls0.DPLL_RR2[649].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2650 GTM_gtm_cls0.DPLL_RR2[650].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2651 GTM_gtm_cls0.DPLL_RR2[651].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2652 GTM_gtm_cls0.DPLL_RR2[652].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2653 GTM_gtm_cls0.DPLL_RR2[653].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2654 GTM_gtm_cls0.DPLL_RR2[654].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2655 GTM_gtm_cls0.DPLL_RR2[655].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2656 GTM_gtm_cls0.DPLL_RR2[656].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2657 GTM_gtm_cls0.DPLL_RR2[657].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2658 GTM_gtm_cls0.DPLL_RR2[658].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2659 GTM_gtm_cls0.DPLL_RR2[659].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2660 GTM_gtm_cls0.DPLL_RR2[660].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2661 GTM_gtm_cls0.DPLL_RR2[661].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2662 GTM_gtm_cls0.DPLL_RR2[662].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2663 GTM_gtm_cls0.DPLL_RR2[663].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2664 GTM_gtm_cls0.DPLL_RR2[664].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2665 GTM_gtm_cls0.DPLL_RR2[665].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2666 GTM_gtm_cls0.DPLL_RR2[666].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2667 GTM_gtm_cls0.DPLL_RR2[667].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2668 GTM_gtm_cls0.DPLL_RR2[668].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2669 GTM_gtm_cls0.DPLL_RR2[669].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2670 GTM_gtm_cls0.DPLL_RR2[670].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2671 GTM_gtm_cls0.DPLL_RR2[671].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2672 GTM_gtm_cls0.DPLL_RR2[672].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2673 GTM_gtm_cls0.DPLL_RR2[673].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2674 GTM_gtm_cls0.DPLL_RR2[674].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2675 GTM_gtm_cls0.DPLL_RR2[675].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2676 GTM_gtm_cls0.DPLL_RR2[676].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2677 GTM_gtm_cls0.DPLL_RR2[677].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2678 GTM_gtm_cls0.DPLL_RR2[678].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2679 GTM_gtm_cls0.DPLL_RR2[679].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2680 GTM_gtm_cls0.DPLL_RR2[680].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2681 GTM_gtm_cls0.DPLL_RR2[681].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2682 GTM_gtm_cls0.DPLL_RR2[682].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2683 GTM_gtm_cls0.DPLL_RR2[683].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2684 GTM_gtm_cls0.DPLL_RR2[684].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2685 GTM_gtm_cls0.DPLL_RR2[685].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2686 GTM_gtm_cls0.DPLL_RR2[686].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2687 GTM_gtm_cls0.DPLL_RR2[687].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2688 GTM_gtm_cls0.DPLL_RR2[688].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2689 GTM_gtm_cls0.DPLL_RR2[689].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2690 GTM_gtm_cls0.DPLL_RR2[690].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2691 GTM_gtm_cls0.DPLL_RR2[691].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2692 GTM_gtm_cls0.DPLL_RR2[692].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2693 GTM_gtm_cls0.DPLL_RR2[693].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2694 GTM_gtm_cls0.DPLL_RR2[694].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2695 GTM_gtm_cls0.DPLL_RR2[695].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2696 GTM_gtm_cls0.DPLL_RR2[696].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2697 GTM_gtm_cls0.DPLL_RR2[697].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2698 GTM_gtm_cls0.DPLL_RR2[698].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2699 GTM_gtm_cls0.DPLL_RR2[699].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2700 GTM_gtm_cls0.DPLL_RR2[700].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2701 GTM_gtm_cls0.DPLL_RR2[701].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2702 GTM_gtm_cls0.DPLL_RR2[702].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2703 GTM_gtm_cls0.DPLL_RR2[703].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2704 GTM_gtm_cls0.DPLL_RR2[704].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2705 GTM_gtm_cls0.DPLL_RR2[705].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2706 GTM_gtm_cls0.DPLL_RR2[706].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2707 GTM_gtm_cls0.DPLL_RR2[707].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2708 GTM_gtm_cls0.DPLL_RR2[708].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2709 GTM_gtm_cls0.DPLL_RR2[709].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2710 GTM_gtm_cls0.DPLL_RR2[710].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2711 GTM_gtm_cls0.DPLL_RR2[711].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2712 GTM_gtm_cls0.DPLL_RR2[712].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2713 GTM_gtm_cls0.DPLL_RR2[713].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2714 GTM_gtm_cls0.DPLL_RR2[714].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2715 GTM_gtm_cls0.DPLL_RR2[715].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2716 GTM_gtm_cls0.DPLL_RR2[716].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2717 GTM_gtm_cls0.DPLL_RR2[717].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2718 GTM_gtm_cls0.DPLL_RR2[718].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2719 GTM_gtm_cls0.DPLL_RR2[719].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2720 GTM_gtm_cls0.DPLL_RR2[720].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2721 GTM_gtm_cls0.DPLL_RR2[721].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2722 GTM_gtm_cls0.DPLL_RR2[722].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2723 GTM_gtm_cls0.DPLL_RR2[723].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2724 GTM_gtm_cls0.DPLL_RR2[724].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2725 GTM_gtm_cls0.DPLL_RR2[725].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2726 GTM_gtm_cls0.DPLL_RR2[726].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2727 GTM_gtm_cls0.DPLL_RR2[727].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2728 GTM_gtm_cls0.DPLL_RR2[728].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2729 GTM_gtm_cls0.DPLL_RR2[729].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2730 GTM_gtm_cls0.DPLL_RR2[730].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2731 GTM_gtm_cls0.DPLL_RR2[731].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2732 GTM_gtm_cls0.DPLL_RR2[732].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2733 GTM_gtm_cls0.DPLL_RR2[733].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2734 GTM_gtm_cls0.DPLL_RR2[734].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2735 GTM_gtm_cls0.DPLL_RR2[735].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2736 GTM_gtm_cls0.DPLL_RR2[736].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2737 GTM_gtm_cls0.DPLL_RR2[737].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2738 GTM_gtm_cls0.DPLL_RR2[738].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2739 GTM_gtm_cls0.DPLL_RR2[739].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2740 GTM_gtm_cls0.DPLL_RR2[740].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2741 GTM_gtm_cls0.DPLL_RR2[741].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2742 GTM_gtm_cls0.DPLL_RR2[742].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2743 GTM_gtm_cls0.DPLL_RR2[743].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2744 GTM_gtm_cls0.DPLL_RR2[744].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2745 GTM_gtm_cls0.DPLL_RR2[745].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2746 GTM_gtm_cls0.DPLL_RR2[746].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2747 GTM_gtm_cls0.DPLL_RR2[747].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2748 GTM_gtm_cls0.DPLL_RR2[748].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2749 GTM_gtm_cls0.DPLL_RR2[749].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2750 GTM_gtm_cls0.DPLL_RR2[750].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2751 GTM_gtm_cls0.DPLL_RR2[751].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2752 GTM_gtm_cls0.DPLL_RR2[752].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2753 GTM_gtm_cls0.DPLL_RR2[753].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2754 GTM_gtm_cls0.DPLL_RR2[754].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2755 GTM_gtm_cls0.DPLL_RR2[755].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2756 GTM_gtm_cls0.DPLL_RR2[756].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2757 GTM_gtm_cls0.DPLL_RR2[757].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2758 GTM_gtm_cls0.DPLL_RR2[758].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2759 GTM_gtm_cls0.DPLL_RR2[759].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2760 GTM_gtm_cls0.DPLL_RR2[760].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2761 GTM_gtm_cls0.DPLL_RR2[761].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2762 GTM_gtm_cls0.DPLL_RR2[762].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2763 GTM_gtm_cls0.DPLL_RR2[763].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2764 GTM_gtm_cls0.DPLL_RR2[764].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2765 GTM_gtm_cls0.DPLL_RR2[765].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2766 GTM_gtm_cls0.DPLL_RR2[766].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2767 GTM_gtm_cls0.DPLL_RR2[767].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2768 GTM_gtm_cls0.DPLL_RR2[768].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2769 GTM_gtm_cls0.DPLL_RR2[769].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2770 GTM_gtm_cls0.DPLL_RR2[770].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2771 GTM_gtm_cls0.DPLL_RR2[771].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2772 GTM_gtm_cls0.DPLL_RR2[772].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2773 GTM_gtm_cls0.DPLL_RR2[773].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2774 GTM_gtm_cls0.DPLL_RR2[774].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2775 GTM_gtm_cls0.DPLL_RR2[775].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2776 GTM_gtm_cls0.DPLL_RR2[776].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2777 GTM_gtm_cls0.DPLL_RR2[777].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2778 GTM_gtm_cls0.DPLL_RR2[778].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2779 GTM_gtm_cls0.DPLL_RR2[779].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2780 GTM_gtm_cls0.DPLL_RR2[780].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2781 GTM_gtm_cls0.DPLL_RR2[781].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2782 GTM_gtm_cls0.DPLL_RR2[782].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2783 GTM_gtm_cls0.DPLL_RR2[783].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2784 GTM_gtm_cls0.DPLL_RR2[784].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2785 GTM_gtm_cls0.DPLL_RR2[785].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2786 GTM_gtm_cls0.DPLL_RR2[786].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2787 GTM_gtm_cls0.DPLL_RR2[787].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2788 GTM_gtm_cls0.DPLL_RR2[788].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2789 GTM_gtm_cls0.DPLL_RR2[789].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2790 GTM_gtm_cls0.DPLL_RR2[790].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2791 GTM_gtm_cls0.DPLL_RR2[791].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2792 GTM_gtm_cls0.DPLL_RR2[792].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2793 GTM_gtm_cls0.DPLL_RR2[793].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2794 GTM_gtm_cls0.DPLL_RR2[794].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2795 GTM_gtm_cls0.DPLL_RR2[795].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2796 GTM_gtm_cls0.DPLL_RR2[796].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2797 GTM_gtm_cls0.DPLL_RR2[797].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2798 GTM_gtm_cls0.DPLL_RR2[798].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2799 GTM_gtm_cls0.DPLL_RR2[799].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2800 GTM_gtm_cls0.DPLL_RR2[800].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2801 GTM_gtm_cls0.DPLL_RR2[801].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2802 GTM_gtm_cls0.DPLL_RR2[802].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2803 GTM_gtm_cls0.DPLL_RR2[803].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2804 GTM_gtm_cls0.DPLL_RR2[804].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2805 GTM_gtm_cls0.DPLL_RR2[805].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2806 GTM_gtm_cls0.DPLL_RR2[806].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2807 GTM_gtm_cls0.DPLL_RR2[807].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2808 GTM_gtm_cls0.DPLL_RR2[808].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2809 GTM_gtm_cls0.DPLL_RR2[809].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2810 GTM_gtm_cls0.DPLL_RR2[810].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2811 GTM_gtm_cls0.DPLL_RR2[811].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2812 GTM_gtm_cls0.DPLL_RR2[812].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2813 GTM_gtm_cls0.DPLL_RR2[813].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2814 GTM_gtm_cls0.DPLL_RR2[814].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2815 GTM_gtm_cls0.DPLL_RR2[815].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2816 GTM_gtm_cls0.DPLL_RR2[816].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2817 GTM_gtm_cls0.DPLL_RR2[817].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2818 GTM_gtm_cls0.DPLL_RR2[818].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2819 GTM_gtm_cls0.DPLL_RR2[819].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2820 GTM_gtm_cls0.DPLL_RR2[820].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2821 GTM_gtm_cls0.DPLL_RR2[821].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2822 GTM_gtm_cls0.DPLL_RR2[822].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2823 GTM_gtm_cls0.DPLL_RR2[823].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2824 GTM_gtm_cls0.DPLL_RR2[824].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2825 GTM_gtm_cls0.DPLL_RR2[825].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2826 GTM_gtm_cls0.DPLL_RR2[826].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2827 GTM_gtm_cls0.DPLL_RR2[827].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2828 GTM_gtm_cls0.DPLL_RR2[828].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2829 GTM_gtm_cls0.DPLL_RR2[829].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2830 GTM_gtm_cls0.DPLL_RR2[830].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2831 GTM_gtm_cls0.DPLL_RR2[831].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2832 GTM_gtm_cls0.DPLL_RR2[832].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2833 GTM_gtm_cls0.DPLL_RR2[833].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2834 GTM_gtm_cls0.DPLL_RR2[834].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2835 GTM_gtm_cls0.DPLL_RR2[835].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2836 GTM_gtm_cls0.DPLL_RR2[836].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2837 GTM_gtm_cls0.DPLL_RR2[837].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2838 GTM_gtm_cls0.DPLL_RR2[838].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2839 GTM_gtm_cls0.DPLL_RR2[839].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2840 GTM_gtm_cls0.DPLL_RR2[840].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2841 GTM_gtm_cls0.DPLL_RR2[841].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2842 GTM_gtm_cls0.DPLL_RR2[842].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2843 GTM_gtm_cls0.DPLL_RR2[843].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2844 GTM_gtm_cls0.DPLL_RR2[844].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2845 GTM_gtm_cls0.DPLL_RR2[845].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2846 GTM_gtm_cls0.DPLL_RR2[846].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2847 GTM_gtm_cls0.DPLL_RR2[847].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2848 GTM_gtm_cls0.DPLL_RR2[848].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2849 GTM_gtm_cls0.DPLL_RR2[849].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2850 GTM_gtm_cls0.DPLL_RR2[850].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2851 GTM_gtm_cls0.DPLL_RR2[851].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2852 GTM_gtm_cls0.DPLL_RR2[852].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2853 GTM_gtm_cls0.DPLL_RR2[853].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2854 GTM_gtm_cls0.DPLL_RR2[854].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2855 GTM_gtm_cls0.DPLL_RR2[855].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2856 GTM_gtm_cls0.DPLL_RR2[856].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2857 GTM_gtm_cls0.DPLL_RR2[857].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2858 GTM_gtm_cls0.DPLL_RR2[858].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2859 GTM_gtm_cls0.DPLL_RR2[859].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2860 GTM_gtm_cls0.DPLL_RR2[860].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2861 GTM_gtm_cls0.DPLL_RR2[861].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2862 GTM_gtm_cls0.DPLL_RR2[862].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2863 GTM_gtm_cls0.DPLL_RR2[863].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2864 GTM_gtm_cls0.DPLL_RR2[864].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2865 GTM_gtm_cls0.DPLL_RR2[865].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2866 GTM_gtm_cls0.DPLL_RR2[866].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2867 GTM_gtm_cls0.DPLL_RR2[867].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2868 GTM_gtm_cls0.DPLL_RR2[868].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2869 GTM_gtm_cls0.DPLL_RR2[869].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2870 GTM_gtm_cls0.DPLL_RR2[870].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2871 GTM_gtm_cls0.DPLL_RR2[871].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2872 GTM_gtm_cls0.DPLL_RR2[872].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2873 GTM_gtm_cls0.DPLL_RR2[873].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2874 GTM_gtm_cls0.DPLL_RR2[874].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2875 GTM_gtm_cls0.DPLL_RR2[875].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2876 GTM_gtm_cls0.DPLL_RR2[876].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2877 GTM_gtm_cls0.DPLL_RR2[877].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2878 GTM_gtm_cls0.DPLL_RR2[878].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2879 GTM_gtm_cls0.DPLL_RR2[879].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2880 GTM_gtm_cls0.DPLL_RR2[880].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2881 GTM_gtm_cls0.DPLL_RR2[881].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2882 GTM_gtm_cls0.DPLL_RR2[882].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2883 GTM_gtm_cls0.DPLL_RR2[883].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2884 GTM_gtm_cls0.DPLL_RR2[884].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2885 GTM_gtm_cls0.DPLL_RR2[885].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2886 GTM_gtm_cls0.DPLL_RR2[886].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2887 GTM_gtm_cls0.DPLL_RR2[887].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2888 GTM_gtm_cls0.DPLL_RR2[888].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2889 GTM_gtm_cls0.DPLL_RR2[889].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2890 GTM_gtm_cls0.DPLL_RR2[890].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2891 GTM_gtm_cls0.DPLL_RR2[891].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2892 GTM_gtm_cls0.DPLL_RR2[892].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2893 GTM_gtm_cls0.DPLL_RR2[893].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2894 GTM_gtm_cls0.DPLL_RR2[894].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2895 GTM_gtm_cls0.DPLL_RR2[895].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2896 GTM_gtm_cls0.DPLL_RR2[896].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2897 GTM_gtm_cls0.DPLL_RR2[897].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2898 GTM_gtm_cls0.DPLL_RR2[898].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2899 GTM_gtm_cls0.DPLL_RR2[899].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2900 GTM_gtm_cls0.DPLL_RR2[900].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2901 GTM_gtm_cls0.DPLL_RR2[901].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2902 GTM_gtm_cls0.DPLL_RR2[902].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2903 GTM_gtm_cls0.DPLL_RR2[903].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2904 GTM_gtm_cls0.DPLL_RR2[904].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2905 GTM_gtm_cls0.DPLL_RR2[905].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2906 GTM_gtm_cls0.DPLL_RR2[906].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2907 GTM_gtm_cls0.DPLL_RR2[907].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2908 GTM_gtm_cls0.DPLL_RR2[908].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2909 GTM_gtm_cls0.DPLL_RR2[909].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2910 GTM_gtm_cls0.DPLL_RR2[910].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2911 GTM_gtm_cls0.DPLL_RR2[911].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2912 GTM_gtm_cls0.DPLL_RR2[912].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2913 GTM_gtm_cls0.DPLL_RR2[913].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2914 GTM_gtm_cls0.DPLL_RR2[914].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2915 GTM_gtm_cls0.DPLL_RR2[915].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2916 GTM_gtm_cls0.DPLL_RR2[916].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2917 GTM_gtm_cls0.DPLL_RR2[917].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2918 GTM_gtm_cls0.DPLL_RR2[918].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2919 GTM_gtm_cls0.DPLL_RR2[919].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2920 GTM_gtm_cls0.DPLL_RR2[920].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2921 GTM_gtm_cls0.DPLL_RR2[921].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2922 GTM_gtm_cls0.DPLL_RR2[922].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2923 GTM_gtm_cls0.DPLL_RR2[923].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2924 GTM_gtm_cls0.DPLL_RR2[924].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2925 GTM_gtm_cls0.DPLL_RR2[925].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2926 GTM_gtm_cls0.DPLL_RR2[926].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2927 GTM_gtm_cls0.DPLL_RR2[927].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2928 GTM_gtm_cls0.DPLL_RR2[928].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2929 GTM_gtm_cls0.DPLL_RR2[929].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2930 GTM_gtm_cls0.DPLL_RR2[930].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2931 GTM_gtm_cls0.DPLL_RR2[931].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2932 GTM_gtm_cls0.DPLL_RR2[932].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2933 GTM_gtm_cls0.DPLL_RR2[933].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2934 GTM_gtm_cls0.DPLL_RR2[934].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2935 GTM_gtm_cls0.DPLL_RR2[935].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2936 GTM_gtm_cls0.DPLL_RR2[936].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2937 GTM_gtm_cls0.DPLL_RR2[937].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2938 GTM_gtm_cls0.DPLL_RR2[938].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2939 GTM_gtm_cls0.DPLL_RR2[939].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2940 GTM_gtm_cls0.DPLL_RR2[940].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2941 GTM_gtm_cls0.DPLL_RR2[941].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2942 GTM_gtm_cls0.DPLL_RR2[942].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2943 GTM_gtm_cls0.DPLL_RR2[943].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2944 GTM_gtm_cls0.DPLL_RR2[944].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2945 GTM_gtm_cls0.DPLL_RR2[945].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2946 GTM_gtm_cls0.DPLL_RR2[946].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2947 GTM_gtm_cls0.DPLL_RR2[947].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2948 GTM_gtm_cls0.DPLL_RR2[948].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2949 GTM_gtm_cls0.DPLL_RR2[949].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2950 GTM_gtm_cls0.DPLL_RR2[950].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2951 GTM_gtm_cls0.DPLL_RR2[951].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2952 GTM_gtm_cls0.DPLL_RR2[952].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2953 GTM_gtm_cls0.DPLL_RR2[953].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2954 GTM_gtm_cls0.DPLL_RR2[954].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2955 GTM_gtm_cls0.DPLL_RR2[955].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2956 GTM_gtm_cls0.DPLL_RR2[956].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2957 GTM_gtm_cls0.DPLL_RR2[957].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2958 GTM_gtm_cls0.DPLL_RR2[958].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2959 GTM_gtm_cls0.DPLL_RR2[959].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2960 GTM_gtm_cls0.DPLL_RR2[960].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2961 GTM_gtm_cls0.DPLL_RR2[961].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2962 GTM_gtm_cls0.DPLL_RR2[962].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2963 GTM_gtm_cls0.DPLL_RR2[963].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2964 GTM_gtm_cls0.DPLL_RR2[964].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2965 GTM_gtm_cls0.DPLL_RR2[965].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2966 GTM_gtm_cls0.DPLL_RR2[966].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2967 GTM_gtm_cls0.DPLL_RR2[967].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2968 GTM_gtm_cls0.DPLL_RR2[968].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2969 GTM_gtm_cls0.DPLL_RR2[969].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2970 GTM_gtm_cls0.DPLL_RR2[970].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2971 GTM_gtm_cls0.DPLL_RR2[971].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2972 GTM_gtm_cls0.DPLL_RR2[972].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2973 GTM_gtm_cls0.DPLL_RR2[973].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2974 GTM_gtm_cls0.DPLL_RR2[974].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2975 GTM_gtm_cls0.DPLL_RR2[975].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2976 GTM_gtm_cls0.DPLL_RR2[976].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2977 GTM_gtm_cls0.DPLL_RR2[977].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2978 GTM_gtm_cls0.DPLL_RR2[978].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2979 GTM_gtm_cls0.DPLL_RR2[979].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2980 GTM_gtm_cls0.DPLL_RR2[980].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2981 GTM_gtm_cls0.DPLL_RR2[981].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2982 GTM_gtm_cls0.DPLL_RR2[982].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2983 GTM_gtm_cls0.DPLL_RR2[983].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2984 GTM_gtm_cls0.DPLL_RR2[984].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2985 GTM_gtm_cls0.DPLL_RR2[985].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2986 GTM_gtm_cls0.DPLL_RR2[986].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2987 GTM_gtm_cls0.DPLL_RR2[987].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2988 GTM_gtm_cls0.DPLL_RR2[988].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2989 GTM_gtm_cls0.DPLL_RR2[989].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2990 GTM_gtm_cls0.DPLL_RR2[990].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2991 GTM_gtm_cls0.DPLL_RR2[991].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2992 GTM_gtm_cls0.DPLL_RR2[992].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2993 GTM_gtm_cls0.DPLL_RR2[993].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2994 GTM_gtm_cls0.DPLL_RR2[994].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2995 GTM_gtm_cls0.DPLL_RR2[995].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2996 GTM_gtm_cls0.DPLL_RR2[996].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2997 GTM_gtm_cls0.DPLL_RR2[997].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2998 GTM_gtm_cls0.DPLL_RR2[998].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR2999 GTM_gtm_cls0.DPLL_RR2[999].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21000 GTM_gtm_cls0.DPLL_RR2[1000].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21001 GTM_gtm_cls0.DPLL_RR2[1001].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21002 GTM_gtm_cls0.DPLL_RR2[1002].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21003 GTM_gtm_cls0.DPLL_RR2[1003].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21004 GTM_gtm_cls0.DPLL_RR2[1004].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21005 GTM_gtm_cls0.DPLL_RR2[1005].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21006 GTM_gtm_cls0.DPLL_RR2[1006].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21007 GTM_gtm_cls0.DPLL_RR2[1007].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21008 GTM_gtm_cls0.DPLL_RR2[1008].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21009 GTM_gtm_cls0.DPLL_RR2[1009].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21010 GTM_gtm_cls0.DPLL_RR2[1010].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21011 GTM_gtm_cls0.DPLL_RR2[1011].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21012 GTM_gtm_cls0.DPLL_RR2[1012].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21013 GTM_gtm_cls0.DPLL_RR2[1013].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21014 GTM_gtm_cls0.DPLL_RR2[1014].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21015 GTM_gtm_cls0.DPLL_RR2[1015].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21016 GTM_gtm_cls0.DPLL_RR2[1016].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21017 GTM_gtm_cls0.DPLL_RR2[1017].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21018 GTM_gtm_cls0.DPLL_RR2[1018].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21019 GTM_gtm_cls0.DPLL_RR2[1019].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21020 GTM_gtm_cls0.DPLL_RR2[1020].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21021 GTM_gtm_cls0.DPLL_RR2[1021].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21022 GTM_gtm_cls0.DPLL_RR2[1022].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21023 GTM_gtm_cls0.DPLL_RR2[1023].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21024 GTM_gtm_cls0.DPLL_RR2[1024].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21025 GTM_gtm_cls0.DPLL_RR2[1025].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21026 GTM_gtm_cls0.DPLL_RR2[1026].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21027 GTM_gtm_cls0.DPLL_RR2[1027].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21028 GTM_gtm_cls0.DPLL_RR2[1028].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21029 GTM_gtm_cls0.DPLL_RR2[1029].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21030 GTM_gtm_cls0.DPLL_RR2[1030].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21031 GTM_gtm_cls0.DPLL_RR2[1031].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21032 GTM_gtm_cls0.DPLL_RR2[1032].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21033 GTM_gtm_cls0.DPLL_RR2[1033].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21034 GTM_gtm_cls0.DPLL_RR2[1034].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21035 GTM_gtm_cls0.DPLL_RR2[1035].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21036 GTM_gtm_cls0.DPLL_RR2[1036].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21037 GTM_gtm_cls0.DPLL_RR2[1037].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21038 GTM_gtm_cls0.DPLL_RR2[1038].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21039 GTM_gtm_cls0.DPLL_RR2[1039].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21040 GTM_gtm_cls0.DPLL_RR2[1040].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21041 GTM_gtm_cls0.DPLL_RR2[1041].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21042 GTM_gtm_cls0.DPLL_RR2[1042].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21043 GTM_gtm_cls0.DPLL_RR2[1043].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21044 GTM_gtm_cls0.DPLL_RR2[1044].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21045 GTM_gtm_cls0.DPLL_RR2[1045].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21046 GTM_gtm_cls0.DPLL_RR2[1046].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21047 GTM_gtm_cls0.DPLL_RR2[1047].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21048 GTM_gtm_cls0.DPLL_RR2[1048].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21049 GTM_gtm_cls0.DPLL_RR2[1049].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21050 GTM_gtm_cls0.DPLL_RR2[1050].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21051 GTM_gtm_cls0.DPLL_RR2[1051].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21052 GTM_gtm_cls0.DPLL_RR2[1052].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21053 GTM_gtm_cls0.DPLL_RR2[1053].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21054 GTM_gtm_cls0.DPLL_RR2[1054].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21055 GTM_gtm_cls0.DPLL_RR2[1055].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21056 GTM_gtm_cls0.DPLL_RR2[1056].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21057 GTM_gtm_cls0.DPLL_RR2[1057].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21058 GTM_gtm_cls0.DPLL_RR2[1058].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21059 GTM_gtm_cls0.DPLL_RR2[1059].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21060 GTM_gtm_cls0.DPLL_RR2[1060].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21061 GTM_gtm_cls0.DPLL_RR2[1061].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21062 GTM_gtm_cls0.DPLL_RR2[1062].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21063 GTM_gtm_cls0.DPLL_RR2[1063].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21064 GTM_gtm_cls0.DPLL_RR2[1064].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21065 GTM_gtm_cls0.DPLL_RR2[1065].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21066 GTM_gtm_cls0.DPLL_RR2[1066].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21067 GTM_gtm_cls0.DPLL_RR2[1067].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21068 GTM_gtm_cls0.DPLL_RR2[1068].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21069 GTM_gtm_cls0.DPLL_RR2[1069].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21070 GTM_gtm_cls0.DPLL_RR2[1070].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21071 GTM_gtm_cls0.DPLL_RR2[1071].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21072 GTM_gtm_cls0.DPLL_RR2[1072].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21073 GTM_gtm_cls0.DPLL_RR2[1073].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21074 GTM_gtm_cls0.DPLL_RR2[1074].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21075 GTM_gtm_cls0.DPLL_RR2[1075].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21076 GTM_gtm_cls0.DPLL_RR2[1076].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21077 GTM_gtm_cls0.DPLL_RR2[1077].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21078 GTM_gtm_cls0.DPLL_RR2[1078].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21079 GTM_gtm_cls0.DPLL_RR2[1079].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21080 GTM_gtm_cls0.DPLL_RR2[1080].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21081 GTM_gtm_cls0.DPLL_RR2[1081].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21082 GTM_gtm_cls0.DPLL_RR2[1082].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21083 GTM_gtm_cls0.DPLL_RR2[1083].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21084 GTM_gtm_cls0.DPLL_RR2[1084].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21085 GTM_gtm_cls0.DPLL_RR2[1085].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21086 GTM_gtm_cls0.DPLL_RR2[1086].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21087 GTM_gtm_cls0.DPLL_RR2[1087].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21088 GTM_gtm_cls0.DPLL_RR2[1088].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21089 GTM_gtm_cls0.DPLL_RR2[1089].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21090 GTM_gtm_cls0.DPLL_RR2[1090].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21091 GTM_gtm_cls0.DPLL_RR2[1091].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21092 GTM_gtm_cls0.DPLL_RR2[1092].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21093 GTM_gtm_cls0.DPLL_RR2[1093].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21094 GTM_gtm_cls0.DPLL_RR2[1094].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21095 GTM_gtm_cls0.DPLL_RR2[1095].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21096 GTM_gtm_cls0.DPLL_RR2[1096].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21097 GTM_gtm_cls0.DPLL_RR2[1097].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21098 GTM_gtm_cls0.DPLL_RR2[1098].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21099 GTM_gtm_cls0.DPLL_RR2[1099].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21100 GTM_gtm_cls0.DPLL_RR2[1100].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21101 GTM_gtm_cls0.DPLL_RR2[1101].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21102 GTM_gtm_cls0.DPLL_RR2[1102].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21103 GTM_gtm_cls0.DPLL_RR2[1103].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21104 GTM_gtm_cls0.DPLL_RR2[1104].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21105 GTM_gtm_cls0.DPLL_RR2[1105].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21106 GTM_gtm_cls0.DPLL_RR2[1106].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21107 GTM_gtm_cls0.DPLL_RR2[1107].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21108 GTM_gtm_cls0.DPLL_RR2[1108].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21109 GTM_gtm_cls0.DPLL_RR2[1109].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21110 GTM_gtm_cls0.DPLL_RR2[1110].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21111 GTM_gtm_cls0.DPLL_RR2[1111].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21112 GTM_gtm_cls0.DPLL_RR2[1112].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21113 GTM_gtm_cls0.DPLL_RR2[1113].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21114 GTM_gtm_cls0.DPLL_RR2[1114].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21115 GTM_gtm_cls0.DPLL_RR2[1115].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21116 GTM_gtm_cls0.DPLL_RR2[1116].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21117 GTM_gtm_cls0.DPLL_RR2[1117].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21118 GTM_gtm_cls0.DPLL_RR2[1118].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21119 GTM_gtm_cls0.DPLL_RR2[1119].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21120 GTM_gtm_cls0.DPLL_RR2[1120].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21121 GTM_gtm_cls0.DPLL_RR2[1121].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21122 GTM_gtm_cls0.DPLL_RR2[1122].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21123 GTM_gtm_cls0.DPLL_RR2[1123].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21124 GTM_gtm_cls0.DPLL_RR2[1124].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21125 GTM_gtm_cls0.DPLL_RR2[1125].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21126 GTM_gtm_cls0.DPLL_RR2[1126].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21127 GTM_gtm_cls0.DPLL_RR2[1127].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21128 GTM_gtm_cls0.DPLL_RR2[1128].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21129 GTM_gtm_cls0.DPLL_RR2[1129].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21130 GTM_gtm_cls0.DPLL_RR2[1130].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21131 GTM_gtm_cls0.DPLL_RR2[1131].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21132 GTM_gtm_cls0.DPLL_RR2[1132].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21133 GTM_gtm_cls0.DPLL_RR2[1133].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21134 GTM_gtm_cls0.DPLL_RR2[1134].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21135 GTM_gtm_cls0.DPLL_RR2[1135].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21136 GTM_gtm_cls0.DPLL_RR2[1136].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21137 GTM_gtm_cls0.DPLL_RR2[1137].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21138 GTM_gtm_cls0.DPLL_RR2[1138].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21139 GTM_gtm_cls0.DPLL_RR2[1139].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21140 GTM_gtm_cls0.DPLL_RR2[1140].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21141 GTM_gtm_cls0.DPLL_RR2[1141].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21142 GTM_gtm_cls0.DPLL_RR2[1142].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21143 GTM_gtm_cls0.DPLL_RR2[1143].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21144 GTM_gtm_cls0.DPLL_RR2[1144].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21145 GTM_gtm_cls0.DPLL_RR2[1145].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21146 GTM_gtm_cls0.DPLL_RR2[1146].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21147 GTM_gtm_cls0.DPLL_RR2[1147].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21148 GTM_gtm_cls0.DPLL_RR2[1148].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21149 GTM_gtm_cls0.DPLL_RR2[1149].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21150 GTM_gtm_cls0.DPLL_RR2[1150].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21151 GTM_gtm_cls0.DPLL_RR2[1151].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21152 GTM_gtm_cls0.DPLL_RR2[1152].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21153 GTM_gtm_cls0.DPLL_RR2[1153].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21154 GTM_gtm_cls0.DPLL_RR2[1154].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21155 GTM_gtm_cls0.DPLL_RR2[1155].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21156 GTM_gtm_cls0.DPLL_RR2[1156].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21157 GTM_gtm_cls0.DPLL_RR2[1157].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21158 GTM_gtm_cls0.DPLL_RR2[1158].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21159 GTM_gtm_cls0.DPLL_RR2[1159].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21160 GTM_gtm_cls0.DPLL_RR2[1160].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21161 GTM_gtm_cls0.DPLL_RR2[1161].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21162 GTM_gtm_cls0.DPLL_RR2[1162].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21163 GTM_gtm_cls0.DPLL_RR2[1163].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21164 GTM_gtm_cls0.DPLL_RR2[1164].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21165 GTM_gtm_cls0.DPLL_RR2[1165].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21166 GTM_gtm_cls0.DPLL_RR2[1166].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21167 GTM_gtm_cls0.DPLL_RR2[1167].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21168 GTM_gtm_cls0.DPLL_RR2[1168].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21169 GTM_gtm_cls0.DPLL_RR2[1169].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21170 GTM_gtm_cls0.DPLL_RR2[1170].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21171 GTM_gtm_cls0.DPLL_RR2[1171].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21172 GTM_gtm_cls0.DPLL_RR2[1172].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21173 GTM_gtm_cls0.DPLL_RR2[1173].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21174 GTM_gtm_cls0.DPLL_RR2[1174].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21175 GTM_gtm_cls0.DPLL_RR2[1175].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21176 GTM_gtm_cls0.DPLL_RR2[1176].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21177 GTM_gtm_cls0.DPLL_RR2[1177].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21178 GTM_gtm_cls0.DPLL_RR2[1178].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21179 GTM_gtm_cls0.DPLL_RR2[1179].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21180 GTM_gtm_cls0.DPLL_RR2[1180].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21181 GTM_gtm_cls0.DPLL_RR2[1181].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21182 GTM_gtm_cls0.DPLL_RR2[1182].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21183 GTM_gtm_cls0.DPLL_RR2[1183].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21184 GTM_gtm_cls0.DPLL_RR2[1184].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21185 GTM_gtm_cls0.DPLL_RR2[1185].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21186 GTM_gtm_cls0.DPLL_RR2[1186].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21187 GTM_gtm_cls0.DPLL_RR2[1187].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21188 GTM_gtm_cls0.DPLL_RR2[1188].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21189 GTM_gtm_cls0.DPLL_RR2[1189].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21190 GTM_gtm_cls0.DPLL_RR2[1190].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21191 GTM_gtm_cls0.DPLL_RR2[1191].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21192 GTM_gtm_cls0.DPLL_RR2[1192].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21193 GTM_gtm_cls0.DPLL_RR2[1193].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21194 GTM_gtm_cls0.DPLL_RR2[1194].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21195 GTM_gtm_cls0.DPLL_RR2[1195].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21196 GTM_gtm_cls0.DPLL_RR2[1196].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21197 GTM_gtm_cls0.DPLL_RR2[1197].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21198 GTM_gtm_cls0.DPLL_RR2[1198].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21199 GTM_gtm_cls0.DPLL_RR2[1199].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21200 GTM_gtm_cls0.DPLL_RR2[1200].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21201 GTM_gtm_cls0.DPLL_RR2[1201].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21202 GTM_gtm_cls0.DPLL_RR2[1202].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21203 GTM_gtm_cls0.DPLL_RR2[1203].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21204 GTM_gtm_cls0.DPLL_RR2[1204].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21205 GTM_gtm_cls0.DPLL_RR2[1205].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21206 GTM_gtm_cls0.DPLL_RR2[1206].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21207 GTM_gtm_cls0.DPLL_RR2[1207].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21208 GTM_gtm_cls0.DPLL_RR2[1208].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21209 GTM_gtm_cls0.DPLL_RR2[1209].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21210 GTM_gtm_cls0.DPLL_RR2[1210].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21211 GTM_gtm_cls0.DPLL_RR2[1211].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21212 GTM_gtm_cls0.DPLL_RR2[1212].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21213 GTM_gtm_cls0.DPLL_RR2[1213].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21214 GTM_gtm_cls0.DPLL_RR2[1214].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21215 GTM_gtm_cls0.DPLL_RR2[1215].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21216 GTM_gtm_cls0.DPLL_RR2[1216].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21217 GTM_gtm_cls0.DPLL_RR2[1217].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21218 GTM_gtm_cls0.DPLL_RR2[1218].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21219 GTM_gtm_cls0.DPLL_RR2[1219].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21220 GTM_gtm_cls0.DPLL_RR2[1220].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21221 GTM_gtm_cls0.DPLL_RR2[1221].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21222 GTM_gtm_cls0.DPLL_RR2[1222].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21223 GTM_gtm_cls0.DPLL_RR2[1223].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21224 GTM_gtm_cls0.DPLL_RR2[1224].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21225 GTM_gtm_cls0.DPLL_RR2[1225].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21226 GTM_gtm_cls0.DPLL_RR2[1226].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21227 GTM_gtm_cls0.DPLL_RR2[1227].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21228 GTM_gtm_cls0.DPLL_RR2[1228].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21229 GTM_gtm_cls0.DPLL_RR2[1229].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21230 GTM_gtm_cls0.DPLL_RR2[1230].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21231 GTM_gtm_cls0.DPLL_RR2[1231].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21232 GTM_gtm_cls0.DPLL_RR2[1232].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21233 GTM_gtm_cls0.DPLL_RR2[1233].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21234 GTM_gtm_cls0.DPLL_RR2[1234].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21235 GTM_gtm_cls0.DPLL_RR2[1235].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21236 GTM_gtm_cls0.DPLL_RR2[1236].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21237 GTM_gtm_cls0.DPLL_RR2[1237].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21238 GTM_gtm_cls0.DPLL_RR2[1238].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21239 GTM_gtm_cls0.DPLL_RR2[1239].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21240 GTM_gtm_cls0.DPLL_RR2[1240].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21241 GTM_gtm_cls0.DPLL_RR2[1241].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21242 GTM_gtm_cls0.DPLL_RR2[1242].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21243 GTM_gtm_cls0.DPLL_RR2[1243].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21244 GTM_gtm_cls0.DPLL_RR2[1244].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21245 GTM_gtm_cls0.DPLL_RR2[1245].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21246 GTM_gtm_cls0.DPLL_RR2[1246].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21247 GTM_gtm_cls0.DPLL_RR2[1247].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21248 GTM_gtm_cls0.DPLL_RR2[1248].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21249 GTM_gtm_cls0.DPLL_RR2[1249].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21250 GTM_gtm_cls0.DPLL_RR2[1250].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21251 GTM_gtm_cls0.DPLL_RR2[1251].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21252 GTM_gtm_cls0.DPLL_RR2[1252].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21253 GTM_gtm_cls0.DPLL_RR2[1253].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21254 GTM_gtm_cls0.DPLL_RR2[1254].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21255 GTM_gtm_cls0.DPLL_RR2[1255].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21256 GTM_gtm_cls0.DPLL_RR2[1256].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21257 GTM_gtm_cls0.DPLL_RR2[1257].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21258 GTM_gtm_cls0.DPLL_RR2[1258].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21259 GTM_gtm_cls0.DPLL_RR2[1259].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21260 GTM_gtm_cls0.DPLL_RR2[1260].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21261 GTM_gtm_cls0.DPLL_RR2[1261].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21262 GTM_gtm_cls0.DPLL_RR2[1262].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21263 GTM_gtm_cls0.DPLL_RR2[1263].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21264 GTM_gtm_cls0.DPLL_RR2[1264].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21265 GTM_gtm_cls0.DPLL_RR2[1265].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21266 GTM_gtm_cls0.DPLL_RR2[1266].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21267 GTM_gtm_cls0.DPLL_RR2[1267].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21268 GTM_gtm_cls0.DPLL_RR2[1268].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21269 GTM_gtm_cls0.DPLL_RR2[1269].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21270 GTM_gtm_cls0.DPLL_RR2[1270].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21271 GTM_gtm_cls0.DPLL_RR2[1271].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21272 GTM_gtm_cls0.DPLL_RR2[1272].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21273 GTM_gtm_cls0.DPLL_RR2[1273].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21274 GTM_gtm_cls0.DPLL_RR2[1274].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21275 GTM_gtm_cls0.DPLL_RR2[1275].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21276 GTM_gtm_cls0.DPLL_RR2[1276].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21277 GTM_gtm_cls0.DPLL_RR2[1277].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21278 GTM_gtm_cls0.DPLL_RR2[1278].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21279 GTM_gtm_cls0.DPLL_RR2[1279].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21280 GTM_gtm_cls0.DPLL_RR2[1280].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21281 GTM_gtm_cls0.DPLL_RR2[1281].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21282 GTM_gtm_cls0.DPLL_RR2[1282].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21283 GTM_gtm_cls0.DPLL_RR2[1283].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21284 GTM_gtm_cls0.DPLL_RR2[1284].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21285 GTM_gtm_cls0.DPLL_RR2[1285].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21286 GTM_gtm_cls0.DPLL_RR2[1286].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21287 GTM_gtm_cls0.DPLL_RR2[1287].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21288 GTM_gtm_cls0.DPLL_RR2[1288].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21289 GTM_gtm_cls0.DPLL_RR2[1289].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21290 GTM_gtm_cls0.DPLL_RR2[1290].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21291 GTM_gtm_cls0.DPLL_RR2[1291].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21292 GTM_gtm_cls0.DPLL_RR2[1292].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21293 GTM_gtm_cls0.DPLL_RR2[1293].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21294 GTM_gtm_cls0.DPLL_RR2[1294].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21295 GTM_gtm_cls0.DPLL_RR2[1295].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21296 GTM_gtm_cls0.DPLL_RR2[1296].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21297 GTM_gtm_cls0.DPLL_RR2[1297].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21298 GTM_gtm_cls0.DPLL_RR2[1298].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21299 GTM_gtm_cls0.DPLL_RR2[1299].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21300 GTM_gtm_cls0.DPLL_RR2[1300].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21301 GTM_gtm_cls0.DPLL_RR2[1301].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21302 GTM_gtm_cls0.DPLL_RR2[1302].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21303 GTM_gtm_cls0.DPLL_RR2[1303].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21304 GTM_gtm_cls0.DPLL_RR2[1304].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21305 GTM_gtm_cls0.DPLL_RR2[1305].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21306 GTM_gtm_cls0.DPLL_RR2[1306].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21307 GTM_gtm_cls0.DPLL_RR2[1307].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21308 GTM_gtm_cls0.DPLL_RR2[1308].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21309 GTM_gtm_cls0.DPLL_RR2[1309].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21310 GTM_gtm_cls0.DPLL_RR2[1310].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21311 GTM_gtm_cls0.DPLL_RR2[1311].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21312 GTM_gtm_cls0.DPLL_RR2[1312].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21313 GTM_gtm_cls0.DPLL_RR2[1313].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21314 GTM_gtm_cls0.DPLL_RR2[1314].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21315 GTM_gtm_cls0.DPLL_RR2[1315].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21316 GTM_gtm_cls0.DPLL_RR2[1316].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21317 GTM_gtm_cls0.DPLL_RR2[1317].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21318 GTM_gtm_cls0.DPLL_RR2[1318].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21319 GTM_gtm_cls0.DPLL_RR2[1319].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21320 GTM_gtm_cls0.DPLL_RR2[1320].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21321 GTM_gtm_cls0.DPLL_RR2[1321].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21322 GTM_gtm_cls0.DPLL_RR2[1322].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21323 GTM_gtm_cls0.DPLL_RR2[1323].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21324 GTM_gtm_cls0.DPLL_RR2[1324].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21325 GTM_gtm_cls0.DPLL_RR2[1325].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21326 GTM_gtm_cls0.DPLL_RR2[1326].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21327 GTM_gtm_cls0.DPLL_RR2[1327].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21328 GTM_gtm_cls0.DPLL_RR2[1328].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21329 GTM_gtm_cls0.DPLL_RR2[1329].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21330 GTM_gtm_cls0.DPLL_RR2[1330].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21331 GTM_gtm_cls0.DPLL_RR2[1331].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21332 GTM_gtm_cls0.DPLL_RR2[1332].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21333 GTM_gtm_cls0.DPLL_RR2[1333].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21334 GTM_gtm_cls0.DPLL_RR2[1334].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21335 GTM_gtm_cls0.DPLL_RR2[1335].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21336 GTM_gtm_cls0.DPLL_RR2[1336].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21337 GTM_gtm_cls0.DPLL_RR2[1337].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21338 GTM_gtm_cls0.DPLL_RR2[1338].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21339 GTM_gtm_cls0.DPLL_RR2[1339].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21340 GTM_gtm_cls0.DPLL_RR2[1340].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21341 GTM_gtm_cls0.DPLL_RR2[1341].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21342 GTM_gtm_cls0.DPLL_RR2[1342].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21343 GTM_gtm_cls0.DPLL_RR2[1343].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21344 GTM_gtm_cls0.DPLL_RR2[1344].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21345 GTM_gtm_cls0.DPLL_RR2[1345].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21346 GTM_gtm_cls0.DPLL_RR2[1346].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21347 GTM_gtm_cls0.DPLL_RR2[1347].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21348 GTM_gtm_cls0.DPLL_RR2[1348].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21349 GTM_gtm_cls0.DPLL_RR2[1349].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21350 GTM_gtm_cls0.DPLL_RR2[1350].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21351 GTM_gtm_cls0.DPLL_RR2[1351].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21352 GTM_gtm_cls0.DPLL_RR2[1352].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21353 GTM_gtm_cls0.DPLL_RR2[1353].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21354 GTM_gtm_cls0.DPLL_RR2[1354].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21355 GTM_gtm_cls0.DPLL_RR2[1355].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21356 GTM_gtm_cls0.DPLL_RR2[1356].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21357 GTM_gtm_cls0.DPLL_RR2[1357].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21358 GTM_gtm_cls0.DPLL_RR2[1358].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21359 GTM_gtm_cls0.DPLL_RR2[1359].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21360 GTM_gtm_cls0.DPLL_RR2[1360].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21361 GTM_gtm_cls0.DPLL_RR2[1361].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21362 GTM_gtm_cls0.DPLL_RR2[1362].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21363 GTM_gtm_cls0.DPLL_RR2[1363].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21364 GTM_gtm_cls0.DPLL_RR2[1364].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21365 GTM_gtm_cls0.DPLL_RR2[1365].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21366 GTM_gtm_cls0.DPLL_RR2[1366].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21367 GTM_gtm_cls0.DPLL_RR2[1367].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21368 GTM_gtm_cls0.DPLL_RR2[1368].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21369 GTM_gtm_cls0.DPLL_RR2[1369].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21370 GTM_gtm_cls0.DPLL_RR2[1370].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21371 GTM_gtm_cls0.DPLL_RR2[1371].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21372 GTM_gtm_cls0.DPLL_RR2[1372].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21373 GTM_gtm_cls0.DPLL_RR2[1373].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21374 GTM_gtm_cls0.DPLL_RR2[1374].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21375 GTM_gtm_cls0.DPLL_RR2[1375].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21376 GTM_gtm_cls0.DPLL_RR2[1376].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21377 GTM_gtm_cls0.DPLL_RR2[1377].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21378 GTM_gtm_cls0.DPLL_RR2[1378].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21379 GTM_gtm_cls0.DPLL_RR2[1379].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21380 GTM_gtm_cls0.DPLL_RR2[1380].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21381 GTM_gtm_cls0.DPLL_RR2[1381].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21382 GTM_gtm_cls0.DPLL_RR2[1382].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21383 GTM_gtm_cls0.DPLL_RR2[1383].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21384 GTM_gtm_cls0.DPLL_RR2[1384].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21385 GTM_gtm_cls0.DPLL_RR2[1385].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21386 GTM_gtm_cls0.DPLL_RR2[1386].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21387 GTM_gtm_cls0.DPLL_RR2[1387].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21388 GTM_gtm_cls0.DPLL_RR2[1388].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21389 GTM_gtm_cls0.DPLL_RR2[1389].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21390 GTM_gtm_cls0.DPLL_RR2[1390].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21391 GTM_gtm_cls0.DPLL_RR2[1391].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21392 GTM_gtm_cls0.DPLL_RR2[1392].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21393 GTM_gtm_cls0.DPLL_RR2[1393].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21394 GTM_gtm_cls0.DPLL_RR2[1394].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21395 GTM_gtm_cls0.DPLL_RR2[1395].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21396 GTM_gtm_cls0.DPLL_RR2[1396].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21397 GTM_gtm_cls0.DPLL_RR2[1397].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21398 GTM_gtm_cls0.DPLL_RR2[1398].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21399 GTM_gtm_cls0.DPLL_RR2[1399].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21400 GTM_gtm_cls0.DPLL_RR2[1400].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21401 GTM_gtm_cls0.DPLL_RR2[1401].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21402 GTM_gtm_cls0.DPLL_RR2[1402].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21403 GTM_gtm_cls0.DPLL_RR2[1403].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21404 GTM_gtm_cls0.DPLL_RR2[1404].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21405 GTM_gtm_cls0.DPLL_RR2[1405].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21406 GTM_gtm_cls0.DPLL_RR2[1406].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21407 GTM_gtm_cls0.DPLL_RR2[1407].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21408 GTM_gtm_cls0.DPLL_RR2[1408].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21409 GTM_gtm_cls0.DPLL_RR2[1409].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21410 GTM_gtm_cls0.DPLL_RR2[1410].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21411 GTM_gtm_cls0.DPLL_RR2[1411].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21412 GTM_gtm_cls0.DPLL_RR2[1412].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21413 GTM_gtm_cls0.DPLL_RR2[1413].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21414 GTM_gtm_cls0.DPLL_RR2[1414].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21415 GTM_gtm_cls0.DPLL_RR2[1415].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21416 GTM_gtm_cls0.DPLL_RR2[1416].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21417 GTM_gtm_cls0.DPLL_RR2[1417].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21418 GTM_gtm_cls0.DPLL_RR2[1418].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21419 GTM_gtm_cls0.DPLL_RR2[1419].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21420 GTM_gtm_cls0.DPLL_RR2[1420].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21421 GTM_gtm_cls0.DPLL_RR2[1421].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21422 GTM_gtm_cls0.DPLL_RR2[1422].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21423 GTM_gtm_cls0.DPLL_RR2[1423].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21424 GTM_gtm_cls0.DPLL_RR2[1424].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21425 GTM_gtm_cls0.DPLL_RR2[1425].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21426 GTM_gtm_cls0.DPLL_RR2[1426].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21427 GTM_gtm_cls0.DPLL_RR2[1427].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21428 GTM_gtm_cls0.DPLL_RR2[1428].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21429 GTM_gtm_cls0.DPLL_RR2[1429].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21430 GTM_gtm_cls0.DPLL_RR2[1430].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21431 GTM_gtm_cls0.DPLL_RR2[1431].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21432 GTM_gtm_cls0.DPLL_RR2[1432].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21433 GTM_gtm_cls0.DPLL_RR2[1433].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21434 GTM_gtm_cls0.DPLL_RR2[1434].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21435 GTM_gtm_cls0.DPLL_RR2[1435].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21436 GTM_gtm_cls0.DPLL_RR2[1436].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21437 GTM_gtm_cls0.DPLL_RR2[1437].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21438 GTM_gtm_cls0.DPLL_RR2[1438].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21439 GTM_gtm_cls0.DPLL_RR2[1439].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21440 GTM_gtm_cls0.DPLL_RR2[1440].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21441 GTM_gtm_cls0.DPLL_RR2[1441].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21442 GTM_gtm_cls0.DPLL_RR2[1442].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21443 GTM_gtm_cls0.DPLL_RR2[1443].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21444 GTM_gtm_cls0.DPLL_RR2[1444].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21445 GTM_gtm_cls0.DPLL_RR2[1445].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21446 GTM_gtm_cls0.DPLL_RR2[1446].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21447 GTM_gtm_cls0.DPLL_RR2[1447].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21448 GTM_gtm_cls0.DPLL_RR2[1448].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21449 GTM_gtm_cls0.DPLL_RR2[1449].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21450 GTM_gtm_cls0.DPLL_RR2[1450].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21451 GTM_gtm_cls0.DPLL_RR2[1451].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21452 GTM_gtm_cls0.DPLL_RR2[1452].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21453 GTM_gtm_cls0.DPLL_RR2[1453].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21454 GTM_gtm_cls0.DPLL_RR2[1454].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21455 GTM_gtm_cls0.DPLL_RR2[1455].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21456 GTM_gtm_cls0.DPLL_RR2[1456].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21457 GTM_gtm_cls0.DPLL_RR2[1457].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21458 GTM_gtm_cls0.DPLL_RR2[1458].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21459 GTM_gtm_cls0.DPLL_RR2[1459].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21460 GTM_gtm_cls0.DPLL_RR2[1460].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21461 GTM_gtm_cls0.DPLL_RR2[1461].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21462 GTM_gtm_cls0.DPLL_RR2[1462].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21463 GTM_gtm_cls0.DPLL_RR2[1463].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21464 GTM_gtm_cls0.DPLL_RR2[1464].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21465 GTM_gtm_cls0.DPLL_RR2[1465].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21466 GTM_gtm_cls0.DPLL_RR2[1466].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21467 GTM_gtm_cls0.DPLL_RR2[1467].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21468 GTM_gtm_cls0.DPLL_RR2[1468].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21469 GTM_gtm_cls0.DPLL_RR2[1469].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21470 GTM_gtm_cls0.DPLL_RR2[1470].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21471 GTM_gtm_cls0.DPLL_RR2[1471].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21472 GTM_gtm_cls0.DPLL_RR2[1472].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21473 GTM_gtm_cls0.DPLL_RR2[1473].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21474 GTM_gtm_cls0.DPLL_RR2[1474].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21475 GTM_gtm_cls0.DPLL_RR2[1475].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21476 GTM_gtm_cls0.DPLL_RR2[1476].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21477 GTM_gtm_cls0.DPLL_RR2[1477].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21478 GTM_gtm_cls0.DPLL_RR2[1478].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21479 GTM_gtm_cls0.DPLL_RR2[1479].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21480 GTM_gtm_cls0.DPLL_RR2[1480].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21481 GTM_gtm_cls0.DPLL_RR2[1481].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21482 GTM_gtm_cls0.DPLL_RR2[1482].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21483 GTM_gtm_cls0.DPLL_RR2[1483].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21484 GTM_gtm_cls0.DPLL_RR2[1484].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21485 GTM_gtm_cls0.DPLL_RR2[1485].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21486 GTM_gtm_cls0.DPLL_RR2[1486].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21487 GTM_gtm_cls0.DPLL_RR2[1487].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21488 GTM_gtm_cls0.DPLL_RR2[1488].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21489 GTM_gtm_cls0.DPLL_RR2[1489].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21490 GTM_gtm_cls0.DPLL_RR2[1490].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21491 GTM_gtm_cls0.DPLL_RR2[1491].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21492 GTM_gtm_cls0.DPLL_RR2[1492].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21493 GTM_gtm_cls0.DPLL_RR2[1493].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21494 GTM_gtm_cls0.DPLL_RR2[1494].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21495 GTM_gtm_cls0.DPLL_RR2[1495].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21496 GTM_gtm_cls0.DPLL_RR2[1496].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21497 GTM_gtm_cls0.DPLL_RR2[1497].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21498 GTM_gtm_cls0.DPLL_RR2[1498].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21499 GTM_gtm_cls0.DPLL_RR2[1499].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21500 GTM_gtm_cls0.DPLL_RR2[1500].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21501 GTM_gtm_cls0.DPLL_RR2[1501].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21502 GTM_gtm_cls0.DPLL_RR2[1502].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21503 GTM_gtm_cls0.DPLL_RR2[1503].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21504 GTM_gtm_cls0.DPLL_RR2[1504].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21505 GTM_gtm_cls0.DPLL_RR2[1505].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21506 GTM_gtm_cls0.DPLL_RR2[1506].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21507 GTM_gtm_cls0.DPLL_RR2[1507].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21508 GTM_gtm_cls0.DPLL_RR2[1508].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21509 GTM_gtm_cls0.DPLL_RR2[1509].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21510 GTM_gtm_cls0.DPLL_RR2[1510].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21511 GTM_gtm_cls0.DPLL_RR2[1511].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21512 GTM_gtm_cls0.DPLL_RR2[1512].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21513 GTM_gtm_cls0.DPLL_RR2[1513].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21514 GTM_gtm_cls0.DPLL_RR2[1514].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21515 GTM_gtm_cls0.DPLL_RR2[1515].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21516 GTM_gtm_cls0.DPLL_RR2[1516].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21517 GTM_gtm_cls0.DPLL_RR2[1517].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21518 GTM_gtm_cls0.DPLL_RR2[1518].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21519 GTM_gtm_cls0.DPLL_RR2[1519].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21520 GTM_gtm_cls0.DPLL_RR2[1520].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21521 GTM_gtm_cls0.DPLL_RR2[1521].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21522 GTM_gtm_cls0.DPLL_RR2[1522].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21523 GTM_gtm_cls0.DPLL_RR2[1523].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21524 GTM_gtm_cls0.DPLL_RR2[1524].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21525 GTM_gtm_cls0.DPLL_RR2[1525].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21526 GTM_gtm_cls0.DPLL_RR2[1526].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21527 GTM_gtm_cls0.DPLL_RR2[1527].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21528 GTM_gtm_cls0.DPLL_RR2[1528].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21529 GTM_gtm_cls0.DPLL_RR2[1529].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21530 GTM_gtm_cls0.DPLL_RR2[1530].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21531 GTM_gtm_cls0.DPLL_RR2[1531].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21532 GTM_gtm_cls0.DPLL_RR2[1532].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21533 GTM_gtm_cls0.DPLL_RR2[1533].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21534 GTM_gtm_cls0.DPLL_RR2[1534].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21535 GTM_gtm_cls0.DPLL_RR2[1535].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21536 GTM_gtm_cls0.DPLL_RR2[1536].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21537 GTM_gtm_cls0.DPLL_RR2[1537].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21538 GTM_gtm_cls0.DPLL_RR2[1538].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21539 GTM_gtm_cls0.DPLL_RR2[1539].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21540 GTM_gtm_cls0.DPLL_RR2[1540].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21541 GTM_gtm_cls0.DPLL_RR2[1541].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21542 GTM_gtm_cls0.DPLL_RR2[1542].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21543 GTM_gtm_cls0.DPLL_RR2[1543].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21544 GTM_gtm_cls0.DPLL_RR2[1544].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21545 GTM_gtm_cls0.DPLL_RR2[1545].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21546 GTM_gtm_cls0.DPLL_RR2[1546].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21547 GTM_gtm_cls0.DPLL_RR2[1547].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21548 GTM_gtm_cls0.DPLL_RR2[1548].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21549 GTM_gtm_cls0.DPLL_RR2[1549].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21550 GTM_gtm_cls0.DPLL_RR2[1550].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21551 GTM_gtm_cls0.DPLL_RR2[1551].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21552 GTM_gtm_cls0.DPLL_RR2[1552].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21553 GTM_gtm_cls0.DPLL_RR2[1553].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21554 GTM_gtm_cls0.DPLL_RR2[1554].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21555 GTM_gtm_cls0.DPLL_RR2[1555].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21556 GTM_gtm_cls0.DPLL_RR2[1556].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21557 GTM_gtm_cls0.DPLL_RR2[1557].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21558 GTM_gtm_cls0.DPLL_RR2[1558].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21559 GTM_gtm_cls0.DPLL_RR2[1559].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21560 GTM_gtm_cls0.DPLL_RR2[1560].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21561 GTM_gtm_cls0.DPLL_RR2[1561].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21562 GTM_gtm_cls0.DPLL_RR2[1562].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21563 GTM_gtm_cls0.DPLL_RR2[1563].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21564 GTM_gtm_cls0.DPLL_RR2[1564].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21565 GTM_gtm_cls0.DPLL_RR2[1565].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21566 GTM_gtm_cls0.DPLL_RR2[1566].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21567 GTM_gtm_cls0.DPLL_RR2[1567].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21568 GTM_gtm_cls0.DPLL_RR2[1568].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21569 GTM_gtm_cls0.DPLL_RR2[1569].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21570 GTM_gtm_cls0.DPLL_RR2[1570].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21571 GTM_gtm_cls0.DPLL_RR2[1571].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21572 GTM_gtm_cls0.DPLL_RR2[1572].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21573 GTM_gtm_cls0.DPLL_RR2[1573].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21574 GTM_gtm_cls0.DPLL_RR2[1574].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21575 GTM_gtm_cls0.DPLL_RR2[1575].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21576 GTM_gtm_cls0.DPLL_RR2[1576].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21577 GTM_gtm_cls0.DPLL_RR2[1577].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21578 GTM_gtm_cls0.DPLL_RR2[1578].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21579 GTM_gtm_cls0.DPLL_RR2[1579].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21580 GTM_gtm_cls0.DPLL_RR2[1580].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21581 GTM_gtm_cls0.DPLL_RR2[1581].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21582 GTM_gtm_cls0.DPLL_RR2[1582].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21583 GTM_gtm_cls0.DPLL_RR2[1583].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21584 GTM_gtm_cls0.DPLL_RR2[1584].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21585 GTM_gtm_cls0.DPLL_RR2[1585].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21586 GTM_gtm_cls0.DPLL_RR2[1586].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21587 GTM_gtm_cls0.DPLL_RR2[1587].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21588 GTM_gtm_cls0.DPLL_RR2[1588].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21589 GTM_gtm_cls0.DPLL_RR2[1589].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21590 GTM_gtm_cls0.DPLL_RR2[1590].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21591 GTM_gtm_cls0.DPLL_RR2[1591].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21592 GTM_gtm_cls0.DPLL_RR2[1592].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21593 GTM_gtm_cls0.DPLL_RR2[1593].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21594 GTM_gtm_cls0.DPLL_RR2[1594].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21595 GTM_gtm_cls0.DPLL_RR2[1595].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21596 GTM_gtm_cls0.DPLL_RR2[1596].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21597 GTM_gtm_cls0.DPLL_RR2[1597].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21598 GTM_gtm_cls0.DPLL_RR2[1598].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21599 GTM_gtm_cls0.DPLL_RR2[1599].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21600 GTM_gtm_cls0.DPLL_RR2[1600].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21601 GTM_gtm_cls0.DPLL_RR2[1601].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21602 GTM_gtm_cls0.DPLL_RR2[1602].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21603 GTM_gtm_cls0.DPLL_RR2[1603].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21604 GTM_gtm_cls0.DPLL_RR2[1604].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21605 GTM_gtm_cls0.DPLL_RR2[1605].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21606 GTM_gtm_cls0.DPLL_RR2[1606].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21607 GTM_gtm_cls0.DPLL_RR2[1607].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21608 GTM_gtm_cls0.DPLL_RR2[1608].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21609 GTM_gtm_cls0.DPLL_RR2[1609].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21610 GTM_gtm_cls0.DPLL_RR2[1610].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21611 GTM_gtm_cls0.DPLL_RR2[1611].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21612 GTM_gtm_cls0.DPLL_RR2[1612].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21613 GTM_gtm_cls0.DPLL_RR2[1613].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21614 GTM_gtm_cls0.DPLL_RR2[1614].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21615 GTM_gtm_cls0.DPLL_RR2[1615].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21616 GTM_gtm_cls0.DPLL_RR2[1616].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21617 GTM_gtm_cls0.DPLL_RR2[1617].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21618 GTM_gtm_cls0.DPLL_RR2[1618].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21619 GTM_gtm_cls0.DPLL_RR2[1619].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21620 GTM_gtm_cls0.DPLL_RR2[1620].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21621 GTM_gtm_cls0.DPLL_RR2[1621].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21622 GTM_gtm_cls0.DPLL_RR2[1622].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21623 GTM_gtm_cls0.DPLL_RR2[1623].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21624 GTM_gtm_cls0.DPLL_RR2[1624].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21625 GTM_gtm_cls0.DPLL_RR2[1625].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21626 GTM_gtm_cls0.DPLL_RR2[1626].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21627 GTM_gtm_cls0.DPLL_RR2[1627].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21628 GTM_gtm_cls0.DPLL_RR2[1628].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21629 GTM_gtm_cls0.DPLL_RR2[1629].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21630 GTM_gtm_cls0.DPLL_RR2[1630].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21631 GTM_gtm_cls0.DPLL_RR2[1631].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21632 GTM_gtm_cls0.DPLL_RR2[1632].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21633 GTM_gtm_cls0.DPLL_RR2[1633].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21634 GTM_gtm_cls0.DPLL_RR2[1634].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21635 GTM_gtm_cls0.DPLL_RR2[1635].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21636 GTM_gtm_cls0.DPLL_RR2[1636].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21637 GTM_gtm_cls0.DPLL_RR2[1637].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21638 GTM_gtm_cls0.DPLL_RR2[1638].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21639 GTM_gtm_cls0.DPLL_RR2[1639].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21640 GTM_gtm_cls0.DPLL_RR2[1640].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21641 GTM_gtm_cls0.DPLL_RR2[1641].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21642 GTM_gtm_cls0.DPLL_RR2[1642].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21643 GTM_gtm_cls0.DPLL_RR2[1643].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21644 GTM_gtm_cls0.DPLL_RR2[1644].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21645 GTM_gtm_cls0.DPLL_RR2[1645].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21646 GTM_gtm_cls0.DPLL_RR2[1646].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21647 GTM_gtm_cls0.DPLL_RR2[1647].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21648 GTM_gtm_cls0.DPLL_RR2[1648].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21649 GTM_gtm_cls0.DPLL_RR2[1649].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21650 GTM_gtm_cls0.DPLL_RR2[1650].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21651 GTM_gtm_cls0.DPLL_RR2[1651].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21652 GTM_gtm_cls0.DPLL_RR2[1652].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21653 GTM_gtm_cls0.DPLL_RR2[1653].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21654 GTM_gtm_cls0.DPLL_RR2[1654].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21655 GTM_gtm_cls0.DPLL_RR2[1655].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21656 GTM_gtm_cls0.DPLL_RR2[1656].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21657 GTM_gtm_cls0.DPLL_RR2[1657].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21658 GTM_gtm_cls0.DPLL_RR2[1658].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21659 GTM_gtm_cls0.DPLL_RR2[1659].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21660 GTM_gtm_cls0.DPLL_RR2[1660].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21661 GTM_gtm_cls0.DPLL_RR2[1661].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21662 GTM_gtm_cls0.DPLL_RR2[1662].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21663 GTM_gtm_cls0.DPLL_RR2[1663].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21664 GTM_gtm_cls0.DPLL_RR2[1664].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21665 GTM_gtm_cls0.DPLL_RR2[1665].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21666 GTM_gtm_cls0.DPLL_RR2[1666].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21667 GTM_gtm_cls0.DPLL_RR2[1667].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21668 GTM_gtm_cls0.DPLL_RR2[1668].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21669 GTM_gtm_cls0.DPLL_RR2[1669].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21670 GTM_gtm_cls0.DPLL_RR2[1670].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21671 GTM_gtm_cls0.DPLL_RR2[1671].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21672 GTM_gtm_cls0.DPLL_RR2[1672].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21673 GTM_gtm_cls0.DPLL_RR2[1673].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21674 GTM_gtm_cls0.DPLL_RR2[1674].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21675 GTM_gtm_cls0.DPLL_RR2[1675].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21676 GTM_gtm_cls0.DPLL_RR2[1676].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21677 GTM_gtm_cls0.DPLL_RR2[1677].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21678 GTM_gtm_cls0.DPLL_RR2[1678].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21679 GTM_gtm_cls0.DPLL_RR2[1679].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21680 GTM_gtm_cls0.DPLL_RR2[1680].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21681 GTM_gtm_cls0.DPLL_RR2[1681].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21682 GTM_gtm_cls0.DPLL_RR2[1682].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21683 GTM_gtm_cls0.DPLL_RR2[1683].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21684 GTM_gtm_cls0.DPLL_RR2[1684].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21685 GTM_gtm_cls0.DPLL_RR2[1685].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21686 GTM_gtm_cls0.DPLL_RR2[1686].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21687 GTM_gtm_cls0.DPLL_RR2[1687].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21688 GTM_gtm_cls0.DPLL_RR2[1688].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21689 GTM_gtm_cls0.DPLL_RR2[1689].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21690 GTM_gtm_cls0.DPLL_RR2[1690].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21691 GTM_gtm_cls0.DPLL_RR2[1691].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21692 GTM_gtm_cls0.DPLL_RR2[1692].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21693 GTM_gtm_cls0.DPLL_RR2[1693].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21694 GTM_gtm_cls0.DPLL_RR2[1694].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21695 GTM_gtm_cls0.DPLL_RR2[1695].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21696 GTM_gtm_cls0.DPLL_RR2[1696].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21697 GTM_gtm_cls0.DPLL_RR2[1697].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21698 GTM_gtm_cls0.DPLL_RR2[1698].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21699 GTM_gtm_cls0.DPLL_RR2[1699].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21700 GTM_gtm_cls0.DPLL_RR2[1700].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21701 GTM_gtm_cls0.DPLL_RR2[1701].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21702 GTM_gtm_cls0.DPLL_RR2[1702].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21703 GTM_gtm_cls0.DPLL_RR2[1703].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21704 GTM_gtm_cls0.DPLL_RR2[1704].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21705 GTM_gtm_cls0.DPLL_RR2[1705].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21706 GTM_gtm_cls0.DPLL_RR2[1706].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21707 GTM_gtm_cls0.DPLL_RR2[1707].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21708 GTM_gtm_cls0.DPLL_RR2[1708].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21709 GTM_gtm_cls0.DPLL_RR2[1709].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21710 GTM_gtm_cls0.DPLL_RR2[1710].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21711 GTM_gtm_cls0.DPLL_RR2[1711].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21712 GTM_gtm_cls0.DPLL_RR2[1712].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21713 GTM_gtm_cls0.DPLL_RR2[1713].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21714 GTM_gtm_cls0.DPLL_RR2[1714].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21715 GTM_gtm_cls0.DPLL_RR2[1715].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21716 GTM_gtm_cls0.DPLL_RR2[1716].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21717 GTM_gtm_cls0.DPLL_RR2[1717].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21718 GTM_gtm_cls0.DPLL_RR2[1718].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21719 GTM_gtm_cls0.DPLL_RR2[1719].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21720 GTM_gtm_cls0.DPLL_RR2[1720].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21721 GTM_gtm_cls0.DPLL_RR2[1721].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21722 GTM_gtm_cls0.DPLL_RR2[1722].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21723 GTM_gtm_cls0.DPLL_RR2[1723].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21724 GTM_gtm_cls0.DPLL_RR2[1724].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21725 GTM_gtm_cls0.DPLL_RR2[1725].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21726 GTM_gtm_cls0.DPLL_RR2[1726].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21727 GTM_gtm_cls0.DPLL_RR2[1727].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21728 GTM_gtm_cls0.DPLL_RR2[1728].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21729 GTM_gtm_cls0.DPLL_RR2[1729].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21730 GTM_gtm_cls0.DPLL_RR2[1730].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21731 GTM_gtm_cls0.DPLL_RR2[1731].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21732 GTM_gtm_cls0.DPLL_RR2[1732].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21733 GTM_gtm_cls0.DPLL_RR2[1733].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21734 GTM_gtm_cls0.DPLL_RR2[1734].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21735 GTM_gtm_cls0.DPLL_RR2[1735].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21736 GTM_gtm_cls0.DPLL_RR2[1736].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21737 GTM_gtm_cls0.DPLL_RR2[1737].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21738 GTM_gtm_cls0.DPLL_RR2[1738].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21739 GTM_gtm_cls0.DPLL_RR2[1739].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21740 GTM_gtm_cls0.DPLL_RR2[1740].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21741 GTM_gtm_cls0.DPLL_RR2[1741].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21742 GTM_gtm_cls0.DPLL_RR2[1742].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21743 GTM_gtm_cls0.DPLL_RR2[1743].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21744 GTM_gtm_cls0.DPLL_RR2[1744].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21745 GTM_gtm_cls0.DPLL_RR2[1745].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21746 GTM_gtm_cls0.DPLL_RR2[1746].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21747 GTM_gtm_cls0.DPLL_RR2[1747].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21748 GTM_gtm_cls0.DPLL_RR2[1748].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21749 GTM_gtm_cls0.DPLL_RR2[1749].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21750 GTM_gtm_cls0.DPLL_RR2[1750].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21751 GTM_gtm_cls0.DPLL_RR2[1751].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21752 GTM_gtm_cls0.DPLL_RR2[1752].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21753 GTM_gtm_cls0.DPLL_RR2[1753].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21754 GTM_gtm_cls0.DPLL_RR2[1754].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21755 GTM_gtm_cls0.DPLL_RR2[1755].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21756 GTM_gtm_cls0.DPLL_RR2[1756].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21757 GTM_gtm_cls0.DPLL_RR2[1757].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21758 GTM_gtm_cls0.DPLL_RR2[1758].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21759 GTM_gtm_cls0.DPLL_RR2[1759].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21760 GTM_gtm_cls0.DPLL_RR2[1760].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21761 GTM_gtm_cls0.DPLL_RR2[1761].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21762 GTM_gtm_cls0.DPLL_RR2[1762].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21763 GTM_gtm_cls0.DPLL_RR2[1763].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21764 GTM_gtm_cls0.DPLL_RR2[1764].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21765 GTM_gtm_cls0.DPLL_RR2[1765].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21766 GTM_gtm_cls0.DPLL_RR2[1766].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21767 GTM_gtm_cls0.DPLL_RR2[1767].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21768 GTM_gtm_cls0.DPLL_RR2[1768].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21769 GTM_gtm_cls0.DPLL_RR2[1769].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21770 GTM_gtm_cls0.DPLL_RR2[1770].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21771 GTM_gtm_cls0.DPLL_RR2[1771].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21772 GTM_gtm_cls0.DPLL_RR2[1772].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21773 GTM_gtm_cls0.DPLL_RR2[1773].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21774 GTM_gtm_cls0.DPLL_RR2[1774].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21775 GTM_gtm_cls0.DPLL_RR2[1775].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21776 GTM_gtm_cls0.DPLL_RR2[1776].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21777 GTM_gtm_cls0.DPLL_RR2[1777].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21778 GTM_gtm_cls0.DPLL_RR2[1778].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21779 GTM_gtm_cls0.DPLL_RR2[1779].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21780 GTM_gtm_cls0.DPLL_RR2[1780].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21781 GTM_gtm_cls0.DPLL_RR2[1781].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21782 GTM_gtm_cls0.DPLL_RR2[1782].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21783 GTM_gtm_cls0.DPLL_RR2[1783].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21784 GTM_gtm_cls0.DPLL_RR2[1784].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21785 GTM_gtm_cls0.DPLL_RR2[1785].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21786 GTM_gtm_cls0.DPLL_RR2[1786].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21787 GTM_gtm_cls0.DPLL_RR2[1787].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21788 GTM_gtm_cls0.DPLL_RR2[1788].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21789 GTM_gtm_cls0.DPLL_RR2[1789].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21790 GTM_gtm_cls0.DPLL_RR2[1790].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21791 GTM_gtm_cls0.DPLL_RR2[1791].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21792 GTM_gtm_cls0.DPLL_RR2[1792].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21793 GTM_gtm_cls0.DPLL_RR2[1793].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21794 GTM_gtm_cls0.DPLL_RR2[1794].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21795 GTM_gtm_cls0.DPLL_RR2[1795].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21796 GTM_gtm_cls0.DPLL_RR2[1796].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21797 GTM_gtm_cls0.DPLL_RR2[1797].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21798 GTM_gtm_cls0.DPLL_RR2[1798].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21799 GTM_gtm_cls0.DPLL_RR2[1799].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21800 GTM_gtm_cls0.DPLL_RR2[1800].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21801 GTM_gtm_cls0.DPLL_RR2[1801].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21802 GTM_gtm_cls0.DPLL_RR2[1802].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21803 GTM_gtm_cls0.DPLL_RR2[1803].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21804 GTM_gtm_cls0.DPLL_RR2[1804].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21805 GTM_gtm_cls0.DPLL_RR2[1805].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21806 GTM_gtm_cls0.DPLL_RR2[1806].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21807 GTM_gtm_cls0.DPLL_RR2[1807].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21808 GTM_gtm_cls0.DPLL_RR2[1808].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21809 GTM_gtm_cls0.DPLL_RR2[1809].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21810 GTM_gtm_cls0.DPLL_RR2[1810].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21811 GTM_gtm_cls0.DPLL_RR2[1811].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21812 GTM_gtm_cls0.DPLL_RR2[1812].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21813 GTM_gtm_cls0.DPLL_RR2[1813].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21814 GTM_gtm_cls0.DPLL_RR2[1814].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21815 GTM_gtm_cls0.DPLL_RR2[1815].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21816 GTM_gtm_cls0.DPLL_RR2[1816].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21817 GTM_gtm_cls0.DPLL_RR2[1817].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21818 GTM_gtm_cls0.DPLL_RR2[1818].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21819 GTM_gtm_cls0.DPLL_RR2[1819].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21820 GTM_gtm_cls0.DPLL_RR2[1820].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21821 GTM_gtm_cls0.DPLL_RR2[1821].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21822 GTM_gtm_cls0.DPLL_RR2[1822].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21823 GTM_gtm_cls0.DPLL_RR2[1823].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21824 GTM_gtm_cls0.DPLL_RR2[1824].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21825 GTM_gtm_cls0.DPLL_RR2[1825].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21826 GTM_gtm_cls0.DPLL_RR2[1826].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21827 GTM_gtm_cls0.DPLL_RR2[1827].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21828 GTM_gtm_cls0.DPLL_RR2[1828].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21829 GTM_gtm_cls0.DPLL_RR2[1829].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21830 GTM_gtm_cls0.DPLL_RR2[1830].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21831 GTM_gtm_cls0.DPLL_RR2[1831].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21832 GTM_gtm_cls0.DPLL_RR2[1832].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21833 GTM_gtm_cls0.DPLL_RR2[1833].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21834 GTM_gtm_cls0.DPLL_RR2[1834].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21835 GTM_gtm_cls0.DPLL_RR2[1835].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21836 GTM_gtm_cls0.DPLL_RR2[1836].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21837 GTM_gtm_cls0.DPLL_RR2[1837].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21838 GTM_gtm_cls0.DPLL_RR2[1838].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21839 GTM_gtm_cls0.DPLL_RR2[1839].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21840 GTM_gtm_cls0.DPLL_RR2[1840].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21841 GTM_gtm_cls0.DPLL_RR2[1841].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21842 GTM_gtm_cls0.DPLL_RR2[1842].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21843 GTM_gtm_cls0.DPLL_RR2[1843].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21844 GTM_gtm_cls0.DPLL_RR2[1844].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21845 GTM_gtm_cls0.DPLL_RR2[1845].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21846 GTM_gtm_cls0.DPLL_RR2[1846].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21847 GTM_gtm_cls0.DPLL_RR2[1847].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21848 GTM_gtm_cls0.DPLL_RR2[1848].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21849 GTM_gtm_cls0.DPLL_RR2[1849].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21850 GTM_gtm_cls0.DPLL_RR2[1850].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21851 GTM_gtm_cls0.DPLL_RR2[1851].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21852 GTM_gtm_cls0.DPLL_RR2[1852].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21853 GTM_gtm_cls0.DPLL_RR2[1853].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21854 GTM_gtm_cls0.DPLL_RR2[1854].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21855 GTM_gtm_cls0.DPLL_RR2[1855].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21856 GTM_gtm_cls0.DPLL_RR2[1856].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21857 GTM_gtm_cls0.DPLL_RR2[1857].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21858 GTM_gtm_cls0.DPLL_RR2[1858].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21859 GTM_gtm_cls0.DPLL_RR2[1859].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21860 GTM_gtm_cls0.DPLL_RR2[1860].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21861 GTM_gtm_cls0.DPLL_RR2[1861].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21862 GTM_gtm_cls0.DPLL_RR2[1862].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21863 GTM_gtm_cls0.DPLL_RR2[1863].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21864 GTM_gtm_cls0.DPLL_RR2[1864].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21865 GTM_gtm_cls0.DPLL_RR2[1865].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21866 GTM_gtm_cls0.DPLL_RR2[1866].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21867 GTM_gtm_cls0.DPLL_RR2[1867].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21868 GTM_gtm_cls0.DPLL_RR2[1868].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21869 GTM_gtm_cls0.DPLL_RR2[1869].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21870 GTM_gtm_cls0.DPLL_RR2[1870].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21871 GTM_gtm_cls0.DPLL_RR2[1871].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21872 GTM_gtm_cls0.DPLL_RR2[1872].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21873 GTM_gtm_cls0.DPLL_RR2[1873].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21874 GTM_gtm_cls0.DPLL_RR2[1874].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21875 GTM_gtm_cls0.DPLL_RR2[1875].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21876 GTM_gtm_cls0.DPLL_RR2[1876].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21877 GTM_gtm_cls0.DPLL_RR2[1877].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21878 GTM_gtm_cls0.DPLL_RR2[1878].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21879 GTM_gtm_cls0.DPLL_RR2[1879].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21880 GTM_gtm_cls0.DPLL_RR2[1880].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21881 GTM_gtm_cls0.DPLL_RR2[1881].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21882 GTM_gtm_cls0.DPLL_RR2[1882].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21883 GTM_gtm_cls0.DPLL_RR2[1883].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21884 GTM_gtm_cls0.DPLL_RR2[1884].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21885 GTM_gtm_cls0.DPLL_RR2[1885].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21886 GTM_gtm_cls0.DPLL_RR2[1886].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21887 GTM_gtm_cls0.DPLL_RR2[1887].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21888 GTM_gtm_cls0.DPLL_RR2[1888].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21889 GTM_gtm_cls0.DPLL_RR2[1889].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21890 GTM_gtm_cls0.DPLL_RR2[1890].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21891 GTM_gtm_cls0.DPLL_RR2[1891].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21892 GTM_gtm_cls0.DPLL_RR2[1892].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21893 GTM_gtm_cls0.DPLL_RR2[1893].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21894 GTM_gtm_cls0.DPLL_RR2[1894].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21895 GTM_gtm_cls0.DPLL_RR2[1895].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21896 GTM_gtm_cls0.DPLL_RR2[1896].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21897 GTM_gtm_cls0.DPLL_RR2[1897].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21898 GTM_gtm_cls0.DPLL_RR2[1898].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21899 GTM_gtm_cls0.DPLL_RR2[1899].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21900 GTM_gtm_cls0.DPLL_RR2[1900].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21901 GTM_gtm_cls0.DPLL_RR2[1901].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21902 GTM_gtm_cls0.DPLL_RR2[1902].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21903 GTM_gtm_cls0.DPLL_RR2[1903].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21904 GTM_gtm_cls0.DPLL_RR2[1904].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21905 GTM_gtm_cls0.DPLL_RR2[1905].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21906 GTM_gtm_cls0.DPLL_RR2[1906].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21907 GTM_gtm_cls0.DPLL_RR2[1907].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21908 GTM_gtm_cls0.DPLL_RR2[1908].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21909 GTM_gtm_cls0.DPLL_RR2[1909].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21910 GTM_gtm_cls0.DPLL_RR2[1910].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21911 GTM_gtm_cls0.DPLL_RR2[1911].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21912 GTM_gtm_cls0.DPLL_RR2[1912].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21913 GTM_gtm_cls0.DPLL_RR2[1913].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21914 GTM_gtm_cls0.DPLL_RR2[1914].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21915 GTM_gtm_cls0.DPLL_RR2[1915].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21916 GTM_gtm_cls0.DPLL_RR2[1916].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21917 GTM_gtm_cls0.DPLL_RR2[1917].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21918 GTM_gtm_cls0.DPLL_RR2[1918].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21919 GTM_gtm_cls0.DPLL_RR2[1919].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21920 GTM_gtm_cls0.DPLL_RR2[1920].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21921 GTM_gtm_cls0.DPLL_RR2[1921].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21922 GTM_gtm_cls0.DPLL_RR2[1922].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21923 GTM_gtm_cls0.DPLL_RR2[1923].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21924 GTM_gtm_cls0.DPLL_RR2[1924].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21925 GTM_gtm_cls0.DPLL_RR2[1925].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21926 GTM_gtm_cls0.DPLL_RR2[1926].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21927 GTM_gtm_cls0.DPLL_RR2[1927].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21928 GTM_gtm_cls0.DPLL_RR2[1928].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21929 GTM_gtm_cls0.DPLL_RR2[1929].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21930 GTM_gtm_cls0.DPLL_RR2[1930].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21931 GTM_gtm_cls0.DPLL_RR2[1931].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21932 GTM_gtm_cls0.DPLL_RR2[1932].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21933 GTM_gtm_cls0.DPLL_RR2[1933].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21934 GTM_gtm_cls0.DPLL_RR2[1934].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21935 GTM_gtm_cls0.DPLL_RR2[1935].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21936 GTM_gtm_cls0.DPLL_RR2[1936].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21937 GTM_gtm_cls0.DPLL_RR2[1937].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21938 GTM_gtm_cls0.DPLL_RR2[1938].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21939 GTM_gtm_cls0.DPLL_RR2[1939].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21940 GTM_gtm_cls0.DPLL_RR2[1940].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21941 GTM_gtm_cls0.DPLL_RR2[1941].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21942 GTM_gtm_cls0.DPLL_RR2[1942].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21943 GTM_gtm_cls0.DPLL_RR2[1943].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21944 GTM_gtm_cls0.DPLL_RR2[1944].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21945 GTM_gtm_cls0.DPLL_RR2[1945].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21946 GTM_gtm_cls0.DPLL_RR2[1946].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21947 GTM_gtm_cls0.DPLL_RR2[1947].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21948 GTM_gtm_cls0.DPLL_RR2[1948].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21949 GTM_gtm_cls0.DPLL_RR2[1949].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21950 GTM_gtm_cls0.DPLL_RR2[1950].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21951 GTM_gtm_cls0.DPLL_RR2[1951].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21952 GTM_gtm_cls0.DPLL_RR2[1952].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21953 GTM_gtm_cls0.DPLL_RR2[1953].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21954 GTM_gtm_cls0.DPLL_RR2[1954].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21955 GTM_gtm_cls0.DPLL_RR2[1955].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21956 GTM_gtm_cls0.DPLL_RR2[1956].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21957 GTM_gtm_cls0.DPLL_RR2[1957].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21958 GTM_gtm_cls0.DPLL_RR2[1958].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21959 GTM_gtm_cls0.DPLL_RR2[1959].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21960 GTM_gtm_cls0.DPLL_RR2[1960].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21961 GTM_gtm_cls0.DPLL_RR2[1961].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21962 GTM_gtm_cls0.DPLL_RR2[1962].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21963 GTM_gtm_cls0.DPLL_RR2[1963].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21964 GTM_gtm_cls0.DPLL_RR2[1964].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21965 GTM_gtm_cls0.DPLL_RR2[1965].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21966 GTM_gtm_cls0.DPLL_RR2[1966].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21967 GTM_gtm_cls0.DPLL_RR2[1967].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21968 GTM_gtm_cls0.DPLL_RR2[1968].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21969 GTM_gtm_cls0.DPLL_RR2[1969].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21970 GTM_gtm_cls0.DPLL_RR2[1970].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21971 GTM_gtm_cls0.DPLL_RR2[1971].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21972 GTM_gtm_cls0.DPLL_RR2[1972].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21973 GTM_gtm_cls0.DPLL_RR2[1973].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21974 GTM_gtm_cls0.DPLL_RR2[1974].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21975 GTM_gtm_cls0.DPLL_RR2[1975].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21976 GTM_gtm_cls0.DPLL_RR2[1976].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21977 GTM_gtm_cls0.DPLL_RR2[1977].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21978 GTM_gtm_cls0.DPLL_RR2[1978].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21979 GTM_gtm_cls0.DPLL_RR2[1979].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21980 GTM_gtm_cls0.DPLL_RR2[1980].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21981 GTM_gtm_cls0.DPLL_RR2[1981].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21982 GTM_gtm_cls0.DPLL_RR2[1982].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21983 GTM_gtm_cls0.DPLL_RR2[1983].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21984 GTM_gtm_cls0.DPLL_RR2[1984].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21985 GTM_gtm_cls0.DPLL_RR2[1985].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21986 GTM_gtm_cls0.DPLL_RR2[1986].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21987 GTM_gtm_cls0.DPLL_RR2[1987].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21988 GTM_gtm_cls0.DPLL_RR2[1988].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21989 GTM_gtm_cls0.DPLL_RR2[1989].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21990 GTM_gtm_cls0.DPLL_RR2[1990].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21991 GTM_gtm_cls0.DPLL_RR2[1991].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21992 GTM_gtm_cls0.DPLL_RR2[1992].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21993 GTM_gtm_cls0.DPLL_RR2[1993].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21994 GTM_gtm_cls0.DPLL_RR2[1994].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21995 GTM_gtm_cls0.DPLL_RR2[1995].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21996 GTM_gtm_cls0.DPLL_RR2[1996].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21997 GTM_gtm_cls0.DPLL_RR2[1997].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21998 GTM_gtm_cls0.DPLL_RR2[1998].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR21999 GTM_gtm_cls0.DPLL_RR2[1999].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22000 GTM_gtm_cls0.DPLL_RR2[2000].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22001 GTM_gtm_cls0.DPLL_RR2[2001].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22002 GTM_gtm_cls0.DPLL_RR2[2002].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22003 GTM_gtm_cls0.DPLL_RR2[2003].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22004 GTM_gtm_cls0.DPLL_RR2[2004].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22005 GTM_gtm_cls0.DPLL_RR2[2005].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22006 GTM_gtm_cls0.DPLL_RR2[2006].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22007 GTM_gtm_cls0.DPLL_RR2[2007].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22008 GTM_gtm_cls0.DPLL_RR2[2008].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22009 GTM_gtm_cls0.DPLL_RR2[2009].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22010 GTM_gtm_cls0.DPLL_RR2[2010].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22011 GTM_gtm_cls0.DPLL_RR2[2011].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22012 GTM_gtm_cls0.DPLL_RR2[2012].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22013 GTM_gtm_cls0.DPLL_RR2[2013].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22014 GTM_gtm_cls0.DPLL_RR2[2014].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22015 GTM_gtm_cls0.DPLL_RR2[2015].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22016 GTM_gtm_cls0.DPLL_RR2[2016].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22017 GTM_gtm_cls0.DPLL_RR2[2017].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22018 GTM_gtm_cls0.DPLL_RR2[2018].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22019 GTM_gtm_cls0.DPLL_RR2[2019].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22020 GTM_gtm_cls0.DPLL_RR2[2020].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22021 GTM_gtm_cls0.DPLL_RR2[2021].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22022 GTM_gtm_cls0.DPLL_RR2[2022].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22023 GTM_gtm_cls0.DPLL_RR2[2023].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22024 GTM_gtm_cls0.DPLL_RR2[2024].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22025 GTM_gtm_cls0.DPLL_RR2[2025].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22026 GTM_gtm_cls0.DPLL_RR2[2026].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22027 GTM_gtm_cls0.DPLL_RR2[2027].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22028 GTM_gtm_cls0.DPLL_RR2[2028].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22029 GTM_gtm_cls0.DPLL_RR2[2029].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22030 GTM_gtm_cls0.DPLL_RR2[2030].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22031 GTM_gtm_cls0.DPLL_RR2[2031].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22032 GTM_gtm_cls0.DPLL_RR2[2032].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22033 GTM_gtm_cls0.DPLL_RR2[2033].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22034 GTM_gtm_cls0.DPLL_RR2[2034].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22035 GTM_gtm_cls0.DPLL_RR2[2035].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22036 GTM_gtm_cls0.DPLL_RR2[2036].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22037 GTM_gtm_cls0.DPLL_RR2[2037].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22038 GTM_gtm_cls0.DPLL_RR2[2038].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22039 GTM_gtm_cls0.DPLL_RR2[2039].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22040 GTM_gtm_cls0.DPLL_RR2[2040].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22041 GTM_gtm_cls0.DPLL_RR2[2041].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22042 GTM_gtm_cls0.DPLL_RR2[2042].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22043 GTM_gtm_cls0.DPLL_RR2[2043].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22044 GTM_gtm_cls0.DPLL_RR2[2044].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22045 GTM_gtm_cls0.DPLL_RR2[2045].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22046 GTM_gtm_cls0.DPLL_RR2[2046].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22047 GTM_gtm_cls0.DPLL_RR2[2047].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22048 GTM_gtm_cls0.DPLL_RR2[2048].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22049 GTM_gtm_cls0.DPLL_RR2[2049].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22050 GTM_gtm_cls0.DPLL_RR2[2050].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22051 GTM_gtm_cls0.DPLL_RR2[2051].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22052 GTM_gtm_cls0.DPLL_RR2[2052].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22053 GTM_gtm_cls0.DPLL_RR2[2053].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22054 GTM_gtm_cls0.DPLL_RR2[2054].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22055 GTM_gtm_cls0.DPLL_RR2[2055].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22056 GTM_gtm_cls0.DPLL_RR2[2056].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22057 GTM_gtm_cls0.DPLL_RR2[2057].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22058 GTM_gtm_cls0.DPLL_RR2[2058].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22059 GTM_gtm_cls0.DPLL_RR2[2059].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22060 GTM_gtm_cls0.DPLL_RR2[2060].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22061 GTM_gtm_cls0.DPLL_RR2[2061].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22062 GTM_gtm_cls0.DPLL_RR2[2062].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22063 GTM_gtm_cls0.DPLL_RR2[2063].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22064 GTM_gtm_cls0.DPLL_RR2[2064].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22065 GTM_gtm_cls0.DPLL_RR2[2065].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22066 GTM_gtm_cls0.DPLL_RR2[2066].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22067 GTM_gtm_cls0.DPLL_RR2[2067].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22068 GTM_gtm_cls0.DPLL_RR2[2068].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22069 GTM_gtm_cls0.DPLL_RR2[2069].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22070 GTM_gtm_cls0.DPLL_RR2[2070].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22071 GTM_gtm_cls0.DPLL_RR2[2071].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22072 GTM_gtm_cls0.DPLL_RR2[2072].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22073 GTM_gtm_cls0.DPLL_RR2[2073].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22074 GTM_gtm_cls0.DPLL_RR2[2074].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22075 GTM_gtm_cls0.DPLL_RR2[2075].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22076 GTM_gtm_cls0.DPLL_RR2[2076].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22077 GTM_gtm_cls0.DPLL_RR2[2077].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22078 GTM_gtm_cls0.DPLL_RR2[2078].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22079 GTM_gtm_cls0.DPLL_RR2[2079].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22080 GTM_gtm_cls0.DPLL_RR2[2080].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22081 GTM_gtm_cls0.DPLL_RR2[2081].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22082 GTM_gtm_cls0.DPLL_RR2[2082].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22083 GTM_gtm_cls0.DPLL_RR2[2083].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22084 GTM_gtm_cls0.DPLL_RR2[2084].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22085 GTM_gtm_cls0.DPLL_RR2[2085].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22086 GTM_gtm_cls0.DPLL_RR2[2086].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22087 GTM_gtm_cls0.DPLL_RR2[2087].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22088 GTM_gtm_cls0.DPLL_RR2[2088].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22089 GTM_gtm_cls0.DPLL_RR2[2089].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22090 GTM_gtm_cls0.DPLL_RR2[2090].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22091 GTM_gtm_cls0.DPLL_RR2[2091].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22092 GTM_gtm_cls0.DPLL_RR2[2092].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22093 GTM_gtm_cls0.DPLL_RR2[2093].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22094 GTM_gtm_cls0.DPLL_RR2[2094].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22095 GTM_gtm_cls0.DPLL_RR2[2095].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22096 GTM_gtm_cls0.DPLL_RR2[2096].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22097 GTM_gtm_cls0.DPLL_RR2[2097].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22098 GTM_gtm_cls0.DPLL_RR2[2098].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22099 GTM_gtm_cls0.DPLL_RR2[2099].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22100 GTM_gtm_cls0.DPLL_RR2[2100].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22101 GTM_gtm_cls0.DPLL_RR2[2101].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22102 GTM_gtm_cls0.DPLL_RR2[2102].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22103 GTM_gtm_cls0.DPLL_RR2[2103].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22104 GTM_gtm_cls0.DPLL_RR2[2104].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22105 GTM_gtm_cls0.DPLL_RR2[2105].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22106 GTM_gtm_cls0.DPLL_RR2[2106].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22107 GTM_gtm_cls0.DPLL_RR2[2107].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22108 GTM_gtm_cls0.DPLL_RR2[2108].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22109 GTM_gtm_cls0.DPLL_RR2[2109].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22110 GTM_gtm_cls0.DPLL_RR2[2110].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22111 GTM_gtm_cls0.DPLL_RR2[2111].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22112 GTM_gtm_cls0.DPLL_RR2[2112].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22113 GTM_gtm_cls0.DPLL_RR2[2113].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22114 GTM_gtm_cls0.DPLL_RR2[2114].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22115 GTM_gtm_cls0.DPLL_RR2[2115].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22116 GTM_gtm_cls0.DPLL_RR2[2116].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22117 GTM_gtm_cls0.DPLL_RR2[2117].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22118 GTM_gtm_cls0.DPLL_RR2[2118].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22119 GTM_gtm_cls0.DPLL_RR2[2119].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22120 GTM_gtm_cls0.DPLL_RR2[2120].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22121 GTM_gtm_cls0.DPLL_RR2[2121].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22122 GTM_gtm_cls0.DPLL_RR2[2122].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22123 GTM_gtm_cls0.DPLL_RR2[2123].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22124 GTM_gtm_cls0.DPLL_RR2[2124].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22125 GTM_gtm_cls0.DPLL_RR2[2125].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22126 GTM_gtm_cls0.DPLL_RR2[2126].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22127 GTM_gtm_cls0.DPLL_RR2[2127].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22128 GTM_gtm_cls0.DPLL_RR2[2128].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22129 GTM_gtm_cls0.DPLL_RR2[2129].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22130 GTM_gtm_cls0.DPLL_RR2[2130].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22131 GTM_gtm_cls0.DPLL_RR2[2131].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22132 GTM_gtm_cls0.DPLL_RR2[2132].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22133 GTM_gtm_cls0.DPLL_RR2[2133].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22134 GTM_gtm_cls0.DPLL_RR2[2134].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22135 GTM_gtm_cls0.DPLL_RR2[2135].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22136 GTM_gtm_cls0.DPLL_RR2[2136].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22137 GTM_gtm_cls0.DPLL_RR2[2137].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22138 GTM_gtm_cls0.DPLL_RR2[2138].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22139 GTM_gtm_cls0.DPLL_RR2[2139].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22140 GTM_gtm_cls0.DPLL_RR2[2140].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22141 GTM_gtm_cls0.DPLL_RR2[2141].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22142 GTM_gtm_cls0.DPLL_RR2[2142].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22143 GTM_gtm_cls0.DPLL_RR2[2143].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22144 GTM_gtm_cls0.DPLL_RR2[2144].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22145 GTM_gtm_cls0.DPLL_RR2[2145].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22146 GTM_gtm_cls0.DPLL_RR2[2146].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22147 GTM_gtm_cls0.DPLL_RR2[2147].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22148 GTM_gtm_cls0.DPLL_RR2[2148].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22149 GTM_gtm_cls0.DPLL_RR2[2149].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22150 GTM_gtm_cls0.DPLL_RR2[2150].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22151 GTM_gtm_cls0.DPLL_RR2[2151].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22152 GTM_gtm_cls0.DPLL_RR2[2152].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22153 GTM_gtm_cls0.DPLL_RR2[2153].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22154 GTM_gtm_cls0.DPLL_RR2[2154].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22155 GTM_gtm_cls0.DPLL_RR2[2155].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22156 GTM_gtm_cls0.DPLL_RR2[2156].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22157 GTM_gtm_cls0.DPLL_RR2[2157].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22158 GTM_gtm_cls0.DPLL_RR2[2158].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22159 GTM_gtm_cls0.DPLL_RR2[2159].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22160 GTM_gtm_cls0.DPLL_RR2[2160].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22161 GTM_gtm_cls0.DPLL_RR2[2161].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22162 GTM_gtm_cls0.DPLL_RR2[2162].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22163 GTM_gtm_cls0.DPLL_RR2[2163].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22164 GTM_gtm_cls0.DPLL_RR2[2164].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22165 GTM_gtm_cls0.DPLL_RR2[2165].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22166 GTM_gtm_cls0.DPLL_RR2[2166].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22167 GTM_gtm_cls0.DPLL_RR2[2167].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22168 GTM_gtm_cls0.DPLL_RR2[2168].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22169 GTM_gtm_cls0.DPLL_RR2[2169].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22170 GTM_gtm_cls0.DPLL_RR2[2170].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22171 GTM_gtm_cls0.DPLL_RR2[2171].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22172 GTM_gtm_cls0.DPLL_RR2[2172].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22173 GTM_gtm_cls0.DPLL_RR2[2173].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22174 GTM_gtm_cls0.DPLL_RR2[2174].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22175 GTM_gtm_cls0.DPLL_RR2[2175].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22176 GTM_gtm_cls0.DPLL_RR2[2176].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22177 GTM_gtm_cls0.DPLL_RR2[2177].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22178 GTM_gtm_cls0.DPLL_RR2[2178].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22179 GTM_gtm_cls0.DPLL_RR2[2179].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22180 GTM_gtm_cls0.DPLL_RR2[2180].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22181 GTM_gtm_cls0.DPLL_RR2[2181].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22182 GTM_gtm_cls0.DPLL_RR2[2182].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22183 GTM_gtm_cls0.DPLL_RR2[2183].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22184 GTM_gtm_cls0.DPLL_RR2[2184].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22185 GTM_gtm_cls0.DPLL_RR2[2185].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22186 GTM_gtm_cls0.DPLL_RR2[2186].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22187 GTM_gtm_cls0.DPLL_RR2[2187].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22188 GTM_gtm_cls0.DPLL_RR2[2188].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22189 GTM_gtm_cls0.DPLL_RR2[2189].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22190 GTM_gtm_cls0.DPLL_RR2[2190].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22191 GTM_gtm_cls0.DPLL_RR2[2191].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22192 GTM_gtm_cls0.DPLL_RR2[2192].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22193 GTM_gtm_cls0.DPLL_RR2[2193].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22194 GTM_gtm_cls0.DPLL_RR2[2194].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22195 GTM_gtm_cls0.DPLL_RR2[2195].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22196 GTM_gtm_cls0.DPLL_RR2[2196].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22197 GTM_gtm_cls0.DPLL_RR2[2197].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22198 GTM_gtm_cls0.DPLL_RR2[2198].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22199 GTM_gtm_cls0.DPLL_RR2[2199].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22200 GTM_gtm_cls0.DPLL_RR2[2200].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22201 GTM_gtm_cls0.DPLL_RR2[2201].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22202 GTM_gtm_cls0.DPLL_RR2[2202].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22203 GTM_gtm_cls0.DPLL_RR2[2203].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22204 GTM_gtm_cls0.DPLL_RR2[2204].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22205 GTM_gtm_cls0.DPLL_RR2[2205].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22206 GTM_gtm_cls0.DPLL_RR2[2206].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22207 GTM_gtm_cls0.DPLL_RR2[2207].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22208 GTM_gtm_cls0.DPLL_RR2[2208].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22209 GTM_gtm_cls0.DPLL_RR2[2209].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22210 GTM_gtm_cls0.DPLL_RR2[2210].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22211 GTM_gtm_cls0.DPLL_RR2[2211].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22212 GTM_gtm_cls0.DPLL_RR2[2212].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22213 GTM_gtm_cls0.DPLL_RR2[2213].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22214 GTM_gtm_cls0.DPLL_RR2[2214].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22215 GTM_gtm_cls0.DPLL_RR2[2215].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22216 GTM_gtm_cls0.DPLL_RR2[2216].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22217 GTM_gtm_cls0.DPLL_RR2[2217].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22218 GTM_gtm_cls0.DPLL_RR2[2218].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22219 GTM_gtm_cls0.DPLL_RR2[2219].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22220 GTM_gtm_cls0.DPLL_RR2[2220].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22221 GTM_gtm_cls0.DPLL_RR2[2221].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22222 GTM_gtm_cls0.DPLL_RR2[2222].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22223 GTM_gtm_cls0.DPLL_RR2[2223].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22224 GTM_gtm_cls0.DPLL_RR2[2224].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22225 GTM_gtm_cls0.DPLL_RR2[2225].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22226 GTM_gtm_cls0.DPLL_RR2[2226].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22227 GTM_gtm_cls0.DPLL_RR2[2227].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22228 GTM_gtm_cls0.DPLL_RR2[2228].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22229 GTM_gtm_cls0.DPLL_RR2[2229].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22230 GTM_gtm_cls0.DPLL_RR2[2230].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22231 GTM_gtm_cls0.DPLL_RR2[2231].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22232 GTM_gtm_cls0.DPLL_RR2[2232].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22233 GTM_gtm_cls0.DPLL_RR2[2233].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22234 GTM_gtm_cls0.DPLL_RR2[2234].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22235 GTM_gtm_cls0.DPLL_RR2[2235].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22236 GTM_gtm_cls0.DPLL_RR2[2236].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22237 GTM_gtm_cls0.DPLL_RR2[2237].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22238 GTM_gtm_cls0.DPLL_RR2[2238].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22239 GTM_gtm_cls0.DPLL_RR2[2239].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22240 GTM_gtm_cls0.DPLL_RR2[2240].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22241 GTM_gtm_cls0.DPLL_RR2[2241].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22242 GTM_gtm_cls0.DPLL_RR2[2242].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22243 GTM_gtm_cls0.DPLL_RR2[2243].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22244 GTM_gtm_cls0.DPLL_RR2[2244].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22245 GTM_gtm_cls0.DPLL_RR2[2245].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22246 GTM_gtm_cls0.DPLL_RR2[2246].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22247 GTM_gtm_cls0.DPLL_RR2[2247].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22248 GTM_gtm_cls0.DPLL_RR2[2248].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22249 GTM_gtm_cls0.DPLL_RR2[2249].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22250 GTM_gtm_cls0.DPLL_RR2[2250].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22251 GTM_gtm_cls0.DPLL_RR2[2251].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22252 GTM_gtm_cls0.DPLL_RR2[2252].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22253 GTM_gtm_cls0.DPLL_RR2[2253].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22254 GTM_gtm_cls0.DPLL_RR2[2254].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22255 GTM_gtm_cls0.DPLL_RR2[2255].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22256 GTM_gtm_cls0.DPLL_RR2[2256].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22257 GTM_gtm_cls0.DPLL_RR2[2257].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22258 GTM_gtm_cls0.DPLL_RR2[2258].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22259 GTM_gtm_cls0.DPLL_RR2[2259].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22260 GTM_gtm_cls0.DPLL_RR2[2260].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22261 GTM_gtm_cls0.DPLL_RR2[2261].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22262 GTM_gtm_cls0.DPLL_RR2[2262].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22263 GTM_gtm_cls0.DPLL_RR2[2263].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22264 GTM_gtm_cls0.DPLL_RR2[2264].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22265 GTM_gtm_cls0.DPLL_RR2[2265].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22266 GTM_gtm_cls0.DPLL_RR2[2266].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22267 GTM_gtm_cls0.DPLL_RR2[2267].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22268 GTM_gtm_cls0.DPLL_RR2[2268].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22269 GTM_gtm_cls0.DPLL_RR2[2269].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22270 GTM_gtm_cls0.DPLL_RR2[2270].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22271 GTM_gtm_cls0.DPLL_RR2[2271].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22272 GTM_gtm_cls0.DPLL_RR2[2272].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22273 GTM_gtm_cls0.DPLL_RR2[2273].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22274 GTM_gtm_cls0.DPLL_RR2[2274].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22275 GTM_gtm_cls0.DPLL_RR2[2275].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22276 GTM_gtm_cls0.DPLL_RR2[2276].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22277 GTM_gtm_cls0.DPLL_RR2[2277].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22278 GTM_gtm_cls0.DPLL_RR2[2278].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22279 GTM_gtm_cls0.DPLL_RR2[2279].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22280 GTM_gtm_cls0.DPLL_RR2[2280].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22281 GTM_gtm_cls0.DPLL_RR2[2281].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22282 GTM_gtm_cls0.DPLL_RR2[2282].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22283 GTM_gtm_cls0.DPLL_RR2[2283].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22284 GTM_gtm_cls0.DPLL_RR2[2284].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22285 GTM_gtm_cls0.DPLL_RR2[2285].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22286 GTM_gtm_cls0.DPLL_RR2[2286].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22287 GTM_gtm_cls0.DPLL_RR2[2287].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22288 GTM_gtm_cls0.DPLL_RR2[2288].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22289 GTM_gtm_cls0.DPLL_RR2[2289].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22290 GTM_gtm_cls0.DPLL_RR2[2290].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22291 GTM_gtm_cls0.DPLL_RR2[2291].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22292 GTM_gtm_cls0.DPLL_RR2[2292].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22293 GTM_gtm_cls0.DPLL_RR2[2293].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22294 GTM_gtm_cls0.DPLL_RR2[2294].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22295 GTM_gtm_cls0.DPLL_RR2[2295].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22296 GTM_gtm_cls0.DPLL_RR2[2296].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22297 GTM_gtm_cls0.DPLL_RR2[2297].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22298 GTM_gtm_cls0.DPLL_RR2[2298].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22299 GTM_gtm_cls0.DPLL_RR2[2299].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22300 GTM_gtm_cls0.DPLL_RR2[2300].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22301 GTM_gtm_cls0.DPLL_RR2[2301].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22302 GTM_gtm_cls0.DPLL_RR2[2302].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22303 GTM_gtm_cls0.DPLL_RR2[2303].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22304 GTM_gtm_cls0.DPLL_RR2[2304].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22305 GTM_gtm_cls0.DPLL_RR2[2305].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22306 GTM_gtm_cls0.DPLL_RR2[2306].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22307 GTM_gtm_cls0.DPLL_RR2[2307].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22308 GTM_gtm_cls0.DPLL_RR2[2308].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22309 GTM_gtm_cls0.DPLL_RR2[2309].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22310 GTM_gtm_cls0.DPLL_RR2[2310].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22311 GTM_gtm_cls0.DPLL_RR2[2311].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22312 GTM_gtm_cls0.DPLL_RR2[2312].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22313 GTM_gtm_cls0.DPLL_RR2[2313].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22314 GTM_gtm_cls0.DPLL_RR2[2314].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22315 GTM_gtm_cls0.DPLL_RR2[2315].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22316 GTM_gtm_cls0.DPLL_RR2[2316].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22317 GTM_gtm_cls0.DPLL_RR2[2317].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22318 GTM_gtm_cls0.DPLL_RR2[2318].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22319 GTM_gtm_cls0.DPLL_RR2[2319].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22320 GTM_gtm_cls0.DPLL_RR2[2320].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22321 GTM_gtm_cls0.DPLL_RR2[2321].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22322 GTM_gtm_cls0.DPLL_RR2[2322].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22323 GTM_gtm_cls0.DPLL_RR2[2323].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22324 GTM_gtm_cls0.DPLL_RR2[2324].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22325 GTM_gtm_cls0.DPLL_RR2[2325].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22326 GTM_gtm_cls0.DPLL_RR2[2326].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22327 GTM_gtm_cls0.DPLL_RR2[2327].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22328 GTM_gtm_cls0.DPLL_RR2[2328].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22329 GTM_gtm_cls0.DPLL_RR2[2329].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22330 GTM_gtm_cls0.DPLL_RR2[2330].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22331 GTM_gtm_cls0.DPLL_RR2[2331].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22332 GTM_gtm_cls0.DPLL_RR2[2332].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22333 GTM_gtm_cls0.DPLL_RR2[2333].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22334 GTM_gtm_cls0.DPLL_RR2[2334].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22335 GTM_gtm_cls0.DPLL_RR2[2335].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22336 GTM_gtm_cls0.DPLL_RR2[2336].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22337 GTM_gtm_cls0.DPLL_RR2[2337].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22338 GTM_gtm_cls0.DPLL_RR2[2338].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22339 GTM_gtm_cls0.DPLL_RR2[2339].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22340 GTM_gtm_cls0.DPLL_RR2[2340].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22341 GTM_gtm_cls0.DPLL_RR2[2341].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22342 GTM_gtm_cls0.DPLL_RR2[2342].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22343 GTM_gtm_cls0.DPLL_RR2[2343].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22344 GTM_gtm_cls0.DPLL_RR2[2344].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22345 GTM_gtm_cls0.DPLL_RR2[2345].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22346 GTM_gtm_cls0.DPLL_RR2[2346].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22347 GTM_gtm_cls0.DPLL_RR2[2347].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22348 GTM_gtm_cls0.DPLL_RR2[2348].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22349 GTM_gtm_cls0.DPLL_RR2[2349].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22350 GTM_gtm_cls0.DPLL_RR2[2350].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22351 GTM_gtm_cls0.DPLL_RR2[2351].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22352 GTM_gtm_cls0.DPLL_RR2[2352].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22353 GTM_gtm_cls0.DPLL_RR2[2353].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22354 GTM_gtm_cls0.DPLL_RR2[2354].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22355 GTM_gtm_cls0.DPLL_RR2[2355].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22356 GTM_gtm_cls0.DPLL_RR2[2356].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22357 GTM_gtm_cls0.DPLL_RR2[2357].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22358 GTM_gtm_cls0.DPLL_RR2[2358].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22359 GTM_gtm_cls0.DPLL_RR2[2359].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22360 GTM_gtm_cls0.DPLL_RR2[2360].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22361 GTM_gtm_cls0.DPLL_RR2[2361].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22362 GTM_gtm_cls0.DPLL_RR2[2362].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22363 GTM_gtm_cls0.DPLL_RR2[2363].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22364 GTM_gtm_cls0.DPLL_RR2[2364].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22365 GTM_gtm_cls0.DPLL_RR2[2365].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22366 GTM_gtm_cls0.DPLL_RR2[2366].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22367 GTM_gtm_cls0.DPLL_RR2[2367].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22368 GTM_gtm_cls0.DPLL_RR2[2368].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22369 GTM_gtm_cls0.DPLL_RR2[2369].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22370 GTM_gtm_cls0.DPLL_RR2[2370].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22371 GTM_gtm_cls0.DPLL_RR2[2371].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22372 GTM_gtm_cls0.DPLL_RR2[2372].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22373 GTM_gtm_cls0.DPLL_RR2[2373].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22374 GTM_gtm_cls0.DPLL_RR2[2374].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22375 GTM_gtm_cls0.DPLL_RR2[2375].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22376 GTM_gtm_cls0.DPLL_RR2[2376].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22377 GTM_gtm_cls0.DPLL_RR2[2377].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22378 GTM_gtm_cls0.DPLL_RR2[2378].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22379 GTM_gtm_cls0.DPLL_RR2[2379].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22380 GTM_gtm_cls0.DPLL_RR2[2380].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22381 GTM_gtm_cls0.DPLL_RR2[2381].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22382 GTM_gtm_cls0.DPLL_RR2[2382].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22383 GTM_gtm_cls0.DPLL_RR2[2383].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22384 GTM_gtm_cls0.DPLL_RR2[2384].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22385 GTM_gtm_cls0.DPLL_RR2[2385].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22386 GTM_gtm_cls0.DPLL_RR2[2386].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22387 GTM_gtm_cls0.DPLL_RR2[2387].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22388 GTM_gtm_cls0.DPLL_RR2[2388].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22389 GTM_gtm_cls0.DPLL_RR2[2389].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22390 GTM_gtm_cls0.DPLL_RR2[2390].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22391 GTM_gtm_cls0.DPLL_RR2[2391].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22392 GTM_gtm_cls0.DPLL_RR2[2392].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22393 GTM_gtm_cls0.DPLL_RR2[2393].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22394 GTM_gtm_cls0.DPLL_RR2[2394].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22395 GTM_gtm_cls0.DPLL_RR2[2395].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22396 GTM_gtm_cls0.DPLL_RR2[2396].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22397 GTM_gtm_cls0.DPLL_RR2[2397].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22398 GTM_gtm_cls0.DPLL_RR2[2398].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22399 GTM_gtm_cls0.DPLL_RR2[2399].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22400 GTM_gtm_cls0.DPLL_RR2[2400].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22401 GTM_gtm_cls0.DPLL_RR2[2401].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22402 GTM_gtm_cls0.DPLL_RR2[2402].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22403 GTM_gtm_cls0.DPLL_RR2[2403].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22404 GTM_gtm_cls0.DPLL_RR2[2404].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22405 GTM_gtm_cls0.DPLL_RR2[2405].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22406 GTM_gtm_cls0.DPLL_RR2[2406].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22407 GTM_gtm_cls0.DPLL_RR2[2407].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22408 GTM_gtm_cls0.DPLL_RR2[2408].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22409 GTM_gtm_cls0.DPLL_RR2[2409].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22410 GTM_gtm_cls0.DPLL_RR2[2410].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22411 GTM_gtm_cls0.DPLL_RR2[2411].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22412 GTM_gtm_cls0.DPLL_RR2[2412].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22413 GTM_gtm_cls0.DPLL_RR2[2413].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22414 GTM_gtm_cls0.DPLL_RR2[2414].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22415 GTM_gtm_cls0.DPLL_RR2[2415].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22416 GTM_gtm_cls0.DPLL_RR2[2416].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22417 GTM_gtm_cls0.DPLL_RR2[2417].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22418 GTM_gtm_cls0.DPLL_RR2[2418].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22419 GTM_gtm_cls0.DPLL_RR2[2419].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22420 GTM_gtm_cls0.DPLL_RR2[2420].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22421 GTM_gtm_cls0.DPLL_RR2[2421].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22422 GTM_gtm_cls0.DPLL_RR2[2422].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22423 GTM_gtm_cls0.DPLL_RR2[2423].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22424 GTM_gtm_cls0.DPLL_RR2[2424].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22425 GTM_gtm_cls0.DPLL_RR2[2425].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22426 GTM_gtm_cls0.DPLL_RR2[2426].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22427 GTM_gtm_cls0.DPLL_RR2[2427].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22428 GTM_gtm_cls0.DPLL_RR2[2428].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22429 GTM_gtm_cls0.DPLL_RR2[2429].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22430 GTM_gtm_cls0.DPLL_RR2[2430].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22431 GTM_gtm_cls0.DPLL_RR2[2431].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22432 GTM_gtm_cls0.DPLL_RR2[2432].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22433 GTM_gtm_cls0.DPLL_RR2[2433].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22434 GTM_gtm_cls0.DPLL_RR2[2434].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22435 GTM_gtm_cls0.DPLL_RR2[2435].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22436 GTM_gtm_cls0.DPLL_RR2[2436].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22437 GTM_gtm_cls0.DPLL_RR2[2437].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22438 GTM_gtm_cls0.DPLL_RR2[2438].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22439 GTM_gtm_cls0.DPLL_RR2[2439].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22440 GTM_gtm_cls0.DPLL_RR2[2440].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22441 GTM_gtm_cls0.DPLL_RR2[2441].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22442 GTM_gtm_cls0.DPLL_RR2[2442].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22443 GTM_gtm_cls0.DPLL_RR2[2443].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22444 GTM_gtm_cls0.DPLL_RR2[2444].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22445 GTM_gtm_cls0.DPLL_RR2[2445].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22446 GTM_gtm_cls0.DPLL_RR2[2446].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22447 GTM_gtm_cls0.DPLL_RR2[2447].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22448 GTM_gtm_cls0.DPLL_RR2[2448].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22449 GTM_gtm_cls0.DPLL_RR2[2449].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22450 GTM_gtm_cls0.DPLL_RR2[2450].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22451 GTM_gtm_cls0.DPLL_RR2[2451].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22452 GTM_gtm_cls0.DPLL_RR2[2452].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22453 GTM_gtm_cls0.DPLL_RR2[2453].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22454 GTM_gtm_cls0.DPLL_RR2[2454].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22455 GTM_gtm_cls0.DPLL_RR2[2455].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22456 GTM_gtm_cls0.DPLL_RR2[2456].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22457 GTM_gtm_cls0.DPLL_RR2[2457].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22458 GTM_gtm_cls0.DPLL_RR2[2458].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22459 GTM_gtm_cls0.DPLL_RR2[2459].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22460 GTM_gtm_cls0.DPLL_RR2[2460].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22461 GTM_gtm_cls0.DPLL_RR2[2461].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22462 GTM_gtm_cls0.DPLL_RR2[2462].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22463 GTM_gtm_cls0.DPLL_RR2[2463].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22464 GTM_gtm_cls0.DPLL_RR2[2464].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22465 GTM_gtm_cls0.DPLL_RR2[2465].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22466 GTM_gtm_cls0.DPLL_RR2[2466].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22467 GTM_gtm_cls0.DPLL_RR2[2467].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22468 GTM_gtm_cls0.DPLL_RR2[2468].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22469 GTM_gtm_cls0.DPLL_RR2[2469].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22470 GTM_gtm_cls0.DPLL_RR2[2470].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22471 GTM_gtm_cls0.DPLL_RR2[2471].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22472 GTM_gtm_cls0.DPLL_RR2[2472].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22473 GTM_gtm_cls0.DPLL_RR2[2473].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22474 GTM_gtm_cls0.DPLL_RR2[2474].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22475 GTM_gtm_cls0.DPLL_RR2[2475].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22476 GTM_gtm_cls0.DPLL_RR2[2476].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22477 GTM_gtm_cls0.DPLL_RR2[2477].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22478 GTM_gtm_cls0.DPLL_RR2[2478].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22479 GTM_gtm_cls0.DPLL_RR2[2479].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22480 GTM_gtm_cls0.DPLL_RR2[2480].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22481 GTM_gtm_cls0.DPLL_RR2[2481].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22482 GTM_gtm_cls0.DPLL_RR2[2482].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22483 GTM_gtm_cls0.DPLL_RR2[2483].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22484 GTM_gtm_cls0.DPLL_RR2[2484].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22485 GTM_gtm_cls0.DPLL_RR2[2485].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22486 GTM_gtm_cls0.DPLL_RR2[2486].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22487 GTM_gtm_cls0.DPLL_RR2[2487].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22488 GTM_gtm_cls0.DPLL_RR2[2488].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22489 GTM_gtm_cls0.DPLL_RR2[2489].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22490 GTM_gtm_cls0.DPLL_RR2[2490].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22491 GTM_gtm_cls0.DPLL_RR2[2491].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22492 GTM_gtm_cls0.DPLL_RR2[2492].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22493 GTM_gtm_cls0.DPLL_RR2[2493].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22494 GTM_gtm_cls0.DPLL_RR2[2494].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22495 GTM_gtm_cls0.DPLL_RR2[2495].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22496 GTM_gtm_cls0.DPLL_RR2[2496].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22497 GTM_gtm_cls0.DPLL_RR2[2497].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22498 GTM_gtm_cls0.DPLL_RR2[2498].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22499 GTM_gtm_cls0.DPLL_RR2[2499].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22500 GTM_gtm_cls0.DPLL_RR2[2500].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22501 GTM_gtm_cls0.DPLL_RR2[2501].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22502 GTM_gtm_cls0.DPLL_RR2[2502].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22503 GTM_gtm_cls0.DPLL_RR2[2503].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22504 GTM_gtm_cls0.DPLL_RR2[2504].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22505 GTM_gtm_cls0.DPLL_RR2[2505].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22506 GTM_gtm_cls0.DPLL_RR2[2506].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22507 GTM_gtm_cls0.DPLL_RR2[2507].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22508 GTM_gtm_cls0.DPLL_RR2[2508].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22509 GTM_gtm_cls0.DPLL_RR2[2509].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22510 GTM_gtm_cls0.DPLL_RR2[2510].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22511 GTM_gtm_cls0.DPLL_RR2[2511].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22512 GTM_gtm_cls0.DPLL_RR2[2512].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22513 GTM_gtm_cls0.DPLL_RR2[2513].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22514 GTM_gtm_cls0.DPLL_RR2[2514].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22515 GTM_gtm_cls0.DPLL_RR2[2515].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22516 GTM_gtm_cls0.DPLL_RR2[2516].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22517 GTM_gtm_cls0.DPLL_RR2[2517].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22518 GTM_gtm_cls0.DPLL_RR2[2518].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22519 GTM_gtm_cls0.DPLL_RR2[2519].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22520 GTM_gtm_cls0.DPLL_RR2[2520].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22521 GTM_gtm_cls0.DPLL_RR2[2521].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22522 GTM_gtm_cls0.DPLL_RR2[2522].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22523 GTM_gtm_cls0.DPLL_RR2[2523].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22524 GTM_gtm_cls0.DPLL_RR2[2524].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22525 GTM_gtm_cls0.DPLL_RR2[2525].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22526 GTM_gtm_cls0.DPLL_RR2[2526].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22527 GTM_gtm_cls0.DPLL_RR2[2527].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22528 GTM_gtm_cls0.DPLL_RR2[2528].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22529 GTM_gtm_cls0.DPLL_RR2[2529].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22530 GTM_gtm_cls0.DPLL_RR2[2530].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22531 GTM_gtm_cls0.DPLL_RR2[2531].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22532 GTM_gtm_cls0.DPLL_RR2[2532].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22533 GTM_gtm_cls0.DPLL_RR2[2533].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22534 GTM_gtm_cls0.DPLL_RR2[2534].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22535 GTM_gtm_cls0.DPLL_RR2[2535].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22536 GTM_gtm_cls0.DPLL_RR2[2536].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22537 GTM_gtm_cls0.DPLL_RR2[2537].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22538 GTM_gtm_cls0.DPLL_RR2[2538].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22539 GTM_gtm_cls0.DPLL_RR2[2539].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22540 GTM_gtm_cls0.DPLL_RR2[2540].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22541 GTM_gtm_cls0.DPLL_RR2[2541].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22542 GTM_gtm_cls0.DPLL_RR2[2542].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22543 GTM_gtm_cls0.DPLL_RR2[2543].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22544 GTM_gtm_cls0.DPLL_RR2[2544].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22545 GTM_gtm_cls0.DPLL_RR2[2545].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22546 GTM_gtm_cls0.DPLL_RR2[2546].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22547 GTM_gtm_cls0.DPLL_RR2[2547].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22548 GTM_gtm_cls0.DPLL_RR2[2548].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22549 GTM_gtm_cls0.DPLL_RR2[2549].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22550 GTM_gtm_cls0.DPLL_RR2[2550].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22551 GTM_gtm_cls0.DPLL_RR2[2551].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22552 GTM_gtm_cls0.DPLL_RR2[2552].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22553 GTM_gtm_cls0.DPLL_RR2[2553].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22554 GTM_gtm_cls0.DPLL_RR2[2554].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22555 GTM_gtm_cls0.DPLL_RR2[2555].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22556 GTM_gtm_cls0.DPLL_RR2[2556].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22557 GTM_gtm_cls0.DPLL_RR2[2557].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22558 GTM_gtm_cls0.DPLL_RR2[2558].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22559 GTM_gtm_cls0.DPLL_RR2[2559].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22560 GTM_gtm_cls0.DPLL_RR2[2560].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22561 GTM_gtm_cls0.DPLL_RR2[2561].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22562 GTM_gtm_cls0.DPLL_RR2[2562].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22563 GTM_gtm_cls0.DPLL_RR2[2563].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22564 GTM_gtm_cls0.DPLL_RR2[2564].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22565 GTM_gtm_cls0.DPLL_RR2[2565].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22566 GTM_gtm_cls0.DPLL_RR2[2566].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22567 GTM_gtm_cls0.DPLL_RR2[2567].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22568 GTM_gtm_cls0.DPLL_RR2[2568].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22569 GTM_gtm_cls0.DPLL_RR2[2569].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22570 GTM_gtm_cls0.DPLL_RR2[2570].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22571 GTM_gtm_cls0.DPLL_RR2[2571].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22572 GTM_gtm_cls0.DPLL_RR2[2572].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22573 GTM_gtm_cls0.DPLL_RR2[2573].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22574 GTM_gtm_cls0.DPLL_RR2[2574].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22575 GTM_gtm_cls0.DPLL_RR2[2575].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22576 GTM_gtm_cls0.DPLL_RR2[2576].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22577 GTM_gtm_cls0.DPLL_RR2[2577].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22578 GTM_gtm_cls0.DPLL_RR2[2578].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22579 GTM_gtm_cls0.DPLL_RR2[2579].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22580 GTM_gtm_cls0.DPLL_RR2[2580].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22581 GTM_gtm_cls0.DPLL_RR2[2581].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22582 GTM_gtm_cls0.DPLL_RR2[2582].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22583 GTM_gtm_cls0.DPLL_RR2[2583].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22584 GTM_gtm_cls0.DPLL_RR2[2584].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22585 GTM_gtm_cls0.DPLL_RR2[2585].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22586 GTM_gtm_cls0.DPLL_RR2[2586].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22587 GTM_gtm_cls0.DPLL_RR2[2587].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22588 GTM_gtm_cls0.DPLL_RR2[2588].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22589 GTM_gtm_cls0.DPLL_RR2[2589].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22590 GTM_gtm_cls0.DPLL_RR2[2590].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22591 GTM_gtm_cls0.DPLL_RR2[2591].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22592 GTM_gtm_cls0.DPLL_RR2[2592].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22593 GTM_gtm_cls0.DPLL_RR2[2593].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22594 GTM_gtm_cls0.DPLL_RR2[2594].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22595 GTM_gtm_cls0.DPLL_RR2[2595].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22596 GTM_gtm_cls0.DPLL_RR2[2596].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22597 GTM_gtm_cls0.DPLL_RR2[2597].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22598 GTM_gtm_cls0.DPLL_RR2[2598].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22599 GTM_gtm_cls0.DPLL_RR2[2599].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22600 GTM_gtm_cls0.DPLL_RR2[2600].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22601 GTM_gtm_cls0.DPLL_RR2[2601].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22602 GTM_gtm_cls0.DPLL_RR2[2602].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22603 GTM_gtm_cls0.DPLL_RR2[2603].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22604 GTM_gtm_cls0.DPLL_RR2[2604].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22605 GTM_gtm_cls0.DPLL_RR2[2605].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22606 GTM_gtm_cls0.DPLL_RR2[2606].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22607 GTM_gtm_cls0.DPLL_RR2[2607].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22608 GTM_gtm_cls0.DPLL_RR2[2608].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22609 GTM_gtm_cls0.DPLL_RR2[2609].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22610 GTM_gtm_cls0.DPLL_RR2[2610].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22611 GTM_gtm_cls0.DPLL_RR2[2611].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22612 GTM_gtm_cls0.DPLL_RR2[2612].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22613 GTM_gtm_cls0.DPLL_RR2[2613].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22614 GTM_gtm_cls0.DPLL_RR2[2614].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22615 GTM_gtm_cls0.DPLL_RR2[2615].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22616 GTM_gtm_cls0.DPLL_RR2[2616].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22617 GTM_gtm_cls0.DPLL_RR2[2617].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22618 GTM_gtm_cls0.DPLL_RR2[2618].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22619 GTM_gtm_cls0.DPLL_RR2[2619].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22620 GTM_gtm_cls0.DPLL_RR2[2620].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22621 GTM_gtm_cls0.DPLL_RR2[2621].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22622 GTM_gtm_cls0.DPLL_RR2[2622].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22623 GTM_gtm_cls0.DPLL_RR2[2623].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22624 GTM_gtm_cls0.DPLL_RR2[2624].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22625 GTM_gtm_cls0.DPLL_RR2[2625].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22626 GTM_gtm_cls0.DPLL_RR2[2626].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22627 GTM_gtm_cls0.DPLL_RR2[2627].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22628 GTM_gtm_cls0.DPLL_RR2[2628].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22629 GTM_gtm_cls0.DPLL_RR2[2629].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22630 GTM_gtm_cls0.DPLL_RR2[2630].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22631 GTM_gtm_cls0.DPLL_RR2[2631].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22632 GTM_gtm_cls0.DPLL_RR2[2632].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22633 GTM_gtm_cls0.DPLL_RR2[2633].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22634 GTM_gtm_cls0.DPLL_RR2[2634].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22635 GTM_gtm_cls0.DPLL_RR2[2635].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22636 GTM_gtm_cls0.DPLL_RR2[2636].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22637 GTM_gtm_cls0.DPLL_RR2[2637].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22638 GTM_gtm_cls0.DPLL_RR2[2638].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22639 GTM_gtm_cls0.DPLL_RR2[2639].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22640 GTM_gtm_cls0.DPLL_RR2[2640].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22641 GTM_gtm_cls0.DPLL_RR2[2641].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22642 GTM_gtm_cls0.DPLL_RR2[2642].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22643 GTM_gtm_cls0.DPLL_RR2[2643].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22644 GTM_gtm_cls0.DPLL_RR2[2644].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22645 GTM_gtm_cls0.DPLL_RR2[2645].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22646 GTM_gtm_cls0.DPLL_RR2[2646].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22647 GTM_gtm_cls0.DPLL_RR2[2647].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22648 GTM_gtm_cls0.DPLL_RR2[2648].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22649 GTM_gtm_cls0.DPLL_RR2[2649].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22650 GTM_gtm_cls0.DPLL_RR2[2650].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22651 GTM_gtm_cls0.DPLL_RR2[2651].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22652 GTM_gtm_cls0.DPLL_RR2[2652].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22653 GTM_gtm_cls0.DPLL_RR2[2653].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22654 GTM_gtm_cls0.DPLL_RR2[2654].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22655 GTM_gtm_cls0.DPLL_RR2[2655].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22656 GTM_gtm_cls0.DPLL_RR2[2656].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22657 GTM_gtm_cls0.DPLL_RR2[2657].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22658 GTM_gtm_cls0.DPLL_RR2[2658].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22659 GTM_gtm_cls0.DPLL_RR2[2659].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22660 GTM_gtm_cls0.DPLL_RR2[2660].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22661 GTM_gtm_cls0.DPLL_RR2[2661].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22662 GTM_gtm_cls0.DPLL_RR2[2662].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22663 GTM_gtm_cls0.DPLL_RR2[2663].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22664 GTM_gtm_cls0.DPLL_RR2[2664].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22665 GTM_gtm_cls0.DPLL_RR2[2665].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22666 GTM_gtm_cls0.DPLL_RR2[2666].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22667 GTM_gtm_cls0.DPLL_RR2[2667].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22668 GTM_gtm_cls0.DPLL_RR2[2668].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22669 GTM_gtm_cls0.DPLL_RR2[2669].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22670 GTM_gtm_cls0.DPLL_RR2[2670].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22671 GTM_gtm_cls0.DPLL_RR2[2671].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22672 GTM_gtm_cls0.DPLL_RR2[2672].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22673 GTM_gtm_cls0.DPLL_RR2[2673].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22674 GTM_gtm_cls0.DPLL_RR2[2674].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22675 GTM_gtm_cls0.DPLL_RR2[2675].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22676 GTM_gtm_cls0.DPLL_RR2[2676].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22677 GTM_gtm_cls0.DPLL_RR2[2677].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22678 GTM_gtm_cls0.DPLL_RR2[2678].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22679 GTM_gtm_cls0.DPLL_RR2[2679].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22680 GTM_gtm_cls0.DPLL_RR2[2680].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22681 GTM_gtm_cls0.DPLL_RR2[2681].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22682 GTM_gtm_cls0.DPLL_RR2[2682].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22683 GTM_gtm_cls0.DPLL_RR2[2683].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22684 GTM_gtm_cls0.DPLL_RR2[2684].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22685 GTM_gtm_cls0.DPLL_RR2[2685].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22686 GTM_gtm_cls0.DPLL_RR2[2686].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22687 GTM_gtm_cls0.DPLL_RR2[2687].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22688 GTM_gtm_cls0.DPLL_RR2[2688].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22689 GTM_gtm_cls0.DPLL_RR2[2689].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22690 GTM_gtm_cls0.DPLL_RR2[2690].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22691 GTM_gtm_cls0.DPLL_RR2[2691].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22692 GTM_gtm_cls0.DPLL_RR2[2692].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22693 GTM_gtm_cls0.DPLL_RR2[2693].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22694 GTM_gtm_cls0.DPLL_RR2[2694].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22695 GTM_gtm_cls0.DPLL_RR2[2695].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22696 GTM_gtm_cls0.DPLL_RR2[2696].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22697 GTM_gtm_cls0.DPLL_RR2[2697].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22698 GTM_gtm_cls0.DPLL_RR2[2698].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22699 GTM_gtm_cls0.DPLL_RR2[2699].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22700 GTM_gtm_cls0.DPLL_RR2[2700].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22701 GTM_gtm_cls0.DPLL_RR2[2701].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22702 GTM_gtm_cls0.DPLL_RR2[2702].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22703 GTM_gtm_cls0.DPLL_RR2[2703].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22704 GTM_gtm_cls0.DPLL_RR2[2704].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22705 GTM_gtm_cls0.DPLL_RR2[2705].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22706 GTM_gtm_cls0.DPLL_RR2[2706].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22707 GTM_gtm_cls0.DPLL_RR2[2707].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22708 GTM_gtm_cls0.DPLL_RR2[2708].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22709 GTM_gtm_cls0.DPLL_RR2[2709].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22710 GTM_gtm_cls0.DPLL_RR2[2710].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22711 GTM_gtm_cls0.DPLL_RR2[2711].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22712 GTM_gtm_cls0.DPLL_RR2[2712].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22713 GTM_gtm_cls0.DPLL_RR2[2713].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22714 GTM_gtm_cls0.DPLL_RR2[2714].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22715 GTM_gtm_cls0.DPLL_RR2[2715].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22716 GTM_gtm_cls0.DPLL_RR2[2716].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22717 GTM_gtm_cls0.DPLL_RR2[2717].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22718 GTM_gtm_cls0.DPLL_RR2[2718].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22719 GTM_gtm_cls0.DPLL_RR2[2719].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22720 GTM_gtm_cls0.DPLL_RR2[2720].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22721 GTM_gtm_cls0.DPLL_RR2[2721].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22722 GTM_gtm_cls0.DPLL_RR2[2722].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22723 GTM_gtm_cls0.DPLL_RR2[2723].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22724 GTM_gtm_cls0.DPLL_RR2[2724].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22725 GTM_gtm_cls0.DPLL_RR2[2725].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22726 GTM_gtm_cls0.DPLL_RR2[2726].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22727 GTM_gtm_cls0.DPLL_RR2[2727].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22728 GTM_gtm_cls0.DPLL_RR2[2728].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22729 GTM_gtm_cls0.DPLL_RR2[2729].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22730 GTM_gtm_cls0.DPLL_RR2[2730].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22731 GTM_gtm_cls0.DPLL_RR2[2731].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22732 GTM_gtm_cls0.DPLL_RR2[2732].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22733 GTM_gtm_cls0.DPLL_RR2[2733].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22734 GTM_gtm_cls0.DPLL_RR2[2734].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22735 GTM_gtm_cls0.DPLL_RR2[2735].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22736 GTM_gtm_cls0.DPLL_RR2[2736].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22737 GTM_gtm_cls0.DPLL_RR2[2737].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22738 GTM_gtm_cls0.DPLL_RR2[2738].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22739 GTM_gtm_cls0.DPLL_RR2[2739].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22740 GTM_gtm_cls0.DPLL_RR2[2740].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22741 GTM_gtm_cls0.DPLL_RR2[2741].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22742 GTM_gtm_cls0.DPLL_RR2[2742].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22743 GTM_gtm_cls0.DPLL_RR2[2743].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22744 GTM_gtm_cls0.DPLL_RR2[2744].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22745 GTM_gtm_cls0.DPLL_RR2[2745].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22746 GTM_gtm_cls0.DPLL_RR2[2746].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22747 GTM_gtm_cls0.DPLL_RR2[2747].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22748 GTM_gtm_cls0.DPLL_RR2[2748].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22749 GTM_gtm_cls0.DPLL_RR2[2749].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22750 GTM_gtm_cls0.DPLL_RR2[2750].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22751 GTM_gtm_cls0.DPLL_RR2[2751].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22752 GTM_gtm_cls0.DPLL_RR2[2752].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22753 GTM_gtm_cls0.DPLL_RR2[2753].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22754 GTM_gtm_cls0.DPLL_RR2[2754].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22755 GTM_gtm_cls0.DPLL_RR2[2755].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22756 GTM_gtm_cls0.DPLL_RR2[2756].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22757 GTM_gtm_cls0.DPLL_RR2[2757].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22758 GTM_gtm_cls0.DPLL_RR2[2758].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22759 GTM_gtm_cls0.DPLL_RR2[2759].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22760 GTM_gtm_cls0.DPLL_RR2[2760].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22761 GTM_gtm_cls0.DPLL_RR2[2761].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22762 GTM_gtm_cls0.DPLL_RR2[2762].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22763 GTM_gtm_cls0.DPLL_RR2[2763].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22764 GTM_gtm_cls0.DPLL_RR2[2764].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22765 GTM_gtm_cls0.DPLL_RR2[2765].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22766 GTM_gtm_cls0.DPLL_RR2[2766].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22767 GTM_gtm_cls0.DPLL_RR2[2767].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22768 GTM_gtm_cls0.DPLL_RR2[2768].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22769 GTM_gtm_cls0.DPLL_RR2[2769].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22770 GTM_gtm_cls0.DPLL_RR2[2770].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22771 GTM_gtm_cls0.DPLL_RR2[2771].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22772 GTM_gtm_cls0.DPLL_RR2[2772].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22773 GTM_gtm_cls0.DPLL_RR2[2773].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22774 GTM_gtm_cls0.DPLL_RR2[2774].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22775 GTM_gtm_cls0.DPLL_RR2[2775].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22776 GTM_gtm_cls0.DPLL_RR2[2776].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22777 GTM_gtm_cls0.DPLL_RR2[2777].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22778 GTM_gtm_cls0.DPLL_RR2[2778].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22779 GTM_gtm_cls0.DPLL_RR2[2779].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22780 GTM_gtm_cls0.DPLL_RR2[2780].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22781 GTM_gtm_cls0.DPLL_RR2[2781].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22782 GTM_gtm_cls0.DPLL_RR2[2782].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22783 GTM_gtm_cls0.DPLL_RR2[2783].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22784 GTM_gtm_cls0.DPLL_RR2[2784].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22785 GTM_gtm_cls0.DPLL_RR2[2785].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22786 GTM_gtm_cls0.DPLL_RR2[2786].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22787 GTM_gtm_cls0.DPLL_RR2[2787].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22788 GTM_gtm_cls0.DPLL_RR2[2788].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22789 GTM_gtm_cls0.DPLL_RR2[2789].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22790 GTM_gtm_cls0.DPLL_RR2[2790].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22791 GTM_gtm_cls0.DPLL_RR2[2791].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22792 GTM_gtm_cls0.DPLL_RR2[2792].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22793 GTM_gtm_cls0.DPLL_RR2[2793].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22794 GTM_gtm_cls0.DPLL_RR2[2794].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22795 GTM_gtm_cls0.DPLL_RR2[2795].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22796 GTM_gtm_cls0.DPLL_RR2[2796].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22797 GTM_gtm_cls0.DPLL_RR2[2797].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22798 GTM_gtm_cls0.DPLL_RR2[2798].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22799 GTM_gtm_cls0.DPLL_RR2[2799].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22800 GTM_gtm_cls0.DPLL_RR2[2800].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22801 GTM_gtm_cls0.DPLL_RR2[2801].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22802 GTM_gtm_cls0.DPLL_RR2[2802].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22803 GTM_gtm_cls0.DPLL_RR2[2803].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22804 GTM_gtm_cls0.DPLL_RR2[2804].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22805 GTM_gtm_cls0.DPLL_RR2[2805].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22806 GTM_gtm_cls0.DPLL_RR2[2806].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22807 GTM_gtm_cls0.DPLL_RR2[2807].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22808 GTM_gtm_cls0.DPLL_RR2[2808].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22809 GTM_gtm_cls0.DPLL_RR2[2809].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22810 GTM_gtm_cls0.DPLL_RR2[2810].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22811 GTM_gtm_cls0.DPLL_RR2[2811].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22812 GTM_gtm_cls0.DPLL_RR2[2812].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22813 GTM_gtm_cls0.DPLL_RR2[2813].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22814 GTM_gtm_cls0.DPLL_RR2[2814].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22815 GTM_gtm_cls0.DPLL_RR2[2815].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22816 GTM_gtm_cls0.DPLL_RR2[2816].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22817 GTM_gtm_cls0.DPLL_RR2[2817].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22818 GTM_gtm_cls0.DPLL_RR2[2818].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22819 GTM_gtm_cls0.DPLL_RR2[2819].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22820 GTM_gtm_cls0.DPLL_RR2[2820].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22821 GTM_gtm_cls0.DPLL_RR2[2821].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22822 GTM_gtm_cls0.DPLL_RR2[2822].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22823 GTM_gtm_cls0.DPLL_RR2[2823].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22824 GTM_gtm_cls0.DPLL_RR2[2824].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22825 GTM_gtm_cls0.DPLL_RR2[2825].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22826 GTM_gtm_cls0.DPLL_RR2[2826].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22827 GTM_gtm_cls0.DPLL_RR2[2827].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22828 GTM_gtm_cls0.DPLL_RR2[2828].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22829 GTM_gtm_cls0.DPLL_RR2[2829].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22830 GTM_gtm_cls0.DPLL_RR2[2830].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22831 GTM_gtm_cls0.DPLL_RR2[2831].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22832 GTM_gtm_cls0.DPLL_RR2[2832].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22833 GTM_gtm_cls0.DPLL_RR2[2833].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22834 GTM_gtm_cls0.DPLL_RR2[2834].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22835 GTM_gtm_cls0.DPLL_RR2[2835].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22836 GTM_gtm_cls0.DPLL_RR2[2836].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22837 GTM_gtm_cls0.DPLL_RR2[2837].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22838 GTM_gtm_cls0.DPLL_RR2[2838].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22839 GTM_gtm_cls0.DPLL_RR2[2839].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22840 GTM_gtm_cls0.DPLL_RR2[2840].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22841 GTM_gtm_cls0.DPLL_RR2[2841].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22842 GTM_gtm_cls0.DPLL_RR2[2842].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22843 GTM_gtm_cls0.DPLL_RR2[2843].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22844 GTM_gtm_cls0.DPLL_RR2[2844].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22845 GTM_gtm_cls0.DPLL_RR2[2845].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22846 GTM_gtm_cls0.DPLL_RR2[2846].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22847 GTM_gtm_cls0.DPLL_RR2[2847].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22848 GTM_gtm_cls0.DPLL_RR2[2848].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22849 GTM_gtm_cls0.DPLL_RR2[2849].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22850 GTM_gtm_cls0.DPLL_RR2[2850].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22851 GTM_gtm_cls0.DPLL_RR2[2851].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22852 GTM_gtm_cls0.DPLL_RR2[2852].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22853 GTM_gtm_cls0.DPLL_RR2[2853].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22854 GTM_gtm_cls0.DPLL_RR2[2854].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22855 GTM_gtm_cls0.DPLL_RR2[2855].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22856 GTM_gtm_cls0.DPLL_RR2[2856].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22857 GTM_gtm_cls0.DPLL_RR2[2857].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22858 GTM_gtm_cls0.DPLL_RR2[2858].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22859 GTM_gtm_cls0.DPLL_RR2[2859].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22860 GTM_gtm_cls0.DPLL_RR2[2860].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22861 GTM_gtm_cls0.DPLL_RR2[2861].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22862 GTM_gtm_cls0.DPLL_RR2[2862].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22863 GTM_gtm_cls0.DPLL_RR2[2863].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22864 GTM_gtm_cls0.DPLL_RR2[2864].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22865 GTM_gtm_cls0.DPLL_RR2[2865].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22866 GTM_gtm_cls0.DPLL_RR2[2866].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22867 GTM_gtm_cls0.DPLL_RR2[2867].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22868 GTM_gtm_cls0.DPLL_RR2[2868].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22869 GTM_gtm_cls0.DPLL_RR2[2869].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22870 GTM_gtm_cls0.DPLL_RR2[2870].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22871 GTM_gtm_cls0.DPLL_RR2[2871].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22872 GTM_gtm_cls0.DPLL_RR2[2872].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22873 GTM_gtm_cls0.DPLL_RR2[2873].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22874 GTM_gtm_cls0.DPLL_RR2[2874].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22875 GTM_gtm_cls0.DPLL_RR2[2875].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22876 GTM_gtm_cls0.DPLL_RR2[2876].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22877 GTM_gtm_cls0.DPLL_RR2[2877].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22878 GTM_gtm_cls0.DPLL_RR2[2878].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22879 GTM_gtm_cls0.DPLL_RR2[2879].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22880 GTM_gtm_cls0.DPLL_RR2[2880].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22881 GTM_gtm_cls0.DPLL_RR2[2881].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22882 GTM_gtm_cls0.DPLL_RR2[2882].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22883 GTM_gtm_cls0.DPLL_RR2[2883].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22884 GTM_gtm_cls0.DPLL_RR2[2884].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22885 GTM_gtm_cls0.DPLL_RR2[2885].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22886 GTM_gtm_cls0.DPLL_RR2[2886].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22887 GTM_gtm_cls0.DPLL_RR2[2887].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22888 GTM_gtm_cls0.DPLL_RR2[2888].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22889 GTM_gtm_cls0.DPLL_RR2[2889].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22890 GTM_gtm_cls0.DPLL_RR2[2890].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22891 GTM_gtm_cls0.DPLL_RR2[2891].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22892 GTM_gtm_cls0.DPLL_RR2[2892].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22893 GTM_gtm_cls0.DPLL_RR2[2893].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22894 GTM_gtm_cls0.DPLL_RR2[2894].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22895 GTM_gtm_cls0.DPLL_RR2[2895].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22896 GTM_gtm_cls0.DPLL_RR2[2896].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22897 GTM_gtm_cls0.DPLL_RR2[2897].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22898 GTM_gtm_cls0.DPLL_RR2[2898].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22899 GTM_gtm_cls0.DPLL_RR2[2899].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22900 GTM_gtm_cls0.DPLL_RR2[2900].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22901 GTM_gtm_cls0.DPLL_RR2[2901].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22902 GTM_gtm_cls0.DPLL_RR2[2902].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22903 GTM_gtm_cls0.DPLL_RR2[2903].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22904 GTM_gtm_cls0.DPLL_RR2[2904].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22905 GTM_gtm_cls0.DPLL_RR2[2905].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22906 GTM_gtm_cls0.DPLL_RR2[2906].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22907 GTM_gtm_cls0.DPLL_RR2[2907].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22908 GTM_gtm_cls0.DPLL_RR2[2908].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22909 GTM_gtm_cls0.DPLL_RR2[2909].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22910 GTM_gtm_cls0.DPLL_RR2[2910].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22911 GTM_gtm_cls0.DPLL_RR2[2911].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22912 GTM_gtm_cls0.DPLL_RR2[2912].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22913 GTM_gtm_cls0.DPLL_RR2[2913].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22914 GTM_gtm_cls0.DPLL_RR2[2914].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22915 GTM_gtm_cls0.DPLL_RR2[2915].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22916 GTM_gtm_cls0.DPLL_RR2[2916].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22917 GTM_gtm_cls0.DPLL_RR2[2917].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22918 GTM_gtm_cls0.DPLL_RR2[2918].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22919 GTM_gtm_cls0.DPLL_RR2[2919].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22920 GTM_gtm_cls0.DPLL_RR2[2920].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22921 GTM_gtm_cls0.DPLL_RR2[2921].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22922 GTM_gtm_cls0.DPLL_RR2[2922].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22923 GTM_gtm_cls0.DPLL_RR2[2923].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22924 GTM_gtm_cls0.DPLL_RR2[2924].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22925 GTM_gtm_cls0.DPLL_RR2[2925].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22926 GTM_gtm_cls0.DPLL_RR2[2926].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22927 GTM_gtm_cls0.DPLL_RR2[2927].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22928 GTM_gtm_cls0.DPLL_RR2[2928].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22929 GTM_gtm_cls0.DPLL_RR2[2929].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22930 GTM_gtm_cls0.DPLL_RR2[2930].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22931 GTM_gtm_cls0.DPLL_RR2[2931].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22932 GTM_gtm_cls0.DPLL_RR2[2932].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22933 GTM_gtm_cls0.DPLL_RR2[2933].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22934 GTM_gtm_cls0.DPLL_RR2[2934].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22935 GTM_gtm_cls0.DPLL_RR2[2935].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22936 GTM_gtm_cls0.DPLL_RR2[2936].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22937 GTM_gtm_cls0.DPLL_RR2[2937].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22938 GTM_gtm_cls0.DPLL_RR2[2938].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22939 GTM_gtm_cls0.DPLL_RR2[2939].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22940 GTM_gtm_cls0.DPLL_RR2[2940].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22941 GTM_gtm_cls0.DPLL_RR2[2941].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22942 GTM_gtm_cls0.DPLL_RR2[2942].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22943 GTM_gtm_cls0.DPLL_RR2[2943].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22944 GTM_gtm_cls0.DPLL_RR2[2944].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22945 GTM_gtm_cls0.DPLL_RR2[2945].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22946 GTM_gtm_cls0.DPLL_RR2[2946].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22947 GTM_gtm_cls0.DPLL_RR2[2947].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22948 GTM_gtm_cls0.DPLL_RR2[2948].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22949 GTM_gtm_cls0.DPLL_RR2[2949].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22950 GTM_gtm_cls0.DPLL_RR2[2950].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22951 GTM_gtm_cls0.DPLL_RR2[2951].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22952 GTM_gtm_cls0.DPLL_RR2[2952].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22953 GTM_gtm_cls0.DPLL_RR2[2953].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22954 GTM_gtm_cls0.DPLL_RR2[2954].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22955 GTM_gtm_cls0.DPLL_RR2[2955].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22956 GTM_gtm_cls0.DPLL_RR2[2956].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22957 GTM_gtm_cls0.DPLL_RR2[2957].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22958 GTM_gtm_cls0.DPLL_RR2[2958].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22959 GTM_gtm_cls0.DPLL_RR2[2959].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22960 GTM_gtm_cls0.DPLL_RR2[2960].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22961 GTM_gtm_cls0.DPLL_RR2[2961].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22962 GTM_gtm_cls0.DPLL_RR2[2962].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22963 GTM_gtm_cls0.DPLL_RR2[2963].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22964 GTM_gtm_cls0.DPLL_RR2[2964].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22965 GTM_gtm_cls0.DPLL_RR2[2965].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22966 GTM_gtm_cls0.DPLL_RR2[2966].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22967 GTM_gtm_cls0.DPLL_RR2[2967].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22968 GTM_gtm_cls0.DPLL_RR2[2968].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22969 GTM_gtm_cls0.DPLL_RR2[2969].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22970 GTM_gtm_cls0.DPLL_RR2[2970].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22971 GTM_gtm_cls0.DPLL_RR2[2971].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22972 GTM_gtm_cls0.DPLL_RR2[2972].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22973 GTM_gtm_cls0.DPLL_RR2[2973].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22974 GTM_gtm_cls0.DPLL_RR2[2974].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22975 GTM_gtm_cls0.DPLL_RR2[2975].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22976 GTM_gtm_cls0.DPLL_RR2[2976].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22977 GTM_gtm_cls0.DPLL_RR2[2977].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22978 GTM_gtm_cls0.DPLL_RR2[2978].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22979 GTM_gtm_cls0.DPLL_RR2[2979].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22980 GTM_gtm_cls0.DPLL_RR2[2980].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22981 GTM_gtm_cls0.DPLL_RR2[2981].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22982 GTM_gtm_cls0.DPLL_RR2[2982].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22983 GTM_gtm_cls0.DPLL_RR2[2983].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22984 GTM_gtm_cls0.DPLL_RR2[2984].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22985 GTM_gtm_cls0.DPLL_RR2[2985].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22986 GTM_gtm_cls0.DPLL_RR2[2986].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22987 GTM_gtm_cls0.DPLL_RR2[2987].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22988 GTM_gtm_cls0.DPLL_RR2[2988].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22989 GTM_gtm_cls0.DPLL_RR2[2989].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22990 GTM_gtm_cls0.DPLL_RR2[2990].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22991 GTM_gtm_cls0.DPLL_RR2[2991].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22992 GTM_gtm_cls0.DPLL_RR2[2992].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22993 GTM_gtm_cls0.DPLL_RR2[2993].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22994 GTM_gtm_cls0.DPLL_RR2[2994].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22995 GTM_gtm_cls0.DPLL_RR2[2995].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22996 GTM_gtm_cls0.DPLL_RR2[2996].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22997 GTM_gtm_cls0.DPLL_RR2[2997].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22998 GTM_gtm_cls0.DPLL_RR2[2998].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR22999 GTM_gtm_cls0.DPLL_RR2[2999].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23000 GTM_gtm_cls0.DPLL_RR2[3000].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23001 GTM_gtm_cls0.DPLL_RR2[3001].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23002 GTM_gtm_cls0.DPLL_RR2[3002].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23003 GTM_gtm_cls0.DPLL_RR2[3003].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23004 GTM_gtm_cls0.DPLL_RR2[3004].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23005 GTM_gtm_cls0.DPLL_RR2[3005].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23006 GTM_gtm_cls0.DPLL_RR2[3006].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23007 GTM_gtm_cls0.DPLL_RR2[3007].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23008 GTM_gtm_cls0.DPLL_RR2[3008].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23009 GTM_gtm_cls0.DPLL_RR2[3009].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23010 GTM_gtm_cls0.DPLL_RR2[3010].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23011 GTM_gtm_cls0.DPLL_RR2[3011].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23012 GTM_gtm_cls0.DPLL_RR2[3012].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23013 GTM_gtm_cls0.DPLL_RR2[3013].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23014 GTM_gtm_cls0.DPLL_RR2[3014].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23015 GTM_gtm_cls0.DPLL_RR2[3015].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23016 GTM_gtm_cls0.DPLL_RR2[3016].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23017 GTM_gtm_cls0.DPLL_RR2[3017].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23018 GTM_gtm_cls0.DPLL_RR2[3018].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23019 GTM_gtm_cls0.DPLL_RR2[3019].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23020 GTM_gtm_cls0.DPLL_RR2[3020].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23021 GTM_gtm_cls0.DPLL_RR2[3021].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23022 GTM_gtm_cls0.DPLL_RR2[3022].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23023 GTM_gtm_cls0.DPLL_RR2[3023].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23024 GTM_gtm_cls0.DPLL_RR2[3024].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23025 GTM_gtm_cls0.DPLL_RR2[3025].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23026 GTM_gtm_cls0.DPLL_RR2[3026].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23027 GTM_gtm_cls0.DPLL_RR2[3027].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23028 GTM_gtm_cls0.DPLL_RR2[3028].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23029 GTM_gtm_cls0.DPLL_RR2[3029].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23030 GTM_gtm_cls0.DPLL_RR2[3030].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23031 GTM_gtm_cls0.DPLL_RR2[3031].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23032 GTM_gtm_cls0.DPLL_RR2[3032].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23033 GTM_gtm_cls0.DPLL_RR2[3033].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23034 GTM_gtm_cls0.DPLL_RR2[3034].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23035 GTM_gtm_cls0.DPLL_RR2[3035].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23036 GTM_gtm_cls0.DPLL_RR2[3036].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23037 GTM_gtm_cls0.DPLL_RR2[3037].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23038 GTM_gtm_cls0.DPLL_RR2[3038].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23039 GTM_gtm_cls0.DPLL_RR2[3039].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23040 GTM_gtm_cls0.DPLL_RR2[3040].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23041 GTM_gtm_cls0.DPLL_RR2[3041].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23042 GTM_gtm_cls0.DPLL_RR2[3042].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23043 GTM_gtm_cls0.DPLL_RR2[3043].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23044 GTM_gtm_cls0.DPLL_RR2[3044].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23045 GTM_gtm_cls0.DPLL_RR2[3045].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23046 GTM_gtm_cls0.DPLL_RR2[3046].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23047 GTM_gtm_cls0.DPLL_RR2[3047].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23048 GTM_gtm_cls0.DPLL_RR2[3048].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23049 GTM_gtm_cls0.DPLL_RR2[3049].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23050 GTM_gtm_cls0.DPLL_RR2[3050].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23051 GTM_gtm_cls0.DPLL_RR2[3051].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23052 GTM_gtm_cls0.DPLL_RR2[3052].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23053 GTM_gtm_cls0.DPLL_RR2[3053].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23054 GTM_gtm_cls0.DPLL_RR2[3054].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23055 GTM_gtm_cls0.DPLL_RR2[3055].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23056 GTM_gtm_cls0.DPLL_RR2[3056].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23057 GTM_gtm_cls0.DPLL_RR2[3057].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23058 GTM_gtm_cls0.DPLL_RR2[3058].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23059 GTM_gtm_cls0.DPLL_RR2[3059].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23060 GTM_gtm_cls0.DPLL_RR2[3060].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23061 GTM_gtm_cls0.DPLL_RR2[3061].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23062 GTM_gtm_cls0.DPLL_RR2[3062].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23063 GTM_gtm_cls0.DPLL_RR2[3063].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23064 GTM_gtm_cls0.DPLL_RR2[3064].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23065 GTM_gtm_cls0.DPLL_RR2[3065].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23066 GTM_gtm_cls0.DPLL_RR2[3066].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23067 GTM_gtm_cls0.DPLL_RR2[3067].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23068 GTM_gtm_cls0.DPLL_RR2[3068].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23069 GTM_gtm_cls0.DPLL_RR2[3069].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23070 GTM_gtm_cls0.DPLL_RR2[3070].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23071 GTM_gtm_cls0.DPLL_RR2[3071].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23072 GTM_gtm_cls0.DPLL_RR2[3072].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23073 GTM_gtm_cls0.DPLL_RR2[3073].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23074 GTM_gtm_cls0.DPLL_RR2[3074].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23075 GTM_gtm_cls0.DPLL_RR2[3075].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23076 GTM_gtm_cls0.DPLL_RR2[3076].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23077 GTM_gtm_cls0.DPLL_RR2[3077].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23078 GTM_gtm_cls0.DPLL_RR2[3078].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23079 GTM_gtm_cls0.DPLL_RR2[3079].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23080 GTM_gtm_cls0.DPLL_RR2[3080].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23081 GTM_gtm_cls0.DPLL_RR2[3081].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23082 GTM_gtm_cls0.DPLL_RR2[3082].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23083 GTM_gtm_cls0.DPLL_RR2[3083].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23084 GTM_gtm_cls0.DPLL_RR2[3084].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23085 GTM_gtm_cls0.DPLL_RR2[3085].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23086 GTM_gtm_cls0.DPLL_RR2[3086].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23087 GTM_gtm_cls0.DPLL_RR2[3087].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23088 GTM_gtm_cls0.DPLL_RR2[3088].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23089 GTM_gtm_cls0.DPLL_RR2[3089].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23090 GTM_gtm_cls0.DPLL_RR2[3090].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23091 GTM_gtm_cls0.DPLL_RR2[3091].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23092 GTM_gtm_cls0.DPLL_RR2[3092].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23093 GTM_gtm_cls0.DPLL_RR2[3093].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23094 GTM_gtm_cls0.DPLL_RR2[3094].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23095 GTM_gtm_cls0.DPLL_RR2[3095].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23096 GTM_gtm_cls0.DPLL_RR2[3096].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23097 GTM_gtm_cls0.DPLL_RR2[3097].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23098 GTM_gtm_cls0.DPLL_RR2[3098].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23099 GTM_gtm_cls0.DPLL_RR2[3099].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23100 GTM_gtm_cls0.DPLL_RR2[3100].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23101 GTM_gtm_cls0.DPLL_RR2[3101].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23102 GTM_gtm_cls0.DPLL_RR2[3102].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23103 GTM_gtm_cls0.DPLL_RR2[3103].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23104 GTM_gtm_cls0.DPLL_RR2[3104].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23105 GTM_gtm_cls0.DPLL_RR2[3105].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23106 GTM_gtm_cls0.DPLL_RR2[3106].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23107 GTM_gtm_cls0.DPLL_RR2[3107].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23108 GTM_gtm_cls0.DPLL_RR2[3108].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23109 GTM_gtm_cls0.DPLL_RR2[3109].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23110 GTM_gtm_cls0.DPLL_RR2[3110].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23111 GTM_gtm_cls0.DPLL_RR2[3111].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23112 GTM_gtm_cls0.DPLL_RR2[3112].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23113 GTM_gtm_cls0.DPLL_RR2[3113].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23114 GTM_gtm_cls0.DPLL_RR2[3114].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23115 GTM_gtm_cls0.DPLL_RR2[3115].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23116 GTM_gtm_cls0.DPLL_RR2[3116].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23117 GTM_gtm_cls0.DPLL_RR2[3117].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23118 GTM_gtm_cls0.DPLL_RR2[3118].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23119 GTM_gtm_cls0.DPLL_RR2[3119].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23120 GTM_gtm_cls0.DPLL_RR2[3120].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23121 GTM_gtm_cls0.DPLL_RR2[3121].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23122 GTM_gtm_cls0.DPLL_RR2[3122].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23123 GTM_gtm_cls0.DPLL_RR2[3123].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23124 GTM_gtm_cls0.DPLL_RR2[3124].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23125 GTM_gtm_cls0.DPLL_RR2[3125].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23126 GTM_gtm_cls0.DPLL_RR2[3126].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23127 GTM_gtm_cls0.DPLL_RR2[3127].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23128 GTM_gtm_cls0.DPLL_RR2[3128].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23129 GTM_gtm_cls0.DPLL_RR2[3129].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23130 GTM_gtm_cls0.DPLL_RR2[3130].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23131 GTM_gtm_cls0.DPLL_RR2[3131].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23132 GTM_gtm_cls0.DPLL_RR2[3132].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23133 GTM_gtm_cls0.DPLL_RR2[3133].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23134 GTM_gtm_cls0.DPLL_RR2[3134].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23135 GTM_gtm_cls0.DPLL_RR2[3135].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23136 GTM_gtm_cls0.DPLL_RR2[3136].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23137 GTM_gtm_cls0.DPLL_RR2[3137].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23138 GTM_gtm_cls0.DPLL_RR2[3138].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23139 GTM_gtm_cls0.DPLL_RR2[3139].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23140 GTM_gtm_cls0.DPLL_RR2[3140].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23141 GTM_gtm_cls0.DPLL_RR2[3141].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23142 GTM_gtm_cls0.DPLL_RR2[3142].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23143 GTM_gtm_cls0.DPLL_RR2[3143].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23144 GTM_gtm_cls0.DPLL_RR2[3144].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23145 GTM_gtm_cls0.DPLL_RR2[3145].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23146 GTM_gtm_cls0.DPLL_RR2[3146].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23147 GTM_gtm_cls0.DPLL_RR2[3147].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23148 GTM_gtm_cls0.DPLL_RR2[3148].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23149 GTM_gtm_cls0.DPLL_RR2[3149].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23150 GTM_gtm_cls0.DPLL_RR2[3150].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23151 GTM_gtm_cls0.DPLL_RR2[3151].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23152 GTM_gtm_cls0.DPLL_RR2[3152].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23153 GTM_gtm_cls0.DPLL_RR2[3153].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23154 GTM_gtm_cls0.DPLL_RR2[3154].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23155 GTM_gtm_cls0.DPLL_RR2[3155].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23156 GTM_gtm_cls0.DPLL_RR2[3156].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23157 GTM_gtm_cls0.DPLL_RR2[3157].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23158 GTM_gtm_cls0.DPLL_RR2[3158].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23159 GTM_gtm_cls0.DPLL_RR2[3159].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23160 GTM_gtm_cls0.DPLL_RR2[3160].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23161 GTM_gtm_cls0.DPLL_RR2[3161].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23162 GTM_gtm_cls0.DPLL_RR2[3162].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23163 GTM_gtm_cls0.DPLL_RR2[3163].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23164 GTM_gtm_cls0.DPLL_RR2[3164].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23165 GTM_gtm_cls0.DPLL_RR2[3165].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23166 GTM_gtm_cls0.DPLL_RR2[3166].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23167 GTM_gtm_cls0.DPLL_RR2[3167].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23168 GTM_gtm_cls0.DPLL_RR2[3168].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23169 GTM_gtm_cls0.DPLL_RR2[3169].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23170 GTM_gtm_cls0.DPLL_RR2[3170].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23171 GTM_gtm_cls0.DPLL_RR2[3171].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23172 GTM_gtm_cls0.DPLL_RR2[3172].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23173 GTM_gtm_cls0.DPLL_RR2[3173].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23174 GTM_gtm_cls0.DPLL_RR2[3174].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23175 GTM_gtm_cls0.DPLL_RR2[3175].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23176 GTM_gtm_cls0.DPLL_RR2[3176].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23177 GTM_gtm_cls0.DPLL_RR2[3177].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23178 GTM_gtm_cls0.DPLL_RR2[3178].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23179 GTM_gtm_cls0.DPLL_RR2[3179].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23180 GTM_gtm_cls0.DPLL_RR2[3180].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23181 GTM_gtm_cls0.DPLL_RR2[3181].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23182 GTM_gtm_cls0.DPLL_RR2[3182].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23183 GTM_gtm_cls0.DPLL_RR2[3183].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23184 GTM_gtm_cls0.DPLL_RR2[3184].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23185 GTM_gtm_cls0.DPLL_RR2[3185].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23186 GTM_gtm_cls0.DPLL_RR2[3186].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23187 GTM_gtm_cls0.DPLL_RR2[3187].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23188 GTM_gtm_cls0.DPLL_RR2[3188].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23189 GTM_gtm_cls0.DPLL_RR2[3189].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23190 GTM_gtm_cls0.DPLL_RR2[3190].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23191 GTM_gtm_cls0.DPLL_RR2[3191].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23192 GTM_gtm_cls0.DPLL_RR2[3192].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23193 GTM_gtm_cls0.DPLL_RR2[3193].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23194 GTM_gtm_cls0.DPLL_RR2[3194].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23195 GTM_gtm_cls0.DPLL_RR2[3195].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23196 GTM_gtm_cls0.DPLL_RR2[3196].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23197 GTM_gtm_cls0.DPLL_RR2[3197].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23198 GTM_gtm_cls0.DPLL_RR2[3198].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23199 GTM_gtm_cls0.DPLL_RR2[3199].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23200 GTM_gtm_cls0.DPLL_RR2[3200].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23201 GTM_gtm_cls0.DPLL_RR2[3201].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23202 GTM_gtm_cls0.DPLL_RR2[3202].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23203 GTM_gtm_cls0.DPLL_RR2[3203].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23204 GTM_gtm_cls0.DPLL_RR2[3204].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23205 GTM_gtm_cls0.DPLL_RR2[3205].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23206 GTM_gtm_cls0.DPLL_RR2[3206].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23207 GTM_gtm_cls0.DPLL_RR2[3207].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23208 GTM_gtm_cls0.DPLL_RR2[3208].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23209 GTM_gtm_cls0.DPLL_RR2[3209].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23210 GTM_gtm_cls0.DPLL_RR2[3210].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23211 GTM_gtm_cls0.DPLL_RR2[3211].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23212 GTM_gtm_cls0.DPLL_RR2[3212].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23213 GTM_gtm_cls0.DPLL_RR2[3213].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23214 GTM_gtm_cls0.DPLL_RR2[3214].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23215 GTM_gtm_cls0.DPLL_RR2[3215].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23216 GTM_gtm_cls0.DPLL_RR2[3216].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23217 GTM_gtm_cls0.DPLL_RR2[3217].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23218 GTM_gtm_cls0.DPLL_RR2[3218].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23219 GTM_gtm_cls0.DPLL_RR2[3219].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23220 GTM_gtm_cls0.DPLL_RR2[3220].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23221 GTM_gtm_cls0.DPLL_RR2[3221].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23222 GTM_gtm_cls0.DPLL_RR2[3222].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23223 GTM_gtm_cls0.DPLL_RR2[3223].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23224 GTM_gtm_cls0.DPLL_RR2[3224].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23225 GTM_gtm_cls0.DPLL_RR2[3225].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23226 GTM_gtm_cls0.DPLL_RR2[3226].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23227 GTM_gtm_cls0.DPLL_RR2[3227].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23228 GTM_gtm_cls0.DPLL_RR2[3228].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23229 GTM_gtm_cls0.DPLL_RR2[3229].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23230 GTM_gtm_cls0.DPLL_RR2[3230].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23231 GTM_gtm_cls0.DPLL_RR2[3231].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23232 GTM_gtm_cls0.DPLL_RR2[3232].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23233 GTM_gtm_cls0.DPLL_RR2[3233].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23234 GTM_gtm_cls0.DPLL_RR2[3234].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23235 GTM_gtm_cls0.DPLL_RR2[3235].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23236 GTM_gtm_cls0.DPLL_RR2[3236].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23237 GTM_gtm_cls0.DPLL_RR2[3237].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23238 GTM_gtm_cls0.DPLL_RR2[3238].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23239 GTM_gtm_cls0.DPLL_RR2[3239].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23240 GTM_gtm_cls0.DPLL_RR2[3240].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23241 GTM_gtm_cls0.DPLL_RR2[3241].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23242 GTM_gtm_cls0.DPLL_RR2[3242].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23243 GTM_gtm_cls0.DPLL_RR2[3243].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23244 GTM_gtm_cls0.DPLL_RR2[3244].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23245 GTM_gtm_cls0.DPLL_RR2[3245].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23246 GTM_gtm_cls0.DPLL_RR2[3246].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23247 GTM_gtm_cls0.DPLL_RR2[3247].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23248 GTM_gtm_cls0.DPLL_RR2[3248].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23249 GTM_gtm_cls0.DPLL_RR2[3249].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23250 GTM_gtm_cls0.DPLL_RR2[3250].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23251 GTM_gtm_cls0.DPLL_RR2[3251].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23252 GTM_gtm_cls0.DPLL_RR2[3252].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23253 GTM_gtm_cls0.DPLL_RR2[3253].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23254 GTM_gtm_cls0.DPLL_RR2[3254].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23255 GTM_gtm_cls0.DPLL_RR2[3255].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23256 GTM_gtm_cls0.DPLL_RR2[3256].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23257 GTM_gtm_cls0.DPLL_RR2[3257].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23258 GTM_gtm_cls0.DPLL_RR2[3258].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23259 GTM_gtm_cls0.DPLL_RR2[3259].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23260 GTM_gtm_cls0.DPLL_RR2[3260].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23261 GTM_gtm_cls0.DPLL_RR2[3261].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23262 GTM_gtm_cls0.DPLL_RR2[3262].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23263 GTM_gtm_cls0.DPLL_RR2[3263].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23264 GTM_gtm_cls0.DPLL_RR2[3264].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23265 GTM_gtm_cls0.DPLL_RR2[3265].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23266 GTM_gtm_cls0.DPLL_RR2[3266].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23267 GTM_gtm_cls0.DPLL_RR2[3267].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23268 GTM_gtm_cls0.DPLL_RR2[3268].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23269 GTM_gtm_cls0.DPLL_RR2[3269].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23270 GTM_gtm_cls0.DPLL_RR2[3270].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23271 GTM_gtm_cls0.DPLL_RR2[3271].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23272 GTM_gtm_cls0.DPLL_RR2[3272].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23273 GTM_gtm_cls0.DPLL_RR2[3273].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23274 GTM_gtm_cls0.DPLL_RR2[3274].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23275 GTM_gtm_cls0.DPLL_RR2[3275].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23276 GTM_gtm_cls0.DPLL_RR2[3276].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23277 GTM_gtm_cls0.DPLL_RR2[3277].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23278 GTM_gtm_cls0.DPLL_RR2[3278].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23279 GTM_gtm_cls0.DPLL_RR2[3279].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23280 GTM_gtm_cls0.DPLL_RR2[3280].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23281 GTM_gtm_cls0.DPLL_RR2[3281].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23282 GTM_gtm_cls0.DPLL_RR2[3282].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23283 GTM_gtm_cls0.DPLL_RR2[3283].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23284 GTM_gtm_cls0.DPLL_RR2[3284].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23285 GTM_gtm_cls0.DPLL_RR2[3285].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23286 GTM_gtm_cls0.DPLL_RR2[3286].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23287 GTM_gtm_cls0.DPLL_RR2[3287].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23288 GTM_gtm_cls0.DPLL_RR2[3288].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23289 GTM_gtm_cls0.DPLL_RR2[3289].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23290 GTM_gtm_cls0.DPLL_RR2[3290].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23291 GTM_gtm_cls0.DPLL_RR2[3291].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23292 GTM_gtm_cls0.DPLL_RR2[3292].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23293 GTM_gtm_cls0.DPLL_RR2[3293].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23294 GTM_gtm_cls0.DPLL_RR2[3294].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23295 GTM_gtm_cls0.DPLL_RR2[3295].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23296 GTM_gtm_cls0.DPLL_RR2[3296].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23297 GTM_gtm_cls0.DPLL_RR2[3297].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23298 GTM_gtm_cls0.DPLL_RR2[3298].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23299 GTM_gtm_cls0.DPLL_RR2[3299].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23300 GTM_gtm_cls0.DPLL_RR2[3300].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23301 GTM_gtm_cls0.DPLL_RR2[3301].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23302 GTM_gtm_cls0.DPLL_RR2[3302].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23303 GTM_gtm_cls0.DPLL_RR2[3303].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23304 GTM_gtm_cls0.DPLL_RR2[3304].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23305 GTM_gtm_cls0.DPLL_RR2[3305].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23306 GTM_gtm_cls0.DPLL_RR2[3306].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23307 GTM_gtm_cls0.DPLL_RR2[3307].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23308 GTM_gtm_cls0.DPLL_RR2[3308].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23309 GTM_gtm_cls0.DPLL_RR2[3309].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23310 GTM_gtm_cls0.DPLL_RR2[3310].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23311 GTM_gtm_cls0.DPLL_RR2[3311].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23312 GTM_gtm_cls0.DPLL_RR2[3312].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23313 GTM_gtm_cls0.DPLL_RR2[3313].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23314 GTM_gtm_cls0.DPLL_RR2[3314].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23315 GTM_gtm_cls0.DPLL_RR2[3315].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23316 GTM_gtm_cls0.DPLL_RR2[3316].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23317 GTM_gtm_cls0.DPLL_RR2[3317].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23318 GTM_gtm_cls0.DPLL_RR2[3318].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23319 GTM_gtm_cls0.DPLL_RR2[3319].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23320 GTM_gtm_cls0.DPLL_RR2[3320].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23321 GTM_gtm_cls0.DPLL_RR2[3321].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23322 GTM_gtm_cls0.DPLL_RR2[3322].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23323 GTM_gtm_cls0.DPLL_RR2[3323].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23324 GTM_gtm_cls0.DPLL_RR2[3324].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23325 GTM_gtm_cls0.DPLL_RR2[3325].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23326 GTM_gtm_cls0.DPLL_RR2[3326].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23327 GTM_gtm_cls0.DPLL_RR2[3327].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23328 GTM_gtm_cls0.DPLL_RR2[3328].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23329 GTM_gtm_cls0.DPLL_RR2[3329].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23330 GTM_gtm_cls0.DPLL_RR2[3330].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23331 GTM_gtm_cls0.DPLL_RR2[3331].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23332 GTM_gtm_cls0.DPLL_RR2[3332].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23333 GTM_gtm_cls0.DPLL_RR2[3333].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23334 GTM_gtm_cls0.DPLL_RR2[3334].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23335 GTM_gtm_cls0.DPLL_RR2[3335].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23336 GTM_gtm_cls0.DPLL_RR2[3336].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23337 GTM_gtm_cls0.DPLL_RR2[3337].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23338 GTM_gtm_cls0.DPLL_RR2[3338].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23339 GTM_gtm_cls0.DPLL_RR2[3339].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23340 GTM_gtm_cls0.DPLL_RR2[3340].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23341 GTM_gtm_cls0.DPLL_RR2[3341].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23342 GTM_gtm_cls0.DPLL_RR2[3342].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23343 GTM_gtm_cls0.DPLL_RR2[3343].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23344 GTM_gtm_cls0.DPLL_RR2[3344].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23345 GTM_gtm_cls0.DPLL_RR2[3345].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23346 GTM_gtm_cls0.DPLL_RR2[3346].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23347 GTM_gtm_cls0.DPLL_RR2[3347].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23348 GTM_gtm_cls0.DPLL_RR2[3348].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23349 GTM_gtm_cls0.DPLL_RR2[3349].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23350 GTM_gtm_cls0.DPLL_RR2[3350].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23351 GTM_gtm_cls0.DPLL_RR2[3351].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23352 GTM_gtm_cls0.DPLL_RR2[3352].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23353 GTM_gtm_cls0.DPLL_RR2[3353].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23354 GTM_gtm_cls0.DPLL_RR2[3354].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23355 GTM_gtm_cls0.DPLL_RR2[3355].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23356 GTM_gtm_cls0.DPLL_RR2[3356].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23357 GTM_gtm_cls0.DPLL_RR2[3357].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23358 GTM_gtm_cls0.DPLL_RR2[3358].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23359 GTM_gtm_cls0.DPLL_RR2[3359].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23360 GTM_gtm_cls0.DPLL_RR2[3360].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23361 GTM_gtm_cls0.DPLL_RR2[3361].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23362 GTM_gtm_cls0.DPLL_RR2[3362].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23363 GTM_gtm_cls0.DPLL_RR2[3363].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23364 GTM_gtm_cls0.DPLL_RR2[3364].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23365 GTM_gtm_cls0.DPLL_RR2[3365].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23366 GTM_gtm_cls0.DPLL_RR2[3366].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23367 GTM_gtm_cls0.DPLL_RR2[3367].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23368 GTM_gtm_cls0.DPLL_RR2[3368].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23369 GTM_gtm_cls0.DPLL_RR2[3369].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23370 GTM_gtm_cls0.DPLL_RR2[3370].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23371 GTM_gtm_cls0.DPLL_RR2[3371].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23372 GTM_gtm_cls0.DPLL_RR2[3372].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23373 GTM_gtm_cls0.DPLL_RR2[3373].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23374 GTM_gtm_cls0.DPLL_RR2[3374].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23375 GTM_gtm_cls0.DPLL_RR2[3375].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23376 GTM_gtm_cls0.DPLL_RR2[3376].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23377 GTM_gtm_cls0.DPLL_RR2[3377].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23378 GTM_gtm_cls0.DPLL_RR2[3378].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23379 GTM_gtm_cls0.DPLL_RR2[3379].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23380 GTM_gtm_cls0.DPLL_RR2[3380].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23381 GTM_gtm_cls0.DPLL_RR2[3381].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23382 GTM_gtm_cls0.DPLL_RR2[3382].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23383 GTM_gtm_cls0.DPLL_RR2[3383].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23384 GTM_gtm_cls0.DPLL_RR2[3384].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23385 GTM_gtm_cls0.DPLL_RR2[3385].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23386 GTM_gtm_cls0.DPLL_RR2[3386].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23387 GTM_gtm_cls0.DPLL_RR2[3387].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23388 GTM_gtm_cls0.DPLL_RR2[3388].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23389 GTM_gtm_cls0.DPLL_RR2[3389].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23390 GTM_gtm_cls0.DPLL_RR2[3390].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23391 GTM_gtm_cls0.DPLL_RR2[3391].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23392 GTM_gtm_cls0.DPLL_RR2[3392].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23393 GTM_gtm_cls0.DPLL_RR2[3393].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23394 GTM_gtm_cls0.DPLL_RR2[3394].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23395 GTM_gtm_cls0.DPLL_RR2[3395].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23396 GTM_gtm_cls0.DPLL_RR2[3396].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23397 GTM_gtm_cls0.DPLL_RR2[3397].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23398 GTM_gtm_cls0.DPLL_RR2[3398].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23399 GTM_gtm_cls0.DPLL_RR2[3399].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23400 GTM_gtm_cls0.DPLL_RR2[3400].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23401 GTM_gtm_cls0.DPLL_RR2[3401].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23402 GTM_gtm_cls0.DPLL_RR2[3402].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23403 GTM_gtm_cls0.DPLL_RR2[3403].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23404 GTM_gtm_cls0.DPLL_RR2[3404].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23405 GTM_gtm_cls0.DPLL_RR2[3405].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23406 GTM_gtm_cls0.DPLL_RR2[3406].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23407 GTM_gtm_cls0.DPLL_RR2[3407].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23408 GTM_gtm_cls0.DPLL_RR2[3408].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23409 GTM_gtm_cls0.DPLL_RR2[3409].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23410 GTM_gtm_cls0.DPLL_RR2[3410].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23411 GTM_gtm_cls0.DPLL_RR2[3411].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23412 GTM_gtm_cls0.DPLL_RR2[3412].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23413 GTM_gtm_cls0.DPLL_RR2[3413].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23414 GTM_gtm_cls0.DPLL_RR2[3414].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23415 GTM_gtm_cls0.DPLL_RR2[3415].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23416 GTM_gtm_cls0.DPLL_RR2[3416].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23417 GTM_gtm_cls0.DPLL_RR2[3417].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23418 GTM_gtm_cls0.DPLL_RR2[3418].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23419 GTM_gtm_cls0.DPLL_RR2[3419].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23420 GTM_gtm_cls0.DPLL_RR2[3420].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23421 GTM_gtm_cls0.DPLL_RR2[3421].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23422 GTM_gtm_cls0.DPLL_RR2[3422].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23423 GTM_gtm_cls0.DPLL_RR2[3423].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23424 GTM_gtm_cls0.DPLL_RR2[3424].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23425 GTM_gtm_cls0.DPLL_RR2[3425].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23426 GTM_gtm_cls0.DPLL_RR2[3426].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23427 GTM_gtm_cls0.DPLL_RR2[3427].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23428 GTM_gtm_cls0.DPLL_RR2[3428].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23429 GTM_gtm_cls0.DPLL_RR2[3429].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23430 GTM_gtm_cls0.DPLL_RR2[3430].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23431 GTM_gtm_cls0.DPLL_RR2[3431].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23432 GTM_gtm_cls0.DPLL_RR2[3432].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23433 GTM_gtm_cls0.DPLL_RR2[3433].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23434 GTM_gtm_cls0.DPLL_RR2[3434].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23435 GTM_gtm_cls0.DPLL_RR2[3435].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23436 GTM_gtm_cls0.DPLL_RR2[3436].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23437 GTM_gtm_cls0.DPLL_RR2[3437].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23438 GTM_gtm_cls0.DPLL_RR2[3438].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23439 GTM_gtm_cls0.DPLL_RR2[3439].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23440 GTM_gtm_cls0.DPLL_RR2[3440].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23441 GTM_gtm_cls0.DPLL_RR2[3441].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23442 GTM_gtm_cls0.DPLL_RR2[3442].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23443 GTM_gtm_cls0.DPLL_RR2[3443].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23444 GTM_gtm_cls0.DPLL_RR2[3444].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23445 GTM_gtm_cls0.DPLL_RR2[3445].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23446 GTM_gtm_cls0.DPLL_RR2[3446].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23447 GTM_gtm_cls0.DPLL_RR2[3447].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23448 GTM_gtm_cls0.DPLL_RR2[3448].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23449 GTM_gtm_cls0.DPLL_RR2[3449].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23450 GTM_gtm_cls0.DPLL_RR2[3450].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23451 GTM_gtm_cls0.DPLL_RR2[3451].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23452 GTM_gtm_cls0.DPLL_RR2[3452].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23453 GTM_gtm_cls0.DPLL_RR2[3453].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23454 GTM_gtm_cls0.DPLL_RR2[3454].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23455 GTM_gtm_cls0.DPLL_RR2[3455].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23456 GTM_gtm_cls0.DPLL_RR2[3456].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23457 GTM_gtm_cls0.DPLL_RR2[3457].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23458 GTM_gtm_cls0.DPLL_RR2[3458].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23459 GTM_gtm_cls0.DPLL_RR2[3459].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23460 GTM_gtm_cls0.DPLL_RR2[3460].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23461 GTM_gtm_cls0.DPLL_RR2[3461].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23462 GTM_gtm_cls0.DPLL_RR2[3462].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23463 GTM_gtm_cls0.DPLL_RR2[3463].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23464 GTM_gtm_cls0.DPLL_RR2[3464].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23465 GTM_gtm_cls0.DPLL_RR2[3465].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23466 GTM_gtm_cls0.DPLL_RR2[3466].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23467 GTM_gtm_cls0.DPLL_RR2[3467].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23468 GTM_gtm_cls0.DPLL_RR2[3468].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23469 GTM_gtm_cls0.DPLL_RR2[3469].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23470 GTM_gtm_cls0.DPLL_RR2[3470].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23471 GTM_gtm_cls0.DPLL_RR2[3471].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23472 GTM_gtm_cls0.DPLL_RR2[3472].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23473 GTM_gtm_cls0.DPLL_RR2[3473].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23474 GTM_gtm_cls0.DPLL_RR2[3474].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23475 GTM_gtm_cls0.DPLL_RR2[3475].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23476 GTM_gtm_cls0.DPLL_RR2[3476].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23477 GTM_gtm_cls0.DPLL_RR2[3477].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23478 GTM_gtm_cls0.DPLL_RR2[3478].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23479 GTM_gtm_cls0.DPLL_RR2[3479].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23480 GTM_gtm_cls0.DPLL_RR2[3480].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23481 GTM_gtm_cls0.DPLL_RR2[3481].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23482 GTM_gtm_cls0.DPLL_RR2[3482].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23483 GTM_gtm_cls0.DPLL_RR2[3483].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23484 GTM_gtm_cls0.DPLL_RR2[3484].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23485 GTM_gtm_cls0.DPLL_RR2[3485].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23486 GTM_gtm_cls0.DPLL_RR2[3486].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23487 GTM_gtm_cls0.DPLL_RR2[3487].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23488 GTM_gtm_cls0.DPLL_RR2[3488].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23489 GTM_gtm_cls0.DPLL_RR2[3489].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23490 GTM_gtm_cls0.DPLL_RR2[3490].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23491 GTM_gtm_cls0.DPLL_RR2[3491].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23492 GTM_gtm_cls0.DPLL_RR2[3492].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23493 GTM_gtm_cls0.DPLL_RR2[3493].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23494 GTM_gtm_cls0.DPLL_RR2[3494].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23495 GTM_gtm_cls0.DPLL_RR2[3495].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23496 GTM_gtm_cls0.DPLL_RR2[3496].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23497 GTM_gtm_cls0.DPLL_RR2[3497].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23498 GTM_gtm_cls0.DPLL_RR2[3498].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23499 GTM_gtm_cls0.DPLL_RR2[3499].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23500 GTM_gtm_cls0.DPLL_RR2[3500].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23501 GTM_gtm_cls0.DPLL_RR2[3501].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23502 GTM_gtm_cls0.DPLL_RR2[3502].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23503 GTM_gtm_cls0.DPLL_RR2[3503].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23504 GTM_gtm_cls0.DPLL_RR2[3504].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23505 GTM_gtm_cls0.DPLL_RR2[3505].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23506 GTM_gtm_cls0.DPLL_RR2[3506].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23507 GTM_gtm_cls0.DPLL_RR2[3507].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23508 GTM_gtm_cls0.DPLL_RR2[3508].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23509 GTM_gtm_cls0.DPLL_RR2[3509].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23510 GTM_gtm_cls0.DPLL_RR2[3510].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23511 GTM_gtm_cls0.DPLL_RR2[3511].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23512 GTM_gtm_cls0.DPLL_RR2[3512].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23513 GTM_gtm_cls0.DPLL_RR2[3513].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23514 GTM_gtm_cls0.DPLL_RR2[3514].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23515 GTM_gtm_cls0.DPLL_RR2[3515].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23516 GTM_gtm_cls0.DPLL_RR2[3516].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23517 GTM_gtm_cls0.DPLL_RR2[3517].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23518 GTM_gtm_cls0.DPLL_RR2[3518].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23519 GTM_gtm_cls0.DPLL_RR2[3519].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23520 GTM_gtm_cls0.DPLL_RR2[3520].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23521 GTM_gtm_cls0.DPLL_RR2[3521].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23522 GTM_gtm_cls0.DPLL_RR2[3522].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23523 GTM_gtm_cls0.DPLL_RR2[3523].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23524 GTM_gtm_cls0.DPLL_RR2[3524].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23525 GTM_gtm_cls0.DPLL_RR2[3525].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23526 GTM_gtm_cls0.DPLL_RR2[3526].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23527 GTM_gtm_cls0.DPLL_RR2[3527].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23528 GTM_gtm_cls0.DPLL_RR2[3528].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23529 GTM_gtm_cls0.DPLL_RR2[3529].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23530 GTM_gtm_cls0.DPLL_RR2[3530].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23531 GTM_gtm_cls0.DPLL_RR2[3531].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23532 GTM_gtm_cls0.DPLL_RR2[3532].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23533 GTM_gtm_cls0.DPLL_RR2[3533].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23534 GTM_gtm_cls0.DPLL_RR2[3534].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23535 GTM_gtm_cls0.DPLL_RR2[3535].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23536 GTM_gtm_cls0.DPLL_RR2[3536].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23537 GTM_gtm_cls0.DPLL_RR2[3537].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23538 GTM_gtm_cls0.DPLL_RR2[3538].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23539 GTM_gtm_cls0.DPLL_RR2[3539].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23540 GTM_gtm_cls0.DPLL_RR2[3540].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23541 GTM_gtm_cls0.DPLL_RR2[3541].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23542 GTM_gtm_cls0.DPLL_RR2[3542].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23543 GTM_gtm_cls0.DPLL_RR2[3543].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23544 GTM_gtm_cls0.DPLL_RR2[3544].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23545 GTM_gtm_cls0.DPLL_RR2[3545].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23546 GTM_gtm_cls0.DPLL_RR2[3546].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23547 GTM_gtm_cls0.DPLL_RR2[3547].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23548 GTM_gtm_cls0.DPLL_RR2[3548].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23549 GTM_gtm_cls0.DPLL_RR2[3549].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23550 GTM_gtm_cls0.DPLL_RR2[3550].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23551 GTM_gtm_cls0.DPLL_RR2[3551].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23552 GTM_gtm_cls0.DPLL_RR2[3552].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23553 GTM_gtm_cls0.DPLL_RR2[3553].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23554 GTM_gtm_cls0.DPLL_RR2[3554].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23555 GTM_gtm_cls0.DPLL_RR2[3555].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23556 GTM_gtm_cls0.DPLL_RR2[3556].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23557 GTM_gtm_cls0.DPLL_RR2[3557].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23558 GTM_gtm_cls0.DPLL_RR2[3558].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23559 GTM_gtm_cls0.DPLL_RR2[3559].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23560 GTM_gtm_cls0.DPLL_RR2[3560].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23561 GTM_gtm_cls0.DPLL_RR2[3561].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23562 GTM_gtm_cls0.DPLL_RR2[3562].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23563 GTM_gtm_cls0.DPLL_RR2[3563].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23564 GTM_gtm_cls0.DPLL_RR2[3564].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23565 GTM_gtm_cls0.DPLL_RR2[3565].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23566 GTM_gtm_cls0.DPLL_RR2[3566].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23567 GTM_gtm_cls0.DPLL_RR2[3567].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23568 GTM_gtm_cls0.DPLL_RR2[3568].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23569 GTM_gtm_cls0.DPLL_RR2[3569].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23570 GTM_gtm_cls0.DPLL_RR2[3570].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23571 GTM_gtm_cls0.DPLL_RR2[3571].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23572 GTM_gtm_cls0.DPLL_RR2[3572].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23573 GTM_gtm_cls0.DPLL_RR2[3573].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23574 GTM_gtm_cls0.DPLL_RR2[3574].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23575 GTM_gtm_cls0.DPLL_RR2[3575].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23576 GTM_gtm_cls0.DPLL_RR2[3576].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23577 GTM_gtm_cls0.DPLL_RR2[3577].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23578 GTM_gtm_cls0.DPLL_RR2[3578].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23579 GTM_gtm_cls0.DPLL_RR2[3579].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23580 GTM_gtm_cls0.DPLL_RR2[3580].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23581 GTM_gtm_cls0.DPLL_RR2[3581].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23582 GTM_gtm_cls0.DPLL_RR2[3582].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23583 GTM_gtm_cls0.DPLL_RR2[3583].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23584 GTM_gtm_cls0.DPLL_RR2[3584].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23585 GTM_gtm_cls0.DPLL_RR2[3585].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23586 GTM_gtm_cls0.DPLL_RR2[3586].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23587 GTM_gtm_cls0.DPLL_RR2[3587].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23588 GTM_gtm_cls0.DPLL_RR2[3588].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23589 GTM_gtm_cls0.DPLL_RR2[3589].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23590 GTM_gtm_cls0.DPLL_RR2[3590].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23591 GTM_gtm_cls0.DPLL_RR2[3591].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23592 GTM_gtm_cls0.DPLL_RR2[3592].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23593 GTM_gtm_cls0.DPLL_RR2[3593].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23594 GTM_gtm_cls0.DPLL_RR2[3594].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23595 GTM_gtm_cls0.DPLL_RR2[3595].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23596 GTM_gtm_cls0.DPLL_RR2[3596].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23597 GTM_gtm_cls0.DPLL_RR2[3597].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23598 GTM_gtm_cls0.DPLL_RR2[3598].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23599 GTM_gtm_cls0.DPLL_RR2[3599].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23600 GTM_gtm_cls0.DPLL_RR2[3600].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23601 GTM_gtm_cls0.DPLL_RR2[3601].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23602 GTM_gtm_cls0.DPLL_RR2[3602].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23603 GTM_gtm_cls0.DPLL_RR2[3603].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23604 GTM_gtm_cls0.DPLL_RR2[3604].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23605 GTM_gtm_cls0.DPLL_RR2[3605].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23606 GTM_gtm_cls0.DPLL_RR2[3606].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23607 GTM_gtm_cls0.DPLL_RR2[3607].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23608 GTM_gtm_cls0.DPLL_RR2[3608].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23609 GTM_gtm_cls0.DPLL_RR2[3609].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23610 GTM_gtm_cls0.DPLL_RR2[3610].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23611 GTM_gtm_cls0.DPLL_RR2[3611].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23612 GTM_gtm_cls0.DPLL_RR2[3612].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23613 GTM_gtm_cls0.DPLL_RR2[3613].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23614 GTM_gtm_cls0.DPLL_RR2[3614].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23615 GTM_gtm_cls0.DPLL_RR2[3615].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23616 GTM_gtm_cls0.DPLL_RR2[3616].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23617 GTM_gtm_cls0.DPLL_RR2[3617].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23618 GTM_gtm_cls0.DPLL_RR2[3618].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23619 GTM_gtm_cls0.DPLL_RR2[3619].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23620 GTM_gtm_cls0.DPLL_RR2[3620].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23621 GTM_gtm_cls0.DPLL_RR2[3621].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23622 GTM_gtm_cls0.DPLL_RR2[3622].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23623 GTM_gtm_cls0.DPLL_RR2[3623].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23624 GTM_gtm_cls0.DPLL_RR2[3624].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23625 GTM_gtm_cls0.DPLL_RR2[3625].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23626 GTM_gtm_cls0.DPLL_RR2[3626].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23627 GTM_gtm_cls0.DPLL_RR2[3627].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23628 GTM_gtm_cls0.DPLL_RR2[3628].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23629 GTM_gtm_cls0.DPLL_RR2[3629].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23630 GTM_gtm_cls0.DPLL_RR2[3630].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23631 GTM_gtm_cls0.DPLL_RR2[3631].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23632 GTM_gtm_cls0.DPLL_RR2[3632].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23633 GTM_gtm_cls0.DPLL_RR2[3633].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23634 GTM_gtm_cls0.DPLL_RR2[3634].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23635 GTM_gtm_cls0.DPLL_RR2[3635].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23636 GTM_gtm_cls0.DPLL_RR2[3636].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23637 GTM_gtm_cls0.DPLL_RR2[3637].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23638 GTM_gtm_cls0.DPLL_RR2[3638].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23639 GTM_gtm_cls0.DPLL_RR2[3639].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23640 GTM_gtm_cls0.DPLL_RR2[3640].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23641 GTM_gtm_cls0.DPLL_RR2[3641].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23642 GTM_gtm_cls0.DPLL_RR2[3642].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23643 GTM_gtm_cls0.DPLL_RR2[3643].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23644 GTM_gtm_cls0.DPLL_RR2[3644].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23645 GTM_gtm_cls0.DPLL_RR2[3645].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23646 GTM_gtm_cls0.DPLL_RR2[3646].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23647 GTM_gtm_cls0.DPLL_RR2[3647].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23648 GTM_gtm_cls0.DPLL_RR2[3648].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23649 GTM_gtm_cls0.DPLL_RR2[3649].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23650 GTM_gtm_cls0.DPLL_RR2[3650].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23651 GTM_gtm_cls0.DPLL_RR2[3651].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23652 GTM_gtm_cls0.DPLL_RR2[3652].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23653 GTM_gtm_cls0.DPLL_RR2[3653].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23654 GTM_gtm_cls0.DPLL_RR2[3654].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23655 GTM_gtm_cls0.DPLL_RR2[3655].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23656 GTM_gtm_cls0.DPLL_RR2[3656].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23657 GTM_gtm_cls0.DPLL_RR2[3657].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23658 GTM_gtm_cls0.DPLL_RR2[3658].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23659 GTM_gtm_cls0.DPLL_RR2[3659].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23660 GTM_gtm_cls0.DPLL_RR2[3660].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23661 GTM_gtm_cls0.DPLL_RR2[3661].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23662 GTM_gtm_cls0.DPLL_RR2[3662].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23663 GTM_gtm_cls0.DPLL_RR2[3663].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23664 GTM_gtm_cls0.DPLL_RR2[3664].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23665 GTM_gtm_cls0.DPLL_RR2[3665].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23666 GTM_gtm_cls0.DPLL_RR2[3666].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23667 GTM_gtm_cls0.DPLL_RR2[3667].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23668 GTM_gtm_cls0.DPLL_RR2[3668].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23669 GTM_gtm_cls0.DPLL_RR2[3669].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23670 GTM_gtm_cls0.DPLL_RR2[3670].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23671 GTM_gtm_cls0.DPLL_RR2[3671].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23672 GTM_gtm_cls0.DPLL_RR2[3672].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23673 GTM_gtm_cls0.DPLL_RR2[3673].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23674 GTM_gtm_cls0.DPLL_RR2[3674].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23675 GTM_gtm_cls0.DPLL_RR2[3675].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23676 GTM_gtm_cls0.DPLL_RR2[3676].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23677 GTM_gtm_cls0.DPLL_RR2[3677].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23678 GTM_gtm_cls0.DPLL_RR2[3678].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23679 GTM_gtm_cls0.DPLL_RR2[3679].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23680 GTM_gtm_cls0.DPLL_RR2[3680].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23681 GTM_gtm_cls0.DPLL_RR2[3681].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23682 GTM_gtm_cls0.DPLL_RR2[3682].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23683 GTM_gtm_cls0.DPLL_RR2[3683].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23684 GTM_gtm_cls0.DPLL_RR2[3684].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23685 GTM_gtm_cls0.DPLL_RR2[3685].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23686 GTM_gtm_cls0.DPLL_RR2[3686].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23687 GTM_gtm_cls0.DPLL_RR2[3687].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23688 GTM_gtm_cls0.DPLL_RR2[3688].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23689 GTM_gtm_cls0.DPLL_RR2[3689].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23690 GTM_gtm_cls0.DPLL_RR2[3690].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23691 GTM_gtm_cls0.DPLL_RR2[3691].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23692 GTM_gtm_cls0.DPLL_RR2[3692].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23693 GTM_gtm_cls0.DPLL_RR2[3693].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23694 GTM_gtm_cls0.DPLL_RR2[3694].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23695 GTM_gtm_cls0.DPLL_RR2[3695].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23696 GTM_gtm_cls0.DPLL_RR2[3696].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23697 GTM_gtm_cls0.DPLL_RR2[3697].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23698 GTM_gtm_cls0.DPLL_RR2[3698].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23699 GTM_gtm_cls0.DPLL_RR2[3699].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23700 GTM_gtm_cls0.DPLL_RR2[3700].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23701 GTM_gtm_cls0.DPLL_RR2[3701].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23702 GTM_gtm_cls0.DPLL_RR2[3702].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23703 GTM_gtm_cls0.DPLL_RR2[3703].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23704 GTM_gtm_cls0.DPLL_RR2[3704].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23705 GTM_gtm_cls0.DPLL_RR2[3705].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23706 GTM_gtm_cls0.DPLL_RR2[3706].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23707 GTM_gtm_cls0.DPLL_RR2[3707].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23708 GTM_gtm_cls0.DPLL_RR2[3708].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23709 GTM_gtm_cls0.DPLL_RR2[3709].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23710 GTM_gtm_cls0.DPLL_RR2[3710].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23711 GTM_gtm_cls0.DPLL_RR2[3711].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23712 GTM_gtm_cls0.DPLL_RR2[3712].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23713 GTM_gtm_cls0.DPLL_RR2[3713].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23714 GTM_gtm_cls0.DPLL_RR2[3714].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23715 GTM_gtm_cls0.DPLL_RR2[3715].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23716 GTM_gtm_cls0.DPLL_RR2[3716].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23717 GTM_gtm_cls0.DPLL_RR2[3717].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23718 GTM_gtm_cls0.DPLL_RR2[3718].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23719 GTM_gtm_cls0.DPLL_RR2[3719].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23720 GTM_gtm_cls0.DPLL_RR2[3720].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23721 GTM_gtm_cls0.DPLL_RR2[3721].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23722 GTM_gtm_cls0.DPLL_RR2[3722].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23723 GTM_gtm_cls0.DPLL_RR2[3723].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23724 GTM_gtm_cls0.DPLL_RR2[3724].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23725 GTM_gtm_cls0.DPLL_RR2[3725].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23726 GTM_gtm_cls0.DPLL_RR2[3726].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23727 GTM_gtm_cls0.DPLL_RR2[3727].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23728 GTM_gtm_cls0.DPLL_RR2[3728].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23729 GTM_gtm_cls0.DPLL_RR2[3729].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23730 GTM_gtm_cls0.DPLL_RR2[3730].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23731 GTM_gtm_cls0.DPLL_RR2[3731].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23732 GTM_gtm_cls0.DPLL_RR2[3732].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23733 GTM_gtm_cls0.DPLL_RR2[3733].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23734 GTM_gtm_cls0.DPLL_RR2[3734].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23735 GTM_gtm_cls0.DPLL_RR2[3735].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23736 GTM_gtm_cls0.DPLL_RR2[3736].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23737 GTM_gtm_cls0.DPLL_RR2[3737].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23738 GTM_gtm_cls0.DPLL_RR2[3738].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23739 GTM_gtm_cls0.DPLL_RR2[3739].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23740 GTM_gtm_cls0.DPLL_RR2[3740].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23741 GTM_gtm_cls0.DPLL_RR2[3741].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23742 GTM_gtm_cls0.DPLL_RR2[3742].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23743 GTM_gtm_cls0.DPLL_RR2[3743].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23744 GTM_gtm_cls0.DPLL_RR2[3744].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23745 GTM_gtm_cls0.DPLL_RR2[3745].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23746 GTM_gtm_cls0.DPLL_RR2[3746].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23747 GTM_gtm_cls0.DPLL_RR2[3747].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23748 GTM_gtm_cls0.DPLL_RR2[3748].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23749 GTM_gtm_cls0.DPLL_RR2[3749].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23750 GTM_gtm_cls0.DPLL_RR2[3750].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23751 GTM_gtm_cls0.DPLL_RR2[3751].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23752 GTM_gtm_cls0.DPLL_RR2[3752].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23753 GTM_gtm_cls0.DPLL_RR2[3753].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23754 GTM_gtm_cls0.DPLL_RR2[3754].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23755 GTM_gtm_cls0.DPLL_RR2[3755].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23756 GTM_gtm_cls0.DPLL_RR2[3756].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23757 GTM_gtm_cls0.DPLL_RR2[3757].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23758 GTM_gtm_cls0.DPLL_RR2[3758].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23759 GTM_gtm_cls0.DPLL_RR2[3759].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23760 GTM_gtm_cls0.DPLL_RR2[3760].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23761 GTM_gtm_cls0.DPLL_RR2[3761].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23762 GTM_gtm_cls0.DPLL_RR2[3762].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23763 GTM_gtm_cls0.DPLL_RR2[3763].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23764 GTM_gtm_cls0.DPLL_RR2[3764].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23765 GTM_gtm_cls0.DPLL_RR2[3765].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23766 GTM_gtm_cls0.DPLL_RR2[3766].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23767 GTM_gtm_cls0.DPLL_RR2[3767].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23768 GTM_gtm_cls0.DPLL_RR2[3768].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23769 GTM_gtm_cls0.DPLL_RR2[3769].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23770 GTM_gtm_cls0.DPLL_RR2[3770].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23771 GTM_gtm_cls0.DPLL_RR2[3771].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23772 GTM_gtm_cls0.DPLL_RR2[3772].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23773 GTM_gtm_cls0.DPLL_RR2[3773].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23774 GTM_gtm_cls0.DPLL_RR2[3774].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23775 GTM_gtm_cls0.DPLL_RR2[3775].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23776 GTM_gtm_cls0.DPLL_RR2[3776].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23777 GTM_gtm_cls0.DPLL_RR2[3777].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23778 GTM_gtm_cls0.DPLL_RR2[3778].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23779 GTM_gtm_cls0.DPLL_RR2[3779].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23780 GTM_gtm_cls0.DPLL_RR2[3780].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23781 GTM_gtm_cls0.DPLL_RR2[3781].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23782 GTM_gtm_cls0.DPLL_RR2[3782].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23783 GTM_gtm_cls0.DPLL_RR2[3783].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23784 GTM_gtm_cls0.DPLL_RR2[3784].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23785 GTM_gtm_cls0.DPLL_RR2[3785].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23786 GTM_gtm_cls0.DPLL_RR2[3786].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23787 GTM_gtm_cls0.DPLL_RR2[3787].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23788 GTM_gtm_cls0.DPLL_RR2[3788].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23789 GTM_gtm_cls0.DPLL_RR2[3789].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23790 GTM_gtm_cls0.DPLL_RR2[3790].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23791 GTM_gtm_cls0.DPLL_RR2[3791].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23792 GTM_gtm_cls0.DPLL_RR2[3792].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23793 GTM_gtm_cls0.DPLL_RR2[3793].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23794 GTM_gtm_cls0.DPLL_RR2[3794].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23795 GTM_gtm_cls0.DPLL_RR2[3795].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23796 GTM_gtm_cls0.DPLL_RR2[3796].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23797 GTM_gtm_cls0.DPLL_RR2[3797].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23798 GTM_gtm_cls0.DPLL_RR2[3798].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23799 GTM_gtm_cls0.DPLL_RR2[3799].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23800 GTM_gtm_cls0.DPLL_RR2[3800].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23801 GTM_gtm_cls0.DPLL_RR2[3801].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23802 GTM_gtm_cls0.DPLL_RR2[3802].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23803 GTM_gtm_cls0.DPLL_RR2[3803].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23804 GTM_gtm_cls0.DPLL_RR2[3804].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23805 GTM_gtm_cls0.DPLL_RR2[3805].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23806 GTM_gtm_cls0.DPLL_RR2[3806].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23807 GTM_gtm_cls0.DPLL_RR2[3807].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23808 GTM_gtm_cls0.DPLL_RR2[3808].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23809 GTM_gtm_cls0.DPLL_RR2[3809].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23810 GTM_gtm_cls0.DPLL_RR2[3810].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23811 GTM_gtm_cls0.DPLL_RR2[3811].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23812 GTM_gtm_cls0.DPLL_RR2[3812].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23813 GTM_gtm_cls0.DPLL_RR2[3813].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23814 GTM_gtm_cls0.DPLL_RR2[3814].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23815 GTM_gtm_cls0.DPLL_RR2[3815].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23816 GTM_gtm_cls0.DPLL_RR2[3816].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23817 GTM_gtm_cls0.DPLL_RR2[3817].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23818 GTM_gtm_cls0.DPLL_RR2[3818].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23819 GTM_gtm_cls0.DPLL_RR2[3819].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23820 GTM_gtm_cls0.DPLL_RR2[3820].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23821 GTM_gtm_cls0.DPLL_RR2[3821].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23822 GTM_gtm_cls0.DPLL_RR2[3822].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23823 GTM_gtm_cls0.DPLL_RR2[3823].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23824 GTM_gtm_cls0.DPLL_RR2[3824].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23825 GTM_gtm_cls0.DPLL_RR2[3825].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23826 GTM_gtm_cls0.DPLL_RR2[3826].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23827 GTM_gtm_cls0.DPLL_RR2[3827].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23828 GTM_gtm_cls0.DPLL_RR2[3828].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23829 GTM_gtm_cls0.DPLL_RR2[3829].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23830 GTM_gtm_cls0.DPLL_RR2[3830].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23831 GTM_gtm_cls0.DPLL_RR2[3831].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23832 GTM_gtm_cls0.DPLL_RR2[3832].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23833 GTM_gtm_cls0.DPLL_RR2[3833].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23834 GTM_gtm_cls0.DPLL_RR2[3834].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23835 GTM_gtm_cls0.DPLL_RR2[3835].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23836 GTM_gtm_cls0.DPLL_RR2[3836].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23837 GTM_gtm_cls0.DPLL_RR2[3837].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23838 GTM_gtm_cls0.DPLL_RR2[3838].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23839 GTM_gtm_cls0.DPLL_RR2[3839].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23840 GTM_gtm_cls0.DPLL_RR2[3840].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23841 GTM_gtm_cls0.DPLL_RR2[3841].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23842 GTM_gtm_cls0.DPLL_RR2[3842].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23843 GTM_gtm_cls0.DPLL_RR2[3843].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23844 GTM_gtm_cls0.DPLL_RR2[3844].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23845 GTM_gtm_cls0.DPLL_RR2[3845].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23846 GTM_gtm_cls0.DPLL_RR2[3846].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23847 GTM_gtm_cls0.DPLL_RR2[3847].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23848 GTM_gtm_cls0.DPLL_RR2[3848].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23849 GTM_gtm_cls0.DPLL_RR2[3849].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23850 GTM_gtm_cls0.DPLL_RR2[3850].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23851 GTM_gtm_cls0.DPLL_RR2[3851].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23852 GTM_gtm_cls0.DPLL_RR2[3852].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23853 GTM_gtm_cls0.DPLL_RR2[3853].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23854 GTM_gtm_cls0.DPLL_RR2[3854].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23855 GTM_gtm_cls0.DPLL_RR2[3855].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23856 GTM_gtm_cls0.DPLL_RR2[3856].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23857 GTM_gtm_cls0.DPLL_RR2[3857].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23858 GTM_gtm_cls0.DPLL_RR2[3858].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23859 GTM_gtm_cls0.DPLL_RR2[3859].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23860 GTM_gtm_cls0.DPLL_RR2[3860].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23861 GTM_gtm_cls0.DPLL_RR2[3861].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23862 GTM_gtm_cls0.DPLL_RR2[3862].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23863 GTM_gtm_cls0.DPLL_RR2[3863].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23864 GTM_gtm_cls0.DPLL_RR2[3864].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23865 GTM_gtm_cls0.DPLL_RR2[3865].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23866 GTM_gtm_cls0.DPLL_RR2[3866].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23867 GTM_gtm_cls0.DPLL_RR2[3867].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23868 GTM_gtm_cls0.DPLL_RR2[3868].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23869 GTM_gtm_cls0.DPLL_RR2[3869].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23870 GTM_gtm_cls0.DPLL_RR2[3870].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23871 GTM_gtm_cls0.DPLL_RR2[3871].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23872 GTM_gtm_cls0.DPLL_RR2[3872].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23873 GTM_gtm_cls0.DPLL_RR2[3873].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23874 GTM_gtm_cls0.DPLL_RR2[3874].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23875 GTM_gtm_cls0.DPLL_RR2[3875].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23876 GTM_gtm_cls0.DPLL_RR2[3876].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23877 GTM_gtm_cls0.DPLL_RR2[3877].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23878 GTM_gtm_cls0.DPLL_RR2[3878].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23879 GTM_gtm_cls0.DPLL_RR2[3879].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23880 GTM_gtm_cls0.DPLL_RR2[3880].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23881 GTM_gtm_cls0.DPLL_RR2[3881].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23882 GTM_gtm_cls0.DPLL_RR2[3882].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23883 GTM_gtm_cls0.DPLL_RR2[3883].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23884 GTM_gtm_cls0.DPLL_RR2[3884].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23885 GTM_gtm_cls0.DPLL_RR2[3885].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23886 GTM_gtm_cls0.DPLL_RR2[3886].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23887 GTM_gtm_cls0.DPLL_RR2[3887].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23888 GTM_gtm_cls0.DPLL_RR2[3888].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23889 GTM_gtm_cls0.DPLL_RR2[3889].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23890 GTM_gtm_cls0.DPLL_RR2[3890].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23891 GTM_gtm_cls0.DPLL_RR2[3891].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23892 GTM_gtm_cls0.DPLL_RR2[3892].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23893 GTM_gtm_cls0.DPLL_RR2[3893].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23894 GTM_gtm_cls0.DPLL_RR2[3894].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23895 GTM_gtm_cls0.DPLL_RR2[3895].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23896 GTM_gtm_cls0.DPLL_RR2[3896].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23897 GTM_gtm_cls0.DPLL_RR2[3897].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23898 GTM_gtm_cls0.DPLL_RR2[3898].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23899 GTM_gtm_cls0.DPLL_RR2[3899].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23900 GTM_gtm_cls0.DPLL_RR2[3900].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23901 GTM_gtm_cls0.DPLL_RR2[3901].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23902 GTM_gtm_cls0.DPLL_RR2[3902].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23903 GTM_gtm_cls0.DPLL_RR2[3903].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23904 GTM_gtm_cls0.DPLL_RR2[3904].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23905 GTM_gtm_cls0.DPLL_RR2[3905].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23906 GTM_gtm_cls0.DPLL_RR2[3906].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23907 GTM_gtm_cls0.DPLL_RR2[3907].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23908 GTM_gtm_cls0.DPLL_RR2[3908].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23909 GTM_gtm_cls0.DPLL_RR2[3909].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23910 GTM_gtm_cls0.DPLL_RR2[3910].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23911 GTM_gtm_cls0.DPLL_RR2[3911].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23912 GTM_gtm_cls0.DPLL_RR2[3912].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23913 GTM_gtm_cls0.DPLL_RR2[3913].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23914 GTM_gtm_cls0.DPLL_RR2[3914].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23915 GTM_gtm_cls0.DPLL_RR2[3915].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23916 GTM_gtm_cls0.DPLL_RR2[3916].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23917 GTM_gtm_cls0.DPLL_RR2[3917].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23918 GTM_gtm_cls0.DPLL_RR2[3918].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23919 GTM_gtm_cls0.DPLL_RR2[3919].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23920 GTM_gtm_cls0.DPLL_RR2[3920].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23921 GTM_gtm_cls0.DPLL_RR2[3921].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23922 GTM_gtm_cls0.DPLL_RR2[3922].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23923 GTM_gtm_cls0.DPLL_RR2[3923].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23924 GTM_gtm_cls0.DPLL_RR2[3924].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23925 GTM_gtm_cls0.DPLL_RR2[3925].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23926 GTM_gtm_cls0.DPLL_RR2[3926].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23927 GTM_gtm_cls0.DPLL_RR2[3927].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23928 GTM_gtm_cls0.DPLL_RR2[3928].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23929 GTM_gtm_cls0.DPLL_RR2[3929].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23930 GTM_gtm_cls0.DPLL_RR2[3930].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23931 GTM_gtm_cls0.DPLL_RR2[3931].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23932 GTM_gtm_cls0.DPLL_RR2[3932].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23933 GTM_gtm_cls0.DPLL_RR2[3933].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23934 GTM_gtm_cls0.DPLL_RR2[3934].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23935 GTM_gtm_cls0.DPLL_RR2[3935].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23936 GTM_gtm_cls0.DPLL_RR2[3936].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23937 GTM_gtm_cls0.DPLL_RR2[3937].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23938 GTM_gtm_cls0.DPLL_RR2[3938].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23939 GTM_gtm_cls0.DPLL_RR2[3939].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23940 GTM_gtm_cls0.DPLL_RR2[3940].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23941 GTM_gtm_cls0.DPLL_RR2[3941].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23942 GTM_gtm_cls0.DPLL_RR2[3942].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23943 GTM_gtm_cls0.DPLL_RR2[3943].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23944 GTM_gtm_cls0.DPLL_RR2[3944].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23945 GTM_gtm_cls0.DPLL_RR2[3945].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23946 GTM_gtm_cls0.DPLL_RR2[3946].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23947 GTM_gtm_cls0.DPLL_RR2[3947].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23948 GTM_gtm_cls0.DPLL_RR2[3948].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23949 GTM_gtm_cls0.DPLL_RR2[3949].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23950 GTM_gtm_cls0.DPLL_RR2[3950].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23951 GTM_gtm_cls0.DPLL_RR2[3951].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23952 GTM_gtm_cls0.DPLL_RR2[3952].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23953 GTM_gtm_cls0.DPLL_RR2[3953].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23954 GTM_gtm_cls0.DPLL_RR2[3954].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23955 GTM_gtm_cls0.DPLL_RR2[3955].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23956 GTM_gtm_cls0.DPLL_RR2[3956].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23957 GTM_gtm_cls0.DPLL_RR2[3957].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23958 GTM_gtm_cls0.DPLL_RR2[3958].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23959 GTM_gtm_cls0.DPLL_RR2[3959].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23960 GTM_gtm_cls0.DPLL_RR2[3960].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23961 GTM_gtm_cls0.DPLL_RR2[3961].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23962 GTM_gtm_cls0.DPLL_RR2[3962].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23963 GTM_gtm_cls0.DPLL_RR2[3963].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23964 GTM_gtm_cls0.DPLL_RR2[3964].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23965 GTM_gtm_cls0.DPLL_RR2[3965].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23966 GTM_gtm_cls0.DPLL_RR2[3966].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23967 GTM_gtm_cls0.DPLL_RR2[3967].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23968 GTM_gtm_cls0.DPLL_RR2[3968].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23969 GTM_gtm_cls0.DPLL_RR2[3969].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23970 GTM_gtm_cls0.DPLL_RR2[3970].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23971 GTM_gtm_cls0.DPLL_RR2[3971].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23972 GTM_gtm_cls0.DPLL_RR2[3972].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23973 GTM_gtm_cls0.DPLL_RR2[3973].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23974 GTM_gtm_cls0.DPLL_RR2[3974].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23975 GTM_gtm_cls0.DPLL_RR2[3975].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23976 GTM_gtm_cls0.DPLL_RR2[3976].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23977 GTM_gtm_cls0.DPLL_RR2[3977].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23978 GTM_gtm_cls0.DPLL_RR2[3978].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23979 GTM_gtm_cls0.DPLL_RR2[3979].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23980 GTM_gtm_cls0.DPLL_RR2[3980].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23981 GTM_gtm_cls0.DPLL_RR2[3981].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23982 GTM_gtm_cls0.DPLL_RR2[3982].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23983 GTM_gtm_cls0.DPLL_RR2[3983].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23984 GTM_gtm_cls0.DPLL_RR2[3984].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23985 GTM_gtm_cls0.DPLL_RR2[3985].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23986 GTM_gtm_cls0.DPLL_RR2[3986].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23987 GTM_gtm_cls0.DPLL_RR2[3987].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23988 GTM_gtm_cls0.DPLL_RR2[3988].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23989 GTM_gtm_cls0.DPLL_RR2[3989].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23990 GTM_gtm_cls0.DPLL_RR2[3990].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23991 GTM_gtm_cls0.DPLL_RR2[3991].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23992 GTM_gtm_cls0.DPLL_RR2[3992].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23993 GTM_gtm_cls0.DPLL_RR2[3993].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23994 GTM_gtm_cls0.DPLL_RR2[3994].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23995 GTM_gtm_cls0.DPLL_RR2[3995].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23996 GTM_gtm_cls0.DPLL_RR2[3996].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23997 GTM_gtm_cls0.DPLL_RR2[3997].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23998 GTM_gtm_cls0.DPLL_RR2[3998].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR23999 GTM_gtm_cls0.DPLL_RR2[3999].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24000 GTM_gtm_cls0.DPLL_RR2[4000].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24001 GTM_gtm_cls0.DPLL_RR2[4001].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24002 GTM_gtm_cls0.DPLL_RR2[4002].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24003 GTM_gtm_cls0.DPLL_RR2[4003].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24004 GTM_gtm_cls0.DPLL_RR2[4004].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24005 GTM_gtm_cls0.DPLL_RR2[4005].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24006 GTM_gtm_cls0.DPLL_RR2[4006].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24007 GTM_gtm_cls0.DPLL_RR2[4007].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24008 GTM_gtm_cls0.DPLL_RR2[4008].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24009 GTM_gtm_cls0.DPLL_RR2[4009].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24010 GTM_gtm_cls0.DPLL_RR2[4010].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24011 GTM_gtm_cls0.DPLL_RR2[4011].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24012 GTM_gtm_cls0.DPLL_RR2[4012].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24013 GTM_gtm_cls0.DPLL_RR2[4013].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24014 GTM_gtm_cls0.DPLL_RR2[4014].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24015 GTM_gtm_cls0.DPLL_RR2[4015].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24016 GTM_gtm_cls0.DPLL_RR2[4016].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24017 GTM_gtm_cls0.DPLL_RR2[4017].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24018 GTM_gtm_cls0.DPLL_RR2[4018].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24019 GTM_gtm_cls0.DPLL_RR2[4019].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24020 GTM_gtm_cls0.DPLL_RR2[4020].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24021 GTM_gtm_cls0.DPLL_RR2[4021].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24022 GTM_gtm_cls0.DPLL_RR2[4022].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24023 GTM_gtm_cls0.DPLL_RR2[4023].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24024 GTM_gtm_cls0.DPLL_RR2[4024].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24025 GTM_gtm_cls0.DPLL_RR2[4025].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24026 GTM_gtm_cls0.DPLL_RR2[4026].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24027 GTM_gtm_cls0.DPLL_RR2[4027].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24028 GTM_gtm_cls0.DPLL_RR2[4028].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24029 GTM_gtm_cls0.DPLL_RR2[4029].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24030 GTM_gtm_cls0.DPLL_RR2[4030].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24031 GTM_gtm_cls0.DPLL_RR2[4031].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24032 GTM_gtm_cls0.DPLL_RR2[4032].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24033 GTM_gtm_cls0.DPLL_RR2[4033].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24034 GTM_gtm_cls0.DPLL_RR2[4034].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24035 GTM_gtm_cls0.DPLL_RR2[4035].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24036 GTM_gtm_cls0.DPLL_RR2[4036].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24037 GTM_gtm_cls0.DPLL_RR2[4037].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24038 GTM_gtm_cls0.DPLL_RR2[4038].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24039 GTM_gtm_cls0.DPLL_RR2[4039].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24040 GTM_gtm_cls0.DPLL_RR2[4040].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24041 GTM_gtm_cls0.DPLL_RR2[4041].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24042 GTM_gtm_cls0.DPLL_RR2[4042].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24043 GTM_gtm_cls0.DPLL_RR2[4043].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24044 GTM_gtm_cls0.DPLL_RR2[4044].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24045 GTM_gtm_cls0.DPLL_RR2[4045].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24046 GTM_gtm_cls0.DPLL_RR2[4046].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24047 GTM_gtm_cls0.DPLL_RR2[4047].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24048 GTM_gtm_cls0.DPLL_RR2[4048].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24049 GTM_gtm_cls0.DPLL_RR2[4049].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24050 GTM_gtm_cls0.DPLL_RR2[4050].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24051 GTM_gtm_cls0.DPLL_RR2[4051].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24052 GTM_gtm_cls0.DPLL_RR2[4052].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24053 GTM_gtm_cls0.DPLL_RR2[4053].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24054 GTM_gtm_cls0.DPLL_RR2[4054].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24055 GTM_gtm_cls0.DPLL_RR2[4055].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24056 GTM_gtm_cls0.DPLL_RR2[4056].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24057 GTM_gtm_cls0.DPLL_RR2[4057].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24058 GTM_gtm_cls0.DPLL_RR2[4058].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24059 GTM_gtm_cls0.DPLL_RR2[4059].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24060 GTM_gtm_cls0.DPLL_RR2[4060].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24061 GTM_gtm_cls0.DPLL_RR2[4061].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24062 GTM_gtm_cls0.DPLL_RR2[4062].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24063 GTM_gtm_cls0.DPLL_RR2[4063].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24064 GTM_gtm_cls0.DPLL_RR2[4064].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24065 GTM_gtm_cls0.DPLL_RR2[4065].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24066 GTM_gtm_cls0.DPLL_RR2[4066].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24067 GTM_gtm_cls0.DPLL_RR2[4067].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24068 GTM_gtm_cls0.DPLL_RR2[4068].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24069 GTM_gtm_cls0.DPLL_RR2[4069].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24070 GTM_gtm_cls0.DPLL_RR2[4070].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24071 GTM_gtm_cls0.DPLL_RR2[4071].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24072 GTM_gtm_cls0.DPLL_RR2[4072].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24073 GTM_gtm_cls0.DPLL_RR2[4073].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24074 GTM_gtm_cls0.DPLL_RR2[4074].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24075 GTM_gtm_cls0.DPLL_RR2[4075].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24076 GTM_gtm_cls0.DPLL_RR2[4076].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24077 GTM_gtm_cls0.DPLL_RR2[4077].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24078 GTM_gtm_cls0.DPLL_RR2[4078].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24079 GTM_gtm_cls0.DPLL_RR2[4079].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24080 GTM_gtm_cls0.DPLL_RR2[4080].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24081 GTM_gtm_cls0.DPLL_RR2[4081].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24082 GTM_gtm_cls0.DPLL_RR2[4082].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24083 GTM_gtm_cls0.DPLL_RR2[4083].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24084 GTM_gtm_cls0.DPLL_RR2[4084].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24085 GTM_gtm_cls0.DPLL_RR2[4085].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24086 GTM_gtm_cls0.DPLL_RR2[4086].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24087 GTM_gtm_cls0.DPLL_RR2[4087].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24088 GTM_gtm_cls0.DPLL_RR2[4088].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24089 GTM_gtm_cls0.DPLL_RR2[4089].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24090 GTM_gtm_cls0.DPLL_RR2[4090].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24091 GTM_gtm_cls0.DPLL_RR2[4091].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24092 GTM_gtm_cls0.DPLL_RR2[4092].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24093 GTM_gtm_cls0.DPLL_RR2[4093].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24094 GTM_gtm_cls0.DPLL_RR2[4094].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_DPLL_RR24095 GTM_gtm_cls0.DPLL_RR2[4095].R /* DPLL memory RR2 */
#define GTM_gtm_cls0_MCS0_MEM0 GTM_gtm_cls0.MCS0_MEM[0].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1 GTM_gtm_cls0.MCS0_MEM[1].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2 GTM_gtm_cls0.MCS0_MEM[2].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3 GTM_gtm_cls0.MCS0_MEM[3].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4 GTM_gtm_cls0.MCS0_MEM[4].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5 GTM_gtm_cls0.MCS0_MEM[5].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6 GTM_gtm_cls0.MCS0_MEM[6].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM7 GTM_gtm_cls0.MCS0_MEM[7].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM8 GTM_gtm_cls0.MCS0_MEM[8].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM9 GTM_gtm_cls0.MCS0_MEM[9].R  /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM10 GTM_gtm_cls0.MCS0_MEM[10].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM11 GTM_gtm_cls0.MCS0_MEM[11].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM12 GTM_gtm_cls0.MCS0_MEM[12].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM13 GTM_gtm_cls0.MCS0_MEM[13].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM14 GTM_gtm_cls0.MCS0_MEM[14].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM15 GTM_gtm_cls0.MCS0_MEM[15].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM16 GTM_gtm_cls0.MCS0_MEM[16].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM17 GTM_gtm_cls0.MCS0_MEM[17].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM18 GTM_gtm_cls0.MCS0_MEM[18].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM19 GTM_gtm_cls0.MCS0_MEM[19].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM20 GTM_gtm_cls0.MCS0_MEM[20].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM21 GTM_gtm_cls0.MCS0_MEM[21].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM22 GTM_gtm_cls0.MCS0_MEM[22].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM23 GTM_gtm_cls0.MCS0_MEM[23].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM24 GTM_gtm_cls0.MCS0_MEM[24].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM25 GTM_gtm_cls0.MCS0_MEM[25].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM26 GTM_gtm_cls0.MCS0_MEM[26].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM27 GTM_gtm_cls0.MCS0_MEM[27].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM28 GTM_gtm_cls0.MCS0_MEM[28].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM29 GTM_gtm_cls0.MCS0_MEM[29].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM30 GTM_gtm_cls0.MCS0_MEM[30].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM31 GTM_gtm_cls0.MCS0_MEM[31].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM32 GTM_gtm_cls0.MCS0_MEM[32].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM33 GTM_gtm_cls0.MCS0_MEM[33].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM34 GTM_gtm_cls0.MCS0_MEM[34].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM35 GTM_gtm_cls0.MCS0_MEM[35].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM36 GTM_gtm_cls0.MCS0_MEM[36].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM37 GTM_gtm_cls0.MCS0_MEM[37].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM38 GTM_gtm_cls0.MCS0_MEM[38].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM39 GTM_gtm_cls0.MCS0_MEM[39].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM40 GTM_gtm_cls0.MCS0_MEM[40].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM41 GTM_gtm_cls0.MCS0_MEM[41].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM42 GTM_gtm_cls0.MCS0_MEM[42].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM43 GTM_gtm_cls0.MCS0_MEM[43].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM44 GTM_gtm_cls0.MCS0_MEM[44].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM45 GTM_gtm_cls0.MCS0_MEM[45].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM46 GTM_gtm_cls0.MCS0_MEM[46].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM47 GTM_gtm_cls0.MCS0_MEM[47].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM48 GTM_gtm_cls0.MCS0_MEM[48].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM49 GTM_gtm_cls0.MCS0_MEM[49].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM50 GTM_gtm_cls0.MCS0_MEM[50].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM51 GTM_gtm_cls0.MCS0_MEM[51].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM52 GTM_gtm_cls0.MCS0_MEM[52].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM53 GTM_gtm_cls0.MCS0_MEM[53].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM54 GTM_gtm_cls0.MCS0_MEM[54].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM55 GTM_gtm_cls0.MCS0_MEM[55].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM56 GTM_gtm_cls0.MCS0_MEM[56].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM57 GTM_gtm_cls0.MCS0_MEM[57].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM58 GTM_gtm_cls0.MCS0_MEM[58].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM59 GTM_gtm_cls0.MCS0_MEM[59].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM60 GTM_gtm_cls0.MCS0_MEM[60].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM61 GTM_gtm_cls0.MCS0_MEM[61].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM62 GTM_gtm_cls0.MCS0_MEM[62].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM63 GTM_gtm_cls0.MCS0_MEM[63].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM64 GTM_gtm_cls0.MCS0_MEM[64].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM65 GTM_gtm_cls0.MCS0_MEM[65].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM66 GTM_gtm_cls0.MCS0_MEM[66].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM67 GTM_gtm_cls0.MCS0_MEM[67].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM68 GTM_gtm_cls0.MCS0_MEM[68].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM69 GTM_gtm_cls0.MCS0_MEM[69].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM70 GTM_gtm_cls0.MCS0_MEM[70].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM71 GTM_gtm_cls0.MCS0_MEM[71].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM72 GTM_gtm_cls0.MCS0_MEM[72].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM73 GTM_gtm_cls0.MCS0_MEM[73].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM74 GTM_gtm_cls0.MCS0_MEM[74].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM75 GTM_gtm_cls0.MCS0_MEM[75].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM76 GTM_gtm_cls0.MCS0_MEM[76].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM77 GTM_gtm_cls0.MCS0_MEM[77].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM78 GTM_gtm_cls0.MCS0_MEM[78].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM79 GTM_gtm_cls0.MCS0_MEM[79].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM80 GTM_gtm_cls0.MCS0_MEM[80].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM81 GTM_gtm_cls0.MCS0_MEM[81].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM82 GTM_gtm_cls0.MCS0_MEM[82].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM83 GTM_gtm_cls0.MCS0_MEM[83].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM84 GTM_gtm_cls0.MCS0_MEM[84].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM85 GTM_gtm_cls0.MCS0_MEM[85].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM86 GTM_gtm_cls0.MCS0_MEM[86].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM87 GTM_gtm_cls0.MCS0_MEM[87].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM88 GTM_gtm_cls0.MCS0_MEM[88].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM89 GTM_gtm_cls0.MCS0_MEM[89].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM90 GTM_gtm_cls0.MCS0_MEM[90].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM91 GTM_gtm_cls0.MCS0_MEM[91].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM92 GTM_gtm_cls0.MCS0_MEM[92].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM93 GTM_gtm_cls0.MCS0_MEM[93].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM94 GTM_gtm_cls0.MCS0_MEM[94].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM95 GTM_gtm_cls0.MCS0_MEM[95].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM96 GTM_gtm_cls0.MCS0_MEM[96].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM97 GTM_gtm_cls0.MCS0_MEM[97].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM98 GTM_gtm_cls0.MCS0_MEM[98].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM99 GTM_gtm_cls0.MCS0_MEM[99].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM100 GTM_gtm_cls0.MCS0_MEM[100].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM101 GTM_gtm_cls0.MCS0_MEM[101].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM102 GTM_gtm_cls0.MCS0_MEM[102].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM103 GTM_gtm_cls0.MCS0_MEM[103].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM104 GTM_gtm_cls0.MCS0_MEM[104].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM105 GTM_gtm_cls0.MCS0_MEM[105].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM106 GTM_gtm_cls0.MCS0_MEM[106].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM107 GTM_gtm_cls0.MCS0_MEM[107].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM108 GTM_gtm_cls0.MCS0_MEM[108].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM109 GTM_gtm_cls0.MCS0_MEM[109].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM110 GTM_gtm_cls0.MCS0_MEM[110].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM111 GTM_gtm_cls0.MCS0_MEM[111].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM112 GTM_gtm_cls0.MCS0_MEM[112].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM113 GTM_gtm_cls0.MCS0_MEM[113].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM114 GTM_gtm_cls0.MCS0_MEM[114].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM115 GTM_gtm_cls0.MCS0_MEM[115].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM116 GTM_gtm_cls0.MCS0_MEM[116].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM117 GTM_gtm_cls0.MCS0_MEM[117].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM118 GTM_gtm_cls0.MCS0_MEM[118].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM119 GTM_gtm_cls0.MCS0_MEM[119].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM120 GTM_gtm_cls0.MCS0_MEM[120].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM121 GTM_gtm_cls0.MCS0_MEM[121].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM122 GTM_gtm_cls0.MCS0_MEM[122].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM123 GTM_gtm_cls0.MCS0_MEM[123].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM124 GTM_gtm_cls0.MCS0_MEM[124].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM125 GTM_gtm_cls0.MCS0_MEM[125].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM126 GTM_gtm_cls0.MCS0_MEM[126].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM127 GTM_gtm_cls0.MCS0_MEM[127].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM128 GTM_gtm_cls0.MCS0_MEM[128].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM129 GTM_gtm_cls0.MCS0_MEM[129].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM130 GTM_gtm_cls0.MCS0_MEM[130].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM131 GTM_gtm_cls0.MCS0_MEM[131].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM132 GTM_gtm_cls0.MCS0_MEM[132].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM133 GTM_gtm_cls0.MCS0_MEM[133].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM134 GTM_gtm_cls0.MCS0_MEM[134].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM135 GTM_gtm_cls0.MCS0_MEM[135].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM136 GTM_gtm_cls0.MCS0_MEM[136].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM137 GTM_gtm_cls0.MCS0_MEM[137].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM138 GTM_gtm_cls0.MCS0_MEM[138].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM139 GTM_gtm_cls0.MCS0_MEM[139].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM140 GTM_gtm_cls0.MCS0_MEM[140].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM141 GTM_gtm_cls0.MCS0_MEM[141].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM142 GTM_gtm_cls0.MCS0_MEM[142].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM143 GTM_gtm_cls0.MCS0_MEM[143].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM144 GTM_gtm_cls0.MCS0_MEM[144].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM145 GTM_gtm_cls0.MCS0_MEM[145].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM146 GTM_gtm_cls0.MCS0_MEM[146].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM147 GTM_gtm_cls0.MCS0_MEM[147].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM148 GTM_gtm_cls0.MCS0_MEM[148].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM149 GTM_gtm_cls0.MCS0_MEM[149].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM150 GTM_gtm_cls0.MCS0_MEM[150].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM151 GTM_gtm_cls0.MCS0_MEM[151].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM152 GTM_gtm_cls0.MCS0_MEM[152].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM153 GTM_gtm_cls0.MCS0_MEM[153].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM154 GTM_gtm_cls0.MCS0_MEM[154].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM155 GTM_gtm_cls0.MCS0_MEM[155].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM156 GTM_gtm_cls0.MCS0_MEM[156].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM157 GTM_gtm_cls0.MCS0_MEM[157].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM158 GTM_gtm_cls0.MCS0_MEM[158].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM159 GTM_gtm_cls0.MCS0_MEM[159].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM160 GTM_gtm_cls0.MCS0_MEM[160].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM161 GTM_gtm_cls0.MCS0_MEM[161].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM162 GTM_gtm_cls0.MCS0_MEM[162].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM163 GTM_gtm_cls0.MCS0_MEM[163].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM164 GTM_gtm_cls0.MCS0_MEM[164].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM165 GTM_gtm_cls0.MCS0_MEM[165].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM166 GTM_gtm_cls0.MCS0_MEM[166].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM167 GTM_gtm_cls0.MCS0_MEM[167].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM168 GTM_gtm_cls0.MCS0_MEM[168].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM169 GTM_gtm_cls0.MCS0_MEM[169].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM170 GTM_gtm_cls0.MCS0_MEM[170].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM171 GTM_gtm_cls0.MCS0_MEM[171].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM172 GTM_gtm_cls0.MCS0_MEM[172].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM173 GTM_gtm_cls0.MCS0_MEM[173].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM174 GTM_gtm_cls0.MCS0_MEM[174].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM175 GTM_gtm_cls0.MCS0_MEM[175].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM176 GTM_gtm_cls0.MCS0_MEM[176].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM177 GTM_gtm_cls0.MCS0_MEM[177].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM178 GTM_gtm_cls0.MCS0_MEM[178].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM179 GTM_gtm_cls0.MCS0_MEM[179].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM180 GTM_gtm_cls0.MCS0_MEM[180].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM181 GTM_gtm_cls0.MCS0_MEM[181].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM182 GTM_gtm_cls0.MCS0_MEM[182].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM183 GTM_gtm_cls0.MCS0_MEM[183].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM184 GTM_gtm_cls0.MCS0_MEM[184].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM185 GTM_gtm_cls0.MCS0_MEM[185].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM186 GTM_gtm_cls0.MCS0_MEM[186].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM187 GTM_gtm_cls0.MCS0_MEM[187].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM188 GTM_gtm_cls0.MCS0_MEM[188].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM189 GTM_gtm_cls0.MCS0_MEM[189].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM190 GTM_gtm_cls0.MCS0_MEM[190].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM191 GTM_gtm_cls0.MCS0_MEM[191].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM192 GTM_gtm_cls0.MCS0_MEM[192].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM193 GTM_gtm_cls0.MCS0_MEM[193].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM194 GTM_gtm_cls0.MCS0_MEM[194].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM195 GTM_gtm_cls0.MCS0_MEM[195].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM196 GTM_gtm_cls0.MCS0_MEM[196].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM197 GTM_gtm_cls0.MCS0_MEM[197].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM198 GTM_gtm_cls0.MCS0_MEM[198].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM199 GTM_gtm_cls0.MCS0_MEM[199].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM200 GTM_gtm_cls0.MCS0_MEM[200].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM201 GTM_gtm_cls0.MCS0_MEM[201].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM202 GTM_gtm_cls0.MCS0_MEM[202].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM203 GTM_gtm_cls0.MCS0_MEM[203].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM204 GTM_gtm_cls0.MCS0_MEM[204].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM205 GTM_gtm_cls0.MCS0_MEM[205].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM206 GTM_gtm_cls0.MCS0_MEM[206].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM207 GTM_gtm_cls0.MCS0_MEM[207].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM208 GTM_gtm_cls0.MCS0_MEM[208].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM209 GTM_gtm_cls0.MCS0_MEM[209].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM210 GTM_gtm_cls0.MCS0_MEM[210].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM211 GTM_gtm_cls0.MCS0_MEM[211].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM212 GTM_gtm_cls0.MCS0_MEM[212].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM213 GTM_gtm_cls0.MCS0_MEM[213].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM214 GTM_gtm_cls0.MCS0_MEM[214].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM215 GTM_gtm_cls0.MCS0_MEM[215].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM216 GTM_gtm_cls0.MCS0_MEM[216].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM217 GTM_gtm_cls0.MCS0_MEM[217].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM218 GTM_gtm_cls0.MCS0_MEM[218].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM219 GTM_gtm_cls0.MCS0_MEM[219].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM220 GTM_gtm_cls0.MCS0_MEM[220].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM221 GTM_gtm_cls0.MCS0_MEM[221].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM222 GTM_gtm_cls0.MCS0_MEM[222].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM223 GTM_gtm_cls0.MCS0_MEM[223].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM224 GTM_gtm_cls0.MCS0_MEM[224].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM225 GTM_gtm_cls0.MCS0_MEM[225].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM226 GTM_gtm_cls0.MCS0_MEM[226].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM227 GTM_gtm_cls0.MCS0_MEM[227].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM228 GTM_gtm_cls0.MCS0_MEM[228].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM229 GTM_gtm_cls0.MCS0_MEM[229].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM230 GTM_gtm_cls0.MCS0_MEM[230].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM231 GTM_gtm_cls0.MCS0_MEM[231].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM232 GTM_gtm_cls0.MCS0_MEM[232].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM233 GTM_gtm_cls0.MCS0_MEM[233].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM234 GTM_gtm_cls0.MCS0_MEM[234].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM235 GTM_gtm_cls0.MCS0_MEM[235].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM236 GTM_gtm_cls0.MCS0_MEM[236].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM237 GTM_gtm_cls0.MCS0_MEM[237].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM238 GTM_gtm_cls0.MCS0_MEM[238].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM239 GTM_gtm_cls0.MCS0_MEM[239].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM240 GTM_gtm_cls0.MCS0_MEM[240].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM241 GTM_gtm_cls0.MCS0_MEM[241].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM242 GTM_gtm_cls0.MCS0_MEM[242].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM243 GTM_gtm_cls0.MCS0_MEM[243].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM244 GTM_gtm_cls0.MCS0_MEM[244].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM245 GTM_gtm_cls0.MCS0_MEM[245].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM246 GTM_gtm_cls0.MCS0_MEM[246].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM247 GTM_gtm_cls0.MCS0_MEM[247].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM248 GTM_gtm_cls0.MCS0_MEM[248].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM249 GTM_gtm_cls0.MCS0_MEM[249].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM250 GTM_gtm_cls0.MCS0_MEM[250].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM251 GTM_gtm_cls0.MCS0_MEM[251].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM252 GTM_gtm_cls0.MCS0_MEM[252].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM253 GTM_gtm_cls0.MCS0_MEM[253].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM254 GTM_gtm_cls0.MCS0_MEM[254].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM255 GTM_gtm_cls0.MCS0_MEM[255].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM256 GTM_gtm_cls0.MCS0_MEM[256].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM257 GTM_gtm_cls0.MCS0_MEM[257].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM258 GTM_gtm_cls0.MCS0_MEM[258].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM259 GTM_gtm_cls0.MCS0_MEM[259].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM260 GTM_gtm_cls0.MCS0_MEM[260].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM261 GTM_gtm_cls0.MCS0_MEM[261].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM262 GTM_gtm_cls0.MCS0_MEM[262].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM263 GTM_gtm_cls0.MCS0_MEM[263].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM264 GTM_gtm_cls0.MCS0_MEM[264].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM265 GTM_gtm_cls0.MCS0_MEM[265].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM266 GTM_gtm_cls0.MCS0_MEM[266].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM267 GTM_gtm_cls0.MCS0_MEM[267].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM268 GTM_gtm_cls0.MCS0_MEM[268].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM269 GTM_gtm_cls0.MCS0_MEM[269].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM270 GTM_gtm_cls0.MCS0_MEM[270].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM271 GTM_gtm_cls0.MCS0_MEM[271].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM272 GTM_gtm_cls0.MCS0_MEM[272].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM273 GTM_gtm_cls0.MCS0_MEM[273].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM274 GTM_gtm_cls0.MCS0_MEM[274].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM275 GTM_gtm_cls0.MCS0_MEM[275].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM276 GTM_gtm_cls0.MCS0_MEM[276].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM277 GTM_gtm_cls0.MCS0_MEM[277].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM278 GTM_gtm_cls0.MCS0_MEM[278].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM279 GTM_gtm_cls0.MCS0_MEM[279].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM280 GTM_gtm_cls0.MCS0_MEM[280].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM281 GTM_gtm_cls0.MCS0_MEM[281].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM282 GTM_gtm_cls0.MCS0_MEM[282].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM283 GTM_gtm_cls0.MCS0_MEM[283].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM284 GTM_gtm_cls0.MCS0_MEM[284].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM285 GTM_gtm_cls0.MCS0_MEM[285].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM286 GTM_gtm_cls0.MCS0_MEM[286].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM287 GTM_gtm_cls0.MCS0_MEM[287].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM288 GTM_gtm_cls0.MCS0_MEM[288].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM289 GTM_gtm_cls0.MCS0_MEM[289].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM290 GTM_gtm_cls0.MCS0_MEM[290].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM291 GTM_gtm_cls0.MCS0_MEM[291].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM292 GTM_gtm_cls0.MCS0_MEM[292].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM293 GTM_gtm_cls0.MCS0_MEM[293].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM294 GTM_gtm_cls0.MCS0_MEM[294].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM295 GTM_gtm_cls0.MCS0_MEM[295].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM296 GTM_gtm_cls0.MCS0_MEM[296].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM297 GTM_gtm_cls0.MCS0_MEM[297].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM298 GTM_gtm_cls0.MCS0_MEM[298].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM299 GTM_gtm_cls0.MCS0_MEM[299].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM300 GTM_gtm_cls0.MCS0_MEM[300].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM301 GTM_gtm_cls0.MCS0_MEM[301].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM302 GTM_gtm_cls0.MCS0_MEM[302].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM303 GTM_gtm_cls0.MCS0_MEM[303].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM304 GTM_gtm_cls0.MCS0_MEM[304].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM305 GTM_gtm_cls0.MCS0_MEM[305].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM306 GTM_gtm_cls0.MCS0_MEM[306].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM307 GTM_gtm_cls0.MCS0_MEM[307].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM308 GTM_gtm_cls0.MCS0_MEM[308].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM309 GTM_gtm_cls0.MCS0_MEM[309].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM310 GTM_gtm_cls0.MCS0_MEM[310].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM311 GTM_gtm_cls0.MCS0_MEM[311].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM312 GTM_gtm_cls0.MCS0_MEM[312].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM313 GTM_gtm_cls0.MCS0_MEM[313].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM314 GTM_gtm_cls0.MCS0_MEM[314].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM315 GTM_gtm_cls0.MCS0_MEM[315].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM316 GTM_gtm_cls0.MCS0_MEM[316].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM317 GTM_gtm_cls0.MCS0_MEM[317].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM318 GTM_gtm_cls0.MCS0_MEM[318].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM319 GTM_gtm_cls0.MCS0_MEM[319].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM320 GTM_gtm_cls0.MCS0_MEM[320].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM321 GTM_gtm_cls0.MCS0_MEM[321].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM322 GTM_gtm_cls0.MCS0_MEM[322].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM323 GTM_gtm_cls0.MCS0_MEM[323].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM324 GTM_gtm_cls0.MCS0_MEM[324].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM325 GTM_gtm_cls0.MCS0_MEM[325].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM326 GTM_gtm_cls0.MCS0_MEM[326].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM327 GTM_gtm_cls0.MCS0_MEM[327].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM328 GTM_gtm_cls0.MCS0_MEM[328].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM329 GTM_gtm_cls0.MCS0_MEM[329].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM330 GTM_gtm_cls0.MCS0_MEM[330].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM331 GTM_gtm_cls0.MCS0_MEM[331].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM332 GTM_gtm_cls0.MCS0_MEM[332].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM333 GTM_gtm_cls0.MCS0_MEM[333].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM334 GTM_gtm_cls0.MCS0_MEM[334].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM335 GTM_gtm_cls0.MCS0_MEM[335].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM336 GTM_gtm_cls0.MCS0_MEM[336].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM337 GTM_gtm_cls0.MCS0_MEM[337].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM338 GTM_gtm_cls0.MCS0_MEM[338].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM339 GTM_gtm_cls0.MCS0_MEM[339].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM340 GTM_gtm_cls0.MCS0_MEM[340].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM341 GTM_gtm_cls0.MCS0_MEM[341].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM342 GTM_gtm_cls0.MCS0_MEM[342].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM343 GTM_gtm_cls0.MCS0_MEM[343].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM344 GTM_gtm_cls0.MCS0_MEM[344].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM345 GTM_gtm_cls0.MCS0_MEM[345].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM346 GTM_gtm_cls0.MCS0_MEM[346].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM347 GTM_gtm_cls0.MCS0_MEM[347].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM348 GTM_gtm_cls0.MCS0_MEM[348].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM349 GTM_gtm_cls0.MCS0_MEM[349].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM350 GTM_gtm_cls0.MCS0_MEM[350].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM351 GTM_gtm_cls0.MCS0_MEM[351].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM352 GTM_gtm_cls0.MCS0_MEM[352].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM353 GTM_gtm_cls0.MCS0_MEM[353].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM354 GTM_gtm_cls0.MCS0_MEM[354].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM355 GTM_gtm_cls0.MCS0_MEM[355].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM356 GTM_gtm_cls0.MCS0_MEM[356].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM357 GTM_gtm_cls0.MCS0_MEM[357].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM358 GTM_gtm_cls0.MCS0_MEM[358].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM359 GTM_gtm_cls0.MCS0_MEM[359].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM360 GTM_gtm_cls0.MCS0_MEM[360].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM361 GTM_gtm_cls0.MCS0_MEM[361].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM362 GTM_gtm_cls0.MCS0_MEM[362].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM363 GTM_gtm_cls0.MCS0_MEM[363].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM364 GTM_gtm_cls0.MCS0_MEM[364].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM365 GTM_gtm_cls0.MCS0_MEM[365].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM366 GTM_gtm_cls0.MCS0_MEM[366].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM367 GTM_gtm_cls0.MCS0_MEM[367].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM368 GTM_gtm_cls0.MCS0_MEM[368].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM369 GTM_gtm_cls0.MCS0_MEM[369].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM370 GTM_gtm_cls0.MCS0_MEM[370].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM371 GTM_gtm_cls0.MCS0_MEM[371].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM372 GTM_gtm_cls0.MCS0_MEM[372].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM373 GTM_gtm_cls0.MCS0_MEM[373].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM374 GTM_gtm_cls0.MCS0_MEM[374].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM375 GTM_gtm_cls0.MCS0_MEM[375].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM376 GTM_gtm_cls0.MCS0_MEM[376].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM377 GTM_gtm_cls0.MCS0_MEM[377].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM378 GTM_gtm_cls0.MCS0_MEM[378].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM379 GTM_gtm_cls0.MCS0_MEM[379].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM380 GTM_gtm_cls0.MCS0_MEM[380].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM381 GTM_gtm_cls0.MCS0_MEM[381].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM382 GTM_gtm_cls0.MCS0_MEM[382].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM383 GTM_gtm_cls0.MCS0_MEM[383].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM384 GTM_gtm_cls0.MCS0_MEM[384].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM385 GTM_gtm_cls0.MCS0_MEM[385].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM386 GTM_gtm_cls0.MCS0_MEM[386].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM387 GTM_gtm_cls0.MCS0_MEM[387].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM388 GTM_gtm_cls0.MCS0_MEM[388].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM389 GTM_gtm_cls0.MCS0_MEM[389].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM390 GTM_gtm_cls0.MCS0_MEM[390].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM391 GTM_gtm_cls0.MCS0_MEM[391].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM392 GTM_gtm_cls0.MCS0_MEM[392].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM393 GTM_gtm_cls0.MCS0_MEM[393].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM394 GTM_gtm_cls0.MCS0_MEM[394].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM395 GTM_gtm_cls0.MCS0_MEM[395].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM396 GTM_gtm_cls0.MCS0_MEM[396].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM397 GTM_gtm_cls0.MCS0_MEM[397].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM398 GTM_gtm_cls0.MCS0_MEM[398].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM399 GTM_gtm_cls0.MCS0_MEM[399].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM400 GTM_gtm_cls0.MCS0_MEM[400].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM401 GTM_gtm_cls0.MCS0_MEM[401].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM402 GTM_gtm_cls0.MCS0_MEM[402].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM403 GTM_gtm_cls0.MCS0_MEM[403].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM404 GTM_gtm_cls0.MCS0_MEM[404].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM405 GTM_gtm_cls0.MCS0_MEM[405].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM406 GTM_gtm_cls0.MCS0_MEM[406].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM407 GTM_gtm_cls0.MCS0_MEM[407].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM408 GTM_gtm_cls0.MCS0_MEM[408].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM409 GTM_gtm_cls0.MCS0_MEM[409].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM410 GTM_gtm_cls0.MCS0_MEM[410].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM411 GTM_gtm_cls0.MCS0_MEM[411].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM412 GTM_gtm_cls0.MCS0_MEM[412].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM413 GTM_gtm_cls0.MCS0_MEM[413].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM414 GTM_gtm_cls0.MCS0_MEM[414].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM415 GTM_gtm_cls0.MCS0_MEM[415].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM416 GTM_gtm_cls0.MCS0_MEM[416].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM417 GTM_gtm_cls0.MCS0_MEM[417].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM418 GTM_gtm_cls0.MCS0_MEM[418].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM419 GTM_gtm_cls0.MCS0_MEM[419].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM420 GTM_gtm_cls0.MCS0_MEM[420].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM421 GTM_gtm_cls0.MCS0_MEM[421].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM422 GTM_gtm_cls0.MCS0_MEM[422].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM423 GTM_gtm_cls0.MCS0_MEM[423].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM424 GTM_gtm_cls0.MCS0_MEM[424].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM425 GTM_gtm_cls0.MCS0_MEM[425].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM426 GTM_gtm_cls0.MCS0_MEM[426].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM427 GTM_gtm_cls0.MCS0_MEM[427].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM428 GTM_gtm_cls0.MCS0_MEM[428].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM429 GTM_gtm_cls0.MCS0_MEM[429].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM430 GTM_gtm_cls0.MCS0_MEM[430].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM431 GTM_gtm_cls0.MCS0_MEM[431].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM432 GTM_gtm_cls0.MCS0_MEM[432].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM433 GTM_gtm_cls0.MCS0_MEM[433].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM434 GTM_gtm_cls0.MCS0_MEM[434].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM435 GTM_gtm_cls0.MCS0_MEM[435].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM436 GTM_gtm_cls0.MCS0_MEM[436].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM437 GTM_gtm_cls0.MCS0_MEM[437].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM438 GTM_gtm_cls0.MCS0_MEM[438].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM439 GTM_gtm_cls0.MCS0_MEM[439].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM440 GTM_gtm_cls0.MCS0_MEM[440].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM441 GTM_gtm_cls0.MCS0_MEM[441].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM442 GTM_gtm_cls0.MCS0_MEM[442].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM443 GTM_gtm_cls0.MCS0_MEM[443].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM444 GTM_gtm_cls0.MCS0_MEM[444].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM445 GTM_gtm_cls0.MCS0_MEM[445].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM446 GTM_gtm_cls0.MCS0_MEM[446].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM447 GTM_gtm_cls0.MCS0_MEM[447].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM448 GTM_gtm_cls0.MCS0_MEM[448].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM449 GTM_gtm_cls0.MCS0_MEM[449].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM450 GTM_gtm_cls0.MCS0_MEM[450].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM451 GTM_gtm_cls0.MCS0_MEM[451].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM452 GTM_gtm_cls0.MCS0_MEM[452].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM453 GTM_gtm_cls0.MCS0_MEM[453].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM454 GTM_gtm_cls0.MCS0_MEM[454].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM455 GTM_gtm_cls0.MCS0_MEM[455].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM456 GTM_gtm_cls0.MCS0_MEM[456].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM457 GTM_gtm_cls0.MCS0_MEM[457].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM458 GTM_gtm_cls0.MCS0_MEM[458].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM459 GTM_gtm_cls0.MCS0_MEM[459].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM460 GTM_gtm_cls0.MCS0_MEM[460].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM461 GTM_gtm_cls0.MCS0_MEM[461].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM462 GTM_gtm_cls0.MCS0_MEM[462].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM463 GTM_gtm_cls0.MCS0_MEM[463].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM464 GTM_gtm_cls0.MCS0_MEM[464].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM465 GTM_gtm_cls0.MCS0_MEM[465].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM466 GTM_gtm_cls0.MCS0_MEM[466].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM467 GTM_gtm_cls0.MCS0_MEM[467].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM468 GTM_gtm_cls0.MCS0_MEM[468].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM469 GTM_gtm_cls0.MCS0_MEM[469].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM470 GTM_gtm_cls0.MCS0_MEM[470].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM471 GTM_gtm_cls0.MCS0_MEM[471].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM472 GTM_gtm_cls0.MCS0_MEM[472].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM473 GTM_gtm_cls0.MCS0_MEM[473].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM474 GTM_gtm_cls0.MCS0_MEM[474].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM475 GTM_gtm_cls0.MCS0_MEM[475].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM476 GTM_gtm_cls0.MCS0_MEM[476].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM477 GTM_gtm_cls0.MCS0_MEM[477].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM478 GTM_gtm_cls0.MCS0_MEM[478].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM479 GTM_gtm_cls0.MCS0_MEM[479].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM480 GTM_gtm_cls0.MCS0_MEM[480].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM481 GTM_gtm_cls0.MCS0_MEM[481].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM482 GTM_gtm_cls0.MCS0_MEM[482].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM483 GTM_gtm_cls0.MCS0_MEM[483].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM484 GTM_gtm_cls0.MCS0_MEM[484].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM485 GTM_gtm_cls0.MCS0_MEM[485].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM486 GTM_gtm_cls0.MCS0_MEM[486].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM487 GTM_gtm_cls0.MCS0_MEM[487].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM488 GTM_gtm_cls0.MCS0_MEM[488].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM489 GTM_gtm_cls0.MCS0_MEM[489].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM490 GTM_gtm_cls0.MCS0_MEM[490].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM491 GTM_gtm_cls0.MCS0_MEM[491].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM492 GTM_gtm_cls0.MCS0_MEM[492].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM493 GTM_gtm_cls0.MCS0_MEM[493].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM494 GTM_gtm_cls0.MCS0_MEM[494].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM495 GTM_gtm_cls0.MCS0_MEM[495].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM496 GTM_gtm_cls0.MCS0_MEM[496].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM497 GTM_gtm_cls0.MCS0_MEM[497].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM498 GTM_gtm_cls0.MCS0_MEM[498].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM499 GTM_gtm_cls0.MCS0_MEM[499].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM500 GTM_gtm_cls0.MCS0_MEM[500].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM501 GTM_gtm_cls0.MCS0_MEM[501].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM502 GTM_gtm_cls0.MCS0_MEM[502].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM503 GTM_gtm_cls0.MCS0_MEM[503].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM504 GTM_gtm_cls0.MCS0_MEM[504].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM505 GTM_gtm_cls0.MCS0_MEM[505].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM506 GTM_gtm_cls0.MCS0_MEM[506].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM507 GTM_gtm_cls0.MCS0_MEM[507].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM508 GTM_gtm_cls0.MCS0_MEM[508].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM509 GTM_gtm_cls0.MCS0_MEM[509].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM510 GTM_gtm_cls0.MCS0_MEM[510].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM511 GTM_gtm_cls0.MCS0_MEM[511].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM512 GTM_gtm_cls0.MCS0_MEM[512].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM513 GTM_gtm_cls0.MCS0_MEM[513].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM514 GTM_gtm_cls0.MCS0_MEM[514].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM515 GTM_gtm_cls0.MCS0_MEM[515].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM516 GTM_gtm_cls0.MCS0_MEM[516].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM517 GTM_gtm_cls0.MCS0_MEM[517].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM518 GTM_gtm_cls0.MCS0_MEM[518].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM519 GTM_gtm_cls0.MCS0_MEM[519].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM520 GTM_gtm_cls0.MCS0_MEM[520].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM521 GTM_gtm_cls0.MCS0_MEM[521].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM522 GTM_gtm_cls0.MCS0_MEM[522].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM523 GTM_gtm_cls0.MCS0_MEM[523].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM524 GTM_gtm_cls0.MCS0_MEM[524].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM525 GTM_gtm_cls0.MCS0_MEM[525].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM526 GTM_gtm_cls0.MCS0_MEM[526].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM527 GTM_gtm_cls0.MCS0_MEM[527].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM528 GTM_gtm_cls0.MCS0_MEM[528].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM529 GTM_gtm_cls0.MCS0_MEM[529].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM530 GTM_gtm_cls0.MCS0_MEM[530].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM531 GTM_gtm_cls0.MCS0_MEM[531].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM532 GTM_gtm_cls0.MCS0_MEM[532].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM533 GTM_gtm_cls0.MCS0_MEM[533].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM534 GTM_gtm_cls0.MCS0_MEM[534].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM535 GTM_gtm_cls0.MCS0_MEM[535].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM536 GTM_gtm_cls0.MCS0_MEM[536].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM537 GTM_gtm_cls0.MCS0_MEM[537].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM538 GTM_gtm_cls0.MCS0_MEM[538].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM539 GTM_gtm_cls0.MCS0_MEM[539].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM540 GTM_gtm_cls0.MCS0_MEM[540].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM541 GTM_gtm_cls0.MCS0_MEM[541].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM542 GTM_gtm_cls0.MCS0_MEM[542].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM543 GTM_gtm_cls0.MCS0_MEM[543].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM544 GTM_gtm_cls0.MCS0_MEM[544].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM545 GTM_gtm_cls0.MCS0_MEM[545].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM546 GTM_gtm_cls0.MCS0_MEM[546].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM547 GTM_gtm_cls0.MCS0_MEM[547].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM548 GTM_gtm_cls0.MCS0_MEM[548].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM549 GTM_gtm_cls0.MCS0_MEM[549].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM550 GTM_gtm_cls0.MCS0_MEM[550].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM551 GTM_gtm_cls0.MCS0_MEM[551].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM552 GTM_gtm_cls0.MCS0_MEM[552].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM553 GTM_gtm_cls0.MCS0_MEM[553].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM554 GTM_gtm_cls0.MCS0_MEM[554].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM555 GTM_gtm_cls0.MCS0_MEM[555].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM556 GTM_gtm_cls0.MCS0_MEM[556].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM557 GTM_gtm_cls0.MCS0_MEM[557].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM558 GTM_gtm_cls0.MCS0_MEM[558].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM559 GTM_gtm_cls0.MCS0_MEM[559].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM560 GTM_gtm_cls0.MCS0_MEM[560].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM561 GTM_gtm_cls0.MCS0_MEM[561].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM562 GTM_gtm_cls0.MCS0_MEM[562].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM563 GTM_gtm_cls0.MCS0_MEM[563].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM564 GTM_gtm_cls0.MCS0_MEM[564].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM565 GTM_gtm_cls0.MCS0_MEM[565].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM566 GTM_gtm_cls0.MCS0_MEM[566].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM567 GTM_gtm_cls0.MCS0_MEM[567].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM568 GTM_gtm_cls0.MCS0_MEM[568].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM569 GTM_gtm_cls0.MCS0_MEM[569].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM570 GTM_gtm_cls0.MCS0_MEM[570].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM571 GTM_gtm_cls0.MCS0_MEM[571].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM572 GTM_gtm_cls0.MCS0_MEM[572].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM573 GTM_gtm_cls0.MCS0_MEM[573].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM574 GTM_gtm_cls0.MCS0_MEM[574].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM575 GTM_gtm_cls0.MCS0_MEM[575].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM576 GTM_gtm_cls0.MCS0_MEM[576].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM577 GTM_gtm_cls0.MCS0_MEM[577].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM578 GTM_gtm_cls0.MCS0_MEM[578].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM579 GTM_gtm_cls0.MCS0_MEM[579].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM580 GTM_gtm_cls0.MCS0_MEM[580].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM581 GTM_gtm_cls0.MCS0_MEM[581].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM582 GTM_gtm_cls0.MCS0_MEM[582].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM583 GTM_gtm_cls0.MCS0_MEM[583].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM584 GTM_gtm_cls0.MCS0_MEM[584].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM585 GTM_gtm_cls0.MCS0_MEM[585].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM586 GTM_gtm_cls0.MCS0_MEM[586].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM587 GTM_gtm_cls0.MCS0_MEM[587].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM588 GTM_gtm_cls0.MCS0_MEM[588].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM589 GTM_gtm_cls0.MCS0_MEM[589].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM590 GTM_gtm_cls0.MCS0_MEM[590].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM591 GTM_gtm_cls0.MCS0_MEM[591].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM592 GTM_gtm_cls0.MCS0_MEM[592].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM593 GTM_gtm_cls0.MCS0_MEM[593].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM594 GTM_gtm_cls0.MCS0_MEM[594].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM595 GTM_gtm_cls0.MCS0_MEM[595].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM596 GTM_gtm_cls0.MCS0_MEM[596].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM597 GTM_gtm_cls0.MCS0_MEM[597].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM598 GTM_gtm_cls0.MCS0_MEM[598].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM599 GTM_gtm_cls0.MCS0_MEM[599].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM600 GTM_gtm_cls0.MCS0_MEM[600].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM601 GTM_gtm_cls0.MCS0_MEM[601].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM602 GTM_gtm_cls0.MCS0_MEM[602].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM603 GTM_gtm_cls0.MCS0_MEM[603].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM604 GTM_gtm_cls0.MCS0_MEM[604].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM605 GTM_gtm_cls0.MCS0_MEM[605].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM606 GTM_gtm_cls0.MCS0_MEM[606].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM607 GTM_gtm_cls0.MCS0_MEM[607].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM608 GTM_gtm_cls0.MCS0_MEM[608].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM609 GTM_gtm_cls0.MCS0_MEM[609].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM610 GTM_gtm_cls0.MCS0_MEM[610].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM611 GTM_gtm_cls0.MCS0_MEM[611].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM612 GTM_gtm_cls0.MCS0_MEM[612].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM613 GTM_gtm_cls0.MCS0_MEM[613].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM614 GTM_gtm_cls0.MCS0_MEM[614].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM615 GTM_gtm_cls0.MCS0_MEM[615].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM616 GTM_gtm_cls0.MCS0_MEM[616].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM617 GTM_gtm_cls0.MCS0_MEM[617].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM618 GTM_gtm_cls0.MCS0_MEM[618].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM619 GTM_gtm_cls0.MCS0_MEM[619].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM620 GTM_gtm_cls0.MCS0_MEM[620].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM621 GTM_gtm_cls0.MCS0_MEM[621].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM622 GTM_gtm_cls0.MCS0_MEM[622].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM623 GTM_gtm_cls0.MCS0_MEM[623].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM624 GTM_gtm_cls0.MCS0_MEM[624].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM625 GTM_gtm_cls0.MCS0_MEM[625].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM626 GTM_gtm_cls0.MCS0_MEM[626].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM627 GTM_gtm_cls0.MCS0_MEM[627].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM628 GTM_gtm_cls0.MCS0_MEM[628].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM629 GTM_gtm_cls0.MCS0_MEM[629].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM630 GTM_gtm_cls0.MCS0_MEM[630].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM631 GTM_gtm_cls0.MCS0_MEM[631].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM632 GTM_gtm_cls0.MCS0_MEM[632].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM633 GTM_gtm_cls0.MCS0_MEM[633].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM634 GTM_gtm_cls0.MCS0_MEM[634].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM635 GTM_gtm_cls0.MCS0_MEM[635].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM636 GTM_gtm_cls0.MCS0_MEM[636].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM637 GTM_gtm_cls0.MCS0_MEM[637].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM638 GTM_gtm_cls0.MCS0_MEM[638].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM639 GTM_gtm_cls0.MCS0_MEM[639].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM640 GTM_gtm_cls0.MCS0_MEM[640].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM641 GTM_gtm_cls0.MCS0_MEM[641].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM642 GTM_gtm_cls0.MCS0_MEM[642].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM643 GTM_gtm_cls0.MCS0_MEM[643].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM644 GTM_gtm_cls0.MCS0_MEM[644].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM645 GTM_gtm_cls0.MCS0_MEM[645].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM646 GTM_gtm_cls0.MCS0_MEM[646].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM647 GTM_gtm_cls0.MCS0_MEM[647].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM648 GTM_gtm_cls0.MCS0_MEM[648].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM649 GTM_gtm_cls0.MCS0_MEM[649].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM650 GTM_gtm_cls0.MCS0_MEM[650].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM651 GTM_gtm_cls0.MCS0_MEM[651].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM652 GTM_gtm_cls0.MCS0_MEM[652].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM653 GTM_gtm_cls0.MCS0_MEM[653].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM654 GTM_gtm_cls0.MCS0_MEM[654].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM655 GTM_gtm_cls0.MCS0_MEM[655].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM656 GTM_gtm_cls0.MCS0_MEM[656].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM657 GTM_gtm_cls0.MCS0_MEM[657].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM658 GTM_gtm_cls0.MCS0_MEM[658].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM659 GTM_gtm_cls0.MCS0_MEM[659].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM660 GTM_gtm_cls0.MCS0_MEM[660].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM661 GTM_gtm_cls0.MCS0_MEM[661].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM662 GTM_gtm_cls0.MCS0_MEM[662].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM663 GTM_gtm_cls0.MCS0_MEM[663].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM664 GTM_gtm_cls0.MCS0_MEM[664].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM665 GTM_gtm_cls0.MCS0_MEM[665].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM666 GTM_gtm_cls0.MCS0_MEM[666].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM667 GTM_gtm_cls0.MCS0_MEM[667].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM668 GTM_gtm_cls0.MCS0_MEM[668].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM669 GTM_gtm_cls0.MCS0_MEM[669].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM670 GTM_gtm_cls0.MCS0_MEM[670].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM671 GTM_gtm_cls0.MCS0_MEM[671].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM672 GTM_gtm_cls0.MCS0_MEM[672].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM673 GTM_gtm_cls0.MCS0_MEM[673].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM674 GTM_gtm_cls0.MCS0_MEM[674].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM675 GTM_gtm_cls0.MCS0_MEM[675].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM676 GTM_gtm_cls0.MCS0_MEM[676].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM677 GTM_gtm_cls0.MCS0_MEM[677].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM678 GTM_gtm_cls0.MCS0_MEM[678].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM679 GTM_gtm_cls0.MCS0_MEM[679].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM680 GTM_gtm_cls0.MCS0_MEM[680].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM681 GTM_gtm_cls0.MCS0_MEM[681].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM682 GTM_gtm_cls0.MCS0_MEM[682].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM683 GTM_gtm_cls0.MCS0_MEM[683].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM684 GTM_gtm_cls0.MCS0_MEM[684].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM685 GTM_gtm_cls0.MCS0_MEM[685].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM686 GTM_gtm_cls0.MCS0_MEM[686].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM687 GTM_gtm_cls0.MCS0_MEM[687].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM688 GTM_gtm_cls0.MCS0_MEM[688].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM689 GTM_gtm_cls0.MCS0_MEM[689].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM690 GTM_gtm_cls0.MCS0_MEM[690].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM691 GTM_gtm_cls0.MCS0_MEM[691].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM692 GTM_gtm_cls0.MCS0_MEM[692].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM693 GTM_gtm_cls0.MCS0_MEM[693].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM694 GTM_gtm_cls0.MCS0_MEM[694].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM695 GTM_gtm_cls0.MCS0_MEM[695].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM696 GTM_gtm_cls0.MCS0_MEM[696].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM697 GTM_gtm_cls0.MCS0_MEM[697].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM698 GTM_gtm_cls0.MCS0_MEM[698].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM699 GTM_gtm_cls0.MCS0_MEM[699].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM700 GTM_gtm_cls0.MCS0_MEM[700].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM701 GTM_gtm_cls0.MCS0_MEM[701].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM702 GTM_gtm_cls0.MCS0_MEM[702].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM703 GTM_gtm_cls0.MCS0_MEM[703].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM704 GTM_gtm_cls0.MCS0_MEM[704].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM705 GTM_gtm_cls0.MCS0_MEM[705].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM706 GTM_gtm_cls0.MCS0_MEM[706].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM707 GTM_gtm_cls0.MCS0_MEM[707].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM708 GTM_gtm_cls0.MCS0_MEM[708].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM709 GTM_gtm_cls0.MCS0_MEM[709].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM710 GTM_gtm_cls0.MCS0_MEM[710].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM711 GTM_gtm_cls0.MCS0_MEM[711].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM712 GTM_gtm_cls0.MCS0_MEM[712].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM713 GTM_gtm_cls0.MCS0_MEM[713].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM714 GTM_gtm_cls0.MCS0_MEM[714].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM715 GTM_gtm_cls0.MCS0_MEM[715].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM716 GTM_gtm_cls0.MCS0_MEM[716].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM717 GTM_gtm_cls0.MCS0_MEM[717].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM718 GTM_gtm_cls0.MCS0_MEM[718].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM719 GTM_gtm_cls0.MCS0_MEM[719].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM720 GTM_gtm_cls0.MCS0_MEM[720].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM721 GTM_gtm_cls0.MCS0_MEM[721].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM722 GTM_gtm_cls0.MCS0_MEM[722].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM723 GTM_gtm_cls0.MCS0_MEM[723].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM724 GTM_gtm_cls0.MCS0_MEM[724].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM725 GTM_gtm_cls0.MCS0_MEM[725].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM726 GTM_gtm_cls0.MCS0_MEM[726].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM727 GTM_gtm_cls0.MCS0_MEM[727].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM728 GTM_gtm_cls0.MCS0_MEM[728].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM729 GTM_gtm_cls0.MCS0_MEM[729].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM730 GTM_gtm_cls0.MCS0_MEM[730].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM731 GTM_gtm_cls0.MCS0_MEM[731].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM732 GTM_gtm_cls0.MCS0_MEM[732].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM733 GTM_gtm_cls0.MCS0_MEM[733].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM734 GTM_gtm_cls0.MCS0_MEM[734].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM735 GTM_gtm_cls0.MCS0_MEM[735].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM736 GTM_gtm_cls0.MCS0_MEM[736].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM737 GTM_gtm_cls0.MCS0_MEM[737].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM738 GTM_gtm_cls0.MCS0_MEM[738].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM739 GTM_gtm_cls0.MCS0_MEM[739].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM740 GTM_gtm_cls0.MCS0_MEM[740].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM741 GTM_gtm_cls0.MCS0_MEM[741].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM742 GTM_gtm_cls0.MCS0_MEM[742].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM743 GTM_gtm_cls0.MCS0_MEM[743].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM744 GTM_gtm_cls0.MCS0_MEM[744].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM745 GTM_gtm_cls0.MCS0_MEM[745].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM746 GTM_gtm_cls0.MCS0_MEM[746].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM747 GTM_gtm_cls0.MCS0_MEM[747].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM748 GTM_gtm_cls0.MCS0_MEM[748].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM749 GTM_gtm_cls0.MCS0_MEM[749].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM750 GTM_gtm_cls0.MCS0_MEM[750].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM751 GTM_gtm_cls0.MCS0_MEM[751].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM752 GTM_gtm_cls0.MCS0_MEM[752].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM753 GTM_gtm_cls0.MCS0_MEM[753].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM754 GTM_gtm_cls0.MCS0_MEM[754].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM755 GTM_gtm_cls0.MCS0_MEM[755].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM756 GTM_gtm_cls0.MCS0_MEM[756].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM757 GTM_gtm_cls0.MCS0_MEM[757].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM758 GTM_gtm_cls0.MCS0_MEM[758].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM759 GTM_gtm_cls0.MCS0_MEM[759].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM760 GTM_gtm_cls0.MCS0_MEM[760].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM761 GTM_gtm_cls0.MCS0_MEM[761].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM762 GTM_gtm_cls0.MCS0_MEM[762].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM763 GTM_gtm_cls0.MCS0_MEM[763].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM764 GTM_gtm_cls0.MCS0_MEM[764].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM765 GTM_gtm_cls0.MCS0_MEM[765].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM766 GTM_gtm_cls0.MCS0_MEM[766].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM767 GTM_gtm_cls0.MCS0_MEM[767].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM768 GTM_gtm_cls0.MCS0_MEM[768].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM769 GTM_gtm_cls0.MCS0_MEM[769].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM770 GTM_gtm_cls0.MCS0_MEM[770].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM771 GTM_gtm_cls0.MCS0_MEM[771].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM772 GTM_gtm_cls0.MCS0_MEM[772].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM773 GTM_gtm_cls0.MCS0_MEM[773].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM774 GTM_gtm_cls0.MCS0_MEM[774].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM775 GTM_gtm_cls0.MCS0_MEM[775].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM776 GTM_gtm_cls0.MCS0_MEM[776].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM777 GTM_gtm_cls0.MCS0_MEM[777].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM778 GTM_gtm_cls0.MCS0_MEM[778].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM779 GTM_gtm_cls0.MCS0_MEM[779].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM780 GTM_gtm_cls0.MCS0_MEM[780].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM781 GTM_gtm_cls0.MCS0_MEM[781].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM782 GTM_gtm_cls0.MCS0_MEM[782].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM783 GTM_gtm_cls0.MCS0_MEM[783].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM784 GTM_gtm_cls0.MCS0_MEM[784].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM785 GTM_gtm_cls0.MCS0_MEM[785].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM786 GTM_gtm_cls0.MCS0_MEM[786].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM787 GTM_gtm_cls0.MCS0_MEM[787].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM788 GTM_gtm_cls0.MCS0_MEM[788].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM789 GTM_gtm_cls0.MCS0_MEM[789].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM790 GTM_gtm_cls0.MCS0_MEM[790].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM791 GTM_gtm_cls0.MCS0_MEM[791].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM792 GTM_gtm_cls0.MCS0_MEM[792].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM793 GTM_gtm_cls0.MCS0_MEM[793].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM794 GTM_gtm_cls0.MCS0_MEM[794].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM795 GTM_gtm_cls0.MCS0_MEM[795].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM796 GTM_gtm_cls0.MCS0_MEM[796].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM797 GTM_gtm_cls0.MCS0_MEM[797].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM798 GTM_gtm_cls0.MCS0_MEM[798].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM799 GTM_gtm_cls0.MCS0_MEM[799].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM800 GTM_gtm_cls0.MCS0_MEM[800].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM801 GTM_gtm_cls0.MCS0_MEM[801].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM802 GTM_gtm_cls0.MCS0_MEM[802].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM803 GTM_gtm_cls0.MCS0_MEM[803].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM804 GTM_gtm_cls0.MCS0_MEM[804].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM805 GTM_gtm_cls0.MCS0_MEM[805].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM806 GTM_gtm_cls0.MCS0_MEM[806].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM807 GTM_gtm_cls0.MCS0_MEM[807].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM808 GTM_gtm_cls0.MCS0_MEM[808].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM809 GTM_gtm_cls0.MCS0_MEM[809].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM810 GTM_gtm_cls0.MCS0_MEM[810].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM811 GTM_gtm_cls0.MCS0_MEM[811].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM812 GTM_gtm_cls0.MCS0_MEM[812].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM813 GTM_gtm_cls0.MCS0_MEM[813].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM814 GTM_gtm_cls0.MCS0_MEM[814].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM815 GTM_gtm_cls0.MCS0_MEM[815].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM816 GTM_gtm_cls0.MCS0_MEM[816].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM817 GTM_gtm_cls0.MCS0_MEM[817].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM818 GTM_gtm_cls0.MCS0_MEM[818].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM819 GTM_gtm_cls0.MCS0_MEM[819].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM820 GTM_gtm_cls0.MCS0_MEM[820].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM821 GTM_gtm_cls0.MCS0_MEM[821].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM822 GTM_gtm_cls0.MCS0_MEM[822].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM823 GTM_gtm_cls0.MCS0_MEM[823].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM824 GTM_gtm_cls0.MCS0_MEM[824].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM825 GTM_gtm_cls0.MCS0_MEM[825].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM826 GTM_gtm_cls0.MCS0_MEM[826].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM827 GTM_gtm_cls0.MCS0_MEM[827].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM828 GTM_gtm_cls0.MCS0_MEM[828].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM829 GTM_gtm_cls0.MCS0_MEM[829].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM830 GTM_gtm_cls0.MCS0_MEM[830].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM831 GTM_gtm_cls0.MCS0_MEM[831].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM832 GTM_gtm_cls0.MCS0_MEM[832].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM833 GTM_gtm_cls0.MCS0_MEM[833].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM834 GTM_gtm_cls0.MCS0_MEM[834].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM835 GTM_gtm_cls0.MCS0_MEM[835].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM836 GTM_gtm_cls0.MCS0_MEM[836].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM837 GTM_gtm_cls0.MCS0_MEM[837].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM838 GTM_gtm_cls0.MCS0_MEM[838].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM839 GTM_gtm_cls0.MCS0_MEM[839].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM840 GTM_gtm_cls0.MCS0_MEM[840].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM841 GTM_gtm_cls0.MCS0_MEM[841].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM842 GTM_gtm_cls0.MCS0_MEM[842].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM843 GTM_gtm_cls0.MCS0_MEM[843].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM844 GTM_gtm_cls0.MCS0_MEM[844].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM845 GTM_gtm_cls0.MCS0_MEM[845].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM846 GTM_gtm_cls0.MCS0_MEM[846].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM847 GTM_gtm_cls0.MCS0_MEM[847].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM848 GTM_gtm_cls0.MCS0_MEM[848].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM849 GTM_gtm_cls0.MCS0_MEM[849].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM850 GTM_gtm_cls0.MCS0_MEM[850].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM851 GTM_gtm_cls0.MCS0_MEM[851].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM852 GTM_gtm_cls0.MCS0_MEM[852].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM853 GTM_gtm_cls0.MCS0_MEM[853].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM854 GTM_gtm_cls0.MCS0_MEM[854].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM855 GTM_gtm_cls0.MCS0_MEM[855].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM856 GTM_gtm_cls0.MCS0_MEM[856].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM857 GTM_gtm_cls0.MCS0_MEM[857].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM858 GTM_gtm_cls0.MCS0_MEM[858].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM859 GTM_gtm_cls0.MCS0_MEM[859].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM860 GTM_gtm_cls0.MCS0_MEM[860].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM861 GTM_gtm_cls0.MCS0_MEM[861].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM862 GTM_gtm_cls0.MCS0_MEM[862].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM863 GTM_gtm_cls0.MCS0_MEM[863].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM864 GTM_gtm_cls0.MCS0_MEM[864].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM865 GTM_gtm_cls0.MCS0_MEM[865].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM866 GTM_gtm_cls0.MCS0_MEM[866].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM867 GTM_gtm_cls0.MCS0_MEM[867].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM868 GTM_gtm_cls0.MCS0_MEM[868].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM869 GTM_gtm_cls0.MCS0_MEM[869].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM870 GTM_gtm_cls0.MCS0_MEM[870].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM871 GTM_gtm_cls0.MCS0_MEM[871].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM872 GTM_gtm_cls0.MCS0_MEM[872].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM873 GTM_gtm_cls0.MCS0_MEM[873].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM874 GTM_gtm_cls0.MCS0_MEM[874].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM875 GTM_gtm_cls0.MCS0_MEM[875].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM876 GTM_gtm_cls0.MCS0_MEM[876].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM877 GTM_gtm_cls0.MCS0_MEM[877].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM878 GTM_gtm_cls0.MCS0_MEM[878].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM879 GTM_gtm_cls0.MCS0_MEM[879].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM880 GTM_gtm_cls0.MCS0_MEM[880].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM881 GTM_gtm_cls0.MCS0_MEM[881].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM882 GTM_gtm_cls0.MCS0_MEM[882].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM883 GTM_gtm_cls0.MCS0_MEM[883].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM884 GTM_gtm_cls0.MCS0_MEM[884].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM885 GTM_gtm_cls0.MCS0_MEM[885].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM886 GTM_gtm_cls0.MCS0_MEM[886].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM887 GTM_gtm_cls0.MCS0_MEM[887].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM888 GTM_gtm_cls0.MCS0_MEM[888].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM889 GTM_gtm_cls0.MCS0_MEM[889].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM890 GTM_gtm_cls0.MCS0_MEM[890].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM891 GTM_gtm_cls0.MCS0_MEM[891].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM892 GTM_gtm_cls0.MCS0_MEM[892].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM893 GTM_gtm_cls0.MCS0_MEM[893].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM894 GTM_gtm_cls0.MCS0_MEM[894].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM895 GTM_gtm_cls0.MCS0_MEM[895].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM896 GTM_gtm_cls0.MCS0_MEM[896].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM897 GTM_gtm_cls0.MCS0_MEM[897].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM898 GTM_gtm_cls0.MCS0_MEM[898].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM899 GTM_gtm_cls0.MCS0_MEM[899].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM900 GTM_gtm_cls0.MCS0_MEM[900].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM901 GTM_gtm_cls0.MCS0_MEM[901].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM902 GTM_gtm_cls0.MCS0_MEM[902].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM903 GTM_gtm_cls0.MCS0_MEM[903].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM904 GTM_gtm_cls0.MCS0_MEM[904].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM905 GTM_gtm_cls0.MCS0_MEM[905].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM906 GTM_gtm_cls0.MCS0_MEM[906].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM907 GTM_gtm_cls0.MCS0_MEM[907].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM908 GTM_gtm_cls0.MCS0_MEM[908].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM909 GTM_gtm_cls0.MCS0_MEM[909].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM910 GTM_gtm_cls0.MCS0_MEM[910].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM911 GTM_gtm_cls0.MCS0_MEM[911].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM912 GTM_gtm_cls0.MCS0_MEM[912].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM913 GTM_gtm_cls0.MCS0_MEM[913].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM914 GTM_gtm_cls0.MCS0_MEM[914].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM915 GTM_gtm_cls0.MCS0_MEM[915].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM916 GTM_gtm_cls0.MCS0_MEM[916].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM917 GTM_gtm_cls0.MCS0_MEM[917].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM918 GTM_gtm_cls0.MCS0_MEM[918].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM919 GTM_gtm_cls0.MCS0_MEM[919].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM920 GTM_gtm_cls0.MCS0_MEM[920].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM921 GTM_gtm_cls0.MCS0_MEM[921].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM922 GTM_gtm_cls0.MCS0_MEM[922].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM923 GTM_gtm_cls0.MCS0_MEM[923].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM924 GTM_gtm_cls0.MCS0_MEM[924].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM925 GTM_gtm_cls0.MCS0_MEM[925].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM926 GTM_gtm_cls0.MCS0_MEM[926].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM927 GTM_gtm_cls0.MCS0_MEM[927].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM928 GTM_gtm_cls0.MCS0_MEM[928].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM929 GTM_gtm_cls0.MCS0_MEM[929].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM930 GTM_gtm_cls0.MCS0_MEM[930].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM931 GTM_gtm_cls0.MCS0_MEM[931].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM932 GTM_gtm_cls0.MCS0_MEM[932].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM933 GTM_gtm_cls0.MCS0_MEM[933].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM934 GTM_gtm_cls0.MCS0_MEM[934].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM935 GTM_gtm_cls0.MCS0_MEM[935].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM936 GTM_gtm_cls0.MCS0_MEM[936].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM937 GTM_gtm_cls0.MCS0_MEM[937].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM938 GTM_gtm_cls0.MCS0_MEM[938].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM939 GTM_gtm_cls0.MCS0_MEM[939].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM940 GTM_gtm_cls0.MCS0_MEM[940].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM941 GTM_gtm_cls0.MCS0_MEM[941].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM942 GTM_gtm_cls0.MCS0_MEM[942].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM943 GTM_gtm_cls0.MCS0_MEM[943].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM944 GTM_gtm_cls0.MCS0_MEM[944].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM945 GTM_gtm_cls0.MCS0_MEM[945].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM946 GTM_gtm_cls0.MCS0_MEM[946].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM947 GTM_gtm_cls0.MCS0_MEM[947].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM948 GTM_gtm_cls0.MCS0_MEM[948].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM949 GTM_gtm_cls0.MCS0_MEM[949].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM950 GTM_gtm_cls0.MCS0_MEM[950].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM951 GTM_gtm_cls0.MCS0_MEM[951].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM952 GTM_gtm_cls0.MCS0_MEM[952].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM953 GTM_gtm_cls0.MCS0_MEM[953].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM954 GTM_gtm_cls0.MCS0_MEM[954].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM955 GTM_gtm_cls0.MCS0_MEM[955].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM956 GTM_gtm_cls0.MCS0_MEM[956].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM957 GTM_gtm_cls0.MCS0_MEM[957].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM958 GTM_gtm_cls0.MCS0_MEM[958].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM959 GTM_gtm_cls0.MCS0_MEM[959].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM960 GTM_gtm_cls0.MCS0_MEM[960].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM961 GTM_gtm_cls0.MCS0_MEM[961].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM962 GTM_gtm_cls0.MCS0_MEM[962].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM963 GTM_gtm_cls0.MCS0_MEM[963].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM964 GTM_gtm_cls0.MCS0_MEM[964].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM965 GTM_gtm_cls0.MCS0_MEM[965].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM966 GTM_gtm_cls0.MCS0_MEM[966].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM967 GTM_gtm_cls0.MCS0_MEM[967].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM968 GTM_gtm_cls0.MCS0_MEM[968].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM969 GTM_gtm_cls0.MCS0_MEM[969].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM970 GTM_gtm_cls0.MCS0_MEM[970].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM971 GTM_gtm_cls0.MCS0_MEM[971].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM972 GTM_gtm_cls0.MCS0_MEM[972].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM973 GTM_gtm_cls0.MCS0_MEM[973].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM974 GTM_gtm_cls0.MCS0_MEM[974].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM975 GTM_gtm_cls0.MCS0_MEM[975].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM976 GTM_gtm_cls0.MCS0_MEM[976].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM977 GTM_gtm_cls0.MCS0_MEM[977].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM978 GTM_gtm_cls0.MCS0_MEM[978].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM979 GTM_gtm_cls0.MCS0_MEM[979].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM980 GTM_gtm_cls0.MCS0_MEM[980].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM981 GTM_gtm_cls0.MCS0_MEM[981].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM982 GTM_gtm_cls0.MCS0_MEM[982].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM983 GTM_gtm_cls0.MCS0_MEM[983].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM984 GTM_gtm_cls0.MCS0_MEM[984].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM985 GTM_gtm_cls0.MCS0_MEM[985].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM986 GTM_gtm_cls0.MCS0_MEM[986].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM987 GTM_gtm_cls0.MCS0_MEM[987].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM988 GTM_gtm_cls0.MCS0_MEM[988].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM989 GTM_gtm_cls0.MCS0_MEM[989].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM990 GTM_gtm_cls0.MCS0_MEM[990].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM991 GTM_gtm_cls0.MCS0_MEM[991].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM992 GTM_gtm_cls0.MCS0_MEM[992].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM993 GTM_gtm_cls0.MCS0_MEM[993].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM994 GTM_gtm_cls0.MCS0_MEM[994].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM995 GTM_gtm_cls0.MCS0_MEM[995].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM996 GTM_gtm_cls0.MCS0_MEM[996].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM997 GTM_gtm_cls0.MCS0_MEM[997].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM998 GTM_gtm_cls0.MCS0_MEM[998].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM999 GTM_gtm_cls0.MCS0_MEM[999].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1000 GTM_gtm_cls0.MCS0_MEM[1000].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1001 GTM_gtm_cls0.MCS0_MEM[1001].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1002 GTM_gtm_cls0.MCS0_MEM[1002].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1003 GTM_gtm_cls0.MCS0_MEM[1003].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1004 GTM_gtm_cls0.MCS0_MEM[1004].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1005 GTM_gtm_cls0.MCS0_MEM[1005].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1006 GTM_gtm_cls0.MCS0_MEM[1006].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1007 GTM_gtm_cls0.MCS0_MEM[1007].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1008 GTM_gtm_cls0.MCS0_MEM[1008].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1009 GTM_gtm_cls0.MCS0_MEM[1009].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1010 GTM_gtm_cls0.MCS0_MEM[1010].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1011 GTM_gtm_cls0.MCS0_MEM[1011].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1012 GTM_gtm_cls0.MCS0_MEM[1012].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1013 GTM_gtm_cls0.MCS0_MEM[1013].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1014 GTM_gtm_cls0.MCS0_MEM[1014].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1015 GTM_gtm_cls0.MCS0_MEM[1015].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1016 GTM_gtm_cls0.MCS0_MEM[1016].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1017 GTM_gtm_cls0.MCS0_MEM[1017].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1018 GTM_gtm_cls0.MCS0_MEM[1018].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1019 GTM_gtm_cls0.MCS0_MEM[1019].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1020 GTM_gtm_cls0.MCS0_MEM[1020].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1021 GTM_gtm_cls0.MCS0_MEM[1021].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1022 GTM_gtm_cls0.MCS0_MEM[1022].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1023 GTM_gtm_cls0.MCS0_MEM[1023].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1024 GTM_gtm_cls0.MCS0_MEM[1024].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1025 GTM_gtm_cls0.MCS0_MEM[1025].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1026 GTM_gtm_cls0.MCS0_MEM[1026].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1027 GTM_gtm_cls0.MCS0_MEM[1027].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1028 GTM_gtm_cls0.MCS0_MEM[1028].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1029 GTM_gtm_cls0.MCS0_MEM[1029].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1030 GTM_gtm_cls0.MCS0_MEM[1030].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1031 GTM_gtm_cls0.MCS0_MEM[1031].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1032 GTM_gtm_cls0.MCS0_MEM[1032].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1033 GTM_gtm_cls0.MCS0_MEM[1033].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1034 GTM_gtm_cls0.MCS0_MEM[1034].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1035 GTM_gtm_cls0.MCS0_MEM[1035].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1036 GTM_gtm_cls0.MCS0_MEM[1036].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1037 GTM_gtm_cls0.MCS0_MEM[1037].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1038 GTM_gtm_cls0.MCS0_MEM[1038].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1039 GTM_gtm_cls0.MCS0_MEM[1039].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1040 GTM_gtm_cls0.MCS0_MEM[1040].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1041 GTM_gtm_cls0.MCS0_MEM[1041].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1042 GTM_gtm_cls0.MCS0_MEM[1042].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1043 GTM_gtm_cls0.MCS0_MEM[1043].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1044 GTM_gtm_cls0.MCS0_MEM[1044].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1045 GTM_gtm_cls0.MCS0_MEM[1045].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1046 GTM_gtm_cls0.MCS0_MEM[1046].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1047 GTM_gtm_cls0.MCS0_MEM[1047].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1048 GTM_gtm_cls0.MCS0_MEM[1048].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1049 GTM_gtm_cls0.MCS0_MEM[1049].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1050 GTM_gtm_cls0.MCS0_MEM[1050].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1051 GTM_gtm_cls0.MCS0_MEM[1051].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1052 GTM_gtm_cls0.MCS0_MEM[1052].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1053 GTM_gtm_cls0.MCS0_MEM[1053].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1054 GTM_gtm_cls0.MCS0_MEM[1054].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1055 GTM_gtm_cls0.MCS0_MEM[1055].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1056 GTM_gtm_cls0.MCS0_MEM[1056].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1057 GTM_gtm_cls0.MCS0_MEM[1057].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1058 GTM_gtm_cls0.MCS0_MEM[1058].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1059 GTM_gtm_cls0.MCS0_MEM[1059].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1060 GTM_gtm_cls0.MCS0_MEM[1060].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1061 GTM_gtm_cls0.MCS0_MEM[1061].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1062 GTM_gtm_cls0.MCS0_MEM[1062].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1063 GTM_gtm_cls0.MCS0_MEM[1063].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1064 GTM_gtm_cls0.MCS0_MEM[1064].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1065 GTM_gtm_cls0.MCS0_MEM[1065].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1066 GTM_gtm_cls0.MCS0_MEM[1066].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1067 GTM_gtm_cls0.MCS0_MEM[1067].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1068 GTM_gtm_cls0.MCS0_MEM[1068].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1069 GTM_gtm_cls0.MCS0_MEM[1069].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1070 GTM_gtm_cls0.MCS0_MEM[1070].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1071 GTM_gtm_cls0.MCS0_MEM[1071].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1072 GTM_gtm_cls0.MCS0_MEM[1072].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1073 GTM_gtm_cls0.MCS0_MEM[1073].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1074 GTM_gtm_cls0.MCS0_MEM[1074].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1075 GTM_gtm_cls0.MCS0_MEM[1075].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1076 GTM_gtm_cls0.MCS0_MEM[1076].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1077 GTM_gtm_cls0.MCS0_MEM[1077].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1078 GTM_gtm_cls0.MCS0_MEM[1078].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1079 GTM_gtm_cls0.MCS0_MEM[1079].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1080 GTM_gtm_cls0.MCS0_MEM[1080].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1081 GTM_gtm_cls0.MCS0_MEM[1081].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1082 GTM_gtm_cls0.MCS0_MEM[1082].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1083 GTM_gtm_cls0.MCS0_MEM[1083].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1084 GTM_gtm_cls0.MCS0_MEM[1084].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1085 GTM_gtm_cls0.MCS0_MEM[1085].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1086 GTM_gtm_cls0.MCS0_MEM[1086].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1087 GTM_gtm_cls0.MCS0_MEM[1087].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1088 GTM_gtm_cls0.MCS0_MEM[1088].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1089 GTM_gtm_cls0.MCS0_MEM[1089].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1090 GTM_gtm_cls0.MCS0_MEM[1090].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1091 GTM_gtm_cls0.MCS0_MEM[1091].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1092 GTM_gtm_cls0.MCS0_MEM[1092].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1093 GTM_gtm_cls0.MCS0_MEM[1093].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1094 GTM_gtm_cls0.MCS0_MEM[1094].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1095 GTM_gtm_cls0.MCS0_MEM[1095].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1096 GTM_gtm_cls0.MCS0_MEM[1096].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1097 GTM_gtm_cls0.MCS0_MEM[1097].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1098 GTM_gtm_cls0.MCS0_MEM[1098].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1099 GTM_gtm_cls0.MCS0_MEM[1099].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1100 GTM_gtm_cls0.MCS0_MEM[1100].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1101 GTM_gtm_cls0.MCS0_MEM[1101].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1102 GTM_gtm_cls0.MCS0_MEM[1102].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1103 GTM_gtm_cls0.MCS0_MEM[1103].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1104 GTM_gtm_cls0.MCS0_MEM[1104].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1105 GTM_gtm_cls0.MCS0_MEM[1105].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1106 GTM_gtm_cls0.MCS0_MEM[1106].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1107 GTM_gtm_cls0.MCS0_MEM[1107].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1108 GTM_gtm_cls0.MCS0_MEM[1108].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1109 GTM_gtm_cls0.MCS0_MEM[1109].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1110 GTM_gtm_cls0.MCS0_MEM[1110].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1111 GTM_gtm_cls0.MCS0_MEM[1111].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1112 GTM_gtm_cls0.MCS0_MEM[1112].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1113 GTM_gtm_cls0.MCS0_MEM[1113].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1114 GTM_gtm_cls0.MCS0_MEM[1114].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1115 GTM_gtm_cls0.MCS0_MEM[1115].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1116 GTM_gtm_cls0.MCS0_MEM[1116].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1117 GTM_gtm_cls0.MCS0_MEM[1117].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1118 GTM_gtm_cls0.MCS0_MEM[1118].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1119 GTM_gtm_cls0.MCS0_MEM[1119].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1120 GTM_gtm_cls0.MCS0_MEM[1120].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1121 GTM_gtm_cls0.MCS0_MEM[1121].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1122 GTM_gtm_cls0.MCS0_MEM[1122].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1123 GTM_gtm_cls0.MCS0_MEM[1123].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1124 GTM_gtm_cls0.MCS0_MEM[1124].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1125 GTM_gtm_cls0.MCS0_MEM[1125].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1126 GTM_gtm_cls0.MCS0_MEM[1126].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1127 GTM_gtm_cls0.MCS0_MEM[1127].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1128 GTM_gtm_cls0.MCS0_MEM[1128].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1129 GTM_gtm_cls0.MCS0_MEM[1129].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1130 GTM_gtm_cls0.MCS0_MEM[1130].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1131 GTM_gtm_cls0.MCS0_MEM[1131].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1132 GTM_gtm_cls0.MCS0_MEM[1132].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1133 GTM_gtm_cls0.MCS0_MEM[1133].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1134 GTM_gtm_cls0.MCS0_MEM[1134].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1135 GTM_gtm_cls0.MCS0_MEM[1135].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1136 GTM_gtm_cls0.MCS0_MEM[1136].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1137 GTM_gtm_cls0.MCS0_MEM[1137].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1138 GTM_gtm_cls0.MCS0_MEM[1138].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1139 GTM_gtm_cls0.MCS0_MEM[1139].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1140 GTM_gtm_cls0.MCS0_MEM[1140].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1141 GTM_gtm_cls0.MCS0_MEM[1141].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1142 GTM_gtm_cls0.MCS0_MEM[1142].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1143 GTM_gtm_cls0.MCS0_MEM[1143].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1144 GTM_gtm_cls0.MCS0_MEM[1144].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1145 GTM_gtm_cls0.MCS0_MEM[1145].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1146 GTM_gtm_cls0.MCS0_MEM[1146].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1147 GTM_gtm_cls0.MCS0_MEM[1147].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1148 GTM_gtm_cls0.MCS0_MEM[1148].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1149 GTM_gtm_cls0.MCS0_MEM[1149].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1150 GTM_gtm_cls0.MCS0_MEM[1150].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1151 GTM_gtm_cls0.MCS0_MEM[1151].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1152 GTM_gtm_cls0.MCS0_MEM[1152].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1153 GTM_gtm_cls0.MCS0_MEM[1153].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1154 GTM_gtm_cls0.MCS0_MEM[1154].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1155 GTM_gtm_cls0.MCS0_MEM[1155].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1156 GTM_gtm_cls0.MCS0_MEM[1156].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1157 GTM_gtm_cls0.MCS0_MEM[1157].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1158 GTM_gtm_cls0.MCS0_MEM[1158].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1159 GTM_gtm_cls0.MCS0_MEM[1159].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1160 GTM_gtm_cls0.MCS0_MEM[1160].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1161 GTM_gtm_cls0.MCS0_MEM[1161].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1162 GTM_gtm_cls0.MCS0_MEM[1162].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1163 GTM_gtm_cls0.MCS0_MEM[1163].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1164 GTM_gtm_cls0.MCS0_MEM[1164].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1165 GTM_gtm_cls0.MCS0_MEM[1165].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1166 GTM_gtm_cls0.MCS0_MEM[1166].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1167 GTM_gtm_cls0.MCS0_MEM[1167].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1168 GTM_gtm_cls0.MCS0_MEM[1168].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1169 GTM_gtm_cls0.MCS0_MEM[1169].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1170 GTM_gtm_cls0.MCS0_MEM[1170].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1171 GTM_gtm_cls0.MCS0_MEM[1171].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1172 GTM_gtm_cls0.MCS0_MEM[1172].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1173 GTM_gtm_cls0.MCS0_MEM[1173].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1174 GTM_gtm_cls0.MCS0_MEM[1174].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1175 GTM_gtm_cls0.MCS0_MEM[1175].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1176 GTM_gtm_cls0.MCS0_MEM[1176].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1177 GTM_gtm_cls0.MCS0_MEM[1177].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1178 GTM_gtm_cls0.MCS0_MEM[1178].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1179 GTM_gtm_cls0.MCS0_MEM[1179].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1180 GTM_gtm_cls0.MCS0_MEM[1180].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1181 GTM_gtm_cls0.MCS0_MEM[1181].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1182 GTM_gtm_cls0.MCS0_MEM[1182].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1183 GTM_gtm_cls0.MCS0_MEM[1183].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1184 GTM_gtm_cls0.MCS0_MEM[1184].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1185 GTM_gtm_cls0.MCS0_MEM[1185].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1186 GTM_gtm_cls0.MCS0_MEM[1186].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1187 GTM_gtm_cls0.MCS0_MEM[1187].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1188 GTM_gtm_cls0.MCS0_MEM[1188].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1189 GTM_gtm_cls0.MCS0_MEM[1189].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1190 GTM_gtm_cls0.MCS0_MEM[1190].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1191 GTM_gtm_cls0.MCS0_MEM[1191].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1192 GTM_gtm_cls0.MCS0_MEM[1192].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1193 GTM_gtm_cls0.MCS0_MEM[1193].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1194 GTM_gtm_cls0.MCS0_MEM[1194].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1195 GTM_gtm_cls0.MCS0_MEM[1195].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1196 GTM_gtm_cls0.MCS0_MEM[1196].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1197 GTM_gtm_cls0.MCS0_MEM[1197].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1198 GTM_gtm_cls0.MCS0_MEM[1198].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1199 GTM_gtm_cls0.MCS0_MEM[1199].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1200 GTM_gtm_cls0.MCS0_MEM[1200].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1201 GTM_gtm_cls0.MCS0_MEM[1201].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1202 GTM_gtm_cls0.MCS0_MEM[1202].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1203 GTM_gtm_cls0.MCS0_MEM[1203].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1204 GTM_gtm_cls0.MCS0_MEM[1204].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1205 GTM_gtm_cls0.MCS0_MEM[1205].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1206 GTM_gtm_cls0.MCS0_MEM[1206].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1207 GTM_gtm_cls0.MCS0_MEM[1207].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1208 GTM_gtm_cls0.MCS0_MEM[1208].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1209 GTM_gtm_cls0.MCS0_MEM[1209].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1210 GTM_gtm_cls0.MCS0_MEM[1210].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1211 GTM_gtm_cls0.MCS0_MEM[1211].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1212 GTM_gtm_cls0.MCS0_MEM[1212].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1213 GTM_gtm_cls0.MCS0_MEM[1213].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1214 GTM_gtm_cls0.MCS0_MEM[1214].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1215 GTM_gtm_cls0.MCS0_MEM[1215].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1216 GTM_gtm_cls0.MCS0_MEM[1216].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1217 GTM_gtm_cls0.MCS0_MEM[1217].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1218 GTM_gtm_cls0.MCS0_MEM[1218].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1219 GTM_gtm_cls0.MCS0_MEM[1219].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1220 GTM_gtm_cls0.MCS0_MEM[1220].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1221 GTM_gtm_cls0.MCS0_MEM[1221].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1222 GTM_gtm_cls0.MCS0_MEM[1222].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1223 GTM_gtm_cls0.MCS0_MEM[1223].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1224 GTM_gtm_cls0.MCS0_MEM[1224].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1225 GTM_gtm_cls0.MCS0_MEM[1225].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1226 GTM_gtm_cls0.MCS0_MEM[1226].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1227 GTM_gtm_cls0.MCS0_MEM[1227].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1228 GTM_gtm_cls0.MCS0_MEM[1228].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1229 GTM_gtm_cls0.MCS0_MEM[1229].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1230 GTM_gtm_cls0.MCS0_MEM[1230].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1231 GTM_gtm_cls0.MCS0_MEM[1231].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1232 GTM_gtm_cls0.MCS0_MEM[1232].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1233 GTM_gtm_cls0.MCS0_MEM[1233].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1234 GTM_gtm_cls0.MCS0_MEM[1234].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1235 GTM_gtm_cls0.MCS0_MEM[1235].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1236 GTM_gtm_cls0.MCS0_MEM[1236].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1237 GTM_gtm_cls0.MCS0_MEM[1237].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1238 GTM_gtm_cls0.MCS0_MEM[1238].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1239 GTM_gtm_cls0.MCS0_MEM[1239].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1240 GTM_gtm_cls0.MCS0_MEM[1240].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1241 GTM_gtm_cls0.MCS0_MEM[1241].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1242 GTM_gtm_cls0.MCS0_MEM[1242].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1243 GTM_gtm_cls0.MCS0_MEM[1243].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1244 GTM_gtm_cls0.MCS0_MEM[1244].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1245 GTM_gtm_cls0.MCS0_MEM[1245].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1246 GTM_gtm_cls0.MCS0_MEM[1246].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1247 GTM_gtm_cls0.MCS0_MEM[1247].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1248 GTM_gtm_cls0.MCS0_MEM[1248].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1249 GTM_gtm_cls0.MCS0_MEM[1249].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1250 GTM_gtm_cls0.MCS0_MEM[1250].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1251 GTM_gtm_cls0.MCS0_MEM[1251].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1252 GTM_gtm_cls0.MCS0_MEM[1252].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1253 GTM_gtm_cls0.MCS0_MEM[1253].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1254 GTM_gtm_cls0.MCS0_MEM[1254].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1255 GTM_gtm_cls0.MCS0_MEM[1255].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1256 GTM_gtm_cls0.MCS0_MEM[1256].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1257 GTM_gtm_cls0.MCS0_MEM[1257].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1258 GTM_gtm_cls0.MCS0_MEM[1258].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1259 GTM_gtm_cls0.MCS0_MEM[1259].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1260 GTM_gtm_cls0.MCS0_MEM[1260].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1261 GTM_gtm_cls0.MCS0_MEM[1261].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1262 GTM_gtm_cls0.MCS0_MEM[1262].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1263 GTM_gtm_cls0.MCS0_MEM[1263].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1264 GTM_gtm_cls0.MCS0_MEM[1264].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1265 GTM_gtm_cls0.MCS0_MEM[1265].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1266 GTM_gtm_cls0.MCS0_MEM[1266].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1267 GTM_gtm_cls0.MCS0_MEM[1267].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1268 GTM_gtm_cls0.MCS0_MEM[1268].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1269 GTM_gtm_cls0.MCS0_MEM[1269].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1270 GTM_gtm_cls0.MCS0_MEM[1270].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1271 GTM_gtm_cls0.MCS0_MEM[1271].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1272 GTM_gtm_cls0.MCS0_MEM[1272].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1273 GTM_gtm_cls0.MCS0_MEM[1273].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1274 GTM_gtm_cls0.MCS0_MEM[1274].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1275 GTM_gtm_cls0.MCS0_MEM[1275].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1276 GTM_gtm_cls0.MCS0_MEM[1276].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1277 GTM_gtm_cls0.MCS0_MEM[1277].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1278 GTM_gtm_cls0.MCS0_MEM[1278].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1279 GTM_gtm_cls0.MCS0_MEM[1279].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1280 GTM_gtm_cls0.MCS0_MEM[1280].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1281 GTM_gtm_cls0.MCS0_MEM[1281].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1282 GTM_gtm_cls0.MCS0_MEM[1282].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1283 GTM_gtm_cls0.MCS0_MEM[1283].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1284 GTM_gtm_cls0.MCS0_MEM[1284].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1285 GTM_gtm_cls0.MCS0_MEM[1285].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1286 GTM_gtm_cls0.MCS0_MEM[1286].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1287 GTM_gtm_cls0.MCS0_MEM[1287].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1288 GTM_gtm_cls0.MCS0_MEM[1288].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1289 GTM_gtm_cls0.MCS0_MEM[1289].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1290 GTM_gtm_cls0.MCS0_MEM[1290].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1291 GTM_gtm_cls0.MCS0_MEM[1291].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1292 GTM_gtm_cls0.MCS0_MEM[1292].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1293 GTM_gtm_cls0.MCS0_MEM[1293].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1294 GTM_gtm_cls0.MCS0_MEM[1294].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1295 GTM_gtm_cls0.MCS0_MEM[1295].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1296 GTM_gtm_cls0.MCS0_MEM[1296].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1297 GTM_gtm_cls0.MCS0_MEM[1297].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1298 GTM_gtm_cls0.MCS0_MEM[1298].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1299 GTM_gtm_cls0.MCS0_MEM[1299].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1300 GTM_gtm_cls0.MCS0_MEM[1300].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1301 GTM_gtm_cls0.MCS0_MEM[1301].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1302 GTM_gtm_cls0.MCS0_MEM[1302].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1303 GTM_gtm_cls0.MCS0_MEM[1303].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1304 GTM_gtm_cls0.MCS0_MEM[1304].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1305 GTM_gtm_cls0.MCS0_MEM[1305].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1306 GTM_gtm_cls0.MCS0_MEM[1306].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1307 GTM_gtm_cls0.MCS0_MEM[1307].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1308 GTM_gtm_cls0.MCS0_MEM[1308].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1309 GTM_gtm_cls0.MCS0_MEM[1309].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1310 GTM_gtm_cls0.MCS0_MEM[1310].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1311 GTM_gtm_cls0.MCS0_MEM[1311].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1312 GTM_gtm_cls0.MCS0_MEM[1312].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1313 GTM_gtm_cls0.MCS0_MEM[1313].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1314 GTM_gtm_cls0.MCS0_MEM[1314].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1315 GTM_gtm_cls0.MCS0_MEM[1315].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1316 GTM_gtm_cls0.MCS0_MEM[1316].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1317 GTM_gtm_cls0.MCS0_MEM[1317].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1318 GTM_gtm_cls0.MCS0_MEM[1318].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1319 GTM_gtm_cls0.MCS0_MEM[1319].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1320 GTM_gtm_cls0.MCS0_MEM[1320].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1321 GTM_gtm_cls0.MCS0_MEM[1321].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1322 GTM_gtm_cls0.MCS0_MEM[1322].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1323 GTM_gtm_cls0.MCS0_MEM[1323].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1324 GTM_gtm_cls0.MCS0_MEM[1324].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1325 GTM_gtm_cls0.MCS0_MEM[1325].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1326 GTM_gtm_cls0.MCS0_MEM[1326].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1327 GTM_gtm_cls0.MCS0_MEM[1327].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1328 GTM_gtm_cls0.MCS0_MEM[1328].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1329 GTM_gtm_cls0.MCS0_MEM[1329].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1330 GTM_gtm_cls0.MCS0_MEM[1330].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1331 GTM_gtm_cls0.MCS0_MEM[1331].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1332 GTM_gtm_cls0.MCS0_MEM[1332].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1333 GTM_gtm_cls0.MCS0_MEM[1333].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1334 GTM_gtm_cls0.MCS0_MEM[1334].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1335 GTM_gtm_cls0.MCS0_MEM[1335].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1336 GTM_gtm_cls0.MCS0_MEM[1336].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1337 GTM_gtm_cls0.MCS0_MEM[1337].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1338 GTM_gtm_cls0.MCS0_MEM[1338].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1339 GTM_gtm_cls0.MCS0_MEM[1339].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1340 GTM_gtm_cls0.MCS0_MEM[1340].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1341 GTM_gtm_cls0.MCS0_MEM[1341].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1342 GTM_gtm_cls0.MCS0_MEM[1342].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1343 GTM_gtm_cls0.MCS0_MEM[1343].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1344 GTM_gtm_cls0.MCS0_MEM[1344].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1345 GTM_gtm_cls0.MCS0_MEM[1345].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1346 GTM_gtm_cls0.MCS0_MEM[1346].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1347 GTM_gtm_cls0.MCS0_MEM[1347].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1348 GTM_gtm_cls0.MCS0_MEM[1348].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1349 GTM_gtm_cls0.MCS0_MEM[1349].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1350 GTM_gtm_cls0.MCS0_MEM[1350].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1351 GTM_gtm_cls0.MCS0_MEM[1351].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1352 GTM_gtm_cls0.MCS0_MEM[1352].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1353 GTM_gtm_cls0.MCS0_MEM[1353].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1354 GTM_gtm_cls0.MCS0_MEM[1354].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1355 GTM_gtm_cls0.MCS0_MEM[1355].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1356 GTM_gtm_cls0.MCS0_MEM[1356].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1357 GTM_gtm_cls0.MCS0_MEM[1357].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1358 GTM_gtm_cls0.MCS0_MEM[1358].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1359 GTM_gtm_cls0.MCS0_MEM[1359].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1360 GTM_gtm_cls0.MCS0_MEM[1360].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1361 GTM_gtm_cls0.MCS0_MEM[1361].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1362 GTM_gtm_cls0.MCS0_MEM[1362].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1363 GTM_gtm_cls0.MCS0_MEM[1363].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1364 GTM_gtm_cls0.MCS0_MEM[1364].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1365 GTM_gtm_cls0.MCS0_MEM[1365].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1366 GTM_gtm_cls0.MCS0_MEM[1366].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1367 GTM_gtm_cls0.MCS0_MEM[1367].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1368 GTM_gtm_cls0.MCS0_MEM[1368].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1369 GTM_gtm_cls0.MCS0_MEM[1369].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1370 GTM_gtm_cls0.MCS0_MEM[1370].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1371 GTM_gtm_cls0.MCS0_MEM[1371].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1372 GTM_gtm_cls0.MCS0_MEM[1372].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1373 GTM_gtm_cls0.MCS0_MEM[1373].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1374 GTM_gtm_cls0.MCS0_MEM[1374].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1375 GTM_gtm_cls0.MCS0_MEM[1375].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1376 GTM_gtm_cls0.MCS0_MEM[1376].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1377 GTM_gtm_cls0.MCS0_MEM[1377].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1378 GTM_gtm_cls0.MCS0_MEM[1378].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1379 GTM_gtm_cls0.MCS0_MEM[1379].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1380 GTM_gtm_cls0.MCS0_MEM[1380].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1381 GTM_gtm_cls0.MCS0_MEM[1381].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1382 GTM_gtm_cls0.MCS0_MEM[1382].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1383 GTM_gtm_cls0.MCS0_MEM[1383].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1384 GTM_gtm_cls0.MCS0_MEM[1384].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1385 GTM_gtm_cls0.MCS0_MEM[1385].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1386 GTM_gtm_cls0.MCS0_MEM[1386].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1387 GTM_gtm_cls0.MCS0_MEM[1387].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1388 GTM_gtm_cls0.MCS0_MEM[1388].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1389 GTM_gtm_cls0.MCS0_MEM[1389].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1390 GTM_gtm_cls0.MCS0_MEM[1390].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1391 GTM_gtm_cls0.MCS0_MEM[1391].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1392 GTM_gtm_cls0.MCS0_MEM[1392].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1393 GTM_gtm_cls0.MCS0_MEM[1393].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1394 GTM_gtm_cls0.MCS0_MEM[1394].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1395 GTM_gtm_cls0.MCS0_MEM[1395].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1396 GTM_gtm_cls0.MCS0_MEM[1396].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1397 GTM_gtm_cls0.MCS0_MEM[1397].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1398 GTM_gtm_cls0.MCS0_MEM[1398].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1399 GTM_gtm_cls0.MCS0_MEM[1399].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1400 GTM_gtm_cls0.MCS0_MEM[1400].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1401 GTM_gtm_cls0.MCS0_MEM[1401].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1402 GTM_gtm_cls0.MCS0_MEM[1402].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1403 GTM_gtm_cls0.MCS0_MEM[1403].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1404 GTM_gtm_cls0.MCS0_MEM[1404].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1405 GTM_gtm_cls0.MCS0_MEM[1405].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1406 GTM_gtm_cls0.MCS0_MEM[1406].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1407 GTM_gtm_cls0.MCS0_MEM[1407].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1408 GTM_gtm_cls0.MCS0_MEM[1408].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1409 GTM_gtm_cls0.MCS0_MEM[1409].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1410 GTM_gtm_cls0.MCS0_MEM[1410].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1411 GTM_gtm_cls0.MCS0_MEM[1411].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1412 GTM_gtm_cls0.MCS0_MEM[1412].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1413 GTM_gtm_cls0.MCS0_MEM[1413].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1414 GTM_gtm_cls0.MCS0_MEM[1414].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1415 GTM_gtm_cls0.MCS0_MEM[1415].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1416 GTM_gtm_cls0.MCS0_MEM[1416].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1417 GTM_gtm_cls0.MCS0_MEM[1417].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1418 GTM_gtm_cls0.MCS0_MEM[1418].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1419 GTM_gtm_cls0.MCS0_MEM[1419].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1420 GTM_gtm_cls0.MCS0_MEM[1420].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1421 GTM_gtm_cls0.MCS0_MEM[1421].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1422 GTM_gtm_cls0.MCS0_MEM[1422].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1423 GTM_gtm_cls0.MCS0_MEM[1423].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1424 GTM_gtm_cls0.MCS0_MEM[1424].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1425 GTM_gtm_cls0.MCS0_MEM[1425].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1426 GTM_gtm_cls0.MCS0_MEM[1426].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1427 GTM_gtm_cls0.MCS0_MEM[1427].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1428 GTM_gtm_cls0.MCS0_MEM[1428].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1429 GTM_gtm_cls0.MCS0_MEM[1429].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1430 GTM_gtm_cls0.MCS0_MEM[1430].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1431 GTM_gtm_cls0.MCS0_MEM[1431].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1432 GTM_gtm_cls0.MCS0_MEM[1432].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1433 GTM_gtm_cls0.MCS0_MEM[1433].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1434 GTM_gtm_cls0.MCS0_MEM[1434].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1435 GTM_gtm_cls0.MCS0_MEM[1435].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1436 GTM_gtm_cls0.MCS0_MEM[1436].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1437 GTM_gtm_cls0.MCS0_MEM[1437].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1438 GTM_gtm_cls0.MCS0_MEM[1438].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1439 GTM_gtm_cls0.MCS0_MEM[1439].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1440 GTM_gtm_cls0.MCS0_MEM[1440].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1441 GTM_gtm_cls0.MCS0_MEM[1441].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1442 GTM_gtm_cls0.MCS0_MEM[1442].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1443 GTM_gtm_cls0.MCS0_MEM[1443].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1444 GTM_gtm_cls0.MCS0_MEM[1444].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1445 GTM_gtm_cls0.MCS0_MEM[1445].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1446 GTM_gtm_cls0.MCS0_MEM[1446].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1447 GTM_gtm_cls0.MCS0_MEM[1447].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1448 GTM_gtm_cls0.MCS0_MEM[1448].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1449 GTM_gtm_cls0.MCS0_MEM[1449].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1450 GTM_gtm_cls0.MCS0_MEM[1450].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1451 GTM_gtm_cls0.MCS0_MEM[1451].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1452 GTM_gtm_cls0.MCS0_MEM[1452].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1453 GTM_gtm_cls0.MCS0_MEM[1453].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1454 GTM_gtm_cls0.MCS0_MEM[1454].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1455 GTM_gtm_cls0.MCS0_MEM[1455].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1456 GTM_gtm_cls0.MCS0_MEM[1456].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1457 GTM_gtm_cls0.MCS0_MEM[1457].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1458 GTM_gtm_cls0.MCS0_MEM[1458].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1459 GTM_gtm_cls0.MCS0_MEM[1459].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1460 GTM_gtm_cls0.MCS0_MEM[1460].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1461 GTM_gtm_cls0.MCS0_MEM[1461].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1462 GTM_gtm_cls0.MCS0_MEM[1462].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1463 GTM_gtm_cls0.MCS0_MEM[1463].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1464 GTM_gtm_cls0.MCS0_MEM[1464].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1465 GTM_gtm_cls0.MCS0_MEM[1465].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1466 GTM_gtm_cls0.MCS0_MEM[1466].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1467 GTM_gtm_cls0.MCS0_MEM[1467].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1468 GTM_gtm_cls0.MCS0_MEM[1468].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1469 GTM_gtm_cls0.MCS0_MEM[1469].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1470 GTM_gtm_cls0.MCS0_MEM[1470].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1471 GTM_gtm_cls0.MCS0_MEM[1471].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1472 GTM_gtm_cls0.MCS0_MEM[1472].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1473 GTM_gtm_cls0.MCS0_MEM[1473].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1474 GTM_gtm_cls0.MCS0_MEM[1474].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1475 GTM_gtm_cls0.MCS0_MEM[1475].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1476 GTM_gtm_cls0.MCS0_MEM[1476].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1477 GTM_gtm_cls0.MCS0_MEM[1477].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1478 GTM_gtm_cls0.MCS0_MEM[1478].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1479 GTM_gtm_cls0.MCS0_MEM[1479].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1480 GTM_gtm_cls0.MCS0_MEM[1480].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1481 GTM_gtm_cls0.MCS0_MEM[1481].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1482 GTM_gtm_cls0.MCS0_MEM[1482].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1483 GTM_gtm_cls0.MCS0_MEM[1483].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1484 GTM_gtm_cls0.MCS0_MEM[1484].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1485 GTM_gtm_cls0.MCS0_MEM[1485].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1486 GTM_gtm_cls0.MCS0_MEM[1486].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1487 GTM_gtm_cls0.MCS0_MEM[1487].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1488 GTM_gtm_cls0.MCS0_MEM[1488].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1489 GTM_gtm_cls0.MCS0_MEM[1489].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1490 GTM_gtm_cls0.MCS0_MEM[1490].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1491 GTM_gtm_cls0.MCS0_MEM[1491].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1492 GTM_gtm_cls0.MCS0_MEM[1492].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1493 GTM_gtm_cls0.MCS0_MEM[1493].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1494 GTM_gtm_cls0.MCS0_MEM[1494].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1495 GTM_gtm_cls0.MCS0_MEM[1495].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1496 GTM_gtm_cls0.MCS0_MEM[1496].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1497 GTM_gtm_cls0.MCS0_MEM[1497].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1498 GTM_gtm_cls0.MCS0_MEM[1498].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1499 GTM_gtm_cls0.MCS0_MEM[1499].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1500 GTM_gtm_cls0.MCS0_MEM[1500].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1501 GTM_gtm_cls0.MCS0_MEM[1501].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1502 GTM_gtm_cls0.MCS0_MEM[1502].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1503 GTM_gtm_cls0.MCS0_MEM[1503].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1504 GTM_gtm_cls0.MCS0_MEM[1504].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1505 GTM_gtm_cls0.MCS0_MEM[1505].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1506 GTM_gtm_cls0.MCS0_MEM[1506].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1507 GTM_gtm_cls0.MCS0_MEM[1507].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1508 GTM_gtm_cls0.MCS0_MEM[1508].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1509 GTM_gtm_cls0.MCS0_MEM[1509].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1510 GTM_gtm_cls0.MCS0_MEM[1510].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1511 GTM_gtm_cls0.MCS0_MEM[1511].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1512 GTM_gtm_cls0.MCS0_MEM[1512].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1513 GTM_gtm_cls0.MCS0_MEM[1513].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1514 GTM_gtm_cls0.MCS0_MEM[1514].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1515 GTM_gtm_cls0.MCS0_MEM[1515].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1516 GTM_gtm_cls0.MCS0_MEM[1516].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1517 GTM_gtm_cls0.MCS0_MEM[1517].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1518 GTM_gtm_cls0.MCS0_MEM[1518].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1519 GTM_gtm_cls0.MCS0_MEM[1519].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1520 GTM_gtm_cls0.MCS0_MEM[1520].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1521 GTM_gtm_cls0.MCS0_MEM[1521].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1522 GTM_gtm_cls0.MCS0_MEM[1522].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1523 GTM_gtm_cls0.MCS0_MEM[1523].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1524 GTM_gtm_cls0.MCS0_MEM[1524].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1525 GTM_gtm_cls0.MCS0_MEM[1525].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1526 GTM_gtm_cls0.MCS0_MEM[1526].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1527 GTM_gtm_cls0.MCS0_MEM[1527].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1528 GTM_gtm_cls0.MCS0_MEM[1528].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1529 GTM_gtm_cls0.MCS0_MEM[1529].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1530 GTM_gtm_cls0.MCS0_MEM[1530].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1531 GTM_gtm_cls0.MCS0_MEM[1531].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1532 GTM_gtm_cls0.MCS0_MEM[1532].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1533 GTM_gtm_cls0.MCS0_MEM[1533].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1534 GTM_gtm_cls0.MCS0_MEM[1534].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1535 GTM_gtm_cls0.MCS0_MEM[1535].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1536 GTM_gtm_cls0.MCS0_MEM[1536].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1537 GTM_gtm_cls0.MCS0_MEM[1537].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1538 GTM_gtm_cls0.MCS0_MEM[1538].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1539 GTM_gtm_cls0.MCS0_MEM[1539].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1540 GTM_gtm_cls0.MCS0_MEM[1540].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1541 GTM_gtm_cls0.MCS0_MEM[1541].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1542 GTM_gtm_cls0.MCS0_MEM[1542].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1543 GTM_gtm_cls0.MCS0_MEM[1543].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1544 GTM_gtm_cls0.MCS0_MEM[1544].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1545 GTM_gtm_cls0.MCS0_MEM[1545].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1546 GTM_gtm_cls0.MCS0_MEM[1546].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1547 GTM_gtm_cls0.MCS0_MEM[1547].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1548 GTM_gtm_cls0.MCS0_MEM[1548].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1549 GTM_gtm_cls0.MCS0_MEM[1549].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1550 GTM_gtm_cls0.MCS0_MEM[1550].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1551 GTM_gtm_cls0.MCS0_MEM[1551].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1552 GTM_gtm_cls0.MCS0_MEM[1552].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1553 GTM_gtm_cls0.MCS0_MEM[1553].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1554 GTM_gtm_cls0.MCS0_MEM[1554].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1555 GTM_gtm_cls0.MCS0_MEM[1555].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1556 GTM_gtm_cls0.MCS0_MEM[1556].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1557 GTM_gtm_cls0.MCS0_MEM[1557].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1558 GTM_gtm_cls0.MCS0_MEM[1558].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1559 GTM_gtm_cls0.MCS0_MEM[1559].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1560 GTM_gtm_cls0.MCS0_MEM[1560].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1561 GTM_gtm_cls0.MCS0_MEM[1561].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1562 GTM_gtm_cls0.MCS0_MEM[1562].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1563 GTM_gtm_cls0.MCS0_MEM[1563].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1564 GTM_gtm_cls0.MCS0_MEM[1564].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1565 GTM_gtm_cls0.MCS0_MEM[1565].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1566 GTM_gtm_cls0.MCS0_MEM[1566].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1567 GTM_gtm_cls0.MCS0_MEM[1567].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1568 GTM_gtm_cls0.MCS0_MEM[1568].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1569 GTM_gtm_cls0.MCS0_MEM[1569].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1570 GTM_gtm_cls0.MCS0_MEM[1570].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1571 GTM_gtm_cls0.MCS0_MEM[1571].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1572 GTM_gtm_cls0.MCS0_MEM[1572].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1573 GTM_gtm_cls0.MCS0_MEM[1573].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1574 GTM_gtm_cls0.MCS0_MEM[1574].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1575 GTM_gtm_cls0.MCS0_MEM[1575].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1576 GTM_gtm_cls0.MCS0_MEM[1576].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1577 GTM_gtm_cls0.MCS0_MEM[1577].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1578 GTM_gtm_cls0.MCS0_MEM[1578].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1579 GTM_gtm_cls0.MCS0_MEM[1579].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1580 GTM_gtm_cls0.MCS0_MEM[1580].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1581 GTM_gtm_cls0.MCS0_MEM[1581].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1582 GTM_gtm_cls0.MCS0_MEM[1582].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1583 GTM_gtm_cls0.MCS0_MEM[1583].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1584 GTM_gtm_cls0.MCS0_MEM[1584].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1585 GTM_gtm_cls0.MCS0_MEM[1585].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1586 GTM_gtm_cls0.MCS0_MEM[1586].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1587 GTM_gtm_cls0.MCS0_MEM[1587].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1588 GTM_gtm_cls0.MCS0_MEM[1588].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1589 GTM_gtm_cls0.MCS0_MEM[1589].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1590 GTM_gtm_cls0.MCS0_MEM[1590].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1591 GTM_gtm_cls0.MCS0_MEM[1591].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1592 GTM_gtm_cls0.MCS0_MEM[1592].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1593 GTM_gtm_cls0.MCS0_MEM[1593].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1594 GTM_gtm_cls0.MCS0_MEM[1594].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1595 GTM_gtm_cls0.MCS0_MEM[1595].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1596 GTM_gtm_cls0.MCS0_MEM[1596].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1597 GTM_gtm_cls0.MCS0_MEM[1597].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1598 GTM_gtm_cls0.MCS0_MEM[1598].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1599 GTM_gtm_cls0.MCS0_MEM[1599].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1600 GTM_gtm_cls0.MCS0_MEM[1600].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1601 GTM_gtm_cls0.MCS0_MEM[1601].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1602 GTM_gtm_cls0.MCS0_MEM[1602].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1603 GTM_gtm_cls0.MCS0_MEM[1603].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1604 GTM_gtm_cls0.MCS0_MEM[1604].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1605 GTM_gtm_cls0.MCS0_MEM[1605].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1606 GTM_gtm_cls0.MCS0_MEM[1606].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1607 GTM_gtm_cls0.MCS0_MEM[1607].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1608 GTM_gtm_cls0.MCS0_MEM[1608].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1609 GTM_gtm_cls0.MCS0_MEM[1609].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1610 GTM_gtm_cls0.MCS0_MEM[1610].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1611 GTM_gtm_cls0.MCS0_MEM[1611].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1612 GTM_gtm_cls0.MCS0_MEM[1612].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1613 GTM_gtm_cls0.MCS0_MEM[1613].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1614 GTM_gtm_cls0.MCS0_MEM[1614].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1615 GTM_gtm_cls0.MCS0_MEM[1615].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1616 GTM_gtm_cls0.MCS0_MEM[1616].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1617 GTM_gtm_cls0.MCS0_MEM[1617].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1618 GTM_gtm_cls0.MCS0_MEM[1618].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1619 GTM_gtm_cls0.MCS0_MEM[1619].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1620 GTM_gtm_cls0.MCS0_MEM[1620].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1621 GTM_gtm_cls0.MCS0_MEM[1621].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1622 GTM_gtm_cls0.MCS0_MEM[1622].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1623 GTM_gtm_cls0.MCS0_MEM[1623].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1624 GTM_gtm_cls0.MCS0_MEM[1624].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1625 GTM_gtm_cls0.MCS0_MEM[1625].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1626 GTM_gtm_cls0.MCS0_MEM[1626].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1627 GTM_gtm_cls0.MCS0_MEM[1627].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1628 GTM_gtm_cls0.MCS0_MEM[1628].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1629 GTM_gtm_cls0.MCS0_MEM[1629].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1630 GTM_gtm_cls0.MCS0_MEM[1630].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1631 GTM_gtm_cls0.MCS0_MEM[1631].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1632 GTM_gtm_cls0.MCS0_MEM[1632].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1633 GTM_gtm_cls0.MCS0_MEM[1633].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1634 GTM_gtm_cls0.MCS0_MEM[1634].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1635 GTM_gtm_cls0.MCS0_MEM[1635].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1636 GTM_gtm_cls0.MCS0_MEM[1636].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1637 GTM_gtm_cls0.MCS0_MEM[1637].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1638 GTM_gtm_cls0.MCS0_MEM[1638].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1639 GTM_gtm_cls0.MCS0_MEM[1639].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1640 GTM_gtm_cls0.MCS0_MEM[1640].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1641 GTM_gtm_cls0.MCS0_MEM[1641].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1642 GTM_gtm_cls0.MCS0_MEM[1642].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1643 GTM_gtm_cls0.MCS0_MEM[1643].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1644 GTM_gtm_cls0.MCS0_MEM[1644].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1645 GTM_gtm_cls0.MCS0_MEM[1645].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1646 GTM_gtm_cls0.MCS0_MEM[1646].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1647 GTM_gtm_cls0.MCS0_MEM[1647].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1648 GTM_gtm_cls0.MCS0_MEM[1648].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1649 GTM_gtm_cls0.MCS0_MEM[1649].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1650 GTM_gtm_cls0.MCS0_MEM[1650].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1651 GTM_gtm_cls0.MCS0_MEM[1651].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1652 GTM_gtm_cls0.MCS0_MEM[1652].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1653 GTM_gtm_cls0.MCS0_MEM[1653].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1654 GTM_gtm_cls0.MCS0_MEM[1654].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1655 GTM_gtm_cls0.MCS0_MEM[1655].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1656 GTM_gtm_cls0.MCS0_MEM[1656].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1657 GTM_gtm_cls0.MCS0_MEM[1657].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1658 GTM_gtm_cls0.MCS0_MEM[1658].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1659 GTM_gtm_cls0.MCS0_MEM[1659].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1660 GTM_gtm_cls0.MCS0_MEM[1660].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1661 GTM_gtm_cls0.MCS0_MEM[1661].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1662 GTM_gtm_cls0.MCS0_MEM[1662].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1663 GTM_gtm_cls0.MCS0_MEM[1663].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1664 GTM_gtm_cls0.MCS0_MEM[1664].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1665 GTM_gtm_cls0.MCS0_MEM[1665].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1666 GTM_gtm_cls0.MCS0_MEM[1666].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1667 GTM_gtm_cls0.MCS0_MEM[1667].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1668 GTM_gtm_cls0.MCS0_MEM[1668].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1669 GTM_gtm_cls0.MCS0_MEM[1669].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1670 GTM_gtm_cls0.MCS0_MEM[1670].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1671 GTM_gtm_cls0.MCS0_MEM[1671].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1672 GTM_gtm_cls0.MCS0_MEM[1672].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1673 GTM_gtm_cls0.MCS0_MEM[1673].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1674 GTM_gtm_cls0.MCS0_MEM[1674].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1675 GTM_gtm_cls0.MCS0_MEM[1675].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1676 GTM_gtm_cls0.MCS0_MEM[1676].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1677 GTM_gtm_cls0.MCS0_MEM[1677].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1678 GTM_gtm_cls0.MCS0_MEM[1678].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1679 GTM_gtm_cls0.MCS0_MEM[1679].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1680 GTM_gtm_cls0.MCS0_MEM[1680].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1681 GTM_gtm_cls0.MCS0_MEM[1681].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1682 GTM_gtm_cls0.MCS0_MEM[1682].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1683 GTM_gtm_cls0.MCS0_MEM[1683].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1684 GTM_gtm_cls0.MCS0_MEM[1684].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1685 GTM_gtm_cls0.MCS0_MEM[1685].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1686 GTM_gtm_cls0.MCS0_MEM[1686].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1687 GTM_gtm_cls0.MCS0_MEM[1687].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1688 GTM_gtm_cls0.MCS0_MEM[1688].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1689 GTM_gtm_cls0.MCS0_MEM[1689].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1690 GTM_gtm_cls0.MCS0_MEM[1690].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1691 GTM_gtm_cls0.MCS0_MEM[1691].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1692 GTM_gtm_cls0.MCS0_MEM[1692].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1693 GTM_gtm_cls0.MCS0_MEM[1693].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1694 GTM_gtm_cls0.MCS0_MEM[1694].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1695 GTM_gtm_cls0.MCS0_MEM[1695].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1696 GTM_gtm_cls0.MCS0_MEM[1696].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1697 GTM_gtm_cls0.MCS0_MEM[1697].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1698 GTM_gtm_cls0.MCS0_MEM[1698].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1699 GTM_gtm_cls0.MCS0_MEM[1699].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1700 GTM_gtm_cls0.MCS0_MEM[1700].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1701 GTM_gtm_cls0.MCS0_MEM[1701].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1702 GTM_gtm_cls0.MCS0_MEM[1702].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1703 GTM_gtm_cls0.MCS0_MEM[1703].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1704 GTM_gtm_cls0.MCS0_MEM[1704].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1705 GTM_gtm_cls0.MCS0_MEM[1705].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1706 GTM_gtm_cls0.MCS0_MEM[1706].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1707 GTM_gtm_cls0.MCS0_MEM[1707].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1708 GTM_gtm_cls0.MCS0_MEM[1708].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1709 GTM_gtm_cls0.MCS0_MEM[1709].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1710 GTM_gtm_cls0.MCS0_MEM[1710].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1711 GTM_gtm_cls0.MCS0_MEM[1711].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1712 GTM_gtm_cls0.MCS0_MEM[1712].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1713 GTM_gtm_cls0.MCS0_MEM[1713].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1714 GTM_gtm_cls0.MCS0_MEM[1714].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1715 GTM_gtm_cls0.MCS0_MEM[1715].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1716 GTM_gtm_cls0.MCS0_MEM[1716].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1717 GTM_gtm_cls0.MCS0_MEM[1717].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1718 GTM_gtm_cls0.MCS0_MEM[1718].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1719 GTM_gtm_cls0.MCS0_MEM[1719].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1720 GTM_gtm_cls0.MCS0_MEM[1720].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1721 GTM_gtm_cls0.MCS0_MEM[1721].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1722 GTM_gtm_cls0.MCS0_MEM[1722].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1723 GTM_gtm_cls0.MCS0_MEM[1723].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1724 GTM_gtm_cls0.MCS0_MEM[1724].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1725 GTM_gtm_cls0.MCS0_MEM[1725].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1726 GTM_gtm_cls0.MCS0_MEM[1726].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1727 GTM_gtm_cls0.MCS0_MEM[1727].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1728 GTM_gtm_cls0.MCS0_MEM[1728].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1729 GTM_gtm_cls0.MCS0_MEM[1729].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1730 GTM_gtm_cls0.MCS0_MEM[1730].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1731 GTM_gtm_cls0.MCS0_MEM[1731].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1732 GTM_gtm_cls0.MCS0_MEM[1732].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1733 GTM_gtm_cls0.MCS0_MEM[1733].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1734 GTM_gtm_cls0.MCS0_MEM[1734].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1735 GTM_gtm_cls0.MCS0_MEM[1735].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1736 GTM_gtm_cls0.MCS0_MEM[1736].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1737 GTM_gtm_cls0.MCS0_MEM[1737].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1738 GTM_gtm_cls0.MCS0_MEM[1738].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1739 GTM_gtm_cls0.MCS0_MEM[1739].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1740 GTM_gtm_cls0.MCS0_MEM[1740].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1741 GTM_gtm_cls0.MCS0_MEM[1741].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1742 GTM_gtm_cls0.MCS0_MEM[1742].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1743 GTM_gtm_cls0.MCS0_MEM[1743].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1744 GTM_gtm_cls0.MCS0_MEM[1744].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1745 GTM_gtm_cls0.MCS0_MEM[1745].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1746 GTM_gtm_cls0.MCS0_MEM[1746].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1747 GTM_gtm_cls0.MCS0_MEM[1747].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1748 GTM_gtm_cls0.MCS0_MEM[1748].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1749 GTM_gtm_cls0.MCS0_MEM[1749].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1750 GTM_gtm_cls0.MCS0_MEM[1750].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1751 GTM_gtm_cls0.MCS0_MEM[1751].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1752 GTM_gtm_cls0.MCS0_MEM[1752].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1753 GTM_gtm_cls0.MCS0_MEM[1753].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1754 GTM_gtm_cls0.MCS0_MEM[1754].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1755 GTM_gtm_cls0.MCS0_MEM[1755].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1756 GTM_gtm_cls0.MCS0_MEM[1756].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1757 GTM_gtm_cls0.MCS0_MEM[1757].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1758 GTM_gtm_cls0.MCS0_MEM[1758].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1759 GTM_gtm_cls0.MCS0_MEM[1759].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1760 GTM_gtm_cls0.MCS0_MEM[1760].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1761 GTM_gtm_cls0.MCS0_MEM[1761].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1762 GTM_gtm_cls0.MCS0_MEM[1762].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1763 GTM_gtm_cls0.MCS0_MEM[1763].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1764 GTM_gtm_cls0.MCS0_MEM[1764].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1765 GTM_gtm_cls0.MCS0_MEM[1765].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1766 GTM_gtm_cls0.MCS0_MEM[1766].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1767 GTM_gtm_cls0.MCS0_MEM[1767].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1768 GTM_gtm_cls0.MCS0_MEM[1768].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1769 GTM_gtm_cls0.MCS0_MEM[1769].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1770 GTM_gtm_cls0.MCS0_MEM[1770].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1771 GTM_gtm_cls0.MCS0_MEM[1771].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1772 GTM_gtm_cls0.MCS0_MEM[1772].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1773 GTM_gtm_cls0.MCS0_MEM[1773].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1774 GTM_gtm_cls0.MCS0_MEM[1774].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1775 GTM_gtm_cls0.MCS0_MEM[1775].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1776 GTM_gtm_cls0.MCS0_MEM[1776].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1777 GTM_gtm_cls0.MCS0_MEM[1777].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1778 GTM_gtm_cls0.MCS0_MEM[1778].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1779 GTM_gtm_cls0.MCS0_MEM[1779].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1780 GTM_gtm_cls0.MCS0_MEM[1780].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1781 GTM_gtm_cls0.MCS0_MEM[1781].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1782 GTM_gtm_cls0.MCS0_MEM[1782].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1783 GTM_gtm_cls0.MCS0_MEM[1783].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1784 GTM_gtm_cls0.MCS0_MEM[1784].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1785 GTM_gtm_cls0.MCS0_MEM[1785].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1786 GTM_gtm_cls0.MCS0_MEM[1786].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1787 GTM_gtm_cls0.MCS0_MEM[1787].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1788 GTM_gtm_cls0.MCS0_MEM[1788].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1789 GTM_gtm_cls0.MCS0_MEM[1789].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1790 GTM_gtm_cls0.MCS0_MEM[1790].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1791 GTM_gtm_cls0.MCS0_MEM[1791].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1792 GTM_gtm_cls0.MCS0_MEM[1792].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1793 GTM_gtm_cls0.MCS0_MEM[1793].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1794 GTM_gtm_cls0.MCS0_MEM[1794].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1795 GTM_gtm_cls0.MCS0_MEM[1795].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1796 GTM_gtm_cls0.MCS0_MEM[1796].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1797 GTM_gtm_cls0.MCS0_MEM[1797].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1798 GTM_gtm_cls0.MCS0_MEM[1798].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1799 GTM_gtm_cls0.MCS0_MEM[1799].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1800 GTM_gtm_cls0.MCS0_MEM[1800].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1801 GTM_gtm_cls0.MCS0_MEM[1801].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1802 GTM_gtm_cls0.MCS0_MEM[1802].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1803 GTM_gtm_cls0.MCS0_MEM[1803].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1804 GTM_gtm_cls0.MCS0_MEM[1804].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1805 GTM_gtm_cls0.MCS0_MEM[1805].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1806 GTM_gtm_cls0.MCS0_MEM[1806].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1807 GTM_gtm_cls0.MCS0_MEM[1807].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1808 GTM_gtm_cls0.MCS0_MEM[1808].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1809 GTM_gtm_cls0.MCS0_MEM[1809].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1810 GTM_gtm_cls0.MCS0_MEM[1810].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1811 GTM_gtm_cls0.MCS0_MEM[1811].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1812 GTM_gtm_cls0.MCS0_MEM[1812].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1813 GTM_gtm_cls0.MCS0_MEM[1813].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1814 GTM_gtm_cls0.MCS0_MEM[1814].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1815 GTM_gtm_cls0.MCS0_MEM[1815].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1816 GTM_gtm_cls0.MCS0_MEM[1816].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1817 GTM_gtm_cls0.MCS0_MEM[1817].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1818 GTM_gtm_cls0.MCS0_MEM[1818].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1819 GTM_gtm_cls0.MCS0_MEM[1819].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1820 GTM_gtm_cls0.MCS0_MEM[1820].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1821 GTM_gtm_cls0.MCS0_MEM[1821].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1822 GTM_gtm_cls0.MCS0_MEM[1822].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1823 GTM_gtm_cls0.MCS0_MEM[1823].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1824 GTM_gtm_cls0.MCS0_MEM[1824].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1825 GTM_gtm_cls0.MCS0_MEM[1825].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1826 GTM_gtm_cls0.MCS0_MEM[1826].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1827 GTM_gtm_cls0.MCS0_MEM[1827].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1828 GTM_gtm_cls0.MCS0_MEM[1828].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1829 GTM_gtm_cls0.MCS0_MEM[1829].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1830 GTM_gtm_cls0.MCS0_MEM[1830].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1831 GTM_gtm_cls0.MCS0_MEM[1831].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1832 GTM_gtm_cls0.MCS0_MEM[1832].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1833 GTM_gtm_cls0.MCS0_MEM[1833].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1834 GTM_gtm_cls0.MCS0_MEM[1834].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1835 GTM_gtm_cls0.MCS0_MEM[1835].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1836 GTM_gtm_cls0.MCS0_MEM[1836].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1837 GTM_gtm_cls0.MCS0_MEM[1837].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1838 GTM_gtm_cls0.MCS0_MEM[1838].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1839 GTM_gtm_cls0.MCS0_MEM[1839].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1840 GTM_gtm_cls0.MCS0_MEM[1840].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1841 GTM_gtm_cls0.MCS0_MEM[1841].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1842 GTM_gtm_cls0.MCS0_MEM[1842].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1843 GTM_gtm_cls0.MCS0_MEM[1843].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1844 GTM_gtm_cls0.MCS0_MEM[1844].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1845 GTM_gtm_cls0.MCS0_MEM[1845].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1846 GTM_gtm_cls0.MCS0_MEM[1846].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1847 GTM_gtm_cls0.MCS0_MEM[1847].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1848 GTM_gtm_cls0.MCS0_MEM[1848].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1849 GTM_gtm_cls0.MCS0_MEM[1849].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1850 GTM_gtm_cls0.MCS0_MEM[1850].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1851 GTM_gtm_cls0.MCS0_MEM[1851].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1852 GTM_gtm_cls0.MCS0_MEM[1852].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1853 GTM_gtm_cls0.MCS0_MEM[1853].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1854 GTM_gtm_cls0.MCS0_MEM[1854].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1855 GTM_gtm_cls0.MCS0_MEM[1855].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1856 GTM_gtm_cls0.MCS0_MEM[1856].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1857 GTM_gtm_cls0.MCS0_MEM[1857].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1858 GTM_gtm_cls0.MCS0_MEM[1858].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1859 GTM_gtm_cls0.MCS0_MEM[1859].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1860 GTM_gtm_cls0.MCS0_MEM[1860].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1861 GTM_gtm_cls0.MCS0_MEM[1861].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1862 GTM_gtm_cls0.MCS0_MEM[1862].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1863 GTM_gtm_cls0.MCS0_MEM[1863].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1864 GTM_gtm_cls0.MCS0_MEM[1864].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1865 GTM_gtm_cls0.MCS0_MEM[1865].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1866 GTM_gtm_cls0.MCS0_MEM[1866].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1867 GTM_gtm_cls0.MCS0_MEM[1867].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1868 GTM_gtm_cls0.MCS0_MEM[1868].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1869 GTM_gtm_cls0.MCS0_MEM[1869].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1870 GTM_gtm_cls0.MCS0_MEM[1870].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1871 GTM_gtm_cls0.MCS0_MEM[1871].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1872 GTM_gtm_cls0.MCS0_MEM[1872].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1873 GTM_gtm_cls0.MCS0_MEM[1873].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1874 GTM_gtm_cls0.MCS0_MEM[1874].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1875 GTM_gtm_cls0.MCS0_MEM[1875].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1876 GTM_gtm_cls0.MCS0_MEM[1876].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1877 GTM_gtm_cls0.MCS0_MEM[1877].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1878 GTM_gtm_cls0.MCS0_MEM[1878].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1879 GTM_gtm_cls0.MCS0_MEM[1879].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1880 GTM_gtm_cls0.MCS0_MEM[1880].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1881 GTM_gtm_cls0.MCS0_MEM[1881].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1882 GTM_gtm_cls0.MCS0_MEM[1882].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1883 GTM_gtm_cls0.MCS0_MEM[1883].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1884 GTM_gtm_cls0.MCS0_MEM[1884].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1885 GTM_gtm_cls0.MCS0_MEM[1885].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1886 GTM_gtm_cls0.MCS0_MEM[1886].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1887 GTM_gtm_cls0.MCS0_MEM[1887].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1888 GTM_gtm_cls0.MCS0_MEM[1888].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1889 GTM_gtm_cls0.MCS0_MEM[1889].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1890 GTM_gtm_cls0.MCS0_MEM[1890].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1891 GTM_gtm_cls0.MCS0_MEM[1891].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1892 GTM_gtm_cls0.MCS0_MEM[1892].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1893 GTM_gtm_cls0.MCS0_MEM[1893].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1894 GTM_gtm_cls0.MCS0_MEM[1894].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1895 GTM_gtm_cls0.MCS0_MEM[1895].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1896 GTM_gtm_cls0.MCS0_MEM[1896].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1897 GTM_gtm_cls0.MCS0_MEM[1897].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1898 GTM_gtm_cls0.MCS0_MEM[1898].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1899 GTM_gtm_cls0.MCS0_MEM[1899].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1900 GTM_gtm_cls0.MCS0_MEM[1900].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1901 GTM_gtm_cls0.MCS0_MEM[1901].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1902 GTM_gtm_cls0.MCS0_MEM[1902].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1903 GTM_gtm_cls0.MCS0_MEM[1903].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1904 GTM_gtm_cls0.MCS0_MEM[1904].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1905 GTM_gtm_cls0.MCS0_MEM[1905].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1906 GTM_gtm_cls0.MCS0_MEM[1906].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1907 GTM_gtm_cls0.MCS0_MEM[1907].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1908 GTM_gtm_cls0.MCS0_MEM[1908].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1909 GTM_gtm_cls0.MCS0_MEM[1909].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1910 GTM_gtm_cls0.MCS0_MEM[1910].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1911 GTM_gtm_cls0.MCS0_MEM[1911].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1912 GTM_gtm_cls0.MCS0_MEM[1912].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1913 GTM_gtm_cls0.MCS0_MEM[1913].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1914 GTM_gtm_cls0.MCS0_MEM[1914].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1915 GTM_gtm_cls0.MCS0_MEM[1915].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1916 GTM_gtm_cls0.MCS0_MEM[1916].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1917 GTM_gtm_cls0.MCS0_MEM[1917].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1918 GTM_gtm_cls0.MCS0_MEM[1918].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1919 GTM_gtm_cls0.MCS0_MEM[1919].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1920 GTM_gtm_cls0.MCS0_MEM[1920].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1921 GTM_gtm_cls0.MCS0_MEM[1921].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1922 GTM_gtm_cls0.MCS0_MEM[1922].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1923 GTM_gtm_cls0.MCS0_MEM[1923].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1924 GTM_gtm_cls0.MCS0_MEM[1924].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1925 GTM_gtm_cls0.MCS0_MEM[1925].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1926 GTM_gtm_cls0.MCS0_MEM[1926].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1927 GTM_gtm_cls0.MCS0_MEM[1927].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1928 GTM_gtm_cls0.MCS0_MEM[1928].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1929 GTM_gtm_cls0.MCS0_MEM[1929].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1930 GTM_gtm_cls0.MCS0_MEM[1930].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1931 GTM_gtm_cls0.MCS0_MEM[1931].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1932 GTM_gtm_cls0.MCS0_MEM[1932].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1933 GTM_gtm_cls0.MCS0_MEM[1933].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1934 GTM_gtm_cls0.MCS0_MEM[1934].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1935 GTM_gtm_cls0.MCS0_MEM[1935].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1936 GTM_gtm_cls0.MCS0_MEM[1936].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1937 GTM_gtm_cls0.MCS0_MEM[1937].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1938 GTM_gtm_cls0.MCS0_MEM[1938].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1939 GTM_gtm_cls0.MCS0_MEM[1939].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1940 GTM_gtm_cls0.MCS0_MEM[1940].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1941 GTM_gtm_cls0.MCS0_MEM[1941].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1942 GTM_gtm_cls0.MCS0_MEM[1942].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1943 GTM_gtm_cls0.MCS0_MEM[1943].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1944 GTM_gtm_cls0.MCS0_MEM[1944].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1945 GTM_gtm_cls0.MCS0_MEM[1945].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1946 GTM_gtm_cls0.MCS0_MEM[1946].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1947 GTM_gtm_cls0.MCS0_MEM[1947].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1948 GTM_gtm_cls0.MCS0_MEM[1948].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1949 GTM_gtm_cls0.MCS0_MEM[1949].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1950 GTM_gtm_cls0.MCS0_MEM[1950].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1951 GTM_gtm_cls0.MCS0_MEM[1951].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1952 GTM_gtm_cls0.MCS0_MEM[1952].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1953 GTM_gtm_cls0.MCS0_MEM[1953].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1954 GTM_gtm_cls0.MCS0_MEM[1954].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1955 GTM_gtm_cls0.MCS0_MEM[1955].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1956 GTM_gtm_cls0.MCS0_MEM[1956].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1957 GTM_gtm_cls0.MCS0_MEM[1957].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1958 GTM_gtm_cls0.MCS0_MEM[1958].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1959 GTM_gtm_cls0.MCS0_MEM[1959].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1960 GTM_gtm_cls0.MCS0_MEM[1960].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1961 GTM_gtm_cls0.MCS0_MEM[1961].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1962 GTM_gtm_cls0.MCS0_MEM[1962].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1963 GTM_gtm_cls0.MCS0_MEM[1963].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1964 GTM_gtm_cls0.MCS0_MEM[1964].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1965 GTM_gtm_cls0.MCS0_MEM[1965].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1966 GTM_gtm_cls0.MCS0_MEM[1966].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1967 GTM_gtm_cls0.MCS0_MEM[1967].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1968 GTM_gtm_cls0.MCS0_MEM[1968].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1969 GTM_gtm_cls0.MCS0_MEM[1969].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1970 GTM_gtm_cls0.MCS0_MEM[1970].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1971 GTM_gtm_cls0.MCS0_MEM[1971].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1972 GTM_gtm_cls0.MCS0_MEM[1972].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1973 GTM_gtm_cls0.MCS0_MEM[1973].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1974 GTM_gtm_cls0.MCS0_MEM[1974].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1975 GTM_gtm_cls0.MCS0_MEM[1975].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1976 GTM_gtm_cls0.MCS0_MEM[1976].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1977 GTM_gtm_cls0.MCS0_MEM[1977].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1978 GTM_gtm_cls0.MCS0_MEM[1978].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1979 GTM_gtm_cls0.MCS0_MEM[1979].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1980 GTM_gtm_cls0.MCS0_MEM[1980].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1981 GTM_gtm_cls0.MCS0_MEM[1981].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1982 GTM_gtm_cls0.MCS0_MEM[1982].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1983 GTM_gtm_cls0.MCS0_MEM[1983].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1984 GTM_gtm_cls0.MCS0_MEM[1984].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1985 GTM_gtm_cls0.MCS0_MEM[1985].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1986 GTM_gtm_cls0.MCS0_MEM[1986].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1987 GTM_gtm_cls0.MCS0_MEM[1987].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1988 GTM_gtm_cls0.MCS0_MEM[1988].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1989 GTM_gtm_cls0.MCS0_MEM[1989].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1990 GTM_gtm_cls0.MCS0_MEM[1990].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1991 GTM_gtm_cls0.MCS0_MEM[1991].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1992 GTM_gtm_cls0.MCS0_MEM[1992].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1993 GTM_gtm_cls0.MCS0_MEM[1993].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1994 GTM_gtm_cls0.MCS0_MEM[1994].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1995 GTM_gtm_cls0.MCS0_MEM[1995].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1996 GTM_gtm_cls0.MCS0_MEM[1996].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1997 GTM_gtm_cls0.MCS0_MEM[1997].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1998 GTM_gtm_cls0.MCS0_MEM[1998].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM1999 GTM_gtm_cls0.MCS0_MEM[1999].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2000 GTM_gtm_cls0.MCS0_MEM[2000].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2001 GTM_gtm_cls0.MCS0_MEM[2001].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2002 GTM_gtm_cls0.MCS0_MEM[2002].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2003 GTM_gtm_cls0.MCS0_MEM[2003].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2004 GTM_gtm_cls0.MCS0_MEM[2004].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2005 GTM_gtm_cls0.MCS0_MEM[2005].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2006 GTM_gtm_cls0.MCS0_MEM[2006].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2007 GTM_gtm_cls0.MCS0_MEM[2007].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2008 GTM_gtm_cls0.MCS0_MEM[2008].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2009 GTM_gtm_cls0.MCS0_MEM[2009].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2010 GTM_gtm_cls0.MCS0_MEM[2010].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2011 GTM_gtm_cls0.MCS0_MEM[2011].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2012 GTM_gtm_cls0.MCS0_MEM[2012].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2013 GTM_gtm_cls0.MCS0_MEM[2013].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2014 GTM_gtm_cls0.MCS0_MEM[2014].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2015 GTM_gtm_cls0.MCS0_MEM[2015].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2016 GTM_gtm_cls0.MCS0_MEM[2016].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2017 GTM_gtm_cls0.MCS0_MEM[2017].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2018 GTM_gtm_cls0.MCS0_MEM[2018].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2019 GTM_gtm_cls0.MCS0_MEM[2019].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2020 GTM_gtm_cls0.MCS0_MEM[2020].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2021 GTM_gtm_cls0.MCS0_MEM[2021].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2022 GTM_gtm_cls0.MCS0_MEM[2022].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2023 GTM_gtm_cls0.MCS0_MEM[2023].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2024 GTM_gtm_cls0.MCS0_MEM[2024].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2025 GTM_gtm_cls0.MCS0_MEM[2025].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2026 GTM_gtm_cls0.MCS0_MEM[2026].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2027 GTM_gtm_cls0.MCS0_MEM[2027].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2028 GTM_gtm_cls0.MCS0_MEM[2028].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2029 GTM_gtm_cls0.MCS0_MEM[2029].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2030 GTM_gtm_cls0.MCS0_MEM[2030].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2031 GTM_gtm_cls0.MCS0_MEM[2031].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2032 GTM_gtm_cls0.MCS0_MEM[2032].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2033 GTM_gtm_cls0.MCS0_MEM[2033].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2034 GTM_gtm_cls0.MCS0_MEM[2034].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2035 GTM_gtm_cls0.MCS0_MEM[2035].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2036 GTM_gtm_cls0.MCS0_MEM[2036].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2037 GTM_gtm_cls0.MCS0_MEM[2037].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2038 GTM_gtm_cls0.MCS0_MEM[2038].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2039 GTM_gtm_cls0.MCS0_MEM[2039].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2040 GTM_gtm_cls0.MCS0_MEM[2040].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2041 GTM_gtm_cls0.MCS0_MEM[2041].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2042 GTM_gtm_cls0.MCS0_MEM[2042].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2043 GTM_gtm_cls0.MCS0_MEM[2043].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2044 GTM_gtm_cls0.MCS0_MEM[2044].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2045 GTM_gtm_cls0.MCS0_MEM[2045].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2046 GTM_gtm_cls0.MCS0_MEM[2046].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2047 GTM_gtm_cls0.MCS0_MEM[2047].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2048 GTM_gtm_cls0.MCS0_MEM[2048].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2049 GTM_gtm_cls0.MCS0_MEM[2049].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2050 GTM_gtm_cls0.MCS0_MEM[2050].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2051 GTM_gtm_cls0.MCS0_MEM[2051].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2052 GTM_gtm_cls0.MCS0_MEM[2052].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2053 GTM_gtm_cls0.MCS0_MEM[2053].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2054 GTM_gtm_cls0.MCS0_MEM[2054].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2055 GTM_gtm_cls0.MCS0_MEM[2055].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2056 GTM_gtm_cls0.MCS0_MEM[2056].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2057 GTM_gtm_cls0.MCS0_MEM[2057].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2058 GTM_gtm_cls0.MCS0_MEM[2058].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2059 GTM_gtm_cls0.MCS0_MEM[2059].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2060 GTM_gtm_cls0.MCS0_MEM[2060].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2061 GTM_gtm_cls0.MCS0_MEM[2061].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2062 GTM_gtm_cls0.MCS0_MEM[2062].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2063 GTM_gtm_cls0.MCS0_MEM[2063].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2064 GTM_gtm_cls0.MCS0_MEM[2064].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2065 GTM_gtm_cls0.MCS0_MEM[2065].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2066 GTM_gtm_cls0.MCS0_MEM[2066].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2067 GTM_gtm_cls0.MCS0_MEM[2067].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2068 GTM_gtm_cls0.MCS0_MEM[2068].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2069 GTM_gtm_cls0.MCS0_MEM[2069].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2070 GTM_gtm_cls0.MCS0_MEM[2070].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2071 GTM_gtm_cls0.MCS0_MEM[2071].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2072 GTM_gtm_cls0.MCS0_MEM[2072].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2073 GTM_gtm_cls0.MCS0_MEM[2073].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2074 GTM_gtm_cls0.MCS0_MEM[2074].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2075 GTM_gtm_cls0.MCS0_MEM[2075].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2076 GTM_gtm_cls0.MCS0_MEM[2076].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2077 GTM_gtm_cls0.MCS0_MEM[2077].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2078 GTM_gtm_cls0.MCS0_MEM[2078].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2079 GTM_gtm_cls0.MCS0_MEM[2079].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2080 GTM_gtm_cls0.MCS0_MEM[2080].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2081 GTM_gtm_cls0.MCS0_MEM[2081].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2082 GTM_gtm_cls0.MCS0_MEM[2082].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2083 GTM_gtm_cls0.MCS0_MEM[2083].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2084 GTM_gtm_cls0.MCS0_MEM[2084].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2085 GTM_gtm_cls0.MCS0_MEM[2085].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2086 GTM_gtm_cls0.MCS0_MEM[2086].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2087 GTM_gtm_cls0.MCS0_MEM[2087].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2088 GTM_gtm_cls0.MCS0_MEM[2088].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2089 GTM_gtm_cls0.MCS0_MEM[2089].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2090 GTM_gtm_cls0.MCS0_MEM[2090].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2091 GTM_gtm_cls0.MCS0_MEM[2091].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2092 GTM_gtm_cls0.MCS0_MEM[2092].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2093 GTM_gtm_cls0.MCS0_MEM[2093].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2094 GTM_gtm_cls0.MCS0_MEM[2094].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2095 GTM_gtm_cls0.MCS0_MEM[2095].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2096 GTM_gtm_cls0.MCS0_MEM[2096].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2097 GTM_gtm_cls0.MCS0_MEM[2097].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2098 GTM_gtm_cls0.MCS0_MEM[2098].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2099 GTM_gtm_cls0.MCS0_MEM[2099].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2100 GTM_gtm_cls0.MCS0_MEM[2100].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2101 GTM_gtm_cls0.MCS0_MEM[2101].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2102 GTM_gtm_cls0.MCS0_MEM[2102].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2103 GTM_gtm_cls0.MCS0_MEM[2103].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2104 GTM_gtm_cls0.MCS0_MEM[2104].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2105 GTM_gtm_cls0.MCS0_MEM[2105].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2106 GTM_gtm_cls0.MCS0_MEM[2106].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2107 GTM_gtm_cls0.MCS0_MEM[2107].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2108 GTM_gtm_cls0.MCS0_MEM[2108].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2109 GTM_gtm_cls0.MCS0_MEM[2109].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2110 GTM_gtm_cls0.MCS0_MEM[2110].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2111 GTM_gtm_cls0.MCS0_MEM[2111].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2112 GTM_gtm_cls0.MCS0_MEM[2112].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2113 GTM_gtm_cls0.MCS0_MEM[2113].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2114 GTM_gtm_cls0.MCS0_MEM[2114].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2115 GTM_gtm_cls0.MCS0_MEM[2115].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2116 GTM_gtm_cls0.MCS0_MEM[2116].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2117 GTM_gtm_cls0.MCS0_MEM[2117].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2118 GTM_gtm_cls0.MCS0_MEM[2118].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2119 GTM_gtm_cls0.MCS0_MEM[2119].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2120 GTM_gtm_cls0.MCS0_MEM[2120].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2121 GTM_gtm_cls0.MCS0_MEM[2121].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2122 GTM_gtm_cls0.MCS0_MEM[2122].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2123 GTM_gtm_cls0.MCS0_MEM[2123].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2124 GTM_gtm_cls0.MCS0_MEM[2124].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2125 GTM_gtm_cls0.MCS0_MEM[2125].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2126 GTM_gtm_cls0.MCS0_MEM[2126].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2127 GTM_gtm_cls0.MCS0_MEM[2127].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2128 GTM_gtm_cls0.MCS0_MEM[2128].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2129 GTM_gtm_cls0.MCS0_MEM[2129].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2130 GTM_gtm_cls0.MCS0_MEM[2130].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2131 GTM_gtm_cls0.MCS0_MEM[2131].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2132 GTM_gtm_cls0.MCS0_MEM[2132].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2133 GTM_gtm_cls0.MCS0_MEM[2133].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2134 GTM_gtm_cls0.MCS0_MEM[2134].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2135 GTM_gtm_cls0.MCS0_MEM[2135].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2136 GTM_gtm_cls0.MCS0_MEM[2136].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2137 GTM_gtm_cls0.MCS0_MEM[2137].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2138 GTM_gtm_cls0.MCS0_MEM[2138].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2139 GTM_gtm_cls0.MCS0_MEM[2139].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2140 GTM_gtm_cls0.MCS0_MEM[2140].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2141 GTM_gtm_cls0.MCS0_MEM[2141].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2142 GTM_gtm_cls0.MCS0_MEM[2142].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2143 GTM_gtm_cls0.MCS0_MEM[2143].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2144 GTM_gtm_cls0.MCS0_MEM[2144].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2145 GTM_gtm_cls0.MCS0_MEM[2145].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2146 GTM_gtm_cls0.MCS0_MEM[2146].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2147 GTM_gtm_cls0.MCS0_MEM[2147].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2148 GTM_gtm_cls0.MCS0_MEM[2148].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2149 GTM_gtm_cls0.MCS0_MEM[2149].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2150 GTM_gtm_cls0.MCS0_MEM[2150].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2151 GTM_gtm_cls0.MCS0_MEM[2151].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2152 GTM_gtm_cls0.MCS0_MEM[2152].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2153 GTM_gtm_cls0.MCS0_MEM[2153].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2154 GTM_gtm_cls0.MCS0_MEM[2154].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2155 GTM_gtm_cls0.MCS0_MEM[2155].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2156 GTM_gtm_cls0.MCS0_MEM[2156].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2157 GTM_gtm_cls0.MCS0_MEM[2157].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2158 GTM_gtm_cls0.MCS0_MEM[2158].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2159 GTM_gtm_cls0.MCS0_MEM[2159].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2160 GTM_gtm_cls0.MCS0_MEM[2160].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2161 GTM_gtm_cls0.MCS0_MEM[2161].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2162 GTM_gtm_cls0.MCS0_MEM[2162].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2163 GTM_gtm_cls0.MCS0_MEM[2163].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2164 GTM_gtm_cls0.MCS0_MEM[2164].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2165 GTM_gtm_cls0.MCS0_MEM[2165].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2166 GTM_gtm_cls0.MCS0_MEM[2166].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2167 GTM_gtm_cls0.MCS0_MEM[2167].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2168 GTM_gtm_cls0.MCS0_MEM[2168].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2169 GTM_gtm_cls0.MCS0_MEM[2169].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2170 GTM_gtm_cls0.MCS0_MEM[2170].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2171 GTM_gtm_cls0.MCS0_MEM[2171].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2172 GTM_gtm_cls0.MCS0_MEM[2172].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2173 GTM_gtm_cls0.MCS0_MEM[2173].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2174 GTM_gtm_cls0.MCS0_MEM[2174].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2175 GTM_gtm_cls0.MCS0_MEM[2175].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2176 GTM_gtm_cls0.MCS0_MEM[2176].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2177 GTM_gtm_cls0.MCS0_MEM[2177].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2178 GTM_gtm_cls0.MCS0_MEM[2178].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2179 GTM_gtm_cls0.MCS0_MEM[2179].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2180 GTM_gtm_cls0.MCS0_MEM[2180].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2181 GTM_gtm_cls0.MCS0_MEM[2181].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2182 GTM_gtm_cls0.MCS0_MEM[2182].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2183 GTM_gtm_cls0.MCS0_MEM[2183].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2184 GTM_gtm_cls0.MCS0_MEM[2184].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2185 GTM_gtm_cls0.MCS0_MEM[2185].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2186 GTM_gtm_cls0.MCS0_MEM[2186].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2187 GTM_gtm_cls0.MCS0_MEM[2187].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2188 GTM_gtm_cls0.MCS0_MEM[2188].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2189 GTM_gtm_cls0.MCS0_MEM[2189].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2190 GTM_gtm_cls0.MCS0_MEM[2190].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2191 GTM_gtm_cls0.MCS0_MEM[2191].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2192 GTM_gtm_cls0.MCS0_MEM[2192].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2193 GTM_gtm_cls0.MCS0_MEM[2193].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2194 GTM_gtm_cls0.MCS0_MEM[2194].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2195 GTM_gtm_cls0.MCS0_MEM[2195].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2196 GTM_gtm_cls0.MCS0_MEM[2196].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2197 GTM_gtm_cls0.MCS0_MEM[2197].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2198 GTM_gtm_cls0.MCS0_MEM[2198].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2199 GTM_gtm_cls0.MCS0_MEM[2199].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2200 GTM_gtm_cls0.MCS0_MEM[2200].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2201 GTM_gtm_cls0.MCS0_MEM[2201].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2202 GTM_gtm_cls0.MCS0_MEM[2202].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2203 GTM_gtm_cls0.MCS0_MEM[2203].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2204 GTM_gtm_cls0.MCS0_MEM[2204].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2205 GTM_gtm_cls0.MCS0_MEM[2205].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2206 GTM_gtm_cls0.MCS0_MEM[2206].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2207 GTM_gtm_cls0.MCS0_MEM[2207].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2208 GTM_gtm_cls0.MCS0_MEM[2208].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2209 GTM_gtm_cls0.MCS0_MEM[2209].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2210 GTM_gtm_cls0.MCS0_MEM[2210].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2211 GTM_gtm_cls0.MCS0_MEM[2211].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2212 GTM_gtm_cls0.MCS0_MEM[2212].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2213 GTM_gtm_cls0.MCS0_MEM[2213].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2214 GTM_gtm_cls0.MCS0_MEM[2214].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2215 GTM_gtm_cls0.MCS0_MEM[2215].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2216 GTM_gtm_cls0.MCS0_MEM[2216].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2217 GTM_gtm_cls0.MCS0_MEM[2217].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2218 GTM_gtm_cls0.MCS0_MEM[2218].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2219 GTM_gtm_cls0.MCS0_MEM[2219].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2220 GTM_gtm_cls0.MCS0_MEM[2220].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2221 GTM_gtm_cls0.MCS0_MEM[2221].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2222 GTM_gtm_cls0.MCS0_MEM[2222].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2223 GTM_gtm_cls0.MCS0_MEM[2223].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2224 GTM_gtm_cls0.MCS0_MEM[2224].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2225 GTM_gtm_cls0.MCS0_MEM[2225].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2226 GTM_gtm_cls0.MCS0_MEM[2226].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2227 GTM_gtm_cls0.MCS0_MEM[2227].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2228 GTM_gtm_cls0.MCS0_MEM[2228].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2229 GTM_gtm_cls0.MCS0_MEM[2229].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2230 GTM_gtm_cls0.MCS0_MEM[2230].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2231 GTM_gtm_cls0.MCS0_MEM[2231].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2232 GTM_gtm_cls0.MCS0_MEM[2232].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2233 GTM_gtm_cls0.MCS0_MEM[2233].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2234 GTM_gtm_cls0.MCS0_MEM[2234].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2235 GTM_gtm_cls0.MCS0_MEM[2235].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2236 GTM_gtm_cls0.MCS0_MEM[2236].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2237 GTM_gtm_cls0.MCS0_MEM[2237].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2238 GTM_gtm_cls0.MCS0_MEM[2238].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2239 GTM_gtm_cls0.MCS0_MEM[2239].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2240 GTM_gtm_cls0.MCS0_MEM[2240].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2241 GTM_gtm_cls0.MCS0_MEM[2241].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2242 GTM_gtm_cls0.MCS0_MEM[2242].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2243 GTM_gtm_cls0.MCS0_MEM[2243].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2244 GTM_gtm_cls0.MCS0_MEM[2244].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2245 GTM_gtm_cls0.MCS0_MEM[2245].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2246 GTM_gtm_cls0.MCS0_MEM[2246].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2247 GTM_gtm_cls0.MCS0_MEM[2247].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2248 GTM_gtm_cls0.MCS0_MEM[2248].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2249 GTM_gtm_cls0.MCS0_MEM[2249].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2250 GTM_gtm_cls0.MCS0_MEM[2250].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2251 GTM_gtm_cls0.MCS0_MEM[2251].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2252 GTM_gtm_cls0.MCS0_MEM[2252].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2253 GTM_gtm_cls0.MCS0_MEM[2253].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2254 GTM_gtm_cls0.MCS0_MEM[2254].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2255 GTM_gtm_cls0.MCS0_MEM[2255].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2256 GTM_gtm_cls0.MCS0_MEM[2256].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2257 GTM_gtm_cls0.MCS0_MEM[2257].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2258 GTM_gtm_cls0.MCS0_MEM[2258].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2259 GTM_gtm_cls0.MCS0_MEM[2259].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2260 GTM_gtm_cls0.MCS0_MEM[2260].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2261 GTM_gtm_cls0.MCS0_MEM[2261].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2262 GTM_gtm_cls0.MCS0_MEM[2262].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2263 GTM_gtm_cls0.MCS0_MEM[2263].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2264 GTM_gtm_cls0.MCS0_MEM[2264].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2265 GTM_gtm_cls0.MCS0_MEM[2265].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2266 GTM_gtm_cls0.MCS0_MEM[2266].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2267 GTM_gtm_cls0.MCS0_MEM[2267].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2268 GTM_gtm_cls0.MCS0_MEM[2268].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2269 GTM_gtm_cls0.MCS0_MEM[2269].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2270 GTM_gtm_cls0.MCS0_MEM[2270].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2271 GTM_gtm_cls0.MCS0_MEM[2271].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2272 GTM_gtm_cls0.MCS0_MEM[2272].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2273 GTM_gtm_cls0.MCS0_MEM[2273].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2274 GTM_gtm_cls0.MCS0_MEM[2274].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2275 GTM_gtm_cls0.MCS0_MEM[2275].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2276 GTM_gtm_cls0.MCS0_MEM[2276].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2277 GTM_gtm_cls0.MCS0_MEM[2277].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2278 GTM_gtm_cls0.MCS0_MEM[2278].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2279 GTM_gtm_cls0.MCS0_MEM[2279].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2280 GTM_gtm_cls0.MCS0_MEM[2280].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2281 GTM_gtm_cls0.MCS0_MEM[2281].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2282 GTM_gtm_cls0.MCS0_MEM[2282].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2283 GTM_gtm_cls0.MCS0_MEM[2283].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2284 GTM_gtm_cls0.MCS0_MEM[2284].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2285 GTM_gtm_cls0.MCS0_MEM[2285].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2286 GTM_gtm_cls0.MCS0_MEM[2286].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2287 GTM_gtm_cls0.MCS0_MEM[2287].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2288 GTM_gtm_cls0.MCS0_MEM[2288].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2289 GTM_gtm_cls0.MCS0_MEM[2289].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2290 GTM_gtm_cls0.MCS0_MEM[2290].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2291 GTM_gtm_cls0.MCS0_MEM[2291].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2292 GTM_gtm_cls0.MCS0_MEM[2292].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2293 GTM_gtm_cls0.MCS0_MEM[2293].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2294 GTM_gtm_cls0.MCS0_MEM[2294].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2295 GTM_gtm_cls0.MCS0_MEM[2295].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2296 GTM_gtm_cls0.MCS0_MEM[2296].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2297 GTM_gtm_cls0.MCS0_MEM[2297].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2298 GTM_gtm_cls0.MCS0_MEM[2298].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2299 GTM_gtm_cls0.MCS0_MEM[2299].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2300 GTM_gtm_cls0.MCS0_MEM[2300].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2301 GTM_gtm_cls0.MCS0_MEM[2301].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2302 GTM_gtm_cls0.MCS0_MEM[2302].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2303 GTM_gtm_cls0.MCS0_MEM[2303].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2304 GTM_gtm_cls0.MCS0_MEM[2304].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2305 GTM_gtm_cls0.MCS0_MEM[2305].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2306 GTM_gtm_cls0.MCS0_MEM[2306].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2307 GTM_gtm_cls0.MCS0_MEM[2307].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2308 GTM_gtm_cls0.MCS0_MEM[2308].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2309 GTM_gtm_cls0.MCS0_MEM[2309].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2310 GTM_gtm_cls0.MCS0_MEM[2310].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2311 GTM_gtm_cls0.MCS0_MEM[2311].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2312 GTM_gtm_cls0.MCS0_MEM[2312].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2313 GTM_gtm_cls0.MCS0_MEM[2313].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2314 GTM_gtm_cls0.MCS0_MEM[2314].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2315 GTM_gtm_cls0.MCS0_MEM[2315].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2316 GTM_gtm_cls0.MCS0_MEM[2316].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2317 GTM_gtm_cls0.MCS0_MEM[2317].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2318 GTM_gtm_cls0.MCS0_MEM[2318].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2319 GTM_gtm_cls0.MCS0_MEM[2319].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2320 GTM_gtm_cls0.MCS0_MEM[2320].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2321 GTM_gtm_cls0.MCS0_MEM[2321].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2322 GTM_gtm_cls0.MCS0_MEM[2322].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2323 GTM_gtm_cls0.MCS0_MEM[2323].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2324 GTM_gtm_cls0.MCS0_MEM[2324].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2325 GTM_gtm_cls0.MCS0_MEM[2325].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2326 GTM_gtm_cls0.MCS0_MEM[2326].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2327 GTM_gtm_cls0.MCS0_MEM[2327].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2328 GTM_gtm_cls0.MCS0_MEM[2328].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2329 GTM_gtm_cls0.MCS0_MEM[2329].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2330 GTM_gtm_cls0.MCS0_MEM[2330].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2331 GTM_gtm_cls0.MCS0_MEM[2331].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2332 GTM_gtm_cls0.MCS0_MEM[2332].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2333 GTM_gtm_cls0.MCS0_MEM[2333].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2334 GTM_gtm_cls0.MCS0_MEM[2334].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2335 GTM_gtm_cls0.MCS0_MEM[2335].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2336 GTM_gtm_cls0.MCS0_MEM[2336].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2337 GTM_gtm_cls0.MCS0_MEM[2337].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2338 GTM_gtm_cls0.MCS0_MEM[2338].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2339 GTM_gtm_cls0.MCS0_MEM[2339].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2340 GTM_gtm_cls0.MCS0_MEM[2340].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2341 GTM_gtm_cls0.MCS0_MEM[2341].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2342 GTM_gtm_cls0.MCS0_MEM[2342].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2343 GTM_gtm_cls0.MCS0_MEM[2343].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2344 GTM_gtm_cls0.MCS0_MEM[2344].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2345 GTM_gtm_cls0.MCS0_MEM[2345].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2346 GTM_gtm_cls0.MCS0_MEM[2346].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2347 GTM_gtm_cls0.MCS0_MEM[2347].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2348 GTM_gtm_cls0.MCS0_MEM[2348].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2349 GTM_gtm_cls0.MCS0_MEM[2349].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2350 GTM_gtm_cls0.MCS0_MEM[2350].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2351 GTM_gtm_cls0.MCS0_MEM[2351].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2352 GTM_gtm_cls0.MCS0_MEM[2352].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2353 GTM_gtm_cls0.MCS0_MEM[2353].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2354 GTM_gtm_cls0.MCS0_MEM[2354].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2355 GTM_gtm_cls0.MCS0_MEM[2355].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2356 GTM_gtm_cls0.MCS0_MEM[2356].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2357 GTM_gtm_cls0.MCS0_MEM[2357].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2358 GTM_gtm_cls0.MCS0_MEM[2358].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2359 GTM_gtm_cls0.MCS0_MEM[2359].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2360 GTM_gtm_cls0.MCS0_MEM[2360].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2361 GTM_gtm_cls0.MCS0_MEM[2361].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2362 GTM_gtm_cls0.MCS0_MEM[2362].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2363 GTM_gtm_cls0.MCS0_MEM[2363].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2364 GTM_gtm_cls0.MCS0_MEM[2364].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2365 GTM_gtm_cls0.MCS0_MEM[2365].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2366 GTM_gtm_cls0.MCS0_MEM[2366].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2367 GTM_gtm_cls0.MCS0_MEM[2367].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2368 GTM_gtm_cls0.MCS0_MEM[2368].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2369 GTM_gtm_cls0.MCS0_MEM[2369].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2370 GTM_gtm_cls0.MCS0_MEM[2370].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2371 GTM_gtm_cls0.MCS0_MEM[2371].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2372 GTM_gtm_cls0.MCS0_MEM[2372].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2373 GTM_gtm_cls0.MCS0_MEM[2373].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2374 GTM_gtm_cls0.MCS0_MEM[2374].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2375 GTM_gtm_cls0.MCS0_MEM[2375].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2376 GTM_gtm_cls0.MCS0_MEM[2376].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2377 GTM_gtm_cls0.MCS0_MEM[2377].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2378 GTM_gtm_cls0.MCS0_MEM[2378].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2379 GTM_gtm_cls0.MCS0_MEM[2379].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2380 GTM_gtm_cls0.MCS0_MEM[2380].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2381 GTM_gtm_cls0.MCS0_MEM[2381].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2382 GTM_gtm_cls0.MCS0_MEM[2382].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2383 GTM_gtm_cls0.MCS0_MEM[2383].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2384 GTM_gtm_cls0.MCS0_MEM[2384].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2385 GTM_gtm_cls0.MCS0_MEM[2385].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2386 GTM_gtm_cls0.MCS0_MEM[2386].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2387 GTM_gtm_cls0.MCS0_MEM[2387].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2388 GTM_gtm_cls0.MCS0_MEM[2388].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2389 GTM_gtm_cls0.MCS0_MEM[2389].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2390 GTM_gtm_cls0.MCS0_MEM[2390].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2391 GTM_gtm_cls0.MCS0_MEM[2391].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2392 GTM_gtm_cls0.MCS0_MEM[2392].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2393 GTM_gtm_cls0.MCS0_MEM[2393].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2394 GTM_gtm_cls0.MCS0_MEM[2394].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2395 GTM_gtm_cls0.MCS0_MEM[2395].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2396 GTM_gtm_cls0.MCS0_MEM[2396].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2397 GTM_gtm_cls0.MCS0_MEM[2397].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2398 GTM_gtm_cls0.MCS0_MEM[2398].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2399 GTM_gtm_cls0.MCS0_MEM[2399].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2400 GTM_gtm_cls0.MCS0_MEM[2400].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2401 GTM_gtm_cls0.MCS0_MEM[2401].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2402 GTM_gtm_cls0.MCS0_MEM[2402].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2403 GTM_gtm_cls0.MCS0_MEM[2403].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2404 GTM_gtm_cls0.MCS0_MEM[2404].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2405 GTM_gtm_cls0.MCS0_MEM[2405].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2406 GTM_gtm_cls0.MCS0_MEM[2406].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2407 GTM_gtm_cls0.MCS0_MEM[2407].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2408 GTM_gtm_cls0.MCS0_MEM[2408].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2409 GTM_gtm_cls0.MCS0_MEM[2409].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2410 GTM_gtm_cls0.MCS0_MEM[2410].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2411 GTM_gtm_cls0.MCS0_MEM[2411].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2412 GTM_gtm_cls0.MCS0_MEM[2412].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2413 GTM_gtm_cls0.MCS0_MEM[2413].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2414 GTM_gtm_cls0.MCS0_MEM[2414].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2415 GTM_gtm_cls0.MCS0_MEM[2415].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2416 GTM_gtm_cls0.MCS0_MEM[2416].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2417 GTM_gtm_cls0.MCS0_MEM[2417].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2418 GTM_gtm_cls0.MCS0_MEM[2418].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2419 GTM_gtm_cls0.MCS0_MEM[2419].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2420 GTM_gtm_cls0.MCS0_MEM[2420].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2421 GTM_gtm_cls0.MCS0_MEM[2421].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2422 GTM_gtm_cls0.MCS0_MEM[2422].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2423 GTM_gtm_cls0.MCS0_MEM[2423].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2424 GTM_gtm_cls0.MCS0_MEM[2424].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2425 GTM_gtm_cls0.MCS0_MEM[2425].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2426 GTM_gtm_cls0.MCS0_MEM[2426].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2427 GTM_gtm_cls0.MCS0_MEM[2427].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2428 GTM_gtm_cls0.MCS0_MEM[2428].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2429 GTM_gtm_cls0.MCS0_MEM[2429].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2430 GTM_gtm_cls0.MCS0_MEM[2430].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2431 GTM_gtm_cls0.MCS0_MEM[2431].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2432 GTM_gtm_cls0.MCS0_MEM[2432].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2433 GTM_gtm_cls0.MCS0_MEM[2433].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2434 GTM_gtm_cls0.MCS0_MEM[2434].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2435 GTM_gtm_cls0.MCS0_MEM[2435].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2436 GTM_gtm_cls0.MCS0_MEM[2436].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2437 GTM_gtm_cls0.MCS0_MEM[2437].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2438 GTM_gtm_cls0.MCS0_MEM[2438].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2439 GTM_gtm_cls0.MCS0_MEM[2439].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2440 GTM_gtm_cls0.MCS0_MEM[2440].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2441 GTM_gtm_cls0.MCS0_MEM[2441].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2442 GTM_gtm_cls0.MCS0_MEM[2442].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2443 GTM_gtm_cls0.MCS0_MEM[2443].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2444 GTM_gtm_cls0.MCS0_MEM[2444].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2445 GTM_gtm_cls0.MCS0_MEM[2445].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2446 GTM_gtm_cls0.MCS0_MEM[2446].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2447 GTM_gtm_cls0.MCS0_MEM[2447].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2448 GTM_gtm_cls0.MCS0_MEM[2448].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2449 GTM_gtm_cls0.MCS0_MEM[2449].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2450 GTM_gtm_cls0.MCS0_MEM[2450].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2451 GTM_gtm_cls0.MCS0_MEM[2451].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2452 GTM_gtm_cls0.MCS0_MEM[2452].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2453 GTM_gtm_cls0.MCS0_MEM[2453].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2454 GTM_gtm_cls0.MCS0_MEM[2454].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2455 GTM_gtm_cls0.MCS0_MEM[2455].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2456 GTM_gtm_cls0.MCS0_MEM[2456].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2457 GTM_gtm_cls0.MCS0_MEM[2457].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2458 GTM_gtm_cls0.MCS0_MEM[2458].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2459 GTM_gtm_cls0.MCS0_MEM[2459].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2460 GTM_gtm_cls0.MCS0_MEM[2460].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2461 GTM_gtm_cls0.MCS0_MEM[2461].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2462 GTM_gtm_cls0.MCS0_MEM[2462].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2463 GTM_gtm_cls0.MCS0_MEM[2463].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2464 GTM_gtm_cls0.MCS0_MEM[2464].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2465 GTM_gtm_cls0.MCS0_MEM[2465].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2466 GTM_gtm_cls0.MCS0_MEM[2466].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2467 GTM_gtm_cls0.MCS0_MEM[2467].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2468 GTM_gtm_cls0.MCS0_MEM[2468].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2469 GTM_gtm_cls0.MCS0_MEM[2469].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2470 GTM_gtm_cls0.MCS0_MEM[2470].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2471 GTM_gtm_cls0.MCS0_MEM[2471].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2472 GTM_gtm_cls0.MCS0_MEM[2472].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2473 GTM_gtm_cls0.MCS0_MEM[2473].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2474 GTM_gtm_cls0.MCS0_MEM[2474].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2475 GTM_gtm_cls0.MCS0_MEM[2475].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2476 GTM_gtm_cls0.MCS0_MEM[2476].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2477 GTM_gtm_cls0.MCS0_MEM[2477].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2478 GTM_gtm_cls0.MCS0_MEM[2478].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2479 GTM_gtm_cls0.MCS0_MEM[2479].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2480 GTM_gtm_cls0.MCS0_MEM[2480].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2481 GTM_gtm_cls0.MCS0_MEM[2481].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2482 GTM_gtm_cls0.MCS0_MEM[2482].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2483 GTM_gtm_cls0.MCS0_MEM[2483].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2484 GTM_gtm_cls0.MCS0_MEM[2484].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2485 GTM_gtm_cls0.MCS0_MEM[2485].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2486 GTM_gtm_cls0.MCS0_MEM[2486].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2487 GTM_gtm_cls0.MCS0_MEM[2487].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2488 GTM_gtm_cls0.MCS0_MEM[2488].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2489 GTM_gtm_cls0.MCS0_MEM[2489].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2490 GTM_gtm_cls0.MCS0_MEM[2490].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2491 GTM_gtm_cls0.MCS0_MEM[2491].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2492 GTM_gtm_cls0.MCS0_MEM[2492].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2493 GTM_gtm_cls0.MCS0_MEM[2493].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2494 GTM_gtm_cls0.MCS0_MEM[2494].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2495 GTM_gtm_cls0.MCS0_MEM[2495].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2496 GTM_gtm_cls0.MCS0_MEM[2496].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2497 GTM_gtm_cls0.MCS0_MEM[2497].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2498 GTM_gtm_cls0.MCS0_MEM[2498].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2499 GTM_gtm_cls0.MCS0_MEM[2499].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2500 GTM_gtm_cls0.MCS0_MEM[2500].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2501 GTM_gtm_cls0.MCS0_MEM[2501].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2502 GTM_gtm_cls0.MCS0_MEM[2502].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2503 GTM_gtm_cls0.MCS0_MEM[2503].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2504 GTM_gtm_cls0.MCS0_MEM[2504].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2505 GTM_gtm_cls0.MCS0_MEM[2505].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2506 GTM_gtm_cls0.MCS0_MEM[2506].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2507 GTM_gtm_cls0.MCS0_MEM[2507].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2508 GTM_gtm_cls0.MCS0_MEM[2508].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2509 GTM_gtm_cls0.MCS0_MEM[2509].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2510 GTM_gtm_cls0.MCS0_MEM[2510].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2511 GTM_gtm_cls0.MCS0_MEM[2511].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2512 GTM_gtm_cls0.MCS0_MEM[2512].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2513 GTM_gtm_cls0.MCS0_MEM[2513].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2514 GTM_gtm_cls0.MCS0_MEM[2514].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2515 GTM_gtm_cls0.MCS0_MEM[2515].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2516 GTM_gtm_cls0.MCS0_MEM[2516].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2517 GTM_gtm_cls0.MCS0_MEM[2517].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2518 GTM_gtm_cls0.MCS0_MEM[2518].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2519 GTM_gtm_cls0.MCS0_MEM[2519].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2520 GTM_gtm_cls0.MCS0_MEM[2520].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2521 GTM_gtm_cls0.MCS0_MEM[2521].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2522 GTM_gtm_cls0.MCS0_MEM[2522].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2523 GTM_gtm_cls0.MCS0_MEM[2523].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2524 GTM_gtm_cls0.MCS0_MEM[2524].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2525 GTM_gtm_cls0.MCS0_MEM[2525].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2526 GTM_gtm_cls0.MCS0_MEM[2526].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2527 GTM_gtm_cls0.MCS0_MEM[2527].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2528 GTM_gtm_cls0.MCS0_MEM[2528].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2529 GTM_gtm_cls0.MCS0_MEM[2529].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2530 GTM_gtm_cls0.MCS0_MEM[2530].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2531 GTM_gtm_cls0.MCS0_MEM[2531].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2532 GTM_gtm_cls0.MCS0_MEM[2532].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2533 GTM_gtm_cls0.MCS0_MEM[2533].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2534 GTM_gtm_cls0.MCS0_MEM[2534].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2535 GTM_gtm_cls0.MCS0_MEM[2535].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2536 GTM_gtm_cls0.MCS0_MEM[2536].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2537 GTM_gtm_cls0.MCS0_MEM[2537].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2538 GTM_gtm_cls0.MCS0_MEM[2538].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2539 GTM_gtm_cls0.MCS0_MEM[2539].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2540 GTM_gtm_cls0.MCS0_MEM[2540].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2541 GTM_gtm_cls0.MCS0_MEM[2541].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2542 GTM_gtm_cls0.MCS0_MEM[2542].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2543 GTM_gtm_cls0.MCS0_MEM[2543].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2544 GTM_gtm_cls0.MCS0_MEM[2544].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2545 GTM_gtm_cls0.MCS0_MEM[2545].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2546 GTM_gtm_cls0.MCS0_MEM[2546].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2547 GTM_gtm_cls0.MCS0_MEM[2547].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2548 GTM_gtm_cls0.MCS0_MEM[2548].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2549 GTM_gtm_cls0.MCS0_MEM[2549].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2550 GTM_gtm_cls0.MCS0_MEM[2550].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2551 GTM_gtm_cls0.MCS0_MEM[2551].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2552 GTM_gtm_cls0.MCS0_MEM[2552].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2553 GTM_gtm_cls0.MCS0_MEM[2553].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2554 GTM_gtm_cls0.MCS0_MEM[2554].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2555 GTM_gtm_cls0.MCS0_MEM[2555].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2556 GTM_gtm_cls0.MCS0_MEM[2556].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2557 GTM_gtm_cls0.MCS0_MEM[2557].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2558 GTM_gtm_cls0.MCS0_MEM[2558].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2559 GTM_gtm_cls0.MCS0_MEM[2559].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2560 GTM_gtm_cls0.MCS0_MEM[2560].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2561 GTM_gtm_cls0.MCS0_MEM[2561].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2562 GTM_gtm_cls0.MCS0_MEM[2562].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2563 GTM_gtm_cls0.MCS0_MEM[2563].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2564 GTM_gtm_cls0.MCS0_MEM[2564].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2565 GTM_gtm_cls0.MCS0_MEM[2565].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2566 GTM_gtm_cls0.MCS0_MEM[2566].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2567 GTM_gtm_cls0.MCS0_MEM[2567].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2568 GTM_gtm_cls0.MCS0_MEM[2568].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2569 GTM_gtm_cls0.MCS0_MEM[2569].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2570 GTM_gtm_cls0.MCS0_MEM[2570].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2571 GTM_gtm_cls0.MCS0_MEM[2571].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2572 GTM_gtm_cls0.MCS0_MEM[2572].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2573 GTM_gtm_cls0.MCS0_MEM[2573].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2574 GTM_gtm_cls0.MCS0_MEM[2574].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2575 GTM_gtm_cls0.MCS0_MEM[2575].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2576 GTM_gtm_cls0.MCS0_MEM[2576].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2577 GTM_gtm_cls0.MCS0_MEM[2577].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2578 GTM_gtm_cls0.MCS0_MEM[2578].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2579 GTM_gtm_cls0.MCS0_MEM[2579].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2580 GTM_gtm_cls0.MCS0_MEM[2580].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2581 GTM_gtm_cls0.MCS0_MEM[2581].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2582 GTM_gtm_cls0.MCS0_MEM[2582].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2583 GTM_gtm_cls0.MCS0_MEM[2583].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2584 GTM_gtm_cls0.MCS0_MEM[2584].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2585 GTM_gtm_cls0.MCS0_MEM[2585].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2586 GTM_gtm_cls0.MCS0_MEM[2586].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2587 GTM_gtm_cls0.MCS0_MEM[2587].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2588 GTM_gtm_cls0.MCS0_MEM[2588].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2589 GTM_gtm_cls0.MCS0_MEM[2589].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2590 GTM_gtm_cls0.MCS0_MEM[2590].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2591 GTM_gtm_cls0.MCS0_MEM[2591].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2592 GTM_gtm_cls0.MCS0_MEM[2592].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2593 GTM_gtm_cls0.MCS0_MEM[2593].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2594 GTM_gtm_cls0.MCS0_MEM[2594].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2595 GTM_gtm_cls0.MCS0_MEM[2595].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2596 GTM_gtm_cls0.MCS0_MEM[2596].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2597 GTM_gtm_cls0.MCS0_MEM[2597].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2598 GTM_gtm_cls0.MCS0_MEM[2598].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2599 GTM_gtm_cls0.MCS0_MEM[2599].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2600 GTM_gtm_cls0.MCS0_MEM[2600].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2601 GTM_gtm_cls0.MCS0_MEM[2601].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2602 GTM_gtm_cls0.MCS0_MEM[2602].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2603 GTM_gtm_cls0.MCS0_MEM[2603].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2604 GTM_gtm_cls0.MCS0_MEM[2604].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2605 GTM_gtm_cls0.MCS0_MEM[2605].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2606 GTM_gtm_cls0.MCS0_MEM[2606].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2607 GTM_gtm_cls0.MCS0_MEM[2607].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2608 GTM_gtm_cls0.MCS0_MEM[2608].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2609 GTM_gtm_cls0.MCS0_MEM[2609].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2610 GTM_gtm_cls0.MCS0_MEM[2610].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2611 GTM_gtm_cls0.MCS0_MEM[2611].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2612 GTM_gtm_cls0.MCS0_MEM[2612].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2613 GTM_gtm_cls0.MCS0_MEM[2613].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2614 GTM_gtm_cls0.MCS0_MEM[2614].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2615 GTM_gtm_cls0.MCS0_MEM[2615].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2616 GTM_gtm_cls0.MCS0_MEM[2616].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2617 GTM_gtm_cls0.MCS0_MEM[2617].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2618 GTM_gtm_cls0.MCS0_MEM[2618].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2619 GTM_gtm_cls0.MCS0_MEM[2619].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2620 GTM_gtm_cls0.MCS0_MEM[2620].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2621 GTM_gtm_cls0.MCS0_MEM[2621].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2622 GTM_gtm_cls0.MCS0_MEM[2622].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2623 GTM_gtm_cls0.MCS0_MEM[2623].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2624 GTM_gtm_cls0.MCS0_MEM[2624].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2625 GTM_gtm_cls0.MCS0_MEM[2625].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2626 GTM_gtm_cls0.MCS0_MEM[2626].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2627 GTM_gtm_cls0.MCS0_MEM[2627].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2628 GTM_gtm_cls0.MCS0_MEM[2628].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2629 GTM_gtm_cls0.MCS0_MEM[2629].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2630 GTM_gtm_cls0.MCS0_MEM[2630].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2631 GTM_gtm_cls0.MCS0_MEM[2631].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2632 GTM_gtm_cls0.MCS0_MEM[2632].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2633 GTM_gtm_cls0.MCS0_MEM[2633].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2634 GTM_gtm_cls0.MCS0_MEM[2634].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2635 GTM_gtm_cls0.MCS0_MEM[2635].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2636 GTM_gtm_cls0.MCS0_MEM[2636].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2637 GTM_gtm_cls0.MCS0_MEM[2637].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2638 GTM_gtm_cls0.MCS0_MEM[2638].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2639 GTM_gtm_cls0.MCS0_MEM[2639].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2640 GTM_gtm_cls0.MCS0_MEM[2640].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2641 GTM_gtm_cls0.MCS0_MEM[2641].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2642 GTM_gtm_cls0.MCS0_MEM[2642].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2643 GTM_gtm_cls0.MCS0_MEM[2643].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2644 GTM_gtm_cls0.MCS0_MEM[2644].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2645 GTM_gtm_cls0.MCS0_MEM[2645].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2646 GTM_gtm_cls0.MCS0_MEM[2646].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2647 GTM_gtm_cls0.MCS0_MEM[2647].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2648 GTM_gtm_cls0.MCS0_MEM[2648].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2649 GTM_gtm_cls0.MCS0_MEM[2649].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2650 GTM_gtm_cls0.MCS0_MEM[2650].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2651 GTM_gtm_cls0.MCS0_MEM[2651].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2652 GTM_gtm_cls0.MCS0_MEM[2652].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2653 GTM_gtm_cls0.MCS0_MEM[2653].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2654 GTM_gtm_cls0.MCS0_MEM[2654].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2655 GTM_gtm_cls0.MCS0_MEM[2655].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2656 GTM_gtm_cls0.MCS0_MEM[2656].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2657 GTM_gtm_cls0.MCS0_MEM[2657].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2658 GTM_gtm_cls0.MCS0_MEM[2658].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2659 GTM_gtm_cls0.MCS0_MEM[2659].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2660 GTM_gtm_cls0.MCS0_MEM[2660].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2661 GTM_gtm_cls0.MCS0_MEM[2661].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2662 GTM_gtm_cls0.MCS0_MEM[2662].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2663 GTM_gtm_cls0.MCS0_MEM[2663].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2664 GTM_gtm_cls0.MCS0_MEM[2664].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2665 GTM_gtm_cls0.MCS0_MEM[2665].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2666 GTM_gtm_cls0.MCS0_MEM[2666].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2667 GTM_gtm_cls0.MCS0_MEM[2667].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2668 GTM_gtm_cls0.MCS0_MEM[2668].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2669 GTM_gtm_cls0.MCS0_MEM[2669].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2670 GTM_gtm_cls0.MCS0_MEM[2670].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2671 GTM_gtm_cls0.MCS0_MEM[2671].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2672 GTM_gtm_cls0.MCS0_MEM[2672].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2673 GTM_gtm_cls0.MCS0_MEM[2673].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2674 GTM_gtm_cls0.MCS0_MEM[2674].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2675 GTM_gtm_cls0.MCS0_MEM[2675].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2676 GTM_gtm_cls0.MCS0_MEM[2676].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2677 GTM_gtm_cls0.MCS0_MEM[2677].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2678 GTM_gtm_cls0.MCS0_MEM[2678].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2679 GTM_gtm_cls0.MCS0_MEM[2679].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2680 GTM_gtm_cls0.MCS0_MEM[2680].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2681 GTM_gtm_cls0.MCS0_MEM[2681].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2682 GTM_gtm_cls0.MCS0_MEM[2682].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2683 GTM_gtm_cls0.MCS0_MEM[2683].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2684 GTM_gtm_cls0.MCS0_MEM[2684].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2685 GTM_gtm_cls0.MCS0_MEM[2685].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2686 GTM_gtm_cls0.MCS0_MEM[2686].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2687 GTM_gtm_cls0.MCS0_MEM[2687].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2688 GTM_gtm_cls0.MCS0_MEM[2688].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2689 GTM_gtm_cls0.MCS0_MEM[2689].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2690 GTM_gtm_cls0.MCS0_MEM[2690].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2691 GTM_gtm_cls0.MCS0_MEM[2691].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2692 GTM_gtm_cls0.MCS0_MEM[2692].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2693 GTM_gtm_cls0.MCS0_MEM[2693].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2694 GTM_gtm_cls0.MCS0_MEM[2694].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2695 GTM_gtm_cls0.MCS0_MEM[2695].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2696 GTM_gtm_cls0.MCS0_MEM[2696].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2697 GTM_gtm_cls0.MCS0_MEM[2697].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2698 GTM_gtm_cls0.MCS0_MEM[2698].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2699 GTM_gtm_cls0.MCS0_MEM[2699].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2700 GTM_gtm_cls0.MCS0_MEM[2700].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2701 GTM_gtm_cls0.MCS0_MEM[2701].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2702 GTM_gtm_cls0.MCS0_MEM[2702].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2703 GTM_gtm_cls0.MCS0_MEM[2703].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2704 GTM_gtm_cls0.MCS0_MEM[2704].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2705 GTM_gtm_cls0.MCS0_MEM[2705].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2706 GTM_gtm_cls0.MCS0_MEM[2706].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2707 GTM_gtm_cls0.MCS0_MEM[2707].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2708 GTM_gtm_cls0.MCS0_MEM[2708].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2709 GTM_gtm_cls0.MCS0_MEM[2709].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2710 GTM_gtm_cls0.MCS0_MEM[2710].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2711 GTM_gtm_cls0.MCS0_MEM[2711].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2712 GTM_gtm_cls0.MCS0_MEM[2712].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2713 GTM_gtm_cls0.MCS0_MEM[2713].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2714 GTM_gtm_cls0.MCS0_MEM[2714].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2715 GTM_gtm_cls0.MCS0_MEM[2715].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2716 GTM_gtm_cls0.MCS0_MEM[2716].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2717 GTM_gtm_cls0.MCS0_MEM[2717].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2718 GTM_gtm_cls0.MCS0_MEM[2718].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2719 GTM_gtm_cls0.MCS0_MEM[2719].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2720 GTM_gtm_cls0.MCS0_MEM[2720].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2721 GTM_gtm_cls0.MCS0_MEM[2721].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2722 GTM_gtm_cls0.MCS0_MEM[2722].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2723 GTM_gtm_cls0.MCS0_MEM[2723].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2724 GTM_gtm_cls0.MCS0_MEM[2724].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2725 GTM_gtm_cls0.MCS0_MEM[2725].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2726 GTM_gtm_cls0.MCS0_MEM[2726].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2727 GTM_gtm_cls0.MCS0_MEM[2727].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2728 GTM_gtm_cls0.MCS0_MEM[2728].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2729 GTM_gtm_cls0.MCS0_MEM[2729].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2730 GTM_gtm_cls0.MCS0_MEM[2730].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2731 GTM_gtm_cls0.MCS0_MEM[2731].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2732 GTM_gtm_cls0.MCS0_MEM[2732].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2733 GTM_gtm_cls0.MCS0_MEM[2733].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2734 GTM_gtm_cls0.MCS0_MEM[2734].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2735 GTM_gtm_cls0.MCS0_MEM[2735].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2736 GTM_gtm_cls0.MCS0_MEM[2736].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2737 GTM_gtm_cls0.MCS0_MEM[2737].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2738 GTM_gtm_cls0.MCS0_MEM[2738].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2739 GTM_gtm_cls0.MCS0_MEM[2739].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2740 GTM_gtm_cls0.MCS0_MEM[2740].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2741 GTM_gtm_cls0.MCS0_MEM[2741].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2742 GTM_gtm_cls0.MCS0_MEM[2742].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2743 GTM_gtm_cls0.MCS0_MEM[2743].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2744 GTM_gtm_cls0.MCS0_MEM[2744].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2745 GTM_gtm_cls0.MCS0_MEM[2745].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2746 GTM_gtm_cls0.MCS0_MEM[2746].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2747 GTM_gtm_cls0.MCS0_MEM[2747].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2748 GTM_gtm_cls0.MCS0_MEM[2748].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2749 GTM_gtm_cls0.MCS0_MEM[2749].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2750 GTM_gtm_cls0.MCS0_MEM[2750].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2751 GTM_gtm_cls0.MCS0_MEM[2751].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2752 GTM_gtm_cls0.MCS0_MEM[2752].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2753 GTM_gtm_cls0.MCS0_MEM[2753].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2754 GTM_gtm_cls0.MCS0_MEM[2754].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2755 GTM_gtm_cls0.MCS0_MEM[2755].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2756 GTM_gtm_cls0.MCS0_MEM[2756].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2757 GTM_gtm_cls0.MCS0_MEM[2757].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2758 GTM_gtm_cls0.MCS0_MEM[2758].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2759 GTM_gtm_cls0.MCS0_MEM[2759].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2760 GTM_gtm_cls0.MCS0_MEM[2760].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2761 GTM_gtm_cls0.MCS0_MEM[2761].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2762 GTM_gtm_cls0.MCS0_MEM[2762].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2763 GTM_gtm_cls0.MCS0_MEM[2763].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2764 GTM_gtm_cls0.MCS0_MEM[2764].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2765 GTM_gtm_cls0.MCS0_MEM[2765].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2766 GTM_gtm_cls0.MCS0_MEM[2766].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2767 GTM_gtm_cls0.MCS0_MEM[2767].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2768 GTM_gtm_cls0.MCS0_MEM[2768].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2769 GTM_gtm_cls0.MCS0_MEM[2769].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2770 GTM_gtm_cls0.MCS0_MEM[2770].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2771 GTM_gtm_cls0.MCS0_MEM[2771].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2772 GTM_gtm_cls0.MCS0_MEM[2772].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2773 GTM_gtm_cls0.MCS0_MEM[2773].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2774 GTM_gtm_cls0.MCS0_MEM[2774].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2775 GTM_gtm_cls0.MCS0_MEM[2775].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2776 GTM_gtm_cls0.MCS0_MEM[2776].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2777 GTM_gtm_cls0.MCS0_MEM[2777].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2778 GTM_gtm_cls0.MCS0_MEM[2778].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2779 GTM_gtm_cls0.MCS0_MEM[2779].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2780 GTM_gtm_cls0.MCS0_MEM[2780].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2781 GTM_gtm_cls0.MCS0_MEM[2781].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2782 GTM_gtm_cls0.MCS0_MEM[2782].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2783 GTM_gtm_cls0.MCS0_MEM[2783].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2784 GTM_gtm_cls0.MCS0_MEM[2784].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2785 GTM_gtm_cls0.MCS0_MEM[2785].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2786 GTM_gtm_cls0.MCS0_MEM[2786].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2787 GTM_gtm_cls0.MCS0_MEM[2787].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2788 GTM_gtm_cls0.MCS0_MEM[2788].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2789 GTM_gtm_cls0.MCS0_MEM[2789].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2790 GTM_gtm_cls0.MCS0_MEM[2790].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2791 GTM_gtm_cls0.MCS0_MEM[2791].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2792 GTM_gtm_cls0.MCS0_MEM[2792].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2793 GTM_gtm_cls0.MCS0_MEM[2793].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2794 GTM_gtm_cls0.MCS0_MEM[2794].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2795 GTM_gtm_cls0.MCS0_MEM[2795].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2796 GTM_gtm_cls0.MCS0_MEM[2796].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2797 GTM_gtm_cls0.MCS0_MEM[2797].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2798 GTM_gtm_cls0.MCS0_MEM[2798].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2799 GTM_gtm_cls0.MCS0_MEM[2799].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2800 GTM_gtm_cls0.MCS0_MEM[2800].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2801 GTM_gtm_cls0.MCS0_MEM[2801].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2802 GTM_gtm_cls0.MCS0_MEM[2802].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2803 GTM_gtm_cls0.MCS0_MEM[2803].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2804 GTM_gtm_cls0.MCS0_MEM[2804].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2805 GTM_gtm_cls0.MCS0_MEM[2805].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2806 GTM_gtm_cls0.MCS0_MEM[2806].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2807 GTM_gtm_cls0.MCS0_MEM[2807].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2808 GTM_gtm_cls0.MCS0_MEM[2808].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2809 GTM_gtm_cls0.MCS0_MEM[2809].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2810 GTM_gtm_cls0.MCS0_MEM[2810].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2811 GTM_gtm_cls0.MCS0_MEM[2811].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2812 GTM_gtm_cls0.MCS0_MEM[2812].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2813 GTM_gtm_cls0.MCS0_MEM[2813].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2814 GTM_gtm_cls0.MCS0_MEM[2814].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2815 GTM_gtm_cls0.MCS0_MEM[2815].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2816 GTM_gtm_cls0.MCS0_MEM[2816].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2817 GTM_gtm_cls0.MCS0_MEM[2817].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2818 GTM_gtm_cls0.MCS0_MEM[2818].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2819 GTM_gtm_cls0.MCS0_MEM[2819].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2820 GTM_gtm_cls0.MCS0_MEM[2820].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2821 GTM_gtm_cls0.MCS0_MEM[2821].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2822 GTM_gtm_cls0.MCS0_MEM[2822].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2823 GTM_gtm_cls0.MCS0_MEM[2823].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2824 GTM_gtm_cls0.MCS0_MEM[2824].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2825 GTM_gtm_cls0.MCS0_MEM[2825].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2826 GTM_gtm_cls0.MCS0_MEM[2826].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2827 GTM_gtm_cls0.MCS0_MEM[2827].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2828 GTM_gtm_cls0.MCS0_MEM[2828].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2829 GTM_gtm_cls0.MCS0_MEM[2829].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2830 GTM_gtm_cls0.MCS0_MEM[2830].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2831 GTM_gtm_cls0.MCS0_MEM[2831].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2832 GTM_gtm_cls0.MCS0_MEM[2832].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2833 GTM_gtm_cls0.MCS0_MEM[2833].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2834 GTM_gtm_cls0.MCS0_MEM[2834].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2835 GTM_gtm_cls0.MCS0_MEM[2835].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2836 GTM_gtm_cls0.MCS0_MEM[2836].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2837 GTM_gtm_cls0.MCS0_MEM[2837].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2838 GTM_gtm_cls0.MCS0_MEM[2838].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2839 GTM_gtm_cls0.MCS0_MEM[2839].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2840 GTM_gtm_cls0.MCS0_MEM[2840].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2841 GTM_gtm_cls0.MCS0_MEM[2841].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2842 GTM_gtm_cls0.MCS0_MEM[2842].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2843 GTM_gtm_cls0.MCS0_MEM[2843].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2844 GTM_gtm_cls0.MCS0_MEM[2844].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2845 GTM_gtm_cls0.MCS0_MEM[2845].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2846 GTM_gtm_cls0.MCS0_MEM[2846].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2847 GTM_gtm_cls0.MCS0_MEM[2847].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2848 GTM_gtm_cls0.MCS0_MEM[2848].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2849 GTM_gtm_cls0.MCS0_MEM[2849].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2850 GTM_gtm_cls0.MCS0_MEM[2850].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2851 GTM_gtm_cls0.MCS0_MEM[2851].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2852 GTM_gtm_cls0.MCS0_MEM[2852].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2853 GTM_gtm_cls0.MCS0_MEM[2853].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2854 GTM_gtm_cls0.MCS0_MEM[2854].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2855 GTM_gtm_cls0.MCS0_MEM[2855].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2856 GTM_gtm_cls0.MCS0_MEM[2856].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2857 GTM_gtm_cls0.MCS0_MEM[2857].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2858 GTM_gtm_cls0.MCS0_MEM[2858].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2859 GTM_gtm_cls0.MCS0_MEM[2859].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2860 GTM_gtm_cls0.MCS0_MEM[2860].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2861 GTM_gtm_cls0.MCS0_MEM[2861].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2862 GTM_gtm_cls0.MCS0_MEM[2862].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2863 GTM_gtm_cls0.MCS0_MEM[2863].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2864 GTM_gtm_cls0.MCS0_MEM[2864].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2865 GTM_gtm_cls0.MCS0_MEM[2865].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2866 GTM_gtm_cls0.MCS0_MEM[2866].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2867 GTM_gtm_cls0.MCS0_MEM[2867].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2868 GTM_gtm_cls0.MCS0_MEM[2868].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2869 GTM_gtm_cls0.MCS0_MEM[2869].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2870 GTM_gtm_cls0.MCS0_MEM[2870].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2871 GTM_gtm_cls0.MCS0_MEM[2871].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2872 GTM_gtm_cls0.MCS0_MEM[2872].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2873 GTM_gtm_cls0.MCS0_MEM[2873].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2874 GTM_gtm_cls0.MCS0_MEM[2874].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2875 GTM_gtm_cls0.MCS0_MEM[2875].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2876 GTM_gtm_cls0.MCS0_MEM[2876].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2877 GTM_gtm_cls0.MCS0_MEM[2877].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2878 GTM_gtm_cls0.MCS0_MEM[2878].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2879 GTM_gtm_cls0.MCS0_MEM[2879].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2880 GTM_gtm_cls0.MCS0_MEM[2880].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2881 GTM_gtm_cls0.MCS0_MEM[2881].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2882 GTM_gtm_cls0.MCS0_MEM[2882].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2883 GTM_gtm_cls0.MCS0_MEM[2883].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2884 GTM_gtm_cls0.MCS0_MEM[2884].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2885 GTM_gtm_cls0.MCS0_MEM[2885].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2886 GTM_gtm_cls0.MCS0_MEM[2886].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2887 GTM_gtm_cls0.MCS0_MEM[2887].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2888 GTM_gtm_cls0.MCS0_MEM[2888].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2889 GTM_gtm_cls0.MCS0_MEM[2889].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2890 GTM_gtm_cls0.MCS0_MEM[2890].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2891 GTM_gtm_cls0.MCS0_MEM[2891].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2892 GTM_gtm_cls0.MCS0_MEM[2892].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2893 GTM_gtm_cls0.MCS0_MEM[2893].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2894 GTM_gtm_cls0.MCS0_MEM[2894].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2895 GTM_gtm_cls0.MCS0_MEM[2895].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2896 GTM_gtm_cls0.MCS0_MEM[2896].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2897 GTM_gtm_cls0.MCS0_MEM[2897].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2898 GTM_gtm_cls0.MCS0_MEM[2898].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2899 GTM_gtm_cls0.MCS0_MEM[2899].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2900 GTM_gtm_cls0.MCS0_MEM[2900].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2901 GTM_gtm_cls0.MCS0_MEM[2901].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2902 GTM_gtm_cls0.MCS0_MEM[2902].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2903 GTM_gtm_cls0.MCS0_MEM[2903].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2904 GTM_gtm_cls0.MCS0_MEM[2904].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2905 GTM_gtm_cls0.MCS0_MEM[2905].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2906 GTM_gtm_cls0.MCS0_MEM[2906].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2907 GTM_gtm_cls0.MCS0_MEM[2907].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2908 GTM_gtm_cls0.MCS0_MEM[2908].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2909 GTM_gtm_cls0.MCS0_MEM[2909].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2910 GTM_gtm_cls0.MCS0_MEM[2910].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2911 GTM_gtm_cls0.MCS0_MEM[2911].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2912 GTM_gtm_cls0.MCS0_MEM[2912].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2913 GTM_gtm_cls0.MCS0_MEM[2913].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2914 GTM_gtm_cls0.MCS0_MEM[2914].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2915 GTM_gtm_cls0.MCS0_MEM[2915].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2916 GTM_gtm_cls0.MCS0_MEM[2916].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2917 GTM_gtm_cls0.MCS0_MEM[2917].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2918 GTM_gtm_cls0.MCS0_MEM[2918].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2919 GTM_gtm_cls0.MCS0_MEM[2919].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2920 GTM_gtm_cls0.MCS0_MEM[2920].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2921 GTM_gtm_cls0.MCS0_MEM[2921].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2922 GTM_gtm_cls0.MCS0_MEM[2922].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2923 GTM_gtm_cls0.MCS0_MEM[2923].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2924 GTM_gtm_cls0.MCS0_MEM[2924].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2925 GTM_gtm_cls0.MCS0_MEM[2925].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2926 GTM_gtm_cls0.MCS0_MEM[2926].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2927 GTM_gtm_cls0.MCS0_MEM[2927].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2928 GTM_gtm_cls0.MCS0_MEM[2928].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2929 GTM_gtm_cls0.MCS0_MEM[2929].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2930 GTM_gtm_cls0.MCS0_MEM[2930].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2931 GTM_gtm_cls0.MCS0_MEM[2931].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2932 GTM_gtm_cls0.MCS0_MEM[2932].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2933 GTM_gtm_cls0.MCS0_MEM[2933].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2934 GTM_gtm_cls0.MCS0_MEM[2934].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2935 GTM_gtm_cls0.MCS0_MEM[2935].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2936 GTM_gtm_cls0.MCS0_MEM[2936].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2937 GTM_gtm_cls0.MCS0_MEM[2937].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2938 GTM_gtm_cls0.MCS0_MEM[2938].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2939 GTM_gtm_cls0.MCS0_MEM[2939].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2940 GTM_gtm_cls0.MCS0_MEM[2940].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2941 GTM_gtm_cls0.MCS0_MEM[2941].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2942 GTM_gtm_cls0.MCS0_MEM[2942].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2943 GTM_gtm_cls0.MCS0_MEM[2943].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2944 GTM_gtm_cls0.MCS0_MEM[2944].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2945 GTM_gtm_cls0.MCS0_MEM[2945].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2946 GTM_gtm_cls0.MCS0_MEM[2946].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2947 GTM_gtm_cls0.MCS0_MEM[2947].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2948 GTM_gtm_cls0.MCS0_MEM[2948].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2949 GTM_gtm_cls0.MCS0_MEM[2949].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2950 GTM_gtm_cls0.MCS0_MEM[2950].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2951 GTM_gtm_cls0.MCS0_MEM[2951].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2952 GTM_gtm_cls0.MCS0_MEM[2952].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2953 GTM_gtm_cls0.MCS0_MEM[2953].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2954 GTM_gtm_cls0.MCS0_MEM[2954].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2955 GTM_gtm_cls0.MCS0_MEM[2955].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2956 GTM_gtm_cls0.MCS0_MEM[2956].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2957 GTM_gtm_cls0.MCS0_MEM[2957].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2958 GTM_gtm_cls0.MCS0_MEM[2958].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2959 GTM_gtm_cls0.MCS0_MEM[2959].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2960 GTM_gtm_cls0.MCS0_MEM[2960].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2961 GTM_gtm_cls0.MCS0_MEM[2961].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2962 GTM_gtm_cls0.MCS0_MEM[2962].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2963 GTM_gtm_cls0.MCS0_MEM[2963].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2964 GTM_gtm_cls0.MCS0_MEM[2964].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2965 GTM_gtm_cls0.MCS0_MEM[2965].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2966 GTM_gtm_cls0.MCS0_MEM[2966].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2967 GTM_gtm_cls0.MCS0_MEM[2967].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2968 GTM_gtm_cls0.MCS0_MEM[2968].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2969 GTM_gtm_cls0.MCS0_MEM[2969].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2970 GTM_gtm_cls0.MCS0_MEM[2970].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2971 GTM_gtm_cls0.MCS0_MEM[2971].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2972 GTM_gtm_cls0.MCS0_MEM[2972].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2973 GTM_gtm_cls0.MCS0_MEM[2973].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2974 GTM_gtm_cls0.MCS0_MEM[2974].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2975 GTM_gtm_cls0.MCS0_MEM[2975].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2976 GTM_gtm_cls0.MCS0_MEM[2976].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2977 GTM_gtm_cls0.MCS0_MEM[2977].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2978 GTM_gtm_cls0.MCS0_MEM[2978].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2979 GTM_gtm_cls0.MCS0_MEM[2979].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2980 GTM_gtm_cls0.MCS0_MEM[2980].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2981 GTM_gtm_cls0.MCS0_MEM[2981].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2982 GTM_gtm_cls0.MCS0_MEM[2982].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2983 GTM_gtm_cls0.MCS0_MEM[2983].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2984 GTM_gtm_cls0.MCS0_MEM[2984].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2985 GTM_gtm_cls0.MCS0_MEM[2985].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2986 GTM_gtm_cls0.MCS0_MEM[2986].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2987 GTM_gtm_cls0.MCS0_MEM[2987].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2988 GTM_gtm_cls0.MCS0_MEM[2988].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2989 GTM_gtm_cls0.MCS0_MEM[2989].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2990 GTM_gtm_cls0.MCS0_MEM[2990].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2991 GTM_gtm_cls0.MCS0_MEM[2991].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2992 GTM_gtm_cls0.MCS0_MEM[2992].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2993 GTM_gtm_cls0.MCS0_MEM[2993].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2994 GTM_gtm_cls0.MCS0_MEM[2994].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2995 GTM_gtm_cls0.MCS0_MEM[2995].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2996 GTM_gtm_cls0.MCS0_MEM[2996].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2997 GTM_gtm_cls0.MCS0_MEM[2997].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2998 GTM_gtm_cls0.MCS0_MEM[2998].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM2999 GTM_gtm_cls0.MCS0_MEM[2999].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3000 GTM_gtm_cls0.MCS0_MEM[3000].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3001 GTM_gtm_cls0.MCS0_MEM[3001].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3002 GTM_gtm_cls0.MCS0_MEM[3002].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3003 GTM_gtm_cls0.MCS0_MEM[3003].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3004 GTM_gtm_cls0.MCS0_MEM[3004].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3005 GTM_gtm_cls0.MCS0_MEM[3005].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3006 GTM_gtm_cls0.MCS0_MEM[3006].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3007 GTM_gtm_cls0.MCS0_MEM[3007].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3008 GTM_gtm_cls0.MCS0_MEM[3008].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3009 GTM_gtm_cls0.MCS0_MEM[3009].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3010 GTM_gtm_cls0.MCS0_MEM[3010].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3011 GTM_gtm_cls0.MCS0_MEM[3011].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3012 GTM_gtm_cls0.MCS0_MEM[3012].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3013 GTM_gtm_cls0.MCS0_MEM[3013].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3014 GTM_gtm_cls0.MCS0_MEM[3014].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3015 GTM_gtm_cls0.MCS0_MEM[3015].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3016 GTM_gtm_cls0.MCS0_MEM[3016].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3017 GTM_gtm_cls0.MCS0_MEM[3017].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3018 GTM_gtm_cls0.MCS0_MEM[3018].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3019 GTM_gtm_cls0.MCS0_MEM[3019].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3020 GTM_gtm_cls0.MCS0_MEM[3020].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3021 GTM_gtm_cls0.MCS0_MEM[3021].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3022 GTM_gtm_cls0.MCS0_MEM[3022].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3023 GTM_gtm_cls0.MCS0_MEM[3023].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3024 GTM_gtm_cls0.MCS0_MEM[3024].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3025 GTM_gtm_cls0.MCS0_MEM[3025].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3026 GTM_gtm_cls0.MCS0_MEM[3026].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3027 GTM_gtm_cls0.MCS0_MEM[3027].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3028 GTM_gtm_cls0.MCS0_MEM[3028].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3029 GTM_gtm_cls0.MCS0_MEM[3029].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3030 GTM_gtm_cls0.MCS0_MEM[3030].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3031 GTM_gtm_cls0.MCS0_MEM[3031].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3032 GTM_gtm_cls0.MCS0_MEM[3032].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3033 GTM_gtm_cls0.MCS0_MEM[3033].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3034 GTM_gtm_cls0.MCS0_MEM[3034].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3035 GTM_gtm_cls0.MCS0_MEM[3035].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3036 GTM_gtm_cls0.MCS0_MEM[3036].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3037 GTM_gtm_cls0.MCS0_MEM[3037].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3038 GTM_gtm_cls0.MCS0_MEM[3038].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3039 GTM_gtm_cls0.MCS0_MEM[3039].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3040 GTM_gtm_cls0.MCS0_MEM[3040].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3041 GTM_gtm_cls0.MCS0_MEM[3041].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3042 GTM_gtm_cls0.MCS0_MEM[3042].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3043 GTM_gtm_cls0.MCS0_MEM[3043].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3044 GTM_gtm_cls0.MCS0_MEM[3044].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3045 GTM_gtm_cls0.MCS0_MEM[3045].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3046 GTM_gtm_cls0.MCS0_MEM[3046].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3047 GTM_gtm_cls0.MCS0_MEM[3047].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3048 GTM_gtm_cls0.MCS0_MEM[3048].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3049 GTM_gtm_cls0.MCS0_MEM[3049].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3050 GTM_gtm_cls0.MCS0_MEM[3050].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3051 GTM_gtm_cls0.MCS0_MEM[3051].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3052 GTM_gtm_cls0.MCS0_MEM[3052].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3053 GTM_gtm_cls0.MCS0_MEM[3053].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3054 GTM_gtm_cls0.MCS0_MEM[3054].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3055 GTM_gtm_cls0.MCS0_MEM[3055].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3056 GTM_gtm_cls0.MCS0_MEM[3056].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3057 GTM_gtm_cls0.MCS0_MEM[3057].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3058 GTM_gtm_cls0.MCS0_MEM[3058].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3059 GTM_gtm_cls0.MCS0_MEM[3059].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3060 GTM_gtm_cls0.MCS0_MEM[3060].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3061 GTM_gtm_cls0.MCS0_MEM[3061].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3062 GTM_gtm_cls0.MCS0_MEM[3062].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3063 GTM_gtm_cls0.MCS0_MEM[3063].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3064 GTM_gtm_cls0.MCS0_MEM[3064].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3065 GTM_gtm_cls0.MCS0_MEM[3065].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3066 GTM_gtm_cls0.MCS0_MEM[3066].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3067 GTM_gtm_cls0.MCS0_MEM[3067].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3068 GTM_gtm_cls0.MCS0_MEM[3068].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3069 GTM_gtm_cls0.MCS0_MEM[3069].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3070 GTM_gtm_cls0.MCS0_MEM[3070].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3071 GTM_gtm_cls0.MCS0_MEM[3071].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3072 GTM_gtm_cls0.MCS0_MEM[3072].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3073 GTM_gtm_cls0.MCS0_MEM[3073].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3074 GTM_gtm_cls0.MCS0_MEM[3074].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3075 GTM_gtm_cls0.MCS0_MEM[3075].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3076 GTM_gtm_cls0.MCS0_MEM[3076].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3077 GTM_gtm_cls0.MCS0_MEM[3077].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3078 GTM_gtm_cls0.MCS0_MEM[3078].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3079 GTM_gtm_cls0.MCS0_MEM[3079].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3080 GTM_gtm_cls0.MCS0_MEM[3080].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3081 GTM_gtm_cls0.MCS0_MEM[3081].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3082 GTM_gtm_cls0.MCS0_MEM[3082].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3083 GTM_gtm_cls0.MCS0_MEM[3083].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3084 GTM_gtm_cls0.MCS0_MEM[3084].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3085 GTM_gtm_cls0.MCS0_MEM[3085].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3086 GTM_gtm_cls0.MCS0_MEM[3086].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3087 GTM_gtm_cls0.MCS0_MEM[3087].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3088 GTM_gtm_cls0.MCS0_MEM[3088].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3089 GTM_gtm_cls0.MCS0_MEM[3089].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3090 GTM_gtm_cls0.MCS0_MEM[3090].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3091 GTM_gtm_cls0.MCS0_MEM[3091].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3092 GTM_gtm_cls0.MCS0_MEM[3092].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3093 GTM_gtm_cls0.MCS0_MEM[3093].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3094 GTM_gtm_cls0.MCS0_MEM[3094].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3095 GTM_gtm_cls0.MCS0_MEM[3095].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3096 GTM_gtm_cls0.MCS0_MEM[3096].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3097 GTM_gtm_cls0.MCS0_MEM[3097].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3098 GTM_gtm_cls0.MCS0_MEM[3098].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3099 GTM_gtm_cls0.MCS0_MEM[3099].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3100 GTM_gtm_cls0.MCS0_MEM[3100].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3101 GTM_gtm_cls0.MCS0_MEM[3101].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3102 GTM_gtm_cls0.MCS0_MEM[3102].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3103 GTM_gtm_cls0.MCS0_MEM[3103].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3104 GTM_gtm_cls0.MCS0_MEM[3104].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3105 GTM_gtm_cls0.MCS0_MEM[3105].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3106 GTM_gtm_cls0.MCS0_MEM[3106].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3107 GTM_gtm_cls0.MCS0_MEM[3107].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3108 GTM_gtm_cls0.MCS0_MEM[3108].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3109 GTM_gtm_cls0.MCS0_MEM[3109].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3110 GTM_gtm_cls0.MCS0_MEM[3110].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3111 GTM_gtm_cls0.MCS0_MEM[3111].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3112 GTM_gtm_cls0.MCS0_MEM[3112].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3113 GTM_gtm_cls0.MCS0_MEM[3113].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3114 GTM_gtm_cls0.MCS0_MEM[3114].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3115 GTM_gtm_cls0.MCS0_MEM[3115].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3116 GTM_gtm_cls0.MCS0_MEM[3116].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3117 GTM_gtm_cls0.MCS0_MEM[3117].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3118 GTM_gtm_cls0.MCS0_MEM[3118].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3119 GTM_gtm_cls0.MCS0_MEM[3119].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3120 GTM_gtm_cls0.MCS0_MEM[3120].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3121 GTM_gtm_cls0.MCS0_MEM[3121].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3122 GTM_gtm_cls0.MCS0_MEM[3122].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3123 GTM_gtm_cls0.MCS0_MEM[3123].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3124 GTM_gtm_cls0.MCS0_MEM[3124].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3125 GTM_gtm_cls0.MCS0_MEM[3125].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3126 GTM_gtm_cls0.MCS0_MEM[3126].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3127 GTM_gtm_cls0.MCS0_MEM[3127].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3128 GTM_gtm_cls0.MCS0_MEM[3128].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3129 GTM_gtm_cls0.MCS0_MEM[3129].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3130 GTM_gtm_cls0.MCS0_MEM[3130].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3131 GTM_gtm_cls0.MCS0_MEM[3131].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3132 GTM_gtm_cls0.MCS0_MEM[3132].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3133 GTM_gtm_cls0.MCS0_MEM[3133].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3134 GTM_gtm_cls0.MCS0_MEM[3134].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3135 GTM_gtm_cls0.MCS0_MEM[3135].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3136 GTM_gtm_cls0.MCS0_MEM[3136].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3137 GTM_gtm_cls0.MCS0_MEM[3137].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3138 GTM_gtm_cls0.MCS0_MEM[3138].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3139 GTM_gtm_cls0.MCS0_MEM[3139].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3140 GTM_gtm_cls0.MCS0_MEM[3140].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3141 GTM_gtm_cls0.MCS0_MEM[3141].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3142 GTM_gtm_cls0.MCS0_MEM[3142].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3143 GTM_gtm_cls0.MCS0_MEM[3143].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3144 GTM_gtm_cls0.MCS0_MEM[3144].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3145 GTM_gtm_cls0.MCS0_MEM[3145].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3146 GTM_gtm_cls0.MCS0_MEM[3146].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3147 GTM_gtm_cls0.MCS0_MEM[3147].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3148 GTM_gtm_cls0.MCS0_MEM[3148].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3149 GTM_gtm_cls0.MCS0_MEM[3149].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3150 GTM_gtm_cls0.MCS0_MEM[3150].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3151 GTM_gtm_cls0.MCS0_MEM[3151].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3152 GTM_gtm_cls0.MCS0_MEM[3152].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3153 GTM_gtm_cls0.MCS0_MEM[3153].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3154 GTM_gtm_cls0.MCS0_MEM[3154].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3155 GTM_gtm_cls0.MCS0_MEM[3155].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3156 GTM_gtm_cls0.MCS0_MEM[3156].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3157 GTM_gtm_cls0.MCS0_MEM[3157].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3158 GTM_gtm_cls0.MCS0_MEM[3158].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3159 GTM_gtm_cls0.MCS0_MEM[3159].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3160 GTM_gtm_cls0.MCS0_MEM[3160].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3161 GTM_gtm_cls0.MCS0_MEM[3161].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3162 GTM_gtm_cls0.MCS0_MEM[3162].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3163 GTM_gtm_cls0.MCS0_MEM[3163].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3164 GTM_gtm_cls0.MCS0_MEM[3164].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3165 GTM_gtm_cls0.MCS0_MEM[3165].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3166 GTM_gtm_cls0.MCS0_MEM[3166].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3167 GTM_gtm_cls0.MCS0_MEM[3167].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3168 GTM_gtm_cls0.MCS0_MEM[3168].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3169 GTM_gtm_cls0.MCS0_MEM[3169].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3170 GTM_gtm_cls0.MCS0_MEM[3170].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3171 GTM_gtm_cls0.MCS0_MEM[3171].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3172 GTM_gtm_cls0.MCS0_MEM[3172].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3173 GTM_gtm_cls0.MCS0_MEM[3173].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3174 GTM_gtm_cls0.MCS0_MEM[3174].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3175 GTM_gtm_cls0.MCS0_MEM[3175].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3176 GTM_gtm_cls0.MCS0_MEM[3176].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3177 GTM_gtm_cls0.MCS0_MEM[3177].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3178 GTM_gtm_cls0.MCS0_MEM[3178].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3179 GTM_gtm_cls0.MCS0_MEM[3179].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3180 GTM_gtm_cls0.MCS0_MEM[3180].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3181 GTM_gtm_cls0.MCS0_MEM[3181].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3182 GTM_gtm_cls0.MCS0_MEM[3182].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3183 GTM_gtm_cls0.MCS0_MEM[3183].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3184 GTM_gtm_cls0.MCS0_MEM[3184].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3185 GTM_gtm_cls0.MCS0_MEM[3185].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3186 GTM_gtm_cls0.MCS0_MEM[3186].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3187 GTM_gtm_cls0.MCS0_MEM[3187].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3188 GTM_gtm_cls0.MCS0_MEM[3188].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3189 GTM_gtm_cls0.MCS0_MEM[3189].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3190 GTM_gtm_cls0.MCS0_MEM[3190].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3191 GTM_gtm_cls0.MCS0_MEM[3191].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3192 GTM_gtm_cls0.MCS0_MEM[3192].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3193 GTM_gtm_cls0.MCS0_MEM[3193].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3194 GTM_gtm_cls0.MCS0_MEM[3194].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3195 GTM_gtm_cls0.MCS0_MEM[3195].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3196 GTM_gtm_cls0.MCS0_MEM[3196].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3197 GTM_gtm_cls0.MCS0_MEM[3197].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3198 GTM_gtm_cls0.MCS0_MEM[3198].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3199 GTM_gtm_cls0.MCS0_MEM[3199].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3200 GTM_gtm_cls0.MCS0_MEM[3200].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3201 GTM_gtm_cls0.MCS0_MEM[3201].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3202 GTM_gtm_cls0.MCS0_MEM[3202].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3203 GTM_gtm_cls0.MCS0_MEM[3203].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3204 GTM_gtm_cls0.MCS0_MEM[3204].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3205 GTM_gtm_cls0.MCS0_MEM[3205].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3206 GTM_gtm_cls0.MCS0_MEM[3206].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3207 GTM_gtm_cls0.MCS0_MEM[3207].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3208 GTM_gtm_cls0.MCS0_MEM[3208].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3209 GTM_gtm_cls0.MCS0_MEM[3209].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3210 GTM_gtm_cls0.MCS0_MEM[3210].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3211 GTM_gtm_cls0.MCS0_MEM[3211].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3212 GTM_gtm_cls0.MCS0_MEM[3212].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3213 GTM_gtm_cls0.MCS0_MEM[3213].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3214 GTM_gtm_cls0.MCS0_MEM[3214].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3215 GTM_gtm_cls0.MCS0_MEM[3215].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3216 GTM_gtm_cls0.MCS0_MEM[3216].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3217 GTM_gtm_cls0.MCS0_MEM[3217].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3218 GTM_gtm_cls0.MCS0_MEM[3218].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3219 GTM_gtm_cls0.MCS0_MEM[3219].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3220 GTM_gtm_cls0.MCS0_MEM[3220].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3221 GTM_gtm_cls0.MCS0_MEM[3221].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3222 GTM_gtm_cls0.MCS0_MEM[3222].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3223 GTM_gtm_cls0.MCS0_MEM[3223].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3224 GTM_gtm_cls0.MCS0_MEM[3224].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3225 GTM_gtm_cls0.MCS0_MEM[3225].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3226 GTM_gtm_cls0.MCS0_MEM[3226].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3227 GTM_gtm_cls0.MCS0_MEM[3227].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3228 GTM_gtm_cls0.MCS0_MEM[3228].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3229 GTM_gtm_cls0.MCS0_MEM[3229].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3230 GTM_gtm_cls0.MCS0_MEM[3230].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3231 GTM_gtm_cls0.MCS0_MEM[3231].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3232 GTM_gtm_cls0.MCS0_MEM[3232].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3233 GTM_gtm_cls0.MCS0_MEM[3233].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3234 GTM_gtm_cls0.MCS0_MEM[3234].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3235 GTM_gtm_cls0.MCS0_MEM[3235].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3236 GTM_gtm_cls0.MCS0_MEM[3236].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3237 GTM_gtm_cls0.MCS0_MEM[3237].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3238 GTM_gtm_cls0.MCS0_MEM[3238].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3239 GTM_gtm_cls0.MCS0_MEM[3239].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3240 GTM_gtm_cls0.MCS0_MEM[3240].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3241 GTM_gtm_cls0.MCS0_MEM[3241].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3242 GTM_gtm_cls0.MCS0_MEM[3242].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3243 GTM_gtm_cls0.MCS0_MEM[3243].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3244 GTM_gtm_cls0.MCS0_MEM[3244].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3245 GTM_gtm_cls0.MCS0_MEM[3245].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3246 GTM_gtm_cls0.MCS0_MEM[3246].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3247 GTM_gtm_cls0.MCS0_MEM[3247].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3248 GTM_gtm_cls0.MCS0_MEM[3248].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3249 GTM_gtm_cls0.MCS0_MEM[3249].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3250 GTM_gtm_cls0.MCS0_MEM[3250].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3251 GTM_gtm_cls0.MCS0_MEM[3251].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3252 GTM_gtm_cls0.MCS0_MEM[3252].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3253 GTM_gtm_cls0.MCS0_MEM[3253].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3254 GTM_gtm_cls0.MCS0_MEM[3254].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3255 GTM_gtm_cls0.MCS0_MEM[3255].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3256 GTM_gtm_cls0.MCS0_MEM[3256].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3257 GTM_gtm_cls0.MCS0_MEM[3257].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3258 GTM_gtm_cls0.MCS0_MEM[3258].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3259 GTM_gtm_cls0.MCS0_MEM[3259].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3260 GTM_gtm_cls0.MCS0_MEM[3260].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3261 GTM_gtm_cls0.MCS0_MEM[3261].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3262 GTM_gtm_cls0.MCS0_MEM[3262].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3263 GTM_gtm_cls0.MCS0_MEM[3263].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3264 GTM_gtm_cls0.MCS0_MEM[3264].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3265 GTM_gtm_cls0.MCS0_MEM[3265].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3266 GTM_gtm_cls0.MCS0_MEM[3266].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3267 GTM_gtm_cls0.MCS0_MEM[3267].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3268 GTM_gtm_cls0.MCS0_MEM[3268].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3269 GTM_gtm_cls0.MCS0_MEM[3269].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3270 GTM_gtm_cls0.MCS0_MEM[3270].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3271 GTM_gtm_cls0.MCS0_MEM[3271].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3272 GTM_gtm_cls0.MCS0_MEM[3272].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3273 GTM_gtm_cls0.MCS0_MEM[3273].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3274 GTM_gtm_cls0.MCS0_MEM[3274].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3275 GTM_gtm_cls0.MCS0_MEM[3275].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3276 GTM_gtm_cls0.MCS0_MEM[3276].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3277 GTM_gtm_cls0.MCS0_MEM[3277].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3278 GTM_gtm_cls0.MCS0_MEM[3278].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3279 GTM_gtm_cls0.MCS0_MEM[3279].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3280 GTM_gtm_cls0.MCS0_MEM[3280].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3281 GTM_gtm_cls0.MCS0_MEM[3281].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3282 GTM_gtm_cls0.MCS0_MEM[3282].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3283 GTM_gtm_cls0.MCS0_MEM[3283].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3284 GTM_gtm_cls0.MCS0_MEM[3284].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3285 GTM_gtm_cls0.MCS0_MEM[3285].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3286 GTM_gtm_cls0.MCS0_MEM[3286].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3287 GTM_gtm_cls0.MCS0_MEM[3287].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3288 GTM_gtm_cls0.MCS0_MEM[3288].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3289 GTM_gtm_cls0.MCS0_MEM[3289].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3290 GTM_gtm_cls0.MCS0_MEM[3290].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3291 GTM_gtm_cls0.MCS0_MEM[3291].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3292 GTM_gtm_cls0.MCS0_MEM[3292].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3293 GTM_gtm_cls0.MCS0_MEM[3293].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3294 GTM_gtm_cls0.MCS0_MEM[3294].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3295 GTM_gtm_cls0.MCS0_MEM[3295].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3296 GTM_gtm_cls0.MCS0_MEM[3296].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3297 GTM_gtm_cls0.MCS0_MEM[3297].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3298 GTM_gtm_cls0.MCS0_MEM[3298].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3299 GTM_gtm_cls0.MCS0_MEM[3299].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3300 GTM_gtm_cls0.MCS0_MEM[3300].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3301 GTM_gtm_cls0.MCS0_MEM[3301].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3302 GTM_gtm_cls0.MCS0_MEM[3302].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3303 GTM_gtm_cls0.MCS0_MEM[3303].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3304 GTM_gtm_cls0.MCS0_MEM[3304].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3305 GTM_gtm_cls0.MCS0_MEM[3305].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3306 GTM_gtm_cls0.MCS0_MEM[3306].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3307 GTM_gtm_cls0.MCS0_MEM[3307].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3308 GTM_gtm_cls0.MCS0_MEM[3308].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3309 GTM_gtm_cls0.MCS0_MEM[3309].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3310 GTM_gtm_cls0.MCS0_MEM[3310].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3311 GTM_gtm_cls0.MCS0_MEM[3311].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3312 GTM_gtm_cls0.MCS0_MEM[3312].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3313 GTM_gtm_cls0.MCS0_MEM[3313].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3314 GTM_gtm_cls0.MCS0_MEM[3314].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3315 GTM_gtm_cls0.MCS0_MEM[3315].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3316 GTM_gtm_cls0.MCS0_MEM[3316].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3317 GTM_gtm_cls0.MCS0_MEM[3317].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3318 GTM_gtm_cls0.MCS0_MEM[3318].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3319 GTM_gtm_cls0.MCS0_MEM[3319].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3320 GTM_gtm_cls0.MCS0_MEM[3320].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3321 GTM_gtm_cls0.MCS0_MEM[3321].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3322 GTM_gtm_cls0.MCS0_MEM[3322].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3323 GTM_gtm_cls0.MCS0_MEM[3323].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3324 GTM_gtm_cls0.MCS0_MEM[3324].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3325 GTM_gtm_cls0.MCS0_MEM[3325].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3326 GTM_gtm_cls0.MCS0_MEM[3326].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3327 GTM_gtm_cls0.MCS0_MEM[3327].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3328 GTM_gtm_cls0.MCS0_MEM[3328].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3329 GTM_gtm_cls0.MCS0_MEM[3329].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3330 GTM_gtm_cls0.MCS0_MEM[3330].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3331 GTM_gtm_cls0.MCS0_MEM[3331].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3332 GTM_gtm_cls0.MCS0_MEM[3332].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3333 GTM_gtm_cls0.MCS0_MEM[3333].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3334 GTM_gtm_cls0.MCS0_MEM[3334].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3335 GTM_gtm_cls0.MCS0_MEM[3335].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3336 GTM_gtm_cls0.MCS0_MEM[3336].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3337 GTM_gtm_cls0.MCS0_MEM[3337].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3338 GTM_gtm_cls0.MCS0_MEM[3338].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3339 GTM_gtm_cls0.MCS0_MEM[3339].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3340 GTM_gtm_cls0.MCS0_MEM[3340].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3341 GTM_gtm_cls0.MCS0_MEM[3341].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3342 GTM_gtm_cls0.MCS0_MEM[3342].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3343 GTM_gtm_cls0.MCS0_MEM[3343].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3344 GTM_gtm_cls0.MCS0_MEM[3344].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3345 GTM_gtm_cls0.MCS0_MEM[3345].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3346 GTM_gtm_cls0.MCS0_MEM[3346].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3347 GTM_gtm_cls0.MCS0_MEM[3347].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3348 GTM_gtm_cls0.MCS0_MEM[3348].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3349 GTM_gtm_cls0.MCS0_MEM[3349].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3350 GTM_gtm_cls0.MCS0_MEM[3350].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3351 GTM_gtm_cls0.MCS0_MEM[3351].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3352 GTM_gtm_cls0.MCS0_MEM[3352].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3353 GTM_gtm_cls0.MCS0_MEM[3353].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3354 GTM_gtm_cls0.MCS0_MEM[3354].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3355 GTM_gtm_cls0.MCS0_MEM[3355].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3356 GTM_gtm_cls0.MCS0_MEM[3356].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3357 GTM_gtm_cls0.MCS0_MEM[3357].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3358 GTM_gtm_cls0.MCS0_MEM[3358].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3359 GTM_gtm_cls0.MCS0_MEM[3359].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3360 GTM_gtm_cls0.MCS0_MEM[3360].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3361 GTM_gtm_cls0.MCS0_MEM[3361].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3362 GTM_gtm_cls0.MCS0_MEM[3362].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3363 GTM_gtm_cls0.MCS0_MEM[3363].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3364 GTM_gtm_cls0.MCS0_MEM[3364].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3365 GTM_gtm_cls0.MCS0_MEM[3365].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3366 GTM_gtm_cls0.MCS0_MEM[3366].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3367 GTM_gtm_cls0.MCS0_MEM[3367].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3368 GTM_gtm_cls0.MCS0_MEM[3368].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3369 GTM_gtm_cls0.MCS0_MEM[3369].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3370 GTM_gtm_cls0.MCS0_MEM[3370].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3371 GTM_gtm_cls0.MCS0_MEM[3371].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3372 GTM_gtm_cls0.MCS0_MEM[3372].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3373 GTM_gtm_cls0.MCS0_MEM[3373].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3374 GTM_gtm_cls0.MCS0_MEM[3374].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3375 GTM_gtm_cls0.MCS0_MEM[3375].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3376 GTM_gtm_cls0.MCS0_MEM[3376].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3377 GTM_gtm_cls0.MCS0_MEM[3377].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3378 GTM_gtm_cls0.MCS0_MEM[3378].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3379 GTM_gtm_cls0.MCS0_MEM[3379].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3380 GTM_gtm_cls0.MCS0_MEM[3380].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3381 GTM_gtm_cls0.MCS0_MEM[3381].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3382 GTM_gtm_cls0.MCS0_MEM[3382].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3383 GTM_gtm_cls0.MCS0_MEM[3383].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3384 GTM_gtm_cls0.MCS0_MEM[3384].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3385 GTM_gtm_cls0.MCS0_MEM[3385].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3386 GTM_gtm_cls0.MCS0_MEM[3386].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3387 GTM_gtm_cls0.MCS0_MEM[3387].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3388 GTM_gtm_cls0.MCS0_MEM[3388].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3389 GTM_gtm_cls0.MCS0_MEM[3389].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3390 GTM_gtm_cls0.MCS0_MEM[3390].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3391 GTM_gtm_cls0.MCS0_MEM[3391].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3392 GTM_gtm_cls0.MCS0_MEM[3392].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3393 GTM_gtm_cls0.MCS0_MEM[3393].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3394 GTM_gtm_cls0.MCS0_MEM[3394].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3395 GTM_gtm_cls0.MCS0_MEM[3395].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3396 GTM_gtm_cls0.MCS0_MEM[3396].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3397 GTM_gtm_cls0.MCS0_MEM[3397].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3398 GTM_gtm_cls0.MCS0_MEM[3398].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3399 GTM_gtm_cls0.MCS0_MEM[3399].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3400 GTM_gtm_cls0.MCS0_MEM[3400].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3401 GTM_gtm_cls0.MCS0_MEM[3401].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3402 GTM_gtm_cls0.MCS0_MEM[3402].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3403 GTM_gtm_cls0.MCS0_MEM[3403].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3404 GTM_gtm_cls0.MCS0_MEM[3404].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3405 GTM_gtm_cls0.MCS0_MEM[3405].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3406 GTM_gtm_cls0.MCS0_MEM[3406].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3407 GTM_gtm_cls0.MCS0_MEM[3407].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3408 GTM_gtm_cls0.MCS0_MEM[3408].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3409 GTM_gtm_cls0.MCS0_MEM[3409].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3410 GTM_gtm_cls0.MCS0_MEM[3410].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3411 GTM_gtm_cls0.MCS0_MEM[3411].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3412 GTM_gtm_cls0.MCS0_MEM[3412].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3413 GTM_gtm_cls0.MCS0_MEM[3413].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3414 GTM_gtm_cls0.MCS0_MEM[3414].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3415 GTM_gtm_cls0.MCS0_MEM[3415].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3416 GTM_gtm_cls0.MCS0_MEM[3416].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3417 GTM_gtm_cls0.MCS0_MEM[3417].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3418 GTM_gtm_cls0.MCS0_MEM[3418].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3419 GTM_gtm_cls0.MCS0_MEM[3419].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3420 GTM_gtm_cls0.MCS0_MEM[3420].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3421 GTM_gtm_cls0.MCS0_MEM[3421].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3422 GTM_gtm_cls0.MCS0_MEM[3422].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3423 GTM_gtm_cls0.MCS0_MEM[3423].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3424 GTM_gtm_cls0.MCS0_MEM[3424].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3425 GTM_gtm_cls0.MCS0_MEM[3425].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3426 GTM_gtm_cls0.MCS0_MEM[3426].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3427 GTM_gtm_cls0.MCS0_MEM[3427].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3428 GTM_gtm_cls0.MCS0_MEM[3428].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3429 GTM_gtm_cls0.MCS0_MEM[3429].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3430 GTM_gtm_cls0.MCS0_MEM[3430].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3431 GTM_gtm_cls0.MCS0_MEM[3431].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3432 GTM_gtm_cls0.MCS0_MEM[3432].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3433 GTM_gtm_cls0.MCS0_MEM[3433].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3434 GTM_gtm_cls0.MCS0_MEM[3434].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3435 GTM_gtm_cls0.MCS0_MEM[3435].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3436 GTM_gtm_cls0.MCS0_MEM[3436].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3437 GTM_gtm_cls0.MCS0_MEM[3437].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3438 GTM_gtm_cls0.MCS0_MEM[3438].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3439 GTM_gtm_cls0.MCS0_MEM[3439].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3440 GTM_gtm_cls0.MCS0_MEM[3440].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3441 GTM_gtm_cls0.MCS0_MEM[3441].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3442 GTM_gtm_cls0.MCS0_MEM[3442].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3443 GTM_gtm_cls0.MCS0_MEM[3443].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3444 GTM_gtm_cls0.MCS0_MEM[3444].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3445 GTM_gtm_cls0.MCS0_MEM[3445].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3446 GTM_gtm_cls0.MCS0_MEM[3446].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3447 GTM_gtm_cls0.MCS0_MEM[3447].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3448 GTM_gtm_cls0.MCS0_MEM[3448].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3449 GTM_gtm_cls0.MCS0_MEM[3449].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3450 GTM_gtm_cls0.MCS0_MEM[3450].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3451 GTM_gtm_cls0.MCS0_MEM[3451].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3452 GTM_gtm_cls0.MCS0_MEM[3452].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3453 GTM_gtm_cls0.MCS0_MEM[3453].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3454 GTM_gtm_cls0.MCS0_MEM[3454].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3455 GTM_gtm_cls0.MCS0_MEM[3455].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3456 GTM_gtm_cls0.MCS0_MEM[3456].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3457 GTM_gtm_cls0.MCS0_MEM[3457].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3458 GTM_gtm_cls0.MCS0_MEM[3458].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3459 GTM_gtm_cls0.MCS0_MEM[3459].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3460 GTM_gtm_cls0.MCS0_MEM[3460].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3461 GTM_gtm_cls0.MCS0_MEM[3461].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3462 GTM_gtm_cls0.MCS0_MEM[3462].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3463 GTM_gtm_cls0.MCS0_MEM[3463].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3464 GTM_gtm_cls0.MCS0_MEM[3464].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3465 GTM_gtm_cls0.MCS0_MEM[3465].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3466 GTM_gtm_cls0.MCS0_MEM[3466].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3467 GTM_gtm_cls0.MCS0_MEM[3467].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3468 GTM_gtm_cls0.MCS0_MEM[3468].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3469 GTM_gtm_cls0.MCS0_MEM[3469].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3470 GTM_gtm_cls0.MCS0_MEM[3470].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3471 GTM_gtm_cls0.MCS0_MEM[3471].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3472 GTM_gtm_cls0.MCS0_MEM[3472].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3473 GTM_gtm_cls0.MCS0_MEM[3473].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3474 GTM_gtm_cls0.MCS0_MEM[3474].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3475 GTM_gtm_cls0.MCS0_MEM[3475].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3476 GTM_gtm_cls0.MCS0_MEM[3476].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3477 GTM_gtm_cls0.MCS0_MEM[3477].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3478 GTM_gtm_cls0.MCS0_MEM[3478].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3479 GTM_gtm_cls0.MCS0_MEM[3479].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3480 GTM_gtm_cls0.MCS0_MEM[3480].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3481 GTM_gtm_cls0.MCS0_MEM[3481].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3482 GTM_gtm_cls0.MCS0_MEM[3482].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3483 GTM_gtm_cls0.MCS0_MEM[3483].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3484 GTM_gtm_cls0.MCS0_MEM[3484].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3485 GTM_gtm_cls0.MCS0_MEM[3485].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3486 GTM_gtm_cls0.MCS0_MEM[3486].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3487 GTM_gtm_cls0.MCS0_MEM[3487].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3488 GTM_gtm_cls0.MCS0_MEM[3488].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3489 GTM_gtm_cls0.MCS0_MEM[3489].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3490 GTM_gtm_cls0.MCS0_MEM[3490].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3491 GTM_gtm_cls0.MCS0_MEM[3491].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3492 GTM_gtm_cls0.MCS0_MEM[3492].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3493 GTM_gtm_cls0.MCS0_MEM[3493].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3494 GTM_gtm_cls0.MCS0_MEM[3494].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3495 GTM_gtm_cls0.MCS0_MEM[3495].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3496 GTM_gtm_cls0.MCS0_MEM[3496].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3497 GTM_gtm_cls0.MCS0_MEM[3497].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3498 GTM_gtm_cls0.MCS0_MEM[3498].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3499 GTM_gtm_cls0.MCS0_MEM[3499].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3500 GTM_gtm_cls0.MCS0_MEM[3500].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3501 GTM_gtm_cls0.MCS0_MEM[3501].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3502 GTM_gtm_cls0.MCS0_MEM[3502].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3503 GTM_gtm_cls0.MCS0_MEM[3503].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3504 GTM_gtm_cls0.MCS0_MEM[3504].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3505 GTM_gtm_cls0.MCS0_MEM[3505].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3506 GTM_gtm_cls0.MCS0_MEM[3506].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3507 GTM_gtm_cls0.MCS0_MEM[3507].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3508 GTM_gtm_cls0.MCS0_MEM[3508].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3509 GTM_gtm_cls0.MCS0_MEM[3509].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3510 GTM_gtm_cls0.MCS0_MEM[3510].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3511 GTM_gtm_cls0.MCS0_MEM[3511].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3512 GTM_gtm_cls0.MCS0_MEM[3512].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3513 GTM_gtm_cls0.MCS0_MEM[3513].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3514 GTM_gtm_cls0.MCS0_MEM[3514].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3515 GTM_gtm_cls0.MCS0_MEM[3515].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3516 GTM_gtm_cls0.MCS0_MEM[3516].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3517 GTM_gtm_cls0.MCS0_MEM[3517].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3518 GTM_gtm_cls0.MCS0_MEM[3518].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3519 GTM_gtm_cls0.MCS0_MEM[3519].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3520 GTM_gtm_cls0.MCS0_MEM[3520].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3521 GTM_gtm_cls0.MCS0_MEM[3521].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3522 GTM_gtm_cls0.MCS0_MEM[3522].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3523 GTM_gtm_cls0.MCS0_MEM[3523].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3524 GTM_gtm_cls0.MCS0_MEM[3524].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3525 GTM_gtm_cls0.MCS0_MEM[3525].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3526 GTM_gtm_cls0.MCS0_MEM[3526].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3527 GTM_gtm_cls0.MCS0_MEM[3527].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3528 GTM_gtm_cls0.MCS0_MEM[3528].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3529 GTM_gtm_cls0.MCS0_MEM[3529].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3530 GTM_gtm_cls0.MCS0_MEM[3530].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3531 GTM_gtm_cls0.MCS0_MEM[3531].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3532 GTM_gtm_cls0.MCS0_MEM[3532].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3533 GTM_gtm_cls0.MCS0_MEM[3533].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3534 GTM_gtm_cls0.MCS0_MEM[3534].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3535 GTM_gtm_cls0.MCS0_MEM[3535].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3536 GTM_gtm_cls0.MCS0_MEM[3536].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3537 GTM_gtm_cls0.MCS0_MEM[3537].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3538 GTM_gtm_cls0.MCS0_MEM[3538].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3539 GTM_gtm_cls0.MCS0_MEM[3539].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3540 GTM_gtm_cls0.MCS0_MEM[3540].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3541 GTM_gtm_cls0.MCS0_MEM[3541].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3542 GTM_gtm_cls0.MCS0_MEM[3542].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3543 GTM_gtm_cls0.MCS0_MEM[3543].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3544 GTM_gtm_cls0.MCS0_MEM[3544].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3545 GTM_gtm_cls0.MCS0_MEM[3545].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3546 GTM_gtm_cls0.MCS0_MEM[3546].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3547 GTM_gtm_cls0.MCS0_MEM[3547].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3548 GTM_gtm_cls0.MCS0_MEM[3548].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3549 GTM_gtm_cls0.MCS0_MEM[3549].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3550 GTM_gtm_cls0.MCS0_MEM[3550].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3551 GTM_gtm_cls0.MCS0_MEM[3551].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3552 GTM_gtm_cls0.MCS0_MEM[3552].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3553 GTM_gtm_cls0.MCS0_MEM[3553].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3554 GTM_gtm_cls0.MCS0_MEM[3554].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3555 GTM_gtm_cls0.MCS0_MEM[3555].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3556 GTM_gtm_cls0.MCS0_MEM[3556].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3557 GTM_gtm_cls0.MCS0_MEM[3557].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3558 GTM_gtm_cls0.MCS0_MEM[3558].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3559 GTM_gtm_cls0.MCS0_MEM[3559].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3560 GTM_gtm_cls0.MCS0_MEM[3560].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3561 GTM_gtm_cls0.MCS0_MEM[3561].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3562 GTM_gtm_cls0.MCS0_MEM[3562].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3563 GTM_gtm_cls0.MCS0_MEM[3563].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3564 GTM_gtm_cls0.MCS0_MEM[3564].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3565 GTM_gtm_cls0.MCS0_MEM[3565].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3566 GTM_gtm_cls0.MCS0_MEM[3566].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3567 GTM_gtm_cls0.MCS0_MEM[3567].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3568 GTM_gtm_cls0.MCS0_MEM[3568].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3569 GTM_gtm_cls0.MCS0_MEM[3569].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3570 GTM_gtm_cls0.MCS0_MEM[3570].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3571 GTM_gtm_cls0.MCS0_MEM[3571].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3572 GTM_gtm_cls0.MCS0_MEM[3572].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3573 GTM_gtm_cls0.MCS0_MEM[3573].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3574 GTM_gtm_cls0.MCS0_MEM[3574].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3575 GTM_gtm_cls0.MCS0_MEM[3575].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3576 GTM_gtm_cls0.MCS0_MEM[3576].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3577 GTM_gtm_cls0.MCS0_MEM[3577].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3578 GTM_gtm_cls0.MCS0_MEM[3578].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3579 GTM_gtm_cls0.MCS0_MEM[3579].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3580 GTM_gtm_cls0.MCS0_MEM[3580].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3581 GTM_gtm_cls0.MCS0_MEM[3581].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3582 GTM_gtm_cls0.MCS0_MEM[3582].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3583 GTM_gtm_cls0.MCS0_MEM[3583].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3584 GTM_gtm_cls0.MCS0_MEM[3584].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3585 GTM_gtm_cls0.MCS0_MEM[3585].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3586 GTM_gtm_cls0.MCS0_MEM[3586].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3587 GTM_gtm_cls0.MCS0_MEM[3587].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3588 GTM_gtm_cls0.MCS0_MEM[3588].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3589 GTM_gtm_cls0.MCS0_MEM[3589].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3590 GTM_gtm_cls0.MCS0_MEM[3590].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3591 GTM_gtm_cls0.MCS0_MEM[3591].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3592 GTM_gtm_cls0.MCS0_MEM[3592].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3593 GTM_gtm_cls0.MCS0_MEM[3593].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3594 GTM_gtm_cls0.MCS0_MEM[3594].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3595 GTM_gtm_cls0.MCS0_MEM[3595].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3596 GTM_gtm_cls0.MCS0_MEM[3596].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3597 GTM_gtm_cls0.MCS0_MEM[3597].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3598 GTM_gtm_cls0.MCS0_MEM[3598].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3599 GTM_gtm_cls0.MCS0_MEM[3599].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3600 GTM_gtm_cls0.MCS0_MEM[3600].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3601 GTM_gtm_cls0.MCS0_MEM[3601].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3602 GTM_gtm_cls0.MCS0_MEM[3602].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3603 GTM_gtm_cls0.MCS0_MEM[3603].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3604 GTM_gtm_cls0.MCS0_MEM[3604].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3605 GTM_gtm_cls0.MCS0_MEM[3605].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3606 GTM_gtm_cls0.MCS0_MEM[3606].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3607 GTM_gtm_cls0.MCS0_MEM[3607].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3608 GTM_gtm_cls0.MCS0_MEM[3608].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3609 GTM_gtm_cls0.MCS0_MEM[3609].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3610 GTM_gtm_cls0.MCS0_MEM[3610].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3611 GTM_gtm_cls0.MCS0_MEM[3611].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3612 GTM_gtm_cls0.MCS0_MEM[3612].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3613 GTM_gtm_cls0.MCS0_MEM[3613].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3614 GTM_gtm_cls0.MCS0_MEM[3614].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3615 GTM_gtm_cls0.MCS0_MEM[3615].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3616 GTM_gtm_cls0.MCS0_MEM[3616].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3617 GTM_gtm_cls0.MCS0_MEM[3617].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3618 GTM_gtm_cls0.MCS0_MEM[3618].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3619 GTM_gtm_cls0.MCS0_MEM[3619].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3620 GTM_gtm_cls0.MCS0_MEM[3620].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3621 GTM_gtm_cls0.MCS0_MEM[3621].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3622 GTM_gtm_cls0.MCS0_MEM[3622].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3623 GTM_gtm_cls0.MCS0_MEM[3623].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3624 GTM_gtm_cls0.MCS0_MEM[3624].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3625 GTM_gtm_cls0.MCS0_MEM[3625].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3626 GTM_gtm_cls0.MCS0_MEM[3626].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3627 GTM_gtm_cls0.MCS0_MEM[3627].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3628 GTM_gtm_cls0.MCS0_MEM[3628].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3629 GTM_gtm_cls0.MCS0_MEM[3629].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3630 GTM_gtm_cls0.MCS0_MEM[3630].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3631 GTM_gtm_cls0.MCS0_MEM[3631].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3632 GTM_gtm_cls0.MCS0_MEM[3632].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3633 GTM_gtm_cls0.MCS0_MEM[3633].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3634 GTM_gtm_cls0.MCS0_MEM[3634].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3635 GTM_gtm_cls0.MCS0_MEM[3635].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3636 GTM_gtm_cls0.MCS0_MEM[3636].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3637 GTM_gtm_cls0.MCS0_MEM[3637].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3638 GTM_gtm_cls0.MCS0_MEM[3638].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3639 GTM_gtm_cls0.MCS0_MEM[3639].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3640 GTM_gtm_cls0.MCS0_MEM[3640].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3641 GTM_gtm_cls0.MCS0_MEM[3641].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3642 GTM_gtm_cls0.MCS0_MEM[3642].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3643 GTM_gtm_cls0.MCS0_MEM[3643].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3644 GTM_gtm_cls0.MCS0_MEM[3644].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3645 GTM_gtm_cls0.MCS0_MEM[3645].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3646 GTM_gtm_cls0.MCS0_MEM[3646].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3647 GTM_gtm_cls0.MCS0_MEM[3647].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3648 GTM_gtm_cls0.MCS0_MEM[3648].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3649 GTM_gtm_cls0.MCS0_MEM[3649].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3650 GTM_gtm_cls0.MCS0_MEM[3650].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3651 GTM_gtm_cls0.MCS0_MEM[3651].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3652 GTM_gtm_cls0.MCS0_MEM[3652].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3653 GTM_gtm_cls0.MCS0_MEM[3653].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3654 GTM_gtm_cls0.MCS0_MEM[3654].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3655 GTM_gtm_cls0.MCS0_MEM[3655].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3656 GTM_gtm_cls0.MCS0_MEM[3656].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3657 GTM_gtm_cls0.MCS0_MEM[3657].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3658 GTM_gtm_cls0.MCS0_MEM[3658].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3659 GTM_gtm_cls0.MCS0_MEM[3659].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3660 GTM_gtm_cls0.MCS0_MEM[3660].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3661 GTM_gtm_cls0.MCS0_MEM[3661].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3662 GTM_gtm_cls0.MCS0_MEM[3662].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3663 GTM_gtm_cls0.MCS0_MEM[3663].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3664 GTM_gtm_cls0.MCS0_MEM[3664].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3665 GTM_gtm_cls0.MCS0_MEM[3665].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3666 GTM_gtm_cls0.MCS0_MEM[3666].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3667 GTM_gtm_cls0.MCS0_MEM[3667].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3668 GTM_gtm_cls0.MCS0_MEM[3668].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3669 GTM_gtm_cls0.MCS0_MEM[3669].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3670 GTM_gtm_cls0.MCS0_MEM[3670].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3671 GTM_gtm_cls0.MCS0_MEM[3671].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3672 GTM_gtm_cls0.MCS0_MEM[3672].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3673 GTM_gtm_cls0.MCS0_MEM[3673].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3674 GTM_gtm_cls0.MCS0_MEM[3674].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3675 GTM_gtm_cls0.MCS0_MEM[3675].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3676 GTM_gtm_cls0.MCS0_MEM[3676].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3677 GTM_gtm_cls0.MCS0_MEM[3677].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3678 GTM_gtm_cls0.MCS0_MEM[3678].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3679 GTM_gtm_cls0.MCS0_MEM[3679].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3680 GTM_gtm_cls0.MCS0_MEM[3680].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3681 GTM_gtm_cls0.MCS0_MEM[3681].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3682 GTM_gtm_cls0.MCS0_MEM[3682].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3683 GTM_gtm_cls0.MCS0_MEM[3683].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3684 GTM_gtm_cls0.MCS0_MEM[3684].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3685 GTM_gtm_cls0.MCS0_MEM[3685].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3686 GTM_gtm_cls0.MCS0_MEM[3686].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3687 GTM_gtm_cls0.MCS0_MEM[3687].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3688 GTM_gtm_cls0.MCS0_MEM[3688].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3689 GTM_gtm_cls0.MCS0_MEM[3689].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3690 GTM_gtm_cls0.MCS0_MEM[3690].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3691 GTM_gtm_cls0.MCS0_MEM[3691].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3692 GTM_gtm_cls0.MCS0_MEM[3692].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3693 GTM_gtm_cls0.MCS0_MEM[3693].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3694 GTM_gtm_cls0.MCS0_MEM[3694].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3695 GTM_gtm_cls0.MCS0_MEM[3695].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3696 GTM_gtm_cls0.MCS0_MEM[3696].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3697 GTM_gtm_cls0.MCS0_MEM[3697].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3698 GTM_gtm_cls0.MCS0_MEM[3698].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3699 GTM_gtm_cls0.MCS0_MEM[3699].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3700 GTM_gtm_cls0.MCS0_MEM[3700].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3701 GTM_gtm_cls0.MCS0_MEM[3701].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3702 GTM_gtm_cls0.MCS0_MEM[3702].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3703 GTM_gtm_cls0.MCS0_MEM[3703].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3704 GTM_gtm_cls0.MCS0_MEM[3704].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3705 GTM_gtm_cls0.MCS0_MEM[3705].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3706 GTM_gtm_cls0.MCS0_MEM[3706].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3707 GTM_gtm_cls0.MCS0_MEM[3707].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3708 GTM_gtm_cls0.MCS0_MEM[3708].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3709 GTM_gtm_cls0.MCS0_MEM[3709].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3710 GTM_gtm_cls0.MCS0_MEM[3710].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3711 GTM_gtm_cls0.MCS0_MEM[3711].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3712 GTM_gtm_cls0.MCS0_MEM[3712].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3713 GTM_gtm_cls0.MCS0_MEM[3713].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3714 GTM_gtm_cls0.MCS0_MEM[3714].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3715 GTM_gtm_cls0.MCS0_MEM[3715].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3716 GTM_gtm_cls0.MCS0_MEM[3716].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3717 GTM_gtm_cls0.MCS0_MEM[3717].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3718 GTM_gtm_cls0.MCS0_MEM[3718].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3719 GTM_gtm_cls0.MCS0_MEM[3719].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3720 GTM_gtm_cls0.MCS0_MEM[3720].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3721 GTM_gtm_cls0.MCS0_MEM[3721].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3722 GTM_gtm_cls0.MCS0_MEM[3722].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3723 GTM_gtm_cls0.MCS0_MEM[3723].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3724 GTM_gtm_cls0.MCS0_MEM[3724].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3725 GTM_gtm_cls0.MCS0_MEM[3725].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3726 GTM_gtm_cls0.MCS0_MEM[3726].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3727 GTM_gtm_cls0.MCS0_MEM[3727].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3728 GTM_gtm_cls0.MCS0_MEM[3728].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3729 GTM_gtm_cls0.MCS0_MEM[3729].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3730 GTM_gtm_cls0.MCS0_MEM[3730].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3731 GTM_gtm_cls0.MCS0_MEM[3731].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3732 GTM_gtm_cls0.MCS0_MEM[3732].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3733 GTM_gtm_cls0.MCS0_MEM[3733].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3734 GTM_gtm_cls0.MCS0_MEM[3734].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3735 GTM_gtm_cls0.MCS0_MEM[3735].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3736 GTM_gtm_cls0.MCS0_MEM[3736].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3737 GTM_gtm_cls0.MCS0_MEM[3737].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3738 GTM_gtm_cls0.MCS0_MEM[3738].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3739 GTM_gtm_cls0.MCS0_MEM[3739].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3740 GTM_gtm_cls0.MCS0_MEM[3740].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3741 GTM_gtm_cls0.MCS0_MEM[3741].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3742 GTM_gtm_cls0.MCS0_MEM[3742].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3743 GTM_gtm_cls0.MCS0_MEM[3743].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3744 GTM_gtm_cls0.MCS0_MEM[3744].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3745 GTM_gtm_cls0.MCS0_MEM[3745].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3746 GTM_gtm_cls0.MCS0_MEM[3746].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3747 GTM_gtm_cls0.MCS0_MEM[3747].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3748 GTM_gtm_cls0.MCS0_MEM[3748].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3749 GTM_gtm_cls0.MCS0_MEM[3749].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3750 GTM_gtm_cls0.MCS0_MEM[3750].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3751 GTM_gtm_cls0.MCS0_MEM[3751].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3752 GTM_gtm_cls0.MCS0_MEM[3752].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3753 GTM_gtm_cls0.MCS0_MEM[3753].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3754 GTM_gtm_cls0.MCS0_MEM[3754].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3755 GTM_gtm_cls0.MCS0_MEM[3755].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3756 GTM_gtm_cls0.MCS0_MEM[3756].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3757 GTM_gtm_cls0.MCS0_MEM[3757].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3758 GTM_gtm_cls0.MCS0_MEM[3758].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3759 GTM_gtm_cls0.MCS0_MEM[3759].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3760 GTM_gtm_cls0.MCS0_MEM[3760].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3761 GTM_gtm_cls0.MCS0_MEM[3761].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3762 GTM_gtm_cls0.MCS0_MEM[3762].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3763 GTM_gtm_cls0.MCS0_MEM[3763].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3764 GTM_gtm_cls0.MCS0_MEM[3764].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3765 GTM_gtm_cls0.MCS0_MEM[3765].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3766 GTM_gtm_cls0.MCS0_MEM[3766].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3767 GTM_gtm_cls0.MCS0_MEM[3767].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3768 GTM_gtm_cls0.MCS0_MEM[3768].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3769 GTM_gtm_cls0.MCS0_MEM[3769].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3770 GTM_gtm_cls0.MCS0_MEM[3770].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3771 GTM_gtm_cls0.MCS0_MEM[3771].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3772 GTM_gtm_cls0.MCS0_MEM[3772].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3773 GTM_gtm_cls0.MCS0_MEM[3773].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3774 GTM_gtm_cls0.MCS0_MEM[3774].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3775 GTM_gtm_cls0.MCS0_MEM[3775].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3776 GTM_gtm_cls0.MCS0_MEM[3776].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3777 GTM_gtm_cls0.MCS0_MEM[3777].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3778 GTM_gtm_cls0.MCS0_MEM[3778].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3779 GTM_gtm_cls0.MCS0_MEM[3779].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3780 GTM_gtm_cls0.MCS0_MEM[3780].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3781 GTM_gtm_cls0.MCS0_MEM[3781].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3782 GTM_gtm_cls0.MCS0_MEM[3782].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3783 GTM_gtm_cls0.MCS0_MEM[3783].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3784 GTM_gtm_cls0.MCS0_MEM[3784].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3785 GTM_gtm_cls0.MCS0_MEM[3785].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3786 GTM_gtm_cls0.MCS0_MEM[3786].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3787 GTM_gtm_cls0.MCS0_MEM[3787].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3788 GTM_gtm_cls0.MCS0_MEM[3788].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3789 GTM_gtm_cls0.MCS0_MEM[3789].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3790 GTM_gtm_cls0.MCS0_MEM[3790].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3791 GTM_gtm_cls0.MCS0_MEM[3791].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3792 GTM_gtm_cls0.MCS0_MEM[3792].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3793 GTM_gtm_cls0.MCS0_MEM[3793].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3794 GTM_gtm_cls0.MCS0_MEM[3794].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3795 GTM_gtm_cls0.MCS0_MEM[3795].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3796 GTM_gtm_cls0.MCS0_MEM[3796].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3797 GTM_gtm_cls0.MCS0_MEM[3797].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3798 GTM_gtm_cls0.MCS0_MEM[3798].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3799 GTM_gtm_cls0.MCS0_MEM[3799].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3800 GTM_gtm_cls0.MCS0_MEM[3800].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3801 GTM_gtm_cls0.MCS0_MEM[3801].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3802 GTM_gtm_cls0.MCS0_MEM[3802].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3803 GTM_gtm_cls0.MCS0_MEM[3803].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3804 GTM_gtm_cls0.MCS0_MEM[3804].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3805 GTM_gtm_cls0.MCS0_MEM[3805].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3806 GTM_gtm_cls0.MCS0_MEM[3806].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3807 GTM_gtm_cls0.MCS0_MEM[3807].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3808 GTM_gtm_cls0.MCS0_MEM[3808].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3809 GTM_gtm_cls0.MCS0_MEM[3809].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3810 GTM_gtm_cls0.MCS0_MEM[3810].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3811 GTM_gtm_cls0.MCS0_MEM[3811].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3812 GTM_gtm_cls0.MCS0_MEM[3812].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3813 GTM_gtm_cls0.MCS0_MEM[3813].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3814 GTM_gtm_cls0.MCS0_MEM[3814].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3815 GTM_gtm_cls0.MCS0_MEM[3815].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3816 GTM_gtm_cls0.MCS0_MEM[3816].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3817 GTM_gtm_cls0.MCS0_MEM[3817].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3818 GTM_gtm_cls0.MCS0_MEM[3818].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3819 GTM_gtm_cls0.MCS0_MEM[3819].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3820 GTM_gtm_cls0.MCS0_MEM[3820].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3821 GTM_gtm_cls0.MCS0_MEM[3821].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3822 GTM_gtm_cls0.MCS0_MEM[3822].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3823 GTM_gtm_cls0.MCS0_MEM[3823].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3824 GTM_gtm_cls0.MCS0_MEM[3824].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3825 GTM_gtm_cls0.MCS0_MEM[3825].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3826 GTM_gtm_cls0.MCS0_MEM[3826].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3827 GTM_gtm_cls0.MCS0_MEM[3827].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3828 GTM_gtm_cls0.MCS0_MEM[3828].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3829 GTM_gtm_cls0.MCS0_MEM[3829].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3830 GTM_gtm_cls0.MCS0_MEM[3830].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3831 GTM_gtm_cls0.MCS0_MEM[3831].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3832 GTM_gtm_cls0.MCS0_MEM[3832].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3833 GTM_gtm_cls0.MCS0_MEM[3833].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3834 GTM_gtm_cls0.MCS0_MEM[3834].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3835 GTM_gtm_cls0.MCS0_MEM[3835].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3836 GTM_gtm_cls0.MCS0_MEM[3836].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3837 GTM_gtm_cls0.MCS0_MEM[3837].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3838 GTM_gtm_cls0.MCS0_MEM[3838].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3839 GTM_gtm_cls0.MCS0_MEM[3839].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3840 GTM_gtm_cls0.MCS0_MEM[3840].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3841 GTM_gtm_cls0.MCS0_MEM[3841].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3842 GTM_gtm_cls0.MCS0_MEM[3842].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3843 GTM_gtm_cls0.MCS0_MEM[3843].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3844 GTM_gtm_cls0.MCS0_MEM[3844].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3845 GTM_gtm_cls0.MCS0_MEM[3845].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3846 GTM_gtm_cls0.MCS0_MEM[3846].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3847 GTM_gtm_cls0.MCS0_MEM[3847].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3848 GTM_gtm_cls0.MCS0_MEM[3848].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3849 GTM_gtm_cls0.MCS0_MEM[3849].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3850 GTM_gtm_cls0.MCS0_MEM[3850].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3851 GTM_gtm_cls0.MCS0_MEM[3851].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3852 GTM_gtm_cls0.MCS0_MEM[3852].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3853 GTM_gtm_cls0.MCS0_MEM[3853].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3854 GTM_gtm_cls0.MCS0_MEM[3854].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3855 GTM_gtm_cls0.MCS0_MEM[3855].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3856 GTM_gtm_cls0.MCS0_MEM[3856].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3857 GTM_gtm_cls0.MCS0_MEM[3857].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3858 GTM_gtm_cls0.MCS0_MEM[3858].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3859 GTM_gtm_cls0.MCS0_MEM[3859].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3860 GTM_gtm_cls0.MCS0_MEM[3860].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3861 GTM_gtm_cls0.MCS0_MEM[3861].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3862 GTM_gtm_cls0.MCS0_MEM[3862].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3863 GTM_gtm_cls0.MCS0_MEM[3863].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3864 GTM_gtm_cls0.MCS0_MEM[3864].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3865 GTM_gtm_cls0.MCS0_MEM[3865].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3866 GTM_gtm_cls0.MCS0_MEM[3866].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3867 GTM_gtm_cls0.MCS0_MEM[3867].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3868 GTM_gtm_cls0.MCS0_MEM[3868].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3869 GTM_gtm_cls0.MCS0_MEM[3869].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3870 GTM_gtm_cls0.MCS0_MEM[3870].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3871 GTM_gtm_cls0.MCS0_MEM[3871].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3872 GTM_gtm_cls0.MCS0_MEM[3872].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3873 GTM_gtm_cls0.MCS0_MEM[3873].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3874 GTM_gtm_cls0.MCS0_MEM[3874].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3875 GTM_gtm_cls0.MCS0_MEM[3875].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3876 GTM_gtm_cls0.MCS0_MEM[3876].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3877 GTM_gtm_cls0.MCS0_MEM[3877].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3878 GTM_gtm_cls0.MCS0_MEM[3878].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3879 GTM_gtm_cls0.MCS0_MEM[3879].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3880 GTM_gtm_cls0.MCS0_MEM[3880].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3881 GTM_gtm_cls0.MCS0_MEM[3881].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3882 GTM_gtm_cls0.MCS0_MEM[3882].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3883 GTM_gtm_cls0.MCS0_MEM[3883].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3884 GTM_gtm_cls0.MCS0_MEM[3884].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3885 GTM_gtm_cls0.MCS0_MEM[3885].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3886 GTM_gtm_cls0.MCS0_MEM[3886].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3887 GTM_gtm_cls0.MCS0_MEM[3887].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3888 GTM_gtm_cls0.MCS0_MEM[3888].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3889 GTM_gtm_cls0.MCS0_MEM[3889].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3890 GTM_gtm_cls0.MCS0_MEM[3890].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3891 GTM_gtm_cls0.MCS0_MEM[3891].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3892 GTM_gtm_cls0.MCS0_MEM[3892].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3893 GTM_gtm_cls0.MCS0_MEM[3893].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3894 GTM_gtm_cls0.MCS0_MEM[3894].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3895 GTM_gtm_cls0.MCS0_MEM[3895].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3896 GTM_gtm_cls0.MCS0_MEM[3896].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3897 GTM_gtm_cls0.MCS0_MEM[3897].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3898 GTM_gtm_cls0.MCS0_MEM[3898].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3899 GTM_gtm_cls0.MCS0_MEM[3899].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3900 GTM_gtm_cls0.MCS0_MEM[3900].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3901 GTM_gtm_cls0.MCS0_MEM[3901].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3902 GTM_gtm_cls0.MCS0_MEM[3902].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3903 GTM_gtm_cls0.MCS0_MEM[3903].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3904 GTM_gtm_cls0.MCS0_MEM[3904].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3905 GTM_gtm_cls0.MCS0_MEM[3905].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3906 GTM_gtm_cls0.MCS0_MEM[3906].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3907 GTM_gtm_cls0.MCS0_MEM[3907].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3908 GTM_gtm_cls0.MCS0_MEM[3908].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3909 GTM_gtm_cls0.MCS0_MEM[3909].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3910 GTM_gtm_cls0.MCS0_MEM[3910].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3911 GTM_gtm_cls0.MCS0_MEM[3911].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3912 GTM_gtm_cls0.MCS0_MEM[3912].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3913 GTM_gtm_cls0.MCS0_MEM[3913].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3914 GTM_gtm_cls0.MCS0_MEM[3914].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3915 GTM_gtm_cls0.MCS0_MEM[3915].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3916 GTM_gtm_cls0.MCS0_MEM[3916].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3917 GTM_gtm_cls0.MCS0_MEM[3917].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3918 GTM_gtm_cls0.MCS0_MEM[3918].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3919 GTM_gtm_cls0.MCS0_MEM[3919].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3920 GTM_gtm_cls0.MCS0_MEM[3920].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3921 GTM_gtm_cls0.MCS0_MEM[3921].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3922 GTM_gtm_cls0.MCS0_MEM[3922].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3923 GTM_gtm_cls0.MCS0_MEM[3923].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3924 GTM_gtm_cls0.MCS0_MEM[3924].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3925 GTM_gtm_cls0.MCS0_MEM[3925].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3926 GTM_gtm_cls0.MCS0_MEM[3926].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3927 GTM_gtm_cls0.MCS0_MEM[3927].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3928 GTM_gtm_cls0.MCS0_MEM[3928].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3929 GTM_gtm_cls0.MCS0_MEM[3929].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3930 GTM_gtm_cls0.MCS0_MEM[3930].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3931 GTM_gtm_cls0.MCS0_MEM[3931].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3932 GTM_gtm_cls0.MCS0_MEM[3932].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3933 GTM_gtm_cls0.MCS0_MEM[3933].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3934 GTM_gtm_cls0.MCS0_MEM[3934].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3935 GTM_gtm_cls0.MCS0_MEM[3935].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3936 GTM_gtm_cls0.MCS0_MEM[3936].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3937 GTM_gtm_cls0.MCS0_MEM[3937].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3938 GTM_gtm_cls0.MCS0_MEM[3938].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3939 GTM_gtm_cls0.MCS0_MEM[3939].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3940 GTM_gtm_cls0.MCS0_MEM[3940].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3941 GTM_gtm_cls0.MCS0_MEM[3941].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3942 GTM_gtm_cls0.MCS0_MEM[3942].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3943 GTM_gtm_cls0.MCS0_MEM[3943].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3944 GTM_gtm_cls0.MCS0_MEM[3944].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3945 GTM_gtm_cls0.MCS0_MEM[3945].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3946 GTM_gtm_cls0.MCS0_MEM[3946].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3947 GTM_gtm_cls0.MCS0_MEM[3947].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3948 GTM_gtm_cls0.MCS0_MEM[3948].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3949 GTM_gtm_cls0.MCS0_MEM[3949].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3950 GTM_gtm_cls0.MCS0_MEM[3950].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3951 GTM_gtm_cls0.MCS0_MEM[3951].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3952 GTM_gtm_cls0.MCS0_MEM[3952].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3953 GTM_gtm_cls0.MCS0_MEM[3953].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3954 GTM_gtm_cls0.MCS0_MEM[3954].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3955 GTM_gtm_cls0.MCS0_MEM[3955].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3956 GTM_gtm_cls0.MCS0_MEM[3956].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3957 GTM_gtm_cls0.MCS0_MEM[3957].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3958 GTM_gtm_cls0.MCS0_MEM[3958].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3959 GTM_gtm_cls0.MCS0_MEM[3959].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3960 GTM_gtm_cls0.MCS0_MEM[3960].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3961 GTM_gtm_cls0.MCS0_MEM[3961].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3962 GTM_gtm_cls0.MCS0_MEM[3962].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3963 GTM_gtm_cls0.MCS0_MEM[3963].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3964 GTM_gtm_cls0.MCS0_MEM[3964].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3965 GTM_gtm_cls0.MCS0_MEM[3965].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3966 GTM_gtm_cls0.MCS0_MEM[3966].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3967 GTM_gtm_cls0.MCS0_MEM[3967].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3968 GTM_gtm_cls0.MCS0_MEM[3968].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3969 GTM_gtm_cls0.MCS0_MEM[3969].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3970 GTM_gtm_cls0.MCS0_MEM[3970].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3971 GTM_gtm_cls0.MCS0_MEM[3971].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3972 GTM_gtm_cls0.MCS0_MEM[3972].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3973 GTM_gtm_cls0.MCS0_MEM[3973].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3974 GTM_gtm_cls0.MCS0_MEM[3974].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3975 GTM_gtm_cls0.MCS0_MEM[3975].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3976 GTM_gtm_cls0.MCS0_MEM[3976].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3977 GTM_gtm_cls0.MCS0_MEM[3977].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3978 GTM_gtm_cls0.MCS0_MEM[3978].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3979 GTM_gtm_cls0.MCS0_MEM[3979].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3980 GTM_gtm_cls0.MCS0_MEM[3980].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3981 GTM_gtm_cls0.MCS0_MEM[3981].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3982 GTM_gtm_cls0.MCS0_MEM[3982].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3983 GTM_gtm_cls0.MCS0_MEM[3983].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3984 GTM_gtm_cls0.MCS0_MEM[3984].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3985 GTM_gtm_cls0.MCS0_MEM[3985].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3986 GTM_gtm_cls0.MCS0_MEM[3986].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3987 GTM_gtm_cls0.MCS0_MEM[3987].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3988 GTM_gtm_cls0.MCS0_MEM[3988].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3989 GTM_gtm_cls0.MCS0_MEM[3989].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3990 GTM_gtm_cls0.MCS0_MEM[3990].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3991 GTM_gtm_cls0.MCS0_MEM[3991].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3992 GTM_gtm_cls0.MCS0_MEM[3992].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3993 GTM_gtm_cls0.MCS0_MEM[3993].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3994 GTM_gtm_cls0.MCS0_MEM[3994].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3995 GTM_gtm_cls0.MCS0_MEM[3995].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3996 GTM_gtm_cls0.MCS0_MEM[3996].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3997 GTM_gtm_cls0.MCS0_MEM[3997].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3998 GTM_gtm_cls0.MCS0_MEM[3998].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM3999 GTM_gtm_cls0.MCS0_MEM[3999].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4000 GTM_gtm_cls0.MCS0_MEM[4000].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4001 GTM_gtm_cls0.MCS0_MEM[4001].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4002 GTM_gtm_cls0.MCS0_MEM[4002].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4003 GTM_gtm_cls0.MCS0_MEM[4003].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4004 GTM_gtm_cls0.MCS0_MEM[4004].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4005 GTM_gtm_cls0.MCS0_MEM[4005].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4006 GTM_gtm_cls0.MCS0_MEM[4006].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4007 GTM_gtm_cls0.MCS0_MEM[4007].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4008 GTM_gtm_cls0.MCS0_MEM[4008].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4009 GTM_gtm_cls0.MCS0_MEM[4009].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4010 GTM_gtm_cls0.MCS0_MEM[4010].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4011 GTM_gtm_cls0.MCS0_MEM[4011].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4012 GTM_gtm_cls0.MCS0_MEM[4012].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4013 GTM_gtm_cls0.MCS0_MEM[4013].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4014 GTM_gtm_cls0.MCS0_MEM[4014].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4015 GTM_gtm_cls0.MCS0_MEM[4015].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4016 GTM_gtm_cls0.MCS0_MEM[4016].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4017 GTM_gtm_cls0.MCS0_MEM[4017].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4018 GTM_gtm_cls0.MCS0_MEM[4018].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4019 GTM_gtm_cls0.MCS0_MEM[4019].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4020 GTM_gtm_cls0.MCS0_MEM[4020].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4021 GTM_gtm_cls0.MCS0_MEM[4021].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4022 GTM_gtm_cls0.MCS0_MEM[4022].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4023 GTM_gtm_cls0.MCS0_MEM[4023].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4024 GTM_gtm_cls0.MCS0_MEM[4024].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4025 GTM_gtm_cls0.MCS0_MEM[4025].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4026 GTM_gtm_cls0.MCS0_MEM[4026].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4027 GTM_gtm_cls0.MCS0_MEM[4027].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4028 GTM_gtm_cls0.MCS0_MEM[4028].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4029 GTM_gtm_cls0.MCS0_MEM[4029].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4030 GTM_gtm_cls0.MCS0_MEM[4030].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4031 GTM_gtm_cls0.MCS0_MEM[4031].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4032 GTM_gtm_cls0.MCS0_MEM[4032].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4033 GTM_gtm_cls0.MCS0_MEM[4033].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4034 GTM_gtm_cls0.MCS0_MEM[4034].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4035 GTM_gtm_cls0.MCS0_MEM[4035].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4036 GTM_gtm_cls0.MCS0_MEM[4036].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4037 GTM_gtm_cls0.MCS0_MEM[4037].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4038 GTM_gtm_cls0.MCS0_MEM[4038].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4039 GTM_gtm_cls0.MCS0_MEM[4039].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4040 GTM_gtm_cls0.MCS0_MEM[4040].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4041 GTM_gtm_cls0.MCS0_MEM[4041].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4042 GTM_gtm_cls0.MCS0_MEM[4042].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4043 GTM_gtm_cls0.MCS0_MEM[4043].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4044 GTM_gtm_cls0.MCS0_MEM[4044].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4045 GTM_gtm_cls0.MCS0_MEM[4045].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4046 GTM_gtm_cls0.MCS0_MEM[4046].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4047 GTM_gtm_cls0.MCS0_MEM[4047].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4048 GTM_gtm_cls0.MCS0_MEM[4048].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4049 GTM_gtm_cls0.MCS0_MEM[4049].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4050 GTM_gtm_cls0.MCS0_MEM[4050].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4051 GTM_gtm_cls0.MCS0_MEM[4051].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4052 GTM_gtm_cls0.MCS0_MEM[4052].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4053 GTM_gtm_cls0.MCS0_MEM[4053].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4054 GTM_gtm_cls0.MCS0_MEM[4054].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4055 GTM_gtm_cls0.MCS0_MEM[4055].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4056 GTM_gtm_cls0.MCS0_MEM[4056].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4057 GTM_gtm_cls0.MCS0_MEM[4057].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4058 GTM_gtm_cls0.MCS0_MEM[4058].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4059 GTM_gtm_cls0.MCS0_MEM[4059].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4060 GTM_gtm_cls0.MCS0_MEM[4060].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4061 GTM_gtm_cls0.MCS0_MEM[4061].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4062 GTM_gtm_cls0.MCS0_MEM[4062].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4063 GTM_gtm_cls0.MCS0_MEM[4063].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4064 GTM_gtm_cls0.MCS0_MEM[4064].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4065 GTM_gtm_cls0.MCS0_MEM[4065].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4066 GTM_gtm_cls0.MCS0_MEM[4066].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4067 GTM_gtm_cls0.MCS0_MEM[4067].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4068 GTM_gtm_cls0.MCS0_MEM[4068].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4069 GTM_gtm_cls0.MCS0_MEM[4069].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4070 GTM_gtm_cls0.MCS0_MEM[4070].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4071 GTM_gtm_cls0.MCS0_MEM[4071].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4072 GTM_gtm_cls0.MCS0_MEM[4072].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4073 GTM_gtm_cls0.MCS0_MEM[4073].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4074 GTM_gtm_cls0.MCS0_MEM[4074].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4075 GTM_gtm_cls0.MCS0_MEM[4075].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4076 GTM_gtm_cls0.MCS0_MEM[4076].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4077 GTM_gtm_cls0.MCS0_MEM[4077].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4078 GTM_gtm_cls0.MCS0_MEM[4078].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4079 GTM_gtm_cls0.MCS0_MEM[4079].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4080 GTM_gtm_cls0.MCS0_MEM[4080].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4081 GTM_gtm_cls0.MCS0_MEM[4081].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4082 GTM_gtm_cls0.MCS0_MEM[4082].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4083 GTM_gtm_cls0.MCS0_MEM[4083].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4084 GTM_gtm_cls0.MCS0_MEM[4084].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4085 GTM_gtm_cls0.MCS0_MEM[4085].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4086 GTM_gtm_cls0.MCS0_MEM[4086].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4087 GTM_gtm_cls0.MCS0_MEM[4087].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4088 GTM_gtm_cls0.MCS0_MEM[4088].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4089 GTM_gtm_cls0.MCS0_MEM[4089].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4090 GTM_gtm_cls0.MCS0_MEM[4090].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4091 GTM_gtm_cls0.MCS0_MEM[4091].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4092 GTM_gtm_cls0.MCS0_MEM[4092].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4093 GTM_gtm_cls0.MCS0_MEM[4093].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4094 GTM_gtm_cls0.MCS0_MEM[4094].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4095 GTM_gtm_cls0.MCS0_MEM[4095].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4096 GTM_gtm_cls0.MCS0_MEM[4096].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4097 GTM_gtm_cls0.MCS0_MEM[4097].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4098 GTM_gtm_cls0.MCS0_MEM[4098].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4099 GTM_gtm_cls0.MCS0_MEM[4099].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4100 GTM_gtm_cls0.MCS0_MEM[4100].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4101 GTM_gtm_cls0.MCS0_MEM[4101].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4102 GTM_gtm_cls0.MCS0_MEM[4102].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4103 GTM_gtm_cls0.MCS0_MEM[4103].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4104 GTM_gtm_cls0.MCS0_MEM[4104].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4105 GTM_gtm_cls0.MCS0_MEM[4105].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4106 GTM_gtm_cls0.MCS0_MEM[4106].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4107 GTM_gtm_cls0.MCS0_MEM[4107].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4108 GTM_gtm_cls0.MCS0_MEM[4108].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4109 GTM_gtm_cls0.MCS0_MEM[4109].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4110 GTM_gtm_cls0.MCS0_MEM[4110].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4111 GTM_gtm_cls0.MCS0_MEM[4111].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4112 GTM_gtm_cls0.MCS0_MEM[4112].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4113 GTM_gtm_cls0.MCS0_MEM[4113].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4114 GTM_gtm_cls0.MCS0_MEM[4114].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4115 GTM_gtm_cls0.MCS0_MEM[4115].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4116 GTM_gtm_cls0.MCS0_MEM[4116].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4117 GTM_gtm_cls0.MCS0_MEM[4117].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4118 GTM_gtm_cls0.MCS0_MEM[4118].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4119 GTM_gtm_cls0.MCS0_MEM[4119].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4120 GTM_gtm_cls0.MCS0_MEM[4120].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4121 GTM_gtm_cls0.MCS0_MEM[4121].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4122 GTM_gtm_cls0.MCS0_MEM[4122].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4123 GTM_gtm_cls0.MCS0_MEM[4123].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4124 GTM_gtm_cls0.MCS0_MEM[4124].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4125 GTM_gtm_cls0.MCS0_MEM[4125].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4126 GTM_gtm_cls0.MCS0_MEM[4126].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4127 GTM_gtm_cls0.MCS0_MEM[4127].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4128 GTM_gtm_cls0.MCS0_MEM[4128].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4129 GTM_gtm_cls0.MCS0_MEM[4129].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4130 GTM_gtm_cls0.MCS0_MEM[4130].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4131 GTM_gtm_cls0.MCS0_MEM[4131].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4132 GTM_gtm_cls0.MCS0_MEM[4132].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4133 GTM_gtm_cls0.MCS0_MEM[4133].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4134 GTM_gtm_cls0.MCS0_MEM[4134].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4135 GTM_gtm_cls0.MCS0_MEM[4135].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4136 GTM_gtm_cls0.MCS0_MEM[4136].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4137 GTM_gtm_cls0.MCS0_MEM[4137].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4138 GTM_gtm_cls0.MCS0_MEM[4138].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4139 GTM_gtm_cls0.MCS0_MEM[4139].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4140 GTM_gtm_cls0.MCS0_MEM[4140].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4141 GTM_gtm_cls0.MCS0_MEM[4141].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4142 GTM_gtm_cls0.MCS0_MEM[4142].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4143 GTM_gtm_cls0.MCS0_MEM[4143].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4144 GTM_gtm_cls0.MCS0_MEM[4144].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4145 GTM_gtm_cls0.MCS0_MEM[4145].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4146 GTM_gtm_cls0.MCS0_MEM[4146].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4147 GTM_gtm_cls0.MCS0_MEM[4147].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4148 GTM_gtm_cls0.MCS0_MEM[4148].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4149 GTM_gtm_cls0.MCS0_MEM[4149].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4150 GTM_gtm_cls0.MCS0_MEM[4150].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4151 GTM_gtm_cls0.MCS0_MEM[4151].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4152 GTM_gtm_cls0.MCS0_MEM[4152].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4153 GTM_gtm_cls0.MCS0_MEM[4153].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4154 GTM_gtm_cls0.MCS0_MEM[4154].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4155 GTM_gtm_cls0.MCS0_MEM[4155].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4156 GTM_gtm_cls0.MCS0_MEM[4156].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4157 GTM_gtm_cls0.MCS0_MEM[4157].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4158 GTM_gtm_cls0.MCS0_MEM[4158].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4159 GTM_gtm_cls0.MCS0_MEM[4159].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4160 GTM_gtm_cls0.MCS0_MEM[4160].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4161 GTM_gtm_cls0.MCS0_MEM[4161].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4162 GTM_gtm_cls0.MCS0_MEM[4162].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4163 GTM_gtm_cls0.MCS0_MEM[4163].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4164 GTM_gtm_cls0.MCS0_MEM[4164].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4165 GTM_gtm_cls0.MCS0_MEM[4165].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4166 GTM_gtm_cls0.MCS0_MEM[4166].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4167 GTM_gtm_cls0.MCS0_MEM[4167].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4168 GTM_gtm_cls0.MCS0_MEM[4168].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4169 GTM_gtm_cls0.MCS0_MEM[4169].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4170 GTM_gtm_cls0.MCS0_MEM[4170].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4171 GTM_gtm_cls0.MCS0_MEM[4171].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4172 GTM_gtm_cls0.MCS0_MEM[4172].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4173 GTM_gtm_cls0.MCS0_MEM[4173].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4174 GTM_gtm_cls0.MCS0_MEM[4174].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4175 GTM_gtm_cls0.MCS0_MEM[4175].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4176 GTM_gtm_cls0.MCS0_MEM[4176].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4177 GTM_gtm_cls0.MCS0_MEM[4177].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4178 GTM_gtm_cls0.MCS0_MEM[4178].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4179 GTM_gtm_cls0.MCS0_MEM[4179].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4180 GTM_gtm_cls0.MCS0_MEM[4180].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4181 GTM_gtm_cls0.MCS0_MEM[4181].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4182 GTM_gtm_cls0.MCS0_MEM[4182].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4183 GTM_gtm_cls0.MCS0_MEM[4183].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4184 GTM_gtm_cls0.MCS0_MEM[4184].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4185 GTM_gtm_cls0.MCS0_MEM[4185].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4186 GTM_gtm_cls0.MCS0_MEM[4186].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4187 GTM_gtm_cls0.MCS0_MEM[4187].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4188 GTM_gtm_cls0.MCS0_MEM[4188].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4189 GTM_gtm_cls0.MCS0_MEM[4189].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4190 GTM_gtm_cls0.MCS0_MEM[4190].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4191 GTM_gtm_cls0.MCS0_MEM[4191].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4192 GTM_gtm_cls0.MCS0_MEM[4192].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4193 GTM_gtm_cls0.MCS0_MEM[4193].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4194 GTM_gtm_cls0.MCS0_MEM[4194].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4195 GTM_gtm_cls0.MCS0_MEM[4195].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4196 GTM_gtm_cls0.MCS0_MEM[4196].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4197 GTM_gtm_cls0.MCS0_MEM[4197].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4198 GTM_gtm_cls0.MCS0_MEM[4198].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4199 GTM_gtm_cls0.MCS0_MEM[4199].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4200 GTM_gtm_cls0.MCS0_MEM[4200].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4201 GTM_gtm_cls0.MCS0_MEM[4201].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4202 GTM_gtm_cls0.MCS0_MEM[4202].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4203 GTM_gtm_cls0.MCS0_MEM[4203].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4204 GTM_gtm_cls0.MCS0_MEM[4204].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4205 GTM_gtm_cls0.MCS0_MEM[4205].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4206 GTM_gtm_cls0.MCS0_MEM[4206].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4207 GTM_gtm_cls0.MCS0_MEM[4207].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4208 GTM_gtm_cls0.MCS0_MEM[4208].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4209 GTM_gtm_cls0.MCS0_MEM[4209].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4210 GTM_gtm_cls0.MCS0_MEM[4210].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4211 GTM_gtm_cls0.MCS0_MEM[4211].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4212 GTM_gtm_cls0.MCS0_MEM[4212].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4213 GTM_gtm_cls0.MCS0_MEM[4213].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4214 GTM_gtm_cls0.MCS0_MEM[4214].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4215 GTM_gtm_cls0.MCS0_MEM[4215].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4216 GTM_gtm_cls0.MCS0_MEM[4216].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4217 GTM_gtm_cls0.MCS0_MEM[4217].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4218 GTM_gtm_cls0.MCS0_MEM[4218].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4219 GTM_gtm_cls0.MCS0_MEM[4219].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4220 GTM_gtm_cls0.MCS0_MEM[4220].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4221 GTM_gtm_cls0.MCS0_MEM[4221].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4222 GTM_gtm_cls0.MCS0_MEM[4222].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4223 GTM_gtm_cls0.MCS0_MEM[4223].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4224 GTM_gtm_cls0.MCS0_MEM[4224].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4225 GTM_gtm_cls0.MCS0_MEM[4225].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4226 GTM_gtm_cls0.MCS0_MEM[4226].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4227 GTM_gtm_cls0.MCS0_MEM[4227].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4228 GTM_gtm_cls0.MCS0_MEM[4228].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4229 GTM_gtm_cls0.MCS0_MEM[4229].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4230 GTM_gtm_cls0.MCS0_MEM[4230].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4231 GTM_gtm_cls0.MCS0_MEM[4231].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4232 GTM_gtm_cls0.MCS0_MEM[4232].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4233 GTM_gtm_cls0.MCS0_MEM[4233].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4234 GTM_gtm_cls0.MCS0_MEM[4234].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4235 GTM_gtm_cls0.MCS0_MEM[4235].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4236 GTM_gtm_cls0.MCS0_MEM[4236].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4237 GTM_gtm_cls0.MCS0_MEM[4237].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4238 GTM_gtm_cls0.MCS0_MEM[4238].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4239 GTM_gtm_cls0.MCS0_MEM[4239].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4240 GTM_gtm_cls0.MCS0_MEM[4240].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4241 GTM_gtm_cls0.MCS0_MEM[4241].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4242 GTM_gtm_cls0.MCS0_MEM[4242].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4243 GTM_gtm_cls0.MCS0_MEM[4243].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4244 GTM_gtm_cls0.MCS0_MEM[4244].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4245 GTM_gtm_cls0.MCS0_MEM[4245].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4246 GTM_gtm_cls0.MCS0_MEM[4246].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4247 GTM_gtm_cls0.MCS0_MEM[4247].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4248 GTM_gtm_cls0.MCS0_MEM[4248].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4249 GTM_gtm_cls0.MCS0_MEM[4249].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4250 GTM_gtm_cls0.MCS0_MEM[4250].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4251 GTM_gtm_cls0.MCS0_MEM[4251].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4252 GTM_gtm_cls0.MCS0_MEM[4252].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4253 GTM_gtm_cls0.MCS0_MEM[4253].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4254 GTM_gtm_cls0.MCS0_MEM[4254].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4255 GTM_gtm_cls0.MCS0_MEM[4255].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4256 GTM_gtm_cls0.MCS0_MEM[4256].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4257 GTM_gtm_cls0.MCS0_MEM[4257].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4258 GTM_gtm_cls0.MCS0_MEM[4258].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4259 GTM_gtm_cls0.MCS0_MEM[4259].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4260 GTM_gtm_cls0.MCS0_MEM[4260].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4261 GTM_gtm_cls0.MCS0_MEM[4261].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4262 GTM_gtm_cls0.MCS0_MEM[4262].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4263 GTM_gtm_cls0.MCS0_MEM[4263].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4264 GTM_gtm_cls0.MCS0_MEM[4264].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4265 GTM_gtm_cls0.MCS0_MEM[4265].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4266 GTM_gtm_cls0.MCS0_MEM[4266].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4267 GTM_gtm_cls0.MCS0_MEM[4267].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4268 GTM_gtm_cls0.MCS0_MEM[4268].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4269 GTM_gtm_cls0.MCS0_MEM[4269].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4270 GTM_gtm_cls0.MCS0_MEM[4270].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4271 GTM_gtm_cls0.MCS0_MEM[4271].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4272 GTM_gtm_cls0.MCS0_MEM[4272].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4273 GTM_gtm_cls0.MCS0_MEM[4273].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4274 GTM_gtm_cls0.MCS0_MEM[4274].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4275 GTM_gtm_cls0.MCS0_MEM[4275].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4276 GTM_gtm_cls0.MCS0_MEM[4276].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4277 GTM_gtm_cls0.MCS0_MEM[4277].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4278 GTM_gtm_cls0.MCS0_MEM[4278].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4279 GTM_gtm_cls0.MCS0_MEM[4279].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4280 GTM_gtm_cls0.MCS0_MEM[4280].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4281 GTM_gtm_cls0.MCS0_MEM[4281].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4282 GTM_gtm_cls0.MCS0_MEM[4282].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4283 GTM_gtm_cls0.MCS0_MEM[4283].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4284 GTM_gtm_cls0.MCS0_MEM[4284].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4285 GTM_gtm_cls0.MCS0_MEM[4285].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4286 GTM_gtm_cls0.MCS0_MEM[4286].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4287 GTM_gtm_cls0.MCS0_MEM[4287].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4288 GTM_gtm_cls0.MCS0_MEM[4288].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4289 GTM_gtm_cls0.MCS0_MEM[4289].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4290 GTM_gtm_cls0.MCS0_MEM[4290].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4291 GTM_gtm_cls0.MCS0_MEM[4291].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4292 GTM_gtm_cls0.MCS0_MEM[4292].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4293 GTM_gtm_cls0.MCS0_MEM[4293].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4294 GTM_gtm_cls0.MCS0_MEM[4294].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4295 GTM_gtm_cls0.MCS0_MEM[4295].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4296 GTM_gtm_cls0.MCS0_MEM[4296].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4297 GTM_gtm_cls0.MCS0_MEM[4297].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4298 GTM_gtm_cls0.MCS0_MEM[4298].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4299 GTM_gtm_cls0.MCS0_MEM[4299].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4300 GTM_gtm_cls0.MCS0_MEM[4300].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4301 GTM_gtm_cls0.MCS0_MEM[4301].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4302 GTM_gtm_cls0.MCS0_MEM[4302].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4303 GTM_gtm_cls0.MCS0_MEM[4303].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4304 GTM_gtm_cls0.MCS0_MEM[4304].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4305 GTM_gtm_cls0.MCS0_MEM[4305].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4306 GTM_gtm_cls0.MCS0_MEM[4306].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4307 GTM_gtm_cls0.MCS0_MEM[4307].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4308 GTM_gtm_cls0.MCS0_MEM[4308].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4309 GTM_gtm_cls0.MCS0_MEM[4309].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4310 GTM_gtm_cls0.MCS0_MEM[4310].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4311 GTM_gtm_cls0.MCS0_MEM[4311].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4312 GTM_gtm_cls0.MCS0_MEM[4312].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4313 GTM_gtm_cls0.MCS0_MEM[4313].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4314 GTM_gtm_cls0.MCS0_MEM[4314].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4315 GTM_gtm_cls0.MCS0_MEM[4315].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4316 GTM_gtm_cls0.MCS0_MEM[4316].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4317 GTM_gtm_cls0.MCS0_MEM[4317].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4318 GTM_gtm_cls0.MCS0_MEM[4318].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4319 GTM_gtm_cls0.MCS0_MEM[4319].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4320 GTM_gtm_cls0.MCS0_MEM[4320].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4321 GTM_gtm_cls0.MCS0_MEM[4321].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4322 GTM_gtm_cls0.MCS0_MEM[4322].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4323 GTM_gtm_cls0.MCS0_MEM[4323].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4324 GTM_gtm_cls0.MCS0_MEM[4324].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4325 GTM_gtm_cls0.MCS0_MEM[4325].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4326 GTM_gtm_cls0.MCS0_MEM[4326].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4327 GTM_gtm_cls0.MCS0_MEM[4327].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4328 GTM_gtm_cls0.MCS0_MEM[4328].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4329 GTM_gtm_cls0.MCS0_MEM[4329].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4330 GTM_gtm_cls0.MCS0_MEM[4330].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4331 GTM_gtm_cls0.MCS0_MEM[4331].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4332 GTM_gtm_cls0.MCS0_MEM[4332].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4333 GTM_gtm_cls0.MCS0_MEM[4333].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4334 GTM_gtm_cls0.MCS0_MEM[4334].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4335 GTM_gtm_cls0.MCS0_MEM[4335].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4336 GTM_gtm_cls0.MCS0_MEM[4336].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4337 GTM_gtm_cls0.MCS0_MEM[4337].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4338 GTM_gtm_cls0.MCS0_MEM[4338].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4339 GTM_gtm_cls0.MCS0_MEM[4339].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4340 GTM_gtm_cls0.MCS0_MEM[4340].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4341 GTM_gtm_cls0.MCS0_MEM[4341].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4342 GTM_gtm_cls0.MCS0_MEM[4342].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4343 GTM_gtm_cls0.MCS0_MEM[4343].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4344 GTM_gtm_cls0.MCS0_MEM[4344].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4345 GTM_gtm_cls0.MCS0_MEM[4345].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4346 GTM_gtm_cls0.MCS0_MEM[4346].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4347 GTM_gtm_cls0.MCS0_MEM[4347].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4348 GTM_gtm_cls0.MCS0_MEM[4348].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4349 GTM_gtm_cls0.MCS0_MEM[4349].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4350 GTM_gtm_cls0.MCS0_MEM[4350].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4351 GTM_gtm_cls0.MCS0_MEM[4351].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4352 GTM_gtm_cls0.MCS0_MEM[4352].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4353 GTM_gtm_cls0.MCS0_MEM[4353].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4354 GTM_gtm_cls0.MCS0_MEM[4354].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4355 GTM_gtm_cls0.MCS0_MEM[4355].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4356 GTM_gtm_cls0.MCS0_MEM[4356].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4357 GTM_gtm_cls0.MCS0_MEM[4357].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4358 GTM_gtm_cls0.MCS0_MEM[4358].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4359 GTM_gtm_cls0.MCS0_MEM[4359].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4360 GTM_gtm_cls0.MCS0_MEM[4360].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4361 GTM_gtm_cls0.MCS0_MEM[4361].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4362 GTM_gtm_cls0.MCS0_MEM[4362].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4363 GTM_gtm_cls0.MCS0_MEM[4363].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4364 GTM_gtm_cls0.MCS0_MEM[4364].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4365 GTM_gtm_cls0.MCS0_MEM[4365].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4366 GTM_gtm_cls0.MCS0_MEM[4366].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4367 GTM_gtm_cls0.MCS0_MEM[4367].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4368 GTM_gtm_cls0.MCS0_MEM[4368].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4369 GTM_gtm_cls0.MCS0_MEM[4369].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4370 GTM_gtm_cls0.MCS0_MEM[4370].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4371 GTM_gtm_cls0.MCS0_MEM[4371].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4372 GTM_gtm_cls0.MCS0_MEM[4372].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4373 GTM_gtm_cls0.MCS0_MEM[4373].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4374 GTM_gtm_cls0.MCS0_MEM[4374].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4375 GTM_gtm_cls0.MCS0_MEM[4375].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4376 GTM_gtm_cls0.MCS0_MEM[4376].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4377 GTM_gtm_cls0.MCS0_MEM[4377].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4378 GTM_gtm_cls0.MCS0_MEM[4378].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4379 GTM_gtm_cls0.MCS0_MEM[4379].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4380 GTM_gtm_cls0.MCS0_MEM[4380].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4381 GTM_gtm_cls0.MCS0_MEM[4381].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4382 GTM_gtm_cls0.MCS0_MEM[4382].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4383 GTM_gtm_cls0.MCS0_MEM[4383].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4384 GTM_gtm_cls0.MCS0_MEM[4384].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4385 GTM_gtm_cls0.MCS0_MEM[4385].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4386 GTM_gtm_cls0.MCS0_MEM[4386].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4387 GTM_gtm_cls0.MCS0_MEM[4387].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4388 GTM_gtm_cls0.MCS0_MEM[4388].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4389 GTM_gtm_cls0.MCS0_MEM[4389].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4390 GTM_gtm_cls0.MCS0_MEM[4390].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4391 GTM_gtm_cls0.MCS0_MEM[4391].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4392 GTM_gtm_cls0.MCS0_MEM[4392].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4393 GTM_gtm_cls0.MCS0_MEM[4393].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4394 GTM_gtm_cls0.MCS0_MEM[4394].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4395 GTM_gtm_cls0.MCS0_MEM[4395].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4396 GTM_gtm_cls0.MCS0_MEM[4396].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4397 GTM_gtm_cls0.MCS0_MEM[4397].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4398 GTM_gtm_cls0.MCS0_MEM[4398].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4399 GTM_gtm_cls0.MCS0_MEM[4399].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4400 GTM_gtm_cls0.MCS0_MEM[4400].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4401 GTM_gtm_cls0.MCS0_MEM[4401].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4402 GTM_gtm_cls0.MCS0_MEM[4402].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4403 GTM_gtm_cls0.MCS0_MEM[4403].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4404 GTM_gtm_cls0.MCS0_MEM[4404].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4405 GTM_gtm_cls0.MCS0_MEM[4405].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4406 GTM_gtm_cls0.MCS0_MEM[4406].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4407 GTM_gtm_cls0.MCS0_MEM[4407].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4408 GTM_gtm_cls0.MCS0_MEM[4408].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4409 GTM_gtm_cls0.MCS0_MEM[4409].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4410 GTM_gtm_cls0.MCS0_MEM[4410].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4411 GTM_gtm_cls0.MCS0_MEM[4411].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4412 GTM_gtm_cls0.MCS0_MEM[4412].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4413 GTM_gtm_cls0.MCS0_MEM[4413].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4414 GTM_gtm_cls0.MCS0_MEM[4414].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4415 GTM_gtm_cls0.MCS0_MEM[4415].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4416 GTM_gtm_cls0.MCS0_MEM[4416].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4417 GTM_gtm_cls0.MCS0_MEM[4417].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4418 GTM_gtm_cls0.MCS0_MEM[4418].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4419 GTM_gtm_cls0.MCS0_MEM[4419].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4420 GTM_gtm_cls0.MCS0_MEM[4420].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4421 GTM_gtm_cls0.MCS0_MEM[4421].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4422 GTM_gtm_cls0.MCS0_MEM[4422].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4423 GTM_gtm_cls0.MCS0_MEM[4423].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4424 GTM_gtm_cls0.MCS0_MEM[4424].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4425 GTM_gtm_cls0.MCS0_MEM[4425].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4426 GTM_gtm_cls0.MCS0_MEM[4426].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4427 GTM_gtm_cls0.MCS0_MEM[4427].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4428 GTM_gtm_cls0.MCS0_MEM[4428].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4429 GTM_gtm_cls0.MCS0_MEM[4429].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4430 GTM_gtm_cls0.MCS0_MEM[4430].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4431 GTM_gtm_cls0.MCS0_MEM[4431].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4432 GTM_gtm_cls0.MCS0_MEM[4432].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4433 GTM_gtm_cls0.MCS0_MEM[4433].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4434 GTM_gtm_cls0.MCS0_MEM[4434].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4435 GTM_gtm_cls0.MCS0_MEM[4435].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4436 GTM_gtm_cls0.MCS0_MEM[4436].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4437 GTM_gtm_cls0.MCS0_MEM[4437].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4438 GTM_gtm_cls0.MCS0_MEM[4438].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4439 GTM_gtm_cls0.MCS0_MEM[4439].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4440 GTM_gtm_cls0.MCS0_MEM[4440].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4441 GTM_gtm_cls0.MCS0_MEM[4441].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4442 GTM_gtm_cls0.MCS0_MEM[4442].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4443 GTM_gtm_cls0.MCS0_MEM[4443].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4444 GTM_gtm_cls0.MCS0_MEM[4444].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4445 GTM_gtm_cls0.MCS0_MEM[4445].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4446 GTM_gtm_cls0.MCS0_MEM[4446].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4447 GTM_gtm_cls0.MCS0_MEM[4447].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4448 GTM_gtm_cls0.MCS0_MEM[4448].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4449 GTM_gtm_cls0.MCS0_MEM[4449].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4450 GTM_gtm_cls0.MCS0_MEM[4450].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4451 GTM_gtm_cls0.MCS0_MEM[4451].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4452 GTM_gtm_cls0.MCS0_MEM[4452].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4453 GTM_gtm_cls0.MCS0_MEM[4453].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4454 GTM_gtm_cls0.MCS0_MEM[4454].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4455 GTM_gtm_cls0.MCS0_MEM[4455].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4456 GTM_gtm_cls0.MCS0_MEM[4456].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4457 GTM_gtm_cls0.MCS0_MEM[4457].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4458 GTM_gtm_cls0.MCS0_MEM[4458].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4459 GTM_gtm_cls0.MCS0_MEM[4459].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4460 GTM_gtm_cls0.MCS0_MEM[4460].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4461 GTM_gtm_cls0.MCS0_MEM[4461].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4462 GTM_gtm_cls0.MCS0_MEM[4462].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4463 GTM_gtm_cls0.MCS0_MEM[4463].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4464 GTM_gtm_cls0.MCS0_MEM[4464].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4465 GTM_gtm_cls0.MCS0_MEM[4465].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4466 GTM_gtm_cls0.MCS0_MEM[4466].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4467 GTM_gtm_cls0.MCS0_MEM[4467].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4468 GTM_gtm_cls0.MCS0_MEM[4468].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4469 GTM_gtm_cls0.MCS0_MEM[4469].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4470 GTM_gtm_cls0.MCS0_MEM[4470].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4471 GTM_gtm_cls0.MCS0_MEM[4471].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4472 GTM_gtm_cls0.MCS0_MEM[4472].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4473 GTM_gtm_cls0.MCS0_MEM[4473].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4474 GTM_gtm_cls0.MCS0_MEM[4474].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4475 GTM_gtm_cls0.MCS0_MEM[4475].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4476 GTM_gtm_cls0.MCS0_MEM[4476].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4477 GTM_gtm_cls0.MCS0_MEM[4477].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4478 GTM_gtm_cls0.MCS0_MEM[4478].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4479 GTM_gtm_cls0.MCS0_MEM[4479].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4480 GTM_gtm_cls0.MCS0_MEM[4480].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4481 GTM_gtm_cls0.MCS0_MEM[4481].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4482 GTM_gtm_cls0.MCS0_MEM[4482].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4483 GTM_gtm_cls0.MCS0_MEM[4483].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4484 GTM_gtm_cls0.MCS0_MEM[4484].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4485 GTM_gtm_cls0.MCS0_MEM[4485].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4486 GTM_gtm_cls0.MCS0_MEM[4486].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4487 GTM_gtm_cls0.MCS0_MEM[4487].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4488 GTM_gtm_cls0.MCS0_MEM[4488].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4489 GTM_gtm_cls0.MCS0_MEM[4489].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4490 GTM_gtm_cls0.MCS0_MEM[4490].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4491 GTM_gtm_cls0.MCS0_MEM[4491].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4492 GTM_gtm_cls0.MCS0_MEM[4492].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4493 GTM_gtm_cls0.MCS0_MEM[4493].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4494 GTM_gtm_cls0.MCS0_MEM[4494].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4495 GTM_gtm_cls0.MCS0_MEM[4495].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4496 GTM_gtm_cls0.MCS0_MEM[4496].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4497 GTM_gtm_cls0.MCS0_MEM[4497].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4498 GTM_gtm_cls0.MCS0_MEM[4498].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4499 GTM_gtm_cls0.MCS0_MEM[4499].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4500 GTM_gtm_cls0.MCS0_MEM[4500].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4501 GTM_gtm_cls0.MCS0_MEM[4501].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4502 GTM_gtm_cls0.MCS0_MEM[4502].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4503 GTM_gtm_cls0.MCS0_MEM[4503].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4504 GTM_gtm_cls0.MCS0_MEM[4504].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4505 GTM_gtm_cls0.MCS0_MEM[4505].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4506 GTM_gtm_cls0.MCS0_MEM[4506].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4507 GTM_gtm_cls0.MCS0_MEM[4507].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4508 GTM_gtm_cls0.MCS0_MEM[4508].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4509 GTM_gtm_cls0.MCS0_MEM[4509].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4510 GTM_gtm_cls0.MCS0_MEM[4510].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4511 GTM_gtm_cls0.MCS0_MEM[4511].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4512 GTM_gtm_cls0.MCS0_MEM[4512].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4513 GTM_gtm_cls0.MCS0_MEM[4513].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4514 GTM_gtm_cls0.MCS0_MEM[4514].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4515 GTM_gtm_cls0.MCS0_MEM[4515].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4516 GTM_gtm_cls0.MCS0_MEM[4516].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4517 GTM_gtm_cls0.MCS0_MEM[4517].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4518 GTM_gtm_cls0.MCS0_MEM[4518].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4519 GTM_gtm_cls0.MCS0_MEM[4519].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4520 GTM_gtm_cls0.MCS0_MEM[4520].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4521 GTM_gtm_cls0.MCS0_MEM[4521].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4522 GTM_gtm_cls0.MCS0_MEM[4522].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4523 GTM_gtm_cls0.MCS0_MEM[4523].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4524 GTM_gtm_cls0.MCS0_MEM[4524].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4525 GTM_gtm_cls0.MCS0_MEM[4525].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4526 GTM_gtm_cls0.MCS0_MEM[4526].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4527 GTM_gtm_cls0.MCS0_MEM[4527].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4528 GTM_gtm_cls0.MCS0_MEM[4528].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4529 GTM_gtm_cls0.MCS0_MEM[4529].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4530 GTM_gtm_cls0.MCS0_MEM[4530].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4531 GTM_gtm_cls0.MCS0_MEM[4531].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4532 GTM_gtm_cls0.MCS0_MEM[4532].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4533 GTM_gtm_cls0.MCS0_MEM[4533].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4534 GTM_gtm_cls0.MCS0_MEM[4534].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4535 GTM_gtm_cls0.MCS0_MEM[4535].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4536 GTM_gtm_cls0.MCS0_MEM[4536].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4537 GTM_gtm_cls0.MCS0_MEM[4537].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4538 GTM_gtm_cls0.MCS0_MEM[4538].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4539 GTM_gtm_cls0.MCS0_MEM[4539].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4540 GTM_gtm_cls0.MCS0_MEM[4540].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4541 GTM_gtm_cls0.MCS0_MEM[4541].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4542 GTM_gtm_cls0.MCS0_MEM[4542].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4543 GTM_gtm_cls0.MCS0_MEM[4543].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4544 GTM_gtm_cls0.MCS0_MEM[4544].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4545 GTM_gtm_cls0.MCS0_MEM[4545].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4546 GTM_gtm_cls0.MCS0_MEM[4546].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4547 GTM_gtm_cls0.MCS0_MEM[4547].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4548 GTM_gtm_cls0.MCS0_MEM[4548].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4549 GTM_gtm_cls0.MCS0_MEM[4549].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4550 GTM_gtm_cls0.MCS0_MEM[4550].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4551 GTM_gtm_cls0.MCS0_MEM[4551].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4552 GTM_gtm_cls0.MCS0_MEM[4552].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4553 GTM_gtm_cls0.MCS0_MEM[4553].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4554 GTM_gtm_cls0.MCS0_MEM[4554].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4555 GTM_gtm_cls0.MCS0_MEM[4555].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4556 GTM_gtm_cls0.MCS0_MEM[4556].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4557 GTM_gtm_cls0.MCS0_MEM[4557].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4558 GTM_gtm_cls0.MCS0_MEM[4558].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4559 GTM_gtm_cls0.MCS0_MEM[4559].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4560 GTM_gtm_cls0.MCS0_MEM[4560].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4561 GTM_gtm_cls0.MCS0_MEM[4561].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4562 GTM_gtm_cls0.MCS0_MEM[4562].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4563 GTM_gtm_cls0.MCS0_MEM[4563].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4564 GTM_gtm_cls0.MCS0_MEM[4564].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4565 GTM_gtm_cls0.MCS0_MEM[4565].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4566 GTM_gtm_cls0.MCS0_MEM[4566].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4567 GTM_gtm_cls0.MCS0_MEM[4567].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4568 GTM_gtm_cls0.MCS0_MEM[4568].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4569 GTM_gtm_cls0.MCS0_MEM[4569].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4570 GTM_gtm_cls0.MCS0_MEM[4570].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4571 GTM_gtm_cls0.MCS0_MEM[4571].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4572 GTM_gtm_cls0.MCS0_MEM[4572].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4573 GTM_gtm_cls0.MCS0_MEM[4573].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4574 GTM_gtm_cls0.MCS0_MEM[4574].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4575 GTM_gtm_cls0.MCS0_MEM[4575].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4576 GTM_gtm_cls0.MCS0_MEM[4576].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4577 GTM_gtm_cls0.MCS0_MEM[4577].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4578 GTM_gtm_cls0.MCS0_MEM[4578].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4579 GTM_gtm_cls0.MCS0_MEM[4579].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4580 GTM_gtm_cls0.MCS0_MEM[4580].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4581 GTM_gtm_cls0.MCS0_MEM[4581].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4582 GTM_gtm_cls0.MCS0_MEM[4582].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4583 GTM_gtm_cls0.MCS0_MEM[4583].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4584 GTM_gtm_cls0.MCS0_MEM[4584].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4585 GTM_gtm_cls0.MCS0_MEM[4585].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4586 GTM_gtm_cls0.MCS0_MEM[4586].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4587 GTM_gtm_cls0.MCS0_MEM[4587].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4588 GTM_gtm_cls0.MCS0_MEM[4588].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4589 GTM_gtm_cls0.MCS0_MEM[4589].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4590 GTM_gtm_cls0.MCS0_MEM[4590].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4591 GTM_gtm_cls0.MCS0_MEM[4591].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4592 GTM_gtm_cls0.MCS0_MEM[4592].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4593 GTM_gtm_cls0.MCS0_MEM[4593].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4594 GTM_gtm_cls0.MCS0_MEM[4594].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4595 GTM_gtm_cls0.MCS0_MEM[4595].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4596 GTM_gtm_cls0.MCS0_MEM[4596].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4597 GTM_gtm_cls0.MCS0_MEM[4597].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4598 GTM_gtm_cls0.MCS0_MEM[4598].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4599 GTM_gtm_cls0.MCS0_MEM[4599].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4600 GTM_gtm_cls0.MCS0_MEM[4600].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4601 GTM_gtm_cls0.MCS0_MEM[4601].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4602 GTM_gtm_cls0.MCS0_MEM[4602].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4603 GTM_gtm_cls0.MCS0_MEM[4603].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4604 GTM_gtm_cls0.MCS0_MEM[4604].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4605 GTM_gtm_cls0.MCS0_MEM[4605].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4606 GTM_gtm_cls0.MCS0_MEM[4606].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4607 GTM_gtm_cls0.MCS0_MEM[4607].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4608 GTM_gtm_cls0.MCS0_MEM[4608].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4609 GTM_gtm_cls0.MCS0_MEM[4609].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4610 GTM_gtm_cls0.MCS0_MEM[4610].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4611 GTM_gtm_cls0.MCS0_MEM[4611].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4612 GTM_gtm_cls0.MCS0_MEM[4612].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4613 GTM_gtm_cls0.MCS0_MEM[4613].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4614 GTM_gtm_cls0.MCS0_MEM[4614].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4615 GTM_gtm_cls0.MCS0_MEM[4615].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4616 GTM_gtm_cls0.MCS0_MEM[4616].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4617 GTM_gtm_cls0.MCS0_MEM[4617].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4618 GTM_gtm_cls0.MCS0_MEM[4618].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4619 GTM_gtm_cls0.MCS0_MEM[4619].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4620 GTM_gtm_cls0.MCS0_MEM[4620].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4621 GTM_gtm_cls0.MCS0_MEM[4621].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4622 GTM_gtm_cls0.MCS0_MEM[4622].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4623 GTM_gtm_cls0.MCS0_MEM[4623].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4624 GTM_gtm_cls0.MCS0_MEM[4624].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4625 GTM_gtm_cls0.MCS0_MEM[4625].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4626 GTM_gtm_cls0.MCS0_MEM[4626].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4627 GTM_gtm_cls0.MCS0_MEM[4627].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4628 GTM_gtm_cls0.MCS0_MEM[4628].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4629 GTM_gtm_cls0.MCS0_MEM[4629].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4630 GTM_gtm_cls0.MCS0_MEM[4630].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4631 GTM_gtm_cls0.MCS0_MEM[4631].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4632 GTM_gtm_cls0.MCS0_MEM[4632].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4633 GTM_gtm_cls0.MCS0_MEM[4633].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4634 GTM_gtm_cls0.MCS0_MEM[4634].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4635 GTM_gtm_cls0.MCS0_MEM[4635].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4636 GTM_gtm_cls0.MCS0_MEM[4636].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4637 GTM_gtm_cls0.MCS0_MEM[4637].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4638 GTM_gtm_cls0.MCS0_MEM[4638].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4639 GTM_gtm_cls0.MCS0_MEM[4639].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4640 GTM_gtm_cls0.MCS0_MEM[4640].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4641 GTM_gtm_cls0.MCS0_MEM[4641].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4642 GTM_gtm_cls0.MCS0_MEM[4642].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4643 GTM_gtm_cls0.MCS0_MEM[4643].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4644 GTM_gtm_cls0.MCS0_MEM[4644].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4645 GTM_gtm_cls0.MCS0_MEM[4645].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4646 GTM_gtm_cls0.MCS0_MEM[4646].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4647 GTM_gtm_cls0.MCS0_MEM[4647].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4648 GTM_gtm_cls0.MCS0_MEM[4648].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4649 GTM_gtm_cls0.MCS0_MEM[4649].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4650 GTM_gtm_cls0.MCS0_MEM[4650].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4651 GTM_gtm_cls0.MCS0_MEM[4651].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4652 GTM_gtm_cls0.MCS0_MEM[4652].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4653 GTM_gtm_cls0.MCS0_MEM[4653].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4654 GTM_gtm_cls0.MCS0_MEM[4654].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4655 GTM_gtm_cls0.MCS0_MEM[4655].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4656 GTM_gtm_cls0.MCS0_MEM[4656].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4657 GTM_gtm_cls0.MCS0_MEM[4657].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4658 GTM_gtm_cls0.MCS0_MEM[4658].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4659 GTM_gtm_cls0.MCS0_MEM[4659].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4660 GTM_gtm_cls0.MCS0_MEM[4660].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4661 GTM_gtm_cls0.MCS0_MEM[4661].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4662 GTM_gtm_cls0.MCS0_MEM[4662].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4663 GTM_gtm_cls0.MCS0_MEM[4663].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4664 GTM_gtm_cls0.MCS0_MEM[4664].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4665 GTM_gtm_cls0.MCS0_MEM[4665].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4666 GTM_gtm_cls0.MCS0_MEM[4666].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4667 GTM_gtm_cls0.MCS0_MEM[4667].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4668 GTM_gtm_cls0.MCS0_MEM[4668].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4669 GTM_gtm_cls0.MCS0_MEM[4669].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4670 GTM_gtm_cls0.MCS0_MEM[4670].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4671 GTM_gtm_cls0.MCS0_MEM[4671].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4672 GTM_gtm_cls0.MCS0_MEM[4672].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4673 GTM_gtm_cls0.MCS0_MEM[4673].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4674 GTM_gtm_cls0.MCS0_MEM[4674].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4675 GTM_gtm_cls0.MCS0_MEM[4675].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4676 GTM_gtm_cls0.MCS0_MEM[4676].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4677 GTM_gtm_cls0.MCS0_MEM[4677].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4678 GTM_gtm_cls0.MCS0_MEM[4678].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4679 GTM_gtm_cls0.MCS0_MEM[4679].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4680 GTM_gtm_cls0.MCS0_MEM[4680].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4681 GTM_gtm_cls0.MCS0_MEM[4681].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4682 GTM_gtm_cls0.MCS0_MEM[4682].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4683 GTM_gtm_cls0.MCS0_MEM[4683].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4684 GTM_gtm_cls0.MCS0_MEM[4684].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4685 GTM_gtm_cls0.MCS0_MEM[4685].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4686 GTM_gtm_cls0.MCS0_MEM[4686].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4687 GTM_gtm_cls0.MCS0_MEM[4687].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4688 GTM_gtm_cls0.MCS0_MEM[4688].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4689 GTM_gtm_cls0.MCS0_MEM[4689].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4690 GTM_gtm_cls0.MCS0_MEM[4690].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4691 GTM_gtm_cls0.MCS0_MEM[4691].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4692 GTM_gtm_cls0.MCS0_MEM[4692].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4693 GTM_gtm_cls0.MCS0_MEM[4693].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4694 GTM_gtm_cls0.MCS0_MEM[4694].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4695 GTM_gtm_cls0.MCS0_MEM[4695].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4696 GTM_gtm_cls0.MCS0_MEM[4696].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4697 GTM_gtm_cls0.MCS0_MEM[4697].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4698 GTM_gtm_cls0.MCS0_MEM[4698].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4699 GTM_gtm_cls0.MCS0_MEM[4699].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4700 GTM_gtm_cls0.MCS0_MEM[4700].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4701 GTM_gtm_cls0.MCS0_MEM[4701].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4702 GTM_gtm_cls0.MCS0_MEM[4702].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4703 GTM_gtm_cls0.MCS0_MEM[4703].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4704 GTM_gtm_cls0.MCS0_MEM[4704].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4705 GTM_gtm_cls0.MCS0_MEM[4705].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4706 GTM_gtm_cls0.MCS0_MEM[4706].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4707 GTM_gtm_cls0.MCS0_MEM[4707].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4708 GTM_gtm_cls0.MCS0_MEM[4708].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4709 GTM_gtm_cls0.MCS0_MEM[4709].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4710 GTM_gtm_cls0.MCS0_MEM[4710].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4711 GTM_gtm_cls0.MCS0_MEM[4711].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4712 GTM_gtm_cls0.MCS0_MEM[4712].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4713 GTM_gtm_cls0.MCS0_MEM[4713].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4714 GTM_gtm_cls0.MCS0_MEM[4714].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4715 GTM_gtm_cls0.MCS0_MEM[4715].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4716 GTM_gtm_cls0.MCS0_MEM[4716].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4717 GTM_gtm_cls0.MCS0_MEM[4717].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4718 GTM_gtm_cls0.MCS0_MEM[4718].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4719 GTM_gtm_cls0.MCS0_MEM[4719].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4720 GTM_gtm_cls0.MCS0_MEM[4720].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4721 GTM_gtm_cls0.MCS0_MEM[4721].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4722 GTM_gtm_cls0.MCS0_MEM[4722].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4723 GTM_gtm_cls0.MCS0_MEM[4723].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4724 GTM_gtm_cls0.MCS0_MEM[4724].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4725 GTM_gtm_cls0.MCS0_MEM[4725].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4726 GTM_gtm_cls0.MCS0_MEM[4726].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4727 GTM_gtm_cls0.MCS0_MEM[4727].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4728 GTM_gtm_cls0.MCS0_MEM[4728].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4729 GTM_gtm_cls0.MCS0_MEM[4729].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4730 GTM_gtm_cls0.MCS0_MEM[4730].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4731 GTM_gtm_cls0.MCS0_MEM[4731].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4732 GTM_gtm_cls0.MCS0_MEM[4732].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4733 GTM_gtm_cls0.MCS0_MEM[4733].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4734 GTM_gtm_cls0.MCS0_MEM[4734].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4735 GTM_gtm_cls0.MCS0_MEM[4735].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4736 GTM_gtm_cls0.MCS0_MEM[4736].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4737 GTM_gtm_cls0.MCS0_MEM[4737].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4738 GTM_gtm_cls0.MCS0_MEM[4738].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4739 GTM_gtm_cls0.MCS0_MEM[4739].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4740 GTM_gtm_cls0.MCS0_MEM[4740].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4741 GTM_gtm_cls0.MCS0_MEM[4741].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4742 GTM_gtm_cls0.MCS0_MEM[4742].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4743 GTM_gtm_cls0.MCS0_MEM[4743].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4744 GTM_gtm_cls0.MCS0_MEM[4744].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4745 GTM_gtm_cls0.MCS0_MEM[4745].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4746 GTM_gtm_cls0.MCS0_MEM[4746].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4747 GTM_gtm_cls0.MCS0_MEM[4747].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4748 GTM_gtm_cls0.MCS0_MEM[4748].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4749 GTM_gtm_cls0.MCS0_MEM[4749].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4750 GTM_gtm_cls0.MCS0_MEM[4750].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4751 GTM_gtm_cls0.MCS0_MEM[4751].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4752 GTM_gtm_cls0.MCS0_MEM[4752].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4753 GTM_gtm_cls0.MCS0_MEM[4753].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4754 GTM_gtm_cls0.MCS0_MEM[4754].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4755 GTM_gtm_cls0.MCS0_MEM[4755].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4756 GTM_gtm_cls0.MCS0_MEM[4756].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4757 GTM_gtm_cls0.MCS0_MEM[4757].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4758 GTM_gtm_cls0.MCS0_MEM[4758].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4759 GTM_gtm_cls0.MCS0_MEM[4759].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4760 GTM_gtm_cls0.MCS0_MEM[4760].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4761 GTM_gtm_cls0.MCS0_MEM[4761].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4762 GTM_gtm_cls0.MCS0_MEM[4762].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4763 GTM_gtm_cls0.MCS0_MEM[4763].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4764 GTM_gtm_cls0.MCS0_MEM[4764].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4765 GTM_gtm_cls0.MCS0_MEM[4765].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4766 GTM_gtm_cls0.MCS0_MEM[4766].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4767 GTM_gtm_cls0.MCS0_MEM[4767].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4768 GTM_gtm_cls0.MCS0_MEM[4768].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4769 GTM_gtm_cls0.MCS0_MEM[4769].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4770 GTM_gtm_cls0.MCS0_MEM[4770].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4771 GTM_gtm_cls0.MCS0_MEM[4771].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4772 GTM_gtm_cls0.MCS0_MEM[4772].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4773 GTM_gtm_cls0.MCS0_MEM[4773].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4774 GTM_gtm_cls0.MCS0_MEM[4774].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4775 GTM_gtm_cls0.MCS0_MEM[4775].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4776 GTM_gtm_cls0.MCS0_MEM[4776].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4777 GTM_gtm_cls0.MCS0_MEM[4777].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4778 GTM_gtm_cls0.MCS0_MEM[4778].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4779 GTM_gtm_cls0.MCS0_MEM[4779].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4780 GTM_gtm_cls0.MCS0_MEM[4780].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4781 GTM_gtm_cls0.MCS0_MEM[4781].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4782 GTM_gtm_cls0.MCS0_MEM[4782].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4783 GTM_gtm_cls0.MCS0_MEM[4783].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4784 GTM_gtm_cls0.MCS0_MEM[4784].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4785 GTM_gtm_cls0.MCS0_MEM[4785].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4786 GTM_gtm_cls0.MCS0_MEM[4786].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4787 GTM_gtm_cls0.MCS0_MEM[4787].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4788 GTM_gtm_cls0.MCS0_MEM[4788].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4789 GTM_gtm_cls0.MCS0_MEM[4789].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4790 GTM_gtm_cls0.MCS0_MEM[4790].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4791 GTM_gtm_cls0.MCS0_MEM[4791].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4792 GTM_gtm_cls0.MCS0_MEM[4792].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4793 GTM_gtm_cls0.MCS0_MEM[4793].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4794 GTM_gtm_cls0.MCS0_MEM[4794].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4795 GTM_gtm_cls0.MCS0_MEM[4795].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4796 GTM_gtm_cls0.MCS0_MEM[4796].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4797 GTM_gtm_cls0.MCS0_MEM[4797].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4798 GTM_gtm_cls0.MCS0_MEM[4798].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4799 GTM_gtm_cls0.MCS0_MEM[4799].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4800 GTM_gtm_cls0.MCS0_MEM[4800].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4801 GTM_gtm_cls0.MCS0_MEM[4801].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4802 GTM_gtm_cls0.MCS0_MEM[4802].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4803 GTM_gtm_cls0.MCS0_MEM[4803].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4804 GTM_gtm_cls0.MCS0_MEM[4804].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4805 GTM_gtm_cls0.MCS0_MEM[4805].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4806 GTM_gtm_cls0.MCS0_MEM[4806].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4807 GTM_gtm_cls0.MCS0_MEM[4807].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4808 GTM_gtm_cls0.MCS0_MEM[4808].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4809 GTM_gtm_cls0.MCS0_MEM[4809].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4810 GTM_gtm_cls0.MCS0_MEM[4810].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4811 GTM_gtm_cls0.MCS0_MEM[4811].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4812 GTM_gtm_cls0.MCS0_MEM[4812].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4813 GTM_gtm_cls0.MCS0_MEM[4813].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4814 GTM_gtm_cls0.MCS0_MEM[4814].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4815 GTM_gtm_cls0.MCS0_MEM[4815].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4816 GTM_gtm_cls0.MCS0_MEM[4816].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4817 GTM_gtm_cls0.MCS0_MEM[4817].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4818 GTM_gtm_cls0.MCS0_MEM[4818].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4819 GTM_gtm_cls0.MCS0_MEM[4819].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4820 GTM_gtm_cls0.MCS0_MEM[4820].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4821 GTM_gtm_cls0.MCS0_MEM[4821].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4822 GTM_gtm_cls0.MCS0_MEM[4822].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4823 GTM_gtm_cls0.MCS0_MEM[4823].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4824 GTM_gtm_cls0.MCS0_MEM[4824].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4825 GTM_gtm_cls0.MCS0_MEM[4825].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4826 GTM_gtm_cls0.MCS0_MEM[4826].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4827 GTM_gtm_cls0.MCS0_MEM[4827].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4828 GTM_gtm_cls0.MCS0_MEM[4828].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4829 GTM_gtm_cls0.MCS0_MEM[4829].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4830 GTM_gtm_cls0.MCS0_MEM[4830].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4831 GTM_gtm_cls0.MCS0_MEM[4831].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4832 GTM_gtm_cls0.MCS0_MEM[4832].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4833 GTM_gtm_cls0.MCS0_MEM[4833].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4834 GTM_gtm_cls0.MCS0_MEM[4834].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4835 GTM_gtm_cls0.MCS0_MEM[4835].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4836 GTM_gtm_cls0.MCS0_MEM[4836].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4837 GTM_gtm_cls0.MCS0_MEM[4837].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4838 GTM_gtm_cls0.MCS0_MEM[4838].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4839 GTM_gtm_cls0.MCS0_MEM[4839].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4840 GTM_gtm_cls0.MCS0_MEM[4840].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4841 GTM_gtm_cls0.MCS0_MEM[4841].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4842 GTM_gtm_cls0.MCS0_MEM[4842].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4843 GTM_gtm_cls0.MCS0_MEM[4843].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4844 GTM_gtm_cls0.MCS0_MEM[4844].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4845 GTM_gtm_cls0.MCS0_MEM[4845].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4846 GTM_gtm_cls0.MCS0_MEM[4846].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4847 GTM_gtm_cls0.MCS0_MEM[4847].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4848 GTM_gtm_cls0.MCS0_MEM[4848].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4849 GTM_gtm_cls0.MCS0_MEM[4849].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4850 GTM_gtm_cls0.MCS0_MEM[4850].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4851 GTM_gtm_cls0.MCS0_MEM[4851].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4852 GTM_gtm_cls0.MCS0_MEM[4852].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4853 GTM_gtm_cls0.MCS0_MEM[4853].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4854 GTM_gtm_cls0.MCS0_MEM[4854].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4855 GTM_gtm_cls0.MCS0_MEM[4855].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4856 GTM_gtm_cls0.MCS0_MEM[4856].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4857 GTM_gtm_cls0.MCS0_MEM[4857].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4858 GTM_gtm_cls0.MCS0_MEM[4858].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4859 GTM_gtm_cls0.MCS0_MEM[4859].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4860 GTM_gtm_cls0.MCS0_MEM[4860].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4861 GTM_gtm_cls0.MCS0_MEM[4861].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4862 GTM_gtm_cls0.MCS0_MEM[4862].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4863 GTM_gtm_cls0.MCS0_MEM[4863].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4864 GTM_gtm_cls0.MCS0_MEM[4864].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4865 GTM_gtm_cls0.MCS0_MEM[4865].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4866 GTM_gtm_cls0.MCS0_MEM[4866].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4867 GTM_gtm_cls0.MCS0_MEM[4867].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4868 GTM_gtm_cls0.MCS0_MEM[4868].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4869 GTM_gtm_cls0.MCS0_MEM[4869].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4870 GTM_gtm_cls0.MCS0_MEM[4870].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4871 GTM_gtm_cls0.MCS0_MEM[4871].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4872 GTM_gtm_cls0.MCS0_MEM[4872].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4873 GTM_gtm_cls0.MCS0_MEM[4873].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4874 GTM_gtm_cls0.MCS0_MEM[4874].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4875 GTM_gtm_cls0.MCS0_MEM[4875].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4876 GTM_gtm_cls0.MCS0_MEM[4876].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4877 GTM_gtm_cls0.MCS0_MEM[4877].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4878 GTM_gtm_cls0.MCS0_MEM[4878].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4879 GTM_gtm_cls0.MCS0_MEM[4879].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4880 GTM_gtm_cls0.MCS0_MEM[4880].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4881 GTM_gtm_cls0.MCS0_MEM[4881].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4882 GTM_gtm_cls0.MCS0_MEM[4882].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4883 GTM_gtm_cls0.MCS0_MEM[4883].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4884 GTM_gtm_cls0.MCS0_MEM[4884].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4885 GTM_gtm_cls0.MCS0_MEM[4885].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4886 GTM_gtm_cls0.MCS0_MEM[4886].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4887 GTM_gtm_cls0.MCS0_MEM[4887].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4888 GTM_gtm_cls0.MCS0_MEM[4888].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4889 GTM_gtm_cls0.MCS0_MEM[4889].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4890 GTM_gtm_cls0.MCS0_MEM[4890].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4891 GTM_gtm_cls0.MCS0_MEM[4891].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4892 GTM_gtm_cls0.MCS0_MEM[4892].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4893 GTM_gtm_cls0.MCS0_MEM[4893].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4894 GTM_gtm_cls0.MCS0_MEM[4894].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4895 GTM_gtm_cls0.MCS0_MEM[4895].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4896 GTM_gtm_cls0.MCS0_MEM[4896].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4897 GTM_gtm_cls0.MCS0_MEM[4897].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4898 GTM_gtm_cls0.MCS0_MEM[4898].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4899 GTM_gtm_cls0.MCS0_MEM[4899].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4900 GTM_gtm_cls0.MCS0_MEM[4900].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4901 GTM_gtm_cls0.MCS0_MEM[4901].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4902 GTM_gtm_cls0.MCS0_MEM[4902].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4903 GTM_gtm_cls0.MCS0_MEM[4903].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4904 GTM_gtm_cls0.MCS0_MEM[4904].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4905 GTM_gtm_cls0.MCS0_MEM[4905].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4906 GTM_gtm_cls0.MCS0_MEM[4906].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4907 GTM_gtm_cls0.MCS0_MEM[4907].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4908 GTM_gtm_cls0.MCS0_MEM[4908].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4909 GTM_gtm_cls0.MCS0_MEM[4909].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4910 GTM_gtm_cls0.MCS0_MEM[4910].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4911 GTM_gtm_cls0.MCS0_MEM[4911].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4912 GTM_gtm_cls0.MCS0_MEM[4912].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4913 GTM_gtm_cls0.MCS0_MEM[4913].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4914 GTM_gtm_cls0.MCS0_MEM[4914].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4915 GTM_gtm_cls0.MCS0_MEM[4915].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4916 GTM_gtm_cls0.MCS0_MEM[4916].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4917 GTM_gtm_cls0.MCS0_MEM[4917].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4918 GTM_gtm_cls0.MCS0_MEM[4918].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4919 GTM_gtm_cls0.MCS0_MEM[4919].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4920 GTM_gtm_cls0.MCS0_MEM[4920].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4921 GTM_gtm_cls0.MCS0_MEM[4921].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4922 GTM_gtm_cls0.MCS0_MEM[4922].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4923 GTM_gtm_cls0.MCS0_MEM[4923].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4924 GTM_gtm_cls0.MCS0_MEM[4924].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4925 GTM_gtm_cls0.MCS0_MEM[4925].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4926 GTM_gtm_cls0.MCS0_MEM[4926].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4927 GTM_gtm_cls0.MCS0_MEM[4927].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4928 GTM_gtm_cls0.MCS0_MEM[4928].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4929 GTM_gtm_cls0.MCS0_MEM[4929].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4930 GTM_gtm_cls0.MCS0_MEM[4930].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4931 GTM_gtm_cls0.MCS0_MEM[4931].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4932 GTM_gtm_cls0.MCS0_MEM[4932].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4933 GTM_gtm_cls0.MCS0_MEM[4933].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4934 GTM_gtm_cls0.MCS0_MEM[4934].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4935 GTM_gtm_cls0.MCS0_MEM[4935].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4936 GTM_gtm_cls0.MCS0_MEM[4936].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4937 GTM_gtm_cls0.MCS0_MEM[4937].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4938 GTM_gtm_cls0.MCS0_MEM[4938].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4939 GTM_gtm_cls0.MCS0_MEM[4939].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4940 GTM_gtm_cls0.MCS0_MEM[4940].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4941 GTM_gtm_cls0.MCS0_MEM[4941].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4942 GTM_gtm_cls0.MCS0_MEM[4942].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4943 GTM_gtm_cls0.MCS0_MEM[4943].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4944 GTM_gtm_cls0.MCS0_MEM[4944].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4945 GTM_gtm_cls0.MCS0_MEM[4945].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4946 GTM_gtm_cls0.MCS0_MEM[4946].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4947 GTM_gtm_cls0.MCS0_MEM[4947].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4948 GTM_gtm_cls0.MCS0_MEM[4948].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4949 GTM_gtm_cls0.MCS0_MEM[4949].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4950 GTM_gtm_cls0.MCS0_MEM[4950].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4951 GTM_gtm_cls0.MCS0_MEM[4951].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4952 GTM_gtm_cls0.MCS0_MEM[4952].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4953 GTM_gtm_cls0.MCS0_MEM[4953].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4954 GTM_gtm_cls0.MCS0_MEM[4954].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4955 GTM_gtm_cls0.MCS0_MEM[4955].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4956 GTM_gtm_cls0.MCS0_MEM[4956].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4957 GTM_gtm_cls0.MCS0_MEM[4957].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4958 GTM_gtm_cls0.MCS0_MEM[4958].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4959 GTM_gtm_cls0.MCS0_MEM[4959].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4960 GTM_gtm_cls0.MCS0_MEM[4960].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4961 GTM_gtm_cls0.MCS0_MEM[4961].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4962 GTM_gtm_cls0.MCS0_MEM[4962].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4963 GTM_gtm_cls0.MCS0_MEM[4963].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4964 GTM_gtm_cls0.MCS0_MEM[4964].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4965 GTM_gtm_cls0.MCS0_MEM[4965].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4966 GTM_gtm_cls0.MCS0_MEM[4966].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4967 GTM_gtm_cls0.MCS0_MEM[4967].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4968 GTM_gtm_cls0.MCS0_MEM[4968].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4969 GTM_gtm_cls0.MCS0_MEM[4969].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4970 GTM_gtm_cls0.MCS0_MEM[4970].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4971 GTM_gtm_cls0.MCS0_MEM[4971].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4972 GTM_gtm_cls0.MCS0_MEM[4972].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4973 GTM_gtm_cls0.MCS0_MEM[4973].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4974 GTM_gtm_cls0.MCS0_MEM[4974].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4975 GTM_gtm_cls0.MCS0_MEM[4975].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4976 GTM_gtm_cls0.MCS0_MEM[4976].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4977 GTM_gtm_cls0.MCS0_MEM[4977].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4978 GTM_gtm_cls0.MCS0_MEM[4978].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4979 GTM_gtm_cls0.MCS0_MEM[4979].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4980 GTM_gtm_cls0.MCS0_MEM[4980].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4981 GTM_gtm_cls0.MCS0_MEM[4981].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4982 GTM_gtm_cls0.MCS0_MEM[4982].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4983 GTM_gtm_cls0.MCS0_MEM[4983].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4984 GTM_gtm_cls0.MCS0_MEM[4984].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4985 GTM_gtm_cls0.MCS0_MEM[4985].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4986 GTM_gtm_cls0.MCS0_MEM[4986].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4987 GTM_gtm_cls0.MCS0_MEM[4987].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4988 GTM_gtm_cls0.MCS0_MEM[4988].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4989 GTM_gtm_cls0.MCS0_MEM[4989].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4990 GTM_gtm_cls0.MCS0_MEM[4990].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4991 GTM_gtm_cls0.MCS0_MEM[4991].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4992 GTM_gtm_cls0.MCS0_MEM[4992].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4993 GTM_gtm_cls0.MCS0_MEM[4993].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4994 GTM_gtm_cls0.MCS0_MEM[4994].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4995 GTM_gtm_cls0.MCS0_MEM[4995].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4996 GTM_gtm_cls0.MCS0_MEM[4996].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4997 GTM_gtm_cls0.MCS0_MEM[4997].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4998 GTM_gtm_cls0.MCS0_MEM[4998].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM4999 GTM_gtm_cls0.MCS0_MEM[4999].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5000 GTM_gtm_cls0.MCS0_MEM[5000].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5001 GTM_gtm_cls0.MCS0_MEM[5001].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5002 GTM_gtm_cls0.MCS0_MEM[5002].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5003 GTM_gtm_cls0.MCS0_MEM[5003].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5004 GTM_gtm_cls0.MCS0_MEM[5004].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5005 GTM_gtm_cls0.MCS0_MEM[5005].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5006 GTM_gtm_cls0.MCS0_MEM[5006].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5007 GTM_gtm_cls0.MCS0_MEM[5007].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5008 GTM_gtm_cls0.MCS0_MEM[5008].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5009 GTM_gtm_cls0.MCS0_MEM[5009].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5010 GTM_gtm_cls0.MCS0_MEM[5010].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5011 GTM_gtm_cls0.MCS0_MEM[5011].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5012 GTM_gtm_cls0.MCS0_MEM[5012].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5013 GTM_gtm_cls0.MCS0_MEM[5013].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5014 GTM_gtm_cls0.MCS0_MEM[5014].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5015 GTM_gtm_cls0.MCS0_MEM[5015].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5016 GTM_gtm_cls0.MCS0_MEM[5016].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5017 GTM_gtm_cls0.MCS0_MEM[5017].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5018 GTM_gtm_cls0.MCS0_MEM[5018].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5019 GTM_gtm_cls0.MCS0_MEM[5019].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5020 GTM_gtm_cls0.MCS0_MEM[5020].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5021 GTM_gtm_cls0.MCS0_MEM[5021].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5022 GTM_gtm_cls0.MCS0_MEM[5022].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5023 GTM_gtm_cls0.MCS0_MEM[5023].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5024 GTM_gtm_cls0.MCS0_MEM[5024].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5025 GTM_gtm_cls0.MCS0_MEM[5025].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5026 GTM_gtm_cls0.MCS0_MEM[5026].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5027 GTM_gtm_cls0.MCS0_MEM[5027].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5028 GTM_gtm_cls0.MCS0_MEM[5028].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5029 GTM_gtm_cls0.MCS0_MEM[5029].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5030 GTM_gtm_cls0.MCS0_MEM[5030].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5031 GTM_gtm_cls0.MCS0_MEM[5031].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5032 GTM_gtm_cls0.MCS0_MEM[5032].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5033 GTM_gtm_cls0.MCS0_MEM[5033].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5034 GTM_gtm_cls0.MCS0_MEM[5034].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5035 GTM_gtm_cls0.MCS0_MEM[5035].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5036 GTM_gtm_cls0.MCS0_MEM[5036].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5037 GTM_gtm_cls0.MCS0_MEM[5037].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5038 GTM_gtm_cls0.MCS0_MEM[5038].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5039 GTM_gtm_cls0.MCS0_MEM[5039].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5040 GTM_gtm_cls0.MCS0_MEM[5040].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5041 GTM_gtm_cls0.MCS0_MEM[5041].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5042 GTM_gtm_cls0.MCS0_MEM[5042].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5043 GTM_gtm_cls0.MCS0_MEM[5043].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5044 GTM_gtm_cls0.MCS0_MEM[5044].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5045 GTM_gtm_cls0.MCS0_MEM[5045].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5046 GTM_gtm_cls0.MCS0_MEM[5046].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5047 GTM_gtm_cls0.MCS0_MEM[5047].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5048 GTM_gtm_cls0.MCS0_MEM[5048].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5049 GTM_gtm_cls0.MCS0_MEM[5049].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5050 GTM_gtm_cls0.MCS0_MEM[5050].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5051 GTM_gtm_cls0.MCS0_MEM[5051].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5052 GTM_gtm_cls0.MCS0_MEM[5052].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5053 GTM_gtm_cls0.MCS0_MEM[5053].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5054 GTM_gtm_cls0.MCS0_MEM[5054].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5055 GTM_gtm_cls0.MCS0_MEM[5055].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5056 GTM_gtm_cls0.MCS0_MEM[5056].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5057 GTM_gtm_cls0.MCS0_MEM[5057].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5058 GTM_gtm_cls0.MCS0_MEM[5058].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5059 GTM_gtm_cls0.MCS0_MEM[5059].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5060 GTM_gtm_cls0.MCS0_MEM[5060].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5061 GTM_gtm_cls0.MCS0_MEM[5061].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5062 GTM_gtm_cls0.MCS0_MEM[5062].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5063 GTM_gtm_cls0.MCS0_MEM[5063].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5064 GTM_gtm_cls0.MCS0_MEM[5064].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5065 GTM_gtm_cls0.MCS0_MEM[5065].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5066 GTM_gtm_cls0.MCS0_MEM[5066].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5067 GTM_gtm_cls0.MCS0_MEM[5067].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5068 GTM_gtm_cls0.MCS0_MEM[5068].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5069 GTM_gtm_cls0.MCS0_MEM[5069].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5070 GTM_gtm_cls0.MCS0_MEM[5070].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5071 GTM_gtm_cls0.MCS0_MEM[5071].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5072 GTM_gtm_cls0.MCS0_MEM[5072].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5073 GTM_gtm_cls0.MCS0_MEM[5073].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5074 GTM_gtm_cls0.MCS0_MEM[5074].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5075 GTM_gtm_cls0.MCS0_MEM[5075].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5076 GTM_gtm_cls0.MCS0_MEM[5076].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5077 GTM_gtm_cls0.MCS0_MEM[5077].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5078 GTM_gtm_cls0.MCS0_MEM[5078].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5079 GTM_gtm_cls0.MCS0_MEM[5079].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5080 GTM_gtm_cls0.MCS0_MEM[5080].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5081 GTM_gtm_cls0.MCS0_MEM[5081].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5082 GTM_gtm_cls0.MCS0_MEM[5082].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5083 GTM_gtm_cls0.MCS0_MEM[5083].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5084 GTM_gtm_cls0.MCS0_MEM[5084].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5085 GTM_gtm_cls0.MCS0_MEM[5085].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5086 GTM_gtm_cls0.MCS0_MEM[5086].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5087 GTM_gtm_cls0.MCS0_MEM[5087].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5088 GTM_gtm_cls0.MCS0_MEM[5088].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5089 GTM_gtm_cls0.MCS0_MEM[5089].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5090 GTM_gtm_cls0.MCS0_MEM[5090].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5091 GTM_gtm_cls0.MCS0_MEM[5091].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5092 GTM_gtm_cls0.MCS0_MEM[5092].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5093 GTM_gtm_cls0.MCS0_MEM[5093].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5094 GTM_gtm_cls0.MCS0_MEM[5094].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5095 GTM_gtm_cls0.MCS0_MEM[5095].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5096 GTM_gtm_cls0.MCS0_MEM[5096].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5097 GTM_gtm_cls0.MCS0_MEM[5097].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5098 GTM_gtm_cls0.MCS0_MEM[5098].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5099 GTM_gtm_cls0.MCS0_MEM[5099].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5100 GTM_gtm_cls0.MCS0_MEM[5100].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5101 GTM_gtm_cls0.MCS0_MEM[5101].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5102 GTM_gtm_cls0.MCS0_MEM[5102].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5103 GTM_gtm_cls0.MCS0_MEM[5103].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5104 GTM_gtm_cls0.MCS0_MEM[5104].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5105 GTM_gtm_cls0.MCS0_MEM[5105].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5106 GTM_gtm_cls0.MCS0_MEM[5106].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5107 GTM_gtm_cls0.MCS0_MEM[5107].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5108 GTM_gtm_cls0.MCS0_MEM[5108].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5109 GTM_gtm_cls0.MCS0_MEM[5109].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5110 GTM_gtm_cls0.MCS0_MEM[5110].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5111 GTM_gtm_cls0.MCS0_MEM[5111].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5112 GTM_gtm_cls0.MCS0_MEM[5112].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5113 GTM_gtm_cls0.MCS0_MEM[5113].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5114 GTM_gtm_cls0.MCS0_MEM[5114].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5115 GTM_gtm_cls0.MCS0_MEM[5115].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5116 GTM_gtm_cls0.MCS0_MEM[5116].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5117 GTM_gtm_cls0.MCS0_MEM[5117].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5118 GTM_gtm_cls0.MCS0_MEM[5118].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5119 GTM_gtm_cls0.MCS0_MEM[5119].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5120 GTM_gtm_cls0.MCS0_MEM[5120].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5121 GTM_gtm_cls0.MCS0_MEM[5121].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5122 GTM_gtm_cls0.MCS0_MEM[5122].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5123 GTM_gtm_cls0.MCS0_MEM[5123].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5124 GTM_gtm_cls0.MCS0_MEM[5124].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5125 GTM_gtm_cls0.MCS0_MEM[5125].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5126 GTM_gtm_cls0.MCS0_MEM[5126].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5127 GTM_gtm_cls0.MCS0_MEM[5127].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5128 GTM_gtm_cls0.MCS0_MEM[5128].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5129 GTM_gtm_cls0.MCS0_MEM[5129].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5130 GTM_gtm_cls0.MCS0_MEM[5130].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5131 GTM_gtm_cls0.MCS0_MEM[5131].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5132 GTM_gtm_cls0.MCS0_MEM[5132].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5133 GTM_gtm_cls0.MCS0_MEM[5133].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5134 GTM_gtm_cls0.MCS0_MEM[5134].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5135 GTM_gtm_cls0.MCS0_MEM[5135].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5136 GTM_gtm_cls0.MCS0_MEM[5136].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5137 GTM_gtm_cls0.MCS0_MEM[5137].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5138 GTM_gtm_cls0.MCS0_MEM[5138].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5139 GTM_gtm_cls0.MCS0_MEM[5139].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5140 GTM_gtm_cls0.MCS0_MEM[5140].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5141 GTM_gtm_cls0.MCS0_MEM[5141].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5142 GTM_gtm_cls0.MCS0_MEM[5142].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5143 GTM_gtm_cls0.MCS0_MEM[5143].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5144 GTM_gtm_cls0.MCS0_MEM[5144].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5145 GTM_gtm_cls0.MCS0_MEM[5145].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5146 GTM_gtm_cls0.MCS0_MEM[5146].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5147 GTM_gtm_cls0.MCS0_MEM[5147].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5148 GTM_gtm_cls0.MCS0_MEM[5148].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5149 GTM_gtm_cls0.MCS0_MEM[5149].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5150 GTM_gtm_cls0.MCS0_MEM[5150].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5151 GTM_gtm_cls0.MCS0_MEM[5151].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5152 GTM_gtm_cls0.MCS0_MEM[5152].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5153 GTM_gtm_cls0.MCS0_MEM[5153].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5154 GTM_gtm_cls0.MCS0_MEM[5154].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5155 GTM_gtm_cls0.MCS0_MEM[5155].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5156 GTM_gtm_cls0.MCS0_MEM[5156].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5157 GTM_gtm_cls0.MCS0_MEM[5157].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5158 GTM_gtm_cls0.MCS0_MEM[5158].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5159 GTM_gtm_cls0.MCS0_MEM[5159].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5160 GTM_gtm_cls0.MCS0_MEM[5160].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5161 GTM_gtm_cls0.MCS0_MEM[5161].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5162 GTM_gtm_cls0.MCS0_MEM[5162].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5163 GTM_gtm_cls0.MCS0_MEM[5163].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5164 GTM_gtm_cls0.MCS0_MEM[5164].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5165 GTM_gtm_cls0.MCS0_MEM[5165].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5166 GTM_gtm_cls0.MCS0_MEM[5166].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5167 GTM_gtm_cls0.MCS0_MEM[5167].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5168 GTM_gtm_cls0.MCS0_MEM[5168].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5169 GTM_gtm_cls0.MCS0_MEM[5169].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5170 GTM_gtm_cls0.MCS0_MEM[5170].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5171 GTM_gtm_cls0.MCS0_MEM[5171].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5172 GTM_gtm_cls0.MCS0_MEM[5172].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5173 GTM_gtm_cls0.MCS0_MEM[5173].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5174 GTM_gtm_cls0.MCS0_MEM[5174].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5175 GTM_gtm_cls0.MCS0_MEM[5175].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5176 GTM_gtm_cls0.MCS0_MEM[5176].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5177 GTM_gtm_cls0.MCS0_MEM[5177].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5178 GTM_gtm_cls0.MCS0_MEM[5178].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5179 GTM_gtm_cls0.MCS0_MEM[5179].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5180 GTM_gtm_cls0.MCS0_MEM[5180].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5181 GTM_gtm_cls0.MCS0_MEM[5181].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5182 GTM_gtm_cls0.MCS0_MEM[5182].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5183 GTM_gtm_cls0.MCS0_MEM[5183].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5184 GTM_gtm_cls0.MCS0_MEM[5184].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5185 GTM_gtm_cls0.MCS0_MEM[5185].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5186 GTM_gtm_cls0.MCS0_MEM[5186].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5187 GTM_gtm_cls0.MCS0_MEM[5187].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5188 GTM_gtm_cls0.MCS0_MEM[5188].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5189 GTM_gtm_cls0.MCS0_MEM[5189].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5190 GTM_gtm_cls0.MCS0_MEM[5190].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5191 GTM_gtm_cls0.MCS0_MEM[5191].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5192 GTM_gtm_cls0.MCS0_MEM[5192].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5193 GTM_gtm_cls0.MCS0_MEM[5193].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5194 GTM_gtm_cls0.MCS0_MEM[5194].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5195 GTM_gtm_cls0.MCS0_MEM[5195].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5196 GTM_gtm_cls0.MCS0_MEM[5196].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5197 GTM_gtm_cls0.MCS0_MEM[5197].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5198 GTM_gtm_cls0.MCS0_MEM[5198].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5199 GTM_gtm_cls0.MCS0_MEM[5199].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5200 GTM_gtm_cls0.MCS0_MEM[5200].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5201 GTM_gtm_cls0.MCS0_MEM[5201].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5202 GTM_gtm_cls0.MCS0_MEM[5202].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5203 GTM_gtm_cls0.MCS0_MEM[5203].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5204 GTM_gtm_cls0.MCS0_MEM[5204].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5205 GTM_gtm_cls0.MCS0_MEM[5205].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5206 GTM_gtm_cls0.MCS0_MEM[5206].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5207 GTM_gtm_cls0.MCS0_MEM[5207].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5208 GTM_gtm_cls0.MCS0_MEM[5208].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5209 GTM_gtm_cls0.MCS0_MEM[5209].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5210 GTM_gtm_cls0.MCS0_MEM[5210].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5211 GTM_gtm_cls0.MCS0_MEM[5211].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5212 GTM_gtm_cls0.MCS0_MEM[5212].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5213 GTM_gtm_cls0.MCS0_MEM[5213].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5214 GTM_gtm_cls0.MCS0_MEM[5214].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5215 GTM_gtm_cls0.MCS0_MEM[5215].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5216 GTM_gtm_cls0.MCS0_MEM[5216].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5217 GTM_gtm_cls0.MCS0_MEM[5217].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5218 GTM_gtm_cls0.MCS0_MEM[5218].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5219 GTM_gtm_cls0.MCS0_MEM[5219].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5220 GTM_gtm_cls0.MCS0_MEM[5220].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5221 GTM_gtm_cls0.MCS0_MEM[5221].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5222 GTM_gtm_cls0.MCS0_MEM[5222].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5223 GTM_gtm_cls0.MCS0_MEM[5223].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5224 GTM_gtm_cls0.MCS0_MEM[5224].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5225 GTM_gtm_cls0.MCS0_MEM[5225].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5226 GTM_gtm_cls0.MCS0_MEM[5226].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5227 GTM_gtm_cls0.MCS0_MEM[5227].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5228 GTM_gtm_cls0.MCS0_MEM[5228].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5229 GTM_gtm_cls0.MCS0_MEM[5229].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5230 GTM_gtm_cls0.MCS0_MEM[5230].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5231 GTM_gtm_cls0.MCS0_MEM[5231].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5232 GTM_gtm_cls0.MCS0_MEM[5232].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5233 GTM_gtm_cls0.MCS0_MEM[5233].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5234 GTM_gtm_cls0.MCS0_MEM[5234].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5235 GTM_gtm_cls0.MCS0_MEM[5235].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5236 GTM_gtm_cls0.MCS0_MEM[5236].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5237 GTM_gtm_cls0.MCS0_MEM[5237].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5238 GTM_gtm_cls0.MCS0_MEM[5238].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5239 GTM_gtm_cls0.MCS0_MEM[5239].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5240 GTM_gtm_cls0.MCS0_MEM[5240].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5241 GTM_gtm_cls0.MCS0_MEM[5241].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5242 GTM_gtm_cls0.MCS0_MEM[5242].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5243 GTM_gtm_cls0.MCS0_MEM[5243].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5244 GTM_gtm_cls0.MCS0_MEM[5244].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5245 GTM_gtm_cls0.MCS0_MEM[5245].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5246 GTM_gtm_cls0.MCS0_MEM[5246].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5247 GTM_gtm_cls0.MCS0_MEM[5247].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5248 GTM_gtm_cls0.MCS0_MEM[5248].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5249 GTM_gtm_cls0.MCS0_MEM[5249].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5250 GTM_gtm_cls0.MCS0_MEM[5250].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5251 GTM_gtm_cls0.MCS0_MEM[5251].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5252 GTM_gtm_cls0.MCS0_MEM[5252].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5253 GTM_gtm_cls0.MCS0_MEM[5253].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5254 GTM_gtm_cls0.MCS0_MEM[5254].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5255 GTM_gtm_cls0.MCS0_MEM[5255].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5256 GTM_gtm_cls0.MCS0_MEM[5256].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5257 GTM_gtm_cls0.MCS0_MEM[5257].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5258 GTM_gtm_cls0.MCS0_MEM[5258].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5259 GTM_gtm_cls0.MCS0_MEM[5259].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5260 GTM_gtm_cls0.MCS0_MEM[5260].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5261 GTM_gtm_cls0.MCS0_MEM[5261].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5262 GTM_gtm_cls0.MCS0_MEM[5262].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5263 GTM_gtm_cls0.MCS0_MEM[5263].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5264 GTM_gtm_cls0.MCS0_MEM[5264].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5265 GTM_gtm_cls0.MCS0_MEM[5265].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5266 GTM_gtm_cls0.MCS0_MEM[5266].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5267 GTM_gtm_cls0.MCS0_MEM[5267].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5268 GTM_gtm_cls0.MCS0_MEM[5268].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5269 GTM_gtm_cls0.MCS0_MEM[5269].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5270 GTM_gtm_cls0.MCS0_MEM[5270].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5271 GTM_gtm_cls0.MCS0_MEM[5271].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5272 GTM_gtm_cls0.MCS0_MEM[5272].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5273 GTM_gtm_cls0.MCS0_MEM[5273].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5274 GTM_gtm_cls0.MCS0_MEM[5274].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5275 GTM_gtm_cls0.MCS0_MEM[5275].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5276 GTM_gtm_cls0.MCS0_MEM[5276].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5277 GTM_gtm_cls0.MCS0_MEM[5277].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5278 GTM_gtm_cls0.MCS0_MEM[5278].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5279 GTM_gtm_cls0.MCS0_MEM[5279].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5280 GTM_gtm_cls0.MCS0_MEM[5280].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5281 GTM_gtm_cls0.MCS0_MEM[5281].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5282 GTM_gtm_cls0.MCS0_MEM[5282].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5283 GTM_gtm_cls0.MCS0_MEM[5283].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5284 GTM_gtm_cls0.MCS0_MEM[5284].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5285 GTM_gtm_cls0.MCS0_MEM[5285].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5286 GTM_gtm_cls0.MCS0_MEM[5286].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5287 GTM_gtm_cls0.MCS0_MEM[5287].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5288 GTM_gtm_cls0.MCS0_MEM[5288].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5289 GTM_gtm_cls0.MCS0_MEM[5289].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5290 GTM_gtm_cls0.MCS0_MEM[5290].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5291 GTM_gtm_cls0.MCS0_MEM[5291].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5292 GTM_gtm_cls0.MCS0_MEM[5292].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5293 GTM_gtm_cls0.MCS0_MEM[5293].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5294 GTM_gtm_cls0.MCS0_MEM[5294].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5295 GTM_gtm_cls0.MCS0_MEM[5295].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5296 GTM_gtm_cls0.MCS0_MEM[5296].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5297 GTM_gtm_cls0.MCS0_MEM[5297].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5298 GTM_gtm_cls0.MCS0_MEM[5298].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5299 GTM_gtm_cls0.MCS0_MEM[5299].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5300 GTM_gtm_cls0.MCS0_MEM[5300].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5301 GTM_gtm_cls0.MCS0_MEM[5301].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5302 GTM_gtm_cls0.MCS0_MEM[5302].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5303 GTM_gtm_cls0.MCS0_MEM[5303].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5304 GTM_gtm_cls0.MCS0_MEM[5304].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5305 GTM_gtm_cls0.MCS0_MEM[5305].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5306 GTM_gtm_cls0.MCS0_MEM[5306].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5307 GTM_gtm_cls0.MCS0_MEM[5307].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5308 GTM_gtm_cls0.MCS0_MEM[5308].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5309 GTM_gtm_cls0.MCS0_MEM[5309].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5310 GTM_gtm_cls0.MCS0_MEM[5310].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5311 GTM_gtm_cls0.MCS0_MEM[5311].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5312 GTM_gtm_cls0.MCS0_MEM[5312].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5313 GTM_gtm_cls0.MCS0_MEM[5313].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5314 GTM_gtm_cls0.MCS0_MEM[5314].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5315 GTM_gtm_cls0.MCS0_MEM[5315].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5316 GTM_gtm_cls0.MCS0_MEM[5316].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5317 GTM_gtm_cls0.MCS0_MEM[5317].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5318 GTM_gtm_cls0.MCS0_MEM[5318].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5319 GTM_gtm_cls0.MCS0_MEM[5319].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5320 GTM_gtm_cls0.MCS0_MEM[5320].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5321 GTM_gtm_cls0.MCS0_MEM[5321].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5322 GTM_gtm_cls0.MCS0_MEM[5322].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5323 GTM_gtm_cls0.MCS0_MEM[5323].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5324 GTM_gtm_cls0.MCS0_MEM[5324].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5325 GTM_gtm_cls0.MCS0_MEM[5325].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5326 GTM_gtm_cls0.MCS0_MEM[5326].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5327 GTM_gtm_cls0.MCS0_MEM[5327].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5328 GTM_gtm_cls0.MCS0_MEM[5328].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5329 GTM_gtm_cls0.MCS0_MEM[5329].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5330 GTM_gtm_cls0.MCS0_MEM[5330].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5331 GTM_gtm_cls0.MCS0_MEM[5331].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5332 GTM_gtm_cls0.MCS0_MEM[5332].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5333 GTM_gtm_cls0.MCS0_MEM[5333].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5334 GTM_gtm_cls0.MCS0_MEM[5334].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5335 GTM_gtm_cls0.MCS0_MEM[5335].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5336 GTM_gtm_cls0.MCS0_MEM[5336].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5337 GTM_gtm_cls0.MCS0_MEM[5337].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5338 GTM_gtm_cls0.MCS0_MEM[5338].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5339 GTM_gtm_cls0.MCS0_MEM[5339].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5340 GTM_gtm_cls0.MCS0_MEM[5340].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5341 GTM_gtm_cls0.MCS0_MEM[5341].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5342 GTM_gtm_cls0.MCS0_MEM[5342].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5343 GTM_gtm_cls0.MCS0_MEM[5343].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5344 GTM_gtm_cls0.MCS0_MEM[5344].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5345 GTM_gtm_cls0.MCS0_MEM[5345].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5346 GTM_gtm_cls0.MCS0_MEM[5346].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5347 GTM_gtm_cls0.MCS0_MEM[5347].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5348 GTM_gtm_cls0.MCS0_MEM[5348].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5349 GTM_gtm_cls0.MCS0_MEM[5349].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5350 GTM_gtm_cls0.MCS0_MEM[5350].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5351 GTM_gtm_cls0.MCS0_MEM[5351].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5352 GTM_gtm_cls0.MCS0_MEM[5352].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5353 GTM_gtm_cls0.MCS0_MEM[5353].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5354 GTM_gtm_cls0.MCS0_MEM[5354].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5355 GTM_gtm_cls0.MCS0_MEM[5355].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5356 GTM_gtm_cls0.MCS0_MEM[5356].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5357 GTM_gtm_cls0.MCS0_MEM[5357].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5358 GTM_gtm_cls0.MCS0_MEM[5358].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5359 GTM_gtm_cls0.MCS0_MEM[5359].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5360 GTM_gtm_cls0.MCS0_MEM[5360].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5361 GTM_gtm_cls0.MCS0_MEM[5361].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5362 GTM_gtm_cls0.MCS0_MEM[5362].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5363 GTM_gtm_cls0.MCS0_MEM[5363].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5364 GTM_gtm_cls0.MCS0_MEM[5364].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5365 GTM_gtm_cls0.MCS0_MEM[5365].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5366 GTM_gtm_cls0.MCS0_MEM[5366].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5367 GTM_gtm_cls0.MCS0_MEM[5367].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5368 GTM_gtm_cls0.MCS0_MEM[5368].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5369 GTM_gtm_cls0.MCS0_MEM[5369].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5370 GTM_gtm_cls0.MCS0_MEM[5370].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5371 GTM_gtm_cls0.MCS0_MEM[5371].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5372 GTM_gtm_cls0.MCS0_MEM[5372].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5373 GTM_gtm_cls0.MCS0_MEM[5373].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5374 GTM_gtm_cls0.MCS0_MEM[5374].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5375 GTM_gtm_cls0.MCS0_MEM[5375].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5376 GTM_gtm_cls0.MCS0_MEM[5376].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5377 GTM_gtm_cls0.MCS0_MEM[5377].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5378 GTM_gtm_cls0.MCS0_MEM[5378].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5379 GTM_gtm_cls0.MCS0_MEM[5379].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5380 GTM_gtm_cls0.MCS0_MEM[5380].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5381 GTM_gtm_cls0.MCS0_MEM[5381].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5382 GTM_gtm_cls0.MCS0_MEM[5382].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5383 GTM_gtm_cls0.MCS0_MEM[5383].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5384 GTM_gtm_cls0.MCS0_MEM[5384].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5385 GTM_gtm_cls0.MCS0_MEM[5385].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5386 GTM_gtm_cls0.MCS0_MEM[5386].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5387 GTM_gtm_cls0.MCS0_MEM[5387].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5388 GTM_gtm_cls0.MCS0_MEM[5388].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5389 GTM_gtm_cls0.MCS0_MEM[5389].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5390 GTM_gtm_cls0.MCS0_MEM[5390].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5391 GTM_gtm_cls0.MCS0_MEM[5391].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5392 GTM_gtm_cls0.MCS0_MEM[5392].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5393 GTM_gtm_cls0.MCS0_MEM[5393].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5394 GTM_gtm_cls0.MCS0_MEM[5394].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5395 GTM_gtm_cls0.MCS0_MEM[5395].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5396 GTM_gtm_cls0.MCS0_MEM[5396].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5397 GTM_gtm_cls0.MCS0_MEM[5397].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5398 GTM_gtm_cls0.MCS0_MEM[5398].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5399 GTM_gtm_cls0.MCS0_MEM[5399].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5400 GTM_gtm_cls0.MCS0_MEM[5400].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5401 GTM_gtm_cls0.MCS0_MEM[5401].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5402 GTM_gtm_cls0.MCS0_MEM[5402].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5403 GTM_gtm_cls0.MCS0_MEM[5403].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5404 GTM_gtm_cls0.MCS0_MEM[5404].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5405 GTM_gtm_cls0.MCS0_MEM[5405].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5406 GTM_gtm_cls0.MCS0_MEM[5406].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5407 GTM_gtm_cls0.MCS0_MEM[5407].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5408 GTM_gtm_cls0.MCS0_MEM[5408].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5409 GTM_gtm_cls0.MCS0_MEM[5409].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5410 GTM_gtm_cls0.MCS0_MEM[5410].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5411 GTM_gtm_cls0.MCS0_MEM[5411].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5412 GTM_gtm_cls0.MCS0_MEM[5412].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5413 GTM_gtm_cls0.MCS0_MEM[5413].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5414 GTM_gtm_cls0.MCS0_MEM[5414].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5415 GTM_gtm_cls0.MCS0_MEM[5415].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5416 GTM_gtm_cls0.MCS0_MEM[5416].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5417 GTM_gtm_cls0.MCS0_MEM[5417].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5418 GTM_gtm_cls0.MCS0_MEM[5418].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5419 GTM_gtm_cls0.MCS0_MEM[5419].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5420 GTM_gtm_cls0.MCS0_MEM[5420].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5421 GTM_gtm_cls0.MCS0_MEM[5421].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5422 GTM_gtm_cls0.MCS0_MEM[5422].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5423 GTM_gtm_cls0.MCS0_MEM[5423].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5424 GTM_gtm_cls0.MCS0_MEM[5424].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5425 GTM_gtm_cls0.MCS0_MEM[5425].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5426 GTM_gtm_cls0.MCS0_MEM[5426].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5427 GTM_gtm_cls0.MCS0_MEM[5427].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5428 GTM_gtm_cls0.MCS0_MEM[5428].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5429 GTM_gtm_cls0.MCS0_MEM[5429].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5430 GTM_gtm_cls0.MCS0_MEM[5430].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5431 GTM_gtm_cls0.MCS0_MEM[5431].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5432 GTM_gtm_cls0.MCS0_MEM[5432].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5433 GTM_gtm_cls0.MCS0_MEM[5433].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5434 GTM_gtm_cls0.MCS0_MEM[5434].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5435 GTM_gtm_cls0.MCS0_MEM[5435].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5436 GTM_gtm_cls0.MCS0_MEM[5436].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5437 GTM_gtm_cls0.MCS0_MEM[5437].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5438 GTM_gtm_cls0.MCS0_MEM[5438].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5439 GTM_gtm_cls0.MCS0_MEM[5439].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5440 GTM_gtm_cls0.MCS0_MEM[5440].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5441 GTM_gtm_cls0.MCS0_MEM[5441].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5442 GTM_gtm_cls0.MCS0_MEM[5442].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5443 GTM_gtm_cls0.MCS0_MEM[5443].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5444 GTM_gtm_cls0.MCS0_MEM[5444].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5445 GTM_gtm_cls0.MCS0_MEM[5445].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5446 GTM_gtm_cls0.MCS0_MEM[5446].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5447 GTM_gtm_cls0.MCS0_MEM[5447].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5448 GTM_gtm_cls0.MCS0_MEM[5448].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5449 GTM_gtm_cls0.MCS0_MEM[5449].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5450 GTM_gtm_cls0.MCS0_MEM[5450].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5451 GTM_gtm_cls0.MCS0_MEM[5451].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5452 GTM_gtm_cls0.MCS0_MEM[5452].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5453 GTM_gtm_cls0.MCS0_MEM[5453].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5454 GTM_gtm_cls0.MCS0_MEM[5454].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5455 GTM_gtm_cls0.MCS0_MEM[5455].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5456 GTM_gtm_cls0.MCS0_MEM[5456].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5457 GTM_gtm_cls0.MCS0_MEM[5457].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5458 GTM_gtm_cls0.MCS0_MEM[5458].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5459 GTM_gtm_cls0.MCS0_MEM[5459].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5460 GTM_gtm_cls0.MCS0_MEM[5460].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5461 GTM_gtm_cls0.MCS0_MEM[5461].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5462 GTM_gtm_cls0.MCS0_MEM[5462].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5463 GTM_gtm_cls0.MCS0_MEM[5463].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5464 GTM_gtm_cls0.MCS0_MEM[5464].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5465 GTM_gtm_cls0.MCS0_MEM[5465].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5466 GTM_gtm_cls0.MCS0_MEM[5466].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5467 GTM_gtm_cls0.MCS0_MEM[5467].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5468 GTM_gtm_cls0.MCS0_MEM[5468].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5469 GTM_gtm_cls0.MCS0_MEM[5469].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5470 GTM_gtm_cls0.MCS0_MEM[5470].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5471 GTM_gtm_cls0.MCS0_MEM[5471].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5472 GTM_gtm_cls0.MCS0_MEM[5472].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5473 GTM_gtm_cls0.MCS0_MEM[5473].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5474 GTM_gtm_cls0.MCS0_MEM[5474].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5475 GTM_gtm_cls0.MCS0_MEM[5475].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5476 GTM_gtm_cls0.MCS0_MEM[5476].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5477 GTM_gtm_cls0.MCS0_MEM[5477].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5478 GTM_gtm_cls0.MCS0_MEM[5478].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5479 GTM_gtm_cls0.MCS0_MEM[5479].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5480 GTM_gtm_cls0.MCS0_MEM[5480].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5481 GTM_gtm_cls0.MCS0_MEM[5481].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5482 GTM_gtm_cls0.MCS0_MEM[5482].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5483 GTM_gtm_cls0.MCS0_MEM[5483].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5484 GTM_gtm_cls0.MCS0_MEM[5484].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5485 GTM_gtm_cls0.MCS0_MEM[5485].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5486 GTM_gtm_cls0.MCS0_MEM[5486].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5487 GTM_gtm_cls0.MCS0_MEM[5487].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5488 GTM_gtm_cls0.MCS0_MEM[5488].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5489 GTM_gtm_cls0.MCS0_MEM[5489].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5490 GTM_gtm_cls0.MCS0_MEM[5490].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5491 GTM_gtm_cls0.MCS0_MEM[5491].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5492 GTM_gtm_cls0.MCS0_MEM[5492].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5493 GTM_gtm_cls0.MCS0_MEM[5493].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5494 GTM_gtm_cls0.MCS0_MEM[5494].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5495 GTM_gtm_cls0.MCS0_MEM[5495].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5496 GTM_gtm_cls0.MCS0_MEM[5496].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5497 GTM_gtm_cls0.MCS0_MEM[5497].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5498 GTM_gtm_cls0.MCS0_MEM[5498].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5499 GTM_gtm_cls0.MCS0_MEM[5499].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5500 GTM_gtm_cls0.MCS0_MEM[5500].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5501 GTM_gtm_cls0.MCS0_MEM[5501].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5502 GTM_gtm_cls0.MCS0_MEM[5502].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5503 GTM_gtm_cls0.MCS0_MEM[5503].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5504 GTM_gtm_cls0.MCS0_MEM[5504].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5505 GTM_gtm_cls0.MCS0_MEM[5505].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5506 GTM_gtm_cls0.MCS0_MEM[5506].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5507 GTM_gtm_cls0.MCS0_MEM[5507].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5508 GTM_gtm_cls0.MCS0_MEM[5508].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5509 GTM_gtm_cls0.MCS0_MEM[5509].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5510 GTM_gtm_cls0.MCS0_MEM[5510].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5511 GTM_gtm_cls0.MCS0_MEM[5511].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5512 GTM_gtm_cls0.MCS0_MEM[5512].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5513 GTM_gtm_cls0.MCS0_MEM[5513].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5514 GTM_gtm_cls0.MCS0_MEM[5514].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5515 GTM_gtm_cls0.MCS0_MEM[5515].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5516 GTM_gtm_cls0.MCS0_MEM[5516].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5517 GTM_gtm_cls0.MCS0_MEM[5517].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5518 GTM_gtm_cls0.MCS0_MEM[5518].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5519 GTM_gtm_cls0.MCS0_MEM[5519].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5520 GTM_gtm_cls0.MCS0_MEM[5520].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5521 GTM_gtm_cls0.MCS0_MEM[5521].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5522 GTM_gtm_cls0.MCS0_MEM[5522].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5523 GTM_gtm_cls0.MCS0_MEM[5523].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5524 GTM_gtm_cls0.MCS0_MEM[5524].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5525 GTM_gtm_cls0.MCS0_MEM[5525].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5526 GTM_gtm_cls0.MCS0_MEM[5526].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5527 GTM_gtm_cls0.MCS0_MEM[5527].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5528 GTM_gtm_cls0.MCS0_MEM[5528].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5529 GTM_gtm_cls0.MCS0_MEM[5529].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5530 GTM_gtm_cls0.MCS0_MEM[5530].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5531 GTM_gtm_cls0.MCS0_MEM[5531].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5532 GTM_gtm_cls0.MCS0_MEM[5532].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5533 GTM_gtm_cls0.MCS0_MEM[5533].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5534 GTM_gtm_cls0.MCS0_MEM[5534].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5535 GTM_gtm_cls0.MCS0_MEM[5535].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5536 GTM_gtm_cls0.MCS0_MEM[5536].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5537 GTM_gtm_cls0.MCS0_MEM[5537].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5538 GTM_gtm_cls0.MCS0_MEM[5538].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5539 GTM_gtm_cls0.MCS0_MEM[5539].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5540 GTM_gtm_cls0.MCS0_MEM[5540].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5541 GTM_gtm_cls0.MCS0_MEM[5541].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5542 GTM_gtm_cls0.MCS0_MEM[5542].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5543 GTM_gtm_cls0.MCS0_MEM[5543].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5544 GTM_gtm_cls0.MCS0_MEM[5544].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5545 GTM_gtm_cls0.MCS0_MEM[5545].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5546 GTM_gtm_cls0.MCS0_MEM[5546].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5547 GTM_gtm_cls0.MCS0_MEM[5547].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5548 GTM_gtm_cls0.MCS0_MEM[5548].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5549 GTM_gtm_cls0.MCS0_MEM[5549].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5550 GTM_gtm_cls0.MCS0_MEM[5550].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5551 GTM_gtm_cls0.MCS0_MEM[5551].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5552 GTM_gtm_cls0.MCS0_MEM[5552].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5553 GTM_gtm_cls0.MCS0_MEM[5553].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5554 GTM_gtm_cls0.MCS0_MEM[5554].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5555 GTM_gtm_cls0.MCS0_MEM[5555].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5556 GTM_gtm_cls0.MCS0_MEM[5556].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5557 GTM_gtm_cls0.MCS0_MEM[5557].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5558 GTM_gtm_cls0.MCS0_MEM[5558].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5559 GTM_gtm_cls0.MCS0_MEM[5559].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5560 GTM_gtm_cls0.MCS0_MEM[5560].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5561 GTM_gtm_cls0.MCS0_MEM[5561].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5562 GTM_gtm_cls0.MCS0_MEM[5562].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5563 GTM_gtm_cls0.MCS0_MEM[5563].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5564 GTM_gtm_cls0.MCS0_MEM[5564].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5565 GTM_gtm_cls0.MCS0_MEM[5565].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5566 GTM_gtm_cls0.MCS0_MEM[5566].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5567 GTM_gtm_cls0.MCS0_MEM[5567].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5568 GTM_gtm_cls0.MCS0_MEM[5568].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5569 GTM_gtm_cls0.MCS0_MEM[5569].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5570 GTM_gtm_cls0.MCS0_MEM[5570].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5571 GTM_gtm_cls0.MCS0_MEM[5571].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5572 GTM_gtm_cls0.MCS0_MEM[5572].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5573 GTM_gtm_cls0.MCS0_MEM[5573].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5574 GTM_gtm_cls0.MCS0_MEM[5574].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5575 GTM_gtm_cls0.MCS0_MEM[5575].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5576 GTM_gtm_cls0.MCS0_MEM[5576].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5577 GTM_gtm_cls0.MCS0_MEM[5577].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5578 GTM_gtm_cls0.MCS0_MEM[5578].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5579 GTM_gtm_cls0.MCS0_MEM[5579].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5580 GTM_gtm_cls0.MCS0_MEM[5580].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5581 GTM_gtm_cls0.MCS0_MEM[5581].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5582 GTM_gtm_cls0.MCS0_MEM[5582].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5583 GTM_gtm_cls0.MCS0_MEM[5583].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5584 GTM_gtm_cls0.MCS0_MEM[5584].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5585 GTM_gtm_cls0.MCS0_MEM[5585].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5586 GTM_gtm_cls0.MCS0_MEM[5586].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5587 GTM_gtm_cls0.MCS0_MEM[5587].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5588 GTM_gtm_cls0.MCS0_MEM[5588].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5589 GTM_gtm_cls0.MCS0_MEM[5589].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5590 GTM_gtm_cls0.MCS0_MEM[5590].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5591 GTM_gtm_cls0.MCS0_MEM[5591].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5592 GTM_gtm_cls0.MCS0_MEM[5592].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5593 GTM_gtm_cls0.MCS0_MEM[5593].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5594 GTM_gtm_cls0.MCS0_MEM[5594].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5595 GTM_gtm_cls0.MCS0_MEM[5595].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5596 GTM_gtm_cls0.MCS0_MEM[5596].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5597 GTM_gtm_cls0.MCS0_MEM[5597].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5598 GTM_gtm_cls0.MCS0_MEM[5598].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5599 GTM_gtm_cls0.MCS0_MEM[5599].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5600 GTM_gtm_cls0.MCS0_MEM[5600].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5601 GTM_gtm_cls0.MCS0_MEM[5601].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5602 GTM_gtm_cls0.MCS0_MEM[5602].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5603 GTM_gtm_cls0.MCS0_MEM[5603].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5604 GTM_gtm_cls0.MCS0_MEM[5604].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5605 GTM_gtm_cls0.MCS0_MEM[5605].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5606 GTM_gtm_cls0.MCS0_MEM[5606].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5607 GTM_gtm_cls0.MCS0_MEM[5607].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5608 GTM_gtm_cls0.MCS0_MEM[5608].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5609 GTM_gtm_cls0.MCS0_MEM[5609].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5610 GTM_gtm_cls0.MCS0_MEM[5610].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5611 GTM_gtm_cls0.MCS0_MEM[5611].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5612 GTM_gtm_cls0.MCS0_MEM[5612].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5613 GTM_gtm_cls0.MCS0_MEM[5613].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5614 GTM_gtm_cls0.MCS0_MEM[5614].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5615 GTM_gtm_cls0.MCS0_MEM[5615].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5616 GTM_gtm_cls0.MCS0_MEM[5616].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5617 GTM_gtm_cls0.MCS0_MEM[5617].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5618 GTM_gtm_cls0.MCS0_MEM[5618].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5619 GTM_gtm_cls0.MCS0_MEM[5619].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5620 GTM_gtm_cls0.MCS0_MEM[5620].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5621 GTM_gtm_cls0.MCS0_MEM[5621].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5622 GTM_gtm_cls0.MCS0_MEM[5622].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5623 GTM_gtm_cls0.MCS0_MEM[5623].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5624 GTM_gtm_cls0.MCS0_MEM[5624].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5625 GTM_gtm_cls0.MCS0_MEM[5625].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5626 GTM_gtm_cls0.MCS0_MEM[5626].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5627 GTM_gtm_cls0.MCS0_MEM[5627].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5628 GTM_gtm_cls0.MCS0_MEM[5628].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5629 GTM_gtm_cls0.MCS0_MEM[5629].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5630 GTM_gtm_cls0.MCS0_MEM[5630].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5631 GTM_gtm_cls0.MCS0_MEM[5631].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5632 GTM_gtm_cls0.MCS0_MEM[5632].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5633 GTM_gtm_cls0.MCS0_MEM[5633].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5634 GTM_gtm_cls0.MCS0_MEM[5634].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5635 GTM_gtm_cls0.MCS0_MEM[5635].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5636 GTM_gtm_cls0.MCS0_MEM[5636].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5637 GTM_gtm_cls0.MCS0_MEM[5637].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5638 GTM_gtm_cls0.MCS0_MEM[5638].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5639 GTM_gtm_cls0.MCS0_MEM[5639].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5640 GTM_gtm_cls0.MCS0_MEM[5640].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5641 GTM_gtm_cls0.MCS0_MEM[5641].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5642 GTM_gtm_cls0.MCS0_MEM[5642].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5643 GTM_gtm_cls0.MCS0_MEM[5643].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5644 GTM_gtm_cls0.MCS0_MEM[5644].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5645 GTM_gtm_cls0.MCS0_MEM[5645].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5646 GTM_gtm_cls0.MCS0_MEM[5646].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5647 GTM_gtm_cls0.MCS0_MEM[5647].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5648 GTM_gtm_cls0.MCS0_MEM[5648].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5649 GTM_gtm_cls0.MCS0_MEM[5649].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5650 GTM_gtm_cls0.MCS0_MEM[5650].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5651 GTM_gtm_cls0.MCS0_MEM[5651].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5652 GTM_gtm_cls0.MCS0_MEM[5652].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5653 GTM_gtm_cls0.MCS0_MEM[5653].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5654 GTM_gtm_cls0.MCS0_MEM[5654].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5655 GTM_gtm_cls0.MCS0_MEM[5655].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5656 GTM_gtm_cls0.MCS0_MEM[5656].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5657 GTM_gtm_cls0.MCS0_MEM[5657].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5658 GTM_gtm_cls0.MCS0_MEM[5658].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5659 GTM_gtm_cls0.MCS0_MEM[5659].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5660 GTM_gtm_cls0.MCS0_MEM[5660].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5661 GTM_gtm_cls0.MCS0_MEM[5661].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5662 GTM_gtm_cls0.MCS0_MEM[5662].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5663 GTM_gtm_cls0.MCS0_MEM[5663].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5664 GTM_gtm_cls0.MCS0_MEM[5664].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5665 GTM_gtm_cls0.MCS0_MEM[5665].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5666 GTM_gtm_cls0.MCS0_MEM[5666].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5667 GTM_gtm_cls0.MCS0_MEM[5667].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5668 GTM_gtm_cls0.MCS0_MEM[5668].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5669 GTM_gtm_cls0.MCS0_MEM[5669].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5670 GTM_gtm_cls0.MCS0_MEM[5670].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5671 GTM_gtm_cls0.MCS0_MEM[5671].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5672 GTM_gtm_cls0.MCS0_MEM[5672].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5673 GTM_gtm_cls0.MCS0_MEM[5673].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5674 GTM_gtm_cls0.MCS0_MEM[5674].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5675 GTM_gtm_cls0.MCS0_MEM[5675].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5676 GTM_gtm_cls0.MCS0_MEM[5676].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5677 GTM_gtm_cls0.MCS0_MEM[5677].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5678 GTM_gtm_cls0.MCS0_MEM[5678].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5679 GTM_gtm_cls0.MCS0_MEM[5679].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5680 GTM_gtm_cls0.MCS0_MEM[5680].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5681 GTM_gtm_cls0.MCS0_MEM[5681].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5682 GTM_gtm_cls0.MCS0_MEM[5682].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5683 GTM_gtm_cls0.MCS0_MEM[5683].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5684 GTM_gtm_cls0.MCS0_MEM[5684].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5685 GTM_gtm_cls0.MCS0_MEM[5685].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5686 GTM_gtm_cls0.MCS0_MEM[5686].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5687 GTM_gtm_cls0.MCS0_MEM[5687].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5688 GTM_gtm_cls0.MCS0_MEM[5688].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5689 GTM_gtm_cls0.MCS0_MEM[5689].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5690 GTM_gtm_cls0.MCS0_MEM[5690].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5691 GTM_gtm_cls0.MCS0_MEM[5691].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5692 GTM_gtm_cls0.MCS0_MEM[5692].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5693 GTM_gtm_cls0.MCS0_MEM[5693].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5694 GTM_gtm_cls0.MCS0_MEM[5694].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5695 GTM_gtm_cls0.MCS0_MEM[5695].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5696 GTM_gtm_cls0.MCS0_MEM[5696].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5697 GTM_gtm_cls0.MCS0_MEM[5697].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5698 GTM_gtm_cls0.MCS0_MEM[5698].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5699 GTM_gtm_cls0.MCS0_MEM[5699].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5700 GTM_gtm_cls0.MCS0_MEM[5700].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5701 GTM_gtm_cls0.MCS0_MEM[5701].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5702 GTM_gtm_cls0.MCS0_MEM[5702].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5703 GTM_gtm_cls0.MCS0_MEM[5703].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5704 GTM_gtm_cls0.MCS0_MEM[5704].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5705 GTM_gtm_cls0.MCS0_MEM[5705].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5706 GTM_gtm_cls0.MCS0_MEM[5706].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5707 GTM_gtm_cls0.MCS0_MEM[5707].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5708 GTM_gtm_cls0.MCS0_MEM[5708].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5709 GTM_gtm_cls0.MCS0_MEM[5709].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5710 GTM_gtm_cls0.MCS0_MEM[5710].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5711 GTM_gtm_cls0.MCS0_MEM[5711].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5712 GTM_gtm_cls0.MCS0_MEM[5712].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5713 GTM_gtm_cls0.MCS0_MEM[5713].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5714 GTM_gtm_cls0.MCS0_MEM[5714].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5715 GTM_gtm_cls0.MCS0_MEM[5715].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5716 GTM_gtm_cls0.MCS0_MEM[5716].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5717 GTM_gtm_cls0.MCS0_MEM[5717].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5718 GTM_gtm_cls0.MCS0_MEM[5718].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5719 GTM_gtm_cls0.MCS0_MEM[5719].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5720 GTM_gtm_cls0.MCS0_MEM[5720].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5721 GTM_gtm_cls0.MCS0_MEM[5721].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5722 GTM_gtm_cls0.MCS0_MEM[5722].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5723 GTM_gtm_cls0.MCS0_MEM[5723].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5724 GTM_gtm_cls0.MCS0_MEM[5724].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5725 GTM_gtm_cls0.MCS0_MEM[5725].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5726 GTM_gtm_cls0.MCS0_MEM[5726].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5727 GTM_gtm_cls0.MCS0_MEM[5727].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5728 GTM_gtm_cls0.MCS0_MEM[5728].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5729 GTM_gtm_cls0.MCS0_MEM[5729].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5730 GTM_gtm_cls0.MCS0_MEM[5730].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5731 GTM_gtm_cls0.MCS0_MEM[5731].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5732 GTM_gtm_cls0.MCS0_MEM[5732].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5733 GTM_gtm_cls0.MCS0_MEM[5733].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5734 GTM_gtm_cls0.MCS0_MEM[5734].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5735 GTM_gtm_cls0.MCS0_MEM[5735].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5736 GTM_gtm_cls0.MCS0_MEM[5736].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5737 GTM_gtm_cls0.MCS0_MEM[5737].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5738 GTM_gtm_cls0.MCS0_MEM[5738].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5739 GTM_gtm_cls0.MCS0_MEM[5739].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5740 GTM_gtm_cls0.MCS0_MEM[5740].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5741 GTM_gtm_cls0.MCS0_MEM[5741].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5742 GTM_gtm_cls0.MCS0_MEM[5742].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5743 GTM_gtm_cls0.MCS0_MEM[5743].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5744 GTM_gtm_cls0.MCS0_MEM[5744].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5745 GTM_gtm_cls0.MCS0_MEM[5745].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5746 GTM_gtm_cls0.MCS0_MEM[5746].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5747 GTM_gtm_cls0.MCS0_MEM[5747].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5748 GTM_gtm_cls0.MCS0_MEM[5748].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5749 GTM_gtm_cls0.MCS0_MEM[5749].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5750 GTM_gtm_cls0.MCS0_MEM[5750].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5751 GTM_gtm_cls0.MCS0_MEM[5751].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5752 GTM_gtm_cls0.MCS0_MEM[5752].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5753 GTM_gtm_cls0.MCS0_MEM[5753].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5754 GTM_gtm_cls0.MCS0_MEM[5754].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5755 GTM_gtm_cls0.MCS0_MEM[5755].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5756 GTM_gtm_cls0.MCS0_MEM[5756].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5757 GTM_gtm_cls0.MCS0_MEM[5757].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5758 GTM_gtm_cls0.MCS0_MEM[5758].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5759 GTM_gtm_cls0.MCS0_MEM[5759].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5760 GTM_gtm_cls0.MCS0_MEM[5760].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5761 GTM_gtm_cls0.MCS0_MEM[5761].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5762 GTM_gtm_cls0.MCS0_MEM[5762].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5763 GTM_gtm_cls0.MCS0_MEM[5763].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5764 GTM_gtm_cls0.MCS0_MEM[5764].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5765 GTM_gtm_cls0.MCS0_MEM[5765].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5766 GTM_gtm_cls0.MCS0_MEM[5766].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5767 GTM_gtm_cls0.MCS0_MEM[5767].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5768 GTM_gtm_cls0.MCS0_MEM[5768].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5769 GTM_gtm_cls0.MCS0_MEM[5769].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5770 GTM_gtm_cls0.MCS0_MEM[5770].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5771 GTM_gtm_cls0.MCS0_MEM[5771].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5772 GTM_gtm_cls0.MCS0_MEM[5772].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5773 GTM_gtm_cls0.MCS0_MEM[5773].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5774 GTM_gtm_cls0.MCS0_MEM[5774].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5775 GTM_gtm_cls0.MCS0_MEM[5775].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5776 GTM_gtm_cls0.MCS0_MEM[5776].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5777 GTM_gtm_cls0.MCS0_MEM[5777].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5778 GTM_gtm_cls0.MCS0_MEM[5778].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5779 GTM_gtm_cls0.MCS0_MEM[5779].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5780 GTM_gtm_cls0.MCS0_MEM[5780].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5781 GTM_gtm_cls0.MCS0_MEM[5781].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5782 GTM_gtm_cls0.MCS0_MEM[5782].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5783 GTM_gtm_cls0.MCS0_MEM[5783].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5784 GTM_gtm_cls0.MCS0_MEM[5784].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5785 GTM_gtm_cls0.MCS0_MEM[5785].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5786 GTM_gtm_cls0.MCS0_MEM[5786].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5787 GTM_gtm_cls0.MCS0_MEM[5787].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5788 GTM_gtm_cls0.MCS0_MEM[5788].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5789 GTM_gtm_cls0.MCS0_MEM[5789].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5790 GTM_gtm_cls0.MCS0_MEM[5790].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5791 GTM_gtm_cls0.MCS0_MEM[5791].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5792 GTM_gtm_cls0.MCS0_MEM[5792].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5793 GTM_gtm_cls0.MCS0_MEM[5793].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5794 GTM_gtm_cls0.MCS0_MEM[5794].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5795 GTM_gtm_cls0.MCS0_MEM[5795].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5796 GTM_gtm_cls0.MCS0_MEM[5796].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5797 GTM_gtm_cls0.MCS0_MEM[5797].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5798 GTM_gtm_cls0.MCS0_MEM[5798].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5799 GTM_gtm_cls0.MCS0_MEM[5799].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5800 GTM_gtm_cls0.MCS0_MEM[5800].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5801 GTM_gtm_cls0.MCS0_MEM[5801].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5802 GTM_gtm_cls0.MCS0_MEM[5802].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5803 GTM_gtm_cls0.MCS0_MEM[5803].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5804 GTM_gtm_cls0.MCS0_MEM[5804].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5805 GTM_gtm_cls0.MCS0_MEM[5805].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5806 GTM_gtm_cls0.MCS0_MEM[5806].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5807 GTM_gtm_cls0.MCS0_MEM[5807].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5808 GTM_gtm_cls0.MCS0_MEM[5808].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5809 GTM_gtm_cls0.MCS0_MEM[5809].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5810 GTM_gtm_cls0.MCS0_MEM[5810].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5811 GTM_gtm_cls0.MCS0_MEM[5811].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5812 GTM_gtm_cls0.MCS0_MEM[5812].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5813 GTM_gtm_cls0.MCS0_MEM[5813].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5814 GTM_gtm_cls0.MCS0_MEM[5814].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5815 GTM_gtm_cls0.MCS0_MEM[5815].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5816 GTM_gtm_cls0.MCS0_MEM[5816].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5817 GTM_gtm_cls0.MCS0_MEM[5817].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5818 GTM_gtm_cls0.MCS0_MEM[5818].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5819 GTM_gtm_cls0.MCS0_MEM[5819].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5820 GTM_gtm_cls0.MCS0_MEM[5820].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5821 GTM_gtm_cls0.MCS0_MEM[5821].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5822 GTM_gtm_cls0.MCS0_MEM[5822].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5823 GTM_gtm_cls0.MCS0_MEM[5823].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5824 GTM_gtm_cls0.MCS0_MEM[5824].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5825 GTM_gtm_cls0.MCS0_MEM[5825].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5826 GTM_gtm_cls0.MCS0_MEM[5826].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5827 GTM_gtm_cls0.MCS0_MEM[5827].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5828 GTM_gtm_cls0.MCS0_MEM[5828].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5829 GTM_gtm_cls0.MCS0_MEM[5829].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5830 GTM_gtm_cls0.MCS0_MEM[5830].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5831 GTM_gtm_cls0.MCS0_MEM[5831].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5832 GTM_gtm_cls0.MCS0_MEM[5832].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5833 GTM_gtm_cls0.MCS0_MEM[5833].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5834 GTM_gtm_cls0.MCS0_MEM[5834].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5835 GTM_gtm_cls0.MCS0_MEM[5835].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5836 GTM_gtm_cls0.MCS0_MEM[5836].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5837 GTM_gtm_cls0.MCS0_MEM[5837].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5838 GTM_gtm_cls0.MCS0_MEM[5838].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5839 GTM_gtm_cls0.MCS0_MEM[5839].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5840 GTM_gtm_cls0.MCS0_MEM[5840].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5841 GTM_gtm_cls0.MCS0_MEM[5841].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5842 GTM_gtm_cls0.MCS0_MEM[5842].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5843 GTM_gtm_cls0.MCS0_MEM[5843].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5844 GTM_gtm_cls0.MCS0_MEM[5844].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5845 GTM_gtm_cls0.MCS0_MEM[5845].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5846 GTM_gtm_cls0.MCS0_MEM[5846].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5847 GTM_gtm_cls0.MCS0_MEM[5847].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5848 GTM_gtm_cls0.MCS0_MEM[5848].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5849 GTM_gtm_cls0.MCS0_MEM[5849].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5850 GTM_gtm_cls0.MCS0_MEM[5850].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5851 GTM_gtm_cls0.MCS0_MEM[5851].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5852 GTM_gtm_cls0.MCS0_MEM[5852].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5853 GTM_gtm_cls0.MCS0_MEM[5853].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5854 GTM_gtm_cls0.MCS0_MEM[5854].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5855 GTM_gtm_cls0.MCS0_MEM[5855].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5856 GTM_gtm_cls0.MCS0_MEM[5856].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5857 GTM_gtm_cls0.MCS0_MEM[5857].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5858 GTM_gtm_cls0.MCS0_MEM[5858].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5859 GTM_gtm_cls0.MCS0_MEM[5859].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5860 GTM_gtm_cls0.MCS0_MEM[5860].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5861 GTM_gtm_cls0.MCS0_MEM[5861].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5862 GTM_gtm_cls0.MCS0_MEM[5862].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5863 GTM_gtm_cls0.MCS0_MEM[5863].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5864 GTM_gtm_cls0.MCS0_MEM[5864].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5865 GTM_gtm_cls0.MCS0_MEM[5865].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5866 GTM_gtm_cls0.MCS0_MEM[5866].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5867 GTM_gtm_cls0.MCS0_MEM[5867].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5868 GTM_gtm_cls0.MCS0_MEM[5868].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5869 GTM_gtm_cls0.MCS0_MEM[5869].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5870 GTM_gtm_cls0.MCS0_MEM[5870].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5871 GTM_gtm_cls0.MCS0_MEM[5871].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5872 GTM_gtm_cls0.MCS0_MEM[5872].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5873 GTM_gtm_cls0.MCS0_MEM[5873].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5874 GTM_gtm_cls0.MCS0_MEM[5874].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5875 GTM_gtm_cls0.MCS0_MEM[5875].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5876 GTM_gtm_cls0.MCS0_MEM[5876].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5877 GTM_gtm_cls0.MCS0_MEM[5877].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5878 GTM_gtm_cls0.MCS0_MEM[5878].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5879 GTM_gtm_cls0.MCS0_MEM[5879].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5880 GTM_gtm_cls0.MCS0_MEM[5880].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5881 GTM_gtm_cls0.MCS0_MEM[5881].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5882 GTM_gtm_cls0.MCS0_MEM[5882].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5883 GTM_gtm_cls0.MCS0_MEM[5883].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5884 GTM_gtm_cls0.MCS0_MEM[5884].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5885 GTM_gtm_cls0.MCS0_MEM[5885].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5886 GTM_gtm_cls0.MCS0_MEM[5886].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5887 GTM_gtm_cls0.MCS0_MEM[5887].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5888 GTM_gtm_cls0.MCS0_MEM[5888].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5889 GTM_gtm_cls0.MCS0_MEM[5889].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5890 GTM_gtm_cls0.MCS0_MEM[5890].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5891 GTM_gtm_cls0.MCS0_MEM[5891].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5892 GTM_gtm_cls0.MCS0_MEM[5892].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5893 GTM_gtm_cls0.MCS0_MEM[5893].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5894 GTM_gtm_cls0.MCS0_MEM[5894].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5895 GTM_gtm_cls0.MCS0_MEM[5895].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5896 GTM_gtm_cls0.MCS0_MEM[5896].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5897 GTM_gtm_cls0.MCS0_MEM[5897].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5898 GTM_gtm_cls0.MCS0_MEM[5898].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5899 GTM_gtm_cls0.MCS0_MEM[5899].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5900 GTM_gtm_cls0.MCS0_MEM[5900].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5901 GTM_gtm_cls0.MCS0_MEM[5901].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5902 GTM_gtm_cls0.MCS0_MEM[5902].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5903 GTM_gtm_cls0.MCS0_MEM[5903].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5904 GTM_gtm_cls0.MCS0_MEM[5904].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5905 GTM_gtm_cls0.MCS0_MEM[5905].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5906 GTM_gtm_cls0.MCS0_MEM[5906].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5907 GTM_gtm_cls0.MCS0_MEM[5907].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5908 GTM_gtm_cls0.MCS0_MEM[5908].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5909 GTM_gtm_cls0.MCS0_MEM[5909].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5910 GTM_gtm_cls0.MCS0_MEM[5910].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5911 GTM_gtm_cls0.MCS0_MEM[5911].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5912 GTM_gtm_cls0.MCS0_MEM[5912].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5913 GTM_gtm_cls0.MCS0_MEM[5913].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5914 GTM_gtm_cls0.MCS0_MEM[5914].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5915 GTM_gtm_cls0.MCS0_MEM[5915].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5916 GTM_gtm_cls0.MCS0_MEM[5916].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5917 GTM_gtm_cls0.MCS0_MEM[5917].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5918 GTM_gtm_cls0.MCS0_MEM[5918].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5919 GTM_gtm_cls0.MCS0_MEM[5919].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5920 GTM_gtm_cls0.MCS0_MEM[5920].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5921 GTM_gtm_cls0.MCS0_MEM[5921].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5922 GTM_gtm_cls0.MCS0_MEM[5922].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5923 GTM_gtm_cls0.MCS0_MEM[5923].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5924 GTM_gtm_cls0.MCS0_MEM[5924].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5925 GTM_gtm_cls0.MCS0_MEM[5925].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5926 GTM_gtm_cls0.MCS0_MEM[5926].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5927 GTM_gtm_cls0.MCS0_MEM[5927].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5928 GTM_gtm_cls0.MCS0_MEM[5928].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5929 GTM_gtm_cls0.MCS0_MEM[5929].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5930 GTM_gtm_cls0.MCS0_MEM[5930].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5931 GTM_gtm_cls0.MCS0_MEM[5931].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5932 GTM_gtm_cls0.MCS0_MEM[5932].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5933 GTM_gtm_cls0.MCS0_MEM[5933].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5934 GTM_gtm_cls0.MCS0_MEM[5934].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5935 GTM_gtm_cls0.MCS0_MEM[5935].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5936 GTM_gtm_cls0.MCS0_MEM[5936].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5937 GTM_gtm_cls0.MCS0_MEM[5937].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5938 GTM_gtm_cls0.MCS0_MEM[5938].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5939 GTM_gtm_cls0.MCS0_MEM[5939].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5940 GTM_gtm_cls0.MCS0_MEM[5940].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5941 GTM_gtm_cls0.MCS0_MEM[5941].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5942 GTM_gtm_cls0.MCS0_MEM[5942].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5943 GTM_gtm_cls0.MCS0_MEM[5943].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5944 GTM_gtm_cls0.MCS0_MEM[5944].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5945 GTM_gtm_cls0.MCS0_MEM[5945].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5946 GTM_gtm_cls0.MCS0_MEM[5946].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5947 GTM_gtm_cls0.MCS0_MEM[5947].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5948 GTM_gtm_cls0.MCS0_MEM[5948].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5949 GTM_gtm_cls0.MCS0_MEM[5949].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5950 GTM_gtm_cls0.MCS0_MEM[5950].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5951 GTM_gtm_cls0.MCS0_MEM[5951].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5952 GTM_gtm_cls0.MCS0_MEM[5952].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5953 GTM_gtm_cls0.MCS0_MEM[5953].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5954 GTM_gtm_cls0.MCS0_MEM[5954].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5955 GTM_gtm_cls0.MCS0_MEM[5955].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5956 GTM_gtm_cls0.MCS0_MEM[5956].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5957 GTM_gtm_cls0.MCS0_MEM[5957].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5958 GTM_gtm_cls0.MCS0_MEM[5958].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5959 GTM_gtm_cls0.MCS0_MEM[5959].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5960 GTM_gtm_cls0.MCS0_MEM[5960].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5961 GTM_gtm_cls0.MCS0_MEM[5961].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5962 GTM_gtm_cls0.MCS0_MEM[5962].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5963 GTM_gtm_cls0.MCS0_MEM[5963].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5964 GTM_gtm_cls0.MCS0_MEM[5964].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5965 GTM_gtm_cls0.MCS0_MEM[5965].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5966 GTM_gtm_cls0.MCS0_MEM[5966].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5967 GTM_gtm_cls0.MCS0_MEM[5967].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5968 GTM_gtm_cls0.MCS0_MEM[5968].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5969 GTM_gtm_cls0.MCS0_MEM[5969].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5970 GTM_gtm_cls0.MCS0_MEM[5970].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5971 GTM_gtm_cls0.MCS0_MEM[5971].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5972 GTM_gtm_cls0.MCS0_MEM[5972].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5973 GTM_gtm_cls0.MCS0_MEM[5973].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5974 GTM_gtm_cls0.MCS0_MEM[5974].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5975 GTM_gtm_cls0.MCS0_MEM[5975].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5976 GTM_gtm_cls0.MCS0_MEM[5976].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5977 GTM_gtm_cls0.MCS0_MEM[5977].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5978 GTM_gtm_cls0.MCS0_MEM[5978].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5979 GTM_gtm_cls0.MCS0_MEM[5979].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5980 GTM_gtm_cls0.MCS0_MEM[5980].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5981 GTM_gtm_cls0.MCS0_MEM[5981].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5982 GTM_gtm_cls0.MCS0_MEM[5982].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5983 GTM_gtm_cls0.MCS0_MEM[5983].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5984 GTM_gtm_cls0.MCS0_MEM[5984].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5985 GTM_gtm_cls0.MCS0_MEM[5985].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5986 GTM_gtm_cls0.MCS0_MEM[5986].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5987 GTM_gtm_cls0.MCS0_MEM[5987].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5988 GTM_gtm_cls0.MCS0_MEM[5988].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5989 GTM_gtm_cls0.MCS0_MEM[5989].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5990 GTM_gtm_cls0.MCS0_MEM[5990].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5991 GTM_gtm_cls0.MCS0_MEM[5991].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5992 GTM_gtm_cls0.MCS0_MEM[5992].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5993 GTM_gtm_cls0.MCS0_MEM[5993].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5994 GTM_gtm_cls0.MCS0_MEM[5994].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5995 GTM_gtm_cls0.MCS0_MEM[5995].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5996 GTM_gtm_cls0.MCS0_MEM[5996].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5997 GTM_gtm_cls0.MCS0_MEM[5997].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5998 GTM_gtm_cls0.MCS0_MEM[5998].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM5999 GTM_gtm_cls0.MCS0_MEM[5999].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6000 GTM_gtm_cls0.MCS0_MEM[6000].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6001 GTM_gtm_cls0.MCS0_MEM[6001].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6002 GTM_gtm_cls0.MCS0_MEM[6002].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6003 GTM_gtm_cls0.MCS0_MEM[6003].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6004 GTM_gtm_cls0.MCS0_MEM[6004].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6005 GTM_gtm_cls0.MCS0_MEM[6005].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6006 GTM_gtm_cls0.MCS0_MEM[6006].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6007 GTM_gtm_cls0.MCS0_MEM[6007].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6008 GTM_gtm_cls0.MCS0_MEM[6008].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6009 GTM_gtm_cls0.MCS0_MEM[6009].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6010 GTM_gtm_cls0.MCS0_MEM[6010].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6011 GTM_gtm_cls0.MCS0_MEM[6011].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6012 GTM_gtm_cls0.MCS0_MEM[6012].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6013 GTM_gtm_cls0.MCS0_MEM[6013].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6014 GTM_gtm_cls0.MCS0_MEM[6014].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6015 GTM_gtm_cls0.MCS0_MEM[6015].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6016 GTM_gtm_cls0.MCS0_MEM[6016].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6017 GTM_gtm_cls0.MCS0_MEM[6017].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6018 GTM_gtm_cls0.MCS0_MEM[6018].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6019 GTM_gtm_cls0.MCS0_MEM[6019].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6020 GTM_gtm_cls0.MCS0_MEM[6020].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6021 GTM_gtm_cls0.MCS0_MEM[6021].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6022 GTM_gtm_cls0.MCS0_MEM[6022].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6023 GTM_gtm_cls0.MCS0_MEM[6023].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6024 GTM_gtm_cls0.MCS0_MEM[6024].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6025 GTM_gtm_cls0.MCS0_MEM[6025].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6026 GTM_gtm_cls0.MCS0_MEM[6026].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6027 GTM_gtm_cls0.MCS0_MEM[6027].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6028 GTM_gtm_cls0.MCS0_MEM[6028].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6029 GTM_gtm_cls0.MCS0_MEM[6029].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6030 GTM_gtm_cls0.MCS0_MEM[6030].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6031 GTM_gtm_cls0.MCS0_MEM[6031].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6032 GTM_gtm_cls0.MCS0_MEM[6032].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6033 GTM_gtm_cls0.MCS0_MEM[6033].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6034 GTM_gtm_cls0.MCS0_MEM[6034].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6035 GTM_gtm_cls0.MCS0_MEM[6035].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6036 GTM_gtm_cls0.MCS0_MEM[6036].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6037 GTM_gtm_cls0.MCS0_MEM[6037].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6038 GTM_gtm_cls0.MCS0_MEM[6038].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6039 GTM_gtm_cls0.MCS0_MEM[6039].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6040 GTM_gtm_cls0.MCS0_MEM[6040].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6041 GTM_gtm_cls0.MCS0_MEM[6041].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6042 GTM_gtm_cls0.MCS0_MEM[6042].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6043 GTM_gtm_cls0.MCS0_MEM[6043].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6044 GTM_gtm_cls0.MCS0_MEM[6044].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6045 GTM_gtm_cls0.MCS0_MEM[6045].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6046 GTM_gtm_cls0.MCS0_MEM[6046].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6047 GTM_gtm_cls0.MCS0_MEM[6047].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6048 GTM_gtm_cls0.MCS0_MEM[6048].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6049 GTM_gtm_cls0.MCS0_MEM[6049].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6050 GTM_gtm_cls0.MCS0_MEM[6050].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6051 GTM_gtm_cls0.MCS0_MEM[6051].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6052 GTM_gtm_cls0.MCS0_MEM[6052].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6053 GTM_gtm_cls0.MCS0_MEM[6053].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6054 GTM_gtm_cls0.MCS0_MEM[6054].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6055 GTM_gtm_cls0.MCS0_MEM[6055].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6056 GTM_gtm_cls0.MCS0_MEM[6056].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6057 GTM_gtm_cls0.MCS0_MEM[6057].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6058 GTM_gtm_cls0.MCS0_MEM[6058].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6059 GTM_gtm_cls0.MCS0_MEM[6059].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6060 GTM_gtm_cls0.MCS0_MEM[6060].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6061 GTM_gtm_cls0.MCS0_MEM[6061].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6062 GTM_gtm_cls0.MCS0_MEM[6062].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6063 GTM_gtm_cls0.MCS0_MEM[6063].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6064 GTM_gtm_cls0.MCS0_MEM[6064].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6065 GTM_gtm_cls0.MCS0_MEM[6065].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6066 GTM_gtm_cls0.MCS0_MEM[6066].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6067 GTM_gtm_cls0.MCS0_MEM[6067].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6068 GTM_gtm_cls0.MCS0_MEM[6068].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6069 GTM_gtm_cls0.MCS0_MEM[6069].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6070 GTM_gtm_cls0.MCS0_MEM[6070].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6071 GTM_gtm_cls0.MCS0_MEM[6071].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6072 GTM_gtm_cls0.MCS0_MEM[6072].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6073 GTM_gtm_cls0.MCS0_MEM[6073].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6074 GTM_gtm_cls0.MCS0_MEM[6074].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6075 GTM_gtm_cls0.MCS0_MEM[6075].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6076 GTM_gtm_cls0.MCS0_MEM[6076].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6077 GTM_gtm_cls0.MCS0_MEM[6077].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6078 GTM_gtm_cls0.MCS0_MEM[6078].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6079 GTM_gtm_cls0.MCS0_MEM[6079].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6080 GTM_gtm_cls0.MCS0_MEM[6080].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6081 GTM_gtm_cls0.MCS0_MEM[6081].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6082 GTM_gtm_cls0.MCS0_MEM[6082].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6083 GTM_gtm_cls0.MCS0_MEM[6083].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6084 GTM_gtm_cls0.MCS0_MEM[6084].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6085 GTM_gtm_cls0.MCS0_MEM[6085].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6086 GTM_gtm_cls0.MCS0_MEM[6086].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6087 GTM_gtm_cls0.MCS0_MEM[6087].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6088 GTM_gtm_cls0.MCS0_MEM[6088].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6089 GTM_gtm_cls0.MCS0_MEM[6089].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6090 GTM_gtm_cls0.MCS0_MEM[6090].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6091 GTM_gtm_cls0.MCS0_MEM[6091].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6092 GTM_gtm_cls0.MCS0_MEM[6092].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6093 GTM_gtm_cls0.MCS0_MEM[6093].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6094 GTM_gtm_cls0.MCS0_MEM[6094].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6095 GTM_gtm_cls0.MCS0_MEM[6095].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6096 GTM_gtm_cls0.MCS0_MEM[6096].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6097 GTM_gtm_cls0.MCS0_MEM[6097].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6098 GTM_gtm_cls0.MCS0_MEM[6098].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6099 GTM_gtm_cls0.MCS0_MEM[6099].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6100 GTM_gtm_cls0.MCS0_MEM[6100].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6101 GTM_gtm_cls0.MCS0_MEM[6101].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6102 GTM_gtm_cls0.MCS0_MEM[6102].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6103 GTM_gtm_cls0.MCS0_MEM[6103].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6104 GTM_gtm_cls0.MCS0_MEM[6104].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6105 GTM_gtm_cls0.MCS0_MEM[6105].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6106 GTM_gtm_cls0.MCS0_MEM[6106].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6107 GTM_gtm_cls0.MCS0_MEM[6107].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6108 GTM_gtm_cls0.MCS0_MEM[6108].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6109 GTM_gtm_cls0.MCS0_MEM[6109].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6110 GTM_gtm_cls0.MCS0_MEM[6110].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6111 GTM_gtm_cls0.MCS0_MEM[6111].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6112 GTM_gtm_cls0.MCS0_MEM[6112].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6113 GTM_gtm_cls0.MCS0_MEM[6113].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6114 GTM_gtm_cls0.MCS0_MEM[6114].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6115 GTM_gtm_cls0.MCS0_MEM[6115].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6116 GTM_gtm_cls0.MCS0_MEM[6116].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6117 GTM_gtm_cls0.MCS0_MEM[6117].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6118 GTM_gtm_cls0.MCS0_MEM[6118].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6119 GTM_gtm_cls0.MCS0_MEM[6119].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6120 GTM_gtm_cls0.MCS0_MEM[6120].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6121 GTM_gtm_cls0.MCS0_MEM[6121].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6122 GTM_gtm_cls0.MCS0_MEM[6122].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6123 GTM_gtm_cls0.MCS0_MEM[6123].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6124 GTM_gtm_cls0.MCS0_MEM[6124].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6125 GTM_gtm_cls0.MCS0_MEM[6125].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6126 GTM_gtm_cls0.MCS0_MEM[6126].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6127 GTM_gtm_cls0.MCS0_MEM[6127].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6128 GTM_gtm_cls0.MCS0_MEM[6128].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6129 GTM_gtm_cls0.MCS0_MEM[6129].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6130 GTM_gtm_cls0.MCS0_MEM[6130].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6131 GTM_gtm_cls0.MCS0_MEM[6131].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6132 GTM_gtm_cls0.MCS0_MEM[6132].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6133 GTM_gtm_cls0.MCS0_MEM[6133].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6134 GTM_gtm_cls0.MCS0_MEM[6134].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6135 GTM_gtm_cls0.MCS0_MEM[6135].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6136 GTM_gtm_cls0.MCS0_MEM[6136].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6137 GTM_gtm_cls0.MCS0_MEM[6137].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6138 GTM_gtm_cls0.MCS0_MEM[6138].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6139 GTM_gtm_cls0.MCS0_MEM[6139].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6140 GTM_gtm_cls0.MCS0_MEM[6140].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6141 GTM_gtm_cls0.MCS0_MEM[6141].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6142 GTM_gtm_cls0.MCS0_MEM[6142].R /* MCS[i] memory region */
#define GTM_gtm_cls0_MCS0_MEM6143 GTM_gtm_cls0.MCS0_MEM[6143].R /* MCS[i] memory region */

/* GTM_gtm_cls1 */
#define GTM_gtm_cls1_MON_STATUS GTM_gtm_cls1.MON_STATUS.R  /* MON status register */
#define GTM_gtm_cls1_MON_ACTIVITY_0 GTM_gtm_cls1.MON_ACTIVITY_0.R /* MON activity register 0 */
#define GTM_gtm_cls1_MON_ACTIVITY_MCS0 GTM_gtm_cls1.MON_ACTIVITY_MCS0.R /* MON activity register for MCS [j] */
#define GTM_gtm_cls1_MON_ACTIVITY_MCS1 GTM_gtm_cls1.MON_ACTIVITY_MCS1.R /* MON activity register for MCS [j] */
#define GTM_gtm_cls1_MON_ACTIVITY_MCS2 GTM_gtm_cls1.MON_ACTIVITY_MCS2.R /* MON activity register for MCS [j] */
#define GTM_gtm_cls1_MON_ACTIVITY_MCS3 GTM_gtm_cls1.MON_ACTIVITY_MCS3.R /* MON activity register for MCS [j] */
#define GTM_gtm_cls1_CMP_EN  GTM_gtm_cls1.CMP_EN.R         /* CMP comparator enable register */
#define GTM_gtm_cls1_CMP_IRQ_NOTIFY GTM_gtm_cls1.CMP_IRQ_NOTIFY.R /* CMP event notification register */
#define GTM_gtm_cls1_CMP_IRQ_EN GTM_gtm_cls1.CMP_IRQ_EN.R  /* CMP interrupt enable register */
#define GTM_gtm_cls1_CMP_IRQ_FORCINT GTM_gtm_cls1.CMP_IRQ_FORCINT.R /* CMP interrupt force register */
#define GTM_gtm_cls1_CMP_IRQ_MODE GTM_gtm_cls1.CMP_IRQ_MODE.R /* CMP interrupt mode configuration register */
#define GTM_gtm_cls1_CMP_EIRQ_EN GTM_gtm_cls1.CMP_EIRQ_EN.R /* CMP error interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH0_GPR0 GTM_gtm_cls1.TIM1_CH0_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH0_GPR1 GTM_gtm_cls1.TIM1_CH0_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH0_CNT GTM_gtm_cls1.TIM1_CH0_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls1_TIM1_CH0_ECNT GTM_gtm_cls1.TIM1_CH0_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls1_TIM1_CH0_CNTS GTM_gtm_cls1.TIM1_CH0_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls1_TIM1_CH0_TDUC GTM_gtm_cls1.TIM1_CH0_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls1_TIM1_CH0_TDUV GTM_gtm_cls1.TIM1_CH0_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls1_TIM1_CH0_FLT_RE GTM_gtm_cls1.TIM1_CH0_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls1_TIM1_CH0_FLT_FE GTM_gtm_cls1.TIM1_CH0_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls1_TIM1_CH0_CTRL GTM_gtm_cls1.TIM1_CH0_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls1_TIM1_CH0_ECTRL GTM_gtm_cls1.TIM1_CH0_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls1_TIM1_CH0_IRQ_NOTIFY GTM_gtm_cls1.TIM1_CH0_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TIM1_CH0_IRQ_EN GTM_gtm_cls1.TIM1_CH0_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH0_IRQ_FORCINT GTM_gtm_cls1.TIM1_CH0_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TIM1_CH0_IRQ_MODE GTM_gtm_cls1.TIM1_CH0_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_TIM1_CH0_EIRQ_EN GTM_gtm_cls1.TIM1_CH0_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH1_GPR0 GTM_gtm_cls1.TIM1_CH1_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH1_GPR1 GTM_gtm_cls1.TIM1_CH1_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH1_CNT GTM_gtm_cls1.TIM1_CH1_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls1_TIM1_CH1_ECNT GTM_gtm_cls1.TIM1_CH1_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls1_TIM1_CH1_CNTS GTM_gtm_cls1.TIM1_CH1_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls1_TIM1_CH1_TDUC GTM_gtm_cls1.TIM1_CH1_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls1_TIM1_CH1_TDUV GTM_gtm_cls1.TIM1_CH1_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls1_TIM1_CH1_FLT_RE GTM_gtm_cls1.TIM1_CH1_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls1_TIM1_CH1_FLT_FE GTM_gtm_cls1.TIM1_CH1_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls1_TIM1_CH1_CTRL GTM_gtm_cls1.TIM1_CH1_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls1_TIM1_CH1_ECTRL GTM_gtm_cls1.TIM1_CH1_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls1_TIM1_CH1_IRQ_NOTIFY GTM_gtm_cls1.TIM1_CH1_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TIM1_CH1_IRQ_EN GTM_gtm_cls1.TIM1_CH1_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH1_IRQ_FORCINT GTM_gtm_cls1.TIM1_CH1_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TIM1_CH1_IRQ_MODE GTM_gtm_cls1.TIM1_CH1_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_TIM1_CH1_EIRQ_EN GTM_gtm_cls1.TIM1_CH1_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH2_GPR0 GTM_gtm_cls1.TIM1_CH2_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH2_GPR1 GTM_gtm_cls1.TIM1_CH2_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH2_CNT GTM_gtm_cls1.TIM1_CH2_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls1_TIM1_CH2_ECNT GTM_gtm_cls1.TIM1_CH2_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls1_TIM1_CH2_CNTS GTM_gtm_cls1.TIM1_CH2_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls1_TIM1_CH2_TDUC GTM_gtm_cls1.TIM1_CH2_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls1_TIM1_CH2_TDUV GTM_gtm_cls1.TIM1_CH2_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls1_TIM1_CH2_FLT_RE GTM_gtm_cls1.TIM1_CH2_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls1_TIM1_CH2_FLT_FE GTM_gtm_cls1.TIM1_CH2_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls1_TIM1_CH2_CTRL GTM_gtm_cls1.TIM1_CH2_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls1_TIM1_CH2_ECTRL GTM_gtm_cls1.TIM1_CH2_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls1_TIM1_CH2_IRQ_NOTIFY GTM_gtm_cls1.TIM1_CH2_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TIM1_CH2_IRQ_EN GTM_gtm_cls1.TIM1_CH2_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH2_IRQ_FORCINT GTM_gtm_cls1.TIM1_CH2_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TIM1_CH2_IRQ_MODE GTM_gtm_cls1.TIM1_CH2_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_TIM1_CH2_EIRQ_EN GTM_gtm_cls1.TIM1_CH2_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH3_GPR0 GTM_gtm_cls1.TIM1_CH3_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH3_GPR1 GTM_gtm_cls1.TIM1_CH3_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH3_CNT GTM_gtm_cls1.TIM1_CH3_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls1_TIM1_CH3_ECNT GTM_gtm_cls1.TIM1_CH3_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls1_TIM1_CH3_CNTS GTM_gtm_cls1.TIM1_CH3_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls1_TIM1_CH3_TDUC GTM_gtm_cls1.TIM1_CH3_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls1_TIM1_CH3_TDUV GTM_gtm_cls1.TIM1_CH3_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls1_TIM1_CH3_FLT_RE GTM_gtm_cls1.TIM1_CH3_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls1_TIM1_CH3_FLT_FE GTM_gtm_cls1.TIM1_CH3_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls1_TIM1_CH3_CTRL GTM_gtm_cls1.TIM1_CH3_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls1_TIM1_CH3_ECTRL GTM_gtm_cls1.TIM1_CH3_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls1_TIM1_CH3_IRQ_NOTIFY GTM_gtm_cls1.TIM1_CH3_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TIM1_CH3_IRQ_EN GTM_gtm_cls1.TIM1_CH3_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH3_IRQ_FORCINT GTM_gtm_cls1.TIM1_CH3_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TIM1_CH3_IRQ_MODE GTM_gtm_cls1.TIM1_CH3_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_TIM1_CH3_EIRQ_EN GTM_gtm_cls1.TIM1_CH3_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH4_GPR0 GTM_gtm_cls1.TIM1_CH4_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH4_GPR1 GTM_gtm_cls1.TIM1_CH4_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH4_CNT GTM_gtm_cls1.TIM1_CH4_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls1_TIM1_CH4_ECNT GTM_gtm_cls1.TIM1_CH4_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls1_TIM1_CH4_CNTS GTM_gtm_cls1.TIM1_CH4_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls1_TIM1_CH4_TDUC GTM_gtm_cls1.TIM1_CH4_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls1_TIM1_CH4_TDUV GTM_gtm_cls1.TIM1_CH4_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls1_TIM1_CH4_FLT_RE GTM_gtm_cls1.TIM1_CH4_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls1_TIM1_CH4_FLT_FE GTM_gtm_cls1.TIM1_CH4_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls1_TIM1_CH4_CTRL GTM_gtm_cls1.TIM1_CH4_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls1_TIM1_CH4_ECTRL GTM_gtm_cls1.TIM1_CH4_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls1_TIM1_CH4_IRQ_NOTIFY GTM_gtm_cls1.TIM1_CH4_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TIM1_CH4_IRQ_EN GTM_gtm_cls1.TIM1_CH4_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH4_IRQ_FORCINT GTM_gtm_cls1.TIM1_CH4_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TIM1_CH4_IRQ_MODE GTM_gtm_cls1.TIM1_CH4_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_TIM1_CH4_EIRQ_EN GTM_gtm_cls1.TIM1_CH4_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH5_GPR0 GTM_gtm_cls1.TIM1_CH5_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH5_GPR1 GTM_gtm_cls1.TIM1_CH5_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH5_CNT GTM_gtm_cls1.TIM1_CH5_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls1_TIM1_CH5_ECNT GTM_gtm_cls1.TIM1_CH5_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls1_TIM1_CH5_CNTS GTM_gtm_cls1.TIM1_CH5_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls1_TIM1_CH5_TDUC GTM_gtm_cls1.TIM1_CH5_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls1_TIM1_CH5_TDUV GTM_gtm_cls1.TIM1_CH5_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls1_TIM1_CH5_FLT_RE GTM_gtm_cls1.TIM1_CH5_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls1_TIM1_CH5_FLT_FE GTM_gtm_cls1.TIM1_CH5_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls1_TIM1_CH5_CTRL GTM_gtm_cls1.TIM1_CH5_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls1_TIM1_CH5_ECTRL GTM_gtm_cls1.TIM1_CH5_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls1_TIM1_CH5_IRQ_NOTIFY GTM_gtm_cls1.TIM1_CH5_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TIM1_CH5_IRQ_EN GTM_gtm_cls1.TIM1_CH5_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH5_IRQ_FORCINT GTM_gtm_cls1.TIM1_CH5_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TIM1_CH5_IRQ_MODE GTM_gtm_cls1.TIM1_CH5_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_TIM1_CH5_EIRQ_EN GTM_gtm_cls1.TIM1_CH5_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH6_GPR0 GTM_gtm_cls1.TIM1_CH6_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH6_GPR1 GTM_gtm_cls1.TIM1_CH6_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH6_CNT GTM_gtm_cls1.TIM1_CH6_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls1_TIM1_CH6_ECNT GTM_gtm_cls1.TIM1_CH6_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls1_TIM1_CH6_CNTS GTM_gtm_cls1.TIM1_CH6_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls1_TIM1_CH6_TDUC GTM_gtm_cls1.TIM1_CH6_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls1_TIM1_CH6_TDUV GTM_gtm_cls1.TIM1_CH6_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls1_TIM1_CH6_FLT_RE GTM_gtm_cls1.TIM1_CH6_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls1_TIM1_CH6_FLT_FE GTM_gtm_cls1.TIM1_CH6_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls1_TIM1_CH6_CTRL GTM_gtm_cls1.TIM1_CH6_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls1_TIM1_CH6_ECTRL GTM_gtm_cls1.TIM1_CH6_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls1_TIM1_CH6_IRQ_NOTIFY GTM_gtm_cls1.TIM1_CH6_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TIM1_CH6_IRQ_EN GTM_gtm_cls1.TIM1_CH6_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH6_IRQ_FORCINT GTM_gtm_cls1.TIM1_CH6_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TIM1_CH6_IRQ_MODE GTM_gtm_cls1.TIM1_CH6_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_TIM1_CH6_EIRQ_EN GTM_gtm_cls1.TIM1_CH6_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH7_GPR0 GTM_gtm_cls1.TIM1_CH7_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH7_GPR1 GTM_gtm_cls1.TIM1_CH7_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls1_TIM1_CH7_CNT GTM_gtm_cls1.TIM1_CH7_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls1_TIM1_CH7_ECNT GTM_gtm_cls1.TIM1_CH7_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls1_TIM1_CH7_CNTS GTM_gtm_cls1.TIM1_CH7_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls1_TIM1_CH7_TDUC GTM_gtm_cls1.TIM1_CH7_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls1_TIM1_CH7_TDUV GTM_gtm_cls1.TIM1_CH7_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls1_TIM1_CH7_FLT_RE GTM_gtm_cls1.TIM1_CH7_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls1_TIM1_CH7_FLT_FE GTM_gtm_cls1.TIM1_CH7_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls1_TIM1_CH7_CTRL GTM_gtm_cls1.TIM1_CH7_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls1_TIM1_CH7_ECTRL GTM_gtm_cls1.TIM1_CH7_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls1_TIM1_CH7_IRQ_NOTIFY GTM_gtm_cls1.TIM1_CH7_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TIM1_CH7_IRQ_EN GTM_gtm_cls1.TIM1_CH7_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TIM1_CH7_IRQ_FORCINT GTM_gtm_cls1.TIM1_CH7_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TIM1_CH7_IRQ_MODE GTM_gtm_cls1.TIM1_CH7_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_TIM1_CH7_EIRQ_EN GTM_gtm_cls1.TIM1_CH7_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls1_TIM1_INP_VAL GTM_gtm_cls1.TIM1_INP_VAL.R /* TIM[i] input value observation register */
#define GTM_gtm_cls1_TIM1_IN_SRC GTM_gtm_cls1.TIM1_IN_SRC.R /* TIM[i] AUX IN source selection register */
#define GTM_gtm_cls1_TIM1_RST GTM_gtm_cls1.TIM1_RST.R      /* TIM[i] global software reset register */
#define GTM_gtm_cls1_TOM1_CH0_CTRL GTM_gtm_cls1.TOM1_CH0_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH0_SR0 GTM_gtm_cls1.TOM1_CH0_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH0_SR1 GTM_gtm_cls1.TOM1_CH0_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH0_CM0 GTM_gtm_cls1.TOM1_CH0_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH0_CM1 GTM_gtm_cls1.TOM1_CH0_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH0_CN0 GTM_gtm_cls1.TOM1_CH0_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH0_STAT GTM_gtm_cls1.TOM1_CH0_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH0_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH0_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH0_IRQ_EN GTM_gtm_cls1.TOM1_CH0_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH0_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH0_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH0_IRQ_MODE GTM_gtm_cls1.TOM1_CH0_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH0_CTRL_SR GTM_gtm_cls1.TOM1_CH0_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH1_CTRL GTM_gtm_cls1.TOM1_CH1_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH1_SR0 GTM_gtm_cls1.TOM1_CH1_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH1_SR1 GTM_gtm_cls1.TOM1_CH1_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH1_CM0 GTM_gtm_cls1.TOM1_CH1_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH1_CM1 GTM_gtm_cls1.TOM1_CH1_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH1_CN0 GTM_gtm_cls1.TOM1_CH1_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH1_STAT GTM_gtm_cls1.TOM1_CH1_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH1_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH1_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH1_IRQ_EN GTM_gtm_cls1.TOM1_CH1_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH1_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH1_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH1_IRQ_MODE GTM_gtm_cls1.TOM1_CH1_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH1_CTRL_SR GTM_gtm_cls1.TOM1_CH1_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH2_CTRL GTM_gtm_cls1.TOM1_CH2_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH2_SR0 GTM_gtm_cls1.TOM1_CH2_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH2_SR1 GTM_gtm_cls1.TOM1_CH2_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH2_CM0 GTM_gtm_cls1.TOM1_CH2_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH2_CM1 GTM_gtm_cls1.TOM1_CH2_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH2_CN0 GTM_gtm_cls1.TOM1_CH2_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH2_STAT GTM_gtm_cls1.TOM1_CH2_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH2_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH2_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH2_IRQ_EN GTM_gtm_cls1.TOM1_CH2_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH2_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH2_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH2_IRQ_MODE GTM_gtm_cls1.TOM1_CH2_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH2_CTRL_SR GTM_gtm_cls1.TOM1_CH2_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH3_CTRL GTM_gtm_cls1.TOM1_CH3_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH3_SR0 GTM_gtm_cls1.TOM1_CH3_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH3_SR1 GTM_gtm_cls1.TOM1_CH3_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH3_CM0 GTM_gtm_cls1.TOM1_CH3_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH3_CM1 GTM_gtm_cls1.TOM1_CH3_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH3_CN0 GTM_gtm_cls1.TOM1_CH3_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH3_STAT GTM_gtm_cls1.TOM1_CH3_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH3_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH3_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH3_IRQ_EN GTM_gtm_cls1.TOM1_CH3_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH3_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH3_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH3_IRQ_MODE GTM_gtm_cls1.TOM1_CH3_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH3_CTRL_SR GTM_gtm_cls1.TOM1_CH3_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH4_CTRL GTM_gtm_cls1.TOM1_CH4_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH4_SR0 GTM_gtm_cls1.TOM1_CH4_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH4_SR1 GTM_gtm_cls1.TOM1_CH4_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH4_CM0 GTM_gtm_cls1.TOM1_CH4_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH4_CM1 GTM_gtm_cls1.TOM1_CH4_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH4_CN0 GTM_gtm_cls1.TOM1_CH4_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH4_STAT GTM_gtm_cls1.TOM1_CH4_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH4_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH4_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH4_IRQ_EN GTM_gtm_cls1.TOM1_CH4_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH4_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH4_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH4_IRQ_MODE GTM_gtm_cls1.TOM1_CH4_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH4_CTRL_SR GTM_gtm_cls1.TOM1_CH4_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH5_CTRL GTM_gtm_cls1.TOM1_CH5_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH5_SR0 GTM_gtm_cls1.TOM1_CH5_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH5_SR1 GTM_gtm_cls1.TOM1_CH5_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH5_CM0 GTM_gtm_cls1.TOM1_CH5_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH5_CM1 GTM_gtm_cls1.TOM1_CH5_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH5_CN0 GTM_gtm_cls1.TOM1_CH5_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH5_STAT GTM_gtm_cls1.TOM1_CH5_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH5_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH5_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH5_IRQ_EN GTM_gtm_cls1.TOM1_CH5_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH5_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH5_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH5_IRQ_MODE GTM_gtm_cls1.TOM1_CH5_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH5_CTRL_SR GTM_gtm_cls1.TOM1_CH5_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH6_CTRL GTM_gtm_cls1.TOM1_CH6_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH6_SR0 GTM_gtm_cls1.TOM1_CH6_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH6_SR1 GTM_gtm_cls1.TOM1_CH6_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH6_CM0 GTM_gtm_cls1.TOM1_CH6_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH6_CM1 GTM_gtm_cls1.TOM1_CH6_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH6_CN0 GTM_gtm_cls1.TOM1_CH6_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH6_STAT GTM_gtm_cls1.TOM1_CH6_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH6_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH6_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH6_IRQ_EN GTM_gtm_cls1.TOM1_CH6_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH6_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH6_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH6_IRQ_MODE GTM_gtm_cls1.TOM1_CH6_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH6_CTRL_SR GTM_gtm_cls1.TOM1_CH6_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH7_CTRL GTM_gtm_cls1.TOM1_CH7_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH7_SR0 GTM_gtm_cls1.TOM1_CH7_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH7_SR1 GTM_gtm_cls1.TOM1_CH7_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH7_CM0 GTM_gtm_cls1.TOM1_CH7_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH7_CM1 GTM_gtm_cls1.TOM1_CH7_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH7_CN0 GTM_gtm_cls1.TOM1_CH7_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH7_STAT GTM_gtm_cls1.TOM1_CH7_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH7_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH7_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH7_IRQ_EN GTM_gtm_cls1.TOM1_CH7_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH7_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH7_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH7_IRQ_MODE GTM_gtm_cls1.TOM1_CH7_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH7_CTRL_SR GTM_gtm_cls1.TOM1_CH7_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH8_CTRL GTM_gtm_cls1.TOM1_CH8_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH8_SR0 GTM_gtm_cls1.TOM1_CH8_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH8_SR1 GTM_gtm_cls1.TOM1_CH8_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH8_CM0 GTM_gtm_cls1.TOM1_CH8_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH8_CM1 GTM_gtm_cls1.TOM1_CH8_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH8_CN0 GTM_gtm_cls1.TOM1_CH8_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH8_STAT GTM_gtm_cls1.TOM1_CH8_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH8_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH8_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH8_IRQ_EN GTM_gtm_cls1.TOM1_CH8_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH8_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH8_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH8_IRQ_MODE GTM_gtm_cls1.TOM1_CH8_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH8_CTRL_SR GTM_gtm_cls1.TOM1_CH8_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH9_CTRL GTM_gtm_cls1.TOM1_CH9_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH9_SR0 GTM_gtm_cls1.TOM1_CH9_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH9_SR1 GTM_gtm_cls1.TOM1_CH9_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH9_CM0 GTM_gtm_cls1.TOM1_CH9_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH9_CM1 GTM_gtm_cls1.TOM1_CH9_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH9_CN0 GTM_gtm_cls1.TOM1_CH9_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH9_STAT GTM_gtm_cls1.TOM1_CH9_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH9_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH9_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH9_IRQ_EN GTM_gtm_cls1.TOM1_CH9_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH9_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH9_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH9_IRQ_MODE GTM_gtm_cls1.TOM1_CH9_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH9_CTRL_SR GTM_gtm_cls1.TOM1_CH9_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH10_CTRL GTM_gtm_cls1.TOM1_CH10_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH10_SR0 GTM_gtm_cls1.TOM1_CH10_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH10_SR1 GTM_gtm_cls1.TOM1_CH10_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH10_CM0 GTM_gtm_cls1.TOM1_CH10_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH10_CM1 GTM_gtm_cls1.TOM1_CH10_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH10_CN0 GTM_gtm_cls1.TOM1_CH10_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH10_STAT GTM_gtm_cls1.TOM1_CH10_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH10_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH10_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH10_IRQ_EN GTM_gtm_cls1.TOM1_CH10_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH10_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH10_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH10_IRQ_MODE GTM_gtm_cls1.TOM1_CH10_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH10_CTRL_SR GTM_gtm_cls1.TOM1_CH10_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH11_CTRL GTM_gtm_cls1.TOM1_CH11_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH11_SR0 GTM_gtm_cls1.TOM1_CH11_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH11_SR1 GTM_gtm_cls1.TOM1_CH11_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH11_CM0 GTM_gtm_cls1.TOM1_CH11_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH11_CM1 GTM_gtm_cls1.TOM1_CH11_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH11_CN0 GTM_gtm_cls1.TOM1_CH11_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH11_STAT GTM_gtm_cls1.TOM1_CH11_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH11_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH11_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH11_IRQ_EN GTM_gtm_cls1.TOM1_CH11_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH11_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH11_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH11_IRQ_MODE GTM_gtm_cls1.TOM1_CH11_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH11_CTRL_SR GTM_gtm_cls1.TOM1_CH11_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH12_CTRL GTM_gtm_cls1.TOM1_CH12_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH12_SR0 GTM_gtm_cls1.TOM1_CH12_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH12_SR1 GTM_gtm_cls1.TOM1_CH12_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH12_CM0 GTM_gtm_cls1.TOM1_CH12_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH12_CM1 GTM_gtm_cls1.TOM1_CH12_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH12_CN0 GTM_gtm_cls1.TOM1_CH12_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH12_STAT GTM_gtm_cls1.TOM1_CH12_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH12_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH12_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH12_IRQ_EN GTM_gtm_cls1.TOM1_CH12_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH12_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH12_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH12_IRQ_MODE GTM_gtm_cls1.TOM1_CH12_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH12_CTRL_SR GTM_gtm_cls1.TOM1_CH12_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH13_CTRL GTM_gtm_cls1.TOM1_CH13_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH13_SR0 GTM_gtm_cls1.TOM1_CH13_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH13_SR1 GTM_gtm_cls1.TOM1_CH13_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH13_CM0 GTM_gtm_cls1.TOM1_CH13_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH13_CM1 GTM_gtm_cls1.TOM1_CH13_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH13_CN0 GTM_gtm_cls1.TOM1_CH13_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH13_STAT GTM_gtm_cls1.TOM1_CH13_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH13_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH13_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH13_IRQ_EN GTM_gtm_cls1.TOM1_CH13_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH13_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH13_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH13_IRQ_MODE GTM_gtm_cls1.TOM1_CH13_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH13_CTRL_SR GTM_gtm_cls1.TOM1_CH13_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH14_CTRL GTM_gtm_cls1.TOM1_CH14_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH14_SR0 GTM_gtm_cls1.TOM1_CH14_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH14_SR1 GTM_gtm_cls1.TOM1_CH14_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH14_CM0 GTM_gtm_cls1.TOM1_CH14_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH14_CM1 GTM_gtm_cls1.TOM1_CH14_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH14_CN0 GTM_gtm_cls1.TOM1_CH14_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH14_STAT GTM_gtm_cls1.TOM1_CH14_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH14_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH14_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH14_IRQ_EN GTM_gtm_cls1.TOM1_CH14_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH14_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH14_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH14_IRQ_MODE GTM_gtm_cls1.TOM1_CH14_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH14_CTRL_SR GTM_gtm_cls1.TOM1_CH14_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_CH15_CTRL GTM_gtm_cls1.TOM1_CH15_CTRL.R /* TOM[i] channel [x] control register */
#define GTM_gtm_cls1_TOM1_CH15_SR0 GTM_gtm_cls1.TOM1_CH15_SR0.R /* TOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH15_SR1 GTM_gtm_cls1.TOM1_CH15_SR1.R /* TOM[i] channel [x] CCU1 compare shadow register */
#define GTM_gtm_cls1_TOM1_CH15_CM0 GTM_gtm_cls1.TOM1_CH15_CM0.R /* TOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_TOM1_CH15_CM1 GTM_gtm_cls1.TOM1_CH15_CM1.R /* TOM[i] channel [x] CCU1 compare register */
#define GTM_gtm_cls1_TOM1_CH15_CN0 GTM_gtm_cls1.TOM1_CH15_CN0.R /* TOM[i] channel [x] CCU0 counter */
#define GTM_gtm_cls1_TOM1_CH15_STAT GTM_gtm_cls1.TOM1_CH15_STAT.R /* TOM[i] channel [x] status register */
#define GTM_gtm_cls1_TOM1_CH15_IRQ_NOTIFY GTM_gtm_cls1.TOM1_CH15_IRQ_NOTIFY.R /* TOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_TOM1_CH15_IRQ_EN GTM_gtm_cls1.TOM1_CH15_IRQ_EN.R /* TOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_TOM1_CH15_IRQ_FORCINT GTM_gtm_cls1.TOM1_CH15_IRQ_FORCINT.R /* TOM[i] channel [x] force interrupt register */
#define GTM_gtm_cls1_TOM1_CH15_IRQ_MODE GTM_gtm_cls1.TOM1_CH15_IRQ_MODE.R /* TOM[i] channel [x] interrupt mode register */
#define GTM_gtm_cls1_TOM1_CH15_CTRL_SR GTM_gtm_cls1.TOM1_CH15_CTRL_SR.R /* TOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_TOM1_TGC0_GLB_CTRL GTM_gtm_cls1.TOM1_TGC0_GLB_CTRL.R /* TOM[i] TGC [g] global control register */
#define GTM_gtm_cls1_TOM1_TGC0_ACT_TB GTM_gtm_cls1.TOM1_TGC0_ACT_TB.R /* TOM[i] TGC [g] action time base register */
#define GTM_gtm_cls1_TOM1_TGC0_FUPD_CTRL GTM_gtm_cls1.TOM1_TGC0_FUPD_CTRL.R /* TOM[i] TGC [g] force update control register */
#define GTM_gtm_cls1_TOM1_TGC0_INT_TRIG GTM_gtm_cls1.TOM1_TGC0_INT_TRIG.R /* TOM[i] TGC [g] internal trigger control register */
#define GTM_gtm_cls1_TOM1_TGC0_ENDIS_CTRL GTM_gtm_cls1.TOM1_TGC0_ENDIS_CTRL.R /* TOM[i] TGC [g] enable/disable control register */
#define GTM_gtm_cls1_TOM1_TGC0_ENDIS_STAT GTM_gtm_cls1.TOM1_TGC0_ENDIS_STAT.R /* TOM[i] TGC [g] enable/disable status register */
#define GTM_gtm_cls1_TOM1_TGC0_OUTEN_CTRL GTM_gtm_cls1.TOM1_TGC0_OUTEN_CTRL.R /* TOM[i] TGC [g] output enable control register */
#define GTM_gtm_cls1_TOM1_TGC0_OUTEN_STAT GTM_gtm_cls1.TOM1_TGC0_OUTEN_STAT.R /* TOM[i] TGC [g] output enable status register */
#define GTM_gtm_cls1_TOM1_TGC1_GLB_CTRL GTM_gtm_cls1.TOM1_TGC1_GLB_CTRL.R /* TOM[i] TGC [g] global control register */
#define GTM_gtm_cls1_TOM1_TGC1_ACT_TB GTM_gtm_cls1.TOM1_TGC1_ACT_TB.R /* TOM[i] TGC [g] action time base register */
#define GTM_gtm_cls1_TOM1_TGC1_FUPD_CTRL GTM_gtm_cls1.TOM1_TGC1_FUPD_CTRL.R /* TOM[i] TGC [g] force update control register */
#define GTM_gtm_cls1_TOM1_TGC1_INT_TRIG GTM_gtm_cls1.TOM1_TGC1_INT_TRIG.R /* TOM[i] TGC [g] internal trigger control register */
#define GTM_gtm_cls1_TOM1_TGC1_ENDIS_CTRL GTM_gtm_cls1.TOM1_TGC1_ENDIS_CTRL.R /* TOM[i] TGC [g] enable/disable control register */
#define GTM_gtm_cls1_TOM1_TGC1_ENDIS_STAT GTM_gtm_cls1.TOM1_TGC1_ENDIS_STAT.R /* TOM[i] TGC [g] enable/disable status register */
#define GTM_gtm_cls1_TOM1_TGC1_OUTEN_CTRL GTM_gtm_cls1.TOM1_TGC1_OUTEN_CTRL.R /* TOM[i] TGC [g] output enable control register */
#define GTM_gtm_cls1_TOM1_TGC1_OUTEN_STAT GTM_gtm_cls1.TOM1_TGC1_OUTEN_STAT.R /* TOM[i] TGC [g] output enable status register */
#define GTM_gtm_cls1_ATOM1_CH0_RDADDR GTM_gtm_cls1.ATOM1_CH0_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls1_ATOM1_CH0_CTRL GTM_gtm_cls1.ATOM1_CH0_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls1_ATOM1_CH0_SR0 GTM_gtm_cls1.ATOM1_CH0_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH0_SR1 GTM_gtm_cls1.ATOM1_CH0_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH0_CM0 GTM_gtm_cls1.ATOM1_CH0_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH0_CM1 GTM_gtm_cls1.ATOM1_CH0_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH0_CN0 GTM_gtm_cls1.ATOM1_CH0_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls1_ATOM1_CH0_STAT GTM_gtm_cls1.ATOM1_CH0_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls1_ATOM1_CH0_IRQ_NOTIFY GTM_gtm_cls1.ATOM1_CH0_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_ATOM1_CH0_IRQ_EN GTM_gtm_cls1.ATOM1_CH0_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_ATOM1_CH0_IRQ_FORCINT GTM_gtm_cls1.ATOM1_CH0_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls1_ATOM1_CH0_IRQ_MODE GTM_gtm_cls1.ATOM1_CH0_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_ATOM1_CH0_CTRL2 GTM_gtm_cls1.ATOM1_CH0_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls1_ATOM1_CH0_CTRL_SR GTM_gtm_cls1.ATOM1_CH0_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_ATOM1_CH1_RDADDR GTM_gtm_cls1.ATOM1_CH1_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls1_ATOM1_CH1_CTRL GTM_gtm_cls1.ATOM1_CH1_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls1_ATOM1_CH1_SR0 GTM_gtm_cls1.ATOM1_CH1_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH1_SR1 GTM_gtm_cls1.ATOM1_CH1_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH1_CM0 GTM_gtm_cls1.ATOM1_CH1_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH1_CM1 GTM_gtm_cls1.ATOM1_CH1_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH1_CN0 GTM_gtm_cls1.ATOM1_CH1_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls1_ATOM1_CH1_STAT GTM_gtm_cls1.ATOM1_CH1_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls1_ATOM1_CH1_IRQ_NOTIFY GTM_gtm_cls1.ATOM1_CH1_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_ATOM1_CH1_IRQ_EN GTM_gtm_cls1.ATOM1_CH1_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_ATOM1_CH1_IRQ_FORCINT GTM_gtm_cls1.ATOM1_CH1_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls1_ATOM1_CH1_IRQ_MODE GTM_gtm_cls1.ATOM1_CH1_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_ATOM1_CH1_CTRL2 GTM_gtm_cls1.ATOM1_CH1_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls1_ATOM1_CH1_CTRL_SR GTM_gtm_cls1.ATOM1_CH1_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_ATOM1_CH2_RDADDR GTM_gtm_cls1.ATOM1_CH2_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls1_ATOM1_CH2_CTRL GTM_gtm_cls1.ATOM1_CH2_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls1_ATOM1_CH2_SR0 GTM_gtm_cls1.ATOM1_CH2_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH2_SR1 GTM_gtm_cls1.ATOM1_CH2_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH2_CM0 GTM_gtm_cls1.ATOM1_CH2_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH2_CM1 GTM_gtm_cls1.ATOM1_CH2_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH2_CN0 GTM_gtm_cls1.ATOM1_CH2_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls1_ATOM1_CH2_STAT GTM_gtm_cls1.ATOM1_CH2_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls1_ATOM1_CH2_IRQ_NOTIFY GTM_gtm_cls1.ATOM1_CH2_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_ATOM1_CH2_IRQ_EN GTM_gtm_cls1.ATOM1_CH2_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_ATOM1_CH2_IRQ_FORCINT GTM_gtm_cls1.ATOM1_CH2_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls1_ATOM1_CH2_IRQ_MODE GTM_gtm_cls1.ATOM1_CH2_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_ATOM1_CH2_CTRL2 GTM_gtm_cls1.ATOM1_CH2_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls1_ATOM1_CH2_CTRL_SR GTM_gtm_cls1.ATOM1_CH2_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_ATOM1_CH3_RDADDR GTM_gtm_cls1.ATOM1_CH3_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls1_ATOM1_CH3_CTRL GTM_gtm_cls1.ATOM1_CH3_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls1_ATOM1_CH3_SR0 GTM_gtm_cls1.ATOM1_CH3_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH3_SR1 GTM_gtm_cls1.ATOM1_CH3_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH3_CM0 GTM_gtm_cls1.ATOM1_CH3_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH3_CM1 GTM_gtm_cls1.ATOM1_CH3_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH3_CN0 GTM_gtm_cls1.ATOM1_CH3_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls1_ATOM1_CH3_STAT GTM_gtm_cls1.ATOM1_CH3_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls1_ATOM1_CH3_IRQ_NOTIFY GTM_gtm_cls1.ATOM1_CH3_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_ATOM1_CH3_IRQ_EN GTM_gtm_cls1.ATOM1_CH3_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_ATOM1_CH3_IRQ_FORCINT GTM_gtm_cls1.ATOM1_CH3_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls1_ATOM1_CH3_IRQ_MODE GTM_gtm_cls1.ATOM1_CH3_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_ATOM1_CH3_CTRL2 GTM_gtm_cls1.ATOM1_CH3_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls1_ATOM1_CH3_CTRL_SR GTM_gtm_cls1.ATOM1_CH3_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_ATOM1_CH4_RDADDR GTM_gtm_cls1.ATOM1_CH4_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls1_ATOM1_CH4_CTRL GTM_gtm_cls1.ATOM1_CH4_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls1_ATOM1_CH4_SR0 GTM_gtm_cls1.ATOM1_CH4_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH4_SR1 GTM_gtm_cls1.ATOM1_CH4_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH4_CM0 GTM_gtm_cls1.ATOM1_CH4_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH4_CM1 GTM_gtm_cls1.ATOM1_CH4_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH4_CN0 GTM_gtm_cls1.ATOM1_CH4_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls1_ATOM1_CH4_STAT GTM_gtm_cls1.ATOM1_CH4_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls1_ATOM1_CH4_IRQ_NOTIFY GTM_gtm_cls1.ATOM1_CH4_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_ATOM1_CH4_IRQ_EN GTM_gtm_cls1.ATOM1_CH4_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_ATOM1_CH4_IRQ_FORCINT GTM_gtm_cls1.ATOM1_CH4_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls1_ATOM1_CH4_IRQ_MODE GTM_gtm_cls1.ATOM1_CH4_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_ATOM1_CH4_CTRL2 GTM_gtm_cls1.ATOM1_CH4_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls1_ATOM1_CH4_CTRL_SR GTM_gtm_cls1.ATOM1_CH4_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_ATOM1_CH5_RDADDR GTM_gtm_cls1.ATOM1_CH5_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls1_ATOM1_CH5_CTRL GTM_gtm_cls1.ATOM1_CH5_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls1_ATOM1_CH5_SR0 GTM_gtm_cls1.ATOM1_CH5_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH5_SR1 GTM_gtm_cls1.ATOM1_CH5_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH5_CM0 GTM_gtm_cls1.ATOM1_CH5_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH5_CM1 GTM_gtm_cls1.ATOM1_CH5_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH5_CN0 GTM_gtm_cls1.ATOM1_CH5_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls1_ATOM1_CH5_STAT GTM_gtm_cls1.ATOM1_CH5_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls1_ATOM1_CH5_IRQ_NOTIFY GTM_gtm_cls1.ATOM1_CH5_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_ATOM1_CH5_IRQ_EN GTM_gtm_cls1.ATOM1_CH5_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_ATOM1_CH5_IRQ_FORCINT GTM_gtm_cls1.ATOM1_CH5_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls1_ATOM1_CH5_IRQ_MODE GTM_gtm_cls1.ATOM1_CH5_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_ATOM1_CH5_CTRL2 GTM_gtm_cls1.ATOM1_CH5_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls1_ATOM1_CH5_CTRL_SR GTM_gtm_cls1.ATOM1_CH5_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_ATOM1_CH6_RDADDR GTM_gtm_cls1.ATOM1_CH6_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls1_ATOM1_CH6_CTRL GTM_gtm_cls1.ATOM1_CH6_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls1_ATOM1_CH6_SR0 GTM_gtm_cls1.ATOM1_CH6_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH6_SR1 GTM_gtm_cls1.ATOM1_CH6_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH6_CM0 GTM_gtm_cls1.ATOM1_CH6_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH6_CM1 GTM_gtm_cls1.ATOM1_CH6_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH6_CN0 GTM_gtm_cls1.ATOM1_CH6_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls1_ATOM1_CH6_STAT GTM_gtm_cls1.ATOM1_CH6_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls1_ATOM1_CH6_IRQ_NOTIFY GTM_gtm_cls1.ATOM1_CH6_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_ATOM1_CH6_IRQ_EN GTM_gtm_cls1.ATOM1_CH6_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_ATOM1_CH6_IRQ_FORCINT GTM_gtm_cls1.ATOM1_CH6_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls1_ATOM1_CH6_IRQ_MODE GTM_gtm_cls1.ATOM1_CH6_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_ATOM1_CH6_CTRL2 GTM_gtm_cls1.ATOM1_CH6_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls1_ATOM1_CH6_CTRL_SR GTM_gtm_cls1.ATOM1_CH6_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_ATOM1_CH7_RDADDR GTM_gtm_cls1.ATOM1_CH7_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls1_ATOM1_CH7_CTRL GTM_gtm_cls1.ATOM1_CH7_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls1_ATOM1_CH7_SR0 GTM_gtm_cls1.ATOM1_CH7_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH7_SR1 GTM_gtm_cls1.ATOM1_CH7_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls1_ATOM1_CH7_CM0 GTM_gtm_cls1.ATOM1_CH7_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH7_CM1 GTM_gtm_cls1.ATOM1_CH7_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls1_ATOM1_CH7_CN0 GTM_gtm_cls1.ATOM1_CH7_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls1_ATOM1_CH7_STAT GTM_gtm_cls1.ATOM1_CH7_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls1_ATOM1_CH7_IRQ_NOTIFY GTM_gtm_cls1.ATOM1_CH7_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls1_ATOM1_CH7_IRQ_EN GTM_gtm_cls1.ATOM1_CH7_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls1_ATOM1_CH7_IRQ_FORCINT GTM_gtm_cls1.ATOM1_CH7_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls1_ATOM1_CH7_IRQ_MODE GTM_gtm_cls1.ATOM1_CH7_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls1_ATOM1_CH7_CTRL2 GTM_gtm_cls1.ATOM1_CH7_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls1_ATOM1_CH7_CTRL_SR GTM_gtm_cls1.ATOM1_CH7_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls1_ATOM1_AGC_GLB_CTRL GTM_gtm_cls1.ATOM1_AGC_GLB_CTRL.R /* ATOM[i] AGC global control register */
#define GTM_gtm_cls1_ATOM1_AGC_ENDIS_CTRL GTM_gtm_cls1.ATOM1_AGC_ENDIS_CTRL.R /* ATOM[i] AGC enable/disable control register */
#define GTM_gtm_cls1_ATOM1_AGC_ENDIS_STAT GTM_gtm_cls1.ATOM1_AGC_ENDIS_STAT.R /* ATOM[i] AGC enable/disable status register */
#define GTM_gtm_cls1_ATOM1_AGC_ACT_TB GTM_gtm_cls1.ATOM1_AGC_ACT_TB.R /* ATOM[i] AGC action time base register */
#define GTM_gtm_cls1_ATOM1_AGC_OUTEN_CTRL GTM_gtm_cls1.ATOM1_AGC_OUTEN_CTRL.R /* ATOM[i] AGC output enable control register */
#define GTM_gtm_cls1_ATOM1_AGC_OUTEN_STAT GTM_gtm_cls1.ATOM1_AGC_OUTEN_STAT.R /* ATOM[i] AGC output enable status register */
#define GTM_gtm_cls1_ATOM1_AGC_FUPD_CTRL GTM_gtm_cls1.ATOM1_AGC_FUPD_CTRL.R /* ATOM[i] AGC force update control register */
#define GTM_gtm_cls1_ATOM1_AGC_INT_TRIG GTM_gtm_cls1.ATOM1_AGC_INT_TRIG.R /* ATOM[i] AGC internal trigger control register */
#define GTM_gtm_cls1_MCS1_CH0_R0 GTM_gtm_cls1.MCS1_CH0_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH0_R1 GTM_gtm_cls1.MCS1_CH0_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH0_R2 GTM_gtm_cls1.MCS1_CH0_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH0_R3 GTM_gtm_cls1.MCS1_CH0_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH0_R4 GTM_gtm_cls1.MCS1_CH0_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH0_R5 GTM_gtm_cls1.MCS1_CH0_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH0_R6 GTM_gtm_cls1.MCS1_CH0_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH0_R7 GTM_gtm_cls1.MCS1_CH0_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH0_CTRL GTM_gtm_cls1.MCS1_CH0_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls1_MCS1_CH0_ACB GTM_gtm_cls1.MCS1_CH0_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls1_MCS1_CH0_MHB GTM_gtm_cls1.MCS1_CH0_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls1_MCS1_CH0_PC GTM_gtm_cls1.MCS1_CH0_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls1_MCS1_CH0_IRQ_NOTIFY GTM_gtm_cls1.MCS1_CH0_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls1_MCS1_CH0_IRQ_EN GTM_gtm_cls1.MCS1_CH0_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH0_IRQ_FORCINT GTM_gtm_cls1.MCS1_CH0_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls1_MCS1_CH0_IRQ_MODE GTM_gtm_cls1.MCS1_CH0_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls1_MCS1_CH0_EIRQ_EN GTM_gtm_cls1.MCS1_CH0_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH1_R0 GTM_gtm_cls1.MCS1_CH1_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH1_R1 GTM_gtm_cls1.MCS1_CH1_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH1_R2 GTM_gtm_cls1.MCS1_CH1_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH1_R3 GTM_gtm_cls1.MCS1_CH1_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH1_R4 GTM_gtm_cls1.MCS1_CH1_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH1_R5 GTM_gtm_cls1.MCS1_CH1_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH1_R6 GTM_gtm_cls1.MCS1_CH1_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH1_R7 GTM_gtm_cls1.MCS1_CH1_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH1_CTRL GTM_gtm_cls1.MCS1_CH1_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls1_MCS1_CH1_ACB GTM_gtm_cls1.MCS1_CH1_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls1_MCS1_CH1_MHB GTM_gtm_cls1.MCS1_CH1_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls1_MCS1_CH1_PC GTM_gtm_cls1.MCS1_CH1_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls1_MCS1_CH1_IRQ_NOTIFY GTM_gtm_cls1.MCS1_CH1_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls1_MCS1_CH1_IRQ_EN GTM_gtm_cls1.MCS1_CH1_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH1_IRQ_FORCINT GTM_gtm_cls1.MCS1_CH1_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls1_MCS1_CH1_IRQ_MODE GTM_gtm_cls1.MCS1_CH1_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls1_MCS1_CH1_EIRQ_EN GTM_gtm_cls1.MCS1_CH1_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH2_R0 GTM_gtm_cls1.MCS1_CH2_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH2_R1 GTM_gtm_cls1.MCS1_CH2_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH2_R2 GTM_gtm_cls1.MCS1_CH2_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH2_R3 GTM_gtm_cls1.MCS1_CH2_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH2_R4 GTM_gtm_cls1.MCS1_CH2_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH2_R5 GTM_gtm_cls1.MCS1_CH2_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH2_R6 GTM_gtm_cls1.MCS1_CH2_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH2_R7 GTM_gtm_cls1.MCS1_CH2_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH2_CTRL GTM_gtm_cls1.MCS1_CH2_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls1_MCS1_CH2_ACB GTM_gtm_cls1.MCS1_CH2_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls1_MCS1_CH2_MHB GTM_gtm_cls1.MCS1_CH2_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls1_MCS1_CH2_PC GTM_gtm_cls1.MCS1_CH2_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls1_MCS1_CH2_IRQ_NOTIFY GTM_gtm_cls1.MCS1_CH2_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls1_MCS1_CH2_IRQ_EN GTM_gtm_cls1.MCS1_CH2_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH2_IRQ_FORCINT GTM_gtm_cls1.MCS1_CH2_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls1_MCS1_CH2_IRQ_MODE GTM_gtm_cls1.MCS1_CH2_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls1_MCS1_CH2_EIRQ_EN GTM_gtm_cls1.MCS1_CH2_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH3_R0 GTM_gtm_cls1.MCS1_CH3_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH3_R1 GTM_gtm_cls1.MCS1_CH3_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH3_R2 GTM_gtm_cls1.MCS1_CH3_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH3_R3 GTM_gtm_cls1.MCS1_CH3_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH3_R4 GTM_gtm_cls1.MCS1_CH3_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH3_R5 GTM_gtm_cls1.MCS1_CH3_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH3_R6 GTM_gtm_cls1.MCS1_CH3_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH3_R7 GTM_gtm_cls1.MCS1_CH3_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH3_CTRL GTM_gtm_cls1.MCS1_CH3_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls1_MCS1_CH3_ACB GTM_gtm_cls1.MCS1_CH3_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls1_MCS1_CH3_MHB GTM_gtm_cls1.MCS1_CH3_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls1_MCS1_CH3_PC GTM_gtm_cls1.MCS1_CH3_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls1_MCS1_CH3_IRQ_NOTIFY GTM_gtm_cls1.MCS1_CH3_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls1_MCS1_CH3_IRQ_EN GTM_gtm_cls1.MCS1_CH3_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH3_IRQ_FORCINT GTM_gtm_cls1.MCS1_CH3_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls1_MCS1_CH3_IRQ_MODE GTM_gtm_cls1.MCS1_CH3_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls1_MCS1_CH3_EIRQ_EN GTM_gtm_cls1.MCS1_CH3_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH4_R0 GTM_gtm_cls1.MCS1_CH4_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH4_R1 GTM_gtm_cls1.MCS1_CH4_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH4_R2 GTM_gtm_cls1.MCS1_CH4_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH4_R3 GTM_gtm_cls1.MCS1_CH4_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH4_R4 GTM_gtm_cls1.MCS1_CH4_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH4_R5 GTM_gtm_cls1.MCS1_CH4_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH4_R6 GTM_gtm_cls1.MCS1_CH4_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH4_R7 GTM_gtm_cls1.MCS1_CH4_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH4_CTRL GTM_gtm_cls1.MCS1_CH4_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls1_MCS1_CH4_ACB GTM_gtm_cls1.MCS1_CH4_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls1_MCS1_CH4_MHB GTM_gtm_cls1.MCS1_CH4_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls1_MCS1_CH4_PC GTM_gtm_cls1.MCS1_CH4_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls1_MCS1_CH4_IRQ_NOTIFY GTM_gtm_cls1.MCS1_CH4_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls1_MCS1_CH4_IRQ_EN GTM_gtm_cls1.MCS1_CH4_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH4_IRQ_FORCINT GTM_gtm_cls1.MCS1_CH4_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls1_MCS1_CH4_IRQ_MODE GTM_gtm_cls1.MCS1_CH4_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls1_MCS1_CH4_EIRQ_EN GTM_gtm_cls1.MCS1_CH4_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH5_R0 GTM_gtm_cls1.MCS1_CH5_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH5_R1 GTM_gtm_cls1.MCS1_CH5_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH5_R2 GTM_gtm_cls1.MCS1_CH5_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH5_R3 GTM_gtm_cls1.MCS1_CH5_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH5_R4 GTM_gtm_cls1.MCS1_CH5_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH5_R5 GTM_gtm_cls1.MCS1_CH5_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH5_R6 GTM_gtm_cls1.MCS1_CH5_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH5_R7 GTM_gtm_cls1.MCS1_CH5_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH5_CTRL GTM_gtm_cls1.MCS1_CH5_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls1_MCS1_CH5_ACB GTM_gtm_cls1.MCS1_CH5_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls1_MCS1_CH5_MHB GTM_gtm_cls1.MCS1_CH5_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls1_MCS1_CH5_PC GTM_gtm_cls1.MCS1_CH5_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls1_MCS1_CH5_IRQ_NOTIFY GTM_gtm_cls1.MCS1_CH5_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls1_MCS1_CH5_IRQ_EN GTM_gtm_cls1.MCS1_CH5_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH5_IRQ_FORCINT GTM_gtm_cls1.MCS1_CH5_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls1_MCS1_CH5_IRQ_MODE GTM_gtm_cls1.MCS1_CH5_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls1_MCS1_CH5_EIRQ_EN GTM_gtm_cls1.MCS1_CH5_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH6_R0 GTM_gtm_cls1.MCS1_CH6_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH6_R1 GTM_gtm_cls1.MCS1_CH6_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH6_R2 GTM_gtm_cls1.MCS1_CH6_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH6_R3 GTM_gtm_cls1.MCS1_CH6_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH6_R4 GTM_gtm_cls1.MCS1_CH6_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH6_R5 GTM_gtm_cls1.MCS1_CH6_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH6_R6 GTM_gtm_cls1.MCS1_CH6_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH6_R7 GTM_gtm_cls1.MCS1_CH6_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH6_CTRL GTM_gtm_cls1.MCS1_CH6_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls1_MCS1_CH6_ACB GTM_gtm_cls1.MCS1_CH6_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls1_MCS1_CH6_MHB GTM_gtm_cls1.MCS1_CH6_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls1_MCS1_CH6_PC GTM_gtm_cls1.MCS1_CH6_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls1_MCS1_CH6_IRQ_NOTIFY GTM_gtm_cls1.MCS1_CH6_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls1_MCS1_CH6_IRQ_EN GTM_gtm_cls1.MCS1_CH6_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH6_IRQ_FORCINT GTM_gtm_cls1.MCS1_CH6_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls1_MCS1_CH6_IRQ_MODE GTM_gtm_cls1.MCS1_CH6_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls1_MCS1_CH6_EIRQ_EN GTM_gtm_cls1.MCS1_CH6_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH7_R0 GTM_gtm_cls1.MCS1_CH7_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH7_R1 GTM_gtm_cls1.MCS1_CH7_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH7_R2 GTM_gtm_cls1.MCS1_CH7_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH7_R3 GTM_gtm_cls1.MCS1_CH7_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH7_R4 GTM_gtm_cls1.MCS1_CH7_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH7_R5 GTM_gtm_cls1.MCS1_CH7_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH7_R6 GTM_gtm_cls1.MCS1_CH7_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH7_R7 GTM_gtm_cls1.MCS1_CH7_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls1_MCS1_CH7_CTRL GTM_gtm_cls1.MCS1_CH7_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls1_MCS1_CH7_ACB GTM_gtm_cls1.MCS1_CH7_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls1_MCS1_CH7_MHB GTM_gtm_cls1.MCS1_CH7_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls1_MCS1_CH7_PC GTM_gtm_cls1.MCS1_CH7_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls1_MCS1_CH7_IRQ_NOTIFY GTM_gtm_cls1.MCS1_CH7_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls1_MCS1_CH7_IRQ_EN GTM_gtm_cls1.MCS1_CH7_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls1_MCS1_CH7_IRQ_FORCINT GTM_gtm_cls1.MCS1_CH7_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls1_MCS1_CH7_IRQ_MODE GTM_gtm_cls1.MCS1_CH7_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls1_MCS1_CH7_EIRQ_EN GTM_gtm_cls1.MCS1_CH7_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls1_MCS1_CTRG GTM_gtm_cls1.MCS1_CTRG.R    /* MCS[i] clear trigger control register */
#define GTM_gtm_cls1_MCS1_STRG GTM_gtm_cls1.MCS1_STRG.R    /* MCS[i] set trigger control register */
#define GTM_gtm_cls1_MCS1_CTRL_STAT GTM_gtm_cls1.MCS1_CTRL_STAT.R /* MCS[i] control and status register */
#define GTM_gtm_cls1_MCS1_RESET GTM_gtm_cls1.MCS1_RESET.R  /* MCS[i] reset register */
#define GTM_gtm_cls1_MCS1_CAT GTM_gtm_cls1.MCS1_CAT.R      /* MCS[i] cancel ARU transfer instruction */
#define GTM_gtm_cls1_MCS1_CWT GTM_gtm_cls1.MCS1_CWT.R      /* MCS[i] cancel waiting instruction */
#define GTM_gtm_cls1_MCS1_ERR GTM_gtm_cls1.MCS1_ERR.R      /* MCS[i] error register */
#define GTM_gtm_cls1_MCS1_REG_PROT GTM_gtm_cls1.MCS1_REG_PROT.R /* MCS[i] write protection register */
#define GTM_gtm_cls1_MCS1_SINT_IRQ_NOTIFY GTM_gtm_cls1.MCS1_SINT_IRQ_NOTIFY.R /* MCS[i] shared interrupt notification register */
#define GTM_gtm_cls1_MCS1_SINT_IRQ_EN GTM_gtm_cls1.MCS1_SINT_IRQ_EN.R /* MCS[i] shared interrupt enable register */
#define GTM_gtm_cls1_MCS1_SINT_IRQ_FORCINT GTM_gtm_cls1.MCS1_SINT_IRQ_FORCINT.R /* MCS[i] force shared interrupt register */
#define GTM_gtm_cls1_MCS1_SINT_IRQ_MODE GTM_gtm_cls1.MCS1_SINT_IRQ_MODE.R /* MCS[i] shared interrupt mode configuration register */
#define GTM_gtm_cls1_MCS1_HBP0_CTRL GTM_gtm_cls1.MCS1_HBP0_CTRL.R /* MCS[i] hardware break point h control register */
#define GTM_gtm_cls1_MCS1_HBP0_PATTERN GTM_gtm_cls1.MCS1_HBP0_PATTERN.R /* MCS[i] hardware break point pattern register */
#define GTM_gtm_cls1_MCS1_HBP0_STATUS GTM_gtm_cls1.MCS1_HBP0_STATUS.R /* MCS[i] hardware break point status register */
#define GTM_gtm_cls1_MCS1_HBP0_IRQ_NOTIFY GTM_gtm_cls1.MCS1_HBP0_IRQ_NOTIFY.R /* MCS[i] hardware break point interrupt notification register */
#define GTM_gtm_cls1_MCS1_HBP0_IRQ_EN GTM_gtm_cls1.MCS1_HBP0_IRQ_EN.R /* MCS[i] hardware break point interrupt enable register */
#define GTM_gtm_cls1_MCS1_HBP0_IRQ_FORCINT GTM_gtm_cls1.MCS1_HBP0_IRQ_FORCINT.R /* MCS[i] force hardware break point interrupt register */
#define GTM_gtm_cls1_MCS1_HBP0_IRQ_MODE GTM_gtm_cls1.MCS1_HBP0_IRQ_MODE.R /* MCS[i] break point h interrupt mode configuration register */
#define GTM_gtm_cls1_MCS1_HBP1_CTRL GTM_gtm_cls1.MCS1_HBP1_CTRL.R /* MCS[i] hardware break point h control register */
#define GTM_gtm_cls1_MCS1_HBP1_PATTERN GTM_gtm_cls1.MCS1_HBP1_PATTERN.R /* MCS[i] hardware break point pattern register */
#define GTM_gtm_cls1_MCS1_HBP1_STATUS GTM_gtm_cls1.MCS1_HBP1_STATUS.R /* MCS[i] hardware break point status register */
#define GTM_gtm_cls1_MCS1_HBP1_IRQ_NOTIFY GTM_gtm_cls1.MCS1_HBP1_IRQ_NOTIFY.R /* MCS[i] hardware break point interrupt notification register */
#define GTM_gtm_cls1_MCS1_HBP1_IRQ_EN GTM_gtm_cls1.MCS1_HBP1_IRQ_EN.R /* MCS[i] hardware break point interrupt enable register */
#define GTM_gtm_cls1_MCS1_HBP1_IRQ_FORCINT GTM_gtm_cls1.MCS1_HBP1_IRQ_FORCINT.R /* MCS[i] force hardware break point interrupt register */
#define GTM_gtm_cls1_MCS1_HBP1_IRQ_MODE GTM_gtm_cls1.MCS1_HBP1_IRQ_MODE.R /* MCS[i] break point h interrupt mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH0_CTRL GTM_gtm_cls1.TIO1_G0_CH0_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH0_IRQ_NOTIFY GTM_gtm_cls1.TIO1_G0_CH0_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls1_TIO1_G0_CH0_IRQ_EN GTM_gtm_cls1.TIO1_G0_CH0_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls1_TIO1_G0_CH0_IRQ_FORCINT GTM_gtm_cls1.TIO1_G0_CH0_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls1_TIO1_G0_CH0_IRQ_MODE GTM_gtm_cls1.TIO1_G0_CH0_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH0_CTRL2 GTM_gtm_cls1.TIO1_G0_CH0_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH0_SINST GTM_gtm_cls1.TIO1_G0_CH0_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH0_SCMD GTM_gtm_cls1.TIO1_G0_CH0_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH0_SOP GTM_gtm_cls1.TIO1_G0_CH0_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls1_TIO1_G0_CH0_OINST GTM_gtm_cls1.TIO1_G0_CH0_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH0_OCMD GTM_gtm_cls1.TIO1_G0_CH0_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH0_OOP GTM_gtm_cls1.TIO1_G0_CH0_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls1_TIO1_G0_CH0_SHIFTCNT GTM_gtm_cls1.TIO1_G0_CH0_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls1_TIO1_G0_CH1_CTRL GTM_gtm_cls1.TIO1_G0_CH1_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH1_IRQ_NOTIFY GTM_gtm_cls1.TIO1_G0_CH1_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls1_TIO1_G0_CH1_IRQ_EN GTM_gtm_cls1.TIO1_G0_CH1_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls1_TIO1_G0_CH1_IRQ_FORCINT GTM_gtm_cls1.TIO1_G0_CH1_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls1_TIO1_G0_CH1_IRQ_MODE GTM_gtm_cls1.TIO1_G0_CH1_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH1_CTRL2 GTM_gtm_cls1.TIO1_G0_CH1_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH1_SINST GTM_gtm_cls1.TIO1_G0_CH1_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH1_SCMD GTM_gtm_cls1.TIO1_G0_CH1_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH1_SOP GTM_gtm_cls1.TIO1_G0_CH1_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls1_TIO1_G0_CH1_OINST GTM_gtm_cls1.TIO1_G0_CH1_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH1_OCMD GTM_gtm_cls1.TIO1_G0_CH1_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH1_OOP GTM_gtm_cls1.TIO1_G0_CH1_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls1_TIO1_G0_CH1_SHIFTCNT GTM_gtm_cls1.TIO1_G0_CH1_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls1_TIO1_G0_CH2_CTRL GTM_gtm_cls1.TIO1_G0_CH2_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH2_IRQ_NOTIFY GTM_gtm_cls1.TIO1_G0_CH2_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls1_TIO1_G0_CH2_IRQ_EN GTM_gtm_cls1.TIO1_G0_CH2_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls1_TIO1_G0_CH2_IRQ_FORCINT GTM_gtm_cls1.TIO1_G0_CH2_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls1_TIO1_G0_CH2_IRQ_MODE GTM_gtm_cls1.TIO1_G0_CH2_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH2_CTRL2 GTM_gtm_cls1.TIO1_G0_CH2_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH2_SINST GTM_gtm_cls1.TIO1_G0_CH2_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH2_SCMD GTM_gtm_cls1.TIO1_G0_CH2_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH2_SOP GTM_gtm_cls1.TIO1_G0_CH2_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls1_TIO1_G0_CH2_OINST GTM_gtm_cls1.TIO1_G0_CH2_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH2_OCMD GTM_gtm_cls1.TIO1_G0_CH2_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH2_OOP GTM_gtm_cls1.TIO1_G0_CH2_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls1_TIO1_G0_CH2_SHIFTCNT GTM_gtm_cls1.TIO1_G0_CH2_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls1_TIO1_G0_CH3_CTRL GTM_gtm_cls1.TIO1_G0_CH3_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH3_IRQ_NOTIFY GTM_gtm_cls1.TIO1_G0_CH3_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls1_TIO1_G0_CH3_IRQ_EN GTM_gtm_cls1.TIO1_G0_CH3_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls1_TIO1_G0_CH3_IRQ_FORCINT GTM_gtm_cls1.TIO1_G0_CH3_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls1_TIO1_G0_CH3_IRQ_MODE GTM_gtm_cls1.TIO1_G0_CH3_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH3_CTRL2 GTM_gtm_cls1.TIO1_G0_CH3_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH3_SINST GTM_gtm_cls1.TIO1_G0_CH3_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH3_SCMD GTM_gtm_cls1.TIO1_G0_CH3_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH3_SOP GTM_gtm_cls1.TIO1_G0_CH3_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls1_TIO1_G0_CH3_OINST GTM_gtm_cls1.TIO1_G0_CH3_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH3_OCMD GTM_gtm_cls1.TIO1_G0_CH3_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH3_OOP GTM_gtm_cls1.TIO1_G0_CH3_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls1_TIO1_G0_CH3_SHIFTCNT GTM_gtm_cls1.TIO1_G0_CH3_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls1_TIO1_G0_CH4_CTRL GTM_gtm_cls1.TIO1_G0_CH4_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH4_IRQ_NOTIFY GTM_gtm_cls1.TIO1_G0_CH4_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls1_TIO1_G0_CH4_IRQ_EN GTM_gtm_cls1.TIO1_G0_CH4_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls1_TIO1_G0_CH4_IRQ_FORCINT GTM_gtm_cls1.TIO1_G0_CH4_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls1_TIO1_G0_CH4_IRQ_MODE GTM_gtm_cls1.TIO1_G0_CH4_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH4_CTRL2 GTM_gtm_cls1.TIO1_G0_CH4_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH4_SINST GTM_gtm_cls1.TIO1_G0_CH4_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH4_SCMD GTM_gtm_cls1.TIO1_G0_CH4_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH4_SOP GTM_gtm_cls1.TIO1_G0_CH4_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls1_TIO1_G0_CH4_OINST GTM_gtm_cls1.TIO1_G0_CH4_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH4_OCMD GTM_gtm_cls1.TIO1_G0_CH4_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH4_OOP GTM_gtm_cls1.TIO1_G0_CH4_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls1_TIO1_G0_CH4_SHIFTCNT GTM_gtm_cls1.TIO1_G0_CH4_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls1_TIO1_G0_CH5_CTRL GTM_gtm_cls1.TIO1_G0_CH5_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH5_IRQ_NOTIFY GTM_gtm_cls1.TIO1_G0_CH5_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls1_TIO1_G0_CH5_IRQ_EN GTM_gtm_cls1.TIO1_G0_CH5_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls1_TIO1_G0_CH5_IRQ_FORCINT GTM_gtm_cls1.TIO1_G0_CH5_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls1_TIO1_G0_CH5_IRQ_MODE GTM_gtm_cls1.TIO1_G0_CH5_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH5_CTRL2 GTM_gtm_cls1.TIO1_G0_CH5_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH5_SINST GTM_gtm_cls1.TIO1_G0_CH5_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH5_SCMD GTM_gtm_cls1.TIO1_G0_CH5_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH5_SOP GTM_gtm_cls1.TIO1_G0_CH5_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls1_TIO1_G0_CH5_OINST GTM_gtm_cls1.TIO1_G0_CH5_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH5_OCMD GTM_gtm_cls1.TIO1_G0_CH5_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH5_OOP GTM_gtm_cls1.TIO1_G0_CH5_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls1_TIO1_G0_CH5_SHIFTCNT GTM_gtm_cls1.TIO1_G0_CH5_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls1_TIO1_G0_CH6_CTRL GTM_gtm_cls1.TIO1_G0_CH6_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH6_IRQ_NOTIFY GTM_gtm_cls1.TIO1_G0_CH6_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls1_TIO1_G0_CH6_IRQ_EN GTM_gtm_cls1.TIO1_G0_CH6_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls1_TIO1_G0_CH6_IRQ_FORCINT GTM_gtm_cls1.TIO1_G0_CH6_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls1_TIO1_G0_CH6_IRQ_MODE GTM_gtm_cls1.TIO1_G0_CH6_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH6_CTRL2 GTM_gtm_cls1.TIO1_G0_CH6_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH6_SINST GTM_gtm_cls1.TIO1_G0_CH6_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH6_SCMD GTM_gtm_cls1.TIO1_G0_CH6_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH6_SOP GTM_gtm_cls1.TIO1_G0_CH6_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls1_TIO1_G0_CH6_OINST GTM_gtm_cls1.TIO1_G0_CH6_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH6_OCMD GTM_gtm_cls1.TIO1_G0_CH6_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH6_OOP GTM_gtm_cls1.TIO1_G0_CH6_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls1_TIO1_G0_CH6_SHIFTCNT GTM_gtm_cls1.TIO1_G0_CH6_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls1_TIO1_G0_CH7_CTRL GTM_gtm_cls1.TIO1_G0_CH7_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH7_IRQ_NOTIFY GTM_gtm_cls1.TIO1_G0_CH7_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls1_TIO1_G0_CH7_IRQ_EN GTM_gtm_cls1.TIO1_G0_CH7_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls1_TIO1_G0_CH7_IRQ_FORCINT GTM_gtm_cls1.TIO1_G0_CH7_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls1_TIO1_G0_CH7_IRQ_MODE GTM_gtm_cls1.TIO1_G0_CH7_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls1_TIO1_G0_CH7_CTRL2 GTM_gtm_cls1.TIO1_G0_CH7_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls1_TIO1_G0_CH7_SINST GTM_gtm_cls1.TIO1_G0_CH7_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH7_SCMD GTM_gtm_cls1.TIO1_G0_CH7_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls1_TIO1_G0_CH7_SOP GTM_gtm_cls1.TIO1_G0_CH7_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls1_TIO1_G0_CH7_OINST GTM_gtm_cls1.TIO1_G0_CH7_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH7_OCMD GTM_gtm_cls1.TIO1_G0_CH7_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls1_TIO1_G0_CH7_OOP GTM_gtm_cls1.TIO1_G0_CH7_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls1_TIO1_G0_CH7_SHIFTCNT GTM_gtm_cls1.TIO1_G0_CH7_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls1_TIO1_G0_ISEL0_CTRL1 GTM_gtm_cls1.TIO1_G0_ISEL0_CTRL1.R /* TIO[i] input selection register 1 */
#define GTM_gtm_cls1_TIO1_G0_ISEL0_CTRL2 GTM_gtm_cls1.TIO1_G0_ISEL0_CTRL2.R /* TIO[i] input selection register 2 */
#define GTM_gtm_cls1_TIO1_G0_ISEL1_CTRL1 GTM_gtm_cls1.TIO1_G0_ISEL1_CTRL1.R /* TIO[i] input selection register 1 */
#define GTM_gtm_cls1_TIO1_G0_ISEL1_CTRL2 GTM_gtm_cls1.TIO1_G0_ISEL1_CTRL2.R /* TIO[i] input selection register 2 */
#define GTM_gtm_cls1_TIO1_G0_OP_USAGE GTM_gtm_cls1.TIO1_G0_OP_USAGE.R /* TIO[i] operand usage selection register */
#define GTM_gtm_cls1_TIO1_S  GTM_gtm_cls1.TIO1_S.R         /* TIO[i] signal sampling register */
#define GTM_gtm_cls1_TIO1_O  GTM_gtm_cls1.TIO1_O.R         /* TIO[i] output register */
#define GTM_gtm_cls1_TIO1_ENDIS GTM_gtm_cls1.TIO1_ENDIS.R  /* TIO[i] enable/disable register */
#define GTM_gtm_cls1_TIO1_INVERT GTM_gtm_cls1.TIO1_INVERT.R /* TIO[i] signal invert register */
#define GTM_gtm_cls1_TIO1_INPUT_MODE GTM_gtm_cls1.TIO1_INPUT_MODE.R /* TIO[i] input mode register */
#define GTM_gtm_cls1_TIO1_CYCLIC_MODE GTM_gtm_cls1.TIO1_CYCLIC_MODE.R /* TIO[i] cyclic mode register */
#define GTM_gtm_cls1_TIO1_TRIG_OUT_GATE_EN GTM_gtm_cls1.TIO1_TRIG_OUT_GATE_EN.R /* TIO[i] enable Trigger Output, output gating register */
#define GTM_gtm_cls1_TIO1_PLTRIG_OUT_GATE_EN GTM_gtm_cls1.TIO1_PLTRIG_OUT_GATE_EN.R /* TIO[i] enable PL_TRIG_OUT output gating register */
#define GTM_gtm_cls1_TIO1_CS GTM_gtm_cls1.TIO1_CS.R        /* TIO[i] clear signal sampling register */
#define GTM_gtm_cls1_TIO1_CO GTM_gtm_cls1.TIO1_CO.R        /* TIO[i] clear output register */
#define GTM_gtm_cls1_TIO1_CENDIS GTM_gtm_cls1.TIO1_CENDIS.R /* TIO[i] disable register */
#define GTM_gtm_cls1_TIO1_CINVERT GTM_gtm_cls1.TIO1_CINVERT.R /* TIO[i] clear signal invert register */
#define GTM_gtm_cls1_TIO1_CINPUT_MODE GTM_gtm_cls1.TIO1_CINPUT_MODE.R /* TIO[i] disable input mode register */
#define GTM_gtm_cls1_TIO1_CCYCLIC_MODE GTM_gtm_cls1.TIO1_CCYCLIC_MODE.R /* TIO[i] disable cyclic mode register */
#define GTM_gtm_cls1_TIO1_CTRIG_OUT_GATE_EN GTM_gtm_cls1.TIO1_CTRIG_OUT_GATE_EN.R /* TIO[i] clear Trigger Output, output gating register */
#define GTM_gtm_cls1_TIO1_CPLTRIG_OUT_GATE_EN GTM_gtm_cls1.TIO1_CPLTRIG_OUT_GATE_EN.R /* TIO[i] clear PL_TRIG_OUT output gating register */
#define GTM_gtm_cls1_TIO1_SS GTM_gtm_cls1.TIO1_SS.R        /* TIO[i] set signal sampling register */
#define GTM_gtm_cls1_TIO1_SO GTM_gtm_cls1.TIO1_SO.R        /* TIO[i] set output register */
#define GTM_gtm_cls1_TIO1_SENDIS GTM_gtm_cls1.TIO1_SENDIS.R /* TIO[i] enable register */
#define GTM_gtm_cls1_TIO1_SINVERT GTM_gtm_cls1.TIO1_SINVERT.R /* TIO[i] set signal invert register */
#define GTM_gtm_cls1_TIO1_SINPUT_MODE GTM_gtm_cls1.TIO1_SINPUT_MODE.R /* TIO[i] enable input mode register */
#define GTM_gtm_cls1_TIO1_SCYCLIC_MODE GTM_gtm_cls1.TIO1_SCYCLIC_MODE.R /* TIO[i] enable cyclic mode register */
#define GTM_gtm_cls1_TIO1_STRIG_OUT_GATE_EN GTM_gtm_cls1.TIO1_STRIG_OUT_GATE_EN.R /* TIO[i] set Trigger Output, output gating register */
#define GTM_gtm_cls1_TIO1_SPLTRIG_OUT_GATE_EN GTM_gtm_cls1.TIO1_SPLTRIG_OUT_GATE_EN.R /* TIO[i] set PL_TRIG_OUT output gating register */
#define GTM_gtm_cls1_TIO1_IS GTM_gtm_cls1.TIO1_IS.R        /* TIO[i] invert signal sampling register */
#define GTM_gtm_cls1_TIO1_IO GTM_gtm_cls1.TIO1_IO.R        /* TIO[i] invert output register */
#define GTM_gtm_cls1_TIO1_IENDIS GTM_gtm_cls1.TIO1_IENDIS.R /* TIO[i] toggle enable/disable register */
#define GTM_gtm_cls1_TIO1_IINVERT GTM_gtm_cls1.TIO1_IINVERT.R /* TIO[i] toggle signal invert register */
#define GTM_gtm_cls1_TIO1_IINPUT_MODE GTM_gtm_cls1.TIO1_IINPUT_MODE.R /* TIO[i] enable input mode register */
#define GTM_gtm_cls1_TIO1_ICYCLIC_MODE GTM_gtm_cls1.TIO1_ICYCLIC_MODE.R /* TIO[i] enable cyclic mode register */
#define GTM_gtm_cls1_TIO1_FUPD GTM_gtm_cls1.TIO1_FUPD.R    /* TIO[i] force update register */
#define GTM_gtm_cls1_TIO1_HW_CONF GTM_gtm_cls1.TIO1_HW_CONF.R /* TIO[i] configuration register */
#define GTM_gtm_cls1_TIO1_RSEL_CTRL1 GTM_gtm_cls1.TIO1_RSEL_CTRL1.R /* TIO[i] resource selection control register 1 */
#define GTM_gtm_cls1_TIO1_RSEL_CTRL2 GTM_gtm_cls1.TIO1_RSEL_CTRL2.R /* TIO[i] resource selection control register 2 */
#define GTM_gtm_cls1_TIO1_PL_SWRST GTM_gtm_cls1.TIO1_PL_SWRST.R /* TIO[i] software reset for TIO Plus functionality */
#define GTM_gtm_cls1_CCM1_ARP0_CTRL GTM_gtm_cls1.CCM1_ARP0_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP0_PROT GTM_gtm_cls1.CCM1_ARP0_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP1_CTRL GTM_gtm_cls1.CCM1_ARP1_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP1_PROT GTM_gtm_cls1.CCM1_ARP1_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP2_CTRL GTM_gtm_cls1.CCM1_ARP2_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP2_PROT GTM_gtm_cls1.CCM1_ARP2_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP3_CTRL GTM_gtm_cls1.CCM1_ARP3_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP3_PROT GTM_gtm_cls1.CCM1_ARP3_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP4_CTRL GTM_gtm_cls1.CCM1_ARP4_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP4_PROT GTM_gtm_cls1.CCM1_ARP4_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP5_CTRL GTM_gtm_cls1.CCM1_ARP5_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP5_PROT GTM_gtm_cls1.CCM1_ARP5_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP6_CTRL GTM_gtm_cls1.CCM1_ARP6_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP6_PROT GTM_gtm_cls1.CCM1_ARP6_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP7_CTRL GTM_gtm_cls1.CCM1_ARP7_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP7_PROT GTM_gtm_cls1.CCM1_ARP7_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP8_CTRL GTM_gtm_cls1.CCM1_ARP8_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP8_PROT GTM_gtm_cls1.CCM1_ARP8_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_ARP9_CTRL GTM_gtm_cls1.CCM1_ARP9_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls1_CCM1_ARP9_PROT GTM_gtm_cls1.CCM1_ARP9_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls1_CCM1_TIO_G0_OUT GTM_gtm_cls1.CCM1_TIO_G0_OUT.R /* CCM[i] TIO Group 0,1 Output Register */
#define GTM_gtm_cls1_CCM1_HW_CONF2 GTM_gtm_cls1.CCM1_HW_CONF2.R /* CCM[i] 2. Hardware Configuration Register */
#define GTM_gtm_cls1_CCM1_AEIM_STA GTM_gtm_cls1.CCM1_AEIM_STA.R /* CCM[i] MCS Bus Master Status Register */
#define GTM_gtm_cls1_CCM1_HW_CONF GTM_gtm_cls1.CCM1_HW_CONF.R /* CCM[i] Hardware Configuration Register */
#define GTM_gtm_cls1_CCM1_TIM_AUX_IN_SRC GTM_gtm_cls1.CCM1_TIM_AUX_IN_SRC.R /* CCM[i] TIM AUX Input Source Register */
#define GTM_gtm_cls1_CCM1_EXT_CAP_EN GTM_gtm_cls1.CCM1_EXT_CAP_EN.R /* CCM[i] External Capture Enable Register */
#define GTM_gtm_cls1_CCM1_TOM_OUT GTM_gtm_cls1.CCM1_TOM_OUT.R /* CCM[i] TOM Output Register */
#define GTM_gtm_cls1_CCM1_ATOM_OUT GTM_gtm_cls1.CCM1_ATOM_OUT.R /* CCM[i] ATOM Output Register */
#define GTM_gtm_cls1_CCM1_CMU_CLK_CFG GTM_gtm_cls1.CCM1_CMU_CLK_CFG.R /* CCM[i] CMU Clock Configuration Register */
#define GTM_gtm_cls1_CCM1_CMU_FXCLK_CFG GTM_gtm_cls1.CCM1_CMU_FXCLK_CFG.R /* CCM[i] CMU Fixed Clock Configuration Register */
#define GTM_gtm_cls1_CCM1_CFG GTM_gtm_cls1.CCM1_CFG.R      /* CCM[i] Configuration Register */
#define GTM_gtm_cls1_CCM1_PROT GTM_gtm_cls1.CCM1_PROT.R    /* CCM[i] Protection Register */
#define GTM_gtm_cls1_CDTM1_DTM4_CTRL GTM_gtm_cls1.CDTM1_DTM4_CTRL.R /* CDTM[i]_DTM[d] global configuration and control register */
#define GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL1 GTM_gtm_cls1.CDTM1_DTM4_CH_CTRL1.R /* CDTM[i]_DTM[d] channel control register 1 */
#define GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL2 GTM_gtm_cls1.CDTM1_DTM4_CH_CTRL2.R /* CDTM[i]_DTM[d] channel control register 2 */
#define GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL2_SR GTM_gtm_cls1.CDTM1_DTM4_CH_CTRL2_SR.R /* CDTM[i] DTM[j] channel control register 2 shadow */
#define GTM_gtm_cls1_CDTM1_DTM4_PS_CTRL GTM_gtm_cls1.CDTM1_DTM4_PS_CTRL.R /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
#define GTM_gtm_cls1_CDTM1_DTM4_CH0_DTV GTM_gtm_cls1.CDTM1_DTM4_CH_DTV[0].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls1_CDTM1_DTM4_CH1_DTV GTM_gtm_cls1.CDTM1_DTM4_CH_DTV[1].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls1_CDTM1_DTM4_CH2_DTV GTM_gtm_cls1.CDTM1_DTM4_CH_DTV[2].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls1_CDTM1_DTM4_CH3_DTV GTM_gtm_cls1.CDTM1_DTM4_CH_DTV[3].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls1_CDTM1_DTM4_CH_SR GTM_gtm_cls1.CDTM1_DTM4_CH_SR.R /* CDTM[i]_DTM[d] channel shadow register */
#define GTM_gtm_cls1_CDTM1_DTM4_CH_CTRL3 GTM_gtm_cls1.CDTM1_DTM4_CH_CTRL3.R /* CDTM[i]_DTM[d] channel control register 3 */
#define GTM_gtm_cls1_CDTM1_DTM4_CTRL2 GTM_gtm_cls1.CDTM1_DTM4_CTRL2.R /* CDTM[i]_DTM[d] global configuration and control register 2 */
#define GTM_gtm_cls1_CDTM1_DTM4_CH0_DTV_SR GTM_gtm_cls1.CDTM1_DTM4_CH0_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls1_CDTM1_DTM4_CH1_DTV_SR GTM_gtm_cls1.CDTM1_DTM4_CH1_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls1_CDTM1_DTM4_CH2_DTV_SR GTM_gtm_cls1.CDTM1_DTM4_CH2_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls1_CDTM1_DTM4_CH3_DTV_SR GTM_gtm_cls1.CDTM1_DTM4_CH3_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls1_CDTM1_DTM5_CTRL GTM_gtm_cls1.CDTM1_DTM5_CTRL.R /* CDTM[i]_DTM[d] global configuration and control register */
#define GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL1 GTM_gtm_cls1.CDTM1_DTM5_CH_CTRL1.R /* CDTM[i]_DTM[d] channel control register 1 */
#define GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL2 GTM_gtm_cls1.CDTM1_DTM5_CH_CTRL2.R /* CDTM[i]_DTM[d] channel control register 2 */
#define GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL2_SR GTM_gtm_cls1.CDTM1_DTM5_CH_CTRL2_SR.R /* CDTM[i] DTM[j] channel control register 2 shadow */
#define GTM_gtm_cls1_CDTM1_DTM5_PS_CTRL GTM_gtm_cls1.CDTM1_DTM5_PS_CTRL.R /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
#define GTM_gtm_cls1_CDTM1_DTM5_CH0_DTV GTM_gtm_cls1.CDTM1_DTM5_CH_DTV[0].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls1_CDTM1_DTM5_CH1_DTV GTM_gtm_cls1.CDTM1_DTM5_CH_DTV[1].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls1_CDTM1_DTM5_CH2_DTV GTM_gtm_cls1.CDTM1_DTM5_CH_DTV[2].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls1_CDTM1_DTM5_CH3_DTV GTM_gtm_cls1.CDTM1_DTM5_CH_DTV[3].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls1_CDTM1_DTM5_CH_SR GTM_gtm_cls1.CDTM1_DTM5_CH_SR.R /* CDTM[i]_DTM[d] channel shadow register */
#define GTM_gtm_cls1_CDTM1_DTM5_CH_CTRL3 GTM_gtm_cls1.CDTM1_DTM5_CH_CTRL3.R /* CDTM[i]_DTM[d] channel control register 3 */
#define GTM_gtm_cls1_CDTM1_DTM5_CTRL2 GTM_gtm_cls1.CDTM1_DTM5_CTRL2.R /* CDTM[i]_DTM[d] global configuration and control register 2 */
#define GTM_gtm_cls1_CDTM1_DTM5_CH0_DTV_SR GTM_gtm_cls1.CDTM1_DTM5_CH0_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls1_CDTM1_DTM5_CH1_DTV_SR GTM_gtm_cls1.CDTM1_DTM5_CH1_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls1_CDTM1_DTM5_CH2_DTV_SR GTM_gtm_cls1.CDTM1_DTM5_CH2_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls1_CDTM1_DTM5_CH3_DTV_SR GTM_gtm_cls1.CDTM1_DTM5_CH3_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls1_SPE1_CTRL_STAT GTM_gtm_cls1.SPE1_CTRL_STAT.R /* SPE[i] Control Status Register */
#define GTM_gtm_cls1_SPE1_PAT GTM_gtm_cls1.SPE1_PAT.R      /* SPE[i] Input Pattern Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_PAT0 GTM_gtm_cls1.SPE1_OUT_PAT0.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_PAT1 GTM_gtm_cls1.SPE1_OUT_PAT1.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_PAT2 GTM_gtm_cls1.SPE1_OUT_PAT2.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_PAT3 GTM_gtm_cls1.SPE1_OUT_PAT3.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_PAT4 GTM_gtm_cls1.SPE1_OUT_PAT4.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_PAT5 GTM_gtm_cls1.SPE1_OUT_PAT5.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_PAT6 GTM_gtm_cls1.SPE1_OUT_PAT6.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_PAT7 GTM_gtm_cls1.SPE1_OUT_PAT7.R /* SPE[i] Output Definition Register */
#define GTM_gtm_cls1_SPE1_OUT_CTRL GTM_gtm_cls1.SPE1_OUT_CTRL.R /* SPE[i] Output Control Register */
#define GTM_gtm_cls1_SPE1_IRQ_NOTIFY GTM_gtm_cls1.SPE1_IRQ_NOTIFY.R /* SPE[i] Interrupt Notification Register */
#define GTM_gtm_cls1_SPE1_IRQ_EN GTM_gtm_cls1.SPE1_IRQ_EN.R /* SPE[i] Interrupt Enable Register */
#define GTM_gtm_cls1_SPE1_IRQ_FORCINT GTM_gtm_cls1.SPE1_IRQ_FORCINT.R /* SPE[i] Interrupt Generation By Software */
#define GTM_gtm_cls1_SPE1_IRQ_MODE GTM_gtm_cls1.SPE1_IRQ_MODE.R /* SPE[i] Interrupt Mode Configuration Register */
#define GTM_gtm_cls1_SPE1_EIRQ_EN GTM_gtm_cls1.SPE1_EIRQ_EN.R /* SPE[i] Error Interrupt Enable Register */
#define GTM_gtm_cls1_SPE1_REV_CNT GTM_gtm_cls1.SPE1_REV_CNT.R /* SPE[i] Input Revolution Counter */
#define GTM_gtm_cls1_SPE1_REV_CMP GTM_gtm_cls1.SPE1_REV_CMP.R /* SPE[i] Revolution Counter Compare Value */
#define GTM_gtm_cls1_SPE1_CTRL_STAT2 GTM_gtm_cls1.SPE1_CTRL_STAT2.R /* SPE[i] Control Status Register 2 */
#define GTM_gtm_cls1_SPE1_CMD GTM_gtm_cls1.SPE1_CMD.R      /* SPE[i] Command Register */
#define GTM_gtm_cls1_AXIM1_FREE GTM_gtm_cls1.AXIM1_FREE.R  /* AXIM[i] slot allocation status. */
#define GTM_gtm_cls1_AXIM1_REQUEST GTM_gtm_cls1.AXIM1_REQUEST.R /* AXIM[i] slot request (allocation). */
#define GTM_gtm_cls1_AXIM1_RELEASE GTM_gtm_cls1.AXIM1_RELEASE.R /* AXIM[i] slot release (de-allocation). */
#define GTM_gtm_cls1_AXIM1_SLOT0_ADDR_LOW GTM_gtm_cls1.AXIM1_SLOT0_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls1_AXIM1_SLOT0_DATA_LOW GTM_gtm_cls1.AXIM1_SLOT0_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls1_AXIM1_SLOT0_CFG1 GTM_gtm_cls1.AXIM1_SLOT0_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls1_AXIM1_SLOT0_CFG2 GTM_gtm_cls1.AXIM1_SLOT0_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls1_AXIM1_SLOT0_STATUS GTM_gtm_cls1.AXIM1_SLOT0_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls1_AXIM1_SLOT1_ADDR_LOW GTM_gtm_cls1.AXIM1_SLOT1_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls1_AXIM1_SLOT1_DATA_LOW GTM_gtm_cls1.AXIM1_SLOT1_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls1_AXIM1_SLOT1_CFG1 GTM_gtm_cls1.AXIM1_SLOT1_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls1_AXIM1_SLOT1_CFG2 GTM_gtm_cls1.AXIM1_SLOT1_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls1_AXIM1_SLOT1_STATUS GTM_gtm_cls1.AXIM1_SLOT1_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls1_AXIM1_SLOT2_ADDR_LOW GTM_gtm_cls1.AXIM1_SLOT2_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls1_AXIM1_SLOT2_DATA_LOW GTM_gtm_cls1.AXIM1_SLOT2_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls1_AXIM1_SLOT2_CFG1 GTM_gtm_cls1.AXIM1_SLOT2_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls1_AXIM1_SLOT2_CFG2 GTM_gtm_cls1.AXIM1_SLOT2_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls1_AXIM1_SLOT2_STATUS GTM_gtm_cls1.AXIM1_SLOT2_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls1_AXIM1_SLOT3_ADDR_LOW GTM_gtm_cls1.AXIM1_SLOT3_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls1_AXIM1_SLOT3_DATA_LOW GTM_gtm_cls1.AXIM1_SLOT3_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls1_AXIM1_SLOT3_CFG1 GTM_gtm_cls1.AXIM1_SLOT3_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls1_AXIM1_SLOT3_CFG2 GTM_gtm_cls1.AXIM1_SLOT3_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls1_AXIM1_SLOT3_STATUS GTM_gtm_cls1.AXIM1_SLOT3_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls1_MCS1_MEM0 GTM_gtm_cls1.MCS1_MEM[0].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1 GTM_gtm_cls1.MCS1_MEM[1].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2 GTM_gtm_cls1.MCS1_MEM[2].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3 GTM_gtm_cls1.MCS1_MEM[3].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM4 GTM_gtm_cls1.MCS1_MEM[4].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM5 GTM_gtm_cls1.MCS1_MEM[5].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM6 GTM_gtm_cls1.MCS1_MEM[6].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM7 GTM_gtm_cls1.MCS1_MEM[7].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM8 GTM_gtm_cls1.MCS1_MEM[8].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM9 GTM_gtm_cls1.MCS1_MEM[9].R  /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM10 GTM_gtm_cls1.MCS1_MEM[10].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM11 GTM_gtm_cls1.MCS1_MEM[11].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM12 GTM_gtm_cls1.MCS1_MEM[12].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM13 GTM_gtm_cls1.MCS1_MEM[13].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM14 GTM_gtm_cls1.MCS1_MEM[14].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM15 GTM_gtm_cls1.MCS1_MEM[15].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM16 GTM_gtm_cls1.MCS1_MEM[16].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM17 GTM_gtm_cls1.MCS1_MEM[17].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM18 GTM_gtm_cls1.MCS1_MEM[18].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM19 GTM_gtm_cls1.MCS1_MEM[19].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM20 GTM_gtm_cls1.MCS1_MEM[20].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM21 GTM_gtm_cls1.MCS1_MEM[21].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM22 GTM_gtm_cls1.MCS1_MEM[22].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM23 GTM_gtm_cls1.MCS1_MEM[23].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM24 GTM_gtm_cls1.MCS1_MEM[24].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM25 GTM_gtm_cls1.MCS1_MEM[25].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM26 GTM_gtm_cls1.MCS1_MEM[26].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM27 GTM_gtm_cls1.MCS1_MEM[27].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM28 GTM_gtm_cls1.MCS1_MEM[28].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM29 GTM_gtm_cls1.MCS1_MEM[29].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM30 GTM_gtm_cls1.MCS1_MEM[30].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM31 GTM_gtm_cls1.MCS1_MEM[31].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM32 GTM_gtm_cls1.MCS1_MEM[32].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM33 GTM_gtm_cls1.MCS1_MEM[33].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM34 GTM_gtm_cls1.MCS1_MEM[34].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM35 GTM_gtm_cls1.MCS1_MEM[35].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM36 GTM_gtm_cls1.MCS1_MEM[36].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM37 GTM_gtm_cls1.MCS1_MEM[37].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM38 GTM_gtm_cls1.MCS1_MEM[38].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM39 GTM_gtm_cls1.MCS1_MEM[39].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM40 GTM_gtm_cls1.MCS1_MEM[40].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM41 GTM_gtm_cls1.MCS1_MEM[41].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM42 GTM_gtm_cls1.MCS1_MEM[42].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM43 GTM_gtm_cls1.MCS1_MEM[43].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM44 GTM_gtm_cls1.MCS1_MEM[44].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM45 GTM_gtm_cls1.MCS1_MEM[45].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM46 GTM_gtm_cls1.MCS1_MEM[46].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM47 GTM_gtm_cls1.MCS1_MEM[47].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM48 GTM_gtm_cls1.MCS1_MEM[48].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM49 GTM_gtm_cls1.MCS1_MEM[49].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM50 GTM_gtm_cls1.MCS1_MEM[50].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM51 GTM_gtm_cls1.MCS1_MEM[51].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM52 GTM_gtm_cls1.MCS1_MEM[52].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM53 GTM_gtm_cls1.MCS1_MEM[53].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM54 GTM_gtm_cls1.MCS1_MEM[54].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM55 GTM_gtm_cls1.MCS1_MEM[55].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM56 GTM_gtm_cls1.MCS1_MEM[56].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM57 GTM_gtm_cls1.MCS1_MEM[57].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM58 GTM_gtm_cls1.MCS1_MEM[58].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM59 GTM_gtm_cls1.MCS1_MEM[59].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM60 GTM_gtm_cls1.MCS1_MEM[60].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM61 GTM_gtm_cls1.MCS1_MEM[61].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM62 GTM_gtm_cls1.MCS1_MEM[62].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM63 GTM_gtm_cls1.MCS1_MEM[63].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM64 GTM_gtm_cls1.MCS1_MEM[64].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM65 GTM_gtm_cls1.MCS1_MEM[65].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM66 GTM_gtm_cls1.MCS1_MEM[66].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM67 GTM_gtm_cls1.MCS1_MEM[67].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM68 GTM_gtm_cls1.MCS1_MEM[68].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM69 GTM_gtm_cls1.MCS1_MEM[69].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM70 GTM_gtm_cls1.MCS1_MEM[70].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM71 GTM_gtm_cls1.MCS1_MEM[71].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM72 GTM_gtm_cls1.MCS1_MEM[72].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM73 GTM_gtm_cls1.MCS1_MEM[73].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM74 GTM_gtm_cls1.MCS1_MEM[74].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM75 GTM_gtm_cls1.MCS1_MEM[75].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM76 GTM_gtm_cls1.MCS1_MEM[76].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM77 GTM_gtm_cls1.MCS1_MEM[77].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM78 GTM_gtm_cls1.MCS1_MEM[78].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM79 GTM_gtm_cls1.MCS1_MEM[79].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM80 GTM_gtm_cls1.MCS1_MEM[80].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM81 GTM_gtm_cls1.MCS1_MEM[81].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM82 GTM_gtm_cls1.MCS1_MEM[82].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM83 GTM_gtm_cls1.MCS1_MEM[83].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM84 GTM_gtm_cls1.MCS1_MEM[84].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM85 GTM_gtm_cls1.MCS1_MEM[85].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM86 GTM_gtm_cls1.MCS1_MEM[86].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM87 GTM_gtm_cls1.MCS1_MEM[87].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM88 GTM_gtm_cls1.MCS1_MEM[88].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM89 GTM_gtm_cls1.MCS1_MEM[89].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM90 GTM_gtm_cls1.MCS1_MEM[90].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM91 GTM_gtm_cls1.MCS1_MEM[91].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM92 GTM_gtm_cls1.MCS1_MEM[92].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM93 GTM_gtm_cls1.MCS1_MEM[93].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM94 GTM_gtm_cls1.MCS1_MEM[94].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM95 GTM_gtm_cls1.MCS1_MEM[95].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM96 GTM_gtm_cls1.MCS1_MEM[96].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM97 GTM_gtm_cls1.MCS1_MEM[97].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM98 GTM_gtm_cls1.MCS1_MEM[98].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM99 GTM_gtm_cls1.MCS1_MEM[99].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM100 GTM_gtm_cls1.MCS1_MEM[100].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM101 GTM_gtm_cls1.MCS1_MEM[101].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM102 GTM_gtm_cls1.MCS1_MEM[102].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM103 GTM_gtm_cls1.MCS1_MEM[103].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM104 GTM_gtm_cls1.MCS1_MEM[104].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM105 GTM_gtm_cls1.MCS1_MEM[105].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM106 GTM_gtm_cls1.MCS1_MEM[106].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM107 GTM_gtm_cls1.MCS1_MEM[107].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM108 GTM_gtm_cls1.MCS1_MEM[108].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM109 GTM_gtm_cls1.MCS1_MEM[109].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM110 GTM_gtm_cls1.MCS1_MEM[110].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM111 GTM_gtm_cls1.MCS1_MEM[111].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM112 GTM_gtm_cls1.MCS1_MEM[112].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM113 GTM_gtm_cls1.MCS1_MEM[113].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM114 GTM_gtm_cls1.MCS1_MEM[114].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM115 GTM_gtm_cls1.MCS1_MEM[115].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM116 GTM_gtm_cls1.MCS1_MEM[116].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM117 GTM_gtm_cls1.MCS1_MEM[117].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM118 GTM_gtm_cls1.MCS1_MEM[118].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM119 GTM_gtm_cls1.MCS1_MEM[119].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM120 GTM_gtm_cls1.MCS1_MEM[120].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM121 GTM_gtm_cls1.MCS1_MEM[121].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM122 GTM_gtm_cls1.MCS1_MEM[122].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM123 GTM_gtm_cls1.MCS1_MEM[123].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM124 GTM_gtm_cls1.MCS1_MEM[124].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM125 GTM_gtm_cls1.MCS1_MEM[125].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM126 GTM_gtm_cls1.MCS1_MEM[126].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM127 GTM_gtm_cls1.MCS1_MEM[127].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM128 GTM_gtm_cls1.MCS1_MEM[128].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM129 GTM_gtm_cls1.MCS1_MEM[129].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM130 GTM_gtm_cls1.MCS1_MEM[130].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM131 GTM_gtm_cls1.MCS1_MEM[131].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM132 GTM_gtm_cls1.MCS1_MEM[132].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM133 GTM_gtm_cls1.MCS1_MEM[133].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM134 GTM_gtm_cls1.MCS1_MEM[134].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM135 GTM_gtm_cls1.MCS1_MEM[135].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM136 GTM_gtm_cls1.MCS1_MEM[136].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM137 GTM_gtm_cls1.MCS1_MEM[137].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM138 GTM_gtm_cls1.MCS1_MEM[138].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM139 GTM_gtm_cls1.MCS1_MEM[139].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM140 GTM_gtm_cls1.MCS1_MEM[140].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM141 GTM_gtm_cls1.MCS1_MEM[141].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM142 GTM_gtm_cls1.MCS1_MEM[142].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM143 GTM_gtm_cls1.MCS1_MEM[143].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM144 GTM_gtm_cls1.MCS1_MEM[144].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM145 GTM_gtm_cls1.MCS1_MEM[145].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM146 GTM_gtm_cls1.MCS1_MEM[146].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM147 GTM_gtm_cls1.MCS1_MEM[147].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM148 GTM_gtm_cls1.MCS1_MEM[148].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM149 GTM_gtm_cls1.MCS1_MEM[149].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM150 GTM_gtm_cls1.MCS1_MEM[150].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM151 GTM_gtm_cls1.MCS1_MEM[151].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM152 GTM_gtm_cls1.MCS1_MEM[152].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM153 GTM_gtm_cls1.MCS1_MEM[153].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM154 GTM_gtm_cls1.MCS1_MEM[154].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM155 GTM_gtm_cls1.MCS1_MEM[155].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM156 GTM_gtm_cls1.MCS1_MEM[156].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM157 GTM_gtm_cls1.MCS1_MEM[157].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM158 GTM_gtm_cls1.MCS1_MEM[158].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM159 GTM_gtm_cls1.MCS1_MEM[159].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM160 GTM_gtm_cls1.MCS1_MEM[160].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM161 GTM_gtm_cls1.MCS1_MEM[161].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM162 GTM_gtm_cls1.MCS1_MEM[162].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM163 GTM_gtm_cls1.MCS1_MEM[163].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM164 GTM_gtm_cls1.MCS1_MEM[164].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM165 GTM_gtm_cls1.MCS1_MEM[165].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM166 GTM_gtm_cls1.MCS1_MEM[166].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM167 GTM_gtm_cls1.MCS1_MEM[167].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM168 GTM_gtm_cls1.MCS1_MEM[168].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM169 GTM_gtm_cls1.MCS1_MEM[169].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM170 GTM_gtm_cls1.MCS1_MEM[170].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM171 GTM_gtm_cls1.MCS1_MEM[171].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM172 GTM_gtm_cls1.MCS1_MEM[172].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM173 GTM_gtm_cls1.MCS1_MEM[173].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM174 GTM_gtm_cls1.MCS1_MEM[174].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM175 GTM_gtm_cls1.MCS1_MEM[175].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM176 GTM_gtm_cls1.MCS1_MEM[176].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM177 GTM_gtm_cls1.MCS1_MEM[177].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM178 GTM_gtm_cls1.MCS1_MEM[178].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM179 GTM_gtm_cls1.MCS1_MEM[179].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM180 GTM_gtm_cls1.MCS1_MEM[180].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM181 GTM_gtm_cls1.MCS1_MEM[181].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM182 GTM_gtm_cls1.MCS1_MEM[182].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM183 GTM_gtm_cls1.MCS1_MEM[183].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM184 GTM_gtm_cls1.MCS1_MEM[184].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM185 GTM_gtm_cls1.MCS1_MEM[185].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM186 GTM_gtm_cls1.MCS1_MEM[186].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM187 GTM_gtm_cls1.MCS1_MEM[187].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM188 GTM_gtm_cls1.MCS1_MEM[188].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM189 GTM_gtm_cls1.MCS1_MEM[189].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM190 GTM_gtm_cls1.MCS1_MEM[190].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM191 GTM_gtm_cls1.MCS1_MEM[191].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM192 GTM_gtm_cls1.MCS1_MEM[192].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM193 GTM_gtm_cls1.MCS1_MEM[193].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM194 GTM_gtm_cls1.MCS1_MEM[194].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM195 GTM_gtm_cls1.MCS1_MEM[195].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM196 GTM_gtm_cls1.MCS1_MEM[196].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM197 GTM_gtm_cls1.MCS1_MEM[197].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM198 GTM_gtm_cls1.MCS1_MEM[198].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM199 GTM_gtm_cls1.MCS1_MEM[199].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM200 GTM_gtm_cls1.MCS1_MEM[200].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM201 GTM_gtm_cls1.MCS1_MEM[201].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM202 GTM_gtm_cls1.MCS1_MEM[202].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM203 GTM_gtm_cls1.MCS1_MEM[203].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM204 GTM_gtm_cls1.MCS1_MEM[204].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM205 GTM_gtm_cls1.MCS1_MEM[205].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM206 GTM_gtm_cls1.MCS1_MEM[206].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM207 GTM_gtm_cls1.MCS1_MEM[207].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM208 GTM_gtm_cls1.MCS1_MEM[208].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM209 GTM_gtm_cls1.MCS1_MEM[209].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM210 GTM_gtm_cls1.MCS1_MEM[210].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM211 GTM_gtm_cls1.MCS1_MEM[211].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM212 GTM_gtm_cls1.MCS1_MEM[212].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM213 GTM_gtm_cls1.MCS1_MEM[213].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM214 GTM_gtm_cls1.MCS1_MEM[214].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM215 GTM_gtm_cls1.MCS1_MEM[215].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM216 GTM_gtm_cls1.MCS1_MEM[216].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM217 GTM_gtm_cls1.MCS1_MEM[217].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM218 GTM_gtm_cls1.MCS1_MEM[218].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM219 GTM_gtm_cls1.MCS1_MEM[219].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM220 GTM_gtm_cls1.MCS1_MEM[220].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM221 GTM_gtm_cls1.MCS1_MEM[221].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM222 GTM_gtm_cls1.MCS1_MEM[222].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM223 GTM_gtm_cls1.MCS1_MEM[223].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM224 GTM_gtm_cls1.MCS1_MEM[224].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM225 GTM_gtm_cls1.MCS1_MEM[225].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM226 GTM_gtm_cls1.MCS1_MEM[226].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM227 GTM_gtm_cls1.MCS1_MEM[227].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM228 GTM_gtm_cls1.MCS1_MEM[228].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM229 GTM_gtm_cls1.MCS1_MEM[229].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM230 GTM_gtm_cls1.MCS1_MEM[230].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM231 GTM_gtm_cls1.MCS1_MEM[231].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM232 GTM_gtm_cls1.MCS1_MEM[232].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM233 GTM_gtm_cls1.MCS1_MEM[233].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM234 GTM_gtm_cls1.MCS1_MEM[234].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM235 GTM_gtm_cls1.MCS1_MEM[235].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM236 GTM_gtm_cls1.MCS1_MEM[236].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM237 GTM_gtm_cls1.MCS1_MEM[237].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM238 GTM_gtm_cls1.MCS1_MEM[238].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM239 GTM_gtm_cls1.MCS1_MEM[239].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM240 GTM_gtm_cls1.MCS1_MEM[240].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM241 GTM_gtm_cls1.MCS1_MEM[241].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM242 GTM_gtm_cls1.MCS1_MEM[242].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM243 GTM_gtm_cls1.MCS1_MEM[243].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM244 GTM_gtm_cls1.MCS1_MEM[244].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM245 GTM_gtm_cls1.MCS1_MEM[245].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM246 GTM_gtm_cls1.MCS1_MEM[246].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM247 GTM_gtm_cls1.MCS1_MEM[247].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM248 GTM_gtm_cls1.MCS1_MEM[248].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM249 GTM_gtm_cls1.MCS1_MEM[249].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM250 GTM_gtm_cls1.MCS1_MEM[250].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM251 GTM_gtm_cls1.MCS1_MEM[251].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM252 GTM_gtm_cls1.MCS1_MEM[252].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM253 GTM_gtm_cls1.MCS1_MEM[253].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM254 GTM_gtm_cls1.MCS1_MEM[254].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM255 GTM_gtm_cls1.MCS1_MEM[255].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM256 GTM_gtm_cls1.MCS1_MEM[256].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM257 GTM_gtm_cls1.MCS1_MEM[257].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM258 GTM_gtm_cls1.MCS1_MEM[258].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM259 GTM_gtm_cls1.MCS1_MEM[259].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM260 GTM_gtm_cls1.MCS1_MEM[260].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM261 GTM_gtm_cls1.MCS1_MEM[261].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM262 GTM_gtm_cls1.MCS1_MEM[262].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM263 GTM_gtm_cls1.MCS1_MEM[263].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM264 GTM_gtm_cls1.MCS1_MEM[264].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM265 GTM_gtm_cls1.MCS1_MEM[265].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM266 GTM_gtm_cls1.MCS1_MEM[266].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM267 GTM_gtm_cls1.MCS1_MEM[267].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM268 GTM_gtm_cls1.MCS1_MEM[268].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM269 GTM_gtm_cls1.MCS1_MEM[269].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM270 GTM_gtm_cls1.MCS1_MEM[270].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM271 GTM_gtm_cls1.MCS1_MEM[271].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM272 GTM_gtm_cls1.MCS1_MEM[272].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM273 GTM_gtm_cls1.MCS1_MEM[273].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM274 GTM_gtm_cls1.MCS1_MEM[274].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM275 GTM_gtm_cls1.MCS1_MEM[275].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM276 GTM_gtm_cls1.MCS1_MEM[276].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM277 GTM_gtm_cls1.MCS1_MEM[277].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM278 GTM_gtm_cls1.MCS1_MEM[278].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM279 GTM_gtm_cls1.MCS1_MEM[279].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM280 GTM_gtm_cls1.MCS1_MEM[280].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM281 GTM_gtm_cls1.MCS1_MEM[281].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM282 GTM_gtm_cls1.MCS1_MEM[282].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM283 GTM_gtm_cls1.MCS1_MEM[283].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM284 GTM_gtm_cls1.MCS1_MEM[284].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM285 GTM_gtm_cls1.MCS1_MEM[285].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM286 GTM_gtm_cls1.MCS1_MEM[286].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM287 GTM_gtm_cls1.MCS1_MEM[287].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM288 GTM_gtm_cls1.MCS1_MEM[288].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM289 GTM_gtm_cls1.MCS1_MEM[289].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM290 GTM_gtm_cls1.MCS1_MEM[290].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM291 GTM_gtm_cls1.MCS1_MEM[291].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM292 GTM_gtm_cls1.MCS1_MEM[292].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM293 GTM_gtm_cls1.MCS1_MEM[293].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM294 GTM_gtm_cls1.MCS1_MEM[294].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM295 GTM_gtm_cls1.MCS1_MEM[295].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM296 GTM_gtm_cls1.MCS1_MEM[296].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM297 GTM_gtm_cls1.MCS1_MEM[297].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM298 GTM_gtm_cls1.MCS1_MEM[298].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM299 GTM_gtm_cls1.MCS1_MEM[299].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM300 GTM_gtm_cls1.MCS1_MEM[300].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM301 GTM_gtm_cls1.MCS1_MEM[301].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM302 GTM_gtm_cls1.MCS1_MEM[302].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM303 GTM_gtm_cls1.MCS1_MEM[303].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM304 GTM_gtm_cls1.MCS1_MEM[304].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM305 GTM_gtm_cls1.MCS1_MEM[305].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM306 GTM_gtm_cls1.MCS1_MEM[306].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM307 GTM_gtm_cls1.MCS1_MEM[307].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM308 GTM_gtm_cls1.MCS1_MEM[308].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM309 GTM_gtm_cls1.MCS1_MEM[309].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM310 GTM_gtm_cls1.MCS1_MEM[310].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM311 GTM_gtm_cls1.MCS1_MEM[311].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM312 GTM_gtm_cls1.MCS1_MEM[312].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM313 GTM_gtm_cls1.MCS1_MEM[313].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM314 GTM_gtm_cls1.MCS1_MEM[314].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM315 GTM_gtm_cls1.MCS1_MEM[315].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM316 GTM_gtm_cls1.MCS1_MEM[316].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM317 GTM_gtm_cls1.MCS1_MEM[317].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM318 GTM_gtm_cls1.MCS1_MEM[318].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM319 GTM_gtm_cls1.MCS1_MEM[319].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM320 GTM_gtm_cls1.MCS1_MEM[320].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM321 GTM_gtm_cls1.MCS1_MEM[321].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM322 GTM_gtm_cls1.MCS1_MEM[322].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM323 GTM_gtm_cls1.MCS1_MEM[323].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM324 GTM_gtm_cls1.MCS1_MEM[324].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM325 GTM_gtm_cls1.MCS1_MEM[325].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM326 GTM_gtm_cls1.MCS1_MEM[326].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM327 GTM_gtm_cls1.MCS1_MEM[327].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM328 GTM_gtm_cls1.MCS1_MEM[328].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM329 GTM_gtm_cls1.MCS1_MEM[329].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM330 GTM_gtm_cls1.MCS1_MEM[330].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM331 GTM_gtm_cls1.MCS1_MEM[331].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM332 GTM_gtm_cls1.MCS1_MEM[332].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM333 GTM_gtm_cls1.MCS1_MEM[333].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM334 GTM_gtm_cls1.MCS1_MEM[334].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM335 GTM_gtm_cls1.MCS1_MEM[335].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM336 GTM_gtm_cls1.MCS1_MEM[336].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM337 GTM_gtm_cls1.MCS1_MEM[337].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM338 GTM_gtm_cls1.MCS1_MEM[338].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM339 GTM_gtm_cls1.MCS1_MEM[339].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM340 GTM_gtm_cls1.MCS1_MEM[340].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM341 GTM_gtm_cls1.MCS1_MEM[341].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM342 GTM_gtm_cls1.MCS1_MEM[342].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM343 GTM_gtm_cls1.MCS1_MEM[343].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM344 GTM_gtm_cls1.MCS1_MEM[344].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM345 GTM_gtm_cls1.MCS1_MEM[345].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM346 GTM_gtm_cls1.MCS1_MEM[346].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM347 GTM_gtm_cls1.MCS1_MEM[347].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM348 GTM_gtm_cls1.MCS1_MEM[348].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM349 GTM_gtm_cls1.MCS1_MEM[349].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM350 GTM_gtm_cls1.MCS1_MEM[350].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM351 GTM_gtm_cls1.MCS1_MEM[351].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM352 GTM_gtm_cls1.MCS1_MEM[352].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM353 GTM_gtm_cls1.MCS1_MEM[353].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM354 GTM_gtm_cls1.MCS1_MEM[354].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM355 GTM_gtm_cls1.MCS1_MEM[355].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM356 GTM_gtm_cls1.MCS1_MEM[356].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM357 GTM_gtm_cls1.MCS1_MEM[357].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM358 GTM_gtm_cls1.MCS1_MEM[358].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM359 GTM_gtm_cls1.MCS1_MEM[359].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM360 GTM_gtm_cls1.MCS1_MEM[360].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM361 GTM_gtm_cls1.MCS1_MEM[361].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM362 GTM_gtm_cls1.MCS1_MEM[362].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM363 GTM_gtm_cls1.MCS1_MEM[363].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM364 GTM_gtm_cls1.MCS1_MEM[364].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM365 GTM_gtm_cls1.MCS1_MEM[365].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM366 GTM_gtm_cls1.MCS1_MEM[366].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM367 GTM_gtm_cls1.MCS1_MEM[367].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM368 GTM_gtm_cls1.MCS1_MEM[368].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM369 GTM_gtm_cls1.MCS1_MEM[369].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM370 GTM_gtm_cls1.MCS1_MEM[370].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM371 GTM_gtm_cls1.MCS1_MEM[371].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM372 GTM_gtm_cls1.MCS1_MEM[372].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM373 GTM_gtm_cls1.MCS1_MEM[373].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM374 GTM_gtm_cls1.MCS1_MEM[374].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM375 GTM_gtm_cls1.MCS1_MEM[375].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM376 GTM_gtm_cls1.MCS1_MEM[376].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM377 GTM_gtm_cls1.MCS1_MEM[377].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM378 GTM_gtm_cls1.MCS1_MEM[378].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM379 GTM_gtm_cls1.MCS1_MEM[379].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM380 GTM_gtm_cls1.MCS1_MEM[380].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM381 GTM_gtm_cls1.MCS1_MEM[381].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM382 GTM_gtm_cls1.MCS1_MEM[382].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM383 GTM_gtm_cls1.MCS1_MEM[383].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM384 GTM_gtm_cls1.MCS1_MEM[384].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM385 GTM_gtm_cls1.MCS1_MEM[385].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM386 GTM_gtm_cls1.MCS1_MEM[386].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM387 GTM_gtm_cls1.MCS1_MEM[387].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM388 GTM_gtm_cls1.MCS1_MEM[388].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM389 GTM_gtm_cls1.MCS1_MEM[389].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM390 GTM_gtm_cls1.MCS1_MEM[390].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM391 GTM_gtm_cls1.MCS1_MEM[391].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM392 GTM_gtm_cls1.MCS1_MEM[392].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM393 GTM_gtm_cls1.MCS1_MEM[393].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM394 GTM_gtm_cls1.MCS1_MEM[394].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM395 GTM_gtm_cls1.MCS1_MEM[395].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM396 GTM_gtm_cls1.MCS1_MEM[396].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM397 GTM_gtm_cls1.MCS1_MEM[397].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM398 GTM_gtm_cls1.MCS1_MEM[398].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM399 GTM_gtm_cls1.MCS1_MEM[399].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM400 GTM_gtm_cls1.MCS1_MEM[400].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM401 GTM_gtm_cls1.MCS1_MEM[401].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM402 GTM_gtm_cls1.MCS1_MEM[402].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM403 GTM_gtm_cls1.MCS1_MEM[403].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM404 GTM_gtm_cls1.MCS1_MEM[404].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM405 GTM_gtm_cls1.MCS1_MEM[405].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM406 GTM_gtm_cls1.MCS1_MEM[406].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM407 GTM_gtm_cls1.MCS1_MEM[407].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM408 GTM_gtm_cls1.MCS1_MEM[408].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM409 GTM_gtm_cls1.MCS1_MEM[409].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM410 GTM_gtm_cls1.MCS1_MEM[410].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM411 GTM_gtm_cls1.MCS1_MEM[411].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM412 GTM_gtm_cls1.MCS1_MEM[412].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM413 GTM_gtm_cls1.MCS1_MEM[413].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM414 GTM_gtm_cls1.MCS1_MEM[414].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM415 GTM_gtm_cls1.MCS1_MEM[415].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM416 GTM_gtm_cls1.MCS1_MEM[416].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM417 GTM_gtm_cls1.MCS1_MEM[417].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM418 GTM_gtm_cls1.MCS1_MEM[418].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM419 GTM_gtm_cls1.MCS1_MEM[419].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM420 GTM_gtm_cls1.MCS1_MEM[420].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM421 GTM_gtm_cls1.MCS1_MEM[421].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM422 GTM_gtm_cls1.MCS1_MEM[422].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM423 GTM_gtm_cls1.MCS1_MEM[423].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM424 GTM_gtm_cls1.MCS1_MEM[424].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM425 GTM_gtm_cls1.MCS1_MEM[425].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM426 GTM_gtm_cls1.MCS1_MEM[426].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM427 GTM_gtm_cls1.MCS1_MEM[427].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM428 GTM_gtm_cls1.MCS1_MEM[428].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM429 GTM_gtm_cls1.MCS1_MEM[429].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM430 GTM_gtm_cls1.MCS1_MEM[430].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM431 GTM_gtm_cls1.MCS1_MEM[431].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM432 GTM_gtm_cls1.MCS1_MEM[432].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM433 GTM_gtm_cls1.MCS1_MEM[433].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM434 GTM_gtm_cls1.MCS1_MEM[434].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM435 GTM_gtm_cls1.MCS1_MEM[435].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM436 GTM_gtm_cls1.MCS1_MEM[436].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM437 GTM_gtm_cls1.MCS1_MEM[437].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM438 GTM_gtm_cls1.MCS1_MEM[438].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM439 GTM_gtm_cls1.MCS1_MEM[439].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM440 GTM_gtm_cls1.MCS1_MEM[440].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM441 GTM_gtm_cls1.MCS1_MEM[441].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM442 GTM_gtm_cls1.MCS1_MEM[442].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM443 GTM_gtm_cls1.MCS1_MEM[443].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM444 GTM_gtm_cls1.MCS1_MEM[444].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM445 GTM_gtm_cls1.MCS1_MEM[445].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM446 GTM_gtm_cls1.MCS1_MEM[446].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM447 GTM_gtm_cls1.MCS1_MEM[447].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM448 GTM_gtm_cls1.MCS1_MEM[448].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM449 GTM_gtm_cls1.MCS1_MEM[449].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM450 GTM_gtm_cls1.MCS1_MEM[450].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM451 GTM_gtm_cls1.MCS1_MEM[451].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM452 GTM_gtm_cls1.MCS1_MEM[452].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM453 GTM_gtm_cls1.MCS1_MEM[453].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM454 GTM_gtm_cls1.MCS1_MEM[454].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM455 GTM_gtm_cls1.MCS1_MEM[455].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM456 GTM_gtm_cls1.MCS1_MEM[456].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM457 GTM_gtm_cls1.MCS1_MEM[457].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM458 GTM_gtm_cls1.MCS1_MEM[458].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM459 GTM_gtm_cls1.MCS1_MEM[459].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM460 GTM_gtm_cls1.MCS1_MEM[460].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM461 GTM_gtm_cls1.MCS1_MEM[461].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM462 GTM_gtm_cls1.MCS1_MEM[462].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM463 GTM_gtm_cls1.MCS1_MEM[463].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM464 GTM_gtm_cls1.MCS1_MEM[464].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM465 GTM_gtm_cls1.MCS1_MEM[465].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM466 GTM_gtm_cls1.MCS1_MEM[466].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM467 GTM_gtm_cls1.MCS1_MEM[467].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM468 GTM_gtm_cls1.MCS1_MEM[468].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM469 GTM_gtm_cls1.MCS1_MEM[469].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM470 GTM_gtm_cls1.MCS1_MEM[470].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM471 GTM_gtm_cls1.MCS1_MEM[471].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM472 GTM_gtm_cls1.MCS1_MEM[472].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM473 GTM_gtm_cls1.MCS1_MEM[473].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM474 GTM_gtm_cls1.MCS1_MEM[474].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM475 GTM_gtm_cls1.MCS1_MEM[475].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM476 GTM_gtm_cls1.MCS1_MEM[476].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM477 GTM_gtm_cls1.MCS1_MEM[477].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM478 GTM_gtm_cls1.MCS1_MEM[478].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM479 GTM_gtm_cls1.MCS1_MEM[479].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM480 GTM_gtm_cls1.MCS1_MEM[480].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM481 GTM_gtm_cls1.MCS1_MEM[481].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM482 GTM_gtm_cls1.MCS1_MEM[482].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM483 GTM_gtm_cls1.MCS1_MEM[483].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM484 GTM_gtm_cls1.MCS1_MEM[484].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM485 GTM_gtm_cls1.MCS1_MEM[485].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM486 GTM_gtm_cls1.MCS1_MEM[486].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM487 GTM_gtm_cls1.MCS1_MEM[487].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM488 GTM_gtm_cls1.MCS1_MEM[488].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM489 GTM_gtm_cls1.MCS1_MEM[489].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM490 GTM_gtm_cls1.MCS1_MEM[490].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM491 GTM_gtm_cls1.MCS1_MEM[491].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM492 GTM_gtm_cls1.MCS1_MEM[492].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM493 GTM_gtm_cls1.MCS1_MEM[493].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM494 GTM_gtm_cls1.MCS1_MEM[494].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM495 GTM_gtm_cls1.MCS1_MEM[495].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM496 GTM_gtm_cls1.MCS1_MEM[496].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM497 GTM_gtm_cls1.MCS1_MEM[497].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM498 GTM_gtm_cls1.MCS1_MEM[498].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM499 GTM_gtm_cls1.MCS1_MEM[499].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM500 GTM_gtm_cls1.MCS1_MEM[500].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM501 GTM_gtm_cls1.MCS1_MEM[501].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM502 GTM_gtm_cls1.MCS1_MEM[502].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM503 GTM_gtm_cls1.MCS1_MEM[503].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM504 GTM_gtm_cls1.MCS1_MEM[504].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM505 GTM_gtm_cls1.MCS1_MEM[505].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM506 GTM_gtm_cls1.MCS1_MEM[506].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM507 GTM_gtm_cls1.MCS1_MEM[507].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM508 GTM_gtm_cls1.MCS1_MEM[508].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM509 GTM_gtm_cls1.MCS1_MEM[509].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM510 GTM_gtm_cls1.MCS1_MEM[510].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM511 GTM_gtm_cls1.MCS1_MEM[511].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM512 GTM_gtm_cls1.MCS1_MEM[512].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM513 GTM_gtm_cls1.MCS1_MEM[513].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM514 GTM_gtm_cls1.MCS1_MEM[514].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM515 GTM_gtm_cls1.MCS1_MEM[515].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM516 GTM_gtm_cls1.MCS1_MEM[516].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM517 GTM_gtm_cls1.MCS1_MEM[517].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM518 GTM_gtm_cls1.MCS1_MEM[518].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM519 GTM_gtm_cls1.MCS1_MEM[519].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM520 GTM_gtm_cls1.MCS1_MEM[520].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM521 GTM_gtm_cls1.MCS1_MEM[521].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM522 GTM_gtm_cls1.MCS1_MEM[522].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM523 GTM_gtm_cls1.MCS1_MEM[523].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM524 GTM_gtm_cls1.MCS1_MEM[524].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM525 GTM_gtm_cls1.MCS1_MEM[525].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM526 GTM_gtm_cls1.MCS1_MEM[526].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM527 GTM_gtm_cls1.MCS1_MEM[527].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM528 GTM_gtm_cls1.MCS1_MEM[528].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM529 GTM_gtm_cls1.MCS1_MEM[529].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM530 GTM_gtm_cls1.MCS1_MEM[530].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM531 GTM_gtm_cls1.MCS1_MEM[531].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM532 GTM_gtm_cls1.MCS1_MEM[532].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM533 GTM_gtm_cls1.MCS1_MEM[533].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM534 GTM_gtm_cls1.MCS1_MEM[534].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM535 GTM_gtm_cls1.MCS1_MEM[535].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM536 GTM_gtm_cls1.MCS1_MEM[536].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM537 GTM_gtm_cls1.MCS1_MEM[537].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM538 GTM_gtm_cls1.MCS1_MEM[538].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM539 GTM_gtm_cls1.MCS1_MEM[539].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM540 GTM_gtm_cls1.MCS1_MEM[540].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM541 GTM_gtm_cls1.MCS1_MEM[541].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM542 GTM_gtm_cls1.MCS1_MEM[542].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM543 GTM_gtm_cls1.MCS1_MEM[543].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM544 GTM_gtm_cls1.MCS1_MEM[544].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM545 GTM_gtm_cls1.MCS1_MEM[545].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM546 GTM_gtm_cls1.MCS1_MEM[546].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM547 GTM_gtm_cls1.MCS1_MEM[547].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM548 GTM_gtm_cls1.MCS1_MEM[548].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM549 GTM_gtm_cls1.MCS1_MEM[549].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM550 GTM_gtm_cls1.MCS1_MEM[550].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM551 GTM_gtm_cls1.MCS1_MEM[551].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM552 GTM_gtm_cls1.MCS1_MEM[552].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM553 GTM_gtm_cls1.MCS1_MEM[553].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM554 GTM_gtm_cls1.MCS1_MEM[554].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM555 GTM_gtm_cls1.MCS1_MEM[555].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM556 GTM_gtm_cls1.MCS1_MEM[556].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM557 GTM_gtm_cls1.MCS1_MEM[557].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM558 GTM_gtm_cls1.MCS1_MEM[558].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM559 GTM_gtm_cls1.MCS1_MEM[559].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM560 GTM_gtm_cls1.MCS1_MEM[560].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM561 GTM_gtm_cls1.MCS1_MEM[561].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM562 GTM_gtm_cls1.MCS1_MEM[562].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM563 GTM_gtm_cls1.MCS1_MEM[563].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM564 GTM_gtm_cls1.MCS1_MEM[564].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM565 GTM_gtm_cls1.MCS1_MEM[565].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM566 GTM_gtm_cls1.MCS1_MEM[566].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM567 GTM_gtm_cls1.MCS1_MEM[567].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM568 GTM_gtm_cls1.MCS1_MEM[568].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM569 GTM_gtm_cls1.MCS1_MEM[569].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM570 GTM_gtm_cls1.MCS1_MEM[570].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM571 GTM_gtm_cls1.MCS1_MEM[571].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM572 GTM_gtm_cls1.MCS1_MEM[572].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM573 GTM_gtm_cls1.MCS1_MEM[573].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM574 GTM_gtm_cls1.MCS1_MEM[574].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM575 GTM_gtm_cls1.MCS1_MEM[575].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM576 GTM_gtm_cls1.MCS1_MEM[576].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM577 GTM_gtm_cls1.MCS1_MEM[577].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM578 GTM_gtm_cls1.MCS1_MEM[578].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM579 GTM_gtm_cls1.MCS1_MEM[579].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM580 GTM_gtm_cls1.MCS1_MEM[580].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM581 GTM_gtm_cls1.MCS1_MEM[581].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM582 GTM_gtm_cls1.MCS1_MEM[582].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM583 GTM_gtm_cls1.MCS1_MEM[583].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM584 GTM_gtm_cls1.MCS1_MEM[584].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM585 GTM_gtm_cls1.MCS1_MEM[585].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM586 GTM_gtm_cls1.MCS1_MEM[586].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM587 GTM_gtm_cls1.MCS1_MEM[587].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM588 GTM_gtm_cls1.MCS1_MEM[588].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM589 GTM_gtm_cls1.MCS1_MEM[589].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM590 GTM_gtm_cls1.MCS1_MEM[590].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM591 GTM_gtm_cls1.MCS1_MEM[591].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM592 GTM_gtm_cls1.MCS1_MEM[592].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM593 GTM_gtm_cls1.MCS1_MEM[593].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM594 GTM_gtm_cls1.MCS1_MEM[594].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM595 GTM_gtm_cls1.MCS1_MEM[595].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM596 GTM_gtm_cls1.MCS1_MEM[596].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM597 GTM_gtm_cls1.MCS1_MEM[597].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM598 GTM_gtm_cls1.MCS1_MEM[598].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM599 GTM_gtm_cls1.MCS1_MEM[599].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM600 GTM_gtm_cls1.MCS1_MEM[600].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM601 GTM_gtm_cls1.MCS1_MEM[601].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM602 GTM_gtm_cls1.MCS1_MEM[602].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM603 GTM_gtm_cls1.MCS1_MEM[603].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM604 GTM_gtm_cls1.MCS1_MEM[604].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM605 GTM_gtm_cls1.MCS1_MEM[605].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM606 GTM_gtm_cls1.MCS1_MEM[606].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM607 GTM_gtm_cls1.MCS1_MEM[607].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM608 GTM_gtm_cls1.MCS1_MEM[608].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM609 GTM_gtm_cls1.MCS1_MEM[609].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM610 GTM_gtm_cls1.MCS1_MEM[610].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM611 GTM_gtm_cls1.MCS1_MEM[611].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM612 GTM_gtm_cls1.MCS1_MEM[612].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM613 GTM_gtm_cls1.MCS1_MEM[613].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM614 GTM_gtm_cls1.MCS1_MEM[614].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM615 GTM_gtm_cls1.MCS1_MEM[615].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM616 GTM_gtm_cls1.MCS1_MEM[616].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM617 GTM_gtm_cls1.MCS1_MEM[617].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM618 GTM_gtm_cls1.MCS1_MEM[618].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM619 GTM_gtm_cls1.MCS1_MEM[619].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM620 GTM_gtm_cls1.MCS1_MEM[620].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM621 GTM_gtm_cls1.MCS1_MEM[621].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM622 GTM_gtm_cls1.MCS1_MEM[622].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM623 GTM_gtm_cls1.MCS1_MEM[623].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM624 GTM_gtm_cls1.MCS1_MEM[624].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM625 GTM_gtm_cls1.MCS1_MEM[625].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM626 GTM_gtm_cls1.MCS1_MEM[626].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM627 GTM_gtm_cls1.MCS1_MEM[627].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM628 GTM_gtm_cls1.MCS1_MEM[628].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM629 GTM_gtm_cls1.MCS1_MEM[629].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM630 GTM_gtm_cls1.MCS1_MEM[630].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM631 GTM_gtm_cls1.MCS1_MEM[631].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM632 GTM_gtm_cls1.MCS1_MEM[632].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM633 GTM_gtm_cls1.MCS1_MEM[633].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM634 GTM_gtm_cls1.MCS1_MEM[634].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM635 GTM_gtm_cls1.MCS1_MEM[635].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM636 GTM_gtm_cls1.MCS1_MEM[636].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM637 GTM_gtm_cls1.MCS1_MEM[637].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM638 GTM_gtm_cls1.MCS1_MEM[638].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM639 GTM_gtm_cls1.MCS1_MEM[639].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM640 GTM_gtm_cls1.MCS1_MEM[640].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM641 GTM_gtm_cls1.MCS1_MEM[641].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM642 GTM_gtm_cls1.MCS1_MEM[642].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM643 GTM_gtm_cls1.MCS1_MEM[643].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM644 GTM_gtm_cls1.MCS1_MEM[644].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM645 GTM_gtm_cls1.MCS1_MEM[645].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM646 GTM_gtm_cls1.MCS1_MEM[646].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM647 GTM_gtm_cls1.MCS1_MEM[647].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM648 GTM_gtm_cls1.MCS1_MEM[648].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM649 GTM_gtm_cls1.MCS1_MEM[649].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM650 GTM_gtm_cls1.MCS1_MEM[650].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM651 GTM_gtm_cls1.MCS1_MEM[651].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM652 GTM_gtm_cls1.MCS1_MEM[652].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM653 GTM_gtm_cls1.MCS1_MEM[653].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM654 GTM_gtm_cls1.MCS1_MEM[654].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM655 GTM_gtm_cls1.MCS1_MEM[655].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM656 GTM_gtm_cls1.MCS1_MEM[656].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM657 GTM_gtm_cls1.MCS1_MEM[657].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM658 GTM_gtm_cls1.MCS1_MEM[658].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM659 GTM_gtm_cls1.MCS1_MEM[659].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM660 GTM_gtm_cls1.MCS1_MEM[660].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM661 GTM_gtm_cls1.MCS1_MEM[661].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM662 GTM_gtm_cls1.MCS1_MEM[662].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM663 GTM_gtm_cls1.MCS1_MEM[663].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM664 GTM_gtm_cls1.MCS1_MEM[664].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM665 GTM_gtm_cls1.MCS1_MEM[665].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM666 GTM_gtm_cls1.MCS1_MEM[666].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM667 GTM_gtm_cls1.MCS1_MEM[667].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM668 GTM_gtm_cls1.MCS1_MEM[668].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM669 GTM_gtm_cls1.MCS1_MEM[669].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM670 GTM_gtm_cls1.MCS1_MEM[670].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM671 GTM_gtm_cls1.MCS1_MEM[671].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM672 GTM_gtm_cls1.MCS1_MEM[672].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM673 GTM_gtm_cls1.MCS1_MEM[673].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM674 GTM_gtm_cls1.MCS1_MEM[674].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM675 GTM_gtm_cls1.MCS1_MEM[675].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM676 GTM_gtm_cls1.MCS1_MEM[676].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM677 GTM_gtm_cls1.MCS1_MEM[677].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM678 GTM_gtm_cls1.MCS1_MEM[678].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM679 GTM_gtm_cls1.MCS1_MEM[679].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM680 GTM_gtm_cls1.MCS1_MEM[680].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM681 GTM_gtm_cls1.MCS1_MEM[681].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM682 GTM_gtm_cls1.MCS1_MEM[682].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM683 GTM_gtm_cls1.MCS1_MEM[683].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM684 GTM_gtm_cls1.MCS1_MEM[684].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM685 GTM_gtm_cls1.MCS1_MEM[685].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM686 GTM_gtm_cls1.MCS1_MEM[686].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM687 GTM_gtm_cls1.MCS1_MEM[687].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM688 GTM_gtm_cls1.MCS1_MEM[688].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM689 GTM_gtm_cls1.MCS1_MEM[689].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM690 GTM_gtm_cls1.MCS1_MEM[690].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM691 GTM_gtm_cls1.MCS1_MEM[691].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM692 GTM_gtm_cls1.MCS1_MEM[692].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM693 GTM_gtm_cls1.MCS1_MEM[693].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM694 GTM_gtm_cls1.MCS1_MEM[694].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM695 GTM_gtm_cls1.MCS1_MEM[695].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM696 GTM_gtm_cls1.MCS1_MEM[696].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM697 GTM_gtm_cls1.MCS1_MEM[697].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM698 GTM_gtm_cls1.MCS1_MEM[698].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM699 GTM_gtm_cls1.MCS1_MEM[699].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM700 GTM_gtm_cls1.MCS1_MEM[700].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM701 GTM_gtm_cls1.MCS1_MEM[701].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM702 GTM_gtm_cls1.MCS1_MEM[702].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM703 GTM_gtm_cls1.MCS1_MEM[703].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM704 GTM_gtm_cls1.MCS1_MEM[704].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM705 GTM_gtm_cls1.MCS1_MEM[705].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM706 GTM_gtm_cls1.MCS1_MEM[706].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM707 GTM_gtm_cls1.MCS1_MEM[707].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM708 GTM_gtm_cls1.MCS1_MEM[708].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM709 GTM_gtm_cls1.MCS1_MEM[709].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM710 GTM_gtm_cls1.MCS1_MEM[710].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM711 GTM_gtm_cls1.MCS1_MEM[711].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM712 GTM_gtm_cls1.MCS1_MEM[712].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM713 GTM_gtm_cls1.MCS1_MEM[713].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM714 GTM_gtm_cls1.MCS1_MEM[714].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM715 GTM_gtm_cls1.MCS1_MEM[715].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM716 GTM_gtm_cls1.MCS1_MEM[716].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM717 GTM_gtm_cls1.MCS1_MEM[717].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM718 GTM_gtm_cls1.MCS1_MEM[718].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM719 GTM_gtm_cls1.MCS1_MEM[719].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM720 GTM_gtm_cls1.MCS1_MEM[720].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM721 GTM_gtm_cls1.MCS1_MEM[721].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM722 GTM_gtm_cls1.MCS1_MEM[722].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM723 GTM_gtm_cls1.MCS1_MEM[723].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM724 GTM_gtm_cls1.MCS1_MEM[724].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM725 GTM_gtm_cls1.MCS1_MEM[725].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM726 GTM_gtm_cls1.MCS1_MEM[726].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM727 GTM_gtm_cls1.MCS1_MEM[727].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM728 GTM_gtm_cls1.MCS1_MEM[728].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM729 GTM_gtm_cls1.MCS1_MEM[729].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM730 GTM_gtm_cls1.MCS1_MEM[730].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM731 GTM_gtm_cls1.MCS1_MEM[731].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM732 GTM_gtm_cls1.MCS1_MEM[732].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM733 GTM_gtm_cls1.MCS1_MEM[733].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM734 GTM_gtm_cls1.MCS1_MEM[734].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM735 GTM_gtm_cls1.MCS1_MEM[735].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM736 GTM_gtm_cls1.MCS1_MEM[736].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM737 GTM_gtm_cls1.MCS1_MEM[737].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM738 GTM_gtm_cls1.MCS1_MEM[738].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM739 GTM_gtm_cls1.MCS1_MEM[739].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM740 GTM_gtm_cls1.MCS1_MEM[740].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM741 GTM_gtm_cls1.MCS1_MEM[741].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM742 GTM_gtm_cls1.MCS1_MEM[742].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM743 GTM_gtm_cls1.MCS1_MEM[743].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM744 GTM_gtm_cls1.MCS1_MEM[744].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM745 GTM_gtm_cls1.MCS1_MEM[745].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM746 GTM_gtm_cls1.MCS1_MEM[746].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM747 GTM_gtm_cls1.MCS1_MEM[747].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM748 GTM_gtm_cls1.MCS1_MEM[748].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM749 GTM_gtm_cls1.MCS1_MEM[749].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM750 GTM_gtm_cls1.MCS1_MEM[750].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM751 GTM_gtm_cls1.MCS1_MEM[751].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM752 GTM_gtm_cls1.MCS1_MEM[752].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM753 GTM_gtm_cls1.MCS1_MEM[753].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM754 GTM_gtm_cls1.MCS1_MEM[754].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM755 GTM_gtm_cls1.MCS1_MEM[755].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM756 GTM_gtm_cls1.MCS1_MEM[756].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM757 GTM_gtm_cls1.MCS1_MEM[757].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM758 GTM_gtm_cls1.MCS1_MEM[758].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM759 GTM_gtm_cls1.MCS1_MEM[759].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM760 GTM_gtm_cls1.MCS1_MEM[760].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM761 GTM_gtm_cls1.MCS1_MEM[761].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM762 GTM_gtm_cls1.MCS1_MEM[762].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM763 GTM_gtm_cls1.MCS1_MEM[763].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM764 GTM_gtm_cls1.MCS1_MEM[764].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM765 GTM_gtm_cls1.MCS1_MEM[765].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM766 GTM_gtm_cls1.MCS1_MEM[766].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM767 GTM_gtm_cls1.MCS1_MEM[767].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM768 GTM_gtm_cls1.MCS1_MEM[768].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM769 GTM_gtm_cls1.MCS1_MEM[769].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM770 GTM_gtm_cls1.MCS1_MEM[770].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM771 GTM_gtm_cls1.MCS1_MEM[771].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM772 GTM_gtm_cls1.MCS1_MEM[772].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM773 GTM_gtm_cls1.MCS1_MEM[773].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM774 GTM_gtm_cls1.MCS1_MEM[774].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM775 GTM_gtm_cls1.MCS1_MEM[775].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM776 GTM_gtm_cls1.MCS1_MEM[776].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM777 GTM_gtm_cls1.MCS1_MEM[777].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM778 GTM_gtm_cls1.MCS1_MEM[778].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM779 GTM_gtm_cls1.MCS1_MEM[779].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM780 GTM_gtm_cls1.MCS1_MEM[780].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM781 GTM_gtm_cls1.MCS1_MEM[781].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM782 GTM_gtm_cls1.MCS1_MEM[782].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM783 GTM_gtm_cls1.MCS1_MEM[783].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM784 GTM_gtm_cls1.MCS1_MEM[784].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM785 GTM_gtm_cls1.MCS1_MEM[785].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM786 GTM_gtm_cls1.MCS1_MEM[786].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM787 GTM_gtm_cls1.MCS1_MEM[787].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM788 GTM_gtm_cls1.MCS1_MEM[788].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM789 GTM_gtm_cls1.MCS1_MEM[789].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM790 GTM_gtm_cls1.MCS1_MEM[790].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM791 GTM_gtm_cls1.MCS1_MEM[791].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM792 GTM_gtm_cls1.MCS1_MEM[792].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM793 GTM_gtm_cls1.MCS1_MEM[793].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM794 GTM_gtm_cls1.MCS1_MEM[794].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM795 GTM_gtm_cls1.MCS1_MEM[795].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM796 GTM_gtm_cls1.MCS1_MEM[796].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM797 GTM_gtm_cls1.MCS1_MEM[797].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM798 GTM_gtm_cls1.MCS1_MEM[798].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM799 GTM_gtm_cls1.MCS1_MEM[799].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM800 GTM_gtm_cls1.MCS1_MEM[800].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM801 GTM_gtm_cls1.MCS1_MEM[801].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM802 GTM_gtm_cls1.MCS1_MEM[802].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM803 GTM_gtm_cls1.MCS1_MEM[803].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM804 GTM_gtm_cls1.MCS1_MEM[804].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM805 GTM_gtm_cls1.MCS1_MEM[805].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM806 GTM_gtm_cls1.MCS1_MEM[806].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM807 GTM_gtm_cls1.MCS1_MEM[807].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM808 GTM_gtm_cls1.MCS1_MEM[808].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM809 GTM_gtm_cls1.MCS1_MEM[809].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM810 GTM_gtm_cls1.MCS1_MEM[810].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM811 GTM_gtm_cls1.MCS1_MEM[811].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM812 GTM_gtm_cls1.MCS1_MEM[812].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM813 GTM_gtm_cls1.MCS1_MEM[813].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM814 GTM_gtm_cls1.MCS1_MEM[814].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM815 GTM_gtm_cls1.MCS1_MEM[815].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM816 GTM_gtm_cls1.MCS1_MEM[816].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM817 GTM_gtm_cls1.MCS1_MEM[817].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM818 GTM_gtm_cls1.MCS1_MEM[818].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM819 GTM_gtm_cls1.MCS1_MEM[819].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM820 GTM_gtm_cls1.MCS1_MEM[820].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM821 GTM_gtm_cls1.MCS1_MEM[821].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM822 GTM_gtm_cls1.MCS1_MEM[822].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM823 GTM_gtm_cls1.MCS1_MEM[823].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM824 GTM_gtm_cls1.MCS1_MEM[824].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM825 GTM_gtm_cls1.MCS1_MEM[825].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM826 GTM_gtm_cls1.MCS1_MEM[826].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM827 GTM_gtm_cls1.MCS1_MEM[827].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM828 GTM_gtm_cls1.MCS1_MEM[828].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM829 GTM_gtm_cls1.MCS1_MEM[829].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM830 GTM_gtm_cls1.MCS1_MEM[830].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM831 GTM_gtm_cls1.MCS1_MEM[831].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM832 GTM_gtm_cls1.MCS1_MEM[832].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM833 GTM_gtm_cls1.MCS1_MEM[833].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM834 GTM_gtm_cls1.MCS1_MEM[834].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM835 GTM_gtm_cls1.MCS1_MEM[835].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM836 GTM_gtm_cls1.MCS1_MEM[836].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM837 GTM_gtm_cls1.MCS1_MEM[837].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM838 GTM_gtm_cls1.MCS1_MEM[838].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM839 GTM_gtm_cls1.MCS1_MEM[839].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM840 GTM_gtm_cls1.MCS1_MEM[840].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM841 GTM_gtm_cls1.MCS1_MEM[841].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM842 GTM_gtm_cls1.MCS1_MEM[842].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM843 GTM_gtm_cls1.MCS1_MEM[843].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM844 GTM_gtm_cls1.MCS1_MEM[844].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM845 GTM_gtm_cls1.MCS1_MEM[845].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM846 GTM_gtm_cls1.MCS1_MEM[846].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM847 GTM_gtm_cls1.MCS1_MEM[847].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM848 GTM_gtm_cls1.MCS1_MEM[848].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM849 GTM_gtm_cls1.MCS1_MEM[849].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM850 GTM_gtm_cls1.MCS1_MEM[850].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM851 GTM_gtm_cls1.MCS1_MEM[851].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM852 GTM_gtm_cls1.MCS1_MEM[852].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM853 GTM_gtm_cls1.MCS1_MEM[853].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM854 GTM_gtm_cls1.MCS1_MEM[854].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM855 GTM_gtm_cls1.MCS1_MEM[855].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM856 GTM_gtm_cls1.MCS1_MEM[856].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM857 GTM_gtm_cls1.MCS1_MEM[857].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM858 GTM_gtm_cls1.MCS1_MEM[858].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM859 GTM_gtm_cls1.MCS1_MEM[859].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM860 GTM_gtm_cls1.MCS1_MEM[860].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM861 GTM_gtm_cls1.MCS1_MEM[861].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM862 GTM_gtm_cls1.MCS1_MEM[862].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM863 GTM_gtm_cls1.MCS1_MEM[863].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM864 GTM_gtm_cls1.MCS1_MEM[864].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM865 GTM_gtm_cls1.MCS1_MEM[865].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM866 GTM_gtm_cls1.MCS1_MEM[866].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM867 GTM_gtm_cls1.MCS1_MEM[867].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM868 GTM_gtm_cls1.MCS1_MEM[868].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM869 GTM_gtm_cls1.MCS1_MEM[869].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM870 GTM_gtm_cls1.MCS1_MEM[870].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM871 GTM_gtm_cls1.MCS1_MEM[871].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM872 GTM_gtm_cls1.MCS1_MEM[872].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM873 GTM_gtm_cls1.MCS1_MEM[873].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM874 GTM_gtm_cls1.MCS1_MEM[874].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM875 GTM_gtm_cls1.MCS1_MEM[875].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM876 GTM_gtm_cls1.MCS1_MEM[876].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM877 GTM_gtm_cls1.MCS1_MEM[877].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM878 GTM_gtm_cls1.MCS1_MEM[878].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM879 GTM_gtm_cls1.MCS1_MEM[879].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM880 GTM_gtm_cls1.MCS1_MEM[880].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM881 GTM_gtm_cls1.MCS1_MEM[881].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM882 GTM_gtm_cls1.MCS1_MEM[882].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM883 GTM_gtm_cls1.MCS1_MEM[883].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM884 GTM_gtm_cls1.MCS1_MEM[884].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM885 GTM_gtm_cls1.MCS1_MEM[885].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM886 GTM_gtm_cls1.MCS1_MEM[886].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM887 GTM_gtm_cls1.MCS1_MEM[887].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM888 GTM_gtm_cls1.MCS1_MEM[888].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM889 GTM_gtm_cls1.MCS1_MEM[889].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM890 GTM_gtm_cls1.MCS1_MEM[890].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM891 GTM_gtm_cls1.MCS1_MEM[891].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM892 GTM_gtm_cls1.MCS1_MEM[892].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM893 GTM_gtm_cls1.MCS1_MEM[893].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM894 GTM_gtm_cls1.MCS1_MEM[894].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM895 GTM_gtm_cls1.MCS1_MEM[895].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM896 GTM_gtm_cls1.MCS1_MEM[896].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM897 GTM_gtm_cls1.MCS1_MEM[897].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM898 GTM_gtm_cls1.MCS1_MEM[898].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM899 GTM_gtm_cls1.MCS1_MEM[899].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM900 GTM_gtm_cls1.MCS1_MEM[900].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM901 GTM_gtm_cls1.MCS1_MEM[901].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM902 GTM_gtm_cls1.MCS1_MEM[902].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM903 GTM_gtm_cls1.MCS1_MEM[903].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM904 GTM_gtm_cls1.MCS1_MEM[904].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM905 GTM_gtm_cls1.MCS1_MEM[905].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM906 GTM_gtm_cls1.MCS1_MEM[906].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM907 GTM_gtm_cls1.MCS1_MEM[907].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM908 GTM_gtm_cls1.MCS1_MEM[908].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM909 GTM_gtm_cls1.MCS1_MEM[909].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM910 GTM_gtm_cls1.MCS1_MEM[910].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM911 GTM_gtm_cls1.MCS1_MEM[911].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM912 GTM_gtm_cls1.MCS1_MEM[912].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM913 GTM_gtm_cls1.MCS1_MEM[913].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM914 GTM_gtm_cls1.MCS1_MEM[914].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM915 GTM_gtm_cls1.MCS1_MEM[915].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM916 GTM_gtm_cls1.MCS1_MEM[916].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM917 GTM_gtm_cls1.MCS1_MEM[917].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM918 GTM_gtm_cls1.MCS1_MEM[918].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM919 GTM_gtm_cls1.MCS1_MEM[919].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM920 GTM_gtm_cls1.MCS1_MEM[920].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM921 GTM_gtm_cls1.MCS1_MEM[921].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM922 GTM_gtm_cls1.MCS1_MEM[922].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM923 GTM_gtm_cls1.MCS1_MEM[923].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM924 GTM_gtm_cls1.MCS1_MEM[924].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM925 GTM_gtm_cls1.MCS1_MEM[925].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM926 GTM_gtm_cls1.MCS1_MEM[926].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM927 GTM_gtm_cls1.MCS1_MEM[927].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM928 GTM_gtm_cls1.MCS1_MEM[928].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM929 GTM_gtm_cls1.MCS1_MEM[929].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM930 GTM_gtm_cls1.MCS1_MEM[930].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM931 GTM_gtm_cls1.MCS1_MEM[931].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM932 GTM_gtm_cls1.MCS1_MEM[932].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM933 GTM_gtm_cls1.MCS1_MEM[933].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM934 GTM_gtm_cls1.MCS1_MEM[934].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM935 GTM_gtm_cls1.MCS1_MEM[935].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM936 GTM_gtm_cls1.MCS1_MEM[936].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM937 GTM_gtm_cls1.MCS1_MEM[937].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM938 GTM_gtm_cls1.MCS1_MEM[938].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM939 GTM_gtm_cls1.MCS1_MEM[939].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM940 GTM_gtm_cls1.MCS1_MEM[940].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM941 GTM_gtm_cls1.MCS1_MEM[941].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM942 GTM_gtm_cls1.MCS1_MEM[942].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM943 GTM_gtm_cls1.MCS1_MEM[943].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM944 GTM_gtm_cls1.MCS1_MEM[944].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM945 GTM_gtm_cls1.MCS1_MEM[945].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM946 GTM_gtm_cls1.MCS1_MEM[946].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM947 GTM_gtm_cls1.MCS1_MEM[947].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM948 GTM_gtm_cls1.MCS1_MEM[948].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM949 GTM_gtm_cls1.MCS1_MEM[949].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM950 GTM_gtm_cls1.MCS1_MEM[950].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM951 GTM_gtm_cls1.MCS1_MEM[951].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM952 GTM_gtm_cls1.MCS1_MEM[952].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM953 GTM_gtm_cls1.MCS1_MEM[953].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM954 GTM_gtm_cls1.MCS1_MEM[954].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM955 GTM_gtm_cls1.MCS1_MEM[955].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM956 GTM_gtm_cls1.MCS1_MEM[956].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM957 GTM_gtm_cls1.MCS1_MEM[957].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM958 GTM_gtm_cls1.MCS1_MEM[958].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM959 GTM_gtm_cls1.MCS1_MEM[959].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM960 GTM_gtm_cls1.MCS1_MEM[960].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM961 GTM_gtm_cls1.MCS1_MEM[961].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM962 GTM_gtm_cls1.MCS1_MEM[962].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM963 GTM_gtm_cls1.MCS1_MEM[963].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM964 GTM_gtm_cls1.MCS1_MEM[964].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM965 GTM_gtm_cls1.MCS1_MEM[965].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM966 GTM_gtm_cls1.MCS1_MEM[966].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM967 GTM_gtm_cls1.MCS1_MEM[967].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM968 GTM_gtm_cls1.MCS1_MEM[968].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM969 GTM_gtm_cls1.MCS1_MEM[969].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM970 GTM_gtm_cls1.MCS1_MEM[970].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM971 GTM_gtm_cls1.MCS1_MEM[971].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM972 GTM_gtm_cls1.MCS1_MEM[972].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM973 GTM_gtm_cls1.MCS1_MEM[973].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM974 GTM_gtm_cls1.MCS1_MEM[974].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM975 GTM_gtm_cls1.MCS1_MEM[975].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM976 GTM_gtm_cls1.MCS1_MEM[976].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM977 GTM_gtm_cls1.MCS1_MEM[977].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM978 GTM_gtm_cls1.MCS1_MEM[978].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM979 GTM_gtm_cls1.MCS1_MEM[979].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM980 GTM_gtm_cls1.MCS1_MEM[980].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM981 GTM_gtm_cls1.MCS1_MEM[981].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM982 GTM_gtm_cls1.MCS1_MEM[982].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM983 GTM_gtm_cls1.MCS1_MEM[983].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM984 GTM_gtm_cls1.MCS1_MEM[984].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM985 GTM_gtm_cls1.MCS1_MEM[985].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM986 GTM_gtm_cls1.MCS1_MEM[986].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM987 GTM_gtm_cls1.MCS1_MEM[987].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM988 GTM_gtm_cls1.MCS1_MEM[988].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM989 GTM_gtm_cls1.MCS1_MEM[989].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM990 GTM_gtm_cls1.MCS1_MEM[990].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM991 GTM_gtm_cls1.MCS1_MEM[991].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM992 GTM_gtm_cls1.MCS1_MEM[992].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM993 GTM_gtm_cls1.MCS1_MEM[993].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM994 GTM_gtm_cls1.MCS1_MEM[994].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM995 GTM_gtm_cls1.MCS1_MEM[995].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM996 GTM_gtm_cls1.MCS1_MEM[996].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM997 GTM_gtm_cls1.MCS1_MEM[997].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM998 GTM_gtm_cls1.MCS1_MEM[998].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM999 GTM_gtm_cls1.MCS1_MEM[999].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1000 GTM_gtm_cls1.MCS1_MEM[1000].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1001 GTM_gtm_cls1.MCS1_MEM[1001].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1002 GTM_gtm_cls1.MCS1_MEM[1002].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1003 GTM_gtm_cls1.MCS1_MEM[1003].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1004 GTM_gtm_cls1.MCS1_MEM[1004].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1005 GTM_gtm_cls1.MCS1_MEM[1005].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1006 GTM_gtm_cls1.MCS1_MEM[1006].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1007 GTM_gtm_cls1.MCS1_MEM[1007].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1008 GTM_gtm_cls1.MCS1_MEM[1008].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1009 GTM_gtm_cls1.MCS1_MEM[1009].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1010 GTM_gtm_cls1.MCS1_MEM[1010].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1011 GTM_gtm_cls1.MCS1_MEM[1011].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1012 GTM_gtm_cls1.MCS1_MEM[1012].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1013 GTM_gtm_cls1.MCS1_MEM[1013].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1014 GTM_gtm_cls1.MCS1_MEM[1014].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1015 GTM_gtm_cls1.MCS1_MEM[1015].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1016 GTM_gtm_cls1.MCS1_MEM[1016].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1017 GTM_gtm_cls1.MCS1_MEM[1017].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1018 GTM_gtm_cls1.MCS1_MEM[1018].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1019 GTM_gtm_cls1.MCS1_MEM[1019].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1020 GTM_gtm_cls1.MCS1_MEM[1020].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1021 GTM_gtm_cls1.MCS1_MEM[1021].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1022 GTM_gtm_cls1.MCS1_MEM[1022].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1023 GTM_gtm_cls1.MCS1_MEM[1023].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1024 GTM_gtm_cls1.MCS1_MEM[1024].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1025 GTM_gtm_cls1.MCS1_MEM[1025].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1026 GTM_gtm_cls1.MCS1_MEM[1026].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1027 GTM_gtm_cls1.MCS1_MEM[1027].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1028 GTM_gtm_cls1.MCS1_MEM[1028].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1029 GTM_gtm_cls1.MCS1_MEM[1029].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1030 GTM_gtm_cls1.MCS1_MEM[1030].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1031 GTM_gtm_cls1.MCS1_MEM[1031].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1032 GTM_gtm_cls1.MCS1_MEM[1032].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1033 GTM_gtm_cls1.MCS1_MEM[1033].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1034 GTM_gtm_cls1.MCS1_MEM[1034].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1035 GTM_gtm_cls1.MCS1_MEM[1035].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1036 GTM_gtm_cls1.MCS1_MEM[1036].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1037 GTM_gtm_cls1.MCS1_MEM[1037].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1038 GTM_gtm_cls1.MCS1_MEM[1038].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1039 GTM_gtm_cls1.MCS1_MEM[1039].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1040 GTM_gtm_cls1.MCS1_MEM[1040].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1041 GTM_gtm_cls1.MCS1_MEM[1041].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1042 GTM_gtm_cls1.MCS1_MEM[1042].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1043 GTM_gtm_cls1.MCS1_MEM[1043].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1044 GTM_gtm_cls1.MCS1_MEM[1044].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1045 GTM_gtm_cls1.MCS1_MEM[1045].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1046 GTM_gtm_cls1.MCS1_MEM[1046].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1047 GTM_gtm_cls1.MCS1_MEM[1047].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1048 GTM_gtm_cls1.MCS1_MEM[1048].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1049 GTM_gtm_cls1.MCS1_MEM[1049].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1050 GTM_gtm_cls1.MCS1_MEM[1050].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1051 GTM_gtm_cls1.MCS1_MEM[1051].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1052 GTM_gtm_cls1.MCS1_MEM[1052].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1053 GTM_gtm_cls1.MCS1_MEM[1053].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1054 GTM_gtm_cls1.MCS1_MEM[1054].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1055 GTM_gtm_cls1.MCS1_MEM[1055].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1056 GTM_gtm_cls1.MCS1_MEM[1056].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1057 GTM_gtm_cls1.MCS1_MEM[1057].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1058 GTM_gtm_cls1.MCS1_MEM[1058].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1059 GTM_gtm_cls1.MCS1_MEM[1059].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1060 GTM_gtm_cls1.MCS1_MEM[1060].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1061 GTM_gtm_cls1.MCS1_MEM[1061].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1062 GTM_gtm_cls1.MCS1_MEM[1062].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1063 GTM_gtm_cls1.MCS1_MEM[1063].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1064 GTM_gtm_cls1.MCS1_MEM[1064].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1065 GTM_gtm_cls1.MCS1_MEM[1065].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1066 GTM_gtm_cls1.MCS1_MEM[1066].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1067 GTM_gtm_cls1.MCS1_MEM[1067].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1068 GTM_gtm_cls1.MCS1_MEM[1068].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1069 GTM_gtm_cls1.MCS1_MEM[1069].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1070 GTM_gtm_cls1.MCS1_MEM[1070].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1071 GTM_gtm_cls1.MCS1_MEM[1071].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1072 GTM_gtm_cls1.MCS1_MEM[1072].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1073 GTM_gtm_cls1.MCS1_MEM[1073].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1074 GTM_gtm_cls1.MCS1_MEM[1074].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1075 GTM_gtm_cls1.MCS1_MEM[1075].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1076 GTM_gtm_cls1.MCS1_MEM[1076].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1077 GTM_gtm_cls1.MCS1_MEM[1077].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1078 GTM_gtm_cls1.MCS1_MEM[1078].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1079 GTM_gtm_cls1.MCS1_MEM[1079].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1080 GTM_gtm_cls1.MCS1_MEM[1080].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1081 GTM_gtm_cls1.MCS1_MEM[1081].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1082 GTM_gtm_cls1.MCS1_MEM[1082].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1083 GTM_gtm_cls1.MCS1_MEM[1083].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1084 GTM_gtm_cls1.MCS1_MEM[1084].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1085 GTM_gtm_cls1.MCS1_MEM[1085].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1086 GTM_gtm_cls1.MCS1_MEM[1086].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1087 GTM_gtm_cls1.MCS1_MEM[1087].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1088 GTM_gtm_cls1.MCS1_MEM[1088].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1089 GTM_gtm_cls1.MCS1_MEM[1089].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1090 GTM_gtm_cls1.MCS1_MEM[1090].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1091 GTM_gtm_cls1.MCS1_MEM[1091].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1092 GTM_gtm_cls1.MCS1_MEM[1092].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1093 GTM_gtm_cls1.MCS1_MEM[1093].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1094 GTM_gtm_cls1.MCS1_MEM[1094].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1095 GTM_gtm_cls1.MCS1_MEM[1095].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1096 GTM_gtm_cls1.MCS1_MEM[1096].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1097 GTM_gtm_cls1.MCS1_MEM[1097].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1098 GTM_gtm_cls1.MCS1_MEM[1098].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1099 GTM_gtm_cls1.MCS1_MEM[1099].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1100 GTM_gtm_cls1.MCS1_MEM[1100].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1101 GTM_gtm_cls1.MCS1_MEM[1101].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1102 GTM_gtm_cls1.MCS1_MEM[1102].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1103 GTM_gtm_cls1.MCS1_MEM[1103].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1104 GTM_gtm_cls1.MCS1_MEM[1104].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1105 GTM_gtm_cls1.MCS1_MEM[1105].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1106 GTM_gtm_cls1.MCS1_MEM[1106].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1107 GTM_gtm_cls1.MCS1_MEM[1107].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1108 GTM_gtm_cls1.MCS1_MEM[1108].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1109 GTM_gtm_cls1.MCS1_MEM[1109].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1110 GTM_gtm_cls1.MCS1_MEM[1110].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1111 GTM_gtm_cls1.MCS1_MEM[1111].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1112 GTM_gtm_cls1.MCS1_MEM[1112].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1113 GTM_gtm_cls1.MCS1_MEM[1113].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1114 GTM_gtm_cls1.MCS1_MEM[1114].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1115 GTM_gtm_cls1.MCS1_MEM[1115].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1116 GTM_gtm_cls1.MCS1_MEM[1116].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1117 GTM_gtm_cls1.MCS1_MEM[1117].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1118 GTM_gtm_cls1.MCS1_MEM[1118].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1119 GTM_gtm_cls1.MCS1_MEM[1119].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1120 GTM_gtm_cls1.MCS1_MEM[1120].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1121 GTM_gtm_cls1.MCS1_MEM[1121].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1122 GTM_gtm_cls1.MCS1_MEM[1122].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1123 GTM_gtm_cls1.MCS1_MEM[1123].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1124 GTM_gtm_cls1.MCS1_MEM[1124].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1125 GTM_gtm_cls1.MCS1_MEM[1125].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1126 GTM_gtm_cls1.MCS1_MEM[1126].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1127 GTM_gtm_cls1.MCS1_MEM[1127].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1128 GTM_gtm_cls1.MCS1_MEM[1128].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1129 GTM_gtm_cls1.MCS1_MEM[1129].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1130 GTM_gtm_cls1.MCS1_MEM[1130].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1131 GTM_gtm_cls1.MCS1_MEM[1131].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1132 GTM_gtm_cls1.MCS1_MEM[1132].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1133 GTM_gtm_cls1.MCS1_MEM[1133].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1134 GTM_gtm_cls1.MCS1_MEM[1134].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1135 GTM_gtm_cls1.MCS1_MEM[1135].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1136 GTM_gtm_cls1.MCS1_MEM[1136].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1137 GTM_gtm_cls1.MCS1_MEM[1137].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1138 GTM_gtm_cls1.MCS1_MEM[1138].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1139 GTM_gtm_cls1.MCS1_MEM[1139].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1140 GTM_gtm_cls1.MCS1_MEM[1140].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1141 GTM_gtm_cls1.MCS1_MEM[1141].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1142 GTM_gtm_cls1.MCS1_MEM[1142].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1143 GTM_gtm_cls1.MCS1_MEM[1143].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1144 GTM_gtm_cls1.MCS1_MEM[1144].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1145 GTM_gtm_cls1.MCS1_MEM[1145].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1146 GTM_gtm_cls1.MCS1_MEM[1146].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1147 GTM_gtm_cls1.MCS1_MEM[1147].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1148 GTM_gtm_cls1.MCS1_MEM[1148].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1149 GTM_gtm_cls1.MCS1_MEM[1149].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1150 GTM_gtm_cls1.MCS1_MEM[1150].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1151 GTM_gtm_cls1.MCS1_MEM[1151].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1152 GTM_gtm_cls1.MCS1_MEM[1152].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1153 GTM_gtm_cls1.MCS1_MEM[1153].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1154 GTM_gtm_cls1.MCS1_MEM[1154].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1155 GTM_gtm_cls1.MCS1_MEM[1155].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1156 GTM_gtm_cls1.MCS1_MEM[1156].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1157 GTM_gtm_cls1.MCS1_MEM[1157].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1158 GTM_gtm_cls1.MCS1_MEM[1158].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1159 GTM_gtm_cls1.MCS1_MEM[1159].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1160 GTM_gtm_cls1.MCS1_MEM[1160].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1161 GTM_gtm_cls1.MCS1_MEM[1161].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1162 GTM_gtm_cls1.MCS1_MEM[1162].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1163 GTM_gtm_cls1.MCS1_MEM[1163].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1164 GTM_gtm_cls1.MCS1_MEM[1164].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1165 GTM_gtm_cls1.MCS1_MEM[1165].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1166 GTM_gtm_cls1.MCS1_MEM[1166].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1167 GTM_gtm_cls1.MCS1_MEM[1167].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1168 GTM_gtm_cls1.MCS1_MEM[1168].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1169 GTM_gtm_cls1.MCS1_MEM[1169].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1170 GTM_gtm_cls1.MCS1_MEM[1170].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1171 GTM_gtm_cls1.MCS1_MEM[1171].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1172 GTM_gtm_cls1.MCS1_MEM[1172].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1173 GTM_gtm_cls1.MCS1_MEM[1173].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1174 GTM_gtm_cls1.MCS1_MEM[1174].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1175 GTM_gtm_cls1.MCS1_MEM[1175].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1176 GTM_gtm_cls1.MCS1_MEM[1176].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1177 GTM_gtm_cls1.MCS1_MEM[1177].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1178 GTM_gtm_cls1.MCS1_MEM[1178].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1179 GTM_gtm_cls1.MCS1_MEM[1179].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1180 GTM_gtm_cls1.MCS1_MEM[1180].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1181 GTM_gtm_cls1.MCS1_MEM[1181].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1182 GTM_gtm_cls1.MCS1_MEM[1182].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1183 GTM_gtm_cls1.MCS1_MEM[1183].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1184 GTM_gtm_cls1.MCS1_MEM[1184].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1185 GTM_gtm_cls1.MCS1_MEM[1185].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1186 GTM_gtm_cls1.MCS1_MEM[1186].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1187 GTM_gtm_cls1.MCS1_MEM[1187].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1188 GTM_gtm_cls1.MCS1_MEM[1188].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1189 GTM_gtm_cls1.MCS1_MEM[1189].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1190 GTM_gtm_cls1.MCS1_MEM[1190].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1191 GTM_gtm_cls1.MCS1_MEM[1191].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1192 GTM_gtm_cls1.MCS1_MEM[1192].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1193 GTM_gtm_cls1.MCS1_MEM[1193].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1194 GTM_gtm_cls1.MCS1_MEM[1194].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1195 GTM_gtm_cls1.MCS1_MEM[1195].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1196 GTM_gtm_cls1.MCS1_MEM[1196].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1197 GTM_gtm_cls1.MCS1_MEM[1197].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1198 GTM_gtm_cls1.MCS1_MEM[1198].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1199 GTM_gtm_cls1.MCS1_MEM[1199].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1200 GTM_gtm_cls1.MCS1_MEM[1200].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1201 GTM_gtm_cls1.MCS1_MEM[1201].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1202 GTM_gtm_cls1.MCS1_MEM[1202].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1203 GTM_gtm_cls1.MCS1_MEM[1203].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1204 GTM_gtm_cls1.MCS1_MEM[1204].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1205 GTM_gtm_cls1.MCS1_MEM[1205].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1206 GTM_gtm_cls1.MCS1_MEM[1206].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1207 GTM_gtm_cls1.MCS1_MEM[1207].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1208 GTM_gtm_cls1.MCS1_MEM[1208].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1209 GTM_gtm_cls1.MCS1_MEM[1209].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1210 GTM_gtm_cls1.MCS1_MEM[1210].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1211 GTM_gtm_cls1.MCS1_MEM[1211].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1212 GTM_gtm_cls1.MCS1_MEM[1212].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1213 GTM_gtm_cls1.MCS1_MEM[1213].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1214 GTM_gtm_cls1.MCS1_MEM[1214].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1215 GTM_gtm_cls1.MCS1_MEM[1215].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1216 GTM_gtm_cls1.MCS1_MEM[1216].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1217 GTM_gtm_cls1.MCS1_MEM[1217].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1218 GTM_gtm_cls1.MCS1_MEM[1218].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1219 GTM_gtm_cls1.MCS1_MEM[1219].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1220 GTM_gtm_cls1.MCS1_MEM[1220].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1221 GTM_gtm_cls1.MCS1_MEM[1221].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1222 GTM_gtm_cls1.MCS1_MEM[1222].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1223 GTM_gtm_cls1.MCS1_MEM[1223].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1224 GTM_gtm_cls1.MCS1_MEM[1224].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1225 GTM_gtm_cls1.MCS1_MEM[1225].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1226 GTM_gtm_cls1.MCS1_MEM[1226].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1227 GTM_gtm_cls1.MCS1_MEM[1227].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1228 GTM_gtm_cls1.MCS1_MEM[1228].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1229 GTM_gtm_cls1.MCS1_MEM[1229].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1230 GTM_gtm_cls1.MCS1_MEM[1230].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1231 GTM_gtm_cls1.MCS1_MEM[1231].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1232 GTM_gtm_cls1.MCS1_MEM[1232].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1233 GTM_gtm_cls1.MCS1_MEM[1233].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1234 GTM_gtm_cls1.MCS1_MEM[1234].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1235 GTM_gtm_cls1.MCS1_MEM[1235].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1236 GTM_gtm_cls1.MCS1_MEM[1236].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1237 GTM_gtm_cls1.MCS1_MEM[1237].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1238 GTM_gtm_cls1.MCS1_MEM[1238].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1239 GTM_gtm_cls1.MCS1_MEM[1239].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1240 GTM_gtm_cls1.MCS1_MEM[1240].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1241 GTM_gtm_cls1.MCS1_MEM[1241].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1242 GTM_gtm_cls1.MCS1_MEM[1242].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1243 GTM_gtm_cls1.MCS1_MEM[1243].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1244 GTM_gtm_cls1.MCS1_MEM[1244].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1245 GTM_gtm_cls1.MCS1_MEM[1245].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1246 GTM_gtm_cls1.MCS1_MEM[1246].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1247 GTM_gtm_cls1.MCS1_MEM[1247].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1248 GTM_gtm_cls1.MCS1_MEM[1248].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1249 GTM_gtm_cls1.MCS1_MEM[1249].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1250 GTM_gtm_cls1.MCS1_MEM[1250].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1251 GTM_gtm_cls1.MCS1_MEM[1251].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1252 GTM_gtm_cls1.MCS1_MEM[1252].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1253 GTM_gtm_cls1.MCS1_MEM[1253].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1254 GTM_gtm_cls1.MCS1_MEM[1254].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1255 GTM_gtm_cls1.MCS1_MEM[1255].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1256 GTM_gtm_cls1.MCS1_MEM[1256].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1257 GTM_gtm_cls1.MCS1_MEM[1257].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1258 GTM_gtm_cls1.MCS1_MEM[1258].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1259 GTM_gtm_cls1.MCS1_MEM[1259].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1260 GTM_gtm_cls1.MCS1_MEM[1260].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1261 GTM_gtm_cls1.MCS1_MEM[1261].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1262 GTM_gtm_cls1.MCS1_MEM[1262].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1263 GTM_gtm_cls1.MCS1_MEM[1263].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1264 GTM_gtm_cls1.MCS1_MEM[1264].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1265 GTM_gtm_cls1.MCS1_MEM[1265].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1266 GTM_gtm_cls1.MCS1_MEM[1266].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1267 GTM_gtm_cls1.MCS1_MEM[1267].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1268 GTM_gtm_cls1.MCS1_MEM[1268].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1269 GTM_gtm_cls1.MCS1_MEM[1269].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1270 GTM_gtm_cls1.MCS1_MEM[1270].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1271 GTM_gtm_cls1.MCS1_MEM[1271].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1272 GTM_gtm_cls1.MCS1_MEM[1272].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1273 GTM_gtm_cls1.MCS1_MEM[1273].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1274 GTM_gtm_cls1.MCS1_MEM[1274].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1275 GTM_gtm_cls1.MCS1_MEM[1275].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1276 GTM_gtm_cls1.MCS1_MEM[1276].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1277 GTM_gtm_cls1.MCS1_MEM[1277].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1278 GTM_gtm_cls1.MCS1_MEM[1278].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1279 GTM_gtm_cls1.MCS1_MEM[1279].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1280 GTM_gtm_cls1.MCS1_MEM[1280].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1281 GTM_gtm_cls1.MCS1_MEM[1281].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1282 GTM_gtm_cls1.MCS1_MEM[1282].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1283 GTM_gtm_cls1.MCS1_MEM[1283].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1284 GTM_gtm_cls1.MCS1_MEM[1284].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1285 GTM_gtm_cls1.MCS1_MEM[1285].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1286 GTM_gtm_cls1.MCS1_MEM[1286].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1287 GTM_gtm_cls1.MCS1_MEM[1287].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1288 GTM_gtm_cls1.MCS1_MEM[1288].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1289 GTM_gtm_cls1.MCS1_MEM[1289].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1290 GTM_gtm_cls1.MCS1_MEM[1290].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1291 GTM_gtm_cls1.MCS1_MEM[1291].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1292 GTM_gtm_cls1.MCS1_MEM[1292].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1293 GTM_gtm_cls1.MCS1_MEM[1293].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1294 GTM_gtm_cls1.MCS1_MEM[1294].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1295 GTM_gtm_cls1.MCS1_MEM[1295].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1296 GTM_gtm_cls1.MCS1_MEM[1296].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1297 GTM_gtm_cls1.MCS1_MEM[1297].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1298 GTM_gtm_cls1.MCS1_MEM[1298].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1299 GTM_gtm_cls1.MCS1_MEM[1299].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1300 GTM_gtm_cls1.MCS1_MEM[1300].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1301 GTM_gtm_cls1.MCS1_MEM[1301].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1302 GTM_gtm_cls1.MCS1_MEM[1302].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1303 GTM_gtm_cls1.MCS1_MEM[1303].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1304 GTM_gtm_cls1.MCS1_MEM[1304].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1305 GTM_gtm_cls1.MCS1_MEM[1305].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1306 GTM_gtm_cls1.MCS1_MEM[1306].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1307 GTM_gtm_cls1.MCS1_MEM[1307].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1308 GTM_gtm_cls1.MCS1_MEM[1308].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1309 GTM_gtm_cls1.MCS1_MEM[1309].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1310 GTM_gtm_cls1.MCS1_MEM[1310].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1311 GTM_gtm_cls1.MCS1_MEM[1311].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1312 GTM_gtm_cls1.MCS1_MEM[1312].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1313 GTM_gtm_cls1.MCS1_MEM[1313].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1314 GTM_gtm_cls1.MCS1_MEM[1314].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1315 GTM_gtm_cls1.MCS1_MEM[1315].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1316 GTM_gtm_cls1.MCS1_MEM[1316].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1317 GTM_gtm_cls1.MCS1_MEM[1317].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1318 GTM_gtm_cls1.MCS1_MEM[1318].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1319 GTM_gtm_cls1.MCS1_MEM[1319].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1320 GTM_gtm_cls1.MCS1_MEM[1320].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1321 GTM_gtm_cls1.MCS1_MEM[1321].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1322 GTM_gtm_cls1.MCS1_MEM[1322].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1323 GTM_gtm_cls1.MCS1_MEM[1323].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1324 GTM_gtm_cls1.MCS1_MEM[1324].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1325 GTM_gtm_cls1.MCS1_MEM[1325].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1326 GTM_gtm_cls1.MCS1_MEM[1326].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1327 GTM_gtm_cls1.MCS1_MEM[1327].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1328 GTM_gtm_cls1.MCS1_MEM[1328].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1329 GTM_gtm_cls1.MCS1_MEM[1329].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1330 GTM_gtm_cls1.MCS1_MEM[1330].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1331 GTM_gtm_cls1.MCS1_MEM[1331].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1332 GTM_gtm_cls1.MCS1_MEM[1332].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1333 GTM_gtm_cls1.MCS1_MEM[1333].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1334 GTM_gtm_cls1.MCS1_MEM[1334].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1335 GTM_gtm_cls1.MCS1_MEM[1335].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1336 GTM_gtm_cls1.MCS1_MEM[1336].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1337 GTM_gtm_cls1.MCS1_MEM[1337].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1338 GTM_gtm_cls1.MCS1_MEM[1338].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1339 GTM_gtm_cls1.MCS1_MEM[1339].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1340 GTM_gtm_cls1.MCS1_MEM[1340].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1341 GTM_gtm_cls1.MCS1_MEM[1341].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1342 GTM_gtm_cls1.MCS1_MEM[1342].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1343 GTM_gtm_cls1.MCS1_MEM[1343].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1344 GTM_gtm_cls1.MCS1_MEM[1344].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1345 GTM_gtm_cls1.MCS1_MEM[1345].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1346 GTM_gtm_cls1.MCS1_MEM[1346].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1347 GTM_gtm_cls1.MCS1_MEM[1347].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1348 GTM_gtm_cls1.MCS1_MEM[1348].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1349 GTM_gtm_cls1.MCS1_MEM[1349].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1350 GTM_gtm_cls1.MCS1_MEM[1350].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1351 GTM_gtm_cls1.MCS1_MEM[1351].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1352 GTM_gtm_cls1.MCS1_MEM[1352].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1353 GTM_gtm_cls1.MCS1_MEM[1353].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1354 GTM_gtm_cls1.MCS1_MEM[1354].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1355 GTM_gtm_cls1.MCS1_MEM[1355].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1356 GTM_gtm_cls1.MCS1_MEM[1356].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1357 GTM_gtm_cls1.MCS1_MEM[1357].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1358 GTM_gtm_cls1.MCS1_MEM[1358].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1359 GTM_gtm_cls1.MCS1_MEM[1359].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1360 GTM_gtm_cls1.MCS1_MEM[1360].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1361 GTM_gtm_cls1.MCS1_MEM[1361].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1362 GTM_gtm_cls1.MCS1_MEM[1362].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1363 GTM_gtm_cls1.MCS1_MEM[1363].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1364 GTM_gtm_cls1.MCS1_MEM[1364].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1365 GTM_gtm_cls1.MCS1_MEM[1365].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1366 GTM_gtm_cls1.MCS1_MEM[1366].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1367 GTM_gtm_cls1.MCS1_MEM[1367].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1368 GTM_gtm_cls1.MCS1_MEM[1368].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1369 GTM_gtm_cls1.MCS1_MEM[1369].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1370 GTM_gtm_cls1.MCS1_MEM[1370].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1371 GTM_gtm_cls1.MCS1_MEM[1371].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1372 GTM_gtm_cls1.MCS1_MEM[1372].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1373 GTM_gtm_cls1.MCS1_MEM[1373].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1374 GTM_gtm_cls1.MCS1_MEM[1374].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1375 GTM_gtm_cls1.MCS1_MEM[1375].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1376 GTM_gtm_cls1.MCS1_MEM[1376].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1377 GTM_gtm_cls1.MCS1_MEM[1377].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1378 GTM_gtm_cls1.MCS1_MEM[1378].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1379 GTM_gtm_cls1.MCS1_MEM[1379].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1380 GTM_gtm_cls1.MCS1_MEM[1380].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1381 GTM_gtm_cls1.MCS1_MEM[1381].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1382 GTM_gtm_cls1.MCS1_MEM[1382].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1383 GTM_gtm_cls1.MCS1_MEM[1383].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1384 GTM_gtm_cls1.MCS1_MEM[1384].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1385 GTM_gtm_cls1.MCS1_MEM[1385].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1386 GTM_gtm_cls1.MCS1_MEM[1386].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1387 GTM_gtm_cls1.MCS1_MEM[1387].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1388 GTM_gtm_cls1.MCS1_MEM[1388].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1389 GTM_gtm_cls1.MCS1_MEM[1389].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1390 GTM_gtm_cls1.MCS1_MEM[1390].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1391 GTM_gtm_cls1.MCS1_MEM[1391].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1392 GTM_gtm_cls1.MCS1_MEM[1392].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1393 GTM_gtm_cls1.MCS1_MEM[1393].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1394 GTM_gtm_cls1.MCS1_MEM[1394].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1395 GTM_gtm_cls1.MCS1_MEM[1395].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1396 GTM_gtm_cls1.MCS1_MEM[1396].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1397 GTM_gtm_cls1.MCS1_MEM[1397].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1398 GTM_gtm_cls1.MCS1_MEM[1398].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1399 GTM_gtm_cls1.MCS1_MEM[1399].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1400 GTM_gtm_cls1.MCS1_MEM[1400].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1401 GTM_gtm_cls1.MCS1_MEM[1401].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1402 GTM_gtm_cls1.MCS1_MEM[1402].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1403 GTM_gtm_cls1.MCS1_MEM[1403].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1404 GTM_gtm_cls1.MCS1_MEM[1404].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1405 GTM_gtm_cls1.MCS1_MEM[1405].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1406 GTM_gtm_cls1.MCS1_MEM[1406].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1407 GTM_gtm_cls1.MCS1_MEM[1407].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1408 GTM_gtm_cls1.MCS1_MEM[1408].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1409 GTM_gtm_cls1.MCS1_MEM[1409].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1410 GTM_gtm_cls1.MCS1_MEM[1410].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1411 GTM_gtm_cls1.MCS1_MEM[1411].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1412 GTM_gtm_cls1.MCS1_MEM[1412].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1413 GTM_gtm_cls1.MCS1_MEM[1413].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1414 GTM_gtm_cls1.MCS1_MEM[1414].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1415 GTM_gtm_cls1.MCS1_MEM[1415].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1416 GTM_gtm_cls1.MCS1_MEM[1416].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1417 GTM_gtm_cls1.MCS1_MEM[1417].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1418 GTM_gtm_cls1.MCS1_MEM[1418].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1419 GTM_gtm_cls1.MCS1_MEM[1419].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1420 GTM_gtm_cls1.MCS1_MEM[1420].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1421 GTM_gtm_cls1.MCS1_MEM[1421].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1422 GTM_gtm_cls1.MCS1_MEM[1422].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1423 GTM_gtm_cls1.MCS1_MEM[1423].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1424 GTM_gtm_cls1.MCS1_MEM[1424].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1425 GTM_gtm_cls1.MCS1_MEM[1425].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1426 GTM_gtm_cls1.MCS1_MEM[1426].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1427 GTM_gtm_cls1.MCS1_MEM[1427].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1428 GTM_gtm_cls1.MCS1_MEM[1428].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1429 GTM_gtm_cls1.MCS1_MEM[1429].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1430 GTM_gtm_cls1.MCS1_MEM[1430].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1431 GTM_gtm_cls1.MCS1_MEM[1431].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1432 GTM_gtm_cls1.MCS1_MEM[1432].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1433 GTM_gtm_cls1.MCS1_MEM[1433].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1434 GTM_gtm_cls1.MCS1_MEM[1434].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1435 GTM_gtm_cls1.MCS1_MEM[1435].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1436 GTM_gtm_cls1.MCS1_MEM[1436].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1437 GTM_gtm_cls1.MCS1_MEM[1437].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1438 GTM_gtm_cls1.MCS1_MEM[1438].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1439 GTM_gtm_cls1.MCS1_MEM[1439].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1440 GTM_gtm_cls1.MCS1_MEM[1440].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1441 GTM_gtm_cls1.MCS1_MEM[1441].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1442 GTM_gtm_cls1.MCS1_MEM[1442].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1443 GTM_gtm_cls1.MCS1_MEM[1443].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1444 GTM_gtm_cls1.MCS1_MEM[1444].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1445 GTM_gtm_cls1.MCS1_MEM[1445].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1446 GTM_gtm_cls1.MCS1_MEM[1446].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1447 GTM_gtm_cls1.MCS1_MEM[1447].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1448 GTM_gtm_cls1.MCS1_MEM[1448].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1449 GTM_gtm_cls1.MCS1_MEM[1449].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1450 GTM_gtm_cls1.MCS1_MEM[1450].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1451 GTM_gtm_cls1.MCS1_MEM[1451].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1452 GTM_gtm_cls1.MCS1_MEM[1452].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1453 GTM_gtm_cls1.MCS1_MEM[1453].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1454 GTM_gtm_cls1.MCS1_MEM[1454].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1455 GTM_gtm_cls1.MCS1_MEM[1455].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1456 GTM_gtm_cls1.MCS1_MEM[1456].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1457 GTM_gtm_cls1.MCS1_MEM[1457].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1458 GTM_gtm_cls1.MCS1_MEM[1458].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1459 GTM_gtm_cls1.MCS1_MEM[1459].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1460 GTM_gtm_cls1.MCS1_MEM[1460].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1461 GTM_gtm_cls1.MCS1_MEM[1461].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1462 GTM_gtm_cls1.MCS1_MEM[1462].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1463 GTM_gtm_cls1.MCS1_MEM[1463].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1464 GTM_gtm_cls1.MCS1_MEM[1464].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1465 GTM_gtm_cls1.MCS1_MEM[1465].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1466 GTM_gtm_cls1.MCS1_MEM[1466].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1467 GTM_gtm_cls1.MCS1_MEM[1467].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1468 GTM_gtm_cls1.MCS1_MEM[1468].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1469 GTM_gtm_cls1.MCS1_MEM[1469].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1470 GTM_gtm_cls1.MCS1_MEM[1470].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1471 GTM_gtm_cls1.MCS1_MEM[1471].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1472 GTM_gtm_cls1.MCS1_MEM[1472].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1473 GTM_gtm_cls1.MCS1_MEM[1473].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1474 GTM_gtm_cls1.MCS1_MEM[1474].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1475 GTM_gtm_cls1.MCS1_MEM[1475].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1476 GTM_gtm_cls1.MCS1_MEM[1476].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1477 GTM_gtm_cls1.MCS1_MEM[1477].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1478 GTM_gtm_cls1.MCS1_MEM[1478].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1479 GTM_gtm_cls1.MCS1_MEM[1479].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1480 GTM_gtm_cls1.MCS1_MEM[1480].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1481 GTM_gtm_cls1.MCS1_MEM[1481].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1482 GTM_gtm_cls1.MCS1_MEM[1482].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1483 GTM_gtm_cls1.MCS1_MEM[1483].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1484 GTM_gtm_cls1.MCS1_MEM[1484].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1485 GTM_gtm_cls1.MCS1_MEM[1485].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1486 GTM_gtm_cls1.MCS1_MEM[1486].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1487 GTM_gtm_cls1.MCS1_MEM[1487].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1488 GTM_gtm_cls1.MCS1_MEM[1488].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1489 GTM_gtm_cls1.MCS1_MEM[1489].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1490 GTM_gtm_cls1.MCS1_MEM[1490].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1491 GTM_gtm_cls1.MCS1_MEM[1491].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1492 GTM_gtm_cls1.MCS1_MEM[1492].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1493 GTM_gtm_cls1.MCS1_MEM[1493].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1494 GTM_gtm_cls1.MCS1_MEM[1494].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1495 GTM_gtm_cls1.MCS1_MEM[1495].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1496 GTM_gtm_cls1.MCS1_MEM[1496].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1497 GTM_gtm_cls1.MCS1_MEM[1497].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1498 GTM_gtm_cls1.MCS1_MEM[1498].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1499 GTM_gtm_cls1.MCS1_MEM[1499].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1500 GTM_gtm_cls1.MCS1_MEM[1500].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1501 GTM_gtm_cls1.MCS1_MEM[1501].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1502 GTM_gtm_cls1.MCS1_MEM[1502].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1503 GTM_gtm_cls1.MCS1_MEM[1503].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1504 GTM_gtm_cls1.MCS1_MEM[1504].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1505 GTM_gtm_cls1.MCS1_MEM[1505].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1506 GTM_gtm_cls1.MCS1_MEM[1506].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1507 GTM_gtm_cls1.MCS1_MEM[1507].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1508 GTM_gtm_cls1.MCS1_MEM[1508].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1509 GTM_gtm_cls1.MCS1_MEM[1509].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1510 GTM_gtm_cls1.MCS1_MEM[1510].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1511 GTM_gtm_cls1.MCS1_MEM[1511].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1512 GTM_gtm_cls1.MCS1_MEM[1512].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1513 GTM_gtm_cls1.MCS1_MEM[1513].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1514 GTM_gtm_cls1.MCS1_MEM[1514].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1515 GTM_gtm_cls1.MCS1_MEM[1515].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1516 GTM_gtm_cls1.MCS1_MEM[1516].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1517 GTM_gtm_cls1.MCS1_MEM[1517].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1518 GTM_gtm_cls1.MCS1_MEM[1518].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1519 GTM_gtm_cls1.MCS1_MEM[1519].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1520 GTM_gtm_cls1.MCS1_MEM[1520].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1521 GTM_gtm_cls1.MCS1_MEM[1521].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1522 GTM_gtm_cls1.MCS1_MEM[1522].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1523 GTM_gtm_cls1.MCS1_MEM[1523].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1524 GTM_gtm_cls1.MCS1_MEM[1524].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1525 GTM_gtm_cls1.MCS1_MEM[1525].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1526 GTM_gtm_cls1.MCS1_MEM[1526].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1527 GTM_gtm_cls1.MCS1_MEM[1527].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1528 GTM_gtm_cls1.MCS1_MEM[1528].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1529 GTM_gtm_cls1.MCS1_MEM[1529].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1530 GTM_gtm_cls1.MCS1_MEM[1530].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1531 GTM_gtm_cls1.MCS1_MEM[1531].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1532 GTM_gtm_cls1.MCS1_MEM[1532].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1533 GTM_gtm_cls1.MCS1_MEM[1533].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1534 GTM_gtm_cls1.MCS1_MEM[1534].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1535 GTM_gtm_cls1.MCS1_MEM[1535].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1536 GTM_gtm_cls1.MCS1_MEM[1536].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1537 GTM_gtm_cls1.MCS1_MEM[1537].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1538 GTM_gtm_cls1.MCS1_MEM[1538].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1539 GTM_gtm_cls1.MCS1_MEM[1539].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1540 GTM_gtm_cls1.MCS1_MEM[1540].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1541 GTM_gtm_cls1.MCS1_MEM[1541].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1542 GTM_gtm_cls1.MCS1_MEM[1542].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1543 GTM_gtm_cls1.MCS1_MEM[1543].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1544 GTM_gtm_cls1.MCS1_MEM[1544].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1545 GTM_gtm_cls1.MCS1_MEM[1545].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1546 GTM_gtm_cls1.MCS1_MEM[1546].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1547 GTM_gtm_cls1.MCS1_MEM[1547].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1548 GTM_gtm_cls1.MCS1_MEM[1548].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1549 GTM_gtm_cls1.MCS1_MEM[1549].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1550 GTM_gtm_cls1.MCS1_MEM[1550].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1551 GTM_gtm_cls1.MCS1_MEM[1551].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1552 GTM_gtm_cls1.MCS1_MEM[1552].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1553 GTM_gtm_cls1.MCS1_MEM[1553].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1554 GTM_gtm_cls1.MCS1_MEM[1554].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1555 GTM_gtm_cls1.MCS1_MEM[1555].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1556 GTM_gtm_cls1.MCS1_MEM[1556].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1557 GTM_gtm_cls1.MCS1_MEM[1557].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1558 GTM_gtm_cls1.MCS1_MEM[1558].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1559 GTM_gtm_cls1.MCS1_MEM[1559].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1560 GTM_gtm_cls1.MCS1_MEM[1560].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1561 GTM_gtm_cls1.MCS1_MEM[1561].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1562 GTM_gtm_cls1.MCS1_MEM[1562].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1563 GTM_gtm_cls1.MCS1_MEM[1563].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1564 GTM_gtm_cls1.MCS1_MEM[1564].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1565 GTM_gtm_cls1.MCS1_MEM[1565].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1566 GTM_gtm_cls1.MCS1_MEM[1566].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1567 GTM_gtm_cls1.MCS1_MEM[1567].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1568 GTM_gtm_cls1.MCS1_MEM[1568].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1569 GTM_gtm_cls1.MCS1_MEM[1569].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1570 GTM_gtm_cls1.MCS1_MEM[1570].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1571 GTM_gtm_cls1.MCS1_MEM[1571].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1572 GTM_gtm_cls1.MCS1_MEM[1572].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1573 GTM_gtm_cls1.MCS1_MEM[1573].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1574 GTM_gtm_cls1.MCS1_MEM[1574].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1575 GTM_gtm_cls1.MCS1_MEM[1575].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1576 GTM_gtm_cls1.MCS1_MEM[1576].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1577 GTM_gtm_cls1.MCS1_MEM[1577].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1578 GTM_gtm_cls1.MCS1_MEM[1578].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1579 GTM_gtm_cls1.MCS1_MEM[1579].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1580 GTM_gtm_cls1.MCS1_MEM[1580].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1581 GTM_gtm_cls1.MCS1_MEM[1581].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1582 GTM_gtm_cls1.MCS1_MEM[1582].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1583 GTM_gtm_cls1.MCS1_MEM[1583].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1584 GTM_gtm_cls1.MCS1_MEM[1584].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1585 GTM_gtm_cls1.MCS1_MEM[1585].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1586 GTM_gtm_cls1.MCS1_MEM[1586].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1587 GTM_gtm_cls1.MCS1_MEM[1587].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1588 GTM_gtm_cls1.MCS1_MEM[1588].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1589 GTM_gtm_cls1.MCS1_MEM[1589].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1590 GTM_gtm_cls1.MCS1_MEM[1590].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1591 GTM_gtm_cls1.MCS1_MEM[1591].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1592 GTM_gtm_cls1.MCS1_MEM[1592].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1593 GTM_gtm_cls1.MCS1_MEM[1593].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1594 GTM_gtm_cls1.MCS1_MEM[1594].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1595 GTM_gtm_cls1.MCS1_MEM[1595].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1596 GTM_gtm_cls1.MCS1_MEM[1596].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1597 GTM_gtm_cls1.MCS1_MEM[1597].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1598 GTM_gtm_cls1.MCS1_MEM[1598].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1599 GTM_gtm_cls1.MCS1_MEM[1599].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1600 GTM_gtm_cls1.MCS1_MEM[1600].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1601 GTM_gtm_cls1.MCS1_MEM[1601].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1602 GTM_gtm_cls1.MCS1_MEM[1602].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1603 GTM_gtm_cls1.MCS1_MEM[1603].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1604 GTM_gtm_cls1.MCS1_MEM[1604].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1605 GTM_gtm_cls1.MCS1_MEM[1605].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1606 GTM_gtm_cls1.MCS1_MEM[1606].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1607 GTM_gtm_cls1.MCS1_MEM[1607].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1608 GTM_gtm_cls1.MCS1_MEM[1608].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1609 GTM_gtm_cls1.MCS1_MEM[1609].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1610 GTM_gtm_cls1.MCS1_MEM[1610].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1611 GTM_gtm_cls1.MCS1_MEM[1611].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1612 GTM_gtm_cls1.MCS1_MEM[1612].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1613 GTM_gtm_cls1.MCS1_MEM[1613].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1614 GTM_gtm_cls1.MCS1_MEM[1614].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1615 GTM_gtm_cls1.MCS1_MEM[1615].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1616 GTM_gtm_cls1.MCS1_MEM[1616].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1617 GTM_gtm_cls1.MCS1_MEM[1617].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1618 GTM_gtm_cls1.MCS1_MEM[1618].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1619 GTM_gtm_cls1.MCS1_MEM[1619].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1620 GTM_gtm_cls1.MCS1_MEM[1620].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1621 GTM_gtm_cls1.MCS1_MEM[1621].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1622 GTM_gtm_cls1.MCS1_MEM[1622].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1623 GTM_gtm_cls1.MCS1_MEM[1623].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1624 GTM_gtm_cls1.MCS1_MEM[1624].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1625 GTM_gtm_cls1.MCS1_MEM[1625].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1626 GTM_gtm_cls1.MCS1_MEM[1626].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1627 GTM_gtm_cls1.MCS1_MEM[1627].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1628 GTM_gtm_cls1.MCS1_MEM[1628].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1629 GTM_gtm_cls1.MCS1_MEM[1629].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1630 GTM_gtm_cls1.MCS1_MEM[1630].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1631 GTM_gtm_cls1.MCS1_MEM[1631].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1632 GTM_gtm_cls1.MCS1_MEM[1632].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1633 GTM_gtm_cls1.MCS1_MEM[1633].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1634 GTM_gtm_cls1.MCS1_MEM[1634].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1635 GTM_gtm_cls1.MCS1_MEM[1635].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1636 GTM_gtm_cls1.MCS1_MEM[1636].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1637 GTM_gtm_cls1.MCS1_MEM[1637].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1638 GTM_gtm_cls1.MCS1_MEM[1638].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1639 GTM_gtm_cls1.MCS1_MEM[1639].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1640 GTM_gtm_cls1.MCS1_MEM[1640].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1641 GTM_gtm_cls1.MCS1_MEM[1641].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1642 GTM_gtm_cls1.MCS1_MEM[1642].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1643 GTM_gtm_cls1.MCS1_MEM[1643].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1644 GTM_gtm_cls1.MCS1_MEM[1644].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1645 GTM_gtm_cls1.MCS1_MEM[1645].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1646 GTM_gtm_cls1.MCS1_MEM[1646].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1647 GTM_gtm_cls1.MCS1_MEM[1647].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1648 GTM_gtm_cls1.MCS1_MEM[1648].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1649 GTM_gtm_cls1.MCS1_MEM[1649].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1650 GTM_gtm_cls1.MCS1_MEM[1650].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1651 GTM_gtm_cls1.MCS1_MEM[1651].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1652 GTM_gtm_cls1.MCS1_MEM[1652].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1653 GTM_gtm_cls1.MCS1_MEM[1653].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1654 GTM_gtm_cls1.MCS1_MEM[1654].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1655 GTM_gtm_cls1.MCS1_MEM[1655].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1656 GTM_gtm_cls1.MCS1_MEM[1656].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1657 GTM_gtm_cls1.MCS1_MEM[1657].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1658 GTM_gtm_cls1.MCS1_MEM[1658].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1659 GTM_gtm_cls1.MCS1_MEM[1659].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1660 GTM_gtm_cls1.MCS1_MEM[1660].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1661 GTM_gtm_cls1.MCS1_MEM[1661].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1662 GTM_gtm_cls1.MCS1_MEM[1662].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1663 GTM_gtm_cls1.MCS1_MEM[1663].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1664 GTM_gtm_cls1.MCS1_MEM[1664].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1665 GTM_gtm_cls1.MCS1_MEM[1665].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1666 GTM_gtm_cls1.MCS1_MEM[1666].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1667 GTM_gtm_cls1.MCS1_MEM[1667].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1668 GTM_gtm_cls1.MCS1_MEM[1668].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1669 GTM_gtm_cls1.MCS1_MEM[1669].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1670 GTM_gtm_cls1.MCS1_MEM[1670].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1671 GTM_gtm_cls1.MCS1_MEM[1671].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1672 GTM_gtm_cls1.MCS1_MEM[1672].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1673 GTM_gtm_cls1.MCS1_MEM[1673].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1674 GTM_gtm_cls1.MCS1_MEM[1674].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1675 GTM_gtm_cls1.MCS1_MEM[1675].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1676 GTM_gtm_cls1.MCS1_MEM[1676].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1677 GTM_gtm_cls1.MCS1_MEM[1677].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1678 GTM_gtm_cls1.MCS1_MEM[1678].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1679 GTM_gtm_cls1.MCS1_MEM[1679].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1680 GTM_gtm_cls1.MCS1_MEM[1680].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1681 GTM_gtm_cls1.MCS1_MEM[1681].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1682 GTM_gtm_cls1.MCS1_MEM[1682].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1683 GTM_gtm_cls1.MCS1_MEM[1683].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1684 GTM_gtm_cls1.MCS1_MEM[1684].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1685 GTM_gtm_cls1.MCS1_MEM[1685].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1686 GTM_gtm_cls1.MCS1_MEM[1686].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1687 GTM_gtm_cls1.MCS1_MEM[1687].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1688 GTM_gtm_cls1.MCS1_MEM[1688].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1689 GTM_gtm_cls1.MCS1_MEM[1689].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1690 GTM_gtm_cls1.MCS1_MEM[1690].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1691 GTM_gtm_cls1.MCS1_MEM[1691].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1692 GTM_gtm_cls1.MCS1_MEM[1692].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1693 GTM_gtm_cls1.MCS1_MEM[1693].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1694 GTM_gtm_cls1.MCS1_MEM[1694].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1695 GTM_gtm_cls1.MCS1_MEM[1695].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1696 GTM_gtm_cls1.MCS1_MEM[1696].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1697 GTM_gtm_cls1.MCS1_MEM[1697].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1698 GTM_gtm_cls1.MCS1_MEM[1698].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1699 GTM_gtm_cls1.MCS1_MEM[1699].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1700 GTM_gtm_cls1.MCS1_MEM[1700].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1701 GTM_gtm_cls1.MCS1_MEM[1701].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1702 GTM_gtm_cls1.MCS1_MEM[1702].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1703 GTM_gtm_cls1.MCS1_MEM[1703].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1704 GTM_gtm_cls1.MCS1_MEM[1704].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1705 GTM_gtm_cls1.MCS1_MEM[1705].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1706 GTM_gtm_cls1.MCS1_MEM[1706].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1707 GTM_gtm_cls1.MCS1_MEM[1707].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1708 GTM_gtm_cls1.MCS1_MEM[1708].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1709 GTM_gtm_cls1.MCS1_MEM[1709].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1710 GTM_gtm_cls1.MCS1_MEM[1710].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1711 GTM_gtm_cls1.MCS1_MEM[1711].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1712 GTM_gtm_cls1.MCS1_MEM[1712].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1713 GTM_gtm_cls1.MCS1_MEM[1713].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1714 GTM_gtm_cls1.MCS1_MEM[1714].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1715 GTM_gtm_cls1.MCS1_MEM[1715].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1716 GTM_gtm_cls1.MCS1_MEM[1716].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1717 GTM_gtm_cls1.MCS1_MEM[1717].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1718 GTM_gtm_cls1.MCS1_MEM[1718].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1719 GTM_gtm_cls1.MCS1_MEM[1719].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1720 GTM_gtm_cls1.MCS1_MEM[1720].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1721 GTM_gtm_cls1.MCS1_MEM[1721].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1722 GTM_gtm_cls1.MCS1_MEM[1722].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1723 GTM_gtm_cls1.MCS1_MEM[1723].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1724 GTM_gtm_cls1.MCS1_MEM[1724].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1725 GTM_gtm_cls1.MCS1_MEM[1725].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1726 GTM_gtm_cls1.MCS1_MEM[1726].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1727 GTM_gtm_cls1.MCS1_MEM[1727].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1728 GTM_gtm_cls1.MCS1_MEM[1728].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1729 GTM_gtm_cls1.MCS1_MEM[1729].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1730 GTM_gtm_cls1.MCS1_MEM[1730].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1731 GTM_gtm_cls1.MCS1_MEM[1731].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1732 GTM_gtm_cls1.MCS1_MEM[1732].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1733 GTM_gtm_cls1.MCS1_MEM[1733].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1734 GTM_gtm_cls1.MCS1_MEM[1734].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1735 GTM_gtm_cls1.MCS1_MEM[1735].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1736 GTM_gtm_cls1.MCS1_MEM[1736].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1737 GTM_gtm_cls1.MCS1_MEM[1737].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1738 GTM_gtm_cls1.MCS1_MEM[1738].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1739 GTM_gtm_cls1.MCS1_MEM[1739].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1740 GTM_gtm_cls1.MCS1_MEM[1740].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1741 GTM_gtm_cls1.MCS1_MEM[1741].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1742 GTM_gtm_cls1.MCS1_MEM[1742].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1743 GTM_gtm_cls1.MCS1_MEM[1743].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1744 GTM_gtm_cls1.MCS1_MEM[1744].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1745 GTM_gtm_cls1.MCS1_MEM[1745].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1746 GTM_gtm_cls1.MCS1_MEM[1746].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1747 GTM_gtm_cls1.MCS1_MEM[1747].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1748 GTM_gtm_cls1.MCS1_MEM[1748].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1749 GTM_gtm_cls1.MCS1_MEM[1749].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1750 GTM_gtm_cls1.MCS1_MEM[1750].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1751 GTM_gtm_cls1.MCS1_MEM[1751].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1752 GTM_gtm_cls1.MCS1_MEM[1752].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1753 GTM_gtm_cls1.MCS1_MEM[1753].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1754 GTM_gtm_cls1.MCS1_MEM[1754].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1755 GTM_gtm_cls1.MCS1_MEM[1755].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1756 GTM_gtm_cls1.MCS1_MEM[1756].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1757 GTM_gtm_cls1.MCS1_MEM[1757].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1758 GTM_gtm_cls1.MCS1_MEM[1758].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1759 GTM_gtm_cls1.MCS1_MEM[1759].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1760 GTM_gtm_cls1.MCS1_MEM[1760].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1761 GTM_gtm_cls1.MCS1_MEM[1761].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1762 GTM_gtm_cls1.MCS1_MEM[1762].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1763 GTM_gtm_cls1.MCS1_MEM[1763].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1764 GTM_gtm_cls1.MCS1_MEM[1764].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1765 GTM_gtm_cls1.MCS1_MEM[1765].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1766 GTM_gtm_cls1.MCS1_MEM[1766].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1767 GTM_gtm_cls1.MCS1_MEM[1767].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1768 GTM_gtm_cls1.MCS1_MEM[1768].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1769 GTM_gtm_cls1.MCS1_MEM[1769].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1770 GTM_gtm_cls1.MCS1_MEM[1770].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1771 GTM_gtm_cls1.MCS1_MEM[1771].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1772 GTM_gtm_cls1.MCS1_MEM[1772].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1773 GTM_gtm_cls1.MCS1_MEM[1773].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1774 GTM_gtm_cls1.MCS1_MEM[1774].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1775 GTM_gtm_cls1.MCS1_MEM[1775].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1776 GTM_gtm_cls1.MCS1_MEM[1776].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1777 GTM_gtm_cls1.MCS1_MEM[1777].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1778 GTM_gtm_cls1.MCS1_MEM[1778].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1779 GTM_gtm_cls1.MCS1_MEM[1779].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1780 GTM_gtm_cls1.MCS1_MEM[1780].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1781 GTM_gtm_cls1.MCS1_MEM[1781].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1782 GTM_gtm_cls1.MCS1_MEM[1782].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1783 GTM_gtm_cls1.MCS1_MEM[1783].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1784 GTM_gtm_cls1.MCS1_MEM[1784].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1785 GTM_gtm_cls1.MCS1_MEM[1785].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1786 GTM_gtm_cls1.MCS1_MEM[1786].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1787 GTM_gtm_cls1.MCS1_MEM[1787].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1788 GTM_gtm_cls1.MCS1_MEM[1788].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1789 GTM_gtm_cls1.MCS1_MEM[1789].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1790 GTM_gtm_cls1.MCS1_MEM[1790].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1791 GTM_gtm_cls1.MCS1_MEM[1791].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1792 GTM_gtm_cls1.MCS1_MEM[1792].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1793 GTM_gtm_cls1.MCS1_MEM[1793].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1794 GTM_gtm_cls1.MCS1_MEM[1794].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1795 GTM_gtm_cls1.MCS1_MEM[1795].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1796 GTM_gtm_cls1.MCS1_MEM[1796].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1797 GTM_gtm_cls1.MCS1_MEM[1797].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1798 GTM_gtm_cls1.MCS1_MEM[1798].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1799 GTM_gtm_cls1.MCS1_MEM[1799].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1800 GTM_gtm_cls1.MCS1_MEM[1800].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1801 GTM_gtm_cls1.MCS1_MEM[1801].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1802 GTM_gtm_cls1.MCS1_MEM[1802].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1803 GTM_gtm_cls1.MCS1_MEM[1803].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1804 GTM_gtm_cls1.MCS1_MEM[1804].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1805 GTM_gtm_cls1.MCS1_MEM[1805].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1806 GTM_gtm_cls1.MCS1_MEM[1806].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1807 GTM_gtm_cls1.MCS1_MEM[1807].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1808 GTM_gtm_cls1.MCS1_MEM[1808].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1809 GTM_gtm_cls1.MCS1_MEM[1809].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1810 GTM_gtm_cls1.MCS1_MEM[1810].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1811 GTM_gtm_cls1.MCS1_MEM[1811].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1812 GTM_gtm_cls1.MCS1_MEM[1812].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1813 GTM_gtm_cls1.MCS1_MEM[1813].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1814 GTM_gtm_cls1.MCS1_MEM[1814].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1815 GTM_gtm_cls1.MCS1_MEM[1815].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1816 GTM_gtm_cls1.MCS1_MEM[1816].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1817 GTM_gtm_cls1.MCS1_MEM[1817].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1818 GTM_gtm_cls1.MCS1_MEM[1818].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1819 GTM_gtm_cls1.MCS1_MEM[1819].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1820 GTM_gtm_cls1.MCS1_MEM[1820].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1821 GTM_gtm_cls1.MCS1_MEM[1821].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1822 GTM_gtm_cls1.MCS1_MEM[1822].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1823 GTM_gtm_cls1.MCS1_MEM[1823].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1824 GTM_gtm_cls1.MCS1_MEM[1824].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1825 GTM_gtm_cls1.MCS1_MEM[1825].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1826 GTM_gtm_cls1.MCS1_MEM[1826].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1827 GTM_gtm_cls1.MCS1_MEM[1827].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1828 GTM_gtm_cls1.MCS1_MEM[1828].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1829 GTM_gtm_cls1.MCS1_MEM[1829].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1830 GTM_gtm_cls1.MCS1_MEM[1830].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1831 GTM_gtm_cls1.MCS1_MEM[1831].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1832 GTM_gtm_cls1.MCS1_MEM[1832].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1833 GTM_gtm_cls1.MCS1_MEM[1833].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1834 GTM_gtm_cls1.MCS1_MEM[1834].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1835 GTM_gtm_cls1.MCS1_MEM[1835].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1836 GTM_gtm_cls1.MCS1_MEM[1836].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1837 GTM_gtm_cls1.MCS1_MEM[1837].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1838 GTM_gtm_cls1.MCS1_MEM[1838].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1839 GTM_gtm_cls1.MCS1_MEM[1839].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1840 GTM_gtm_cls1.MCS1_MEM[1840].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1841 GTM_gtm_cls1.MCS1_MEM[1841].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1842 GTM_gtm_cls1.MCS1_MEM[1842].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1843 GTM_gtm_cls1.MCS1_MEM[1843].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1844 GTM_gtm_cls1.MCS1_MEM[1844].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1845 GTM_gtm_cls1.MCS1_MEM[1845].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1846 GTM_gtm_cls1.MCS1_MEM[1846].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1847 GTM_gtm_cls1.MCS1_MEM[1847].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1848 GTM_gtm_cls1.MCS1_MEM[1848].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1849 GTM_gtm_cls1.MCS1_MEM[1849].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1850 GTM_gtm_cls1.MCS1_MEM[1850].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1851 GTM_gtm_cls1.MCS1_MEM[1851].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1852 GTM_gtm_cls1.MCS1_MEM[1852].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1853 GTM_gtm_cls1.MCS1_MEM[1853].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1854 GTM_gtm_cls1.MCS1_MEM[1854].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1855 GTM_gtm_cls1.MCS1_MEM[1855].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1856 GTM_gtm_cls1.MCS1_MEM[1856].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1857 GTM_gtm_cls1.MCS1_MEM[1857].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1858 GTM_gtm_cls1.MCS1_MEM[1858].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1859 GTM_gtm_cls1.MCS1_MEM[1859].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1860 GTM_gtm_cls1.MCS1_MEM[1860].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1861 GTM_gtm_cls1.MCS1_MEM[1861].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1862 GTM_gtm_cls1.MCS1_MEM[1862].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1863 GTM_gtm_cls1.MCS1_MEM[1863].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1864 GTM_gtm_cls1.MCS1_MEM[1864].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1865 GTM_gtm_cls1.MCS1_MEM[1865].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1866 GTM_gtm_cls1.MCS1_MEM[1866].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1867 GTM_gtm_cls1.MCS1_MEM[1867].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1868 GTM_gtm_cls1.MCS1_MEM[1868].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1869 GTM_gtm_cls1.MCS1_MEM[1869].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1870 GTM_gtm_cls1.MCS1_MEM[1870].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1871 GTM_gtm_cls1.MCS1_MEM[1871].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1872 GTM_gtm_cls1.MCS1_MEM[1872].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1873 GTM_gtm_cls1.MCS1_MEM[1873].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1874 GTM_gtm_cls1.MCS1_MEM[1874].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1875 GTM_gtm_cls1.MCS1_MEM[1875].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1876 GTM_gtm_cls1.MCS1_MEM[1876].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1877 GTM_gtm_cls1.MCS1_MEM[1877].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1878 GTM_gtm_cls1.MCS1_MEM[1878].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1879 GTM_gtm_cls1.MCS1_MEM[1879].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1880 GTM_gtm_cls1.MCS1_MEM[1880].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1881 GTM_gtm_cls1.MCS1_MEM[1881].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1882 GTM_gtm_cls1.MCS1_MEM[1882].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1883 GTM_gtm_cls1.MCS1_MEM[1883].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1884 GTM_gtm_cls1.MCS1_MEM[1884].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1885 GTM_gtm_cls1.MCS1_MEM[1885].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1886 GTM_gtm_cls1.MCS1_MEM[1886].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1887 GTM_gtm_cls1.MCS1_MEM[1887].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1888 GTM_gtm_cls1.MCS1_MEM[1888].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1889 GTM_gtm_cls1.MCS1_MEM[1889].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1890 GTM_gtm_cls1.MCS1_MEM[1890].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1891 GTM_gtm_cls1.MCS1_MEM[1891].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1892 GTM_gtm_cls1.MCS1_MEM[1892].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1893 GTM_gtm_cls1.MCS1_MEM[1893].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1894 GTM_gtm_cls1.MCS1_MEM[1894].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1895 GTM_gtm_cls1.MCS1_MEM[1895].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1896 GTM_gtm_cls1.MCS1_MEM[1896].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1897 GTM_gtm_cls1.MCS1_MEM[1897].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1898 GTM_gtm_cls1.MCS1_MEM[1898].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1899 GTM_gtm_cls1.MCS1_MEM[1899].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1900 GTM_gtm_cls1.MCS1_MEM[1900].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1901 GTM_gtm_cls1.MCS1_MEM[1901].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1902 GTM_gtm_cls1.MCS1_MEM[1902].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1903 GTM_gtm_cls1.MCS1_MEM[1903].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1904 GTM_gtm_cls1.MCS1_MEM[1904].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1905 GTM_gtm_cls1.MCS1_MEM[1905].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1906 GTM_gtm_cls1.MCS1_MEM[1906].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1907 GTM_gtm_cls1.MCS1_MEM[1907].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1908 GTM_gtm_cls1.MCS1_MEM[1908].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1909 GTM_gtm_cls1.MCS1_MEM[1909].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1910 GTM_gtm_cls1.MCS1_MEM[1910].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1911 GTM_gtm_cls1.MCS1_MEM[1911].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1912 GTM_gtm_cls1.MCS1_MEM[1912].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1913 GTM_gtm_cls1.MCS1_MEM[1913].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1914 GTM_gtm_cls1.MCS1_MEM[1914].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1915 GTM_gtm_cls1.MCS1_MEM[1915].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1916 GTM_gtm_cls1.MCS1_MEM[1916].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1917 GTM_gtm_cls1.MCS1_MEM[1917].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1918 GTM_gtm_cls1.MCS1_MEM[1918].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1919 GTM_gtm_cls1.MCS1_MEM[1919].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1920 GTM_gtm_cls1.MCS1_MEM[1920].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1921 GTM_gtm_cls1.MCS1_MEM[1921].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1922 GTM_gtm_cls1.MCS1_MEM[1922].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1923 GTM_gtm_cls1.MCS1_MEM[1923].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1924 GTM_gtm_cls1.MCS1_MEM[1924].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1925 GTM_gtm_cls1.MCS1_MEM[1925].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1926 GTM_gtm_cls1.MCS1_MEM[1926].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1927 GTM_gtm_cls1.MCS1_MEM[1927].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1928 GTM_gtm_cls1.MCS1_MEM[1928].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1929 GTM_gtm_cls1.MCS1_MEM[1929].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1930 GTM_gtm_cls1.MCS1_MEM[1930].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1931 GTM_gtm_cls1.MCS1_MEM[1931].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1932 GTM_gtm_cls1.MCS1_MEM[1932].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1933 GTM_gtm_cls1.MCS1_MEM[1933].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1934 GTM_gtm_cls1.MCS1_MEM[1934].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1935 GTM_gtm_cls1.MCS1_MEM[1935].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1936 GTM_gtm_cls1.MCS1_MEM[1936].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1937 GTM_gtm_cls1.MCS1_MEM[1937].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1938 GTM_gtm_cls1.MCS1_MEM[1938].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1939 GTM_gtm_cls1.MCS1_MEM[1939].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1940 GTM_gtm_cls1.MCS1_MEM[1940].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1941 GTM_gtm_cls1.MCS1_MEM[1941].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1942 GTM_gtm_cls1.MCS1_MEM[1942].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1943 GTM_gtm_cls1.MCS1_MEM[1943].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1944 GTM_gtm_cls1.MCS1_MEM[1944].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1945 GTM_gtm_cls1.MCS1_MEM[1945].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1946 GTM_gtm_cls1.MCS1_MEM[1946].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1947 GTM_gtm_cls1.MCS1_MEM[1947].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1948 GTM_gtm_cls1.MCS1_MEM[1948].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1949 GTM_gtm_cls1.MCS1_MEM[1949].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1950 GTM_gtm_cls1.MCS1_MEM[1950].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1951 GTM_gtm_cls1.MCS1_MEM[1951].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1952 GTM_gtm_cls1.MCS1_MEM[1952].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1953 GTM_gtm_cls1.MCS1_MEM[1953].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1954 GTM_gtm_cls1.MCS1_MEM[1954].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1955 GTM_gtm_cls1.MCS1_MEM[1955].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1956 GTM_gtm_cls1.MCS1_MEM[1956].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1957 GTM_gtm_cls1.MCS1_MEM[1957].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1958 GTM_gtm_cls1.MCS1_MEM[1958].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1959 GTM_gtm_cls1.MCS1_MEM[1959].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1960 GTM_gtm_cls1.MCS1_MEM[1960].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1961 GTM_gtm_cls1.MCS1_MEM[1961].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1962 GTM_gtm_cls1.MCS1_MEM[1962].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1963 GTM_gtm_cls1.MCS1_MEM[1963].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1964 GTM_gtm_cls1.MCS1_MEM[1964].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1965 GTM_gtm_cls1.MCS1_MEM[1965].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1966 GTM_gtm_cls1.MCS1_MEM[1966].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1967 GTM_gtm_cls1.MCS1_MEM[1967].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1968 GTM_gtm_cls1.MCS1_MEM[1968].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1969 GTM_gtm_cls1.MCS1_MEM[1969].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1970 GTM_gtm_cls1.MCS1_MEM[1970].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1971 GTM_gtm_cls1.MCS1_MEM[1971].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1972 GTM_gtm_cls1.MCS1_MEM[1972].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1973 GTM_gtm_cls1.MCS1_MEM[1973].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1974 GTM_gtm_cls1.MCS1_MEM[1974].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1975 GTM_gtm_cls1.MCS1_MEM[1975].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1976 GTM_gtm_cls1.MCS1_MEM[1976].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1977 GTM_gtm_cls1.MCS1_MEM[1977].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1978 GTM_gtm_cls1.MCS1_MEM[1978].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1979 GTM_gtm_cls1.MCS1_MEM[1979].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1980 GTM_gtm_cls1.MCS1_MEM[1980].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1981 GTM_gtm_cls1.MCS1_MEM[1981].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1982 GTM_gtm_cls1.MCS1_MEM[1982].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1983 GTM_gtm_cls1.MCS1_MEM[1983].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1984 GTM_gtm_cls1.MCS1_MEM[1984].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1985 GTM_gtm_cls1.MCS1_MEM[1985].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1986 GTM_gtm_cls1.MCS1_MEM[1986].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1987 GTM_gtm_cls1.MCS1_MEM[1987].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1988 GTM_gtm_cls1.MCS1_MEM[1988].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1989 GTM_gtm_cls1.MCS1_MEM[1989].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1990 GTM_gtm_cls1.MCS1_MEM[1990].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1991 GTM_gtm_cls1.MCS1_MEM[1991].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1992 GTM_gtm_cls1.MCS1_MEM[1992].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1993 GTM_gtm_cls1.MCS1_MEM[1993].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1994 GTM_gtm_cls1.MCS1_MEM[1994].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1995 GTM_gtm_cls1.MCS1_MEM[1995].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1996 GTM_gtm_cls1.MCS1_MEM[1996].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1997 GTM_gtm_cls1.MCS1_MEM[1997].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1998 GTM_gtm_cls1.MCS1_MEM[1998].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM1999 GTM_gtm_cls1.MCS1_MEM[1999].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2000 GTM_gtm_cls1.MCS1_MEM[2000].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2001 GTM_gtm_cls1.MCS1_MEM[2001].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2002 GTM_gtm_cls1.MCS1_MEM[2002].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2003 GTM_gtm_cls1.MCS1_MEM[2003].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2004 GTM_gtm_cls1.MCS1_MEM[2004].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2005 GTM_gtm_cls1.MCS1_MEM[2005].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2006 GTM_gtm_cls1.MCS1_MEM[2006].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2007 GTM_gtm_cls1.MCS1_MEM[2007].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2008 GTM_gtm_cls1.MCS1_MEM[2008].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2009 GTM_gtm_cls1.MCS1_MEM[2009].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2010 GTM_gtm_cls1.MCS1_MEM[2010].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2011 GTM_gtm_cls1.MCS1_MEM[2011].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2012 GTM_gtm_cls1.MCS1_MEM[2012].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2013 GTM_gtm_cls1.MCS1_MEM[2013].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2014 GTM_gtm_cls1.MCS1_MEM[2014].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2015 GTM_gtm_cls1.MCS1_MEM[2015].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2016 GTM_gtm_cls1.MCS1_MEM[2016].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2017 GTM_gtm_cls1.MCS1_MEM[2017].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2018 GTM_gtm_cls1.MCS1_MEM[2018].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2019 GTM_gtm_cls1.MCS1_MEM[2019].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2020 GTM_gtm_cls1.MCS1_MEM[2020].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2021 GTM_gtm_cls1.MCS1_MEM[2021].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2022 GTM_gtm_cls1.MCS1_MEM[2022].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2023 GTM_gtm_cls1.MCS1_MEM[2023].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2024 GTM_gtm_cls1.MCS1_MEM[2024].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2025 GTM_gtm_cls1.MCS1_MEM[2025].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2026 GTM_gtm_cls1.MCS1_MEM[2026].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2027 GTM_gtm_cls1.MCS1_MEM[2027].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2028 GTM_gtm_cls1.MCS1_MEM[2028].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2029 GTM_gtm_cls1.MCS1_MEM[2029].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2030 GTM_gtm_cls1.MCS1_MEM[2030].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2031 GTM_gtm_cls1.MCS1_MEM[2031].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2032 GTM_gtm_cls1.MCS1_MEM[2032].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2033 GTM_gtm_cls1.MCS1_MEM[2033].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2034 GTM_gtm_cls1.MCS1_MEM[2034].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2035 GTM_gtm_cls1.MCS1_MEM[2035].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2036 GTM_gtm_cls1.MCS1_MEM[2036].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2037 GTM_gtm_cls1.MCS1_MEM[2037].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2038 GTM_gtm_cls1.MCS1_MEM[2038].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2039 GTM_gtm_cls1.MCS1_MEM[2039].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2040 GTM_gtm_cls1.MCS1_MEM[2040].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2041 GTM_gtm_cls1.MCS1_MEM[2041].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2042 GTM_gtm_cls1.MCS1_MEM[2042].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2043 GTM_gtm_cls1.MCS1_MEM[2043].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2044 GTM_gtm_cls1.MCS1_MEM[2044].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2045 GTM_gtm_cls1.MCS1_MEM[2045].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2046 GTM_gtm_cls1.MCS1_MEM[2046].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2047 GTM_gtm_cls1.MCS1_MEM[2047].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2048 GTM_gtm_cls1.MCS1_MEM[2048].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2049 GTM_gtm_cls1.MCS1_MEM[2049].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2050 GTM_gtm_cls1.MCS1_MEM[2050].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2051 GTM_gtm_cls1.MCS1_MEM[2051].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2052 GTM_gtm_cls1.MCS1_MEM[2052].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2053 GTM_gtm_cls1.MCS1_MEM[2053].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2054 GTM_gtm_cls1.MCS1_MEM[2054].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2055 GTM_gtm_cls1.MCS1_MEM[2055].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2056 GTM_gtm_cls1.MCS1_MEM[2056].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2057 GTM_gtm_cls1.MCS1_MEM[2057].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2058 GTM_gtm_cls1.MCS1_MEM[2058].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2059 GTM_gtm_cls1.MCS1_MEM[2059].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2060 GTM_gtm_cls1.MCS1_MEM[2060].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2061 GTM_gtm_cls1.MCS1_MEM[2061].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2062 GTM_gtm_cls1.MCS1_MEM[2062].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2063 GTM_gtm_cls1.MCS1_MEM[2063].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2064 GTM_gtm_cls1.MCS1_MEM[2064].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2065 GTM_gtm_cls1.MCS1_MEM[2065].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2066 GTM_gtm_cls1.MCS1_MEM[2066].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2067 GTM_gtm_cls1.MCS1_MEM[2067].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2068 GTM_gtm_cls1.MCS1_MEM[2068].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2069 GTM_gtm_cls1.MCS1_MEM[2069].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2070 GTM_gtm_cls1.MCS1_MEM[2070].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2071 GTM_gtm_cls1.MCS1_MEM[2071].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2072 GTM_gtm_cls1.MCS1_MEM[2072].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2073 GTM_gtm_cls1.MCS1_MEM[2073].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2074 GTM_gtm_cls1.MCS1_MEM[2074].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2075 GTM_gtm_cls1.MCS1_MEM[2075].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2076 GTM_gtm_cls1.MCS1_MEM[2076].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2077 GTM_gtm_cls1.MCS1_MEM[2077].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2078 GTM_gtm_cls1.MCS1_MEM[2078].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2079 GTM_gtm_cls1.MCS1_MEM[2079].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2080 GTM_gtm_cls1.MCS1_MEM[2080].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2081 GTM_gtm_cls1.MCS1_MEM[2081].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2082 GTM_gtm_cls1.MCS1_MEM[2082].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2083 GTM_gtm_cls1.MCS1_MEM[2083].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2084 GTM_gtm_cls1.MCS1_MEM[2084].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2085 GTM_gtm_cls1.MCS1_MEM[2085].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2086 GTM_gtm_cls1.MCS1_MEM[2086].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2087 GTM_gtm_cls1.MCS1_MEM[2087].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2088 GTM_gtm_cls1.MCS1_MEM[2088].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2089 GTM_gtm_cls1.MCS1_MEM[2089].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2090 GTM_gtm_cls1.MCS1_MEM[2090].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2091 GTM_gtm_cls1.MCS1_MEM[2091].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2092 GTM_gtm_cls1.MCS1_MEM[2092].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2093 GTM_gtm_cls1.MCS1_MEM[2093].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2094 GTM_gtm_cls1.MCS1_MEM[2094].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2095 GTM_gtm_cls1.MCS1_MEM[2095].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2096 GTM_gtm_cls1.MCS1_MEM[2096].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2097 GTM_gtm_cls1.MCS1_MEM[2097].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2098 GTM_gtm_cls1.MCS1_MEM[2098].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2099 GTM_gtm_cls1.MCS1_MEM[2099].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2100 GTM_gtm_cls1.MCS1_MEM[2100].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2101 GTM_gtm_cls1.MCS1_MEM[2101].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2102 GTM_gtm_cls1.MCS1_MEM[2102].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2103 GTM_gtm_cls1.MCS1_MEM[2103].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2104 GTM_gtm_cls1.MCS1_MEM[2104].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2105 GTM_gtm_cls1.MCS1_MEM[2105].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2106 GTM_gtm_cls1.MCS1_MEM[2106].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2107 GTM_gtm_cls1.MCS1_MEM[2107].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2108 GTM_gtm_cls1.MCS1_MEM[2108].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2109 GTM_gtm_cls1.MCS1_MEM[2109].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2110 GTM_gtm_cls1.MCS1_MEM[2110].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2111 GTM_gtm_cls1.MCS1_MEM[2111].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2112 GTM_gtm_cls1.MCS1_MEM[2112].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2113 GTM_gtm_cls1.MCS1_MEM[2113].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2114 GTM_gtm_cls1.MCS1_MEM[2114].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2115 GTM_gtm_cls1.MCS1_MEM[2115].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2116 GTM_gtm_cls1.MCS1_MEM[2116].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2117 GTM_gtm_cls1.MCS1_MEM[2117].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2118 GTM_gtm_cls1.MCS1_MEM[2118].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2119 GTM_gtm_cls1.MCS1_MEM[2119].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2120 GTM_gtm_cls1.MCS1_MEM[2120].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2121 GTM_gtm_cls1.MCS1_MEM[2121].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2122 GTM_gtm_cls1.MCS1_MEM[2122].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2123 GTM_gtm_cls1.MCS1_MEM[2123].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2124 GTM_gtm_cls1.MCS1_MEM[2124].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2125 GTM_gtm_cls1.MCS1_MEM[2125].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2126 GTM_gtm_cls1.MCS1_MEM[2126].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2127 GTM_gtm_cls1.MCS1_MEM[2127].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2128 GTM_gtm_cls1.MCS1_MEM[2128].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2129 GTM_gtm_cls1.MCS1_MEM[2129].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2130 GTM_gtm_cls1.MCS1_MEM[2130].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2131 GTM_gtm_cls1.MCS1_MEM[2131].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2132 GTM_gtm_cls1.MCS1_MEM[2132].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2133 GTM_gtm_cls1.MCS1_MEM[2133].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2134 GTM_gtm_cls1.MCS1_MEM[2134].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2135 GTM_gtm_cls1.MCS1_MEM[2135].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2136 GTM_gtm_cls1.MCS1_MEM[2136].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2137 GTM_gtm_cls1.MCS1_MEM[2137].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2138 GTM_gtm_cls1.MCS1_MEM[2138].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2139 GTM_gtm_cls1.MCS1_MEM[2139].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2140 GTM_gtm_cls1.MCS1_MEM[2140].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2141 GTM_gtm_cls1.MCS1_MEM[2141].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2142 GTM_gtm_cls1.MCS1_MEM[2142].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2143 GTM_gtm_cls1.MCS1_MEM[2143].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2144 GTM_gtm_cls1.MCS1_MEM[2144].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2145 GTM_gtm_cls1.MCS1_MEM[2145].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2146 GTM_gtm_cls1.MCS1_MEM[2146].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2147 GTM_gtm_cls1.MCS1_MEM[2147].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2148 GTM_gtm_cls1.MCS1_MEM[2148].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2149 GTM_gtm_cls1.MCS1_MEM[2149].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2150 GTM_gtm_cls1.MCS1_MEM[2150].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2151 GTM_gtm_cls1.MCS1_MEM[2151].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2152 GTM_gtm_cls1.MCS1_MEM[2152].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2153 GTM_gtm_cls1.MCS1_MEM[2153].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2154 GTM_gtm_cls1.MCS1_MEM[2154].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2155 GTM_gtm_cls1.MCS1_MEM[2155].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2156 GTM_gtm_cls1.MCS1_MEM[2156].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2157 GTM_gtm_cls1.MCS1_MEM[2157].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2158 GTM_gtm_cls1.MCS1_MEM[2158].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2159 GTM_gtm_cls1.MCS1_MEM[2159].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2160 GTM_gtm_cls1.MCS1_MEM[2160].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2161 GTM_gtm_cls1.MCS1_MEM[2161].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2162 GTM_gtm_cls1.MCS1_MEM[2162].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2163 GTM_gtm_cls1.MCS1_MEM[2163].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2164 GTM_gtm_cls1.MCS1_MEM[2164].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2165 GTM_gtm_cls1.MCS1_MEM[2165].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2166 GTM_gtm_cls1.MCS1_MEM[2166].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2167 GTM_gtm_cls1.MCS1_MEM[2167].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2168 GTM_gtm_cls1.MCS1_MEM[2168].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2169 GTM_gtm_cls1.MCS1_MEM[2169].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2170 GTM_gtm_cls1.MCS1_MEM[2170].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2171 GTM_gtm_cls1.MCS1_MEM[2171].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2172 GTM_gtm_cls1.MCS1_MEM[2172].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2173 GTM_gtm_cls1.MCS1_MEM[2173].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2174 GTM_gtm_cls1.MCS1_MEM[2174].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2175 GTM_gtm_cls1.MCS1_MEM[2175].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2176 GTM_gtm_cls1.MCS1_MEM[2176].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2177 GTM_gtm_cls1.MCS1_MEM[2177].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2178 GTM_gtm_cls1.MCS1_MEM[2178].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2179 GTM_gtm_cls1.MCS1_MEM[2179].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2180 GTM_gtm_cls1.MCS1_MEM[2180].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2181 GTM_gtm_cls1.MCS1_MEM[2181].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2182 GTM_gtm_cls1.MCS1_MEM[2182].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2183 GTM_gtm_cls1.MCS1_MEM[2183].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2184 GTM_gtm_cls1.MCS1_MEM[2184].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2185 GTM_gtm_cls1.MCS1_MEM[2185].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2186 GTM_gtm_cls1.MCS1_MEM[2186].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2187 GTM_gtm_cls1.MCS1_MEM[2187].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2188 GTM_gtm_cls1.MCS1_MEM[2188].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2189 GTM_gtm_cls1.MCS1_MEM[2189].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2190 GTM_gtm_cls1.MCS1_MEM[2190].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2191 GTM_gtm_cls1.MCS1_MEM[2191].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2192 GTM_gtm_cls1.MCS1_MEM[2192].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2193 GTM_gtm_cls1.MCS1_MEM[2193].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2194 GTM_gtm_cls1.MCS1_MEM[2194].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2195 GTM_gtm_cls1.MCS1_MEM[2195].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2196 GTM_gtm_cls1.MCS1_MEM[2196].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2197 GTM_gtm_cls1.MCS1_MEM[2197].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2198 GTM_gtm_cls1.MCS1_MEM[2198].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2199 GTM_gtm_cls1.MCS1_MEM[2199].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2200 GTM_gtm_cls1.MCS1_MEM[2200].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2201 GTM_gtm_cls1.MCS1_MEM[2201].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2202 GTM_gtm_cls1.MCS1_MEM[2202].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2203 GTM_gtm_cls1.MCS1_MEM[2203].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2204 GTM_gtm_cls1.MCS1_MEM[2204].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2205 GTM_gtm_cls1.MCS1_MEM[2205].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2206 GTM_gtm_cls1.MCS1_MEM[2206].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2207 GTM_gtm_cls1.MCS1_MEM[2207].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2208 GTM_gtm_cls1.MCS1_MEM[2208].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2209 GTM_gtm_cls1.MCS1_MEM[2209].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2210 GTM_gtm_cls1.MCS1_MEM[2210].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2211 GTM_gtm_cls1.MCS1_MEM[2211].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2212 GTM_gtm_cls1.MCS1_MEM[2212].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2213 GTM_gtm_cls1.MCS1_MEM[2213].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2214 GTM_gtm_cls1.MCS1_MEM[2214].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2215 GTM_gtm_cls1.MCS1_MEM[2215].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2216 GTM_gtm_cls1.MCS1_MEM[2216].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2217 GTM_gtm_cls1.MCS1_MEM[2217].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2218 GTM_gtm_cls1.MCS1_MEM[2218].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2219 GTM_gtm_cls1.MCS1_MEM[2219].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2220 GTM_gtm_cls1.MCS1_MEM[2220].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2221 GTM_gtm_cls1.MCS1_MEM[2221].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2222 GTM_gtm_cls1.MCS1_MEM[2222].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2223 GTM_gtm_cls1.MCS1_MEM[2223].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2224 GTM_gtm_cls1.MCS1_MEM[2224].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2225 GTM_gtm_cls1.MCS1_MEM[2225].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2226 GTM_gtm_cls1.MCS1_MEM[2226].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2227 GTM_gtm_cls1.MCS1_MEM[2227].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2228 GTM_gtm_cls1.MCS1_MEM[2228].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2229 GTM_gtm_cls1.MCS1_MEM[2229].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2230 GTM_gtm_cls1.MCS1_MEM[2230].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2231 GTM_gtm_cls1.MCS1_MEM[2231].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2232 GTM_gtm_cls1.MCS1_MEM[2232].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2233 GTM_gtm_cls1.MCS1_MEM[2233].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2234 GTM_gtm_cls1.MCS1_MEM[2234].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2235 GTM_gtm_cls1.MCS1_MEM[2235].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2236 GTM_gtm_cls1.MCS1_MEM[2236].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2237 GTM_gtm_cls1.MCS1_MEM[2237].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2238 GTM_gtm_cls1.MCS1_MEM[2238].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2239 GTM_gtm_cls1.MCS1_MEM[2239].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2240 GTM_gtm_cls1.MCS1_MEM[2240].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2241 GTM_gtm_cls1.MCS1_MEM[2241].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2242 GTM_gtm_cls1.MCS1_MEM[2242].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2243 GTM_gtm_cls1.MCS1_MEM[2243].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2244 GTM_gtm_cls1.MCS1_MEM[2244].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2245 GTM_gtm_cls1.MCS1_MEM[2245].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2246 GTM_gtm_cls1.MCS1_MEM[2246].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2247 GTM_gtm_cls1.MCS1_MEM[2247].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2248 GTM_gtm_cls1.MCS1_MEM[2248].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2249 GTM_gtm_cls1.MCS1_MEM[2249].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2250 GTM_gtm_cls1.MCS1_MEM[2250].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2251 GTM_gtm_cls1.MCS1_MEM[2251].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2252 GTM_gtm_cls1.MCS1_MEM[2252].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2253 GTM_gtm_cls1.MCS1_MEM[2253].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2254 GTM_gtm_cls1.MCS1_MEM[2254].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2255 GTM_gtm_cls1.MCS1_MEM[2255].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2256 GTM_gtm_cls1.MCS1_MEM[2256].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2257 GTM_gtm_cls1.MCS1_MEM[2257].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2258 GTM_gtm_cls1.MCS1_MEM[2258].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2259 GTM_gtm_cls1.MCS1_MEM[2259].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2260 GTM_gtm_cls1.MCS1_MEM[2260].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2261 GTM_gtm_cls1.MCS1_MEM[2261].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2262 GTM_gtm_cls1.MCS1_MEM[2262].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2263 GTM_gtm_cls1.MCS1_MEM[2263].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2264 GTM_gtm_cls1.MCS1_MEM[2264].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2265 GTM_gtm_cls1.MCS1_MEM[2265].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2266 GTM_gtm_cls1.MCS1_MEM[2266].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2267 GTM_gtm_cls1.MCS1_MEM[2267].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2268 GTM_gtm_cls1.MCS1_MEM[2268].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2269 GTM_gtm_cls1.MCS1_MEM[2269].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2270 GTM_gtm_cls1.MCS1_MEM[2270].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2271 GTM_gtm_cls1.MCS1_MEM[2271].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2272 GTM_gtm_cls1.MCS1_MEM[2272].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2273 GTM_gtm_cls1.MCS1_MEM[2273].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2274 GTM_gtm_cls1.MCS1_MEM[2274].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2275 GTM_gtm_cls1.MCS1_MEM[2275].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2276 GTM_gtm_cls1.MCS1_MEM[2276].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2277 GTM_gtm_cls1.MCS1_MEM[2277].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2278 GTM_gtm_cls1.MCS1_MEM[2278].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2279 GTM_gtm_cls1.MCS1_MEM[2279].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2280 GTM_gtm_cls1.MCS1_MEM[2280].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2281 GTM_gtm_cls1.MCS1_MEM[2281].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2282 GTM_gtm_cls1.MCS1_MEM[2282].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2283 GTM_gtm_cls1.MCS1_MEM[2283].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2284 GTM_gtm_cls1.MCS1_MEM[2284].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2285 GTM_gtm_cls1.MCS1_MEM[2285].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2286 GTM_gtm_cls1.MCS1_MEM[2286].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2287 GTM_gtm_cls1.MCS1_MEM[2287].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2288 GTM_gtm_cls1.MCS1_MEM[2288].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2289 GTM_gtm_cls1.MCS1_MEM[2289].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2290 GTM_gtm_cls1.MCS1_MEM[2290].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2291 GTM_gtm_cls1.MCS1_MEM[2291].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2292 GTM_gtm_cls1.MCS1_MEM[2292].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2293 GTM_gtm_cls1.MCS1_MEM[2293].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2294 GTM_gtm_cls1.MCS1_MEM[2294].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2295 GTM_gtm_cls1.MCS1_MEM[2295].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2296 GTM_gtm_cls1.MCS1_MEM[2296].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2297 GTM_gtm_cls1.MCS1_MEM[2297].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2298 GTM_gtm_cls1.MCS1_MEM[2298].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2299 GTM_gtm_cls1.MCS1_MEM[2299].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2300 GTM_gtm_cls1.MCS1_MEM[2300].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2301 GTM_gtm_cls1.MCS1_MEM[2301].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2302 GTM_gtm_cls1.MCS1_MEM[2302].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2303 GTM_gtm_cls1.MCS1_MEM[2303].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2304 GTM_gtm_cls1.MCS1_MEM[2304].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2305 GTM_gtm_cls1.MCS1_MEM[2305].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2306 GTM_gtm_cls1.MCS1_MEM[2306].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2307 GTM_gtm_cls1.MCS1_MEM[2307].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2308 GTM_gtm_cls1.MCS1_MEM[2308].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2309 GTM_gtm_cls1.MCS1_MEM[2309].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2310 GTM_gtm_cls1.MCS1_MEM[2310].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2311 GTM_gtm_cls1.MCS1_MEM[2311].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2312 GTM_gtm_cls1.MCS1_MEM[2312].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2313 GTM_gtm_cls1.MCS1_MEM[2313].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2314 GTM_gtm_cls1.MCS1_MEM[2314].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2315 GTM_gtm_cls1.MCS1_MEM[2315].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2316 GTM_gtm_cls1.MCS1_MEM[2316].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2317 GTM_gtm_cls1.MCS1_MEM[2317].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2318 GTM_gtm_cls1.MCS1_MEM[2318].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2319 GTM_gtm_cls1.MCS1_MEM[2319].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2320 GTM_gtm_cls1.MCS1_MEM[2320].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2321 GTM_gtm_cls1.MCS1_MEM[2321].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2322 GTM_gtm_cls1.MCS1_MEM[2322].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2323 GTM_gtm_cls1.MCS1_MEM[2323].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2324 GTM_gtm_cls1.MCS1_MEM[2324].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2325 GTM_gtm_cls1.MCS1_MEM[2325].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2326 GTM_gtm_cls1.MCS1_MEM[2326].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2327 GTM_gtm_cls1.MCS1_MEM[2327].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2328 GTM_gtm_cls1.MCS1_MEM[2328].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2329 GTM_gtm_cls1.MCS1_MEM[2329].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2330 GTM_gtm_cls1.MCS1_MEM[2330].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2331 GTM_gtm_cls1.MCS1_MEM[2331].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2332 GTM_gtm_cls1.MCS1_MEM[2332].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2333 GTM_gtm_cls1.MCS1_MEM[2333].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2334 GTM_gtm_cls1.MCS1_MEM[2334].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2335 GTM_gtm_cls1.MCS1_MEM[2335].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2336 GTM_gtm_cls1.MCS1_MEM[2336].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2337 GTM_gtm_cls1.MCS1_MEM[2337].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2338 GTM_gtm_cls1.MCS1_MEM[2338].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2339 GTM_gtm_cls1.MCS1_MEM[2339].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2340 GTM_gtm_cls1.MCS1_MEM[2340].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2341 GTM_gtm_cls1.MCS1_MEM[2341].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2342 GTM_gtm_cls1.MCS1_MEM[2342].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2343 GTM_gtm_cls1.MCS1_MEM[2343].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2344 GTM_gtm_cls1.MCS1_MEM[2344].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2345 GTM_gtm_cls1.MCS1_MEM[2345].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2346 GTM_gtm_cls1.MCS1_MEM[2346].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2347 GTM_gtm_cls1.MCS1_MEM[2347].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2348 GTM_gtm_cls1.MCS1_MEM[2348].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2349 GTM_gtm_cls1.MCS1_MEM[2349].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2350 GTM_gtm_cls1.MCS1_MEM[2350].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2351 GTM_gtm_cls1.MCS1_MEM[2351].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2352 GTM_gtm_cls1.MCS1_MEM[2352].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2353 GTM_gtm_cls1.MCS1_MEM[2353].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2354 GTM_gtm_cls1.MCS1_MEM[2354].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2355 GTM_gtm_cls1.MCS1_MEM[2355].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2356 GTM_gtm_cls1.MCS1_MEM[2356].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2357 GTM_gtm_cls1.MCS1_MEM[2357].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2358 GTM_gtm_cls1.MCS1_MEM[2358].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2359 GTM_gtm_cls1.MCS1_MEM[2359].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2360 GTM_gtm_cls1.MCS1_MEM[2360].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2361 GTM_gtm_cls1.MCS1_MEM[2361].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2362 GTM_gtm_cls1.MCS1_MEM[2362].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2363 GTM_gtm_cls1.MCS1_MEM[2363].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2364 GTM_gtm_cls1.MCS1_MEM[2364].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2365 GTM_gtm_cls1.MCS1_MEM[2365].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2366 GTM_gtm_cls1.MCS1_MEM[2366].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2367 GTM_gtm_cls1.MCS1_MEM[2367].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2368 GTM_gtm_cls1.MCS1_MEM[2368].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2369 GTM_gtm_cls1.MCS1_MEM[2369].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2370 GTM_gtm_cls1.MCS1_MEM[2370].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2371 GTM_gtm_cls1.MCS1_MEM[2371].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2372 GTM_gtm_cls1.MCS1_MEM[2372].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2373 GTM_gtm_cls1.MCS1_MEM[2373].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2374 GTM_gtm_cls1.MCS1_MEM[2374].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2375 GTM_gtm_cls1.MCS1_MEM[2375].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2376 GTM_gtm_cls1.MCS1_MEM[2376].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2377 GTM_gtm_cls1.MCS1_MEM[2377].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2378 GTM_gtm_cls1.MCS1_MEM[2378].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2379 GTM_gtm_cls1.MCS1_MEM[2379].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2380 GTM_gtm_cls1.MCS1_MEM[2380].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2381 GTM_gtm_cls1.MCS1_MEM[2381].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2382 GTM_gtm_cls1.MCS1_MEM[2382].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2383 GTM_gtm_cls1.MCS1_MEM[2383].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2384 GTM_gtm_cls1.MCS1_MEM[2384].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2385 GTM_gtm_cls1.MCS1_MEM[2385].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2386 GTM_gtm_cls1.MCS1_MEM[2386].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2387 GTM_gtm_cls1.MCS1_MEM[2387].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2388 GTM_gtm_cls1.MCS1_MEM[2388].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2389 GTM_gtm_cls1.MCS1_MEM[2389].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2390 GTM_gtm_cls1.MCS1_MEM[2390].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2391 GTM_gtm_cls1.MCS1_MEM[2391].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2392 GTM_gtm_cls1.MCS1_MEM[2392].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2393 GTM_gtm_cls1.MCS1_MEM[2393].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2394 GTM_gtm_cls1.MCS1_MEM[2394].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2395 GTM_gtm_cls1.MCS1_MEM[2395].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2396 GTM_gtm_cls1.MCS1_MEM[2396].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2397 GTM_gtm_cls1.MCS1_MEM[2397].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2398 GTM_gtm_cls1.MCS1_MEM[2398].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2399 GTM_gtm_cls1.MCS1_MEM[2399].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2400 GTM_gtm_cls1.MCS1_MEM[2400].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2401 GTM_gtm_cls1.MCS1_MEM[2401].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2402 GTM_gtm_cls1.MCS1_MEM[2402].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2403 GTM_gtm_cls1.MCS1_MEM[2403].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2404 GTM_gtm_cls1.MCS1_MEM[2404].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2405 GTM_gtm_cls1.MCS1_MEM[2405].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2406 GTM_gtm_cls1.MCS1_MEM[2406].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2407 GTM_gtm_cls1.MCS1_MEM[2407].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2408 GTM_gtm_cls1.MCS1_MEM[2408].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2409 GTM_gtm_cls1.MCS1_MEM[2409].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2410 GTM_gtm_cls1.MCS1_MEM[2410].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2411 GTM_gtm_cls1.MCS1_MEM[2411].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2412 GTM_gtm_cls1.MCS1_MEM[2412].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2413 GTM_gtm_cls1.MCS1_MEM[2413].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2414 GTM_gtm_cls1.MCS1_MEM[2414].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2415 GTM_gtm_cls1.MCS1_MEM[2415].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2416 GTM_gtm_cls1.MCS1_MEM[2416].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2417 GTM_gtm_cls1.MCS1_MEM[2417].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2418 GTM_gtm_cls1.MCS1_MEM[2418].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2419 GTM_gtm_cls1.MCS1_MEM[2419].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2420 GTM_gtm_cls1.MCS1_MEM[2420].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2421 GTM_gtm_cls1.MCS1_MEM[2421].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2422 GTM_gtm_cls1.MCS1_MEM[2422].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2423 GTM_gtm_cls1.MCS1_MEM[2423].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2424 GTM_gtm_cls1.MCS1_MEM[2424].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2425 GTM_gtm_cls1.MCS1_MEM[2425].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2426 GTM_gtm_cls1.MCS1_MEM[2426].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2427 GTM_gtm_cls1.MCS1_MEM[2427].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2428 GTM_gtm_cls1.MCS1_MEM[2428].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2429 GTM_gtm_cls1.MCS1_MEM[2429].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2430 GTM_gtm_cls1.MCS1_MEM[2430].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2431 GTM_gtm_cls1.MCS1_MEM[2431].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2432 GTM_gtm_cls1.MCS1_MEM[2432].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2433 GTM_gtm_cls1.MCS1_MEM[2433].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2434 GTM_gtm_cls1.MCS1_MEM[2434].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2435 GTM_gtm_cls1.MCS1_MEM[2435].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2436 GTM_gtm_cls1.MCS1_MEM[2436].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2437 GTM_gtm_cls1.MCS1_MEM[2437].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2438 GTM_gtm_cls1.MCS1_MEM[2438].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2439 GTM_gtm_cls1.MCS1_MEM[2439].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2440 GTM_gtm_cls1.MCS1_MEM[2440].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2441 GTM_gtm_cls1.MCS1_MEM[2441].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2442 GTM_gtm_cls1.MCS1_MEM[2442].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2443 GTM_gtm_cls1.MCS1_MEM[2443].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2444 GTM_gtm_cls1.MCS1_MEM[2444].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2445 GTM_gtm_cls1.MCS1_MEM[2445].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2446 GTM_gtm_cls1.MCS1_MEM[2446].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2447 GTM_gtm_cls1.MCS1_MEM[2447].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2448 GTM_gtm_cls1.MCS1_MEM[2448].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2449 GTM_gtm_cls1.MCS1_MEM[2449].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2450 GTM_gtm_cls1.MCS1_MEM[2450].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2451 GTM_gtm_cls1.MCS1_MEM[2451].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2452 GTM_gtm_cls1.MCS1_MEM[2452].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2453 GTM_gtm_cls1.MCS1_MEM[2453].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2454 GTM_gtm_cls1.MCS1_MEM[2454].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2455 GTM_gtm_cls1.MCS1_MEM[2455].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2456 GTM_gtm_cls1.MCS1_MEM[2456].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2457 GTM_gtm_cls1.MCS1_MEM[2457].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2458 GTM_gtm_cls1.MCS1_MEM[2458].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2459 GTM_gtm_cls1.MCS1_MEM[2459].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2460 GTM_gtm_cls1.MCS1_MEM[2460].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2461 GTM_gtm_cls1.MCS1_MEM[2461].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2462 GTM_gtm_cls1.MCS1_MEM[2462].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2463 GTM_gtm_cls1.MCS1_MEM[2463].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2464 GTM_gtm_cls1.MCS1_MEM[2464].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2465 GTM_gtm_cls1.MCS1_MEM[2465].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2466 GTM_gtm_cls1.MCS1_MEM[2466].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2467 GTM_gtm_cls1.MCS1_MEM[2467].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2468 GTM_gtm_cls1.MCS1_MEM[2468].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2469 GTM_gtm_cls1.MCS1_MEM[2469].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2470 GTM_gtm_cls1.MCS1_MEM[2470].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2471 GTM_gtm_cls1.MCS1_MEM[2471].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2472 GTM_gtm_cls1.MCS1_MEM[2472].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2473 GTM_gtm_cls1.MCS1_MEM[2473].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2474 GTM_gtm_cls1.MCS1_MEM[2474].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2475 GTM_gtm_cls1.MCS1_MEM[2475].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2476 GTM_gtm_cls1.MCS1_MEM[2476].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2477 GTM_gtm_cls1.MCS1_MEM[2477].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2478 GTM_gtm_cls1.MCS1_MEM[2478].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2479 GTM_gtm_cls1.MCS1_MEM[2479].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2480 GTM_gtm_cls1.MCS1_MEM[2480].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2481 GTM_gtm_cls1.MCS1_MEM[2481].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2482 GTM_gtm_cls1.MCS1_MEM[2482].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2483 GTM_gtm_cls1.MCS1_MEM[2483].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2484 GTM_gtm_cls1.MCS1_MEM[2484].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2485 GTM_gtm_cls1.MCS1_MEM[2485].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2486 GTM_gtm_cls1.MCS1_MEM[2486].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2487 GTM_gtm_cls1.MCS1_MEM[2487].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2488 GTM_gtm_cls1.MCS1_MEM[2488].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2489 GTM_gtm_cls1.MCS1_MEM[2489].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2490 GTM_gtm_cls1.MCS1_MEM[2490].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2491 GTM_gtm_cls1.MCS1_MEM[2491].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2492 GTM_gtm_cls1.MCS1_MEM[2492].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2493 GTM_gtm_cls1.MCS1_MEM[2493].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2494 GTM_gtm_cls1.MCS1_MEM[2494].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2495 GTM_gtm_cls1.MCS1_MEM[2495].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2496 GTM_gtm_cls1.MCS1_MEM[2496].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2497 GTM_gtm_cls1.MCS1_MEM[2497].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2498 GTM_gtm_cls1.MCS1_MEM[2498].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2499 GTM_gtm_cls1.MCS1_MEM[2499].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2500 GTM_gtm_cls1.MCS1_MEM[2500].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2501 GTM_gtm_cls1.MCS1_MEM[2501].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2502 GTM_gtm_cls1.MCS1_MEM[2502].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2503 GTM_gtm_cls1.MCS1_MEM[2503].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2504 GTM_gtm_cls1.MCS1_MEM[2504].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2505 GTM_gtm_cls1.MCS1_MEM[2505].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2506 GTM_gtm_cls1.MCS1_MEM[2506].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2507 GTM_gtm_cls1.MCS1_MEM[2507].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2508 GTM_gtm_cls1.MCS1_MEM[2508].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2509 GTM_gtm_cls1.MCS1_MEM[2509].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2510 GTM_gtm_cls1.MCS1_MEM[2510].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2511 GTM_gtm_cls1.MCS1_MEM[2511].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2512 GTM_gtm_cls1.MCS1_MEM[2512].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2513 GTM_gtm_cls1.MCS1_MEM[2513].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2514 GTM_gtm_cls1.MCS1_MEM[2514].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2515 GTM_gtm_cls1.MCS1_MEM[2515].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2516 GTM_gtm_cls1.MCS1_MEM[2516].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2517 GTM_gtm_cls1.MCS1_MEM[2517].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2518 GTM_gtm_cls1.MCS1_MEM[2518].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2519 GTM_gtm_cls1.MCS1_MEM[2519].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2520 GTM_gtm_cls1.MCS1_MEM[2520].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2521 GTM_gtm_cls1.MCS1_MEM[2521].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2522 GTM_gtm_cls1.MCS1_MEM[2522].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2523 GTM_gtm_cls1.MCS1_MEM[2523].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2524 GTM_gtm_cls1.MCS1_MEM[2524].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2525 GTM_gtm_cls1.MCS1_MEM[2525].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2526 GTM_gtm_cls1.MCS1_MEM[2526].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2527 GTM_gtm_cls1.MCS1_MEM[2527].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2528 GTM_gtm_cls1.MCS1_MEM[2528].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2529 GTM_gtm_cls1.MCS1_MEM[2529].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2530 GTM_gtm_cls1.MCS1_MEM[2530].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2531 GTM_gtm_cls1.MCS1_MEM[2531].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2532 GTM_gtm_cls1.MCS1_MEM[2532].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2533 GTM_gtm_cls1.MCS1_MEM[2533].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2534 GTM_gtm_cls1.MCS1_MEM[2534].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2535 GTM_gtm_cls1.MCS1_MEM[2535].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2536 GTM_gtm_cls1.MCS1_MEM[2536].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2537 GTM_gtm_cls1.MCS1_MEM[2537].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2538 GTM_gtm_cls1.MCS1_MEM[2538].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2539 GTM_gtm_cls1.MCS1_MEM[2539].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2540 GTM_gtm_cls1.MCS1_MEM[2540].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2541 GTM_gtm_cls1.MCS1_MEM[2541].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2542 GTM_gtm_cls1.MCS1_MEM[2542].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2543 GTM_gtm_cls1.MCS1_MEM[2543].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2544 GTM_gtm_cls1.MCS1_MEM[2544].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2545 GTM_gtm_cls1.MCS1_MEM[2545].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2546 GTM_gtm_cls1.MCS1_MEM[2546].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2547 GTM_gtm_cls1.MCS1_MEM[2547].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2548 GTM_gtm_cls1.MCS1_MEM[2548].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2549 GTM_gtm_cls1.MCS1_MEM[2549].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2550 GTM_gtm_cls1.MCS1_MEM[2550].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2551 GTM_gtm_cls1.MCS1_MEM[2551].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2552 GTM_gtm_cls1.MCS1_MEM[2552].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2553 GTM_gtm_cls1.MCS1_MEM[2553].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2554 GTM_gtm_cls1.MCS1_MEM[2554].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2555 GTM_gtm_cls1.MCS1_MEM[2555].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2556 GTM_gtm_cls1.MCS1_MEM[2556].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2557 GTM_gtm_cls1.MCS1_MEM[2557].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2558 GTM_gtm_cls1.MCS1_MEM[2558].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2559 GTM_gtm_cls1.MCS1_MEM[2559].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2560 GTM_gtm_cls1.MCS1_MEM[2560].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2561 GTM_gtm_cls1.MCS1_MEM[2561].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2562 GTM_gtm_cls1.MCS1_MEM[2562].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2563 GTM_gtm_cls1.MCS1_MEM[2563].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2564 GTM_gtm_cls1.MCS1_MEM[2564].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2565 GTM_gtm_cls1.MCS1_MEM[2565].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2566 GTM_gtm_cls1.MCS1_MEM[2566].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2567 GTM_gtm_cls1.MCS1_MEM[2567].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2568 GTM_gtm_cls1.MCS1_MEM[2568].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2569 GTM_gtm_cls1.MCS1_MEM[2569].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2570 GTM_gtm_cls1.MCS1_MEM[2570].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2571 GTM_gtm_cls1.MCS1_MEM[2571].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2572 GTM_gtm_cls1.MCS1_MEM[2572].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2573 GTM_gtm_cls1.MCS1_MEM[2573].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2574 GTM_gtm_cls1.MCS1_MEM[2574].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2575 GTM_gtm_cls1.MCS1_MEM[2575].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2576 GTM_gtm_cls1.MCS1_MEM[2576].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2577 GTM_gtm_cls1.MCS1_MEM[2577].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2578 GTM_gtm_cls1.MCS1_MEM[2578].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2579 GTM_gtm_cls1.MCS1_MEM[2579].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2580 GTM_gtm_cls1.MCS1_MEM[2580].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2581 GTM_gtm_cls1.MCS1_MEM[2581].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2582 GTM_gtm_cls1.MCS1_MEM[2582].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2583 GTM_gtm_cls1.MCS1_MEM[2583].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2584 GTM_gtm_cls1.MCS1_MEM[2584].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2585 GTM_gtm_cls1.MCS1_MEM[2585].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2586 GTM_gtm_cls1.MCS1_MEM[2586].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2587 GTM_gtm_cls1.MCS1_MEM[2587].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2588 GTM_gtm_cls1.MCS1_MEM[2588].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2589 GTM_gtm_cls1.MCS1_MEM[2589].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2590 GTM_gtm_cls1.MCS1_MEM[2590].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2591 GTM_gtm_cls1.MCS1_MEM[2591].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2592 GTM_gtm_cls1.MCS1_MEM[2592].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2593 GTM_gtm_cls1.MCS1_MEM[2593].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2594 GTM_gtm_cls1.MCS1_MEM[2594].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2595 GTM_gtm_cls1.MCS1_MEM[2595].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2596 GTM_gtm_cls1.MCS1_MEM[2596].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2597 GTM_gtm_cls1.MCS1_MEM[2597].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2598 GTM_gtm_cls1.MCS1_MEM[2598].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2599 GTM_gtm_cls1.MCS1_MEM[2599].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2600 GTM_gtm_cls1.MCS1_MEM[2600].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2601 GTM_gtm_cls1.MCS1_MEM[2601].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2602 GTM_gtm_cls1.MCS1_MEM[2602].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2603 GTM_gtm_cls1.MCS1_MEM[2603].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2604 GTM_gtm_cls1.MCS1_MEM[2604].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2605 GTM_gtm_cls1.MCS1_MEM[2605].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2606 GTM_gtm_cls1.MCS1_MEM[2606].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2607 GTM_gtm_cls1.MCS1_MEM[2607].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2608 GTM_gtm_cls1.MCS1_MEM[2608].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2609 GTM_gtm_cls1.MCS1_MEM[2609].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2610 GTM_gtm_cls1.MCS1_MEM[2610].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2611 GTM_gtm_cls1.MCS1_MEM[2611].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2612 GTM_gtm_cls1.MCS1_MEM[2612].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2613 GTM_gtm_cls1.MCS1_MEM[2613].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2614 GTM_gtm_cls1.MCS1_MEM[2614].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2615 GTM_gtm_cls1.MCS1_MEM[2615].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2616 GTM_gtm_cls1.MCS1_MEM[2616].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2617 GTM_gtm_cls1.MCS1_MEM[2617].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2618 GTM_gtm_cls1.MCS1_MEM[2618].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2619 GTM_gtm_cls1.MCS1_MEM[2619].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2620 GTM_gtm_cls1.MCS1_MEM[2620].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2621 GTM_gtm_cls1.MCS1_MEM[2621].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2622 GTM_gtm_cls1.MCS1_MEM[2622].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2623 GTM_gtm_cls1.MCS1_MEM[2623].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2624 GTM_gtm_cls1.MCS1_MEM[2624].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2625 GTM_gtm_cls1.MCS1_MEM[2625].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2626 GTM_gtm_cls1.MCS1_MEM[2626].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2627 GTM_gtm_cls1.MCS1_MEM[2627].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2628 GTM_gtm_cls1.MCS1_MEM[2628].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2629 GTM_gtm_cls1.MCS1_MEM[2629].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2630 GTM_gtm_cls1.MCS1_MEM[2630].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2631 GTM_gtm_cls1.MCS1_MEM[2631].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2632 GTM_gtm_cls1.MCS1_MEM[2632].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2633 GTM_gtm_cls1.MCS1_MEM[2633].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2634 GTM_gtm_cls1.MCS1_MEM[2634].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2635 GTM_gtm_cls1.MCS1_MEM[2635].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2636 GTM_gtm_cls1.MCS1_MEM[2636].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2637 GTM_gtm_cls1.MCS1_MEM[2637].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2638 GTM_gtm_cls1.MCS1_MEM[2638].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2639 GTM_gtm_cls1.MCS1_MEM[2639].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2640 GTM_gtm_cls1.MCS1_MEM[2640].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2641 GTM_gtm_cls1.MCS1_MEM[2641].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2642 GTM_gtm_cls1.MCS1_MEM[2642].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2643 GTM_gtm_cls1.MCS1_MEM[2643].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2644 GTM_gtm_cls1.MCS1_MEM[2644].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2645 GTM_gtm_cls1.MCS1_MEM[2645].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2646 GTM_gtm_cls1.MCS1_MEM[2646].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2647 GTM_gtm_cls1.MCS1_MEM[2647].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2648 GTM_gtm_cls1.MCS1_MEM[2648].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2649 GTM_gtm_cls1.MCS1_MEM[2649].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2650 GTM_gtm_cls1.MCS1_MEM[2650].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2651 GTM_gtm_cls1.MCS1_MEM[2651].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2652 GTM_gtm_cls1.MCS1_MEM[2652].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2653 GTM_gtm_cls1.MCS1_MEM[2653].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2654 GTM_gtm_cls1.MCS1_MEM[2654].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2655 GTM_gtm_cls1.MCS1_MEM[2655].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2656 GTM_gtm_cls1.MCS1_MEM[2656].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2657 GTM_gtm_cls1.MCS1_MEM[2657].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2658 GTM_gtm_cls1.MCS1_MEM[2658].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2659 GTM_gtm_cls1.MCS1_MEM[2659].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2660 GTM_gtm_cls1.MCS1_MEM[2660].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2661 GTM_gtm_cls1.MCS1_MEM[2661].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2662 GTM_gtm_cls1.MCS1_MEM[2662].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2663 GTM_gtm_cls1.MCS1_MEM[2663].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2664 GTM_gtm_cls1.MCS1_MEM[2664].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2665 GTM_gtm_cls1.MCS1_MEM[2665].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2666 GTM_gtm_cls1.MCS1_MEM[2666].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2667 GTM_gtm_cls1.MCS1_MEM[2667].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2668 GTM_gtm_cls1.MCS1_MEM[2668].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2669 GTM_gtm_cls1.MCS1_MEM[2669].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2670 GTM_gtm_cls1.MCS1_MEM[2670].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2671 GTM_gtm_cls1.MCS1_MEM[2671].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2672 GTM_gtm_cls1.MCS1_MEM[2672].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2673 GTM_gtm_cls1.MCS1_MEM[2673].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2674 GTM_gtm_cls1.MCS1_MEM[2674].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2675 GTM_gtm_cls1.MCS1_MEM[2675].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2676 GTM_gtm_cls1.MCS1_MEM[2676].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2677 GTM_gtm_cls1.MCS1_MEM[2677].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2678 GTM_gtm_cls1.MCS1_MEM[2678].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2679 GTM_gtm_cls1.MCS1_MEM[2679].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2680 GTM_gtm_cls1.MCS1_MEM[2680].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2681 GTM_gtm_cls1.MCS1_MEM[2681].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2682 GTM_gtm_cls1.MCS1_MEM[2682].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2683 GTM_gtm_cls1.MCS1_MEM[2683].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2684 GTM_gtm_cls1.MCS1_MEM[2684].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2685 GTM_gtm_cls1.MCS1_MEM[2685].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2686 GTM_gtm_cls1.MCS1_MEM[2686].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2687 GTM_gtm_cls1.MCS1_MEM[2687].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2688 GTM_gtm_cls1.MCS1_MEM[2688].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2689 GTM_gtm_cls1.MCS1_MEM[2689].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2690 GTM_gtm_cls1.MCS1_MEM[2690].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2691 GTM_gtm_cls1.MCS1_MEM[2691].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2692 GTM_gtm_cls1.MCS1_MEM[2692].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2693 GTM_gtm_cls1.MCS1_MEM[2693].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2694 GTM_gtm_cls1.MCS1_MEM[2694].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2695 GTM_gtm_cls1.MCS1_MEM[2695].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2696 GTM_gtm_cls1.MCS1_MEM[2696].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2697 GTM_gtm_cls1.MCS1_MEM[2697].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2698 GTM_gtm_cls1.MCS1_MEM[2698].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2699 GTM_gtm_cls1.MCS1_MEM[2699].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2700 GTM_gtm_cls1.MCS1_MEM[2700].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2701 GTM_gtm_cls1.MCS1_MEM[2701].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2702 GTM_gtm_cls1.MCS1_MEM[2702].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2703 GTM_gtm_cls1.MCS1_MEM[2703].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2704 GTM_gtm_cls1.MCS1_MEM[2704].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2705 GTM_gtm_cls1.MCS1_MEM[2705].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2706 GTM_gtm_cls1.MCS1_MEM[2706].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2707 GTM_gtm_cls1.MCS1_MEM[2707].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2708 GTM_gtm_cls1.MCS1_MEM[2708].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2709 GTM_gtm_cls1.MCS1_MEM[2709].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2710 GTM_gtm_cls1.MCS1_MEM[2710].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2711 GTM_gtm_cls1.MCS1_MEM[2711].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2712 GTM_gtm_cls1.MCS1_MEM[2712].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2713 GTM_gtm_cls1.MCS1_MEM[2713].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2714 GTM_gtm_cls1.MCS1_MEM[2714].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2715 GTM_gtm_cls1.MCS1_MEM[2715].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2716 GTM_gtm_cls1.MCS1_MEM[2716].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2717 GTM_gtm_cls1.MCS1_MEM[2717].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2718 GTM_gtm_cls1.MCS1_MEM[2718].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2719 GTM_gtm_cls1.MCS1_MEM[2719].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2720 GTM_gtm_cls1.MCS1_MEM[2720].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2721 GTM_gtm_cls1.MCS1_MEM[2721].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2722 GTM_gtm_cls1.MCS1_MEM[2722].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2723 GTM_gtm_cls1.MCS1_MEM[2723].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2724 GTM_gtm_cls1.MCS1_MEM[2724].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2725 GTM_gtm_cls1.MCS1_MEM[2725].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2726 GTM_gtm_cls1.MCS1_MEM[2726].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2727 GTM_gtm_cls1.MCS1_MEM[2727].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2728 GTM_gtm_cls1.MCS1_MEM[2728].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2729 GTM_gtm_cls1.MCS1_MEM[2729].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2730 GTM_gtm_cls1.MCS1_MEM[2730].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2731 GTM_gtm_cls1.MCS1_MEM[2731].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2732 GTM_gtm_cls1.MCS1_MEM[2732].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2733 GTM_gtm_cls1.MCS1_MEM[2733].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2734 GTM_gtm_cls1.MCS1_MEM[2734].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2735 GTM_gtm_cls1.MCS1_MEM[2735].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2736 GTM_gtm_cls1.MCS1_MEM[2736].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2737 GTM_gtm_cls1.MCS1_MEM[2737].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2738 GTM_gtm_cls1.MCS1_MEM[2738].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2739 GTM_gtm_cls1.MCS1_MEM[2739].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2740 GTM_gtm_cls1.MCS1_MEM[2740].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2741 GTM_gtm_cls1.MCS1_MEM[2741].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2742 GTM_gtm_cls1.MCS1_MEM[2742].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2743 GTM_gtm_cls1.MCS1_MEM[2743].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2744 GTM_gtm_cls1.MCS1_MEM[2744].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2745 GTM_gtm_cls1.MCS1_MEM[2745].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2746 GTM_gtm_cls1.MCS1_MEM[2746].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2747 GTM_gtm_cls1.MCS1_MEM[2747].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2748 GTM_gtm_cls1.MCS1_MEM[2748].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2749 GTM_gtm_cls1.MCS1_MEM[2749].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2750 GTM_gtm_cls1.MCS1_MEM[2750].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2751 GTM_gtm_cls1.MCS1_MEM[2751].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2752 GTM_gtm_cls1.MCS1_MEM[2752].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2753 GTM_gtm_cls1.MCS1_MEM[2753].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2754 GTM_gtm_cls1.MCS1_MEM[2754].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2755 GTM_gtm_cls1.MCS1_MEM[2755].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2756 GTM_gtm_cls1.MCS1_MEM[2756].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2757 GTM_gtm_cls1.MCS1_MEM[2757].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2758 GTM_gtm_cls1.MCS1_MEM[2758].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2759 GTM_gtm_cls1.MCS1_MEM[2759].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2760 GTM_gtm_cls1.MCS1_MEM[2760].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2761 GTM_gtm_cls1.MCS1_MEM[2761].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2762 GTM_gtm_cls1.MCS1_MEM[2762].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2763 GTM_gtm_cls1.MCS1_MEM[2763].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2764 GTM_gtm_cls1.MCS1_MEM[2764].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2765 GTM_gtm_cls1.MCS1_MEM[2765].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2766 GTM_gtm_cls1.MCS1_MEM[2766].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2767 GTM_gtm_cls1.MCS1_MEM[2767].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2768 GTM_gtm_cls1.MCS1_MEM[2768].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2769 GTM_gtm_cls1.MCS1_MEM[2769].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2770 GTM_gtm_cls1.MCS1_MEM[2770].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2771 GTM_gtm_cls1.MCS1_MEM[2771].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2772 GTM_gtm_cls1.MCS1_MEM[2772].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2773 GTM_gtm_cls1.MCS1_MEM[2773].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2774 GTM_gtm_cls1.MCS1_MEM[2774].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2775 GTM_gtm_cls1.MCS1_MEM[2775].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2776 GTM_gtm_cls1.MCS1_MEM[2776].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2777 GTM_gtm_cls1.MCS1_MEM[2777].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2778 GTM_gtm_cls1.MCS1_MEM[2778].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2779 GTM_gtm_cls1.MCS1_MEM[2779].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2780 GTM_gtm_cls1.MCS1_MEM[2780].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2781 GTM_gtm_cls1.MCS1_MEM[2781].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2782 GTM_gtm_cls1.MCS1_MEM[2782].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2783 GTM_gtm_cls1.MCS1_MEM[2783].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2784 GTM_gtm_cls1.MCS1_MEM[2784].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2785 GTM_gtm_cls1.MCS1_MEM[2785].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2786 GTM_gtm_cls1.MCS1_MEM[2786].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2787 GTM_gtm_cls1.MCS1_MEM[2787].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2788 GTM_gtm_cls1.MCS1_MEM[2788].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2789 GTM_gtm_cls1.MCS1_MEM[2789].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2790 GTM_gtm_cls1.MCS1_MEM[2790].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2791 GTM_gtm_cls1.MCS1_MEM[2791].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2792 GTM_gtm_cls1.MCS1_MEM[2792].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2793 GTM_gtm_cls1.MCS1_MEM[2793].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2794 GTM_gtm_cls1.MCS1_MEM[2794].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2795 GTM_gtm_cls1.MCS1_MEM[2795].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2796 GTM_gtm_cls1.MCS1_MEM[2796].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2797 GTM_gtm_cls1.MCS1_MEM[2797].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2798 GTM_gtm_cls1.MCS1_MEM[2798].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2799 GTM_gtm_cls1.MCS1_MEM[2799].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2800 GTM_gtm_cls1.MCS1_MEM[2800].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2801 GTM_gtm_cls1.MCS1_MEM[2801].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2802 GTM_gtm_cls1.MCS1_MEM[2802].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2803 GTM_gtm_cls1.MCS1_MEM[2803].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2804 GTM_gtm_cls1.MCS1_MEM[2804].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2805 GTM_gtm_cls1.MCS1_MEM[2805].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2806 GTM_gtm_cls1.MCS1_MEM[2806].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2807 GTM_gtm_cls1.MCS1_MEM[2807].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2808 GTM_gtm_cls1.MCS1_MEM[2808].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2809 GTM_gtm_cls1.MCS1_MEM[2809].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2810 GTM_gtm_cls1.MCS1_MEM[2810].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2811 GTM_gtm_cls1.MCS1_MEM[2811].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2812 GTM_gtm_cls1.MCS1_MEM[2812].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2813 GTM_gtm_cls1.MCS1_MEM[2813].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2814 GTM_gtm_cls1.MCS1_MEM[2814].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2815 GTM_gtm_cls1.MCS1_MEM[2815].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2816 GTM_gtm_cls1.MCS1_MEM[2816].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2817 GTM_gtm_cls1.MCS1_MEM[2817].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2818 GTM_gtm_cls1.MCS1_MEM[2818].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2819 GTM_gtm_cls1.MCS1_MEM[2819].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2820 GTM_gtm_cls1.MCS1_MEM[2820].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2821 GTM_gtm_cls1.MCS1_MEM[2821].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2822 GTM_gtm_cls1.MCS1_MEM[2822].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2823 GTM_gtm_cls1.MCS1_MEM[2823].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2824 GTM_gtm_cls1.MCS1_MEM[2824].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2825 GTM_gtm_cls1.MCS1_MEM[2825].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2826 GTM_gtm_cls1.MCS1_MEM[2826].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2827 GTM_gtm_cls1.MCS1_MEM[2827].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2828 GTM_gtm_cls1.MCS1_MEM[2828].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2829 GTM_gtm_cls1.MCS1_MEM[2829].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2830 GTM_gtm_cls1.MCS1_MEM[2830].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2831 GTM_gtm_cls1.MCS1_MEM[2831].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2832 GTM_gtm_cls1.MCS1_MEM[2832].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2833 GTM_gtm_cls1.MCS1_MEM[2833].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2834 GTM_gtm_cls1.MCS1_MEM[2834].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2835 GTM_gtm_cls1.MCS1_MEM[2835].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2836 GTM_gtm_cls1.MCS1_MEM[2836].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2837 GTM_gtm_cls1.MCS1_MEM[2837].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2838 GTM_gtm_cls1.MCS1_MEM[2838].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2839 GTM_gtm_cls1.MCS1_MEM[2839].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2840 GTM_gtm_cls1.MCS1_MEM[2840].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2841 GTM_gtm_cls1.MCS1_MEM[2841].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2842 GTM_gtm_cls1.MCS1_MEM[2842].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2843 GTM_gtm_cls1.MCS1_MEM[2843].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2844 GTM_gtm_cls1.MCS1_MEM[2844].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2845 GTM_gtm_cls1.MCS1_MEM[2845].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2846 GTM_gtm_cls1.MCS1_MEM[2846].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2847 GTM_gtm_cls1.MCS1_MEM[2847].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2848 GTM_gtm_cls1.MCS1_MEM[2848].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2849 GTM_gtm_cls1.MCS1_MEM[2849].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2850 GTM_gtm_cls1.MCS1_MEM[2850].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2851 GTM_gtm_cls1.MCS1_MEM[2851].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2852 GTM_gtm_cls1.MCS1_MEM[2852].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2853 GTM_gtm_cls1.MCS1_MEM[2853].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2854 GTM_gtm_cls1.MCS1_MEM[2854].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2855 GTM_gtm_cls1.MCS1_MEM[2855].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2856 GTM_gtm_cls1.MCS1_MEM[2856].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2857 GTM_gtm_cls1.MCS1_MEM[2857].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2858 GTM_gtm_cls1.MCS1_MEM[2858].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2859 GTM_gtm_cls1.MCS1_MEM[2859].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2860 GTM_gtm_cls1.MCS1_MEM[2860].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2861 GTM_gtm_cls1.MCS1_MEM[2861].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2862 GTM_gtm_cls1.MCS1_MEM[2862].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2863 GTM_gtm_cls1.MCS1_MEM[2863].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2864 GTM_gtm_cls1.MCS1_MEM[2864].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2865 GTM_gtm_cls1.MCS1_MEM[2865].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2866 GTM_gtm_cls1.MCS1_MEM[2866].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2867 GTM_gtm_cls1.MCS1_MEM[2867].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2868 GTM_gtm_cls1.MCS1_MEM[2868].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2869 GTM_gtm_cls1.MCS1_MEM[2869].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2870 GTM_gtm_cls1.MCS1_MEM[2870].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2871 GTM_gtm_cls1.MCS1_MEM[2871].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2872 GTM_gtm_cls1.MCS1_MEM[2872].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2873 GTM_gtm_cls1.MCS1_MEM[2873].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2874 GTM_gtm_cls1.MCS1_MEM[2874].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2875 GTM_gtm_cls1.MCS1_MEM[2875].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2876 GTM_gtm_cls1.MCS1_MEM[2876].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2877 GTM_gtm_cls1.MCS1_MEM[2877].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2878 GTM_gtm_cls1.MCS1_MEM[2878].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2879 GTM_gtm_cls1.MCS1_MEM[2879].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2880 GTM_gtm_cls1.MCS1_MEM[2880].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2881 GTM_gtm_cls1.MCS1_MEM[2881].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2882 GTM_gtm_cls1.MCS1_MEM[2882].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2883 GTM_gtm_cls1.MCS1_MEM[2883].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2884 GTM_gtm_cls1.MCS1_MEM[2884].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2885 GTM_gtm_cls1.MCS1_MEM[2885].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2886 GTM_gtm_cls1.MCS1_MEM[2886].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2887 GTM_gtm_cls1.MCS1_MEM[2887].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2888 GTM_gtm_cls1.MCS1_MEM[2888].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2889 GTM_gtm_cls1.MCS1_MEM[2889].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2890 GTM_gtm_cls1.MCS1_MEM[2890].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2891 GTM_gtm_cls1.MCS1_MEM[2891].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2892 GTM_gtm_cls1.MCS1_MEM[2892].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2893 GTM_gtm_cls1.MCS1_MEM[2893].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2894 GTM_gtm_cls1.MCS1_MEM[2894].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2895 GTM_gtm_cls1.MCS1_MEM[2895].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2896 GTM_gtm_cls1.MCS1_MEM[2896].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2897 GTM_gtm_cls1.MCS1_MEM[2897].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2898 GTM_gtm_cls1.MCS1_MEM[2898].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2899 GTM_gtm_cls1.MCS1_MEM[2899].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2900 GTM_gtm_cls1.MCS1_MEM[2900].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2901 GTM_gtm_cls1.MCS1_MEM[2901].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2902 GTM_gtm_cls1.MCS1_MEM[2902].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2903 GTM_gtm_cls1.MCS1_MEM[2903].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2904 GTM_gtm_cls1.MCS1_MEM[2904].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2905 GTM_gtm_cls1.MCS1_MEM[2905].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2906 GTM_gtm_cls1.MCS1_MEM[2906].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2907 GTM_gtm_cls1.MCS1_MEM[2907].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2908 GTM_gtm_cls1.MCS1_MEM[2908].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2909 GTM_gtm_cls1.MCS1_MEM[2909].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2910 GTM_gtm_cls1.MCS1_MEM[2910].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2911 GTM_gtm_cls1.MCS1_MEM[2911].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2912 GTM_gtm_cls1.MCS1_MEM[2912].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2913 GTM_gtm_cls1.MCS1_MEM[2913].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2914 GTM_gtm_cls1.MCS1_MEM[2914].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2915 GTM_gtm_cls1.MCS1_MEM[2915].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2916 GTM_gtm_cls1.MCS1_MEM[2916].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2917 GTM_gtm_cls1.MCS1_MEM[2917].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2918 GTM_gtm_cls1.MCS1_MEM[2918].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2919 GTM_gtm_cls1.MCS1_MEM[2919].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2920 GTM_gtm_cls1.MCS1_MEM[2920].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2921 GTM_gtm_cls1.MCS1_MEM[2921].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2922 GTM_gtm_cls1.MCS1_MEM[2922].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2923 GTM_gtm_cls1.MCS1_MEM[2923].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2924 GTM_gtm_cls1.MCS1_MEM[2924].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2925 GTM_gtm_cls1.MCS1_MEM[2925].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2926 GTM_gtm_cls1.MCS1_MEM[2926].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2927 GTM_gtm_cls1.MCS1_MEM[2927].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2928 GTM_gtm_cls1.MCS1_MEM[2928].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2929 GTM_gtm_cls1.MCS1_MEM[2929].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2930 GTM_gtm_cls1.MCS1_MEM[2930].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2931 GTM_gtm_cls1.MCS1_MEM[2931].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2932 GTM_gtm_cls1.MCS1_MEM[2932].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2933 GTM_gtm_cls1.MCS1_MEM[2933].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2934 GTM_gtm_cls1.MCS1_MEM[2934].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2935 GTM_gtm_cls1.MCS1_MEM[2935].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2936 GTM_gtm_cls1.MCS1_MEM[2936].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2937 GTM_gtm_cls1.MCS1_MEM[2937].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2938 GTM_gtm_cls1.MCS1_MEM[2938].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2939 GTM_gtm_cls1.MCS1_MEM[2939].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2940 GTM_gtm_cls1.MCS1_MEM[2940].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2941 GTM_gtm_cls1.MCS1_MEM[2941].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2942 GTM_gtm_cls1.MCS1_MEM[2942].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2943 GTM_gtm_cls1.MCS1_MEM[2943].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2944 GTM_gtm_cls1.MCS1_MEM[2944].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2945 GTM_gtm_cls1.MCS1_MEM[2945].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2946 GTM_gtm_cls1.MCS1_MEM[2946].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2947 GTM_gtm_cls1.MCS1_MEM[2947].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2948 GTM_gtm_cls1.MCS1_MEM[2948].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2949 GTM_gtm_cls1.MCS1_MEM[2949].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2950 GTM_gtm_cls1.MCS1_MEM[2950].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2951 GTM_gtm_cls1.MCS1_MEM[2951].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2952 GTM_gtm_cls1.MCS1_MEM[2952].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2953 GTM_gtm_cls1.MCS1_MEM[2953].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2954 GTM_gtm_cls1.MCS1_MEM[2954].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2955 GTM_gtm_cls1.MCS1_MEM[2955].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2956 GTM_gtm_cls1.MCS1_MEM[2956].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2957 GTM_gtm_cls1.MCS1_MEM[2957].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2958 GTM_gtm_cls1.MCS1_MEM[2958].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2959 GTM_gtm_cls1.MCS1_MEM[2959].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2960 GTM_gtm_cls1.MCS1_MEM[2960].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2961 GTM_gtm_cls1.MCS1_MEM[2961].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2962 GTM_gtm_cls1.MCS1_MEM[2962].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2963 GTM_gtm_cls1.MCS1_MEM[2963].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2964 GTM_gtm_cls1.MCS1_MEM[2964].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2965 GTM_gtm_cls1.MCS1_MEM[2965].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2966 GTM_gtm_cls1.MCS1_MEM[2966].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2967 GTM_gtm_cls1.MCS1_MEM[2967].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2968 GTM_gtm_cls1.MCS1_MEM[2968].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2969 GTM_gtm_cls1.MCS1_MEM[2969].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2970 GTM_gtm_cls1.MCS1_MEM[2970].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2971 GTM_gtm_cls1.MCS1_MEM[2971].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2972 GTM_gtm_cls1.MCS1_MEM[2972].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2973 GTM_gtm_cls1.MCS1_MEM[2973].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2974 GTM_gtm_cls1.MCS1_MEM[2974].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2975 GTM_gtm_cls1.MCS1_MEM[2975].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2976 GTM_gtm_cls1.MCS1_MEM[2976].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2977 GTM_gtm_cls1.MCS1_MEM[2977].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2978 GTM_gtm_cls1.MCS1_MEM[2978].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2979 GTM_gtm_cls1.MCS1_MEM[2979].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2980 GTM_gtm_cls1.MCS1_MEM[2980].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2981 GTM_gtm_cls1.MCS1_MEM[2981].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2982 GTM_gtm_cls1.MCS1_MEM[2982].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2983 GTM_gtm_cls1.MCS1_MEM[2983].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2984 GTM_gtm_cls1.MCS1_MEM[2984].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2985 GTM_gtm_cls1.MCS1_MEM[2985].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2986 GTM_gtm_cls1.MCS1_MEM[2986].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2987 GTM_gtm_cls1.MCS1_MEM[2987].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2988 GTM_gtm_cls1.MCS1_MEM[2988].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2989 GTM_gtm_cls1.MCS1_MEM[2989].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2990 GTM_gtm_cls1.MCS1_MEM[2990].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2991 GTM_gtm_cls1.MCS1_MEM[2991].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2992 GTM_gtm_cls1.MCS1_MEM[2992].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2993 GTM_gtm_cls1.MCS1_MEM[2993].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2994 GTM_gtm_cls1.MCS1_MEM[2994].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2995 GTM_gtm_cls1.MCS1_MEM[2995].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2996 GTM_gtm_cls1.MCS1_MEM[2996].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2997 GTM_gtm_cls1.MCS1_MEM[2997].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2998 GTM_gtm_cls1.MCS1_MEM[2998].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM2999 GTM_gtm_cls1.MCS1_MEM[2999].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3000 GTM_gtm_cls1.MCS1_MEM[3000].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3001 GTM_gtm_cls1.MCS1_MEM[3001].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3002 GTM_gtm_cls1.MCS1_MEM[3002].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3003 GTM_gtm_cls1.MCS1_MEM[3003].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3004 GTM_gtm_cls1.MCS1_MEM[3004].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3005 GTM_gtm_cls1.MCS1_MEM[3005].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3006 GTM_gtm_cls1.MCS1_MEM[3006].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3007 GTM_gtm_cls1.MCS1_MEM[3007].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3008 GTM_gtm_cls1.MCS1_MEM[3008].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3009 GTM_gtm_cls1.MCS1_MEM[3009].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3010 GTM_gtm_cls1.MCS1_MEM[3010].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3011 GTM_gtm_cls1.MCS1_MEM[3011].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3012 GTM_gtm_cls1.MCS1_MEM[3012].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3013 GTM_gtm_cls1.MCS1_MEM[3013].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3014 GTM_gtm_cls1.MCS1_MEM[3014].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3015 GTM_gtm_cls1.MCS1_MEM[3015].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3016 GTM_gtm_cls1.MCS1_MEM[3016].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3017 GTM_gtm_cls1.MCS1_MEM[3017].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3018 GTM_gtm_cls1.MCS1_MEM[3018].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3019 GTM_gtm_cls1.MCS1_MEM[3019].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3020 GTM_gtm_cls1.MCS1_MEM[3020].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3021 GTM_gtm_cls1.MCS1_MEM[3021].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3022 GTM_gtm_cls1.MCS1_MEM[3022].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3023 GTM_gtm_cls1.MCS1_MEM[3023].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3024 GTM_gtm_cls1.MCS1_MEM[3024].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3025 GTM_gtm_cls1.MCS1_MEM[3025].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3026 GTM_gtm_cls1.MCS1_MEM[3026].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3027 GTM_gtm_cls1.MCS1_MEM[3027].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3028 GTM_gtm_cls1.MCS1_MEM[3028].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3029 GTM_gtm_cls1.MCS1_MEM[3029].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3030 GTM_gtm_cls1.MCS1_MEM[3030].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3031 GTM_gtm_cls1.MCS1_MEM[3031].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3032 GTM_gtm_cls1.MCS1_MEM[3032].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3033 GTM_gtm_cls1.MCS1_MEM[3033].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3034 GTM_gtm_cls1.MCS1_MEM[3034].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3035 GTM_gtm_cls1.MCS1_MEM[3035].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3036 GTM_gtm_cls1.MCS1_MEM[3036].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3037 GTM_gtm_cls1.MCS1_MEM[3037].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3038 GTM_gtm_cls1.MCS1_MEM[3038].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3039 GTM_gtm_cls1.MCS1_MEM[3039].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3040 GTM_gtm_cls1.MCS1_MEM[3040].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3041 GTM_gtm_cls1.MCS1_MEM[3041].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3042 GTM_gtm_cls1.MCS1_MEM[3042].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3043 GTM_gtm_cls1.MCS1_MEM[3043].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3044 GTM_gtm_cls1.MCS1_MEM[3044].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3045 GTM_gtm_cls1.MCS1_MEM[3045].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3046 GTM_gtm_cls1.MCS1_MEM[3046].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3047 GTM_gtm_cls1.MCS1_MEM[3047].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3048 GTM_gtm_cls1.MCS1_MEM[3048].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3049 GTM_gtm_cls1.MCS1_MEM[3049].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3050 GTM_gtm_cls1.MCS1_MEM[3050].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3051 GTM_gtm_cls1.MCS1_MEM[3051].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3052 GTM_gtm_cls1.MCS1_MEM[3052].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3053 GTM_gtm_cls1.MCS1_MEM[3053].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3054 GTM_gtm_cls1.MCS1_MEM[3054].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3055 GTM_gtm_cls1.MCS1_MEM[3055].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3056 GTM_gtm_cls1.MCS1_MEM[3056].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3057 GTM_gtm_cls1.MCS1_MEM[3057].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3058 GTM_gtm_cls1.MCS1_MEM[3058].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3059 GTM_gtm_cls1.MCS1_MEM[3059].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3060 GTM_gtm_cls1.MCS1_MEM[3060].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3061 GTM_gtm_cls1.MCS1_MEM[3061].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3062 GTM_gtm_cls1.MCS1_MEM[3062].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3063 GTM_gtm_cls1.MCS1_MEM[3063].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3064 GTM_gtm_cls1.MCS1_MEM[3064].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3065 GTM_gtm_cls1.MCS1_MEM[3065].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3066 GTM_gtm_cls1.MCS1_MEM[3066].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3067 GTM_gtm_cls1.MCS1_MEM[3067].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3068 GTM_gtm_cls1.MCS1_MEM[3068].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3069 GTM_gtm_cls1.MCS1_MEM[3069].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3070 GTM_gtm_cls1.MCS1_MEM[3070].R /* MCS[i] memory region */
#define GTM_gtm_cls1_MCS1_MEM3071 GTM_gtm_cls1.MCS1_MEM[3071].R /* MCS[i] memory region */

/* GTM_gtm_cls2 */
#define GTM_gtm_cls2_TIM2_CH0_GPR0 GTM_gtm_cls2.TIM2_CH0_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH0_GPR1 GTM_gtm_cls2.TIM2_CH0_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH0_CNT GTM_gtm_cls2.TIM2_CH0_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls2_TIM2_CH0_ECNT GTM_gtm_cls2.TIM2_CH0_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls2_TIM2_CH0_CNTS GTM_gtm_cls2.TIM2_CH0_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls2_TIM2_CH0_TDUC GTM_gtm_cls2.TIM2_CH0_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls2_TIM2_CH0_TDUV GTM_gtm_cls2.TIM2_CH0_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls2_TIM2_CH0_FLT_RE GTM_gtm_cls2.TIM2_CH0_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls2_TIM2_CH0_FLT_FE GTM_gtm_cls2.TIM2_CH0_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls2_TIM2_CH0_CTRL GTM_gtm_cls2.TIM2_CH0_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls2_TIM2_CH0_ECTRL GTM_gtm_cls2.TIM2_CH0_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls2_TIM2_CH0_IRQ_NOTIFY GTM_gtm_cls2.TIM2_CH0_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_TIM2_CH0_IRQ_EN GTM_gtm_cls2.TIM2_CH0_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH0_IRQ_FORCINT GTM_gtm_cls2.TIM2_CH0_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls2_TIM2_CH0_IRQ_MODE GTM_gtm_cls2.TIM2_CH0_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_TIM2_CH0_EIRQ_EN GTM_gtm_cls2.TIM2_CH0_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH1_GPR0 GTM_gtm_cls2.TIM2_CH1_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH1_GPR1 GTM_gtm_cls2.TIM2_CH1_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH1_CNT GTM_gtm_cls2.TIM2_CH1_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls2_TIM2_CH1_ECNT GTM_gtm_cls2.TIM2_CH1_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls2_TIM2_CH1_CNTS GTM_gtm_cls2.TIM2_CH1_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls2_TIM2_CH1_TDUC GTM_gtm_cls2.TIM2_CH1_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls2_TIM2_CH1_TDUV GTM_gtm_cls2.TIM2_CH1_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls2_TIM2_CH1_FLT_RE GTM_gtm_cls2.TIM2_CH1_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls2_TIM2_CH1_FLT_FE GTM_gtm_cls2.TIM2_CH1_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls2_TIM2_CH1_CTRL GTM_gtm_cls2.TIM2_CH1_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls2_TIM2_CH1_ECTRL GTM_gtm_cls2.TIM2_CH1_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls2_TIM2_CH1_IRQ_NOTIFY GTM_gtm_cls2.TIM2_CH1_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_TIM2_CH1_IRQ_EN GTM_gtm_cls2.TIM2_CH1_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH1_IRQ_FORCINT GTM_gtm_cls2.TIM2_CH1_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls2_TIM2_CH1_IRQ_MODE GTM_gtm_cls2.TIM2_CH1_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_TIM2_CH1_EIRQ_EN GTM_gtm_cls2.TIM2_CH1_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH2_GPR0 GTM_gtm_cls2.TIM2_CH2_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH2_GPR1 GTM_gtm_cls2.TIM2_CH2_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH2_CNT GTM_gtm_cls2.TIM2_CH2_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls2_TIM2_CH2_ECNT GTM_gtm_cls2.TIM2_CH2_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls2_TIM2_CH2_CNTS GTM_gtm_cls2.TIM2_CH2_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls2_TIM2_CH2_TDUC GTM_gtm_cls2.TIM2_CH2_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls2_TIM2_CH2_TDUV GTM_gtm_cls2.TIM2_CH2_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls2_TIM2_CH2_FLT_RE GTM_gtm_cls2.TIM2_CH2_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls2_TIM2_CH2_FLT_FE GTM_gtm_cls2.TIM2_CH2_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls2_TIM2_CH2_CTRL GTM_gtm_cls2.TIM2_CH2_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls2_TIM2_CH2_ECTRL GTM_gtm_cls2.TIM2_CH2_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls2_TIM2_CH2_IRQ_NOTIFY GTM_gtm_cls2.TIM2_CH2_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_TIM2_CH2_IRQ_EN GTM_gtm_cls2.TIM2_CH2_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH2_IRQ_FORCINT GTM_gtm_cls2.TIM2_CH2_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls2_TIM2_CH2_IRQ_MODE GTM_gtm_cls2.TIM2_CH2_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_TIM2_CH2_EIRQ_EN GTM_gtm_cls2.TIM2_CH2_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH3_GPR0 GTM_gtm_cls2.TIM2_CH3_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH3_GPR1 GTM_gtm_cls2.TIM2_CH3_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH3_CNT GTM_gtm_cls2.TIM2_CH3_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls2_TIM2_CH3_ECNT GTM_gtm_cls2.TIM2_CH3_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls2_TIM2_CH3_CNTS GTM_gtm_cls2.TIM2_CH3_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls2_TIM2_CH3_TDUC GTM_gtm_cls2.TIM2_CH3_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls2_TIM2_CH3_TDUV GTM_gtm_cls2.TIM2_CH3_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls2_TIM2_CH3_FLT_RE GTM_gtm_cls2.TIM2_CH3_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls2_TIM2_CH3_FLT_FE GTM_gtm_cls2.TIM2_CH3_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls2_TIM2_CH3_CTRL GTM_gtm_cls2.TIM2_CH3_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls2_TIM2_CH3_ECTRL GTM_gtm_cls2.TIM2_CH3_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls2_TIM2_CH3_IRQ_NOTIFY GTM_gtm_cls2.TIM2_CH3_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_TIM2_CH3_IRQ_EN GTM_gtm_cls2.TIM2_CH3_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH3_IRQ_FORCINT GTM_gtm_cls2.TIM2_CH3_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls2_TIM2_CH3_IRQ_MODE GTM_gtm_cls2.TIM2_CH3_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_TIM2_CH3_EIRQ_EN GTM_gtm_cls2.TIM2_CH3_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH4_GPR0 GTM_gtm_cls2.TIM2_CH4_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH4_GPR1 GTM_gtm_cls2.TIM2_CH4_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH4_CNT GTM_gtm_cls2.TIM2_CH4_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls2_TIM2_CH4_ECNT GTM_gtm_cls2.TIM2_CH4_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls2_TIM2_CH4_CNTS GTM_gtm_cls2.TIM2_CH4_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls2_TIM2_CH4_TDUC GTM_gtm_cls2.TIM2_CH4_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls2_TIM2_CH4_TDUV GTM_gtm_cls2.TIM2_CH4_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls2_TIM2_CH4_FLT_RE GTM_gtm_cls2.TIM2_CH4_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls2_TIM2_CH4_FLT_FE GTM_gtm_cls2.TIM2_CH4_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls2_TIM2_CH4_CTRL GTM_gtm_cls2.TIM2_CH4_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls2_TIM2_CH4_ECTRL GTM_gtm_cls2.TIM2_CH4_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls2_TIM2_CH4_IRQ_NOTIFY GTM_gtm_cls2.TIM2_CH4_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_TIM2_CH4_IRQ_EN GTM_gtm_cls2.TIM2_CH4_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH4_IRQ_FORCINT GTM_gtm_cls2.TIM2_CH4_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls2_TIM2_CH4_IRQ_MODE GTM_gtm_cls2.TIM2_CH4_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_TIM2_CH4_EIRQ_EN GTM_gtm_cls2.TIM2_CH4_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH5_GPR0 GTM_gtm_cls2.TIM2_CH5_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH5_GPR1 GTM_gtm_cls2.TIM2_CH5_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH5_CNT GTM_gtm_cls2.TIM2_CH5_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls2_TIM2_CH5_ECNT GTM_gtm_cls2.TIM2_CH5_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls2_TIM2_CH5_CNTS GTM_gtm_cls2.TIM2_CH5_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls2_TIM2_CH5_TDUC GTM_gtm_cls2.TIM2_CH5_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls2_TIM2_CH5_TDUV GTM_gtm_cls2.TIM2_CH5_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls2_TIM2_CH5_FLT_RE GTM_gtm_cls2.TIM2_CH5_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls2_TIM2_CH5_FLT_FE GTM_gtm_cls2.TIM2_CH5_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls2_TIM2_CH5_CTRL GTM_gtm_cls2.TIM2_CH5_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls2_TIM2_CH5_ECTRL GTM_gtm_cls2.TIM2_CH5_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls2_TIM2_CH5_IRQ_NOTIFY GTM_gtm_cls2.TIM2_CH5_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_TIM2_CH5_IRQ_EN GTM_gtm_cls2.TIM2_CH5_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH5_IRQ_FORCINT GTM_gtm_cls2.TIM2_CH5_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls2_TIM2_CH5_IRQ_MODE GTM_gtm_cls2.TIM2_CH5_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_TIM2_CH5_EIRQ_EN GTM_gtm_cls2.TIM2_CH5_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH6_GPR0 GTM_gtm_cls2.TIM2_CH6_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH6_GPR1 GTM_gtm_cls2.TIM2_CH6_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH6_CNT GTM_gtm_cls2.TIM2_CH6_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls2_TIM2_CH6_ECNT GTM_gtm_cls2.TIM2_CH6_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls2_TIM2_CH6_CNTS GTM_gtm_cls2.TIM2_CH6_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls2_TIM2_CH6_TDUC GTM_gtm_cls2.TIM2_CH6_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls2_TIM2_CH6_TDUV GTM_gtm_cls2.TIM2_CH6_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls2_TIM2_CH6_FLT_RE GTM_gtm_cls2.TIM2_CH6_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls2_TIM2_CH6_FLT_FE GTM_gtm_cls2.TIM2_CH6_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls2_TIM2_CH6_CTRL GTM_gtm_cls2.TIM2_CH6_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls2_TIM2_CH6_ECTRL GTM_gtm_cls2.TIM2_CH6_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls2_TIM2_CH6_IRQ_NOTIFY GTM_gtm_cls2.TIM2_CH6_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_TIM2_CH6_IRQ_EN GTM_gtm_cls2.TIM2_CH6_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH6_IRQ_FORCINT GTM_gtm_cls2.TIM2_CH6_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls2_TIM2_CH6_IRQ_MODE GTM_gtm_cls2.TIM2_CH6_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_TIM2_CH6_EIRQ_EN GTM_gtm_cls2.TIM2_CH6_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH7_GPR0 GTM_gtm_cls2.TIM2_CH7_GPR0.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH7_GPR1 GTM_gtm_cls2.TIM2_CH7_GPR1.R /* TIM[i] channel [x] general purpose 0 register */
#define GTM_gtm_cls2_TIM2_CH7_CNT GTM_gtm_cls2.TIM2_CH7_CNT.R /* TIM[i] channel [x] SMU counter register */
#define GTM_gtm_cls2_TIM2_CH7_ECNT GTM_gtm_cls2.TIM2_CH7_ECNT.R /* TIM[i] channel [x] SMU edge counter register */
#define GTM_gtm_cls2_TIM2_CH7_CNTS GTM_gtm_cls2.TIM2_CH7_CNTS.R /* TIM[i] channel [x] SMU shadow counter register */
#define GTM_gtm_cls2_TIM2_CH7_TDUC GTM_gtm_cls2.TIM2_CH7_TDUC.R /* TIM[i] channel [x] TDU counter register */
#define GTM_gtm_cls2_TIM2_CH7_TDUV GTM_gtm_cls2.TIM2_CH7_TDUV.R /* TIM[i] channel [x] TDU control register */
#define GTM_gtm_cls2_TIM2_CH7_FLT_RE GTM_gtm_cls2.TIM2_CH7_FLT_RE.R /* TIM[i] channel [x] filter parameter 0 register */
#define GTM_gtm_cls2_TIM2_CH7_FLT_FE GTM_gtm_cls2.TIM2_CH7_FLT_FE.R /* TIM[i] channel [x] filter parameter 1 register */
#define GTM_gtm_cls2_TIM2_CH7_CTRL GTM_gtm_cls2.TIM2_CH7_CTRL.R /* TIM[i] channel [x] control register */
#define GTM_gtm_cls2_TIM2_CH7_ECTRL GTM_gtm_cls2.TIM2_CH7_ECTRL.R /* TIM[i] channel [x] extended control register */
#define GTM_gtm_cls2_TIM2_CH7_IRQ_NOTIFY GTM_gtm_cls2.TIM2_CH7_IRQ_NOTIFY.R /* TIM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_TIM2_CH7_IRQ_EN GTM_gtm_cls2.TIM2_CH7_IRQ_EN.R /* TIM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_TIM2_CH7_IRQ_FORCINT GTM_gtm_cls2.TIM2_CH7_IRQ_FORCINT.R /* TIM[i] channel [x] force interrupt register */
#define GTM_gtm_cls2_TIM2_CH7_IRQ_MODE GTM_gtm_cls2.TIM2_CH7_IRQ_MODE.R /* TIM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_TIM2_CH7_EIRQ_EN GTM_gtm_cls2.TIM2_CH7_EIRQ_EN.R /* TIM[i] channel [x] error interrupt enable register */
#define GTM_gtm_cls2_TIM2_INP_VAL GTM_gtm_cls2.TIM2_INP_VAL.R /* TIM[i] input value observation register */
#define GTM_gtm_cls2_TIM2_IN_SRC GTM_gtm_cls2.TIM2_IN_SRC.R /* TIM[i] AUX IN source selection register */
#define GTM_gtm_cls2_TIM2_RST GTM_gtm_cls2.TIM2_RST.R      /* TIM[i] global software reset register */
#define GTM_gtm_cls2_ATOM2_CH0_RDADDR GTM_gtm_cls2.ATOM2_CH0_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls2_ATOM2_CH0_CTRL GTM_gtm_cls2.ATOM2_CH0_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls2_ATOM2_CH0_SR0 GTM_gtm_cls2.ATOM2_CH0_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH0_SR1 GTM_gtm_cls2.ATOM2_CH0_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH0_CM0 GTM_gtm_cls2.ATOM2_CH0_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH0_CM1 GTM_gtm_cls2.ATOM2_CH0_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH0_CN0 GTM_gtm_cls2.ATOM2_CH0_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls2_ATOM2_CH0_STAT GTM_gtm_cls2.ATOM2_CH0_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls2_ATOM2_CH0_IRQ_NOTIFY GTM_gtm_cls2.ATOM2_CH0_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_ATOM2_CH0_IRQ_EN GTM_gtm_cls2.ATOM2_CH0_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_ATOM2_CH0_IRQ_FORCINT GTM_gtm_cls2.ATOM2_CH0_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls2_ATOM2_CH0_IRQ_MODE GTM_gtm_cls2.ATOM2_CH0_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_ATOM2_CH0_CTRL2 GTM_gtm_cls2.ATOM2_CH0_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls2_ATOM2_CH0_CTRL_SR GTM_gtm_cls2.ATOM2_CH0_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls2_ATOM2_CH1_RDADDR GTM_gtm_cls2.ATOM2_CH1_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls2_ATOM2_CH1_CTRL GTM_gtm_cls2.ATOM2_CH1_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls2_ATOM2_CH1_SR0 GTM_gtm_cls2.ATOM2_CH1_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH1_SR1 GTM_gtm_cls2.ATOM2_CH1_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH1_CM0 GTM_gtm_cls2.ATOM2_CH1_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH1_CM1 GTM_gtm_cls2.ATOM2_CH1_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH1_CN0 GTM_gtm_cls2.ATOM2_CH1_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls2_ATOM2_CH1_STAT GTM_gtm_cls2.ATOM2_CH1_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls2_ATOM2_CH1_IRQ_NOTIFY GTM_gtm_cls2.ATOM2_CH1_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_ATOM2_CH1_IRQ_EN GTM_gtm_cls2.ATOM2_CH1_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_ATOM2_CH1_IRQ_FORCINT GTM_gtm_cls2.ATOM2_CH1_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls2_ATOM2_CH1_IRQ_MODE GTM_gtm_cls2.ATOM2_CH1_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_ATOM2_CH1_CTRL2 GTM_gtm_cls2.ATOM2_CH1_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls2_ATOM2_CH1_CTRL_SR GTM_gtm_cls2.ATOM2_CH1_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls2_ATOM2_CH2_RDADDR GTM_gtm_cls2.ATOM2_CH2_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls2_ATOM2_CH2_CTRL GTM_gtm_cls2.ATOM2_CH2_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls2_ATOM2_CH2_SR0 GTM_gtm_cls2.ATOM2_CH2_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH2_SR1 GTM_gtm_cls2.ATOM2_CH2_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH2_CM0 GTM_gtm_cls2.ATOM2_CH2_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH2_CM1 GTM_gtm_cls2.ATOM2_CH2_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH2_CN0 GTM_gtm_cls2.ATOM2_CH2_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls2_ATOM2_CH2_STAT GTM_gtm_cls2.ATOM2_CH2_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls2_ATOM2_CH2_IRQ_NOTIFY GTM_gtm_cls2.ATOM2_CH2_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_ATOM2_CH2_IRQ_EN GTM_gtm_cls2.ATOM2_CH2_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_ATOM2_CH2_IRQ_FORCINT GTM_gtm_cls2.ATOM2_CH2_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls2_ATOM2_CH2_IRQ_MODE GTM_gtm_cls2.ATOM2_CH2_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_ATOM2_CH2_CTRL2 GTM_gtm_cls2.ATOM2_CH2_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls2_ATOM2_CH2_CTRL_SR GTM_gtm_cls2.ATOM2_CH2_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls2_ATOM2_CH3_RDADDR GTM_gtm_cls2.ATOM2_CH3_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls2_ATOM2_CH3_CTRL GTM_gtm_cls2.ATOM2_CH3_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls2_ATOM2_CH3_SR0 GTM_gtm_cls2.ATOM2_CH3_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH3_SR1 GTM_gtm_cls2.ATOM2_CH3_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH3_CM0 GTM_gtm_cls2.ATOM2_CH3_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH3_CM1 GTM_gtm_cls2.ATOM2_CH3_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH3_CN0 GTM_gtm_cls2.ATOM2_CH3_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls2_ATOM2_CH3_STAT GTM_gtm_cls2.ATOM2_CH3_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls2_ATOM2_CH3_IRQ_NOTIFY GTM_gtm_cls2.ATOM2_CH3_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_ATOM2_CH3_IRQ_EN GTM_gtm_cls2.ATOM2_CH3_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_ATOM2_CH3_IRQ_FORCINT GTM_gtm_cls2.ATOM2_CH3_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls2_ATOM2_CH3_IRQ_MODE GTM_gtm_cls2.ATOM2_CH3_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_ATOM2_CH3_CTRL2 GTM_gtm_cls2.ATOM2_CH3_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls2_ATOM2_CH3_CTRL_SR GTM_gtm_cls2.ATOM2_CH3_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls2_ATOM2_CH4_RDADDR GTM_gtm_cls2.ATOM2_CH4_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls2_ATOM2_CH4_CTRL GTM_gtm_cls2.ATOM2_CH4_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls2_ATOM2_CH4_SR0 GTM_gtm_cls2.ATOM2_CH4_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH4_SR1 GTM_gtm_cls2.ATOM2_CH4_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH4_CM0 GTM_gtm_cls2.ATOM2_CH4_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH4_CM1 GTM_gtm_cls2.ATOM2_CH4_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH4_CN0 GTM_gtm_cls2.ATOM2_CH4_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls2_ATOM2_CH4_STAT GTM_gtm_cls2.ATOM2_CH4_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls2_ATOM2_CH4_IRQ_NOTIFY GTM_gtm_cls2.ATOM2_CH4_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_ATOM2_CH4_IRQ_EN GTM_gtm_cls2.ATOM2_CH4_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_ATOM2_CH4_IRQ_FORCINT GTM_gtm_cls2.ATOM2_CH4_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls2_ATOM2_CH4_IRQ_MODE GTM_gtm_cls2.ATOM2_CH4_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_ATOM2_CH4_CTRL2 GTM_gtm_cls2.ATOM2_CH4_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls2_ATOM2_CH4_CTRL_SR GTM_gtm_cls2.ATOM2_CH4_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls2_ATOM2_CH5_RDADDR GTM_gtm_cls2.ATOM2_CH5_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls2_ATOM2_CH5_CTRL GTM_gtm_cls2.ATOM2_CH5_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls2_ATOM2_CH5_SR0 GTM_gtm_cls2.ATOM2_CH5_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH5_SR1 GTM_gtm_cls2.ATOM2_CH5_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH5_CM0 GTM_gtm_cls2.ATOM2_CH5_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH5_CM1 GTM_gtm_cls2.ATOM2_CH5_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH5_CN0 GTM_gtm_cls2.ATOM2_CH5_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls2_ATOM2_CH5_STAT GTM_gtm_cls2.ATOM2_CH5_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls2_ATOM2_CH5_IRQ_NOTIFY GTM_gtm_cls2.ATOM2_CH5_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_ATOM2_CH5_IRQ_EN GTM_gtm_cls2.ATOM2_CH5_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_ATOM2_CH5_IRQ_FORCINT GTM_gtm_cls2.ATOM2_CH5_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls2_ATOM2_CH5_IRQ_MODE GTM_gtm_cls2.ATOM2_CH5_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_ATOM2_CH5_CTRL2 GTM_gtm_cls2.ATOM2_CH5_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls2_ATOM2_CH5_CTRL_SR GTM_gtm_cls2.ATOM2_CH5_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls2_ATOM2_CH6_RDADDR GTM_gtm_cls2.ATOM2_CH6_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls2_ATOM2_CH6_CTRL GTM_gtm_cls2.ATOM2_CH6_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls2_ATOM2_CH6_SR0 GTM_gtm_cls2.ATOM2_CH6_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH6_SR1 GTM_gtm_cls2.ATOM2_CH6_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH6_CM0 GTM_gtm_cls2.ATOM2_CH6_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH6_CM1 GTM_gtm_cls2.ATOM2_CH6_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH6_CN0 GTM_gtm_cls2.ATOM2_CH6_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls2_ATOM2_CH6_STAT GTM_gtm_cls2.ATOM2_CH6_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls2_ATOM2_CH6_IRQ_NOTIFY GTM_gtm_cls2.ATOM2_CH6_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_ATOM2_CH6_IRQ_EN GTM_gtm_cls2.ATOM2_CH6_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_ATOM2_CH6_IRQ_FORCINT GTM_gtm_cls2.ATOM2_CH6_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls2_ATOM2_CH6_IRQ_MODE GTM_gtm_cls2.ATOM2_CH6_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_ATOM2_CH6_CTRL2 GTM_gtm_cls2.ATOM2_CH6_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls2_ATOM2_CH6_CTRL_SR GTM_gtm_cls2.ATOM2_CH6_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls2_ATOM2_CH7_RDADDR GTM_gtm_cls2.ATOM2_CH7_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls2_ATOM2_CH7_CTRL GTM_gtm_cls2.ATOM2_CH7_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls2_ATOM2_CH7_SR0 GTM_gtm_cls2.ATOM2_CH7_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH7_SR1 GTM_gtm_cls2.ATOM2_CH7_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls2_ATOM2_CH7_CM0 GTM_gtm_cls2.ATOM2_CH7_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH7_CM1 GTM_gtm_cls2.ATOM2_CH7_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls2_ATOM2_CH7_CN0 GTM_gtm_cls2.ATOM2_CH7_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls2_ATOM2_CH7_STAT GTM_gtm_cls2.ATOM2_CH7_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls2_ATOM2_CH7_IRQ_NOTIFY GTM_gtm_cls2.ATOM2_CH7_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls2_ATOM2_CH7_IRQ_EN GTM_gtm_cls2.ATOM2_CH7_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls2_ATOM2_CH7_IRQ_FORCINT GTM_gtm_cls2.ATOM2_CH7_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls2_ATOM2_CH7_IRQ_MODE GTM_gtm_cls2.ATOM2_CH7_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls2_ATOM2_CH7_CTRL2 GTM_gtm_cls2.ATOM2_CH7_CTRL2.R /* ATOM[i] channel [x] control2 register */
#define GTM_gtm_cls2_ATOM2_CH7_CTRL_SR GTM_gtm_cls2.ATOM2_CH7_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls2_ATOM2_AGC_GLB_CTRL GTM_gtm_cls2.ATOM2_AGC_GLB_CTRL.R /* ATOM[i] AGC global control register */
#define GTM_gtm_cls2_ATOM2_AGC_ENDIS_CTRL GTM_gtm_cls2.ATOM2_AGC_ENDIS_CTRL.R /* ATOM[i] AGC enable/disable control register */
#define GTM_gtm_cls2_ATOM2_AGC_ENDIS_STAT GTM_gtm_cls2.ATOM2_AGC_ENDIS_STAT.R /* ATOM[i] AGC enable/disable status register */
#define GTM_gtm_cls2_ATOM2_AGC_ACT_TB GTM_gtm_cls2.ATOM2_AGC_ACT_TB.R /* ATOM[i] AGC action time base register */
#define GTM_gtm_cls2_ATOM2_AGC_OUTEN_CTRL GTM_gtm_cls2.ATOM2_AGC_OUTEN_CTRL.R /* ATOM[i] AGC output enable control register */
#define GTM_gtm_cls2_ATOM2_AGC_OUTEN_STAT GTM_gtm_cls2.ATOM2_AGC_OUTEN_STAT.R /* ATOM[i] AGC output enable status register */
#define GTM_gtm_cls2_ATOM2_AGC_FUPD_CTRL GTM_gtm_cls2.ATOM2_AGC_FUPD_CTRL.R /* ATOM[i] AGC force update control register */
#define GTM_gtm_cls2_ATOM2_AGC_INT_TRIG GTM_gtm_cls2.ATOM2_AGC_INT_TRIG.R /* ATOM[i] AGC internal trigger control register */
#define GTM_gtm_cls2_MCS2_CH0_R0 GTM_gtm_cls2.MCS2_CH0_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH0_R1 GTM_gtm_cls2.MCS2_CH0_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH0_R2 GTM_gtm_cls2.MCS2_CH0_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH0_R3 GTM_gtm_cls2.MCS2_CH0_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH0_R4 GTM_gtm_cls2.MCS2_CH0_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH0_R5 GTM_gtm_cls2.MCS2_CH0_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH0_R6 GTM_gtm_cls2.MCS2_CH0_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH0_R7 GTM_gtm_cls2.MCS2_CH0_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH0_CTRL GTM_gtm_cls2.MCS2_CH0_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls2_MCS2_CH0_ACB GTM_gtm_cls2.MCS2_CH0_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls2_MCS2_CH0_MHB GTM_gtm_cls2.MCS2_CH0_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls2_MCS2_CH0_PC GTM_gtm_cls2.MCS2_CH0_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls2_MCS2_CH0_IRQ_NOTIFY GTM_gtm_cls2.MCS2_CH0_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls2_MCS2_CH0_IRQ_EN GTM_gtm_cls2.MCS2_CH0_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH0_IRQ_FORCINT GTM_gtm_cls2.MCS2_CH0_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls2_MCS2_CH0_IRQ_MODE GTM_gtm_cls2.MCS2_CH0_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls2_MCS2_CH0_EIRQ_EN GTM_gtm_cls2.MCS2_CH0_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH1_R0 GTM_gtm_cls2.MCS2_CH1_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH1_R1 GTM_gtm_cls2.MCS2_CH1_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH1_R2 GTM_gtm_cls2.MCS2_CH1_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH1_R3 GTM_gtm_cls2.MCS2_CH1_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH1_R4 GTM_gtm_cls2.MCS2_CH1_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH1_R5 GTM_gtm_cls2.MCS2_CH1_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH1_R6 GTM_gtm_cls2.MCS2_CH1_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH1_R7 GTM_gtm_cls2.MCS2_CH1_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH1_CTRL GTM_gtm_cls2.MCS2_CH1_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls2_MCS2_CH1_ACB GTM_gtm_cls2.MCS2_CH1_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls2_MCS2_CH1_MHB GTM_gtm_cls2.MCS2_CH1_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls2_MCS2_CH1_PC GTM_gtm_cls2.MCS2_CH1_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls2_MCS2_CH1_IRQ_NOTIFY GTM_gtm_cls2.MCS2_CH1_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls2_MCS2_CH1_IRQ_EN GTM_gtm_cls2.MCS2_CH1_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH1_IRQ_FORCINT GTM_gtm_cls2.MCS2_CH1_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls2_MCS2_CH1_IRQ_MODE GTM_gtm_cls2.MCS2_CH1_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls2_MCS2_CH1_EIRQ_EN GTM_gtm_cls2.MCS2_CH1_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH2_R0 GTM_gtm_cls2.MCS2_CH2_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH2_R1 GTM_gtm_cls2.MCS2_CH2_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH2_R2 GTM_gtm_cls2.MCS2_CH2_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH2_R3 GTM_gtm_cls2.MCS2_CH2_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH2_R4 GTM_gtm_cls2.MCS2_CH2_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH2_R5 GTM_gtm_cls2.MCS2_CH2_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH2_R6 GTM_gtm_cls2.MCS2_CH2_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH2_R7 GTM_gtm_cls2.MCS2_CH2_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH2_CTRL GTM_gtm_cls2.MCS2_CH2_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls2_MCS2_CH2_ACB GTM_gtm_cls2.MCS2_CH2_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls2_MCS2_CH2_MHB GTM_gtm_cls2.MCS2_CH2_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls2_MCS2_CH2_PC GTM_gtm_cls2.MCS2_CH2_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls2_MCS2_CH2_IRQ_NOTIFY GTM_gtm_cls2.MCS2_CH2_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls2_MCS2_CH2_IRQ_EN GTM_gtm_cls2.MCS2_CH2_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH2_IRQ_FORCINT GTM_gtm_cls2.MCS2_CH2_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls2_MCS2_CH2_IRQ_MODE GTM_gtm_cls2.MCS2_CH2_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls2_MCS2_CH2_EIRQ_EN GTM_gtm_cls2.MCS2_CH2_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH3_R0 GTM_gtm_cls2.MCS2_CH3_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH3_R1 GTM_gtm_cls2.MCS2_CH3_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH3_R2 GTM_gtm_cls2.MCS2_CH3_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH3_R3 GTM_gtm_cls2.MCS2_CH3_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH3_R4 GTM_gtm_cls2.MCS2_CH3_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH3_R5 GTM_gtm_cls2.MCS2_CH3_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH3_R6 GTM_gtm_cls2.MCS2_CH3_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH3_R7 GTM_gtm_cls2.MCS2_CH3_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH3_CTRL GTM_gtm_cls2.MCS2_CH3_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls2_MCS2_CH3_ACB GTM_gtm_cls2.MCS2_CH3_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls2_MCS2_CH3_MHB GTM_gtm_cls2.MCS2_CH3_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls2_MCS2_CH3_PC GTM_gtm_cls2.MCS2_CH3_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls2_MCS2_CH3_IRQ_NOTIFY GTM_gtm_cls2.MCS2_CH3_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls2_MCS2_CH3_IRQ_EN GTM_gtm_cls2.MCS2_CH3_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH3_IRQ_FORCINT GTM_gtm_cls2.MCS2_CH3_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls2_MCS2_CH3_IRQ_MODE GTM_gtm_cls2.MCS2_CH3_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls2_MCS2_CH3_EIRQ_EN GTM_gtm_cls2.MCS2_CH3_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH4_R0 GTM_gtm_cls2.MCS2_CH4_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH4_R1 GTM_gtm_cls2.MCS2_CH4_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH4_R2 GTM_gtm_cls2.MCS2_CH4_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH4_R3 GTM_gtm_cls2.MCS2_CH4_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH4_R4 GTM_gtm_cls2.MCS2_CH4_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH4_R5 GTM_gtm_cls2.MCS2_CH4_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH4_R6 GTM_gtm_cls2.MCS2_CH4_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH4_R7 GTM_gtm_cls2.MCS2_CH4_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH4_CTRL GTM_gtm_cls2.MCS2_CH4_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls2_MCS2_CH4_ACB GTM_gtm_cls2.MCS2_CH4_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls2_MCS2_CH4_MHB GTM_gtm_cls2.MCS2_CH4_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls2_MCS2_CH4_PC GTM_gtm_cls2.MCS2_CH4_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls2_MCS2_CH4_IRQ_NOTIFY GTM_gtm_cls2.MCS2_CH4_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls2_MCS2_CH4_IRQ_EN GTM_gtm_cls2.MCS2_CH4_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH4_IRQ_FORCINT GTM_gtm_cls2.MCS2_CH4_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls2_MCS2_CH4_IRQ_MODE GTM_gtm_cls2.MCS2_CH4_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls2_MCS2_CH4_EIRQ_EN GTM_gtm_cls2.MCS2_CH4_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH5_R0 GTM_gtm_cls2.MCS2_CH5_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH5_R1 GTM_gtm_cls2.MCS2_CH5_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH5_R2 GTM_gtm_cls2.MCS2_CH5_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH5_R3 GTM_gtm_cls2.MCS2_CH5_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH5_R4 GTM_gtm_cls2.MCS2_CH5_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH5_R5 GTM_gtm_cls2.MCS2_CH5_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH5_R6 GTM_gtm_cls2.MCS2_CH5_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH5_R7 GTM_gtm_cls2.MCS2_CH5_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH5_CTRL GTM_gtm_cls2.MCS2_CH5_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls2_MCS2_CH5_ACB GTM_gtm_cls2.MCS2_CH5_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls2_MCS2_CH5_MHB GTM_gtm_cls2.MCS2_CH5_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls2_MCS2_CH5_PC GTM_gtm_cls2.MCS2_CH5_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls2_MCS2_CH5_IRQ_NOTIFY GTM_gtm_cls2.MCS2_CH5_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls2_MCS2_CH5_IRQ_EN GTM_gtm_cls2.MCS2_CH5_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH5_IRQ_FORCINT GTM_gtm_cls2.MCS2_CH5_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls2_MCS2_CH5_IRQ_MODE GTM_gtm_cls2.MCS2_CH5_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls2_MCS2_CH5_EIRQ_EN GTM_gtm_cls2.MCS2_CH5_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH6_R0 GTM_gtm_cls2.MCS2_CH6_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH6_R1 GTM_gtm_cls2.MCS2_CH6_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH6_R2 GTM_gtm_cls2.MCS2_CH6_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH6_R3 GTM_gtm_cls2.MCS2_CH6_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH6_R4 GTM_gtm_cls2.MCS2_CH6_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH6_R5 GTM_gtm_cls2.MCS2_CH6_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH6_R6 GTM_gtm_cls2.MCS2_CH6_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH6_R7 GTM_gtm_cls2.MCS2_CH6_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH6_CTRL GTM_gtm_cls2.MCS2_CH6_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls2_MCS2_CH6_ACB GTM_gtm_cls2.MCS2_CH6_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls2_MCS2_CH6_MHB GTM_gtm_cls2.MCS2_CH6_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls2_MCS2_CH6_PC GTM_gtm_cls2.MCS2_CH6_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls2_MCS2_CH6_IRQ_NOTIFY GTM_gtm_cls2.MCS2_CH6_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls2_MCS2_CH6_IRQ_EN GTM_gtm_cls2.MCS2_CH6_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH6_IRQ_FORCINT GTM_gtm_cls2.MCS2_CH6_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls2_MCS2_CH6_IRQ_MODE GTM_gtm_cls2.MCS2_CH6_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls2_MCS2_CH6_EIRQ_EN GTM_gtm_cls2.MCS2_CH6_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH7_R0 GTM_gtm_cls2.MCS2_CH7_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH7_R1 GTM_gtm_cls2.MCS2_CH7_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH7_R2 GTM_gtm_cls2.MCS2_CH7_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH7_R3 GTM_gtm_cls2.MCS2_CH7_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH7_R4 GTM_gtm_cls2.MCS2_CH7_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH7_R5 GTM_gtm_cls2.MCS2_CH7_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH7_R6 GTM_gtm_cls2.MCS2_CH7_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH7_R7 GTM_gtm_cls2.MCS2_CH7_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls2_MCS2_CH7_CTRL GTM_gtm_cls2.MCS2_CH7_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls2_MCS2_CH7_ACB GTM_gtm_cls2.MCS2_CH7_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls2_MCS2_CH7_MHB GTM_gtm_cls2.MCS2_CH7_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls2_MCS2_CH7_PC GTM_gtm_cls2.MCS2_CH7_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls2_MCS2_CH7_IRQ_NOTIFY GTM_gtm_cls2.MCS2_CH7_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls2_MCS2_CH7_IRQ_EN GTM_gtm_cls2.MCS2_CH7_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls2_MCS2_CH7_IRQ_FORCINT GTM_gtm_cls2.MCS2_CH7_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls2_MCS2_CH7_IRQ_MODE GTM_gtm_cls2.MCS2_CH7_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls2_MCS2_CH7_EIRQ_EN GTM_gtm_cls2.MCS2_CH7_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls2_MCS2_CTRG GTM_gtm_cls2.MCS2_CTRG.R    /* MCS[i] clear trigger control register */
#define GTM_gtm_cls2_MCS2_STRG GTM_gtm_cls2.MCS2_STRG.R    /* MCS[i] set trigger control register */
#define GTM_gtm_cls2_MCS2_CTRL_STAT GTM_gtm_cls2.MCS2_CTRL_STAT.R /* MCS[i] control and status register */
#define GTM_gtm_cls2_MCS2_RESET GTM_gtm_cls2.MCS2_RESET.R  /* MCS[i] reset register */
#define GTM_gtm_cls2_MCS2_CAT GTM_gtm_cls2.MCS2_CAT.R      /* MCS[i] cancel ARU transfer instruction */
#define GTM_gtm_cls2_MCS2_CWT GTM_gtm_cls2.MCS2_CWT.R      /* MCS[i] cancel waiting instruction */
#define GTM_gtm_cls2_MCS2_ERR GTM_gtm_cls2.MCS2_ERR.R      /* MCS[i] error register */
#define GTM_gtm_cls2_MCS2_REG_PROT GTM_gtm_cls2.MCS2_REG_PROT.R /* MCS[i] write protection register */
#define GTM_gtm_cls2_MCS2_SINT_IRQ_NOTIFY GTM_gtm_cls2.MCS2_SINT_IRQ_NOTIFY.R /* MCS[i] shared interrupt notification register */
#define GTM_gtm_cls2_MCS2_SINT_IRQ_EN GTM_gtm_cls2.MCS2_SINT_IRQ_EN.R /* MCS[i] shared interrupt enable register */
#define GTM_gtm_cls2_MCS2_SINT_IRQ_FORCINT GTM_gtm_cls2.MCS2_SINT_IRQ_FORCINT.R /* MCS[i] force shared interrupt register */
#define GTM_gtm_cls2_MCS2_SINT_IRQ_MODE GTM_gtm_cls2.MCS2_SINT_IRQ_MODE.R /* MCS[i] shared interrupt mode configuration register */
#define GTM_gtm_cls2_MCS2_HBP0_CTRL GTM_gtm_cls2.MCS2_HBP0_CTRL.R /* MCS[i] hardware break point h control register */
#define GTM_gtm_cls2_MCS2_HBP0_PATTERN GTM_gtm_cls2.MCS2_HBP0_PATTERN.R /* MCS[i] hardware break point pattern register */
#define GTM_gtm_cls2_MCS2_HBP0_STATUS GTM_gtm_cls2.MCS2_HBP0_STATUS.R /* MCS[i] hardware break point status register */
#define GTM_gtm_cls2_MCS2_HBP0_IRQ_NOTIFY GTM_gtm_cls2.MCS2_HBP0_IRQ_NOTIFY.R /* MCS[i] hardware break point interrupt notification register */
#define GTM_gtm_cls2_MCS2_HBP0_IRQ_EN GTM_gtm_cls2.MCS2_HBP0_IRQ_EN.R /* MCS[i] hardware break point interrupt enable register */
#define GTM_gtm_cls2_MCS2_HBP0_IRQ_FORCINT GTM_gtm_cls2.MCS2_HBP0_IRQ_FORCINT.R /* MCS[i] force hardware break point interrupt register */
#define GTM_gtm_cls2_MCS2_HBP0_IRQ_MODE GTM_gtm_cls2.MCS2_HBP0_IRQ_MODE.R /* MCS[i] break point h interrupt mode configuration register */
#define GTM_gtm_cls2_MCS2_HBP1_CTRL GTM_gtm_cls2.MCS2_HBP1_CTRL.R /* MCS[i] hardware break point h control register */
#define GTM_gtm_cls2_MCS2_HBP1_PATTERN GTM_gtm_cls2.MCS2_HBP1_PATTERN.R /* MCS[i] hardware break point pattern register */
#define GTM_gtm_cls2_MCS2_HBP1_STATUS GTM_gtm_cls2.MCS2_HBP1_STATUS.R /* MCS[i] hardware break point status register */
#define GTM_gtm_cls2_MCS2_HBP1_IRQ_NOTIFY GTM_gtm_cls2.MCS2_HBP1_IRQ_NOTIFY.R /* MCS[i] hardware break point interrupt notification register */
#define GTM_gtm_cls2_MCS2_HBP1_IRQ_EN GTM_gtm_cls2.MCS2_HBP1_IRQ_EN.R /* MCS[i] hardware break point interrupt enable register */
#define GTM_gtm_cls2_MCS2_HBP1_IRQ_FORCINT GTM_gtm_cls2.MCS2_HBP1_IRQ_FORCINT.R /* MCS[i] force hardware break point interrupt register */
#define GTM_gtm_cls2_MCS2_HBP1_IRQ_MODE GTM_gtm_cls2.MCS2_HBP1_IRQ_MODE.R /* MCS[i] break point h interrupt mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH0_CTRL GTM_gtm_cls2.TIO2_G0_CH0_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH0_IRQ_NOTIFY GTM_gtm_cls2.TIO2_G0_CH0_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls2_TIO2_G0_CH0_IRQ_EN GTM_gtm_cls2.TIO2_G0_CH0_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls2_TIO2_G0_CH0_IRQ_FORCINT GTM_gtm_cls2.TIO2_G0_CH0_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls2_TIO2_G0_CH0_IRQ_MODE GTM_gtm_cls2.TIO2_G0_CH0_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH0_CTRL2 GTM_gtm_cls2.TIO2_G0_CH0_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH0_SINST GTM_gtm_cls2.TIO2_G0_CH0_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH0_SCMD GTM_gtm_cls2.TIO2_G0_CH0_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH0_SOP GTM_gtm_cls2.TIO2_G0_CH0_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls2_TIO2_G0_CH0_OINST GTM_gtm_cls2.TIO2_G0_CH0_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH0_OCMD GTM_gtm_cls2.TIO2_G0_CH0_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH0_OOP GTM_gtm_cls2.TIO2_G0_CH0_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls2_TIO2_G0_CH0_SHIFTCNT GTM_gtm_cls2.TIO2_G0_CH0_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls2_TIO2_G0_CH1_CTRL GTM_gtm_cls2.TIO2_G0_CH1_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH1_IRQ_NOTIFY GTM_gtm_cls2.TIO2_G0_CH1_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls2_TIO2_G0_CH1_IRQ_EN GTM_gtm_cls2.TIO2_G0_CH1_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls2_TIO2_G0_CH1_IRQ_FORCINT GTM_gtm_cls2.TIO2_G0_CH1_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls2_TIO2_G0_CH1_IRQ_MODE GTM_gtm_cls2.TIO2_G0_CH1_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH1_CTRL2 GTM_gtm_cls2.TIO2_G0_CH1_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH1_SINST GTM_gtm_cls2.TIO2_G0_CH1_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH1_SCMD GTM_gtm_cls2.TIO2_G0_CH1_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH1_SOP GTM_gtm_cls2.TIO2_G0_CH1_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls2_TIO2_G0_CH1_OINST GTM_gtm_cls2.TIO2_G0_CH1_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH1_OCMD GTM_gtm_cls2.TIO2_G0_CH1_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH1_OOP GTM_gtm_cls2.TIO2_G0_CH1_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls2_TIO2_G0_CH1_SHIFTCNT GTM_gtm_cls2.TIO2_G0_CH1_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls2_TIO2_G0_CH2_CTRL GTM_gtm_cls2.TIO2_G0_CH2_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH2_IRQ_NOTIFY GTM_gtm_cls2.TIO2_G0_CH2_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls2_TIO2_G0_CH2_IRQ_EN GTM_gtm_cls2.TIO2_G0_CH2_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls2_TIO2_G0_CH2_IRQ_FORCINT GTM_gtm_cls2.TIO2_G0_CH2_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls2_TIO2_G0_CH2_IRQ_MODE GTM_gtm_cls2.TIO2_G0_CH2_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH2_CTRL2 GTM_gtm_cls2.TIO2_G0_CH2_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH2_SINST GTM_gtm_cls2.TIO2_G0_CH2_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH2_SCMD GTM_gtm_cls2.TIO2_G0_CH2_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH2_SOP GTM_gtm_cls2.TIO2_G0_CH2_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls2_TIO2_G0_CH2_OINST GTM_gtm_cls2.TIO2_G0_CH2_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH2_OCMD GTM_gtm_cls2.TIO2_G0_CH2_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH2_OOP GTM_gtm_cls2.TIO2_G0_CH2_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls2_TIO2_G0_CH2_SHIFTCNT GTM_gtm_cls2.TIO2_G0_CH2_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls2_TIO2_G0_CH3_CTRL GTM_gtm_cls2.TIO2_G0_CH3_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH3_IRQ_NOTIFY GTM_gtm_cls2.TIO2_G0_CH3_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls2_TIO2_G0_CH3_IRQ_EN GTM_gtm_cls2.TIO2_G0_CH3_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls2_TIO2_G0_CH3_IRQ_FORCINT GTM_gtm_cls2.TIO2_G0_CH3_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls2_TIO2_G0_CH3_IRQ_MODE GTM_gtm_cls2.TIO2_G0_CH3_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH3_CTRL2 GTM_gtm_cls2.TIO2_G0_CH3_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH3_SINST GTM_gtm_cls2.TIO2_G0_CH3_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH3_SCMD GTM_gtm_cls2.TIO2_G0_CH3_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH3_SOP GTM_gtm_cls2.TIO2_G0_CH3_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls2_TIO2_G0_CH3_OINST GTM_gtm_cls2.TIO2_G0_CH3_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH3_OCMD GTM_gtm_cls2.TIO2_G0_CH3_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH3_OOP GTM_gtm_cls2.TIO2_G0_CH3_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls2_TIO2_G0_CH3_SHIFTCNT GTM_gtm_cls2.TIO2_G0_CH3_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls2_TIO2_G0_CH4_CTRL GTM_gtm_cls2.TIO2_G0_CH4_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH4_IRQ_NOTIFY GTM_gtm_cls2.TIO2_G0_CH4_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls2_TIO2_G0_CH4_IRQ_EN GTM_gtm_cls2.TIO2_G0_CH4_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls2_TIO2_G0_CH4_IRQ_FORCINT GTM_gtm_cls2.TIO2_G0_CH4_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls2_TIO2_G0_CH4_IRQ_MODE GTM_gtm_cls2.TIO2_G0_CH4_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH4_CTRL2 GTM_gtm_cls2.TIO2_G0_CH4_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH4_SINST GTM_gtm_cls2.TIO2_G0_CH4_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH4_SCMD GTM_gtm_cls2.TIO2_G0_CH4_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH4_SOP GTM_gtm_cls2.TIO2_G0_CH4_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls2_TIO2_G0_CH4_OINST GTM_gtm_cls2.TIO2_G0_CH4_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH4_OCMD GTM_gtm_cls2.TIO2_G0_CH4_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH4_OOP GTM_gtm_cls2.TIO2_G0_CH4_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls2_TIO2_G0_CH4_SHIFTCNT GTM_gtm_cls2.TIO2_G0_CH4_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls2_TIO2_G0_CH5_CTRL GTM_gtm_cls2.TIO2_G0_CH5_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH5_IRQ_NOTIFY GTM_gtm_cls2.TIO2_G0_CH5_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls2_TIO2_G0_CH5_IRQ_EN GTM_gtm_cls2.TIO2_G0_CH5_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls2_TIO2_G0_CH5_IRQ_FORCINT GTM_gtm_cls2.TIO2_G0_CH5_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls2_TIO2_G0_CH5_IRQ_MODE GTM_gtm_cls2.TIO2_G0_CH5_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH5_CTRL2 GTM_gtm_cls2.TIO2_G0_CH5_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH5_SINST GTM_gtm_cls2.TIO2_G0_CH5_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH5_SCMD GTM_gtm_cls2.TIO2_G0_CH5_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH5_SOP GTM_gtm_cls2.TIO2_G0_CH5_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls2_TIO2_G0_CH5_OINST GTM_gtm_cls2.TIO2_G0_CH5_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH5_OCMD GTM_gtm_cls2.TIO2_G0_CH5_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH5_OOP GTM_gtm_cls2.TIO2_G0_CH5_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls2_TIO2_G0_CH5_SHIFTCNT GTM_gtm_cls2.TIO2_G0_CH5_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls2_TIO2_G0_CH6_CTRL GTM_gtm_cls2.TIO2_G0_CH6_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH6_IRQ_NOTIFY GTM_gtm_cls2.TIO2_G0_CH6_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls2_TIO2_G0_CH6_IRQ_EN GTM_gtm_cls2.TIO2_G0_CH6_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls2_TIO2_G0_CH6_IRQ_FORCINT GTM_gtm_cls2.TIO2_G0_CH6_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls2_TIO2_G0_CH6_IRQ_MODE GTM_gtm_cls2.TIO2_G0_CH6_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH6_CTRL2 GTM_gtm_cls2.TIO2_G0_CH6_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH6_SINST GTM_gtm_cls2.TIO2_G0_CH6_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH6_SCMD GTM_gtm_cls2.TIO2_G0_CH6_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH6_SOP GTM_gtm_cls2.TIO2_G0_CH6_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls2_TIO2_G0_CH6_OINST GTM_gtm_cls2.TIO2_G0_CH6_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH6_OCMD GTM_gtm_cls2.TIO2_G0_CH6_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH6_OOP GTM_gtm_cls2.TIO2_G0_CH6_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls2_TIO2_G0_CH6_SHIFTCNT GTM_gtm_cls2.TIO2_G0_CH6_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls2_TIO2_G0_CH7_CTRL GTM_gtm_cls2.TIO2_G0_CH7_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH7_IRQ_NOTIFY GTM_gtm_cls2.TIO2_G0_CH7_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls2_TIO2_G0_CH7_IRQ_EN GTM_gtm_cls2.TIO2_G0_CH7_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls2_TIO2_G0_CH7_IRQ_FORCINT GTM_gtm_cls2.TIO2_G0_CH7_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls2_TIO2_G0_CH7_IRQ_MODE GTM_gtm_cls2.TIO2_G0_CH7_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls2_TIO2_G0_CH7_CTRL2 GTM_gtm_cls2.TIO2_G0_CH7_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls2_TIO2_G0_CH7_SINST GTM_gtm_cls2.TIO2_G0_CH7_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH7_SCMD GTM_gtm_cls2.TIO2_G0_CH7_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls2_TIO2_G0_CH7_SOP GTM_gtm_cls2.TIO2_G0_CH7_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls2_TIO2_G0_CH7_OINST GTM_gtm_cls2.TIO2_G0_CH7_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH7_OCMD GTM_gtm_cls2.TIO2_G0_CH7_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls2_TIO2_G0_CH7_OOP GTM_gtm_cls2.TIO2_G0_CH7_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls2_TIO2_G0_CH7_SHIFTCNT GTM_gtm_cls2.TIO2_G0_CH7_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls2_TIO2_G0_ISEL0_CTRL1 GTM_gtm_cls2.TIO2_G0_ISEL0_CTRL1.R /* TIO[i] input selection register 1 */
#define GTM_gtm_cls2_TIO2_G0_ISEL0_CTRL2 GTM_gtm_cls2.TIO2_G0_ISEL0_CTRL2.R /* TIO[i] input selection register 2 */
#define GTM_gtm_cls2_TIO2_G0_ISEL1_CTRL1 GTM_gtm_cls2.TIO2_G0_ISEL1_CTRL1.R /* TIO[i] input selection register 1 */
#define GTM_gtm_cls2_TIO2_G0_ISEL1_CTRL2 GTM_gtm_cls2.TIO2_G0_ISEL1_CTRL2.R /* TIO[i] input selection register 2 */
#define GTM_gtm_cls2_TIO2_G0_OP_USAGE GTM_gtm_cls2.TIO2_G0_OP_USAGE.R /* TIO[i] operand usage selection register */
#define GTM_gtm_cls2_TIO2_S  GTM_gtm_cls2.TIO2_S.R         /* TIO[i] signal sampling register */
#define GTM_gtm_cls2_TIO2_O  GTM_gtm_cls2.TIO2_O.R         /* TIO[i] output register */
#define GTM_gtm_cls2_TIO2_ENDIS GTM_gtm_cls2.TIO2_ENDIS.R  /* TIO[i] enable/disable register */
#define GTM_gtm_cls2_TIO2_INVERT GTM_gtm_cls2.TIO2_INVERT.R /* TIO[i] signal invert register */
#define GTM_gtm_cls2_TIO2_INPUT_MODE GTM_gtm_cls2.TIO2_INPUT_MODE.R /* TIO[i] input mode register */
#define GTM_gtm_cls2_TIO2_CYCLIC_MODE GTM_gtm_cls2.TIO2_CYCLIC_MODE.R /* TIO[i] cyclic mode register */
#define GTM_gtm_cls2_TIO2_TRIG_OUT_GATE_EN GTM_gtm_cls2.TIO2_TRIG_OUT_GATE_EN.R /* TIO[i] enable Trigger Output, output gating register */
#define GTM_gtm_cls2_TIO2_PLTRIG_OUT_GATE_EN GTM_gtm_cls2.TIO2_PLTRIG_OUT_GATE_EN.R /* TIO[i] enable PL_TRIG_OUT output gating register */
#define GTM_gtm_cls2_TIO2_CS GTM_gtm_cls2.TIO2_CS.R        /* TIO[i] clear signal sampling register */
#define GTM_gtm_cls2_TIO2_CO GTM_gtm_cls2.TIO2_CO.R        /* TIO[i] clear output register */
#define GTM_gtm_cls2_TIO2_CENDIS GTM_gtm_cls2.TIO2_CENDIS.R /* TIO[i] disable register */
#define GTM_gtm_cls2_TIO2_CINVERT GTM_gtm_cls2.TIO2_CINVERT.R /* TIO[i] clear signal invert register */
#define GTM_gtm_cls2_TIO2_CINPUT_MODE GTM_gtm_cls2.TIO2_CINPUT_MODE.R /* TIO[i] disable input mode register */
#define GTM_gtm_cls2_TIO2_CCYCLIC_MODE GTM_gtm_cls2.TIO2_CCYCLIC_MODE.R /* TIO[i] disable cyclic mode register */
#define GTM_gtm_cls2_TIO2_CTRIG_OUT_GATE_EN GTM_gtm_cls2.TIO2_CTRIG_OUT_GATE_EN.R /* TIO[i] clear Trigger Output, output gating register */
#define GTM_gtm_cls2_TIO2_CPLTRIG_OUT_GATE_EN GTM_gtm_cls2.TIO2_CPLTRIG_OUT_GATE_EN.R /* TIO[i] clear PL_TRIG_OUT output gating register */
#define GTM_gtm_cls2_TIO2_SS GTM_gtm_cls2.TIO2_SS.R        /* TIO[i] set signal sampling register */
#define GTM_gtm_cls2_TIO2_SO GTM_gtm_cls2.TIO2_SO.R        /* TIO[i] set output register */
#define GTM_gtm_cls2_TIO2_SENDIS GTM_gtm_cls2.TIO2_SENDIS.R /* TIO[i] enable register */
#define GTM_gtm_cls2_TIO2_SINVERT GTM_gtm_cls2.TIO2_SINVERT.R /* TIO[i] set signal invert register */
#define GTM_gtm_cls2_TIO2_SINPUT_MODE GTM_gtm_cls2.TIO2_SINPUT_MODE.R /* TIO[i] enable input mode register */
#define GTM_gtm_cls2_TIO2_SCYCLIC_MODE GTM_gtm_cls2.TIO2_SCYCLIC_MODE.R /* TIO[i] enable cyclic mode register */
#define GTM_gtm_cls2_TIO2_STRIG_OUT_GATE_EN GTM_gtm_cls2.TIO2_STRIG_OUT_GATE_EN.R /* TIO[i] set Trigger Output, output gating register */
#define GTM_gtm_cls2_TIO2_SPLTRIG_OUT_GATE_EN GTM_gtm_cls2.TIO2_SPLTRIG_OUT_GATE_EN.R /* TIO[i] set PL_TRIG_OUT output gating register */
#define GTM_gtm_cls2_TIO2_IS GTM_gtm_cls2.TIO2_IS.R        /* TIO[i] invert signal sampling register */
#define GTM_gtm_cls2_TIO2_IO GTM_gtm_cls2.TIO2_IO.R        /* TIO[i] invert output register */
#define GTM_gtm_cls2_TIO2_IENDIS GTM_gtm_cls2.TIO2_IENDIS.R /* TIO[i] toggle enable/disable register */
#define GTM_gtm_cls2_TIO2_IINVERT GTM_gtm_cls2.TIO2_IINVERT.R /* TIO[i] toggle signal invert register */
#define GTM_gtm_cls2_TIO2_IINPUT_MODE GTM_gtm_cls2.TIO2_IINPUT_MODE.R /* TIO[i] enable input mode register */
#define GTM_gtm_cls2_TIO2_ICYCLIC_MODE GTM_gtm_cls2.TIO2_ICYCLIC_MODE.R /* TIO[i] enable cyclic mode register */
#define GTM_gtm_cls2_TIO2_FUPD GTM_gtm_cls2.TIO2_FUPD.R    /* TIO[i] force update register */
#define GTM_gtm_cls2_TIO2_HW_CONF GTM_gtm_cls2.TIO2_HW_CONF.R /* TIO[i] configuration register */
#define GTM_gtm_cls2_TIO2_RSEL_CTRL1 GTM_gtm_cls2.TIO2_RSEL_CTRL1.R /* TIO[i] resource selection control register 1 */
#define GTM_gtm_cls2_TIO2_RSEL_CTRL2 GTM_gtm_cls2.TIO2_RSEL_CTRL2.R /* TIO[i] resource selection control register 2 */
#define GTM_gtm_cls2_TIO2_PL_SWRST GTM_gtm_cls2.TIO2_PL_SWRST.R /* TIO[i] software reset for TIO Plus functionality */
#define GTM_gtm_cls2_CCM2_ARP0_CTRL GTM_gtm_cls2.CCM2_ARP0_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP0_PROT GTM_gtm_cls2.CCM2_ARP0_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP1_CTRL GTM_gtm_cls2.CCM2_ARP1_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP1_PROT GTM_gtm_cls2.CCM2_ARP1_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP2_CTRL GTM_gtm_cls2.CCM2_ARP2_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP2_PROT GTM_gtm_cls2.CCM2_ARP2_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP3_CTRL GTM_gtm_cls2.CCM2_ARP3_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP3_PROT GTM_gtm_cls2.CCM2_ARP3_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP4_CTRL GTM_gtm_cls2.CCM2_ARP4_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP4_PROT GTM_gtm_cls2.CCM2_ARP4_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP5_CTRL GTM_gtm_cls2.CCM2_ARP5_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP5_PROT GTM_gtm_cls2.CCM2_ARP5_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP6_CTRL GTM_gtm_cls2.CCM2_ARP6_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP6_PROT GTM_gtm_cls2.CCM2_ARP6_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP7_CTRL GTM_gtm_cls2.CCM2_ARP7_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP7_PROT GTM_gtm_cls2.CCM2_ARP7_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP8_CTRL GTM_gtm_cls2.CCM2_ARP8_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP8_PROT GTM_gtm_cls2.CCM2_ARP8_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_ARP9_CTRL GTM_gtm_cls2.CCM2_ARP9_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls2_CCM2_ARP9_PROT GTM_gtm_cls2.CCM2_ARP9_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls2_CCM2_TIO_G0_OUT GTM_gtm_cls2.CCM2_TIO_G0_OUT.R /* CCM[i] TIO Group 0,1 Output Register */
#define GTM_gtm_cls2_CCM2_HW_CONF2 GTM_gtm_cls2.CCM2_HW_CONF2.R /* CCM[i] 2. Hardware Configuration Register */
#define GTM_gtm_cls2_CCM2_AEIM_STA GTM_gtm_cls2.CCM2_AEIM_STA.R /* CCM[i] MCS Bus Master Status Register */
#define GTM_gtm_cls2_CCM2_HW_CONF GTM_gtm_cls2.CCM2_HW_CONF.R /* CCM[i] Hardware Configuration Register */
#define GTM_gtm_cls2_CCM2_TIM_AUX_IN_SRC GTM_gtm_cls2.CCM2_TIM_AUX_IN_SRC.R /* CCM[i] TIM AUX Input Source Register */
#define GTM_gtm_cls2_CCM2_EXT_CAP_EN GTM_gtm_cls2.CCM2_EXT_CAP_EN.R /* CCM[i] External Capture Enable Register */
#define GTM_gtm_cls2_CCM2_ATOM_OUT GTM_gtm_cls2.CCM2_ATOM_OUT.R /* CCM[i] ATOM Output Register */
#define GTM_gtm_cls2_CCM2_CMU_CLK_CFG GTM_gtm_cls2.CCM2_CMU_CLK_CFG.R /* CCM[i] CMU Clock Configuration Register */
#define GTM_gtm_cls2_CCM2_CFG GTM_gtm_cls2.CCM2_CFG.R      /* CCM[i] Configuration Register */
#define GTM_gtm_cls2_CCM2_PROT GTM_gtm_cls2.CCM2_PROT.R    /* CCM[i] Protection Register */
#define GTM_gtm_cls2_CDTM2_DTM4_CTRL GTM_gtm_cls2.CDTM2_DTM4_CTRL.R /* CDTM[i]_DTM[d] global configuration and control register */
#define GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL1 GTM_gtm_cls2.CDTM2_DTM4_CH_CTRL1.R /* CDTM[i]_DTM[d] channel control register 1 */
#define GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL2 GTM_gtm_cls2.CDTM2_DTM4_CH_CTRL2.R /* CDTM[i]_DTM[d] channel control register 2 */
#define GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL2_SR GTM_gtm_cls2.CDTM2_DTM4_CH_CTRL2_SR.R /* CDTM[i] DTM[j] channel control register 2 shadow */
#define GTM_gtm_cls2_CDTM2_DTM4_PS_CTRL GTM_gtm_cls2.CDTM2_DTM4_PS_CTRL.R /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
#define GTM_gtm_cls2_CDTM2_DTM4_CH0_DTV GTM_gtm_cls2.CDTM2_DTM4_CH_DTV[0].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls2_CDTM2_DTM4_CH1_DTV GTM_gtm_cls2.CDTM2_DTM4_CH_DTV[1].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls2_CDTM2_DTM4_CH2_DTV GTM_gtm_cls2.CDTM2_DTM4_CH_DTV[2].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls2_CDTM2_DTM4_CH3_DTV GTM_gtm_cls2.CDTM2_DTM4_CH_DTV[3].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls2_CDTM2_DTM4_CH_SR GTM_gtm_cls2.CDTM2_DTM4_CH_SR.R /* CDTM[i]_DTM[d] channel shadow register */
#define GTM_gtm_cls2_CDTM2_DTM4_CH_CTRL3 GTM_gtm_cls2.CDTM2_DTM4_CH_CTRL3.R /* CDTM[i]_DTM[d] channel control register 3 */
#define GTM_gtm_cls2_CDTM2_DTM4_CTRL2 GTM_gtm_cls2.CDTM2_DTM4_CTRL2.R /* CDTM[i]_DTM[d] global configuration and control register 2 */
#define GTM_gtm_cls2_CDTM2_DTM4_CH0_DTV_SR GTM_gtm_cls2.CDTM2_DTM4_CH0_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls2_CDTM2_DTM4_CH1_DTV_SR GTM_gtm_cls2.CDTM2_DTM4_CH1_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls2_CDTM2_DTM4_CH2_DTV_SR GTM_gtm_cls2.CDTM2_DTM4_CH2_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls2_CDTM2_DTM4_CH3_DTV_SR GTM_gtm_cls2.CDTM2_DTM4_CH3_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls2_CDTM2_DTM5_CTRL GTM_gtm_cls2.CDTM2_DTM5_CTRL.R /* CDTM[i]_DTM[d] global configuration and control register */
#define GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL1 GTM_gtm_cls2.CDTM2_DTM5_CH_CTRL1.R /* CDTM[i]_DTM[d] channel control register 1 */
#define GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL2 GTM_gtm_cls2.CDTM2_DTM5_CH_CTRL2.R /* CDTM[i]_DTM[d] channel control register 2 */
#define GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL2_SR GTM_gtm_cls2.CDTM2_DTM5_CH_CTRL2_SR.R /* CDTM[i] DTM[j] channel control register 2 shadow */
#define GTM_gtm_cls2_CDTM2_DTM5_PS_CTRL GTM_gtm_cls2.CDTM2_DTM5_PS_CTRL.R /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
#define GTM_gtm_cls2_CDTM2_DTM5_CH0_DTV GTM_gtm_cls2.CDTM2_DTM5_CH_DTV[0].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls2_CDTM2_DTM5_CH1_DTV GTM_gtm_cls2.CDTM2_DTM5_CH_DTV[1].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls2_CDTM2_DTM5_CH2_DTV GTM_gtm_cls2.CDTM2_DTM5_CH_DTV[2].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls2_CDTM2_DTM5_CH3_DTV GTM_gtm_cls2.CDTM2_DTM5_CH_DTV[3].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls2_CDTM2_DTM5_CH_SR GTM_gtm_cls2.CDTM2_DTM5_CH_SR.R /* CDTM[i]_DTM[d] channel shadow register */
#define GTM_gtm_cls2_CDTM2_DTM5_CH_CTRL3 GTM_gtm_cls2.CDTM2_DTM5_CH_CTRL3.R /* CDTM[i]_DTM[d] channel control register 3 */
#define GTM_gtm_cls2_CDTM2_DTM5_CTRL2 GTM_gtm_cls2.CDTM2_DTM5_CTRL2.R /* CDTM[i]_DTM[d] global configuration and control register 2 */
#define GTM_gtm_cls2_CDTM2_DTM5_CH0_DTV_SR GTM_gtm_cls2.CDTM2_DTM5_CH0_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls2_CDTM2_DTM5_CH1_DTV_SR GTM_gtm_cls2.CDTM2_DTM5_CH1_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls2_CDTM2_DTM5_CH2_DTV_SR GTM_gtm_cls2.CDTM2_DTM5_CH2_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls2_CDTM2_DTM5_CH3_DTV_SR GTM_gtm_cls2.CDTM2_DTM5_CH3_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls2_AXIM2_FREE GTM_gtm_cls2.AXIM2_FREE.R  /* AXIM[i] slot allocation status. */
#define GTM_gtm_cls2_AXIM2_REQUEST GTM_gtm_cls2.AXIM2_REQUEST.R /* AXIM[i] slot request (allocation). */
#define GTM_gtm_cls2_AXIM2_RELEASE GTM_gtm_cls2.AXIM2_RELEASE.R /* AXIM[i] slot release (de-allocation). */
#define GTM_gtm_cls2_AXIM2_SLOT0_ADDR_LOW GTM_gtm_cls2.AXIM2_SLOT0_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls2_AXIM2_SLOT0_DATA_LOW GTM_gtm_cls2.AXIM2_SLOT0_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls2_AXIM2_SLOT0_CFG1 GTM_gtm_cls2.AXIM2_SLOT0_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls2_AXIM2_SLOT0_CFG2 GTM_gtm_cls2.AXIM2_SLOT0_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls2_AXIM2_SLOT0_STATUS GTM_gtm_cls2.AXIM2_SLOT0_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls2_AXIM2_SLOT1_ADDR_LOW GTM_gtm_cls2.AXIM2_SLOT1_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls2_AXIM2_SLOT1_DATA_LOW GTM_gtm_cls2.AXIM2_SLOT1_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls2_AXIM2_SLOT1_CFG1 GTM_gtm_cls2.AXIM2_SLOT1_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls2_AXIM2_SLOT1_CFG2 GTM_gtm_cls2.AXIM2_SLOT1_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls2_AXIM2_SLOT1_STATUS GTM_gtm_cls2.AXIM2_SLOT1_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls2_AXIM2_SLOT2_ADDR_LOW GTM_gtm_cls2.AXIM2_SLOT2_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls2_AXIM2_SLOT2_DATA_LOW GTM_gtm_cls2.AXIM2_SLOT2_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls2_AXIM2_SLOT2_CFG1 GTM_gtm_cls2.AXIM2_SLOT2_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls2_AXIM2_SLOT2_CFG2 GTM_gtm_cls2.AXIM2_SLOT2_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls2_AXIM2_SLOT2_STATUS GTM_gtm_cls2.AXIM2_SLOT2_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls2_AXIM2_SLOT3_ADDR_LOW GTM_gtm_cls2.AXIM2_SLOT3_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls2_AXIM2_SLOT3_DATA_LOW GTM_gtm_cls2.AXIM2_SLOT3_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls2_AXIM2_SLOT3_CFG1 GTM_gtm_cls2.AXIM2_SLOT3_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls2_AXIM2_SLOT3_CFG2 GTM_gtm_cls2.AXIM2_SLOT3_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls2_AXIM2_SLOT3_STATUS GTM_gtm_cls2.AXIM2_SLOT3_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls2_MCS2_MEM0 GTM_gtm_cls2.MCS2_MEM[0].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1 GTM_gtm_cls2.MCS2_MEM[1].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2 GTM_gtm_cls2.MCS2_MEM[2].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3 GTM_gtm_cls2.MCS2_MEM[3].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM4 GTM_gtm_cls2.MCS2_MEM[4].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM5 GTM_gtm_cls2.MCS2_MEM[5].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM6 GTM_gtm_cls2.MCS2_MEM[6].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM7 GTM_gtm_cls2.MCS2_MEM[7].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM8 GTM_gtm_cls2.MCS2_MEM[8].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM9 GTM_gtm_cls2.MCS2_MEM[9].R  /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM10 GTM_gtm_cls2.MCS2_MEM[10].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM11 GTM_gtm_cls2.MCS2_MEM[11].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM12 GTM_gtm_cls2.MCS2_MEM[12].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM13 GTM_gtm_cls2.MCS2_MEM[13].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM14 GTM_gtm_cls2.MCS2_MEM[14].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM15 GTM_gtm_cls2.MCS2_MEM[15].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM16 GTM_gtm_cls2.MCS2_MEM[16].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM17 GTM_gtm_cls2.MCS2_MEM[17].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM18 GTM_gtm_cls2.MCS2_MEM[18].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM19 GTM_gtm_cls2.MCS2_MEM[19].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM20 GTM_gtm_cls2.MCS2_MEM[20].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM21 GTM_gtm_cls2.MCS2_MEM[21].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM22 GTM_gtm_cls2.MCS2_MEM[22].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM23 GTM_gtm_cls2.MCS2_MEM[23].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM24 GTM_gtm_cls2.MCS2_MEM[24].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM25 GTM_gtm_cls2.MCS2_MEM[25].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM26 GTM_gtm_cls2.MCS2_MEM[26].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM27 GTM_gtm_cls2.MCS2_MEM[27].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM28 GTM_gtm_cls2.MCS2_MEM[28].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM29 GTM_gtm_cls2.MCS2_MEM[29].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM30 GTM_gtm_cls2.MCS2_MEM[30].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM31 GTM_gtm_cls2.MCS2_MEM[31].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM32 GTM_gtm_cls2.MCS2_MEM[32].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM33 GTM_gtm_cls2.MCS2_MEM[33].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM34 GTM_gtm_cls2.MCS2_MEM[34].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM35 GTM_gtm_cls2.MCS2_MEM[35].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM36 GTM_gtm_cls2.MCS2_MEM[36].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM37 GTM_gtm_cls2.MCS2_MEM[37].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM38 GTM_gtm_cls2.MCS2_MEM[38].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM39 GTM_gtm_cls2.MCS2_MEM[39].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM40 GTM_gtm_cls2.MCS2_MEM[40].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM41 GTM_gtm_cls2.MCS2_MEM[41].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM42 GTM_gtm_cls2.MCS2_MEM[42].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM43 GTM_gtm_cls2.MCS2_MEM[43].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM44 GTM_gtm_cls2.MCS2_MEM[44].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM45 GTM_gtm_cls2.MCS2_MEM[45].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM46 GTM_gtm_cls2.MCS2_MEM[46].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM47 GTM_gtm_cls2.MCS2_MEM[47].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM48 GTM_gtm_cls2.MCS2_MEM[48].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM49 GTM_gtm_cls2.MCS2_MEM[49].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM50 GTM_gtm_cls2.MCS2_MEM[50].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM51 GTM_gtm_cls2.MCS2_MEM[51].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM52 GTM_gtm_cls2.MCS2_MEM[52].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM53 GTM_gtm_cls2.MCS2_MEM[53].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM54 GTM_gtm_cls2.MCS2_MEM[54].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM55 GTM_gtm_cls2.MCS2_MEM[55].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM56 GTM_gtm_cls2.MCS2_MEM[56].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM57 GTM_gtm_cls2.MCS2_MEM[57].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM58 GTM_gtm_cls2.MCS2_MEM[58].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM59 GTM_gtm_cls2.MCS2_MEM[59].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM60 GTM_gtm_cls2.MCS2_MEM[60].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM61 GTM_gtm_cls2.MCS2_MEM[61].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM62 GTM_gtm_cls2.MCS2_MEM[62].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM63 GTM_gtm_cls2.MCS2_MEM[63].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM64 GTM_gtm_cls2.MCS2_MEM[64].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM65 GTM_gtm_cls2.MCS2_MEM[65].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM66 GTM_gtm_cls2.MCS2_MEM[66].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM67 GTM_gtm_cls2.MCS2_MEM[67].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM68 GTM_gtm_cls2.MCS2_MEM[68].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM69 GTM_gtm_cls2.MCS2_MEM[69].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM70 GTM_gtm_cls2.MCS2_MEM[70].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM71 GTM_gtm_cls2.MCS2_MEM[71].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM72 GTM_gtm_cls2.MCS2_MEM[72].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM73 GTM_gtm_cls2.MCS2_MEM[73].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM74 GTM_gtm_cls2.MCS2_MEM[74].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM75 GTM_gtm_cls2.MCS2_MEM[75].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM76 GTM_gtm_cls2.MCS2_MEM[76].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM77 GTM_gtm_cls2.MCS2_MEM[77].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM78 GTM_gtm_cls2.MCS2_MEM[78].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM79 GTM_gtm_cls2.MCS2_MEM[79].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM80 GTM_gtm_cls2.MCS2_MEM[80].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM81 GTM_gtm_cls2.MCS2_MEM[81].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM82 GTM_gtm_cls2.MCS2_MEM[82].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM83 GTM_gtm_cls2.MCS2_MEM[83].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM84 GTM_gtm_cls2.MCS2_MEM[84].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM85 GTM_gtm_cls2.MCS2_MEM[85].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM86 GTM_gtm_cls2.MCS2_MEM[86].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM87 GTM_gtm_cls2.MCS2_MEM[87].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM88 GTM_gtm_cls2.MCS2_MEM[88].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM89 GTM_gtm_cls2.MCS2_MEM[89].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM90 GTM_gtm_cls2.MCS2_MEM[90].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM91 GTM_gtm_cls2.MCS2_MEM[91].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM92 GTM_gtm_cls2.MCS2_MEM[92].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM93 GTM_gtm_cls2.MCS2_MEM[93].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM94 GTM_gtm_cls2.MCS2_MEM[94].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM95 GTM_gtm_cls2.MCS2_MEM[95].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM96 GTM_gtm_cls2.MCS2_MEM[96].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM97 GTM_gtm_cls2.MCS2_MEM[97].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM98 GTM_gtm_cls2.MCS2_MEM[98].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM99 GTM_gtm_cls2.MCS2_MEM[99].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM100 GTM_gtm_cls2.MCS2_MEM[100].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM101 GTM_gtm_cls2.MCS2_MEM[101].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM102 GTM_gtm_cls2.MCS2_MEM[102].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM103 GTM_gtm_cls2.MCS2_MEM[103].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM104 GTM_gtm_cls2.MCS2_MEM[104].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM105 GTM_gtm_cls2.MCS2_MEM[105].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM106 GTM_gtm_cls2.MCS2_MEM[106].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM107 GTM_gtm_cls2.MCS2_MEM[107].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM108 GTM_gtm_cls2.MCS2_MEM[108].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM109 GTM_gtm_cls2.MCS2_MEM[109].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM110 GTM_gtm_cls2.MCS2_MEM[110].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM111 GTM_gtm_cls2.MCS2_MEM[111].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM112 GTM_gtm_cls2.MCS2_MEM[112].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM113 GTM_gtm_cls2.MCS2_MEM[113].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM114 GTM_gtm_cls2.MCS2_MEM[114].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM115 GTM_gtm_cls2.MCS2_MEM[115].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM116 GTM_gtm_cls2.MCS2_MEM[116].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM117 GTM_gtm_cls2.MCS2_MEM[117].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM118 GTM_gtm_cls2.MCS2_MEM[118].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM119 GTM_gtm_cls2.MCS2_MEM[119].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM120 GTM_gtm_cls2.MCS2_MEM[120].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM121 GTM_gtm_cls2.MCS2_MEM[121].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM122 GTM_gtm_cls2.MCS2_MEM[122].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM123 GTM_gtm_cls2.MCS2_MEM[123].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM124 GTM_gtm_cls2.MCS2_MEM[124].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM125 GTM_gtm_cls2.MCS2_MEM[125].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM126 GTM_gtm_cls2.MCS2_MEM[126].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM127 GTM_gtm_cls2.MCS2_MEM[127].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM128 GTM_gtm_cls2.MCS2_MEM[128].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM129 GTM_gtm_cls2.MCS2_MEM[129].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM130 GTM_gtm_cls2.MCS2_MEM[130].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM131 GTM_gtm_cls2.MCS2_MEM[131].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM132 GTM_gtm_cls2.MCS2_MEM[132].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM133 GTM_gtm_cls2.MCS2_MEM[133].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM134 GTM_gtm_cls2.MCS2_MEM[134].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM135 GTM_gtm_cls2.MCS2_MEM[135].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM136 GTM_gtm_cls2.MCS2_MEM[136].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM137 GTM_gtm_cls2.MCS2_MEM[137].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM138 GTM_gtm_cls2.MCS2_MEM[138].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM139 GTM_gtm_cls2.MCS2_MEM[139].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM140 GTM_gtm_cls2.MCS2_MEM[140].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM141 GTM_gtm_cls2.MCS2_MEM[141].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM142 GTM_gtm_cls2.MCS2_MEM[142].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM143 GTM_gtm_cls2.MCS2_MEM[143].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM144 GTM_gtm_cls2.MCS2_MEM[144].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM145 GTM_gtm_cls2.MCS2_MEM[145].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM146 GTM_gtm_cls2.MCS2_MEM[146].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM147 GTM_gtm_cls2.MCS2_MEM[147].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM148 GTM_gtm_cls2.MCS2_MEM[148].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM149 GTM_gtm_cls2.MCS2_MEM[149].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM150 GTM_gtm_cls2.MCS2_MEM[150].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM151 GTM_gtm_cls2.MCS2_MEM[151].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM152 GTM_gtm_cls2.MCS2_MEM[152].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM153 GTM_gtm_cls2.MCS2_MEM[153].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM154 GTM_gtm_cls2.MCS2_MEM[154].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM155 GTM_gtm_cls2.MCS2_MEM[155].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM156 GTM_gtm_cls2.MCS2_MEM[156].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM157 GTM_gtm_cls2.MCS2_MEM[157].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM158 GTM_gtm_cls2.MCS2_MEM[158].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM159 GTM_gtm_cls2.MCS2_MEM[159].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM160 GTM_gtm_cls2.MCS2_MEM[160].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM161 GTM_gtm_cls2.MCS2_MEM[161].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM162 GTM_gtm_cls2.MCS2_MEM[162].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM163 GTM_gtm_cls2.MCS2_MEM[163].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM164 GTM_gtm_cls2.MCS2_MEM[164].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM165 GTM_gtm_cls2.MCS2_MEM[165].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM166 GTM_gtm_cls2.MCS2_MEM[166].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM167 GTM_gtm_cls2.MCS2_MEM[167].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM168 GTM_gtm_cls2.MCS2_MEM[168].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM169 GTM_gtm_cls2.MCS2_MEM[169].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM170 GTM_gtm_cls2.MCS2_MEM[170].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM171 GTM_gtm_cls2.MCS2_MEM[171].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM172 GTM_gtm_cls2.MCS2_MEM[172].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM173 GTM_gtm_cls2.MCS2_MEM[173].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM174 GTM_gtm_cls2.MCS2_MEM[174].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM175 GTM_gtm_cls2.MCS2_MEM[175].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM176 GTM_gtm_cls2.MCS2_MEM[176].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM177 GTM_gtm_cls2.MCS2_MEM[177].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM178 GTM_gtm_cls2.MCS2_MEM[178].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM179 GTM_gtm_cls2.MCS2_MEM[179].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM180 GTM_gtm_cls2.MCS2_MEM[180].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM181 GTM_gtm_cls2.MCS2_MEM[181].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM182 GTM_gtm_cls2.MCS2_MEM[182].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM183 GTM_gtm_cls2.MCS2_MEM[183].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM184 GTM_gtm_cls2.MCS2_MEM[184].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM185 GTM_gtm_cls2.MCS2_MEM[185].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM186 GTM_gtm_cls2.MCS2_MEM[186].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM187 GTM_gtm_cls2.MCS2_MEM[187].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM188 GTM_gtm_cls2.MCS2_MEM[188].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM189 GTM_gtm_cls2.MCS2_MEM[189].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM190 GTM_gtm_cls2.MCS2_MEM[190].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM191 GTM_gtm_cls2.MCS2_MEM[191].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM192 GTM_gtm_cls2.MCS2_MEM[192].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM193 GTM_gtm_cls2.MCS2_MEM[193].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM194 GTM_gtm_cls2.MCS2_MEM[194].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM195 GTM_gtm_cls2.MCS2_MEM[195].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM196 GTM_gtm_cls2.MCS2_MEM[196].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM197 GTM_gtm_cls2.MCS2_MEM[197].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM198 GTM_gtm_cls2.MCS2_MEM[198].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM199 GTM_gtm_cls2.MCS2_MEM[199].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM200 GTM_gtm_cls2.MCS2_MEM[200].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM201 GTM_gtm_cls2.MCS2_MEM[201].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM202 GTM_gtm_cls2.MCS2_MEM[202].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM203 GTM_gtm_cls2.MCS2_MEM[203].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM204 GTM_gtm_cls2.MCS2_MEM[204].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM205 GTM_gtm_cls2.MCS2_MEM[205].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM206 GTM_gtm_cls2.MCS2_MEM[206].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM207 GTM_gtm_cls2.MCS2_MEM[207].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM208 GTM_gtm_cls2.MCS2_MEM[208].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM209 GTM_gtm_cls2.MCS2_MEM[209].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM210 GTM_gtm_cls2.MCS2_MEM[210].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM211 GTM_gtm_cls2.MCS2_MEM[211].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM212 GTM_gtm_cls2.MCS2_MEM[212].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM213 GTM_gtm_cls2.MCS2_MEM[213].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM214 GTM_gtm_cls2.MCS2_MEM[214].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM215 GTM_gtm_cls2.MCS2_MEM[215].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM216 GTM_gtm_cls2.MCS2_MEM[216].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM217 GTM_gtm_cls2.MCS2_MEM[217].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM218 GTM_gtm_cls2.MCS2_MEM[218].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM219 GTM_gtm_cls2.MCS2_MEM[219].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM220 GTM_gtm_cls2.MCS2_MEM[220].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM221 GTM_gtm_cls2.MCS2_MEM[221].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM222 GTM_gtm_cls2.MCS2_MEM[222].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM223 GTM_gtm_cls2.MCS2_MEM[223].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM224 GTM_gtm_cls2.MCS2_MEM[224].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM225 GTM_gtm_cls2.MCS2_MEM[225].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM226 GTM_gtm_cls2.MCS2_MEM[226].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM227 GTM_gtm_cls2.MCS2_MEM[227].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM228 GTM_gtm_cls2.MCS2_MEM[228].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM229 GTM_gtm_cls2.MCS2_MEM[229].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM230 GTM_gtm_cls2.MCS2_MEM[230].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM231 GTM_gtm_cls2.MCS2_MEM[231].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM232 GTM_gtm_cls2.MCS2_MEM[232].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM233 GTM_gtm_cls2.MCS2_MEM[233].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM234 GTM_gtm_cls2.MCS2_MEM[234].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM235 GTM_gtm_cls2.MCS2_MEM[235].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM236 GTM_gtm_cls2.MCS2_MEM[236].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM237 GTM_gtm_cls2.MCS2_MEM[237].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM238 GTM_gtm_cls2.MCS2_MEM[238].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM239 GTM_gtm_cls2.MCS2_MEM[239].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM240 GTM_gtm_cls2.MCS2_MEM[240].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM241 GTM_gtm_cls2.MCS2_MEM[241].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM242 GTM_gtm_cls2.MCS2_MEM[242].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM243 GTM_gtm_cls2.MCS2_MEM[243].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM244 GTM_gtm_cls2.MCS2_MEM[244].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM245 GTM_gtm_cls2.MCS2_MEM[245].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM246 GTM_gtm_cls2.MCS2_MEM[246].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM247 GTM_gtm_cls2.MCS2_MEM[247].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM248 GTM_gtm_cls2.MCS2_MEM[248].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM249 GTM_gtm_cls2.MCS2_MEM[249].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM250 GTM_gtm_cls2.MCS2_MEM[250].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM251 GTM_gtm_cls2.MCS2_MEM[251].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM252 GTM_gtm_cls2.MCS2_MEM[252].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM253 GTM_gtm_cls2.MCS2_MEM[253].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM254 GTM_gtm_cls2.MCS2_MEM[254].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM255 GTM_gtm_cls2.MCS2_MEM[255].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM256 GTM_gtm_cls2.MCS2_MEM[256].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM257 GTM_gtm_cls2.MCS2_MEM[257].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM258 GTM_gtm_cls2.MCS2_MEM[258].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM259 GTM_gtm_cls2.MCS2_MEM[259].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM260 GTM_gtm_cls2.MCS2_MEM[260].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM261 GTM_gtm_cls2.MCS2_MEM[261].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM262 GTM_gtm_cls2.MCS2_MEM[262].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM263 GTM_gtm_cls2.MCS2_MEM[263].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM264 GTM_gtm_cls2.MCS2_MEM[264].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM265 GTM_gtm_cls2.MCS2_MEM[265].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM266 GTM_gtm_cls2.MCS2_MEM[266].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM267 GTM_gtm_cls2.MCS2_MEM[267].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM268 GTM_gtm_cls2.MCS2_MEM[268].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM269 GTM_gtm_cls2.MCS2_MEM[269].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM270 GTM_gtm_cls2.MCS2_MEM[270].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM271 GTM_gtm_cls2.MCS2_MEM[271].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM272 GTM_gtm_cls2.MCS2_MEM[272].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM273 GTM_gtm_cls2.MCS2_MEM[273].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM274 GTM_gtm_cls2.MCS2_MEM[274].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM275 GTM_gtm_cls2.MCS2_MEM[275].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM276 GTM_gtm_cls2.MCS2_MEM[276].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM277 GTM_gtm_cls2.MCS2_MEM[277].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM278 GTM_gtm_cls2.MCS2_MEM[278].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM279 GTM_gtm_cls2.MCS2_MEM[279].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM280 GTM_gtm_cls2.MCS2_MEM[280].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM281 GTM_gtm_cls2.MCS2_MEM[281].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM282 GTM_gtm_cls2.MCS2_MEM[282].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM283 GTM_gtm_cls2.MCS2_MEM[283].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM284 GTM_gtm_cls2.MCS2_MEM[284].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM285 GTM_gtm_cls2.MCS2_MEM[285].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM286 GTM_gtm_cls2.MCS2_MEM[286].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM287 GTM_gtm_cls2.MCS2_MEM[287].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM288 GTM_gtm_cls2.MCS2_MEM[288].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM289 GTM_gtm_cls2.MCS2_MEM[289].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM290 GTM_gtm_cls2.MCS2_MEM[290].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM291 GTM_gtm_cls2.MCS2_MEM[291].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM292 GTM_gtm_cls2.MCS2_MEM[292].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM293 GTM_gtm_cls2.MCS2_MEM[293].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM294 GTM_gtm_cls2.MCS2_MEM[294].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM295 GTM_gtm_cls2.MCS2_MEM[295].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM296 GTM_gtm_cls2.MCS2_MEM[296].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM297 GTM_gtm_cls2.MCS2_MEM[297].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM298 GTM_gtm_cls2.MCS2_MEM[298].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM299 GTM_gtm_cls2.MCS2_MEM[299].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM300 GTM_gtm_cls2.MCS2_MEM[300].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM301 GTM_gtm_cls2.MCS2_MEM[301].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM302 GTM_gtm_cls2.MCS2_MEM[302].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM303 GTM_gtm_cls2.MCS2_MEM[303].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM304 GTM_gtm_cls2.MCS2_MEM[304].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM305 GTM_gtm_cls2.MCS2_MEM[305].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM306 GTM_gtm_cls2.MCS2_MEM[306].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM307 GTM_gtm_cls2.MCS2_MEM[307].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM308 GTM_gtm_cls2.MCS2_MEM[308].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM309 GTM_gtm_cls2.MCS2_MEM[309].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM310 GTM_gtm_cls2.MCS2_MEM[310].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM311 GTM_gtm_cls2.MCS2_MEM[311].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM312 GTM_gtm_cls2.MCS2_MEM[312].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM313 GTM_gtm_cls2.MCS2_MEM[313].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM314 GTM_gtm_cls2.MCS2_MEM[314].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM315 GTM_gtm_cls2.MCS2_MEM[315].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM316 GTM_gtm_cls2.MCS2_MEM[316].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM317 GTM_gtm_cls2.MCS2_MEM[317].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM318 GTM_gtm_cls2.MCS2_MEM[318].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM319 GTM_gtm_cls2.MCS2_MEM[319].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM320 GTM_gtm_cls2.MCS2_MEM[320].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM321 GTM_gtm_cls2.MCS2_MEM[321].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM322 GTM_gtm_cls2.MCS2_MEM[322].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM323 GTM_gtm_cls2.MCS2_MEM[323].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM324 GTM_gtm_cls2.MCS2_MEM[324].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM325 GTM_gtm_cls2.MCS2_MEM[325].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM326 GTM_gtm_cls2.MCS2_MEM[326].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM327 GTM_gtm_cls2.MCS2_MEM[327].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM328 GTM_gtm_cls2.MCS2_MEM[328].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM329 GTM_gtm_cls2.MCS2_MEM[329].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM330 GTM_gtm_cls2.MCS2_MEM[330].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM331 GTM_gtm_cls2.MCS2_MEM[331].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM332 GTM_gtm_cls2.MCS2_MEM[332].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM333 GTM_gtm_cls2.MCS2_MEM[333].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM334 GTM_gtm_cls2.MCS2_MEM[334].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM335 GTM_gtm_cls2.MCS2_MEM[335].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM336 GTM_gtm_cls2.MCS2_MEM[336].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM337 GTM_gtm_cls2.MCS2_MEM[337].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM338 GTM_gtm_cls2.MCS2_MEM[338].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM339 GTM_gtm_cls2.MCS2_MEM[339].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM340 GTM_gtm_cls2.MCS2_MEM[340].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM341 GTM_gtm_cls2.MCS2_MEM[341].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM342 GTM_gtm_cls2.MCS2_MEM[342].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM343 GTM_gtm_cls2.MCS2_MEM[343].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM344 GTM_gtm_cls2.MCS2_MEM[344].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM345 GTM_gtm_cls2.MCS2_MEM[345].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM346 GTM_gtm_cls2.MCS2_MEM[346].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM347 GTM_gtm_cls2.MCS2_MEM[347].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM348 GTM_gtm_cls2.MCS2_MEM[348].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM349 GTM_gtm_cls2.MCS2_MEM[349].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM350 GTM_gtm_cls2.MCS2_MEM[350].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM351 GTM_gtm_cls2.MCS2_MEM[351].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM352 GTM_gtm_cls2.MCS2_MEM[352].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM353 GTM_gtm_cls2.MCS2_MEM[353].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM354 GTM_gtm_cls2.MCS2_MEM[354].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM355 GTM_gtm_cls2.MCS2_MEM[355].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM356 GTM_gtm_cls2.MCS2_MEM[356].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM357 GTM_gtm_cls2.MCS2_MEM[357].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM358 GTM_gtm_cls2.MCS2_MEM[358].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM359 GTM_gtm_cls2.MCS2_MEM[359].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM360 GTM_gtm_cls2.MCS2_MEM[360].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM361 GTM_gtm_cls2.MCS2_MEM[361].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM362 GTM_gtm_cls2.MCS2_MEM[362].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM363 GTM_gtm_cls2.MCS2_MEM[363].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM364 GTM_gtm_cls2.MCS2_MEM[364].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM365 GTM_gtm_cls2.MCS2_MEM[365].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM366 GTM_gtm_cls2.MCS2_MEM[366].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM367 GTM_gtm_cls2.MCS2_MEM[367].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM368 GTM_gtm_cls2.MCS2_MEM[368].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM369 GTM_gtm_cls2.MCS2_MEM[369].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM370 GTM_gtm_cls2.MCS2_MEM[370].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM371 GTM_gtm_cls2.MCS2_MEM[371].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM372 GTM_gtm_cls2.MCS2_MEM[372].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM373 GTM_gtm_cls2.MCS2_MEM[373].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM374 GTM_gtm_cls2.MCS2_MEM[374].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM375 GTM_gtm_cls2.MCS2_MEM[375].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM376 GTM_gtm_cls2.MCS2_MEM[376].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM377 GTM_gtm_cls2.MCS2_MEM[377].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM378 GTM_gtm_cls2.MCS2_MEM[378].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM379 GTM_gtm_cls2.MCS2_MEM[379].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM380 GTM_gtm_cls2.MCS2_MEM[380].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM381 GTM_gtm_cls2.MCS2_MEM[381].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM382 GTM_gtm_cls2.MCS2_MEM[382].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM383 GTM_gtm_cls2.MCS2_MEM[383].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM384 GTM_gtm_cls2.MCS2_MEM[384].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM385 GTM_gtm_cls2.MCS2_MEM[385].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM386 GTM_gtm_cls2.MCS2_MEM[386].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM387 GTM_gtm_cls2.MCS2_MEM[387].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM388 GTM_gtm_cls2.MCS2_MEM[388].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM389 GTM_gtm_cls2.MCS2_MEM[389].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM390 GTM_gtm_cls2.MCS2_MEM[390].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM391 GTM_gtm_cls2.MCS2_MEM[391].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM392 GTM_gtm_cls2.MCS2_MEM[392].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM393 GTM_gtm_cls2.MCS2_MEM[393].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM394 GTM_gtm_cls2.MCS2_MEM[394].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM395 GTM_gtm_cls2.MCS2_MEM[395].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM396 GTM_gtm_cls2.MCS2_MEM[396].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM397 GTM_gtm_cls2.MCS2_MEM[397].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM398 GTM_gtm_cls2.MCS2_MEM[398].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM399 GTM_gtm_cls2.MCS2_MEM[399].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM400 GTM_gtm_cls2.MCS2_MEM[400].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM401 GTM_gtm_cls2.MCS2_MEM[401].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM402 GTM_gtm_cls2.MCS2_MEM[402].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM403 GTM_gtm_cls2.MCS2_MEM[403].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM404 GTM_gtm_cls2.MCS2_MEM[404].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM405 GTM_gtm_cls2.MCS2_MEM[405].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM406 GTM_gtm_cls2.MCS2_MEM[406].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM407 GTM_gtm_cls2.MCS2_MEM[407].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM408 GTM_gtm_cls2.MCS2_MEM[408].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM409 GTM_gtm_cls2.MCS2_MEM[409].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM410 GTM_gtm_cls2.MCS2_MEM[410].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM411 GTM_gtm_cls2.MCS2_MEM[411].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM412 GTM_gtm_cls2.MCS2_MEM[412].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM413 GTM_gtm_cls2.MCS2_MEM[413].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM414 GTM_gtm_cls2.MCS2_MEM[414].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM415 GTM_gtm_cls2.MCS2_MEM[415].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM416 GTM_gtm_cls2.MCS2_MEM[416].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM417 GTM_gtm_cls2.MCS2_MEM[417].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM418 GTM_gtm_cls2.MCS2_MEM[418].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM419 GTM_gtm_cls2.MCS2_MEM[419].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM420 GTM_gtm_cls2.MCS2_MEM[420].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM421 GTM_gtm_cls2.MCS2_MEM[421].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM422 GTM_gtm_cls2.MCS2_MEM[422].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM423 GTM_gtm_cls2.MCS2_MEM[423].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM424 GTM_gtm_cls2.MCS2_MEM[424].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM425 GTM_gtm_cls2.MCS2_MEM[425].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM426 GTM_gtm_cls2.MCS2_MEM[426].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM427 GTM_gtm_cls2.MCS2_MEM[427].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM428 GTM_gtm_cls2.MCS2_MEM[428].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM429 GTM_gtm_cls2.MCS2_MEM[429].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM430 GTM_gtm_cls2.MCS2_MEM[430].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM431 GTM_gtm_cls2.MCS2_MEM[431].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM432 GTM_gtm_cls2.MCS2_MEM[432].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM433 GTM_gtm_cls2.MCS2_MEM[433].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM434 GTM_gtm_cls2.MCS2_MEM[434].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM435 GTM_gtm_cls2.MCS2_MEM[435].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM436 GTM_gtm_cls2.MCS2_MEM[436].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM437 GTM_gtm_cls2.MCS2_MEM[437].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM438 GTM_gtm_cls2.MCS2_MEM[438].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM439 GTM_gtm_cls2.MCS2_MEM[439].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM440 GTM_gtm_cls2.MCS2_MEM[440].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM441 GTM_gtm_cls2.MCS2_MEM[441].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM442 GTM_gtm_cls2.MCS2_MEM[442].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM443 GTM_gtm_cls2.MCS2_MEM[443].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM444 GTM_gtm_cls2.MCS2_MEM[444].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM445 GTM_gtm_cls2.MCS2_MEM[445].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM446 GTM_gtm_cls2.MCS2_MEM[446].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM447 GTM_gtm_cls2.MCS2_MEM[447].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM448 GTM_gtm_cls2.MCS2_MEM[448].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM449 GTM_gtm_cls2.MCS2_MEM[449].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM450 GTM_gtm_cls2.MCS2_MEM[450].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM451 GTM_gtm_cls2.MCS2_MEM[451].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM452 GTM_gtm_cls2.MCS2_MEM[452].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM453 GTM_gtm_cls2.MCS2_MEM[453].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM454 GTM_gtm_cls2.MCS2_MEM[454].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM455 GTM_gtm_cls2.MCS2_MEM[455].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM456 GTM_gtm_cls2.MCS2_MEM[456].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM457 GTM_gtm_cls2.MCS2_MEM[457].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM458 GTM_gtm_cls2.MCS2_MEM[458].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM459 GTM_gtm_cls2.MCS2_MEM[459].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM460 GTM_gtm_cls2.MCS2_MEM[460].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM461 GTM_gtm_cls2.MCS2_MEM[461].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM462 GTM_gtm_cls2.MCS2_MEM[462].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM463 GTM_gtm_cls2.MCS2_MEM[463].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM464 GTM_gtm_cls2.MCS2_MEM[464].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM465 GTM_gtm_cls2.MCS2_MEM[465].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM466 GTM_gtm_cls2.MCS2_MEM[466].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM467 GTM_gtm_cls2.MCS2_MEM[467].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM468 GTM_gtm_cls2.MCS2_MEM[468].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM469 GTM_gtm_cls2.MCS2_MEM[469].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM470 GTM_gtm_cls2.MCS2_MEM[470].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM471 GTM_gtm_cls2.MCS2_MEM[471].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM472 GTM_gtm_cls2.MCS2_MEM[472].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM473 GTM_gtm_cls2.MCS2_MEM[473].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM474 GTM_gtm_cls2.MCS2_MEM[474].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM475 GTM_gtm_cls2.MCS2_MEM[475].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM476 GTM_gtm_cls2.MCS2_MEM[476].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM477 GTM_gtm_cls2.MCS2_MEM[477].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM478 GTM_gtm_cls2.MCS2_MEM[478].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM479 GTM_gtm_cls2.MCS2_MEM[479].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM480 GTM_gtm_cls2.MCS2_MEM[480].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM481 GTM_gtm_cls2.MCS2_MEM[481].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM482 GTM_gtm_cls2.MCS2_MEM[482].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM483 GTM_gtm_cls2.MCS2_MEM[483].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM484 GTM_gtm_cls2.MCS2_MEM[484].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM485 GTM_gtm_cls2.MCS2_MEM[485].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM486 GTM_gtm_cls2.MCS2_MEM[486].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM487 GTM_gtm_cls2.MCS2_MEM[487].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM488 GTM_gtm_cls2.MCS2_MEM[488].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM489 GTM_gtm_cls2.MCS2_MEM[489].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM490 GTM_gtm_cls2.MCS2_MEM[490].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM491 GTM_gtm_cls2.MCS2_MEM[491].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM492 GTM_gtm_cls2.MCS2_MEM[492].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM493 GTM_gtm_cls2.MCS2_MEM[493].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM494 GTM_gtm_cls2.MCS2_MEM[494].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM495 GTM_gtm_cls2.MCS2_MEM[495].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM496 GTM_gtm_cls2.MCS2_MEM[496].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM497 GTM_gtm_cls2.MCS2_MEM[497].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM498 GTM_gtm_cls2.MCS2_MEM[498].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM499 GTM_gtm_cls2.MCS2_MEM[499].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM500 GTM_gtm_cls2.MCS2_MEM[500].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM501 GTM_gtm_cls2.MCS2_MEM[501].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM502 GTM_gtm_cls2.MCS2_MEM[502].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM503 GTM_gtm_cls2.MCS2_MEM[503].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM504 GTM_gtm_cls2.MCS2_MEM[504].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM505 GTM_gtm_cls2.MCS2_MEM[505].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM506 GTM_gtm_cls2.MCS2_MEM[506].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM507 GTM_gtm_cls2.MCS2_MEM[507].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM508 GTM_gtm_cls2.MCS2_MEM[508].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM509 GTM_gtm_cls2.MCS2_MEM[509].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM510 GTM_gtm_cls2.MCS2_MEM[510].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM511 GTM_gtm_cls2.MCS2_MEM[511].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM512 GTM_gtm_cls2.MCS2_MEM[512].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM513 GTM_gtm_cls2.MCS2_MEM[513].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM514 GTM_gtm_cls2.MCS2_MEM[514].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM515 GTM_gtm_cls2.MCS2_MEM[515].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM516 GTM_gtm_cls2.MCS2_MEM[516].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM517 GTM_gtm_cls2.MCS2_MEM[517].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM518 GTM_gtm_cls2.MCS2_MEM[518].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM519 GTM_gtm_cls2.MCS2_MEM[519].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM520 GTM_gtm_cls2.MCS2_MEM[520].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM521 GTM_gtm_cls2.MCS2_MEM[521].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM522 GTM_gtm_cls2.MCS2_MEM[522].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM523 GTM_gtm_cls2.MCS2_MEM[523].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM524 GTM_gtm_cls2.MCS2_MEM[524].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM525 GTM_gtm_cls2.MCS2_MEM[525].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM526 GTM_gtm_cls2.MCS2_MEM[526].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM527 GTM_gtm_cls2.MCS2_MEM[527].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM528 GTM_gtm_cls2.MCS2_MEM[528].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM529 GTM_gtm_cls2.MCS2_MEM[529].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM530 GTM_gtm_cls2.MCS2_MEM[530].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM531 GTM_gtm_cls2.MCS2_MEM[531].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM532 GTM_gtm_cls2.MCS2_MEM[532].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM533 GTM_gtm_cls2.MCS2_MEM[533].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM534 GTM_gtm_cls2.MCS2_MEM[534].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM535 GTM_gtm_cls2.MCS2_MEM[535].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM536 GTM_gtm_cls2.MCS2_MEM[536].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM537 GTM_gtm_cls2.MCS2_MEM[537].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM538 GTM_gtm_cls2.MCS2_MEM[538].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM539 GTM_gtm_cls2.MCS2_MEM[539].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM540 GTM_gtm_cls2.MCS2_MEM[540].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM541 GTM_gtm_cls2.MCS2_MEM[541].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM542 GTM_gtm_cls2.MCS2_MEM[542].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM543 GTM_gtm_cls2.MCS2_MEM[543].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM544 GTM_gtm_cls2.MCS2_MEM[544].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM545 GTM_gtm_cls2.MCS2_MEM[545].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM546 GTM_gtm_cls2.MCS2_MEM[546].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM547 GTM_gtm_cls2.MCS2_MEM[547].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM548 GTM_gtm_cls2.MCS2_MEM[548].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM549 GTM_gtm_cls2.MCS2_MEM[549].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM550 GTM_gtm_cls2.MCS2_MEM[550].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM551 GTM_gtm_cls2.MCS2_MEM[551].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM552 GTM_gtm_cls2.MCS2_MEM[552].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM553 GTM_gtm_cls2.MCS2_MEM[553].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM554 GTM_gtm_cls2.MCS2_MEM[554].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM555 GTM_gtm_cls2.MCS2_MEM[555].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM556 GTM_gtm_cls2.MCS2_MEM[556].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM557 GTM_gtm_cls2.MCS2_MEM[557].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM558 GTM_gtm_cls2.MCS2_MEM[558].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM559 GTM_gtm_cls2.MCS2_MEM[559].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM560 GTM_gtm_cls2.MCS2_MEM[560].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM561 GTM_gtm_cls2.MCS2_MEM[561].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM562 GTM_gtm_cls2.MCS2_MEM[562].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM563 GTM_gtm_cls2.MCS2_MEM[563].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM564 GTM_gtm_cls2.MCS2_MEM[564].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM565 GTM_gtm_cls2.MCS2_MEM[565].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM566 GTM_gtm_cls2.MCS2_MEM[566].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM567 GTM_gtm_cls2.MCS2_MEM[567].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM568 GTM_gtm_cls2.MCS2_MEM[568].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM569 GTM_gtm_cls2.MCS2_MEM[569].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM570 GTM_gtm_cls2.MCS2_MEM[570].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM571 GTM_gtm_cls2.MCS2_MEM[571].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM572 GTM_gtm_cls2.MCS2_MEM[572].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM573 GTM_gtm_cls2.MCS2_MEM[573].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM574 GTM_gtm_cls2.MCS2_MEM[574].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM575 GTM_gtm_cls2.MCS2_MEM[575].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM576 GTM_gtm_cls2.MCS2_MEM[576].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM577 GTM_gtm_cls2.MCS2_MEM[577].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM578 GTM_gtm_cls2.MCS2_MEM[578].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM579 GTM_gtm_cls2.MCS2_MEM[579].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM580 GTM_gtm_cls2.MCS2_MEM[580].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM581 GTM_gtm_cls2.MCS2_MEM[581].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM582 GTM_gtm_cls2.MCS2_MEM[582].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM583 GTM_gtm_cls2.MCS2_MEM[583].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM584 GTM_gtm_cls2.MCS2_MEM[584].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM585 GTM_gtm_cls2.MCS2_MEM[585].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM586 GTM_gtm_cls2.MCS2_MEM[586].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM587 GTM_gtm_cls2.MCS2_MEM[587].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM588 GTM_gtm_cls2.MCS2_MEM[588].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM589 GTM_gtm_cls2.MCS2_MEM[589].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM590 GTM_gtm_cls2.MCS2_MEM[590].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM591 GTM_gtm_cls2.MCS2_MEM[591].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM592 GTM_gtm_cls2.MCS2_MEM[592].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM593 GTM_gtm_cls2.MCS2_MEM[593].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM594 GTM_gtm_cls2.MCS2_MEM[594].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM595 GTM_gtm_cls2.MCS2_MEM[595].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM596 GTM_gtm_cls2.MCS2_MEM[596].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM597 GTM_gtm_cls2.MCS2_MEM[597].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM598 GTM_gtm_cls2.MCS2_MEM[598].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM599 GTM_gtm_cls2.MCS2_MEM[599].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM600 GTM_gtm_cls2.MCS2_MEM[600].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM601 GTM_gtm_cls2.MCS2_MEM[601].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM602 GTM_gtm_cls2.MCS2_MEM[602].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM603 GTM_gtm_cls2.MCS2_MEM[603].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM604 GTM_gtm_cls2.MCS2_MEM[604].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM605 GTM_gtm_cls2.MCS2_MEM[605].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM606 GTM_gtm_cls2.MCS2_MEM[606].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM607 GTM_gtm_cls2.MCS2_MEM[607].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM608 GTM_gtm_cls2.MCS2_MEM[608].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM609 GTM_gtm_cls2.MCS2_MEM[609].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM610 GTM_gtm_cls2.MCS2_MEM[610].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM611 GTM_gtm_cls2.MCS2_MEM[611].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM612 GTM_gtm_cls2.MCS2_MEM[612].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM613 GTM_gtm_cls2.MCS2_MEM[613].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM614 GTM_gtm_cls2.MCS2_MEM[614].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM615 GTM_gtm_cls2.MCS2_MEM[615].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM616 GTM_gtm_cls2.MCS2_MEM[616].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM617 GTM_gtm_cls2.MCS2_MEM[617].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM618 GTM_gtm_cls2.MCS2_MEM[618].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM619 GTM_gtm_cls2.MCS2_MEM[619].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM620 GTM_gtm_cls2.MCS2_MEM[620].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM621 GTM_gtm_cls2.MCS2_MEM[621].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM622 GTM_gtm_cls2.MCS2_MEM[622].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM623 GTM_gtm_cls2.MCS2_MEM[623].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM624 GTM_gtm_cls2.MCS2_MEM[624].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM625 GTM_gtm_cls2.MCS2_MEM[625].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM626 GTM_gtm_cls2.MCS2_MEM[626].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM627 GTM_gtm_cls2.MCS2_MEM[627].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM628 GTM_gtm_cls2.MCS2_MEM[628].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM629 GTM_gtm_cls2.MCS2_MEM[629].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM630 GTM_gtm_cls2.MCS2_MEM[630].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM631 GTM_gtm_cls2.MCS2_MEM[631].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM632 GTM_gtm_cls2.MCS2_MEM[632].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM633 GTM_gtm_cls2.MCS2_MEM[633].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM634 GTM_gtm_cls2.MCS2_MEM[634].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM635 GTM_gtm_cls2.MCS2_MEM[635].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM636 GTM_gtm_cls2.MCS2_MEM[636].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM637 GTM_gtm_cls2.MCS2_MEM[637].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM638 GTM_gtm_cls2.MCS2_MEM[638].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM639 GTM_gtm_cls2.MCS2_MEM[639].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM640 GTM_gtm_cls2.MCS2_MEM[640].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM641 GTM_gtm_cls2.MCS2_MEM[641].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM642 GTM_gtm_cls2.MCS2_MEM[642].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM643 GTM_gtm_cls2.MCS2_MEM[643].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM644 GTM_gtm_cls2.MCS2_MEM[644].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM645 GTM_gtm_cls2.MCS2_MEM[645].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM646 GTM_gtm_cls2.MCS2_MEM[646].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM647 GTM_gtm_cls2.MCS2_MEM[647].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM648 GTM_gtm_cls2.MCS2_MEM[648].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM649 GTM_gtm_cls2.MCS2_MEM[649].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM650 GTM_gtm_cls2.MCS2_MEM[650].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM651 GTM_gtm_cls2.MCS2_MEM[651].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM652 GTM_gtm_cls2.MCS2_MEM[652].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM653 GTM_gtm_cls2.MCS2_MEM[653].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM654 GTM_gtm_cls2.MCS2_MEM[654].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM655 GTM_gtm_cls2.MCS2_MEM[655].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM656 GTM_gtm_cls2.MCS2_MEM[656].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM657 GTM_gtm_cls2.MCS2_MEM[657].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM658 GTM_gtm_cls2.MCS2_MEM[658].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM659 GTM_gtm_cls2.MCS2_MEM[659].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM660 GTM_gtm_cls2.MCS2_MEM[660].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM661 GTM_gtm_cls2.MCS2_MEM[661].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM662 GTM_gtm_cls2.MCS2_MEM[662].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM663 GTM_gtm_cls2.MCS2_MEM[663].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM664 GTM_gtm_cls2.MCS2_MEM[664].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM665 GTM_gtm_cls2.MCS2_MEM[665].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM666 GTM_gtm_cls2.MCS2_MEM[666].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM667 GTM_gtm_cls2.MCS2_MEM[667].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM668 GTM_gtm_cls2.MCS2_MEM[668].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM669 GTM_gtm_cls2.MCS2_MEM[669].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM670 GTM_gtm_cls2.MCS2_MEM[670].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM671 GTM_gtm_cls2.MCS2_MEM[671].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM672 GTM_gtm_cls2.MCS2_MEM[672].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM673 GTM_gtm_cls2.MCS2_MEM[673].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM674 GTM_gtm_cls2.MCS2_MEM[674].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM675 GTM_gtm_cls2.MCS2_MEM[675].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM676 GTM_gtm_cls2.MCS2_MEM[676].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM677 GTM_gtm_cls2.MCS2_MEM[677].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM678 GTM_gtm_cls2.MCS2_MEM[678].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM679 GTM_gtm_cls2.MCS2_MEM[679].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM680 GTM_gtm_cls2.MCS2_MEM[680].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM681 GTM_gtm_cls2.MCS2_MEM[681].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM682 GTM_gtm_cls2.MCS2_MEM[682].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM683 GTM_gtm_cls2.MCS2_MEM[683].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM684 GTM_gtm_cls2.MCS2_MEM[684].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM685 GTM_gtm_cls2.MCS2_MEM[685].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM686 GTM_gtm_cls2.MCS2_MEM[686].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM687 GTM_gtm_cls2.MCS2_MEM[687].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM688 GTM_gtm_cls2.MCS2_MEM[688].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM689 GTM_gtm_cls2.MCS2_MEM[689].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM690 GTM_gtm_cls2.MCS2_MEM[690].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM691 GTM_gtm_cls2.MCS2_MEM[691].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM692 GTM_gtm_cls2.MCS2_MEM[692].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM693 GTM_gtm_cls2.MCS2_MEM[693].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM694 GTM_gtm_cls2.MCS2_MEM[694].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM695 GTM_gtm_cls2.MCS2_MEM[695].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM696 GTM_gtm_cls2.MCS2_MEM[696].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM697 GTM_gtm_cls2.MCS2_MEM[697].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM698 GTM_gtm_cls2.MCS2_MEM[698].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM699 GTM_gtm_cls2.MCS2_MEM[699].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM700 GTM_gtm_cls2.MCS2_MEM[700].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM701 GTM_gtm_cls2.MCS2_MEM[701].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM702 GTM_gtm_cls2.MCS2_MEM[702].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM703 GTM_gtm_cls2.MCS2_MEM[703].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM704 GTM_gtm_cls2.MCS2_MEM[704].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM705 GTM_gtm_cls2.MCS2_MEM[705].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM706 GTM_gtm_cls2.MCS2_MEM[706].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM707 GTM_gtm_cls2.MCS2_MEM[707].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM708 GTM_gtm_cls2.MCS2_MEM[708].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM709 GTM_gtm_cls2.MCS2_MEM[709].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM710 GTM_gtm_cls2.MCS2_MEM[710].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM711 GTM_gtm_cls2.MCS2_MEM[711].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM712 GTM_gtm_cls2.MCS2_MEM[712].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM713 GTM_gtm_cls2.MCS2_MEM[713].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM714 GTM_gtm_cls2.MCS2_MEM[714].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM715 GTM_gtm_cls2.MCS2_MEM[715].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM716 GTM_gtm_cls2.MCS2_MEM[716].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM717 GTM_gtm_cls2.MCS2_MEM[717].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM718 GTM_gtm_cls2.MCS2_MEM[718].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM719 GTM_gtm_cls2.MCS2_MEM[719].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM720 GTM_gtm_cls2.MCS2_MEM[720].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM721 GTM_gtm_cls2.MCS2_MEM[721].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM722 GTM_gtm_cls2.MCS2_MEM[722].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM723 GTM_gtm_cls2.MCS2_MEM[723].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM724 GTM_gtm_cls2.MCS2_MEM[724].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM725 GTM_gtm_cls2.MCS2_MEM[725].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM726 GTM_gtm_cls2.MCS2_MEM[726].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM727 GTM_gtm_cls2.MCS2_MEM[727].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM728 GTM_gtm_cls2.MCS2_MEM[728].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM729 GTM_gtm_cls2.MCS2_MEM[729].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM730 GTM_gtm_cls2.MCS2_MEM[730].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM731 GTM_gtm_cls2.MCS2_MEM[731].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM732 GTM_gtm_cls2.MCS2_MEM[732].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM733 GTM_gtm_cls2.MCS2_MEM[733].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM734 GTM_gtm_cls2.MCS2_MEM[734].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM735 GTM_gtm_cls2.MCS2_MEM[735].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM736 GTM_gtm_cls2.MCS2_MEM[736].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM737 GTM_gtm_cls2.MCS2_MEM[737].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM738 GTM_gtm_cls2.MCS2_MEM[738].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM739 GTM_gtm_cls2.MCS2_MEM[739].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM740 GTM_gtm_cls2.MCS2_MEM[740].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM741 GTM_gtm_cls2.MCS2_MEM[741].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM742 GTM_gtm_cls2.MCS2_MEM[742].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM743 GTM_gtm_cls2.MCS2_MEM[743].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM744 GTM_gtm_cls2.MCS2_MEM[744].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM745 GTM_gtm_cls2.MCS2_MEM[745].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM746 GTM_gtm_cls2.MCS2_MEM[746].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM747 GTM_gtm_cls2.MCS2_MEM[747].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM748 GTM_gtm_cls2.MCS2_MEM[748].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM749 GTM_gtm_cls2.MCS2_MEM[749].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM750 GTM_gtm_cls2.MCS2_MEM[750].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM751 GTM_gtm_cls2.MCS2_MEM[751].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM752 GTM_gtm_cls2.MCS2_MEM[752].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM753 GTM_gtm_cls2.MCS2_MEM[753].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM754 GTM_gtm_cls2.MCS2_MEM[754].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM755 GTM_gtm_cls2.MCS2_MEM[755].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM756 GTM_gtm_cls2.MCS2_MEM[756].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM757 GTM_gtm_cls2.MCS2_MEM[757].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM758 GTM_gtm_cls2.MCS2_MEM[758].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM759 GTM_gtm_cls2.MCS2_MEM[759].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM760 GTM_gtm_cls2.MCS2_MEM[760].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM761 GTM_gtm_cls2.MCS2_MEM[761].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM762 GTM_gtm_cls2.MCS2_MEM[762].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM763 GTM_gtm_cls2.MCS2_MEM[763].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM764 GTM_gtm_cls2.MCS2_MEM[764].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM765 GTM_gtm_cls2.MCS2_MEM[765].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM766 GTM_gtm_cls2.MCS2_MEM[766].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM767 GTM_gtm_cls2.MCS2_MEM[767].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM768 GTM_gtm_cls2.MCS2_MEM[768].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM769 GTM_gtm_cls2.MCS2_MEM[769].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM770 GTM_gtm_cls2.MCS2_MEM[770].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM771 GTM_gtm_cls2.MCS2_MEM[771].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM772 GTM_gtm_cls2.MCS2_MEM[772].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM773 GTM_gtm_cls2.MCS2_MEM[773].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM774 GTM_gtm_cls2.MCS2_MEM[774].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM775 GTM_gtm_cls2.MCS2_MEM[775].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM776 GTM_gtm_cls2.MCS2_MEM[776].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM777 GTM_gtm_cls2.MCS2_MEM[777].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM778 GTM_gtm_cls2.MCS2_MEM[778].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM779 GTM_gtm_cls2.MCS2_MEM[779].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM780 GTM_gtm_cls2.MCS2_MEM[780].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM781 GTM_gtm_cls2.MCS2_MEM[781].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM782 GTM_gtm_cls2.MCS2_MEM[782].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM783 GTM_gtm_cls2.MCS2_MEM[783].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM784 GTM_gtm_cls2.MCS2_MEM[784].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM785 GTM_gtm_cls2.MCS2_MEM[785].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM786 GTM_gtm_cls2.MCS2_MEM[786].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM787 GTM_gtm_cls2.MCS2_MEM[787].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM788 GTM_gtm_cls2.MCS2_MEM[788].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM789 GTM_gtm_cls2.MCS2_MEM[789].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM790 GTM_gtm_cls2.MCS2_MEM[790].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM791 GTM_gtm_cls2.MCS2_MEM[791].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM792 GTM_gtm_cls2.MCS2_MEM[792].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM793 GTM_gtm_cls2.MCS2_MEM[793].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM794 GTM_gtm_cls2.MCS2_MEM[794].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM795 GTM_gtm_cls2.MCS2_MEM[795].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM796 GTM_gtm_cls2.MCS2_MEM[796].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM797 GTM_gtm_cls2.MCS2_MEM[797].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM798 GTM_gtm_cls2.MCS2_MEM[798].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM799 GTM_gtm_cls2.MCS2_MEM[799].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM800 GTM_gtm_cls2.MCS2_MEM[800].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM801 GTM_gtm_cls2.MCS2_MEM[801].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM802 GTM_gtm_cls2.MCS2_MEM[802].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM803 GTM_gtm_cls2.MCS2_MEM[803].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM804 GTM_gtm_cls2.MCS2_MEM[804].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM805 GTM_gtm_cls2.MCS2_MEM[805].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM806 GTM_gtm_cls2.MCS2_MEM[806].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM807 GTM_gtm_cls2.MCS2_MEM[807].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM808 GTM_gtm_cls2.MCS2_MEM[808].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM809 GTM_gtm_cls2.MCS2_MEM[809].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM810 GTM_gtm_cls2.MCS2_MEM[810].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM811 GTM_gtm_cls2.MCS2_MEM[811].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM812 GTM_gtm_cls2.MCS2_MEM[812].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM813 GTM_gtm_cls2.MCS2_MEM[813].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM814 GTM_gtm_cls2.MCS2_MEM[814].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM815 GTM_gtm_cls2.MCS2_MEM[815].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM816 GTM_gtm_cls2.MCS2_MEM[816].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM817 GTM_gtm_cls2.MCS2_MEM[817].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM818 GTM_gtm_cls2.MCS2_MEM[818].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM819 GTM_gtm_cls2.MCS2_MEM[819].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM820 GTM_gtm_cls2.MCS2_MEM[820].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM821 GTM_gtm_cls2.MCS2_MEM[821].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM822 GTM_gtm_cls2.MCS2_MEM[822].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM823 GTM_gtm_cls2.MCS2_MEM[823].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM824 GTM_gtm_cls2.MCS2_MEM[824].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM825 GTM_gtm_cls2.MCS2_MEM[825].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM826 GTM_gtm_cls2.MCS2_MEM[826].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM827 GTM_gtm_cls2.MCS2_MEM[827].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM828 GTM_gtm_cls2.MCS2_MEM[828].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM829 GTM_gtm_cls2.MCS2_MEM[829].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM830 GTM_gtm_cls2.MCS2_MEM[830].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM831 GTM_gtm_cls2.MCS2_MEM[831].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM832 GTM_gtm_cls2.MCS2_MEM[832].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM833 GTM_gtm_cls2.MCS2_MEM[833].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM834 GTM_gtm_cls2.MCS2_MEM[834].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM835 GTM_gtm_cls2.MCS2_MEM[835].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM836 GTM_gtm_cls2.MCS2_MEM[836].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM837 GTM_gtm_cls2.MCS2_MEM[837].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM838 GTM_gtm_cls2.MCS2_MEM[838].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM839 GTM_gtm_cls2.MCS2_MEM[839].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM840 GTM_gtm_cls2.MCS2_MEM[840].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM841 GTM_gtm_cls2.MCS2_MEM[841].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM842 GTM_gtm_cls2.MCS2_MEM[842].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM843 GTM_gtm_cls2.MCS2_MEM[843].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM844 GTM_gtm_cls2.MCS2_MEM[844].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM845 GTM_gtm_cls2.MCS2_MEM[845].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM846 GTM_gtm_cls2.MCS2_MEM[846].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM847 GTM_gtm_cls2.MCS2_MEM[847].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM848 GTM_gtm_cls2.MCS2_MEM[848].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM849 GTM_gtm_cls2.MCS2_MEM[849].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM850 GTM_gtm_cls2.MCS2_MEM[850].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM851 GTM_gtm_cls2.MCS2_MEM[851].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM852 GTM_gtm_cls2.MCS2_MEM[852].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM853 GTM_gtm_cls2.MCS2_MEM[853].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM854 GTM_gtm_cls2.MCS2_MEM[854].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM855 GTM_gtm_cls2.MCS2_MEM[855].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM856 GTM_gtm_cls2.MCS2_MEM[856].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM857 GTM_gtm_cls2.MCS2_MEM[857].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM858 GTM_gtm_cls2.MCS2_MEM[858].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM859 GTM_gtm_cls2.MCS2_MEM[859].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM860 GTM_gtm_cls2.MCS2_MEM[860].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM861 GTM_gtm_cls2.MCS2_MEM[861].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM862 GTM_gtm_cls2.MCS2_MEM[862].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM863 GTM_gtm_cls2.MCS2_MEM[863].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM864 GTM_gtm_cls2.MCS2_MEM[864].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM865 GTM_gtm_cls2.MCS2_MEM[865].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM866 GTM_gtm_cls2.MCS2_MEM[866].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM867 GTM_gtm_cls2.MCS2_MEM[867].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM868 GTM_gtm_cls2.MCS2_MEM[868].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM869 GTM_gtm_cls2.MCS2_MEM[869].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM870 GTM_gtm_cls2.MCS2_MEM[870].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM871 GTM_gtm_cls2.MCS2_MEM[871].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM872 GTM_gtm_cls2.MCS2_MEM[872].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM873 GTM_gtm_cls2.MCS2_MEM[873].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM874 GTM_gtm_cls2.MCS2_MEM[874].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM875 GTM_gtm_cls2.MCS2_MEM[875].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM876 GTM_gtm_cls2.MCS2_MEM[876].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM877 GTM_gtm_cls2.MCS2_MEM[877].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM878 GTM_gtm_cls2.MCS2_MEM[878].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM879 GTM_gtm_cls2.MCS2_MEM[879].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM880 GTM_gtm_cls2.MCS2_MEM[880].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM881 GTM_gtm_cls2.MCS2_MEM[881].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM882 GTM_gtm_cls2.MCS2_MEM[882].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM883 GTM_gtm_cls2.MCS2_MEM[883].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM884 GTM_gtm_cls2.MCS2_MEM[884].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM885 GTM_gtm_cls2.MCS2_MEM[885].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM886 GTM_gtm_cls2.MCS2_MEM[886].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM887 GTM_gtm_cls2.MCS2_MEM[887].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM888 GTM_gtm_cls2.MCS2_MEM[888].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM889 GTM_gtm_cls2.MCS2_MEM[889].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM890 GTM_gtm_cls2.MCS2_MEM[890].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM891 GTM_gtm_cls2.MCS2_MEM[891].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM892 GTM_gtm_cls2.MCS2_MEM[892].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM893 GTM_gtm_cls2.MCS2_MEM[893].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM894 GTM_gtm_cls2.MCS2_MEM[894].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM895 GTM_gtm_cls2.MCS2_MEM[895].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM896 GTM_gtm_cls2.MCS2_MEM[896].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM897 GTM_gtm_cls2.MCS2_MEM[897].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM898 GTM_gtm_cls2.MCS2_MEM[898].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM899 GTM_gtm_cls2.MCS2_MEM[899].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM900 GTM_gtm_cls2.MCS2_MEM[900].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM901 GTM_gtm_cls2.MCS2_MEM[901].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM902 GTM_gtm_cls2.MCS2_MEM[902].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM903 GTM_gtm_cls2.MCS2_MEM[903].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM904 GTM_gtm_cls2.MCS2_MEM[904].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM905 GTM_gtm_cls2.MCS2_MEM[905].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM906 GTM_gtm_cls2.MCS2_MEM[906].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM907 GTM_gtm_cls2.MCS2_MEM[907].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM908 GTM_gtm_cls2.MCS2_MEM[908].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM909 GTM_gtm_cls2.MCS2_MEM[909].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM910 GTM_gtm_cls2.MCS2_MEM[910].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM911 GTM_gtm_cls2.MCS2_MEM[911].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM912 GTM_gtm_cls2.MCS2_MEM[912].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM913 GTM_gtm_cls2.MCS2_MEM[913].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM914 GTM_gtm_cls2.MCS2_MEM[914].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM915 GTM_gtm_cls2.MCS2_MEM[915].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM916 GTM_gtm_cls2.MCS2_MEM[916].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM917 GTM_gtm_cls2.MCS2_MEM[917].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM918 GTM_gtm_cls2.MCS2_MEM[918].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM919 GTM_gtm_cls2.MCS2_MEM[919].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM920 GTM_gtm_cls2.MCS2_MEM[920].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM921 GTM_gtm_cls2.MCS2_MEM[921].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM922 GTM_gtm_cls2.MCS2_MEM[922].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM923 GTM_gtm_cls2.MCS2_MEM[923].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM924 GTM_gtm_cls2.MCS2_MEM[924].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM925 GTM_gtm_cls2.MCS2_MEM[925].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM926 GTM_gtm_cls2.MCS2_MEM[926].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM927 GTM_gtm_cls2.MCS2_MEM[927].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM928 GTM_gtm_cls2.MCS2_MEM[928].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM929 GTM_gtm_cls2.MCS2_MEM[929].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM930 GTM_gtm_cls2.MCS2_MEM[930].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM931 GTM_gtm_cls2.MCS2_MEM[931].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM932 GTM_gtm_cls2.MCS2_MEM[932].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM933 GTM_gtm_cls2.MCS2_MEM[933].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM934 GTM_gtm_cls2.MCS2_MEM[934].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM935 GTM_gtm_cls2.MCS2_MEM[935].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM936 GTM_gtm_cls2.MCS2_MEM[936].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM937 GTM_gtm_cls2.MCS2_MEM[937].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM938 GTM_gtm_cls2.MCS2_MEM[938].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM939 GTM_gtm_cls2.MCS2_MEM[939].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM940 GTM_gtm_cls2.MCS2_MEM[940].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM941 GTM_gtm_cls2.MCS2_MEM[941].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM942 GTM_gtm_cls2.MCS2_MEM[942].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM943 GTM_gtm_cls2.MCS2_MEM[943].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM944 GTM_gtm_cls2.MCS2_MEM[944].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM945 GTM_gtm_cls2.MCS2_MEM[945].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM946 GTM_gtm_cls2.MCS2_MEM[946].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM947 GTM_gtm_cls2.MCS2_MEM[947].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM948 GTM_gtm_cls2.MCS2_MEM[948].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM949 GTM_gtm_cls2.MCS2_MEM[949].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM950 GTM_gtm_cls2.MCS2_MEM[950].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM951 GTM_gtm_cls2.MCS2_MEM[951].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM952 GTM_gtm_cls2.MCS2_MEM[952].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM953 GTM_gtm_cls2.MCS2_MEM[953].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM954 GTM_gtm_cls2.MCS2_MEM[954].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM955 GTM_gtm_cls2.MCS2_MEM[955].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM956 GTM_gtm_cls2.MCS2_MEM[956].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM957 GTM_gtm_cls2.MCS2_MEM[957].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM958 GTM_gtm_cls2.MCS2_MEM[958].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM959 GTM_gtm_cls2.MCS2_MEM[959].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM960 GTM_gtm_cls2.MCS2_MEM[960].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM961 GTM_gtm_cls2.MCS2_MEM[961].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM962 GTM_gtm_cls2.MCS2_MEM[962].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM963 GTM_gtm_cls2.MCS2_MEM[963].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM964 GTM_gtm_cls2.MCS2_MEM[964].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM965 GTM_gtm_cls2.MCS2_MEM[965].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM966 GTM_gtm_cls2.MCS2_MEM[966].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM967 GTM_gtm_cls2.MCS2_MEM[967].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM968 GTM_gtm_cls2.MCS2_MEM[968].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM969 GTM_gtm_cls2.MCS2_MEM[969].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM970 GTM_gtm_cls2.MCS2_MEM[970].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM971 GTM_gtm_cls2.MCS2_MEM[971].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM972 GTM_gtm_cls2.MCS2_MEM[972].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM973 GTM_gtm_cls2.MCS2_MEM[973].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM974 GTM_gtm_cls2.MCS2_MEM[974].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM975 GTM_gtm_cls2.MCS2_MEM[975].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM976 GTM_gtm_cls2.MCS2_MEM[976].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM977 GTM_gtm_cls2.MCS2_MEM[977].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM978 GTM_gtm_cls2.MCS2_MEM[978].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM979 GTM_gtm_cls2.MCS2_MEM[979].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM980 GTM_gtm_cls2.MCS2_MEM[980].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM981 GTM_gtm_cls2.MCS2_MEM[981].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM982 GTM_gtm_cls2.MCS2_MEM[982].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM983 GTM_gtm_cls2.MCS2_MEM[983].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM984 GTM_gtm_cls2.MCS2_MEM[984].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM985 GTM_gtm_cls2.MCS2_MEM[985].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM986 GTM_gtm_cls2.MCS2_MEM[986].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM987 GTM_gtm_cls2.MCS2_MEM[987].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM988 GTM_gtm_cls2.MCS2_MEM[988].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM989 GTM_gtm_cls2.MCS2_MEM[989].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM990 GTM_gtm_cls2.MCS2_MEM[990].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM991 GTM_gtm_cls2.MCS2_MEM[991].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM992 GTM_gtm_cls2.MCS2_MEM[992].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM993 GTM_gtm_cls2.MCS2_MEM[993].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM994 GTM_gtm_cls2.MCS2_MEM[994].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM995 GTM_gtm_cls2.MCS2_MEM[995].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM996 GTM_gtm_cls2.MCS2_MEM[996].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM997 GTM_gtm_cls2.MCS2_MEM[997].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM998 GTM_gtm_cls2.MCS2_MEM[998].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM999 GTM_gtm_cls2.MCS2_MEM[999].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1000 GTM_gtm_cls2.MCS2_MEM[1000].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1001 GTM_gtm_cls2.MCS2_MEM[1001].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1002 GTM_gtm_cls2.MCS2_MEM[1002].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1003 GTM_gtm_cls2.MCS2_MEM[1003].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1004 GTM_gtm_cls2.MCS2_MEM[1004].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1005 GTM_gtm_cls2.MCS2_MEM[1005].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1006 GTM_gtm_cls2.MCS2_MEM[1006].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1007 GTM_gtm_cls2.MCS2_MEM[1007].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1008 GTM_gtm_cls2.MCS2_MEM[1008].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1009 GTM_gtm_cls2.MCS2_MEM[1009].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1010 GTM_gtm_cls2.MCS2_MEM[1010].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1011 GTM_gtm_cls2.MCS2_MEM[1011].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1012 GTM_gtm_cls2.MCS2_MEM[1012].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1013 GTM_gtm_cls2.MCS2_MEM[1013].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1014 GTM_gtm_cls2.MCS2_MEM[1014].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1015 GTM_gtm_cls2.MCS2_MEM[1015].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1016 GTM_gtm_cls2.MCS2_MEM[1016].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1017 GTM_gtm_cls2.MCS2_MEM[1017].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1018 GTM_gtm_cls2.MCS2_MEM[1018].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1019 GTM_gtm_cls2.MCS2_MEM[1019].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1020 GTM_gtm_cls2.MCS2_MEM[1020].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1021 GTM_gtm_cls2.MCS2_MEM[1021].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1022 GTM_gtm_cls2.MCS2_MEM[1022].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1023 GTM_gtm_cls2.MCS2_MEM[1023].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1024 GTM_gtm_cls2.MCS2_MEM[1024].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1025 GTM_gtm_cls2.MCS2_MEM[1025].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1026 GTM_gtm_cls2.MCS2_MEM[1026].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1027 GTM_gtm_cls2.MCS2_MEM[1027].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1028 GTM_gtm_cls2.MCS2_MEM[1028].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1029 GTM_gtm_cls2.MCS2_MEM[1029].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1030 GTM_gtm_cls2.MCS2_MEM[1030].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1031 GTM_gtm_cls2.MCS2_MEM[1031].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1032 GTM_gtm_cls2.MCS2_MEM[1032].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1033 GTM_gtm_cls2.MCS2_MEM[1033].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1034 GTM_gtm_cls2.MCS2_MEM[1034].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1035 GTM_gtm_cls2.MCS2_MEM[1035].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1036 GTM_gtm_cls2.MCS2_MEM[1036].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1037 GTM_gtm_cls2.MCS2_MEM[1037].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1038 GTM_gtm_cls2.MCS2_MEM[1038].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1039 GTM_gtm_cls2.MCS2_MEM[1039].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1040 GTM_gtm_cls2.MCS2_MEM[1040].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1041 GTM_gtm_cls2.MCS2_MEM[1041].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1042 GTM_gtm_cls2.MCS2_MEM[1042].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1043 GTM_gtm_cls2.MCS2_MEM[1043].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1044 GTM_gtm_cls2.MCS2_MEM[1044].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1045 GTM_gtm_cls2.MCS2_MEM[1045].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1046 GTM_gtm_cls2.MCS2_MEM[1046].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1047 GTM_gtm_cls2.MCS2_MEM[1047].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1048 GTM_gtm_cls2.MCS2_MEM[1048].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1049 GTM_gtm_cls2.MCS2_MEM[1049].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1050 GTM_gtm_cls2.MCS2_MEM[1050].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1051 GTM_gtm_cls2.MCS2_MEM[1051].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1052 GTM_gtm_cls2.MCS2_MEM[1052].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1053 GTM_gtm_cls2.MCS2_MEM[1053].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1054 GTM_gtm_cls2.MCS2_MEM[1054].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1055 GTM_gtm_cls2.MCS2_MEM[1055].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1056 GTM_gtm_cls2.MCS2_MEM[1056].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1057 GTM_gtm_cls2.MCS2_MEM[1057].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1058 GTM_gtm_cls2.MCS2_MEM[1058].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1059 GTM_gtm_cls2.MCS2_MEM[1059].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1060 GTM_gtm_cls2.MCS2_MEM[1060].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1061 GTM_gtm_cls2.MCS2_MEM[1061].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1062 GTM_gtm_cls2.MCS2_MEM[1062].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1063 GTM_gtm_cls2.MCS2_MEM[1063].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1064 GTM_gtm_cls2.MCS2_MEM[1064].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1065 GTM_gtm_cls2.MCS2_MEM[1065].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1066 GTM_gtm_cls2.MCS2_MEM[1066].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1067 GTM_gtm_cls2.MCS2_MEM[1067].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1068 GTM_gtm_cls2.MCS2_MEM[1068].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1069 GTM_gtm_cls2.MCS2_MEM[1069].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1070 GTM_gtm_cls2.MCS2_MEM[1070].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1071 GTM_gtm_cls2.MCS2_MEM[1071].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1072 GTM_gtm_cls2.MCS2_MEM[1072].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1073 GTM_gtm_cls2.MCS2_MEM[1073].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1074 GTM_gtm_cls2.MCS2_MEM[1074].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1075 GTM_gtm_cls2.MCS2_MEM[1075].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1076 GTM_gtm_cls2.MCS2_MEM[1076].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1077 GTM_gtm_cls2.MCS2_MEM[1077].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1078 GTM_gtm_cls2.MCS2_MEM[1078].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1079 GTM_gtm_cls2.MCS2_MEM[1079].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1080 GTM_gtm_cls2.MCS2_MEM[1080].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1081 GTM_gtm_cls2.MCS2_MEM[1081].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1082 GTM_gtm_cls2.MCS2_MEM[1082].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1083 GTM_gtm_cls2.MCS2_MEM[1083].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1084 GTM_gtm_cls2.MCS2_MEM[1084].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1085 GTM_gtm_cls2.MCS2_MEM[1085].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1086 GTM_gtm_cls2.MCS2_MEM[1086].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1087 GTM_gtm_cls2.MCS2_MEM[1087].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1088 GTM_gtm_cls2.MCS2_MEM[1088].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1089 GTM_gtm_cls2.MCS2_MEM[1089].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1090 GTM_gtm_cls2.MCS2_MEM[1090].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1091 GTM_gtm_cls2.MCS2_MEM[1091].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1092 GTM_gtm_cls2.MCS2_MEM[1092].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1093 GTM_gtm_cls2.MCS2_MEM[1093].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1094 GTM_gtm_cls2.MCS2_MEM[1094].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1095 GTM_gtm_cls2.MCS2_MEM[1095].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1096 GTM_gtm_cls2.MCS2_MEM[1096].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1097 GTM_gtm_cls2.MCS2_MEM[1097].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1098 GTM_gtm_cls2.MCS2_MEM[1098].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1099 GTM_gtm_cls2.MCS2_MEM[1099].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1100 GTM_gtm_cls2.MCS2_MEM[1100].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1101 GTM_gtm_cls2.MCS2_MEM[1101].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1102 GTM_gtm_cls2.MCS2_MEM[1102].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1103 GTM_gtm_cls2.MCS2_MEM[1103].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1104 GTM_gtm_cls2.MCS2_MEM[1104].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1105 GTM_gtm_cls2.MCS2_MEM[1105].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1106 GTM_gtm_cls2.MCS2_MEM[1106].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1107 GTM_gtm_cls2.MCS2_MEM[1107].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1108 GTM_gtm_cls2.MCS2_MEM[1108].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1109 GTM_gtm_cls2.MCS2_MEM[1109].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1110 GTM_gtm_cls2.MCS2_MEM[1110].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1111 GTM_gtm_cls2.MCS2_MEM[1111].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1112 GTM_gtm_cls2.MCS2_MEM[1112].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1113 GTM_gtm_cls2.MCS2_MEM[1113].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1114 GTM_gtm_cls2.MCS2_MEM[1114].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1115 GTM_gtm_cls2.MCS2_MEM[1115].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1116 GTM_gtm_cls2.MCS2_MEM[1116].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1117 GTM_gtm_cls2.MCS2_MEM[1117].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1118 GTM_gtm_cls2.MCS2_MEM[1118].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1119 GTM_gtm_cls2.MCS2_MEM[1119].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1120 GTM_gtm_cls2.MCS2_MEM[1120].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1121 GTM_gtm_cls2.MCS2_MEM[1121].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1122 GTM_gtm_cls2.MCS2_MEM[1122].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1123 GTM_gtm_cls2.MCS2_MEM[1123].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1124 GTM_gtm_cls2.MCS2_MEM[1124].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1125 GTM_gtm_cls2.MCS2_MEM[1125].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1126 GTM_gtm_cls2.MCS2_MEM[1126].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1127 GTM_gtm_cls2.MCS2_MEM[1127].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1128 GTM_gtm_cls2.MCS2_MEM[1128].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1129 GTM_gtm_cls2.MCS2_MEM[1129].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1130 GTM_gtm_cls2.MCS2_MEM[1130].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1131 GTM_gtm_cls2.MCS2_MEM[1131].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1132 GTM_gtm_cls2.MCS2_MEM[1132].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1133 GTM_gtm_cls2.MCS2_MEM[1133].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1134 GTM_gtm_cls2.MCS2_MEM[1134].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1135 GTM_gtm_cls2.MCS2_MEM[1135].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1136 GTM_gtm_cls2.MCS2_MEM[1136].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1137 GTM_gtm_cls2.MCS2_MEM[1137].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1138 GTM_gtm_cls2.MCS2_MEM[1138].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1139 GTM_gtm_cls2.MCS2_MEM[1139].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1140 GTM_gtm_cls2.MCS2_MEM[1140].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1141 GTM_gtm_cls2.MCS2_MEM[1141].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1142 GTM_gtm_cls2.MCS2_MEM[1142].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1143 GTM_gtm_cls2.MCS2_MEM[1143].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1144 GTM_gtm_cls2.MCS2_MEM[1144].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1145 GTM_gtm_cls2.MCS2_MEM[1145].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1146 GTM_gtm_cls2.MCS2_MEM[1146].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1147 GTM_gtm_cls2.MCS2_MEM[1147].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1148 GTM_gtm_cls2.MCS2_MEM[1148].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1149 GTM_gtm_cls2.MCS2_MEM[1149].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1150 GTM_gtm_cls2.MCS2_MEM[1150].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1151 GTM_gtm_cls2.MCS2_MEM[1151].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1152 GTM_gtm_cls2.MCS2_MEM[1152].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1153 GTM_gtm_cls2.MCS2_MEM[1153].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1154 GTM_gtm_cls2.MCS2_MEM[1154].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1155 GTM_gtm_cls2.MCS2_MEM[1155].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1156 GTM_gtm_cls2.MCS2_MEM[1156].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1157 GTM_gtm_cls2.MCS2_MEM[1157].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1158 GTM_gtm_cls2.MCS2_MEM[1158].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1159 GTM_gtm_cls2.MCS2_MEM[1159].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1160 GTM_gtm_cls2.MCS2_MEM[1160].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1161 GTM_gtm_cls2.MCS2_MEM[1161].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1162 GTM_gtm_cls2.MCS2_MEM[1162].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1163 GTM_gtm_cls2.MCS2_MEM[1163].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1164 GTM_gtm_cls2.MCS2_MEM[1164].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1165 GTM_gtm_cls2.MCS2_MEM[1165].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1166 GTM_gtm_cls2.MCS2_MEM[1166].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1167 GTM_gtm_cls2.MCS2_MEM[1167].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1168 GTM_gtm_cls2.MCS2_MEM[1168].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1169 GTM_gtm_cls2.MCS2_MEM[1169].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1170 GTM_gtm_cls2.MCS2_MEM[1170].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1171 GTM_gtm_cls2.MCS2_MEM[1171].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1172 GTM_gtm_cls2.MCS2_MEM[1172].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1173 GTM_gtm_cls2.MCS2_MEM[1173].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1174 GTM_gtm_cls2.MCS2_MEM[1174].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1175 GTM_gtm_cls2.MCS2_MEM[1175].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1176 GTM_gtm_cls2.MCS2_MEM[1176].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1177 GTM_gtm_cls2.MCS2_MEM[1177].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1178 GTM_gtm_cls2.MCS2_MEM[1178].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1179 GTM_gtm_cls2.MCS2_MEM[1179].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1180 GTM_gtm_cls2.MCS2_MEM[1180].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1181 GTM_gtm_cls2.MCS2_MEM[1181].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1182 GTM_gtm_cls2.MCS2_MEM[1182].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1183 GTM_gtm_cls2.MCS2_MEM[1183].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1184 GTM_gtm_cls2.MCS2_MEM[1184].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1185 GTM_gtm_cls2.MCS2_MEM[1185].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1186 GTM_gtm_cls2.MCS2_MEM[1186].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1187 GTM_gtm_cls2.MCS2_MEM[1187].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1188 GTM_gtm_cls2.MCS2_MEM[1188].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1189 GTM_gtm_cls2.MCS2_MEM[1189].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1190 GTM_gtm_cls2.MCS2_MEM[1190].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1191 GTM_gtm_cls2.MCS2_MEM[1191].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1192 GTM_gtm_cls2.MCS2_MEM[1192].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1193 GTM_gtm_cls2.MCS2_MEM[1193].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1194 GTM_gtm_cls2.MCS2_MEM[1194].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1195 GTM_gtm_cls2.MCS2_MEM[1195].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1196 GTM_gtm_cls2.MCS2_MEM[1196].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1197 GTM_gtm_cls2.MCS2_MEM[1197].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1198 GTM_gtm_cls2.MCS2_MEM[1198].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1199 GTM_gtm_cls2.MCS2_MEM[1199].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1200 GTM_gtm_cls2.MCS2_MEM[1200].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1201 GTM_gtm_cls2.MCS2_MEM[1201].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1202 GTM_gtm_cls2.MCS2_MEM[1202].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1203 GTM_gtm_cls2.MCS2_MEM[1203].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1204 GTM_gtm_cls2.MCS2_MEM[1204].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1205 GTM_gtm_cls2.MCS2_MEM[1205].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1206 GTM_gtm_cls2.MCS2_MEM[1206].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1207 GTM_gtm_cls2.MCS2_MEM[1207].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1208 GTM_gtm_cls2.MCS2_MEM[1208].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1209 GTM_gtm_cls2.MCS2_MEM[1209].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1210 GTM_gtm_cls2.MCS2_MEM[1210].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1211 GTM_gtm_cls2.MCS2_MEM[1211].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1212 GTM_gtm_cls2.MCS2_MEM[1212].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1213 GTM_gtm_cls2.MCS2_MEM[1213].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1214 GTM_gtm_cls2.MCS2_MEM[1214].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1215 GTM_gtm_cls2.MCS2_MEM[1215].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1216 GTM_gtm_cls2.MCS2_MEM[1216].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1217 GTM_gtm_cls2.MCS2_MEM[1217].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1218 GTM_gtm_cls2.MCS2_MEM[1218].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1219 GTM_gtm_cls2.MCS2_MEM[1219].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1220 GTM_gtm_cls2.MCS2_MEM[1220].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1221 GTM_gtm_cls2.MCS2_MEM[1221].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1222 GTM_gtm_cls2.MCS2_MEM[1222].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1223 GTM_gtm_cls2.MCS2_MEM[1223].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1224 GTM_gtm_cls2.MCS2_MEM[1224].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1225 GTM_gtm_cls2.MCS2_MEM[1225].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1226 GTM_gtm_cls2.MCS2_MEM[1226].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1227 GTM_gtm_cls2.MCS2_MEM[1227].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1228 GTM_gtm_cls2.MCS2_MEM[1228].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1229 GTM_gtm_cls2.MCS2_MEM[1229].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1230 GTM_gtm_cls2.MCS2_MEM[1230].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1231 GTM_gtm_cls2.MCS2_MEM[1231].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1232 GTM_gtm_cls2.MCS2_MEM[1232].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1233 GTM_gtm_cls2.MCS2_MEM[1233].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1234 GTM_gtm_cls2.MCS2_MEM[1234].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1235 GTM_gtm_cls2.MCS2_MEM[1235].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1236 GTM_gtm_cls2.MCS2_MEM[1236].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1237 GTM_gtm_cls2.MCS2_MEM[1237].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1238 GTM_gtm_cls2.MCS2_MEM[1238].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1239 GTM_gtm_cls2.MCS2_MEM[1239].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1240 GTM_gtm_cls2.MCS2_MEM[1240].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1241 GTM_gtm_cls2.MCS2_MEM[1241].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1242 GTM_gtm_cls2.MCS2_MEM[1242].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1243 GTM_gtm_cls2.MCS2_MEM[1243].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1244 GTM_gtm_cls2.MCS2_MEM[1244].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1245 GTM_gtm_cls2.MCS2_MEM[1245].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1246 GTM_gtm_cls2.MCS2_MEM[1246].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1247 GTM_gtm_cls2.MCS2_MEM[1247].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1248 GTM_gtm_cls2.MCS2_MEM[1248].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1249 GTM_gtm_cls2.MCS2_MEM[1249].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1250 GTM_gtm_cls2.MCS2_MEM[1250].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1251 GTM_gtm_cls2.MCS2_MEM[1251].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1252 GTM_gtm_cls2.MCS2_MEM[1252].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1253 GTM_gtm_cls2.MCS2_MEM[1253].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1254 GTM_gtm_cls2.MCS2_MEM[1254].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1255 GTM_gtm_cls2.MCS2_MEM[1255].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1256 GTM_gtm_cls2.MCS2_MEM[1256].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1257 GTM_gtm_cls2.MCS2_MEM[1257].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1258 GTM_gtm_cls2.MCS2_MEM[1258].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1259 GTM_gtm_cls2.MCS2_MEM[1259].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1260 GTM_gtm_cls2.MCS2_MEM[1260].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1261 GTM_gtm_cls2.MCS2_MEM[1261].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1262 GTM_gtm_cls2.MCS2_MEM[1262].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1263 GTM_gtm_cls2.MCS2_MEM[1263].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1264 GTM_gtm_cls2.MCS2_MEM[1264].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1265 GTM_gtm_cls2.MCS2_MEM[1265].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1266 GTM_gtm_cls2.MCS2_MEM[1266].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1267 GTM_gtm_cls2.MCS2_MEM[1267].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1268 GTM_gtm_cls2.MCS2_MEM[1268].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1269 GTM_gtm_cls2.MCS2_MEM[1269].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1270 GTM_gtm_cls2.MCS2_MEM[1270].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1271 GTM_gtm_cls2.MCS2_MEM[1271].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1272 GTM_gtm_cls2.MCS2_MEM[1272].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1273 GTM_gtm_cls2.MCS2_MEM[1273].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1274 GTM_gtm_cls2.MCS2_MEM[1274].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1275 GTM_gtm_cls2.MCS2_MEM[1275].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1276 GTM_gtm_cls2.MCS2_MEM[1276].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1277 GTM_gtm_cls2.MCS2_MEM[1277].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1278 GTM_gtm_cls2.MCS2_MEM[1278].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1279 GTM_gtm_cls2.MCS2_MEM[1279].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1280 GTM_gtm_cls2.MCS2_MEM[1280].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1281 GTM_gtm_cls2.MCS2_MEM[1281].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1282 GTM_gtm_cls2.MCS2_MEM[1282].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1283 GTM_gtm_cls2.MCS2_MEM[1283].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1284 GTM_gtm_cls2.MCS2_MEM[1284].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1285 GTM_gtm_cls2.MCS2_MEM[1285].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1286 GTM_gtm_cls2.MCS2_MEM[1286].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1287 GTM_gtm_cls2.MCS2_MEM[1287].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1288 GTM_gtm_cls2.MCS2_MEM[1288].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1289 GTM_gtm_cls2.MCS2_MEM[1289].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1290 GTM_gtm_cls2.MCS2_MEM[1290].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1291 GTM_gtm_cls2.MCS2_MEM[1291].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1292 GTM_gtm_cls2.MCS2_MEM[1292].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1293 GTM_gtm_cls2.MCS2_MEM[1293].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1294 GTM_gtm_cls2.MCS2_MEM[1294].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1295 GTM_gtm_cls2.MCS2_MEM[1295].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1296 GTM_gtm_cls2.MCS2_MEM[1296].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1297 GTM_gtm_cls2.MCS2_MEM[1297].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1298 GTM_gtm_cls2.MCS2_MEM[1298].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1299 GTM_gtm_cls2.MCS2_MEM[1299].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1300 GTM_gtm_cls2.MCS2_MEM[1300].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1301 GTM_gtm_cls2.MCS2_MEM[1301].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1302 GTM_gtm_cls2.MCS2_MEM[1302].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1303 GTM_gtm_cls2.MCS2_MEM[1303].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1304 GTM_gtm_cls2.MCS2_MEM[1304].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1305 GTM_gtm_cls2.MCS2_MEM[1305].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1306 GTM_gtm_cls2.MCS2_MEM[1306].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1307 GTM_gtm_cls2.MCS2_MEM[1307].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1308 GTM_gtm_cls2.MCS2_MEM[1308].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1309 GTM_gtm_cls2.MCS2_MEM[1309].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1310 GTM_gtm_cls2.MCS2_MEM[1310].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1311 GTM_gtm_cls2.MCS2_MEM[1311].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1312 GTM_gtm_cls2.MCS2_MEM[1312].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1313 GTM_gtm_cls2.MCS2_MEM[1313].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1314 GTM_gtm_cls2.MCS2_MEM[1314].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1315 GTM_gtm_cls2.MCS2_MEM[1315].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1316 GTM_gtm_cls2.MCS2_MEM[1316].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1317 GTM_gtm_cls2.MCS2_MEM[1317].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1318 GTM_gtm_cls2.MCS2_MEM[1318].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1319 GTM_gtm_cls2.MCS2_MEM[1319].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1320 GTM_gtm_cls2.MCS2_MEM[1320].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1321 GTM_gtm_cls2.MCS2_MEM[1321].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1322 GTM_gtm_cls2.MCS2_MEM[1322].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1323 GTM_gtm_cls2.MCS2_MEM[1323].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1324 GTM_gtm_cls2.MCS2_MEM[1324].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1325 GTM_gtm_cls2.MCS2_MEM[1325].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1326 GTM_gtm_cls2.MCS2_MEM[1326].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1327 GTM_gtm_cls2.MCS2_MEM[1327].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1328 GTM_gtm_cls2.MCS2_MEM[1328].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1329 GTM_gtm_cls2.MCS2_MEM[1329].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1330 GTM_gtm_cls2.MCS2_MEM[1330].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1331 GTM_gtm_cls2.MCS2_MEM[1331].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1332 GTM_gtm_cls2.MCS2_MEM[1332].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1333 GTM_gtm_cls2.MCS2_MEM[1333].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1334 GTM_gtm_cls2.MCS2_MEM[1334].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1335 GTM_gtm_cls2.MCS2_MEM[1335].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1336 GTM_gtm_cls2.MCS2_MEM[1336].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1337 GTM_gtm_cls2.MCS2_MEM[1337].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1338 GTM_gtm_cls2.MCS2_MEM[1338].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1339 GTM_gtm_cls2.MCS2_MEM[1339].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1340 GTM_gtm_cls2.MCS2_MEM[1340].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1341 GTM_gtm_cls2.MCS2_MEM[1341].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1342 GTM_gtm_cls2.MCS2_MEM[1342].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1343 GTM_gtm_cls2.MCS2_MEM[1343].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1344 GTM_gtm_cls2.MCS2_MEM[1344].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1345 GTM_gtm_cls2.MCS2_MEM[1345].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1346 GTM_gtm_cls2.MCS2_MEM[1346].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1347 GTM_gtm_cls2.MCS2_MEM[1347].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1348 GTM_gtm_cls2.MCS2_MEM[1348].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1349 GTM_gtm_cls2.MCS2_MEM[1349].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1350 GTM_gtm_cls2.MCS2_MEM[1350].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1351 GTM_gtm_cls2.MCS2_MEM[1351].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1352 GTM_gtm_cls2.MCS2_MEM[1352].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1353 GTM_gtm_cls2.MCS2_MEM[1353].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1354 GTM_gtm_cls2.MCS2_MEM[1354].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1355 GTM_gtm_cls2.MCS2_MEM[1355].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1356 GTM_gtm_cls2.MCS2_MEM[1356].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1357 GTM_gtm_cls2.MCS2_MEM[1357].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1358 GTM_gtm_cls2.MCS2_MEM[1358].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1359 GTM_gtm_cls2.MCS2_MEM[1359].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1360 GTM_gtm_cls2.MCS2_MEM[1360].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1361 GTM_gtm_cls2.MCS2_MEM[1361].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1362 GTM_gtm_cls2.MCS2_MEM[1362].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1363 GTM_gtm_cls2.MCS2_MEM[1363].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1364 GTM_gtm_cls2.MCS2_MEM[1364].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1365 GTM_gtm_cls2.MCS2_MEM[1365].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1366 GTM_gtm_cls2.MCS2_MEM[1366].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1367 GTM_gtm_cls2.MCS2_MEM[1367].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1368 GTM_gtm_cls2.MCS2_MEM[1368].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1369 GTM_gtm_cls2.MCS2_MEM[1369].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1370 GTM_gtm_cls2.MCS2_MEM[1370].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1371 GTM_gtm_cls2.MCS2_MEM[1371].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1372 GTM_gtm_cls2.MCS2_MEM[1372].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1373 GTM_gtm_cls2.MCS2_MEM[1373].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1374 GTM_gtm_cls2.MCS2_MEM[1374].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1375 GTM_gtm_cls2.MCS2_MEM[1375].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1376 GTM_gtm_cls2.MCS2_MEM[1376].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1377 GTM_gtm_cls2.MCS2_MEM[1377].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1378 GTM_gtm_cls2.MCS2_MEM[1378].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1379 GTM_gtm_cls2.MCS2_MEM[1379].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1380 GTM_gtm_cls2.MCS2_MEM[1380].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1381 GTM_gtm_cls2.MCS2_MEM[1381].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1382 GTM_gtm_cls2.MCS2_MEM[1382].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1383 GTM_gtm_cls2.MCS2_MEM[1383].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1384 GTM_gtm_cls2.MCS2_MEM[1384].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1385 GTM_gtm_cls2.MCS2_MEM[1385].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1386 GTM_gtm_cls2.MCS2_MEM[1386].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1387 GTM_gtm_cls2.MCS2_MEM[1387].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1388 GTM_gtm_cls2.MCS2_MEM[1388].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1389 GTM_gtm_cls2.MCS2_MEM[1389].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1390 GTM_gtm_cls2.MCS2_MEM[1390].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1391 GTM_gtm_cls2.MCS2_MEM[1391].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1392 GTM_gtm_cls2.MCS2_MEM[1392].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1393 GTM_gtm_cls2.MCS2_MEM[1393].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1394 GTM_gtm_cls2.MCS2_MEM[1394].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1395 GTM_gtm_cls2.MCS2_MEM[1395].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1396 GTM_gtm_cls2.MCS2_MEM[1396].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1397 GTM_gtm_cls2.MCS2_MEM[1397].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1398 GTM_gtm_cls2.MCS2_MEM[1398].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1399 GTM_gtm_cls2.MCS2_MEM[1399].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1400 GTM_gtm_cls2.MCS2_MEM[1400].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1401 GTM_gtm_cls2.MCS2_MEM[1401].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1402 GTM_gtm_cls2.MCS2_MEM[1402].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1403 GTM_gtm_cls2.MCS2_MEM[1403].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1404 GTM_gtm_cls2.MCS2_MEM[1404].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1405 GTM_gtm_cls2.MCS2_MEM[1405].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1406 GTM_gtm_cls2.MCS2_MEM[1406].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1407 GTM_gtm_cls2.MCS2_MEM[1407].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1408 GTM_gtm_cls2.MCS2_MEM[1408].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1409 GTM_gtm_cls2.MCS2_MEM[1409].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1410 GTM_gtm_cls2.MCS2_MEM[1410].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1411 GTM_gtm_cls2.MCS2_MEM[1411].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1412 GTM_gtm_cls2.MCS2_MEM[1412].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1413 GTM_gtm_cls2.MCS2_MEM[1413].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1414 GTM_gtm_cls2.MCS2_MEM[1414].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1415 GTM_gtm_cls2.MCS2_MEM[1415].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1416 GTM_gtm_cls2.MCS2_MEM[1416].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1417 GTM_gtm_cls2.MCS2_MEM[1417].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1418 GTM_gtm_cls2.MCS2_MEM[1418].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1419 GTM_gtm_cls2.MCS2_MEM[1419].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1420 GTM_gtm_cls2.MCS2_MEM[1420].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1421 GTM_gtm_cls2.MCS2_MEM[1421].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1422 GTM_gtm_cls2.MCS2_MEM[1422].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1423 GTM_gtm_cls2.MCS2_MEM[1423].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1424 GTM_gtm_cls2.MCS2_MEM[1424].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1425 GTM_gtm_cls2.MCS2_MEM[1425].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1426 GTM_gtm_cls2.MCS2_MEM[1426].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1427 GTM_gtm_cls2.MCS2_MEM[1427].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1428 GTM_gtm_cls2.MCS2_MEM[1428].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1429 GTM_gtm_cls2.MCS2_MEM[1429].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1430 GTM_gtm_cls2.MCS2_MEM[1430].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1431 GTM_gtm_cls2.MCS2_MEM[1431].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1432 GTM_gtm_cls2.MCS2_MEM[1432].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1433 GTM_gtm_cls2.MCS2_MEM[1433].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1434 GTM_gtm_cls2.MCS2_MEM[1434].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1435 GTM_gtm_cls2.MCS2_MEM[1435].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1436 GTM_gtm_cls2.MCS2_MEM[1436].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1437 GTM_gtm_cls2.MCS2_MEM[1437].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1438 GTM_gtm_cls2.MCS2_MEM[1438].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1439 GTM_gtm_cls2.MCS2_MEM[1439].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1440 GTM_gtm_cls2.MCS2_MEM[1440].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1441 GTM_gtm_cls2.MCS2_MEM[1441].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1442 GTM_gtm_cls2.MCS2_MEM[1442].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1443 GTM_gtm_cls2.MCS2_MEM[1443].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1444 GTM_gtm_cls2.MCS2_MEM[1444].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1445 GTM_gtm_cls2.MCS2_MEM[1445].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1446 GTM_gtm_cls2.MCS2_MEM[1446].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1447 GTM_gtm_cls2.MCS2_MEM[1447].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1448 GTM_gtm_cls2.MCS2_MEM[1448].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1449 GTM_gtm_cls2.MCS2_MEM[1449].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1450 GTM_gtm_cls2.MCS2_MEM[1450].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1451 GTM_gtm_cls2.MCS2_MEM[1451].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1452 GTM_gtm_cls2.MCS2_MEM[1452].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1453 GTM_gtm_cls2.MCS2_MEM[1453].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1454 GTM_gtm_cls2.MCS2_MEM[1454].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1455 GTM_gtm_cls2.MCS2_MEM[1455].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1456 GTM_gtm_cls2.MCS2_MEM[1456].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1457 GTM_gtm_cls2.MCS2_MEM[1457].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1458 GTM_gtm_cls2.MCS2_MEM[1458].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1459 GTM_gtm_cls2.MCS2_MEM[1459].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1460 GTM_gtm_cls2.MCS2_MEM[1460].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1461 GTM_gtm_cls2.MCS2_MEM[1461].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1462 GTM_gtm_cls2.MCS2_MEM[1462].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1463 GTM_gtm_cls2.MCS2_MEM[1463].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1464 GTM_gtm_cls2.MCS2_MEM[1464].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1465 GTM_gtm_cls2.MCS2_MEM[1465].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1466 GTM_gtm_cls2.MCS2_MEM[1466].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1467 GTM_gtm_cls2.MCS2_MEM[1467].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1468 GTM_gtm_cls2.MCS2_MEM[1468].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1469 GTM_gtm_cls2.MCS2_MEM[1469].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1470 GTM_gtm_cls2.MCS2_MEM[1470].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1471 GTM_gtm_cls2.MCS2_MEM[1471].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1472 GTM_gtm_cls2.MCS2_MEM[1472].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1473 GTM_gtm_cls2.MCS2_MEM[1473].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1474 GTM_gtm_cls2.MCS2_MEM[1474].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1475 GTM_gtm_cls2.MCS2_MEM[1475].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1476 GTM_gtm_cls2.MCS2_MEM[1476].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1477 GTM_gtm_cls2.MCS2_MEM[1477].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1478 GTM_gtm_cls2.MCS2_MEM[1478].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1479 GTM_gtm_cls2.MCS2_MEM[1479].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1480 GTM_gtm_cls2.MCS2_MEM[1480].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1481 GTM_gtm_cls2.MCS2_MEM[1481].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1482 GTM_gtm_cls2.MCS2_MEM[1482].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1483 GTM_gtm_cls2.MCS2_MEM[1483].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1484 GTM_gtm_cls2.MCS2_MEM[1484].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1485 GTM_gtm_cls2.MCS2_MEM[1485].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1486 GTM_gtm_cls2.MCS2_MEM[1486].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1487 GTM_gtm_cls2.MCS2_MEM[1487].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1488 GTM_gtm_cls2.MCS2_MEM[1488].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1489 GTM_gtm_cls2.MCS2_MEM[1489].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1490 GTM_gtm_cls2.MCS2_MEM[1490].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1491 GTM_gtm_cls2.MCS2_MEM[1491].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1492 GTM_gtm_cls2.MCS2_MEM[1492].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1493 GTM_gtm_cls2.MCS2_MEM[1493].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1494 GTM_gtm_cls2.MCS2_MEM[1494].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1495 GTM_gtm_cls2.MCS2_MEM[1495].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1496 GTM_gtm_cls2.MCS2_MEM[1496].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1497 GTM_gtm_cls2.MCS2_MEM[1497].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1498 GTM_gtm_cls2.MCS2_MEM[1498].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1499 GTM_gtm_cls2.MCS2_MEM[1499].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1500 GTM_gtm_cls2.MCS2_MEM[1500].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1501 GTM_gtm_cls2.MCS2_MEM[1501].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1502 GTM_gtm_cls2.MCS2_MEM[1502].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1503 GTM_gtm_cls2.MCS2_MEM[1503].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1504 GTM_gtm_cls2.MCS2_MEM[1504].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1505 GTM_gtm_cls2.MCS2_MEM[1505].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1506 GTM_gtm_cls2.MCS2_MEM[1506].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1507 GTM_gtm_cls2.MCS2_MEM[1507].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1508 GTM_gtm_cls2.MCS2_MEM[1508].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1509 GTM_gtm_cls2.MCS2_MEM[1509].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1510 GTM_gtm_cls2.MCS2_MEM[1510].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1511 GTM_gtm_cls2.MCS2_MEM[1511].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1512 GTM_gtm_cls2.MCS2_MEM[1512].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1513 GTM_gtm_cls2.MCS2_MEM[1513].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1514 GTM_gtm_cls2.MCS2_MEM[1514].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1515 GTM_gtm_cls2.MCS2_MEM[1515].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1516 GTM_gtm_cls2.MCS2_MEM[1516].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1517 GTM_gtm_cls2.MCS2_MEM[1517].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1518 GTM_gtm_cls2.MCS2_MEM[1518].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1519 GTM_gtm_cls2.MCS2_MEM[1519].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1520 GTM_gtm_cls2.MCS2_MEM[1520].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1521 GTM_gtm_cls2.MCS2_MEM[1521].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1522 GTM_gtm_cls2.MCS2_MEM[1522].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1523 GTM_gtm_cls2.MCS2_MEM[1523].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1524 GTM_gtm_cls2.MCS2_MEM[1524].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1525 GTM_gtm_cls2.MCS2_MEM[1525].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1526 GTM_gtm_cls2.MCS2_MEM[1526].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1527 GTM_gtm_cls2.MCS2_MEM[1527].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1528 GTM_gtm_cls2.MCS2_MEM[1528].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1529 GTM_gtm_cls2.MCS2_MEM[1529].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1530 GTM_gtm_cls2.MCS2_MEM[1530].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1531 GTM_gtm_cls2.MCS2_MEM[1531].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1532 GTM_gtm_cls2.MCS2_MEM[1532].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1533 GTM_gtm_cls2.MCS2_MEM[1533].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1534 GTM_gtm_cls2.MCS2_MEM[1534].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1535 GTM_gtm_cls2.MCS2_MEM[1535].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1536 GTM_gtm_cls2.MCS2_MEM[1536].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1537 GTM_gtm_cls2.MCS2_MEM[1537].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1538 GTM_gtm_cls2.MCS2_MEM[1538].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1539 GTM_gtm_cls2.MCS2_MEM[1539].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1540 GTM_gtm_cls2.MCS2_MEM[1540].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1541 GTM_gtm_cls2.MCS2_MEM[1541].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1542 GTM_gtm_cls2.MCS2_MEM[1542].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1543 GTM_gtm_cls2.MCS2_MEM[1543].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1544 GTM_gtm_cls2.MCS2_MEM[1544].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1545 GTM_gtm_cls2.MCS2_MEM[1545].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1546 GTM_gtm_cls2.MCS2_MEM[1546].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1547 GTM_gtm_cls2.MCS2_MEM[1547].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1548 GTM_gtm_cls2.MCS2_MEM[1548].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1549 GTM_gtm_cls2.MCS2_MEM[1549].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1550 GTM_gtm_cls2.MCS2_MEM[1550].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1551 GTM_gtm_cls2.MCS2_MEM[1551].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1552 GTM_gtm_cls2.MCS2_MEM[1552].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1553 GTM_gtm_cls2.MCS2_MEM[1553].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1554 GTM_gtm_cls2.MCS2_MEM[1554].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1555 GTM_gtm_cls2.MCS2_MEM[1555].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1556 GTM_gtm_cls2.MCS2_MEM[1556].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1557 GTM_gtm_cls2.MCS2_MEM[1557].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1558 GTM_gtm_cls2.MCS2_MEM[1558].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1559 GTM_gtm_cls2.MCS2_MEM[1559].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1560 GTM_gtm_cls2.MCS2_MEM[1560].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1561 GTM_gtm_cls2.MCS2_MEM[1561].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1562 GTM_gtm_cls2.MCS2_MEM[1562].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1563 GTM_gtm_cls2.MCS2_MEM[1563].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1564 GTM_gtm_cls2.MCS2_MEM[1564].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1565 GTM_gtm_cls2.MCS2_MEM[1565].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1566 GTM_gtm_cls2.MCS2_MEM[1566].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1567 GTM_gtm_cls2.MCS2_MEM[1567].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1568 GTM_gtm_cls2.MCS2_MEM[1568].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1569 GTM_gtm_cls2.MCS2_MEM[1569].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1570 GTM_gtm_cls2.MCS2_MEM[1570].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1571 GTM_gtm_cls2.MCS2_MEM[1571].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1572 GTM_gtm_cls2.MCS2_MEM[1572].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1573 GTM_gtm_cls2.MCS2_MEM[1573].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1574 GTM_gtm_cls2.MCS2_MEM[1574].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1575 GTM_gtm_cls2.MCS2_MEM[1575].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1576 GTM_gtm_cls2.MCS2_MEM[1576].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1577 GTM_gtm_cls2.MCS2_MEM[1577].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1578 GTM_gtm_cls2.MCS2_MEM[1578].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1579 GTM_gtm_cls2.MCS2_MEM[1579].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1580 GTM_gtm_cls2.MCS2_MEM[1580].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1581 GTM_gtm_cls2.MCS2_MEM[1581].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1582 GTM_gtm_cls2.MCS2_MEM[1582].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1583 GTM_gtm_cls2.MCS2_MEM[1583].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1584 GTM_gtm_cls2.MCS2_MEM[1584].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1585 GTM_gtm_cls2.MCS2_MEM[1585].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1586 GTM_gtm_cls2.MCS2_MEM[1586].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1587 GTM_gtm_cls2.MCS2_MEM[1587].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1588 GTM_gtm_cls2.MCS2_MEM[1588].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1589 GTM_gtm_cls2.MCS2_MEM[1589].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1590 GTM_gtm_cls2.MCS2_MEM[1590].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1591 GTM_gtm_cls2.MCS2_MEM[1591].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1592 GTM_gtm_cls2.MCS2_MEM[1592].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1593 GTM_gtm_cls2.MCS2_MEM[1593].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1594 GTM_gtm_cls2.MCS2_MEM[1594].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1595 GTM_gtm_cls2.MCS2_MEM[1595].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1596 GTM_gtm_cls2.MCS2_MEM[1596].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1597 GTM_gtm_cls2.MCS2_MEM[1597].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1598 GTM_gtm_cls2.MCS2_MEM[1598].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1599 GTM_gtm_cls2.MCS2_MEM[1599].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1600 GTM_gtm_cls2.MCS2_MEM[1600].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1601 GTM_gtm_cls2.MCS2_MEM[1601].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1602 GTM_gtm_cls2.MCS2_MEM[1602].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1603 GTM_gtm_cls2.MCS2_MEM[1603].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1604 GTM_gtm_cls2.MCS2_MEM[1604].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1605 GTM_gtm_cls2.MCS2_MEM[1605].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1606 GTM_gtm_cls2.MCS2_MEM[1606].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1607 GTM_gtm_cls2.MCS2_MEM[1607].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1608 GTM_gtm_cls2.MCS2_MEM[1608].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1609 GTM_gtm_cls2.MCS2_MEM[1609].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1610 GTM_gtm_cls2.MCS2_MEM[1610].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1611 GTM_gtm_cls2.MCS2_MEM[1611].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1612 GTM_gtm_cls2.MCS2_MEM[1612].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1613 GTM_gtm_cls2.MCS2_MEM[1613].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1614 GTM_gtm_cls2.MCS2_MEM[1614].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1615 GTM_gtm_cls2.MCS2_MEM[1615].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1616 GTM_gtm_cls2.MCS2_MEM[1616].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1617 GTM_gtm_cls2.MCS2_MEM[1617].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1618 GTM_gtm_cls2.MCS2_MEM[1618].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1619 GTM_gtm_cls2.MCS2_MEM[1619].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1620 GTM_gtm_cls2.MCS2_MEM[1620].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1621 GTM_gtm_cls2.MCS2_MEM[1621].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1622 GTM_gtm_cls2.MCS2_MEM[1622].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1623 GTM_gtm_cls2.MCS2_MEM[1623].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1624 GTM_gtm_cls2.MCS2_MEM[1624].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1625 GTM_gtm_cls2.MCS2_MEM[1625].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1626 GTM_gtm_cls2.MCS2_MEM[1626].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1627 GTM_gtm_cls2.MCS2_MEM[1627].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1628 GTM_gtm_cls2.MCS2_MEM[1628].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1629 GTM_gtm_cls2.MCS2_MEM[1629].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1630 GTM_gtm_cls2.MCS2_MEM[1630].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1631 GTM_gtm_cls2.MCS2_MEM[1631].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1632 GTM_gtm_cls2.MCS2_MEM[1632].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1633 GTM_gtm_cls2.MCS2_MEM[1633].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1634 GTM_gtm_cls2.MCS2_MEM[1634].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1635 GTM_gtm_cls2.MCS2_MEM[1635].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1636 GTM_gtm_cls2.MCS2_MEM[1636].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1637 GTM_gtm_cls2.MCS2_MEM[1637].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1638 GTM_gtm_cls2.MCS2_MEM[1638].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1639 GTM_gtm_cls2.MCS2_MEM[1639].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1640 GTM_gtm_cls2.MCS2_MEM[1640].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1641 GTM_gtm_cls2.MCS2_MEM[1641].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1642 GTM_gtm_cls2.MCS2_MEM[1642].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1643 GTM_gtm_cls2.MCS2_MEM[1643].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1644 GTM_gtm_cls2.MCS2_MEM[1644].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1645 GTM_gtm_cls2.MCS2_MEM[1645].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1646 GTM_gtm_cls2.MCS2_MEM[1646].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1647 GTM_gtm_cls2.MCS2_MEM[1647].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1648 GTM_gtm_cls2.MCS2_MEM[1648].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1649 GTM_gtm_cls2.MCS2_MEM[1649].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1650 GTM_gtm_cls2.MCS2_MEM[1650].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1651 GTM_gtm_cls2.MCS2_MEM[1651].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1652 GTM_gtm_cls2.MCS2_MEM[1652].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1653 GTM_gtm_cls2.MCS2_MEM[1653].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1654 GTM_gtm_cls2.MCS2_MEM[1654].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1655 GTM_gtm_cls2.MCS2_MEM[1655].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1656 GTM_gtm_cls2.MCS2_MEM[1656].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1657 GTM_gtm_cls2.MCS2_MEM[1657].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1658 GTM_gtm_cls2.MCS2_MEM[1658].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1659 GTM_gtm_cls2.MCS2_MEM[1659].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1660 GTM_gtm_cls2.MCS2_MEM[1660].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1661 GTM_gtm_cls2.MCS2_MEM[1661].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1662 GTM_gtm_cls2.MCS2_MEM[1662].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1663 GTM_gtm_cls2.MCS2_MEM[1663].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1664 GTM_gtm_cls2.MCS2_MEM[1664].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1665 GTM_gtm_cls2.MCS2_MEM[1665].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1666 GTM_gtm_cls2.MCS2_MEM[1666].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1667 GTM_gtm_cls2.MCS2_MEM[1667].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1668 GTM_gtm_cls2.MCS2_MEM[1668].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1669 GTM_gtm_cls2.MCS2_MEM[1669].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1670 GTM_gtm_cls2.MCS2_MEM[1670].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1671 GTM_gtm_cls2.MCS2_MEM[1671].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1672 GTM_gtm_cls2.MCS2_MEM[1672].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1673 GTM_gtm_cls2.MCS2_MEM[1673].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1674 GTM_gtm_cls2.MCS2_MEM[1674].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1675 GTM_gtm_cls2.MCS2_MEM[1675].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1676 GTM_gtm_cls2.MCS2_MEM[1676].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1677 GTM_gtm_cls2.MCS2_MEM[1677].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1678 GTM_gtm_cls2.MCS2_MEM[1678].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1679 GTM_gtm_cls2.MCS2_MEM[1679].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1680 GTM_gtm_cls2.MCS2_MEM[1680].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1681 GTM_gtm_cls2.MCS2_MEM[1681].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1682 GTM_gtm_cls2.MCS2_MEM[1682].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1683 GTM_gtm_cls2.MCS2_MEM[1683].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1684 GTM_gtm_cls2.MCS2_MEM[1684].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1685 GTM_gtm_cls2.MCS2_MEM[1685].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1686 GTM_gtm_cls2.MCS2_MEM[1686].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1687 GTM_gtm_cls2.MCS2_MEM[1687].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1688 GTM_gtm_cls2.MCS2_MEM[1688].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1689 GTM_gtm_cls2.MCS2_MEM[1689].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1690 GTM_gtm_cls2.MCS2_MEM[1690].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1691 GTM_gtm_cls2.MCS2_MEM[1691].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1692 GTM_gtm_cls2.MCS2_MEM[1692].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1693 GTM_gtm_cls2.MCS2_MEM[1693].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1694 GTM_gtm_cls2.MCS2_MEM[1694].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1695 GTM_gtm_cls2.MCS2_MEM[1695].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1696 GTM_gtm_cls2.MCS2_MEM[1696].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1697 GTM_gtm_cls2.MCS2_MEM[1697].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1698 GTM_gtm_cls2.MCS2_MEM[1698].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1699 GTM_gtm_cls2.MCS2_MEM[1699].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1700 GTM_gtm_cls2.MCS2_MEM[1700].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1701 GTM_gtm_cls2.MCS2_MEM[1701].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1702 GTM_gtm_cls2.MCS2_MEM[1702].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1703 GTM_gtm_cls2.MCS2_MEM[1703].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1704 GTM_gtm_cls2.MCS2_MEM[1704].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1705 GTM_gtm_cls2.MCS2_MEM[1705].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1706 GTM_gtm_cls2.MCS2_MEM[1706].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1707 GTM_gtm_cls2.MCS2_MEM[1707].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1708 GTM_gtm_cls2.MCS2_MEM[1708].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1709 GTM_gtm_cls2.MCS2_MEM[1709].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1710 GTM_gtm_cls2.MCS2_MEM[1710].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1711 GTM_gtm_cls2.MCS2_MEM[1711].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1712 GTM_gtm_cls2.MCS2_MEM[1712].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1713 GTM_gtm_cls2.MCS2_MEM[1713].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1714 GTM_gtm_cls2.MCS2_MEM[1714].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1715 GTM_gtm_cls2.MCS2_MEM[1715].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1716 GTM_gtm_cls2.MCS2_MEM[1716].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1717 GTM_gtm_cls2.MCS2_MEM[1717].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1718 GTM_gtm_cls2.MCS2_MEM[1718].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1719 GTM_gtm_cls2.MCS2_MEM[1719].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1720 GTM_gtm_cls2.MCS2_MEM[1720].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1721 GTM_gtm_cls2.MCS2_MEM[1721].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1722 GTM_gtm_cls2.MCS2_MEM[1722].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1723 GTM_gtm_cls2.MCS2_MEM[1723].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1724 GTM_gtm_cls2.MCS2_MEM[1724].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1725 GTM_gtm_cls2.MCS2_MEM[1725].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1726 GTM_gtm_cls2.MCS2_MEM[1726].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1727 GTM_gtm_cls2.MCS2_MEM[1727].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1728 GTM_gtm_cls2.MCS2_MEM[1728].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1729 GTM_gtm_cls2.MCS2_MEM[1729].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1730 GTM_gtm_cls2.MCS2_MEM[1730].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1731 GTM_gtm_cls2.MCS2_MEM[1731].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1732 GTM_gtm_cls2.MCS2_MEM[1732].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1733 GTM_gtm_cls2.MCS2_MEM[1733].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1734 GTM_gtm_cls2.MCS2_MEM[1734].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1735 GTM_gtm_cls2.MCS2_MEM[1735].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1736 GTM_gtm_cls2.MCS2_MEM[1736].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1737 GTM_gtm_cls2.MCS2_MEM[1737].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1738 GTM_gtm_cls2.MCS2_MEM[1738].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1739 GTM_gtm_cls2.MCS2_MEM[1739].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1740 GTM_gtm_cls2.MCS2_MEM[1740].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1741 GTM_gtm_cls2.MCS2_MEM[1741].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1742 GTM_gtm_cls2.MCS2_MEM[1742].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1743 GTM_gtm_cls2.MCS2_MEM[1743].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1744 GTM_gtm_cls2.MCS2_MEM[1744].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1745 GTM_gtm_cls2.MCS2_MEM[1745].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1746 GTM_gtm_cls2.MCS2_MEM[1746].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1747 GTM_gtm_cls2.MCS2_MEM[1747].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1748 GTM_gtm_cls2.MCS2_MEM[1748].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1749 GTM_gtm_cls2.MCS2_MEM[1749].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1750 GTM_gtm_cls2.MCS2_MEM[1750].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1751 GTM_gtm_cls2.MCS2_MEM[1751].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1752 GTM_gtm_cls2.MCS2_MEM[1752].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1753 GTM_gtm_cls2.MCS2_MEM[1753].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1754 GTM_gtm_cls2.MCS2_MEM[1754].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1755 GTM_gtm_cls2.MCS2_MEM[1755].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1756 GTM_gtm_cls2.MCS2_MEM[1756].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1757 GTM_gtm_cls2.MCS2_MEM[1757].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1758 GTM_gtm_cls2.MCS2_MEM[1758].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1759 GTM_gtm_cls2.MCS2_MEM[1759].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1760 GTM_gtm_cls2.MCS2_MEM[1760].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1761 GTM_gtm_cls2.MCS2_MEM[1761].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1762 GTM_gtm_cls2.MCS2_MEM[1762].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1763 GTM_gtm_cls2.MCS2_MEM[1763].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1764 GTM_gtm_cls2.MCS2_MEM[1764].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1765 GTM_gtm_cls2.MCS2_MEM[1765].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1766 GTM_gtm_cls2.MCS2_MEM[1766].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1767 GTM_gtm_cls2.MCS2_MEM[1767].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1768 GTM_gtm_cls2.MCS2_MEM[1768].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1769 GTM_gtm_cls2.MCS2_MEM[1769].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1770 GTM_gtm_cls2.MCS2_MEM[1770].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1771 GTM_gtm_cls2.MCS2_MEM[1771].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1772 GTM_gtm_cls2.MCS2_MEM[1772].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1773 GTM_gtm_cls2.MCS2_MEM[1773].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1774 GTM_gtm_cls2.MCS2_MEM[1774].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1775 GTM_gtm_cls2.MCS2_MEM[1775].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1776 GTM_gtm_cls2.MCS2_MEM[1776].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1777 GTM_gtm_cls2.MCS2_MEM[1777].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1778 GTM_gtm_cls2.MCS2_MEM[1778].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1779 GTM_gtm_cls2.MCS2_MEM[1779].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1780 GTM_gtm_cls2.MCS2_MEM[1780].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1781 GTM_gtm_cls2.MCS2_MEM[1781].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1782 GTM_gtm_cls2.MCS2_MEM[1782].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1783 GTM_gtm_cls2.MCS2_MEM[1783].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1784 GTM_gtm_cls2.MCS2_MEM[1784].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1785 GTM_gtm_cls2.MCS2_MEM[1785].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1786 GTM_gtm_cls2.MCS2_MEM[1786].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1787 GTM_gtm_cls2.MCS2_MEM[1787].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1788 GTM_gtm_cls2.MCS2_MEM[1788].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1789 GTM_gtm_cls2.MCS2_MEM[1789].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1790 GTM_gtm_cls2.MCS2_MEM[1790].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1791 GTM_gtm_cls2.MCS2_MEM[1791].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1792 GTM_gtm_cls2.MCS2_MEM[1792].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1793 GTM_gtm_cls2.MCS2_MEM[1793].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1794 GTM_gtm_cls2.MCS2_MEM[1794].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1795 GTM_gtm_cls2.MCS2_MEM[1795].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1796 GTM_gtm_cls2.MCS2_MEM[1796].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1797 GTM_gtm_cls2.MCS2_MEM[1797].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1798 GTM_gtm_cls2.MCS2_MEM[1798].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1799 GTM_gtm_cls2.MCS2_MEM[1799].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1800 GTM_gtm_cls2.MCS2_MEM[1800].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1801 GTM_gtm_cls2.MCS2_MEM[1801].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1802 GTM_gtm_cls2.MCS2_MEM[1802].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1803 GTM_gtm_cls2.MCS2_MEM[1803].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1804 GTM_gtm_cls2.MCS2_MEM[1804].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1805 GTM_gtm_cls2.MCS2_MEM[1805].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1806 GTM_gtm_cls2.MCS2_MEM[1806].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1807 GTM_gtm_cls2.MCS2_MEM[1807].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1808 GTM_gtm_cls2.MCS2_MEM[1808].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1809 GTM_gtm_cls2.MCS2_MEM[1809].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1810 GTM_gtm_cls2.MCS2_MEM[1810].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1811 GTM_gtm_cls2.MCS2_MEM[1811].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1812 GTM_gtm_cls2.MCS2_MEM[1812].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1813 GTM_gtm_cls2.MCS2_MEM[1813].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1814 GTM_gtm_cls2.MCS2_MEM[1814].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1815 GTM_gtm_cls2.MCS2_MEM[1815].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1816 GTM_gtm_cls2.MCS2_MEM[1816].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1817 GTM_gtm_cls2.MCS2_MEM[1817].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1818 GTM_gtm_cls2.MCS2_MEM[1818].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1819 GTM_gtm_cls2.MCS2_MEM[1819].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1820 GTM_gtm_cls2.MCS2_MEM[1820].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1821 GTM_gtm_cls2.MCS2_MEM[1821].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1822 GTM_gtm_cls2.MCS2_MEM[1822].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1823 GTM_gtm_cls2.MCS2_MEM[1823].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1824 GTM_gtm_cls2.MCS2_MEM[1824].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1825 GTM_gtm_cls2.MCS2_MEM[1825].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1826 GTM_gtm_cls2.MCS2_MEM[1826].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1827 GTM_gtm_cls2.MCS2_MEM[1827].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1828 GTM_gtm_cls2.MCS2_MEM[1828].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1829 GTM_gtm_cls2.MCS2_MEM[1829].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1830 GTM_gtm_cls2.MCS2_MEM[1830].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1831 GTM_gtm_cls2.MCS2_MEM[1831].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1832 GTM_gtm_cls2.MCS2_MEM[1832].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1833 GTM_gtm_cls2.MCS2_MEM[1833].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1834 GTM_gtm_cls2.MCS2_MEM[1834].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1835 GTM_gtm_cls2.MCS2_MEM[1835].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1836 GTM_gtm_cls2.MCS2_MEM[1836].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1837 GTM_gtm_cls2.MCS2_MEM[1837].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1838 GTM_gtm_cls2.MCS2_MEM[1838].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1839 GTM_gtm_cls2.MCS2_MEM[1839].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1840 GTM_gtm_cls2.MCS2_MEM[1840].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1841 GTM_gtm_cls2.MCS2_MEM[1841].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1842 GTM_gtm_cls2.MCS2_MEM[1842].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1843 GTM_gtm_cls2.MCS2_MEM[1843].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1844 GTM_gtm_cls2.MCS2_MEM[1844].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1845 GTM_gtm_cls2.MCS2_MEM[1845].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1846 GTM_gtm_cls2.MCS2_MEM[1846].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1847 GTM_gtm_cls2.MCS2_MEM[1847].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1848 GTM_gtm_cls2.MCS2_MEM[1848].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1849 GTM_gtm_cls2.MCS2_MEM[1849].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1850 GTM_gtm_cls2.MCS2_MEM[1850].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1851 GTM_gtm_cls2.MCS2_MEM[1851].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1852 GTM_gtm_cls2.MCS2_MEM[1852].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1853 GTM_gtm_cls2.MCS2_MEM[1853].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1854 GTM_gtm_cls2.MCS2_MEM[1854].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1855 GTM_gtm_cls2.MCS2_MEM[1855].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1856 GTM_gtm_cls2.MCS2_MEM[1856].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1857 GTM_gtm_cls2.MCS2_MEM[1857].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1858 GTM_gtm_cls2.MCS2_MEM[1858].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1859 GTM_gtm_cls2.MCS2_MEM[1859].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1860 GTM_gtm_cls2.MCS2_MEM[1860].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1861 GTM_gtm_cls2.MCS2_MEM[1861].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1862 GTM_gtm_cls2.MCS2_MEM[1862].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1863 GTM_gtm_cls2.MCS2_MEM[1863].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1864 GTM_gtm_cls2.MCS2_MEM[1864].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1865 GTM_gtm_cls2.MCS2_MEM[1865].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1866 GTM_gtm_cls2.MCS2_MEM[1866].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1867 GTM_gtm_cls2.MCS2_MEM[1867].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1868 GTM_gtm_cls2.MCS2_MEM[1868].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1869 GTM_gtm_cls2.MCS2_MEM[1869].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1870 GTM_gtm_cls2.MCS2_MEM[1870].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1871 GTM_gtm_cls2.MCS2_MEM[1871].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1872 GTM_gtm_cls2.MCS2_MEM[1872].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1873 GTM_gtm_cls2.MCS2_MEM[1873].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1874 GTM_gtm_cls2.MCS2_MEM[1874].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1875 GTM_gtm_cls2.MCS2_MEM[1875].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1876 GTM_gtm_cls2.MCS2_MEM[1876].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1877 GTM_gtm_cls2.MCS2_MEM[1877].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1878 GTM_gtm_cls2.MCS2_MEM[1878].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1879 GTM_gtm_cls2.MCS2_MEM[1879].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1880 GTM_gtm_cls2.MCS2_MEM[1880].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1881 GTM_gtm_cls2.MCS2_MEM[1881].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1882 GTM_gtm_cls2.MCS2_MEM[1882].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1883 GTM_gtm_cls2.MCS2_MEM[1883].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1884 GTM_gtm_cls2.MCS2_MEM[1884].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1885 GTM_gtm_cls2.MCS2_MEM[1885].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1886 GTM_gtm_cls2.MCS2_MEM[1886].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1887 GTM_gtm_cls2.MCS2_MEM[1887].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1888 GTM_gtm_cls2.MCS2_MEM[1888].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1889 GTM_gtm_cls2.MCS2_MEM[1889].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1890 GTM_gtm_cls2.MCS2_MEM[1890].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1891 GTM_gtm_cls2.MCS2_MEM[1891].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1892 GTM_gtm_cls2.MCS2_MEM[1892].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1893 GTM_gtm_cls2.MCS2_MEM[1893].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1894 GTM_gtm_cls2.MCS2_MEM[1894].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1895 GTM_gtm_cls2.MCS2_MEM[1895].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1896 GTM_gtm_cls2.MCS2_MEM[1896].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1897 GTM_gtm_cls2.MCS2_MEM[1897].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1898 GTM_gtm_cls2.MCS2_MEM[1898].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1899 GTM_gtm_cls2.MCS2_MEM[1899].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1900 GTM_gtm_cls2.MCS2_MEM[1900].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1901 GTM_gtm_cls2.MCS2_MEM[1901].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1902 GTM_gtm_cls2.MCS2_MEM[1902].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1903 GTM_gtm_cls2.MCS2_MEM[1903].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1904 GTM_gtm_cls2.MCS2_MEM[1904].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1905 GTM_gtm_cls2.MCS2_MEM[1905].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1906 GTM_gtm_cls2.MCS2_MEM[1906].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1907 GTM_gtm_cls2.MCS2_MEM[1907].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1908 GTM_gtm_cls2.MCS2_MEM[1908].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1909 GTM_gtm_cls2.MCS2_MEM[1909].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1910 GTM_gtm_cls2.MCS2_MEM[1910].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1911 GTM_gtm_cls2.MCS2_MEM[1911].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1912 GTM_gtm_cls2.MCS2_MEM[1912].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1913 GTM_gtm_cls2.MCS2_MEM[1913].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1914 GTM_gtm_cls2.MCS2_MEM[1914].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1915 GTM_gtm_cls2.MCS2_MEM[1915].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1916 GTM_gtm_cls2.MCS2_MEM[1916].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1917 GTM_gtm_cls2.MCS2_MEM[1917].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1918 GTM_gtm_cls2.MCS2_MEM[1918].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1919 GTM_gtm_cls2.MCS2_MEM[1919].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1920 GTM_gtm_cls2.MCS2_MEM[1920].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1921 GTM_gtm_cls2.MCS2_MEM[1921].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1922 GTM_gtm_cls2.MCS2_MEM[1922].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1923 GTM_gtm_cls2.MCS2_MEM[1923].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1924 GTM_gtm_cls2.MCS2_MEM[1924].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1925 GTM_gtm_cls2.MCS2_MEM[1925].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1926 GTM_gtm_cls2.MCS2_MEM[1926].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1927 GTM_gtm_cls2.MCS2_MEM[1927].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1928 GTM_gtm_cls2.MCS2_MEM[1928].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1929 GTM_gtm_cls2.MCS2_MEM[1929].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1930 GTM_gtm_cls2.MCS2_MEM[1930].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1931 GTM_gtm_cls2.MCS2_MEM[1931].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1932 GTM_gtm_cls2.MCS2_MEM[1932].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1933 GTM_gtm_cls2.MCS2_MEM[1933].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1934 GTM_gtm_cls2.MCS2_MEM[1934].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1935 GTM_gtm_cls2.MCS2_MEM[1935].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1936 GTM_gtm_cls2.MCS2_MEM[1936].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1937 GTM_gtm_cls2.MCS2_MEM[1937].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1938 GTM_gtm_cls2.MCS2_MEM[1938].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1939 GTM_gtm_cls2.MCS2_MEM[1939].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1940 GTM_gtm_cls2.MCS2_MEM[1940].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1941 GTM_gtm_cls2.MCS2_MEM[1941].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1942 GTM_gtm_cls2.MCS2_MEM[1942].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1943 GTM_gtm_cls2.MCS2_MEM[1943].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1944 GTM_gtm_cls2.MCS2_MEM[1944].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1945 GTM_gtm_cls2.MCS2_MEM[1945].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1946 GTM_gtm_cls2.MCS2_MEM[1946].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1947 GTM_gtm_cls2.MCS2_MEM[1947].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1948 GTM_gtm_cls2.MCS2_MEM[1948].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1949 GTM_gtm_cls2.MCS2_MEM[1949].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1950 GTM_gtm_cls2.MCS2_MEM[1950].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1951 GTM_gtm_cls2.MCS2_MEM[1951].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1952 GTM_gtm_cls2.MCS2_MEM[1952].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1953 GTM_gtm_cls2.MCS2_MEM[1953].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1954 GTM_gtm_cls2.MCS2_MEM[1954].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1955 GTM_gtm_cls2.MCS2_MEM[1955].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1956 GTM_gtm_cls2.MCS2_MEM[1956].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1957 GTM_gtm_cls2.MCS2_MEM[1957].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1958 GTM_gtm_cls2.MCS2_MEM[1958].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1959 GTM_gtm_cls2.MCS2_MEM[1959].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1960 GTM_gtm_cls2.MCS2_MEM[1960].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1961 GTM_gtm_cls2.MCS2_MEM[1961].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1962 GTM_gtm_cls2.MCS2_MEM[1962].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1963 GTM_gtm_cls2.MCS2_MEM[1963].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1964 GTM_gtm_cls2.MCS2_MEM[1964].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1965 GTM_gtm_cls2.MCS2_MEM[1965].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1966 GTM_gtm_cls2.MCS2_MEM[1966].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1967 GTM_gtm_cls2.MCS2_MEM[1967].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1968 GTM_gtm_cls2.MCS2_MEM[1968].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1969 GTM_gtm_cls2.MCS2_MEM[1969].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1970 GTM_gtm_cls2.MCS2_MEM[1970].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1971 GTM_gtm_cls2.MCS2_MEM[1971].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1972 GTM_gtm_cls2.MCS2_MEM[1972].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1973 GTM_gtm_cls2.MCS2_MEM[1973].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1974 GTM_gtm_cls2.MCS2_MEM[1974].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1975 GTM_gtm_cls2.MCS2_MEM[1975].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1976 GTM_gtm_cls2.MCS2_MEM[1976].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1977 GTM_gtm_cls2.MCS2_MEM[1977].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1978 GTM_gtm_cls2.MCS2_MEM[1978].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1979 GTM_gtm_cls2.MCS2_MEM[1979].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1980 GTM_gtm_cls2.MCS2_MEM[1980].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1981 GTM_gtm_cls2.MCS2_MEM[1981].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1982 GTM_gtm_cls2.MCS2_MEM[1982].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1983 GTM_gtm_cls2.MCS2_MEM[1983].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1984 GTM_gtm_cls2.MCS2_MEM[1984].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1985 GTM_gtm_cls2.MCS2_MEM[1985].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1986 GTM_gtm_cls2.MCS2_MEM[1986].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1987 GTM_gtm_cls2.MCS2_MEM[1987].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1988 GTM_gtm_cls2.MCS2_MEM[1988].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1989 GTM_gtm_cls2.MCS2_MEM[1989].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1990 GTM_gtm_cls2.MCS2_MEM[1990].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1991 GTM_gtm_cls2.MCS2_MEM[1991].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1992 GTM_gtm_cls2.MCS2_MEM[1992].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1993 GTM_gtm_cls2.MCS2_MEM[1993].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1994 GTM_gtm_cls2.MCS2_MEM[1994].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1995 GTM_gtm_cls2.MCS2_MEM[1995].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1996 GTM_gtm_cls2.MCS2_MEM[1996].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1997 GTM_gtm_cls2.MCS2_MEM[1997].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1998 GTM_gtm_cls2.MCS2_MEM[1998].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM1999 GTM_gtm_cls2.MCS2_MEM[1999].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2000 GTM_gtm_cls2.MCS2_MEM[2000].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2001 GTM_gtm_cls2.MCS2_MEM[2001].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2002 GTM_gtm_cls2.MCS2_MEM[2002].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2003 GTM_gtm_cls2.MCS2_MEM[2003].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2004 GTM_gtm_cls2.MCS2_MEM[2004].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2005 GTM_gtm_cls2.MCS2_MEM[2005].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2006 GTM_gtm_cls2.MCS2_MEM[2006].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2007 GTM_gtm_cls2.MCS2_MEM[2007].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2008 GTM_gtm_cls2.MCS2_MEM[2008].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2009 GTM_gtm_cls2.MCS2_MEM[2009].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2010 GTM_gtm_cls2.MCS2_MEM[2010].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2011 GTM_gtm_cls2.MCS2_MEM[2011].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2012 GTM_gtm_cls2.MCS2_MEM[2012].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2013 GTM_gtm_cls2.MCS2_MEM[2013].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2014 GTM_gtm_cls2.MCS2_MEM[2014].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2015 GTM_gtm_cls2.MCS2_MEM[2015].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2016 GTM_gtm_cls2.MCS2_MEM[2016].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2017 GTM_gtm_cls2.MCS2_MEM[2017].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2018 GTM_gtm_cls2.MCS2_MEM[2018].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2019 GTM_gtm_cls2.MCS2_MEM[2019].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2020 GTM_gtm_cls2.MCS2_MEM[2020].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2021 GTM_gtm_cls2.MCS2_MEM[2021].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2022 GTM_gtm_cls2.MCS2_MEM[2022].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2023 GTM_gtm_cls2.MCS2_MEM[2023].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2024 GTM_gtm_cls2.MCS2_MEM[2024].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2025 GTM_gtm_cls2.MCS2_MEM[2025].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2026 GTM_gtm_cls2.MCS2_MEM[2026].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2027 GTM_gtm_cls2.MCS2_MEM[2027].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2028 GTM_gtm_cls2.MCS2_MEM[2028].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2029 GTM_gtm_cls2.MCS2_MEM[2029].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2030 GTM_gtm_cls2.MCS2_MEM[2030].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2031 GTM_gtm_cls2.MCS2_MEM[2031].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2032 GTM_gtm_cls2.MCS2_MEM[2032].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2033 GTM_gtm_cls2.MCS2_MEM[2033].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2034 GTM_gtm_cls2.MCS2_MEM[2034].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2035 GTM_gtm_cls2.MCS2_MEM[2035].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2036 GTM_gtm_cls2.MCS2_MEM[2036].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2037 GTM_gtm_cls2.MCS2_MEM[2037].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2038 GTM_gtm_cls2.MCS2_MEM[2038].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2039 GTM_gtm_cls2.MCS2_MEM[2039].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2040 GTM_gtm_cls2.MCS2_MEM[2040].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2041 GTM_gtm_cls2.MCS2_MEM[2041].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2042 GTM_gtm_cls2.MCS2_MEM[2042].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2043 GTM_gtm_cls2.MCS2_MEM[2043].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2044 GTM_gtm_cls2.MCS2_MEM[2044].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2045 GTM_gtm_cls2.MCS2_MEM[2045].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2046 GTM_gtm_cls2.MCS2_MEM[2046].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2047 GTM_gtm_cls2.MCS2_MEM[2047].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2048 GTM_gtm_cls2.MCS2_MEM[2048].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2049 GTM_gtm_cls2.MCS2_MEM[2049].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2050 GTM_gtm_cls2.MCS2_MEM[2050].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2051 GTM_gtm_cls2.MCS2_MEM[2051].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2052 GTM_gtm_cls2.MCS2_MEM[2052].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2053 GTM_gtm_cls2.MCS2_MEM[2053].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2054 GTM_gtm_cls2.MCS2_MEM[2054].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2055 GTM_gtm_cls2.MCS2_MEM[2055].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2056 GTM_gtm_cls2.MCS2_MEM[2056].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2057 GTM_gtm_cls2.MCS2_MEM[2057].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2058 GTM_gtm_cls2.MCS2_MEM[2058].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2059 GTM_gtm_cls2.MCS2_MEM[2059].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2060 GTM_gtm_cls2.MCS2_MEM[2060].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2061 GTM_gtm_cls2.MCS2_MEM[2061].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2062 GTM_gtm_cls2.MCS2_MEM[2062].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2063 GTM_gtm_cls2.MCS2_MEM[2063].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2064 GTM_gtm_cls2.MCS2_MEM[2064].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2065 GTM_gtm_cls2.MCS2_MEM[2065].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2066 GTM_gtm_cls2.MCS2_MEM[2066].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2067 GTM_gtm_cls2.MCS2_MEM[2067].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2068 GTM_gtm_cls2.MCS2_MEM[2068].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2069 GTM_gtm_cls2.MCS2_MEM[2069].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2070 GTM_gtm_cls2.MCS2_MEM[2070].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2071 GTM_gtm_cls2.MCS2_MEM[2071].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2072 GTM_gtm_cls2.MCS2_MEM[2072].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2073 GTM_gtm_cls2.MCS2_MEM[2073].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2074 GTM_gtm_cls2.MCS2_MEM[2074].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2075 GTM_gtm_cls2.MCS2_MEM[2075].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2076 GTM_gtm_cls2.MCS2_MEM[2076].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2077 GTM_gtm_cls2.MCS2_MEM[2077].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2078 GTM_gtm_cls2.MCS2_MEM[2078].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2079 GTM_gtm_cls2.MCS2_MEM[2079].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2080 GTM_gtm_cls2.MCS2_MEM[2080].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2081 GTM_gtm_cls2.MCS2_MEM[2081].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2082 GTM_gtm_cls2.MCS2_MEM[2082].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2083 GTM_gtm_cls2.MCS2_MEM[2083].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2084 GTM_gtm_cls2.MCS2_MEM[2084].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2085 GTM_gtm_cls2.MCS2_MEM[2085].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2086 GTM_gtm_cls2.MCS2_MEM[2086].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2087 GTM_gtm_cls2.MCS2_MEM[2087].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2088 GTM_gtm_cls2.MCS2_MEM[2088].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2089 GTM_gtm_cls2.MCS2_MEM[2089].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2090 GTM_gtm_cls2.MCS2_MEM[2090].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2091 GTM_gtm_cls2.MCS2_MEM[2091].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2092 GTM_gtm_cls2.MCS2_MEM[2092].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2093 GTM_gtm_cls2.MCS2_MEM[2093].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2094 GTM_gtm_cls2.MCS2_MEM[2094].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2095 GTM_gtm_cls2.MCS2_MEM[2095].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2096 GTM_gtm_cls2.MCS2_MEM[2096].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2097 GTM_gtm_cls2.MCS2_MEM[2097].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2098 GTM_gtm_cls2.MCS2_MEM[2098].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2099 GTM_gtm_cls2.MCS2_MEM[2099].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2100 GTM_gtm_cls2.MCS2_MEM[2100].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2101 GTM_gtm_cls2.MCS2_MEM[2101].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2102 GTM_gtm_cls2.MCS2_MEM[2102].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2103 GTM_gtm_cls2.MCS2_MEM[2103].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2104 GTM_gtm_cls2.MCS2_MEM[2104].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2105 GTM_gtm_cls2.MCS2_MEM[2105].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2106 GTM_gtm_cls2.MCS2_MEM[2106].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2107 GTM_gtm_cls2.MCS2_MEM[2107].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2108 GTM_gtm_cls2.MCS2_MEM[2108].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2109 GTM_gtm_cls2.MCS2_MEM[2109].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2110 GTM_gtm_cls2.MCS2_MEM[2110].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2111 GTM_gtm_cls2.MCS2_MEM[2111].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2112 GTM_gtm_cls2.MCS2_MEM[2112].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2113 GTM_gtm_cls2.MCS2_MEM[2113].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2114 GTM_gtm_cls2.MCS2_MEM[2114].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2115 GTM_gtm_cls2.MCS2_MEM[2115].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2116 GTM_gtm_cls2.MCS2_MEM[2116].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2117 GTM_gtm_cls2.MCS2_MEM[2117].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2118 GTM_gtm_cls2.MCS2_MEM[2118].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2119 GTM_gtm_cls2.MCS2_MEM[2119].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2120 GTM_gtm_cls2.MCS2_MEM[2120].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2121 GTM_gtm_cls2.MCS2_MEM[2121].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2122 GTM_gtm_cls2.MCS2_MEM[2122].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2123 GTM_gtm_cls2.MCS2_MEM[2123].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2124 GTM_gtm_cls2.MCS2_MEM[2124].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2125 GTM_gtm_cls2.MCS2_MEM[2125].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2126 GTM_gtm_cls2.MCS2_MEM[2126].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2127 GTM_gtm_cls2.MCS2_MEM[2127].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2128 GTM_gtm_cls2.MCS2_MEM[2128].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2129 GTM_gtm_cls2.MCS2_MEM[2129].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2130 GTM_gtm_cls2.MCS2_MEM[2130].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2131 GTM_gtm_cls2.MCS2_MEM[2131].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2132 GTM_gtm_cls2.MCS2_MEM[2132].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2133 GTM_gtm_cls2.MCS2_MEM[2133].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2134 GTM_gtm_cls2.MCS2_MEM[2134].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2135 GTM_gtm_cls2.MCS2_MEM[2135].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2136 GTM_gtm_cls2.MCS2_MEM[2136].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2137 GTM_gtm_cls2.MCS2_MEM[2137].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2138 GTM_gtm_cls2.MCS2_MEM[2138].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2139 GTM_gtm_cls2.MCS2_MEM[2139].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2140 GTM_gtm_cls2.MCS2_MEM[2140].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2141 GTM_gtm_cls2.MCS2_MEM[2141].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2142 GTM_gtm_cls2.MCS2_MEM[2142].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2143 GTM_gtm_cls2.MCS2_MEM[2143].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2144 GTM_gtm_cls2.MCS2_MEM[2144].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2145 GTM_gtm_cls2.MCS2_MEM[2145].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2146 GTM_gtm_cls2.MCS2_MEM[2146].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2147 GTM_gtm_cls2.MCS2_MEM[2147].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2148 GTM_gtm_cls2.MCS2_MEM[2148].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2149 GTM_gtm_cls2.MCS2_MEM[2149].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2150 GTM_gtm_cls2.MCS2_MEM[2150].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2151 GTM_gtm_cls2.MCS2_MEM[2151].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2152 GTM_gtm_cls2.MCS2_MEM[2152].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2153 GTM_gtm_cls2.MCS2_MEM[2153].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2154 GTM_gtm_cls2.MCS2_MEM[2154].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2155 GTM_gtm_cls2.MCS2_MEM[2155].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2156 GTM_gtm_cls2.MCS2_MEM[2156].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2157 GTM_gtm_cls2.MCS2_MEM[2157].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2158 GTM_gtm_cls2.MCS2_MEM[2158].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2159 GTM_gtm_cls2.MCS2_MEM[2159].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2160 GTM_gtm_cls2.MCS2_MEM[2160].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2161 GTM_gtm_cls2.MCS2_MEM[2161].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2162 GTM_gtm_cls2.MCS2_MEM[2162].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2163 GTM_gtm_cls2.MCS2_MEM[2163].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2164 GTM_gtm_cls2.MCS2_MEM[2164].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2165 GTM_gtm_cls2.MCS2_MEM[2165].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2166 GTM_gtm_cls2.MCS2_MEM[2166].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2167 GTM_gtm_cls2.MCS2_MEM[2167].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2168 GTM_gtm_cls2.MCS2_MEM[2168].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2169 GTM_gtm_cls2.MCS2_MEM[2169].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2170 GTM_gtm_cls2.MCS2_MEM[2170].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2171 GTM_gtm_cls2.MCS2_MEM[2171].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2172 GTM_gtm_cls2.MCS2_MEM[2172].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2173 GTM_gtm_cls2.MCS2_MEM[2173].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2174 GTM_gtm_cls2.MCS2_MEM[2174].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2175 GTM_gtm_cls2.MCS2_MEM[2175].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2176 GTM_gtm_cls2.MCS2_MEM[2176].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2177 GTM_gtm_cls2.MCS2_MEM[2177].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2178 GTM_gtm_cls2.MCS2_MEM[2178].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2179 GTM_gtm_cls2.MCS2_MEM[2179].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2180 GTM_gtm_cls2.MCS2_MEM[2180].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2181 GTM_gtm_cls2.MCS2_MEM[2181].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2182 GTM_gtm_cls2.MCS2_MEM[2182].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2183 GTM_gtm_cls2.MCS2_MEM[2183].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2184 GTM_gtm_cls2.MCS2_MEM[2184].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2185 GTM_gtm_cls2.MCS2_MEM[2185].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2186 GTM_gtm_cls2.MCS2_MEM[2186].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2187 GTM_gtm_cls2.MCS2_MEM[2187].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2188 GTM_gtm_cls2.MCS2_MEM[2188].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2189 GTM_gtm_cls2.MCS2_MEM[2189].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2190 GTM_gtm_cls2.MCS2_MEM[2190].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2191 GTM_gtm_cls2.MCS2_MEM[2191].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2192 GTM_gtm_cls2.MCS2_MEM[2192].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2193 GTM_gtm_cls2.MCS2_MEM[2193].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2194 GTM_gtm_cls2.MCS2_MEM[2194].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2195 GTM_gtm_cls2.MCS2_MEM[2195].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2196 GTM_gtm_cls2.MCS2_MEM[2196].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2197 GTM_gtm_cls2.MCS2_MEM[2197].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2198 GTM_gtm_cls2.MCS2_MEM[2198].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2199 GTM_gtm_cls2.MCS2_MEM[2199].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2200 GTM_gtm_cls2.MCS2_MEM[2200].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2201 GTM_gtm_cls2.MCS2_MEM[2201].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2202 GTM_gtm_cls2.MCS2_MEM[2202].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2203 GTM_gtm_cls2.MCS2_MEM[2203].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2204 GTM_gtm_cls2.MCS2_MEM[2204].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2205 GTM_gtm_cls2.MCS2_MEM[2205].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2206 GTM_gtm_cls2.MCS2_MEM[2206].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2207 GTM_gtm_cls2.MCS2_MEM[2207].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2208 GTM_gtm_cls2.MCS2_MEM[2208].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2209 GTM_gtm_cls2.MCS2_MEM[2209].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2210 GTM_gtm_cls2.MCS2_MEM[2210].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2211 GTM_gtm_cls2.MCS2_MEM[2211].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2212 GTM_gtm_cls2.MCS2_MEM[2212].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2213 GTM_gtm_cls2.MCS2_MEM[2213].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2214 GTM_gtm_cls2.MCS2_MEM[2214].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2215 GTM_gtm_cls2.MCS2_MEM[2215].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2216 GTM_gtm_cls2.MCS2_MEM[2216].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2217 GTM_gtm_cls2.MCS2_MEM[2217].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2218 GTM_gtm_cls2.MCS2_MEM[2218].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2219 GTM_gtm_cls2.MCS2_MEM[2219].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2220 GTM_gtm_cls2.MCS2_MEM[2220].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2221 GTM_gtm_cls2.MCS2_MEM[2221].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2222 GTM_gtm_cls2.MCS2_MEM[2222].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2223 GTM_gtm_cls2.MCS2_MEM[2223].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2224 GTM_gtm_cls2.MCS2_MEM[2224].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2225 GTM_gtm_cls2.MCS2_MEM[2225].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2226 GTM_gtm_cls2.MCS2_MEM[2226].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2227 GTM_gtm_cls2.MCS2_MEM[2227].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2228 GTM_gtm_cls2.MCS2_MEM[2228].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2229 GTM_gtm_cls2.MCS2_MEM[2229].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2230 GTM_gtm_cls2.MCS2_MEM[2230].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2231 GTM_gtm_cls2.MCS2_MEM[2231].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2232 GTM_gtm_cls2.MCS2_MEM[2232].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2233 GTM_gtm_cls2.MCS2_MEM[2233].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2234 GTM_gtm_cls2.MCS2_MEM[2234].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2235 GTM_gtm_cls2.MCS2_MEM[2235].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2236 GTM_gtm_cls2.MCS2_MEM[2236].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2237 GTM_gtm_cls2.MCS2_MEM[2237].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2238 GTM_gtm_cls2.MCS2_MEM[2238].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2239 GTM_gtm_cls2.MCS2_MEM[2239].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2240 GTM_gtm_cls2.MCS2_MEM[2240].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2241 GTM_gtm_cls2.MCS2_MEM[2241].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2242 GTM_gtm_cls2.MCS2_MEM[2242].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2243 GTM_gtm_cls2.MCS2_MEM[2243].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2244 GTM_gtm_cls2.MCS2_MEM[2244].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2245 GTM_gtm_cls2.MCS2_MEM[2245].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2246 GTM_gtm_cls2.MCS2_MEM[2246].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2247 GTM_gtm_cls2.MCS2_MEM[2247].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2248 GTM_gtm_cls2.MCS2_MEM[2248].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2249 GTM_gtm_cls2.MCS2_MEM[2249].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2250 GTM_gtm_cls2.MCS2_MEM[2250].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2251 GTM_gtm_cls2.MCS2_MEM[2251].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2252 GTM_gtm_cls2.MCS2_MEM[2252].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2253 GTM_gtm_cls2.MCS2_MEM[2253].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2254 GTM_gtm_cls2.MCS2_MEM[2254].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2255 GTM_gtm_cls2.MCS2_MEM[2255].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2256 GTM_gtm_cls2.MCS2_MEM[2256].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2257 GTM_gtm_cls2.MCS2_MEM[2257].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2258 GTM_gtm_cls2.MCS2_MEM[2258].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2259 GTM_gtm_cls2.MCS2_MEM[2259].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2260 GTM_gtm_cls2.MCS2_MEM[2260].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2261 GTM_gtm_cls2.MCS2_MEM[2261].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2262 GTM_gtm_cls2.MCS2_MEM[2262].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2263 GTM_gtm_cls2.MCS2_MEM[2263].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2264 GTM_gtm_cls2.MCS2_MEM[2264].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2265 GTM_gtm_cls2.MCS2_MEM[2265].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2266 GTM_gtm_cls2.MCS2_MEM[2266].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2267 GTM_gtm_cls2.MCS2_MEM[2267].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2268 GTM_gtm_cls2.MCS2_MEM[2268].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2269 GTM_gtm_cls2.MCS2_MEM[2269].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2270 GTM_gtm_cls2.MCS2_MEM[2270].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2271 GTM_gtm_cls2.MCS2_MEM[2271].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2272 GTM_gtm_cls2.MCS2_MEM[2272].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2273 GTM_gtm_cls2.MCS2_MEM[2273].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2274 GTM_gtm_cls2.MCS2_MEM[2274].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2275 GTM_gtm_cls2.MCS2_MEM[2275].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2276 GTM_gtm_cls2.MCS2_MEM[2276].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2277 GTM_gtm_cls2.MCS2_MEM[2277].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2278 GTM_gtm_cls2.MCS2_MEM[2278].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2279 GTM_gtm_cls2.MCS2_MEM[2279].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2280 GTM_gtm_cls2.MCS2_MEM[2280].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2281 GTM_gtm_cls2.MCS2_MEM[2281].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2282 GTM_gtm_cls2.MCS2_MEM[2282].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2283 GTM_gtm_cls2.MCS2_MEM[2283].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2284 GTM_gtm_cls2.MCS2_MEM[2284].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2285 GTM_gtm_cls2.MCS2_MEM[2285].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2286 GTM_gtm_cls2.MCS2_MEM[2286].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2287 GTM_gtm_cls2.MCS2_MEM[2287].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2288 GTM_gtm_cls2.MCS2_MEM[2288].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2289 GTM_gtm_cls2.MCS2_MEM[2289].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2290 GTM_gtm_cls2.MCS2_MEM[2290].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2291 GTM_gtm_cls2.MCS2_MEM[2291].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2292 GTM_gtm_cls2.MCS2_MEM[2292].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2293 GTM_gtm_cls2.MCS2_MEM[2293].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2294 GTM_gtm_cls2.MCS2_MEM[2294].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2295 GTM_gtm_cls2.MCS2_MEM[2295].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2296 GTM_gtm_cls2.MCS2_MEM[2296].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2297 GTM_gtm_cls2.MCS2_MEM[2297].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2298 GTM_gtm_cls2.MCS2_MEM[2298].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2299 GTM_gtm_cls2.MCS2_MEM[2299].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2300 GTM_gtm_cls2.MCS2_MEM[2300].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2301 GTM_gtm_cls2.MCS2_MEM[2301].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2302 GTM_gtm_cls2.MCS2_MEM[2302].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2303 GTM_gtm_cls2.MCS2_MEM[2303].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2304 GTM_gtm_cls2.MCS2_MEM[2304].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2305 GTM_gtm_cls2.MCS2_MEM[2305].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2306 GTM_gtm_cls2.MCS2_MEM[2306].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2307 GTM_gtm_cls2.MCS2_MEM[2307].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2308 GTM_gtm_cls2.MCS2_MEM[2308].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2309 GTM_gtm_cls2.MCS2_MEM[2309].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2310 GTM_gtm_cls2.MCS2_MEM[2310].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2311 GTM_gtm_cls2.MCS2_MEM[2311].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2312 GTM_gtm_cls2.MCS2_MEM[2312].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2313 GTM_gtm_cls2.MCS2_MEM[2313].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2314 GTM_gtm_cls2.MCS2_MEM[2314].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2315 GTM_gtm_cls2.MCS2_MEM[2315].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2316 GTM_gtm_cls2.MCS2_MEM[2316].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2317 GTM_gtm_cls2.MCS2_MEM[2317].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2318 GTM_gtm_cls2.MCS2_MEM[2318].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2319 GTM_gtm_cls2.MCS2_MEM[2319].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2320 GTM_gtm_cls2.MCS2_MEM[2320].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2321 GTM_gtm_cls2.MCS2_MEM[2321].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2322 GTM_gtm_cls2.MCS2_MEM[2322].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2323 GTM_gtm_cls2.MCS2_MEM[2323].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2324 GTM_gtm_cls2.MCS2_MEM[2324].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2325 GTM_gtm_cls2.MCS2_MEM[2325].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2326 GTM_gtm_cls2.MCS2_MEM[2326].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2327 GTM_gtm_cls2.MCS2_MEM[2327].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2328 GTM_gtm_cls2.MCS2_MEM[2328].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2329 GTM_gtm_cls2.MCS2_MEM[2329].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2330 GTM_gtm_cls2.MCS2_MEM[2330].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2331 GTM_gtm_cls2.MCS2_MEM[2331].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2332 GTM_gtm_cls2.MCS2_MEM[2332].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2333 GTM_gtm_cls2.MCS2_MEM[2333].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2334 GTM_gtm_cls2.MCS2_MEM[2334].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2335 GTM_gtm_cls2.MCS2_MEM[2335].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2336 GTM_gtm_cls2.MCS2_MEM[2336].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2337 GTM_gtm_cls2.MCS2_MEM[2337].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2338 GTM_gtm_cls2.MCS2_MEM[2338].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2339 GTM_gtm_cls2.MCS2_MEM[2339].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2340 GTM_gtm_cls2.MCS2_MEM[2340].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2341 GTM_gtm_cls2.MCS2_MEM[2341].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2342 GTM_gtm_cls2.MCS2_MEM[2342].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2343 GTM_gtm_cls2.MCS2_MEM[2343].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2344 GTM_gtm_cls2.MCS2_MEM[2344].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2345 GTM_gtm_cls2.MCS2_MEM[2345].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2346 GTM_gtm_cls2.MCS2_MEM[2346].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2347 GTM_gtm_cls2.MCS2_MEM[2347].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2348 GTM_gtm_cls2.MCS2_MEM[2348].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2349 GTM_gtm_cls2.MCS2_MEM[2349].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2350 GTM_gtm_cls2.MCS2_MEM[2350].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2351 GTM_gtm_cls2.MCS2_MEM[2351].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2352 GTM_gtm_cls2.MCS2_MEM[2352].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2353 GTM_gtm_cls2.MCS2_MEM[2353].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2354 GTM_gtm_cls2.MCS2_MEM[2354].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2355 GTM_gtm_cls2.MCS2_MEM[2355].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2356 GTM_gtm_cls2.MCS2_MEM[2356].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2357 GTM_gtm_cls2.MCS2_MEM[2357].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2358 GTM_gtm_cls2.MCS2_MEM[2358].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2359 GTM_gtm_cls2.MCS2_MEM[2359].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2360 GTM_gtm_cls2.MCS2_MEM[2360].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2361 GTM_gtm_cls2.MCS2_MEM[2361].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2362 GTM_gtm_cls2.MCS2_MEM[2362].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2363 GTM_gtm_cls2.MCS2_MEM[2363].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2364 GTM_gtm_cls2.MCS2_MEM[2364].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2365 GTM_gtm_cls2.MCS2_MEM[2365].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2366 GTM_gtm_cls2.MCS2_MEM[2366].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2367 GTM_gtm_cls2.MCS2_MEM[2367].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2368 GTM_gtm_cls2.MCS2_MEM[2368].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2369 GTM_gtm_cls2.MCS2_MEM[2369].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2370 GTM_gtm_cls2.MCS2_MEM[2370].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2371 GTM_gtm_cls2.MCS2_MEM[2371].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2372 GTM_gtm_cls2.MCS2_MEM[2372].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2373 GTM_gtm_cls2.MCS2_MEM[2373].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2374 GTM_gtm_cls2.MCS2_MEM[2374].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2375 GTM_gtm_cls2.MCS2_MEM[2375].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2376 GTM_gtm_cls2.MCS2_MEM[2376].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2377 GTM_gtm_cls2.MCS2_MEM[2377].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2378 GTM_gtm_cls2.MCS2_MEM[2378].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2379 GTM_gtm_cls2.MCS2_MEM[2379].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2380 GTM_gtm_cls2.MCS2_MEM[2380].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2381 GTM_gtm_cls2.MCS2_MEM[2381].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2382 GTM_gtm_cls2.MCS2_MEM[2382].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2383 GTM_gtm_cls2.MCS2_MEM[2383].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2384 GTM_gtm_cls2.MCS2_MEM[2384].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2385 GTM_gtm_cls2.MCS2_MEM[2385].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2386 GTM_gtm_cls2.MCS2_MEM[2386].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2387 GTM_gtm_cls2.MCS2_MEM[2387].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2388 GTM_gtm_cls2.MCS2_MEM[2388].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2389 GTM_gtm_cls2.MCS2_MEM[2389].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2390 GTM_gtm_cls2.MCS2_MEM[2390].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2391 GTM_gtm_cls2.MCS2_MEM[2391].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2392 GTM_gtm_cls2.MCS2_MEM[2392].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2393 GTM_gtm_cls2.MCS2_MEM[2393].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2394 GTM_gtm_cls2.MCS2_MEM[2394].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2395 GTM_gtm_cls2.MCS2_MEM[2395].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2396 GTM_gtm_cls2.MCS2_MEM[2396].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2397 GTM_gtm_cls2.MCS2_MEM[2397].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2398 GTM_gtm_cls2.MCS2_MEM[2398].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2399 GTM_gtm_cls2.MCS2_MEM[2399].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2400 GTM_gtm_cls2.MCS2_MEM[2400].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2401 GTM_gtm_cls2.MCS2_MEM[2401].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2402 GTM_gtm_cls2.MCS2_MEM[2402].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2403 GTM_gtm_cls2.MCS2_MEM[2403].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2404 GTM_gtm_cls2.MCS2_MEM[2404].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2405 GTM_gtm_cls2.MCS2_MEM[2405].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2406 GTM_gtm_cls2.MCS2_MEM[2406].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2407 GTM_gtm_cls2.MCS2_MEM[2407].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2408 GTM_gtm_cls2.MCS2_MEM[2408].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2409 GTM_gtm_cls2.MCS2_MEM[2409].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2410 GTM_gtm_cls2.MCS2_MEM[2410].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2411 GTM_gtm_cls2.MCS2_MEM[2411].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2412 GTM_gtm_cls2.MCS2_MEM[2412].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2413 GTM_gtm_cls2.MCS2_MEM[2413].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2414 GTM_gtm_cls2.MCS2_MEM[2414].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2415 GTM_gtm_cls2.MCS2_MEM[2415].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2416 GTM_gtm_cls2.MCS2_MEM[2416].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2417 GTM_gtm_cls2.MCS2_MEM[2417].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2418 GTM_gtm_cls2.MCS2_MEM[2418].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2419 GTM_gtm_cls2.MCS2_MEM[2419].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2420 GTM_gtm_cls2.MCS2_MEM[2420].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2421 GTM_gtm_cls2.MCS2_MEM[2421].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2422 GTM_gtm_cls2.MCS2_MEM[2422].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2423 GTM_gtm_cls2.MCS2_MEM[2423].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2424 GTM_gtm_cls2.MCS2_MEM[2424].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2425 GTM_gtm_cls2.MCS2_MEM[2425].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2426 GTM_gtm_cls2.MCS2_MEM[2426].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2427 GTM_gtm_cls2.MCS2_MEM[2427].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2428 GTM_gtm_cls2.MCS2_MEM[2428].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2429 GTM_gtm_cls2.MCS2_MEM[2429].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2430 GTM_gtm_cls2.MCS2_MEM[2430].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2431 GTM_gtm_cls2.MCS2_MEM[2431].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2432 GTM_gtm_cls2.MCS2_MEM[2432].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2433 GTM_gtm_cls2.MCS2_MEM[2433].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2434 GTM_gtm_cls2.MCS2_MEM[2434].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2435 GTM_gtm_cls2.MCS2_MEM[2435].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2436 GTM_gtm_cls2.MCS2_MEM[2436].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2437 GTM_gtm_cls2.MCS2_MEM[2437].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2438 GTM_gtm_cls2.MCS2_MEM[2438].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2439 GTM_gtm_cls2.MCS2_MEM[2439].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2440 GTM_gtm_cls2.MCS2_MEM[2440].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2441 GTM_gtm_cls2.MCS2_MEM[2441].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2442 GTM_gtm_cls2.MCS2_MEM[2442].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2443 GTM_gtm_cls2.MCS2_MEM[2443].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2444 GTM_gtm_cls2.MCS2_MEM[2444].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2445 GTM_gtm_cls2.MCS2_MEM[2445].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2446 GTM_gtm_cls2.MCS2_MEM[2446].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2447 GTM_gtm_cls2.MCS2_MEM[2447].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2448 GTM_gtm_cls2.MCS2_MEM[2448].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2449 GTM_gtm_cls2.MCS2_MEM[2449].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2450 GTM_gtm_cls2.MCS2_MEM[2450].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2451 GTM_gtm_cls2.MCS2_MEM[2451].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2452 GTM_gtm_cls2.MCS2_MEM[2452].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2453 GTM_gtm_cls2.MCS2_MEM[2453].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2454 GTM_gtm_cls2.MCS2_MEM[2454].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2455 GTM_gtm_cls2.MCS2_MEM[2455].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2456 GTM_gtm_cls2.MCS2_MEM[2456].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2457 GTM_gtm_cls2.MCS2_MEM[2457].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2458 GTM_gtm_cls2.MCS2_MEM[2458].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2459 GTM_gtm_cls2.MCS2_MEM[2459].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2460 GTM_gtm_cls2.MCS2_MEM[2460].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2461 GTM_gtm_cls2.MCS2_MEM[2461].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2462 GTM_gtm_cls2.MCS2_MEM[2462].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2463 GTM_gtm_cls2.MCS2_MEM[2463].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2464 GTM_gtm_cls2.MCS2_MEM[2464].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2465 GTM_gtm_cls2.MCS2_MEM[2465].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2466 GTM_gtm_cls2.MCS2_MEM[2466].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2467 GTM_gtm_cls2.MCS2_MEM[2467].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2468 GTM_gtm_cls2.MCS2_MEM[2468].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2469 GTM_gtm_cls2.MCS2_MEM[2469].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2470 GTM_gtm_cls2.MCS2_MEM[2470].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2471 GTM_gtm_cls2.MCS2_MEM[2471].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2472 GTM_gtm_cls2.MCS2_MEM[2472].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2473 GTM_gtm_cls2.MCS2_MEM[2473].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2474 GTM_gtm_cls2.MCS2_MEM[2474].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2475 GTM_gtm_cls2.MCS2_MEM[2475].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2476 GTM_gtm_cls2.MCS2_MEM[2476].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2477 GTM_gtm_cls2.MCS2_MEM[2477].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2478 GTM_gtm_cls2.MCS2_MEM[2478].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2479 GTM_gtm_cls2.MCS2_MEM[2479].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2480 GTM_gtm_cls2.MCS2_MEM[2480].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2481 GTM_gtm_cls2.MCS2_MEM[2481].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2482 GTM_gtm_cls2.MCS2_MEM[2482].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2483 GTM_gtm_cls2.MCS2_MEM[2483].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2484 GTM_gtm_cls2.MCS2_MEM[2484].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2485 GTM_gtm_cls2.MCS2_MEM[2485].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2486 GTM_gtm_cls2.MCS2_MEM[2486].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2487 GTM_gtm_cls2.MCS2_MEM[2487].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2488 GTM_gtm_cls2.MCS2_MEM[2488].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2489 GTM_gtm_cls2.MCS2_MEM[2489].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2490 GTM_gtm_cls2.MCS2_MEM[2490].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2491 GTM_gtm_cls2.MCS2_MEM[2491].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2492 GTM_gtm_cls2.MCS2_MEM[2492].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2493 GTM_gtm_cls2.MCS2_MEM[2493].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2494 GTM_gtm_cls2.MCS2_MEM[2494].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2495 GTM_gtm_cls2.MCS2_MEM[2495].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2496 GTM_gtm_cls2.MCS2_MEM[2496].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2497 GTM_gtm_cls2.MCS2_MEM[2497].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2498 GTM_gtm_cls2.MCS2_MEM[2498].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2499 GTM_gtm_cls2.MCS2_MEM[2499].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2500 GTM_gtm_cls2.MCS2_MEM[2500].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2501 GTM_gtm_cls2.MCS2_MEM[2501].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2502 GTM_gtm_cls2.MCS2_MEM[2502].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2503 GTM_gtm_cls2.MCS2_MEM[2503].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2504 GTM_gtm_cls2.MCS2_MEM[2504].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2505 GTM_gtm_cls2.MCS2_MEM[2505].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2506 GTM_gtm_cls2.MCS2_MEM[2506].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2507 GTM_gtm_cls2.MCS2_MEM[2507].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2508 GTM_gtm_cls2.MCS2_MEM[2508].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2509 GTM_gtm_cls2.MCS2_MEM[2509].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2510 GTM_gtm_cls2.MCS2_MEM[2510].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2511 GTM_gtm_cls2.MCS2_MEM[2511].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2512 GTM_gtm_cls2.MCS2_MEM[2512].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2513 GTM_gtm_cls2.MCS2_MEM[2513].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2514 GTM_gtm_cls2.MCS2_MEM[2514].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2515 GTM_gtm_cls2.MCS2_MEM[2515].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2516 GTM_gtm_cls2.MCS2_MEM[2516].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2517 GTM_gtm_cls2.MCS2_MEM[2517].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2518 GTM_gtm_cls2.MCS2_MEM[2518].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2519 GTM_gtm_cls2.MCS2_MEM[2519].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2520 GTM_gtm_cls2.MCS2_MEM[2520].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2521 GTM_gtm_cls2.MCS2_MEM[2521].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2522 GTM_gtm_cls2.MCS2_MEM[2522].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2523 GTM_gtm_cls2.MCS2_MEM[2523].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2524 GTM_gtm_cls2.MCS2_MEM[2524].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2525 GTM_gtm_cls2.MCS2_MEM[2525].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2526 GTM_gtm_cls2.MCS2_MEM[2526].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2527 GTM_gtm_cls2.MCS2_MEM[2527].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2528 GTM_gtm_cls2.MCS2_MEM[2528].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2529 GTM_gtm_cls2.MCS2_MEM[2529].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2530 GTM_gtm_cls2.MCS2_MEM[2530].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2531 GTM_gtm_cls2.MCS2_MEM[2531].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2532 GTM_gtm_cls2.MCS2_MEM[2532].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2533 GTM_gtm_cls2.MCS2_MEM[2533].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2534 GTM_gtm_cls2.MCS2_MEM[2534].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2535 GTM_gtm_cls2.MCS2_MEM[2535].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2536 GTM_gtm_cls2.MCS2_MEM[2536].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2537 GTM_gtm_cls2.MCS2_MEM[2537].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2538 GTM_gtm_cls2.MCS2_MEM[2538].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2539 GTM_gtm_cls2.MCS2_MEM[2539].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2540 GTM_gtm_cls2.MCS2_MEM[2540].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2541 GTM_gtm_cls2.MCS2_MEM[2541].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2542 GTM_gtm_cls2.MCS2_MEM[2542].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2543 GTM_gtm_cls2.MCS2_MEM[2543].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2544 GTM_gtm_cls2.MCS2_MEM[2544].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2545 GTM_gtm_cls2.MCS2_MEM[2545].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2546 GTM_gtm_cls2.MCS2_MEM[2546].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2547 GTM_gtm_cls2.MCS2_MEM[2547].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2548 GTM_gtm_cls2.MCS2_MEM[2548].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2549 GTM_gtm_cls2.MCS2_MEM[2549].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2550 GTM_gtm_cls2.MCS2_MEM[2550].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2551 GTM_gtm_cls2.MCS2_MEM[2551].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2552 GTM_gtm_cls2.MCS2_MEM[2552].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2553 GTM_gtm_cls2.MCS2_MEM[2553].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2554 GTM_gtm_cls2.MCS2_MEM[2554].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2555 GTM_gtm_cls2.MCS2_MEM[2555].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2556 GTM_gtm_cls2.MCS2_MEM[2556].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2557 GTM_gtm_cls2.MCS2_MEM[2557].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2558 GTM_gtm_cls2.MCS2_MEM[2558].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2559 GTM_gtm_cls2.MCS2_MEM[2559].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2560 GTM_gtm_cls2.MCS2_MEM[2560].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2561 GTM_gtm_cls2.MCS2_MEM[2561].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2562 GTM_gtm_cls2.MCS2_MEM[2562].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2563 GTM_gtm_cls2.MCS2_MEM[2563].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2564 GTM_gtm_cls2.MCS2_MEM[2564].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2565 GTM_gtm_cls2.MCS2_MEM[2565].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2566 GTM_gtm_cls2.MCS2_MEM[2566].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2567 GTM_gtm_cls2.MCS2_MEM[2567].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2568 GTM_gtm_cls2.MCS2_MEM[2568].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2569 GTM_gtm_cls2.MCS2_MEM[2569].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2570 GTM_gtm_cls2.MCS2_MEM[2570].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2571 GTM_gtm_cls2.MCS2_MEM[2571].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2572 GTM_gtm_cls2.MCS2_MEM[2572].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2573 GTM_gtm_cls2.MCS2_MEM[2573].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2574 GTM_gtm_cls2.MCS2_MEM[2574].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2575 GTM_gtm_cls2.MCS2_MEM[2575].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2576 GTM_gtm_cls2.MCS2_MEM[2576].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2577 GTM_gtm_cls2.MCS2_MEM[2577].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2578 GTM_gtm_cls2.MCS2_MEM[2578].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2579 GTM_gtm_cls2.MCS2_MEM[2579].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2580 GTM_gtm_cls2.MCS2_MEM[2580].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2581 GTM_gtm_cls2.MCS2_MEM[2581].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2582 GTM_gtm_cls2.MCS2_MEM[2582].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2583 GTM_gtm_cls2.MCS2_MEM[2583].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2584 GTM_gtm_cls2.MCS2_MEM[2584].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2585 GTM_gtm_cls2.MCS2_MEM[2585].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2586 GTM_gtm_cls2.MCS2_MEM[2586].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2587 GTM_gtm_cls2.MCS2_MEM[2587].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2588 GTM_gtm_cls2.MCS2_MEM[2588].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2589 GTM_gtm_cls2.MCS2_MEM[2589].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2590 GTM_gtm_cls2.MCS2_MEM[2590].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2591 GTM_gtm_cls2.MCS2_MEM[2591].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2592 GTM_gtm_cls2.MCS2_MEM[2592].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2593 GTM_gtm_cls2.MCS2_MEM[2593].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2594 GTM_gtm_cls2.MCS2_MEM[2594].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2595 GTM_gtm_cls2.MCS2_MEM[2595].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2596 GTM_gtm_cls2.MCS2_MEM[2596].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2597 GTM_gtm_cls2.MCS2_MEM[2597].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2598 GTM_gtm_cls2.MCS2_MEM[2598].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2599 GTM_gtm_cls2.MCS2_MEM[2599].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2600 GTM_gtm_cls2.MCS2_MEM[2600].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2601 GTM_gtm_cls2.MCS2_MEM[2601].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2602 GTM_gtm_cls2.MCS2_MEM[2602].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2603 GTM_gtm_cls2.MCS2_MEM[2603].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2604 GTM_gtm_cls2.MCS2_MEM[2604].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2605 GTM_gtm_cls2.MCS2_MEM[2605].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2606 GTM_gtm_cls2.MCS2_MEM[2606].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2607 GTM_gtm_cls2.MCS2_MEM[2607].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2608 GTM_gtm_cls2.MCS2_MEM[2608].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2609 GTM_gtm_cls2.MCS2_MEM[2609].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2610 GTM_gtm_cls2.MCS2_MEM[2610].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2611 GTM_gtm_cls2.MCS2_MEM[2611].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2612 GTM_gtm_cls2.MCS2_MEM[2612].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2613 GTM_gtm_cls2.MCS2_MEM[2613].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2614 GTM_gtm_cls2.MCS2_MEM[2614].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2615 GTM_gtm_cls2.MCS2_MEM[2615].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2616 GTM_gtm_cls2.MCS2_MEM[2616].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2617 GTM_gtm_cls2.MCS2_MEM[2617].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2618 GTM_gtm_cls2.MCS2_MEM[2618].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2619 GTM_gtm_cls2.MCS2_MEM[2619].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2620 GTM_gtm_cls2.MCS2_MEM[2620].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2621 GTM_gtm_cls2.MCS2_MEM[2621].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2622 GTM_gtm_cls2.MCS2_MEM[2622].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2623 GTM_gtm_cls2.MCS2_MEM[2623].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2624 GTM_gtm_cls2.MCS2_MEM[2624].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2625 GTM_gtm_cls2.MCS2_MEM[2625].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2626 GTM_gtm_cls2.MCS2_MEM[2626].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2627 GTM_gtm_cls2.MCS2_MEM[2627].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2628 GTM_gtm_cls2.MCS2_MEM[2628].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2629 GTM_gtm_cls2.MCS2_MEM[2629].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2630 GTM_gtm_cls2.MCS2_MEM[2630].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2631 GTM_gtm_cls2.MCS2_MEM[2631].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2632 GTM_gtm_cls2.MCS2_MEM[2632].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2633 GTM_gtm_cls2.MCS2_MEM[2633].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2634 GTM_gtm_cls2.MCS2_MEM[2634].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2635 GTM_gtm_cls2.MCS2_MEM[2635].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2636 GTM_gtm_cls2.MCS2_MEM[2636].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2637 GTM_gtm_cls2.MCS2_MEM[2637].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2638 GTM_gtm_cls2.MCS2_MEM[2638].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2639 GTM_gtm_cls2.MCS2_MEM[2639].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2640 GTM_gtm_cls2.MCS2_MEM[2640].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2641 GTM_gtm_cls2.MCS2_MEM[2641].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2642 GTM_gtm_cls2.MCS2_MEM[2642].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2643 GTM_gtm_cls2.MCS2_MEM[2643].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2644 GTM_gtm_cls2.MCS2_MEM[2644].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2645 GTM_gtm_cls2.MCS2_MEM[2645].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2646 GTM_gtm_cls2.MCS2_MEM[2646].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2647 GTM_gtm_cls2.MCS2_MEM[2647].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2648 GTM_gtm_cls2.MCS2_MEM[2648].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2649 GTM_gtm_cls2.MCS2_MEM[2649].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2650 GTM_gtm_cls2.MCS2_MEM[2650].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2651 GTM_gtm_cls2.MCS2_MEM[2651].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2652 GTM_gtm_cls2.MCS2_MEM[2652].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2653 GTM_gtm_cls2.MCS2_MEM[2653].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2654 GTM_gtm_cls2.MCS2_MEM[2654].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2655 GTM_gtm_cls2.MCS2_MEM[2655].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2656 GTM_gtm_cls2.MCS2_MEM[2656].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2657 GTM_gtm_cls2.MCS2_MEM[2657].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2658 GTM_gtm_cls2.MCS2_MEM[2658].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2659 GTM_gtm_cls2.MCS2_MEM[2659].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2660 GTM_gtm_cls2.MCS2_MEM[2660].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2661 GTM_gtm_cls2.MCS2_MEM[2661].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2662 GTM_gtm_cls2.MCS2_MEM[2662].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2663 GTM_gtm_cls2.MCS2_MEM[2663].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2664 GTM_gtm_cls2.MCS2_MEM[2664].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2665 GTM_gtm_cls2.MCS2_MEM[2665].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2666 GTM_gtm_cls2.MCS2_MEM[2666].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2667 GTM_gtm_cls2.MCS2_MEM[2667].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2668 GTM_gtm_cls2.MCS2_MEM[2668].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2669 GTM_gtm_cls2.MCS2_MEM[2669].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2670 GTM_gtm_cls2.MCS2_MEM[2670].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2671 GTM_gtm_cls2.MCS2_MEM[2671].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2672 GTM_gtm_cls2.MCS2_MEM[2672].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2673 GTM_gtm_cls2.MCS2_MEM[2673].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2674 GTM_gtm_cls2.MCS2_MEM[2674].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2675 GTM_gtm_cls2.MCS2_MEM[2675].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2676 GTM_gtm_cls2.MCS2_MEM[2676].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2677 GTM_gtm_cls2.MCS2_MEM[2677].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2678 GTM_gtm_cls2.MCS2_MEM[2678].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2679 GTM_gtm_cls2.MCS2_MEM[2679].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2680 GTM_gtm_cls2.MCS2_MEM[2680].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2681 GTM_gtm_cls2.MCS2_MEM[2681].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2682 GTM_gtm_cls2.MCS2_MEM[2682].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2683 GTM_gtm_cls2.MCS2_MEM[2683].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2684 GTM_gtm_cls2.MCS2_MEM[2684].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2685 GTM_gtm_cls2.MCS2_MEM[2685].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2686 GTM_gtm_cls2.MCS2_MEM[2686].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2687 GTM_gtm_cls2.MCS2_MEM[2687].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2688 GTM_gtm_cls2.MCS2_MEM[2688].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2689 GTM_gtm_cls2.MCS2_MEM[2689].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2690 GTM_gtm_cls2.MCS2_MEM[2690].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2691 GTM_gtm_cls2.MCS2_MEM[2691].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2692 GTM_gtm_cls2.MCS2_MEM[2692].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2693 GTM_gtm_cls2.MCS2_MEM[2693].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2694 GTM_gtm_cls2.MCS2_MEM[2694].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2695 GTM_gtm_cls2.MCS2_MEM[2695].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2696 GTM_gtm_cls2.MCS2_MEM[2696].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2697 GTM_gtm_cls2.MCS2_MEM[2697].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2698 GTM_gtm_cls2.MCS2_MEM[2698].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2699 GTM_gtm_cls2.MCS2_MEM[2699].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2700 GTM_gtm_cls2.MCS2_MEM[2700].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2701 GTM_gtm_cls2.MCS2_MEM[2701].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2702 GTM_gtm_cls2.MCS2_MEM[2702].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2703 GTM_gtm_cls2.MCS2_MEM[2703].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2704 GTM_gtm_cls2.MCS2_MEM[2704].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2705 GTM_gtm_cls2.MCS2_MEM[2705].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2706 GTM_gtm_cls2.MCS2_MEM[2706].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2707 GTM_gtm_cls2.MCS2_MEM[2707].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2708 GTM_gtm_cls2.MCS2_MEM[2708].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2709 GTM_gtm_cls2.MCS2_MEM[2709].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2710 GTM_gtm_cls2.MCS2_MEM[2710].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2711 GTM_gtm_cls2.MCS2_MEM[2711].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2712 GTM_gtm_cls2.MCS2_MEM[2712].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2713 GTM_gtm_cls2.MCS2_MEM[2713].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2714 GTM_gtm_cls2.MCS2_MEM[2714].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2715 GTM_gtm_cls2.MCS2_MEM[2715].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2716 GTM_gtm_cls2.MCS2_MEM[2716].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2717 GTM_gtm_cls2.MCS2_MEM[2717].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2718 GTM_gtm_cls2.MCS2_MEM[2718].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2719 GTM_gtm_cls2.MCS2_MEM[2719].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2720 GTM_gtm_cls2.MCS2_MEM[2720].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2721 GTM_gtm_cls2.MCS2_MEM[2721].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2722 GTM_gtm_cls2.MCS2_MEM[2722].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2723 GTM_gtm_cls2.MCS2_MEM[2723].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2724 GTM_gtm_cls2.MCS2_MEM[2724].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2725 GTM_gtm_cls2.MCS2_MEM[2725].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2726 GTM_gtm_cls2.MCS2_MEM[2726].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2727 GTM_gtm_cls2.MCS2_MEM[2727].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2728 GTM_gtm_cls2.MCS2_MEM[2728].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2729 GTM_gtm_cls2.MCS2_MEM[2729].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2730 GTM_gtm_cls2.MCS2_MEM[2730].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2731 GTM_gtm_cls2.MCS2_MEM[2731].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2732 GTM_gtm_cls2.MCS2_MEM[2732].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2733 GTM_gtm_cls2.MCS2_MEM[2733].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2734 GTM_gtm_cls2.MCS2_MEM[2734].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2735 GTM_gtm_cls2.MCS2_MEM[2735].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2736 GTM_gtm_cls2.MCS2_MEM[2736].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2737 GTM_gtm_cls2.MCS2_MEM[2737].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2738 GTM_gtm_cls2.MCS2_MEM[2738].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2739 GTM_gtm_cls2.MCS2_MEM[2739].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2740 GTM_gtm_cls2.MCS2_MEM[2740].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2741 GTM_gtm_cls2.MCS2_MEM[2741].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2742 GTM_gtm_cls2.MCS2_MEM[2742].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2743 GTM_gtm_cls2.MCS2_MEM[2743].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2744 GTM_gtm_cls2.MCS2_MEM[2744].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2745 GTM_gtm_cls2.MCS2_MEM[2745].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2746 GTM_gtm_cls2.MCS2_MEM[2746].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2747 GTM_gtm_cls2.MCS2_MEM[2747].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2748 GTM_gtm_cls2.MCS2_MEM[2748].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2749 GTM_gtm_cls2.MCS2_MEM[2749].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2750 GTM_gtm_cls2.MCS2_MEM[2750].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2751 GTM_gtm_cls2.MCS2_MEM[2751].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2752 GTM_gtm_cls2.MCS2_MEM[2752].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2753 GTM_gtm_cls2.MCS2_MEM[2753].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2754 GTM_gtm_cls2.MCS2_MEM[2754].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2755 GTM_gtm_cls2.MCS2_MEM[2755].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2756 GTM_gtm_cls2.MCS2_MEM[2756].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2757 GTM_gtm_cls2.MCS2_MEM[2757].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2758 GTM_gtm_cls2.MCS2_MEM[2758].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2759 GTM_gtm_cls2.MCS2_MEM[2759].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2760 GTM_gtm_cls2.MCS2_MEM[2760].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2761 GTM_gtm_cls2.MCS2_MEM[2761].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2762 GTM_gtm_cls2.MCS2_MEM[2762].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2763 GTM_gtm_cls2.MCS2_MEM[2763].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2764 GTM_gtm_cls2.MCS2_MEM[2764].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2765 GTM_gtm_cls2.MCS2_MEM[2765].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2766 GTM_gtm_cls2.MCS2_MEM[2766].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2767 GTM_gtm_cls2.MCS2_MEM[2767].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2768 GTM_gtm_cls2.MCS2_MEM[2768].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2769 GTM_gtm_cls2.MCS2_MEM[2769].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2770 GTM_gtm_cls2.MCS2_MEM[2770].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2771 GTM_gtm_cls2.MCS2_MEM[2771].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2772 GTM_gtm_cls2.MCS2_MEM[2772].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2773 GTM_gtm_cls2.MCS2_MEM[2773].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2774 GTM_gtm_cls2.MCS2_MEM[2774].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2775 GTM_gtm_cls2.MCS2_MEM[2775].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2776 GTM_gtm_cls2.MCS2_MEM[2776].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2777 GTM_gtm_cls2.MCS2_MEM[2777].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2778 GTM_gtm_cls2.MCS2_MEM[2778].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2779 GTM_gtm_cls2.MCS2_MEM[2779].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2780 GTM_gtm_cls2.MCS2_MEM[2780].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2781 GTM_gtm_cls2.MCS2_MEM[2781].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2782 GTM_gtm_cls2.MCS2_MEM[2782].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2783 GTM_gtm_cls2.MCS2_MEM[2783].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2784 GTM_gtm_cls2.MCS2_MEM[2784].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2785 GTM_gtm_cls2.MCS2_MEM[2785].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2786 GTM_gtm_cls2.MCS2_MEM[2786].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2787 GTM_gtm_cls2.MCS2_MEM[2787].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2788 GTM_gtm_cls2.MCS2_MEM[2788].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2789 GTM_gtm_cls2.MCS2_MEM[2789].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2790 GTM_gtm_cls2.MCS2_MEM[2790].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2791 GTM_gtm_cls2.MCS2_MEM[2791].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2792 GTM_gtm_cls2.MCS2_MEM[2792].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2793 GTM_gtm_cls2.MCS2_MEM[2793].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2794 GTM_gtm_cls2.MCS2_MEM[2794].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2795 GTM_gtm_cls2.MCS2_MEM[2795].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2796 GTM_gtm_cls2.MCS2_MEM[2796].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2797 GTM_gtm_cls2.MCS2_MEM[2797].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2798 GTM_gtm_cls2.MCS2_MEM[2798].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2799 GTM_gtm_cls2.MCS2_MEM[2799].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2800 GTM_gtm_cls2.MCS2_MEM[2800].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2801 GTM_gtm_cls2.MCS2_MEM[2801].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2802 GTM_gtm_cls2.MCS2_MEM[2802].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2803 GTM_gtm_cls2.MCS2_MEM[2803].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2804 GTM_gtm_cls2.MCS2_MEM[2804].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2805 GTM_gtm_cls2.MCS2_MEM[2805].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2806 GTM_gtm_cls2.MCS2_MEM[2806].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2807 GTM_gtm_cls2.MCS2_MEM[2807].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2808 GTM_gtm_cls2.MCS2_MEM[2808].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2809 GTM_gtm_cls2.MCS2_MEM[2809].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2810 GTM_gtm_cls2.MCS2_MEM[2810].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2811 GTM_gtm_cls2.MCS2_MEM[2811].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2812 GTM_gtm_cls2.MCS2_MEM[2812].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2813 GTM_gtm_cls2.MCS2_MEM[2813].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2814 GTM_gtm_cls2.MCS2_MEM[2814].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2815 GTM_gtm_cls2.MCS2_MEM[2815].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2816 GTM_gtm_cls2.MCS2_MEM[2816].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2817 GTM_gtm_cls2.MCS2_MEM[2817].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2818 GTM_gtm_cls2.MCS2_MEM[2818].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2819 GTM_gtm_cls2.MCS2_MEM[2819].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2820 GTM_gtm_cls2.MCS2_MEM[2820].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2821 GTM_gtm_cls2.MCS2_MEM[2821].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2822 GTM_gtm_cls2.MCS2_MEM[2822].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2823 GTM_gtm_cls2.MCS2_MEM[2823].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2824 GTM_gtm_cls2.MCS2_MEM[2824].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2825 GTM_gtm_cls2.MCS2_MEM[2825].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2826 GTM_gtm_cls2.MCS2_MEM[2826].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2827 GTM_gtm_cls2.MCS2_MEM[2827].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2828 GTM_gtm_cls2.MCS2_MEM[2828].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2829 GTM_gtm_cls2.MCS2_MEM[2829].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2830 GTM_gtm_cls2.MCS2_MEM[2830].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2831 GTM_gtm_cls2.MCS2_MEM[2831].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2832 GTM_gtm_cls2.MCS2_MEM[2832].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2833 GTM_gtm_cls2.MCS2_MEM[2833].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2834 GTM_gtm_cls2.MCS2_MEM[2834].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2835 GTM_gtm_cls2.MCS2_MEM[2835].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2836 GTM_gtm_cls2.MCS2_MEM[2836].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2837 GTM_gtm_cls2.MCS2_MEM[2837].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2838 GTM_gtm_cls2.MCS2_MEM[2838].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2839 GTM_gtm_cls2.MCS2_MEM[2839].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2840 GTM_gtm_cls2.MCS2_MEM[2840].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2841 GTM_gtm_cls2.MCS2_MEM[2841].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2842 GTM_gtm_cls2.MCS2_MEM[2842].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2843 GTM_gtm_cls2.MCS2_MEM[2843].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2844 GTM_gtm_cls2.MCS2_MEM[2844].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2845 GTM_gtm_cls2.MCS2_MEM[2845].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2846 GTM_gtm_cls2.MCS2_MEM[2846].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2847 GTM_gtm_cls2.MCS2_MEM[2847].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2848 GTM_gtm_cls2.MCS2_MEM[2848].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2849 GTM_gtm_cls2.MCS2_MEM[2849].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2850 GTM_gtm_cls2.MCS2_MEM[2850].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2851 GTM_gtm_cls2.MCS2_MEM[2851].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2852 GTM_gtm_cls2.MCS2_MEM[2852].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2853 GTM_gtm_cls2.MCS2_MEM[2853].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2854 GTM_gtm_cls2.MCS2_MEM[2854].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2855 GTM_gtm_cls2.MCS2_MEM[2855].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2856 GTM_gtm_cls2.MCS2_MEM[2856].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2857 GTM_gtm_cls2.MCS2_MEM[2857].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2858 GTM_gtm_cls2.MCS2_MEM[2858].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2859 GTM_gtm_cls2.MCS2_MEM[2859].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2860 GTM_gtm_cls2.MCS2_MEM[2860].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2861 GTM_gtm_cls2.MCS2_MEM[2861].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2862 GTM_gtm_cls2.MCS2_MEM[2862].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2863 GTM_gtm_cls2.MCS2_MEM[2863].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2864 GTM_gtm_cls2.MCS2_MEM[2864].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2865 GTM_gtm_cls2.MCS2_MEM[2865].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2866 GTM_gtm_cls2.MCS2_MEM[2866].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2867 GTM_gtm_cls2.MCS2_MEM[2867].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2868 GTM_gtm_cls2.MCS2_MEM[2868].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2869 GTM_gtm_cls2.MCS2_MEM[2869].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2870 GTM_gtm_cls2.MCS2_MEM[2870].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2871 GTM_gtm_cls2.MCS2_MEM[2871].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2872 GTM_gtm_cls2.MCS2_MEM[2872].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2873 GTM_gtm_cls2.MCS2_MEM[2873].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2874 GTM_gtm_cls2.MCS2_MEM[2874].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2875 GTM_gtm_cls2.MCS2_MEM[2875].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2876 GTM_gtm_cls2.MCS2_MEM[2876].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2877 GTM_gtm_cls2.MCS2_MEM[2877].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2878 GTM_gtm_cls2.MCS2_MEM[2878].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2879 GTM_gtm_cls2.MCS2_MEM[2879].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2880 GTM_gtm_cls2.MCS2_MEM[2880].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2881 GTM_gtm_cls2.MCS2_MEM[2881].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2882 GTM_gtm_cls2.MCS2_MEM[2882].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2883 GTM_gtm_cls2.MCS2_MEM[2883].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2884 GTM_gtm_cls2.MCS2_MEM[2884].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2885 GTM_gtm_cls2.MCS2_MEM[2885].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2886 GTM_gtm_cls2.MCS2_MEM[2886].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2887 GTM_gtm_cls2.MCS2_MEM[2887].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2888 GTM_gtm_cls2.MCS2_MEM[2888].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2889 GTM_gtm_cls2.MCS2_MEM[2889].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2890 GTM_gtm_cls2.MCS2_MEM[2890].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2891 GTM_gtm_cls2.MCS2_MEM[2891].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2892 GTM_gtm_cls2.MCS2_MEM[2892].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2893 GTM_gtm_cls2.MCS2_MEM[2893].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2894 GTM_gtm_cls2.MCS2_MEM[2894].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2895 GTM_gtm_cls2.MCS2_MEM[2895].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2896 GTM_gtm_cls2.MCS2_MEM[2896].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2897 GTM_gtm_cls2.MCS2_MEM[2897].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2898 GTM_gtm_cls2.MCS2_MEM[2898].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2899 GTM_gtm_cls2.MCS2_MEM[2899].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2900 GTM_gtm_cls2.MCS2_MEM[2900].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2901 GTM_gtm_cls2.MCS2_MEM[2901].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2902 GTM_gtm_cls2.MCS2_MEM[2902].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2903 GTM_gtm_cls2.MCS2_MEM[2903].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2904 GTM_gtm_cls2.MCS2_MEM[2904].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2905 GTM_gtm_cls2.MCS2_MEM[2905].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2906 GTM_gtm_cls2.MCS2_MEM[2906].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2907 GTM_gtm_cls2.MCS2_MEM[2907].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2908 GTM_gtm_cls2.MCS2_MEM[2908].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2909 GTM_gtm_cls2.MCS2_MEM[2909].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2910 GTM_gtm_cls2.MCS2_MEM[2910].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2911 GTM_gtm_cls2.MCS2_MEM[2911].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2912 GTM_gtm_cls2.MCS2_MEM[2912].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2913 GTM_gtm_cls2.MCS2_MEM[2913].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2914 GTM_gtm_cls2.MCS2_MEM[2914].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2915 GTM_gtm_cls2.MCS2_MEM[2915].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2916 GTM_gtm_cls2.MCS2_MEM[2916].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2917 GTM_gtm_cls2.MCS2_MEM[2917].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2918 GTM_gtm_cls2.MCS2_MEM[2918].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2919 GTM_gtm_cls2.MCS2_MEM[2919].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2920 GTM_gtm_cls2.MCS2_MEM[2920].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2921 GTM_gtm_cls2.MCS2_MEM[2921].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2922 GTM_gtm_cls2.MCS2_MEM[2922].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2923 GTM_gtm_cls2.MCS2_MEM[2923].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2924 GTM_gtm_cls2.MCS2_MEM[2924].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2925 GTM_gtm_cls2.MCS2_MEM[2925].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2926 GTM_gtm_cls2.MCS2_MEM[2926].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2927 GTM_gtm_cls2.MCS2_MEM[2927].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2928 GTM_gtm_cls2.MCS2_MEM[2928].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2929 GTM_gtm_cls2.MCS2_MEM[2929].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2930 GTM_gtm_cls2.MCS2_MEM[2930].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2931 GTM_gtm_cls2.MCS2_MEM[2931].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2932 GTM_gtm_cls2.MCS2_MEM[2932].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2933 GTM_gtm_cls2.MCS2_MEM[2933].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2934 GTM_gtm_cls2.MCS2_MEM[2934].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2935 GTM_gtm_cls2.MCS2_MEM[2935].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2936 GTM_gtm_cls2.MCS2_MEM[2936].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2937 GTM_gtm_cls2.MCS2_MEM[2937].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2938 GTM_gtm_cls2.MCS2_MEM[2938].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2939 GTM_gtm_cls2.MCS2_MEM[2939].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2940 GTM_gtm_cls2.MCS2_MEM[2940].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2941 GTM_gtm_cls2.MCS2_MEM[2941].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2942 GTM_gtm_cls2.MCS2_MEM[2942].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2943 GTM_gtm_cls2.MCS2_MEM[2943].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2944 GTM_gtm_cls2.MCS2_MEM[2944].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2945 GTM_gtm_cls2.MCS2_MEM[2945].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2946 GTM_gtm_cls2.MCS2_MEM[2946].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2947 GTM_gtm_cls2.MCS2_MEM[2947].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2948 GTM_gtm_cls2.MCS2_MEM[2948].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2949 GTM_gtm_cls2.MCS2_MEM[2949].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2950 GTM_gtm_cls2.MCS2_MEM[2950].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2951 GTM_gtm_cls2.MCS2_MEM[2951].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2952 GTM_gtm_cls2.MCS2_MEM[2952].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2953 GTM_gtm_cls2.MCS2_MEM[2953].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2954 GTM_gtm_cls2.MCS2_MEM[2954].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2955 GTM_gtm_cls2.MCS2_MEM[2955].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2956 GTM_gtm_cls2.MCS2_MEM[2956].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2957 GTM_gtm_cls2.MCS2_MEM[2957].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2958 GTM_gtm_cls2.MCS2_MEM[2958].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2959 GTM_gtm_cls2.MCS2_MEM[2959].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2960 GTM_gtm_cls2.MCS2_MEM[2960].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2961 GTM_gtm_cls2.MCS2_MEM[2961].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2962 GTM_gtm_cls2.MCS2_MEM[2962].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2963 GTM_gtm_cls2.MCS2_MEM[2963].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2964 GTM_gtm_cls2.MCS2_MEM[2964].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2965 GTM_gtm_cls2.MCS2_MEM[2965].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2966 GTM_gtm_cls2.MCS2_MEM[2966].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2967 GTM_gtm_cls2.MCS2_MEM[2967].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2968 GTM_gtm_cls2.MCS2_MEM[2968].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2969 GTM_gtm_cls2.MCS2_MEM[2969].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2970 GTM_gtm_cls2.MCS2_MEM[2970].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2971 GTM_gtm_cls2.MCS2_MEM[2971].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2972 GTM_gtm_cls2.MCS2_MEM[2972].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2973 GTM_gtm_cls2.MCS2_MEM[2973].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2974 GTM_gtm_cls2.MCS2_MEM[2974].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2975 GTM_gtm_cls2.MCS2_MEM[2975].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2976 GTM_gtm_cls2.MCS2_MEM[2976].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2977 GTM_gtm_cls2.MCS2_MEM[2977].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2978 GTM_gtm_cls2.MCS2_MEM[2978].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2979 GTM_gtm_cls2.MCS2_MEM[2979].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2980 GTM_gtm_cls2.MCS2_MEM[2980].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2981 GTM_gtm_cls2.MCS2_MEM[2981].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2982 GTM_gtm_cls2.MCS2_MEM[2982].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2983 GTM_gtm_cls2.MCS2_MEM[2983].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2984 GTM_gtm_cls2.MCS2_MEM[2984].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2985 GTM_gtm_cls2.MCS2_MEM[2985].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2986 GTM_gtm_cls2.MCS2_MEM[2986].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2987 GTM_gtm_cls2.MCS2_MEM[2987].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2988 GTM_gtm_cls2.MCS2_MEM[2988].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2989 GTM_gtm_cls2.MCS2_MEM[2989].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2990 GTM_gtm_cls2.MCS2_MEM[2990].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2991 GTM_gtm_cls2.MCS2_MEM[2991].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2992 GTM_gtm_cls2.MCS2_MEM[2992].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2993 GTM_gtm_cls2.MCS2_MEM[2993].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2994 GTM_gtm_cls2.MCS2_MEM[2994].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2995 GTM_gtm_cls2.MCS2_MEM[2995].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2996 GTM_gtm_cls2.MCS2_MEM[2996].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2997 GTM_gtm_cls2.MCS2_MEM[2997].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2998 GTM_gtm_cls2.MCS2_MEM[2998].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM2999 GTM_gtm_cls2.MCS2_MEM[2999].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3000 GTM_gtm_cls2.MCS2_MEM[3000].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3001 GTM_gtm_cls2.MCS2_MEM[3001].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3002 GTM_gtm_cls2.MCS2_MEM[3002].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3003 GTM_gtm_cls2.MCS2_MEM[3003].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3004 GTM_gtm_cls2.MCS2_MEM[3004].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3005 GTM_gtm_cls2.MCS2_MEM[3005].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3006 GTM_gtm_cls2.MCS2_MEM[3006].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3007 GTM_gtm_cls2.MCS2_MEM[3007].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3008 GTM_gtm_cls2.MCS2_MEM[3008].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3009 GTM_gtm_cls2.MCS2_MEM[3009].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3010 GTM_gtm_cls2.MCS2_MEM[3010].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3011 GTM_gtm_cls2.MCS2_MEM[3011].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3012 GTM_gtm_cls2.MCS2_MEM[3012].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3013 GTM_gtm_cls2.MCS2_MEM[3013].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3014 GTM_gtm_cls2.MCS2_MEM[3014].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3015 GTM_gtm_cls2.MCS2_MEM[3015].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3016 GTM_gtm_cls2.MCS2_MEM[3016].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3017 GTM_gtm_cls2.MCS2_MEM[3017].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3018 GTM_gtm_cls2.MCS2_MEM[3018].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3019 GTM_gtm_cls2.MCS2_MEM[3019].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3020 GTM_gtm_cls2.MCS2_MEM[3020].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3021 GTM_gtm_cls2.MCS2_MEM[3021].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3022 GTM_gtm_cls2.MCS2_MEM[3022].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3023 GTM_gtm_cls2.MCS2_MEM[3023].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3024 GTM_gtm_cls2.MCS2_MEM[3024].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3025 GTM_gtm_cls2.MCS2_MEM[3025].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3026 GTM_gtm_cls2.MCS2_MEM[3026].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3027 GTM_gtm_cls2.MCS2_MEM[3027].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3028 GTM_gtm_cls2.MCS2_MEM[3028].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3029 GTM_gtm_cls2.MCS2_MEM[3029].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3030 GTM_gtm_cls2.MCS2_MEM[3030].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3031 GTM_gtm_cls2.MCS2_MEM[3031].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3032 GTM_gtm_cls2.MCS2_MEM[3032].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3033 GTM_gtm_cls2.MCS2_MEM[3033].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3034 GTM_gtm_cls2.MCS2_MEM[3034].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3035 GTM_gtm_cls2.MCS2_MEM[3035].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3036 GTM_gtm_cls2.MCS2_MEM[3036].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3037 GTM_gtm_cls2.MCS2_MEM[3037].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3038 GTM_gtm_cls2.MCS2_MEM[3038].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3039 GTM_gtm_cls2.MCS2_MEM[3039].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3040 GTM_gtm_cls2.MCS2_MEM[3040].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3041 GTM_gtm_cls2.MCS2_MEM[3041].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3042 GTM_gtm_cls2.MCS2_MEM[3042].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3043 GTM_gtm_cls2.MCS2_MEM[3043].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3044 GTM_gtm_cls2.MCS2_MEM[3044].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3045 GTM_gtm_cls2.MCS2_MEM[3045].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3046 GTM_gtm_cls2.MCS2_MEM[3046].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3047 GTM_gtm_cls2.MCS2_MEM[3047].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3048 GTM_gtm_cls2.MCS2_MEM[3048].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3049 GTM_gtm_cls2.MCS2_MEM[3049].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3050 GTM_gtm_cls2.MCS2_MEM[3050].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3051 GTM_gtm_cls2.MCS2_MEM[3051].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3052 GTM_gtm_cls2.MCS2_MEM[3052].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3053 GTM_gtm_cls2.MCS2_MEM[3053].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3054 GTM_gtm_cls2.MCS2_MEM[3054].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3055 GTM_gtm_cls2.MCS2_MEM[3055].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3056 GTM_gtm_cls2.MCS2_MEM[3056].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3057 GTM_gtm_cls2.MCS2_MEM[3057].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3058 GTM_gtm_cls2.MCS2_MEM[3058].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3059 GTM_gtm_cls2.MCS2_MEM[3059].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3060 GTM_gtm_cls2.MCS2_MEM[3060].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3061 GTM_gtm_cls2.MCS2_MEM[3061].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3062 GTM_gtm_cls2.MCS2_MEM[3062].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3063 GTM_gtm_cls2.MCS2_MEM[3063].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3064 GTM_gtm_cls2.MCS2_MEM[3064].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3065 GTM_gtm_cls2.MCS2_MEM[3065].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3066 GTM_gtm_cls2.MCS2_MEM[3066].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3067 GTM_gtm_cls2.MCS2_MEM[3067].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3068 GTM_gtm_cls2.MCS2_MEM[3068].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3069 GTM_gtm_cls2.MCS2_MEM[3069].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3070 GTM_gtm_cls2.MCS2_MEM[3070].R /* MCS[i] memory region */
#define GTM_gtm_cls2_MCS2_MEM3071 GTM_gtm_cls2.MCS2_MEM[3071].R /* MCS[i] memory region */

/* GTM_gtm_cls3 */
#define GTM_gtm_cls3_ATOM3_CH0_RDADDR GTM_gtm_cls3.ATOM3_CH0_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls3_ATOM3_CH0_CTRL GTM_gtm_cls3.ATOM3_CH0_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls3_ATOM3_CH0_SR0 GTM_gtm_cls3.ATOM3_CH0_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH0_SR1 GTM_gtm_cls3.ATOM3_CH0_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH0_CM0 GTM_gtm_cls3.ATOM3_CH0_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH0_CM1 GTM_gtm_cls3.ATOM3_CH0_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH0_CN0 GTM_gtm_cls3.ATOM3_CH0_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls3_ATOM3_CH0_STAT GTM_gtm_cls3.ATOM3_CH0_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls3_ATOM3_CH0_IRQ_NOTIFY GTM_gtm_cls3.ATOM3_CH0_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls3_ATOM3_CH0_IRQ_EN GTM_gtm_cls3.ATOM3_CH0_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls3_ATOM3_CH0_IRQ_FORCINT GTM_gtm_cls3.ATOM3_CH0_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls3_ATOM3_CH0_IRQ_MODE GTM_gtm_cls3.ATOM3_CH0_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls3_ATOM3_CH0_CTRL_SR GTM_gtm_cls3.ATOM3_CH0_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls3_ATOM3_CH1_RDADDR GTM_gtm_cls3.ATOM3_CH1_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls3_ATOM3_CH1_CTRL GTM_gtm_cls3.ATOM3_CH1_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls3_ATOM3_CH1_SR0 GTM_gtm_cls3.ATOM3_CH1_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH1_SR1 GTM_gtm_cls3.ATOM3_CH1_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH1_CM0 GTM_gtm_cls3.ATOM3_CH1_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH1_CM1 GTM_gtm_cls3.ATOM3_CH1_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH1_CN0 GTM_gtm_cls3.ATOM3_CH1_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls3_ATOM3_CH1_STAT GTM_gtm_cls3.ATOM3_CH1_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls3_ATOM3_CH1_IRQ_NOTIFY GTM_gtm_cls3.ATOM3_CH1_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls3_ATOM3_CH1_IRQ_EN GTM_gtm_cls3.ATOM3_CH1_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls3_ATOM3_CH1_IRQ_FORCINT GTM_gtm_cls3.ATOM3_CH1_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls3_ATOM3_CH1_IRQ_MODE GTM_gtm_cls3.ATOM3_CH1_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls3_ATOM3_CH1_CTRL_SR GTM_gtm_cls3.ATOM3_CH1_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls3_ATOM3_CH2_RDADDR GTM_gtm_cls3.ATOM3_CH2_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls3_ATOM3_CH2_CTRL GTM_gtm_cls3.ATOM3_CH2_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls3_ATOM3_CH2_SR0 GTM_gtm_cls3.ATOM3_CH2_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH2_SR1 GTM_gtm_cls3.ATOM3_CH2_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH2_CM0 GTM_gtm_cls3.ATOM3_CH2_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH2_CM1 GTM_gtm_cls3.ATOM3_CH2_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH2_CN0 GTM_gtm_cls3.ATOM3_CH2_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls3_ATOM3_CH2_STAT GTM_gtm_cls3.ATOM3_CH2_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls3_ATOM3_CH2_IRQ_NOTIFY GTM_gtm_cls3.ATOM3_CH2_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls3_ATOM3_CH2_IRQ_EN GTM_gtm_cls3.ATOM3_CH2_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls3_ATOM3_CH2_IRQ_FORCINT GTM_gtm_cls3.ATOM3_CH2_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls3_ATOM3_CH2_IRQ_MODE GTM_gtm_cls3.ATOM3_CH2_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls3_ATOM3_CH2_CTRL_SR GTM_gtm_cls3.ATOM3_CH2_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls3_ATOM3_CH3_RDADDR GTM_gtm_cls3.ATOM3_CH3_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls3_ATOM3_CH3_CTRL GTM_gtm_cls3.ATOM3_CH3_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls3_ATOM3_CH3_SR0 GTM_gtm_cls3.ATOM3_CH3_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH3_SR1 GTM_gtm_cls3.ATOM3_CH3_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH3_CM0 GTM_gtm_cls3.ATOM3_CH3_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH3_CM1 GTM_gtm_cls3.ATOM3_CH3_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH3_CN0 GTM_gtm_cls3.ATOM3_CH3_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls3_ATOM3_CH3_STAT GTM_gtm_cls3.ATOM3_CH3_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls3_ATOM3_CH3_IRQ_NOTIFY GTM_gtm_cls3.ATOM3_CH3_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls3_ATOM3_CH3_IRQ_EN GTM_gtm_cls3.ATOM3_CH3_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls3_ATOM3_CH3_IRQ_FORCINT GTM_gtm_cls3.ATOM3_CH3_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls3_ATOM3_CH3_IRQ_MODE GTM_gtm_cls3.ATOM3_CH3_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls3_ATOM3_CH3_CTRL_SR GTM_gtm_cls3.ATOM3_CH3_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls3_ATOM3_CH4_RDADDR GTM_gtm_cls3.ATOM3_CH4_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls3_ATOM3_CH4_CTRL GTM_gtm_cls3.ATOM3_CH4_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls3_ATOM3_CH4_SR0 GTM_gtm_cls3.ATOM3_CH4_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH4_SR1 GTM_gtm_cls3.ATOM3_CH4_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH4_CM0 GTM_gtm_cls3.ATOM3_CH4_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH4_CM1 GTM_gtm_cls3.ATOM3_CH4_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH4_CN0 GTM_gtm_cls3.ATOM3_CH4_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls3_ATOM3_CH4_STAT GTM_gtm_cls3.ATOM3_CH4_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls3_ATOM3_CH4_IRQ_NOTIFY GTM_gtm_cls3.ATOM3_CH4_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls3_ATOM3_CH4_IRQ_EN GTM_gtm_cls3.ATOM3_CH4_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls3_ATOM3_CH4_IRQ_FORCINT GTM_gtm_cls3.ATOM3_CH4_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls3_ATOM3_CH4_IRQ_MODE GTM_gtm_cls3.ATOM3_CH4_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls3_ATOM3_CH4_CTRL_SR GTM_gtm_cls3.ATOM3_CH4_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls3_ATOM3_CH5_RDADDR GTM_gtm_cls3.ATOM3_CH5_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls3_ATOM3_CH5_CTRL GTM_gtm_cls3.ATOM3_CH5_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls3_ATOM3_CH5_SR0 GTM_gtm_cls3.ATOM3_CH5_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH5_SR1 GTM_gtm_cls3.ATOM3_CH5_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH5_CM0 GTM_gtm_cls3.ATOM3_CH5_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH5_CM1 GTM_gtm_cls3.ATOM3_CH5_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH5_CN0 GTM_gtm_cls3.ATOM3_CH5_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls3_ATOM3_CH5_STAT GTM_gtm_cls3.ATOM3_CH5_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls3_ATOM3_CH5_IRQ_NOTIFY GTM_gtm_cls3.ATOM3_CH5_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls3_ATOM3_CH5_IRQ_EN GTM_gtm_cls3.ATOM3_CH5_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls3_ATOM3_CH5_IRQ_FORCINT GTM_gtm_cls3.ATOM3_CH5_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls3_ATOM3_CH5_IRQ_MODE GTM_gtm_cls3.ATOM3_CH5_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls3_ATOM3_CH5_CTRL_SR GTM_gtm_cls3.ATOM3_CH5_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls3_ATOM3_CH6_RDADDR GTM_gtm_cls3.ATOM3_CH6_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls3_ATOM3_CH6_CTRL GTM_gtm_cls3.ATOM3_CH6_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls3_ATOM3_CH6_SR0 GTM_gtm_cls3.ATOM3_CH6_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH6_SR1 GTM_gtm_cls3.ATOM3_CH6_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH6_CM0 GTM_gtm_cls3.ATOM3_CH6_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH6_CM1 GTM_gtm_cls3.ATOM3_CH6_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH6_CN0 GTM_gtm_cls3.ATOM3_CH6_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls3_ATOM3_CH6_STAT GTM_gtm_cls3.ATOM3_CH6_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls3_ATOM3_CH6_IRQ_NOTIFY GTM_gtm_cls3.ATOM3_CH6_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls3_ATOM3_CH6_IRQ_EN GTM_gtm_cls3.ATOM3_CH6_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls3_ATOM3_CH6_IRQ_FORCINT GTM_gtm_cls3.ATOM3_CH6_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls3_ATOM3_CH6_IRQ_MODE GTM_gtm_cls3.ATOM3_CH6_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls3_ATOM3_CH6_CTRL_SR GTM_gtm_cls3.ATOM3_CH6_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls3_ATOM3_CH7_RDADDR GTM_gtm_cls3.ATOM3_CH7_RDADDR.R /* ATOM[i] channel[x] ARU read address register */
#define GTM_gtm_cls3_ATOM3_CH7_CTRL GTM_gtm_cls3.ATOM3_CH7_CTRL.R /* ATOM[i] channel [x] control register */
#define GTM_gtm_cls3_ATOM3_CH7_SR0 GTM_gtm_cls3.ATOM3_CH7_SR0.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH7_SR1 GTM_gtm_cls3.ATOM3_CH7_SR1.R /* ATOM[i] channel [x] CCU0 compare shadow register */
#define GTM_gtm_cls3_ATOM3_CH7_CM0 GTM_gtm_cls3.ATOM3_CH7_CM0.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH7_CM1 GTM_gtm_cls3.ATOM3_CH7_CM1.R /* ATOM[i] channel [x] CCU0 compare register */
#define GTM_gtm_cls3_ATOM3_CH7_CN0 GTM_gtm_cls3.ATOM3_CH7_CN0.R /* ATOM[i] channel [x] CCU0 counter register */
#define GTM_gtm_cls3_ATOM3_CH7_STAT GTM_gtm_cls3.ATOM3_CH7_STAT.R /* ATOM[i] channel [x] status register */
#define GTM_gtm_cls3_ATOM3_CH7_IRQ_NOTIFY GTM_gtm_cls3.ATOM3_CH7_IRQ_NOTIFY.R /* ATOM[i] channel [x] interrupt notification register */
#define GTM_gtm_cls3_ATOM3_CH7_IRQ_EN GTM_gtm_cls3.ATOM3_CH7_IRQ_EN.R /* ATOM[i] channel [x] interrupt enable register */
#define GTM_gtm_cls3_ATOM3_CH7_IRQ_FORCINT GTM_gtm_cls3.ATOM3_CH7_IRQ_FORCINT.R /* ATOM[i] channel [x] software interrupt generation */
#define GTM_gtm_cls3_ATOM3_CH7_IRQ_MODE GTM_gtm_cls3.ATOM3_CH7_IRQ_MODE.R /* ATOM[i] channel [x] interrupt mode configuration register */
#define GTM_gtm_cls3_ATOM3_CH7_CTRL_SR GTM_gtm_cls3.ATOM3_CH7_CTRL_SR.R /* ATOM[i] channel [x] control shadow register */
#define GTM_gtm_cls3_ATOM3_AGC_GLB_CTRL GTM_gtm_cls3.ATOM3_AGC_GLB_CTRL.R /* ATOM[i] AGC global control register */
#define GTM_gtm_cls3_ATOM3_AGC_ENDIS_CTRL GTM_gtm_cls3.ATOM3_AGC_ENDIS_CTRL.R /* ATOM[i] AGC enable/disable control register */
#define GTM_gtm_cls3_ATOM3_AGC_ENDIS_STAT GTM_gtm_cls3.ATOM3_AGC_ENDIS_STAT.R /* ATOM[i] AGC enable/disable status register */
#define GTM_gtm_cls3_ATOM3_AGC_ACT_TB GTM_gtm_cls3.ATOM3_AGC_ACT_TB.R /* ATOM[i] AGC action time base register */
#define GTM_gtm_cls3_ATOM3_AGC_OUTEN_CTRL GTM_gtm_cls3.ATOM3_AGC_OUTEN_CTRL.R /* ATOM[i] AGC output enable control register */
#define GTM_gtm_cls3_ATOM3_AGC_OUTEN_STAT GTM_gtm_cls3.ATOM3_AGC_OUTEN_STAT.R /* ATOM[i] AGC output enable status register */
#define GTM_gtm_cls3_ATOM3_AGC_FUPD_CTRL GTM_gtm_cls3.ATOM3_AGC_FUPD_CTRL.R /* ATOM[i] AGC force update control register */
#define GTM_gtm_cls3_ATOM3_AGC_INT_TRIG GTM_gtm_cls3.ATOM3_AGC_INT_TRIG.R /* ATOM[i] AGC internal trigger control register */
#define GTM_gtm_cls3_MCS3_CH0_R0 GTM_gtm_cls3.MCS3_CH0_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH0_R1 GTM_gtm_cls3.MCS3_CH0_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH0_R2 GTM_gtm_cls3.MCS3_CH0_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH0_R3 GTM_gtm_cls3.MCS3_CH0_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH0_R4 GTM_gtm_cls3.MCS3_CH0_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH0_R5 GTM_gtm_cls3.MCS3_CH0_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH0_R6 GTM_gtm_cls3.MCS3_CH0_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH0_R7 GTM_gtm_cls3.MCS3_CH0_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH0_CTRL GTM_gtm_cls3.MCS3_CH0_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls3_MCS3_CH0_ACB GTM_gtm_cls3.MCS3_CH0_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls3_MCS3_CH0_MHB GTM_gtm_cls3.MCS3_CH0_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls3_MCS3_CH0_PC GTM_gtm_cls3.MCS3_CH0_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls3_MCS3_CH0_IRQ_NOTIFY GTM_gtm_cls3.MCS3_CH0_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls3_MCS3_CH0_IRQ_EN GTM_gtm_cls3.MCS3_CH0_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH0_IRQ_FORCINT GTM_gtm_cls3.MCS3_CH0_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls3_MCS3_CH0_IRQ_MODE GTM_gtm_cls3.MCS3_CH0_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls3_MCS3_CH0_EIRQ_EN GTM_gtm_cls3.MCS3_CH0_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH1_R0 GTM_gtm_cls3.MCS3_CH1_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH1_R1 GTM_gtm_cls3.MCS3_CH1_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH1_R2 GTM_gtm_cls3.MCS3_CH1_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH1_R3 GTM_gtm_cls3.MCS3_CH1_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH1_R4 GTM_gtm_cls3.MCS3_CH1_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH1_R5 GTM_gtm_cls3.MCS3_CH1_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH1_R6 GTM_gtm_cls3.MCS3_CH1_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH1_R7 GTM_gtm_cls3.MCS3_CH1_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH1_CTRL GTM_gtm_cls3.MCS3_CH1_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls3_MCS3_CH1_ACB GTM_gtm_cls3.MCS3_CH1_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls3_MCS3_CH1_MHB GTM_gtm_cls3.MCS3_CH1_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls3_MCS3_CH1_PC GTM_gtm_cls3.MCS3_CH1_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls3_MCS3_CH1_IRQ_NOTIFY GTM_gtm_cls3.MCS3_CH1_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls3_MCS3_CH1_IRQ_EN GTM_gtm_cls3.MCS3_CH1_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH1_IRQ_FORCINT GTM_gtm_cls3.MCS3_CH1_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls3_MCS3_CH1_IRQ_MODE GTM_gtm_cls3.MCS3_CH1_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls3_MCS3_CH1_EIRQ_EN GTM_gtm_cls3.MCS3_CH1_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH2_R0 GTM_gtm_cls3.MCS3_CH2_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH2_R1 GTM_gtm_cls3.MCS3_CH2_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH2_R2 GTM_gtm_cls3.MCS3_CH2_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH2_R3 GTM_gtm_cls3.MCS3_CH2_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH2_R4 GTM_gtm_cls3.MCS3_CH2_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH2_R5 GTM_gtm_cls3.MCS3_CH2_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH2_R6 GTM_gtm_cls3.MCS3_CH2_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH2_R7 GTM_gtm_cls3.MCS3_CH2_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH2_CTRL GTM_gtm_cls3.MCS3_CH2_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls3_MCS3_CH2_ACB GTM_gtm_cls3.MCS3_CH2_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls3_MCS3_CH2_MHB GTM_gtm_cls3.MCS3_CH2_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls3_MCS3_CH2_PC GTM_gtm_cls3.MCS3_CH2_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls3_MCS3_CH2_IRQ_NOTIFY GTM_gtm_cls3.MCS3_CH2_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls3_MCS3_CH2_IRQ_EN GTM_gtm_cls3.MCS3_CH2_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH2_IRQ_FORCINT GTM_gtm_cls3.MCS3_CH2_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls3_MCS3_CH2_IRQ_MODE GTM_gtm_cls3.MCS3_CH2_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls3_MCS3_CH2_EIRQ_EN GTM_gtm_cls3.MCS3_CH2_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH3_R0 GTM_gtm_cls3.MCS3_CH3_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH3_R1 GTM_gtm_cls3.MCS3_CH3_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH3_R2 GTM_gtm_cls3.MCS3_CH3_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH3_R3 GTM_gtm_cls3.MCS3_CH3_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH3_R4 GTM_gtm_cls3.MCS3_CH3_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH3_R5 GTM_gtm_cls3.MCS3_CH3_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH3_R6 GTM_gtm_cls3.MCS3_CH3_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH3_R7 GTM_gtm_cls3.MCS3_CH3_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH3_CTRL GTM_gtm_cls3.MCS3_CH3_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls3_MCS3_CH3_ACB GTM_gtm_cls3.MCS3_CH3_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls3_MCS3_CH3_MHB GTM_gtm_cls3.MCS3_CH3_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls3_MCS3_CH3_PC GTM_gtm_cls3.MCS3_CH3_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls3_MCS3_CH3_IRQ_NOTIFY GTM_gtm_cls3.MCS3_CH3_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls3_MCS3_CH3_IRQ_EN GTM_gtm_cls3.MCS3_CH3_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH3_IRQ_FORCINT GTM_gtm_cls3.MCS3_CH3_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls3_MCS3_CH3_IRQ_MODE GTM_gtm_cls3.MCS3_CH3_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls3_MCS3_CH3_EIRQ_EN GTM_gtm_cls3.MCS3_CH3_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH4_R0 GTM_gtm_cls3.MCS3_CH4_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH4_R1 GTM_gtm_cls3.MCS3_CH4_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH4_R2 GTM_gtm_cls3.MCS3_CH4_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH4_R3 GTM_gtm_cls3.MCS3_CH4_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH4_R4 GTM_gtm_cls3.MCS3_CH4_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH4_R5 GTM_gtm_cls3.MCS3_CH4_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH4_R6 GTM_gtm_cls3.MCS3_CH4_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH4_R7 GTM_gtm_cls3.MCS3_CH4_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH4_CTRL GTM_gtm_cls3.MCS3_CH4_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls3_MCS3_CH4_ACB GTM_gtm_cls3.MCS3_CH4_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls3_MCS3_CH4_MHB GTM_gtm_cls3.MCS3_CH4_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls3_MCS3_CH4_PC GTM_gtm_cls3.MCS3_CH4_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls3_MCS3_CH4_IRQ_NOTIFY GTM_gtm_cls3.MCS3_CH4_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls3_MCS3_CH4_IRQ_EN GTM_gtm_cls3.MCS3_CH4_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH4_IRQ_FORCINT GTM_gtm_cls3.MCS3_CH4_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls3_MCS3_CH4_IRQ_MODE GTM_gtm_cls3.MCS3_CH4_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls3_MCS3_CH4_EIRQ_EN GTM_gtm_cls3.MCS3_CH4_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH5_R0 GTM_gtm_cls3.MCS3_CH5_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH5_R1 GTM_gtm_cls3.MCS3_CH5_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH5_R2 GTM_gtm_cls3.MCS3_CH5_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH5_R3 GTM_gtm_cls3.MCS3_CH5_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH5_R4 GTM_gtm_cls3.MCS3_CH5_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH5_R5 GTM_gtm_cls3.MCS3_CH5_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH5_R6 GTM_gtm_cls3.MCS3_CH5_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH5_R7 GTM_gtm_cls3.MCS3_CH5_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH5_CTRL GTM_gtm_cls3.MCS3_CH5_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls3_MCS3_CH5_ACB GTM_gtm_cls3.MCS3_CH5_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls3_MCS3_CH5_MHB GTM_gtm_cls3.MCS3_CH5_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls3_MCS3_CH5_PC GTM_gtm_cls3.MCS3_CH5_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls3_MCS3_CH5_IRQ_NOTIFY GTM_gtm_cls3.MCS3_CH5_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls3_MCS3_CH5_IRQ_EN GTM_gtm_cls3.MCS3_CH5_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH5_IRQ_FORCINT GTM_gtm_cls3.MCS3_CH5_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls3_MCS3_CH5_IRQ_MODE GTM_gtm_cls3.MCS3_CH5_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls3_MCS3_CH5_EIRQ_EN GTM_gtm_cls3.MCS3_CH5_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH6_R0 GTM_gtm_cls3.MCS3_CH6_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH6_R1 GTM_gtm_cls3.MCS3_CH6_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH6_R2 GTM_gtm_cls3.MCS3_CH6_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH6_R3 GTM_gtm_cls3.MCS3_CH6_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH6_R4 GTM_gtm_cls3.MCS3_CH6_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH6_R5 GTM_gtm_cls3.MCS3_CH6_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH6_R6 GTM_gtm_cls3.MCS3_CH6_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH6_R7 GTM_gtm_cls3.MCS3_CH6_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH6_CTRL GTM_gtm_cls3.MCS3_CH6_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls3_MCS3_CH6_ACB GTM_gtm_cls3.MCS3_CH6_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls3_MCS3_CH6_MHB GTM_gtm_cls3.MCS3_CH6_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls3_MCS3_CH6_PC GTM_gtm_cls3.MCS3_CH6_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls3_MCS3_CH6_IRQ_NOTIFY GTM_gtm_cls3.MCS3_CH6_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls3_MCS3_CH6_IRQ_EN GTM_gtm_cls3.MCS3_CH6_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH6_IRQ_FORCINT GTM_gtm_cls3.MCS3_CH6_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls3_MCS3_CH6_IRQ_MODE GTM_gtm_cls3.MCS3_CH6_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls3_MCS3_CH6_EIRQ_EN GTM_gtm_cls3.MCS3_CH6_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH7_R0 GTM_gtm_cls3.MCS3_CH7_R0.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH7_R1 GTM_gtm_cls3.MCS3_CH7_R1.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH7_R2 GTM_gtm_cls3.MCS3_CH7_R2.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH7_R3 GTM_gtm_cls3.MCS3_CH7_R3.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH7_R4 GTM_gtm_cls3.MCS3_CH7_R4.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH7_R5 GTM_gtm_cls3.MCS3_CH7_R5.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH7_R6 GTM_gtm_cls3.MCS3_CH7_R6.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH7_R7 GTM_gtm_cls3.MCS3_CH7_R7.R /* MCS[i] channel x general purpose register [y] */
#define GTM_gtm_cls3_MCS3_CH7_CTRL GTM_gtm_cls3.MCS3_CH7_CTRL.R /* MCS[i] channel x control register */
#define GTM_gtm_cls3_MCS3_CH7_ACB GTM_gtm_cls3.MCS3_CH7_ACB.R /* MCS[i] channel x ARU control Bit register */
#define GTM_gtm_cls3_MCS3_CH7_MHB GTM_gtm_cls3.MCS3_CH7_MHB.R /* MCS[i] channel x memory high byte register */
#define GTM_gtm_cls3_MCS3_CH7_PC GTM_gtm_cls3.MCS3_CH7_PC.R /* MCS[i] channel x program counter register */
#define GTM_gtm_cls3_MCS3_CH7_IRQ_NOTIFY GTM_gtm_cls3.MCS3_CH7_IRQ_NOTIFY.R /* MCS[i] channel x interrupt notification register */
#define GTM_gtm_cls3_MCS3_CH7_IRQ_EN GTM_gtm_cls3.MCS3_CH7_IRQ_EN.R /* MCS[i] channel x interrupt enable register */
#define GTM_gtm_cls3_MCS3_CH7_IRQ_FORCINT GTM_gtm_cls3.MCS3_CH7_IRQ_FORCINT.R /* MCS[i] channel x force interrupt register */
#define GTM_gtm_cls3_MCS3_CH7_IRQ_MODE GTM_gtm_cls3.MCS3_CH7_IRQ_MODE.R /* MCS[i] channel x IRQ mode configuration register */
#define GTM_gtm_cls3_MCS3_CH7_EIRQ_EN GTM_gtm_cls3.MCS3_CH7_EIRQ_EN.R /* MCS[i] channel x error interrupt enable register */
#define GTM_gtm_cls3_MCS3_CTRG GTM_gtm_cls3.MCS3_CTRG.R    /* MCS[i] clear trigger control register */
#define GTM_gtm_cls3_MCS3_STRG GTM_gtm_cls3.MCS3_STRG.R    /* MCS[i] set trigger control register */
#define GTM_gtm_cls3_MCS3_CTRL_STAT GTM_gtm_cls3.MCS3_CTRL_STAT.R /* MCS[i] control and status register */
#define GTM_gtm_cls3_MCS3_RESET GTM_gtm_cls3.MCS3_RESET.R  /* MCS[i] reset register */
#define GTM_gtm_cls3_MCS3_CAT GTM_gtm_cls3.MCS3_CAT.R      /* MCS[i] cancel ARU transfer instruction */
#define GTM_gtm_cls3_MCS3_CWT GTM_gtm_cls3.MCS3_CWT.R      /* MCS[i] cancel waiting instruction */
#define GTM_gtm_cls3_MCS3_ERR GTM_gtm_cls3.MCS3_ERR.R      /* MCS[i] error register */
#define GTM_gtm_cls3_MCS3_REG_PROT GTM_gtm_cls3.MCS3_REG_PROT.R /* MCS[i] write protection register */
#define GTM_gtm_cls3_MCS3_SINT_IRQ_NOTIFY GTM_gtm_cls3.MCS3_SINT_IRQ_NOTIFY.R /* MCS[i] shared interrupt notification register */
#define GTM_gtm_cls3_MCS3_SINT_IRQ_EN GTM_gtm_cls3.MCS3_SINT_IRQ_EN.R /* MCS[i] shared interrupt enable register */
#define GTM_gtm_cls3_MCS3_SINT_IRQ_FORCINT GTM_gtm_cls3.MCS3_SINT_IRQ_FORCINT.R /* MCS[i] force shared interrupt register */
#define GTM_gtm_cls3_MCS3_SINT_IRQ_MODE GTM_gtm_cls3.MCS3_SINT_IRQ_MODE.R /* MCS[i] shared interrupt mode configuration register */
#define GTM_gtm_cls3_MCS3_HBP0_CTRL GTM_gtm_cls3.MCS3_HBP0_CTRL.R /* MCS[i] hardware break point h control register */
#define GTM_gtm_cls3_MCS3_HBP0_PATTERN GTM_gtm_cls3.MCS3_HBP0_PATTERN.R /* MCS[i] hardware break point pattern register */
#define GTM_gtm_cls3_MCS3_HBP0_STATUS GTM_gtm_cls3.MCS3_HBP0_STATUS.R /* MCS[i] hardware break point status register */
#define GTM_gtm_cls3_MCS3_HBP0_IRQ_NOTIFY GTM_gtm_cls3.MCS3_HBP0_IRQ_NOTIFY.R /* MCS[i] hardware break point interrupt notification register */
#define GTM_gtm_cls3_MCS3_HBP0_IRQ_EN GTM_gtm_cls3.MCS3_HBP0_IRQ_EN.R /* MCS[i] hardware break point interrupt enable register */
#define GTM_gtm_cls3_MCS3_HBP0_IRQ_FORCINT GTM_gtm_cls3.MCS3_HBP0_IRQ_FORCINT.R /* MCS[i] force hardware break point interrupt register */
#define GTM_gtm_cls3_MCS3_HBP0_IRQ_MODE GTM_gtm_cls3.MCS3_HBP0_IRQ_MODE.R /* MCS[i] break point h interrupt mode configuration register */
#define GTM_gtm_cls3_MCS3_HBP1_CTRL GTM_gtm_cls3.MCS3_HBP1_CTRL.R /* MCS[i] hardware break point h control register */
#define GTM_gtm_cls3_MCS3_HBP1_PATTERN GTM_gtm_cls3.MCS3_HBP1_PATTERN.R /* MCS[i] hardware break point pattern register */
#define GTM_gtm_cls3_MCS3_HBP1_STATUS GTM_gtm_cls3.MCS3_HBP1_STATUS.R /* MCS[i] hardware break point status register */
#define GTM_gtm_cls3_MCS3_HBP1_IRQ_NOTIFY GTM_gtm_cls3.MCS3_HBP1_IRQ_NOTIFY.R /* MCS[i] hardware break point interrupt notification register */
#define GTM_gtm_cls3_MCS3_HBP1_IRQ_EN GTM_gtm_cls3.MCS3_HBP1_IRQ_EN.R /* MCS[i] hardware break point interrupt enable register */
#define GTM_gtm_cls3_MCS3_HBP1_IRQ_FORCINT GTM_gtm_cls3.MCS3_HBP1_IRQ_FORCINT.R /* MCS[i] force hardware break point interrupt register */
#define GTM_gtm_cls3_MCS3_HBP1_IRQ_MODE GTM_gtm_cls3.MCS3_HBP1_IRQ_MODE.R /* MCS[i] break point h interrupt mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH0_CTRL GTM_gtm_cls3.TIO3_G0_CH0_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH0_IRQ_NOTIFY GTM_gtm_cls3.TIO3_G0_CH0_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls3_TIO3_G0_CH0_IRQ_EN GTM_gtm_cls3.TIO3_G0_CH0_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls3_TIO3_G0_CH0_IRQ_FORCINT GTM_gtm_cls3.TIO3_G0_CH0_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls3_TIO3_G0_CH0_IRQ_MODE GTM_gtm_cls3.TIO3_G0_CH0_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH0_CTRL2 GTM_gtm_cls3.TIO3_G0_CH0_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH0_SINST GTM_gtm_cls3.TIO3_G0_CH0_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH0_SCMD GTM_gtm_cls3.TIO3_G0_CH0_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH0_SOP GTM_gtm_cls3.TIO3_G0_CH0_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls3_TIO3_G0_CH0_OINST GTM_gtm_cls3.TIO3_G0_CH0_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH0_OCMD GTM_gtm_cls3.TIO3_G0_CH0_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH0_OOP GTM_gtm_cls3.TIO3_G0_CH0_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls3_TIO3_G0_CH0_SHIFTCNT GTM_gtm_cls3.TIO3_G0_CH0_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls3_TIO3_G0_CH1_CTRL GTM_gtm_cls3.TIO3_G0_CH1_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH1_IRQ_NOTIFY GTM_gtm_cls3.TIO3_G0_CH1_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls3_TIO3_G0_CH1_IRQ_EN GTM_gtm_cls3.TIO3_G0_CH1_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls3_TIO3_G0_CH1_IRQ_FORCINT GTM_gtm_cls3.TIO3_G0_CH1_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls3_TIO3_G0_CH1_IRQ_MODE GTM_gtm_cls3.TIO3_G0_CH1_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH1_CTRL2 GTM_gtm_cls3.TIO3_G0_CH1_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH1_SINST GTM_gtm_cls3.TIO3_G0_CH1_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH1_SCMD GTM_gtm_cls3.TIO3_G0_CH1_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH1_SOP GTM_gtm_cls3.TIO3_G0_CH1_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls3_TIO3_G0_CH1_OINST GTM_gtm_cls3.TIO3_G0_CH1_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH1_OCMD GTM_gtm_cls3.TIO3_G0_CH1_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH1_OOP GTM_gtm_cls3.TIO3_G0_CH1_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls3_TIO3_G0_CH1_SHIFTCNT GTM_gtm_cls3.TIO3_G0_CH1_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls3_TIO3_G0_CH2_CTRL GTM_gtm_cls3.TIO3_G0_CH2_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH2_IRQ_NOTIFY GTM_gtm_cls3.TIO3_G0_CH2_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls3_TIO3_G0_CH2_IRQ_EN GTM_gtm_cls3.TIO3_G0_CH2_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls3_TIO3_G0_CH2_IRQ_FORCINT GTM_gtm_cls3.TIO3_G0_CH2_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls3_TIO3_G0_CH2_IRQ_MODE GTM_gtm_cls3.TIO3_G0_CH2_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH2_CTRL2 GTM_gtm_cls3.TIO3_G0_CH2_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH2_SINST GTM_gtm_cls3.TIO3_G0_CH2_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH2_SCMD GTM_gtm_cls3.TIO3_G0_CH2_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH2_SOP GTM_gtm_cls3.TIO3_G0_CH2_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls3_TIO3_G0_CH2_OINST GTM_gtm_cls3.TIO3_G0_CH2_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH2_OCMD GTM_gtm_cls3.TIO3_G0_CH2_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH2_OOP GTM_gtm_cls3.TIO3_G0_CH2_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls3_TIO3_G0_CH2_SHIFTCNT GTM_gtm_cls3.TIO3_G0_CH2_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls3_TIO3_G0_CH3_CTRL GTM_gtm_cls3.TIO3_G0_CH3_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH3_IRQ_NOTIFY GTM_gtm_cls3.TIO3_G0_CH3_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls3_TIO3_G0_CH3_IRQ_EN GTM_gtm_cls3.TIO3_G0_CH3_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls3_TIO3_G0_CH3_IRQ_FORCINT GTM_gtm_cls3.TIO3_G0_CH3_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls3_TIO3_G0_CH3_IRQ_MODE GTM_gtm_cls3.TIO3_G0_CH3_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH3_CTRL2 GTM_gtm_cls3.TIO3_G0_CH3_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH3_SINST GTM_gtm_cls3.TIO3_G0_CH3_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH3_SCMD GTM_gtm_cls3.TIO3_G0_CH3_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH3_SOP GTM_gtm_cls3.TIO3_G0_CH3_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls3_TIO3_G0_CH3_OINST GTM_gtm_cls3.TIO3_G0_CH3_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH3_OCMD GTM_gtm_cls3.TIO3_G0_CH3_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH3_OOP GTM_gtm_cls3.TIO3_G0_CH3_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls3_TIO3_G0_CH3_SHIFTCNT GTM_gtm_cls3.TIO3_G0_CH3_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls3_TIO3_G0_CH4_CTRL GTM_gtm_cls3.TIO3_G0_CH4_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH4_IRQ_NOTIFY GTM_gtm_cls3.TIO3_G0_CH4_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls3_TIO3_G0_CH4_IRQ_EN GTM_gtm_cls3.TIO3_G0_CH4_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls3_TIO3_G0_CH4_IRQ_FORCINT GTM_gtm_cls3.TIO3_G0_CH4_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls3_TIO3_G0_CH4_IRQ_MODE GTM_gtm_cls3.TIO3_G0_CH4_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH4_CTRL2 GTM_gtm_cls3.TIO3_G0_CH4_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH4_SINST GTM_gtm_cls3.TIO3_G0_CH4_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH4_SCMD GTM_gtm_cls3.TIO3_G0_CH4_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH4_SOP GTM_gtm_cls3.TIO3_G0_CH4_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls3_TIO3_G0_CH4_OINST GTM_gtm_cls3.TIO3_G0_CH4_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH4_OCMD GTM_gtm_cls3.TIO3_G0_CH4_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH4_OOP GTM_gtm_cls3.TIO3_G0_CH4_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls3_TIO3_G0_CH4_SHIFTCNT GTM_gtm_cls3.TIO3_G0_CH4_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls3_TIO3_G0_CH5_CTRL GTM_gtm_cls3.TIO3_G0_CH5_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH5_IRQ_NOTIFY GTM_gtm_cls3.TIO3_G0_CH5_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls3_TIO3_G0_CH5_IRQ_EN GTM_gtm_cls3.TIO3_G0_CH5_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls3_TIO3_G0_CH5_IRQ_FORCINT GTM_gtm_cls3.TIO3_G0_CH5_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls3_TIO3_G0_CH5_IRQ_MODE GTM_gtm_cls3.TIO3_G0_CH5_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH5_CTRL2 GTM_gtm_cls3.TIO3_G0_CH5_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH5_SINST GTM_gtm_cls3.TIO3_G0_CH5_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH5_SCMD GTM_gtm_cls3.TIO3_G0_CH5_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH5_SOP GTM_gtm_cls3.TIO3_G0_CH5_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls3_TIO3_G0_CH5_OINST GTM_gtm_cls3.TIO3_G0_CH5_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH5_OCMD GTM_gtm_cls3.TIO3_G0_CH5_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH5_OOP GTM_gtm_cls3.TIO3_G0_CH5_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls3_TIO3_G0_CH5_SHIFTCNT GTM_gtm_cls3.TIO3_G0_CH5_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls3_TIO3_G0_CH6_CTRL GTM_gtm_cls3.TIO3_G0_CH6_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH6_IRQ_NOTIFY GTM_gtm_cls3.TIO3_G0_CH6_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls3_TIO3_G0_CH6_IRQ_EN GTM_gtm_cls3.TIO3_G0_CH6_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls3_TIO3_G0_CH6_IRQ_FORCINT GTM_gtm_cls3.TIO3_G0_CH6_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls3_TIO3_G0_CH6_IRQ_MODE GTM_gtm_cls3.TIO3_G0_CH6_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH6_CTRL2 GTM_gtm_cls3.TIO3_G0_CH6_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH6_SINST GTM_gtm_cls3.TIO3_G0_CH6_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH6_SCMD GTM_gtm_cls3.TIO3_G0_CH6_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH6_SOP GTM_gtm_cls3.TIO3_G0_CH6_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls3_TIO3_G0_CH6_OINST GTM_gtm_cls3.TIO3_G0_CH6_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH6_OCMD GTM_gtm_cls3.TIO3_G0_CH6_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH6_OOP GTM_gtm_cls3.TIO3_G0_CH6_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls3_TIO3_G0_CH6_SHIFTCNT GTM_gtm_cls3.TIO3_G0_CH6_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls3_TIO3_G0_CH7_CTRL GTM_gtm_cls3.TIO3_G0_CH7_CTRL.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH7_IRQ_NOTIFY GTM_gtm_cls3.TIO3_G0_CH7_IRQ_NOTIFY.R /* TIO[i] channel [c] interrupt notification register */
#define GTM_gtm_cls3_TIO3_G0_CH7_IRQ_EN GTM_gtm_cls3.TIO3_G0_CH7_IRQ_EN.R /* TIO[i] channel [c] interrupt enable register */
#define GTM_gtm_cls3_TIO3_G0_CH7_IRQ_FORCINT GTM_gtm_cls3.TIO3_G0_CH7_IRQ_FORCINT.R /* TIO[i] channel [c] force interrupt register */
#define GTM_gtm_cls3_TIO3_G0_CH7_IRQ_MODE GTM_gtm_cls3.TIO3_G0_CH7_IRQ_MODE.R /* TIO[i] channel [c] IRQ mode configuration register */
#define GTM_gtm_cls3_TIO3_G0_CH7_CTRL2 GTM_gtm_cls3.TIO3_G0_CH7_CTRL2.R /* TIO[i] group [g] channel [c] control register */
#define GTM_gtm_cls3_TIO3_G0_CH7_SINST GTM_gtm_cls3.TIO3_G0_CH7_SINST.R /* TIO[i] channel [c] resource S instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH7_SCMD GTM_gtm_cls3.TIO3_G0_CH7_SCMD.R /* TIO[i] channel [c] resource S command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- */
#define GTM_gtm_cls3_TIO3_G0_CH7_SOP GTM_gtm_cls3.TIO3_G0_CH7_SOP.R /* TIO[i] channel [c] resource S operand register TIO[i]_G[g]_CH[c]_CTRL.PL_S_MODE=0b0- or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b10- or TIO[i]_G[g]_OP_USAGE.MODE[c]=0b1-0 ) */
#define GTM_gtm_cls3_TIO3_G0_CH7_OINST GTM_gtm_cls3.TIO3_G0_CH7_OINST.R /* TIO[i] channel [c] resource O instruction register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE =0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH7_OCMD GTM_gtm_cls3.TIO3_G0_CH7_OCMD.R /* TIO[i] channel [c] resource O command register (buffer operation) TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b00- */
#define GTM_gtm_cls3_TIO3_G0_CH7_OOP GTM_gtm_cls3.TIO3_G0_CH7_OOP.R /* TIO[i] channel [c] resource O operand register !(TIO[i]_G[g]_CH[c]_CTRL.PL_O_MODE=0b1--) or (TIO[i]_G[g]_OP_USAGE.MODE[c]=0b11-) */
#define GTM_gtm_cls3_TIO3_G0_CH7_SHIFTCNT GTM_gtm_cls3.TIO3_G0_CH7_SHIFTCNT.R /* TIO[i] channel [c] resource shift count register */
#define GTM_gtm_cls3_TIO3_G0_ISEL0_CTRL1 GTM_gtm_cls3.TIO3_G0_ISEL0_CTRL1.R /* TIO[i] input selection register 1 */
#define GTM_gtm_cls3_TIO3_G0_ISEL0_CTRL2 GTM_gtm_cls3.TIO3_G0_ISEL0_CTRL2.R /* TIO[i] input selection register 2 */
#define GTM_gtm_cls3_TIO3_G0_ISEL1_CTRL1 GTM_gtm_cls3.TIO3_G0_ISEL1_CTRL1.R /* TIO[i] input selection register 1 */
#define GTM_gtm_cls3_TIO3_G0_ISEL1_CTRL2 GTM_gtm_cls3.TIO3_G0_ISEL1_CTRL2.R /* TIO[i] input selection register 2 */
#define GTM_gtm_cls3_TIO3_G0_OP_USAGE GTM_gtm_cls3.TIO3_G0_OP_USAGE.R /* TIO[i] operand usage selection register */
#define GTM_gtm_cls3_TIO3_S  GTM_gtm_cls3.TIO3_S.R         /* TIO[i] signal sampling register */
#define GTM_gtm_cls3_TIO3_O  GTM_gtm_cls3.TIO3_O.R         /* TIO[i] output register */
#define GTM_gtm_cls3_TIO3_ENDIS GTM_gtm_cls3.TIO3_ENDIS.R  /* TIO[i] enable/disable register */
#define GTM_gtm_cls3_TIO3_INVERT GTM_gtm_cls3.TIO3_INVERT.R /* TIO[i] signal invert register */
#define GTM_gtm_cls3_TIO3_INPUT_MODE GTM_gtm_cls3.TIO3_INPUT_MODE.R /* TIO[i] input mode register */
#define GTM_gtm_cls3_TIO3_CYCLIC_MODE GTM_gtm_cls3.TIO3_CYCLIC_MODE.R /* TIO[i] cyclic mode register */
#define GTM_gtm_cls3_TIO3_TRIG_OUT_GATE_EN GTM_gtm_cls3.TIO3_TRIG_OUT_GATE_EN.R /* TIO[i] enable Trigger Output, output gating register */
#define GTM_gtm_cls3_TIO3_PLTRIG_OUT_GATE_EN GTM_gtm_cls3.TIO3_PLTRIG_OUT_GATE_EN.R /* TIO[i] enable PL_TRIG_OUT output gating register */
#define GTM_gtm_cls3_TIO3_CS GTM_gtm_cls3.TIO3_CS.R        /* TIO[i] clear signal sampling register */
#define GTM_gtm_cls3_TIO3_CO GTM_gtm_cls3.TIO3_CO.R        /* TIO[i] clear output register */
#define GTM_gtm_cls3_TIO3_CENDIS GTM_gtm_cls3.TIO3_CENDIS.R /* TIO[i] disable register */
#define GTM_gtm_cls3_TIO3_CINVERT GTM_gtm_cls3.TIO3_CINVERT.R /* TIO[i] clear signal invert register */
#define GTM_gtm_cls3_TIO3_CINPUT_MODE GTM_gtm_cls3.TIO3_CINPUT_MODE.R /* TIO[i] disable input mode register */
#define GTM_gtm_cls3_TIO3_CCYCLIC_MODE GTM_gtm_cls3.TIO3_CCYCLIC_MODE.R /* TIO[i] disable cyclic mode register */
#define GTM_gtm_cls3_TIO3_CTRIG_OUT_GATE_EN GTM_gtm_cls3.TIO3_CTRIG_OUT_GATE_EN.R /* TIO[i] clear Trigger Output, output gating register */
#define GTM_gtm_cls3_TIO3_CPLTRIG_OUT_GATE_EN GTM_gtm_cls3.TIO3_CPLTRIG_OUT_GATE_EN.R /* TIO[i] clear PL_TRIG_OUT output gating register */
#define GTM_gtm_cls3_TIO3_SS GTM_gtm_cls3.TIO3_SS.R        /* TIO[i] set signal sampling register */
#define GTM_gtm_cls3_TIO3_SO GTM_gtm_cls3.TIO3_SO.R        /* TIO[i] set output register */
#define GTM_gtm_cls3_TIO3_SENDIS GTM_gtm_cls3.TIO3_SENDIS.R /* TIO[i] enable register */
#define GTM_gtm_cls3_TIO3_SINVERT GTM_gtm_cls3.TIO3_SINVERT.R /* TIO[i] set signal invert register */
#define GTM_gtm_cls3_TIO3_SINPUT_MODE GTM_gtm_cls3.TIO3_SINPUT_MODE.R /* TIO[i] enable input mode register */
#define GTM_gtm_cls3_TIO3_SCYCLIC_MODE GTM_gtm_cls3.TIO3_SCYCLIC_MODE.R /* TIO[i] enable cyclic mode register */
#define GTM_gtm_cls3_TIO3_STRIG_OUT_GATE_EN GTM_gtm_cls3.TIO3_STRIG_OUT_GATE_EN.R /* TIO[i] set Trigger Output, output gating register */
#define GTM_gtm_cls3_TIO3_SPLTRIG_OUT_GATE_EN GTM_gtm_cls3.TIO3_SPLTRIG_OUT_GATE_EN.R /* TIO[i] set PL_TRIG_OUT output gating register */
#define GTM_gtm_cls3_TIO3_IS GTM_gtm_cls3.TIO3_IS.R        /* TIO[i] invert signal sampling register */
#define GTM_gtm_cls3_TIO3_IO GTM_gtm_cls3.TIO3_IO.R        /* TIO[i] invert output register */
#define GTM_gtm_cls3_TIO3_IENDIS GTM_gtm_cls3.TIO3_IENDIS.R /* TIO[i] toggle enable/disable register */
#define GTM_gtm_cls3_TIO3_IINVERT GTM_gtm_cls3.TIO3_IINVERT.R /* TIO[i] toggle signal invert register */
#define GTM_gtm_cls3_TIO3_IINPUT_MODE GTM_gtm_cls3.TIO3_IINPUT_MODE.R /* TIO[i] enable input mode register */
#define GTM_gtm_cls3_TIO3_ICYCLIC_MODE GTM_gtm_cls3.TIO3_ICYCLIC_MODE.R /* TIO[i] enable cyclic mode register */
#define GTM_gtm_cls3_TIO3_FUPD GTM_gtm_cls3.TIO3_FUPD.R    /* TIO[i] force update register */
#define GTM_gtm_cls3_TIO3_HW_CONF GTM_gtm_cls3.TIO3_HW_CONF.R /* TIO[i] configuration register */
#define GTM_gtm_cls3_TIO3_RSEL_CTRL1 GTM_gtm_cls3.TIO3_RSEL_CTRL1.R /* TIO[i] resource selection control register 1 */
#define GTM_gtm_cls3_TIO3_RSEL_CTRL2 GTM_gtm_cls3.TIO3_RSEL_CTRL2.R /* TIO[i] resource selection control register 2 */
#define GTM_gtm_cls3_TIO3_PL_SWRST GTM_gtm_cls3.TIO3_PL_SWRST.R /* TIO[i] software reset for TIO Plus functionality */
#define GTM_gtm_cls3_CCM3_ARP0_CTRL GTM_gtm_cls3.CCM3_ARP0_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP0_PROT GTM_gtm_cls3.CCM3_ARP0_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP1_CTRL GTM_gtm_cls3.CCM3_ARP1_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP1_PROT GTM_gtm_cls3.CCM3_ARP1_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP2_CTRL GTM_gtm_cls3.CCM3_ARP2_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP2_PROT GTM_gtm_cls3.CCM3_ARP2_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP3_CTRL GTM_gtm_cls3.CCM3_ARP3_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP3_PROT GTM_gtm_cls3.CCM3_ARP3_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP4_CTRL GTM_gtm_cls3.CCM3_ARP4_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP4_PROT GTM_gtm_cls3.CCM3_ARP4_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP5_CTRL GTM_gtm_cls3.CCM3_ARP5_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP5_PROT GTM_gtm_cls3.CCM3_ARP5_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP6_CTRL GTM_gtm_cls3.CCM3_ARP6_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP6_PROT GTM_gtm_cls3.CCM3_ARP6_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP7_CTRL GTM_gtm_cls3.CCM3_ARP7_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP7_PROT GTM_gtm_cls3.CCM3_ARP7_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP8_CTRL GTM_gtm_cls3.CCM3_ARP8_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP8_PROT GTM_gtm_cls3.CCM3_ARP8_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_ARP9_CTRL GTM_gtm_cls3.CCM3_ARP9_CTRL.R /* CCM[i] Address Range Protector [a] Control Register */
#define GTM_gtm_cls3_CCM3_ARP9_PROT GTM_gtm_cls3.CCM3_ARP9_PROT.R /* CCM[i] Address Range Protector [a] Protection Register */
#define GTM_gtm_cls3_CCM3_TIO_G0_OUT GTM_gtm_cls3.CCM3_TIO_G0_OUT.R /* CCM[i] TIO Group 0,1 Output Register */
#define GTM_gtm_cls3_CCM3_HW_CONF2 GTM_gtm_cls3.CCM3_HW_CONF2.R /* CCM[i] 2. Hardware Configuration Register */
#define GTM_gtm_cls3_CCM3_AEIM_STA GTM_gtm_cls3.CCM3_AEIM_STA.R /* CCM[i] MCS Bus Master Status Register */
#define GTM_gtm_cls3_CCM3_HW_CONF GTM_gtm_cls3.CCM3_HW_CONF.R /* CCM[i] Hardware Configuration Register */
#define GTM_gtm_cls3_CCM3_ATOM_OUT GTM_gtm_cls3.CCM3_ATOM_OUT.R /* CCM[i] ATOM Output Register */
#define GTM_gtm_cls3_CCM3_CMU_CLK_CFG GTM_gtm_cls3.CCM3_CMU_CLK_CFG.R /* CCM[i] CMU Clock Configuration Register */
#define GTM_gtm_cls3_CCM3_CFG GTM_gtm_cls3.CCM3_CFG.R      /* CCM[i] Configuration Register */
#define GTM_gtm_cls3_CCM3_PROT GTM_gtm_cls3.CCM3_PROT.R    /* CCM[i] Protection Register */
#define GTM_gtm_cls3_CDTM3_DTM4_CTRL GTM_gtm_cls3.CDTM3_DTM4_CTRL.R /* CDTM[i]_DTM[d] global configuration and control register */
#define GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL1 GTM_gtm_cls3.CDTM3_DTM4_CH_CTRL1.R /* CDTM[i]_DTM[d] channel control register 1 */
#define GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL2 GTM_gtm_cls3.CDTM3_DTM4_CH_CTRL2.R /* CDTM[i]_DTM[d] channel control register 2 */
#define GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL2_SR GTM_gtm_cls3.CDTM3_DTM4_CH_CTRL2_SR.R /* CDTM[i] DTM[j] channel control register 2 shadow */
#define GTM_gtm_cls3_CDTM3_DTM4_PS_CTRL GTM_gtm_cls3.CDTM3_DTM4_PS_CTRL.R /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
#define GTM_gtm_cls3_CDTM3_DTM4_CH0_DTV GTM_gtm_cls3.CDTM3_DTM4_CH_DTV[0].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls3_CDTM3_DTM4_CH1_DTV GTM_gtm_cls3.CDTM3_DTM4_CH_DTV[1].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls3_CDTM3_DTM4_CH2_DTV GTM_gtm_cls3.CDTM3_DTM4_CH_DTV[2].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls3_CDTM3_DTM4_CH3_DTV GTM_gtm_cls3.CDTM3_DTM4_CH_DTV[3].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls3_CDTM3_DTM4_CH_SR GTM_gtm_cls3.CDTM3_DTM4_CH_SR.R /* CDTM[i]_DTM[d] channel shadow register */
#define GTM_gtm_cls3_CDTM3_DTM4_CH_CTRL3 GTM_gtm_cls3.CDTM3_DTM4_CH_CTRL3.R /* CDTM[i]_DTM[d] channel control register 3 */
#define GTM_gtm_cls3_CDTM3_DTM4_CTRL2 GTM_gtm_cls3.CDTM3_DTM4_CTRL2.R /* CDTM[i]_DTM[d] global configuration and control register 2 */
#define GTM_gtm_cls3_CDTM3_DTM4_CH0_DTV_SR GTM_gtm_cls3.CDTM3_DTM4_CH0_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls3_CDTM3_DTM4_CH1_DTV_SR GTM_gtm_cls3.CDTM3_DTM4_CH1_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls3_CDTM3_DTM4_CH2_DTV_SR GTM_gtm_cls3.CDTM3_DTM4_CH2_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls3_CDTM3_DTM4_CH3_DTV_SR GTM_gtm_cls3.CDTM3_DTM4_CH3_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls3_CDTM3_DTM5_CTRL GTM_gtm_cls3.CDTM3_DTM5_CTRL.R /* CDTM[i]_DTM[d] global configuration and control register */
#define GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL1 GTM_gtm_cls3.CDTM3_DTM5_CH_CTRL1.R /* CDTM[i]_DTM[d] channel control register 1 */
#define GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL2 GTM_gtm_cls3.CDTM3_DTM5_CH_CTRL2.R /* CDTM[i]_DTM[d] channel control register 2 */
#define GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL2_SR GTM_gtm_cls3.CDTM3_DTM5_CH_CTRL2_SR.R /* CDTM[i] DTM[j] channel control register 2 shadow */
#define GTM_gtm_cls3_CDTM3_DTM5_PS_CTRL GTM_gtm_cls3.CDTM3_DTM5_PS_CTRL.R /* CDTM[i]_DTM[d] phase shift unit configuration and control register */
#define GTM_gtm_cls3_CDTM3_DTM5_CH0_DTV GTM_gtm_cls3.CDTM3_DTM5_CH_DTV[0].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls3_CDTM3_DTM5_CH1_DTV GTM_gtm_cls3.CDTM3_DTM5_CH_DTV[1].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls3_CDTM3_DTM5_CH2_DTV GTM_gtm_cls3.CDTM3_DTM5_CH_DTV[2].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls3_CDTM3_DTM5_CH3_DTV GTM_gtm_cls3.CDTM3_DTM5_CH_DTV[3].R /* CDTM[i]_DTM[d] channel [x] dead time reload values */
#define GTM_gtm_cls3_CDTM3_DTM5_CH_SR GTM_gtm_cls3.CDTM3_DTM5_CH_SR.R /* CDTM[i]_DTM[d] channel shadow register */
#define GTM_gtm_cls3_CDTM3_DTM5_CH_CTRL3 GTM_gtm_cls3.CDTM3_DTM5_CH_CTRL3.R /* CDTM[i]_DTM[d] channel control register 3 */
#define GTM_gtm_cls3_CDTM3_DTM5_CTRL2 GTM_gtm_cls3.CDTM3_DTM5_CTRL2.R /* CDTM[i]_DTM[d] global configuration and control register 2 */
#define GTM_gtm_cls3_CDTM3_DTM5_CH0_DTV_SR GTM_gtm_cls3.CDTM3_DTM5_CH0_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls3_CDTM3_DTM5_CH1_DTV_SR GTM_gtm_cls3.CDTM3_DTM5_CH1_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls3_CDTM3_DTM5_CH2_DTV_SR GTM_gtm_cls3.CDTM3_DTM5_CH2_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls3_CDTM3_DTM5_CH3_DTV_SR GTM_gtm_cls3.CDTM3_DTM5_CH3_DTV_SR.R /* CDTM[i]_DTM[d] channel [x] dead time shadow values */
#define GTM_gtm_cls3_AXIM3_FREE GTM_gtm_cls3.AXIM3_FREE.R  /* AXIM[i] slot allocation status. */
#define GTM_gtm_cls3_AXIM3_REQUEST GTM_gtm_cls3.AXIM3_REQUEST.R /* AXIM[i] slot request (allocation). */
#define GTM_gtm_cls3_AXIM3_RELEASE GTM_gtm_cls3.AXIM3_RELEASE.R /* AXIM[i] slot release (de-allocation). */
#define GTM_gtm_cls3_AXIM3_SLOT0_ADDR_LOW GTM_gtm_cls3.AXIM3_SLOT0_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls3_AXIM3_SLOT0_DATA_LOW GTM_gtm_cls3.AXIM3_SLOT0_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls3_AXIM3_SLOT0_CFG1 GTM_gtm_cls3.AXIM3_SLOT0_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls3_AXIM3_SLOT0_CFG2 GTM_gtm_cls3.AXIM3_SLOT0_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls3_AXIM3_SLOT0_STATUS GTM_gtm_cls3.AXIM3_SLOT0_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls3_AXIM3_SLOT1_ADDR_LOW GTM_gtm_cls3.AXIM3_SLOT1_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls3_AXIM3_SLOT1_DATA_LOW GTM_gtm_cls3.AXIM3_SLOT1_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls3_AXIM3_SLOT1_CFG1 GTM_gtm_cls3.AXIM3_SLOT1_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls3_AXIM3_SLOT1_CFG2 GTM_gtm_cls3.AXIM3_SLOT1_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls3_AXIM3_SLOT1_STATUS GTM_gtm_cls3.AXIM3_SLOT1_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls3_AXIM3_SLOT2_ADDR_LOW GTM_gtm_cls3.AXIM3_SLOT2_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls3_AXIM3_SLOT2_DATA_LOW GTM_gtm_cls3.AXIM3_SLOT2_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls3_AXIM3_SLOT2_CFG1 GTM_gtm_cls3.AXIM3_SLOT2_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls3_AXIM3_SLOT2_CFG2 GTM_gtm_cls3.AXIM3_SLOT2_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls3_AXIM3_SLOT2_STATUS GTM_gtm_cls3.AXIM3_SLOT2_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls3_AXIM3_SLOT3_ADDR_LOW GTM_gtm_cls3.AXIM3_SLOT3_ADDR_LOW.R /* AXIM[i] slot[s] address bits 31:0 of AXI transaction. */
#define GTM_gtm_cls3_AXIM3_SLOT3_DATA_LOW GTM_gtm_cls3.AXIM3_SLOT3_DATA_LOW.R /* AXIM[i] slot[s] data bits 31:0 of AXI transaction. */
#define GTM_gtm_cls3_AXIM3_SLOT3_CFG1 GTM_gtm_cls3.AXIM3_SLOT3_CFG1.R /* AXIM[i] slot [s] configuration 1 */
#define GTM_gtm_cls3_AXIM3_SLOT3_CFG2 GTM_gtm_cls3.AXIM3_SLOT3_CFG2.R /* AXIM[i] slot[s] configuration 2 */
#define GTM_gtm_cls3_AXIM3_SLOT3_STATUS GTM_gtm_cls3.AXIM3_SLOT3_STATUS.R /* AXIM[i] slot[s] status */
#define GTM_gtm_cls3_MCS3_MEM0 GTM_gtm_cls3.MCS3_MEM[0].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1 GTM_gtm_cls3.MCS3_MEM[1].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2 GTM_gtm_cls3.MCS3_MEM[2].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3 GTM_gtm_cls3.MCS3_MEM[3].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM4 GTM_gtm_cls3.MCS3_MEM[4].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM5 GTM_gtm_cls3.MCS3_MEM[5].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM6 GTM_gtm_cls3.MCS3_MEM[6].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM7 GTM_gtm_cls3.MCS3_MEM[7].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM8 GTM_gtm_cls3.MCS3_MEM[8].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM9 GTM_gtm_cls3.MCS3_MEM[9].R  /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM10 GTM_gtm_cls3.MCS3_MEM[10].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM11 GTM_gtm_cls3.MCS3_MEM[11].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM12 GTM_gtm_cls3.MCS3_MEM[12].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM13 GTM_gtm_cls3.MCS3_MEM[13].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM14 GTM_gtm_cls3.MCS3_MEM[14].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM15 GTM_gtm_cls3.MCS3_MEM[15].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM16 GTM_gtm_cls3.MCS3_MEM[16].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM17 GTM_gtm_cls3.MCS3_MEM[17].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM18 GTM_gtm_cls3.MCS3_MEM[18].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM19 GTM_gtm_cls3.MCS3_MEM[19].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM20 GTM_gtm_cls3.MCS3_MEM[20].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM21 GTM_gtm_cls3.MCS3_MEM[21].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM22 GTM_gtm_cls3.MCS3_MEM[22].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM23 GTM_gtm_cls3.MCS3_MEM[23].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM24 GTM_gtm_cls3.MCS3_MEM[24].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM25 GTM_gtm_cls3.MCS3_MEM[25].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM26 GTM_gtm_cls3.MCS3_MEM[26].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM27 GTM_gtm_cls3.MCS3_MEM[27].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM28 GTM_gtm_cls3.MCS3_MEM[28].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM29 GTM_gtm_cls3.MCS3_MEM[29].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM30 GTM_gtm_cls3.MCS3_MEM[30].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM31 GTM_gtm_cls3.MCS3_MEM[31].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM32 GTM_gtm_cls3.MCS3_MEM[32].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM33 GTM_gtm_cls3.MCS3_MEM[33].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM34 GTM_gtm_cls3.MCS3_MEM[34].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM35 GTM_gtm_cls3.MCS3_MEM[35].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM36 GTM_gtm_cls3.MCS3_MEM[36].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM37 GTM_gtm_cls3.MCS3_MEM[37].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM38 GTM_gtm_cls3.MCS3_MEM[38].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM39 GTM_gtm_cls3.MCS3_MEM[39].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM40 GTM_gtm_cls3.MCS3_MEM[40].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM41 GTM_gtm_cls3.MCS3_MEM[41].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM42 GTM_gtm_cls3.MCS3_MEM[42].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM43 GTM_gtm_cls3.MCS3_MEM[43].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM44 GTM_gtm_cls3.MCS3_MEM[44].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM45 GTM_gtm_cls3.MCS3_MEM[45].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM46 GTM_gtm_cls3.MCS3_MEM[46].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM47 GTM_gtm_cls3.MCS3_MEM[47].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM48 GTM_gtm_cls3.MCS3_MEM[48].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM49 GTM_gtm_cls3.MCS3_MEM[49].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM50 GTM_gtm_cls3.MCS3_MEM[50].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM51 GTM_gtm_cls3.MCS3_MEM[51].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM52 GTM_gtm_cls3.MCS3_MEM[52].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM53 GTM_gtm_cls3.MCS3_MEM[53].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM54 GTM_gtm_cls3.MCS3_MEM[54].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM55 GTM_gtm_cls3.MCS3_MEM[55].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM56 GTM_gtm_cls3.MCS3_MEM[56].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM57 GTM_gtm_cls3.MCS3_MEM[57].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM58 GTM_gtm_cls3.MCS3_MEM[58].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM59 GTM_gtm_cls3.MCS3_MEM[59].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM60 GTM_gtm_cls3.MCS3_MEM[60].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM61 GTM_gtm_cls3.MCS3_MEM[61].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM62 GTM_gtm_cls3.MCS3_MEM[62].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM63 GTM_gtm_cls3.MCS3_MEM[63].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM64 GTM_gtm_cls3.MCS3_MEM[64].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM65 GTM_gtm_cls3.MCS3_MEM[65].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM66 GTM_gtm_cls3.MCS3_MEM[66].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM67 GTM_gtm_cls3.MCS3_MEM[67].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM68 GTM_gtm_cls3.MCS3_MEM[68].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM69 GTM_gtm_cls3.MCS3_MEM[69].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM70 GTM_gtm_cls3.MCS3_MEM[70].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM71 GTM_gtm_cls3.MCS3_MEM[71].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM72 GTM_gtm_cls3.MCS3_MEM[72].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM73 GTM_gtm_cls3.MCS3_MEM[73].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM74 GTM_gtm_cls3.MCS3_MEM[74].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM75 GTM_gtm_cls3.MCS3_MEM[75].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM76 GTM_gtm_cls3.MCS3_MEM[76].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM77 GTM_gtm_cls3.MCS3_MEM[77].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM78 GTM_gtm_cls3.MCS3_MEM[78].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM79 GTM_gtm_cls3.MCS3_MEM[79].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM80 GTM_gtm_cls3.MCS3_MEM[80].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM81 GTM_gtm_cls3.MCS3_MEM[81].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM82 GTM_gtm_cls3.MCS3_MEM[82].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM83 GTM_gtm_cls3.MCS3_MEM[83].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM84 GTM_gtm_cls3.MCS3_MEM[84].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM85 GTM_gtm_cls3.MCS3_MEM[85].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM86 GTM_gtm_cls3.MCS3_MEM[86].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM87 GTM_gtm_cls3.MCS3_MEM[87].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM88 GTM_gtm_cls3.MCS3_MEM[88].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM89 GTM_gtm_cls3.MCS3_MEM[89].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM90 GTM_gtm_cls3.MCS3_MEM[90].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM91 GTM_gtm_cls3.MCS3_MEM[91].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM92 GTM_gtm_cls3.MCS3_MEM[92].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM93 GTM_gtm_cls3.MCS3_MEM[93].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM94 GTM_gtm_cls3.MCS3_MEM[94].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM95 GTM_gtm_cls3.MCS3_MEM[95].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM96 GTM_gtm_cls3.MCS3_MEM[96].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM97 GTM_gtm_cls3.MCS3_MEM[97].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM98 GTM_gtm_cls3.MCS3_MEM[98].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM99 GTM_gtm_cls3.MCS3_MEM[99].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM100 GTM_gtm_cls3.MCS3_MEM[100].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM101 GTM_gtm_cls3.MCS3_MEM[101].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM102 GTM_gtm_cls3.MCS3_MEM[102].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM103 GTM_gtm_cls3.MCS3_MEM[103].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM104 GTM_gtm_cls3.MCS3_MEM[104].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM105 GTM_gtm_cls3.MCS3_MEM[105].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM106 GTM_gtm_cls3.MCS3_MEM[106].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM107 GTM_gtm_cls3.MCS3_MEM[107].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM108 GTM_gtm_cls3.MCS3_MEM[108].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM109 GTM_gtm_cls3.MCS3_MEM[109].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM110 GTM_gtm_cls3.MCS3_MEM[110].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM111 GTM_gtm_cls3.MCS3_MEM[111].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM112 GTM_gtm_cls3.MCS3_MEM[112].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM113 GTM_gtm_cls3.MCS3_MEM[113].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM114 GTM_gtm_cls3.MCS3_MEM[114].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM115 GTM_gtm_cls3.MCS3_MEM[115].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM116 GTM_gtm_cls3.MCS3_MEM[116].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM117 GTM_gtm_cls3.MCS3_MEM[117].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM118 GTM_gtm_cls3.MCS3_MEM[118].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM119 GTM_gtm_cls3.MCS3_MEM[119].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM120 GTM_gtm_cls3.MCS3_MEM[120].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM121 GTM_gtm_cls3.MCS3_MEM[121].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM122 GTM_gtm_cls3.MCS3_MEM[122].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM123 GTM_gtm_cls3.MCS3_MEM[123].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM124 GTM_gtm_cls3.MCS3_MEM[124].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM125 GTM_gtm_cls3.MCS3_MEM[125].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM126 GTM_gtm_cls3.MCS3_MEM[126].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM127 GTM_gtm_cls3.MCS3_MEM[127].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM128 GTM_gtm_cls3.MCS3_MEM[128].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM129 GTM_gtm_cls3.MCS3_MEM[129].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM130 GTM_gtm_cls3.MCS3_MEM[130].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM131 GTM_gtm_cls3.MCS3_MEM[131].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM132 GTM_gtm_cls3.MCS3_MEM[132].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM133 GTM_gtm_cls3.MCS3_MEM[133].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM134 GTM_gtm_cls3.MCS3_MEM[134].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM135 GTM_gtm_cls3.MCS3_MEM[135].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM136 GTM_gtm_cls3.MCS3_MEM[136].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM137 GTM_gtm_cls3.MCS3_MEM[137].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM138 GTM_gtm_cls3.MCS3_MEM[138].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM139 GTM_gtm_cls3.MCS3_MEM[139].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM140 GTM_gtm_cls3.MCS3_MEM[140].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM141 GTM_gtm_cls3.MCS3_MEM[141].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM142 GTM_gtm_cls3.MCS3_MEM[142].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM143 GTM_gtm_cls3.MCS3_MEM[143].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM144 GTM_gtm_cls3.MCS3_MEM[144].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM145 GTM_gtm_cls3.MCS3_MEM[145].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM146 GTM_gtm_cls3.MCS3_MEM[146].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM147 GTM_gtm_cls3.MCS3_MEM[147].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM148 GTM_gtm_cls3.MCS3_MEM[148].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM149 GTM_gtm_cls3.MCS3_MEM[149].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM150 GTM_gtm_cls3.MCS3_MEM[150].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM151 GTM_gtm_cls3.MCS3_MEM[151].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM152 GTM_gtm_cls3.MCS3_MEM[152].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM153 GTM_gtm_cls3.MCS3_MEM[153].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM154 GTM_gtm_cls3.MCS3_MEM[154].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM155 GTM_gtm_cls3.MCS3_MEM[155].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM156 GTM_gtm_cls3.MCS3_MEM[156].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM157 GTM_gtm_cls3.MCS3_MEM[157].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM158 GTM_gtm_cls3.MCS3_MEM[158].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM159 GTM_gtm_cls3.MCS3_MEM[159].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM160 GTM_gtm_cls3.MCS3_MEM[160].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM161 GTM_gtm_cls3.MCS3_MEM[161].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM162 GTM_gtm_cls3.MCS3_MEM[162].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM163 GTM_gtm_cls3.MCS3_MEM[163].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM164 GTM_gtm_cls3.MCS3_MEM[164].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM165 GTM_gtm_cls3.MCS3_MEM[165].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM166 GTM_gtm_cls3.MCS3_MEM[166].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM167 GTM_gtm_cls3.MCS3_MEM[167].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM168 GTM_gtm_cls3.MCS3_MEM[168].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM169 GTM_gtm_cls3.MCS3_MEM[169].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM170 GTM_gtm_cls3.MCS3_MEM[170].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM171 GTM_gtm_cls3.MCS3_MEM[171].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM172 GTM_gtm_cls3.MCS3_MEM[172].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM173 GTM_gtm_cls3.MCS3_MEM[173].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM174 GTM_gtm_cls3.MCS3_MEM[174].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM175 GTM_gtm_cls3.MCS3_MEM[175].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM176 GTM_gtm_cls3.MCS3_MEM[176].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM177 GTM_gtm_cls3.MCS3_MEM[177].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM178 GTM_gtm_cls3.MCS3_MEM[178].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM179 GTM_gtm_cls3.MCS3_MEM[179].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM180 GTM_gtm_cls3.MCS3_MEM[180].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM181 GTM_gtm_cls3.MCS3_MEM[181].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM182 GTM_gtm_cls3.MCS3_MEM[182].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM183 GTM_gtm_cls3.MCS3_MEM[183].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM184 GTM_gtm_cls3.MCS3_MEM[184].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM185 GTM_gtm_cls3.MCS3_MEM[185].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM186 GTM_gtm_cls3.MCS3_MEM[186].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM187 GTM_gtm_cls3.MCS3_MEM[187].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM188 GTM_gtm_cls3.MCS3_MEM[188].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM189 GTM_gtm_cls3.MCS3_MEM[189].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM190 GTM_gtm_cls3.MCS3_MEM[190].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM191 GTM_gtm_cls3.MCS3_MEM[191].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM192 GTM_gtm_cls3.MCS3_MEM[192].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM193 GTM_gtm_cls3.MCS3_MEM[193].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM194 GTM_gtm_cls3.MCS3_MEM[194].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM195 GTM_gtm_cls3.MCS3_MEM[195].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM196 GTM_gtm_cls3.MCS3_MEM[196].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM197 GTM_gtm_cls3.MCS3_MEM[197].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM198 GTM_gtm_cls3.MCS3_MEM[198].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM199 GTM_gtm_cls3.MCS3_MEM[199].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM200 GTM_gtm_cls3.MCS3_MEM[200].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM201 GTM_gtm_cls3.MCS3_MEM[201].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM202 GTM_gtm_cls3.MCS3_MEM[202].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM203 GTM_gtm_cls3.MCS3_MEM[203].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM204 GTM_gtm_cls3.MCS3_MEM[204].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM205 GTM_gtm_cls3.MCS3_MEM[205].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM206 GTM_gtm_cls3.MCS3_MEM[206].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM207 GTM_gtm_cls3.MCS3_MEM[207].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM208 GTM_gtm_cls3.MCS3_MEM[208].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM209 GTM_gtm_cls3.MCS3_MEM[209].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM210 GTM_gtm_cls3.MCS3_MEM[210].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM211 GTM_gtm_cls3.MCS3_MEM[211].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM212 GTM_gtm_cls3.MCS3_MEM[212].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM213 GTM_gtm_cls3.MCS3_MEM[213].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM214 GTM_gtm_cls3.MCS3_MEM[214].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM215 GTM_gtm_cls3.MCS3_MEM[215].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM216 GTM_gtm_cls3.MCS3_MEM[216].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM217 GTM_gtm_cls3.MCS3_MEM[217].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM218 GTM_gtm_cls3.MCS3_MEM[218].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM219 GTM_gtm_cls3.MCS3_MEM[219].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM220 GTM_gtm_cls3.MCS3_MEM[220].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM221 GTM_gtm_cls3.MCS3_MEM[221].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM222 GTM_gtm_cls3.MCS3_MEM[222].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM223 GTM_gtm_cls3.MCS3_MEM[223].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM224 GTM_gtm_cls3.MCS3_MEM[224].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM225 GTM_gtm_cls3.MCS3_MEM[225].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM226 GTM_gtm_cls3.MCS3_MEM[226].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM227 GTM_gtm_cls3.MCS3_MEM[227].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM228 GTM_gtm_cls3.MCS3_MEM[228].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM229 GTM_gtm_cls3.MCS3_MEM[229].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM230 GTM_gtm_cls3.MCS3_MEM[230].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM231 GTM_gtm_cls3.MCS3_MEM[231].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM232 GTM_gtm_cls3.MCS3_MEM[232].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM233 GTM_gtm_cls3.MCS3_MEM[233].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM234 GTM_gtm_cls3.MCS3_MEM[234].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM235 GTM_gtm_cls3.MCS3_MEM[235].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM236 GTM_gtm_cls3.MCS3_MEM[236].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM237 GTM_gtm_cls3.MCS3_MEM[237].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM238 GTM_gtm_cls3.MCS3_MEM[238].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM239 GTM_gtm_cls3.MCS3_MEM[239].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM240 GTM_gtm_cls3.MCS3_MEM[240].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM241 GTM_gtm_cls3.MCS3_MEM[241].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM242 GTM_gtm_cls3.MCS3_MEM[242].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM243 GTM_gtm_cls3.MCS3_MEM[243].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM244 GTM_gtm_cls3.MCS3_MEM[244].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM245 GTM_gtm_cls3.MCS3_MEM[245].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM246 GTM_gtm_cls3.MCS3_MEM[246].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM247 GTM_gtm_cls3.MCS3_MEM[247].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM248 GTM_gtm_cls3.MCS3_MEM[248].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM249 GTM_gtm_cls3.MCS3_MEM[249].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM250 GTM_gtm_cls3.MCS3_MEM[250].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM251 GTM_gtm_cls3.MCS3_MEM[251].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM252 GTM_gtm_cls3.MCS3_MEM[252].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM253 GTM_gtm_cls3.MCS3_MEM[253].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM254 GTM_gtm_cls3.MCS3_MEM[254].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM255 GTM_gtm_cls3.MCS3_MEM[255].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM256 GTM_gtm_cls3.MCS3_MEM[256].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM257 GTM_gtm_cls3.MCS3_MEM[257].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM258 GTM_gtm_cls3.MCS3_MEM[258].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM259 GTM_gtm_cls3.MCS3_MEM[259].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM260 GTM_gtm_cls3.MCS3_MEM[260].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM261 GTM_gtm_cls3.MCS3_MEM[261].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM262 GTM_gtm_cls3.MCS3_MEM[262].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM263 GTM_gtm_cls3.MCS3_MEM[263].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM264 GTM_gtm_cls3.MCS3_MEM[264].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM265 GTM_gtm_cls3.MCS3_MEM[265].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM266 GTM_gtm_cls3.MCS3_MEM[266].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM267 GTM_gtm_cls3.MCS3_MEM[267].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM268 GTM_gtm_cls3.MCS3_MEM[268].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM269 GTM_gtm_cls3.MCS3_MEM[269].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM270 GTM_gtm_cls3.MCS3_MEM[270].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM271 GTM_gtm_cls3.MCS3_MEM[271].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM272 GTM_gtm_cls3.MCS3_MEM[272].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM273 GTM_gtm_cls3.MCS3_MEM[273].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM274 GTM_gtm_cls3.MCS3_MEM[274].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM275 GTM_gtm_cls3.MCS3_MEM[275].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM276 GTM_gtm_cls3.MCS3_MEM[276].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM277 GTM_gtm_cls3.MCS3_MEM[277].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM278 GTM_gtm_cls3.MCS3_MEM[278].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM279 GTM_gtm_cls3.MCS3_MEM[279].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM280 GTM_gtm_cls3.MCS3_MEM[280].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM281 GTM_gtm_cls3.MCS3_MEM[281].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM282 GTM_gtm_cls3.MCS3_MEM[282].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM283 GTM_gtm_cls3.MCS3_MEM[283].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM284 GTM_gtm_cls3.MCS3_MEM[284].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM285 GTM_gtm_cls3.MCS3_MEM[285].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM286 GTM_gtm_cls3.MCS3_MEM[286].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM287 GTM_gtm_cls3.MCS3_MEM[287].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM288 GTM_gtm_cls3.MCS3_MEM[288].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM289 GTM_gtm_cls3.MCS3_MEM[289].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM290 GTM_gtm_cls3.MCS3_MEM[290].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM291 GTM_gtm_cls3.MCS3_MEM[291].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM292 GTM_gtm_cls3.MCS3_MEM[292].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM293 GTM_gtm_cls3.MCS3_MEM[293].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM294 GTM_gtm_cls3.MCS3_MEM[294].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM295 GTM_gtm_cls3.MCS3_MEM[295].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM296 GTM_gtm_cls3.MCS3_MEM[296].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM297 GTM_gtm_cls3.MCS3_MEM[297].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM298 GTM_gtm_cls3.MCS3_MEM[298].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM299 GTM_gtm_cls3.MCS3_MEM[299].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM300 GTM_gtm_cls3.MCS3_MEM[300].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM301 GTM_gtm_cls3.MCS3_MEM[301].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM302 GTM_gtm_cls3.MCS3_MEM[302].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM303 GTM_gtm_cls3.MCS3_MEM[303].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM304 GTM_gtm_cls3.MCS3_MEM[304].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM305 GTM_gtm_cls3.MCS3_MEM[305].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM306 GTM_gtm_cls3.MCS3_MEM[306].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM307 GTM_gtm_cls3.MCS3_MEM[307].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM308 GTM_gtm_cls3.MCS3_MEM[308].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM309 GTM_gtm_cls3.MCS3_MEM[309].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM310 GTM_gtm_cls3.MCS3_MEM[310].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM311 GTM_gtm_cls3.MCS3_MEM[311].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM312 GTM_gtm_cls3.MCS3_MEM[312].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM313 GTM_gtm_cls3.MCS3_MEM[313].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM314 GTM_gtm_cls3.MCS3_MEM[314].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM315 GTM_gtm_cls3.MCS3_MEM[315].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM316 GTM_gtm_cls3.MCS3_MEM[316].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM317 GTM_gtm_cls3.MCS3_MEM[317].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM318 GTM_gtm_cls3.MCS3_MEM[318].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM319 GTM_gtm_cls3.MCS3_MEM[319].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM320 GTM_gtm_cls3.MCS3_MEM[320].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM321 GTM_gtm_cls3.MCS3_MEM[321].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM322 GTM_gtm_cls3.MCS3_MEM[322].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM323 GTM_gtm_cls3.MCS3_MEM[323].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM324 GTM_gtm_cls3.MCS3_MEM[324].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM325 GTM_gtm_cls3.MCS3_MEM[325].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM326 GTM_gtm_cls3.MCS3_MEM[326].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM327 GTM_gtm_cls3.MCS3_MEM[327].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM328 GTM_gtm_cls3.MCS3_MEM[328].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM329 GTM_gtm_cls3.MCS3_MEM[329].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM330 GTM_gtm_cls3.MCS3_MEM[330].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM331 GTM_gtm_cls3.MCS3_MEM[331].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM332 GTM_gtm_cls3.MCS3_MEM[332].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM333 GTM_gtm_cls3.MCS3_MEM[333].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM334 GTM_gtm_cls3.MCS3_MEM[334].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM335 GTM_gtm_cls3.MCS3_MEM[335].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM336 GTM_gtm_cls3.MCS3_MEM[336].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM337 GTM_gtm_cls3.MCS3_MEM[337].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM338 GTM_gtm_cls3.MCS3_MEM[338].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM339 GTM_gtm_cls3.MCS3_MEM[339].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM340 GTM_gtm_cls3.MCS3_MEM[340].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM341 GTM_gtm_cls3.MCS3_MEM[341].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM342 GTM_gtm_cls3.MCS3_MEM[342].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM343 GTM_gtm_cls3.MCS3_MEM[343].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM344 GTM_gtm_cls3.MCS3_MEM[344].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM345 GTM_gtm_cls3.MCS3_MEM[345].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM346 GTM_gtm_cls3.MCS3_MEM[346].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM347 GTM_gtm_cls3.MCS3_MEM[347].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM348 GTM_gtm_cls3.MCS3_MEM[348].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM349 GTM_gtm_cls3.MCS3_MEM[349].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM350 GTM_gtm_cls3.MCS3_MEM[350].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM351 GTM_gtm_cls3.MCS3_MEM[351].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM352 GTM_gtm_cls3.MCS3_MEM[352].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM353 GTM_gtm_cls3.MCS3_MEM[353].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM354 GTM_gtm_cls3.MCS3_MEM[354].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM355 GTM_gtm_cls3.MCS3_MEM[355].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM356 GTM_gtm_cls3.MCS3_MEM[356].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM357 GTM_gtm_cls3.MCS3_MEM[357].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM358 GTM_gtm_cls3.MCS3_MEM[358].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM359 GTM_gtm_cls3.MCS3_MEM[359].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM360 GTM_gtm_cls3.MCS3_MEM[360].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM361 GTM_gtm_cls3.MCS3_MEM[361].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM362 GTM_gtm_cls3.MCS3_MEM[362].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM363 GTM_gtm_cls3.MCS3_MEM[363].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM364 GTM_gtm_cls3.MCS3_MEM[364].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM365 GTM_gtm_cls3.MCS3_MEM[365].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM366 GTM_gtm_cls3.MCS3_MEM[366].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM367 GTM_gtm_cls3.MCS3_MEM[367].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM368 GTM_gtm_cls3.MCS3_MEM[368].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM369 GTM_gtm_cls3.MCS3_MEM[369].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM370 GTM_gtm_cls3.MCS3_MEM[370].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM371 GTM_gtm_cls3.MCS3_MEM[371].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM372 GTM_gtm_cls3.MCS3_MEM[372].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM373 GTM_gtm_cls3.MCS3_MEM[373].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM374 GTM_gtm_cls3.MCS3_MEM[374].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM375 GTM_gtm_cls3.MCS3_MEM[375].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM376 GTM_gtm_cls3.MCS3_MEM[376].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM377 GTM_gtm_cls3.MCS3_MEM[377].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM378 GTM_gtm_cls3.MCS3_MEM[378].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM379 GTM_gtm_cls3.MCS3_MEM[379].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM380 GTM_gtm_cls3.MCS3_MEM[380].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM381 GTM_gtm_cls3.MCS3_MEM[381].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM382 GTM_gtm_cls3.MCS3_MEM[382].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM383 GTM_gtm_cls3.MCS3_MEM[383].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM384 GTM_gtm_cls3.MCS3_MEM[384].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM385 GTM_gtm_cls3.MCS3_MEM[385].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM386 GTM_gtm_cls3.MCS3_MEM[386].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM387 GTM_gtm_cls3.MCS3_MEM[387].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM388 GTM_gtm_cls3.MCS3_MEM[388].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM389 GTM_gtm_cls3.MCS3_MEM[389].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM390 GTM_gtm_cls3.MCS3_MEM[390].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM391 GTM_gtm_cls3.MCS3_MEM[391].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM392 GTM_gtm_cls3.MCS3_MEM[392].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM393 GTM_gtm_cls3.MCS3_MEM[393].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM394 GTM_gtm_cls3.MCS3_MEM[394].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM395 GTM_gtm_cls3.MCS3_MEM[395].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM396 GTM_gtm_cls3.MCS3_MEM[396].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM397 GTM_gtm_cls3.MCS3_MEM[397].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM398 GTM_gtm_cls3.MCS3_MEM[398].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM399 GTM_gtm_cls3.MCS3_MEM[399].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM400 GTM_gtm_cls3.MCS3_MEM[400].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM401 GTM_gtm_cls3.MCS3_MEM[401].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM402 GTM_gtm_cls3.MCS3_MEM[402].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM403 GTM_gtm_cls3.MCS3_MEM[403].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM404 GTM_gtm_cls3.MCS3_MEM[404].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM405 GTM_gtm_cls3.MCS3_MEM[405].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM406 GTM_gtm_cls3.MCS3_MEM[406].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM407 GTM_gtm_cls3.MCS3_MEM[407].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM408 GTM_gtm_cls3.MCS3_MEM[408].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM409 GTM_gtm_cls3.MCS3_MEM[409].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM410 GTM_gtm_cls3.MCS3_MEM[410].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM411 GTM_gtm_cls3.MCS3_MEM[411].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM412 GTM_gtm_cls3.MCS3_MEM[412].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM413 GTM_gtm_cls3.MCS3_MEM[413].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM414 GTM_gtm_cls3.MCS3_MEM[414].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM415 GTM_gtm_cls3.MCS3_MEM[415].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM416 GTM_gtm_cls3.MCS3_MEM[416].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM417 GTM_gtm_cls3.MCS3_MEM[417].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM418 GTM_gtm_cls3.MCS3_MEM[418].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM419 GTM_gtm_cls3.MCS3_MEM[419].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM420 GTM_gtm_cls3.MCS3_MEM[420].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM421 GTM_gtm_cls3.MCS3_MEM[421].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM422 GTM_gtm_cls3.MCS3_MEM[422].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM423 GTM_gtm_cls3.MCS3_MEM[423].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM424 GTM_gtm_cls3.MCS3_MEM[424].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM425 GTM_gtm_cls3.MCS3_MEM[425].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM426 GTM_gtm_cls3.MCS3_MEM[426].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM427 GTM_gtm_cls3.MCS3_MEM[427].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM428 GTM_gtm_cls3.MCS3_MEM[428].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM429 GTM_gtm_cls3.MCS3_MEM[429].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM430 GTM_gtm_cls3.MCS3_MEM[430].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM431 GTM_gtm_cls3.MCS3_MEM[431].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM432 GTM_gtm_cls3.MCS3_MEM[432].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM433 GTM_gtm_cls3.MCS3_MEM[433].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM434 GTM_gtm_cls3.MCS3_MEM[434].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM435 GTM_gtm_cls3.MCS3_MEM[435].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM436 GTM_gtm_cls3.MCS3_MEM[436].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM437 GTM_gtm_cls3.MCS3_MEM[437].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM438 GTM_gtm_cls3.MCS3_MEM[438].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM439 GTM_gtm_cls3.MCS3_MEM[439].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM440 GTM_gtm_cls3.MCS3_MEM[440].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM441 GTM_gtm_cls3.MCS3_MEM[441].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM442 GTM_gtm_cls3.MCS3_MEM[442].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM443 GTM_gtm_cls3.MCS3_MEM[443].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM444 GTM_gtm_cls3.MCS3_MEM[444].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM445 GTM_gtm_cls3.MCS3_MEM[445].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM446 GTM_gtm_cls3.MCS3_MEM[446].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM447 GTM_gtm_cls3.MCS3_MEM[447].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM448 GTM_gtm_cls3.MCS3_MEM[448].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM449 GTM_gtm_cls3.MCS3_MEM[449].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM450 GTM_gtm_cls3.MCS3_MEM[450].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM451 GTM_gtm_cls3.MCS3_MEM[451].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM452 GTM_gtm_cls3.MCS3_MEM[452].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM453 GTM_gtm_cls3.MCS3_MEM[453].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM454 GTM_gtm_cls3.MCS3_MEM[454].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM455 GTM_gtm_cls3.MCS3_MEM[455].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM456 GTM_gtm_cls3.MCS3_MEM[456].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM457 GTM_gtm_cls3.MCS3_MEM[457].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM458 GTM_gtm_cls3.MCS3_MEM[458].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM459 GTM_gtm_cls3.MCS3_MEM[459].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM460 GTM_gtm_cls3.MCS3_MEM[460].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM461 GTM_gtm_cls3.MCS3_MEM[461].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM462 GTM_gtm_cls3.MCS3_MEM[462].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM463 GTM_gtm_cls3.MCS3_MEM[463].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM464 GTM_gtm_cls3.MCS3_MEM[464].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM465 GTM_gtm_cls3.MCS3_MEM[465].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM466 GTM_gtm_cls3.MCS3_MEM[466].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM467 GTM_gtm_cls3.MCS3_MEM[467].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM468 GTM_gtm_cls3.MCS3_MEM[468].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM469 GTM_gtm_cls3.MCS3_MEM[469].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM470 GTM_gtm_cls3.MCS3_MEM[470].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM471 GTM_gtm_cls3.MCS3_MEM[471].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM472 GTM_gtm_cls3.MCS3_MEM[472].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM473 GTM_gtm_cls3.MCS3_MEM[473].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM474 GTM_gtm_cls3.MCS3_MEM[474].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM475 GTM_gtm_cls3.MCS3_MEM[475].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM476 GTM_gtm_cls3.MCS3_MEM[476].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM477 GTM_gtm_cls3.MCS3_MEM[477].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM478 GTM_gtm_cls3.MCS3_MEM[478].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM479 GTM_gtm_cls3.MCS3_MEM[479].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM480 GTM_gtm_cls3.MCS3_MEM[480].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM481 GTM_gtm_cls3.MCS3_MEM[481].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM482 GTM_gtm_cls3.MCS3_MEM[482].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM483 GTM_gtm_cls3.MCS3_MEM[483].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM484 GTM_gtm_cls3.MCS3_MEM[484].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM485 GTM_gtm_cls3.MCS3_MEM[485].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM486 GTM_gtm_cls3.MCS3_MEM[486].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM487 GTM_gtm_cls3.MCS3_MEM[487].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM488 GTM_gtm_cls3.MCS3_MEM[488].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM489 GTM_gtm_cls3.MCS3_MEM[489].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM490 GTM_gtm_cls3.MCS3_MEM[490].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM491 GTM_gtm_cls3.MCS3_MEM[491].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM492 GTM_gtm_cls3.MCS3_MEM[492].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM493 GTM_gtm_cls3.MCS3_MEM[493].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM494 GTM_gtm_cls3.MCS3_MEM[494].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM495 GTM_gtm_cls3.MCS3_MEM[495].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM496 GTM_gtm_cls3.MCS3_MEM[496].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM497 GTM_gtm_cls3.MCS3_MEM[497].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM498 GTM_gtm_cls3.MCS3_MEM[498].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM499 GTM_gtm_cls3.MCS3_MEM[499].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM500 GTM_gtm_cls3.MCS3_MEM[500].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM501 GTM_gtm_cls3.MCS3_MEM[501].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM502 GTM_gtm_cls3.MCS3_MEM[502].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM503 GTM_gtm_cls3.MCS3_MEM[503].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM504 GTM_gtm_cls3.MCS3_MEM[504].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM505 GTM_gtm_cls3.MCS3_MEM[505].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM506 GTM_gtm_cls3.MCS3_MEM[506].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM507 GTM_gtm_cls3.MCS3_MEM[507].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM508 GTM_gtm_cls3.MCS3_MEM[508].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM509 GTM_gtm_cls3.MCS3_MEM[509].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM510 GTM_gtm_cls3.MCS3_MEM[510].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM511 GTM_gtm_cls3.MCS3_MEM[511].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM512 GTM_gtm_cls3.MCS3_MEM[512].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM513 GTM_gtm_cls3.MCS3_MEM[513].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM514 GTM_gtm_cls3.MCS3_MEM[514].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM515 GTM_gtm_cls3.MCS3_MEM[515].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM516 GTM_gtm_cls3.MCS3_MEM[516].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM517 GTM_gtm_cls3.MCS3_MEM[517].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM518 GTM_gtm_cls3.MCS3_MEM[518].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM519 GTM_gtm_cls3.MCS3_MEM[519].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM520 GTM_gtm_cls3.MCS3_MEM[520].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM521 GTM_gtm_cls3.MCS3_MEM[521].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM522 GTM_gtm_cls3.MCS3_MEM[522].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM523 GTM_gtm_cls3.MCS3_MEM[523].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM524 GTM_gtm_cls3.MCS3_MEM[524].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM525 GTM_gtm_cls3.MCS3_MEM[525].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM526 GTM_gtm_cls3.MCS3_MEM[526].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM527 GTM_gtm_cls3.MCS3_MEM[527].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM528 GTM_gtm_cls3.MCS3_MEM[528].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM529 GTM_gtm_cls3.MCS3_MEM[529].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM530 GTM_gtm_cls3.MCS3_MEM[530].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM531 GTM_gtm_cls3.MCS3_MEM[531].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM532 GTM_gtm_cls3.MCS3_MEM[532].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM533 GTM_gtm_cls3.MCS3_MEM[533].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM534 GTM_gtm_cls3.MCS3_MEM[534].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM535 GTM_gtm_cls3.MCS3_MEM[535].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM536 GTM_gtm_cls3.MCS3_MEM[536].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM537 GTM_gtm_cls3.MCS3_MEM[537].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM538 GTM_gtm_cls3.MCS3_MEM[538].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM539 GTM_gtm_cls3.MCS3_MEM[539].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM540 GTM_gtm_cls3.MCS3_MEM[540].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM541 GTM_gtm_cls3.MCS3_MEM[541].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM542 GTM_gtm_cls3.MCS3_MEM[542].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM543 GTM_gtm_cls3.MCS3_MEM[543].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM544 GTM_gtm_cls3.MCS3_MEM[544].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM545 GTM_gtm_cls3.MCS3_MEM[545].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM546 GTM_gtm_cls3.MCS3_MEM[546].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM547 GTM_gtm_cls3.MCS3_MEM[547].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM548 GTM_gtm_cls3.MCS3_MEM[548].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM549 GTM_gtm_cls3.MCS3_MEM[549].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM550 GTM_gtm_cls3.MCS3_MEM[550].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM551 GTM_gtm_cls3.MCS3_MEM[551].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM552 GTM_gtm_cls3.MCS3_MEM[552].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM553 GTM_gtm_cls3.MCS3_MEM[553].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM554 GTM_gtm_cls3.MCS3_MEM[554].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM555 GTM_gtm_cls3.MCS3_MEM[555].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM556 GTM_gtm_cls3.MCS3_MEM[556].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM557 GTM_gtm_cls3.MCS3_MEM[557].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM558 GTM_gtm_cls3.MCS3_MEM[558].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM559 GTM_gtm_cls3.MCS3_MEM[559].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM560 GTM_gtm_cls3.MCS3_MEM[560].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM561 GTM_gtm_cls3.MCS3_MEM[561].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM562 GTM_gtm_cls3.MCS3_MEM[562].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM563 GTM_gtm_cls3.MCS3_MEM[563].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM564 GTM_gtm_cls3.MCS3_MEM[564].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM565 GTM_gtm_cls3.MCS3_MEM[565].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM566 GTM_gtm_cls3.MCS3_MEM[566].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM567 GTM_gtm_cls3.MCS3_MEM[567].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM568 GTM_gtm_cls3.MCS3_MEM[568].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM569 GTM_gtm_cls3.MCS3_MEM[569].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM570 GTM_gtm_cls3.MCS3_MEM[570].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM571 GTM_gtm_cls3.MCS3_MEM[571].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM572 GTM_gtm_cls3.MCS3_MEM[572].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM573 GTM_gtm_cls3.MCS3_MEM[573].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM574 GTM_gtm_cls3.MCS3_MEM[574].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM575 GTM_gtm_cls3.MCS3_MEM[575].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM576 GTM_gtm_cls3.MCS3_MEM[576].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM577 GTM_gtm_cls3.MCS3_MEM[577].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM578 GTM_gtm_cls3.MCS3_MEM[578].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM579 GTM_gtm_cls3.MCS3_MEM[579].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM580 GTM_gtm_cls3.MCS3_MEM[580].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM581 GTM_gtm_cls3.MCS3_MEM[581].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM582 GTM_gtm_cls3.MCS3_MEM[582].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM583 GTM_gtm_cls3.MCS3_MEM[583].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM584 GTM_gtm_cls3.MCS3_MEM[584].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM585 GTM_gtm_cls3.MCS3_MEM[585].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM586 GTM_gtm_cls3.MCS3_MEM[586].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM587 GTM_gtm_cls3.MCS3_MEM[587].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM588 GTM_gtm_cls3.MCS3_MEM[588].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM589 GTM_gtm_cls3.MCS3_MEM[589].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM590 GTM_gtm_cls3.MCS3_MEM[590].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM591 GTM_gtm_cls3.MCS3_MEM[591].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM592 GTM_gtm_cls3.MCS3_MEM[592].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM593 GTM_gtm_cls3.MCS3_MEM[593].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM594 GTM_gtm_cls3.MCS3_MEM[594].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM595 GTM_gtm_cls3.MCS3_MEM[595].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM596 GTM_gtm_cls3.MCS3_MEM[596].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM597 GTM_gtm_cls3.MCS3_MEM[597].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM598 GTM_gtm_cls3.MCS3_MEM[598].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM599 GTM_gtm_cls3.MCS3_MEM[599].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM600 GTM_gtm_cls3.MCS3_MEM[600].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM601 GTM_gtm_cls3.MCS3_MEM[601].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM602 GTM_gtm_cls3.MCS3_MEM[602].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM603 GTM_gtm_cls3.MCS3_MEM[603].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM604 GTM_gtm_cls3.MCS3_MEM[604].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM605 GTM_gtm_cls3.MCS3_MEM[605].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM606 GTM_gtm_cls3.MCS3_MEM[606].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM607 GTM_gtm_cls3.MCS3_MEM[607].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM608 GTM_gtm_cls3.MCS3_MEM[608].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM609 GTM_gtm_cls3.MCS3_MEM[609].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM610 GTM_gtm_cls3.MCS3_MEM[610].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM611 GTM_gtm_cls3.MCS3_MEM[611].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM612 GTM_gtm_cls3.MCS3_MEM[612].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM613 GTM_gtm_cls3.MCS3_MEM[613].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM614 GTM_gtm_cls3.MCS3_MEM[614].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM615 GTM_gtm_cls3.MCS3_MEM[615].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM616 GTM_gtm_cls3.MCS3_MEM[616].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM617 GTM_gtm_cls3.MCS3_MEM[617].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM618 GTM_gtm_cls3.MCS3_MEM[618].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM619 GTM_gtm_cls3.MCS3_MEM[619].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM620 GTM_gtm_cls3.MCS3_MEM[620].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM621 GTM_gtm_cls3.MCS3_MEM[621].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM622 GTM_gtm_cls3.MCS3_MEM[622].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM623 GTM_gtm_cls3.MCS3_MEM[623].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM624 GTM_gtm_cls3.MCS3_MEM[624].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM625 GTM_gtm_cls3.MCS3_MEM[625].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM626 GTM_gtm_cls3.MCS3_MEM[626].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM627 GTM_gtm_cls3.MCS3_MEM[627].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM628 GTM_gtm_cls3.MCS3_MEM[628].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM629 GTM_gtm_cls3.MCS3_MEM[629].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM630 GTM_gtm_cls3.MCS3_MEM[630].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM631 GTM_gtm_cls3.MCS3_MEM[631].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM632 GTM_gtm_cls3.MCS3_MEM[632].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM633 GTM_gtm_cls3.MCS3_MEM[633].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM634 GTM_gtm_cls3.MCS3_MEM[634].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM635 GTM_gtm_cls3.MCS3_MEM[635].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM636 GTM_gtm_cls3.MCS3_MEM[636].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM637 GTM_gtm_cls3.MCS3_MEM[637].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM638 GTM_gtm_cls3.MCS3_MEM[638].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM639 GTM_gtm_cls3.MCS3_MEM[639].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM640 GTM_gtm_cls3.MCS3_MEM[640].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM641 GTM_gtm_cls3.MCS3_MEM[641].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM642 GTM_gtm_cls3.MCS3_MEM[642].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM643 GTM_gtm_cls3.MCS3_MEM[643].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM644 GTM_gtm_cls3.MCS3_MEM[644].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM645 GTM_gtm_cls3.MCS3_MEM[645].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM646 GTM_gtm_cls3.MCS3_MEM[646].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM647 GTM_gtm_cls3.MCS3_MEM[647].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM648 GTM_gtm_cls3.MCS3_MEM[648].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM649 GTM_gtm_cls3.MCS3_MEM[649].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM650 GTM_gtm_cls3.MCS3_MEM[650].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM651 GTM_gtm_cls3.MCS3_MEM[651].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM652 GTM_gtm_cls3.MCS3_MEM[652].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM653 GTM_gtm_cls3.MCS3_MEM[653].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM654 GTM_gtm_cls3.MCS3_MEM[654].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM655 GTM_gtm_cls3.MCS3_MEM[655].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM656 GTM_gtm_cls3.MCS3_MEM[656].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM657 GTM_gtm_cls3.MCS3_MEM[657].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM658 GTM_gtm_cls3.MCS3_MEM[658].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM659 GTM_gtm_cls3.MCS3_MEM[659].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM660 GTM_gtm_cls3.MCS3_MEM[660].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM661 GTM_gtm_cls3.MCS3_MEM[661].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM662 GTM_gtm_cls3.MCS3_MEM[662].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM663 GTM_gtm_cls3.MCS3_MEM[663].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM664 GTM_gtm_cls3.MCS3_MEM[664].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM665 GTM_gtm_cls3.MCS3_MEM[665].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM666 GTM_gtm_cls3.MCS3_MEM[666].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM667 GTM_gtm_cls3.MCS3_MEM[667].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM668 GTM_gtm_cls3.MCS3_MEM[668].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM669 GTM_gtm_cls3.MCS3_MEM[669].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM670 GTM_gtm_cls3.MCS3_MEM[670].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM671 GTM_gtm_cls3.MCS3_MEM[671].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM672 GTM_gtm_cls3.MCS3_MEM[672].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM673 GTM_gtm_cls3.MCS3_MEM[673].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM674 GTM_gtm_cls3.MCS3_MEM[674].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM675 GTM_gtm_cls3.MCS3_MEM[675].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM676 GTM_gtm_cls3.MCS3_MEM[676].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM677 GTM_gtm_cls3.MCS3_MEM[677].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM678 GTM_gtm_cls3.MCS3_MEM[678].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM679 GTM_gtm_cls3.MCS3_MEM[679].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM680 GTM_gtm_cls3.MCS3_MEM[680].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM681 GTM_gtm_cls3.MCS3_MEM[681].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM682 GTM_gtm_cls3.MCS3_MEM[682].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM683 GTM_gtm_cls3.MCS3_MEM[683].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM684 GTM_gtm_cls3.MCS3_MEM[684].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM685 GTM_gtm_cls3.MCS3_MEM[685].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM686 GTM_gtm_cls3.MCS3_MEM[686].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM687 GTM_gtm_cls3.MCS3_MEM[687].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM688 GTM_gtm_cls3.MCS3_MEM[688].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM689 GTM_gtm_cls3.MCS3_MEM[689].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM690 GTM_gtm_cls3.MCS3_MEM[690].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM691 GTM_gtm_cls3.MCS3_MEM[691].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM692 GTM_gtm_cls3.MCS3_MEM[692].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM693 GTM_gtm_cls3.MCS3_MEM[693].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM694 GTM_gtm_cls3.MCS3_MEM[694].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM695 GTM_gtm_cls3.MCS3_MEM[695].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM696 GTM_gtm_cls3.MCS3_MEM[696].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM697 GTM_gtm_cls3.MCS3_MEM[697].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM698 GTM_gtm_cls3.MCS3_MEM[698].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM699 GTM_gtm_cls3.MCS3_MEM[699].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM700 GTM_gtm_cls3.MCS3_MEM[700].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM701 GTM_gtm_cls3.MCS3_MEM[701].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM702 GTM_gtm_cls3.MCS3_MEM[702].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM703 GTM_gtm_cls3.MCS3_MEM[703].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM704 GTM_gtm_cls3.MCS3_MEM[704].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM705 GTM_gtm_cls3.MCS3_MEM[705].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM706 GTM_gtm_cls3.MCS3_MEM[706].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM707 GTM_gtm_cls3.MCS3_MEM[707].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM708 GTM_gtm_cls3.MCS3_MEM[708].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM709 GTM_gtm_cls3.MCS3_MEM[709].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM710 GTM_gtm_cls3.MCS3_MEM[710].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM711 GTM_gtm_cls3.MCS3_MEM[711].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM712 GTM_gtm_cls3.MCS3_MEM[712].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM713 GTM_gtm_cls3.MCS3_MEM[713].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM714 GTM_gtm_cls3.MCS3_MEM[714].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM715 GTM_gtm_cls3.MCS3_MEM[715].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM716 GTM_gtm_cls3.MCS3_MEM[716].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM717 GTM_gtm_cls3.MCS3_MEM[717].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM718 GTM_gtm_cls3.MCS3_MEM[718].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM719 GTM_gtm_cls3.MCS3_MEM[719].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM720 GTM_gtm_cls3.MCS3_MEM[720].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM721 GTM_gtm_cls3.MCS3_MEM[721].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM722 GTM_gtm_cls3.MCS3_MEM[722].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM723 GTM_gtm_cls3.MCS3_MEM[723].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM724 GTM_gtm_cls3.MCS3_MEM[724].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM725 GTM_gtm_cls3.MCS3_MEM[725].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM726 GTM_gtm_cls3.MCS3_MEM[726].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM727 GTM_gtm_cls3.MCS3_MEM[727].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM728 GTM_gtm_cls3.MCS3_MEM[728].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM729 GTM_gtm_cls3.MCS3_MEM[729].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM730 GTM_gtm_cls3.MCS3_MEM[730].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM731 GTM_gtm_cls3.MCS3_MEM[731].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM732 GTM_gtm_cls3.MCS3_MEM[732].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM733 GTM_gtm_cls3.MCS3_MEM[733].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM734 GTM_gtm_cls3.MCS3_MEM[734].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM735 GTM_gtm_cls3.MCS3_MEM[735].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM736 GTM_gtm_cls3.MCS3_MEM[736].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM737 GTM_gtm_cls3.MCS3_MEM[737].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM738 GTM_gtm_cls3.MCS3_MEM[738].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM739 GTM_gtm_cls3.MCS3_MEM[739].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM740 GTM_gtm_cls3.MCS3_MEM[740].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM741 GTM_gtm_cls3.MCS3_MEM[741].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM742 GTM_gtm_cls3.MCS3_MEM[742].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM743 GTM_gtm_cls3.MCS3_MEM[743].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM744 GTM_gtm_cls3.MCS3_MEM[744].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM745 GTM_gtm_cls3.MCS3_MEM[745].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM746 GTM_gtm_cls3.MCS3_MEM[746].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM747 GTM_gtm_cls3.MCS3_MEM[747].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM748 GTM_gtm_cls3.MCS3_MEM[748].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM749 GTM_gtm_cls3.MCS3_MEM[749].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM750 GTM_gtm_cls3.MCS3_MEM[750].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM751 GTM_gtm_cls3.MCS3_MEM[751].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM752 GTM_gtm_cls3.MCS3_MEM[752].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM753 GTM_gtm_cls3.MCS3_MEM[753].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM754 GTM_gtm_cls3.MCS3_MEM[754].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM755 GTM_gtm_cls3.MCS3_MEM[755].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM756 GTM_gtm_cls3.MCS3_MEM[756].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM757 GTM_gtm_cls3.MCS3_MEM[757].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM758 GTM_gtm_cls3.MCS3_MEM[758].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM759 GTM_gtm_cls3.MCS3_MEM[759].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM760 GTM_gtm_cls3.MCS3_MEM[760].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM761 GTM_gtm_cls3.MCS3_MEM[761].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM762 GTM_gtm_cls3.MCS3_MEM[762].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM763 GTM_gtm_cls3.MCS3_MEM[763].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM764 GTM_gtm_cls3.MCS3_MEM[764].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM765 GTM_gtm_cls3.MCS3_MEM[765].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM766 GTM_gtm_cls3.MCS3_MEM[766].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM767 GTM_gtm_cls3.MCS3_MEM[767].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM768 GTM_gtm_cls3.MCS3_MEM[768].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM769 GTM_gtm_cls3.MCS3_MEM[769].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM770 GTM_gtm_cls3.MCS3_MEM[770].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM771 GTM_gtm_cls3.MCS3_MEM[771].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM772 GTM_gtm_cls3.MCS3_MEM[772].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM773 GTM_gtm_cls3.MCS3_MEM[773].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM774 GTM_gtm_cls3.MCS3_MEM[774].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM775 GTM_gtm_cls3.MCS3_MEM[775].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM776 GTM_gtm_cls3.MCS3_MEM[776].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM777 GTM_gtm_cls3.MCS3_MEM[777].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM778 GTM_gtm_cls3.MCS3_MEM[778].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM779 GTM_gtm_cls3.MCS3_MEM[779].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM780 GTM_gtm_cls3.MCS3_MEM[780].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM781 GTM_gtm_cls3.MCS3_MEM[781].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM782 GTM_gtm_cls3.MCS3_MEM[782].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM783 GTM_gtm_cls3.MCS3_MEM[783].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM784 GTM_gtm_cls3.MCS3_MEM[784].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM785 GTM_gtm_cls3.MCS3_MEM[785].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM786 GTM_gtm_cls3.MCS3_MEM[786].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM787 GTM_gtm_cls3.MCS3_MEM[787].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM788 GTM_gtm_cls3.MCS3_MEM[788].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM789 GTM_gtm_cls3.MCS3_MEM[789].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM790 GTM_gtm_cls3.MCS3_MEM[790].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM791 GTM_gtm_cls3.MCS3_MEM[791].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM792 GTM_gtm_cls3.MCS3_MEM[792].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM793 GTM_gtm_cls3.MCS3_MEM[793].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM794 GTM_gtm_cls3.MCS3_MEM[794].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM795 GTM_gtm_cls3.MCS3_MEM[795].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM796 GTM_gtm_cls3.MCS3_MEM[796].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM797 GTM_gtm_cls3.MCS3_MEM[797].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM798 GTM_gtm_cls3.MCS3_MEM[798].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM799 GTM_gtm_cls3.MCS3_MEM[799].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM800 GTM_gtm_cls3.MCS3_MEM[800].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM801 GTM_gtm_cls3.MCS3_MEM[801].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM802 GTM_gtm_cls3.MCS3_MEM[802].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM803 GTM_gtm_cls3.MCS3_MEM[803].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM804 GTM_gtm_cls3.MCS3_MEM[804].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM805 GTM_gtm_cls3.MCS3_MEM[805].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM806 GTM_gtm_cls3.MCS3_MEM[806].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM807 GTM_gtm_cls3.MCS3_MEM[807].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM808 GTM_gtm_cls3.MCS3_MEM[808].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM809 GTM_gtm_cls3.MCS3_MEM[809].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM810 GTM_gtm_cls3.MCS3_MEM[810].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM811 GTM_gtm_cls3.MCS3_MEM[811].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM812 GTM_gtm_cls3.MCS3_MEM[812].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM813 GTM_gtm_cls3.MCS3_MEM[813].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM814 GTM_gtm_cls3.MCS3_MEM[814].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM815 GTM_gtm_cls3.MCS3_MEM[815].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM816 GTM_gtm_cls3.MCS3_MEM[816].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM817 GTM_gtm_cls3.MCS3_MEM[817].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM818 GTM_gtm_cls3.MCS3_MEM[818].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM819 GTM_gtm_cls3.MCS3_MEM[819].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM820 GTM_gtm_cls3.MCS3_MEM[820].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM821 GTM_gtm_cls3.MCS3_MEM[821].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM822 GTM_gtm_cls3.MCS3_MEM[822].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM823 GTM_gtm_cls3.MCS3_MEM[823].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM824 GTM_gtm_cls3.MCS3_MEM[824].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM825 GTM_gtm_cls3.MCS3_MEM[825].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM826 GTM_gtm_cls3.MCS3_MEM[826].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM827 GTM_gtm_cls3.MCS3_MEM[827].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM828 GTM_gtm_cls3.MCS3_MEM[828].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM829 GTM_gtm_cls3.MCS3_MEM[829].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM830 GTM_gtm_cls3.MCS3_MEM[830].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM831 GTM_gtm_cls3.MCS3_MEM[831].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM832 GTM_gtm_cls3.MCS3_MEM[832].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM833 GTM_gtm_cls3.MCS3_MEM[833].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM834 GTM_gtm_cls3.MCS3_MEM[834].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM835 GTM_gtm_cls3.MCS3_MEM[835].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM836 GTM_gtm_cls3.MCS3_MEM[836].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM837 GTM_gtm_cls3.MCS3_MEM[837].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM838 GTM_gtm_cls3.MCS3_MEM[838].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM839 GTM_gtm_cls3.MCS3_MEM[839].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM840 GTM_gtm_cls3.MCS3_MEM[840].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM841 GTM_gtm_cls3.MCS3_MEM[841].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM842 GTM_gtm_cls3.MCS3_MEM[842].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM843 GTM_gtm_cls3.MCS3_MEM[843].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM844 GTM_gtm_cls3.MCS3_MEM[844].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM845 GTM_gtm_cls3.MCS3_MEM[845].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM846 GTM_gtm_cls3.MCS3_MEM[846].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM847 GTM_gtm_cls3.MCS3_MEM[847].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM848 GTM_gtm_cls3.MCS3_MEM[848].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM849 GTM_gtm_cls3.MCS3_MEM[849].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM850 GTM_gtm_cls3.MCS3_MEM[850].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM851 GTM_gtm_cls3.MCS3_MEM[851].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM852 GTM_gtm_cls3.MCS3_MEM[852].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM853 GTM_gtm_cls3.MCS3_MEM[853].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM854 GTM_gtm_cls3.MCS3_MEM[854].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM855 GTM_gtm_cls3.MCS3_MEM[855].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM856 GTM_gtm_cls3.MCS3_MEM[856].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM857 GTM_gtm_cls3.MCS3_MEM[857].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM858 GTM_gtm_cls3.MCS3_MEM[858].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM859 GTM_gtm_cls3.MCS3_MEM[859].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM860 GTM_gtm_cls3.MCS3_MEM[860].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM861 GTM_gtm_cls3.MCS3_MEM[861].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM862 GTM_gtm_cls3.MCS3_MEM[862].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM863 GTM_gtm_cls3.MCS3_MEM[863].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM864 GTM_gtm_cls3.MCS3_MEM[864].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM865 GTM_gtm_cls3.MCS3_MEM[865].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM866 GTM_gtm_cls3.MCS3_MEM[866].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM867 GTM_gtm_cls3.MCS3_MEM[867].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM868 GTM_gtm_cls3.MCS3_MEM[868].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM869 GTM_gtm_cls3.MCS3_MEM[869].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM870 GTM_gtm_cls3.MCS3_MEM[870].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM871 GTM_gtm_cls3.MCS3_MEM[871].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM872 GTM_gtm_cls3.MCS3_MEM[872].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM873 GTM_gtm_cls3.MCS3_MEM[873].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM874 GTM_gtm_cls3.MCS3_MEM[874].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM875 GTM_gtm_cls3.MCS3_MEM[875].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM876 GTM_gtm_cls3.MCS3_MEM[876].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM877 GTM_gtm_cls3.MCS3_MEM[877].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM878 GTM_gtm_cls3.MCS3_MEM[878].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM879 GTM_gtm_cls3.MCS3_MEM[879].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM880 GTM_gtm_cls3.MCS3_MEM[880].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM881 GTM_gtm_cls3.MCS3_MEM[881].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM882 GTM_gtm_cls3.MCS3_MEM[882].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM883 GTM_gtm_cls3.MCS3_MEM[883].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM884 GTM_gtm_cls3.MCS3_MEM[884].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM885 GTM_gtm_cls3.MCS3_MEM[885].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM886 GTM_gtm_cls3.MCS3_MEM[886].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM887 GTM_gtm_cls3.MCS3_MEM[887].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM888 GTM_gtm_cls3.MCS3_MEM[888].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM889 GTM_gtm_cls3.MCS3_MEM[889].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM890 GTM_gtm_cls3.MCS3_MEM[890].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM891 GTM_gtm_cls3.MCS3_MEM[891].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM892 GTM_gtm_cls3.MCS3_MEM[892].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM893 GTM_gtm_cls3.MCS3_MEM[893].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM894 GTM_gtm_cls3.MCS3_MEM[894].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM895 GTM_gtm_cls3.MCS3_MEM[895].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM896 GTM_gtm_cls3.MCS3_MEM[896].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM897 GTM_gtm_cls3.MCS3_MEM[897].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM898 GTM_gtm_cls3.MCS3_MEM[898].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM899 GTM_gtm_cls3.MCS3_MEM[899].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM900 GTM_gtm_cls3.MCS3_MEM[900].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM901 GTM_gtm_cls3.MCS3_MEM[901].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM902 GTM_gtm_cls3.MCS3_MEM[902].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM903 GTM_gtm_cls3.MCS3_MEM[903].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM904 GTM_gtm_cls3.MCS3_MEM[904].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM905 GTM_gtm_cls3.MCS3_MEM[905].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM906 GTM_gtm_cls3.MCS3_MEM[906].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM907 GTM_gtm_cls3.MCS3_MEM[907].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM908 GTM_gtm_cls3.MCS3_MEM[908].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM909 GTM_gtm_cls3.MCS3_MEM[909].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM910 GTM_gtm_cls3.MCS3_MEM[910].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM911 GTM_gtm_cls3.MCS3_MEM[911].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM912 GTM_gtm_cls3.MCS3_MEM[912].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM913 GTM_gtm_cls3.MCS3_MEM[913].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM914 GTM_gtm_cls3.MCS3_MEM[914].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM915 GTM_gtm_cls3.MCS3_MEM[915].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM916 GTM_gtm_cls3.MCS3_MEM[916].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM917 GTM_gtm_cls3.MCS3_MEM[917].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM918 GTM_gtm_cls3.MCS3_MEM[918].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM919 GTM_gtm_cls3.MCS3_MEM[919].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM920 GTM_gtm_cls3.MCS3_MEM[920].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM921 GTM_gtm_cls3.MCS3_MEM[921].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM922 GTM_gtm_cls3.MCS3_MEM[922].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM923 GTM_gtm_cls3.MCS3_MEM[923].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM924 GTM_gtm_cls3.MCS3_MEM[924].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM925 GTM_gtm_cls3.MCS3_MEM[925].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM926 GTM_gtm_cls3.MCS3_MEM[926].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM927 GTM_gtm_cls3.MCS3_MEM[927].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM928 GTM_gtm_cls3.MCS3_MEM[928].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM929 GTM_gtm_cls3.MCS3_MEM[929].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM930 GTM_gtm_cls3.MCS3_MEM[930].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM931 GTM_gtm_cls3.MCS3_MEM[931].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM932 GTM_gtm_cls3.MCS3_MEM[932].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM933 GTM_gtm_cls3.MCS3_MEM[933].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM934 GTM_gtm_cls3.MCS3_MEM[934].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM935 GTM_gtm_cls3.MCS3_MEM[935].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM936 GTM_gtm_cls3.MCS3_MEM[936].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM937 GTM_gtm_cls3.MCS3_MEM[937].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM938 GTM_gtm_cls3.MCS3_MEM[938].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM939 GTM_gtm_cls3.MCS3_MEM[939].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM940 GTM_gtm_cls3.MCS3_MEM[940].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM941 GTM_gtm_cls3.MCS3_MEM[941].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM942 GTM_gtm_cls3.MCS3_MEM[942].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM943 GTM_gtm_cls3.MCS3_MEM[943].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM944 GTM_gtm_cls3.MCS3_MEM[944].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM945 GTM_gtm_cls3.MCS3_MEM[945].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM946 GTM_gtm_cls3.MCS3_MEM[946].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM947 GTM_gtm_cls3.MCS3_MEM[947].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM948 GTM_gtm_cls3.MCS3_MEM[948].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM949 GTM_gtm_cls3.MCS3_MEM[949].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM950 GTM_gtm_cls3.MCS3_MEM[950].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM951 GTM_gtm_cls3.MCS3_MEM[951].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM952 GTM_gtm_cls3.MCS3_MEM[952].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM953 GTM_gtm_cls3.MCS3_MEM[953].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM954 GTM_gtm_cls3.MCS3_MEM[954].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM955 GTM_gtm_cls3.MCS3_MEM[955].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM956 GTM_gtm_cls3.MCS3_MEM[956].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM957 GTM_gtm_cls3.MCS3_MEM[957].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM958 GTM_gtm_cls3.MCS3_MEM[958].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM959 GTM_gtm_cls3.MCS3_MEM[959].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM960 GTM_gtm_cls3.MCS3_MEM[960].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM961 GTM_gtm_cls3.MCS3_MEM[961].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM962 GTM_gtm_cls3.MCS3_MEM[962].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM963 GTM_gtm_cls3.MCS3_MEM[963].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM964 GTM_gtm_cls3.MCS3_MEM[964].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM965 GTM_gtm_cls3.MCS3_MEM[965].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM966 GTM_gtm_cls3.MCS3_MEM[966].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM967 GTM_gtm_cls3.MCS3_MEM[967].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM968 GTM_gtm_cls3.MCS3_MEM[968].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM969 GTM_gtm_cls3.MCS3_MEM[969].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM970 GTM_gtm_cls3.MCS3_MEM[970].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM971 GTM_gtm_cls3.MCS3_MEM[971].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM972 GTM_gtm_cls3.MCS3_MEM[972].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM973 GTM_gtm_cls3.MCS3_MEM[973].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM974 GTM_gtm_cls3.MCS3_MEM[974].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM975 GTM_gtm_cls3.MCS3_MEM[975].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM976 GTM_gtm_cls3.MCS3_MEM[976].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM977 GTM_gtm_cls3.MCS3_MEM[977].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM978 GTM_gtm_cls3.MCS3_MEM[978].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM979 GTM_gtm_cls3.MCS3_MEM[979].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM980 GTM_gtm_cls3.MCS3_MEM[980].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM981 GTM_gtm_cls3.MCS3_MEM[981].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM982 GTM_gtm_cls3.MCS3_MEM[982].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM983 GTM_gtm_cls3.MCS3_MEM[983].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM984 GTM_gtm_cls3.MCS3_MEM[984].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM985 GTM_gtm_cls3.MCS3_MEM[985].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM986 GTM_gtm_cls3.MCS3_MEM[986].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM987 GTM_gtm_cls3.MCS3_MEM[987].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM988 GTM_gtm_cls3.MCS3_MEM[988].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM989 GTM_gtm_cls3.MCS3_MEM[989].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM990 GTM_gtm_cls3.MCS3_MEM[990].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM991 GTM_gtm_cls3.MCS3_MEM[991].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM992 GTM_gtm_cls3.MCS3_MEM[992].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM993 GTM_gtm_cls3.MCS3_MEM[993].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM994 GTM_gtm_cls3.MCS3_MEM[994].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM995 GTM_gtm_cls3.MCS3_MEM[995].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM996 GTM_gtm_cls3.MCS3_MEM[996].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM997 GTM_gtm_cls3.MCS3_MEM[997].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM998 GTM_gtm_cls3.MCS3_MEM[998].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM999 GTM_gtm_cls3.MCS3_MEM[999].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1000 GTM_gtm_cls3.MCS3_MEM[1000].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1001 GTM_gtm_cls3.MCS3_MEM[1001].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1002 GTM_gtm_cls3.MCS3_MEM[1002].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1003 GTM_gtm_cls3.MCS3_MEM[1003].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1004 GTM_gtm_cls3.MCS3_MEM[1004].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1005 GTM_gtm_cls3.MCS3_MEM[1005].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1006 GTM_gtm_cls3.MCS3_MEM[1006].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1007 GTM_gtm_cls3.MCS3_MEM[1007].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1008 GTM_gtm_cls3.MCS3_MEM[1008].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1009 GTM_gtm_cls3.MCS3_MEM[1009].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1010 GTM_gtm_cls3.MCS3_MEM[1010].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1011 GTM_gtm_cls3.MCS3_MEM[1011].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1012 GTM_gtm_cls3.MCS3_MEM[1012].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1013 GTM_gtm_cls3.MCS3_MEM[1013].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1014 GTM_gtm_cls3.MCS3_MEM[1014].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1015 GTM_gtm_cls3.MCS3_MEM[1015].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1016 GTM_gtm_cls3.MCS3_MEM[1016].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1017 GTM_gtm_cls3.MCS3_MEM[1017].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1018 GTM_gtm_cls3.MCS3_MEM[1018].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1019 GTM_gtm_cls3.MCS3_MEM[1019].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1020 GTM_gtm_cls3.MCS3_MEM[1020].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1021 GTM_gtm_cls3.MCS3_MEM[1021].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1022 GTM_gtm_cls3.MCS3_MEM[1022].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1023 GTM_gtm_cls3.MCS3_MEM[1023].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1024 GTM_gtm_cls3.MCS3_MEM[1024].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1025 GTM_gtm_cls3.MCS3_MEM[1025].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1026 GTM_gtm_cls3.MCS3_MEM[1026].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1027 GTM_gtm_cls3.MCS3_MEM[1027].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1028 GTM_gtm_cls3.MCS3_MEM[1028].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1029 GTM_gtm_cls3.MCS3_MEM[1029].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1030 GTM_gtm_cls3.MCS3_MEM[1030].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1031 GTM_gtm_cls3.MCS3_MEM[1031].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1032 GTM_gtm_cls3.MCS3_MEM[1032].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1033 GTM_gtm_cls3.MCS3_MEM[1033].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1034 GTM_gtm_cls3.MCS3_MEM[1034].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1035 GTM_gtm_cls3.MCS3_MEM[1035].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1036 GTM_gtm_cls3.MCS3_MEM[1036].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1037 GTM_gtm_cls3.MCS3_MEM[1037].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1038 GTM_gtm_cls3.MCS3_MEM[1038].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1039 GTM_gtm_cls3.MCS3_MEM[1039].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1040 GTM_gtm_cls3.MCS3_MEM[1040].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1041 GTM_gtm_cls3.MCS3_MEM[1041].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1042 GTM_gtm_cls3.MCS3_MEM[1042].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1043 GTM_gtm_cls3.MCS3_MEM[1043].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1044 GTM_gtm_cls3.MCS3_MEM[1044].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1045 GTM_gtm_cls3.MCS3_MEM[1045].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1046 GTM_gtm_cls3.MCS3_MEM[1046].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1047 GTM_gtm_cls3.MCS3_MEM[1047].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1048 GTM_gtm_cls3.MCS3_MEM[1048].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1049 GTM_gtm_cls3.MCS3_MEM[1049].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1050 GTM_gtm_cls3.MCS3_MEM[1050].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1051 GTM_gtm_cls3.MCS3_MEM[1051].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1052 GTM_gtm_cls3.MCS3_MEM[1052].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1053 GTM_gtm_cls3.MCS3_MEM[1053].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1054 GTM_gtm_cls3.MCS3_MEM[1054].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1055 GTM_gtm_cls3.MCS3_MEM[1055].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1056 GTM_gtm_cls3.MCS3_MEM[1056].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1057 GTM_gtm_cls3.MCS3_MEM[1057].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1058 GTM_gtm_cls3.MCS3_MEM[1058].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1059 GTM_gtm_cls3.MCS3_MEM[1059].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1060 GTM_gtm_cls3.MCS3_MEM[1060].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1061 GTM_gtm_cls3.MCS3_MEM[1061].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1062 GTM_gtm_cls3.MCS3_MEM[1062].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1063 GTM_gtm_cls3.MCS3_MEM[1063].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1064 GTM_gtm_cls3.MCS3_MEM[1064].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1065 GTM_gtm_cls3.MCS3_MEM[1065].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1066 GTM_gtm_cls3.MCS3_MEM[1066].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1067 GTM_gtm_cls3.MCS3_MEM[1067].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1068 GTM_gtm_cls3.MCS3_MEM[1068].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1069 GTM_gtm_cls3.MCS3_MEM[1069].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1070 GTM_gtm_cls3.MCS3_MEM[1070].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1071 GTM_gtm_cls3.MCS3_MEM[1071].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1072 GTM_gtm_cls3.MCS3_MEM[1072].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1073 GTM_gtm_cls3.MCS3_MEM[1073].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1074 GTM_gtm_cls3.MCS3_MEM[1074].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1075 GTM_gtm_cls3.MCS3_MEM[1075].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1076 GTM_gtm_cls3.MCS3_MEM[1076].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1077 GTM_gtm_cls3.MCS3_MEM[1077].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1078 GTM_gtm_cls3.MCS3_MEM[1078].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1079 GTM_gtm_cls3.MCS3_MEM[1079].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1080 GTM_gtm_cls3.MCS3_MEM[1080].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1081 GTM_gtm_cls3.MCS3_MEM[1081].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1082 GTM_gtm_cls3.MCS3_MEM[1082].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1083 GTM_gtm_cls3.MCS3_MEM[1083].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1084 GTM_gtm_cls3.MCS3_MEM[1084].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1085 GTM_gtm_cls3.MCS3_MEM[1085].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1086 GTM_gtm_cls3.MCS3_MEM[1086].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1087 GTM_gtm_cls3.MCS3_MEM[1087].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1088 GTM_gtm_cls3.MCS3_MEM[1088].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1089 GTM_gtm_cls3.MCS3_MEM[1089].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1090 GTM_gtm_cls3.MCS3_MEM[1090].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1091 GTM_gtm_cls3.MCS3_MEM[1091].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1092 GTM_gtm_cls3.MCS3_MEM[1092].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1093 GTM_gtm_cls3.MCS3_MEM[1093].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1094 GTM_gtm_cls3.MCS3_MEM[1094].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1095 GTM_gtm_cls3.MCS3_MEM[1095].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1096 GTM_gtm_cls3.MCS3_MEM[1096].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1097 GTM_gtm_cls3.MCS3_MEM[1097].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1098 GTM_gtm_cls3.MCS3_MEM[1098].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1099 GTM_gtm_cls3.MCS3_MEM[1099].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1100 GTM_gtm_cls3.MCS3_MEM[1100].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1101 GTM_gtm_cls3.MCS3_MEM[1101].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1102 GTM_gtm_cls3.MCS3_MEM[1102].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1103 GTM_gtm_cls3.MCS3_MEM[1103].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1104 GTM_gtm_cls3.MCS3_MEM[1104].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1105 GTM_gtm_cls3.MCS3_MEM[1105].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1106 GTM_gtm_cls3.MCS3_MEM[1106].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1107 GTM_gtm_cls3.MCS3_MEM[1107].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1108 GTM_gtm_cls3.MCS3_MEM[1108].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1109 GTM_gtm_cls3.MCS3_MEM[1109].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1110 GTM_gtm_cls3.MCS3_MEM[1110].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1111 GTM_gtm_cls3.MCS3_MEM[1111].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1112 GTM_gtm_cls3.MCS3_MEM[1112].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1113 GTM_gtm_cls3.MCS3_MEM[1113].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1114 GTM_gtm_cls3.MCS3_MEM[1114].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1115 GTM_gtm_cls3.MCS3_MEM[1115].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1116 GTM_gtm_cls3.MCS3_MEM[1116].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1117 GTM_gtm_cls3.MCS3_MEM[1117].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1118 GTM_gtm_cls3.MCS3_MEM[1118].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1119 GTM_gtm_cls3.MCS3_MEM[1119].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1120 GTM_gtm_cls3.MCS3_MEM[1120].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1121 GTM_gtm_cls3.MCS3_MEM[1121].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1122 GTM_gtm_cls3.MCS3_MEM[1122].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1123 GTM_gtm_cls3.MCS3_MEM[1123].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1124 GTM_gtm_cls3.MCS3_MEM[1124].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1125 GTM_gtm_cls3.MCS3_MEM[1125].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1126 GTM_gtm_cls3.MCS3_MEM[1126].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1127 GTM_gtm_cls3.MCS3_MEM[1127].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1128 GTM_gtm_cls3.MCS3_MEM[1128].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1129 GTM_gtm_cls3.MCS3_MEM[1129].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1130 GTM_gtm_cls3.MCS3_MEM[1130].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1131 GTM_gtm_cls3.MCS3_MEM[1131].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1132 GTM_gtm_cls3.MCS3_MEM[1132].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1133 GTM_gtm_cls3.MCS3_MEM[1133].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1134 GTM_gtm_cls3.MCS3_MEM[1134].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1135 GTM_gtm_cls3.MCS3_MEM[1135].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1136 GTM_gtm_cls3.MCS3_MEM[1136].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1137 GTM_gtm_cls3.MCS3_MEM[1137].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1138 GTM_gtm_cls3.MCS3_MEM[1138].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1139 GTM_gtm_cls3.MCS3_MEM[1139].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1140 GTM_gtm_cls3.MCS3_MEM[1140].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1141 GTM_gtm_cls3.MCS3_MEM[1141].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1142 GTM_gtm_cls3.MCS3_MEM[1142].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1143 GTM_gtm_cls3.MCS3_MEM[1143].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1144 GTM_gtm_cls3.MCS3_MEM[1144].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1145 GTM_gtm_cls3.MCS3_MEM[1145].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1146 GTM_gtm_cls3.MCS3_MEM[1146].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1147 GTM_gtm_cls3.MCS3_MEM[1147].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1148 GTM_gtm_cls3.MCS3_MEM[1148].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1149 GTM_gtm_cls3.MCS3_MEM[1149].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1150 GTM_gtm_cls3.MCS3_MEM[1150].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1151 GTM_gtm_cls3.MCS3_MEM[1151].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1152 GTM_gtm_cls3.MCS3_MEM[1152].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1153 GTM_gtm_cls3.MCS3_MEM[1153].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1154 GTM_gtm_cls3.MCS3_MEM[1154].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1155 GTM_gtm_cls3.MCS3_MEM[1155].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1156 GTM_gtm_cls3.MCS3_MEM[1156].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1157 GTM_gtm_cls3.MCS3_MEM[1157].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1158 GTM_gtm_cls3.MCS3_MEM[1158].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1159 GTM_gtm_cls3.MCS3_MEM[1159].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1160 GTM_gtm_cls3.MCS3_MEM[1160].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1161 GTM_gtm_cls3.MCS3_MEM[1161].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1162 GTM_gtm_cls3.MCS3_MEM[1162].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1163 GTM_gtm_cls3.MCS3_MEM[1163].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1164 GTM_gtm_cls3.MCS3_MEM[1164].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1165 GTM_gtm_cls3.MCS3_MEM[1165].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1166 GTM_gtm_cls3.MCS3_MEM[1166].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1167 GTM_gtm_cls3.MCS3_MEM[1167].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1168 GTM_gtm_cls3.MCS3_MEM[1168].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1169 GTM_gtm_cls3.MCS3_MEM[1169].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1170 GTM_gtm_cls3.MCS3_MEM[1170].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1171 GTM_gtm_cls3.MCS3_MEM[1171].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1172 GTM_gtm_cls3.MCS3_MEM[1172].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1173 GTM_gtm_cls3.MCS3_MEM[1173].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1174 GTM_gtm_cls3.MCS3_MEM[1174].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1175 GTM_gtm_cls3.MCS3_MEM[1175].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1176 GTM_gtm_cls3.MCS3_MEM[1176].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1177 GTM_gtm_cls3.MCS3_MEM[1177].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1178 GTM_gtm_cls3.MCS3_MEM[1178].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1179 GTM_gtm_cls3.MCS3_MEM[1179].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1180 GTM_gtm_cls3.MCS3_MEM[1180].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1181 GTM_gtm_cls3.MCS3_MEM[1181].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1182 GTM_gtm_cls3.MCS3_MEM[1182].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1183 GTM_gtm_cls3.MCS3_MEM[1183].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1184 GTM_gtm_cls3.MCS3_MEM[1184].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1185 GTM_gtm_cls3.MCS3_MEM[1185].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1186 GTM_gtm_cls3.MCS3_MEM[1186].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1187 GTM_gtm_cls3.MCS3_MEM[1187].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1188 GTM_gtm_cls3.MCS3_MEM[1188].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1189 GTM_gtm_cls3.MCS3_MEM[1189].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1190 GTM_gtm_cls3.MCS3_MEM[1190].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1191 GTM_gtm_cls3.MCS3_MEM[1191].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1192 GTM_gtm_cls3.MCS3_MEM[1192].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1193 GTM_gtm_cls3.MCS3_MEM[1193].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1194 GTM_gtm_cls3.MCS3_MEM[1194].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1195 GTM_gtm_cls3.MCS3_MEM[1195].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1196 GTM_gtm_cls3.MCS3_MEM[1196].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1197 GTM_gtm_cls3.MCS3_MEM[1197].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1198 GTM_gtm_cls3.MCS3_MEM[1198].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1199 GTM_gtm_cls3.MCS3_MEM[1199].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1200 GTM_gtm_cls3.MCS3_MEM[1200].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1201 GTM_gtm_cls3.MCS3_MEM[1201].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1202 GTM_gtm_cls3.MCS3_MEM[1202].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1203 GTM_gtm_cls3.MCS3_MEM[1203].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1204 GTM_gtm_cls3.MCS3_MEM[1204].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1205 GTM_gtm_cls3.MCS3_MEM[1205].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1206 GTM_gtm_cls3.MCS3_MEM[1206].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1207 GTM_gtm_cls3.MCS3_MEM[1207].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1208 GTM_gtm_cls3.MCS3_MEM[1208].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1209 GTM_gtm_cls3.MCS3_MEM[1209].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1210 GTM_gtm_cls3.MCS3_MEM[1210].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1211 GTM_gtm_cls3.MCS3_MEM[1211].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1212 GTM_gtm_cls3.MCS3_MEM[1212].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1213 GTM_gtm_cls3.MCS3_MEM[1213].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1214 GTM_gtm_cls3.MCS3_MEM[1214].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1215 GTM_gtm_cls3.MCS3_MEM[1215].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1216 GTM_gtm_cls3.MCS3_MEM[1216].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1217 GTM_gtm_cls3.MCS3_MEM[1217].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1218 GTM_gtm_cls3.MCS3_MEM[1218].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1219 GTM_gtm_cls3.MCS3_MEM[1219].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1220 GTM_gtm_cls3.MCS3_MEM[1220].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1221 GTM_gtm_cls3.MCS3_MEM[1221].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1222 GTM_gtm_cls3.MCS3_MEM[1222].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1223 GTM_gtm_cls3.MCS3_MEM[1223].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1224 GTM_gtm_cls3.MCS3_MEM[1224].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1225 GTM_gtm_cls3.MCS3_MEM[1225].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1226 GTM_gtm_cls3.MCS3_MEM[1226].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1227 GTM_gtm_cls3.MCS3_MEM[1227].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1228 GTM_gtm_cls3.MCS3_MEM[1228].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1229 GTM_gtm_cls3.MCS3_MEM[1229].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1230 GTM_gtm_cls3.MCS3_MEM[1230].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1231 GTM_gtm_cls3.MCS3_MEM[1231].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1232 GTM_gtm_cls3.MCS3_MEM[1232].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1233 GTM_gtm_cls3.MCS3_MEM[1233].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1234 GTM_gtm_cls3.MCS3_MEM[1234].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1235 GTM_gtm_cls3.MCS3_MEM[1235].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1236 GTM_gtm_cls3.MCS3_MEM[1236].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1237 GTM_gtm_cls3.MCS3_MEM[1237].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1238 GTM_gtm_cls3.MCS3_MEM[1238].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1239 GTM_gtm_cls3.MCS3_MEM[1239].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1240 GTM_gtm_cls3.MCS3_MEM[1240].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1241 GTM_gtm_cls3.MCS3_MEM[1241].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1242 GTM_gtm_cls3.MCS3_MEM[1242].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1243 GTM_gtm_cls3.MCS3_MEM[1243].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1244 GTM_gtm_cls3.MCS3_MEM[1244].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1245 GTM_gtm_cls3.MCS3_MEM[1245].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1246 GTM_gtm_cls3.MCS3_MEM[1246].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1247 GTM_gtm_cls3.MCS3_MEM[1247].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1248 GTM_gtm_cls3.MCS3_MEM[1248].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1249 GTM_gtm_cls3.MCS3_MEM[1249].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1250 GTM_gtm_cls3.MCS3_MEM[1250].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1251 GTM_gtm_cls3.MCS3_MEM[1251].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1252 GTM_gtm_cls3.MCS3_MEM[1252].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1253 GTM_gtm_cls3.MCS3_MEM[1253].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1254 GTM_gtm_cls3.MCS3_MEM[1254].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1255 GTM_gtm_cls3.MCS3_MEM[1255].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1256 GTM_gtm_cls3.MCS3_MEM[1256].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1257 GTM_gtm_cls3.MCS3_MEM[1257].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1258 GTM_gtm_cls3.MCS3_MEM[1258].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1259 GTM_gtm_cls3.MCS3_MEM[1259].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1260 GTM_gtm_cls3.MCS3_MEM[1260].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1261 GTM_gtm_cls3.MCS3_MEM[1261].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1262 GTM_gtm_cls3.MCS3_MEM[1262].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1263 GTM_gtm_cls3.MCS3_MEM[1263].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1264 GTM_gtm_cls3.MCS3_MEM[1264].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1265 GTM_gtm_cls3.MCS3_MEM[1265].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1266 GTM_gtm_cls3.MCS3_MEM[1266].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1267 GTM_gtm_cls3.MCS3_MEM[1267].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1268 GTM_gtm_cls3.MCS3_MEM[1268].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1269 GTM_gtm_cls3.MCS3_MEM[1269].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1270 GTM_gtm_cls3.MCS3_MEM[1270].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1271 GTM_gtm_cls3.MCS3_MEM[1271].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1272 GTM_gtm_cls3.MCS3_MEM[1272].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1273 GTM_gtm_cls3.MCS3_MEM[1273].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1274 GTM_gtm_cls3.MCS3_MEM[1274].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1275 GTM_gtm_cls3.MCS3_MEM[1275].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1276 GTM_gtm_cls3.MCS3_MEM[1276].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1277 GTM_gtm_cls3.MCS3_MEM[1277].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1278 GTM_gtm_cls3.MCS3_MEM[1278].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1279 GTM_gtm_cls3.MCS3_MEM[1279].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1280 GTM_gtm_cls3.MCS3_MEM[1280].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1281 GTM_gtm_cls3.MCS3_MEM[1281].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1282 GTM_gtm_cls3.MCS3_MEM[1282].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1283 GTM_gtm_cls3.MCS3_MEM[1283].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1284 GTM_gtm_cls3.MCS3_MEM[1284].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1285 GTM_gtm_cls3.MCS3_MEM[1285].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1286 GTM_gtm_cls3.MCS3_MEM[1286].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1287 GTM_gtm_cls3.MCS3_MEM[1287].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1288 GTM_gtm_cls3.MCS3_MEM[1288].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1289 GTM_gtm_cls3.MCS3_MEM[1289].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1290 GTM_gtm_cls3.MCS3_MEM[1290].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1291 GTM_gtm_cls3.MCS3_MEM[1291].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1292 GTM_gtm_cls3.MCS3_MEM[1292].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1293 GTM_gtm_cls3.MCS3_MEM[1293].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1294 GTM_gtm_cls3.MCS3_MEM[1294].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1295 GTM_gtm_cls3.MCS3_MEM[1295].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1296 GTM_gtm_cls3.MCS3_MEM[1296].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1297 GTM_gtm_cls3.MCS3_MEM[1297].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1298 GTM_gtm_cls3.MCS3_MEM[1298].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1299 GTM_gtm_cls3.MCS3_MEM[1299].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1300 GTM_gtm_cls3.MCS3_MEM[1300].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1301 GTM_gtm_cls3.MCS3_MEM[1301].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1302 GTM_gtm_cls3.MCS3_MEM[1302].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1303 GTM_gtm_cls3.MCS3_MEM[1303].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1304 GTM_gtm_cls3.MCS3_MEM[1304].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1305 GTM_gtm_cls3.MCS3_MEM[1305].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1306 GTM_gtm_cls3.MCS3_MEM[1306].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1307 GTM_gtm_cls3.MCS3_MEM[1307].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1308 GTM_gtm_cls3.MCS3_MEM[1308].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1309 GTM_gtm_cls3.MCS3_MEM[1309].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1310 GTM_gtm_cls3.MCS3_MEM[1310].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1311 GTM_gtm_cls3.MCS3_MEM[1311].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1312 GTM_gtm_cls3.MCS3_MEM[1312].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1313 GTM_gtm_cls3.MCS3_MEM[1313].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1314 GTM_gtm_cls3.MCS3_MEM[1314].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1315 GTM_gtm_cls3.MCS3_MEM[1315].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1316 GTM_gtm_cls3.MCS3_MEM[1316].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1317 GTM_gtm_cls3.MCS3_MEM[1317].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1318 GTM_gtm_cls3.MCS3_MEM[1318].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1319 GTM_gtm_cls3.MCS3_MEM[1319].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1320 GTM_gtm_cls3.MCS3_MEM[1320].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1321 GTM_gtm_cls3.MCS3_MEM[1321].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1322 GTM_gtm_cls3.MCS3_MEM[1322].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1323 GTM_gtm_cls3.MCS3_MEM[1323].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1324 GTM_gtm_cls3.MCS3_MEM[1324].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1325 GTM_gtm_cls3.MCS3_MEM[1325].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1326 GTM_gtm_cls3.MCS3_MEM[1326].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1327 GTM_gtm_cls3.MCS3_MEM[1327].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1328 GTM_gtm_cls3.MCS3_MEM[1328].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1329 GTM_gtm_cls3.MCS3_MEM[1329].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1330 GTM_gtm_cls3.MCS3_MEM[1330].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1331 GTM_gtm_cls3.MCS3_MEM[1331].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1332 GTM_gtm_cls3.MCS3_MEM[1332].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1333 GTM_gtm_cls3.MCS3_MEM[1333].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1334 GTM_gtm_cls3.MCS3_MEM[1334].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1335 GTM_gtm_cls3.MCS3_MEM[1335].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1336 GTM_gtm_cls3.MCS3_MEM[1336].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1337 GTM_gtm_cls3.MCS3_MEM[1337].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1338 GTM_gtm_cls3.MCS3_MEM[1338].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1339 GTM_gtm_cls3.MCS3_MEM[1339].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1340 GTM_gtm_cls3.MCS3_MEM[1340].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1341 GTM_gtm_cls3.MCS3_MEM[1341].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1342 GTM_gtm_cls3.MCS3_MEM[1342].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1343 GTM_gtm_cls3.MCS3_MEM[1343].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1344 GTM_gtm_cls3.MCS3_MEM[1344].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1345 GTM_gtm_cls3.MCS3_MEM[1345].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1346 GTM_gtm_cls3.MCS3_MEM[1346].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1347 GTM_gtm_cls3.MCS3_MEM[1347].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1348 GTM_gtm_cls3.MCS3_MEM[1348].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1349 GTM_gtm_cls3.MCS3_MEM[1349].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1350 GTM_gtm_cls3.MCS3_MEM[1350].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1351 GTM_gtm_cls3.MCS3_MEM[1351].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1352 GTM_gtm_cls3.MCS3_MEM[1352].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1353 GTM_gtm_cls3.MCS3_MEM[1353].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1354 GTM_gtm_cls3.MCS3_MEM[1354].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1355 GTM_gtm_cls3.MCS3_MEM[1355].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1356 GTM_gtm_cls3.MCS3_MEM[1356].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1357 GTM_gtm_cls3.MCS3_MEM[1357].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1358 GTM_gtm_cls3.MCS3_MEM[1358].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1359 GTM_gtm_cls3.MCS3_MEM[1359].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1360 GTM_gtm_cls3.MCS3_MEM[1360].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1361 GTM_gtm_cls3.MCS3_MEM[1361].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1362 GTM_gtm_cls3.MCS3_MEM[1362].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1363 GTM_gtm_cls3.MCS3_MEM[1363].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1364 GTM_gtm_cls3.MCS3_MEM[1364].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1365 GTM_gtm_cls3.MCS3_MEM[1365].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1366 GTM_gtm_cls3.MCS3_MEM[1366].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1367 GTM_gtm_cls3.MCS3_MEM[1367].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1368 GTM_gtm_cls3.MCS3_MEM[1368].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1369 GTM_gtm_cls3.MCS3_MEM[1369].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1370 GTM_gtm_cls3.MCS3_MEM[1370].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1371 GTM_gtm_cls3.MCS3_MEM[1371].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1372 GTM_gtm_cls3.MCS3_MEM[1372].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1373 GTM_gtm_cls3.MCS3_MEM[1373].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1374 GTM_gtm_cls3.MCS3_MEM[1374].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1375 GTM_gtm_cls3.MCS3_MEM[1375].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1376 GTM_gtm_cls3.MCS3_MEM[1376].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1377 GTM_gtm_cls3.MCS3_MEM[1377].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1378 GTM_gtm_cls3.MCS3_MEM[1378].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1379 GTM_gtm_cls3.MCS3_MEM[1379].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1380 GTM_gtm_cls3.MCS3_MEM[1380].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1381 GTM_gtm_cls3.MCS3_MEM[1381].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1382 GTM_gtm_cls3.MCS3_MEM[1382].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1383 GTM_gtm_cls3.MCS3_MEM[1383].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1384 GTM_gtm_cls3.MCS3_MEM[1384].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1385 GTM_gtm_cls3.MCS3_MEM[1385].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1386 GTM_gtm_cls3.MCS3_MEM[1386].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1387 GTM_gtm_cls3.MCS3_MEM[1387].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1388 GTM_gtm_cls3.MCS3_MEM[1388].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1389 GTM_gtm_cls3.MCS3_MEM[1389].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1390 GTM_gtm_cls3.MCS3_MEM[1390].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1391 GTM_gtm_cls3.MCS3_MEM[1391].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1392 GTM_gtm_cls3.MCS3_MEM[1392].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1393 GTM_gtm_cls3.MCS3_MEM[1393].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1394 GTM_gtm_cls3.MCS3_MEM[1394].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1395 GTM_gtm_cls3.MCS3_MEM[1395].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1396 GTM_gtm_cls3.MCS3_MEM[1396].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1397 GTM_gtm_cls3.MCS3_MEM[1397].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1398 GTM_gtm_cls3.MCS3_MEM[1398].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1399 GTM_gtm_cls3.MCS3_MEM[1399].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1400 GTM_gtm_cls3.MCS3_MEM[1400].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1401 GTM_gtm_cls3.MCS3_MEM[1401].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1402 GTM_gtm_cls3.MCS3_MEM[1402].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1403 GTM_gtm_cls3.MCS3_MEM[1403].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1404 GTM_gtm_cls3.MCS3_MEM[1404].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1405 GTM_gtm_cls3.MCS3_MEM[1405].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1406 GTM_gtm_cls3.MCS3_MEM[1406].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1407 GTM_gtm_cls3.MCS3_MEM[1407].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1408 GTM_gtm_cls3.MCS3_MEM[1408].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1409 GTM_gtm_cls3.MCS3_MEM[1409].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1410 GTM_gtm_cls3.MCS3_MEM[1410].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1411 GTM_gtm_cls3.MCS3_MEM[1411].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1412 GTM_gtm_cls3.MCS3_MEM[1412].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1413 GTM_gtm_cls3.MCS3_MEM[1413].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1414 GTM_gtm_cls3.MCS3_MEM[1414].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1415 GTM_gtm_cls3.MCS3_MEM[1415].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1416 GTM_gtm_cls3.MCS3_MEM[1416].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1417 GTM_gtm_cls3.MCS3_MEM[1417].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1418 GTM_gtm_cls3.MCS3_MEM[1418].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1419 GTM_gtm_cls3.MCS3_MEM[1419].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1420 GTM_gtm_cls3.MCS3_MEM[1420].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1421 GTM_gtm_cls3.MCS3_MEM[1421].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1422 GTM_gtm_cls3.MCS3_MEM[1422].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1423 GTM_gtm_cls3.MCS3_MEM[1423].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1424 GTM_gtm_cls3.MCS3_MEM[1424].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1425 GTM_gtm_cls3.MCS3_MEM[1425].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1426 GTM_gtm_cls3.MCS3_MEM[1426].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1427 GTM_gtm_cls3.MCS3_MEM[1427].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1428 GTM_gtm_cls3.MCS3_MEM[1428].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1429 GTM_gtm_cls3.MCS3_MEM[1429].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1430 GTM_gtm_cls3.MCS3_MEM[1430].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1431 GTM_gtm_cls3.MCS3_MEM[1431].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1432 GTM_gtm_cls3.MCS3_MEM[1432].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1433 GTM_gtm_cls3.MCS3_MEM[1433].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1434 GTM_gtm_cls3.MCS3_MEM[1434].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1435 GTM_gtm_cls3.MCS3_MEM[1435].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1436 GTM_gtm_cls3.MCS3_MEM[1436].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1437 GTM_gtm_cls3.MCS3_MEM[1437].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1438 GTM_gtm_cls3.MCS3_MEM[1438].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1439 GTM_gtm_cls3.MCS3_MEM[1439].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1440 GTM_gtm_cls3.MCS3_MEM[1440].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1441 GTM_gtm_cls3.MCS3_MEM[1441].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1442 GTM_gtm_cls3.MCS3_MEM[1442].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1443 GTM_gtm_cls3.MCS3_MEM[1443].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1444 GTM_gtm_cls3.MCS3_MEM[1444].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1445 GTM_gtm_cls3.MCS3_MEM[1445].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1446 GTM_gtm_cls3.MCS3_MEM[1446].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1447 GTM_gtm_cls3.MCS3_MEM[1447].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1448 GTM_gtm_cls3.MCS3_MEM[1448].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1449 GTM_gtm_cls3.MCS3_MEM[1449].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1450 GTM_gtm_cls3.MCS3_MEM[1450].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1451 GTM_gtm_cls3.MCS3_MEM[1451].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1452 GTM_gtm_cls3.MCS3_MEM[1452].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1453 GTM_gtm_cls3.MCS3_MEM[1453].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1454 GTM_gtm_cls3.MCS3_MEM[1454].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1455 GTM_gtm_cls3.MCS3_MEM[1455].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1456 GTM_gtm_cls3.MCS3_MEM[1456].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1457 GTM_gtm_cls3.MCS3_MEM[1457].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1458 GTM_gtm_cls3.MCS3_MEM[1458].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1459 GTM_gtm_cls3.MCS3_MEM[1459].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1460 GTM_gtm_cls3.MCS3_MEM[1460].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1461 GTM_gtm_cls3.MCS3_MEM[1461].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1462 GTM_gtm_cls3.MCS3_MEM[1462].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1463 GTM_gtm_cls3.MCS3_MEM[1463].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1464 GTM_gtm_cls3.MCS3_MEM[1464].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1465 GTM_gtm_cls3.MCS3_MEM[1465].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1466 GTM_gtm_cls3.MCS3_MEM[1466].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1467 GTM_gtm_cls3.MCS3_MEM[1467].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1468 GTM_gtm_cls3.MCS3_MEM[1468].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1469 GTM_gtm_cls3.MCS3_MEM[1469].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1470 GTM_gtm_cls3.MCS3_MEM[1470].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1471 GTM_gtm_cls3.MCS3_MEM[1471].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1472 GTM_gtm_cls3.MCS3_MEM[1472].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1473 GTM_gtm_cls3.MCS3_MEM[1473].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1474 GTM_gtm_cls3.MCS3_MEM[1474].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1475 GTM_gtm_cls3.MCS3_MEM[1475].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1476 GTM_gtm_cls3.MCS3_MEM[1476].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1477 GTM_gtm_cls3.MCS3_MEM[1477].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1478 GTM_gtm_cls3.MCS3_MEM[1478].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1479 GTM_gtm_cls3.MCS3_MEM[1479].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1480 GTM_gtm_cls3.MCS3_MEM[1480].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1481 GTM_gtm_cls3.MCS3_MEM[1481].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1482 GTM_gtm_cls3.MCS3_MEM[1482].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1483 GTM_gtm_cls3.MCS3_MEM[1483].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1484 GTM_gtm_cls3.MCS3_MEM[1484].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1485 GTM_gtm_cls3.MCS3_MEM[1485].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1486 GTM_gtm_cls3.MCS3_MEM[1486].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1487 GTM_gtm_cls3.MCS3_MEM[1487].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1488 GTM_gtm_cls3.MCS3_MEM[1488].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1489 GTM_gtm_cls3.MCS3_MEM[1489].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1490 GTM_gtm_cls3.MCS3_MEM[1490].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1491 GTM_gtm_cls3.MCS3_MEM[1491].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1492 GTM_gtm_cls3.MCS3_MEM[1492].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1493 GTM_gtm_cls3.MCS3_MEM[1493].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1494 GTM_gtm_cls3.MCS3_MEM[1494].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1495 GTM_gtm_cls3.MCS3_MEM[1495].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1496 GTM_gtm_cls3.MCS3_MEM[1496].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1497 GTM_gtm_cls3.MCS3_MEM[1497].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1498 GTM_gtm_cls3.MCS3_MEM[1498].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1499 GTM_gtm_cls3.MCS3_MEM[1499].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1500 GTM_gtm_cls3.MCS3_MEM[1500].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1501 GTM_gtm_cls3.MCS3_MEM[1501].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1502 GTM_gtm_cls3.MCS3_MEM[1502].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1503 GTM_gtm_cls3.MCS3_MEM[1503].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1504 GTM_gtm_cls3.MCS3_MEM[1504].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1505 GTM_gtm_cls3.MCS3_MEM[1505].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1506 GTM_gtm_cls3.MCS3_MEM[1506].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1507 GTM_gtm_cls3.MCS3_MEM[1507].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1508 GTM_gtm_cls3.MCS3_MEM[1508].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1509 GTM_gtm_cls3.MCS3_MEM[1509].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1510 GTM_gtm_cls3.MCS3_MEM[1510].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1511 GTM_gtm_cls3.MCS3_MEM[1511].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1512 GTM_gtm_cls3.MCS3_MEM[1512].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1513 GTM_gtm_cls3.MCS3_MEM[1513].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1514 GTM_gtm_cls3.MCS3_MEM[1514].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1515 GTM_gtm_cls3.MCS3_MEM[1515].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1516 GTM_gtm_cls3.MCS3_MEM[1516].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1517 GTM_gtm_cls3.MCS3_MEM[1517].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1518 GTM_gtm_cls3.MCS3_MEM[1518].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1519 GTM_gtm_cls3.MCS3_MEM[1519].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1520 GTM_gtm_cls3.MCS3_MEM[1520].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1521 GTM_gtm_cls3.MCS3_MEM[1521].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1522 GTM_gtm_cls3.MCS3_MEM[1522].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1523 GTM_gtm_cls3.MCS3_MEM[1523].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1524 GTM_gtm_cls3.MCS3_MEM[1524].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1525 GTM_gtm_cls3.MCS3_MEM[1525].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1526 GTM_gtm_cls3.MCS3_MEM[1526].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1527 GTM_gtm_cls3.MCS3_MEM[1527].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1528 GTM_gtm_cls3.MCS3_MEM[1528].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1529 GTM_gtm_cls3.MCS3_MEM[1529].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1530 GTM_gtm_cls3.MCS3_MEM[1530].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1531 GTM_gtm_cls3.MCS3_MEM[1531].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1532 GTM_gtm_cls3.MCS3_MEM[1532].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1533 GTM_gtm_cls3.MCS3_MEM[1533].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1534 GTM_gtm_cls3.MCS3_MEM[1534].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1535 GTM_gtm_cls3.MCS3_MEM[1535].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1536 GTM_gtm_cls3.MCS3_MEM[1536].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1537 GTM_gtm_cls3.MCS3_MEM[1537].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1538 GTM_gtm_cls3.MCS3_MEM[1538].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1539 GTM_gtm_cls3.MCS3_MEM[1539].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1540 GTM_gtm_cls3.MCS3_MEM[1540].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1541 GTM_gtm_cls3.MCS3_MEM[1541].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1542 GTM_gtm_cls3.MCS3_MEM[1542].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1543 GTM_gtm_cls3.MCS3_MEM[1543].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1544 GTM_gtm_cls3.MCS3_MEM[1544].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1545 GTM_gtm_cls3.MCS3_MEM[1545].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1546 GTM_gtm_cls3.MCS3_MEM[1546].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1547 GTM_gtm_cls3.MCS3_MEM[1547].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1548 GTM_gtm_cls3.MCS3_MEM[1548].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1549 GTM_gtm_cls3.MCS3_MEM[1549].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1550 GTM_gtm_cls3.MCS3_MEM[1550].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1551 GTM_gtm_cls3.MCS3_MEM[1551].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1552 GTM_gtm_cls3.MCS3_MEM[1552].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1553 GTM_gtm_cls3.MCS3_MEM[1553].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1554 GTM_gtm_cls3.MCS3_MEM[1554].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1555 GTM_gtm_cls3.MCS3_MEM[1555].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1556 GTM_gtm_cls3.MCS3_MEM[1556].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1557 GTM_gtm_cls3.MCS3_MEM[1557].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1558 GTM_gtm_cls3.MCS3_MEM[1558].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1559 GTM_gtm_cls3.MCS3_MEM[1559].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1560 GTM_gtm_cls3.MCS3_MEM[1560].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1561 GTM_gtm_cls3.MCS3_MEM[1561].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1562 GTM_gtm_cls3.MCS3_MEM[1562].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1563 GTM_gtm_cls3.MCS3_MEM[1563].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1564 GTM_gtm_cls3.MCS3_MEM[1564].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1565 GTM_gtm_cls3.MCS3_MEM[1565].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1566 GTM_gtm_cls3.MCS3_MEM[1566].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1567 GTM_gtm_cls3.MCS3_MEM[1567].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1568 GTM_gtm_cls3.MCS3_MEM[1568].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1569 GTM_gtm_cls3.MCS3_MEM[1569].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1570 GTM_gtm_cls3.MCS3_MEM[1570].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1571 GTM_gtm_cls3.MCS3_MEM[1571].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1572 GTM_gtm_cls3.MCS3_MEM[1572].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1573 GTM_gtm_cls3.MCS3_MEM[1573].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1574 GTM_gtm_cls3.MCS3_MEM[1574].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1575 GTM_gtm_cls3.MCS3_MEM[1575].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1576 GTM_gtm_cls3.MCS3_MEM[1576].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1577 GTM_gtm_cls3.MCS3_MEM[1577].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1578 GTM_gtm_cls3.MCS3_MEM[1578].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1579 GTM_gtm_cls3.MCS3_MEM[1579].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1580 GTM_gtm_cls3.MCS3_MEM[1580].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1581 GTM_gtm_cls3.MCS3_MEM[1581].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1582 GTM_gtm_cls3.MCS3_MEM[1582].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1583 GTM_gtm_cls3.MCS3_MEM[1583].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1584 GTM_gtm_cls3.MCS3_MEM[1584].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1585 GTM_gtm_cls3.MCS3_MEM[1585].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1586 GTM_gtm_cls3.MCS3_MEM[1586].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1587 GTM_gtm_cls3.MCS3_MEM[1587].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1588 GTM_gtm_cls3.MCS3_MEM[1588].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1589 GTM_gtm_cls3.MCS3_MEM[1589].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1590 GTM_gtm_cls3.MCS3_MEM[1590].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1591 GTM_gtm_cls3.MCS3_MEM[1591].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1592 GTM_gtm_cls3.MCS3_MEM[1592].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1593 GTM_gtm_cls3.MCS3_MEM[1593].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1594 GTM_gtm_cls3.MCS3_MEM[1594].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1595 GTM_gtm_cls3.MCS3_MEM[1595].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1596 GTM_gtm_cls3.MCS3_MEM[1596].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1597 GTM_gtm_cls3.MCS3_MEM[1597].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1598 GTM_gtm_cls3.MCS3_MEM[1598].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1599 GTM_gtm_cls3.MCS3_MEM[1599].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1600 GTM_gtm_cls3.MCS3_MEM[1600].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1601 GTM_gtm_cls3.MCS3_MEM[1601].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1602 GTM_gtm_cls3.MCS3_MEM[1602].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1603 GTM_gtm_cls3.MCS3_MEM[1603].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1604 GTM_gtm_cls3.MCS3_MEM[1604].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1605 GTM_gtm_cls3.MCS3_MEM[1605].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1606 GTM_gtm_cls3.MCS3_MEM[1606].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1607 GTM_gtm_cls3.MCS3_MEM[1607].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1608 GTM_gtm_cls3.MCS3_MEM[1608].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1609 GTM_gtm_cls3.MCS3_MEM[1609].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1610 GTM_gtm_cls3.MCS3_MEM[1610].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1611 GTM_gtm_cls3.MCS3_MEM[1611].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1612 GTM_gtm_cls3.MCS3_MEM[1612].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1613 GTM_gtm_cls3.MCS3_MEM[1613].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1614 GTM_gtm_cls3.MCS3_MEM[1614].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1615 GTM_gtm_cls3.MCS3_MEM[1615].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1616 GTM_gtm_cls3.MCS3_MEM[1616].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1617 GTM_gtm_cls3.MCS3_MEM[1617].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1618 GTM_gtm_cls3.MCS3_MEM[1618].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1619 GTM_gtm_cls3.MCS3_MEM[1619].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1620 GTM_gtm_cls3.MCS3_MEM[1620].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1621 GTM_gtm_cls3.MCS3_MEM[1621].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1622 GTM_gtm_cls3.MCS3_MEM[1622].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1623 GTM_gtm_cls3.MCS3_MEM[1623].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1624 GTM_gtm_cls3.MCS3_MEM[1624].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1625 GTM_gtm_cls3.MCS3_MEM[1625].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1626 GTM_gtm_cls3.MCS3_MEM[1626].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1627 GTM_gtm_cls3.MCS3_MEM[1627].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1628 GTM_gtm_cls3.MCS3_MEM[1628].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1629 GTM_gtm_cls3.MCS3_MEM[1629].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1630 GTM_gtm_cls3.MCS3_MEM[1630].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1631 GTM_gtm_cls3.MCS3_MEM[1631].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1632 GTM_gtm_cls3.MCS3_MEM[1632].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1633 GTM_gtm_cls3.MCS3_MEM[1633].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1634 GTM_gtm_cls3.MCS3_MEM[1634].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1635 GTM_gtm_cls3.MCS3_MEM[1635].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1636 GTM_gtm_cls3.MCS3_MEM[1636].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1637 GTM_gtm_cls3.MCS3_MEM[1637].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1638 GTM_gtm_cls3.MCS3_MEM[1638].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1639 GTM_gtm_cls3.MCS3_MEM[1639].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1640 GTM_gtm_cls3.MCS3_MEM[1640].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1641 GTM_gtm_cls3.MCS3_MEM[1641].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1642 GTM_gtm_cls3.MCS3_MEM[1642].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1643 GTM_gtm_cls3.MCS3_MEM[1643].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1644 GTM_gtm_cls3.MCS3_MEM[1644].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1645 GTM_gtm_cls3.MCS3_MEM[1645].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1646 GTM_gtm_cls3.MCS3_MEM[1646].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1647 GTM_gtm_cls3.MCS3_MEM[1647].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1648 GTM_gtm_cls3.MCS3_MEM[1648].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1649 GTM_gtm_cls3.MCS3_MEM[1649].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1650 GTM_gtm_cls3.MCS3_MEM[1650].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1651 GTM_gtm_cls3.MCS3_MEM[1651].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1652 GTM_gtm_cls3.MCS3_MEM[1652].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1653 GTM_gtm_cls3.MCS3_MEM[1653].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1654 GTM_gtm_cls3.MCS3_MEM[1654].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1655 GTM_gtm_cls3.MCS3_MEM[1655].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1656 GTM_gtm_cls3.MCS3_MEM[1656].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1657 GTM_gtm_cls3.MCS3_MEM[1657].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1658 GTM_gtm_cls3.MCS3_MEM[1658].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1659 GTM_gtm_cls3.MCS3_MEM[1659].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1660 GTM_gtm_cls3.MCS3_MEM[1660].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1661 GTM_gtm_cls3.MCS3_MEM[1661].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1662 GTM_gtm_cls3.MCS3_MEM[1662].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1663 GTM_gtm_cls3.MCS3_MEM[1663].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1664 GTM_gtm_cls3.MCS3_MEM[1664].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1665 GTM_gtm_cls3.MCS3_MEM[1665].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1666 GTM_gtm_cls3.MCS3_MEM[1666].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1667 GTM_gtm_cls3.MCS3_MEM[1667].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1668 GTM_gtm_cls3.MCS3_MEM[1668].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1669 GTM_gtm_cls3.MCS3_MEM[1669].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1670 GTM_gtm_cls3.MCS3_MEM[1670].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1671 GTM_gtm_cls3.MCS3_MEM[1671].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1672 GTM_gtm_cls3.MCS3_MEM[1672].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1673 GTM_gtm_cls3.MCS3_MEM[1673].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1674 GTM_gtm_cls3.MCS3_MEM[1674].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1675 GTM_gtm_cls3.MCS3_MEM[1675].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1676 GTM_gtm_cls3.MCS3_MEM[1676].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1677 GTM_gtm_cls3.MCS3_MEM[1677].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1678 GTM_gtm_cls3.MCS3_MEM[1678].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1679 GTM_gtm_cls3.MCS3_MEM[1679].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1680 GTM_gtm_cls3.MCS3_MEM[1680].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1681 GTM_gtm_cls3.MCS3_MEM[1681].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1682 GTM_gtm_cls3.MCS3_MEM[1682].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1683 GTM_gtm_cls3.MCS3_MEM[1683].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1684 GTM_gtm_cls3.MCS3_MEM[1684].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1685 GTM_gtm_cls3.MCS3_MEM[1685].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1686 GTM_gtm_cls3.MCS3_MEM[1686].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1687 GTM_gtm_cls3.MCS3_MEM[1687].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1688 GTM_gtm_cls3.MCS3_MEM[1688].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1689 GTM_gtm_cls3.MCS3_MEM[1689].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1690 GTM_gtm_cls3.MCS3_MEM[1690].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1691 GTM_gtm_cls3.MCS3_MEM[1691].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1692 GTM_gtm_cls3.MCS3_MEM[1692].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1693 GTM_gtm_cls3.MCS3_MEM[1693].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1694 GTM_gtm_cls3.MCS3_MEM[1694].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1695 GTM_gtm_cls3.MCS3_MEM[1695].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1696 GTM_gtm_cls3.MCS3_MEM[1696].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1697 GTM_gtm_cls3.MCS3_MEM[1697].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1698 GTM_gtm_cls3.MCS3_MEM[1698].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1699 GTM_gtm_cls3.MCS3_MEM[1699].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1700 GTM_gtm_cls3.MCS3_MEM[1700].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1701 GTM_gtm_cls3.MCS3_MEM[1701].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1702 GTM_gtm_cls3.MCS3_MEM[1702].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1703 GTM_gtm_cls3.MCS3_MEM[1703].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1704 GTM_gtm_cls3.MCS3_MEM[1704].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1705 GTM_gtm_cls3.MCS3_MEM[1705].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1706 GTM_gtm_cls3.MCS3_MEM[1706].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1707 GTM_gtm_cls3.MCS3_MEM[1707].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1708 GTM_gtm_cls3.MCS3_MEM[1708].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1709 GTM_gtm_cls3.MCS3_MEM[1709].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1710 GTM_gtm_cls3.MCS3_MEM[1710].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1711 GTM_gtm_cls3.MCS3_MEM[1711].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1712 GTM_gtm_cls3.MCS3_MEM[1712].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1713 GTM_gtm_cls3.MCS3_MEM[1713].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1714 GTM_gtm_cls3.MCS3_MEM[1714].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1715 GTM_gtm_cls3.MCS3_MEM[1715].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1716 GTM_gtm_cls3.MCS3_MEM[1716].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1717 GTM_gtm_cls3.MCS3_MEM[1717].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1718 GTM_gtm_cls3.MCS3_MEM[1718].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1719 GTM_gtm_cls3.MCS3_MEM[1719].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1720 GTM_gtm_cls3.MCS3_MEM[1720].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1721 GTM_gtm_cls3.MCS3_MEM[1721].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1722 GTM_gtm_cls3.MCS3_MEM[1722].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1723 GTM_gtm_cls3.MCS3_MEM[1723].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1724 GTM_gtm_cls3.MCS3_MEM[1724].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1725 GTM_gtm_cls3.MCS3_MEM[1725].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1726 GTM_gtm_cls3.MCS3_MEM[1726].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1727 GTM_gtm_cls3.MCS3_MEM[1727].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1728 GTM_gtm_cls3.MCS3_MEM[1728].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1729 GTM_gtm_cls3.MCS3_MEM[1729].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1730 GTM_gtm_cls3.MCS3_MEM[1730].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1731 GTM_gtm_cls3.MCS3_MEM[1731].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1732 GTM_gtm_cls3.MCS3_MEM[1732].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1733 GTM_gtm_cls3.MCS3_MEM[1733].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1734 GTM_gtm_cls3.MCS3_MEM[1734].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1735 GTM_gtm_cls3.MCS3_MEM[1735].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1736 GTM_gtm_cls3.MCS3_MEM[1736].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1737 GTM_gtm_cls3.MCS3_MEM[1737].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1738 GTM_gtm_cls3.MCS3_MEM[1738].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1739 GTM_gtm_cls3.MCS3_MEM[1739].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1740 GTM_gtm_cls3.MCS3_MEM[1740].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1741 GTM_gtm_cls3.MCS3_MEM[1741].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1742 GTM_gtm_cls3.MCS3_MEM[1742].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1743 GTM_gtm_cls3.MCS3_MEM[1743].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1744 GTM_gtm_cls3.MCS3_MEM[1744].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1745 GTM_gtm_cls3.MCS3_MEM[1745].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1746 GTM_gtm_cls3.MCS3_MEM[1746].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1747 GTM_gtm_cls3.MCS3_MEM[1747].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1748 GTM_gtm_cls3.MCS3_MEM[1748].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1749 GTM_gtm_cls3.MCS3_MEM[1749].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1750 GTM_gtm_cls3.MCS3_MEM[1750].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1751 GTM_gtm_cls3.MCS3_MEM[1751].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1752 GTM_gtm_cls3.MCS3_MEM[1752].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1753 GTM_gtm_cls3.MCS3_MEM[1753].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1754 GTM_gtm_cls3.MCS3_MEM[1754].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1755 GTM_gtm_cls3.MCS3_MEM[1755].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1756 GTM_gtm_cls3.MCS3_MEM[1756].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1757 GTM_gtm_cls3.MCS3_MEM[1757].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1758 GTM_gtm_cls3.MCS3_MEM[1758].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1759 GTM_gtm_cls3.MCS3_MEM[1759].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1760 GTM_gtm_cls3.MCS3_MEM[1760].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1761 GTM_gtm_cls3.MCS3_MEM[1761].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1762 GTM_gtm_cls3.MCS3_MEM[1762].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1763 GTM_gtm_cls3.MCS3_MEM[1763].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1764 GTM_gtm_cls3.MCS3_MEM[1764].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1765 GTM_gtm_cls3.MCS3_MEM[1765].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1766 GTM_gtm_cls3.MCS3_MEM[1766].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1767 GTM_gtm_cls3.MCS3_MEM[1767].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1768 GTM_gtm_cls3.MCS3_MEM[1768].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1769 GTM_gtm_cls3.MCS3_MEM[1769].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1770 GTM_gtm_cls3.MCS3_MEM[1770].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1771 GTM_gtm_cls3.MCS3_MEM[1771].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1772 GTM_gtm_cls3.MCS3_MEM[1772].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1773 GTM_gtm_cls3.MCS3_MEM[1773].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1774 GTM_gtm_cls3.MCS3_MEM[1774].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1775 GTM_gtm_cls3.MCS3_MEM[1775].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1776 GTM_gtm_cls3.MCS3_MEM[1776].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1777 GTM_gtm_cls3.MCS3_MEM[1777].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1778 GTM_gtm_cls3.MCS3_MEM[1778].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1779 GTM_gtm_cls3.MCS3_MEM[1779].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1780 GTM_gtm_cls3.MCS3_MEM[1780].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1781 GTM_gtm_cls3.MCS3_MEM[1781].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1782 GTM_gtm_cls3.MCS3_MEM[1782].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1783 GTM_gtm_cls3.MCS3_MEM[1783].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1784 GTM_gtm_cls3.MCS3_MEM[1784].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1785 GTM_gtm_cls3.MCS3_MEM[1785].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1786 GTM_gtm_cls3.MCS3_MEM[1786].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1787 GTM_gtm_cls3.MCS3_MEM[1787].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1788 GTM_gtm_cls3.MCS3_MEM[1788].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1789 GTM_gtm_cls3.MCS3_MEM[1789].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1790 GTM_gtm_cls3.MCS3_MEM[1790].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1791 GTM_gtm_cls3.MCS3_MEM[1791].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1792 GTM_gtm_cls3.MCS3_MEM[1792].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1793 GTM_gtm_cls3.MCS3_MEM[1793].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1794 GTM_gtm_cls3.MCS3_MEM[1794].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1795 GTM_gtm_cls3.MCS3_MEM[1795].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1796 GTM_gtm_cls3.MCS3_MEM[1796].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1797 GTM_gtm_cls3.MCS3_MEM[1797].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1798 GTM_gtm_cls3.MCS3_MEM[1798].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1799 GTM_gtm_cls3.MCS3_MEM[1799].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1800 GTM_gtm_cls3.MCS3_MEM[1800].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1801 GTM_gtm_cls3.MCS3_MEM[1801].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1802 GTM_gtm_cls3.MCS3_MEM[1802].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1803 GTM_gtm_cls3.MCS3_MEM[1803].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1804 GTM_gtm_cls3.MCS3_MEM[1804].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1805 GTM_gtm_cls3.MCS3_MEM[1805].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1806 GTM_gtm_cls3.MCS3_MEM[1806].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1807 GTM_gtm_cls3.MCS3_MEM[1807].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1808 GTM_gtm_cls3.MCS3_MEM[1808].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1809 GTM_gtm_cls3.MCS3_MEM[1809].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1810 GTM_gtm_cls3.MCS3_MEM[1810].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1811 GTM_gtm_cls3.MCS3_MEM[1811].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1812 GTM_gtm_cls3.MCS3_MEM[1812].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1813 GTM_gtm_cls3.MCS3_MEM[1813].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1814 GTM_gtm_cls3.MCS3_MEM[1814].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1815 GTM_gtm_cls3.MCS3_MEM[1815].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1816 GTM_gtm_cls3.MCS3_MEM[1816].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1817 GTM_gtm_cls3.MCS3_MEM[1817].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1818 GTM_gtm_cls3.MCS3_MEM[1818].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1819 GTM_gtm_cls3.MCS3_MEM[1819].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1820 GTM_gtm_cls3.MCS3_MEM[1820].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1821 GTM_gtm_cls3.MCS3_MEM[1821].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1822 GTM_gtm_cls3.MCS3_MEM[1822].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1823 GTM_gtm_cls3.MCS3_MEM[1823].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1824 GTM_gtm_cls3.MCS3_MEM[1824].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1825 GTM_gtm_cls3.MCS3_MEM[1825].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1826 GTM_gtm_cls3.MCS3_MEM[1826].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1827 GTM_gtm_cls3.MCS3_MEM[1827].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1828 GTM_gtm_cls3.MCS3_MEM[1828].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1829 GTM_gtm_cls3.MCS3_MEM[1829].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1830 GTM_gtm_cls3.MCS3_MEM[1830].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1831 GTM_gtm_cls3.MCS3_MEM[1831].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1832 GTM_gtm_cls3.MCS3_MEM[1832].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1833 GTM_gtm_cls3.MCS3_MEM[1833].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1834 GTM_gtm_cls3.MCS3_MEM[1834].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1835 GTM_gtm_cls3.MCS3_MEM[1835].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1836 GTM_gtm_cls3.MCS3_MEM[1836].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1837 GTM_gtm_cls3.MCS3_MEM[1837].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1838 GTM_gtm_cls3.MCS3_MEM[1838].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1839 GTM_gtm_cls3.MCS3_MEM[1839].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1840 GTM_gtm_cls3.MCS3_MEM[1840].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1841 GTM_gtm_cls3.MCS3_MEM[1841].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1842 GTM_gtm_cls3.MCS3_MEM[1842].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1843 GTM_gtm_cls3.MCS3_MEM[1843].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1844 GTM_gtm_cls3.MCS3_MEM[1844].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1845 GTM_gtm_cls3.MCS3_MEM[1845].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1846 GTM_gtm_cls3.MCS3_MEM[1846].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1847 GTM_gtm_cls3.MCS3_MEM[1847].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1848 GTM_gtm_cls3.MCS3_MEM[1848].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1849 GTM_gtm_cls3.MCS3_MEM[1849].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1850 GTM_gtm_cls3.MCS3_MEM[1850].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1851 GTM_gtm_cls3.MCS3_MEM[1851].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1852 GTM_gtm_cls3.MCS3_MEM[1852].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1853 GTM_gtm_cls3.MCS3_MEM[1853].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1854 GTM_gtm_cls3.MCS3_MEM[1854].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1855 GTM_gtm_cls3.MCS3_MEM[1855].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1856 GTM_gtm_cls3.MCS3_MEM[1856].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1857 GTM_gtm_cls3.MCS3_MEM[1857].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1858 GTM_gtm_cls3.MCS3_MEM[1858].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1859 GTM_gtm_cls3.MCS3_MEM[1859].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1860 GTM_gtm_cls3.MCS3_MEM[1860].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1861 GTM_gtm_cls3.MCS3_MEM[1861].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1862 GTM_gtm_cls3.MCS3_MEM[1862].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1863 GTM_gtm_cls3.MCS3_MEM[1863].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1864 GTM_gtm_cls3.MCS3_MEM[1864].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1865 GTM_gtm_cls3.MCS3_MEM[1865].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1866 GTM_gtm_cls3.MCS3_MEM[1866].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1867 GTM_gtm_cls3.MCS3_MEM[1867].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1868 GTM_gtm_cls3.MCS3_MEM[1868].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1869 GTM_gtm_cls3.MCS3_MEM[1869].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1870 GTM_gtm_cls3.MCS3_MEM[1870].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1871 GTM_gtm_cls3.MCS3_MEM[1871].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1872 GTM_gtm_cls3.MCS3_MEM[1872].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1873 GTM_gtm_cls3.MCS3_MEM[1873].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1874 GTM_gtm_cls3.MCS3_MEM[1874].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1875 GTM_gtm_cls3.MCS3_MEM[1875].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1876 GTM_gtm_cls3.MCS3_MEM[1876].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1877 GTM_gtm_cls3.MCS3_MEM[1877].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1878 GTM_gtm_cls3.MCS3_MEM[1878].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1879 GTM_gtm_cls3.MCS3_MEM[1879].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1880 GTM_gtm_cls3.MCS3_MEM[1880].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1881 GTM_gtm_cls3.MCS3_MEM[1881].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1882 GTM_gtm_cls3.MCS3_MEM[1882].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1883 GTM_gtm_cls3.MCS3_MEM[1883].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1884 GTM_gtm_cls3.MCS3_MEM[1884].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1885 GTM_gtm_cls3.MCS3_MEM[1885].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1886 GTM_gtm_cls3.MCS3_MEM[1886].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1887 GTM_gtm_cls3.MCS3_MEM[1887].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1888 GTM_gtm_cls3.MCS3_MEM[1888].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1889 GTM_gtm_cls3.MCS3_MEM[1889].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1890 GTM_gtm_cls3.MCS3_MEM[1890].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1891 GTM_gtm_cls3.MCS3_MEM[1891].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1892 GTM_gtm_cls3.MCS3_MEM[1892].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1893 GTM_gtm_cls3.MCS3_MEM[1893].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1894 GTM_gtm_cls3.MCS3_MEM[1894].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1895 GTM_gtm_cls3.MCS3_MEM[1895].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1896 GTM_gtm_cls3.MCS3_MEM[1896].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1897 GTM_gtm_cls3.MCS3_MEM[1897].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1898 GTM_gtm_cls3.MCS3_MEM[1898].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1899 GTM_gtm_cls3.MCS3_MEM[1899].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1900 GTM_gtm_cls3.MCS3_MEM[1900].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1901 GTM_gtm_cls3.MCS3_MEM[1901].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1902 GTM_gtm_cls3.MCS3_MEM[1902].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1903 GTM_gtm_cls3.MCS3_MEM[1903].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1904 GTM_gtm_cls3.MCS3_MEM[1904].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1905 GTM_gtm_cls3.MCS3_MEM[1905].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1906 GTM_gtm_cls3.MCS3_MEM[1906].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1907 GTM_gtm_cls3.MCS3_MEM[1907].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1908 GTM_gtm_cls3.MCS3_MEM[1908].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1909 GTM_gtm_cls3.MCS3_MEM[1909].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1910 GTM_gtm_cls3.MCS3_MEM[1910].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1911 GTM_gtm_cls3.MCS3_MEM[1911].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1912 GTM_gtm_cls3.MCS3_MEM[1912].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1913 GTM_gtm_cls3.MCS3_MEM[1913].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1914 GTM_gtm_cls3.MCS3_MEM[1914].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1915 GTM_gtm_cls3.MCS3_MEM[1915].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1916 GTM_gtm_cls3.MCS3_MEM[1916].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1917 GTM_gtm_cls3.MCS3_MEM[1917].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1918 GTM_gtm_cls3.MCS3_MEM[1918].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1919 GTM_gtm_cls3.MCS3_MEM[1919].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1920 GTM_gtm_cls3.MCS3_MEM[1920].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1921 GTM_gtm_cls3.MCS3_MEM[1921].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1922 GTM_gtm_cls3.MCS3_MEM[1922].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1923 GTM_gtm_cls3.MCS3_MEM[1923].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1924 GTM_gtm_cls3.MCS3_MEM[1924].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1925 GTM_gtm_cls3.MCS3_MEM[1925].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1926 GTM_gtm_cls3.MCS3_MEM[1926].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1927 GTM_gtm_cls3.MCS3_MEM[1927].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1928 GTM_gtm_cls3.MCS3_MEM[1928].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1929 GTM_gtm_cls3.MCS3_MEM[1929].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1930 GTM_gtm_cls3.MCS3_MEM[1930].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1931 GTM_gtm_cls3.MCS3_MEM[1931].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1932 GTM_gtm_cls3.MCS3_MEM[1932].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1933 GTM_gtm_cls3.MCS3_MEM[1933].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1934 GTM_gtm_cls3.MCS3_MEM[1934].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1935 GTM_gtm_cls3.MCS3_MEM[1935].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1936 GTM_gtm_cls3.MCS3_MEM[1936].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1937 GTM_gtm_cls3.MCS3_MEM[1937].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1938 GTM_gtm_cls3.MCS3_MEM[1938].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1939 GTM_gtm_cls3.MCS3_MEM[1939].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1940 GTM_gtm_cls3.MCS3_MEM[1940].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1941 GTM_gtm_cls3.MCS3_MEM[1941].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1942 GTM_gtm_cls3.MCS3_MEM[1942].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1943 GTM_gtm_cls3.MCS3_MEM[1943].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1944 GTM_gtm_cls3.MCS3_MEM[1944].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1945 GTM_gtm_cls3.MCS3_MEM[1945].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1946 GTM_gtm_cls3.MCS3_MEM[1946].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1947 GTM_gtm_cls3.MCS3_MEM[1947].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1948 GTM_gtm_cls3.MCS3_MEM[1948].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1949 GTM_gtm_cls3.MCS3_MEM[1949].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1950 GTM_gtm_cls3.MCS3_MEM[1950].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1951 GTM_gtm_cls3.MCS3_MEM[1951].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1952 GTM_gtm_cls3.MCS3_MEM[1952].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1953 GTM_gtm_cls3.MCS3_MEM[1953].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1954 GTM_gtm_cls3.MCS3_MEM[1954].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1955 GTM_gtm_cls3.MCS3_MEM[1955].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1956 GTM_gtm_cls3.MCS3_MEM[1956].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1957 GTM_gtm_cls3.MCS3_MEM[1957].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1958 GTM_gtm_cls3.MCS3_MEM[1958].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1959 GTM_gtm_cls3.MCS3_MEM[1959].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1960 GTM_gtm_cls3.MCS3_MEM[1960].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1961 GTM_gtm_cls3.MCS3_MEM[1961].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1962 GTM_gtm_cls3.MCS3_MEM[1962].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1963 GTM_gtm_cls3.MCS3_MEM[1963].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1964 GTM_gtm_cls3.MCS3_MEM[1964].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1965 GTM_gtm_cls3.MCS3_MEM[1965].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1966 GTM_gtm_cls3.MCS3_MEM[1966].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1967 GTM_gtm_cls3.MCS3_MEM[1967].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1968 GTM_gtm_cls3.MCS3_MEM[1968].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1969 GTM_gtm_cls3.MCS3_MEM[1969].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1970 GTM_gtm_cls3.MCS3_MEM[1970].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1971 GTM_gtm_cls3.MCS3_MEM[1971].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1972 GTM_gtm_cls3.MCS3_MEM[1972].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1973 GTM_gtm_cls3.MCS3_MEM[1973].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1974 GTM_gtm_cls3.MCS3_MEM[1974].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1975 GTM_gtm_cls3.MCS3_MEM[1975].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1976 GTM_gtm_cls3.MCS3_MEM[1976].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1977 GTM_gtm_cls3.MCS3_MEM[1977].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1978 GTM_gtm_cls3.MCS3_MEM[1978].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1979 GTM_gtm_cls3.MCS3_MEM[1979].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1980 GTM_gtm_cls3.MCS3_MEM[1980].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1981 GTM_gtm_cls3.MCS3_MEM[1981].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1982 GTM_gtm_cls3.MCS3_MEM[1982].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1983 GTM_gtm_cls3.MCS3_MEM[1983].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1984 GTM_gtm_cls3.MCS3_MEM[1984].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1985 GTM_gtm_cls3.MCS3_MEM[1985].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1986 GTM_gtm_cls3.MCS3_MEM[1986].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1987 GTM_gtm_cls3.MCS3_MEM[1987].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1988 GTM_gtm_cls3.MCS3_MEM[1988].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1989 GTM_gtm_cls3.MCS3_MEM[1989].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1990 GTM_gtm_cls3.MCS3_MEM[1990].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1991 GTM_gtm_cls3.MCS3_MEM[1991].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1992 GTM_gtm_cls3.MCS3_MEM[1992].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1993 GTM_gtm_cls3.MCS3_MEM[1993].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1994 GTM_gtm_cls3.MCS3_MEM[1994].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1995 GTM_gtm_cls3.MCS3_MEM[1995].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1996 GTM_gtm_cls3.MCS3_MEM[1996].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1997 GTM_gtm_cls3.MCS3_MEM[1997].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1998 GTM_gtm_cls3.MCS3_MEM[1998].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM1999 GTM_gtm_cls3.MCS3_MEM[1999].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2000 GTM_gtm_cls3.MCS3_MEM[2000].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2001 GTM_gtm_cls3.MCS3_MEM[2001].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2002 GTM_gtm_cls3.MCS3_MEM[2002].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2003 GTM_gtm_cls3.MCS3_MEM[2003].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2004 GTM_gtm_cls3.MCS3_MEM[2004].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2005 GTM_gtm_cls3.MCS3_MEM[2005].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2006 GTM_gtm_cls3.MCS3_MEM[2006].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2007 GTM_gtm_cls3.MCS3_MEM[2007].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2008 GTM_gtm_cls3.MCS3_MEM[2008].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2009 GTM_gtm_cls3.MCS3_MEM[2009].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2010 GTM_gtm_cls3.MCS3_MEM[2010].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2011 GTM_gtm_cls3.MCS3_MEM[2011].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2012 GTM_gtm_cls3.MCS3_MEM[2012].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2013 GTM_gtm_cls3.MCS3_MEM[2013].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2014 GTM_gtm_cls3.MCS3_MEM[2014].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2015 GTM_gtm_cls3.MCS3_MEM[2015].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2016 GTM_gtm_cls3.MCS3_MEM[2016].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2017 GTM_gtm_cls3.MCS3_MEM[2017].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2018 GTM_gtm_cls3.MCS3_MEM[2018].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2019 GTM_gtm_cls3.MCS3_MEM[2019].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2020 GTM_gtm_cls3.MCS3_MEM[2020].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2021 GTM_gtm_cls3.MCS3_MEM[2021].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2022 GTM_gtm_cls3.MCS3_MEM[2022].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2023 GTM_gtm_cls3.MCS3_MEM[2023].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2024 GTM_gtm_cls3.MCS3_MEM[2024].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2025 GTM_gtm_cls3.MCS3_MEM[2025].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2026 GTM_gtm_cls3.MCS3_MEM[2026].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2027 GTM_gtm_cls3.MCS3_MEM[2027].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2028 GTM_gtm_cls3.MCS3_MEM[2028].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2029 GTM_gtm_cls3.MCS3_MEM[2029].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2030 GTM_gtm_cls3.MCS3_MEM[2030].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2031 GTM_gtm_cls3.MCS3_MEM[2031].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2032 GTM_gtm_cls3.MCS3_MEM[2032].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2033 GTM_gtm_cls3.MCS3_MEM[2033].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2034 GTM_gtm_cls3.MCS3_MEM[2034].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2035 GTM_gtm_cls3.MCS3_MEM[2035].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2036 GTM_gtm_cls3.MCS3_MEM[2036].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2037 GTM_gtm_cls3.MCS3_MEM[2037].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2038 GTM_gtm_cls3.MCS3_MEM[2038].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2039 GTM_gtm_cls3.MCS3_MEM[2039].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2040 GTM_gtm_cls3.MCS3_MEM[2040].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2041 GTM_gtm_cls3.MCS3_MEM[2041].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2042 GTM_gtm_cls3.MCS3_MEM[2042].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2043 GTM_gtm_cls3.MCS3_MEM[2043].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2044 GTM_gtm_cls3.MCS3_MEM[2044].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2045 GTM_gtm_cls3.MCS3_MEM[2045].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2046 GTM_gtm_cls3.MCS3_MEM[2046].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2047 GTM_gtm_cls3.MCS3_MEM[2047].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2048 GTM_gtm_cls3.MCS3_MEM[2048].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2049 GTM_gtm_cls3.MCS3_MEM[2049].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2050 GTM_gtm_cls3.MCS3_MEM[2050].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2051 GTM_gtm_cls3.MCS3_MEM[2051].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2052 GTM_gtm_cls3.MCS3_MEM[2052].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2053 GTM_gtm_cls3.MCS3_MEM[2053].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2054 GTM_gtm_cls3.MCS3_MEM[2054].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2055 GTM_gtm_cls3.MCS3_MEM[2055].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2056 GTM_gtm_cls3.MCS3_MEM[2056].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2057 GTM_gtm_cls3.MCS3_MEM[2057].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2058 GTM_gtm_cls3.MCS3_MEM[2058].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2059 GTM_gtm_cls3.MCS3_MEM[2059].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2060 GTM_gtm_cls3.MCS3_MEM[2060].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2061 GTM_gtm_cls3.MCS3_MEM[2061].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2062 GTM_gtm_cls3.MCS3_MEM[2062].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2063 GTM_gtm_cls3.MCS3_MEM[2063].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2064 GTM_gtm_cls3.MCS3_MEM[2064].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2065 GTM_gtm_cls3.MCS3_MEM[2065].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2066 GTM_gtm_cls3.MCS3_MEM[2066].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2067 GTM_gtm_cls3.MCS3_MEM[2067].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2068 GTM_gtm_cls3.MCS3_MEM[2068].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2069 GTM_gtm_cls3.MCS3_MEM[2069].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2070 GTM_gtm_cls3.MCS3_MEM[2070].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2071 GTM_gtm_cls3.MCS3_MEM[2071].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2072 GTM_gtm_cls3.MCS3_MEM[2072].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2073 GTM_gtm_cls3.MCS3_MEM[2073].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2074 GTM_gtm_cls3.MCS3_MEM[2074].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2075 GTM_gtm_cls3.MCS3_MEM[2075].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2076 GTM_gtm_cls3.MCS3_MEM[2076].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2077 GTM_gtm_cls3.MCS3_MEM[2077].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2078 GTM_gtm_cls3.MCS3_MEM[2078].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2079 GTM_gtm_cls3.MCS3_MEM[2079].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2080 GTM_gtm_cls3.MCS3_MEM[2080].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2081 GTM_gtm_cls3.MCS3_MEM[2081].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2082 GTM_gtm_cls3.MCS3_MEM[2082].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2083 GTM_gtm_cls3.MCS3_MEM[2083].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2084 GTM_gtm_cls3.MCS3_MEM[2084].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2085 GTM_gtm_cls3.MCS3_MEM[2085].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2086 GTM_gtm_cls3.MCS3_MEM[2086].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2087 GTM_gtm_cls3.MCS3_MEM[2087].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2088 GTM_gtm_cls3.MCS3_MEM[2088].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2089 GTM_gtm_cls3.MCS3_MEM[2089].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2090 GTM_gtm_cls3.MCS3_MEM[2090].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2091 GTM_gtm_cls3.MCS3_MEM[2091].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2092 GTM_gtm_cls3.MCS3_MEM[2092].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2093 GTM_gtm_cls3.MCS3_MEM[2093].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2094 GTM_gtm_cls3.MCS3_MEM[2094].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2095 GTM_gtm_cls3.MCS3_MEM[2095].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2096 GTM_gtm_cls3.MCS3_MEM[2096].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2097 GTM_gtm_cls3.MCS3_MEM[2097].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2098 GTM_gtm_cls3.MCS3_MEM[2098].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2099 GTM_gtm_cls3.MCS3_MEM[2099].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2100 GTM_gtm_cls3.MCS3_MEM[2100].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2101 GTM_gtm_cls3.MCS3_MEM[2101].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2102 GTM_gtm_cls3.MCS3_MEM[2102].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2103 GTM_gtm_cls3.MCS3_MEM[2103].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2104 GTM_gtm_cls3.MCS3_MEM[2104].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2105 GTM_gtm_cls3.MCS3_MEM[2105].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2106 GTM_gtm_cls3.MCS3_MEM[2106].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2107 GTM_gtm_cls3.MCS3_MEM[2107].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2108 GTM_gtm_cls3.MCS3_MEM[2108].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2109 GTM_gtm_cls3.MCS3_MEM[2109].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2110 GTM_gtm_cls3.MCS3_MEM[2110].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2111 GTM_gtm_cls3.MCS3_MEM[2111].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2112 GTM_gtm_cls3.MCS3_MEM[2112].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2113 GTM_gtm_cls3.MCS3_MEM[2113].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2114 GTM_gtm_cls3.MCS3_MEM[2114].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2115 GTM_gtm_cls3.MCS3_MEM[2115].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2116 GTM_gtm_cls3.MCS3_MEM[2116].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2117 GTM_gtm_cls3.MCS3_MEM[2117].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2118 GTM_gtm_cls3.MCS3_MEM[2118].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2119 GTM_gtm_cls3.MCS3_MEM[2119].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2120 GTM_gtm_cls3.MCS3_MEM[2120].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2121 GTM_gtm_cls3.MCS3_MEM[2121].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2122 GTM_gtm_cls3.MCS3_MEM[2122].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2123 GTM_gtm_cls3.MCS3_MEM[2123].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2124 GTM_gtm_cls3.MCS3_MEM[2124].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2125 GTM_gtm_cls3.MCS3_MEM[2125].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2126 GTM_gtm_cls3.MCS3_MEM[2126].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2127 GTM_gtm_cls3.MCS3_MEM[2127].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2128 GTM_gtm_cls3.MCS3_MEM[2128].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2129 GTM_gtm_cls3.MCS3_MEM[2129].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2130 GTM_gtm_cls3.MCS3_MEM[2130].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2131 GTM_gtm_cls3.MCS3_MEM[2131].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2132 GTM_gtm_cls3.MCS3_MEM[2132].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2133 GTM_gtm_cls3.MCS3_MEM[2133].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2134 GTM_gtm_cls3.MCS3_MEM[2134].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2135 GTM_gtm_cls3.MCS3_MEM[2135].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2136 GTM_gtm_cls3.MCS3_MEM[2136].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2137 GTM_gtm_cls3.MCS3_MEM[2137].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2138 GTM_gtm_cls3.MCS3_MEM[2138].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2139 GTM_gtm_cls3.MCS3_MEM[2139].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2140 GTM_gtm_cls3.MCS3_MEM[2140].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2141 GTM_gtm_cls3.MCS3_MEM[2141].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2142 GTM_gtm_cls3.MCS3_MEM[2142].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2143 GTM_gtm_cls3.MCS3_MEM[2143].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2144 GTM_gtm_cls3.MCS3_MEM[2144].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2145 GTM_gtm_cls3.MCS3_MEM[2145].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2146 GTM_gtm_cls3.MCS3_MEM[2146].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2147 GTM_gtm_cls3.MCS3_MEM[2147].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2148 GTM_gtm_cls3.MCS3_MEM[2148].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2149 GTM_gtm_cls3.MCS3_MEM[2149].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2150 GTM_gtm_cls3.MCS3_MEM[2150].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2151 GTM_gtm_cls3.MCS3_MEM[2151].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2152 GTM_gtm_cls3.MCS3_MEM[2152].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2153 GTM_gtm_cls3.MCS3_MEM[2153].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2154 GTM_gtm_cls3.MCS3_MEM[2154].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2155 GTM_gtm_cls3.MCS3_MEM[2155].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2156 GTM_gtm_cls3.MCS3_MEM[2156].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2157 GTM_gtm_cls3.MCS3_MEM[2157].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2158 GTM_gtm_cls3.MCS3_MEM[2158].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2159 GTM_gtm_cls3.MCS3_MEM[2159].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2160 GTM_gtm_cls3.MCS3_MEM[2160].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2161 GTM_gtm_cls3.MCS3_MEM[2161].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2162 GTM_gtm_cls3.MCS3_MEM[2162].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2163 GTM_gtm_cls3.MCS3_MEM[2163].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2164 GTM_gtm_cls3.MCS3_MEM[2164].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2165 GTM_gtm_cls3.MCS3_MEM[2165].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2166 GTM_gtm_cls3.MCS3_MEM[2166].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2167 GTM_gtm_cls3.MCS3_MEM[2167].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2168 GTM_gtm_cls3.MCS3_MEM[2168].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2169 GTM_gtm_cls3.MCS3_MEM[2169].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2170 GTM_gtm_cls3.MCS3_MEM[2170].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2171 GTM_gtm_cls3.MCS3_MEM[2171].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2172 GTM_gtm_cls3.MCS3_MEM[2172].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2173 GTM_gtm_cls3.MCS3_MEM[2173].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2174 GTM_gtm_cls3.MCS3_MEM[2174].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2175 GTM_gtm_cls3.MCS3_MEM[2175].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2176 GTM_gtm_cls3.MCS3_MEM[2176].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2177 GTM_gtm_cls3.MCS3_MEM[2177].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2178 GTM_gtm_cls3.MCS3_MEM[2178].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2179 GTM_gtm_cls3.MCS3_MEM[2179].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2180 GTM_gtm_cls3.MCS3_MEM[2180].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2181 GTM_gtm_cls3.MCS3_MEM[2181].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2182 GTM_gtm_cls3.MCS3_MEM[2182].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2183 GTM_gtm_cls3.MCS3_MEM[2183].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2184 GTM_gtm_cls3.MCS3_MEM[2184].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2185 GTM_gtm_cls3.MCS3_MEM[2185].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2186 GTM_gtm_cls3.MCS3_MEM[2186].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2187 GTM_gtm_cls3.MCS3_MEM[2187].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2188 GTM_gtm_cls3.MCS3_MEM[2188].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2189 GTM_gtm_cls3.MCS3_MEM[2189].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2190 GTM_gtm_cls3.MCS3_MEM[2190].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2191 GTM_gtm_cls3.MCS3_MEM[2191].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2192 GTM_gtm_cls3.MCS3_MEM[2192].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2193 GTM_gtm_cls3.MCS3_MEM[2193].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2194 GTM_gtm_cls3.MCS3_MEM[2194].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2195 GTM_gtm_cls3.MCS3_MEM[2195].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2196 GTM_gtm_cls3.MCS3_MEM[2196].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2197 GTM_gtm_cls3.MCS3_MEM[2197].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2198 GTM_gtm_cls3.MCS3_MEM[2198].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2199 GTM_gtm_cls3.MCS3_MEM[2199].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2200 GTM_gtm_cls3.MCS3_MEM[2200].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2201 GTM_gtm_cls3.MCS3_MEM[2201].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2202 GTM_gtm_cls3.MCS3_MEM[2202].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2203 GTM_gtm_cls3.MCS3_MEM[2203].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2204 GTM_gtm_cls3.MCS3_MEM[2204].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2205 GTM_gtm_cls3.MCS3_MEM[2205].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2206 GTM_gtm_cls3.MCS3_MEM[2206].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2207 GTM_gtm_cls3.MCS3_MEM[2207].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2208 GTM_gtm_cls3.MCS3_MEM[2208].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2209 GTM_gtm_cls3.MCS3_MEM[2209].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2210 GTM_gtm_cls3.MCS3_MEM[2210].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2211 GTM_gtm_cls3.MCS3_MEM[2211].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2212 GTM_gtm_cls3.MCS3_MEM[2212].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2213 GTM_gtm_cls3.MCS3_MEM[2213].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2214 GTM_gtm_cls3.MCS3_MEM[2214].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2215 GTM_gtm_cls3.MCS3_MEM[2215].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2216 GTM_gtm_cls3.MCS3_MEM[2216].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2217 GTM_gtm_cls3.MCS3_MEM[2217].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2218 GTM_gtm_cls3.MCS3_MEM[2218].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2219 GTM_gtm_cls3.MCS3_MEM[2219].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2220 GTM_gtm_cls3.MCS3_MEM[2220].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2221 GTM_gtm_cls3.MCS3_MEM[2221].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2222 GTM_gtm_cls3.MCS3_MEM[2222].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2223 GTM_gtm_cls3.MCS3_MEM[2223].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2224 GTM_gtm_cls3.MCS3_MEM[2224].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2225 GTM_gtm_cls3.MCS3_MEM[2225].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2226 GTM_gtm_cls3.MCS3_MEM[2226].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2227 GTM_gtm_cls3.MCS3_MEM[2227].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2228 GTM_gtm_cls3.MCS3_MEM[2228].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2229 GTM_gtm_cls3.MCS3_MEM[2229].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2230 GTM_gtm_cls3.MCS3_MEM[2230].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2231 GTM_gtm_cls3.MCS3_MEM[2231].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2232 GTM_gtm_cls3.MCS3_MEM[2232].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2233 GTM_gtm_cls3.MCS3_MEM[2233].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2234 GTM_gtm_cls3.MCS3_MEM[2234].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2235 GTM_gtm_cls3.MCS3_MEM[2235].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2236 GTM_gtm_cls3.MCS3_MEM[2236].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2237 GTM_gtm_cls3.MCS3_MEM[2237].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2238 GTM_gtm_cls3.MCS3_MEM[2238].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2239 GTM_gtm_cls3.MCS3_MEM[2239].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2240 GTM_gtm_cls3.MCS3_MEM[2240].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2241 GTM_gtm_cls3.MCS3_MEM[2241].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2242 GTM_gtm_cls3.MCS3_MEM[2242].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2243 GTM_gtm_cls3.MCS3_MEM[2243].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2244 GTM_gtm_cls3.MCS3_MEM[2244].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2245 GTM_gtm_cls3.MCS3_MEM[2245].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2246 GTM_gtm_cls3.MCS3_MEM[2246].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2247 GTM_gtm_cls3.MCS3_MEM[2247].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2248 GTM_gtm_cls3.MCS3_MEM[2248].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2249 GTM_gtm_cls3.MCS3_MEM[2249].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2250 GTM_gtm_cls3.MCS3_MEM[2250].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2251 GTM_gtm_cls3.MCS3_MEM[2251].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2252 GTM_gtm_cls3.MCS3_MEM[2252].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2253 GTM_gtm_cls3.MCS3_MEM[2253].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2254 GTM_gtm_cls3.MCS3_MEM[2254].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2255 GTM_gtm_cls3.MCS3_MEM[2255].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2256 GTM_gtm_cls3.MCS3_MEM[2256].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2257 GTM_gtm_cls3.MCS3_MEM[2257].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2258 GTM_gtm_cls3.MCS3_MEM[2258].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2259 GTM_gtm_cls3.MCS3_MEM[2259].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2260 GTM_gtm_cls3.MCS3_MEM[2260].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2261 GTM_gtm_cls3.MCS3_MEM[2261].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2262 GTM_gtm_cls3.MCS3_MEM[2262].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2263 GTM_gtm_cls3.MCS3_MEM[2263].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2264 GTM_gtm_cls3.MCS3_MEM[2264].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2265 GTM_gtm_cls3.MCS3_MEM[2265].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2266 GTM_gtm_cls3.MCS3_MEM[2266].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2267 GTM_gtm_cls3.MCS3_MEM[2267].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2268 GTM_gtm_cls3.MCS3_MEM[2268].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2269 GTM_gtm_cls3.MCS3_MEM[2269].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2270 GTM_gtm_cls3.MCS3_MEM[2270].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2271 GTM_gtm_cls3.MCS3_MEM[2271].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2272 GTM_gtm_cls3.MCS3_MEM[2272].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2273 GTM_gtm_cls3.MCS3_MEM[2273].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2274 GTM_gtm_cls3.MCS3_MEM[2274].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2275 GTM_gtm_cls3.MCS3_MEM[2275].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2276 GTM_gtm_cls3.MCS3_MEM[2276].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2277 GTM_gtm_cls3.MCS3_MEM[2277].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2278 GTM_gtm_cls3.MCS3_MEM[2278].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2279 GTM_gtm_cls3.MCS3_MEM[2279].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2280 GTM_gtm_cls3.MCS3_MEM[2280].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2281 GTM_gtm_cls3.MCS3_MEM[2281].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2282 GTM_gtm_cls3.MCS3_MEM[2282].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2283 GTM_gtm_cls3.MCS3_MEM[2283].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2284 GTM_gtm_cls3.MCS3_MEM[2284].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2285 GTM_gtm_cls3.MCS3_MEM[2285].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2286 GTM_gtm_cls3.MCS3_MEM[2286].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2287 GTM_gtm_cls3.MCS3_MEM[2287].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2288 GTM_gtm_cls3.MCS3_MEM[2288].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2289 GTM_gtm_cls3.MCS3_MEM[2289].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2290 GTM_gtm_cls3.MCS3_MEM[2290].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2291 GTM_gtm_cls3.MCS3_MEM[2291].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2292 GTM_gtm_cls3.MCS3_MEM[2292].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2293 GTM_gtm_cls3.MCS3_MEM[2293].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2294 GTM_gtm_cls3.MCS3_MEM[2294].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2295 GTM_gtm_cls3.MCS3_MEM[2295].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2296 GTM_gtm_cls3.MCS3_MEM[2296].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2297 GTM_gtm_cls3.MCS3_MEM[2297].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2298 GTM_gtm_cls3.MCS3_MEM[2298].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2299 GTM_gtm_cls3.MCS3_MEM[2299].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2300 GTM_gtm_cls3.MCS3_MEM[2300].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2301 GTM_gtm_cls3.MCS3_MEM[2301].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2302 GTM_gtm_cls3.MCS3_MEM[2302].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2303 GTM_gtm_cls3.MCS3_MEM[2303].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2304 GTM_gtm_cls3.MCS3_MEM[2304].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2305 GTM_gtm_cls3.MCS3_MEM[2305].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2306 GTM_gtm_cls3.MCS3_MEM[2306].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2307 GTM_gtm_cls3.MCS3_MEM[2307].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2308 GTM_gtm_cls3.MCS3_MEM[2308].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2309 GTM_gtm_cls3.MCS3_MEM[2309].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2310 GTM_gtm_cls3.MCS3_MEM[2310].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2311 GTM_gtm_cls3.MCS3_MEM[2311].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2312 GTM_gtm_cls3.MCS3_MEM[2312].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2313 GTM_gtm_cls3.MCS3_MEM[2313].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2314 GTM_gtm_cls3.MCS3_MEM[2314].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2315 GTM_gtm_cls3.MCS3_MEM[2315].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2316 GTM_gtm_cls3.MCS3_MEM[2316].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2317 GTM_gtm_cls3.MCS3_MEM[2317].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2318 GTM_gtm_cls3.MCS3_MEM[2318].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2319 GTM_gtm_cls3.MCS3_MEM[2319].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2320 GTM_gtm_cls3.MCS3_MEM[2320].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2321 GTM_gtm_cls3.MCS3_MEM[2321].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2322 GTM_gtm_cls3.MCS3_MEM[2322].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2323 GTM_gtm_cls3.MCS3_MEM[2323].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2324 GTM_gtm_cls3.MCS3_MEM[2324].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2325 GTM_gtm_cls3.MCS3_MEM[2325].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2326 GTM_gtm_cls3.MCS3_MEM[2326].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2327 GTM_gtm_cls3.MCS3_MEM[2327].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2328 GTM_gtm_cls3.MCS3_MEM[2328].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2329 GTM_gtm_cls3.MCS3_MEM[2329].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2330 GTM_gtm_cls3.MCS3_MEM[2330].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2331 GTM_gtm_cls3.MCS3_MEM[2331].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2332 GTM_gtm_cls3.MCS3_MEM[2332].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2333 GTM_gtm_cls3.MCS3_MEM[2333].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2334 GTM_gtm_cls3.MCS3_MEM[2334].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2335 GTM_gtm_cls3.MCS3_MEM[2335].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2336 GTM_gtm_cls3.MCS3_MEM[2336].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2337 GTM_gtm_cls3.MCS3_MEM[2337].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2338 GTM_gtm_cls3.MCS3_MEM[2338].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2339 GTM_gtm_cls3.MCS3_MEM[2339].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2340 GTM_gtm_cls3.MCS3_MEM[2340].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2341 GTM_gtm_cls3.MCS3_MEM[2341].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2342 GTM_gtm_cls3.MCS3_MEM[2342].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2343 GTM_gtm_cls3.MCS3_MEM[2343].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2344 GTM_gtm_cls3.MCS3_MEM[2344].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2345 GTM_gtm_cls3.MCS3_MEM[2345].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2346 GTM_gtm_cls3.MCS3_MEM[2346].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2347 GTM_gtm_cls3.MCS3_MEM[2347].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2348 GTM_gtm_cls3.MCS3_MEM[2348].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2349 GTM_gtm_cls3.MCS3_MEM[2349].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2350 GTM_gtm_cls3.MCS3_MEM[2350].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2351 GTM_gtm_cls3.MCS3_MEM[2351].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2352 GTM_gtm_cls3.MCS3_MEM[2352].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2353 GTM_gtm_cls3.MCS3_MEM[2353].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2354 GTM_gtm_cls3.MCS3_MEM[2354].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2355 GTM_gtm_cls3.MCS3_MEM[2355].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2356 GTM_gtm_cls3.MCS3_MEM[2356].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2357 GTM_gtm_cls3.MCS3_MEM[2357].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2358 GTM_gtm_cls3.MCS3_MEM[2358].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2359 GTM_gtm_cls3.MCS3_MEM[2359].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2360 GTM_gtm_cls3.MCS3_MEM[2360].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2361 GTM_gtm_cls3.MCS3_MEM[2361].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2362 GTM_gtm_cls3.MCS3_MEM[2362].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2363 GTM_gtm_cls3.MCS3_MEM[2363].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2364 GTM_gtm_cls3.MCS3_MEM[2364].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2365 GTM_gtm_cls3.MCS3_MEM[2365].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2366 GTM_gtm_cls3.MCS3_MEM[2366].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2367 GTM_gtm_cls3.MCS3_MEM[2367].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2368 GTM_gtm_cls3.MCS3_MEM[2368].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2369 GTM_gtm_cls3.MCS3_MEM[2369].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2370 GTM_gtm_cls3.MCS3_MEM[2370].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2371 GTM_gtm_cls3.MCS3_MEM[2371].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2372 GTM_gtm_cls3.MCS3_MEM[2372].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2373 GTM_gtm_cls3.MCS3_MEM[2373].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2374 GTM_gtm_cls3.MCS3_MEM[2374].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2375 GTM_gtm_cls3.MCS3_MEM[2375].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2376 GTM_gtm_cls3.MCS3_MEM[2376].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2377 GTM_gtm_cls3.MCS3_MEM[2377].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2378 GTM_gtm_cls3.MCS3_MEM[2378].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2379 GTM_gtm_cls3.MCS3_MEM[2379].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2380 GTM_gtm_cls3.MCS3_MEM[2380].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2381 GTM_gtm_cls3.MCS3_MEM[2381].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2382 GTM_gtm_cls3.MCS3_MEM[2382].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2383 GTM_gtm_cls3.MCS3_MEM[2383].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2384 GTM_gtm_cls3.MCS3_MEM[2384].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2385 GTM_gtm_cls3.MCS3_MEM[2385].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2386 GTM_gtm_cls3.MCS3_MEM[2386].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2387 GTM_gtm_cls3.MCS3_MEM[2387].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2388 GTM_gtm_cls3.MCS3_MEM[2388].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2389 GTM_gtm_cls3.MCS3_MEM[2389].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2390 GTM_gtm_cls3.MCS3_MEM[2390].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2391 GTM_gtm_cls3.MCS3_MEM[2391].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2392 GTM_gtm_cls3.MCS3_MEM[2392].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2393 GTM_gtm_cls3.MCS3_MEM[2393].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2394 GTM_gtm_cls3.MCS3_MEM[2394].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2395 GTM_gtm_cls3.MCS3_MEM[2395].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2396 GTM_gtm_cls3.MCS3_MEM[2396].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2397 GTM_gtm_cls3.MCS3_MEM[2397].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2398 GTM_gtm_cls3.MCS3_MEM[2398].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2399 GTM_gtm_cls3.MCS3_MEM[2399].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2400 GTM_gtm_cls3.MCS3_MEM[2400].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2401 GTM_gtm_cls3.MCS3_MEM[2401].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2402 GTM_gtm_cls3.MCS3_MEM[2402].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2403 GTM_gtm_cls3.MCS3_MEM[2403].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2404 GTM_gtm_cls3.MCS3_MEM[2404].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2405 GTM_gtm_cls3.MCS3_MEM[2405].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2406 GTM_gtm_cls3.MCS3_MEM[2406].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2407 GTM_gtm_cls3.MCS3_MEM[2407].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2408 GTM_gtm_cls3.MCS3_MEM[2408].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2409 GTM_gtm_cls3.MCS3_MEM[2409].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2410 GTM_gtm_cls3.MCS3_MEM[2410].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2411 GTM_gtm_cls3.MCS3_MEM[2411].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2412 GTM_gtm_cls3.MCS3_MEM[2412].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2413 GTM_gtm_cls3.MCS3_MEM[2413].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2414 GTM_gtm_cls3.MCS3_MEM[2414].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2415 GTM_gtm_cls3.MCS3_MEM[2415].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2416 GTM_gtm_cls3.MCS3_MEM[2416].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2417 GTM_gtm_cls3.MCS3_MEM[2417].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2418 GTM_gtm_cls3.MCS3_MEM[2418].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2419 GTM_gtm_cls3.MCS3_MEM[2419].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2420 GTM_gtm_cls3.MCS3_MEM[2420].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2421 GTM_gtm_cls3.MCS3_MEM[2421].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2422 GTM_gtm_cls3.MCS3_MEM[2422].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2423 GTM_gtm_cls3.MCS3_MEM[2423].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2424 GTM_gtm_cls3.MCS3_MEM[2424].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2425 GTM_gtm_cls3.MCS3_MEM[2425].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2426 GTM_gtm_cls3.MCS3_MEM[2426].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2427 GTM_gtm_cls3.MCS3_MEM[2427].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2428 GTM_gtm_cls3.MCS3_MEM[2428].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2429 GTM_gtm_cls3.MCS3_MEM[2429].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2430 GTM_gtm_cls3.MCS3_MEM[2430].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2431 GTM_gtm_cls3.MCS3_MEM[2431].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2432 GTM_gtm_cls3.MCS3_MEM[2432].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2433 GTM_gtm_cls3.MCS3_MEM[2433].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2434 GTM_gtm_cls3.MCS3_MEM[2434].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2435 GTM_gtm_cls3.MCS3_MEM[2435].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2436 GTM_gtm_cls3.MCS3_MEM[2436].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2437 GTM_gtm_cls3.MCS3_MEM[2437].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2438 GTM_gtm_cls3.MCS3_MEM[2438].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2439 GTM_gtm_cls3.MCS3_MEM[2439].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2440 GTM_gtm_cls3.MCS3_MEM[2440].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2441 GTM_gtm_cls3.MCS3_MEM[2441].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2442 GTM_gtm_cls3.MCS3_MEM[2442].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2443 GTM_gtm_cls3.MCS3_MEM[2443].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2444 GTM_gtm_cls3.MCS3_MEM[2444].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2445 GTM_gtm_cls3.MCS3_MEM[2445].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2446 GTM_gtm_cls3.MCS3_MEM[2446].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2447 GTM_gtm_cls3.MCS3_MEM[2447].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2448 GTM_gtm_cls3.MCS3_MEM[2448].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2449 GTM_gtm_cls3.MCS3_MEM[2449].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2450 GTM_gtm_cls3.MCS3_MEM[2450].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2451 GTM_gtm_cls3.MCS3_MEM[2451].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2452 GTM_gtm_cls3.MCS3_MEM[2452].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2453 GTM_gtm_cls3.MCS3_MEM[2453].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2454 GTM_gtm_cls3.MCS3_MEM[2454].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2455 GTM_gtm_cls3.MCS3_MEM[2455].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2456 GTM_gtm_cls3.MCS3_MEM[2456].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2457 GTM_gtm_cls3.MCS3_MEM[2457].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2458 GTM_gtm_cls3.MCS3_MEM[2458].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2459 GTM_gtm_cls3.MCS3_MEM[2459].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2460 GTM_gtm_cls3.MCS3_MEM[2460].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2461 GTM_gtm_cls3.MCS3_MEM[2461].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2462 GTM_gtm_cls3.MCS3_MEM[2462].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2463 GTM_gtm_cls3.MCS3_MEM[2463].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2464 GTM_gtm_cls3.MCS3_MEM[2464].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2465 GTM_gtm_cls3.MCS3_MEM[2465].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2466 GTM_gtm_cls3.MCS3_MEM[2466].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2467 GTM_gtm_cls3.MCS3_MEM[2467].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2468 GTM_gtm_cls3.MCS3_MEM[2468].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2469 GTM_gtm_cls3.MCS3_MEM[2469].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2470 GTM_gtm_cls3.MCS3_MEM[2470].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2471 GTM_gtm_cls3.MCS3_MEM[2471].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2472 GTM_gtm_cls3.MCS3_MEM[2472].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2473 GTM_gtm_cls3.MCS3_MEM[2473].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2474 GTM_gtm_cls3.MCS3_MEM[2474].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2475 GTM_gtm_cls3.MCS3_MEM[2475].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2476 GTM_gtm_cls3.MCS3_MEM[2476].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2477 GTM_gtm_cls3.MCS3_MEM[2477].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2478 GTM_gtm_cls3.MCS3_MEM[2478].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2479 GTM_gtm_cls3.MCS3_MEM[2479].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2480 GTM_gtm_cls3.MCS3_MEM[2480].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2481 GTM_gtm_cls3.MCS3_MEM[2481].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2482 GTM_gtm_cls3.MCS3_MEM[2482].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2483 GTM_gtm_cls3.MCS3_MEM[2483].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2484 GTM_gtm_cls3.MCS3_MEM[2484].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2485 GTM_gtm_cls3.MCS3_MEM[2485].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2486 GTM_gtm_cls3.MCS3_MEM[2486].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2487 GTM_gtm_cls3.MCS3_MEM[2487].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2488 GTM_gtm_cls3.MCS3_MEM[2488].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2489 GTM_gtm_cls3.MCS3_MEM[2489].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2490 GTM_gtm_cls3.MCS3_MEM[2490].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2491 GTM_gtm_cls3.MCS3_MEM[2491].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2492 GTM_gtm_cls3.MCS3_MEM[2492].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2493 GTM_gtm_cls3.MCS3_MEM[2493].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2494 GTM_gtm_cls3.MCS3_MEM[2494].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2495 GTM_gtm_cls3.MCS3_MEM[2495].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2496 GTM_gtm_cls3.MCS3_MEM[2496].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2497 GTM_gtm_cls3.MCS3_MEM[2497].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2498 GTM_gtm_cls3.MCS3_MEM[2498].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2499 GTM_gtm_cls3.MCS3_MEM[2499].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2500 GTM_gtm_cls3.MCS3_MEM[2500].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2501 GTM_gtm_cls3.MCS3_MEM[2501].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2502 GTM_gtm_cls3.MCS3_MEM[2502].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2503 GTM_gtm_cls3.MCS3_MEM[2503].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2504 GTM_gtm_cls3.MCS3_MEM[2504].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2505 GTM_gtm_cls3.MCS3_MEM[2505].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2506 GTM_gtm_cls3.MCS3_MEM[2506].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2507 GTM_gtm_cls3.MCS3_MEM[2507].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2508 GTM_gtm_cls3.MCS3_MEM[2508].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2509 GTM_gtm_cls3.MCS3_MEM[2509].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2510 GTM_gtm_cls3.MCS3_MEM[2510].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2511 GTM_gtm_cls3.MCS3_MEM[2511].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2512 GTM_gtm_cls3.MCS3_MEM[2512].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2513 GTM_gtm_cls3.MCS3_MEM[2513].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2514 GTM_gtm_cls3.MCS3_MEM[2514].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2515 GTM_gtm_cls3.MCS3_MEM[2515].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2516 GTM_gtm_cls3.MCS3_MEM[2516].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2517 GTM_gtm_cls3.MCS3_MEM[2517].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2518 GTM_gtm_cls3.MCS3_MEM[2518].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2519 GTM_gtm_cls3.MCS3_MEM[2519].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2520 GTM_gtm_cls3.MCS3_MEM[2520].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2521 GTM_gtm_cls3.MCS3_MEM[2521].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2522 GTM_gtm_cls3.MCS3_MEM[2522].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2523 GTM_gtm_cls3.MCS3_MEM[2523].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2524 GTM_gtm_cls3.MCS3_MEM[2524].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2525 GTM_gtm_cls3.MCS3_MEM[2525].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2526 GTM_gtm_cls3.MCS3_MEM[2526].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2527 GTM_gtm_cls3.MCS3_MEM[2527].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2528 GTM_gtm_cls3.MCS3_MEM[2528].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2529 GTM_gtm_cls3.MCS3_MEM[2529].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2530 GTM_gtm_cls3.MCS3_MEM[2530].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2531 GTM_gtm_cls3.MCS3_MEM[2531].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2532 GTM_gtm_cls3.MCS3_MEM[2532].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2533 GTM_gtm_cls3.MCS3_MEM[2533].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2534 GTM_gtm_cls3.MCS3_MEM[2534].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2535 GTM_gtm_cls3.MCS3_MEM[2535].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2536 GTM_gtm_cls3.MCS3_MEM[2536].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2537 GTM_gtm_cls3.MCS3_MEM[2537].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2538 GTM_gtm_cls3.MCS3_MEM[2538].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2539 GTM_gtm_cls3.MCS3_MEM[2539].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2540 GTM_gtm_cls3.MCS3_MEM[2540].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2541 GTM_gtm_cls3.MCS3_MEM[2541].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2542 GTM_gtm_cls3.MCS3_MEM[2542].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2543 GTM_gtm_cls3.MCS3_MEM[2543].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2544 GTM_gtm_cls3.MCS3_MEM[2544].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2545 GTM_gtm_cls3.MCS3_MEM[2545].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2546 GTM_gtm_cls3.MCS3_MEM[2546].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2547 GTM_gtm_cls3.MCS3_MEM[2547].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2548 GTM_gtm_cls3.MCS3_MEM[2548].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2549 GTM_gtm_cls3.MCS3_MEM[2549].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2550 GTM_gtm_cls3.MCS3_MEM[2550].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2551 GTM_gtm_cls3.MCS3_MEM[2551].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2552 GTM_gtm_cls3.MCS3_MEM[2552].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2553 GTM_gtm_cls3.MCS3_MEM[2553].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2554 GTM_gtm_cls3.MCS3_MEM[2554].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2555 GTM_gtm_cls3.MCS3_MEM[2555].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2556 GTM_gtm_cls3.MCS3_MEM[2556].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2557 GTM_gtm_cls3.MCS3_MEM[2557].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2558 GTM_gtm_cls3.MCS3_MEM[2558].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2559 GTM_gtm_cls3.MCS3_MEM[2559].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2560 GTM_gtm_cls3.MCS3_MEM[2560].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2561 GTM_gtm_cls3.MCS3_MEM[2561].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2562 GTM_gtm_cls3.MCS3_MEM[2562].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2563 GTM_gtm_cls3.MCS3_MEM[2563].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2564 GTM_gtm_cls3.MCS3_MEM[2564].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2565 GTM_gtm_cls3.MCS3_MEM[2565].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2566 GTM_gtm_cls3.MCS3_MEM[2566].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2567 GTM_gtm_cls3.MCS3_MEM[2567].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2568 GTM_gtm_cls3.MCS3_MEM[2568].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2569 GTM_gtm_cls3.MCS3_MEM[2569].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2570 GTM_gtm_cls3.MCS3_MEM[2570].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2571 GTM_gtm_cls3.MCS3_MEM[2571].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2572 GTM_gtm_cls3.MCS3_MEM[2572].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2573 GTM_gtm_cls3.MCS3_MEM[2573].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2574 GTM_gtm_cls3.MCS3_MEM[2574].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2575 GTM_gtm_cls3.MCS3_MEM[2575].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2576 GTM_gtm_cls3.MCS3_MEM[2576].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2577 GTM_gtm_cls3.MCS3_MEM[2577].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2578 GTM_gtm_cls3.MCS3_MEM[2578].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2579 GTM_gtm_cls3.MCS3_MEM[2579].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2580 GTM_gtm_cls3.MCS3_MEM[2580].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2581 GTM_gtm_cls3.MCS3_MEM[2581].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2582 GTM_gtm_cls3.MCS3_MEM[2582].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2583 GTM_gtm_cls3.MCS3_MEM[2583].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2584 GTM_gtm_cls3.MCS3_MEM[2584].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2585 GTM_gtm_cls3.MCS3_MEM[2585].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2586 GTM_gtm_cls3.MCS3_MEM[2586].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2587 GTM_gtm_cls3.MCS3_MEM[2587].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2588 GTM_gtm_cls3.MCS3_MEM[2588].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2589 GTM_gtm_cls3.MCS3_MEM[2589].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2590 GTM_gtm_cls3.MCS3_MEM[2590].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2591 GTM_gtm_cls3.MCS3_MEM[2591].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2592 GTM_gtm_cls3.MCS3_MEM[2592].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2593 GTM_gtm_cls3.MCS3_MEM[2593].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2594 GTM_gtm_cls3.MCS3_MEM[2594].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2595 GTM_gtm_cls3.MCS3_MEM[2595].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2596 GTM_gtm_cls3.MCS3_MEM[2596].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2597 GTM_gtm_cls3.MCS3_MEM[2597].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2598 GTM_gtm_cls3.MCS3_MEM[2598].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2599 GTM_gtm_cls3.MCS3_MEM[2599].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2600 GTM_gtm_cls3.MCS3_MEM[2600].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2601 GTM_gtm_cls3.MCS3_MEM[2601].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2602 GTM_gtm_cls3.MCS3_MEM[2602].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2603 GTM_gtm_cls3.MCS3_MEM[2603].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2604 GTM_gtm_cls3.MCS3_MEM[2604].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2605 GTM_gtm_cls3.MCS3_MEM[2605].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2606 GTM_gtm_cls3.MCS3_MEM[2606].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2607 GTM_gtm_cls3.MCS3_MEM[2607].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2608 GTM_gtm_cls3.MCS3_MEM[2608].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2609 GTM_gtm_cls3.MCS3_MEM[2609].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2610 GTM_gtm_cls3.MCS3_MEM[2610].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2611 GTM_gtm_cls3.MCS3_MEM[2611].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2612 GTM_gtm_cls3.MCS3_MEM[2612].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2613 GTM_gtm_cls3.MCS3_MEM[2613].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2614 GTM_gtm_cls3.MCS3_MEM[2614].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2615 GTM_gtm_cls3.MCS3_MEM[2615].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2616 GTM_gtm_cls3.MCS3_MEM[2616].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2617 GTM_gtm_cls3.MCS3_MEM[2617].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2618 GTM_gtm_cls3.MCS3_MEM[2618].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2619 GTM_gtm_cls3.MCS3_MEM[2619].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2620 GTM_gtm_cls3.MCS3_MEM[2620].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2621 GTM_gtm_cls3.MCS3_MEM[2621].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2622 GTM_gtm_cls3.MCS3_MEM[2622].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2623 GTM_gtm_cls3.MCS3_MEM[2623].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2624 GTM_gtm_cls3.MCS3_MEM[2624].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2625 GTM_gtm_cls3.MCS3_MEM[2625].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2626 GTM_gtm_cls3.MCS3_MEM[2626].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2627 GTM_gtm_cls3.MCS3_MEM[2627].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2628 GTM_gtm_cls3.MCS3_MEM[2628].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2629 GTM_gtm_cls3.MCS3_MEM[2629].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2630 GTM_gtm_cls3.MCS3_MEM[2630].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2631 GTM_gtm_cls3.MCS3_MEM[2631].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2632 GTM_gtm_cls3.MCS3_MEM[2632].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2633 GTM_gtm_cls3.MCS3_MEM[2633].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2634 GTM_gtm_cls3.MCS3_MEM[2634].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2635 GTM_gtm_cls3.MCS3_MEM[2635].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2636 GTM_gtm_cls3.MCS3_MEM[2636].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2637 GTM_gtm_cls3.MCS3_MEM[2637].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2638 GTM_gtm_cls3.MCS3_MEM[2638].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2639 GTM_gtm_cls3.MCS3_MEM[2639].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2640 GTM_gtm_cls3.MCS3_MEM[2640].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2641 GTM_gtm_cls3.MCS3_MEM[2641].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2642 GTM_gtm_cls3.MCS3_MEM[2642].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2643 GTM_gtm_cls3.MCS3_MEM[2643].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2644 GTM_gtm_cls3.MCS3_MEM[2644].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2645 GTM_gtm_cls3.MCS3_MEM[2645].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2646 GTM_gtm_cls3.MCS3_MEM[2646].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2647 GTM_gtm_cls3.MCS3_MEM[2647].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2648 GTM_gtm_cls3.MCS3_MEM[2648].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2649 GTM_gtm_cls3.MCS3_MEM[2649].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2650 GTM_gtm_cls3.MCS3_MEM[2650].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2651 GTM_gtm_cls3.MCS3_MEM[2651].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2652 GTM_gtm_cls3.MCS3_MEM[2652].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2653 GTM_gtm_cls3.MCS3_MEM[2653].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2654 GTM_gtm_cls3.MCS3_MEM[2654].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2655 GTM_gtm_cls3.MCS3_MEM[2655].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2656 GTM_gtm_cls3.MCS3_MEM[2656].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2657 GTM_gtm_cls3.MCS3_MEM[2657].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2658 GTM_gtm_cls3.MCS3_MEM[2658].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2659 GTM_gtm_cls3.MCS3_MEM[2659].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2660 GTM_gtm_cls3.MCS3_MEM[2660].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2661 GTM_gtm_cls3.MCS3_MEM[2661].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2662 GTM_gtm_cls3.MCS3_MEM[2662].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2663 GTM_gtm_cls3.MCS3_MEM[2663].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2664 GTM_gtm_cls3.MCS3_MEM[2664].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2665 GTM_gtm_cls3.MCS3_MEM[2665].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2666 GTM_gtm_cls3.MCS3_MEM[2666].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2667 GTM_gtm_cls3.MCS3_MEM[2667].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2668 GTM_gtm_cls3.MCS3_MEM[2668].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2669 GTM_gtm_cls3.MCS3_MEM[2669].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2670 GTM_gtm_cls3.MCS3_MEM[2670].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2671 GTM_gtm_cls3.MCS3_MEM[2671].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2672 GTM_gtm_cls3.MCS3_MEM[2672].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2673 GTM_gtm_cls3.MCS3_MEM[2673].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2674 GTM_gtm_cls3.MCS3_MEM[2674].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2675 GTM_gtm_cls3.MCS3_MEM[2675].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2676 GTM_gtm_cls3.MCS3_MEM[2676].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2677 GTM_gtm_cls3.MCS3_MEM[2677].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2678 GTM_gtm_cls3.MCS3_MEM[2678].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2679 GTM_gtm_cls3.MCS3_MEM[2679].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2680 GTM_gtm_cls3.MCS3_MEM[2680].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2681 GTM_gtm_cls3.MCS3_MEM[2681].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2682 GTM_gtm_cls3.MCS3_MEM[2682].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2683 GTM_gtm_cls3.MCS3_MEM[2683].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2684 GTM_gtm_cls3.MCS3_MEM[2684].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2685 GTM_gtm_cls3.MCS3_MEM[2685].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2686 GTM_gtm_cls3.MCS3_MEM[2686].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2687 GTM_gtm_cls3.MCS3_MEM[2687].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2688 GTM_gtm_cls3.MCS3_MEM[2688].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2689 GTM_gtm_cls3.MCS3_MEM[2689].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2690 GTM_gtm_cls3.MCS3_MEM[2690].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2691 GTM_gtm_cls3.MCS3_MEM[2691].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2692 GTM_gtm_cls3.MCS3_MEM[2692].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2693 GTM_gtm_cls3.MCS3_MEM[2693].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2694 GTM_gtm_cls3.MCS3_MEM[2694].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2695 GTM_gtm_cls3.MCS3_MEM[2695].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2696 GTM_gtm_cls3.MCS3_MEM[2696].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2697 GTM_gtm_cls3.MCS3_MEM[2697].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2698 GTM_gtm_cls3.MCS3_MEM[2698].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2699 GTM_gtm_cls3.MCS3_MEM[2699].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2700 GTM_gtm_cls3.MCS3_MEM[2700].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2701 GTM_gtm_cls3.MCS3_MEM[2701].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2702 GTM_gtm_cls3.MCS3_MEM[2702].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2703 GTM_gtm_cls3.MCS3_MEM[2703].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2704 GTM_gtm_cls3.MCS3_MEM[2704].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2705 GTM_gtm_cls3.MCS3_MEM[2705].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2706 GTM_gtm_cls3.MCS3_MEM[2706].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2707 GTM_gtm_cls3.MCS3_MEM[2707].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2708 GTM_gtm_cls3.MCS3_MEM[2708].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2709 GTM_gtm_cls3.MCS3_MEM[2709].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2710 GTM_gtm_cls3.MCS3_MEM[2710].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2711 GTM_gtm_cls3.MCS3_MEM[2711].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2712 GTM_gtm_cls3.MCS3_MEM[2712].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2713 GTM_gtm_cls3.MCS3_MEM[2713].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2714 GTM_gtm_cls3.MCS3_MEM[2714].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2715 GTM_gtm_cls3.MCS3_MEM[2715].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2716 GTM_gtm_cls3.MCS3_MEM[2716].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2717 GTM_gtm_cls3.MCS3_MEM[2717].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2718 GTM_gtm_cls3.MCS3_MEM[2718].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2719 GTM_gtm_cls3.MCS3_MEM[2719].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2720 GTM_gtm_cls3.MCS3_MEM[2720].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2721 GTM_gtm_cls3.MCS3_MEM[2721].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2722 GTM_gtm_cls3.MCS3_MEM[2722].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2723 GTM_gtm_cls3.MCS3_MEM[2723].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2724 GTM_gtm_cls3.MCS3_MEM[2724].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2725 GTM_gtm_cls3.MCS3_MEM[2725].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2726 GTM_gtm_cls3.MCS3_MEM[2726].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2727 GTM_gtm_cls3.MCS3_MEM[2727].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2728 GTM_gtm_cls3.MCS3_MEM[2728].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2729 GTM_gtm_cls3.MCS3_MEM[2729].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2730 GTM_gtm_cls3.MCS3_MEM[2730].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2731 GTM_gtm_cls3.MCS3_MEM[2731].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2732 GTM_gtm_cls3.MCS3_MEM[2732].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2733 GTM_gtm_cls3.MCS3_MEM[2733].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2734 GTM_gtm_cls3.MCS3_MEM[2734].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2735 GTM_gtm_cls3.MCS3_MEM[2735].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2736 GTM_gtm_cls3.MCS3_MEM[2736].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2737 GTM_gtm_cls3.MCS3_MEM[2737].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2738 GTM_gtm_cls3.MCS3_MEM[2738].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2739 GTM_gtm_cls3.MCS3_MEM[2739].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2740 GTM_gtm_cls3.MCS3_MEM[2740].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2741 GTM_gtm_cls3.MCS3_MEM[2741].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2742 GTM_gtm_cls3.MCS3_MEM[2742].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2743 GTM_gtm_cls3.MCS3_MEM[2743].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2744 GTM_gtm_cls3.MCS3_MEM[2744].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2745 GTM_gtm_cls3.MCS3_MEM[2745].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2746 GTM_gtm_cls3.MCS3_MEM[2746].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2747 GTM_gtm_cls3.MCS3_MEM[2747].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2748 GTM_gtm_cls3.MCS3_MEM[2748].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2749 GTM_gtm_cls3.MCS3_MEM[2749].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2750 GTM_gtm_cls3.MCS3_MEM[2750].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2751 GTM_gtm_cls3.MCS3_MEM[2751].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2752 GTM_gtm_cls3.MCS3_MEM[2752].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2753 GTM_gtm_cls3.MCS3_MEM[2753].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2754 GTM_gtm_cls3.MCS3_MEM[2754].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2755 GTM_gtm_cls3.MCS3_MEM[2755].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2756 GTM_gtm_cls3.MCS3_MEM[2756].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2757 GTM_gtm_cls3.MCS3_MEM[2757].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2758 GTM_gtm_cls3.MCS3_MEM[2758].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2759 GTM_gtm_cls3.MCS3_MEM[2759].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2760 GTM_gtm_cls3.MCS3_MEM[2760].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2761 GTM_gtm_cls3.MCS3_MEM[2761].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2762 GTM_gtm_cls3.MCS3_MEM[2762].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2763 GTM_gtm_cls3.MCS3_MEM[2763].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2764 GTM_gtm_cls3.MCS3_MEM[2764].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2765 GTM_gtm_cls3.MCS3_MEM[2765].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2766 GTM_gtm_cls3.MCS3_MEM[2766].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2767 GTM_gtm_cls3.MCS3_MEM[2767].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2768 GTM_gtm_cls3.MCS3_MEM[2768].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2769 GTM_gtm_cls3.MCS3_MEM[2769].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2770 GTM_gtm_cls3.MCS3_MEM[2770].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2771 GTM_gtm_cls3.MCS3_MEM[2771].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2772 GTM_gtm_cls3.MCS3_MEM[2772].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2773 GTM_gtm_cls3.MCS3_MEM[2773].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2774 GTM_gtm_cls3.MCS3_MEM[2774].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2775 GTM_gtm_cls3.MCS3_MEM[2775].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2776 GTM_gtm_cls3.MCS3_MEM[2776].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2777 GTM_gtm_cls3.MCS3_MEM[2777].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2778 GTM_gtm_cls3.MCS3_MEM[2778].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2779 GTM_gtm_cls3.MCS3_MEM[2779].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2780 GTM_gtm_cls3.MCS3_MEM[2780].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2781 GTM_gtm_cls3.MCS3_MEM[2781].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2782 GTM_gtm_cls3.MCS3_MEM[2782].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2783 GTM_gtm_cls3.MCS3_MEM[2783].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2784 GTM_gtm_cls3.MCS3_MEM[2784].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2785 GTM_gtm_cls3.MCS3_MEM[2785].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2786 GTM_gtm_cls3.MCS3_MEM[2786].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2787 GTM_gtm_cls3.MCS3_MEM[2787].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2788 GTM_gtm_cls3.MCS3_MEM[2788].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2789 GTM_gtm_cls3.MCS3_MEM[2789].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2790 GTM_gtm_cls3.MCS3_MEM[2790].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2791 GTM_gtm_cls3.MCS3_MEM[2791].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2792 GTM_gtm_cls3.MCS3_MEM[2792].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2793 GTM_gtm_cls3.MCS3_MEM[2793].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2794 GTM_gtm_cls3.MCS3_MEM[2794].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2795 GTM_gtm_cls3.MCS3_MEM[2795].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2796 GTM_gtm_cls3.MCS3_MEM[2796].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2797 GTM_gtm_cls3.MCS3_MEM[2797].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2798 GTM_gtm_cls3.MCS3_MEM[2798].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2799 GTM_gtm_cls3.MCS3_MEM[2799].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2800 GTM_gtm_cls3.MCS3_MEM[2800].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2801 GTM_gtm_cls3.MCS3_MEM[2801].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2802 GTM_gtm_cls3.MCS3_MEM[2802].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2803 GTM_gtm_cls3.MCS3_MEM[2803].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2804 GTM_gtm_cls3.MCS3_MEM[2804].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2805 GTM_gtm_cls3.MCS3_MEM[2805].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2806 GTM_gtm_cls3.MCS3_MEM[2806].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2807 GTM_gtm_cls3.MCS3_MEM[2807].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2808 GTM_gtm_cls3.MCS3_MEM[2808].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2809 GTM_gtm_cls3.MCS3_MEM[2809].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2810 GTM_gtm_cls3.MCS3_MEM[2810].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2811 GTM_gtm_cls3.MCS3_MEM[2811].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2812 GTM_gtm_cls3.MCS3_MEM[2812].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2813 GTM_gtm_cls3.MCS3_MEM[2813].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2814 GTM_gtm_cls3.MCS3_MEM[2814].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2815 GTM_gtm_cls3.MCS3_MEM[2815].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2816 GTM_gtm_cls3.MCS3_MEM[2816].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2817 GTM_gtm_cls3.MCS3_MEM[2817].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2818 GTM_gtm_cls3.MCS3_MEM[2818].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2819 GTM_gtm_cls3.MCS3_MEM[2819].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2820 GTM_gtm_cls3.MCS3_MEM[2820].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2821 GTM_gtm_cls3.MCS3_MEM[2821].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2822 GTM_gtm_cls3.MCS3_MEM[2822].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2823 GTM_gtm_cls3.MCS3_MEM[2823].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2824 GTM_gtm_cls3.MCS3_MEM[2824].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2825 GTM_gtm_cls3.MCS3_MEM[2825].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2826 GTM_gtm_cls3.MCS3_MEM[2826].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2827 GTM_gtm_cls3.MCS3_MEM[2827].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2828 GTM_gtm_cls3.MCS3_MEM[2828].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2829 GTM_gtm_cls3.MCS3_MEM[2829].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2830 GTM_gtm_cls3.MCS3_MEM[2830].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2831 GTM_gtm_cls3.MCS3_MEM[2831].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2832 GTM_gtm_cls3.MCS3_MEM[2832].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2833 GTM_gtm_cls3.MCS3_MEM[2833].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2834 GTM_gtm_cls3.MCS3_MEM[2834].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2835 GTM_gtm_cls3.MCS3_MEM[2835].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2836 GTM_gtm_cls3.MCS3_MEM[2836].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2837 GTM_gtm_cls3.MCS3_MEM[2837].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2838 GTM_gtm_cls3.MCS3_MEM[2838].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2839 GTM_gtm_cls3.MCS3_MEM[2839].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2840 GTM_gtm_cls3.MCS3_MEM[2840].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2841 GTM_gtm_cls3.MCS3_MEM[2841].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2842 GTM_gtm_cls3.MCS3_MEM[2842].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2843 GTM_gtm_cls3.MCS3_MEM[2843].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2844 GTM_gtm_cls3.MCS3_MEM[2844].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2845 GTM_gtm_cls3.MCS3_MEM[2845].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2846 GTM_gtm_cls3.MCS3_MEM[2846].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2847 GTM_gtm_cls3.MCS3_MEM[2847].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2848 GTM_gtm_cls3.MCS3_MEM[2848].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2849 GTM_gtm_cls3.MCS3_MEM[2849].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2850 GTM_gtm_cls3.MCS3_MEM[2850].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2851 GTM_gtm_cls3.MCS3_MEM[2851].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2852 GTM_gtm_cls3.MCS3_MEM[2852].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2853 GTM_gtm_cls3.MCS3_MEM[2853].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2854 GTM_gtm_cls3.MCS3_MEM[2854].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2855 GTM_gtm_cls3.MCS3_MEM[2855].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2856 GTM_gtm_cls3.MCS3_MEM[2856].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2857 GTM_gtm_cls3.MCS3_MEM[2857].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2858 GTM_gtm_cls3.MCS3_MEM[2858].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2859 GTM_gtm_cls3.MCS3_MEM[2859].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2860 GTM_gtm_cls3.MCS3_MEM[2860].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2861 GTM_gtm_cls3.MCS3_MEM[2861].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2862 GTM_gtm_cls3.MCS3_MEM[2862].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2863 GTM_gtm_cls3.MCS3_MEM[2863].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2864 GTM_gtm_cls3.MCS3_MEM[2864].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2865 GTM_gtm_cls3.MCS3_MEM[2865].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2866 GTM_gtm_cls3.MCS3_MEM[2866].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2867 GTM_gtm_cls3.MCS3_MEM[2867].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2868 GTM_gtm_cls3.MCS3_MEM[2868].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2869 GTM_gtm_cls3.MCS3_MEM[2869].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2870 GTM_gtm_cls3.MCS3_MEM[2870].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2871 GTM_gtm_cls3.MCS3_MEM[2871].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2872 GTM_gtm_cls3.MCS3_MEM[2872].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2873 GTM_gtm_cls3.MCS3_MEM[2873].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2874 GTM_gtm_cls3.MCS3_MEM[2874].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2875 GTM_gtm_cls3.MCS3_MEM[2875].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2876 GTM_gtm_cls3.MCS3_MEM[2876].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2877 GTM_gtm_cls3.MCS3_MEM[2877].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2878 GTM_gtm_cls3.MCS3_MEM[2878].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2879 GTM_gtm_cls3.MCS3_MEM[2879].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2880 GTM_gtm_cls3.MCS3_MEM[2880].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2881 GTM_gtm_cls3.MCS3_MEM[2881].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2882 GTM_gtm_cls3.MCS3_MEM[2882].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2883 GTM_gtm_cls3.MCS3_MEM[2883].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2884 GTM_gtm_cls3.MCS3_MEM[2884].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2885 GTM_gtm_cls3.MCS3_MEM[2885].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2886 GTM_gtm_cls3.MCS3_MEM[2886].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2887 GTM_gtm_cls3.MCS3_MEM[2887].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2888 GTM_gtm_cls3.MCS3_MEM[2888].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2889 GTM_gtm_cls3.MCS3_MEM[2889].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2890 GTM_gtm_cls3.MCS3_MEM[2890].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2891 GTM_gtm_cls3.MCS3_MEM[2891].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2892 GTM_gtm_cls3.MCS3_MEM[2892].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2893 GTM_gtm_cls3.MCS3_MEM[2893].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2894 GTM_gtm_cls3.MCS3_MEM[2894].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2895 GTM_gtm_cls3.MCS3_MEM[2895].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2896 GTM_gtm_cls3.MCS3_MEM[2896].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2897 GTM_gtm_cls3.MCS3_MEM[2897].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2898 GTM_gtm_cls3.MCS3_MEM[2898].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2899 GTM_gtm_cls3.MCS3_MEM[2899].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2900 GTM_gtm_cls3.MCS3_MEM[2900].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2901 GTM_gtm_cls3.MCS3_MEM[2901].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2902 GTM_gtm_cls3.MCS3_MEM[2902].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2903 GTM_gtm_cls3.MCS3_MEM[2903].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2904 GTM_gtm_cls3.MCS3_MEM[2904].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2905 GTM_gtm_cls3.MCS3_MEM[2905].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2906 GTM_gtm_cls3.MCS3_MEM[2906].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2907 GTM_gtm_cls3.MCS3_MEM[2907].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2908 GTM_gtm_cls3.MCS3_MEM[2908].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2909 GTM_gtm_cls3.MCS3_MEM[2909].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2910 GTM_gtm_cls3.MCS3_MEM[2910].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2911 GTM_gtm_cls3.MCS3_MEM[2911].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2912 GTM_gtm_cls3.MCS3_MEM[2912].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2913 GTM_gtm_cls3.MCS3_MEM[2913].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2914 GTM_gtm_cls3.MCS3_MEM[2914].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2915 GTM_gtm_cls3.MCS3_MEM[2915].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2916 GTM_gtm_cls3.MCS3_MEM[2916].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2917 GTM_gtm_cls3.MCS3_MEM[2917].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2918 GTM_gtm_cls3.MCS3_MEM[2918].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2919 GTM_gtm_cls3.MCS3_MEM[2919].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2920 GTM_gtm_cls3.MCS3_MEM[2920].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2921 GTM_gtm_cls3.MCS3_MEM[2921].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2922 GTM_gtm_cls3.MCS3_MEM[2922].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2923 GTM_gtm_cls3.MCS3_MEM[2923].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2924 GTM_gtm_cls3.MCS3_MEM[2924].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2925 GTM_gtm_cls3.MCS3_MEM[2925].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2926 GTM_gtm_cls3.MCS3_MEM[2926].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2927 GTM_gtm_cls3.MCS3_MEM[2927].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2928 GTM_gtm_cls3.MCS3_MEM[2928].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2929 GTM_gtm_cls3.MCS3_MEM[2929].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2930 GTM_gtm_cls3.MCS3_MEM[2930].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2931 GTM_gtm_cls3.MCS3_MEM[2931].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2932 GTM_gtm_cls3.MCS3_MEM[2932].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2933 GTM_gtm_cls3.MCS3_MEM[2933].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2934 GTM_gtm_cls3.MCS3_MEM[2934].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2935 GTM_gtm_cls3.MCS3_MEM[2935].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2936 GTM_gtm_cls3.MCS3_MEM[2936].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2937 GTM_gtm_cls3.MCS3_MEM[2937].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2938 GTM_gtm_cls3.MCS3_MEM[2938].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2939 GTM_gtm_cls3.MCS3_MEM[2939].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2940 GTM_gtm_cls3.MCS3_MEM[2940].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2941 GTM_gtm_cls3.MCS3_MEM[2941].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2942 GTM_gtm_cls3.MCS3_MEM[2942].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2943 GTM_gtm_cls3.MCS3_MEM[2943].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2944 GTM_gtm_cls3.MCS3_MEM[2944].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2945 GTM_gtm_cls3.MCS3_MEM[2945].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2946 GTM_gtm_cls3.MCS3_MEM[2946].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2947 GTM_gtm_cls3.MCS3_MEM[2947].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2948 GTM_gtm_cls3.MCS3_MEM[2948].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2949 GTM_gtm_cls3.MCS3_MEM[2949].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2950 GTM_gtm_cls3.MCS3_MEM[2950].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2951 GTM_gtm_cls3.MCS3_MEM[2951].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2952 GTM_gtm_cls3.MCS3_MEM[2952].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2953 GTM_gtm_cls3.MCS3_MEM[2953].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2954 GTM_gtm_cls3.MCS3_MEM[2954].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2955 GTM_gtm_cls3.MCS3_MEM[2955].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2956 GTM_gtm_cls3.MCS3_MEM[2956].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2957 GTM_gtm_cls3.MCS3_MEM[2957].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2958 GTM_gtm_cls3.MCS3_MEM[2958].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2959 GTM_gtm_cls3.MCS3_MEM[2959].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2960 GTM_gtm_cls3.MCS3_MEM[2960].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2961 GTM_gtm_cls3.MCS3_MEM[2961].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2962 GTM_gtm_cls3.MCS3_MEM[2962].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2963 GTM_gtm_cls3.MCS3_MEM[2963].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2964 GTM_gtm_cls3.MCS3_MEM[2964].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2965 GTM_gtm_cls3.MCS3_MEM[2965].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2966 GTM_gtm_cls3.MCS3_MEM[2966].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2967 GTM_gtm_cls3.MCS3_MEM[2967].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2968 GTM_gtm_cls3.MCS3_MEM[2968].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2969 GTM_gtm_cls3.MCS3_MEM[2969].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2970 GTM_gtm_cls3.MCS3_MEM[2970].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2971 GTM_gtm_cls3.MCS3_MEM[2971].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2972 GTM_gtm_cls3.MCS3_MEM[2972].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2973 GTM_gtm_cls3.MCS3_MEM[2973].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2974 GTM_gtm_cls3.MCS3_MEM[2974].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2975 GTM_gtm_cls3.MCS3_MEM[2975].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2976 GTM_gtm_cls3.MCS3_MEM[2976].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2977 GTM_gtm_cls3.MCS3_MEM[2977].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2978 GTM_gtm_cls3.MCS3_MEM[2978].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2979 GTM_gtm_cls3.MCS3_MEM[2979].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2980 GTM_gtm_cls3.MCS3_MEM[2980].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2981 GTM_gtm_cls3.MCS3_MEM[2981].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2982 GTM_gtm_cls3.MCS3_MEM[2982].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2983 GTM_gtm_cls3.MCS3_MEM[2983].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2984 GTM_gtm_cls3.MCS3_MEM[2984].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2985 GTM_gtm_cls3.MCS3_MEM[2985].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2986 GTM_gtm_cls3.MCS3_MEM[2986].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2987 GTM_gtm_cls3.MCS3_MEM[2987].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2988 GTM_gtm_cls3.MCS3_MEM[2988].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2989 GTM_gtm_cls3.MCS3_MEM[2989].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2990 GTM_gtm_cls3.MCS3_MEM[2990].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2991 GTM_gtm_cls3.MCS3_MEM[2991].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2992 GTM_gtm_cls3.MCS3_MEM[2992].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2993 GTM_gtm_cls3.MCS3_MEM[2993].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2994 GTM_gtm_cls3.MCS3_MEM[2994].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2995 GTM_gtm_cls3.MCS3_MEM[2995].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2996 GTM_gtm_cls3.MCS3_MEM[2996].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2997 GTM_gtm_cls3.MCS3_MEM[2997].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2998 GTM_gtm_cls3.MCS3_MEM[2998].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM2999 GTM_gtm_cls3.MCS3_MEM[2999].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3000 GTM_gtm_cls3.MCS3_MEM[3000].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3001 GTM_gtm_cls3.MCS3_MEM[3001].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3002 GTM_gtm_cls3.MCS3_MEM[3002].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3003 GTM_gtm_cls3.MCS3_MEM[3003].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3004 GTM_gtm_cls3.MCS3_MEM[3004].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3005 GTM_gtm_cls3.MCS3_MEM[3005].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3006 GTM_gtm_cls3.MCS3_MEM[3006].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3007 GTM_gtm_cls3.MCS3_MEM[3007].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3008 GTM_gtm_cls3.MCS3_MEM[3008].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3009 GTM_gtm_cls3.MCS3_MEM[3009].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3010 GTM_gtm_cls3.MCS3_MEM[3010].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3011 GTM_gtm_cls3.MCS3_MEM[3011].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3012 GTM_gtm_cls3.MCS3_MEM[3012].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3013 GTM_gtm_cls3.MCS3_MEM[3013].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3014 GTM_gtm_cls3.MCS3_MEM[3014].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3015 GTM_gtm_cls3.MCS3_MEM[3015].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3016 GTM_gtm_cls3.MCS3_MEM[3016].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3017 GTM_gtm_cls3.MCS3_MEM[3017].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3018 GTM_gtm_cls3.MCS3_MEM[3018].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3019 GTM_gtm_cls3.MCS3_MEM[3019].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3020 GTM_gtm_cls3.MCS3_MEM[3020].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3021 GTM_gtm_cls3.MCS3_MEM[3021].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3022 GTM_gtm_cls3.MCS3_MEM[3022].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3023 GTM_gtm_cls3.MCS3_MEM[3023].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3024 GTM_gtm_cls3.MCS3_MEM[3024].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3025 GTM_gtm_cls3.MCS3_MEM[3025].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3026 GTM_gtm_cls3.MCS3_MEM[3026].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3027 GTM_gtm_cls3.MCS3_MEM[3027].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3028 GTM_gtm_cls3.MCS3_MEM[3028].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3029 GTM_gtm_cls3.MCS3_MEM[3029].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3030 GTM_gtm_cls3.MCS3_MEM[3030].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3031 GTM_gtm_cls3.MCS3_MEM[3031].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3032 GTM_gtm_cls3.MCS3_MEM[3032].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3033 GTM_gtm_cls3.MCS3_MEM[3033].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3034 GTM_gtm_cls3.MCS3_MEM[3034].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3035 GTM_gtm_cls3.MCS3_MEM[3035].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3036 GTM_gtm_cls3.MCS3_MEM[3036].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3037 GTM_gtm_cls3.MCS3_MEM[3037].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3038 GTM_gtm_cls3.MCS3_MEM[3038].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3039 GTM_gtm_cls3.MCS3_MEM[3039].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3040 GTM_gtm_cls3.MCS3_MEM[3040].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3041 GTM_gtm_cls3.MCS3_MEM[3041].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3042 GTM_gtm_cls3.MCS3_MEM[3042].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3043 GTM_gtm_cls3.MCS3_MEM[3043].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3044 GTM_gtm_cls3.MCS3_MEM[3044].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3045 GTM_gtm_cls3.MCS3_MEM[3045].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3046 GTM_gtm_cls3.MCS3_MEM[3046].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3047 GTM_gtm_cls3.MCS3_MEM[3047].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3048 GTM_gtm_cls3.MCS3_MEM[3048].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3049 GTM_gtm_cls3.MCS3_MEM[3049].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3050 GTM_gtm_cls3.MCS3_MEM[3050].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3051 GTM_gtm_cls3.MCS3_MEM[3051].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3052 GTM_gtm_cls3.MCS3_MEM[3052].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3053 GTM_gtm_cls3.MCS3_MEM[3053].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3054 GTM_gtm_cls3.MCS3_MEM[3054].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3055 GTM_gtm_cls3.MCS3_MEM[3055].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3056 GTM_gtm_cls3.MCS3_MEM[3056].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3057 GTM_gtm_cls3.MCS3_MEM[3057].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3058 GTM_gtm_cls3.MCS3_MEM[3058].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3059 GTM_gtm_cls3.MCS3_MEM[3059].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3060 GTM_gtm_cls3.MCS3_MEM[3060].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3061 GTM_gtm_cls3.MCS3_MEM[3061].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3062 GTM_gtm_cls3.MCS3_MEM[3062].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3063 GTM_gtm_cls3.MCS3_MEM[3063].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3064 GTM_gtm_cls3.MCS3_MEM[3064].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3065 GTM_gtm_cls3.MCS3_MEM[3065].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3066 GTM_gtm_cls3.MCS3_MEM[3066].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3067 GTM_gtm_cls3.MCS3_MEM[3067].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3068 GTM_gtm_cls3.MCS3_MEM[3068].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3069 GTM_gtm_cls3.MCS3_MEM[3069].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3070 GTM_gtm_cls3.MCS3_MEM[3070].R /* MCS[i] memory region */
#define GTM_gtm_cls3_MCS3_MEM3071 GTM_gtm_cls3.MCS3_MEM[3071].R /* MCS[i] memory region */

/* I3C */
#define I3C_0_MCONFIG        I3C_0.MCONFIG.R               /* Controller Configuration */
#define I3C_0_SCONFIG        I3C_0.SCONFIG.R               /* Target Configuration */
#define I3C_0_SSTATUS        I3C_0.SSTATUS.R               /* Target Status */
#define I3C_0_SINTSET        I3C_0.SINTSET.R               /* Target Interrupt Set */
#define I3C_0_SINTCLR        I3C_0.SINTCLR.R               /* Target Interrupt Clear */
#define I3C_0_SINTMASKED     I3C_0.SINTMASKED.R            /* Target Interrupt Mask */
#define I3C_0_SERRWARN       I3C_0.SERRWARN.R              /* Target Errors and Warnings */
#define I3C_0_SDMACTRL       I3C_0.SDMACTRL.R              /* Target DMA Control */
#define I3C_0_SDATACTRL      I3C_0.SDATACTRL.R             /* Target Data Control */
#define I3C_0_SWDATAB        I3C_0.SWDATAB.R               /* Target Write Data Byte */
#define I3C_0_SWDATABE       I3C_0.SWDATABE.R              /* Target Write Data Byte End */
#define I3C_0_SWDATAH        I3C_0.SWDATAH.R               /* Target Write Data Halfword */
#define I3C_0_SWDATAHE       I3C_0.SWDATAHE.R              /* Target Write Data Halfword End */
#define I3C_0_SRDATAB        I3C_0.SRDATAB.R               /* Target Read Data Byte */
#define I3C_0_MRDATAH        I3C_0.SRDATAH_MRDATAH.MRDATAH.R /* Controller Read Data Halfword */
#define I3C_0_SRDATAH        I3C_0.SRDATAH_MRDATAH.SRDATAH.R /* Target Read Data Halfword */
#define I3C_0_SWDATAB1       I3C_0.SWDATAB1.R              /* Target Write Data Byte */
#define I3C_0_SCAPABILITIES2 I3C_0.SCAPABILITIES2.R        /* Target Capabilities 2 */
#define I3C_0_SCAPABILITIES  I3C_0.SCAPABILITIES.R         /* Target Capabilities */
#define I3C_0_SIDEXT         I3C_0.SIDEXT.R                /* Target ID Extension */
#define I3C_0_SMSGLAST       I3C_0.SMSGLAST.R              /* Target Message Last Matched */
#define I3C_0_MCONFIG_EXT    I3C_0.MCONFIG_EXT.R           /* Controller Extended Configuration */
#define I3C_0_MCTRL          I3C_0.MCTRL.R                 /* Controller Control */
#define I3C_0_MSTATUS        I3C_0.MSTATUS.R               /* Controller Status */
#define I3C_0_MINTSET        I3C_0.MINTSET.R               /* Controller Interrupt Set */
#define I3C_0_MINTCLR        I3C_0.MINTCLR.R               /* Controller Interrupt Clear */
#define I3C_0_MINTMASKED     I3C_0.MINTMASKED.R            /* Controller Interrupt Mask */
#define I3C_0_MERRWARN       I3C_0.MERRWARN.R              /* Controller Errors and Warnings */
#define I3C_0_MDMACTRL       I3C_0.MDMACTRL.R              /* Controller DMA Control */
#define I3C_0_MDATACTRL      I3C_0.MDATACTRL.R             /* Controller Data Control */
#define I3C_0_MWDATAB        I3C_0.MWDATAB.R               /* Controller Write Data Byte */
#define I3C_0_MWDATABE       I3C_0.MWDATABE.R              /* Controller Write Data Byte End */
#define I3C_0_MWDATAH        I3C_0.MWDATAH.R               /* Controller Write Data Halfword */
#define I3C_0_MWDATAHE       I3C_0.MWDATAHE.R              /* Controller Write Data Halfword End */
#define I3C_0_MRDATAB        I3C_0.MRDATAB.R               /* Controller Read Data Byte */
#define I3C_0_MWDATAB1       I3C_0.MWDATAB1.R              /* Controller Write Byte Data 1 (to Bus) */
#define I3C_0_MWMSG_SDR_CONTROL I3C_0.MWMSG_SDR.MWMSG_SDR_CONTROL.R /* Controller Write Message Control in SDR mode */
#define I3C_0_MWMSG_SDR_DATA I3C_0.MWMSG_SDR.MWMSG_SDR_DATA.R /* Controller Write Message Data in SDR mode */
#define I3C_0_MRMSG_SDR      I3C_0.MRMSG_SDR.R             /* Controller Read Message in SDR mode */
#define I3C_0_SMAPCTRL1      I3C_0.SMAPCTRL1.R             /* Map Feature Control 1 */
#define I3C_0_SELFRESET      I3C_0.SELFRESET.R             /* Self Reset */

/* INTERFACE_CONFIGURATION */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_PORT_RID CEVA_SPF2__INTERFACE_CONFIGURATION.D_PORT_RID.R /* DMSS AXI Read ID Control */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_PORT_WID CEVA_SPF2__INTERFACE_CONFIGURATION.D_PORT_WID.R /* DMSS AXI Port Write ID Control */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD0_START CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD0_START.R /* DMSS ADD0 START */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD0_ATT0 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD0_ATT0.R /* DMSS ADD0 ATT0 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD0_ATT1 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD0_ATT1.R /* DMSS ADD0 ATT1 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD1_START CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD1_START.R /* DMSS ADD1 START */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD1_ATT0 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD1_ATT0.R /* DMSS ADD1 ATT0 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD1_ATT1 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD1_ATT1.R /* DMSS ADD1 ATT1 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD2_START CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD2_START.R /* DMSS ADD2 START */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD2_ATT0 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD2_ATT0.R /* DMSS ADD2 ATT0 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD2_ATT1 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD2_ATT1.R /* DMSS ADD2 ATT1 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD3_START CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD3_START.R /* DMSS ADD3 START */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD3_ATT0 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD3_ATT0.R /* DMSS ADD3 ATT0 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD3_ATT1 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD3_ATT1.R /* DMSS ADD3 ATT1 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD4_START CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD4_START.R /* DMSS ADD4 START */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD4_ATT0 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD4_ATT0.R /* DMSS ADD4 ATT0 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD4_ATT1 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD4_ATT1.R /* DMSS ADD4 ATT1 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD5_START CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD5_START.R /* DMSS ADD5 START */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD5_ATT0 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD5_ATT0.R /* DMSS ADD5 ATT0 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD5_ATT1 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD5_ATT1.R /* DMSS ADD5 ATT1 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD6_START CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD6_START.R /* DMSS ADD6 START */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD6_ATT0 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD6_ATT0.R /* DMSS ADD6 ATT0 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD6_ATT1 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD6_ATT1.R /* DMSS ADD6 ATT1 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD7_START CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD7_START.R /* DMSS ADD7 START */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD7_ATT0 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD7_ATT0.R /* DMSS ADD7 ATT0 */
#define CEVA_SPF2__INTERFACE_CONFIGURATION_D_ADD7_ATT1 CEVA_SPF2__INTERFACE_CONFIGURATION.D_ADD7_ATT1.R /* DMSS ADD7 ATT1 */

/* INTERRUPT_CONTROL */
#define CEVA_SPF2__INTERRUPT_CONTROL_ISRx0 CEVA_SPF2__INTERRUPT_CONTROL.ISRX[0].R /* Interrupt Status Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICRx0 CEVA_SPF2__INTERRUPT_CONTROL.ICRX[0].R /* Interrupt Clear Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_IGRx0 CEVA_SPF2__INTERRUPT_CONTROL.IGRX[0].R /* Interrupt Generation Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_IMR0 CEVA_SPF2__INTERRUPT_CONTROL.IMR0.R /* Interrupt Mask Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_IGMR CEVA_SPF2__INTERRUPT_CONTROL.IGMR.R /* IGM Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_IVAx0 CEVA_SPF2__INTERRUPT_CONTROL.IVAX[0].R /* Interrupt Vector Address */
#define CEVA_SPF2__INTERRUPT_CONTROL_IVAx1 CEVA_SPF2__INTERRUPT_CONTROL.IVAX[1].R /* Interrupt Vector Address */
#define CEVA_SPF2__INTERRUPT_CONTROL_IVAx2 CEVA_SPF2__INTERRUPT_CONTROL.IVAX[2].R /* Interrupt Vector Address */
#define CEVA_SPF2__INTERRUPT_CONTROL_IVAx3 CEVA_SPF2__INTERRUPT_CONTROL.IVAX[3].R /* Interrupt Vector Address */
#define CEVA_SPF2__INTERRUPT_CONTROL_IVAx4 CEVA_SPF2__INTERRUPT_CONTROL.IVAX[4].R /* Interrupt Vector Address */
#define CEVA_SPF2__INTERRUPT_CONTROL_IVAx5 CEVA_SPF2__INTERRUPT_CONTROL.IVAX[5].R /* Interrupt Vector Address */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFG0 CEVA_SPF2__INTERRUPT_CONTROL.ICFG0.R /* ICFG0 Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFG1 CEVA_SPF2__INTERRUPT_CONTROL.ICFG1.R /* ICFG1 Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFG2 CEVA_SPF2__INTERRUPT_CONTROL.ICFG2.R /* ICFG2 Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFG3 CEVA_SPF2__INTERRUPT_CONTROL.ICFG3.R /* ICFG3 Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFG4 CEVA_SPF2__INTERRUPT_CONTROL.ICFG4.R /* ICFG4 Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFG5 CEVA_SPF2__INTERRUPT_CONTROL.ICFG5.R /* ICFG5 Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFG6 CEVA_SPF2__INTERRUPT_CONTROL.ICFG6.R /* ICFG6 Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFG8 CEVA_SPF2__INTERRUPT_CONTROL.ICFG8.R /* ICFG8 Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx0 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[0].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx1 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[1].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx2 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[2].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx3 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[3].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx4 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[4].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx5 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[5].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx6 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[6].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx7 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[7].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx8 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[8].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx9 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[9].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx10 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[10].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx11 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[11].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx12 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[12].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx13 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[13].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx14 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[14].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx15 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[15].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx16 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[16].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx17 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[17].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx18 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[18].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx19 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[19].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx20 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[20].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx21 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[21].R /* ICFGx Register */
#define CEVA_SPF2__INTERRUPT_CONTROL_ICFGx22 CEVA_SPF2__INTERRUPT_CONTROL.ICFGX[22].R /* ICFGx Register */

/* LCU */
#define LCU_1_LC0_LUTCTRL0   LCU_1.LC[0].LUTCTRL[0].R      /* LC 0 Output 0 LUT Control */
#define LCU_1_LC0_LUTCTRL1   LCU_1.LC[0].LUTCTRL[1].R      /* LC 0 Output 1 LUT Control */
#define LCU_1_LC0_LUTCTRL2   LCU_1.LC[0].LUTCTRL[2].R      /* LC 0 Output 2 LUT Control */
#define LCU_1_LC0_LUTCTRL3   LCU_1.LC[0].LUTCTRL[3].R      /* LC 0 Output 3 LUT Control */
#define LCU_1_LC0_FILT0      LCU_1.LC[0].FILT[0].R         /* LC 0 Output 0 Filter */
#define LCU_1_LC0_FILT1      LCU_1.LC[0].FILT[1].R         /* LC 0 Output 1 Filter */
#define LCU_1_LC0_FILT2      LCU_1.LC[0].FILT[2].R         /* LC 0 Output 2 Filter */
#define LCU_1_LC0_FILT3      LCU_1.LC[0].FILT[3].R         /* LC 0 Output 3 Filter */
#define LCU_1_LC0_INTDMAEN   LCU_1.LC[0].INTDMAEN.R        /* LC 0 Interrupt and DMA Enable */
#define LCU_1_LC0_STS        LCU_1.LC[0].STS.R             /* LC 0 Status */
#define LCU_1_LC0_OUTPOL     LCU_1.LC[0].OUTPOL.R          /* LC 0 Output Polarity Control */
#define LCU_1_LC0_FFILT      LCU_1.LC[0].FFILT.R           /* LC 0 Force Filter */
#define LCU_1_LC0_FCTRL      LCU_1.LC[0].FCTRL.R           /* LC 0 Force Control */
#define LCU_1_LC0_SCTRL      LCU_1.LC[0].SCTRL.R           /* LC 0 Sync Control */
#define LCU_1_LC1_LUTCTRL0   LCU_1.LC[1].LUTCTRL[0].R      /* LC 1 Output 0 LUT Control */
#define LCU_1_LC1_LUTCTRL1   LCU_1.LC[1].LUTCTRL[1].R      /* LC 1 Output 1 LUT Control */
#define LCU_1_LC1_LUTCTRL2   LCU_1.LC[1].LUTCTRL[2].R      /* LC 1 Output 2 LUT Control */
#define LCU_1_LC1_LUTCTRL3   LCU_1.LC[1].LUTCTRL[3].R      /* LC 1 Output 3 LUT Control */
#define LCU_1_LC1_FILT0      LCU_1.LC[1].FILT[0].R         /* LC 1 Output 0 Filter */
#define LCU_1_LC1_FILT1      LCU_1.LC[1].FILT[1].R         /* LC 1 Output 1 Filter */
#define LCU_1_LC1_FILT2      LCU_1.LC[1].FILT[2].R         /* LC 1 Output 2 Filter */
#define LCU_1_LC1_FILT3      LCU_1.LC[1].FILT[3].R         /* LC 1 Output 3 Filter */
#define LCU_1_LC1_INTDMAEN   LCU_1.LC[1].INTDMAEN.R        /* LC 1 Interrupt and DMA Enable */
#define LCU_1_LC1_STS        LCU_1.LC[1].STS.R             /* LC 1 Status */
#define LCU_1_LC1_OUTPOL     LCU_1.LC[1].OUTPOL.R          /* LC 1 Output Polarity Control */
#define LCU_1_LC1_FFILT      LCU_1.LC[1].FFILT.R           /* LC 1 Force Filter */
#define LCU_1_LC1_FCTRL      LCU_1.LC[1].FCTRL.R           /* LC 1 Force Control */
#define LCU_1_LC1_SCTRL      LCU_1.LC[1].SCTRL.R           /* LC 1 Sync Control */
#define LCU_1_LC2_LUTCTRL0   LCU_1.LC[2].LUTCTRL[0].R      /* LC 2 Output 0 LUT Control */
#define LCU_1_LC2_LUTCTRL1   LCU_1.LC[2].LUTCTRL[1].R      /* LC 2 Output 1 LUT Control */
#define LCU_1_LC2_LUTCTRL2   LCU_1.LC[2].LUTCTRL[2].R      /* LC 2 Output 2 LUT Control */
#define LCU_1_LC2_LUTCTRL3   LCU_1.LC[2].LUTCTRL[3].R      /* LC 2 Output 3 LUT Control */
#define LCU_1_LC2_FILT0      LCU_1.LC[2].FILT[0].R         /* LC 2 Output 0 Filter */
#define LCU_1_LC2_FILT1      LCU_1.LC[2].FILT[1].R         /* LC 2 Output 1 Filter */
#define LCU_1_LC2_FILT2      LCU_1.LC[2].FILT[2].R         /* LC 2 Output 2 Filter */
#define LCU_1_LC2_FILT3      LCU_1.LC[2].FILT[3].R         /* LC 2 Output 3 Filter */
#define LCU_1_LC2_INTDMAEN   LCU_1.LC[2].INTDMAEN.R        /* LC 2 Interrupt and DMA Enable */
#define LCU_1_LC2_STS        LCU_1.LC[2].STS.R             /* LC 2 Status */
#define LCU_1_LC2_OUTPOL     LCU_1.LC[2].OUTPOL.R          /* LC 2 Output Polarity Control */
#define LCU_1_LC2_FFILT      LCU_1.LC[2].FFILT.R           /* LC 2 Force Filter */
#define LCU_1_LC2_FCTRL      LCU_1.LC[2].FCTRL.R           /* LC 2 Force Control */
#define LCU_1_LC2_SCTRL      LCU_1.LC[2].SCTRL.R           /* LC 2 Sync Control */
#define LCU_1_MUXSEL0        LCU_1.MUXSEL[0].R             /* Mux Select */
#define LCU_1_MUXSEL1        LCU_1.MUXSEL[1].R             /* Mux Select */
#define LCU_1_MUXSEL2        LCU_1.MUXSEL[2].R             /* Mux Select */
#define LCU_1_MUXSEL3        LCU_1.MUXSEL[3].R             /* Mux Select */
#define LCU_1_MUXSEL4        LCU_1.MUXSEL[4].R             /* Mux Select */
#define LCU_1_MUXSEL5        LCU_1.MUXSEL[5].R             /* Mux Select */
#define LCU_1_MUXSEL6        LCU_1.MUXSEL[6].R             /* Mux Select */
#define LCU_1_MUXSEL7        LCU_1.MUXSEL[7].R             /* Mux Select */
#define LCU_1_MUXSEL8        LCU_1.MUXSEL[8].R             /* Mux Select */
#define LCU_1_MUXSEL9        LCU_1.MUXSEL[9].R             /* Mux Select */
#define LCU_1_MUXSEL10       LCU_1.MUXSEL[10].R            /* Mux Select */
#define LCU_1_MUXSEL11       LCU_1.MUXSEL[11].R            /* Mux Select */
#define LCU_1_CFG            LCU_1.CFG.R                   /* Configuration */
#define LCU_1_SWEN           LCU_1.SWEN.R                  /* Software Override Enable */
#define LCU_1_SWVALUE        LCU_1.SWVALUE.R               /* Software Override Value */
#define LCU_1_OUTEN          LCU_1.OUTEN.R                 /* Output Enable */
#define LCU_1_LCIN           LCU_1.LCIN.R                  /* Logic Inputs */
#define LCU_1_SWOUT          LCU_1.SWOUT.R                 /* Overridden Inputs */
#define LCU_1_LCOUT          LCU_1.LCOUT.R                 /* Logic Outputs */
#define LCU_1_FORCEOUT       LCU_1.FORCEOUT.R              /* Forced Outputs */
#define LCU_1_FORCESTS       LCU_1.FORCESTS.R              /* Force Status */
#define LCU_1_STPEN          LCU_1.STPEN.R                 /* Stop Mode Enable */
#define LCU_1_DBGEN          LCU_1.DBGEN.R                 /* Debug Mode Enable */

#define LCU_0_LC0_LUTCTRL0   LCU_0.LC[0].LUTCTRL[0].R      /* LC 0 Output 0 LUT Control */
#define LCU_0_LC0_LUTCTRL1   LCU_0.LC[0].LUTCTRL[1].R      /* LC 0 Output 1 LUT Control */
#define LCU_0_LC0_LUTCTRL2   LCU_0.LC[0].LUTCTRL[2].R      /* LC 0 Output 2 LUT Control */
#define LCU_0_LC0_LUTCTRL3   LCU_0.LC[0].LUTCTRL[3].R      /* LC 0 Output 3 LUT Control */
#define LCU_0_LC0_FILT0      LCU_0.LC[0].FILT[0].R         /* LC 0 Output 0 Filter */
#define LCU_0_LC0_FILT1      LCU_0.LC[0].FILT[1].R         /* LC 0 Output 1 Filter */
#define LCU_0_LC0_FILT2      LCU_0.LC[0].FILT[2].R         /* LC 0 Output 2 Filter */
#define LCU_0_LC0_FILT3      LCU_0.LC[0].FILT[3].R         /* LC 0 Output 3 Filter */
#define LCU_0_LC0_INTDMAEN   LCU_0.LC[0].INTDMAEN.R        /* LC 0 Interrupt and DMA Enable */
#define LCU_0_LC0_STS        LCU_0.LC[0].STS.R             /* LC 0 Status */
#define LCU_0_LC0_OUTPOL     LCU_0.LC[0].OUTPOL.R          /* LC 0 Output Polarity Control */
#define LCU_0_LC0_FFILT      LCU_0.LC[0].FFILT.R           /* LC 0 Force Filter */
#define LCU_0_LC0_FCTRL      LCU_0.LC[0].FCTRL.R           /* LC 0 Force Control */
#define LCU_0_LC0_SCTRL      LCU_0.LC[0].SCTRL.R           /* LC 0 Sync Control */
#define LCU_0_LC1_LUTCTRL0   LCU_0.LC[1].LUTCTRL[0].R      /* LC 1 Output 0 LUT Control */
#define LCU_0_LC1_LUTCTRL1   LCU_0.LC[1].LUTCTRL[1].R      /* LC 1 Output 1 LUT Control */
#define LCU_0_LC1_LUTCTRL2   LCU_0.LC[1].LUTCTRL[2].R      /* LC 1 Output 2 LUT Control */
#define LCU_0_LC1_LUTCTRL3   LCU_0.LC[1].LUTCTRL[3].R      /* LC 1 Output 3 LUT Control */
#define LCU_0_LC1_FILT0      LCU_0.LC[1].FILT[0].R         /* LC 1 Output 0 Filter */
#define LCU_0_LC1_FILT1      LCU_0.LC[1].FILT[1].R         /* LC 1 Output 1 Filter */
#define LCU_0_LC1_FILT2      LCU_0.LC[1].FILT[2].R         /* LC 1 Output 2 Filter */
#define LCU_0_LC1_FILT3      LCU_0.LC[1].FILT[3].R         /* LC 1 Output 3 Filter */
#define LCU_0_LC1_INTDMAEN   LCU_0.LC[1].INTDMAEN.R        /* LC 1 Interrupt and DMA Enable */
#define LCU_0_LC1_STS        LCU_0.LC[1].STS.R             /* LC 1 Status */
#define LCU_0_LC1_OUTPOL     LCU_0.LC[1].OUTPOL.R          /* LC 1 Output Polarity Control */
#define LCU_0_LC1_FFILT      LCU_0.LC[1].FFILT.R           /* LC 1 Force Filter */
#define LCU_0_LC1_FCTRL      LCU_0.LC[1].FCTRL.R           /* LC 1 Force Control */
#define LCU_0_LC1_SCTRL      LCU_0.LC[1].SCTRL.R           /* LC 1 Sync Control */
#define LCU_0_LC2_LUTCTRL0   LCU_0.LC[2].LUTCTRL[0].R      /* LC 2 Output 0 LUT Control */
#define LCU_0_LC2_LUTCTRL1   LCU_0.LC[2].LUTCTRL[1].R      /* LC 2 Output 1 LUT Control */
#define LCU_0_LC2_LUTCTRL2   LCU_0.LC[2].LUTCTRL[2].R      /* LC 2 Output 2 LUT Control */
#define LCU_0_LC2_LUTCTRL3   LCU_0.LC[2].LUTCTRL[3].R      /* LC 2 Output 3 LUT Control */
#define LCU_0_LC2_FILT0      LCU_0.LC[2].FILT[0].R         /* LC 2 Output 0 Filter */
#define LCU_0_LC2_FILT1      LCU_0.LC[2].FILT[1].R         /* LC 2 Output 1 Filter */
#define LCU_0_LC2_FILT2      LCU_0.LC[2].FILT[2].R         /* LC 2 Output 2 Filter */
#define LCU_0_LC2_FILT3      LCU_0.LC[2].FILT[3].R         /* LC 2 Output 3 Filter */
#define LCU_0_LC2_INTDMAEN   LCU_0.LC[2].INTDMAEN.R        /* LC 2 Interrupt and DMA Enable */
#define LCU_0_LC2_STS        LCU_0.LC[2].STS.R             /* LC 2 Status */
#define LCU_0_LC2_OUTPOL     LCU_0.LC[2].OUTPOL.R          /* LC 2 Output Polarity Control */
#define LCU_0_LC2_FFILT      LCU_0.LC[2].FFILT.R           /* LC 2 Force Filter */
#define LCU_0_LC2_FCTRL      LCU_0.LC[2].FCTRL.R           /* LC 2 Force Control */
#define LCU_0_LC2_SCTRL      LCU_0.LC[2].SCTRL.R           /* LC 2 Sync Control */
#define LCU_0_MUXSEL0        LCU_0.MUXSEL[0].R             /* Mux Select */
#define LCU_0_MUXSEL1        LCU_0.MUXSEL[1].R             /* Mux Select */
#define LCU_0_MUXSEL2        LCU_0.MUXSEL[2].R             /* Mux Select */
#define LCU_0_MUXSEL3        LCU_0.MUXSEL[3].R             /* Mux Select */
#define LCU_0_MUXSEL4        LCU_0.MUXSEL[4].R             /* Mux Select */
#define LCU_0_MUXSEL5        LCU_0.MUXSEL[5].R             /* Mux Select */
#define LCU_0_MUXSEL6        LCU_0.MUXSEL[6].R             /* Mux Select */
#define LCU_0_MUXSEL7        LCU_0.MUXSEL[7].R             /* Mux Select */
#define LCU_0_MUXSEL8        LCU_0.MUXSEL[8].R             /* Mux Select */
#define LCU_0_MUXSEL9        LCU_0.MUXSEL[9].R             /* Mux Select */
#define LCU_0_MUXSEL10       LCU_0.MUXSEL[10].R            /* Mux Select */
#define LCU_0_MUXSEL11       LCU_0.MUXSEL[11].R            /* Mux Select */
#define LCU_0_CFG            LCU_0.CFG.R                   /* Configuration */
#define LCU_0_SWEN           LCU_0.SWEN.R                  /* Software Override Enable */
#define LCU_0_SWVALUE        LCU_0.SWVALUE.R               /* Software Override Value */
#define LCU_0_OUTEN          LCU_0.OUTEN.R                 /* Output Enable */
#define LCU_0_LCIN           LCU_0.LCIN.R                  /* Logic Inputs */
#define LCU_0_SWOUT          LCU_0.SWOUT.R                 /* Overridden Inputs */
#define LCU_0_LCOUT          LCU_0.LCOUT.R                 /* Logic Outputs */
#define LCU_0_FORCEOUT       LCU_0.FORCEOUT.R              /* Forced Outputs */
#define LCU_0_FORCESTS       LCU_0.FORCESTS.R              /* Force Status */
#define LCU_0_STPEN          LCU_0.STPEN.R                 /* Stop Mode Enable */
#define LCU_0_DBGEN          LCU_0.DBGEN.R                 /* Debug Mode Enable */

/* LFAST */
#define LFAST_0_MCR          LFAST_0.MCR.R                 /* LFAST Mode Configuration */
#define LFAST_0_SCR          LFAST_0.SCR.R                 /* LFAST Speed Control */
#define LFAST_0_COCR         LFAST_0.COCR.R                /* LFAST Correlator Control */
#define LFAST_0_TMCR         LFAST_0.TMCR.R                /* LFAST Test Mode Control */
#define LFAST_0_ALCR         LFAST_0.ALCR.R                /* LFAST Auto Loopback Control */
#define LFAST_0_RCDCR        LFAST_0.RCDCR.R               /* LFAST Rate Change Delay Control */
#define LFAST_0_ICR          LFAST_0.ICR.R                 /* LFAST ICLC Control */
#define LFAST_0_PICR         LFAST_0.PICR.R                /* LFAST Ping Control */
#define LFAST_0_RFCR         LFAST_0.RFCR.R                /* LFAST Rx FIFO CTS Control */
#define LFAST_0_TIER         LFAST_0.TIER.R                /* LFAST Tx Interrupt Enable */
#define LFAST_0_RIER         LFAST_0.RIER.R                /* LFAST Rx Interrupt Enable */
#define LFAST_0_RIIER        LFAST_0.RIIER.R               /* LFAST Rx ICLC Interrupt Enable */
#define LFAST_0_PLLCR        LFAST_0.PLLCR.R               /* LFAST PLL Control */
#define LFAST_0_UNSTCR       LFAST_0.UNSTCR.R              /* LFAST Unsolicited Tx Control */
#define LFAST_0_UNSTDR0      LFAST_0.UNSTDR[0].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_UNSTDR1      LFAST_0.UNSTDR[1].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_UNSTDR2      LFAST_0.UNSTDR[2].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_UNSTDR3      LFAST_0.UNSTDR[3].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_UNSTDR4      LFAST_0.UNSTDR[4].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_UNSTDR5      LFAST_0.UNSTDR[5].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_UNSTDR6      LFAST_0.UNSTDR[6].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_UNSTDR7      LFAST_0.UNSTDR[7].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_UNSTDR8      LFAST_0.UNSTDR[8].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_0_GSR          LFAST_0.GSR.R                 /* LFAST Global Status */
#define LFAST_0_PISR         LFAST_0.PISR.R                /* LFAST Ping Status */
#define LFAST_0_DFSR         LFAST_0.DFSR.R                /* LFAST Data Frame Status */
#define LFAST_0_TISR         LFAST_0.TISR.R                /* LFAST Tx Interrupt Status */
#define LFAST_0_RISR         LFAST_0.RISR.R                /* LFAST Rx Interrupt Status */
#define LFAST_0_RIISR        LFAST_0.RIISR.R               /* LFAST Rx ICLC Interrupt Status */
#define LFAST_0_PLLLSR       LFAST_0.PLLLSR.R              /* LFAST PLL and LVDS Status */
#define LFAST_0_UNSRSR       LFAST_0.UNSRSR.R              /* LFAST Unsolicited Rx Status */
#define LFAST_0_UNSRDR0      LFAST_0.UNSRDR[0].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_UNSRDR1      LFAST_0.UNSRDR[1].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_UNSRDR2      LFAST_0.UNSRDR[2].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_UNSRDR3      LFAST_0.UNSRDR[3].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_UNSRDR4      LFAST_0.UNSRDR[4].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_UNSRDR5      LFAST_0.UNSRDR[5].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_UNSRDR6      LFAST_0.UNSRDR[6].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_UNSRDR7      LFAST_0.UNSRDR[7].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_UNSRDR8      LFAST_0.UNSRDR[8].R           /* LFAST Unsolicited Rx Data */
#define LFAST_0_GPR0         LFAST_0.GPR[0].R              /* General Purpose registers */

#define LFAST_1_MCR          LFAST_1.MCR.R                 /* LFAST Mode Configuration */
#define LFAST_1_SCR          LFAST_1.SCR.R                 /* LFAST Speed Control */
#define LFAST_1_COCR         LFAST_1.COCR.R                /* LFAST Correlator Control */
#define LFAST_1_TMCR         LFAST_1.TMCR.R                /* LFAST Test Mode Control */
#define LFAST_1_ALCR         LFAST_1.ALCR.R                /* LFAST Auto Loopback Control */
#define LFAST_1_RCDCR        LFAST_1.RCDCR.R               /* LFAST Rate Change Delay Control */
#define LFAST_1_ICR          LFAST_1.ICR.R                 /* LFAST ICLC Control */
#define LFAST_1_PICR         LFAST_1.PICR.R                /* LFAST Ping Control */
#define LFAST_1_RFCR         LFAST_1.RFCR.R                /* LFAST Rx FIFO CTS Control */
#define LFAST_1_TIER         LFAST_1.TIER.R                /* LFAST Tx Interrupt Enable */
#define LFAST_1_RIER         LFAST_1.RIER.R                /* LFAST Rx Interrupt Enable */
#define LFAST_1_RIIER        LFAST_1.RIIER.R               /* LFAST Rx ICLC Interrupt Enable */
#define LFAST_1_PLLCR        LFAST_1.PLLCR.R               /* LFAST PLL Control */
#define LFAST_1_UNSTCR       LFAST_1.UNSTCR.R              /* LFAST Unsolicited Tx Control */
#define LFAST_1_UNSTDR0      LFAST_1.UNSTDR[0].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_UNSTDR1      LFAST_1.UNSTDR[1].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_UNSTDR2      LFAST_1.UNSTDR[2].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_UNSTDR3      LFAST_1.UNSTDR[3].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_UNSTDR4      LFAST_1.UNSTDR[4].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_UNSTDR5      LFAST_1.UNSTDR[5].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_UNSTDR6      LFAST_1.UNSTDR[6].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_UNSTDR7      LFAST_1.UNSTDR[7].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_UNSTDR8      LFAST_1.UNSTDR[8].R           /* LFAST Unsolicited Tx Data Registers */
#define LFAST_1_GSR          LFAST_1.GSR.R                 /* LFAST Global Status */
#define LFAST_1_PISR         LFAST_1.PISR.R                /* LFAST Ping Status */
#define LFAST_1_DFSR         LFAST_1.DFSR.R                /* LFAST Data Frame Status */
#define LFAST_1_TISR         LFAST_1.TISR.R                /* LFAST Tx Interrupt Status */
#define LFAST_1_RISR         LFAST_1.RISR.R                /* LFAST Rx Interrupt Status */
#define LFAST_1_RIISR        LFAST_1.RIISR.R               /* LFAST Rx ICLC Interrupt Status */
#define LFAST_1_PLLLSR       LFAST_1.PLLLSR.R              /* LFAST PLL and LVDS Status */
#define LFAST_1_UNSRSR       LFAST_1.UNSRSR.R              /* LFAST Unsolicited Rx Status */
#define LFAST_1_UNSRDR0      LFAST_1.UNSRDR[0].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_UNSRDR1      LFAST_1.UNSRDR[1].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_UNSRDR2      LFAST_1.UNSRDR[2].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_UNSRDR3      LFAST_1.UNSRDR[3].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_UNSRDR4      LFAST_1.UNSRDR[4].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_UNSRDR5      LFAST_1.UNSRDR[5].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_UNSRDR6      LFAST_1.UNSRDR[6].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_UNSRDR7      LFAST_1.UNSRDR[7].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_UNSRDR8      LFAST_1.UNSRDR[8].R           /* LFAST Unsolicited Rx Data */
#define LFAST_1_GPR0         LFAST_1.GPR[0].R              /* General Purpose registers */

/* LINFLEXD */
#define LINFLEX_3_LINCR1     LINFLEX_3.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_3_LINIER     LINFLEX_3.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_3_LINSR      LINFLEX_3.LINSR.R             /* LIN Status */
#define LINFLEX_3_LINESR     LINFLEX_3.LINESR.R            /* LIN Error Status */
#define LINFLEX_3_UARTCR     LINFLEX_3.UARTCR.R            /* UART Mode Control */
#define LINFLEX_3_UARTSR     LINFLEX_3.UARTSR.R            /* UART Mode Status */
#define LINFLEX_3_LINTCSR    LINFLEX_3.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_3_LINOCR     LINFLEX_3.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_3_LINTOCR    LINFLEX_3.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_3_LINFBRR    LINFLEX_3.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_3_LINIBRR    LINFLEX_3.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_3_LINCFR     LINFLEX_3.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_3_LINCR2     LINFLEX_3.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_3_BIDR       LINFLEX_3.BIDR.R              /* Buffer Identifier */
#define LINFLEX_3_BDRL       LINFLEX_3.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_3_BDRM       LINFLEX_3.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_3_GCR        LINFLEX_3.GCR.R               /* Global Control */
#define LINFLEX_3_UARTPTO    LINFLEX_3.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_3_UARTCTO    LINFLEX_3.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_3_DMATXE     LINFLEX_3.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_3_DMARXE     LINFLEX_3.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_4_LINCR1     LINFLEX_4.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_4_LINIER     LINFLEX_4.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_4_LINSR      LINFLEX_4.LINSR.R             /* LIN Status */
#define LINFLEX_4_LINESR     LINFLEX_4.LINESR.R            /* LIN Error Status */
#define LINFLEX_4_UARTCR     LINFLEX_4.UARTCR.R            /* UART Mode Control */
#define LINFLEX_4_UARTSR     LINFLEX_4.UARTSR.R            /* UART Mode Status */
#define LINFLEX_4_LINTCSR    LINFLEX_4.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_4_LINOCR     LINFLEX_4.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_4_LINTOCR    LINFLEX_4.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_4_LINFBRR    LINFLEX_4.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_4_LINIBRR    LINFLEX_4.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_4_LINCFR     LINFLEX_4.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_4_LINCR2     LINFLEX_4.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_4_BIDR       LINFLEX_4.BIDR.R              /* Buffer Identifier */
#define LINFLEX_4_BDRL       LINFLEX_4.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_4_BDRM       LINFLEX_4.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_4_GCR        LINFLEX_4.GCR.R               /* Global Control */
#define LINFLEX_4_UARTPTO    LINFLEX_4.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_4_UARTCTO    LINFLEX_4.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_4_DMATXE     LINFLEX_4.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_4_DMARXE     LINFLEX_4.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_5_LINCR1     LINFLEX_5.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_5_LINIER     LINFLEX_5.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_5_LINSR      LINFLEX_5.LINSR.R             /* LIN Status */
#define LINFLEX_5_LINESR     LINFLEX_5.LINESR.R            /* LIN Error Status */
#define LINFLEX_5_UARTCR     LINFLEX_5.UARTCR.R            /* UART Mode Control */
#define LINFLEX_5_UARTSR     LINFLEX_5.UARTSR.R            /* UART Mode Status */
#define LINFLEX_5_LINTCSR    LINFLEX_5.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_5_LINOCR     LINFLEX_5.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_5_LINTOCR    LINFLEX_5.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_5_LINFBRR    LINFLEX_5.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_5_LINIBRR    LINFLEX_5.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_5_LINCFR     LINFLEX_5.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_5_LINCR2     LINFLEX_5.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_5_BIDR       LINFLEX_5.BIDR.R              /* Buffer Identifier */
#define LINFLEX_5_BDRL       LINFLEX_5.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_5_BDRM       LINFLEX_5.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_5_GCR        LINFLEX_5.GCR.R               /* Global Control */
#define LINFLEX_5_UARTPTO    LINFLEX_5.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_5_UARTCTO    LINFLEX_5.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_5_DMATXE     LINFLEX_5.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_5_DMARXE     LINFLEX_5.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_0_LINCR1     LINFLEX_0.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_0_LINIER     LINFLEX_0.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_0_LINSR      LINFLEX_0.LINSR.R             /* LIN Status */
#define LINFLEX_0_LINESR     LINFLEX_0.LINESR.R            /* LIN Error Status */
#define LINFLEX_0_UARTCR     LINFLEX_0.UARTCR.R            /* UART Mode Control */
#define LINFLEX_0_UARTSR     LINFLEX_0.UARTSR.R            /* UART Mode Status */
#define LINFLEX_0_LINTCSR    LINFLEX_0.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_0_LINOCR     LINFLEX_0.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_0_LINTOCR    LINFLEX_0.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_0_LINFBRR    LINFLEX_0.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_0_LINIBRR    LINFLEX_0.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_0_LINCFR     LINFLEX_0.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_0_LINCR2     LINFLEX_0.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_0_BIDR       LINFLEX_0.BIDR.R              /* Buffer Identifier */
#define LINFLEX_0_BDRL       LINFLEX_0.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_0_BDRM       LINFLEX_0.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_0_GCR        LINFLEX_0.GCR.R               /* Global Control */
#define LINFLEX_0_UARTPTO    LINFLEX_0.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_0_UARTCTO    LINFLEX_0.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_0_DMATXE     LINFLEX_0.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_0_DMARXE     LINFLEX_0.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_1_LINCR1     LINFLEX_1.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_1_LINIER     LINFLEX_1.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_1_LINSR      LINFLEX_1.LINSR.R             /* LIN Status */
#define LINFLEX_1_LINESR     LINFLEX_1.LINESR.R            /* LIN Error Status */
#define LINFLEX_1_UARTCR     LINFLEX_1.UARTCR.R            /* UART Mode Control */
#define LINFLEX_1_UARTSR     LINFLEX_1.UARTSR.R            /* UART Mode Status */
#define LINFLEX_1_LINTCSR    LINFLEX_1.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_1_LINOCR     LINFLEX_1.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_1_LINTOCR    LINFLEX_1.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_1_LINFBRR    LINFLEX_1.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_1_LINIBRR    LINFLEX_1.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_1_LINCFR     LINFLEX_1.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_1_LINCR2     LINFLEX_1.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_1_BIDR       LINFLEX_1.BIDR.R              /* Buffer Identifier */
#define LINFLEX_1_BDRL       LINFLEX_1.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_1_BDRM       LINFLEX_1.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_1_GCR        LINFLEX_1.GCR.R               /* Global Control */
#define LINFLEX_1_UARTPTO    LINFLEX_1.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_1_UARTCTO    LINFLEX_1.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_1_DMATXE     LINFLEX_1.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_1_DMARXE     LINFLEX_1.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_2_LINCR1     LINFLEX_2.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_2_LINIER     LINFLEX_2.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_2_LINSR      LINFLEX_2.LINSR.R             /* LIN Status */
#define LINFLEX_2_LINESR     LINFLEX_2.LINESR.R            /* LIN Error Status */
#define LINFLEX_2_UARTCR     LINFLEX_2.UARTCR.R            /* UART Mode Control */
#define LINFLEX_2_UARTSR     LINFLEX_2.UARTSR.R            /* UART Mode Status */
#define LINFLEX_2_LINTCSR    LINFLEX_2.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_2_LINOCR     LINFLEX_2.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_2_LINTOCR    LINFLEX_2.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_2_LINFBRR    LINFLEX_2.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_2_LINIBRR    LINFLEX_2.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_2_LINCFR     LINFLEX_2.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_2_LINCR2     LINFLEX_2.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_2_BIDR       LINFLEX_2.BIDR.R              /* Buffer Identifier */
#define LINFLEX_2_BDRL       LINFLEX_2.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_2_BDRM       LINFLEX_2.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_2_GCR        LINFLEX_2.GCR.R               /* Global Control */
#define LINFLEX_2_UARTPTO    LINFLEX_2.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_2_UARTCTO    LINFLEX_2.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_2_DMATXE     LINFLEX_2.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_2_DMARXE     LINFLEX_2.DMARXE.R            /* DMA Rx Enable */

#define MSC_0_LIN_LINCR1     MSC_0_LIN.LINCR1.R            /* LIN Control Register 1 */
#define MSC_0_LIN_LINIER     MSC_0_LIN.LINIER.R            /* LIN Interrupt Enable */
#define MSC_0_LIN_LINSR      MSC_0_LIN.LINSR.R             /* LIN Status */
#define MSC_0_LIN_LINESR     MSC_0_LIN.LINESR.R            /* LIN Error Status */
#define MSC_0_LIN_UARTCR     MSC_0_LIN.UARTCR.R            /* UART Mode Control */
#define MSC_0_LIN_UARTSR     MSC_0_LIN.UARTSR.R            /* UART Mode Status */
#define MSC_0_LIN_LINTCSR    MSC_0_LIN.LINTCSR.R           /* LIN Time-Out Control Status */
#define MSC_0_LIN_LINOCR     MSC_0_LIN.LINOCR.R            /* LIN Output Compare */
#define MSC_0_LIN_LINTOCR    MSC_0_LIN.LINTOCR.R           /* LIN Time-Out Control */
#define MSC_0_LIN_LINFBRR    MSC_0_LIN.LINFBRR.R           /* LIN Fractional Baud Rate */
#define MSC_0_LIN_LINIBRR    MSC_0_LIN.LINIBRR.R           /* LIN Integer Baud Rate */
#define MSC_0_LIN_LINCFR     MSC_0_LIN.LINCFR.R            /* LIN Checksum Field */
#define MSC_0_LIN_LINCR2     MSC_0_LIN.LINCR2.R            /* LIN Control Register 2 */
#define MSC_0_LIN_BIDR       MSC_0_LIN.BIDR.R              /* Buffer Identifier */
#define MSC_0_LIN_BDRL       MSC_0_LIN.BDRL.R              /* Buffer Data Register Least Significant */
#define MSC_0_LIN_BDRM       MSC_0_LIN.BDRM.R              /* Buffer Data Register Most Significant */
#define MSC_0_LIN_GCR        MSC_0_LIN.GCR.R               /* Global Control */
#define MSC_0_LIN_UARTPTO    MSC_0_LIN.UARTPTO.R           /* UART Preset Timeout */
#define MSC_0_LIN_UARTCTO    MSC_0_LIN.UARTCTO.R           /* UART Current Timeout */
#define MSC_0_LIN_DMATXE     MSC_0_LIN.DMATXE.R            /* DMA Tx Enable */
#define MSC_0_LIN_DMARXE     MSC_0_LIN.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_6_LINCR1     LINFLEX_6.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_6_LINIER     LINFLEX_6.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_6_LINSR      LINFLEX_6.LINSR.R             /* LIN Status */
#define LINFLEX_6_LINESR     LINFLEX_6.LINESR.R            /* LIN Error Status */
#define LINFLEX_6_UARTCR     LINFLEX_6.UARTCR.R            /* UART Mode Control */
#define LINFLEX_6_UARTSR     LINFLEX_6.UARTSR.R            /* UART Mode Status */
#define LINFLEX_6_LINTCSR    LINFLEX_6.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_6_LINOCR     LINFLEX_6.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_6_LINTOCR    LINFLEX_6.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_6_LINFBRR    LINFLEX_6.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_6_LINIBRR    LINFLEX_6.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_6_LINCFR     LINFLEX_6.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_6_LINCR2     LINFLEX_6.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_6_BIDR       LINFLEX_6.BIDR.R              /* Buffer Identifier */
#define LINFLEX_6_BDRL       LINFLEX_6.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_6_BDRM       LINFLEX_6.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_6_GCR        LINFLEX_6.GCR.R               /* Global Control */
#define LINFLEX_6_UARTPTO    LINFLEX_6.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_6_UARTCTO    LINFLEX_6.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_6_DMATXE     LINFLEX_6.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_6_DMARXE     LINFLEX_6.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_7_LINCR1     LINFLEX_7.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_7_LINIER     LINFLEX_7.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_7_LINSR      LINFLEX_7.LINSR.R             /* LIN Status */
#define LINFLEX_7_LINESR     LINFLEX_7.LINESR.R            /* LIN Error Status */
#define LINFLEX_7_UARTCR     LINFLEX_7.UARTCR.R            /* UART Mode Control */
#define LINFLEX_7_UARTSR     LINFLEX_7.UARTSR.R            /* UART Mode Status */
#define LINFLEX_7_LINTCSR    LINFLEX_7.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_7_LINOCR     LINFLEX_7.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_7_LINTOCR    LINFLEX_7.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_7_LINFBRR    LINFLEX_7.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_7_LINIBRR    LINFLEX_7.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_7_LINCFR     LINFLEX_7.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_7_LINCR2     LINFLEX_7.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_7_BIDR       LINFLEX_7.BIDR.R              /* Buffer Identifier */
#define LINFLEX_7_BDRL       LINFLEX_7.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_7_BDRM       LINFLEX_7.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_7_GCR        LINFLEX_7.GCR.R               /* Global Control */
#define LINFLEX_7_UARTPTO    LINFLEX_7.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_7_UARTCTO    LINFLEX_7.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_7_DMATXE     LINFLEX_7.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_7_DMARXE     LINFLEX_7.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_8_LINCR1     LINFLEX_8.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_8_LINIER     LINFLEX_8.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_8_LINSR      LINFLEX_8.LINSR.R             /* LIN Status */
#define LINFLEX_8_LINESR     LINFLEX_8.LINESR.R            /* LIN Error Status */
#define LINFLEX_8_UARTCR     LINFLEX_8.UARTCR.R            /* UART Mode Control */
#define LINFLEX_8_UARTSR     LINFLEX_8.UARTSR.R            /* UART Mode Status */
#define LINFLEX_8_LINTCSR    LINFLEX_8.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_8_LINOCR     LINFLEX_8.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_8_LINTOCR    LINFLEX_8.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_8_LINFBRR    LINFLEX_8.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_8_LINIBRR    LINFLEX_8.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_8_LINCFR     LINFLEX_8.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_8_LINCR2     LINFLEX_8.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_8_BIDR       LINFLEX_8.BIDR.R              /* Buffer Identifier */
#define LINFLEX_8_BDRL       LINFLEX_8.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_8_BDRM       LINFLEX_8.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_8_GCR        LINFLEX_8.GCR.R               /* Global Control */
#define LINFLEX_8_UARTPTO    LINFLEX_8.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_8_UARTCTO    LINFLEX_8.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_8_DMATXE     LINFLEX_8.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_8_DMARXE     LINFLEX_8.DMARXE.R            /* DMA Rx Enable */

#define LINFLEX_10_LINCR1    LINFLEX_10.LINCR1.R           /* LIN Control Register 1 */
#define LINFLEX_10_LINIER    LINFLEX_10.LINIER.R           /* LIN Interrupt Enable */
#define LINFLEX_10_LINSR     LINFLEX_10.LINSR.R            /* LIN Status */
#define LINFLEX_10_LINESR    LINFLEX_10.LINESR.R           /* LIN Error Status */
#define LINFLEX_10_UARTCR    LINFLEX_10.UARTCR.R           /* UART Mode Control */
#define LINFLEX_10_UARTSR    LINFLEX_10.UARTSR.R           /* UART Mode Status */
#define LINFLEX_10_LINTCSR   LINFLEX_10.LINTCSR.R          /* LIN Time-Out Control Status */
#define LINFLEX_10_LINOCR    LINFLEX_10.LINOCR.R           /* LIN Output Compare */
#define LINFLEX_10_LINTOCR   LINFLEX_10.LINTOCR.R          /* LIN Time-Out Control */
#define LINFLEX_10_LINFBRR   LINFLEX_10.LINFBRR.R          /* LIN Fractional Baud Rate */
#define LINFLEX_10_LINIBRR   LINFLEX_10.LINIBRR.R          /* LIN Integer Baud Rate */
#define LINFLEX_10_LINCFR    LINFLEX_10.LINCFR.R           /* LIN Checksum Field */
#define LINFLEX_10_LINCR2    LINFLEX_10.LINCR2.R           /* LIN Control Register 2 */
#define LINFLEX_10_BIDR      LINFLEX_10.BIDR.R             /* Buffer Identifier */
#define LINFLEX_10_BDRL      LINFLEX_10.BDRL.R             /* Buffer Data Register Least Significant */
#define LINFLEX_10_BDRM      LINFLEX_10.BDRM.R             /* Buffer Data Register Most Significant */
#define LINFLEX_10_GCR       LINFLEX_10.GCR.R              /* Global Control */
#define LINFLEX_10_UARTPTO   LINFLEX_10.UARTPTO.R          /* UART Preset Timeout */
#define LINFLEX_10_UARTCTO   LINFLEX_10.UARTCTO.R          /* UART Current Timeout */
#define LINFLEX_10_DMATXE    LINFLEX_10.DMATXE.R           /* DMA Tx Enable */
#define LINFLEX_10_DMARXE    LINFLEX_10.DMARXE.R           /* DMA Rx Enable */

#define LINFLEX_11_LINCR1    LINFLEX_11.LINCR1.R           /* LIN Control Register 1 */
#define LINFLEX_11_LINIER    LINFLEX_11.LINIER.R           /* LIN Interrupt Enable */
#define LINFLEX_11_LINSR     LINFLEX_11.LINSR.R            /* LIN Status */
#define LINFLEX_11_LINESR    LINFLEX_11.LINESR.R           /* LIN Error Status */
#define LINFLEX_11_UARTCR    LINFLEX_11.UARTCR.R           /* UART Mode Control */
#define LINFLEX_11_UARTSR    LINFLEX_11.UARTSR.R           /* UART Mode Status */
#define LINFLEX_11_LINTCSR   LINFLEX_11.LINTCSR.R          /* LIN Time-Out Control Status */
#define LINFLEX_11_LINOCR    LINFLEX_11.LINOCR.R           /* LIN Output Compare */
#define LINFLEX_11_LINTOCR   LINFLEX_11.LINTOCR.R          /* LIN Time-Out Control */
#define LINFLEX_11_LINFBRR   LINFLEX_11.LINFBRR.R          /* LIN Fractional Baud Rate */
#define LINFLEX_11_LINIBRR   LINFLEX_11.LINIBRR.R          /* LIN Integer Baud Rate */
#define LINFLEX_11_LINCFR    LINFLEX_11.LINCFR.R           /* LIN Checksum Field */
#define LINFLEX_11_LINCR2    LINFLEX_11.LINCR2.R           /* LIN Control Register 2 */
#define LINFLEX_11_BIDR      LINFLEX_11.BIDR.R             /* Buffer Identifier */
#define LINFLEX_11_BDRL      LINFLEX_11.BDRL.R             /* Buffer Data Register Least Significant */
#define LINFLEX_11_BDRM      LINFLEX_11.BDRM.R             /* Buffer Data Register Most Significant */
#define LINFLEX_11_GCR       LINFLEX_11.GCR.R              /* Global Control */
#define LINFLEX_11_UARTPTO   LINFLEX_11.UARTPTO.R          /* UART Preset Timeout */
#define LINFLEX_11_UARTCTO   LINFLEX_11.UARTCTO.R          /* UART Current Timeout */
#define LINFLEX_11_DMATXE    LINFLEX_11.DMATXE.R           /* DMA Tx Enable */
#define LINFLEX_11_DMARXE    LINFLEX_11.DMARXE.R           /* DMA Rx Enable */

#define LINFLEX_9_LINCR1     LINFLEX_9.LINCR1.R            /* LIN Control Register 1 */
#define LINFLEX_9_LINIER     LINFLEX_9.LINIER.R            /* LIN Interrupt Enable */
#define LINFLEX_9_LINSR      LINFLEX_9.LINSR.R             /* LIN Status */
#define LINFLEX_9_LINESR     LINFLEX_9.LINESR.R            /* LIN Error Status */
#define LINFLEX_9_UARTCR     LINFLEX_9.UARTCR.R            /* UART Mode Control */
#define LINFLEX_9_UARTSR     LINFLEX_9.UARTSR.R            /* UART Mode Status */
#define LINFLEX_9_LINTCSR    LINFLEX_9.LINTCSR.R           /* LIN Time-Out Control Status */
#define LINFLEX_9_LINOCR     LINFLEX_9.LINOCR.R            /* LIN Output Compare */
#define LINFLEX_9_LINTOCR    LINFLEX_9.LINTOCR.R           /* LIN Time-Out Control */
#define LINFLEX_9_LINFBRR    LINFLEX_9.LINFBRR.R           /* LIN Fractional Baud Rate */
#define LINFLEX_9_LINIBRR    LINFLEX_9.LINIBRR.R           /* LIN Integer Baud Rate */
#define LINFLEX_9_LINCFR     LINFLEX_9.LINCFR.R            /* LIN Checksum Field */
#define LINFLEX_9_LINCR2     LINFLEX_9.LINCR2.R            /* LIN Control Register 2 */
#define LINFLEX_9_BIDR       LINFLEX_9.BIDR.R              /* Buffer Identifier */
#define LINFLEX_9_BDRL       LINFLEX_9.BDRL.R              /* Buffer Data Register Least Significant */
#define LINFLEX_9_BDRM       LINFLEX_9.BDRM.R              /* Buffer Data Register Most Significant */
#define LINFLEX_9_GCR        LINFLEX_9.GCR.R               /* Global Control */
#define LINFLEX_9_UARTPTO    LINFLEX_9.UARTPTO.R           /* UART Preset Timeout */
#define LINFLEX_9_UARTCTO    LINFLEX_9.UARTCTO.R           /* UART Current Timeout */
#define LINFLEX_9_DMATXE     LINFLEX_9.DMATXE.R            /* DMA Tx Enable */
#define LINFLEX_9_DMARXE     LINFLEX_9.DMARXE.R            /* DMA Rx Enable */

/* LLC_CSR */
#define RTU0__LLC_CSR_CCUTCR RTU0__LLC_CSR.CCUTCR.R        /* Transaction Control */
#define RTU0__LLC_CSR_CCUTAR RTU0__LLC_CSR.CCUTAR.R        /* Transaction Activity */
#define RTU0__LLC_CSR_CCUCTCR RTU0__LLC_CSR.CCUCTCR.R      /* Cache Transaction Control */
#define RTU0__LLC_CSR_CCUCTAR RTU0__LLC_CSR.CCUCTAR.R      /* Cache Transaction Activity */
#define RTU0__LLC_CSR_CCUCAOR RTU0__LLC_CSR.CCUCAOR.R      /* Allocation Override */
#define RTU0__LLC_CSR_CCUSPCR0 RTU0__LLC_CSR.CCUSPCR0.R    /* Scratchpad Configuration 0 */
#define RTU0__LLC_CSR_CCUSPCR1 RTU0__LLC_CSR.CCUSPCR1.R    /* Scratchpad Configuration 1 */
#define RTU0__LLC_CSR_CCUSPBR0 RTU0__LLC_CSR.CCUSPBR0.R    /* Scratchpad Base Address 0 */
#define RTU0__LLC_CSR_CCUWPCRA0 RTU0__LLC_CSR.CCUWPCR[0].CCUWPCRA.R /* Way Partitioning Control A 0 */
#define RTU0__LLC_CSR_CCUWPCRB0 RTU0__LLC_CSR.CCUWPCR[0].CCUWPCRB.R /* Way Partitioning Control B 0 */
#define RTU0__LLC_CSR_CCUWPCRA1 RTU0__LLC_CSR.CCUWPCR[1].CCUWPCRA.R /* Way Partitioning Control A 1 */
#define RTU0__LLC_CSR_CCUWPCRB1 RTU0__LLC_CSR.CCUWPCR[1].CCUWPCRB.R /* Way Partitioning Control B 1 */
#define RTU0__LLC_CSR_CCUWPCRA2 RTU0__LLC_CSR.CCUWPCR[2].CCUWPCRA.R /* Way Partitioning Control A 2 */
#define RTU0__LLC_CSR_CCUWPCRB2 RTU0__LLC_CSR.CCUWPCR[2].CCUWPCRB.R /* Way Partitioning Control B 2 */
#define RTU0__LLC_CSR_CCUWPCRA3 RTU0__LLC_CSR.CCUWPCR[3].CCUWPCRA.R /* Way Partitioning Control A 3 */
#define RTU0__LLC_CSR_CCUWPCRB3 RTU0__LLC_CSR.CCUWPCR[3].CCUWPCRB.R /* Way Partitioning Control B 3 */
#define RTU0__LLC_CSR_CCUWPCRA4 RTU0__LLC_CSR.CCUWPCR[4].CCUWPCRA.R /* Way Partitioning Control A 4 */
#define RTU0__LLC_CSR_CCUWPCRB4 RTU0__LLC_CSR.CCUWPCR[4].CCUWPCRB.R /* Way Partitioning Control B 4 */
#define RTU0__LLC_CSR_CCUWPCRA5 RTU0__LLC_CSR.CCUWPCR[5].CCUWPCRA.R /* Way Partitioning Control A 5 */
#define RTU0__LLC_CSR_CCUWPCRB5 RTU0__LLC_CSR.CCUWPCR[5].CCUWPCRB.R /* Way Partitioning Control B 5 */
#define RTU0__LLC_CSR_CCUWPCRA6 RTU0__LLC_CSR.CCUWPCR[6].CCUWPCRA.R /* Way Partitioning Control A 6 */
#define RTU0__LLC_CSR_CCUWPCRB6 RTU0__LLC_CSR.CCUWPCR[6].CCUWPCRB.R /* Way Partitioning Control B 6 */
#define RTU0__LLC_CSR_CCUWPCRA7 RTU0__LLC_CSR.CCUWPCR[7].CCUWPCRA.R /* Way Partitioning Control A 7 */
#define RTU0__LLC_CSR_CCUWPCRB7 RTU0__LLC_CSR.CCUWPCR[7].CCUWPCRB.R /* Way Partitioning Control B 7 */
#define RTU0__LLC_CSR_CCUCMCR RTU0__LLC_CSR.CCUCMCR.R      /* Maintenance Control */
#define RTU0__LLC_CSR_CCUCMAR RTU0__LLC_CSR.CCUCMAR.R      /* Maintenance Activity */
#define RTU0__LLC_CSR_CCUCMLR0 RTU0__LLC_CSR.CCUCMLR0.R    /* Maintenance Location 0 */
#define RTU0__LLC_CSR_CCUCMLR1 RTU0__LLC_CSR.CCUCMLR1.R    /* Maintenance Location 1 */
#define RTU0__LLC_CSR_CCUCMLR2 RTU0__LLC_CSR.CCUCMLR2.R    /* Maintenance Location 2 */
#define RTU0__LLC_CSR_CCUCMDR RTU0__LLC_CSR.CCUCMDR.R      /* Maintenance Data */
#define RTU0__LLC_CSR_CCUCECR RTU0__LLC_CSR.CCUCECR.R      /* Correctable Error Control */
#define RTU0__LLC_CSR_CCUCESR RTU0__LLC_CSR.CCUCESR.R      /* Correctable Error Status */
#define RTU0__LLC_CSR_CCUCESAR RTU0__LLC_CSR.CCUCESAR.R    /* Correctable Error Status Alias */
#define RTU0__LLC_CSR_CCUCELR0 RTU0__LLC_CSR.CCUCELR0.R    /* Correctable Error Location 0 */
#define RTU0__LLC_CSR_CCUCELR1 RTU0__LLC_CSR.CCUCELR1.R    /* Correctable Error Location 1 */
#define RTU0__LLC_CSR_CCUUEDR RTU0__LLC_CSR.CCUUEDR.R      /* Uncorrectable Error Detect */
#define RTU0__LLC_CSR_CCUUEIR RTU0__LLC_CSR.CCUUEIR.R      /* Uncorrectable Error Interrupt */
#define RTU0__LLC_CSR_CCUUESR RTU0__LLC_CSR.CCUUESR.R      /* Uncorrectable Error Status */
#define RTU0__LLC_CSR_CCUUESAR RTU0__LLC_CSR.CCUUESAR.R    /* Uncorrectable Error Status Alias */
#define RTU0__LLC_CSR_CCUUELR0 RTU0__LLC_CSR.CCUUELR0.R    /* Uncorrectable Error Location 0 */
#define RTU0__LLC_CSR_CCUUELR1 RTU0__LLC_CSR.CCUUELR1.R    /* Uncorrectable Error Location 1 */
#define RTU0__LLC_CSR_CCUIDR RTU0__LLC_CSR.CCUIDR.R        /* Identification */
#define RTU0__LLC_CSR_CCUCRTR RTU0__LLC_CSR.CCUCRTR.R      /* Correctable Resiliency Threshold */

#define RTU1__LLC_CSR_CCUTCR RTU1__LLC_CSR.CCUTCR.R        /* Transaction Control */
#define RTU1__LLC_CSR_CCUTAR RTU1__LLC_CSR.CCUTAR.R        /* Transaction Activity */
#define RTU1__LLC_CSR_CCUCTCR RTU1__LLC_CSR.CCUCTCR.R      /* Cache Transaction Control */
#define RTU1__LLC_CSR_CCUCTAR RTU1__LLC_CSR.CCUCTAR.R      /* Cache Transaction Activity */
#define RTU1__LLC_CSR_CCUCAOR RTU1__LLC_CSR.CCUCAOR.R      /* Allocation Override */
#define RTU1__LLC_CSR_CCUSPCR0 RTU1__LLC_CSR.CCUSPCR0.R    /* Scratchpad Configuration 0 */
#define RTU1__LLC_CSR_CCUSPCR1 RTU1__LLC_CSR.CCUSPCR1.R    /* Scratchpad Configuration 1 */
#define RTU1__LLC_CSR_CCUSPBR0 RTU1__LLC_CSR.CCUSPBR0.R    /* Scratchpad Base Address 0 */
#define RTU1__LLC_CSR_CCUWPCRA0 RTU1__LLC_CSR.CCUWPCR[0].CCUWPCRA.R /* Way Partitioning Control A 0 */
#define RTU1__LLC_CSR_CCUWPCRB0 RTU1__LLC_CSR.CCUWPCR[0].CCUWPCRB.R /* Way Partitioning Control B 0 */
#define RTU1__LLC_CSR_CCUWPCRA1 RTU1__LLC_CSR.CCUWPCR[1].CCUWPCRA.R /* Way Partitioning Control A 1 */
#define RTU1__LLC_CSR_CCUWPCRB1 RTU1__LLC_CSR.CCUWPCR[1].CCUWPCRB.R /* Way Partitioning Control B 1 */
#define RTU1__LLC_CSR_CCUWPCRA2 RTU1__LLC_CSR.CCUWPCR[2].CCUWPCRA.R /* Way Partitioning Control A 2 */
#define RTU1__LLC_CSR_CCUWPCRB2 RTU1__LLC_CSR.CCUWPCR[2].CCUWPCRB.R /* Way Partitioning Control B 2 */
#define RTU1__LLC_CSR_CCUWPCRA3 RTU1__LLC_CSR.CCUWPCR[3].CCUWPCRA.R /* Way Partitioning Control A 3 */
#define RTU1__LLC_CSR_CCUWPCRB3 RTU1__LLC_CSR.CCUWPCR[3].CCUWPCRB.R /* Way Partitioning Control B 3 */
#define RTU1__LLC_CSR_CCUWPCRA4 RTU1__LLC_CSR.CCUWPCR[4].CCUWPCRA.R /* Way Partitioning Control A 4 */
#define RTU1__LLC_CSR_CCUWPCRB4 RTU1__LLC_CSR.CCUWPCR[4].CCUWPCRB.R /* Way Partitioning Control B 4 */
#define RTU1__LLC_CSR_CCUWPCRA5 RTU1__LLC_CSR.CCUWPCR[5].CCUWPCRA.R /* Way Partitioning Control A 5 */
#define RTU1__LLC_CSR_CCUWPCRB5 RTU1__LLC_CSR.CCUWPCR[5].CCUWPCRB.R /* Way Partitioning Control B 5 */
#define RTU1__LLC_CSR_CCUWPCRA6 RTU1__LLC_CSR.CCUWPCR[6].CCUWPCRA.R /* Way Partitioning Control A 6 */
#define RTU1__LLC_CSR_CCUWPCRB6 RTU1__LLC_CSR.CCUWPCR[6].CCUWPCRB.R /* Way Partitioning Control B 6 */
#define RTU1__LLC_CSR_CCUWPCRA7 RTU1__LLC_CSR.CCUWPCR[7].CCUWPCRA.R /* Way Partitioning Control A 7 */
#define RTU1__LLC_CSR_CCUWPCRB7 RTU1__LLC_CSR.CCUWPCR[7].CCUWPCRB.R /* Way Partitioning Control B 7 */
#define RTU1__LLC_CSR_CCUCMCR RTU1__LLC_CSR.CCUCMCR.R      /* Maintenance Control */
#define RTU1__LLC_CSR_CCUCMAR RTU1__LLC_CSR.CCUCMAR.R      /* Maintenance Activity */
#define RTU1__LLC_CSR_CCUCMLR0 RTU1__LLC_CSR.CCUCMLR0.R    /* Maintenance Location 0 */
#define RTU1__LLC_CSR_CCUCMLR1 RTU1__LLC_CSR.CCUCMLR1.R    /* Maintenance Location 1 */
#define RTU1__LLC_CSR_CCUCMLR2 RTU1__LLC_CSR.CCUCMLR2.R    /* Maintenance Location 2 */
#define RTU1__LLC_CSR_CCUCMDR RTU1__LLC_CSR.CCUCMDR.R      /* Maintenance Data */
#define RTU1__LLC_CSR_CCUCECR RTU1__LLC_CSR.CCUCECR.R      /* Correctable Error Control */
#define RTU1__LLC_CSR_CCUCESR RTU1__LLC_CSR.CCUCESR.R      /* Correctable Error Status */
#define RTU1__LLC_CSR_CCUCESAR RTU1__LLC_CSR.CCUCESAR.R    /* Correctable Error Status Alias */
#define RTU1__LLC_CSR_CCUCELR0 RTU1__LLC_CSR.CCUCELR0.R    /* Correctable Error Location 0 */
#define RTU1__LLC_CSR_CCUCELR1 RTU1__LLC_CSR.CCUCELR1.R    /* Correctable Error Location 1 */
#define RTU1__LLC_CSR_CCUUEDR RTU1__LLC_CSR.CCUUEDR.R      /* Uncorrectable Error Detect */
#define RTU1__LLC_CSR_CCUUEIR RTU1__LLC_CSR.CCUUEIR.R      /* Uncorrectable Error Interrupt */
#define RTU1__LLC_CSR_CCUUESR RTU1__LLC_CSR.CCUUESR.R      /* Uncorrectable Error Status */
#define RTU1__LLC_CSR_CCUUESAR RTU1__LLC_CSR.CCUUESAR.R    /* Uncorrectable Error Status Alias */
#define RTU1__LLC_CSR_CCUUELR0 RTU1__LLC_CSR.CCUUELR0.R    /* Uncorrectable Error Location 0 */
#define RTU1__LLC_CSR_CCUUELR1 RTU1__LLC_CSR.CCUUELR1.R    /* Uncorrectable Error Location 1 */
#define RTU1__LLC_CSR_CCUIDR RTU1__LLC_CSR.CCUIDR.R        /* Identification */
#define RTU1__LLC_CSR_CCUCRTR RTU1__LLC_CSR.CCUCRTR.R      /* Correctable Resiliency Threshold */

/* LLC_FSC */
#define RTU0__LLC_FSC_FSCBISTCR RTU0__LLC_FSC.FSCBISTCR.R  /* FSC BIST Control */
#define RTU0__LLC_FSC_FSCBISTAR RTU0__LLC_FSC.FSCBISTAR.R  /* FSC BIST Activity */
#define RTU0__LLC_FSC_FSCLF0 RTU0__LLC_FSC.FSCLF0.R        /* FSC Latent Fault Inputs 0 */
#define RTU0__LLC_FSC_FSCLF1 RTU0__LLC_FSC.FSCLF[0].R      /* FSC Latent Fault Inputs */
#define RTU0__LLC_FSC_FSCLF2 RTU0__LLC_FSC.FSCLF[1].R      /* FSC Latent Fault Inputs */
#define RTU0__LLC_FSC_FSCLF3 RTU0__LLC_FSC.FSCLF[2].R      /* FSC Latent Fault Inputs */
#define RTU0__LLC_FSC_FSCMF0 RTU0__LLC_FSC.FSCMF0.R        /* FSC Mission Fault 0 */
#define RTU0__LLC_FSC_FSCMF1 RTU0__LLC_FSC.FSCMF[0].R      /* FSC Mission Fault Inputs */
#define RTU0__LLC_FSC_FSCMF2 RTU0__LLC_FSC.FSCMF[1].R      /* FSC Mission Fault Inputs */
#define RTU0__LLC_FSC_FSCMF3 RTU0__LLC_FSC.FSCMF[2].R      /* FSC Mission Fault Inputs */
#define RTU0__LLC_FSC_FSCCETHF0 RTU0__LLC_FSC.FSCCETHF0.R  /* FSC Correctable Error Over Theshold 0 */
#define RTU0__LLC_FSC_FSCCETHF1 RTU0__LLC_FSC.FSCCETHF[0].R /* FSC Correctable Error Over Threshold 1 */
#define RTU0__LLC_FSC_FSCCETHF2 RTU0__LLC_FSC.FSCCETHF[1].R /* FSC Correctable Error Over Threshold 2 */
#define RTU0__LLC_FSC_FSCCETHF3 RTU0__LLC_FSC.FSCCETHF[2].R /* FSC Correctable Error Over Threshold 3 */

#define RTU1__LLC_FSC_FSCBISTCR RTU1__LLC_FSC.FSCBISTCR.R  /* FSC BIST Control */
#define RTU1__LLC_FSC_FSCBISTAR RTU1__LLC_FSC.FSCBISTAR.R  /* FSC BIST Activity */
#define RTU1__LLC_FSC_FSCLF0 RTU1__LLC_FSC.FSCLF0.R        /* FSC Latent Fault Inputs 0 */
#define RTU1__LLC_FSC_FSCLF1 RTU1__LLC_FSC.FSCLF[0].R      /* FSC Latent Fault Inputs */
#define RTU1__LLC_FSC_FSCLF2 RTU1__LLC_FSC.FSCLF[1].R      /* FSC Latent Fault Inputs */
#define RTU1__LLC_FSC_FSCLF3 RTU1__LLC_FSC.FSCLF[2].R      /* FSC Latent Fault Inputs */
#define RTU1__LLC_FSC_FSCMF0 RTU1__LLC_FSC.FSCMF0.R        /* FSC Mission Fault 0 */
#define RTU1__LLC_FSC_FSCMF1 RTU1__LLC_FSC.FSCMF[0].R      /* FSC Mission Fault Inputs */
#define RTU1__LLC_FSC_FSCMF2 RTU1__LLC_FSC.FSCMF[1].R      /* FSC Mission Fault Inputs */
#define RTU1__LLC_FSC_FSCMF3 RTU1__LLC_FSC.FSCMF[2].R      /* FSC Mission Fault Inputs */
#define RTU1__LLC_FSC_FSCCETHF0 RTU1__LLC_FSC.FSCCETHF0.R  /* FSC Correctable Error Over Theshold 0 */
#define RTU1__LLC_FSC_FSCCETHF1 RTU1__LLC_FSC.FSCCETHF[0].R /* FSC Correctable Error Over Threshold 1 */
#define RTU1__LLC_FSC_FSCCETHF2 RTU1__LLC_FSC.FSCCETHF[1].R /* FSC Correctable Error Over Threshold 2 */
#define RTU1__LLC_FSC_FSCCETHF3 RTU1__LLC_FSC.FSCCETHF[2].R /* FSC Correctable Error Over Threshold 3 */

/* LMEM64 */
#define CE_LMEM64_PCCCR      CE_LMEM64.PCCCR.R             /* PC bus Cache control register */
#define CE_LMEM64_PCCLCR     CE_LMEM64.PCCLCR.R            /* PC bus Cache line control register */
#define CE_LMEM64_PCCSAR     CE_LMEM64.PCCSAR.R            /* PC bus Cache search address register */
#define CE_LMEM64_PCCCVR     CE_LMEM64.PCCCVR.R            /* PC bus Cache read/write value register */
#define CE_LMEM64_PSCCR      CE_LMEM64.PSCCR.R             /* PS bus Cache control register */
#define CE_LMEM64_PSCLCR     CE_LMEM64.PSCLCR.R            /* PS bus Cache line control register */
#define CE_LMEM64_PSCSAR     CE_LMEM64.PSCSAR.R            /* PS bus Cache search address register */
#define CE_LMEM64_PSCCVR     CE_LMEM64.PSCCVR.R            /* PS bus Cache read/write value register */

#define SMU__LMEM64_PCCCR    SMU__LMEM64.PCCCR.R           /* PC bus Cache control register */
#define SMU__LMEM64_PCCLCR   SMU__LMEM64.PCCLCR.R          /* PC bus Cache line control register */
#define SMU__LMEM64_PCCSAR   SMU__LMEM64.PCCSAR.R          /* PC bus Cache search address register */
#define SMU__LMEM64_PCCCVR   SMU__LMEM64.PCCCVR.R          /* PC bus Cache read/write value register */
#define SMU__LMEM64_PSCCR    SMU__LMEM64.PSCCR.R           /* PS bus Cache control register */
#define SMU__LMEM64_PSCLCR   SMU__LMEM64.PSCLCR.R          /* PS bus Cache line control register */
#define SMU__LMEM64_PSCSAR   SMU__LMEM64.PSCSAR.R          /* PS bus Cache search address register */
#define SMU__LMEM64_PSCCVR   SMU__LMEM64.PSCCVR.R          /* PS bus Cache read/write value register */

/* LPI2C */
#define LPI2C_1_VERID        LPI2C_1.VERID.R               /* Version ID */
#define LPI2C_1_PARAM        LPI2C_1.PARAM.R               /* Parameter */
#define LPI2C_1_MCR          LPI2C_1.MCR.R                 /* Controller Control */
#define LPI2C_1_MSR          LPI2C_1.MSR.R                 /* Controller Status */
#define LPI2C_1_MIER         LPI2C_1.MIER.R                /* Controller Interrupt Enable */
#define LPI2C_1_MDER         LPI2C_1.MDER.R                /* Controller DMA Enable */
#define LPI2C_1_MCFGR0       LPI2C_1.MCFGR0.R              /* Controller Configuration 0 */
#define LPI2C_1_MCFGR1       LPI2C_1.MCFGR1.R              /* Controller Configuration 1 */
#define LPI2C_1_MCFGR2       LPI2C_1.MCFGR2.R              /* Controller Configuration 2 */
#define LPI2C_1_MCFGR3       LPI2C_1.MCFGR3.R              /* Controller Configuration 3 */
#define LPI2C_1_MDMR         LPI2C_1.MDMR.R                /* Controller Data Match */
#define LPI2C_1_MCCR0        LPI2C_1.MCCR0.R               /* Controller Clock Configuration 0 */
#define LPI2C_1_MCCR1        LPI2C_1.MCCR1.R               /* Controller Clock Configuration 1 */
#define LPI2C_1_MFCR         LPI2C_1.MFCR.R                /* Controller FIFO Control */
#define LPI2C_1_MFSR         LPI2C_1.MFSR.R                /* Controller FIFO Status */
#define LPI2C_1_MTDR         LPI2C_1.MTDR.R                /* Controller Transmit Data */
#define LPI2C_1_MRDR         LPI2C_1.MRDR.R                /* Controller Receive Data */
#define LPI2C_1_MRDROR       LPI2C_1.MRDROR.R              /* Controller Receive Data Read Only */
#define LPI2C_1_SCR          LPI2C_1.SCR.R                 /* Target Control */
#define LPI2C_1_SSR          LPI2C_1.SSR.R                 /* Target Status */
#define LPI2C_1_SIER         LPI2C_1.SIER.R                /* Target Interrupt Enable */
#define LPI2C_1_SDER         LPI2C_1.SDER.R                /* Target DMA Enable */
#define LPI2C_1_SCFGR0       LPI2C_1.SCFGR0.R              /* Target Configuration 0 */
#define LPI2C_1_SCFGR1       LPI2C_1.SCFGR1.R              /* Target Configuration 1 */
#define LPI2C_1_SCFGR2       LPI2C_1.SCFGR2.R              /* Target Configuration 2 */
#define LPI2C_1_SAMR         LPI2C_1.SAMR.R                /* Target Address Match */
#define LPI2C_1_SASR         LPI2C_1.SASR.R                /* Target Address Status */
#define LPI2C_1_STAR         LPI2C_1.STAR.R                /* Target Transmit ACK */
#define LPI2C_1_STDR         LPI2C_1.STDR.R                /* Target Transmit Data */
#define LPI2C_1_SRDR         LPI2C_1.SRDR.R                /* Target Receive Data */
#define LPI2C_1_SRDROR       LPI2C_1.SRDROR.R              /* Target Receive Data Read Only */
#define LPI2C_1_MTCBR0       LPI2C_1.MTCBR[0].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR1       LPI2C_1.MTCBR[1].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR2       LPI2C_1.MTCBR[2].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR3       LPI2C_1.MTCBR[3].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR4       LPI2C_1.MTCBR[4].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR5       LPI2C_1.MTCBR[5].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR6       LPI2C_1.MTCBR[6].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR7       LPI2C_1.MTCBR[7].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR8       LPI2C_1.MTCBR[8].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR9       LPI2C_1.MTCBR[9].R            /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR10      LPI2C_1.MTCBR[10].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR11      LPI2C_1.MTCBR[11].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR12      LPI2C_1.MTCBR[12].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR13      LPI2C_1.MTCBR[13].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR14      LPI2C_1.MTCBR[14].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR15      LPI2C_1.MTCBR[15].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR16      LPI2C_1.MTCBR[16].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR17      LPI2C_1.MTCBR[17].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR18      LPI2C_1.MTCBR[18].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR19      LPI2C_1.MTCBR[19].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR20      LPI2C_1.MTCBR[20].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR21      LPI2C_1.MTCBR[21].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR22      LPI2C_1.MTCBR[22].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR23      LPI2C_1.MTCBR[23].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR24      LPI2C_1.MTCBR[24].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR25      LPI2C_1.MTCBR[25].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR26      LPI2C_1.MTCBR[26].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR27      LPI2C_1.MTCBR[27].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR28      LPI2C_1.MTCBR[28].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR29      LPI2C_1.MTCBR[29].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR30      LPI2C_1.MTCBR[30].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR31      LPI2C_1.MTCBR[31].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR32      LPI2C_1.MTCBR[32].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR33      LPI2C_1.MTCBR[33].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR34      LPI2C_1.MTCBR[34].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR35      LPI2C_1.MTCBR[35].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR36      LPI2C_1.MTCBR[36].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR37      LPI2C_1.MTCBR[37].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR38      LPI2C_1.MTCBR[38].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR39      LPI2C_1.MTCBR[39].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR40      LPI2C_1.MTCBR[40].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR41      LPI2C_1.MTCBR[41].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR42      LPI2C_1.MTCBR[42].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR43      LPI2C_1.MTCBR[43].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR44      LPI2C_1.MTCBR[44].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR45      LPI2C_1.MTCBR[45].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR46      LPI2C_1.MTCBR[46].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR47      LPI2C_1.MTCBR[47].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR48      LPI2C_1.MTCBR[48].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR49      LPI2C_1.MTCBR[49].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR50      LPI2C_1.MTCBR[50].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR51      LPI2C_1.MTCBR[51].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR52      LPI2C_1.MTCBR[52].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR53      LPI2C_1.MTCBR[53].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR54      LPI2C_1.MTCBR[54].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR55      LPI2C_1.MTCBR[55].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR56      LPI2C_1.MTCBR[56].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR57      LPI2C_1.MTCBR[57].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR58      LPI2C_1.MTCBR[58].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR59      LPI2C_1.MTCBR[59].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR60      LPI2C_1.MTCBR[60].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR61      LPI2C_1.MTCBR[61].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR62      LPI2C_1.MTCBR[62].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR63      LPI2C_1.MTCBR[63].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR64      LPI2C_1.MTCBR[64].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR65      LPI2C_1.MTCBR[65].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR66      LPI2C_1.MTCBR[66].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR67      LPI2C_1.MTCBR[67].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR68      LPI2C_1.MTCBR[68].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR69      LPI2C_1.MTCBR[69].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR70      LPI2C_1.MTCBR[70].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR71      LPI2C_1.MTCBR[71].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR72      LPI2C_1.MTCBR[72].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR73      LPI2C_1.MTCBR[73].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR74      LPI2C_1.MTCBR[74].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR75      LPI2C_1.MTCBR[75].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR76      LPI2C_1.MTCBR[76].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR77      LPI2C_1.MTCBR[77].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR78      LPI2C_1.MTCBR[78].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR79      LPI2C_1.MTCBR[79].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR80      LPI2C_1.MTCBR[80].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR81      LPI2C_1.MTCBR[81].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR82      LPI2C_1.MTCBR[82].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR83      LPI2C_1.MTCBR[83].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR84      LPI2C_1.MTCBR[84].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR85      LPI2C_1.MTCBR[85].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR86      LPI2C_1.MTCBR[86].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR87      LPI2C_1.MTCBR[87].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR88      LPI2C_1.MTCBR[88].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR89      LPI2C_1.MTCBR[89].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR90      LPI2C_1.MTCBR[90].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR91      LPI2C_1.MTCBR[91].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR92      LPI2C_1.MTCBR[92].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR93      LPI2C_1.MTCBR[93].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR94      LPI2C_1.MTCBR[94].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR95      LPI2C_1.MTCBR[95].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR96      LPI2C_1.MTCBR[96].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR97      LPI2C_1.MTCBR[97].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR98      LPI2C_1.MTCBR[98].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR99      LPI2C_1.MTCBR[99].R           /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR100     LPI2C_1.MTCBR[100].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR101     LPI2C_1.MTCBR[101].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR102     LPI2C_1.MTCBR[102].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR103     LPI2C_1.MTCBR[103].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR104     LPI2C_1.MTCBR[104].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR105     LPI2C_1.MTCBR[105].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR106     LPI2C_1.MTCBR[106].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR107     LPI2C_1.MTCBR[107].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR108     LPI2C_1.MTCBR[108].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR109     LPI2C_1.MTCBR[109].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR110     LPI2C_1.MTCBR[110].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR111     LPI2C_1.MTCBR[111].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR112     LPI2C_1.MTCBR[112].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR113     LPI2C_1.MTCBR[113].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR114     LPI2C_1.MTCBR[114].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR115     LPI2C_1.MTCBR[115].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR116     LPI2C_1.MTCBR[116].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR117     LPI2C_1.MTCBR[117].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR118     LPI2C_1.MTCBR[118].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR119     LPI2C_1.MTCBR[119].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR120     LPI2C_1.MTCBR[120].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR121     LPI2C_1.MTCBR[121].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR122     LPI2C_1.MTCBR[122].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR123     LPI2C_1.MTCBR[123].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR124     LPI2C_1.MTCBR[124].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR125     LPI2C_1.MTCBR[125].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR126     LPI2C_1.MTCBR[126].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTCBR127     LPI2C_1.MTCBR[127].R          /* Controller Transmit Command Burst */
#define LPI2C_1_MTDBR0       LPI2C_1.MTDBR[0].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR1       LPI2C_1.MTDBR[1].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR2       LPI2C_1.MTDBR[2].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR3       LPI2C_1.MTDBR[3].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR4       LPI2C_1.MTDBR[4].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR5       LPI2C_1.MTDBR[5].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR6       LPI2C_1.MTDBR[6].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR7       LPI2C_1.MTDBR[7].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR8       LPI2C_1.MTDBR[8].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR9       LPI2C_1.MTDBR[9].R            /* Transmit Data Burst */
#define LPI2C_1_MTDBR10      LPI2C_1.MTDBR[10].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR11      LPI2C_1.MTDBR[11].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR12      LPI2C_1.MTDBR[12].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR13      LPI2C_1.MTDBR[13].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR14      LPI2C_1.MTDBR[14].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR15      LPI2C_1.MTDBR[15].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR16      LPI2C_1.MTDBR[16].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR17      LPI2C_1.MTDBR[17].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR18      LPI2C_1.MTDBR[18].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR19      LPI2C_1.MTDBR[19].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR20      LPI2C_1.MTDBR[20].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR21      LPI2C_1.MTDBR[21].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR22      LPI2C_1.MTDBR[22].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR23      LPI2C_1.MTDBR[23].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR24      LPI2C_1.MTDBR[24].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR25      LPI2C_1.MTDBR[25].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR26      LPI2C_1.MTDBR[26].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR27      LPI2C_1.MTDBR[27].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR28      LPI2C_1.MTDBR[28].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR29      LPI2C_1.MTDBR[29].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR30      LPI2C_1.MTDBR[30].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR31      LPI2C_1.MTDBR[31].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR32      LPI2C_1.MTDBR[32].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR33      LPI2C_1.MTDBR[33].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR34      LPI2C_1.MTDBR[34].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR35      LPI2C_1.MTDBR[35].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR36      LPI2C_1.MTDBR[36].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR37      LPI2C_1.MTDBR[37].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR38      LPI2C_1.MTDBR[38].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR39      LPI2C_1.MTDBR[39].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR40      LPI2C_1.MTDBR[40].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR41      LPI2C_1.MTDBR[41].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR42      LPI2C_1.MTDBR[42].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR43      LPI2C_1.MTDBR[43].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR44      LPI2C_1.MTDBR[44].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR45      LPI2C_1.MTDBR[45].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR46      LPI2C_1.MTDBR[46].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR47      LPI2C_1.MTDBR[47].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR48      LPI2C_1.MTDBR[48].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR49      LPI2C_1.MTDBR[49].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR50      LPI2C_1.MTDBR[50].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR51      LPI2C_1.MTDBR[51].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR52      LPI2C_1.MTDBR[52].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR53      LPI2C_1.MTDBR[53].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR54      LPI2C_1.MTDBR[54].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR55      LPI2C_1.MTDBR[55].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR56      LPI2C_1.MTDBR[56].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR57      LPI2C_1.MTDBR[57].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR58      LPI2C_1.MTDBR[58].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR59      LPI2C_1.MTDBR[59].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR60      LPI2C_1.MTDBR[60].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR61      LPI2C_1.MTDBR[61].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR62      LPI2C_1.MTDBR[62].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR63      LPI2C_1.MTDBR[63].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR64      LPI2C_1.MTDBR[64].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR65      LPI2C_1.MTDBR[65].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR66      LPI2C_1.MTDBR[66].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR67      LPI2C_1.MTDBR[67].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR68      LPI2C_1.MTDBR[68].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR69      LPI2C_1.MTDBR[69].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR70      LPI2C_1.MTDBR[70].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR71      LPI2C_1.MTDBR[71].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR72      LPI2C_1.MTDBR[72].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR73      LPI2C_1.MTDBR[73].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR74      LPI2C_1.MTDBR[74].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR75      LPI2C_1.MTDBR[75].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR76      LPI2C_1.MTDBR[76].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR77      LPI2C_1.MTDBR[77].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR78      LPI2C_1.MTDBR[78].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR79      LPI2C_1.MTDBR[79].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR80      LPI2C_1.MTDBR[80].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR81      LPI2C_1.MTDBR[81].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR82      LPI2C_1.MTDBR[82].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR83      LPI2C_1.MTDBR[83].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR84      LPI2C_1.MTDBR[84].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR85      LPI2C_1.MTDBR[85].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR86      LPI2C_1.MTDBR[86].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR87      LPI2C_1.MTDBR[87].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR88      LPI2C_1.MTDBR[88].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR89      LPI2C_1.MTDBR[89].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR90      LPI2C_1.MTDBR[90].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR91      LPI2C_1.MTDBR[91].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR92      LPI2C_1.MTDBR[92].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR93      LPI2C_1.MTDBR[93].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR94      LPI2C_1.MTDBR[94].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR95      LPI2C_1.MTDBR[95].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR96      LPI2C_1.MTDBR[96].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR97      LPI2C_1.MTDBR[97].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR98      LPI2C_1.MTDBR[98].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR99      LPI2C_1.MTDBR[99].R           /* Transmit Data Burst */
#define LPI2C_1_MTDBR100     LPI2C_1.MTDBR[100].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR101     LPI2C_1.MTDBR[101].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR102     LPI2C_1.MTDBR[102].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR103     LPI2C_1.MTDBR[103].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR104     LPI2C_1.MTDBR[104].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR105     LPI2C_1.MTDBR[105].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR106     LPI2C_1.MTDBR[106].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR107     LPI2C_1.MTDBR[107].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR108     LPI2C_1.MTDBR[108].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR109     LPI2C_1.MTDBR[109].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR110     LPI2C_1.MTDBR[110].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR111     LPI2C_1.MTDBR[111].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR112     LPI2C_1.MTDBR[112].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR113     LPI2C_1.MTDBR[113].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR114     LPI2C_1.MTDBR[114].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR115     LPI2C_1.MTDBR[115].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR116     LPI2C_1.MTDBR[116].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR117     LPI2C_1.MTDBR[117].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR118     LPI2C_1.MTDBR[118].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR119     LPI2C_1.MTDBR[119].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR120     LPI2C_1.MTDBR[120].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR121     LPI2C_1.MTDBR[121].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR122     LPI2C_1.MTDBR[122].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR123     LPI2C_1.MTDBR[123].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR124     LPI2C_1.MTDBR[124].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR125     LPI2C_1.MTDBR[125].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR126     LPI2C_1.MTDBR[126].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR127     LPI2C_1.MTDBR[127].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR128     LPI2C_1.MTDBR[128].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR129     LPI2C_1.MTDBR[129].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR130     LPI2C_1.MTDBR[130].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR131     LPI2C_1.MTDBR[131].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR132     LPI2C_1.MTDBR[132].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR133     LPI2C_1.MTDBR[133].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR134     LPI2C_1.MTDBR[134].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR135     LPI2C_1.MTDBR[135].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR136     LPI2C_1.MTDBR[136].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR137     LPI2C_1.MTDBR[137].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR138     LPI2C_1.MTDBR[138].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR139     LPI2C_1.MTDBR[139].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR140     LPI2C_1.MTDBR[140].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR141     LPI2C_1.MTDBR[141].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR142     LPI2C_1.MTDBR[142].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR143     LPI2C_1.MTDBR[143].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR144     LPI2C_1.MTDBR[144].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR145     LPI2C_1.MTDBR[145].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR146     LPI2C_1.MTDBR[146].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR147     LPI2C_1.MTDBR[147].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR148     LPI2C_1.MTDBR[148].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR149     LPI2C_1.MTDBR[149].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR150     LPI2C_1.MTDBR[150].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR151     LPI2C_1.MTDBR[151].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR152     LPI2C_1.MTDBR[152].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR153     LPI2C_1.MTDBR[153].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR154     LPI2C_1.MTDBR[154].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR155     LPI2C_1.MTDBR[155].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR156     LPI2C_1.MTDBR[156].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR157     LPI2C_1.MTDBR[157].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR158     LPI2C_1.MTDBR[158].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR159     LPI2C_1.MTDBR[159].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR160     LPI2C_1.MTDBR[160].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR161     LPI2C_1.MTDBR[161].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR162     LPI2C_1.MTDBR[162].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR163     LPI2C_1.MTDBR[163].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR164     LPI2C_1.MTDBR[164].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR165     LPI2C_1.MTDBR[165].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR166     LPI2C_1.MTDBR[166].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR167     LPI2C_1.MTDBR[167].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR168     LPI2C_1.MTDBR[168].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR169     LPI2C_1.MTDBR[169].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR170     LPI2C_1.MTDBR[170].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR171     LPI2C_1.MTDBR[171].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR172     LPI2C_1.MTDBR[172].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR173     LPI2C_1.MTDBR[173].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR174     LPI2C_1.MTDBR[174].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR175     LPI2C_1.MTDBR[175].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR176     LPI2C_1.MTDBR[176].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR177     LPI2C_1.MTDBR[177].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR178     LPI2C_1.MTDBR[178].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR179     LPI2C_1.MTDBR[179].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR180     LPI2C_1.MTDBR[180].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR181     LPI2C_1.MTDBR[181].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR182     LPI2C_1.MTDBR[182].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR183     LPI2C_1.MTDBR[183].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR184     LPI2C_1.MTDBR[184].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR185     LPI2C_1.MTDBR[185].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR186     LPI2C_1.MTDBR[186].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR187     LPI2C_1.MTDBR[187].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR188     LPI2C_1.MTDBR[188].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR189     LPI2C_1.MTDBR[189].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR190     LPI2C_1.MTDBR[190].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR191     LPI2C_1.MTDBR[191].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR192     LPI2C_1.MTDBR[192].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR193     LPI2C_1.MTDBR[193].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR194     LPI2C_1.MTDBR[194].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR195     LPI2C_1.MTDBR[195].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR196     LPI2C_1.MTDBR[196].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR197     LPI2C_1.MTDBR[197].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR198     LPI2C_1.MTDBR[198].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR199     LPI2C_1.MTDBR[199].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR200     LPI2C_1.MTDBR[200].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR201     LPI2C_1.MTDBR[201].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR202     LPI2C_1.MTDBR[202].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR203     LPI2C_1.MTDBR[203].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR204     LPI2C_1.MTDBR[204].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR205     LPI2C_1.MTDBR[205].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR206     LPI2C_1.MTDBR[206].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR207     LPI2C_1.MTDBR[207].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR208     LPI2C_1.MTDBR[208].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR209     LPI2C_1.MTDBR[209].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR210     LPI2C_1.MTDBR[210].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR211     LPI2C_1.MTDBR[211].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR212     LPI2C_1.MTDBR[212].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR213     LPI2C_1.MTDBR[213].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR214     LPI2C_1.MTDBR[214].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR215     LPI2C_1.MTDBR[215].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR216     LPI2C_1.MTDBR[216].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR217     LPI2C_1.MTDBR[217].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR218     LPI2C_1.MTDBR[218].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR219     LPI2C_1.MTDBR[219].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR220     LPI2C_1.MTDBR[220].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR221     LPI2C_1.MTDBR[221].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR222     LPI2C_1.MTDBR[222].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR223     LPI2C_1.MTDBR[223].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR224     LPI2C_1.MTDBR[224].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR225     LPI2C_1.MTDBR[225].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR226     LPI2C_1.MTDBR[226].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR227     LPI2C_1.MTDBR[227].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR228     LPI2C_1.MTDBR[228].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR229     LPI2C_1.MTDBR[229].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR230     LPI2C_1.MTDBR[230].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR231     LPI2C_1.MTDBR[231].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR232     LPI2C_1.MTDBR[232].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR233     LPI2C_1.MTDBR[233].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR234     LPI2C_1.MTDBR[234].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR235     LPI2C_1.MTDBR[235].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR236     LPI2C_1.MTDBR[236].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR237     LPI2C_1.MTDBR[237].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR238     LPI2C_1.MTDBR[238].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR239     LPI2C_1.MTDBR[239].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR240     LPI2C_1.MTDBR[240].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR241     LPI2C_1.MTDBR[241].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR242     LPI2C_1.MTDBR[242].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR243     LPI2C_1.MTDBR[243].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR244     LPI2C_1.MTDBR[244].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR245     LPI2C_1.MTDBR[245].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR246     LPI2C_1.MTDBR[246].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR247     LPI2C_1.MTDBR[247].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR248     LPI2C_1.MTDBR[248].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR249     LPI2C_1.MTDBR[249].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR250     LPI2C_1.MTDBR[250].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR251     LPI2C_1.MTDBR[251].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR252     LPI2C_1.MTDBR[252].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR253     LPI2C_1.MTDBR[253].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR254     LPI2C_1.MTDBR[254].R          /* Transmit Data Burst */
#define LPI2C_1_MTDBR255     LPI2C_1.MTDBR[255].R          /* Transmit Data Burst */

#define LPI2C_2_VERID        LPI2C_2.VERID.R               /* Version ID */
#define LPI2C_2_PARAM        LPI2C_2.PARAM.R               /* Parameter */
#define LPI2C_2_MCR          LPI2C_2.MCR.R                 /* Controller Control */
#define LPI2C_2_MSR          LPI2C_2.MSR.R                 /* Controller Status */
#define LPI2C_2_MIER         LPI2C_2.MIER.R                /* Controller Interrupt Enable */
#define LPI2C_2_MDER         LPI2C_2.MDER.R                /* Controller DMA Enable */
#define LPI2C_2_MCFGR0       LPI2C_2.MCFGR0.R              /* Controller Configuration 0 */
#define LPI2C_2_MCFGR1       LPI2C_2.MCFGR1.R              /* Controller Configuration 1 */
#define LPI2C_2_MCFGR2       LPI2C_2.MCFGR2.R              /* Controller Configuration 2 */
#define LPI2C_2_MCFGR3       LPI2C_2.MCFGR3.R              /* Controller Configuration 3 */
#define LPI2C_2_MDMR         LPI2C_2.MDMR.R                /* Controller Data Match */
#define LPI2C_2_MCCR0        LPI2C_2.MCCR0.R               /* Controller Clock Configuration 0 */
#define LPI2C_2_MCCR1        LPI2C_2.MCCR1.R               /* Controller Clock Configuration 1 */
#define LPI2C_2_MFCR         LPI2C_2.MFCR.R                /* Controller FIFO Control */
#define LPI2C_2_MFSR         LPI2C_2.MFSR.R                /* Controller FIFO Status */
#define LPI2C_2_MTDR         LPI2C_2.MTDR.R                /* Controller Transmit Data */
#define LPI2C_2_MRDR         LPI2C_2.MRDR.R                /* Controller Receive Data */
#define LPI2C_2_MRDROR       LPI2C_2.MRDROR.R              /* Controller Receive Data Read Only */
#define LPI2C_2_SCR          LPI2C_2.SCR.R                 /* Target Control */
#define LPI2C_2_SSR          LPI2C_2.SSR.R                 /* Target Status */
#define LPI2C_2_SIER         LPI2C_2.SIER.R                /* Target Interrupt Enable */
#define LPI2C_2_SDER         LPI2C_2.SDER.R                /* Target DMA Enable */
#define LPI2C_2_SCFGR0       LPI2C_2.SCFGR0.R              /* Target Configuration 0 */
#define LPI2C_2_SCFGR1       LPI2C_2.SCFGR1.R              /* Target Configuration 1 */
#define LPI2C_2_SCFGR2       LPI2C_2.SCFGR2.R              /* Target Configuration 2 */
#define LPI2C_2_SAMR         LPI2C_2.SAMR.R                /* Target Address Match */
#define LPI2C_2_SASR         LPI2C_2.SASR.R                /* Target Address Status */
#define LPI2C_2_STAR         LPI2C_2.STAR.R                /* Target Transmit ACK */
#define LPI2C_2_STDR         LPI2C_2.STDR.R                /* Target Transmit Data */
#define LPI2C_2_SRDR         LPI2C_2.SRDR.R                /* Target Receive Data */
#define LPI2C_2_SRDROR       LPI2C_2.SRDROR.R              /* Target Receive Data Read Only */
#define LPI2C_2_MTCBR0       LPI2C_2.MTCBR[0].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR1       LPI2C_2.MTCBR[1].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR2       LPI2C_2.MTCBR[2].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR3       LPI2C_2.MTCBR[3].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR4       LPI2C_2.MTCBR[4].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR5       LPI2C_2.MTCBR[5].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR6       LPI2C_2.MTCBR[6].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR7       LPI2C_2.MTCBR[7].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR8       LPI2C_2.MTCBR[8].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR9       LPI2C_2.MTCBR[9].R            /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR10      LPI2C_2.MTCBR[10].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR11      LPI2C_2.MTCBR[11].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR12      LPI2C_2.MTCBR[12].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR13      LPI2C_2.MTCBR[13].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR14      LPI2C_2.MTCBR[14].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR15      LPI2C_2.MTCBR[15].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR16      LPI2C_2.MTCBR[16].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR17      LPI2C_2.MTCBR[17].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR18      LPI2C_2.MTCBR[18].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR19      LPI2C_2.MTCBR[19].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR20      LPI2C_2.MTCBR[20].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR21      LPI2C_2.MTCBR[21].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR22      LPI2C_2.MTCBR[22].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR23      LPI2C_2.MTCBR[23].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR24      LPI2C_2.MTCBR[24].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR25      LPI2C_2.MTCBR[25].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR26      LPI2C_2.MTCBR[26].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR27      LPI2C_2.MTCBR[27].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR28      LPI2C_2.MTCBR[28].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR29      LPI2C_2.MTCBR[29].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR30      LPI2C_2.MTCBR[30].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR31      LPI2C_2.MTCBR[31].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR32      LPI2C_2.MTCBR[32].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR33      LPI2C_2.MTCBR[33].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR34      LPI2C_2.MTCBR[34].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR35      LPI2C_2.MTCBR[35].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR36      LPI2C_2.MTCBR[36].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR37      LPI2C_2.MTCBR[37].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR38      LPI2C_2.MTCBR[38].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR39      LPI2C_2.MTCBR[39].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR40      LPI2C_2.MTCBR[40].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR41      LPI2C_2.MTCBR[41].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR42      LPI2C_2.MTCBR[42].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR43      LPI2C_2.MTCBR[43].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR44      LPI2C_2.MTCBR[44].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR45      LPI2C_2.MTCBR[45].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR46      LPI2C_2.MTCBR[46].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR47      LPI2C_2.MTCBR[47].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR48      LPI2C_2.MTCBR[48].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR49      LPI2C_2.MTCBR[49].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR50      LPI2C_2.MTCBR[50].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR51      LPI2C_2.MTCBR[51].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR52      LPI2C_2.MTCBR[52].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR53      LPI2C_2.MTCBR[53].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR54      LPI2C_2.MTCBR[54].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR55      LPI2C_2.MTCBR[55].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR56      LPI2C_2.MTCBR[56].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR57      LPI2C_2.MTCBR[57].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR58      LPI2C_2.MTCBR[58].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR59      LPI2C_2.MTCBR[59].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR60      LPI2C_2.MTCBR[60].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR61      LPI2C_2.MTCBR[61].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR62      LPI2C_2.MTCBR[62].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR63      LPI2C_2.MTCBR[63].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR64      LPI2C_2.MTCBR[64].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR65      LPI2C_2.MTCBR[65].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR66      LPI2C_2.MTCBR[66].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR67      LPI2C_2.MTCBR[67].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR68      LPI2C_2.MTCBR[68].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR69      LPI2C_2.MTCBR[69].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR70      LPI2C_2.MTCBR[70].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR71      LPI2C_2.MTCBR[71].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR72      LPI2C_2.MTCBR[72].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR73      LPI2C_2.MTCBR[73].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR74      LPI2C_2.MTCBR[74].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR75      LPI2C_2.MTCBR[75].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR76      LPI2C_2.MTCBR[76].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR77      LPI2C_2.MTCBR[77].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR78      LPI2C_2.MTCBR[78].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR79      LPI2C_2.MTCBR[79].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR80      LPI2C_2.MTCBR[80].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR81      LPI2C_2.MTCBR[81].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR82      LPI2C_2.MTCBR[82].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR83      LPI2C_2.MTCBR[83].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR84      LPI2C_2.MTCBR[84].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR85      LPI2C_2.MTCBR[85].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR86      LPI2C_2.MTCBR[86].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR87      LPI2C_2.MTCBR[87].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR88      LPI2C_2.MTCBR[88].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR89      LPI2C_2.MTCBR[89].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR90      LPI2C_2.MTCBR[90].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR91      LPI2C_2.MTCBR[91].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR92      LPI2C_2.MTCBR[92].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR93      LPI2C_2.MTCBR[93].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR94      LPI2C_2.MTCBR[94].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR95      LPI2C_2.MTCBR[95].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR96      LPI2C_2.MTCBR[96].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR97      LPI2C_2.MTCBR[97].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR98      LPI2C_2.MTCBR[98].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR99      LPI2C_2.MTCBR[99].R           /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR100     LPI2C_2.MTCBR[100].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR101     LPI2C_2.MTCBR[101].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR102     LPI2C_2.MTCBR[102].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR103     LPI2C_2.MTCBR[103].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR104     LPI2C_2.MTCBR[104].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR105     LPI2C_2.MTCBR[105].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR106     LPI2C_2.MTCBR[106].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR107     LPI2C_2.MTCBR[107].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR108     LPI2C_2.MTCBR[108].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR109     LPI2C_2.MTCBR[109].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR110     LPI2C_2.MTCBR[110].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR111     LPI2C_2.MTCBR[111].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR112     LPI2C_2.MTCBR[112].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR113     LPI2C_2.MTCBR[113].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR114     LPI2C_2.MTCBR[114].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR115     LPI2C_2.MTCBR[115].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR116     LPI2C_2.MTCBR[116].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR117     LPI2C_2.MTCBR[117].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR118     LPI2C_2.MTCBR[118].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR119     LPI2C_2.MTCBR[119].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR120     LPI2C_2.MTCBR[120].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR121     LPI2C_2.MTCBR[121].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR122     LPI2C_2.MTCBR[122].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR123     LPI2C_2.MTCBR[123].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR124     LPI2C_2.MTCBR[124].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR125     LPI2C_2.MTCBR[125].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR126     LPI2C_2.MTCBR[126].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTCBR127     LPI2C_2.MTCBR[127].R          /* Controller Transmit Command Burst */
#define LPI2C_2_MTDBR0       LPI2C_2.MTDBR[0].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR1       LPI2C_2.MTDBR[1].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR2       LPI2C_2.MTDBR[2].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR3       LPI2C_2.MTDBR[3].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR4       LPI2C_2.MTDBR[4].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR5       LPI2C_2.MTDBR[5].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR6       LPI2C_2.MTDBR[6].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR7       LPI2C_2.MTDBR[7].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR8       LPI2C_2.MTDBR[8].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR9       LPI2C_2.MTDBR[9].R            /* Transmit Data Burst */
#define LPI2C_2_MTDBR10      LPI2C_2.MTDBR[10].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR11      LPI2C_2.MTDBR[11].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR12      LPI2C_2.MTDBR[12].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR13      LPI2C_2.MTDBR[13].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR14      LPI2C_2.MTDBR[14].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR15      LPI2C_2.MTDBR[15].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR16      LPI2C_2.MTDBR[16].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR17      LPI2C_2.MTDBR[17].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR18      LPI2C_2.MTDBR[18].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR19      LPI2C_2.MTDBR[19].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR20      LPI2C_2.MTDBR[20].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR21      LPI2C_2.MTDBR[21].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR22      LPI2C_2.MTDBR[22].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR23      LPI2C_2.MTDBR[23].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR24      LPI2C_2.MTDBR[24].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR25      LPI2C_2.MTDBR[25].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR26      LPI2C_2.MTDBR[26].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR27      LPI2C_2.MTDBR[27].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR28      LPI2C_2.MTDBR[28].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR29      LPI2C_2.MTDBR[29].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR30      LPI2C_2.MTDBR[30].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR31      LPI2C_2.MTDBR[31].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR32      LPI2C_2.MTDBR[32].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR33      LPI2C_2.MTDBR[33].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR34      LPI2C_2.MTDBR[34].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR35      LPI2C_2.MTDBR[35].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR36      LPI2C_2.MTDBR[36].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR37      LPI2C_2.MTDBR[37].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR38      LPI2C_2.MTDBR[38].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR39      LPI2C_2.MTDBR[39].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR40      LPI2C_2.MTDBR[40].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR41      LPI2C_2.MTDBR[41].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR42      LPI2C_2.MTDBR[42].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR43      LPI2C_2.MTDBR[43].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR44      LPI2C_2.MTDBR[44].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR45      LPI2C_2.MTDBR[45].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR46      LPI2C_2.MTDBR[46].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR47      LPI2C_2.MTDBR[47].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR48      LPI2C_2.MTDBR[48].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR49      LPI2C_2.MTDBR[49].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR50      LPI2C_2.MTDBR[50].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR51      LPI2C_2.MTDBR[51].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR52      LPI2C_2.MTDBR[52].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR53      LPI2C_2.MTDBR[53].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR54      LPI2C_2.MTDBR[54].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR55      LPI2C_2.MTDBR[55].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR56      LPI2C_2.MTDBR[56].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR57      LPI2C_2.MTDBR[57].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR58      LPI2C_2.MTDBR[58].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR59      LPI2C_2.MTDBR[59].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR60      LPI2C_2.MTDBR[60].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR61      LPI2C_2.MTDBR[61].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR62      LPI2C_2.MTDBR[62].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR63      LPI2C_2.MTDBR[63].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR64      LPI2C_2.MTDBR[64].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR65      LPI2C_2.MTDBR[65].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR66      LPI2C_2.MTDBR[66].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR67      LPI2C_2.MTDBR[67].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR68      LPI2C_2.MTDBR[68].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR69      LPI2C_2.MTDBR[69].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR70      LPI2C_2.MTDBR[70].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR71      LPI2C_2.MTDBR[71].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR72      LPI2C_2.MTDBR[72].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR73      LPI2C_2.MTDBR[73].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR74      LPI2C_2.MTDBR[74].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR75      LPI2C_2.MTDBR[75].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR76      LPI2C_2.MTDBR[76].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR77      LPI2C_2.MTDBR[77].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR78      LPI2C_2.MTDBR[78].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR79      LPI2C_2.MTDBR[79].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR80      LPI2C_2.MTDBR[80].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR81      LPI2C_2.MTDBR[81].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR82      LPI2C_2.MTDBR[82].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR83      LPI2C_2.MTDBR[83].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR84      LPI2C_2.MTDBR[84].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR85      LPI2C_2.MTDBR[85].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR86      LPI2C_2.MTDBR[86].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR87      LPI2C_2.MTDBR[87].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR88      LPI2C_2.MTDBR[88].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR89      LPI2C_2.MTDBR[89].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR90      LPI2C_2.MTDBR[90].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR91      LPI2C_2.MTDBR[91].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR92      LPI2C_2.MTDBR[92].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR93      LPI2C_2.MTDBR[93].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR94      LPI2C_2.MTDBR[94].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR95      LPI2C_2.MTDBR[95].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR96      LPI2C_2.MTDBR[96].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR97      LPI2C_2.MTDBR[97].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR98      LPI2C_2.MTDBR[98].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR99      LPI2C_2.MTDBR[99].R           /* Transmit Data Burst */
#define LPI2C_2_MTDBR100     LPI2C_2.MTDBR[100].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR101     LPI2C_2.MTDBR[101].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR102     LPI2C_2.MTDBR[102].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR103     LPI2C_2.MTDBR[103].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR104     LPI2C_2.MTDBR[104].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR105     LPI2C_2.MTDBR[105].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR106     LPI2C_2.MTDBR[106].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR107     LPI2C_2.MTDBR[107].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR108     LPI2C_2.MTDBR[108].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR109     LPI2C_2.MTDBR[109].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR110     LPI2C_2.MTDBR[110].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR111     LPI2C_2.MTDBR[111].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR112     LPI2C_2.MTDBR[112].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR113     LPI2C_2.MTDBR[113].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR114     LPI2C_2.MTDBR[114].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR115     LPI2C_2.MTDBR[115].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR116     LPI2C_2.MTDBR[116].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR117     LPI2C_2.MTDBR[117].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR118     LPI2C_2.MTDBR[118].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR119     LPI2C_2.MTDBR[119].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR120     LPI2C_2.MTDBR[120].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR121     LPI2C_2.MTDBR[121].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR122     LPI2C_2.MTDBR[122].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR123     LPI2C_2.MTDBR[123].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR124     LPI2C_2.MTDBR[124].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR125     LPI2C_2.MTDBR[125].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR126     LPI2C_2.MTDBR[126].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR127     LPI2C_2.MTDBR[127].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR128     LPI2C_2.MTDBR[128].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR129     LPI2C_2.MTDBR[129].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR130     LPI2C_2.MTDBR[130].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR131     LPI2C_2.MTDBR[131].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR132     LPI2C_2.MTDBR[132].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR133     LPI2C_2.MTDBR[133].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR134     LPI2C_2.MTDBR[134].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR135     LPI2C_2.MTDBR[135].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR136     LPI2C_2.MTDBR[136].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR137     LPI2C_2.MTDBR[137].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR138     LPI2C_2.MTDBR[138].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR139     LPI2C_2.MTDBR[139].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR140     LPI2C_2.MTDBR[140].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR141     LPI2C_2.MTDBR[141].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR142     LPI2C_2.MTDBR[142].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR143     LPI2C_2.MTDBR[143].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR144     LPI2C_2.MTDBR[144].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR145     LPI2C_2.MTDBR[145].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR146     LPI2C_2.MTDBR[146].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR147     LPI2C_2.MTDBR[147].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR148     LPI2C_2.MTDBR[148].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR149     LPI2C_2.MTDBR[149].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR150     LPI2C_2.MTDBR[150].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR151     LPI2C_2.MTDBR[151].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR152     LPI2C_2.MTDBR[152].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR153     LPI2C_2.MTDBR[153].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR154     LPI2C_2.MTDBR[154].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR155     LPI2C_2.MTDBR[155].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR156     LPI2C_2.MTDBR[156].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR157     LPI2C_2.MTDBR[157].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR158     LPI2C_2.MTDBR[158].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR159     LPI2C_2.MTDBR[159].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR160     LPI2C_2.MTDBR[160].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR161     LPI2C_2.MTDBR[161].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR162     LPI2C_2.MTDBR[162].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR163     LPI2C_2.MTDBR[163].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR164     LPI2C_2.MTDBR[164].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR165     LPI2C_2.MTDBR[165].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR166     LPI2C_2.MTDBR[166].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR167     LPI2C_2.MTDBR[167].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR168     LPI2C_2.MTDBR[168].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR169     LPI2C_2.MTDBR[169].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR170     LPI2C_2.MTDBR[170].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR171     LPI2C_2.MTDBR[171].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR172     LPI2C_2.MTDBR[172].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR173     LPI2C_2.MTDBR[173].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR174     LPI2C_2.MTDBR[174].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR175     LPI2C_2.MTDBR[175].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR176     LPI2C_2.MTDBR[176].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR177     LPI2C_2.MTDBR[177].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR178     LPI2C_2.MTDBR[178].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR179     LPI2C_2.MTDBR[179].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR180     LPI2C_2.MTDBR[180].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR181     LPI2C_2.MTDBR[181].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR182     LPI2C_2.MTDBR[182].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR183     LPI2C_2.MTDBR[183].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR184     LPI2C_2.MTDBR[184].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR185     LPI2C_2.MTDBR[185].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR186     LPI2C_2.MTDBR[186].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR187     LPI2C_2.MTDBR[187].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR188     LPI2C_2.MTDBR[188].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR189     LPI2C_2.MTDBR[189].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR190     LPI2C_2.MTDBR[190].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR191     LPI2C_2.MTDBR[191].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR192     LPI2C_2.MTDBR[192].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR193     LPI2C_2.MTDBR[193].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR194     LPI2C_2.MTDBR[194].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR195     LPI2C_2.MTDBR[195].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR196     LPI2C_2.MTDBR[196].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR197     LPI2C_2.MTDBR[197].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR198     LPI2C_2.MTDBR[198].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR199     LPI2C_2.MTDBR[199].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR200     LPI2C_2.MTDBR[200].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR201     LPI2C_2.MTDBR[201].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR202     LPI2C_2.MTDBR[202].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR203     LPI2C_2.MTDBR[203].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR204     LPI2C_2.MTDBR[204].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR205     LPI2C_2.MTDBR[205].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR206     LPI2C_2.MTDBR[206].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR207     LPI2C_2.MTDBR[207].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR208     LPI2C_2.MTDBR[208].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR209     LPI2C_2.MTDBR[209].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR210     LPI2C_2.MTDBR[210].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR211     LPI2C_2.MTDBR[211].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR212     LPI2C_2.MTDBR[212].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR213     LPI2C_2.MTDBR[213].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR214     LPI2C_2.MTDBR[214].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR215     LPI2C_2.MTDBR[215].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR216     LPI2C_2.MTDBR[216].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR217     LPI2C_2.MTDBR[217].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR218     LPI2C_2.MTDBR[218].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR219     LPI2C_2.MTDBR[219].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR220     LPI2C_2.MTDBR[220].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR221     LPI2C_2.MTDBR[221].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR222     LPI2C_2.MTDBR[222].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR223     LPI2C_2.MTDBR[223].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR224     LPI2C_2.MTDBR[224].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR225     LPI2C_2.MTDBR[225].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR226     LPI2C_2.MTDBR[226].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR227     LPI2C_2.MTDBR[227].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR228     LPI2C_2.MTDBR[228].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR229     LPI2C_2.MTDBR[229].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR230     LPI2C_2.MTDBR[230].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR231     LPI2C_2.MTDBR[231].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR232     LPI2C_2.MTDBR[232].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR233     LPI2C_2.MTDBR[233].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR234     LPI2C_2.MTDBR[234].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR235     LPI2C_2.MTDBR[235].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR236     LPI2C_2.MTDBR[236].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR237     LPI2C_2.MTDBR[237].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR238     LPI2C_2.MTDBR[238].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR239     LPI2C_2.MTDBR[239].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR240     LPI2C_2.MTDBR[240].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR241     LPI2C_2.MTDBR[241].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR242     LPI2C_2.MTDBR[242].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR243     LPI2C_2.MTDBR[243].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR244     LPI2C_2.MTDBR[244].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR245     LPI2C_2.MTDBR[245].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR246     LPI2C_2.MTDBR[246].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR247     LPI2C_2.MTDBR[247].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR248     LPI2C_2.MTDBR[248].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR249     LPI2C_2.MTDBR[249].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR250     LPI2C_2.MTDBR[250].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR251     LPI2C_2.MTDBR[251].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR252     LPI2C_2.MTDBR[252].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR253     LPI2C_2.MTDBR[253].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR254     LPI2C_2.MTDBR[254].R          /* Transmit Data Burst */
#define LPI2C_2_MTDBR255     LPI2C_2.MTDBR[255].R          /* Transmit Data Burst */

/* LSTCU */
#define LSTCU_6_ERR_STAT     LSTCU_6.ERR_STAT.R            /* Error Status */
#define LSTCU_6_ERR_FM       LSTCU_6.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_6_MB_RSTAT0    LSTCU_6.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_6_LB_RSTAT0    LSTCU_6.LB_RSTAT0.R           /* LBIST Run Status 0 */
#define LSTCU_6_MBFM0        LSTCU_6.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_6_LBFM0        LSTCU_6.LBFM0.R               /* LBIST Fault Mapping 0 */
#define LSTCU_6_STAG         LSTCU_6.STAG.R                /* Stagger */
#define LSTCU_6_PH1_DUR      LSTCU_6.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_6_ISODUR       LSTCU_6.ISODUR.R              /* Isolation Duration */
#define LSTCU_6_SRCENDUR     LSTCU_6.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_6_MBPTR0       LSTCU_6.MBPTR[0].R            /* MBIST Scheduler Pointer */
#define LSTCU_6_LBPTR0       LSTCU_6.LBPTR[0].R            /* LBIST Scheduler Pointer */

#define LSTCU_1_ERR_STAT     LSTCU_1.ERR_STAT.R            /* Error Status */
#define LSTCU_1_ERR_FM       LSTCU_1.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_1_MB_RSTAT0    LSTCU_1.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_1_LB_RSTAT0    LSTCU_1.LB_RSTAT0.R           /* LBIST Run Status 0 */
#define LSTCU_1_MBFM0        LSTCU_1.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_1_LBFM0        LSTCU_1.LBFM0.R               /* LBIST Fault Mapping 0 */
#define LSTCU_1_STAG         LSTCU_1.STAG.R                /* Stagger */
#define LSTCU_1_PH1_DUR      LSTCU_1.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_1_ISODUR       LSTCU_1.ISODUR.R              /* Isolation Duration */
#define LSTCU_1_SRCENDUR     LSTCU_1.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_1_MBPTR0       LSTCU_1.MBPTR[0].R            /* MBIST Scheduler Pointer */
#define LSTCU_1_LBPTR0       LSTCU_1.LBPTR[0].R            /* LBIST Scheduler Pointer */

#define LSTCU_11_ERR_STAT    LSTCU_11.ERR_STAT.R           /* Error Status */
#define LSTCU_11_ERR_FM      LSTCU_11.ERR_FM.R             /* Error Fault Mapping */
#define LSTCU_11_MB_RSTAT0   LSTCU_11.MB_RSTAT0.R          /* MBIST Run Status 0 */
#define LSTCU_11_LB_RSTAT0   LSTCU_11.LB_RSTAT0.R          /* LBIST Run Status 0 */
#define LSTCU_11_MBFM0       LSTCU_11.MBFM0.R              /* MBIST Fault Mapping 0 */
#define LSTCU_11_LBFM0       LSTCU_11.LBFM0.R              /* LBIST Fault Mapping 0 */
#define LSTCU_11_STAG        LSTCU_11.STAG.R               /* Stagger */
#define LSTCU_11_PH1_DUR     LSTCU_11.PH1_DUR.R            /* Phase 1 Duration */
#define LSTCU_11_ISODUR      LSTCU_11.ISODUR.R             /* Isolation Duration */
#define LSTCU_11_SRCENDUR    LSTCU_11.SRCENDUR.R           /* Synchronous Reset Clock Enable Duration */
#define LSTCU_11_MBPTR0      LSTCU_11.MBPTR[0].R           /* MBIST Scheduler Pointer */
#define LSTCU_11_LBPTR0      LSTCU_11.LBPTR[0].R           /* LBIST Scheduler Pointer */

#define LSTCU_10_ERR_STAT    LSTCU_10.ERR_STAT.R           /* Error Status */
#define LSTCU_10_ERR_FM      LSTCU_10.ERR_FM.R             /* Error Fault Mapping */
#define LSTCU_10_MB_RSTAT0   LSTCU_10.MB_RSTAT0.R          /* MBIST Run Status 0 */
#define LSTCU_10_LB_RSTAT0   LSTCU_10.LB_RSTAT0.R          /* LBIST Run Status 0 */
#define LSTCU_10_MBFM0       LSTCU_10.MBFM0.R              /* MBIST Fault Mapping 0 */
#define LSTCU_10_LBFM0       LSTCU_10.LBFM0.R              /* LBIST Fault Mapping 0 */
#define LSTCU_10_STAG        LSTCU_10.STAG.R               /* Stagger */
#define LSTCU_10_PH1_DUR     LSTCU_10.PH1_DUR.R            /* Phase 1 Duration */
#define LSTCU_10_ISODUR      LSTCU_10.ISODUR.R             /* Isolation Duration */
#define LSTCU_10_SRCENDUR    LSTCU_10.SRCENDUR.R           /* Synchronous Reset Clock Enable Duration */
#define LSTCU_10_MBPTR0      LSTCU_10.MBPTR[0].R           /* MBIST Scheduler Pointer */
#define LSTCU_10_LBPTR0      LSTCU_10.LBPTR[0].R           /* LBIST Scheduler Pointer */

#define LSTCU_2_ERR_STAT     LSTCU_2.ERR_STAT.R            /* Error Status */
#define LSTCU_2_ERR_FM       LSTCU_2.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_2_MB_RSTAT0    LSTCU_2.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_2_LB_RSTAT0    LSTCU_2.LB_RSTAT0.R           /* LBIST Run Status 0 */
#define LSTCU_2_MBFM0        LSTCU_2.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_2_LBFM0        LSTCU_2.LBFM0.R               /* LBIST Fault Mapping 0 */
#define LSTCU_2_STAG         LSTCU_2.STAG.R                /* Stagger */
#define LSTCU_2_PH1_DUR      LSTCU_2.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_2_ISODUR       LSTCU_2.ISODUR.R              /* Isolation Duration */
#define LSTCU_2_SRCENDUR     LSTCU_2.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_2_MBPTR0       LSTCU_2.MBPTR[0].R            /* MBIST Scheduler Pointer */
#define LSTCU_2_LBPTR0       LSTCU_2.LBPTR[0].R            /* LBIST Scheduler Pointer */

#define LSTCU_9_ERR_STAT     LSTCU_9.ERR_STAT.R            /* Error Status */
#define LSTCU_9_ERR_FM       LSTCU_9.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_9_MB_RSTAT0    LSTCU_9.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_9_LB_RSTAT0    LSTCU_9.LB_RSTAT0.R           /* LBIST Run Status 0 */
#define LSTCU_9_MBFM0        LSTCU_9.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_9_LBFM0        LSTCU_9.LBFM0.R               /* LBIST Fault Mapping 0 */
#define LSTCU_9_STAG         LSTCU_9.STAG.R                /* Stagger */
#define LSTCU_9_PH1_DUR      LSTCU_9.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_9_ISODUR       LSTCU_9.ISODUR.R              /* Isolation Duration */
#define LSTCU_9_SRCENDUR     LSTCU_9.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_9_MBPTR0       LSTCU_9.MBPTR[0].R            /* MBIST Scheduler Pointer */
#define LSTCU_9_LBPTR0       LSTCU_9.LBPTR[0].R            /* LBIST Scheduler Pointer */

#define LSTCU_0_ERR_STAT     LSTCU_0.ERR_STAT.R            /* Error Status */
#define LSTCU_0_ERR_FM       LSTCU_0.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_0_MB_RSTAT0    LSTCU_0.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_0_LB_RSTAT0    LSTCU_0.LB_RSTAT0.R           /* LBIST Run Status 0 */
#define LSTCU_0_MBFM0        LSTCU_0.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_0_LBFM0        LSTCU_0.LBFM0.R               /* LBIST Fault Mapping 0 */
#define LSTCU_0_STAG         LSTCU_0.STAG.R                /* Stagger */
#define LSTCU_0_PH1_DUR      LSTCU_0.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_0_ISODUR       LSTCU_0.ISODUR.R              /* Isolation Duration */
#define LSTCU_0_SRCENDUR     LSTCU_0.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_0_MBPTR0       LSTCU_0.MBPTR[0].R            /* MBIST Scheduler Pointer */
#define LSTCU_0_LBPTR0       LSTCU_0.LBPTR[0].R            /* LBIST Scheduler Pointer */

#define LSTCU_12_ERR_STAT    LSTCU_12.ERR_STAT.R           /* Error Status */
#define LSTCU_12_ERR_FM      LSTCU_12.ERR_FM.R             /* Error Fault Mapping */
#define LSTCU_12_MB_RSTAT0   LSTCU_12.MB_RSTAT0.R          /* MBIST Run Status 0 */
#define LSTCU_12_LB_RSTAT0   LSTCU_12.LB_RSTAT0.R          /* LBIST Run Status 0 */
#define LSTCU_12_MBFM0       LSTCU_12.MBFM0.R              /* MBIST Fault Mapping 0 */
#define LSTCU_12_LBFM0       LSTCU_12.LBFM0.R              /* LBIST Fault Mapping 0 */
#define LSTCU_12_STAG        LSTCU_12.STAG.R               /* Stagger */
#define LSTCU_12_PH1_DUR     LSTCU_12.PH1_DUR.R            /* Phase 1 Duration */
#define LSTCU_12_ISODUR      LSTCU_12.ISODUR.R             /* Isolation Duration */
#define LSTCU_12_SRCENDUR    LSTCU_12.SRCENDUR.R           /* Synchronous Reset Clock Enable Duration */
#define LSTCU_12_MBPTR0      LSTCU_12.MBPTR[0].R           /* MBIST Scheduler Pointer */
#define LSTCU_12_LBPTR0      LSTCU_12.LBPTR[0].R           /* LBIST Scheduler Pointer */

#define LSTCU_4_ERR_STAT     LSTCU_4.ERR_STAT.R            /* Error Status */
#define LSTCU_4_ERR_FM       LSTCU_4.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_4_MB_RSTAT0    LSTCU_4.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_4_LB_RSTAT0    LSTCU_4.LB_RSTAT0.R           /* LBIST Run Status 0 */
#define LSTCU_4_MBFM0        LSTCU_4.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_4_LBFM0        LSTCU_4.LBFM0.R               /* LBIST Fault Mapping 0 */
#define LSTCU_4_STAG         LSTCU_4.STAG.R                /* Stagger */
#define LSTCU_4_PH1_DUR      LSTCU_4.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_4_ISODUR       LSTCU_4.ISODUR.R              /* Isolation Duration */
#define LSTCU_4_SRCENDUR     LSTCU_4.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_4_MBPTR0       LSTCU_4.MBPTR[0].R            /* MBIST Scheduler Pointer */
#define LSTCU_4_LBPTR0       LSTCU_4.LBPTR[0].R            /* LBIST Scheduler Pointer */

#define LSTCU_5_ERR_STAT     LSTCU_5.ERR_STAT.R            /* Error Status */
#define LSTCU_5_ERR_FM       LSTCU_5.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_5_MB_RSTAT0    LSTCU_5.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_5_LB_RSTAT0    LSTCU_5.LB_RSTAT0.R           /* LBIST Run Status 0 */
#define LSTCU_5_MBFM0        LSTCU_5.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_5_LBFM0        LSTCU_5.LBFM0.R               /* LBIST Fault Mapping 0 */
#define LSTCU_5_STAG         LSTCU_5.STAG.R                /* Stagger */
#define LSTCU_5_PH1_DUR      LSTCU_5.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_5_ISODUR       LSTCU_5.ISODUR.R              /* Isolation Duration */
#define LSTCU_5_SRCENDUR     LSTCU_5.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_5_MBPTR0       LSTCU_5.MBPTR[0].R            /* MBIST Scheduler Pointer */
#define LSTCU_5_LBPTR0       LSTCU_5.LBPTR[0].R            /* LBIST Scheduler Pointer */

/* LSTCU_14_15_17_18 */
#define LSTCU_14_ERR_STAT    LSTCU_14.ERR_STAT.R           /* Error Status */
#define LSTCU_14_ERR_FM      LSTCU_14.ERR_FM.R             /* Error Fault Mapping */
#define LSTCU_14_MB_RSTAT0   LSTCU_14.MB_RSTAT0.R          /* MBIST Run Status 0 */
#define LSTCU_14_MB_RSTAT1   LSTCU_14.MB_RSTAT1.R          /* MBIST Run Status 1 */
#define LSTCU_14_LB_RSTAT0   LSTCU_14.LB_RSTAT0.R          /* LBIST Run Status 0 */
#define LSTCU_14_MBFM0       LSTCU_14.MBFM0.R              /* MBIST Fault Mapping 0 */
#define LSTCU_14_MBFM1       LSTCU_14.MBFM1.R              /* MBIST Fault Mapping 1 */
#define LSTCU_14_LBFM0       LSTCU_14.LBFM0.R              /* LBIST Fault Mapping 0 */
#define LSTCU_14_STAG        LSTCU_14.STAG.R               /* Stagger */
#define LSTCU_14_PH1_DUR     LSTCU_14.PH1_DUR.R            /* Phase 1 Duration */
#define LSTCU_14_MBPTR0      LSTCU_14.MBPTR[0].R           /* MBIST Scheduler Pointer */
#define LSTCU_14_MBPTR1      LSTCU_14.MBPTR[1].R           /* MBIST Scheduler Pointer */
#define LSTCU_14_LBPTR0      LSTCU_14.LBPTR[0].R           /* LBIST Scheduler Pointer */

#define LSTCU_17_ERR_STAT    LSTCU_17.ERR_STAT.R           /* Error Status */
#define LSTCU_17_ERR_FM      LSTCU_17.ERR_FM.R             /* Error Fault Mapping */
#define LSTCU_17_MB_RSTAT0   LSTCU_17.MB_RSTAT0.R          /* MBIST Run Status 0 */
#define LSTCU_17_MB_RSTAT1   LSTCU_17.MB_RSTAT1.R          /* MBIST Run Status 1 */
#define LSTCU_17_LB_RSTAT0   LSTCU_17.LB_RSTAT0.R          /* LBIST Run Status 0 */
#define LSTCU_17_MBFM0       LSTCU_17.MBFM0.R              /* MBIST Fault Mapping 0 */
#define LSTCU_17_MBFM1       LSTCU_17.MBFM1.R              /* MBIST Fault Mapping 1 */
#define LSTCU_17_LBFM0       LSTCU_17.LBFM0.R              /* LBIST Fault Mapping 0 */
#define LSTCU_17_STAG        LSTCU_17.STAG.R               /* Stagger */
#define LSTCU_17_PH1_DUR     LSTCU_17.PH1_DUR.R            /* Phase 1 Duration */
#define LSTCU_17_MBPTR0      LSTCU_17.MBPTR[0].R           /* MBIST Scheduler Pointer */
#define LSTCU_17_MBPTR1      LSTCU_17.MBPTR[1].R           /* MBIST Scheduler Pointer */
#define LSTCU_17_LBPTR0      LSTCU_17.LBPTR[0].R           /* LBIST Scheduler Pointer */

#define LSTCU_15_ERR_STAT    LSTCU_15.ERR_STAT.R           /* Error Status */
#define LSTCU_15_ERR_FM      LSTCU_15.ERR_FM.R             /* Error Fault Mapping */
#define LSTCU_15_MB_RSTAT0   LSTCU_15.MB_RSTAT0.R          /* MBIST Run Status 0 */
#define LSTCU_15_MB_RSTAT1   LSTCU_15.MB_RSTAT1.R          /* MBIST Run Status 1 */
#define LSTCU_15_LB_RSTAT0   LSTCU_15.LB_RSTAT0.R          /* LBIST Run Status 0 */
#define LSTCU_15_MBFM0       LSTCU_15.MBFM0.R              /* MBIST Fault Mapping 0 */
#define LSTCU_15_MBFM1       LSTCU_15.MBFM1.R              /* MBIST Fault Mapping 1 */
#define LSTCU_15_LBFM0       LSTCU_15.LBFM0.R              /* LBIST Fault Mapping 0 */
#define LSTCU_15_STAG        LSTCU_15.STAG.R               /* Stagger */
#define LSTCU_15_PH1_DUR     LSTCU_15.PH1_DUR.R            /* Phase 1 Duration */
#define LSTCU_15_MBPTR0      LSTCU_15.MBPTR[0].R           /* MBIST Scheduler Pointer */
#define LSTCU_15_MBPTR1      LSTCU_15.MBPTR[1].R           /* MBIST Scheduler Pointer */
#define LSTCU_15_LBPTR0      LSTCU_15.LBPTR[0].R           /* LBIST Scheduler Pointer */

#define LSTCU_18_ERR_STAT    LSTCU_18.ERR_STAT.R           /* Error Status */
#define LSTCU_18_ERR_FM      LSTCU_18.ERR_FM.R             /* Error Fault Mapping */
#define LSTCU_18_MB_RSTAT0   LSTCU_18.MB_RSTAT0.R          /* MBIST Run Status 0 */
#define LSTCU_18_MB_RSTAT1   LSTCU_18.MB_RSTAT1.R          /* MBIST Run Status 1 */
#define LSTCU_18_LB_RSTAT0   LSTCU_18.LB_RSTAT0.R          /* LBIST Run Status 0 */
#define LSTCU_18_MBFM0       LSTCU_18.MBFM0.R              /* MBIST Fault Mapping 0 */
#define LSTCU_18_MBFM1       LSTCU_18.MBFM1.R              /* MBIST Fault Mapping 1 */
#define LSTCU_18_LBFM0       LSTCU_18.LBFM0.R              /* LBIST Fault Mapping 0 */
#define LSTCU_18_STAG        LSTCU_18.STAG.R               /* Stagger */
#define LSTCU_18_PH1_DUR     LSTCU_18.PH1_DUR.R            /* Phase 1 Duration */
#define LSTCU_18_MBPTR0      LSTCU_18.MBPTR[0].R           /* MBIST Scheduler Pointer */
#define LSTCU_18_MBPTR1      LSTCU_18.MBPTR[1].R           /* MBIST Scheduler Pointer */
#define LSTCU_18_LBPTR0      LSTCU_18.LBPTR[0].R           /* LBIST Scheduler Pointer */

/* LSTCU_19 */
#define LSTCU_19_ERR_STAT    LSTCU_19.ERR_STAT.R           /* Error Status */
#define LSTCU_19_ERR_FM      LSTCU_19.ERR_FM.R             /* Error Fault Mapping */
#define LSTCU_19_MB_RSTAT0   LSTCU_19.MB_RSTAT0.R          /* MBIST Run Status 0 */
#define LSTCU_19_MBFM0       LSTCU_19.MBFM0.R              /* MBIST Fault Mapping 0 */
#define LSTCU_19_STAG        LSTCU_19.STAG.R               /* Stagger */
#define LSTCU_19_PH1_DUR     LSTCU_19.PH1_DUR.R            /* Phase 1 Duration */
#define LSTCU_19_ISODUR      LSTCU_19.ISODUR.R             /* Isolation Duration */
#define LSTCU_19_SRCENDUR    LSTCU_19.SRCENDUR.R           /* Synchronous Reset Clock Enable Duration */
#define LSTCU_19_MBPTR0      LSTCU_19.MBPTR[0].R           /* MBIST Scheduler Pointer */

/* LSTCU_3_7 */
#define LSTCU_3_ERR_STAT     LSTCU_3.ERR_STAT.R            /* Error Status */
#define LSTCU_3_ERR_FM       LSTCU_3.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_3_MB_RSTAT0    LSTCU_3.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_3_MBFM0        LSTCU_3.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_3_STAG         LSTCU_3.STAG.R                /* Stagger */
#define LSTCU_3_PH1_DUR      LSTCU_3.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_3_ISODUR       LSTCU_3.ISODUR.R              /* Isolation Duration */
#define LSTCU_3_SRCENDUR     LSTCU_3.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_3_MBPTR0       LSTCU_3.MBPTR[0].R            /* MBIST Scheduler Pointer */

#define LSTCU_7_ERR_STAT     LSTCU_7.ERR_STAT.R            /* Error Status */
#define LSTCU_7_ERR_FM       LSTCU_7.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_7_MB_RSTAT0    LSTCU_7.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_7_MBFM0        LSTCU_7.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_7_STAG         LSTCU_7.STAG.R                /* Stagger */
#define LSTCU_7_PH1_DUR      LSTCU_7.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_7_ISODUR       LSTCU_7.ISODUR.R              /* Isolation Duration */
#define LSTCU_7_SRCENDUR     LSTCU_7.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_7_MBPTR0       LSTCU_7.MBPTR[0].R            /* MBIST Scheduler Pointer */

/* LSTCU_8 */
#define LSTCU_8_ERR_STAT     LSTCU_8.ERR_STAT.R            /* Error Status */
#define LSTCU_8_ERR_FM       LSTCU_8.ERR_FM.R              /* Error Fault Mapping */
#define LSTCU_8_MB_RSTAT0    LSTCU_8.MB_RSTAT0.R           /* MBIST Run Status 0 */
#define LSTCU_8_MBFM0        LSTCU_8.MBFM0.R               /* MBIST Fault Mapping 0 */
#define LSTCU_8_STAG         LSTCU_8.STAG.R                /* Stagger */
#define LSTCU_8_PH1_DUR      LSTCU_8.PH1_DUR.R             /* Phase 1 Duration */
#define LSTCU_8_ISODUR       LSTCU_8.ISODUR.R              /* Isolation Duration */
#define LSTCU_8_SRCENDUR     LSTCU_8.SRCENDUR.R            /* Synchronous Reset Clock Enable Duration */
#define LSTCU_8_MBPTR0       LSTCU_8.MBPTR[0].R            /* MBIST Scheduler Pointer */

/* L_VFCCU */
#define L_VFCCU_3_GDFHID_C0  L_VFCCU_3.GDFHID_C0.R         /* Global DID-FHID Map */
#define L_VFCCU_3_GDFHID_C1  L_VFCCU_3.GDFHID_C1.R         /* Global DID-FHID Map */
#define L_VFCCU_3_GFLTPO_C0  L_VFCCU_3.GFLTPO_C0.R         /* Global Fault Polarity */
#define L_VFCCU_3_GFLTRC_C0  L_VFCCU_3.GFLTRC_C0.R         /* Global Fault Recovery */
#define L_VFCCU_3_GFLTOVDC0  L_VFCCU_3.GFLTOVDC0.R         /* Global Fault Overflow Detection */
#define L_VFCCU_3_GCTRL      L_VFCCU_3.GCTRL.R             /* Global Space Control */
#define L_VFCCU_3_GINTOVFS   L_VFCCU_3.GINTOVFS.R          /* Global DID FSM Status */
#define L_VFCCU_3_GDBGCFG    L_VFCCU_3.GDBGCFG.R           /* Global Debug */
#define L_VFCCU_3_GDBGSTAT   L_VFCCU_3.GDBGSTAT.R          /* Global Debug Status */
#define L_VFCCU_3_FHCFG0     L_VFCCU_3.FHCFG0.R            /* Fault Handler */
#define L_VFCCU_3_FHSRVDS0   L_VFCCU_3.FHSRVDS0.R          /* Fault Handler Status */
#define L_VFCCU_3_FHFLTENC00 L_VFCCU_3.FHFLTENC00.R        /* Fault Enable */
#define L_VFCCU_3_FHFLTS00   L_VFCCU_3.FHFLTS00.R          /* Fault Status */
#define L_VFCCU_3_FHFLTRKC00 L_VFCCU_3.FHFLTRKC00.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_3_FHFLTRKC01 L_VFCCU_3.FHFLTRKC01.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_3_FHFLTRKC02 L_VFCCU_3.FHFLTRKC02.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_3_FHFLTRKC03 L_VFCCU_3.FHFLTRKC03.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_3_FHIMRKC0_00 L_VFCCU_3.FHIMRKC0_00.R      /* Immediate Reaction Configuration */
#define L_VFCCU_3_FHIMRKC0_10 L_VFCCU_3.FHIMRKC0_10.R      /* Immediate Reaction Configuration */
#define L_VFCCU_3_FHIMRKC0_20 L_VFCCU_3.FHIMRKC0_20.R      /* Immediate Reaction Configuration */
#define L_VFCCU_3_FHIMRKC0_30 L_VFCCU_3.FHIMRKC0_30.R      /* Immediate Reaction Configuration */
#define L_VFCCU_3_FHIMRKC0_40 L_VFCCU_3.FHIMRKC0_40.R      /* Immediate Reaction Configuration */
#define L_VFCCU_3_FHIMRKC0_50 L_VFCCU_3.FHIMRKC0_50.R      /* Immediate Reaction Configuration */

#define L_VFCCU_6_GDFHID_C0  L_VFCCU_6.GDFHID_C0.R         /* Global DID-FHID Map */
#define L_VFCCU_6_GDFHID_C1  L_VFCCU_6.GDFHID_C1.R         /* Global DID-FHID Map */
#define L_VFCCU_6_GFLTPO_C0  L_VFCCU_6.GFLTPO_C0.R         /* Global Fault Polarity */
#define L_VFCCU_6_GFLTRC_C0  L_VFCCU_6.GFLTRC_C0.R         /* Global Fault Recovery */
#define L_VFCCU_6_GFLTOVDC0  L_VFCCU_6.GFLTOVDC0.R         /* Global Fault Overflow Detection */
#define L_VFCCU_6_GCTRL      L_VFCCU_6.GCTRL.R             /* Global Space Control */
#define L_VFCCU_6_GINTOVFS   L_VFCCU_6.GINTOVFS.R          /* Global DID FSM Status */
#define L_VFCCU_6_GDBGCFG    L_VFCCU_6.GDBGCFG.R           /* Global Debug */
#define L_VFCCU_6_GDBGSTAT   L_VFCCU_6.GDBGSTAT.R          /* Global Debug Status */
#define L_VFCCU_6_FHCFG0     L_VFCCU_6.FHCFG0.R            /* Fault Handler */
#define L_VFCCU_6_FHSRVDS0   L_VFCCU_6.FHSRVDS0.R          /* Fault Handler Status */
#define L_VFCCU_6_FHFLTENC00 L_VFCCU_6.FHFLTENC00.R        /* Fault Enable */
#define L_VFCCU_6_FHFLTS00   L_VFCCU_6.FHFLTS00.R          /* Fault Status */
#define L_VFCCU_6_FHFLTRKC00 L_VFCCU_6.FHFLTRKC00.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_6_FHFLTRKC01 L_VFCCU_6.FHFLTRKC01.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_6_FHFLTRKC02 L_VFCCU_6.FHFLTRKC02.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_6_FHIMRKC0_00 L_VFCCU_6.FHIMRKC0_00.R      /* Immediate Reaction Configuration */
#define L_VFCCU_6_FHIMRKC0_10 L_VFCCU_6.FHIMRKC0_10.R      /* Immediate Reaction Configuration */
#define L_VFCCU_6_FHIMRKC0_20 L_VFCCU_6.FHIMRKC0_20.R      /* Immediate Reaction Configuration */
#define L_VFCCU_6_FHIMRKC0_30 L_VFCCU_6.FHIMRKC0_30.R      /* Immediate Reaction Configuration */
#define L_VFCCU_6_FHIMRKC0_40 L_VFCCU_6.FHIMRKC0_40.R      /* Immediate Reaction Configuration */
#define L_VFCCU_6_FHIMRKC0_50 L_VFCCU_6.FHIMRKC0_50.R      /* Immediate Reaction Configuration */

#define L_VFCCU_1_GDFHID_C0  L_VFCCU_1.GDFHID_C0.R         /* Global DID-FHID Map */
#define L_VFCCU_1_GDFHID_C1  L_VFCCU_1.GDFHID_C1.R         /* Global DID-FHID Map */
#define L_VFCCU_1_GFLTPO_C0  L_VFCCU_1.GFLTPO_C0.R         /* Global Fault Polarity */
#define L_VFCCU_1_GFLTRC_C0  L_VFCCU_1.GFLTRC_C0.R         /* Global Fault Recovery */
#define L_VFCCU_1_GFLTOVDC0  L_VFCCU_1.GFLTOVDC0.R         /* Global Fault Overflow Detection */
#define L_VFCCU_1_GCTRL      L_VFCCU_1.GCTRL.R             /* Global Space Control */
#define L_VFCCU_1_GINTOVFS   L_VFCCU_1.GINTOVFS.R          /* Global DID FSM Status */
#define L_VFCCU_1_GDBGCFG    L_VFCCU_1.GDBGCFG.R           /* Global Debug */
#define L_VFCCU_1_GDBGSTAT   L_VFCCU_1.GDBGSTAT.R          /* Global Debug Status */
#define L_VFCCU_1_FHCFG0     L_VFCCU_1.FHCFG0.R            /* Fault Handler */
#define L_VFCCU_1_FHSRVDS0   L_VFCCU_1.FHSRVDS0.R          /* Fault Handler Status */
#define L_VFCCU_1_FHFLTENC00 L_VFCCU_1.FHFLTENC00.R        /* Fault Enable */
#define L_VFCCU_1_FHFLTS00   L_VFCCU_1.FHFLTS00.R          /* Fault Status */
#define L_VFCCU_1_FHFLTRKC00 L_VFCCU_1.FHFLTRKC00.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_1_FHFLTRKC01 L_VFCCU_1.FHFLTRKC01.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_1_FHFLTRKC02 L_VFCCU_1.FHFLTRKC02.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_1_FHIMRKC0_00 L_VFCCU_1.FHIMRKC0_00.R      /* Immediate Reaction Configuration */
#define L_VFCCU_1_FHIMRKC0_10 L_VFCCU_1.FHIMRKC0_10.R      /* Immediate Reaction Configuration */
#define L_VFCCU_1_FHIMRKC0_20 L_VFCCU_1.FHIMRKC0_20.R      /* Immediate Reaction Configuration */
#define L_VFCCU_1_FHIMRKC0_30 L_VFCCU_1.FHIMRKC0_30.R      /* Immediate Reaction Configuration */
#define L_VFCCU_1_FHIMRKC0_40 L_VFCCU_1.FHIMRKC0_40.R      /* Immediate Reaction Configuration */
#define L_VFCCU_1_FHIMRKC0_50 L_VFCCU_1.FHIMRKC0_50.R      /* Immediate Reaction Configuration */

#define L_VFCCU_2_GDFHID_C0  L_VFCCU_2.GDFHID_C0.R         /* Global DID-FHID Map */
#define L_VFCCU_2_GDFHID_C1  L_VFCCU_2.GDFHID_C1.R         /* Global DID-FHID Map */
#define L_VFCCU_2_GFLTPO_C0  L_VFCCU_2.GFLTPO_C0.R         /* Global Fault Polarity */
#define L_VFCCU_2_GFLTRC_C0  L_VFCCU_2.GFLTRC_C0.R         /* Global Fault Recovery */
#define L_VFCCU_2_GFLTOVDC0  L_VFCCU_2.GFLTOVDC0.R         /* Global Fault Overflow Detection */
#define L_VFCCU_2_GCTRL      L_VFCCU_2.GCTRL.R             /* Global Space Control */
#define L_VFCCU_2_GINTOVFS   L_VFCCU_2.GINTOVFS.R          /* Global DID FSM Status */
#define L_VFCCU_2_GDBGCFG    L_VFCCU_2.GDBGCFG.R           /* Global Debug */
#define L_VFCCU_2_GDBGSTAT   L_VFCCU_2.GDBGSTAT.R          /* Global Debug Status */
#define L_VFCCU_2_FHCFG0     L_VFCCU_2.FHCFG0.R            /* Fault Handler */
#define L_VFCCU_2_FHSRVDS0   L_VFCCU_2.FHSRVDS0.R          /* Fault Handler Status */
#define L_VFCCU_2_FHFLTENC00 L_VFCCU_2.FHFLTENC00.R        /* Fault Enable */
#define L_VFCCU_2_FHFLTS00   L_VFCCU_2.FHFLTS00.R          /* Fault Status */
#define L_VFCCU_2_FHFLTRKC00 L_VFCCU_2.FHFLTRKC00.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_2_FHFLTRKC01 L_VFCCU_2.FHFLTRKC01.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_2_FHFLTRKC02 L_VFCCU_2.FHFLTRKC02.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_2_FHIMRKC0_00 L_VFCCU_2.FHIMRKC0_00.R      /* Immediate Reaction Configuration */
#define L_VFCCU_2_FHIMRKC0_10 L_VFCCU_2.FHIMRKC0_10.R      /* Immediate Reaction Configuration */
#define L_VFCCU_2_FHIMRKC0_20 L_VFCCU_2.FHIMRKC0_20.R      /* Immediate Reaction Configuration */
#define L_VFCCU_2_FHIMRKC0_30 L_VFCCU_2.FHIMRKC0_30.R      /* Immediate Reaction Configuration */
#define L_VFCCU_2_FHIMRKC0_40 L_VFCCU_2.FHIMRKC0_40.R      /* Immediate Reaction Configuration */
#define L_VFCCU_2_FHIMRKC0_50 L_VFCCU_2.FHIMRKC0_50.R      /* Immediate Reaction Configuration */

#define L_VFCCU_0_GDFHID_C0  L_VFCCU_0.GDFHID_C0.R         /* Global DID-FHID Map */
#define L_VFCCU_0_GDFHID_C1  L_VFCCU_0.GDFHID_C1.R         /* Global DID-FHID Map */
#define L_VFCCU_0_GFLTPO_C0  L_VFCCU_0.GFLTPO_C0.R         /* Global Fault Polarity */
#define L_VFCCU_0_GFLTRC_C0  L_VFCCU_0.GFLTRC_C0.R         /* Global Fault Recovery */
#define L_VFCCU_0_GFLTOVDC0  L_VFCCU_0.GFLTOVDC0.R         /* Global Fault Overflow Detection */
#define L_VFCCU_0_GCTRL      L_VFCCU_0.GCTRL.R             /* Global Space Control */
#define L_VFCCU_0_GINTOVFS   L_VFCCU_0.GINTOVFS.R          /* Global DID FSM Status */
#define L_VFCCU_0_GDBGCFG    L_VFCCU_0.GDBGCFG.R           /* Global Debug */
#define L_VFCCU_0_GDBGSTAT   L_VFCCU_0.GDBGSTAT.R          /* Global Debug Status */
#define L_VFCCU_0_FHCFG0     L_VFCCU_0.FHCFG0.R            /* Fault Handler */
#define L_VFCCU_0_FHSRVDS0   L_VFCCU_0.FHSRVDS0.R          /* Fault Handler Status */
#define L_VFCCU_0_FHFLTENC00 L_VFCCU_0.FHFLTENC00.R        /* Fault Enable */
#define L_VFCCU_0_FHFLTS00   L_VFCCU_0.FHFLTS00.R          /* Fault Status */
#define L_VFCCU_0_FHFLTRKC00 L_VFCCU_0.FHFLTRKC00.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_0_FHFLTRKC01 L_VFCCU_0.FHFLTRKC01.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_0_FHFLTRKC02 L_VFCCU_0.FHFLTRKC02.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_0_FHIMRKC0_00 L_VFCCU_0.FHIMRKC0_00.R      /* Immediate Reaction Configuration */
#define L_VFCCU_0_FHIMRKC0_10 L_VFCCU_0.FHIMRKC0_10.R      /* Immediate Reaction Configuration */
#define L_VFCCU_0_FHIMRKC0_20 L_VFCCU_0.FHIMRKC0_20.R      /* Immediate Reaction Configuration */
#define L_VFCCU_0_FHIMRKC0_30 L_VFCCU_0.FHIMRKC0_30.R      /* Immediate Reaction Configuration */
#define L_VFCCU_0_FHIMRKC0_40 L_VFCCU_0.FHIMRKC0_40.R      /* Immediate Reaction Configuration */
#define L_VFCCU_0_FHIMRKC0_50 L_VFCCU_0.FHIMRKC0_50.R      /* Immediate Reaction Configuration */

#define L_VFCCU_4_GDFHID_C0  L_VFCCU_4.GDFHID_C0.R         /* Global DID-FHID Map */
#define L_VFCCU_4_GDFHID_C1  L_VFCCU_4.GDFHID_C1.R         /* Global DID-FHID Map */
#define L_VFCCU_4_GFLTPO_C0  L_VFCCU_4.GFLTPO_C0.R         /* Global Fault Polarity */
#define L_VFCCU_4_GFLTRC_C0  L_VFCCU_4.GFLTRC_C0.R         /* Global Fault Recovery */
#define L_VFCCU_4_GFLTOVDC0  L_VFCCU_4.GFLTOVDC0.R         /* Global Fault Overflow Detection */
#define L_VFCCU_4_GCTRL      L_VFCCU_4.GCTRL.R             /* Global Space Control */
#define L_VFCCU_4_GINTOVFS   L_VFCCU_4.GINTOVFS.R          /* Global DID FSM Status */
#define L_VFCCU_4_GDBGCFG    L_VFCCU_4.GDBGCFG.R           /* Global Debug */
#define L_VFCCU_4_GDBGSTAT   L_VFCCU_4.GDBGSTAT.R          /* Global Debug Status */
#define L_VFCCU_4_FHCFG0     L_VFCCU_4.FHCFG0.R            /* Fault Handler */
#define L_VFCCU_4_FHSRVDS0   L_VFCCU_4.FHSRVDS0.R          /* Fault Handler Status */
#define L_VFCCU_4_FHFLTENC00 L_VFCCU_4.FHFLTENC00.R        /* Fault Enable */
#define L_VFCCU_4_FHFLTS00   L_VFCCU_4.FHFLTS00.R          /* Fault Status */
#define L_VFCCU_4_FHFLTRKC00 L_VFCCU_4.FHFLTRKC00.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_4_FHFLTRKC01 L_VFCCU_4.FHFLTRKC01.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_4_FHFLTRKC02 L_VFCCU_4.FHFLTRKC02.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_4_FHIMRKC0_00 L_VFCCU_4.FHIMRKC0_00.R      /* Immediate Reaction Configuration */
#define L_VFCCU_4_FHIMRKC0_10 L_VFCCU_4.FHIMRKC0_10.R      /* Immediate Reaction Configuration */
#define L_VFCCU_4_FHIMRKC0_20 L_VFCCU_4.FHIMRKC0_20.R      /* Immediate Reaction Configuration */
#define L_VFCCU_4_FHIMRKC0_30 L_VFCCU_4.FHIMRKC0_30.R      /* Immediate Reaction Configuration */
#define L_VFCCU_4_FHIMRKC0_40 L_VFCCU_4.FHIMRKC0_40.R      /* Immediate Reaction Configuration */
#define L_VFCCU_4_FHIMRKC0_50 L_VFCCU_4.FHIMRKC0_50.R      /* Immediate Reaction Configuration */

#define L_VFCCU_5_GDFHID_C0  L_VFCCU_5.GDFHID_C0.R         /* Global DID-FHID Map */
#define L_VFCCU_5_GDFHID_C1  L_VFCCU_5.GDFHID_C1.R         /* Global DID-FHID Map */
#define L_VFCCU_5_GFLTPO_C0  L_VFCCU_5.GFLTPO_C0.R         /* Global Fault Polarity */
#define L_VFCCU_5_GFLTRC_C0  L_VFCCU_5.GFLTRC_C0.R         /* Global Fault Recovery */
#define L_VFCCU_5_GFLTOVDC0  L_VFCCU_5.GFLTOVDC0.R         /* Global Fault Overflow Detection */
#define L_VFCCU_5_GCTRL      L_VFCCU_5.GCTRL.R             /* Global Space Control */
#define L_VFCCU_5_GINTOVFS   L_VFCCU_5.GINTOVFS.R          /* Global DID FSM Status */
#define L_VFCCU_5_GDBGCFG    L_VFCCU_5.GDBGCFG.R           /* Global Debug */
#define L_VFCCU_5_GDBGSTAT   L_VFCCU_5.GDBGSTAT.R          /* Global Debug Status */
#define L_VFCCU_5_FHCFG0     L_VFCCU_5.FHCFG0.R            /* Fault Handler */
#define L_VFCCU_5_FHSRVDS0   L_VFCCU_5.FHSRVDS0.R          /* Fault Handler Status */
#define L_VFCCU_5_FHFLTENC00 L_VFCCU_5.FHFLTENC00.R        /* Fault Enable */
#define L_VFCCU_5_FHFLTS00   L_VFCCU_5.FHFLTS00.R          /* Fault Status */
#define L_VFCCU_5_FHFLTRKC00 L_VFCCU_5.FHFLTRKC00.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_5_FHFLTRKC01 L_VFCCU_5.FHFLTRKC01.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_5_FHFLTRKC02 L_VFCCU_5.FHFLTRKC02.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_5_FHFLTRKC03 L_VFCCU_5.FHFLTRKC03.R        /* Fault Reaction Set Configuration */
#define L_VFCCU_5_FHIMRKC0_00 L_VFCCU_5.FHIMRKC0_00.R      /* Immediate Reaction Configuration */
#define L_VFCCU_5_FHIMRKC0_10 L_VFCCU_5.FHIMRKC0_10.R      /* Immediate Reaction Configuration */
#define L_VFCCU_5_FHIMRKC0_20 L_VFCCU_5.FHIMRKC0_20.R      /* Immediate Reaction Configuration */
#define L_VFCCU_5_FHIMRKC0_30 L_VFCCU_5.FHIMRKC0_30.R      /* Immediate Reaction Configuration */
#define L_VFCCU_5_FHIMRKC0_40 L_VFCCU_5.FHIMRKC0_40.R      /* Immediate Reaction Configuration */
#define L_VFCCU_5_FHIMRKC0_50 L_VFCCU_5.FHIMRKC0_50.R      /* Immediate Reaction Configuration */

/* MCM */
#define CE_MCM_PLASC         CE_MCM.PLASC.R                /* Crossbar Switch (AXBS) Slave Configuration */
#define CE_MCM_PLAMC         CE_MCM.PLAMC.R                /* Crossbar Switch Master Configuration */
#define CE_MCM_ISCR          CE_MCM.ISCR.R                 /* Interrupt Status and Control */
#define CE_MCM_FADR          CE_MCM.FADR.R                 /* Write Buffer Fault Address */
#define CE_MCM_FATR          CE_MCM.FATR.R                 /* Store Buffer Fault Attributes */
#define CE_MCM_FDR           CE_MCM.FDR.R                  /* Store Buffer Fault Data */
#define CE_MCM_CPCR2         CE_MCM.CPCR2.R                /* Core Platform Control 2 */
#define CE_MCM_GPCR0         CE_MCM.GPCR0.R                /* General Purpose Control 0 */
#define CE_MCM_LMDR0         CE_MCM.LMDR0.R                /* Local Memory Descriptor 0 */
#define CE_MCM_LMDR1         CE_MCM.LMDR1.R                /* Local Memory Descriptor 1 */

#define SMU__MCM_PLASC       SMU__MCM.PLASC.R              /* Crossbar Switch (AXBS) Slave Configuration */
#define SMU__MCM_PLAMC       SMU__MCM.PLAMC.R              /* Crossbar Switch Master Configuration */
#define SMU__MCM_ISCR        SMU__MCM.ISCR.R               /* Interrupt Status and Control */
#define SMU__MCM_FADR        SMU__MCM.FADR.R               /* Write Buffer Fault Address */
#define SMU__MCM_FATR        SMU__MCM.FATR.R               /* Store Buffer Fault Attributes */
#define SMU__MCM_FDR         SMU__MCM.FDR.R                /* Store Buffer Fault Data */
#define SMU__MCM_CPCR2       SMU__MCM.CPCR2.R              /* Core Platform Control 2 */
#define SMU__MCM_GPCR0       SMU__MCM.GPCR0.R              /* General Purpose Control 0 */
#define SMU__MCM_LMDR0       SMU__MCM.LMDR0.R              /* Local Memory Descriptor 0 */
#define SMU__MCM_LMDR1       SMU__MCM.LMDR1.R              /* Local Memory Descriptor 1 */

/* MC_CGM_0 */
#define MC_CGM_0_PCFS_SDUR   MC_CGM_0.PCFS_SDUR.R          /* PCFS Step Duration */
#define MC_CGM_0_PCFS_DIVC12 MC_CGM_0.PCFS_DIVC12.R        /* PCFS Divider Change 12 Register */
#define MC_CGM_0_PCFS_DIVE12 MC_CGM_0.PCFS_DIVE12.R        /* PCFS Divider End 12 Register */
#define MC_CGM_0_PCFS_DIVS12 MC_CGM_0.PCFS_DIVS12.R        /* PCFS Divider Start 12 Register */
#define MC_CGM_0_MUX_0_CSC   MC_CGM_0.MUX_0_CSC.R          /* Clock Mux 0 Select Control Register */
#define MC_CGM_0_MUX_0_CSS   MC_CGM_0.MUX_0_CSS.R          /* Clock Mux 0 Select Status Register */
#define MC_CGM_0_MUX_1_CSC   MC_CGM_0.MUX_1_CSC.R          /* Clock Mux 1 Select Control Register */
#define MC_CGM_0_MUX_1_CSS   MC_CGM_0.MUX_1_CSS.R          /* Clock Mux 1 Select Status Register */
#define MC_CGM_0_MUX_1_DC_0  MC_CGM_0.MUX_1_DC_0.R         /* Clock Mux 1 Divider 0 Control Register */
#define MC_CGM_0_MUX_1_DC_1  MC_CGM_0.MUX_1_DC_1.R         /* Clock Mux 1 Divider 1 Control Register */
#define MC_CGM_0_MUX_1_DIV_UPD_STAT MC_CGM_0.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */
#define MC_CGM_0_MUX_2_CSC   MC_CGM_0.MUX_2_CSC.R          /* Clock Mux 2 Select Control Register */
#define MC_CGM_0_MUX_2_CSS   MC_CGM_0.MUX_2_CSS.R          /* Clock Mux 2 Select Status Register */
#define MC_CGM_0_MUX_2_DC_0  MC_CGM_0.MUX_2_DC_0.R         /* Clock Mux 2 Divider 0 Control Register */
#define MC_CGM_0_MUX_2_DC_1  MC_CGM_0.MUX_2_DC_1.R         /* Clock Mux 2 Divider 1 Control Register */
#define MC_CGM_0_MUX_2_DC_2  MC_CGM_0.MUX_2_DC_2.R         /* Clock Mux 2 Divider 2 Control Register */
#define MC_CGM_0_MUX_2_DC_3  MC_CGM_0.MUX_2_DC_3.R         /* Clock Mux 2 Divider 3 Control Register */
#define MC_CGM_0_MUX_2_DC_4  MC_CGM_0.MUX_2_DC_4.R         /* Clock Mux 2 Divider 4 Control Register */
#define MC_CGM_0_MUX_2_DC_5  MC_CGM_0.MUX_2_DC_5.R         /* Clock Mux 2 Divider 5 Control Register */
#define MC_CGM_0_MUX_2_DIV_UPD_STAT MC_CGM_0.MUX_2_DIV_UPD_STAT.R /* Clock Mux 2 Divider Update Status Register */
#define MC_CGM_0_MUX_3_CSC   MC_CGM_0.MUX_3_CSC.R          /* Clock Mux 3 Select Control Register */
#define MC_CGM_0_MUX_3_CSS   MC_CGM_0.MUX_3_CSS.R          /* Clock Mux 3 Select Status Register */
#define MC_CGM_0_MUX_3_DC_0  MC_CGM_0.MUX_3_DC_0.R         /* Clock Mux 3 Divider 0 Control Register */
#define MC_CGM_0_MUX_3_DC_1  MC_CGM_0.MUX_3_DC_1.R         /* Clock Mux 3 Divider 1 Control Register */
#define MC_CGM_0_MUX_3_DC_2  MC_CGM_0.MUX_3_DC_2.R         /* Clock Mux 3 Divider 2 Control Register */
#define MC_CGM_0_MUX_3_DC_3  MC_CGM_0.MUX_3_DC_3.R         /* Clock Mux 3 Divider 3 Control Register */
#define MC_CGM_0_MUX_3_DC_4  MC_CGM_0.MUX_3_DC_4.R         /* Clock Mux 3 Divider 4 Control Register */
#define MC_CGM_0_MUX_3_DC_5  MC_CGM_0.MUX_3_DC_5.R         /* Clock Mux 3 Divider 5 Control Register */
#define MC_CGM_0_MUX_3_DC_6  MC_CGM_0.MUX_3_DC_6.R         /* Clock Mux 3 Divider 6 Control Register */
#define MC_CGM_0_MUX_3_DC_7  MC_CGM_0.MUX_3_DC_7.R         /* Clock Mux 3 Divider 7 Control Register */
#define MC_CGM_0_MUX_3_DIV_UPD_STAT MC_CGM_0.MUX_3_DIV_UPD_STAT.R /* Clock Mux 3 Divider Update Status Register */
#define MC_CGM_0_MUX_4_CSC   MC_CGM_0.MUX_4_CSC.R          /* Clock Mux 4 Select Control Register */
#define MC_CGM_0_MUX_4_CSS   MC_CGM_0.MUX_4_CSS.R          /* Clock Mux 4 Select Status Register */
#define MC_CGM_0_MUX_4_DC_0  MC_CGM_0.MUX_4_DC_0.R         /* Clock Mux 4 Divider 0 Control Register */
#define MC_CGM_0_MUX_4_DIV_UPD_STAT MC_CGM_0.MUX_4_DIV_UPD_STAT.R /* Clock Mux 4 Divider Update Status Register */
#define MC_CGM_0_MUX_5_CSC   MC_CGM_0.MUX_5_CSC.R          /* Clock Mux 5 Select Control Register */
#define MC_CGM_0_MUX_5_CSS   MC_CGM_0.MUX_5_CSS.R          /* Clock Mux 5 Select Status Register */
#define MC_CGM_0_MUX_6_CSC   MC_CGM_0.MUX_6_CSC.R          /* Clock Mux 6 Select Control Register */
#define MC_CGM_0_MUX_6_CSS   MC_CGM_0.MUX_6_CSS.R          /* Clock Mux 6 Select Status Register */
#define MC_CGM_0_MUX_6_DC_0  MC_CGM_0.MUX_6_DC_0.R         /* Clock Mux 6 Divider 0 Control Register */
#define MC_CGM_0_MUX_6_DIV_UPD_STAT MC_CGM_0.MUX_6_DIV_UPD_STAT.R /* Clock Mux 6 Divider Update Status Register */
#define MC_CGM_0_MUX_7_CSC   MC_CGM_0.MUX_7_CSC.R          /* Clock Mux 7 Select Control Register */
#define MC_CGM_0_MUX_7_CSS   MC_CGM_0.MUX_7_CSS.R          /* Clock Mux 7 Select Status Register */
#define MC_CGM_0_MUX_7_DC_0  MC_CGM_0.MUX_7_DC_0.R         /* Clock Mux 7 Divider 0 Control Register */
#define MC_CGM_0_MUX_7_DC_1  MC_CGM_0.MUX_7_DC_1.R         /* Clock Mux 7 Divider 1 Control Register */
#define MC_CGM_0_MUX_7_DIV_TRIG_CTRL MC_CGM_0.MUX_7_DIV_TRIG_CTRL.R /* Clock Mux 7 Divider Trigger Control Register */
#define MC_CGM_0_MUX_7_DIV_TRIG MC_CGM_0.MUX_7_DIV_TRIG.R  /* Clock Mux 7 Divider Trigger Register */
#define MC_CGM_0_MUX_7_DIV_UPD_STAT MC_CGM_0.MUX_7_DIV_UPD_STAT.R /* Clock Mux 7 Divider Update Status Register */
#define MC_CGM_0_MUX_8_CSC   MC_CGM_0.MUX_8_CSC.R          /* Clock Mux 8 Select Control Register */
#define MC_CGM_0_MUX_8_CSS   MC_CGM_0.MUX_8_CSS.R          /* Clock Mux 8 Select Status Register */
#define MC_CGM_0_MUX_8_DC_0  MC_CGM_0.MUX_8_DC_0.R         /* Clock Mux 8 Divider 0 Control Register */
#define MC_CGM_0_MUX_8_DIV_UPD_STAT MC_CGM_0.MUX_8_DIV_UPD_STAT.R /* Clock Mux 8 Divider Update Status Register */
#define MC_CGM_0_MUX_9_CSC   MC_CGM_0.MUX_9_CSC.R          /* Clock Mux 9 Select Control Register */
#define MC_CGM_0_MUX_9_CSS   MC_CGM_0.MUX_9_CSS.R          /* Clock Mux 9 Select Status Register */
#define MC_CGM_0_MUX_9_DC_0  MC_CGM_0.MUX_9_DC_0.R         /* Clock Mux 9 Divider 0 Control Register */
#define MC_CGM_0_MUX_9_DIV_UPD_STAT MC_CGM_0.MUX_9_DIV_UPD_STAT.R /* Clock Mux 9 Divider Update Status Register */
#define MC_CGM_0_MUX_10_CSC  MC_CGM_0.MUX_10_CSC.R         /* Clock Mux 10 Select Control Register */
#define MC_CGM_0_MUX_10_CSS  MC_CGM_0.MUX_10_CSS.R         /* Clock Mux 10 Select Status Register */
#define MC_CGM_0_MUX_10_DC_0 MC_CGM_0.MUX_10_DC_0.R        /* Clock Mux 10 Divider 0 Control Register */
#define MC_CGM_0_MUX_10_DIV_UPD_STAT MC_CGM_0.MUX_10_DIV_UPD_STAT.R /* Clock Mux 10 Divider Update Status Register */

/* MC_CGM_1 */
#define MC_CGM_1_PCFS_SDUR   MC_CGM_1.PCFS_SDUR.R          /* PCFS Step Duration */
#define MC_CGM_1_PCFS_DIVC12 MC_CGM_1.PCFS_DIVC12.R        /* PCFS Divider Change 12 Register */
#define MC_CGM_1_PCFS_DIVE12 MC_CGM_1.PCFS_DIVE12.R        /* PCFS Divider End 12 Register */
#define MC_CGM_1_PCFS_DIVS12 MC_CGM_1.PCFS_DIVS12.R        /* PCFS Divider Start 12 Register */
#define MC_CGM_1_MUX_0_CSC   MC_CGM_1.MUX_0_CSC.R          /* Clock Mux 0 Select Control Register */
#define MC_CGM_1_MUX_0_CSS   MC_CGM_1.MUX_0_CSS.R          /* Clock Mux 0 Select Status Register */
#define MC_CGM_1_MUX_1_CSC   MC_CGM_1.MUX_1_CSC.R          /* Clock Mux 1 Select Control Register */
#define MC_CGM_1_MUX_1_CSS   MC_CGM_1.MUX_1_CSS.R          /* Clock Mux 1 Select Status Register */
#define MC_CGM_1_MUX_1_DC_0  MC_CGM_1.MUX_1_DC_0.R         /* Clock Mux 1 Divider 0 Control Register */
#define MC_CGM_1_MUX_1_DIV_UPD_STAT MC_CGM_1.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */
#define MC_CGM_1_MUX_2_CSC   MC_CGM_1.MUX_2_CSC.R          /* Clock Mux 2 Select Control Register */
#define MC_CGM_1_MUX_2_CSS   MC_CGM_1.MUX_2_CSS.R          /* Clock Mux 2 Select Status Register */
#define MC_CGM_1_MUX_3_CSC   MC_CGM_1.MUX_3_CSC.R          /* Clock Mux 3 Select Control Register */
#define MC_CGM_1_MUX_3_CSS   MC_CGM_1.MUX_3_CSS.R          /* Clock Mux 3 Select Status Register */
#define MC_CGM_1_MUX_4_CSC   MC_CGM_1.MUX_4_CSC.R          /* Clock Mux 4 Select Control Register */
#define MC_CGM_1_MUX_4_CSS   MC_CGM_1.MUX_4_CSS.R          /* Clock Mux 4 Select Status Register */
#define MC_CGM_1_MUX_4_DC_0  MC_CGM_1.MUX_4_DC_0.R         /* Clock Mux 4 Divider 0 Control Register */
#define MC_CGM_1_MUX_4_DIV_UPD_STAT MC_CGM_1.MUX_4_DIV_UPD_STAT.R /* Clock Mux 4 Divider Update Status Register */
#define MC_CGM_1_MUX_5_CSC   MC_CGM_1.MUX_5_CSC.R          /* Clock Mux 5 Select Control Register */
#define MC_CGM_1_MUX_5_CSS   MC_CGM_1.MUX_5_CSS.R          /* Clock Mux 5 Select Status Register */
#define MC_CGM_1_MUX_5_DC_0  MC_CGM_1.MUX_5_DC_0.R         /* Clock Mux 5 Divider 0 Control Register */
#define MC_CGM_1_MUX_5_DIV_UPD_STAT MC_CGM_1.MUX_5_DIV_UPD_STAT.R /* Clock Mux 5 Divider Update Status Register */
#define MC_CGM_1_MUX_6_CSC   MC_CGM_1.MUX_6_CSC.R          /* Clock Mux 6 Select Control Register */
#define MC_CGM_1_MUX_6_CSS   MC_CGM_1.MUX_6_CSS.R          /* Clock Mux 6 Select Status Register */
#define MC_CGM_1_MUX_6_DC_0  MC_CGM_1.MUX_6_DC_0.R         /* Clock Mux 6 Divider 0 Control Register */
#define MC_CGM_1_MUX_6_DC_1  MC_CGM_1.MUX_6_DC_1.R         /* Clock Mux 6 Divider 1 Control Register */
#define MC_CGM_1_MUX_6_DIV_UPD_STAT MC_CGM_1.MUX_6_DIV_UPD_STAT.R /* Clock Mux 6 Divider Update Status Register */
#define MC_CGM_1_MUX_7_CSC   MC_CGM_1.MUX_7_CSC.R          /* Clock Mux 7 Select Control Register */
#define MC_CGM_1_MUX_7_CSS   MC_CGM_1.MUX_7_CSS.R          /* Clock Mux 7 Select Status Register */
#define MC_CGM_1_MUX_7_DC_0  MC_CGM_1.MUX_7_DC_0.R         /* Clock Mux 7 Divider 0 Control Register */
#define MC_CGM_1_MUX_7_DC_1  MC_CGM_1.MUX_7_DC_1.R         /* Clock Mux 7 Divider 1 Control Register */
#define MC_CGM_1_MUX_7_DC_2  MC_CGM_1.MUX_7_DC_2.R         /* Clock Mux 7 Divider 2 Control Register */
#define MC_CGM_1_MUX_7_DIV_UPD_STAT MC_CGM_1.MUX_7_DIV_UPD_STAT.R /* Clock Mux 7 Divider Update Status Register */
#define MC_CGM_1_MUX_8_CSC   MC_CGM_1.MUX_8_CSC.R          /* Clock Mux 8 Select Control Register */
#define MC_CGM_1_MUX_8_CSS   MC_CGM_1.MUX_8_CSS.R          /* Clock Mux 8 Select Status Register */
#define MC_CGM_1_MUX_8_DC_0  MC_CGM_1.MUX_8_DC_0.R         /* Clock Mux 8 Divider 0 Control Register */
#define MC_CGM_1_MUX_8_DC_1  MC_CGM_1.MUX_8_DC_1.R         /* Clock Mux 8 Divider 1 Control Register */
#define MC_CGM_1_MUX_8_DIV_UPD_STAT MC_CGM_1.MUX_8_DIV_UPD_STAT.R /* Clock Mux 8 Divider Update Status Register */
#define MC_CGM_1_MUX_9_CSC   MC_CGM_1.MUX_9_CSC.R          /* Clock Mux 9 Select Control Register */
#define MC_CGM_1_MUX_9_CSS   MC_CGM_1.MUX_9_CSS.R          /* Clock Mux 9 Select Status Register */
#define MC_CGM_1_MUX_9_DC_0  MC_CGM_1.MUX_9_DC_0.R         /* Clock Mux 9 Divider 0 Control Register */
#define MC_CGM_1_MUX_9_DC_1  MC_CGM_1.MUX_9_DC_1.R         /* Clock Mux 9 Divider 1 Control Register */
#define MC_CGM_1_MUX_9_DC_2  MC_CGM_1.MUX_9_DC_2.R         /* Clock Mux 9 Divider 2 Control Register */
#define MC_CGM_1_MUX_9_DIV_UPD_STAT MC_CGM_1.MUX_9_DIV_UPD_STAT.R /* Clock Mux 9 Divider Update Status Register */
#define MC_CGM_1_MUX_10_CSC  MC_CGM_1.MUX_10_CSC.R         /* Clock Mux 10 Select Control Register */
#define MC_CGM_1_MUX_10_CSS  MC_CGM_1.MUX_10_CSS.R         /* Clock Mux 10 Select Status Register */
#define MC_CGM_1_MUX_10_DC_0 MC_CGM_1.MUX_10_DC_0.R        /* Clock Mux 10 Divider 0 Control Register */
#define MC_CGM_1_MUX_10_DIV_UPD_STAT MC_CGM_1.MUX_10_DIV_UPD_STAT.R /* Clock Mux 10 Divider Update Status Register */
#define MC_CGM_1_MUX_11_CSC  MC_CGM_1.MUX_11_CSC.R         /* Clock Mux 11 Select Control Register */
#define MC_CGM_1_MUX_11_CSS  MC_CGM_1.MUX_11_CSS.R         /* Clock Mux 11 Select Status Register */
#define MC_CGM_1_MUX_11_DC_0 MC_CGM_1.MUX_11_DC_0.R        /* Clock Mux 11 Divider 0 Control Register */
#define MC_CGM_1_MUX_11_DIV_UPD_STAT MC_CGM_1.MUX_11_DIV_UPD_STAT.R /* Clock Mux 11 Divider Update Status Register */
#define MC_CGM_1_MUX_12_CSC  MC_CGM_1.MUX_12_CSC.R         /* Clock Mux 12 Select Control Register */
#define MC_CGM_1_MUX_12_CSS  MC_CGM_1.MUX_12_CSS.R         /* Clock Mux 12 Select Status Register */
#define MC_CGM_1_MUX_12_DC_0 MC_CGM_1.MUX_12_DC_0.R        /* Clock Mux 12 Divider 0 Control Register */
#define MC_CGM_1_MUX_12_DIV_UPD_STAT MC_CGM_1.MUX_12_DIV_UPD_STAT.R /* Clock Mux 12 Divider Update Status Register */
#define MC_CGM_1_MUX_13_CSC  MC_CGM_1.MUX_13_CSC.R         /* Clock Mux 13 Select Control Register */
#define MC_CGM_1_MUX_13_CSS  MC_CGM_1.MUX_13_CSS.R         /* Clock Mux 13 Select Status Register */
#define MC_CGM_1_MUX_14_CSC  MC_CGM_1.MUX_14_CSC.R         /* Clock Mux 14 Select Control Register */
#define MC_CGM_1_MUX_14_CSS  MC_CGM_1.MUX_14_CSS.R         /* Clock Mux 14 Select Status Register */
#define MC_CGM_1_MUX_14_DC_0 MC_CGM_1.MUX_14_DC_0.R        /* Clock Mux 14 Divider 0 Control Register */
#define MC_CGM_1_MUX_14_DIV_UPD_STAT MC_CGM_1.MUX_14_DIV_UPD_STAT.R /* Clock Mux 14 Divider Update Status Register */

/* MC_CGM_2 */
#define MC_CGM_2_PCFS_SDUR   MC_CGM_2.PCFS_SDUR.R          /* PCFS Step Duration */
#define MC_CGM_2_PCFS_DIVC15 MC_CGM_2.PCFS_DIVC15.R        /* PCFS Divider Change 15 Register */
#define MC_CGM_2_PCFS_DIVE15 MC_CGM_2.PCFS_DIVE15.R        /* PCFS Divider End 15 Register */
#define MC_CGM_2_PCFS_DIVS15 MC_CGM_2.PCFS_DIVS15.R        /* PCFS Divider Start 15 Register */
#define MC_CGM_2_MUX_0_CSC   MC_CGM_2.MUX_0_CSC.R          /* Clock Mux 0 Select Control Register */
#define MC_CGM_2_MUX_0_CSS   MC_CGM_2.MUX_0_CSS.R          /* Clock Mux 0 Select Status Register */
#define MC_CGM_2_MUX_1_CSC   MC_CGM_2.MUX_1_CSC.R          /* Clock Mux 1 Select Control Register */
#define MC_CGM_2_MUX_1_CSS   MC_CGM_2.MUX_1_CSS.R          /* Clock Mux 1 Select Status Register */
#define MC_CGM_2_MUX_1_DC_0  MC_CGM_2.MUX_1_DC_0.R         /* Clock Mux 1 Divider 0 Control Register */
#define MC_CGM_2_MUX_1_DIV_UPD_STAT MC_CGM_2.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */
#define MC_CGM_2_MUX_2_CSC   MC_CGM_2.MUX_2_CSC.R          /* Clock Mux 2 Select Control Register */
#define MC_CGM_2_MUX_2_CSS   MC_CGM_2.MUX_2_CSS.R          /* Clock Mux 2 Select Status Register */
#define MC_CGM_2_MUX_3_CSC   MC_CGM_2.MUX_3_CSC.R          /* Clock Mux 3 Select Control Register */
#define MC_CGM_2_MUX_3_CSS   MC_CGM_2.MUX_3_CSS.R          /* Clock Mux 3 Select Status Register */

/* MC_CGM_3 */
#define MC_CGM_3_PCFS_SDUR   MC_CGM_3.PCFS_SDUR.R          /* PCFS Step Duration */
#define MC_CGM_3_PCFS_DIVC12 MC_CGM_3.PCFS_DIVC12.R        /* PCFS Divider Change 12 Register */
#define MC_CGM_3_PCFS_DIVE12 MC_CGM_3.PCFS_DIVE12.R        /* PCFS Divider End 12 Register */
#define MC_CGM_3_PCFS_DIVS12 MC_CGM_3.PCFS_DIVS12.R        /* PCFS Divider Start 12 Register */
#define MC_CGM_3_MUX_0_CSC   MC_CGM_3.MUX_0_CSC.R          /* Clock Mux 0 Select Control Register */
#define MC_CGM_3_MUX_0_CSS   MC_CGM_3.MUX_0_CSS.R          /* Clock Mux 0 Select Status Register */
#define MC_CGM_3_MUX_1_CSC   MC_CGM_3.MUX_1_CSC.R          /* Clock Mux 1 Select Control Register */
#define MC_CGM_3_MUX_1_CSS   MC_CGM_3.MUX_1_CSS.R          /* Clock Mux 1 Select Status Register */
#define MC_CGM_3_MUX_1_DC_0  MC_CGM_3.MUX_1_DC_0.R         /* Clock Mux 1 Divider 0 Control Register */
#define MC_CGM_3_MUX_1_DIV_UPD_STAT MC_CGM_3.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */
#define MC_CGM_3_MUX_2_CSC   MC_CGM_3.MUX_2_CSC.R          /* Clock Mux 2 Select Control Register */
#define MC_CGM_3_MUX_2_CSS   MC_CGM_3.MUX_2_CSS.R          /* Clock Mux 2 Select Status Register */
#define MC_CGM_3_MUX_2_DC_0  MC_CGM_3.MUX_2_DC_0.R         /* Clock Mux 2 Divider 0 Control Register */
#define MC_CGM_3_MUX_2_DC_1  MC_CGM_3.MUX_2_DC_1.R         /* Clock Mux 2 Divider 1 Control Register */
#define MC_CGM_3_MUX_2_DIV_UPD_STAT MC_CGM_3.MUX_2_DIV_UPD_STAT.R /* Clock Mux 2 Divider Update Status Register */
#define MC_CGM_3_MUX_3_CSC   MC_CGM_3.MUX_3_CSC.R          /* Clock Mux 3 Select Control Register */
#define MC_CGM_3_MUX_3_CSS   MC_CGM_3.MUX_3_CSS.R          /* Clock Mux 3 Select Status Register */
#define MC_CGM_3_MUX_3_DC_0  MC_CGM_3.MUX_3_DC_0.R         /* Clock Mux 3 Divider 0 Control Register */
#define MC_CGM_3_MUX_3_DIV_UPD_STAT MC_CGM_3.MUX_3_DIV_UPD_STAT.R /* Clock Mux 3 Divider Update Status Register */
#define MC_CGM_3_MUX_4_CSC   MC_CGM_3.MUX_4_CSC.R          /* Clock Mux 4 Select Control Register */
#define MC_CGM_3_MUX_4_CSS   MC_CGM_3.MUX_4_CSS.R          /* Clock Mux 4 Select Status Register */
#define MC_CGM_3_MUX_4_DC_0  MC_CGM_3.MUX_4_DC_0.R         /* Clock Mux 4 Divider 0 Control Register */
#define MC_CGM_3_MUX_4_DIV_UPD_STAT MC_CGM_3.MUX_4_DIV_UPD_STAT.R /* Clock Mux 4 Divider Update Status Register */

/* MC_CGM_4 */
#define MC_CGM_4_PCFS_SDUR   MC_CGM_4.PCFS_SDUR.R          /* PCFS Step Duration */
#define MC_CGM_4_PCFS_DIVC15 MC_CGM_4.PCFS_DIVC15.R        /* PCFS Divider Change 15 Register */
#define MC_CGM_4_PCFS_DIVE15 MC_CGM_4.PCFS_DIVE15.R        /* PCFS Divider End 15 Register */
#define MC_CGM_4_PCFS_DIVS15 MC_CGM_4.PCFS_DIVS15.R        /* PCFS Divider Start 15 Register */
#define MC_CGM_4_MUX_0_CSC   MC_CGM_4.MUX_0_CSC.R          /* Clock Mux 0 Select Control Register */
#define MC_CGM_4_MUX_0_CSS   MC_CGM_4.MUX_0_CSS.R          /* Clock Mux 0 Select Status Register */
#define MC_CGM_4_MUX_1_CSC   MC_CGM_4.MUX_1_CSC.R          /* Clock Mux 1 Select Control Register */
#define MC_CGM_4_MUX_1_CSS   MC_CGM_4.MUX_1_CSS.R          /* Clock Mux 1 Select Status Register */
#define MC_CGM_4_MUX_1_DC_0  MC_CGM_4.MUX_1_DC_0.R         /* Clock Mux 1 Divider 0 Control Register */
#define MC_CGM_4_MUX_1_DC_1  MC_CGM_4.MUX_1_DC_1.R         /* Clock Mux 1 Divider 1 Control Register */
#define MC_CGM_4_MUX_1_DIV_UPD_STAT MC_CGM_4.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */
#define MC_CGM_4_MUX_2_CSC   MC_CGM_4.MUX_2_CSC.R          /* Clock Mux 2 Select Control Register */
#define MC_CGM_4_MUX_2_CSS   MC_CGM_4.MUX_2_CSS.R          /* Clock Mux 2 Select Status Register */
#define MC_CGM_4_MUX_2_DC_0  MC_CGM_4.MUX_2_DC_0.R         /* Clock Mux 2 Divider 0 Control Register */
#define MC_CGM_4_MUX_2_DC_1  MC_CGM_4.MUX_2_DC_1.R         /* Clock Mux 2 Divider 1 Control Register */
#define MC_CGM_4_MUX_2_DC_2  MC_CGM_4.MUX_2_DC_2.R         /* Clock Mux 2 Divider 2 Control Register */
#define MC_CGM_4_MUX_2_DC_3  MC_CGM_4.MUX_2_DC_3.R         /* Clock Mux 2 Divider 3 Control Register */
#define MC_CGM_4_MUX_2_DC_4  MC_CGM_4.MUX_2_DC_4.R         /* Clock Mux 2 Divider 4 Control Register */
#define MC_CGM_4_MUX_2_DC_5  MC_CGM_4.MUX_2_DC_5.R         /* Clock Mux 2 Divider 5 Control Register */
#define MC_CGM_4_MUX_2_DIV_UPD_STAT MC_CGM_4.MUX_2_DIV_UPD_STAT.R /* Clock Mux 2 Divider Update Status Register */
#define MC_CGM_4_MUX_3_CSC   MC_CGM_4.MUX_3_CSC.R          /* Clock Mux 3 Select Control Register */
#define MC_CGM_4_MUX_3_CSS   MC_CGM_4.MUX_3_CSS.R          /* Clock Mux 3 Select Status Register */
#define MC_CGM_4_MUX_3_DC_0  MC_CGM_4.MUX_3_DC_0.R         /* Clock Mux 3 Divider 0 Control Register */
#define MC_CGM_4_MUX_3_DC_1  MC_CGM_4.MUX_3_DC_1.R         /* Clock Mux 3 Divider 1 Control Register */
#define MC_CGM_4_MUX_3_DC_2  MC_CGM_4.MUX_3_DC_2.R         /* Clock Mux 3 Divider 2 Control Register */
#define MC_CGM_4_MUX_3_DC_3  MC_CGM_4.MUX_3_DC_3.R         /* Clock Mux 3 Divider 3 Control Register */
#define MC_CGM_4_MUX_3_DC_4  MC_CGM_4.MUX_3_DC_4.R         /* Clock Mux 3 Divider 4 Control Register */
#define MC_CGM_4_MUX_3_DC_5  MC_CGM_4.MUX_3_DC_5.R         /* Clock Mux 3 Divider 5 Control Register */
#define MC_CGM_4_MUX_3_DC_6  MC_CGM_4.MUX_3_DC_6.R         /* Clock Mux 3 Divider 6 Control Register */
#define MC_CGM_4_MUX_3_DC_7  MC_CGM_4.MUX_3_DC_7.R         /* Clock Mux 3 Divider 7 Control Register */
#define MC_CGM_4_MUX_3_DIV_UPD_STAT MC_CGM_4.MUX_3_DIV_UPD_STAT.R /* Clock Mux 3 Divider Update Status Register */
#define MC_CGM_4_MUX_4_CSC   MC_CGM_4.MUX_4_CSC.R          /* Clock Mux 4 Select Control Register */
#define MC_CGM_4_MUX_4_CSS   MC_CGM_4.MUX_4_CSS.R          /* Clock Mux 4 Select Status Register */
#define MC_CGM_4_MUX_5_CSC   MC_CGM_4.MUX_5_CSC.R          /* Clock Mux 5 Select Control Register */
#define MC_CGM_4_MUX_5_CSS   MC_CGM_4.MUX_5_CSS.R          /* Clock Mux 5 Select Status Register */
#define MC_CGM_4_MUX_6_CSC   MC_CGM_4.MUX_6_CSC.R          /* Clock Mux 6 Select Control Register */
#define MC_CGM_4_MUX_6_CSS   MC_CGM_4.MUX_6_CSS.R          /* Clock Mux 6 Select Status Register */
#define MC_CGM_4_MUX_6_DC_0  MC_CGM_4.MUX_6_DC_0.R         /* Clock Mux 6 Divider 0 Control Register */
#define MC_CGM_4_MUX_6_DIV_UPD_STAT MC_CGM_4.MUX_6_DIV_UPD_STAT.R /* Clock Mux 6 Divider Update Status Register */
#define MC_CGM_4_MUX_7_CSC   MC_CGM_4.MUX_7_CSC.R          /* Clock Mux 7 Select Control Register */
#define MC_CGM_4_MUX_7_CSS   MC_CGM_4.MUX_7_CSS.R          /* Clock Mux 7 Select Status Register */
#define MC_CGM_4_MUX_7_DC_0  MC_CGM_4.MUX_7_DC_0.R         /* Clock Mux 7 Divider 0 Control Register */
#define MC_CGM_4_MUX_7_DIV_UPD_STAT MC_CGM_4.MUX_7_DIV_UPD_STAT.R /* Clock Mux 7 Divider Update Status Register */
#define MC_CGM_4_MUX_8_CSC   MC_CGM_4.MUX_8_CSC.R          /* Clock Mux 8 Select Control Register */
#define MC_CGM_4_MUX_8_CSS   MC_CGM_4.MUX_8_CSS.R          /* Clock Mux 8 Select Status Register */
#define MC_CGM_4_MUX_8_DC_0  MC_CGM_4.MUX_8_DC_0.R         /* Clock Mux 8 Divider 0 Control Register */
#define MC_CGM_4_MUX_8_DIV_UPD_STAT MC_CGM_4.MUX_8_DIV_UPD_STAT.R /* Clock Mux 8 Divider Update Status Register */
#define MC_CGM_4_MUX_9_CSC   MC_CGM_4.MUX_9_CSC.R          /* Clock Mux 9 Select Control Register */
#define MC_CGM_4_MUX_9_CSS   MC_CGM_4.MUX_9_CSS.R          /* Clock Mux 9 Select Status Register */
#define MC_CGM_4_MUX_9_DC_0  MC_CGM_4.MUX_9_DC_0.R         /* Clock Mux 9 Divider 0 Control Register */
#define MC_CGM_4_MUX_9_DC_1  MC_CGM_4.MUX_9_DC_1.R         /* Clock Mux 9 Divider 1 Control Register */
#define MC_CGM_4_MUX_9_DIV_UPD_STAT MC_CGM_4.MUX_9_DIV_UPD_STAT.R /* Clock Mux 9 Divider Update Status Register */
#define MC_CGM_4_MUX_10_CSC  MC_CGM_4.MUX_10_CSC.R         /* Clock Mux 10 Select Control Register */
#define MC_CGM_4_MUX_10_CSS  MC_CGM_4.MUX_10_CSS.R         /* Clock Mux 10 Select Status Register */
#define MC_CGM_4_MUX_11_CSC  MC_CGM_4.MUX_11_CSC.R         /* Clock Mux 11 Select Control Register */
#define MC_CGM_4_MUX_11_CSS  MC_CGM_4.MUX_11_CSS.R         /* Clock Mux 11 Select Status Register */

/* MC_CGM_5 */
#define MC_CGM_5_PCFS_SDUR   MC_CGM_5.PCFS_SDUR.R          /* PCFS Step Duration */
#define MC_CGM_5_PCFS_DIVC15 MC_CGM_5.PCFS_DIVC15.R        /* PCFS Divider Change 15 Register */
#define MC_CGM_5_PCFS_DIVE15 MC_CGM_5.PCFS_DIVE15.R        /* PCFS Divider End 15 Register */
#define MC_CGM_5_PCFS_DIVS15 MC_CGM_5.PCFS_DIVS15.R        /* PCFS Divider Start 15 Register */
#define MC_CGM_5_MUX_0_CSC   MC_CGM_5.MUX_0_CSC.R          /* Clock Mux 0 Select Control Register */
#define MC_CGM_5_MUX_0_CSS   MC_CGM_5.MUX_0_CSS.R          /* Clock Mux 0 Select Status Register */
#define MC_CGM_5_MUX_0_DC_0  MC_CGM_5.MUX_0_DC_0.R         /* Clock Mux 0 Divider 0 Control Register */
#define MC_CGM_5_MUX_0_DIV_UPD_STAT MC_CGM_5.MUX_0_DIV_UPD_STAT.R /* Clock Mux 0 Divider Update Status Register */
#define MC_CGM_5_MUX_1_CSC   MC_CGM_5.MUX_1_CSC.R          /* Clock Mux 1 Select Control Register */
#define MC_CGM_5_MUX_1_CSS   MC_CGM_5.MUX_1_CSS.R          /* Clock Mux 1 Select Status Register */
#define MC_CGM_5_MUX_1_DC_0  MC_CGM_5.MUX_1_DC_0.R         /* Clock Mux 1 Divider 0 Control Register */
#define MC_CGM_5_MUX_1_DIV_UPD_STAT MC_CGM_5.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */
#define MC_CGM_5_MUX_2_CSC   MC_CGM_5.MUX_2_CSC.R          /* Clock Mux 2 Select Control Register */
#define MC_CGM_5_MUX_2_CSS   MC_CGM_5.MUX_2_CSS.R          /* Clock Mux 2 Select Status Register */
#define MC_CGM_5_MUX_2_DC_0  MC_CGM_5.MUX_2_DC_0.R         /* Clock Mux 2 Divider 0 Control Register */
#define MC_CGM_5_MUX_2_DIV_UPD_STAT MC_CGM_5.MUX_2_DIV_UPD_STAT.R /* Clock Mux 2 Divider Update Status Register */
#define MC_CGM_5_MUX_3_CSC   MC_CGM_5.MUX_3_CSC.R          /* Clock Mux 3 Select Control Register */
#define MC_CGM_5_MUX_3_CSS   MC_CGM_5.MUX_3_CSS.R          /* Clock Mux 3 Select Status Register */
#define MC_CGM_5_MUX_4_CSC   MC_CGM_5.MUX_4_CSC.R          /* Clock Mux 4 Select Control Register */
#define MC_CGM_5_MUX_4_CSS   MC_CGM_5.MUX_4_CSS.R          /* Clock Mux 4 Select Status Register */
#define MC_CGM_5_MUX_4_DC_0  MC_CGM_5.MUX_4_DC_0.R         /* Clock Mux 4 Divider 0 Control Register */
#define MC_CGM_5_MUX_4_DIV_UPD_STAT MC_CGM_5.MUX_4_DIV_UPD_STAT.R /* Clock Mux 4 Divider Update Status Register */
#define MC_CGM_5_MUX_5_CSC   MC_CGM_5.MUX_5_CSC.R          /* Clock Mux 5 Select Control Register */
#define MC_CGM_5_MUX_5_CSS   MC_CGM_5.MUX_5_CSS.R          /* Clock Mux 5 Select Status Register */
#define MC_CGM_5_MUX_5_DC_0  MC_CGM_5.MUX_5_DC_0.R         /* Clock Mux 5 Divider 0 Control Register */
#define MC_CGM_5_MUX_5_DC_1  MC_CGM_5.MUX_5_DC_1.R         /* Clock Mux 5 Divider 1 Control Register */
#define MC_CGM_5_MUX_5_DC_2  MC_CGM_5.MUX_5_DC_2.R         /* Clock Mux 5 Divider 2 Control Register */
#define MC_CGM_5_MUX_5_DIV_UPD_STAT MC_CGM_5.MUX_5_DIV_UPD_STAT.R /* Clock Mux 5 Divider Update Status Register */

/* MC_CGM_6 */
#define MC_CGM_6_PCFS_SDUR   MC_CGM_6.PCFS_SDUR.R          /* PCFS Step Duration */
#define MC_CGM_6_PCFS_DIVC40 MC_CGM_6.PCFS_DIVC40.R        /* PCFS Divider Change 40 Register */
#define MC_CGM_6_PCFS_DIVE40 MC_CGM_6.PCFS_DIVE40.R        /* PCFS Divider End 40 Register */
#define MC_CGM_6_PCFS_DIVS40 MC_CGM_6.PCFS_DIVS40.R        /* PCFS Divider Start 40 Register */
#define MC_CGM_6_MUX_0_CSC   MC_CGM_6.MUX_0_CSC.R          /* Clock Mux 0 Select Control Register */
#define MC_CGM_6_MUX_0_CSS   MC_CGM_6.MUX_0_CSS.R          /* Clock Mux 0 Select Status Register */
#define MC_CGM_6_MUX_0_DC_0  MC_CGM_6.MUX_0_DC_0.R         /* Clock Mux 0 Divider 0 Control Register */
#define MC_CGM_6_MUX_0_DC_1  MC_CGM_6.MUX_0_DC_1.R         /* Clock Mux 0 Divider 1 Control Register */
#define MC_CGM_6_MUX_0_DIV_UPD_STAT MC_CGM_6.MUX_0_DIV_UPD_STAT.R /* Clock Mux 0 Divider Update Status Register */
#define MC_CGM_6_MUX_1_CSC   MC_CGM_6.MUX_1_CSC.R          /* Clock Mux 1 Select Control Register */
#define MC_CGM_6_MUX_1_CSS   MC_CGM_6.MUX_1_CSS.R          /* Clock Mux 1 Select Status Register */
#define MC_CGM_6_MUX_1_DC_0  MC_CGM_6.MUX_1_DC_0.R         /* Clock Mux 1 Divider 0 Control Register */
#define MC_CGM_6_MUX_1_DIV_UPD_STAT MC_CGM_6.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */

/* MC_ME */
#define MC_ME_CTL_KEY        MC_ME.CTL_KEY.R               /* Control Key Register */
#define MC_ME_MODE_CONF      MC_ME.MODE_CONF.R             /* Mode Configuration Register */
#define MC_ME_MODE_UPD       MC_ME.MODE_UPD.R              /* Mode Update Register */
#define MC_ME_PRTN0_PCONF    MC_ME.PRTN0_PCONF.R           /* Partition 0 Process Configuration Register */
#define MC_ME_PRTN0_PUPD     MC_ME.PRTN0_PUPD.R            /* Partition 0 Process Update Register */
#define MC_ME_PRTN0_STAT     MC_ME.PRTN0_STAT.R            /* Partition 0 Status Register */
#define MC_ME_PRTN0_CORE0_PCONF MC_ME.PRTN0_CORE0_PCONF.R  /* Partition 0 Core 0 Process Configuration Register */
#define MC_ME_PRTN0_CORE0_PUPD MC_ME.PRTN0_CORE0_PUPD.R    /* Partition 0 Core 0 Process Update Register */
#define MC_ME_PRTN0_CORE0_STAT MC_ME.PRTN0_CORE0_STAT.R    /* Partition 0 Core 0 Status Register */
#define MC_ME_PRTN0_CORE0_ADDR MC_ME.PRTN0_CORE0_ADDR.R    /* Partition 0 Core 0 Address Register */
#define MC_ME_PRTN0_CORE1_PCONF MC_ME.PRTN0_CORE1_PCONF.R  /* Partition 0 Core 1 Process Configuration Register */
#define MC_ME_PRTN0_CORE1_PUPD MC_ME.PRTN0_CORE1_PUPD.R    /* Partition 0 Core 1 Process Update Register */
#define MC_ME_PRTN0_CORE1_STAT MC_ME.PRTN0_CORE1_STAT.R    /* Partition 0 Core 1 Status Register */
#define MC_ME_PRTN0_CORE1_ADDR MC_ME.PRTN0_CORE1_ADDR.R    /* Partition 0 Core 1 Address Register */
#define MC_ME_PRTN0_CORE2_PCONF MC_ME.PRTN0_CORE2_PCONF.R  /* Partition 0 Core 2 Process Configuration Register */
#define MC_ME_PRTN0_CORE2_PUPD MC_ME.PRTN0_CORE2_PUPD.R    /* Partition 0 Core 2 Process Update Register */
#define MC_ME_PRTN0_CORE2_STAT MC_ME.PRTN0_CORE2_STAT.R    /* Partition 0 Core 2 Status Register */
#define MC_ME_PRTN0_CORE2_ADDR MC_ME.PRTN0_CORE2_ADDR.R    /* Partition 0 Core 2 Address Register */
#define MC_ME_PRTN0_CORE3_PCONF MC_ME.PRTN0_CORE3_PCONF.R  /* Partition 0 Core 3 Process Configuration Register */
#define MC_ME_PRTN0_CORE3_PUPD MC_ME.PRTN0_CORE3_PUPD.R    /* Partition 0 Core 3 Process Update Register */
#define MC_ME_PRTN0_CORE3_STAT MC_ME.PRTN0_CORE3_STAT.R    /* Partition 0 Core 3 Status Register */
#define MC_ME_PRTN0_CORE3_ADDR MC_ME.PRTN0_CORE3_ADDR.R    /* Partition 0 Core 3 Address Register */
#define MC_ME_PRTN0_CORE4_PCONF MC_ME.PRTN0_CORE4_PCONF.R  /* Partition 0 Core 4 Process Configuration Register */
#define MC_ME_PRTN0_CORE4_PUPD MC_ME.PRTN0_CORE4_PUPD.R    /* Partition 0 Core 4 Process Update Register */
#define MC_ME_PRTN0_CORE4_STAT MC_ME.PRTN0_CORE4_STAT.R    /* Partition 0 Core 4 Status Register */
#define MC_ME_PRTN0_CORE4_ADDR MC_ME.PRTN0_CORE4_ADDR.R    /* Partition 0 Core 4 Address Register */
#define MC_ME_PRTN0_CORE5_PCONF MC_ME.PRTN0_CORE5_PCONF.R  /* Partition 0 Core 5 Process Configuration Register */
#define MC_ME_PRTN0_CORE5_PUPD MC_ME.PRTN0_CORE5_PUPD.R    /* Partition 0 Core 5 Process Update Register */
#define MC_ME_PRTN0_CORE5_STAT MC_ME.PRTN0_CORE5_STAT.R    /* Partition 0 Core 5 Status Register */
#define MC_ME_PRTN0_CORE5_ADDR MC_ME.PRTN0_CORE5_ADDR.R    /* Partition 0 Core 5 Address Register */
#define MC_ME_PRTN0_CORE6_PCONF MC_ME.PRTN0_CORE6_PCONF.R  /* Partition 0 Core 6 Process Configuration Register */
#define MC_ME_PRTN0_CORE6_PUPD MC_ME.PRTN0_CORE6_PUPD.R    /* Partition 0 Core 6 Process Update Register */
#define MC_ME_PRTN0_CORE6_STAT MC_ME.PRTN0_CORE6_STAT.R    /* Partition 0 Core 6 Status Register */
#define MC_ME_PRTN0_CORE6_ADDR MC_ME.PRTN0_CORE6_ADDR.R    /* Partition 0 Core 6 Address Register */
#define MC_ME_PRTN0_CORE7_PCONF MC_ME.PRTN0_CORE7_PCONF.R  /* Partition 0 Core 7 Process Configuration Register */
#define MC_ME_PRTN0_CORE7_PUPD MC_ME.PRTN0_CORE7_PUPD.R    /* Partition 0 Core 7 Process Update Register */
#define MC_ME_PRTN0_CORE7_STAT MC_ME.PRTN0_CORE7_STAT.R    /* Partition 0 Core 7 Status Register */
#define MC_ME_PRTN0_CORE7_ADDR MC_ME.PRTN0_CORE7_ADDR.R    /* Partition 0 Core 7 Address Register */
#define MC_ME_PRTN1_PCONF    MC_ME.PRTN1_PCONF.R           /* Partition 1 Process Configuration Register */
#define MC_ME_PRTN1_PUPD     MC_ME.PRTN1_PUPD.R            /* Partition 1 Process Update Register */
#define MC_ME_PRTN1_STAT     MC_ME.PRTN1_STAT.R            /* Partition 1 Status Register */
#define MC_ME_PRTN1_CORE0_PCONF MC_ME.PRTN1_CORE0_PCONF.R  /* Partition 1 Core 0 Process Configuration Register */
#define MC_ME_PRTN1_CORE0_PUPD MC_ME.PRTN1_CORE0_PUPD.R    /* Partition 1 Core 0 Process Update Register */
#define MC_ME_PRTN1_CORE0_STAT MC_ME.PRTN1_CORE0_STAT.R    /* Partition 1 Core 0 Status Register */
#define MC_ME_PRTN1_CORE0_ADDR MC_ME.PRTN1_CORE0_ADDR.R    /* Partition 1 Core 0 Address Register */
#define MC_ME_PRTN1_CORE1_PCONF MC_ME.PRTN1_CORE1_PCONF.R  /* Partition 1 Core 1 Process Configuration Register */
#define MC_ME_PRTN1_CORE1_PUPD MC_ME.PRTN1_CORE1_PUPD.R    /* Partition 1 Core 1 Process Update Register */
#define MC_ME_PRTN1_CORE1_STAT MC_ME.PRTN1_CORE1_STAT.R    /* Partition 1 Core 1 Status Register */
#define MC_ME_PRTN1_CORE1_ADDR MC_ME.PRTN1_CORE1_ADDR.R    /* Partition 1 Core 1 Address Register */
#define MC_ME_PRTN1_CORE2_PCONF MC_ME.PRTN1_CORE2_PCONF.R  /* Partition 1 Core 2 Process Configuration Register */
#define MC_ME_PRTN1_CORE2_PUPD MC_ME.PRTN1_CORE2_PUPD.R    /* Partition 1 Core 2 Process Update Register */
#define MC_ME_PRTN1_CORE2_STAT MC_ME.PRTN1_CORE2_STAT.R    /* Partition 1 Core 2 Status Register */
#define MC_ME_PRTN1_CORE2_ADDR MC_ME.PRTN1_CORE2_ADDR.R    /* Partition 1 Core 2 Address Register */
#define MC_ME_PRTN1_CORE3_PCONF MC_ME.PRTN1_CORE3_PCONF.R  /* Partition 1 Core 3 Process Configuration Register */
#define MC_ME_PRTN1_CORE3_PUPD MC_ME.PRTN1_CORE3_PUPD.R    /* Partition 1 Core 3 Process Update Register */
#define MC_ME_PRTN1_CORE3_STAT MC_ME.PRTN1_CORE3_STAT.R    /* Partition 1 Core 3 Status Register */
#define MC_ME_PRTN1_CORE3_ADDR MC_ME.PRTN1_CORE3_ADDR.R    /* Partition 1 Core 3 Address Register */
#define MC_ME_PRTN2_PCONF    MC_ME.PRTN2_PCONF.R           /* Partition 2 Process Configuration Register */
#define MC_ME_PRTN2_PUPD     MC_ME.PRTN2_PUPD.R            /* Partition 2 Process Update Register */
#define MC_ME_PRTN2_STAT     MC_ME.PRTN2_STAT.R            /* Partition 2 Status Register */
#define MC_ME_PRTN2_CORE0_PCONF MC_ME.PRTN2_CORE0_PCONF.R  /* Partition 2 Core 0 Process Configuration Register */
#define MC_ME_PRTN2_CORE0_PUPD MC_ME.PRTN2_CORE0_PUPD.R    /* Partition 2 Core 0 Process Update Register */
#define MC_ME_PRTN2_CORE0_STAT MC_ME.PRTN2_CORE0_STAT.R    /* Partition 2 Core 0 Status Register */
#define MC_ME_PRTN2_CORE0_ADDR MC_ME.PRTN2_CORE0_ADDR.R    /* Partition 2 Core 0 Address Register */
#define MC_ME_PRTN2_CORE1_PCONF MC_ME.PRTN2_CORE1_PCONF.R  /* Partition 2 Core 1 Process Configuration Register */
#define MC_ME_PRTN2_CORE1_PUPD MC_ME.PRTN2_CORE1_PUPD.R    /* Partition 2 Core 1 Process Update Register */
#define MC_ME_PRTN2_CORE1_STAT MC_ME.PRTN2_CORE1_STAT.R    /* Partition 2 Core 1 Status Register */
#define MC_ME_PRTN2_CORE1_ADDR MC_ME.PRTN2_CORE1_ADDR.R    /* Partition 2 Core 1 Address Register */
#define MC_ME_PRTN2_CORE2_PCONF MC_ME.PRTN2_CORE2_PCONF.R  /* Partition 2 Core 2 Process Configuration Register */
#define MC_ME_PRTN2_CORE2_PUPD MC_ME.PRTN2_CORE2_PUPD.R    /* Partition 2 Core 2 Process Update Register */
#define MC_ME_PRTN2_CORE2_STAT MC_ME.PRTN2_CORE2_STAT.R    /* Partition 2 Core 2 Status Register */
#define MC_ME_PRTN2_CORE2_ADDR MC_ME.PRTN2_CORE2_ADDR.R    /* Partition 2 Core 2 Address Register */
#define MC_ME_PRTN2_CORE3_PCONF MC_ME.PRTN2_CORE3_PCONF.R  /* Partition 2 Core 3 Process Configuration Register */
#define MC_ME_PRTN2_CORE3_PUPD MC_ME.PRTN2_CORE3_PUPD.R    /* Partition 2 Core 3 Process Update Register */
#define MC_ME_PRTN2_CORE3_STAT MC_ME.PRTN2_CORE3_STAT.R    /* Partition 2 Core 3 Status Register */
#define MC_ME_PRTN2_CORE3_ADDR MC_ME.PRTN2_CORE3_ADDR.R    /* Partition 2 Core 3 Address Register */

/* MC_RGM */
#define MC_RGM_DES           MC_RGM.DES.R                  /* Destructive Event Status Register */
#define MC_RGM_FES           MC_RGM.FES.R                  /* Functional /External Reset Status Register */
#define MC_RGM_FERD          MC_RGM.FERD.R                 /* Functional Event Reset Disable Register */
#define MC_RGM_FBRE          MC_RGM.FBRE.R                 /* Functional Bidirectional Reset Enable Register */
#define MC_RGM_FREC          MC_RGM.FREC.R                 /* Functional Reset Escalation Counter Register */
#define MC_RGM_FRET          MC_RGM.FRET.R                 /* Functional Reset Escalation Threshold Register */
#define MC_RGM_DRET          MC_RGM.DRET.R                 /* Destructive Reset Escalation Threshold Register */
#define MC_RGM_ERCTRL        MC_RGM.ERCTRL.R               /* External Reset Control Register */
#define MC_RGM_FRENTC        MC_RGM.FRENTC.R               /* Functional Reset Entry Timeout Control Register */
#define MC_RGM_PRST0_0       MC_RGM.PRST0_0.R              /* Peripheral Reset */
#define MC_RGM_PRST1_0       MC_RGM.PRST1_0.R              /* Peripheral Reset */
#define MC_RGM_PRST2_0       MC_RGM.PRST2_0.R              /* Peripheral Reset */
#define MC_RGM_PSTAT0_0      MC_RGM.PSTAT0_0.R             /* Peripheral Reset Status Register */
#define MC_RGM_PSTAT1_0      MC_RGM.PSTAT1_0.R             /* Peripheral Reset Status Register */
#define MC_RGM_PSTAT2_0      MC_RGM.PSTAT2_0.R             /* Peripheral Reset Status Register */

/* MDM_AP */
#define MDM_AP_STATUS        MDM_AP.STATUS.R               /* MDM_AP status */
#define MDM_AP_CONTROL       MDM_AP.CONTROL.R              /* MDM_AP control */
#define MDM_AP_STATUS2       MDM_AP.STATUS2.R              /* MDM_AP status 2 */
#define MDM_AP_CONTROL2      MDM_AP.CONTROL2.R             /* MDM_AP Control 2 */
#define MDM_AP_CONTROL3      MDM_AP.CONTROL3.R             /* MDM_AP Control 3 */
#define MDM_AP_RTU0_Core_0_IP_DBG MDM_AP.RTU0_CORE_0_IP_DBG.R /* RTU0 core 0 debug */
#define MDM_AP_RTU0_Core_1_IP_DBG MDM_AP.RTU0_CORE_1_IP_DBG.R /* RTU0 core 1 debug */
#define MDM_AP_RTU0_Core_2_IP_DBG MDM_AP.RTU0_CORE_2_IP_DBG.R /* RTU0 core 2 debug */
#define MDM_AP_RTU0_Core_3_IP_DBG MDM_AP.RTU0_CORE_3_IP_DBG.R /* RTU0 core 3 debug */
#define MDM_AP_RTU1_Core_0_IP_DBG MDM_AP.RTU1_CORE_0_IP_DBG.R /* RTU1 core 0 debug */
#define MDM_AP_RTU1_Core_1_IP_DBG MDM_AP.RTU1_CORE_1_IP_DBG.R /* RTU1 core 1 debug */
#define MDM_AP_RTU1_Core_2_IP_DBG MDM_AP.RTU1_CORE_2_IP_DBG.R /* RTU1 core 2 debug */
#define MDM_AP_RTU1_Core_3_IP_DBG MDM_AP.RTU1_CORE_3_IP_DBG.R /* RTU1 core 3 debug */
#define MDM_AP_SMU_Cortex_M33_IP_DBG MDM_AP.SMU_CORTEX_M33_IP_DBG.R /* Cortex-M33 Core Debug */
#define MDM_AP_CE_Cortex_M33_0_IP_DBG MDM_AP.CE_CORTEX_M33_0_IP_DBG.R /* CE Cortex-M33 Core 0 Debug */
#define MDM_AP_CE_Cortex_M33_1_IP_DBG MDM_AP.CE_CORTEX_M33_1_IP_DBG.R /* CE Cortex-M33 Core 1 Debug */
#define MDM_AP_RTU0_SWTMASK_IP_DBG MDM_AP.RTU0_SWTMASK_IP_DBG.R /* RTU0 SWT debug mask register */
#define MDM_AP_RTU1_SWTMASK_IP_DBG MDM_AP.RTU1_SWTMASK_IP_DBG.R /* RTU1 SWT debug mask register */
#define MDM_AP_RTU0_STMMASK_IP_DBG MDM_AP.RTU0_STMMASK_IP_DBG.R /* RTU0 STM debug mask register */
#define MDM_AP_RTU1_STMMASK_IP_DBG MDM_AP.RTU1_STMMASK_IP_DBG.R /* RTU1 STM debug mask register */
#define MDM_AP_DBGENCNTRL    MDM_AP.DBGENCNTRL.R           /* Debug enable control */
#define MDM_AP_DBGENCNTRL2   MDM_AP.DBGENCNTRL2.R          /* Debug enable control 2 */
#define MDM_AP_DBGENCNTRL3   MDM_AP.DBGENCNTRL3.R          /* Debug enable control 3 */
#define MDM_AP_DTS_ENABLE    MDM_AP.DTS_ENABLE.R           /* DTS output enable register */
#define MDM_AP_DTS_STARTUP   MDM_AP.DTS_STARTUP.R          /* DTS startup register */
#define MDM_AP_DTS_STARTUP_B MDM_AP.DTS_STARTUP_B.R        /* DTS startup register B */
#define MDM_AP_DTS_STARTUP_C MDM_AP.DTS_STARTUP_C.R        /* DTS startup register C */
#define MDM_AP_DTS_STARTUP_D MDM_AP.DTS_STARTUP_D.R        /* DTS startup register D */
#define MDM_AP_DTS_SEMAPHORE MDM_AP.DTS_SEMAPHORE.R        /* DTS SEMAPHORE register */
#define MDM_AP_DTS_SEMAPHORE_B MDM_AP.DTS_SEMAPHORE_B.R    /* DTS SEMAPHORE_B register */
#define MDM_AP_DTS_SEMAPHORE_C MDM_AP.DTS_SEMAPHORE_C.R    /* DTS SEMAPHORE_C register */
#define MDM_AP_DTS_SEMAPHORE_D MDM_AP.DTS_SEMAPHORE_D.R    /* DTS SEMAPHORE_D register */
#define MDM_AP_IDR           MDM_AP.IDR.R                  /* Identification Register */
#define MDM_AP_DEVARCH       MDM_AP.DEVARCH.R              /* CoreSight Device Architecture Register */
#define MDM_AP_DEVTYPE       MDM_AP.DEVTYPE.R              /* CoreSight Device Type Identifier Register */
#define MDM_AP_PIDR4         MDM_AP.PIDR4.R                /* CoreSight Peripheral Identification Register 4 */
#define MDM_AP_PIDR0         MDM_AP.PIDR0.R                /* CoreSight Peripheral Identification Register 0 */
#define MDM_AP_PIDR1         MDM_AP.PIDR1.R                /* CoreSight Peripheral Identification Register 1 */
#define MDM_AP_PIDR2         MDM_AP.PIDR2.R                /* CoreSight Peripheral Identification Register 2 */
#define MDM_AP_PIDR3         MDM_AP.PIDR3.R                /* CoreSight Peripheral Identification Register 3 */
#define MDM_AP_CIDR0         MDM_AP.CIDR0.R                /* CoreSight Component Identification Register 0 */
#define MDM_AP_CIDR1         MDM_AP.CIDR1.R                /* CoreSight Component Identification Register 1 */
#define MDM_AP_CIDR2         MDM_AP.CIDR2.R                /* CoreSight Component Identification Register 2 */
#define MDM_AP_CIDR3         MDM_AP.CIDR3.R                /* CoreSight Component Identification Register 3 */

/* MEW */
#define MEW_0_AXI_ECC_GLBL_CTRL MEW_0_AXI.ECC_GLBL_CTRL.R  /* AXI ECC Global Control */
#define MEW_0_AXI_ECC_MX_EPA MEW_0_AXI.ECC_MX_EPA.R        /* AXI ECC Maximum ECC Protected Address */
#define MEW_0_AXI_ECC_MN_EPA MEW_0_AXI.ECC_MN_EPA.R        /* AXI ECC Minimum ECC Protected Address */
#define MEW_0_AXI_ECC_LK_PTN MEW_0_AXI.ECC_LK_PTN.R        /* AXI ECC Lock Pattern */
#define MEW_0_AXI_ECC_ULK_PTN MEW_0_AXI.ECC_ULK_PTN.R      /* AXI ECC Unlock Pattern */
#define MEW_0_AXI_ECC_EERAR  MEW_0_AXI.ECC_EERAR.R         /* ECC Error Report Address */
#define MEW_0_AXI_ECC_EERDSR0 MEW_0_AXI.ECC_EERDSR[0].R    /* AXI ECC Error Report Data And Syndrome */
#define MEW_0_AXI_ECC_EERDSR1 MEW_0_AXI.ECC_EERDSR[1].R    /* AXI ECC Error Report Data And Syndrome */
#define MEW_0_AXI_ECC_EERDSR2 MEW_0_AXI.ECC_EERDSR[2].R    /* AXI ECC Error Report Data And Syndrome */
#define MEW_0_AXI_ECC_EERDSR3 MEW_0_AXI.ECC_EERDSR[3].R    /* AXI ECC Error Report Data And Syndrome */
#define MEW_0_AXI_ECC_ERR_IE MEW_0_AXI.ECC_ERR_IE.R        /* AXI ECC Error Interrupt Enable */
#define MEW_0_AXI_ECC_ERR_IN_STCLR MEW_0_AXI.ECC_ERR_IN_STCLR.R /* AXI ECC Error Interupt Status And Clear */
#define MEW_0_AXI_EDC_ERR_IE MEW_0_AXI.EDC_ERR_IE.R        /* AXI EDC Error Interrupt Enable */
#define MEW_0_AXI_EDC_ERR_IN_STCLR MEW_0_AXI.EDC_ERR_IN_STCLR.R /* AXI EDC Error Interrupt Status And Clear */
#define MEW_0_AXI_ECC_SHD_STAT_CTRL MEW_0_AXI.ECC_SHD_STAT_CTRL.R /* Shadow Control, RW Path Status And Status Clear */
#define MEW_0_AXI_ECC_CBL_UNCBL_BIT_EC MEW_0_AXI.ECC_CBL_UNCBL_BIT_EC.R /* Correctable And Uncorrectable Bit Error Counter */
#define MEW_0_AXI_ECC_CBL_UCBL_BEAT_EC MEW_0_AXI.ECC_CBL_UCBL_BEAT_EC.R /* Correctable And Uncorrectable Beat Error Counter */
#define MEW_0_AXI_ECC_DBG_CTRL MEW_0_AXI.ECC_DBG_CTRL.R    /* Debug And Debug Control */

#define MEW_1_AXI_ECC_GLBL_CTRL MEW_1_AXI.ECC_GLBL_CTRL.R  /* AXI ECC Global Control */
#define MEW_1_AXI_ECC_MX_EPA MEW_1_AXI.ECC_MX_EPA.R        /* AXI ECC Maximum ECC Protected Address */
#define MEW_1_AXI_ECC_MN_EPA MEW_1_AXI.ECC_MN_EPA.R        /* AXI ECC Minimum ECC Protected Address */
#define MEW_1_AXI_ECC_LK_PTN MEW_1_AXI.ECC_LK_PTN.R        /* AXI ECC Lock Pattern */
#define MEW_1_AXI_ECC_ULK_PTN MEW_1_AXI.ECC_ULK_PTN.R      /* AXI ECC Unlock Pattern */
#define MEW_1_AXI_ECC_EERAR  MEW_1_AXI.ECC_EERAR.R         /* ECC Error Report Address */
#define MEW_1_AXI_ECC_EERDSR0 MEW_1_AXI.ECC_EERDSR[0].R    /* AXI ECC Error Report Data And Syndrome */
#define MEW_1_AXI_ECC_EERDSR1 MEW_1_AXI.ECC_EERDSR[1].R    /* AXI ECC Error Report Data And Syndrome */
#define MEW_1_AXI_ECC_EERDSR2 MEW_1_AXI.ECC_EERDSR[2].R    /* AXI ECC Error Report Data And Syndrome */
#define MEW_1_AXI_ECC_EERDSR3 MEW_1_AXI.ECC_EERDSR[3].R    /* AXI ECC Error Report Data And Syndrome */
#define MEW_1_AXI_ECC_ERR_IE MEW_1_AXI.ECC_ERR_IE.R        /* AXI ECC Error Interrupt Enable */
#define MEW_1_AXI_ECC_ERR_IN_STCLR MEW_1_AXI.ECC_ERR_IN_STCLR.R /* AXI ECC Error Interupt Status And Clear */
#define MEW_1_AXI_EDC_ERR_IE MEW_1_AXI.EDC_ERR_IE.R        /* AXI EDC Error Interrupt Enable */
#define MEW_1_AXI_EDC_ERR_IN_STCLR MEW_1_AXI.EDC_ERR_IN_STCLR.R /* AXI EDC Error Interrupt Status And Clear */
#define MEW_1_AXI_ECC_SHD_STAT_CTRL MEW_1_AXI.ECC_SHD_STAT_CTRL.R /* Shadow Control, RW Path Status And Status Clear */
#define MEW_1_AXI_ECC_CBL_UNCBL_BIT_EC MEW_1_AXI.ECC_CBL_UNCBL_BIT_EC.R /* Correctable And Uncorrectable Bit Error Counter */
#define MEW_1_AXI_ECC_CBL_UCBL_BEAT_EC MEW_1_AXI.ECC_CBL_UCBL_BEAT_EC.R /* Correctable And Uncorrectable Beat Error Counter */
#define MEW_1_AXI_ECC_DBG_CTRL MEW_1_AXI.ECC_DBG_CTRL.R    /* Debug And Debug Control */

/* MRU */
#define CE_MRU_0_CH1_CFG0    CE_MRU_0.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_0_CH1_CFG1    CE_MRU_0.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_0_CH1_MBSTAT  CE_MRU_0.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_0_CH2_CFG0    CE_MRU_0.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_0_CH2_CFG1    CE_MRU_0.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_0_CH2_MBSTAT  CE_MRU_0.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_0_CH3_CFG0    CE_MRU_0.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_0_CH3_CFG1    CE_MRU_0.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_0_CH3_MBSTAT  CE_MRU_0.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_0_CH4_CFG0    CE_MRU_0.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_0_CH4_CFG1    CE_MRU_0.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_0_CH4_MBSTAT  CE_MRU_0.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_0_NOTIFY0     CE_MRU_0.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_0_NOTIFY1     CE_MRU_0.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_0_CH1_MB0     CE_MRU_0.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB1     CE_MRU_0.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB2     CE_MRU_0.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB3     CE_MRU_0.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB4     CE_MRU_0.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB5     CE_MRU_0.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB6     CE_MRU_0.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB7     CE_MRU_0.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB8     CE_MRU_0.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB9     CE_MRU_0.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB10    CE_MRU_0.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB11    CE_MRU_0.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB12    CE_MRU_0.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB13    CE_MRU_0.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB14    CE_MRU_0.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH1_MB15    CE_MRU_0.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB0     CE_MRU_0.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB1     CE_MRU_0.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB2     CE_MRU_0.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB3     CE_MRU_0.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB4     CE_MRU_0.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB5     CE_MRU_0.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB6     CE_MRU_0.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB7     CE_MRU_0.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB8     CE_MRU_0.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB9     CE_MRU_0.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB10    CE_MRU_0.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB11    CE_MRU_0.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB12    CE_MRU_0.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB13    CE_MRU_0.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB14    CE_MRU_0.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH2_MB15    CE_MRU_0.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB0     CE_MRU_0.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB1     CE_MRU_0.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB2     CE_MRU_0.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB3     CE_MRU_0.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB4     CE_MRU_0.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB5     CE_MRU_0.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB6     CE_MRU_0.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB7     CE_MRU_0.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB8     CE_MRU_0.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB9     CE_MRU_0.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB10    CE_MRU_0.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB11    CE_MRU_0.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB12    CE_MRU_0.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB13    CE_MRU_0.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB14    CE_MRU_0.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH3_MB15    CE_MRU_0.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB0     CE_MRU_0.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB1     CE_MRU_0.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB2     CE_MRU_0.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB3     CE_MRU_0.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB4     CE_MRU_0.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB5     CE_MRU_0.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB6     CE_MRU_0.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB7     CE_MRU_0.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB8     CE_MRU_0.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB9     CE_MRU_0.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB10    CE_MRU_0.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB11    CE_MRU_0.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB12    CE_MRU_0.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB13    CE_MRU_0.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB14    CE_MRU_0.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_0_CH4_MB15    CE_MRU_0.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_1_CH1_CFG0    CE_MRU_1.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_1_CH1_CFG1    CE_MRU_1.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_1_CH1_MBSTAT  CE_MRU_1.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_1_CH2_CFG0    CE_MRU_1.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_1_CH2_CFG1    CE_MRU_1.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_1_CH2_MBSTAT  CE_MRU_1.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_1_CH3_CFG0    CE_MRU_1.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_1_CH3_CFG1    CE_MRU_1.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_1_CH3_MBSTAT  CE_MRU_1.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_1_CH4_CFG0    CE_MRU_1.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_1_CH4_CFG1    CE_MRU_1.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_1_CH4_MBSTAT  CE_MRU_1.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_1_NOTIFY0     CE_MRU_1.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_1_NOTIFY1     CE_MRU_1.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_1_CH1_MB0     CE_MRU_1.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB1     CE_MRU_1.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB2     CE_MRU_1.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB3     CE_MRU_1.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB4     CE_MRU_1.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB5     CE_MRU_1.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB6     CE_MRU_1.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB7     CE_MRU_1.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB8     CE_MRU_1.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB9     CE_MRU_1.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB10    CE_MRU_1.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB11    CE_MRU_1.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB12    CE_MRU_1.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB13    CE_MRU_1.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB14    CE_MRU_1.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH1_MB15    CE_MRU_1.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB0     CE_MRU_1.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB1     CE_MRU_1.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB2     CE_MRU_1.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB3     CE_MRU_1.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB4     CE_MRU_1.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB5     CE_MRU_1.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB6     CE_MRU_1.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB7     CE_MRU_1.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB8     CE_MRU_1.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB9     CE_MRU_1.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB10    CE_MRU_1.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB11    CE_MRU_1.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB12    CE_MRU_1.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB13    CE_MRU_1.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB14    CE_MRU_1.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH2_MB15    CE_MRU_1.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB0     CE_MRU_1.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB1     CE_MRU_1.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB2     CE_MRU_1.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB3     CE_MRU_1.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB4     CE_MRU_1.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB5     CE_MRU_1.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB6     CE_MRU_1.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB7     CE_MRU_1.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB8     CE_MRU_1.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB9     CE_MRU_1.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB10    CE_MRU_1.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB11    CE_MRU_1.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB12    CE_MRU_1.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB13    CE_MRU_1.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB14    CE_MRU_1.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH3_MB15    CE_MRU_1.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB0     CE_MRU_1.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB1     CE_MRU_1.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB2     CE_MRU_1.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB3     CE_MRU_1.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB4     CE_MRU_1.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB5     CE_MRU_1.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB6     CE_MRU_1.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB7     CE_MRU_1.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB8     CE_MRU_1.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB9     CE_MRU_1.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB10    CE_MRU_1.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB11    CE_MRU_1.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB12    CE_MRU_1.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB13    CE_MRU_1.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB14    CE_MRU_1.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_1_CH4_MB15    CE_MRU_1.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_10_CH1_CFG0   CE_MRU_10.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_10_CH1_CFG1   CE_MRU_10.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_10_CH1_MBSTAT CE_MRU_10.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_10_CH2_CFG0   CE_MRU_10.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_10_CH2_CFG1   CE_MRU_10.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_10_CH2_MBSTAT CE_MRU_10.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_10_CH3_CFG0   CE_MRU_10.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_10_CH3_CFG1   CE_MRU_10.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_10_CH3_MBSTAT CE_MRU_10.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_10_CH4_CFG0   CE_MRU_10.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_10_CH4_CFG1   CE_MRU_10.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_10_CH4_MBSTAT CE_MRU_10.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_10_NOTIFY0    CE_MRU_10.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_10_NOTIFY1    CE_MRU_10.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_10_CH1_MB0    CE_MRU_10.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB1    CE_MRU_10.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB2    CE_MRU_10.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB3    CE_MRU_10.CH1_MB3.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB4    CE_MRU_10.CH1_MB4.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB5    CE_MRU_10.CH1_MB5.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB6    CE_MRU_10.CH1_MB6.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB7    CE_MRU_10.CH1_MB7.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB8    CE_MRU_10.CH1_MB8.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB9    CE_MRU_10.CH1_MB9.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB10   CE_MRU_10.CH1_MB10.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB11   CE_MRU_10.CH1_MB11.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB12   CE_MRU_10.CH1_MB12.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB13   CE_MRU_10.CH1_MB13.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB14   CE_MRU_10.CH1_MB14.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH1_MB15   CE_MRU_10.CH1_MB15.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB0    CE_MRU_10.CH2_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB1    CE_MRU_10.CH2_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB2    CE_MRU_10.CH2_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB3    CE_MRU_10.CH2_MB3.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB4    CE_MRU_10.CH2_MB4.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB5    CE_MRU_10.CH2_MB5.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB6    CE_MRU_10.CH2_MB6.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB7    CE_MRU_10.CH2_MB7.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB8    CE_MRU_10.CH2_MB8.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB9    CE_MRU_10.CH2_MB9.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB10   CE_MRU_10.CH2_MB10.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB11   CE_MRU_10.CH2_MB11.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB12   CE_MRU_10.CH2_MB12.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB13   CE_MRU_10.CH2_MB13.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB14   CE_MRU_10.CH2_MB14.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH2_MB15   CE_MRU_10.CH2_MB15.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB0    CE_MRU_10.CH3_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB1    CE_MRU_10.CH3_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB2    CE_MRU_10.CH3_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB3    CE_MRU_10.CH3_MB3.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB4    CE_MRU_10.CH3_MB4.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB5    CE_MRU_10.CH3_MB5.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB6    CE_MRU_10.CH3_MB6.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB7    CE_MRU_10.CH3_MB7.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB8    CE_MRU_10.CH3_MB8.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB9    CE_MRU_10.CH3_MB9.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB10   CE_MRU_10.CH3_MB10.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB11   CE_MRU_10.CH3_MB11.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB12   CE_MRU_10.CH3_MB12.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB13   CE_MRU_10.CH3_MB13.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB14   CE_MRU_10.CH3_MB14.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH3_MB15   CE_MRU_10.CH3_MB15.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB0    CE_MRU_10.CH4_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB1    CE_MRU_10.CH4_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB2    CE_MRU_10.CH4_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB3    CE_MRU_10.CH4_MB3.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB4    CE_MRU_10.CH4_MB4.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB5    CE_MRU_10.CH4_MB5.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB6    CE_MRU_10.CH4_MB6.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB7    CE_MRU_10.CH4_MB7.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB8    CE_MRU_10.CH4_MB8.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB9    CE_MRU_10.CH4_MB9.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB10   CE_MRU_10.CH4_MB10.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB11   CE_MRU_10.CH4_MB11.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB12   CE_MRU_10.CH4_MB12.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB13   CE_MRU_10.CH4_MB13.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB14   CE_MRU_10.CH4_MB14.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_10_CH4_MB15   CE_MRU_10.CH4_MB15.R          /* Channel (x) Mailbox (n) */

#define CE_MRU_11_CH1_CFG0   CE_MRU_11.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_11_CH1_CFG1   CE_MRU_11.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_11_CH1_MBSTAT CE_MRU_11.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_11_CH2_CFG0   CE_MRU_11.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_11_CH2_CFG1   CE_MRU_11.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_11_CH2_MBSTAT CE_MRU_11.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_11_CH3_CFG0   CE_MRU_11.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_11_CH3_CFG1   CE_MRU_11.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_11_CH3_MBSTAT CE_MRU_11.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_11_CH4_CFG0   CE_MRU_11.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_11_CH4_CFG1   CE_MRU_11.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_11_CH4_MBSTAT CE_MRU_11.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_11_NOTIFY0    CE_MRU_11.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_11_NOTIFY1    CE_MRU_11.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_11_CH1_MB0    CE_MRU_11.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB1    CE_MRU_11.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB2    CE_MRU_11.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB3    CE_MRU_11.CH1_MB3.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB4    CE_MRU_11.CH1_MB4.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB5    CE_MRU_11.CH1_MB5.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB6    CE_MRU_11.CH1_MB6.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB7    CE_MRU_11.CH1_MB7.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB8    CE_MRU_11.CH1_MB8.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB9    CE_MRU_11.CH1_MB9.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB10   CE_MRU_11.CH1_MB10.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB11   CE_MRU_11.CH1_MB11.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB12   CE_MRU_11.CH1_MB12.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB13   CE_MRU_11.CH1_MB13.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB14   CE_MRU_11.CH1_MB14.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH1_MB15   CE_MRU_11.CH1_MB15.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB0    CE_MRU_11.CH2_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB1    CE_MRU_11.CH2_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB2    CE_MRU_11.CH2_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB3    CE_MRU_11.CH2_MB3.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB4    CE_MRU_11.CH2_MB4.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB5    CE_MRU_11.CH2_MB5.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB6    CE_MRU_11.CH2_MB6.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB7    CE_MRU_11.CH2_MB7.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB8    CE_MRU_11.CH2_MB8.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB9    CE_MRU_11.CH2_MB9.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB10   CE_MRU_11.CH2_MB10.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB11   CE_MRU_11.CH2_MB11.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB12   CE_MRU_11.CH2_MB12.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB13   CE_MRU_11.CH2_MB13.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB14   CE_MRU_11.CH2_MB14.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH2_MB15   CE_MRU_11.CH2_MB15.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB0    CE_MRU_11.CH3_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB1    CE_MRU_11.CH3_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB2    CE_MRU_11.CH3_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB3    CE_MRU_11.CH3_MB3.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB4    CE_MRU_11.CH3_MB4.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB5    CE_MRU_11.CH3_MB5.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB6    CE_MRU_11.CH3_MB6.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB7    CE_MRU_11.CH3_MB7.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB8    CE_MRU_11.CH3_MB8.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB9    CE_MRU_11.CH3_MB9.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB10   CE_MRU_11.CH3_MB10.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB11   CE_MRU_11.CH3_MB11.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB12   CE_MRU_11.CH3_MB12.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB13   CE_MRU_11.CH3_MB13.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB14   CE_MRU_11.CH3_MB14.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH3_MB15   CE_MRU_11.CH3_MB15.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB0    CE_MRU_11.CH4_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB1    CE_MRU_11.CH4_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB2    CE_MRU_11.CH4_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB3    CE_MRU_11.CH4_MB3.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB4    CE_MRU_11.CH4_MB4.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB5    CE_MRU_11.CH4_MB5.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB6    CE_MRU_11.CH4_MB6.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB7    CE_MRU_11.CH4_MB7.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB8    CE_MRU_11.CH4_MB8.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB9    CE_MRU_11.CH4_MB9.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB10   CE_MRU_11.CH4_MB10.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB11   CE_MRU_11.CH4_MB11.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB12   CE_MRU_11.CH4_MB12.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB13   CE_MRU_11.CH4_MB13.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB14   CE_MRU_11.CH4_MB14.R          /* Channel (x) Mailbox (n) */
#define CE_MRU_11_CH4_MB15   CE_MRU_11.CH4_MB15.R          /* Channel (x) Mailbox (n) */

#define CE_MRU_12_CH1_CFG0   CE_MRU_12.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_12_CH1_CFG1   CE_MRU_12.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_12_CH1_MBSTAT CE_MRU_12.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_12_NOTIFY0    CE_MRU_12.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_12_NOTIFY1    CE_MRU_12.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_12_CH1_MB0    CE_MRU_12.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_12_CH1_MB1    CE_MRU_12.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_12_CH1_MB2    CE_MRU_12.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_12_CH1_MB3    CE_MRU_12.CH1_MB3.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_13_CH1_CFG0   CE_MRU_13.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_13_CH1_CFG1   CE_MRU_13.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_13_CH1_MBSTAT CE_MRU_13.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_13_NOTIFY0    CE_MRU_13.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_13_NOTIFY1    CE_MRU_13.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_13_CH1_MB0    CE_MRU_13.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_13_CH1_MB1    CE_MRU_13.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_13_CH1_MB2    CE_MRU_13.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_13_CH1_MB3    CE_MRU_13.CH1_MB3.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_14_CH1_CFG0   CE_MRU_14.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_14_CH1_CFG1   CE_MRU_14.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_14_CH1_MBSTAT CE_MRU_14.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_14_NOTIFY0    CE_MRU_14.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_14_NOTIFY1    CE_MRU_14.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_14_CH1_MB0    CE_MRU_14.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_14_CH1_MB1    CE_MRU_14.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_14_CH1_MB2    CE_MRU_14.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_14_CH1_MB3    CE_MRU_14.CH1_MB3.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_15_CH1_CFG0   CE_MRU_15.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_15_CH1_CFG1   CE_MRU_15.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_15_CH1_MBSTAT CE_MRU_15.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_15_NOTIFY0    CE_MRU_15.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_15_NOTIFY1    CE_MRU_15.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_15_CH1_MB0    CE_MRU_15.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_15_CH1_MB1    CE_MRU_15.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_15_CH1_MB2    CE_MRU_15.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_15_CH1_MB3    CE_MRU_15.CH1_MB3.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_16_CH1_CFG0   CE_MRU_16.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_16_CH1_CFG1   CE_MRU_16.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_16_CH1_MBSTAT CE_MRU_16.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_16_NOTIFY0    CE_MRU_16.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_16_NOTIFY1    CE_MRU_16.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_16_CH1_MB0    CE_MRU_16.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_16_CH1_MB1    CE_MRU_16.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_16_CH1_MB2    CE_MRU_16.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_16_CH1_MB3    CE_MRU_16.CH1_MB3.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_17_CH1_CFG0   CE_MRU_17.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_17_CH1_CFG1   CE_MRU_17.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_17_CH1_MBSTAT CE_MRU_17.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_17_NOTIFY0    CE_MRU_17.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_17_NOTIFY1    CE_MRU_17.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_17_CH1_MB0    CE_MRU_17.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_17_CH1_MB1    CE_MRU_17.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_17_CH1_MB2    CE_MRU_17.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_17_CH1_MB3    CE_MRU_17.CH1_MB3.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_18_CH1_CFG0   CE_MRU_18.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_18_CH1_CFG1   CE_MRU_18.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_18_CH1_MBSTAT CE_MRU_18.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_18_NOTIFY0    CE_MRU_18.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_18_NOTIFY1    CE_MRU_18.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_18_CH1_MB0    CE_MRU_18.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_18_CH1_MB1    CE_MRU_18.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_18_CH1_MB2    CE_MRU_18.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_18_CH1_MB3    CE_MRU_18.CH1_MB3.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_19_CH1_CFG0   CE_MRU_19.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_19_CH1_CFG1   CE_MRU_19.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_19_CH1_MBSTAT CE_MRU_19.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_19_NOTIFY0    CE_MRU_19.NOTIFY0.R           /* Notification 0 Status */
#define CE_MRU_19_NOTIFY1    CE_MRU_19.NOTIFY1.R           /* Notification 1 Status */
#define CE_MRU_19_CH1_MB0    CE_MRU_19.CH1_MB0.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_19_CH1_MB1    CE_MRU_19.CH1_MB1.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_19_CH1_MB2    CE_MRU_19.CH1_MB2.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_19_CH1_MB3    CE_MRU_19.CH1_MB3.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_2_CH1_CFG0    CE_MRU_2.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_2_CH1_CFG1    CE_MRU_2.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_2_CH1_MBSTAT  CE_MRU_2.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_2_CH2_CFG0    CE_MRU_2.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_2_CH2_CFG1    CE_MRU_2.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_2_CH2_MBSTAT  CE_MRU_2.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_2_CH3_CFG0    CE_MRU_2.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_2_CH3_CFG1    CE_MRU_2.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_2_CH3_MBSTAT  CE_MRU_2.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_2_CH4_CFG0    CE_MRU_2.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_2_CH4_CFG1    CE_MRU_2.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_2_CH4_MBSTAT  CE_MRU_2.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_2_NOTIFY0     CE_MRU_2.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_2_NOTIFY1     CE_MRU_2.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_2_CH1_MB0     CE_MRU_2.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB1     CE_MRU_2.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB2     CE_MRU_2.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB3     CE_MRU_2.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB4     CE_MRU_2.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB5     CE_MRU_2.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB6     CE_MRU_2.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB7     CE_MRU_2.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB8     CE_MRU_2.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB9     CE_MRU_2.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB10    CE_MRU_2.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB11    CE_MRU_2.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB12    CE_MRU_2.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB13    CE_MRU_2.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB14    CE_MRU_2.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH1_MB15    CE_MRU_2.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB0     CE_MRU_2.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB1     CE_MRU_2.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB2     CE_MRU_2.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB3     CE_MRU_2.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB4     CE_MRU_2.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB5     CE_MRU_2.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB6     CE_MRU_2.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB7     CE_MRU_2.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB8     CE_MRU_2.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB9     CE_MRU_2.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB10    CE_MRU_2.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB11    CE_MRU_2.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB12    CE_MRU_2.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB13    CE_MRU_2.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB14    CE_MRU_2.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH2_MB15    CE_MRU_2.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB0     CE_MRU_2.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB1     CE_MRU_2.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB2     CE_MRU_2.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB3     CE_MRU_2.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB4     CE_MRU_2.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB5     CE_MRU_2.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB6     CE_MRU_2.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB7     CE_MRU_2.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB8     CE_MRU_2.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB9     CE_MRU_2.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB10    CE_MRU_2.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB11    CE_MRU_2.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB12    CE_MRU_2.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB13    CE_MRU_2.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB14    CE_MRU_2.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH3_MB15    CE_MRU_2.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB0     CE_MRU_2.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB1     CE_MRU_2.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB2     CE_MRU_2.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB3     CE_MRU_2.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB4     CE_MRU_2.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB5     CE_MRU_2.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB6     CE_MRU_2.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB7     CE_MRU_2.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB8     CE_MRU_2.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB9     CE_MRU_2.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB10    CE_MRU_2.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB11    CE_MRU_2.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB12    CE_MRU_2.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB13    CE_MRU_2.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB14    CE_MRU_2.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_2_CH4_MB15    CE_MRU_2.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_3_CH1_CFG0    CE_MRU_3.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_3_CH1_CFG1    CE_MRU_3.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_3_CH1_MBSTAT  CE_MRU_3.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_3_CH2_CFG0    CE_MRU_3.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_3_CH2_CFG1    CE_MRU_3.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_3_CH2_MBSTAT  CE_MRU_3.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_3_CH3_CFG0    CE_MRU_3.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_3_CH3_CFG1    CE_MRU_3.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_3_CH3_MBSTAT  CE_MRU_3.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_3_CH4_CFG0    CE_MRU_3.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_3_CH4_CFG1    CE_MRU_3.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_3_CH4_MBSTAT  CE_MRU_3.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_3_NOTIFY0     CE_MRU_3.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_3_NOTIFY1     CE_MRU_3.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_3_CH1_MB0     CE_MRU_3.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB1     CE_MRU_3.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB2     CE_MRU_3.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB3     CE_MRU_3.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB4     CE_MRU_3.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB5     CE_MRU_3.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB6     CE_MRU_3.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB7     CE_MRU_3.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB8     CE_MRU_3.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB9     CE_MRU_3.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB10    CE_MRU_3.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB11    CE_MRU_3.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB12    CE_MRU_3.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB13    CE_MRU_3.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB14    CE_MRU_3.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH1_MB15    CE_MRU_3.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB0     CE_MRU_3.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB1     CE_MRU_3.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB2     CE_MRU_3.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB3     CE_MRU_3.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB4     CE_MRU_3.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB5     CE_MRU_3.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB6     CE_MRU_3.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB7     CE_MRU_3.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB8     CE_MRU_3.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB9     CE_MRU_3.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB10    CE_MRU_3.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB11    CE_MRU_3.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB12    CE_MRU_3.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB13    CE_MRU_3.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB14    CE_MRU_3.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH2_MB15    CE_MRU_3.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB0     CE_MRU_3.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB1     CE_MRU_3.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB2     CE_MRU_3.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB3     CE_MRU_3.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB4     CE_MRU_3.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB5     CE_MRU_3.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB6     CE_MRU_3.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB7     CE_MRU_3.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB8     CE_MRU_3.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB9     CE_MRU_3.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB10    CE_MRU_3.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB11    CE_MRU_3.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB12    CE_MRU_3.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB13    CE_MRU_3.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB14    CE_MRU_3.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH3_MB15    CE_MRU_3.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB0     CE_MRU_3.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB1     CE_MRU_3.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB2     CE_MRU_3.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB3     CE_MRU_3.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB4     CE_MRU_3.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB5     CE_MRU_3.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB6     CE_MRU_3.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB7     CE_MRU_3.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB8     CE_MRU_3.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB9     CE_MRU_3.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB10    CE_MRU_3.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB11    CE_MRU_3.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB12    CE_MRU_3.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB13    CE_MRU_3.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB14    CE_MRU_3.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_3_CH4_MB15    CE_MRU_3.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_4_CH1_CFG0    CE_MRU_4.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_4_CH1_CFG1    CE_MRU_4.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_4_CH1_MBSTAT  CE_MRU_4.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_4_CH2_CFG0    CE_MRU_4.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_4_CH2_CFG1    CE_MRU_4.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_4_CH2_MBSTAT  CE_MRU_4.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_4_CH3_CFG0    CE_MRU_4.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_4_CH3_CFG1    CE_MRU_4.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_4_CH3_MBSTAT  CE_MRU_4.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_4_CH4_CFG0    CE_MRU_4.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_4_CH4_CFG1    CE_MRU_4.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_4_CH4_MBSTAT  CE_MRU_4.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_4_NOTIFY0     CE_MRU_4.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_4_NOTIFY1     CE_MRU_4.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_4_CH1_MB0     CE_MRU_4.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB1     CE_MRU_4.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB2     CE_MRU_4.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB3     CE_MRU_4.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB4     CE_MRU_4.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB5     CE_MRU_4.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB6     CE_MRU_4.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB7     CE_MRU_4.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB8     CE_MRU_4.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB9     CE_MRU_4.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB10    CE_MRU_4.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB11    CE_MRU_4.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB12    CE_MRU_4.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB13    CE_MRU_4.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB14    CE_MRU_4.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH1_MB15    CE_MRU_4.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB0     CE_MRU_4.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB1     CE_MRU_4.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB2     CE_MRU_4.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB3     CE_MRU_4.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB4     CE_MRU_4.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB5     CE_MRU_4.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB6     CE_MRU_4.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB7     CE_MRU_4.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB8     CE_MRU_4.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB9     CE_MRU_4.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB10    CE_MRU_4.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB11    CE_MRU_4.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB12    CE_MRU_4.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB13    CE_MRU_4.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB14    CE_MRU_4.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH2_MB15    CE_MRU_4.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB0     CE_MRU_4.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB1     CE_MRU_4.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB2     CE_MRU_4.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB3     CE_MRU_4.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB4     CE_MRU_4.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB5     CE_MRU_4.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB6     CE_MRU_4.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB7     CE_MRU_4.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB8     CE_MRU_4.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB9     CE_MRU_4.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB10    CE_MRU_4.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB11    CE_MRU_4.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB12    CE_MRU_4.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB13    CE_MRU_4.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB14    CE_MRU_4.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH3_MB15    CE_MRU_4.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB0     CE_MRU_4.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB1     CE_MRU_4.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB2     CE_MRU_4.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB3     CE_MRU_4.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB4     CE_MRU_4.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB5     CE_MRU_4.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB6     CE_MRU_4.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB7     CE_MRU_4.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB8     CE_MRU_4.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB9     CE_MRU_4.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB10    CE_MRU_4.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB11    CE_MRU_4.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB12    CE_MRU_4.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB13    CE_MRU_4.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB14    CE_MRU_4.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_4_CH4_MB15    CE_MRU_4.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_5_CH1_CFG0    CE_MRU_5.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_5_CH1_CFG1    CE_MRU_5.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_5_CH1_MBSTAT  CE_MRU_5.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_5_CH2_CFG0    CE_MRU_5.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_5_CH2_CFG1    CE_MRU_5.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_5_CH2_MBSTAT  CE_MRU_5.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_5_CH3_CFG0    CE_MRU_5.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_5_CH3_CFG1    CE_MRU_5.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_5_CH3_MBSTAT  CE_MRU_5.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_5_CH4_CFG0    CE_MRU_5.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_5_CH4_CFG1    CE_MRU_5.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_5_CH4_MBSTAT  CE_MRU_5.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_5_NOTIFY0     CE_MRU_5.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_5_NOTIFY1     CE_MRU_5.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_5_CH1_MB0     CE_MRU_5.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB1     CE_MRU_5.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB2     CE_MRU_5.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB3     CE_MRU_5.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB4     CE_MRU_5.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB5     CE_MRU_5.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB6     CE_MRU_5.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB7     CE_MRU_5.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB8     CE_MRU_5.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB9     CE_MRU_5.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB10    CE_MRU_5.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB11    CE_MRU_5.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB12    CE_MRU_5.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB13    CE_MRU_5.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB14    CE_MRU_5.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH1_MB15    CE_MRU_5.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB0     CE_MRU_5.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB1     CE_MRU_5.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB2     CE_MRU_5.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB3     CE_MRU_5.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB4     CE_MRU_5.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB5     CE_MRU_5.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB6     CE_MRU_5.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB7     CE_MRU_5.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB8     CE_MRU_5.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB9     CE_MRU_5.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB10    CE_MRU_5.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB11    CE_MRU_5.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB12    CE_MRU_5.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB13    CE_MRU_5.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB14    CE_MRU_5.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH2_MB15    CE_MRU_5.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB0     CE_MRU_5.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB1     CE_MRU_5.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB2     CE_MRU_5.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB3     CE_MRU_5.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB4     CE_MRU_5.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB5     CE_MRU_5.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB6     CE_MRU_5.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB7     CE_MRU_5.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB8     CE_MRU_5.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB9     CE_MRU_5.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB10    CE_MRU_5.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB11    CE_MRU_5.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB12    CE_MRU_5.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB13    CE_MRU_5.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB14    CE_MRU_5.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH3_MB15    CE_MRU_5.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB0     CE_MRU_5.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB1     CE_MRU_5.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB2     CE_MRU_5.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB3     CE_MRU_5.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB4     CE_MRU_5.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB5     CE_MRU_5.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB6     CE_MRU_5.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB7     CE_MRU_5.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB8     CE_MRU_5.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB9     CE_MRU_5.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB10    CE_MRU_5.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB11    CE_MRU_5.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB12    CE_MRU_5.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB13    CE_MRU_5.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB14    CE_MRU_5.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_5_CH4_MB15    CE_MRU_5.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_6_CH1_CFG0    CE_MRU_6.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_6_CH1_CFG1    CE_MRU_6.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_6_CH1_MBSTAT  CE_MRU_6.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_6_CH2_CFG0    CE_MRU_6.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_6_CH2_CFG1    CE_MRU_6.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_6_CH2_MBSTAT  CE_MRU_6.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_6_CH3_CFG0    CE_MRU_6.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_6_CH3_CFG1    CE_MRU_6.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_6_CH3_MBSTAT  CE_MRU_6.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_6_CH4_CFG0    CE_MRU_6.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_6_CH4_CFG1    CE_MRU_6.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_6_CH4_MBSTAT  CE_MRU_6.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_6_NOTIFY0     CE_MRU_6.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_6_NOTIFY1     CE_MRU_6.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_6_CH1_MB0     CE_MRU_6.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB1     CE_MRU_6.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB2     CE_MRU_6.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB3     CE_MRU_6.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB4     CE_MRU_6.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB5     CE_MRU_6.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB6     CE_MRU_6.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB7     CE_MRU_6.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB8     CE_MRU_6.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB9     CE_MRU_6.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB10    CE_MRU_6.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB11    CE_MRU_6.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB12    CE_MRU_6.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB13    CE_MRU_6.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB14    CE_MRU_6.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH1_MB15    CE_MRU_6.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB0     CE_MRU_6.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB1     CE_MRU_6.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB2     CE_MRU_6.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB3     CE_MRU_6.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB4     CE_MRU_6.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB5     CE_MRU_6.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB6     CE_MRU_6.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB7     CE_MRU_6.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB8     CE_MRU_6.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB9     CE_MRU_6.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB10    CE_MRU_6.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB11    CE_MRU_6.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB12    CE_MRU_6.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB13    CE_MRU_6.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB14    CE_MRU_6.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH2_MB15    CE_MRU_6.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB0     CE_MRU_6.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB1     CE_MRU_6.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB2     CE_MRU_6.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB3     CE_MRU_6.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB4     CE_MRU_6.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB5     CE_MRU_6.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB6     CE_MRU_6.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB7     CE_MRU_6.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB8     CE_MRU_6.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB9     CE_MRU_6.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB10    CE_MRU_6.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB11    CE_MRU_6.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB12    CE_MRU_6.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB13    CE_MRU_6.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB14    CE_MRU_6.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH3_MB15    CE_MRU_6.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB0     CE_MRU_6.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB1     CE_MRU_6.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB2     CE_MRU_6.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB3     CE_MRU_6.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB4     CE_MRU_6.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB5     CE_MRU_6.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB6     CE_MRU_6.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB7     CE_MRU_6.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB8     CE_MRU_6.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB9     CE_MRU_6.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB10    CE_MRU_6.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB11    CE_MRU_6.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB12    CE_MRU_6.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB13    CE_MRU_6.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB14    CE_MRU_6.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_6_CH4_MB15    CE_MRU_6.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_7_CH1_CFG0    CE_MRU_7.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_7_CH1_CFG1    CE_MRU_7.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_7_CH1_MBSTAT  CE_MRU_7.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_7_CH2_CFG0    CE_MRU_7.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_7_CH2_CFG1    CE_MRU_7.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_7_CH2_MBSTAT  CE_MRU_7.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_7_CH3_CFG0    CE_MRU_7.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_7_CH3_CFG1    CE_MRU_7.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_7_CH3_MBSTAT  CE_MRU_7.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_7_CH4_CFG0    CE_MRU_7.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_7_CH4_CFG1    CE_MRU_7.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_7_CH4_MBSTAT  CE_MRU_7.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_7_NOTIFY0     CE_MRU_7.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_7_NOTIFY1     CE_MRU_7.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_7_CH1_MB0     CE_MRU_7.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB1     CE_MRU_7.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB2     CE_MRU_7.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB3     CE_MRU_7.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB4     CE_MRU_7.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB5     CE_MRU_7.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB6     CE_MRU_7.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB7     CE_MRU_7.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB8     CE_MRU_7.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB9     CE_MRU_7.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB10    CE_MRU_7.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB11    CE_MRU_7.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB12    CE_MRU_7.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB13    CE_MRU_7.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB14    CE_MRU_7.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH1_MB15    CE_MRU_7.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB0     CE_MRU_7.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB1     CE_MRU_7.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB2     CE_MRU_7.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB3     CE_MRU_7.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB4     CE_MRU_7.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB5     CE_MRU_7.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB6     CE_MRU_7.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB7     CE_MRU_7.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB8     CE_MRU_7.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB9     CE_MRU_7.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB10    CE_MRU_7.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB11    CE_MRU_7.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB12    CE_MRU_7.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB13    CE_MRU_7.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB14    CE_MRU_7.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH2_MB15    CE_MRU_7.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB0     CE_MRU_7.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB1     CE_MRU_7.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB2     CE_MRU_7.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB3     CE_MRU_7.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB4     CE_MRU_7.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB5     CE_MRU_7.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB6     CE_MRU_7.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB7     CE_MRU_7.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB8     CE_MRU_7.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB9     CE_MRU_7.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB10    CE_MRU_7.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB11    CE_MRU_7.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB12    CE_MRU_7.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB13    CE_MRU_7.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB14    CE_MRU_7.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH3_MB15    CE_MRU_7.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB0     CE_MRU_7.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB1     CE_MRU_7.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB2     CE_MRU_7.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB3     CE_MRU_7.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB4     CE_MRU_7.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB5     CE_MRU_7.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB6     CE_MRU_7.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB7     CE_MRU_7.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB8     CE_MRU_7.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB9     CE_MRU_7.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB10    CE_MRU_7.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB11    CE_MRU_7.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB12    CE_MRU_7.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB13    CE_MRU_7.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB14    CE_MRU_7.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_7_CH4_MB15    CE_MRU_7.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_8_CH1_CFG0    CE_MRU_8.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_8_CH1_CFG1    CE_MRU_8.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_8_CH1_MBSTAT  CE_MRU_8.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_8_CH2_CFG0    CE_MRU_8.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_8_CH2_CFG1    CE_MRU_8.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_8_CH2_MBSTAT  CE_MRU_8.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_8_CH3_CFG0    CE_MRU_8.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_8_CH3_CFG1    CE_MRU_8.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_8_CH3_MBSTAT  CE_MRU_8.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_8_CH4_CFG0    CE_MRU_8.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_8_CH4_CFG1    CE_MRU_8.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_8_CH4_MBSTAT  CE_MRU_8.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_8_NOTIFY0     CE_MRU_8.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_8_NOTIFY1     CE_MRU_8.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_8_CH1_MB0     CE_MRU_8.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB1     CE_MRU_8.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB2     CE_MRU_8.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB3     CE_MRU_8.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB4     CE_MRU_8.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB5     CE_MRU_8.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB6     CE_MRU_8.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB7     CE_MRU_8.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB8     CE_MRU_8.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB9     CE_MRU_8.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB10    CE_MRU_8.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB11    CE_MRU_8.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB12    CE_MRU_8.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB13    CE_MRU_8.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB14    CE_MRU_8.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH1_MB15    CE_MRU_8.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB0     CE_MRU_8.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB1     CE_MRU_8.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB2     CE_MRU_8.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB3     CE_MRU_8.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB4     CE_MRU_8.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB5     CE_MRU_8.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB6     CE_MRU_8.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB7     CE_MRU_8.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB8     CE_MRU_8.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB9     CE_MRU_8.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB10    CE_MRU_8.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB11    CE_MRU_8.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB12    CE_MRU_8.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB13    CE_MRU_8.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB14    CE_MRU_8.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH2_MB15    CE_MRU_8.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB0     CE_MRU_8.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB1     CE_MRU_8.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB2     CE_MRU_8.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB3     CE_MRU_8.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB4     CE_MRU_8.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB5     CE_MRU_8.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB6     CE_MRU_8.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB7     CE_MRU_8.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB8     CE_MRU_8.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB9     CE_MRU_8.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB10    CE_MRU_8.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB11    CE_MRU_8.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB12    CE_MRU_8.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB13    CE_MRU_8.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB14    CE_MRU_8.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH3_MB15    CE_MRU_8.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB0     CE_MRU_8.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB1     CE_MRU_8.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB2     CE_MRU_8.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB3     CE_MRU_8.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB4     CE_MRU_8.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB5     CE_MRU_8.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB6     CE_MRU_8.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB7     CE_MRU_8.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB8     CE_MRU_8.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB9     CE_MRU_8.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB10    CE_MRU_8.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB11    CE_MRU_8.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB12    CE_MRU_8.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB13    CE_MRU_8.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB14    CE_MRU_8.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_8_CH4_MB15    CE_MRU_8.CH4_MB15.R           /* Channel (x) Mailbox (n) */

#define CE_MRU_9_CH1_CFG0    CE_MRU_9.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_9_CH1_CFG1    CE_MRU_9.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_9_CH1_MBSTAT  CE_MRU_9.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_9_CH2_CFG0    CE_MRU_9.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_9_CH2_CFG1    CE_MRU_9.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_9_CH2_MBSTAT  CE_MRU_9.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_9_CH3_CFG0    CE_MRU_9.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_9_CH3_CFG1    CE_MRU_9.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_9_CH3_MBSTAT  CE_MRU_9.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_9_CH4_CFG0    CE_MRU_9.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define CE_MRU_9_CH4_CFG1    CE_MRU_9.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define CE_MRU_9_CH4_MBSTAT  CE_MRU_9.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define CE_MRU_9_NOTIFY0     CE_MRU_9.NOTIFY0.R            /* Notification 0 Status */
#define CE_MRU_9_NOTIFY1     CE_MRU_9.NOTIFY1.R            /* Notification 1 Status */
#define CE_MRU_9_CH1_MB0     CE_MRU_9.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB1     CE_MRU_9.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB2     CE_MRU_9.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB3     CE_MRU_9.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB4     CE_MRU_9.CH1_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB5     CE_MRU_9.CH1_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB6     CE_MRU_9.CH1_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB7     CE_MRU_9.CH1_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB8     CE_MRU_9.CH1_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB9     CE_MRU_9.CH1_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB10    CE_MRU_9.CH1_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB11    CE_MRU_9.CH1_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB12    CE_MRU_9.CH1_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB13    CE_MRU_9.CH1_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB14    CE_MRU_9.CH1_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH1_MB15    CE_MRU_9.CH1_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB0     CE_MRU_9.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB1     CE_MRU_9.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB2     CE_MRU_9.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB3     CE_MRU_9.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB4     CE_MRU_9.CH2_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB5     CE_MRU_9.CH2_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB6     CE_MRU_9.CH2_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB7     CE_MRU_9.CH2_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB8     CE_MRU_9.CH2_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB9     CE_MRU_9.CH2_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB10    CE_MRU_9.CH2_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB11    CE_MRU_9.CH2_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB12    CE_MRU_9.CH2_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB13    CE_MRU_9.CH2_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB14    CE_MRU_9.CH2_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH2_MB15    CE_MRU_9.CH2_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB0     CE_MRU_9.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB1     CE_MRU_9.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB2     CE_MRU_9.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB3     CE_MRU_9.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB4     CE_MRU_9.CH3_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB5     CE_MRU_9.CH3_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB6     CE_MRU_9.CH3_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB7     CE_MRU_9.CH3_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB8     CE_MRU_9.CH3_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB9     CE_MRU_9.CH3_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB10    CE_MRU_9.CH3_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB11    CE_MRU_9.CH3_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB12    CE_MRU_9.CH3_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB13    CE_MRU_9.CH3_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB14    CE_MRU_9.CH3_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH3_MB15    CE_MRU_9.CH3_MB15.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB0     CE_MRU_9.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB1     CE_MRU_9.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB2     CE_MRU_9.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB3     CE_MRU_9.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB4     CE_MRU_9.CH4_MB4.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB5     CE_MRU_9.CH4_MB5.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB6     CE_MRU_9.CH4_MB6.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB7     CE_MRU_9.CH4_MB7.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB8     CE_MRU_9.CH4_MB8.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB9     CE_MRU_9.CH4_MB9.R            /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB10    CE_MRU_9.CH4_MB10.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB11    CE_MRU_9.CH4_MB11.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB12    CE_MRU_9.CH4_MB12.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB13    CE_MRU_9.CH4_MB13.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB14    CE_MRU_9.CH4_MB14.R           /* Channel (x) Mailbox (n) */
#define CE_MRU_9_CH4_MB15    CE_MRU_9.CH4_MB15.R           /* Channel (x) Mailbox (n) */

/* MSCM */
#define MSCM_CPXTYPE         MSCM.CPXTYPE.R                /* Core Processor x Type */
#define MSCM_CPXNUM          MSCM.CPXNUM.R                 /* Core Processor x Number */
#define MSCM_CPXREV          MSCM.CPXREV.R                 /* Core Processor x Revision */
#define MSCM_CPXCFG0         MSCM.CPXCFG0.R                /* Core Processor x Configuration 0 */
#define MSCM_CPXCFG1         MSCM.CPXCFG1.R                /* Core or Cluster Processor x Configuration 1 */
#define MSCM_CPXCFG2         MSCM.CPXCFG2.R                /* Core Processor x Configuration 2 */
#define MSCM_CPXCFG3         MSCM.CPXCFG3.R                /* Core Processor x Configuration 3 */
#define MSCM_CP0TYPE         MSCM.CP0TYPE.R                /* Cluster Processor 0 Type */
#define MSCM_CP0NUM          MSCM.CP0NUM.R                 /* Cluster Processor 0 Number */
#define MSCM_CP0REV          MSCM.CP0REV.R                 /* Core Processor 0 Revision */
#define MSCM_CP0CFG0         MSCM.CP0CFG0.R                /* Core Processor 0 Configuration 0 */
#define MSCM_CP0CFG1         MSCM.CP0CFG1.R                /* Cluster Processor 0 Configuration 1 */
#define MSCM_CP0CFG2         MSCM.CP0CFG2.R                /* Core Processor 0 Configuration 2 */
#define MSCM_CP0CFG3         MSCM.CP0CFG3.R                /* Core Processor 0 Configuration 3 */
#define MSCM_CP1TYPE         MSCM.CP1TYPE.R                /* Cluster Processor 1 Type */
#define MSCM_CP1NUM          MSCM.CP1NUM.R                 /* Cluster Processor 1 Number */
#define MSCM_CP1REV          MSCM.CP1REV.R                 /* Core Processor 1 Revision */
#define MSCM_CP1CFG0         MSCM.CP1CFG0.R                /* Core Processor 1 Configuration 0 */
#define MSCM_CP1CFG1         MSCM.CP1CFG1.R                /* Cluster Processor 1 Configuration 1 */
#define MSCM_CP1CFG2         MSCM.CP1CFG2.R                /* Core Processor 1 Configuration 2 */
#define MSCM_CP1CFG3         MSCM.CP1CFG3.R                /* Core Processor 1 Configuration 3 */
#define MSCM_CP2TYPE         MSCM.CP2TYPE.R                /* Core Processor 2 Type */
#define MSCM_CP2NUM          MSCM.CP2NUM.R                 /* Core Processor 2 Number */
#define MSCM_CP2REV          MSCM.CP2REV.R                 /* Core Processor 2 Revision */
#define MSCM_CP2CFG0         MSCM.CP2CFG0.R                /* Core Processor 2 Configuration 0 */
#define MSCM_CP2CFG1         MSCM.CP2CFG1.R                /* Core Processor 2 Configuration 1 */
#define MSCM_CP2CFG2         MSCM.CP2CFG2.R                /* Core Processor 2 Configuration 2 */
#define MSCM_CP2CFG3         MSCM.CP2CFG3.R                /* Core Processor 2 Configuration 3 */
#define MSCM_CP3TYPE         MSCM.CP3TYPE.R                /* Core Processor 3 Type */
#define MSCM_CP3NUM          MSCM.CP3NUM.R                 /* Core Processor 3 Number */
#define MSCM_CP3REV          MSCM.CP3REV.R                 /* Core Processor 3 Revision */
#define MSCM_CP3CFG0         MSCM.CP3CFG0.R                /* Core Processor 3 Configuration 0 */
#define MSCM_CP3CFG1         MSCM.CP3CFG1.R                /* Core Processor 3 Configuration 1 */
#define MSCM_CP3CFG2         MSCM.CP3CFG2.R                /* Core Processor 3 Configuration 2 */
#define MSCM_CP3CFG3         MSCM.CP3CFG3.R                /* Core Processor 3 Configuration 3 */
#define MSCM_IRSPRC0         MSCM.IRSPRC[0].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC1         MSCM.IRSPRC[1].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC2         MSCM.IRSPRC[2].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC3         MSCM.IRSPRC[3].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC4         MSCM.IRSPRC[4].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC5         MSCM.IRSPRC[5].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC6         MSCM.IRSPRC[6].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC7         MSCM.IRSPRC[7].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC8         MSCM.IRSPRC[8].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC9         MSCM.IRSPRC[9].R              /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC10        MSCM.IRSPRC[10].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC11        MSCM.IRSPRC[11].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC12        MSCM.IRSPRC[12].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC13        MSCM.IRSPRC[13].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC14        MSCM.IRSPRC[14].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC15        MSCM.IRSPRC[15].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC16        MSCM.IRSPRC[16].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC17        MSCM.IRSPRC[17].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC18        MSCM.IRSPRC[18].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC19        MSCM.IRSPRC[19].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC20        MSCM.IRSPRC[20].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC21        MSCM.IRSPRC[21].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC22        MSCM.IRSPRC[22].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC23        MSCM.IRSPRC[23].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC24        MSCM.IRSPRC[24].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC25        MSCM.IRSPRC[25].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC26        MSCM.IRSPRC[26].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC27        MSCM.IRSPRC[27].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC28        MSCM.IRSPRC[28].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC29        MSCM.IRSPRC[29].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC30        MSCM.IRSPRC[30].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC31        MSCM.IRSPRC[31].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC32        MSCM.IRSPRC[32].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC33        MSCM.IRSPRC[33].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC34        MSCM.IRSPRC[34].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC35        MSCM.IRSPRC[35].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC36        MSCM.IRSPRC[36].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC37        MSCM.IRSPRC[37].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC38        MSCM.IRSPRC[38].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC39        MSCM.IRSPRC[39].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC40        MSCM.IRSPRC[40].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC41        MSCM.IRSPRC[41].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC42        MSCM.IRSPRC[42].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC43        MSCM.IRSPRC[43].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC44        MSCM.IRSPRC[44].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC45        MSCM.IRSPRC[45].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC46        MSCM.IRSPRC[46].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC47        MSCM.IRSPRC[47].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC48        MSCM.IRSPRC[48].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC49        MSCM.IRSPRC[49].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC50        MSCM.IRSPRC[50].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC51        MSCM.IRSPRC[51].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC52        MSCM.IRSPRC[52].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC53        MSCM.IRSPRC[53].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC54        MSCM.IRSPRC[54].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC55        MSCM.IRSPRC[55].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC56        MSCM.IRSPRC[56].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC57        MSCM.IRSPRC[57].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC58        MSCM.IRSPRC[58].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC59        MSCM.IRSPRC[59].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC60        MSCM.IRSPRC[60].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC61        MSCM.IRSPRC[61].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC62        MSCM.IRSPRC[62].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC63        MSCM.IRSPRC[63].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC64        MSCM.IRSPRC[64].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC65        MSCM.IRSPRC[65].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC66        MSCM.IRSPRC[66].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC67        MSCM.IRSPRC[67].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC68        MSCM.IRSPRC[68].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC69        MSCM.IRSPRC[69].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC70        MSCM.IRSPRC[70].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC71        MSCM.IRSPRC[71].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC72        MSCM.IRSPRC[72].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC73        MSCM.IRSPRC[73].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC74        MSCM.IRSPRC[74].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC75        MSCM.IRSPRC[75].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC76        MSCM.IRSPRC[76].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC77        MSCM.IRSPRC[77].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC78        MSCM.IRSPRC[78].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC79        MSCM.IRSPRC[79].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC80        MSCM.IRSPRC[80].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC81        MSCM.IRSPRC[81].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC82        MSCM.IRSPRC[82].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC83        MSCM.IRSPRC[83].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC84        MSCM.IRSPRC[84].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC85        MSCM.IRSPRC[85].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC86        MSCM.IRSPRC[86].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC87        MSCM.IRSPRC[87].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC88        MSCM.IRSPRC[88].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC89        MSCM.IRSPRC[89].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC90        MSCM.IRSPRC[90].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC91        MSCM.IRSPRC[91].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC92        MSCM.IRSPRC[92].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC93        MSCM.IRSPRC[93].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC94        MSCM.IRSPRC[94].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC95        MSCM.IRSPRC[95].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC96        MSCM.IRSPRC[96].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC97        MSCM.IRSPRC[97].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC98        MSCM.IRSPRC[98].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC99        MSCM.IRSPRC[99].R             /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC100       MSCM.IRSPRC[100].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC101       MSCM.IRSPRC[101].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC102       MSCM.IRSPRC[102].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC103       MSCM.IRSPRC[103].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC104       MSCM.IRSPRC[104].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC105       MSCM.IRSPRC[105].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC106       MSCM.IRSPRC[106].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC107       MSCM.IRSPRC[107].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC108       MSCM.IRSPRC[108].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC109       MSCM.IRSPRC[109].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC110       MSCM.IRSPRC[110].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC111       MSCM.IRSPRC[111].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC112       MSCM.IRSPRC[112].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC113       MSCM.IRSPRC[113].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC114       MSCM.IRSPRC[114].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC115       MSCM.IRSPRC[115].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC116       MSCM.IRSPRC[116].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC117       MSCM.IRSPRC[117].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC118       MSCM.IRSPRC[118].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC119       MSCM.IRSPRC[119].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC120       MSCM.IRSPRC[120].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC121       MSCM.IRSPRC[121].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC122       MSCM.IRSPRC[122].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC123       MSCM.IRSPRC[123].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC124       MSCM.IRSPRC[124].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC125       MSCM.IRSPRC[125].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC126       MSCM.IRSPRC[126].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC127       MSCM.IRSPRC[127].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC128       MSCM.IRSPRC[128].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC129       MSCM.IRSPRC[129].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC130       MSCM.IRSPRC[130].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC131       MSCM.IRSPRC[131].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC132       MSCM.IRSPRC[132].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC133       MSCM.IRSPRC[133].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC134       MSCM.IRSPRC[134].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC135       MSCM.IRSPRC[135].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC136       MSCM.IRSPRC[136].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC137       MSCM.IRSPRC[137].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC138       MSCM.IRSPRC[138].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC139       MSCM.IRSPRC[139].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC140       MSCM.IRSPRC[140].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC141       MSCM.IRSPRC[141].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC142       MSCM.IRSPRC[142].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC143       MSCM.IRSPRC[143].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC144       MSCM.IRSPRC[144].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC145       MSCM.IRSPRC[145].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC146       MSCM.IRSPRC[146].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC147       MSCM.IRSPRC[147].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC148       MSCM.IRSPRC[148].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC149       MSCM.IRSPRC[149].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC150       MSCM.IRSPRC[150].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC151       MSCM.IRSPRC[151].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC152       MSCM.IRSPRC[152].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC153       MSCM.IRSPRC[153].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC154       MSCM.IRSPRC[154].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC155       MSCM.IRSPRC[155].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC156       MSCM.IRSPRC[156].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC157       MSCM.IRSPRC[157].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC158       MSCM.IRSPRC[158].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC159       MSCM.IRSPRC[159].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC160       MSCM.IRSPRC[160].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC161       MSCM.IRSPRC[161].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC162       MSCM.IRSPRC[162].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC163       MSCM.IRSPRC[163].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC164       MSCM.IRSPRC[164].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC165       MSCM.IRSPRC[165].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC166       MSCM.IRSPRC[166].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC167       MSCM.IRSPRC[167].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC168       MSCM.IRSPRC[168].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC169       MSCM.IRSPRC[169].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC170       MSCM.IRSPRC[170].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC171       MSCM.IRSPRC[171].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC172       MSCM.IRSPRC[172].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC173       MSCM.IRSPRC[173].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC174       MSCM.IRSPRC[174].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC175       MSCM.IRSPRC[175].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC176       MSCM.IRSPRC[176].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC177       MSCM.IRSPRC[177].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC178       MSCM.IRSPRC[178].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC179       MSCM.IRSPRC[179].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC180       MSCM.IRSPRC[180].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC181       MSCM.IRSPRC[181].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC182       MSCM.IRSPRC[182].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC183       MSCM.IRSPRC[183].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC184       MSCM.IRSPRC[184].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC185       MSCM.IRSPRC[185].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC186       MSCM.IRSPRC[186].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC187       MSCM.IRSPRC[187].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC188       MSCM.IRSPRC[188].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC189       MSCM.IRSPRC[189].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC190       MSCM.IRSPRC[190].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC191       MSCM.IRSPRC[191].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC192       MSCM.IRSPRC[192].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC193       MSCM.IRSPRC[193].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC194       MSCM.IRSPRC[194].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC195       MSCM.IRSPRC[195].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC196       MSCM.IRSPRC[196].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC197       MSCM.IRSPRC[197].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC198       MSCM.IRSPRC[198].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC199       MSCM.IRSPRC[199].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC200       MSCM.IRSPRC[200].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC201       MSCM.IRSPRC[201].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC202       MSCM.IRSPRC[202].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC203       MSCM.IRSPRC[203].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC204       MSCM.IRSPRC[204].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC205       MSCM.IRSPRC[205].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC206       MSCM.IRSPRC[206].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC207       MSCM.IRSPRC[207].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC208       MSCM.IRSPRC[208].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC209       MSCM.IRSPRC[209].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC210       MSCM.IRSPRC[210].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC211       MSCM.IRSPRC[211].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC212       MSCM.IRSPRC[212].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC213       MSCM.IRSPRC[213].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC214       MSCM.IRSPRC[214].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC215       MSCM.IRSPRC[215].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC216       MSCM.IRSPRC[216].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC217       MSCM.IRSPRC[217].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC218       MSCM.IRSPRC[218].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC219       MSCM.IRSPRC[219].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC220       MSCM.IRSPRC[220].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC221       MSCM.IRSPRC[221].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC222       MSCM.IRSPRC[222].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC223       MSCM.IRSPRC[223].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC224       MSCM.IRSPRC[224].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC225       MSCM.IRSPRC[225].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC226       MSCM.IRSPRC[226].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC227       MSCM.IRSPRC[227].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC228       MSCM.IRSPRC[228].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC229       MSCM.IRSPRC[229].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC230       MSCM.IRSPRC[230].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC231       MSCM.IRSPRC[231].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC232       MSCM.IRSPRC[232].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC233       MSCM.IRSPRC[233].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC234       MSCM.IRSPRC[234].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC235       MSCM.IRSPRC[235].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC236       MSCM.IRSPRC[236].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC237       MSCM.IRSPRC[237].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC238       MSCM.IRSPRC[238].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC239       MSCM.IRSPRC[239].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC240       MSCM.IRSPRC[240].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC241       MSCM.IRSPRC[241].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC242       MSCM.IRSPRC[242].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC243       MSCM.IRSPRC[243].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC244       MSCM.IRSPRC[244].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC245       MSCM.IRSPRC[245].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC246       MSCM.IRSPRC[246].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC247       MSCM.IRSPRC[247].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC248       MSCM.IRSPRC[248].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC249       MSCM.IRSPRC[249].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC250       MSCM.IRSPRC[250].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC251       MSCM.IRSPRC[251].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC252       MSCM.IRSPRC[252].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC253       MSCM.IRSPRC[253].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC254       MSCM.IRSPRC[254].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC255       MSCM.IRSPRC[255].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC256       MSCM.IRSPRC[256].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC257       MSCM.IRSPRC[257].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC258       MSCM.IRSPRC[258].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC259       MSCM.IRSPRC[259].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC260       MSCM.IRSPRC[260].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC261       MSCM.IRSPRC[261].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC262       MSCM.IRSPRC[262].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC263       MSCM.IRSPRC[263].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC264       MSCM.IRSPRC[264].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC265       MSCM.IRSPRC[265].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC266       MSCM.IRSPRC[266].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC267       MSCM.IRSPRC[267].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC268       MSCM.IRSPRC[268].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC269       MSCM.IRSPRC[269].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC270       MSCM.IRSPRC[270].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC271       MSCM.IRSPRC[271].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC272       MSCM.IRSPRC[272].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC273       MSCM.IRSPRC[273].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC274       MSCM.IRSPRC[274].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC275       MSCM.IRSPRC[275].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC276       MSCM.IRSPRC[276].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC277       MSCM.IRSPRC[277].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC278       MSCM.IRSPRC[278].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC279       MSCM.IRSPRC[279].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC280       MSCM.IRSPRC[280].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC281       MSCM.IRSPRC[281].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC282       MSCM.IRSPRC[282].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC283       MSCM.IRSPRC[283].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC284       MSCM.IRSPRC[284].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC285       MSCM.IRSPRC[285].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC286       MSCM.IRSPRC[286].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC287       MSCM.IRSPRC[287].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC288       MSCM.IRSPRC[288].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC289       MSCM.IRSPRC[289].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC290       MSCM.IRSPRC[290].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC291       MSCM.IRSPRC[291].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC292       MSCM.IRSPRC[292].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC293       MSCM.IRSPRC[293].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC294       MSCM.IRSPRC[294].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC295       MSCM.IRSPRC[295].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC296       MSCM.IRSPRC[296].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC297       MSCM.IRSPRC[297].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC298       MSCM.IRSPRC[298].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC299       MSCM.IRSPRC[299].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC300       MSCM.IRSPRC[300].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC301       MSCM.IRSPRC[301].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC302       MSCM.IRSPRC[302].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC303       MSCM.IRSPRC[303].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC304       MSCM.IRSPRC[304].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC305       MSCM.IRSPRC[305].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC306       MSCM.IRSPRC[306].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC307       MSCM.IRSPRC[307].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC308       MSCM.IRSPRC[308].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC309       MSCM.IRSPRC[309].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC310       MSCM.IRSPRC[310].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC311       MSCM.IRSPRC[311].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC312       MSCM.IRSPRC[312].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC313       MSCM.IRSPRC[313].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC314       MSCM.IRSPRC[314].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC315       MSCM.IRSPRC[315].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC316       MSCM.IRSPRC[316].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC317       MSCM.IRSPRC[317].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC318       MSCM.IRSPRC[318].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC319       MSCM.IRSPRC[319].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC320       MSCM.IRSPRC[320].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC321       MSCM.IRSPRC[321].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC322       MSCM.IRSPRC[322].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC323       MSCM.IRSPRC[323].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC324       MSCM.IRSPRC[324].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC325       MSCM.IRSPRC[325].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC326       MSCM.IRSPRC[326].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC327       MSCM.IRSPRC[327].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC328       MSCM.IRSPRC[328].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC329       MSCM.IRSPRC[329].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC330       MSCM.IRSPRC[330].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC331       MSCM.IRSPRC[331].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC332       MSCM.IRSPRC[332].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC333       MSCM.IRSPRC[333].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC334       MSCM.IRSPRC[334].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC335       MSCM.IRSPRC[335].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC336       MSCM.IRSPRC[336].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC337       MSCM.IRSPRC[337].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC338       MSCM.IRSPRC[338].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC339       MSCM.IRSPRC[339].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC340       MSCM.IRSPRC[340].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC341       MSCM.IRSPRC[341].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC342       MSCM.IRSPRC[342].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC343       MSCM.IRSPRC[343].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC344       MSCM.IRSPRC[344].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC345       MSCM.IRSPRC[345].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC346       MSCM.IRSPRC[346].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC347       MSCM.IRSPRC[347].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC348       MSCM.IRSPRC[348].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC349       MSCM.IRSPRC[349].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC350       MSCM.IRSPRC[350].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC351       MSCM.IRSPRC[351].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC352       MSCM.IRSPRC[352].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC353       MSCM.IRSPRC[353].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC354       MSCM.IRSPRC[354].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC355       MSCM.IRSPRC[355].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC356       MSCM.IRSPRC[356].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC357       MSCM.IRSPRC[357].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC358       MSCM.IRSPRC[358].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC359       MSCM.IRSPRC[359].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC360       MSCM.IRSPRC[360].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC361       MSCM.IRSPRC[361].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC362       MSCM.IRSPRC[362].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC363       MSCM.IRSPRC[363].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC364       MSCM.IRSPRC[364].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC365       MSCM.IRSPRC[365].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC366       MSCM.IRSPRC[366].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC367       MSCM.IRSPRC[367].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC368       MSCM.IRSPRC[368].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC369       MSCM.IRSPRC[369].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC370       MSCM.IRSPRC[370].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC371       MSCM.IRSPRC[371].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC372       MSCM.IRSPRC[372].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC373       MSCM.IRSPRC[373].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC374       MSCM.IRSPRC[374].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC375       MSCM.IRSPRC[375].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC376       MSCM.IRSPRC[376].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC377       MSCM.IRSPRC[377].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC378       MSCM.IRSPRC[378].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC379       MSCM.IRSPRC[379].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC380       MSCM.IRSPRC[380].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC381       MSCM.IRSPRC[381].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC382       MSCM.IRSPRC[382].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC383       MSCM.IRSPRC[383].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC384       MSCM.IRSPRC[384].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC385       MSCM.IRSPRC[385].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC386       MSCM.IRSPRC[386].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC387       MSCM.IRSPRC[387].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC388       MSCM.IRSPRC[388].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC389       MSCM.IRSPRC[389].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC390       MSCM.IRSPRC[390].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC391       MSCM.IRSPRC[391].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC392       MSCM.IRSPRC[392].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC393       MSCM.IRSPRC[393].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC394       MSCM.IRSPRC[394].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC395       MSCM.IRSPRC[395].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC396       MSCM.IRSPRC[396].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC397       MSCM.IRSPRC[397].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC398       MSCM.IRSPRC[398].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC399       MSCM.IRSPRC[399].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC400       MSCM.IRSPRC[400].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC401       MSCM.IRSPRC[401].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC402       MSCM.IRSPRC[402].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC403       MSCM.IRSPRC[403].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC404       MSCM.IRSPRC[404].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC405       MSCM.IRSPRC[405].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC406       MSCM.IRSPRC[406].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC407       MSCM.IRSPRC[407].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC408       MSCM.IRSPRC[408].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC409       MSCM.IRSPRC[409].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC410       MSCM.IRSPRC[410].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC411       MSCM.IRSPRC[411].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC412       MSCM.IRSPRC[412].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC413       MSCM.IRSPRC[413].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC414       MSCM.IRSPRC[414].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC415       MSCM.IRSPRC[415].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC416       MSCM.IRSPRC[416].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC417       MSCM.IRSPRC[417].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC418       MSCM.IRSPRC[418].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC419       MSCM.IRSPRC[419].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC420       MSCM.IRSPRC[420].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC421       MSCM.IRSPRC[421].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC422       MSCM.IRSPRC[422].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC423       MSCM.IRSPRC[423].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC424       MSCM.IRSPRC[424].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC425       MSCM.IRSPRC[425].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC426       MSCM.IRSPRC[426].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC427       MSCM.IRSPRC[427].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC428       MSCM.IRSPRC[428].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC429       MSCM.IRSPRC[429].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC430       MSCM.IRSPRC[430].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC431       MSCM.IRSPRC[431].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC432       MSCM.IRSPRC[432].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC433       MSCM.IRSPRC[433].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC434       MSCM.IRSPRC[434].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC435       MSCM.IRSPRC[435].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC436       MSCM.IRSPRC[436].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC437       MSCM.IRSPRC[437].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC438       MSCM.IRSPRC[438].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC439       MSCM.IRSPRC[439].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC440       MSCM.IRSPRC[440].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC441       MSCM.IRSPRC[441].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC442       MSCM.IRSPRC[442].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC443       MSCM.IRSPRC[443].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC444       MSCM.IRSPRC[444].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC445       MSCM.IRSPRC[445].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC446       MSCM.IRSPRC[446].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC447       MSCM.IRSPRC[447].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC448       MSCM.IRSPRC[448].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC449       MSCM.IRSPRC[449].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC450       MSCM.IRSPRC[450].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC451       MSCM.IRSPRC[451].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC452       MSCM.IRSPRC[452].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC453       MSCM.IRSPRC[453].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC454       MSCM.IRSPRC[454].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC455       MSCM.IRSPRC[455].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC456       MSCM.IRSPRC[456].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC457       MSCM.IRSPRC[457].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC458       MSCM.IRSPRC[458].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC459       MSCM.IRSPRC[459].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC460       MSCM.IRSPRC[460].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC461       MSCM.IRSPRC[461].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC462       MSCM.IRSPRC[462].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC463       MSCM.IRSPRC[463].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC464       MSCM.IRSPRC[464].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC465       MSCM.IRSPRC[465].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC466       MSCM.IRSPRC[466].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC467       MSCM.IRSPRC[467].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC468       MSCM.IRSPRC[468].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC469       MSCM.IRSPRC[469].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC470       MSCM.IRSPRC[470].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC471       MSCM.IRSPRC[471].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC472       MSCM.IRSPRC[472].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC473       MSCM.IRSPRC[473].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC474       MSCM.IRSPRC[474].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC475       MSCM.IRSPRC[475].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC476       MSCM.IRSPRC[476].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC477       MSCM.IRSPRC[477].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC478       MSCM.IRSPRC[478].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC479       MSCM.IRSPRC[479].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC480       MSCM.IRSPRC[480].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC481       MSCM.IRSPRC[481].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC482       MSCM.IRSPRC[482].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC483       MSCM.IRSPRC[483].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC484       MSCM.IRSPRC[484].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC485       MSCM.IRSPRC[485].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC486       MSCM.IRSPRC[486].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC487       MSCM.IRSPRC[487].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC488       MSCM.IRSPRC[488].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC489       MSCM.IRSPRC[489].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC490       MSCM.IRSPRC[490].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC491       MSCM.IRSPRC[491].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC492       MSCM.IRSPRC[492].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC493       MSCM.IRSPRC[493].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC494       MSCM.IRSPRC[494].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC495       MSCM.IRSPRC[495].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC496       MSCM.IRSPRC[496].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC497       MSCM.IRSPRC[497].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC498       MSCM.IRSPRC[498].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC499       MSCM.IRSPRC[499].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC500       MSCM.IRSPRC[500].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC501       MSCM.IRSPRC[501].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC502       MSCM.IRSPRC[502].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC503       MSCM.IRSPRC[503].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC504       MSCM.IRSPRC[504].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC505       MSCM.IRSPRC[505].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC506       MSCM.IRSPRC[506].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC507       MSCM.IRSPRC[507].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC508       MSCM.IRSPRC[508].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC509       MSCM.IRSPRC[509].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC510       MSCM.IRSPRC[510].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC511       MSCM.IRSPRC[511].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC512       MSCM.IRSPRC[512].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC513       MSCM.IRSPRC[513].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC514       MSCM.IRSPRC[514].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC515       MSCM.IRSPRC[515].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC516       MSCM.IRSPRC[516].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC517       MSCM.IRSPRC[517].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC518       MSCM.IRSPRC[518].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC519       MSCM.IRSPRC[519].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC520       MSCM.IRSPRC[520].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC521       MSCM.IRSPRC[521].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC522       MSCM.IRSPRC[522].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC523       MSCM.IRSPRC[523].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC524       MSCM.IRSPRC[524].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC525       MSCM.IRSPRC[525].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC526       MSCM.IRSPRC[526].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC527       MSCM.IRSPRC[527].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC528       MSCM.IRSPRC[528].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC529       MSCM.IRSPRC[529].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC530       MSCM.IRSPRC[530].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC531       MSCM.IRSPRC[531].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC532       MSCM.IRSPRC[532].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC533       MSCM.IRSPRC[533].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC534       MSCM.IRSPRC[534].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC535       MSCM.IRSPRC[535].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC536       MSCM.IRSPRC[536].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC537       MSCM.IRSPRC[537].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC538       MSCM.IRSPRC[538].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC539       MSCM.IRSPRC[539].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC540       MSCM.IRSPRC[540].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC541       MSCM.IRSPRC[541].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC542       MSCM.IRSPRC[542].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC543       MSCM.IRSPRC[543].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC544       MSCM.IRSPRC[544].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC545       MSCM.IRSPRC[545].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC546       MSCM.IRSPRC[546].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC547       MSCM.IRSPRC[547].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC548       MSCM.IRSPRC[548].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC549       MSCM.IRSPRC[549].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC550       MSCM.IRSPRC[550].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC551       MSCM.IRSPRC[551].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC552       MSCM.IRSPRC[552].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC553       MSCM.IRSPRC[553].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC554       MSCM.IRSPRC[554].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC555       MSCM.IRSPRC[555].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC556       MSCM.IRSPRC[556].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC557       MSCM.IRSPRC[557].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC558       MSCM.IRSPRC[558].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC559       MSCM.IRSPRC[559].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC560       MSCM.IRSPRC[560].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC561       MSCM.IRSPRC[561].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC562       MSCM.IRSPRC[562].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC563       MSCM.IRSPRC[563].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC564       MSCM.IRSPRC[564].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC565       MSCM.IRSPRC[565].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC566       MSCM.IRSPRC[566].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC567       MSCM.IRSPRC[567].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC568       MSCM.IRSPRC[568].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC569       MSCM.IRSPRC[569].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC570       MSCM.IRSPRC[570].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC571       MSCM.IRSPRC[571].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC572       MSCM.IRSPRC[572].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC573       MSCM.IRSPRC[573].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC574       MSCM.IRSPRC[574].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC575       MSCM.IRSPRC[575].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC576       MSCM.IRSPRC[576].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC577       MSCM.IRSPRC[577].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC578       MSCM.IRSPRC[578].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC579       MSCM.IRSPRC[579].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC580       MSCM.IRSPRC[580].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC581       MSCM.IRSPRC[581].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC582       MSCM.IRSPRC[582].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC583       MSCM.IRSPRC[583].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC584       MSCM.IRSPRC[584].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC585       MSCM.IRSPRC[585].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC586       MSCM.IRSPRC[586].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC587       MSCM.IRSPRC[587].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC588       MSCM.IRSPRC[588].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC589       MSCM.IRSPRC[589].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC590       MSCM.IRSPRC[590].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC591       MSCM.IRSPRC[591].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC592       MSCM.IRSPRC[592].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC593       MSCM.IRSPRC[593].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC594       MSCM.IRSPRC[594].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC595       MSCM.IRSPRC[595].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC596       MSCM.IRSPRC[596].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC597       MSCM.IRSPRC[597].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC598       MSCM.IRSPRC[598].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC599       MSCM.IRSPRC[599].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC600       MSCM.IRSPRC[600].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC601       MSCM.IRSPRC[601].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC602       MSCM.IRSPRC[602].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC603       MSCM.IRSPRC[603].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC604       MSCM.IRSPRC[604].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC605       MSCM.IRSPRC[605].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC606       MSCM.IRSPRC[606].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC607       MSCM.IRSPRC[607].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC608       MSCM.IRSPRC[608].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC609       MSCM.IRSPRC[609].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC610       MSCM.IRSPRC[610].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC611       MSCM.IRSPRC[611].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC612       MSCM.IRSPRC[612].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC613       MSCM.IRSPRC[613].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC614       MSCM.IRSPRC[614].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC615       MSCM.IRSPRC[615].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC616       MSCM.IRSPRC[616].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC617       MSCM.IRSPRC[617].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC618       MSCM.IRSPRC[618].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC619       MSCM.IRSPRC[619].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC620       MSCM.IRSPRC[620].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC621       MSCM.IRSPRC[621].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC622       MSCM.IRSPRC[622].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC623       MSCM.IRSPRC[623].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC624       MSCM.IRSPRC[624].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC625       MSCM.IRSPRC[625].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC626       MSCM.IRSPRC[626].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC627       MSCM.IRSPRC[627].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC628       MSCM.IRSPRC[628].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC629       MSCM.IRSPRC[629].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC630       MSCM.IRSPRC[630].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC631       MSCM.IRSPRC[631].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC632       MSCM.IRSPRC[632].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC633       MSCM.IRSPRC[633].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC634       MSCM.IRSPRC[634].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC635       MSCM.IRSPRC[635].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC636       MSCM.IRSPRC[636].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC637       MSCM.IRSPRC[637].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC638       MSCM.IRSPRC[638].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC639       MSCM.IRSPRC[639].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC640       MSCM.IRSPRC[640].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC641       MSCM.IRSPRC[641].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC642       MSCM.IRSPRC[642].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC643       MSCM.IRSPRC[643].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC644       MSCM.IRSPRC[644].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC645       MSCM.IRSPRC[645].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC646       MSCM.IRSPRC[646].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC647       MSCM.IRSPRC[647].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC648       MSCM.IRSPRC[648].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC649       MSCM.IRSPRC[649].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC650       MSCM.IRSPRC[650].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC651       MSCM.IRSPRC[651].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC652       MSCM.IRSPRC[652].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC653       MSCM.IRSPRC[653].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC654       MSCM.IRSPRC[654].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC655       MSCM.IRSPRC[655].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC656       MSCM.IRSPRC[656].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC657       MSCM.IRSPRC[657].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC658       MSCM.IRSPRC[658].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC659       MSCM.IRSPRC[659].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC660       MSCM.IRSPRC[660].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC661       MSCM.IRSPRC[661].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC662       MSCM.IRSPRC[662].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC663       MSCM.IRSPRC[663].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC664       MSCM.IRSPRC[664].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC665       MSCM.IRSPRC[665].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC666       MSCM.IRSPRC[666].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC667       MSCM.IRSPRC[667].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC668       MSCM.IRSPRC[668].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC669       MSCM.IRSPRC[669].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC670       MSCM.IRSPRC[670].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC671       MSCM.IRSPRC[671].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC672       MSCM.IRSPRC[672].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC673       MSCM.IRSPRC[673].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC674       MSCM.IRSPRC[674].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC675       MSCM.IRSPRC[675].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC676       MSCM.IRSPRC[676].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC677       MSCM.IRSPRC[677].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC678       MSCM.IRSPRC[678].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC679       MSCM.IRSPRC[679].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC680       MSCM.IRSPRC[680].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC681       MSCM.IRSPRC[681].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC682       MSCM.IRSPRC[682].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC683       MSCM.IRSPRC[683].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC684       MSCM.IRSPRC[684].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC685       MSCM.IRSPRC[685].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC686       MSCM.IRSPRC[686].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC687       MSCM.IRSPRC[687].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC688       MSCM.IRSPRC[688].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC689       MSCM.IRSPRC[689].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC690       MSCM.IRSPRC[690].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC691       MSCM.IRSPRC[691].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC692       MSCM.IRSPRC[692].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC693       MSCM.IRSPRC[693].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC694       MSCM.IRSPRC[694].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC695       MSCM.IRSPRC[695].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC696       MSCM.IRSPRC[696].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC697       MSCM.IRSPRC[697].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC698       MSCM.IRSPRC[698].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC699       MSCM.IRSPRC[699].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC700       MSCM.IRSPRC[700].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC701       MSCM.IRSPRC[701].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC702       MSCM.IRSPRC[702].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC703       MSCM.IRSPRC[703].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC704       MSCM.IRSPRC[704].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC705       MSCM.IRSPRC[705].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC706       MSCM.IRSPRC[706].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC707       MSCM.IRSPRC[707].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC708       MSCM.IRSPRC[708].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC709       MSCM.IRSPRC[709].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC710       MSCM.IRSPRC[710].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC711       MSCM.IRSPRC[711].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC712       MSCM.IRSPRC[712].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC713       MSCM.IRSPRC[713].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC714       MSCM.IRSPRC[714].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC715       MSCM.IRSPRC[715].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC716       MSCM.IRSPRC[716].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC717       MSCM.IRSPRC[717].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC718       MSCM.IRSPRC[718].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC719       MSCM.IRSPRC[719].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC720       MSCM.IRSPRC[720].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC721       MSCM.IRSPRC[721].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC722       MSCM.IRSPRC[722].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC723       MSCM.IRSPRC[723].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC724       MSCM.IRSPRC[724].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC725       MSCM.IRSPRC[725].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC726       MSCM.IRSPRC[726].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC727       MSCM.IRSPRC[727].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC728       MSCM.IRSPRC[728].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC729       MSCM.IRSPRC[729].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC730       MSCM.IRSPRC[730].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC731       MSCM.IRSPRC[731].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC732       MSCM.IRSPRC[732].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC733       MSCM.IRSPRC[733].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC734       MSCM.IRSPRC[734].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC735       MSCM.IRSPRC[735].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC736       MSCM.IRSPRC[736].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC737       MSCM.IRSPRC[737].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC738       MSCM.IRSPRC[738].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC739       MSCM.IRSPRC[739].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC740       MSCM.IRSPRC[740].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC741       MSCM.IRSPRC[741].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC742       MSCM.IRSPRC[742].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC743       MSCM.IRSPRC[743].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC744       MSCM.IRSPRC[744].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC745       MSCM.IRSPRC[745].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC746       MSCM.IRSPRC[746].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC747       MSCM.IRSPRC[747].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC748       MSCM.IRSPRC[748].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC749       MSCM.IRSPRC[749].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC750       MSCM.IRSPRC[750].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC751       MSCM.IRSPRC[751].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC752       MSCM.IRSPRC[752].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC753       MSCM.IRSPRC[753].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC754       MSCM.IRSPRC[754].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC755       MSCM.IRSPRC[755].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC756       MSCM.IRSPRC[756].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC757       MSCM.IRSPRC[757].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC758       MSCM.IRSPRC[758].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC759       MSCM.IRSPRC[759].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC760       MSCM.IRSPRC[760].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC761       MSCM.IRSPRC[761].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC762       MSCM.IRSPRC[762].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC763       MSCM.IRSPRC[763].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC764       MSCM.IRSPRC[764].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC765       MSCM.IRSPRC[765].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC766       MSCM.IRSPRC[766].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC767       MSCM.IRSPRC[767].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC768       MSCM.IRSPRC[768].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC769       MSCM.IRSPRC[769].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC770       MSCM.IRSPRC[770].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC771       MSCM.IRSPRC[771].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC772       MSCM.IRSPRC[772].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC773       MSCM.IRSPRC[773].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC774       MSCM.IRSPRC[774].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC775       MSCM.IRSPRC[775].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC776       MSCM.IRSPRC[776].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC777       MSCM.IRSPRC[777].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC778       MSCM.IRSPRC[778].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC779       MSCM.IRSPRC[779].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC780       MSCM.IRSPRC[780].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC781       MSCM.IRSPRC[781].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC782       MSCM.IRSPRC[782].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC783       MSCM.IRSPRC[783].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC784       MSCM.IRSPRC[784].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC785       MSCM.IRSPRC[785].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC786       MSCM.IRSPRC[786].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC787       MSCM.IRSPRC[787].R            /* Interrupt Router Shared Peripheral Routing Control */
#define MSCM_IRSPRC788       MSCM.IRSPRC[788].R            /* Interrupt Router Shared Peripheral Routing Control */

/* MU */
#define MU0__MUA_VER         MU0__MUA.VER.R                /* Version ID */
#define MU0__MUA_PAR         MU0__MUA.PAR.R                /* Parameter Register */
#define MU0__MUA_CR          MU0__MUA.CR.R                 /* Control Register */
#define MU0__MUA_SR          MU0__MUA.SR.R                 /* Status Register */
#define MU0__MUA_CCR0        MU0__MUA.CCR0.R               /* Core Control Register 0 */
#define MU0__MUA_CSSR0       MU0__MUA.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU0__MUA_FCR         MU0__MUA.FCR.R                /* Flag Control Register */
#define MU0__MUA_FSR         MU0__MUA.FSR.R                /* Flag Status Register */
#define MU0__MUA_GIER        MU0__MUA.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU0__MUA_GCR         MU0__MUA.GCR.R                /* General-purpose Control Register */
#define MU0__MUA_GSR         MU0__MUA.GSR.R                /* General-purpose Status Register */
#define MU0__MUA_TCR         MU0__MUA.TCR.R                /* Transmit Control Register */
#define MU0__MUA_TSR         MU0__MUA.TSR.R                /* Transmit Status Register */
#define MU0__MUA_RCR         MU0__MUA.RCR.R                /* Receive Control Register */
#define MU0__MUA_RSR         MU0__MUA.RSR.R                /* Receive Status Register */
#define MU0__MUA_TR0         MU0__MUA.TR[0].R              /* Transmit Register */
#define MU0__MUA_TR1         MU0__MUA.TR[1].R              /* Transmit Register */
#define MU0__MUA_TR2         MU0__MUA.TR[2].R              /* Transmit Register */
#define MU0__MUA_TR3         MU0__MUA.TR[3].R              /* Transmit Register */
#define MU0__MUA_TR4         MU0__MUA.TR[4].R              /* Transmit Register */
#define MU0__MUA_TR5         MU0__MUA.TR[5].R              /* Transmit Register */
#define MU0__MUA_TR6         MU0__MUA.TR[6].R              /* Transmit Register */
#define MU0__MUA_TR7         MU0__MUA.TR[7].R              /* Transmit Register */
#define MU0__MUA_TR8         MU0__MUA.TR[8].R              /* Transmit Register */
#define MU0__MUA_TR9         MU0__MUA.TR[9].R              /* Transmit Register */
#define MU0__MUA_TR10        MU0__MUA.TR[10].R             /* Transmit Register */
#define MU0__MUA_TR11        MU0__MUA.TR[11].R             /* Transmit Register */
#define MU0__MUA_TR12        MU0__MUA.TR[12].R             /* Transmit Register */
#define MU0__MUA_TR13        MU0__MUA.TR[13].R             /* Transmit Register */
#define MU0__MUA_TR14        MU0__MUA.TR[14].R             /* Transmit Register */
#define MU0__MUA_TR15        MU0__MUA.TR[15].R             /* Transmit Register */
#define MU0__MUA_RR0         MU0__MUA.RR[0].R              /* Receive Register */
#define MU0__MUA_RR1         MU0__MUA.RR[1].R              /* Receive Register */
#define MU0__MUA_RR2         MU0__MUA.RR[2].R              /* Receive Register */
#define MU0__MUA_RR3         MU0__MUA.RR[3].R              /* Receive Register */
#define MU0__MUA_RR4         MU0__MUA.RR[4].R              /* Receive Register */
#define MU0__MUA_RR5         MU0__MUA.RR[5].R              /* Receive Register */
#define MU0__MUA_RR6         MU0__MUA.RR[6].R              /* Receive Register */
#define MU0__MUA_RR7         MU0__MUA.RR[7].R              /* Receive Register */
#define MU0__MUA_RR8         MU0__MUA.RR[8].R              /* Receive Register */
#define MU0__MUA_RR9         MU0__MUA.RR[9].R              /* Receive Register */
#define MU0__MUA_RR10        MU0__MUA.RR[10].R             /* Receive Register */
#define MU0__MUA_RR11        MU0__MUA.RR[11].R             /* Receive Register */
#define MU0__MUA_RR12        MU0__MUA.RR[12].R             /* Receive Register */
#define MU0__MUA_RR13        MU0__MUA.RR[13].R             /* Receive Register */
#define MU0__MUA_RR14        MU0__MUA.RR[14].R             /* Receive Register */
#define MU0__MUA_RR15        MU0__MUA.RR[15].R             /* Receive Register */

#define MU1__MUA_VER         MU1__MUA.VER.R                /* Version ID */
#define MU1__MUA_PAR         MU1__MUA.PAR.R                /* Parameter Register */
#define MU1__MUA_CR          MU1__MUA.CR.R                 /* Control Register */
#define MU1__MUA_SR          MU1__MUA.SR.R                 /* Status Register */
#define MU1__MUA_CCR0        MU1__MUA.CCR0.R               /* Core Control Register 0 */
#define MU1__MUA_CSSR0       MU1__MUA.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU1__MUA_FCR         MU1__MUA.FCR.R                /* Flag Control Register */
#define MU1__MUA_FSR         MU1__MUA.FSR.R                /* Flag Status Register */
#define MU1__MUA_GIER        MU1__MUA.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU1__MUA_GCR         MU1__MUA.GCR.R                /* General-purpose Control Register */
#define MU1__MUA_GSR         MU1__MUA.GSR.R                /* General-purpose Status Register */
#define MU1__MUA_TCR         MU1__MUA.TCR.R                /* Transmit Control Register */
#define MU1__MUA_TSR         MU1__MUA.TSR.R                /* Transmit Status Register */
#define MU1__MUA_RCR         MU1__MUA.RCR.R                /* Receive Control Register */
#define MU1__MUA_RSR         MU1__MUA.RSR.R                /* Receive Status Register */
#define MU1__MUA_TR0         MU1__MUA.TR[0].R              /* Transmit Register */
#define MU1__MUA_TR1         MU1__MUA.TR[1].R              /* Transmit Register */
#define MU1__MUA_TR2         MU1__MUA.TR[2].R              /* Transmit Register */
#define MU1__MUA_TR3         MU1__MUA.TR[3].R              /* Transmit Register */
#define MU1__MUA_TR4         MU1__MUA.TR[4].R              /* Transmit Register */
#define MU1__MUA_TR5         MU1__MUA.TR[5].R              /* Transmit Register */
#define MU1__MUA_TR6         MU1__MUA.TR[6].R              /* Transmit Register */
#define MU1__MUA_TR7         MU1__MUA.TR[7].R              /* Transmit Register */
#define MU1__MUA_TR8         MU1__MUA.TR[8].R              /* Transmit Register */
#define MU1__MUA_TR9         MU1__MUA.TR[9].R              /* Transmit Register */
#define MU1__MUA_TR10        MU1__MUA.TR[10].R             /* Transmit Register */
#define MU1__MUA_TR11        MU1__MUA.TR[11].R             /* Transmit Register */
#define MU1__MUA_TR12        MU1__MUA.TR[12].R             /* Transmit Register */
#define MU1__MUA_TR13        MU1__MUA.TR[13].R             /* Transmit Register */
#define MU1__MUA_TR14        MU1__MUA.TR[14].R             /* Transmit Register */
#define MU1__MUA_TR15        MU1__MUA.TR[15].R             /* Transmit Register */
#define MU1__MUA_RR0         MU1__MUA.RR[0].R              /* Receive Register */
#define MU1__MUA_RR1         MU1__MUA.RR[1].R              /* Receive Register */
#define MU1__MUA_RR2         MU1__MUA.RR[2].R              /* Receive Register */
#define MU1__MUA_RR3         MU1__MUA.RR[3].R              /* Receive Register */
#define MU1__MUA_RR4         MU1__MUA.RR[4].R              /* Receive Register */
#define MU1__MUA_RR5         MU1__MUA.RR[5].R              /* Receive Register */
#define MU1__MUA_RR6         MU1__MUA.RR[6].R              /* Receive Register */
#define MU1__MUA_RR7         MU1__MUA.RR[7].R              /* Receive Register */
#define MU1__MUA_RR8         MU1__MUA.RR[8].R              /* Receive Register */
#define MU1__MUA_RR9         MU1__MUA.RR[9].R              /* Receive Register */
#define MU1__MUA_RR10        MU1__MUA.RR[10].R             /* Receive Register */
#define MU1__MUA_RR11        MU1__MUA.RR[11].R             /* Receive Register */
#define MU1__MUA_RR12        MU1__MUA.RR[12].R             /* Receive Register */
#define MU1__MUA_RR13        MU1__MUA.RR[13].R             /* Receive Register */
#define MU1__MUA_RR14        MU1__MUA.RR[14].R             /* Receive Register */
#define MU1__MUA_RR15        MU1__MUA.RR[15].R             /* Receive Register */

#define MU2__MUA_VER         MU2__MUA.VER.R                /* Version ID */
#define MU2__MUA_PAR         MU2__MUA.PAR.R                /* Parameter Register */
#define MU2__MUA_CR          MU2__MUA.CR.R                 /* Control Register */
#define MU2__MUA_SR          MU2__MUA.SR.R                 /* Status Register */
#define MU2__MUA_CCR0        MU2__MUA.CCR0.R               /* Core Control Register 0 */
#define MU2__MUA_CSSR0       MU2__MUA.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU2__MUA_FCR         MU2__MUA.FCR.R                /* Flag Control Register */
#define MU2__MUA_FSR         MU2__MUA.FSR.R                /* Flag Status Register */
#define MU2__MUA_GIER        MU2__MUA.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU2__MUA_GCR         MU2__MUA.GCR.R                /* General-purpose Control Register */
#define MU2__MUA_GSR         MU2__MUA.GSR.R                /* General-purpose Status Register */
#define MU2__MUA_TCR         MU2__MUA.TCR.R                /* Transmit Control Register */
#define MU2__MUA_TSR         MU2__MUA.TSR.R                /* Transmit Status Register */
#define MU2__MUA_RCR         MU2__MUA.RCR.R                /* Receive Control Register */
#define MU2__MUA_RSR         MU2__MUA.RSR.R                /* Receive Status Register */
#define MU2__MUA_TR0         MU2__MUA.TR[0].R              /* Transmit Register */
#define MU2__MUA_TR1         MU2__MUA.TR[1].R              /* Transmit Register */
#define MU2__MUA_TR2         MU2__MUA.TR[2].R              /* Transmit Register */
#define MU2__MUA_TR3         MU2__MUA.TR[3].R              /* Transmit Register */
#define MU2__MUA_TR4         MU2__MUA.TR[4].R              /* Transmit Register */
#define MU2__MUA_TR5         MU2__MUA.TR[5].R              /* Transmit Register */
#define MU2__MUA_TR6         MU2__MUA.TR[6].R              /* Transmit Register */
#define MU2__MUA_TR7         MU2__MUA.TR[7].R              /* Transmit Register */
#define MU2__MUA_TR8         MU2__MUA.TR[8].R              /* Transmit Register */
#define MU2__MUA_TR9         MU2__MUA.TR[9].R              /* Transmit Register */
#define MU2__MUA_TR10        MU2__MUA.TR[10].R             /* Transmit Register */
#define MU2__MUA_TR11        MU2__MUA.TR[11].R             /* Transmit Register */
#define MU2__MUA_TR12        MU2__MUA.TR[12].R             /* Transmit Register */
#define MU2__MUA_TR13        MU2__MUA.TR[13].R             /* Transmit Register */
#define MU2__MUA_TR14        MU2__MUA.TR[14].R             /* Transmit Register */
#define MU2__MUA_TR15        MU2__MUA.TR[15].R             /* Transmit Register */
#define MU2__MUA_RR0         MU2__MUA.RR[0].R              /* Receive Register */
#define MU2__MUA_RR1         MU2__MUA.RR[1].R              /* Receive Register */
#define MU2__MUA_RR2         MU2__MUA.RR[2].R              /* Receive Register */
#define MU2__MUA_RR3         MU2__MUA.RR[3].R              /* Receive Register */
#define MU2__MUA_RR4         MU2__MUA.RR[4].R              /* Receive Register */
#define MU2__MUA_RR5         MU2__MUA.RR[5].R              /* Receive Register */
#define MU2__MUA_RR6         MU2__MUA.RR[6].R              /* Receive Register */
#define MU2__MUA_RR7         MU2__MUA.RR[7].R              /* Receive Register */
#define MU2__MUA_RR8         MU2__MUA.RR[8].R              /* Receive Register */
#define MU2__MUA_RR9         MU2__MUA.RR[9].R              /* Receive Register */
#define MU2__MUA_RR10        MU2__MUA.RR[10].R             /* Receive Register */
#define MU2__MUA_RR11        MU2__MUA.RR[11].R             /* Receive Register */
#define MU2__MUA_RR12        MU2__MUA.RR[12].R             /* Receive Register */
#define MU2__MUA_RR13        MU2__MUA.RR[13].R             /* Receive Register */
#define MU2__MUA_RR14        MU2__MUA.RR[14].R             /* Receive Register */
#define MU2__MUA_RR15        MU2__MUA.RR[15].R             /* Receive Register */

#define MU3__MUA_VER         MU3__MUA.VER.R                /* Version ID */
#define MU3__MUA_PAR         MU3__MUA.PAR.R                /* Parameter Register */
#define MU3__MUA_CR          MU3__MUA.CR.R                 /* Control Register */
#define MU3__MUA_SR          MU3__MUA.SR.R                 /* Status Register */
#define MU3__MUA_CCR0        MU3__MUA.CCR0.R               /* Core Control Register 0 */
#define MU3__MUA_CSSR0       MU3__MUA.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU3__MUA_FCR         MU3__MUA.FCR.R                /* Flag Control Register */
#define MU3__MUA_FSR         MU3__MUA.FSR.R                /* Flag Status Register */
#define MU3__MUA_GIER        MU3__MUA.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU3__MUA_GCR         MU3__MUA.GCR.R                /* General-purpose Control Register */
#define MU3__MUA_GSR         MU3__MUA.GSR.R                /* General-purpose Status Register */
#define MU3__MUA_TCR         MU3__MUA.TCR.R                /* Transmit Control Register */
#define MU3__MUA_TSR         MU3__MUA.TSR.R                /* Transmit Status Register */
#define MU3__MUA_RCR         MU3__MUA.RCR.R                /* Receive Control Register */
#define MU3__MUA_RSR         MU3__MUA.RSR.R                /* Receive Status Register */
#define MU3__MUA_TR0         MU3__MUA.TR[0].R              /* Transmit Register */
#define MU3__MUA_TR1         MU3__MUA.TR[1].R              /* Transmit Register */
#define MU3__MUA_TR2         MU3__MUA.TR[2].R              /* Transmit Register */
#define MU3__MUA_TR3         MU3__MUA.TR[3].R              /* Transmit Register */
#define MU3__MUA_TR4         MU3__MUA.TR[4].R              /* Transmit Register */
#define MU3__MUA_TR5         MU3__MUA.TR[5].R              /* Transmit Register */
#define MU3__MUA_TR6         MU3__MUA.TR[6].R              /* Transmit Register */
#define MU3__MUA_TR7         MU3__MUA.TR[7].R              /* Transmit Register */
#define MU3__MUA_TR8         MU3__MUA.TR[8].R              /* Transmit Register */
#define MU3__MUA_TR9         MU3__MUA.TR[9].R              /* Transmit Register */
#define MU3__MUA_TR10        MU3__MUA.TR[10].R             /* Transmit Register */
#define MU3__MUA_TR11        MU3__MUA.TR[11].R             /* Transmit Register */
#define MU3__MUA_TR12        MU3__MUA.TR[12].R             /* Transmit Register */
#define MU3__MUA_TR13        MU3__MUA.TR[13].R             /* Transmit Register */
#define MU3__MUA_TR14        MU3__MUA.TR[14].R             /* Transmit Register */
#define MU3__MUA_TR15        MU3__MUA.TR[15].R             /* Transmit Register */
#define MU3__MUA_RR0         MU3__MUA.RR[0].R              /* Receive Register */
#define MU3__MUA_RR1         MU3__MUA.RR[1].R              /* Receive Register */
#define MU3__MUA_RR2         MU3__MUA.RR[2].R              /* Receive Register */
#define MU3__MUA_RR3         MU3__MUA.RR[3].R              /* Receive Register */
#define MU3__MUA_RR4         MU3__MUA.RR[4].R              /* Receive Register */
#define MU3__MUA_RR5         MU3__MUA.RR[5].R              /* Receive Register */
#define MU3__MUA_RR6         MU3__MUA.RR[6].R              /* Receive Register */
#define MU3__MUA_RR7         MU3__MUA.RR[7].R              /* Receive Register */
#define MU3__MUA_RR8         MU3__MUA.RR[8].R              /* Receive Register */
#define MU3__MUA_RR9         MU3__MUA.RR[9].R              /* Receive Register */
#define MU3__MUA_RR10        MU3__MUA.RR[10].R             /* Receive Register */
#define MU3__MUA_RR11        MU3__MUA.RR[11].R             /* Receive Register */
#define MU3__MUA_RR12        MU3__MUA.RR[12].R             /* Receive Register */
#define MU3__MUA_RR13        MU3__MUA.RR[13].R             /* Receive Register */
#define MU3__MUA_RR14        MU3__MUA.RR[14].R             /* Receive Register */
#define MU3__MUA_RR15        MU3__MUA.RR[15].R             /* Receive Register */

#define MU4__MUA_VER         MU4__MUA.VER.R                /* Version ID */
#define MU4__MUA_PAR         MU4__MUA.PAR.R                /* Parameter Register */
#define MU4__MUA_CR          MU4__MUA.CR.R                 /* Control Register */
#define MU4__MUA_SR          MU4__MUA.SR.R                 /* Status Register */
#define MU4__MUA_CCR0        MU4__MUA.CCR0.R               /* Core Control Register 0 */
#define MU4__MUA_CSSR0       MU4__MUA.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU4__MUA_FCR         MU4__MUA.FCR.R                /* Flag Control Register */
#define MU4__MUA_FSR         MU4__MUA.FSR.R                /* Flag Status Register */
#define MU4__MUA_GIER        MU4__MUA.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU4__MUA_GCR         MU4__MUA.GCR.R                /* General-purpose Control Register */
#define MU4__MUA_GSR         MU4__MUA.GSR.R                /* General-purpose Status Register */
#define MU4__MUA_TCR         MU4__MUA.TCR.R                /* Transmit Control Register */
#define MU4__MUA_TSR         MU4__MUA.TSR.R                /* Transmit Status Register */
#define MU4__MUA_RCR         MU4__MUA.RCR.R                /* Receive Control Register */
#define MU4__MUA_RSR         MU4__MUA.RSR.R                /* Receive Status Register */
#define MU4__MUA_TR0         MU4__MUA.TR[0].R              /* Transmit Register */
#define MU4__MUA_TR1         MU4__MUA.TR[1].R              /* Transmit Register */
#define MU4__MUA_TR2         MU4__MUA.TR[2].R              /* Transmit Register */
#define MU4__MUA_TR3         MU4__MUA.TR[3].R              /* Transmit Register */
#define MU4__MUA_TR4         MU4__MUA.TR[4].R              /* Transmit Register */
#define MU4__MUA_TR5         MU4__MUA.TR[5].R              /* Transmit Register */
#define MU4__MUA_TR6         MU4__MUA.TR[6].R              /* Transmit Register */
#define MU4__MUA_TR7         MU4__MUA.TR[7].R              /* Transmit Register */
#define MU4__MUA_TR8         MU4__MUA.TR[8].R              /* Transmit Register */
#define MU4__MUA_TR9         MU4__MUA.TR[9].R              /* Transmit Register */
#define MU4__MUA_TR10        MU4__MUA.TR[10].R             /* Transmit Register */
#define MU4__MUA_TR11        MU4__MUA.TR[11].R             /* Transmit Register */
#define MU4__MUA_TR12        MU4__MUA.TR[12].R             /* Transmit Register */
#define MU4__MUA_TR13        MU4__MUA.TR[13].R             /* Transmit Register */
#define MU4__MUA_TR14        MU4__MUA.TR[14].R             /* Transmit Register */
#define MU4__MUA_TR15        MU4__MUA.TR[15].R             /* Transmit Register */
#define MU4__MUA_RR0         MU4__MUA.RR[0].R              /* Receive Register */
#define MU4__MUA_RR1         MU4__MUA.RR[1].R              /* Receive Register */
#define MU4__MUA_RR2         MU4__MUA.RR[2].R              /* Receive Register */
#define MU4__MUA_RR3         MU4__MUA.RR[3].R              /* Receive Register */
#define MU4__MUA_RR4         MU4__MUA.RR[4].R              /* Receive Register */
#define MU4__MUA_RR5         MU4__MUA.RR[5].R              /* Receive Register */
#define MU4__MUA_RR6         MU4__MUA.RR[6].R              /* Receive Register */
#define MU4__MUA_RR7         MU4__MUA.RR[7].R              /* Receive Register */
#define MU4__MUA_RR8         MU4__MUA.RR[8].R              /* Receive Register */
#define MU4__MUA_RR9         MU4__MUA.RR[9].R              /* Receive Register */
#define MU4__MUA_RR10        MU4__MUA.RR[10].R             /* Receive Register */
#define MU4__MUA_RR11        MU4__MUA.RR[11].R             /* Receive Register */
#define MU4__MUA_RR12        MU4__MUA.RR[12].R             /* Receive Register */
#define MU4__MUA_RR13        MU4__MUA.RR[13].R             /* Receive Register */
#define MU4__MUA_RR14        MU4__MUA.RR[14].R             /* Receive Register */
#define MU4__MUA_RR15        MU4__MUA.RR[15].R             /* Receive Register */

#define MU5__MUA_VER         MU5__MUA.VER.R                /* Version ID */
#define MU5__MUA_PAR         MU5__MUA.PAR.R                /* Parameter Register */
#define MU5__MUA_CR          MU5__MUA.CR.R                 /* Control Register */
#define MU5__MUA_SR          MU5__MUA.SR.R                 /* Status Register */
#define MU5__MUA_CCR0        MU5__MUA.CCR0.R               /* Core Control Register 0 */
#define MU5__MUA_CSSR0       MU5__MUA.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU5__MUA_FCR         MU5__MUA.FCR.R                /* Flag Control Register */
#define MU5__MUA_FSR         MU5__MUA.FSR.R                /* Flag Status Register */
#define MU5__MUA_GIER        MU5__MUA.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU5__MUA_GCR         MU5__MUA.GCR.R                /* General-purpose Control Register */
#define MU5__MUA_GSR         MU5__MUA.GSR.R                /* General-purpose Status Register */
#define MU5__MUA_TCR         MU5__MUA.TCR.R                /* Transmit Control Register */
#define MU5__MUA_TSR         MU5__MUA.TSR.R                /* Transmit Status Register */
#define MU5__MUA_RCR         MU5__MUA.RCR.R                /* Receive Control Register */
#define MU5__MUA_RSR         MU5__MUA.RSR.R                /* Receive Status Register */
#define MU5__MUA_TR0         MU5__MUA.TR[0].R              /* Transmit Register */
#define MU5__MUA_TR1         MU5__MUA.TR[1].R              /* Transmit Register */
#define MU5__MUA_TR2         MU5__MUA.TR[2].R              /* Transmit Register */
#define MU5__MUA_TR3         MU5__MUA.TR[3].R              /* Transmit Register */
#define MU5__MUA_TR4         MU5__MUA.TR[4].R              /* Transmit Register */
#define MU5__MUA_TR5         MU5__MUA.TR[5].R              /* Transmit Register */
#define MU5__MUA_TR6         MU5__MUA.TR[6].R              /* Transmit Register */
#define MU5__MUA_TR7         MU5__MUA.TR[7].R              /* Transmit Register */
#define MU5__MUA_TR8         MU5__MUA.TR[8].R              /* Transmit Register */
#define MU5__MUA_TR9         MU5__MUA.TR[9].R              /* Transmit Register */
#define MU5__MUA_TR10        MU5__MUA.TR[10].R             /* Transmit Register */
#define MU5__MUA_TR11        MU5__MUA.TR[11].R             /* Transmit Register */
#define MU5__MUA_TR12        MU5__MUA.TR[12].R             /* Transmit Register */
#define MU5__MUA_TR13        MU5__MUA.TR[13].R             /* Transmit Register */
#define MU5__MUA_TR14        MU5__MUA.TR[14].R             /* Transmit Register */
#define MU5__MUA_TR15        MU5__MUA.TR[15].R             /* Transmit Register */
#define MU5__MUA_RR0         MU5__MUA.RR[0].R              /* Receive Register */
#define MU5__MUA_RR1         MU5__MUA.RR[1].R              /* Receive Register */
#define MU5__MUA_RR2         MU5__MUA.RR[2].R              /* Receive Register */
#define MU5__MUA_RR3         MU5__MUA.RR[3].R              /* Receive Register */
#define MU5__MUA_RR4         MU5__MUA.RR[4].R              /* Receive Register */
#define MU5__MUA_RR5         MU5__MUA.RR[5].R              /* Receive Register */
#define MU5__MUA_RR6         MU5__MUA.RR[6].R              /* Receive Register */
#define MU5__MUA_RR7         MU5__MUA.RR[7].R              /* Receive Register */
#define MU5__MUA_RR8         MU5__MUA.RR[8].R              /* Receive Register */
#define MU5__MUA_RR9         MU5__MUA.RR[9].R              /* Receive Register */
#define MU5__MUA_RR10        MU5__MUA.RR[10].R             /* Receive Register */
#define MU5__MUA_RR11        MU5__MUA.RR[11].R             /* Receive Register */
#define MU5__MUA_RR12        MU5__MUA.RR[12].R             /* Receive Register */
#define MU5__MUA_RR13        MU5__MUA.RR[13].R             /* Receive Register */
#define MU5__MUA_RR14        MU5__MUA.RR[14].R             /* Receive Register */
#define MU5__MUA_RR15        MU5__MUA.RR[15].R             /* Receive Register */

#define MU6__MUA_VER         MU6__MUA.VER.R                /* Version ID */
#define MU6__MUA_PAR         MU6__MUA.PAR.R                /* Parameter Register */
#define MU6__MUA_CR          MU6__MUA.CR.R                 /* Control Register */
#define MU6__MUA_SR          MU6__MUA.SR.R                 /* Status Register */
#define MU6__MUA_CCR0        MU6__MUA.CCR0.R               /* Core Control Register 0 */
#define MU6__MUA_CSSR0       MU6__MUA.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU6__MUA_FCR         MU6__MUA.FCR.R                /* Flag Control Register */
#define MU6__MUA_FSR         MU6__MUA.FSR.R                /* Flag Status Register */
#define MU6__MUA_GIER        MU6__MUA.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU6__MUA_GCR         MU6__MUA.GCR.R                /* General-purpose Control Register */
#define MU6__MUA_GSR         MU6__MUA.GSR.R                /* General-purpose Status Register */
#define MU6__MUA_TCR         MU6__MUA.TCR.R                /* Transmit Control Register */
#define MU6__MUA_TSR         MU6__MUA.TSR.R                /* Transmit Status Register */
#define MU6__MUA_RCR         MU6__MUA.RCR.R                /* Receive Control Register */
#define MU6__MUA_RSR         MU6__MUA.RSR.R                /* Receive Status Register */
#define MU6__MUA_TR0         MU6__MUA.TR[0].R              /* Transmit Register */
#define MU6__MUA_TR1         MU6__MUA.TR[1].R              /* Transmit Register */
#define MU6__MUA_TR2         MU6__MUA.TR[2].R              /* Transmit Register */
#define MU6__MUA_TR3         MU6__MUA.TR[3].R              /* Transmit Register */
#define MU6__MUA_TR4         MU6__MUA.TR[4].R              /* Transmit Register */
#define MU6__MUA_TR5         MU6__MUA.TR[5].R              /* Transmit Register */
#define MU6__MUA_TR6         MU6__MUA.TR[6].R              /* Transmit Register */
#define MU6__MUA_TR7         MU6__MUA.TR[7].R              /* Transmit Register */
#define MU6__MUA_TR8         MU6__MUA.TR[8].R              /* Transmit Register */
#define MU6__MUA_TR9         MU6__MUA.TR[9].R              /* Transmit Register */
#define MU6__MUA_TR10        MU6__MUA.TR[10].R             /* Transmit Register */
#define MU6__MUA_TR11        MU6__MUA.TR[11].R             /* Transmit Register */
#define MU6__MUA_TR12        MU6__MUA.TR[12].R             /* Transmit Register */
#define MU6__MUA_TR13        MU6__MUA.TR[13].R             /* Transmit Register */
#define MU6__MUA_TR14        MU6__MUA.TR[14].R             /* Transmit Register */
#define MU6__MUA_TR15        MU6__MUA.TR[15].R             /* Transmit Register */
#define MU6__MUA_RR0         MU6__MUA.RR[0].R              /* Receive Register */
#define MU6__MUA_RR1         MU6__MUA.RR[1].R              /* Receive Register */
#define MU6__MUA_RR2         MU6__MUA.RR[2].R              /* Receive Register */
#define MU6__MUA_RR3         MU6__MUA.RR[3].R              /* Receive Register */
#define MU6__MUA_RR4         MU6__MUA.RR[4].R              /* Receive Register */
#define MU6__MUA_RR5         MU6__MUA.RR[5].R              /* Receive Register */
#define MU6__MUA_RR6         MU6__MUA.RR[6].R              /* Receive Register */
#define MU6__MUA_RR7         MU6__MUA.RR[7].R              /* Receive Register */
#define MU6__MUA_RR8         MU6__MUA.RR[8].R              /* Receive Register */
#define MU6__MUA_RR9         MU6__MUA.RR[9].R              /* Receive Register */
#define MU6__MUA_RR10        MU6__MUA.RR[10].R             /* Receive Register */
#define MU6__MUA_RR11        MU6__MUA.RR[11].R             /* Receive Register */
#define MU6__MUA_RR12        MU6__MUA.RR[12].R             /* Receive Register */
#define MU6__MUA_RR13        MU6__MUA.RR[13].R             /* Receive Register */
#define MU6__MUA_RR14        MU6__MUA.RR[14].R             /* Receive Register */
#define MU6__MUA_RR15        MU6__MUA.RR[15].R             /* Receive Register */

#define MU7__MUA_VER         MU7__MUA.VER.R                /* Version ID */
#define MU7__MUA_PAR         MU7__MUA.PAR.R                /* Parameter Register */
#define MU7__MUA_CR          MU7__MUA.CR.R                 /* Control Register */
#define MU7__MUA_SR          MU7__MUA.SR.R                 /* Status Register */
#define MU7__MUA_CCR0        MU7__MUA.CCR0.R               /* Core Control Register 0 */
#define MU7__MUA_CSSR0       MU7__MUA.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU7__MUA_FCR         MU7__MUA.FCR.R                /* Flag Control Register */
#define MU7__MUA_FSR         MU7__MUA.FSR.R                /* Flag Status Register */
#define MU7__MUA_GIER        MU7__MUA.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU7__MUA_GCR         MU7__MUA.GCR.R                /* General-purpose Control Register */
#define MU7__MUA_GSR         MU7__MUA.GSR.R                /* General-purpose Status Register */
#define MU7__MUA_TCR         MU7__MUA.TCR.R                /* Transmit Control Register */
#define MU7__MUA_TSR         MU7__MUA.TSR.R                /* Transmit Status Register */
#define MU7__MUA_RCR         MU7__MUA.RCR.R                /* Receive Control Register */
#define MU7__MUA_RSR         MU7__MUA.RSR.R                /* Receive Status Register */
#define MU7__MUA_TR0         MU7__MUA.TR[0].R              /* Transmit Register */
#define MU7__MUA_TR1         MU7__MUA.TR[1].R              /* Transmit Register */
#define MU7__MUA_TR2         MU7__MUA.TR[2].R              /* Transmit Register */
#define MU7__MUA_TR3         MU7__MUA.TR[3].R              /* Transmit Register */
#define MU7__MUA_TR4         MU7__MUA.TR[4].R              /* Transmit Register */
#define MU7__MUA_TR5         MU7__MUA.TR[5].R              /* Transmit Register */
#define MU7__MUA_TR6         MU7__MUA.TR[6].R              /* Transmit Register */
#define MU7__MUA_TR7         MU7__MUA.TR[7].R              /* Transmit Register */
#define MU7__MUA_TR8         MU7__MUA.TR[8].R              /* Transmit Register */
#define MU7__MUA_TR9         MU7__MUA.TR[9].R              /* Transmit Register */
#define MU7__MUA_TR10        MU7__MUA.TR[10].R             /* Transmit Register */
#define MU7__MUA_TR11        MU7__MUA.TR[11].R             /* Transmit Register */
#define MU7__MUA_TR12        MU7__MUA.TR[12].R             /* Transmit Register */
#define MU7__MUA_TR13        MU7__MUA.TR[13].R             /* Transmit Register */
#define MU7__MUA_TR14        MU7__MUA.TR[14].R             /* Transmit Register */
#define MU7__MUA_TR15        MU7__MUA.TR[15].R             /* Transmit Register */
#define MU7__MUA_RR0         MU7__MUA.RR[0].R              /* Receive Register */
#define MU7__MUA_RR1         MU7__MUA.RR[1].R              /* Receive Register */
#define MU7__MUA_RR2         MU7__MUA.RR[2].R              /* Receive Register */
#define MU7__MUA_RR3         MU7__MUA.RR[3].R              /* Receive Register */
#define MU7__MUA_RR4         MU7__MUA.RR[4].R              /* Receive Register */
#define MU7__MUA_RR5         MU7__MUA.RR[5].R              /* Receive Register */
#define MU7__MUA_RR6         MU7__MUA.RR[6].R              /* Receive Register */
#define MU7__MUA_RR7         MU7__MUA.RR[7].R              /* Receive Register */
#define MU7__MUA_RR8         MU7__MUA.RR[8].R              /* Receive Register */
#define MU7__MUA_RR9         MU7__MUA.RR[9].R              /* Receive Register */
#define MU7__MUA_RR10        MU7__MUA.RR[10].R             /* Receive Register */
#define MU7__MUA_RR11        MU7__MUA.RR[11].R             /* Receive Register */
#define MU7__MUA_RR12        MU7__MUA.RR[12].R             /* Receive Register */
#define MU7__MUA_RR13        MU7__MUA.RR[13].R             /* Receive Register */
#define MU7__MUA_RR14        MU7__MUA.RR[14].R             /* Receive Register */
#define MU7__MUA_RR15        MU7__MUA.RR[15].R             /* Receive Register */

#define MU0__MUB_VER         MU0__MUB.VER.R                /* Version ID */
#define MU0__MUB_PAR         MU0__MUB.PAR.R                /* Parameter Register */
#define MU0__MUB_CR          MU0__MUB.CR.R                 /* Control Register */
#define MU0__MUB_SR          MU0__MUB.SR.R                 /* Status Register */
#define MU0__MUB_CCR0        MU0__MUB.CCR0.R               /* Core Control Register 0 */
#define MU0__MUB_CSSR0       MU0__MUB.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU0__MUB_FCR         MU0__MUB.FCR.R                /* Flag Control Register */
#define MU0__MUB_FSR         MU0__MUB.FSR.R                /* Flag Status Register */
#define MU0__MUB_GIER        MU0__MUB.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU0__MUB_GCR         MU0__MUB.GCR.R                /* General-purpose Control Register */
#define MU0__MUB_GSR         MU0__MUB.GSR.R                /* General-purpose Status Register */
#define MU0__MUB_TCR         MU0__MUB.TCR.R                /* Transmit Control Register */
#define MU0__MUB_TSR         MU0__MUB.TSR.R                /* Transmit Status Register */
#define MU0__MUB_RCR         MU0__MUB.RCR.R                /* Receive Control Register */
#define MU0__MUB_RSR         MU0__MUB.RSR.R                /* Receive Status Register */
#define MU0__MUB_TR0         MU0__MUB.TR[0].R              /* Transmit Register */
#define MU0__MUB_TR1         MU0__MUB.TR[1].R              /* Transmit Register */
#define MU0__MUB_TR2         MU0__MUB.TR[2].R              /* Transmit Register */
#define MU0__MUB_TR3         MU0__MUB.TR[3].R              /* Transmit Register */
#define MU0__MUB_TR4         MU0__MUB.TR[4].R              /* Transmit Register */
#define MU0__MUB_TR5         MU0__MUB.TR[5].R              /* Transmit Register */
#define MU0__MUB_TR6         MU0__MUB.TR[6].R              /* Transmit Register */
#define MU0__MUB_TR7         MU0__MUB.TR[7].R              /* Transmit Register */
#define MU0__MUB_TR8         MU0__MUB.TR[8].R              /* Transmit Register */
#define MU0__MUB_TR9         MU0__MUB.TR[9].R              /* Transmit Register */
#define MU0__MUB_TR10        MU0__MUB.TR[10].R             /* Transmit Register */
#define MU0__MUB_TR11        MU0__MUB.TR[11].R             /* Transmit Register */
#define MU0__MUB_TR12        MU0__MUB.TR[12].R             /* Transmit Register */
#define MU0__MUB_TR13        MU0__MUB.TR[13].R             /* Transmit Register */
#define MU0__MUB_TR14        MU0__MUB.TR[14].R             /* Transmit Register */
#define MU0__MUB_TR15        MU0__MUB.TR[15].R             /* Transmit Register */
#define MU0__MUB_RR0         MU0__MUB.RR[0].R              /* Receive Register */
#define MU0__MUB_RR1         MU0__MUB.RR[1].R              /* Receive Register */
#define MU0__MUB_RR2         MU0__MUB.RR[2].R              /* Receive Register */
#define MU0__MUB_RR3         MU0__MUB.RR[3].R              /* Receive Register */
#define MU0__MUB_RR4         MU0__MUB.RR[4].R              /* Receive Register */
#define MU0__MUB_RR5         MU0__MUB.RR[5].R              /* Receive Register */
#define MU0__MUB_RR6         MU0__MUB.RR[6].R              /* Receive Register */
#define MU0__MUB_RR7         MU0__MUB.RR[7].R              /* Receive Register */
#define MU0__MUB_RR8         MU0__MUB.RR[8].R              /* Receive Register */
#define MU0__MUB_RR9         MU0__MUB.RR[9].R              /* Receive Register */
#define MU0__MUB_RR10        MU0__MUB.RR[10].R             /* Receive Register */
#define MU0__MUB_RR11        MU0__MUB.RR[11].R             /* Receive Register */
#define MU0__MUB_RR12        MU0__MUB.RR[12].R             /* Receive Register */
#define MU0__MUB_RR13        MU0__MUB.RR[13].R             /* Receive Register */
#define MU0__MUB_RR14        MU0__MUB.RR[14].R             /* Receive Register */
#define MU0__MUB_RR15        MU0__MUB.RR[15].R             /* Receive Register */

#define MU1__MUB_VER         MU1__MUB.VER.R                /* Version ID */
#define MU1__MUB_PAR         MU1__MUB.PAR.R                /* Parameter Register */
#define MU1__MUB_CR          MU1__MUB.CR.R                 /* Control Register */
#define MU1__MUB_SR          MU1__MUB.SR.R                 /* Status Register */
#define MU1__MUB_CCR0        MU1__MUB.CCR0.R               /* Core Control Register 0 */
#define MU1__MUB_CSSR0       MU1__MUB.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU1__MUB_FCR         MU1__MUB.FCR.R                /* Flag Control Register */
#define MU1__MUB_FSR         MU1__MUB.FSR.R                /* Flag Status Register */
#define MU1__MUB_GIER        MU1__MUB.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU1__MUB_GCR         MU1__MUB.GCR.R                /* General-purpose Control Register */
#define MU1__MUB_GSR         MU1__MUB.GSR.R                /* General-purpose Status Register */
#define MU1__MUB_TCR         MU1__MUB.TCR.R                /* Transmit Control Register */
#define MU1__MUB_TSR         MU1__MUB.TSR.R                /* Transmit Status Register */
#define MU1__MUB_RCR         MU1__MUB.RCR.R                /* Receive Control Register */
#define MU1__MUB_RSR         MU1__MUB.RSR.R                /* Receive Status Register */
#define MU1__MUB_TR0         MU1__MUB.TR[0].R              /* Transmit Register */
#define MU1__MUB_TR1         MU1__MUB.TR[1].R              /* Transmit Register */
#define MU1__MUB_TR2         MU1__MUB.TR[2].R              /* Transmit Register */
#define MU1__MUB_TR3         MU1__MUB.TR[3].R              /* Transmit Register */
#define MU1__MUB_TR4         MU1__MUB.TR[4].R              /* Transmit Register */
#define MU1__MUB_TR5         MU1__MUB.TR[5].R              /* Transmit Register */
#define MU1__MUB_TR6         MU1__MUB.TR[6].R              /* Transmit Register */
#define MU1__MUB_TR7         MU1__MUB.TR[7].R              /* Transmit Register */
#define MU1__MUB_TR8         MU1__MUB.TR[8].R              /* Transmit Register */
#define MU1__MUB_TR9         MU1__MUB.TR[9].R              /* Transmit Register */
#define MU1__MUB_TR10        MU1__MUB.TR[10].R             /* Transmit Register */
#define MU1__MUB_TR11        MU1__MUB.TR[11].R             /* Transmit Register */
#define MU1__MUB_TR12        MU1__MUB.TR[12].R             /* Transmit Register */
#define MU1__MUB_TR13        MU1__MUB.TR[13].R             /* Transmit Register */
#define MU1__MUB_TR14        MU1__MUB.TR[14].R             /* Transmit Register */
#define MU1__MUB_TR15        MU1__MUB.TR[15].R             /* Transmit Register */
#define MU1__MUB_RR0         MU1__MUB.RR[0].R              /* Receive Register */
#define MU1__MUB_RR1         MU1__MUB.RR[1].R              /* Receive Register */
#define MU1__MUB_RR2         MU1__MUB.RR[2].R              /* Receive Register */
#define MU1__MUB_RR3         MU1__MUB.RR[3].R              /* Receive Register */
#define MU1__MUB_RR4         MU1__MUB.RR[4].R              /* Receive Register */
#define MU1__MUB_RR5         MU1__MUB.RR[5].R              /* Receive Register */
#define MU1__MUB_RR6         MU1__MUB.RR[6].R              /* Receive Register */
#define MU1__MUB_RR7         MU1__MUB.RR[7].R              /* Receive Register */
#define MU1__MUB_RR8         MU1__MUB.RR[8].R              /* Receive Register */
#define MU1__MUB_RR9         MU1__MUB.RR[9].R              /* Receive Register */
#define MU1__MUB_RR10        MU1__MUB.RR[10].R             /* Receive Register */
#define MU1__MUB_RR11        MU1__MUB.RR[11].R             /* Receive Register */
#define MU1__MUB_RR12        MU1__MUB.RR[12].R             /* Receive Register */
#define MU1__MUB_RR13        MU1__MUB.RR[13].R             /* Receive Register */
#define MU1__MUB_RR14        MU1__MUB.RR[14].R             /* Receive Register */
#define MU1__MUB_RR15        MU1__MUB.RR[15].R             /* Receive Register */

#define MU2__MUB_VER         MU2__MUB.VER.R                /* Version ID */
#define MU2__MUB_PAR         MU2__MUB.PAR.R                /* Parameter Register */
#define MU2__MUB_CR          MU2__MUB.CR.R                 /* Control Register */
#define MU2__MUB_SR          MU2__MUB.SR.R                 /* Status Register */
#define MU2__MUB_CCR0        MU2__MUB.CCR0.R               /* Core Control Register 0 */
#define MU2__MUB_CSSR0       MU2__MUB.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU2__MUB_FCR         MU2__MUB.FCR.R                /* Flag Control Register */
#define MU2__MUB_FSR         MU2__MUB.FSR.R                /* Flag Status Register */
#define MU2__MUB_GIER        MU2__MUB.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU2__MUB_GCR         MU2__MUB.GCR.R                /* General-purpose Control Register */
#define MU2__MUB_GSR         MU2__MUB.GSR.R                /* General-purpose Status Register */
#define MU2__MUB_TCR         MU2__MUB.TCR.R                /* Transmit Control Register */
#define MU2__MUB_TSR         MU2__MUB.TSR.R                /* Transmit Status Register */
#define MU2__MUB_RCR         MU2__MUB.RCR.R                /* Receive Control Register */
#define MU2__MUB_RSR         MU2__MUB.RSR.R                /* Receive Status Register */
#define MU2__MUB_TR0         MU2__MUB.TR[0].R              /* Transmit Register */
#define MU2__MUB_TR1         MU2__MUB.TR[1].R              /* Transmit Register */
#define MU2__MUB_TR2         MU2__MUB.TR[2].R              /* Transmit Register */
#define MU2__MUB_TR3         MU2__MUB.TR[3].R              /* Transmit Register */
#define MU2__MUB_TR4         MU2__MUB.TR[4].R              /* Transmit Register */
#define MU2__MUB_TR5         MU2__MUB.TR[5].R              /* Transmit Register */
#define MU2__MUB_TR6         MU2__MUB.TR[6].R              /* Transmit Register */
#define MU2__MUB_TR7         MU2__MUB.TR[7].R              /* Transmit Register */
#define MU2__MUB_TR8         MU2__MUB.TR[8].R              /* Transmit Register */
#define MU2__MUB_TR9         MU2__MUB.TR[9].R              /* Transmit Register */
#define MU2__MUB_TR10        MU2__MUB.TR[10].R             /* Transmit Register */
#define MU2__MUB_TR11        MU2__MUB.TR[11].R             /* Transmit Register */
#define MU2__MUB_TR12        MU2__MUB.TR[12].R             /* Transmit Register */
#define MU2__MUB_TR13        MU2__MUB.TR[13].R             /* Transmit Register */
#define MU2__MUB_TR14        MU2__MUB.TR[14].R             /* Transmit Register */
#define MU2__MUB_TR15        MU2__MUB.TR[15].R             /* Transmit Register */
#define MU2__MUB_RR0         MU2__MUB.RR[0].R              /* Receive Register */
#define MU2__MUB_RR1         MU2__MUB.RR[1].R              /* Receive Register */
#define MU2__MUB_RR2         MU2__MUB.RR[2].R              /* Receive Register */
#define MU2__MUB_RR3         MU2__MUB.RR[3].R              /* Receive Register */
#define MU2__MUB_RR4         MU2__MUB.RR[4].R              /* Receive Register */
#define MU2__MUB_RR5         MU2__MUB.RR[5].R              /* Receive Register */
#define MU2__MUB_RR6         MU2__MUB.RR[6].R              /* Receive Register */
#define MU2__MUB_RR7         MU2__MUB.RR[7].R              /* Receive Register */
#define MU2__MUB_RR8         MU2__MUB.RR[8].R              /* Receive Register */
#define MU2__MUB_RR9         MU2__MUB.RR[9].R              /* Receive Register */
#define MU2__MUB_RR10        MU2__MUB.RR[10].R             /* Receive Register */
#define MU2__MUB_RR11        MU2__MUB.RR[11].R             /* Receive Register */
#define MU2__MUB_RR12        MU2__MUB.RR[12].R             /* Receive Register */
#define MU2__MUB_RR13        MU2__MUB.RR[13].R             /* Receive Register */
#define MU2__MUB_RR14        MU2__MUB.RR[14].R             /* Receive Register */
#define MU2__MUB_RR15        MU2__MUB.RR[15].R             /* Receive Register */

#define MU3__MUB_VER         MU3__MUB.VER.R                /* Version ID */
#define MU3__MUB_PAR         MU3__MUB.PAR.R                /* Parameter Register */
#define MU3__MUB_CR          MU3__MUB.CR.R                 /* Control Register */
#define MU3__MUB_SR          MU3__MUB.SR.R                 /* Status Register */
#define MU3__MUB_CCR0        MU3__MUB.CCR0.R               /* Core Control Register 0 */
#define MU3__MUB_CSSR0       MU3__MUB.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU3__MUB_FCR         MU3__MUB.FCR.R                /* Flag Control Register */
#define MU3__MUB_FSR         MU3__MUB.FSR.R                /* Flag Status Register */
#define MU3__MUB_GIER        MU3__MUB.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU3__MUB_GCR         MU3__MUB.GCR.R                /* General-purpose Control Register */
#define MU3__MUB_GSR         MU3__MUB.GSR.R                /* General-purpose Status Register */
#define MU3__MUB_TCR         MU3__MUB.TCR.R                /* Transmit Control Register */
#define MU3__MUB_TSR         MU3__MUB.TSR.R                /* Transmit Status Register */
#define MU3__MUB_RCR         MU3__MUB.RCR.R                /* Receive Control Register */
#define MU3__MUB_RSR         MU3__MUB.RSR.R                /* Receive Status Register */
#define MU3__MUB_TR0         MU3__MUB.TR[0].R              /* Transmit Register */
#define MU3__MUB_TR1         MU3__MUB.TR[1].R              /* Transmit Register */
#define MU3__MUB_TR2         MU3__MUB.TR[2].R              /* Transmit Register */
#define MU3__MUB_TR3         MU3__MUB.TR[3].R              /* Transmit Register */
#define MU3__MUB_TR4         MU3__MUB.TR[4].R              /* Transmit Register */
#define MU3__MUB_TR5         MU3__MUB.TR[5].R              /* Transmit Register */
#define MU3__MUB_TR6         MU3__MUB.TR[6].R              /* Transmit Register */
#define MU3__MUB_TR7         MU3__MUB.TR[7].R              /* Transmit Register */
#define MU3__MUB_TR8         MU3__MUB.TR[8].R              /* Transmit Register */
#define MU3__MUB_TR9         MU3__MUB.TR[9].R              /* Transmit Register */
#define MU3__MUB_TR10        MU3__MUB.TR[10].R             /* Transmit Register */
#define MU3__MUB_TR11        MU3__MUB.TR[11].R             /* Transmit Register */
#define MU3__MUB_TR12        MU3__MUB.TR[12].R             /* Transmit Register */
#define MU3__MUB_TR13        MU3__MUB.TR[13].R             /* Transmit Register */
#define MU3__MUB_TR14        MU3__MUB.TR[14].R             /* Transmit Register */
#define MU3__MUB_TR15        MU3__MUB.TR[15].R             /* Transmit Register */
#define MU3__MUB_RR0         MU3__MUB.RR[0].R              /* Receive Register */
#define MU3__MUB_RR1         MU3__MUB.RR[1].R              /* Receive Register */
#define MU3__MUB_RR2         MU3__MUB.RR[2].R              /* Receive Register */
#define MU3__MUB_RR3         MU3__MUB.RR[3].R              /* Receive Register */
#define MU3__MUB_RR4         MU3__MUB.RR[4].R              /* Receive Register */
#define MU3__MUB_RR5         MU3__MUB.RR[5].R              /* Receive Register */
#define MU3__MUB_RR6         MU3__MUB.RR[6].R              /* Receive Register */
#define MU3__MUB_RR7         MU3__MUB.RR[7].R              /* Receive Register */
#define MU3__MUB_RR8         MU3__MUB.RR[8].R              /* Receive Register */
#define MU3__MUB_RR9         MU3__MUB.RR[9].R              /* Receive Register */
#define MU3__MUB_RR10        MU3__MUB.RR[10].R             /* Receive Register */
#define MU3__MUB_RR11        MU3__MUB.RR[11].R             /* Receive Register */
#define MU3__MUB_RR12        MU3__MUB.RR[12].R             /* Receive Register */
#define MU3__MUB_RR13        MU3__MUB.RR[13].R             /* Receive Register */
#define MU3__MUB_RR14        MU3__MUB.RR[14].R             /* Receive Register */
#define MU3__MUB_RR15        MU3__MUB.RR[15].R             /* Receive Register */

#define MU4__MUB_VER         MU4__MUB.VER.R                /* Version ID */
#define MU4__MUB_PAR         MU4__MUB.PAR.R                /* Parameter Register */
#define MU4__MUB_CR          MU4__MUB.CR.R                 /* Control Register */
#define MU4__MUB_SR          MU4__MUB.SR.R                 /* Status Register */
#define MU4__MUB_CCR0        MU4__MUB.CCR0.R               /* Core Control Register 0 */
#define MU4__MUB_CSSR0       MU4__MUB.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU4__MUB_FCR         MU4__MUB.FCR.R                /* Flag Control Register */
#define MU4__MUB_FSR         MU4__MUB.FSR.R                /* Flag Status Register */
#define MU4__MUB_GIER        MU4__MUB.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU4__MUB_GCR         MU4__MUB.GCR.R                /* General-purpose Control Register */
#define MU4__MUB_GSR         MU4__MUB.GSR.R                /* General-purpose Status Register */
#define MU4__MUB_TCR         MU4__MUB.TCR.R                /* Transmit Control Register */
#define MU4__MUB_TSR         MU4__MUB.TSR.R                /* Transmit Status Register */
#define MU4__MUB_RCR         MU4__MUB.RCR.R                /* Receive Control Register */
#define MU4__MUB_RSR         MU4__MUB.RSR.R                /* Receive Status Register */
#define MU4__MUB_TR0         MU4__MUB.TR[0].R              /* Transmit Register */
#define MU4__MUB_TR1         MU4__MUB.TR[1].R              /* Transmit Register */
#define MU4__MUB_TR2         MU4__MUB.TR[2].R              /* Transmit Register */
#define MU4__MUB_TR3         MU4__MUB.TR[3].R              /* Transmit Register */
#define MU4__MUB_TR4         MU4__MUB.TR[4].R              /* Transmit Register */
#define MU4__MUB_TR5         MU4__MUB.TR[5].R              /* Transmit Register */
#define MU4__MUB_TR6         MU4__MUB.TR[6].R              /* Transmit Register */
#define MU4__MUB_TR7         MU4__MUB.TR[7].R              /* Transmit Register */
#define MU4__MUB_TR8         MU4__MUB.TR[8].R              /* Transmit Register */
#define MU4__MUB_TR9         MU4__MUB.TR[9].R              /* Transmit Register */
#define MU4__MUB_TR10        MU4__MUB.TR[10].R             /* Transmit Register */
#define MU4__MUB_TR11        MU4__MUB.TR[11].R             /* Transmit Register */
#define MU4__MUB_TR12        MU4__MUB.TR[12].R             /* Transmit Register */
#define MU4__MUB_TR13        MU4__MUB.TR[13].R             /* Transmit Register */
#define MU4__MUB_TR14        MU4__MUB.TR[14].R             /* Transmit Register */
#define MU4__MUB_TR15        MU4__MUB.TR[15].R             /* Transmit Register */
#define MU4__MUB_RR0         MU4__MUB.RR[0].R              /* Receive Register */
#define MU4__MUB_RR1         MU4__MUB.RR[1].R              /* Receive Register */
#define MU4__MUB_RR2         MU4__MUB.RR[2].R              /* Receive Register */
#define MU4__MUB_RR3         MU4__MUB.RR[3].R              /* Receive Register */
#define MU4__MUB_RR4         MU4__MUB.RR[4].R              /* Receive Register */
#define MU4__MUB_RR5         MU4__MUB.RR[5].R              /* Receive Register */
#define MU4__MUB_RR6         MU4__MUB.RR[6].R              /* Receive Register */
#define MU4__MUB_RR7         MU4__MUB.RR[7].R              /* Receive Register */
#define MU4__MUB_RR8         MU4__MUB.RR[8].R              /* Receive Register */
#define MU4__MUB_RR9         MU4__MUB.RR[9].R              /* Receive Register */
#define MU4__MUB_RR10        MU4__MUB.RR[10].R             /* Receive Register */
#define MU4__MUB_RR11        MU4__MUB.RR[11].R             /* Receive Register */
#define MU4__MUB_RR12        MU4__MUB.RR[12].R             /* Receive Register */
#define MU4__MUB_RR13        MU4__MUB.RR[13].R             /* Receive Register */
#define MU4__MUB_RR14        MU4__MUB.RR[14].R             /* Receive Register */
#define MU4__MUB_RR15        MU4__MUB.RR[15].R             /* Receive Register */

#define MU5__MUB_VER         MU5__MUB.VER.R                /* Version ID */
#define MU5__MUB_PAR         MU5__MUB.PAR.R                /* Parameter Register */
#define MU5__MUB_CR          MU5__MUB.CR.R                 /* Control Register */
#define MU5__MUB_SR          MU5__MUB.SR.R                 /* Status Register */
#define MU5__MUB_CCR0        MU5__MUB.CCR0.R               /* Core Control Register 0 */
#define MU5__MUB_CSSR0       MU5__MUB.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU5__MUB_FCR         MU5__MUB.FCR.R                /* Flag Control Register */
#define MU5__MUB_FSR         MU5__MUB.FSR.R                /* Flag Status Register */
#define MU5__MUB_GIER        MU5__MUB.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU5__MUB_GCR         MU5__MUB.GCR.R                /* General-purpose Control Register */
#define MU5__MUB_GSR         MU5__MUB.GSR.R                /* General-purpose Status Register */
#define MU5__MUB_TCR         MU5__MUB.TCR.R                /* Transmit Control Register */
#define MU5__MUB_TSR         MU5__MUB.TSR.R                /* Transmit Status Register */
#define MU5__MUB_RCR         MU5__MUB.RCR.R                /* Receive Control Register */
#define MU5__MUB_RSR         MU5__MUB.RSR.R                /* Receive Status Register */
#define MU5__MUB_TR0         MU5__MUB.TR[0].R              /* Transmit Register */
#define MU5__MUB_TR1         MU5__MUB.TR[1].R              /* Transmit Register */
#define MU5__MUB_TR2         MU5__MUB.TR[2].R              /* Transmit Register */
#define MU5__MUB_TR3         MU5__MUB.TR[3].R              /* Transmit Register */
#define MU5__MUB_TR4         MU5__MUB.TR[4].R              /* Transmit Register */
#define MU5__MUB_TR5         MU5__MUB.TR[5].R              /* Transmit Register */
#define MU5__MUB_TR6         MU5__MUB.TR[6].R              /* Transmit Register */
#define MU5__MUB_TR7         MU5__MUB.TR[7].R              /* Transmit Register */
#define MU5__MUB_TR8         MU5__MUB.TR[8].R              /* Transmit Register */
#define MU5__MUB_TR9         MU5__MUB.TR[9].R              /* Transmit Register */
#define MU5__MUB_TR10        MU5__MUB.TR[10].R             /* Transmit Register */
#define MU5__MUB_TR11        MU5__MUB.TR[11].R             /* Transmit Register */
#define MU5__MUB_TR12        MU5__MUB.TR[12].R             /* Transmit Register */
#define MU5__MUB_TR13        MU5__MUB.TR[13].R             /* Transmit Register */
#define MU5__MUB_TR14        MU5__MUB.TR[14].R             /* Transmit Register */
#define MU5__MUB_TR15        MU5__MUB.TR[15].R             /* Transmit Register */
#define MU5__MUB_RR0         MU5__MUB.RR[0].R              /* Receive Register */
#define MU5__MUB_RR1         MU5__MUB.RR[1].R              /* Receive Register */
#define MU5__MUB_RR2         MU5__MUB.RR[2].R              /* Receive Register */
#define MU5__MUB_RR3         MU5__MUB.RR[3].R              /* Receive Register */
#define MU5__MUB_RR4         MU5__MUB.RR[4].R              /* Receive Register */
#define MU5__MUB_RR5         MU5__MUB.RR[5].R              /* Receive Register */
#define MU5__MUB_RR6         MU5__MUB.RR[6].R              /* Receive Register */
#define MU5__MUB_RR7         MU5__MUB.RR[7].R              /* Receive Register */
#define MU5__MUB_RR8         MU5__MUB.RR[8].R              /* Receive Register */
#define MU5__MUB_RR9         MU5__MUB.RR[9].R              /* Receive Register */
#define MU5__MUB_RR10        MU5__MUB.RR[10].R             /* Receive Register */
#define MU5__MUB_RR11        MU5__MUB.RR[11].R             /* Receive Register */
#define MU5__MUB_RR12        MU5__MUB.RR[12].R             /* Receive Register */
#define MU5__MUB_RR13        MU5__MUB.RR[13].R             /* Receive Register */
#define MU5__MUB_RR14        MU5__MUB.RR[14].R             /* Receive Register */
#define MU5__MUB_RR15        MU5__MUB.RR[15].R             /* Receive Register */

#define MU6__MUB_VER         MU6__MUB.VER.R                /* Version ID */
#define MU6__MUB_PAR         MU6__MUB.PAR.R                /* Parameter Register */
#define MU6__MUB_CR          MU6__MUB.CR.R                 /* Control Register */
#define MU6__MUB_SR          MU6__MUB.SR.R                 /* Status Register */
#define MU6__MUB_CCR0        MU6__MUB.CCR0.R               /* Core Control Register 0 */
#define MU6__MUB_CSSR0       MU6__MUB.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU6__MUB_FCR         MU6__MUB.FCR.R                /* Flag Control Register */
#define MU6__MUB_FSR         MU6__MUB.FSR.R                /* Flag Status Register */
#define MU6__MUB_GIER        MU6__MUB.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU6__MUB_GCR         MU6__MUB.GCR.R                /* General-purpose Control Register */
#define MU6__MUB_GSR         MU6__MUB.GSR.R                /* General-purpose Status Register */
#define MU6__MUB_TCR         MU6__MUB.TCR.R                /* Transmit Control Register */
#define MU6__MUB_TSR         MU6__MUB.TSR.R                /* Transmit Status Register */
#define MU6__MUB_RCR         MU6__MUB.RCR.R                /* Receive Control Register */
#define MU6__MUB_RSR         MU6__MUB.RSR.R                /* Receive Status Register */
#define MU6__MUB_TR0         MU6__MUB.TR[0].R              /* Transmit Register */
#define MU6__MUB_TR1         MU6__MUB.TR[1].R              /* Transmit Register */
#define MU6__MUB_TR2         MU6__MUB.TR[2].R              /* Transmit Register */
#define MU6__MUB_TR3         MU6__MUB.TR[3].R              /* Transmit Register */
#define MU6__MUB_TR4         MU6__MUB.TR[4].R              /* Transmit Register */
#define MU6__MUB_TR5         MU6__MUB.TR[5].R              /* Transmit Register */
#define MU6__MUB_TR6         MU6__MUB.TR[6].R              /* Transmit Register */
#define MU6__MUB_TR7         MU6__MUB.TR[7].R              /* Transmit Register */
#define MU6__MUB_TR8         MU6__MUB.TR[8].R              /* Transmit Register */
#define MU6__MUB_TR9         MU6__MUB.TR[9].R              /* Transmit Register */
#define MU6__MUB_TR10        MU6__MUB.TR[10].R             /* Transmit Register */
#define MU6__MUB_TR11        MU6__MUB.TR[11].R             /* Transmit Register */
#define MU6__MUB_TR12        MU6__MUB.TR[12].R             /* Transmit Register */
#define MU6__MUB_TR13        MU6__MUB.TR[13].R             /* Transmit Register */
#define MU6__MUB_TR14        MU6__MUB.TR[14].R             /* Transmit Register */
#define MU6__MUB_TR15        MU6__MUB.TR[15].R             /* Transmit Register */
#define MU6__MUB_RR0         MU6__MUB.RR[0].R              /* Receive Register */
#define MU6__MUB_RR1         MU6__MUB.RR[1].R              /* Receive Register */
#define MU6__MUB_RR2         MU6__MUB.RR[2].R              /* Receive Register */
#define MU6__MUB_RR3         MU6__MUB.RR[3].R              /* Receive Register */
#define MU6__MUB_RR4         MU6__MUB.RR[4].R              /* Receive Register */
#define MU6__MUB_RR5         MU6__MUB.RR[5].R              /* Receive Register */
#define MU6__MUB_RR6         MU6__MUB.RR[6].R              /* Receive Register */
#define MU6__MUB_RR7         MU6__MUB.RR[7].R              /* Receive Register */
#define MU6__MUB_RR8         MU6__MUB.RR[8].R              /* Receive Register */
#define MU6__MUB_RR9         MU6__MUB.RR[9].R              /* Receive Register */
#define MU6__MUB_RR10        MU6__MUB.RR[10].R             /* Receive Register */
#define MU6__MUB_RR11        MU6__MUB.RR[11].R             /* Receive Register */
#define MU6__MUB_RR12        MU6__MUB.RR[12].R             /* Receive Register */
#define MU6__MUB_RR13        MU6__MUB.RR[13].R             /* Receive Register */
#define MU6__MUB_RR14        MU6__MUB.RR[14].R             /* Receive Register */
#define MU6__MUB_RR15        MU6__MUB.RR[15].R             /* Receive Register */

#define MU7__MUB_VER         MU7__MUB.VER.R                /* Version ID */
#define MU7__MUB_PAR         MU7__MUB.PAR.R                /* Parameter Register */
#define MU7__MUB_CR          MU7__MUB.CR.R                 /* Control Register */
#define MU7__MUB_SR          MU7__MUB.SR.R                 /* Status Register */
#define MU7__MUB_CCR0        MU7__MUB.CCR0.R               /* Core Control Register 0 */
#define MU7__MUB_CSSR0       MU7__MUB.CSSR0.R              /* Core Sticky Status Register 0 */
#define MU7__MUB_FCR         MU7__MUB.FCR.R                /* Flag Control Register */
#define MU7__MUB_FSR         MU7__MUB.FSR.R                /* Flag Status Register */
#define MU7__MUB_GIER        MU7__MUB.GIER.R               /* General-purpose Interrupt Enable Register */
#define MU7__MUB_GCR         MU7__MUB.GCR.R                /* General-purpose Control Register */
#define MU7__MUB_GSR         MU7__MUB.GSR.R                /* General-purpose Status Register */
#define MU7__MUB_TCR         MU7__MUB.TCR.R                /* Transmit Control Register */
#define MU7__MUB_TSR         MU7__MUB.TSR.R                /* Transmit Status Register */
#define MU7__MUB_RCR         MU7__MUB.RCR.R                /* Receive Control Register */
#define MU7__MUB_RSR         MU7__MUB.RSR.R                /* Receive Status Register */
#define MU7__MUB_TR0         MU7__MUB.TR[0].R              /* Transmit Register */
#define MU7__MUB_TR1         MU7__MUB.TR[1].R              /* Transmit Register */
#define MU7__MUB_TR2         MU7__MUB.TR[2].R              /* Transmit Register */
#define MU7__MUB_TR3         MU7__MUB.TR[3].R              /* Transmit Register */
#define MU7__MUB_TR4         MU7__MUB.TR[4].R              /* Transmit Register */
#define MU7__MUB_TR5         MU7__MUB.TR[5].R              /* Transmit Register */
#define MU7__MUB_TR6         MU7__MUB.TR[6].R              /* Transmit Register */
#define MU7__MUB_TR7         MU7__MUB.TR[7].R              /* Transmit Register */
#define MU7__MUB_TR8         MU7__MUB.TR[8].R              /* Transmit Register */
#define MU7__MUB_TR9         MU7__MUB.TR[9].R              /* Transmit Register */
#define MU7__MUB_TR10        MU7__MUB.TR[10].R             /* Transmit Register */
#define MU7__MUB_TR11        MU7__MUB.TR[11].R             /* Transmit Register */
#define MU7__MUB_TR12        MU7__MUB.TR[12].R             /* Transmit Register */
#define MU7__MUB_TR13        MU7__MUB.TR[13].R             /* Transmit Register */
#define MU7__MUB_TR14        MU7__MUB.TR[14].R             /* Transmit Register */
#define MU7__MUB_TR15        MU7__MUB.TR[15].R             /* Transmit Register */
#define MU7__MUB_RR0         MU7__MUB.RR[0].R              /* Receive Register */
#define MU7__MUB_RR1         MU7__MUB.RR[1].R              /* Receive Register */
#define MU7__MUB_RR2         MU7__MUB.RR[2].R              /* Receive Register */
#define MU7__MUB_RR3         MU7__MUB.RR[3].R              /* Receive Register */
#define MU7__MUB_RR4         MU7__MUB.RR[4].R              /* Receive Register */
#define MU7__MUB_RR5         MU7__MUB.RR[5].R              /* Receive Register */
#define MU7__MUB_RR6         MU7__MUB.RR[6].R              /* Receive Register */
#define MU7__MUB_RR7         MU7__MUB.RR[7].R              /* Receive Register */
#define MU7__MUB_RR8         MU7__MUB.RR[8].R              /* Receive Register */
#define MU7__MUB_RR9         MU7__MUB.RR[9].R              /* Receive Register */
#define MU7__MUB_RR10        MU7__MUB.RR[10].R             /* Receive Register */
#define MU7__MUB_RR11        MU7__MUB.RR[11].R             /* Receive Register */
#define MU7__MUB_RR12        MU7__MUB.RR[12].R             /* Receive Register */
#define MU7__MUB_RR13        MU7__MUB.RR[13].R             /* Receive Register */
#define MU7__MUB_RR14        MU7__MUB.RR[14].R             /* Receive Register */
#define MU7__MUB_RR15        MU7__MUB.RR[15].R             /* Receive Register */

/* MULTICORE_CONFIGURATION */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_REGx0 CEVA_SPF2__MULTICORE_CONFIGURATION.COM_REGX[0].R /* MCCI Message Register */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_REGx1 CEVA_SPF2__MULTICORE_CONFIGURATION.COM_REGX[1].R /* MCCI Message Register */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_REGx2 CEVA_SPF2__MULTICORE_CONFIGURATION.COM_REGX[2].R /* MCCI Message Register */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_REGx3 CEVA_SPF2__MULTICORE_CONFIGURATION.COM_REGX[3].R /* MCCI Message Register */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_REGx4 CEVA_SPF2__MULTICORE_CONFIGURATION.COM_REGX[4].R /* MCCI Message Register */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_REGx5 CEVA_SPF2__MULTICORE_CONFIGURATION.COM_REGX[5].R /* MCCI Message Register */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_REGx6 CEVA_SPF2__MULTICORE_CONFIGURATION.COM_REGX[6].R /* MCCI Message Register */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_REGx7 CEVA_SPF2__MULTICORE_CONFIGURATION.COM_REGX[7].R /* MCCI Message Register */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_STS CEVA_SPF2__MULTICORE_CONFIGURATION.COM_STS.R /* MCCI Status */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_COM_INT_EN CEVA_SPF2__MULTICORE_CONFIGURATION.COM_INT_EN.R /* MCCI Interrupt Enable */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_START_ADDR0 CEVA_SPF2__MULTICORE_CONFIGURATION.START_ADDR0.R /* IDM Snoop Start Address 0 */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_TOP_ADDR0 CEVA_SPF2__MULTICORE_CONFIGURATION.TOP_ADDR0.R /* IDM Snoop End Address 0 */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_SNOOP_CSR0 CEVA_SPF2__MULTICORE_CONFIGURATION.SNOOP_CSR0.R /* IDM Snoop Control 0 */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_START_ADDR1 CEVA_SPF2__MULTICORE_CONFIGURATION.START_ADDR1.R /* IDM Snoop Start Address 1 */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_TOP_ADDR1 CEVA_SPF2__MULTICORE_CONFIGURATION.TOP_ADDR1.R /* IDM Snoop End Address 1 */
#define CEVA_SPF2__MULTICORE_CONFIGURATION_SNOOP_CSR1 CEVA_SPF2__MULTICORE_CONFIGURATION.SNOOP_CSR1.R /* IDM Snoop Control 1 */

/* NETC_F0_GLOBAL */
#define NETC__TMR0_GLOBAL_SMCAPR NETC__TMR0_GLOBAL.SMCAPR.R /* Shared memory capability register */
#define NETC__TMR0_GLOBAL_SMDTR NETC__TMR0_GLOBAL.SMDTR.R  /* Shared memory depletion threshold register */
#define NETC__TMR0_GLOBAL_SMACR NETC__TMR0_GLOBAL.SMACR.R  /* Shared memory available count register */
#define NETC__TMR0_GLOBAL_SMCLWMR NETC__TMR0_GLOBAL.SMCLWMR.R /* Shared memory count low watermark register */
#define NETC__TMR0_GLOBAL_SMBUCR NETC__TMR0_GLOBAL.SMBUCR.R /* Shared memory buffer unassigned count register */
#define NETC__TMR0_GLOBAL_SMBUCHWMR NETC__TMR0_GLOBAL.SMBUCHWMR.R /* Shared memory buffer unassigned count high watermark register */
#define NETC__TMR0_GLOBAL_SMLCR NETC__TMR0_GLOBAL.SMLCR.R  /* Shared memory loss count register */
#define NETC__TMR0_GLOBAL_HBTCAPR NETC__TMR0_GLOBAL.HBTCAPR.R /* Hash bucket table capability register */
#define NETC__TMR0_GLOBAL_HBTOR0 NETC__TMR0_GLOBAL.HBTOR0.R /* Hash bucket table operational register 0 */
#define NETC__TMR0_GLOBAL_HBTOR2 NETC__TMR0_GLOBAL.HBTOR2.R /* Hash bucket table operational register 2 */
#define NETC__TMR0_GLOBAL_SMERBCAPR NETC__TMR0_GLOBAL.SMERBCAPR.R /* Shared memory ENETC receive buffer capability register */
#define NETC__TMR0_GLOBAL_SMERBOR0 NETC__TMR0_GLOBAL.SMERBOR0.R /* Shared memory ENETC receive buffer operational register 0 */
#define NETC__TMR0_GLOBAL_SMERBOR1 NETC__TMR0_GLOBAL.SMERBOR1.R /* Shared memory ENETC receive buffer operational 1 */
#define NETC__TMR0_GLOBAL_PCE0OR NETC__TMR0_GLOBAL.PCE_SL[0].PCEOR.R /* PCE 0 operational register */
#define NETC__TMR0_GLOBAL_RFE0OR NETC__TMR0_GLOBAL.PCE_SL[0].RFEOR.R /* Replication Forwarding Engine 0 operational register */
#define NETC__TMR0_GLOBAL_NETCCLKR NETC__TMR0_GLOBAL.NETCCLKR.R /* NETC clock register */
#define NETC__TMR0_GLOBAL_HTA0CAPR NETC__TMR0_GLOBAL.HTA_LOOP[0].HTACAPR.R /* HTA 0 capability register */
#define NETC__TMR0_GLOBAL_HTA0RFCOR NETC__TMR0_GLOBAL.HTA_LOOP[0].HTARFCOR.R /* HTA 0 receive frame count operational register */
#define NETC__TMR0_GLOBAL_HTA0HPBCOR NETC__TMR0_GLOBAL.HTA_LOOP[0].HTAHPBCOR.R /* HTA 0 high priority byte count operational register */
#define NETC__TMR0_GLOBAL_HTA0LPBCOR NETC__TMR0_GLOBAL.HTA_LOOP[0].HTALPBCOR.R /* HTA 0 low priority byte count operational register */
#define NETC__TMR0_GLOBAL_HTA0TFCOR NETC__TMR0_GLOBAL.HTA_LOOP[0].HTATFCOR.R /* HTA 0 transmit frame count operational register */
#define NETC__TMR0_GLOBAL_RC0SBRLAR NETC__TMR0_GLOBAL.ARRAY_NUM_RC[0].RCSBRLAR.R /* Root complex 0 system bus read latency average register */
#define NETC__TMR0_GLOBAL_RC0SBRLHWMR NETC__TMR0_GLOBAL.ARRAY_NUM_RC[0].RCSBRLHWMR.R /* Root complex 0 system bus read latency high watermark register */
#define NETC__TMR0_GLOBAL_RC0SBWLAR NETC__TMR0_GLOBAL.ARRAY_NUM_RC[0].RCSBWLAR.R /* Root complex 0 system bus write latency average register */
#define NETC__TMR0_GLOBAL_RC0SBWLHWMR NETC__TMR0_GLOBAL.ARRAY_NUM_RC[0].RCSBWLHWMR.R /* Root complex 0 system bus write latency high watermark register */
#define NETC__TMR0_GLOBAL_IPBRR0 NETC__TMR0_GLOBAL.IPBRR0.R /* IP block revision register 0 */
#define NETC__TMR0_GLOBAL_IPBRR1 NETC__TMR0_GLOBAL.IPBRR1.R /* IP block revision register 1 */
#define NETC__TMR0_GLOBAL_FBLPR0 NETC__TMR0_GLOBAL.FBLPR[0].R /* Function boot loader parameter register 0 */
#define NETC__TMR0_GLOBAL_FBLPR1 NETC__TMR0_GLOBAL.FBLPR[1].R /* Function boot loader parameter register 1 */
#define NETC__TMR0_GLOBAL_TUFSBECR NETC__TMR0_GLOBAL.TIMER.TUFSBECR.R /* Timer uncorrectable fatal system bus error configuration register */
#define NETC__TMR0_GLOBAL_TUFSBESR NETC__TMR0_GLOBAL.TIMER.TUFSBESR.R /* Timer uncorrectable fatal system bus error status register */

/* NETC_F0_PCI_HDR_TYPE0 */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP_LIST.R /* PCI PCI-PM capabilities list register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP.R /* PCI PCI-PM capabilities register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CTL_STAT.R /* PCI PCI-PM control and status register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIPM_DATA.R /* PCI PCI-PM capabilities data register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_EA_CAP_LIST.R /* PCI EA capabilities list register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_CAP NETC__NETC_F0_PCI_HDR_TYPE0.PCI_CFC_EA_CAP.R /* PCI EA capabilities register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_FMT NETC__NETC_F0_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 0 format register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_BASE NETC__NETC_F0_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 0 base register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_MAXOFF NETC__NETC_F0_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 0 max offset register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_EXT_BASE NETC__NETC_F0_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 0 extended base register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_FMT NETC__NETC_F0_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 1 format register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_BASE NETC__NETC_F0_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 1 base register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_MAXOFF NETC__NETC_F0_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 1 max offset register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_EXT_BASE NETC__NETC_F0_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 1 extended base register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_F0_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_F0_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_F1 */
#define NETC__EMDIO_BASE_EMDIO_CFG NETC__EMDIO_BASE.EMDIO_CFG.R /* External MDIO configuration register */
#define NETC__EMDIO_BASE_EMDIO_CTL NETC__EMDIO_BASE.EMDIO_CTL.R /* External MDIO interface control register */
#define NETC__EMDIO_BASE_EMDIO_DATA NETC__EMDIO_BASE.EMDIO_DATA.R /* External MDIO interface data register */
#define NETC__EMDIO_BASE_EMDIO_ADDR NETC__EMDIO_BASE.EMDIO_ADDR.R /* External MDIO register address register */
#define NETC__EMDIO_BASE_EMDIO_STAT NETC__EMDIO_BASE.EMDIO_STAT.R /* External MDIO status register */
#define NETC__EMDIO_BASE_PHY_STATUS_CFG NETC__EMDIO_BASE.PHY_STATUS_CFG.R /* PHY status configuration register */
#define NETC__EMDIO_BASE_PHY_STATUS_CTL NETC__EMDIO_BASE.PHY_STATUS_CTL.R /* PHY status control register */
#define NETC__EMDIO_BASE_PHY_STATUS_DATA NETC__EMDIO_BASE.PHY_STATUS_DATA.R /* PHY status data register */
#define NETC__EMDIO_BASE_PHY_STATUS_ADDR NETC__EMDIO_BASE.PHY_STATUS_ADDR.R /* PHY status register address register */
#define NETC__EMDIO_BASE_PHY_STATUS_EVENT NETC__EMDIO_BASE.PHY_STATUS_EVENT.R /* PHY status event register */
#define NETC__EMDIO_BASE_PHY_STATUS_MASK NETC__EMDIO_BASE.PHY_STATUS_MASK.R /* PHY status mask register */
#define NETC__EMDIO_BASE_MDIO_CFG NETC__EMDIO_BASE.MDIO_CFG.R /* MDIO configuration register */

/* NETC_F1_GLOBAL */
#define NETC__EMDIO_GLOBAL_SMCAPR NETC__EMDIO_GLOBAL.SMCAPR.R /* Shared memory capability register */
#define NETC__EMDIO_GLOBAL_SMDTR NETC__EMDIO_GLOBAL.SMDTR.R /* Shared memory depletion threshold register */
#define NETC__EMDIO_GLOBAL_SMACR NETC__EMDIO_GLOBAL.SMACR.R /* Shared memory available count register */
#define NETC__EMDIO_GLOBAL_SMCLWMR NETC__EMDIO_GLOBAL.SMCLWMR.R /* Shared memory count low watermark register */
#define NETC__EMDIO_GLOBAL_SMBUCR NETC__EMDIO_GLOBAL.SMBUCR.R /* Shared memory buffer unassigned count register */
#define NETC__EMDIO_GLOBAL_SMBUCHWMR NETC__EMDIO_GLOBAL.SMBUCHWMR.R /* Shared memory buffer unassigned count high watermark register */
#define NETC__EMDIO_GLOBAL_SMLCR NETC__EMDIO_GLOBAL.SMLCR.R /* Shared memory loss count register */
#define NETC__EMDIO_GLOBAL_HBTCAPR NETC__EMDIO_GLOBAL.HBTCAPR.R /* Hash bucket table capability register */
#define NETC__EMDIO_GLOBAL_HBTOR0 NETC__EMDIO_GLOBAL.HBTOR0.R /* Hash bucket table operational register 0 */
#define NETC__EMDIO_GLOBAL_HBTOR2 NETC__EMDIO_GLOBAL.HBTOR2.R /* Hash bucket table operational register 2 */
#define NETC__EMDIO_GLOBAL_SMERBCAPR NETC__EMDIO_GLOBAL.SMERBCAPR.R /* Shared memory ENETC receive buffer capability register */
#define NETC__EMDIO_GLOBAL_SMERBOR0 NETC__EMDIO_GLOBAL.SMERBOR0.R /* Shared memory ENETC receive buffer operational register 0 */
#define NETC__EMDIO_GLOBAL_SMERBOR1 NETC__EMDIO_GLOBAL.SMERBOR1.R /* Shared memory ENETC receive buffer operational 1 */
#define NETC__EMDIO_GLOBAL_PCE0OR NETC__EMDIO_GLOBAL.PCE_SL[0].PCEOR.R /* PCE 0 operational register */
#define NETC__EMDIO_GLOBAL_RFE0OR NETC__EMDIO_GLOBAL.PCE_SL[0].RFEOR.R /* Replication Forwarding Engine 0 operational register */
#define NETC__EMDIO_GLOBAL_NETCCLKR NETC__EMDIO_GLOBAL.NETCCLKR.R /* NETC clock register */
#define NETC__EMDIO_GLOBAL_HTA0CAPR NETC__EMDIO_GLOBAL.HTA_LOOP[0].HTACAPR.R /* HTA 0 capability register */
#define NETC__EMDIO_GLOBAL_HTA0RFCOR NETC__EMDIO_GLOBAL.HTA_LOOP[0].HTARFCOR.R /* HTA 0 receive frame count operational register */
#define NETC__EMDIO_GLOBAL_HTA0HPBCOR NETC__EMDIO_GLOBAL.HTA_LOOP[0].HTAHPBCOR.R /* HTA 0 high priority byte count operational register */
#define NETC__EMDIO_GLOBAL_HTA0LPBCOR NETC__EMDIO_GLOBAL.HTA_LOOP[0].HTALPBCOR.R /* HTA 0 low priority byte count operational register */
#define NETC__EMDIO_GLOBAL_HTA0TFCOR NETC__EMDIO_GLOBAL.HTA_LOOP[0].HTATFCOR.R /* HTA 0 transmit frame count operational register */
#define NETC__EMDIO_GLOBAL_RC0SBRLAR NETC__EMDIO_GLOBAL.ARRAY_NUM_RC[0].RCSBRLAR.R /* Root complex 0 system bus read latency average register */
#define NETC__EMDIO_GLOBAL_RC0SBRLHWMR NETC__EMDIO_GLOBAL.ARRAY_NUM_RC[0].RCSBRLHWMR.R /* Root complex 0 system bus read latency high watermark register */
#define NETC__EMDIO_GLOBAL_RC0SBWLAR NETC__EMDIO_GLOBAL.ARRAY_NUM_RC[0].RCSBWLAR.R /* Root complex 0 system bus write latency average register */
#define NETC__EMDIO_GLOBAL_RC0SBWLHWMR NETC__EMDIO_GLOBAL.ARRAY_NUM_RC[0].RCSBWLHWMR.R /* Root complex 0 system bus write latency high watermark register */
#define NETC__EMDIO_GLOBAL_IPBRR0 NETC__EMDIO_GLOBAL.IPBRR0.R /* IP block revision register 0 */
#define NETC__EMDIO_GLOBAL_IPBRR1 NETC__EMDIO_GLOBAL.IPBRR1.R /* IP block revision register 1 */
#define NETC__EMDIO_GLOBAL_FBLPR0 NETC__EMDIO_GLOBAL.FBLPR[0].R /* Function boot loader parameter register 0 */
#define NETC__EMDIO_GLOBAL_FBLPR1 NETC__EMDIO_GLOBAL.FBLPR[1].R /* Function boot loader parameter register 1 */
#define NETC__EMDIO_GLOBAL_EMDIOUFSBECR NETC__EMDIO_GLOBAL.EMDIO.EMDIOUFSBECR.R /* EMDIO uncorrectable fatal system bus error configuration register */
#define NETC__EMDIO_GLOBAL_EMDIOUFSBESR NETC__EMDIO_GLOBAL.EMDIO.EMDIOUFSBESR.R /* EMDIO uncorrectable fatal system bus error status register */
#define NETC__EMDIO_GLOBAL_EMDIOUNIECR NETC__EMDIO_GLOBAL.EMDIO.EMDIOUNIECR.R /* EMDIO uncorrectable non-fatal integrity error configuration register */
#define NETC__EMDIO_GLOBAL_EMDIOUNIESR NETC__EMDIO_GLOBAL.EMDIO.EMDIOUNIESR.R /* EMDIO uncorrectable non-fatal integrity error status register */
#define NETC__EMDIO_GLOBAL_EMDIOUNIECTR NETC__EMDIO_GLOBAL.EMDIO.EMDIOUNIECTR.R /* EMDIO uncorrectable non-fatal integrity error count register */

/* NETC_F1_PCI_HDR_TYPE0 */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP_LIST.R /* PCI PCI-PM capabilities list register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP.R /* PCI PCI-PM capabilities register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CTL_STAT.R /* PCI PCI-PM control and status register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_PCIPM_DATA.R /* PCI PCI-PM capabilities data register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_EA_CAP_LIST.R /* PCI EA capabilities list register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_CAP NETC__NETC_F1_PCI_HDR_TYPE0.PCI_CFC_EA_CAP.R /* PCI EA capabilities register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_FMT NETC__NETC_F1_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 0 format register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_BASE NETC__NETC_F1_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 0 base register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_MAXOFF NETC__NETC_F1_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 0 max offset register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_EXT_BASE NETC__NETC_F1_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 0 extended base register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_FMT NETC__NETC_F1_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 1 format register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_BASE NETC__NETC_F1_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 1 base register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_MAXOFF NETC__NETC_F1_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 1 max offset register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_EXT_BASE NETC__NETC_F1_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 1 extended base register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_F1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_F1_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_F2 */
#define NETC__SW0_BASE_SCAPR0 NETC__SW0_BASE.SCAPR0.R      /* Switch capability register 0 */
#define NETC__SW0_BASE_SCAPR1 NETC__SW0_BASE.SCAPR1.R      /* Switch capability register 1 */
#define NETC__SW0_BASE_BPCAPR NETC__SW0_BASE.BPCAPR.R      /* Buffer pool capability register */
#define NETC__SW0_BASE_FCAPR NETC__SW0_BASE.FCAPR.R        /* Forwarding capability register */
#define NETC__SW0_BASE_SMBCAPR NETC__SW0_BASE.SMBCAPR.R    /* Shared memory buffer capability register */
#define NETC__SW0_BASE_SMBOR0 NETC__SW0_BASE.SMBOR0.R      /* Shared memory buffer operational register 0 */
#define NETC__SW0_BASE_SMBOR1 NETC__SW0_BASE.SMBOR1.R      /* Shared memory buffer operational register 1 */
#define NETC__SW0_BASE_CCAR  NETC__SW0_BASE.CCAR.R         /* Command cache attribute register */
#define NETC__SW0_BASE_MPCR  NETC__SW0_BASE.MPCR.R         /* Management port configuration register */
#define NETC__SW0_BASE_IMDCR0 NETC__SW0_BASE.IMDCR0.R      /* Ingress mirror destination configuration register 0 */
#define NETC__SW0_BASE_IMDCR1 NETC__SW0_BASE.IMDCR1.R      /* Ingress mirror destination configuration register 1 */
#define NETC__SW0_BASE_CTFCR NETC__SW0_BASE.CTFCR.R        /* Cut-through forwarding count register */
#define NETC__SW0_BASE_CBDR0MR NETC__SW0_BASE.NUM_CBDR[0].CBDRMR.R /* Command BDR 0 mode register */
#define NETC__SW0_BASE_CBDR0SR NETC__SW0_BASE.NUM_CBDR[0].CBDRSR.R /* Command BDR 0 status register */
#define NETC__SW0_BASE_CBDR0BAR0 NETC__SW0_BASE.NUM_CBDR[0].CBDRBAR0.R /* Command BDR base address register 0 */
#define NETC__SW0_BASE_CBDR0BAR1 NETC__SW0_BASE.NUM_CBDR[0].CBDRBAR1.R /* Command BDR 0 base address register 1 */
#define NETC__SW0_BASE_CBDR0PIR NETC__SW0_BASE.NUM_CBDR[0].CBDRPIR.R /* Command BDR 0 producer index register */
#define NETC__SW0_BASE_CBDR0CIR NETC__SW0_BASE.NUM_CBDR[0].CBDRCIR.R /* Command BDR 0 consumer index register */
#define NETC__SW0_BASE_CBDR0LENR NETC__SW0_BASE.NUM_CBDR[0].CBDRLENR.R /* Command BDR 0 length register */
#define NETC__SW0_BASE_CBDR1MR NETC__SW0_BASE.NUM_CBDR[1].CBDRMR.R /* Command BDR 1 mode register */
#define NETC__SW0_BASE_CBDR1SR NETC__SW0_BASE.NUM_CBDR[1].CBDRSR.R /* Command BDR 1 status register */
#define NETC__SW0_BASE_CBDR1BAR0 NETC__SW0_BASE.NUM_CBDR[1].CBDRBAR0.R /* Command BDR base address register 0 */
#define NETC__SW0_BASE_CBDR1BAR1 NETC__SW0_BASE.NUM_CBDR[1].CBDRBAR1.R /* Command BDR 1 base address register 1 */
#define NETC__SW0_BASE_CBDR1PIR NETC__SW0_BASE.NUM_CBDR[1].CBDRPIR.R /* Command BDR 1 producer index register */
#define NETC__SW0_BASE_CBDR1CIR NETC__SW0_BASE.NUM_CBDR[1].CBDRCIR.R /* Command BDR 1 consumer index register */
#define NETC__SW0_BASE_CBDR1LENR NETC__SW0_BASE.NUM_CBDR[1].CBDRLENR.R /* Command BDR 1 length register */
#define NETC__SW0_BASE_CBDR0IER NETC__SW0_BASE.NUM_CBDR_INT[0].CBDRIER.R /* Command BDR 0 interrupt enable register */
#define NETC__SW0_BASE_CBDR0IDR NETC__SW0_BASE.NUM_CBDR_INT[0].CBDRIDR.R /* Command BDR 0 interrupt detect register */
#define NETC__SW0_BASE_CBDR0MSIVR NETC__SW0_BASE.NUM_CBDR_INT[0].CBDRMSIVR.R /* Command BDR 0 MSI-X vector register */
#define NETC__SW0_BASE_CBDR1IER NETC__SW0_BASE.NUM_CBDR_INT[1].CBDRIER.R /* Command BDR 1 interrupt enable register */
#define NETC__SW0_BASE_CBDR1IDR NETC__SW0_BASE.NUM_CBDR_INT[1].CBDRIDR.R /* Command BDR 1 interrupt detect register */
#define NETC__SW0_BASE_CBDR1MSIVR NETC__SW0_BASE.NUM_CBDR_INT[1].CBDRMSIVR.R /* Command BDR 1 MSI-X vector register */
#define NETC__SW0_BASE_QOSVLANMP0R0 NETC__SW0_BASE.QOSVLANMP0R0.R /* QoS to VLAN mapping profile 0 register 0 */
#define NETC__SW0_BASE_QOSVLANMP0R1 NETC__SW0_BASE.QOSVLANMP0R1.R /* QoS to VLAN mapping profile 0 register 1 */
#define NETC__SW0_BASE_QOSVLANMP0R2 NETC__SW0_BASE.QOSVLANMP0R2.R /* QoS to VLAN mapping profile 0 register 2 */
#define NETC__SW0_BASE_QOSVLANMP0R3 NETC__SW0_BASE.QOSVLANMP0R3.R /* QoS to VLAN mapping profile 0 register 3 */
#define NETC__SW0_BASE_QOSVLANMP1R0 NETC__SW0_BASE.QOSVLANMP1R0.R /* QoS to VLAN mapping profile 1 register 0 */
#define NETC__SW0_BASE_QOSVLANMP1R1 NETC__SW0_BASE.QOSVLANMP1R1.R /* QoS to VLAN mapping profile 1 register 1 */
#define NETC__SW0_BASE_QOSVLANMP1R2 NETC__SW0_BASE.QOSVLANMP1R2.R /* QoS to VLAN mapping profile 1 register 2 */
#define NETC__SW0_BASE_QOSVLANMP1R3 NETC__SW0_BASE.QOSVLANMP1R3.R /* QoS to VLAN mapping profile 1 register 3 */
#define NETC__SW0_BASE_PCP2PCPMP0R NETC__SW0_BASE.PCP2PCPMPR[0].R /* PCP to PCP mapping profile 0 register */
#define NETC__SW0_BASE_PCP2PCPMP1R NETC__SW0_BASE.PCP2PCPMPR[1].R /* PCP to PCP mapping profile 1 register */
#define NETC__SW0_BASE_BRCAPR NETC__SW0_BASE.BRCAPR.R      /* Bridge capability register */
#define NETC__SW0_BASE_VFHTCAPR NETC__SW0_BASE.VFHTCAPR.R  /* VLAN filter hash table capability register */
#define NETC__SW0_BASE_VFHTOR NETC__SW0_BASE.VFHTOR.R      /* VLAN filter hash table operational register */
#define NETC__SW0_BASE_VFHTDECR0 NETC__SW0_BASE.VFHTDECR0.R /* VLAN Filter (hash) table default entry configuration registers 0 */
#define NETC__SW0_BASE_VFHTDECR1 NETC__SW0_BASE.VFHTDECR1.R /* VLAN filter hash table default entry configuration registers 1 */
#define NETC__SW0_BASE_VFHTDECR2 NETC__SW0_BASE.VFHTDECR2.R /* VLAN filter hash table default entry configuration registers 2 */
#define NETC__SW0_BASE_FDBHTCAPR NETC__SW0_BASE.FDBHTCAPR.R /* FDB hash table capability register */
#define NETC__SW0_BASE_FDBHTMCR NETC__SW0_BASE.FDBHTMCR.R  /* FDB hash table memory configuration register */
#define NETC__SW0_BASE_FDBHTOR0 NETC__SW0_BASE.FDBHTOR0.R  /* FDB hash table operational register 0 */
#define NETC__SW0_BASE_FDBHTOR1 NETC__SW0_BASE.FDBHTOR1.R  /* FDB hash table operational register 1 */
#define NETC__SW0_BASE_IPMFHTCAPR NETC__SW0_BASE.IPMFHTCAPR.R /* IP multicast filter hash table capability register */
#define NETC__SW0_BASE_IPv4MFHTOR NETC__SW0_BASE.IPV4MFHTOR.R /* IPv4 multicast filter hash table operation register */

/* NETC_F2_COMMON */
#define NETC__SW0_COMMON_IPCAPR NETC__SW0_COMMON.IPCAPR.R  /* Ingress port capability register */
#define NETC__SW0_COMMON_EPCAPR NETC__SW0_COMMON.EPCAPR.R  /* Egress port capability register */
#define NETC__SW0_COMMON_OSR NETC__SW0_COMMON.OSR.R        /* Operational state register */
#define NETC__SW0_COMMON_CMECR NETC__SW0_COMMON.CMECR.R    /* Correctable memory error configuration register */
#define NETC__SW0_COMMON_CMESR NETC__SW0_COMMON.CMESR.R    /* Correctable memory error status register */
#define NETC__SW0_COMMON_CMECTR NETC__SW0_COMMON.CMECTR.R  /* Correctable memory error count register */
#define NETC__SW0_COMMON_UNMACECR NETC__SW0_COMMON.UNMACECR.R /* Uncorrectable non-fatal MAC error configuration register */
#define NETC__SW0_COMMON_UNMACESR NETC__SW0_COMMON.UNMACESR.R /* Uncorrectable non-fatal MAC error status register */
#define NETC__SW0_COMMON_UNSBECR NETC__SW0_COMMON.UNSBECR.R /* Uncorrectable non-fatal system bus error configuration register */
#define NETC__SW0_COMMON_UNSBESR NETC__SW0_COMMON.UNSBESR.R /* Uncorrectable non-fatal system bus error status register */
#define NETC__SW0_COMMON_UNSBECTR NETC__SW0_COMMON.UNSBECTR.R /* Uncorrectable non-fatal system bus error count register */
#define NETC__SW0_COMMON_UFSBECR NETC__SW0_COMMON.UFSBECR.R /* Uncorrectable fatal system bus error configuration register */
#define NETC__SW0_COMMON_UFSBESR NETC__SW0_COMMON.UFSBESR.R /* Uncorrectable fatal system bus error status register */
#define NETC__SW0_COMMON_UNMECR NETC__SW0_COMMON.UNMECR.R  /* Uncorrectable non-fatal memory error configuration register */
#define NETC__SW0_COMMON_UNMESR0 NETC__SW0_COMMON.UNMESR0.R /* Uncorrectable non-fatal memory error status register 0 */
#define NETC__SW0_COMMON_UNMESR1 NETC__SW0_COMMON.UNMESR1.R /* Uncorrectable non-fatal memory error status register 1 */
#define NETC__SW0_COMMON_UNMECTR NETC__SW0_COMMON.UNMECTR.R /* Uncorrectable non-fatal memory error count register */
#define NETC__SW0_COMMON_UFMECR NETC__SW0_COMMON.UFMECR.R  /* Uncorrectable fatal memory error configuration register */
#define NETC__SW0_COMMON_UFMESR0 NETC__SW0_COMMON.UFMESR0.R /* Uncorrectable fatal memory error status register 0 */
#define NETC__SW0_COMMON_UFMESR1 NETC__SW0_COMMON.UFMESR1.R /* Uncorrectable fatal memory error status register 1 */
#define NETC__SW0_COMMON_UNIECR NETC__SW0_COMMON.UNIECR.R  /* Uncorrectable non-fatal integrity error configuration register */
#define NETC__SW0_COMMON_UNIESR NETC__SW0_COMMON.UNIESR.R  /* Uncorrectable non-fatal integrity error status register */
#define NETC__SW0_COMMON_UNIECTR NETC__SW0_COMMON.UNIECTR.R /* Uncorrectable non-fatal integrity error count register */
#define NETC__SW0_COMMON_UFIECR NETC__SW0_COMMON.UFIECR.R  /* Uncorrectable fatal integrity error configuration register */
#define NETC__SW0_COMMON_UFIESR NETC__SW0_COMMON.UFIESR.R  /* Uncorrectable fatal integrity error status register */
#define NETC__SW0_COMMON_EMDIOIRR NETC__SW0_COMMON.EMDIOIRR.R /* External MDIO interrupt reason register */
#define NETC__SW0_COMMON_EMDIOMSIVR NETC__SW0_COMMON.EMDIOMSIVR.R /* External MDIO MSI-X vector register */
#define NETC__SW0_COMMON_TCCR NETC__SW0_COMMON.TCCR.R      /* Time capture configuration register */
#define NETC__SW0_COMMON_TCIER NETC__SW0_COMMON.TCIER.R    /* Time capture interrupt enable register */
#define NETC__SW0_COMMON_TCRPIDR NETC__SW0_COMMON.TCRPIDR.R /* Time capture receive port interrupt detect register */
#define NETC__SW0_COMMON_TCRPSR NETC__SW0_COMMON.TCRPSR.R  /* Time capture receive port status register */
#define NETC__SW0_COMMON_TCRPTSR NETC__SW0_COMMON.TCRPTSR.R /* Time capture receive port timestamp register */
#define NETC__SW0_COMMON_TCMSIVR NETC__SW0_COMMON.TCMSIVR.R /* Time capture MSI-X vector register */
#define NETC__SW0_COMMON_CVLANR1 NETC__SW0_COMMON.CVLANR1.R /* Custom VLAN EtherType register 1 */
#define NETC__SW0_COMMON_CVLANR2 NETC__SW0_COMMON.CVLANR2.R /* Custom VLAN EtherType register 2 */
#define NETC__SW0_COMMON_PSRTAGETR NETC__SW0_COMMON.PSRTAGETR.R /* Pre-Standard RTAG EtherType register */
#define NETC__SW0_COMMON_DOSL2CR NETC__SW0_COMMON.DOSL2CR.R /* DoS L2 configuration register */
#define NETC__SW0_COMMON_VLANIPVMP0R0 NETC__SW0_COMMON.NUM_PROFILE[0].VLANIPVMPR0.R /* VLAN to IPV mapping profile 0 register 0 */
#define NETC__SW0_COMMON_VLANIPVMP0R1 NETC__SW0_COMMON.NUM_PROFILE[0].VLANIPVMPR1.R /* VLAN to IPV mapping profile 0 register 1 */
#define NETC__SW0_COMMON_VLANDRMP0R NETC__SW0_COMMON.NUM_PROFILE[0].VLANDRMPR.R /* VLAN to DR mapping profile 0 register */
#define NETC__SW0_COMMON_VLANIPVMP1R0 NETC__SW0_COMMON.NUM_PROFILE[1].VLANIPVMPR0.R /* VLAN to IPV mapping profile 1 register 0 */
#define NETC__SW0_COMMON_VLANIPVMP1R1 NETC__SW0_COMMON.NUM_PROFILE[1].VLANIPVMPR1.R /* VLAN to IPV mapping profile 1 register 1 */
#define NETC__SW0_COMMON_VLANDRMP1R NETC__SW0_COMMON.NUM_PROFILE[1].VLANDRMPR.R /* VLAN to DR mapping profile 1 register */
#define NETC__SW0_COMMON_IPFCAPR NETC__SW0_COMMON.IPFCAPR.R /* Ingress port filter capability register */
#define NETC__SW0_COMMON_IPFTCAPR NETC__SW0_COMMON.IPFTCAPR.R /* Ingress port filter table capability register */
#define NETC__SW0_COMMON_IPFTMOR NETC__SW0_COMMON.IPFTMOR.R /* Ingress port filter table memory operational register */
#define NETC__SW0_COMMON_ITMCAPR NETC__SW0_COMMON.ITMCAPR.R /* Index table memory capability register */
#define NETC__SW0_COMMON_RPCAPR NETC__SW0_COMMON.RPCAPR.R  /* Rate policer capability register */
#define NETC__SW0_COMMON_RPITCAPR NETC__SW0_COMMON.RPITCAPR.R /* Rate policer index table capability register */
#define NETC__SW0_COMMON_RPITMAR NETC__SW0_COMMON.RPITMAR.R /* Rate policer index table memory allocation register */
#define NETC__SW0_COMMON_RPITOR NETC__SW0_COMMON.RPITOR.R  /* Rate policer index table operational register */
#define NETC__SW0_COMMON_ISCITCAPR NETC__SW0_COMMON.ISCITCAPR.R /* Ingress stream counter index table capability register */
#define NETC__SW0_COMMON_ISCITMAR NETC__SW0_COMMON.ISCITMAR.R /* Ingress stream counter index table memory allocation register */
#define NETC__SW0_COMMON_ISCITOR NETC__SW0_COMMON.ISCITOR.R /* Ingress stream counter index table operational register */
#define NETC__SW0_COMMON_ISCAPR NETC__SW0_COMMON.ISCAPR.R  /* Ingress stream capability register */
#define NETC__SW0_COMMON_ISITCAPR NETC__SW0_COMMON.ISITCAPR.R /* Ingress stream index table capability register */
#define NETC__SW0_COMMON_ISITMAR NETC__SW0_COMMON.ISITMAR.R /* Ingress stream index table memory allocation register */
#define NETC__SW0_COMMON_ISITOR NETC__SW0_COMMON.ISITOR.R  /* Ingress stream index table operational register */
#define NETC__SW0_COMMON_ISQGITCAPR NETC__SW0_COMMON.ISQGITCAPR.R /* Ingress sequence generation index table capability register */
#define NETC__SW0_COMMON_ISQGITMAR NETC__SW0_COMMON.ISQGITMAR.R /* Ingress sequence generation index table memory allocation register */
#define NETC__SW0_COMMON_ISQGITOR NETC__SW0_COMMON.ISQGITOR.R /* Ingress sequence generation index table operational register */
#define NETC__SW0_COMMON_SGCAPR NETC__SW0_COMMON.SGCAPR.R  /* Stream gate capability register */
#define NETC__SW0_COMMON_SGIITCAPR NETC__SW0_COMMON.SGIITCAPR.R /* Stream gate instance index table capability register */
#define NETC__SW0_COMMON_SGIITMAR NETC__SW0_COMMON.SGIITMAR.R /* Stream gate instance index table memory allocation register */
#define NETC__SW0_COMMON_SGIITOR NETC__SW0_COMMON.SGIITOR.R /* Stream gate instance index table operational register */
#define NETC__SW0_COMMON_SGCLITCAPR NETC__SW0_COMMON.SGCLITCAPR.R /* Stream gate control list index table capability register */
#define NETC__SW0_COMMON_SGCLITMAR NETC__SW0_COMMON.SGCLITMAR.R /* Stream gate control list index table memory allocation register */
#define NETC__SW0_COMMON_SGCLTMOR NETC__SW0_COMMON.SGCLTMOR.R /* Stream gate control list table memory operational register */
#define NETC__SW0_COMMON_FMICAPR NETC__SW0_COMMON.FMICAPR.R /* Frame modification ingress capability register */
#define NETC__SW0_COMMON_FMECAPR NETC__SW0_COMMON.FMECAPR.R /* Frame modification egress capability register */
#define NETC__SW0_COMMON_FMITCAPR NETC__SW0_COMMON.FMITCAPR.R /* Frame modification index table capability register */
#define NETC__SW0_COMMON_FMITMAR NETC__SW0_COMMON.FMITMAR.R /* Frame modification index table memory allocation register */
#define NETC__SW0_COMMON_FMITOR NETC__SW0_COMMON.FMITOR.R  /* Frame modification index table operational register */
#define NETC__SW0_COMMON_FMDITCAPR NETC__SW0_COMMON.FMDITCAPR.R /* Frame modification data index table capability register */
#define NETC__SW0_COMMON_FMDITMAR NETC__SW0_COMMON.FMDITMAR.R /* Frame modification data index table memory allocation register */
#define NETC__SW0_COMMON_ETCAPR NETC__SW0_COMMON.ETCAPR.R  /* Egress treatment capability register */
#define NETC__SW0_COMMON_ETTCAPR NETC__SW0_COMMON.ETTCAPR.R /* Egress treatment table capability register */
#define NETC__SW0_COMMON_ETTOR NETC__SW0_COMMON.ETTOR.R    /* Egress treatment table operational register */
#define NETC__SW0_COMMON_TGSTCAPR NETC__SW0_COMMON.TGSTCAPR.R /* Time gate scheduling table capability register */
#define NETC__SW0_COMMON_TGSTMOR NETC__SW0_COMMON.TGSTMOR.R /* Time gate scheduling table memory operation register */
#define NETC__SW0_COMMON_ESQRCAPR NETC__SW0_COMMON.ESQRCAPR.R /* Egress sequence recovery capability register */
#define NETC__SW0_COMMON_ESQRTCAPR NETC__SW0_COMMON.ESQRTCAPR.R /* Egress sequence recovery table capability register */
#define NETC__SW0_COMMON_ECTCAPR NETC__SW0_COMMON.ECTCAPR.R /* Egress counter table capability register */
#define NETC__SW0_COMMON_HTMCAPR NETC__SW0_COMMON.HTMCAPR.R /* Hash table memory capability register */
#define NETC__SW0_COMMON_HTMOR NETC__SW0_COMMON.HTMOR.R    /* Hash table memory operational register */
#define NETC__SW0_COMMON_ISIDCAPR NETC__SW0_COMMON.ISIDCAPR.R /* Ingress stream identification capability register */
#define NETC__SW0_COMMON_ISIDHTCAPR NETC__SW0_COMMON.ISIDHTCAPR.R /* Ingress stream identification hash table capability register */
#define NETC__SW0_COMMON_ISIDKC0OR NETC__SW0_COMMON.ISIDKC0OR.R /* Ingress stream identification key construction 0 operational register */
#define NETC__SW0_COMMON_ISIDKC0CR0 NETC__SW0_COMMON.ISIDKC0CR0.R /* Ingress stream identification key construction 0 configuration register 0 */
#define NETC__SW0_COMMON_ISIDKC0PF0CR NETC__SW0_COMMON.ISIDKC0PF0CR.R /* Ingress stream identification key construction 0 payload field 0 configuration register */
#define NETC__SW0_COMMON_ISIDKC0PF1CR NETC__SW0_COMMON.ISIDKC0PF1CR.R /* Ingress stream identification key construction 0 payload field 1 configuration register */
#define NETC__SW0_COMMON_ISIDKC0PF2CR NETC__SW0_COMMON.ISIDKC0PF2CR.R /* Ingress stream identification key construction 0 payload field 2 configuration register */
#define NETC__SW0_COMMON_ISIDKC0PF3CR NETC__SW0_COMMON.ISIDKC0PF3CR.R /* Ingress stream identification key construction 0 payload field 3 configuration register */
#define NETC__SW0_COMMON_ISIDKC1OR NETC__SW0_COMMON.ISIDKC1OR.R /* Ingress stream identification key construction 1 operational register */
#define NETC__SW0_COMMON_ISIDKC1CR0 NETC__SW0_COMMON.ISIDKC1CR0.R /* Ingress stream identification key construction 1 configuration register 0 */
#define NETC__SW0_COMMON_ISIDKC1PF0CR NETC__SW0_COMMON.ISIDKC1PF0CR.R /* Ingress stream identification key construction 1 payload field 0 configuration register */
#define NETC__SW0_COMMON_ISIDKC1PF1CR NETC__SW0_COMMON.ISIDKC1PF1CR.R /* Ingress stream identification key construction 1 payload field 1 configuration register */
#define NETC__SW0_COMMON_ISIDKC1PF2CR NETC__SW0_COMMON.ISIDKC1PF2CR.R /* Ingress stream identification key construction 1 payload field 2 configuration register */
#define NETC__SW0_COMMON_ISIDKC1PF3CR NETC__SW0_COMMON.ISIDKC1PF3CR.R /* Ingress stream identification key construction 1 payload field 3 configuration register */
#define NETC__SW0_COMMON_ISIDKC2OR NETC__SW0_COMMON.ISIDKC2OR.R /* Ingress stream identification key construction 2 operational register */
#define NETC__SW0_COMMON_ISIDKC2CR0 NETC__SW0_COMMON.ISIDKC2CR0.R /* Ingress stream identification key construction 2 configuration register 0 */
#define NETC__SW0_COMMON_ISIDKC2PF0CR NETC__SW0_COMMON.ISIDKC2PF0CR.R /* Ingress stream identification key construction 2 payload field 0 configuration register */
#define NETC__SW0_COMMON_ISIDKC2PF1CR NETC__SW0_COMMON.ISIDKC2PF1CR.R /* Ingress stream identification key construction 2 payload field 1 configuration register */
#define NETC__SW0_COMMON_ISIDKC2PF2CR NETC__SW0_COMMON.ISIDKC2PF2CR.R /* Ingress stream identification key construction 2 payload field 2 configuration register */
#define NETC__SW0_COMMON_ISIDKC2PF3CR NETC__SW0_COMMON.ISIDKC2PF3CR.R /* Ingress stream identification key construction 2 payload field 3 configuration register */
#define NETC__SW0_COMMON_ISIDKC3OR NETC__SW0_COMMON.ISIDKC3OR.R /* Ingress stream identification key construction 3 operational register */
#define NETC__SW0_COMMON_ISIDKC3CR0 NETC__SW0_COMMON.ISIDKC3CR0.R /* Ingress stream identification key construction 3 configuration register 0 */
#define NETC__SW0_COMMON_ISIDKC3PF0CR NETC__SW0_COMMON.ISIDKC3PF0CR.R /* Ingress stream identification key construction 3 payload field 0 configuration register */
#define NETC__SW0_COMMON_ISIDKC3PF1CR NETC__SW0_COMMON.ISIDKC3PF1CR.R /* Ingress stream identification key construction 3 payload field 1 configuration register */
#define NETC__SW0_COMMON_ISIDKC3PF2CR NETC__SW0_COMMON.ISIDKC3PF2CR.R /* Ingress stream identification key construction 3 payload field 2 configuration register */
#define NETC__SW0_COMMON_ISIDKC3PF3CR NETC__SW0_COMMON.ISIDKC3PF3CR.R /* Ingress stream identification key construction 3 payload field 3 configuration register */
#define NETC__SW0_COMMON_ISFHTCAPR NETC__SW0_COMMON.ISFHTCAPR.R /* Ingress stream filter hash table capability register */
#define NETC__SW0_COMMON_ISFHTOR NETC__SW0_COMMON.ISFHTOR.R /* Ingress stream filter hash table operational register */

/* NETC_F2_GLOBAL */
#define NETC__SW0_GLOBAL_SMCAPR NETC__SW0_GLOBAL.SMCAPR.R  /* Shared memory capability register */
#define NETC__SW0_GLOBAL_SMDTR NETC__SW0_GLOBAL.SMDTR.R    /* Shared memory depletion threshold register */
#define NETC__SW0_GLOBAL_SMACR NETC__SW0_GLOBAL.SMACR.R    /* Shared memory available count register */
#define NETC__SW0_GLOBAL_SMCLWMR NETC__SW0_GLOBAL.SMCLWMR.R /* Shared memory count low watermark register */
#define NETC__SW0_GLOBAL_SMBUCR NETC__SW0_GLOBAL.SMBUCR.R  /* Shared memory buffer unassigned count register */
#define NETC__SW0_GLOBAL_SMBUCHWMR NETC__SW0_GLOBAL.SMBUCHWMR.R /* Shared memory buffer unassigned count high watermark register */
#define NETC__SW0_GLOBAL_SMLCR NETC__SW0_GLOBAL.SMLCR.R    /* Shared memory loss count register */
#define NETC__SW0_GLOBAL_HBTCAPR NETC__SW0_GLOBAL.HBTCAPR.R /* Hash bucket table capability register */
#define NETC__SW0_GLOBAL_HBTOR0 NETC__SW0_GLOBAL.HBTOR0.R  /* Hash bucket table operational register 0 */
#define NETC__SW0_GLOBAL_HBTOR2 NETC__SW0_GLOBAL.HBTOR2.R  /* Hash bucket table operational register 2 */
#define NETC__SW0_GLOBAL_SMERBCAPR NETC__SW0_GLOBAL.SMERBCAPR.R /* Shared memory ENETC receive buffer capability register */
#define NETC__SW0_GLOBAL_SMERBOR0 NETC__SW0_GLOBAL.SMERBOR0.R /* Shared memory ENETC receive buffer operational register 0 */
#define NETC__SW0_GLOBAL_SMERBOR1 NETC__SW0_GLOBAL.SMERBOR1.R /* Shared memory ENETC receive buffer operational 1 */
#define NETC__SW0_GLOBAL_PCE0OR NETC__SW0_GLOBAL.PCE_SL[0].PCEOR.R /* PCE 0 operational register */
#define NETC__SW0_GLOBAL_RFE0OR NETC__SW0_GLOBAL.PCE_SL[0].RFEOR.R /* Replication Forwarding Engine 0 operational register */
#define NETC__SW0_GLOBAL_NETCCLKR NETC__SW0_GLOBAL.NETCCLKR.R /* NETC clock register */
#define NETC__SW0_GLOBAL_HTA0CAPR NETC__SW0_GLOBAL.HTA_LOOP[0].HTACAPR.R /* HTA 0 capability register */
#define NETC__SW0_GLOBAL_HTA0RFCOR NETC__SW0_GLOBAL.HTA_LOOP[0].HTARFCOR.R /* HTA 0 receive frame count operational register */
#define NETC__SW0_GLOBAL_HTA0HPBCOR NETC__SW0_GLOBAL.HTA_LOOP[0].HTAHPBCOR.R /* HTA 0 high priority byte count operational register */
#define NETC__SW0_GLOBAL_HTA0LPBCOR NETC__SW0_GLOBAL.HTA_LOOP[0].HTALPBCOR.R /* HTA 0 low priority byte count operational register */
#define NETC__SW0_GLOBAL_HTA0TFCOR NETC__SW0_GLOBAL.HTA_LOOP[0].HTATFCOR.R /* HTA 0 transmit frame count operational register */
#define NETC__SW0_GLOBAL_RC0SBRLAR NETC__SW0_GLOBAL.ARRAY_NUM_RC[0].RCSBRLAR.R /* Root complex 0 system bus read latency average register */
#define NETC__SW0_GLOBAL_RC0SBRLHWMR NETC__SW0_GLOBAL.ARRAY_NUM_RC[0].RCSBRLHWMR.R /* Root complex 0 system bus read latency high watermark register */
#define NETC__SW0_GLOBAL_RC0SBWLAR NETC__SW0_GLOBAL.ARRAY_NUM_RC[0].RCSBWLAR.R /* Root complex 0 system bus write latency average register */
#define NETC__SW0_GLOBAL_RC0SBWLHWMR NETC__SW0_GLOBAL.ARRAY_NUM_RC[0].RCSBWLHWMR.R /* Root complex 0 system bus write latency high watermark register */
#define NETC__SW0_GLOBAL_IPBRR0 NETC__SW0_GLOBAL.IPBRR0.R  /* IP block revision register 0 */
#define NETC__SW0_GLOBAL_IPBRR1 NETC__SW0_GLOBAL.IPBRR1.R  /* IP block revision register 1 */
#define NETC__SW0_GLOBAL_FBLPR0 NETC__SW0_GLOBAL.FBLPR[0].R /* Function boot loader parameter register 0 */
#define NETC__SW0_GLOBAL_FBLPR1 NETC__SW0_GLOBAL.FBLPR[1].R /* Function boot loader parameter register 1 */

/* NETC_F2_PCI_HDR_TYPE0 */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP_LIST.R /* PCI PCI-PM capabilities list register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP.R /* PCI PCI-PM capabilities register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CTL_STAT.R /* PCI PCI-PM control and status register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_PCIPM_DATA.R /* PCI PCI-PM capabilities data register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_EA_CAP_LIST.R /* PCI EA capabilities list register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_CAP NETC__NETC_F2_PCI_HDR_TYPE0.PCI_CFC_EA_CAP.R /* PCI EA capabilities register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_FMT NETC__NETC_F2_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 0 format register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_BASE NETC__NETC_F2_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 0 base register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_MAXOFF NETC__NETC_F2_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 0 max offset register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_EXT_BASE NETC__NETC_F2_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 0 extended base register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_FMT NETC__NETC_F2_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 1 format register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_BASE NETC__NETC_F2_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 1 base register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_MAXOFF NETC__NETC_F2_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 1 max offset register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_EXT_BASE NETC__NETC_F2_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 1 extended base register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_F2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_F2_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_F3 */
#define NETC__ENETC0_BASE_ECAPR0 NETC__ENETC0_BASE.ECAPR0.R /* ENETC capability register 0 */
#define NETC__ENETC0_BASE_ECAPR1 NETC__ENETC0_BASE.ECAPR1.R /* ENETC capability register 1 */
#define NETC__ENETC0_BASE_ECAPR2 NETC__ENETC0_BASE.ECAPR2.R /* ENETC capability register 2 */
#define NETC__ENETC0_BASE_PMR NETC__ENETC0_BASE.PMR.R      /* Port mode register */
#define NETC__ENETC0_BASE_PONVLANR NETC__ENETC0_BASE.PONVLANR.R /* Port outer native VLAN register */
#define NETC__ENETC0_BASE_PINVLANR NETC__ENETC0_BASE.PINVLANR.R /* Port inner native VLAN register */
#define NETC__ENETC0_BASE_PVCLCTR NETC__ENETC0_BASE.PVCLCTR.R /* Port VLAN classification control register */
#define NETC__ENETC0_BASE_PARCSCR NETC__ENETC0_BASE.PARCSCR.R /* Parser checksum configuration register */
#define NETC__ENETC0_BASE_PARCE0CR NETC__ENETC0_BASE.PARCECR[0].R /* Parser custom Ethertype 0 configuration register */
#define NETC__ENETC0_BASE_PARCE1CR NETC__ENETC0_BASE.PARCECR[1].R /* Parser custom Ethertype 1 configuration register */
#define NETC__ENETC0_BASE_PARCE2CR NETC__ENETC0_BASE.PARCECR[2].R /* Parser custom Ethertype 2 configuration register */
#define NETC__ENETC0_BASE_PARCE3CR NETC__ENETC0_BASE.PARCECR[3].R /* Parser custom Ethertype 3 configuration register */
#define NETC__ENETC0_BASE_PPAUONTR NETC__ENETC0_BASE.PPAUONTR.R /* Port pause ON threshold register */
#define NETC__ENETC0_BASE_PPAUOFFTR NETC__ENETC0_BASE.PPAUOFFTR.R /* Port pause OFF threshold register */
#define NETC__ENETC0_BASE_PRXMBER NETC__ENETC0_BASE.PRXMBER.R /* Port receive memory buffer entitlement register */
#define NETC__ENETC0_BASE_PRXMBLR NETC__ENETC0_BASE.PRXMBLR.R /* Port receive memory buffer limit register */
#define NETC__ENETC0_BASE_PRXBCR NETC__ENETC0_BASE.PRXBCR.R /* Port receive buffer count register */
#define NETC__ENETC0_BASE_PRXBCHWMR NETC__ENETC0_BASE.PRXBCHWMR.R /* Port receive buffer count high watermark register */
#define NETC__ENETC0_BASE_PICDR0DCR NETC__ENETC0_BASE.PICDRADCR[0].PICDRDCR.R /* Port ingress congestion DR0 discard count register */
#define NETC__ENETC0_BASE_PICDR0DCRRR NETC__ENETC0_BASE.PICDRADCR[0].PICDRDCRRR.R /* Port ingress congestion DR0 discard count read-reset register */
#define NETC__ENETC0_BASE_PICDR1DCR NETC__ENETC0_BASE.PICDRADCR[1].PICDRDCR.R /* Port ingress congestion DR1 discard count register */
#define NETC__ENETC0_BASE_PICDR1DCRRR NETC__ENETC0_BASE.PICDRADCR[1].PICDRDCRRR.R /* Port ingress congestion DR1 discard count read-reset register */
#define NETC__ENETC0_BASE_PICDR2DCR NETC__ENETC0_BASE.PICDRADCR[2].PICDRDCR.R /* Port ingress congestion DR2 discard count register */
#define NETC__ENETC0_BASE_PICDR2DCRRR NETC__ENETC0_BASE.PICDRADCR[2].PICDRDCRRR.R /* Port ingress congestion DR2 discard count read-reset register */
#define NETC__ENETC0_BASE_PICDR3DCR NETC__ENETC0_BASE.PICDRADCR[3].PICDRDCR.R /* Port ingress congestion DR3 discard count register */
#define NETC__ENETC0_BASE_PICDR3DCRRR NETC__ENETC0_BASE.PICDRADCR[3].PICDRDCRRR.R /* Port ingress congestion DR3 discard count read-reset register */
#define NETC__ENETC0_BASE_PICPDSR NETC__ENETC0_BASE.PICPDSR.R /* Port ingress congestion priority discard status register */
#define NETC__ENETC0_BASE_PSIPMMR NETC__ENETC0_BASE.PSIPMMR.R /* Port station interface promiscuous MAC mode register */
#define NETC__ENETC0_BASE_PSIPVMR NETC__ENETC0_BASE.PSIPVMR.R /* Port station interface promiscuous VLAN mode register */
#define NETC__ENETC0_BASE_PBFDSIR NETC__ENETC0_BASE.PBFDSIR.R /* Port broadcast frames dropped due to MAC filtering register */
#define NETC__ENETC0_BASE_PFDMSAPR NETC__ENETC0_BASE.PFDMSAPR.R /* Port frame drop MAC source address pruning register */
#define NETC__ENETC0_BASE_PSIMAFCAPR NETC__ENETC0_BASE.PSIMAFCAPR.R /* Port station interface MAC address filtering capability register */
#define NETC__ENETC0_BASE_PUFDMFR NETC__ENETC0_BASE.PUFDMFR.R /* Port unicast frames dropped due to MAC filtering register */
#define NETC__ENETC0_BASE_PMFDMFR NETC__ENETC0_BASE.PMFDMFR.R /* Port multicast frames dropped due to MAC filtering register */
#define NETC__ENETC0_BASE_PSIVLANFCAPR NETC__ENETC0_BASE.PSIVLANFCAPR.R /* Port station interface VLAN filtering capability register */
#define NETC__ENETC0_BASE_PSIVLANFMR NETC__ENETC0_BASE.PSIVLANFMR.R /* Port station interface VLAN filtering mode register */
#define NETC__ENETC0_BASE_PUFDVFR NETC__ENETC0_BASE.PUFDVFR.R /* Port unicast frames dropped VLAN filtering register */
#define NETC__ENETC0_BASE_PMFDVFR NETC__ENETC0_BASE.PMFDVFR.R /* Port multicast frames dropped VLAN filtering register */
#define NETC__ENETC0_BASE_PBFDVFR NETC__ENETC0_BASE.PBFDVFR.R /* Port broadcast frames dropped VLAN filtering register */
#define NETC__ENETC0_BASE_PRFSCAPR NETC__ENETC0_BASE.PRFSCAPR.R /* Port RFS capability register */
#define NETC__ENETC0_BASE_PRFSMR NETC__ENETC0_BASE.PRFSMR.R /* Port RFS mode register */
#define NETC__ENETC0_BASE_IPV2ICMPMR0 NETC__ENETC0_BASE.IPV2ICMPMR0.R /* Receive IPV to ICM priority mapping register 0 */
#define NETC__ENETC0_BASE_PRIO2TCMR0 NETC__ENETC0_BASE.PRIO2TCMR0.R /* Transmit priority to traffic class mapping register 0 */
#define NETC__ENETC0_BASE_PTC0TSDR NETC__ENETC0_BASE.PTCTSDR[0].R /* Port traffic class 0 time specific departure register */
#define NETC__ENETC0_BASE_PTC1TSDR NETC__ENETC0_BASE.PTCTSDR[1].R /* Port traffic class 1 time specific departure register */
#define NETC__ENETC0_BASE_PTC2TSDR NETC__ENETC0_BASE.PTCTSDR[2].R /* Port traffic class 2 time specific departure register */
#define NETC__ENETC0_BASE_PTC3TSDR NETC__ENETC0_BASE.PTCTSDR[3].R /* Port traffic class 3 time specific departure register */
#define NETC__ENETC0_BASE_PTC4TSDR NETC__ENETC0_BASE.PTCTSDR[4].R /* Port traffic class 4 time specific departure register */
#define NETC__ENETC0_BASE_PTC5TSDR NETC__ENETC0_BASE.PTCTSDR[5].R /* Port traffic class 5 time specific departure register */
#define NETC__ENETC0_BASE_PTC6TSDR NETC__ENETC0_BASE.PTCTSDR[6].R /* Port traffic class 6 time specific departure register */
#define NETC__ENETC0_BASE_PTC7TSDR NETC__ENETC0_BASE.PTCTSDR[7].R /* Port traffic class 7 time specific departure register */
#define NETC__ENETC0_BASE_SMCAPR NETC__ENETC0_BASE.SMCAPR.R /* Switch management capability register */
#define NETC__ENETC0_BASE_SMHR1BDRMR NETC__ENETC0_BASE.SMHRBDRMR[0].R /* Switch management host reason 1 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR2BDRMR NETC__ENETC0_BASE.SMHRBDRMR[1].R /* Switch management host reason 2 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR3BDRMR NETC__ENETC0_BASE.SMHRBDRMR[2].R /* Switch management host reason 3 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR4BDRMR NETC__ENETC0_BASE.SMHRBDRMR[3].R /* Switch management host reason 4 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR5BDRMR NETC__ENETC0_BASE.SMHRBDRMR[4].R /* Switch management host reason 5 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR6BDRMR NETC__ENETC0_BASE.SMHRBDRMR[5].R /* Switch management host reason 6 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR7BDRMR NETC__ENETC0_BASE.SMHRBDRMR[6].R /* Switch management host reason 7 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR8BDRMR NETC__ENETC0_BASE.SMHRBDRMR[7].R /* Switch management host reason 8 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR9BDRMR NETC__ENETC0_BASE.SMHRBDRMR[8].R /* Switch management host reason 9 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR10BDRMR NETC__ENETC0_BASE.SMHRBDRMR[9].R /* Switch management host reason 10 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR11BDRMR NETC__ENETC0_BASE.SMHRBDRMR[10].R /* Switch management host reason 11 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR12BDRMR NETC__ENETC0_BASE.SMHRBDRMR[11].R /* Switch management host reason 12 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR13BDRMR NETC__ENETC0_BASE.SMHRBDRMR[12].R /* Switch management host reason 13 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR14BDRMR NETC__ENETC0_BASE.SMHRBDRMR[13].R /* Switch management host reason 14 receive BD ring mapping register */
#define NETC__ENETC0_BASE_SMHR15BDRMR NETC__ENETC0_BASE.SMHRBDRMR[14].R /* Switch management host reason 15 receive BD ring mapping register */
#define NETC__ENETC0_BASE_PSI0PMAR0 NETC__ENETC0_BASE.NUM_SI[0].PSIPMAR0.R /* Port station interface 0 primary MAC address register 0 */
#define NETC__ENETC0_BASE_PSI0PMAR1 NETC__ENETC0_BASE.NUM_SI[0].PSIPMAR1.R /* Port station interface 0 primary MAC address register 1 */
#define NETC__ENETC0_BASE_PSI0VLANR NETC__ENETC0_BASE.NUM_SI[0].PSIVLANR.R /* Port station interface 0 VLAN register */
#define NETC__ENETC0_BASE_PSI0CFGR0 NETC__ENETC0_BASE.NUM_SI[0].PSICFGR0.R /* Port station interface 0 configuration register 0 */
#define NETC__ENETC0_BASE_PSI0CFGR2 NETC__ENETC0_BASE.NUM_SI[0].PSICFGR2.R /* Port station interface 0 configuration register 2 */
#define NETC__ENETC0_BASE_PSI0VMAFCFGR NETC__ENETC0_BASE.NUM_SI[0].PSIVMAFCFGR.R /* Port station interface 0 VSI MAC address filtering configuration register */
#define NETC__ENETC0_BASE_PSI0VLANFCFGR NETC__ENETC0_BASE.NUM_SI[0].PSIVLANFCFGR.R /* Port station interface 0 VLAN filtering configuration register */
#define NETC__ENETC0_BASE_PSI0RFSCFGR NETC__ENETC0_BASE.NUM_SI[0].PSIRFSCFGR.R /* Port station interface 0 RFS configuration register */
#define NETC__ENETC0_BASE_PSI0UMHFR0 NETC__ENETC0_BASE.NUM_SI[0].PSIUMHFR0.R /* Port station interface 0 unicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI0UMHFR1 NETC__ENETC0_BASE.NUM_SI[0].PSIUMHFR1.R /* Port station interface 0 unicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI0MMHFR0 NETC__ENETC0_BASE.NUM_SI[0].PSIMMHFR0.R /* Port station interface 0 multicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI0MMHFR1 NETC__ENETC0_BASE.NUM_SI[0].PSIMMHFR1.R /* Port station interface 0 multicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI0VHFR0 NETC__ENETC0_BASE.NUM_SI[0].PSIVHFR0.R /* Port station interface 0 VLAN hash filter register 0 */
#define NETC__ENETC0_BASE_PSI0VHFR1 NETC__ENETC0_BASE.NUM_SI[0].PSIVHFR1.R /* Port station interface 0 VLAN hash filter register 1 */
#define NETC__ENETC0_BASE_PSI1PMAR0 NETC__ENETC0_BASE.NUM_SI[1].PSIPMAR0.R /* Port station interface 1 primary MAC address register 0 */
#define NETC__ENETC0_BASE_PSI1PMAR1 NETC__ENETC0_BASE.NUM_SI[1].PSIPMAR1.R /* Port station interface 1 primary MAC address register 1 */
#define NETC__ENETC0_BASE_PSI1VLANR NETC__ENETC0_BASE.NUM_SI[1].PSIVLANR.R /* Port station interface 1 VLAN register */
#define NETC__ENETC0_BASE_PSI1CFGR0 NETC__ENETC0_BASE.NUM_SI[1].PSICFGR0.R /* Port station interface 1 configuration register 0 */
#define NETC__ENETC0_BASE_PSI1CFGR1 NETC__ENETC0_BASE.NUM_SI[1].PSICFGR1.R /* Port station interface 1 configuration register 1 */
#define NETC__ENETC0_BASE_PSI1CFGR2 NETC__ENETC0_BASE.NUM_SI[1].PSICFGR2.R /* Port station interface 1 configuration register 2 */
#define NETC__ENETC0_BASE_PSI1VMAFCFGR NETC__ENETC0_BASE.NUM_SI[1].PSIVMAFCFGR.R /* Port station interface 1 VSI MAC address filtering configuration register */
#define NETC__ENETC0_BASE_PSI1VLANFCFGR NETC__ENETC0_BASE.NUM_SI[1].PSIVLANFCFGR.R /* Port station interface 1 VLAN filtering configuration register */
#define NETC__ENETC0_BASE_PSI1RFSCFGR NETC__ENETC0_BASE.NUM_SI[1].PSIRFSCFGR.R /* Port station interface 1 RFS configuration register */
#define NETC__ENETC0_BASE_PSI1UMHFR0 NETC__ENETC0_BASE.NUM_SI[1].PSIUMHFR0.R /* Port station interface 1 unicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI1UMHFR1 NETC__ENETC0_BASE.NUM_SI[1].PSIUMHFR1.R /* Port station interface 1 unicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI1MMHFR0 NETC__ENETC0_BASE.NUM_SI[1].PSIMMHFR0.R /* Port station interface 1 multicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI1MMHFR1 NETC__ENETC0_BASE.NUM_SI[1].PSIMMHFR1.R /* Port station interface 1 multicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI1VHFR0 NETC__ENETC0_BASE.NUM_SI[1].PSIVHFR0.R /* Port station interface 1 VLAN hash filter register 0 */
#define NETC__ENETC0_BASE_PSI1VHFR1 NETC__ENETC0_BASE.NUM_SI[1].PSIVHFR1.R /* Port station interface 1 VLAN hash filter register 1 */
#define NETC__ENETC0_BASE_PSI2PMAR0 NETC__ENETC0_BASE.NUM_SI[2].PSIPMAR0.R /* Port station interface 2 primary MAC address register 0 */
#define NETC__ENETC0_BASE_PSI2PMAR1 NETC__ENETC0_BASE.NUM_SI[2].PSIPMAR1.R /* Port station interface 2 primary MAC address register 1 */
#define NETC__ENETC0_BASE_PSI2VLANR NETC__ENETC0_BASE.NUM_SI[2].PSIVLANR.R /* Port station interface 2 VLAN register */
#define NETC__ENETC0_BASE_PSI2CFGR0 NETC__ENETC0_BASE.NUM_SI[2].PSICFGR0.R /* Port station interface 2 configuration register 0 */
#define NETC__ENETC0_BASE_PSI2CFGR1 NETC__ENETC0_BASE.NUM_SI[2].PSICFGR1.R /* Port station interface 2 configuration register 1 */
#define NETC__ENETC0_BASE_PSI2CFGR2 NETC__ENETC0_BASE.NUM_SI[2].PSICFGR2.R /* Port station interface 2 configuration register 2 */
#define NETC__ENETC0_BASE_PSI2VMAFCFGR NETC__ENETC0_BASE.NUM_SI[2].PSIVMAFCFGR.R /* Port station interface 2 VSI MAC address filtering configuration register */
#define NETC__ENETC0_BASE_PSI2VLANFCFGR NETC__ENETC0_BASE.NUM_SI[2].PSIVLANFCFGR.R /* Port station interface 2 VLAN filtering configuration register */
#define NETC__ENETC0_BASE_PSI2RFSCFGR NETC__ENETC0_BASE.NUM_SI[2].PSIRFSCFGR.R /* Port station interface 2 RFS configuration register */
#define NETC__ENETC0_BASE_PSI2UMHFR0 NETC__ENETC0_BASE.NUM_SI[2].PSIUMHFR0.R /* Port station interface 2 unicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI2UMHFR1 NETC__ENETC0_BASE.NUM_SI[2].PSIUMHFR1.R /* Port station interface 2 unicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI2MMHFR0 NETC__ENETC0_BASE.NUM_SI[2].PSIMMHFR0.R /* Port station interface 2 multicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI2MMHFR1 NETC__ENETC0_BASE.NUM_SI[2].PSIMMHFR1.R /* Port station interface 2 multicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI2VHFR0 NETC__ENETC0_BASE.NUM_SI[2].PSIVHFR0.R /* Port station interface 2 VLAN hash filter register 0 */
#define NETC__ENETC0_BASE_PSI2VHFR1 NETC__ENETC0_BASE.NUM_SI[2].PSIVHFR1.R /* Port station interface 2 VLAN hash filter register 1 */
#define NETC__ENETC0_BASE_PSI3PMAR0 NETC__ENETC0_BASE.NUM_SI[3].PSIPMAR0.R /* Port station interface 3 primary MAC address register 0 */
#define NETC__ENETC0_BASE_PSI3PMAR1 NETC__ENETC0_BASE.NUM_SI[3].PSIPMAR1.R /* Port station interface 3 primary MAC address register 1 */
#define NETC__ENETC0_BASE_PSI3VLANR NETC__ENETC0_BASE.NUM_SI[3].PSIVLANR.R /* Port station interface 3 VLAN register */
#define NETC__ENETC0_BASE_PSI3CFGR0 NETC__ENETC0_BASE.NUM_SI[3].PSICFGR0.R /* Port station interface 3 configuration register 0 */
#define NETC__ENETC0_BASE_PSI3CFGR1 NETC__ENETC0_BASE.NUM_SI[3].PSICFGR1.R /* Port station interface 3 configuration register 1 */
#define NETC__ENETC0_BASE_PSI3CFGR2 NETC__ENETC0_BASE.NUM_SI[3].PSICFGR2.R /* Port station interface 3 configuration register 2 */
#define NETC__ENETC0_BASE_PSI3VMAFCFGR NETC__ENETC0_BASE.NUM_SI[3].PSIVMAFCFGR.R /* Port station interface 3 VSI MAC address filtering configuration register */
#define NETC__ENETC0_BASE_PSI3VLANFCFGR NETC__ENETC0_BASE.NUM_SI[3].PSIVLANFCFGR.R /* Port station interface 3 VLAN filtering configuration register */
#define NETC__ENETC0_BASE_PSI3RFSCFGR NETC__ENETC0_BASE.NUM_SI[3].PSIRFSCFGR.R /* Port station interface 3 RFS configuration register */
#define NETC__ENETC0_BASE_PSI3UMHFR0 NETC__ENETC0_BASE.NUM_SI[3].PSIUMHFR0.R /* Port station interface 3 unicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI3UMHFR1 NETC__ENETC0_BASE.NUM_SI[3].PSIUMHFR1.R /* Port station interface 3 unicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI3MMHFR0 NETC__ENETC0_BASE.NUM_SI[3].PSIMMHFR0.R /* Port station interface 3 multicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI3MMHFR1 NETC__ENETC0_BASE.NUM_SI[3].PSIMMHFR1.R /* Port station interface 3 multicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI3VHFR0 NETC__ENETC0_BASE.NUM_SI[3].PSIVHFR0.R /* Port station interface 3 VLAN hash filter register 0 */
#define NETC__ENETC0_BASE_PSI3VHFR1 NETC__ENETC0_BASE.NUM_SI[3].PSIVHFR1.R /* Port station interface 3 VLAN hash filter register 1 */
#define NETC__ENETC0_BASE_PSI4PMAR0 NETC__ENETC0_BASE.NUM_SI[4].PSIPMAR0.R /* Port station interface 4 primary MAC address register 0 */
#define NETC__ENETC0_BASE_PSI4PMAR1 NETC__ENETC0_BASE.NUM_SI[4].PSIPMAR1.R /* Port station interface 4 primary MAC address register 1 */
#define NETC__ENETC0_BASE_PSI4VLANR NETC__ENETC0_BASE.NUM_SI[4].PSIVLANR.R /* Port station interface 4 VLAN register */
#define NETC__ENETC0_BASE_PSI4CFGR0 NETC__ENETC0_BASE.NUM_SI[4].PSICFGR0.R /* Port station interface 4 configuration register 0 */
#define NETC__ENETC0_BASE_PSI4CFGR1 NETC__ENETC0_BASE.NUM_SI[4].PSICFGR1.R /* Port station interface 4 configuration register 1 */
#define NETC__ENETC0_BASE_PSI4CFGR2 NETC__ENETC0_BASE.NUM_SI[4].PSICFGR2.R /* Port station interface 4 configuration register 2 */
#define NETC__ENETC0_BASE_PSI4VMAFCFGR NETC__ENETC0_BASE.NUM_SI[4].PSIVMAFCFGR.R /* Port station interface 4 VSI MAC address filtering configuration register */
#define NETC__ENETC0_BASE_PSI4VLANFCFGR NETC__ENETC0_BASE.NUM_SI[4].PSIVLANFCFGR.R /* Port station interface 4 VLAN filtering configuration register */
#define NETC__ENETC0_BASE_PSI4RFSCFGR NETC__ENETC0_BASE.NUM_SI[4].PSIRFSCFGR.R /* Port station interface 4 RFS configuration register */
#define NETC__ENETC0_BASE_PSI4UMHFR0 NETC__ENETC0_BASE.NUM_SI[4].PSIUMHFR0.R /* Port station interface 4 unicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI4UMHFR1 NETC__ENETC0_BASE.NUM_SI[4].PSIUMHFR1.R /* Port station interface 4 unicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI4MMHFR0 NETC__ENETC0_BASE.NUM_SI[4].PSIMMHFR0.R /* Port station interface 4 multicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI4MMHFR1 NETC__ENETC0_BASE.NUM_SI[4].PSIMMHFR1.R /* Port station interface 4 multicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI4VHFR0 NETC__ENETC0_BASE.NUM_SI[4].PSIVHFR0.R /* Port station interface 4 VLAN hash filter register 0 */
#define NETC__ENETC0_BASE_PSI4VHFR1 NETC__ENETC0_BASE.NUM_SI[4].PSIVHFR1.R /* Port station interface 4 VLAN hash filter register 1 */
#define NETC__ENETC0_BASE_PSI5PMAR0 NETC__ENETC0_BASE.NUM_SI[5].PSIPMAR0.R /* Port station interface 5 primary MAC address register 0 */
#define NETC__ENETC0_BASE_PSI5PMAR1 NETC__ENETC0_BASE.NUM_SI[5].PSIPMAR1.R /* Port station interface 5 primary MAC address register 1 */
#define NETC__ENETC0_BASE_PSI5VLANR NETC__ENETC0_BASE.NUM_SI[5].PSIVLANR.R /* Port station interface 5 VLAN register */
#define NETC__ENETC0_BASE_PSI5CFGR0 NETC__ENETC0_BASE.NUM_SI[5].PSICFGR0.R /* Port station interface 5 configuration register 0 */
#define NETC__ENETC0_BASE_PSI5CFGR1 NETC__ENETC0_BASE.NUM_SI[5].PSICFGR1.R /* Port station interface 5 configuration register 1 */
#define NETC__ENETC0_BASE_PSI5CFGR2 NETC__ENETC0_BASE.NUM_SI[5].PSICFGR2.R /* Port station interface 5 configuration register 2 */
#define NETC__ENETC0_BASE_PSI5VMAFCFGR NETC__ENETC0_BASE.NUM_SI[5].PSIVMAFCFGR.R /* Port station interface 5 VSI MAC address filtering configuration register */
#define NETC__ENETC0_BASE_PSI5VLANFCFGR NETC__ENETC0_BASE.NUM_SI[5].PSIVLANFCFGR.R /* Port station interface 5 VLAN filtering configuration register */
#define NETC__ENETC0_BASE_PSI5RFSCFGR NETC__ENETC0_BASE.NUM_SI[5].PSIRFSCFGR.R /* Port station interface 5 RFS configuration register */
#define NETC__ENETC0_BASE_PSI5UMHFR0 NETC__ENETC0_BASE.NUM_SI[5].PSIUMHFR0.R /* Port station interface 5 unicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI5UMHFR1 NETC__ENETC0_BASE.NUM_SI[5].PSIUMHFR1.R /* Port station interface 5 unicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI5MMHFR0 NETC__ENETC0_BASE.NUM_SI[5].PSIMMHFR0.R /* Port station interface 5 multicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI5MMHFR1 NETC__ENETC0_BASE.NUM_SI[5].PSIMMHFR1.R /* Port station interface 5 multicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI5VHFR0 NETC__ENETC0_BASE.NUM_SI[5].PSIVHFR0.R /* Port station interface 5 VLAN hash filter register 0 */
#define NETC__ENETC0_BASE_PSI5VHFR1 NETC__ENETC0_BASE.NUM_SI[5].PSIVHFR1.R /* Port station interface 5 VLAN hash filter register 1 */
#define NETC__ENETC0_BASE_PSI6PMAR0 NETC__ENETC0_BASE.NUM_SI[6].PSIPMAR0.R /* Port station interface 6 primary MAC address register 0 */
#define NETC__ENETC0_BASE_PSI6PMAR1 NETC__ENETC0_BASE.NUM_SI[6].PSIPMAR1.R /* Port station interface 6 primary MAC address register 1 */
#define NETC__ENETC0_BASE_PSI6VLANR NETC__ENETC0_BASE.NUM_SI[6].PSIVLANR.R /* Port station interface 6 VLAN register */
#define NETC__ENETC0_BASE_PSI6CFGR0 NETC__ENETC0_BASE.NUM_SI[6].PSICFGR0.R /* Port station interface 6 configuration register 0 */
#define NETC__ENETC0_BASE_PSI6CFGR1 NETC__ENETC0_BASE.NUM_SI[6].PSICFGR1.R /* Port station interface 6 configuration register 1 */
#define NETC__ENETC0_BASE_PSI6CFGR2 NETC__ENETC0_BASE.NUM_SI[6].PSICFGR2.R /* Port station interface 6 configuration register 2 */
#define NETC__ENETC0_BASE_PSI6VMAFCFGR NETC__ENETC0_BASE.NUM_SI[6].PSIVMAFCFGR.R /* Port station interface 6 VSI MAC address filtering configuration register */
#define NETC__ENETC0_BASE_PSI6VLANFCFGR NETC__ENETC0_BASE.NUM_SI[6].PSIVLANFCFGR.R /* Port station interface 6 VLAN filtering configuration register */
#define NETC__ENETC0_BASE_PSI6RFSCFGR NETC__ENETC0_BASE.NUM_SI[6].PSIRFSCFGR.R /* Port station interface 6 RFS configuration register */
#define NETC__ENETC0_BASE_PSI6UMHFR0 NETC__ENETC0_BASE.NUM_SI[6].PSIUMHFR0.R /* Port station interface 6 unicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI6UMHFR1 NETC__ENETC0_BASE.NUM_SI[6].PSIUMHFR1.R /* Port station interface 6 unicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI6MMHFR0 NETC__ENETC0_BASE.NUM_SI[6].PSIMMHFR0.R /* Port station interface 6 multicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI6MMHFR1 NETC__ENETC0_BASE.NUM_SI[6].PSIMMHFR1.R /* Port station interface 6 multicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI6VHFR0 NETC__ENETC0_BASE.NUM_SI[6].PSIVHFR0.R /* Port station interface 6 VLAN hash filter register 0 */
#define NETC__ENETC0_BASE_PSI6VHFR1 NETC__ENETC0_BASE.NUM_SI[6].PSIVHFR1.R /* Port station interface 6 VLAN hash filter register 1 */
#define NETC__ENETC0_BASE_PSI7PMAR0 NETC__ENETC0_BASE.NUM_SI[7].PSIPMAR0.R /* Port station interface 7 primary MAC address register 0 */
#define NETC__ENETC0_BASE_PSI7PMAR1 NETC__ENETC0_BASE.NUM_SI[7].PSIPMAR1.R /* Port station interface 7 primary MAC address register 1 */
#define NETC__ENETC0_BASE_PSI7VLANR NETC__ENETC0_BASE.NUM_SI[7].PSIVLANR.R /* Port station interface 7 VLAN register */
#define NETC__ENETC0_BASE_PSI7CFGR0 NETC__ENETC0_BASE.NUM_SI[7].PSICFGR0.R /* Port station interface 7 configuration register 0 */
#define NETC__ENETC0_BASE_PSI7CFGR1 NETC__ENETC0_BASE.NUM_SI[7].PSICFGR1.R /* Port station interface 7 configuration register 1 */
#define NETC__ENETC0_BASE_PSI7CFGR2 NETC__ENETC0_BASE.NUM_SI[7].PSICFGR2.R /* Port station interface 7 configuration register 2 */
#define NETC__ENETC0_BASE_PSI7VMAFCFGR NETC__ENETC0_BASE.NUM_SI[7].PSIVMAFCFGR.R /* Port station interface 7 VSI MAC address filtering configuration register */
#define NETC__ENETC0_BASE_PSI7VLANFCFGR NETC__ENETC0_BASE.NUM_SI[7].PSIVLANFCFGR.R /* Port station interface 7 VLAN filtering configuration register */
#define NETC__ENETC0_BASE_PSI7RFSCFGR NETC__ENETC0_BASE.NUM_SI[7].PSIRFSCFGR.R /* Port station interface 7 RFS configuration register */
#define NETC__ENETC0_BASE_PSI7UMHFR0 NETC__ENETC0_BASE.NUM_SI[7].PSIUMHFR0.R /* Port station interface 7 unicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI7UMHFR1 NETC__ENETC0_BASE.NUM_SI[7].PSIUMHFR1.R /* Port station interface 7 unicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI7MMHFR0 NETC__ENETC0_BASE.NUM_SI[7].PSIMMHFR0.R /* Port station interface 7 multicast MAC hash filter register 0 */
#define NETC__ENETC0_BASE_PSI7MMHFR1 NETC__ENETC0_BASE.NUM_SI[7].PSIMMHFR1.R /* Port station interface 7 multicast MAC hash filter register 1 */
#define NETC__ENETC0_BASE_PSI7VHFR0 NETC__ENETC0_BASE.NUM_SI[7].PSIVHFR0.R /* Port station interface 7 VLAN hash filter register 0 */
#define NETC__ENETC0_BASE_PSI7VHFR1 NETC__ENETC0_BASE.NUM_SI[7].PSIVHFR1.R /* Port station interface 7 VLAN hash filter register 1 */

/* NETC_F3_COMMON */
#define NETC__ENETC0_COMMON_IPCAPR NETC__ENETC0_COMMON.IPCAPR.R /* Ingress port capability register */
#define NETC__ENETC0_COMMON_EPCAPR NETC__ENETC0_COMMON.EPCAPR.R /* Egress port capability register */
#define NETC__ENETC0_COMMON_OSR NETC__ENETC0_COMMON.OSR.R  /* Operational state register */
#define NETC__ENETC0_COMMON_CMECR NETC__ENETC0_COMMON.CMECR.R /* Correctable memory error configuration register */
#define NETC__ENETC0_COMMON_CMESR NETC__ENETC0_COMMON.CMESR.R /* Correctable memory error status register */
#define NETC__ENETC0_COMMON_CMECTR NETC__ENETC0_COMMON.CMECTR.R /* Correctable memory error count register */
#define NETC__ENETC0_COMMON_UNMACECR NETC__ENETC0_COMMON.UNMACECR.R /* Uncorrectable non-fatal MAC error configuration register */
#define NETC__ENETC0_COMMON_UNMACESR NETC__ENETC0_COMMON.UNMACESR.R /* Uncorrectable non-fatal MAC error status register */
#define NETC__ENETC0_COMMON_UNMECR NETC__ENETC0_COMMON.UNMECR.R /* Uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_COMMON_UNMESR0 NETC__ENETC0_COMMON.UNMESR0.R /* Uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_COMMON_UNMESR1 NETC__ENETC0_COMMON.UNMESR1.R /* Uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_COMMON_UNMECTR NETC__ENETC0_COMMON.UNMECTR.R /* Uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_COMMON_UFMECR NETC__ENETC0_COMMON.UFMECR.R /* Uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_COMMON_UFMESR0 NETC__ENETC0_COMMON.UFMESR0.R /* Uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_COMMON_UFMESR1 NETC__ENETC0_COMMON.UFMESR1.R /* Uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_COMMON_UNIECR NETC__ENETC0_COMMON.UNIECR.R /* Uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_COMMON_UNIESR NETC__ENETC0_COMMON.UNIESR.R /* Uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_COMMON_UNIECTR NETC__ENETC0_COMMON.UNIECTR.R /* Uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_COMMON_UFIECR NETC__ENETC0_COMMON.UFIECR.R /* Uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_COMMON_UFIESR NETC__ENETC0_COMMON.UFIESR.R /* Uncorrectable fatal integrity error status register */
#define NETC__ENETC0_COMMON_CVLANR1 NETC__ENETC0_COMMON.CVLANR1.R /* Custom VLAN EtherType register 1 */
#define NETC__ENETC0_COMMON_CVLANR2 NETC__ENETC0_COMMON.CVLANR2.R /* Custom VLAN EtherType register 2 */
#define NETC__ENETC0_COMMON_DOSL2CR NETC__ENETC0_COMMON.DOSL2CR.R /* DoS L2 configuration register */
#define NETC__ENETC0_COMMON_VLANIPVMP0R0 NETC__ENETC0_COMMON.NUM_PROFILE[0].VLANIPVMPR0.R /* VLAN to IPV mapping profile 0 register 0 */
#define NETC__ENETC0_COMMON_VLANIPVMP0R1 NETC__ENETC0_COMMON.NUM_PROFILE[0].VLANIPVMPR1.R /* VLAN to IPV mapping profile 0 register 1 */
#define NETC__ENETC0_COMMON_VLANDRMP0R NETC__ENETC0_COMMON.NUM_PROFILE[0].VLANDRMPR.R /* VLAN to DR mapping profile 0 register */
#define NETC__ENETC0_COMMON_IPFCAPR NETC__ENETC0_COMMON.IPFCAPR.R /* Ingress port filter capability register */
#define NETC__ENETC0_COMMON_IPFTCAPR NETC__ENETC0_COMMON.IPFTCAPR.R /* Ingress port filter table capability register */
#define NETC__ENETC0_COMMON_IPFTMOR NETC__ENETC0_COMMON.IPFTMOR.R /* Ingress port filter table memory operational register */
#define NETC__ENETC0_COMMON_ITMCAPR NETC__ENETC0_COMMON.ITMCAPR.R /* Index table memory capability register */
#define NETC__ENETC0_COMMON_RPCAPR NETC__ENETC0_COMMON.RPCAPR.R /* Rate policer capability register */
#define NETC__ENETC0_COMMON_RPITCAPR NETC__ENETC0_COMMON.RPITCAPR.R /* Rate policer index table capability register */
#define NETC__ENETC0_COMMON_RPITMAR NETC__ENETC0_COMMON.RPITMAR.R /* Rate policer index table memory allocation register */
#define NETC__ENETC0_COMMON_RPITOR NETC__ENETC0_COMMON.RPITOR.R /* Rate policer index table operational register */
#define NETC__ENETC0_COMMON_ISCITCAPR NETC__ENETC0_COMMON.ISCITCAPR.R /* Ingress stream counter index table capability register */
#define NETC__ENETC0_COMMON_ISCITMAR NETC__ENETC0_COMMON.ISCITMAR.R /* Ingress stream counter index table memory allocation register */
#define NETC__ENETC0_COMMON_ISCITOR NETC__ENETC0_COMMON.ISCITOR.R /* Ingress stream counter index table operational register */
#define NETC__ENETC0_COMMON_ISCAPR NETC__ENETC0_COMMON.ISCAPR.R /* Ingress stream capability register */
#define NETC__ENETC0_COMMON_ISITCAPR NETC__ENETC0_COMMON.ISITCAPR.R /* Ingress stream index table capability register */
#define NETC__ENETC0_COMMON_ISITMAR NETC__ENETC0_COMMON.ISITMAR.R /* Ingress stream index table memory allocation register */
#define NETC__ENETC0_COMMON_ISITOR NETC__ENETC0_COMMON.ISITOR.R /* Ingress stream index table operational register */
#define NETC__ENETC0_COMMON_SGCAPR NETC__ENETC0_COMMON.SGCAPR.R /* Stream gate capability register */
#define NETC__ENETC0_COMMON_SGIITCAPR NETC__ENETC0_COMMON.SGIITCAPR.R /* Stream gate instance index table capability register */
#define NETC__ENETC0_COMMON_SGIITMAR NETC__ENETC0_COMMON.SGIITMAR.R /* Stream gate instance index table memory allocation register */
#define NETC__ENETC0_COMMON_SGIITOR NETC__ENETC0_COMMON.SGIITOR.R /* Stream gate instance index table operational register */
#define NETC__ENETC0_COMMON_SGCLITCAPR NETC__ENETC0_COMMON.SGCLITCAPR.R /* Stream gate control list index table capability register */
#define NETC__ENETC0_COMMON_SGCLITMAR NETC__ENETC0_COMMON.SGCLITMAR.R /* Stream gate control list index table memory allocation register */
#define NETC__ENETC0_COMMON_SGCLTMOR NETC__ENETC0_COMMON.SGCLTMOR.R /* Stream gate control list table memory operational register */
#define NETC__ENETC0_COMMON_TGSTCAPR NETC__ENETC0_COMMON.TGSTCAPR.R /* Time gate scheduling table capability register */
#define NETC__ENETC0_COMMON_TGSTMOR NETC__ENETC0_COMMON.TGSTMOR.R /* Time gate scheduling table memory operation register */
#define NETC__ENETC0_COMMON_HTMCAPR NETC__ENETC0_COMMON.HTMCAPR.R /* Hash table memory capability register */
#define NETC__ENETC0_COMMON_HTMOR NETC__ENETC0_COMMON.HTMOR.R /* Hash table memory operational register */
#define NETC__ENETC0_COMMON_ISIDCAPR NETC__ENETC0_COMMON.ISIDCAPR.R /* Ingress stream identification capability register */
#define NETC__ENETC0_COMMON_ISIDHTCAPR NETC__ENETC0_COMMON.ISIDHTCAPR.R /* Ingress stream identification hash table capability register */
#define NETC__ENETC0_COMMON_ISIDKC0OR NETC__ENETC0_COMMON.ISIDKC0OR.R /* Ingress stream identification key construction 0 operational register */
#define NETC__ENETC0_COMMON_ISIDKC0CR0 NETC__ENETC0_COMMON.ISIDKC0CR0.R /* Ingress stream identification key construction 0 configuration register 0 */
#define NETC__ENETC0_COMMON_ISIDKC0PF0CR NETC__ENETC0_COMMON.ISIDKC0PF0CR.R /* Ingress stream identification key construction 0 payload field 0 configuration register */
#define NETC__ENETC0_COMMON_ISIDKC0PF1CR NETC__ENETC0_COMMON.ISIDKC0PF1CR.R /* Ingress stream identification key construction 0 payload field 1 configuration register */
#define NETC__ENETC0_COMMON_ISIDKC0PF2CR NETC__ENETC0_COMMON.ISIDKC0PF2CR.R /* Ingress stream identification key construction 0 payload field 2 configuration register */
#define NETC__ENETC0_COMMON_ISIDKC0PF3CR NETC__ENETC0_COMMON.ISIDKC0PF3CR.R /* Ingress stream identification key construction 0 payload field 3 configuration register */
#define NETC__ENETC0_COMMON_ISIDKC1OR NETC__ENETC0_COMMON.ISIDKC1OR.R /* Ingress stream identification key construction 1 operational register */
#define NETC__ENETC0_COMMON_ISIDKC1CR0 NETC__ENETC0_COMMON.ISIDKC1CR0.R /* Ingress stream identification key construction 1 configuration register 0 */
#define NETC__ENETC0_COMMON_ISIDKC1PF0CR NETC__ENETC0_COMMON.ISIDKC1PF0CR.R /* Ingress stream identification key construction 1 payload field 0 configuration register */
#define NETC__ENETC0_COMMON_ISIDKC1PF1CR NETC__ENETC0_COMMON.ISIDKC1PF1CR.R /* Ingress stream identification key construction 1 payload field 1 configuration register */
#define NETC__ENETC0_COMMON_ISIDKC1PF2CR NETC__ENETC0_COMMON.ISIDKC1PF2CR.R /* Ingress stream identification key construction 1 payload field 2 configuration register */
#define NETC__ENETC0_COMMON_ISIDKC1PF3CR NETC__ENETC0_COMMON.ISIDKC1PF3CR.R /* Ingress stream identification key construction 1 payload field 3 configuration register */
#define NETC__ENETC0_COMMON_ISFHTCAPR NETC__ENETC0_COMMON.ISFHTCAPR.R /* Ingress stream filter hash table capability register */
#define NETC__ENETC0_COMMON_ISFHTOR NETC__ENETC0_COMMON.ISFHTOR.R /* Ingress stream filter hash table operational register */

/* NETC_F3_GLOBAL */
#define NETC__ENETC0_GLOBAL_SMCAPR NETC__ENETC0_GLOBAL.SMCAPR.R /* Shared memory capability register */
#define NETC__ENETC0_GLOBAL_SMDTR NETC__ENETC0_GLOBAL.SMDTR.R /* Shared memory depletion threshold register */
#define NETC__ENETC0_GLOBAL_SMACR NETC__ENETC0_GLOBAL.SMACR.R /* Shared memory available count register */
#define NETC__ENETC0_GLOBAL_SMCLWMR NETC__ENETC0_GLOBAL.SMCLWMR.R /* Shared memory count low watermark register */
#define NETC__ENETC0_GLOBAL_SMBUCR NETC__ENETC0_GLOBAL.SMBUCR.R /* Shared memory buffer unassigned count register */
#define NETC__ENETC0_GLOBAL_SMBUCHWMR NETC__ENETC0_GLOBAL.SMBUCHWMR.R /* Shared memory buffer unassigned count high watermark register */
#define NETC__ENETC0_GLOBAL_SMLCR NETC__ENETC0_GLOBAL.SMLCR.R /* Shared memory loss count register */
#define NETC__ENETC0_GLOBAL_HBTCAPR NETC__ENETC0_GLOBAL.HBTCAPR.R /* Hash bucket table capability register */
#define NETC__ENETC0_GLOBAL_HBTOR0 NETC__ENETC0_GLOBAL.HBTOR0.R /* Hash bucket table operational register 0 */
#define NETC__ENETC0_GLOBAL_HBTOR2 NETC__ENETC0_GLOBAL.HBTOR2.R /* Hash bucket table operational register 2 */
#define NETC__ENETC0_GLOBAL_SMERBCAPR NETC__ENETC0_GLOBAL.SMERBCAPR.R /* Shared memory ENETC receive buffer capability register */
#define NETC__ENETC0_GLOBAL_SMERBOR0 NETC__ENETC0_GLOBAL.SMERBOR0.R /* Shared memory ENETC receive buffer operational register 0 */
#define NETC__ENETC0_GLOBAL_SMERBOR1 NETC__ENETC0_GLOBAL.SMERBOR1.R /* Shared memory ENETC receive buffer operational 1 */
#define NETC__ENETC0_GLOBAL_PCE0OR NETC__ENETC0_GLOBAL.PCE_SL[0].PCEOR.R /* PCE 0 operational register */
#define NETC__ENETC0_GLOBAL_RFE0OR NETC__ENETC0_GLOBAL.PCE_SL[0].RFEOR.R /* Replication Forwarding Engine 0 operational register */
#define NETC__ENETC0_GLOBAL_NETCCLKR NETC__ENETC0_GLOBAL.NETCCLKR.R /* NETC clock register */
#define NETC__ENETC0_GLOBAL_HTA0CAPR NETC__ENETC0_GLOBAL.HTA_LOOP[0].HTACAPR.R /* HTA 0 capability register */
#define NETC__ENETC0_GLOBAL_HTA0RFCOR NETC__ENETC0_GLOBAL.HTA_LOOP[0].HTARFCOR.R /* HTA 0 receive frame count operational register */
#define NETC__ENETC0_GLOBAL_HTA0HPBCOR NETC__ENETC0_GLOBAL.HTA_LOOP[0].HTAHPBCOR.R /* HTA 0 high priority byte count operational register */
#define NETC__ENETC0_GLOBAL_HTA0LPBCOR NETC__ENETC0_GLOBAL.HTA_LOOP[0].HTALPBCOR.R /* HTA 0 low priority byte count operational register */
#define NETC__ENETC0_GLOBAL_HTA0TFCOR NETC__ENETC0_GLOBAL.HTA_LOOP[0].HTATFCOR.R /* HTA 0 transmit frame count operational register */
#define NETC__ENETC0_GLOBAL_RC0SBRLAR NETC__ENETC0_GLOBAL.ARRAY_NUM_RC[0].RCSBRLAR.R /* Root complex 0 system bus read latency average register */
#define NETC__ENETC0_GLOBAL_RC0SBRLHWMR NETC__ENETC0_GLOBAL.ARRAY_NUM_RC[0].RCSBRLHWMR.R /* Root complex 0 system bus read latency high watermark register */
#define NETC__ENETC0_GLOBAL_RC0SBWLAR NETC__ENETC0_GLOBAL.ARRAY_NUM_RC[0].RCSBWLAR.R /* Root complex 0 system bus write latency average register */
#define NETC__ENETC0_GLOBAL_RC0SBWLHWMR NETC__ENETC0_GLOBAL.ARRAY_NUM_RC[0].RCSBWLHWMR.R /* Root complex 0 system bus write latency high watermark register */
#define NETC__ENETC0_GLOBAL_IPBRR0 NETC__ENETC0_GLOBAL.IPBRR0.R /* IP block revision register 0 */
#define NETC__ENETC0_GLOBAL_IPBRR1 NETC__ENETC0_GLOBAL.IPBRR1.R /* IP block revision register 1 */
#define NETC__ENETC0_GLOBAL_FBLPR0 NETC__ENETC0_GLOBAL.FBLPR[0].R /* Function boot loader parameter register 0 */
#define NETC__ENETC0_GLOBAL_FBLPR1 NETC__ENETC0_GLOBAL.FBLPR[1].R /* Function boot loader parameter register 1 */

/* NETC_F3_PCI_HDR_TYPE0 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP_LIST.R /* PCI PCI-PM capabilities list register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP.R /* PCI PCI-PM capabilities register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CTL_STAT.R /* PCI PCI-PM control and status register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_PCIPM_DATA.R /* PCI PCI-PM capabilities data register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_CAP_LIST NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_EA_CAP_LIST.R /* PCI EA capabilities list register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_CAP NETC__NETC_F3_PCI_HDR_TYPE0.PCI_CFC_EA_CAP.R /* PCI EA capabilities register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_FMT NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 0 format register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_BASE NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 0 base register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_MAXOFF NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 0 max offset register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE0_EXT_BASE NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[0].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 0 extended base register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_FMT NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 1 format register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_BASE NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 1 base register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_MAXOFF NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 1 max offset register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE1_EXT_BASE NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[1].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 1 extended base register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE2_FMT NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[2].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 2 format register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE2_BASE NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[2].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 2 base register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE2_MAXOFF NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[2].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 2 max offset register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE2_EXT_BASE NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[2].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 2 extended base register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE3_FMT NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[3].PCI_CFC_EA_PE_FMT.R /* PCI EA per-entry 3 format register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE3_BASE NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[3].PCI_CFC_EA_PE_BASE.R /* PCI EA per-entry 3 base register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE3_MAXOFF NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[3].PCI_CFC_EA_PE_MAXOFF.R /* PCI EA per-entry 3 max offset register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCI_CFC_EA_PE3_EXT_BASE NETC__NETC_F3_PCI_HDR_TYPE0.NUM_EA[3].PCI_CFC_EA_PE_EXT_BASE.R /* PCI EA per-entry 3 extended base register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CAP_HDR NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_CAP_HDR.R /* PCIe SR-IOV capability header */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CAP NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_CAP.R /* PCIe SR-IOV capability register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_CTL NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_CTL.R /* PCIe SR-IOV control register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_STAT NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_STAT.R /* PCIe SR-IOV status register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_INIT_VFS NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_INIT_VFS.R /* PCIe SR-IOV initial VFs register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_TOTAL_VFS.R /* PCIe SR-IOV total VFs register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_NUM_VFS NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_NUM_VFS.R /* PCIe SR-IOV num VFs register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_FUNC_DEP_LIST.R /* PCIe SR-IOV function dependency list register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_FIRST_VF_OFF.R /* PCIe SR-IOV first VF offset register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_VF_STRIDE.R /* PCIe SR-IOV VF stride register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_VF_DEV_ID.R /* PCIe SR-IOV VF device ID register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_SUP_PAGE_SIZES.R /* PCIe SR-IOV supported page sizes ID register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_SYS_PAGE_SIZE.R /* PCIe SR-IOV system page size ID register */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR0 NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_VF_BAR[0].R /* PCIe SR-IOV VF base address register 0 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR1 NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_VF_BAR[1].R /* PCIe SR-IOV VF base address register 1 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR2 NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_VF_BAR[2].R /* PCIe SR-IOV VF base address register 2 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR3 NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_VF_BAR[3].R /* PCIe SR-IOV VF base address register 3 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR4 NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_VF_BAR[4].R /* PCIe SR-IOV VF base address register 4 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_VF_BAR5 NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_VF_BAR[5].R /* PCIe SR-IOV VF base address register 5 */
#define NETC__NETC_F3_PCI_HDR_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF NETC__NETC_F3_PCI_HDR_TYPE0.PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF.R /* PCIe SR-IOV VF migration state array offset register */

/* NETC_F3_SI0 */
#define NETC__ENETC0_SI0_SIMR NETC__ENETC0_SI0.SIMR.R      /* Station interface mode register */
#define NETC__ENETC0_SI0_SISR NETC__ENETC0_SI0.SISR.R      /* Station interface status register */
#define NETC__ENETC0_SI0_SICTR0 NETC__ENETC0_SI0.SICTR0.R  /* Station interface current time register 0 */
#define NETC__ENETC0_SI0_SICTR1 NETC__ENETC0_SI0.SICTR1.R  /* Station interface current time register 1 */
#define NETC__ENETC0_SI0_SIPCAPR0 NETC__ENETC0_SI0.SIPCAPR0.R /* Station interface port capability register 0 */
#define NETC__ENETC0_SI0_SIPCAPR1 NETC__ENETC0_SI0.SIPCAPR1.R /* Station interface port capability register 1 */
#define NETC__ENETC0_SI0_SITSR NETC__ENETC0_SI0.SITSR.R    /* Station interface timer status register */
#define NETC__ENETC0_SI0_SIRBGCR NETC__ENETC0_SI0.SIRBGCR.R /* Station interface receive BDR group control register */
#define NETC__ENETC0_SI0_SIBCAR NETC__ENETC0_SI0.SIBCAR.R  /* Station interface buffer cache attribute register */
#define NETC__ENETC0_SI0_SIMCAR NETC__ENETC0_SI0.SIMCAR.R  /* Station interface message cache attribute register */
#define NETC__ENETC0_SI0_SICCAR NETC__ENETC0_SI0.SICCAR.R  /* Station interface command cache attribute register */
#define NETC__ENETC0_SI0_SIPMAR0 NETC__ENETC0_SI0.SIPMAR0.R /* Station interface primary MAC address register 0 */
#define NETC__ENETC0_SI0_SIPMAR1 NETC__ENETC0_SI0.SIPMAR1.R /* Station interface primary MAC address register 1 */
#define NETC__ENETC0_SI0_SICVLANR1 NETC__ENETC0_SI0.SICVLANR1.R /* Station interface custom VLAN register 1 */
#define NETC__ENETC0_SI0_SICVLANR2 NETC__ENETC0_SI0.SICVLANR2.R /* Station interface custom VLAN register 2 */
#define NETC__ENETC0_SI0_SIVLANIPVMR0 NETC__ENETC0_SI0.SIVLANIPVMR0.R /* Station interface VLAN to IPV mapping register 0 */
#define NETC__ENETC0_SI0_SIVLANIPVMR1 NETC__ENETC0_SI0.SIVLANIPVMR1.R /* Station interface VLAN to IPV mapping register 1 */
#define NETC__ENETC0_SI0_SIIPVBDRMR0 NETC__ENETC0_SI0.SIIPVBDRMR0.R /* Station interface IPV to ring mapping register */
#define NETC__ENETC0_SI0_PSIMSGRR NETC__ENETC0_SI0.PSI_A.PSIMSGRR.R /* Physical station interface message receive register */
#define NETC__ENETC0_SI0_PSIMSGSR NETC__ENETC0_SI0.PSI_A.PSIMSGSR.R /* Physical station interface message send register */
#define NETC__ENETC0_SI0_PSIV1MSGRCVAR0 NETC__ENETC0_SI0.PSI_A.VSI_NUM[0].PSIVMSGRCVAR0.R /* PSI VSI 1 message receive address register 0 */
#define NETC__ENETC0_SI0_PSIV1MSGRCVAR1 NETC__ENETC0_SI0.PSI_A.VSI_NUM[0].PSIVMSGRCVAR1.R /* PSI VSI 1 message receive address register 1 */
#define NETC__ENETC0_SI0_PSIV2MSGRCVAR0 NETC__ENETC0_SI0.PSI_A.VSI_NUM[1].PSIVMSGRCVAR0.R /* PSI VSI 2 message receive address register 0 */
#define NETC__ENETC0_SI0_PSIV2MSGRCVAR1 NETC__ENETC0_SI0.PSI_A.VSI_NUM[1].PSIVMSGRCVAR1.R /* PSI VSI 2 message receive address register 1 */
#define NETC__ENETC0_SI0_PSIV3MSGRCVAR0 NETC__ENETC0_SI0.PSI_A.VSI_NUM[2].PSIVMSGRCVAR0.R /* PSI VSI 3 message receive address register 0 */
#define NETC__ENETC0_SI0_PSIV3MSGRCVAR1 NETC__ENETC0_SI0.PSI_A.VSI_NUM[2].PSIVMSGRCVAR1.R /* PSI VSI 3 message receive address register 1 */
#define NETC__ENETC0_SI0_PSIV4MSGRCVAR0 NETC__ENETC0_SI0.PSI_A.VSI_NUM[3].PSIVMSGRCVAR0.R /* PSI VSI 4 message receive address register 0 */
#define NETC__ENETC0_SI0_PSIV4MSGRCVAR1 NETC__ENETC0_SI0.PSI_A.VSI_NUM[3].PSIVMSGRCVAR1.R /* PSI VSI 4 message receive address register 1 */
#define NETC__ENETC0_SI0_PSIV5MSGRCVAR0 NETC__ENETC0_SI0.PSI_A.VSI_NUM[4].PSIVMSGRCVAR0.R /* PSI VSI 5 message receive address register 0 */
#define NETC__ENETC0_SI0_PSIV5MSGRCVAR1 NETC__ENETC0_SI0.PSI_A.VSI_NUM[4].PSIVMSGRCVAR1.R /* PSI VSI 5 message receive address register 1 */
#define NETC__ENETC0_SI0_PSIV6MSGRCVAR0 NETC__ENETC0_SI0.PSI_A.VSI_NUM[5].PSIVMSGRCVAR0.R /* PSI VSI 6 message receive address register 0 */
#define NETC__ENETC0_SI0_PSIV6MSGRCVAR1 NETC__ENETC0_SI0.PSI_A.VSI_NUM[5].PSIVMSGRCVAR1.R /* PSI VSI 6 message receive address register 1 */
#define NETC__ENETC0_SI0_PSIV7MSGRCVAR0 NETC__ENETC0_SI0.PSI_A.VSI_NUM[6].PSIVMSGRCVAR0.R /* PSI VSI 7 message receive address register 0 */
#define NETC__ENETC0_SI0_PSIV7MSGRCVAR1 NETC__ENETC0_SI0.PSI_A.VSI_NUM[6].PSIVMSGRCVAR1.R /* PSI VSI 7 message receive address register 1 */
#define NETC__ENETC0_SI0_SIROCT0 NETC__ENETC0_SI0.SIROCT0.R /* Station interface receive octets counter (ifInOctets) 0 */
#define NETC__ENETC0_SI0_SIROCT1 NETC__ENETC0_SI0.SIROCT1.R /* Station interface receive octets counter (ifInOctets) 1 */
#define NETC__ENETC0_SI0_SIRFRM0 NETC__ENETC0_SI0.SIRFRM0.R /* Station interface receive frame counter (aFrameReceivedOK) 0 */
#define NETC__ENETC0_SI0_SIRFRM1 NETC__ENETC0_SI0.SIRFRM1.R /* Station interface receive frame counter (aFrameReceivedOK) 1 */
#define NETC__ENETC0_SI0_SIRUCA0 NETC__ENETC0_SI0.SIRUCA0.R /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
#define NETC__ENETC0_SI0_SIRUCA1 NETC__ENETC0_SI0.SIRUCA1.R /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
#define NETC__ENETC0_SI0_SIRMCA0 NETC__ENETC0_SI0.SIRMCA0.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
#define NETC__ENETC0_SI0_SIRMCA1 NETC__ENETC0_SI0.SIRMCA1.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
#define NETC__ENETC0_SI0_SITOCT0 NETC__ENETC0_SI0.SITOCT0.R /* Station interface transmit octets counter (ifOutOctets) 0 */
#define NETC__ENETC0_SI0_SITOCT1 NETC__ENETC0_SI0.SITOCT1.R /* Station interface transmit octets counter (ifOutOctets) 1 */
#define NETC__ENETC0_SI0_SITFRM0 NETC__ENETC0_SI0.SITFRM0.R /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
#define NETC__ENETC0_SI0_SITFRM1 NETC__ENETC0_SI0.SITFRM1.R /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
#define NETC__ENETC0_SI0_SITUCA0 NETC__ENETC0_SI0.SITUCA0.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
#define NETC__ENETC0_SI0_SITUCA1 NETC__ENETC0_SI0.SITUCA1.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
#define NETC__ENETC0_SI0_SITMCA0 NETC__ENETC0_SI0.SITMCA0.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
#define NETC__ENETC0_SI0_SITMCA1 NETC__ENETC0_SI0.SITMCA1.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
#define NETC__ENETC0_SI0_SICBDRMR NETC__ENETC0_SI0.SICBDRMR.R /* Station interface command BDR mode register */
#define NETC__ENETC0_SI0_SICBDRSR NETC__ENETC0_SI0.SICBDRSR.R /* Station interface command BDR status register */
#define NETC__ENETC0_SI0_SICBDRBAR0 NETC__ENETC0_SI0.SICBDRBAR0.R /* Station interface command BDR base address register 0 */
#define NETC__ENETC0_SI0_SICBDRBAR1 NETC__ENETC0_SI0.SICBDRBAR1.R /* Station interface command BDR base address register 1 */
#define NETC__ENETC0_SI0_SICBDRPIR NETC__ENETC0_SI0.SICBDRPIR.R /* Station interface command BDR producer index register */
#define NETC__ENETC0_SI0_SICBDRCIR NETC__ENETC0_SI0.SICBDRCIR.R /* Station interface command BDR consumer index register */
#define NETC__ENETC0_SI0_SICBDRLENR NETC__ENETC0_SI0.SICBDRLENR.R /* Station interface command BDR length register */
#define NETC__ENETC0_SI0_SICBDRIER NETC__ENETC0_SI0.SICBDRIER.R /* Station interface command BDR interrupt enable register */
#define NETC__ENETC0_SI0_SICBDRIDR NETC__ENETC0_SI0.SICBDRIDR.R /* Station interface command BDR interrupt detect register */
#define NETC__ENETC0_SI0_SICAPR0 NETC__ENETC0_SI0.SICAPR0.R /* Station interface capability register 0 */
#define NETC__ENETC0_SI0_SICAPR1 NETC__ENETC0_SI0.SICAPR1.R /* Station interface capability register 1 */
#define NETC__ENETC0_SI0_SICAPR2 NETC__ENETC0_SI0.SICAPR2.R /* Station interface capability register 2 */
#define NETC__ENETC0_SI0_PSIIER NETC__ENETC0_SI0.PSI.PSIIER.R /* Physical station interface interrupt enable register */
#define NETC__ENETC0_SI0_PSIIDR NETC__ENETC0_SI0.PSI.PSIIDR.R /* Physical station interface interrupt detect register */
#define NETC__ENETC0_SI0_SITXIDR0 NETC__ENETC0_SI0.SITXIDR0.R /* Station interface transmit interrupt detect register 0 */
#define NETC__ENETC0_SI0_SITXIDR1 NETC__ENETC0_SI0.SITXIDR1.R /* Station interface transmit interrupt detect register 1 */
#define NETC__ENETC0_SI0_SIRXIDR0 NETC__ENETC0_SI0.SIRXIDR0.R /* Station interface receive interrupt detect register 0 */
#define NETC__ENETC0_SI0_SIRXIDR1 NETC__ENETC0_SI0.SIRXIDR1.R /* Station interface receive interrupt detect register 1 */
#define NETC__ENETC0_SI0_SIMSIVR NETC__ENETC0_SI0.SIMSIVR.R /* Station interface MSI-X vector register */
#define NETC__ENETC0_SI0_SICMSIVR NETC__ENETC0_SI0.SICMSIVR.R /* Station interface command MSI-X vector register */
#define NETC__ENETC0_SI0_SITMRIER NETC__ENETC0_SI0.SITMRIER.R /* Station interface timer interrupt enable register */
#define NETC__ENETC0_SI0_SITMRIDR NETC__ENETC0_SI0.SITMRIDR.R /* Station interface timer interrupt detect register */
#define NETC__ENETC0_SI0_SITMRMSIVR NETC__ENETC0_SI0.SITMRMSIVR.R /* Station interface timer MSI-X vector register */
#define NETC__ENETC0_SI0_SIMSITR0VR NETC__ENETC0_SI0.SIMSITRVR[0].R /* Station interface MSI-X transmit ring 0 vector register */
#define NETC__ENETC0_SI0_SIMSITR1VR NETC__ENETC0_SI0.SIMSITRVR[1].R /* Station interface MSI-X transmit ring 1 vector register */
#define NETC__ENETC0_SI0_SIMSITR2VR NETC__ENETC0_SI0.SIMSITRVR[2].R /* Station interface MSI-X transmit ring 2 vector register */
#define NETC__ENETC0_SI0_SIMSITR3VR NETC__ENETC0_SI0.SIMSITRVR[3].R /* Station interface MSI-X transmit ring 3 vector register */
#define NETC__ENETC0_SI0_SIMSITR4VR NETC__ENETC0_SI0.SIMSITRVR[4].R /* Station interface MSI-X transmit ring 4 vector register */
#define NETC__ENETC0_SI0_SIMSITR5VR NETC__ENETC0_SI0.SIMSITRVR[5].R /* Station interface MSI-X transmit ring 5 vector register */
#define NETC__ENETC0_SI0_SIMSITR6VR NETC__ENETC0_SI0.SIMSITRVR[6].R /* Station interface MSI-X transmit ring 6 vector register */
#define NETC__ENETC0_SI0_SIMSITR7VR NETC__ENETC0_SI0.SIMSITRVR[7].R /* Station interface MSI-X transmit ring 7 vector register */
#define NETC__ENETC0_SI0_SIMSITR8VR NETC__ENETC0_SI0.SIMSITRVR[8].R /* Station interface MSI-X transmit ring 8 vector register */
#define NETC__ENETC0_SI0_SIMSITR9VR NETC__ENETC0_SI0.SIMSITRVR[9].R /* Station interface MSI-X transmit ring 9 vector register */
#define NETC__ENETC0_SI0_SIMSITR10VR NETC__ENETC0_SI0.SIMSITRVR[10].R /* Station interface MSI-X transmit ring 10 vector register */
#define NETC__ENETC0_SI0_SIMSITR11VR NETC__ENETC0_SI0.SIMSITRVR[11].R /* Station interface MSI-X transmit ring 11 vector register */
#define NETC__ENETC0_SI0_SIMSITR12VR NETC__ENETC0_SI0.SIMSITRVR[12].R /* Station interface MSI-X transmit ring 12 vector register */
#define NETC__ENETC0_SI0_SIMSITR13VR NETC__ENETC0_SI0.SIMSITRVR[13].R /* Station interface MSI-X transmit ring 13 vector register */
#define NETC__ENETC0_SI0_SIMSITR14VR NETC__ENETC0_SI0.SIMSITRVR[14].R /* Station interface MSI-X transmit ring 14 vector register */
#define NETC__ENETC0_SI0_SIMSITR15VR NETC__ENETC0_SI0.SIMSITRVR[15].R /* Station interface MSI-X transmit ring 15 vector register */
#define NETC__ENETC0_SI0_SIMSITR16VR NETC__ENETC0_SI0.SIMSITRVR[16].R /* Station interface MSI-X transmit ring 16 vector register */
#define NETC__ENETC0_SI0_SIMSITR17VR NETC__ENETC0_SI0.SIMSITRVR[17].R /* Station interface MSI-X transmit ring 17 vector register */
#define NETC__ENETC0_SI0_SIMSIRR0VR NETC__ENETC0_SI0.SIMSIRRVR[0].R /* Station interface MSI-X receive ring 0 vector register */
#define NETC__ENETC0_SI0_SIMSIRR1VR NETC__ENETC0_SI0.SIMSIRRVR[1].R /* Station interface MSI-X receive ring 1 vector register */
#define NETC__ENETC0_SI0_SIMSIRR2VR NETC__ENETC0_SI0.SIMSIRRVR[2].R /* Station interface MSI-X receive ring 2 vector register */
#define NETC__ENETC0_SI0_SIMSIRR3VR NETC__ENETC0_SI0.SIMSIRRVR[3].R /* Station interface MSI-X receive ring 3 vector register */
#define NETC__ENETC0_SI0_SIMSIRR4VR NETC__ENETC0_SI0.SIMSIRRVR[4].R /* Station interface MSI-X receive ring 4 vector register */
#define NETC__ENETC0_SI0_SIMSIRR5VR NETC__ENETC0_SI0.SIMSIRRVR[5].R /* Station interface MSI-X receive ring 5 vector register */
#define NETC__ENETC0_SI0_SIMSIRR6VR NETC__ENETC0_SI0.SIMSIRRVR[6].R /* Station interface MSI-X receive ring 6 vector register */
#define NETC__ENETC0_SI0_SIMSIRR7VR NETC__ENETC0_SI0.SIMSIRRVR[7].R /* Station interface MSI-X receive ring 7 vector register */
#define NETC__ENETC0_SI0_SIMSIRR8VR NETC__ENETC0_SI0.SIMSIRRVR[8].R /* Station interface MSI-X receive ring 8 vector register */
#define NETC__ENETC0_SI0_SIMSIRR9VR NETC__ENETC0_SI0.SIMSIRRVR[9].R /* Station interface MSI-X receive ring 9 vector register */
#define NETC__ENETC0_SI0_SIMSIRR10VR NETC__ENETC0_SI0.SIMSIRRVR[10].R /* Station interface MSI-X receive ring 10 vector register */
#define NETC__ENETC0_SI0_SIMSIRR11VR NETC__ENETC0_SI0.SIMSIRRVR[11].R /* Station interface MSI-X receive ring 11 vector register */
#define NETC__ENETC0_SI0_SIMSIRR12VR NETC__ENETC0_SI0.SIMSIRRVR[12].R /* Station interface MSI-X receive ring 12 vector register */
#define NETC__ENETC0_SI0_SIMSIRR13VR NETC__ENETC0_SI0.SIMSIRRVR[13].R /* Station interface MSI-X receive ring 13 vector register */
#define NETC__ENETC0_SI0_SIMSIRR14VR NETC__ENETC0_SI0.SIMSIRRVR[14].R /* Station interface MSI-X receive ring 14 vector register */
#define NETC__ENETC0_SI0_SIMSIRR15VR NETC__ENETC0_SI0.SIMSIRRVR[15].R /* Station interface MSI-X receive ring 15 vector register */
#define NETC__ENETC0_SI0_SIMSIRR16VR NETC__ENETC0_SI0.SIMSIRRVR[16].R /* Station interface MSI-X receive ring 16 vector register */
#define NETC__ENETC0_SI0_SIMSIRR17VR NETC__ENETC0_SI0.SIMSIRRVR[17].R /* Station interface MSI-X receive ring 17 vector register */
#define NETC__ENETC0_SI0_SICMECR NETC__ENETC0_SI0.SICMECR.R /* Station interface correctable memory error configuration register */
#define NETC__ENETC0_SI0_SICMESR NETC__ENETC0_SI0.SICMESR.R /* Station interface correctable memory error status register */
#define NETC__ENETC0_SI0_SICMECTR NETC__ENETC0_SI0.SICMECTR.R /* Station interface correctable memory error count register */
#define NETC__ENETC0_SI0_SIUPECR NETC__ENETC0_SI0.SIUPECR.R /* Station interface uncorrectable programming error configuration register */
#define NETC__ENETC0_SI0_SIUPESR NETC__ENETC0_SI0.SIUPESR.R /* Station interface uncorrectable programming error status register */
#define NETC__ENETC0_SI0_SIUPECTR NETC__ENETC0_SI0.SIUPECTR.R /* Station interface uncorrectable programming error count register */
#define NETC__ENETC0_SI0_SIUNSBECR NETC__ENETC0_SI0.SIUNSBECR.R /* Station interface uncorrectable non-fatal system bus error configuration register */
#define NETC__ENETC0_SI0_SIUNSBESR NETC__ENETC0_SI0.SIUNSBESR.R /* Station interface uncorrectable non-fatal system bus error status register */
#define NETC__ENETC0_SI0_SIUNSBECTR NETC__ENETC0_SI0.SIUNSBECTR.R /* Station interface uncorrectable non-fatal system bus error count register */
#define NETC__ENETC0_SI0_SIUFSBECR NETC__ENETC0_SI0.SIUFSBECR.R /* Station interface uncorrectable fatal system bus error configuration register */
#define NETC__ENETC0_SI0_SIUFSBESR NETC__ENETC0_SI0.SIUFSBESR.R /* Station interface uncorrectable fatal system bus error status register */
#define NETC__ENETC0_SI0_SIUNMECR NETC__ENETC0_SI0.SIUNMECR.R /* Station interface uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_SI0_SIUNMESR0 NETC__ENETC0_SI0.SIUNMESR0.R /* Station interface uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_SI0_SIUNMESR1 NETC__ENETC0_SI0.SIUNMESR1.R /* Station interface uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_SI0_SIUNMECTR NETC__ENETC0_SI0.SIUNMECTR.R /* Station interface uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_SI0_SIUFMECR NETC__ENETC0_SI0.SIUFMECR.R /* Station interface uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_SI0_SIUFMESR0 NETC__ENETC0_SI0.SIUFMESR0.R /* Station interface uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_SI0_SIUFMESR1 NETC__ENETC0_SI0.SIUFMESR1.R /* Station interface uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_SI0_SIUNIECR NETC__ENETC0_SI0.SIUNIECR.R /* Station interface uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_SI0_SIUNIESR NETC__ENETC0_SI0.SIUNIESR.R /* Station interface uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_SI0_SIUNIECTR NETC__ENETC0_SI0.SIUNIECTR.R /* Station interface uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_SI0_SIUFIECR NETC__ENETC0_SI0.SIUFIECR.R /* Station interface uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_SI0_SIUFIESR NETC__ENETC0_SI0.SIUFIESR.R /* Station interface uncorrectable fatal integrity error status register */
#define NETC__ENETC0_SI0_SIMAFTCAPR NETC__ENETC0_SI0.SIMAFTCAPR.R /* Station interface MAC address filter table capability register */
#define NETC__ENETC0_SI0_SIVFTCAPR NETC__ENETC0_SI0.SIVFTCAPR.R /* Station interface VLAN filter table capability register */
#define NETC__ENETC0_SI0_SIRFSCAPR NETC__ENETC0_SI0.SIRFSCAPR.R /* Station interface RFS capability register */
#define NETC__ENETC0_SI0_TB0MR NETC__ENETC0_SI0.BDR_NUM[0].TBMR.R /* Tx BDR 0 mode register */
#define NETC__ENETC0_SI0_TB0SR NETC__ENETC0_SI0.BDR_NUM[0].TBSR.R /* Tx BDR 0 status register */
#define NETC__ENETC0_SI0_TB0BAR0 NETC__ENETC0_SI0.BDR_NUM[0].TBBAR0.R /* Tx BDR 0 base address register 0 */
#define NETC__ENETC0_SI0_TB0BAR1 NETC__ENETC0_SI0.BDR_NUM[0].TBBAR1.R /* Tx BDR 0 base address register 1 */
#define NETC__ENETC0_SI0_TB0PIR NETC__ENETC0_SI0.BDR_NUM[0].TBPIR.R /* Tx BDR 0 producer index register */
#define NETC__ENETC0_SI0_TB0CIR NETC__ENETC0_SI0.BDR_NUM[0].TBCIR.R /* Tx BDR 0 consumer index register */
#define NETC__ENETC0_SI0_TB0LENR NETC__ENETC0_SI0.BDR_NUM[0].TBLENR.R /* Tx BDR 0 length register */
#define NETC__ENETC0_SI0_TB0IER NETC__ENETC0_SI0.BDR_NUM[0].TBIER.R /* Tx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI0_TB0IDR NETC__ENETC0_SI0.BDR_NUM[0].TBIDR.R /* Tx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI0_TB0ICR0 NETC__ENETC0_SI0.BDR_NUM[0].TBICR0.R /* Tx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB0ICR1 NETC__ENETC0_SI0.BDR_NUM[0].TBICR1.R /* Tx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB0MR NETC__ENETC0_SI0.BDR_NUM[0].RBMR.R /* Rx BDR 0 mode register */
#define NETC__ENETC0_SI0_RB0SR NETC__ENETC0_SI0.BDR_NUM[0].RBSR.R /* Rx BDR 0 status register */
#define NETC__ENETC0_SI0_RB0BSR NETC__ENETC0_SI0.BDR_NUM[0].RBBSR.R /* Rx BDR 0 buffer size register */
#define NETC__ENETC0_SI0_RB0CIR NETC__ENETC0_SI0.BDR_NUM[0].RBCIR.R /* Rx BDR 0 consumer index register */
#define NETC__ENETC0_SI0_RB0BAR0 NETC__ENETC0_SI0.BDR_NUM[0].RBBAR0.R /* Rx BDR 0 base address register 0 */
#define NETC__ENETC0_SI0_RB0BAR1 NETC__ENETC0_SI0.BDR_NUM[0].RBBAR1.R /* Rx BDR 0 base address register 1 */
#define NETC__ENETC0_SI0_RB0PIR NETC__ENETC0_SI0.BDR_NUM[0].RBPIR.R /* Rx BDR 0 producer index register */
#define NETC__ENETC0_SI0_RB0LENR NETC__ENETC0_SI0.BDR_NUM[0].RBLENR.R /* Rx BDR 0 length register */
#define NETC__ENETC0_SI0_RB0DCR NETC__ENETC0_SI0.BDR_NUM[0].RBDCR.R /* Rx BDR 0 drop count register */
#define NETC__ENETC0_SI0_RB0IER NETC__ENETC0_SI0.BDR_NUM[0].RBIER.R /* Rx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI0_RB0IDR NETC__ENETC0_SI0.BDR_NUM[0].RBIDR.R /* Rx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI0_RB0ICR0 NETC__ENETC0_SI0.BDR_NUM[0].RBICR0.R /* Rx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB0ICR1 NETC__ENETC0_SI0.BDR_NUM[0].RBICR1.R /* Rx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB1MR NETC__ENETC0_SI0.BDR_NUM[1].TBMR.R /* Tx BDR 1 mode register */
#define NETC__ENETC0_SI0_TB1SR NETC__ENETC0_SI0.BDR_NUM[1].TBSR.R /* Tx BDR 1 status register */
#define NETC__ENETC0_SI0_TB1BAR0 NETC__ENETC0_SI0.BDR_NUM[1].TBBAR0.R /* Tx BDR 1 base address register 0 */
#define NETC__ENETC0_SI0_TB1BAR1 NETC__ENETC0_SI0.BDR_NUM[1].TBBAR1.R /* Tx BDR 1 base address register 1 */
#define NETC__ENETC0_SI0_TB1PIR NETC__ENETC0_SI0.BDR_NUM[1].TBPIR.R /* Tx BDR 1 producer index register */
#define NETC__ENETC0_SI0_TB1CIR NETC__ENETC0_SI0.BDR_NUM[1].TBCIR.R /* Tx BDR 1 consumer index register */
#define NETC__ENETC0_SI0_TB1LENR NETC__ENETC0_SI0.BDR_NUM[1].TBLENR.R /* Tx BDR 1 length register */
#define NETC__ENETC0_SI0_TB1IER NETC__ENETC0_SI0.BDR_NUM[1].TBIER.R /* Tx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI0_TB1IDR NETC__ENETC0_SI0.BDR_NUM[1].TBIDR.R /* Tx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI0_TB1ICR0 NETC__ENETC0_SI0.BDR_NUM[1].TBICR0.R /* Tx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB1ICR1 NETC__ENETC0_SI0.BDR_NUM[1].TBICR1.R /* Tx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB1MR NETC__ENETC0_SI0.BDR_NUM[1].RBMR.R /* Rx BDR 1 mode register */
#define NETC__ENETC0_SI0_RB1SR NETC__ENETC0_SI0.BDR_NUM[1].RBSR.R /* Rx BDR 1 status register */
#define NETC__ENETC0_SI0_RB1BSR NETC__ENETC0_SI0.BDR_NUM[1].RBBSR.R /* Rx BDR 1 buffer size register */
#define NETC__ENETC0_SI0_RB1CIR NETC__ENETC0_SI0.BDR_NUM[1].RBCIR.R /* Rx BDR 1 consumer index register */
#define NETC__ENETC0_SI0_RB1BAR0 NETC__ENETC0_SI0.BDR_NUM[1].RBBAR0.R /* Rx BDR 1 base address register 0 */
#define NETC__ENETC0_SI0_RB1BAR1 NETC__ENETC0_SI0.BDR_NUM[1].RBBAR1.R /* Rx BDR 1 base address register 1 */
#define NETC__ENETC0_SI0_RB1PIR NETC__ENETC0_SI0.BDR_NUM[1].RBPIR.R /* Rx BDR 1 producer index register */
#define NETC__ENETC0_SI0_RB1LENR NETC__ENETC0_SI0.BDR_NUM[1].RBLENR.R /* Rx BDR 1 length register */
#define NETC__ENETC0_SI0_RB1DCR NETC__ENETC0_SI0.BDR_NUM[1].RBDCR.R /* Rx BDR 1 drop count register */
#define NETC__ENETC0_SI0_RB1IER NETC__ENETC0_SI0.BDR_NUM[1].RBIER.R /* Rx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI0_RB1IDR NETC__ENETC0_SI0.BDR_NUM[1].RBIDR.R /* Rx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI0_RB1ICR0 NETC__ENETC0_SI0.BDR_NUM[1].RBICR0.R /* Rx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB1ICR1 NETC__ENETC0_SI0.BDR_NUM[1].RBICR1.R /* Rx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB2MR NETC__ENETC0_SI0.BDR_NUM[2].TBMR.R /* Tx BDR 2 mode register */
#define NETC__ENETC0_SI0_TB2SR NETC__ENETC0_SI0.BDR_NUM[2].TBSR.R /* Tx BDR 2 status register */
#define NETC__ENETC0_SI0_TB2BAR0 NETC__ENETC0_SI0.BDR_NUM[2].TBBAR0.R /* Tx BDR 2 base address register 0 */
#define NETC__ENETC0_SI0_TB2BAR1 NETC__ENETC0_SI0.BDR_NUM[2].TBBAR1.R /* Tx BDR 2 base address register 1 */
#define NETC__ENETC0_SI0_TB2PIR NETC__ENETC0_SI0.BDR_NUM[2].TBPIR.R /* Tx BDR 2 producer index register */
#define NETC__ENETC0_SI0_TB2CIR NETC__ENETC0_SI0.BDR_NUM[2].TBCIR.R /* Tx BDR 2 consumer index register */
#define NETC__ENETC0_SI0_TB2LENR NETC__ENETC0_SI0.BDR_NUM[2].TBLENR.R /* Tx BDR 2 length register */
#define NETC__ENETC0_SI0_TB2IER NETC__ENETC0_SI0.BDR_NUM[2].TBIER.R /* Tx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI0_TB2IDR NETC__ENETC0_SI0.BDR_NUM[2].TBIDR.R /* Tx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI0_TB2ICR0 NETC__ENETC0_SI0.BDR_NUM[2].TBICR0.R /* Tx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB2ICR1 NETC__ENETC0_SI0.BDR_NUM[2].TBICR1.R /* Tx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB2MR NETC__ENETC0_SI0.BDR_NUM[2].RBMR.R /* Rx BDR 2 mode register */
#define NETC__ENETC0_SI0_RB2SR NETC__ENETC0_SI0.BDR_NUM[2].RBSR.R /* Rx BDR 2 status register */
#define NETC__ENETC0_SI0_RB2BSR NETC__ENETC0_SI0.BDR_NUM[2].RBBSR.R /* Rx BDR 2 buffer size register */
#define NETC__ENETC0_SI0_RB2CIR NETC__ENETC0_SI0.BDR_NUM[2].RBCIR.R /* Rx BDR 2 consumer index register */
#define NETC__ENETC0_SI0_RB2BAR0 NETC__ENETC0_SI0.BDR_NUM[2].RBBAR0.R /* Rx BDR 2 base address register 0 */
#define NETC__ENETC0_SI0_RB2BAR1 NETC__ENETC0_SI0.BDR_NUM[2].RBBAR1.R /* Rx BDR 2 base address register 1 */
#define NETC__ENETC0_SI0_RB2PIR NETC__ENETC0_SI0.BDR_NUM[2].RBPIR.R /* Rx BDR 2 producer index register */
#define NETC__ENETC0_SI0_RB2LENR NETC__ENETC0_SI0.BDR_NUM[2].RBLENR.R /* Rx BDR 2 length register */
#define NETC__ENETC0_SI0_RB2DCR NETC__ENETC0_SI0.BDR_NUM[2].RBDCR.R /* Rx BDR 2 drop count register */
#define NETC__ENETC0_SI0_RB2IER NETC__ENETC0_SI0.BDR_NUM[2].RBIER.R /* Rx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI0_RB2IDR NETC__ENETC0_SI0.BDR_NUM[2].RBIDR.R /* Rx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI0_RB2ICR0 NETC__ENETC0_SI0.BDR_NUM[2].RBICR0.R /* Rx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB2ICR1 NETC__ENETC0_SI0.BDR_NUM[2].RBICR1.R /* Rx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB3MR NETC__ENETC0_SI0.BDR_NUM[3].TBMR.R /* Tx BDR 3 mode register */
#define NETC__ENETC0_SI0_TB3SR NETC__ENETC0_SI0.BDR_NUM[3].TBSR.R /* Tx BDR 3 status register */
#define NETC__ENETC0_SI0_TB3BAR0 NETC__ENETC0_SI0.BDR_NUM[3].TBBAR0.R /* Tx BDR 3 base address register 0 */
#define NETC__ENETC0_SI0_TB3BAR1 NETC__ENETC0_SI0.BDR_NUM[3].TBBAR1.R /* Tx BDR 3 base address register 1 */
#define NETC__ENETC0_SI0_TB3PIR NETC__ENETC0_SI0.BDR_NUM[3].TBPIR.R /* Tx BDR 3 producer index register */
#define NETC__ENETC0_SI0_TB3CIR NETC__ENETC0_SI0.BDR_NUM[3].TBCIR.R /* Tx BDR 3 consumer index register */
#define NETC__ENETC0_SI0_TB3LENR NETC__ENETC0_SI0.BDR_NUM[3].TBLENR.R /* Tx BDR 3 length register */
#define NETC__ENETC0_SI0_TB3IER NETC__ENETC0_SI0.BDR_NUM[3].TBIER.R /* Tx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI0_TB3IDR NETC__ENETC0_SI0.BDR_NUM[3].TBIDR.R /* Tx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI0_TB3ICR0 NETC__ENETC0_SI0.BDR_NUM[3].TBICR0.R /* Tx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB3ICR1 NETC__ENETC0_SI0.BDR_NUM[3].TBICR1.R /* Tx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB3MR NETC__ENETC0_SI0.BDR_NUM[3].RBMR.R /* Rx BDR 3 mode register */
#define NETC__ENETC0_SI0_RB3SR NETC__ENETC0_SI0.BDR_NUM[3].RBSR.R /* Rx BDR 3 status register */
#define NETC__ENETC0_SI0_RB3BSR NETC__ENETC0_SI0.BDR_NUM[3].RBBSR.R /* Rx BDR 3 buffer size register */
#define NETC__ENETC0_SI0_RB3CIR NETC__ENETC0_SI0.BDR_NUM[3].RBCIR.R /* Rx BDR 3 consumer index register */
#define NETC__ENETC0_SI0_RB3BAR0 NETC__ENETC0_SI0.BDR_NUM[3].RBBAR0.R /* Rx BDR 3 base address register 0 */
#define NETC__ENETC0_SI0_RB3BAR1 NETC__ENETC0_SI0.BDR_NUM[3].RBBAR1.R /* Rx BDR 3 base address register 1 */
#define NETC__ENETC0_SI0_RB3PIR NETC__ENETC0_SI0.BDR_NUM[3].RBPIR.R /* Rx BDR 3 producer index register */
#define NETC__ENETC0_SI0_RB3LENR NETC__ENETC0_SI0.BDR_NUM[3].RBLENR.R /* Rx BDR 3 length register */
#define NETC__ENETC0_SI0_RB3DCR NETC__ENETC0_SI0.BDR_NUM[3].RBDCR.R /* Rx BDR 3 drop count register */
#define NETC__ENETC0_SI0_RB3IER NETC__ENETC0_SI0.BDR_NUM[3].RBIER.R /* Rx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI0_RB3IDR NETC__ENETC0_SI0.BDR_NUM[3].RBIDR.R /* Rx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI0_RB3ICR0 NETC__ENETC0_SI0.BDR_NUM[3].RBICR0.R /* Rx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB3ICR1 NETC__ENETC0_SI0.BDR_NUM[3].RBICR1.R /* Rx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB4MR NETC__ENETC0_SI0.BDR_NUM[4].TBMR.R /* Tx BDR 4 mode register */
#define NETC__ENETC0_SI0_TB4SR NETC__ENETC0_SI0.BDR_NUM[4].TBSR.R /* Tx BDR 4 status register */
#define NETC__ENETC0_SI0_TB4BAR0 NETC__ENETC0_SI0.BDR_NUM[4].TBBAR0.R /* Tx BDR 4 base address register 0 */
#define NETC__ENETC0_SI0_TB4BAR1 NETC__ENETC0_SI0.BDR_NUM[4].TBBAR1.R /* Tx BDR 4 base address register 1 */
#define NETC__ENETC0_SI0_TB4PIR NETC__ENETC0_SI0.BDR_NUM[4].TBPIR.R /* Tx BDR 4 producer index register */
#define NETC__ENETC0_SI0_TB4CIR NETC__ENETC0_SI0.BDR_NUM[4].TBCIR.R /* Tx BDR 4 consumer index register */
#define NETC__ENETC0_SI0_TB4LENR NETC__ENETC0_SI0.BDR_NUM[4].TBLENR.R /* Tx BDR 4 length register */
#define NETC__ENETC0_SI0_TB4IER NETC__ENETC0_SI0.BDR_NUM[4].TBIER.R /* Tx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI0_TB4IDR NETC__ENETC0_SI0.BDR_NUM[4].TBIDR.R /* Tx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI0_TB4ICR0 NETC__ENETC0_SI0.BDR_NUM[4].TBICR0.R /* Tx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB4ICR1 NETC__ENETC0_SI0.BDR_NUM[4].TBICR1.R /* Tx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB4MR NETC__ENETC0_SI0.BDR_NUM[4].RBMR.R /* Rx BDR 4 mode register */
#define NETC__ENETC0_SI0_RB4SR NETC__ENETC0_SI0.BDR_NUM[4].RBSR.R /* Rx BDR 4 status register */
#define NETC__ENETC0_SI0_RB4BSR NETC__ENETC0_SI0.BDR_NUM[4].RBBSR.R /* Rx BDR 4 buffer size register */
#define NETC__ENETC0_SI0_RB4CIR NETC__ENETC0_SI0.BDR_NUM[4].RBCIR.R /* Rx BDR 4 consumer index register */
#define NETC__ENETC0_SI0_RB4BAR0 NETC__ENETC0_SI0.BDR_NUM[4].RBBAR0.R /* Rx BDR 4 base address register 0 */
#define NETC__ENETC0_SI0_RB4BAR1 NETC__ENETC0_SI0.BDR_NUM[4].RBBAR1.R /* Rx BDR 4 base address register 1 */
#define NETC__ENETC0_SI0_RB4PIR NETC__ENETC0_SI0.BDR_NUM[4].RBPIR.R /* Rx BDR 4 producer index register */
#define NETC__ENETC0_SI0_RB4LENR NETC__ENETC0_SI0.BDR_NUM[4].RBLENR.R /* Rx BDR 4 length register */
#define NETC__ENETC0_SI0_RB4DCR NETC__ENETC0_SI0.BDR_NUM[4].RBDCR.R /* Rx BDR 4 drop count register */
#define NETC__ENETC0_SI0_RB4IER NETC__ENETC0_SI0.BDR_NUM[4].RBIER.R /* Rx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI0_RB4IDR NETC__ENETC0_SI0.BDR_NUM[4].RBIDR.R /* Rx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI0_RB4ICR0 NETC__ENETC0_SI0.BDR_NUM[4].RBICR0.R /* Rx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB4ICR1 NETC__ENETC0_SI0.BDR_NUM[4].RBICR1.R /* Rx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB5MR NETC__ENETC0_SI0.BDR_NUM[5].TBMR.R /* Tx BDR 5 mode register */
#define NETC__ENETC0_SI0_TB5SR NETC__ENETC0_SI0.BDR_NUM[5].TBSR.R /* Tx BDR 5 status register */
#define NETC__ENETC0_SI0_TB5BAR0 NETC__ENETC0_SI0.BDR_NUM[5].TBBAR0.R /* Tx BDR 5 base address register 0 */
#define NETC__ENETC0_SI0_TB5BAR1 NETC__ENETC0_SI0.BDR_NUM[5].TBBAR1.R /* Tx BDR 5 base address register 1 */
#define NETC__ENETC0_SI0_TB5PIR NETC__ENETC0_SI0.BDR_NUM[5].TBPIR.R /* Tx BDR 5 producer index register */
#define NETC__ENETC0_SI0_TB5CIR NETC__ENETC0_SI0.BDR_NUM[5].TBCIR.R /* Tx BDR 5 consumer index register */
#define NETC__ENETC0_SI0_TB5LENR NETC__ENETC0_SI0.BDR_NUM[5].TBLENR.R /* Tx BDR 5 length register */
#define NETC__ENETC0_SI0_TB5IER NETC__ENETC0_SI0.BDR_NUM[5].TBIER.R /* Tx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI0_TB5IDR NETC__ENETC0_SI0.BDR_NUM[5].TBIDR.R /* Tx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI0_TB5ICR0 NETC__ENETC0_SI0.BDR_NUM[5].TBICR0.R /* Tx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB5ICR1 NETC__ENETC0_SI0.BDR_NUM[5].TBICR1.R /* Tx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB5MR NETC__ENETC0_SI0.BDR_NUM[5].RBMR.R /* Rx BDR 5 mode register */
#define NETC__ENETC0_SI0_RB5SR NETC__ENETC0_SI0.BDR_NUM[5].RBSR.R /* Rx BDR 5 status register */
#define NETC__ENETC0_SI0_RB5BSR NETC__ENETC0_SI0.BDR_NUM[5].RBBSR.R /* Rx BDR 5 buffer size register */
#define NETC__ENETC0_SI0_RB5CIR NETC__ENETC0_SI0.BDR_NUM[5].RBCIR.R /* Rx BDR 5 consumer index register */
#define NETC__ENETC0_SI0_RB5BAR0 NETC__ENETC0_SI0.BDR_NUM[5].RBBAR0.R /* Rx BDR 5 base address register 0 */
#define NETC__ENETC0_SI0_RB5BAR1 NETC__ENETC0_SI0.BDR_NUM[5].RBBAR1.R /* Rx BDR 5 base address register 1 */
#define NETC__ENETC0_SI0_RB5PIR NETC__ENETC0_SI0.BDR_NUM[5].RBPIR.R /* Rx BDR 5 producer index register */
#define NETC__ENETC0_SI0_RB5LENR NETC__ENETC0_SI0.BDR_NUM[5].RBLENR.R /* Rx BDR 5 length register */
#define NETC__ENETC0_SI0_RB5DCR NETC__ENETC0_SI0.BDR_NUM[5].RBDCR.R /* Rx BDR 5 drop count register */
#define NETC__ENETC0_SI0_RB5IER NETC__ENETC0_SI0.BDR_NUM[5].RBIER.R /* Rx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI0_RB5IDR NETC__ENETC0_SI0.BDR_NUM[5].RBIDR.R /* Rx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI0_RB5ICR0 NETC__ENETC0_SI0.BDR_NUM[5].RBICR0.R /* Rx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB5ICR1 NETC__ENETC0_SI0.BDR_NUM[5].RBICR1.R /* Rx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB6MR NETC__ENETC0_SI0.BDR_NUM[6].TBMR.R /* Tx BDR 6 mode register */
#define NETC__ENETC0_SI0_TB6SR NETC__ENETC0_SI0.BDR_NUM[6].TBSR.R /* Tx BDR 6 status register */
#define NETC__ENETC0_SI0_TB6BAR0 NETC__ENETC0_SI0.BDR_NUM[6].TBBAR0.R /* Tx BDR 6 base address register 0 */
#define NETC__ENETC0_SI0_TB6BAR1 NETC__ENETC0_SI0.BDR_NUM[6].TBBAR1.R /* Tx BDR 6 base address register 1 */
#define NETC__ENETC0_SI0_TB6PIR NETC__ENETC0_SI0.BDR_NUM[6].TBPIR.R /* Tx BDR 6 producer index register */
#define NETC__ENETC0_SI0_TB6CIR NETC__ENETC0_SI0.BDR_NUM[6].TBCIR.R /* Tx BDR 6 consumer index register */
#define NETC__ENETC0_SI0_TB6LENR NETC__ENETC0_SI0.BDR_NUM[6].TBLENR.R /* Tx BDR 6 length register */
#define NETC__ENETC0_SI0_TB6IER NETC__ENETC0_SI0.BDR_NUM[6].TBIER.R /* Tx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI0_TB6IDR NETC__ENETC0_SI0.BDR_NUM[6].TBIDR.R /* Tx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI0_TB6ICR0 NETC__ENETC0_SI0.BDR_NUM[6].TBICR0.R /* Tx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB6ICR1 NETC__ENETC0_SI0.BDR_NUM[6].TBICR1.R /* Tx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB6MR NETC__ENETC0_SI0.BDR_NUM[6].RBMR.R /* Rx BDR 6 mode register */
#define NETC__ENETC0_SI0_RB6SR NETC__ENETC0_SI0.BDR_NUM[6].RBSR.R /* Rx BDR 6 status register */
#define NETC__ENETC0_SI0_RB6BSR NETC__ENETC0_SI0.BDR_NUM[6].RBBSR.R /* Rx BDR 6 buffer size register */
#define NETC__ENETC0_SI0_RB6CIR NETC__ENETC0_SI0.BDR_NUM[6].RBCIR.R /* Rx BDR 6 consumer index register */
#define NETC__ENETC0_SI0_RB6BAR0 NETC__ENETC0_SI0.BDR_NUM[6].RBBAR0.R /* Rx BDR 6 base address register 0 */
#define NETC__ENETC0_SI0_RB6BAR1 NETC__ENETC0_SI0.BDR_NUM[6].RBBAR1.R /* Rx BDR 6 base address register 1 */
#define NETC__ENETC0_SI0_RB6PIR NETC__ENETC0_SI0.BDR_NUM[6].RBPIR.R /* Rx BDR 6 producer index register */
#define NETC__ENETC0_SI0_RB6LENR NETC__ENETC0_SI0.BDR_NUM[6].RBLENR.R /* Rx BDR 6 length register */
#define NETC__ENETC0_SI0_RB6DCR NETC__ENETC0_SI0.BDR_NUM[6].RBDCR.R /* Rx BDR 6 drop count register */
#define NETC__ENETC0_SI0_RB6IER NETC__ENETC0_SI0.BDR_NUM[6].RBIER.R /* Rx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI0_RB6IDR NETC__ENETC0_SI0.BDR_NUM[6].RBIDR.R /* Rx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI0_RB6ICR0 NETC__ENETC0_SI0.BDR_NUM[6].RBICR0.R /* Rx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB6ICR1 NETC__ENETC0_SI0.BDR_NUM[6].RBICR1.R /* Rx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB7MR NETC__ENETC0_SI0.BDR_NUM[7].TBMR.R /* Tx BDR 7 mode register */
#define NETC__ENETC0_SI0_TB7SR NETC__ENETC0_SI0.BDR_NUM[7].TBSR.R /* Tx BDR 7 status register */
#define NETC__ENETC0_SI0_TB7BAR0 NETC__ENETC0_SI0.BDR_NUM[7].TBBAR0.R /* Tx BDR 7 base address register 0 */
#define NETC__ENETC0_SI0_TB7BAR1 NETC__ENETC0_SI0.BDR_NUM[7].TBBAR1.R /* Tx BDR 7 base address register 1 */
#define NETC__ENETC0_SI0_TB7PIR NETC__ENETC0_SI0.BDR_NUM[7].TBPIR.R /* Tx BDR 7 producer index register */
#define NETC__ENETC0_SI0_TB7CIR NETC__ENETC0_SI0.BDR_NUM[7].TBCIR.R /* Tx BDR 7 consumer index register */
#define NETC__ENETC0_SI0_TB7LENR NETC__ENETC0_SI0.BDR_NUM[7].TBLENR.R /* Tx BDR 7 length register */
#define NETC__ENETC0_SI0_TB7IER NETC__ENETC0_SI0.BDR_NUM[7].TBIER.R /* Tx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI0_TB7IDR NETC__ENETC0_SI0.BDR_NUM[7].TBIDR.R /* Tx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI0_TB7ICR0 NETC__ENETC0_SI0.BDR_NUM[7].TBICR0.R /* Tx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB7ICR1 NETC__ENETC0_SI0.BDR_NUM[7].TBICR1.R /* Tx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB7MR NETC__ENETC0_SI0.BDR_NUM[7].RBMR.R /* Rx BDR 7 mode register */
#define NETC__ENETC0_SI0_RB7SR NETC__ENETC0_SI0.BDR_NUM[7].RBSR.R /* Rx BDR 7 status register */
#define NETC__ENETC0_SI0_RB7BSR NETC__ENETC0_SI0.BDR_NUM[7].RBBSR.R /* Rx BDR 7 buffer size register */
#define NETC__ENETC0_SI0_RB7CIR NETC__ENETC0_SI0.BDR_NUM[7].RBCIR.R /* Rx BDR 7 consumer index register */
#define NETC__ENETC0_SI0_RB7BAR0 NETC__ENETC0_SI0.BDR_NUM[7].RBBAR0.R /* Rx BDR 7 base address register 0 */
#define NETC__ENETC0_SI0_RB7BAR1 NETC__ENETC0_SI0.BDR_NUM[7].RBBAR1.R /* Rx BDR 7 base address register 1 */
#define NETC__ENETC0_SI0_RB7PIR NETC__ENETC0_SI0.BDR_NUM[7].RBPIR.R /* Rx BDR 7 producer index register */
#define NETC__ENETC0_SI0_RB7LENR NETC__ENETC0_SI0.BDR_NUM[7].RBLENR.R /* Rx BDR 7 length register */
#define NETC__ENETC0_SI0_RB7DCR NETC__ENETC0_SI0.BDR_NUM[7].RBDCR.R /* Rx BDR 7 drop count register */
#define NETC__ENETC0_SI0_RB7IER NETC__ENETC0_SI0.BDR_NUM[7].RBIER.R /* Rx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI0_RB7IDR NETC__ENETC0_SI0.BDR_NUM[7].RBIDR.R /* Rx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI0_RB7ICR0 NETC__ENETC0_SI0.BDR_NUM[7].RBICR0.R /* Rx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB7ICR1 NETC__ENETC0_SI0.BDR_NUM[7].RBICR1.R /* Rx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB8MR NETC__ENETC0_SI0.BDR_NUM[8].TBMR.R /* Tx BDR 8 mode register */
#define NETC__ENETC0_SI0_TB8SR NETC__ENETC0_SI0.BDR_NUM[8].TBSR.R /* Tx BDR 8 status register */
#define NETC__ENETC0_SI0_TB8BAR0 NETC__ENETC0_SI0.BDR_NUM[8].TBBAR0.R /* Tx BDR 8 base address register 0 */
#define NETC__ENETC0_SI0_TB8BAR1 NETC__ENETC0_SI0.BDR_NUM[8].TBBAR1.R /* Tx BDR 8 base address register 1 */
#define NETC__ENETC0_SI0_TB8PIR NETC__ENETC0_SI0.BDR_NUM[8].TBPIR.R /* Tx BDR 8 producer index register */
#define NETC__ENETC0_SI0_TB8CIR NETC__ENETC0_SI0.BDR_NUM[8].TBCIR.R /* Tx BDR 8 consumer index register */
#define NETC__ENETC0_SI0_TB8LENR NETC__ENETC0_SI0.BDR_NUM[8].TBLENR.R /* Tx BDR 8 length register */
#define NETC__ENETC0_SI0_TB8IER NETC__ENETC0_SI0.BDR_NUM[8].TBIER.R /* Tx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI0_TB8IDR NETC__ENETC0_SI0.BDR_NUM[8].TBIDR.R /* Tx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI0_TB8ICR0 NETC__ENETC0_SI0.BDR_NUM[8].TBICR0.R /* Tx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB8ICR1 NETC__ENETC0_SI0.BDR_NUM[8].TBICR1.R /* Tx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB8MR NETC__ENETC0_SI0.BDR_NUM[8].RBMR.R /* Rx BDR 8 mode register */
#define NETC__ENETC0_SI0_RB8SR NETC__ENETC0_SI0.BDR_NUM[8].RBSR.R /* Rx BDR 8 status register */
#define NETC__ENETC0_SI0_RB8BSR NETC__ENETC0_SI0.BDR_NUM[8].RBBSR.R /* Rx BDR 8 buffer size register */
#define NETC__ENETC0_SI0_RB8CIR NETC__ENETC0_SI0.BDR_NUM[8].RBCIR.R /* Rx BDR 8 consumer index register */
#define NETC__ENETC0_SI0_RB8BAR0 NETC__ENETC0_SI0.BDR_NUM[8].RBBAR0.R /* Rx BDR 8 base address register 0 */
#define NETC__ENETC0_SI0_RB8BAR1 NETC__ENETC0_SI0.BDR_NUM[8].RBBAR1.R /* Rx BDR 8 base address register 1 */
#define NETC__ENETC0_SI0_RB8PIR NETC__ENETC0_SI0.BDR_NUM[8].RBPIR.R /* Rx BDR 8 producer index register */
#define NETC__ENETC0_SI0_RB8LENR NETC__ENETC0_SI0.BDR_NUM[8].RBLENR.R /* Rx BDR 8 length register */
#define NETC__ENETC0_SI0_RB8DCR NETC__ENETC0_SI0.BDR_NUM[8].RBDCR.R /* Rx BDR 8 drop count register */
#define NETC__ENETC0_SI0_RB8IER NETC__ENETC0_SI0.BDR_NUM[8].RBIER.R /* Rx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI0_RB8IDR NETC__ENETC0_SI0.BDR_NUM[8].RBIDR.R /* Rx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI0_RB8ICR0 NETC__ENETC0_SI0.BDR_NUM[8].RBICR0.R /* Rx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB8ICR1 NETC__ENETC0_SI0.BDR_NUM[8].RBICR1.R /* Rx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB9MR NETC__ENETC0_SI0.BDR_NUM[9].TBMR.R /* Tx BDR 9 mode register */
#define NETC__ENETC0_SI0_TB9SR NETC__ENETC0_SI0.BDR_NUM[9].TBSR.R /* Tx BDR 9 status register */
#define NETC__ENETC0_SI0_TB9BAR0 NETC__ENETC0_SI0.BDR_NUM[9].TBBAR0.R /* Tx BDR 9 base address register 0 */
#define NETC__ENETC0_SI0_TB9BAR1 NETC__ENETC0_SI0.BDR_NUM[9].TBBAR1.R /* Tx BDR 9 base address register 1 */
#define NETC__ENETC0_SI0_TB9PIR NETC__ENETC0_SI0.BDR_NUM[9].TBPIR.R /* Tx BDR 9 producer index register */
#define NETC__ENETC0_SI0_TB9CIR NETC__ENETC0_SI0.BDR_NUM[9].TBCIR.R /* Tx BDR 9 consumer index register */
#define NETC__ENETC0_SI0_TB9LENR NETC__ENETC0_SI0.BDR_NUM[9].TBLENR.R /* Tx BDR 9 length register */
#define NETC__ENETC0_SI0_TB9IER NETC__ENETC0_SI0.BDR_NUM[9].TBIER.R /* Tx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI0_TB9IDR NETC__ENETC0_SI0.BDR_NUM[9].TBIDR.R /* Tx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI0_TB9ICR0 NETC__ENETC0_SI0.BDR_NUM[9].TBICR0.R /* Tx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB9ICR1 NETC__ENETC0_SI0.BDR_NUM[9].TBICR1.R /* Tx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB9MR NETC__ENETC0_SI0.BDR_NUM[9].RBMR.R /* Rx BDR 9 mode register */
#define NETC__ENETC0_SI0_RB9SR NETC__ENETC0_SI0.BDR_NUM[9].RBSR.R /* Rx BDR 9 status register */
#define NETC__ENETC0_SI0_RB9BSR NETC__ENETC0_SI0.BDR_NUM[9].RBBSR.R /* Rx BDR 9 buffer size register */
#define NETC__ENETC0_SI0_RB9CIR NETC__ENETC0_SI0.BDR_NUM[9].RBCIR.R /* Rx BDR 9 consumer index register */
#define NETC__ENETC0_SI0_RB9BAR0 NETC__ENETC0_SI0.BDR_NUM[9].RBBAR0.R /* Rx BDR 9 base address register 0 */
#define NETC__ENETC0_SI0_RB9BAR1 NETC__ENETC0_SI0.BDR_NUM[9].RBBAR1.R /* Rx BDR 9 base address register 1 */
#define NETC__ENETC0_SI0_RB9PIR NETC__ENETC0_SI0.BDR_NUM[9].RBPIR.R /* Rx BDR 9 producer index register */
#define NETC__ENETC0_SI0_RB9LENR NETC__ENETC0_SI0.BDR_NUM[9].RBLENR.R /* Rx BDR 9 length register */
#define NETC__ENETC0_SI0_RB9DCR NETC__ENETC0_SI0.BDR_NUM[9].RBDCR.R /* Rx BDR 9 drop count register */
#define NETC__ENETC0_SI0_RB9IER NETC__ENETC0_SI0.BDR_NUM[9].RBIER.R /* Rx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI0_RB9IDR NETC__ENETC0_SI0.BDR_NUM[9].RBIDR.R /* Rx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI0_RB9ICR0 NETC__ENETC0_SI0.BDR_NUM[9].RBICR0.R /* Rx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB9ICR1 NETC__ENETC0_SI0.BDR_NUM[9].RBICR1.R /* Rx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB10MR NETC__ENETC0_SI0.BDR_NUM[10].TBMR.R /* Tx BDR 10 mode register */
#define NETC__ENETC0_SI0_TB10SR NETC__ENETC0_SI0.BDR_NUM[10].TBSR.R /* Tx BDR 10 status register */
#define NETC__ENETC0_SI0_TB10BAR0 NETC__ENETC0_SI0.BDR_NUM[10].TBBAR0.R /* Tx BDR 10 base address register 0 */
#define NETC__ENETC0_SI0_TB10BAR1 NETC__ENETC0_SI0.BDR_NUM[10].TBBAR1.R /* Tx BDR 10 base address register 1 */
#define NETC__ENETC0_SI0_TB10PIR NETC__ENETC0_SI0.BDR_NUM[10].TBPIR.R /* Tx BDR 10 producer index register */
#define NETC__ENETC0_SI0_TB10CIR NETC__ENETC0_SI0.BDR_NUM[10].TBCIR.R /* Tx BDR 10 consumer index register */
#define NETC__ENETC0_SI0_TB10LENR NETC__ENETC0_SI0.BDR_NUM[10].TBLENR.R /* Tx BDR 10 length register */
#define NETC__ENETC0_SI0_TB10IER NETC__ENETC0_SI0.BDR_NUM[10].TBIER.R /* Tx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI0_TB10IDR NETC__ENETC0_SI0.BDR_NUM[10].TBIDR.R /* Tx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI0_TB10ICR0 NETC__ENETC0_SI0.BDR_NUM[10].TBICR0.R /* Tx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB10ICR1 NETC__ENETC0_SI0.BDR_NUM[10].TBICR1.R /* Tx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB10MR NETC__ENETC0_SI0.BDR_NUM[10].RBMR.R /* Rx BDR 10 mode register */
#define NETC__ENETC0_SI0_RB10SR NETC__ENETC0_SI0.BDR_NUM[10].RBSR.R /* Rx BDR 10 status register */
#define NETC__ENETC0_SI0_RB10BSR NETC__ENETC0_SI0.BDR_NUM[10].RBBSR.R /* Rx BDR 10 buffer size register */
#define NETC__ENETC0_SI0_RB10CIR NETC__ENETC0_SI0.BDR_NUM[10].RBCIR.R /* Rx BDR 10 consumer index register */
#define NETC__ENETC0_SI0_RB10BAR0 NETC__ENETC0_SI0.BDR_NUM[10].RBBAR0.R /* Rx BDR 10 base address register 0 */
#define NETC__ENETC0_SI0_RB10BAR1 NETC__ENETC0_SI0.BDR_NUM[10].RBBAR1.R /* Rx BDR 10 base address register 1 */
#define NETC__ENETC0_SI0_RB10PIR NETC__ENETC0_SI0.BDR_NUM[10].RBPIR.R /* Rx BDR 10 producer index register */
#define NETC__ENETC0_SI0_RB10LENR NETC__ENETC0_SI0.BDR_NUM[10].RBLENR.R /* Rx BDR 10 length register */
#define NETC__ENETC0_SI0_RB10DCR NETC__ENETC0_SI0.BDR_NUM[10].RBDCR.R /* Rx BDR 10 drop count register */
#define NETC__ENETC0_SI0_RB10IER NETC__ENETC0_SI0.BDR_NUM[10].RBIER.R /* Rx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI0_RB10IDR NETC__ENETC0_SI0.BDR_NUM[10].RBIDR.R /* Rx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI0_RB10ICR0 NETC__ENETC0_SI0.BDR_NUM[10].RBICR0.R /* Rx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB10ICR1 NETC__ENETC0_SI0.BDR_NUM[10].RBICR1.R /* Rx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB11MR NETC__ENETC0_SI0.BDR_NUM[11].TBMR.R /* Tx BDR 11 mode register */
#define NETC__ENETC0_SI0_TB11SR NETC__ENETC0_SI0.BDR_NUM[11].TBSR.R /* Tx BDR 11 status register */
#define NETC__ENETC0_SI0_TB11BAR0 NETC__ENETC0_SI0.BDR_NUM[11].TBBAR0.R /* Tx BDR 11 base address register 0 */
#define NETC__ENETC0_SI0_TB11BAR1 NETC__ENETC0_SI0.BDR_NUM[11].TBBAR1.R /* Tx BDR 11 base address register 1 */
#define NETC__ENETC0_SI0_TB11PIR NETC__ENETC0_SI0.BDR_NUM[11].TBPIR.R /* Tx BDR 11 producer index register */
#define NETC__ENETC0_SI0_TB11CIR NETC__ENETC0_SI0.BDR_NUM[11].TBCIR.R /* Tx BDR 11 consumer index register */
#define NETC__ENETC0_SI0_TB11LENR NETC__ENETC0_SI0.BDR_NUM[11].TBLENR.R /* Tx BDR 11 length register */
#define NETC__ENETC0_SI0_TB11IER NETC__ENETC0_SI0.BDR_NUM[11].TBIER.R /* Tx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI0_TB11IDR NETC__ENETC0_SI0.BDR_NUM[11].TBIDR.R /* Tx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI0_TB11ICR0 NETC__ENETC0_SI0.BDR_NUM[11].TBICR0.R /* Tx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB11ICR1 NETC__ENETC0_SI0.BDR_NUM[11].TBICR1.R /* Tx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB11MR NETC__ENETC0_SI0.BDR_NUM[11].RBMR.R /* Rx BDR 11 mode register */
#define NETC__ENETC0_SI0_RB11SR NETC__ENETC0_SI0.BDR_NUM[11].RBSR.R /* Rx BDR 11 status register */
#define NETC__ENETC0_SI0_RB11BSR NETC__ENETC0_SI0.BDR_NUM[11].RBBSR.R /* Rx BDR 11 buffer size register */
#define NETC__ENETC0_SI0_RB11CIR NETC__ENETC0_SI0.BDR_NUM[11].RBCIR.R /* Rx BDR 11 consumer index register */
#define NETC__ENETC0_SI0_RB11BAR0 NETC__ENETC0_SI0.BDR_NUM[11].RBBAR0.R /* Rx BDR 11 base address register 0 */
#define NETC__ENETC0_SI0_RB11BAR1 NETC__ENETC0_SI0.BDR_NUM[11].RBBAR1.R /* Rx BDR 11 base address register 1 */
#define NETC__ENETC0_SI0_RB11PIR NETC__ENETC0_SI0.BDR_NUM[11].RBPIR.R /* Rx BDR 11 producer index register */
#define NETC__ENETC0_SI0_RB11LENR NETC__ENETC0_SI0.BDR_NUM[11].RBLENR.R /* Rx BDR 11 length register */
#define NETC__ENETC0_SI0_RB11DCR NETC__ENETC0_SI0.BDR_NUM[11].RBDCR.R /* Rx BDR 11 drop count register */
#define NETC__ENETC0_SI0_RB11IER NETC__ENETC0_SI0.BDR_NUM[11].RBIER.R /* Rx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI0_RB11IDR NETC__ENETC0_SI0.BDR_NUM[11].RBIDR.R /* Rx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI0_RB11ICR0 NETC__ENETC0_SI0.BDR_NUM[11].RBICR0.R /* Rx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB11ICR1 NETC__ENETC0_SI0.BDR_NUM[11].RBICR1.R /* Rx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB12MR NETC__ENETC0_SI0.BDR_NUM[12].TBMR.R /* Tx BDR 12 mode register */
#define NETC__ENETC0_SI0_TB12SR NETC__ENETC0_SI0.BDR_NUM[12].TBSR.R /* Tx BDR 12 status register */
#define NETC__ENETC0_SI0_TB12BAR0 NETC__ENETC0_SI0.BDR_NUM[12].TBBAR0.R /* Tx BDR 12 base address register 0 */
#define NETC__ENETC0_SI0_TB12BAR1 NETC__ENETC0_SI0.BDR_NUM[12].TBBAR1.R /* Tx BDR 12 base address register 1 */
#define NETC__ENETC0_SI0_TB12PIR NETC__ENETC0_SI0.BDR_NUM[12].TBPIR.R /* Tx BDR 12 producer index register */
#define NETC__ENETC0_SI0_TB12CIR NETC__ENETC0_SI0.BDR_NUM[12].TBCIR.R /* Tx BDR 12 consumer index register */
#define NETC__ENETC0_SI0_TB12LENR NETC__ENETC0_SI0.BDR_NUM[12].TBLENR.R /* Tx BDR 12 length register */
#define NETC__ENETC0_SI0_TB12IER NETC__ENETC0_SI0.BDR_NUM[12].TBIER.R /* Tx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI0_TB12IDR NETC__ENETC0_SI0.BDR_NUM[12].TBIDR.R /* Tx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI0_TB12ICR0 NETC__ENETC0_SI0.BDR_NUM[12].TBICR0.R /* Tx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB12ICR1 NETC__ENETC0_SI0.BDR_NUM[12].TBICR1.R /* Tx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB12MR NETC__ENETC0_SI0.BDR_NUM[12].RBMR.R /* Rx BDR 12 mode register */
#define NETC__ENETC0_SI0_RB12SR NETC__ENETC0_SI0.BDR_NUM[12].RBSR.R /* Rx BDR 12 status register */
#define NETC__ENETC0_SI0_RB12BSR NETC__ENETC0_SI0.BDR_NUM[12].RBBSR.R /* Rx BDR 12 buffer size register */
#define NETC__ENETC0_SI0_RB12CIR NETC__ENETC0_SI0.BDR_NUM[12].RBCIR.R /* Rx BDR 12 consumer index register */
#define NETC__ENETC0_SI0_RB12BAR0 NETC__ENETC0_SI0.BDR_NUM[12].RBBAR0.R /* Rx BDR 12 base address register 0 */
#define NETC__ENETC0_SI0_RB12BAR1 NETC__ENETC0_SI0.BDR_NUM[12].RBBAR1.R /* Rx BDR 12 base address register 1 */
#define NETC__ENETC0_SI0_RB12PIR NETC__ENETC0_SI0.BDR_NUM[12].RBPIR.R /* Rx BDR 12 producer index register */
#define NETC__ENETC0_SI0_RB12LENR NETC__ENETC0_SI0.BDR_NUM[12].RBLENR.R /* Rx BDR 12 length register */
#define NETC__ENETC0_SI0_RB12DCR NETC__ENETC0_SI0.BDR_NUM[12].RBDCR.R /* Rx BDR 12 drop count register */
#define NETC__ENETC0_SI0_RB12IER NETC__ENETC0_SI0.BDR_NUM[12].RBIER.R /* Rx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI0_RB12IDR NETC__ENETC0_SI0.BDR_NUM[12].RBIDR.R /* Rx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI0_RB12ICR0 NETC__ENETC0_SI0.BDR_NUM[12].RBICR0.R /* Rx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB12ICR1 NETC__ENETC0_SI0.BDR_NUM[12].RBICR1.R /* Rx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB13MR NETC__ENETC0_SI0.BDR_NUM[13].TBMR.R /* Tx BDR 13 mode register */
#define NETC__ENETC0_SI0_TB13SR NETC__ENETC0_SI0.BDR_NUM[13].TBSR.R /* Tx BDR 13 status register */
#define NETC__ENETC0_SI0_TB13BAR0 NETC__ENETC0_SI0.BDR_NUM[13].TBBAR0.R /* Tx BDR 13 base address register 0 */
#define NETC__ENETC0_SI0_TB13BAR1 NETC__ENETC0_SI0.BDR_NUM[13].TBBAR1.R /* Tx BDR 13 base address register 1 */
#define NETC__ENETC0_SI0_TB13PIR NETC__ENETC0_SI0.BDR_NUM[13].TBPIR.R /* Tx BDR 13 producer index register */
#define NETC__ENETC0_SI0_TB13CIR NETC__ENETC0_SI0.BDR_NUM[13].TBCIR.R /* Tx BDR 13 consumer index register */
#define NETC__ENETC0_SI0_TB13LENR NETC__ENETC0_SI0.BDR_NUM[13].TBLENR.R /* Tx BDR 13 length register */
#define NETC__ENETC0_SI0_TB13IER NETC__ENETC0_SI0.BDR_NUM[13].TBIER.R /* Tx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI0_TB13IDR NETC__ENETC0_SI0.BDR_NUM[13].TBIDR.R /* Tx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI0_TB13ICR0 NETC__ENETC0_SI0.BDR_NUM[13].TBICR0.R /* Tx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB13ICR1 NETC__ENETC0_SI0.BDR_NUM[13].TBICR1.R /* Tx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB13MR NETC__ENETC0_SI0.BDR_NUM[13].RBMR.R /* Rx BDR 13 mode register */
#define NETC__ENETC0_SI0_RB13SR NETC__ENETC0_SI0.BDR_NUM[13].RBSR.R /* Rx BDR 13 status register */
#define NETC__ENETC0_SI0_RB13BSR NETC__ENETC0_SI0.BDR_NUM[13].RBBSR.R /* Rx BDR 13 buffer size register */
#define NETC__ENETC0_SI0_RB13CIR NETC__ENETC0_SI0.BDR_NUM[13].RBCIR.R /* Rx BDR 13 consumer index register */
#define NETC__ENETC0_SI0_RB13BAR0 NETC__ENETC0_SI0.BDR_NUM[13].RBBAR0.R /* Rx BDR 13 base address register 0 */
#define NETC__ENETC0_SI0_RB13BAR1 NETC__ENETC0_SI0.BDR_NUM[13].RBBAR1.R /* Rx BDR 13 base address register 1 */
#define NETC__ENETC0_SI0_RB13PIR NETC__ENETC0_SI0.BDR_NUM[13].RBPIR.R /* Rx BDR 13 producer index register */
#define NETC__ENETC0_SI0_RB13LENR NETC__ENETC0_SI0.BDR_NUM[13].RBLENR.R /* Rx BDR 13 length register */
#define NETC__ENETC0_SI0_RB13DCR NETC__ENETC0_SI0.BDR_NUM[13].RBDCR.R /* Rx BDR 13 drop count register */
#define NETC__ENETC0_SI0_RB13IER NETC__ENETC0_SI0.BDR_NUM[13].RBIER.R /* Rx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI0_RB13IDR NETC__ENETC0_SI0.BDR_NUM[13].RBIDR.R /* Rx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI0_RB13ICR0 NETC__ENETC0_SI0.BDR_NUM[13].RBICR0.R /* Rx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB13ICR1 NETC__ENETC0_SI0.BDR_NUM[13].RBICR1.R /* Rx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB14MR NETC__ENETC0_SI0.BDR_NUM[14].TBMR.R /* Tx BDR 14 mode register */
#define NETC__ENETC0_SI0_TB14SR NETC__ENETC0_SI0.BDR_NUM[14].TBSR.R /* Tx BDR 14 status register */
#define NETC__ENETC0_SI0_TB14BAR0 NETC__ENETC0_SI0.BDR_NUM[14].TBBAR0.R /* Tx BDR 14 base address register 0 */
#define NETC__ENETC0_SI0_TB14BAR1 NETC__ENETC0_SI0.BDR_NUM[14].TBBAR1.R /* Tx BDR 14 base address register 1 */
#define NETC__ENETC0_SI0_TB14PIR NETC__ENETC0_SI0.BDR_NUM[14].TBPIR.R /* Tx BDR 14 producer index register */
#define NETC__ENETC0_SI0_TB14CIR NETC__ENETC0_SI0.BDR_NUM[14].TBCIR.R /* Tx BDR 14 consumer index register */
#define NETC__ENETC0_SI0_TB14LENR NETC__ENETC0_SI0.BDR_NUM[14].TBLENR.R /* Tx BDR 14 length register */
#define NETC__ENETC0_SI0_TB14IER NETC__ENETC0_SI0.BDR_NUM[14].TBIER.R /* Tx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI0_TB14IDR NETC__ENETC0_SI0.BDR_NUM[14].TBIDR.R /* Tx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI0_TB14ICR0 NETC__ENETC0_SI0.BDR_NUM[14].TBICR0.R /* Tx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB14ICR1 NETC__ENETC0_SI0.BDR_NUM[14].TBICR1.R /* Tx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB14MR NETC__ENETC0_SI0.BDR_NUM[14].RBMR.R /* Rx BDR 14 mode register */
#define NETC__ENETC0_SI0_RB14SR NETC__ENETC0_SI0.BDR_NUM[14].RBSR.R /* Rx BDR 14 status register */
#define NETC__ENETC0_SI0_RB14BSR NETC__ENETC0_SI0.BDR_NUM[14].RBBSR.R /* Rx BDR 14 buffer size register */
#define NETC__ENETC0_SI0_RB14CIR NETC__ENETC0_SI0.BDR_NUM[14].RBCIR.R /* Rx BDR 14 consumer index register */
#define NETC__ENETC0_SI0_RB14BAR0 NETC__ENETC0_SI0.BDR_NUM[14].RBBAR0.R /* Rx BDR 14 base address register 0 */
#define NETC__ENETC0_SI0_RB14BAR1 NETC__ENETC0_SI0.BDR_NUM[14].RBBAR1.R /* Rx BDR 14 base address register 1 */
#define NETC__ENETC0_SI0_RB14PIR NETC__ENETC0_SI0.BDR_NUM[14].RBPIR.R /* Rx BDR 14 producer index register */
#define NETC__ENETC0_SI0_RB14LENR NETC__ENETC0_SI0.BDR_NUM[14].RBLENR.R /* Rx BDR 14 length register */
#define NETC__ENETC0_SI0_RB14DCR NETC__ENETC0_SI0.BDR_NUM[14].RBDCR.R /* Rx BDR 14 drop count register */
#define NETC__ENETC0_SI0_RB14IER NETC__ENETC0_SI0.BDR_NUM[14].RBIER.R /* Rx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI0_RB14IDR NETC__ENETC0_SI0.BDR_NUM[14].RBIDR.R /* Rx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI0_RB14ICR0 NETC__ENETC0_SI0.BDR_NUM[14].RBICR0.R /* Rx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB14ICR1 NETC__ENETC0_SI0.BDR_NUM[14].RBICR1.R /* Rx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB15MR NETC__ENETC0_SI0.BDR_NUM[15].TBMR.R /* Tx BDR 15 mode register */
#define NETC__ENETC0_SI0_TB15SR NETC__ENETC0_SI0.BDR_NUM[15].TBSR.R /* Tx BDR 15 status register */
#define NETC__ENETC0_SI0_TB15BAR0 NETC__ENETC0_SI0.BDR_NUM[15].TBBAR0.R /* Tx BDR 15 base address register 0 */
#define NETC__ENETC0_SI0_TB15BAR1 NETC__ENETC0_SI0.BDR_NUM[15].TBBAR1.R /* Tx BDR 15 base address register 1 */
#define NETC__ENETC0_SI0_TB15PIR NETC__ENETC0_SI0.BDR_NUM[15].TBPIR.R /* Tx BDR 15 producer index register */
#define NETC__ENETC0_SI0_TB15CIR NETC__ENETC0_SI0.BDR_NUM[15].TBCIR.R /* Tx BDR 15 consumer index register */
#define NETC__ENETC0_SI0_TB15LENR NETC__ENETC0_SI0.BDR_NUM[15].TBLENR.R /* Tx BDR 15 length register */
#define NETC__ENETC0_SI0_TB15IER NETC__ENETC0_SI0.BDR_NUM[15].TBIER.R /* Tx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI0_TB15IDR NETC__ENETC0_SI0.BDR_NUM[15].TBIDR.R /* Tx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI0_TB15ICR0 NETC__ENETC0_SI0.BDR_NUM[15].TBICR0.R /* Tx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB15ICR1 NETC__ENETC0_SI0.BDR_NUM[15].TBICR1.R /* Tx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB15MR NETC__ENETC0_SI0.BDR_NUM[15].RBMR.R /* Rx BDR 15 mode register */
#define NETC__ENETC0_SI0_RB15SR NETC__ENETC0_SI0.BDR_NUM[15].RBSR.R /* Rx BDR 15 status register */
#define NETC__ENETC0_SI0_RB15BSR NETC__ENETC0_SI0.BDR_NUM[15].RBBSR.R /* Rx BDR 15 buffer size register */
#define NETC__ENETC0_SI0_RB15CIR NETC__ENETC0_SI0.BDR_NUM[15].RBCIR.R /* Rx BDR 15 consumer index register */
#define NETC__ENETC0_SI0_RB15BAR0 NETC__ENETC0_SI0.BDR_NUM[15].RBBAR0.R /* Rx BDR 15 base address register 0 */
#define NETC__ENETC0_SI0_RB15BAR1 NETC__ENETC0_SI0.BDR_NUM[15].RBBAR1.R /* Rx BDR 15 base address register 1 */
#define NETC__ENETC0_SI0_RB15PIR NETC__ENETC0_SI0.BDR_NUM[15].RBPIR.R /* Rx BDR 15 producer index register */
#define NETC__ENETC0_SI0_RB15LENR NETC__ENETC0_SI0.BDR_NUM[15].RBLENR.R /* Rx BDR 15 length register */
#define NETC__ENETC0_SI0_RB15DCR NETC__ENETC0_SI0.BDR_NUM[15].RBDCR.R /* Rx BDR 15 drop count register */
#define NETC__ENETC0_SI0_RB15IER NETC__ENETC0_SI0.BDR_NUM[15].RBIER.R /* Rx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI0_RB15IDR NETC__ENETC0_SI0.BDR_NUM[15].RBIDR.R /* Rx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI0_RB15ICR0 NETC__ENETC0_SI0.BDR_NUM[15].RBICR0.R /* Rx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB15ICR1 NETC__ENETC0_SI0.BDR_NUM[15].RBICR1.R /* Rx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB16MR NETC__ENETC0_SI0.BDR_NUM[16].TBMR.R /* Tx BDR 16 mode register */
#define NETC__ENETC0_SI0_TB16SR NETC__ENETC0_SI0.BDR_NUM[16].TBSR.R /* Tx BDR 16 status register */
#define NETC__ENETC0_SI0_TB16BAR0 NETC__ENETC0_SI0.BDR_NUM[16].TBBAR0.R /* Tx BDR 16 base address register 0 */
#define NETC__ENETC0_SI0_TB16BAR1 NETC__ENETC0_SI0.BDR_NUM[16].TBBAR1.R /* Tx BDR 16 base address register 1 */
#define NETC__ENETC0_SI0_TB16PIR NETC__ENETC0_SI0.BDR_NUM[16].TBPIR.R /* Tx BDR 16 producer index register */
#define NETC__ENETC0_SI0_TB16CIR NETC__ENETC0_SI0.BDR_NUM[16].TBCIR.R /* Tx BDR 16 consumer index register */
#define NETC__ENETC0_SI0_TB16LENR NETC__ENETC0_SI0.BDR_NUM[16].TBLENR.R /* Tx BDR 16 length register */
#define NETC__ENETC0_SI0_TB16IER NETC__ENETC0_SI0.BDR_NUM[16].TBIER.R /* Tx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI0_TB16IDR NETC__ENETC0_SI0.BDR_NUM[16].TBIDR.R /* Tx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI0_TB16ICR0 NETC__ENETC0_SI0.BDR_NUM[16].TBICR0.R /* Tx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB16ICR1 NETC__ENETC0_SI0.BDR_NUM[16].TBICR1.R /* Tx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB16MR NETC__ENETC0_SI0.BDR_NUM[16].RBMR.R /* Rx BDR 16 mode register */
#define NETC__ENETC0_SI0_RB16SR NETC__ENETC0_SI0.BDR_NUM[16].RBSR.R /* Rx BDR 16 status register */
#define NETC__ENETC0_SI0_RB16BSR NETC__ENETC0_SI0.BDR_NUM[16].RBBSR.R /* Rx BDR 16 buffer size register */
#define NETC__ENETC0_SI0_RB16CIR NETC__ENETC0_SI0.BDR_NUM[16].RBCIR.R /* Rx BDR 16 consumer index register */
#define NETC__ENETC0_SI0_RB16BAR0 NETC__ENETC0_SI0.BDR_NUM[16].RBBAR0.R /* Rx BDR 16 base address register 0 */
#define NETC__ENETC0_SI0_RB16BAR1 NETC__ENETC0_SI0.BDR_NUM[16].RBBAR1.R /* Rx BDR 16 base address register 1 */
#define NETC__ENETC0_SI0_RB16PIR NETC__ENETC0_SI0.BDR_NUM[16].RBPIR.R /* Rx BDR 16 producer index register */
#define NETC__ENETC0_SI0_RB16LENR NETC__ENETC0_SI0.BDR_NUM[16].RBLENR.R /* Rx BDR 16 length register */
#define NETC__ENETC0_SI0_RB16DCR NETC__ENETC0_SI0.BDR_NUM[16].RBDCR.R /* Rx BDR 16 drop count register */
#define NETC__ENETC0_SI0_RB16IER NETC__ENETC0_SI0.BDR_NUM[16].RBIER.R /* Rx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI0_RB16IDR NETC__ENETC0_SI0.BDR_NUM[16].RBIDR.R /* Rx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI0_RB16ICR0 NETC__ENETC0_SI0.BDR_NUM[16].RBICR0.R /* Rx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB16ICR1 NETC__ENETC0_SI0.BDR_NUM[16].RBICR1.R /* Rx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_TB17MR NETC__ENETC0_SI0.BDR_NUM[17].TBMR.R /* Tx BDR 17 mode register */
#define NETC__ENETC0_SI0_TB17SR NETC__ENETC0_SI0.BDR_NUM[17].TBSR.R /* Tx BDR 17 status register */
#define NETC__ENETC0_SI0_TB17BAR0 NETC__ENETC0_SI0.BDR_NUM[17].TBBAR0.R /* Tx BDR 17 base address register 0 */
#define NETC__ENETC0_SI0_TB17BAR1 NETC__ENETC0_SI0.BDR_NUM[17].TBBAR1.R /* Tx BDR 17 base address register 1 */
#define NETC__ENETC0_SI0_TB17PIR NETC__ENETC0_SI0.BDR_NUM[17].TBPIR.R /* Tx BDR 17 producer index register */
#define NETC__ENETC0_SI0_TB17CIR NETC__ENETC0_SI0.BDR_NUM[17].TBCIR.R /* Tx BDR 17 consumer index register */
#define NETC__ENETC0_SI0_TB17LENR NETC__ENETC0_SI0.BDR_NUM[17].TBLENR.R /* Tx BDR 17 length register */
#define NETC__ENETC0_SI0_TB17IER NETC__ENETC0_SI0.BDR_NUM[17].TBIER.R /* Tx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI0_TB17IDR NETC__ENETC0_SI0.BDR_NUM[17].TBIDR.R /* Tx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI0_TB17ICR0 NETC__ENETC0_SI0.BDR_NUM[17].TBICR0.R /* Tx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_TB17ICR1 NETC__ENETC0_SI0.BDR_NUM[17].TBICR1.R /* Tx BDR 17 interrupt coalescing register 1 */
#define NETC__ENETC0_SI0_RB17MR NETC__ENETC0_SI0.BDR_NUM[17].RBMR.R /* Rx BDR 17 mode register */
#define NETC__ENETC0_SI0_RB17SR NETC__ENETC0_SI0.BDR_NUM[17].RBSR.R /* Rx BDR 17 status register */
#define NETC__ENETC0_SI0_RB17BSR NETC__ENETC0_SI0.BDR_NUM[17].RBBSR.R /* Rx BDR 17 buffer size register */
#define NETC__ENETC0_SI0_RB17CIR NETC__ENETC0_SI0.BDR_NUM[17].RBCIR.R /* Rx BDR 17 consumer index register */
#define NETC__ENETC0_SI0_RB17BAR0 NETC__ENETC0_SI0.BDR_NUM[17].RBBAR0.R /* Rx BDR 17 base address register 0 */
#define NETC__ENETC0_SI0_RB17BAR1 NETC__ENETC0_SI0.BDR_NUM[17].RBBAR1.R /* Rx BDR 17 base address register 1 */
#define NETC__ENETC0_SI0_RB17PIR NETC__ENETC0_SI0.BDR_NUM[17].RBPIR.R /* Rx BDR 17 producer index register */
#define NETC__ENETC0_SI0_RB17LENR NETC__ENETC0_SI0.BDR_NUM[17].RBLENR.R /* Rx BDR 17 length register */
#define NETC__ENETC0_SI0_RB17DCR NETC__ENETC0_SI0.BDR_NUM[17].RBDCR.R /* Rx BDR 17 drop count register */
#define NETC__ENETC0_SI0_RB17IER NETC__ENETC0_SI0.BDR_NUM[17].RBIER.R /* Rx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI0_RB17IDR NETC__ENETC0_SI0.BDR_NUM[17].RBIDR.R /* Rx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI0_RB17ICR0 NETC__ENETC0_SI0.BDR_NUM[17].RBICR0.R /* Rx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI0_RB17ICR1 NETC__ENETC0_SI0.BDR_NUM[17].RBICR1.R /* Rx BDR 17 interrupt coalescing register 1 */

/* NETC_F3_SI1 */
#define NETC__ENETC0_SI1_SIMR NETC__ENETC0_SI1.SIMR.R      /* Station interface mode register */
#define NETC__ENETC0_SI1_SISR NETC__ENETC0_SI1.SISR.R      /* Station interface status register */
#define NETC__ENETC0_SI1_SICTR0 NETC__ENETC0_SI1.SICTR0.R  /* Station interface current time register 0 */
#define NETC__ENETC0_SI1_SICTR1 NETC__ENETC0_SI1.SICTR1.R  /* Station interface current time register 1 */
#define NETC__ENETC0_SI1_SIPCAPR0 NETC__ENETC0_SI1.SIPCAPR0.R /* Station interface port capability register 0 */
#define NETC__ENETC0_SI1_SIPCAPR1 NETC__ENETC0_SI1.SIPCAPR1.R /* Station interface port capability register 1 */
#define NETC__ENETC0_SI1_SIRBGCR NETC__ENETC0_SI1.SIRBGCR.R /* Station interface receive BDR group control register */
#define NETC__ENETC0_SI1_SIBCAR NETC__ENETC0_SI1.SIBCAR.R  /* Station interface buffer cache attribute register */
#define NETC__ENETC0_SI1_SICCAR NETC__ENETC0_SI1.SICCAR.R  /* Station interface command cache attribute register */
#define NETC__ENETC0_SI1_SIPMAR0 NETC__ENETC0_SI1.SIPMAR0.R /* Station interface primary MAC address register 0 */
#define NETC__ENETC0_SI1_SIPMAR1 NETC__ENETC0_SI1.SIPMAR1.R /* Station interface primary MAC address register 1 */
#define NETC__ENETC0_SI1_SICVLANR1 NETC__ENETC0_SI1.SICVLANR1.R /* Station interface custom VLAN register 1 */
#define NETC__ENETC0_SI1_SICVLANR2 NETC__ENETC0_SI1.SICVLANR2.R /* Station interface custom VLAN register 2 */
#define NETC__ENETC0_SI1_SIVLANIPVMR0 NETC__ENETC0_SI1.SIVLANIPVMR0.R /* Station interface VLAN to IPV mapping register 0 */
#define NETC__ENETC0_SI1_SIVLANIPVMR1 NETC__ENETC0_SI1.SIVLANIPVMR1.R /* Station interface VLAN to IPV mapping register 1 */
#define NETC__ENETC0_SI1_SIIPVBDRMR0 NETC__ENETC0_SI1.SIIPVBDRMR0.R /* Station interface IPV to ring mapping register */
#define NETC__ENETC0_SI1_VSIMSGSR NETC__ENETC0_SI1.VSI_A.VSIMSGSR.R /* Virtual station interface message send register */
#define NETC__ENETC0_SI1_VSIMSGRR NETC__ENETC0_SI1.VSI_A.VSIMSGRR.R /* Virtual station interface message receive register */
#define NETC__ENETC0_SI1_VSIMSGSNDAR0 NETC__ENETC0_SI1.VSI_A.VSIMSGSNDAR0.R /* Virtual station interface message send register 0 */
#define NETC__ENETC0_SI1_VSIMSGSNDAR1 NETC__ENETC0_SI1.VSI_A.VSIMSGSNDAR1.R /* Virtual station interface message send address register 1 */
#define NETC__ENETC0_SI1_SIROCT0 NETC__ENETC0_SI1.SIROCT0.R /* Station interface receive octets counter (ifInOctets) 0 */
#define NETC__ENETC0_SI1_SIROCT1 NETC__ENETC0_SI1.SIROCT1.R /* Station interface receive octets counter (ifInOctets) 1 */
#define NETC__ENETC0_SI1_SIRFRM0 NETC__ENETC0_SI1.SIRFRM0.R /* Station interface receive frame counter (aFrameReceivedOK) 0 */
#define NETC__ENETC0_SI1_SIRFRM1 NETC__ENETC0_SI1.SIRFRM1.R /* Station interface receive frame counter (aFrameReceivedOK) 1 */
#define NETC__ENETC0_SI1_SIRUCA0 NETC__ENETC0_SI1.SIRUCA0.R /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
#define NETC__ENETC0_SI1_SIRUCA1 NETC__ENETC0_SI1.SIRUCA1.R /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
#define NETC__ENETC0_SI1_SIRMCA0 NETC__ENETC0_SI1.SIRMCA0.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
#define NETC__ENETC0_SI1_SIRMCA1 NETC__ENETC0_SI1.SIRMCA1.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
#define NETC__ENETC0_SI1_SITOCT0 NETC__ENETC0_SI1.SITOCT0.R /* Station interface transmit octets counter (ifOutOctets) 0 */
#define NETC__ENETC0_SI1_SITOCT1 NETC__ENETC0_SI1.SITOCT1.R /* Station interface transmit octets counter (ifOutOctets) 1 */
#define NETC__ENETC0_SI1_SITFRM0 NETC__ENETC0_SI1.SITFRM0.R /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
#define NETC__ENETC0_SI1_SITFRM1 NETC__ENETC0_SI1.SITFRM1.R /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
#define NETC__ENETC0_SI1_SITUCA0 NETC__ENETC0_SI1.SITUCA0.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
#define NETC__ENETC0_SI1_SITUCA1 NETC__ENETC0_SI1.SITUCA1.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
#define NETC__ENETC0_SI1_SITMCA0 NETC__ENETC0_SI1.SITMCA0.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
#define NETC__ENETC0_SI1_SITMCA1 NETC__ENETC0_SI1.SITMCA1.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
#define NETC__ENETC0_SI1_SIBLPR0 NETC__ENETC0_SI1.SIBLPR[0].R /* Station interface boot loader parameter register 0 */
#define NETC__ENETC0_SI1_SIBLPR1 NETC__ENETC0_SI1.SIBLPR[1].R /* Station interface boot loader parameter register 1 */
#define NETC__ENETC0_SI1_SICBDRMR NETC__ENETC0_SI1.SICBDRMR.R /* Station interface command BDR mode register */
#define NETC__ENETC0_SI1_SICBDRSR NETC__ENETC0_SI1.SICBDRSR.R /* Station interface command BDR status register */
#define NETC__ENETC0_SI1_SICBDRBAR0 NETC__ENETC0_SI1.SICBDRBAR0.R /* Station interface command BDR base address register 0 */
#define NETC__ENETC0_SI1_SICBDRBAR1 NETC__ENETC0_SI1.SICBDRBAR1.R /* Station interface command BDR base address register 1 */
#define NETC__ENETC0_SI1_SICBDRPIR NETC__ENETC0_SI1.SICBDRPIR.R /* Station interface command BDR producer index register */
#define NETC__ENETC0_SI1_SICBDRCIR NETC__ENETC0_SI1.SICBDRCIR.R /* Station interface command BDR consumer index register */
#define NETC__ENETC0_SI1_SICBDRLENR NETC__ENETC0_SI1.SICBDRLENR.R /* Station interface command BDR length register */
#define NETC__ENETC0_SI1_SICBDRIER NETC__ENETC0_SI1.SICBDRIER.R /* Station interface command BDR interrupt enable register */
#define NETC__ENETC0_SI1_SICBDRIDR NETC__ENETC0_SI1.SICBDRIDR.R /* Station interface command BDR interrupt detect register */
#define NETC__ENETC0_SI1_SICAPR0 NETC__ENETC0_SI1.SICAPR0.R /* Station interface capability register 0 */
#define NETC__ENETC0_SI1_SICAPR1 NETC__ENETC0_SI1.SICAPR1.R /* Station interface capability register 1 */
#define NETC__ENETC0_SI1_SICAPR2 NETC__ENETC0_SI1.SICAPR2.R /* Station interface capability register 2 */
#define NETC__ENETC0_SI1_VSIIER NETC__ENETC0_SI1.VSI.VSIIER.R /* Virtual station interface interrupt enable register */
#define NETC__ENETC0_SI1_VSIIDR NETC__ENETC0_SI1.VSI.VSIIDR.R /* Virtual station interface interrupt detect register */
#define NETC__ENETC0_SI1_SITXIDR0 NETC__ENETC0_SI1.SITXIDR0.R /* Station interface transmit interrupt detect register 0 */
#define NETC__ENETC0_SI1_SITXIDR1 NETC__ENETC0_SI1.SITXIDR1.R /* Station interface transmit interrupt detect register 1 */
#define NETC__ENETC0_SI1_SIRXIDR0 NETC__ENETC0_SI1.SIRXIDR0.R /* Station interface receive interrupt detect register 0 */
#define NETC__ENETC0_SI1_SIRXIDR1 NETC__ENETC0_SI1.SIRXIDR1.R /* Station interface receive interrupt detect register 1 */
#define NETC__ENETC0_SI1_SIMSIVR NETC__ENETC0_SI1.SIMSIVR.R /* Station interface MSI-X vector register */
#define NETC__ENETC0_SI1_SICMSIVR NETC__ENETC0_SI1.SICMSIVR.R /* Station interface command MSI-X vector register */
#define NETC__ENETC0_SI1_SITMRIER NETC__ENETC0_SI1.SITMRIER.R /* Station interface timer interrupt enable register */
#define NETC__ENETC0_SI1_SITMRIDR NETC__ENETC0_SI1.SITMRIDR.R /* Station interface timer interrupt detect register */
#define NETC__ENETC0_SI1_SITMRMSIVR NETC__ENETC0_SI1.SITMRMSIVR.R /* Station interface timer MSI-X vector register */
#define NETC__ENETC0_SI1_SIMSITR0VR NETC__ENETC0_SI1.SIMSITRVR[0].R /* Station interface MSI-X transmit ring 0 vector register */
#define NETC__ENETC0_SI1_SIMSITR1VR NETC__ENETC0_SI1.SIMSITRVR[1].R /* Station interface MSI-X transmit ring 1 vector register */
#define NETC__ENETC0_SI1_SIMSITR2VR NETC__ENETC0_SI1.SIMSITRVR[2].R /* Station interface MSI-X transmit ring 2 vector register */
#define NETC__ENETC0_SI1_SIMSITR3VR NETC__ENETC0_SI1.SIMSITRVR[3].R /* Station interface MSI-X transmit ring 3 vector register */
#define NETC__ENETC0_SI1_SIMSITR4VR NETC__ENETC0_SI1.SIMSITRVR[4].R /* Station interface MSI-X transmit ring 4 vector register */
#define NETC__ENETC0_SI1_SIMSITR5VR NETC__ENETC0_SI1.SIMSITRVR[5].R /* Station interface MSI-X transmit ring 5 vector register */
#define NETC__ENETC0_SI1_SIMSITR6VR NETC__ENETC0_SI1.SIMSITRVR[6].R /* Station interface MSI-X transmit ring 6 vector register */
#define NETC__ENETC0_SI1_SIMSITR7VR NETC__ENETC0_SI1.SIMSITRVR[7].R /* Station interface MSI-X transmit ring 7 vector register */
#define NETC__ENETC0_SI1_SIMSITR8VR NETC__ENETC0_SI1.SIMSITRVR[8].R /* Station interface MSI-X transmit ring 8 vector register */
#define NETC__ENETC0_SI1_SIMSITR9VR NETC__ENETC0_SI1.SIMSITRVR[9].R /* Station interface MSI-X transmit ring 9 vector register */
#define NETC__ENETC0_SI1_SIMSITR10VR NETC__ENETC0_SI1.SIMSITRVR[10].R /* Station interface MSI-X transmit ring 10 vector register */
#define NETC__ENETC0_SI1_SIMSITR11VR NETC__ENETC0_SI1.SIMSITRVR[11].R /* Station interface MSI-X transmit ring 11 vector register */
#define NETC__ENETC0_SI1_SIMSITR12VR NETC__ENETC0_SI1.SIMSITRVR[12].R /* Station interface MSI-X transmit ring 12 vector register */
#define NETC__ENETC0_SI1_SIMSITR13VR NETC__ENETC0_SI1.SIMSITRVR[13].R /* Station interface MSI-X transmit ring 13 vector register */
#define NETC__ENETC0_SI1_SIMSITR14VR NETC__ENETC0_SI1.SIMSITRVR[14].R /* Station interface MSI-X transmit ring 14 vector register */
#define NETC__ENETC0_SI1_SIMSITR15VR NETC__ENETC0_SI1.SIMSITRVR[15].R /* Station interface MSI-X transmit ring 15 vector register */
#define NETC__ENETC0_SI1_SIMSITR16VR NETC__ENETC0_SI1.SIMSITRVR[16].R /* Station interface MSI-X transmit ring 16 vector register */
#define NETC__ENETC0_SI1_SIMSITR17VR NETC__ENETC0_SI1.SIMSITRVR[17].R /* Station interface MSI-X transmit ring 17 vector register */
#define NETC__ENETC0_SI1_SIMSIRR0VR NETC__ENETC0_SI1.SIMSIRRVR[0].R /* Station interface MSI-X receive ring 0 vector register */
#define NETC__ENETC0_SI1_SIMSIRR1VR NETC__ENETC0_SI1.SIMSIRRVR[1].R /* Station interface MSI-X receive ring 1 vector register */
#define NETC__ENETC0_SI1_SIMSIRR2VR NETC__ENETC0_SI1.SIMSIRRVR[2].R /* Station interface MSI-X receive ring 2 vector register */
#define NETC__ENETC0_SI1_SIMSIRR3VR NETC__ENETC0_SI1.SIMSIRRVR[3].R /* Station interface MSI-X receive ring 3 vector register */
#define NETC__ENETC0_SI1_SIMSIRR4VR NETC__ENETC0_SI1.SIMSIRRVR[4].R /* Station interface MSI-X receive ring 4 vector register */
#define NETC__ENETC0_SI1_SIMSIRR5VR NETC__ENETC0_SI1.SIMSIRRVR[5].R /* Station interface MSI-X receive ring 5 vector register */
#define NETC__ENETC0_SI1_SIMSIRR6VR NETC__ENETC0_SI1.SIMSIRRVR[6].R /* Station interface MSI-X receive ring 6 vector register */
#define NETC__ENETC0_SI1_SIMSIRR7VR NETC__ENETC0_SI1.SIMSIRRVR[7].R /* Station interface MSI-X receive ring 7 vector register */
#define NETC__ENETC0_SI1_SIMSIRR8VR NETC__ENETC0_SI1.SIMSIRRVR[8].R /* Station interface MSI-X receive ring 8 vector register */
#define NETC__ENETC0_SI1_SIMSIRR9VR NETC__ENETC0_SI1.SIMSIRRVR[9].R /* Station interface MSI-X receive ring 9 vector register */
#define NETC__ENETC0_SI1_SIMSIRR10VR NETC__ENETC0_SI1.SIMSIRRVR[10].R /* Station interface MSI-X receive ring 10 vector register */
#define NETC__ENETC0_SI1_SIMSIRR11VR NETC__ENETC0_SI1.SIMSIRRVR[11].R /* Station interface MSI-X receive ring 11 vector register */
#define NETC__ENETC0_SI1_SIMSIRR12VR NETC__ENETC0_SI1.SIMSIRRVR[12].R /* Station interface MSI-X receive ring 12 vector register */
#define NETC__ENETC0_SI1_SIMSIRR13VR NETC__ENETC0_SI1.SIMSIRRVR[13].R /* Station interface MSI-X receive ring 13 vector register */
#define NETC__ENETC0_SI1_SIMSIRR14VR NETC__ENETC0_SI1.SIMSIRRVR[14].R /* Station interface MSI-X receive ring 14 vector register */
#define NETC__ENETC0_SI1_SIMSIRR15VR NETC__ENETC0_SI1.SIMSIRRVR[15].R /* Station interface MSI-X receive ring 15 vector register */
#define NETC__ENETC0_SI1_SIMSIRR16VR NETC__ENETC0_SI1.SIMSIRRVR[16].R /* Station interface MSI-X receive ring 16 vector register */
#define NETC__ENETC0_SI1_SIMSIRR17VR NETC__ENETC0_SI1.SIMSIRRVR[17].R /* Station interface MSI-X receive ring 17 vector register */
#define NETC__ENETC0_SI1_SICMECR NETC__ENETC0_SI1.SICMECR.R /* Station interface correctable memory error configuration register */
#define NETC__ENETC0_SI1_SICMESR NETC__ENETC0_SI1.SICMESR.R /* Station interface correctable memory error status register */
#define NETC__ENETC0_SI1_SICMECTR NETC__ENETC0_SI1.SICMECTR.R /* Station interface correctable memory error count register */
#define NETC__ENETC0_SI1_SIUPECR NETC__ENETC0_SI1.SIUPECR.R /* Station interface uncorrectable programming error configuration register */
#define NETC__ENETC0_SI1_SIUPESR NETC__ENETC0_SI1.SIUPESR.R /* Station interface uncorrectable programming error status register */
#define NETC__ENETC0_SI1_SIUPECTR NETC__ENETC0_SI1.SIUPECTR.R /* Station interface uncorrectable programming error count register */
#define NETC__ENETC0_SI1_SIUNSBECR NETC__ENETC0_SI1.SIUNSBECR.R /* Station interface uncorrectable non-fatal system bus error configuration register */
#define NETC__ENETC0_SI1_SIUNSBESR NETC__ENETC0_SI1.SIUNSBESR.R /* Station interface uncorrectable non-fatal system bus error status register */
#define NETC__ENETC0_SI1_SIUNSBECTR NETC__ENETC0_SI1.SIUNSBECTR.R /* Station interface uncorrectable non-fatal system bus error count register */
#define NETC__ENETC0_SI1_SIUFSBECR NETC__ENETC0_SI1.SIUFSBECR.R /* Station interface uncorrectable fatal system bus error configuration register */
#define NETC__ENETC0_SI1_SIUFSBESR NETC__ENETC0_SI1.SIUFSBESR.R /* Station interface uncorrectable fatal system bus error status register */
#define NETC__ENETC0_SI1_SIUNMECR NETC__ENETC0_SI1.SIUNMECR.R /* Station interface uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_SI1_SIUNMESR0 NETC__ENETC0_SI1.SIUNMESR0.R /* Station interface uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_SI1_SIUNMESR1 NETC__ENETC0_SI1.SIUNMESR1.R /* Station interface uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_SI1_SIUNMECTR NETC__ENETC0_SI1.SIUNMECTR.R /* Station interface uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_SI1_SIUFMECR NETC__ENETC0_SI1.SIUFMECR.R /* Station interface uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_SI1_SIUFMESR0 NETC__ENETC0_SI1.SIUFMESR0.R /* Station interface uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_SI1_SIUFMESR1 NETC__ENETC0_SI1.SIUFMESR1.R /* Station interface uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_SI1_SIUNIECR NETC__ENETC0_SI1.SIUNIECR.R /* Station interface uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_SI1_SIUNIESR NETC__ENETC0_SI1.SIUNIESR.R /* Station interface uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_SI1_SIUNIECTR NETC__ENETC0_SI1.SIUNIECTR.R /* Station interface uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_SI1_SIUFIECR NETC__ENETC0_SI1.SIUFIECR.R /* Station interface uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_SI1_SIUFIESR NETC__ENETC0_SI1.SIUFIESR.R /* Station interface uncorrectable fatal integrity error status register */
#define NETC__ENETC0_SI1_SIMAFTCAPR NETC__ENETC0_SI1.SIMAFTCAPR.R /* Station interface MAC address filter table capability register */
#define NETC__ENETC0_SI1_SIVFTCAPR NETC__ENETC0_SI1.SIVFTCAPR.R /* Station interface VLAN filter table capability register */
#define NETC__ENETC0_SI1_SIRFSCAPR NETC__ENETC0_SI1.SIRFSCAPR.R /* Station interface RFS capability register */
#define NETC__ENETC0_SI1_TB0MR NETC__ENETC0_SI1.BDR_NUM[0].TBMR.R /* Tx BDR 0 mode register */
#define NETC__ENETC0_SI1_TB0SR NETC__ENETC0_SI1.BDR_NUM[0].TBSR.R /* Tx BDR 0 status register */
#define NETC__ENETC0_SI1_TB0BAR0 NETC__ENETC0_SI1.BDR_NUM[0].TBBAR0.R /* Tx BDR 0 base address register 0 */
#define NETC__ENETC0_SI1_TB0BAR1 NETC__ENETC0_SI1.BDR_NUM[0].TBBAR1.R /* Tx BDR 0 base address register 1 */
#define NETC__ENETC0_SI1_TB0PIR NETC__ENETC0_SI1.BDR_NUM[0].TBPIR.R /* Tx BDR 0 producer index register */
#define NETC__ENETC0_SI1_TB0CIR NETC__ENETC0_SI1.BDR_NUM[0].TBCIR.R /* Tx BDR 0 consumer index register */
#define NETC__ENETC0_SI1_TB0LENR NETC__ENETC0_SI1.BDR_NUM[0].TBLENR.R /* Tx BDR 0 length register */
#define NETC__ENETC0_SI1_TB0IER NETC__ENETC0_SI1.BDR_NUM[0].TBIER.R /* Tx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI1_TB0IDR NETC__ENETC0_SI1.BDR_NUM[0].TBIDR.R /* Tx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI1_TB0ICR0 NETC__ENETC0_SI1.BDR_NUM[0].TBICR0.R /* Tx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB0ICR1 NETC__ENETC0_SI1.BDR_NUM[0].TBICR1.R /* Tx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB0MR NETC__ENETC0_SI1.BDR_NUM[0].RBMR.R /* Rx BDR 0 mode register */
#define NETC__ENETC0_SI1_RB0SR NETC__ENETC0_SI1.BDR_NUM[0].RBSR.R /* Rx BDR 0 status register */
#define NETC__ENETC0_SI1_RB0BSR NETC__ENETC0_SI1.BDR_NUM[0].RBBSR.R /* Rx BDR 0 buffer size register */
#define NETC__ENETC0_SI1_RB0CIR NETC__ENETC0_SI1.BDR_NUM[0].RBCIR.R /* Rx BDR 0 consumer index register */
#define NETC__ENETC0_SI1_RB0BAR0 NETC__ENETC0_SI1.BDR_NUM[0].RBBAR0.R /* Rx BDR 0 base address register 0 */
#define NETC__ENETC0_SI1_RB0BAR1 NETC__ENETC0_SI1.BDR_NUM[0].RBBAR1.R /* Rx BDR 0 base address register 1 */
#define NETC__ENETC0_SI1_RB0PIR NETC__ENETC0_SI1.BDR_NUM[0].RBPIR.R /* Rx BDR 0 producer index register */
#define NETC__ENETC0_SI1_RB0LENR NETC__ENETC0_SI1.BDR_NUM[0].RBLENR.R /* Rx BDR 0 length register */
#define NETC__ENETC0_SI1_RB0DCR NETC__ENETC0_SI1.BDR_NUM[0].RBDCR.R /* Rx BDR 0 drop count register */
#define NETC__ENETC0_SI1_RB0IER NETC__ENETC0_SI1.BDR_NUM[0].RBIER.R /* Rx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI1_RB0IDR NETC__ENETC0_SI1.BDR_NUM[0].RBIDR.R /* Rx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI1_RB0ICR0 NETC__ENETC0_SI1.BDR_NUM[0].RBICR0.R /* Rx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB0ICR1 NETC__ENETC0_SI1.BDR_NUM[0].RBICR1.R /* Rx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB1MR NETC__ENETC0_SI1.BDR_NUM[1].TBMR.R /* Tx BDR 1 mode register */
#define NETC__ENETC0_SI1_TB1SR NETC__ENETC0_SI1.BDR_NUM[1].TBSR.R /* Tx BDR 1 status register */
#define NETC__ENETC0_SI1_TB1BAR0 NETC__ENETC0_SI1.BDR_NUM[1].TBBAR0.R /* Tx BDR 1 base address register 0 */
#define NETC__ENETC0_SI1_TB1BAR1 NETC__ENETC0_SI1.BDR_NUM[1].TBBAR1.R /* Tx BDR 1 base address register 1 */
#define NETC__ENETC0_SI1_TB1PIR NETC__ENETC0_SI1.BDR_NUM[1].TBPIR.R /* Tx BDR 1 producer index register */
#define NETC__ENETC0_SI1_TB1CIR NETC__ENETC0_SI1.BDR_NUM[1].TBCIR.R /* Tx BDR 1 consumer index register */
#define NETC__ENETC0_SI1_TB1LENR NETC__ENETC0_SI1.BDR_NUM[1].TBLENR.R /* Tx BDR 1 length register */
#define NETC__ENETC0_SI1_TB1IER NETC__ENETC0_SI1.BDR_NUM[1].TBIER.R /* Tx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI1_TB1IDR NETC__ENETC0_SI1.BDR_NUM[1].TBIDR.R /* Tx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI1_TB1ICR0 NETC__ENETC0_SI1.BDR_NUM[1].TBICR0.R /* Tx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB1ICR1 NETC__ENETC0_SI1.BDR_NUM[1].TBICR1.R /* Tx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB1MR NETC__ENETC0_SI1.BDR_NUM[1].RBMR.R /* Rx BDR 1 mode register */
#define NETC__ENETC0_SI1_RB1SR NETC__ENETC0_SI1.BDR_NUM[1].RBSR.R /* Rx BDR 1 status register */
#define NETC__ENETC0_SI1_RB1BSR NETC__ENETC0_SI1.BDR_NUM[1].RBBSR.R /* Rx BDR 1 buffer size register */
#define NETC__ENETC0_SI1_RB1CIR NETC__ENETC0_SI1.BDR_NUM[1].RBCIR.R /* Rx BDR 1 consumer index register */
#define NETC__ENETC0_SI1_RB1BAR0 NETC__ENETC0_SI1.BDR_NUM[1].RBBAR0.R /* Rx BDR 1 base address register 0 */
#define NETC__ENETC0_SI1_RB1BAR1 NETC__ENETC0_SI1.BDR_NUM[1].RBBAR1.R /* Rx BDR 1 base address register 1 */
#define NETC__ENETC0_SI1_RB1PIR NETC__ENETC0_SI1.BDR_NUM[1].RBPIR.R /* Rx BDR 1 producer index register */
#define NETC__ENETC0_SI1_RB1LENR NETC__ENETC0_SI1.BDR_NUM[1].RBLENR.R /* Rx BDR 1 length register */
#define NETC__ENETC0_SI1_RB1DCR NETC__ENETC0_SI1.BDR_NUM[1].RBDCR.R /* Rx BDR 1 drop count register */
#define NETC__ENETC0_SI1_RB1IER NETC__ENETC0_SI1.BDR_NUM[1].RBIER.R /* Rx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI1_RB1IDR NETC__ENETC0_SI1.BDR_NUM[1].RBIDR.R /* Rx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI1_RB1ICR0 NETC__ENETC0_SI1.BDR_NUM[1].RBICR0.R /* Rx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB1ICR1 NETC__ENETC0_SI1.BDR_NUM[1].RBICR1.R /* Rx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB2MR NETC__ENETC0_SI1.BDR_NUM[2].TBMR.R /* Tx BDR 2 mode register */
#define NETC__ENETC0_SI1_TB2SR NETC__ENETC0_SI1.BDR_NUM[2].TBSR.R /* Tx BDR 2 status register */
#define NETC__ENETC0_SI1_TB2BAR0 NETC__ENETC0_SI1.BDR_NUM[2].TBBAR0.R /* Tx BDR 2 base address register 0 */
#define NETC__ENETC0_SI1_TB2BAR1 NETC__ENETC0_SI1.BDR_NUM[2].TBBAR1.R /* Tx BDR 2 base address register 1 */
#define NETC__ENETC0_SI1_TB2PIR NETC__ENETC0_SI1.BDR_NUM[2].TBPIR.R /* Tx BDR 2 producer index register */
#define NETC__ENETC0_SI1_TB2CIR NETC__ENETC0_SI1.BDR_NUM[2].TBCIR.R /* Tx BDR 2 consumer index register */
#define NETC__ENETC0_SI1_TB2LENR NETC__ENETC0_SI1.BDR_NUM[2].TBLENR.R /* Tx BDR 2 length register */
#define NETC__ENETC0_SI1_TB2IER NETC__ENETC0_SI1.BDR_NUM[2].TBIER.R /* Tx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI1_TB2IDR NETC__ENETC0_SI1.BDR_NUM[2].TBIDR.R /* Tx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI1_TB2ICR0 NETC__ENETC0_SI1.BDR_NUM[2].TBICR0.R /* Tx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB2ICR1 NETC__ENETC0_SI1.BDR_NUM[2].TBICR1.R /* Tx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB2MR NETC__ENETC0_SI1.BDR_NUM[2].RBMR.R /* Rx BDR 2 mode register */
#define NETC__ENETC0_SI1_RB2SR NETC__ENETC0_SI1.BDR_NUM[2].RBSR.R /* Rx BDR 2 status register */
#define NETC__ENETC0_SI1_RB2BSR NETC__ENETC0_SI1.BDR_NUM[2].RBBSR.R /* Rx BDR 2 buffer size register */
#define NETC__ENETC0_SI1_RB2CIR NETC__ENETC0_SI1.BDR_NUM[2].RBCIR.R /* Rx BDR 2 consumer index register */
#define NETC__ENETC0_SI1_RB2BAR0 NETC__ENETC0_SI1.BDR_NUM[2].RBBAR0.R /* Rx BDR 2 base address register 0 */
#define NETC__ENETC0_SI1_RB2BAR1 NETC__ENETC0_SI1.BDR_NUM[2].RBBAR1.R /* Rx BDR 2 base address register 1 */
#define NETC__ENETC0_SI1_RB2PIR NETC__ENETC0_SI1.BDR_NUM[2].RBPIR.R /* Rx BDR 2 producer index register */
#define NETC__ENETC0_SI1_RB2LENR NETC__ENETC0_SI1.BDR_NUM[2].RBLENR.R /* Rx BDR 2 length register */
#define NETC__ENETC0_SI1_RB2DCR NETC__ENETC0_SI1.BDR_NUM[2].RBDCR.R /* Rx BDR 2 drop count register */
#define NETC__ENETC0_SI1_RB2IER NETC__ENETC0_SI1.BDR_NUM[2].RBIER.R /* Rx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI1_RB2IDR NETC__ENETC0_SI1.BDR_NUM[2].RBIDR.R /* Rx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI1_RB2ICR0 NETC__ENETC0_SI1.BDR_NUM[2].RBICR0.R /* Rx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB2ICR1 NETC__ENETC0_SI1.BDR_NUM[2].RBICR1.R /* Rx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB3MR NETC__ENETC0_SI1.BDR_NUM[3].TBMR.R /* Tx BDR 3 mode register */
#define NETC__ENETC0_SI1_TB3SR NETC__ENETC0_SI1.BDR_NUM[3].TBSR.R /* Tx BDR 3 status register */
#define NETC__ENETC0_SI1_TB3BAR0 NETC__ENETC0_SI1.BDR_NUM[3].TBBAR0.R /* Tx BDR 3 base address register 0 */
#define NETC__ENETC0_SI1_TB3BAR1 NETC__ENETC0_SI1.BDR_NUM[3].TBBAR1.R /* Tx BDR 3 base address register 1 */
#define NETC__ENETC0_SI1_TB3PIR NETC__ENETC0_SI1.BDR_NUM[3].TBPIR.R /* Tx BDR 3 producer index register */
#define NETC__ENETC0_SI1_TB3CIR NETC__ENETC0_SI1.BDR_NUM[3].TBCIR.R /* Tx BDR 3 consumer index register */
#define NETC__ENETC0_SI1_TB3LENR NETC__ENETC0_SI1.BDR_NUM[3].TBLENR.R /* Tx BDR 3 length register */
#define NETC__ENETC0_SI1_TB3IER NETC__ENETC0_SI1.BDR_NUM[3].TBIER.R /* Tx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI1_TB3IDR NETC__ENETC0_SI1.BDR_NUM[3].TBIDR.R /* Tx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI1_TB3ICR0 NETC__ENETC0_SI1.BDR_NUM[3].TBICR0.R /* Tx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB3ICR1 NETC__ENETC0_SI1.BDR_NUM[3].TBICR1.R /* Tx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB3MR NETC__ENETC0_SI1.BDR_NUM[3].RBMR.R /* Rx BDR 3 mode register */
#define NETC__ENETC0_SI1_RB3SR NETC__ENETC0_SI1.BDR_NUM[3].RBSR.R /* Rx BDR 3 status register */
#define NETC__ENETC0_SI1_RB3BSR NETC__ENETC0_SI1.BDR_NUM[3].RBBSR.R /* Rx BDR 3 buffer size register */
#define NETC__ENETC0_SI1_RB3CIR NETC__ENETC0_SI1.BDR_NUM[3].RBCIR.R /* Rx BDR 3 consumer index register */
#define NETC__ENETC0_SI1_RB3BAR0 NETC__ENETC0_SI1.BDR_NUM[3].RBBAR0.R /* Rx BDR 3 base address register 0 */
#define NETC__ENETC0_SI1_RB3BAR1 NETC__ENETC0_SI1.BDR_NUM[3].RBBAR1.R /* Rx BDR 3 base address register 1 */
#define NETC__ENETC0_SI1_RB3PIR NETC__ENETC0_SI1.BDR_NUM[3].RBPIR.R /* Rx BDR 3 producer index register */
#define NETC__ENETC0_SI1_RB3LENR NETC__ENETC0_SI1.BDR_NUM[3].RBLENR.R /* Rx BDR 3 length register */
#define NETC__ENETC0_SI1_RB3DCR NETC__ENETC0_SI1.BDR_NUM[3].RBDCR.R /* Rx BDR 3 drop count register */
#define NETC__ENETC0_SI1_RB3IER NETC__ENETC0_SI1.BDR_NUM[3].RBIER.R /* Rx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI1_RB3IDR NETC__ENETC0_SI1.BDR_NUM[3].RBIDR.R /* Rx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI1_RB3ICR0 NETC__ENETC0_SI1.BDR_NUM[3].RBICR0.R /* Rx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB3ICR1 NETC__ENETC0_SI1.BDR_NUM[3].RBICR1.R /* Rx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB4MR NETC__ENETC0_SI1.BDR_NUM[4].TBMR.R /* Tx BDR 4 mode register */
#define NETC__ENETC0_SI1_TB4SR NETC__ENETC0_SI1.BDR_NUM[4].TBSR.R /* Tx BDR 4 status register */
#define NETC__ENETC0_SI1_TB4BAR0 NETC__ENETC0_SI1.BDR_NUM[4].TBBAR0.R /* Tx BDR 4 base address register 0 */
#define NETC__ENETC0_SI1_TB4BAR1 NETC__ENETC0_SI1.BDR_NUM[4].TBBAR1.R /* Tx BDR 4 base address register 1 */
#define NETC__ENETC0_SI1_TB4PIR NETC__ENETC0_SI1.BDR_NUM[4].TBPIR.R /* Tx BDR 4 producer index register */
#define NETC__ENETC0_SI1_TB4CIR NETC__ENETC0_SI1.BDR_NUM[4].TBCIR.R /* Tx BDR 4 consumer index register */
#define NETC__ENETC0_SI1_TB4LENR NETC__ENETC0_SI1.BDR_NUM[4].TBLENR.R /* Tx BDR 4 length register */
#define NETC__ENETC0_SI1_TB4IER NETC__ENETC0_SI1.BDR_NUM[4].TBIER.R /* Tx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI1_TB4IDR NETC__ENETC0_SI1.BDR_NUM[4].TBIDR.R /* Tx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI1_TB4ICR0 NETC__ENETC0_SI1.BDR_NUM[4].TBICR0.R /* Tx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB4ICR1 NETC__ENETC0_SI1.BDR_NUM[4].TBICR1.R /* Tx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB4MR NETC__ENETC0_SI1.BDR_NUM[4].RBMR.R /* Rx BDR 4 mode register */
#define NETC__ENETC0_SI1_RB4SR NETC__ENETC0_SI1.BDR_NUM[4].RBSR.R /* Rx BDR 4 status register */
#define NETC__ENETC0_SI1_RB4BSR NETC__ENETC0_SI1.BDR_NUM[4].RBBSR.R /* Rx BDR 4 buffer size register */
#define NETC__ENETC0_SI1_RB4CIR NETC__ENETC0_SI1.BDR_NUM[4].RBCIR.R /* Rx BDR 4 consumer index register */
#define NETC__ENETC0_SI1_RB4BAR0 NETC__ENETC0_SI1.BDR_NUM[4].RBBAR0.R /* Rx BDR 4 base address register 0 */
#define NETC__ENETC0_SI1_RB4BAR1 NETC__ENETC0_SI1.BDR_NUM[4].RBBAR1.R /* Rx BDR 4 base address register 1 */
#define NETC__ENETC0_SI1_RB4PIR NETC__ENETC0_SI1.BDR_NUM[4].RBPIR.R /* Rx BDR 4 producer index register */
#define NETC__ENETC0_SI1_RB4LENR NETC__ENETC0_SI1.BDR_NUM[4].RBLENR.R /* Rx BDR 4 length register */
#define NETC__ENETC0_SI1_RB4DCR NETC__ENETC0_SI1.BDR_NUM[4].RBDCR.R /* Rx BDR 4 drop count register */
#define NETC__ENETC0_SI1_RB4IER NETC__ENETC0_SI1.BDR_NUM[4].RBIER.R /* Rx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI1_RB4IDR NETC__ENETC0_SI1.BDR_NUM[4].RBIDR.R /* Rx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI1_RB4ICR0 NETC__ENETC0_SI1.BDR_NUM[4].RBICR0.R /* Rx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB4ICR1 NETC__ENETC0_SI1.BDR_NUM[4].RBICR1.R /* Rx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB5MR NETC__ENETC0_SI1.BDR_NUM[5].TBMR.R /* Tx BDR 5 mode register */
#define NETC__ENETC0_SI1_TB5SR NETC__ENETC0_SI1.BDR_NUM[5].TBSR.R /* Tx BDR 5 status register */
#define NETC__ENETC0_SI1_TB5BAR0 NETC__ENETC0_SI1.BDR_NUM[5].TBBAR0.R /* Tx BDR 5 base address register 0 */
#define NETC__ENETC0_SI1_TB5BAR1 NETC__ENETC0_SI1.BDR_NUM[5].TBBAR1.R /* Tx BDR 5 base address register 1 */
#define NETC__ENETC0_SI1_TB5PIR NETC__ENETC0_SI1.BDR_NUM[5].TBPIR.R /* Tx BDR 5 producer index register */
#define NETC__ENETC0_SI1_TB5CIR NETC__ENETC0_SI1.BDR_NUM[5].TBCIR.R /* Tx BDR 5 consumer index register */
#define NETC__ENETC0_SI1_TB5LENR NETC__ENETC0_SI1.BDR_NUM[5].TBLENR.R /* Tx BDR 5 length register */
#define NETC__ENETC0_SI1_TB5IER NETC__ENETC0_SI1.BDR_NUM[5].TBIER.R /* Tx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI1_TB5IDR NETC__ENETC0_SI1.BDR_NUM[5].TBIDR.R /* Tx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI1_TB5ICR0 NETC__ENETC0_SI1.BDR_NUM[5].TBICR0.R /* Tx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB5ICR1 NETC__ENETC0_SI1.BDR_NUM[5].TBICR1.R /* Tx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB5MR NETC__ENETC0_SI1.BDR_NUM[5].RBMR.R /* Rx BDR 5 mode register */
#define NETC__ENETC0_SI1_RB5SR NETC__ENETC0_SI1.BDR_NUM[5].RBSR.R /* Rx BDR 5 status register */
#define NETC__ENETC0_SI1_RB5BSR NETC__ENETC0_SI1.BDR_NUM[5].RBBSR.R /* Rx BDR 5 buffer size register */
#define NETC__ENETC0_SI1_RB5CIR NETC__ENETC0_SI1.BDR_NUM[5].RBCIR.R /* Rx BDR 5 consumer index register */
#define NETC__ENETC0_SI1_RB5BAR0 NETC__ENETC0_SI1.BDR_NUM[5].RBBAR0.R /* Rx BDR 5 base address register 0 */
#define NETC__ENETC0_SI1_RB5BAR1 NETC__ENETC0_SI1.BDR_NUM[5].RBBAR1.R /* Rx BDR 5 base address register 1 */
#define NETC__ENETC0_SI1_RB5PIR NETC__ENETC0_SI1.BDR_NUM[5].RBPIR.R /* Rx BDR 5 producer index register */
#define NETC__ENETC0_SI1_RB5LENR NETC__ENETC0_SI1.BDR_NUM[5].RBLENR.R /* Rx BDR 5 length register */
#define NETC__ENETC0_SI1_RB5DCR NETC__ENETC0_SI1.BDR_NUM[5].RBDCR.R /* Rx BDR 5 drop count register */
#define NETC__ENETC0_SI1_RB5IER NETC__ENETC0_SI1.BDR_NUM[5].RBIER.R /* Rx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI1_RB5IDR NETC__ENETC0_SI1.BDR_NUM[5].RBIDR.R /* Rx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI1_RB5ICR0 NETC__ENETC0_SI1.BDR_NUM[5].RBICR0.R /* Rx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB5ICR1 NETC__ENETC0_SI1.BDR_NUM[5].RBICR1.R /* Rx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB6MR NETC__ENETC0_SI1.BDR_NUM[6].TBMR.R /* Tx BDR 6 mode register */
#define NETC__ENETC0_SI1_TB6SR NETC__ENETC0_SI1.BDR_NUM[6].TBSR.R /* Tx BDR 6 status register */
#define NETC__ENETC0_SI1_TB6BAR0 NETC__ENETC0_SI1.BDR_NUM[6].TBBAR0.R /* Tx BDR 6 base address register 0 */
#define NETC__ENETC0_SI1_TB6BAR1 NETC__ENETC0_SI1.BDR_NUM[6].TBBAR1.R /* Tx BDR 6 base address register 1 */
#define NETC__ENETC0_SI1_TB6PIR NETC__ENETC0_SI1.BDR_NUM[6].TBPIR.R /* Tx BDR 6 producer index register */
#define NETC__ENETC0_SI1_TB6CIR NETC__ENETC0_SI1.BDR_NUM[6].TBCIR.R /* Tx BDR 6 consumer index register */
#define NETC__ENETC0_SI1_TB6LENR NETC__ENETC0_SI1.BDR_NUM[6].TBLENR.R /* Tx BDR 6 length register */
#define NETC__ENETC0_SI1_TB6IER NETC__ENETC0_SI1.BDR_NUM[6].TBIER.R /* Tx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI1_TB6IDR NETC__ENETC0_SI1.BDR_NUM[6].TBIDR.R /* Tx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI1_TB6ICR0 NETC__ENETC0_SI1.BDR_NUM[6].TBICR0.R /* Tx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB6ICR1 NETC__ENETC0_SI1.BDR_NUM[6].TBICR1.R /* Tx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB6MR NETC__ENETC0_SI1.BDR_NUM[6].RBMR.R /* Rx BDR 6 mode register */
#define NETC__ENETC0_SI1_RB6SR NETC__ENETC0_SI1.BDR_NUM[6].RBSR.R /* Rx BDR 6 status register */
#define NETC__ENETC0_SI1_RB6BSR NETC__ENETC0_SI1.BDR_NUM[6].RBBSR.R /* Rx BDR 6 buffer size register */
#define NETC__ENETC0_SI1_RB6CIR NETC__ENETC0_SI1.BDR_NUM[6].RBCIR.R /* Rx BDR 6 consumer index register */
#define NETC__ENETC0_SI1_RB6BAR0 NETC__ENETC0_SI1.BDR_NUM[6].RBBAR0.R /* Rx BDR 6 base address register 0 */
#define NETC__ENETC0_SI1_RB6BAR1 NETC__ENETC0_SI1.BDR_NUM[6].RBBAR1.R /* Rx BDR 6 base address register 1 */
#define NETC__ENETC0_SI1_RB6PIR NETC__ENETC0_SI1.BDR_NUM[6].RBPIR.R /* Rx BDR 6 producer index register */
#define NETC__ENETC0_SI1_RB6LENR NETC__ENETC0_SI1.BDR_NUM[6].RBLENR.R /* Rx BDR 6 length register */
#define NETC__ENETC0_SI1_RB6DCR NETC__ENETC0_SI1.BDR_NUM[6].RBDCR.R /* Rx BDR 6 drop count register */
#define NETC__ENETC0_SI1_RB6IER NETC__ENETC0_SI1.BDR_NUM[6].RBIER.R /* Rx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI1_RB6IDR NETC__ENETC0_SI1.BDR_NUM[6].RBIDR.R /* Rx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI1_RB6ICR0 NETC__ENETC0_SI1.BDR_NUM[6].RBICR0.R /* Rx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB6ICR1 NETC__ENETC0_SI1.BDR_NUM[6].RBICR1.R /* Rx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB7MR NETC__ENETC0_SI1.BDR_NUM[7].TBMR.R /* Tx BDR 7 mode register */
#define NETC__ENETC0_SI1_TB7SR NETC__ENETC0_SI1.BDR_NUM[7].TBSR.R /* Tx BDR 7 status register */
#define NETC__ENETC0_SI1_TB7BAR0 NETC__ENETC0_SI1.BDR_NUM[7].TBBAR0.R /* Tx BDR 7 base address register 0 */
#define NETC__ENETC0_SI1_TB7BAR1 NETC__ENETC0_SI1.BDR_NUM[7].TBBAR1.R /* Tx BDR 7 base address register 1 */
#define NETC__ENETC0_SI1_TB7PIR NETC__ENETC0_SI1.BDR_NUM[7].TBPIR.R /* Tx BDR 7 producer index register */
#define NETC__ENETC0_SI1_TB7CIR NETC__ENETC0_SI1.BDR_NUM[7].TBCIR.R /* Tx BDR 7 consumer index register */
#define NETC__ENETC0_SI1_TB7LENR NETC__ENETC0_SI1.BDR_NUM[7].TBLENR.R /* Tx BDR 7 length register */
#define NETC__ENETC0_SI1_TB7IER NETC__ENETC0_SI1.BDR_NUM[7].TBIER.R /* Tx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI1_TB7IDR NETC__ENETC0_SI1.BDR_NUM[7].TBIDR.R /* Tx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI1_TB7ICR0 NETC__ENETC0_SI1.BDR_NUM[7].TBICR0.R /* Tx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB7ICR1 NETC__ENETC0_SI1.BDR_NUM[7].TBICR1.R /* Tx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB7MR NETC__ENETC0_SI1.BDR_NUM[7].RBMR.R /* Rx BDR 7 mode register */
#define NETC__ENETC0_SI1_RB7SR NETC__ENETC0_SI1.BDR_NUM[7].RBSR.R /* Rx BDR 7 status register */
#define NETC__ENETC0_SI1_RB7BSR NETC__ENETC0_SI1.BDR_NUM[7].RBBSR.R /* Rx BDR 7 buffer size register */
#define NETC__ENETC0_SI1_RB7CIR NETC__ENETC0_SI1.BDR_NUM[7].RBCIR.R /* Rx BDR 7 consumer index register */
#define NETC__ENETC0_SI1_RB7BAR0 NETC__ENETC0_SI1.BDR_NUM[7].RBBAR0.R /* Rx BDR 7 base address register 0 */
#define NETC__ENETC0_SI1_RB7BAR1 NETC__ENETC0_SI1.BDR_NUM[7].RBBAR1.R /* Rx BDR 7 base address register 1 */
#define NETC__ENETC0_SI1_RB7PIR NETC__ENETC0_SI1.BDR_NUM[7].RBPIR.R /* Rx BDR 7 producer index register */
#define NETC__ENETC0_SI1_RB7LENR NETC__ENETC0_SI1.BDR_NUM[7].RBLENR.R /* Rx BDR 7 length register */
#define NETC__ENETC0_SI1_RB7DCR NETC__ENETC0_SI1.BDR_NUM[7].RBDCR.R /* Rx BDR 7 drop count register */
#define NETC__ENETC0_SI1_RB7IER NETC__ENETC0_SI1.BDR_NUM[7].RBIER.R /* Rx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI1_RB7IDR NETC__ENETC0_SI1.BDR_NUM[7].RBIDR.R /* Rx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI1_RB7ICR0 NETC__ENETC0_SI1.BDR_NUM[7].RBICR0.R /* Rx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB7ICR1 NETC__ENETC0_SI1.BDR_NUM[7].RBICR1.R /* Rx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB8MR NETC__ENETC0_SI1.BDR_NUM[8].TBMR.R /* Tx BDR 8 mode register */
#define NETC__ENETC0_SI1_TB8SR NETC__ENETC0_SI1.BDR_NUM[8].TBSR.R /* Tx BDR 8 status register */
#define NETC__ENETC0_SI1_TB8BAR0 NETC__ENETC0_SI1.BDR_NUM[8].TBBAR0.R /* Tx BDR 8 base address register 0 */
#define NETC__ENETC0_SI1_TB8BAR1 NETC__ENETC0_SI1.BDR_NUM[8].TBBAR1.R /* Tx BDR 8 base address register 1 */
#define NETC__ENETC0_SI1_TB8PIR NETC__ENETC0_SI1.BDR_NUM[8].TBPIR.R /* Tx BDR 8 producer index register */
#define NETC__ENETC0_SI1_TB8CIR NETC__ENETC0_SI1.BDR_NUM[8].TBCIR.R /* Tx BDR 8 consumer index register */
#define NETC__ENETC0_SI1_TB8LENR NETC__ENETC0_SI1.BDR_NUM[8].TBLENR.R /* Tx BDR 8 length register */
#define NETC__ENETC0_SI1_TB8IER NETC__ENETC0_SI1.BDR_NUM[8].TBIER.R /* Tx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI1_TB8IDR NETC__ENETC0_SI1.BDR_NUM[8].TBIDR.R /* Tx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI1_TB8ICR0 NETC__ENETC0_SI1.BDR_NUM[8].TBICR0.R /* Tx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB8ICR1 NETC__ENETC0_SI1.BDR_NUM[8].TBICR1.R /* Tx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB8MR NETC__ENETC0_SI1.BDR_NUM[8].RBMR.R /* Rx BDR 8 mode register */
#define NETC__ENETC0_SI1_RB8SR NETC__ENETC0_SI1.BDR_NUM[8].RBSR.R /* Rx BDR 8 status register */
#define NETC__ENETC0_SI1_RB8BSR NETC__ENETC0_SI1.BDR_NUM[8].RBBSR.R /* Rx BDR 8 buffer size register */
#define NETC__ENETC0_SI1_RB8CIR NETC__ENETC0_SI1.BDR_NUM[8].RBCIR.R /* Rx BDR 8 consumer index register */
#define NETC__ENETC0_SI1_RB8BAR0 NETC__ENETC0_SI1.BDR_NUM[8].RBBAR0.R /* Rx BDR 8 base address register 0 */
#define NETC__ENETC0_SI1_RB8BAR1 NETC__ENETC0_SI1.BDR_NUM[8].RBBAR1.R /* Rx BDR 8 base address register 1 */
#define NETC__ENETC0_SI1_RB8PIR NETC__ENETC0_SI1.BDR_NUM[8].RBPIR.R /* Rx BDR 8 producer index register */
#define NETC__ENETC0_SI1_RB8LENR NETC__ENETC0_SI1.BDR_NUM[8].RBLENR.R /* Rx BDR 8 length register */
#define NETC__ENETC0_SI1_RB8DCR NETC__ENETC0_SI1.BDR_NUM[8].RBDCR.R /* Rx BDR 8 drop count register */
#define NETC__ENETC0_SI1_RB8IER NETC__ENETC0_SI1.BDR_NUM[8].RBIER.R /* Rx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI1_RB8IDR NETC__ENETC0_SI1.BDR_NUM[8].RBIDR.R /* Rx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI1_RB8ICR0 NETC__ENETC0_SI1.BDR_NUM[8].RBICR0.R /* Rx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB8ICR1 NETC__ENETC0_SI1.BDR_NUM[8].RBICR1.R /* Rx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB9MR NETC__ENETC0_SI1.BDR_NUM[9].TBMR.R /* Tx BDR 9 mode register */
#define NETC__ENETC0_SI1_TB9SR NETC__ENETC0_SI1.BDR_NUM[9].TBSR.R /* Tx BDR 9 status register */
#define NETC__ENETC0_SI1_TB9BAR0 NETC__ENETC0_SI1.BDR_NUM[9].TBBAR0.R /* Tx BDR 9 base address register 0 */
#define NETC__ENETC0_SI1_TB9BAR1 NETC__ENETC0_SI1.BDR_NUM[9].TBBAR1.R /* Tx BDR 9 base address register 1 */
#define NETC__ENETC0_SI1_TB9PIR NETC__ENETC0_SI1.BDR_NUM[9].TBPIR.R /* Tx BDR 9 producer index register */
#define NETC__ENETC0_SI1_TB9CIR NETC__ENETC0_SI1.BDR_NUM[9].TBCIR.R /* Tx BDR 9 consumer index register */
#define NETC__ENETC0_SI1_TB9LENR NETC__ENETC0_SI1.BDR_NUM[9].TBLENR.R /* Tx BDR 9 length register */
#define NETC__ENETC0_SI1_TB9IER NETC__ENETC0_SI1.BDR_NUM[9].TBIER.R /* Tx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI1_TB9IDR NETC__ENETC0_SI1.BDR_NUM[9].TBIDR.R /* Tx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI1_TB9ICR0 NETC__ENETC0_SI1.BDR_NUM[9].TBICR0.R /* Tx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB9ICR1 NETC__ENETC0_SI1.BDR_NUM[9].TBICR1.R /* Tx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB9MR NETC__ENETC0_SI1.BDR_NUM[9].RBMR.R /* Rx BDR 9 mode register */
#define NETC__ENETC0_SI1_RB9SR NETC__ENETC0_SI1.BDR_NUM[9].RBSR.R /* Rx BDR 9 status register */
#define NETC__ENETC0_SI1_RB9BSR NETC__ENETC0_SI1.BDR_NUM[9].RBBSR.R /* Rx BDR 9 buffer size register */
#define NETC__ENETC0_SI1_RB9CIR NETC__ENETC0_SI1.BDR_NUM[9].RBCIR.R /* Rx BDR 9 consumer index register */
#define NETC__ENETC0_SI1_RB9BAR0 NETC__ENETC0_SI1.BDR_NUM[9].RBBAR0.R /* Rx BDR 9 base address register 0 */
#define NETC__ENETC0_SI1_RB9BAR1 NETC__ENETC0_SI1.BDR_NUM[9].RBBAR1.R /* Rx BDR 9 base address register 1 */
#define NETC__ENETC0_SI1_RB9PIR NETC__ENETC0_SI1.BDR_NUM[9].RBPIR.R /* Rx BDR 9 producer index register */
#define NETC__ENETC0_SI1_RB9LENR NETC__ENETC0_SI1.BDR_NUM[9].RBLENR.R /* Rx BDR 9 length register */
#define NETC__ENETC0_SI1_RB9DCR NETC__ENETC0_SI1.BDR_NUM[9].RBDCR.R /* Rx BDR 9 drop count register */
#define NETC__ENETC0_SI1_RB9IER NETC__ENETC0_SI1.BDR_NUM[9].RBIER.R /* Rx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI1_RB9IDR NETC__ENETC0_SI1.BDR_NUM[9].RBIDR.R /* Rx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI1_RB9ICR0 NETC__ENETC0_SI1.BDR_NUM[9].RBICR0.R /* Rx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB9ICR1 NETC__ENETC0_SI1.BDR_NUM[9].RBICR1.R /* Rx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB10MR NETC__ENETC0_SI1.BDR_NUM[10].TBMR.R /* Tx BDR 10 mode register */
#define NETC__ENETC0_SI1_TB10SR NETC__ENETC0_SI1.BDR_NUM[10].TBSR.R /* Tx BDR 10 status register */
#define NETC__ENETC0_SI1_TB10BAR0 NETC__ENETC0_SI1.BDR_NUM[10].TBBAR0.R /* Tx BDR 10 base address register 0 */
#define NETC__ENETC0_SI1_TB10BAR1 NETC__ENETC0_SI1.BDR_NUM[10].TBBAR1.R /* Tx BDR 10 base address register 1 */
#define NETC__ENETC0_SI1_TB10PIR NETC__ENETC0_SI1.BDR_NUM[10].TBPIR.R /* Tx BDR 10 producer index register */
#define NETC__ENETC0_SI1_TB10CIR NETC__ENETC0_SI1.BDR_NUM[10].TBCIR.R /* Tx BDR 10 consumer index register */
#define NETC__ENETC0_SI1_TB10LENR NETC__ENETC0_SI1.BDR_NUM[10].TBLENR.R /* Tx BDR 10 length register */
#define NETC__ENETC0_SI1_TB10IER NETC__ENETC0_SI1.BDR_NUM[10].TBIER.R /* Tx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI1_TB10IDR NETC__ENETC0_SI1.BDR_NUM[10].TBIDR.R /* Tx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI1_TB10ICR0 NETC__ENETC0_SI1.BDR_NUM[10].TBICR0.R /* Tx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB10ICR1 NETC__ENETC0_SI1.BDR_NUM[10].TBICR1.R /* Tx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB10MR NETC__ENETC0_SI1.BDR_NUM[10].RBMR.R /* Rx BDR 10 mode register */
#define NETC__ENETC0_SI1_RB10SR NETC__ENETC0_SI1.BDR_NUM[10].RBSR.R /* Rx BDR 10 status register */
#define NETC__ENETC0_SI1_RB10BSR NETC__ENETC0_SI1.BDR_NUM[10].RBBSR.R /* Rx BDR 10 buffer size register */
#define NETC__ENETC0_SI1_RB10CIR NETC__ENETC0_SI1.BDR_NUM[10].RBCIR.R /* Rx BDR 10 consumer index register */
#define NETC__ENETC0_SI1_RB10BAR0 NETC__ENETC0_SI1.BDR_NUM[10].RBBAR0.R /* Rx BDR 10 base address register 0 */
#define NETC__ENETC0_SI1_RB10BAR1 NETC__ENETC0_SI1.BDR_NUM[10].RBBAR1.R /* Rx BDR 10 base address register 1 */
#define NETC__ENETC0_SI1_RB10PIR NETC__ENETC0_SI1.BDR_NUM[10].RBPIR.R /* Rx BDR 10 producer index register */
#define NETC__ENETC0_SI1_RB10LENR NETC__ENETC0_SI1.BDR_NUM[10].RBLENR.R /* Rx BDR 10 length register */
#define NETC__ENETC0_SI1_RB10DCR NETC__ENETC0_SI1.BDR_NUM[10].RBDCR.R /* Rx BDR 10 drop count register */
#define NETC__ENETC0_SI1_RB10IER NETC__ENETC0_SI1.BDR_NUM[10].RBIER.R /* Rx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI1_RB10IDR NETC__ENETC0_SI1.BDR_NUM[10].RBIDR.R /* Rx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI1_RB10ICR0 NETC__ENETC0_SI1.BDR_NUM[10].RBICR0.R /* Rx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB10ICR1 NETC__ENETC0_SI1.BDR_NUM[10].RBICR1.R /* Rx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB11MR NETC__ENETC0_SI1.BDR_NUM[11].TBMR.R /* Tx BDR 11 mode register */
#define NETC__ENETC0_SI1_TB11SR NETC__ENETC0_SI1.BDR_NUM[11].TBSR.R /* Tx BDR 11 status register */
#define NETC__ENETC0_SI1_TB11BAR0 NETC__ENETC0_SI1.BDR_NUM[11].TBBAR0.R /* Tx BDR 11 base address register 0 */
#define NETC__ENETC0_SI1_TB11BAR1 NETC__ENETC0_SI1.BDR_NUM[11].TBBAR1.R /* Tx BDR 11 base address register 1 */
#define NETC__ENETC0_SI1_TB11PIR NETC__ENETC0_SI1.BDR_NUM[11].TBPIR.R /* Tx BDR 11 producer index register */
#define NETC__ENETC0_SI1_TB11CIR NETC__ENETC0_SI1.BDR_NUM[11].TBCIR.R /* Tx BDR 11 consumer index register */
#define NETC__ENETC0_SI1_TB11LENR NETC__ENETC0_SI1.BDR_NUM[11].TBLENR.R /* Tx BDR 11 length register */
#define NETC__ENETC0_SI1_TB11IER NETC__ENETC0_SI1.BDR_NUM[11].TBIER.R /* Tx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI1_TB11IDR NETC__ENETC0_SI1.BDR_NUM[11].TBIDR.R /* Tx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI1_TB11ICR0 NETC__ENETC0_SI1.BDR_NUM[11].TBICR0.R /* Tx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB11ICR1 NETC__ENETC0_SI1.BDR_NUM[11].TBICR1.R /* Tx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB11MR NETC__ENETC0_SI1.BDR_NUM[11].RBMR.R /* Rx BDR 11 mode register */
#define NETC__ENETC0_SI1_RB11SR NETC__ENETC0_SI1.BDR_NUM[11].RBSR.R /* Rx BDR 11 status register */
#define NETC__ENETC0_SI1_RB11BSR NETC__ENETC0_SI1.BDR_NUM[11].RBBSR.R /* Rx BDR 11 buffer size register */
#define NETC__ENETC0_SI1_RB11CIR NETC__ENETC0_SI1.BDR_NUM[11].RBCIR.R /* Rx BDR 11 consumer index register */
#define NETC__ENETC0_SI1_RB11BAR0 NETC__ENETC0_SI1.BDR_NUM[11].RBBAR0.R /* Rx BDR 11 base address register 0 */
#define NETC__ENETC0_SI1_RB11BAR1 NETC__ENETC0_SI1.BDR_NUM[11].RBBAR1.R /* Rx BDR 11 base address register 1 */
#define NETC__ENETC0_SI1_RB11PIR NETC__ENETC0_SI1.BDR_NUM[11].RBPIR.R /* Rx BDR 11 producer index register */
#define NETC__ENETC0_SI1_RB11LENR NETC__ENETC0_SI1.BDR_NUM[11].RBLENR.R /* Rx BDR 11 length register */
#define NETC__ENETC0_SI1_RB11DCR NETC__ENETC0_SI1.BDR_NUM[11].RBDCR.R /* Rx BDR 11 drop count register */
#define NETC__ENETC0_SI1_RB11IER NETC__ENETC0_SI1.BDR_NUM[11].RBIER.R /* Rx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI1_RB11IDR NETC__ENETC0_SI1.BDR_NUM[11].RBIDR.R /* Rx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI1_RB11ICR0 NETC__ENETC0_SI1.BDR_NUM[11].RBICR0.R /* Rx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB11ICR1 NETC__ENETC0_SI1.BDR_NUM[11].RBICR1.R /* Rx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB12MR NETC__ENETC0_SI1.BDR_NUM[12].TBMR.R /* Tx BDR 12 mode register */
#define NETC__ENETC0_SI1_TB12SR NETC__ENETC0_SI1.BDR_NUM[12].TBSR.R /* Tx BDR 12 status register */
#define NETC__ENETC0_SI1_TB12BAR0 NETC__ENETC0_SI1.BDR_NUM[12].TBBAR0.R /* Tx BDR 12 base address register 0 */
#define NETC__ENETC0_SI1_TB12BAR1 NETC__ENETC0_SI1.BDR_NUM[12].TBBAR1.R /* Tx BDR 12 base address register 1 */
#define NETC__ENETC0_SI1_TB12PIR NETC__ENETC0_SI1.BDR_NUM[12].TBPIR.R /* Tx BDR 12 producer index register */
#define NETC__ENETC0_SI1_TB12CIR NETC__ENETC0_SI1.BDR_NUM[12].TBCIR.R /* Tx BDR 12 consumer index register */
#define NETC__ENETC0_SI1_TB12LENR NETC__ENETC0_SI1.BDR_NUM[12].TBLENR.R /* Tx BDR 12 length register */
#define NETC__ENETC0_SI1_TB12IER NETC__ENETC0_SI1.BDR_NUM[12].TBIER.R /* Tx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI1_TB12IDR NETC__ENETC0_SI1.BDR_NUM[12].TBIDR.R /* Tx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI1_TB12ICR0 NETC__ENETC0_SI1.BDR_NUM[12].TBICR0.R /* Tx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB12ICR1 NETC__ENETC0_SI1.BDR_NUM[12].TBICR1.R /* Tx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB12MR NETC__ENETC0_SI1.BDR_NUM[12].RBMR.R /* Rx BDR 12 mode register */
#define NETC__ENETC0_SI1_RB12SR NETC__ENETC0_SI1.BDR_NUM[12].RBSR.R /* Rx BDR 12 status register */
#define NETC__ENETC0_SI1_RB12BSR NETC__ENETC0_SI1.BDR_NUM[12].RBBSR.R /* Rx BDR 12 buffer size register */
#define NETC__ENETC0_SI1_RB12CIR NETC__ENETC0_SI1.BDR_NUM[12].RBCIR.R /* Rx BDR 12 consumer index register */
#define NETC__ENETC0_SI1_RB12BAR0 NETC__ENETC0_SI1.BDR_NUM[12].RBBAR0.R /* Rx BDR 12 base address register 0 */
#define NETC__ENETC0_SI1_RB12BAR1 NETC__ENETC0_SI1.BDR_NUM[12].RBBAR1.R /* Rx BDR 12 base address register 1 */
#define NETC__ENETC0_SI1_RB12PIR NETC__ENETC0_SI1.BDR_NUM[12].RBPIR.R /* Rx BDR 12 producer index register */
#define NETC__ENETC0_SI1_RB12LENR NETC__ENETC0_SI1.BDR_NUM[12].RBLENR.R /* Rx BDR 12 length register */
#define NETC__ENETC0_SI1_RB12DCR NETC__ENETC0_SI1.BDR_NUM[12].RBDCR.R /* Rx BDR 12 drop count register */
#define NETC__ENETC0_SI1_RB12IER NETC__ENETC0_SI1.BDR_NUM[12].RBIER.R /* Rx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI1_RB12IDR NETC__ENETC0_SI1.BDR_NUM[12].RBIDR.R /* Rx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI1_RB12ICR0 NETC__ENETC0_SI1.BDR_NUM[12].RBICR0.R /* Rx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB12ICR1 NETC__ENETC0_SI1.BDR_NUM[12].RBICR1.R /* Rx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB13MR NETC__ENETC0_SI1.BDR_NUM[13].TBMR.R /* Tx BDR 13 mode register */
#define NETC__ENETC0_SI1_TB13SR NETC__ENETC0_SI1.BDR_NUM[13].TBSR.R /* Tx BDR 13 status register */
#define NETC__ENETC0_SI1_TB13BAR0 NETC__ENETC0_SI1.BDR_NUM[13].TBBAR0.R /* Tx BDR 13 base address register 0 */
#define NETC__ENETC0_SI1_TB13BAR1 NETC__ENETC0_SI1.BDR_NUM[13].TBBAR1.R /* Tx BDR 13 base address register 1 */
#define NETC__ENETC0_SI1_TB13PIR NETC__ENETC0_SI1.BDR_NUM[13].TBPIR.R /* Tx BDR 13 producer index register */
#define NETC__ENETC0_SI1_TB13CIR NETC__ENETC0_SI1.BDR_NUM[13].TBCIR.R /* Tx BDR 13 consumer index register */
#define NETC__ENETC0_SI1_TB13LENR NETC__ENETC0_SI1.BDR_NUM[13].TBLENR.R /* Tx BDR 13 length register */
#define NETC__ENETC0_SI1_TB13IER NETC__ENETC0_SI1.BDR_NUM[13].TBIER.R /* Tx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI1_TB13IDR NETC__ENETC0_SI1.BDR_NUM[13].TBIDR.R /* Tx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI1_TB13ICR0 NETC__ENETC0_SI1.BDR_NUM[13].TBICR0.R /* Tx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB13ICR1 NETC__ENETC0_SI1.BDR_NUM[13].TBICR1.R /* Tx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB13MR NETC__ENETC0_SI1.BDR_NUM[13].RBMR.R /* Rx BDR 13 mode register */
#define NETC__ENETC0_SI1_RB13SR NETC__ENETC0_SI1.BDR_NUM[13].RBSR.R /* Rx BDR 13 status register */
#define NETC__ENETC0_SI1_RB13BSR NETC__ENETC0_SI1.BDR_NUM[13].RBBSR.R /* Rx BDR 13 buffer size register */
#define NETC__ENETC0_SI1_RB13CIR NETC__ENETC0_SI1.BDR_NUM[13].RBCIR.R /* Rx BDR 13 consumer index register */
#define NETC__ENETC0_SI1_RB13BAR0 NETC__ENETC0_SI1.BDR_NUM[13].RBBAR0.R /* Rx BDR 13 base address register 0 */
#define NETC__ENETC0_SI1_RB13BAR1 NETC__ENETC0_SI1.BDR_NUM[13].RBBAR1.R /* Rx BDR 13 base address register 1 */
#define NETC__ENETC0_SI1_RB13PIR NETC__ENETC0_SI1.BDR_NUM[13].RBPIR.R /* Rx BDR 13 producer index register */
#define NETC__ENETC0_SI1_RB13LENR NETC__ENETC0_SI1.BDR_NUM[13].RBLENR.R /* Rx BDR 13 length register */
#define NETC__ENETC0_SI1_RB13DCR NETC__ENETC0_SI1.BDR_NUM[13].RBDCR.R /* Rx BDR 13 drop count register */
#define NETC__ENETC0_SI1_RB13IER NETC__ENETC0_SI1.BDR_NUM[13].RBIER.R /* Rx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI1_RB13IDR NETC__ENETC0_SI1.BDR_NUM[13].RBIDR.R /* Rx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI1_RB13ICR0 NETC__ENETC0_SI1.BDR_NUM[13].RBICR0.R /* Rx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB13ICR1 NETC__ENETC0_SI1.BDR_NUM[13].RBICR1.R /* Rx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB14MR NETC__ENETC0_SI1.BDR_NUM[14].TBMR.R /* Tx BDR 14 mode register */
#define NETC__ENETC0_SI1_TB14SR NETC__ENETC0_SI1.BDR_NUM[14].TBSR.R /* Tx BDR 14 status register */
#define NETC__ENETC0_SI1_TB14BAR0 NETC__ENETC0_SI1.BDR_NUM[14].TBBAR0.R /* Tx BDR 14 base address register 0 */
#define NETC__ENETC0_SI1_TB14BAR1 NETC__ENETC0_SI1.BDR_NUM[14].TBBAR1.R /* Tx BDR 14 base address register 1 */
#define NETC__ENETC0_SI1_TB14PIR NETC__ENETC0_SI1.BDR_NUM[14].TBPIR.R /* Tx BDR 14 producer index register */
#define NETC__ENETC0_SI1_TB14CIR NETC__ENETC0_SI1.BDR_NUM[14].TBCIR.R /* Tx BDR 14 consumer index register */
#define NETC__ENETC0_SI1_TB14LENR NETC__ENETC0_SI1.BDR_NUM[14].TBLENR.R /* Tx BDR 14 length register */
#define NETC__ENETC0_SI1_TB14IER NETC__ENETC0_SI1.BDR_NUM[14].TBIER.R /* Tx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI1_TB14IDR NETC__ENETC0_SI1.BDR_NUM[14].TBIDR.R /* Tx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI1_TB14ICR0 NETC__ENETC0_SI1.BDR_NUM[14].TBICR0.R /* Tx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB14ICR1 NETC__ENETC0_SI1.BDR_NUM[14].TBICR1.R /* Tx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB14MR NETC__ENETC0_SI1.BDR_NUM[14].RBMR.R /* Rx BDR 14 mode register */
#define NETC__ENETC0_SI1_RB14SR NETC__ENETC0_SI1.BDR_NUM[14].RBSR.R /* Rx BDR 14 status register */
#define NETC__ENETC0_SI1_RB14BSR NETC__ENETC0_SI1.BDR_NUM[14].RBBSR.R /* Rx BDR 14 buffer size register */
#define NETC__ENETC0_SI1_RB14CIR NETC__ENETC0_SI1.BDR_NUM[14].RBCIR.R /* Rx BDR 14 consumer index register */
#define NETC__ENETC0_SI1_RB14BAR0 NETC__ENETC0_SI1.BDR_NUM[14].RBBAR0.R /* Rx BDR 14 base address register 0 */
#define NETC__ENETC0_SI1_RB14BAR1 NETC__ENETC0_SI1.BDR_NUM[14].RBBAR1.R /* Rx BDR 14 base address register 1 */
#define NETC__ENETC0_SI1_RB14PIR NETC__ENETC0_SI1.BDR_NUM[14].RBPIR.R /* Rx BDR 14 producer index register */
#define NETC__ENETC0_SI1_RB14LENR NETC__ENETC0_SI1.BDR_NUM[14].RBLENR.R /* Rx BDR 14 length register */
#define NETC__ENETC0_SI1_RB14DCR NETC__ENETC0_SI1.BDR_NUM[14].RBDCR.R /* Rx BDR 14 drop count register */
#define NETC__ENETC0_SI1_RB14IER NETC__ENETC0_SI1.BDR_NUM[14].RBIER.R /* Rx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI1_RB14IDR NETC__ENETC0_SI1.BDR_NUM[14].RBIDR.R /* Rx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI1_RB14ICR0 NETC__ENETC0_SI1.BDR_NUM[14].RBICR0.R /* Rx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB14ICR1 NETC__ENETC0_SI1.BDR_NUM[14].RBICR1.R /* Rx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB15MR NETC__ENETC0_SI1.BDR_NUM[15].TBMR.R /* Tx BDR 15 mode register */
#define NETC__ENETC0_SI1_TB15SR NETC__ENETC0_SI1.BDR_NUM[15].TBSR.R /* Tx BDR 15 status register */
#define NETC__ENETC0_SI1_TB15BAR0 NETC__ENETC0_SI1.BDR_NUM[15].TBBAR0.R /* Tx BDR 15 base address register 0 */
#define NETC__ENETC0_SI1_TB15BAR1 NETC__ENETC0_SI1.BDR_NUM[15].TBBAR1.R /* Tx BDR 15 base address register 1 */
#define NETC__ENETC0_SI1_TB15PIR NETC__ENETC0_SI1.BDR_NUM[15].TBPIR.R /* Tx BDR 15 producer index register */
#define NETC__ENETC0_SI1_TB15CIR NETC__ENETC0_SI1.BDR_NUM[15].TBCIR.R /* Tx BDR 15 consumer index register */
#define NETC__ENETC0_SI1_TB15LENR NETC__ENETC0_SI1.BDR_NUM[15].TBLENR.R /* Tx BDR 15 length register */
#define NETC__ENETC0_SI1_TB15IER NETC__ENETC0_SI1.BDR_NUM[15].TBIER.R /* Tx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI1_TB15IDR NETC__ENETC0_SI1.BDR_NUM[15].TBIDR.R /* Tx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI1_TB15ICR0 NETC__ENETC0_SI1.BDR_NUM[15].TBICR0.R /* Tx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB15ICR1 NETC__ENETC0_SI1.BDR_NUM[15].TBICR1.R /* Tx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB15MR NETC__ENETC0_SI1.BDR_NUM[15].RBMR.R /* Rx BDR 15 mode register */
#define NETC__ENETC0_SI1_RB15SR NETC__ENETC0_SI1.BDR_NUM[15].RBSR.R /* Rx BDR 15 status register */
#define NETC__ENETC0_SI1_RB15BSR NETC__ENETC0_SI1.BDR_NUM[15].RBBSR.R /* Rx BDR 15 buffer size register */
#define NETC__ENETC0_SI1_RB15CIR NETC__ENETC0_SI1.BDR_NUM[15].RBCIR.R /* Rx BDR 15 consumer index register */
#define NETC__ENETC0_SI1_RB15BAR0 NETC__ENETC0_SI1.BDR_NUM[15].RBBAR0.R /* Rx BDR 15 base address register 0 */
#define NETC__ENETC0_SI1_RB15BAR1 NETC__ENETC0_SI1.BDR_NUM[15].RBBAR1.R /* Rx BDR 15 base address register 1 */
#define NETC__ENETC0_SI1_RB15PIR NETC__ENETC0_SI1.BDR_NUM[15].RBPIR.R /* Rx BDR 15 producer index register */
#define NETC__ENETC0_SI1_RB15LENR NETC__ENETC0_SI1.BDR_NUM[15].RBLENR.R /* Rx BDR 15 length register */
#define NETC__ENETC0_SI1_RB15DCR NETC__ENETC0_SI1.BDR_NUM[15].RBDCR.R /* Rx BDR 15 drop count register */
#define NETC__ENETC0_SI1_RB15IER NETC__ENETC0_SI1.BDR_NUM[15].RBIER.R /* Rx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI1_RB15IDR NETC__ENETC0_SI1.BDR_NUM[15].RBIDR.R /* Rx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI1_RB15ICR0 NETC__ENETC0_SI1.BDR_NUM[15].RBICR0.R /* Rx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB15ICR1 NETC__ENETC0_SI1.BDR_NUM[15].RBICR1.R /* Rx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB16MR NETC__ENETC0_SI1.BDR_NUM[16].TBMR.R /* Tx BDR 16 mode register */
#define NETC__ENETC0_SI1_TB16SR NETC__ENETC0_SI1.BDR_NUM[16].TBSR.R /* Tx BDR 16 status register */
#define NETC__ENETC0_SI1_TB16BAR0 NETC__ENETC0_SI1.BDR_NUM[16].TBBAR0.R /* Tx BDR 16 base address register 0 */
#define NETC__ENETC0_SI1_TB16BAR1 NETC__ENETC0_SI1.BDR_NUM[16].TBBAR1.R /* Tx BDR 16 base address register 1 */
#define NETC__ENETC0_SI1_TB16PIR NETC__ENETC0_SI1.BDR_NUM[16].TBPIR.R /* Tx BDR 16 producer index register */
#define NETC__ENETC0_SI1_TB16CIR NETC__ENETC0_SI1.BDR_NUM[16].TBCIR.R /* Tx BDR 16 consumer index register */
#define NETC__ENETC0_SI1_TB16LENR NETC__ENETC0_SI1.BDR_NUM[16].TBLENR.R /* Tx BDR 16 length register */
#define NETC__ENETC0_SI1_TB16IER NETC__ENETC0_SI1.BDR_NUM[16].TBIER.R /* Tx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI1_TB16IDR NETC__ENETC0_SI1.BDR_NUM[16].TBIDR.R /* Tx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI1_TB16ICR0 NETC__ENETC0_SI1.BDR_NUM[16].TBICR0.R /* Tx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB16ICR1 NETC__ENETC0_SI1.BDR_NUM[16].TBICR1.R /* Tx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB16MR NETC__ENETC0_SI1.BDR_NUM[16].RBMR.R /* Rx BDR 16 mode register */
#define NETC__ENETC0_SI1_RB16SR NETC__ENETC0_SI1.BDR_NUM[16].RBSR.R /* Rx BDR 16 status register */
#define NETC__ENETC0_SI1_RB16BSR NETC__ENETC0_SI1.BDR_NUM[16].RBBSR.R /* Rx BDR 16 buffer size register */
#define NETC__ENETC0_SI1_RB16CIR NETC__ENETC0_SI1.BDR_NUM[16].RBCIR.R /* Rx BDR 16 consumer index register */
#define NETC__ENETC0_SI1_RB16BAR0 NETC__ENETC0_SI1.BDR_NUM[16].RBBAR0.R /* Rx BDR 16 base address register 0 */
#define NETC__ENETC0_SI1_RB16BAR1 NETC__ENETC0_SI1.BDR_NUM[16].RBBAR1.R /* Rx BDR 16 base address register 1 */
#define NETC__ENETC0_SI1_RB16PIR NETC__ENETC0_SI1.BDR_NUM[16].RBPIR.R /* Rx BDR 16 producer index register */
#define NETC__ENETC0_SI1_RB16LENR NETC__ENETC0_SI1.BDR_NUM[16].RBLENR.R /* Rx BDR 16 length register */
#define NETC__ENETC0_SI1_RB16DCR NETC__ENETC0_SI1.BDR_NUM[16].RBDCR.R /* Rx BDR 16 drop count register */
#define NETC__ENETC0_SI1_RB16IER NETC__ENETC0_SI1.BDR_NUM[16].RBIER.R /* Rx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI1_RB16IDR NETC__ENETC0_SI1.BDR_NUM[16].RBIDR.R /* Rx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI1_RB16ICR0 NETC__ENETC0_SI1.BDR_NUM[16].RBICR0.R /* Rx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB16ICR1 NETC__ENETC0_SI1.BDR_NUM[16].RBICR1.R /* Rx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_TB17MR NETC__ENETC0_SI1.BDR_NUM[17].TBMR.R /* Tx BDR 17 mode register */
#define NETC__ENETC0_SI1_TB17SR NETC__ENETC0_SI1.BDR_NUM[17].TBSR.R /* Tx BDR 17 status register */
#define NETC__ENETC0_SI1_TB17BAR0 NETC__ENETC0_SI1.BDR_NUM[17].TBBAR0.R /* Tx BDR 17 base address register 0 */
#define NETC__ENETC0_SI1_TB17BAR1 NETC__ENETC0_SI1.BDR_NUM[17].TBBAR1.R /* Tx BDR 17 base address register 1 */
#define NETC__ENETC0_SI1_TB17PIR NETC__ENETC0_SI1.BDR_NUM[17].TBPIR.R /* Tx BDR 17 producer index register */
#define NETC__ENETC0_SI1_TB17CIR NETC__ENETC0_SI1.BDR_NUM[17].TBCIR.R /* Tx BDR 17 consumer index register */
#define NETC__ENETC0_SI1_TB17LENR NETC__ENETC0_SI1.BDR_NUM[17].TBLENR.R /* Tx BDR 17 length register */
#define NETC__ENETC0_SI1_TB17IER NETC__ENETC0_SI1.BDR_NUM[17].TBIER.R /* Tx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI1_TB17IDR NETC__ENETC0_SI1.BDR_NUM[17].TBIDR.R /* Tx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI1_TB17ICR0 NETC__ENETC0_SI1.BDR_NUM[17].TBICR0.R /* Tx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_TB17ICR1 NETC__ENETC0_SI1.BDR_NUM[17].TBICR1.R /* Tx BDR 17 interrupt coalescing register 1 */
#define NETC__ENETC0_SI1_RB17MR NETC__ENETC0_SI1.BDR_NUM[17].RBMR.R /* Rx BDR 17 mode register */
#define NETC__ENETC0_SI1_RB17SR NETC__ENETC0_SI1.BDR_NUM[17].RBSR.R /* Rx BDR 17 status register */
#define NETC__ENETC0_SI1_RB17BSR NETC__ENETC0_SI1.BDR_NUM[17].RBBSR.R /* Rx BDR 17 buffer size register */
#define NETC__ENETC0_SI1_RB17CIR NETC__ENETC0_SI1.BDR_NUM[17].RBCIR.R /* Rx BDR 17 consumer index register */
#define NETC__ENETC0_SI1_RB17BAR0 NETC__ENETC0_SI1.BDR_NUM[17].RBBAR0.R /* Rx BDR 17 base address register 0 */
#define NETC__ENETC0_SI1_RB17BAR1 NETC__ENETC0_SI1.BDR_NUM[17].RBBAR1.R /* Rx BDR 17 base address register 1 */
#define NETC__ENETC0_SI1_RB17PIR NETC__ENETC0_SI1.BDR_NUM[17].RBPIR.R /* Rx BDR 17 producer index register */
#define NETC__ENETC0_SI1_RB17LENR NETC__ENETC0_SI1.BDR_NUM[17].RBLENR.R /* Rx BDR 17 length register */
#define NETC__ENETC0_SI1_RB17DCR NETC__ENETC0_SI1.BDR_NUM[17].RBDCR.R /* Rx BDR 17 drop count register */
#define NETC__ENETC0_SI1_RB17IER NETC__ENETC0_SI1.BDR_NUM[17].RBIER.R /* Rx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI1_RB17IDR NETC__ENETC0_SI1.BDR_NUM[17].RBIDR.R /* Rx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI1_RB17ICR0 NETC__ENETC0_SI1.BDR_NUM[17].RBICR0.R /* Rx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI1_RB17ICR1 NETC__ENETC0_SI1.BDR_NUM[17].RBICR1.R /* Rx BDR 17 interrupt coalescing register 1 */

/* NETC_F3_SI2 */
#define NETC__ENETC0_SI2_SIMR NETC__ENETC0_SI2.SIMR.R      /* Station interface mode register */
#define NETC__ENETC0_SI2_SISR NETC__ENETC0_SI2.SISR.R      /* Station interface status register */
#define NETC__ENETC0_SI2_SICTR0 NETC__ENETC0_SI2.SICTR0.R  /* Station interface current time register 0 */
#define NETC__ENETC0_SI2_SICTR1 NETC__ENETC0_SI2.SICTR1.R  /* Station interface current time register 1 */
#define NETC__ENETC0_SI2_SIPCAPR0 NETC__ENETC0_SI2.SIPCAPR0.R /* Station interface port capability register 0 */
#define NETC__ENETC0_SI2_SIPCAPR1 NETC__ENETC0_SI2.SIPCAPR1.R /* Station interface port capability register 1 */
#define NETC__ENETC0_SI2_SIRBGCR NETC__ENETC0_SI2.SIRBGCR.R /* Station interface receive BDR group control register */
#define NETC__ENETC0_SI2_SIBCAR NETC__ENETC0_SI2.SIBCAR.R  /* Station interface buffer cache attribute register */
#define NETC__ENETC0_SI2_SICCAR NETC__ENETC0_SI2.SICCAR.R  /* Station interface command cache attribute register */
#define NETC__ENETC0_SI2_SIPMAR0 NETC__ENETC0_SI2.SIPMAR0.R /* Station interface primary MAC address register 0 */
#define NETC__ENETC0_SI2_SIPMAR1 NETC__ENETC0_SI2.SIPMAR1.R /* Station interface primary MAC address register 1 */
#define NETC__ENETC0_SI2_SICVLANR1 NETC__ENETC0_SI2.SICVLANR1.R /* Station interface custom VLAN register 1 */
#define NETC__ENETC0_SI2_SICVLANR2 NETC__ENETC0_SI2.SICVLANR2.R /* Station interface custom VLAN register 2 */
#define NETC__ENETC0_SI2_SIVLANIPVMR0 NETC__ENETC0_SI2.SIVLANIPVMR0.R /* Station interface VLAN to IPV mapping register 0 */
#define NETC__ENETC0_SI2_SIVLANIPVMR1 NETC__ENETC0_SI2.SIVLANIPVMR1.R /* Station interface VLAN to IPV mapping register 1 */
#define NETC__ENETC0_SI2_SIIPVBDRMR0 NETC__ENETC0_SI2.SIIPVBDRMR0.R /* Station interface IPV to ring mapping register */
#define NETC__ENETC0_SI2_VSIMSGSR NETC__ENETC0_SI2.VSI_A.VSIMSGSR.R /* Virtual station interface message send register */
#define NETC__ENETC0_SI2_VSIMSGRR NETC__ENETC0_SI2.VSI_A.VSIMSGRR.R /* Virtual station interface message receive register */
#define NETC__ENETC0_SI2_VSIMSGSNDAR0 NETC__ENETC0_SI2.VSI_A.VSIMSGSNDAR0.R /* Virtual station interface message send register 0 */
#define NETC__ENETC0_SI2_VSIMSGSNDAR1 NETC__ENETC0_SI2.VSI_A.VSIMSGSNDAR1.R /* Virtual station interface message send address register 1 */
#define NETC__ENETC0_SI2_SIROCT0 NETC__ENETC0_SI2.SIROCT0.R /* Station interface receive octets counter (ifInOctets) 0 */
#define NETC__ENETC0_SI2_SIROCT1 NETC__ENETC0_SI2.SIROCT1.R /* Station interface receive octets counter (ifInOctets) 1 */
#define NETC__ENETC0_SI2_SIRFRM0 NETC__ENETC0_SI2.SIRFRM0.R /* Station interface receive frame counter (aFrameReceivedOK) 0 */
#define NETC__ENETC0_SI2_SIRFRM1 NETC__ENETC0_SI2.SIRFRM1.R /* Station interface receive frame counter (aFrameReceivedOK) 1 */
#define NETC__ENETC0_SI2_SIRUCA0 NETC__ENETC0_SI2.SIRUCA0.R /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
#define NETC__ENETC0_SI2_SIRUCA1 NETC__ENETC0_SI2.SIRUCA1.R /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
#define NETC__ENETC0_SI2_SIRMCA0 NETC__ENETC0_SI2.SIRMCA0.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
#define NETC__ENETC0_SI2_SIRMCA1 NETC__ENETC0_SI2.SIRMCA1.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
#define NETC__ENETC0_SI2_SITOCT0 NETC__ENETC0_SI2.SITOCT0.R /* Station interface transmit octets counter (ifOutOctets) 0 */
#define NETC__ENETC0_SI2_SITOCT1 NETC__ENETC0_SI2.SITOCT1.R /* Station interface transmit octets counter (ifOutOctets) 1 */
#define NETC__ENETC0_SI2_SITFRM0 NETC__ENETC0_SI2.SITFRM0.R /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
#define NETC__ENETC0_SI2_SITFRM1 NETC__ENETC0_SI2.SITFRM1.R /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
#define NETC__ENETC0_SI2_SITUCA0 NETC__ENETC0_SI2.SITUCA0.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
#define NETC__ENETC0_SI2_SITUCA1 NETC__ENETC0_SI2.SITUCA1.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
#define NETC__ENETC0_SI2_SITMCA0 NETC__ENETC0_SI2.SITMCA0.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
#define NETC__ENETC0_SI2_SITMCA1 NETC__ENETC0_SI2.SITMCA1.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
#define NETC__ENETC0_SI2_SIBLPR0 NETC__ENETC0_SI2.SIBLPR[0].R /* Station interface boot loader parameter register 0 */
#define NETC__ENETC0_SI2_SIBLPR1 NETC__ENETC0_SI2.SIBLPR[1].R /* Station interface boot loader parameter register 1 */
#define NETC__ENETC0_SI2_SICBDRMR NETC__ENETC0_SI2.SICBDRMR.R /* Station interface command BDR mode register */
#define NETC__ENETC0_SI2_SICBDRSR NETC__ENETC0_SI2.SICBDRSR.R /* Station interface command BDR status register */
#define NETC__ENETC0_SI2_SICBDRBAR0 NETC__ENETC0_SI2.SICBDRBAR0.R /* Station interface command BDR base address register 0 */
#define NETC__ENETC0_SI2_SICBDRBAR1 NETC__ENETC0_SI2.SICBDRBAR1.R /* Station interface command BDR base address register 1 */
#define NETC__ENETC0_SI2_SICBDRPIR NETC__ENETC0_SI2.SICBDRPIR.R /* Station interface command BDR producer index register */
#define NETC__ENETC0_SI2_SICBDRCIR NETC__ENETC0_SI2.SICBDRCIR.R /* Station interface command BDR consumer index register */
#define NETC__ENETC0_SI2_SICBDRLENR NETC__ENETC0_SI2.SICBDRLENR.R /* Station interface command BDR length register */
#define NETC__ENETC0_SI2_SICBDRIER NETC__ENETC0_SI2.SICBDRIER.R /* Station interface command BDR interrupt enable register */
#define NETC__ENETC0_SI2_SICBDRIDR NETC__ENETC0_SI2.SICBDRIDR.R /* Station interface command BDR interrupt detect register */
#define NETC__ENETC0_SI2_SICAPR0 NETC__ENETC0_SI2.SICAPR0.R /* Station interface capability register 0 */
#define NETC__ENETC0_SI2_SICAPR1 NETC__ENETC0_SI2.SICAPR1.R /* Station interface capability register 1 */
#define NETC__ENETC0_SI2_SICAPR2 NETC__ENETC0_SI2.SICAPR2.R /* Station interface capability register 2 */
#define NETC__ENETC0_SI2_VSIIER NETC__ENETC0_SI2.VSI.VSIIER.R /* Virtual station interface interrupt enable register */
#define NETC__ENETC0_SI2_VSIIDR NETC__ENETC0_SI2.VSI.VSIIDR.R /* Virtual station interface interrupt detect register */
#define NETC__ENETC0_SI2_SITXIDR0 NETC__ENETC0_SI2.SITXIDR0.R /* Station interface transmit interrupt detect register 0 */
#define NETC__ENETC0_SI2_SITXIDR1 NETC__ENETC0_SI2.SITXIDR1.R /* Station interface transmit interrupt detect register 1 */
#define NETC__ENETC0_SI2_SIRXIDR0 NETC__ENETC0_SI2.SIRXIDR0.R /* Station interface receive interrupt detect register 0 */
#define NETC__ENETC0_SI2_SIRXIDR1 NETC__ENETC0_SI2.SIRXIDR1.R /* Station interface receive interrupt detect register 1 */
#define NETC__ENETC0_SI2_SIMSIVR NETC__ENETC0_SI2.SIMSIVR.R /* Station interface MSI-X vector register */
#define NETC__ENETC0_SI2_SICMSIVR NETC__ENETC0_SI2.SICMSIVR.R /* Station interface command MSI-X vector register */
#define NETC__ENETC0_SI2_SITMRIER NETC__ENETC0_SI2.SITMRIER.R /* Station interface timer interrupt enable register */
#define NETC__ENETC0_SI2_SITMRIDR NETC__ENETC0_SI2.SITMRIDR.R /* Station interface timer interrupt detect register */
#define NETC__ENETC0_SI2_SITMRMSIVR NETC__ENETC0_SI2.SITMRMSIVR.R /* Station interface timer MSI-X vector register */
#define NETC__ENETC0_SI2_SIMSITR0VR NETC__ENETC0_SI2.SIMSITRVR[0].R /* Station interface MSI-X transmit ring 0 vector register */
#define NETC__ENETC0_SI2_SIMSITR1VR NETC__ENETC0_SI2.SIMSITRVR[1].R /* Station interface MSI-X transmit ring 1 vector register */
#define NETC__ENETC0_SI2_SIMSITR2VR NETC__ENETC0_SI2.SIMSITRVR[2].R /* Station interface MSI-X transmit ring 2 vector register */
#define NETC__ENETC0_SI2_SIMSITR3VR NETC__ENETC0_SI2.SIMSITRVR[3].R /* Station interface MSI-X transmit ring 3 vector register */
#define NETC__ENETC0_SI2_SIMSITR4VR NETC__ENETC0_SI2.SIMSITRVR[4].R /* Station interface MSI-X transmit ring 4 vector register */
#define NETC__ENETC0_SI2_SIMSITR5VR NETC__ENETC0_SI2.SIMSITRVR[5].R /* Station interface MSI-X transmit ring 5 vector register */
#define NETC__ENETC0_SI2_SIMSITR6VR NETC__ENETC0_SI2.SIMSITRVR[6].R /* Station interface MSI-X transmit ring 6 vector register */
#define NETC__ENETC0_SI2_SIMSITR7VR NETC__ENETC0_SI2.SIMSITRVR[7].R /* Station interface MSI-X transmit ring 7 vector register */
#define NETC__ENETC0_SI2_SIMSITR8VR NETC__ENETC0_SI2.SIMSITRVR[8].R /* Station interface MSI-X transmit ring 8 vector register */
#define NETC__ENETC0_SI2_SIMSITR9VR NETC__ENETC0_SI2.SIMSITRVR[9].R /* Station interface MSI-X transmit ring 9 vector register */
#define NETC__ENETC0_SI2_SIMSITR10VR NETC__ENETC0_SI2.SIMSITRVR[10].R /* Station interface MSI-X transmit ring 10 vector register */
#define NETC__ENETC0_SI2_SIMSITR11VR NETC__ENETC0_SI2.SIMSITRVR[11].R /* Station interface MSI-X transmit ring 11 vector register */
#define NETC__ENETC0_SI2_SIMSITR12VR NETC__ENETC0_SI2.SIMSITRVR[12].R /* Station interface MSI-X transmit ring 12 vector register */
#define NETC__ENETC0_SI2_SIMSITR13VR NETC__ENETC0_SI2.SIMSITRVR[13].R /* Station interface MSI-X transmit ring 13 vector register */
#define NETC__ENETC0_SI2_SIMSITR14VR NETC__ENETC0_SI2.SIMSITRVR[14].R /* Station interface MSI-X transmit ring 14 vector register */
#define NETC__ENETC0_SI2_SIMSITR15VR NETC__ENETC0_SI2.SIMSITRVR[15].R /* Station interface MSI-X transmit ring 15 vector register */
#define NETC__ENETC0_SI2_SIMSITR16VR NETC__ENETC0_SI2.SIMSITRVR[16].R /* Station interface MSI-X transmit ring 16 vector register */
#define NETC__ENETC0_SI2_SIMSITR17VR NETC__ENETC0_SI2.SIMSITRVR[17].R /* Station interface MSI-X transmit ring 17 vector register */
#define NETC__ENETC0_SI2_SIMSIRR0VR NETC__ENETC0_SI2.SIMSIRRVR[0].R /* Station interface MSI-X receive ring 0 vector register */
#define NETC__ENETC0_SI2_SIMSIRR1VR NETC__ENETC0_SI2.SIMSIRRVR[1].R /* Station interface MSI-X receive ring 1 vector register */
#define NETC__ENETC0_SI2_SIMSIRR2VR NETC__ENETC0_SI2.SIMSIRRVR[2].R /* Station interface MSI-X receive ring 2 vector register */
#define NETC__ENETC0_SI2_SIMSIRR3VR NETC__ENETC0_SI2.SIMSIRRVR[3].R /* Station interface MSI-X receive ring 3 vector register */
#define NETC__ENETC0_SI2_SIMSIRR4VR NETC__ENETC0_SI2.SIMSIRRVR[4].R /* Station interface MSI-X receive ring 4 vector register */
#define NETC__ENETC0_SI2_SIMSIRR5VR NETC__ENETC0_SI2.SIMSIRRVR[5].R /* Station interface MSI-X receive ring 5 vector register */
#define NETC__ENETC0_SI2_SIMSIRR6VR NETC__ENETC0_SI2.SIMSIRRVR[6].R /* Station interface MSI-X receive ring 6 vector register */
#define NETC__ENETC0_SI2_SIMSIRR7VR NETC__ENETC0_SI2.SIMSIRRVR[7].R /* Station interface MSI-X receive ring 7 vector register */
#define NETC__ENETC0_SI2_SIMSIRR8VR NETC__ENETC0_SI2.SIMSIRRVR[8].R /* Station interface MSI-X receive ring 8 vector register */
#define NETC__ENETC0_SI2_SIMSIRR9VR NETC__ENETC0_SI2.SIMSIRRVR[9].R /* Station interface MSI-X receive ring 9 vector register */
#define NETC__ENETC0_SI2_SIMSIRR10VR NETC__ENETC0_SI2.SIMSIRRVR[10].R /* Station interface MSI-X receive ring 10 vector register */
#define NETC__ENETC0_SI2_SIMSIRR11VR NETC__ENETC0_SI2.SIMSIRRVR[11].R /* Station interface MSI-X receive ring 11 vector register */
#define NETC__ENETC0_SI2_SIMSIRR12VR NETC__ENETC0_SI2.SIMSIRRVR[12].R /* Station interface MSI-X receive ring 12 vector register */
#define NETC__ENETC0_SI2_SIMSIRR13VR NETC__ENETC0_SI2.SIMSIRRVR[13].R /* Station interface MSI-X receive ring 13 vector register */
#define NETC__ENETC0_SI2_SIMSIRR14VR NETC__ENETC0_SI2.SIMSIRRVR[14].R /* Station interface MSI-X receive ring 14 vector register */
#define NETC__ENETC0_SI2_SIMSIRR15VR NETC__ENETC0_SI2.SIMSIRRVR[15].R /* Station interface MSI-X receive ring 15 vector register */
#define NETC__ENETC0_SI2_SIMSIRR16VR NETC__ENETC0_SI2.SIMSIRRVR[16].R /* Station interface MSI-X receive ring 16 vector register */
#define NETC__ENETC0_SI2_SIMSIRR17VR NETC__ENETC0_SI2.SIMSIRRVR[17].R /* Station interface MSI-X receive ring 17 vector register */
#define NETC__ENETC0_SI2_SICMECR NETC__ENETC0_SI2.SICMECR.R /* Station interface correctable memory error configuration register */
#define NETC__ENETC0_SI2_SICMESR NETC__ENETC0_SI2.SICMESR.R /* Station interface correctable memory error status register */
#define NETC__ENETC0_SI2_SICMECTR NETC__ENETC0_SI2.SICMECTR.R /* Station interface correctable memory error count register */
#define NETC__ENETC0_SI2_SIUPECR NETC__ENETC0_SI2.SIUPECR.R /* Station interface uncorrectable programming error configuration register */
#define NETC__ENETC0_SI2_SIUPESR NETC__ENETC0_SI2.SIUPESR.R /* Station interface uncorrectable programming error status register */
#define NETC__ENETC0_SI2_SIUPECTR NETC__ENETC0_SI2.SIUPECTR.R /* Station interface uncorrectable programming error count register */
#define NETC__ENETC0_SI2_SIUNSBECR NETC__ENETC0_SI2.SIUNSBECR.R /* Station interface uncorrectable non-fatal system bus error configuration register */
#define NETC__ENETC0_SI2_SIUNSBESR NETC__ENETC0_SI2.SIUNSBESR.R /* Station interface uncorrectable non-fatal system bus error status register */
#define NETC__ENETC0_SI2_SIUNSBECTR NETC__ENETC0_SI2.SIUNSBECTR.R /* Station interface uncorrectable non-fatal system bus error count register */
#define NETC__ENETC0_SI2_SIUFSBECR NETC__ENETC0_SI2.SIUFSBECR.R /* Station interface uncorrectable fatal system bus error configuration register */
#define NETC__ENETC0_SI2_SIUFSBESR NETC__ENETC0_SI2.SIUFSBESR.R /* Station interface uncorrectable fatal system bus error status register */
#define NETC__ENETC0_SI2_SIUNMECR NETC__ENETC0_SI2.SIUNMECR.R /* Station interface uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_SI2_SIUNMESR0 NETC__ENETC0_SI2.SIUNMESR0.R /* Station interface uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_SI2_SIUNMESR1 NETC__ENETC0_SI2.SIUNMESR1.R /* Station interface uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_SI2_SIUNMECTR NETC__ENETC0_SI2.SIUNMECTR.R /* Station interface uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_SI2_SIUFMECR NETC__ENETC0_SI2.SIUFMECR.R /* Station interface uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_SI2_SIUFMESR0 NETC__ENETC0_SI2.SIUFMESR0.R /* Station interface uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_SI2_SIUFMESR1 NETC__ENETC0_SI2.SIUFMESR1.R /* Station interface uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_SI2_SIUNIECR NETC__ENETC0_SI2.SIUNIECR.R /* Station interface uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_SI2_SIUNIESR NETC__ENETC0_SI2.SIUNIESR.R /* Station interface uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_SI2_SIUNIECTR NETC__ENETC0_SI2.SIUNIECTR.R /* Station interface uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_SI2_SIUFIECR NETC__ENETC0_SI2.SIUFIECR.R /* Station interface uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_SI2_SIUFIESR NETC__ENETC0_SI2.SIUFIESR.R /* Station interface uncorrectable fatal integrity error status register */
#define NETC__ENETC0_SI2_SIMAFTCAPR NETC__ENETC0_SI2.SIMAFTCAPR.R /* Station interface MAC address filter table capability register */
#define NETC__ENETC0_SI2_SIVFTCAPR NETC__ENETC0_SI2.SIVFTCAPR.R /* Station interface VLAN filter table capability register */
#define NETC__ENETC0_SI2_SIRFSCAPR NETC__ENETC0_SI2.SIRFSCAPR.R /* Station interface RFS capability register */
#define NETC__ENETC0_SI2_TB0MR NETC__ENETC0_SI2.BDR_NUM[0].TBMR.R /* Tx BDR 0 mode register */
#define NETC__ENETC0_SI2_TB0SR NETC__ENETC0_SI2.BDR_NUM[0].TBSR.R /* Tx BDR 0 status register */
#define NETC__ENETC0_SI2_TB0BAR0 NETC__ENETC0_SI2.BDR_NUM[0].TBBAR0.R /* Tx BDR 0 base address register 0 */
#define NETC__ENETC0_SI2_TB0BAR1 NETC__ENETC0_SI2.BDR_NUM[0].TBBAR1.R /* Tx BDR 0 base address register 1 */
#define NETC__ENETC0_SI2_TB0PIR NETC__ENETC0_SI2.BDR_NUM[0].TBPIR.R /* Tx BDR 0 producer index register */
#define NETC__ENETC0_SI2_TB0CIR NETC__ENETC0_SI2.BDR_NUM[0].TBCIR.R /* Tx BDR 0 consumer index register */
#define NETC__ENETC0_SI2_TB0LENR NETC__ENETC0_SI2.BDR_NUM[0].TBLENR.R /* Tx BDR 0 length register */
#define NETC__ENETC0_SI2_TB0IER NETC__ENETC0_SI2.BDR_NUM[0].TBIER.R /* Tx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI2_TB0IDR NETC__ENETC0_SI2.BDR_NUM[0].TBIDR.R /* Tx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI2_TB0ICR0 NETC__ENETC0_SI2.BDR_NUM[0].TBICR0.R /* Tx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB0ICR1 NETC__ENETC0_SI2.BDR_NUM[0].TBICR1.R /* Tx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB0MR NETC__ENETC0_SI2.BDR_NUM[0].RBMR.R /* Rx BDR 0 mode register */
#define NETC__ENETC0_SI2_RB0SR NETC__ENETC0_SI2.BDR_NUM[0].RBSR.R /* Rx BDR 0 status register */
#define NETC__ENETC0_SI2_RB0BSR NETC__ENETC0_SI2.BDR_NUM[0].RBBSR.R /* Rx BDR 0 buffer size register */
#define NETC__ENETC0_SI2_RB0CIR NETC__ENETC0_SI2.BDR_NUM[0].RBCIR.R /* Rx BDR 0 consumer index register */
#define NETC__ENETC0_SI2_RB0BAR0 NETC__ENETC0_SI2.BDR_NUM[0].RBBAR0.R /* Rx BDR 0 base address register 0 */
#define NETC__ENETC0_SI2_RB0BAR1 NETC__ENETC0_SI2.BDR_NUM[0].RBBAR1.R /* Rx BDR 0 base address register 1 */
#define NETC__ENETC0_SI2_RB0PIR NETC__ENETC0_SI2.BDR_NUM[0].RBPIR.R /* Rx BDR 0 producer index register */
#define NETC__ENETC0_SI2_RB0LENR NETC__ENETC0_SI2.BDR_NUM[0].RBLENR.R /* Rx BDR 0 length register */
#define NETC__ENETC0_SI2_RB0DCR NETC__ENETC0_SI2.BDR_NUM[0].RBDCR.R /* Rx BDR 0 drop count register */
#define NETC__ENETC0_SI2_RB0IER NETC__ENETC0_SI2.BDR_NUM[0].RBIER.R /* Rx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI2_RB0IDR NETC__ENETC0_SI2.BDR_NUM[0].RBIDR.R /* Rx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI2_RB0ICR0 NETC__ENETC0_SI2.BDR_NUM[0].RBICR0.R /* Rx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB0ICR1 NETC__ENETC0_SI2.BDR_NUM[0].RBICR1.R /* Rx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB1MR NETC__ENETC0_SI2.BDR_NUM[1].TBMR.R /* Tx BDR 1 mode register */
#define NETC__ENETC0_SI2_TB1SR NETC__ENETC0_SI2.BDR_NUM[1].TBSR.R /* Tx BDR 1 status register */
#define NETC__ENETC0_SI2_TB1BAR0 NETC__ENETC0_SI2.BDR_NUM[1].TBBAR0.R /* Tx BDR 1 base address register 0 */
#define NETC__ENETC0_SI2_TB1BAR1 NETC__ENETC0_SI2.BDR_NUM[1].TBBAR1.R /* Tx BDR 1 base address register 1 */
#define NETC__ENETC0_SI2_TB1PIR NETC__ENETC0_SI2.BDR_NUM[1].TBPIR.R /* Tx BDR 1 producer index register */
#define NETC__ENETC0_SI2_TB1CIR NETC__ENETC0_SI2.BDR_NUM[1].TBCIR.R /* Tx BDR 1 consumer index register */
#define NETC__ENETC0_SI2_TB1LENR NETC__ENETC0_SI2.BDR_NUM[1].TBLENR.R /* Tx BDR 1 length register */
#define NETC__ENETC0_SI2_TB1IER NETC__ENETC0_SI2.BDR_NUM[1].TBIER.R /* Tx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI2_TB1IDR NETC__ENETC0_SI2.BDR_NUM[1].TBIDR.R /* Tx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI2_TB1ICR0 NETC__ENETC0_SI2.BDR_NUM[1].TBICR0.R /* Tx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB1ICR1 NETC__ENETC0_SI2.BDR_NUM[1].TBICR1.R /* Tx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB1MR NETC__ENETC0_SI2.BDR_NUM[1].RBMR.R /* Rx BDR 1 mode register */
#define NETC__ENETC0_SI2_RB1SR NETC__ENETC0_SI2.BDR_NUM[1].RBSR.R /* Rx BDR 1 status register */
#define NETC__ENETC0_SI2_RB1BSR NETC__ENETC0_SI2.BDR_NUM[1].RBBSR.R /* Rx BDR 1 buffer size register */
#define NETC__ENETC0_SI2_RB1CIR NETC__ENETC0_SI2.BDR_NUM[1].RBCIR.R /* Rx BDR 1 consumer index register */
#define NETC__ENETC0_SI2_RB1BAR0 NETC__ENETC0_SI2.BDR_NUM[1].RBBAR0.R /* Rx BDR 1 base address register 0 */
#define NETC__ENETC0_SI2_RB1BAR1 NETC__ENETC0_SI2.BDR_NUM[1].RBBAR1.R /* Rx BDR 1 base address register 1 */
#define NETC__ENETC0_SI2_RB1PIR NETC__ENETC0_SI2.BDR_NUM[1].RBPIR.R /* Rx BDR 1 producer index register */
#define NETC__ENETC0_SI2_RB1LENR NETC__ENETC0_SI2.BDR_NUM[1].RBLENR.R /* Rx BDR 1 length register */
#define NETC__ENETC0_SI2_RB1DCR NETC__ENETC0_SI2.BDR_NUM[1].RBDCR.R /* Rx BDR 1 drop count register */
#define NETC__ENETC0_SI2_RB1IER NETC__ENETC0_SI2.BDR_NUM[1].RBIER.R /* Rx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI2_RB1IDR NETC__ENETC0_SI2.BDR_NUM[1].RBIDR.R /* Rx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI2_RB1ICR0 NETC__ENETC0_SI2.BDR_NUM[1].RBICR0.R /* Rx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB1ICR1 NETC__ENETC0_SI2.BDR_NUM[1].RBICR1.R /* Rx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB2MR NETC__ENETC0_SI2.BDR_NUM[2].TBMR.R /* Tx BDR 2 mode register */
#define NETC__ENETC0_SI2_TB2SR NETC__ENETC0_SI2.BDR_NUM[2].TBSR.R /* Tx BDR 2 status register */
#define NETC__ENETC0_SI2_TB2BAR0 NETC__ENETC0_SI2.BDR_NUM[2].TBBAR0.R /* Tx BDR 2 base address register 0 */
#define NETC__ENETC0_SI2_TB2BAR1 NETC__ENETC0_SI2.BDR_NUM[2].TBBAR1.R /* Tx BDR 2 base address register 1 */
#define NETC__ENETC0_SI2_TB2PIR NETC__ENETC0_SI2.BDR_NUM[2].TBPIR.R /* Tx BDR 2 producer index register */
#define NETC__ENETC0_SI2_TB2CIR NETC__ENETC0_SI2.BDR_NUM[2].TBCIR.R /* Tx BDR 2 consumer index register */
#define NETC__ENETC0_SI2_TB2LENR NETC__ENETC0_SI2.BDR_NUM[2].TBLENR.R /* Tx BDR 2 length register */
#define NETC__ENETC0_SI2_TB2IER NETC__ENETC0_SI2.BDR_NUM[2].TBIER.R /* Tx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI2_TB2IDR NETC__ENETC0_SI2.BDR_NUM[2].TBIDR.R /* Tx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI2_TB2ICR0 NETC__ENETC0_SI2.BDR_NUM[2].TBICR0.R /* Tx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB2ICR1 NETC__ENETC0_SI2.BDR_NUM[2].TBICR1.R /* Tx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB2MR NETC__ENETC0_SI2.BDR_NUM[2].RBMR.R /* Rx BDR 2 mode register */
#define NETC__ENETC0_SI2_RB2SR NETC__ENETC0_SI2.BDR_NUM[2].RBSR.R /* Rx BDR 2 status register */
#define NETC__ENETC0_SI2_RB2BSR NETC__ENETC0_SI2.BDR_NUM[2].RBBSR.R /* Rx BDR 2 buffer size register */
#define NETC__ENETC0_SI2_RB2CIR NETC__ENETC0_SI2.BDR_NUM[2].RBCIR.R /* Rx BDR 2 consumer index register */
#define NETC__ENETC0_SI2_RB2BAR0 NETC__ENETC0_SI2.BDR_NUM[2].RBBAR0.R /* Rx BDR 2 base address register 0 */
#define NETC__ENETC0_SI2_RB2BAR1 NETC__ENETC0_SI2.BDR_NUM[2].RBBAR1.R /* Rx BDR 2 base address register 1 */
#define NETC__ENETC0_SI2_RB2PIR NETC__ENETC0_SI2.BDR_NUM[2].RBPIR.R /* Rx BDR 2 producer index register */
#define NETC__ENETC0_SI2_RB2LENR NETC__ENETC0_SI2.BDR_NUM[2].RBLENR.R /* Rx BDR 2 length register */
#define NETC__ENETC0_SI2_RB2DCR NETC__ENETC0_SI2.BDR_NUM[2].RBDCR.R /* Rx BDR 2 drop count register */
#define NETC__ENETC0_SI2_RB2IER NETC__ENETC0_SI2.BDR_NUM[2].RBIER.R /* Rx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI2_RB2IDR NETC__ENETC0_SI2.BDR_NUM[2].RBIDR.R /* Rx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI2_RB2ICR0 NETC__ENETC0_SI2.BDR_NUM[2].RBICR0.R /* Rx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB2ICR1 NETC__ENETC0_SI2.BDR_NUM[2].RBICR1.R /* Rx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB3MR NETC__ENETC0_SI2.BDR_NUM[3].TBMR.R /* Tx BDR 3 mode register */
#define NETC__ENETC0_SI2_TB3SR NETC__ENETC0_SI2.BDR_NUM[3].TBSR.R /* Tx BDR 3 status register */
#define NETC__ENETC0_SI2_TB3BAR0 NETC__ENETC0_SI2.BDR_NUM[3].TBBAR0.R /* Tx BDR 3 base address register 0 */
#define NETC__ENETC0_SI2_TB3BAR1 NETC__ENETC0_SI2.BDR_NUM[3].TBBAR1.R /* Tx BDR 3 base address register 1 */
#define NETC__ENETC0_SI2_TB3PIR NETC__ENETC0_SI2.BDR_NUM[3].TBPIR.R /* Tx BDR 3 producer index register */
#define NETC__ENETC0_SI2_TB3CIR NETC__ENETC0_SI2.BDR_NUM[3].TBCIR.R /* Tx BDR 3 consumer index register */
#define NETC__ENETC0_SI2_TB3LENR NETC__ENETC0_SI2.BDR_NUM[3].TBLENR.R /* Tx BDR 3 length register */
#define NETC__ENETC0_SI2_TB3IER NETC__ENETC0_SI2.BDR_NUM[3].TBIER.R /* Tx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI2_TB3IDR NETC__ENETC0_SI2.BDR_NUM[3].TBIDR.R /* Tx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI2_TB3ICR0 NETC__ENETC0_SI2.BDR_NUM[3].TBICR0.R /* Tx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB3ICR1 NETC__ENETC0_SI2.BDR_NUM[3].TBICR1.R /* Tx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB3MR NETC__ENETC0_SI2.BDR_NUM[3].RBMR.R /* Rx BDR 3 mode register */
#define NETC__ENETC0_SI2_RB3SR NETC__ENETC0_SI2.BDR_NUM[3].RBSR.R /* Rx BDR 3 status register */
#define NETC__ENETC0_SI2_RB3BSR NETC__ENETC0_SI2.BDR_NUM[3].RBBSR.R /* Rx BDR 3 buffer size register */
#define NETC__ENETC0_SI2_RB3CIR NETC__ENETC0_SI2.BDR_NUM[3].RBCIR.R /* Rx BDR 3 consumer index register */
#define NETC__ENETC0_SI2_RB3BAR0 NETC__ENETC0_SI2.BDR_NUM[3].RBBAR0.R /* Rx BDR 3 base address register 0 */
#define NETC__ENETC0_SI2_RB3BAR1 NETC__ENETC0_SI2.BDR_NUM[3].RBBAR1.R /* Rx BDR 3 base address register 1 */
#define NETC__ENETC0_SI2_RB3PIR NETC__ENETC0_SI2.BDR_NUM[3].RBPIR.R /* Rx BDR 3 producer index register */
#define NETC__ENETC0_SI2_RB3LENR NETC__ENETC0_SI2.BDR_NUM[3].RBLENR.R /* Rx BDR 3 length register */
#define NETC__ENETC0_SI2_RB3DCR NETC__ENETC0_SI2.BDR_NUM[3].RBDCR.R /* Rx BDR 3 drop count register */
#define NETC__ENETC0_SI2_RB3IER NETC__ENETC0_SI2.BDR_NUM[3].RBIER.R /* Rx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI2_RB3IDR NETC__ENETC0_SI2.BDR_NUM[3].RBIDR.R /* Rx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI2_RB3ICR0 NETC__ENETC0_SI2.BDR_NUM[3].RBICR0.R /* Rx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB3ICR1 NETC__ENETC0_SI2.BDR_NUM[3].RBICR1.R /* Rx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB4MR NETC__ENETC0_SI2.BDR_NUM[4].TBMR.R /* Tx BDR 4 mode register */
#define NETC__ENETC0_SI2_TB4SR NETC__ENETC0_SI2.BDR_NUM[4].TBSR.R /* Tx BDR 4 status register */
#define NETC__ENETC0_SI2_TB4BAR0 NETC__ENETC0_SI2.BDR_NUM[4].TBBAR0.R /* Tx BDR 4 base address register 0 */
#define NETC__ENETC0_SI2_TB4BAR1 NETC__ENETC0_SI2.BDR_NUM[4].TBBAR1.R /* Tx BDR 4 base address register 1 */
#define NETC__ENETC0_SI2_TB4PIR NETC__ENETC0_SI2.BDR_NUM[4].TBPIR.R /* Tx BDR 4 producer index register */
#define NETC__ENETC0_SI2_TB4CIR NETC__ENETC0_SI2.BDR_NUM[4].TBCIR.R /* Tx BDR 4 consumer index register */
#define NETC__ENETC0_SI2_TB4LENR NETC__ENETC0_SI2.BDR_NUM[4].TBLENR.R /* Tx BDR 4 length register */
#define NETC__ENETC0_SI2_TB4IER NETC__ENETC0_SI2.BDR_NUM[4].TBIER.R /* Tx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI2_TB4IDR NETC__ENETC0_SI2.BDR_NUM[4].TBIDR.R /* Tx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI2_TB4ICR0 NETC__ENETC0_SI2.BDR_NUM[4].TBICR0.R /* Tx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB4ICR1 NETC__ENETC0_SI2.BDR_NUM[4].TBICR1.R /* Tx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB4MR NETC__ENETC0_SI2.BDR_NUM[4].RBMR.R /* Rx BDR 4 mode register */
#define NETC__ENETC0_SI2_RB4SR NETC__ENETC0_SI2.BDR_NUM[4].RBSR.R /* Rx BDR 4 status register */
#define NETC__ENETC0_SI2_RB4BSR NETC__ENETC0_SI2.BDR_NUM[4].RBBSR.R /* Rx BDR 4 buffer size register */
#define NETC__ENETC0_SI2_RB4CIR NETC__ENETC0_SI2.BDR_NUM[4].RBCIR.R /* Rx BDR 4 consumer index register */
#define NETC__ENETC0_SI2_RB4BAR0 NETC__ENETC0_SI2.BDR_NUM[4].RBBAR0.R /* Rx BDR 4 base address register 0 */
#define NETC__ENETC0_SI2_RB4BAR1 NETC__ENETC0_SI2.BDR_NUM[4].RBBAR1.R /* Rx BDR 4 base address register 1 */
#define NETC__ENETC0_SI2_RB4PIR NETC__ENETC0_SI2.BDR_NUM[4].RBPIR.R /* Rx BDR 4 producer index register */
#define NETC__ENETC0_SI2_RB4LENR NETC__ENETC0_SI2.BDR_NUM[4].RBLENR.R /* Rx BDR 4 length register */
#define NETC__ENETC0_SI2_RB4DCR NETC__ENETC0_SI2.BDR_NUM[4].RBDCR.R /* Rx BDR 4 drop count register */
#define NETC__ENETC0_SI2_RB4IER NETC__ENETC0_SI2.BDR_NUM[4].RBIER.R /* Rx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI2_RB4IDR NETC__ENETC0_SI2.BDR_NUM[4].RBIDR.R /* Rx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI2_RB4ICR0 NETC__ENETC0_SI2.BDR_NUM[4].RBICR0.R /* Rx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB4ICR1 NETC__ENETC0_SI2.BDR_NUM[4].RBICR1.R /* Rx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB5MR NETC__ENETC0_SI2.BDR_NUM[5].TBMR.R /* Tx BDR 5 mode register */
#define NETC__ENETC0_SI2_TB5SR NETC__ENETC0_SI2.BDR_NUM[5].TBSR.R /* Tx BDR 5 status register */
#define NETC__ENETC0_SI2_TB5BAR0 NETC__ENETC0_SI2.BDR_NUM[5].TBBAR0.R /* Tx BDR 5 base address register 0 */
#define NETC__ENETC0_SI2_TB5BAR1 NETC__ENETC0_SI2.BDR_NUM[5].TBBAR1.R /* Tx BDR 5 base address register 1 */
#define NETC__ENETC0_SI2_TB5PIR NETC__ENETC0_SI2.BDR_NUM[5].TBPIR.R /* Tx BDR 5 producer index register */
#define NETC__ENETC0_SI2_TB5CIR NETC__ENETC0_SI2.BDR_NUM[5].TBCIR.R /* Tx BDR 5 consumer index register */
#define NETC__ENETC0_SI2_TB5LENR NETC__ENETC0_SI2.BDR_NUM[5].TBLENR.R /* Tx BDR 5 length register */
#define NETC__ENETC0_SI2_TB5IER NETC__ENETC0_SI2.BDR_NUM[5].TBIER.R /* Tx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI2_TB5IDR NETC__ENETC0_SI2.BDR_NUM[5].TBIDR.R /* Tx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI2_TB5ICR0 NETC__ENETC0_SI2.BDR_NUM[5].TBICR0.R /* Tx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB5ICR1 NETC__ENETC0_SI2.BDR_NUM[5].TBICR1.R /* Tx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB5MR NETC__ENETC0_SI2.BDR_NUM[5].RBMR.R /* Rx BDR 5 mode register */
#define NETC__ENETC0_SI2_RB5SR NETC__ENETC0_SI2.BDR_NUM[5].RBSR.R /* Rx BDR 5 status register */
#define NETC__ENETC0_SI2_RB5BSR NETC__ENETC0_SI2.BDR_NUM[5].RBBSR.R /* Rx BDR 5 buffer size register */
#define NETC__ENETC0_SI2_RB5CIR NETC__ENETC0_SI2.BDR_NUM[5].RBCIR.R /* Rx BDR 5 consumer index register */
#define NETC__ENETC0_SI2_RB5BAR0 NETC__ENETC0_SI2.BDR_NUM[5].RBBAR0.R /* Rx BDR 5 base address register 0 */
#define NETC__ENETC0_SI2_RB5BAR1 NETC__ENETC0_SI2.BDR_NUM[5].RBBAR1.R /* Rx BDR 5 base address register 1 */
#define NETC__ENETC0_SI2_RB5PIR NETC__ENETC0_SI2.BDR_NUM[5].RBPIR.R /* Rx BDR 5 producer index register */
#define NETC__ENETC0_SI2_RB5LENR NETC__ENETC0_SI2.BDR_NUM[5].RBLENR.R /* Rx BDR 5 length register */
#define NETC__ENETC0_SI2_RB5DCR NETC__ENETC0_SI2.BDR_NUM[5].RBDCR.R /* Rx BDR 5 drop count register */
#define NETC__ENETC0_SI2_RB5IER NETC__ENETC0_SI2.BDR_NUM[5].RBIER.R /* Rx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI2_RB5IDR NETC__ENETC0_SI2.BDR_NUM[5].RBIDR.R /* Rx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI2_RB5ICR0 NETC__ENETC0_SI2.BDR_NUM[5].RBICR0.R /* Rx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB5ICR1 NETC__ENETC0_SI2.BDR_NUM[5].RBICR1.R /* Rx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB6MR NETC__ENETC0_SI2.BDR_NUM[6].TBMR.R /* Tx BDR 6 mode register */
#define NETC__ENETC0_SI2_TB6SR NETC__ENETC0_SI2.BDR_NUM[6].TBSR.R /* Tx BDR 6 status register */
#define NETC__ENETC0_SI2_TB6BAR0 NETC__ENETC0_SI2.BDR_NUM[6].TBBAR0.R /* Tx BDR 6 base address register 0 */
#define NETC__ENETC0_SI2_TB6BAR1 NETC__ENETC0_SI2.BDR_NUM[6].TBBAR1.R /* Tx BDR 6 base address register 1 */
#define NETC__ENETC0_SI2_TB6PIR NETC__ENETC0_SI2.BDR_NUM[6].TBPIR.R /* Tx BDR 6 producer index register */
#define NETC__ENETC0_SI2_TB6CIR NETC__ENETC0_SI2.BDR_NUM[6].TBCIR.R /* Tx BDR 6 consumer index register */
#define NETC__ENETC0_SI2_TB6LENR NETC__ENETC0_SI2.BDR_NUM[6].TBLENR.R /* Tx BDR 6 length register */
#define NETC__ENETC0_SI2_TB6IER NETC__ENETC0_SI2.BDR_NUM[6].TBIER.R /* Tx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI2_TB6IDR NETC__ENETC0_SI2.BDR_NUM[6].TBIDR.R /* Tx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI2_TB6ICR0 NETC__ENETC0_SI2.BDR_NUM[6].TBICR0.R /* Tx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB6ICR1 NETC__ENETC0_SI2.BDR_NUM[6].TBICR1.R /* Tx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB6MR NETC__ENETC0_SI2.BDR_NUM[6].RBMR.R /* Rx BDR 6 mode register */
#define NETC__ENETC0_SI2_RB6SR NETC__ENETC0_SI2.BDR_NUM[6].RBSR.R /* Rx BDR 6 status register */
#define NETC__ENETC0_SI2_RB6BSR NETC__ENETC0_SI2.BDR_NUM[6].RBBSR.R /* Rx BDR 6 buffer size register */
#define NETC__ENETC0_SI2_RB6CIR NETC__ENETC0_SI2.BDR_NUM[6].RBCIR.R /* Rx BDR 6 consumer index register */
#define NETC__ENETC0_SI2_RB6BAR0 NETC__ENETC0_SI2.BDR_NUM[6].RBBAR0.R /* Rx BDR 6 base address register 0 */
#define NETC__ENETC0_SI2_RB6BAR1 NETC__ENETC0_SI2.BDR_NUM[6].RBBAR1.R /* Rx BDR 6 base address register 1 */
#define NETC__ENETC0_SI2_RB6PIR NETC__ENETC0_SI2.BDR_NUM[6].RBPIR.R /* Rx BDR 6 producer index register */
#define NETC__ENETC0_SI2_RB6LENR NETC__ENETC0_SI2.BDR_NUM[6].RBLENR.R /* Rx BDR 6 length register */
#define NETC__ENETC0_SI2_RB6DCR NETC__ENETC0_SI2.BDR_NUM[6].RBDCR.R /* Rx BDR 6 drop count register */
#define NETC__ENETC0_SI2_RB6IER NETC__ENETC0_SI2.BDR_NUM[6].RBIER.R /* Rx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI2_RB6IDR NETC__ENETC0_SI2.BDR_NUM[6].RBIDR.R /* Rx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI2_RB6ICR0 NETC__ENETC0_SI2.BDR_NUM[6].RBICR0.R /* Rx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB6ICR1 NETC__ENETC0_SI2.BDR_NUM[6].RBICR1.R /* Rx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB7MR NETC__ENETC0_SI2.BDR_NUM[7].TBMR.R /* Tx BDR 7 mode register */
#define NETC__ENETC0_SI2_TB7SR NETC__ENETC0_SI2.BDR_NUM[7].TBSR.R /* Tx BDR 7 status register */
#define NETC__ENETC0_SI2_TB7BAR0 NETC__ENETC0_SI2.BDR_NUM[7].TBBAR0.R /* Tx BDR 7 base address register 0 */
#define NETC__ENETC0_SI2_TB7BAR1 NETC__ENETC0_SI2.BDR_NUM[7].TBBAR1.R /* Tx BDR 7 base address register 1 */
#define NETC__ENETC0_SI2_TB7PIR NETC__ENETC0_SI2.BDR_NUM[7].TBPIR.R /* Tx BDR 7 producer index register */
#define NETC__ENETC0_SI2_TB7CIR NETC__ENETC0_SI2.BDR_NUM[7].TBCIR.R /* Tx BDR 7 consumer index register */
#define NETC__ENETC0_SI2_TB7LENR NETC__ENETC0_SI2.BDR_NUM[7].TBLENR.R /* Tx BDR 7 length register */
#define NETC__ENETC0_SI2_TB7IER NETC__ENETC0_SI2.BDR_NUM[7].TBIER.R /* Tx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI2_TB7IDR NETC__ENETC0_SI2.BDR_NUM[7].TBIDR.R /* Tx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI2_TB7ICR0 NETC__ENETC0_SI2.BDR_NUM[7].TBICR0.R /* Tx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB7ICR1 NETC__ENETC0_SI2.BDR_NUM[7].TBICR1.R /* Tx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB7MR NETC__ENETC0_SI2.BDR_NUM[7].RBMR.R /* Rx BDR 7 mode register */
#define NETC__ENETC0_SI2_RB7SR NETC__ENETC0_SI2.BDR_NUM[7].RBSR.R /* Rx BDR 7 status register */
#define NETC__ENETC0_SI2_RB7BSR NETC__ENETC0_SI2.BDR_NUM[7].RBBSR.R /* Rx BDR 7 buffer size register */
#define NETC__ENETC0_SI2_RB7CIR NETC__ENETC0_SI2.BDR_NUM[7].RBCIR.R /* Rx BDR 7 consumer index register */
#define NETC__ENETC0_SI2_RB7BAR0 NETC__ENETC0_SI2.BDR_NUM[7].RBBAR0.R /* Rx BDR 7 base address register 0 */
#define NETC__ENETC0_SI2_RB7BAR1 NETC__ENETC0_SI2.BDR_NUM[7].RBBAR1.R /* Rx BDR 7 base address register 1 */
#define NETC__ENETC0_SI2_RB7PIR NETC__ENETC0_SI2.BDR_NUM[7].RBPIR.R /* Rx BDR 7 producer index register */
#define NETC__ENETC0_SI2_RB7LENR NETC__ENETC0_SI2.BDR_NUM[7].RBLENR.R /* Rx BDR 7 length register */
#define NETC__ENETC0_SI2_RB7DCR NETC__ENETC0_SI2.BDR_NUM[7].RBDCR.R /* Rx BDR 7 drop count register */
#define NETC__ENETC0_SI2_RB7IER NETC__ENETC0_SI2.BDR_NUM[7].RBIER.R /* Rx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI2_RB7IDR NETC__ENETC0_SI2.BDR_NUM[7].RBIDR.R /* Rx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI2_RB7ICR0 NETC__ENETC0_SI2.BDR_NUM[7].RBICR0.R /* Rx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB7ICR1 NETC__ENETC0_SI2.BDR_NUM[7].RBICR1.R /* Rx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB8MR NETC__ENETC0_SI2.BDR_NUM[8].TBMR.R /* Tx BDR 8 mode register */
#define NETC__ENETC0_SI2_TB8SR NETC__ENETC0_SI2.BDR_NUM[8].TBSR.R /* Tx BDR 8 status register */
#define NETC__ENETC0_SI2_TB8BAR0 NETC__ENETC0_SI2.BDR_NUM[8].TBBAR0.R /* Tx BDR 8 base address register 0 */
#define NETC__ENETC0_SI2_TB8BAR1 NETC__ENETC0_SI2.BDR_NUM[8].TBBAR1.R /* Tx BDR 8 base address register 1 */
#define NETC__ENETC0_SI2_TB8PIR NETC__ENETC0_SI2.BDR_NUM[8].TBPIR.R /* Tx BDR 8 producer index register */
#define NETC__ENETC0_SI2_TB8CIR NETC__ENETC0_SI2.BDR_NUM[8].TBCIR.R /* Tx BDR 8 consumer index register */
#define NETC__ENETC0_SI2_TB8LENR NETC__ENETC0_SI2.BDR_NUM[8].TBLENR.R /* Tx BDR 8 length register */
#define NETC__ENETC0_SI2_TB8IER NETC__ENETC0_SI2.BDR_NUM[8].TBIER.R /* Tx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI2_TB8IDR NETC__ENETC0_SI2.BDR_NUM[8].TBIDR.R /* Tx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI2_TB8ICR0 NETC__ENETC0_SI2.BDR_NUM[8].TBICR0.R /* Tx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB8ICR1 NETC__ENETC0_SI2.BDR_NUM[8].TBICR1.R /* Tx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB8MR NETC__ENETC0_SI2.BDR_NUM[8].RBMR.R /* Rx BDR 8 mode register */
#define NETC__ENETC0_SI2_RB8SR NETC__ENETC0_SI2.BDR_NUM[8].RBSR.R /* Rx BDR 8 status register */
#define NETC__ENETC0_SI2_RB8BSR NETC__ENETC0_SI2.BDR_NUM[8].RBBSR.R /* Rx BDR 8 buffer size register */
#define NETC__ENETC0_SI2_RB8CIR NETC__ENETC0_SI2.BDR_NUM[8].RBCIR.R /* Rx BDR 8 consumer index register */
#define NETC__ENETC0_SI2_RB8BAR0 NETC__ENETC0_SI2.BDR_NUM[8].RBBAR0.R /* Rx BDR 8 base address register 0 */
#define NETC__ENETC0_SI2_RB8BAR1 NETC__ENETC0_SI2.BDR_NUM[8].RBBAR1.R /* Rx BDR 8 base address register 1 */
#define NETC__ENETC0_SI2_RB8PIR NETC__ENETC0_SI2.BDR_NUM[8].RBPIR.R /* Rx BDR 8 producer index register */
#define NETC__ENETC0_SI2_RB8LENR NETC__ENETC0_SI2.BDR_NUM[8].RBLENR.R /* Rx BDR 8 length register */
#define NETC__ENETC0_SI2_RB8DCR NETC__ENETC0_SI2.BDR_NUM[8].RBDCR.R /* Rx BDR 8 drop count register */
#define NETC__ENETC0_SI2_RB8IER NETC__ENETC0_SI2.BDR_NUM[8].RBIER.R /* Rx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI2_RB8IDR NETC__ENETC0_SI2.BDR_NUM[8].RBIDR.R /* Rx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI2_RB8ICR0 NETC__ENETC0_SI2.BDR_NUM[8].RBICR0.R /* Rx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB8ICR1 NETC__ENETC0_SI2.BDR_NUM[8].RBICR1.R /* Rx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB9MR NETC__ENETC0_SI2.BDR_NUM[9].TBMR.R /* Tx BDR 9 mode register */
#define NETC__ENETC0_SI2_TB9SR NETC__ENETC0_SI2.BDR_NUM[9].TBSR.R /* Tx BDR 9 status register */
#define NETC__ENETC0_SI2_TB9BAR0 NETC__ENETC0_SI2.BDR_NUM[9].TBBAR0.R /* Tx BDR 9 base address register 0 */
#define NETC__ENETC0_SI2_TB9BAR1 NETC__ENETC0_SI2.BDR_NUM[9].TBBAR1.R /* Tx BDR 9 base address register 1 */
#define NETC__ENETC0_SI2_TB9PIR NETC__ENETC0_SI2.BDR_NUM[9].TBPIR.R /* Tx BDR 9 producer index register */
#define NETC__ENETC0_SI2_TB9CIR NETC__ENETC0_SI2.BDR_NUM[9].TBCIR.R /* Tx BDR 9 consumer index register */
#define NETC__ENETC0_SI2_TB9LENR NETC__ENETC0_SI2.BDR_NUM[9].TBLENR.R /* Tx BDR 9 length register */
#define NETC__ENETC0_SI2_TB9IER NETC__ENETC0_SI2.BDR_NUM[9].TBIER.R /* Tx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI2_TB9IDR NETC__ENETC0_SI2.BDR_NUM[9].TBIDR.R /* Tx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI2_TB9ICR0 NETC__ENETC0_SI2.BDR_NUM[9].TBICR0.R /* Tx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB9ICR1 NETC__ENETC0_SI2.BDR_NUM[9].TBICR1.R /* Tx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB9MR NETC__ENETC0_SI2.BDR_NUM[9].RBMR.R /* Rx BDR 9 mode register */
#define NETC__ENETC0_SI2_RB9SR NETC__ENETC0_SI2.BDR_NUM[9].RBSR.R /* Rx BDR 9 status register */
#define NETC__ENETC0_SI2_RB9BSR NETC__ENETC0_SI2.BDR_NUM[9].RBBSR.R /* Rx BDR 9 buffer size register */
#define NETC__ENETC0_SI2_RB9CIR NETC__ENETC0_SI2.BDR_NUM[9].RBCIR.R /* Rx BDR 9 consumer index register */
#define NETC__ENETC0_SI2_RB9BAR0 NETC__ENETC0_SI2.BDR_NUM[9].RBBAR0.R /* Rx BDR 9 base address register 0 */
#define NETC__ENETC0_SI2_RB9BAR1 NETC__ENETC0_SI2.BDR_NUM[9].RBBAR1.R /* Rx BDR 9 base address register 1 */
#define NETC__ENETC0_SI2_RB9PIR NETC__ENETC0_SI2.BDR_NUM[9].RBPIR.R /* Rx BDR 9 producer index register */
#define NETC__ENETC0_SI2_RB9LENR NETC__ENETC0_SI2.BDR_NUM[9].RBLENR.R /* Rx BDR 9 length register */
#define NETC__ENETC0_SI2_RB9DCR NETC__ENETC0_SI2.BDR_NUM[9].RBDCR.R /* Rx BDR 9 drop count register */
#define NETC__ENETC0_SI2_RB9IER NETC__ENETC0_SI2.BDR_NUM[9].RBIER.R /* Rx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI2_RB9IDR NETC__ENETC0_SI2.BDR_NUM[9].RBIDR.R /* Rx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI2_RB9ICR0 NETC__ENETC0_SI2.BDR_NUM[9].RBICR0.R /* Rx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB9ICR1 NETC__ENETC0_SI2.BDR_NUM[9].RBICR1.R /* Rx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB10MR NETC__ENETC0_SI2.BDR_NUM[10].TBMR.R /* Tx BDR 10 mode register */
#define NETC__ENETC0_SI2_TB10SR NETC__ENETC0_SI2.BDR_NUM[10].TBSR.R /* Tx BDR 10 status register */
#define NETC__ENETC0_SI2_TB10BAR0 NETC__ENETC0_SI2.BDR_NUM[10].TBBAR0.R /* Tx BDR 10 base address register 0 */
#define NETC__ENETC0_SI2_TB10BAR1 NETC__ENETC0_SI2.BDR_NUM[10].TBBAR1.R /* Tx BDR 10 base address register 1 */
#define NETC__ENETC0_SI2_TB10PIR NETC__ENETC0_SI2.BDR_NUM[10].TBPIR.R /* Tx BDR 10 producer index register */
#define NETC__ENETC0_SI2_TB10CIR NETC__ENETC0_SI2.BDR_NUM[10].TBCIR.R /* Tx BDR 10 consumer index register */
#define NETC__ENETC0_SI2_TB10LENR NETC__ENETC0_SI2.BDR_NUM[10].TBLENR.R /* Tx BDR 10 length register */
#define NETC__ENETC0_SI2_TB10IER NETC__ENETC0_SI2.BDR_NUM[10].TBIER.R /* Tx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI2_TB10IDR NETC__ENETC0_SI2.BDR_NUM[10].TBIDR.R /* Tx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI2_TB10ICR0 NETC__ENETC0_SI2.BDR_NUM[10].TBICR0.R /* Tx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB10ICR1 NETC__ENETC0_SI2.BDR_NUM[10].TBICR1.R /* Tx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB10MR NETC__ENETC0_SI2.BDR_NUM[10].RBMR.R /* Rx BDR 10 mode register */
#define NETC__ENETC0_SI2_RB10SR NETC__ENETC0_SI2.BDR_NUM[10].RBSR.R /* Rx BDR 10 status register */
#define NETC__ENETC0_SI2_RB10BSR NETC__ENETC0_SI2.BDR_NUM[10].RBBSR.R /* Rx BDR 10 buffer size register */
#define NETC__ENETC0_SI2_RB10CIR NETC__ENETC0_SI2.BDR_NUM[10].RBCIR.R /* Rx BDR 10 consumer index register */
#define NETC__ENETC0_SI2_RB10BAR0 NETC__ENETC0_SI2.BDR_NUM[10].RBBAR0.R /* Rx BDR 10 base address register 0 */
#define NETC__ENETC0_SI2_RB10BAR1 NETC__ENETC0_SI2.BDR_NUM[10].RBBAR1.R /* Rx BDR 10 base address register 1 */
#define NETC__ENETC0_SI2_RB10PIR NETC__ENETC0_SI2.BDR_NUM[10].RBPIR.R /* Rx BDR 10 producer index register */
#define NETC__ENETC0_SI2_RB10LENR NETC__ENETC0_SI2.BDR_NUM[10].RBLENR.R /* Rx BDR 10 length register */
#define NETC__ENETC0_SI2_RB10DCR NETC__ENETC0_SI2.BDR_NUM[10].RBDCR.R /* Rx BDR 10 drop count register */
#define NETC__ENETC0_SI2_RB10IER NETC__ENETC0_SI2.BDR_NUM[10].RBIER.R /* Rx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI2_RB10IDR NETC__ENETC0_SI2.BDR_NUM[10].RBIDR.R /* Rx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI2_RB10ICR0 NETC__ENETC0_SI2.BDR_NUM[10].RBICR0.R /* Rx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB10ICR1 NETC__ENETC0_SI2.BDR_NUM[10].RBICR1.R /* Rx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB11MR NETC__ENETC0_SI2.BDR_NUM[11].TBMR.R /* Tx BDR 11 mode register */
#define NETC__ENETC0_SI2_TB11SR NETC__ENETC0_SI2.BDR_NUM[11].TBSR.R /* Tx BDR 11 status register */
#define NETC__ENETC0_SI2_TB11BAR0 NETC__ENETC0_SI2.BDR_NUM[11].TBBAR0.R /* Tx BDR 11 base address register 0 */
#define NETC__ENETC0_SI2_TB11BAR1 NETC__ENETC0_SI2.BDR_NUM[11].TBBAR1.R /* Tx BDR 11 base address register 1 */
#define NETC__ENETC0_SI2_TB11PIR NETC__ENETC0_SI2.BDR_NUM[11].TBPIR.R /* Tx BDR 11 producer index register */
#define NETC__ENETC0_SI2_TB11CIR NETC__ENETC0_SI2.BDR_NUM[11].TBCIR.R /* Tx BDR 11 consumer index register */
#define NETC__ENETC0_SI2_TB11LENR NETC__ENETC0_SI2.BDR_NUM[11].TBLENR.R /* Tx BDR 11 length register */
#define NETC__ENETC0_SI2_TB11IER NETC__ENETC0_SI2.BDR_NUM[11].TBIER.R /* Tx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI2_TB11IDR NETC__ENETC0_SI2.BDR_NUM[11].TBIDR.R /* Tx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI2_TB11ICR0 NETC__ENETC0_SI2.BDR_NUM[11].TBICR0.R /* Tx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB11ICR1 NETC__ENETC0_SI2.BDR_NUM[11].TBICR1.R /* Tx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB11MR NETC__ENETC0_SI2.BDR_NUM[11].RBMR.R /* Rx BDR 11 mode register */
#define NETC__ENETC0_SI2_RB11SR NETC__ENETC0_SI2.BDR_NUM[11].RBSR.R /* Rx BDR 11 status register */
#define NETC__ENETC0_SI2_RB11BSR NETC__ENETC0_SI2.BDR_NUM[11].RBBSR.R /* Rx BDR 11 buffer size register */
#define NETC__ENETC0_SI2_RB11CIR NETC__ENETC0_SI2.BDR_NUM[11].RBCIR.R /* Rx BDR 11 consumer index register */
#define NETC__ENETC0_SI2_RB11BAR0 NETC__ENETC0_SI2.BDR_NUM[11].RBBAR0.R /* Rx BDR 11 base address register 0 */
#define NETC__ENETC0_SI2_RB11BAR1 NETC__ENETC0_SI2.BDR_NUM[11].RBBAR1.R /* Rx BDR 11 base address register 1 */
#define NETC__ENETC0_SI2_RB11PIR NETC__ENETC0_SI2.BDR_NUM[11].RBPIR.R /* Rx BDR 11 producer index register */
#define NETC__ENETC0_SI2_RB11LENR NETC__ENETC0_SI2.BDR_NUM[11].RBLENR.R /* Rx BDR 11 length register */
#define NETC__ENETC0_SI2_RB11DCR NETC__ENETC0_SI2.BDR_NUM[11].RBDCR.R /* Rx BDR 11 drop count register */
#define NETC__ENETC0_SI2_RB11IER NETC__ENETC0_SI2.BDR_NUM[11].RBIER.R /* Rx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI2_RB11IDR NETC__ENETC0_SI2.BDR_NUM[11].RBIDR.R /* Rx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI2_RB11ICR0 NETC__ENETC0_SI2.BDR_NUM[11].RBICR0.R /* Rx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB11ICR1 NETC__ENETC0_SI2.BDR_NUM[11].RBICR1.R /* Rx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB12MR NETC__ENETC0_SI2.BDR_NUM[12].TBMR.R /* Tx BDR 12 mode register */
#define NETC__ENETC0_SI2_TB12SR NETC__ENETC0_SI2.BDR_NUM[12].TBSR.R /* Tx BDR 12 status register */
#define NETC__ENETC0_SI2_TB12BAR0 NETC__ENETC0_SI2.BDR_NUM[12].TBBAR0.R /* Tx BDR 12 base address register 0 */
#define NETC__ENETC0_SI2_TB12BAR1 NETC__ENETC0_SI2.BDR_NUM[12].TBBAR1.R /* Tx BDR 12 base address register 1 */
#define NETC__ENETC0_SI2_TB12PIR NETC__ENETC0_SI2.BDR_NUM[12].TBPIR.R /* Tx BDR 12 producer index register */
#define NETC__ENETC0_SI2_TB12CIR NETC__ENETC0_SI2.BDR_NUM[12].TBCIR.R /* Tx BDR 12 consumer index register */
#define NETC__ENETC0_SI2_TB12LENR NETC__ENETC0_SI2.BDR_NUM[12].TBLENR.R /* Tx BDR 12 length register */
#define NETC__ENETC0_SI2_TB12IER NETC__ENETC0_SI2.BDR_NUM[12].TBIER.R /* Tx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI2_TB12IDR NETC__ENETC0_SI2.BDR_NUM[12].TBIDR.R /* Tx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI2_TB12ICR0 NETC__ENETC0_SI2.BDR_NUM[12].TBICR0.R /* Tx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB12ICR1 NETC__ENETC0_SI2.BDR_NUM[12].TBICR1.R /* Tx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB12MR NETC__ENETC0_SI2.BDR_NUM[12].RBMR.R /* Rx BDR 12 mode register */
#define NETC__ENETC0_SI2_RB12SR NETC__ENETC0_SI2.BDR_NUM[12].RBSR.R /* Rx BDR 12 status register */
#define NETC__ENETC0_SI2_RB12BSR NETC__ENETC0_SI2.BDR_NUM[12].RBBSR.R /* Rx BDR 12 buffer size register */
#define NETC__ENETC0_SI2_RB12CIR NETC__ENETC0_SI2.BDR_NUM[12].RBCIR.R /* Rx BDR 12 consumer index register */
#define NETC__ENETC0_SI2_RB12BAR0 NETC__ENETC0_SI2.BDR_NUM[12].RBBAR0.R /* Rx BDR 12 base address register 0 */
#define NETC__ENETC0_SI2_RB12BAR1 NETC__ENETC0_SI2.BDR_NUM[12].RBBAR1.R /* Rx BDR 12 base address register 1 */
#define NETC__ENETC0_SI2_RB12PIR NETC__ENETC0_SI2.BDR_NUM[12].RBPIR.R /* Rx BDR 12 producer index register */
#define NETC__ENETC0_SI2_RB12LENR NETC__ENETC0_SI2.BDR_NUM[12].RBLENR.R /* Rx BDR 12 length register */
#define NETC__ENETC0_SI2_RB12DCR NETC__ENETC0_SI2.BDR_NUM[12].RBDCR.R /* Rx BDR 12 drop count register */
#define NETC__ENETC0_SI2_RB12IER NETC__ENETC0_SI2.BDR_NUM[12].RBIER.R /* Rx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI2_RB12IDR NETC__ENETC0_SI2.BDR_NUM[12].RBIDR.R /* Rx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI2_RB12ICR0 NETC__ENETC0_SI2.BDR_NUM[12].RBICR0.R /* Rx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB12ICR1 NETC__ENETC0_SI2.BDR_NUM[12].RBICR1.R /* Rx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB13MR NETC__ENETC0_SI2.BDR_NUM[13].TBMR.R /* Tx BDR 13 mode register */
#define NETC__ENETC0_SI2_TB13SR NETC__ENETC0_SI2.BDR_NUM[13].TBSR.R /* Tx BDR 13 status register */
#define NETC__ENETC0_SI2_TB13BAR0 NETC__ENETC0_SI2.BDR_NUM[13].TBBAR0.R /* Tx BDR 13 base address register 0 */
#define NETC__ENETC0_SI2_TB13BAR1 NETC__ENETC0_SI2.BDR_NUM[13].TBBAR1.R /* Tx BDR 13 base address register 1 */
#define NETC__ENETC0_SI2_TB13PIR NETC__ENETC0_SI2.BDR_NUM[13].TBPIR.R /* Tx BDR 13 producer index register */
#define NETC__ENETC0_SI2_TB13CIR NETC__ENETC0_SI2.BDR_NUM[13].TBCIR.R /* Tx BDR 13 consumer index register */
#define NETC__ENETC0_SI2_TB13LENR NETC__ENETC0_SI2.BDR_NUM[13].TBLENR.R /* Tx BDR 13 length register */
#define NETC__ENETC0_SI2_TB13IER NETC__ENETC0_SI2.BDR_NUM[13].TBIER.R /* Tx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI2_TB13IDR NETC__ENETC0_SI2.BDR_NUM[13].TBIDR.R /* Tx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI2_TB13ICR0 NETC__ENETC0_SI2.BDR_NUM[13].TBICR0.R /* Tx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB13ICR1 NETC__ENETC0_SI2.BDR_NUM[13].TBICR1.R /* Tx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB13MR NETC__ENETC0_SI2.BDR_NUM[13].RBMR.R /* Rx BDR 13 mode register */
#define NETC__ENETC0_SI2_RB13SR NETC__ENETC0_SI2.BDR_NUM[13].RBSR.R /* Rx BDR 13 status register */
#define NETC__ENETC0_SI2_RB13BSR NETC__ENETC0_SI2.BDR_NUM[13].RBBSR.R /* Rx BDR 13 buffer size register */
#define NETC__ENETC0_SI2_RB13CIR NETC__ENETC0_SI2.BDR_NUM[13].RBCIR.R /* Rx BDR 13 consumer index register */
#define NETC__ENETC0_SI2_RB13BAR0 NETC__ENETC0_SI2.BDR_NUM[13].RBBAR0.R /* Rx BDR 13 base address register 0 */
#define NETC__ENETC0_SI2_RB13BAR1 NETC__ENETC0_SI2.BDR_NUM[13].RBBAR1.R /* Rx BDR 13 base address register 1 */
#define NETC__ENETC0_SI2_RB13PIR NETC__ENETC0_SI2.BDR_NUM[13].RBPIR.R /* Rx BDR 13 producer index register */
#define NETC__ENETC0_SI2_RB13LENR NETC__ENETC0_SI2.BDR_NUM[13].RBLENR.R /* Rx BDR 13 length register */
#define NETC__ENETC0_SI2_RB13DCR NETC__ENETC0_SI2.BDR_NUM[13].RBDCR.R /* Rx BDR 13 drop count register */
#define NETC__ENETC0_SI2_RB13IER NETC__ENETC0_SI2.BDR_NUM[13].RBIER.R /* Rx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI2_RB13IDR NETC__ENETC0_SI2.BDR_NUM[13].RBIDR.R /* Rx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI2_RB13ICR0 NETC__ENETC0_SI2.BDR_NUM[13].RBICR0.R /* Rx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB13ICR1 NETC__ENETC0_SI2.BDR_NUM[13].RBICR1.R /* Rx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB14MR NETC__ENETC0_SI2.BDR_NUM[14].TBMR.R /* Tx BDR 14 mode register */
#define NETC__ENETC0_SI2_TB14SR NETC__ENETC0_SI2.BDR_NUM[14].TBSR.R /* Tx BDR 14 status register */
#define NETC__ENETC0_SI2_TB14BAR0 NETC__ENETC0_SI2.BDR_NUM[14].TBBAR0.R /* Tx BDR 14 base address register 0 */
#define NETC__ENETC0_SI2_TB14BAR1 NETC__ENETC0_SI2.BDR_NUM[14].TBBAR1.R /* Tx BDR 14 base address register 1 */
#define NETC__ENETC0_SI2_TB14PIR NETC__ENETC0_SI2.BDR_NUM[14].TBPIR.R /* Tx BDR 14 producer index register */
#define NETC__ENETC0_SI2_TB14CIR NETC__ENETC0_SI2.BDR_NUM[14].TBCIR.R /* Tx BDR 14 consumer index register */
#define NETC__ENETC0_SI2_TB14LENR NETC__ENETC0_SI2.BDR_NUM[14].TBLENR.R /* Tx BDR 14 length register */
#define NETC__ENETC0_SI2_TB14IER NETC__ENETC0_SI2.BDR_NUM[14].TBIER.R /* Tx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI2_TB14IDR NETC__ENETC0_SI2.BDR_NUM[14].TBIDR.R /* Tx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI2_TB14ICR0 NETC__ENETC0_SI2.BDR_NUM[14].TBICR0.R /* Tx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB14ICR1 NETC__ENETC0_SI2.BDR_NUM[14].TBICR1.R /* Tx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB14MR NETC__ENETC0_SI2.BDR_NUM[14].RBMR.R /* Rx BDR 14 mode register */
#define NETC__ENETC0_SI2_RB14SR NETC__ENETC0_SI2.BDR_NUM[14].RBSR.R /* Rx BDR 14 status register */
#define NETC__ENETC0_SI2_RB14BSR NETC__ENETC0_SI2.BDR_NUM[14].RBBSR.R /* Rx BDR 14 buffer size register */
#define NETC__ENETC0_SI2_RB14CIR NETC__ENETC0_SI2.BDR_NUM[14].RBCIR.R /* Rx BDR 14 consumer index register */
#define NETC__ENETC0_SI2_RB14BAR0 NETC__ENETC0_SI2.BDR_NUM[14].RBBAR0.R /* Rx BDR 14 base address register 0 */
#define NETC__ENETC0_SI2_RB14BAR1 NETC__ENETC0_SI2.BDR_NUM[14].RBBAR1.R /* Rx BDR 14 base address register 1 */
#define NETC__ENETC0_SI2_RB14PIR NETC__ENETC0_SI2.BDR_NUM[14].RBPIR.R /* Rx BDR 14 producer index register */
#define NETC__ENETC0_SI2_RB14LENR NETC__ENETC0_SI2.BDR_NUM[14].RBLENR.R /* Rx BDR 14 length register */
#define NETC__ENETC0_SI2_RB14DCR NETC__ENETC0_SI2.BDR_NUM[14].RBDCR.R /* Rx BDR 14 drop count register */
#define NETC__ENETC0_SI2_RB14IER NETC__ENETC0_SI2.BDR_NUM[14].RBIER.R /* Rx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI2_RB14IDR NETC__ENETC0_SI2.BDR_NUM[14].RBIDR.R /* Rx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI2_RB14ICR0 NETC__ENETC0_SI2.BDR_NUM[14].RBICR0.R /* Rx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB14ICR1 NETC__ENETC0_SI2.BDR_NUM[14].RBICR1.R /* Rx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB15MR NETC__ENETC0_SI2.BDR_NUM[15].TBMR.R /* Tx BDR 15 mode register */
#define NETC__ENETC0_SI2_TB15SR NETC__ENETC0_SI2.BDR_NUM[15].TBSR.R /* Tx BDR 15 status register */
#define NETC__ENETC0_SI2_TB15BAR0 NETC__ENETC0_SI2.BDR_NUM[15].TBBAR0.R /* Tx BDR 15 base address register 0 */
#define NETC__ENETC0_SI2_TB15BAR1 NETC__ENETC0_SI2.BDR_NUM[15].TBBAR1.R /* Tx BDR 15 base address register 1 */
#define NETC__ENETC0_SI2_TB15PIR NETC__ENETC0_SI2.BDR_NUM[15].TBPIR.R /* Tx BDR 15 producer index register */
#define NETC__ENETC0_SI2_TB15CIR NETC__ENETC0_SI2.BDR_NUM[15].TBCIR.R /* Tx BDR 15 consumer index register */
#define NETC__ENETC0_SI2_TB15LENR NETC__ENETC0_SI2.BDR_NUM[15].TBLENR.R /* Tx BDR 15 length register */
#define NETC__ENETC0_SI2_TB15IER NETC__ENETC0_SI2.BDR_NUM[15].TBIER.R /* Tx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI2_TB15IDR NETC__ENETC0_SI2.BDR_NUM[15].TBIDR.R /* Tx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI2_TB15ICR0 NETC__ENETC0_SI2.BDR_NUM[15].TBICR0.R /* Tx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB15ICR1 NETC__ENETC0_SI2.BDR_NUM[15].TBICR1.R /* Tx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB15MR NETC__ENETC0_SI2.BDR_NUM[15].RBMR.R /* Rx BDR 15 mode register */
#define NETC__ENETC0_SI2_RB15SR NETC__ENETC0_SI2.BDR_NUM[15].RBSR.R /* Rx BDR 15 status register */
#define NETC__ENETC0_SI2_RB15BSR NETC__ENETC0_SI2.BDR_NUM[15].RBBSR.R /* Rx BDR 15 buffer size register */
#define NETC__ENETC0_SI2_RB15CIR NETC__ENETC0_SI2.BDR_NUM[15].RBCIR.R /* Rx BDR 15 consumer index register */
#define NETC__ENETC0_SI2_RB15BAR0 NETC__ENETC0_SI2.BDR_NUM[15].RBBAR0.R /* Rx BDR 15 base address register 0 */
#define NETC__ENETC0_SI2_RB15BAR1 NETC__ENETC0_SI2.BDR_NUM[15].RBBAR1.R /* Rx BDR 15 base address register 1 */
#define NETC__ENETC0_SI2_RB15PIR NETC__ENETC0_SI2.BDR_NUM[15].RBPIR.R /* Rx BDR 15 producer index register */
#define NETC__ENETC0_SI2_RB15LENR NETC__ENETC0_SI2.BDR_NUM[15].RBLENR.R /* Rx BDR 15 length register */
#define NETC__ENETC0_SI2_RB15DCR NETC__ENETC0_SI2.BDR_NUM[15].RBDCR.R /* Rx BDR 15 drop count register */
#define NETC__ENETC0_SI2_RB15IER NETC__ENETC0_SI2.BDR_NUM[15].RBIER.R /* Rx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI2_RB15IDR NETC__ENETC0_SI2.BDR_NUM[15].RBIDR.R /* Rx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI2_RB15ICR0 NETC__ENETC0_SI2.BDR_NUM[15].RBICR0.R /* Rx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB15ICR1 NETC__ENETC0_SI2.BDR_NUM[15].RBICR1.R /* Rx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB16MR NETC__ENETC0_SI2.BDR_NUM[16].TBMR.R /* Tx BDR 16 mode register */
#define NETC__ENETC0_SI2_TB16SR NETC__ENETC0_SI2.BDR_NUM[16].TBSR.R /* Tx BDR 16 status register */
#define NETC__ENETC0_SI2_TB16BAR0 NETC__ENETC0_SI2.BDR_NUM[16].TBBAR0.R /* Tx BDR 16 base address register 0 */
#define NETC__ENETC0_SI2_TB16BAR1 NETC__ENETC0_SI2.BDR_NUM[16].TBBAR1.R /* Tx BDR 16 base address register 1 */
#define NETC__ENETC0_SI2_TB16PIR NETC__ENETC0_SI2.BDR_NUM[16].TBPIR.R /* Tx BDR 16 producer index register */
#define NETC__ENETC0_SI2_TB16CIR NETC__ENETC0_SI2.BDR_NUM[16].TBCIR.R /* Tx BDR 16 consumer index register */
#define NETC__ENETC0_SI2_TB16LENR NETC__ENETC0_SI2.BDR_NUM[16].TBLENR.R /* Tx BDR 16 length register */
#define NETC__ENETC0_SI2_TB16IER NETC__ENETC0_SI2.BDR_NUM[16].TBIER.R /* Tx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI2_TB16IDR NETC__ENETC0_SI2.BDR_NUM[16].TBIDR.R /* Tx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI2_TB16ICR0 NETC__ENETC0_SI2.BDR_NUM[16].TBICR0.R /* Tx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB16ICR1 NETC__ENETC0_SI2.BDR_NUM[16].TBICR1.R /* Tx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB16MR NETC__ENETC0_SI2.BDR_NUM[16].RBMR.R /* Rx BDR 16 mode register */
#define NETC__ENETC0_SI2_RB16SR NETC__ENETC0_SI2.BDR_NUM[16].RBSR.R /* Rx BDR 16 status register */
#define NETC__ENETC0_SI2_RB16BSR NETC__ENETC0_SI2.BDR_NUM[16].RBBSR.R /* Rx BDR 16 buffer size register */
#define NETC__ENETC0_SI2_RB16CIR NETC__ENETC0_SI2.BDR_NUM[16].RBCIR.R /* Rx BDR 16 consumer index register */
#define NETC__ENETC0_SI2_RB16BAR0 NETC__ENETC0_SI2.BDR_NUM[16].RBBAR0.R /* Rx BDR 16 base address register 0 */
#define NETC__ENETC0_SI2_RB16BAR1 NETC__ENETC0_SI2.BDR_NUM[16].RBBAR1.R /* Rx BDR 16 base address register 1 */
#define NETC__ENETC0_SI2_RB16PIR NETC__ENETC0_SI2.BDR_NUM[16].RBPIR.R /* Rx BDR 16 producer index register */
#define NETC__ENETC0_SI2_RB16LENR NETC__ENETC0_SI2.BDR_NUM[16].RBLENR.R /* Rx BDR 16 length register */
#define NETC__ENETC0_SI2_RB16DCR NETC__ENETC0_SI2.BDR_NUM[16].RBDCR.R /* Rx BDR 16 drop count register */
#define NETC__ENETC0_SI2_RB16IER NETC__ENETC0_SI2.BDR_NUM[16].RBIER.R /* Rx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI2_RB16IDR NETC__ENETC0_SI2.BDR_NUM[16].RBIDR.R /* Rx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI2_RB16ICR0 NETC__ENETC0_SI2.BDR_NUM[16].RBICR0.R /* Rx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB16ICR1 NETC__ENETC0_SI2.BDR_NUM[16].RBICR1.R /* Rx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_TB17MR NETC__ENETC0_SI2.BDR_NUM[17].TBMR.R /* Tx BDR 17 mode register */
#define NETC__ENETC0_SI2_TB17SR NETC__ENETC0_SI2.BDR_NUM[17].TBSR.R /* Tx BDR 17 status register */
#define NETC__ENETC0_SI2_TB17BAR0 NETC__ENETC0_SI2.BDR_NUM[17].TBBAR0.R /* Tx BDR 17 base address register 0 */
#define NETC__ENETC0_SI2_TB17BAR1 NETC__ENETC0_SI2.BDR_NUM[17].TBBAR1.R /* Tx BDR 17 base address register 1 */
#define NETC__ENETC0_SI2_TB17PIR NETC__ENETC0_SI2.BDR_NUM[17].TBPIR.R /* Tx BDR 17 producer index register */
#define NETC__ENETC0_SI2_TB17CIR NETC__ENETC0_SI2.BDR_NUM[17].TBCIR.R /* Tx BDR 17 consumer index register */
#define NETC__ENETC0_SI2_TB17LENR NETC__ENETC0_SI2.BDR_NUM[17].TBLENR.R /* Tx BDR 17 length register */
#define NETC__ENETC0_SI2_TB17IER NETC__ENETC0_SI2.BDR_NUM[17].TBIER.R /* Tx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI2_TB17IDR NETC__ENETC0_SI2.BDR_NUM[17].TBIDR.R /* Tx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI2_TB17ICR0 NETC__ENETC0_SI2.BDR_NUM[17].TBICR0.R /* Tx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_TB17ICR1 NETC__ENETC0_SI2.BDR_NUM[17].TBICR1.R /* Tx BDR 17 interrupt coalescing register 1 */
#define NETC__ENETC0_SI2_RB17MR NETC__ENETC0_SI2.BDR_NUM[17].RBMR.R /* Rx BDR 17 mode register */
#define NETC__ENETC0_SI2_RB17SR NETC__ENETC0_SI2.BDR_NUM[17].RBSR.R /* Rx BDR 17 status register */
#define NETC__ENETC0_SI2_RB17BSR NETC__ENETC0_SI2.BDR_NUM[17].RBBSR.R /* Rx BDR 17 buffer size register */
#define NETC__ENETC0_SI2_RB17CIR NETC__ENETC0_SI2.BDR_NUM[17].RBCIR.R /* Rx BDR 17 consumer index register */
#define NETC__ENETC0_SI2_RB17BAR0 NETC__ENETC0_SI2.BDR_NUM[17].RBBAR0.R /* Rx BDR 17 base address register 0 */
#define NETC__ENETC0_SI2_RB17BAR1 NETC__ENETC0_SI2.BDR_NUM[17].RBBAR1.R /* Rx BDR 17 base address register 1 */
#define NETC__ENETC0_SI2_RB17PIR NETC__ENETC0_SI2.BDR_NUM[17].RBPIR.R /* Rx BDR 17 producer index register */
#define NETC__ENETC0_SI2_RB17LENR NETC__ENETC0_SI2.BDR_NUM[17].RBLENR.R /* Rx BDR 17 length register */
#define NETC__ENETC0_SI2_RB17DCR NETC__ENETC0_SI2.BDR_NUM[17].RBDCR.R /* Rx BDR 17 drop count register */
#define NETC__ENETC0_SI2_RB17IER NETC__ENETC0_SI2.BDR_NUM[17].RBIER.R /* Rx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI2_RB17IDR NETC__ENETC0_SI2.BDR_NUM[17].RBIDR.R /* Rx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI2_RB17ICR0 NETC__ENETC0_SI2.BDR_NUM[17].RBICR0.R /* Rx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI2_RB17ICR1 NETC__ENETC0_SI2.BDR_NUM[17].RBICR1.R /* Rx BDR 17 interrupt coalescing register 1 */

/* NETC_F3_SI3 */
#define NETC__ENETC0_SI3_SIMR NETC__ENETC0_SI3.SIMR.R      /* Station interface mode register */
#define NETC__ENETC0_SI3_SISR NETC__ENETC0_SI3.SISR.R      /* Station interface status register */
#define NETC__ENETC0_SI3_SICTR0 NETC__ENETC0_SI3.SICTR0.R  /* Station interface current time register 0 */
#define NETC__ENETC0_SI3_SICTR1 NETC__ENETC0_SI3.SICTR1.R  /* Station interface current time register 1 */
#define NETC__ENETC0_SI3_SIPCAPR0 NETC__ENETC0_SI3.SIPCAPR0.R /* Station interface port capability register 0 */
#define NETC__ENETC0_SI3_SIPCAPR1 NETC__ENETC0_SI3.SIPCAPR1.R /* Station interface port capability register 1 */
#define NETC__ENETC0_SI3_SIRBGCR NETC__ENETC0_SI3.SIRBGCR.R /* Station interface receive BDR group control register */
#define NETC__ENETC0_SI3_SIBCAR NETC__ENETC0_SI3.SIBCAR.R  /* Station interface buffer cache attribute register */
#define NETC__ENETC0_SI3_SICCAR NETC__ENETC0_SI3.SICCAR.R  /* Station interface command cache attribute register */
#define NETC__ENETC0_SI3_SIPMAR0 NETC__ENETC0_SI3.SIPMAR0.R /* Station interface primary MAC address register 0 */
#define NETC__ENETC0_SI3_SIPMAR1 NETC__ENETC0_SI3.SIPMAR1.R /* Station interface primary MAC address register 1 */
#define NETC__ENETC0_SI3_SICVLANR1 NETC__ENETC0_SI3.SICVLANR1.R /* Station interface custom VLAN register 1 */
#define NETC__ENETC0_SI3_SICVLANR2 NETC__ENETC0_SI3.SICVLANR2.R /* Station interface custom VLAN register 2 */
#define NETC__ENETC0_SI3_SIVLANIPVMR0 NETC__ENETC0_SI3.SIVLANIPVMR0.R /* Station interface VLAN to IPV mapping register 0 */
#define NETC__ENETC0_SI3_SIVLANIPVMR1 NETC__ENETC0_SI3.SIVLANIPVMR1.R /* Station interface VLAN to IPV mapping register 1 */
#define NETC__ENETC0_SI3_SIIPVBDRMR0 NETC__ENETC0_SI3.SIIPVBDRMR0.R /* Station interface IPV to ring mapping register */
#define NETC__ENETC0_SI3_VSIMSGSR NETC__ENETC0_SI3.VSI_A.VSIMSGSR.R /* Virtual station interface message send register */
#define NETC__ENETC0_SI3_VSIMSGRR NETC__ENETC0_SI3.VSI_A.VSIMSGRR.R /* Virtual station interface message receive register */
#define NETC__ENETC0_SI3_VSIMSGSNDAR0 NETC__ENETC0_SI3.VSI_A.VSIMSGSNDAR0.R /* Virtual station interface message send register 0 */
#define NETC__ENETC0_SI3_VSIMSGSNDAR1 NETC__ENETC0_SI3.VSI_A.VSIMSGSNDAR1.R /* Virtual station interface message send address register 1 */
#define NETC__ENETC0_SI3_SIROCT0 NETC__ENETC0_SI3.SIROCT0.R /* Station interface receive octets counter (ifInOctets) 0 */
#define NETC__ENETC0_SI3_SIROCT1 NETC__ENETC0_SI3.SIROCT1.R /* Station interface receive octets counter (ifInOctets) 1 */
#define NETC__ENETC0_SI3_SIRFRM0 NETC__ENETC0_SI3.SIRFRM0.R /* Station interface receive frame counter (aFrameReceivedOK) 0 */
#define NETC__ENETC0_SI3_SIRFRM1 NETC__ENETC0_SI3.SIRFRM1.R /* Station interface receive frame counter (aFrameReceivedOK) 1 */
#define NETC__ENETC0_SI3_SIRUCA0 NETC__ENETC0_SI3.SIRUCA0.R /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
#define NETC__ENETC0_SI3_SIRUCA1 NETC__ENETC0_SI3.SIRUCA1.R /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
#define NETC__ENETC0_SI3_SIRMCA0 NETC__ENETC0_SI3.SIRMCA0.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
#define NETC__ENETC0_SI3_SIRMCA1 NETC__ENETC0_SI3.SIRMCA1.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
#define NETC__ENETC0_SI3_SITOCT0 NETC__ENETC0_SI3.SITOCT0.R /* Station interface transmit octets counter (ifOutOctets) 0 */
#define NETC__ENETC0_SI3_SITOCT1 NETC__ENETC0_SI3.SITOCT1.R /* Station interface transmit octets counter (ifOutOctets) 1 */
#define NETC__ENETC0_SI3_SITFRM0 NETC__ENETC0_SI3.SITFRM0.R /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
#define NETC__ENETC0_SI3_SITFRM1 NETC__ENETC0_SI3.SITFRM1.R /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
#define NETC__ENETC0_SI3_SITUCA0 NETC__ENETC0_SI3.SITUCA0.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
#define NETC__ENETC0_SI3_SITUCA1 NETC__ENETC0_SI3.SITUCA1.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
#define NETC__ENETC0_SI3_SITMCA0 NETC__ENETC0_SI3.SITMCA0.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
#define NETC__ENETC0_SI3_SITMCA1 NETC__ENETC0_SI3.SITMCA1.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
#define NETC__ENETC0_SI3_SIBLPR0 NETC__ENETC0_SI3.SIBLPR[0].R /* Station interface boot loader parameter register 0 */
#define NETC__ENETC0_SI3_SIBLPR1 NETC__ENETC0_SI3.SIBLPR[1].R /* Station interface boot loader parameter register 1 */
#define NETC__ENETC0_SI3_SICBDRMR NETC__ENETC0_SI3.SICBDRMR.R /* Station interface command BDR mode register */
#define NETC__ENETC0_SI3_SICBDRSR NETC__ENETC0_SI3.SICBDRSR.R /* Station interface command BDR status register */
#define NETC__ENETC0_SI3_SICBDRBAR0 NETC__ENETC0_SI3.SICBDRBAR0.R /* Station interface command BDR base address register 0 */
#define NETC__ENETC0_SI3_SICBDRBAR1 NETC__ENETC0_SI3.SICBDRBAR1.R /* Station interface command BDR base address register 1 */
#define NETC__ENETC0_SI3_SICBDRPIR NETC__ENETC0_SI3.SICBDRPIR.R /* Station interface command BDR producer index register */
#define NETC__ENETC0_SI3_SICBDRCIR NETC__ENETC0_SI3.SICBDRCIR.R /* Station interface command BDR consumer index register */
#define NETC__ENETC0_SI3_SICBDRLENR NETC__ENETC0_SI3.SICBDRLENR.R /* Station interface command BDR length register */
#define NETC__ENETC0_SI3_SICBDRIER NETC__ENETC0_SI3.SICBDRIER.R /* Station interface command BDR interrupt enable register */
#define NETC__ENETC0_SI3_SICBDRIDR NETC__ENETC0_SI3.SICBDRIDR.R /* Station interface command BDR interrupt detect register */
#define NETC__ENETC0_SI3_SICAPR0 NETC__ENETC0_SI3.SICAPR0.R /* Station interface capability register 0 */
#define NETC__ENETC0_SI3_SICAPR1 NETC__ENETC0_SI3.SICAPR1.R /* Station interface capability register 1 */
#define NETC__ENETC0_SI3_SICAPR2 NETC__ENETC0_SI3.SICAPR2.R /* Station interface capability register 2 */
#define NETC__ENETC0_SI3_VSIIER NETC__ENETC0_SI3.VSI.VSIIER.R /* Virtual station interface interrupt enable register */
#define NETC__ENETC0_SI3_VSIIDR NETC__ENETC0_SI3.VSI.VSIIDR.R /* Virtual station interface interrupt detect register */
#define NETC__ENETC0_SI3_SITXIDR0 NETC__ENETC0_SI3.SITXIDR0.R /* Station interface transmit interrupt detect register 0 */
#define NETC__ENETC0_SI3_SITXIDR1 NETC__ENETC0_SI3.SITXIDR1.R /* Station interface transmit interrupt detect register 1 */
#define NETC__ENETC0_SI3_SIRXIDR0 NETC__ENETC0_SI3.SIRXIDR0.R /* Station interface receive interrupt detect register 0 */
#define NETC__ENETC0_SI3_SIRXIDR1 NETC__ENETC0_SI3.SIRXIDR1.R /* Station interface receive interrupt detect register 1 */
#define NETC__ENETC0_SI3_SIMSIVR NETC__ENETC0_SI3.SIMSIVR.R /* Station interface MSI-X vector register */
#define NETC__ENETC0_SI3_SICMSIVR NETC__ENETC0_SI3.SICMSIVR.R /* Station interface command MSI-X vector register */
#define NETC__ENETC0_SI3_SITMRIER NETC__ENETC0_SI3.SITMRIER.R /* Station interface timer interrupt enable register */
#define NETC__ENETC0_SI3_SITMRIDR NETC__ENETC0_SI3.SITMRIDR.R /* Station interface timer interrupt detect register */
#define NETC__ENETC0_SI3_SITMRMSIVR NETC__ENETC0_SI3.SITMRMSIVR.R /* Station interface timer MSI-X vector register */
#define NETC__ENETC0_SI3_SIMSITR0VR NETC__ENETC0_SI3.SIMSITRVR[0].R /* Station interface MSI-X transmit ring 0 vector register */
#define NETC__ENETC0_SI3_SIMSITR1VR NETC__ENETC0_SI3.SIMSITRVR[1].R /* Station interface MSI-X transmit ring 1 vector register */
#define NETC__ENETC0_SI3_SIMSITR2VR NETC__ENETC0_SI3.SIMSITRVR[2].R /* Station interface MSI-X transmit ring 2 vector register */
#define NETC__ENETC0_SI3_SIMSITR3VR NETC__ENETC0_SI3.SIMSITRVR[3].R /* Station interface MSI-X transmit ring 3 vector register */
#define NETC__ENETC0_SI3_SIMSITR4VR NETC__ENETC0_SI3.SIMSITRVR[4].R /* Station interface MSI-X transmit ring 4 vector register */
#define NETC__ENETC0_SI3_SIMSITR5VR NETC__ENETC0_SI3.SIMSITRVR[5].R /* Station interface MSI-X transmit ring 5 vector register */
#define NETC__ENETC0_SI3_SIMSITR6VR NETC__ENETC0_SI3.SIMSITRVR[6].R /* Station interface MSI-X transmit ring 6 vector register */
#define NETC__ENETC0_SI3_SIMSITR7VR NETC__ENETC0_SI3.SIMSITRVR[7].R /* Station interface MSI-X transmit ring 7 vector register */
#define NETC__ENETC0_SI3_SIMSITR8VR NETC__ENETC0_SI3.SIMSITRVR[8].R /* Station interface MSI-X transmit ring 8 vector register */
#define NETC__ENETC0_SI3_SIMSITR9VR NETC__ENETC0_SI3.SIMSITRVR[9].R /* Station interface MSI-X transmit ring 9 vector register */
#define NETC__ENETC0_SI3_SIMSITR10VR NETC__ENETC0_SI3.SIMSITRVR[10].R /* Station interface MSI-X transmit ring 10 vector register */
#define NETC__ENETC0_SI3_SIMSITR11VR NETC__ENETC0_SI3.SIMSITRVR[11].R /* Station interface MSI-X transmit ring 11 vector register */
#define NETC__ENETC0_SI3_SIMSITR12VR NETC__ENETC0_SI3.SIMSITRVR[12].R /* Station interface MSI-X transmit ring 12 vector register */
#define NETC__ENETC0_SI3_SIMSITR13VR NETC__ENETC0_SI3.SIMSITRVR[13].R /* Station interface MSI-X transmit ring 13 vector register */
#define NETC__ENETC0_SI3_SIMSITR14VR NETC__ENETC0_SI3.SIMSITRVR[14].R /* Station interface MSI-X transmit ring 14 vector register */
#define NETC__ENETC0_SI3_SIMSITR15VR NETC__ENETC0_SI3.SIMSITRVR[15].R /* Station interface MSI-X transmit ring 15 vector register */
#define NETC__ENETC0_SI3_SIMSITR16VR NETC__ENETC0_SI3.SIMSITRVR[16].R /* Station interface MSI-X transmit ring 16 vector register */
#define NETC__ENETC0_SI3_SIMSITR17VR NETC__ENETC0_SI3.SIMSITRVR[17].R /* Station interface MSI-X transmit ring 17 vector register */
#define NETC__ENETC0_SI3_SIMSIRR0VR NETC__ENETC0_SI3.SIMSIRRVR[0].R /* Station interface MSI-X receive ring 0 vector register */
#define NETC__ENETC0_SI3_SIMSIRR1VR NETC__ENETC0_SI3.SIMSIRRVR[1].R /* Station interface MSI-X receive ring 1 vector register */
#define NETC__ENETC0_SI3_SIMSIRR2VR NETC__ENETC0_SI3.SIMSIRRVR[2].R /* Station interface MSI-X receive ring 2 vector register */
#define NETC__ENETC0_SI3_SIMSIRR3VR NETC__ENETC0_SI3.SIMSIRRVR[3].R /* Station interface MSI-X receive ring 3 vector register */
#define NETC__ENETC0_SI3_SIMSIRR4VR NETC__ENETC0_SI3.SIMSIRRVR[4].R /* Station interface MSI-X receive ring 4 vector register */
#define NETC__ENETC0_SI3_SIMSIRR5VR NETC__ENETC0_SI3.SIMSIRRVR[5].R /* Station interface MSI-X receive ring 5 vector register */
#define NETC__ENETC0_SI3_SIMSIRR6VR NETC__ENETC0_SI3.SIMSIRRVR[6].R /* Station interface MSI-X receive ring 6 vector register */
#define NETC__ENETC0_SI3_SIMSIRR7VR NETC__ENETC0_SI3.SIMSIRRVR[7].R /* Station interface MSI-X receive ring 7 vector register */
#define NETC__ENETC0_SI3_SIMSIRR8VR NETC__ENETC0_SI3.SIMSIRRVR[8].R /* Station interface MSI-X receive ring 8 vector register */
#define NETC__ENETC0_SI3_SIMSIRR9VR NETC__ENETC0_SI3.SIMSIRRVR[9].R /* Station interface MSI-X receive ring 9 vector register */
#define NETC__ENETC0_SI3_SIMSIRR10VR NETC__ENETC0_SI3.SIMSIRRVR[10].R /* Station interface MSI-X receive ring 10 vector register */
#define NETC__ENETC0_SI3_SIMSIRR11VR NETC__ENETC0_SI3.SIMSIRRVR[11].R /* Station interface MSI-X receive ring 11 vector register */
#define NETC__ENETC0_SI3_SIMSIRR12VR NETC__ENETC0_SI3.SIMSIRRVR[12].R /* Station interface MSI-X receive ring 12 vector register */
#define NETC__ENETC0_SI3_SIMSIRR13VR NETC__ENETC0_SI3.SIMSIRRVR[13].R /* Station interface MSI-X receive ring 13 vector register */
#define NETC__ENETC0_SI3_SIMSIRR14VR NETC__ENETC0_SI3.SIMSIRRVR[14].R /* Station interface MSI-X receive ring 14 vector register */
#define NETC__ENETC0_SI3_SIMSIRR15VR NETC__ENETC0_SI3.SIMSIRRVR[15].R /* Station interface MSI-X receive ring 15 vector register */
#define NETC__ENETC0_SI3_SIMSIRR16VR NETC__ENETC0_SI3.SIMSIRRVR[16].R /* Station interface MSI-X receive ring 16 vector register */
#define NETC__ENETC0_SI3_SIMSIRR17VR NETC__ENETC0_SI3.SIMSIRRVR[17].R /* Station interface MSI-X receive ring 17 vector register */
#define NETC__ENETC0_SI3_SICMECR NETC__ENETC0_SI3.SICMECR.R /* Station interface correctable memory error configuration register */
#define NETC__ENETC0_SI3_SICMESR NETC__ENETC0_SI3.SICMESR.R /* Station interface correctable memory error status register */
#define NETC__ENETC0_SI3_SICMECTR NETC__ENETC0_SI3.SICMECTR.R /* Station interface correctable memory error count register */
#define NETC__ENETC0_SI3_SIUPECR NETC__ENETC0_SI3.SIUPECR.R /* Station interface uncorrectable programming error configuration register */
#define NETC__ENETC0_SI3_SIUPESR NETC__ENETC0_SI3.SIUPESR.R /* Station interface uncorrectable programming error status register */
#define NETC__ENETC0_SI3_SIUPECTR NETC__ENETC0_SI3.SIUPECTR.R /* Station interface uncorrectable programming error count register */
#define NETC__ENETC0_SI3_SIUNSBECR NETC__ENETC0_SI3.SIUNSBECR.R /* Station interface uncorrectable non-fatal system bus error configuration register */
#define NETC__ENETC0_SI3_SIUNSBESR NETC__ENETC0_SI3.SIUNSBESR.R /* Station interface uncorrectable non-fatal system bus error status register */
#define NETC__ENETC0_SI3_SIUNSBECTR NETC__ENETC0_SI3.SIUNSBECTR.R /* Station interface uncorrectable non-fatal system bus error count register */
#define NETC__ENETC0_SI3_SIUFSBECR NETC__ENETC0_SI3.SIUFSBECR.R /* Station interface uncorrectable fatal system bus error configuration register */
#define NETC__ENETC0_SI3_SIUFSBESR NETC__ENETC0_SI3.SIUFSBESR.R /* Station interface uncorrectable fatal system bus error status register */
#define NETC__ENETC0_SI3_SIUNMECR NETC__ENETC0_SI3.SIUNMECR.R /* Station interface uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_SI3_SIUNMESR0 NETC__ENETC0_SI3.SIUNMESR0.R /* Station interface uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_SI3_SIUNMESR1 NETC__ENETC0_SI3.SIUNMESR1.R /* Station interface uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_SI3_SIUNMECTR NETC__ENETC0_SI3.SIUNMECTR.R /* Station interface uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_SI3_SIUFMECR NETC__ENETC0_SI3.SIUFMECR.R /* Station interface uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_SI3_SIUFMESR0 NETC__ENETC0_SI3.SIUFMESR0.R /* Station interface uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_SI3_SIUFMESR1 NETC__ENETC0_SI3.SIUFMESR1.R /* Station interface uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_SI3_SIUNIECR NETC__ENETC0_SI3.SIUNIECR.R /* Station interface uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_SI3_SIUNIESR NETC__ENETC0_SI3.SIUNIESR.R /* Station interface uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_SI3_SIUNIECTR NETC__ENETC0_SI3.SIUNIECTR.R /* Station interface uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_SI3_SIUFIECR NETC__ENETC0_SI3.SIUFIECR.R /* Station interface uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_SI3_SIUFIESR NETC__ENETC0_SI3.SIUFIESR.R /* Station interface uncorrectable fatal integrity error status register */
#define NETC__ENETC0_SI3_SIMAFTCAPR NETC__ENETC0_SI3.SIMAFTCAPR.R /* Station interface MAC address filter table capability register */
#define NETC__ENETC0_SI3_SIVFTCAPR NETC__ENETC0_SI3.SIVFTCAPR.R /* Station interface VLAN filter table capability register */
#define NETC__ENETC0_SI3_SIRFSCAPR NETC__ENETC0_SI3.SIRFSCAPR.R /* Station interface RFS capability register */
#define NETC__ENETC0_SI3_TB0MR NETC__ENETC0_SI3.BDR_NUM[0].TBMR.R /* Tx BDR 0 mode register */
#define NETC__ENETC0_SI3_TB0SR NETC__ENETC0_SI3.BDR_NUM[0].TBSR.R /* Tx BDR 0 status register */
#define NETC__ENETC0_SI3_TB0BAR0 NETC__ENETC0_SI3.BDR_NUM[0].TBBAR0.R /* Tx BDR 0 base address register 0 */
#define NETC__ENETC0_SI3_TB0BAR1 NETC__ENETC0_SI3.BDR_NUM[0].TBBAR1.R /* Tx BDR 0 base address register 1 */
#define NETC__ENETC0_SI3_TB0PIR NETC__ENETC0_SI3.BDR_NUM[0].TBPIR.R /* Tx BDR 0 producer index register */
#define NETC__ENETC0_SI3_TB0CIR NETC__ENETC0_SI3.BDR_NUM[0].TBCIR.R /* Tx BDR 0 consumer index register */
#define NETC__ENETC0_SI3_TB0LENR NETC__ENETC0_SI3.BDR_NUM[0].TBLENR.R /* Tx BDR 0 length register */
#define NETC__ENETC0_SI3_TB0IER NETC__ENETC0_SI3.BDR_NUM[0].TBIER.R /* Tx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI3_TB0IDR NETC__ENETC0_SI3.BDR_NUM[0].TBIDR.R /* Tx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI3_TB0ICR0 NETC__ENETC0_SI3.BDR_NUM[0].TBICR0.R /* Tx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB0ICR1 NETC__ENETC0_SI3.BDR_NUM[0].TBICR1.R /* Tx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB0MR NETC__ENETC0_SI3.BDR_NUM[0].RBMR.R /* Rx BDR 0 mode register */
#define NETC__ENETC0_SI3_RB0SR NETC__ENETC0_SI3.BDR_NUM[0].RBSR.R /* Rx BDR 0 status register */
#define NETC__ENETC0_SI3_RB0BSR NETC__ENETC0_SI3.BDR_NUM[0].RBBSR.R /* Rx BDR 0 buffer size register */
#define NETC__ENETC0_SI3_RB0CIR NETC__ENETC0_SI3.BDR_NUM[0].RBCIR.R /* Rx BDR 0 consumer index register */
#define NETC__ENETC0_SI3_RB0BAR0 NETC__ENETC0_SI3.BDR_NUM[0].RBBAR0.R /* Rx BDR 0 base address register 0 */
#define NETC__ENETC0_SI3_RB0BAR1 NETC__ENETC0_SI3.BDR_NUM[0].RBBAR1.R /* Rx BDR 0 base address register 1 */
#define NETC__ENETC0_SI3_RB0PIR NETC__ENETC0_SI3.BDR_NUM[0].RBPIR.R /* Rx BDR 0 producer index register */
#define NETC__ENETC0_SI3_RB0LENR NETC__ENETC0_SI3.BDR_NUM[0].RBLENR.R /* Rx BDR 0 length register */
#define NETC__ENETC0_SI3_RB0DCR NETC__ENETC0_SI3.BDR_NUM[0].RBDCR.R /* Rx BDR 0 drop count register */
#define NETC__ENETC0_SI3_RB0IER NETC__ENETC0_SI3.BDR_NUM[0].RBIER.R /* Rx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI3_RB0IDR NETC__ENETC0_SI3.BDR_NUM[0].RBIDR.R /* Rx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI3_RB0ICR0 NETC__ENETC0_SI3.BDR_NUM[0].RBICR0.R /* Rx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB0ICR1 NETC__ENETC0_SI3.BDR_NUM[0].RBICR1.R /* Rx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB1MR NETC__ENETC0_SI3.BDR_NUM[1].TBMR.R /* Tx BDR 1 mode register */
#define NETC__ENETC0_SI3_TB1SR NETC__ENETC0_SI3.BDR_NUM[1].TBSR.R /* Tx BDR 1 status register */
#define NETC__ENETC0_SI3_TB1BAR0 NETC__ENETC0_SI3.BDR_NUM[1].TBBAR0.R /* Tx BDR 1 base address register 0 */
#define NETC__ENETC0_SI3_TB1BAR1 NETC__ENETC0_SI3.BDR_NUM[1].TBBAR1.R /* Tx BDR 1 base address register 1 */
#define NETC__ENETC0_SI3_TB1PIR NETC__ENETC0_SI3.BDR_NUM[1].TBPIR.R /* Tx BDR 1 producer index register */
#define NETC__ENETC0_SI3_TB1CIR NETC__ENETC0_SI3.BDR_NUM[1].TBCIR.R /* Tx BDR 1 consumer index register */
#define NETC__ENETC0_SI3_TB1LENR NETC__ENETC0_SI3.BDR_NUM[1].TBLENR.R /* Tx BDR 1 length register */
#define NETC__ENETC0_SI3_TB1IER NETC__ENETC0_SI3.BDR_NUM[1].TBIER.R /* Tx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI3_TB1IDR NETC__ENETC0_SI3.BDR_NUM[1].TBIDR.R /* Tx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI3_TB1ICR0 NETC__ENETC0_SI3.BDR_NUM[1].TBICR0.R /* Tx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB1ICR1 NETC__ENETC0_SI3.BDR_NUM[1].TBICR1.R /* Tx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB1MR NETC__ENETC0_SI3.BDR_NUM[1].RBMR.R /* Rx BDR 1 mode register */
#define NETC__ENETC0_SI3_RB1SR NETC__ENETC0_SI3.BDR_NUM[1].RBSR.R /* Rx BDR 1 status register */
#define NETC__ENETC0_SI3_RB1BSR NETC__ENETC0_SI3.BDR_NUM[1].RBBSR.R /* Rx BDR 1 buffer size register */
#define NETC__ENETC0_SI3_RB1CIR NETC__ENETC0_SI3.BDR_NUM[1].RBCIR.R /* Rx BDR 1 consumer index register */
#define NETC__ENETC0_SI3_RB1BAR0 NETC__ENETC0_SI3.BDR_NUM[1].RBBAR0.R /* Rx BDR 1 base address register 0 */
#define NETC__ENETC0_SI3_RB1BAR1 NETC__ENETC0_SI3.BDR_NUM[1].RBBAR1.R /* Rx BDR 1 base address register 1 */
#define NETC__ENETC0_SI3_RB1PIR NETC__ENETC0_SI3.BDR_NUM[1].RBPIR.R /* Rx BDR 1 producer index register */
#define NETC__ENETC0_SI3_RB1LENR NETC__ENETC0_SI3.BDR_NUM[1].RBLENR.R /* Rx BDR 1 length register */
#define NETC__ENETC0_SI3_RB1DCR NETC__ENETC0_SI3.BDR_NUM[1].RBDCR.R /* Rx BDR 1 drop count register */
#define NETC__ENETC0_SI3_RB1IER NETC__ENETC0_SI3.BDR_NUM[1].RBIER.R /* Rx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI3_RB1IDR NETC__ENETC0_SI3.BDR_NUM[1].RBIDR.R /* Rx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI3_RB1ICR0 NETC__ENETC0_SI3.BDR_NUM[1].RBICR0.R /* Rx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB1ICR1 NETC__ENETC0_SI3.BDR_NUM[1].RBICR1.R /* Rx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB2MR NETC__ENETC0_SI3.BDR_NUM[2].TBMR.R /* Tx BDR 2 mode register */
#define NETC__ENETC0_SI3_TB2SR NETC__ENETC0_SI3.BDR_NUM[2].TBSR.R /* Tx BDR 2 status register */
#define NETC__ENETC0_SI3_TB2BAR0 NETC__ENETC0_SI3.BDR_NUM[2].TBBAR0.R /* Tx BDR 2 base address register 0 */
#define NETC__ENETC0_SI3_TB2BAR1 NETC__ENETC0_SI3.BDR_NUM[2].TBBAR1.R /* Tx BDR 2 base address register 1 */
#define NETC__ENETC0_SI3_TB2PIR NETC__ENETC0_SI3.BDR_NUM[2].TBPIR.R /* Tx BDR 2 producer index register */
#define NETC__ENETC0_SI3_TB2CIR NETC__ENETC0_SI3.BDR_NUM[2].TBCIR.R /* Tx BDR 2 consumer index register */
#define NETC__ENETC0_SI3_TB2LENR NETC__ENETC0_SI3.BDR_NUM[2].TBLENR.R /* Tx BDR 2 length register */
#define NETC__ENETC0_SI3_TB2IER NETC__ENETC0_SI3.BDR_NUM[2].TBIER.R /* Tx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI3_TB2IDR NETC__ENETC0_SI3.BDR_NUM[2].TBIDR.R /* Tx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI3_TB2ICR0 NETC__ENETC0_SI3.BDR_NUM[2].TBICR0.R /* Tx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB2ICR1 NETC__ENETC0_SI3.BDR_NUM[2].TBICR1.R /* Tx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB2MR NETC__ENETC0_SI3.BDR_NUM[2].RBMR.R /* Rx BDR 2 mode register */
#define NETC__ENETC0_SI3_RB2SR NETC__ENETC0_SI3.BDR_NUM[2].RBSR.R /* Rx BDR 2 status register */
#define NETC__ENETC0_SI3_RB2BSR NETC__ENETC0_SI3.BDR_NUM[2].RBBSR.R /* Rx BDR 2 buffer size register */
#define NETC__ENETC0_SI3_RB2CIR NETC__ENETC0_SI3.BDR_NUM[2].RBCIR.R /* Rx BDR 2 consumer index register */
#define NETC__ENETC0_SI3_RB2BAR0 NETC__ENETC0_SI3.BDR_NUM[2].RBBAR0.R /* Rx BDR 2 base address register 0 */
#define NETC__ENETC0_SI3_RB2BAR1 NETC__ENETC0_SI3.BDR_NUM[2].RBBAR1.R /* Rx BDR 2 base address register 1 */
#define NETC__ENETC0_SI3_RB2PIR NETC__ENETC0_SI3.BDR_NUM[2].RBPIR.R /* Rx BDR 2 producer index register */
#define NETC__ENETC0_SI3_RB2LENR NETC__ENETC0_SI3.BDR_NUM[2].RBLENR.R /* Rx BDR 2 length register */
#define NETC__ENETC0_SI3_RB2DCR NETC__ENETC0_SI3.BDR_NUM[2].RBDCR.R /* Rx BDR 2 drop count register */
#define NETC__ENETC0_SI3_RB2IER NETC__ENETC0_SI3.BDR_NUM[2].RBIER.R /* Rx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI3_RB2IDR NETC__ENETC0_SI3.BDR_NUM[2].RBIDR.R /* Rx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI3_RB2ICR0 NETC__ENETC0_SI3.BDR_NUM[2].RBICR0.R /* Rx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB2ICR1 NETC__ENETC0_SI3.BDR_NUM[2].RBICR1.R /* Rx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB3MR NETC__ENETC0_SI3.BDR_NUM[3].TBMR.R /* Tx BDR 3 mode register */
#define NETC__ENETC0_SI3_TB3SR NETC__ENETC0_SI3.BDR_NUM[3].TBSR.R /* Tx BDR 3 status register */
#define NETC__ENETC0_SI3_TB3BAR0 NETC__ENETC0_SI3.BDR_NUM[3].TBBAR0.R /* Tx BDR 3 base address register 0 */
#define NETC__ENETC0_SI3_TB3BAR1 NETC__ENETC0_SI3.BDR_NUM[3].TBBAR1.R /* Tx BDR 3 base address register 1 */
#define NETC__ENETC0_SI3_TB3PIR NETC__ENETC0_SI3.BDR_NUM[3].TBPIR.R /* Tx BDR 3 producer index register */
#define NETC__ENETC0_SI3_TB3CIR NETC__ENETC0_SI3.BDR_NUM[3].TBCIR.R /* Tx BDR 3 consumer index register */
#define NETC__ENETC0_SI3_TB3LENR NETC__ENETC0_SI3.BDR_NUM[3].TBLENR.R /* Tx BDR 3 length register */
#define NETC__ENETC0_SI3_TB3IER NETC__ENETC0_SI3.BDR_NUM[3].TBIER.R /* Tx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI3_TB3IDR NETC__ENETC0_SI3.BDR_NUM[3].TBIDR.R /* Tx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI3_TB3ICR0 NETC__ENETC0_SI3.BDR_NUM[3].TBICR0.R /* Tx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB3ICR1 NETC__ENETC0_SI3.BDR_NUM[3].TBICR1.R /* Tx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB3MR NETC__ENETC0_SI3.BDR_NUM[3].RBMR.R /* Rx BDR 3 mode register */
#define NETC__ENETC0_SI3_RB3SR NETC__ENETC0_SI3.BDR_NUM[3].RBSR.R /* Rx BDR 3 status register */
#define NETC__ENETC0_SI3_RB3BSR NETC__ENETC0_SI3.BDR_NUM[3].RBBSR.R /* Rx BDR 3 buffer size register */
#define NETC__ENETC0_SI3_RB3CIR NETC__ENETC0_SI3.BDR_NUM[3].RBCIR.R /* Rx BDR 3 consumer index register */
#define NETC__ENETC0_SI3_RB3BAR0 NETC__ENETC0_SI3.BDR_NUM[3].RBBAR0.R /* Rx BDR 3 base address register 0 */
#define NETC__ENETC0_SI3_RB3BAR1 NETC__ENETC0_SI3.BDR_NUM[3].RBBAR1.R /* Rx BDR 3 base address register 1 */
#define NETC__ENETC0_SI3_RB3PIR NETC__ENETC0_SI3.BDR_NUM[3].RBPIR.R /* Rx BDR 3 producer index register */
#define NETC__ENETC0_SI3_RB3LENR NETC__ENETC0_SI3.BDR_NUM[3].RBLENR.R /* Rx BDR 3 length register */
#define NETC__ENETC0_SI3_RB3DCR NETC__ENETC0_SI3.BDR_NUM[3].RBDCR.R /* Rx BDR 3 drop count register */
#define NETC__ENETC0_SI3_RB3IER NETC__ENETC0_SI3.BDR_NUM[3].RBIER.R /* Rx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI3_RB3IDR NETC__ENETC0_SI3.BDR_NUM[3].RBIDR.R /* Rx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI3_RB3ICR0 NETC__ENETC0_SI3.BDR_NUM[3].RBICR0.R /* Rx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB3ICR1 NETC__ENETC0_SI3.BDR_NUM[3].RBICR1.R /* Rx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB4MR NETC__ENETC0_SI3.BDR_NUM[4].TBMR.R /* Tx BDR 4 mode register */
#define NETC__ENETC0_SI3_TB4SR NETC__ENETC0_SI3.BDR_NUM[4].TBSR.R /* Tx BDR 4 status register */
#define NETC__ENETC0_SI3_TB4BAR0 NETC__ENETC0_SI3.BDR_NUM[4].TBBAR0.R /* Tx BDR 4 base address register 0 */
#define NETC__ENETC0_SI3_TB4BAR1 NETC__ENETC0_SI3.BDR_NUM[4].TBBAR1.R /* Tx BDR 4 base address register 1 */
#define NETC__ENETC0_SI3_TB4PIR NETC__ENETC0_SI3.BDR_NUM[4].TBPIR.R /* Tx BDR 4 producer index register */
#define NETC__ENETC0_SI3_TB4CIR NETC__ENETC0_SI3.BDR_NUM[4].TBCIR.R /* Tx BDR 4 consumer index register */
#define NETC__ENETC0_SI3_TB4LENR NETC__ENETC0_SI3.BDR_NUM[4].TBLENR.R /* Tx BDR 4 length register */
#define NETC__ENETC0_SI3_TB4IER NETC__ENETC0_SI3.BDR_NUM[4].TBIER.R /* Tx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI3_TB4IDR NETC__ENETC0_SI3.BDR_NUM[4].TBIDR.R /* Tx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI3_TB4ICR0 NETC__ENETC0_SI3.BDR_NUM[4].TBICR0.R /* Tx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB4ICR1 NETC__ENETC0_SI3.BDR_NUM[4].TBICR1.R /* Tx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB4MR NETC__ENETC0_SI3.BDR_NUM[4].RBMR.R /* Rx BDR 4 mode register */
#define NETC__ENETC0_SI3_RB4SR NETC__ENETC0_SI3.BDR_NUM[4].RBSR.R /* Rx BDR 4 status register */
#define NETC__ENETC0_SI3_RB4BSR NETC__ENETC0_SI3.BDR_NUM[4].RBBSR.R /* Rx BDR 4 buffer size register */
#define NETC__ENETC0_SI3_RB4CIR NETC__ENETC0_SI3.BDR_NUM[4].RBCIR.R /* Rx BDR 4 consumer index register */
#define NETC__ENETC0_SI3_RB4BAR0 NETC__ENETC0_SI3.BDR_NUM[4].RBBAR0.R /* Rx BDR 4 base address register 0 */
#define NETC__ENETC0_SI3_RB4BAR1 NETC__ENETC0_SI3.BDR_NUM[4].RBBAR1.R /* Rx BDR 4 base address register 1 */
#define NETC__ENETC0_SI3_RB4PIR NETC__ENETC0_SI3.BDR_NUM[4].RBPIR.R /* Rx BDR 4 producer index register */
#define NETC__ENETC0_SI3_RB4LENR NETC__ENETC0_SI3.BDR_NUM[4].RBLENR.R /* Rx BDR 4 length register */
#define NETC__ENETC0_SI3_RB4DCR NETC__ENETC0_SI3.BDR_NUM[4].RBDCR.R /* Rx BDR 4 drop count register */
#define NETC__ENETC0_SI3_RB4IER NETC__ENETC0_SI3.BDR_NUM[4].RBIER.R /* Rx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI3_RB4IDR NETC__ENETC0_SI3.BDR_NUM[4].RBIDR.R /* Rx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI3_RB4ICR0 NETC__ENETC0_SI3.BDR_NUM[4].RBICR0.R /* Rx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB4ICR1 NETC__ENETC0_SI3.BDR_NUM[4].RBICR1.R /* Rx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB5MR NETC__ENETC0_SI3.BDR_NUM[5].TBMR.R /* Tx BDR 5 mode register */
#define NETC__ENETC0_SI3_TB5SR NETC__ENETC0_SI3.BDR_NUM[5].TBSR.R /* Tx BDR 5 status register */
#define NETC__ENETC0_SI3_TB5BAR0 NETC__ENETC0_SI3.BDR_NUM[5].TBBAR0.R /* Tx BDR 5 base address register 0 */
#define NETC__ENETC0_SI3_TB5BAR1 NETC__ENETC0_SI3.BDR_NUM[5].TBBAR1.R /* Tx BDR 5 base address register 1 */
#define NETC__ENETC0_SI3_TB5PIR NETC__ENETC0_SI3.BDR_NUM[5].TBPIR.R /* Tx BDR 5 producer index register */
#define NETC__ENETC0_SI3_TB5CIR NETC__ENETC0_SI3.BDR_NUM[5].TBCIR.R /* Tx BDR 5 consumer index register */
#define NETC__ENETC0_SI3_TB5LENR NETC__ENETC0_SI3.BDR_NUM[5].TBLENR.R /* Tx BDR 5 length register */
#define NETC__ENETC0_SI3_TB5IER NETC__ENETC0_SI3.BDR_NUM[5].TBIER.R /* Tx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI3_TB5IDR NETC__ENETC0_SI3.BDR_NUM[5].TBIDR.R /* Tx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI3_TB5ICR0 NETC__ENETC0_SI3.BDR_NUM[5].TBICR0.R /* Tx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB5ICR1 NETC__ENETC0_SI3.BDR_NUM[5].TBICR1.R /* Tx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB5MR NETC__ENETC0_SI3.BDR_NUM[5].RBMR.R /* Rx BDR 5 mode register */
#define NETC__ENETC0_SI3_RB5SR NETC__ENETC0_SI3.BDR_NUM[5].RBSR.R /* Rx BDR 5 status register */
#define NETC__ENETC0_SI3_RB5BSR NETC__ENETC0_SI3.BDR_NUM[5].RBBSR.R /* Rx BDR 5 buffer size register */
#define NETC__ENETC0_SI3_RB5CIR NETC__ENETC0_SI3.BDR_NUM[5].RBCIR.R /* Rx BDR 5 consumer index register */
#define NETC__ENETC0_SI3_RB5BAR0 NETC__ENETC0_SI3.BDR_NUM[5].RBBAR0.R /* Rx BDR 5 base address register 0 */
#define NETC__ENETC0_SI3_RB5BAR1 NETC__ENETC0_SI3.BDR_NUM[5].RBBAR1.R /* Rx BDR 5 base address register 1 */
#define NETC__ENETC0_SI3_RB5PIR NETC__ENETC0_SI3.BDR_NUM[5].RBPIR.R /* Rx BDR 5 producer index register */
#define NETC__ENETC0_SI3_RB5LENR NETC__ENETC0_SI3.BDR_NUM[5].RBLENR.R /* Rx BDR 5 length register */
#define NETC__ENETC0_SI3_RB5DCR NETC__ENETC0_SI3.BDR_NUM[5].RBDCR.R /* Rx BDR 5 drop count register */
#define NETC__ENETC0_SI3_RB5IER NETC__ENETC0_SI3.BDR_NUM[5].RBIER.R /* Rx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI3_RB5IDR NETC__ENETC0_SI3.BDR_NUM[5].RBIDR.R /* Rx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI3_RB5ICR0 NETC__ENETC0_SI3.BDR_NUM[5].RBICR0.R /* Rx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB5ICR1 NETC__ENETC0_SI3.BDR_NUM[5].RBICR1.R /* Rx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB6MR NETC__ENETC0_SI3.BDR_NUM[6].TBMR.R /* Tx BDR 6 mode register */
#define NETC__ENETC0_SI3_TB6SR NETC__ENETC0_SI3.BDR_NUM[6].TBSR.R /* Tx BDR 6 status register */
#define NETC__ENETC0_SI3_TB6BAR0 NETC__ENETC0_SI3.BDR_NUM[6].TBBAR0.R /* Tx BDR 6 base address register 0 */
#define NETC__ENETC0_SI3_TB6BAR1 NETC__ENETC0_SI3.BDR_NUM[6].TBBAR1.R /* Tx BDR 6 base address register 1 */
#define NETC__ENETC0_SI3_TB6PIR NETC__ENETC0_SI3.BDR_NUM[6].TBPIR.R /* Tx BDR 6 producer index register */
#define NETC__ENETC0_SI3_TB6CIR NETC__ENETC0_SI3.BDR_NUM[6].TBCIR.R /* Tx BDR 6 consumer index register */
#define NETC__ENETC0_SI3_TB6LENR NETC__ENETC0_SI3.BDR_NUM[6].TBLENR.R /* Tx BDR 6 length register */
#define NETC__ENETC0_SI3_TB6IER NETC__ENETC0_SI3.BDR_NUM[6].TBIER.R /* Tx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI3_TB6IDR NETC__ENETC0_SI3.BDR_NUM[6].TBIDR.R /* Tx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI3_TB6ICR0 NETC__ENETC0_SI3.BDR_NUM[6].TBICR0.R /* Tx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB6ICR1 NETC__ENETC0_SI3.BDR_NUM[6].TBICR1.R /* Tx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB6MR NETC__ENETC0_SI3.BDR_NUM[6].RBMR.R /* Rx BDR 6 mode register */
#define NETC__ENETC0_SI3_RB6SR NETC__ENETC0_SI3.BDR_NUM[6].RBSR.R /* Rx BDR 6 status register */
#define NETC__ENETC0_SI3_RB6BSR NETC__ENETC0_SI3.BDR_NUM[6].RBBSR.R /* Rx BDR 6 buffer size register */
#define NETC__ENETC0_SI3_RB6CIR NETC__ENETC0_SI3.BDR_NUM[6].RBCIR.R /* Rx BDR 6 consumer index register */
#define NETC__ENETC0_SI3_RB6BAR0 NETC__ENETC0_SI3.BDR_NUM[6].RBBAR0.R /* Rx BDR 6 base address register 0 */
#define NETC__ENETC0_SI3_RB6BAR1 NETC__ENETC0_SI3.BDR_NUM[6].RBBAR1.R /* Rx BDR 6 base address register 1 */
#define NETC__ENETC0_SI3_RB6PIR NETC__ENETC0_SI3.BDR_NUM[6].RBPIR.R /* Rx BDR 6 producer index register */
#define NETC__ENETC0_SI3_RB6LENR NETC__ENETC0_SI3.BDR_NUM[6].RBLENR.R /* Rx BDR 6 length register */
#define NETC__ENETC0_SI3_RB6DCR NETC__ENETC0_SI3.BDR_NUM[6].RBDCR.R /* Rx BDR 6 drop count register */
#define NETC__ENETC0_SI3_RB6IER NETC__ENETC0_SI3.BDR_NUM[6].RBIER.R /* Rx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI3_RB6IDR NETC__ENETC0_SI3.BDR_NUM[6].RBIDR.R /* Rx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI3_RB6ICR0 NETC__ENETC0_SI3.BDR_NUM[6].RBICR0.R /* Rx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB6ICR1 NETC__ENETC0_SI3.BDR_NUM[6].RBICR1.R /* Rx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB7MR NETC__ENETC0_SI3.BDR_NUM[7].TBMR.R /* Tx BDR 7 mode register */
#define NETC__ENETC0_SI3_TB7SR NETC__ENETC0_SI3.BDR_NUM[7].TBSR.R /* Tx BDR 7 status register */
#define NETC__ENETC0_SI3_TB7BAR0 NETC__ENETC0_SI3.BDR_NUM[7].TBBAR0.R /* Tx BDR 7 base address register 0 */
#define NETC__ENETC0_SI3_TB7BAR1 NETC__ENETC0_SI3.BDR_NUM[7].TBBAR1.R /* Tx BDR 7 base address register 1 */
#define NETC__ENETC0_SI3_TB7PIR NETC__ENETC0_SI3.BDR_NUM[7].TBPIR.R /* Tx BDR 7 producer index register */
#define NETC__ENETC0_SI3_TB7CIR NETC__ENETC0_SI3.BDR_NUM[7].TBCIR.R /* Tx BDR 7 consumer index register */
#define NETC__ENETC0_SI3_TB7LENR NETC__ENETC0_SI3.BDR_NUM[7].TBLENR.R /* Tx BDR 7 length register */
#define NETC__ENETC0_SI3_TB7IER NETC__ENETC0_SI3.BDR_NUM[7].TBIER.R /* Tx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI3_TB7IDR NETC__ENETC0_SI3.BDR_NUM[7].TBIDR.R /* Tx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI3_TB7ICR0 NETC__ENETC0_SI3.BDR_NUM[7].TBICR0.R /* Tx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB7ICR1 NETC__ENETC0_SI3.BDR_NUM[7].TBICR1.R /* Tx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB7MR NETC__ENETC0_SI3.BDR_NUM[7].RBMR.R /* Rx BDR 7 mode register */
#define NETC__ENETC0_SI3_RB7SR NETC__ENETC0_SI3.BDR_NUM[7].RBSR.R /* Rx BDR 7 status register */
#define NETC__ENETC0_SI3_RB7BSR NETC__ENETC0_SI3.BDR_NUM[7].RBBSR.R /* Rx BDR 7 buffer size register */
#define NETC__ENETC0_SI3_RB7CIR NETC__ENETC0_SI3.BDR_NUM[7].RBCIR.R /* Rx BDR 7 consumer index register */
#define NETC__ENETC0_SI3_RB7BAR0 NETC__ENETC0_SI3.BDR_NUM[7].RBBAR0.R /* Rx BDR 7 base address register 0 */
#define NETC__ENETC0_SI3_RB7BAR1 NETC__ENETC0_SI3.BDR_NUM[7].RBBAR1.R /* Rx BDR 7 base address register 1 */
#define NETC__ENETC0_SI3_RB7PIR NETC__ENETC0_SI3.BDR_NUM[7].RBPIR.R /* Rx BDR 7 producer index register */
#define NETC__ENETC0_SI3_RB7LENR NETC__ENETC0_SI3.BDR_NUM[7].RBLENR.R /* Rx BDR 7 length register */
#define NETC__ENETC0_SI3_RB7DCR NETC__ENETC0_SI3.BDR_NUM[7].RBDCR.R /* Rx BDR 7 drop count register */
#define NETC__ENETC0_SI3_RB7IER NETC__ENETC0_SI3.BDR_NUM[7].RBIER.R /* Rx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI3_RB7IDR NETC__ENETC0_SI3.BDR_NUM[7].RBIDR.R /* Rx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI3_RB7ICR0 NETC__ENETC0_SI3.BDR_NUM[7].RBICR0.R /* Rx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB7ICR1 NETC__ENETC0_SI3.BDR_NUM[7].RBICR1.R /* Rx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB8MR NETC__ENETC0_SI3.BDR_NUM[8].TBMR.R /* Tx BDR 8 mode register */
#define NETC__ENETC0_SI3_TB8SR NETC__ENETC0_SI3.BDR_NUM[8].TBSR.R /* Tx BDR 8 status register */
#define NETC__ENETC0_SI3_TB8BAR0 NETC__ENETC0_SI3.BDR_NUM[8].TBBAR0.R /* Tx BDR 8 base address register 0 */
#define NETC__ENETC0_SI3_TB8BAR1 NETC__ENETC0_SI3.BDR_NUM[8].TBBAR1.R /* Tx BDR 8 base address register 1 */
#define NETC__ENETC0_SI3_TB8PIR NETC__ENETC0_SI3.BDR_NUM[8].TBPIR.R /* Tx BDR 8 producer index register */
#define NETC__ENETC0_SI3_TB8CIR NETC__ENETC0_SI3.BDR_NUM[8].TBCIR.R /* Tx BDR 8 consumer index register */
#define NETC__ENETC0_SI3_TB8LENR NETC__ENETC0_SI3.BDR_NUM[8].TBLENR.R /* Tx BDR 8 length register */
#define NETC__ENETC0_SI3_TB8IER NETC__ENETC0_SI3.BDR_NUM[8].TBIER.R /* Tx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI3_TB8IDR NETC__ENETC0_SI3.BDR_NUM[8].TBIDR.R /* Tx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI3_TB8ICR0 NETC__ENETC0_SI3.BDR_NUM[8].TBICR0.R /* Tx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB8ICR1 NETC__ENETC0_SI3.BDR_NUM[8].TBICR1.R /* Tx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB8MR NETC__ENETC0_SI3.BDR_NUM[8].RBMR.R /* Rx BDR 8 mode register */
#define NETC__ENETC0_SI3_RB8SR NETC__ENETC0_SI3.BDR_NUM[8].RBSR.R /* Rx BDR 8 status register */
#define NETC__ENETC0_SI3_RB8BSR NETC__ENETC0_SI3.BDR_NUM[8].RBBSR.R /* Rx BDR 8 buffer size register */
#define NETC__ENETC0_SI3_RB8CIR NETC__ENETC0_SI3.BDR_NUM[8].RBCIR.R /* Rx BDR 8 consumer index register */
#define NETC__ENETC0_SI3_RB8BAR0 NETC__ENETC0_SI3.BDR_NUM[8].RBBAR0.R /* Rx BDR 8 base address register 0 */
#define NETC__ENETC0_SI3_RB8BAR1 NETC__ENETC0_SI3.BDR_NUM[8].RBBAR1.R /* Rx BDR 8 base address register 1 */
#define NETC__ENETC0_SI3_RB8PIR NETC__ENETC0_SI3.BDR_NUM[8].RBPIR.R /* Rx BDR 8 producer index register */
#define NETC__ENETC0_SI3_RB8LENR NETC__ENETC0_SI3.BDR_NUM[8].RBLENR.R /* Rx BDR 8 length register */
#define NETC__ENETC0_SI3_RB8DCR NETC__ENETC0_SI3.BDR_NUM[8].RBDCR.R /* Rx BDR 8 drop count register */
#define NETC__ENETC0_SI3_RB8IER NETC__ENETC0_SI3.BDR_NUM[8].RBIER.R /* Rx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI3_RB8IDR NETC__ENETC0_SI3.BDR_NUM[8].RBIDR.R /* Rx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI3_RB8ICR0 NETC__ENETC0_SI3.BDR_NUM[8].RBICR0.R /* Rx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB8ICR1 NETC__ENETC0_SI3.BDR_NUM[8].RBICR1.R /* Rx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB9MR NETC__ENETC0_SI3.BDR_NUM[9].TBMR.R /* Tx BDR 9 mode register */
#define NETC__ENETC0_SI3_TB9SR NETC__ENETC0_SI3.BDR_NUM[9].TBSR.R /* Tx BDR 9 status register */
#define NETC__ENETC0_SI3_TB9BAR0 NETC__ENETC0_SI3.BDR_NUM[9].TBBAR0.R /* Tx BDR 9 base address register 0 */
#define NETC__ENETC0_SI3_TB9BAR1 NETC__ENETC0_SI3.BDR_NUM[9].TBBAR1.R /* Tx BDR 9 base address register 1 */
#define NETC__ENETC0_SI3_TB9PIR NETC__ENETC0_SI3.BDR_NUM[9].TBPIR.R /* Tx BDR 9 producer index register */
#define NETC__ENETC0_SI3_TB9CIR NETC__ENETC0_SI3.BDR_NUM[9].TBCIR.R /* Tx BDR 9 consumer index register */
#define NETC__ENETC0_SI3_TB9LENR NETC__ENETC0_SI3.BDR_NUM[9].TBLENR.R /* Tx BDR 9 length register */
#define NETC__ENETC0_SI3_TB9IER NETC__ENETC0_SI3.BDR_NUM[9].TBIER.R /* Tx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI3_TB9IDR NETC__ENETC0_SI3.BDR_NUM[9].TBIDR.R /* Tx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI3_TB9ICR0 NETC__ENETC0_SI3.BDR_NUM[9].TBICR0.R /* Tx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB9ICR1 NETC__ENETC0_SI3.BDR_NUM[9].TBICR1.R /* Tx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB9MR NETC__ENETC0_SI3.BDR_NUM[9].RBMR.R /* Rx BDR 9 mode register */
#define NETC__ENETC0_SI3_RB9SR NETC__ENETC0_SI3.BDR_NUM[9].RBSR.R /* Rx BDR 9 status register */
#define NETC__ENETC0_SI3_RB9BSR NETC__ENETC0_SI3.BDR_NUM[9].RBBSR.R /* Rx BDR 9 buffer size register */
#define NETC__ENETC0_SI3_RB9CIR NETC__ENETC0_SI3.BDR_NUM[9].RBCIR.R /* Rx BDR 9 consumer index register */
#define NETC__ENETC0_SI3_RB9BAR0 NETC__ENETC0_SI3.BDR_NUM[9].RBBAR0.R /* Rx BDR 9 base address register 0 */
#define NETC__ENETC0_SI3_RB9BAR1 NETC__ENETC0_SI3.BDR_NUM[9].RBBAR1.R /* Rx BDR 9 base address register 1 */
#define NETC__ENETC0_SI3_RB9PIR NETC__ENETC0_SI3.BDR_NUM[9].RBPIR.R /* Rx BDR 9 producer index register */
#define NETC__ENETC0_SI3_RB9LENR NETC__ENETC0_SI3.BDR_NUM[9].RBLENR.R /* Rx BDR 9 length register */
#define NETC__ENETC0_SI3_RB9DCR NETC__ENETC0_SI3.BDR_NUM[9].RBDCR.R /* Rx BDR 9 drop count register */
#define NETC__ENETC0_SI3_RB9IER NETC__ENETC0_SI3.BDR_NUM[9].RBIER.R /* Rx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI3_RB9IDR NETC__ENETC0_SI3.BDR_NUM[9].RBIDR.R /* Rx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI3_RB9ICR0 NETC__ENETC0_SI3.BDR_NUM[9].RBICR0.R /* Rx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB9ICR1 NETC__ENETC0_SI3.BDR_NUM[9].RBICR1.R /* Rx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB10MR NETC__ENETC0_SI3.BDR_NUM[10].TBMR.R /* Tx BDR 10 mode register */
#define NETC__ENETC0_SI3_TB10SR NETC__ENETC0_SI3.BDR_NUM[10].TBSR.R /* Tx BDR 10 status register */
#define NETC__ENETC0_SI3_TB10BAR0 NETC__ENETC0_SI3.BDR_NUM[10].TBBAR0.R /* Tx BDR 10 base address register 0 */
#define NETC__ENETC0_SI3_TB10BAR1 NETC__ENETC0_SI3.BDR_NUM[10].TBBAR1.R /* Tx BDR 10 base address register 1 */
#define NETC__ENETC0_SI3_TB10PIR NETC__ENETC0_SI3.BDR_NUM[10].TBPIR.R /* Tx BDR 10 producer index register */
#define NETC__ENETC0_SI3_TB10CIR NETC__ENETC0_SI3.BDR_NUM[10].TBCIR.R /* Tx BDR 10 consumer index register */
#define NETC__ENETC0_SI3_TB10LENR NETC__ENETC0_SI3.BDR_NUM[10].TBLENR.R /* Tx BDR 10 length register */
#define NETC__ENETC0_SI3_TB10IER NETC__ENETC0_SI3.BDR_NUM[10].TBIER.R /* Tx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI3_TB10IDR NETC__ENETC0_SI3.BDR_NUM[10].TBIDR.R /* Tx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI3_TB10ICR0 NETC__ENETC0_SI3.BDR_NUM[10].TBICR0.R /* Tx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB10ICR1 NETC__ENETC0_SI3.BDR_NUM[10].TBICR1.R /* Tx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB10MR NETC__ENETC0_SI3.BDR_NUM[10].RBMR.R /* Rx BDR 10 mode register */
#define NETC__ENETC0_SI3_RB10SR NETC__ENETC0_SI3.BDR_NUM[10].RBSR.R /* Rx BDR 10 status register */
#define NETC__ENETC0_SI3_RB10BSR NETC__ENETC0_SI3.BDR_NUM[10].RBBSR.R /* Rx BDR 10 buffer size register */
#define NETC__ENETC0_SI3_RB10CIR NETC__ENETC0_SI3.BDR_NUM[10].RBCIR.R /* Rx BDR 10 consumer index register */
#define NETC__ENETC0_SI3_RB10BAR0 NETC__ENETC0_SI3.BDR_NUM[10].RBBAR0.R /* Rx BDR 10 base address register 0 */
#define NETC__ENETC0_SI3_RB10BAR1 NETC__ENETC0_SI3.BDR_NUM[10].RBBAR1.R /* Rx BDR 10 base address register 1 */
#define NETC__ENETC0_SI3_RB10PIR NETC__ENETC0_SI3.BDR_NUM[10].RBPIR.R /* Rx BDR 10 producer index register */
#define NETC__ENETC0_SI3_RB10LENR NETC__ENETC0_SI3.BDR_NUM[10].RBLENR.R /* Rx BDR 10 length register */
#define NETC__ENETC0_SI3_RB10DCR NETC__ENETC0_SI3.BDR_NUM[10].RBDCR.R /* Rx BDR 10 drop count register */
#define NETC__ENETC0_SI3_RB10IER NETC__ENETC0_SI3.BDR_NUM[10].RBIER.R /* Rx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI3_RB10IDR NETC__ENETC0_SI3.BDR_NUM[10].RBIDR.R /* Rx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI3_RB10ICR0 NETC__ENETC0_SI3.BDR_NUM[10].RBICR0.R /* Rx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB10ICR1 NETC__ENETC0_SI3.BDR_NUM[10].RBICR1.R /* Rx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB11MR NETC__ENETC0_SI3.BDR_NUM[11].TBMR.R /* Tx BDR 11 mode register */
#define NETC__ENETC0_SI3_TB11SR NETC__ENETC0_SI3.BDR_NUM[11].TBSR.R /* Tx BDR 11 status register */
#define NETC__ENETC0_SI3_TB11BAR0 NETC__ENETC0_SI3.BDR_NUM[11].TBBAR0.R /* Tx BDR 11 base address register 0 */
#define NETC__ENETC0_SI3_TB11BAR1 NETC__ENETC0_SI3.BDR_NUM[11].TBBAR1.R /* Tx BDR 11 base address register 1 */
#define NETC__ENETC0_SI3_TB11PIR NETC__ENETC0_SI3.BDR_NUM[11].TBPIR.R /* Tx BDR 11 producer index register */
#define NETC__ENETC0_SI3_TB11CIR NETC__ENETC0_SI3.BDR_NUM[11].TBCIR.R /* Tx BDR 11 consumer index register */
#define NETC__ENETC0_SI3_TB11LENR NETC__ENETC0_SI3.BDR_NUM[11].TBLENR.R /* Tx BDR 11 length register */
#define NETC__ENETC0_SI3_TB11IER NETC__ENETC0_SI3.BDR_NUM[11].TBIER.R /* Tx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI3_TB11IDR NETC__ENETC0_SI3.BDR_NUM[11].TBIDR.R /* Tx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI3_TB11ICR0 NETC__ENETC0_SI3.BDR_NUM[11].TBICR0.R /* Tx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB11ICR1 NETC__ENETC0_SI3.BDR_NUM[11].TBICR1.R /* Tx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB11MR NETC__ENETC0_SI3.BDR_NUM[11].RBMR.R /* Rx BDR 11 mode register */
#define NETC__ENETC0_SI3_RB11SR NETC__ENETC0_SI3.BDR_NUM[11].RBSR.R /* Rx BDR 11 status register */
#define NETC__ENETC0_SI3_RB11BSR NETC__ENETC0_SI3.BDR_NUM[11].RBBSR.R /* Rx BDR 11 buffer size register */
#define NETC__ENETC0_SI3_RB11CIR NETC__ENETC0_SI3.BDR_NUM[11].RBCIR.R /* Rx BDR 11 consumer index register */
#define NETC__ENETC0_SI3_RB11BAR0 NETC__ENETC0_SI3.BDR_NUM[11].RBBAR0.R /* Rx BDR 11 base address register 0 */
#define NETC__ENETC0_SI3_RB11BAR1 NETC__ENETC0_SI3.BDR_NUM[11].RBBAR1.R /* Rx BDR 11 base address register 1 */
#define NETC__ENETC0_SI3_RB11PIR NETC__ENETC0_SI3.BDR_NUM[11].RBPIR.R /* Rx BDR 11 producer index register */
#define NETC__ENETC0_SI3_RB11LENR NETC__ENETC0_SI3.BDR_NUM[11].RBLENR.R /* Rx BDR 11 length register */
#define NETC__ENETC0_SI3_RB11DCR NETC__ENETC0_SI3.BDR_NUM[11].RBDCR.R /* Rx BDR 11 drop count register */
#define NETC__ENETC0_SI3_RB11IER NETC__ENETC0_SI3.BDR_NUM[11].RBIER.R /* Rx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI3_RB11IDR NETC__ENETC0_SI3.BDR_NUM[11].RBIDR.R /* Rx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI3_RB11ICR0 NETC__ENETC0_SI3.BDR_NUM[11].RBICR0.R /* Rx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB11ICR1 NETC__ENETC0_SI3.BDR_NUM[11].RBICR1.R /* Rx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB12MR NETC__ENETC0_SI3.BDR_NUM[12].TBMR.R /* Tx BDR 12 mode register */
#define NETC__ENETC0_SI3_TB12SR NETC__ENETC0_SI3.BDR_NUM[12].TBSR.R /* Tx BDR 12 status register */
#define NETC__ENETC0_SI3_TB12BAR0 NETC__ENETC0_SI3.BDR_NUM[12].TBBAR0.R /* Tx BDR 12 base address register 0 */
#define NETC__ENETC0_SI3_TB12BAR1 NETC__ENETC0_SI3.BDR_NUM[12].TBBAR1.R /* Tx BDR 12 base address register 1 */
#define NETC__ENETC0_SI3_TB12PIR NETC__ENETC0_SI3.BDR_NUM[12].TBPIR.R /* Tx BDR 12 producer index register */
#define NETC__ENETC0_SI3_TB12CIR NETC__ENETC0_SI3.BDR_NUM[12].TBCIR.R /* Tx BDR 12 consumer index register */
#define NETC__ENETC0_SI3_TB12LENR NETC__ENETC0_SI3.BDR_NUM[12].TBLENR.R /* Tx BDR 12 length register */
#define NETC__ENETC0_SI3_TB12IER NETC__ENETC0_SI3.BDR_NUM[12].TBIER.R /* Tx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI3_TB12IDR NETC__ENETC0_SI3.BDR_NUM[12].TBIDR.R /* Tx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI3_TB12ICR0 NETC__ENETC0_SI3.BDR_NUM[12].TBICR0.R /* Tx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB12ICR1 NETC__ENETC0_SI3.BDR_NUM[12].TBICR1.R /* Tx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB12MR NETC__ENETC0_SI3.BDR_NUM[12].RBMR.R /* Rx BDR 12 mode register */
#define NETC__ENETC0_SI3_RB12SR NETC__ENETC0_SI3.BDR_NUM[12].RBSR.R /* Rx BDR 12 status register */
#define NETC__ENETC0_SI3_RB12BSR NETC__ENETC0_SI3.BDR_NUM[12].RBBSR.R /* Rx BDR 12 buffer size register */
#define NETC__ENETC0_SI3_RB12CIR NETC__ENETC0_SI3.BDR_NUM[12].RBCIR.R /* Rx BDR 12 consumer index register */
#define NETC__ENETC0_SI3_RB12BAR0 NETC__ENETC0_SI3.BDR_NUM[12].RBBAR0.R /* Rx BDR 12 base address register 0 */
#define NETC__ENETC0_SI3_RB12BAR1 NETC__ENETC0_SI3.BDR_NUM[12].RBBAR1.R /* Rx BDR 12 base address register 1 */
#define NETC__ENETC0_SI3_RB12PIR NETC__ENETC0_SI3.BDR_NUM[12].RBPIR.R /* Rx BDR 12 producer index register */
#define NETC__ENETC0_SI3_RB12LENR NETC__ENETC0_SI3.BDR_NUM[12].RBLENR.R /* Rx BDR 12 length register */
#define NETC__ENETC0_SI3_RB12DCR NETC__ENETC0_SI3.BDR_NUM[12].RBDCR.R /* Rx BDR 12 drop count register */
#define NETC__ENETC0_SI3_RB12IER NETC__ENETC0_SI3.BDR_NUM[12].RBIER.R /* Rx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI3_RB12IDR NETC__ENETC0_SI3.BDR_NUM[12].RBIDR.R /* Rx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI3_RB12ICR0 NETC__ENETC0_SI3.BDR_NUM[12].RBICR0.R /* Rx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB12ICR1 NETC__ENETC0_SI3.BDR_NUM[12].RBICR1.R /* Rx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB13MR NETC__ENETC0_SI3.BDR_NUM[13].TBMR.R /* Tx BDR 13 mode register */
#define NETC__ENETC0_SI3_TB13SR NETC__ENETC0_SI3.BDR_NUM[13].TBSR.R /* Tx BDR 13 status register */
#define NETC__ENETC0_SI3_TB13BAR0 NETC__ENETC0_SI3.BDR_NUM[13].TBBAR0.R /* Tx BDR 13 base address register 0 */
#define NETC__ENETC0_SI3_TB13BAR1 NETC__ENETC0_SI3.BDR_NUM[13].TBBAR1.R /* Tx BDR 13 base address register 1 */
#define NETC__ENETC0_SI3_TB13PIR NETC__ENETC0_SI3.BDR_NUM[13].TBPIR.R /* Tx BDR 13 producer index register */
#define NETC__ENETC0_SI3_TB13CIR NETC__ENETC0_SI3.BDR_NUM[13].TBCIR.R /* Tx BDR 13 consumer index register */
#define NETC__ENETC0_SI3_TB13LENR NETC__ENETC0_SI3.BDR_NUM[13].TBLENR.R /* Tx BDR 13 length register */
#define NETC__ENETC0_SI3_TB13IER NETC__ENETC0_SI3.BDR_NUM[13].TBIER.R /* Tx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI3_TB13IDR NETC__ENETC0_SI3.BDR_NUM[13].TBIDR.R /* Tx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI3_TB13ICR0 NETC__ENETC0_SI3.BDR_NUM[13].TBICR0.R /* Tx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB13ICR1 NETC__ENETC0_SI3.BDR_NUM[13].TBICR1.R /* Tx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB13MR NETC__ENETC0_SI3.BDR_NUM[13].RBMR.R /* Rx BDR 13 mode register */
#define NETC__ENETC0_SI3_RB13SR NETC__ENETC0_SI3.BDR_NUM[13].RBSR.R /* Rx BDR 13 status register */
#define NETC__ENETC0_SI3_RB13BSR NETC__ENETC0_SI3.BDR_NUM[13].RBBSR.R /* Rx BDR 13 buffer size register */
#define NETC__ENETC0_SI3_RB13CIR NETC__ENETC0_SI3.BDR_NUM[13].RBCIR.R /* Rx BDR 13 consumer index register */
#define NETC__ENETC0_SI3_RB13BAR0 NETC__ENETC0_SI3.BDR_NUM[13].RBBAR0.R /* Rx BDR 13 base address register 0 */
#define NETC__ENETC0_SI3_RB13BAR1 NETC__ENETC0_SI3.BDR_NUM[13].RBBAR1.R /* Rx BDR 13 base address register 1 */
#define NETC__ENETC0_SI3_RB13PIR NETC__ENETC0_SI3.BDR_NUM[13].RBPIR.R /* Rx BDR 13 producer index register */
#define NETC__ENETC0_SI3_RB13LENR NETC__ENETC0_SI3.BDR_NUM[13].RBLENR.R /* Rx BDR 13 length register */
#define NETC__ENETC0_SI3_RB13DCR NETC__ENETC0_SI3.BDR_NUM[13].RBDCR.R /* Rx BDR 13 drop count register */
#define NETC__ENETC0_SI3_RB13IER NETC__ENETC0_SI3.BDR_NUM[13].RBIER.R /* Rx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI3_RB13IDR NETC__ENETC0_SI3.BDR_NUM[13].RBIDR.R /* Rx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI3_RB13ICR0 NETC__ENETC0_SI3.BDR_NUM[13].RBICR0.R /* Rx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB13ICR1 NETC__ENETC0_SI3.BDR_NUM[13].RBICR1.R /* Rx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB14MR NETC__ENETC0_SI3.BDR_NUM[14].TBMR.R /* Tx BDR 14 mode register */
#define NETC__ENETC0_SI3_TB14SR NETC__ENETC0_SI3.BDR_NUM[14].TBSR.R /* Tx BDR 14 status register */
#define NETC__ENETC0_SI3_TB14BAR0 NETC__ENETC0_SI3.BDR_NUM[14].TBBAR0.R /* Tx BDR 14 base address register 0 */
#define NETC__ENETC0_SI3_TB14BAR1 NETC__ENETC0_SI3.BDR_NUM[14].TBBAR1.R /* Tx BDR 14 base address register 1 */
#define NETC__ENETC0_SI3_TB14PIR NETC__ENETC0_SI3.BDR_NUM[14].TBPIR.R /* Tx BDR 14 producer index register */
#define NETC__ENETC0_SI3_TB14CIR NETC__ENETC0_SI3.BDR_NUM[14].TBCIR.R /* Tx BDR 14 consumer index register */
#define NETC__ENETC0_SI3_TB14LENR NETC__ENETC0_SI3.BDR_NUM[14].TBLENR.R /* Tx BDR 14 length register */
#define NETC__ENETC0_SI3_TB14IER NETC__ENETC0_SI3.BDR_NUM[14].TBIER.R /* Tx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI3_TB14IDR NETC__ENETC0_SI3.BDR_NUM[14].TBIDR.R /* Tx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI3_TB14ICR0 NETC__ENETC0_SI3.BDR_NUM[14].TBICR0.R /* Tx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB14ICR1 NETC__ENETC0_SI3.BDR_NUM[14].TBICR1.R /* Tx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB14MR NETC__ENETC0_SI3.BDR_NUM[14].RBMR.R /* Rx BDR 14 mode register */
#define NETC__ENETC0_SI3_RB14SR NETC__ENETC0_SI3.BDR_NUM[14].RBSR.R /* Rx BDR 14 status register */
#define NETC__ENETC0_SI3_RB14BSR NETC__ENETC0_SI3.BDR_NUM[14].RBBSR.R /* Rx BDR 14 buffer size register */
#define NETC__ENETC0_SI3_RB14CIR NETC__ENETC0_SI3.BDR_NUM[14].RBCIR.R /* Rx BDR 14 consumer index register */
#define NETC__ENETC0_SI3_RB14BAR0 NETC__ENETC0_SI3.BDR_NUM[14].RBBAR0.R /* Rx BDR 14 base address register 0 */
#define NETC__ENETC0_SI3_RB14BAR1 NETC__ENETC0_SI3.BDR_NUM[14].RBBAR1.R /* Rx BDR 14 base address register 1 */
#define NETC__ENETC0_SI3_RB14PIR NETC__ENETC0_SI3.BDR_NUM[14].RBPIR.R /* Rx BDR 14 producer index register */
#define NETC__ENETC0_SI3_RB14LENR NETC__ENETC0_SI3.BDR_NUM[14].RBLENR.R /* Rx BDR 14 length register */
#define NETC__ENETC0_SI3_RB14DCR NETC__ENETC0_SI3.BDR_NUM[14].RBDCR.R /* Rx BDR 14 drop count register */
#define NETC__ENETC0_SI3_RB14IER NETC__ENETC0_SI3.BDR_NUM[14].RBIER.R /* Rx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI3_RB14IDR NETC__ENETC0_SI3.BDR_NUM[14].RBIDR.R /* Rx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI3_RB14ICR0 NETC__ENETC0_SI3.BDR_NUM[14].RBICR0.R /* Rx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB14ICR1 NETC__ENETC0_SI3.BDR_NUM[14].RBICR1.R /* Rx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB15MR NETC__ENETC0_SI3.BDR_NUM[15].TBMR.R /* Tx BDR 15 mode register */
#define NETC__ENETC0_SI3_TB15SR NETC__ENETC0_SI3.BDR_NUM[15].TBSR.R /* Tx BDR 15 status register */
#define NETC__ENETC0_SI3_TB15BAR0 NETC__ENETC0_SI3.BDR_NUM[15].TBBAR0.R /* Tx BDR 15 base address register 0 */
#define NETC__ENETC0_SI3_TB15BAR1 NETC__ENETC0_SI3.BDR_NUM[15].TBBAR1.R /* Tx BDR 15 base address register 1 */
#define NETC__ENETC0_SI3_TB15PIR NETC__ENETC0_SI3.BDR_NUM[15].TBPIR.R /* Tx BDR 15 producer index register */
#define NETC__ENETC0_SI3_TB15CIR NETC__ENETC0_SI3.BDR_NUM[15].TBCIR.R /* Tx BDR 15 consumer index register */
#define NETC__ENETC0_SI3_TB15LENR NETC__ENETC0_SI3.BDR_NUM[15].TBLENR.R /* Tx BDR 15 length register */
#define NETC__ENETC0_SI3_TB15IER NETC__ENETC0_SI3.BDR_NUM[15].TBIER.R /* Tx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI3_TB15IDR NETC__ENETC0_SI3.BDR_NUM[15].TBIDR.R /* Tx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI3_TB15ICR0 NETC__ENETC0_SI3.BDR_NUM[15].TBICR0.R /* Tx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB15ICR1 NETC__ENETC0_SI3.BDR_NUM[15].TBICR1.R /* Tx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB15MR NETC__ENETC0_SI3.BDR_NUM[15].RBMR.R /* Rx BDR 15 mode register */
#define NETC__ENETC0_SI3_RB15SR NETC__ENETC0_SI3.BDR_NUM[15].RBSR.R /* Rx BDR 15 status register */
#define NETC__ENETC0_SI3_RB15BSR NETC__ENETC0_SI3.BDR_NUM[15].RBBSR.R /* Rx BDR 15 buffer size register */
#define NETC__ENETC0_SI3_RB15CIR NETC__ENETC0_SI3.BDR_NUM[15].RBCIR.R /* Rx BDR 15 consumer index register */
#define NETC__ENETC0_SI3_RB15BAR0 NETC__ENETC0_SI3.BDR_NUM[15].RBBAR0.R /* Rx BDR 15 base address register 0 */
#define NETC__ENETC0_SI3_RB15BAR1 NETC__ENETC0_SI3.BDR_NUM[15].RBBAR1.R /* Rx BDR 15 base address register 1 */
#define NETC__ENETC0_SI3_RB15PIR NETC__ENETC0_SI3.BDR_NUM[15].RBPIR.R /* Rx BDR 15 producer index register */
#define NETC__ENETC0_SI3_RB15LENR NETC__ENETC0_SI3.BDR_NUM[15].RBLENR.R /* Rx BDR 15 length register */
#define NETC__ENETC0_SI3_RB15DCR NETC__ENETC0_SI3.BDR_NUM[15].RBDCR.R /* Rx BDR 15 drop count register */
#define NETC__ENETC0_SI3_RB15IER NETC__ENETC0_SI3.BDR_NUM[15].RBIER.R /* Rx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI3_RB15IDR NETC__ENETC0_SI3.BDR_NUM[15].RBIDR.R /* Rx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI3_RB15ICR0 NETC__ENETC0_SI3.BDR_NUM[15].RBICR0.R /* Rx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB15ICR1 NETC__ENETC0_SI3.BDR_NUM[15].RBICR1.R /* Rx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB16MR NETC__ENETC0_SI3.BDR_NUM[16].TBMR.R /* Tx BDR 16 mode register */
#define NETC__ENETC0_SI3_TB16SR NETC__ENETC0_SI3.BDR_NUM[16].TBSR.R /* Tx BDR 16 status register */
#define NETC__ENETC0_SI3_TB16BAR0 NETC__ENETC0_SI3.BDR_NUM[16].TBBAR0.R /* Tx BDR 16 base address register 0 */
#define NETC__ENETC0_SI3_TB16BAR1 NETC__ENETC0_SI3.BDR_NUM[16].TBBAR1.R /* Tx BDR 16 base address register 1 */
#define NETC__ENETC0_SI3_TB16PIR NETC__ENETC0_SI3.BDR_NUM[16].TBPIR.R /* Tx BDR 16 producer index register */
#define NETC__ENETC0_SI3_TB16CIR NETC__ENETC0_SI3.BDR_NUM[16].TBCIR.R /* Tx BDR 16 consumer index register */
#define NETC__ENETC0_SI3_TB16LENR NETC__ENETC0_SI3.BDR_NUM[16].TBLENR.R /* Tx BDR 16 length register */
#define NETC__ENETC0_SI3_TB16IER NETC__ENETC0_SI3.BDR_NUM[16].TBIER.R /* Tx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI3_TB16IDR NETC__ENETC0_SI3.BDR_NUM[16].TBIDR.R /* Tx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI3_TB16ICR0 NETC__ENETC0_SI3.BDR_NUM[16].TBICR0.R /* Tx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB16ICR1 NETC__ENETC0_SI3.BDR_NUM[16].TBICR1.R /* Tx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB16MR NETC__ENETC0_SI3.BDR_NUM[16].RBMR.R /* Rx BDR 16 mode register */
#define NETC__ENETC0_SI3_RB16SR NETC__ENETC0_SI3.BDR_NUM[16].RBSR.R /* Rx BDR 16 status register */
#define NETC__ENETC0_SI3_RB16BSR NETC__ENETC0_SI3.BDR_NUM[16].RBBSR.R /* Rx BDR 16 buffer size register */
#define NETC__ENETC0_SI3_RB16CIR NETC__ENETC0_SI3.BDR_NUM[16].RBCIR.R /* Rx BDR 16 consumer index register */
#define NETC__ENETC0_SI3_RB16BAR0 NETC__ENETC0_SI3.BDR_NUM[16].RBBAR0.R /* Rx BDR 16 base address register 0 */
#define NETC__ENETC0_SI3_RB16BAR1 NETC__ENETC0_SI3.BDR_NUM[16].RBBAR1.R /* Rx BDR 16 base address register 1 */
#define NETC__ENETC0_SI3_RB16PIR NETC__ENETC0_SI3.BDR_NUM[16].RBPIR.R /* Rx BDR 16 producer index register */
#define NETC__ENETC0_SI3_RB16LENR NETC__ENETC0_SI3.BDR_NUM[16].RBLENR.R /* Rx BDR 16 length register */
#define NETC__ENETC0_SI3_RB16DCR NETC__ENETC0_SI3.BDR_NUM[16].RBDCR.R /* Rx BDR 16 drop count register */
#define NETC__ENETC0_SI3_RB16IER NETC__ENETC0_SI3.BDR_NUM[16].RBIER.R /* Rx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI3_RB16IDR NETC__ENETC0_SI3.BDR_NUM[16].RBIDR.R /* Rx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI3_RB16ICR0 NETC__ENETC0_SI3.BDR_NUM[16].RBICR0.R /* Rx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB16ICR1 NETC__ENETC0_SI3.BDR_NUM[16].RBICR1.R /* Rx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_TB17MR NETC__ENETC0_SI3.BDR_NUM[17].TBMR.R /* Tx BDR 17 mode register */
#define NETC__ENETC0_SI3_TB17SR NETC__ENETC0_SI3.BDR_NUM[17].TBSR.R /* Tx BDR 17 status register */
#define NETC__ENETC0_SI3_TB17BAR0 NETC__ENETC0_SI3.BDR_NUM[17].TBBAR0.R /* Tx BDR 17 base address register 0 */
#define NETC__ENETC0_SI3_TB17BAR1 NETC__ENETC0_SI3.BDR_NUM[17].TBBAR1.R /* Tx BDR 17 base address register 1 */
#define NETC__ENETC0_SI3_TB17PIR NETC__ENETC0_SI3.BDR_NUM[17].TBPIR.R /* Tx BDR 17 producer index register */
#define NETC__ENETC0_SI3_TB17CIR NETC__ENETC0_SI3.BDR_NUM[17].TBCIR.R /* Tx BDR 17 consumer index register */
#define NETC__ENETC0_SI3_TB17LENR NETC__ENETC0_SI3.BDR_NUM[17].TBLENR.R /* Tx BDR 17 length register */
#define NETC__ENETC0_SI3_TB17IER NETC__ENETC0_SI3.BDR_NUM[17].TBIER.R /* Tx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI3_TB17IDR NETC__ENETC0_SI3.BDR_NUM[17].TBIDR.R /* Tx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI3_TB17ICR0 NETC__ENETC0_SI3.BDR_NUM[17].TBICR0.R /* Tx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_TB17ICR1 NETC__ENETC0_SI3.BDR_NUM[17].TBICR1.R /* Tx BDR 17 interrupt coalescing register 1 */
#define NETC__ENETC0_SI3_RB17MR NETC__ENETC0_SI3.BDR_NUM[17].RBMR.R /* Rx BDR 17 mode register */
#define NETC__ENETC0_SI3_RB17SR NETC__ENETC0_SI3.BDR_NUM[17].RBSR.R /* Rx BDR 17 status register */
#define NETC__ENETC0_SI3_RB17BSR NETC__ENETC0_SI3.BDR_NUM[17].RBBSR.R /* Rx BDR 17 buffer size register */
#define NETC__ENETC0_SI3_RB17CIR NETC__ENETC0_SI3.BDR_NUM[17].RBCIR.R /* Rx BDR 17 consumer index register */
#define NETC__ENETC0_SI3_RB17BAR0 NETC__ENETC0_SI3.BDR_NUM[17].RBBAR0.R /* Rx BDR 17 base address register 0 */
#define NETC__ENETC0_SI3_RB17BAR1 NETC__ENETC0_SI3.BDR_NUM[17].RBBAR1.R /* Rx BDR 17 base address register 1 */
#define NETC__ENETC0_SI3_RB17PIR NETC__ENETC0_SI3.BDR_NUM[17].RBPIR.R /* Rx BDR 17 producer index register */
#define NETC__ENETC0_SI3_RB17LENR NETC__ENETC0_SI3.BDR_NUM[17].RBLENR.R /* Rx BDR 17 length register */
#define NETC__ENETC0_SI3_RB17DCR NETC__ENETC0_SI3.BDR_NUM[17].RBDCR.R /* Rx BDR 17 drop count register */
#define NETC__ENETC0_SI3_RB17IER NETC__ENETC0_SI3.BDR_NUM[17].RBIER.R /* Rx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI3_RB17IDR NETC__ENETC0_SI3.BDR_NUM[17].RBIDR.R /* Rx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI3_RB17ICR0 NETC__ENETC0_SI3.BDR_NUM[17].RBICR0.R /* Rx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI3_RB17ICR1 NETC__ENETC0_SI3.BDR_NUM[17].RBICR1.R /* Rx BDR 17 interrupt coalescing register 1 */

/* NETC_F3_SI4 */
#define NETC__ENETC0_SI4_SIMR NETC__ENETC0_SI4.SIMR.R      /* Station interface mode register */
#define NETC__ENETC0_SI4_SISR NETC__ENETC0_SI4.SISR.R      /* Station interface status register */
#define NETC__ENETC0_SI4_SICTR0 NETC__ENETC0_SI4.SICTR0.R  /* Station interface current time register 0 */
#define NETC__ENETC0_SI4_SICTR1 NETC__ENETC0_SI4.SICTR1.R  /* Station interface current time register 1 */
#define NETC__ENETC0_SI4_SIPCAPR0 NETC__ENETC0_SI4.SIPCAPR0.R /* Station interface port capability register 0 */
#define NETC__ENETC0_SI4_SIPCAPR1 NETC__ENETC0_SI4.SIPCAPR1.R /* Station interface port capability register 1 */
#define NETC__ENETC0_SI4_SIRBGCR NETC__ENETC0_SI4.SIRBGCR.R /* Station interface receive BDR group control register */
#define NETC__ENETC0_SI4_SIBCAR NETC__ENETC0_SI4.SIBCAR.R  /* Station interface buffer cache attribute register */
#define NETC__ENETC0_SI4_SICCAR NETC__ENETC0_SI4.SICCAR.R  /* Station interface command cache attribute register */
#define NETC__ENETC0_SI4_SIPMAR0 NETC__ENETC0_SI4.SIPMAR0.R /* Station interface primary MAC address register 0 */
#define NETC__ENETC0_SI4_SIPMAR1 NETC__ENETC0_SI4.SIPMAR1.R /* Station interface primary MAC address register 1 */
#define NETC__ENETC0_SI4_SICVLANR1 NETC__ENETC0_SI4.SICVLANR1.R /* Station interface custom VLAN register 1 */
#define NETC__ENETC0_SI4_SICVLANR2 NETC__ENETC0_SI4.SICVLANR2.R /* Station interface custom VLAN register 2 */
#define NETC__ENETC0_SI4_SIVLANIPVMR0 NETC__ENETC0_SI4.SIVLANIPVMR0.R /* Station interface VLAN to IPV mapping register 0 */
#define NETC__ENETC0_SI4_SIVLANIPVMR1 NETC__ENETC0_SI4.SIVLANIPVMR1.R /* Station interface VLAN to IPV mapping register 1 */
#define NETC__ENETC0_SI4_SIIPVBDRMR0 NETC__ENETC0_SI4.SIIPVBDRMR0.R /* Station interface IPV to ring mapping register */
#define NETC__ENETC0_SI4_VSIMSGSR NETC__ENETC0_SI4.VSI_A.VSIMSGSR.R /* Virtual station interface message send register */
#define NETC__ENETC0_SI4_VSIMSGRR NETC__ENETC0_SI4.VSI_A.VSIMSGRR.R /* Virtual station interface message receive register */
#define NETC__ENETC0_SI4_VSIMSGSNDAR0 NETC__ENETC0_SI4.VSI_A.VSIMSGSNDAR0.R /* Virtual station interface message send register 0 */
#define NETC__ENETC0_SI4_VSIMSGSNDAR1 NETC__ENETC0_SI4.VSI_A.VSIMSGSNDAR1.R /* Virtual station interface message send address register 1 */
#define NETC__ENETC0_SI4_SIROCT0 NETC__ENETC0_SI4.SIROCT0.R /* Station interface receive octets counter (ifInOctets) 0 */
#define NETC__ENETC0_SI4_SIROCT1 NETC__ENETC0_SI4.SIROCT1.R /* Station interface receive octets counter (ifInOctets) 1 */
#define NETC__ENETC0_SI4_SIRFRM0 NETC__ENETC0_SI4.SIRFRM0.R /* Station interface receive frame counter (aFrameReceivedOK) 0 */
#define NETC__ENETC0_SI4_SIRFRM1 NETC__ENETC0_SI4.SIRFRM1.R /* Station interface receive frame counter (aFrameReceivedOK) 1 */
#define NETC__ENETC0_SI4_SIRUCA0 NETC__ENETC0_SI4.SIRUCA0.R /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
#define NETC__ENETC0_SI4_SIRUCA1 NETC__ENETC0_SI4.SIRUCA1.R /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
#define NETC__ENETC0_SI4_SIRMCA0 NETC__ENETC0_SI4.SIRMCA0.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
#define NETC__ENETC0_SI4_SIRMCA1 NETC__ENETC0_SI4.SIRMCA1.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
#define NETC__ENETC0_SI4_SITOCT0 NETC__ENETC0_SI4.SITOCT0.R /* Station interface transmit octets counter (ifOutOctets) 0 */
#define NETC__ENETC0_SI4_SITOCT1 NETC__ENETC0_SI4.SITOCT1.R /* Station interface transmit octets counter (ifOutOctets) 1 */
#define NETC__ENETC0_SI4_SITFRM0 NETC__ENETC0_SI4.SITFRM0.R /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
#define NETC__ENETC0_SI4_SITFRM1 NETC__ENETC0_SI4.SITFRM1.R /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
#define NETC__ENETC0_SI4_SITUCA0 NETC__ENETC0_SI4.SITUCA0.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
#define NETC__ENETC0_SI4_SITUCA1 NETC__ENETC0_SI4.SITUCA1.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
#define NETC__ENETC0_SI4_SITMCA0 NETC__ENETC0_SI4.SITMCA0.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
#define NETC__ENETC0_SI4_SITMCA1 NETC__ENETC0_SI4.SITMCA1.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
#define NETC__ENETC0_SI4_SIBLPR0 NETC__ENETC0_SI4.SIBLPR[0].R /* Station interface boot loader parameter register 0 */
#define NETC__ENETC0_SI4_SIBLPR1 NETC__ENETC0_SI4.SIBLPR[1].R /* Station interface boot loader parameter register 1 */
#define NETC__ENETC0_SI4_SICBDRMR NETC__ENETC0_SI4.SICBDRMR.R /* Station interface command BDR mode register */
#define NETC__ENETC0_SI4_SICBDRSR NETC__ENETC0_SI4.SICBDRSR.R /* Station interface command BDR status register */
#define NETC__ENETC0_SI4_SICBDRBAR0 NETC__ENETC0_SI4.SICBDRBAR0.R /* Station interface command BDR base address register 0 */
#define NETC__ENETC0_SI4_SICBDRBAR1 NETC__ENETC0_SI4.SICBDRBAR1.R /* Station interface command BDR base address register 1 */
#define NETC__ENETC0_SI4_SICBDRPIR NETC__ENETC0_SI4.SICBDRPIR.R /* Station interface command BDR producer index register */
#define NETC__ENETC0_SI4_SICBDRCIR NETC__ENETC0_SI4.SICBDRCIR.R /* Station interface command BDR consumer index register */
#define NETC__ENETC0_SI4_SICBDRLENR NETC__ENETC0_SI4.SICBDRLENR.R /* Station interface command BDR length register */
#define NETC__ENETC0_SI4_SICBDRIER NETC__ENETC0_SI4.SICBDRIER.R /* Station interface command BDR interrupt enable register */
#define NETC__ENETC0_SI4_SICBDRIDR NETC__ENETC0_SI4.SICBDRIDR.R /* Station interface command BDR interrupt detect register */
#define NETC__ENETC0_SI4_SICAPR0 NETC__ENETC0_SI4.SICAPR0.R /* Station interface capability register 0 */
#define NETC__ENETC0_SI4_SICAPR1 NETC__ENETC0_SI4.SICAPR1.R /* Station interface capability register 1 */
#define NETC__ENETC0_SI4_SICAPR2 NETC__ENETC0_SI4.SICAPR2.R /* Station interface capability register 2 */
#define NETC__ENETC0_SI4_VSIIER NETC__ENETC0_SI4.VSI.VSIIER.R /* Virtual station interface interrupt enable register */
#define NETC__ENETC0_SI4_VSIIDR NETC__ENETC0_SI4.VSI.VSIIDR.R /* Virtual station interface interrupt detect register */
#define NETC__ENETC0_SI4_SITXIDR0 NETC__ENETC0_SI4.SITXIDR0.R /* Station interface transmit interrupt detect register 0 */
#define NETC__ENETC0_SI4_SITXIDR1 NETC__ENETC0_SI4.SITXIDR1.R /* Station interface transmit interrupt detect register 1 */
#define NETC__ENETC0_SI4_SIRXIDR0 NETC__ENETC0_SI4.SIRXIDR0.R /* Station interface receive interrupt detect register 0 */
#define NETC__ENETC0_SI4_SIRXIDR1 NETC__ENETC0_SI4.SIRXIDR1.R /* Station interface receive interrupt detect register 1 */
#define NETC__ENETC0_SI4_SIMSIVR NETC__ENETC0_SI4.SIMSIVR.R /* Station interface MSI-X vector register */
#define NETC__ENETC0_SI4_SICMSIVR NETC__ENETC0_SI4.SICMSIVR.R /* Station interface command MSI-X vector register */
#define NETC__ENETC0_SI4_SITMRIER NETC__ENETC0_SI4.SITMRIER.R /* Station interface timer interrupt enable register */
#define NETC__ENETC0_SI4_SITMRIDR NETC__ENETC0_SI4.SITMRIDR.R /* Station interface timer interrupt detect register */
#define NETC__ENETC0_SI4_SITMRMSIVR NETC__ENETC0_SI4.SITMRMSIVR.R /* Station interface timer MSI-X vector register */
#define NETC__ENETC0_SI4_SIMSITR0VR NETC__ENETC0_SI4.SIMSITRVR[0].R /* Station interface MSI-X transmit ring 0 vector register */
#define NETC__ENETC0_SI4_SIMSITR1VR NETC__ENETC0_SI4.SIMSITRVR[1].R /* Station interface MSI-X transmit ring 1 vector register */
#define NETC__ENETC0_SI4_SIMSITR2VR NETC__ENETC0_SI4.SIMSITRVR[2].R /* Station interface MSI-X transmit ring 2 vector register */
#define NETC__ENETC0_SI4_SIMSITR3VR NETC__ENETC0_SI4.SIMSITRVR[3].R /* Station interface MSI-X transmit ring 3 vector register */
#define NETC__ENETC0_SI4_SIMSITR4VR NETC__ENETC0_SI4.SIMSITRVR[4].R /* Station interface MSI-X transmit ring 4 vector register */
#define NETC__ENETC0_SI4_SIMSITR5VR NETC__ENETC0_SI4.SIMSITRVR[5].R /* Station interface MSI-X transmit ring 5 vector register */
#define NETC__ENETC0_SI4_SIMSITR6VR NETC__ENETC0_SI4.SIMSITRVR[6].R /* Station interface MSI-X transmit ring 6 vector register */
#define NETC__ENETC0_SI4_SIMSITR7VR NETC__ENETC0_SI4.SIMSITRVR[7].R /* Station interface MSI-X transmit ring 7 vector register */
#define NETC__ENETC0_SI4_SIMSITR8VR NETC__ENETC0_SI4.SIMSITRVR[8].R /* Station interface MSI-X transmit ring 8 vector register */
#define NETC__ENETC0_SI4_SIMSITR9VR NETC__ENETC0_SI4.SIMSITRVR[9].R /* Station interface MSI-X transmit ring 9 vector register */
#define NETC__ENETC0_SI4_SIMSITR10VR NETC__ENETC0_SI4.SIMSITRVR[10].R /* Station interface MSI-X transmit ring 10 vector register */
#define NETC__ENETC0_SI4_SIMSITR11VR NETC__ENETC0_SI4.SIMSITRVR[11].R /* Station interface MSI-X transmit ring 11 vector register */
#define NETC__ENETC0_SI4_SIMSITR12VR NETC__ENETC0_SI4.SIMSITRVR[12].R /* Station interface MSI-X transmit ring 12 vector register */
#define NETC__ENETC0_SI4_SIMSITR13VR NETC__ENETC0_SI4.SIMSITRVR[13].R /* Station interface MSI-X transmit ring 13 vector register */
#define NETC__ENETC0_SI4_SIMSITR14VR NETC__ENETC0_SI4.SIMSITRVR[14].R /* Station interface MSI-X transmit ring 14 vector register */
#define NETC__ENETC0_SI4_SIMSITR15VR NETC__ENETC0_SI4.SIMSITRVR[15].R /* Station interface MSI-X transmit ring 15 vector register */
#define NETC__ENETC0_SI4_SIMSITR16VR NETC__ENETC0_SI4.SIMSITRVR[16].R /* Station interface MSI-X transmit ring 16 vector register */
#define NETC__ENETC0_SI4_SIMSITR17VR NETC__ENETC0_SI4.SIMSITRVR[17].R /* Station interface MSI-X transmit ring 17 vector register */
#define NETC__ENETC0_SI4_SIMSIRR0VR NETC__ENETC0_SI4.SIMSIRRVR[0].R /* Station interface MSI-X receive ring 0 vector register */
#define NETC__ENETC0_SI4_SIMSIRR1VR NETC__ENETC0_SI4.SIMSIRRVR[1].R /* Station interface MSI-X receive ring 1 vector register */
#define NETC__ENETC0_SI4_SIMSIRR2VR NETC__ENETC0_SI4.SIMSIRRVR[2].R /* Station interface MSI-X receive ring 2 vector register */
#define NETC__ENETC0_SI4_SIMSIRR3VR NETC__ENETC0_SI4.SIMSIRRVR[3].R /* Station interface MSI-X receive ring 3 vector register */
#define NETC__ENETC0_SI4_SIMSIRR4VR NETC__ENETC0_SI4.SIMSIRRVR[4].R /* Station interface MSI-X receive ring 4 vector register */
#define NETC__ENETC0_SI4_SIMSIRR5VR NETC__ENETC0_SI4.SIMSIRRVR[5].R /* Station interface MSI-X receive ring 5 vector register */
#define NETC__ENETC0_SI4_SIMSIRR6VR NETC__ENETC0_SI4.SIMSIRRVR[6].R /* Station interface MSI-X receive ring 6 vector register */
#define NETC__ENETC0_SI4_SIMSIRR7VR NETC__ENETC0_SI4.SIMSIRRVR[7].R /* Station interface MSI-X receive ring 7 vector register */
#define NETC__ENETC0_SI4_SIMSIRR8VR NETC__ENETC0_SI4.SIMSIRRVR[8].R /* Station interface MSI-X receive ring 8 vector register */
#define NETC__ENETC0_SI4_SIMSIRR9VR NETC__ENETC0_SI4.SIMSIRRVR[9].R /* Station interface MSI-X receive ring 9 vector register */
#define NETC__ENETC0_SI4_SIMSIRR10VR NETC__ENETC0_SI4.SIMSIRRVR[10].R /* Station interface MSI-X receive ring 10 vector register */
#define NETC__ENETC0_SI4_SIMSIRR11VR NETC__ENETC0_SI4.SIMSIRRVR[11].R /* Station interface MSI-X receive ring 11 vector register */
#define NETC__ENETC0_SI4_SIMSIRR12VR NETC__ENETC0_SI4.SIMSIRRVR[12].R /* Station interface MSI-X receive ring 12 vector register */
#define NETC__ENETC0_SI4_SIMSIRR13VR NETC__ENETC0_SI4.SIMSIRRVR[13].R /* Station interface MSI-X receive ring 13 vector register */
#define NETC__ENETC0_SI4_SIMSIRR14VR NETC__ENETC0_SI4.SIMSIRRVR[14].R /* Station interface MSI-X receive ring 14 vector register */
#define NETC__ENETC0_SI4_SIMSIRR15VR NETC__ENETC0_SI4.SIMSIRRVR[15].R /* Station interface MSI-X receive ring 15 vector register */
#define NETC__ENETC0_SI4_SIMSIRR16VR NETC__ENETC0_SI4.SIMSIRRVR[16].R /* Station interface MSI-X receive ring 16 vector register */
#define NETC__ENETC0_SI4_SIMSIRR17VR NETC__ENETC0_SI4.SIMSIRRVR[17].R /* Station interface MSI-X receive ring 17 vector register */
#define NETC__ENETC0_SI4_SICMECR NETC__ENETC0_SI4.SICMECR.R /* Station interface correctable memory error configuration register */
#define NETC__ENETC0_SI4_SICMESR NETC__ENETC0_SI4.SICMESR.R /* Station interface correctable memory error status register */
#define NETC__ENETC0_SI4_SICMECTR NETC__ENETC0_SI4.SICMECTR.R /* Station interface correctable memory error count register */
#define NETC__ENETC0_SI4_SIUPECR NETC__ENETC0_SI4.SIUPECR.R /* Station interface uncorrectable programming error configuration register */
#define NETC__ENETC0_SI4_SIUPESR NETC__ENETC0_SI4.SIUPESR.R /* Station interface uncorrectable programming error status register */
#define NETC__ENETC0_SI4_SIUPECTR NETC__ENETC0_SI4.SIUPECTR.R /* Station interface uncorrectable programming error count register */
#define NETC__ENETC0_SI4_SIUNSBECR NETC__ENETC0_SI4.SIUNSBECR.R /* Station interface uncorrectable non-fatal system bus error configuration register */
#define NETC__ENETC0_SI4_SIUNSBESR NETC__ENETC0_SI4.SIUNSBESR.R /* Station interface uncorrectable non-fatal system bus error status register */
#define NETC__ENETC0_SI4_SIUNSBECTR NETC__ENETC0_SI4.SIUNSBECTR.R /* Station interface uncorrectable non-fatal system bus error count register */
#define NETC__ENETC0_SI4_SIUFSBECR NETC__ENETC0_SI4.SIUFSBECR.R /* Station interface uncorrectable fatal system bus error configuration register */
#define NETC__ENETC0_SI4_SIUFSBESR NETC__ENETC0_SI4.SIUFSBESR.R /* Station interface uncorrectable fatal system bus error status register */
#define NETC__ENETC0_SI4_SIUNMECR NETC__ENETC0_SI4.SIUNMECR.R /* Station interface uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_SI4_SIUNMESR0 NETC__ENETC0_SI4.SIUNMESR0.R /* Station interface uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_SI4_SIUNMESR1 NETC__ENETC0_SI4.SIUNMESR1.R /* Station interface uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_SI4_SIUNMECTR NETC__ENETC0_SI4.SIUNMECTR.R /* Station interface uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_SI4_SIUFMECR NETC__ENETC0_SI4.SIUFMECR.R /* Station interface uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_SI4_SIUFMESR0 NETC__ENETC0_SI4.SIUFMESR0.R /* Station interface uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_SI4_SIUFMESR1 NETC__ENETC0_SI4.SIUFMESR1.R /* Station interface uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_SI4_SIUNIECR NETC__ENETC0_SI4.SIUNIECR.R /* Station interface uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_SI4_SIUNIESR NETC__ENETC0_SI4.SIUNIESR.R /* Station interface uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_SI4_SIUNIECTR NETC__ENETC0_SI4.SIUNIECTR.R /* Station interface uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_SI4_SIUFIECR NETC__ENETC0_SI4.SIUFIECR.R /* Station interface uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_SI4_SIUFIESR NETC__ENETC0_SI4.SIUFIESR.R /* Station interface uncorrectable fatal integrity error status register */
#define NETC__ENETC0_SI4_SIMAFTCAPR NETC__ENETC0_SI4.SIMAFTCAPR.R /* Station interface MAC address filter table capability register */
#define NETC__ENETC0_SI4_SIVFTCAPR NETC__ENETC0_SI4.SIVFTCAPR.R /* Station interface VLAN filter table capability register */
#define NETC__ENETC0_SI4_SIRFSCAPR NETC__ENETC0_SI4.SIRFSCAPR.R /* Station interface RFS capability register */
#define NETC__ENETC0_SI4_TB0MR NETC__ENETC0_SI4.BDR_NUM[0].TBMR.R /* Tx BDR 0 mode register */
#define NETC__ENETC0_SI4_TB0SR NETC__ENETC0_SI4.BDR_NUM[0].TBSR.R /* Tx BDR 0 status register */
#define NETC__ENETC0_SI4_TB0BAR0 NETC__ENETC0_SI4.BDR_NUM[0].TBBAR0.R /* Tx BDR 0 base address register 0 */
#define NETC__ENETC0_SI4_TB0BAR1 NETC__ENETC0_SI4.BDR_NUM[0].TBBAR1.R /* Tx BDR 0 base address register 1 */
#define NETC__ENETC0_SI4_TB0PIR NETC__ENETC0_SI4.BDR_NUM[0].TBPIR.R /* Tx BDR 0 producer index register */
#define NETC__ENETC0_SI4_TB0CIR NETC__ENETC0_SI4.BDR_NUM[0].TBCIR.R /* Tx BDR 0 consumer index register */
#define NETC__ENETC0_SI4_TB0LENR NETC__ENETC0_SI4.BDR_NUM[0].TBLENR.R /* Tx BDR 0 length register */
#define NETC__ENETC0_SI4_TB0IER NETC__ENETC0_SI4.BDR_NUM[0].TBIER.R /* Tx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI4_TB0IDR NETC__ENETC0_SI4.BDR_NUM[0].TBIDR.R /* Tx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI4_TB0ICR0 NETC__ENETC0_SI4.BDR_NUM[0].TBICR0.R /* Tx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB0ICR1 NETC__ENETC0_SI4.BDR_NUM[0].TBICR1.R /* Tx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB0MR NETC__ENETC0_SI4.BDR_NUM[0].RBMR.R /* Rx BDR 0 mode register */
#define NETC__ENETC0_SI4_RB0SR NETC__ENETC0_SI4.BDR_NUM[0].RBSR.R /* Rx BDR 0 status register */
#define NETC__ENETC0_SI4_RB0BSR NETC__ENETC0_SI4.BDR_NUM[0].RBBSR.R /* Rx BDR 0 buffer size register */
#define NETC__ENETC0_SI4_RB0CIR NETC__ENETC0_SI4.BDR_NUM[0].RBCIR.R /* Rx BDR 0 consumer index register */
#define NETC__ENETC0_SI4_RB0BAR0 NETC__ENETC0_SI4.BDR_NUM[0].RBBAR0.R /* Rx BDR 0 base address register 0 */
#define NETC__ENETC0_SI4_RB0BAR1 NETC__ENETC0_SI4.BDR_NUM[0].RBBAR1.R /* Rx BDR 0 base address register 1 */
#define NETC__ENETC0_SI4_RB0PIR NETC__ENETC0_SI4.BDR_NUM[0].RBPIR.R /* Rx BDR 0 producer index register */
#define NETC__ENETC0_SI4_RB0LENR NETC__ENETC0_SI4.BDR_NUM[0].RBLENR.R /* Rx BDR 0 length register */
#define NETC__ENETC0_SI4_RB0DCR NETC__ENETC0_SI4.BDR_NUM[0].RBDCR.R /* Rx BDR 0 drop count register */
#define NETC__ENETC0_SI4_RB0IER NETC__ENETC0_SI4.BDR_NUM[0].RBIER.R /* Rx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI4_RB0IDR NETC__ENETC0_SI4.BDR_NUM[0].RBIDR.R /* Rx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI4_RB0ICR0 NETC__ENETC0_SI4.BDR_NUM[0].RBICR0.R /* Rx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB0ICR1 NETC__ENETC0_SI4.BDR_NUM[0].RBICR1.R /* Rx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB1MR NETC__ENETC0_SI4.BDR_NUM[1].TBMR.R /* Tx BDR 1 mode register */
#define NETC__ENETC0_SI4_TB1SR NETC__ENETC0_SI4.BDR_NUM[1].TBSR.R /* Tx BDR 1 status register */
#define NETC__ENETC0_SI4_TB1BAR0 NETC__ENETC0_SI4.BDR_NUM[1].TBBAR0.R /* Tx BDR 1 base address register 0 */
#define NETC__ENETC0_SI4_TB1BAR1 NETC__ENETC0_SI4.BDR_NUM[1].TBBAR1.R /* Tx BDR 1 base address register 1 */
#define NETC__ENETC0_SI4_TB1PIR NETC__ENETC0_SI4.BDR_NUM[1].TBPIR.R /* Tx BDR 1 producer index register */
#define NETC__ENETC0_SI4_TB1CIR NETC__ENETC0_SI4.BDR_NUM[1].TBCIR.R /* Tx BDR 1 consumer index register */
#define NETC__ENETC0_SI4_TB1LENR NETC__ENETC0_SI4.BDR_NUM[1].TBLENR.R /* Tx BDR 1 length register */
#define NETC__ENETC0_SI4_TB1IER NETC__ENETC0_SI4.BDR_NUM[1].TBIER.R /* Tx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI4_TB1IDR NETC__ENETC0_SI4.BDR_NUM[1].TBIDR.R /* Tx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI4_TB1ICR0 NETC__ENETC0_SI4.BDR_NUM[1].TBICR0.R /* Tx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB1ICR1 NETC__ENETC0_SI4.BDR_NUM[1].TBICR1.R /* Tx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB1MR NETC__ENETC0_SI4.BDR_NUM[1].RBMR.R /* Rx BDR 1 mode register */
#define NETC__ENETC0_SI4_RB1SR NETC__ENETC0_SI4.BDR_NUM[1].RBSR.R /* Rx BDR 1 status register */
#define NETC__ENETC0_SI4_RB1BSR NETC__ENETC0_SI4.BDR_NUM[1].RBBSR.R /* Rx BDR 1 buffer size register */
#define NETC__ENETC0_SI4_RB1CIR NETC__ENETC0_SI4.BDR_NUM[1].RBCIR.R /* Rx BDR 1 consumer index register */
#define NETC__ENETC0_SI4_RB1BAR0 NETC__ENETC0_SI4.BDR_NUM[1].RBBAR0.R /* Rx BDR 1 base address register 0 */
#define NETC__ENETC0_SI4_RB1BAR1 NETC__ENETC0_SI4.BDR_NUM[1].RBBAR1.R /* Rx BDR 1 base address register 1 */
#define NETC__ENETC0_SI4_RB1PIR NETC__ENETC0_SI4.BDR_NUM[1].RBPIR.R /* Rx BDR 1 producer index register */
#define NETC__ENETC0_SI4_RB1LENR NETC__ENETC0_SI4.BDR_NUM[1].RBLENR.R /* Rx BDR 1 length register */
#define NETC__ENETC0_SI4_RB1DCR NETC__ENETC0_SI4.BDR_NUM[1].RBDCR.R /* Rx BDR 1 drop count register */
#define NETC__ENETC0_SI4_RB1IER NETC__ENETC0_SI4.BDR_NUM[1].RBIER.R /* Rx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI4_RB1IDR NETC__ENETC0_SI4.BDR_NUM[1].RBIDR.R /* Rx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI4_RB1ICR0 NETC__ENETC0_SI4.BDR_NUM[1].RBICR0.R /* Rx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB1ICR1 NETC__ENETC0_SI4.BDR_NUM[1].RBICR1.R /* Rx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB2MR NETC__ENETC0_SI4.BDR_NUM[2].TBMR.R /* Tx BDR 2 mode register */
#define NETC__ENETC0_SI4_TB2SR NETC__ENETC0_SI4.BDR_NUM[2].TBSR.R /* Tx BDR 2 status register */
#define NETC__ENETC0_SI4_TB2BAR0 NETC__ENETC0_SI4.BDR_NUM[2].TBBAR0.R /* Tx BDR 2 base address register 0 */
#define NETC__ENETC0_SI4_TB2BAR1 NETC__ENETC0_SI4.BDR_NUM[2].TBBAR1.R /* Tx BDR 2 base address register 1 */
#define NETC__ENETC0_SI4_TB2PIR NETC__ENETC0_SI4.BDR_NUM[2].TBPIR.R /* Tx BDR 2 producer index register */
#define NETC__ENETC0_SI4_TB2CIR NETC__ENETC0_SI4.BDR_NUM[2].TBCIR.R /* Tx BDR 2 consumer index register */
#define NETC__ENETC0_SI4_TB2LENR NETC__ENETC0_SI4.BDR_NUM[2].TBLENR.R /* Tx BDR 2 length register */
#define NETC__ENETC0_SI4_TB2IER NETC__ENETC0_SI4.BDR_NUM[2].TBIER.R /* Tx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI4_TB2IDR NETC__ENETC0_SI4.BDR_NUM[2].TBIDR.R /* Tx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI4_TB2ICR0 NETC__ENETC0_SI4.BDR_NUM[2].TBICR0.R /* Tx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB2ICR1 NETC__ENETC0_SI4.BDR_NUM[2].TBICR1.R /* Tx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB2MR NETC__ENETC0_SI4.BDR_NUM[2].RBMR.R /* Rx BDR 2 mode register */
#define NETC__ENETC0_SI4_RB2SR NETC__ENETC0_SI4.BDR_NUM[2].RBSR.R /* Rx BDR 2 status register */
#define NETC__ENETC0_SI4_RB2BSR NETC__ENETC0_SI4.BDR_NUM[2].RBBSR.R /* Rx BDR 2 buffer size register */
#define NETC__ENETC0_SI4_RB2CIR NETC__ENETC0_SI4.BDR_NUM[2].RBCIR.R /* Rx BDR 2 consumer index register */
#define NETC__ENETC0_SI4_RB2BAR0 NETC__ENETC0_SI4.BDR_NUM[2].RBBAR0.R /* Rx BDR 2 base address register 0 */
#define NETC__ENETC0_SI4_RB2BAR1 NETC__ENETC0_SI4.BDR_NUM[2].RBBAR1.R /* Rx BDR 2 base address register 1 */
#define NETC__ENETC0_SI4_RB2PIR NETC__ENETC0_SI4.BDR_NUM[2].RBPIR.R /* Rx BDR 2 producer index register */
#define NETC__ENETC0_SI4_RB2LENR NETC__ENETC0_SI4.BDR_NUM[2].RBLENR.R /* Rx BDR 2 length register */
#define NETC__ENETC0_SI4_RB2DCR NETC__ENETC0_SI4.BDR_NUM[2].RBDCR.R /* Rx BDR 2 drop count register */
#define NETC__ENETC0_SI4_RB2IER NETC__ENETC0_SI4.BDR_NUM[2].RBIER.R /* Rx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI4_RB2IDR NETC__ENETC0_SI4.BDR_NUM[2].RBIDR.R /* Rx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI4_RB2ICR0 NETC__ENETC0_SI4.BDR_NUM[2].RBICR0.R /* Rx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB2ICR1 NETC__ENETC0_SI4.BDR_NUM[2].RBICR1.R /* Rx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB3MR NETC__ENETC0_SI4.BDR_NUM[3].TBMR.R /* Tx BDR 3 mode register */
#define NETC__ENETC0_SI4_TB3SR NETC__ENETC0_SI4.BDR_NUM[3].TBSR.R /* Tx BDR 3 status register */
#define NETC__ENETC0_SI4_TB3BAR0 NETC__ENETC0_SI4.BDR_NUM[3].TBBAR0.R /* Tx BDR 3 base address register 0 */
#define NETC__ENETC0_SI4_TB3BAR1 NETC__ENETC0_SI4.BDR_NUM[3].TBBAR1.R /* Tx BDR 3 base address register 1 */
#define NETC__ENETC0_SI4_TB3PIR NETC__ENETC0_SI4.BDR_NUM[3].TBPIR.R /* Tx BDR 3 producer index register */
#define NETC__ENETC0_SI4_TB3CIR NETC__ENETC0_SI4.BDR_NUM[3].TBCIR.R /* Tx BDR 3 consumer index register */
#define NETC__ENETC0_SI4_TB3LENR NETC__ENETC0_SI4.BDR_NUM[3].TBLENR.R /* Tx BDR 3 length register */
#define NETC__ENETC0_SI4_TB3IER NETC__ENETC0_SI4.BDR_NUM[3].TBIER.R /* Tx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI4_TB3IDR NETC__ENETC0_SI4.BDR_NUM[3].TBIDR.R /* Tx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI4_TB3ICR0 NETC__ENETC0_SI4.BDR_NUM[3].TBICR0.R /* Tx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB3ICR1 NETC__ENETC0_SI4.BDR_NUM[3].TBICR1.R /* Tx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB3MR NETC__ENETC0_SI4.BDR_NUM[3].RBMR.R /* Rx BDR 3 mode register */
#define NETC__ENETC0_SI4_RB3SR NETC__ENETC0_SI4.BDR_NUM[3].RBSR.R /* Rx BDR 3 status register */
#define NETC__ENETC0_SI4_RB3BSR NETC__ENETC0_SI4.BDR_NUM[3].RBBSR.R /* Rx BDR 3 buffer size register */
#define NETC__ENETC0_SI4_RB3CIR NETC__ENETC0_SI4.BDR_NUM[3].RBCIR.R /* Rx BDR 3 consumer index register */
#define NETC__ENETC0_SI4_RB3BAR0 NETC__ENETC0_SI4.BDR_NUM[3].RBBAR0.R /* Rx BDR 3 base address register 0 */
#define NETC__ENETC0_SI4_RB3BAR1 NETC__ENETC0_SI4.BDR_NUM[3].RBBAR1.R /* Rx BDR 3 base address register 1 */
#define NETC__ENETC0_SI4_RB3PIR NETC__ENETC0_SI4.BDR_NUM[3].RBPIR.R /* Rx BDR 3 producer index register */
#define NETC__ENETC0_SI4_RB3LENR NETC__ENETC0_SI4.BDR_NUM[3].RBLENR.R /* Rx BDR 3 length register */
#define NETC__ENETC0_SI4_RB3DCR NETC__ENETC0_SI4.BDR_NUM[3].RBDCR.R /* Rx BDR 3 drop count register */
#define NETC__ENETC0_SI4_RB3IER NETC__ENETC0_SI4.BDR_NUM[3].RBIER.R /* Rx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI4_RB3IDR NETC__ENETC0_SI4.BDR_NUM[3].RBIDR.R /* Rx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI4_RB3ICR0 NETC__ENETC0_SI4.BDR_NUM[3].RBICR0.R /* Rx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB3ICR1 NETC__ENETC0_SI4.BDR_NUM[3].RBICR1.R /* Rx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB4MR NETC__ENETC0_SI4.BDR_NUM[4].TBMR.R /* Tx BDR 4 mode register */
#define NETC__ENETC0_SI4_TB4SR NETC__ENETC0_SI4.BDR_NUM[4].TBSR.R /* Tx BDR 4 status register */
#define NETC__ENETC0_SI4_TB4BAR0 NETC__ENETC0_SI4.BDR_NUM[4].TBBAR0.R /* Tx BDR 4 base address register 0 */
#define NETC__ENETC0_SI4_TB4BAR1 NETC__ENETC0_SI4.BDR_NUM[4].TBBAR1.R /* Tx BDR 4 base address register 1 */
#define NETC__ENETC0_SI4_TB4PIR NETC__ENETC0_SI4.BDR_NUM[4].TBPIR.R /* Tx BDR 4 producer index register */
#define NETC__ENETC0_SI4_TB4CIR NETC__ENETC0_SI4.BDR_NUM[4].TBCIR.R /* Tx BDR 4 consumer index register */
#define NETC__ENETC0_SI4_TB4LENR NETC__ENETC0_SI4.BDR_NUM[4].TBLENR.R /* Tx BDR 4 length register */
#define NETC__ENETC0_SI4_TB4IER NETC__ENETC0_SI4.BDR_NUM[4].TBIER.R /* Tx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI4_TB4IDR NETC__ENETC0_SI4.BDR_NUM[4].TBIDR.R /* Tx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI4_TB4ICR0 NETC__ENETC0_SI4.BDR_NUM[4].TBICR0.R /* Tx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB4ICR1 NETC__ENETC0_SI4.BDR_NUM[4].TBICR1.R /* Tx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB4MR NETC__ENETC0_SI4.BDR_NUM[4].RBMR.R /* Rx BDR 4 mode register */
#define NETC__ENETC0_SI4_RB4SR NETC__ENETC0_SI4.BDR_NUM[4].RBSR.R /* Rx BDR 4 status register */
#define NETC__ENETC0_SI4_RB4BSR NETC__ENETC0_SI4.BDR_NUM[4].RBBSR.R /* Rx BDR 4 buffer size register */
#define NETC__ENETC0_SI4_RB4CIR NETC__ENETC0_SI4.BDR_NUM[4].RBCIR.R /* Rx BDR 4 consumer index register */
#define NETC__ENETC0_SI4_RB4BAR0 NETC__ENETC0_SI4.BDR_NUM[4].RBBAR0.R /* Rx BDR 4 base address register 0 */
#define NETC__ENETC0_SI4_RB4BAR1 NETC__ENETC0_SI4.BDR_NUM[4].RBBAR1.R /* Rx BDR 4 base address register 1 */
#define NETC__ENETC0_SI4_RB4PIR NETC__ENETC0_SI4.BDR_NUM[4].RBPIR.R /* Rx BDR 4 producer index register */
#define NETC__ENETC0_SI4_RB4LENR NETC__ENETC0_SI4.BDR_NUM[4].RBLENR.R /* Rx BDR 4 length register */
#define NETC__ENETC0_SI4_RB4DCR NETC__ENETC0_SI4.BDR_NUM[4].RBDCR.R /* Rx BDR 4 drop count register */
#define NETC__ENETC0_SI4_RB4IER NETC__ENETC0_SI4.BDR_NUM[4].RBIER.R /* Rx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI4_RB4IDR NETC__ENETC0_SI4.BDR_NUM[4].RBIDR.R /* Rx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI4_RB4ICR0 NETC__ENETC0_SI4.BDR_NUM[4].RBICR0.R /* Rx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB4ICR1 NETC__ENETC0_SI4.BDR_NUM[4].RBICR1.R /* Rx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB5MR NETC__ENETC0_SI4.BDR_NUM[5].TBMR.R /* Tx BDR 5 mode register */
#define NETC__ENETC0_SI4_TB5SR NETC__ENETC0_SI4.BDR_NUM[5].TBSR.R /* Tx BDR 5 status register */
#define NETC__ENETC0_SI4_TB5BAR0 NETC__ENETC0_SI4.BDR_NUM[5].TBBAR0.R /* Tx BDR 5 base address register 0 */
#define NETC__ENETC0_SI4_TB5BAR1 NETC__ENETC0_SI4.BDR_NUM[5].TBBAR1.R /* Tx BDR 5 base address register 1 */
#define NETC__ENETC0_SI4_TB5PIR NETC__ENETC0_SI4.BDR_NUM[5].TBPIR.R /* Tx BDR 5 producer index register */
#define NETC__ENETC0_SI4_TB5CIR NETC__ENETC0_SI4.BDR_NUM[5].TBCIR.R /* Tx BDR 5 consumer index register */
#define NETC__ENETC0_SI4_TB5LENR NETC__ENETC0_SI4.BDR_NUM[5].TBLENR.R /* Tx BDR 5 length register */
#define NETC__ENETC0_SI4_TB5IER NETC__ENETC0_SI4.BDR_NUM[5].TBIER.R /* Tx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI4_TB5IDR NETC__ENETC0_SI4.BDR_NUM[5].TBIDR.R /* Tx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI4_TB5ICR0 NETC__ENETC0_SI4.BDR_NUM[5].TBICR0.R /* Tx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB5ICR1 NETC__ENETC0_SI4.BDR_NUM[5].TBICR1.R /* Tx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB5MR NETC__ENETC0_SI4.BDR_NUM[5].RBMR.R /* Rx BDR 5 mode register */
#define NETC__ENETC0_SI4_RB5SR NETC__ENETC0_SI4.BDR_NUM[5].RBSR.R /* Rx BDR 5 status register */
#define NETC__ENETC0_SI4_RB5BSR NETC__ENETC0_SI4.BDR_NUM[5].RBBSR.R /* Rx BDR 5 buffer size register */
#define NETC__ENETC0_SI4_RB5CIR NETC__ENETC0_SI4.BDR_NUM[5].RBCIR.R /* Rx BDR 5 consumer index register */
#define NETC__ENETC0_SI4_RB5BAR0 NETC__ENETC0_SI4.BDR_NUM[5].RBBAR0.R /* Rx BDR 5 base address register 0 */
#define NETC__ENETC0_SI4_RB5BAR1 NETC__ENETC0_SI4.BDR_NUM[5].RBBAR1.R /* Rx BDR 5 base address register 1 */
#define NETC__ENETC0_SI4_RB5PIR NETC__ENETC0_SI4.BDR_NUM[5].RBPIR.R /* Rx BDR 5 producer index register */
#define NETC__ENETC0_SI4_RB5LENR NETC__ENETC0_SI4.BDR_NUM[5].RBLENR.R /* Rx BDR 5 length register */
#define NETC__ENETC0_SI4_RB5DCR NETC__ENETC0_SI4.BDR_NUM[5].RBDCR.R /* Rx BDR 5 drop count register */
#define NETC__ENETC0_SI4_RB5IER NETC__ENETC0_SI4.BDR_NUM[5].RBIER.R /* Rx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI4_RB5IDR NETC__ENETC0_SI4.BDR_NUM[5].RBIDR.R /* Rx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI4_RB5ICR0 NETC__ENETC0_SI4.BDR_NUM[5].RBICR0.R /* Rx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB5ICR1 NETC__ENETC0_SI4.BDR_NUM[5].RBICR1.R /* Rx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB6MR NETC__ENETC0_SI4.BDR_NUM[6].TBMR.R /* Tx BDR 6 mode register */
#define NETC__ENETC0_SI4_TB6SR NETC__ENETC0_SI4.BDR_NUM[6].TBSR.R /* Tx BDR 6 status register */
#define NETC__ENETC0_SI4_TB6BAR0 NETC__ENETC0_SI4.BDR_NUM[6].TBBAR0.R /* Tx BDR 6 base address register 0 */
#define NETC__ENETC0_SI4_TB6BAR1 NETC__ENETC0_SI4.BDR_NUM[6].TBBAR1.R /* Tx BDR 6 base address register 1 */
#define NETC__ENETC0_SI4_TB6PIR NETC__ENETC0_SI4.BDR_NUM[6].TBPIR.R /* Tx BDR 6 producer index register */
#define NETC__ENETC0_SI4_TB6CIR NETC__ENETC0_SI4.BDR_NUM[6].TBCIR.R /* Tx BDR 6 consumer index register */
#define NETC__ENETC0_SI4_TB6LENR NETC__ENETC0_SI4.BDR_NUM[6].TBLENR.R /* Tx BDR 6 length register */
#define NETC__ENETC0_SI4_TB6IER NETC__ENETC0_SI4.BDR_NUM[6].TBIER.R /* Tx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI4_TB6IDR NETC__ENETC0_SI4.BDR_NUM[6].TBIDR.R /* Tx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI4_TB6ICR0 NETC__ENETC0_SI4.BDR_NUM[6].TBICR0.R /* Tx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB6ICR1 NETC__ENETC0_SI4.BDR_NUM[6].TBICR1.R /* Tx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB6MR NETC__ENETC0_SI4.BDR_NUM[6].RBMR.R /* Rx BDR 6 mode register */
#define NETC__ENETC0_SI4_RB6SR NETC__ENETC0_SI4.BDR_NUM[6].RBSR.R /* Rx BDR 6 status register */
#define NETC__ENETC0_SI4_RB6BSR NETC__ENETC0_SI4.BDR_NUM[6].RBBSR.R /* Rx BDR 6 buffer size register */
#define NETC__ENETC0_SI4_RB6CIR NETC__ENETC0_SI4.BDR_NUM[6].RBCIR.R /* Rx BDR 6 consumer index register */
#define NETC__ENETC0_SI4_RB6BAR0 NETC__ENETC0_SI4.BDR_NUM[6].RBBAR0.R /* Rx BDR 6 base address register 0 */
#define NETC__ENETC0_SI4_RB6BAR1 NETC__ENETC0_SI4.BDR_NUM[6].RBBAR1.R /* Rx BDR 6 base address register 1 */
#define NETC__ENETC0_SI4_RB6PIR NETC__ENETC0_SI4.BDR_NUM[6].RBPIR.R /* Rx BDR 6 producer index register */
#define NETC__ENETC0_SI4_RB6LENR NETC__ENETC0_SI4.BDR_NUM[6].RBLENR.R /* Rx BDR 6 length register */
#define NETC__ENETC0_SI4_RB6DCR NETC__ENETC0_SI4.BDR_NUM[6].RBDCR.R /* Rx BDR 6 drop count register */
#define NETC__ENETC0_SI4_RB6IER NETC__ENETC0_SI4.BDR_NUM[6].RBIER.R /* Rx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI4_RB6IDR NETC__ENETC0_SI4.BDR_NUM[6].RBIDR.R /* Rx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI4_RB6ICR0 NETC__ENETC0_SI4.BDR_NUM[6].RBICR0.R /* Rx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB6ICR1 NETC__ENETC0_SI4.BDR_NUM[6].RBICR1.R /* Rx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB7MR NETC__ENETC0_SI4.BDR_NUM[7].TBMR.R /* Tx BDR 7 mode register */
#define NETC__ENETC0_SI4_TB7SR NETC__ENETC0_SI4.BDR_NUM[7].TBSR.R /* Tx BDR 7 status register */
#define NETC__ENETC0_SI4_TB7BAR0 NETC__ENETC0_SI4.BDR_NUM[7].TBBAR0.R /* Tx BDR 7 base address register 0 */
#define NETC__ENETC0_SI4_TB7BAR1 NETC__ENETC0_SI4.BDR_NUM[7].TBBAR1.R /* Tx BDR 7 base address register 1 */
#define NETC__ENETC0_SI4_TB7PIR NETC__ENETC0_SI4.BDR_NUM[7].TBPIR.R /* Tx BDR 7 producer index register */
#define NETC__ENETC0_SI4_TB7CIR NETC__ENETC0_SI4.BDR_NUM[7].TBCIR.R /* Tx BDR 7 consumer index register */
#define NETC__ENETC0_SI4_TB7LENR NETC__ENETC0_SI4.BDR_NUM[7].TBLENR.R /* Tx BDR 7 length register */
#define NETC__ENETC0_SI4_TB7IER NETC__ENETC0_SI4.BDR_NUM[7].TBIER.R /* Tx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI4_TB7IDR NETC__ENETC0_SI4.BDR_NUM[7].TBIDR.R /* Tx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI4_TB7ICR0 NETC__ENETC0_SI4.BDR_NUM[7].TBICR0.R /* Tx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB7ICR1 NETC__ENETC0_SI4.BDR_NUM[7].TBICR1.R /* Tx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB7MR NETC__ENETC0_SI4.BDR_NUM[7].RBMR.R /* Rx BDR 7 mode register */
#define NETC__ENETC0_SI4_RB7SR NETC__ENETC0_SI4.BDR_NUM[7].RBSR.R /* Rx BDR 7 status register */
#define NETC__ENETC0_SI4_RB7BSR NETC__ENETC0_SI4.BDR_NUM[7].RBBSR.R /* Rx BDR 7 buffer size register */
#define NETC__ENETC0_SI4_RB7CIR NETC__ENETC0_SI4.BDR_NUM[7].RBCIR.R /* Rx BDR 7 consumer index register */
#define NETC__ENETC0_SI4_RB7BAR0 NETC__ENETC0_SI4.BDR_NUM[7].RBBAR0.R /* Rx BDR 7 base address register 0 */
#define NETC__ENETC0_SI4_RB7BAR1 NETC__ENETC0_SI4.BDR_NUM[7].RBBAR1.R /* Rx BDR 7 base address register 1 */
#define NETC__ENETC0_SI4_RB7PIR NETC__ENETC0_SI4.BDR_NUM[7].RBPIR.R /* Rx BDR 7 producer index register */
#define NETC__ENETC0_SI4_RB7LENR NETC__ENETC0_SI4.BDR_NUM[7].RBLENR.R /* Rx BDR 7 length register */
#define NETC__ENETC0_SI4_RB7DCR NETC__ENETC0_SI4.BDR_NUM[7].RBDCR.R /* Rx BDR 7 drop count register */
#define NETC__ENETC0_SI4_RB7IER NETC__ENETC0_SI4.BDR_NUM[7].RBIER.R /* Rx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI4_RB7IDR NETC__ENETC0_SI4.BDR_NUM[7].RBIDR.R /* Rx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI4_RB7ICR0 NETC__ENETC0_SI4.BDR_NUM[7].RBICR0.R /* Rx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB7ICR1 NETC__ENETC0_SI4.BDR_NUM[7].RBICR1.R /* Rx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB8MR NETC__ENETC0_SI4.BDR_NUM[8].TBMR.R /* Tx BDR 8 mode register */
#define NETC__ENETC0_SI4_TB8SR NETC__ENETC0_SI4.BDR_NUM[8].TBSR.R /* Tx BDR 8 status register */
#define NETC__ENETC0_SI4_TB8BAR0 NETC__ENETC0_SI4.BDR_NUM[8].TBBAR0.R /* Tx BDR 8 base address register 0 */
#define NETC__ENETC0_SI4_TB8BAR1 NETC__ENETC0_SI4.BDR_NUM[8].TBBAR1.R /* Tx BDR 8 base address register 1 */
#define NETC__ENETC0_SI4_TB8PIR NETC__ENETC0_SI4.BDR_NUM[8].TBPIR.R /* Tx BDR 8 producer index register */
#define NETC__ENETC0_SI4_TB8CIR NETC__ENETC0_SI4.BDR_NUM[8].TBCIR.R /* Tx BDR 8 consumer index register */
#define NETC__ENETC0_SI4_TB8LENR NETC__ENETC0_SI4.BDR_NUM[8].TBLENR.R /* Tx BDR 8 length register */
#define NETC__ENETC0_SI4_TB8IER NETC__ENETC0_SI4.BDR_NUM[8].TBIER.R /* Tx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI4_TB8IDR NETC__ENETC0_SI4.BDR_NUM[8].TBIDR.R /* Tx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI4_TB8ICR0 NETC__ENETC0_SI4.BDR_NUM[8].TBICR0.R /* Tx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB8ICR1 NETC__ENETC0_SI4.BDR_NUM[8].TBICR1.R /* Tx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB8MR NETC__ENETC0_SI4.BDR_NUM[8].RBMR.R /* Rx BDR 8 mode register */
#define NETC__ENETC0_SI4_RB8SR NETC__ENETC0_SI4.BDR_NUM[8].RBSR.R /* Rx BDR 8 status register */
#define NETC__ENETC0_SI4_RB8BSR NETC__ENETC0_SI4.BDR_NUM[8].RBBSR.R /* Rx BDR 8 buffer size register */
#define NETC__ENETC0_SI4_RB8CIR NETC__ENETC0_SI4.BDR_NUM[8].RBCIR.R /* Rx BDR 8 consumer index register */
#define NETC__ENETC0_SI4_RB8BAR0 NETC__ENETC0_SI4.BDR_NUM[8].RBBAR0.R /* Rx BDR 8 base address register 0 */
#define NETC__ENETC0_SI4_RB8BAR1 NETC__ENETC0_SI4.BDR_NUM[8].RBBAR1.R /* Rx BDR 8 base address register 1 */
#define NETC__ENETC0_SI4_RB8PIR NETC__ENETC0_SI4.BDR_NUM[8].RBPIR.R /* Rx BDR 8 producer index register */
#define NETC__ENETC0_SI4_RB8LENR NETC__ENETC0_SI4.BDR_NUM[8].RBLENR.R /* Rx BDR 8 length register */
#define NETC__ENETC0_SI4_RB8DCR NETC__ENETC0_SI4.BDR_NUM[8].RBDCR.R /* Rx BDR 8 drop count register */
#define NETC__ENETC0_SI4_RB8IER NETC__ENETC0_SI4.BDR_NUM[8].RBIER.R /* Rx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI4_RB8IDR NETC__ENETC0_SI4.BDR_NUM[8].RBIDR.R /* Rx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI4_RB8ICR0 NETC__ENETC0_SI4.BDR_NUM[8].RBICR0.R /* Rx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB8ICR1 NETC__ENETC0_SI4.BDR_NUM[8].RBICR1.R /* Rx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB9MR NETC__ENETC0_SI4.BDR_NUM[9].TBMR.R /* Tx BDR 9 mode register */
#define NETC__ENETC0_SI4_TB9SR NETC__ENETC0_SI4.BDR_NUM[9].TBSR.R /* Tx BDR 9 status register */
#define NETC__ENETC0_SI4_TB9BAR0 NETC__ENETC0_SI4.BDR_NUM[9].TBBAR0.R /* Tx BDR 9 base address register 0 */
#define NETC__ENETC0_SI4_TB9BAR1 NETC__ENETC0_SI4.BDR_NUM[9].TBBAR1.R /* Tx BDR 9 base address register 1 */
#define NETC__ENETC0_SI4_TB9PIR NETC__ENETC0_SI4.BDR_NUM[9].TBPIR.R /* Tx BDR 9 producer index register */
#define NETC__ENETC0_SI4_TB9CIR NETC__ENETC0_SI4.BDR_NUM[9].TBCIR.R /* Tx BDR 9 consumer index register */
#define NETC__ENETC0_SI4_TB9LENR NETC__ENETC0_SI4.BDR_NUM[9].TBLENR.R /* Tx BDR 9 length register */
#define NETC__ENETC0_SI4_TB9IER NETC__ENETC0_SI4.BDR_NUM[9].TBIER.R /* Tx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI4_TB9IDR NETC__ENETC0_SI4.BDR_NUM[9].TBIDR.R /* Tx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI4_TB9ICR0 NETC__ENETC0_SI4.BDR_NUM[9].TBICR0.R /* Tx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB9ICR1 NETC__ENETC0_SI4.BDR_NUM[9].TBICR1.R /* Tx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB9MR NETC__ENETC0_SI4.BDR_NUM[9].RBMR.R /* Rx BDR 9 mode register */
#define NETC__ENETC0_SI4_RB9SR NETC__ENETC0_SI4.BDR_NUM[9].RBSR.R /* Rx BDR 9 status register */
#define NETC__ENETC0_SI4_RB9BSR NETC__ENETC0_SI4.BDR_NUM[9].RBBSR.R /* Rx BDR 9 buffer size register */
#define NETC__ENETC0_SI4_RB9CIR NETC__ENETC0_SI4.BDR_NUM[9].RBCIR.R /* Rx BDR 9 consumer index register */
#define NETC__ENETC0_SI4_RB9BAR0 NETC__ENETC0_SI4.BDR_NUM[9].RBBAR0.R /* Rx BDR 9 base address register 0 */
#define NETC__ENETC0_SI4_RB9BAR1 NETC__ENETC0_SI4.BDR_NUM[9].RBBAR1.R /* Rx BDR 9 base address register 1 */
#define NETC__ENETC0_SI4_RB9PIR NETC__ENETC0_SI4.BDR_NUM[9].RBPIR.R /* Rx BDR 9 producer index register */
#define NETC__ENETC0_SI4_RB9LENR NETC__ENETC0_SI4.BDR_NUM[9].RBLENR.R /* Rx BDR 9 length register */
#define NETC__ENETC0_SI4_RB9DCR NETC__ENETC0_SI4.BDR_NUM[9].RBDCR.R /* Rx BDR 9 drop count register */
#define NETC__ENETC0_SI4_RB9IER NETC__ENETC0_SI4.BDR_NUM[9].RBIER.R /* Rx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI4_RB9IDR NETC__ENETC0_SI4.BDR_NUM[9].RBIDR.R /* Rx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI4_RB9ICR0 NETC__ENETC0_SI4.BDR_NUM[9].RBICR0.R /* Rx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB9ICR1 NETC__ENETC0_SI4.BDR_NUM[9].RBICR1.R /* Rx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB10MR NETC__ENETC0_SI4.BDR_NUM[10].TBMR.R /* Tx BDR 10 mode register */
#define NETC__ENETC0_SI4_TB10SR NETC__ENETC0_SI4.BDR_NUM[10].TBSR.R /* Tx BDR 10 status register */
#define NETC__ENETC0_SI4_TB10BAR0 NETC__ENETC0_SI4.BDR_NUM[10].TBBAR0.R /* Tx BDR 10 base address register 0 */
#define NETC__ENETC0_SI4_TB10BAR1 NETC__ENETC0_SI4.BDR_NUM[10].TBBAR1.R /* Tx BDR 10 base address register 1 */
#define NETC__ENETC0_SI4_TB10PIR NETC__ENETC0_SI4.BDR_NUM[10].TBPIR.R /* Tx BDR 10 producer index register */
#define NETC__ENETC0_SI4_TB10CIR NETC__ENETC0_SI4.BDR_NUM[10].TBCIR.R /* Tx BDR 10 consumer index register */
#define NETC__ENETC0_SI4_TB10LENR NETC__ENETC0_SI4.BDR_NUM[10].TBLENR.R /* Tx BDR 10 length register */
#define NETC__ENETC0_SI4_TB10IER NETC__ENETC0_SI4.BDR_NUM[10].TBIER.R /* Tx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI4_TB10IDR NETC__ENETC0_SI4.BDR_NUM[10].TBIDR.R /* Tx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI4_TB10ICR0 NETC__ENETC0_SI4.BDR_NUM[10].TBICR0.R /* Tx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB10ICR1 NETC__ENETC0_SI4.BDR_NUM[10].TBICR1.R /* Tx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB10MR NETC__ENETC0_SI4.BDR_NUM[10].RBMR.R /* Rx BDR 10 mode register */
#define NETC__ENETC0_SI4_RB10SR NETC__ENETC0_SI4.BDR_NUM[10].RBSR.R /* Rx BDR 10 status register */
#define NETC__ENETC0_SI4_RB10BSR NETC__ENETC0_SI4.BDR_NUM[10].RBBSR.R /* Rx BDR 10 buffer size register */
#define NETC__ENETC0_SI4_RB10CIR NETC__ENETC0_SI4.BDR_NUM[10].RBCIR.R /* Rx BDR 10 consumer index register */
#define NETC__ENETC0_SI4_RB10BAR0 NETC__ENETC0_SI4.BDR_NUM[10].RBBAR0.R /* Rx BDR 10 base address register 0 */
#define NETC__ENETC0_SI4_RB10BAR1 NETC__ENETC0_SI4.BDR_NUM[10].RBBAR1.R /* Rx BDR 10 base address register 1 */
#define NETC__ENETC0_SI4_RB10PIR NETC__ENETC0_SI4.BDR_NUM[10].RBPIR.R /* Rx BDR 10 producer index register */
#define NETC__ENETC0_SI4_RB10LENR NETC__ENETC0_SI4.BDR_NUM[10].RBLENR.R /* Rx BDR 10 length register */
#define NETC__ENETC0_SI4_RB10DCR NETC__ENETC0_SI4.BDR_NUM[10].RBDCR.R /* Rx BDR 10 drop count register */
#define NETC__ENETC0_SI4_RB10IER NETC__ENETC0_SI4.BDR_NUM[10].RBIER.R /* Rx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI4_RB10IDR NETC__ENETC0_SI4.BDR_NUM[10].RBIDR.R /* Rx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI4_RB10ICR0 NETC__ENETC0_SI4.BDR_NUM[10].RBICR0.R /* Rx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB10ICR1 NETC__ENETC0_SI4.BDR_NUM[10].RBICR1.R /* Rx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB11MR NETC__ENETC0_SI4.BDR_NUM[11].TBMR.R /* Tx BDR 11 mode register */
#define NETC__ENETC0_SI4_TB11SR NETC__ENETC0_SI4.BDR_NUM[11].TBSR.R /* Tx BDR 11 status register */
#define NETC__ENETC0_SI4_TB11BAR0 NETC__ENETC0_SI4.BDR_NUM[11].TBBAR0.R /* Tx BDR 11 base address register 0 */
#define NETC__ENETC0_SI4_TB11BAR1 NETC__ENETC0_SI4.BDR_NUM[11].TBBAR1.R /* Tx BDR 11 base address register 1 */
#define NETC__ENETC0_SI4_TB11PIR NETC__ENETC0_SI4.BDR_NUM[11].TBPIR.R /* Tx BDR 11 producer index register */
#define NETC__ENETC0_SI4_TB11CIR NETC__ENETC0_SI4.BDR_NUM[11].TBCIR.R /* Tx BDR 11 consumer index register */
#define NETC__ENETC0_SI4_TB11LENR NETC__ENETC0_SI4.BDR_NUM[11].TBLENR.R /* Tx BDR 11 length register */
#define NETC__ENETC0_SI4_TB11IER NETC__ENETC0_SI4.BDR_NUM[11].TBIER.R /* Tx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI4_TB11IDR NETC__ENETC0_SI4.BDR_NUM[11].TBIDR.R /* Tx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI4_TB11ICR0 NETC__ENETC0_SI4.BDR_NUM[11].TBICR0.R /* Tx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB11ICR1 NETC__ENETC0_SI4.BDR_NUM[11].TBICR1.R /* Tx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB11MR NETC__ENETC0_SI4.BDR_NUM[11].RBMR.R /* Rx BDR 11 mode register */
#define NETC__ENETC0_SI4_RB11SR NETC__ENETC0_SI4.BDR_NUM[11].RBSR.R /* Rx BDR 11 status register */
#define NETC__ENETC0_SI4_RB11BSR NETC__ENETC0_SI4.BDR_NUM[11].RBBSR.R /* Rx BDR 11 buffer size register */
#define NETC__ENETC0_SI4_RB11CIR NETC__ENETC0_SI4.BDR_NUM[11].RBCIR.R /* Rx BDR 11 consumer index register */
#define NETC__ENETC0_SI4_RB11BAR0 NETC__ENETC0_SI4.BDR_NUM[11].RBBAR0.R /* Rx BDR 11 base address register 0 */
#define NETC__ENETC0_SI4_RB11BAR1 NETC__ENETC0_SI4.BDR_NUM[11].RBBAR1.R /* Rx BDR 11 base address register 1 */
#define NETC__ENETC0_SI4_RB11PIR NETC__ENETC0_SI4.BDR_NUM[11].RBPIR.R /* Rx BDR 11 producer index register */
#define NETC__ENETC0_SI4_RB11LENR NETC__ENETC0_SI4.BDR_NUM[11].RBLENR.R /* Rx BDR 11 length register */
#define NETC__ENETC0_SI4_RB11DCR NETC__ENETC0_SI4.BDR_NUM[11].RBDCR.R /* Rx BDR 11 drop count register */
#define NETC__ENETC0_SI4_RB11IER NETC__ENETC0_SI4.BDR_NUM[11].RBIER.R /* Rx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI4_RB11IDR NETC__ENETC0_SI4.BDR_NUM[11].RBIDR.R /* Rx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI4_RB11ICR0 NETC__ENETC0_SI4.BDR_NUM[11].RBICR0.R /* Rx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB11ICR1 NETC__ENETC0_SI4.BDR_NUM[11].RBICR1.R /* Rx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB12MR NETC__ENETC0_SI4.BDR_NUM[12].TBMR.R /* Tx BDR 12 mode register */
#define NETC__ENETC0_SI4_TB12SR NETC__ENETC0_SI4.BDR_NUM[12].TBSR.R /* Tx BDR 12 status register */
#define NETC__ENETC0_SI4_TB12BAR0 NETC__ENETC0_SI4.BDR_NUM[12].TBBAR0.R /* Tx BDR 12 base address register 0 */
#define NETC__ENETC0_SI4_TB12BAR1 NETC__ENETC0_SI4.BDR_NUM[12].TBBAR1.R /* Tx BDR 12 base address register 1 */
#define NETC__ENETC0_SI4_TB12PIR NETC__ENETC0_SI4.BDR_NUM[12].TBPIR.R /* Tx BDR 12 producer index register */
#define NETC__ENETC0_SI4_TB12CIR NETC__ENETC0_SI4.BDR_NUM[12].TBCIR.R /* Tx BDR 12 consumer index register */
#define NETC__ENETC0_SI4_TB12LENR NETC__ENETC0_SI4.BDR_NUM[12].TBLENR.R /* Tx BDR 12 length register */
#define NETC__ENETC0_SI4_TB12IER NETC__ENETC0_SI4.BDR_NUM[12].TBIER.R /* Tx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI4_TB12IDR NETC__ENETC0_SI4.BDR_NUM[12].TBIDR.R /* Tx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI4_TB12ICR0 NETC__ENETC0_SI4.BDR_NUM[12].TBICR0.R /* Tx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB12ICR1 NETC__ENETC0_SI4.BDR_NUM[12].TBICR1.R /* Tx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB12MR NETC__ENETC0_SI4.BDR_NUM[12].RBMR.R /* Rx BDR 12 mode register */
#define NETC__ENETC0_SI4_RB12SR NETC__ENETC0_SI4.BDR_NUM[12].RBSR.R /* Rx BDR 12 status register */
#define NETC__ENETC0_SI4_RB12BSR NETC__ENETC0_SI4.BDR_NUM[12].RBBSR.R /* Rx BDR 12 buffer size register */
#define NETC__ENETC0_SI4_RB12CIR NETC__ENETC0_SI4.BDR_NUM[12].RBCIR.R /* Rx BDR 12 consumer index register */
#define NETC__ENETC0_SI4_RB12BAR0 NETC__ENETC0_SI4.BDR_NUM[12].RBBAR0.R /* Rx BDR 12 base address register 0 */
#define NETC__ENETC0_SI4_RB12BAR1 NETC__ENETC0_SI4.BDR_NUM[12].RBBAR1.R /* Rx BDR 12 base address register 1 */
#define NETC__ENETC0_SI4_RB12PIR NETC__ENETC0_SI4.BDR_NUM[12].RBPIR.R /* Rx BDR 12 producer index register */
#define NETC__ENETC0_SI4_RB12LENR NETC__ENETC0_SI4.BDR_NUM[12].RBLENR.R /* Rx BDR 12 length register */
#define NETC__ENETC0_SI4_RB12DCR NETC__ENETC0_SI4.BDR_NUM[12].RBDCR.R /* Rx BDR 12 drop count register */
#define NETC__ENETC0_SI4_RB12IER NETC__ENETC0_SI4.BDR_NUM[12].RBIER.R /* Rx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI4_RB12IDR NETC__ENETC0_SI4.BDR_NUM[12].RBIDR.R /* Rx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI4_RB12ICR0 NETC__ENETC0_SI4.BDR_NUM[12].RBICR0.R /* Rx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB12ICR1 NETC__ENETC0_SI4.BDR_NUM[12].RBICR1.R /* Rx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB13MR NETC__ENETC0_SI4.BDR_NUM[13].TBMR.R /* Tx BDR 13 mode register */
#define NETC__ENETC0_SI4_TB13SR NETC__ENETC0_SI4.BDR_NUM[13].TBSR.R /* Tx BDR 13 status register */
#define NETC__ENETC0_SI4_TB13BAR0 NETC__ENETC0_SI4.BDR_NUM[13].TBBAR0.R /* Tx BDR 13 base address register 0 */
#define NETC__ENETC0_SI4_TB13BAR1 NETC__ENETC0_SI4.BDR_NUM[13].TBBAR1.R /* Tx BDR 13 base address register 1 */
#define NETC__ENETC0_SI4_TB13PIR NETC__ENETC0_SI4.BDR_NUM[13].TBPIR.R /* Tx BDR 13 producer index register */
#define NETC__ENETC0_SI4_TB13CIR NETC__ENETC0_SI4.BDR_NUM[13].TBCIR.R /* Tx BDR 13 consumer index register */
#define NETC__ENETC0_SI4_TB13LENR NETC__ENETC0_SI4.BDR_NUM[13].TBLENR.R /* Tx BDR 13 length register */
#define NETC__ENETC0_SI4_TB13IER NETC__ENETC0_SI4.BDR_NUM[13].TBIER.R /* Tx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI4_TB13IDR NETC__ENETC0_SI4.BDR_NUM[13].TBIDR.R /* Tx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI4_TB13ICR0 NETC__ENETC0_SI4.BDR_NUM[13].TBICR0.R /* Tx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB13ICR1 NETC__ENETC0_SI4.BDR_NUM[13].TBICR1.R /* Tx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB13MR NETC__ENETC0_SI4.BDR_NUM[13].RBMR.R /* Rx BDR 13 mode register */
#define NETC__ENETC0_SI4_RB13SR NETC__ENETC0_SI4.BDR_NUM[13].RBSR.R /* Rx BDR 13 status register */
#define NETC__ENETC0_SI4_RB13BSR NETC__ENETC0_SI4.BDR_NUM[13].RBBSR.R /* Rx BDR 13 buffer size register */
#define NETC__ENETC0_SI4_RB13CIR NETC__ENETC0_SI4.BDR_NUM[13].RBCIR.R /* Rx BDR 13 consumer index register */
#define NETC__ENETC0_SI4_RB13BAR0 NETC__ENETC0_SI4.BDR_NUM[13].RBBAR0.R /* Rx BDR 13 base address register 0 */
#define NETC__ENETC0_SI4_RB13BAR1 NETC__ENETC0_SI4.BDR_NUM[13].RBBAR1.R /* Rx BDR 13 base address register 1 */
#define NETC__ENETC0_SI4_RB13PIR NETC__ENETC0_SI4.BDR_NUM[13].RBPIR.R /* Rx BDR 13 producer index register */
#define NETC__ENETC0_SI4_RB13LENR NETC__ENETC0_SI4.BDR_NUM[13].RBLENR.R /* Rx BDR 13 length register */
#define NETC__ENETC0_SI4_RB13DCR NETC__ENETC0_SI4.BDR_NUM[13].RBDCR.R /* Rx BDR 13 drop count register */
#define NETC__ENETC0_SI4_RB13IER NETC__ENETC0_SI4.BDR_NUM[13].RBIER.R /* Rx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI4_RB13IDR NETC__ENETC0_SI4.BDR_NUM[13].RBIDR.R /* Rx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI4_RB13ICR0 NETC__ENETC0_SI4.BDR_NUM[13].RBICR0.R /* Rx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB13ICR1 NETC__ENETC0_SI4.BDR_NUM[13].RBICR1.R /* Rx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB14MR NETC__ENETC0_SI4.BDR_NUM[14].TBMR.R /* Tx BDR 14 mode register */
#define NETC__ENETC0_SI4_TB14SR NETC__ENETC0_SI4.BDR_NUM[14].TBSR.R /* Tx BDR 14 status register */
#define NETC__ENETC0_SI4_TB14BAR0 NETC__ENETC0_SI4.BDR_NUM[14].TBBAR0.R /* Tx BDR 14 base address register 0 */
#define NETC__ENETC0_SI4_TB14BAR1 NETC__ENETC0_SI4.BDR_NUM[14].TBBAR1.R /* Tx BDR 14 base address register 1 */
#define NETC__ENETC0_SI4_TB14PIR NETC__ENETC0_SI4.BDR_NUM[14].TBPIR.R /* Tx BDR 14 producer index register */
#define NETC__ENETC0_SI4_TB14CIR NETC__ENETC0_SI4.BDR_NUM[14].TBCIR.R /* Tx BDR 14 consumer index register */
#define NETC__ENETC0_SI4_TB14LENR NETC__ENETC0_SI4.BDR_NUM[14].TBLENR.R /* Tx BDR 14 length register */
#define NETC__ENETC0_SI4_TB14IER NETC__ENETC0_SI4.BDR_NUM[14].TBIER.R /* Tx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI4_TB14IDR NETC__ENETC0_SI4.BDR_NUM[14].TBIDR.R /* Tx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI4_TB14ICR0 NETC__ENETC0_SI4.BDR_NUM[14].TBICR0.R /* Tx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB14ICR1 NETC__ENETC0_SI4.BDR_NUM[14].TBICR1.R /* Tx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB14MR NETC__ENETC0_SI4.BDR_NUM[14].RBMR.R /* Rx BDR 14 mode register */
#define NETC__ENETC0_SI4_RB14SR NETC__ENETC0_SI4.BDR_NUM[14].RBSR.R /* Rx BDR 14 status register */
#define NETC__ENETC0_SI4_RB14BSR NETC__ENETC0_SI4.BDR_NUM[14].RBBSR.R /* Rx BDR 14 buffer size register */
#define NETC__ENETC0_SI4_RB14CIR NETC__ENETC0_SI4.BDR_NUM[14].RBCIR.R /* Rx BDR 14 consumer index register */
#define NETC__ENETC0_SI4_RB14BAR0 NETC__ENETC0_SI4.BDR_NUM[14].RBBAR0.R /* Rx BDR 14 base address register 0 */
#define NETC__ENETC0_SI4_RB14BAR1 NETC__ENETC0_SI4.BDR_NUM[14].RBBAR1.R /* Rx BDR 14 base address register 1 */
#define NETC__ENETC0_SI4_RB14PIR NETC__ENETC0_SI4.BDR_NUM[14].RBPIR.R /* Rx BDR 14 producer index register */
#define NETC__ENETC0_SI4_RB14LENR NETC__ENETC0_SI4.BDR_NUM[14].RBLENR.R /* Rx BDR 14 length register */
#define NETC__ENETC0_SI4_RB14DCR NETC__ENETC0_SI4.BDR_NUM[14].RBDCR.R /* Rx BDR 14 drop count register */
#define NETC__ENETC0_SI4_RB14IER NETC__ENETC0_SI4.BDR_NUM[14].RBIER.R /* Rx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI4_RB14IDR NETC__ENETC0_SI4.BDR_NUM[14].RBIDR.R /* Rx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI4_RB14ICR0 NETC__ENETC0_SI4.BDR_NUM[14].RBICR0.R /* Rx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB14ICR1 NETC__ENETC0_SI4.BDR_NUM[14].RBICR1.R /* Rx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB15MR NETC__ENETC0_SI4.BDR_NUM[15].TBMR.R /* Tx BDR 15 mode register */
#define NETC__ENETC0_SI4_TB15SR NETC__ENETC0_SI4.BDR_NUM[15].TBSR.R /* Tx BDR 15 status register */
#define NETC__ENETC0_SI4_TB15BAR0 NETC__ENETC0_SI4.BDR_NUM[15].TBBAR0.R /* Tx BDR 15 base address register 0 */
#define NETC__ENETC0_SI4_TB15BAR1 NETC__ENETC0_SI4.BDR_NUM[15].TBBAR1.R /* Tx BDR 15 base address register 1 */
#define NETC__ENETC0_SI4_TB15PIR NETC__ENETC0_SI4.BDR_NUM[15].TBPIR.R /* Tx BDR 15 producer index register */
#define NETC__ENETC0_SI4_TB15CIR NETC__ENETC0_SI4.BDR_NUM[15].TBCIR.R /* Tx BDR 15 consumer index register */
#define NETC__ENETC0_SI4_TB15LENR NETC__ENETC0_SI4.BDR_NUM[15].TBLENR.R /* Tx BDR 15 length register */
#define NETC__ENETC0_SI4_TB15IER NETC__ENETC0_SI4.BDR_NUM[15].TBIER.R /* Tx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI4_TB15IDR NETC__ENETC0_SI4.BDR_NUM[15].TBIDR.R /* Tx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI4_TB15ICR0 NETC__ENETC0_SI4.BDR_NUM[15].TBICR0.R /* Tx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB15ICR1 NETC__ENETC0_SI4.BDR_NUM[15].TBICR1.R /* Tx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB15MR NETC__ENETC0_SI4.BDR_NUM[15].RBMR.R /* Rx BDR 15 mode register */
#define NETC__ENETC0_SI4_RB15SR NETC__ENETC0_SI4.BDR_NUM[15].RBSR.R /* Rx BDR 15 status register */
#define NETC__ENETC0_SI4_RB15BSR NETC__ENETC0_SI4.BDR_NUM[15].RBBSR.R /* Rx BDR 15 buffer size register */
#define NETC__ENETC0_SI4_RB15CIR NETC__ENETC0_SI4.BDR_NUM[15].RBCIR.R /* Rx BDR 15 consumer index register */
#define NETC__ENETC0_SI4_RB15BAR0 NETC__ENETC0_SI4.BDR_NUM[15].RBBAR0.R /* Rx BDR 15 base address register 0 */
#define NETC__ENETC0_SI4_RB15BAR1 NETC__ENETC0_SI4.BDR_NUM[15].RBBAR1.R /* Rx BDR 15 base address register 1 */
#define NETC__ENETC0_SI4_RB15PIR NETC__ENETC0_SI4.BDR_NUM[15].RBPIR.R /* Rx BDR 15 producer index register */
#define NETC__ENETC0_SI4_RB15LENR NETC__ENETC0_SI4.BDR_NUM[15].RBLENR.R /* Rx BDR 15 length register */
#define NETC__ENETC0_SI4_RB15DCR NETC__ENETC0_SI4.BDR_NUM[15].RBDCR.R /* Rx BDR 15 drop count register */
#define NETC__ENETC0_SI4_RB15IER NETC__ENETC0_SI4.BDR_NUM[15].RBIER.R /* Rx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI4_RB15IDR NETC__ENETC0_SI4.BDR_NUM[15].RBIDR.R /* Rx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI4_RB15ICR0 NETC__ENETC0_SI4.BDR_NUM[15].RBICR0.R /* Rx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB15ICR1 NETC__ENETC0_SI4.BDR_NUM[15].RBICR1.R /* Rx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB16MR NETC__ENETC0_SI4.BDR_NUM[16].TBMR.R /* Tx BDR 16 mode register */
#define NETC__ENETC0_SI4_TB16SR NETC__ENETC0_SI4.BDR_NUM[16].TBSR.R /* Tx BDR 16 status register */
#define NETC__ENETC0_SI4_TB16BAR0 NETC__ENETC0_SI4.BDR_NUM[16].TBBAR0.R /* Tx BDR 16 base address register 0 */
#define NETC__ENETC0_SI4_TB16BAR1 NETC__ENETC0_SI4.BDR_NUM[16].TBBAR1.R /* Tx BDR 16 base address register 1 */
#define NETC__ENETC0_SI4_TB16PIR NETC__ENETC0_SI4.BDR_NUM[16].TBPIR.R /* Tx BDR 16 producer index register */
#define NETC__ENETC0_SI4_TB16CIR NETC__ENETC0_SI4.BDR_NUM[16].TBCIR.R /* Tx BDR 16 consumer index register */
#define NETC__ENETC0_SI4_TB16LENR NETC__ENETC0_SI4.BDR_NUM[16].TBLENR.R /* Tx BDR 16 length register */
#define NETC__ENETC0_SI4_TB16IER NETC__ENETC0_SI4.BDR_NUM[16].TBIER.R /* Tx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI4_TB16IDR NETC__ENETC0_SI4.BDR_NUM[16].TBIDR.R /* Tx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI4_TB16ICR0 NETC__ENETC0_SI4.BDR_NUM[16].TBICR0.R /* Tx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB16ICR1 NETC__ENETC0_SI4.BDR_NUM[16].TBICR1.R /* Tx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB16MR NETC__ENETC0_SI4.BDR_NUM[16].RBMR.R /* Rx BDR 16 mode register */
#define NETC__ENETC0_SI4_RB16SR NETC__ENETC0_SI4.BDR_NUM[16].RBSR.R /* Rx BDR 16 status register */
#define NETC__ENETC0_SI4_RB16BSR NETC__ENETC0_SI4.BDR_NUM[16].RBBSR.R /* Rx BDR 16 buffer size register */
#define NETC__ENETC0_SI4_RB16CIR NETC__ENETC0_SI4.BDR_NUM[16].RBCIR.R /* Rx BDR 16 consumer index register */
#define NETC__ENETC0_SI4_RB16BAR0 NETC__ENETC0_SI4.BDR_NUM[16].RBBAR0.R /* Rx BDR 16 base address register 0 */
#define NETC__ENETC0_SI4_RB16BAR1 NETC__ENETC0_SI4.BDR_NUM[16].RBBAR1.R /* Rx BDR 16 base address register 1 */
#define NETC__ENETC0_SI4_RB16PIR NETC__ENETC0_SI4.BDR_NUM[16].RBPIR.R /* Rx BDR 16 producer index register */
#define NETC__ENETC0_SI4_RB16LENR NETC__ENETC0_SI4.BDR_NUM[16].RBLENR.R /* Rx BDR 16 length register */
#define NETC__ENETC0_SI4_RB16DCR NETC__ENETC0_SI4.BDR_NUM[16].RBDCR.R /* Rx BDR 16 drop count register */
#define NETC__ENETC0_SI4_RB16IER NETC__ENETC0_SI4.BDR_NUM[16].RBIER.R /* Rx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI4_RB16IDR NETC__ENETC0_SI4.BDR_NUM[16].RBIDR.R /* Rx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI4_RB16ICR0 NETC__ENETC0_SI4.BDR_NUM[16].RBICR0.R /* Rx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB16ICR1 NETC__ENETC0_SI4.BDR_NUM[16].RBICR1.R /* Rx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_TB17MR NETC__ENETC0_SI4.BDR_NUM[17].TBMR.R /* Tx BDR 17 mode register */
#define NETC__ENETC0_SI4_TB17SR NETC__ENETC0_SI4.BDR_NUM[17].TBSR.R /* Tx BDR 17 status register */
#define NETC__ENETC0_SI4_TB17BAR0 NETC__ENETC0_SI4.BDR_NUM[17].TBBAR0.R /* Tx BDR 17 base address register 0 */
#define NETC__ENETC0_SI4_TB17BAR1 NETC__ENETC0_SI4.BDR_NUM[17].TBBAR1.R /* Tx BDR 17 base address register 1 */
#define NETC__ENETC0_SI4_TB17PIR NETC__ENETC0_SI4.BDR_NUM[17].TBPIR.R /* Tx BDR 17 producer index register */
#define NETC__ENETC0_SI4_TB17CIR NETC__ENETC0_SI4.BDR_NUM[17].TBCIR.R /* Tx BDR 17 consumer index register */
#define NETC__ENETC0_SI4_TB17LENR NETC__ENETC0_SI4.BDR_NUM[17].TBLENR.R /* Tx BDR 17 length register */
#define NETC__ENETC0_SI4_TB17IER NETC__ENETC0_SI4.BDR_NUM[17].TBIER.R /* Tx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI4_TB17IDR NETC__ENETC0_SI4.BDR_NUM[17].TBIDR.R /* Tx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI4_TB17ICR0 NETC__ENETC0_SI4.BDR_NUM[17].TBICR0.R /* Tx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_TB17ICR1 NETC__ENETC0_SI4.BDR_NUM[17].TBICR1.R /* Tx BDR 17 interrupt coalescing register 1 */
#define NETC__ENETC0_SI4_RB17MR NETC__ENETC0_SI4.BDR_NUM[17].RBMR.R /* Rx BDR 17 mode register */
#define NETC__ENETC0_SI4_RB17SR NETC__ENETC0_SI4.BDR_NUM[17].RBSR.R /* Rx BDR 17 status register */
#define NETC__ENETC0_SI4_RB17BSR NETC__ENETC0_SI4.BDR_NUM[17].RBBSR.R /* Rx BDR 17 buffer size register */
#define NETC__ENETC0_SI4_RB17CIR NETC__ENETC0_SI4.BDR_NUM[17].RBCIR.R /* Rx BDR 17 consumer index register */
#define NETC__ENETC0_SI4_RB17BAR0 NETC__ENETC0_SI4.BDR_NUM[17].RBBAR0.R /* Rx BDR 17 base address register 0 */
#define NETC__ENETC0_SI4_RB17BAR1 NETC__ENETC0_SI4.BDR_NUM[17].RBBAR1.R /* Rx BDR 17 base address register 1 */
#define NETC__ENETC0_SI4_RB17PIR NETC__ENETC0_SI4.BDR_NUM[17].RBPIR.R /* Rx BDR 17 producer index register */
#define NETC__ENETC0_SI4_RB17LENR NETC__ENETC0_SI4.BDR_NUM[17].RBLENR.R /* Rx BDR 17 length register */
#define NETC__ENETC0_SI4_RB17DCR NETC__ENETC0_SI4.BDR_NUM[17].RBDCR.R /* Rx BDR 17 drop count register */
#define NETC__ENETC0_SI4_RB17IER NETC__ENETC0_SI4.BDR_NUM[17].RBIER.R /* Rx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI4_RB17IDR NETC__ENETC0_SI4.BDR_NUM[17].RBIDR.R /* Rx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI4_RB17ICR0 NETC__ENETC0_SI4.BDR_NUM[17].RBICR0.R /* Rx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI4_RB17ICR1 NETC__ENETC0_SI4.BDR_NUM[17].RBICR1.R /* Rx BDR 17 interrupt coalescing register 1 */

/* NETC_F3_SI5 */
#define NETC__ENETC0_SI5_SIMR NETC__ENETC0_SI5.SIMR.R      /* Station interface mode register */
#define NETC__ENETC0_SI5_SISR NETC__ENETC0_SI5.SISR.R      /* Station interface status register */
#define NETC__ENETC0_SI5_SICTR0 NETC__ENETC0_SI5.SICTR0.R  /* Station interface current time register 0 */
#define NETC__ENETC0_SI5_SICTR1 NETC__ENETC0_SI5.SICTR1.R  /* Station interface current time register 1 */
#define NETC__ENETC0_SI5_SIPCAPR0 NETC__ENETC0_SI5.SIPCAPR0.R /* Station interface port capability register 0 */
#define NETC__ENETC0_SI5_SIPCAPR1 NETC__ENETC0_SI5.SIPCAPR1.R /* Station interface port capability register 1 */
#define NETC__ENETC0_SI5_SIRBGCR NETC__ENETC0_SI5.SIRBGCR.R /* Station interface receive BDR group control register */
#define NETC__ENETC0_SI5_SIBCAR NETC__ENETC0_SI5.SIBCAR.R  /* Station interface buffer cache attribute register */
#define NETC__ENETC0_SI5_SICCAR NETC__ENETC0_SI5.SICCAR.R  /* Station interface command cache attribute register */
#define NETC__ENETC0_SI5_SIPMAR0 NETC__ENETC0_SI5.SIPMAR0.R /* Station interface primary MAC address register 0 */
#define NETC__ENETC0_SI5_SIPMAR1 NETC__ENETC0_SI5.SIPMAR1.R /* Station interface primary MAC address register 1 */
#define NETC__ENETC0_SI5_SICVLANR1 NETC__ENETC0_SI5.SICVLANR1.R /* Station interface custom VLAN register 1 */
#define NETC__ENETC0_SI5_SICVLANR2 NETC__ENETC0_SI5.SICVLANR2.R /* Station interface custom VLAN register 2 */
#define NETC__ENETC0_SI5_SIVLANIPVMR0 NETC__ENETC0_SI5.SIVLANIPVMR0.R /* Station interface VLAN to IPV mapping register 0 */
#define NETC__ENETC0_SI5_SIVLANIPVMR1 NETC__ENETC0_SI5.SIVLANIPVMR1.R /* Station interface VLAN to IPV mapping register 1 */
#define NETC__ENETC0_SI5_SIIPVBDRMR0 NETC__ENETC0_SI5.SIIPVBDRMR0.R /* Station interface IPV to ring mapping register */
#define NETC__ENETC0_SI5_VSIMSGSR NETC__ENETC0_SI5.VSI_A.VSIMSGSR.R /* Virtual station interface message send register */
#define NETC__ENETC0_SI5_VSIMSGRR NETC__ENETC0_SI5.VSI_A.VSIMSGRR.R /* Virtual station interface message receive register */
#define NETC__ENETC0_SI5_VSIMSGSNDAR0 NETC__ENETC0_SI5.VSI_A.VSIMSGSNDAR0.R /* Virtual station interface message send register 0 */
#define NETC__ENETC0_SI5_VSIMSGSNDAR1 NETC__ENETC0_SI5.VSI_A.VSIMSGSNDAR1.R /* Virtual station interface message send address register 1 */
#define NETC__ENETC0_SI5_SIROCT0 NETC__ENETC0_SI5.SIROCT0.R /* Station interface receive octets counter (ifInOctets) 0 */
#define NETC__ENETC0_SI5_SIROCT1 NETC__ENETC0_SI5.SIROCT1.R /* Station interface receive octets counter (ifInOctets) 1 */
#define NETC__ENETC0_SI5_SIRFRM0 NETC__ENETC0_SI5.SIRFRM0.R /* Station interface receive frame counter (aFrameReceivedOK) 0 */
#define NETC__ENETC0_SI5_SIRFRM1 NETC__ENETC0_SI5.SIRFRM1.R /* Station interface receive frame counter (aFrameReceivedOK) 1 */
#define NETC__ENETC0_SI5_SIRUCA0 NETC__ENETC0_SI5.SIRUCA0.R /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
#define NETC__ENETC0_SI5_SIRUCA1 NETC__ENETC0_SI5.SIRUCA1.R /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
#define NETC__ENETC0_SI5_SIRMCA0 NETC__ENETC0_SI5.SIRMCA0.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
#define NETC__ENETC0_SI5_SIRMCA1 NETC__ENETC0_SI5.SIRMCA1.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
#define NETC__ENETC0_SI5_SITOCT0 NETC__ENETC0_SI5.SITOCT0.R /* Station interface transmit octets counter (ifOutOctets) 0 */
#define NETC__ENETC0_SI5_SITOCT1 NETC__ENETC0_SI5.SITOCT1.R /* Station interface transmit octets counter (ifOutOctets) 1 */
#define NETC__ENETC0_SI5_SITFRM0 NETC__ENETC0_SI5.SITFRM0.R /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
#define NETC__ENETC0_SI5_SITFRM1 NETC__ENETC0_SI5.SITFRM1.R /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
#define NETC__ENETC0_SI5_SITUCA0 NETC__ENETC0_SI5.SITUCA0.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
#define NETC__ENETC0_SI5_SITUCA1 NETC__ENETC0_SI5.SITUCA1.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
#define NETC__ENETC0_SI5_SITMCA0 NETC__ENETC0_SI5.SITMCA0.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
#define NETC__ENETC0_SI5_SITMCA1 NETC__ENETC0_SI5.SITMCA1.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
#define NETC__ENETC0_SI5_SIBLPR0 NETC__ENETC0_SI5.SIBLPR[0].R /* Station interface boot loader parameter register 0 */
#define NETC__ENETC0_SI5_SIBLPR1 NETC__ENETC0_SI5.SIBLPR[1].R /* Station interface boot loader parameter register 1 */
#define NETC__ENETC0_SI5_SICBDRMR NETC__ENETC0_SI5.SICBDRMR.R /* Station interface command BDR mode register */
#define NETC__ENETC0_SI5_SICBDRSR NETC__ENETC0_SI5.SICBDRSR.R /* Station interface command BDR status register */
#define NETC__ENETC0_SI5_SICBDRBAR0 NETC__ENETC0_SI5.SICBDRBAR0.R /* Station interface command BDR base address register 0 */
#define NETC__ENETC0_SI5_SICBDRBAR1 NETC__ENETC0_SI5.SICBDRBAR1.R /* Station interface command BDR base address register 1 */
#define NETC__ENETC0_SI5_SICBDRPIR NETC__ENETC0_SI5.SICBDRPIR.R /* Station interface command BDR producer index register */
#define NETC__ENETC0_SI5_SICBDRCIR NETC__ENETC0_SI5.SICBDRCIR.R /* Station interface command BDR consumer index register */
#define NETC__ENETC0_SI5_SICBDRLENR NETC__ENETC0_SI5.SICBDRLENR.R /* Station interface command BDR length register */
#define NETC__ENETC0_SI5_SICBDRIER NETC__ENETC0_SI5.SICBDRIER.R /* Station interface command BDR interrupt enable register */
#define NETC__ENETC0_SI5_SICBDRIDR NETC__ENETC0_SI5.SICBDRIDR.R /* Station interface command BDR interrupt detect register */
#define NETC__ENETC0_SI5_SICAPR0 NETC__ENETC0_SI5.SICAPR0.R /* Station interface capability register 0 */
#define NETC__ENETC0_SI5_SICAPR1 NETC__ENETC0_SI5.SICAPR1.R /* Station interface capability register 1 */
#define NETC__ENETC0_SI5_SICAPR2 NETC__ENETC0_SI5.SICAPR2.R /* Station interface capability register 2 */
#define NETC__ENETC0_SI5_VSIIER NETC__ENETC0_SI5.VSI.VSIIER.R /* Virtual station interface interrupt enable register */
#define NETC__ENETC0_SI5_VSIIDR NETC__ENETC0_SI5.VSI.VSIIDR.R /* Virtual station interface interrupt detect register */
#define NETC__ENETC0_SI5_SITXIDR0 NETC__ENETC0_SI5.SITXIDR0.R /* Station interface transmit interrupt detect register 0 */
#define NETC__ENETC0_SI5_SITXIDR1 NETC__ENETC0_SI5.SITXIDR1.R /* Station interface transmit interrupt detect register 1 */
#define NETC__ENETC0_SI5_SIRXIDR0 NETC__ENETC0_SI5.SIRXIDR0.R /* Station interface receive interrupt detect register 0 */
#define NETC__ENETC0_SI5_SIRXIDR1 NETC__ENETC0_SI5.SIRXIDR1.R /* Station interface receive interrupt detect register 1 */
#define NETC__ENETC0_SI5_SIMSIVR NETC__ENETC0_SI5.SIMSIVR.R /* Station interface MSI-X vector register */
#define NETC__ENETC0_SI5_SICMSIVR NETC__ENETC0_SI5.SICMSIVR.R /* Station interface command MSI-X vector register */
#define NETC__ENETC0_SI5_SITMRIER NETC__ENETC0_SI5.SITMRIER.R /* Station interface timer interrupt enable register */
#define NETC__ENETC0_SI5_SITMRIDR NETC__ENETC0_SI5.SITMRIDR.R /* Station interface timer interrupt detect register */
#define NETC__ENETC0_SI5_SITMRMSIVR NETC__ENETC0_SI5.SITMRMSIVR.R /* Station interface timer MSI-X vector register */
#define NETC__ENETC0_SI5_SIMSITR0VR NETC__ENETC0_SI5.SIMSITRVR[0].R /* Station interface MSI-X transmit ring 0 vector register */
#define NETC__ENETC0_SI5_SIMSITR1VR NETC__ENETC0_SI5.SIMSITRVR[1].R /* Station interface MSI-X transmit ring 1 vector register */
#define NETC__ENETC0_SI5_SIMSITR2VR NETC__ENETC0_SI5.SIMSITRVR[2].R /* Station interface MSI-X transmit ring 2 vector register */
#define NETC__ENETC0_SI5_SIMSITR3VR NETC__ENETC0_SI5.SIMSITRVR[3].R /* Station interface MSI-X transmit ring 3 vector register */
#define NETC__ENETC0_SI5_SIMSITR4VR NETC__ENETC0_SI5.SIMSITRVR[4].R /* Station interface MSI-X transmit ring 4 vector register */
#define NETC__ENETC0_SI5_SIMSITR5VR NETC__ENETC0_SI5.SIMSITRVR[5].R /* Station interface MSI-X transmit ring 5 vector register */
#define NETC__ENETC0_SI5_SIMSITR6VR NETC__ENETC0_SI5.SIMSITRVR[6].R /* Station interface MSI-X transmit ring 6 vector register */
#define NETC__ENETC0_SI5_SIMSITR7VR NETC__ENETC0_SI5.SIMSITRVR[7].R /* Station interface MSI-X transmit ring 7 vector register */
#define NETC__ENETC0_SI5_SIMSITR8VR NETC__ENETC0_SI5.SIMSITRVR[8].R /* Station interface MSI-X transmit ring 8 vector register */
#define NETC__ENETC0_SI5_SIMSITR9VR NETC__ENETC0_SI5.SIMSITRVR[9].R /* Station interface MSI-X transmit ring 9 vector register */
#define NETC__ENETC0_SI5_SIMSITR10VR NETC__ENETC0_SI5.SIMSITRVR[10].R /* Station interface MSI-X transmit ring 10 vector register */
#define NETC__ENETC0_SI5_SIMSITR11VR NETC__ENETC0_SI5.SIMSITRVR[11].R /* Station interface MSI-X transmit ring 11 vector register */
#define NETC__ENETC0_SI5_SIMSITR12VR NETC__ENETC0_SI5.SIMSITRVR[12].R /* Station interface MSI-X transmit ring 12 vector register */
#define NETC__ENETC0_SI5_SIMSITR13VR NETC__ENETC0_SI5.SIMSITRVR[13].R /* Station interface MSI-X transmit ring 13 vector register */
#define NETC__ENETC0_SI5_SIMSITR14VR NETC__ENETC0_SI5.SIMSITRVR[14].R /* Station interface MSI-X transmit ring 14 vector register */
#define NETC__ENETC0_SI5_SIMSITR15VR NETC__ENETC0_SI5.SIMSITRVR[15].R /* Station interface MSI-X transmit ring 15 vector register */
#define NETC__ENETC0_SI5_SIMSITR16VR NETC__ENETC0_SI5.SIMSITRVR[16].R /* Station interface MSI-X transmit ring 16 vector register */
#define NETC__ENETC0_SI5_SIMSITR17VR NETC__ENETC0_SI5.SIMSITRVR[17].R /* Station interface MSI-X transmit ring 17 vector register */
#define NETC__ENETC0_SI5_SIMSIRR0VR NETC__ENETC0_SI5.SIMSIRRVR[0].R /* Station interface MSI-X receive ring 0 vector register */
#define NETC__ENETC0_SI5_SIMSIRR1VR NETC__ENETC0_SI5.SIMSIRRVR[1].R /* Station interface MSI-X receive ring 1 vector register */
#define NETC__ENETC0_SI5_SIMSIRR2VR NETC__ENETC0_SI5.SIMSIRRVR[2].R /* Station interface MSI-X receive ring 2 vector register */
#define NETC__ENETC0_SI5_SIMSIRR3VR NETC__ENETC0_SI5.SIMSIRRVR[3].R /* Station interface MSI-X receive ring 3 vector register */
#define NETC__ENETC0_SI5_SIMSIRR4VR NETC__ENETC0_SI5.SIMSIRRVR[4].R /* Station interface MSI-X receive ring 4 vector register */
#define NETC__ENETC0_SI5_SIMSIRR5VR NETC__ENETC0_SI5.SIMSIRRVR[5].R /* Station interface MSI-X receive ring 5 vector register */
#define NETC__ENETC0_SI5_SIMSIRR6VR NETC__ENETC0_SI5.SIMSIRRVR[6].R /* Station interface MSI-X receive ring 6 vector register */
#define NETC__ENETC0_SI5_SIMSIRR7VR NETC__ENETC0_SI5.SIMSIRRVR[7].R /* Station interface MSI-X receive ring 7 vector register */
#define NETC__ENETC0_SI5_SIMSIRR8VR NETC__ENETC0_SI5.SIMSIRRVR[8].R /* Station interface MSI-X receive ring 8 vector register */
#define NETC__ENETC0_SI5_SIMSIRR9VR NETC__ENETC0_SI5.SIMSIRRVR[9].R /* Station interface MSI-X receive ring 9 vector register */
#define NETC__ENETC0_SI5_SIMSIRR10VR NETC__ENETC0_SI5.SIMSIRRVR[10].R /* Station interface MSI-X receive ring 10 vector register */
#define NETC__ENETC0_SI5_SIMSIRR11VR NETC__ENETC0_SI5.SIMSIRRVR[11].R /* Station interface MSI-X receive ring 11 vector register */
#define NETC__ENETC0_SI5_SIMSIRR12VR NETC__ENETC0_SI5.SIMSIRRVR[12].R /* Station interface MSI-X receive ring 12 vector register */
#define NETC__ENETC0_SI5_SIMSIRR13VR NETC__ENETC0_SI5.SIMSIRRVR[13].R /* Station interface MSI-X receive ring 13 vector register */
#define NETC__ENETC0_SI5_SIMSIRR14VR NETC__ENETC0_SI5.SIMSIRRVR[14].R /* Station interface MSI-X receive ring 14 vector register */
#define NETC__ENETC0_SI5_SIMSIRR15VR NETC__ENETC0_SI5.SIMSIRRVR[15].R /* Station interface MSI-X receive ring 15 vector register */
#define NETC__ENETC0_SI5_SIMSIRR16VR NETC__ENETC0_SI5.SIMSIRRVR[16].R /* Station interface MSI-X receive ring 16 vector register */
#define NETC__ENETC0_SI5_SIMSIRR17VR NETC__ENETC0_SI5.SIMSIRRVR[17].R /* Station interface MSI-X receive ring 17 vector register */
#define NETC__ENETC0_SI5_SICMECR NETC__ENETC0_SI5.SICMECR.R /* Station interface correctable memory error configuration register */
#define NETC__ENETC0_SI5_SICMESR NETC__ENETC0_SI5.SICMESR.R /* Station interface correctable memory error status register */
#define NETC__ENETC0_SI5_SICMECTR NETC__ENETC0_SI5.SICMECTR.R /* Station interface correctable memory error count register */
#define NETC__ENETC0_SI5_SIUPECR NETC__ENETC0_SI5.SIUPECR.R /* Station interface uncorrectable programming error configuration register */
#define NETC__ENETC0_SI5_SIUPESR NETC__ENETC0_SI5.SIUPESR.R /* Station interface uncorrectable programming error status register */
#define NETC__ENETC0_SI5_SIUPECTR NETC__ENETC0_SI5.SIUPECTR.R /* Station interface uncorrectable programming error count register */
#define NETC__ENETC0_SI5_SIUNSBECR NETC__ENETC0_SI5.SIUNSBECR.R /* Station interface uncorrectable non-fatal system bus error configuration register */
#define NETC__ENETC0_SI5_SIUNSBESR NETC__ENETC0_SI5.SIUNSBESR.R /* Station interface uncorrectable non-fatal system bus error status register */
#define NETC__ENETC0_SI5_SIUNSBECTR NETC__ENETC0_SI5.SIUNSBECTR.R /* Station interface uncorrectable non-fatal system bus error count register */
#define NETC__ENETC0_SI5_SIUFSBECR NETC__ENETC0_SI5.SIUFSBECR.R /* Station interface uncorrectable fatal system bus error configuration register */
#define NETC__ENETC0_SI5_SIUFSBESR NETC__ENETC0_SI5.SIUFSBESR.R /* Station interface uncorrectable fatal system bus error status register */
#define NETC__ENETC0_SI5_SIUNMECR NETC__ENETC0_SI5.SIUNMECR.R /* Station interface uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_SI5_SIUNMESR0 NETC__ENETC0_SI5.SIUNMESR0.R /* Station interface uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_SI5_SIUNMESR1 NETC__ENETC0_SI5.SIUNMESR1.R /* Station interface uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_SI5_SIUNMECTR NETC__ENETC0_SI5.SIUNMECTR.R /* Station interface uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_SI5_SIUFMECR NETC__ENETC0_SI5.SIUFMECR.R /* Station interface uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_SI5_SIUFMESR0 NETC__ENETC0_SI5.SIUFMESR0.R /* Station interface uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_SI5_SIUFMESR1 NETC__ENETC0_SI5.SIUFMESR1.R /* Station interface uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_SI5_SIUNIECR NETC__ENETC0_SI5.SIUNIECR.R /* Station interface uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_SI5_SIUNIESR NETC__ENETC0_SI5.SIUNIESR.R /* Station interface uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_SI5_SIUNIECTR NETC__ENETC0_SI5.SIUNIECTR.R /* Station interface uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_SI5_SIUFIECR NETC__ENETC0_SI5.SIUFIECR.R /* Station interface uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_SI5_SIUFIESR NETC__ENETC0_SI5.SIUFIESR.R /* Station interface uncorrectable fatal integrity error status register */
#define NETC__ENETC0_SI5_SIMAFTCAPR NETC__ENETC0_SI5.SIMAFTCAPR.R /* Station interface MAC address filter table capability register */
#define NETC__ENETC0_SI5_SIVFTCAPR NETC__ENETC0_SI5.SIVFTCAPR.R /* Station interface VLAN filter table capability register */
#define NETC__ENETC0_SI5_SIRFSCAPR NETC__ENETC0_SI5.SIRFSCAPR.R /* Station interface RFS capability register */
#define NETC__ENETC0_SI5_TB0MR NETC__ENETC0_SI5.BDR_NUM[0].TBMR.R /* Tx BDR 0 mode register */
#define NETC__ENETC0_SI5_TB0SR NETC__ENETC0_SI5.BDR_NUM[0].TBSR.R /* Tx BDR 0 status register */
#define NETC__ENETC0_SI5_TB0BAR0 NETC__ENETC0_SI5.BDR_NUM[0].TBBAR0.R /* Tx BDR 0 base address register 0 */
#define NETC__ENETC0_SI5_TB0BAR1 NETC__ENETC0_SI5.BDR_NUM[0].TBBAR1.R /* Tx BDR 0 base address register 1 */
#define NETC__ENETC0_SI5_TB0PIR NETC__ENETC0_SI5.BDR_NUM[0].TBPIR.R /* Tx BDR 0 producer index register */
#define NETC__ENETC0_SI5_TB0CIR NETC__ENETC0_SI5.BDR_NUM[0].TBCIR.R /* Tx BDR 0 consumer index register */
#define NETC__ENETC0_SI5_TB0LENR NETC__ENETC0_SI5.BDR_NUM[0].TBLENR.R /* Tx BDR 0 length register */
#define NETC__ENETC0_SI5_TB0IER NETC__ENETC0_SI5.BDR_NUM[0].TBIER.R /* Tx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI5_TB0IDR NETC__ENETC0_SI5.BDR_NUM[0].TBIDR.R /* Tx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI5_TB0ICR0 NETC__ENETC0_SI5.BDR_NUM[0].TBICR0.R /* Tx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB0ICR1 NETC__ENETC0_SI5.BDR_NUM[0].TBICR1.R /* Tx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB0MR NETC__ENETC0_SI5.BDR_NUM[0].RBMR.R /* Rx BDR 0 mode register */
#define NETC__ENETC0_SI5_RB0SR NETC__ENETC0_SI5.BDR_NUM[0].RBSR.R /* Rx BDR 0 status register */
#define NETC__ENETC0_SI5_RB0BSR NETC__ENETC0_SI5.BDR_NUM[0].RBBSR.R /* Rx BDR 0 buffer size register */
#define NETC__ENETC0_SI5_RB0CIR NETC__ENETC0_SI5.BDR_NUM[0].RBCIR.R /* Rx BDR 0 consumer index register */
#define NETC__ENETC0_SI5_RB0BAR0 NETC__ENETC0_SI5.BDR_NUM[0].RBBAR0.R /* Rx BDR 0 base address register 0 */
#define NETC__ENETC0_SI5_RB0BAR1 NETC__ENETC0_SI5.BDR_NUM[0].RBBAR1.R /* Rx BDR 0 base address register 1 */
#define NETC__ENETC0_SI5_RB0PIR NETC__ENETC0_SI5.BDR_NUM[0].RBPIR.R /* Rx BDR 0 producer index register */
#define NETC__ENETC0_SI5_RB0LENR NETC__ENETC0_SI5.BDR_NUM[0].RBLENR.R /* Rx BDR 0 length register */
#define NETC__ENETC0_SI5_RB0DCR NETC__ENETC0_SI5.BDR_NUM[0].RBDCR.R /* Rx BDR 0 drop count register */
#define NETC__ENETC0_SI5_RB0IER NETC__ENETC0_SI5.BDR_NUM[0].RBIER.R /* Rx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI5_RB0IDR NETC__ENETC0_SI5.BDR_NUM[0].RBIDR.R /* Rx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI5_RB0ICR0 NETC__ENETC0_SI5.BDR_NUM[0].RBICR0.R /* Rx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB0ICR1 NETC__ENETC0_SI5.BDR_NUM[0].RBICR1.R /* Rx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB1MR NETC__ENETC0_SI5.BDR_NUM[1].TBMR.R /* Tx BDR 1 mode register */
#define NETC__ENETC0_SI5_TB1SR NETC__ENETC0_SI5.BDR_NUM[1].TBSR.R /* Tx BDR 1 status register */
#define NETC__ENETC0_SI5_TB1BAR0 NETC__ENETC0_SI5.BDR_NUM[1].TBBAR0.R /* Tx BDR 1 base address register 0 */
#define NETC__ENETC0_SI5_TB1BAR1 NETC__ENETC0_SI5.BDR_NUM[1].TBBAR1.R /* Tx BDR 1 base address register 1 */
#define NETC__ENETC0_SI5_TB1PIR NETC__ENETC0_SI5.BDR_NUM[1].TBPIR.R /* Tx BDR 1 producer index register */
#define NETC__ENETC0_SI5_TB1CIR NETC__ENETC0_SI5.BDR_NUM[1].TBCIR.R /* Tx BDR 1 consumer index register */
#define NETC__ENETC0_SI5_TB1LENR NETC__ENETC0_SI5.BDR_NUM[1].TBLENR.R /* Tx BDR 1 length register */
#define NETC__ENETC0_SI5_TB1IER NETC__ENETC0_SI5.BDR_NUM[1].TBIER.R /* Tx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI5_TB1IDR NETC__ENETC0_SI5.BDR_NUM[1].TBIDR.R /* Tx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI5_TB1ICR0 NETC__ENETC0_SI5.BDR_NUM[1].TBICR0.R /* Tx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB1ICR1 NETC__ENETC0_SI5.BDR_NUM[1].TBICR1.R /* Tx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB1MR NETC__ENETC0_SI5.BDR_NUM[1].RBMR.R /* Rx BDR 1 mode register */
#define NETC__ENETC0_SI5_RB1SR NETC__ENETC0_SI5.BDR_NUM[1].RBSR.R /* Rx BDR 1 status register */
#define NETC__ENETC0_SI5_RB1BSR NETC__ENETC0_SI5.BDR_NUM[1].RBBSR.R /* Rx BDR 1 buffer size register */
#define NETC__ENETC0_SI5_RB1CIR NETC__ENETC0_SI5.BDR_NUM[1].RBCIR.R /* Rx BDR 1 consumer index register */
#define NETC__ENETC0_SI5_RB1BAR0 NETC__ENETC0_SI5.BDR_NUM[1].RBBAR0.R /* Rx BDR 1 base address register 0 */
#define NETC__ENETC0_SI5_RB1BAR1 NETC__ENETC0_SI5.BDR_NUM[1].RBBAR1.R /* Rx BDR 1 base address register 1 */
#define NETC__ENETC0_SI5_RB1PIR NETC__ENETC0_SI5.BDR_NUM[1].RBPIR.R /* Rx BDR 1 producer index register */
#define NETC__ENETC0_SI5_RB1LENR NETC__ENETC0_SI5.BDR_NUM[1].RBLENR.R /* Rx BDR 1 length register */
#define NETC__ENETC0_SI5_RB1DCR NETC__ENETC0_SI5.BDR_NUM[1].RBDCR.R /* Rx BDR 1 drop count register */
#define NETC__ENETC0_SI5_RB1IER NETC__ENETC0_SI5.BDR_NUM[1].RBIER.R /* Rx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI5_RB1IDR NETC__ENETC0_SI5.BDR_NUM[1].RBIDR.R /* Rx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI5_RB1ICR0 NETC__ENETC0_SI5.BDR_NUM[1].RBICR0.R /* Rx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB1ICR1 NETC__ENETC0_SI5.BDR_NUM[1].RBICR1.R /* Rx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB2MR NETC__ENETC0_SI5.BDR_NUM[2].TBMR.R /* Tx BDR 2 mode register */
#define NETC__ENETC0_SI5_TB2SR NETC__ENETC0_SI5.BDR_NUM[2].TBSR.R /* Tx BDR 2 status register */
#define NETC__ENETC0_SI5_TB2BAR0 NETC__ENETC0_SI5.BDR_NUM[2].TBBAR0.R /* Tx BDR 2 base address register 0 */
#define NETC__ENETC0_SI5_TB2BAR1 NETC__ENETC0_SI5.BDR_NUM[2].TBBAR1.R /* Tx BDR 2 base address register 1 */
#define NETC__ENETC0_SI5_TB2PIR NETC__ENETC0_SI5.BDR_NUM[2].TBPIR.R /* Tx BDR 2 producer index register */
#define NETC__ENETC0_SI5_TB2CIR NETC__ENETC0_SI5.BDR_NUM[2].TBCIR.R /* Tx BDR 2 consumer index register */
#define NETC__ENETC0_SI5_TB2LENR NETC__ENETC0_SI5.BDR_NUM[2].TBLENR.R /* Tx BDR 2 length register */
#define NETC__ENETC0_SI5_TB2IER NETC__ENETC0_SI5.BDR_NUM[2].TBIER.R /* Tx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI5_TB2IDR NETC__ENETC0_SI5.BDR_NUM[2].TBIDR.R /* Tx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI5_TB2ICR0 NETC__ENETC0_SI5.BDR_NUM[2].TBICR0.R /* Tx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB2ICR1 NETC__ENETC0_SI5.BDR_NUM[2].TBICR1.R /* Tx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB2MR NETC__ENETC0_SI5.BDR_NUM[2].RBMR.R /* Rx BDR 2 mode register */
#define NETC__ENETC0_SI5_RB2SR NETC__ENETC0_SI5.BDR_NUM[2].RBSR.R /* Rx BDR 2 status register */
#define NETC__ENETC0_SI5_RB2BSR NETC__ENETC0_SI5.BDR_NUM[2].RBBSR.R /* Rx BDR 2 buffer size register */
#define NETC__ENETC0_SI5_RB2CIR NETC__ENETC0_SI5.BDR_NUM[2].RBCIR.R /* Rx BDR 2 consumer index register */
#define NETC__ENETC0_SI5_RB2BAR0 NETC__ENETC0_SI5.BDR_NUM[2].RBBAR0.R /* Rx BDR 2 base address register 0 */
#define NETC__ENETC0_SI5_RB2BAR1 NETC__ENETC0_SI5.BDR_NUM[2].RBBAR1.R /* Rx BDR 2 base address register 1 */
#define NETC__ENETC0_SI5_RB2PIR NETC__ENETC0_SI5.BDR_NUM[2].RBPIR.R /* Rx BDR 2 producer index register */
#define NETC__ENETC0_SI5_RB2LENR NETC__ENETC0_SI5.BDR_NUM[2].RBLENR.R /* Rx BDR 2 length register */
#define NETC__ENETC0_SI5_RB2DCR NETC__ENETC0_SI5.BDR_NUM[2].RBDCR.R /* Rx BDR 2 drop count register */
#define NETC__ENETC0_SI5_RB2IER NETC__ENETC0_SI5.BDR_NUM[2].RBIER.R /* Rx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI5_RB2IDR NETC__ENETC0_SI5.BDR_NUM[2].RBIDR.R /* Rx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI5_RB2ICR0 NETC__ENETC0_SI5.BDR_NUM[2].RBICR0.R /* Rx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB2ICR1 NETC__ENETC0_SI5.BDR_NUM[2].RBICR1.R /* Rx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB3MR NETC__ENETC0_SI5.BDR_NUM[3].TBMR.R /* Tx BDR 3 mode register */
#define NETC__ENETC0_SI5_TB3SR NETC__ENETC0_SI5.BDR_NUM[3].TBSR.R /* Tx BDR 3 status register */
#define NETC__ENETC0_SI5_TB3BAR0 NETC__ENETC0_SI5.BDR_NUM[3].TBBAR0.R /* Tx BDR 3 base address register 0 */
#define NETC__ENETC0_SI5_TB3BAR1 NETC__ENETC0_SI5.BDR_NUM[3].TBBAR1.R /* Tx BDR 3 base address register 1 */
#define NETC__ENETC0_SI5_TB3PIR NETC__ENETC0_SI5.BDR_NUM[3].TBPIR.R /* Tx BDR 3 producer index register */
#define NETC__ENETC0_SI5_TB3CIR NETC__ENETC0_SI5.BDR_NUM[3].TBCIR.R /* Tx BDR 3 consumer index register */
#define NETC__ENETC0_SI5_TB3LENR NETC__ENETC0_SI5.BDR_NUM[3].TBLENR.R /* Tx BDR 3 length register */
#define NETC__ENETC0_SI5_TB3IER NETC__ENETC0_SI5.BDR_NUM[3].TBIER.R /* Tx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI5_TB3IDR NETC__ENETC0_SI5.BDR_NUM[3].TBIDR.R /* Tx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI5_TB3ICR0 NETC__ENETC0_SI5.BDR_NUM[3].TBICR0.R /* Tx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB3ICR1 NETC__ENETC0_SI5.BDR_NUM[3].TBICR1.R /* Tx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB3MR NETC__ENETC0_SI5.BDR_NUM[3].RBMR.R /* Rx BDR 3 mode register */
#define NETC__ENETC0_SI5_RB3SR NETC__ENETC0_SI5.BDR_NUM[3].RBSR.R /* Rx BDR 3 status register */
#define NETC__ENETC0_SI5_RB3BSR NETC__ENETC0_SI5.BDR_NUM[3].RBBSR.R /* Rx BDR 3 buffer size register */
#define NETC__ENETC0_SI5_RB3CIR NETC__ENETC0_SI5.BDR_NUM[3].RBCIR.R /* Rx BDR 3 consumer index register */
#define NETC__ENETC0_SI5_RB3BAR0 NETC__ENETC0_SI5.BDR_NUM[3].RBBAR0.R /* Rx BDR 3 base address register 0 */
#define NETC__ENETC0_SI5_RB3BAR1 NETC__ENETC0_SI5.BDR_NUM[3].RBBAR1.R /* Rx BDR 3 base address register 1 */
#define NETC__ENETC0_SI5_RB3PIR NETC__ENETC0_SI5.BDR_NUM[3].RBPIR.R /* Rx BDR 3 producer index register */
#define NETC__ENETC0_SI5_RB3LENR NETC__ENETC0_SI5.BDR_NUM[3].RBLENR.R /* Rx BDR 3 length register */
#define NETC__ENETC0_SI5_RB3DCR NETC__ENETC0_SI5.BDR_NUM[3].RBDCR.R /* Rx BDR 3 drop count register */
#define NETC__ENETC0_SI5_RB3IER NETC__ENETC0_SI5.BDR_NUM[3].RBIER.R /* Rx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI5_RB3IDR NETC__ENETC0_SI5.BDR_NUM[3].RBIDR.R /* Rx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI5_RB3ICR0 NETC__ENETC0_SI5.BDR_NUM[3].RBICR0.R /* Rx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB3ICR1 NETC__ENETC0_SI5.BDR_NUM[3].RBICR1.R /* Rx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB4MR NETC__ENETC0_SI5.BDR_NUM[4].TBMR.R /* Tx BDR 4 mode register */
#define NETC__ENETC0_SI5_TB4SR NETC__ENETC0_SI5.BDR_NUM[4].TBSR.R /* Tx BDR 4 status register */
#define NETC__ENETC0_SI5_TB4BAR0 NETC__ENETC0_SI5.BDR_NUM[4].TBBAR0.R /* Tx BDR 4 base address register 0 */
#define NETC__ENETC0_SI5_TB4BAR1 NETC__ENETC0_SI5.BDR_NUM[4].TBBAR1.R /* Tx BDR 4 base address register 1 */
#define NETC__ENETC0_SI5_TB4PIR NETC__ENETC0_SI5.BDR_NUM[4].TBPIR.R /* Tx BDR 4 producer index register */
#define NETC__ENETC0_SI5_TB4CIR NETC__ENETC0_SI5.BDR_NUM[4].TBCIR.R /* Tx BDR 4 consumer index register */
#define NETC__ENETC0_SI5_TB4LENR NETC__ENETC0_SI5.BDR_NUM[4].TBLENR.R /* Tx BDR 4 length register */
#define NETC__ENETC0_SI5_TB4IER NETC__ENETC0_SI5.BDR_NUM[4].TBIER.R /* Tx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI5_TB4IDR NETC__ENETC0_SI5.BDR_NUM[4].TBIDR.R /* Tx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI5_TB4ICR0 NETC__ENETC0_SI5.BDR_NUM[4].TBICR0.R /* Tx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB4ICR1 NETC__ENETC0_SI5.BDR_NUM[4].TBICR1.R /* Tx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB4MR NETC__ENETC0_SI5.BDR_NUM[4].RBMR.R /* Rx BDR 4 mode register */
#define NETC__ENETC0_SI5_RB4SR NETC__ENETC0_SI5.BDR_NUM[4].RBSR.R /* Rx BDR 4 status register */
#define NETC__ENETC0_SI5_RB4BSR NETC__ENETC0_SI5.BDR_NUM[4].RBBSR.R /* Rx BDR 4 buffer size register */
#define NETC__ENETC0_SI5_RB4CIR NETC__ENETC0_SI5.BDR_NUM[4].RBCIR.R /* Rx BDR 4 consumer index register */
#define NETC__ENETC0_SI5_RB4BAR0 NETC__ENETC0_SI5.BDR_NUM[4].RBBAR0.R /* Rx BDR 4 base address register 0 */
#define NETC__ENETC0_SI5_RB4BAR1 NETC__ENETC0_SI5.BDR_NUM[4].RBBAR1.R /* Rx BDR 4 base address register 1 */
#define NETC__ENETC0_SI5_RB4PIR NETC__ENETC0_SI5.BDR_NUM[4].RBPIR.R /* Rx BDR 4 producer index register */
#define NETC__ENETC0_SI5_RB4LENR NETC__ENETC0_SI5.BDR_NUM[4].RBLENR.R /* Rx BDR 4 length register */
#define NETC__ENETC0_SI5_RB4DCR NETC__ENETC0_SI5.BDR_NUM[4].RBDCR.R /* Rx BDR 4 drop count register */
#define NETC__ENETC0_SI5_RB4IER NETC__ENETC0_SI5.BDR_NUM[4].RBIER.R /* Rx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI5_RB4IDR NETC__ENETC0_SI5.BDR_NUM[4].RBIDR.R /* Rx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI5_RB4ICR0 NETC__ENETC0_SI5.BDR_NUM[4].RBICR0.R /* Rx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB4ICR1 NETC__ENETC0_SI5.BDR_NUM[4].RBICR1.R /* Rx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB5MR NETC__ENETC0_SI5.BDR_NUM[5].TBMR.R /* Tx BDR 5 mode register */
#define NETC__ENETC0_SI5_TB5SR NETC__ENETC0_SI5.BDR_NUM[5].TBSR.R /* Tx BDR 5 status register */
#define NETC__ENETC0_SI5_TB5BAR0 NETC__ENETC0_SI5.BDR_NUM[5].TBBAR0.R /* Tx BDR 5 base address register 0 */
#define NETC__ENETC0_SI5_TB5BAR1 NETC__ENETC0_SI5.BDR_NUM[5].TBBAR1.R /* Tx BDR 5 base address register 1 */
#define NETC__ENETC0_SI5_TB5PIR NETC__ENETC0_SI5.BDR_NUM[5].TBPIR.R /* Tx BDR 5 producer index register */
#define NETC__ENETC0_SI5_TB5CIR NETC__ENETC0_SI5.BDR_NUM[5].TBCIR.R /* Tx BDR 5 consumer index register */
#define NETC__ENETC0_SI5_TB5LENR NETC__ENETC0_SI5.BDR_NUM[5].TBLENR.R /* Tx BDR 5 length register */
#define NETC__ENETC0_SI5_TB5IER NETC__ENETC0_SI5.BDR_NUM[5].TBIER.R /* Tx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI5_TB5IDR NETC__ENETC0_SI5.BDR_NUM[5].TBIDR.R /* Tx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI5_TB5ICR0 NETC__ENETC0_SI5.BDR_NUM[5].TBICR0.R /* Tx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB5ICR1 NETC__ENETC0_SI5.BDR_NUM[5].TBICR1.R /* Tx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB5MR NETC__ENETC0_SI5.BDR_NUM[5].RBMR.R /* Rx BDR 5 mode register */
#define NETC__ENETC0_SI5_RB5SR NETC__ENETC0_SI5.BDR_NUM[5].RBSR.R /* Rx BDR 5 status register */
#define NETC__ENETC0_SI5_RB5BSR NETC__ENETC0_SI5.BDR_NUM[5].RBBSR.R /* Rx BDR 5 buffer size register */
#define NETC__ENETC0_SI5_RB5CIR NETC__ENETC0_SI5.BDR_NUM[5].RBCIR.R /* Rx BDR 5 consumer index register */
#define NETC__ENETC0_SI5_RB5BAR0 NETC__ENETC0_SI5.BDR_NUM[5].RBBAR0.R /* Rx BDR 5 base address register 0 */
#define NETC__ENETC0_SI5_RB5BAR1 NETC__ENETC0_SI5.BDR_NUM[5].RBBAR1.R /* Rx BDR 5 base address register 1 */
#define NETC__ENETC0_SI5_RB5PIR NETC__ENETC0_SI5.BDR_NUM[5].RBPIR.R /* Rx BDR 5 producer index register */
#define NETC__ENETC0_SI5_RB5LENR NETC__ENETC0_SI5.BDR_NUM[5].RBLENR.R /* Rx BDR 5 length register */
#define NETC__ENETC0_SI5_RB5DCR NETC__ENETC0_SI5.BDR_NUM[5].RBDCR.R /* Rx BDR 5 drop count register */
#define NETC__ENETC0_SI5_RB5IER NETC__ENETC0_SI5.BDR_NUM[5].RBIER.R /* Rx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI5_RB5IDR NETC__ENETC0_SI5.BDR_NUM[5].RBIDR.R /* Rx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI5_RB5ICR0 NETC__ENETC0_SI5.BDR_NUM[5].RBICR0.R /* Rx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB5ICR1 NETC__ENETC0_SI5.BDR_NUM[5].RBICR1.R /* Rx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB6MR NETC__ENETC0_SI5.BDR_NUM[6].TBMR.R /* Tx BDR 6 mode register */
#define NETC__ENETC0_SI5_TB6SR NETC__ENETC0_SI5.BDR_NUM[6].TBSR.R /* Tx BDR 6 status register */
#define NETC__ENETC0_SI5_TB6BAR0 NETC__ENETC0_SI5.BDR_NUM[6].TBBAR0.R /* Tx BDR 6 base address register 0 */
#define NETC__ENETC0_SI5_TB6BAR1 NETC__ENETC0_SI5.BDR_NUM[6].TBBAR1.R /* Tx BDR 6 base address register 1 */
#define NETC__ENETC0_SI5_TB6PIR NETC__ENETC0_SI5.BDR_NUM[6].TBPIR.R /* Tx BDR 6 producer index register */
#define NETC__ENETC0_SI5_TB6CIR NETC__ENETC0_SI5.BDR_NUM[6].TBCIR.R /* Tx BDR 6 consumer index register */
#define NETC__ENETC0_SI5_TB6LENR NETC__ENETC0_SI5.BDR_NUM[6].TBLENR.R /* Tx BDR 6 length register */
#define NETC__ENETC0_SI5_TB6IER NETC__ENETC0_SI5.BDR_NUM[6].TBIER.R /* Tx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI5_TB6IDR NETC__ENETC0_SI5.BDR_NUM[6].TBIDR.R /* Tx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI5_TB6ICR0 NETC__ENETC0_SI5.BDR_NUM[6].TBICR0.R /* Tx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB6ICR1 NETC__ENETC0_SI5.BDR_NUM[6].TBICR1.R /* Tx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB6MR NETC__ENETC0_SI5.BDR_NUM[6].RBMR.R /* Rx BDR 6 mode register */
#define NETC__ENETC0_SI5_RB6SR NETC__ENETC0_SI5.BDR_NUM[6].RBSR.R /* Rx BDR 6 status register */
#define NETC__ENETC0_SI5_RB6BSR NETC__ENETC0_SI5.BDR_NUM[6].RBBSR.R /* Rx BDR 6 buffer size register */
#define NETC__ENETC0_SI5_RB6CIR NETC__ENETC0_SI5.BDR_NUM[6].RBCIR.R /* Rx BDR 6 consumer index register */
#define NETC__ENETC0_SI5_RB6BAR0 NETC__ENETC0_SI5.BDR_NUM[6].RBBAR0.R /* Rx BDR 6 base address register 0 */
#define NETC__ENETC0_SI5_RB6BAR1 NETC__ENETC0_SI5.BDR_NUM[6].RBBAR1.R /* Rx BDR 6 base address register 1 */
#define NETC__ENETC0_SI5_RB6PIR NETC__ENETC0_SI5.BDR_NUM[6].RBPIR.R /* Rx BDR 6 producer index register */
#define NETC__ENETC0_SI5_RB6LENR NETC__ENETC0_SI5.BDR_NUM[6].RBLENR.R /* Rx BDR 6 length register */
#define NETC__ENETC0_SI5_RB6DCR NETC__ENETC0_SI5.BDR_NUM[6].RBDCR.R /* Rx BDR 6 drop count register */
#define NETC__ENETC0_SI5_RB6IER NETC__ENETC0_SI5.BDR_NUM[6].RBIER.R /* Rx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI5_RB6IDR NETC__ENETC0_SI5.BDR_NUM[6].RBIDR.R /* Rx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI5_RB6ICR0 NETC__ENETC0_SI5.BDR_NUM[6].RBICR0.R /* Rx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB6ICR1 NETC__ENETC0_SI5.BDR_NUM[6].RBICR1.R /* Rx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB7MR NETC__ENETC0_SI5.BDR_NUM[7].TBMR.R /* Tx BDR 7 mode register */
#define NETC__ENETC0_SI5_TB7SR NETC__ENETC0_SI5.BDR_NUM[7].TBSR.R /* Tx BDR 7 status register */
#define NETC__ENETC0_SI5_TB7BAR0 NETC__ENETC0_SI5.BDR_NUM[7].TBBAR0.R /* Tx BDR 7 base address register 0 */
#define NETC__ENETC0_SI5_TB7BAR1 NETC__ENETC0_SI5.BDR_NUM[7].TBBAR1.R /* Tx BDR 7 base address register 1 */
#define NETC__ENETC0_SI5_TB7PIR NETC__ENETC0_SI5.BDR_NUM[7].TBPIR.R /* Tx BDR 7 producer index register */
#define NETC__ENETC0_SI5_TB7CIR NETC__ENETC0_SI5.BDR_NUM[7].TBCIR.R /* Tx BDR 7 consumer index register */
#define NETC__ENETC0_SI5_TB7LENR NETC__ENETC0_SI5.BDR_NUM[7].TBLENR.R /* Tx BDR 7 length register */
#define NETC__ENETC0_SI5_TB7IER NETC__ENETC0_SI5.BDR_NUM[7].TBIER.R /* Tx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI5_TB7IDR NETC__ENETC0_SI5.BDR_NUM[7].TBIDR.R /* Tx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI5_TB7ICR0 NETC__ENETC0_SI5.BDR_NUM[7].TBICR0.R /* Tx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB7ICR1 NETC__ENETC0_SI5.BDR_NUM[7].TBICR1.R /* Tx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB7MR NETC__ENETC0_SI5.BDR_NUM[7].RBMR.R /* Rx BDR 7 mode register */
#define NETC__ENETC0_SI5_RB7SR NETC__ENETC0_SI5.BDR_NUM[7].RBSR.R /* Rx BDR 7 status register */
#define NETC__ENETC0_SI5_RB7BSR NETC__ENETC0_SI5.BDR_NUM[7].RBBSR.R /* Rx BDR 7 buffer size register */
#define NETC__ENETC0_SI5_RB7CIR NETC__ENETC0_SI5.BDR_NUM[7].RBCIR.R /* Rx BDR 7 consumer index register */
#define NETC__ENETC0_SI5_RB7BAR0 NETC__ENETC0_SI5.BDR_NUM[7].RBBAR0.R /* Rx BDR 7 base address register 0 */
#define NETC__ENETC0_SI5_RB7BAR1 NETC__ENETC0_SI5.BDR_NUM[7].RBBAR1.R /* Rx BDR 7 base address register 1 */
#define NETC__ENETC0_SI5_RB7PIR NETC__ENETC0_SI5.BDR_NUM[7].RBPIR.R /* Rx BDR 7 producer index register */
#define NETC__ENETC0_SI5_RB7LENR NETC__ENETC0_SI5.BDR_NUM[7].RBLENR.R /* Rx BDR 7 length register */
#define NETC__ENETC0_SI5_RB7DCR NETC__ENETC0_SI5.BDR_NUM[7].RBDCR.R /* Rx BDR 7 drop count register */
#define NETC__ENETC0_SI5_RB7IER NETC__ENETC0_SI5.BDR_NUM[7].RBIER.R /* Rx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI5_RB7IDR NETC__ENETC0_SI5.BDR_NUM[7].RBIDR.R /* Rx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI5_RB7ICR0 NETC__ENETC0_SI5.BDR_NUM[7].RBICR0.R /* Rx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB7ICR1 NETC__ENETC0_SI5.BDR_NUM[7].RBICR1.R /* Rx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB8MR NETC__ENETC0_SI5.BDR_NUM[8].TBMR.R /* Tx BDR 8 mode register */
#define NETC__ENETC0_SI5_TB8SR NETC__ENETC0_SI5.BDR_NUM[8].TBSR.R /* Tx BDR 8 status register */
#define NETC__ENETC0_SI5_TB8BAR0 NETC__ENETC0_SI5.BDR_NUM[8].TBBAR0.R /* Tx BDR 8 base address register 0 */
#define NETC__ENETC0_SI5_TB8BAR1 NETC__ENETC0_SI5.BDR_NUM[8].TBBAR1.R /* Tx BDR 8 base address register 1 */
#define NETC__ENETC0_SI5_TB8PIR NETC__ENETC0_SI5.BDR_NUM[8].TBPIR.R /* Tx BDR 8 producer index register */
#define NETC__ENETC0_SI5_TB8CIR NETC__ENETC0_SI5.BDR_NUM[8].TBCIR.R /* Tx BDR 8 consumer index register */
#define NETC__ENETC0_SI5_TB8LENR NETC__ENETC0_SI5.BDR_NUM[8].TBLENR.R /* Tx BDR 8 length register */
#define NETC__ENETC0_SI5_TB8IER NETC__ENETC0_SI5.BDR_NUM[8].TBIER.R /* Tx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI5_TB8IDR NETC__ENETC0_SI5.BDR_NUM[8].TBIDR.R /* Tx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI5_TB8ICR0 NETC__ENETC0_SI5.BDR_NUM[8].TBICR0.R /* Tx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB8ICR1 NETC__ENETC0_SI5.BDR_NUM[8].TBICR1.R /* Tx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB8MR NETC__ENETC0_SI5.BDR_NUM[8].RBMR.R /* Rx BDR 8 mode register */
#define NETC__ENETC0_SI5_RB8SR NETC__ENETC0_SI5.BDR_NUM[8].RBSR.R /* Rx BDR 8 status register */
#define NETC__ENETC0_SI5_RB8BSR NETC__ENETC0_SI5.BDR_NUM[8].RBBSR.R /* Rx BDR 8 buffer size register */
#define NETC__ENETC0_SI5_RB8CIR NETC__ENETC0_SI5.BDR_NUM[8].RBCIR.R /* Rx BDR 8 consumer index register */
#define NETC__ENETC0_SI5_RB8BAR0 NETC__ENETC0_SI5.BDR_NUM[8].RBBAR0.R /* Rx BDR 8 base address register 0 */
#define NETC__ENETC0_SI5_RB8BAR1 NETC__ENETC0_SI5.BDR_NUM[8].RBBAR1.R /* Rx BDR 8 base address register 1 */
#define NETC__ENETC0_SI5_RB8PIR NETC__ENETC0_SI5.BDR_NUM[8].RBPIR.R /* Rx BDR 8 producer index register */
#define NETC__ENETC0_SI5_RB8LENR NETC__ENETC0_SI5.BDR_NUM[8].RBLENR.R /* Rx BDR 8 length register */
#define NETC__ENETC0_SI5_RB8DCR NETC__ENETC0_SI5.BDR_NUM[8].RBDCR.R /* Rx BDR 8 drop count register */
#define NETC__ENETC0_SI5_RB8IER NETC__ENETC0_SI5.BDR_NUM[8].RBIER.R /* Rx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI5_RB8IDR NETC__ENETC0_SI5.BDR_NUM[8].RBIDR.R /* Rx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI5_RB8ICR0 NETC__ENETC0_SI5.BDR_NUM[8].RBICR0.R /* Rx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB8ICR1 NETC__ENETC0_SI5.BDR_NUM[8].RBICR1.R /* Rx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB9MR NETC__ENETC0_SI5.BDR_NUM[9].TBMR.R /* Tx BDR 9 mode register */
#define NETC__ENETC0_SI5_TB9SR NETC__ENETC0_SI5.BDR_NUM[9].TBSR.R /* Tx BDR 9 status register */
#define NETC__ENETC0_SI5_TB9BAR0 NETC__ENETC0_SI5.BDR_NUM[9].TBBAR0.R /* Tx BDR 9 base address register 0 */
#define NETC__ENETC0_SI5_TB9BAR1 NETC__ENETC0_SI5.BDR_NUM[9].TBBAR1.R /* Tx BDR 9 base address register 1 */
#define NETC__ENETC0_SI5_TB9PIR NETC__ENETC0_SI5.BDR_NUM[9].TBPIR.R /* Tx BDR 9 producer index register */
#define NETC__ENETC0_SI5_TB9CIR NETC__ENETC0_SI5.BDR_NUM[9].TBCIR.R /* Tx BDR 9 consumer index register */
#define NETC__ENETC0_SI5_TB9LENR NETC__ENETC0_SI5.BDR_NUM[9].TBLENR.R /* Tx BDR 9 length register */
#define NETC__ENETC0_SI5_TB9IER NETC__ENETC0_SI5.BDR_NUM[9].TBIER.R /* Tx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI5_TB9IDR NETC__ENETC0_SI5.BDR_NUM[9].TBIDR.R /* Tx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI5_TB9ICR0 NETC__ENETC0_SI5.BDR_NUM[9].TBICR0.R /* Tx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB9ICR1 NETC__ENETC0_SI5.BDR_NUM[9].TBICR1.R /* Tx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB9MR NETC__ENETC0_SI5.BDR_NUM[9].RBMR.R /* Rx BDR 9 mode register */
#define NETC__ENETC0_SI5_RB9SR NETC__ENETC0_SI5.BDR_NUM[9].RBSR.R /* Rx BDR 9 status register */
#define NETC__ENETC0_SI5_RB9BSR NETC__ENETC0_SI5.BDR_NUM[9].RBBSR.R /* Rx BDR 9 buffer size register */
#define NETC__ENETC0_SI5_RB9CIR NETC__ENETC0_SI5.BDR_NUM[9].RBCIR.R /* Rx BDR 9 consumer index register */
#define NETC__ENETC0_SI5_RB9BAR0 NETC__ENETC0_SI5.BDR_NUM[9].RBBAR0.R /* Rx BDR 9 base address register 0 */
#define NETC__ENETC0_SI5_RB9BAR1 NETC__ENETC0_SI5.BDR_NUM[9].RBBAR1.R /* Rx BDR 9 base address register 1 */
#define NETC__ENETC0_SI5_RB9PIR NETC__ENETC0_SI5.BDR_NUM[9].RBPIR.R /* Rx BDR 9 producer index register */
#define NETC__ENETC0_SI5_RB9LENR NETC__ENETC0_SI5.BDR_NUM[9].RBLENR.R /* Rx BDR 9 length register */
#define NETC__ENETC0_SI5_RB9DCR NETC__ENETC0_SI5.BDR_NUM[9].RBDCR.R /* Rx BDR 9 drop count register */
#define NETC__ENETC0_SI5_RB9IER NETC__ENETC0_SI5.BDR_NUM[9].RBIER.R /* Rx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI5_RB9IDR NETC__ENETC0_SI5.BDR_NUM[9].RBIDR.R /* Rx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI5_RB9ICR0 NETC__ENETC0_SI5.BDR_NUM[9].RBICR0.R /* Rx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB9ICR1 NETC__ENETC0_SI5.BDR_NUM[9].RBICR1.R /* Rx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB10MR NETC__ENETC0_SI5.BDR_NUM[10].TBMR.R /* Tx BDR 10 mode register */
#define NETC__ENETC0_SI5_TB10SR NETC__ENETC0_SI5.BDR_NUM[10].TBSR.R /* Tx BDR 10 status register */
#define NETC__ENETC0_SI5_TB10BAR0 NETC__ENETC0_SI5.BDR_NUM[10].TBBAR0.R /* Tx BDR 10 base address register 0 */
#define NETC__ENETC0_SI5_TB10BAR1 NETC__ENETC0_SI5.BDR_NUM[10].TBBAR1.R /* Tx BDR 10 base address register 1 */
#define NETC__ENETC0_SI5_TB10PIR NETC__ENETC0_SI5.BDR_NUM[10].TBPIR.R /* Tx BDR 10 producer index register */
#define NETC__ENETC0_SI5_TB10CIR NETC__ENETC0_SI5.BDR_NUM[10].TBCIR.R /* Tx BDR 10 consumer index register */
#define NETC__ENETC0_SI5_TB10LENR NETC__ENETC0_SI5.BDR_NUM[10].TBLENR.R /* Tx BDR 10 length register */
#define NETC__ENETC0_SI5_TB10IER NETC__ENETC0_SI5.BDR_NUM[10].TBIER.R /* Tx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI5_TB10IDR NETC__ENETC0_SI5.BDR_NUM[10].TBIDR.R /* Tx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI5_TB10ICR0 NETC__ENETC0_SI5.BDR_NUM[10].TBICR0.R /* Tx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB10ICR1 NETC__ENETC0_SI5.BDR_NUM[10].TBICR1.R /* Tx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB10MR NETC__ENETC0_SI5.BDR_NUM[10].RBMR.R /* Rx BDR 10 mode register */
#define NETC__ENETC0_SI5_RB10SR NETC__ENETC0_SI5.BDR_NUM[10].RBSR.R /* Rx BDR 10 status register */
#define NETC__ENETC0_SI5_RB10BSR NETC__ENETC0_SI5.BDR_NUM[10].RBBSR.R /* Rx BDR 10 buffer size register */
#define NETC__ENETC0_SI5_RB10CIR NETC__ENETC0_SI5.BDR_NUM[10].RBCIR.R /* Rx BDR 10 consumer index register */
#define NETC__ENETC0_SI5_RB10BAR0 NETC__ENETC0_SI5.BDR_NUM[10].RBBAR0.R /* Rx BDR 10 base address register 0 */
#define NETC__ENETC0_SI5_RB10BAR1 NETC__ENETC0_SI5.BDR_NUM[10].RBBAR1.R /* Rx BDR 10 base address register 1 */
#define NETC__ENETC0_SI5_RB10PIR NETC__ENETC0_SI5.BDR_NUM[10].RBPIR.R /* Rx BDR 10 producer index register */
#define NETC__ENETC0_SI5_RB10LENR NETC__ENETC0_SI5.BDR_NUM[10].RBLENR.R /* Rx BDR 10 length register */
#define NETC__ENETC0_SI5_RB10DCR NETC__ENETC0_SI5.BDR_NUM[10].RBDCR.R /* Rx BDR 10 drop count register */
#define NETC__ENETC0_SI5_RB10IER NETC__ENETC0_SI5.BDR_NUM[10].RBIER.R /* Rx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI5_RB10IDR NETC__ENETC0_SI5.BDR_NUM[10].RBIDR.R /* Rx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI5_RB10ICR0 NETC__ENETC0_SI5.BDR_NUM[10].RBICR0.R /* Rx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB10ICR1 NETC__ENETC0_SI5.BDR_NUM[10].RBICR1.R /* Rx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB11MR NETC__ENETC0_SI5.BDR_NUM[11].TBMR.R /* Tx BDR 11 mode register */
#define NETC__ENETC0_SI5_TB11SR NETC__ENETC0_SI5.BDR_NUM[11].TBSR.R /* Tx BDR 11 status register */
#define NETC__ENETC0_SI5_TB11BAR0 NETC__ENETC0_SI5.BDR_NUM[11].TBBAR0.R /* Tx BDR 11 base address register 0 */
#define NETC__ENETC0_SI5_TB11BAR1 NETC__ENETC0_SI5.BDR_NUM[11].TBBAR1.R /* Tx BDR 11 base address register 1 */
#define NETC__ENETC0_SI5_TB11PIR NETC__ENETC0_SI5.BDR_NUM[11].TBPIR.R /* Tx BDR 11 producer index register */
#define NETC__ENETC0_SI5_TB11CIR NETC__ENETC0_SI5.BDR_NUM[11].TBCIR.R /* Tx BDR 11 consumer index register */
#define NETC__ENETC0_SI5_TB11LENR NETC__ENETC0_SI5.BDR_NUM[11].TBLENR.R /* Tx BDR 11 length register */
#define NETC__ENETC0_SI5_TB11IER NETC__ENETC0_SI5.BDR_NUM[11].TBIER.R /* Tx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI5_TB11IDR NETC__ENETC0_SI5.BDR_NUM[11].TBIDR.R /* Tx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI5_TB11ICR0 NETC__ENETC0_SI5.BDR_NUM[11].TBICR0.R /* Tx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB11ICR1 NETC__ENETC0_SI5.BDR_NUM[11].TBICR1.R /* Tx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB11MR NETC__ENETC0_SI5.BDR_NUM[11].RBMR.R /* Rx BDR 11 mode register */
#define NETC__ENETC0_SI5_RB11SR NETC__ENETC0_SI5.BDR_NUM[11].RBSR.R /* Rx BDR 11 status register */
#define NETC__ENETC0_SI5_RB11BSR NETC__ENETC0_SI5.BDR_NUM[11].RBBSR.R /* Rx BDR 11 buffer size register */
#define NETC__ENETC0_SI5_RB11CIR NETC__ENETC0_SI5.BDR_NUM[11].RBCIR.R /* Rx BDR 11 consumer index register */
#define NETC__ENETC0_SI5_RB11BAR0 NETC__ENETC0_SI5.BDR_NUM[11].RBBAR0.R /* Rx BDR 11 base address register 0 */
#define NETC__ENETC0_SI5_RB11BAR1 NETC__ENETC0_SI5.BDR_NUM[11].RBBAR1.R /* Rx BDR 11 base address register 1 */
#define NETC__ENETC0_SI5_RB11PIR NETC__ENETC0_SI5.BDR_NUM[11].RBPIR.R /* Rx BDR 11 producer index register */
#define NETC__ENETC0_SI5_RB11LENR NETC__ENETC0_SI5.BDR_NUM[11].RBLENR.R /* Rx BDR 11 length register */
#define NETC__ENETC0_SI5_RB11DCR NETC__ENETC0_SI5.BDR_NUM[11].RBDCR.R /* Rx BDR 11 drop count register */
#define NETC__ENETC0_SI5_RB11IER NETC__ENETC0_SI5.BDR_NUM[11].RBIER.R /* Rx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI5_RB11IDR NETC__ENETC0_SI5.BDR_NUM[11].RBIDR.R /* Rx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI5_RB11ICR0 NETC__ENETC0_SI5.BDR_NUM[11].RBICR0.R /* Rx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB11ICR1 NETC__ENETC0_SI5.BDR_NUM[11].RBICR1.R /* Rx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB12MR NETC__ENETC0_SI5.BDR_NUM[12].TBMR.R /* Tx BDR 12 mode register */
#define NETC__ENETC0_SI5_TB12SR NETC__ENETC0_SI5.BDR_NUM[12].TBSR.R /* Tx BDR 12 status register */
#define NETC__ENETC0_SI5_TB12BAR0 NETC__ENETC0_SI5.BDR_NUM[12].TBBAR0.R /* Tx BDR 12 base address register 0 */
#define NETC__ENETC0_SI5_TB12BAR1 NETC__ENETC0_SI5.BDR_NUM[12].TBBAR1.R /* Tx BDR 12 base address register 1 */
#define NETC__ENETC0_SI5_TB12PIR NETC__ENETC0_SI5.BDR_NUM[12].TBPIR.R /* Tx BDR 12 producer index register */
#define NETC__ENETC0_SI5_TB12CIR NETC__ENETC0_SI5.BDR_NUM[12].TBCIR.R /* Tx BDR 12 consumer index register */
#define NETC__ENETC0_SI5_TB12LENR NETC__ENETC0_SI5.BDR_NUM[12].TBLENR.R /* Tx BDR 12 length register */
#define NETC__ENETC0_SI5_TB12IER NETC__ENETC0_SI5.BDR_NUM[12].TBIER.R /* Tx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI5_TB12IDR NETC__ENETC0_SI5.BDR_NUM[12].TBIDR.R /* Tx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI5_TB12ICR0 NETC__ENETC0_SI5.BDR_NUM[12].TBICR0.R /* Tx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB12ICR1 NETC__ENETC0_SI5.BDR_NUM[12].TBICR1.R /* Tx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB12MR NETC__ENETC0_SI5.BDR_NUM[12].RBMR.R /* Rx BDR 12 mode register */
#define NETC__ENETC0_SI5_RB12SR NETC__ENETC0_SI5.BDR_NUM[12].RBSR.R /* Rx BDR 12 status register */
#define NETC__ENETC0_SI5_RB12BSR NETC__ENETC0_SI5.BDR_NUM[12].RBBSR.R /* Rx BDR 12 buffer size register */
#define NETC__ENETC0_SI5_RB12CIR NETC__ENETC0_SI5.BDR_NUM[12].RBCIR.R /* Rx BDR 12 consumer index register */
#define NETC__ENETC0_SI5_RB12BAR0 NETC__ENETC0_SI5.BDR_NUM[12].RBBAR0.R /* Rx BDR 12 base address register 0 */
#define NETC__ENETC0_SI5_RB12BAR1 NETC__ENETC0_SI5.BDR_NUM[12].RBBAR1.R /* Rx BDR 12 base address register 1 */
#define NETC__ENETC0_SI5_RB12PIR NETC__ENETC0_SI5.BDR_NUM[12].RBPIR.R /* Rx BDR 12 producer index register */
#define NETC__ENETC0_SI5_RB12LENR NETC__ENETC0_SI5.BDR_NUM[12].RBLENR.R /* Rx BDR 12 length register */
#define NETC__ENETC0_SI5_RB12DCR NETC__ENETC0_SI5.BDR_NUM[12].RBDCR.R /* Rx BDR 12 drop count register */
#define NETC__ENETC0_SI5_RB12IER NETC__ENETC0_SI5.BDR_NUM[12].RBIER.R /* Rx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI5_RB12IDR NETC__ENETC0_SI5.BDR_NUM[12].RBIDR.R /* Rx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI5_RB12ICR0 NETC__ENETC0_SI5.BDR_NUM[12].RBICR0.R /* Rx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB12ICR1 NETC__ENETC0_SI5.BDR_NUM[12].RBICR1.R /* Rx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB13MR NETC__ENETC0_SI5.BDR_NUM[13].TBMR.R /* Tx BDR 13 mode register */
#define NETC__ENETC0_SI5_TB13SR NETC__ENETC0_SI5.BDR_NUM[13].TBSR.R /* Tx BDR 13 status register */
#define NETC__ENETC0_SI5_TB13BAR0 NETC__ENETC0_SI5.BDR_NUM[13].TBBAR0.R /* Tx BDR 13 base address register 0 */
#define NETC__ENETC0_SI5_TB13BAR1 NETC__ENETC0_SI5.BDR_NUM[13].TBBAR1.R /* Tx BDR 13 base address register 1 */
#define NETC__ENETC0_SI5_TB13PIR NETC__ENETC0_SI5.BDR_NUM[13].TBPIR.R /* Tx BDR 13 producer index register */
#define NETC__ENETC0_SI5_TB13CIR NETC__ENETC0_SI5.BDR_NUM[13].TBCIR.R /* Tx BDR 13 consumer index register */
#define NETC__ENETC0_SI5_TB13LENR NETC__ENETC0_SI5.BDR_NUM[13].TBLENR.R /* Tx BDR 13 length register */
#define NETC__ENETC0_SI5_TB13IER NETC__ENETC0_SI5.BDR_NUM[13].TBIER.R /* Tx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI5_TB13IDR NETC__ENETC0_SI5.BDR_NUM[13].TBIDR.R /* Tx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI5_TB13ICR0 NETC__ENETC0_SI5.BDR_NUM[13].TBICR0.R /* Tx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB13ICR1 NETC__ENETC0_SI5.BDR_NUM[13].TBICR1.R /* Tx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB13MR NETC__ENETC0_SI5.BDR_NUM[13].RBMR.R /* Rx BDR 13 mode register */
#define NETC__ENETC0_SI5_RB13SR NETC__ENETC0_SI5.BDR_NUM[13].RBSR.R /* Rx BDR 13 status register */
#define NETC__ENETC0_SI5_RB13BSR NETC__ENETC0_SI5.BDR_NUM[13].RBBSR.R /* Rx BDR 13 buffer size register */
#define NETC__ENETC0_SI5_RB13CIR NETC__ENETC0_SI5.BDR_NUM[13].RBCIR.R /* Rx BDR 13 consumer index register */
#define NETC__ENETC0_SI5_RB13BAR0 NETC__ENETC0_SI5.BDR_NUM[13].RBBAR0.R /* Rx BDR 13 base address register 0 */
#define NETC__ENETC0_SI5_RB13BAR1 NETC__ENETC0_SI5.BDR_NUM[13].RBBAR1.R /* Rx BDR 13 base address register 1 */
#define NETC__ENETC0_SI5_RB13PIR NETC__ENETC0_SI5.BDR_NUM[13].RBPIR.R /* Rx BDR 13 producer index register */
#define NETC__ENETC0_SI5_RB13LENR NETC__ENETC0_SI5.BDR_NUM[13].RBLENR.R /* Rx BDR 13 length register */
#define NETC__ENETC0_SI5_RB13DCR NETC__ENETC0_SI5.BDR_NUM[13].RBDCR.R /* Rx BDR 13 drop count register */
#define NETC__ENETC0_SI5_RB13IER NETC__ENETC0_SI5.BDR_NUM[13].RBIER.R /* Rx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI5_RB13IDR NETC__ENETC0_SI5.BDR_NUM[13].RBIDR.R /* Rx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI5_RB13ICR0 NETC__ENETC0_SI5.BDR_NUM[13].RBICR0.R /* Rx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB13ICR1 NETC__ENETC0_SI5.BDR_NUM[13].RBICR1.R /* Rx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB14MR NETC__ENETC0_SI5.BDR_NUM[14].TBMR.R /* Tx BDR 14 mode register */
#define NETC__ENETC0_SI5_TB14SR NETC__ENETC0_SI5.BDR_NUM[14].TBSR.R /* Tx BDR 14 status register */
#define NETC__ENETC0_SI5_TB14BAR0 NETC__ENETC0_SI5.BDR_NUM[14].TBBAR0.R /* Tx BDR 14 base address register 0 */
#define NETC__ENETC0_SI5_TB14BAR1 NETC__ENETC0_SI5.BDR_NUM[14].TBBAR1.R /* Tx BDR 14 base address register 1 */
#define NETC__ENETC0_SI5_TB14PIR NETC__ENETC0_SI5.BDR_NUM[14].TBPIR.R /* Tx BDR 14 producer index register */
#define NETC__ENETC0_SI5_TB14CIR NETC__ENETC0_SI5.BDR_NUM[14].TBCIR.R /* Tx BDR 14 consumer index register */
#define NETC__ENETC0_SI5_TB14LENR NETC__ENETC0_SI5.BDR_NUM[14].TBLENR.R /* Tx BDR 14 length register */
#define NETC__ENETC0_SI5_TB14IER NETC__ENETC0_SI5.BDR_NUM[14].TBIER.R /* Tx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI5_TB14IDR NETC__ENETC0_SI5.BDR_NUM[14].TBIDR.R /* Tx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI5_TB14ICR0 NETC__ENETC0_SI5.BDR_NUM[14].TBICR0.R /* Tx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB14ICR1 NETC__ENETC0_SI5.BDR_NUM[14].TBICR1.R /* Tx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB14MR NETC__ENETC0_SI5.BDR_NUM[14].RBMR.R /* Rx BDR 14 mode register */
#define NETC__ENETC0_SI5_RB14SR NETC__ENETC0_SI5.BDR_NUM[14].RBSR.R /* Rx BDR 14 status register */
#define NETC__ENETC0_SI5_RB14BSR NETC__ENETC0_SI5.BDR_NUM[14].RBBSR.R /* Rx BDR 14 buffer size register */
#define NETC__ENETC0_SI5_RB14CIR NETC__ENETC0_SI5.BDR_NUM[14].RBCIR.R /* Rx BDR 14 consumer index register */
#define NETC__ENETC0_SI5_RB14BAR0 NETC__ENETC0_SI5.BDR_NUM[14].RBBAR0.R /* Rx BDR 14 base address register 0 */
#define NETC__ENETC0_SI5_RB14BAR1 NETC__ENETC0_SI5.BDR_NUM[14].RBBAR1.R /* Rx BDR 14 base address register 1 */
#define NETC__ENETC0_SI5_RB14PIR NETC__ENETC0_SI5.BDR_NUM[14].RBPIR.R /* Rx BDR 14 producer index register */
#define NETC__ENETC0_SI5_RB14LENR NETC__ENETC0_SI5.BDR_NUM[14].RBLENR.R /* Rx BDR 14 length register */
#define NETC__ENETC0_SI5_RB14DCR NETC__ENETC0_SI5.BDR_NUM[14].RBDCR.R /* Rx BDR 14 drop count register */
#define NETC__ENETC0_SI5_RB14IER NETC__ENETC0_SI5.BDR_NUM[14].RBIER.R /* Rx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI5_RB14IDR NETC__ENETC0_SI5.BDR_NUM[14].RBIDR.R /* Rx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI5_RB14ICR0 NETC__ENETC0_SI5.BDR_NUM[14].RBICR0.R /* Rx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB14ICR1 NETC__ENETC0_SI5.BDR_NUM[14].RBICR1.R /* Rx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB15MR NETC__ENETC0_SI5.BDR_NUM[15].TBMR.R /* Tx BDR 15 mode register */
#define NETC__ENETC0_SI5_TB15SR NETC__ENETC0_SI5.BDR_NUM[15].TBSR.R /* Tx BDR 15 status register */
#define NETC__ENETC0_SI5_TB15BAR0 NETC__ENETC0_SI5.BDR_NUM[15].TBBAR0.R /* Tx BDR 15 base address register 0 */
#define NETC__ENETC0_SI5_TB15BAR1 NETC__ENETC0_SI5.BDR_NUM[15].TBBAR1.R /* Tx BDR 15 base address register 1 */
#define NETC__ENETC0_SI5_TB15PIR NETC__ENETC0_SI5.BDR_NUM[15].TBPIR.R /* Tx BDR 15 producer index register */
#define NETC__ENETC0_SI5_TB15CIR NETC__ENETC0_SI5.BDR_NUM[15].TBCIR.R /* Tx BDR 15 consumer index register */
#define NETC__ENETC0_SI5_TB15LENR NETC__ENETC0_SI5.BDR_NUM[15].TBLENR.R /* Tx BDR 15 length register */
#define NETC__ENETC0_SI5_TB15IER NETC__ENETC0_SI5.BDR_NUM[15].TBIER.R /* Tx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI5_TB15IDR NETC__ENETC0_SI5.BDR_NUM[15].TBIDR.R /* Tx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI5_TB15ICR0 NETC__ENETC0_SI5.BDR_NUM[15].TBICR0.R /* Tx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB15ICR1 NETC__ENETC0_SI5.BDR_NUM[15].TBICR1.R /* Tx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB15MR NETC__ENETC0_SI5.BDR_NUM[15].RBMR.R /* Rx BDR 15 mode register */
#define NETC__ENETC0_SI5_RB15SR NETC__ENETC0_SI5.BDR_NUM[15].RBSR.R /* Rx BDR 15 status register */
#define NETC__ENETC0_SI5_RB15BSR NETC__ENETC0_SI5.BDR_NUM[15].RBBSR.R /* Rx BDR 15 buffer size register */
#define NETC__ENETC0_SI5_RB15CIR NETC__ENETC0_SI5.BDR_NUM[15].RBCIR.R /* Rx BDR 15 consumer index register */
#define NETC__ENETC0_SI5_RB15BAR0 NETC__ENETC0_SI5.BDR_NUM[15].RBBAR0.R /* Rx BDR 15 base address register 0 */
#define NETC__ENETC0_SI5_RB15BAR1 NETC__ENETC0_SI5.BDR_NUM[15].RBBAR1.R /* Rx BDR 15 base address register 1 */
#define NETC__ENETC0_SI5_RB15PIR NETC__ENETC0_SI5.BDR_NUM[15].RBPIR.R /* Rx BDR 15 producer index register */
#define NETC__ENETC0_SI5_RB15LENR NETC__ENETC0_SI5.BDR_NUM[15].RBLENR.R /* Rx BDR 15 length register */
#define NETC__ENETC0_SI5_RB15DCR NETC__ENETC0_SI5.BDR_NUM[15].RBDCR.R /* Rx BDR 15 drop count register */
#define NETC__ENETC0_SI5_RB15IER NETC__ENETC0_SI5.BDR_NUM[15].RBIER.R /* Rx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI5_RB15IDR NETC__ENETC0_SI5.BDR_NUM[15].RBIDR.R /* Rx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI5_RB15ICR0 NETC__ENETC0_SI5.BDR_NUM[15].RBICR0.R /* Rx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB15ICR1 NETC__ENETC0_SI5.BDR_NUM[15].RBICR1.R /* Rx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB16MR NETC__ENETC0_SI5.BDR_NUM[16].TBMR.R /* Tx BDR 16 mode register */
#define NETC__ENETC0_SI5_TB16SR NETC__ENETC0_SI5.BDR_NUM[16].TBSR.R /* Tx BDR 16 status register */
#define NETC__ENETC0_SI5_TB16BAR0 NETC__ENETC0_SI5.BDR_NUM[16].TBBAR0.R /* Tx BDR 16 base address register 0 */
#define NETC__ENETC0_SI5_TB16BAR1 NETC__ENETC0_SI5.BDR_NUM[16].TBBAR1.R /* Tx BDR 16 base address register 1 */
#define NETC__ENETC0_SI5_TB16PIR NETC__ENETC0_SI5.BDR_NUM[16].TBPIR.R /* Tx BDR 16 producer index register */
#define NETC__ENETC0_SI5_TB16CIR NETC__ENETC0_SI5.BDR_NUM[16].TBCIR.R /* Tx BDR 16 consumer index register */
#define NETC__ENETC0_SI5_TB16LENR NETC__ENETC0_SI5.BDR_NUM[16].TBLENR.R /* Tx BDR 16 length register */
#define NETC__ENETC0_SI5_TB16IER NETC__ENETC0_SI5.BDR_NUM[16].TBIER.R /* Tx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI5_TB16IDR NETC__ENETC0_SI5.BDR_NUM[16].TBIDR.R /* Tx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI5_TB16ICR0 NETC__ENETC0_SI5.BDR_NUM[16].TBICR0.R /* Tx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB16ICR1 NETC__ENETC0_SI5.BDR_NUM[16].TBICR1.R /* Tx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB16MR NETC__ENETC0_SI5.BDR_NUM[16].RBMR.R /* Rx BDR 16 mode register */
#define NETC__ENETC0_SI5_RB16SR NETC__ENETC0_SI5.BDR_NUM[16].RBSR.R /* Rx BDR 16 status register */
#define NETC__ENETC0_SI5_RB16BSR NETC__ENETC0_SI5.BDR_NUM[16].RBBSR.R /* Rx BDR 16 buffer size register */
#define NETC__ENETC0_SI5_RB16CIR NETC__ENETC0_SI5.BDR_NUM[16].RBCIR.R /* Rx BDR 16 consumer index register */
#define NETC__ENETC0_SI5_RB16BAR0 NETC__ENETC0_SI5.BDR_NUM[16].RBBAR0.R /* Rx BDR 16 base address register 0 */
#define NETC__ENETC0_SI5_RB16BAR1 NETC__ENETC0_SI5.BDR_NUM[16].RBBAR1.R /* Rx BDR 16 base address register 1 */
#define NETC__ENETC0_SI5_RB16PIR NETC__ENETC0_SI5.BDR_NUM[16].RBPIR.R /* Rx BDR 16 producer index register */
#define NETC__ENETC0_SI5_RB16LENR NETC__ENETC0_SI5.BDR_NUM[16].RBLENR.R /* Rx BDR 16 length register */
#define NETC__ENETC0_SI5_RB16DCR NETC__ENETC0_SI5.BDR_NUM[16].RBDCR.R /* Rx BDR 16 drop count register */
#define NETC__ENETC0_SI5_RB16IER NETC__ENETC0_SI5.BDR_NUM[16].RBIER.R /* Rx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI5_RB16IDR NETC__ENETC0_SI5.BDR_NUM[16].RBIDR.R /* Rx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI5_RB16ICR0 NETC__ENETC0_SI5.BDR_NUM[16].RBICR0.R /* Rx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB16ICR1 NETC__ENETC0_SI5.BDR_NUM[16].RBICR1.R /* Rx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_TB17MR NETC__ENETC0_SI5.BDR_NUM[17].TBMR.R /* Tx BDR 17 mode register */
#define NETC__ENETC0_SI5_TB17SR NETC__ENETC0_SI5.BDR_NUM[17].TBSR.R /* Tx BDR 17 status register */
#define NETC__ENETC0_SI5_TB17BAR0 NETC__ENETC0_SI5.BDR_NUM[17].TBBAR0.R /* Tx BDR 17 base address register 0 */
#define NETC__ENETC0_SI5_TB17BAR1 NETC__ENETC0_SI5.BDR_NUM[17].TBBAR1.R /* Tx BDR 17 base address register 1 */
#define NETC__ENETC0_SI5_TB17PIR NETC__ENETC0_SI5.BDR_NUM[17].TBPIR.R /* Tx BDR 17 producer index register */
#define NETC__ENETC0_SI5_TB17CIR NETC__ENETC0_SI5.BDR_NUM[17].TBCIR.R /* Tx BDR 17 consumer index register */
#define NETC__ENETC0_SI5_TB17LENR NETC__ENETC0_SI5.BDR_NUM[17].TBLENR.R /* Tx BDR 17 length register */
#define NETC__ENETC0_SI5_TB17IER NETC__ENETC0_SI5.BDR_NUM[17].TBIER.R /* Tx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI5_TB17IDR NETC__ENETC0_SI5.BDR_NUM[17].TBIDR.R /* Tx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI5_TB17ICR0 NETC__ENETC0_SI5.BDR_NUM[17].TBICR0.R /* Tx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_TB17ICR1 NETC__ENETC0_SI5.BDR_NUM[17].TBICR1.R /* Tx BDR 17 interrupt coalescing register 1 */
#define NETC__ENETC0_SI5_RB17MR NETC__ENETC0_SI5.BDR_NUM[17].RBMR.R /* Rx BDR 17 mode register */
#define NETC__ENETC0_SI5_RB17SR NETC__ENETC0_SI5.BDR_NUM[17].RBSR.R /* Rx BDR 17 status register */
#define NETC__ENETC0_SI5_RB17BSR NETC__ENETC0_SI5.BDR_NUM[17].RBBSR.R /* Rx BDR 17 buffer size register */
#define NETC__ENETC0_SI5_RB17CIR NETC__ENETC0_SI5.BDR_NUM[17].RBCIR.R /* Rx BDR 17 consumer index register */
#define NETC__ENETC0_SI5_RB17BAR0 NETC__ENETC0_SI5.BDR_NUM[17].RBBAR0.R /* Rx BDR 17 base address register 0 */
#define NETC__ENETC0_SI5_RB17BAR1 NETC__ENETC0_SI5.BDR_NUM[17].RBBAR1.R /* Rx BDR 17 base address register 1 */
#define NETC__ENETC0_SI5_RB17PIR NETC__ENETC0_SI5.BDR_NUM[17].RBPIR.R /* Rx BDR 17 producer index register */
#define NETC__ENETC0_SI5_RB17LENR NETC__ENETC0_SI5.BDR_NUM[17].RBLENR.R /* Rx BDR 17 length register */
#define NETC__ENETC0_SI5_RB17DCR NETC__ENETC0_SI5.BDR_NUM[17].RBDCR.R /* Rx BDR 17 drop count register */
#define NETC__ENETC0_SI5_RB17IER NETC__ENETC0_SI5.BDR_NUM[17].RBIER.R /* Rx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI5_RB17IDR NETC__ENETC0_SI5.BDR_NUM[17].RBIDR.R /* Rx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI5_RB17ICR0 NETC__ENETC0_SI5.BDR_NUM[17].RBICR0.R /* Rx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI5_RB17ICR1 NETC__ENETC0_SI5.BDR_NUM[17].RBICR1.R /* Rx BDR 17 interrupt coalescing register 1 */

/* NETC_F3_SI6 */
#define NETC__ENETC0_SI6_SIMR NETC__ENETC0_SI6.SIMR.R      /* Station interface mode register */
#define NETC__ENETC0_SI6_SISR NETC__ENETC0_SI6.SISR.R      /* Station interface status register */
#define NETC__ENETC0_SI6_SICTR0 NETC__ENETC0_SI6.SICTR0.R  /* Station interface current time register 0 */
#define NETC__ENETC0_SI6_SICTR1 NETC__ENETC0_SI6.SICTR1.R  /* Station interface current time register 1 */
#define NETC__ENETC0_SI6_SIPCAPR0 NETC__ENETC0_SI6.SIPCAPR0.R /* Station interface port capability register 0 */
#define NETC__ENETC0_SI6_SIPCAPR1 NETC__ENETC0_SI6.SIPCAPR1.R /* Station interface port capability register 1 */
#define NETC__ENETC0_SI6_SIRBGCR NETC__ENETC0_SI6.SIRBGCR.R /* Station interface receive BDR group control register */
#define NETC__ENETC0_SI6_SIBCAR NETC__ENETC0_SI6.SIBCAR.R  /* Station interface buffer cache attribute register */
#define NETC__ENETC0_SI6_SICCAR NETC__ENETC0_SI6.SICCAR.R  /* Station interface command cache attribute register */
#define NETC__ENETC0_SI6_SIPMAR0 NETC__ENETC0_SI6.SIPMAR0.R /* Station interface primary MAC address register 0 */
#define NETC__ENETC0_SI6_SIPMAR1 NETC__ENETC0_SI6.SIPMAR1.R /* Station interface primary MAC address register 1 */
#define NETC__ENETC0_SI6_SICVLANR1 NETC__ENETC0_SI6.SICVLANR1.R /* Station interface custom VLAN register 1 */
#define NETC__ENETC0_SI6_SICVLANR2 NETC__ENETC0_SI6.SICVLANR2.R /* Station interface custom VLAN register 2 */
#define NETC__ENETC0_SI6_SIVLANIPVMR0 NETC__ENETC0_SI6.SIVLANIPVMR0.R /* Station interface VLAN to IPV mapping register 0 */
#define NETC__ENETC0_SI6_SIVLANIPVMR1 NETC__ENETC0_SI6.SIVLANIPVMR1.R /* Station interface VLAN to IPV mapping register 1 */
#define NETC__ENETC0_SI6_SIIPVBDRMR0 NETC__ENETC0_SI6.SIIPVBDRMR0.R /* Station interface IPV to ring mapping register */
#define NETC__ENETC0_SI6_VSIMSGSR NETC__ENETC0_SI6.VSI_A.VSIMSGSR.R /* Virtual station interface message send register */
#define NETC__ENETC0_SI6_VSIMSGRR NETC__ENETC0_SI6.VSI_A.VSIMSGRR.R /* Virtual station interface message receive register */
#define NETC__ENETC0_SI6_VSIMSGSNDAR0 NETC__ENETC0_SI6.VSI_A.VSIMSGSNDAR0.R /* Virtual station interface message send register 0 */
#define NETC__ENETC0_SI6_VSIMSGSNDAR1 NETC__ENETC0_SI6.VSI_A.VSIMSGSNDAR1.R /* Virtual station interface message send address register 1 */
#define NETC__ENETC0_SI6_SIROCT0 NETC__ENETC0_SI6.SIROCT0.R /* Station interface receive octets counter (ifInOctets) 0 */
#define NETC__ENETC0_SI6_SIROCT1 NETC__ENETC0_SI6.SIROCT1.R /* Station interface receive octets counter (ifInOctets) 1 */
#define NETC__ENETC0_SI6_SIRFRM0 NETC__ENETC0_SI6.SIRFRM0.R /* Station interface receive frame counter (aFrameReceivedOK) 0 */
#define NETC__ENETC0_SI6_SIRFRM1 NETC__ENETC0_SI6.SIRFRM1.R /* Station interface receive frame counter (aFrameReceivedOK) 1 */
#define NETC__ENETC0_SI6_SIRUCA0 NETC__ENETC0_SI6.SIRUCA0.R /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
#define NETC__ENETC0_SI6_SIRUCA1 NETC__ENETC0_SI6.SIRUCA1.R /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
#define NETC__ENETC0_SI6_SIRMCA0 NETC__ENETC0_SI6.SIRMCA0.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
#define NETC__ENETC0_SI6_SIRMCA1 NETC__ENETC0_SI6.SIRMCA1.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
#define NETC__ENETC0_SI6_SITOCT0 NETC__ENETC0_SI6.SITOCT0.R /* Station interface transmit octets counter (ifOutOctets) 0 */
#define NETC__ENETC0_SI6_SITOCT1 NETC__ENETC0_SI6.SITOCT1.R /* Station interface transmit octets counter (ifOutOctets) 1 */
#define NETC__ENETC0_SI6_SITFRM0 NETC__ENETC0_SI6.SITFRM0.R /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
#define NETC__ENETC0_SI6_SITFRM1 NETC__ENETC0_SI6.SITFRM1.R /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
#define NETC__ENETC0_SI6_SITUCA0 NETC__ENETC0_SI6.SITUCA0.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
#define NETC__ENETC0_SI6_SITUCA1 NETC__ENETC0_SI6.SITUCA1.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
#define NETC__ENETC0_SI6_SITMCA0 NETC__ENETC0_SI6.SITMCA0.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
#define NETC__ENETC0_SI6_SITMCA1 NETC__ENETC0_SI6.SITMCA1.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
#define NETC__ENETC0_SI6_SIBLPR0 NETC__ENETC0_SI6.SIBLPR[0].R /* Station interface boot loader parameter register 0 */
#define NETC__ENETC0_SI6_SIBLPR1 NETC__ENETC0_SI6.SIBLPR[1].R /* Station interface boot loader parameter register 1 */
#define NETC__ENETC0_SI6_SICBDRMR NETC__ENETC0_SI6.SICBDRMR.R /* Station interface command BDR mode register */
#define NETC__ENETC0_SI6_SICBDRSR NETC__ENETC0_SI6.SICBDRSR.R /* Station interface command BDR status register */
#define NETC__ENETC0_SI6_SICBDRBAR0 NETC__ENETC0_SI6.SICBDRBAR0.R /* Station interface command BDR base address register 0 */
#define NETC__ENETC0_SI6_SICBDRBAR1 NETC__ENETC0_SI6.SICBDRBAR1.R /* Station interface command BDR base address register 1 */
#define NETC__ENETC0_SI6_SICBDRPIR NETC__ENETC0_SI6.SICBDRPIR.R /* Station interface command BDR producer index register */
#define NETC__ENETC0_SI6_SICBDRCIR NETC__ENETC0_SI6.SICBDRCIR.R /* Station interface command BDR consumer index register */
#define NETC__ENETC0_SI6_SICBDRLENR NETC__ENETC0_SI6.SICBDRLENR.R /* Station interface command BDR length register */
#define NETC__ENETC0_SI6_SICBDRIER NETC__ENETC0_SI6.SICBDRIER.R /* Station interface command BDR interrupt enable register */
#define NETC__ENETC0_SI6_SICBDRIDR NETC__ENETC0_SI6.SICBDRIDR.R /* Station interface command BDR interrupt detect register */
#define NETC__ENETC0_SI6_SICAPR0 NETC__ENETC0_SI6.SICAPR0.R /* Station interface capability register 0 */
#define NETC__ENETC0_SI6_SICAPR1 NETC__ENETC0_SI6.SICAPR1.R /* Station interface capability register 1 */
#define NETC__ENETC0_SI6_SICAPR2 NETC__ENETC0_SI6.SICAPR2.R /* Station interface capability register 2 */
#define NETC__ENETC0_SI6_VSIIER NETC__ENETC0_SI6.VSI.VSIIER.R /* Virtual station interface interrupt enable register */
#define NETC__ENETC0_SI6_VSIIDR NETC__ENETC0_SI6.VSI.VSIIDR.R /* Virtual station interface interrupt detect register */
#define NETC__ENETC0_SI6_SITXIDR0 NETC__ENETC0_SI6.SITXIDR0.R /* Station interface transmit interrupt detect register 0 */
#define NETC__ENETC0_SI6_SITXIDR1 NETC__ENETC0_SI6.SITXIDR1.R /* Station interface transmit interrupt detect register 1 */
#define NETC__ENETC0_SI6_SIRXIDR0 NETC__ENETC0_SI6.SIRXIDR0.R /* Station interface receive interrupt detect register 0 */
#define NETC__ENETC0_SI6_SIRXIDR1 NETC__ENETC0_SI6.SIRXIDR1.R /* Station interface receive interrupt detect register 1 */
#define NETC__ENETC0_SI6_SIMSIVR NETC__ENETC0_SI6.SIMSIVR.R /* Station interface MSI-X vector register */
#define NETC__ENETC0_SI6_SICMSIVR NETC__ENETC0_SI6.SICMSIVR.R /* Station interface command MSI-X vector register */
#define NETC__ENETC0_SI6_SITMRIER NETC__ENETC0_SI6.SITMRIER.R /* Station interface timer interrupt enable register */
#define NETC__ENETC0_SI6_SITMRIDR NETC__ENETC0_SI6.SITMRIDR.R /* Station interface timer interrupt detect register */
#define NETC__ENETC0_SI6_SITMRMSIVR NETC__ENETC0_SI6.SITMRMSIVR.R /* Station interface timer MSI-X vector register */
#define NETC__ENETC0_SI6_SIMSITR0VR NETC__ENETC0_SI6.SIMSITRVR[0].R /* Station interface MSI-X transmit ring 0 vector register */
#define NETC__ENETC0_SI6_SIMSITR1VR NETC__ENETC0_SI6.SIMSITRVR[1].R /* Station interface MSI-X transmit ring 1 vector register */
#define NETC__ENETC0_SI6_SIMSITR2VR NETC__ENETC0_SI6.SIMSITRVR[2].R /* Station interface MSI-X transmit ring 2 vector register */
#define NETC__ENETC0_SI6_SIMSITR3VR NETC__ENETC0_SI6.SIMSITRVR[3].R /* Station interface MSI-X transmit ring 3 vector register */
#define NETC__ENETC0_SI6_SIMSITR4VR NETC__ENETC0_SI6.SIMSITRVR[4].R /* Station interface MSI-X transmit ring 4 vector register */
#define NETC__ENETC0_SI6_SIMSITR5VR NETC__ENETC0_SI6.SIMSITRVR[5].R /* Station interface MSI-X transmit ring 5 vector register */
#define NETC__ENETC0_SI6_SIMSITR6VR NETC__ENETC0_SI6.SIMSITRVR[6].R /* Station interface MSI-X transmit ring 6 vector register */
#define NETC__ENETC0_SI6_SIMSITR7VR NETC__ENETC0_SI6.SIMSITRVR[7].R /* Station interface MSI-X transmit ring 7 vector register */
#define NETC__ENETC0_SI6_SIMSITR8VR NETC__ENETC0_SI6.SIMSITRVR[8].R /* Station interface MSI-X transmit ring 8 vector register */
#define NETC__ENETC0_SI6_SIMSITR9VR NETC__ENETC0_SI6.SIMSITRVR[9].R /* Station interface MSI-X transmit ring 9 vector register */
#define NETC__ENETC0_SI6_SIMSITR10VR NETC__ENETC0_SI6.SIMSITRVR[10].R /* Station interface MSI-X transmit ring 10 vector register */
#define NETC__ENETC0_SI6_SIMSITR11VR NETC__ENETC0_SI6.SIMSITRVR[11].R /* Station interface MSI-X transmit ring 11 vector register */
#define NETC__ENETC0_SI6_SIMSITR12VR NETC__ENETC0_SI6.SIMSITRVR[12].R /* Station interface MSI-X transmit ring 12 vector register */
#define NETC__ENETC0_SI6_SIMSITR13VR NETC__ENETC0_SI6.SIMSITRVR[13].R /* Station interface MSI-X transmit ring 13 vector register */
#define NETC__ENETC0_SI6_SIMSITR14VR NETC__ENETC0_SI6.SIMSITRVR[14].R /* Station interface MSI-X transmit ring 14 vector register */
#define NETC__ENETC0_SI6_SIMSITR15VR NETC__ENETC0_SI6.SIMSITRVR[15].R /* Station interface MSI-X transmit ring 15 vector register */
#define NETC__ENETC0_SI6_SIMSITR16VR NETC__ENETC0_SI6.SIMSITRVR[16].R /* Station interface MSI-X transmit ring 16 vector register */
#define NETC__ENETC0_SI6_SIMSITR17VR NETC__ENETC0_SI6.SIMSITRVR[17].R /* Station interface MSI-X transmit ring 17 vector register */
#define NETC__ENETC0_SI6_SIMSIRR0VR NETC__ENETC0_SI6.SIMSIRRVR[0].R /* Station interface MSI-X receive ring 0 vector register */
#define NETC__ENETC0_SI6_SIMSIRR1VR NETC__ENETC0_SI6.SIMSIRRVR[1].R /* Station interface MSI-X receive ring 1 vector register */
#define NETC__ENETC0_SI6_SIMSIRR2VR NETC__ENETC0_SI6.SIMSIRRVR[2].R /* Station interface MSI-X receive ring 2 vector register */
#define NETC__ENETC0_SI6_SIMSIRR3VR NETC__ENETC0_SI6.SIMSIRRVR[3].R /* Station interface MSI-X receive ring 3 vector register */
#define NETC__ENETC0_SI6_SIMSIRR4VR NETC__ENETC0_SI6.SIMSIRRVR[4].R /* Station interface MSI-X receive ring 4 vector register */
#define NETC__ENETC0_SI6_SIMSIRR5VR NETC__ENETC0_SI6.SIMSIRRVR[5].R /* Station interface MSI-X receive ring 5 vector register */
#define NETC__ENETC0_SI6_SIMSIRR6VR NETC__ENETC0_SI6.SIMSIRRVR[6].R /* Station interface MSI-X receive ring 6 vector register */
#define NETC__ENETC0_SI6_SIMSIRR7VR NETC__ENETC0_SI6.SIMSIRRVR[7].R /* Station interface MSI-X receive ring 7 vector register */
#define NETC__ENETC0_SI6_SIMSIRR8VR NETC__ENETC0_SI6.SIMSIRRVR[8].R /* Station interface MSI-X receive ring 8 vector register */
#define NETC__ENETC0_SI6_SIMSIRR9VR NETC__ENETC0_SI6.SIMSIRRVR[9].R /* Station interface MSI-X receive ring 9 vector register */
#define NETC__ENETC0_SI6_SIMSIRR10VR NETC__ENETC0_SI6.SIMSIRRVR[10].R /* Station interface MSI-X receive ring 10 vector register */
#define NETC__ENETC0_SI6_SIMSIRR11VR NETC__ENETC0_SI6.SIMSIRRVR[11].R /* Station interface MSI-X receive ring 11 vector register */
#define NETC__ENETC0_SI6_SIMSIRR12VR NETC__ENETC0_SI6.SIMSIRRVR[12].R /* Station interface MSI-X receive ring 12 vector register */
#define NETC__ENETC0_SI6_SIMSIRR13VR NETC__ENETC0_SI6.SIMSIRRVR[13].R /* Station interface MSI-X receive ring 13 vector register */
#define NETC__ENETC0_SI6_SIMSIRR14VR NETC__ENETC0_SI6.SIMSIRRVR[14].R /* Station interface MSI-X receive ring 14 vector register */
#define NETC__ENETC0_SI6_SIMSIRR15VR NETC__ENETC0_SI6.SIMSIRRVR[15].R /* Station interface MSI-X receive ring 15 vector register */
#define NETC__ENETC0_SI6_SIMSIRR16VR NETC__ENETC0_SI6.SIMSIRRVR[16].R /* Station interface MSI-X receive ring 16 vector register */
#define NETC__ENETC0_SI6_SIMSIRR17VR NETC__ENETC0_SI6.SIMSIRRVR[17].R /* Station interface MSI-X receive ring 17 vector register */
#define NETC__ENETC0_SI6_SICMECR NETC__ENETC0_SI6.SICMECR.R /* Station interface correctable memory error configuration register */
#define NETC__ENETC0_SI6_SICMESR NETC__ENETC0_SI6.SICMESR.R /* Station interface correctable memory error status register */
#define NETC__ENETC0_SI6_SICMECTR NETC__ENETC0_SI6.SICMECTR.R /* Station interface correctable memory error count register */
#define NETC__ENETC0_SI6_SIUPECR NETC__ENETC0_SI6.SIUPECR.R /* Station interface uncorrectable programming error configuration register */
#define NETC__ENETC0_SI6_SIUPESR NETC__ENETC0_SI6.SIUPESR.R /* Station interface uncorrectable programming error status register */
#define NETC__ENETC0_SI6_SIUPECTR NETC__ENETC0_SI6.SIUPECTR.R /* Station interface uncorrectable programming error count register */
#define NETC__ENETC0_SI6_SIUNSBECR NETC__ENETC0_SI6.SIUNSBECR.R /* Station interface uncorrectable non-fatal system bus error configuration register */
#define NETC__ENETC0_SI6_SIUNSBESR NETC__ENETC0_SI6.SIUNSBESR.R /* Station interface uncorrectable non-fatal system bus error status register */
#define NETC__ENETC0_SI6_SIUNSBECTR NETC__ENETC0_SI6.SIUNSBECTR.R /* Station interface uncorrectable non-fatal system bus error count register */
#define NETC__ENETC0_SI6_SIUFSBECR NETC__ENETC0_SI6.SIUFSBECR.R /* Station interface uncorrectable fatal system bus error configuration register */
#define NETC__ENETC0_SI6_SIUFSBESR NETC__ENETC0_SI6.SIUFSBESR.R /* Station interface uncorrectable fatal system bus error status register */
#define NETC__ENETC0_SI6_SIUNMECR NETC__ENETC0_SI6.SIUNMECR.R /* Station interface uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_SI6_SIUNMESR0 NETC__ENETC0_SI6.SIUNMESR0.R /* Station interface uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_SI6_SIUNMESR1 NETC__ENETC0_SI6.SIUNMESR1.R /* Station interface uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_SI6_SIUNMECTR NETC__ENETC0_SI6.SIUNMECTR.R /* Station interface uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_SI6_SIUFMECR NETC__ENETC0_SI6.SIUFMECR.R /* Station interface uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_SI6_SIUFMESR0 NETC__ENETC0_SI6.SIUFMESR0.R /* Station interface uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_SI6_SIUFMESR1 NETC__ENETC0_SI6.SIUFMESR1.R /* Station interface uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_SI6_SIUNIECR NETC__ENETC0_SI6.SIUNIECR.R /* Station interface uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_SI6_SIUNIESR NETC__ENETC0_SI6.SIUNIESR.R /* Station interface uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_SI6_SIUNIECTR NETC__ENETC0_SI6.SIUNIECTR.R /* Station interface uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_SI6_SIUFIECR NETC__ENETC0_SI6.SIUFIECR.R /* Station interface uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_SI6_SIUFIESR NETC__ENETC0_SI6.SIUFIESR.R /* Station interface uncorrectable fatal integrity error status register */
#define NETC__ENETC0_SI6_SIMAFTCAPR NETC__ENETC0_SI6.SIMAFTCAPR.R /* Station interface MAC address filter table capability register */
#define NETC__ENETC0_SI6_SIVFTCAPR NETC__ENETC0_SI6.SIVFTCAPR.R /* Station interface VLAN filter table capability register */
#define NETC__ENETC0_SI6_SIRFSCAPR NETC__ENETC0_SI6.SIRFSCAPR.R /* Station interface RFS capability register */
#define NETC__ENETC0_SI6_TB0MR NETC__ENETC0_SI6.BDR_NUM[0].TBMR.R /* Tx BDR 0 mode register */
#define NETC__ENETC0_SI6_TB0SR NETC__ENETC0_SI6.BDR_NUM[0].TBSR.R /* Tx BDR 0 status register */
#define NETC__ENETC0_SI6_TB0BAR0 NETC__ENETC0_SI6.BDR_NUM[0].TBBAR0.R /* Tx BDR 0 base address register 0 */
#define NETC__ENETC0_SI6_TB0BAR1 NETC__ENETC0_SI6.BDR_NUM[0].TBBAR1.R /* Tx BDR 0 base address register 1 */
#define NETC__ENETC0_SI6_TB0PIR NETC__ENETC0_SI6.BDR_NUM[0].TBPIR.R /* Tx BDR 0 producer index register */
#define NETC__ENETC0_SI6_TB0CIR NETC__ENETC0_SI6.BDR_NUM[0].TBCIR.R /* Tx BDR 0 consumer index register */
#define NETC__ENETC0_SI6_TB0LENR NETC__ENETC0_SI6.BDR_NUM[0].TBLENR.R /* Tx BDR 0 length register */
#define NETC__ENETC0_SI6_TB0IER NETC__ENETC0_SI6.BDR_NUM[0].TBIER.R /* Tx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI6_TB0IDR NETC__ENETC0_SI6.BDR_NUM[0].TBIDR.R /* Tx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI6_TB0ICR0 NETC__ENETC0_SI6.BDR_NUM[0].TBICR0.R /* Tx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB0ICR1 NETC__ENETC0_SI6.BDR_NUM[0].TBICR1.R /* Tx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB0MR NETC__ENETC0_SI6.BDR_NUM[0].RBMR.R /* Rx BDR 0 mode register */
#define NETC__ENETC0_SI6_RB0SR NETC__ENETC0_SI6.BDR_NUM[0].RBSR.R /* Rx BDR 0 status register */
#define NETC__ENETC0_SI6_RB0BSR NETC__ENETC0_SI6.BDR_NUM[0].RBBSR.R /* Rx BDR 0 buffer size register */
#define NETC__ENETC0_SI6_RB0CIR NETC__ENETC0_SI6.BDR_NUM[0].RBCIR.R /* Rx BDR 0 consumer index register */
#define NETC__ENETC0_SI6_RB0BAR0 NETC__ENETC0_SI6.BDR_NUM[0].RBBAR0.R /* Rx BDR 0 base address register 0 */
#define NETC__ENETC0_SI6_RB0BAR1 NETC__ENETC0_SI6.BDR_NUM[0].RBBAR1.R /* Rx BDR 0 base address register 1 */
#define NETC__ENETC0_SI6_RB0PIR NETC__ENETC0_SI6.BDR_NUM[0].RBPIR.R /* Rx BDR 0 producer index register */
#define NETC__ENETC0_SI6_RB0LENR NETC__ENETC0_SI6.BDR_NUM[0].RBLENR.R /* Rx BDR 0 length register */
#define NETC__ENETC0_SI6_RB0DCR NETC__ENETC0_SI6.BDR_NUM[0].RBDCR.R /* Rx BDR 0 drop count register */
#define NETC__ENETC0_SI6_RB0IER NETC__ENETC0_SI6.BDR_NUM[0].RBIER.R /* Rx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI6_RB0IDR NETC__ENETC0_SI6.BDR_NUM[0].RBIDR.R /* Rx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI6_RB0ICR0 NETC__ENETC0_SI6.BDR_NUM[0].RBICR0.R /* Rx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB0ICR1 NETC__ENETC0_SI6.BDR_NUM[0].RBICR1.R /* Rx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB1MR NETC__ENETC0_SI6.BDR_NUM[1].TBMR.R /* Tx BDR 1 mode register */
#define NETC__ENETC0_SI6_TB1SR NETC__ENETC0_SI6.BDR_NUM[1].TBSR.R /* Tx BDR 1 status register */
#define NETC__ENETC0_SI6_TB1BAR0 NETC__ENETC0_SI6.BDR_NUM[1].TBBAR0.R /* Tx BDR 1 base address register 0 */
#define NETC__ENETC0_SI6_TB1BAR1 NETC__ENETC0_SI6.BDR_NUM[1].TBBAR1.R /* Tx BDR 1 base address register 1 */
#define NETC__ENETC0_SI6_TB1PIR NETC__ENETC0_SI6.BDR_NUM[1].TBPIR.R /* Tx BDR 1 producer index register */
#define NETC__ENETC0_SI6_TB1CIR NETC__ENETC0_SI6.BDR_NUM[1].TBCIR.R /* Tx BDR 1 consumer index register */
#define NETC__ENETC0_SI6_TB1LENR NETC__ENETC0_SI6.BDR_NUM[1].TBLENR.R /* Tx BDR 1 length register */
#define NETC__ENETC0_SI6_TB1IER NETC__ENETC0_SI6.BDR_NUM[1].TBIER.R /* Tx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI6_TB1IDR NETC__ENETC0_SI6.BDR_NUM[1].TBIDR.R /* Tx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI6_TB1ICR0 NETC__ENETC0_SI6.BDR_NUM[1].TBICR0.R /* Tx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB1ICR1 NETC__ENETC0_SI6.BDR_NUM[1].TBICR1.R /* Tx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB1MR NETC__ENETC0_SI6.BDR_NUM[1].RBMR.R /* Rx BDR 1 mode register */
#define NETC__ENETC0_SI6_RB1SR NETC__ENETC0_SI6.BDR_NUM[1].RBSR.R /* Rx BDR 1 status register */
#define NETC__ENETC0_SI6_RB1BSR NETC__ENETC0_SI6.BDR_NUM[1].RBBSR.R /* Rx BDR 1 buffer size register */
#define NETC__ENETC0_SI6_RB1CIR NETC__ENETC0_SI6.BDR_NUM[1].RBCIR.R /* Rx BDR 1 consumer index register */
#define NETC__ENETC0_SI6_RB1BAR0 NETC__ENETC0_SI6.BDR_NUM[1].RBBAR0.R /* Rx BDR 1 base address register 0 */
#define NETC__ENETC0_SI6_RB1BAR1 NETC__ENETC0_SI6.BDR_NUM[1].RBBAR1.R /* Rx BDR 1 base address register 1 */
#define NETC__ENETC0_SI6_RB1PIR NETC__ENETC0_SI6.BDR_NUM[1].RBPIR.R /* Rx BDR 1 producer index register */
#define NETC__ENETC0_SI6_RB1LENR NETC__ENETC0_SI6.BDR_NUM[1].RBLENR.R /* Rx BDR 1 length register */
#define NETC__ENETC0_SI6_RB1DCR NETC__ENETC0_SI6.BDR_NUM[1].RBDCR.R /* Rx BDR 1 drop count register */
#define NETC__ENETC0_SI6_RB1IER NETC__ENETC0_SI6.BDR_NUM[1].RBIER.R /* Rx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI6_RB1IDR NETC__ENETC0_SI6.BDR_NUM[1].RBIDR.R /* Rx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI6_RB1ICR0 NETC__ENETC0_SI6.BDR_NUM[1].RBICR0.R /* Rx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB1ICR1 NETC__ENETC0_SI6.BDR_NUM[1].RBICR1.R /* Rx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB2MR NETC__ENETC0_SI6.BDR_NUM[2].TBMR.R /* Tx BDR 2 mode register */
#define NETC__ENETC0_SI6_TB2SR NETC__ENETC0_SI6.BDR_NUM[2].TBSR.R /* Tx BDR 2 status register */
#define NETC__ENETC0_SI6_TB2BAR0 NETC__ENETC0_SI6.BDR_NUM[2].TBBAR0.R /* Tx BDR 2 base address register 0 */
#define NETC__ENETC0_SI6_TB2BAR1 NETC__ENETC0_SI6.BDR_NUM[2].TBBAR1.R /* Tx BDR 2 base address register 1 */
#define NETC__ENETC0_SI6_TB2PIR NETC__ENETC0_SI6.BDR_NUM[2].TBPIR.R /* Tx BDR 2 producer index register */
#define NETC__ENETC0_SI6_TB2CIR NETC__ENETC0_SI6.BDR_NUM[2].TBCIR.R /* Tx BDR 2 consumer index register */
#define NETC__ENETC0_SI6_TB2LENR NETC__ENETC0_SI6.BDR_NUM[2].TBLENR.R /* Tx BDR 2 length register */
#define NETC__ENETC0_SI6_TB2IER NETC__ENETC0_SI6.BDR_NUM[2].TBIER.R /* Tx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI6_TB2IDR NETC__ENETC0_SI6.BDR_NUM[2].TBIDR.R /* Tx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI6_TB2ICR0 NETC__ENETC0_SI6.BDR_NUM[2].TBICR0.R /* Tx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB2ICR1 NETC__ENETC0_SI6.BDR_NUM[2].TBICR1.R /* Tx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB2MR NETC__ENETC0_SI6.BDR_NUM[2].RBMR.R /* Rx BDR 2 mode register */
#define NETC__ENETC0_SI6_RB2SR NETC__ENETC0_SI6.BDR_NUM[2].RBSR.R /* Rx BDR 2 status register */
#define NETC__ENETC0_SI6_RB2BSR NETC__ENETC0_SI6.BDR_NUM[2].RBBSR.R /* Rx BDR 2 buffer size register */
#define NETC__ENETC0_SI6_RB2CIR NETC__ENETC0_SI6.BDR_NUM[2].RBCIR.R /* Rx BDR 2 consumer index register */
#define NETC__ENETC0_SI6_RB2BAR0 NETC__ENETC0_SI6.BDR_NUM[2].RBBAR0.R /* Rx BDR 2 base address register 0 */
#define NETC__ENETC0_SI6_RB2BAR1 NETC__ENETC0_SI6.BDR_NUM[2].RBBAR1.R /* Rx BDR 2 base address register 1 */
#define NETC__ENETC0_SI6_RB2PIR NETC__ENETC0_SI6.BDR_NUM[2].RBPIR.R /* Rx BDR 2 producer index register */
#define NETC__ENETC0_SI6_RB2LENR NETC__ENETC0_SI6.BDR_NUM[2].RBLENR.R /* Rx BDR 2 length register */
#define NETC__ENETC0_SI6_RB2DCR NETC__ENETC0_SI6.BDR_NUM[2].RBDCR.R /* Rx BDR 2 drop count register */
#define NETC__ENETC0_SI6_RB2IER NETC__ENETC0_SI6.BDR_NUM[2].RBIER.R /* Rx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI6_RB2IDR NETC__ENETC0_SI6.BDR_NUM[2].RBIDR.R /* Rx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI6_RB2ICR0 NETC__ENETC0_SI6.BDR_NUM[2].RBICR0.R /* Rx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB2ICR1 NETC__ENETC0_SI6.BDR_NUM[2].RBICR1.R /* Rx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB3MR NETC__ENETC0_SI6.BDR_NUM[3].TBMR.R /* Tx BDR 3 mode register */
#define NETC__ENETC0_SI6_TB3SR NETC__ENETC0_SI6.BDR_NUM[3].TBSR.R /* Tx BDR 3 status register */
#define NETC__ENETC0_SI6_TB3BAR0 NETC__ENETC0_SI6.BDR_NUM[3].TBBAR0.R /* Tx BDR 3 base address register 0 */
#define NETC__ENETC0_SI6_TB3BAR1 NETC__ENETC0_SI6.BDR_NUM[3].TBBAR1.R /* Tx BDR 3 base address register 1 */
#define NETC__ENETC0_SI6_TB3PIR NETC__ENETC0_SI6.BDR_NUM[3].TBPIR.R /* Tx BDR 3 producer index register */
#define NETC__ENETC0_SI6_TB3CIR NETC__ENETC0_SI6.BDR_NUM[3].TBCIR.R /* Tx BDR 3 consumer index register */
#define NETC__ENETC0_SI6_TB3LENR NETC__ENETC0_SI6.BDR_NUM[3].TBLENR.R /* Tx BDR 3 length register */
#define NETC__ENETC0_SI6_TB3IER NETC__ENETC0_SI6.BDR_NUM[3].TBIER.R /* Tx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI6_TB3IDR NETC__ENETC0_SI6.BDR_NUM[3].TBIDR.R /* Tx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI6_TB3ICR0 NETC__ENETC0_SI6.BDR_NUM[3].TBICR0.R /* Tx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB3ICR1 NETC__ENETC0_SI6.BDR_NUM[3].TBICR1.R /* Tx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB3MR NETC__ENETC0_SI6.BDR_NUM[3].RBMR.R /* Rx BDR 3 mode register */
#define NETC__ENETC0_SI6_RB3SR NETC__ENETC0_SI6.BDR_NUM[3].RBSR.R /* Rx BDR 3 status register */
#define NETC__ENETC0_SI6_RB3BSR NETC__ENETC0_SI6.BDR_NUM[3].RBBSR.R /* Rx BDR 3 buffer size register */
#define NETC__ENETC0_SI6_RB3CIR NETC__ENETC0_SI6.BDR_NUM[3].RBCIR.R /* Rx BDR 3 consumer index register */
#define NETC__ENETC0_SI6_RB3BAR0 NETC__ENETC0_SI6.BDR_NUM[3].RBBAR0.R /* Rx BDR 3 base address register 0 */
#define NETC__ENETC0_SI6_RB3BAR1 NETC__ENETC0_SI6.BDR_NUM[3].RBBAR1.R /* Rx BDR 3 base address register 1 */
#define NETC__ENETC0_SI6_RB3PIR NETC__ENETC0_SI6.BDR_NUM[3].RBPIR.R /* Rx BDR 3 producer index register */
#define NETC__ENETC0_SI6_RB3LENR NETC__ENETC0_SI6.BDR_NUM[3].RBLENR.R /* Rx BDR 3 length register */
#define NETC__ENETC0_SI6_RB3DCR NETC__ENETC0_SI6.BDR_NUM[3].RBDCR.R /* Rx BDR 3 drop count register */
#define NETC__ENETC0_SI6_RB3IER NETC__ENETC0_SI6.BDR_NUM[3].RBIER.R /* Rx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI6_RB3IDR NETC__ENETC0_SI6.BDR_NUM[3].RBIDR.R /* Rx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI6_RB3ICR0 NETC__ENETC0_SI6.BDR_NUM[3].RBICR0.R /* Rx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB3ICR1 NETC__ENETC0_SI6.BDR_NUM[3].RBICR1.R /* Rx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB4MR NETC__ENETC0_SI6.BDR_NUM[4].TBMR.R /* Tx BDR 4 mode register */
#define NETC__ENETC0_SI6_TB4SR NETC__ENETC0_SI6.BDR_NUM[4].TBSR.R /* Tx BDR 4 status register */
#define NETC__ENETC0_SI6_TB4BAR0 NETC__ENETC0_SI6.BDR_NUM[4].TBBAR0.R /* Tx BDR 4 base address register 0 */
#define NETC__ENETC0_SI6_TB4BAR1 NETC__ENETC0_SI6.BDR_NUM[4].TBBAR1.R /* Tx BDR 4 base address register 1 */
#define NETC__ENETC0_SI6_TB4PIR NETC__ENETC0_SI6.BDR_NUM[4].TBPIR.R /* Tx BDR 4 producer index register */
#define NETC__ENETC0_SI6_TB4CIR NETC__ENETC0_SI6.BDR_NUM[4].TBCIR.R /* Tx BDR 4 consumer index register */
#define NETC__ENETC0_SI6_TB4LENR NETC__ENETC0_SI6.BDR_NUM[4].TBLENR.R /* Tx BDR 4 length register */
#define NETC__ENETC0_SI6_TB4IER NETC__ENETC0_SI6.BDR_NUM[4].TBIER.R /* Tx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI6_TB4IDR NETC__ENETC0_SI6.BDR_NUM[4].TBIDR.R /* Tx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI6_TB4ICR0 NETC__ENETC0_SI6.BDR_NUM[4].TBICR0.R /* Tx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB4ICR1 NETC__ENETC0_SI6.BDR_NUM[4].TBICR1.R /* Tx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB4MR NETC__ENETC0_SI6.BDR_NUM[4].RBMR.R /* Rx BDR 4 mode register */
#define NETC__ENETC0_SI6_RB4SR NETC__ENETC0_SI6.BDR_NUM[4].RBSR.R /* Rx BDR 4 status register */
#define NETC__ENETC0_SI6_RB4BSR NETC__ENETC0_SI6.BDR_NUM[4].RBBSR.R /* Rx BDR 4 buffer size register */
#define NETC__ENETC0_SI6_RB4CIR NETC__ENETC0_SI6.BDR_NUM[4].RBCIR.R /* Rx BDR 4 consumer index register */
#define NETC__ENETC0_SI6_RB4BAR0 NETC__ENETC0_SI6.BDR_NUM[4].RBBAR0.R /* Rx BDR 4 base address register 0 */
#define NETC__ENETC0_SI6_RB4BAR1 NETC__ENETC0_SI6.BDR_NUM[4].RBBAR1.R /* Rx BDR 4 base address register 1 */
#define NETC__ENETC0_SI6_RB4PIR NETC__ENETC0_SI6.BDR_NUM[4].RBPIR.R /* Rx BDR 4 producer index register */
#define NETC__ENETC0_SI6_RB4LENR NETC__ENETC0_SI6.BDR_NUM[4].RBLENR.R /* Rx BDR 4 length register */
#define NETC__ENETC0_SI6_RB4DCR NETC__ENETC0_SI6.BDR_NUM[4].RBDCR.R /* Rx BDR 4 drop count register */
#define NETC__ENETC0_SI6_RB4IER NETC__ENETC0_SI6.BDR_NUM[4].RBIER.R /* Rx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI6_RB4IDR NETC__ENETC0_SI6.BDR_NUM[4].RBIDR.R /* Rx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI6_RB4ICR0 NETC__ENETC0_SI6.BDR_NUM[4].RBICR0.R /* Rx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB4ICR1 NETC__ENETC0_SI6.BDR_NUM[4].RBICR1.R /* Rx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB5MR NETC__ENETC0_SI6.BDR_NUM[5].TBMR.R /* Tx BDR 5 mode register */
#define NETC__ENETC0_SI6_TB5SR NETC__ENETC0_SI6.BDR_NUM[5].TBSR.R /* Tx BDR 5 status register */
#define NETC__ENETC0_SI6_TB5BAR0 NETC__ENETC0_SI6.BDR_NUM[5].TBBAR0.R /* Tx BDR 5 base address register 0 */
#define NETC__ENETC0_SI6_TB5BAR1 NETC__ENETC0_SI6.BDR_NUM[5].TBBAR1.R /* Tx BDR 5 base address register 1 */
#define NETC__ENETC0_SI6_TB5PIR NETC__ENETC0_SI6.BDR_NUM[5].TBPIR.R /* Tx BDR 5 producer index register */
#define NETC__ENETC0_SI6_TB5CIR NETC__ENETC0_SI6.BDR_NUM[5].TBCIR.R /* Tx BDR 5 consumer index register */
#define NETC__ENETC0_SI6_TB5LENR NETC__ENETC0_SI6.BDR_NUM[5].TBLENR.R /* Tx BDR 5 length register */
#define NETC__ENETC0_SI6_TB5IER NETC__ENETC0_SI6.BDR_NUM[5].TBIER.R /* Tx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI6_TB5IDR NETC__ENETC0_SI6.BDR_NUM[5].TBIDR.R /* Tx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI6_TB5ICR0 NETC__ENETC0_SI6.BDR_NUM[5].TBICR0.R /* Tx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB5ICR1 NETC__ENETC0_SI6.BDR_NUM[5].TBICR1.R /* Tx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB5MR NETC__ENETC0_SI6.BDR_NUM[5].RBMR.R /* Rx BDR 5 mode register */
#define NETC__ENETC0_SI6_RB5SR NETC__ENETC0_SI6.BDR_NUM[5].RBSR.R /* Rx BDR 5 status register */
#define NETC__ENETC0_SI6_RB5BSR NETC__ENETC0_SI6.BDR_NUM[5].RBBSR.R /* Rx BDR 5 buffer size register */
#define NETC__ENETC0_SI6_RB5CIR NETC__ENETC0_SI6.BDR_NUM[5].RBCIR.R /* Rx BDR 5 consumer index register */
#define NETC__ENETC0_SI6_RB5BAR0 NETC__ENETC0_SI6.BDR_NUM[5].RBBAR0.R /* Rx BDR 5 base address register 0 */
#define NETC__ENETC0_SI6_RB5BAR1 NETC__ENETC0_SI6.BDR_NUM[5].RBBAR1.R /* Rx BDR 5 base address register 1 */
#define NETC__ENETC0_SI6_RB5PIR NETC__ENETC0_SI6.BDR_NUM[5].RBPIR.R /* Rx BDR 5 producer index register */
#define NETC__ENETC0_SI6_RB5LENR NETC__ENETC0_SI6.BDR_NUM[5].RBLENR.R /* Rx BDR 5 length register */
#define NETC__ENETC0_SI6_RB5DCR NETC__ENETC0_SI6.BDR_NUM[5].RBDCR.R /* Rx BDR 5 drop count register */
#define NETC__ENETC0_SI6_RB5IER NETC__ENETC0_SI6.BDR_NUM[5].RBIER.R /* Rx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI6_RB5IDR NETC__ENETC0_SI6.BDR_NUM[5].RBIDR.R /* Rx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI6_RB5ICR0 NETC__ENETC0_SI6.BDR_NUM[5].RBICR0.R /* Rx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB5ICR1 NETC__ENETC0_SI6.BDR_NUM[5].RBICR1.R /* Rx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB6MR NETC__ENETC0_SI6.BDR_NUM[6].TBMR.R /* Tx BDR 6 mode register */
#define NETC__ENETC0_SI6_TB6SR NETC__ENETC0_SI6.BDR_NUM[6].TBSR.R /* Tx BDR 6 status register */
#define NETC__ENETC0_SI6_TB6BAR0 NETC__ENETC0_SI6.BDR_NUM[6].TBBAR0.R /* Tx BDR 6 base address register 0 */
#define NETC__ENETC0_SI6_TB6BAR1 NETC__ENETC0_SI6.BDR_NUM[6].TBBAR1.R /* Tx BDR 6 base address register 1 */
#define NETC__ENETC0_SI6_TB6PIR NETC__ENETC0_SI6.BDR_NUM[6].TBPIR.R /* Tx BDR 6 producer index register */
#define NETC__ENETC0_SI6_TB6CIR NETC__ENETC0_SI6.BDR_NUM[6].TBCIR.R /* Tx BDR 6 consumer index register */
#define NETC__ENETC0_SI6_TB6LENR NETC__ENETC0_SI6.BDR_NUM[6].TBLENR.R /* Tx BDR 6 length register */
#define NETC__ENETC0_SI6_TB6IER NETC__ENETC0_SI6.BDR_NUM[6].TBIER.R /* Tx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI6_TB6IDR NETC__ENETC0_SI6.BDR_NUM[6].TBIDR.R /* Tx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI6_TB6ICR0 NETC__ENETC0_SI6.BDR_NUM[6].TBICR0.R /* Tx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB6ICR1 NETC__ENETC0_SI6.BDR_NUM[6].TBICR1.R /* Tx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB6MR NETC__ENETC0_SI6.BDR_NUM[6].RBMR.R /* Rx BDR 6 mode register */
#define NETC__ENETC0_SI6_RB6SR NETC__ENETC0_SI6.BDR_NUM[6].RBSR.R /* Rx BDR 6 status register */
#define NETC__ENETC0_SI6_RB6BSR NETC__ENETC0_SI6.BDR_NUM[6].RBBSR.R /* Rx BDR 6 buffer size register */
#define NETC__ENETC0_SI6_RB6CIR NETC__ENETC0_SI6.BDR_NUM[6].RBCIR.R /* Rx BDR 6 consumer index register */
#define NETC__ENETC0_SI6_RB6BAR0 NETC__ENETC0_SI6.BDR_NUM[6].RBBAR0.R /* Rx BDR 6 base address register 0 */
#define NETC__ENETC0_SI6_RB6BAR1 NETC__ENETC0_SI6.BDR_NUM[6].RBBAR1.R /* Rx BDR 6 base address register 1 */
#define NETC__ENETC0_SI6_RB6PIR NETC__ENETC0_SI6.BDR_NUM[6].RBPIR.R /* Rx BDR 6 producer index register */
#define NETC__ENETC0_SI6_RB6LENR NETC__ENETC0_SI6.BDR_NUM[6].RBLENR.R /* Rx BDR 6 length register */
#define NETC__ENETC0_SI6_RB6DCR NETC__ENETC0_SI6.BDR_NUM[6].RBDCR.R /* Rx BDR 6 drop count register */
#define NETC__ENETC0_SI6_RB6IER NETC__ENETC0_SI6.BDR_NUM[6].RBIER.R /* Rx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI6_RB6IDR NETC__ENETC0_SI6.BDR_NUM[6].RBIDR.R /* Rx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI6_RB6ICR0 NETC__ENETC0_SI6.BDR_NUM[6].RBICR0.R /* Rx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB6ICR1 NETC__ENETC0_SI6.BDR_NUM[6].RBICR1.R /* Rx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB7MR NETC__ENETC0_SI6.BDR_NUM[7].TBMR.R /* Tx BDR 7 mode register */
#define NETC__ENETC0_SI6_TB7SR NETC__ENETC0_SI6.BDR_NUM[7].TBSR.R /* Tx BDR 7 status register */
#define NETC__ENETC0_SI6_TB7BAR0 NETC__ENETC0_SI6.BDR_NUM[7].TBBAR0.R /* Tx BDR 7 base address register 0 */
#define NETC__ENETC0_SI6_TB7BAR1 NETC__ENETC0_SI6.BDR_NUM[7].TBBAR1.R /* Tx BDR 7 base address register 1 */
#define NETC__ENETC0_SI6_TB7PIR NETC__ENETC0_SI6.BDR_NUM[7].TBPIR.R /* Tx BDR 7 producer index register */
#define NETC__ENETC0_SI6_TB7CIR NETC__ENETC0_SI6.BDR_NUM[7].TBCIR.R /* Tx BDR 7 consumer index register */
#define NETC__ENETC0_SI6_TB7LENR NETC__ENETC0_SI6.BDR_NUM[7].TBLENR.R /* Tx BDR 7 length register */
#define NETC__ENETC0_SI6_TB7IER NETC__ENETC0_SI6.BDR_NUM[7].TBIER.R /* Tx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI6_TB7IDR NETC__ENETC0_SI6.BDR_NUM[7].TBIDR.R /* Tx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI6_TB7ICR0 NETC__ENETC0_SI6.BDR_NUM[7].TBICR0.R /* Tx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB7ICR1 NETC__ENETC0_SI6.BDR_NUM[7].TBICR1.R /* Tx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB7MR NETC__ENETC0_SI6.BDR_NUM[7].RBMR.R /* Rx BDR 7 mode register */
#define NETC__ENETC0_SI6_RB7SR NETC__ENETC0_SI6.BDR_NUM[7].RBSR.R /* Rx BDR 7 status register */
#define NETC__ENETC0_SI6_RB7BSR NETC__ENETC0_SI6.BDR_NUM[7].RBBSR.R /* Rx BDR 7 buffer size register */
#define NETC__ENETC0_SI6_RB7CIR NETC__ENETC0_SI6.BDR_NUM[7].RBCIR.R /* Rx BDR 7 consumer index register */
#define NETC__ENETC0_SI6_RB7BAR0 NETC__ENETC0_SI6.BDR_NUM[7].RBBAR0.R /* Rx BDR 7 base address register 0 */
#define NETC__ENETC0_SI6_RB7BAR1 NETC__ENETC0_SI6.BDR_NUM[7].RBBAR1.R /* Rx BDR 7 base address register 1 */
#define NETC__ENETC0_SI6_RB7PIR NETC__ENETC0_SI6.BDR_NUM[7].RBPIR.R /* Rx BDR 7 producer index register */
#define NETC__ENETC0_SI6_RB7LENR NETC__ENETC0_SI6.BDR_NUM[7].RBLENR.R /* Rx BDR 7 length register */
#define NETC__ENETC0_SI6_RB7DCR NETC__ENETC0_SI6.BDR_NUM[7].RBDCR.R /* Rx BDR 7 drop count register */
#define NETC__ENETC0_SI6_RB7IER NETC__ENETC0_SI6.BDR_NUM[7].RBIER.R /* Rx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI6_RB7IDR NETC__ENETC0_SI6.BDR_NUM[7].RBIDR.R /* Rx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI6_RB7ICR0 NETC__ENETC0_SI6.BDR_NUM[7].RBICR0.R /* Rx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB7ICR1 NETC__ENETC0_SI6.BDR_NUM[7].RBICR1.R /* Rx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB8MR NETC__ENETC0_SI6.BDR_NUM[8].TBMR.R /* Tx BDR 8 mode register */
#define NETC__ENETC0_SI6_TB8SR NETC__ENETC0_SI6.BDR_NUM[8].TBSR.R /* Tx BDR 8 status register */
#define NETC__ENETC0_SI6_TB8BAR0 NETC__ENETC0_SI6.BDR_NUM[8].TBBAR0.R /* Tx BDR 8 base address register 0 */
#define NETC__ENETC0_SI6_TB8BAR1 NETC__ENETC0_SI6.BDR_NUM[8].TBBAR1.R /* Tx BDR 8 base address register 1 */
#define NETC__ENETC0_SI6_TB8PIR NETC__ENETC0_SI6.BDR_NUM[8].TBPIR.R /* Tx BDR 8 producer index register */
#define NETC__ENETC0_SI6_TB8CIR NETC__ENETC0_SI6.BDR_NUM[8].TBCIR.R /* Tx BDR 8 consumer index register */
#define NETC__ENETC0_SI6_TB8LENR NETC__ENETC0_SI6.BDR_NUM[8].TBLENR.R /* Tx BDR 8 length register */
#define NETC__ENETC0_SI6_TB8IER NETC__ENETC0_SI6.BDR_NUM[8].TBIER.R /* Tx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI6_TB8IDR NETC__ENETC0_SI6.BDR_NUM[8].TBIDR.R /* Tx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI6_TB8ICR0 NETC__ENETC0_SI6.BDR_NUM[8].TBICR0.R /* Tx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB8ICR1 NETC__ENETC0_SI6.BDR_NUM[8].TBICR1.R /* Tx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB8MR NETC__ENETC0_SI6.BDR_NUM[8].RBMR.R /* Rx BDR 8 mode register */
#define NETC__ENETC0_SI6_RB8SR NETC__ENETC0_SI6.BDR_NUM[8].RBSR.R /* Rx BDR 8 status register */
#define NETC__ENETC0_SI6_RB8BSR NETC__ENETC0_SI6.BDR_NUM[8].RBBSR.R /* Rx BDR 8 buffer size register */
#define NETC__ENETC0_SI6_RB8CIR NETC__ENETC0_SI6.BDR_NUM[8].RBCIR.R /* Rx BDR 8 consumer index register */
#define NETC__ENETC0_SI6_RB8BAR0 NETC__ENETC0_SI6.BDR_NUM[8].RBBAR0.R /* Rx BDR 8 base address register 0 */
#define NETC__ENETC0_SI6_RB8BAR1 NETC__ENETC0_SI6.BDR_NUM[8].RBBAR1.R /* Rx BDR 8 base address register 1 */
#define NETC__ENETC0_SI6_RB8PIR NETC__ENETC0_SI6.BDR_NUM[8].RBPIR.R /* Rx BDR 8 producer index register */
#define NETC__ENETC0_SI6_RB8LENR NETC__ENETC0_SI6.BDR_NUM[8].RBLENR.R /* Rx BDR 8 length register */
#define NETC__ENETC0_SI6_RB8DCR NETC__ENETC0_SI6.BDR_NUM[8].RBDCR.R /* Rx BDR 8 drop count register */
#define NETC__ENETC0_SI6_RB8IER NETC__ENETC0_SI6.BDR_NUM[8].RBIER.R /* Rx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI6_RB8IDR NETC__ENETC0_SI6.BDR_NUM[8].RBIDR.R /* Rx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI6_RB8ICR0 NETC__ENETC0_SI6.BDR_NUM[8].RBICR0.R /* Rx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB8ICR1 NETC__ENETC0_SI6.BDR_NUM[8].RBICR1.R /* Rx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB9MR NETC__ENETC0_SI6.BDR_NUM[9].TBMR.R /* Tx BDR 9 mode register */
#define NETC__ENETC0_SI6_TB9SR NETC__ENETC0_SI6.BDR_NUM[9].TBSR.R /* Tx BDR 9 status register */
#define NETC__ENETC0_SI6_TB9BAR0 NETC__ENETC0_SI6.BDR_NUM[9].TBBAR0.R /* Tx BDR 9 base address register 0 */
#define NETC__ENETC0_SI6_TB9BAR1 NETC__ENETC0_SI6.BDR_NUM[9].TBBAR1.R /* Tx BDR 9 base address register 1 */
#define NETC__ENETC0_SI6_TB9PIR NETC__ENETC0_SI6.BDR_NUM[9].TBPIR.R /* Tx BDR 9 producer index register */
#define NETC__ENETC0_SI6_TB9CIR NETC__ENETC0_SI6.BDR_NUM[9].TBCIR.R /* Tx BDR 9 consumer index register */
#define NETC__ENETC0_SI6_TB9LENR NETC__ENETC0_SI6.BDR_NUM[9].TBLENR.R /* Tx BDR 9 length register */
#define NETC__ENETC0_SI6_TB9IER NETC__ENETC0_SI6.BDR_NUM[9].TBIER.R /* Tx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI6_TB9IDR NETC__ENETC0_SI6.BDR_NUM[9].TBIDR.R /* Tx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI6_TB9ICR0 NETC__ENETC0_SI6.BDR_NUM[9].TBICR0.R /* Tx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB9ICR1 NETC__ENETC0_SI6.BDR_NUM[9].TBICR1.R /* Tx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB9MR NETC__ENETC0_SI6.BDR_NUM[9].RBMR.R /* Rx BDR 9 mode register */
#define NETC__ENETC0_SI6_RB9SR NETC__ENETC0_SI6.BDR_NUM[9].RBSR.R /* Rx BDR 9 status register */
#define NETC__ENETC0_SI6_RB9BSR NETC__ENETC0_SI6.BDR_NUM[9].RBBSR.R /* Rx BDR 9 buffer size register */
#define NETC__ENETC0_SI6_RB9CIR NETC__ENETC0_SI6.BDR_NUM[9].RBCIR.R /* Rx BDR 9 consumer index register */
#define NETC__ENETC0_SI6_RB9BAR0 NETC__ENETC0_SI6.BDR_NUM[9].RBBAR0.R /* Rx BDR 9 base address register 0 */
#define NETC__ENETC0_SI6_RB9BAR1 NETC__ENETC0_SI6.BDR_NUM[9].RBBAR1.R /* Rx BDR 9 base address register 1 */
#define NETC__ENETC0_SI6_RB9PIR NETC__ENETC0_SI6.BDR_NUM[9].RBPIR.R /* Rx BDR 9 producer index register */
#define NETC__ENETC0_SI6_RB9LENR NETC__ENETC0_SI6.BDR_NUM[9].RBLENR.R /* Rx BDR 9 length register */
#define NETC__ENETC0_SI6_RB9DCR NETC__ENETC0_SI6.BDR_NUM[9].RBDCR.R /* Rx BDR 9 drop count register */
#define NETC__ENETC0_SI6_RB9IER NETC__ENETC0_SI6.BDR_NUM[9].RBIER.R /* Rx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI6_RB9IDR NETC__ENETC0_SI6.BDR_NUM[9].RBIDR.R /* Rx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI6_RB9ICR0 NETC__ENETC0_SI6.BDR_NUM[9].RBICR0.R /* Rx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB9ICR1 NETC__ENETC0_SI6.BDR_NUM[9].RBICR1.R /* Rx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB10MR NETC__ENETC0_SI6.BDR_NUM[10].TBMR.R /* Tx BDR 10 mode register */
#define NETC__ENETC0_SI6_TB10SR NETC__ENETC0_SI6.BDR_NUM[10].TBSR.R /* Tx BDR 10 status register */
#define NETC__ENETC0_SI6_TB10BAR0 NETC__ENETC0_SI6.BDR_NUM[10].TBBAR0.R /* Tx BDR 10 base address register 0 */
#define NETC__ENETC0_SI6_TB10BAR1 NETC__ENETC0_SI6.BDR_NUM[10].TBBAR1.R /* Tx BDR 10 base address register 1 */
#define NETC__ENETC0_SI6_TB10PIR NETC__ENETC0_SI6.BDR_NUM[10].TBPIR.R /* Tx BDR 10 producer index register */
#define NETC__ENETC0_SI6_TB10CIR NETC__ENETC0_SI6.BDR_NUM[10].TBCIR.R /* Tx BDR 10 consumer index register */
#define NETC__ENETC0_SI6_TB10LENR NETC__ENETC0_SI6.BDR_NUM[10].TBLENR.R /* Tx BDR 10 length register */
#define NETC__ENETC0_SI6_TB10IER NETC__ENETC0_SI6.BDR_NUM[10].TBIER.R /* Tx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI6_TB10IDR NETC__ENETC0_SI6.BDR_NUM[10].TBIDR.R /* Tx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI6_TB10ICR0 NETC__ENETC0_SI6.BDR_NUM[10].TBICR0.R /* Tx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB10ICR1 NETC__ENETC0_SI6.BDR_NUM[10].TBICR1.R /* Tx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB10MR NETC__ENETC0_SI6.BDR_NUM[10].RBMR.R /* Rx BDR 10 mode register */
#define NETC__ENETC0_SI6_RB10SR NETC__ENETC0_SI6.BDR_NUM[10].RBSR.R /* Rx BDR 10 status register */
#define NETC__ENETC0_SI6_RB10BSR NETC__ENETC0_SI6.BDR_NUM[10].RBBSR.R /* Rx BDR 10 buffer size register */
#define NETC__ENETC0_SI6_RB10CIR NETC__ENETC0_SI6.BDR_NUM[10].RBCIR.R /* Rx BDR 10 consumer index register */
#define NETC__ENETC0_SI6_RB10BAR0 NETC__ENETC0_SI6.BDR_NUM[10].RBBAR0.R /* Rx BDR 10 base address register 0 */
#define NETC__ENETC0_SI6_RB10BAR1 NETC__ENETC0_SI6.BDR_NUM[10].RBBAR1.R /* Rx BDR 10 base address register 1 */
#define NETC__ENETC0_SI6_RB10PIR NETC__ENETC0_SI6.BDR_NUM[10].RBPIR.R /* Rx BDR 10 producer index register */
#define NETC__ENETC0_SI6_RB10LENR NETC__ENETC0_SI6.BDR_NUM[10].RBLENR.R /* Rx BDR 10 length register */
#define NETC__ENETC0_SI6_RB10DCR NETC__ENETC0_SI6.BDR_NUM[10].RBDCR.R /* Rx BDR 10 drop count register */
#define NETC__ENETC0_SI6_RB10IER NETC__ENETC0_SI6.BDR_NUM[10].RBIER.R /* Rx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI6_RB10IDR NETC__ENETC0_SI6.BDR_NUM[10].RBIDR.R /* Rx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI6_RB10ICR0 NETC__ENETC0_SI6.BDR_NUM[10].RBICR0.R /* Rx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB10ICR1 NETC__ENETC0_SI6.BDR_NUM[10].RBICR1.R /* Rx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB11MR NETC__ENETC0_SI6.BDR_NUM[11].TBMR.R /* Tx BDR 11 mode register */
#define NETC__ENETC0_SI6_TB11SR NETC__ENETC0_SI6.BDR_NUM[11].TBSR.R /* Tx BDR 11 status register */
#define NETC__ENETC0_SI6_TB11BAR0 NETC__ENETC0_SI6.BDR_NUM[11].TBBAR0.R /* Tx BDR 11 base address register 0 */
#define NETC__ENETC0_SI6_TB11BAR1 NETC__ENETC0_SI6.BDR_NUM[11].TBBAR1.R /* Tx BDR 11 base address register 1 */
#define NETC__ENETC0_SI6_TB11PIR NETC__ENETC0_SI6.BDR_NUM[11].TBPIR.R /* Tx BDR 11 producer index register */
#define NETC__ENETC0_SI6_TB11CIR NETC__ENETC0_SI6.BDR_NUM[11].TBCIR.R /* Tx BDR 11 consumer index register */
#define NETC__ENETC0_SI6_TB11LENR NETC__ENETC0_SI6.BDR_NUM[11].TBLENR.R /* Tx BDR 11 length register */
#define NETC__ENETC0_SI6_TB11IER NETC__ENETC0_SI6.BDR_NUM[11].TBIER.R /* Tx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI6_TB11IDR NETC__ENETC0_SI6.BDR_NUM[11].TBIDR.R /* Tx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI6_TB11ICR0 NETC__ENETC0_SI6.BDR_NUM[11].TBICR0.R /* Tx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB11ICR1 NETC__ENETC0_SI6.BDR_NUM[11].TBICR1.R /* Tx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB11MR NETC__ENETC0_SI6.BDR_NUM[11].RBMR.R /* Rx BDR 11 mode register */
#define NETC__ENETC0_SI6_RB11SR NETC__ENETC0_SI6.BDR_NUM[11].RBSR.R /* Rx BDR 11 status register */
#define NETC__ENETC0_SI6_RB11BSR NETC__ENETC0_SI6.BDR_NUM[11].RBBSR.R /* Rx BDR 11 buffer size register */
#define NETC__ENETC0_SI6_RB11CIR NETC__ENETC0_SI6.BDR_NUM[11].RBCIR.R /* Rx BDR 11 consumer index register */
#define NETC__ENETC0_SI6_RB11BAR0 NETC__ENETC0_SI6.BDR_NUM[11].RBBAR0.R /* Rx BDR 11 base address register 0 */
#define NETC__ENETC0_SI6_RB11BAR1 NETC__ENETC0_SI6.BDR_NUM[11].RBBAR1.R /* Rx BDR 11 base address register 1 */
#define NETC__ENETC0_SI6_RB11PIR NETC__ENETC0_SI6.BDR_NUM[11].RBPIR.R /* Rx BDR 11 producer index register */
#define NETC__ENETC0_SI6_RB11LENR NETC__ENETC0_SI6.BDR_NUM[11].RBLENR.R /* Rx BDR 11 length register */
#define NETC__ENETC0_SI6_RB11DCR NETC__ENETC0_SI6.BDR_NUM[11].RBDCR.R /* Rx BDR 11 drop count register */
#define NETC__ENETC0_SI6_RB11IER NETC__ENETC0_SI6.BDR_NUM[11].RBIER.R /* Rx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI6_RB11IDR NETC__ENETC0_SI6.BDR_NUM[11].RBIDR.R /* Rx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI6_RB11ICR0 NETC__ENETC0_SI6.BDR_NUM[11].RBICR0.R /* Rx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB11ICR1 NETC__ENETC0_SI6.BDR_NUM[11].RBICR1.R /* Rx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB12MR NETC__ENETC0_SI6.BDR_NUM[12].TBMR.R /* Tx BDR 12 mode register */
#define NETC__ENETC0_SI6_TB12SR NETC__ENETC0_SI6.BDR_NUM[12].TBSR.R /* Tx BDR 12 status register */
#define NETC__ENETC0_SI6_TB12BAR0 NETC__ENETC0_SI6.BDR_NUM[12].TBBAR0.R /* Tx BDR 12 base address register 0 */
#define NETC__ENETC0_SI6_TB12BAR1 NETC__ENETC0_SI6.BDR_NUM[12].TBBAR1.R /* Tx BDR 12 base address register 1 */
#define NETC__ENETC0_SI6_TB12PIR NETC__ENETC0_SI6.BDR_NUM[12].TBPIR.R /* Tx BDR 12 producer index register */
#define NETC__ENETC0_SI6_TB12CIR NETC__ENETC0_SI6.BDR_NUM[12].TBCIR.R /* Tx BDR 12 consumer index register */
#define NETC__ENETC0_SI6_TB12LENR NETC__ENETC0_SI6.BDR_NUM[12].TBLENR.R /* Tx BDR 12 length register */
#define NETC__ENETC0_SI6_TB12IER NETC__ENETC0_SI6.BDR_NUM[12].TBIER.R /* Tx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI6_TB12IDR NETC__ENETC0_SI6.BDR_NUM[12].TBIDR.R /* Tx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI6_TB12ICR0 NETC__ENETC0_SI6.BDR_NUM[12].TBICR0.R /* Tx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB12ICR1 NETC__ENETC0_SI6.BDR_NUM[12].TBICR1.R /* Tx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB12MR NETC__ENETC0_SI6.BDR_NUM[12].RBMR.R /* Rx BDR 12 mode register */
#define NETC__ENETC0_SI6_RB12SR NETC__ENETC0_SI6.BDR_NUM[12].RBSR.R /* Rx BDR 12 status register */
#define NETC__ENETC0_SI6_RB12BSR NETC__ENETC0_SI6.BDR_NUM[12].RBBSR.R /* Rx BDR 12 buffer size register */
#define NETC__ENETC0_SI6_RB12CIR NETC__ENETC0_SI6.BDR_NUM[12].RBCIR.R /* Rx BDR 12 consumer index register */
#define NETC__ENETC0_SI6_RB12BAR0 NETC__ENETC0_SI6.BDR_NUM[12].RBBAR0.R /* Rx BDR 12 base address register 0 */
#define NETC__ENETC0_SI6_RB12BAR1 NETC__ENETC0_SI6.BDR_NUM[12].RBBAR1.R /* Rx BDR 12 base address register 1 */
#define NETC__ENETC0_SI6_RB12PIR NETC__ENETC0_SI6.BDR_NUM[12].RBPIR.R /* Rx BDR 12 producer index register */
#define NETC__ENETC0_SI6_RB12LENR NETC__ENETC0_SI6.BDR_NUM[12].RBLENR.R /* Rx BDR 12 length register */
#define NETC__ENETC0_SI6_RB12DCR NETC__ENETC0_SI6.BDR_NUM[12].RBDCR.R /* Rx BDR 12 drop count register */
#define NETC__ENETC0_SI6_RB12IER NETC__ENETC0_SI6.BDR_NUM[12].RBIER.R /* Rx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI6_RB12IDR NETC__ENETC0_SI6.BDR_NUM[12].RBIDR.R /* Rx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI6_RB12ICR0 NETC__ENETC0_SI6.BDR_NUM[12].RBICR0.R /* Rx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB12ICR1 NETC__ENETC0_SI6.BDR_NUM[12].RBICR1.R /* Rx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB13MR NETC__ENETC0_SI6.BDR_NUM[13].TBMR.R /* Tx BDR 13 mode register */
#define NETC__ENETC0_SI6_TB13SR NETC__ENETC0_SI6.BDR_NUM[13].TBSR.R /* Tx BDR 13 status register */
#define NETC__ENETC0_SI6_TB13BAR0 NETC__ENETC0_SI6.BDR_NUM[13].TBBAR0.R /* Tx BDR 13 base address register 0 */
#define NETC__ENETC0_SI6_TB13BAR1 NETC__ENETC0_SI6.BDR_NUM[13].TBBAR1.R /* Tx BDR 13 base address register 1 */
#define NETC__ENETC0_SI6_TB13PIR NETC__ENETC0_SI6.BDR_NUM[13].TBPIR.R /* Tx BDR 13 producer index register */
#define NETC__ENETC0_SI6_TB13CIR NETC__ENETC0_SI6.BDR_NUM[13].TBCIR.R /* Tx BDR 13 consumer index register */
#define NETC__ENETC0_SI6_TB13LENR NETC__ENETC0_SI6.BDR_NUM[13].TBLENR.R /* Tx BDR 13 length register */
#define NETC__ENETC0_SI6_TB13IER NETC__ENETC0_SI6.BDR_NUM[13].TBIER.R /* Tx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI6_TB13IDR NETC__ENETC0_SI6.BDR_NUM[13].TBIDR.R /* Tx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI6_TB13ICR0 NETC__ENETC0_SI6.BDR_NUM[13].TBICR0.R /* Tx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB13ICR1 NETC__ENETC0_SI6.BDR_NUM[13].TBICR1.R /* Tx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB13MR NETC__ENETC0_SI6.BDR_NUM[13].RBMR.R /* Rx BDR 13 mode register */
#define NETC__ENETC0_SI6_RB13SR NETC__ENETC0_SI6.BDR_NUM[13].RBSR.R /* Rx BDR 13 status register */
#define NETC__ENETC0_SI6_RB13BSR NETC__ENETC0_SI6.BDR_NUM[13].RBBSR.R /* Rx BDR 13 buffer size register */
#define NETC__ENETC0_SI6_RB13CIR NETC__ENETC0_SI6.BDR_NUM[13].RBCIR.R /* Rx BDR 13 consumer index register */
#define NETC__ENETC0_SI6_RB13BAR0 NETC__ENETC0_SI6.BDR_NUM[13].RBBAR0.R /* Rx BDR 13 base address register 0 */
#define NETC__ENETC0_SI6_RB13BAR1 NETC__ENETC0_SI6.BDR_NUM[13].RBBAR1.R /* Rx BDR 13 base address register 1 */
#define NETC__ENETC0_SI6_RB13PIR NETC__ENETC0_SI6.BDR_NUM[13].RBPIR.R /* Rx BDR 13 producer index register */
#define NETC__ENETC0_SI6_RB13LENR NETC__ENETC0_SI6.BDR_NUM[13].RBLENR.R /* Rx BDR 13 length register */
#define NETC__ENETC0_SI6_RB13DCR NETC__ENETC0_SI6.BDR_NUM[13].RBDCR.R /* Rx BDR 13 drop count register */
#define NETC__ENETC0_SI6_RB13IER NETC__ENETC0_SI6.BDR_NUM[13].RBIER.R /* Rx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI6_RB13IDR NETC__ENETC0_SI6.BDR_NUM[13].RBIDR.R /* Rx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI6_RB13ICR0 NETC__ENETC0_SI6.BDR_NUM[13].RBICR0.R /* Rx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB13ICR1 NETC__ENETC0_SI6.BDR_NUM[13].RBICR1.R /* Rx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB14MR NETC__ENETC0_SI6.BDR_NUM[14].TBMR.R /* Tx BDR 14 mode register */
#define NETC__ENETC0_SI6_TB14SR NETC__ENETC0_SI6.BDR_NUM[14].TBSR.R /* Tx BDR 14 status register */
#define NETC__ENETC0_SI6_TB14BAR0 NETC__ENETC0_SI6.BDR_NUM[14].TBBAR0.R /* Tx BDR 14 base address register 0 */
#define NETC__ENETC0_SI6_TB14BAR1 NETC__ENETC0_SI6.BDR_NUM[14].TBBAR1.R /* Tx BDR 14 base address register 1 */
#define NETC__ENETC0_SI6_TB14PIR NETC__ENETC0_SI6.BDR_NUM[14].TBPIR.R /* Tx BDR 14 producer index register */
#define NETC__ENETC0_SI6_TB14CIR NETC__ENETC0_SI6.BDR_NUM[14].TBCIR.R /* Tx BDR 14 consumer index register */
#define NETC__ENETC0_SI6_TB14LENR NETC__ENETC0_SI6.BDR_NUM[14].TBLENR.R /* Tx BDR 14 length register */
#define NETC__ENETC0_SI6_TB14IER NETC__ENETC0_SI6.BDR_NUM[14].TBIER.R /* Tx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI6_TB14IDR NETC__ENETC0_SI6.BDR_NUM[14].TBIDR.R /* Tx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI6_TB14ICR0 NETC__ENETC0_SI6.BDR_NUM[14].TBICR0.R /* Tx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB14ICR1 NETC__ENETC0_SI6.BDR_NUM[14].TBICR1.R /* Tx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB14MR NETC__ENETC0_SI6.BDR_NUM[14].RBMR.R /* Rx BDR 14 mode register */
#define NETC__ENETC0_SI6_RB14SR NETC__ENETC0_SI6.BDR_NUM[14].RBSR.R /* Rx BDR 14 status register */
#define NETC__ENETC0_SI6_RB14BSR NETC__ENETC0_SI6.BDR_NUM[14].RBBSR.R /* Rx BDR 14 buffer size register */
#define NETC__ENETC0_SI6_RB14CIR NETC__ENETC0_SI6.BDR_NUM[14].RBCIR.R /* Rx BDR 14 consumer index register */
#define NETC__ENETC0_SI6_RB14BAR0 NETC__ENETC0_SI6.BDR_NUM[14].RBBAR0.R /* Rx BDR 14 base address register 0 */
#define NETC__ENETC0_SI6_RB14BAR1 NETC__ENETC0_SI6.BDR_NUM[14].RBBAR1.R /* Rx BDR 14 base address register 1 */
#define NETC__ENETC0_SI6_RB14PIR NETC__ENETC0_SI6.BDR_NUM[14].RBPIR.R /* Rx BDR 14 producer index register */
#define NETC__ENETC0_SI6_RB14LENR NETC__ENETC0_SI6.BDR_NUM[14].RBLENR.R /* Rx BDR 14 length register */
#define NETC__ENETC0_SI6_RB14DCR NETC__ENETC0_SI6.BDR_NUM[14].RBDCR.R /* Rx BDR 14 drop count register */
#define NETC__ENETC0_SI6_RB14IER NETC__ENETC0_SI6.BDR_NUM[14].RBIER.R /* Rx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI6_RB14IDR NETC__ENETC0_SI6.BDR_NUM[14].RBIDR.R /* Rx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI6_RB14ICR0 NETC__ENETC0_SI6.BDR_NUM[14].RBICR0.R /* Rx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB14ICR1 NETC__ENETC0_SI6.BDR_NUM[14].RBICR1.R /* Rx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB15MR NETC__ENETC0_SI6.BDR_NUM[15].TBMR.R /* Tx BDR 15 mode register */
#define NETC__ENETC0_SI6_TB15SR NETC__ENETC0_SI6.BDR_NUM[15].TBSR.R /* Tx BDR 15 status register */
#define NETC__ENETC0_SI6_TB15BAR0 NETC__ENETC0_SI6.BDR_NUM[15].TBBAR0.R /* Tx BDR 15 base address register 0 */
#define NETC__ENETC0_SI6_TB15BAR1 NETC__ENETC0_SI6.BDR_NUM[15].TBBAR1.R /* Tx BDR 15 base address register 1 */
#define NETC__ENETC0_SI6_TB15PIR NETC__ENETC0_SI6.BDR_NUM[15].TBPIR.R /* Tx BDR 15 producer index register */
#define NETC__ENETC0_SI6_TB15CIR NETC__ENETC0_SI6.BDR_NUM[15].TBCIR.R /* Tx BDR 15 consumer index register */
#define NETC__ENETC0_SI6_TB15LENR NETC__ENETC0_SI6.BDR_NUM[15].TBLENR.R /* Tx BDR 15 length register */
#define NETC__ENETC0_SI6_TB15IER NETC__ENETC0_SI6.BDR_NUM[15].TBIER.R /* Tx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI6_TB15IDR NETC__ENETC0_SI6.BDR_NUM[15].TBIDR.R /* Tx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI6_TB15ICR0 NETC__ENETC0_SI6.BDR_NUM[15].TBICR0.R /* Tx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB15ICR1 NETC__ENETC0_SI6.BDR_NUM[15].TBICR1.R /* Tx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB15MR NETC__ENETC0_SI6.BDR_NUM[15].RBMR.R /* Rx BDR 15 mode register */
#define NETC__ENETC0_SI6_RB15SR NETC__ENETC0_SI6.BDR_NUM[15].RBSR.R /* Rx BDR 15 status register */
#define NETC__ENETC0_SI6_RB15BSR NETC__ENETC0_SI6.BDR_NUM[15].RBBSR.R /* Rx BDR 15 buffer size register */
#define NETC__ENETC0_SI6_RB15CIR NETC__ENETC0_SI6.BDR_NUM[15].RBCIR.R /* Rx BDR 15 consumer index register */
#define NETC__ENETC0_SI6_RB15BAR0 NETC__ENETC0_SI6.BDR_NUM[15].RBBAR0.R /* Rx BDR 15 base address register 0 */
#define NETC__ENETC0_SI6_RB15BAR1 NETC__ENETC0_SI6.BDR_NUM[15].RBBAR1.R /* Rx BDR 15 base address register 1 */
#define NETC__ENETC0_SI6_RB15PIR NETC__ENETC0_SI6.BDR_NUM[15].RBPIR.R /* Rx BDR 15 producer index register */
#define NETC__ENETC0_SI6_RB15LENR NETC__ENETC0_SI6.BDR_NUM[15].RBLENR.R /* Rx BDR 15 length register */
#define NETC__ENETC0_SI6_RB15DCR NETC__ENETC0_SI6.BDR_NUM[15].RBDCR.R /* Rx BDR 15 drop count register */
#define NETC__ENETC0_SI6_RB15IER NETC__ENETC0_SI6.BDR_NUM[15].RBIER.R /* Rx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI6_RB15IDR NETC__ENETC0_SI6.BDR_NUM[15].RBIDR.R /* Rx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI6_RB15ICR0 NETC__ENETC0_SI6.BDR_NUM[15].RBICR0.R /* Rx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB15ICR1 NETC__ENETC0_SI6.BDR_NUM[15].RBICR1.R /* Rx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB16MR NETC__ENETC0_SI6.BDR_NUM[16].TBMR.R /* Tx BDR 16 mode register */
#define NETC__ENETC0_SI6_TB16SR NETC__ENETC0_SI6.BDR_NUM[16].TBSR.R /* Tx BDR 16 status register */
#define NETC__ENETC0_SI6_TB16BAR0 NETC__ENETC0_SI6.BDR_NUM[16].TBBAR0.R /* Tx BDR 16 base address register 0 */
#define NETC__ENETC0_SI6_TB16BAR1 NETC__ENETC0_SI6.BDR_NUM[16].TBBAR1.R /* Tx BDR 16 base address register 1 */
#define NETC__ENETC0_SI6_TB16PIR NETC__ENETC0_SI6.BDR_NUM[16].TBPIR.R /* Tx BDR 16 producer index register */
#define NETC__ENETC0_SI6_TB16CIR NETC__ENETC0_SI6.BDR_NUM[16].TBCIR.R /* Tx BDR 16 consumer index register */
#define NETC__ENETC0_SI6_TB16LENR NETC__ENETC0_SI6.BDR_NUM[16].TBLENR.R /* Tx BDR 16 length register */
#define NETC__ENETC0_SI6_TB16IER NETC__ENETC0_SI6.BDR_NUM[16].TBIER.R /* Tx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI6_TB16IDR NETC__ENETC0_SI6.BDR_NUM[16].TBIDR.R /* Tx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI6_TB16ICR0 NETC__ENETC0_SI6.BDR_NUM[16].TBICR0.R /* Tx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB16ICR1 NETC__ENETC0_SI6.BDR_NUM[16].TBICR1.R /* Tx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB16MR NETC__ENETC0_SI6.BDR_NUM[16].RBMR.R /* Rx BDR 16 mode register */
#define NETC__ENETC0_SI6_RB16SR NETC__ENETC0_SI6.BDR_NUM[16].RBSR.R /* Rx BDR 16 status register */
#define NETC__ENETC0_SI6_RB16BSR NETC__ENETC0_SI6.BDR_NUM[16].RBBSR.R /* Rx BDR 16 buffer size register */
#define NETC__ENETC0_SI6_RB16CIR NETC__ENETC0_SI6.BDR_NUM[16].RBCIR.R /* Rx BDR 16 consumer index register */
#define NETC__ENETC0_SI6_RB16BAR0 NETC__ENETC0_SI6.BDR_NUM[16].RBBAR0.R /* Rx BDR 16 base address register 0 */
#define NETC__ENETC0_SI6_RB16BAR1 NETC__ENETC0_SI6.BDR_NUM[16].RBBAR1.R /* Rx BDR 16 base address register 1 */
#define NETC__ENETC0_SI6_RB16PIR NETC__ENETC0_SI6.BDR_NUM[16].RBPIR.R /* Rx BDR 16 producer index register */
#define NETC__ENETC0_SI6_RB16LENR NETC__ENETC0_SI6.BDR_NUM[16].RBLENR.R /* Rx BDR 16 length register */
#define NETC__ENETC0_SI6_RB16DCR NETC__ENETC0_SI6.BDR_NUM[16].RBDCR.R /* Rx BDR 16 drop count register */
#define NETC__ENETC0_SI6_RB16IER NETC__ENETC0_SI6.BDR_NUM[16].RBIER.R /* Rx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI6_RB16IDR NETC__ENETC0_SI6.BDR_NUM[16].RBIDR.R /* Rx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI6_RB16ICR0 NETC__ENETC0_SI6.BDR_NUM[16].RBICR0.R /* Rx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB16ICR1 NETC__ENETC0_SI6.BDR_NUM[16].RBICR1.R /* Rx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_TB17MR NETC__ENETC0_SI6.BDR_NUM[17].TBMR.R /* Tx BDR 17 mode register */
#define NETC__ENETC0_SI6_TB17SR NETC__ENETC0_SI6.BDR_NUM[17].TBSR.R /* Tx BDR 17 status register */
#define NETC__ENETC0_SI6_TB17BAR0 NETC__ENETC0_SI6.BDR_NUM[17].TBBAR0.R /* Tx BDR 17 base address register 0 */
#define NETC__ENETC0_SI6_TB17BAR1 NETC__ENETC0_SI6.BDR_NUM[17].TBBAR1.R /* Tx BDR 17 base address register 1 */
#define NETC__ENETC0_SI6_TB17PIR NETC__ENETC0_SI6.BDR_NUM[17].TBPIR.R /* Tx BDR 17 producer index register */
#define NETC__ENETC0_SI6_TB17CIR NETC__ENETC0_SI6.BDR_NUM[17].TBCIR.R /* Tx BDR 17 consumer index register */
#define NETC__ENETC0_SI6_TB17LENR NETC__ENETC0_SI6.BDR_NUM[17].TBLENR.R /* Tx BDR 17 length register */
#define NETC__ENETC0_SI6_TB17IER NETC__ENETC0_SI6.BDR_NUM[17].TBIER.R /* Tx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI6_TB17IDR NETC__ENETC0_SI6.BDR_NUM[17].TBIDR.R /* Tx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI6_TB17ICR0 NETC__ENETC0_SI6.BDR_NUM[17].TBICR0.R /* Tx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_TB17ICR1 NETC__ENETC0_SI6.BDR_NUM[17].TBICR1.R /* Tx BDR 17 interrupt coalescing register 1 */
#define NETC__ENETC0_SI6_RB17MR NETC__ENETC0_SI6.BDR_NUM[17].RBMR.R /* Rx BDR 17 mode register */
#define NETC__ENETC0_SI6_RB17SR NETC__ENETC0_SI6.BDR_NUM[17].RBSR.R /* Rx BDR 17 status register */
#define NETC__ENETC0_SI6_RB17BSR NETC__ENETC0_SI6.BDR_NUM[17].RBBSR.R /* Rx BDR 17 buffer size register */
#define NETC__ENETC0_SI6_RB17CIR NETC__ENETC0_SI6.BDR_NUM[17].RBCIR.R /* Rx BDR 17 consumer index register */
#define NETC__ENETC0_SI6_RB17BAR0 NETC__ENETC0_SI6.BDR_NUM[17].RBBAR0.R /* Rx BDR 17 base address register 0 */
#define NETC__ENETC0_SI6_RB17BAR1 NETC__ENETC0_SI6.BDR_NUM[17].RBBAR1.R /* Rx BDR 17 base address register 1 */
#define NETC__ENETC0_SI6_RB17PIR NETC__ENETC0_SI6.BDR_NUM[17].RBPIR.R /* Rx BDR 17 producer index register */
#define NETC__ENETC0_SI6_RB17LENR NETC__ENETC0_SI6.BDR_NUM[17].RBLENR.R /* Rx BDR 17 length register */
#define NETC__ENETC0_SI6_RB17DCR NETC__ENETC0_SI6.BDR_NUM[17].RBDCR.R /* Rx BDR 17 drop count register */
#define NETC__ENETC0_SI6_RB17IER NETC__ENETC0_SI6.BDR_NUM[17].RBIER.R /* Rx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI6_RB17IDR NETC__ENETC0_SI6.BDR_NUM[17].RBIDR.R /* Rx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI6_RB17ICR0 NETC__ENETC0_SI6.BDR_NUM[17].RBICR0.R /* Rx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI6_RB17ICR1 NETC__ENETC0_SI6.BDR_NUM[17].RBICR1.R /* Rx BDR 17 interrupt coalescing register 1 */

/* NETC_F3_SI7 */
#define NETC__ENETC0_SI7_SIMR NETC__ENETC0_SI7.SIMR.R      /* Station interface mode register */
#define NETC__ENETC0_SI7_SISR NETC__ENETC0_SI7.SISR.R      /* Station interface status register */
#define NETC__ENETC0_SI7_SICTR0 NETC__ENETC0_SI7.SICTR0.R  /* Station interface current time register 0 */
#define NETC__ENETC0_SI7_SICTR1 NETC__ENETC0_SI7.SICTR1.R  /* Station interface current time register 1 */
#define NETC__ENETC0_SI7_SIPCAPR0 NETC__ENETC0_SI7.SIPCAPR0.R /* Station interface port capability register 0 */
#define NETC__ENETC0_SI7_SIPCAPR1 NETC__ENETC0_SI7.SIPCAPR1.R /* Station interface port capability register 1 */
#define NETC__ENETC0_SI7_SIRBGCR NETC__ENETC0_SI7.SIRBGCR.R /* Station interface receive BDR group control register */
#define NETC__ENETC0_SI7_SIBCAR NETC__ENETC0_SI7.SIBCAR.R  /* Station interface buffer cache attribute register */
#define NETC__ENETC0_SI7_SICCAR NETC__ENETC0_SI7.SICCAR.R  /* Station interface command cache attribute register */
#define NETC__ENETC0_SI7_SIPMAR0 NETC__ENETC0_SI7.SIPMAR0.R /* Station interface primary MAC address register 0 */
#define NETC__ENETC0_SI7_SIPMAR1 NETC__ENETC0_SI7.SIPMAR1.R /* Station interface primary MAC address register 1 */
#define NETC__ENETC0_SI7_SICVLANR1 NETC__ENETC0_SI7.SICVLANR1.R /* Station interface custom VLAN register 1 */
#define NETC__ENETC0_SI7_SICVLANR2 NETC__ENETC0_SI7.SICVLANR2.R /* Station interface custom VLAN register 2 */
#define NETC__ENETC0_SI7_SIVLANIPVMR0 NETC__ENETC0_SI7.SIVLANIPVMR0.R /* Station interface VLAN to IPV mapping register 0 */
#define NETC__ENETC0_SI7_SIVLANIPVMR1 NETC__ENETC0_SI7.SIVLANIPVMR1.R /* Station interface VLAN to IPV mapping register 1 */
#define NETC__ENETC0_SI7_SIIPVBDRMR0 NETC__ENETC0_SI7.SIIPVBDRMR0.R /* Station interface IPV to ring mapping register */
#define NETC__ENETC0_SI7_VSIMSGSR NETC__ENETC0_SI7.VSI_A.VSIMSGSR.R /* Virtual station interface message send register */
#define NETC__ENETC0_SI7_VSIMSGRR NETC__ENETC0_SI7.VSI_A.VSIMSGRR.R /* Virtual station interface message receive register */
#define NETC__ENETC0_SI7_VSIMSGSNDAR0 NETC__ENETC0_SI7.VSI_A.VSIMSGSNDAR0.R /* Virtual station interface message send register 0 */
#define NETC__ENETC0_SI7_VSIMSGSNDAR1 NETC__ENETC0_SI7.VSI_A.VSIMSGSNDAR1.R /* Virtual station interface message send address register 1 */
#define NETC__ENETC0_SI7_SIROCT0 NETC__ENETC0_SI7.SIROCT0.R /* Station interface receive octets counter (ifInOctets) 0 */
#define NETC__ENETC0_SI7_SIROCT1 NETC__ENETC0_SI7.SIROCT1.R /* Station interface receive octets counter (ifInOctets) 1 */
#define NETC__ENETC0_SI7_SIRFRM0 NETC__ENETC0_SI7.SIRFRM0.R /* Station interface receive frame counter (aFrameReceivedOK) 0 */
#define NETC__ENETC0_SI7_SIRFRM1 NETC__ENETC0_SI7.SIRFRM1.R /* Station interface receive frame counter (aFrameReceivedOK) 1 */
#define NETC__ENETC0_SI7_SIRUCA0 NETC__ENETC0_SI7.SIRUCA0.R /* Station interface receive unicast frame counter (ifInUcastPkts) 0 */
#define NETC__ENETC0_SI7_SIRUCA1 NETC__ENETC0_SI7.SIRUCA1.R /* Station interface receive unicast frame counter (ifInUcastPkts) 1 */
#define NETC__ENETC0_SI7_SIRMCA0 NETC__ENETC0_SI7.SIRMCA0.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
#define NETC__ENETC0_SI7_SIRMCA1 NETC__ENETC0_SI7.SIRMCA1.R /* Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
#define NETC__ENETC0_SI7_SITOCT0 NETC__ENETC0_SI7.SITOCT0.R /* Station interface transmit octets counter (ifOutOctets) 0 */
#define NETC__ENETC0_SI7_SITOCT1 NETC__ENETC0_SI7.SITOCT1.R /* Station interface transmit octets counter (ifOutOctets) 1 */
#define NETC__ENETC0_SI7_SITFRM0 NETC__ENETC0_SI7.SITFRM0.R /* Station interface transmit frame counter (aFrameTransmittedOK) 0 */
#define NETC__ENETC0_SI7_SITFRM1 NETC__ENETC0_SI7.SITFRM1.R /* Station interface transmit frame counter (aFrameTransmittedOK) 1 */
#define NETC__ENETC0_SI7_SITUCA0 NETC__ENETC0_SI7.SITUCA0.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
#define NETC__ENETC0_SI7_SITUCA1 NETC__ENETC0_SI7.SITUCA1.R /* Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
#define NETC__ENETC0_SI7_SITMCA0 NETC__ENETC0_SI7.SITMCA0.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
#define NETC__ENETC0_SI7_SITMCA1 NETC__ENETC0_SI7.SITMCA1.R /* Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
#define NETC__ENETC0_SI7_SIBLPR0 NETC__ENETC0_SI7.SIBLPR[0].R /* Station interface boot loader parameter register 0 */
#define NETC__ENETC0_SI7_SIBLPR1 NETC__ENETC0_SI7.SIBLPR[1].R /* Station interface boot loader parameter register 1 */
#define NETC__ENETC0_SI7_SICBDRMR NETC__ENETC0_SI7.SICBDRMR.R /* Station interface command BDR mode register */
#define NETC__ENETC0_SI7_SICBDRSR NETC__ENETC0_SI7.SICBDRSR.R /* Station interface command BDR status register */
#define NETC__ENETC0_SI7_SICBDRBAR0 NETC__ENETC0_SI7.SICBDRBAR0.R /* Station interface command BDR base address register 0 */
#define NETC__ENETC0_SI7_SICBDRBAR1 NETC__ENETC0_SI7.SICBDRBAR1.R /* Station interface command BDR base address register 1 */
#define NETC__ENETC0_SI7_SICBDRPIR NETC__ENETC0_SI7.SICBDRPIR.R /* Station interface command BDR producer index register */
#define NETC__ENETC0_SI7_SICBDRCIR NETC__ENETC0_SI7.SICBDRCIR.R /* Station interface command BDR consumer index register */
#define NETC__ENETC0_SI7_SICBDRLENR NETC__ENETC0_SI7.SICBDRLENR.R /* Station interface command BDR length register */
#define NETC__ENETC0_SI7_SICBDRIER NETC__ENETC0_SI7.SICBDRIER.R /* Station interface command BDR interrupt enable register */
#define NETC__ENETC0_SI7_SICBDRIDR NETC__ENETC0_SI7.SICBDRIDR.R /* Station interface command BDR interrupt detect register */
#define NETC__ENETC0_SI7_SICAPR0 NETC__ENETC0_SI7.SICAPR0.R /* Station interface capability register 0 */
#define NETC__ENETC0_SI7_SICAPR1 NETC__ENETC0_SI7.SICAPR1.R /* Station interface capability register 1 */
#define NETC__ENETC0_SI7_SICAPR2 NETC__ENETC0_SI7.SICAPR2.R /* Station interface capability register 2 */
#define NETC__ENETC0_SI7_VSIIER NETC__ENETC0_SI7.VSI.VSIIER.R /* Virtual station interface interrupt enable register */
#define NETC__ENETC0_SI7_VSIIDR NETC__ENETC0_SI7.VSI.VSIIDR.R /* Virtual station interface interrupt detect register */
#define NETC__ENETC0_SI7_SITXIDR0 NETC__ENETC0_SI7.SITXIDR0.R /* Station interface transmit interrupt detect register 0 */
#define NETC__ENETC0_SI7_SITXIDR1 NETC__ENETC0_SI7.SITXIDR1.R /* Station interface transmit interrupt detect register 1 */
#define NETC__ENETC0_SI7_SIRXIDR0 NETC__ENETC0_SI7.SIRXIDR0.R /* Station interface receive interrupt detect register 0 */
#define NETC__ENETC0_SI7_SIRXIDR1 NETC__ENETC0_SI7.SIRXIDR1.R /* Station interface receive interrupt detect register 1 */
#define NETC__ENETC0_SI7_SIMSIVR NETC__ENETC0_SI7.SIMSIVR.R /* Station interface MSI-X vector register */
#define NETC__ENETC0_SI7_SICMSIVR NETC__ENETC0_SI7.SICMSIVR.R /* Station interface command MSI-X vector register */
#define NETC__ENETC0_SI7_SITMRIER NETC__ENETC0_SI7.SITMRIER.R /* Station interface timer interrupt enable register */
#define NETC__ENETC0_SI7_SITMRIDR NETC__ENETC0_SI7.SITMRIDR.R /* Station interface timer interrupt detect register */
#define NETC__ENETC0_SI7_SITMRMSIVR NETC__ENETC0_SI7.SITMRMSIVR.R /* Station interface timer MSI-X vector register */
#define NETC__ENETC0_SI7_SIMSITR0VR NETC__ENETC0_SI7.SIMSITRVR[0].R /* Station interface MSI-X transmit ring 0 vector register */
#define NETC__ENETC0_SI7_SIMSITR1VR NETC__ENETC0_SI7.SIMSITRVR[1].R /* Station interface MSI-X transmit ring 1 vector register */
#define NETC__ENETC0_SI7_SIMSITR2VR NETC__ENETC0_SI7.SIMSITRVR[2].R /* Station interface MSI-X transmit ring 2 vector register */
#define NETC__ENETC0_SI7_SIMSITR3VR NETC__ENETC0_SI7.SIMSITRVR[3].R /* Station interface MSI-X transmit ring 3 vector register */
#define NETC__ENETC0_SI7_SIMSITR4VR NETC__ENETC0_SI7.SIMSITRVR[4].R /* Station interface MSI-X transmit ring 4 vector register */
#define NETC__ENETC0_SI7_SIMSITR5VR NETC__ENETC0_SI7.SIMSITRVR[5].R /* Station interface MSI-X transmit ring 5 vector register */
#define NETC__ENETC0_SI7_SIMSITR6VR NETC__ENETC0_SI7.SIMSITRVR[6].R /* Station interface MSI-X transmit ring 6 vector register */
#define NETC__ENETC0_SI7_SIMSITR7VR NETC__ENETC0_SI7.SIMSITRVR[7].R /* Station interface MSI-X transmit ring 7 vector register */
#define NETC__ENETC0_SI7_SIMSITR8VR NETC__ENETC0_SI7.SIMSITRVR[8].R /* Station interface MSI-X transmit ring 8 vector register */
#define NETC__ENETC0_SI7_SIMSITR9VR NETC__ENETC0_SI7.SIMSITRVR[9].R /* Station interface MSI-X transmit ring 9 vector register */
#define NETC__ENETC0_SI7_SIMSITR10VR NETC__ENETC0_SI7.SIMSITRVR[10].R /* Station interface MSI-X transmit ring 10 vector register */
#define NETC__ENETC0_SI7_SIMSITR11VR NETC__ENETC0_SI7.SIMSITRVR[11].R /* Station interface MSI-X transmit ring 11 vector register */
#define NETC__ENETC0_SI7_SIMSITR12VR NETC__ENETC0_SI7.SIMSITRVR[12].R /* Station interface MSI-X transmit ring 12 vector register */
#define NETC__ENETC0_SI7_SIMSITR13VR NETC__ENETC0_SI7.SIMSITRVR[13].R /* Station interface MSI-X transmit ring 13 vector register */
#define NETC__ENETC0_SI7_SIMSITR14VR NETC__ENETC0_SI7.SIMSITRVR[14].R /* Station interface MSI-X transmit ring 14 vector register */
#define NETC__ENETC0_SI7_SIMSITR15VR NETC__ENETC0_SI7.SIMSITRVR[15].R /* Station interface MSI-X transmit ring 15 vector register */
#define NETC__ENETC0_SI7_SIMSITR16VR NETC__ENETC0_SI7.SIMSITRVR[16].R /* Station interface MSI-X transmit ring 16 vector register */
#define NETC__ENETC0_SI7_SIMSITR17VR NETC__ENETC0_SI7.SIMSITRVR[17].R /* Station interface MSI-X transmit ring 17 vector register */
#define NETC__ENETC0_SI7_SIMSIRR0VR NETC__ENETC0_SI7.SIMSIRRVR[0].R /* Station interface MSI-X receive ring 0 vector register */
#define NETC__ENETC0_SI7_SIMSIRR1VR NETC__ENETC0_SI7.SIMSIRRVR[1].R /* Station interface MSI-X receive ring 1 vector register */
#define NETC__ENETC0_SI7_SIMSIRR2VR NETC__ENETC0_SI7.SIMSIRRVR[2].R /* Station interface MSI-X receive ring 2 vector register */
#define NETC__ENETC0_SI7_SIMSIRR3VR NETC__ENETC0_SI7.SIMSIRRVR[3].R /* Station interface MSI-X receive ring 3 vector register */
#define NETC__ENETC0_SI7_SIMSIRR4VR NETC__ENETC0_SI7.SIMSIRRVR[4].R /* Station interface MSI-X receive ring 4 vector register */
#define NETC__ENETC0_SI7_SIMSIRR5VR NETC__ENETC0_SI7.SIMSIRRVR[5].R /* Station interface MSI-X receive ring 5 vector register */
#define NETC__ENETC0_SI7_SIMSIRR6VR NETC__ENETC0_SI7.SIMSIRRVR[6].R /* Station interface MSI-X receive ring 6 vector register */
#define NETC__ENETC0_SI7_SIMSIRR7VR NETC__ENETC0_SI7.SIMSIRRVR[7].R /* Station interface MSI-X receive ring 7 vector register */
#define NETC__ENETC0_SI7_SIMSIRR8VR NETC__ENETC0_SI7.SIMSIRRVR[8].R /* Station interface MSI-X receive ring 8 vector register */
#define NETC__ENETC0_SI7_SIMSIRR9VR NETC__ENETC0_SI7.SIMSIRRVR[9].R /* Station interface MSI-X receive ring 9 vector register */
#define NETC__ENETC0_SI7_SIMSIRR10VR NETC__ENETC0_SI7.SIMSIRRVR[10].R /* Station interface MSI-X receive ring 10 vector register */
#define NETC__ENETC0_SI7_SIMSIRR11VR NETC__ENETC0_SI7.SIMSIRRVR[11].R /* Station interface MSI-X receive ring 11 vector register */
#define NETC__ENETC0_SI7_SIMSIRR12VR NETC__ENETC0_SI7.SIMSIRRVR[12].R /* Station interface MSI-X receive ring 12 vector register */
#define NETC__ENETC0_SI7_SIMSIRR13VR NETC__ENETC0_SI7.SIMSIRRVR[13].R /* Station interface MSI-X receive ring 13 vector register */
#define NETC__ENETC0_SI7_SIMSIRR14VR NETC__ENETC0_SI7.SIMSIRRVR[14].R /* Station interface MSI-X receive ring 14 vector register */
#define NETC__ENETC0_SI7_SIMSIRR15VR NETC__ENETC0_SI7.SIMSIRRVR[15].R /* Station interface MSI-X receive ring 15 vector register */
#define NETC__ENETC0_SI7_SIMSIRR16VR NETC__ENETC0_SI7.SIMSIRRVR[16].R /* Station interface MSI-X receive ring 16 vector register */
#define NETC__ENETC0_SI7_SIMSIRR17VR NETC__ENETC0_SI7.SIMSIRRVR[17].R /* Station interface MSI-X receive ring 17 vector register */
#define NETC__ENETC0_SI7_SICMECR NETC__ENETC0_SI7.SICMECR.R /* Station interface correctable memory error configuration register */
#define NETC__ENETC0_SI7_SICMESR NETC__ENETC0_SI7.SICMESR.R /* Station interface correctable memory error status register */
#define NETC__ENETC0_SI7_SICMECTR NETC__ENETC0_SI7.SICMECTR.R /* Station interface correctable memory error count register */
#define NETC__ENETC0_SI7_SIUPECR NETC__ENETC0_SI7.SIUPECR.R /* Station interface uncorrectable programming error configuration register */
#define NETC__ENETC0_SI7_SIUPESR NETC__ENETC0_SI7.SIUPESR.R /* Station interface uncorrectable programming error status register */
#define NETC__ENETC0_SI7_SIUPECTR NETC__ENETC0_SI7.SIUPECTR.R /* Station interface uncorrectable programming error count register */
#define NETC__ENETC0_SI7_SIUNSBECR NETC__ENETC0_SI7.SIUNSBECR.R /* Station interface uncorrectable non-fatal system bus error configuration register */
#define NETC__ENETC0_SI7_SIUNSBESR NETC__ENETC0_SI7.SIUNSBESR.R /* Station interface uncorrectable non-fatal system bus error status register */
#define NETC__ENETC0_SI7_SIUNSBECTR NETC__ENETC0_SI7.SIUNSBECTR.R /* Station interface uncorrectable non-fatal system bus error count register */
#define NETC__ENETC0_SI7_SIUFSBECR NETC__ENETC0_SI7.SIUFSBECR.R /* Station interface uncorrectable fatal system bus error configuration register */
#define NETC__ENETC0_SI7_SIUFSBESR NETC__ENETC0_SI7.SIUFSBESR.R /* Station interface uncorrectable fatal system bus error status register */
#define NETC__ENETC0_SI7_SIUNMECR NETC__ENETC0_SI7.SIUNMECR.R /* Station interface uncorrectable non-fatal memory error configuration register */
#define NETC__ENETC0_SI7_SIUNMESR0 NETC__ENETC0_SI7.SIUNMESR0.R /* Station interface uncorrectable non-fatal memory error status register 0 */
#define NETC__ENETC0_SI7_SIUNMESR1 NETC__ENETC0_SI7.SIUNMESR1.R /* Station interface uncorrectable non-fatal memory error status register 1 */
#define NETC__ENETC0_SI7_SIUNMECTR NETC__ENETC0_SI7.SIUNMECTR.R /* Station interface uncorrectable non-fatal memory error count register */
#define NETC__ENETC0_SI7_SIUFMECR NETC__ENETC0_SI7.SIUFMECR.R /* Station interface uncorrectable fatal memory error configuration register */
#define NETC__ENETC0_SI7_SIUFMESR0 NETC__ENETC0_SI7.SIUFMESR0.R /* Station interface uncorrectable fatal memory error status register 0 */
#define NETC__ENETC0_SI7_SIUFMESR1 NETC__ENETC0_SI7.SIUFMESR1.R /* Station interface uncorrectable fatal memory error status register 1 */
#define NETC__ENETC0_SI7_SIUNIECR NETC__ENETC0_SI7.SIUNIECR.R /* Station interface uncorrectable non-fatal integrity error configuration register */
#define NETC__ENETC0_SI7_SIUNIESR NETC__ENETC0_SI7.SIUNIESR.R /* Station interface uncorrectable non-fatal integrity error status register */
#define NETC__ENETC0_SI7_SIUNIECTR NETC__ENETC0_SI7.SIUNIECTR.R /* Station interface uncorrectable non-fatal integrity error count register */
#define NETC__ENETC0_SI7_SIUFIECR NETC__ENETC0_SI7.SIUFIECR.R /* Station interface uncorrectable fatal integrity error configuration register */
#define NETC__ENETC0_SI7_SIUFIESR NETC__ENETC0_SI7.SIUFIESR.R /* Station interface uncorrectable fatal integrity error status register */
#define NETC__ENETC0_SI7_SIMAFTCAPR NETC__ENETC0_SI7.SIMAFTCAPR.R /* Station interface MAC address filter table capability register */
#define NETC__ENETC0_SI7_SIVFTCAPR NETC__ENETC0_SI7.SIVFTCAPR.R /* Station interface VLAN filter table capability register */
#define NETC__ENETC0_SI7_SIRFSCAPR NETC__ENETC0_SI7.SIRFSCAPR.R /* Station interface RFS capability register */
#define NETC__ENETC0_SI7_TB0MR NETC__ENETC0_SI7.BDR_NUM[0].TBMR.R /* Tx BDR 0 mode register */
#define NETC__ENETC0_SI7_TB0SR NETC__ENETC0_SI7.BDR_NUM[0].TBSR.R /* Tx BDR 0 status register */
#define NETC__ENETC0_SI7_TB0BAR0 NETC__ENETC0_SI7.BDR_NUM[0].TBBAR0.R /* Tx BDR 0 base address register 0 */
#define NETC__ENETC0_SI7_TB0BAR1 NETC__ENETC0_SI7.BDR_NUM[0].TBBAR1.R /* Tx BDR 0 base address register 1 */
#define NETC__ENETC0_SI7_TB0PIR NETC__ENETC0_SI7.BDR_NUM[0].TBPIR.R /* Tx BDR 0 producer index register */
#define NETC__ENETC0_SI7_TB0CIR NETC__ENETC0_SI7.BDR_NUM[0].TBCIR.R /* Tx BDR 0 consumer index register */
#define NETC__ENETC0_SI7_TB0LENR NETC__ENETC0_SI7.BDR_NUM[0].TBLENR.R /* Tx BDR 0 length register */
#define NETC__ENETC0_SI7_TB0IER NETC__ENETC0_SI7.BDR_NUM[0].TBIER.R /* Tx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI7_TB0IDR NETC__ENETC0_SI7.BDR_NUM[0].TBIDR.R /* Tx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI7_TB0ICR0 NETC__ENETC0_SI7.BDR_NUM[0].TBICR0.R /* Tx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB0ICR1 NETC__ENETC0_SI7.BDR_NUM[0].TBICR1.R /* Tx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB0MR NETC__ENETC0_SI7.BDR_NUM[0].RBMR.R /* Rx BDR 0 mode register */
#define NETC__ENETC0_SI7_RB0SR NETC__ENETC0_SI7.BDR_NUM[0].RBSR.R /* Rx BDR 0 status register */
#define NETC__ENETC0_SI7_RB0BSR NETC__ENETC0_SI7.BDR_NUM[0].RBBSR.R /* Rx BDR 0 buffer size register */
#define NETC__ENETC0_SI7_RB0CIR NETC__ENETC0_SI7.BDR_NUM[0].RBCIR.R /* Rx BDR 0 consumer index register */
#define NETC__ENETC0_SI7_RB0BAR0 NETC__ENETC0_SI7.BDR_NUM[0].RBBAR0.R /* Rx BDR 0 base address register 0 */
#define NETC__ENETC0_SI7_RB0BAR1 NETC__ENETC0_SI7.BDR_NUM[0].RBBAR1.R /* Rx BDR 0 base address register 1 */
#define NETC__ENETC0_SI7_RB0PIR NETC__ENETC0_SI7.BDR_NUM[0].RBPIR.R /* Rx BDR 0 producer index register */
#define NETC__ENETC0_SI7_RB0LENR NETC__ENETC0_SI7.BDR_NUM[0].RBLENR.R /* Rx BDR 0 length register */
#define NETC__ENETC0_SI7_RB0DCR NETC__ENETC0_SI7.BDR_NUM[0].RBDCR.R /* Rx BDR 0 drop count register */
#define NETC__ENETC0_SI7_RB0IER NETC__ENETC0_SI7.BDR_NUM[0].RBIER.R /* Rx BDR 0 interrupt enable register */
#define NETC__ENETC0_SI7_RB0IDR NETC__ENETC0_SI7.BDR_NUM[0].RBIDR.R /* Rx BDR 0 interrupt detect register */
#define NETC__ENETC0_SI7_RB0ICR0 NETC__ENETC0_SI7.BDR_NUM[0].RBICR0.R /* Rx BDR 0 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB0ICR1 NETC__ENETC0_SI7.BDR_NUM[0].RBICR1.R /* Rx BDR 0 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB1MR NETC__ENETC0_SI7.BDR_NUM[1].TBMR.R /* Tx BDR 1 mode register */
#define NETC__ENETC0_SI7_TB1SR NETC__ENETC0_SI7.BDR_NUM[1].TBSR.R /* Tx BDR 1 status register */
#define NETC__ENETC0_SI7_TB1BAR0 NETC__ENETC0_SI7.BDR_NUM[1].TBBAR0.R /* Tx BDR 1 base address register 0 */
#define NETC__ENETC0_SI7_TB1BAR1 NETC__ENETC0_SI7.BDR_NUM[1].TBBAR1.R /* Tx BDR 1 base address register 1 */
#define NETC__ENETC0_SI7_TB1PIR NETC__ENETC0_SI7.BDR_NUM[1].TBPIR.R /* Tx BDR 1 producer index register */
#define NETC__ENETC0_SI7_TB1CIR NETC__ENETC0_SI7.BDR_NUM[1].TBCIR.R /* Tx BDR 1 consumer index register */
#define NETC__ENETC0_SI7_TB1LENR NETC__ENETC0_SI7.BDR_NUM[1].TBLENR.R /* Tx BDR 1 length register */
#define NETC__ENETC0_SI7_TB1IER NETC__ENETC0_SI7.BDR_NUM[1].TBIER.R /* Tx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI7_TB1IDR NETC__ENETC0_SI7.BDR_NUM[1].TBIDR.R /* Tx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI7_TB1ICR0 NETC__ENETC0_SI7.BDR_NUM[1].TBICR0.R /* Tx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB1ICR1 NETC__ENETC0_SI7.BDR_NUM[1].TBICR1.R /* Tx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB1MR NETC__ENETC0_SI7.BDR_NUM[1].RBMR.R /* Rx BDR 1 mode register */
#define NETC__ENETC0_SI7_RB1SR NETC__ENETC0_SI7.BDR_NUM[1].RBSR.R /* Rx BDR 1 status register */
#define NETC__ENETC0_SI7_RB1BSR NETC__ENETC0_SI7.BDR_NUM[1].RBBSR.R /* Rx BDR 1 buffer size register */
#define NETC__ENETC0_SI7_RB1CIR NETC__ENETC0_SI7.BDR_NUM[1].RBCIR.R /* Rx BDR 1 consumer index register */
#define NETC__ENETC0_SI7_RB1BAR0 NETC__ENETC0_SI7.BDR_NUM[1].RBBAR0.R /* Rx BDR 1 base address register 0 */
#define NETC__ENETC0_SI7_RB1BAR1 NETC__ENETC0_SI7.BDR_NUM[1].RBBAR1.R /* Rx BDR 1 base address register 1 */
#define NETC__ENETC0_SI7_RB1PIR NETC__ENETC0_SI7.BDR_NUM[1].RBPIR.R /* Rx BDR 1 producer index register */
#define NETC__ENETC0_SI7_RB1LENR NETC__ENETC0_SI7.BDR_NUM[1].RBLENR.R /* Rx BDR 1 length register */
#define NETC__ENETC0_SI7_RB1DCR NETC__ENETC0_SI7.BDR_NUM[1].RBDCR.R /* Rx BDR 1 drop count register */
#define NETC__ENETC0_SI7_RB1IER NETC__ENETC0_SI7.BDR_NUM[1].RBIER.R /* Rx BDR 1 interrupt enable register */
#define NETC__ENETC0_SI7_RB1IDR NETC__ENETC0_SI7.BDR_NUM[1].RBIDR.R /* Rx BDR 1 interrupt detect register */
#define NETC__ENETC0_SI7_RB1ICR0 NETC__ENETC0_SI7.BDR_NUM[1].RBICR0.R /* Rx BDR 1 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB1ICR1 NETC__ENETC0_SI7.BDR_NUM[1].RBICR1.R /* Rx BDR 1 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB2MR NETC__ENETC0_SI7.BDR_NUM[2].TBMR.R /* Tx BDR 2 mode register */
#define NETC__ENETC0_SI7_TB2SR NETC__ENETC0_SI7.BDR_NUM[2].TBSR.R /* Tx BDR 2 status register */
#define NETC__ENETC0_SI7_TB2BAR0 NETC__ENETC0_SI7.BDR_NUM[2].TBBAR0.R /* Tx BDR 2 base address register 0 */
#define NETC__ENETC0_SI7_TB2BAR1 NETC__ENETC0_SI7.BDR_NUM[2].TBBAR1.R /* Tx BDR 2 base address register 1 */
#define NETC__ENETC0_SI7_TB2PIR NETC__ENETC0_SI7.BDR_NUM[2].TBPIR.R /* Tx BDR 2 producer index register */
#define NETC__ENETC0_SI7_TB2CIR NETC__ENETC0_SI7.BDR_NUM[2].TBCIR.R /* Tx BDR 2 consumer index register */
#define NETC__ENETC0_SI7_TB2LENR NETC__ENETC0_SI7.BDR_NUM[2].TBLENR.R /* Tx BDR 2 length register */
#define NETC__ENETC0_SI7_TB2IER NETC__ENETC0_SI7.BDR_NUM[2].TBIER.R /* Tx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI7_TB2IDR NETC__ENETC0_SI7.BDR_NUM[2].TBIDR.R /* Tx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI7_TB2ICR0 NETC__ENETC0_SI7.BDR_NUM[2].TBICR0.R /* Tx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB2ICR1 NETC__ENETC0_SI7.BDR_NUM[2].TBICR1.R /* Tx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB2MR NETC__ENETC0_SI7.BDR_NUM[2].RBMR.R /* Rx BDR 2 mode register */
#define NETC__ENETC0_SI7_RB2SR NETC__ENETC0_SI7.BDR_NUM[2].RBSR.R /* Rx BDR 2 status register */
#define NETC__ENETC0_SI7_RB2BSR NETC__ENETC0_SI7.BDR_NUM[2].RBBSR.R /* Rx BDR 2 buffer size register */
#define NETC__ENETC0_SI7_RB2CIR NETC__ENETC0_SI7.BDR_NUM[2].RBCIR.R /* Rx BDR 2 consumer index register */
#define NETC__ENETC0_SI7_RB2BAR0 NETC__ENETC0_SI7.BDR_NUM[2].RBBAR0.R /* Rx BDR 2 base address register 0 */
#define NETC__ENETC0_SI7_RB2BAR1 NETC__ENETC0_SI7.BDR_NUM[2].RBBAR1.R /* Rx BDR 2 base address register 1 */
#define NETC__ENETC0_SI7_RB2PIR NETC__ENETC0_SI7.BDR_NUM[2].RBPIR.R /* Rx BDR 2 producer index register */
#define NETC__ENETC0_SI7_RB2LENR NETC__ENETC0_SI7.BDR_NUM[2].RBLENR.R /* Rx BDR 2 length register */
#define NETC__ENETC0_SI7_RB2DCR NETC__ENETC0_SI7.BDR_NUM[2].RBDCR.R /* Rx BDR 2 drop count register */
#define NETC__ENETC0_SI7_RB2IER NETC__ENETC0_SI7.BDR_NUM[2].RBIER.R /* Rx BDR 2 interrupt enable register */
#define NETC__ENETC0_SI7_RB2IDR NETC__ENETC0_SI7.BDR_NUM[2].RBIDR.R /* Rx BDR 2 interrupt detect register */
#define NETC__ENETC0_SI7_RB2ICR0 NETC__ENETC0_SI7.BDR_NUM[2].RBICR0.R /* Rx BDR 2 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB2ICR1 NETC__ENETC0_SI7.BDR_NUM[2].RBICR1.R /* Rx BDR 2 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB3MR NETC__ENETC0_SI7.BDR_NUM[3].TBMR.R /* Tx BDR 3 mode register */
#define NETC__ENETC0_SI7_TB3SR NETC__ENETC0_SI7.BDR_NUM[3].TBSR.R /* Tx BDR 3 status register */
#define NETC__ENETC0_SI7_TB3BAR0 NETC__ENETC0_SI7.BDR_NUM[3].TBBAR0.R /* Tx BDR 3 base address register 0 */
#define NETC__ENETC0_SI7_TB3BAR1 NETC__ENETC0_SI7.BDR_NUM[3].TBBAR1.R /* Tx BDR 3 base address register 1 */
#define NETC__ENETC0_SI7_TB3PIR NETC__ENETC0_SI7.BDR_NUM[3].TBPIR.R /* Tx BDR 3 producer index register */
#define NETC__ENETC0_SI7_TB3CIR NETC__ENETC0_SI7.BDR_NUM[3].TBCIR.R /* Tx BDR 3 consumer index register */
#define NETC__ENETC0_SI7_TB3LENR NETC__ENETC0_SI7.BDR_NUM[3].TBLENR.R /* Tx BDR 3 length register */
#define NETC__ENETC0_SI7_TB3IER NETC__ENETC0_SI7.BDR_NUM[3].TBIER.R /* Tx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI7_TB3IDR NETC__ENETC0_SI7.BDR_NUM[3].TBIDR.R /* Tx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI7_TB3ICR0 NETC__ENETC0_SI7.BDR_NUM[3].TBICR0.R /* Tx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB3ICR1 NETC__ENETC0_SI7.BDR_NUM[3].TBICR1.R /* Tx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB3MR NETC__ENETC0_SI7.BDR_NUM[3].RBMR.R /* Rx BDR 3 mode register */
#define NETC__ENETC0_SI7_RB3SR NETC__ENETC0_SI7.BDR_NUM[3].RBSR.R /* Rx BDR 3 status register */
#define NETC__ENETC0_SI7_RB3BSR NETC__ENETC0_SI7.BDR_NUM[3].RBBSR.R /* Rx BDR 3 buffer size register */
#define NETC__ENETC0_SI7_RB3CIR NETC__ENETC0_SI7.BDR_NUM[3].RBCIR.R /* Rx BDR 3 consumer index register */
#define NETC__ENETC0_SI7_RB3BAR0 NETC__ENETC0_SI7.BDR_NUM[3].RBBAR0.R /* Rx BDR 3 base address register 0 */
#define NETC__ENETC0_SI7_RB3BAR1 NETC__ENETC0_SI7.BDR_NUM[3].RBBAR1.R /* Rx BDR 3 base address register 1 */
#define NETC__ENETC0_SI7_RB3PIR NETC__ENETC0_SI7.BDR_NUM[3].RBPIR.R /* Rx BDR 3 producer index register */
#define NETC__ENETC0_SI7_RB3LENR NETC__ENETC0_SI7.BDR_NUM[3].RBLENR.R /* Rx BDR 3 length register */
#define NETC__ENETC0_SI7_RB3DCR NETC__ENETC0_SI7.BDR_NUM[3].RBDCR.R /* Rx BDR 3 drop count register */
#define NETC__ENETC0_SI7_RB3IER NETC__ENETC0_SI7.BDR_NUM[3].RBIER.R /* Rx BDR 3 interrupt enable register */
#define NETC__ENETC0_SI7_RB3IDR NETC__ENETC0_SI7.BDR_NUM[3].RBIDR.R /* Rx BDR 3 interrupt detect register */
#define NETC__ENETC0_SI7_RB3ICR0 NETC__ENETC0_SI7.BDR_NUM[3].RBICR0.R /* Rx BDR 3 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB3ICR1 NETC__ENETC0_SI7.BDR_NUM[3].RBICR1.R /* Rx BDR 3 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB4MR NETC__ENETC0_SI7.BDR_NUM[4].TBMR.R /* Tx BDR 4 mode register */
#define NETC__ENETC0_SI7_TB4SR NETC__ENETC0_SI7.BDR_NUM[4].TBSR.R /* Tx BDR 4 status register */
#define NETC__ENETC0_SI7_TB4BAR0 NETC__ENETC0_SI7.BDR_NUM[4].TBBAR0.R /* Tx BDR 4 base address register 0 */
#define NETC__ENETC0_SI7_TB4BAR1 NETC__ENETC0_SI7.BDR_NUM[4].TBBAR1.R /* Tx BDR 4 base address register 1 */
#define NETC__ENETC0_SI7_TB4PIR NETC__ENETC0_SI7.BDR_NUM[4].TBPIR.R /* Tx BDR 4 producer index register */
#define NETC__ENETC0_SI7_TB4CIR NETC__ENETC0_SI7.BDR_NUM[4].TBCIR.R /* Tx BDR 4 consumer index register */
#define NETC__ENETC0_SI7_TB4LENR NETC__ENETC0_SI7.BDR_NUM[4].TBLENR.R /* Tx BDR 4 length register */
#define NETC__ENETC0_SI7_TB4IER NETC__ENETC0_SI7.BDR_NUM[4].TBIER.R /* Tx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI7_TB4IDR NETC__ENETC0_SI7.BDR_NUM[4].TBIDR.R /* Tx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI7_TB4ICR0 NETC__ENETC0_SI7.BDR_NUM[4].TBICR0.R /* Tx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB4ICR1 NETC__ENETC0_SI7.BDR_NUM[4].TBICR1.R /* Tx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB4MR NETC__ENETC0_SI7.BDR_NUM[4].RBMR.R /* Rx BDR 4 mode register */
#define NETC__ENETC0_SI7_RB4SR NETC__ENETC0_SI7.BDR_NUM[4].RBSR.R /* Rx BDR 4 status register */
#define NETC__ENETC0_SI7_RB4BSR NETC__ENETC0_SI7.BDR_NUM[4].RBBSR.R /* Rx BDR 4 buffer size register */
#define NETC__ENETC0_SI7_RB4CIR NETC__ENETC0_SI7.BDR_NUM[4].RBCIR.R /* Rx BDR 4 consumer index register */
#define NETC__ENETC0_SI7_RB4BAR0 NETC__ENETC0_SI7.BDR_NUM[4].RBBAR0.R /* Rx BDR 4 base address register 0 */
#define NETC__ENETC0_SI7_RB4BAR1 NETC__ENETC0_SI7.BDR_NUM[4].RBBAR1.R /* Rx BDR 4 base address register 1 */
#define NETC__ENETC0_SI7_RB4PIR NETC__ENETC0_SI7.BDR_NUM[4].RBPIR.R /* Rx BDR 4 producer index register */
#define NETC__ENETC0_SI7_RB4LENR NETC__ENETC0_SI7.BDR_NUM[4].RBLENR.R /* Rx BDR 4 length register */
#define NETC__ENETC0_SI7_RB4DCR NETC__ENETC0_SI7.BDR_NUM[4].RBDCR.R /* Rx BDR 4 drop count register */
#define NETC__ENETC0_SI7_RB4IER NETC__ENETC0_SI7.BDR_NUM[4].RBIER.R /* Rx BDR 4 interrupt enable register */
#define NETC__ENETC0_SI7_RB4IDR NETC__ENETC0_SI7.BDR_NUM[4].RBIDR.R /* Rx BDR 4 interrupt detect register */
#define NETC__ENETC0_SI7_RB4ICR0 NETC__ENETC0_SI7.BDR_NUM[4].RBICR0.R /* Rx BDR 4 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB4ICR1 NETC__ENETC0_SI7.BDR_NUM[4].RBICR1.R /* Rx BDR 4 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB5MR NETC__ENETC0_SI7.BDR_NUM[5].TBMR.R /* Tx BDR 5 mode register */
#define NETC__ENETC0_SI7_TB5SR NETC__ENETC0_SI7.BDR_NUM[5].TBSR.R /* Tx BDR 5 status register */
#define NETC__ENETC0_SI7_TB5BAR0 NETC__ENETC0_SI7.BDR_NUM[5].TBBAR0.R /* Tx BDR 5 base address register 0 */
#define NETC__ENETC0_SI7_TB5BAR1 NETC__ENETC0_SI7.BDR_NUM[5].TBBAR1.R /* Tx BDR 5 base address register 1 */
#define NETC__ENETC0_SI7_TB5PIR NETC__ENETC0_SI7.BDR_NUM[5].TBPIR.R /* Tx BDR 5 producer index register */
#define NETC__ENETC0_SI7_TB5CIR NETC__ENETC0_SI7.BDR_NUM[5].TBCIR.R /* Tx BDR 5 consumer index register */
#define NETC__ENETC0_SI7_TB5LENR NETC__ENETC0_SI7.BDR_NUM[5].TBLENR.R /* Tx BDR 5 length register */
#define NETC__ENETC0_SI7_TB5IER NETC__ENETC0_SI7.BDR_NUM[5].TBIER.R /* Tx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI7_TB5IDR NETC__ENETC0_SI7.BDR_NUM[5].TBIDR.R /* Tx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI7_TB5ICR0 NETC__ENETC0_SI7.BDR_NUM[5].TBICR0.R /* Tx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB5ICR1 NETC__ENETC0_SI7.BDR_NUM[5].TBICR1.R /* Tx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB5MR NETC__ENETC0_SI7.BDR_NUM[5].RBMR.R /* Rx BDR 5 mode register */
#define NETC__ENETC0_SI7_RB5SR NETC__ENETC0_SI7.BDR_NUM[5].RBSR.R /* Rx BDR 5 status register */
#define NETC__ENETC0_SI7_RB5BSR NETC__ENETC0_SI7.BDR_NUM[5].RBBSR.R /* Rx BDR 5 buffer size register */
#define NETC__ENETC0_SI7_RB5CIR NETC__ENETC0_SI7.BDR_NUM[5].RBCIR.R /* Rx BDR 5 consumer index register */
#define NETC__ENETC0_SI7_RB5BAR0 NETC__ENETC0_SI7.BDR_NUM[5].RBBAR0.R /* Rx BDR 5 base address register 0 */
#define NETC__ENETC0_SI7_RB5BAR1 NETC__ENETC0_SI7.BDR_NUM[5].RBBAR1.R /* Rx BDR 5 base address register 1 */
#define NETC__ENETC0_SI7_RB5PIR NETC__ENETC0_SI7.BDR_NUM[5].RBPIR.R /* Rx BDR 5 producer index register */
#define NETC__ENETC0_SI7_RB5LENR NETC__ENETC0_SI7.BDR_NUM[5].RBLENR.R /* Rx BDR 5 length register */
#define NETC__ENETC0_SI7_RB5DCR NETC__ENETC0_SI7.BDR_NUM[5].RBDCR.R /* Rx BDR 5 drop count register */
#define NETC__ENETC0_SI7_RB5IER NETC__ENETC0_SI7.BDR_NUM[5].RBIER.R /* Rx BDR 5 interrupt enable register */
#define NETC__ENETC0_SI7_RB5IDR NETC__ENETC0_SI7.BDR_NUM[5].RBIDR.R /* Rx BDR 5 interrupt detect register */
#define NETC__ENETC0_SI7_RB5ICR0 NETC__ENETC0_SI7.BDR_NUM[5].RBICR0.R /* Rx BDR 5 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB5ICR1 NETC__ENETC0_SI7.BDR_NUM[5].RBICR1.R /* Rx BDR 5 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB6MR NETC__ENETC0_SI7.BDR_NUM[6].TBMR.R /* Tx BDR 6 mode register */
#define NETC__ENETC0_SI7_TB6SR NETC__ENETC0_SI7.BDR_NUM[6].TBSR.R /* Tx BDR 6 status register */
#define NETC__ENETC0_SI7_TB6BAR0 NETC__ENETC0_SI7.BDR_NUM[6].TBBAR0.R /* Tx BDR 6 base address register 0 */
#define NETC__ENETC0_SI7_TB6BAR1 NETC__ENETC0_SI7.BDR_NUM[6].TBBAR1.R /* Tx BDR 6 base address register 1 */
#define NETC__ENETC0_SI7_TB6PIR NETC__ENETC0_SI7.BDR_NUM[6].TBPIR.R /* Tx BDR 6 producer index register */
#define NETC__ENETC0_SI7_TB6CIR NETC__ENETC0_SI7.BDR_NUM[6].TBCIR.R /* Tx BDR 6 consumer index register */
#define NETC__ENETC0_SI7_TB6LENR NETC__ENETC0_SI7.BDR_NUM[6].TBLENR.R /* Tx BDR 6 length register */
#define NETC__ENETC0_SI7_TB6IER NETC__ENETC0_SI7.BDR_NUM[6].TBIER.R /* Tx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI7_TB6IDR NETC__ENETC0_SI7.BDR_NUM[6].TBIDR.R /* Tx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI7_TB6ICR0 NETC__ENETC0_SI7.BDR_NUM[6].TBICR0.R /* Tx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB6ICR1 NETC__ENETC0_SI7.BDR_NUM[6].TBICR1.R /* Tx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB6MR NETC__ENETC0_SI7.BDR_NUM[6].RBMR.R /* Rx BDR 6 mode register */
#define NETC__ENETC0_SI7_RB6SR NETC__ENETC0_SI7.BDR_NUM[6].RBSR.R /* Rx BDR 6 status register */
#define NETC__ENETC0_SI7_RB6BSR NETC__ENETC0_SI7.BDR_NUM[6].RBBSR.R /* Rx BDR 6 buffer size register */
#define NETC__ENETC0_SI7_RB6CIR NETC__ENETC0_SI7.BDR_NUM[6].RBCIR.R /* Rx BDR 6 consumer index register */
#define NETC__ENETC0_SI7_RB6BAR0 NETC__ENETC0_SI7.BDR_NUM[6].RBBAR0.R /* Rx BDR 6 base address register 0 */
#define NETC__ENETC0_SI7_RB6BAR1 NETC__ENETC0_SI7.BDR_NUM[6].RBBAR1.R /* Rx BDR 6 base address register 1 */
#define NETC__ENETC0_SI7_RB6PIR NETC__ENETC0_SI7.BDR_NUM[6].RBPIR.R /* Rx BDR 6 producer index register */
#define NETC__ENETC0_SI7_RB6LENR NETC__ENETC0_SI7.BDR_NUM[6].RBLENR.R /* Rx BDR 6 length register */
#define NETC__ENETC0_SI7_RB6DCR NETC__ENETC0_SI7.BDR_NUM[6].RBDCR.R /* Rx BDR 6 drop count register */
#define NETC__ENETC0_SI7_RB6IER NETC__ENETC0_SI7.BDR_NUM[6].RBIER.R /* Rx BDR 6 interrupt enable register */
#define NETC__ENETC0_SI7_RB6IDR NETC__ENETC0_SI7.BDR_NUM[6].RBIDR.R /* Rx BDR 6 interrupt detect register */
#define NETC__ENETC0_SI7_RB6ICR0 NETC__ENETC0_SI7.BDR_NUM[6].RBICR0.R /* Rx BDR 6 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB6ICR1 NETC__ENETC0_SI7.BDR_NUM[6].RBICR1.R /* Rx BDR 6 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB7MR NETC__ENETC0_SI7.BDR_NUM[7].TBMR.R /* Tx BDR 7 mode register */
#define NETC__ENETC0_SI7_TB7SR NETC__ENETC0_SI7.BDR_NUM[7].TBSR.R /* Tx BDR 7 status register */
#define NETC__ENETC0_SI7_TB7BAR0 NETC__ENETC0_SI7.BDR_NUM[7].TBBAR0.R /* Tx BDR 7 base address register 0 */
#define NETC__ENETC0_SI7_TB7BAR1 NETC__ENETC0_SI7.BDR_NUM[7].TBBAR1.R /* Tx BDR 7 base address register 1 */
#define NETC__ENETC0_SI7_TB7PIR NETC__ENETC0_SI7.BDR_NUM[7].TBPIR.R /* Tx BDR 7 producer index register */
#define NETC__ENETC0_SI7_TB7CIR NETC__ENETC0_SI7.BDR_NUM[7].TBCIR.R /* Tx BDR 7 consumer index register */
#define NETC__ENETC0_SI7_TB7LENR NETC__ENETC0_SI7.BDR_NUM[7].TBLENR.R /* Tx BDR 7 length register */
#define NETC__ENETC0_SI7_TB7IER NETC__ENETC0_SI7.BDR_NUM[7].TBIER.R /* Tx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI7_TB7IDR NETC__ENETC0_SI7.BDR_NUM[7].TBIDR.R /* Tx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI7_TB7ICR0 NETC__ENETC0_SI7.BDR_NUM[7].TBICR0.R /* Tx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB7ICR1 NETC__ENETC0_SI7.BDR_NUM[7].TBICR1.R /* Tx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB7MR NETC__ENETC0_SI7.BDR_NUM[7].RBMR.R /* Rx BDR 7 mode register */
#define NETC__ENETC0_SI7_RB7SR NETC__ENETC0_SI7.BDR_NUM[7].RBSR.R /* Rx BDR 7 status register */
#define NETC__ENETC0_SI7_RB7BSR NETC__ENETC0_SI7.BDR_NUM[7].RBBSR.R /* Rx BDR 7 buffer size register */
#define NETC__ENETC0_SI7_RB7CIR NETC__ENETC0_SI7.BDR_NUM[7].RBCIR.R /* Rx BDR 7 consumer index register */
#define NETC__ENETC0_SI7_RB7BAR0 NETC__ENETC0_SI7.BDR_NUM[7].RBBAR0.R /* Rx BDR 7 base address register 0 */
#define NETC__ENETC0_SI7_RB7BAR1 NETC__ENETC0_SI7.BDR_NUM[7].RBBAR1.R /* Rx BDR 7 base address register 1 */
#define NETC__ENETC0_SI7_RB7PIR NETC__ENETC0_SI7.BDR_NUM[7].RBPIR.R /* Rx BDR 7 producer index register */
#define NETC__ENETC0_SI7_RB7LENR NETC__ENETC0_SI7.BDR_NUM[7].RBLENR.R /* Rx BDR 7 length register */
#define NETC__ENETC0_SI7_RB7DCR NETC__ENETC0_SI7.BDR_NUM[7].RBDCR.R /* Rx BDR 7 drop count register */
#define NETC__ENETC0_SI7_RB7IER NETC__ENETC0_SI7.BDR_NUM[7].RBIER.R /* Rx BDR 7 interrupt enable register */
#define NETC__ENETC0_SI7_RB7IDR NETC__ENETC0_SI7.BDR_NUM[7].RBIDR.R /* Rx BDR 7 interrupt detect register */
#define NETC__ENETC0_SI7_RB7ICR0 NETC__ENETC0_SI7.BDR_NUM[7].RBICR0.R /* Rx BDR 7 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB7ICR1 NETC__ENETC0_SI7.BDR_NUM[7].RBICR1.R /* Rx BDR 7 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB8MR NETC__ENETC0_SI7.BDR_NUM[8].TBMR.R /* Tx BDR 8 mode register */
#define NETC__ENETC0_SI7_TB8SR NETC__ENETC0_SI7.BDR_NUM[8].TBSR.R /* Tx BDR 8 status register */
#define NETC__ENETC0_SI7_TB8BAR0 NETC__ENETC0_SI7.BDR_NUM[8].TBBAR0.R /* Tx BDR 8 base address register 0 */
#define NETC__ENETC0_SI7_TB8BAR1 NETC__ENETC0_SI7.BDR_NUM[8].TBBAR1.R /* Tx BDR 8 base address register 1 */
#define NETC__ENETC0_SI7_TB8PIR NETC__ENETC0_SI7.BDR_NUM[8].TBPIR.R /* Tx BDR 8 producer index register */
#define NETC__ENETC0_SI7_TB8CIR NETC__ENETC0_SI7.BDR_NUM[8].TBCIR.R /* Tx BDR 8 consumer index register */
#define NETC__ENETC0_SI7_TB8LENR NETC__ENETC0_SI7.BDR_NUM[8].TBLENR.R /* Tx BDR 8 length register */
#define NETC__ENETC0_SI7_TB8IER NETC__ENETC0_SI7.BDR_NUM[8].TBIER.R /* Tx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI7_TB8IDR NETC__ENETC0_SI7.BDR_NUM[8].TBIDR.R /* Tx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI7_TB8ICR0 NETC__ENETC0_SI7.BDR_NUM[8].TBICR0.R /* Tx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB8ICR1 NETC__ENETC0_SI7.BDR_NUM[8].TBICR1.R /* Tx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB8MR NETC__ENETC0_SI7.BDR_NUM[8].RBMR.R /* Rx BDR 8 mode register */
#define NETC__ENETC0_SI7_RB8SR NETC__ENETC0_SI7.BDR_NUM[8].RBSR.R /* Rx BDR 8 status register */
#define NETC__ENETC0_SI7_RB8BSR NETC__ENETC0_SI7.BDR_NUM[8].RBBSR.R /* Rx BDR 8 buffer size register */
#define NETC__ENETC0_SI7_RB8CIR NETC__ENETC0_SI7.BDR_NUM[8].RBCIR.R /* Rx BDR 8 consumer index register */
#define NETC__ENETC0_SI7_RB8BAR0 NETC__ENETC0_SI7.BDR_NUM[8].RBBAR0.R /* Rx BDR 8 base address register 0 */
#define NETC__ENETC0_SI7_RB8BAR1 NETC__ENETC0_SI7.BDR_NUM[8].RBBAR1.R /* Rx BDR 8 base address register 1 */
#define NETC__ENETC0_SI7_RB8PIR NETC__ENETC0_SI7.BDR_NUM[8].RBPIR.R /* Rx BDR 8 producer index register */
#define NETC__ENETC0_SI7_RB8LENR NETC__ENETC0_SI7.BDR_NUM[8].RBLENR.R /* Rx BDR 8 length register */
#define NETC__ENETC0_SI7_RB8DCR NETC__ENETC0_SI7.BDR_NUM[8].RBDCR.R /* Rx BDR 8 drop count register */
#define NETC__ENETC0_SI7_RB8IER NETC__ENETC0_SI7.BDR_NUM[8].RBIER.R /* Rx BDR 8 interrupt enable register */
#define NETC__ENETC0_SI7_RB8IDR NETC__ENETC0_SI7.BDR_NUM[8].RBIDR.R /* Rx BDR 8 interrupt detect register */
#define NETC__ENETC0_SI7_RB8ICR0 NETC__ENETC0_SI7.BDR_NUM[8].RBICR0.R /* Rx BDR 8 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB8ICR1 NETC__ENETC0_SI7.BDR_NUM[8].RBICR1.R /* Rx BDR 8 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB9MR NETC__ENETC0_SI7.BDR_NUM[9].TBMR.R /* Tx BDR 9 mode register */
#define NETC__ENETC0_SI7_TB9SR NETC__ENETC0_SI7.BDR_NUM[9].TBSR.R /* Tx BDR 9 status register */
#define NETC__ENETC0_SI7_TB9BAR0 NETC__ENETC0_SI7.BDR_NUM[9].TBBAR0.R /* Tx BDR 9 base address register 0 */
#define NETC__ENETC0_SI7_TB9BAR1 NETC__ENETC0_SI7.BDR_NUM[9].TBBAR1.R /* Tx BDR 9 base address register 1 */
#define NETC__ENETC0_SI7_TB9PIR NETC__ENETC0_SI7.BDR_NUM[9].TBPIR.R /* Tx BDR 9 producer index register */
#define NETC__ENETC0_SI7_TB9CIR NETC__ENETC0_SI7.BDR_NUM[9].TBCIR.R /* Tx BDR 9 consumer index register */
#define NETC__ENETC0_SI7_TB9LENR NETC__ENETC0_SI7.BDR_NUM[9].TBLENR.R /* Tx BDR 9 length register */
#define NETC__ENETC0_SI7_TB9IER NETC__ENETC0_SI7.BDR_NUM[9].TBIER.R /* Tx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI7_TB9IDR NETC__ENETC0_SI7.BDR_NUM[9].TBIDR.R /* Tx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI7_TB9ICR0 NETC__ENETC0_SI7.BDR_NUM[9].TBICR0.R /* Tx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB9ICR1 NETC__ENETC0_SI7.BDR_NUM[9].TBICR1.R /* Tx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB9MR NETC__ENETC0_SI7.BDR_NUM[9].RBMR.R /* Rx BDR 9 mode register */
#define NETC__ENETC0_SI7_RB9SR NETC__ENETC0_SI7.BDR_NUM[9].RBSR.R /* Rx BDR 9 status register */
#define NETC__ENETC0_SI7_RB9BSR NETC__ENETC0_SI7.BDR_NUM[9].RBBSR.R /* Rx BDR 9 buffer size register */
#define NETC__ENETC0_SI7_RB9CIR NETC__ENETC0_SI7.BDR_NUM[9].RBCIR.R /* Rx BDR 9 consumer index register */
#define NETC__ENETC0_SI7_RB9BAR0 NETC__ENETC0_SI7.BDR_NUM[9].RBBAR0.R /* Rx BDR 9 base address register 0 */
#define NETC__ENETC0_SI7_RB9BAR1 NETC__ENETC0_SI7.BDR_NUM[9].RBBAR1.R /* Rx BDR 9 base address register 1 */
#define NETC__ENETC0_SI7_RB9PIR NETC__ENETC0_SI7.BDR_NUM[9].RBPIR.R /* Rx BDR 9 producer index register */
#define NETC__ENETC0_SI7_RB9LENR NETC__ENETC0_SI7.BDR_NUM[9].RBLENR.R /* Rx BDR 9 length register */
#define NETC__ENETC0_SI7_RB9DCR NETC__ENETC0_SI7.BDR_NUM[9].RBDCR.R /* Rx BDR 9 drop count register */
#define NETC__ENETC0_SI7_RB9IER NETC__ENETC0_SI7.BDR_NUM[9].RBIER.R /* Rx BDR 9 interrupt enable register */
#define NETC__ENETC0_SI7_RB9IDR NETC__ENETC0_SI7.BDR_NUM[9].RBIDR.R /* Rx BDR 9 interrupt detect register */
#define NETC__ENETC0_SI7_RB9ICR0 NETC__ENETC0_SI7.BDR_NUM[9].RBICR0.R /* Rx BDR 9 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB9ICR1 NETC__ENETC0_SI7.BDR_NUM[9].RBICR1.R /* Rx BDR 9 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB10MR NETC__ENETC0_SI7.BDR_NUM[10].TBMR.R /* Tx BDR 10 mode register */
#define NETC__ENETC0_SI7_TB10SR NETC__ENETC0_SI7.BDR_NUM[10].TBSR.R /* Tx BDR 10 status register */
#define NETC__ENETC0_SI7_TB10BAR0 NETC__ENETC0_SI7.BDR_NUM[10].TBBAR0.R /* Tx BDR 10 base address register 0 */
#define NETC__ENETC0_SI7_TB10BAR1 NETC__ENETC0_SI7.BDR_NUM[10].TBBAR1.R /* Tx BDR 10 base address register 1 */
#define NETC__ENETC0_SI7_TB10PIR NETC__ENETC0_SI7.BDR_NUM[10].TBPIR.R /* Tx BDR 10 producer index register */
#define NETC__ENETC0_SI7_TB10CIR NETC__ENETC0_SI7.BDR_NUM[10].TBCIR.R /* Tx BDR 10 consumer index register */
#define NETC__ENETC0_SI7_TB10LENR NETC__ENETC0_SI7.BDR_NUM[10].TBLENR.R /* Tx BDR 10 length register */
#define NETC__ENETC0_SI7_TB10IER NETC__ENETC0_SI7.BDR_NUM[10].TBIER.R /* Tx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI7_TB10IDR NETC__ENETC0_SI7.BDR_NUM[10].TBIDR.R /* Tx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI7_TB10ICR0 NETC__ENETC0_SI7.BDR_NUM[10].TBICR0.R /* Tx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB10ICR1 NETC__ENETC0_SI7.BDR_NUM[10].TBICR1.R /* Tx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB10MR NETC__ENETC0_SI7.BDR_NUM[10].RBMR.R /* Rx BDR 10 mode register */
#define NETC__ENETC0_SI7_RB10SR NETC__ENETC0_SI7.BDR_NUM[10].RBSR.R /* Rx BDR 10 status register */
#define NETC__ENETC0_SI7_RB10BSR NETC__ENETC0_SI7.BDR_NUM[10].RBBSR.R /* Rx BDR 10 buffer size register */
#define NETC__ENETC0_SI7_RB10CIR NETC__ENETC0_SI7.BDR_NUM[10].RBCIR.R /* Rx BDR 10 consumer index register */
#define NETC__ENETC0_SI7_RB10BAR0 NETC__ENETC0_SI7.BDR_NUM[10].RBBAR0.R /* Rx BDR 10 base address register 0 */
#define NETC__ENETC0_SI7_RB10BAR1 NETC__ENETC0_SI7.BDR_NUM[10].RBBAR1.R /* Rx BDR 10 base address register 1 */
#define NETC__ENETC0_SI7_RB10PIR NETC__ENETC0_SI7.BDR_NUM[10].RBPIR.R /* Rx BDR 10 producer index register */
#define NETC__ENETC0_SI7_RB10LENR NETC__ENETC0_SI7.BDR_NUM[10].RBLENR.R /* Rx BDR 10 length register */
#define NETC__ENETC0_SI7_RB10DCR NETC__ENETC0_SI7.BDR_NUM[10].RBDCR.R /* Rx BDR 10 drop count register */
#define NETC__ENETC0_SI7_RB10IER NETC__ENETC0_SI7.BDR_NUM[10].RBIER.R /* Rx BDR 10 interrupt enable register */
#define NETC__ENETC0_SI7_RB10IDR NETC__ENETC0_SI7.BDR_NUM[10].RBIDR.R /* Rx BDR 10 interrupt detect register */
#define NETC__ENETC0_SI7_RB10ICR0 NETC__ENETC0_SI7.BDR_NUM[10].RBICR0.R /* Rx BDR 10 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB10ICR1 NETC__ENETC0_SI7.BDR_NUM[10].RBICR1.R /* Rx BDR 10 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB11MR NETC__ENETC0_SI7.BDR_NUM[11].TBMR.R /* Tx BDR 11 mode register */
#define NETC__ENETC0_SI7_TB11SR NETC__ENETC0_SI7.BDR_NUM[11].TBSR.R /* Tx BDR 11 status register */
#define NETC__ENETC0_SI7_TB11BAR0 NETC__ENETC0_SI7.BDR_NUM[11].TBBAR0.R /* Tx BDR 11 base address register 0 */
#define NETC__ENETC0_SI7_TB11BAR1 NETC__ENETC0_SI7.BDR_NUM[11].TBBAR1.R /* Tx BDR 11 base address register 1 */
#define NETC__ENETC0_SI7_TB11PIR NETC__ENETC0_SI7.BDR_NUM[11].TBPIR.R /* Tx BDR 11 producer index register */
#define NETC__ENETC0_SI7_TB11CIR NETC__ENETC0_SI7.BDR_NUM[11].TBCIR.R /* Tx BDR 11 consumer index register */
#define NETC__ENETC0_SI7_TB11LENR NETC__ENETC0_SI7.BDR_NUM[11].TBLENR.R /* Tx BDR 11 length register */
#define NETC__ENETC0_SI7_TB11IER NETC__ENETC0_SI7.BDR_NUM[11].TBIER.R /* Tx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI7_TB11IDR NETC__ENETC0_SI7.BDR_NUM[11].TBIDR.R /* Tx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI7_TB11ICR0 NETC__ENETC0_SI7.BDR_NUM[11].TBICR0.R /* Tx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB11ICR1 NETC__ENETC0_SI7.BDR_NUM[11].TBICR1.R /* Tx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB11MR NETC__ENETC0_SI7.BDR_NUM[11].RBMR.R /* Rx BDR 11 mode register */
#define NETC__ENETC0_SI7_RB11SR NETC__ENETC0_SI7.BDR_NUM[11].RBSR.R /* Rx BDR 11 status register */
#define NETC__ENETC0_SI7_RB11BSR NETC__ENETC0_SI7.BDR_NUM[11].RBBSR.R /* Rx BDR 11 buffer size register */
#define NETC__ENETC0_SI7_RB11CIR NETC__ENETC0_SI7.BDR_NUM[11].RBCIR.R /* Rx BDR 11 consumer index register */
#define NETC__ENETC0_SI7_RB11BAR0 NETC__ENETC0_SI7.BDR_NUM[11].RBBAR0.R /* Rx BDR 11 base address register 0 */
#define NETC__ENETC0_SI7_RB11BAR1 NETC__ENETC0_SI7.BDR_NUM[11].RBBAR1.R /* Rx BDR 11 base address register 1 */
#define NETC__ENETC0_SI7_RB11PIR NETC__ENETC0_SI7.BDR_NUM[11].RBPIR.R /* Rx BDR 11 producer index register */
#define NETC__ENETC0_SI7_RB11LENR NETC__ENETC0_SI7.BDR_NUM[11].RBLENR.R /* Rx BDR 11 length register */
#define NETC__ENETC0_SI7_RB11DCR NETC__ENETC0_SI7.BDR_NUM[11].RBDCR.R /* Rx BDR 11 drop count register */
#define NETC__ENETC0_SI7_RB11IER NETC__ENETC0_SI7.BDR_NUM[11].RBIER.R /* Rx BDR 11 interrupt enable register */
#define NETC__ENETC0_SI7_RB11IDR NETC__ENETC0_SI7.BDR_NUM[11].RBIDR.R /* Rx BDR 11 interrupt detect register */
#define NETC__ENETC0_SI7_RB11ICR0 NETC__ENETC0_SI7.BDR_NUM[11].RBICR0.R /* Rx BDR 11 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB11ICR1 NETC__ENETC0_SI7.BDR_NUM[11].RBICR1.R /* Rx BDR 11 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB12MR NETC__ENETC0_SI7.BDR_NUM[12].TBMR.R /* Tx BDR 12 mode register */
#define NETC__ENETC0_SI7_TB12SR NETC__ENETC0_SI7.BDR_NUM[12].TBSR.R /* Tx BDR 12 status register */
#define NETC__ENETC0_SI7_TB12BAR0 NETC__ENETC0_SI7.BDR_NUM[12].TBBAR0.R /* Tx BDR 12 base address register 0 */
#define NETC__ENETC0_SI7_TB12BAR1 NETC__ENETC0_SI7.BDR_NUM[12].TBBAR1.R /* Tx BDR 12 base address register 1 */
#define NETC__ENETC0_SI7_TB12PIR NETC__ENETC0_SI7.BDR_NUM[12].TBPIR.R /* Tx BDR 12 producer index register */
#define NETC__ENETC0_SI7_TB12CIR NETC__ENETC0_SI7.BDR_NUM[12].TBCIR.R /* Tx BDR 12 consumer index register */
#define NETC__ENETC0_SI7_TB12LENR NETC__ENETC0_SI7.BDR_NUM[12].TBLENR.R /* Tx BDR 12 length register */
#define NETC__ENETC0_SI7_TB12IER NETC__ENETC0_SI7.BDR_NUM[12].TBIER.R /* Tx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI7_TB12IDR NETC__ENETC0_SI7.BDR_NUM[12].TBIDR.R /* Tx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI7_TB12ICR0 NETC__ENETC0_SI7.BDR_NUM[12].TBICR0.R /* Tx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB12ICR1 NETC__ENETC0_SI7.BDR_NUM[12].TBICR1.R /* Tx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB12MR NETC__ENETC0_SI7.BDR_NUM[12].RBMR.R /* Rx BDR 12 mode register */
#define NETC__ENETC0_SI7_RB12SR NETC__ENETC0_SI7.BDR_NUM[12].RBSR.R /* Rx BDR 12 status register */
#define NETC__ENETC0_SI7_RB12BSR NETC__ENETC0_SI7.BDR_NUM[12].RBBSR.R /* Rx BDR 12 buffer size register */
#define NETC__ENETC0_SI7_RB12CIR NETC__ENETC0_SI7.BDR_NUM[12].RBCIR.R /* Rx BDR 12 consumer index register */
#define NETC__ENETC0_SI7_RB12BAR0 NETC__ENETC0_SI7.BDR_NUM[12].RBBAR0.R /* Rx BDR 12 base address register 0 */
#define NETC__ENETC0_SI7_RB12BAR1 NETC__ENETC0_SI7.BDR_NUM[12].RBBAR1.R /* Rx BDR 12 base address register 1 */
#define NETC__ENETC0_SI7_RB12PIR NETC__ENETC0_SI7.BDR_NUM[12].RBPIR.R /* Rx BDR 12 producer index register */
#define NETC__ENETC0_SI7_RB12LENR NETC__ENETC0_SI7.BDR_NUM[12].RBLENR.R /* Rx BDR 12 length register */
#define NETC__ENETC0_SI7_RB12DCR NETC__ENETC0_SI7.BDR_NUM[12].RBDCR.R /* Rx BDR 12 drop count register */
#define NETC__ENETC0_SI7_RB12IER NETC__ENETC0_SI7.BDR_NUM[12].RBIER.R /* Rx BDR 12 interrupt enable register */
#define NETC__ENETC0_SI7_RB12IDR NETC__ENETC0_SI7.BDR_NUM[12].RBIDR.R /* Rx BDR 12 interrupt detect register */
#define NETC__ENETC0_SI7_RB12ICR0 NETC__ENETC0_SI7.BDR_NUM[12].RBICR0.R /* Rx BDR 12 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB12ICR1 NETC__ENETC0_SI7.BDR_NUM[12].RBICR1.R /* Rx BDR 12 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB13MR NETC__ENETC0_SI7.BDR_NUM[13].TBMR.R /* Tx BDR 13 mode register */
#define NETC__ENETC0_SI7_TB13SR NETC__ENETC0_SI7.BDR_NUM[13].TBSR.R /* Tx BDR 13 status register */
#define NETC__ENETC0_SI7_TB13BAR0 NETC__ENETC0_SI7.BDR_NUM[13].TBBAR0.R /* Tx BDR 13 base address register 0 */
#define NETC__ENETC0_SI7_TB13BAR1 NETC__ENETC0_SI7.BDR_NUM[13].TBBAR1.R /* Tx BDR 13 base address register 1 */
#define NETC__ENETC0_SI7_TB13PIR NETC__ENETC0_SI7.BDR_NUM[13].TBPIR.R /* Tx BDR 13 producer index register */
#define NETC__ENETC0_SI7_TB13CIR NETC__ENETC0_SI7.BDR_NUM[13].TBCIR.R /* Tx BDR 13 consumer index register */
#define NETC__ENETC0_SI7_TB13LENR NETC__ENETC0_SI7.BDR_NUM[13].TBLENR.R /* Tx BDR 13 length register */
#define NETC__ENETC0_SI7_TB13IER NETC__ENETC0_SI7.BDR_NUM[13].TBIER.R /* Tx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI7_TB13IDR NETC__ENETC0_SI7.BDR_NUM[13].TBIDR.R /* Tx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI7_TB13ICR0 NETC__ENETC0_SI7.BDR_NUM[13].TBICR0.R /* Tx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB13ICR1 NETC__ENETC0_SI7.BDR_NUM[13].TBICR1.R /* Tx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB13MR NETC__ENETC0_SI7.BDR_NUM[13].RBMR.R /* Rx BDR 13 mode register */
#define NETC__ENETC0_SI7_RB13SR NETC__ENETC0_SI7.BDR_NUM[13].RBSR.R /* Rx BDR 13 status register */
#define NETC__ENETC0_SI7_RB13BSR NETC__ENETC0_SI7.BDR_NUM[13].RBBSR.R /* Rx BDR 13 buffer size register */
#define NETC__ENETC0_SI7_RB13CIR NETC__ENETC0_SI7.BDR_NUM[13].RBCIR.R /* Rx BDR 13 consumer index register */
#define NETC__ENETC0_SI7_RB13BAR0 NETC__ENETC0_SI7.BDR_NUM[13].RBBAR0.R /* Rx BDR 13 base address register 0 */
#define NETC__ENETC0_SI7_RB13BAR1 NETC__ENETC0_SI7.BDR_NUM[13].RBBAR1.R /* Rx BDR 13 base address register 1 */
#define NETC__ENETC0_SI7_RB13PIR NETC__ENETC0_SI7.BDR_NUM[13].RBPIR.R /* Rx BDR 13 producer index register */
#define NETC__ENETC0_SI7_RB13LENR NETC__ENETC0_SI7.BDR_NUM[13].RBLENR.R /* Rx BDR 13 length register */
#define NETC__ENETC0_SI7_RB13DCR NETC__ENETC0_SI7.BDR_NUM[13].RBDCR.R /* Rx BDR 13 drop count register */
#define NETC__ENETC0_SI7_RB13IER NETC__ENETC0_SI7.BDR_NUM[13].RBIER.R /* Rx BDR 13 interrupt enable register */
#define NETC__ENETC0_SI7_RB13IDR NETC__ENETC0_SI7.BDR_NUM[13].RBIDR.R /* Rx BDR 13 interrupt detect register */
#define NETC__ENETC0_SI7_RB13ICR0 NETC__ENETC0_SI7.BDR_NUM[13].RBICR0.R /* Rx BDR 13 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB13ICR1 NETC__ENETC0_SI7.BDR_NUM[13].RBICR1.R /* Rx BDR 13 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB14MR NETC__ENETC0_SI7.BDR_NUM[14].TBMR.R /* Tx BDR 14 mode register */
#define NETC__ENETC0_SI7_TB14SR NETC__ENETC0_SI7.BDR_NUM[14].TBSR.R /* Tx BDR 14 status register */
#define NETC__ENETC0_SI7_TB14BAR0 NETC__ENETC0_SI7.BDR_NUM[14].TBBAR0.R /* Tx BDR 14 base address register 0 */
#define NETC__ENETC0_SI7_TB14BAR1 NETC__ENETC0_SI7.BDR_NUM[14].TBBAR1.R /* Tx BDR 14 base address register 1 */
#define NETC__ENETC0_SI7_TB14PIR NETC__ENETC0_SI7.BDR_NUM[14].TBPIR.R /* Tx BDR 14 producer index register */
#define NETC__ENETC0_SI7_TB14CIR NETC__ENETC0_SI7.BDR_NUM[14].TBCIR.R /* Tx BDR 14 consumer index register */
#define NETC__ENETC0_SI7_TB14LENR NETC__ENETC0_SI7.BDR_NUM[14].TBLENR.R /* Tx BDR 14 length register */
#define NETC__ENETC0_SI7_TB14IER NETC__ENETC0_SI7.BDR_NUM[14].TBIER.R /* Tx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI7_TB14IDR NETC__ENETC0_SI7.BDR_NUM[14].TBIDR.R /* Tx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI7_TB14ICR0 NETC__ENETC0_SI7.BDR_NUM[14].TBICR0.R /* Tx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB14ICR1 NETC__ENETC0_SI7.BDR_NUM[14].TBICR1.R /* Tx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB14MR NETC__ENETC0_SI7.BDR_NUM[14].RBMR.R /* Rx BDR 14 mode register */
#define NETC__ENETC0_SI7_RB14SR NETC__ENETC0_SI7.BDR_NUM[14].RBSR.R /* Rx BDR 14 status register */
#define NETC__ENETC0_SI7_RB14BSR NETC__ENETC0_SI7.BDR_NUM[14].RBBSR.R /* Rx BDR 14 buffer size register */
#define NETC__ENETC0_SI7_RB14CIR NETC__ENETC0_SI7.BDR_NUM[14].RBCIR.R /* Rx BDR 14 consumer index register */
#define NETC__ENETC0_SI7_RB14BAR0 NETC__ENETC0_SI7.BDR_NUM[14].RBBAR0.R /* Rx BDR 14 base address register 0 */
#define NETC__ENETC0_SI7_RB14BAR1 NETC__ENETC0_SI7.BDR_NUM[14].RBBAR1.R /* Rx BDR 14 base address register 1 */
#define NETC__ENETC0_SI7_RB14PIR NETC__ENETC0_SI7.BDR_NUM[14].RBPIR.R /* Rx BDR 14 producer index register */
#define NETC__ENETC0_SI7_RB14LENR NETC__ENETC0_SI7.BDR_NUM[14].RBLENR.R /* Rx BDR 14 length register */
#define NETC__ENETC0_SI7_RB14DCR NETC__ENETC0_SI7.BDR_NUM[14].RBDCR.R /* Rx BDR 14 drop count register */
#define NETC__ENETC0_SI7_RB14IER NETC__ENETC0_SI7.BDR_NUM[14].RBIER.R /* Rx BDR 14 interrupt enable register */
#define NETC__ENETC0_SI7_RB14IDR NETC__ENETC0_SI7.BDR_NUM[14].RBIDR.R /* Rx BDR 14 interrupt detect register */
#define NETC__ENETC0_SI7_RB14ICR0 NETC__ENETC0_SI7.BDR_NUM[14].RBICR0.R /* Rx BDR 14 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB14ICR1 NETC__ENETC0_SI7.BDR_NUM[14].RBICR1.R /* Rx BDR 14 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB15MR NETC__ENETC0_SI7.BDR_NUM[15].TBMR.R /* Tx BDR 15 mode register */
#define NETC__ENETC0_SI7_TB15SR NETC__ENETC0_SI7.BDR_NUM[15].TBSR.R /* Tx BDR 15 status register */
#define NETC__ENETC0_SI7_TB15BAR0 NETC__ENETC0_SI7.BDR_NUM[15].TBBAR0.R /* Tx BDR 15 base address register 0 */
#define NETC__ENETC0_SI7_TB15BAR1 NETC__ENETC0_SI7.BDR_NUM[15].TBBAR1.R /* Tx BDR 15 base address register 1 */
#define NETC__ENETC0_SI7_TB15PIR NETC__ENETC0_SI7.BDR_NUM[15].TBPIR.R /* Tx BDR 15 producer index register */
#define NETC__ENETC0_SI7_TB15CIR NETC__ENETC0_SI7.BDR_NUM[15].TBCIR.R /* Tx BDR 15 consumer index register */
#define NETC__ENETC0_SI7_TB15LENR NETC__ENETC0_SI7.BDR_NUM[15].TBLENR.R /* Tx BDR 15 length register */
#define NETC__ENETC0_SI7_TB15IER NETC__ENETC0_SI7.BDR_NUM[15].TBIER.R /* Tx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI7_TB15IDR NETC__ENETC0_SI7.BDR_NUM[15].TBIDR.R /* Tx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI7_TB15ICR0 NETC__ENETC0_SI7.BDR_NUM[15].TBICR0.R /* Tx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB15ICR1 NETC__ENETC0_SI7.BDR_NUM[15].TBICR1.R /* Tx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB15MR NETC__ENETC0_SI7.BDR_NUM[15].RBMR.R /* Rx BDR 15 mode register */
#define NETC__ENETC0_SI7_RB15SR NETC__ENETC0_SI7.BDR_NUM[15].RBSR.R /* Rx BDR 15 status register */
#define NETC__ENETC0_SI7_RB15BSR NETC__ENETC0_SI7.BDR_NUM[15].RBBSR.R /* Rx BDR 15 buffer size register */
#define NETC__ENETC0_SI7_RB15CIR NETC__ENETC0_SI7.BDR_NUM[15].RBCIR.R /* Rx BDR 15 consumer index register */
#define NETC__ENETC0_SI7_RB15BAR0 NETC__ENETC0_SI7.BDR_NUM[15].RBBAR0.R /* Rx BDR 15 base address register 0 */
#define NETC__ENETC0_SI7_RB15BAR1 NETC__ENETC0_SI7.BDR_NUM[15].RBBAR1.R /* Rx BDR 15 base address register 1 */
#define NETC__ENETC0_SI7_RB15PIR NETC__ENETC0_SI7.BDR_NUM[15].RBPIR.R /* Rx BDR 15 producer index register */
#define NETC__ENETC0_SI7_RB15LENR NETC__ENETC0_SI7.BDR_NUM[15].RBLENR.R /* Rx BDR 15 length register */
#define NETC__ENETC0_SI7_RB15DCR NETC__ENETC0_SI7.BDR_NUM[15].RBDCR.R /* Rx BDR 15 drop count register */
#define NETC__ENETC0_SI7_RB15IER NETC__ENETC0_SI7.BDR_NUM[15].RBIER.R /* Rx BDR 15 interrupt enable register */
#define NETC__ENETC0_SI7_RB15IDR NETC__ENETC0_SI7.BDR_NUM[15].RBIDR.R /* Rx BDR 15 interrupt detect register */
#define NETC__ENETC0_SI7_RB15ICR0 NETC__ENETC0_SI7.BDR_NUM[15].RBICR0.R /* Rx BDR 15 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB15ICR1 NETC__ENETC0_SI7.BDR_NUM[15].RBICR1.R /* Rx BDR 15 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB16MR NETC__ENETC0_SI7.BDR_NUM[16].TBMR.R /* Tx BDR 16 mode register */
#define NETC__ENETC0_SI7_TB16SR NETC__ENETC0_SI7.BDR_NUM[16].TBSR.R /* Tx BDR 16 status register */
#define NETC__ENETC0_SI7_TB16BAR0 NETC__ENETC0_SI7.BDR_NUM[16].TBBAR0.R /* Tx BDR 16 base address register 0 */
#define NETC__ENETC0_SI7_TB16BAR1 NETC__ENETC0_SI7.BDR_NUM[16].TBBAR1.R /* Tx BDR 16 base address register 1 */
#define NETC__ENETC0_SI7_TB16PIR NETC__ENETC0_SI7.BDR_NUM[16].TBPIR.R /* Tx BDR 16 producer index register */
#define NETC__ENETC0_SI7_TB16CIR NETC__ENETC0_SI7.BDR_NUM[16].TBCIR.R /* Tx BDR 16 consumer index register */
#define NETC__ENETC0_SI7_TB16LENR NETC__ENETC0_SI7.BDR_NUM[16].TBLENR.R /* Tx BDR 16 length register */
#define NETC__ENETC0_SI7_TB16IER NETC__ENETC0_SI7.BDR_NUM[16].TBIER.R /* Tx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI7_TB16IDR NETC__ENETC0_SI7.BDR_NUM[16].TBIDR.R /* Tx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI7_TB16ICR0 NETC__ENETC0_SI7.BDR_NUM[16].TBICR0.R /* Tx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB16ICR1 NETC__ENETC0_SI7.BDR_NUM[16].TBICR1.R /* Tx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB16MR NETC__ENETC0_SI7.BDR_NUM[16].RBMR.R /* Rx BDR 16 mode register */
#define NETC__ENETC0_SI7_RB16SR NETC__ENETC0_SI7.BDR_NUM[16].RBSR.R /* Rx BDR 16 status register */
#define NETC__ENETC0_SI7_RB16BSR NETC__ENETC0_SI7.BDR_NUM[16].RBBSR.R /* Rx BDR 16 buffer size register */
#define NETC__ENETC0_SI7_RB16CIR NETC__ENETC0_SI7.BDR_NUM[16].RBCIR.R /* Rx BDR 16 consumer index register */
#define NETC__ENETC0_SI7_RB16BAR0 NETC__ENETC0_SI7.BDR_NUM[16].RBBAR0.R /* Rx BDR 16 base address register 0 */
#define NETC__ENETC0_SI7_RB16BAR1 NETC__ENETC0_SI7.BDR_NUM[16].RBBAR1.R /* Rx BDR 16 base address register 1 */
#define NETC__ENETC0_SI7_RB16PIR NETC__ENETC0_SI7.BDR_NUM[16].RBPIR.R /* Rx BDR 16 producer index register */
#define NETC__ENETC0_SI7_RB16LENR NETC__ENETC0_SI7.BDR_NUM[16].RBLENR.R /* Rx BDR 16 length register */
#define NETC__ENETC0_SI7_RB16DCR NETC__ENETC0_SI7.BDR_NUM[16].RBDCR.R /* Rx BDR 16 drop count register */
#define NETC__ENETC0_SI7_RB16IER NETC__ENETC0_SI7.BDR_NUM[16].RBIER.R /* Rx BDR 16 interrupt enable register */
#define NETC__ENETC0_SI7_RB16IDR NETC__ENETC0_SI7.BDR_NUM[16].RBIDR.R /* Rx BDR 16 interrupt detect register */
#define NETC__ENETC0_SI7_RB16ICR0 NETC__ENETC0_SI7.BDR_NUM[16].RBICR0.R /* Rx BDR 16 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB16ICR1 NETC__ENETC0_SI7.BDR_NUM[16].RBICR1.R /* Rx BDR 16 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_TB17MR NETC__ENETC0_SI7.BDR_NUM[17].TBMR.R /* Tx BDR 17 mode register */
#define NETC__ENETC0_SI7_TB17SR NETC__ENETC0_SI7.BDR_NUM[17].TBSR.R /* Tx BDR 17 status register */
#define NETC__ENETC0_SI7_TB17BAR0 NETC__ENETC0_SI7.BDR_NUM[17].TBBAR0.R /* Tx BDR 17 base address register 0 */
#define NETC__ENETC0_SI7_TB17BAR1 NETC__ENETC0_SI7.BDR_NUM[17].TBBAR1.R /* Tx BDR 17 base address register 1 */
#define NETC__ENETC0_SI7_TB17PIR NETC__ENETC0_SI7.BDR_NUM[17].TBPIR.R /* Tx BDR 17 producer index register */
#define NETC__ENETC0_SI7_TB17CIR NETC__ENETC0_SI7.BDR_NUM[17].TBCIR.R /* Tx BDR 17 consumer index register */
#define NETC__ENETC0_SI7_TB17LENR NETC__ENETC0_SI7.BDR_NUM[17].TBLENR.R /* Tx BDR 17 length register */
#define NETC__ENETC0_SI7_TB17IER NETC__ENETC0_SI7.BDR_NUM[17].TBIER.R /* Tx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI7_TB17IDR NETC__ENETC0_SI7.BDR_NUM[17].TBIDR.R /* Tx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI7_TB17ICR0 NETC__ENETC0_SI7.BDR_NUM[17].TBICR0.R /* Tx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_TB17ICR1 NETC__ENETC0_SI7.BDR_NUM[17].TBICR1.R /* Tx BDR 17 interrupt coalescing register 1 */
#define NETC__ENETC0_SI7_RB17MR NETC__ENETC0_SI7.BDR_NUM[17].RBMR.R /* Rx BDR 17 mode register */
#define NETC__ENETC0_SI7_RB17SR NETC__ENETC0_SI7.BDR_NUM[17].RBSR.R /* Rx BDR 17 status register */
#define NETC__ENETC0_SI7_RB17BSR NETC__ENETC0_SI7.BDR_NUM[17].RBBSR.R /* Rx BDR 17 buffer size register */
#define NETC__ENETC0_SI7_RB17CIR NETC__ENETC0_SI7.BDR_NUM[17].RBCIR.R /* Rx BDR 17 consumer index register */
#define NETC__ENETC0_SI7_RB17BAR0 NETC__ENETC0_SI7.BDR_NUM[17].RBBAR0.R /* Rx BDR 17 base address register 0 */
#define NETC__ENETC0_SI7_RB17BAR1 NETC__ENETC0_SI7.BDR_NUM[17].RBBAR1.R /* Rx BDR 17 base address register 1 */
#define NETC__ENETC0_SI7_RB17PIR NETC__ENETC0_SI7.BDR_NUM[17].RBPIR.R /* Rx BDR 17 producer index register */
#define NETC__ENETC0_SI7_RB17LENR NETC__ENETC0_SI7.BDR_NUM[17].RBLENR.R /* Rx BDR 17 length register */
#define NETC__ENETC0_SI7_RB17DCR NETC__ENETC0_SI7.BDR_NUM[17].RBDCR.R /* Rx BDR 17 drop count register */
#define NETC__ENETC0_SI7_RB17IER NETC__ENETC0_SI7.BDR_NUM[17].RBIER.R /* Rx BDR 17 interrupt enable register */
#define NETC__ENETC0_SI7_RB17IDR NETC__ENETC0_SI7.BDR_NUM[17].RBIDR.R /* Rx BDR 17 interrupt detect register */
#define NETC__ENETC0_SI7_RB17ICR0 NETC__ENETC0_SI7.BDR_NUM[17].RBICR0.R /* Rx BDR 17 interrupt coalescing register 0 */
#define NETC__ENETC0_SI7_RB17ICR1 NETC__ENETC0_SI7.BDR_NUM[17].RBICR1.R /* Rx BDR 17 interrupt coalescing register 1 */

/* NETC_IERB */
#define NETC__NETC_IERB_CAPR0 NETC__NETC_IERB.CAPR0.R      /* Capability register 0 */
#define NETC__NETC_IERB_CAPR1 NETC__NETC_IERB.CAPR1.R      /* Capability register 1 */
#define NETC__NETC_IERB_CAPR2 NETC__NETC_IERB.CAPR2.R      /* Capability register 2 */
#define NETC__NETC_IERB_CAPR3 NETC__NETC_IERB.CAPR3.R      /* Capability register 3 */
#define NETC__NETC_IERB_CMCAPR NETC__NETC_IERB.CMCAPR.R    /* Common memory capability register */
#define NETC__NETC_IERB_IPFTMCAPR NETC__NETC_IERB.IPFTMCAPR.R /* Ingress port filter ternary memory capability register */
#define NETC__NETC_IERB_TGSMCAPR NETC__NETC_IERB.TGSMCAPR.R /* Time gate scheduling memory capability register */
#define NETC__NETC_IERB_SMDTR NETC__NETC_IERB.SMDTR.R      /* Shared memory depletion threshold register */
#define NETC__NETC_IERB_ERSMBAR NETC__NETC_IERB.ERSMBAR.R  /* ENETC receive shared memory buffer allotment register */
#define NETC__NETC_IERB_HTA0HPCR NETC__NETC_IERB.HTA_NUM[0].HTAHPCR.R /* HTA 0 HP configuration register */
#define NETC__NETC_IERB_HTA0LPCR NETC__NETC_IERB.HTA_NUM[0].HTALPCR.R /* HTA 0 LP configuration register */
#define NETC__NETC_IERB_HBTMAR NETC__NETC_IERB.HBTMAR.R    /* Hash bucket table memory allocation register */
#define NETC__NETC_IERB_HBTCR NETC__NETC_IERB.HBTCR.R      /* Hash bucket table configuration register */
#define NETC__NETC_IERB_GHTEMCAPR NETC__NETC_IERB.GHTEMCAPR.R /* Guaranteed hash table entry memory capability register */
#define NETC__NETC_IERB_NETCFLRCR NETC__NETC_IERB.NETCFLRCR.R /* NETC FLR configuration register */
#define NETC__NETC_IERB_NETCCLKFR NETC__NETC_IERB.NETCCLKFR.R /* NETC clock period fractional register */
#define NETC__NETC_IERB_NETCCLKCR NETC__NETC_IERB.NETCCLKCR.R /* NETC clock configuration register */
#define NETC__NETC_IERB_SBCR NETC__NETC_IERB.SBCR.R        /* System bus configuration register */
#define NETC__NETC_IERB_SBOTCR NETC__NETC_IERB.SBOTCR.R    /* System bus outstanding transaction control register */
#define NETC__NETC_IERB_SGLTTR NETC__NETC_IERB.SGLTTR.R    /* Stream gating lag time for refresh register */
#define NETC__NETC_IERB_R0BCR NETC__NETC_IERB.ARRAY_NUM_RC[0].RBCR.R /* Root complex 0 binding configuration register */
#define NETC__NETC_IERB_RC0MSICAR NETC__NETC_IERB.ARRAY_NUM_RC[0].RCMSICAR.R /* Root complex 0 MSI-X cache attribute register */
#define NETC__NETC_IERB_RC0MSIAMQR NETC__NETC_IERB.ARRAY_NUM_RC[0].RCMSIAMQR.R /* Root complex 0 MSI access management qualifier register */
#define NETC__NETC_IERB_EMDIOBCR NETC__NETC_IERB.EMDIOBCR.R /* EMDIO binding configuration register */
#define NETC__NETC_IERB_EMDIOMCR NETC__NETC_IERB.EMDIOMCR.R /* EMDIO MSI-X configuration register */
#define NETC__NETC_IERB_EMDIO_CFH_DIDVID NETC__NETC_IERB.EMDIO_CFH_DIDVID.R /* EMDIO config header device ID and vendor ID register */
#define NETC__NETC_IERB_EMDIO_CFH_SIDSVID NETC__NETC_IERB.EMDIO_CFH_SIDSVID.R /* EMDIO config header subsystem ID and subsystem vendor ID register */
#define NETC__NETC_IERB_EMDIOBLPR0 NETC__NETC_IERB.EMDIOBLPR[0].R /* EMDIO boot loader parameter register 0 */
#define NETC__NETC_IERB_EMDIOBLPR1 NETC__NETC_IERB.EMDIOBLPR[1].R /* EMDIO boot loader parameter register 1 */
#define NETC__NETC_IERB_EMDIO_CFG NETC__NETC_IERB.EMDIO_CFG.R /* EMDIO configuration register */
#define NETC__NETC_IERB_T0BCR NETC__NETC_IERB.NUM_TMR_ARRAY[0].TBCR.R /* Timer 0 binding configuration register */
#define NETC__NETC_IERB_T0MCR NETC__NETC_IERB.NUM_TMR_ARRAY[0].TMCR.R /* Timer 0 MSI-X configuration register */
#define NETC__NETC_IERB_T0_CFH_DIDVID NETC__NETC_IERB.NUM_TMR_ARRAY[0].T_CFH_DIDVID.R /* Timer 0 config header device ID and vendor ID register */
#define NETC__NETC_IERB_T0_CFH_SIDSVID NETC__NETC_IERB.NUM_TMR_ARRAY[0].T_CFH_SIDSVID.R /* Timer 0 config header subsystem ID and subsystem vendor ID register */
#define NETC__NETC_IERB_T0BLPR0 NETC__NETC_IERB.NUM_TMR_ARRAY[0].TBLPR[0].R /* Timer 0 boot loader parameter register 0 */
#define NETC__NETC_IERB_T0BLPR1 NETC__NETC_IERB.NUM_TMR_ARRAY[0].TBLPR[1].R /* Timer 0 boot loader parameter register 1 */
#define NETC__NETC_IERB_L0CAPR NETC__NETC_IERB.L0CAPR.R    /* Link 0 capability register */
#define NETC__NETC_IERB_L0MCAPR NETC__NETC_IERB.L0MCAPR.R  /* Link 0 MAC capability register */
#define NETC__NETC_IERB_L0IOCAPR NETC__NETC_IERB.L0IOCAPR.R /* Link 0 I/O capability register */
#define NETC__NETC_IERB_L0BCR NETC__NETC_IERB.L0BCR.R      /* Link 0 binding configuration register */
#define NETC__NETC_IERB_L0TXBCCTR NETC__NETC_IERB.L0TXBCCTR.R /* Link 0 transmit byte credit comfort threshold register */
#define NETC__NETC_IERB_L0E0MAR0 NETC__NETC_IERB.L0E0MAR0.R /* Link 0 end 0 MAC address register 0 */
#define NETC__NETC_IERB_L0E0MAR1 NETC__NETC_IERB.L0E0MAR1.R /* Link 0 end 0 MAC address register 1 */
#define NETC__NETC_IERB_L1CAPR NETC__NETC_IERB.L1CAPR.R    /* Link 1 capability register */
#define NETC__NETC_IERB_L1MCAPR NETC__NETC_IERB.L1MCAPR.R  /* Link 1 MAC capability register */
#define NETC__NETC_IERB_L1IOCAPR NETC__NETC_IERB.L1IOCAPR.R /* Link 1 I/O capability register */
#define NETC__NETC_IERB_L1BCR NETC__NETC_IERB.L1BCR.R      /* Link 1 binding configuration register */
#define NETC__NETC_IERB_L1TXBCCTR NETC__NETC_IERB.L1TXBCCTR.R /* Link 1 transmit byte credit comfort threshold register */
#define NETC__NETC_IERB_L1E0MAR0 NETC__NETC_IERB.L1E0MAR0.R /* Link 1 end 0 MAC address register 0 */
#define NETC__NETC_IERB_L1E0MAR1 NETC__NETC_IERB.L1E0MAR1.R /* Link 1 end 0 MAC address register 1 */
#define NETC__NETC_IERB_L2CAPR NETC__NETC_IERB.L2CAPR.R    /* Link 2 capability register */
#define NETC__NETC_IERB_L2MCAPR NETC__NETC_IERB.L2MCAPR.R  /* Link 2 MAC capability register */
#define NETC__NETC_IERB_L2BCR NETC__NETC_IERB.L2BCR.R      /* Link 2 binding configuration register */
#define NETC__NETC_IERB_L2TXBCCTR NETC__NETC_IERB.L2TXBCCTR.R /* Link 2 transmit byte credit comfort threshold register */
#define NETC__NETC_IERB_L2E0MAR0 NETC__NETC_IERB.L2E0MAR0.R /* Link 2 end 0 MAC address register 0 */
#define NETC__NETC_IERB_L2E0MAR1 NETC__NETC_IERB.L2E0MAR1.R /* Link 2 end 0 MAC address register 1 */
#define NETC__NETC_IERB_L2E1MAR0 NETC__NETC_IERB.L2E1MAR0.R /* Link 2 end 1 MAC address register 0 */
#define NETC__NETC_IERB_L2E1MAR1 NETC__NETC_IERB.L2E1MAR1.R /* Link 2 end 1 MAC address register 1 */
#define NETC__NETC_IERB_S0BCR NETC__NETC_IERB.CFG_SW_INST[0].SBCR.R /* Switch 0 binding configuration register */
#define NETC__NETC_IERB_S0MCR NETC__NETC_IERB.CFG_SW_INST[0].SMCR.R /* Switch 0 MSI-X configuration register */
#define NETC__NETC_IERB_S0_CFH_DIDVID NETC__NETC_IERB.CFG_SW_INST[0].S_CFH_DIDVID.R /* Switch 0 config header device ID and vendor ID register */
#define NETC__NETC_IERB_S0_CFH_SIDSVID NETC__NETC_IERB.CFG_SW_INST[0].S_CFH_SIDSVID.R /* Switch 0 config header subsystem ID and subsystem vendor ID register */
#define NETC__NETC_IERB_S0CCAR NETC__NETC_IERB.CFG_SW_INST[0].SCCAR.R /* Switch 0 command cache attribute register */
#define NETC__NETC_IERB_S0AMQR NETC__NETC_IERB.CFG_SW_INST[0].SAMQR.R /* Switch 0 access management qualifier register */
#define NETC__NETC_IERB_S0BLPR0 NETC__NETC_IERB.CFG_SW_INST[0].SBLPR[0].R /* Switch 0 boot loader parameter register 0 */
#define NETC__NETC_IERB_S0BLPR1 NETC__NETC_IERB.CFG_SW_INST[0].SBLPR[1].R /* Switch 0 boot loader parameter register 1 */
#define NETC__NETC_IERB_S0SMBAR NETC__NETC_IERB.CFG_SW_INST[0].SSMBAR.R /* Switch 0 shared memory buffer allotment register */
#define NETC__NETC_IERB_S0HTMAR NETC__NETC_IERB.CFG_SW_INST[0].SHTMAR.R /* Switch 0 hash table memory allotment register */
#define NETC__NETC_IERB_S0ITMAR NETC__NETC_IERB.CFG_SW_INST[0].SITMAR.R /* Switch 0 index table memory allocation register */
#define NETC__NETC_IERB_S0IPFTMAR NETC__NETC_IERB.CFG_SW_INST[0].SIPFTMAR.R /* Switch 0 ingress port filter table memory allocation register */
#define NETC__NETC_IERB_S0RPITMAR NETC__NETC_IERB.CFG_SW_INST[0].SRPITMAR.R /* Switch 0 rate policer index table memory allocation register */
#define NETC__NETC_IERB_S0ISCITMAR NETC__NETC_IERB.CFG_SW_INST[0].SISCITMAR.R /* Switch 0 ingress stream counter index table memory allocation register */
#define NETC__NETC_IERB_S0ISITMAR NETC__NETC_IERB.CFG_SW_INST[0].SISITMAR.R /* Switch 0 ingress stream index table memory allocation register */
#define NETC__NETC_IERB_S0ISQGITMAR NETC__NETC_IERB.CFG_SW_INST[0].SISQGITMAR.R /* Switch 0 ingress sequence generation index table memory allocation register */
#define NETC__NETC_IERB_S0SGIITMAR NETC__NETC_IERB.CFG_SW_INST[0].SSGIITMAR.R /* Switch 0 stream gate instance index table memory allocation register */
#define NETC__NETC_IERB_S0SGCLITMAR NETC__NETC_IERB.CFG_SW_INST[0].SSGCLITMAR.R /* Switch 0 stream gate control list index table memory allocation register */
#define NETC__NETC_IERB_S0FMITMAR NETC__NETC_IERB.CFG_SW_INST[0].SFMITMAR.R /* Switch 0 frame modification index table memory allocation register */
#define NETC__NETC_IERB_S0FMDITMAR NETC__NETC_IERB.CFG_SW_INST[0].SFMDITMAR.R /* Switch 0 frame modification data index table memory allocation register */
#define NETC__NETC_IERB_S0TGSTAR NETC__NETC_IERB.CFG_SW_INST[0].STGSTAR.R /* Switch 0 time gate scheduling table allocation register */
#define NETC__NETC_IERB_S0TGSLR NETC__NETC_IERB.CFG_SW_INST[0].STGSLR.R /* Switch 0 time gate scheduling lookahead register */
#define NETC__NETC_IERB_S0MPCR NETC__NETC_IERB.CFG_SW_INST[0].SMPCR.R /* Switch 0 management port configuration register */
#define NETC__NETC_IERB_S0VFHTDECR0 NETC__NETC_IERB.CFG_SW_INST[0].SVFHTDECR0.R /* Switch 0 VLAN Filter (hash) table default entry configuration registers 0 */
#define NETC__NETC_IERB_S0VFHTDECR1 NETC__NETC_IERB.CFG_SW_INST[0].SVFHTDECR1.R /* Switch 0 VLAN filter hash table default entry configuration registers 1 */
#define NETC__NETC_IERB_S0VFHTDECR2 NETC__NETC_IERB.CFG_SW_INST[0].SVFHTDECR2.R /* Switch 0 VLAN filter hash table default entry configuration registers 2 */
#define NETC__NETC_IERB_E0BCR0 NETC__NETC_IERB.CFG_ENETC_INST[0].EBCR0.R /* ENETC 0 binding configuration register 0 */
#define NETC__NETC_IERB_E0BCR1 NETC__NETC_IERB.CFG_ENETC_INST[0].EBCR1.R /* ENETC 0 binding configuration register 1 */
#define NETC__NETC_IERB_E0BCR2 NETC__NETC_IERB.CFG_ENETC_INST[0].EBCR2.R /* ENETC 0 binding configuration register 2 */
#define NETC__NETC_IERB_E0VBCR NETC__NETC_IERB.CFG_ENETC_INST[0].EVBCR.R /* ENETC 0 VSI binding configuration register */
#define NETC__NETC_IERB_E0MCR NETC__NETC_IERB.CFG_ENETC_INST[0].EMCR.R /* ENETC 0 MSI-X configuration register */
#define NETC__NETC_IERB_E0_CFH_DIDVID NETC__NETC_IERB.CFG_ENETC_INST[0].E_CFH_DIDVID.R /* ENETC 0 config header device ID and vendor ID register */
#define NETC__NETC_IERB_E0_CFH_SIDSVID NETC__NETC_IERB.CFG_ENETC_INST[0].E_CFH_SIDSVID.R /* ENETC 0 config header subsystem ID and subsystem vendor ID register */
#define NETC__NETC_IERB_E0_CFC_VFDID NETC__NETC_IERB.CFG_ENETC_INST[0].E_CFC_VFDID.R /* ENETC 0 config capability VF device ID register */
#define NETC__NETC_IERB_E0BCAR NETC__NETC_IERB.CFG_ENETC_INST[0].EBCAR.R /* ENETC 0 buffer cache attribute register 0 */
#define NETC__NETC_IERB_E0MCAR NETC__NETC_IERB.CFG_ENETC_INST[0].EMCAR.R /* ENETC 0 message cache attribute register */
#define NETC__NETC_IERB_E0CAR NETC__NETC_IERB.CFG_ENETC_INST[0].ECAR.R /* ENETC 0 command cache attribute register */
#define NETC__NETC_IERB_E0AMQR NETC__NETC_IERB.CFG_ENETC_INST[0].EAMQR.R /* ENETC 0 access management qualifier register */
#define NETC__NETC_IERB_E0BLPR0 NETC__NETC_IERB.CFG_ENETC_INST[0].EBLPR[0].R /* ENETC 0 boot loader parameter register 0 */
#define NETC__NETC_IERB_E0BLPR1 NETC__NETC_IERB.CFG_ENETC_INST[0].EBLPR[1].R /* ENETC 0 boot loader parameter register 1 */
#define NETC__NETC_IERB_E0RXMBER NETC__NETC_IERB.CFG_ENETC_INST[0].ERXMBER.R /* ENETC 0 receive memory buffer entitlement register */
#define NETC__NETC_IERB_E0RXMBLR NETC__NETC_IERB.CFG_ENETC_INST[0].ERXMBLR.R /* ENETC 0 receive memory buffer limit register */
#define NETC__NETC_IERB_E0TXHPTBCR NETC__NETC_IERB.CFG_ENETC_INST[0].ETXHPTBCR.R /* ENETC 0 transmit high priority tier byte credit register */
#define NETC__NETC_IERB_E0TXLPTBCR NETC__NETC_IERB.CFG_ENETC_INST[0].ETXLPTBCR.R /* ENETC 0 transmit low priority tier byte credit register */
#define NETC__NETC_IERB_E0HTMAR NETC__NETC_IERB.CFG_ENETC_INST[0].EHTMAR.R /* ENETC 0 hash table memory allotment register */
#define NETC__NETC_IERB_E0ITMAR NETC__NETC_IERB.CFG_ENETC_INST[0].EITMAR.R /* ENETC 0 index table memory allocation register */
#define NETC__NETC_IERB_E0IPFTMAR NETC__NETC_IERB.CFG_ENETC_INST[0].EIPFTMAR.R /* ENETC 0 ingress port filter table memory allocation register */
#define NETC__NETC_IERB_E0RTMAR NETC__NETC_IERB.CFG_ENETC_INST[0].ERTMAR.R /* ENETC 0 RFS ternary memory allocation register */
#define NETC__NETC_IERB_E0RPITMAR NETC__NETC_IERB.CFG_ENETC_INST[0].ERPITMAR.R /* ENETC 0 rate policer index table memory allocation register */
#define NETC__NETC_IERB_E0ISCITMAR NETC__NETC_IERB.CFG_ENETC_INST[0].EISCITMAR.R /* ENETC 0 ingress stream counter index table memory allocation register */
#define NETC__NETC_IERB_E0ISITMAR NETC__NETC_IERB.CFG_ENETC_INST[0].EISITMAR.R /* ENETC 0 ingress stream index table memory allocation register */
#define NETC__NETC_IERB_E0SGIITMAR NETC__NETC_IERB.CFG_ENETC_INST[0].ESGIITMAR.R /* ENETC 0 stream gate instance index table memory allocation register */
#define NETC__NETC_IERB_E0SGCLITMAR NETC__NETC_IERB.CFG_ENETC_INST[0].ESGCLITMAR.R /* ENETC 0 stream gate control list index table memory allocation register */
#define NETC__NETC_IERB_E0TGSTAR NETC__NETC_IERB.CFG_ENETC_INST[0].ETGSTAR.R /* ENETC 0 time gate scheduling table allocation register */
#define NETC__NETC_IERB_E0TGSLR NETC__NETC_IERB.CFG_ENETC_INST[0].ETGSLR.R /* ENETC 0 time gate scheduling lookahead register */
#define NETC__NETC_IERB_V0AMQR NETC__NETC_IERB.CFG_VSI_INST[0].VAMQR.R /* VSI 0 access management qualifier register */
#define NETC__NETC_IERB_V0BLPR0 NETC__NETC_IERB.CFG_VSI_INST[0].VBLPR[0].R /* VSI 0 boot loader parameter register 0 */
#define NETC__NETC_IERB_V0BLPR1 NETC__NETC_IERB.CFG_VSI_INST[0].VBLPR[1].R /* VSI 0 boot loader parameter register 1 */
#define NETC__NETC_IERB_V0PMAR0 NETC__NETC_IERB.CFG_VSI_INST[0].VPMAR0.R /* VSI 0 primary MAC address register 0 */
#define NETC__NETC_IERB_V0PMAR1 NETC__NETC_IERB.CFG_VSI_INST[0].VPMAR1.R /* VSI 0 primary MAC address register 1 */
#define NETC__NETC_IERB_V1AMQR NETC__NETC_IERB.CFG_VSI_INST[1].VAMQR.R /* VSI 1 access management qualifier register */
#define NETC__NETC_IERB_V1BLPR0 NETC__NETC_IERB.CFG_VSI_INST[1].VBLPR[0].R /* VSI 1 boot loader parameter register 0 */
#define NETC__NETC_IERB_V1BLPR1 NETC__NETC_IERB.CFG_VSI_INST[1].VBLPR[1].R /* VSI 1 boot loader parameter register 1 */
#define NETC__NETC_IERB_V1PMAR0 NETC__NETC_IERB.CFG_VSI_INST[1].VPMAR0.R /* VSI 1 primary MAC address register 0 */
#define NETC__NETC_IERB_V1PMAR1 NETC__NETC_IERB.CFG_VSI_INST[1].VPMAR1.R /* VSI 1 primary MAC address register 1 */
#define NETC__NETC_IERB_V2AMQR NETC__NETC_IERB.CFG_VSI_INST[2].VAMQR.R /* VSI 2 access management qualifier register */
#define NETC__NETC_IERB_V2BLPR0 NETC__NETC_IERB.CFG_VSI_INST[2].VBLPR[0].R /* VSI 2 boot loader parameter register 0 */
#define NETC__NETC_IERB_V2BLPR1 NETC__NETC_IERB.CFG_VSI_INST[2].VBLPR[1].R /* VSI 2 boot loader parameter register 1 */
#define NETC__NETC_IERB_V2PMAR0 NETC__NETC_IERB.CFG_VSI_INST[2].VPMAR0.R /* VSI 2 primary MAC address register 0 */
#define NETC__NETC_IERB_V2PMAR1 NETC__NETC_IERB.CFG_VSI_INST[2].VPMAR1.R /* VSI 2 primary MAC address register 1 */
#define NETC__NETC_IERB_V3AMQR NETC__NETC_IERB.CFG_VSI_INST[3].VAMQR.R /* VSI 3 access management qualifier register */
#define NETC__NETC_IERB_V3BLPR0 NETC__NETC_IERB.CFG_VSI_INST[3].VBLPR[0].R /* VSI 3 boot loader parameter register 0 */
#define NETC__NETC_IERB_V3BLPR1 NETC__NETC_IERB.CFG_VSI_INST[3].VBLPR[1].R /* VSI 3 boot loader parameter register 1 */
#define NETC__NETC_IERB_V3PMAR0 NETC__NETC_IERB.CFG_VSI_INST[3].VPMAR0.R /* VSI 3 primary MAC address register 0 */
#define NETC__NETC_IERB_V3PMAR1 NETC__NETC_IERB.CFG_VSI_INST[3].VPMAR1.R /* VSI 3 primary MAC address register 1 */
#define NETC__NETC_IERB_V4AMQR NETC__NETC_IERB.CFG_VSI_INST[4].VAMQR.R /* VSI 4 access management qualifier register */
#define NETC__NETC_IERB_V4BLPR0 NETC__NETC_IERB.CFG_VSI_INST[4].VBLPR[0].R /* VSI 4 boot loader parameter register 0 */
#define NETC__NETC_IERB_V4BLPR1 NETC__NETC_IERB.CFG_VSI_INST[4].VBLPR[1].R /* VSI 4 boot loader parameter register 1 */
#define NETC__NETC_IERB_V4PMAR0 NETC__NETC_IERB.CFG_VSI_INST[4].VPMAR0.R /* VSI 4 primary MAC address register 0 */
#define NETC__NETC_IERB_V4PMAR1 NETC__NETC_IERB.CFG_VSI_INST[4].VPMAR1.R /* VSI 4 primary MAC address register 1 */
#define NETC__NETC_IERB_V5AMQR NETC__NETC_IERB.CFG_VSI_INST[5].VAMQR.R /* VSI 5 access management qualifier register */
#define NETC__NETC_IERB_V5BLPR0 NETC__NETC_IERB.CFG_VSI_INST[5].VBLPR[0].R /* VSI 5 boot loader parameter register 0 */
#define NETC__NETC_IERB_V5BLPR1 NETC__NETC_IERB.CFG_VSI_INST[5].VBLPR[1].R /* VSI 5 boot loader parameter register 1 */
#define NETC__NETC_IERB_V5PMAR0 NETC__NETC_IERB.CFG_VSI_INST[5].VPMAR0.R /* VSI 5 primary MAC address register 0 */
#define NETC__NETC_IERB_V5PMAR1 NETC__NETC_IERB.CFG_VSI_INST[5].VPMAR1.R /* VSI 5 primary MAC address register 1 */
#define NETC__NETC_IERB_V6AMQR NETC__NETC_IERB.CFG_VSI_INST[6].VAMQR.R /* VSI 6 access management qualifier register */
#define NETC__NETC_IERB_V6BLPR0 NETC__NETC_IERB.CFG_VSI_INST[6].VBLPR[0].R /* VSI 6 boot loader parameter register 0 */
#define NETC__NETC_IERB_V6BLPR1 NETC__NETC_IERB.CFG_VSI_INST[6].VBLPR[1].R /* VSI 6 boot loader parameter register 1 */
#define NETC__NETC_IERB_V6PMAR0 NETC__NETC_IERB.CFG_VSI_INST[6].VPMAR0.R /* VSI 6 primary MAC address register 0 */
#define NETC__NETC_IERB_V6PMAR1 NETC__NETC_IERB.CFG_VSI_INST[6].VPMAR1.R /* VSI 6 primary MAC address register 1 */

/* NETC_PRIV */
#define NETC__NETC_PRIV_NETCRR NETC__NETC_PRIV.NETCRR.R    /* NETC reset register */
#define NETC__NETC_PRIV_NETCSR NETC__NETC_PRIV.NETCSR.R    /* NETC status register */
#define NETC__NETC_PRIV_IEICR0 NETC__NETC_PRIV.IEICR0.R    /* Integrity Error Injection Config Register 0 */
#define NETC__NETC_PRIV_IEICR1 NETC__NETC_PRIV.IEICR1.R    /* Integrity Error Injection Config Register 1 */
#define NETC__NETC_PRIV_MEICR NETC__NETC_PRIV.MEICR.R      /* Memory Error Injection Config Register */
#define NETC__NETC_PRIV_NETCWDTR NETC__NETC_PRIV.NETCWDTR.R /* NETC watch dog timer register */
#define NETC__NETC_PRIV_CMECR NETC__NETC_PRIV.CMECR.R      /* Correctable memory error configuration register */
#define NETC__NETC_PRIV_CMESR NETC__NETC_PRIV.CMESR.R      /* Correctable memory error status register */
#define NETC__NETC_PRIV_CMECTR NETC__NETC_PRIV.CMECTR.R    /* Correctable memory error count register */
#define NETC__NETC_PRIV_UNMECR NETC__NETC_PRIV.UNMECR.R    /* Uncorrectable non-fatal memory error configuration register */
#define NETC__NETC_PRIV_UNMESR0 NETC__NETC_PRIV.UNMESR0.R  /* Uncorrectable non-fatal memory error status register 0 */
#define NETC__NETC_PRIV_UNMESR1 NETC__NETC_PRIV.UNMESR1.R  /* Uncorrectable non-fatal memory error status register 1 */
#define NETC__NETC_PRIV_UNMECTR NETC__NETC_PRIV.UNMECTR.R  /* Uncorrectable non-fatal memory error count register */
#define NETC__NETC_PRIV_UFMECR NETC__NETC_PRIV.UFMECR.R    /* Uncorrectable fatal memory error configuration register */
#define NETC__NETC_PRIV_UFMESR0 NETC__NETC_PRIV.UFMESR0.R  /* Uncorrectable fatal memory error status register 0 */
#define NETC__NETC_PRIV_UFMESR1 NETC__NETC_PRIV.UFMESR1.R  /* Uncorrectable fatal memory error status register 1 */
#define NETC__NETC_PRIV_UNIECR NETC__NETC_PRIV.UNIECR.R    /* Uncorrectable non-fatal integrity error configuration register */
#define NETC__NETC_PRIV_UNIESR NETC__NETC_PRIV.UNIESR.R    /* Uncorrectable non-fatal integrity error status register */
#define NETC__NETC_PRIV_UNIECTR NETC__NETC_PRIV.UNIECTR.R  /* Uncorrectable non-fatal integrity error count register */
#define NETC__NETC_PRIV_UFIECR NETC__NETC_PRIV.UFIECR.R    /* Uncorrectable fatal integrity error configuration register */
#define NETC__NETC_PRIV_UFIESR NETC__NETC_PRIV.UFIESR.R    /* Uncorrectable fatal integrity error status register */

/* NETC_VF1_PCI_HDR_TYPE0 */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_LAT_TIMER.R /* PCI latency timer register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BIST NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_BIST.R /* PCI BIST register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_CARDBUS_CIS.R /* PCI cardbus CIS register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_EXP_ROM_BA.R /* PCI expansion ROM base address register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_VF1_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_VF1_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_VF1_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_VF2_PCI_HDR_TYPE0 */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_LAT_TIMER.R /* PCI latency timer register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BIST NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_BIST.R /* PCI BIST register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_CARDBUS_CIS.R /* PCI cardbus CIS register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_EXP_ROM_BA.R /* PCI expansion ROM base address register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_VF2_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_VF2_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_VF2_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_VF3_PCI_HDR_TYPE0 */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_LAT_TIMER.R /* PCI latency timer register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BIST NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_BIST.R /* PCI BIST register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_CARDBUS_CIS.R /* PCI cardbus CIS register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_EXP_ROM_BA.R /* PCI expansion ROM base address register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_VF3_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_VF3_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_VF3_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_VF4_PCI_HDR_TYPE0 */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_LAT_TIMER.R /* PCI latency timer register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BIST NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_BIST.R /* PCI BIST register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_CARDBUS_CIS.R /* PCI cardbus CIS register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_EXP_ROM_BA.R /* PCI expansion ROM base address register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_VF4_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_VF4_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_VF4_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_VF5_PCI_HDR_TYPE0 */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_LAT_TIMER.R /* PCI latency timer register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BIST NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_BIST.R /* PCI BIST register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_CARDBUS_CIS.R /* PCI cardbus CIS register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_EXP_ROM_BA.R /* PCI expansion ROM base address register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_VF5_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_VF5_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_VF5_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_VF6_PCI_HDR_TYPE0 */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_LAT_TIMER.R /* PCI latency timer register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BIST NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_BIST.R /* PCI BIST register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_CARDBUS_CIS.R /* PCI cardbus CIS register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_EXP_ROM_BA.R /* PCI expansion ROM base address register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_VF6_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_VF6_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_VF6_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* NETC_VF7_PCI_HDR_TYPE0 */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_LAT_TIMER NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_LAT_TIMER.R /* PCI latency timer register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BIST NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_BIST.R /* PCI BIST register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR0 NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_BAR0.R /* PCI base address register 0 */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR1 NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_BAR1.R /* PCI base address register 1 */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR2 NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_BAR2.R /* PCI base address register 2 */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR3 NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_BAR3.R /* PCI base address register 3 */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR4 NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_BAR4.R /* PCI base address register 4 */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_BAR5 NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_BAR5.R /* PCI base address register 5 */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CARDBUS_CIS NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_CARDBUS_CIS.R /* PCI cardbus CIS register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_EXP_ROM_BA NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_EXP_ROM_BA.R /* PCI expansion ROM base address register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL.R /* PCI PCIe device control register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP2 NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP2.R /* PCI PCIe device capabilities 2 register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CTL2 NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CTL2.R /* PCI PCIe device control 2 register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_CAP_LIST NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_MSIX_CAP_LIST.R /* PCI MSI-X capabilities list register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_MSG_CTL NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_MSIX_MSG_CTL.R /* PCI MSI-X message control register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_MSIX_TABLE_OFF_BIR.R /* PCI MSI-X table offset/BIR register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR NETC__NETC_VF7_PCI_HDR_TYPE0.PCI_CFC_MSIX_PBA_OFF_BIR.R /* PCI MSI-X PBA offset/BIR register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_STAT.R /* PCIe AER uncorrectable error status register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_MASK.R /* PCIe AER uncorrectable error mask register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_AER_UCORR_ERR_SEV.R /* PCIe AER uncorrectable error severity register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_STAT.R /* PCIe AER correctable error status register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_AER_CORR_ERR_MASK.R /* PCIe AER correctable error mask register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_AER_CAP_CTL NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_AER_CAP_CTL.R /* PCIe AER capabilities and control register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP_HDR NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP_HDR.R /* PCIe ACS capability header */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CAP NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_ACS_CAP.R /* PCIe ACS capability register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_ACS_CTL NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_ACS_CTL.R /* PCIe ACS control register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_CAP_HDR NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_RTR_CAP_HDR.R /* PCIe readiness time reporting capability header */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR1 NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR1.R /* PCIe RTR readiness time reporting 1 register */
#define NETC__NETC_VF7_PCI_HDR_TYPE0_PCIE_CFC_RTR_RTR2 NETC__NETC_VF7_PCI_HDR_TYPE0.PCIE_CFC_RTR_RTR2.R /* PCIe RTR readiness time reporting 2 register */

/* OCOTP */
#define OCOTP_CTRL_SYS       OCOTP.CTRL_SYS.R              /* System master's control */
#define OCOTP_ADDR_SYS       OCOTP.ADDR_SYS.R              /* System master's eFuse address for read or write operation */
#define OCOTP_WRDATA_SYS     OCOTP.WRDATA_SYS.R            /* System master's eFuse write data */
#define OCOTP_RDATA_SYS      OCOTP.RDATA_SYS.R             /* System master's eFuse read data */
#define OCOTP_CTRL_FBX       OCOTP.CTRL_FBX.R              /* FBXC control */
#define OCOTP_CRC_RGN_SYS    OCOTP.CRC_RGN_SYS.R           /* eFuse region for CRC validation */
#define OCOTP_CRC_ADDR_SYS   OCOTP.CRC_ADDR_SYS.R          /* Pre-calculated CRC eFuse address for comparison */
#define OCOTP_CRC_VALUE_SYS  OCOTP.CRC_VALUE_SYS.R         /* CRC calculated from the eFuses */
#define OCOTP_GPR0           OCOTP.GPR[0].R                /* General purpose */
#define OCOTP_GPR1           OCOTP.GPR[1].R                /* General purpose */
#define OCOTP_STATUS_SYS     OCOTP.STATUS_SYS.R            /* System master's access status */
#define OCOTP_VERSION        OCOTP.VERSION.R               /* OCOTP design version */
#define OCOTP_STR0           OCOTP.STR[0].R                /* Sticky */
#define OCOTP_STR1           OCOTP.STR[1].R                /* Sticky */
#define OCOTP_SEC0           OCOTP.SEC0.R                  /* ECC status for single-bit ECC error */
#define OCOTP_SEC1           OCOTP.SEC1.R                  /* ECC status for single-bit ECC error */
#define OCOTP_SEC2           OCOTP.SEC2.R                  /* ECC status for single-bit ECC error */
#define OCOTP_SEC3           OCOTP.SEC3.R                  /* ECC status for single-bit ECC error */
#define OCOTP_DED0           OCOTP.DED0.R                  /* ECC status for double-bit ECC error */
#define OCOTP_DED1           OCOTP.DED1.R                  /* ECC status for double-bit ECC error */
#define OCOTP_DED2           OCOTP.DED2.R                  /* ECC status for double-bit ECC error */
#define OCOTP_DED3           OCOTP.DED3.R                  /* ECC status for double-bit ECC error */
#define OCOTP_ERR_INJCTR     OCOTP.ERR_INJCTR.R            /* ECC error injector */
#define OCOTP_SHADOWS0       OCOTP.SHADOWS[0].R            /* Shadow */
#define OCOTP_SHADOWS1       OCOTP.SHADOWS[1].R            /* Shadow */
#define OCOTP_SHADOWS2       OCOTP.SHADOWS[2].R            /* Shadow */
#define OCOTP_SHADOWS3       OCOTP.SHADOWS[3].R            /* Shadow */
#define OCOTP_SHADOWS4       OCOTP.SHADOWS[4].R            /* Shadow */
#define OCOTP_SHADOWS5       OCOTP.SHADOWS[5].R            /* Shadow */
#define OCOTP_SHADOWS6       OCOTP.SHADOWS[6].R            /* Shadow */
#define OCOTP_SHADOWS7       OCOTP.SHADOWS[7].R            /* Shadow */
#define OCOTP_SHADOWS8       OCOTP.SHADOWS[8].R            /* Shadow */
#define OCOTP_SHADOWS9       OCOTP.SHADOWS[9].R            /* Shadow */
#define OCOTP_SHADOWS10      OCOTP.SHADOWS[10].R           /* Shadow */
#define OCOTP_SHADOWS11      OCOTP.SHADOWS[11].R           /* Shadow */
#define OCOTP_SHADOWS12      OCOTP.SHADOWS[12].R           /* Shadow */
#define OCOTP_SHADOWS13      OCOTP.SHADOWS[13].R           /* Shadow */
#define OCOTP_SHADOWS14      OCOTP.SHADOWS[14].R           /* Shadow */
#define OCOTP_SHADOWS15      OCOTP.SHADOWS[15].R           /* Shadow */
#define OCOTP_SHADOWS16      OCOTP.SHADOWS[16].R           /* Shadow */
#define OCOTP_SHADOWS17      OCOTP.SHADOWS[17].R           /* Shadow */
#define OCOTP_SHADOWS18      OCOTP.SHADOWS[18].R           /* Shadow */
#define OCOTP_SHADOWS19      OCOTP.SHADOWS[19].R           /* Shadow */
#define OCOTP_SHADOWS20      OCOTP.SHADOWS[20].R           /* Shadow */
#define OCOTP_SHADOWS21      OCOTP.SHADOWS[21].R           /* Shadow */
#define OCOTP_SHADOWS22      OCOTP.SHADOWS[22].R           /* Shadow */
#define OCOTP_SHADOWS23      OCOTP.SHADOWS[23].R           /* Shadow */
#define OCOTP_SHADOWS24      OCOTP.SHADOWS[24].R           /* Shadow */
#define OCOTP_SHADOWS25      OCOTP.SHADOWS[25].R           /* Shadow */
#define OCOTP_SHADOWS26      OCOTP.SHADOWS[26].R           /* Shadow */
#define OCOTP_SHADOWS27      OCOTP.SHADOWS[27].R           /* Shadow */
#define OCOTP_SHADOWS28      OCOTP.SHADOWS[28].R           /* Shadow */
#define OCOTP_SHADOWS29      OCOTP.SHADOWS[29].R           /* Shadow */
#define OCOTP_SHADOWS30      OCOTP.SHADOWS[30].R           /* Shadow */
#define OCOTP_SHADOWS31      OCOTP.SHADOWS[31].R           /* Shadow */
#define OCOTP_SHADOWS32      OCOTP.SHADOWS[32].R           /* Shadow */
#define OCOTP_SHADOWS33      OCOTP.SHADOWS[33].R           /* Shadow */
#define OCOTP_SHADOWS34      OCOTP.SHADOWS[34].R           /* Shadow */
#define OCOTP_SHADOWS35      OCOTP.SHADOWS[35].R           /* Shadow */
#define OCOTP_SHADOWS36      OCOTP.SHADOWS[36].R           /* Shadow */
#define OCOTP_SHADOWS37      OCOTP.SHADOWS[37].R           /* Shadow */
#define OCOTP_SHADOWS38      OCOTP.SHADOWS[38].R           /* Shadow */
#define OCOTP_SHADOWS39      OCOTP.SHADOWS[39].R           /* Shadow */
#define OCOTP_SHADOWS40      OCOTP.SHADOWS[40].R           /* Shadow */
#define OCOTP_SHADOWS41      OCOTP.SHADOWS[41].R           /* Shadow */
#define OCOTP_SHADOWS42      OCOTP.SHADOWS[42].R           /* Shadow */
#define OCOTP_SHADOWS43      OCOTP.SHADOWS[43].R           /* Shadow */
#define OCOTP_SHADOWS44      OCOTP.SHADOWS[44].R           /* Shadow */
#define OCOTP_SHADOWS45      OCOTP.SHADOWS[45].R           /* Shadow */
#define OCOTP_SHADOWS46      OCOTP.SHADOWS[46].R           /* Shadow */
#define OCOTP_SHADOWS47      OCOTP.SHADOWS[47].R           /* Shadow */
#define OCOTP_SHADOWS48      OCOTP.SHADOWS[48].R           /* Shadow */
#define OCOTP_SHADOWS49      OCOTP.SHADOWS[49].R           /* Shadow */
#define OCOTP_SHADOWS50      OCOTP.SHADOWS[50].R           /* Shadow */
#define OCOTP_SHADOWS51      OCOTP.SHADOWS[51].R           /* Shadow */
#define OCOTP_SHADOWS52      OCOTP.SHADOWS[52].R           /* Shadow */
#define OCOTP_SHADOWS53      OCOTP.SHADOWS[53].R           /* Shadow */
#define OCOTP_SHADOWS54      OCOTP.SHADOWS[54].R           /* Shadow */
#define OCOTP_SHADOWS55      OCOTP.SHADOWS[55].R           /* Shadow */
#define OCOTP_SHADOWS56      OCOTP.SHADOWS[56].R           /* Shadow */
#define OCOTP_SHADOWS57      OCOTP.SHADOWS[57].R           /* Shadow */
#define OCOTP_SHADOWS58      OCOTP.SHADOWS[58].R           /* Shadow */
#define OCOTP_SHADOWS59      OCOTP.SHADOWS[59].R           /* Shadow */
#define OCOTP_SHADOWS60      OCOTP.SHADOWS[60].R           /* Shadow */
#define OCOTP_SHADOWS61      OCOTP.SHADOWS[61].R           /* Shadow */
#define OCOTP_SHADOWS62      OCOTP.SHADOWS[62].R           /* Shadow */
#define OCOTP_SHADOWS63      OCOTP.SHADOWS[63].R           /* Shadow */
#define OCOTP_SHADOWS64      OCOTP.SHADOWS[64].R           /* Shadow */
#define OCOTP_SHADOWS65      OCOTP.SHADOWS[65].R           /* Shadow */
#define OCOTP_SHADOWS66      OCOTP.SHADOWS[66].R           /* Shadow */
#define OCOTP_SHADOWS67      OCOTP.SHADOWS[67].R           /* Shadow */
#define OCOTP_SHADOWS68      OCOTP.SHADOWS[68].R           /* Shadow */
#define OCOTP_SHADOWS69      OCOTP.SHADOWS[69].R           /* Shadow */
#define OCOTP_SHADOWS70      OCOTP.SHADOWS[70].R           /* Shadow */
#define OCOTP_SHADOWS71      OCOTP.SHADOWS[71].R           /* Shadow */
#define OCOTP_SHADOWS72      OCOTP.SHADOWS[72].R           /* Shadow */
#define OCOTP_SHADOWS73      OCOTP.SHADOWS[73].R           /* Shadow */
#define OCOTP_SHADOWS74      OCOTP.SHADOWS[74].R           /* Shadow */
#define OCOTP_SHADOWS75      OCOTP.SHADOWS[75].R           /* Shadow */
#define OCOTP_SHADOWS76      OCOTP.SHADOWS[76].R           /* Shadow */
#define OCOTP_SHADOWS77      OCOTP.SHADOWS[77].R           /* Shadow */
#define OCOTP_SHADOWS78      OCOTP.SHADOWS[78].R           /* Shadow */
#define OCOTP_SHADOWS79      OCOTP.SHADOWS[79].R           /* Shadow */
#define OCOTP_SHADOWS80      OCOTP.SHADOWS[80].R           /* Shadow */
#define OCOTP_SHADOWS81      OCOTP.SHADOWS[81].R           /* Shadow */
#define OCOTP_SHADOWS82      OCOTP.SHADOWS[82].R           /* Shadow */
#define OCOTP_SHADOWS83      OCOTP.SHADOWS[83].R           /* Shadow */
#define OCOTP_SHADOWS84      OCOTP.SHADOWS[84].R           /* Shadow */
#define OCOTP_SHADOWS85      OCOTP.SHADOWS[85].R           /* Shadow */
#define OCOTP_SHADOWS86      OCOTP.SHADOWS[86].R           /* Shadow */
#define OCOTP_SHADOWS87      OCOTP.SHADOWS[87].R           /* Shadow */
#define OCOTP_SHADOWS88      OCOTP.SHADOWS[88].R           /* Shadow */
#define OCOTP_SHADOWS89      OCOTP.SHADOWS[89].R           /* Shadow */
#define OCOTP_SHADOWS90      OCOTP.SHADOWS[90].R           /* Shadow */
#define OCOTP_SHADOWS91      OCOTP.SHADOWS[91].R           /* Shadow */
#define OCOTP_SHADOWS92      OCOTP.SHADOWS[92].R           /* Shadow */
#define OCOTP_SHADOWS93      OCOTP.SHADOWS[93].R           /* Shadow */
#define OCOTP_SHADOWS94      OCOTP.SHADOWS[94].R           /* Shadow */
#define OCOTP_SHADOWS95      OCOTP.SHADOWS[95].R           /* Shadow */
#define OCOTP_SHADOWS96      OCOTP.SHADOWS[96].R           /* Shadow */
#define OCOTP_SHADOWS97      OCOTP.SHADOWS[97].R           /* Shadow */
#define OCOTP_SHADOWS98      OCOTP.SHADOWS[98].R           /* Shadow */
#define OCOTP_SHADOWS99      OCOTP.SHADOWS[99].R           /* Shadow */
#define OCOTP_SHADOWS100     OCOTP.SHADOWS[100].R          /* Shadow */
#define OCOTP_SHADOWS101     OCOTP.SHADOWS[101].R          /* Shadow */
#define OCOTP_SHADOWS102     OCOTP.SHADOWS[102].R          /* Shadow */
#define OCOTP_SHADOWS103     OCOTP.SHADOWS[103].R          /* Shadow */
#define OCOTP_SHADOWS104     OCOTP.SHADOWS[104].R          /* Shadow */
#define OCOTP_SHADOWS105     OCOTP.SHADOWS[105].R          /* Shadow */
#define OCOTP_SHADOWS106     OCOTP.SHADOWS[106].R          /* Shadow */
#define OCOTP_SHADOWS107     OCOTP.SHADOWS[107].R          /* Shadow */
#define OCOTP_SHADOWS108     OCOTP.SHADOWS[108].R          /* Shadow */
#define OCOTP_SHADOWS109     OCOTP.SHADOWS[109].R          /* Shadow */
#define OCOTP_SHADOWS110     OCOTP.SHADOWS[110].R          /* Shadow */
#define OCOTP_SHADOWS111     OCOTP.SHADOWS[111].R          /* Shadow */
#define OCOTP_SHADOWS112     OCOTP.SHADOWS[112].R          /* Shadow */
#define OCOTP_SHADOWS113     OCOTP.SHADOWS[113].R          /* Shadow */
#define OCOTP_SHADOWS114     OCOTP.SHADOWS[114].R          /* Shadow */
#define OCOTP_SHADOWS115     OCOTP.SHADOWS[115].R          /* Shadow */
#define OCOTP_SHADOWS116     OCOTP.SHADOWS[116].R          /* Shadow */
#define OCOTP_SHADOWS117     OCOTP.SHADOWS[117].R          /* Shadow */
#define OCOTP_SHADOWS118     OCOTP.SHADOWS[118].R          /* Shadow */
#define OCOTP_SHADOWS119     OCOTP.SHADOWS[119].R          /* Shadow */
#define OCOTP_SHADOWS120     OCOTP.SHADOWS[120].R          /* Shadow */
#define OCOTP_SHADOWS121     OCOTP.SHADOWS[121].R          /* Shadow */
#define OCOTP_SHADOWS122     OCOTP.SHADOWS[122].R          /* Shadow */
#define OCOTP_SHADOWS123     OCOTP.SHADOWS[123].R          /* Shadow */
#define OCOTP_SHADOWS124     OCOTP.SHADOWS[124].R          /* Shadow */
#define OCOTP_SHADOWS125     OCOTP.SHADOWS[125].R          /* Shadow */
#define OCOTP_SHADOWS126     OCOTP.SHADOWS[126].R          /* Shadow */
#define OCOTP_SHADOWS127     OCOTP.SHADOWS[127].R          /* Shadow */

/* OMU */
#define RTU0__OMU_z0ZRSR     RTU0__OMU.Z0ZRSR.R            /* Zone Request Status 0 */
#define RTU0__OMU_ZER0       RTU0__OMU.ZER0.R              /* Zone Enable 0 */
#define RTU0__OMU_z1ZRSR     RTU0__OMU.Z1ZRSR.R            /* Zone Request Status 1 */
#define RTU0__OMU_ZER1       RTU0__OMU.ZER1.R              /* Zone Enable 1 */
#define RTU0__OMU_z2ZRSR     RTU0__OMU.Z2ZRSR.R            /* Zone Request Status 2 */
#define RTU0__OMU_ZER2       RTU0__OMU.ZER2.R              /* Zone Enable 2 */
#define RTU0__OMU_z3ZRSR     RTU0__OMU.Z3ZRSR.R            /* Zone Request Status 3 */
#define RTU0__OMU_ZER3       RTU0__OMU.ZER3.R              /* Zone Enable 3 */
#define RTU0__OMU_OER        RTU0__OMU.OER.R               /* OMU Enable */
#define RTU0__OMU_ZRR        RTU0__OMU.ZRR.R               /* Zone Request */
#define RTU0__OMU_ZESR       RTU0__OMU.ZESR.R              /* Zone Enable Status */
#define RTU0__OMU_ORD0LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[0].ORDLSA.R /* ORD0 Logical Start Address */
#define RTU0__OMU_ORD0PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[0].ORDPSA.R /* ORD0 Physical Start Address */
#define RTU0__OMU_ORD0RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[0].ORDRS.R /* ORD0 Region Size */
#define RTU0__OMU_ORD0RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[0].ORDRZA.R /* ORD0 Region Zone Assignment */
#define RTU0__OMU_ORD1LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[1].ORDLSA.R /* ORD1 Logical Start Address */
#define RTU0__OMU_ORD1PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[1].ORDPSA.R /* ORD1 Physical Start Address */
#define RTU0__OMU_ORD1RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[1].ORDRS.R /* ORD1 Region Size */
#define RTU0__OMU_ORD1RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[1].ORDRZA.R /* ORD1 Region Zone Assignment */
#define RTU0__OMU_ORD2LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[2].ORDLSA.R /* ORD2 Logical Start Address */
#define RTU0__OMU_ORD2PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[2].ORDPSA.R /* ORD2 Physical Start Address */
#define RTU0__OMU_ORD2RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[2].ORDRS.R /* ORD2 Region Size */
#define RTU0__OMU_ORD2RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[2].ORDRZA.R /* ORD2 Region Zone Assignment */
#define RTU0__OMU_ORD3LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[3].ORDLSA.R /* ORD3 Logical Start Address */
#define RTU0__OMU_ORD3PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[3].ORDPSA.R /* ORD3 Physical Start Address */
#define RTU0__OMU_ORD3RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[3].ORDRS.R /* ORD3 Region Size */
#define RTU0__OMU_ORD3RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[3].ORDRZA.R /* ORD3 Region Zone Assignment */
#define RTU0__OMU_ORD4LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[4].ORDLSA.R /* ORD4 Logical Start Address */
#define RTU0__OMU_ORD4PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[4].ORDPSA.R /* ORD4 Physical Start Address */
#define RTU0__OMU_ORD4RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[4].ORDRS.R /* ORD4 Region Size */
#define RTU0__OMU_ORD4RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[4].ORDRZA.R /* ORD4 Region Zone Assignment */
#define RTU0__OMU_ORD5LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[5].ORDLSA.R /* ORD5 Logical Start Address */
#define RTU0__OMU_ORD5PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[5].ORDPSA.R /* ORD5 Physical Start Address */
#define RTU0__OMU_ORD5RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[5].ORDRS.R /* ORD5 Region Size */
#define RTU0__OMU_ORD5RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[5].ORDRZA.R /* ORD5 Region Zone Assignment */
#define RTU0__OMU_ORD6LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[6].ORDLSA.R /* ORD6 Logical Start Address */
#define RTU0__OMU_ORD6PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[6].ORDPSA.R /* ORD6 Physical Start Address */
#define RTU0__OMU_ORD6RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[6].ORDRS.R /* ORD6 Region Size */
#define RTU0__OMU_ORD6RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[6].ORDRZA.R /* ORD6 Region Zone Assignment */
#define RTU0__OMU_ORD7LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[7].ORDLSA.R /* ORD7 Logical Start Address */
#define RTU0__OMU_ORD7PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[7].ORDPSA.R /* ORD7 Physical Start Address */
#define RTU0__OMU_ORD7RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[7].ORDRS.R /* ORD7 Region Size */
#define RTU0__OMU_ORD7RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[7].ORDRZA.R /* ORD7 Region Zone Assignment */
#define RTU0__OMU_ORD8LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[8].ORDLSA.R /* ORD8 Logical Start Address */
#define RTU0__OMU_ORD8PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[8].ORDPSA.R /* ORD8 Physical Start Address */
#define RTU0__OMU_ORD8RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[8].ORDRS.R /* ORD8 Region Size */
#define RTU0__OMU_ORD8RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[8].ORDRZA.R /* ORD8 Region Zone Assignment */
#define RTU0__OMU_ORD9LSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[9].ORDLSA.R /* ORD9 Logical Start Address */
#define RTU0__OMU_ORD9PSA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[9].ORDPSA.R /* ORD9 Physical Start Address */
#define RTU0__OMU_ORD9RS     RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[9].ORDRS.R /* ORD9 Region Size */
#define RTU0__OMU_ORD9RZA    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[9].ORDRZA.R /* ORD9 Region Zone Assignment */
#define RTU0__OMU_ORD10LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[10].ORDLSA.R /* ORD10 Logical Start Address */
#define RTU0__OMU_ORD10PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[10].ORDPSA.R /* ORD10 Physical Start Address */
#define RTU0__OMU_ORD10RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[10].ORDRS.R /* ORD10 Region Size */
#define RTU0__OMU_ORD10RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[10].ORDRZA.R /* ORD10 Region Zone Assignment */
#define RTU0__OMU_ORD11LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[11].ORDLSA.R /* ORD11 Logical Start Address */
#define RTU0__OMU_ORD11PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[11].ORDPSA.R /* ORD11 Physical Start Address */
#define RTU0__OMU_ORD11RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[11].ORDRS.R /* ORD11 Region Size */
#define RTU0__OMU_ORD11RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[11].ORDRZA.R /* ORD11 Region Zone Assignment */
#define RTU0__OMU_ORD12LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[12].ORDLSA.R /* ORD12 Logical Start Address */
#define RTU0__OMU_ORD12PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[12].ORDPSA.R /* ORD12 Physical Start Address */
#define RTU0__OMU_ORD12RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[12].ORDRS.R /* ORD12 Region Size */
#define RTU0__OMU_ORD12RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[12].ORDRZA.R /* ORD12 Region Zone Assignment */
#define RTU0__OMU_ORD13LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[13].ORDLSA.R /* ORD13 Logical Start Address */
#define RTU0__OMU_ORD13PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[13].ORDPSA.R /* ORD13 Physical Start Address */
#define RTU0__OMU_ORD13RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[13].ORDRS.R /* ORD13 Region Size */
#define RTU0__OMU_ORD13RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[13].ORDRZA.R /* ORD13 Region Zone Assignment */
#define RTU0__OMU_ORD14LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[14].ORDLSA.R /* ORD14 Logical Start Address */
#define RTU0__OMU_ORD14PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[14].ORDPSA.R /* ORD14 Physical Start Address */
#define RTU0__OMU_ORD14RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[14].ORDRS.R /* ORD14 Region Size */
#define RTU0__OMU_ORD14RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[14].ORDRZA.R /* ORD14 Region Zone Assignment */
#define RTU0__OMU_ORD15LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[15].ORDLSA.R /* ORD15 Logical Start Address */
#define RTU0__OMU_ORD15PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[15].ORDPSA.R /* ORD15 Physical Start Address */
#define RTU0__OMU_ORD15RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[15].ORDRS.R /* ORD15 Region Size */
#define RTU0__OMU_ORD15RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[15].ORDRZA.R /* ORD15 Region Zone Assignment */
#define RTU0__OMU_ORD16LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[16].ORDLSA.R /* ORD16 Logical Start Address */
#define RTU0__OMU_ORD16PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[16].ORDPSA.R /* ORD16 Physical Start Address */
#define RTU0__OMU_ORD16RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[16].ORDRS.R /* ORD16 Region Size */
#define RTU0__OMU_ORD16RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[16].ORDRZA.R /* ORD16 Region Zone Assignment */
#define RTU0__OMU_ORD17LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[17].ORDLSA.R /* ORD17 Logical Start Address */
#define RTU0__OMU_ORD17PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[17].ORDPSA.R /* ORD17 Physical Start Address */
#define RTU0__OMU_ORD17RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[17].ORDRS.R /* ORD17 Region Size */
#define RTU0__OMU_ORD17RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[17].ORDRZA.R /* ORD17 Region Zone Assignment */
#define RTU0__OMU_ORD18LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[18].ORDLSA.R /* ORD18 Logical Start Address */
#define RTU0__OMU_ORD18PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[18].ORDPSA.R /* ORD18 Physical Start Address */
#define RTU0__OMU_ORD18RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[18].ORDRS.R /* ORD18 Region Size */
#define RTU0__OMU_ORD18RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[18].ORDRZA.R /* ORD18 Region Zone Assignment */
#define RTU0__OMU_ORD19LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[19].ORDLSA.R /* ORD19 Logical Start Address */
#define RTU0__OMU_ORD19PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[19].ORDPSA.R /* ORD19 Physical Start Address */
#define RTU0__OMU_ORD19RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[19].ORDRS.R /* ORD19 Region Size */
#define RTU0__OMU_ORD19RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[19].ORDRZA.R /* ORD19 Region Zone Assignment */
#define RTU0__OMU_ORD20LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[20].ORDLSA.R /* ORD20 Logical Start Address */
#define RTU0__OMU_ORD20PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[20].ORDPSA.R /* ORD20 Physical Start Address */
#define RTU0__OMU_ORD20RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[20].ORDRS.R /* ORD20 Region Size */
#define RTU0__OMU_ORD20RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[20].ORDRZA.R /* ORD20 Region Zone Assignment */
#define RTU0__OMU_ORD21LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[21].ORDLSA.R /* ORD21 Logical Start Address */
#define RTU0__OMU_ORD21PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[21].ORDPSA.R /* ORD21 Physical Start Address */
#define RTU0__OMU_ORD21RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[21].ORDRS.R /* ORD21 Region Size */
#define RTU0__OMU_ORD21RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[21].ORDRZA.R /* ORD21 Region Zone Assignment */
#define RTU0__OMU_ORD22LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[22].ORDLSA.R /* ORD22 Logical Start Address */
#define RTU0__OMU_ORD22PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[22].ORDPSA.R /* ORD22 Physical Start Address */
#define RTU0__OMU_ORD22RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[22].ORDRS.R /* ORD22 Region Size */
#define RTU0__OMU_ORD22RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[22].ORDRZA.R /* ORD22 Region Zone Assignment */
#define RTU0__OMU_ORD23LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[23].ORDLSA.R /* ORD23 Logical Start Address */
#define RTU0__OMU_ORD23PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[23].ORDPSA.R /* ORD23 Physical Start Address */
#define RTU0__OMU_ORD23RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[23].ORDRS.R /* ORD23 Region Size */
#define RTU0__OMU_ORD23RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[23].ORDRZA.R /* ORD23 Region Zone Assignment */
#define RTU0__OMU_ORD24LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[24].ORDLSA.R /* ORD24 Logical Start Address */
#define RTU0__OMU_ORD24PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[24].ORDPSA.R /* ORD24 Physical Start Address */
#define RTU0__OMU_ORD24RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[24].ORDRS.R /* ORD24 Region Size */
#define RTU0__OMU_ORD24RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[24].ORDRZA.R /* ORD24 Region Zone Assignment */
#define RTU0__OMU_ORD25LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[25].ORDLSA.R /* ORD25 Logical Start Address */
#define RTU0__OMU_ORD25PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[25].ORDPSA.R /* ORD25 Physical Start Address */
#define RTU0__OMU_ORD25RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[25].ORDRS.R /* ORD25 Region Size */
#define RTU0__OMU_ORD25RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[25].ORDRZA.R /* ORD25 Region Zone Assignment */
#define RTU0__OMU_ORD26LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[26].ORDLSA.R /* ORD26 Logical Start Address */
#define RTU0__OMU_ORD26PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[26].ORDPSA.R /* ORD26 Physical Start Address */
#define RTU0__OMU_ORD26RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[26].ORDRS.R /* ORD26 Region Size */
#define RTU0__OMU_ORD26RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[26].ORDRZA.R /* ORD26 Region Zone Assignment */
#define RTU0__OMU_ORD27LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[27].ORDLSA.R /* ORD27 Logical Start Address */
#define RTU0__OMU_ORD27PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[27].ORDPSA.R /* ORD27 Physical Start Address */
#define RTU0__OMU_ORD27RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[27].ORDRS.R /* ORD27 Region Size */
#define RTU0__OMU_ORD27RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[27].ORDRZA.R /* ORD27 Region Zone Assignment */
#define RTU0__OMU_ORD28LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[28].ORDLSA.R /* ORD28 Logical Start Address */
#define RTU0__OMU_ORD28PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[28].ORDPSA.R /* ORD28 Physical Start Address */
#define RTU0__OMU_ORD28RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[28].ORDRS.R /* ORD28 Region Size */
#define RTU0__OMU_ORD28RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[28].ORDRZA.R /* ORD28 Region Zone Assignment */
#define RTU0__OMU_ORD29LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[29].ORDLSA.R /* ORD29 Logical Start Address */
#define RTU0__OMU_ORD29PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[29].ORDPSA.R /* ORD29 Physical Start Address */
#define RTU0__OMU_ORD29RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[29].ORDRS.R /* ORD29 Region Size */
#define RTU0__OMU_ORD29RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[29].ORDRZA.R /* ORD29 Region Zone Assignment */
#define RTU0__OMU_ORD30LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[30].ORDLSA.R /* ORD30 Logical Start Address */
#define RTU0__OMU_ORD30PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[30].ORDPSA.R /* ORD30 Physical Start Address */
#define RTU0__OMU_ORD30RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[30].ORDRS.R /* ORD30 Region Size */
#define RTU0__OMU_ORD30RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[30].ORDRZA.R /* ORD30 Region Zone Assignment */
#define RTU0__OMU_ORD31LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[31].ORDLSA.R /* ORD31 Logical Start Address */
#define RTU0__OMU_ORD31PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[31].ORDPSA.R /* ORD31 Physical Start Address */
#define RTU0__OMU_ORD31RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[31].ORDRS.R /* ORD31 Region Size */
#define RTU0__OMU_ORD31RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[31].ORDRZA.R /* ORD31 Region Zone Assignment */
#define RTU0__OMU_ORD32LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[32].ORDLSA.R /* ORD32 Logical Start Address */
#define RTU0__OMU_ORD32PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[32].ORDPSA.R /* ORD32 Physical Start Address */
#define RTU0__OMU_ORD32RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[32].ORDRS.R /* ORD32 Region Size */
#define RTU0__OMU_ORD32RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[32].ORDRZA.R /* ORD32 Region Zone Assignment */
#define RTU0__OMU_ORD33LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[33].ORDLSA.R /* ORD33 Logical Start Address */
#define RTU0__OMU_ORD33PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[33].ORDPSA.R /* ORD33 Physical Start Address */
#define RTU0__OMU_ORD33RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[33].ORDRS.R /* ORD33 Region Size */
#define RTU0__OMU_ORD33RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[33].ORDRZA.R /* ORD33 Region Zone Assignment */
#define RTU0__OMU_ORD34LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[34].ORDLSA.R /* ORD34 Logical Start Address */
#define RTU0__OMU_ORD34PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[34].ORDPSA.R /* ORD34 Physical Start Address */
#define RTU0__OMU_ORD34RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[34].ORDRS.R /* ORD34 Region Size */
#define RTU0__OMU_ORD34RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[34].ORDRZA.R /* ORD34 Region Zone Assignment */
#define RTU0__OMU_ORD35LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[35].ORDLSA.R /* ORD35 Logical Start Address */
#define RTU0__OMU_ORD35PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[35].ORDPSA.R /* ORD35 Physical Start Address */
#define RTU0__OMU_ORD35RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[35].ORDRS.R /* ORD35 Region Size */
#define RTU0__OMU_ORD35RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[35].ORDRZA.R /* ORD35 Region Zone Assignment */
#define RTU0__OMU_ORD36LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[36].ORDLSA.R /* ORD36 Logical Start Address */
#define RTU0__OMU_ORD36PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[36].ORDPSA.R /* ORD36 Physical Start Address */
#define RTU0__OMU_ORD36RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[36].ORDRS.R /* ORD36 Region Size */
#define RTU0__OMU_ORD36RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[36].ORDRZA.R /* ORD36 Region Zone Assignment */
#define RTU0__OMU_ORD37LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[37].ORDLSA.R /* ORD37 Logical Start Address */
#define RTU0__OMU_ORD37PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[37].ORDPSA.R /* ORD37 Physical Start Address */
#define RTU0__OMU_ORD37RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[37].ORDRS.R /* ORD37 Region Size */
#define RTU0__OMU_ORD37RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[37].ORDRZA.R /* ORD37 Region Zone Assignment */
#define RTU0__OMU_ORD38LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[38].ORDLSA.R /* ORD38 Logical Start Address */
#define RTU0__OMU_ORD38PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[38].ORDPSA.R /* ORD38 Physical Start Address */
#define RTU0__OMU_ORD38RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[38].ORDRS.R /* ORD38 Region Size */
#define RTU0__OMU_ORD38RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[38].ORDRZA.R /* ORD38 Region Zone Assignment */
#define RTU0__OMU_ORD39LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[39].ORDLSA.R /* ORD39 Logical Start Address */
#define RTU0__OMU_ORD39PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[39].ORDPSA.R /* ORD39 Physical Start Address */
#define RTU0__OMU_ORD39RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[39].ORDRS.R /* ORD39 Region Size */
#define RTU0__OMU_ORD39RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[39].ORDRZA.R /* ORD39 Region Zone Assignment */
#define RTU0__OMU_ORD40LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[40].ORDLSA.R /* ORD40 Logical Start Address */
#define RTU0__OMU_ORD40PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[40].ORDPSA.R /* ORD40 Physical Start Address */
#define RTU0__OMU_ORD40RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[40].ORDRS.R /* ORD40 Region Size */
#define RTU0__OMU_ORD40RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[40].ORDRZA.R /* ORD40 Region Zone Assignment */
#define RTU0__OMU_ORD41LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[41].ORDLSA.R /* ORD41 Logical Start Address */
#define RTU0__OMU_ORD41PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[41].ORDPSA.R /* ORD41 Physical Start Address */
#define RTU0__OMU_ORD41RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[41].ORDRS.R /* ORD41 Region Size */
#define RTU0__OMU_ORD41RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[41].ORDRZA.R /* ORD41 Region Zone Assignment */
#define RTU0__OMU_ORD42LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[42].ORDLSA.R /* ORD42 Logical Start Address */
#define RTU0__OMU_ORD42PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[42].ORDPSA.R /* ORD42 Physical Start Address */
#define RTU0__OMU_ORD42RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[42].ORDRS.R /* ORD42 Region Size */
#define RTU0__OMU_ORD42RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[42].ORDRZA.R /* ORD42 Region Zone Assignment */
#define RTU0__OMU_ORD43LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[43].ORDLSA.R /* ORD43 Logical Start Address */
#define RTU0__OMU_ORD43PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[43].ORDPSA.R /* ORD43 Physical Start Address */
#define RTU0__OMU_ORD43RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[43].ORDRS.R /* ORD43 Region Size */
#define RTU0__OMU_ORD43RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[43].ORDRZA.R /* ORD43 Region Zone Assignment */
#define RTU0__OMU_ORD44LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[44].ORDLSA.R /* ORD44 Logical Start Address */
#define RTU0__OMU_ORD44PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[44].ORDPSA.R /* ORD44 Physical Start Address */
#define RTU0__OMU_ORD44RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[44].ORDRS.R /* ORD44 Region Size */
#define RTU0__OMU_ORD44RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[44].ORDRZA.R /* ORD44 Region Zone Assignment */
#define RTU0__OMU_ORD45LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[45].ORDLSA.R /* ORD45 Logical Start Address */
#define RTU0__OMU_ORD45PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[45].ORDPSA.R /* ORD45 Physical Start Address */
#define RTU0__OMU_ORD45RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[45].ORDRS.R /* ORD45 Region Size */
#define RTU0__OMU_ORD45RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[45].ORDRZA.R /* ORD45 Region Zone Assignment */
#define RTU0__OMU_ORD46LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[46].ORDLSA.R /* ORD46 Logical Start Address */
#define RTU0__OMU_ORD46PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[46].ORDPSA.R /* ORD46 Physical Start Address */
#define RTU0__OMU_ORD46RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[46].ORDRS.R /* ORD46 Region Size */
#define RTU0__OMU_ORD46RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[46].ORDRZA.R /* ORD46 Region Zone Assignment */
#define RTU0__OMU_ORD47LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[47].ORDLSA.R /* ORD47 Logical Start Address */
#define RTU0__OMU_ORD47PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[47].ORDPSA.R /* ORD47 Physical Start Address */
#define RTU0__OMU_ORD47RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[47].ORDRS.R /* ORD47 Region Size */
#define RTU0__OMU_ORD47RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[47].ORDRZA.R /* ORD47 Region Zone Assignment */
#define RTU0__OMU_ORD48LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[48].ORDLSA.R /* ORD48 Logical Start Address */
#define RTU0__OMU_ORD48PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[48].ORDPSA.R /* ORD48 Physical Start Address */
#define RTU0__OMU_ORD48RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[48].ORDRS.R /* ORD48 Region Size */
#define RTU0__OMU_ORD48RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[48].ORDRZA.R /* ORD48 Region Zone Assignment */
#define RTU0__OMU_ORD49LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[49].ORDLSA.R /* ORD49 Logical Start Address */
#define RTU0__OMU_ORD49PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[49].ORDPSA.R /* ORD49 Physical Start Address */
#define RTU0__OMU_ORD49RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[49].ORDRS.R /* ORD49 Region Size */
#define RTU0__OMU_ORD49RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[49].ORDRZA.R /* ORD49 Region Zone Assignment */
#define RTU0__OMU_ORD50LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[50].ORDLSA.R /* ORD50 Logical Start Address */
#define RTU0__OMU_ORD50PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[50].ORDPSA.R /* ORD50 Physical Start Address */
#define RTU0__OMU_ORD50RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[50].ORDRS.R /* ORD50 Region Size */
#define RTU0__OMU_ORD50RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[50].ORDRZA.R /* ORD50 Region Zone Assignment */
#define RTU0__OMU_ORD51LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[51].ORDLSA.R /* ORD51 Logical Start Address */
#define RTU0__OMU_ORD51PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[51].ORDPSA.R /* ORD51 Physical Start Address */
#define RTU0__OMU_ORD51RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[51].ORDRS.R /* ORD51 Region Size */
#define RTU0__OMU_ORD51RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[51].ORDRZA.R /* ORD51 Region Zone Assignment */
#define RTU0__OMU_ORD52LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[52].ORDLSA.R /* ORD52 Logical Start Address */
#define RTU0__OMU_ORD52PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[52].ORDPSA.R /* ORD52 Physical Start Address */
#define RTU0__OMU_ORD52RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[52].ORDRS.R /* ORD52 Region Size */
#define RTU0__OMU_ORD52RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[52].ORDRZA.R /* ORD52 Region Zone Assignment */
#define RTU0__OMU_ORD53LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[53].ORDLSA.R /* ORD53 Logical Start Address */
#define RTU0__OMU_ORD53PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[53].ORDPSA.R /* ORD53 Physical Start Address */
#define RTU0__OMU_ORD53RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[53].ORDRS.R /* ORD53 Region Size */
#define RTU0__OMU_ORD53RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[53].ORDRZA.R /* ORD53 Region Zone Assignment */
#define RTU0__OMU_ORD54LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[54].ORDLSA.R /* ORD54 Logical Start Address */
#define RTU0__OMU_ORD54PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[54].ORDPSA.R /* ORD54 Physical Start Address */
#define RTU0__OMU_ORD54RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[54].ORDRS.R /* ORD54 Region Size */
#define RTU0__OMU_ORD54RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[54].ORDRZA.R /* ORD54 Region Zone Assignment */
#define RTU0__OMU_ORD55LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[55].ORDLSA.R /* ORD55 Logical Start Address */
#define RTU0__OMU_ORD55PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[55].ORDPSA.R /* ORD55 Physical Start Address */
#define RTU0__OMU_ORD55RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[55].ORDRS.R /* ORD55 Region Size */
#define RTU0__OMU_ORD55RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[55].ORDRZA.R /* ORD55 Region Zone Assignment */
#define RTU0__OMU_ORD56LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[56].ORDLSA.R /* ORD56 Logical Start Address */
#define RTU0__OMU_ORD56PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[56].ORDPSA.R /* ORD56 Physical Start Address */
#define RTU0__OMU_ORD56RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[56].ORDRS.R /* ORD56 Region Size */
#define RTU0__OMU_ORD56RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[56].ORDRZA.R /* ORD56 Region Zone Assignment */
#define RTU0__OMU_ORD57LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[57].ORDLSA.R /* ORD57 Logical Start Address */
#define RTU0__OMU_ORD57PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[57].ORDPSA.R /* ORD57 Physical Start Address */
#define RTU0__OMU_ORD57RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[57].ORDRS.R /* ORD57 Region Size */
#define RTU0__OMU_ORD57RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[57].ORDRZA.R /* ORD57 Region Zone Assignment */
#define RTU0__OMU_ORD58LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[58].ORDLSA.R /* ORD58 Logical Start Address */
#define RTU0__OMU_ORD58PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[58].ORDPSA.R /* ORD58 Physical Start Address */
#define RTU0__OMU_ORD58RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[58].ORDRS.R /* ORD58 Region Size */
#define RTU0__OMU_ORD58RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[58].ORDRZA.R /* ORD58 Region Zone Assignment */
#define RTU0__OMU_ORD59LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[59].ORDLSA.R /* ORD59 Logical Start Address */
#define RTU0__OMU_ORD59PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[59].ORDPSA.R /* ORD59 Physical Start Address */
#define RTU0__OMU_ORD59RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[59].ORDRS.R /* ORD59 Region Size */
#define RTU0__OMU_ORD59RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[59].ORDRZA.R /* ORD59 Region Zone Assignment */
#define RTU0__OMU_ORD60LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[60].ORDLSA.R /* ORD60 Logical Start Address */
#define RTU0__OMU_ORD60PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[60].ORDPSA.R /* ORD60 Physical Start Address */
#define RTU0__OMU_ORD60RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[60].ORDRS.R /* ORD60 Region Size */
#define RTU0__OMU_ORD60RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[60].ORDRZA.R /* ORD60 Region Zone Assignment */
#define RTU0__OMU_ORD61LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[61].ORDLSA.R /* ORD61 Logical Start Address */
#define RTU0__OMU_ORD61PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[61].ORDPSA.R /* ORD61 Physical Start Address */
#define RTU0__OMU_ORD61RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[61].ORDRS.R /* ORD61 Region Size */
#define RTU0__OMU_ORD61RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[61].ORDRZA.R /* ORD61 Region Zone Assignment */
#define RTU0__OMU_ORD62LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[62].ORDLSA.R /* ORD62 Logical Start Address */
#define RTU0__OMU_ORD62PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[62].ORDPSA.R /* ORD62 Physical Start Address */
#define RTU0__OMU_ORD62RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[62].ORDRS.R /* ORD62 Region Size */
#define RTU0__OMU_ORD62RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[62].ORDRZA.R /* ORD62 Region Zone Assignment */
#define RTU0__OMU_ORD63LSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[63].ORDLSA.R /* ORD63 Logical Start Address */
#define RTU0__OMU_ORD63PSA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[63].ORDPSA.R /* ORD63 Physical Start Address */
#define RTU0__OMU_ORD63RS    RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[63].ORDRS.R /* ORD63 Region Size */
#define RTU0__OMU_ORD63RZA   RTU0__OMU.OVERLAY_REGION_DESCRIPTOR[63].ORDRZA.R /* ORD63 Region Zone Assignment */

#define RTU1__OMU_z0ZRSR     RTU1__OMU.Z0ZRSR.R            /* Zone Request Status 0 */
#define RTU1__OMU_ZER0       RTU1__OMU.ZER0.R              /* Zone Enable 0 */
#define RTU1__OMU_z1ZRSR     RTU1__OMU.Z1ZRSR.R            /* Zone Request Status 1 */
#define RTU1__OMU_ZER1       RTU1__OMU.ZER1.R              /* Zone Enable 1 */
#define RTU1__OMU_z2ZRSR     RTU1__OMU.Z2ZRSR.R            /* Zone Request Status 2 */
#define RTU1__OMU_ZER2       RTU1__OMU.ZER2.R              /* Zone Enable 2 */
#define RTU1__OMU_z3ZRSR     RTU1__OMU.Z3ZRSR.R            /* Zone Request Status 3 */
#define RTU1__OMU_ZER3       RTU1__OMU.ZER3.R              /* Zone Enable 3 */
#define RTU1__OMU_OER        RTU1__OMU.OER.R               /* OMU Enable */
#define RTU1__OMU_ZRR        RTU1__OMU.ZRR.R               /* Zone Request */
#define RTU1__OMU_ZESR       RTU1__OMU.ZESR.R              /* Zone Enable Status */
#define RTU1__OMU_ORD0LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[0].ORDLSA.R /* ORD0 Logical Start Address */
#define RTU1__OMU_ORD0PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[0].ORDPSA.R /* ORD0 Physical Start Address */
#define RTU1__OMU_ORD0RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[0].ORDRS.R /* ORD0 Region Size */
#define RTU1__OMU_ORD0RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[0].ORDRZA.R /* ORD0 Region Zone Assignment */
#define RTU1__OMU_ORD1LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[1].ORDLSA.R /* ORD1 Logical Start Address */
#define RTU1__OMU_ORD1PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[1].ORDPSA.R /* ORD1 Physical Start Address */
#define RTU1__OMU_ORD1RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[1].ORDRS.R /* ORD1 Region Size */
#define RTU1__OMU_ORD1RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[1].ORDRZA.R /* ORD1 Region Zone Assignment */
#define RTU1__OMU_ORD2LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[2].ORDLSA.R /* ORD2 Logical Start Address */
#define RTU1__OMU_ORD2PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[2].ORDPSA.R /* ORD2 Physical Start Address */
#define RTU1__OMU_ORD2RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[2].ORDRS.R /* ORD2 Region Size */
#define RTU1__OMU_ORD2RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[2].ORDRZA.R /* ORD2 Region Zone Assignment */
#define RTU1__OMU_ORD3LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[3].ORDLSA.R /* ORD3 Logical Start Address */
#define RTU1__OMU_ORD3PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[3].ORDPSA.R /* ORD3 Physical Start Address */
#define RTU1__OMU_ORD3RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[3].ORDRS.R /* ORD3 Region Size */
#define RTU1__OMU_ORD3RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[3].ORDRZA.R /* ORD3 Region Zone Assignment */
#define RTU1__OMU_ORD4LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[4].ORDLSA.R /* ORD4 Logical Start Address */
#define RTU1__OMU_ORD4PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[4].ORDPSA.R /* ORD4 Physical Start Address */
#define RTU1__OMU_ORD4RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[4].ORDRS.R /* ORD4 Region Size */
#define RTU1__OMU_ORD4RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[4].ORDRZA.R /* ORD4 Region Zone Assignment */
#define RTU1__OMU_ORD5LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[5].ORDLSA.R /* ORD5 Logical Start Address */
#define RTU1__OMU_ORD5PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[5].ORDPSA.R /* ORD5 Physical Start Address */
#define RTU1__OMU_ORD5RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[5].ORDRS.R /* ORD5 Region Size */
#define RTU1__OMU_ORD5RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[5].ORDRZA.R /* ORD5 Region Zone Assignment */
#define RTU1__OMU_ORD6LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[6].ORDLSA.R /* ORD6 Logical Start Address */
#define RTU1__OMU_ORD6PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[6].ORDPSA.R /* ORD6 Physical Start Address */
#define RTU1__OMU_ORD6RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[6].ORDRS.R /* ORD6 Region Size */
#define RTU1__OMU_ORD6RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[6].ORDRZA.R /* ORD6 Region Zone Assignment */
#define RTU1__OMU_ORD7LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[7].ORDLSA.R /* ORD7 Logical Start Address */
#define RTU1__OMU_ORD7PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[7].ORDPSA.R /* ORD7 Physical Start Address */
#define RTU1__OMU_ORD7RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[7].ORDRS.R /* ORD7 Region Size */
#define RTU1__OMU_ORD7RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[7].ORDRZA.R /* ORD7 Region Zone Assignment */
#define RTU1__OMU_ORD8LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[8].ORDLSA.R /* ORD8 Logical Start Address */
#define RTU1__OMU_ORD8PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[8].ORDPSA.R /* ORD8 Physical Start Address */
#define RTU1__OMU_ORD8RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[8].ORDRS.R /* ORD8 Region Size */
#define RTU1__OMU_ORD8RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[8].ORDRZA.R /* ORD8 Region Zone Assignment */
#define RTU1__OMU_ORD9LSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[9].ORDLSA.R /* ORD9 Logical Start Address */
#define RTU1__OMU_ORD9PSA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[9].ORDPSA.R /* ORD9 Physical Start Address */
#define RTU1__OMU_ORD9RS     RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[9].ORDRS.R /* ORD9 Region Size */
#define RTU1__OMU_ORD9RZA    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[9].ORDRZA.R /* ORD9 Region Zone Assignment */
#define RTU1__OMU_ORD10LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[10].ORDLSA.R /* ORD10 Logical Start Address */
#define RTU1__OMU_ORD10PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[10].ORDPSA.R /* ORD10 Physical Start Address */
#define RTU1__OMU_ORD10RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[10].ORDRS.R /* ORD10 Region Size */
#define RTU1__OMU_ORD10RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[10].ORDRZA.R /* ORD10 Region Zone Assignment */
#define RTU1__OMU_ORD11LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[11].ORDLSA.R /* ORD11 Logical Start Address */
#define RTU1__OMU_ORD11PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[11].ORDPSA.R /* ORD11 Physical Start Address */
#define RTU1__OMU_ORD11RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[11].ORDRS.R /* ORD11 Region Size */
#define RTU1__OMU_ORD11RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[11].ORDRZA.R /* ORD11 Region Zone Assignment */
#define RTU1__OMU_ORD12LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[12].ORDLSA.R /* ORD12 Logical Start Address */
#define RTU1__OMU_ORD12PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[12].ORDPSA.R /* ORD12 Physical Start Address */
#define RTU1__OMU_ORD12RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[12].ORDRS.R /* ORD12 Region Size */
#define RTU1__OMU_ORD12RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[12].ORDRZA.R /* ORD12 Region Zone Assignment */
#define RTU1__OMU_ORD13LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[13].ORDLSA.R /* ORD13 Logical Start Address */
#define RTU1__OMU_ORD13PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[13].ORDPSA.R /* ORD13 Physical Start Address */
#define RTU1__OMU_ORD13RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[13].ORDRS.R /* ORD13 Region Size */
#define RTU1__OMU_ORD13RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[13].ORDRZA.R /* ORD13 Region Zone Assignment */
#define RTU1__OMU_ORD14LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[14].ORDLSA.R /* ORD14 Logical Start Address */
#define RTU1__OMU_ORD14PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[14].ORDPSA.R /* ORD14 Physical Start Address */
#define RTU1__OMU_ORD14RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[14].ORDRS.R /* ORD14 Region Size */
#define RTU1__OMU_ORD14RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[14].ORDRZA.R /* ORD14 Region Zone Assignment */
#define RTU1__OMU_ORD15LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[15].ORDLSA.R /* ORD15 Logical Start Address */
#define RTU1__OMU_ORD15PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[15].ORDPSA.R /* ORD15 Physical Start Address */
#define RTU1__OMU_ORD15RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[15].ORDRS.R /* ORD15 Region Size */
#define RTU1__OMU_ORD15RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[15].ORDRZA.R /* ORD15 Region Zone Assignment */
#define RTU1__OMU_ORD16LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[16].ORDLSA.R /* ORD16 Logical Start Address */
#define RTU1__OMU_ORD16PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[16].ORDPSA.R /* ORD16 Physical Start Address */
#define RTU1__OMU_ORD16RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[16].ORDRS.R /* ORD16 Region Size */
#define RTU1__OMU_ORD16RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[16].ORDRZA.R /* ORD16 Region Zone Assignment */
#define RTU1__OMU_ORD17LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[17].ORDLSA.R /* ORD17 Logical Start Address */
#define RTU1__OMU_ORD17PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[17].ORDPSA.R /* ORD17 Physical Start Address */
#define RTU1__OMU_ORD17RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[17].ORDRS.R /* ORD17 Region Size */
#define RTU1__OMU_ORD17RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[17].ORDRZA.R /* ORD17 Region Zone Assignment */
#define RTU1__OMU_ORD18LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[18].ORDLSA.R /* ORD18 Logical Start Address */
#define RTU1__OMU_ORD18PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[18].ORDPSA.R /* ORD18 Physical Start Address */
#define RTU1__OMU_ORD18RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[18].ORDRS.R /* ORD18 Region Size */
#define RTU1__OMU_ORD18RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[18].ORDRZA.R /* ORD18 Region Zone Assignment */
#define RTU1__OMU_ORD19LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[19].ORDLSA.R /* ORD19 Logical Start Address */
#define RTU1__OMU_ORD19PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[19].ORDPSA.R /* ORD19 Physical Start Address */
#define RTU1__OMU_ORD19RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[19].ORDRS.R /* ORD19 Region Size */
#define RTU1__OMU_ORD19RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[19].ORDRZA.R /* ORD19 Region Zone Assignment */
#define RTU1__OMU_ORD20LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[20].ORDLSA.R /* ORD20 Logical Start Address */
#define RTU1__OMU_ORD20PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[20].ORDPSA.R /* ORD20 Physical Start Address */
#define RTU1__OMU_ORD20RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[20].ORDRS.R /* ORD20 Region Size */
#define RTU1__OMU_ORD20RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[20].ORDRZA.R /* ORD20 Region Zone Assignment */
#define RTU1__OMU_ORD21LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[21].ORDLSA.R /* ORD21 Logical Start Address */
#define RTU1__OMU_ORD21PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[21].ORDPSA.R /* ORD21 Physical Start Address */
#define RTU1__OMU_ORD21RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[21].ORDRS.R /* ORD21 Region Size */
#define RTU1__OMU_ORD21RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[21].ORDRZA.R /* ORD21 Region Zone Assignment */
#define RTU1__OMU_ORD22LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[22].ORDLSA.R /* ORD22 Logical Start Address */
#define RTU1__OMU_ORD22PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[22].ORDPSA.R /* ORD22 Physical Start Address */
#define RTU1__OMU_ORD22RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[22].ORDRS.R /* ORD22 Region Size */
#define RTU1__OMU_ORD22RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[22].ORDRZA.R /* ORD22 Region Zone Assignment */
#define RTU1__OMU_ORD23LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[23].ORDLSA.R /* ORD23 Logical Start Address */
#define RTU1__OMU_ORD23PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[23].ORDPSA.R /* ORD23 Physical Start Address */
#define RTU1__OMU_ORD23RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[23].ORDRS.R /* ORD23 Region Size */
#define RTU1__OMU_ORD23RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[23].ORDRZA.R /* ORD23 Region Zone Assignment */
#define RTU1__OMU_ORD24LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[24].ORDLSA.R /* ORD24 Logical Start Address */
#define RTU1__OMU_ORD24PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[24].ORDPSA.R /* ORD24 Physical Start Address */
#define RTU1__OMU_ORD24RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[24].ORDRS.R /* ORD24 Region Size */
#define RTU1__OMU_ORD24RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[24].ORDRZA.R /* ORD24 Region Zone Assignment */
#define RTU1__OMU_ORD25LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[25].ORDLSA.R /* ORD25 Logical Start Address */
#define RTU1__OMU_ORD25PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[25].ORDPSA.R /* ORD25 Physical Start Address */
#define RTU1__OMU_ORD25RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[25].ORDRS.R /* ORD25 Region Size */
#define RTU1__OMU_ORD25RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[25].ORDRZA.R /* ORD25 Region Zone Assignment */
#define RTU1__OMU_ORD26LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[26].ORDLSA.R /* ORD26 Logical Start Address */
#define RTU1__OMU_ORD26PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[26].ORDPSA.R /* ORD26 Physical Start Address */
#define RTU1__OMU_ORD26RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[26].ORDRS.R /* ORD26 Region Size */
#define RTU1__OMU_ORD26RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[26].ORDRZA.R /* ORD26 Region Zone Assignment */
#define RTU1__OMU_ORD27LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[27].ORDLSA.R /* ORD27 Logical Start Address */
#define RTU1__OMU_ORD27PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[27].ORDPSA.R /* ORD27 Physical Start Address */
#define RTU1__OMU_ORD27RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[27].ORDRS.R /* ORD27 Region Size */
#define RTU1__OMU_ORD27RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[27].ORDRZA.R /* ORD27 Region Zone Assignment */
#define RTU1__OMU_ORD28LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[28].ORDLSA.R /* ORD28 Logical Start Address */
#define RTU1__OMU_ORD28PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[28].ORDPSA.R /* ORD28 Physical Start Address */
#define RTU1__OMU_ORD28RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[28].ORDRS.R /* ORD28 Region Size */
#define RTU1__OMU_ORD28RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[28].ORDRZA.R /* ORD28 Region Zone Assignment */
#define RTU1__OMU_ORD29LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[29].ORDLSA.R /* ORD29 Logical Start Address */
#define RTU1__OMU_ORD29PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[29].ORDPSA.R /* ORD29 Physical Start Address */
#define RTU1__OMU_ORD29RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[29].ORDRS.R /* ORD29 Region Size */
#define RTU1__OMU_ORD29RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[29].ORDRZA.R /* ORD29 Region Zone Assignment */
#define RTU1__OMU_ORD30LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[30].ORDLSA.R /* ORD30 Logical Start Address */
#define RTU1__OMU_ORD30PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[30].ORDPSA.R /* ORD30 Physical Start Address */
#define RTU1__OMU_ORD30RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[30].ORDRS.R /* ORD30 Region Size */
#define RTU1__OMU_ORD30RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[30].ORDRZA.R /* ORD30 Region Zone Assignment */
#define RTU1__OMU_ORD31LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[31].ORDLSA.R /* ORD31 Logical Start Address */
#define RTU1__OMU_ORD31PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[31].ORDPSA.R /* ORD31 Physical Start Address */
#define RTU1__OMU_ORD31RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[31].ORDRS.R /* ORD31 Region Size */
#define RTU1__OMU_ORD31RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[31].ORDRZA.R /* ORD31 Region Zone Assignment */
#define RTU1__OMU_ORD32LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[32].ORDLSA.R /* ORD32 Logical Start Address */
#define RTU1__OMU_ORD32PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[32].ORDPSA.R /* ORD32 Physical Start Address */
#define RTU1__OMU_ORD32RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[32].ORDRS.R /* ORD32 Region Size */
#define RTU1__OMU_ORD32RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[32].ORDRZA.R /* ORD32 Region Zone Assignment */
#define RTU1__OMU_ORD33LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[33].ORDLSA.R /* ORD33 Logical Start Address */
#define RTU1__OMU_ORD33PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[33].ORDPSA.R /* ORD33 Physical Start Address */
#define RTU1__OMU_ORD33RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[33].ORDRS.R /* ORD33 Region Size */
#define RTU1__OMU_ORD33RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[33].ORDRZA.R /* ORD33 Region Zone Assignment */
#define RTU1__OMU_ORD34LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[34].ORDLSA.R /* ORD34 Logical Start Address */
#define RTU1__OMU_ORD34PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[34].ORDPSA.R /* ORD34 Physical Start Address */
#define RTU1__OMU_ORD34RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[34].ORDRS.R /* ORD34 Region Size */
#define RTU1__OMU_ORD34RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[34].ORDRZA.R /* ORD34 Region Zone Assignment */
#define RTU1__OMU_ORD35LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[35].ORDLSA.R /* ORD35 Logical Start Address */
#define RTU1__OMU_ORD35PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[35].ORDPSA.R /* ORD35 Physical Start Address */
#define RTU1__OMU_ORD35RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[35].ORDRS.R /* ORD35 Region Size */
#define RTU1__OMU_ORD35RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[35].ORDRZA.R /* ORD35 Region Zone Assignment */
#define RTU1__OMU_ORD36LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[36].ORDLSA.R /* ORD36 Logical Start Address */
#define RTU1__OMU_ORD36PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[36].ORDPSA.R /* ORD36 Physical Start Address */
#define RTU1__OMU_ORD36RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[36].ORDRS.R /* ORD36 Region Size */
#define RTU1__OMU_ORD36RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[36].ORDRZA.R /* ORD36 Region Zone Assignment */
#define RTU1__OMU_ORD37LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[37].ORDLSA.R /* ORD37 Logical Start Address */
#define RTU1__OMU_ORD37PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[37].ORDPSA.R /* ORD37 Physical Start Address */
#define RTU1__OMU_ORD37RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[37].ORDRS.R /* ORD37 Region Size */
#define RTU1__OMU_ORD37RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[37].ORDRZA.R /* ORD37 Region Zone Assignment */
#define RTU1__OMU_ORD38LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[38].ORDLSA.R /* ORD38 Logical Start Address */
#define RTU1__OMU_ORD38PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[38].ORDPSA.R /* ORD38 Physical Start Address */
#define RTU1__OMU_ORD38RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[38].ORDRS.R /* ORD38 Region Size */
#define RTU1__OMU_ORD38RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[38].ORDRZA.R /* ORD38 Region Zone Assignment */
#define RTU1__OMU_ORD39LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[39].ORDLSA.R /* ORD39 Logical Start Address */
#define RTU1__OMU_ORD39PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[39].ORDPSA.R /* ORD39 Physical Start Address */
#define RTU1__OMU_ORD39RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[39].ORDRS.R /* ORD39 Region Size */
#define RTU1__OMU_ORD39RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[39].ORDRZA.R /* ORD39 Region Zone Assignment */
#define RTU1__OMU_ORD40LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[40].ORDLSA.R /* ORD40 Logical Start Address */
#define RTU1__OMU_ORD40PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[40].ORDPSA.R /* ORD40 Physical Start Address */
#define RTU1__OMU_ORD40RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[40].ORDRS.R /* ORD40 Region Size */
#define RTU1__OMU_ORD40RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[40].ORDRZA.R /* ORD40 Region Zone Assignment */
#define RTU1__OMU_ORD41LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[41].ORDLSA.R /* ORD41 Logical Start Address */
#define RTU1__OMU_ORD41PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[41].ORDPSA.R /* ORD41 Physical Start Address */
#define RTU1__OMU_ORD41RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[41].ORDRS.R /* ORD41 Region Size */
#define RTU1__OMU_ORD41RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[41].ORDRZA.R /* ORD41 Region Zone Assignment */
#define RTU1__OMU_ORD42LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[42].ORDLSA.R /* ORD42 Logical Start Address */
#define RTU1__OMU_ORD42PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[42].ORDPSA.R /* ORD42 Physical Start Address */
#define RTU1__OMU_ORD42RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[42].ORDRS.R /* ORD42 Region Size */
#define RTU1__OMU_ORD42RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[42].ORDRZA.R /* ORD42 Region Zone Assignment */
#define RTU1__OMU_ORD43LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[43].ORDLSA.R /* ORD43 Logical Start Address */
#define RTU1__OMU_ORD43PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[43].ORDPSA.R /* ORD43 Physical Start Address */
#define RTU1__OMU_ORD43RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[43].ORDRS.R /* ORD43 Region Size */
#define RTU1__OMU_ORD43RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[43].ORDRZA.R /* ORD43 Region Zone Assignment */
#define RTU1__OMU_ORD44LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[44].ORDLSA.R /* ORD44 Logical Start Address */
#define RTU1__OMU_ORD44PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[44].ORDPSA.R /* ORD44 Physical Start Address */
#define RTU1__OMU_ORD44RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[44].ORDRS.R /* ORD44 Region Size */
#define RTU1__OMU_ORD44RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[44].ORDRZA.R /* ORD44 Region Zone Assignment */
#define RTU1__OMU_ORD45LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[45].ORDLSA.R /* ORD45 Logical Start Address */
#define RTU1__OMU_ORD45PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[45].ORDPSA.R /* ORD45 Physical Start Address */
#define RTU1__OMU_ORD45RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[45].ORDRS.R /* ORD45 Region Size */
#define RTU1__OMU_ORD45RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[45].ORDRZA.R /* ORD45 Region Zone Assignment */
#define RTU1__OMU_ORD46LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[46].ORDLSA.R /* ORD46 Logical Start Address */
#define RTU1__OMU_ORD46PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[46].ORDPSA.R /* ORD46 Physical Start Address */
#define RTU1__OMU_ORD46RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[46].ORDRS.R /* ORD46 Region Size */
#define RTU1__OMU_ORD46RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[46].ORDRZA.R /* ORD46 Region Zone Assignment */
#define RTU1__OMU_ORD47LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[47].ORDLSA.R /* ORD47 Logical Start Address */
#define RTU1__OMU_ORD47PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[47].ORDPSA.R /* ORD47 Physical Start Address */
#define RTU1__OMU_ORD47RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[47].ORDRS.R /* ORD47 Region Size */
#define RTU1__OMU_ORD47RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[47].ORDRZA.R /* ORD47 Region Zone Assignment */
#define RTU1__OMU_ORD48LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[48].ORDLSA.R /* ORD48 Logical Start Address */
#define RTU1__OMU_ORD48PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[48].ORDPSA.R /* ORD48 Physical Start Address */
#define RTU1__OMU_ORD48RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[48].ORDRS.R /* ORD48 Region Size */
#define RTU1__OMU_ORD48RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[48].ORDRZA.R /* ORD48 Region Zone Assignment */
#define RTU1__OMU_ORD49LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[49].ORDLSA.R /* ORD49 Logical Start Address */
#define RTU1__OMU_ORD49PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[49].ORDPSA.R /* ORD49 Physical Start Address */
#define RTU1__OMU_ORD49RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[49].ORDRS.R /* ORD49 Region Size */
#define RTU1__OMU_ORD49RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[49].ORDRZA.R /* ORD49 Region Zone Assignment */
#define RTU1__OMU_ORD50LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[50].ORDLSA.R /* ORD50 Logical Start Address */
#define RTU1__OMU_ORD50PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[50].ORDPSA.R /* ORD50 Physical Start Address */
#define RTU1__OMU_ORD50RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[50].ORDRS.R /* ORD50 Region Size */
#define RTU1__OMU_ORD50RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[50].ORDRZA.R /* ORD50 Region Zone Assignment */
#define RTU1__OMU_ORD51LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[51].ORDLSA.R /* ORD51 Logical Start Address */
#define RTU1__OMU_ORD51PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[51].ORDPSA.R /* ORD51 Physical Start Address */
#define RTU1__OMU_ORD51RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[51].ORDRS.R /* ORD51 Region Size */
#define RTU1__OMU_ORD51RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[51].ORDRZA.R /* ORD51 Region Zone Assignment */
#define RTU1__OMU_ORD52LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[52].ORDLSA.R /* ORD52 Logical Start Address */
#define RTU1__OMU_ORD52PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[52].ORDPSA.R /* ORD52 Physical Start Address */
#define RTU1__OMU_ORD52RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[52].ORDRS.R /* ORD52 Region Size */
#define RTU1__OMU_ORD52RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[52].ORDRZA.R /* ORD52 Region Zone Assignment */
#define RTU1__OMU_ORD53LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[53].ORDLSA.R /* ORD53 Logical Start Address */
#define RTU1__OMU_ORD53PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[53].ORDPSA.R /* ORD53 Physical Start Address */
#define RTU1__OMU_ORD53RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[53].ORDRS.R /* ORD53 Region Size */
#define RTU1__OMU_ORD53RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[53].ORDRZA.R /* ORD53 Region Zone Assignment */
#define RTU1__OMU_ORD54LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[54].ORDLSA.R /* ORD54 Logical Start Address */
#define RTU1__OMU_ORD54PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[54].ORDPSA.R /* ORD54 Physical Start Address */
#define RTU1__OMU_ORD54RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[54].ORDRS.R /* ORD54 Region Size */
#define RTU1__OMU_ORD54RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[54].ORDRZA.R /* ORD54 Region Zone Assignment */
#define RTU1__OMU_ORD55LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[55].ORDLSA.R /* ORD55 Logical Start Address */
#define RTU1__OMU_ORD55PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[55].ORDPSA.R /* ORD55 Physical Start Address */
#define RTU1__OMU_ORD55RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[55].ORDRS.R /* ORD55 Region Size */
#define RTU1__OMU_ORD55RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[55].ORDRZA.R /* ORD55 Region Zone Assignment */
#define RTU1__OMU_ORD56LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[56].ORDLSA.R /* ORD56 Logical Start Address */
#define RTU1__OMU_ORD56PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[56].ORDPSA.R /* ORD56 Physical Start Address */
#define RTU1__OMU_ORD56RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[56].ORDRS.R /* ORD56 Region Size */
#define RTU1__OMU_ORD56RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[56].ORDRZA.R /* ORD56 Region Zone Assignment */
#define RTU1__OMU_ORD57LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[57].ORDLSA.R /* ORD57 Logical Start Address */
#define RTU1__OMU_ORD57PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[57].ORDPSA.R /* ORD57 Physical Start Address */
#define RTU1__OMU_ORD57RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[57].ORDRS.R /* ORD57 Region Size */
#define RTU1__OMU_ORD57RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[57].ORDRZA.R /* ORD57 Region Zone Assignment */
#define RTU1__OMU_ORD58LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[58].ORDLSA.R /* ORD58 Logical Start Address */
#define RTU1__OMU_ORD58PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[58].ORDPSA.R /* ORD58 Physical Start Address */
#define RTU1__OMU_ORD58RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[58].ORDRS.R /* ORD58 Region Size */
#define RTU1__OMU_ORD58RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[58].ORDRZA.R /* ORD58 Region Zone Assignment */
#define RTU1__OMU_ORD59LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[59].ORDLSA.R /* ORD59 Logical Start Address */
#define RTU1__OMU_ORD59PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[59].ORDPSA.R /* ORD59 Physical Start Address */
#define RTU1__OMU_ORD59RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[59].ORDRS.R /* ORD59 Region Size */
#define RTU1__OMU_ORD59RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[59].ORDRZA.R /* ORD59 Region Zone Assignment */
#define RTU1__OMU_ORD60LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[60].ORDLSA.R /* ORD60 Logical Start Address */
#define RTU1__OMU_ORD60PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[60].ORDPSA.R /* ORD60 Physical Start Address */
#define RTU1__OMU_ORD60RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[60].ORDRS.R /* ORD60 Region Size */
#define RTU1__OMU_ORD60RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[60].ORDRZA.R /* ORD60 Region Zone Assignment */
#define RTU1__OMU_ORD61LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[61].ORDLSA.R /* ORD61 Logical Start Address */
#define RTU1__OMU_ORD61PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[61].ORDPSA.R /* ORD61 Physical Start Address */
#define RTU1__OMU_ORD61RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[61].ORDRS.R /* ORD61 Region Size */
#define RTU1__OMU_ORD61RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[61].ORDRZA.R /* ORD61 Region Zone Assignment */
#define RTU1__OMU_ORD62LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[62].ORDLSA.R /* ORD62 Logical Start Address */
#define RTU1__OMU_ORD62PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[62].ORDPSA.R /* ORD62 Physical Start Address */
#define RTU1__OMU_ORD62RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[62].ORDRS.R /* ORD62 Region Size */
#define RTU1__OMU_ORD62RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[62].ORDRZA.R /* ORD62 Region Zone Assignment */
#define RTU1__OMU_ORD63LSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[63].ORDLSA.R /* ORD63 Logical Start Address */
#define RTU1__OMU_ORD63PSA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[63].ORDPSA.R /* ORD63 Physical Start Address */
#define RTU1__OMU_ORD63RS    RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[63].ORDRS.R /* ORD63 Region Size */
#define RTU1__OMU_ORD63RZA   RTU1__OMU.OVERLAY_REGION_DESCRIPTOR[63].ORDRZA.R /* ORD63 Region Zone Assignment */

/* PERIPH_PLL */
#define PERIPH_PLL_PLLCR     PERIPH_PLL.PLLCR.R            /* PLL Control */
#define PERIPH_PLL_PLLSR     PERIPH_PLL.PLLSR.R            /* PLL Status */
#define PERIPH_PLL_PLLDV     PERIPH_PLL.PLLDV.R            /* PLL Divider */
#define PERIPH_PLL_PLLFD     PERIPH_PLL.PLLFD.R            /* PLL Fractional Divider */
#define PERIPH_PLL_PLLCLKMUX PERIPH_PLL.PLLCLKMUX.R        /* PLL Clock Multiplexer */
#define PERIPH_PLL_PLLODIV_0 PERIPH_PLL.PLLODIV[0].R       /* PLL Output Divider */
#define PERIPH_PLL_PLLODIV_1 PERIPH_PLL.PLLODIV[1].R       /* PLL Output Divider */
#define PERIPH_PLL_PLLODIV_2 PERIPH_PLL.PLLODIV[2].R       /* PLL Output Divider */
#define PERIPH_PLL_PLLODIV_3 PERIPH_PLL.PLLODIV[3].R       /* PLL Output Divider */
#define PERIPH_PLL_PLLODIV_4 PERIPH_PLL.PLLODIV[4].R       /* PLL Output Divider */
#define PERIPH_PLL_PLLODIV_5 PERIPH_PLL.PLLODIV[5].R       /* PLL Output Divider */
#define PERIPH_PLL_PLLODIV_6 PERIPH_PLL.PLLODIV[6].R       /* PLL Output Divider */

/* PIT */
#define CE_PIT_0_MCR         CE_PIT_0.MCR.R                /* PIT Module Control */
#define CE_PIT_0_LTMR64H     CE_PIT_0.LTMR64H.R            /* PIT Upper Lifetimer */
#define CE_PIT_0_LTMR64L     CE_PIT_0.LTMR64L.R            /* PIT Lower Lifetimer */
#define CE_PIT_0_LDVAL0      CE_PIT_0.TIMER[0].LDVAL.R     /* Timer Load Value */
#define CE_PIT_0_CVAL0       CE_PIT_0.TIMER[0].CVAL.R      /* Current Timer Value */
#define CE_PIT_0_TCTRL0      CE_PIT_0.TIMER[0].TCTRL.R     /* Timer Control */
#define CE_PIT_0_TFLG0       CE_PIT_0.TIMER[0].TFLG.R      /* Timer Flag */
#define CE_PIT_0_LDVAL1      CE_PIT_0.TIMER[1].LDVAL.R     /* Timer Load Value */
#define CE_PIT_0_CVAL1       CE_PIT_0.TIMER[1].CVAL.R      /* Current Timer Value */
#define CE_PIT_0_TCTRL1      CE_PIT_0.TIMER[1].TCTRL.R     /* Timer Control */
#define CE_PIT_0_TFLG1       CE_PIT_0.TIMER[1].TFLG.R      /* Timer Flag */
#define CE_PIT_0_LDVAL2      CE_PIT_0.TIMER[2].LDVAL.R     /* Timer Load Value */
#define CE_PIT_0_CVAL2       CE_PIT_0.TIMER[2].CVAL.R      /* Current Timer Value */
#define CE_PIT_0_TCTRL2      CE_PIT_0.TIMER[2].TCTRL.R     /* Timer Control */
#define CE_PIT_0_TFLG2       CE_PIT_0.TIMER[2].TFLG.R      /* Timer Flag */
#define CE_PIT_0_LDVAL3      CE_PIT_0.TIMER[3].LDVAL.R     /* Timer Load Value */
#define CE_PIT_0_CVAL3       CE_PIT_0.TIMER[3].CVAL.R      /* Current Timer Value */
#define CE_PIT_0_TCTRL3      CE_PIT_0.TIMER[3].TCTRL.R     /* Timer Control */
#define CE_PIT_0_TFLG3       CE_PIT_0.TIMER[3].TFLG.R      /* Timer Flag */

#define CE_PIT_1_MCR         CE_PIT_1.MCR.R                /* PIT Module Control */
#define CE_PIT_1_LTMR64H     CE_PIT_1.LTMR64H.R            /* PIT Upper Lifetimer */
#define CE_PIT_1_LTMR64L     CE_PIT_1.LTMR64L.R            /* PIT Lower Lifetimer */
#define CE_PIT_1_LDVAL0      CE_PIT_1.TIMER[0].LDVAL.R     /* Timer Load Value */
#define CE_PIT_1_CVAL0       CE_PIT_1.TIMER[0].CVAL.R      /* Current Timer Value */
#define CE_PIT_1_TCTRL0      CE_PIT_1.TIMER[0].TCTRL.R     /* Timer Control */
#define CE_PIT_1_TFLG0       CE_PIT_1.TIMER[0].TFLG.R      /* Timer Flag */
#define CE_PIT_1_LDVAL1      CE_PIT_1.TIMER[1].LDVAL.R     /* Timer Load Value */
#define CE_PIT_1_CVAL1       CE_PIT_1.TIMER[1].CVAL.R      /* Current Timer Value */
#define CE_PIT_1_TCTRL1      CE_PIT_1.TIMER[1].TCTRL.R     /* Timer Control */
#define CE_PIT_1_TFLG1       CE_PIT_1.TIMER[1].TFLG.R      /* Timer Flag */
#define CE_PIT_1_LDVAL2      CE_PIT_1.TIMER[2].LDVAL.R     /* Timer Load Value */
#define CE_PIT_1_CVAL2       CE_PIT_1.TIMER[2].CVAL.R      /* Current Timer Value */
#define CE_PIT_1_TCTRL2      CE_PIT_1.TIMER[2].TCTRL.R     /* Timer Control */
#define CE_PIT_1_TFLG2       CE_PIT_1.TIMER[2].TFLG.R      /* Timer Flag */
#define CE_PIT_1_LDVAL3      CE_PIT_1.TIMER[3].LDVAL.R     /* Timer Load Value */
#define CE_PIT_1_CVAL3       CE_PIT_1.TIMER[3].CVAL.R      /* Current Timer Value */
#define CE_PIT_1_TCTRL3      CE_PIT_1.TIMER[3].TCTRL.R     /* Timer Control */
#define CE_PIT_1_TFLG3       CE_PIT_1.TIMER[3].TFLG.R      /* Timer Flag */

#define CE_PIT_2_MCR         CE_PIT_2.MCR.R                /* PIT Module Control */
#define CE_PIT_2_LTMR64H     CE_PIT_2.LTMR64H.R            /* PIT Upper Lifetimer */
#define CE_PIT_2_LTMR64L     CE_PIT_2.LTMR64L.R            /* PIT Lower Lifetimer */
#define CE_PIT_2_LDVAL0      CE_PIT_2.TIMER[0].LDVAL.R     /* Timer Load Value */
#define CE_PIT_2_CVAL0       CE_PIT_2.TIMER[0].CVAL.R      /* Current Timer Value */
#define CE_PIT_2_TCTRL0      CE_PIT_2.TIMER[0].TCTRL.R     /* Timer Control */
#define CE_PIT_2_TFLG0       CE_PIT_2.TIMER[0].TFLG.R      /* Timer Flag */
#define CE_PIT_2_LDVAL1      CE_PIT_2.TIMER[1].LDVAL.R     /* Timer Load Value */
#define CE_PIT_2_CVAL1       CE_PIT_2.TIMER[1].CVAL.R      /* Current Timer Value */
#define CE_PIT_2_TCTRL1      CE_PIT_2.TIMER[1].TCTRL.R     /* Timer Control */
#define CE_PIT_2_TFLG1       CE_PIT_2.TIMER[1].TFLG.R      /* Timer Flag */
#define CE_PIT_2_LDVAL2      CE_PIT_2.TIMER[2].LDVAL.R     /* Timer Load Value */
#define CE_PIT_2_CVAL2       CE_PIT_2.TIMER[2].CVAL.R      /* Current Timer Value */
#define CE_PIT_2_TCTRL2      CE_PIT_2.TIMER[2].TCTRL.R     /* Timer Control */
#define CE_PIT_2_TFLG2       CE_PIT_2.TIMER[2].TFLG.R      /* Timer Flag */
#define CE_PIT_2_LDVAL3      CE_PIT_2.TIMER[3].LDVAL.R     /* Timer Load Value */
#define CE_PIT_2_CVAL3       CE_PIT_2.TIMER[3].CVAL.R      /* Current Timer Value */
#define CE_PIT_2_TCTRL3      CE_PIT_2.TIMER[3].TCTRL.R     /* Timer Control */
#define CE_PIT_2_TFLG3       CE_PIT_2.TIMER[3].TFLG.R      /* Timer Flag */

#define CE_PIT_3_MCR         CE_PIT_3.MCR.R                /* PIT Module Control */
#define CE_PIT_3_LTMR64H     CE_PIT_3.LTMR64H.R            /* PIT Upper Lifetimer */
#define CE_PIT_3_LTMR64L     CE_PIT_3.LTMR64L.R            /* PIT Lower Lifetimer */
#define CE_PIT_3_LDVAL0      CE_PIT_3.TIMER[0].LDVAL.R     /* Timer Load Value */
#define CE_PIT_3_CVAL0       CE_PIT_3.TIMER[0].CVAL.R      /* Current Timer Value */
#define CE_PIT_3_TCTRL0      CE_PIT_3.TIMER[0].TCTRL.R     /* Timer Control */
#define CE_PIT_3_TFLG0       CE_PIT_3.TIMER[0].TFLG.R      /* Timer Flag */
#define CE_PIT_3_LDVAL1      CE_PIT_3.TIMER[1].LDVAL.R     /* Timer Load Value */
#define CE_PIT_3_CVAL1       CE_PIT_3.TIMER[1].CVAL.R      /* Current Timer Value */
#define CE_PIT_3_TCTRL1      CE_PIT_3.TIMER[1].TCTRL.R     /* Timer Control */
#define CE_PIT_3_TFLG1       CE_PIT_3.TIMER[1].TFLG.R      /* Timer Flag */
#define CE_PIT_3_LDVAL2      CE_PIT_3.TIMER[2].LDVAL.R     /* Timer Load Value */
#define CE_PIT_3_CVAL2       CE_PIT_3.TIMER[2].CVAL.R      /* Current Timer Value */
#define CE_PIT_3_TCTRL2      CE_PIT_3.TIMER[2].TCTRL.R     /* Timer Control */
#define CE_PIT_3_TFLG2       CE_PIT_3.TIMER[2].TFLG.R      /* Timer Flag */
#define CE_PIT_3_LDVAL3      CE_PIT_3.TIMER[3].LDVAL.R     /* Timer Load Value */
#define CE_PIT_3_CVAL3       CE_PIT_3.TIMER[3].CVAL.R      /* Current Timer Value */
#define CE_PIT_3_TCTRL3      CE_PIT_3.TIMER[3].TCTRL.R     /* Timer Control */
#define CE_PIT_3_TFLG3       CE_PIT_3.TIMER[3].TFLG.R      /* Timer Flag */

#define CE_PIT_4_MCR         CE_PIT_4.MCR.R                /* PIT Module Control */
#define CE_PIT_4_LTMR64H     CE_PIT_4.LTMR64H.R            /* PIT Upper Lifetimer */
#define CE_PIT_4_LTMR64L     CE_PIT_4.LTMR64L.R            /* PIT Lower Lifetimer */
#define CE_PIT_4_LDVAL0      CE_PIT_4.TIMER[0].LDVAL.R     /* Timer Load Value */
#define CE_PIT_4_CVAL0       CE_PIT_4.TIMER[0].CVAL.R      /* Current Timer Value */
#define CE_PIT_4_TCTRL0      CE_PIT_4.TIMER[0].TCTRL.R     /* Timer Control */
#define CE_PIT_4_TFLG0       CE_PIT_4.TIMER[0].TFLG.R      /* Timer Flag */
#define CE_PIT_4_LDVAL1      CE_PIT_4.TIMER[1].LDVAL.R     /* Timer Load Value */
#define CE_PIT_4_CVAL1       CE_PIT_4.TIMER[1].CVAL.R      /* Current Timer Value */
#define CE_PIT_4_TCTRL1      CE_PIT_4.TIMER[1].TCTRL.R     /* Timer Control */
#define CE_PIT_4_TFLG1       CE_PIT_4.TIMER[1].TFLG.R      /* Timer Flag */
#define CE_PIT_4_LDVAL2      CE_PIT_4.TIMER[2].LDVAL.R     /* Timer Load Value */
#define CE_PIT_4_CVAL2       CE_PIT_4.TIMER[2].CVAL.R      /* Current Timer Value */
#define CE_PIT_4_TCTRL2      CE_PIT_4.TIMER[2].TCTRL.R     /* Timer Control */
#define CE_PIT_4_TFLG2       CE_PIT_4.TIMER[2].TFLG.R      /* Timer Flag */
#define CE_PIT_4_LDVAL3      CE_PIT_4.TIMER[3].LDVAL.R     /* Timer Load Value */
#define CE_PIT_4_CVAL3       CE_PIT_4.TIMER[3].CVAL.R      /* Current Timer Value */
#define CE_PIT_4_TCTRL3      CE_PIT_4.TIMER[3].TCTRL.R     /* Timer Control */
#define CE_PIT_4_TFLG3       CE_PIT_4.TIMER[3].TFLG.R      /* Timer Flag */

#define CE_PIT_5_MCR         CE_PIT_5.MCR.R                /* PIT Module Control */
#define CE_PIT_5_LTMR64H     CE_PIT_5.LTMR64H.R            /* PIT Upper Lifetimer */
#define CE_PIT_5_LTMR64L     CE_PIT_5.LTMR64L.R            /* PIT Lower Lifetimer */
#define CE_PIT_5_LDVAL0      CE_PIT_5.TIMER[0].LDVAL.R     /* Timer Load Value */
#define CE_PIT_5_CVAL0       CE_PIT_5.TIMER[0].CVAL.R      /* Current Timer Value */
#define CE_PIT_5_TCTRL0      CE_PIT_5.TIMER[0].TCTRL.R     /* Timer Control */
#define CE_PIT_5_TFLG0       CE_PIT_5.TIMER[0].TFLG.R      /* Timer Flag */
#define CE_PIT_5_LDVAL1      CE_PIT_5.TIMER[1].LDVAL.R     /* Timer Load Value */
#define CE_PIT_5_CVAL1       CE_PIT_5.TIMER[1].CVAL.R      /* Current Timer Value */
#define CE_PIT_5_TCTRL1      CE_PIT_5.TIMER[1].TCTRL.R     /* Timer Control */
#define CE_PIT_5_TFLG1       CE_PIT_5.TIMER[1].TFLG.R      /* Timer Flag */
#define CE_PIT_5_LDVAL2      CE_PIT_5.TIMER[2].LDVAL.R     /* Timer Load Value */
#define CE_PIT_5_CVAL2       CE_PIT_5.TIMER[2].CVAL.R      /* Current Timer Value */
#define CE_PIT_5_TCTRL2      CE_PIT_5.TIMER[2].TCTRL.R     /* Timer Control */
#define CE_PIT_5_TFLG2       CE_PIT_5.TIMER[2].TFLG.R      /* Timer Flag */
#define CE_PIT_5_LDVAL3      CE_PIT_5.TIMER[3].LDVAL.R     /* Timer Load Value */
#define CE_PIT_5_CVAL3       CE_PIT_5.TIMER[3].CVAL.R      /* Current Timer Value */
#define CE_PIT_5_TCTRL3      CE_PIT_5.TIMER[3].TCTRL.R     /* Timer Control */
#define CE_PIT_5_TFLG3       CE_PIT_5.TIMER[3].TFLG.R      /* Timer Flag */

#define PIT_1_MCR            PIT_1.MCR.R                   /* PIT Module Control */
#define PIT_1_LTMR64H        PIT_1.LTMR64H.R               /* PIT Upper Lifetimer */
#define PIT_1_LTMR64L        PIT_1.LTMR64L.R               /* PIT Lower Lifetimer */
#define PIT_1_LDVAL0         PIT_1.TIMER[0].LDVAL.R        /* Timer Load Value */
#define PIT_1_CVAL0          PIT_1.TIMER[0].CVAL.R         /* Current Timer Value */
#define PIT_1_TCTRL0         PIT_1.TIMER[0].TCTRL.R        /* Timer Control */
#define PIT_1_TFLG0          PIT_1.TIMER[0].TFLG.R         /* Timer Flag */
#define PIT_1_LDVAL1         PIT_1.TIMER[1].LDVAL.R        /* Timer Load Value */
#define PIT_1_CVAL1          PIT_1.TIMER[1].CVAL.R         /* Current Timer Value */
#define PIT_1_TCTRL1         PIT_1.TIMER[1].TCTRL.R        /* Timer Control */
#define PIT_1_TFLG1          PIT_1.TIMER[1].TFLG.R         /* Timer Flag */
#define PIT_1_LDVAL2         PIT_1.TIMER[2].LDVAL.R        /* Timer Load Value */
#define PIT_1_CVAL2          PIT_1.TIMER[2].CVAL.R         /* Current Timer Value */
#define PIT_1_TCTRL2         PIT_1.TIMER[2].TCTRL.R        /* Timer Control */
#define PIT_1_TFLG2          PIT_1.TIMER[2].TFLG.R         /* Timer Flag */
#define PIT_1_LDVAL3         PIT_1.TIMER[3].LDVAL.R        /* Timer Load Value */
#define PIT_1_CVAL3          PIT_1.TIMER[3].CVAL.R         /* Current Timer Value */
#define PIT_1_TCTRL3         PIT_1.TIMER[3].TCTRL.R        /* Timer Control */
#define PIT_1_TFLG3          PIT_1.TIMER[3].TFLG.R         /* Timer Flag */
#define PIT_1_LDVAL4         PIT_1.TIMER[4].LDVAL.R        /* Timer Load Value */
#define PIT_1_CVAL4          PIT_1.TIMER[4].CVAL.R         /* Current Timer Value */
#define PIT_1_TCTRL4         PIT_1.TIMER[4].TCTRL.R        /* Timer Control */
#define PIT_1_TFLG4          PIT_1.TIMER[4].TFLG.R         /* Timer Flag */

#define PIT_0_MCR            PIT_0.MCR.R                   /* PIT Module Control */
#define PIT_0_LTMR64H        PIT_0.LTMR64H.R               /* PIT Upper Lifetimer */
#define PIT_0_LTMR64L        PIT_0.LTMR64L.R               /* PIT Lower Lifetimer */
#define PIT_0_LDVAL0         PIT_0.TIMER[0].LDVAL.R        /* Timer Load Value */
#define PIT_0_CVAL0          PIT_0.TIMER[0].CVAL.R         /* Current Timer Value */
#define PIT_0_TCTRL0         PIT_0.TIMER[0].TCTRL.R        /* Timer Control */
#define PIT_0_TFLG0          PIT_0.TIMER[0].TFLG.R         /* Timer Flag */
#define PIT_0_LDVAL1         PIT_0.TIMER[1].LDVAL.R        /* Timer Load Value */
#define PIT_0_CVAL1          PIT_0.TIMER[1].CVAL.R         /* Current Timer Value */
#define PIT_0_TCTRL1         PIT_0.TIMER[1].TCTRL.R        /* Timer Control */
#define PIT_0_TFLG1          PIT_0.TIMER[1].TFLG.R         /* Timer Flag */
#define PIT_0_LDVAL2         PIT_0.TIMER[2].LDVAL.R        /* Timer Load Value */
#define PIT_0_CVAL2          PIT_0.TIMER[2].CVAL.R         /* Current Timer Value */
#define PIT_0_TCTRL2         PIT_0.TIMER[2].TCTRL.R        /* Timer Control */
#define PIT_0_TFLG2          PIT_0.TIMER[2].TFLG.R         /* Timer Flag */
#define PIT_0_LDVAL3         PIT_0.TIMER[3].LDVAL.R        /* Timer Load Value */
#define PIT_0_CVAL3          PIT_0.TIMER[3].CVAL.R         /* Current Timer Value */
#define PIT_0_TCTRL3         PIT_0.TIMER[3].TCTRL.R        /* Timer Control */
#define PIT_0_TFLG3          PIT_0.TIMER[3].TFLG.R         /* Timer Flag */
#define PIT_0_LDVAL4         PIT_0.TIMER[4].LDVAL.R        /* Timer Load Value */
#define PIT_0_CVAL4          PIT_0.TIMER[4].CVAL.R         /* Current Timer Value */
#define PIT_0_TCTRL4         PIT_0.TIMER[4].TCTRL.R        /* Timer Control */
#define PIT_0_TFLG4          PIT_0.TIMER[4].TFLG.R         /* Timer Flag */
#define PIT_0_LDVAL5         PIT_0.TIMER[5].LDVAL.R        /* Timer Load Value */
#define PIT_0_CVAL5          PIT_0.TIMER[5].CVAL.R         /* Current Timer Value */
#define PIT_0_TCTRL5         PIT_0.TIMER[5].TCTRL.R        /* Timer Control */
#define PIT_0_TFLG5          PIT_0.TIMER[5].TFLG.R         /* Timer Flag */
#define PIT_0_LDVAL6         PIT_0.TIMER[6].LDVAL.R        /* Timer Load Value */
#define PIT_0_CVAL6          PIT_0.TIMER[6].CVAL.R         /* Current Timer Value */
#define PIT_0_TCTRL6         PIT_0.TIMER[6].TCTRL.R        /* Timer Control */
#define PIT_0_TFLG6          PIT_0.TIMER[6].TFLG.R         /* Timer Flag */

#define PIT_4_MCR            PIT_4.MCR.R                   /* PIT Module Control */
#define PIT_4_LTMR64H        PIT_4.LTMR64H.R               /* PIT Upper Lifetimer */
#define PIT_4_LTMR64L        PIT_4.LTMR64L.R               /* PIT Lower Lifetimer */
#define PIT_4_LDVAL0         PIT_4.TIMER[0].LDVAL.R        /* Timer Load Value */
#define PIT_4_CVAL0          PIT_4.TIMER[0].CVAL.R         /* Current Timer Value */
#define PIT_4_TCTRL0         PIT_4.TIMER[0].TCTRL.R        /* Timer Control */
#define PIT_4_TFLG0          PIT_4.TIMER[0].TFLG.R         /* Timer Flag */
#define PIT_4_LDVAL1         PIT_4.TIMER[1].LDVAL.R        /* Timer Load Value */
#define PIT_4_CVAL1          PIT_4.TIMER[1].CVAL.R         /* Current Timer Value */
#define PIT_4_TCTRL1         PIT_4.TIMER[1].TCTRL.R        /* Timer Control */
#define PIT_4_TFLG1          PIT_4.TIMER[1].TFLG.R         /* Timer Flag */
#define PIT_4_LDVAL2         PIT_4.TIMER[2].LDVAL.R        /* Timer Load Value */
#define PIT_4_CVAL2          PIT_4.TIMER[2].CVAL.R         /* Current Timer Value */
#define PIT_4_TCTRL2         PIT_4.TIMER[2].TCTRL.R        /* Timer Control */
#define PIT_4_TFLG2          PIT_4.TIMER[2].TFLG.R         /* Timer Flag */
#define PIT_4_LDVAL3         PIT_4.TIMER[3].LDVAL.R        /* Timer Load Value */
#define PIT_4_CVAL3          PIT_4.TIMER[3].CVAL.R         /* Current Timer Value */
#define PIT_4_TCTRL3         PIT_4.TIMER[3].TCTRL.R        /* Timer Control */
#define PIT_4_TFLG3          PIT_4.TIMER[3].TFLG.R         /* Timer Flag */
#define PIT_4_LDVAL4         PIT_4.TIMER[4].LDVAL.R        /* Timer Load Value */
#define PIT_4_CVAL4          PIT_4.TIMER[4].CVAL.R         /* Current Timer Value */
#define PIT_4_TCTRL4         PIT_4.TIMER[4].TCTRL.R        /* Timer Control */
#define PIT_4_TFLG4          PIT_4.TIMER[4].TFLG.R         /* Timer Flag */

#define RTU0__PIT_MCR        RTU0__PIT.MCR.R               /* PIT Module Control */
#define RTU0__PIT_LTMR64H    RTU0__PIT.LTMR64H.R           /* PIT Upper Lifetimer */
#define RTU0__PIT_LTMR64L    RTU0__PIT.LTMR64L.R           /* PIT Lower Lifetimer */
#define RTU0__PIT_LDVAL0     RTU0__PIT.TIMER[0].LDVAL.R    /* Timer Load Value */
#define RTU0__PIT_CVAL0      RTU0__PIT.TIMER[0].CVAL.R     /* Current Timer Value */
#define RTU0__PIT_TCTRL0     RTU0__PIT.TIMER[0].TCTRL.R    /* Timer Control */
#define RTU0__PIT_TFLG0      RTU0__PIT.TIMER[0].TFLG.R     /* Timer Flag */
#define RTU0__PIT_LDVAL1     RTU0__PIT.TIMER[1].LDVAL.R    /* Timer Load Value */
#define RTU0__PIT_CVAL1      RTU0__PIT.TIMER[1].CVAL.R     /* Current Timer Value */
#define RTU0__PIT_TCTRL1     RTU0__PIT.TIMER[1].TCTRL.R    /* Timer Control */
#define RTU0__PIT_TFLG1      RTU0__PIT.TIMER[1].TFLG.R     /* Timer Flag */
#define RTU0__PIT_LDVAL2     RTU0__PIT.TIMER[2].LDVAL.R    /* Timer Load Value */
#define RTU0__PIT_CVAL2      RTU0__PIT.TIMER[2].CVAL.R     /* Current Timer Value */
#define RTU0__PIT_TCTRL2     RTU0__PIT.TIMER[2].TCTRL.R    /* Timer Control */
#define RTU0__PIT_TFLG2      RTU0__PIT.TIMER[2].TFLG.R     /* Timer Flag */
#define RTU0__PIT_LDVAL3     RTU0__PIT.TIMER[3].LDVAL.R    /* Timer Load Value */
#define RTU0__PIT_CVAL3      RTU0__PIT.TIMER[3].CVAL.R     /* Current Timer Value */
#define RTU0__PIT_TCTRL3     RTU0__PIT.TIMER[3].TCTRL.R    /* Timer Control */
#define RTU0__PIT_TFLG3      RTU0__PIT.TIMER[3].TFLG.R     /* Timer Flag */
#define RTU0__PIT_LDVAL4     RTU0__PIT.TIMER[4].LDVAL.R    /* Timer Load Value */
#define RTU0__PIT_CVAL4      RTU0__PIT.TIMER[4].CVAL.R     /* Current Timer Value */
#define RTU0__PIT_TCTRL4     RTU0__PIT.TIMER[4].TCTRL.R    /* Timer Control */
#define RTU0__PIT_TFLG4      RTU0__PIT.TIMER[4].TFLG.R     /* Timer Flag */
#define RTU0__PIT_LDVAL5     RTU0__PIT.TIMER[5].LDVAL.R    /* Timer Load Value */
#define RTU0__PIT_CVAL5      RTU0__PIT.TIMER[5].CVAL.R     /* Current Timer Value */
#define RTU0__PIT_TCTRL5     RTU0__PIT.TIMER[5].TCTRL.R    /* Timer Control */
#define RTU0__PIT_TFLG5      RTU0__PIT.TIMER[5].TFLG.R     /* Timer Flag */

#define RTU1__PIT_MCR        RTU1__PIT.MCR.R               /* PIT Module Control */
#define RTU1__PIT_LTMR64H    RTU1__PIT.LTMR64H.R           /* PIT Upper Lifetimer */
#define RTU1__PIT_LTMR64L    RTU1__PIT.LTMR64L.R           /* PIT Lower Lifetimer */
#define RTU1__PIT_LDVAL0     RTU1__PIT.TIMER[0].LDVAL.R    /* Timer Load Value */
#define RTU1__PIT_CVAL0      RTU1__PIT.TIMER[0].CVAL.R     /* Current Timer Value */
#define RTU1__PIT_TCTRL0     RTU1__PIT.TIMER[0].TCTRL.R    /* Timer Control */
#define RTU1__PIT_TFLG0      RTU1__PIT.TIMER[0].TFLG.R     /* Timer Flag */
#define RTU1__PIT_LDVAL1     RTU1__PIT.TIMER[1].LDVAL.R    /* Timer Load Value */
#define RTU1__PIT_CVAL1      RTU1__PIT.TIMER[1].CVAL.R     /* Current Timer Value */
#define RTU1__PIT_TCTRL1     RTU1__PIT.TIMER[1].TCTRL.R    /* Timer Control */
#define RTU1__PIT_TFLG1      RTU1__PIT.TIMER[1].TFLG.R     /* Timer Flag */
#define RTU1__PIT_LDVAL2     RTU1__PIT.TIMER[2].LDVAL.R    /* Timer Load Value */
#define RTU1__PIT_CVAL2      RTU1__PIT.TIMER[2].CVAL.R     /* Current Timer Value */
#define RTU1__PIT_TCTRL2     RTU1__PIT.TIMER[2].TCTRL.R    /* Timer Control */
#define RTU1__PIT_TFLG2      RTU1__PIT.TIMER[2].TFLG.R     /* Timer Flag */
#define RTU1__PIT_LDVAL3     RTU1__PIT.TIMER[3].LDVAL.R    /* Timer Load Value */
#define RTU1__PIT_CVAL3      RTU1__PIT.TIMER[3].CVAL.R     /* Current Timer Value */
#define RTU1__PIT_TCTRL3     RTU1__PIT.TIMER[3].TCTRL.R    /* Timer Control */
#define RTU1__PIT_TFLG3      RTU1__PIT.TIMER[3].TFLG.R     /* Timer Flag */
#define RTU1__PIT_LDVAL4     RTU1__PIT.TIMER[4].LDVAL.R    /* Timer Load Value */
#define RTU1__PIT_CVAL4      RTU1__PIT.TIMER[4].CVAL.R     /* Current Timer Value */
#define RTU1__PIT_TCTRL4     RTU1__PIT.TIMER[4].TCTRL.R    /* Timer Control */
#define RTU1__PIT_TFLG4      RTU1__PIT.TIMER[4].TFLG.R     /* Timer Flag */
#define RTU1__PIT_LDVAL5     RTU1__PIT.TIMER[5].LDVAL.R    /* Timer Load Value */
#define RTU1__PIT_CVAL5      RTU1__PIT.TIMER[5].CVAL.R     /* Current Timer Value */
#define RTU1__PIT_TCTRL5     RTU1__PIT.TIMER[5].TCTRL.R    /* Timer Control */
#define RTU1__PIT_TFLG5      RTU1__PIT.TIMER[5].TFLG.R     /* Timer Flag */

#define PIT_5_MCR            PIT_5.MCR.R                   /* PIT Module Control */
#define PIT_5_LTMR64H        PIT_5.LTMR64H.R               /* PIT Upper Lifetimer */
#define PIT_5_LTMR64L        PIT_5.LTMR64L.R               /* PIT Lower Lifetimer */
#define PIT_5_LDVAL0         PIT_5.TIMER[0].LDVAL.R        /* Timer Load Value */
#define PIT_5_CVAL0          PIT_5.TIMER[0].CVAL.R         /* Current Timer Value */
#define PIT_5_TCTRL0         PIT_5.TIMER[0].TCTRL.R        /* Timer Control */
#define PIT_5_TFLG0          PIT_5.TIMER[0].TFLG.R         /* Timer Flag */
#define PIT_5_LDVAL1         PIT_5.TIMER[1].LDVAL.R        /* Timer Load Value */
#define PIT_5_CVAL1          PIT_5.TIMER[1].CVAL.R         /* Current Timer Value */
#define PIT_5_TCTRL1         PIT_5.TIMER[1].TCTRL.R        /* Timer Control */
#define PIT_5_TFLG1          PIT_5.TIMER[1].TFLG.R         /* Timer Flag */
#define PIT_5_LDVAL2         PIT_5.TIMER[2].LDVAL.R        /* Timer Load Value */
#define PIT_5_CVAL2          PIT_5.TIMER[2].CVAL.R         /* Current Timer Value */
#define PIT_5_TCTRL2         PIT_5.TIMER[2].TCTRL.R        /* Timer Control */
#define PIT_5_TFLG2          PIT_5.TIMER[2].TFLG.R         /* Timer Flag */
#define PIT_5_LDVAL3         PIT_5.TIMER[3].LDVAL.R        /* Timer Load Value */
#define PIT_5_CVAL3          PIT_5.TIMER[3].CVAL.R         /* Current Timer Value */
#define PIT_5_TCTRL3         PIT_5.TIMER[3].TCTRL.R        /* Timer Control */
#define PIT_5_TFLG3          PIT_5.TIMER[3].TFLG.R         /* Timer Flag */
#define PIT_5_LDVAL4         PIT_5.TIMER[4].LDVAL.R        /* Timer Load Value */
#define PIT_5_CVAL4          PIT_5.TIMER[4].CVAL.R         /* Current Timer Value */
#define PIT_5_TCTRL4         PIT_5.TIMER[4].TCTRL.R        /* Timer Control */
#define PIT_5_TFLG4          PIT_5.TIMER[4].TFLG.R         /* Timer Flag */

/* PMC */
#define PMC_SSR              PMC.SSR.R                     /* Source of System Reset */
#define PMC_NCSPD_CTL        PMC.NCSPD_CTL.R               /* Non-Critical Supply Presence Detector Control */
#define PMC_NCSPD_STAT       PMC.NCSPD_STAT.R              /* NCSPD Status Register */
#define PMC_CSPD_EVENT_CAPTURE PMC.CSPD_EVENT_CAPTURE.R    /* CSPD Event Capture (CSPDEF) */
#define PMC_NCSPD_EVENT_CAPTURE PMC.NCSPD_EVENT_CAPTURE.R  /* NCSPD Event Flag (NCSPDEF) */
#define PMC_POR_WDOG_EVENT_CAPTURE0 PMC.POR_WDOG_EVENT_CAPTURE0.R /* Device Status Flag (DSF0) */
#define PMC_POR_WDOG_EVENT_CAPTURE1 PMC.POR_WDOG_EVENT_CAPTURE1.R /* Device Status Flag (DSF1) */
#define PMC_POR_WDOG_EVENT_CAPTURE2 PMC.POR_WDOG_EVENT_CAPTURE2.R /* Device Status Flag (DSF2) */

/* PMSS_SAFETY */
#define CEVA_SPF2__PMSS_SAFETY_P_UCSERR CEVA_SPF2__PMSS_SAFETY.P_UCSERR.R /* PMSS UCS ERR Register */
#define CEVA_SPF2__PMSS_SAFETY_P_CSERR CEVA_SPF2__PMSS_SAFETY.P_CSERR.R /* PMSS CS ERR Register */
#define CEVA_SPF2__PMSS_SAFETY_P_UCSERR_S CEVA_SPF2__PMSS_SAFETY.P_UCSERR_S.R /* PMSS UCS ERR Shadow Register */
#define CEVA_SPF2__PMSS_SAFETY_P_CSERR_S CEVA_SPF2__PMSS_SAFETY.P_CSERR_S.R /* PMSS CE ERR Shadow Register */
#define CEVA_SPF2__PMSS_SAFETY_P_CSERR_M CEVA_SPF2__PMSS_SAFETY.P_CSERR_M.R /* PMSS CS ERR Mask Register */
#define CEVA_SPF2__PMSS_SAFETY_P_ECC_TCFG CEVA_SPF2__PMSS_SAFETY.P_ECC_TCFG.R /* PMSS ECC TCFG Register */
#define CEVA_SPF2__PMSS_SAFETY_P_ECC_TCFG_2 CEVA_SPF2__PMSS_SAFETY.P_ECC_TCFG_2.R /* PMSS ECC TCFG 2 Register */
#define CEVA_SPF2__PMSS_SAFETY_P_CTECC_TCFG CEVA_SPF2__PMSS_SAFETY.P_CTECC_TCFG.R /* PMSS CT ECC TCFG Register */
#define CEVA_SPF2__PMSS_SAFETY_P_BTBECC_TCFG CEVA_SPF2__PMSS_SAFETY.P_BTBECC_TCFG.R /* PMSS BTB ECC TCFG Register */
#define CEVA_SPF2__PMSS_SAFETY_P_UCSERR_CNT CEVA_SPF2__PMSS_SAFETY.P_UCSERR_CNT.R /* PMSS UCSERR CNT Register */
#define CEVA_SPF2__PMSS_SAFETY_P_CSERR_CNT CEVA_SPF2__PMSS_SAFETY.P_CSERR_CNT.R /* PMSS CSERR CNT Register */
#define CEVA_SPF2__PMSS_SAFETY_P_LPCS CEVA_SPF2__PMSS_SAFETY.P_LPCS.R /* PMSS LPCS Register */

/* POWER_SCALING_UNIT */
#define CEVA_SPF2__POWER_SCALING_UNIT_PSVM CEVA_SPF2__POWER_SCALING_UNIT.PSVM.R /* PSU Power Save Mode Register */
#define CEVA_SPF2__POWER_SCALING_UNIT_PGR CEVA_SPF2__POWER_SCALING_UNIT.PGR.R /* PSU General Register */

/* PROFILER */
#define CEVA_SPF2__PROFILER_PROF_CTL CEVA_SPF2__PROFILER.PROF_CTL.R /* PROF CTL Register */
#define CEVA_SPF2__PROFILER_PROF_PAUSE CEVA_SPF2__PROFILER.PROF_PAUSE.R /* PROF PAUSE Register */
#define CEVA_SPF2__PROFILER_PROF_CTRL0 CEVA_SPF2__PROFILER.PROF_CTRL0.R /* PROF CTRL0 Register */
#define CEVA_SPF2__PROFILER_PROF_CTRL1 CEVA_SPF2__PROFILER.PROF_CTRL1.R /* PROF CTRL1 Register */
#define CEVA_SPF2__PROFILER_FRCC CEVA_SPF2__PROFILER.FRCC.R /* FRCC Register */
#define CEVA_SPF2__PROFILER_PROF_CNTx0 CEVA_SPF2__PROFILER.PROF_CNTX[0].R /* PROF CNTx Register */
#define CEVA_SPF2__PROFILER_PROF_CNTx1 CEVA_SPF2__PROFILER.PROF_CNTX[1].R /* PROF CNTx Register */
#define CEVA_SPF2__PROFILER_PROF_CNTx2 CEVA_SPF2__PROFILER.PROF_CNTX[2].R /* PROF CNTx Register */
#define CEVA_SPF2__PROFILER_PROF_CNTx3 CEVA_SPF2__PROFILER.PROF_CNTX[3].R /* PROF CNTx Register */
#define CEVA_SPF2__PROFILER_PROF_CNTx4 CEVA_SPF2__PROFILER.PROF_CNTX[4].R /* PROF CNTx Register */
#define CEVA_SPF2__PROFILER_PROF_CNTx5 CEVA_SPF2__PROFILER.PROF_CNTX[5].R /* PROF CNTx Register */
#define CEVA_SPF2__PROFILER_PROF_CNTx6 CEVA_SPF2__PROFILER.PROF_CNTX[6].R /* PROF CNTx Register */
#define CEVA_SPF2__PROFILER_PROF_CNTx7 CEVA_SPF2__PROFILER.PROF_CNTX[7].R /* PROF CNTx Register */
#define CEVA_SPF2__PROFILER_PROF_EV_MASKx0 CEVA_SPF2__PROFILER.PROF_EV_MASKX[0].R /* PROF EV MASKx Register */
#define CEVA_SPF2__PROFILER_PROF_EV_MASKx1 CEVA_SPF2__PROFILER.PROF_EV_MASKX[1].R /* PROF EV MASKx Register */
#define CEVA_SPF2__PROFILER_PROF_EV_MASKx2 CEVA_SPF2__PROFILER.PROF_EV_MASKX[2].R /* PROF EV MASKx Register */
#define CEVA_SPF2__PROFILER_PROF_EV_MASKx3 CEVA_SPF2__PROFILER.PROF_EV_MASKX[3].R /* PROF EV MASKx Register */
#define CEVA_SPF2__PROFILER_PROF_EV_MASKx4 CEVA_SPF2__PROFILER.PROF_EV_MASKX[4].R /* PROF EV MASKx Register */
#define CEVA_SPF2__PROFILER_PROF_EV_MASKx5 CEVA_SPF2__PROFILER.PROF_EV_MASKX[5].R /* PROF EV MASKx Register */
#define CEVA_SPF2__PROFILER_PROF_EV_MASKx6 CEVA_SPF2__PROFILER.PROF_EV_MASKX[6].R /* PROF EV MASKx Register */
#define CEVA_SPF2__PROFILER_PROF_EV_MASKx7 CEVA_SPF2__PROFILER.PROF_EV_MASKX[7].R /* PROF EV MASKx Register */

/* PROGRAM_MEMORY_SUBSYSTEM */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_MSS_PCR CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.MSS_PCR.R /* Program Control */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_SYS_OU_C CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_SYS_OU_C.R /* PMSS AXI Outstanding Configuration */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_HDCFG CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_HDCFG.R /* PMSS Hardware Configuration */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_PORT_ID CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_PORT_ID.R /* Program Port AXI ID */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD0_START CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD0_START.R /* PMSS ADD0 START */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD0_ATT0 CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD0_ATT0.R /* PMSS ADD0 ATT0 */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD1_START CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD1_START.R /* PMSS ADD1 START */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD1_ATT0 CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD1_ATT0.R /* PMSS ADD1 ATT0 */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD2_START CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD2_START.R /* PMSS ADD2 START */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD2_ATT0 CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD2_ATT0.R /* PMSS ADD2 ATT0 */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD3_START CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD3_START.R /* PMSS ADD3 START */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD3_ATT0 CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD3_ATT0.R /* PMSS ADD3 ATT0 */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD4_START CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD4_START.R /* PMSS ADD4 START */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD4_ATT0 CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD4_ATT0.R /* PMSS ADD4 ATT0 */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD5_START CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD5_START.R /* PMSS ADD5 START */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD5_ATT0 CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD5_ATT0.R /* PMSS ADD5 ATT0 */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD6_START CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD6_START.R /* PMSS ADD6 START */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD6_ATT0 CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD6_ATT0.R /* PMSS ADD6 ATT0 */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD7_START CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD7_START.R /* PMSS ADD7 START */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ADD7_ATT0 CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ADD7_ATT0.R /* PMSS ADD7 ATT0 */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_CCOSAR CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_CCOSAR.R /* Program Cache Software Operations */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_CCOCR CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_CCOCR.R /* Program Cache Software Operations Control */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_P_ECADD CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.P_ECADD.R /* Program ECC Error Address */
#define CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM_PMSSACS CEVA_SPF2__PROGRAM_MEMORY_SUBSYSTEM.PMSSACS.R /* PMSS Access Control */

/* PSI5 */
#define PSI5_0_GCR           PSI5_0.GCR.R                  /* Global Control Register */
#define PSI5_0_CH0_PCCR      PSI5_0.CHANNEL[0].CH_PCCR.R   /* PSI5 Channel Control Register */
#define PSI5_0_CH0_DCR       PSI5_0.CHANNEL[0].CH_DCR.R    /* DMA Control Register */
#define PSI5_0_CH0_DSR       PSI5_0.CHANNEL[0].CH_DSR.R    /* DMA Status Register */
#define PSI5_0_CH0_GICR      PSI5_0.CHANNEL[0].CH_GICR.R   /* General Interrupt Control Register */
#define PSI5_0_CH0_NDICR     PSI5_0.CHANNEL[0].CH_NDICR.R  /* New Data Interrupt Control Register */
#define PSI5_0_CH0_OWICR     PSI5_0.CHANNEL[0].CH_OWICR.R  /* Overwrite Interrupt Control Register */
#define PSI5_0_CH0_EICR      PSI5_0.CHANNEL[0].CH_EICR.R   /* Error Interrupt Control Register */
#define PSI5_0_CH0_GISR      PSI5_0.CHANNEL[0].CH_GISR.R   /* General Interrupt Status Register */
#define PSI5_0_CH0_DPMR      PSI5_0.CHANNEL[0].CH_DPMR.R   /* DMA PSI5 Message Register */
#define PSI5_0_CH0_DSFR      PSI5_0.CHANNEL[0].CH_DSFR.R   /* DMA SMC Frame Register */
#define PSI5_0_CH0_DDSR      PSI5_0.CHANNEL[0].CH_DDSR.R   /* DMA Diagnostic Status Register */
#define PSI5_0_CH0_PMRRL     PSI5_0.CHANNEL[0].CH_PMRRL.R  /* PSI5 Message Receive Register Low */
#define PSI5_0_CH0_PMRRH     PSI5_0.CHANNEL[0].CH_PMRRH.R  /* PSI5 Message Receive Register High */
#define PSI5_0_CH0_PMRL0     PSI5_0.CHANNEL[0].CH_PMR[0].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH0     PSI5_0.CHANNEL[0].CH_PMR[0].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL1     PSI5_0.CHANNEL[0].CH_PMR[1].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH1     PSI5_0.CHANNEL[0].CH_PMR[1].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL2     PSI5_0.CHANNEL[0].CH_PMR[2].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH2     PSI5_0.CHANNEL[0].CH_PMR[2].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL3     PSI5_0.CHANNEL[0].CH_PMR[3].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH3     PSI5_0.CHANNEL[0].CH_PMR[3].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL4     PSI5_0.CHANNEL[0].CH_PMR[4].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH4     PSI5_0.CHANNEL[0].CH_PMR[4].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL5     PSI5_0.CHANNEL[0].CH_PMR[5].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH5     PSI5_0.CHANNEL[0].CH_PMR[5].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL6     PSI5_0.CHANNEL[0].CH_PMR[6].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH6     PSI5_0.CHANNEL[0].CH_PMR[6].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL7     PSI5_0.CHANNEL[0].CH_PMR[7].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH7     PSI5_0.CHANNEL[0].CH_PMR[7].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL8     PSI5_0.CHANNEL[0].CH_PMR[8].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH8     PSI5_0.CHANNEL[0].CH_PMR[8].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL9     PSI5_0.CHANNEL[0].CH_PMR[9].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH9     PSI5_0.CHANNEL[0].CH_PMR[9].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL10    PSI5_0.CHANNEL[0].CH_PMR[10].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH10    PSI5_0.CHANNEL[0].CH_PMR[10].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL11    PSI5_0.CHANNEL[0].CH_PMR[11].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH11    PSI5_0.CHANNEL[0].CH_PMR[11].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL12    PSI5_0.CHANNEL[0].CH_PMR[12].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH12    PSI5_0.CHANNEL[0].CH_PMR[12].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL13    PSI5_0.CHANNEL[0].CH_PMR[13].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH13    PSI5_0.CHANNEL[0].CH_PMR[13].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL14    PSI5_0.CHANNEL[0].CH_PMR[14].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH14    PSI5_0.CHANNEL[0].CH_PMR[14].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL15    PSI5_0.CHANNEL[0].CH_PMR[15].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH15    PSI5_0.CHANNEL[0].CH_PMR[15].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL16    PSI5_0.CHANNEL[0].CH_PMR[16].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH16    PSI5_0.CHANNEL[0].CH_PMR[16].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL17    PSI5_0.CHANNEL[0].CH_PMR[17].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH17    PSI5_0.CHANNEL[0].CH_PMR[17].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL18    PSI5_0.CHANNEL[0].CH_PMR[18].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH18    PSI5_0.CHANNEL[0].CH_PMR[18].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL19    PSI5_0.CHANNEL[0].CH_PMR[19].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH19    PSI5_0.CHANNEL[0].CH_PMR[19].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL20    PSI5_0.CHANNEL[0].CH_PMR[20].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH20    PSI5_0.CHANNEL[0].CH_PMR[20].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL21    PSI5_0.CHANNEL[0].CH_PMR[21].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH21    PSI5_0.CHANNEL[0].CH_PMR[21].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL22    PSI5_0.CHANNEL[0].CH_PMR[22].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH22    PSI5_0.CHANNEL[0].CH_PMR[22].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL23    PSI5_0.CHANNEL[0].CH_PMR[23].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH23    PSI5_0.CHANNEL[0].CH_PMR[23].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL24    PSI5_0.CHANNEL[0].CH_PMR[24].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH24    PSI5_0.CHANNEL[0].CH_PMR[24].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL25    PSI5_0.CHANNEL[0].CH_PMR[25].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH25    PSI5_0.CHANNEL[0].CH_PMR[25].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL26    PSI5_0.CHANNEL[0].CH_PMR[26].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH26    PSI5_0.CHANNEL[0].CH_PMR[26].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL27    PSI5_0.CHANNEL[0].CH_PMR[27].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH27    PSI5_0.CHANNEL[0].CH_PMR[27].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL28    PSI5_0.CHANNEL[0].CH_PMR[28].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH28    PSI5_0.CHANNEL[0].CH_PMR[28].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL29    PSI5_0.CHANNEL[0].CH_PMR[29].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH29    PSI5_0.CHANNEL[0].CH_PMR[29].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL30    PSI5_0.CHANNEL[0].CH_PMR[30].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH30    PSI5_0.CHANNEL[0].CH_PMR[30].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_PMRL31    PSI5_0.CHANNEL[0].CH_PMR[31].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH0_PMRH31    PSI5_0.CHANNEL[0].CH_PMR[31].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH0_SFR1      PSI5_0.CHANNEL[0].CH_SFR[0].R /* SMC Frame Register n */
#define PSI5_0_CH0_SFR2      PSI5_0.CHANNEL[0].CH_SFR[1].R /* SMC Frame Register n */
#define PSI5_0_CH0_SFR3      PSI5_0.CHANNEL[0].CH_SFR[2].R /* SMC Frame Register n */
#define PSI5_0_CH0_SFR4      PSI5_0.CHANNEL[0].CH_SFR[3].R /* SMC Frame Register n */
#define PSI5_0_CH0_SFR5      PSI5_0.CHANNEL[0].CH_SFR[4].R /* SMC Frame Register n */
#define PSI5_0_CH0_SFR6      PSI5_0.CHANNEL[0].CH_SFR[5].R /* SMC Frame Register n */
#define PSI5_0_CH0_NDSR      PSI5_0.CHANNEL[0].CH_NDSR.R   /* New Data Status Register */
#define PSI5_0_CH0_OWSR      PSI5_0.CHANNEL[0].CH_OWSR.R   /* Overwrite Status Register */
#define PSI5_0_CH0_EISR      PSI5_0.CHANNEL[0].CH_EISR.R   /* Error Indication Status Register */
#define PSI5_0_CH0_SNDSR     PSI5_0.CHANNEL[0].CH_SNDSR.R  /* Set New Data Status Register */
#define PSI5_0_CH0_SOWSR     PSI5_0.CHANNEL[0].CH_SOWSR.R  /* Set Overwrite Status Register */
#define PSI5_0_CH0_SEISR     PSI5_0.CHANNEL[0].CH_SEISR.R  /* Set Error Status Register */
#define PSI5_0_CH0_SSESR     PSI5_0.CHANNEL[0].CH_SSESR.R  /* Set SMC Error Status Register */
#define PSI5_0_CH0_STSRR     PSI5_0.CHANNEL[0].CH_STSRR.R  /* Sync Time Stamp Read Register */
#define PSI5_0_CH0_DTSRR     PSI5_0.CHANNEL[0].CH_DTSRR.R  /* Data Time Stamp Read Register */
#define PSI5_0_CH0_S1FCR     PSI5_0.CHANNEL[0].CH_SFCR[0].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH0_S2FCR     PSI5_0.CHANNEL[0].CH_SFCR[1].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH0_S3FCR     PSI5_0.CHANNEL[0].CH_SFCR[2].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH0_S4FCR     PSI5_0.CHANNEL[0].CH_SFCR[3].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH0_S5FCR     PSI5_0.CHANNEL[0].CH_SFCR[4].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH0_S6FCR     PSI5_0.CHANNEL[0].CH_SFCR[5].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH0_S2SBR     PSI5_0.CHANNEL[0].CH_S2SBR.R  /* Slot 2 Start Boundary Register */
#define PSI5_0_CH0_S1SBR     PSI5_0.CHANNEL[0].CH_S1SBR.R  /* Slot 1 Start Boundary Register */
#define PSI5_0_CH0_S4SBR     PSI5_0.CHANNEL[0].CH_S4SBR.R  /* Slot 4 Start Boundary Register */
#define PSI5_0_CH0_S3SBR     PSI5_0.CHANNEL[0].CH_S3SBR.R  /* Slot 3 Start Boundary Register */
#define PSI5_0_CH0_S6SBR     PSI5_0.CHANNEL[0].CH_S6SBR.R  /* Slot 6 Start Boundary Register */
#define PSI5_0_CH0_S5SBR     PSI5_0.CHANNEL[0].CH_S5SBR.R  /* Slot 5 Start Boundary Register */
#define PSI5_0_CH0_SnEBR     PSI5_0.CHANNEL[0].CH_SNEBR.R  /* Slot n End Boundary Register */
#define PSI5_0_CH0_MDDIS_OFF PSI5_0.CHANNEL[0].CH_MDDIS_OFF.R /* Manchestor Decoder Disable Offset */
#define PSI5_0_CH0_DOBCR     PSI5_0.CHANNEL[0].CH_DOBCR.R  /* Data Output Block Configuration Register */
#define PSI5_0_CH0_PW1D      PSI5_0.CHANNEL[0].CH_PW1D.R   /* Pulse Width for Data Bit Value 1 */
#define PSI5_0_CH0_PW0D      PSI5_0.CHANNEL[0].CH_PW0D.R   /* Pulse Width for Data Bit Value 0 */
#define PSI5_0_CH0_CIPR      PSI5_0.CHANNEL[0].CH_CIPR.R   /* Counter Initialize Pulse Register */
#define PSI5_0_CH0_CTPR      PSI5_0.CHANNEL[0].CH_CTPR.R   /* Counter Target Pulse Register */
#define PSI5_0_CH0_DPRL      PSI5_0.CHANNEL[0].CH_DPRL.R   /* Data Preparation Register Low */
#define PSI5_0_CH0_DPRH      PSI5_0.CHANNEL[0].CH_DPRH.R   /* Data Preparation Register High */
#define PSI5_0_CH0_DBRL      PSI5_0.CHANNEL[0].CH_DBRL.R   /* Data Buffer Register Low */
#define PSI5_0_CH0_DBRH      PSI5_0.CHANNEL[0].CH_DBRH.R   /* Data Buffer Register High */
#define PSI5_0_CH0_DSRL      PSI5_0.CHANNEL[0].CH_DSRL.R   /* Data Shift Register Low */
#define PSI5_0_CH0_DSRH      PSI5_0.CHANNEL[0].CH_DSRH.R   /* Data Shift Register High */
#define PSI5_0_CH1_PCCR      PSI5_0.CHANNEL[1].CH_PCCR.R   /* PSI5 Channel Control Register */
#define PSI5_0_CH1_DCR       PSI5_0.CHANNEL[1].CH_DCR.R    /* DMA Control Register */
#define PSI5_0_CH1_DSR       PSI5_0.CHANNEL[1].CH_DSR.R    /* DMA Status Register */
#define PSI5_0_CH1_GICR      PSI5_0.CHANNEL[1].CH_GICR.R   /* General Interrupt Control Register */
#define PSI5_0_CH1_NDICR     PSI5_0.CHANNEL[1].CH_NDICR.R  /* New Data Interrupt Control Register */
#define PSI5_0_CH1_OWICR     PSI5_0.CHANNEL[1].CH_OWICR.R  /* Overwrite Interrupt Control Register */
#define PSI5_0_CH1_EICR      PSI5_0.CHANNEL[1].CH_EICR.R   /* Error Interrupt Control Register */
#define PSI5_0_CH1_GISR      PSI5_0.CHANNEL[1].CH_GISR.R   /* General Interrupt Status Register */
#define PSI5_0_CH1_DPMR      PSI5_0.CHANNEL[1].CH_DPMR.R   /* DMA PSI5 Message Register */
#define PSI5_0_CH1_DSFR      PSI5_0.CHANNEL[1].CH_DSFR.R   /* DMA SMC Frame Register */
#define PSI5_0_CH1_DDSR      PSI5_0.CHANNEL[1].CH_DDSR.R   /* DMA Diagnostic Status Register */
#define PSI5_0_CH1_PMRRL     PSI5_0.CHANNEL[1].CH_PMRRL.R  /* PSI5 Message Receive Register Low */
#define PSI5_0_CH1_PMRRH     PSI5_0.CHANNEL[1].CH_PMRRH.R  /* PSI5 Message Receive Register High */
#define PSI5_0_CH1_PMRL0     PSI5_0.CHANNEL[1].CH_PMR[0].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH0     PSI5_0.CHANNEL[1].CH_PMR[0].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL1     PSI5_0.CHANNEL[1].CH_PMR[1].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH1     PSI5_0.CHANNEL[1].CH_PMR[1].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL2     PSI5_0.CHANNEL[1].CH_PMR[2].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH2     PSI5_0.CHANNEL[1].CH_PMR[2].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL3     PSI5_0.CHANNEL[1].CH_PMR[3].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH3     PSI5_0.CHANNEL[1].CH_PMR[3].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL4     PSI5_0.CHANNEL[1].CH_PMR[4].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH4     PSI5_0.CHANNEL[1].CH_PMR[4].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL5     PSI5_0.CHANNEL[1].CH_PMR[5].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH5     PSI5_0.CHANNEL[1].CH_PMR[5].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL6     PSI5_0.CHANNEL[1].CH_PMR[6].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH6     PSI5_0.CHANNEL[1].CH_PMR[6].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL7     PSI5_0.CHANNEL[1].CH_PMR[7].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH7     PSI5_0.CHANNEL[1].CH_PMR[7].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL8     PSI5_0.CHANNEL[1].CH_PMR[8].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH8     PSI5_0.CHANNEL[1].CH_PMR[8].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL9     PSI5_0.CHANNEL[1].CH_PMR[9].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH9     PSI5_0.CHANNEL[1].CH_PMR[9].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL10    PSI5_0.CHANNEL[1].CH_PMR[10].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH10    PSI5_0.CHANNEL[1].CH_PMR[10].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL11    PSI5_0.CHANNEL[1].CH_PMR[11].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH11    PSI5_0.CHANNEL[1].CH_PMR[11].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL12    PSI5_0.CHANNEL[1].CH_PMR[12].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH12    PSI5_0.CHANNEL[1].CH_PMR[12].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL13    PSI5_0.CHANNEL[1].CH_PMR[13].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH13    PSI5_0.CHANNEL[1].CH_PMR[13].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL14    PSI5_0.CHANNEL[1].CH_PMR[14].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH14    PSI5_0.CHANNEL[1].CH_PMR[14].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL15    PSI5_0.CHANNEL[1].CH_PMR[15].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH15    PSI5_0.CHANNEL[1].CH_PMR[15].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL16    PSI5_0.CHANNEL[1].CH_PMR[16].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH16    PSI5_0.CHANNEL[1].CH_PMR[16].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL17    PSI5_0.CHANNEL[1].CH_PMR[17].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH17    PSI5_0.CHANNEL[1].CH_PMR[17].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL18    PSI5_0.CHANNEL[1].CH_PMR[18].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH18    PSI5_0.CHANNEL[1].CH_PMR[18].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL19    PSI5_0.CHANNEL[1].CH_PMR[19].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH19    PSI5_0.CHANNEL[1].CH_PMR[19].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL20    PSI5_0.CHANNEL[1].CH_PMR[20].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH20    PSI5_0.CHANNEL[1].CH_PMR[20].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL21    PSI5_0.CHANNEL[1].CH_PMR[21].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH21    PSI5_0.CHANNEL[1].CH_PMR[21].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL22    PSI5_0.CHANNEL[1].CH_PMR[22].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH22    PSI5_0.CHANNEL[1].CH_PMR[22].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL23    PSI5_0.CHANNEL[1].CH_PMR[23].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH23    PSI5_0.CHANNEL[1].CH_PMR[23].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL24    PSI5_0.CHANNEL[1].CH_PMR[24].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH24    PSI5_0.CHANNEL[1].CH_PMR[24].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL25    PSI5_0.CHANNEL[1].CH_PMR[25].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH25    PSI5_0.CHANNEL[1].CH_PMR[25].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL26    PSI5_0.CHANNEL[1].CH_PMR[26].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH26    PSI5_0.CHANNEL[1].CH_PMR[26].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL27    PSI5_0.CHANNEL[1].CH_PMR[27].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH27    PSI5_0.CHANNEL[1].CH_PMR[27].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL28    PSI5_0.CHANNEL[1].CH_PMR[28].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH28    PSI5_0.CHANNEL[1].CH_PMR[28].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL29    PSI5_0.CHANNEL[1].CH_PMR[29].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH29    PSI5_0.CHANNEL[1].CH_PMR[29].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL30    PSI5_0.CHANNEL[1].CH_PMR[30].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH30    PSI5_0.CHANNEL[1].CH_PMR[30].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_PMRL31    PSI5_0.CHANNEL[1].CH_PMR[31].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH1_PMRH31    PSI5_0.CHANNEL[1].CH_PMR[31].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH1_SFR1      PSI5_0.CHANNEL[1].CH_SFR[0].R /* SMC Frame Register n */
#define PSI5_0_CH1_SFR2      PSI5_0.CHANNEL[1].CH_SFR[1].R /* SMC Frame Register n */
#define PSI5_0_CH1_SFR3      PSI5_0.CHANNEL[1].CH_SFR[2].R /* SMC Frame Register n */
#define PSI5_0_CH1_SFR4      PSI5_0.CHANNEL[1].CH_SFR[3].R /* SMC Frame Register n */
#define PSI5_0_CH1_SFR5      PSI5_0.CHANNEL[1].CH_SFR[4].R /* SMC Frame Register n */
#define PSI5_0_CH1_SFR6      PSI5_0.CHANNEL[1].CH_SFR[5].R /* SMC Frame Register n */
#define PSI5_0_CH1_NDSR      PSI5_0.CHANNEL[1].CH_NDSR.R   /* New Data Status Register */
#define PSI5_0_CH1_OWSR      PSI5_0.CHANNEL[1].CH_OWSR.R   /* Overwrite Status Register */
#define PSI5_0_CH1_EISR      PSI5_0.CHANNEL[1].CH_EISR.R   /* Error Indication Status Register */
#define PSI5_0_CH1_SNDSR     PSI5_0.CHANNEL[1].CH_SNDSR.R  /* Set New Data Status Register */
#define PSI5_0_CH1_SOWSR     PSI5_0.CHANNEL[1].CH_SOWSR.R  /* Set Overwrite Status Register */
#define PSI5_0_CH1_SEISR     PSI5_0.CHANNEL[1].CH_SEISR.R  /* Set Error Status Register */
#define PSI5_0_CH1_SSESR     PSI5_0.CHANNEL[1].CH_SSESR.R  /* Set SMC Error Status Register */
#define PSI5_0_CH1_STSRR     PSI5_0.CHANNEL[1].CH_STSRR.R  /* Sync Time Stamp Read Register */
#define PSI5_0_CH1_DTSRR     PSI5_0.CHANNEL[1].CH_DTSRR.R  /* Data Time Stamp Read Register */
#define PSI5_0_CH1_S1FCR     PSI5_0.CHANNEL[1].CH_SFCR[0].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH1_S2FCR     PSI5_0.CHANNEL[1].CH_SFCR[1].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH1_S3FCR     PSI5_0.CHANNEL[1].CH_SFCR[2].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH1_S4FCR     PSI5_0.CHANNEL[1].CH_SFCR[3].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH1_S5FCR     PSI5_0.CHANNEL[1].CH_SFCR[4].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH1_S6FCR     PSI5_0.CHANNEL[1].CH_SFCR[5].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH1_S2SBR     PSI5_0.CHANNEL[1].CH_S2SBR.R  /* Slot 2 Start Boundary Register */
#define PSI5_0_CH1_S1SBR     PSI5_0.CHANNEL[1].CH_S1SBR.R  /* Slot 1 Start Boundary Register */
#define PSI5_0_CH1_S4SBR     PSI5_0.CHANNEL[1].CH_S4SBR.R  /* Slot 4 Start Boundary Register */
#define PSI5_0_CH1_S3SBR     PSI5_0.CHANNEL[1].CH_S3SBR.R  /* Slot 3 Start Boundary Register */
#define PSI5_0_CH1_S6SBR     PSI5_0.CHANNEL[1].CH_S6SBR.R  /* Slot 6 Start Boundary Register */
#define PSI5_0_CH1_S5SBR     PSI5_0.CHANNEL[1].CH_S5SBR.R  /* Slot 5 Start Boundary Register */
#define PSI5_0_CH1_SnEBR     PSI5_0.CHANNEL[1].CH_SNEBR.R  /* Slot n End Boundary Register */
#define PSI5_0_CH1_MDDIS_OFF PSI5_0.CHANNEL[1].CH_MDDIS_OFF.R /* Manchestor Decoder Disable Offset */
#define PSI5_0_CH1_DOBCR     PSI5_0.CHANNEL[1].CH_DOBCR.R  /* Data Output Block Configuration Register */
#define PSI5_0_CH1_PW1D      PSI5_0.CHANNEL[1].CH_PW1D.R   /* Pulse Width for Data Bit Value 1 */
#define PSI5_0_CH1_PW0D      PSI5_0.CHANNEL[1].CH_PW0D.R   /* Pulse Width for Data Bit Value 0 */
#define PSI5_0_CH1_CIPR      PSI5_0.CHANNEL[1].CH_CIPR.R   /* Counter Initialize Pulse Register */
#define PSI5_0_CH1_CTPR      PSI5_0.CHANNEL[1].CH_CTPR.R   /* Counter Target Pulse Register */
#define PSI5_0_CH1_DPRL      PSI5_0.CHANNEL[1].CH_DPRL.R   /* Data Preparation Register Low */
#define PSI5_0_CH1_DPRH      PSI5_0.CHANNEL[1].CH_DPRH.R   /* Data Preparation Register High */
#define PSI5_0_CH1_DBRL      PSI5_0.CHANNEL[1].CH_DBRL.R   /* Data Buffer Register Low */
#define PSI5_0_CH1_DBRH      PSI5_0.CHANNEL[1].CH_DBRH.R   /* Data Buffer Register High */
#define PSI5_0_CH1_DSRL      PSI5_0.CHANNEL[1].CH_DSRL.R   /* Data Shift Register Low */
#define PSI5_0_CH1_DSRH      PSI5_0.CHANNEL[1].CH_DSRH.R   /* Data Shift Register High */
#define PSI5_0_CH2_PCCR      PSI5_0.CHANNEL[2].CH_PCCR.R   /* PSI5 Channel Control Register */
#define PSI5_0_CH2_DCR       PSI5_0.CHANNEL[2].CH_DCR.R    /* DMA Control Register */
#define PSI5_0_CH2_DSR       PSI5_0.CHANNEL[2].CH_DSR.R    /* DMA Status Register */
#define PSI5_0_CH2_GICR      PSI5_0.CHANNEL[2].CH_GICR.R   /* General Interrupt Control Register */
#define PSI5_0_CH2_NDICR     PSI5_0.CHANNEL[2].CH_NDICR.R  /* New Data Interrupt Control Register */
#define PSI5_0_CH2_OWICR     PSI5_0.CHANNEL[2].CH_OWICR.R  /* Overwrite Interrupt Control Register */
#define PSI5_0_CH2_EICR      PSI5_0.CHANNEL[2].CH_EICR.R   /* Error Interrupt Control Register */
#define PSI5_0_CH2_GISR      PSI5_0.CHANNEL[2].CH_GISR.R   /* General Interrupt Status Register */
#define PSI5_0_CH2_DPMR      PSI5_0.CHANNEL[2].CH_DPMR.R   /* DMA PSI5 Message Register */
#define PSI5_0_CH2_DSFR      PSI5_0.CHANNEL[2].CH_DSFR.R   /* DMA SMC Frame Register */
#define PSI5_0_CH2_DDSR      PSI5_0.CHANNEL[2].CH_DDSR.R   /* DMA Diagnostic Status Register */
#define PSI5_0_CH2_PMRRL     PSI5_0.CHANNEL[2].CH_PMRRL.R  /* PSI5 Message Receive Register Low */
#define PSI5_0_CH2_PMRRH     PSI5_0.CHANNEL[2].CH_PMRRH.R  /* PSI5 Message Receive Register High */
#define PSI5_0_CH2_PMRL0     PSI5_0.CHANNEL[2].CH_PMR[0].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH0     PSI5_0.CHANNEL[2].CH_PMR[0].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL1     PSI5_0.CHANNEL[2].CH_PMR[1].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH1     PSI5_0.CHANNEL[2].CH_PMR[1].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL2     PSI5_0.CHANNEL[2].CH_PMR[2].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH2     PSI5_0.CHANNEL[2].CH_PMR[2].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL3     PSI5_0.CHANNEL[2].CH_PMR[3].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH3     PSI5_0.CHANNEL[2].CH_PMR[3].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL4     PSI5_0.CHANNEL[2].CH_PMR[4].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH4     PSI5_0.CHANNEL[2].CH_PMR[4].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL5     PSI5_0.CHANNEL[2].CH_PMR[5].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH5     PSI5_0.CHANNEL[2].CH_PMR[5].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL6     PSI5_0.CHANNEL[2].CH_PMR[6].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH6     PSI5_0.CHANNEL[2].CH_PMR[6].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL7     PSI5_0.CHANNEL[2].CH_PMR[7].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH7     PSI5_0.CHANNEL[2].CH_PMR[7].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL8     PSI5_0.CHANNEL[2].CH_PMR[8].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH8     PSI5_0.CHANNEL[2].CH_PMR[8].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL9     PSI5_0.CHANNEL[2].CH_PMR[9].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH9     PSI5_0.CHANNEL[2].CH_PMR[9].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL10    PSI5_0.CHANNEL[2].CH_PMR[10].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH10    PSI5_0.CHANNEL[2].CH_PMR[10].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL11    PSI5_0.CHANNEL[2].CH_PMR[11].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH11    PSI5_0.CHANNEL[2].CH_PMR[11].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL12    PSI5_0.CHANNEL[2].CH_PMR[12].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH12    PSI5_0.CHANNEL[2].CH_PMR[12].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL13    PSI5_0.CHANNEL[2].CH_PMR[13].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH13    PSI5_0.CHANNEL[2].CH_PMR[13].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL14    PSI5_0.CHANNEL[2].CH_PMR[14].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH14    PSI5_0.CHANNEL[2].CH_PMR[14].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL15    PSI5_0.CHANNEL[2].CH_PMR[15].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH15    PSI5_0.CHANNEL[2].CH_PMR[15].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL16    PSI5_0.CHANNEL[2].CH_PMR[16].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH16    PSI5_0.CHANNEL[2].CH_PMR[16].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL17    PSI5_0.CHANNEL[2].CH_PMR[17].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH17    PSI5_0.CHANNEL[2].CH_PMR[17].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL18    PSI5_0.CHANNEL[2].CH_PMR[18].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH18    PSI5_0.CHANNEL[2].CH_PMR[18].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL19    PSI5_0.CHANNEL[2].CH_PMR[19].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH19    PSI5_0.CHANNEL[2].CH_PMR[19].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL20    PSI5_0.CHANNEL[2].CH_PMR[20].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH20    PSI5_0.CHANNEL[2].CH_PMR[20].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL21    PSI5_0.CHANNEL[2].CH_PMR[21].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH21    PSI5_0.CHANNEL[2].CH_PMR[21].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL22    PSI5_0.CHANNEL[2].CH_PMR[22].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH22    PSI5_0.CHANNEL[2].CH_PMR[22].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL23    PSI5_0.CHANNEL[2].CH_PMR[23].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH23    PSI5_0.CHANNEL[2].CH_PMR[23].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL24    PSI5_0.CHANNEL[2].CH_PMR[24].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH24    PSI5_0.CHANNEL[2].CH_PMR[24].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL25    PSI5_0.CHANNEL[2].CH_PMR[25].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH25    PSI5_0.CHANNEL[2].CH_PMR[25].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL26    PSI5_0.CHANNEL[2].CH_PMR[26].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH26    PSI5_0.CHANNEL[2].CH_PMR[26].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL27    PSI5_0.CHANNEL[2].CH_PMR[27].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH27    PSI5_0.CHANNEL[2].CH_PMR[27].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL28    PSI5_0.CHANNEL[2].CH_PMR[28].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH28    PSI5_0.CHANNEL[2].CH_PMR[28].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL29    PSI5_0.CHANNEL[2].CH_PMR[29].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH29    PSI5_0.CHANNEL[2].CH_PMR[29].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL30    PSI5_0.CHANNEL[2].CH_PMR[30].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH30    PSI5_0.CHANNEL[2].CH_PMR[30].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_PMRL31    PSI5_0.CHANNEL[2].CH_PMR[31].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH2_PMRH31    PSI5_0.CHANNEL[2].CH_PMR[31].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH2_SFR1      PSI5_0.CHANNEL[2].CH_SFR[0].R /* SMC Frame Register n */
#define PSI5_0_CH2_SFR2      PSI5_0.CHANNEL[2].CH_SFR[1].R /* SMC Frame Register n */
#define PSI5_0_CH2_SFR3      PSI5_0.CHANNEL[2].CH_SFR[2].R /* SMC Frame Register n */
#define PSI5_0_CH2_SFR4      PSI5_0.CHANNEL[2].CH_SFR[3].R /* SMC Frame Register n */
#define PSI5_0_CH2_SFR5      PSI5_0.CHANNEL[2].CH_SFR[4].R /* SMC Frame Register n */
#define PSI5_0_CH2_SFR6      PSI5_0.CHANNEL[2].CH_SFR[5].R /* SMC Frame Register n */
#define PSI5_0_CH2_NDSR      PSI5_0.CHANNEL[2].CH_NDSR.R   /* New Data Status Register */
#define PSI5_0_CH2_OWSR      PSI5_0.CHANNEL[2].CH_OWSR.R   /* Overwrite Status Register */
#define PSI5_0_CH2_EISR      PSI5_0.CHANNEL[2].CH_EISR.R   /* Error Indication Status Register */
#define PSI5_0_CH2_SNDSR     PSI5_0.CHANNEL[2].CH_SNDSR.R  /* Set New Data Status Register */
#define PSI5_0_CH2_SOWSR     PSI5_0.CHANNEL[2].CH_SOWSR.R  /* Set Overwrite Status Register */
#define PSI5_0_CH2_SEISR     PSI5_0.CHANNEL[2].CH_SEISR.R  /* Set Error Status Register */
#define PSI5_0_CH2_SSESR     PSI5_0.CHANNEL[2].CH_SSESR.R  /* Set SMC Error Status Register */
#define PSI5_0_CH2_STSRR     PSI5_0.CHANNEL[2].CH_STSRR.R  /* Sync Time Stamp Read Register */
#define PSI5_0_CH2_DTSRR     PSI5_0.CHANNEL[2].CH_DTSRR.R  /* Data Time Stamp Read Register */
#define PSI5_0_CH2_S1FCR     PSI5_0.CHANNEL[2].CH_SFCR[0].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH2_S2FCR     PSI5_0.CHANNEL[2].CH_SFCR[1].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH2_S3FCR     PSI5_0.CHANNEL[2].CH_SFCR[2].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH2_S4FCR     PSI5_0.CHANNEL[2].CH_SFCR[3].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH2_S5FCR     PSI5_0.CHANNEL[2].CH_SFCR[4].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH2_S6FCR     PSI5_0.CHANNEL[2].CH_SFCR[5].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH2_S2SBR     PSI5_0.CHANNEL[2].CH_S2SBR.R  /* Slot 2 Start Boundary Register */
#define PSI5_0_CH2_S1SBR     PSI5_0.CHANNEL[2].CH_S1SBR.R  /* Slot 1 Start Boundary Register */
#define PSI5_0_CH2_S4SBR     PSI5_0.CHANNEL[2].CH_S4SBR.R  /* Slot 4 Start Boundary Register */
#define PSI5_0_CH2_S3SBR     PSI5_0.CHANNEL[2].CH_S3SBR.R  /* Slot 3 Start Boundary Register */
#define PSI5_0_CH2_S6SBR     PSI5_0.CHANNEL[2].CH_S6SBR.R  /* Slot 6 Start Boundary Register */
#define PSI5_0_CH2_S5SBR     PSI5_0.CHANNEL[2].CH_S5SBR.R  /* Slot 5 Start Boundary Register */
#define PSI5_0_CH2_SnEBR     PSI5_0.CHANNEL[2].CH_SNEBR.R  /* Slot n End Boundary Register */
#define PSI5_0_CH2_MDDIS_OFF PSI5_0.CHANNEL[2].CH_MDDIS_OFF.R /* Manchestor Decoder Disable Offset */
#define PSI5_0_CH2_DOBCR     PSI5_0.CHANNEL[2].CH_DOBCR.R  /* Data Output Block Configuration Register */
#define PSI5_0_CH2_PW1D      PSI5_0.CHANNEL[2].CH_PW1D.R   /* Pulse Width for Data Bit Value 1 */
#define PSI5_0_CH2_PW0D      PSI5_0.CHANNEL[2].CH_PW0D.R   /* Pulse Width for Data Bit Value 0 */
#define PSI5_0_CH2_CIPR      PSI5_0.CHANNEL[2].CH_CIPR.R   /* Counter Initialize Pulse Register */
#define PSI5_0_CH2_CTPR      PSI5_0.CHANNEL[2].CH_CTPR.R   /* Counter Target Pulse Register */
#define PSI5_0_CH2_DPRL      PSI5_0.CHANNEL[2].CH_DPRL.R   /* Data Preparation Register Low */
#define PSI5_0_CH2_DPRH      PSI5_0.CHANNEL[2].CH_DPRH.R   /* Data Preparation Register High */
#define PSI5_0_CH2_DBRL      PSI5_0.CHANNEL[2].CH_DBRL.R   /* Data Buffer Register Low */
#define PSI5_0_CH2_DBRH      PSI5_0.CHANNEL[2].CH_DBRH.R   /* Data Buffer Register High */
#define PSI5_0_CH2_DSRL      PSI5_0.CHANNEL[2].CH_DSRL.R   /* Data Shift Register Low */
#define PSI5_0_CH2_DSRH      PSI5_0.CHANNEL[2].CH_DSRH.R   /* Data Shift Register High */
#define PSI5_0_CH3_PCCR      PSI5_0.CHANNEL[3].CH_PCCR.R   /* PSI5 Channel Control Register */
#define PSI5_0_CH3_DCR       PSI5_0.CHANNEL[3].CH_DCR.R    /* DMA Control Register */
#define PSI5_0_CH3_DSR       PSI5_0.CHANNEL[3].CH_DSR.R    /* DMA Status Register */
#define PSI5_0_CH3_GICR      PSI5_0.CHANNEL[3].CH_GICR.R   /* General Interrupt Control Register */
#define PSI5_0_CH3_NDICR     PSI5_0.CHANNEL[3].CH_NDICR.R  /* New Data Interrupt Control Register */
#define PSI5_0_CH3_OWICR     PSI5_0.CHANNEL[3].CH_OWICR.R  /* Overwrite Interrupt Control Register */
#define PSI5_0_CH3_EICR      PSI5_0.CHANNEL[3].CH_EICR.R   /* Error Interrupt Control Register */
#define PSI5_0_CH3_GISR      PSI5_0.CHANNEL[3].CH_GISR.R   /* General Interrupt Status Register */
#define PSI5_0_CH3_DPMR      PSI5_0.CHANNEL[3].CH_DPMR.R   /* DMA PSI5 Message Register */
#define PSI5_0_CH3_DSFR      PSI5_0.CHANNEL[3].CH_DSFR.R   /* DMA SMC Frame Register */
#define PSI5_0_CH3_DDSR      PSI5_0.CHANNEL[3].CH_DDSR.R   /* DMA Diagnostic Status Register */
#define PSI5_0_CH3_PMRRL     PSI5_0.CHANNEL[3].CH_PMRRL.R  /* PSI5 Message Receive Register Low */
#define PSI5_0_CH3_PMRRH     PSI5_0.CHANNEL[3].CH_PMRRH.R  /* PSI5 Message Receive Register High */
#define PSI5_0_CH3_PMRL0     PSI5_0.CHANNEL[3].CH_PMR[0].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH0     PSI5_0.CHANNEL[3].CH_PMR[0].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL1     PSI5_0.CHANNEL[3].CH_PMR[1].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH1     PSI5_0.CHANNEL[3].CH_PMR[1].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL2     PSI5_0.CHANNEL[3].CH_PMR[2].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH2     PSI5_0.CHANNEL[3].CH_PMR[2].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL3     PSI5_0.CHANNEL[3].CH_PMR[3].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH3     PSI5_0.CHANNEL[3].CH_PMR[3].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL4     PSI5_0.CHANNEL[3].CH_PMR[4].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH4     PSI5_0.CHANNEL[3].CH_PMR[4].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL5     PSI5_0.CHANNEL[3].CH_PMR[5].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH5     PSI5_0.CHANNEL[3].CH_PMR[5].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL6     PSI5_0.CHANNEL[3].CH_PMR[6].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH6     PSI5_0.CHANNEL[3].CH_PMR[6].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL7     PSI5_0.CHANNEL[3].CH_PMR[7].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH7     PSI5_0.CHANNEL[3].CH_PMR[7].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL8     PSI5_0.CHANNEL[3].CH_PMR[8].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH8     PSI5_0.CHANNEL[3].CH_PMR[8].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL9     PSI5_0.CHANNEL[3].CH_PMR[9].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH9     PSI5_0.CHANNEL[3].CH_PMR[9].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL10    PSI5_0.CHANNEL[3].CH_PMR[10].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH10    PSI5_0.CHANNEL[3].CH_PMR[10].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL11    PSI5_0.CHANNEL[3].CH_PMR[11].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH11    PSI5_0.CHANNEL[3].CH_PMR[11].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL12    PSI5_0.CHANNEL[3].CH_PMR[12].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH12    PSI5_0.CHANNEL[3].CH_PMR[12].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL13    PSI5_0.CHANNEL[3].CH_PMR[13].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH13    PSI5_0.CHANNEL[3].CH_PMR[13].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL14    PSI5_0.CHANNEL[3].CH_PMR[14].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH14    PSI5_0.CHANNEL[3].CH_PMR[14].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL15    PSI5_0.CHANNEL[3].CH_PMR[15].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH15    PSI5_0.CHANNEL[3].CH_PMR[15].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL16    PSI5_0.CHANNEL[3].CH_PMR[16].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH16    PSI5_0.CHANNEL[3].CH_PMR[16].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL17    PSI5_0.CHANNEL[3].CH_PMR[17].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH17    PSI5_0.CHANNEL[3].CH_PMR[17].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL18    PSI5_0.CHANNEL[3].CH_PMR[18].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH18    PSI5_0.CHANNEL[3].CH_PMR[18].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL19    PSI5_0.CHANNEL[3].CH_PMR[19].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH19    PSI5_0.CHANNEL[3].CH_PMR[19].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL20    PSI5_0.CHANNEL[3].CH_PMR[20].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH20    PSI5_0.CHANNEL[3].CH_PMR[20].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL21    PSI5_0.CHANNEL[3].CH_PMR[21].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH21    PSI5_0.CHANNEL[3].CH_PMR[21].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL22    PSI5_0.CHANNEL[3].CH_PMR[22].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH22    PSI5_0.CHANNEL[3].CH_PMR[22].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL23    PSI5_0.CHANNEL[3].CH_PMR[23].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH23    PSI5_0.CHANNEL[3].CH_PMR[23].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL24    PSI5_0.CHANNEL[3].CH_PMR[24].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH24    PSI5_0.CHANNEL[3].CH_PMR[24].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL25    PSI5_0.CHANNEL[3].CH_PMR[25].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH25    PSI5_0.CHANNEL[3].CH_PMR[25].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL26    PSI5_0.CHANNEL[3].CH_PMR[26].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH26    PSI5_0.CHANNEL[3].CH_PMR[26].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL27    PSI5_0.CHANNEL[3].CH_PMR[27].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH27    PSI5_0.CHANNEL[3].CH_PMR[27].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL28    PSI5_0.CHANNEL[3].CH_PMR[28].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH28    PSI5_0.CHANNEL[3].CH_PMR[28].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL29    PSI5_0.CHANNEL[3].CH_PMR[29].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH29    PSI5_0.CHANNEL[3].CH_PMR[29].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL30    PSI5_0.CHANNEL[3].CH_PMR[30].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH30    PSI5_0.CHANNEL[3].CH_PMR[30].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_PMRL31    PSI5_0.CHANNEL[3].CH_PMR[31].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_0_CH3_PMRH31    PSI5_0.CHANNEL[3].CH_PMR[31].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_0_CH3_SFR1      PSI5_0.CHANNEL[3].CH_SFR[0].R /* SMC Frame Register n */
#define PSI5_0_CH3_SFR2      PSI5_0.CHANNEL[3].CH_SFR[1].R /* SMC Frame Register n */
#define PSI5_0_CH3_SFR3      PSI5_0.CHANNEL[3].CH_SFR[2].R /* SMC Frame Register n */
#define PSI5_0_CH3_SFR4      PSI5_0.CHANNEL[3].CH_SFR[3].R /* SMC Frame Register n */
#define PSI5_0_CH3_SFR5      PSI5_0.CHANNEL[3].CH_SFR[4].R /* SMC Frame Register n */
#define PSI5_0_CH3_SFR6      PSI5_0.CHANNEL[3].CH_SFR[5].R /* SMC Frame Register n */
#define PSI5_0_CH3_NDSR      PSI5_0.CHANNEL[3].CH_NDSR.R   /* New Data Status Register */
#define PSI5_0_CH3_OWSR      PSI5_0.CHANNEL[3].CH_OWSR.R   /* Overwrite Status Register */
#define PSI5_0_CH3_EISR      PSI5_0.CHANNEL[3].CH_EISR.R   /* Error Indication Status Register */
#define PSI5_0_CH3_SNDSR     PSI5_0.CHANNEL[3].CH_SNDSR.R  /* Set New Data Status Register */
#define PSI5_0_CH3_SOWSR     PSI5_0.CHANNEL[3].CH_SOWSR.R  /* Set Overwrite Status Register */
#define PSI5_0_CH3_SEISR     PSI5_0.CHANNEL[3].CH_SEISR.R  /* Set Error Status Register */
#define PSI5_0_CH3_SSESR     PSI5_0.CHANNEL[3].CH_SSESR.R  /* Set SMC Error Status Register */
#define PSI5_0_CH3_STSRR     PSI5_0.CHANNEL[3].CH_STSRR.R  /* Sync Time Stamp Read Register */
#define PSI5_0_CH3_DTSRR     PSI5_0.CHANNEL[3].CH_DTSRR.R  /* Data Time Stamp Read Register */
#define PSI5_0_CH3_S1FCR     PSI5_0.CHANNEL[3].CH_SFCR[0].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH3_S2FCR     PSI5_0.CHANNEL[3].CH_SFCR[1].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH3_S3FCR     PSI5_0.CHANNEL[3].CH_SFCR[2].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH3_S4FCR     PSI5_0.CHANNEL[3].CH_SFCR[3].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH3_S5FCR     PSI5_0.CHANNEL[3].CH_SFCR[4].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH3_S6FCR     PSI5_0.CHANNEL[3].CH_SFCR[5].R /* Slot n Frame Configuration Register */
#define PSI5_0_CH3_S2SBR     PSI5_0.CHANNEL[3].CH_S2SBR.R  /* Slot 2 Start Boundary Register */
#define PSI5_0_CH3_S1SBR     PSI5_0.CHANNEL[3].CH_S1SBR.R  /* Slot 1 Start Boundary Register */
#define PSI5_0_CH3_S4SBR     PSI5_0.CHANNEL[3].CH_S4SBR.R  /* Slot 4 Start Boundary Register */
#define PSI5_0_CH3_S3SBR     PSI5_0.CHANNEL[3].CH_S3SBR.R  /* Slot 3 Start Boundary Register */
#define PSI5_0_CH3_S6SBR     PSI5_0.CHANNEL[3].CH_S6SBR.R  /* Slot 6 Start Boundary Register */
#define PSI5_0_CH3_S5SBR     PSI5_0.CHANNEL[3].CH_S5SBR.R  /* Slot 5 Start Boundary Register */
#define PSI5_0_CH3_SnEBR     PSI5_0.CHANNEL[3].CH_SNEBR.R  /* Slot n End Boundary Register */
#define PSI5_0_CH3_MDDIS_OFF PSI5_0.CHANNEL[3].CH_MDDIS_OFF.R /* Manchestor Decoder Disable Offset */
#define PSI5_0_CH3_DOBCR     PSI5_0.CHANNEL[3].CH_DOBCR.R  /* Data Output Block Configuration Register */
#define PSI5_0_CH3_PW1D      PSI5_0.CHANNEL[3].CH_PW1D.R   /* Pulse Width for Data Bit Value 1 */
#define PSI5_0_CH3_PW0D      PSI5_0.CHANNEL[3].CH_PW0D.R   /* Pulse Width for Data Bit Value 0 */
#define PSI5_0_CH3_CIPR      PSI5_0.CHANNEL[3].CH_CIPR.R   /* Counter Initialize Pulse Register */
#define PSI5_0_CH3_CTPR      PSI5_0.CHANNEL[3].CH_CTPR.R   /* Counter Target Pulse Register */
#define PSI5_0_CH3_DPRL      PSI5_0.CHANNEL[3].CH_DPRL.R   /* Data Preparation Register Low */
#define PSI5_0_CH3_DPRH      PSI5_0.CHANNEL[3].CH_DPRH.R   /* Data Preparation Register High */
#define PSI5_0_CH3_DBRL      PSI5_0.CHANNEL[3].CH_DBRL.R   /* Data Buffer Register Low */
#define PSI5_0_CH3_DBRH      PSI5_0.CHANNEL[3].CH_DBRH.R   /* Data Buffer Register High */
#define PSI5_0_CH3_DSRL      PSI5_0.CHANNEL[3].CH_DSRL.R   /* Data Shift Register Low */
#define PSI5_0_CH3_DSRH      PSI5_0.CHANNEL[3].CH_DSRH.R   /* Data Shift Register High */

#define PSI5_1_GCR           PSI5_1.GCR.R                  /* Global Control Register */
#define PSI5_1_CH0_PCCR      PSI5_1.CHANNEL[0].CH_PCCR.R   /* PSI5 Channel Control Register */
#define PSI5_1_CH0_DCR       PSI5_1.CHANNEL[0].CH_DCR.R    /* DMA Control Register */
#define PSI5_1_CH0_DSR       PSI5_1.CHANNEL[0].CH_DSR.R    /* DMA Status Register */
#define PSI5_1_CH0_GICR      PSI5_1.CHANNEL[0].CH_GICR.R   /* General Interrupt Control Register */
#define PSI5_1_CH0_NDICR     PSI5_1.CHANNEL[0].CH_NDICR.R  /* New Data Interrupt Control Register */
#define PSI5_1_CH0_OWICR     PSI5_1.CHANNEL[0].CH_OWICR.R  /* Overwrite Interrupt Control Register */
#define PSI5_1_CH0_EICR      PSI5_1.CHANNEL[0].CH_EICR.R   /* Error Interrupt Control Register */
#define PSI5_1_CH0_GISR      PSI5_1.CHANNEL[0].CH_GISR.R   /* General Interrupt Status Register */
#define PSI5_1_CH0_DPMR      PSI5_1.CHANNEL[0].CH_DPMR.R   /* DMA PSI5 Message Register */
#define PSI5_1_CH0_DSFR      PSI5_1.CHANNEL[0].CH_DSFR.R   /* DMA SMC Frame Register */
#define PSI5_1_CH0_DDSR      PSI5_1.CHANNEL[0].CH_DDSR.R   /* DMA Diagnostic Status Register */
#define PSI5_1_CH0_PMRRL     PSI5_1.CHANNEL[0].CH_PMRRL.R  /* PSI5 Message Receive Register Low */
#define PSI5_1_CH0_PMRRH     PSI5_1.CHANNEL[0].CH_PMRRH.R  /* PSI5 Message Receive Register High */
#define PSI5_1_CH0_PMRL0     PSI5_1.CHANNEL[0].CH_PMR[0].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH0     PSI5_1.CHANNEL[0].CH_PMR[0].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL1     PSI5_1.CHANNEL[0].CH_PMR[1].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH1     PSI5_1.CHANNEL[0].CH_PMR[1].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL2     PSI5_1.CHANNEL[0].CH_PMR[2].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH2     PSI5_1.CHANNEL[0].CH_PMR[2].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL3     PSI5_1.CHANNEL[0].CH_PMR[3].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH3     PSI5_1.CHANNEL[0].CH_PMR[3].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL4     PSI5_1.CHANNEL[0].CH_PMR[4].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH4     PSI5_1.CHANNEL[0].CH_PMR[4].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL5     PSI5_1.CHANNEL[0].CH_PMR[5].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH5     PSI5_1.CHANNEL[0].CH_PMR[5].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL6     PSI5_1.CHANNEL[0].CH_PMR[6].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH6     PSI5_1.CHANNEL[0].CH_PMR[6].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL7     PSI5_1.CHANNEL[0].CH_PMR[7].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH7     PSI5_1.CHANNEL[0].CH_PMR[7].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL8     PSI5_1.CHANNEL[0].CH_PMR[8].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH8     PSI5_1.CHANNEL[0].CH_PMR[8].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL9     PSI5_1.CHANNEL[0].CH_PMR[9].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH9     PSI5_1.CHANNEL[0].CH_PMR[9].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL10    PSI5_1.CHANNEL[0].CH_PMR[10].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH10    PSI5_1.CHANNEL[0].CH_PMR[10].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL11    PSI5_1.CHANNEL[0].CH_PMR[11].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH11    PSI5_1.CHANNEL[0].CH_PMR[11].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL12    PSI5_1.CHANNEL[0].CH_PMR[12].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH12    PSI5_1.CHANNEL[0].CH_PMR[12].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL13    PSI5_1.CHANNEL[0].CH_PMR[13].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH13    PSI5_1.CHANNEL[0].CH_PMR[13].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL14    PSI5_1.CHANNEL[0].CH_PMR[14].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH14    PSI5_1.CHANNEL[0].CH_PMR[14].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL15    PSI5_1.CHANNEL[0].CH_PMR[15].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH15    PSI5_1.CHANNEL[0].CH_PMR[15].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL16    PSI5_1.CHANNEL[0].CH_PMR[16].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH16    PSI5_1.CHANNEL[0].CH_PMR[16].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL17    PSI5_1.CHANNEL[0].CH_PMR[17].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH17    PSI5_1.CHANNEL[0].CH_PMR[17].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL18    PSI5_1.CHANNEL[0].CH_PMR[18].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH18    PSI5_1.CHANNEL[0].CH_PMR[18].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL19    PSI5_1.CHANNEL[0].CH_PMR[19].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH19    PSI5_1.CHANNEL[0].CH_PMR[19].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL20    PSI5_1.CHANNEL[0].CH_PMR[20].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH20    PSI5_1.CHANNEL[0].CH_PMR[20].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL21    PSI5_1.CHANNEL[0].CH_PMR[21].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH21    PSI5_1.CHANNEL[0].CH_PMR[21].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL22    PSI5_1.CHANNEL[0].CH_PMR[22].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH22    PSI5_1.CHANNEL[0].CH_PMR[22].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL23    PSI5_1.CHANNEL[0].CH_PMR[23].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH23    PSI5_1.CHANNEL[0].CH_PMR[23].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL24    PSI5_1.CHANNEL[0].CH_PMR[24].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH24    PSI5_1.CHANNEL[0].CH_PMR[24].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL25    PSI5_1.CHANNEL[0].CH_PMR[25].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH25    PSI5_1.CHANNEL[0].CH_PMR[25].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL26    PSI5_1.CHANNEL[0].CH_PMR[26].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH26    PSI5_1.CHANNEL[0].CH_PMR[26].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL27    PSI5_1.CHANNEL[0].CH_PMR[27].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH27    PSI5_1.CHANNEL[0].CH_PMR[27].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL28    PSI5_1.CHANNEL[0].CH_PMR[28].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH28    PSI5_1.CHANNEL[0].CH_PMR[28].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL29    PSI5_1.CHANNEL[0].CH_PMR[29].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH29    PSI5_1.CHANNEL[0].CH_PMR[29].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL30    PSI5_1.CHANNEL[0].CH_PMR[30].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH30    PSI5_1.CHANNEL[0].CH_PMR[30].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_PMRL31    PSI5_1.CHANNEL[0].CH_PMR[31].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH0_PMRH31    PSI5_1.CHANNEL[0].CH_PMR[31].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH0_SFR1      PSI5_1.CHANNEL[0].CH_SFR[0].R /* SMC Frame Register n */
#define PSI5_1_CH0_SFR2      PSI5_1.CHANNEL[0].CH_SFR[1].R /* SMC Frame Register n */
#define PSI5_1_CH0_SFR3      PSI5_1.CHANNEL[0].CH_SFR[2].R /* SMC Frame Register n */
#define PSI5_1_CH0_SFR4      PSI5_1.CHANNEL[0].CH_SFR[3].R /* SMC Frame Register n */
#define PSI5_1_CH0_SFR5      PSI5_1.CHANNEL[0].CH_SFR[4].R /* SMC Frame Register n */
#define PSI5_1_CH0_SFR6      PSI5_1.CHANNEL[0].CH_SFR[5].R /* SMC Frame Register n */
#define PSI5_1_CH0_NDSR      PSI5_1.CHANNEL[0].CH_NDSR.R   /* New Data Status Register */
#define PSI5_1_CH0_OWSR      PSI5_1.CHANNEL[0].CH_OWSR.R   /* Overwrite Status Register */
#define PSI5_1_CH0_EISR      PSI5_1.CHANNEL[0].CH_EISR.R   /* Error Indication Status Register */
#define PSI5_1_CH0_SNDSR     PSI5_1.CHANNEL[0].CH_SNDSR.R  /* Set New Data Status Register */
#define PSI5_1_CH0_SOWSR     PSI5_1.CHANNEL[0].CH_SOWSR.R  /* Set Overwrite Status Register */
#define PSI5_1_CH0_SEISR     PSI5_1.CHANNEL[0].CH_SEISR.R  /* Set Error Status Register */
#define PSI5_1_CH0_SSESR     PSI5_1.CHANNEL[0].CH_SSESR.R  /* Set SMC Error Status Register */
#define PSI5_1_CH0_STSRR     PSI5_1.CHANNEL[0].CH_STSRR.R  /* Sync Time Stamp Read Register */
#define PSI5_1_CH0_DTSRR     PSI5_1.CHANNEL[0].CH_DTSRR.R  /* Data Time Stamp Read Register */
#define PSI5_1_CH0_S1FCR     PSI5_1.CHANNEL[0].CH_SFCR[0].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH0_S2FCR     PSI5_1.CHANNEL[0].CH_SFCR[1].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH0_S3FCR     PSI5_1.CHANNEL[0].CH_SFCR[2].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH0_S4FCR     PSI5_1.CHANNEL[0].CH_SFCR[3].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH0_S5FCR     PSI5_1.CHANNEL[0].CH_SFCR[4].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH0_S6FCR     PSI5_1.CHANNEL[0].CH_SFCR[5].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH0_S2SBR     PSI5_1.CHANNEL[0].CH_S2SBR.R  /* Slot 2 Start Boundary Register */
#define PSI5_1_CH0_S1SBR     PSI5_1.CHANNEL[0].CH_S1SBR.R  /* Slot 1 Start Boundary Register */
#define PSI5_1_CH0_S4SBR     PSI5_1.CHANNEL[0].CH_S4SBR.R  /* Slot 4 Start Boundary Register */
#define PSI5_1_CH0_S3SBR     PSI5_1.CHANNEL[0].CH_S3SBR.R  /* Slot 3 Start Boundary Register */
#define PSI5_1_CH0_S6SBR     PSI5_1.CHANNEL[0].CH_S6SBR.R  /* Slot 6 Start Boundary Register */
#define PSI5_1_CH0_S5SBR     PSI5_1.CHANNEL[0].CH_S5SBR.R  /* Slot 5 Start Boundary Register */
#define PSI5_1_CH0_SnEBR     PSI5_1.CHANNEL[0].CH_SNEBR.R  /* Slot n End Boundary Register */
#define PSI5_1_CH0_MDDIS_OFF PSI5_1.CHANNEL[0].CH_MDDIS_OFF.R /* Manchestor Decoder Disable Offset */
#define PSI5_1_CH0_DOBCR     PSI5_1.CHANNEL[0].CH_DOBCR.R  /* Data Output Block Configuration Register */
#define PSI5_1_CH0_PW1D      PSI5_1.CHANNEL[0].CH_PW1D.R   /* Pulse Width for Data Bit Value 1 */
#define PSI5_1_CH0_PW0D      PSI5_1.CHANNEL[0].CH_PW0D.R   /* Pulse Width for Data Bit Value 0 */
#define PSI5_1_CH0_CIPR      PSI5_1.CHANNEL[0].CH_CIPR.R   /* Counter Initialize Pulse Register */
#define PSI5_1_CH0_CTPR      PSI5_1.CHANNEL[0].CH_CTPR.R   /* Counter Target Pulse Register */
#define PSI5_1_CH0_DPRL      PSI5_1.CHANNEL[0].CH_DPRL.R   /* Data Preparation Register Low */
#define PSI5_1_CH0_DPRH      PSI5_1.CHANNEL[0].CH_DPRH.R   /* Data Preparation Register High */
#define PSI5_1_CH0_DBRL      PSI5_1.CHANNEL[0].CH_DBRL.R   /* Data Buffer Register Low */
#define PSI5_1_CH0_DBRH      PSI5_1.CHANNEL[0].CH_DBRH.R   /* Data Buffer Register High */
#define PSI5_1_CH0_DSRL      PSI5_1.CHANNEL[0].CH_DSRL.R   /* Data Shift Register Low */
#define PSI5_1_CH0_DSRH      PSI5_1.CHANNEL[0].CH_DSRH.R   /* Data Shift Register High */
#define PSI5_1_CH1_PCCR      PSI5_1.CHANNEL[1].CH_PCCR.R   /* PSI5 Channel Control Register */
#define PSI5_1_CH1_DCR       PSI5_1.CHANNEL[1].CH_DCR.R    /* DMA Control Register */
#define PSI5_1_CH1_DSR       PSI5_1.CHANNEL[1].CH_DSR.R    /* DMA Status Register */
#define PSI5_1_CH1_GICR      PSI5_1.CHANNEL[1].CH_GICR.R   /* General Interrupt Control Register */
#define PSI5_1_CH1_NDICR     PSI5_1.CHANNEL[1].CH_NDICR.R  /* New Data Interrupt Control Register */
#define PSI5_1_CH1_OWICR     PSI5_1.CHANNEL[1].CH_OWICR.R  /* Overwrite Interrupt Control Register */
#define PSI5_1_CH1_EICR      PSI5_1.CHANNEL[1].CH_EICR.R   /* Error Interrupt Control Register */
#define PSI5_1_CH1_GISR      PSI5_1.CHANNEL[1].CH_GISR.R   /* General Interrupt Status Register */
#define PSI5_1_CH1_DPMR      PSI5_1.CHANNEL[1].CH_DPMR.R   /* DMA PSI5 Message Register */
#define PSI5_1_CH1_DSFR      PSI5_1.CHANNEL[1].CH_DSFR.R   /* DMA SMC Frame Register */
#define PSI5_1_CH1_DDSR      PSI5_1.CHANNEL[1].CH_DDSR.R   /* DMA Diagnostic Status Register */
#define PSI5_1_CH1_PMRRL     PSI5_1.CHANNEL[1].CH_PMRRL.R  /* PSI5 Message Receive Register Low */
#define PSI5_1_CH1_PMRRH     PSI5_1.CHANNEL[1].CH_PMRRH.R  /* PSI5 Message Receive Register High */
#define PSI5_1_CH1_PMRL0     PSI5_1.CHANNEL[1].CH_PMR[0].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH0     PSI5_1.CHANNEL[1].CH_PMR[0].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL1     PSI5_1.CHANNEL[1].CH_PMR[1].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH1     PSI5_1.CHANNEL[1].CH_PMR[1].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL2     PSI5_1.CHANNEL[1].CH_PMR[2].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH2     PSI5_1.CHANNEL[1].CH_PMR[2].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL3     PSI5_1.CHANNEL[1].CH_PMR[3].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH3     PSI5_1.CHANNEL[1].CH_PMR[3].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL4     PSI5_1.CHANNEL[1].CH_PMR[4].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH4     PSI5_1.CHANNEL[1].CH_PMR[4].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL5     PSI5_1.CHANNEL[1].CH_PMR[5].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH5     PSI5_1.CHANNEL[1].CH_PMR[5].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL6     PSI5_1.CHANNEL[1].CH_PMR[6].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH6     PSI5_1.CHANNEL[1].CH_PMR[6].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL7     PSI5_1.CHANNEL[1].CH_PMR[7].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH7     PSI5_1.CHANNEL[1].CH_PMR[7].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL8     PSI5_1.CHANNEL[1].CH_PMR[8].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH8     PSI5_1.CHANNEL[1].CH_PMR[8].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL9     PSI5_1.CHANNEL[1].CH_PMR[9].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH9     PSI5_1.CHANNEL[1].CH_PMR[9].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL10    PSI5_1.CHANNEL[1].CH_PMR[10].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH10    PSI5_1.CHANNEL[1].CH_PMR[10].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL11    PSI5_1.CHANNEL[1].CH_PMR[11].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH11    PSI5_1.CHANNEL[1].CH_PMR[11].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL12    PSI5_1.CHANNEL[1].CH_PMR[12].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH12    PSI5_1.CHANNEL[1].CH_PMR[12].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL13    PSI5_1.CHANNEL[1].CH_PMR[13].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH13    PSI5_1.CHANNEL[1].CH_PMR[13].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL14    PSI5_1.CHANNEL[1].CH_PMR[14].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH14    PSI5_1.CHANNEL[1].CH_PMR[14].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL15    PSI5_1.CHANNEL[1].CH_PMR[15].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH15    PSI5_1.CHANNEL[1].CH_PMR[15].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL16    PSI5_1.CHANNEL[1].CH_PMR[16].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH16    PSI5_1.CHANNEL[1].CH_PMR[16].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL17    PSI5_1.CHANNEL[1].CH_PMR[17].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH17    PSI5_1.CHANNEL[1].CH_PMR[17].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL18    PSI5_1.CHANNEL[1].CH_PMR[18].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH18    PSI5_1.CHANNEL[1].CH_PMR[18].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL19    PSI5_1.CHANNEL[1].CH_PMR[19].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH19    PSI5_1.CHANNEL[1].CH_PMR[19].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL20    PSI5_1.CHANNEL[1].CH_PMR[20].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH20    PSI5_1.CHANNEL[1].CH_PMR[20].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL21    PSI5_1.CHANNEL[1].CH_PMR[21].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH21    PSI5_1.CHANNEL[1].CH_PMR[21].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL22    PSI5_1.CHANNEL[1].CH_PMR[22].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH22    PSI5_1.CHANNEL[1].CH_PMR[22].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL23    PSI5_1.CHANNEL[1].CH_PMR[23].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH23    PSI5_1.CHANNEL[1].CH_PMR[23].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL24    PSI5_1.CHANNEL[1].CH_PMR[24].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH24    PSI5_1.CHANNEL[1].CH_PMR[24].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL25    PSI5_1.CHANNEL[1].CH_PMR[25].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH25    PSI5_1.CHANNEL[1].CH_PMR[25].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL26    PSI5_1.CHANNEL[1].CH_PMR[26].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH26    PSI5_1.CHANNEL[1].CH_PMR[26].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL27    PSI5_1.CHANNEL[1].CH_PMR[27].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH27    PSI5_1.CHANNEL[1].CH_PMR[27].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL28    PSI5_1.CHANNEL[1].CH_PMR[28].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH28    PSI5_1.CHANNEL[1].CH_PMR[28].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL29    PSI5_1.CHANNEL[1].CH_PMR[29].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH29    PSI5_1.CHANNEL[1].CH_PMR[29].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL30    PSI5_1.CHANNEL[1].CH_PMR[30].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH30    PSI5_1.CHANNEL[1].CH_PMR[30].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_PMRL31    PSI5_1.CHANNEL[1].CH_PMR[31].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH1_PMRH31    PSI5_1.CHANNEL[1].CH_PMR[31].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH1_SFR1      PSI5_1.CHANNEL[1].CH_SFR[0].R /* SMC Frame Register n */
#define PSI5_1_CH1_SFR2      PSI5_1.CHANNEL[1].CH_SFR[1].R /* SMC Frame Register n */
#define PSI5_1_CH1_SFR3      PSI5_1.CHANNEL[1].CH_SFR[2].R /* SMC Frame Register n */
#define PSI5_1_CH1_SFR4      PSI5_1.CHANNEL[1].CH_SFR[3].R /* SMC Frame Register n */
#define PSI5_1_CH1_SFR5      PSI5_1.CHANNEL[1].CH_SFR[4].R /* SMC Frame Register n */
#define PSI5_1_CH1_SFR6      PSI5_1.CHANNEL[1].CH_SFR[5].R /* SMC Frame Register n */
#define PSI5_1_CH1_NDSR      PSI5_1.CHANNEL[1].CH_NDSR.R   /* New Data Status Register */
#define PSI5_1_CH1_OWSR      PSI5_1.CHANNEL[1].CH_OWSR.R   /* Overwrite Status Register */
#define PSI5_1_CH1_EISR      PSI5_1.CHANNEL[1].CH_EISR.R   /* Error Indication Status Register */
#define PSI5_1_CH1_SNDSR     PSI5_1.CHANNEL[1].CH_SNDSR.R  /* Set New Data Status Register */
#define PSI5_1_CH1_SOWSR     PSI5_1.CHANNEL[1].CH_SOWSR.R  /* Set Overwrite Status Register */
#define PSI5_1_CH1_SEISR     PSI5_1.CHANNEL[1].CH_SEISR.R  /* Set Error Status Register */
#define PSI5_1_CH1_SSESR     PSI5_1.CHANNEL[1].CH_SSESR.R  /* Set SMC Error Status Register */
#define PSI5_1_CH1_STSRR     PSI5_1.CHANNEL[1].CH_STSRR.R  /* Sync Time Stamp Read Register */
#define PSI5_1_CH1_DTSRR     PSI5_1.CHANNEL[1].CH_DTSRR.R  /* Data Time Stamp Read Register */
#define PSI5_1_CH1_S1FCR     PSI5_1.CHANNEL[1].CH_SFCR[0].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH1_S2FCR     PSI5_1.CHANNEL[1].CH_SFCR[1].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH1_S3FCR     PSI5_1.CHANNEL[1].CH_SFCR[2].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH1_S4FCR     PSI5_1.CHANNEL[1].CH_SFCR[3].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH1_S5FCR     PSI5_1.CHANNEL[1].CH_SFCR[4].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH1_S6FCR     PSI5_1.CHANNEL[1].CH_SFCR[5].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH1_S2SBR     PSI5_1.CHANNEL[1].CH_S2SBR.R  /* Slot 2 Start Boundary Register */
#define PSI5_1_CH1_S1SBR     PSI5_1.CHANNEL[1].CH_S1SBR.R  /* Slot 1 Start Boundary Register */
#define PSI5_1_CH1_S4SBR     PSI5_1.CHANNEL[1].CH_S4SBR.R  /* Slot 4 Start Boundary Register */
#define PSI5_1_CH1_S3SBR     PSI5_1.CHANNEL[1].CH_S3SBR.R  /* Slot 3 Start Boundary Register */
#define PSI5_1_CH1_S6SBR     PSI5_1.CHANNEL[1].CH_S6SBR.R  /* Slot 6 Start Boundary Register */
#define PSI5_1_CH1_S5SBR     PSI5_1.CHANNEL[1].CH_S5SBR.R  /* Slot 5 Start Boundary Register */
#define PSI5_1_CH1_SnEBR     PSI5_1.CHANNEL[1].CH_SNEBR.R  /* Slot n End Boundary Register */
#define PSI5_1_CH1_MDDIS_OFF PSI5_1.CHANNEL[1].CH_MDDIS_OFF.R /* Manchestor Decoder Disable Offset */
#define PSI5_1_CH1_DOBCR     PSI5_1.CHANNEL[1].CH_DOBCR.R  /* Data Output Block Configuration Register */
#define PSI5_1_CH1_PW1D      PSI5_1.CHANNEL[1].CH_PW1D.R   /* Pulse Width for Data Bit Value 1 */
#define PSI5_1_CH1_PW0D      PSI5_1.CHANNEL[1].CH_PW0D.R   /* Pulse Width for Data Bit Value 0 */
#define PSI5_1_CH1_CIPR      PSI5_1.CHANNEL[1].CH_CIPR.R   /* Counter Initialize Pulse Register */
#define PSI5_1_CH1_CTPR      PSI5_1.CHANNEL[1].CH_CTPR.R   /* Counter Target Pulse Register */
#define PSI5_1_CH1_DPRL      PSI5_1.CHANNEL[1].CH_DPRL.R   /* Data Preparation Register Low */
#define PSI5_1_CH1_DPRH      PSI5_1.CHANNEL[1].CH_DPRH.R   /* Data Preparation Register High */
#define PSI5_1_CH1_DBRL      PSI5_1.CHANNEL[1].CH_DBRL.R   /* Data Buffer Register Low */
#define PSI5_1_CH1_DBRH      PSI5_1.CHANNEL[1].CH_DBRH.R   /* Data Buffer Register High */
#define PSI5_1_CH1_DSRL      PSI5_1.CHANNEL[1].CH_DSRL.R   /* Data Shift Register Low */
#define PSI5_1_CH1_DSRH      PSI5_1.CHANNEL[1].CH_DSRH.R   /* Data Shift Register High */
#define PSI5_1_CH2_PCCR      PSI5_1.CHANNEL[2].CH_PCCR.R   /* PSI5 Channel Control Register */
#define PSI5_1_CH2_DCR       PSI5_1.CHANNEL[2].CH_DCR.R    /* DMA Control Register */
#define PSI5_1_CH2_DSR       PSI5_1.CHANNEL[2].CH_DSR.R    /* DMA Status Register */
#define PSI5_1_CH2_GICR      PSI5_1.CHANNEL[2].CH_GICR.R   /* General Interrupt Control Register */
#define PSI5_1_CH2_NDICR     PSI5_1.CHANNEL[2].CH_NDICR.R  /* New Data Interrupt Control Register */
#define PSI5_1_CH2_OWICR     PSI5_1.CHANNEL[2].CH_OWICR.R  /* Overwrite Interrupt Control Register */
#define PSI5_1_CH2_EICR      PSI5_1.CHANNEL[2].CH_EICR.R   /* Error Interrupt Control Register */
#define PSI5_1_CH2_GISR      PSI5_1.CHANNEL[2].CH_GISR.R   /* General Interrupt Status Register */
#define PSI5_1_CH2_DPMR      PSI5_1.CHANNEL[2].CH_DPMR.R   /* DMA PSI5 Message Register */
#define PSI5_1_CH2_DSFR      PSI5_1.CHANNEL[2].CH_DSFR.R   /* DMA SMC Frame Register */
#define PSI5_1_CH2_DDSR      PSI5_1.CHANNEL[2].CH_DDSR.R   /* DMA Diagnostic Status Register */
#define PSI5_1_CH2_PMRRL     PSI5_1.CHANNEL[2].CH_PMRRL.R  /* PSI5 Message Receive Register Low */
#define PSI5_1_CH2_PMRRH     PSI5_1.CHANNEL[2].CH_PMRRH.R  /* PSI5 Message Receive Register High */
#define PSI5_1_CH2_PMRL0     PSI5_1.CHANNEL[2].CH_PMR[0].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH0     PSI5_1.CHANNEL[2].CH_PMR[0].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL1     PSI5_1.CHANNEL[2].CH_PMR[1].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH1     PSI5_1.CHANNEL[2].CH_PMR[1].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL2     PSI5_1.CHANNEL[2].CH_PMR[2].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH2     PSI5_1.CHANNEL[2].CH_PMR[2].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL3     PSI5_1.CHANNEL[2].CH_PMR[3].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH3     PSI5_1.CHANNEL[2].CH_PMR[3].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL4     PSI5_1.CHANNEL[2].CH_PMR[4].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH4     PSI5_1.CHANNEL[2].CH_PMR[4].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL5     PSI5_1.CHANNEL[2].CH_PMR[5].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH5     PSI5_1.CHANNEL[2].CH_PMR[5].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL6     PSI5_1.CHANNEL[2].CH_PMR[6].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH6     PSI5_1.CHANNEL[2].CH_PMR[6].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL7     PSI5_1.CHANNEL[2].CH_PMR[7].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH7     PSI5_1.CHANNEL[2].CH_PMR[7].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL8     PSI5_1.CHANNEL[2].CH_PMR[8].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH8     PSI5_1.CHANNEL[2].CH_PMR[8].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL9     PSI5_1.CHANNEL[2].CH_PMR[9].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH9     PSI5_1.CHANNEL[2].CH_PMR[9].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL10    PSI5_1.CHANNEL[2].CH_PMR[10].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH10    PSI5_1.CHANNEL[2].CH_PMR[10].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL11    PSI5_1.CHANNEL[2].CH_PMR[11].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH11    PSI5_1.CHANNEL[2].CH_PMR[11].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL12    PSI5_1.CHANNEL[2].CH_PMR[12].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH12    PSI5_1.CHANNEL[2].CH_PMR[12].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL13    PSI5_1.CHANNEL[2].CH_PMR[13].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH13    PSI5_1.CHANNEL[2].CH_PMR[13].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL14    PSI5_1.CHANNEL[2].CH_PMR[14].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH14    PSI5_1.CHANNEL[2].CH_PMR[14].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL15    PSI5_1.CHANNEL[2].CH_PMR[15].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH15    PSI5_1.CHANNEL[2].CH_PMR[15].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL16    PSI5_1.CHANNEL[2].CH_PMR[16].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH16    PSI5_1.CHANNEL[2].CH_PMR[16].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL17    PSI5_1.CHANNEL[2].CH_PMR[17].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH17    PSI5_1.CHANNEL[2].CH_PMR[17].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL18    PSI5_1.CHANNEL[2].CH_PMR[18].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH18    PSI5_1.CHANNEL[2].CH_PMR[18].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL19    PSI5_1.CHANNEL[2].CH_PMR[19].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH19    PSI5_1.CHANNEL[2].CH_PMR[19].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL20    PSI5_1.CHANNEL[2].CH_PMR[20].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH20    PSI5_1.CHANNEL[2].CH_PMR[20].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL21    PSI5_1.CHANNEL[2].CH_PMR[21].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH21    PSI5_1.CHANNEL[2].CH_PMR[21].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL22    PSI5_1.CHANNEL[2].CH_PMR[22].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH22    PSI5_1.CHANNEL[2].CH_PMR[22].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL23    PSI5_1.CHANNEL[2].CH_PMR[23].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH23    PSI5_1.CHANNEL[2].CH_PMR[23].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL24    PSI5_1.CHANNEL[2].CH_PMR[24].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH24    PSI5_1.CHANNEL[2].CH_PMR[24].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL25    PSI5_1.CHANNEL[2].CH_PMR[25].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH25    PSI5_1.CHANNEL[2].CH_PMR[25].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL26    PSI5_1.CHANNEL[2].CH_PMR[26].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH26    PSI5_1.CHANNEL[2].CH_PMR[26].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL27    PSI5_1.CHANNEL[2].CH_PMR[27].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH27    PSI5_1.CHANNEL[2].CH_PMR[27].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL28    PSI5_1.CHANNEL[2].CH_PMR[28].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH28    PSI5_1.CHANNEL[2].CH_PMR[28].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL29    PSI5_1.CHANNEL[2].CH_PMR[29].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH29    PSI5_1.CHANNEL[2].CH_PMR[29].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL30    PSI5_1.CHANNEL[2].CH_PMR[30].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH30    PSI5_1.CHANNEL[2].CH_PMR[30].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_PMRL31    PSI5_1.CHANNEL[2].CH_PMR[31].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH2_PMRH31    PSI5_1.CHANNEL[2].CH_PMR[31].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH2_SFR1      PSI5_1.CHANNEL[2].CH_SFR[0].R /* SMC Frame Register n */
#define PSI5_1_CH2_SFR2      PSI5_1.CHANNEL[2].CH_SFR[1].R /* SMC Frame Register n */
#define PSI5_1_CH2_SFR3      PSI5_1.CHANNEL[2].CH_SFR[2].R /* SMC Frame Register n */
#define PSI5_1_CH2_SFR4      PSI5_1.CHANNEL[2].CH_SFR[3].R /* SMC Frame Register n */
#define PSI5_1_CH2_SFR5      PSI5_1.CHANNEL[2].CH_SFR[4].R /* SMC Frame Register n */
#define PSI5_1_CH2_SFR6      PSI5_1.CHANNEL[2].CH_SFR[5].R /* SMC Frame Register n */
#define PSI5_1_CH2_NDSR      PSI5_1.CHANNEL[2].CH_NDSR.R   /* New Data Status Register */
#define PSI5_1_CH2_OWSR      PSI5_1.CHANNEL[2].CH_OWSR.R   /* Overwrite Status Register */
#define PSI5_1_CH2_EISR      PSI5_1.CHANNEL[2].CH_EISR.R   /* Error Indication Status Register */
#define PSI5_1_CH2_SNDSR     PSI5_1.CHANNEL[2].CH_SNDSR.R  /* Set New Data Status Register */
#define PSI5_1_CH2_SOWSR     PSI5_1.CHANNEL[2].CH_SOWSR.R  /* Set Overwrite Status Register */
#define PSI5_1_CH2_SEISR     PSI5_1.CHANNEL[2].CH_SEISR.R  /* Set Error Status Register */
#define PSI5_1_CH2_SSESR     PSI5_1.CHANNEL[2].CH_SSESR.R  /* Set SMC Error Status Register */
#define PSI5_1_CH2_STSRR     PSI5_1.CHANNEL[2].CH_STSRR.R  /* Sync Time Stamp Read Register */
#define PSI5_1_CH2_DTSRR     PSI5_1.CHANNEL[2].CH_DTSRR.R  /* Data Time Stamp Read Register */
#define PSI5_1_CH2_S1FCR     PSI5_1.CHANNEL[2].CH_SFCR[0].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH2_S2FCR     PSI5_1.CHANNEL[2].CH_SFCR[1].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH2_S3FCR     PSI5_1.CHANNEL[2].CH_SFCR[2].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH2_S4FCR     PSI5_1.CHANNEL[2].CH_SFCR[3].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH2_S5FCR     PSI5_1.CHANNEL[2].CH_SFCR[4].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH2_S6FCR     PSI5_1.CHANNEL[2].CH_SFCR[5].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH2_S2SBR     PSI5_1.CHANNEL[2].CH_S2SBR.R  /* Slot 2 Start Boundary Register */
#define PSI5_1_CH2_S1SBR     PSI5_1.CHANNEL[2].CH_S1SBR.R  /* Slot 1 Start Boundary Register */
#define PSI5_1_CH2_S4SBR     PSI5_1.CHANNEL[2].CH_S4SBR.R  /* Slot 4 Start Boundary Register */
#define PSI5_1_CH2_S3SBR     PSI5_1.CHANNEL[2].CH_S3SBR.R  /* Slot 3 Start Boundary Register */
#define PSI5_1_CH2_S6SBR     PSI5_1.CHANNEL[2].CH_S6SBR.R  /* Slot 6 Start Boundary Register */
#define PSI5_1_CH2_S5SBR     PSI5_1.CHANNEL[2].CH_S5SBR.R  /* Slot 5 Start Boundary Register */
#define PSI5_1_CH2_SnEBR     PSI5_1.CHANNEL[2].CH_SNEBR.R  /* Slot n End Boundary Register */
#define PSI5_1_CH2_MDDIS_OFF PSI5_1.CHANNEL[2].CH_MDDIS_OFF.R /* Manchestor Decoder Disable Offset */
#define PSI5_1_CH2_DOBCR     PSI5_1.CHANNEL[2].CH_DOBCR.R  /* Data Output Block Configuration Register */
#define PSI5_1_CH2_PW1D      PSI5_1.CHANNEL[2].CH_PW1D.R   /* Pulse Width for Data Bit Value 1 */
#define PSI5_1_CH2_PW0D      PSI5_1.CHANNEL[2].CH_PW0D.R   /* Pulse Width for Data Bit Value 0 */
#define PSI5_1_CH2_CIPR      PSI5_1.CHANNEL[2].CH_CIPR.R   /* Counter Initialize Pulse Register */
#define PSI5_1_CH2_CTPR      PSI5_1.CHANNEL[2].CH_CTPR.R   /* Counter Target Pulse Register */
#define PSI5_1_CH2_DPRL      PSI5_1.CHANNEL[2].CH_DPRL.R   /* Data Preparation Register Low */
#define PSI5_1_CH2_DPRH      PSI5_1.CHANNEL[2].CH_DPRH.R   /* Data Preparation Register High */
#define PSI5_1_CH2_DBRL      PSI5_1.CHANNEL[2].CH_DBRL.R   /* Data Buffer Register Low */
#define PSI5_1_CH2_DBRH      PSI5_1.CHANNEL[2].CH_DBRH.R   /* Data Buffer Register High */
#define PSI5_1_CH2_DSRL      PSI5_1.CHANNEL[2].CH_DSRL.R   /* Data Shift Register Low */
#define PSI5_1_CH2_DSRH      PSI5_1.CHANNEL[2].CH_DSRH.R   /* Data Shift Register High */
#define PSI5_1_CH3_PCCR      PSI5_1.CHANNEL[3].CH_PCCR.R   /* PSI5 Channel Control Register */
#define PSI5_1_CH3_DCR       PSI5_1.CHANNEL[3].CH_DCR.R    /* DMA Control Register */
#define PSI5_1_CH3_DSR       PSI5_1.CHANNEL[3].CH_DSR.R    /* DMA Status Register */
#define PSI5_1_CH3_GICR      PSI5_1.CHANNEL[3].CH_GICR.R   /* General Interrupt Control Register */
#define PSI5_1_CH3_NDICR     PSI5_1.CHANNEL[3].CH_NDICR.R  /* New Data Interrupt Control Register */
#define PSI5_1_CH3_OWICR     PSI5_1.CHANNEL[3].CH_OWICR.R  /* Overwrite Interrupt Control Register */
#define PSI5_1_CH3_EICR      PSI5_1.CHANNEL[3].CH_EICR.R   /* Error Interrupt Control Register */
#define PSI5_1_CH3_GISR      PSI5_1.CHANNEL[3].CH_GISR.R   /* General Interrupt Status Register */
#define PSI5_1_CH3_DPMR      PSI5_1.CHANNEL[3].CH_DPMR.R   /* DMA PSI5 Message Register */
#define PSI5_1_CH3_DSFR      PSI5_1.CHANNEL[3].CH_DSFR.R   /* DMA SMC Frame Register */
#define PSI5_1_CH3_DDSR      PSI5_1.CHANNEL[3].CH_DDSR.R   /* DMA Diagnostic Status Register */
#define PSI5_1_CH3_PMRRL     PSI5_1.CHANNEL[3].CH_PMRRL.R  /* PSI5 Message Receive Register Low */
#define PSI5_1_CH3_PMRRH     PSI5_1.CHANNEL[3].CH_PMRRH.R  /* PSI5 Message Receive Register High */
#define PSI5_1_CH3_PMRL0     PSI5_1.CHANNEL[3].CH_PMR[0].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH0     PSI5_1.CHANNEL[3].CH_PMR[0].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL1     PSI5_1.CHANNEL[3].CH_PMR[1].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH1     PSI5_1.CHANNEL[3].CH_PMR[1].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL2     PSI5_1.CHANNEL[3].CH_PMR[2].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH2     PSI5_1.CHANNEL[3].CH_PMR[2].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL3     PSI5_1.CHANNEL[3].CH_PMR[3].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH3     PSI5_1.CHANNEL[3].CH_PMR[3].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL4     PSI5_1.CHANNEL[3].CH_PMR[4].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH4     PSI5_1.CHANNEL[3].CH_PMR[4].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL5     PSI5_1.CHANNEL[3].CH_PMR[5].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH5     PSI5_1.CHANNEL[3].CH_PMR[5].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL6     PSI5_1.CHANNEL[3].CH_PMR[6].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH6     PSI5_1.CHANNEL[3].CH_PMR[6].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL7     PSI5_1.CHANNEL[3].CH_PMR[7].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH7     PSI5_1.CHANNEL[3].CH_PMR[7].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL8     PSI5_1.CHANNEL[3].CH_PMR[8].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH8     PSI5_1.CHANNEL[3].CH_PMR[8].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL9     PSI5_1.CHANNEL[3].CH_PMR[9].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH9     PSI5_1.CHANNEL[3].CH_PMR[9].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL10    PSI5_1.CHANNEL[3].CH_PMR[10].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH10    PSI5_1.CHANNEL[3].CH_PMR[10].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL11    PSI5_1.CHANNEL[3].CH_PMR[11].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH11    PSI5_1.CHANNEL[3].CH_PMR[11].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL12    PSI5_1.CHANNEL[3].CH_PMR[12].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH12    PSI5_1.CHANNEL[3].CH_PMR[12].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL13    PSI5_1.CHANNEL[3].CH_PMR[13].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH13    PSI5_1.CHANNEL[3].CH_PMR[13].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL14    PSI5_1.CHANNEL[3].CH_PMR[14].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH14    PSI5_1.CHANNEL[3].CH_PMR[14].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL15    PSI5_1.CHANNEL[3].CH_PMR[15].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH15    PSI5_1.CHANNEL[3].CH_PMR[15].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL16    PSI5_1.CHANNEL[3].CH_PMR[16].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH16    PSI5_1.CHANNEL[3].CH_PMR[16].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL17    PSI5_1.CHANNEL[3].CH_PMR[17].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH17    PSI5_1.CHANNEL[3].CH_PMR[17].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL18    PSI5_1.CHANNEL[3].CH_PMR[18].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH18    PSI5_1.CHANNEL[3].CH_PMR[18].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL19    PSI5_1.CHANNEL[3].CH_PMR[19].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH19    PSI5_1.CHANNEL[3].CH_PMR[19].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL20    PSI5_1.CHANNEL[3].CH_PMR[20].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH20    PSI5_1.CHANNEL[3].CH_PMR[20].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL21    PSI5_1.CHANNEL[3].CH_PMR[21].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH21    PSI5_1.CHANNEL[3].CH_PMR[21].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL22    PSI5_1.CHANNEL[3].CH_PMR[22].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH22    PSI5_1.CHANNEL[3].CH_PMR[22].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL23    PSI5_1.CHANNEL[3].CH_PMR[23].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH23    PSI5_1.CHANNEL[3].CH_PMR[23].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL24    PSI5_1.CHANNEL[3].CH_PMR[24].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH24    PSI5_1.CHANNEL[3].CH_PMR[24].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL25    PSI5_1.CHANNEL[3].CH_PMR[25].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH25    PSI5_1.CHANNEL[3].CH_PMR[25].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL26    PSI5_1.CHANNEL[3].CH_PMR[26].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH26    PSI5_1.CHANNEL[3].CH_PMR[26].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL27    PSI5_1.CHANNEL[3].CH_PMR[27].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH27    PSI5_1.CHANNEL[3].CH_PMR[27].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL28    PSI5_1.CHANNEL[3].CH_PMR[28].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH28    PSI5_1.CHANNEL[3].CH_PMR[28].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL29    PSI5_1.CHANNEL[3].CH_PMR[29].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH29    PSI5_1.CHANNEL[3].CH_PMR[29].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL30    PSI5_1.CHANNEL[3].CH_PMR[30].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH30    PSI5_1.CHANNEL[3].CH_PMR[30].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_PMRL31    PSI5_1.CHANNEL[3].CH_PMR[31].CH_PMRL.R /* PSI5 Message Register Low i */
#define PSI5_1_CH3_PMRH31    PSI5_1.CHANNEL[3].CH_PMR[31].CH_PMRH.R /* PSI5 Message Register High i */
#define PSI5_1_CH3_SFR1      PSI5_1.CHANNEL[3].CH_SFR[0].R /* SMC Frame Register n */
#define PSI5_1_CH3_SFR2      PSI5_1.CHANNEL[3].CH_SFR[1].R /* SMC Frame Register n */
#define PSI5_1_CH3_SFR3      PSI5_1.CHANNEL[3].CH_SFR[2].R /* SMC Frame Register n */
#define PSI5_1_CH3_SFR4      PSI5_1.CHANNEL[3].CH_SFR[3].R /* SMC Frame Register n */
#define PSI5_1_CH3_SFR5      PSI5_1.CHANNEL[3].CH_SFR[4].R /* SMC Frame Register n */
#define PSI5_1_CH3_SFR6      PSI5_1.CHANNEL[3].CH_SFR[5].R /* SMC Frame Register n */
#define PSI5_1_CH3_NDSR      PSI5_1.CHANNEL[3].CH_NDSR.R   /* New Data Status Register */
#define PSI5_1_CH3_OWSR      PSI5_1.CHANNEL[3].CH_OWSR.R   /* Overwrite Status Register */
#define PSI5_1_CH3_EISR      PSI5_1.CHANNEL[3].CH_EISR.R   /* Error Indication Status Register */
#define PSI5_1_CH3_SNDSR     PSI5_1.CHANNEL[3].CH_SNDSR.R  /* Set New Data Status Register */
#define PSI5_1_CH3_SOWSR     PSI5_1.CHANNEL[3].CH_SOWSR.R  /* Set Overwrite Status Register */
#define PSI5_1_CH3_SEISR     PSI5_1.CHANNEL[3].CH_SEISR.R  /* Set Error Status Register */
#define PSI5_1_CH3_SSESR     PSI5_1.CHANNEL[3].CH_SSESR.R  /* Set SMC Error Status Register */
#define PSI5_1_CH3_STSRR     PSI5_1.CHANNEL[3].CH_STSRR.R  /* Sync Time Stamp Read Register */
#define PSI5_1_CH3_DTSRR     PSI5_1.CHANNEL[3].CH_DTSRR.R  /* Data Time Stamp Read Register */
#define PSI5_1_CH3_S1FCR     PSI5_1.CHANNEL[3].CH_SFCR[0].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH3_S2FCR     PSI5_1.CHANNEL[3].CH_SFCR[1].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH3_S3FCR     PSI5_1.CHANNEL[3].CH_SFCR[2].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH3_S4FCR     PSI5_1.CHANNEL[3].CH_SFCR[3].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH3_S5FCR     PSI5_1.CHANNEL[3].CH_SFCR[4].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH3_S6FCR     PSI5_1.CHANNEL[3].CH_SFCR[5].R /* Slot n Frame Configuration Register */
#define PSI5_1_CH3_S2SBR     PSI5_1.CHANNEL[3].CH_S2SBR.R  /* Slot 2 Start Boundary Register */
#define PSI5_1_CH3_S1SBR     PSI5_1.CHANNEL[3].CH_S1SBR.R  /* Slot 1 Start Boundary Register */
#define PSI5_1_CH3_S4SBR     PSI5_1.CHANNEL[3].CH_S4SBR.R  /* Slot 4 Start Boundary Register */
#define PSI5_1_CH3_S3SBR     PSI5_1.CHANNEL[3].CH_S3SBR.R  /* Slot 3 Start Boundary Register */
#define PSI5_1_CH3_S6SBR     PSI5_1.CHANNEL[3].CH_S6SBR.R  /* Slot 6 Start Boundary Register */
#define PSI5_1_CH3_S5SBR     PSI5_1.CHANNEL[3].CH_S5SBR.R  /* Slot 5 Start Boundary Register */
#define PSI5_1_CH3_SnEBR     PSI5_1.CHANNEL[3].CH_SNEBR.R  /* Slot n End Boundary Register */
#define PSI5_1_CH3_MDDIS_OFF PSI5_1.CHANNEL[3].CH_MDDIS_OFF.R /* Manchestor Decoder Disable Offset */
#define PSI5_1_CH3_DOBCR     PSI5_1.CHANNEL[3].CH_DOBCR.R  /* Data Output Block Configuration Register */
#define PSI5_1_CH3_PW1D      PSI5_1.CHANNEL[3].CH_PW1D.R   /* Pulse Width for Data Bit Value 1 */
#define PSI5_1_CH3_PW0D      PSI5_1.CHANNEL[3].CH_PW0D.R   /* Pulse Width for Data Bit Value 0 */
#define PSI5_1_CH3_CIPR      PSI5_1.CHANNEL[3].CH_CIPR.R   /* Counter Initialize Pulse Register */
#define PSI5_1_CH3_CTPR      PSI5_1.CHANNEL[3].CH_CTPR.R   /* Counter Target Pulse Register */
#define PSI5_1_CH3_DPRL      PSI5_1.CHANNEL[3].CH_DPRL.R   /* Data Preparation Register Low */
#define PSI5_1_CH3_DPRH      PSI5_1.CHANNEL[3].CH_DPRH.R   /* Data Preparation Register High */
#define PSI5_1_CH3_DBRL      PSI5_1.CHANNEL[3].CH_DBRL.R   /* Data Buffer Register Low */
#define PSI5_1_CH3_DBRH      PSI5_1.CHANNEL[3].CH_DBRH.R   /* Data Buffer Register High */
#define PSI5_1_CH3_DSRL      PSI5_1.CHANNEL[3].CH_DSRL.R   /* Data Shift Register Low */
#define PSI5_1_CH3_DSRH      PSI5_1.CHANNEL[3].CH_DSRH.R   /* Data Shift Register High */

/* PSI5_S */
#define PSI5_S_0_LINCR1      PSI5_S_0.LINCR1.R             /* PSI5_S LIN Control Register 1 */
#define PSI5_S_0_LINIER      PSI5_S_0.LINIER.R             /* PSI5_S LIN Interrupt enable register */
#define PSI5_S_0_LINSR       PSI5_S_0.LINSR.R              /* PSI5_S LIN Status Register */
#define PSI5_S_0_UARTCR      PSI5_S_0.UARTCR.R             /* PSI5_S UART Mode Control Register */
#define PSI5_S_0_UARTSR      PSI5_S_0.UARTSR.R             /* PSI5_S UART Mode Status Register */
#define PSI5_S_0_LINFBRR     PSI5_S_0.LINFBRR.R            /* PSI5_S LIN Fractional Baud Rate Register */
#define PSI5_S_0_LINIBRR     PSI5_S_0.LINIBRR.R            /* PSI5_S LIN Integer Baud Rate Register */
#define PSI5_S_0_LINCR2      PSI5_S_0.LINCR2.R             /* PSI5_S LIN Control Register 2 */
#define PSI5_S_0_BDRL        PSI5_S_0.BDRL.R               /* PSI5_S Buffer Data Register Least Significant */
#define PSI5_S_0_BDRM        PSI5_S_0.BDRM.R               /* PSI5_S Buffer Data Register Most Significant */
#define PSI5_S_0_GCR         PSI5_S_0.GCR.R                /* PSI5_S Global Control register */
#define PSI5_S_0_UARTPTO     PSI5_S_0.UARTPTO.R            /* PSI5_S UART Preset Timeout Register */
#define PSI5_S_0_UARTCTO     PSI5_S_0.UARTCTO.R            /* PSI5_S UART Current Timeout register */
#define PSI5_S_0_DMATXE      PSI5_S_0.DMATXE.R             /* DMA Tx Enable Register */
#define PSI5_S_0_DMARXE      PSI5_S_0.DMARXE.R             /* DMA Rx Enable Register */
#define PSI5_S_0_PTD         PSI5_S_0.PTD.R                /* PSI5_S UART Tx Idle Delay Time Register */
#define PSI5_S_0_GLCR        PSI5_S_0.GLCR.R               /* PSI5_S Global Control Register */
#define PSI5_S_0_GLSR        PSI5_S_0.GLSR.R               /* PSI5_S Global Status Register */
#define PSI5_S_0_CH_BASE_ADDR PSI5_S_0.CH_BASE_ADDR.R      /* PSI5_S CHANNEL_BASE_ADDRESS */
#define PSI5_S_0_MRU_BUF2_REG0 PSI5_S_0.MRU_BUF2_REG0.R    /* PSI5_S MRU OUTPUT BUFFER2 REGISTER0 */
#define PSI5_S_0_MRU_BUF2_REG1 PSI5_S_0.MRU_BUF2_REG1.R    /* PSI5_S MRU OUTPUT BUFFER2 REGISTER1 */
#define PSI5_S_0_MRU_BUF2_REG2 PSI5_S_0.MRU_BUF2_REG2.R    /* PSI5_S MRU OUTPUT BUFFER2 REGISTER2 */
#define PSI5_S_0_MRU_BUF2_REG3 PSI5_S_0.MRU_BUF2_REG3.R    /* PSI5_S MRU OUTPUT BUFFER2 REGISTER3 */
#define PSI5_S_0_MBOX_SR_IRQ PSI5_S_0.MBOX_SR_IRQ.R        /* PSI5_S Mbox Status Irq */
#define PSI5_S_0_ERR_SR_IRQ  PSI5_S_0.ERR_SR_IRQ.R         /* PSI5_S Error Status IRQ */
#define PSI5_S_0_MBOX_SEL_IRQ0 PSI5_S_0.IRQ[0].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_0_ERR_SEL_IRQ0 PSI5_S_0.IRQ[0].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_0_MBOX_SEL_IRQ1 PSI5_S_0.IRQ[1].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_0_ERR_SEL_IRQ1 PSI5_S_0.IRQ[1].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_0_MBOX_SEL_IRQ2 PSI5_S_0.IRQ[2].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_0_ERR_SEL_IRQ2 PSI5_S_0.IRQ[2].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_0_MBOX_SEL_IRQ3 PSI5_S_0.IRQ[3].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_0_ERR_SEL_IRQ3 PSI5_S_0.IRQ[3].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_0_MBOX_SEL_IRQ4 PSI5_S_0.IRQ[4].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_0_ERR_SEL_IRQ4 PSI5_S_0.IRQ[4].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_0_MBOX_SEL_IRQ5 PSI5_S_0.IRQ[5].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_0_ERR_SEL_IRQ5 PSI5_S_0.IRQ[5].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_0_MBOX_SEL_IRQ6 PSI5_S_0.IRQ[6].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_0_ERR_SEL_IRQ6 PSI5_S_0.IRQ[6].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_0_MBOX_SEL_IRQ7 PSI5_S_0.IRQ[7].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_0_ERR_SEL_IRQ7 PSI5_S_0.IRQ[7].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_0_WDGTSSR     PSI5_S_0.WDGTSSR.R            /* PSI5_S Watchdog Error Status and Watchdog Timestamp status register */
#define PSI5_S_0_DIRCMD      PSI5_S_0.DIRCMD.R             /* PSI5_S ECU to Sensor Direct Command Write register */
#define PSI5_S_0_CH0_MSGA    PSI5_S_0.CH0_MSGA.R           /* PSI5_S channel 0 message configuration register A */
#define PSI5_S_0_CH0_MSGB    PSI5_S_0.CH0_MSGB.R           /* PSI5_S channel 0 message configuration register B */
#define PSI5_S_0_CH0_MBOX_SR PSI5_S_0.CH0_MBOX_SR.R        /* PSI5_S Mailbox status register channel0 */
#define PSI5_S_0_CH1_MSGA    PSI5_S_0.CH[0].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_0_CH1_MSGB    PSI5_S_0.CH[0].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_0_CH1_MBOX_SR PSI5_S_0.CH[0].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_0_CH1_WD_CFGR PSI5_S_0.CH[0].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_0_CH1_DDTRIG_OFFR PSI5_S_0.CH[0].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_0_CH1_DDTRIG_PERR PSI5_S_0.CH[0].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_0_CH1_E2SCR   PSI5_S_0.CH[0].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_0_CH1_E2SSR   PSI5_S_0.CH[0].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_0_CH1_DDSR_H  PSI5_S_0.CH[0].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_0_CH1_DDSR_L  PSI5_S_0.CH[0].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_0_CH2_MSGA    PSI5_S_0.CH[1].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_0_CH2_MSGB    PSI5_S_0.CH[1].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_0_CH2_MBOX_SR PSI5_S_0.CH[1].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_0_CH2_WD_CFGR PSI5_S_0.CH[1].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_0_CH2_DDTRIG_OFFR PSI5_S_0.CH[1].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_0_CH2_DDTRIG_PERR PSI5_S_0.CH[1].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_0_CH2_E2SCR   PSI5_S_0.CH[1].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_0_CH2_E2SSR   PSI5_S_0.CH[1].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_0_CH2_DDSR_H  PSI5_S_0.CH[1].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_0_CH2_DDSR_L  PSI5_S_0.CH[1].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_0_CH3_MSGA    PSI5_S_0.CH[2].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_0_CH3_MSGB    PSI5_S_0.CH[2].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_0_CH3_MBOX_SR PSI5_S_0.CH[2].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_0_CH3_WD_CFGR PSI5_S_0.CH[2].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_0_CH3_DDTRIG_OFFR PSI5_S_0.CH[2].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_0_CH3_DDTRIG_PERR PSI5_S_0.CH[2].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_0_CH3_E2SCR   PSI5_S_0.CH[2].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_0_CH3_E2SSR   PSI5_S_0.CH[2].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_0_CH3_DDSR_H  PSI5_S_0.CH[2].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_0_CH3_DDSR_L  PSI5_S_0.CH[2].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_0_CH4_MSGA    PSI5_S_0.CH[3].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_0_CH4_MSGB    PSI5_S_0.CH[3].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_0_CH4_MBOX_SR PSI5_S_0.CH[3].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_0_CH4_WD_CFGR PSI5_S_0.CH[3].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_0_CH4_DDTRIG_OFFR PSI5_S_0.CH[3].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_0_CH4_DDTRIG_PERR PSI5_S_0.CH[3].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_0_CH4_E2SCR   PSI5_S_0.CH[3].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_0_CH4_E2SSR   PSI5_S_0.CH[3].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_0_CH4_DDSR_H  PSI5_S_0.CH[3].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_0_CH4_DDSR_L  PSI5_S_0.CH[3].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_0_CH5_MSGA    PSI5_S_0.CH[4].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_0_CH5_MSGB    PSI5_S_0.CH[4].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_0_CH5_MBOX_SR PSI5_S_0.CH[4].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_0_CH5_WD_CFGR PSI5_S_0.CH[4].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_0_CH5_DDTRIG_OFFR PSI5_S_0.CH[4].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_0_CH5_DDTRIG_PERR PSI5_S_0.CH[4].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_0_CH5_E2SCR   PSI5_S_0.CH[4].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_0_CH5_E2SSR   PSI5_S_0.CH[4].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_0_CH5_DDSR_H  PSI5_S_0.CH[4].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_0_CH5_DDSR_L  PSI5_S_0.CH[4].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_0_CH6_MSGA    PSI5_S_0.CH[5].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_0_CH6_MSGB    PSI5_S_0.CH[5].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_0_CH6_MBOX_SR PSI5_S_0.CH[5].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_0_CH6_WD_CFGR PSI5_S_0.CH[5].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_0_CH6_DDTRIG_OFFR PSI5_S_0.CH[5].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_0_CH6_DDTRIG_PERR PSI5_S_0.CH[5].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_0_CH6_E2SCR   PSI5_S_0.CH[5].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_0_CH6_E2SSR   PSI5_S_0.CH[5].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_0_CH6_DDSR_H  PSI5_S_0.CH[5].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_0_CH6_DDSR_L  PSI5_S_0.CH[5].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_0_CH7_MSGA    PSI5_S_0.CH[6].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_0_CH7_MSGB    PSI5_S_0.CH[6].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_0_CH7_MBOX_SR PSI5_S_0.CH[6].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_0_CH7_WD_CFGR PSI5_S_0.CH[6].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_0_CH7_DDTRIG_OFFR PSI5_S_0.CH[6].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_0_CH7_DDTRIG_PERR PSI5_S_0.CH[6].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_0_CH7_E2SCR   PSI5_S_0.CH[6].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_0_CH7_E2SSR   PSI5_S_0.CH[6].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_0_CH7_DDSR_H  PSI5_S_0.CH[6].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_0_CH7_DDSR_L  PSI5_S_0.CH[6].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */

#define PSI5_S_1_LINCR1      PSI5_S_1.LINCR1.R             /* PSI5_S LIN Control Register 1 */
#define PSI5_S_1_LINIER      PSI5_S_1.LINIER.R             /* PSI5_S LIN Interrupt enable register */
#define PSI5_S_1_LINSR       PSI5_S_1.LINSR.R              /* PSI5_S LIN Status Register */
#define PSI5_S_1_UARTCR      PSI5_S_1.UARTCR.R             /* PSI5_S UART Mode Control Register */
#define PSI5_S_1_UARTSR      PSI5_S_1.UARTSR.R             /* PSI5_S UART Mode Status Register */
#define PSI5_S_1_LINFBRR     PSI5_S_1.LINFBRR.R            /* PSI5_S LIN Fractional Baud Rate Register */
#define PSI5_S_1_LINIBRR     PSI5_S_1.LINIBRR.R            /* PSI5_S LIN Integer Baud Rate Register */
#define PSI5_S_1_LINCR2      PSI5_S_1.LINCR2.R             /* PSI5_S LIN Control Register 2 */
#define PSI5_S_1_BDRL        PSI5_S_1.BDRL.R               /* PSI5_S Buffer Data Register Least Significant */
#define PSI5_S_1_BDRM        PSI5_S_1.BDRM.R               /* PSI5_S Buffer Data Register Most Significant */
#define PSI5_S_1_GCR         PSI5_S_1.GCR.R                /* PSI5_S Global Control register */
#define PSI5_S_1_UARTPTO     PSI5_S_1.UARTPTO.R            /* PSI5_S UART Preset Timeout Register */
#define PSI5_S_1_UARTCTO     PSI5_S_1.UARTCTO.R            /* PSI5_S UART Current Timeout register */
#define PSI5_S_1_DMATXE      PSI5_S_1.DMATXE.R             /* DMA Tx Enable Register */
#define PSI5_S_1_DMARXE      PSI5_S_1.DMARXE.R             /* DMA Rx Enable Register */
#define PSI5_S_1_PTD         PSI5_S_1.PTD.R                /* PSI5_S UART Tx Idle Delay Time Register */
#define PSI5_S_1_GLCR        PSI5_S_1.GLCR.R               /* PSI5_S Global Control Register */
#define PSI5_S_1_GLSR        PSI5_S_1.GLSR.R               /* PSI5_S Global Status Register */
#define PSI5_S_1_CH_BASE_ADDR PSI5_S_1.CH_BASE_ADDR.R      /* PSI5_S CHANNEL_BASE_ADDRESS */
#define PSI5_S_1_MRU_BUF2_REG0 PSI5_S_1.MRU_BUF2_REG0.R    /* PSI5_S MRU OUTPUT BUFFER2 REGISTER0 */
#define PSI5_S_1_MRU_BUF2_REG1 PSI5_S_1.MRU_BUF2_REG1.R    /* PSI5_S MRU OUTPUT BUFFER2 REGISTER1 */
#define PSI5_S_1_MRU_BUF2_REG2 PSI5_S_1.MRU_BUF2_REG2.R    /* PSI5_S MRU OUTPUT BUFFER2 REGISTER2 */
#define PSI5_S_1_MRU_BUF2_REG3 PSI5_S_1.MRU_BUF2_REG3.R    /* PSI5_S MRU OUTPUT BUFFER2 REGISTER3 */
#define PSI5_S_1_MBOX_SR_IRQ PSI5_S_1.MBOX_SR_IRQ.R        /* PSI5_S Mbox Status Irq */
#define PSI5_S_1_ERR_SR_IRQ  PSI5_S_1.ERR_SR_IRQ.R         /* PSI5_S Error Status IRQ */
#define PSI5_S_1_MBOX_SEL_IRQ0 PSI5_S_1.IRQ[0].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_1_ERR_SEL_IRQ0 PSI5_S_1.IRQ[0].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_1_MBOX_SEL_IRQ1 PSI5_S_1.IRQ[1].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_1_ERR_SEL_IRQ1 PSI5_S_1.IRQ[1].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_1_MBOX_SEL_IRQ2 PSI5_S_1.IRQ[2].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_1_ERR_SEL_IRQ2 PSI5_S_1.IRQ[2].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_1_MBOX_SEL_IRQ3 PSI5_S_1.IRQ[3].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_1_ERR_SEL_IRQ3 PSI5_S_1.IRQ[3].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_1_MBOX_SEL_IRQ4 PSI5_S_1.IRQ[4].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_1_ERR_SEL_IRQ4 PSI5_S_1.IRQ[4].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_1_MBOX_SEL_IRQ5 PSI5_S_1.IRQ[5].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_1_ERR_SEL_IRQ5 PSI5_S_1.IRQ[5].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_1_MBOX_SEL_IRQ6 PSI5_S_1.IRQ[6].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_1_ERR_SEL_IRQ6 PSI5_S_1.IRQ[6].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_1_MBOX_SEL_IRQ7 PSI5_S_1.IRQ[7].MBOX_SEL_IRQ.R /* PSI5_S Mailbox select IRQ[irq_n] */
#define PSI5_S_1_ERR_SEL_IRQ7 PSI5_S_1.IRQ[7].ERR_SEL_IRQ.R /* PSI5_S Error Select IRQ[irq_n] */
#define PSI5_S_1_WDGTSSR     PSI5_S_1.WDGTSSR.R            /* PSI5_S Watchdog Error Status and Watchdog Timestamp status register */
#define PSI5_S_1_DIRCMD      PSI5_S_1.DIRCMD.R             /* PSI5_S ECU to Sensor Direct Command Write register */
#define PSI5_S_1_CH0_MSGA    PSI5_S_1.CH0_MSGA.R           /* PSI5_S channel 0 message configuration register A */
#define PSI5_S_1_CH0_MSGB    PSI5_S_1.CH0_MSGB.R           /* PSI5_S channel 0 message configuration register B */
#define PSI5_S_1_CH0_MBOX_SR PSI5_S_1.CH0_MBOX_SR.R        /* PSI5_S Mailbox status register channel0 */
#define PSI5_S_1_CH1_MSGA    PSI5_S_1.CH[0].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_1_CH1_MSGB    PSI5_S_1.CH[0].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_1_CH1_MBOX_SR PSI5_S_1.CH[0].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_1_CH1_WD_CFGR PSI5_S_1.CH[0].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_1_CH1_DDTRIG_OFFR PSI5_S_1.CH[0].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_1_CH1_DDTRIG_PERR PSI5_S_1.CH[0].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_1_CH1_E2SCR   PSI5_S_1.CH[0].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_1_CH1_E2SSR   PSI5_S_1.CH[0].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_1_CH1_DDSR_H  PSI5_S_1.CH[0].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_1_CH1_DDSR_L  PSI5_S_1.CH[0].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_1_CH2_MSGA    PSI5_S_1.CH[1].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_1_CH2_MSGB    PSI5_S_1.CH[1].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_1_CH2_MBOX_SR PSI5_S_1.CH[1].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_1_CH2_WD_CFGR PSI5_S_1.CH[1].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_1_CH2_DDTRIG_OFFR PSI5_S_1.CH[1].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_1_CH2_DDTRIG_PERR PSI5_S_1.CH[1].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_1_CH2_E2SCR   PSI5_S_1.CH[1].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_1_CH2_E2SSR   PSI5_S_1.CH[1].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_1_CH2_DDSR_H  PSI5_S_1.CH[1].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_1_CH2_DDSR_L  PSI5_S_1.CH[1].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_1_CH3_MSGA    PSI5_S_1.CH[2].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_1_CH3_MSGB    PSI5_S_1.CH[2].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_1_CH3_MBOX_SR PSI5_S_1.CH[2].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_1_CH3_WD_CFGR PSI5_S_1.CH[2].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_1_CH3_DDTRIG_OFFR PSI5_S_1.CH[2].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_1_CH3_DDTRIG_PERR PSI5_S_1.CH[2].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_1_CH3_E2SCR   PSI5_S_1.CH[2].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_1_CH3_E2SSR   PSI5_S_1.CH[2].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_1_CH3_DDSR_H  PSI5_S_1.CH[2].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_1_CH3_DDSR_L  PSI5_S_1.CH[2].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_1_CH4_MSGA    PSI5_S_1.CH[3].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_1_CH4_MSGB    PSI5_S_1.CH[3].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_1_CH4_MBOX_SR PSI5_S_1.CH[3].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_1_CH4_WD_CFGR PSI5_S_1.CH[3].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_1_CH4_DDTRIG_OFFR PSI5_S_1.CH[3].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_1_CH4_DDTRIG_PERR PSI5_S_1.CH[3].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_1_CH4_E2SCR   PSI5_S_1.CH[3].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_1_CH4_E2SSR   PSI5_S_1.CH[3].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_1_CH4_DDSR_H  PSI5_S_1.CH[3].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_1_CH4_DDSR_L  PSI5_S_1.CH[3].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_1_CH5_MSGA    PSI5_S_1.CH[4].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_1_CH5_MSGB    PSI5_S_1.CH[4].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_1_CH5_MBOX_SR PSI5_S_1.CH[4].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_1_CH5_WD_CFGR PSI5_S_1.CH[4].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_1_CH5_DDTRIG_OFFR PSI5_S_1.CH[4].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_1_CH5_DDTRIG_PERR PSI5_S_1.CH[4].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_1_CH5_E2SCR   PSI5_S_1.CH[4].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_1_CH5_E2SSR   PSI5_S_1.CH[4].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_1_CH5_DDSR_H  PSI5_S_1.CH[4].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_1_CH5_DDSR_L  PSI5_S_1.CH[4].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_1_CH6_MSGA    PSI5_S_1.CH[5].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_1_CH6_MSGB    PSI5_S_1.CH[5].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_1_CH6_MBOX_SR PSI5_S_1.CH[5].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_1_CH6_WD_CFGR PSI5_S_1.CH[5].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_1_CH6_DDTRIG_OFFR PSI5_S_1.CH[5].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_1_CH6_DDTRIG_PERR PSI5_S_1.CH[5].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_1_CH6_E2SCR   PSI5_S_1.CH[5].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_1_CH6_E2SSR   PSI5_S_1.CH[5].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_1_CH6_DDSR_H  PSI5_S_1.CH[5].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_1_CH6_DDSR_L  PSI5_S_1.CH[5].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */
#define PSI5_S_1_CH7_MSGA    PSI5_S_1.CH[6].MSGA.R         /* PSI5_S channel message configuration register A */
#define PSI5_S_1_CH7_MSGB    PSI5_S_1.CH[6].MSGB.R         /* PSI5_S channel message configuration register B */
#define PSI5_S_1_CH7_MBOX_SR PSI5_S_1.CH[6].MBOX_SR.R      /* PSI5_S Mailbox status register channel */
#define PSI5_S_1_CH7_WD_CFGR PSI5_S_1.CH[6].WD_CFGR.R      /* PSI5_S channel watchdog configuration register */
#define PSI5_S_1_CH7_DDTRIG_OFFR PSI5_S_1.CH[6].DDTRIG_OFFR.R /* PSI5_S DDSR Trigger offset register channel */
#define PSI5_S_1_CH7_DDTRIG_PERR PSI5_S_1.CH[6].DDTRIG_PERR.R /* PSI5_S DDSR Trigger period register channel */
#define PSI5_S_1_CH7_E2SCR   PSI5_S_1.CH[6].E2SCR.R        /* PSI5_S ECU to Sensor Control Register */
#define PSI5_S_1_CH7_E2SSR   PSI5_S_1.CH[6].E2SSR.R        /* PSI5_S ECU to Sensor Status Register */
#define PSI5_S_1_CH7_DDSR_H  PSI5_S_1.CH[6].DDSR_H.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register High */
#define PSI5_S_1_CH7_DDSR_L  PSI5_S_1.CH[6].DDSR_L.R       /* PSI5_S channel1 ECU to Sensor Downstream Data Shift Register Low */

/* QMAN_CNTRL */
#define CEVA_SPF2__QMAN_CNTRL_QMAN_IDM_CROSS CEVA_SPF2__QMAN_CNTRL.QMAN_IDM_CROSS.R /* QMAN IDM CROSS */
#define CEVA_SPF2__QMAN_CNTRL_QMAN_ACTIVE CEVA_SPF2__QMAN_CNTRL.QMAN_ACTIVE.R /* QMAN ACTIVE */
#define CEVA_SPF2__QMAN_CNTRL_QMAN_VI_MASK CEVA_SPF2__QMAN_CNTRL.QMAN_VI_MASK.R /* QMAN VI MASK */
#define CEVA_SPF2__QMAN_CNTRL_QMAN_IRQ_STATUS CEVA_SPF2__QMAN_CNTRL.QMAN_IRQ_STATUS.R /* QMAN IRQ STATUS */
#define CEVA_SPF2__QMAN_CNTRL_QMAN_RST CEVA_SPF2__QMAN_CNTRL.QMAN_RST.R /* QMAN RST */
#define CEVA_SPF2__QMAN_CNTRL_QMAN_QFULL CEVA_SPF2__QMAN_CNTRL.QMAN_QFULL.R /* QMAN QFULL */
#define CEVA_SPF2__QMAN_CNTRL_QMAN_QFULL_MASK CEVA_SPF2__QMAN_CNTRL.QMAN_QFULL_MASK.R /* QMAN QFULL MASK */
#define CEVA_SPF2__QMAN_CNTRL_QMAN_PAUSE CEVA_SPF2__QMAN_CNTRL.QMAN_PAUSE.R /* QMAN PAUSE */

/* QUEUE_DESCRIPTOR */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_DSC_CFG CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_DSC_CFG.R /* QPUSH DSC CFG */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_DSC_SRCP CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_DSC_SRCP.R /* QPUSH DSC SRCP */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_DSC_DSTP CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_DSC_DSTP.R /* QPUSH DSC DSTP */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_DSC_DMASZ CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_DSC_DMASZ.R /* QPUSH DSC DMASZ */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_DSC_SRC CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_DSC_SRC.R /* QPUSH DSC SRC */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_DSC_LINE CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_DSC_LINE.R /* QPUSH DSC LINE */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_DSC_DST CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_DSC_DST.R /* QPUSH DSC DST */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_DSC_HGHT CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_DSC_HGHT.R /* QPUSH DSC HGHT */
#define CEVA_SPF2__QUEUE_DESCRIPTOR_QPUSH_NUM CEVA_SPF2__QUEUE_DESCRIPTOR.QPUSH_NUM.R /* QPUSH NUM */

/* QUEUE_MANAGER */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_EN_DEPTH CEVA_SPF2__QUEUE_MANAGER.Q0_EN_DEPTH.R /* QX EN DEPTH */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_FIRST_ADDR CEVA_SPF2__QUEUE_MANAGER.Q0_FIRST_ADDR.R /* Q0 FIRST ADDR */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_BASE_PTR CEVA_SPF2__QUEUE_MANAGER.Q0_BASE_PTR.R /* Q0 BASE PTR */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_CHNK_SIZE CEVA_SPF2__QUEUE_MANAGER.Q0_CHNK_SIZE.R /* Q0 CHNK SIZE */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_EN_INC0 CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_EN_INC0.R /* Q0 DSC EN INC0 */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_STATUS CEVA_SPF2__QUEUE_MANAGER.Q0_STATUS.R /* Q0 STATUS */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_RPTR_INT CEVA_SPF2__QUEUE_MANAGER.Q0_RPTR_INT.R /* Q0 RPTR INT */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_CFG CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_CFG.R /* Q0 DSC CFG */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_SRCP CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_SRCP.R /* Q0 DSC SRCP */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_DSTP CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_DSTP.R /* Q0 DSC DSTP */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_DMASZ CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_DMASZ.R /* Q0 DSC DMASZ */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_SRC CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_SRC.R /* Q0 DSC SRC */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_LINE CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_LINE.R /* Q0 DSC LINE */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_DST CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_DST.R /* Q0 DSC DST */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_HGHT CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_HGHT.R /* Q0 DSC HGHT */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_DSC_EN_INC1 CEVA_SPF2__QUEUE_MANAGER.Q0_DSC_EN_INC1.R /* Q0 DSC EN INC1 */
#define CEVA_SPF2__QUEUE_MANAGER_Q0_EN_CNT CEVA_SPF2__QUEUE_MANAGER.Q0_EN_CNT.R /* Q0 EN CNT */

/* QUEUE_MANAGER2 */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_EN_DEPTH CEVA_SPF2__QUEUE_MANAGER2.Q2_EN_DEPTH.R /* QX EN DEPTH */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_FIRST_ADDR CEVA_SPF2__QUEUE_MANAGER2.Q2_FIRST_ADDR.R /* Q2 FIRST ADDR */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_BASE_PTR CEVA_SPF2__QUEUE_MANAGER2.Q2_BASE_PTR.R /* Q2 BASE PTR */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_CHNK_SIZE CEVA_SPF2__QUEUE_MANAGER2.Q2_CHNK_SIZE.R /* Q2 CHNK SIZE */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_EN_INC0 CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_EN_INC0.R /* Q2 DSC EN INC0 */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_STATUS CEVA_SPF2__QUEUE_MANAGER2.Q2_STATUS.R /* Q2 STATUS */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_RPTR_INT CEVA_SPF2__QUEUE_MANAGER2.Q2_RPTR_INT.R /* Q2 RPTR INT */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_CFG CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_CFG.R /* Q2 DSC CFG */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_SRCP CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_SRCP.R /* Q2 DSC SRCP */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_DSTP CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_DSTP.R /* Q2 DSC DSTP */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_DMASZ CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_DMASZ.R /* Q2 DSC DMASZ */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_SRC CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_SRC.R /* Q2 DSC SRC */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_LINE CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_LINE.R /* Q2 DSC LINE */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_DST CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_DST.R /* Q2 DSC DST */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_HGHT CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_HGHT.R /* Q2 DSC HGHT */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_DSC_EN_INC1 CEVA_SPF2__QUEUE_MANAGER2.Q2_DSC_EN_INC1.R /* Q2 DSC EN INC1 */
#define CEVA_SPF2__QUEUE_MANAGER2_Q2_EN_CNT CEVA_SPF2__QUEUE_MANAGER2.Q2_EN_CNT.R /* Q2 EN CNT */

/* QUEUE_MANAGER3 */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_EN_DEPTH CEVA_SPF2__QUEUE_MANAGER3.Q3_EN_DEPTH.R /* QX EN DEPTH */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_FIRST_ADDR CEVA_SPF2__QUEUE_MANAGER3.Q3_FIRST_ADDR.R /* Q3 FIRST ADDR */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_BASE_PTR CEVA_SPF2__QUEUE_MANAGER3.Q3_BASE_PTR.R /* Q3 BASE PTR */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_CHNK_SIZE CEVA_SPF2__QUEUE_MANAGER3.Q3_CHNK_SIZE.R /* Q3 CHNK SIZE */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_EN_INC0 CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_EN_INC0.R /* Q3 DSC EN INC0 */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_STATUS CEVA_SPF2__QUEUE_MANAGER3.Q3_STATUS.R /* Q3 STATUS */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_RPTR_INT CEVA_SPF2__QUEUE_MANAGER3.Q3_RPTR_INT.R /* Q3 RPTR INT */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_CFG CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_CFG.R /* Q3 DSC CFG */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_SRCP CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_SRCP.R /* Q3 DSC SRCP */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_DSTP CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_DSTP.R /* Q3 DSC DSTP */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_DMASZ CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_DMASZ.R /* Q3 DSC DMASZ */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_SRC CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_SRC.R /* Q3 DSC SRC */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_LINE CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_LINE.R /* Q3 DSC LINE */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_DST CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_DST.R /* Q3 DSC DST */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_HGHT CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_HGHT.R /* Q3 DSC HGHT */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_DSC_EN_INC1 CEVA_SPF2__QUEUE_MANAGER3.Q3_DSC_EN_INC1.R /* Q3 DSC EN INC1 */
#define CEVA_SPF2__QUEUE_MANAGER3_Q3_EN_CNT CEVA_SPF2__QUEUE_MANAGER3.Q3_EN_CNT.R /* Q3 EN CNT */

/* QuadSPI */
#define QUADSPI_0_MCR        QUADSPI_0.MCR.R               /* Module Configuration Register */
#define QUADSPI_0_IPCR       QUADSPI_0.IPCR.R              /* IP Configuration Register */
#define QUADSPI_0_FLSHCR     QUADSPI_0.FLSHCR.R            /* Flash Memory Configuration Register */
#define QUADSPI_0_BUF0CR     QUADSPI_0.BUF0CR.R            /* Buffer 0 Configuration Register */
#define QUADSPI_0_BUF1CR     QUADSPI_0.BUF1CR.R            /* Buffer 1 Configuration Register */
#define QUADSPI_0_BUF2CR     QUADSPI_0.BUF2CR.R            /* Buffer 2 Configuration Register */
#define QUADSPI_0_BUF3CR     QUADSPI_0.BUF3CR.R            /* Buffer 3 Configuration Register */
#define QUADSPI_0_BFGENCR    QUADSPI_0.BFGENCR.R           /* Buffer Generic Configuration Register */
#define QUADSPI_0_SOCCR      QUADSPI_0.SOCCR.R             /* SOC Configuration Register */
#define QUADSPI_0_BUF0IND    QUADSPI_0.BUF0IND.R           /* Buffer 0 Top Index Register */
#define QUADSPI_0_BUF1IND    QUADSPI_0.BUF1IND.R           /* Buffer 1 Top Index Register */
#define QUADSPI_0_BUF2IND    QUADSPI_0.BUF2IND.R           /* Buffer 2 Top Index Register */
#define QUADSPI_0_AWRCR      QUADSPI_0.AWRCR.R             /* AHB Write Configuration Register */
#define QUADSPI_0_DLLCRA     QUADSPI_0.DLLCRA.R            /* DLL Flash Memory A Configuration Register */
#define QUADSPI_0_PARITYCR   QUADSPI_0.PARITYCR.R          /* Parity Configuration Register */
#define QUADSPI_0_SFAR       QUADSPI_0.SFAR.R              /* Serial Flash Memory Address Register */
#define QUADSPI_0_SFACR      QUADSPI_0.SFACR.R             /* Serial Flash Memory Address Configuration Register */
#define QUADSPI_0_SMPR       QUADSPI_0.SMPR.R              /* Sampling Register */
#define QUADSPI_0_RBSR       QUADSPI_0.RBSR.R              /* RX Buffer Status Register */
#define QUADSPI_0_RBCT       QUADSPI_0.RBCT.R              /* RX Buffer Control Register */
#define QUADSPI_0_AWRSR      QUADSPI_0.AWRSR.R             /* AHB Write Status Register */
#define QUADSPI_0_DLLSR      QUADSPI_0.DLLSR.R             /* DLL Status Register */
#define QUADSPI_0_DLCR       QUADSPI_0.DLCR.R              /* Data Learning Configuration Register */
#define QUADSPI_0_DLSR_FA    QUADSPI_0.DLSR_FA.R           /* Data Learning Status Flash Memory A Register */
#define QUADSPI_0_TBSR       QUADSPI_0.TBSR.R              /* TX Buffer Status Register */
#define QUADSPI_0_TBDR       QUADSPI_0.TBDR.R              /* TX Buffer Data Register */
#define QUADSPI_0_TBCT       QUADSPI_0.TBCT.R              /* TX Buffer Control Register */
#define QUADSPI_0_SR         QUADSPI_0.SR.R                /* Status Register */
#define QUADSPI_0_FR         QUADSPI_0.FR.R                /* Flag Register */
#define QUADSPI_0_RSER       QUADSPI_0.RSER.R              /* Interrupt and DMA Request Select and Enable Register */
#define QUADSPI_0_SPTRCLR    QUADSPI_0.SPTRCLR.R           /* Sequence Pointer Clear Register */
#define QUADSPI_0_SFA1AD     QUADSPI_0.SFA1AD.R            /* Serial Flash Memory A1 Top Address Register */
#define QUADSPI_0_SFA2AD     QUADSPI_0.SFA2AD.R            /* Serial Flash Memory A2 Top Address Register */
#define QUADSPI_0_DLPR       QUADSPI_0.DLPR.R              /* Data Learn Pattern Register */
#define QUADSPI_0_FAILA_ADDR QUADSPI_0.FAILA_ADDR.R        /* Flash Memory A Failing Address Status Register */
#define QUADSPI_0_RBDR0      QUADSPI_0.RBDR[0].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR1      QUADSPI_0.RBDR[1].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR2      QUADSPI_0.RBDR[2].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR3      QUADSPI_0.RBDR[3].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR4      QUADSPI_0.RBDR[4].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR5      QUADSPI_0.RBDR[5].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR6      QUADSPI_0.RBDR[6].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR7      QUADSPI_0.RBDR[7].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR8      QUADSPI_0.RBDR[8].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR9      QUADSPI_0.RBDR[9].R           /* RX Buffer Data Register */
#define QUADSPI_0_RBDR10     QUADSPI_0.RBDR[10].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR11     QUADSPI_0.RBDR[11].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR12     QUADSPI_0.RBDR[12].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR13     QUADSPI_0.RBDR[13].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR14     QUADSPI_0.RBDR[14].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR15     QUADSPI_0.RBDR[15].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR16     QUADSPI_0.RBDR[16].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR17     QUADSPI_0.RBDR[17].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR18     QUADSPI_0.RBDR[18].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR19     QUADSPI_0.RBDR[19].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR20     QUADSPI_0.RBDR[20].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR21     QUADSPI_0.RBDR[21].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR22     QUADSPI_0.RBDR[22].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR23     QUADSPI_0.RBDR[23].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR24     QUADSPI_0.RBDR[24].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR25     QUADSPI_0.RBDR[25].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR26     QUADSPI_0.RBDR[26].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR27     QUADSPI_0.RBDR[27].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR28     QUADSPI_0.RBDR[28].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR29     QUADSPI_0.RBDR[29].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR30     QUADSPI_0.RBDR[30].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR31     QUADSPI_0.RBDR[31].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR32     QUADSPI_0.RBDR[32].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR33     QUADSPI_0.RBDR[33].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR34     QUADSPI_0.RBDR[34].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR35     QUADSPI_0.RBDR[35].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR36     QUADSPI_0.RBDR[36].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR37     QUADSPI_0.RBDR[37].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR38     QUADSPI_0.RBDR[38].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR39     QUADSPI_0.RBDR[39].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR40     QUADSPI_0.RBDR[40].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR41     QUADSPI_0.RBDR[41].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR42     QUADSPI_0.RBDR[42].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR43     QUADSPI_0.RBDR[43].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR44     QUADSPI_0.RBDR[44].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR45     QUADSPI_0.RBDR[45].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR46     QUADSPI_0.RBDR[46].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR47     QUADSPI_0.RBDR[47].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR48     QUADSPI_0.RBDR[48].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR49     QUADSPI_0.RBDR[49].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR50     QUADSPI_0.RBDR[50].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR51     QUADSPI_0.RBDR[51].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR52     QUADSPI_0.RBDR[52].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR53     QUADSPI_0.RBDR[53].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR54     QUADSPI_0.RBDR[54].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR55     QUADSPI_0.RBDR[55].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR56     QUADSPI_0.RBDR[56].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR57     QUADSPI_0.RBDR[57].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR58     QUADSPI_0.RBDR[58].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR59     QUADSPI_0.RBDR[59].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR60     QUADSPI_0.RBDR[60].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR61     QUADSPI_0.RBDR[61].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR62     QUADSPI_0.RBDR[62].R          /* RX Buffer Data Register */
#define QUADSPI_0_RBDR63     QUADSPI_0.RBDR[63].R          /* RX Buffer Data Register */
#define QUADSPI_0_LUTKEY     QUADSPI_0.LUTKEY.R            /* LUT Key Register */
#define QUADSPI_0_LCKCR      QUADSPI_0.LCKCR.R             /* LUT Lock Configuration Register */
#define QUADSPI_0_LUT0       QUADSPI_0.LUT[0].R            /* LUT Register */
#define QUADSPI_0_LUT1       QUADSPI_0.LUT[1].R            /* LUT Register */
#define QUADSPI_0_LUT2       QUADSPI_0.LUT[2].R            /* LUT Register */
#define QUADSPI_0_LUT3       QUADSPI_0.LUT[3].R            /* LUT Register */
#define QUADSPI_0_LUT4       QUADSPI_0.LUT[4].R            /* LUT Register */
#define QUADSPI_0_LUT5       QUADSPI_0.LUT[5].R            /* LUT Register */
#define QUADSPI_0_LUT6       QUADSPI_0.LUT[6].R            /* LUT Register */
#define QUADSPI_0_LUT7       QUADSPI_0.LUT[7].R            /* LUT Register */
#define QUADSPI_0_LUT8       QUADSPI_0.LUT[8].R            /* LUT Register */
#define QUADSPI_0_LUT9       QUADSPI_0.LUT[9].R            /* LUT Register */
#define QUADSPI_0_LUT10      QUADSPI_0.LUT[10].R           /* LUT Register */
#define QUADSPI_0_LUT11      QUADSPI_0.LUT[11].R           /* LUT Register */
#define QUADSPI_0_LUT12      QUADSPI_0.LUT[12].R           /* LUT Register */
#define QUADSPI_0_LUT13      QUADSPI_0.LUT[13].R           /* LUT Register */
#define QUADSPI_0_LUT14      QUADSPI_0.LUT[14].R           /* LUT Register */
#define QUADSPI_0_LUT15      QUADSPI_0.LUT[15].R           /* LUT Register */
#define QUADSPI_0_LUT16      QUADSPI_0.LUT[16].R           /* LUT Register */
#define QUADSPI_0_LUT17      QUADSPI_0.LUT[17].R           /* LUT Register */
#define QUADSPI_0_LUT18      QUADSPI_0.LUT[18].R           /* LUT Register */
#define QUADSPI_0_LUT19      QUADSPI_0.LUT[19].R           /* LUT Register */
#define QUADSPI_0_LUT20      QUADSPI_0.LUT[20].R           /* LUT Register */
#define QUADSPI_0_LUT21      QUADSPI_0.LUT[21].R           /* LUT Register */
#define QUADSPI_0_LUT22      QUADSPI_0.LUT[22].R           /* LUT Register */
#define QUADSPI_0_LUT23      QUADSPI_0.LUT[23].R           /* LUT Register */
#define QUADSPI_0_LUT24      QUADSPI_0.LUT[24].R           /* LUT Register */
#define QUADSPI_0_LUT25      QUADSPI_0.LUT[25].R           /* LUT Register */
#define QUADSPI_0_LUT26      QUADSPI_0.LUT[26].R           /* LUT Register */
#define QUADSPI_0_LUT27      QUADSPI_0.LUT[27].R           /* LUT Register */
#define QUADSPI_0_LUT28      QUADSPI_0.LUT[28].R           /* LUT Register */
#define QUADSPI_0_LUT29      QUADSPI_0.LUT[29].R           /* LUT Register */
#define QUADSPI_0_LUT30      QUADSPI_0.LUT[30].R           /* LUT Register */
#define QUADSPI_0_LUT31      QUADSPI_0.LUT[31].R           /* LUT Register */
#define QUADSPI_0_LUT32      QUADSPI_0.LUT[32].R           /* LUT Register */
#define QUADSPI_0_LUT33      QUADSPI_0.LUT[33].R           /* LUT Register */
#define QUADSPI_0_LUT34      QUADSPI_0.LUT[34].R           /* LUT Register */
#define QUADSPI_0_LUT35      QUADSPI_0.LUT[35].R           /* LUT Register */
#define QUADSPI_0_LUT36      QUADSPI_0.LUT[36].R           /* LUT Register */
#define QUADSPI_0_LUT37      QUADSPI_0.LUT[37].R           /* LUT Register */
#define QUADSPI_0_LUT38      QUADSPI_0.LUT[38].R           /* LUT Register */
#define QUADSPI_0_LUT39      QUADSPI_0.LUT[39].R           /* LUT Register */
#define QUADSPI_0_LUT40      QUADSPI_0.LUT[40].R           /* LUT Register */
#define QUADSPI_0_LUT41      QUADSPI_0.LUT[41].R           /* LUT Register */
#define QUADSPI_0_LUT42      QUADSPI_0.LUT[42].R           /* LUT Register */
#define QUADSPI_0_LUT43      QUADSPI_0.LUT[43].R           /* LUT Register */
#define QUADSPI_0_LUT44      QUADSPI_0.LUT[44].R           /* LUT Register */
#define QUADSPI_0_LUT45      QUADSPI_0.LUT[45].R           /* LUT Register */
#define QUADSPI_0_LUT46      QUADSPI_0.LUT[46].R           /* LUT Register */
#define QUADSPI_0_LUT47      QUADSPI_0.LUT[47].R           /* LUT Register */
#define QUADSPI_0_LUT48      QUADSPI_0.LUT[48].R           /* LUT Register */
#define QUADSPI_0_LUT49      QUADSPI_0.LUT[49].R           /* LUT Register */
#define QUADSPI_0_LUT50      QUADSPI_0.LUT[50].R           /* LUT Register */
#define QUADSPI_0_LUT51      QUADSPI_0.LUT[51].R           /* LUT Register */
#define QUADSPI_0_LUT52      QUADSPI_0.LUT[52].R           /* LUT Register */
#define QUADSPI_0_LUT53      QUADSPI_0.LUT[53].R           /* LUT Register */
#define QUADSPI_0_LUT54      QUADSPI_0.LUT[54].R           /* LUT Register */
#define QUADSPI_0_LUT55      QUADSPI_0.LUT[55].R           /* LUT Register */
#define QUADSPI_0_LUT56      QUADSPI_0.LUT[56].R           /* LUT Register */
#define QUADSPI_0_LUT57      QUADSPI_0.LUT[57].R           /* LUT Register */
#define QUADSPI_0_LUT58      QUADSPI_0.LUT[58].R           /* LUT Register */
#define QUADSPI_0_LUT59      QUADSPI_0.LUT[59].R           /* LUT Register */
#define QUADSPI_0_LUT60      QUADSPI_0.LUT[60].R           /* LUT Register */
#define QUADSPI_0_LUT61      QUADSPI_0.LUT[61].R           /* LUT Register */
#define QUADSPI_0_LUT62      QUADSPI_0.LUT[62].R           /* LUT Register */
#define QUADSPI_0_LUT63      QUADSPI_0.LUT[63].R           /* LUT Register */
#define QUADSPI_0_LUT64      QUADSPI_0.LUT[64].R           /* LUT Register */
#define QUADSPI_0_LUT65      QUADSPI_0.LUT[65].R           /* LUT Register */
#define QUADSPI_0_LUT66      QUADSPI_0.LUT[66].R           /* LUT Register */
#define QUADSPI_0_LUT67      QUADSPI_0.LUT[67].R           /* LUT Register */
#define QUADSPI_0_LUT68      QUADSPI_0.LUT[68].R           /* LUT Register */
#define QUADSPI_0_LUT69      QUADSPI_0.LUT[69].R           /* LUT Register */
#define QUADSPI_0_LUT70      QUADSPI_0.LUT[70].R           /* LUT Register */
#define QUADSPI_0_LUT71      QUADSPI_0.LUT[71].R           /* LUT Register */
#define QUADSPI_0_LUT72      QUADSPI_0.LUT[72].R           /* LUT Register */
#define QUADSPI_0_LUT73      QUADSPI_0.LUT[73].R           /* LUT Register */
#define QUADSPI_0_LUT74      QUADSPI_0.LUT[74].R           /* LUT Register */
#define QUADSPI_0_LUT75      QUADSPI_0.LUT[75].R           /* LUT Register */
#define QUADSPI_0_LUT76      QUADSPI_0.LUT[76].R           /* LUT Register */
#define QUADSPI_0_LUT77      QUADSPI_0.LUT[77].R           /* LUT Register */
#define QUADSPI_0_LUT78      QUADSPI_0.LUT[78].R           /* LUT Register */
#define QUADSPI_0_LUT79      QUADSPI_0.LUT[79].R           /* LUT Register */
#define QUADSPI_0_FRAD0_WORD0 QUADSPI_0.FRAD[0].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_0_FRAD0_WORD1 QUADSPI_0.FRAD[0].WORD1.R    /* Flash Region End Address */
#define QUADSPI_0_FRAD0_WORD2 QUADSPI_0.FRAD[0].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_0_FRAD0_WORD3 QUADSPI_0.FRAD[0].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_0_FRAD0_WORD4 QUADSPI_0.FRAD[0].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_0_FRAD0_WORD5 QUADSPI_0.FRAD[0].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_0_FRAD1_WORD0 QUADSPI_0.FRAD[1].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_0_FRAD1_WORD1 QUADSPI_0.FRAD[1].WORD1.R    /* Flash Region End Address */
#define QUADSPI_0_FRAD1_WORD2 QUADSPI_0.FRAD[1].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_0_FRAD1_WORD3 QUADSPI_0.FRAD[1].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_0_FRAD1_WORD4 QUADSPI_0.FRAD[1].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_0_FRAD1_WORD5 QUADSPI_0.FRAD[1].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_0_FRAD2_WORD0 QUADSPI_0.FRAD[2].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_0_FRAD2_WORD1 QUADSPI_0.FRAD[2].WORD1.R    /* Flash Region End Address */
#define QUADSPI_0_FRAD2_WORD2 QUADSPI_0.FRAD[2].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_0_FRAD2_WORD3 QUADSPI_0.FRAD[2].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_0_FRAD2_WORD4 QUADSPI_0.FRAD[2].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_0_FRAD2_WORD5 QUADSPI_0.FRAD[2].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_0_FRAD3_WORD0 QUADSPI_0.FRAD[3].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_0_FRAD3_WORD1 QUADSPI_0.FRAD[3].WORD1.R    /* Flash Region End Address */
#define QUADSPI_0_FRAD3_WORD2 QUADSPI_0.FRAD[3].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_0_FRAD3_WORD3 QUADSPI_0.FRAD[3].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_0_FRAD3_WORD4 QUADSPI_0.FRAD[3].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_0_FRAD3_WORD5 QUADSPI_0.FRAD[3].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_0_FRAD4_WORD0 QUADSPI_0.FRAD[4].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_0_FRAD4_WORD1 QUADSPI_0.FRAD[4].WORD1.R    /* Flash Region End Address */
#define QUADSPI_0_FRAD4_WORD2 QUADSPI_0.FRAD[4].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_0_FRAD4_WORD3 QUADSPI_0.FRAD[4].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_0_FRAD4_WORD4 QUADSPI_0.FRAD[4].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_0_FRAD4_WORD5 QUADSPI_0.FRAD[4].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_0_FRAD5_WORD0 QUADSPI_0.FRAD[5].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_0_FRAD5_WORD1 QUADSPI_0.FRAD[5].WORD1.R    /* Flash Region End Address */
#define QUADSPI_0_FRAD5_WORD2 QUADSPI_0.FRAD[5].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_0_FRAD5_WORD3 QUADSPI_0.FRAD[5].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_0_FRAD5_WORD4 QUADSPI_0.FRAD[5].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_0_FRAD5_WORD5 QUADSPI_0.FRAD[5].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_0_FRAD6_WORD0 QUADSPI_0.FRAD[6].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_0_FRAD6_WORD1 QUADSPI_0.FRAD[6].WORD1.R    /* Flash Region End Address */
#define QUADSPI_0_FRAD6_WORD2 QUADSPI_0.FRAD[6].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_0_FRAD6_WORD3 QUADSPI_0.FRAD[6].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_0_FRAD6_WORD4 QUADSPI_0.FRAD[6].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_0_FRAD6_WORD5 QUADSPI_0.FRAD[6].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_0_FRAD7_WORD0 QUADSPI_0.FRAD[7].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_0_FRAD7_WORD1 QUADSPI_0.FRAD[7].WORD1.R    /* Flash Region End Address */
#define QUADSPI_0_FRAD7_WORD2 QUADSPI_0.FRAD[7].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_0_FRAD7_WORD3 QUADSPI_0.FRAD[7].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_0_FRAD7_WORD4 QUADSPI_0.FRAD[7].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_0_FRAD7_WORD5 QUADSPI_0.FRAD[7].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_0_TG0MDAD    QUADSPI_0.MDAD[0].TGMDAD.R    /* Target Group n Master Domain Access Descriptor */
#define QUADSPI_0_TG0SFAR    QUADSPI_0.MDAD[0].TGSFAR.R    /* Target Group n SFAR Address */
#define QUADSPI_0_TG0SFARS   QUADSPI_0.MDAD[0].TGSFARS.R   /* Target Group n SFAR Status */
#define QUADSPI_0_TG0IPCRS   QUADSPI_0.MDAD[0].TGIPCRS.R   /* Target Group n IPCR Status */
#define QUADSPI_0_TG1MDAD    QUADSPI_0.MDAD[1].TGMDAD.R    /* Target Group n Master Domain Access Descriptor */
#define QUADSPI_0_TG1SFAR    QUADSPI_0.MDAD[1].TGSFAR.R    /* Target Group n SFAR Address */
#define QUADSPI_0_TG1SFARS   QUADSPI_0.MDAD[1].TGSFARS.R   /* Target Group n SFAR Status */
#define QUADSPI_0_TG1IPCRS   QUADSPI_0.MDAD[1].TGIPCRS.R   /* Target Group n IPCR Status */
#define QUADSPI_0_MGC        QUADSPI_0.MGC.R               /* Master Global Configuration */
#define QUADSPI_0_MRC        QUADSPI_0.MRC.R               /* Master Read Command */
#define QUADSPI_0_MTO        QUADSPI_0.MTO.R               /* Master Timeout */
#define QUADSPI_0_FLSEQREQ   QUADSPI_0.FLSEQREQ.R          /* FlashSeq Request */
#define QUADSPI_0_FSMSTAT    QUADSPI_0.FSMSTAT.R           /* FSM Status */
#define QUADSPI_0_IPSERROR   QUADSPI_0.IPSERROR.R          /* IPS Error */
#define QUADSPI_0_ERRSTAT    QUADSPI_0.ERRSTAT.R           /* Error Status */
#define QUADSPI_0_INT_EN     QUADSPI_0.INT_EN.R            /* Interrupt Enable */

#define QUADSPI_1_MCR        QUADSPI_1.MCR.R               /* Module Configuration Register */
#define QUADSPI_1_IPCR       QUADSPI_1.IPCR.R              /* IP Configuration Register */
#define QUADSPI_1_FLSHCR     QUADSPI_1.FLSHCR.R            /* Flash Memory Configuration Register */
#define QUADSPI_1_BUF0CR     QUADSPI_1.BUF0CR.R            /* Buffer 0 Configuration Register */
#define QUADSPI_1_BUF1CR     QUADSPI_1.BUF1CR.R            /* Buffer 1 Configuration Register */
#define QUADSPI_1_BUF2CR     QUADSPI_1.BUF2CR.R            /* Buffer 2 Configuration Register */
#define QUADSPI_1_BUF3CR     QUADSPI_1.BUF3CR.R            /* Buffer 3 Configuration Register */
#define QUADSPI_1_BFGENCR    QUADSPI_1.BFGENCR.R           /* Buffer Generic Configuration Register */
#define QUADSPI_1_SOCCR      QUADSPI_1.SOCCR.R             /* SOC Configuration Register */
#define QUADSPI_1_BUF0IND    QUADSPI_1.BUF0IND.R           /* Buffer 0 Top Index Register */
#define QUADSPI_1_BUF1IND    QUADSPI_1.BUF1IND.R           /* Buffer 1 Top Index Register */
#define QUADSPI_1_BUF2IND    QUADSPI_1.BUF2IND.R           /* Buffer 2 Top Index Register */
#define QUADSPI_1_AWRCR      QUADSPI_1.AWRCR.R             /* AHB Write Configuration Register */
#define QUADSPI_1_DLLCRA     QUADSPI_1.DLLCRA.R            /* DLL Flash Memory A Configuration Register */
#define QUADSPI_1_DLLCRB     QUADSPI_1.DLLCRB.R            /* DLL Flash Memory B Configuration Register */
#define QUADSPI_1_PARITYCR   QUADSPI_1.PARITYCR.R          /* Parity Configuration Register */
#define QUADSPI_1_SFAR       QUADSPI_1.SFAR.R              /* Serial Flash Memory Address Register */
#define QUADSPI_1_SFACR      QUADSPI_1.SFACR.R             /* Serial Flash Memory Address Configuration Register */
#define QUADSPI_1_SMPR       QUADSPI_1.SMPR.R              /* Sampling Register */
#define QUADSPI_1_RBSR       QUADSPI_1.RBSR.R              /* RX Buffer Status Register */
#define QUADSPI_1_RBCT       QUADSPI_1.RBCT.R              /* RX Buffer Control Register */
#define QUADSPI_1_AWRSR      QUADSPI_1.AWRSR.R             /* AHB Write Status Register */
#define QUADSPI_1_DLLSR      QUADSPI_1.DLLSR.R             /* DLL Status Register */
#define QUADSPI_1_DLCR       QUADSPI_1.DLCR.R              /* Data Learning Configuration Register */
#define QUADSPI_1_DLSR_FA    QUADSPI_1.DLSR_FA.R           /* Data Learning Status Flash Memory A Register */
#define QUADSPI_1_DLSR_FB    QUADSPI_1.DLSR_FB.R           /* Data Learning Status Flash Memory B Register */
#define QUADSPI_1_TBSR       QUADSPI_1.TBSR.R              /* TX Buffer Status Register */
#define QUADSPI_1_TBDR       QUADSPI_1.TBDR.R              /* TX Buffer Data Register */
#define QUADSPI_1_TBCT       QUADSPI_1.TBCT.R              /* TX Buffer Control Register */
#define QUADSPI_1_SR         QUADSPI_1.SR.R                /* Status Register */
#define QUADSPI_1_FR         QUADSPI_1.FR.R                /* Flag Register */
#define QUADSPI_1_RSER       QUADSPI_1.RSER.R              /* Interrupt and DMA Request Select and Enable Register */
#define QUADSPI_1_SPTRCLR    QUADSPI_1.SPTRCLR.R           /* Sequence Pointer Clear Register */
#define QUADSPI_1_SFA1AD     QUADSPI_1.SFA1AD.R            /* Serial Flash Memory A1 Top Address Register */
#define QUADSPI_1_SFA2AD     QUADSPI_1.SFA2AD.R            /* Serial Flash Memory A2 Top Address Register */
#define QUADSPI_1_SFB1AD     QUADSPI_1.SFB1AD.R            /* Serial Flash Memory B1 Top Address Register */
#define QUADSPI_1_SFB2AD     QUADSPI_1.SFB2AD.R            /* Serial Flash Memory B2 Top Address Register */
#define QUADSPI_1_DLPR       QUADSPI_1.DLPR.R              /* Data Learn Pattern Register */
#define QUADSPI_1_FAILA_ADDR QUADSPI_1.FAILA_ADDR.R        /* Flash Memory A Failing Address Status Register */
#define QUADSPI_1_FAILB_ADDR QUADSPI_1.FAILB_ADDR.R        /* flash Memory B Failing Address Status Register */
#define QUADSPI_1_RBDR0      QUADSPI_1.RBDR[0].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR1      QUADSPI_1.RBDR[1].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR2      QUADSPI_1.RBDR[2].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR3      QUADSPI_1.RBDR[3].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR4      QUADSPI_1.RBDR[4].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR5      QUADSPI_1.RBDR[5].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR6      QUADSPI_1.RBDR[6].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR7      QUADSPI_1.RBDR[7].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR8      QUADSPI_1.RBDR[8].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR9      QUADSPI_1.RBDR[9].R           /* RX Buffer Data Register */
#define QUADSPI_1_RBDR10     QUADSPI_1.RBDR[10].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR11     QUADSPI_1.RBDR[11].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR12     QUADSPI_1.RBDR[12].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR13     QUADSPI_1.RBDR[13].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR14     QUADSPI_1.RBDR[14].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR15     QUADSPI_1.RBDR[15].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR16     QUADSPI_1.RBDR[16].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR17     QUADSPI_1.RBDR[17].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR18     QUADSPI_1.RBDR[18].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR19     QUADSPI_1.RBDR[19].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR20     QUADSPI_1.RBDR[20].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR21     QUADSPI_1.RBDR[21].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR22     QUADSPI_1.RBDR[22].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR23     QUADSPI_1.RBDR[23].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR24     QUADSPI_1.RBDR[24].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR25     QUADSPI_1.RBDR[25].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR26     QUADSPI_1.RBDR[26].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR27     QUADSPI_1.RBDR[27].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR28     QUADSPI_1.RBDR[28].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR29     QUADSPI_1.RBDR[29].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR30     QUADSPI_1.RBDR[30].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR31     QUADSPI_1.RBDR[31].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR32     QUADSPI_1.RBDR[32].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR33     QUADSPI_1.RBDR[33].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR34     QUADSPI_1.RBDR[34].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR35     QUADSPI_1.RBDR[35].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR36     QUADSPI_1.RBDR[36].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR37     QUADSPI_1.RBDR[37].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR38     QUADSPI_1.RBDR[38].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR39     QUADSPI_1.RBDR[39].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR40     QUADSPI_1.RBDR[40].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR41     QUADSPI_1.RBDR[41].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR42     QUADSPI_1.RBDR[42].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR43     QUADSPI_1.RBDR[43].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR44     QUADSPI_1.RBDR[44].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR45     QUADSPI_1.RBDR[45].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR46     QUADSPI_1.RBDR[46].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR47     QUADSPI_1.RBDR[47].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR48     QUADSPI_1.RBDR[48].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR49     QUADSPI_1.RBDR[49].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR50     QUADSPI_1.RBDR[50].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR51     QUADSPI_1.RBDR[51].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR52     QUADSPI_1.RBDR[52].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR53     QUADSPI_1.RBDR[53].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR54     QUADSPI_1.RBDR[54].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR55     QUADSPI_1.RBDR[55].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR56     QUADSPI_1.RBDR[56].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR57     QUADSPI_1.RBDR[57].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR58     QUADSPI_1.RBDR[58].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR59     QUADSPI_1.RBDR[59].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR60     QUADSPI_1.RBDR[60].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR61     QUADSPI_1.RBDR[61].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR62     QUADSPI_1.RBDR[62].R          /* RX Buffer Data Register */
#define QUADSPI_1_RBDR63     QUADSPI_1.RBDR[63].R          /* RX Buffer Data Register */
#define QUADSPI_1_LUTKEY     QUADSPI_1.LUTKEY.R            /* LUT Key Register */
#define QUADSPI_1_LCKCR      QUADSPI_1.LCKCR.R             /* LUT Lock Configuration Register */
#define QUADSPI_1_LUT0       QUADSPI_1.LUT[0].R            /* LUT Register */
#define QUADSPI_1_LUT1       QUADSPI_1.LUT[1].R            /* LUT Register */
#define QUADSPI_1_LUT2       QUADSPI_1.LUT[2].R            /* LUT Register */
#define QUADSPI_1_LUT3       QUADSPI_1.LUT[3].R            /* LUT Register */
#define QUADSPI_1_LUT4       QUADSPI_1.LUT[4].R            /* LUT Register */
#define QUADSPI_1_LUT5       QUADSPI_1.LUT[5].R            /* LUT Register */
#define QUADSPI_1_LUT6       QUADSPI_1.LUT[6].R            /* LUT Register */
#define QUADSPI_1_LUT7       QUADSPI_1.LUT[7].R            /* LUT Register */
#define QUADSPI_1_LUT8       QUADSPI_1.LUT[8].R            /* LUT Register */
#define QUADSPI_1_LUT9       QUADSPI_1.LUT[9].R            /* LUT Register */
#define QUADSPI_1_LUT10      QUADSPI_1.LUT[10].R           /* LUT Register */
#define QUADSPI_1_LUT11      QUADSPI_1.LUT[11].R           /* LUT Register */
#define QUADSPI_1_LUT12      QUADSPI_1.LUT[12].R           /* LUT Register */
#define QUADSPI_1_LUT13      QUADSPI_1.LUT[13].R           /* LUT Register */
#define QUADSPI_1_LUT14      QUADSPI_1.LUT[14].R           /* LUT Register */
#define QUADSPI_1_LUT15      QUADSPI_1.LUT[15].R           /* LUT Register */
#define QUADSPI_1_LUT16      QUADSPI_1.LUT[16].R           /* LUT Register */
#define QUADSPI_1_LUT17      QUADSPI_1.LUT[17].R           /* LUT Register */
#define QUADSPI_1_LUT18      QUADSPI_1.LUT[18].R           /* LUT Register */
#define QUADSPI_1_LUT19      QUADSPI_1.LUT[19].R           /* LUT Register */
#define QUADSPI_1_LUT20      QUADSPI_1.LUT[20].R           /* LUT Register */
#define QUADSPI_1_LUT21      QUADSPI_1.LUT[21].R           /* LUT Register */
#define QUADSPI_1_LUT22      QUADSPI_1.LUT[22].R           /* LUT Register */
#define QUADSPI_1_LUT23      QUADSPI_1.LUT[23].R           /* LUT Register */
#define QUADSPI_1_LUT24      QUADSPI_1.LUT[24].R           /* LUT Register */
#define QUADSPI_1_LUT25      QUADSPI_1.LUT[25].R           /* LUT Register */
#define QUADSPI_1_LUT26      QUADSPI_1.LUT[26].R           /* LUT Register */
#define QUADSPI_1_LUT27      QUADSPI_1.LUT[27].R           /* LUT Register */
#define QUADSPI_1_LUT28      QUADSPI_1.LUT[28].R           /* LUT Register */
#define QUADSPI_1_LUT29      QUADSPI_1.LUT[29].R           /* LUT Register */
#define QUADSPI_1_LUT30      QUADSPI_1.LUT[30].R           /* LUT Register */
#define QUADSPI_1_LUT31      QUADSPI_1.LUT[31].R           /* LUT Register */
#define QUADSPI_1_LUT32      QUADSPI_1.LUT[32].R           /* LUT Register */
#define QUADSPI_1_LUT33      QUADSPI_1.LUT[33].R           /* LUT Register */
#define QUADSPI_1_LUT34      QUADSPI_1.LUT[34].R           /* LUT Register */
#define QUADSPI_1_LUT35      QUADSPI_1.LUT[35].R           /* LUT Register */
#define QUADSPI_1_LUT36      QUADSPI_1.LUT[36].R           /* LUT Register */
#define QUADSPI_1_LUT37      QUADSPI_1.LUT[37].R           /* LUT Register */
#define QUADSPI_1_LUT38      QUADSPI_1.LUT[38].R           /* LUT Register */
#define QUADSPI_1_LUT39      QUADSPI_1.LUT[39].R           /* LUT Register */
#define QUADSPI_1_LUT40      QUADSPI_1.LUT[40].R           /* LUT Register */
#define QUADSPI_1_LUT41      QUADSPI_1.LUT[41].R           /* LUT Register */
#define QUADSPI_1_LUT42      QUADSPI_1.LUT[42].R           /* LUT Register */
#define QUADSPI_1_LUT43      QUADSPI_1.LUT[43].R           /* LUT Register */
#define QUADSPI_1_LUT44      QUADSPI_1.LUT[44].R           /* LUT Register */
#define QUADSPI_1_LUT45      QUADSPI_1.LUT[45].R           /* LUT Register */
#define QUADSPI_1_LUT46      QUADSPI_1.LUT[46].R           /* LUT Register */
#define QUADSPI_1_LUT47      QUADSPI_1.LUT[47].R           /* LUT Register */
#define QUADSPI_1_LUT48      QUADSPI_1.LUT[48].R           /* LUT Register */
#define QUADSPI_1_LUT49      QUADSPI_1.LUT[49].R           /* LUT Register */
#define QUADSPI_1_LUT50      QUADSPI_1.LUT[50].R           /* LUT Register */
#define QUADSPI_1_LUT51      QUADSPI_1.LUT[51].R           /* LUT Register */
#define QUADSPI_1_LUT52      QUADSPI_1.LUT[52].R           /* LUT Register */
#define QUADSPI_1_LUT53      QUADSPI_1.LUT[53].R           /* LUT Register */
#define QUADSPI_1_LUT54      QUADSPI_1.LUT[54].R           /* LUT Register */
#define QUADSPI_1_LUT55      QUADSPI_1.LUT[55].R           /* LUT Register */
#define QUADSPI_1_LUT56      QUADSPI_1.LUT[56].R           /* LUT Register */
#define QUADSPI_1_LUT57      QUADSPI_1.LUT[57].R           /* LUT Register */
#define QUADSPI_1_LUT58      QUADSPI_1.LUT[58].R           /* LUT Register */
#define QUADSPI_1_LUT59      QUADSPI_1.LUT[59].R           /* LUT Register */
#define QUADSPI_1_LUT60      QUADSPI_1.LUT[60].R           /* LUT Register */
#define QUADSPI_1_LUT61      QUADSPI_1.LUT[61].R           /* LUT Register */
#define QUADSPI_1_LUT62      QUADSPI_1.LUT[62].R           /* LUT Register */
#define QUADSPI_1_LUT63      QUADSPI_1.LUT[63].R           /* LUT Register */
#define QUADSPI_1_LUT64      QUADSPI_1.LUT[64].R           /* LUT Register */
#define QUADSPI_1_LUT65      QUADSPI_1.LUT[65].R           /* LUT Register */
#define QUADSPI_1_LUT66      QUADSPI_1.LUT[66].R           /* LUT Register */
#define QUADSPI_1_LUT67      QUADSPI_1.LUT[67].R           /* LUT Register */
#define QUADSPI_1_LUT68      QUADSPI_1.LUT[68].R           /* LUT Register */
#define QUADSPI_1_LUT69      QUADSPI_1.LUT[69].R           /* LUT Register */
#define QUADSPI_1_LUT70      QUADSPI_1.LUT[70].R           /* LUT Register */
#define QUADSPI_1_LUT71      QUADSPI_1.LUT[71].R           /* LUT Register */
#define QUADSPI_1_LUT72      QUADSPI_1.LUT[72].R           /* LUT Register */
#define QUADSPI_1_LUT73      QUADSPI_1.LUT[73].R           /* LUT Register */
#define QUADSPI_1_LUT74      QUADSPI_1.LUT[74].R           /* LUT Register */
#define QUADSPI_1_LUT75      QUADSPI_1.LUT[75].R           /* LUT Register */
#define QUADSPI_1_LUT76      QUADSPI_1.LUT[76].R           /* LUT Register */
#define QUADSPI_1_LUT77      QUADSPI_1.LUT[77].R           /* LUT Register */
#define QUADSPI_1_LUT78      QUADSPI_1.LUT[78].R           /* LUT Register */
#define QUADSPI_1_LUT79      QUADSPI_1.LUT[79].R           /* LUT Register */
#define QUADSPI_1_FRAD0_WORD0 QUADSPI_1.FRAD[0].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_1_FRAD0_WORD1 QUADSPI_1.FRAD[0].WORD1.R    /* Flash Region End Address */
#define QUADSPI_1_FRAD0_WORD2 QUADSPI_1.FRAD[0].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_1_FRAD0_WORD3 QUADSPI_1.FRAD[0].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_1_FRAD0_WORD4 QUADSPI_1.FRAD[0].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_1_FRAD0_WORD5 QUADSPI_1.FRAD[0].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_1_FRAD1_WORD0 QUADSPI_1.FRAD[1].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_1_FRAD1_WORD1 QUADSPI_1.FRAD[1].WORD1.R    /* Flash Region End Address */
#define QUADSPI_1_FRAD1_WORD2 QUADSPI_1.FRAD[1].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_1_FRAD1_WORD3 QUADSPI_1.FRAD[1].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_1_FRAD1_WORD4 QUADSPI_1.FRAD[1].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_1_FRAD1_WORD5 QUADSPI_1.FRAD[1].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_1_FRAD2_WORD0 QUADSPI_1.FRAD[2].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_1_FRAD2_WORD1 QUADSPI_1.FRAD[2].WORD1.R    /* Flash Region End Address */
#define QUADSPI_1_FRAD2_WORD2 QUADSPI_1.FRAD[2].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_1_FRAD2_WORD3 QUADSPI_1.FRAD[2].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_1_FRAD2_WORD4 QUADSPI_1.FRAD[2].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_1_FRAD2_WORD5 QUADSPI_1.FRAD[2].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_1_FRAD3_WORD0 QUADSPI_1.FRAD[3].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_1_FRAD3_WORD1 QUADSPI_1.FRAD[3].WORD1.R    /* Flash Region End Address */
#define QUADSPI_1_FRAD3_WORD2 QUADSPI_1.FRAD[3].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_1_FRAD3_WORD3 QUADSPI_1.FRAD[3].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_1_FRAD3_WORD4 QUADSPI_1.FRAD[3].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_1_FRAD3_WORD5 QUADSPI_1.FRAD[3].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_1_FRAD4_WORD0 QUADSPI_1.FRAD[4].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_1_FRAD4_WORD1 QUADSPI_1.FRAD[4].WORD1.R    /* Flash Region End Address */
#define QUADSPI_1_FRAD4_WORD2 QUADSPI_1.FRAD[4].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_1_FRAD4_WORD3 QUADSPI_1.FRAD[4].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_1_FRAD4_WORD4 QUADSPI_1.FRAD[4].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_1_FRAD4_WORD5 QUADSPI_1.FRAD[4].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_1_FRAD5_WORD0 QUADSPI_1.FRAD[5].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_1_FRAD5_WORD1 QUADSPI_1.FRAD[5].WORD1.R    /* Flash Region End Address */
#define QUADSPI_1_FRAD5_WORD2 QUADSPI_1.FRAD[5].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_1_FRAD5_WORD3 QUADSPI_1.FRAD[5].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_1_FRAD5_WORD4 QUADSPI_1.FRAD[5].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_1_FRAD5_WORD5 QUADSPI_1.FRAD[5].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_1_FRAD6_WORD0 QUADSPI_1.FRAD[6].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_1_FRAD6_WORD1 QUADSPI_1.FRAD[6].WORD1.R    /* Flash Region End Address */
#define QUADSPI_1_FRAD6_WORD2 QUADSPI_1.FRAD[6].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_1_FRAD6_WORD3 QUADSPI_1.FRAD[6].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_1_FRAD6_WORD4 QUADSPI_1.FRAD[6].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_1_FRAD6_WORD5 QUADSPI_1.FRAD[6].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_1_FRAD7_WORD0 QUADSPI_1.FRAD[7].WORD0.R    /* Flash Region Start Address */
#define QUADSPI_1_FRAD7_WORD1 QUADSPI_1.FRAD[7].WORD1.R    /* Flash Region End Address */
#define QUADSPI_1_FRAD7_WORD2 QUADSPI_1.FRAD[7].WORD2.R    /* Flash Region Privileges */
#define QUADSPI_1_FRAD7_WORD3 QUADSPI_1.FRAD[7].WORD3.R    /* Flash Region Lock Control */
#define QUADSPI_1_FRAD7_WORD4 QUADSPI_1.FRAD[7].WORD4.R    /* Flash Region Compare Address Status */
#define QUADSPI_1_FRAD7_WORD5 QUADSPI_1.FRAD[7].WORD5.R    /* Flash Region Compare Status Data */
#define QUADSPI_1_TG0MDAD    QUADSPI_1.MDAD[0].TGMDAD.R    /* Target Group n Master Domain Access Descriptor */
#define QUADSPI_1_TG0SFAR    QUADSPI_1.MDAD[0].TGSFAR.R    /* Target Group n SFAR Address */
#define QUADSPI_1_TG0SFARS   QUADSPI_1.MDAD[0].TGSFARS.R   /* Target Group n SFAR Status */
#define QUADSPI_1_TG0IPCRS   QUADSPI_1.MDAD[0].TGIPCRS.R   /* Target Group n IPCR Status */
#define QUADSPI_1_TG1MDAD    QUADSPI_1.MDAD[1].TGMDAD.R    /* Target Group n Master Domain Access Descriptor */
#define QUADSPI_1_TG1SFAR    QUADSPI_1.MDAD[1].TGSFAR.R    /* Target Group n SFAR Address */
#define QUADSPI_1_TG1SFARS   QUADSPI_1.MDAD[1].TGSFARS.R   /* Target Group n SFAR Status */
#define QUADSPI_1_TG1IPCRS   QUADSPI_1.MDAD[1].TGIPCRS.R   /* Target Group n IPCR Status */
#define QUADSPI_1_MGC        QUADSPI_1.MGC.R               /* Master Global Configuration */
#define QUADSPI_1_MRC        QUADSPI_1.MRC.R               /* Master Read Command */
#define QUADSPI_1_MTO        QUADSPI_1.MTO.R               /* Master Timeout */
#define QUADSPI_1_FLSEQREQ   QUADSPI_1.FLSEQREQ.R          /* FlashSeq Request */
#define QUADSPI_1_FSMSTAT    QUADSPI_1.FSMSTAT.R           /* FSM Status */
#define QUADSPI_1_IPSERROR   QUADSPI_1.IPSERROR.R          /* IPS Error */
#define QUADSPI_1_ERRSTAT    QUADSPI_1.ERRSTAT.R           /* Error Status */
#define QUADSPI_1_INT_EN     QUADSPI_1.INT_EN.R            /* Interrupt Enable */

/* QuadSPI_ARDB */
#define QUADSPI_0_ARDB_ARDB0 QUADSPI_0_ARDB.ARDB[0].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB1 QUADSPI_0_ARDB.ARDB[1].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB2 QUADSPI_0_ARDB.ARDB[2].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB3 QUADSPI_0_ARDB.ARDB[3].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB4 QUADSPI_0_ARDB.ARDB[4].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB5 QUADSPI_0_ARDB.ARDB[5].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB6 QUADSPI_0_ARDB.ARDB[6].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB7 QUADSPI_0_ARDB.ARDB[7].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB8 QUADSPI_0_ARDB.ARDB[8].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB9 QUADSPI_0_ARDB.ARDB[9].R      /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB10 QUADSPI_0_ARDB.ARDB[10].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB11 QUADSPI_0_ARDB.ARDB[11].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB12 QUADSPI_0_ARDB.ARDB[12].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB13 QUADSPI_0_ARDB.ARDB[13].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB14 QUADSPI_0_ARDB.ARDB[14].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB15 QUADSPI_0_ARDB.ARDB[15].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB16 QUADSPI_0_ARDB.ARDB[16].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB17 QUADSPI_0_ARDB.ARDB[17].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB18 QUADSPI_0_ARDB.ARDB[18].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB19 QUADSPI_0_ARDB.ARDB[19].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB20 QUADSPI_0_ARDB.ARDB[20].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB21 QUADSPI_0_ARDB.ARDB[21].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB22 QUADSPI_0_ARDB.ARDB[22].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB23 QUADSPI_0_ARDB.ARDB[23].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB24 QUADSPI_0_ARDB.ARDB[24].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB25 QUADSPI_0_ARDB.ARDB[25].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB26 QUADSPI_0_ARDB.ARDB[26].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB27 QUADSPI_0_ARDB.ARDB[27].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB28 QUADSPI_0_ARDB.ARDB[28].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB29 QUADSPI_0_ARDB.ARDB[29].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB30 QUADSPI_0_ARDB.ARDB[30].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB31 QUADSPI_0_ARDB.ARDB[31].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB32 QUADSPI_0_ARDB.ARDB[32].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB33 QUADSPI_0_ARDB.ARDB[33].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB34 QUADSPI_0_ARDB.ARDB[34].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB35 QUADSPI_0_ARDB.ARDB[35].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB36 QUADSPI_0_ARDB.ARDB[36].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB37 QUADSPI_0_ARDB.ARDB[37].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB38 QUADSPI_0_ARDB.ARDB[38].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB39 QUADSPI_0_ARDB.ARDB[39].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB40 QUADSPI_0_ARDB.ARDB[40].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB41 QUADSPI_0_ARDB.ARDB[41].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB42 QUADSPI_0_ARDB.ARDB[42].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB43 QUADSPI_0_ARDB.ARDB[43].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB44 QUADSPI_0_ARDB.ARDB[44].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB45 QUADSPI_0_ARDB.ARDB[45].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB46 QUADSPI_0_ARDB.ARDB[46].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB47 QUADSPI_0_ARDB.ARDB[47].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB48 QUADSPI_0_ARDB.ARDB[48].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB49 QUADSPI_0_ARDB.ARDB[49].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB50 QUADSPI_0_ARDB.ARDB[50].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB51 QUADSPI_0_ARDB.ARDB[51].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB52 QUADSPI_0_ARDB.ARDB[52].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB53 QUADSPI_0_ARDB.ARDB[53].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB54 QUADSPI_0_ARDB.ARDB[54].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB55 QUADSPI_0_ARDB.ARDB[55].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB56 QUADSPI_0_ARDB.ARDB[56].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB57 QUADSPI_0_ARDB.ARDB[57].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB58 QUADSPI_0_ARDB.ARDB[58].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB59 QUADSPI_0_ARDB.ARDB[59].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB60 QUADSPI_0_ARDB.ARDB[60].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB61 QUADSPI_0_ARDB.ARDB[61].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB62 QUADSPI_0_ARDB.ARDB[62].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB63 QUADSPI_0_ARDB.ARDB[63].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB64 QUADSPI_0_ARDB.ARDB[64].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB65 QUADSPI_0_ARDB.ARDB[65].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB66 QUADSPI_0_ARDB.ARDB[66].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB67 QUADSPI_0_ARDB.ARDB[67].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB68 QUADSPI_0_ARDB.ARDB[68].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB69 QUADSPI_0_ARDB.ARDB[69].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB70 QUADSPI_0_ARDB.ARDB[70].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB71 QUADSPI_0_ARDB.ARDB[71].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB72 QUADSPI_0_ARDB.ARDB[72].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB73 QUADSPI_0_ARDB.ARDB[73].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB74 QUADSPI_0_ARDB.ARDB[74].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB75 QUADSPI_0_ARDB.ARDB[75].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB76 QUADSPI_0_ARDB.ARDB[76].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB77 QUADSPI_0_ARDB.ARDB[77].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB78 QUADSPI_0_ARDB.ARDB[78].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB79 QUADSPI_0_ARDB.ARDB[79].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB80 QUADSPI_0_ARDB.ARDB[80].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB81 QUADSPI_0_ARDB.ARDB[81].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB82 QUADSPI_0_ARDB.ARDB[82].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB83 QUADSPI_0_ARDB.ARDB[83].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB84 QUADSPI_0_ARDB.ARDB[84].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB85 QUADSPI_0_ARDB.ARDB[85].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB86 QUADSPI_0_ARDB.ARDB[86].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB87 QUADSPI_0_ARDB.ARDB[87].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB88 QUADSPI_0_ARDB.ARDB[88].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB89 QUADSPI_0_ARDB.ARDB[89].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB90 QUADSPI_0_ARDB.ARDB[90].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB91 QUADSPI_0_ARDB.ARDB[91].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB92 QUADSPI_0_ARDB.ARDB[92].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB93 QUADSPI_0_ARDB.ARDB[93].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB94 QUADSPI_0_ARDB.ARDB[94].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB95 QUADSPI_0_ARDB.ARDB[95].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB96 QUADSPI_0_ARDB.ARDB[96].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB97 QUADSPI_0_ARDB.ARDB[97].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB98 QUADSPI_0_ARDB.ARDB[98].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB99 QUADSPI_0_ARDB.ARDB[99].R    /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB100 QUADSPI_0_ARDB.ARDB[100].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB101 QUADSPI_0_ARDB.ARDB[101].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB102 QUADSPI_0_ARDB.ARDB[102].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB103 QUADSPI_0_ARDB.ARDB[103].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB104 QUADSPI_0_ARDB.ARDB[104].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB105 QUADSPI_0_ARDB.ARDB[105].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB106 QUADSPI_0_ARDB.ARDB[106].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB107 QUADSPI_0_ARDB.ARDB[107].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB108 QUADSPI_0_ARDB.ARDB[108].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB109 QUADSPI_0_ARDB.ARDB[109].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB110 QUADSPI_0_ARDB.ARDB[110].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB111 QUADSPI_0_ARDB.ARDB[111].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB112 QUADSPI_0_ARDB.ARDB[112].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB113 QUADSPI_0_ARDB.ARDB[113].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB114 QUADSPI_0_ARDB.ARDB[114].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB115 QUADSPI_0_ARDB.ARDB[115].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB116 QUADSPI_0_ARDB.ARDB[116].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB117 QUADSPI_0_ARDB.ARDB[117].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB118 QUADSPI_0_ARDB.ARDB[118].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB119 QUADSPI_0_ARDB.ARDB[119].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB120 QUADSPI_0_ARDB.ARDB[120].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB121 QUADSPI_0_ARDB.ARDB[121].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB122 QUADSPI_0_ARDB.ARDB[122].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB123 QUADSPI_0_ARDB.ARDB[123].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB124 QUADSPI_0_ARDB.ARDB[124].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB125 QUADSPI_0_ARDB.ARDB[125].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB126 QUADSPI_0_ARDB.ARDB[126].R  /* AHB RX Data Buffer Register */
#define QUADSPI_0_ARDB_ARDB127 QUADSPI_0_ARDB.ARDB[127].R  /* AHB RX Data Buffer Register */

#define QUADSPI_1_ARDB_ARDB0 QUADSPI_1_ARDB.ARDB[0].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB1 QUADSPI_1_ARDB.ARDB[1].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB2 QUADSPI_1_ARDB.ARDB[2].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB3 QUADSPI_1_ARDB.ARDB[3].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB4 QUADSPI_1_ARDB.ARDB[4].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB5 QUADSPI_1_ARDB.ARDB[5].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB6 QUADSPI_1_ARDB.ARDB[6].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB7 QUADSPI_1_ARDB.ARDB[7].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB8 QUADSPI_1_ARDB.ARDB[8].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB9 QUADSPI_1_ARDB.ARDB[9].R      /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB10 QUADSPI_1_ARDB.ARDB[10].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB11 QUADSPI_1_ARDB.ARDB[11].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB12 QUADSPI_1_ARDB.ARDB[12].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB13 QUADSPI_1_ARDB.ARDB[13].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB14 QUADSPI_1_ARDB.ARDB[14].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB15 QUADSPI_1_ARDB.ARDB[15].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB16 QUADSPI_1_ARDB.ARDB[16].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB17 QUADSPI_1_ARDB.ARDB[17].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB18 QUADSPI_1_ARDB.ARDB[18].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB19 QUADSPI_1_ARDB.ARDB[19].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB20 QUADSPI_1_ARDB.ARDB[20].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB21 QUADSPI_1_ARDB.ARDB[21].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB22 QUADSPI_1_ARDB.ARDB[22].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB23 QUADSPI_1_ARDB.ARDB[23].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB24 QUADSPI_1_ARDB.ARDB[24].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB25 QUADSPI_1_ARDB.ARDB[25].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB26 QUADSPI_1_ARDB.ARDB[26].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB27 QUADSPI_1_ARDB.ARDB[27].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB28 QUADSPI_1_ARDB.ARDB[28].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB29 QUADSPI_1_ARDB.ARDB[29].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB30 QUADSPI_1_ARDB.ARDB[30].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB31 QUADSPI_1_ARDB.ARDB[31].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB32 QUADSPI_1_ARDB.ARDB[32].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB33 QUADSPI_1_ARDB.ARDB[33].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB34 QUADSPI_1_ARDB.ARDB[34].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB35 QUADSPI_1_ARDB.ARDB[35].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB36 QUADSPI_1_ARDB.ARDB[36].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB37 QUADSPI_1_ARDB.ARDB[37].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB38 QUADSPI_1_ARDB.ARDB[38].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB39 QUADSPI_1_ARDB.ARDB[39].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB40 QUADSPI_1_ARDB.ARDB[40].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB41 QUADSPI_1_ARDB.ARDB[41].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB42 QUADSPI_1_ARDB.ARDB[42].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB43 QUADSPI_1_ARDB.ARDB[43].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB44 QUADSPI_1_ARDB.ARDB[44].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB45 QUADSPI_1_ARDB.ARDB[45].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB46 QUADSPI_1_ARDB.ARDB[46].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB47 QUADSPI_1_ARDB.ARDB[47].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB48 QUADSPI_1_ARDB.ARDB[48].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB49 QUADSPI_1_ARDB.ARDB[49].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB50 QUADSPI_1_ARDB.ARDB[50].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB51 QUADSPI_1_ARDB.ARDB[51].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB52 QUADSPI_1_ARDB.ARDB[52].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB53 QUADSPI_1_ARDB.ARDB[53].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB54 QUADSPI_1_ARDB.ARDB[54].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB55 QUADSPI_1_ARDB.ARDB[55].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB56 QUADSPI_1_ARDB.ARDB[56].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB57 QUADSPI_1_ARDB.ARDB[57].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB58 QUADSPI_1_ARDB.ARDB[58].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB59 QUADSPI_1_ARDB.ARDB[59].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB60 QUADSPI_1_ARDB.ARDB[60].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB61 QUADSPI_1_ARDB.ARDB[61].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB62 QUADSPI_1_ARDB.ARDB[62].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB63 QUADSPI_1_ARDB.ARDB[63].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB64 QUADSPI_1_ARDB.ARDB[64].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB65 QUADSPI_1_ARDB.ARDB[65].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB66 QUADSPI_1_ARDB.ARDB[66].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB67 QUADSPI_1_ARDB.ARDB[67].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB68 QUADSPI_1_ARDB.ARDB[68].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB69 QUADSPI_1_ARDB.ARDB[69].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB70 QUADSPI_1_ARDB.ARDB[70].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB71 QUADSPI_1_ARDB.ARDB[71].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB72 QUADSPI_1_ARDB.ARDB[72].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB73 QUADSPI_1_ARDB.ARDB[73].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB74 QUADSPI_1_ARDB.ARDB[74].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB75 QUADSPI_1_ARDB.ARDB[75].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB76 QUADSPI_1_ARDB.ARDB[76].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB77 QUADSPI_1_ARDB.ARDB[77].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB78 QUADSPI_1_ARDB.ARDB[78].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB79 QUADSPI_1_ARDB.ARDB[79].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB80 QUADSPI_1_ARDB.ARDB[80].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB81 QUADSPI_1_ARDB.ARDB[81].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB82 QUADSPI_1_ARDB.ARDB[82].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB83 QUADSPI_1_ARDB.ARDB[83].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB84 QUADSPI_1_ARDB.ARDB[84].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB85 QUADSPI_1_ARDB.ARDB[85].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB86 QUADSPI_1_ARDB.ARDB[86].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB87 QUADSPI_1_ARDB.ARDB[87].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB88 QUADSPI_1_ARDB.ARDB[88].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB89 QUADSPI_1_ARDB.ARDB[89].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB90 QUADSPI_1_ARDB.ARDB[90].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB91 QUADSPI_1_ARDB.ARDB[91].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB92 QUADSPI_1_ARDB.ARDB[92].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB93 QUADSPI_1_ARDB.ARDB[93].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB94 QUADSPI_1_ARDB.ARDB[94].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB95 QUADSPI_1_ARDB.ARDB[95].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB96 QUADSPI_1_ARDB.ARDB[96].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB97 QUADSPI_1_ARDB.ARDB[97].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB98 QUADSPI_1_ARDB.ARDB[98].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB99 QUADSPI_1_ARDB.ARDB[99].R    /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB100 QUADSPI_1_ARDB.ARDB[100].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB101 QUADSPI_1_ARDB.ARDB[101].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB102 QUADSPI_1_ARDB.ARDB[102].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB103 QUADSPI_1_ARDB.ARDB[103].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB104 QUADSPI_1_ARDB.ARDB[104].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB105 QUADSPI_1_ARDB.ARDB[105].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB106 QUADSPI_1_ARDB.ARDB[106].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB107 QUADSPI_1_ARDB.ARDB[107].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB108 QUADSPI_1_ARDB.ARDB[108].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB109 QUADSPI_1_ARDB.ARDB[109].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB110 QUADSPI_1_ARDB.ARDB[110].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB111 QUADSPI_1_ARDB.ARDB[111].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB112 QUADSPI_1_ARDB.ARDB[112].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB113 QUADSPI_1_ARDB.ARDB[113].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB114 QUADSPI_1_ARDB.ARDB[114].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB115 QUADSPI_1_ARDB.ARDB[115].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB116 QUADSPI_1_ARDB.ARDB[116].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB117 QUADSPI_1_ARDB.ARDB[117].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB118 QUADSPI_1_ARDB.ARDB[118].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB119 QUADSPI_1_ARDB.ARDB[119].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB120 QUADSPI_1_ARDB.ARDB[120].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB121 QUADSPI_1_ARDB.ARDB[121].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB122 QUADSPI_1_ARDB.ARDB[122].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB123 QUADSPI_1_ARDB.ARDB[123].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB124 QUADSPI_1_ARDB.ARDB[124].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB125 QUADSPI_1_ARDB.ARDB[125].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB126 QUADSPI_1_ARDB.ARDB[126].R  /* AHB RX Data Buffer Register */
#define QUADSPI_1_ARDB_ARDB127 QUADSPI_1_ARDB.ARDB[127].R  /* AHB RX Data Buffer Register */

/* Queue_Manager1 */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_EN_DEPTH CEVA_SPF2__QUEUE_MANAGER1.Q1_EN_DEPTH.R /* QX EN DEPTH */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_FIRST_ADDR CEVA_SPF2__QUEUE_MANAGER1.Q1_FIRST_ADDR.R /* Q1 FIRST ADDR */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_BASE_PTR CEVA_SPF2__QUEUE_MANAGER1.Q1_BASE_PTR.R /* Q1 BASE PTR */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_CHNK_SIZE CEVA_SPF2__QUEUE_MANAGER1.Q1_CHNK_SIZE.R /* Q1 CHNK SIZE */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_EN_INC0 CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_EN_INC0.R /* Q1 DSC EN INC0 */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_STATUS CEVA_SPF2__QUEUE_MANAGER1.Q1_STATUS.R /* Q1 STATUS */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_RPTR_INT CEVA_SPF2__QUEUE_MANAGER1.Q1_RPTR_INT.R /* Q1 RPTR INT */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_CFG CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_CFG.R /* Q1 DSC CFG */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_SRCP CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_SRCP.R /* Q1 DSC SRCP */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_DSTP CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_DSTP.R /* Q1 DSC DSTP */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_DMASZ CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_DMASZ.R /* Q1 DSC DMASZ */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_SRC CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_SRC.R /* Q1 DSC SRC */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_LINE CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_LINE.R /* Q1 DSC LINE */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_DST CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_DST.R /* Q1 DSC DST */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_HGHT CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_HGHT.R /* Q1 DSC HGHT */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_DSC_EN_INC1 CEVA_SPF2__QUEUE_MANAGER1.Q1_DSC_EN_INC1.R /* Q1 DSC EN INC1 */
#define CEVA_SPF2__QUEUE_MANAGER1_Q1_EN_CNT CEVA_SPF2__QUEUE_MANAGER1.Q1_EN_CNT.R /* Q1 EN CNT */

/* RDC */
#define RDC_0_RD1_CTRL_REG   RDC_0.RD1_CTRL_REG.R          /* Software Reset Domain Control */
#define RDC_0_RD1_STAT_REG   RDC_0.RD1_STAT_REG.R          /* Software Reset Domain Status */

#define RDC_1_RD1_CTRL_REG   RDC_1.RD1_CTRL_REG.R          /* Software Reset Domain Control */
#define RDC_1_RD1_STAT_REG   RDC_1.RD1_STAT_REG.R          /* Software Reset Domain Status */

/* RESULT_DMACRC */
#define AES__RESULT_DMACRC_GEC AES__RESULT_DMACRC.GEC.R    /* Global Enable CRC Register */
#define AES__RESULT_DMACRC_CTL0 AES__RESULT_DMACRC.CONTROL_REGISTER[0].CTL.R /* CRC Control Register */
#define AES__RESULT_DMACRC_ICRC0 AES__RESULT_DMACRC.CONTROL_REGISTER[0].ICRC.R /* Initial CRC Value Register */
#define AES__RESULT_DMACRC_FCRC0 AES__RESULT_DMACRC.CONTROL_REGISTER[0].FCRC.R /* Final CRC Value Register */
#define AES__RESULT_DMACRC_CTL1 AES__RESULT_DMACRC.CONTROL_REGISTER[1].CTL.R /* CRC Control Register */
#define AES__RESULT_DMACRC_ICRC1 AES__RESULT_DMACRC.CONTROL_REGISTER[1].ICRC.R /* Initial CRC Value Register */
#define AES__RESULT_DMACRC_FCRC1 AES__RESULT_DMACRC.CONTROL_REGISTER[1].FCRC.R /* Final CRC Value Register */
#define AES__RESULT_DMACRC_CTL2 AES__RESULT_DMACRC.CONTROL_REGISTER[2].CTL.R /* CRC Control Register */
#define AES__RESULT_DMACRC_ICRC2 AES__RESULT_DMACRC.CONTROL_REGISTER[2].ICRC.R /* Initial CRC Value Register */
#define AES__RESULT_DMACRC_FCRC2 AES__RESULT_DMACRC.CONTROL_REGISTER[2].FCRC.R /* Final CRC Value Register */
#define AES__RESULT_DMACRC_CTL3 AES__RESULT_DMACRC.CONTROL_REGISTER[3].CTL.R /* CRC Control Register */
#define AES__RESULT_DMACRC_ICRC3 AES__RESULT_DMACRC.CONTROL_REGISTER[3].ICRC.R /* Initial CRC Value Register */
#define AES__RESULT_DMACRC_FCRC3 AES__RESULT_DMACRC.CONTROL_REGISTER[3].FCRC.R /* Final CRC Value Register */
#define AES__RESULT_DMACRC_CTL4 AES__RESULT_DMACRC.CONTROL_REGISTER[4].CTL.R /* CRC Control Register */
#define AES__RESULT_DMACRC_ICRC4 AES__RESULT_DMACRC.CONTROL_REGISTER[4].ICRC.R /* Initial CRC Value Register */
#define AES__RESULT_DMACRC_FCRC4 AES__RESULT_DMACRC.CONTROL_REGISTER[4].FCRC.R /* Final CRC Value Register */
#define AES__RESULT_DMACRC_CTL5 AES__RESULT_DMACRC.CONTROL_REGISTER[5].CTL.R /* CRC Control Register */
#define AES__RESULT_DMACRC_ICRC5 AES__RESULT_DMACRC.CONTROL_REGISTER[5].ICRC.R /* Initial CRC Value Register */
#define AES__RESULT_DMACRC_FCRC5 AES__RESULT_DMACRC.CONTROL_REGISTER[5].FCRC.R /* Final CRC Value Register */
#define AES__RESULT_DMACRC_CTL6 AES__RESULT_DMACRC.CONTROL_REGISTER[6].CTL.R /* CRC Control Register */
#define AES__RESULT_DMACRC_ICRC6 AES__RESULT_DMACRC.CONTROL_REGISTER[6].ICRC.R /* Initial CRC Value Register */
#define AES__RESULT_DMACRC_FCRC6 AES__RESULT_DMACRC.CONTROL_REGISTER[6].FCRC.R /* Final CRC Value Register */
#define AES__RESULT_DMACRC_CTL7 AES__RESULT_DMACRC.CONTROL_REGISTER[7].CTL.R /* CRC Control Register */
#define AES__RESULT_DMACRC_ICRC7 AES__RESULT_DMACRC.CONTROL_REGISTER[7].ICRC.R /* Initial CRC Value Register */
#define AES__RESULT_DMACRC_FCRC7 AES__RESULT_DMACRC.CONTROL_REGISTER[7].FCRC.R /* Final CRC Value Register */

/* RESULT_DMA_MP */
#define AES__RESULT_DMA_MP_CSR AES__RESULT_DMA_MP.CSR.R    /* Management Page Control */
#define AES__RESULT_DMA_MP_ES AES__RESULT_DMA_MP.ES.R      /* Management Page Error Status */
#define AES__RESULT_DMA_MP_INT AES__RESULT_DMA_MP.INT.R    /* Management Page Interrupt Request Status */
#define AES__RESULT_DMA_MP_HRS AES__RESULT_DMA_MP.HRS.R    /* Management Page Hardware Request Status */
#define AES__RESULT_DMA_MP_CH0_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[0].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH1_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[1].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH2_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[2].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH3_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[3].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH4_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[4].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH5_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[5].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH6_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[6].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH7_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[7].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH8_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[8].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH9_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[9].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH10_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[10].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH11_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[11].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH12_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[12].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH13_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[13].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH14_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[14].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH15_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[15].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH16_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[16].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH17_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[17].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH18_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[18].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH19_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[19].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH20_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[20].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH21_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[21].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH22_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[22].R /* Channel Arbitration Group */
#define AES__RESULT_DMA_MP_CH23_GRPRI AES__RESULT_DMA_MP.CH_GRPRI[23].R /* Channel Arbitration Group */

/* RESULT_DMA_TCD */
#define AES__RESULT_DMA_TCD_CH0_CSR AES__RESULT_DMA_TCD.TCD[0].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH0_ES AES__RESULT_DMA_TCD.TCD[0].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH0_INT AES__RESULT_DMA_TCD.TCD[0].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH0_SBR AES__RESULT_DMA_TCD.TCD[0].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH0_PRI AES__RESULT_DMA_TCD.TCD[0].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD0_SADDR AES__RESULT_DMA_TCD.TCD[0].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD0_SOFF AES__RESULT_DMA_TCD.TCD[0].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD0_ATTR AES__RESULT_DMA_TCD.TCD[0].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD0_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[0].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD0_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[0].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD0_SLAST_SDA AES__RESULT_DMA_TCD.TCD[0].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD0_DADDR AES__RESULT_DMA_TCD.TCD[0].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD0_DOFF AES__RESULT_DMA_TCD.TCD[0].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD0_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[0].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD0_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[0].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD0_DLAST_SGA AES__RESULT_DMA_TCD.TCD[0].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD0_CSR AES__RESULT_DMA_TCD.TCD[0].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD0_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[0].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD0_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[0].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH1_CSR AES__RESULT_DMA_TCD.TCD[1].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH1_ES AES__RESULT_DMA_TCD.TCD[1].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH1_INT AES__RESULT_DMA_TCD.TCD[1].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH1_SBR AES__RESULT_DMA_TCD.TCD[1].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH1_PRI AES__RESULT_DMA_TCD.TCD[1].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD1_SADDR AES__RESULT_DMA_TCD.TCD[1].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD1_SOFF AES__RESULT_DMA_TCD.TCD[1].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD1_ATTR AES__RESULT_DMA_TCD.TCD[1].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD1_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[1].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD1_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[1].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD1_SLAST_SDA AES__RESULT_DMA_TCD.TCD[1].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD1_DADDR AES__RESULT_DMA_TCD.TCD[1].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD1_DOFF AES__RESULT_DMA_TCD.TCD[1].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD1_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[1].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD1_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[1].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD1_DLAST_SGA AES__RESULT_DMA_TCD.TCD[1].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD1_CSR AES__RESULT_DMA_TCD.TCD[1].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD1_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[1].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD1_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[1].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH2_CSR AES__RESULT_DMA_TCD.TCD[2].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH2_ES AES__RESULT_DMA_TCD.TCD[2].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH2_INT AES__RESULT_DMA_TCD.TCD[2].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH2_SBR AES__RESULT_DMA_TCD.TCD[2].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH2_PRI AES__RESULT_DMA_TCD.TCD[2].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD2_SADDR AES__RESULT_DMA_TCD.TCD[2].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD2_SOFF AES__RESULT_DMA_TCD.TCD[2].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD2_ATTR AES__RESULT_DMA_TCD.TCD[2].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD2_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[2].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD2_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[2].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD2_SLAST_SDA AES__RESULT_DMA_TCD.TCD[2].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD2_DADDR AES__RESULT_DMA_TCD.TCD[2].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD2_DOFF AES__RESULT_DMA_TCD.TCD[2].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD2_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[2].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD2_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[2].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD2_DLAST_SGA AES__RESULT_DMA_TCD.TCD[2].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD2_CSR AES__RESULT_DMA_TCD.TCD[2].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD2_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[2].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD2_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[2].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH3_CSR AES__RESULT_DMA_TCD.TCD[3].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH3_ES AES__RESULT_DMA_TCD.TCD[3].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH3_INT AES__RESULT_DMA_TCD.TCD[3].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH3_SBR AES__RESULT_DMA_TCD.TCD[3].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH3_PRI AES__RESULT_DMA_TCD.TCD[3].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD3_SADDR AES__RESULT_DMA_TCD.TCD[3].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD3_SOFF AES__RESULT_DMA_TCD.TCD[3].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD3_ATTR AES__RESULT_DMA_TCD.TCD[3].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD3_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[3].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD3_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[3].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD3_SLAST_SDA AES__RESULT_DMA_TCD.TCD[3].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD3_DADDR AES__RESULT_DMA_TCD.TCD[3].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD3_DOFF AES__RESULT_DMA_TCD.TCD[3].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD3_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[3].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD3_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[3].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD3_DLAST_SGA AES__RESULT_DMA_TCD.TCD[3].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD3_CSR AES__RESULT_DMA_TCD.TCD[3].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD3_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[3].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD3_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[3].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH4_CSR AES__RESULT_DMA_TCD.TCD[4].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH4_ES AES__RESULT_DMA_TCD.TCD[4].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH4_INT AES__RESULT_DMA_TCD.TCD[4].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH4_SBR AES__RESULT_DMA_TCD.TCD[4].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH4_PRI AES__RESULT_DMA_TCD.TCD[4].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD4_SADDR AES__RESULT_DMA_TCD.TCD[4].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD4_SOFF AES__RESULT_DMA_TCD.TCD[4].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD4_ATTR AES__RESULT_DMA_TCD.TCD[4].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD4_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[4].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD4_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[4].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD4_SLAST_SDA AES__RESULT_DMA_TCD.TCD[4].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD4_DADDR AES__RESULT_DMA_TCD.TCD[4].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD4_DOFF AES__RESULT_DMA_TCD.TCD[4].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD4_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[4].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD4_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[4].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD4_DLAST_SGA AES__RESULT_DMA_TCD.TCD[4].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD4_CSR AES__RESULT_DMA_TCD.TCD[4].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD4_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[4].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD4_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[4].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH5_CSR AES__RESULT_DMA_TCD.TCD[5].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH5_ES AES__RESULT_DMA_TCD.TCD[5].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH5_INT AES__RESULT_DMA_TCD.TCD[5].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH5_SBR AES__RESULT_DMA_TCD.TCD[5].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH5_PRI AES__RESULT_DMA_TCD.TCD[5].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD5_SADDR AES__RESULT_DMA_TCD.TCD[5].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD5_SOFF AES__RESULT_DMA_TCD.TCD[5].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD5_ATTR AES__RESULT_DMA_TCD.TCD[5].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD5_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[5].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD5_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[5].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD5_SLAST_SDA AES__RESULT_DMA_TCD.TCD[5].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD5_DADDR AES__RESULT_DMA_TCD.TCD[5].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD5_DOFF AES__RESULT_DMA_TCD.TCD[5].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD5_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[5].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD5_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[5].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD5_DLAST_SGA AES__RESULT_DMA_TCD.TCD[5].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD5_CSR AES__RESULT_DMA_TCD.TCD[5].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD5_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[5].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD5_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[5].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH6_CSR AES__RESULT_DMA_TCD.TCD[6].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH6_ES AES__RESULT_DMA_TCD.TCD[6].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH6_INT AES__RESULT_DMA_TCD.TCD[6].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH6_SBR AES__RESULT_DMA_TCD.TCD[6].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH6_PRI AES__RESULT_DMA_TCD.TCD[6].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD6_SADDR AES__RESULT_DMA_TCD.TCD[6].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD6_SOFF AES__RESULT_DMA_TCD.TCD[6].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD6_ATTR AES__RESULT_DMA_TCD.TCD[6].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD6_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[6].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD6_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[6].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD6_SLAST_SDA AES__RESULT_DMA_TCD.TCD[6].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD6_DADDR AES__RESULT_DMA_TCD.TCD[6].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD6_DOFF AES__RESULT_DMA_TCD.TCD[6].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD6_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[6].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD6_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[6].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD6_DLAST_SGA AES__RESULT_DMA_TCD.TCD[6].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD6_CSR AES__RESULT_DMA_TCD.TCD[6].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD6_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[6].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD6_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[6].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH7_CSR AES__RESULT_DMA_TCD.TCD[7].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH7_ES AES__RESULT_DMA_TCD.TCD[7].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH7_INT AES__RESULT_DMA_TCD.TCD[7].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH7_SBR AES__RESULT_DMA_TCD.TCD[7].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH7_PRI AES__RESULT_DMA_TCD.TCD[7].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD7_SADDR AES__RESULT_DMA_TCD.TCD[7].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD7_SOFF AES__RESULT_DMA_TCD.TCD[7].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD7_ATTR AES__RESULT_DMA_TCD.TCD[7].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD7_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[7].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD7_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[7].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD7_SLAST_SDA AES__RESULT_DMA_TCD.TCD[7].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD7_DADDR AES__RESULT_DMA_TCD.TCD[7].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD7_DOFF AES__RESULT_DMA_TCD.TCD[7].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD7_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[7].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD7_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[7].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD7_DLAST_SGA AES__RESULT_DMA_TCD.TCD[7].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD7_CSR AES__RESULT_DMA_TCD.TCD[7].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD7_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[7].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD7_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[7].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH8_CSR AES__RESULT_DMA_TCD.TCD[8].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH8_ES AES__RESULT_DMA_TCD.TCD[8].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH8_INT AES__RESULT_DMA_TCD.TCD[8].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH8_SBR AES__RESULT_DMA_TCD.TCD[8].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH8_PRI AES__RESULT_DMA_TCD.TCD[8].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD8_SADDR AES__RESULT_DMA_TCD.TCD[8].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD8_SOFF AES__RESULT_DMA_TCD.TCD[8].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD8_ATTR AES__RESULT_DMA_TCD.TCD[8].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD8_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[8].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD8_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[8].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD8_SLAST_SDA AES__RESULT_DMA_TCD.TCD[8].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD8_DADDR AES__RESULT_DMA_TCD.TCD[8].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD8_DOFF AES__RESULT_DMA_TCD.TCD[8].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD8_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[8].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD8_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[8].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD8_DLAST_SGA AES__RESULT_DMA_TCD.TCD[8].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD8_CSR AES__RESULT_DMA_TCD.TCD[8].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD8_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[8].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD8_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[8].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH9_CSR AES__RESULT_DMA_TCD.TCD[9].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH9_ES AES__RESULT_DMA_TCD.TCD[9].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH9_INT AES__RESULT_DMA_TCD.TCD[9].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH9_SBR AES__RESULT_DMA_TCD.TCD[9].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH9_PRI AES__RESULT_DMA_TCD.TCD[9].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD9_SADDR AES__RESULT_DMA_TCD.TCD[9].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD9_SOFF AES__RESULT_DMA_TCD.TCD[9].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD9_ATTR AES__RESULT_DMA_TCD.TCD[9].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD9_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[9].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD9_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[9].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD9_SLAST_SDA AES__RESULT_DMA_TCD.TCD[9].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD9_DADDR AES__RESULT_DMA_TCD.TCD[9].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD9_DOFF AES__RESULT_DMA_TCD.TCD[9].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD9_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[9].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD9_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[9].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD9_DLAST_SGA AES__RESULT_DMA_TCD.TCD[9].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD9_CSR AES__RESULT_DMA_TCD.TCD[9].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD9_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[9].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD9_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[9].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH10_CSR AES__RESULT_DMA_TCD.TCD[10].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH10_ES AES__RESULT_DMA_TCD.TCD[10].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH10_INT AES__RESULT_DMA_TCD.TCD[10].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH10_SBR AES__RESULT_DMA_TCD.TCD[10].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH10_PRI AES__RESULT_DMA_TCD.TCD[10].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD10_SADDR AES__RESULT_DMA_TCD.TCD[10].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD10_SOFF AES__RESULT_DMA_TCD.TCD[10].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD10_ATTR AES__RESULT_DMA_TCD.TCD[10].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD10_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[10].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD10_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[10].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD10_SLAST_SDA AES__RESULT_DMA_TCD.TCD[10].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD10_DADDR AES__RESULT_DMA_TCD.TCD[10].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD10_DOFF AES__RESULT_DMA_TCD.TCD[10].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD10_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[10].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD10_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[10].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD10_DLAST_SGA AES__RESULT_DMA_TCD.TCD[10].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD10_CSR AES__RESULT_DMA_TCD.TCD[10].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD10_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[10].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD10_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[10].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH11_CSR AES__RESULT_DMA_TCD.TCD[11].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH11_ES AES__RESULT_DMA_TCD.TCD[11].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH11_INT AES__RESULT_DMA_TCD.TCD[11].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH11_SBR AES__RESULT_DMA_TCD.TCD[11].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH11_PRI AES__RESULT_DMA_TCD.TCD[11].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD11_SADDR AES__RESULT_DMA_TCD.TCD[11].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD11_SOFF AES__RESULT_DMA_TCD.TCD[11].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD11_ATTR AES__RESULT_DMA_TCD.TCD[11].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD11_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[11].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD11_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[11].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD11_SLAST_SDA AES__RESULT_DMA_TCD.TCD[11].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD11_DADDR AES__RESULT_DMA_TCD.TCD[11].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD11_DOFF AES__RESULT_DMA_TCD.TCD[11].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD11_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[11].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD11_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[11].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD11_DLAST_SGA AES__RESULT_DMA_TCD.TCD[11].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD11_CSR AES__RESULT_DMA_TCD.TCD[11].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD11_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[11].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD11_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[11].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH12_CSR AES__RESULT_DMA_TCD.TCD[12].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH12_ES AES__RESULT_DMA_TCD.TCD[12].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH12_INT AES__RESULT_DMA_TCD.TCD[12].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH12_SBR AES__RESULT_DMA_TCD.TCD[12].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH12_PRI AES__RESULT_DMA_TCD.TCD[12].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD12_SADDR AES__RESULT_DMA_TCD.TCD[12].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD12_SOFF AES__RESULT_DMA_TCD.TCD[12].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD12_ATTR AES__RESULT_DMA_TCD.TCD[12].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD12_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[12].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD12_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[12].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD12_SLAST_SDA AES__RESULT_DMA_TCD.TCD[12].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD12_DADDR AES__RESULT_DMA_TCD.TCD[12].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD12_DOFF AES__RESULT_DMA_TCD.TCD[12].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD12_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[12].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD12_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[12].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD12_DLAST_SGA AES__RESULT_DMA_TCD.TCD[12].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD12_CSR AES__RESULT_DMA_TCD.TCD[12].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD12_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[12].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD12_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[12].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH13_CSR AES__RESULT_DMA_TCD.TCD[13].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH13_ES AES__RESULT_DMA_TCD.TCD[13].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH13_INT AES__RESULT_DMA_TCD.TCD[13].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH13_SBR AES__RESULT_DMA_TCD.TCD[13].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH13_PRI AES__RESULT_DMA_TCD.TCD[13].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD13_SADDR AES__RESULT_DMA_TCD.TCD[13].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD13_SOFF AES__RESULT_DMA_TCD.TCD[13].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD13_ATTR AES__RESULT_DMA_TCD.TCD[13].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD13_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[13].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD13_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[13].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD13_SLAST_SDA AES__RESULT_DMA_TCD.TCD[13].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD13_DADDR AES__RESULT_DMA_TCD.TCD[13].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD13_DOFF AES__RESULT_DMA_TCD.TCD[13].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD13_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[13].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD13_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[13].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD13_DLAST_SGA AES__RESULT_DMA_TCD.TCD[13].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD13_CSR AES__RESULT_DMA_TCD.TCD[13].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD13_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[13].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD13_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[13].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH14_CSR AES__RESULT_DMA_TCD.TCD[14].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH14_ES AES__RESULT_DMA_TCD.TCD[14].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH14_INT AES__RESULT_DMA_TCD.TCD[14].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH14_SBR AES__RESULT_DMA_TCD.TCD[14].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH14_PRI AES__RESULT_DMA_TCD.TCD[14].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD14_SADDR AES__RESULT_DMA_TCD.TCD[14].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD14_SOFF AES__RESULT_DMA_TCD.TCD[14].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD14_ATTR AES__RESULT_DMA_TCD.TCD[14].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD14_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[14].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD14_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[14].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD14_SLAST_SDA AES__RESULT_DMA_TCD.TCD[14].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD14_DADDR AES__RESULT_DMA_TCD.TCD[14].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD14_DOFF AES__RESULT_DMA_TCD.TCD[14].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD14_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[14].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD14_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[14].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD14_DLAST_SGA AES__RESULT_DMA_TCD.TCD[14].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD14_CSR AES__RESULT_DMA_TCD.TCD[14].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD14_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[14].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD14_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[14].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH15_CSR AES__RESULT_DMA_TCD.TCD[15].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH15_ES AES__RESULT_DMA_TCD.TCD[15].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH15_INT AES__RESULT_DMA_TCD.TCD[15].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH15_SBR AES__RESULT_DMA_TCD.TCD[15].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH15_PRI AES__RESULT_DMA_TCD.TCD[15].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD15_SADDR AES__RESULT_DMA_TCD.TCD[15].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD15_SOFF AES__RESULT_DMA_TCD.TCD[15].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD15_ATTR AES__RESULT_DMA_TCD.TCD[15].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD15_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[15].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD15_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[15].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD15_SLAST_SDA AES__RESULT_DMA_TCD.TCD[15].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD15_DADDR AES__RESULT_DMA_TCD.TCD[15].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD15_DOFF AES__RESULT_DMA_TCD.TCD[15].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD15_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[15].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD15_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[15].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD15_DLAST_SGA AES__RESULT_DMA_TCD.TCD[15].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD15_CSR AES__RESULT_DMA_TCD.TCD[15].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD15_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[15].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD15_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[15].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH16_CSR AES__RESULT_DMA_TCD.TCD[16].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH16_ES AES__RESULT_DMA_TCD.TCD[16].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH16_INT AES__RESULT_DMA_TCD.TCD[16].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH16_SBR AES__RESULT_DMA_TCD.TCD[16].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH16_PRI AES__RESULT_DMA_TCD.TCD[16].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD16_SADDR AES__RESULT_DMA_TCD.TCD[16].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD16_SOFF AES__RESULT_DMA_TCD.TCD[16].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD16_ATTR AES__RESULT_DMA_TCD.TCD[16].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD16_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[16].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD16_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[16].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD16_SLAST_SDA AES__RESULT_DMA_TCD.TCD[16].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD16_DADDR AES__RESULT_DMA_TCD.TCD[16].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD16_DOFF AES__RESULT_DMA_TCD.TCD[16].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD16_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[16].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD16_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[16].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD16_DLAST_SGA AES__RESULT_DMA_TCD.TCD[16].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD16_CSR AES__RESULT_DMA_TCD.TCD[16].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD16_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[16].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD16_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[16].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH17_CSR AES__RESULT_DMA_TCD.TCD[17].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH17_ES AES__RESULT_DMA_TCD.TCD[17].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH17_INT AES__RESULT_DMA_TCD.TCD[17].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH17_SBR AES__RESULT_DMA_TCD.TCD[17].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH17_PRI AES__RESULT_DMA_TCD.TCD[17].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD17_SADDR AES__RESULT_DMA_TCD.TCD[17].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD17_SOFF AES__RESULT_DMA_TCD.TCD[17].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD17_ATTR AES__RESULT_DMA_TCD.TCD[17].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD17_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[17].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD17_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[17].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD17_SLAST_SDA AES__RESULT_DMA_TCD.TCD[17].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD17_DADDR AES__RESULT_DMA_TCD.TCD[17].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD17_DOFF AES__RESULT_DMA_TCD.TCD[17].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD17_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[17].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD17_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[17].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD17_DLAST_SGA AES__RESULT_DMA_TCD.TCD[17].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD17_CSR AES__RESULT_DMA_TCD.TCD[17].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD17_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[17].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD17_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[17].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH18_CSR AES__RESULT_DMA_TCD.TCD[18].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH18_ES AES__RESULT_DMA_TCD.TCD[18].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH18_INT AES__RESULT_DMA_TCD.TCD[18].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH18_SBR AES__RESULT_DMA_TCD.TCD[18].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH18_PRI AES__RESULT_DMA_TCD.TCD[18].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD18_SADDR AES__RESULT_DMA_TCD.TCD[18].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD18_SOFF AES__RESULT_DMA_TCD.TCD[18].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD18_ATTR AES__RESULT_DMA_TCD.TCD[18].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD18_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[18].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD18_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[18].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD18_SLAST_SDA AES__RESULT_DMA_TCD.TCD[18].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD18_DADDR AES__RESULT_DMA_TCD.TCD[18].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD18_DOFF AES__RESULT_DMA_TCD.TCD[18].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD18_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[18].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD18_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[18].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD18_DLAST_SGA AES__RESULT_DMA_TCD.TCD[18].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD18_CSR AES__RESULT_DMA_TCD.TCD[18].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD18_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[18].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD18_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[18].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH19_CSR AES__RESULT_DMA_TCD.TCD[19].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH19_ES AES__RESULT_DMA_TCD.TCD[19].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH19_INT AES__RESULT_DMA_TCD.TCD[19].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH19_SBR AES__RESULT_DMA_TCD.TCD[19].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH19_PRI AES__RESULT_DMA_TCD.TCD[19].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD19_SADDR AES__RESULT_DMA_TCD.TCD[19].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD19_SOFF AES__RESULT_DMA_TCD.TCD[19].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD19_ATTR AES__RESULT_DMA_TCD.TCD[19].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD19_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[19].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD19_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[19].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD19_SLAST_SDA AES__RESULT_DMA_TCD.TCD[19].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD19_DADDR AES__RESULT_DMA_TCD.TCD[19].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD19_DOFF AES__RESULT_DMA_TCD.TCD[19].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD19_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[19].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD19_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[19].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD19_DLAST_SGA AES__RESULT_DMA_TCD.TCD[19].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD19_CSR AES__RESULT_DMA_TCD.TCD[19].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD19_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[19].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD19_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[19].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH20_CSR AES__RESULT_DMA_TCD.TCD[20].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH20_ES AES__RESULT_DMA_TCD.TCD[20].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH20_INT AES__RESULT_DMA_TCD.TCD[20].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH20_SBR AES__RESULT_DMA_TCD.TCD[20].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH20_PRI AES__RESULT_DMA_TCD.TCD[20].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD20_SADDR AES__RESULT_DMA_TCD.TCD[20].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD20_SOFF AES__RESULT_DMA_TCD.TCD[20].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD20_ATTR AES__RESULT_DMA_TCD.TCD[20].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD20_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[20].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD20_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[20].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD20_SLAST_SDA AES__RESULT_DMA_TCD.TCD[20].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD20_DADDR AES__RESULT_DMA_TCD.TCD[20].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD20_DOFF AES__RESULT_DMA_TCD.TCD[20].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD20_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[20].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD20_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[20].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD20_DLAST_SGA AES__RESULT_DMA_TCD.TCD[20].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD20_CSR AES__RESULT_DMA_TCD.TCD[20].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD20_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[20].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD20_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[20].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH21_CSR AES__RESULT_DMA_TCD.TCD[21].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH21_ES AES__RESULT_DMA_TCD.TCD[21].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH21_INT AES__RESULT_DMA_TCD.TCD[21].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH21_SBR AES__RESULT_DMA_TCD.TCD[21].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH21_PRI AES__RESULT_DMA_TCD.TCD[21].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD21_SADDR AES__RESULT_DMA_TCD.TCD[21].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD21_SOFF AES__RESULT_DMA_TCD.TCD[21].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD21_ATTR AES__RESULT_DMA_TCD.TCD[21].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD21_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[21].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD21_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[21].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD21_SLAST_SDA AES__RESULT_DMA_TCD.TCD[21].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD21_DADDR AES__RESULT_DMA_TCD.TCD[21].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD21_DOFF AES__RESULT_DMA_TCD.TCD[21].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD21_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[21].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD21_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[21].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD21_DLAST_SGA AES__RESULT_DMA_TCD.TCD[21].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD21_CSR AES__RESULT_DMA_TCD.TCD[21].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD21_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[21].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD21_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[21].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH22_CSR AES__RESULT_DMA_TCD.TCD[22].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH22_ES AES__RESULT_DMA_TCD.TCD[22].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH22_INT AES__RESULT_DMA_TCD.TCD[22].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH22_SBR AES__RESULT_DMA_TCD.TCD[22].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH22_PRI AES__RESULT_DMA_TCD.TCD[22].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD22_SADDR AES__RESULT_DMA_TCD.TCD[22].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD22_SOFF AES__RESULT_DMA_TCD.TCD[22].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD22_ATTR AES__RESULT_DMA_TCD.TCD[22].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD22_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[22].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD22_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[22].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD22_SLAST_SDA AES__RESULT_DMA_TCD.TCD[22].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD22_DADDR AES__RESULT_DMA_TCD.TCD[22].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD22_DOFF AES__RESULT_DMA_TCD.TCD[22].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD22_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[22].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD22_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[22].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD22_DLAST_SGA AES__RESULT_DMA_TCD.TCD[22].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD22_CSR AES__RESULT_DMA_TCD.TCD[22].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD22_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[22].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD22_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[22].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_CH23_CSR AES__RESULT_DMA_TCD.TCD[23].CH_CSR.R /* Channel Control and Status */
#define AES__RESULT_DMA_TCD_CH23_ES AES__RESULT_DMA_TCD.TCD[23].CH_ES.R /* Channel Error Status */
#define AES__RESULT_DMA_TCD_CH23_INT AES__RESULT_DMA_TCD.TCD[23].CH_INT.R /* Channel Interrupt Status */
#define AES__RESULT_DMA_TCD_CH23_SBR AES__RESULT_DMA_TCD.TCD[23].CH_SBR.R /* Channel System Bus */
#define AES__RESULT_DMA_TCD_CH23_PRI AES__RESULT_DMA_TCD.TCD[23].CH_PRI.R /* Channel Priority */
#define AES__RESULT_DMA_TCD_TCD23_SADDR AES__RESULT_DMA_TCD.TCD[23].SADDR.R /* TCD Source Address */
#define AES__RESULT_DMA_TCD_TCD23_SOFF AES__RESULT_DMA_TCD.TCD[23].SOFF.R /* TCD Signed Source Address Offset */
#define AES__RESULT_DMA_TCD_TCD23_ATTR AES__RESULT_DMA_TCD.TCD[23].ATTR.R /* TCD Transfer Attributes */
#define AES__RESULT_DMA_TCD_TCD23_NBYTES_MLOFFNO AES__RESULT_DMA_TCD.TCD[23].NBYTES.NBYTES_MLOFFNO.R /* TCD Transfer Size Without Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD23_NBYTES_MLOFFYES AES__RESULT_DMA_TCD.TCD[23].NBYTES.NBYTES_MLOFFYES.R /* TCD Transfer Size with Minor Loop Offsets */
#define AES__RESULT_DMA_TCD_TCD23_SLAST_SDA AES__RESULT_DMA_TCD.TCD[23].SLAST_SDA.R /* TCD Last Source Address Adjustment / Store DADDR Address */
#define AES__RESULT_DMA_TCD_TCD23_DADDR AES__RESULT_DMA_TCD.TCD[23].DADDR.R /* TCD Destination Address */
#define AES__RESULT_DMA_TCD_TCD23_DOFF AES__RESULT_DMA_TCD.TCD[23].DOFF.R /* TCD Signed Destination Address Offset */
#define AES__RESULT_DMA_TCD_TCD23_CITER_ELINKNO AES__RESULT_DMA_TCD.TCD[23].CITER.CITER_ELINKNO.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD23_CITER_ELINKYES AES__RESULT_DMA_TCD.TCD[23].CITER.CITER_ELINKYES.R /* TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
#define AES__RESULT_DMA_TCD_TCD23_DLAST_SGA AES__RESULT_DMA_TCD.TCD[23].DLAST_SGA.R /* TCD Last Destination Address Adjustment / Scatter Gather Address */
#define AES__RESULT_DMA_TCD_TCD23_CSR AES__RESULT_DMA_TCD.TCD[23].CSR.R /* TCD Control and Status */
#define AES__RESULT_DMA_TCD_TCD23_BITER_ELINKNO AES__RESULT_DMA_TCD.TCD[23].BITER.BITER_ELINKNO.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
#define AES__RESULT_DMA_TCD_TCD23_BITER_ELINKYES AES__RESULT_DMA_TCD.TCD[23].BITER.BITER_ELINKYES.R /* TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */

/* ROUND_ROBIN_ARBITER */
#define CEVA_SPF2__ROUND_ROBIN_ARBITER_RRA_SLT_LO CEVA_SPF2__ROUND_ROBIN_ARBITER.RRA_SLT_LO.R /* RRA Slot LO Register */
#define CEVA_SPF2__ROUND_ROBIN_ARBITER_RRA_SLT_HI CEVA_SPF2__ROUND_ROBIN_ARBITER.RRA_SLT_HI.R /* RRA Slot HI Register */
#define CEVA_SPF2__ROUND_ROBIN_ARBITER_RRA_SLT_PRI CEVA_SPF2__ROUND_ROBIN_ARBITER.RRA_SLT_PRI.R /* RRA Slot Priority Register */
#define CEVA_SPF2__ROUND_ROBIN_ARBITER_RRA_WEIGHT_LO CEVA_SPF2__ROUND_ROBIN_ARBITER.RRA_WEIGHT_LO.R /* RRA Weight LO Register */
#define CEVA_SPF2__ROUND_ROBIN_ARBITER_RRA_WEIGHT_HI CEVA_SPF2__ROUND_ROBIN_ARBITER.RRA_WEIGHT_HI.R /* RRA Weight HI Register */

/* RTT */
#define CEVA_SPF2__RTT_MODE  CEVA_SPF2__RTT.MODE.R         /* MODE Register */

/* RTUE_NIC_D */
#define RTU0__RTUE_NIC_D_remap RTU0__RTUE_NIC_D.REMAP.R    /* Remap */

#define RTU1__RTUE_NIC_D_remap RTU1__RTUE_NIC_D.REMAP.R    /* Remap */

/* RTUF_NIC_D */
#define RTU0__RTUF_NIC_D_remap RTU0__RTUF_NIC_D.REMAP.R    /* Remap */

#define RTU1__RTUF_NIC_D_remap RTU1__RTUF_NIC_D.REMAP.R    /* Remap */

/* RTUM_NIC_D */
#define RTU0__RTUM_NIC_D_remap RTU0__RTUM_NIC_D.REMAP.R    /* Remap */

#define RTU1__RTUM_NIC_D_remap RTU1__RTUM_NIC_D.REMAP.R    /* Remap */

/* RTUP_NIC_B */
#define RTU0__RTUP_NIC_B_remap RTU0__RTUP_NIC_B.REMAP.R    /* Remap */

#define RTU1__RTUP_NIC_B_remap RTU1__RTUP_NIC_B.REMAP.R    /* Remap */

/* RTU_EIM */
#define RTU0__EIM_EIMCR      RTU0__EIM.EIMCR.R             /* Error Injection Module Configuration Register */
#define RTU0__EIM_EICHEN     RTU0__EIM.EICHEN.R            /* Error Injection Channel Enable register */
#define RTU0__EIM_EICHD0_WORD1 RTU0__EIM.EICHD0_WORD1.R    /* Error Injection Channel Descriptor 0, Word1 */
#define RTU0__EIM_EICHD0_WORD2 RTU0__EIM.EICHD0_WORD2.R    /* Error Injection Channel Descriptor 0, Word2 */
#define RTU0__EIM_EICHD1_WORD1 RTU0__EIM.EICHD1_WORD1.R    /* Error Injection Channel Descriptor 1, Word1 */
#define RTU0__EIM_EICHD1_WORD2 RTU0__EIM.EICHD1_WORD2.R    /* Error Injection Channel Descriptor 1, Word2 */
#define RTU0__EIM_EICHD2_WORD1 RTU0__EIM.EICHD2_WORD1.R    /* Error Injection Channel Descriptor 2, Word1 */
#define RTU0__EIM_EICHD3_WORD1 RTU0__EIM.EICHD3_WORD1.R    /* Error Injection Channel Descriptor 3, Word1 */
#define RTU0__EIM_EICHD4_WORD1 RTU0__EIM.EICHD4_WORD1.R    /* Error Injection Channel Descriptor 4, Word1 */
#define RTU0__EIM_EICHD29_WORD1 RTU0__EIM.EICHD29_WORD1.R  /* Error Injection Channel Descriptor 29, Word1 */
#define RTU0__EIM_EICHD30_WORD1 RTU0__EIM.EICHD30_WORD1.R  /* Error Injection Channel Descriptor 30, Word1 */
#define RTU0__EIM_EICHD30_WORD2 RTU0__EIM.EICHD30_WORD2.R  /* Error Injection Channel Descriptor 30, Word2 */
#define RTU0__EIM_EICHD31_WORD1 RTU0__EIM.EICHD31_WORD1.R  /* Error Injection Channel Descriptor 31, Word1 */
#define RTU0__EIM_EICHD31_WORD2 RTU0__EIM.EICHD31_WORD2.R  /* Error Injection Channel Descriptor 31, Word2 */

#define RTU1__EIM_EIMCR      RTU1__EIM.EIMCR.R             /* Error Injection Module Configuration Register */
#define RTU1__EIM_EICHEN     RTU1__EIM.EICHEN.R            /* Error Injection Channel Enable register */
#define RTU1__EIM_EICHD0_WORD1 RTU1__EIM.EICHD0_WORD1.R    /* Error Injection Channel Descriptor 0, Word1 */
#define RTU1__EIM_EICHD0_WORD2 RTU1__EIM.EICHD0_WORD2.R    /* Error Injection Channel Descriptor 0, Word2 */
#define RTU1__EIM_EICHD1_WORD1 RTU1__EIM.EICHD1_WORD1.R    /* Error Injection Channel Descriptor 1, Word1 */
#define RTU1__EIM_EICHD1_WORD2 RTU1__EIM.EICHD1_WORD2.R    /* Error Injection Channel Descriptor 1, Word2 */
#define RTU1__EIM_EICHD2_WORD1 RTU1__EIM.EICHD2_WORD1.R    /* Error Injection Channel Descriptor 2, Word1 */
#define RTU1__EIM_EICHD3_WORD1 RTU1__EIM.EICHD3_WORD1.R    /* Error Injection Channel Descriptor 3, Word1 */
#define RTU1__EIM_EICHD4_WORD1 RTU1__EIM.EICHD4_WORD1.R    /* Error Injection Channel Descriptor 4, Word1 */
#define RTU1__EIM_EICHD29_WORD1 RTU1__EIM.EICHD29_WORD1.R  /* Error Injection Channel Descriptor 29, Word1 */
#define RTU1__EIM_EICHD30_WORD1 RTU1__EIM.EICHD30_WORD1.R  /* Error Injection Channel Descriptor 30, Word1 */
#define RTU1__EIM_EICHD30_WORD2 RTU1__EIM.EICHD30_WORD2.R  /* Error Injection Channel Descriptor 30, Word2 */
#define RTU1__EIM_EICHD31_WORD1 RTU1__EIM.EICHD31_WORD1.R  /* Error Injection Channel Descriptor 31, Word1 */
#define RTU1__EIM_EICHD31_WORD2 RTU1__EIM.EICHD31_WORD2.R  /* Error Injection Channel Descriptor 31, Word2 */

/* RTU_GPR */
#define RTU0__GPR_CFG_CORE   RTU0__GPR.CFG_CORE.R          /* Core Configuration */
#define RTU0__GPR_CFG_BUPERR_DIS0 RTU0__GPR.CFG_BUPERR_DIS0.R /* Bus Parity Error Disable Group0 */
#define RTU0__GPR_CFG_BUPERR_DIS1 RTU0__GPR.CFG_BUPERR_DIS1.R /* Bus Parity Error Disable Group0 */
#define RTU0__GPR_CFG_FD     RTU0__GPR.CFG_FD.R            /* Core Fence and Drain Configuration */
#define RTU0__GPR_CFG_CNTDV  RTU0__GPR.CFG_CNTDV.R         /* Generic Timer Count Divider Control */
#define RTU0__GPR_CFG_HALT   RTU0__GPR.CFG_HALT.R          /* Core Halt Control */
#define RTU0__GPR_STAT_FD    RTU0__GPR.STAT_FD.R           /* Core Fence and Drain Status */
#define RTU0__GPR_STAT_WFI   RTU0__GPR.STAT_WFI.R          /* Core STANDBYWFI Status */
#define RTU0__GPR_STAT_WFE   RTU0__GPR.STAT_WFE.R          /* Core STANDBYWFE Status */
#define RTU0__GPR_STAT_WRSTREQ RTU0__GPR.STAT_WRSTREQ.R    /* Core WARMRSTREQ Status */
#define RTU0__GPR_STAT_EVC0  RTU0__GPR.STAT_EVC0.R         /* Core0 External Event Status */
#define RTU0__GPR_STAT_EVC1  RTU0__GPR.STAT_EVC1.R         /* Core1 External Event Status */
#define RTU0__GPR_STAT_EVC2  RTU0__GPR.STAT_EVC2.R         /* Core2 External Event Status */
#define RTU0__GPR_STAT_EVC3  RTU0__GPR.STAT_EVC3.R         /* Core3 External Event Status */
#define RTU0__GPR_STAT_MERRPM0 RTU0__GPR.STAT_MERRPM0.R    /* Primary Error Interface Memory Identifier Core0 */
#define RTU0__GPR_STAT_MERRPX0 RTU0__GPR.STAT_MERRPX0.R    /* Primary Error Interface Index Core0 */
#define RTU0__GPR_STAT_MERRSM0 RTU0__GPR.STAT_MERRSM0.R    /* Secondary Error Interface Memory Identifier Core0 */
#define RTU0__GPR_STAT_MERRSX0 RTU0__GPR.STAT_MERRSX0.R    /* Secondary Error Interface Index Core0 */
#define RTU0__GPR_STAT_MERRPM1 RTU0__GPR.STAT_MERRPM1.R    /* Primary Error Interface Memory Identifier Core1 */
#define RTU0__GPR_STAT_MERRPX1 RTU0__GPR.STAT_MERRPX1.R    /* Primary Error Interface Index Core1 */
#define RTU0__GPR_STAT_MERRSM1 RTU0__GPR.STAT_MERRSM1.R    /* Secondary Error Interface Memory Identifier Core1 */
#define RTU0__GPR_STAT_MERRSX1 RTU0__GPR.STAT_MERRSX1.R    /* Secondary Error Interface Index Core1 */
#define RTU0__GPR_STAT_MERRPM2 RTU0__GPR.STAT_MERRPM2.R    /* Primary Error Interface Memory Identifier Core2 */
#define RTU0__GPR_STAT_MERRPX2 RTU0__GPR.STAT_MERRPX2.R    /* Primary Error Interface Index Core2 */
#define RTU0__GPR_STAT_MERRSM2 RTU0__GPR.STAT_MERRSM2.R    /* Secondary Error Interface Memory Identifier Core2 */
#define RTU0__GPR_STAT_MERRSX2 RTU0__GPR.STAT_MERRSX2.R    /* Secondary Error Interface Index Core2 */
#define RTU0__GPR_STAT_MERRPM3 RTU0__GPR.STAT_MERRPM3.R    /* Primary Error Interface Memory Identifier Core3 */
#define RTU0__GPR_STAT_MERRPX3 RTU0__GPR.STAT_MERRPX3.R    /* Primary Error Interface Index Core3 */
#define RTU0__GPR_STAT_MERRSM3 RTU0__GPR.STAT_MERRSM3.R    /* Secondary Error Interface Memory Identifier Core3 */
#define RTU0__GPR_STAT_MERRSX3 RTU0__GPR.STAT_MERRSX3.R    /* Secondary Error Interface Index Core3 */
#define RTU0__GPR_STAT_PMU0EV0 RTU0__GPR.STAT_PMU0EV0.R    /* PMU Event Core0 Group0 */
#define RTU0__GPR_STAT_PMU1EV0 RTU0__GPR.STAT_PMU1EV0.R    /* PMU Event Core0 Group1 */
#define RTU0__GPR_STAT_PMU0EV1 RTU0__GPR.STAT_PMU0EV1.R    /* PMU Event Core1 Group0 */
#define RTU0__GPR_STAT_PMU1EV1 RTU0__GPR.STAT_PMU1EV1.R    /* PMU Event Core1 Group1 */
#define RTU0__GPR_STAT_PMU0EV2 RTU0__GPR.STAT_PMU0EV2.R    /* PMU Event Core2 Group0 */
#define RTU0__GPR_STAT_PMU1EV2 RTU0__GPR.STAT_PMU1EV2.R    /* PMU Event Core2 Group1 */
#define RTU0__GPR_STAT_PMU0EV3 RTU0__GPR.STAT_PMU0EV3.R    /* PMU Event Core3 Group0 */
#define RTU0__GPR_STAT_PMU1EV3 RTU0__GPR.STAT_PMU1EV3.R    /* PMU Event Core3 Group1 */

#define RTU1__GPR_CFG_CORE   RTU1__GPR.CFG_CORE.R          /* Core Configuration */
#define RTU1__GPR_CFG_BUPERR_DIS0 RTU1__GPR.CFG_BUPERR_DIS0.R /* Bus Parity Error Disable Group0 */
#define RTU1__GPR_CFG_BUPERR_DIS1 RTU1__GPR.CFG_BUPERR_DIS1.R /* Bus Parity Error Disable Group0 */
#define RTU1__GPR_CFG_FD     RTU1__GPR.CFG_FD.R            /* Core Fence and Drain Configuration */
#define RTU1__GPR_CFG_CNTDV  RTU1__GPR.CFG_CNTDV.R         /* Generic Timer Count Divider Control */
#define RTU1__GPR_CFG_HALT   RTU1__GPR.CFG_HALT.R          /* Core Halt Control */
#define RTU1__GPR_STAT_FD    RTU1__GPR.STAT_FD.R           /* Core Fence and Drain Status */
#define RTU1__GPR_STAT_WFI   RTU1__GPR.STAT_WFI.R          /* Core STANDBYWFI Status */
#define RTU1__GPR_STAT_WFE   RTU1__GPR.STAT_WFE.R          /* Core STANDBYWFE Status */
#define RTU1__GPR_STAT_WRSTREQ RTU1__GPR.STAT_WRSTREQ.R    /* Core WARMRSTREQ Status */
#define RTU1__GPR_STAT_EVC0  RTU1__GPR.STAT_EVC0.R         /* Core0 External Event Status */
#define RTU1__GPR_STAT_EVC1  RTU1__GPR.STAT_EVC1.R         /* Core1 External Event Status */
#define RTU1__GPR_STAT_EVC2  RTU1__GPR.STAT_EVC2.R         /* Core2 External Event Status */
#define RTU1__GPR_STAT_EVC3  RTU1__GPR.STAT_EVC3.R         /* Core3 External Event Status */
#define RTU1__GPR_STAT_MERRPM0 RTU1__GPR.STAT_MERRPM0.R    /* Primary Error Interface Memory Identifier Core0 */
#define RTU1__GPR_STAT_MERRPX0 RTU1__GPR.STAT_MERRPX0.R    /* Primary Error Interface Index Core0 */
#define RTU1__GPR_STAT_MERRSM0 RTU1__GPR.STAT_MERRSM0.R    /* Secondary Error Interface Memory Identifier Core0 */
#define RTU1__GPR_STAT_MERRSX0 RTU1__GPR.STAT_MERRSX0.R    /* Secondary Error Interface Index Core0 */
#define RTU1__GPR_STAT_MERRPM1 RTU1__GPR.STAT_MERRPM1.R    /* Primary Error Interface Memory Identifier Core1 */
#define RTU1__GPR_STAT_MERRPX1 RTU1__GPR.STAT_MERRPX1.R    /* Primary Error Interface Index Core1 */
#define RTU1__GPR_STAT_MERRSM1 RTU1__GPR.STAT_MERRSM1.R    /* Secondary Error Interface Memory Identifier Core1 */
#define RTU1__GPR_STAT_MERRSX1 RTU1__GPR.STAT_MERRSX1.R    /* Secondary Error Interface Index Core1 */
#define RTU1__GPR_STAT_MERRPM2 RTU1__GPR.STAT_MERRPM2.R    /* Primary Error Interface Memory Identifier Core2 */
#define RTU1__GPR_STAT_MERRPX2 RTU1__GPR.STAT_MERRPX2.R    /* Primary Error Interface Index Core2 */
#define RTU1__GPR_STAT_MERRSM2 RTU1__GPR.STAT_MERRSM2.R    /* Secondary Error Interface Memory Identifier Core2 */
#define RTU1__GPR_STAT_MERRSX2 RTU1__GPR.STAT_MERRSX2.R    /* Secondary Error Interface Index Core2 */
#define RTU1__GPR_STAT_MERRPM3 RTU1__GPR.STAT_MERRPM3.R    /* Primary Error Interface Memory Identifier Core3 */
#define RTU1__GPR_STAT_MERRPX3 RTU1__GPR.STAT_MERRPX3.R    /* Primary Error Interface Index Core3 */
#define RTU1__GPR_STAT_MERRSM3 RTU1__GPR.STAT_MERRSM3.R    /* Secondary Error Interface Memory Identifier Core3 */
#define RTU1__GPR_STAT_MERRSX3 RTU1__GPR.STAT_MERRSX3.R    /* Secondary Error Interface Index Core3 */
#define RTU1__GPR_STAT_PMU0EV0 RTU1__GPR.STAT_PMU0EV0.R    /* PMU Event Core0 Group0 */
#define RTU1__GPR_STAT_PMU1EV0 RTU1__GPR.STAT_PMU1EV0.R    /* PMU Event Core0 Group1 */
#define RTU1__GPR_STAT_PMU0EV1 RTU1__GPR.STAT_PMU0EV1.R    /* PMU Event Core1 Group0 */
#define RTU1__GPR_STAT_PMU1EV1 RTU1__GPR.STAT_PMU1EV1.R    /* PMU Event Core1 Group1 */
#define RTU1__GPR_STAT_PMU0EV2 RTU1__GPR.STAT_PMU0EV2.R    /* PMU Event Core2 Group0 */
#define RTU1__GPR_STAT_PMU1EV2 RTU1__GPR.STAT_PMU1EV2.R    /* PMU Event Core2 Group1 */
#define RTU1__GPR_STAT_PMU0EV3 RTU1__GPR.STAT_PMU0EV3.R    /* PMU Event Core3 Group0 */
#define RTU1__GPR_STAT_PMU1EV3 RTU1__GPR.STAT_PMU1EV3.R    /* PMU Event Core3 Group1 */

/* RTU_L_VFCCU */
#define RTU0__L_VFCCU_GDFHID_C0 RTU0__L_VFCCU.GDFHID_C0.R  /* Global DID-FHID Map */
#define RTU0__L_VFCCU_GDFHID_C1 RTU0__L_VFCCU.GDFHID_C1.R  /* Global DID-FHID Map */
#define RTU0__L_VFCCU_GFLTPO_C0 RTU0__L_VFCCU.GFLTPO_C0.R  /* Global Fault Polarity */
#define RTU0__L_VFCCU_GFLTPO_C1 RTU0__L_VFCCU.GFLTPO_C1.R  /* Global Fault Polarity */
#define RTU0__L_VFCCU_GFLTRC_C0 RTU0__L_VFCCU.GFLTRC_C0.R  /* Global Fault Recovery */
#define RTU0__L_VFCCU_GFLTRC_C1 RTU0__L_VFCCU.GFLTRC_C1.R  /* Global Fault Recovery */
#define RTU0__L_VFCCU_GFLTOVDC0 RTU0__L_VFCCU.GFLTOVDC0.R  /* Global Fault Overflow Detection */
#define RTU0__L_VFCCU_GFLTOVDC1 RTU0__L_VFCCU.GFLTOVDC1.R  /* Global Fault Overflow Detection */
#define RTU0__L_VFCCU_GSWFLODC RTU0__L_VFCCU.GSWFLODC.R    /* Global Software Fault Overflow Detection Disable */
#define RTU0__L_VFCCU_GCTRL  RTU0__L_VFCCU.GCTRL.R         /* Global Space Control */
#define RTU0__L_VFCCU_GINTOVFS RTU0__L_VFCCU.GINTOVFS.R    /* Global DID FSM Status */
#define RTU0__L_VFCCU_GDBGCFG RTU0__L_VFCCU.GDBGCFG.R      /* Global Debug */
#define RTU0__L_VFCCU_GDBGSTAT RTU0__L_VFCCU.GDBGSTAT.R    /* Global Debug Status */
#define RTU0__L_VFCCU_SWRPTDID0 RTU0__L_VFCCU.SW_RKN[0].SWRPTDID.R /* Software Fault Reported DID */
#define RTU0__L_VFCCU_SWRKSET_00 RTU0__L_VFCCU.SW_RKN[0].SWRKSET_0.R /* Software Reaction Set */
#define RTU0__L_VFCCU_SWRKCLR_00 RTU0__L_VFCCU.SW_RKN[0].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU0__L_VFCCU_SWRPTDID1 RTU0__L_VFCCU.SW_RKN[1].SWRPTDID.R /* Software Fault Reported DID */
#define RTU0__L_VFCCU_SWRKSET_10 RTU0__L_VFCCU.SW_RKN[1].SWRKSET_0.R /* Software Reaction Set */
#define RTU0__L_VFCCU_SWRKCLR_10 RTU0__L_VFCCU.SW_RKN[1].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU0__L_VFCCU_SWRPTDID2 RTU0__L_VFCCU.SW_RKN[2].SWRPTDID.R /* Software Fault Reported DID */
#define RTU0__L_VFCCU_SWRKSET_20 RTU0__L_VFCCU.SW_RKN[2].SWRKSET_0.R /* Software Reaction Set */
#define RTU0__L_VFCCU_SWRKCLR_20 RTU0__L_VFCCU.SW_RKN[2].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU0__L_VFCCU_SWRPTDID3 RTU0__L_VFCCU.SW_RKN[3].SWRPTDID.R /* Software Fault Reported DID */
#define RTU0__L_VFCCU_SWRKSET_30 RTU0__L_VFCCU.SW_RKN[3].SWRKSET_0.R /* Software Reaction Set */
#define RTU0__L_VFCCU_SWRKCLR_30 RTU0__L_VFCCU.SW_RKN[3].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU0__L_VFCCU_SWRPTDID4 RTU0__L_VFCCU.SW_RKN[4].SWRPTDID.R /* Software Fault Reported DID */
#define RTU0__L_VFCCU_SWRKSET_40 RTU0__L_VFCCU.SW_RKN[4].SWRKSET_0.R /* Software Reaction Set */
#define RTU0__L_VFCCU_SWRKCLR_40 RTU0__L_VFCCU.SW_RKN[4].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU0__L_VFCCU_SWRPTDID5 RTU0__L_VFCCU.SW_RKN[5].SWRPTDID.R /* Software Fault Reported DID */
#define RTU0__L_VFCCU_SWRKSET_50 RTU0__L_VFCCU.SW_RKN[5].SWRKSET_0.R /* Software Reaction Set */
#define RTU0__L_VFCCU_SWRKCLR_50 RTU0__L_VFCCU.SW_RKN[5].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU0__L_VFCCU_FHCFG0 RTU0__L_VFCCU.FHCFG0.R        /* Fault Handler */
#define RTU0__L_VFCCU_FHSRVDS0 RTU0__L_VFCCU.FHSRVDS0.R    /* Fault Handler Status */
#define RTU0__L_VFCCU_FHFLTENC00 RTU0__L_VFCCU.FHFLTENC00.R /* Fault Enable */
#define RTU0__L_VFCCU_FHFLTENC01 RTU0__L_VFCCU.FHFLTENC01.R /* Fault Enable */
#define RTU0__L_VFCCU_FHFLTS00 RTU0__L_VFCCU.FHFLTS00.R    /* Fault Status */
#define RTU0__L_VFCCU_FHFLTS01 RTU0__L_VFCCU.FHFLTS01.R    /* Fault Status */
#define RTU0__L_VFCCU_FHFLTRKC00 RTU0__L_VFCCU.FHFLTRKC00.R /* Fault Reaction Set Configuration */
#define RTU0__L_VFCCU_FHFLTRKC01 RTU0__L_VFCCU.FHFLTRKC01.R /* Fault Reaction Set Configuration */
#define RTU0__L_VFCCU_FHFLTRKC02 RTU0__L_VFCCU.FHFLTRKC02.R /* Fault Reaction Set Configuration */
#define RTU0__L_VFCCU_FHFLTRKC03 RTU0__L_VFCCU.FHFLTRKC03.R /* Fault Reaction Set Configuration */
#define RTU0__L_VFCCU_FHFLTRKC04 RTU0__L_VFCCU.FHFLTRKC04.R /* Fault Reaction Set Configuration */
#define RTU0__L_VFCCU_FHFLTRKC05 RTU0__L_VFCCU.FHFLTRKC05.R /* Fault Reaction Set Configuration */
#define RTU0__L_VFCCU_FHFLTRKC06 RTU0__L_VFCCU.FHFLTRKC06.R /* Fault Reaction Set Configuration */
#define RTU0__L_VFCCU_FHIMRKC0_00 RTU0__L_VFCCU.FHIMRKC0_00.R /* Immediate Reaction Configuration */
#define RTU0__L_VFCCU_FHIMRKC0_10 RTU0__L_VFCCU.FHIMRKC0_10.R /* Immediate Reaction Configuration */
#define RTU0__L_VFCCU_FHIMRKC0_20 RTU0__L_VFCCU.FHIMRKC0_20.R /* Immediate Reaction Configuration */
#define RTU0__L_VFCCU_FHIMRKC0_30 RTU0__L_VFCCU.FHIMRKC0_30.R /* Immediate Reaction Configuration */
#define RTU0__L_VFCCU_FHIMRKC0_40 RTU0__L_VFCCU.FHIMRKC0_40.R /* Immediate Reaction Configuration */
#define RTU0__L_VFCCU_FHIMRKC0_50 RTU0__L_VFCCU.FHIMRKC0_50.R /* Immediate Reaction Configuration */

#define RTU1__L_VFCCU_GDFHID_C0 RTU1__L_VFCCU.GDFHID_C0.R  /* Global DID-FHID Map */
#define RTU1__L_VFCCU_GDFHID_C1 RTU1__L_VFCCU.GDFHID_C1.R  /* Global DID-FHID Map */
#define RTU1__L_VFCCU_GFLTPO_C0 RTU1__L_VFCCU.GFLTPO_C0.R  /* Global Fault Polarity */
#define RTU1__L_VFCCU_GFLTPO_C1 RTU1__L_VFCCU.GFLTPO_C1.R  /* Global Fault Polarity */
#define RTU1__L_VFCCU_GFLTRC_C0 RTU1__L_VFCCU.GFLTRC_C0.R  /* Global Fault Recovery */
#define RTU1__L_VFCCU_GFLTRC_C1 RTU1__L_VFCCU.GFLTRC_C1.R  /* Global Fault Recovery */
#define RTU1__L_VFCCU_GFLTOVDC0 RTU1__L_VFCCU.GFLTOVDC0.R  /* Global Fault Overflow Detection */
#define RTU1__L_VFCCU_GFLTOVDC1 RTU1__L_VFCCU.GFLTOVDC1.R  /* Global Fault Overflow Detection */
#define RTU1__L_VFCCU_GSWFLODC RTU1__L_VFCCU.GSWFLODC.R    /* Global Software Fault Overflow Detection Disable */
#define RTU1__L_VFCCU_GCTRL  RTU1__L_VFCCU.GCTRL.R         /* Global Space Control */
#define RTU1__L_VFCCU_GINTOVFS RTU1__L_VFCCU.GINTOVFS.R    /* Global DID FSM Status */
#define RTU1__L_VFCCU_GDBGCFG RTU1__L_VFCCU.GDBGCFG.R      /* Global Debug */
#define RTU1__L_VFCCU_GDBGSTAT RTU1__L_VFCCU.GDBGSTAT.R    /* Global Debug Status */
#define RTU1__L_VFCCU_SWRPTDID0 RTU1__L_VFCCU.SW_RKN[0].SWRPTDID.R /* Software Fault Reported DID */
#define RTU1__L_VFCCU_SWRKSET_00 RTU1__L_VFCCU.SW_RKN[0].SWRKSET_0.R /* Software Reaction Set */
#define RTU1__L_VFCCU_SWRKCLR_00 RTU1__L_VFCCU.SW_RKN[0].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU1__L_VFCCU_SWRPTDID1 RTU1__L_VFCCU.SW_RKN[1].SWRPTDID.R /* Software Fault Reported DID */
#define RTU1__L_VFCCU_SWRKSET_10 RTU1__L_VFCCU.SW_RKN[1].SWRKSET_0.R /* Software Reaction Set */
#define RTU1__L_VFCCU_SWRKCLR_10 RTU1__L_VFCCU.SW_RKN[1].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU1__L_VFCCU_SWRPTDID2 RTU1__L_VFCCU.SW_RKN[2].SWRPTDID.R /* Software Fault Reported DID */
#define RTU1__L_VFCCU_SWRKSET_20 RTU1__L_VFCCU.SW_RKN[2].SWRKSET_0.R /* Software Reaction Set */
#define RTU1__L_VFCCU_SWRKCLR_20 RTU1__L_VFCCU.SW_RKN[2].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU1__L_VFCCU_SWRPTDID3 RTU1__L_VFCCU.SW_RKN[3].SWRPTDID.R /* Software Fault Reported DID */
#define RTU1__L_VFCCU_SWRKSET_30 RTU1__L_VFCCU.SW_RKN[3].SWRKSET_0.R /* Software Reaction Set */
#define RTU1__L_VFCCU_SWRKCLR_30 RTU1__L_VFCCU.SW_RKN[3].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU1__L_VFCCU_SWRPTDID4 RTU1__L_VFCCU.SW_RKN[4].SWRPTDID.R /* Software Fault Reported DID */
#define RTU1__L_VFCCU_SWRKSET_40 RTU1__L_VFCCU.SW_RKN[4].SWRKSET_0.R /* Software Reaction Set */
#define RTU1__L_VFCCU_SWRKCLR_40 RTU1__L_VFCCU.SW_RKN[4].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU1__L_VFCCU_SWRPTDID5 RTU1__L_VFCCU.SW_RKN[5].SWRPTDID.R /* Software Fault Reported DID */
#define RTU1__L_VFCCU_SWRKSET_50 RTU1__L_VFCCU.SW_RKN[5].SWRKSET_0.R /* Software Reaction Set */
#define RTU1__L_VFCCU_SWRKCLR_50 RTU1__L_VFCCU.SW_RKN[5].SWRKCLR_0.R /* Software Reaction Clear */
#define RTU1__L_VFCCU_FHCFG0 RTU1__L_VFCCU.FHCFG0.R        /* Fault Handler */
#define RTU1__L_VFCCU_FHSRVDS0 RTU1__L_VFCCU.FHSRVDS0.R    /* Fault Handler Status */
#define RTU1__L_VFCCU_FHFLTENC00 RTU1__L_VFCCU.FHFLTENC00.R /* Fault Enable */
#define RTU1__L_VFCCU_FHFLTENC01 RTU1__L_VFCCU.FHFLTENC01.R /* Fault Enable */
#define RTU1__L_VFCCU_FHFLTS00 RTU1__L_VFCCU.FHFLTS00.R    /* Fault Status */
#define RTU1__L_VFCCU_FHFLTS01 RTU1__L_VFCCU.FHFLTS01.R    /* Fault Status */
#define RTU1__L_VFCCU_FHFLTRKC00 RTU1__L_VFCCU.FHFLTRKC00.R /* Fault Reaction Set Configuration */
#define RTU1__L_VFCCU_FHFLTRKC01 RTU1__L_VFCCU.FHFLTRKC01.R /* Fault Reaction Set Configuration */
#define RTU1__L_VFCCU_FHFLTRKC02 RTU1__L_VFCCU.FHFLTRKC02.R /* Fault Reaction Set Configuration */
#define RTU1__L_VFCCU_FHFLTRKC03 RTU1__L_VFCCU.FHFLTRKC03.R /* Fault Reaction Set Configuration */
#define RTU1__L_VFCCU_FHFLTRKC04 RTU1__L_VFCCU.FHFLTRKC04.R /* Fault Reaction Set Configuration */
#define RTU1__L_VFCCU_FHFLTRKC05 RTU1__L_VFCCU.FHFLTRKC05.R /* Fault Reaction Set Configuration */
#define RTU1__L_VFCCU_FHFLTRKC06 RTU1__L_VFCCU.FHFLTRKC06.R /* Fault Reaction Set Configuration */
#define RTU1__L_VFCCU_FHIMRKC0_00 RTU1__L_VFCCU.FHIMRKC0_00.R /* Immediate Reaction Configuration */
#define RTU1__L_VFCCU_FHIMRKC0_10 RTU1__L_VFCCU.FHIMRKC0_10.R /* Immediate Reaction Configuration */
#define RTU1__L_VFCCU_FHIMRKC0_20 RTU1__L_VFCCU.FHIMRKC0_20.R /* Immediate Reaction Configuration */
#define RTU1__L_VFCCU_FHIMRKC0_30 RTU1__L_VFCCU.FHIMRKC0_30.R /* Immediate Reaction Configuration */
#define RTU1__L_VFCCU_FHIMRKC0_40 RTU1__L_VFCCU.FHIMRKC0_40.R /* Immediate Reaction Configuration */
#define RTU1__L_VFCCU_FHIMRKC0_50 RTU1__L_VFCCU.FHIMRKC0_50.R /* Immediate Reaction Configuration */

/* RTU_MC_CGM */
#define RTU0__MC_CGM_PCFS_SDUR RTU0__MC_CGM.PCFS_SDUR.R    /* PCFS Step Duration */
#define RTU0__MC_CGM_PCFS_DIVC10 RTU0__MC_CGM.PCFS_DIVC10.R /* PCFS Divider Change 10 Register */
#define RTU0__MC_CGM_PCFS_DIVE10 RTU0__MC_CGM.PCFS_DIVE10.R /* PCFS Divider End 10 Register */
#define RTU0__MC_CGM_PCFS_DIVS10 RTU0__MC_CGM.PCFS_DIVS10.R /* PCFS Divider Start 10 Register */
#define RTU0__MC_CGM_PCFS_DIVC11 RTU0__MC_CGM.PCFS_DIVC11.R /* PCFS Divider Change 11 Register */
#define RTU0__MC_CGM_PCFS_DIVE11 RTU0__MC_CGM.PCFS_DIVE11.R /* PCFS Divider End 11 Register */
#define RTU0__MC_CGM_PCFS_DIVS11 RTU0__MC_CGM.PCFS_DIVS11.R /* PCFS Divider Start 11 Register */
#define RTU0__MC_CGM_PCFS_DIVC12 RTU0__MC_CGM.PCFS_DIVC12.R /* PCFS Divider Change 12 Register */
#define RTU0__MC_CGM_PCFS_DIVE12 RTU0__MC_CGM.PCFS_DIVE12.R /* PCFS Divider End 12 Register */
#define RTU0__MC_CGM_PCFS_DIVS12 RTU0__MC_CGM.PCFS_DIVS12.R /* PCFS Divider Start 12 Register */
#define RTU0__MC_CGM_MUX_0_CSC RTU0__MC_CGM.MUX_0_CSC.R    /* Clock Mux 0 Select Control Register */
#define RTU0__MC_CGM_MUX_0_CSS RTU0__MC_CGM.MUX_0_CSS.R    /* Clock Mux 0 Select Status Register */
#define RTU0__MC_CGM_MUX_0_DC_0 RTU0__MC_CGM.MUX_0_DC_0.R  /* Clock Mux 0 Divider 0 Control Register */
#define RTU0__MC_CGM_MUX_0_DIV_UPD_STAT RTU0__MC_CGM.MUX_0_DIV_UPD_STAT.R /* Clock Mux 0 Divider Update Status Register */
#define RTU0__MC_CGM_MUX_1_CSC RTU0__MC_CGM.MUX_1_CSC.R    /* Clock Mux 1 Select Control Register */
#define RTU0__MC_CGM_MUX_1_CSS RTU0__MC_CGM.MUX_1_CSS.R    /* Clock Mux 1 Select Status Register */
#define RTU0__MC_CGM_MUX_1_DC_0 RTU0__MC_CGM.MUX_1_DC_0.R  /* Clock Mux 1 Divider 0 Control Register */
#define RTU0__MC_CGM_MUX_1_DIV_UPD_STAT RTU0__MC_CGM.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */

#define RTU1__MC_CGM_PCFS_SDUR RTU1__MC_CGM.PCFS_SDUR.R    /* PCFS Step Duration */
#define RTU1__MC_CGM_PCFS_DIVC10 RTU1__MC_CGM.PCFS_DIVC10.R /* PCFS Divider Change 10 Register */
#define RTU1__MC_CGM_PCFS_DIVE10 RTU1__MC_CGM.PCFS_DIVE10.R /* PCFS Divider End 10 Register */
#define RTU1__MC_CGM_PCFS_DIVS10 RTU1__MC_CGM.PCFS_DIVS10.R /* PCFS Divider Start 10 Register */
#define RTU1__MC_CGM_PCFS_DIVC11 RTU1__MC_CGM.PCFS_DIVC11.R /* PCFS Divider Change 11 Register */
#define RTU1__MC_CGM_PCFS_DIVE11 RTU1__MC_CGM.PCFS_DIVE11.R /* PCFS Divider End 11 Register */
#define RTU1__MC_CGM_PCFS_DIVS11 RTU1__MC_CGM.PCFS_DIVS11.R /* PCFS Divider Start 11 Register */
#define RTU1__MC_CGM_PCFS_DIVC12 RTU1__MC_CGM.PCFS_DIVC12.R /* PCFS Divider Change 12 Register */
#define RTU1__MC_CGM_PCFS_DIVE12 RTU1__MC_CGM.PCFS_DIVE12.R /* PCFS Divider End 12 Register */
#define RTU1__MC_CGM_PCFS_DIVS12 RTU1__MC_CGM.PCFS_DIVS12.R /* PCFS Divider Start 12 Register */
#define RTU1__MC_CGM_MUX_0_CSC RTU1__MC_CGM.MUX_0_CSC.R    /* Clock Mux 0 Select Control Register */
#define RTU1__MC_CGM_MUX_0_CSS RTU1__MC_CGM.MUX_0_CSS.R    /* Clock Mux 0 Select Status Register */
#define RTU1__MC_CGM_MUX_0_DC_0 RTU1__MC_CGM.MUX_0_DC_0.R  /* Clock Mux 0 Divider 0 Control Register */
#define RTU1__MC_CGM_MUX_0_DIV_UPD_STAT RTU1__MC_CGM.MUX_0_DIV_UPD_STAT.R /* Clock Mux 0 Divider Update Status Register */
#define RTU1__MC_CGM_MUX_1_CSC RTU1__MC_CGM.MUX_1_CSC.R    /* Clock Mux 1 Select Control Register */
#define RTU1__MC_CGM_MUX_1_CSS RTU1__MC_CGM.MUX_1_CSS.R    /* Clock Mux 1 Select Status Register */
#define RTU1__MC_CGM_MUX_1_DC_0 RTU1__MC_CGM.MUX_1_DC_0.R  /* Clock Mux 1 Divider 0 Control Register */
#define RTU1__MC_CGM_MUX_1_DIV_UPD_STAT RTU1__MC_CGM.MUX_1_DIV_UPD_STAT.R /* Clock Mux 1 Divider Update Status Register */

/* RTU_MRU */
#define RTU0__MRU_0_CH1_CFG0 RTU0__MRU_0.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH1_CFG1 RTU0__MRU_0.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH1_MBSTAT RTU0__MRU_0.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH2_CFG0 RTU0__MRU_0.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH2_CFG1 RTU0__MRU_0.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH2_MBSTAT RTU0__MRU_0.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH3_CFG0 RTU0__MRU_0.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH3_CFG1 RTU0__MRU_0.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH3_MBSTAT RTU0__MRU_0.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH4_CFG0 RTU0__MRU_0.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH4_CFG1 RTU0__MRU_0.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH4_MBSTAT RTU0__MRU_0.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH5_CFG0 RTU0__MRU_0.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH5_CFG1 RTU0__MRU_0.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH5_MBSTAT RTU0__MRU_0.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH6_CFG0 RTU0__MRU_0.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH6_CFG1 RTU0__MRU_0.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH6_MBSTAT RTU0__MRU_0.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH7_CFG0 RTU0__MRU_0.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH7_CFG1 RTU0__MRU_0.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH7_MBSTAT RTU0__MRU_0.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH8_CFG0 RTU0__MRU_0.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH8_CFG1 RTU0__MRU_0.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH8_MBSTAT RTU0__MRU_0.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH9_CFG0 RTU0__MRU_0.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH9_CFG1 RTU0__MRU_0.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH9_MBSTAT RTU0__MRU_0.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH10_CFG0 RTU0__MRU_0.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH10_CFG1 RTU0__MRU_0.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH10_MBSTAT RTU0__MRU_0.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH11_CFG0 RTU0__MRU_0.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH11_CFG1 RTU0__MRU_0.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH11_MBSTAT RTU0__MRU_0.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_CH12_CFG0 RTU0__MRU_0.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_0_CH12_CFG1 RTU0__MRU_0.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_0_CH12_MBSTAT RTU0__MRU_0.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_0_NOTIFY0  RTU0__MRU_0.NOTIFY0.R         /* Notification 0 Status */
#define RTU0__MRU_0_NOTIFY1  RTU0__MRU_0.NOTIFY1.R         /* Notification 1 Status */
#define RTU0__MRU_0_CH1_MB0  RTU0__MRU_0.CH1_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH1_MB1  RTU0__MRU_0.CH1_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH2_MB0  RTU0__MRU_0.CH2_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH2_MB1  RTU0__MRU_0.CH2_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH3_MB0  RTU0__MRU_0.CH3_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH3_MB1  RTU0__MRU_0.CH3_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH4_MB0  RTU0__MRU_0.CH4_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH4_MB1  RTU0__MRU_0.CH4_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH5_MB0  RTU0__MRU_0.CH5_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH5_MB1  RTU0__MRU_0.CH5_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH6_MB0  RTU0__MRU_0.CH6_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH6_MB1  RTU0__MRU_0.CH6_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH7_MB0  RTU0__MRU_0.CH7_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH7_MB1  RTU0__MRU_0.CH7_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH8_MB0  RTU0__MRU_0.CH8_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH8_MB1  RTU0__MRU_0.CH8_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH8_MB2  RTU0__MRU_0.CH8_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH8_MB3  RTU0__MRU_0.CH8_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH8_MB4  RTU0__MRU_0.CH8_MB4.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH8_MB5  RTU0__MRU_0.CH8_MB5.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH8_MB6  RTU0__MRU_0.CH8_MB6.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH8_MB7  RTU0__MRU_0.CH8_MB7.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH9_MB0  RTU0__MRU_0.CH9_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH9_MB1  RTU0__MRU_0.CH9_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH9_MB2  RTU0__MRU_0.CH9_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH9_MB3  RTU0__MRU_0.CH9_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH10_MB0 RTU0__MRU_0.CH10_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH10_MB1 RTU0__MRU_0.CH10_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH10_MB2 RTU0__MRU_0.CH10_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH10_MB3 RTU0__MRU_0.CH10_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH11_MB0 RTU0__MRU_0.CH11_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH11_MB1 RTU0__MRU_0.CH11_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH11_MB2 RTU0__MRU_0.CH11_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH11_MB3 RTU0__MRU_0.CH11_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH12_MB0 RTU0__MRU_0.CH12_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH12_MB1 RTU0__MRU_0.CH12_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH12_MB2 RTU0__MRU_0.CH12_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_0_CH12_MB3 RTU0__MRU_0.CH12_MB3.R        /* Channel (x) Mailbox (n) */

#define RTU1__MRU_0_CH1_CFG0 RTU1__MRU_0.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH1_CFG1 RTU1__MRU_0.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH1_MBSTAT RTU1__MRU_0.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH2_CFG0 RTU1__MRU_0.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH2_CFG1 RTU1__MRU_0.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH2_MBSTAT RTU1__MRU_0.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH3_CFG0 RTU1__MRU_0.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH3_CFG1 RTU1__MRU_0.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH3_MBSTAT RTU1__MRU_0.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH4_CFG0 RTU1__MRU_0.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH4_CFG1 RTU1__MRU_0.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH4_MBSTAT RTU1__MRU_0.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH5_CFG0 RTU1__MRU_0.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH5_CFG1 RTU1__MRU_0.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH5_MBSTAT RTU1__MRU_0.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH6_CFG0 RTU1__MRU_0.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH6_CFG1 RTU1__MRU_0.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH6_MBSTAT RTU1__MRU_0.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH7_CFG0 RTU1__MRU_0.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH7_CFG1 RTU1__MRU_0.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH7_MBSTAT RTU1__MRU_0.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH8_CFG0 RTU1__MRU_0.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH8_CFG1 RTU1__MRU_0.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH8_MBSTAT RTU1__MRU_0.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH9_CFG0 RTU1__MRU_0.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH9_CFG1 RTU1__MRU_0.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH9_MBSTAT RTU1__MRU_0.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH10_CFG0 RTU1__MRU_0.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH10_CFG1 RTU1__MRU_0.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH10_MBSTAT RTU1__MRU_0.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH11_CFG0 RTU1__MRU_0.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH11_CFG1 RTU1__MRU_0.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH11_MBSTAT RTU1__MRU_0.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_CH12_CFG0 RTU1__MRU_0.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_0_CH12_CFG1 RTU1__MRU_0.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_0_CH12_MBSTAT RTU1__MRU_0.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_0_NOTIFY0  RTU1__MRU_0.NOTIFY0.R         /* Notification 0 Status */
#define RTU1__MRU_0_NOTIFY1  RTU1__MRU_0.NOTIFY1.R         /* Notification 1 Status */
#define RTU1__MRU_0_CH1_MB0  RTU1__MRU_0.CH1_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH1_MB1  RTU1__MRU_0.CH1_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH2_MB0  RTU1__MRU_0.CH2_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH2_MB1  RTU1__MRU_0.CH2_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH3_MB0  RTU1__MRU_0.CH3_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH3_MB1  RTU1__MRU_0.CH3_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH4_MB0  RTU1__MRU_0.CH4_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH4_MB1  RTU1__MRU_0.CH4_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH5_MB0  RTU1__MRU_0.CH5_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH5_MB1  RTU1__MRU_0.CH5_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH6_MB0  RTU1__MRU_0.CH6_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH6_MB1  RTU1__MRU_0.CH6_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH7_MB0  RTU1__MRU_0.CH7_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH7_MB1  RTU1__MRU_0.CH7_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH8_MB0  RTU1__MRU_0.CH8_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH8_MB1  RTU1__MRU_0.CH8_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH8_MB2  RTU1__MRU_0.CH8_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH8_MB3  RTU1__MRU_0.CH8_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH8_MB4  RTU1__MRU_0.CH8_MB4.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH8_MB5  RTU1__MRU_0.CH8_MB5.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH8_MB6  RTU1__MRU_0.CH8_MB6.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH8_MB7  RTU1__MRU_0.CH8_MB7.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH9_MB0  RTU1__MRU_0.CH9_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH9_MB1  RTU1__MRU_0.CH9_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH9_MB2  RTU1__MRU_0.CH9_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH9_MB3  RTU1__MRU_0.CH9_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH10_MB0 RTU1__MRU_0.CH10_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH10_MB1 RTU1__MRU_0.CH10_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH10_MB2 RTU1__MRU_0.CH10_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH10_MB3 RTU1__MRU_0.CH10_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH11_MB0 RTU1__MRU_0.CH11_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH11_MB1 RTU1__MRU_0.CH11_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH11_MB2 RTU1__MRU_0.CH11_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH11_MB3 RTU1__MRU_0.CH11_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH12_MB0 RTU1__MRU_0.CH12_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH12_MB1 RTU1__MRU_0.CH12_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH12_MB2 RTU1__MRU_0.CH12_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_0_CH12_MB3 RTU1__MRU_0.CH12_MB3.R        /* Channel (x) Mailbox (n) */

#define RTU0__MRU_1_CH1_CFG0 RTU0__MRU_1.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH1_CFG1 RTU0__MRU_1.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH1_MBSTAT RTU0__MRU_1.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH2_CFG0 RTU0__MRU_1.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH2_CFG1 RTU0__MRU_1.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH2_MBSTAT RTU0__MRU_1.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH3_CFG0 RTU0__MRU_1.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH3_CFG1 RTU0__MRU_1.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH3_MBSTAT RTU0__MRU_1.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH4_CFG0 RTU0__MRU_1.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH4_CFG1 RTU0__MRU_1.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH4_MBSTAT RTU0__MRU_1.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH5_CFG0 RTU0__MRU_1.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH5_CFG1 RTU0__MRU_1.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH5_MBSTAT RTU0__MRU_1.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH6_CFG0 RTU0__MRU_1.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH6_CFG1 RTU0__MRU_1.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH6_MBSTAT RTU0__MRU_1.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH7_CFG0 RTU0__MRU_1.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH7_CFG1 RTU0__MRU_1.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH7_MBSTAT RTU0__MRU_1.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH8_CFG0 RTU0__MRU_1.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH8_CFG1 RTU0__MRU_1.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH8_MBSTAT RTU0__MRU_1.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH9_CFG0 RTU0__MRU_1.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH9_CFG1 RTU0__MRU_1.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH9_MBSTAT RTU0__MRU_1.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH10_CFG0 RTU0__MRU_1.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH10_CFG1 RTU0__MRU_1.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH10_MBSTAT RTU0__MRU_1.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH11_CFG0 RTU0__MRU_1.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH11_CFG1 RTU0__MRU_1.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH11_MBSTAT RTU0__MRU_1.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_CH12_CFG0 RTU0__MRU_1.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_1_CH12_CFG1 RTU0__MRU_1.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_1_CH12_MBSTAT RTU0__MRU_1.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_1_NOTIFY0  RTU0__MRU_1.NOTIFY0.R         /* Notification 0 Status */
#define RTU0__MRU_1_NOTIFY1  RTU0__MRU_1.NOTIFY1.R         /* Notification 1 Status */
#define RTU0__MRU_1_CH1_MB0  RTU0__MRU_1.CH1_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH1_MB1  RTU0__MRU_1.CH1_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH2_MB0  RTU0__MRU_1.CH2_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH2_MB1  RTU0__MRU_1.CH2_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH3_MB0  RTU0__MRU_1.CH3_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH3_MB1  RTU0__MRU_1.CH3_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH4_MB0  RTU0__MRU_1.CH4_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH4_MB1  RTU0__MRU_1.CH4_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH5_MB0  RTU0__MRU_1.CH5_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH5_MB1  RTU0__MRU_1.CH5_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH6_MB0  RTU0__MRU_1.CH6_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH6_MB1  RTU0__MRU_1.CH6_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH7_MB0  RTU0__MRU_1.CH7_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH7_MB1  RTU0__MRU_1.CH7_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH8_MB0  RTU0__MRU_1.CH8_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH8_MB1  RTU0__MRU_1.CH8_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH8_MB2  RTU0__MRU_1.CH8_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH8_MB3  RTU0__MRU_1.CH8_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH8_MB4  RTU0__MRU_1.CH8_MB4.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH8_MB5  RTU0__MRU_1.CH8_MB5.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH8_MB6  RTU0__MRU_1.CH8_MB6.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH8_MB7  RTU0__MRU_1.CH8_MB7.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH9_MB0  RTU0__MRU_1.CH9_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH9_MB1  RTU0__MRU_1.CH9_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH9_MB2  RTU0__MRU_1.CH9_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH9_MB3  RTU0__MRU_1.CH9_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH10_MB0 RTU0__MRU_1.CH10_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH10_MB1 RTU0__MRU_1.CH10_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH10_MB2 RTU0__MRU_1.CH10_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH10_MB3 RTU0__MRU_1.CH10_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH11_MB0 RTU0__MRU_1.CH11_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH11_MB1 RTU0__MRU_1.CH11_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH11_MB2 RTU0__MRU_1.CH11_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH11_MB3 RTU0__MRU_1.CH11_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH12_MB0 RTU0__MRU_1.CH12_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH12_MB1 RTU0__MRU_1.CH12_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH12_MB2 RTU0__MRU_1.CH12_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_1_CH12_MB3 RTU0__MRU_1.CH12_MB3.R        /* Channel (x) Mailbox (n) */

#define RTU1__MRU_1_CH1_CFG0 RTU1__MRU_1.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH1_CFG1 RTU1__MRU_1.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH1_MBSTAT RTU1__MRU_1.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH2_CFG0 RTU1__MRU_1.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH2_CFG1 RTU1__MRU_1.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH2_MBSTAT RTU1__MRU_1.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH3_CFG0 RTU1__MRU_1.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH3_CFG1 RTU1__MRU_1.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH3_MBSTAT RTU1__MRU_1.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH4_CFG0 RTU1__MRU_1.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH4_CFG1 RTU1__MRU_1.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH4_MBSTAT RTU1__MRU_1.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH5_CFG0 RTU1__MRU_1.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH5_CFG1 RTU1__MRU_1.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH5_MBSTAT RTU1__MRU_1.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH6_CFG0 RTU1__MRU_1.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH6_CFG1 RTU1__MRU_1.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH6_MBSTAT RTU1__MRU_1.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH7_CFG0 RTU1__MRU_1.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH7_CFG1 RTU1__MRU_1.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH7_MBSTAT RTU1__MRU_1.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH8_CFG0 RTU1__MRU_1.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH8_CFG1 RTU1__MRU_1.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH8_MBSTAT RTU1__MRU_1.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH9_CFG0 RTU1__MRU_1.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH9_CFG1 RTU1__MRU_1.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH9_MBSTAT RTU1__MRU_1.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH10_CFG0 RTU1__MRU_1.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH10_CFG1 RTU1__MRU_1.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH10_MBSTAT RTU1__MRU_1.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH11_CFG0 RTU1__MRU_1.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH11_CFG1 RTU1__MRU_1.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH11_MBSTAT RTU1__MRU_1.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_CH12_CFG0 RTU1__MRU_1.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_1_CH12_CFG1 RTU1__MRU_1.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_1_CH12_MBSTAT RTU1__MRU_1.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_1_NOTIFY0  RTU1__MRU_1.NOTIFY0.R         /* Notification 0 Status */
#define RTU1__MRU_1_NOTIFY1  RTU1__MRU_1.NOTIFY1.R         /* Notification 1 Status */
#define RTU1__MRU_1_CH1_MB0  RTU1__MRU_1.CH1_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH1_MB1  RTU1__MRU_1.CH1_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH2_MB0  RTU1__MRU_1.CH2_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH2_MB1  RTU1__MRU_1.CH2_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH3_MB0  RTU1__MRU_1.CH3_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH3_MB1  RTU1__MRU_1.CH3_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH4_MB0  RTU1__MRU_1.CH4_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH4_MB1  RTU1__MRU_1.CH4_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH5_MB0  RTU1__MRU_1.CH5_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH5_MB1  RTU1__MRU_1.CH5_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH6_MB0  RTU1__MRU_1.CH6_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH6_MB1  RTU1__MRU_1.CH6_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH7_MB0  RTU1__MRU_1.CH7_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH7_MB1  RTU1__MRU_1.CH7_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH8_MB0  RTU1__MRU_1.CH8_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH8_MB1  RTU1__MRU_1.CH8_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH8_MB2  RTU1__MRU_1.CH8_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH8_MB3  RTU1__MRU_1.CH8_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH8_MB4  RTU1__MRU_1.CH8_MB4.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH8_MB5  RTU1__MRU_1.CH8_MB5.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH8_MB6  RTU1__MRU_1.CH8_MB6.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH8_MB7  RTU1__MRU_1.CH8_MB7.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH9_MB0  RTU1__MRU_1.CH9_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH9_MB1  RTU1__MRU_1.CH9_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH9_MB2  RTU1__MRU_1.CH9_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH9_MB3  RTU1__MRU_1.CH9_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH10_MB0 RTU1__MRU_1.CH10_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH10_MB1 RTU1__MRU_1.CH10_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH10_MB2 RTU1__MRU_1.CH10_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH10_MB3 RTU1__MRU_1.CH10_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH11_MB0 RTU1__MRU_1.CH11_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH11_MB1 RTU1__MRU_1.CH11_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH11_MB2 RTU1__MRU_1.CH11_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH11_MB3 RTU1__MRU_1.CH11_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH12_MB0 RTU1__MRU_1.CH12_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH12_MB1 RTU1__MRU_1.CH12_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH12_MB2 RTU1__MRU_1.CH12_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_1_CH12_MB3 RTU1__MRU_1.CH12_MB3.R        /* Channel (x) Mailbox (n) */

#define RTU0__MRU_2_CH1_CFG0 RTU0__MRU_2.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH1_CFG1 RTU0__MRU_2.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH1_MBSTAT RTU0__MRU_2.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH2_CFG0 RTU0__MRU_2.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH2_CFG1 RTU0__MRU_2.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH2_MBSTAT RTU0__MRU_2.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH3_CFG0 RTU0__MRU_2.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH3_CFG1 RTU0__MRU_2.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH3_MBSTAT RTU0__MRU_2.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH4_CFG0 RTU0__MRU_2.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH4_CFG1 RTU0__MRU_2.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH4_MBSTAT RTU0__MRU_2.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH5_CFG0 RTU0__MRU_2.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH5_CFG1 RTU0__MRU_2.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH5_MBSTAT RTU0__MRU_2.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH6_CFG0 RTU0__MRU_2.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH6_CFG1 RTU0__MRU_2.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH6_MBSTAT RTU0__MRU_2.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH7_CFG0 RTU0__MRU_2.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH7_CFG1 RTU0__MRU_2.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH7_MBSTAT RTU0__MRU_2.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH8_CFG0 RTU0__MRU_2.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH8_CFG1 RTU0__MRU_2.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH8_MBSTAT RTU0__MRU_2.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH9_CFG0 RTU0__MRU_2.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH9_CFG1 RTU0__MRU_2.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH9_MBSTAT RTU0__MRU_2.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH10_CFG0 RTU0__MRU_2.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH10_CFG1 RTU0__MRU_2.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH10_MBSTAT RTU0__MRU_2.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH11_CFG0 RTU0__MRU_2.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH11_CFG1 RTU0__MRU_2.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH11_MBSTAT RTU0__MRU_2.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_CH12_CFG0 RTU0__MRU_2.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_2_CH12_CFG1 RTU0__MRU_2.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_2_CH12_MBSTAT RTU0__MRU_2.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_2_NOTIFY0  RTU0__MRU_2.NOTIFY0.R         /* Notification 0 Status */
#define RTU0__MRU_2_NOTIFY1  RTU0__MRU_2.NOTIFY1.R         /* Notification 1 Status */
#define RTU0__MRU_2_CH1_MB0  RTU0__MRU_2.CH1_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH1_MB1  RTU0__MRU_2.CH1_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH2_MB0  RTU0__MRU_2.CH2_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH2_MB1  RTU0__MRU_2.CH2_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH3_MB0  RTU0__MRU_2.CH3_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH3_MB1  RTU0__MRU_2.CH3_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH4_MB0  RTU0__MRU_2.CH4_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH4_MB1  RTU0__MRU_2.CH4_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH5_MB0  RTU0__MRU_2.CH5_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH5_MB1  RTU0__MRU_2.CH5_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH6_MB0  RTU0__MRU_2.CH6_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH6_MB1  RTU0__MRU_2.CH6_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH7_MB0  RTU0__MRU_2.CH7_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH7_MB1  RTU0__MRU_2.CH7_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH8_MB0  RTU0__MRU_2.CH8_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH8_MB1  RTU0__MRU_2.CH8_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH8_MB2  RTU0__MRU_2.CH8_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH8_MB3  RTU0__MRU_2.CH8_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH8_MB4  RTU0__MRU_2.CH8_MB4.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH8_MB5  RTU0__MRU_2.CH8_MB5.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH8_MB6  RTU0__MRU_2.CH8_MB6.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH8_MB7  RTU0__MRU_2.CH8_MB7.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH9_MB0  RTU0__MRU_2.CH9_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH9_MB1  RTU0__MRU_2.CH9_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH9_MB2  RTU0__MRU_2.CH9_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH9_MB3  RTU0__MRU_2.CH9_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH10_MB0 RTU0__MRU_2.CH10_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH10_MB1 RTU0__MRU_2.CH10_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH10_MB2 RTU0__MRU_2.CH10_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH10_MB3 RTU0__MRU_2.CH10_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH11_MB0 RTU0__MRU_2.CH11_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH11_MB1 RTU0__MRU_2.CH11_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH11_MB2 RTU0__MRU_2.CH11_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH11_MB3 RTU0__MRU_2.CH11_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH12_MB0 RTU0__MRU_2.CH12_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH12_MB1 RTU0__MRU_2.CH12_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH12_MB2 RTU0__MRU_2.CH12_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_2_CH12_MB3 RTU0__MRU_2.CH12_MB3.R        /* Channel (x) Mailbox (n) */

#define RTU1__MRU_2_CH1_CFG0 RTU1__MRU_2.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH1_CFG1 RTU1__MRU_2.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH1_MBSTAT RTU1__MRU_2.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH2_CFG0 RTU1__MRU_2.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH2_CFG1 RTU1__MRU_2.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH2_MBSTAT RTU1__MRU_2.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH3_CFG0 RTU1__MRU_2.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH3_CFG1 RTU1__MRU_2.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH3_MBSTAT RTU1__MRU_2.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH4_CFG0 RTU1__MRU_2.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH4_CFG1 RTU1__MRU_2.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH4_MBSTAT RTU1__MRU_2.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH5_CFG0 RTU1__MRU_2.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH5_CFG1 RTU1__MRU_2.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH5_MBSTAT RTU1__MRU_2.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH6_CFG0 RTU1__MRU_2.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH6_CFG1 RTU1__MRU_2.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH6_MBSTAT RTU1__MRU_2.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH7_CFG0 RTU1__MRU_2.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH7_CFG1 RTU1__MRU_2.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH7_MBSTAT RTU1__MRU_2.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH8_CFG0 RTU1__MRU_2.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH8_CFG1 RTU1__MRU_2.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH8_MBSTAT RTU1__MRU_2.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH9_CFG0 RTU1__MRU_2.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH9_CFG1 RTU1__MRU_2.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH9_MBSTAT RTU1__MRU_2.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH10_CFG0 RTU1__MRU_2.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH10_CFG1 RTU1__MRU_2.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH10_MBSTAT RTU1__MRU_2.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH11_CFG0 RTU1__MRU_2.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH11_CFG1 RTU1__MRU_2.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH11_MBSTAT RTU1__MRU_2.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_CH12_CFG0 RTU1__MRU_2.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_2_CH12_CFG1 RTU1__MRU_2.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_2_CH12_MBSTAT RTU1__MRU_2.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_2_NOTIFY0  RTU1__MRU_2.NOTIFY0.R         /* Notification 0 Status */
#define RTU1__MRU_2_NOTIFY1  RTU1__MRU_2.NOTIFY1.R         /* Notification 1 Status */
#define RTU1__MRU_2_CH1_MB0  RTU1__MRU_2.CH1_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH1_MB1  RTU1__MRU_2.CH1_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH2_MB0  RTU1__MRU_2.CH2_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH2_MB1  RTU1__MRU_2.CH2_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH3_MB0  RTU1__MRU_2.CH3_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH3_MB1  RTU1__MRU_2.CH3_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH4_MB0  RTU1__MRU_2.CH4_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH4_MB1  RTU1__MRU_2.CH4_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH5_MB0  RTU1__MRU_2.CH5_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH5_MB1  RTU1__MRU_2.CH5_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH6_MB0  RTU1__MRU_2.CH6_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH6_MB1  RTU1__MRU_2.CH6_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH7_MB0  RTU1__MRU_2.CH7_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH7_MB1  RTU1__MRU_2.CH7_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH8_MB0  RTU1__MRU_2.CH8_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH8_MB1  RTU1__MRU_2.CH8_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH8_MB2  RTU1__MRU_2.CH8_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH8_MB3  RTU1__MRU_2.CH8_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH8_MB4  RTU1__MRU_2.CH8_MB4.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH8_MB5  RTU1__MRU_2.CH8_MB5.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH8_MB6  RTU1__MRU_2.CH8_MB6.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH8_MB7  RTU1__MRU_2.CH8_MB7.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH9_MB0  RTU1__MRU_2.CH9_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH9_MB1  RTU1__MRU_2.CH9_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH9_MB2  RTU1__MRU_2.CH9_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH9_MB3  RTU1__MRU_2.CH9_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH10_MB0 RTU1__MRU_2.CH10_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH10_MB1 RTU1__MRU_2.CH10_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH10_MB2 RTU1__MRU_2.CH10_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH10_MB3 RTU1__MRU_2.CH10_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH11_MB0 RTU1__MRU_2.CH11_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH11_MB1 RTU1__MRU_2.CH11_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH11_MB2 RTU1__MRU_2.CH11_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH11_MB3 RTU1__MRU_2.CH11_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH12_MB0 RTU1__MRU_2.CH12_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH12_MB1 RTU1__MRU_2.CH12_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH12_MB2 RTU1__MRU_2.CH12_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_2_CH12_MB3 RTU1__MRU_2.CH12_MB3.R        /* Channel (x) Mailbox (n) */

#define RTU0__MRU_3_CH1_CFG0 RTU0__MRU_3.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH1_CFG1 RTU0__MRU_3.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH1_MBSTAT RTU0__MRU_3.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH2_CFG0 RTU0__MRU_3.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH2_CFG1 RTU0__MRU_3.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH2_MBSTAT RTU0__MRU_3.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH3_CFG0 RTU0__MRU_3.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH3_CFG1 RTU0__MRU_3.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH3_MBSTAT RTU0__MRU_3.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH4_CFG0 RTU0__MRU_3.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH4_CFG1 RTU0__MRU_3.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH4_MBSTAT RTU0__MRU_3.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH5_CFG0 RTU0__MRU_3.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH5_CFG1 RTU0__MRU_3.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH5_MBSTAT RTU0__MRU_3.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH6_CFG0 RTU0__MRU_3.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH6_CFG1 RTU0__MRU_3.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH6_MBSTAT RTU0__MRU_3.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH7_CFG0 RTU0__MRU_3.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH7_CFG1 RTU0__MRU_3.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH7_MBSTAT RTU0__MRU_3.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH8_CFG0 RTU0__MRU_3.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH8_CFG1 RTU0__MRU_3.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH8_MBSTAT RTU0__MRU_3.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH9_CFG0 RTU0__MRU_3.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH9_CFG1 RTU0__MRU_3.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH9_MBSTAT RTU0__MRU_3.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH10_CFG0 RTU0__MRU_3.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH10_CFG1 RTU0__MRU_3.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH10_MBSTAT RTU0__MRU_3.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH11_CFG0 RTU0__MRU_3.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH11_CFG1 RTU0__MRU_3.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH11_MBSTAT RTU0__MRU_3.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_CH12_CFG0 RTU0__MRU_3.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU0__MRU_3_CH12_CFG1 RTU0__MRU_3.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU0__MRU_3_CH12_MBSTAT RTU0__MRU_3.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU0__MRU_3_NOTIFY0  RTU0__MRU_3.NOTIFY0.R         /* Notification 0 Status */
#define RTU0__MRU_3_NOTIFY1  RTU0__MRU_3.NOTIFY1.R         /* Notification 1 Status */
#define RTU0__MRU_3_CH1_MB0  RTU0__MRU_3.CH1_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH1_MB1  RTU0__MRU_3.CH1_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH2_MB0  RTU0__MRU_3.CH2_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH2_MB1  RTU0__MRU_3.CH2_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH3_MB0  RTU0__MRU_3.CH3_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH3_MB1  RTU0__MRU_3.CH3_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH4_MB0  RTU0__MRU_3.CH4_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH4_MB1  RTU0__MRU_3.CH4_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH5_MB0  RTU0__MRU_3.CH5_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH5_MB1  RTU0__MRU_3.CH5_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH6_MB0  RTU0__MRU_3.CH6_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH6_MB1  RTU0__MRU_3.CH6_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH7_MB0  RTU0__MRU_3.CH7_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH7_MB1  RTU0__MRU_3.CH7_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH8_MB0  RTU0__MRU_3.CH8_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH8_MB1  RTU0__MRU_3.CH8_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH8_MB2  RTU0__MRU_3.CH8_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH8_MB3  RTU0__MRU_3.CH8_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH8_MB4  RTU0__MRU_3.CH8_MB4.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH8_MB5  RTU0__MRU_3.CH8_MB5.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH8_MB6  RTU0__MRU_3.CH8_MB6.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH8_MB7  RTU0__MRU_3.CH8_MB7.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH9_MB0  RTU0__MRU_3.CH9_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH9_MB1  RTU0__MRU_3.CH9_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH9_MB2  RTU0__MRU_3.CH9_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH9_MB3  RTU0__MRU_3.CH9_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH10_MB0 RTU0__MRU_3.CH10_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH10_MB1 RTU0__MRU_3.CH10_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH10_MB2 RTU0__MRU_3.CH10_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH10_MB3 RTU0__MRU_3.CH10_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH11_MB0 RTU0__MRU_3.CH11_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH11_MB1 RTU0__MRU_3.CH11_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH11_MB2 RTU0__MRU_3.CH11_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH11_MB3 RTU0__MRU_3.CH11_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH12_MB0 RTU0__MRU_3.CH12_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH12_MB1 RTU0__MRU_3.CH12_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH12_MB2 RTU0__MRU_3.CH12_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU0__MRU_3_CH12_MB3 RTU0__MRU_3.CH12_MB3.R        /* Channel (x) Mailbox (n) */

#define RTU1__MRU_3_CH1_CFG0 RTU1__MRU_3.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH1_CFG1 RTU1__MRU_3.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH1_MBSTAT RTU1__MRU_3.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH2_CFG0 RTU1__MRU_3.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH2_CFG1 RTU1__MRU_3.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH2_MBSTAT RTU1__MRU_3.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH3_CFG0 RTU1__MRU_3.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH3_CFG1 RTU1__MRU_3.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH3_MBSTAT RTU1__MRU_3.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH4_CFG0 RTU1__MRU_3.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH4_CFG1 RTU1__MRU_3.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH4_MBSTAT RTU1__MRU_3.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH5_CFG0 RTU1__MRU_3.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH5_CFG1 RTU1__MRU_3.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH5_MBSTAT RTU1__MRU_3.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH6_CFG0 RTU1__MRU_3.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH6_CFG1 RTU1__MRU_3.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH6_MBSTAT RTU1__MRU_3.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH7_CFG0 RTU1__MRU_3.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH7_CFG1 RTU1__MRU_3.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH7_MBSTAT RTU1__MRU_3.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH8_CFG0 RTU1__MRU_3.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH8_CFG1 RTU1__MRU_3.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH8_MBSTAT RTU1__MRU_3.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH9_CFG0 RTU1__MRU_3.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH9_CFG1 RTU1__MRU_3.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH9_MBSTAT RTU1__MRU_3.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH10_CFG0 RTU1__MRU_3.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH10_CFG1 RTU1__MRU_3.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH10_MBSTAT RTU1__MRU_3.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH11_CFG0 RTU1__MRU_3.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH11_CFG1 RTU1__MRU_3.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH11_MBSTAT RTU1__MRU_3.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_CH12_CFG0 RTU1__MRU_3.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define RTU1__MRU_3_CH12_CFG1 RTU1__MRU_3.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define RTU1__MRU_3_CH12_MBSTAT RTU1__MRU_3.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define RTU1__MRU_3_NOTIFY0  RTU1__MRU_3.NOTIFY0.R         /* Notification 0 Status */
#define RTU1__MRU_3_NOTIFY1  RTU1__MRU_3.NOTIFY1.R         /* Notification 1 Status */
#define RTU1__MRU_3_CH1_MB0  RTU1__MRU_3.CH1_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH1_MB1  RTU1__MRU_3.CH1_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH2_MB0  RTU1__MRU_3.CH2_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH2_MB1  RTU1__MRU_3.CH2_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH3_MB0  RTU1__MRU_3.CH3_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH3_MB1  RTU1__MRU_3.CH3_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH4_MB0  RTU1__MRU_3.CH4_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH4_MB1  RTU1__MRU_3.CH4_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH5_MB0  RTU1__MRU_3.CH5_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH5_MB1  RTU1__MRU_3.CH5_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH6_MB0  RTU1__MRU_3.CH6_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH6_MB1  RTU1__MRU_3.CH6_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH7_MB0  RTU1__MRU_3.CH7_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH7_MB1  RTU1__MRU_3.CH7_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH8_MB0  RTU1__MRU_3.CH8_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH8_MB1  RTU1__MRU_3.CH8_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH8_MB2  RTU1__MRU_3.CH8_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH8_MB3  RTU1__MRU_3.CH8_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH8_MB4  RTU1__MRU_3.CH8_MB4.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH8_MB5  RTU1__MRU_3.CH8_MB5.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH8_MB6  RTU1__MRU_3.CH8_MB6.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH8_MB7  RTU1__MRU_3.CH8_MB7.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH9_MB0  RTU1__MRU_3.CH9_MB0.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH9_MB1  RTU1__MRU_3.CH9_MB1.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH9_MB2  RTU1__MRU_3.CH9_MB2.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH9_MB3  RTU1__MRU_3.CH9_MB3.R         /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH10_MB0 RTU1__MRU_3.CH10_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH10_MB1 RTU1__MRU_3.CH10_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH10_MB2 RTU1__MRU_3.CH10_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH10_MB3 RTU1__MRU_3.CH10_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH11_MB0 RTU1__MRU_3.CH11_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH11_MB1 RTU1__MRU_3.CH11_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH11_MB2 RTU1__MRU_3.CH11_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH11_MB3 RTU1__MRU_3.CH11_MB3.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH12_MB0 RTU1__MRU_3.CH12_MB0.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH12_MB1 RTU1__MRU_3.CH12_MB1.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH12_MB2 RTU1__MRU_3.CH12_MB2.R        /* Channel (x) Mailbox (n) */
#define RTU1__MRU_3_CH12_MB3 RTU1__MRU_3.CH12_MB3.R        /* Channel (x) Mailbox (n) */

/* RTU_PMC */
#define RTU0__PMC_CTRL       RTU0__PMC.CTRL.R              /* Main Control Register */
#define RTU0__PMC_MCR        RTU0__PMC.MCR.R               /* Memory Control Register */
#define RTU0__PMC_BER        RTU0__PMC.BER.R               /* Byte Enable Register */
#define RTU0__PMC_PCR        RTU0__PMC.PCR.R               /* Program Control Register */
#define RTU0__PMC_RPR        RTU0__PMC.RPR.R               /* Read Pipeline Register */
#define RTU0__PMC_STOPADDR   RTU0__PMC.STOPADDR.R          /* Stop Address Register */
#define RTU0__PMC_CADDR      RTU0__PMC.CADDR.R             /* Column Address Register */
#define RTU0__PMC_RADDR      RTU0__PMC.RADDR.R             /* Row Address Register */
#define RTU0__PMC_X0         RTU0__PMC.X0.R                /* X Data Register 0 */
#define RTU0__PMC_X1         RTU0__PMC.X1.R                /* X Data Register 1 */
#define RTU0__PMC_X2         RTU0__PMC.X2.R                /* X Data Register 2 */
#define RTU0__PMC_X3         RTU0__PMC.X3.R                /* X Data Register 3 */
#define RTU0__PMC_X4         RTU0__PMC.X4.R                /* X Data Register 4 */
#define RTU0__PMC_X5         RTU0__PMC.X5.R                /* X Data Register 5 */
#define RTU0__PMC_X6         RTU0__PMC.X6.R                /* X Data Register 6 */
#define RTU0__PMC_X7         RTU0__PMC.X7.R                /* X Data Register 7 */
#define RTU0__PMC_Y0         RTU0__PMC.Y0.R                /* Y Data Register 0 */
#define RTU0__PMC_Y1         RTU0__PMC.Y1.R                /* Y Data Register 1 */
#define RTU0__PMC_Y2         RTU0__PMC.Y2.R                /* Y Data Register 2 */
#define RTU0__PMC_Y3         RTU0__PMC.Y3.R                /* Y Data Register 3 */
#define RTU0__PMC_Y4         RTU0__PMC.Y4.R                /* Y Data Register 4 */
#define RTU0__PMC_Y5         RTU0__PMC.Y5.R                /* Y Data Register 5 */
#define RTU0__PMC_Y6         RTU0__PMC.Y6.R                /* Y Data Register 6 */
#define RTU0__PMC_Y7         RTU0__PMC.Y7.R                /* Y Data Register 7 */
#define RTU0__PMC_AIR        RTU0__PMC.AIR.R               /* Auxiliary Input Register */
#define RTU0__PMC_AOR        RTU0__PMC.AOR.R               /* Auxiliary Output Register */
#define RTU0__PMC_DM0        RTU0__PMC.DM0.R               /* Data Mask Register 0 */
#define RTU0__PMC_DM1        RTU0__PMC.DM1.R               /* Data Mask Register 1 */
#define RTU0__PMC_DM2        RTU0__PMC.DM2.R               /* Data Mask Register 2 */
#define RTU0__PMC_DM3        RTU0__PMC.DM3.R               /* Data Mask Register 3 */
#define RTU0__PMC_DM4        RTU0__PMC.DM4.R               /* Data Mask Register 4 */
#define RTU0__PMC_DM5        RTU0__PMC.DM5.R               /* Data Mask Register 5 */
#define RTU0__PMC_DM6        RTU0__PMC.DM6.R               /* Data Mask Register 6 */
#define RTU0__PMC_DM7        RTU0__PMC.DM7.R               /* Data Mask Register 7 */
#define RTU0__PMC_P0         RTU0__PMC.P0.R                /* Program Register 0 */
#define RTU0__PMC_P1         RTU0__PMC.P1.R                /* Program Register 1 */
#define RTU0__PMC_P2         RTU0__PMC.P2.R                /* Program Register 2 */
#define RTU0__PMC_P3         RTU0__PMC.P3.R                /* Program Register 3 */
#define RTU0__PMC_P4         RTU0__PMC.P4.R                /* Program Register 4 */
#define RTU0__PMC_P5         RTU0__PMC.P5.R                /* Program Register 5 */
#define RTU0__PMC_P6         RTU0__PMC.P6.R                /* Program Register 6 */
#define RTU0__PMC_P7         RTU0__PMC.P7.R                /* Program Register 7 */
#define RTU0__PMC_P8         RTU0__PMC.P8.R                /* Program Register 8 */
#define RTU0__PMC_P9         RTU0__PMC.P9.R                /* Program Register 9 */
#define RTU0__PMC_P10        RTU0__PMC.P10.R               /* Program Register 10 */
#define RTU0__PMC_P11        RTU0__PMC.P11.R               /* Program Register 11 */
#define RTU0__PMC_P12        RTU0__PMC.P12.R               /* Program Register 12 */
#define RTU0__PMC_P13        RTU0__PMC.P13.R               /* Program Register 13 */
#define RTU0__PMC_P14        RTU0__PMC.P14.R               /* Program Register 14 */
#define RTU0__PMC_P15        RTU0__PMC.P15.R               /* Program Register 15 */
#define RTU0__PMC_P16        RTU0__PMC.P16.R               /* Program Register 16 */
#define RTU0__PMC_P17        RTU0__PMC.P17.R               /* Program Register 17 */
#define RTU0__PMC_P18        RTU0__PMC.P18.R               /* Program Register 18 */
#define RTU0__PMC_P19        RTU0__PMC.P19.R               /* Program Register 19 */
#define RTU0__PMC_P20        RTU0__PMC.P20.R               /* Program Register 20 */
#define RTU0__PMC_P21        RTU0__PMC.P21.R               /* Program Register 21 */
#define RTU0__PMC_P22        RTU0__PMC.P22.R               /* Program Register 22 */
#define RTU0__PMC_P23        RTU0__PMC.P23.R               /* Program Register 23 */
#define RTU0__PMC_P24        RTU0__PMC.P24.R               /* Program Register 24 */
#define RTU0__PMC_P25        RTU0__PMC.P25.R               /* Program Register 25 */
#define RTU0__PMC_P26        RTU0__PMC.P26.R               /* Program Register 26 */
#define RTU0__PMC_P27        RTU0__PMC.P27.R               /* Program Register 27 */
#define RTU0__PMC_P28        RTU0__PMC.P28.R               /* Program Register 28 */
#define RTU0__PMC_P29        RTU0__PMC.P29.R               /* Program Register 29 */
#define RTU0__PMC_P30        RTU0__PMC.P30.R               /* Program Register 30 */
#define RTU0__PMC_P31        RTU0__PMC.P31.R               /* Program Register 31 */
#define RTU0__PMC_ITCTRL     RTU0__PMC.ITCTRL.R            /* Integration Mode Control Register */
#define RTU0__PMC_CLAIMSET   RTU0__PMC.CLAIMSET.R          /* Claim Tag Set Register */
#define RTU0__PMC_CLAIMCLR   RTU0__PMC.CLAIMCLR.R          /* Claim Tag Clear Register */
#define RTU0__PMC_DEVAFF0    RTU0__PMC.DEVAFF0.R           /* Device Affinity 0 Register */
#define RTU0__PMC_DEVAFF1    RTU0__PMC.DEVAFF1.R           /* Device Affinity 1 Register */
#define RTU0__PMC_LAR        RTU0__PMC.LAR.R               /* Lock Access Register */
#define RTU0__PMC_LSR        RTU0__PMC.LSR.R               /* Lock Status Register */
#define RTU0__PMC_AUTHSTATUS RTU0__PMC.AUTHSTATUS.R        /* Authentication Status Register */
#define RTU0__PMC_DEVARCH    RTU0__PMC.DEVARCH.R           /* Device Architecture Register */
#define RTU0__PMC_DEVID2     RTU0__PMC.DEVID2.R            /* Device ID Register 2 */
#define RTU0__PMC_DEVID1     RTU0__PMC.DEVID1.R            /* Device ID Register 1 */
#define RTU0__PMC_DEVID      RTU0__PMC.DEVID.R             /* Device ID Register */
#define RTU0__PMC_DEVTYPE    RTU0__PMC.DEVTYPE.R           /* Device Type Identifier Register */
#define RTU0__PMC_PIDR4      RTU0__PMC.PIDR4.R             /* Peripheral Identification Register 4 */
#define RTU0__PMC_PIDR5      RTU0__PMC.PIDR5.R             /* Peripheral Identification Register 5 */
#define RTU0__PMC_PIDR6      RTU0__PMC.PIDR6.R             /* Peripheral Identification Register 6 */
#define RTU0__PMC_PIDR7      RTU0__PMC.PIDR7.R             /* Peripheral Identification Register 7 */
#define RTU0__PMC_PIDR0      RTU0__PMC.PIDR0.R             /* Peripheral Identification Register 0 */
#define RTU0__PMC_PIDR1      RTU0__PMC.PIDR1.R             /* Peripheral Identification Register 1 */
#define RTU0__PMC_PIDR2      RTU0__PMC.PIDR2.R             /* Peripheral Identification Register 2 */
#define RTU0__PMC_PIDR3      RTU0__PMC.PIDR3.R             /* Peripheral Identification Register 3 */
#define RTU0__PMC_CIDR0      RTU0__PMC.CIDR0.R             /* Component Identification Register 0 */
#define RTU0__PMC_CIDR1      RTU0__PMC.CIDR1.R             /* Component Identification Register 1 */
#define RTU0__PMC_CIDR2      RTU0__PMC.CIDR2.R             /* Component Identification Register 2 */
#define RTU0__PMC_CIDR3      RTU0__PMC.CIDR3.R             /* Component Identification Register 3 */

#define RTU1__PMC_CTRL       RTU1__PMC.CTRL.R              /* Main Control Register */
#define RTU1__PMC_MCR        RTU1__PMC.MCR.R               /* Memory Control Register */
#define RTU1__PMC_BER        RTU1__PMC.BER.R               /* Byte Enable Register */
#define RTU1__PMC_PCR        RTU1__PMC.PCR.R               /* Program Control Register */
#define RTU1__PMC_RPR        RTU1__PMC.RPR.R               /* Read Pipeline Register */
#define RTU1__PMC_STOPADDR   RTU1__PMC.STOPADDR.R          /* Stop Address Register */
#define RTU1__PMC_CADDR      RTU1__PMC.CADDR.R             /* Column Address Register */
#define RTU1__PMC_RADDR      RTU1__PMC.RADDR.R             /* Row Address Register */
#define RTU1__PMC_X0         RTU1__PMC.X0.R                /* X Data Register 0 */
#define RTU1__PMC_X1         RTU1__PMC.X1.R                /* X Data Register 1 */
#define RTU1__PMC_X2         RTU1__PMC.X2.R                /* X Data Register 2 */
#define RTU1__PMC_X3         RTU1__PMC.X3.R                /* X Data Register 3 */
#define RTU1__PMC_X4         RTU1__PMC.X4.R                /* X Data Register 4 */
#define RTU1__PMC_X5         RTU1__PMC.X5.R                /* X Data Register 5 */
#define RTU1__PMC_X6         RTU1__PMC.X6.R                /* X Data Register 6 */
#define RTU1__PMC_X7         RTU1__PMC.X7.R                /* X Data Register 7 */
#define RTU1__PMC_Y0         RTU1__PMC.Y0.R                /* Y Data Register 0 */
#define RTU1__PMC_Y1         RTU1__PMC.Y1.R                /* Y Data Register 1 */
#define RTU1__PMC_Y2         RTU1__PMC.Y2.R                /* Y Data Register 2 */
#define RTU1__PMC_Y3         RTU1__PMC.Y3.R                /* Y Data Register 3 */
#define RTU1__PMC_Y4         RTU1__PMC.Y4.R                /* Y Data Register 4 */
#define RTU1__PMC_Y5         RTU1__PMC.Y5.R                /* Y Data Register 5 */
#define RTU1__PMC_Y6         RTU1__PMC.Y6.R                /* Y Data Register 6 */
#define RTU1__PMC_Y7         RTU1__PMC.Y7.R                /* Y Data Register 7 */
#define RTU1__PMC_AIR        RTU1__PMC.AIR.R               /* Auxiliary Input Register */
#define RTU1__PMC_AOR        RTU1__PMC.AOR.R               /* Auxiliary Output Register */
#define RTU1__PMC_DM0        RTU1__PMC.DM0.R               /* Data Mask Register 0 */
#define RTU1__PMC_DM1        RTU1__PMC.DM1.R               /* Data Mask Register 1 */
#define RTU1__PMC_DM2        RTU1__PMC.DM2.R               /* Data Mask Register 2 */
#define RTU1__PMC_DM3        RTU1__PMC.DM3.R               /* Data Mask Register 3 */
#define RTU1__PMC_DM4        RTU1__PMC.DM4.R               /* Data Mask Register 4 */
#define RTU1__PMC_DM5        RTU1__PMC.DM5.R               /* Data Mask Register 5 */
#define RTU1__PMC_DM6        RTU1__PMC.DM6.R               /* Data Mask Register 6 */
#define RTU1__PMC_DM7        RTU1__PMC.DM7.R               /* Data Mask Register 7 */
#define RTU1__PMC_P0         RTU1__PMC.P0.R                /* Program Register 0 */
#define RTU1__PMC_P1         RTU1__PMC.P1.R                /* Program Register 1 */
#define RTU1__PMC_P2         RTU1__PMC.P2.R                /* Program Register 2 */
#define RTU1__PMC_P3         RTU1__PMC.P3.R                /* Program Register 3 */
#define RTU1__PMC_P4         RTU1__PMC.P4.R                /* Program Register 4 */
#define RTU1__PMC_P5         RTU1__PMC.P5.R                /* Program Register 5 */
#define RTU1__PMC_P6         RTU1__PMC.P6.R                /* Program Register 6 */
#define RTU1__PMC_P7         RTU1__PMC.P7.R                /* Program Register 7 */
#define RTU1__PMC_P8         RTU1__PMC.P8.R                /* Program Register 8 */
#define RTU1__PMC_P9         RTU1__PMC.P9.R                /* Program Register 9 */
#define RTU1__PMC_P10        RTU1__PMC.P10.R               /* Program Register 10 */
#define RTU1__PMC_P11        RTU1__PMC.P11.R               /* Program Register 11 */
#define RTU1__PMC_P12        RTU1__PMC.P12.R               /* Program Register 12 */
#define RTU1__PMC_P13        RTU1__PMC.P13.R               /* Program Register 13 */
#define RTU1__PMC_P14        RTU1__PMC.P14.R               /* Program Register 14 */
#define RTU1__PMC_P15        RTU1__PMC.P15.R               /* Program Register 15 */
#define RTU1__PMC_P16        RTU1__PMC.P16.R               /* Program Register 16 */
#define RTU1__PMC_P17        RTU1__PMC.P17.R               /* Program Register 17 */
#define RTU1__PMC_P18        RTU1__PMC.P18.R               /* Program Register 18 */
#define RTU1__PMC_P19        RTU1__PMC.P19.R               /* Program Register 19 */
#define RTU1__PMC_P20        RTU1__PMC.P20.R               /* Program Register 20 */
#define RTU1__PMC_P21        RTU1__PMC.P21.R               /* Program Register 21 */
#define RTU1__PMC_P22        RTU1__PMC.P22.R               /* Program Register 22 */
#define RTU1__PMC_P23        RTU1__PMC.P23.R               /* Program Register 23 */
#define RTU1__PMC_P24        RTU1__PMC.P24.R               /* Program Register 24 */
#define RTU1__PMC_P25        RTU1__PMC.P25.R               /* Program Register 25 */
#define RTU1__PMC_P26        RTU1__PMC.P26.R               /* Program Register 26 */
#define RTU1__PMC_P27        RTU1__PMC.P27.R               /* Program Register 27 */
#define RTU1__PMC_P28        RTU1__PMC.P28.R               /* Program Register 28 */
#define RTU1__PMC_P29        RTU1__PMC.P29.R               /* Program Register 29 */
#define RTU1__PMC_P30        RTU1__PMC.P30.R               /* Program Register 30 */
#define RTU1__PMC_P31        RTU1__PMC.P31.R               /* Program Register 31 */
#define RTU1__PMC_ITCTRL     RTU1__PMC.ITCTRL.R            /* Integration Mode Control Register */
#define RTU1__PMC_CLAIMSET   RTU1__PMC.CLAIMSET.R          /* Claim Tag Set Register */
#define RTU1__PMC_CLAIMCLR   RTU1__PMC.CLAIMCLR.R          /* Claim Tag Clear Register */
#define RTU1__PMC_DEVAFF0    RTU1__PMC.DEVAFF0.R           /* Device Affinity 0 Register */
#define RTU1__PMC_DEVAFF1    RTU1__PMC.DEVAFF1.R           /* Device Affinity 1 Register */
#define RTU1__PMC_LAR        RTU1__PMC.LAR.R               /* Lock Access Register */
#define RTU1__PMC_LSR        RTU1__PMC.LSR.R               /* Lock Status Register */
#define RTU1__PMC_AUTHSTATUS RTU1__PMC.AUTHSTATUS.R        /* Authentication Status Register */
#define RTU1__PMC_DEVARCH    RTU1__PMC.DEVARCH.R           /* Device Architecture Register */
#define RTU1__PMC_DEVID2     RTU1__PMC.DEVID2.R            /* Device ID Register 2 */
#define RTU1__PMC_DEVID1     RTU1__PMC.DEVID1.R            /* Device ID Register 1 */
#define RTU1__PMC_DEVID      RTU1__PMC.DEVID.R             /* Device ID Register */
#define RTU1__PMC_DEVTYPE    RTU1__PMC.DEVTYPE.R           /* Device Type Identifier Register */
#define RTU1__PMC_PIDR4      RTU1__PMC.PIDR4.R             /* Peripheral Identification Register 4 */
#define RTU1__PMC_PIDR5      RTU1__PMC.PIDR5.R             /* Peripheral Identification Register 5 */
#define RTU1__PMC_PIDR6      RTU1__PMC.PIDR6.R             /* Peripheral Identification Register 6 */
#define RTU1__PMC_PIDR7      RTU1__PMC.PIDR7.R             /* Peripheral Identification Register 7 */
#define RTU1__PMC_PIDR0      RTU1__PMC.PIDR0.R             /* Peripheral Identification Register 0 */
#define RTU1__PMC_PIDR1      RTU1__PMC.PIDR1.R             /* Peripheral Identification Register 1 */
#define RTU1__PMC_PIDR2      RTU1__PMC.PIDR2.R             /* Peripheral Identification Register 2 */
#define RTU1__PMC_PIDR3      RTU1__PMC.PIDR3.R             /* Peripheral Identification Register 3 */
#define RTU1__PMC_CIDR0      RTU1__PMC.CIDR0.R             /* Component Identification Register 0 */
#define RTU1__PMC_CIDR1      RTU1__PMC.CIDR1.R             /* Component Identification Register 1 */
#define RTU1__PMC_CIDR2      RTU1__PMC.CIDR2.R             /* Component Identification Register 2 */
#define RTU1__PMC_CIDR3      RTU1__PMC.CIDR3.R             /* Component Identification Register 3 */

/* RTU_SEMA42 */
#define RTU0__SEMA42_GATE3   RTU0__SEMA42.GATE[0].R        /* Gate */
#define RTU0__SEMA42_GATE2   RTU0__SEMA42.GATE[1].R        /* Gate */
#define RTU0__SEMA42_GATE1   RTU0__SEMA42.GATE[2].R        /* Gate */
#define RTU0__SEMA42_GATE0   RTU0__SEMA42.GATE[3].R        /* Gate */
#define RTU0__SEMA42_GATE7   RTU0__SEMA42.GATE[4].R        /* Gate */
#define RTU0__SEMA42_GATE6   RTU0__SEMA42.GATE[5].R        /* Gate */
#define RTU0__SEMA42_GATE5   RTU0__SEMA42.GATE[6].R        /* Gate */
#define RTU0__SEMA42_GATE4   RTU0__SEMA42.GATE[7].R        /* Gate */
#define RTU0__SEMA42_GATE11  RTU0__SEMA42.GATE[8].R        /* Gate */
#define RTU0__SEMA42_GATE10  RTU0__SEMA42.GATE[9].R        /* Gate */
#define RTU0__SEMA42_GATE9   RTU0__SEMA42.GATE[10].R       /* Gate */
#define RTU0__SEMA42_GATE8   RTU0__SEMA42.GATE[11].R       /* Gate */
#define RTU0__SEMA42_GATE15  RTU0__SEMA42.GATE[12].R       /* Gate */
#define RTU0__SEMA42_GATE14  RTU0__SEMA42.GATE[13].R       /* Gate */
#define RTU0__SEMA42_GATE13  RTU0__SEMA42.GATE[14].R       /* Gate */
#define RTU0__SEMA42_GATE12  RTU0__SEMA42.GATE[15].R       /* Gate */
#define RTU0__SEMA42_RSTGT_R RTU0__SEMA42.RSTGT.R.R        /* Reset Gate Read */
#define RTU0__SEMA42_RSTGT_W RTU0__SEMA42.RSTGT.W.R        /* Reset Gate Write */

#define RTU1__SEMA42_GATE3   RTU1__SEMA42.GATE[0].R        /* Gate */
#define RTU1__SEMA42_GATE2   RTU1__SEMA42.GATE[1].R        /* Gate */
#define RTU1__SEMA42_GATE1   RTU1__SEMA42.GATE[2].R        /* Gate */
#define RTU1__SEMA42_GATE0   RTU1__SEMA42.GATE[3].R        /* Gate */
#define RTU1__SEMA42_GATE7   RTU1__SEMA42.GATE[4].R        /* Gate */
#define RTU1__SEMA42_GATE6   RTU1__SEMA42.GATE[5].R        /* Gate */
#define RTU1__SEMA42_GATE5   RTU1__SEMA42.GATE[6].R        /* Gate */
#define RTU1__SEMA42_GATE4   RTU1__SEMA42.GATE[7].R        /* Gate */
#define RTU1__SEMA42_GATE11  RTU1__SEMA42.GATE[8].R        /* Gate */
#define RTU1__SEMA42_GATE10  RTU1__SEMA42.GATE[9].R        /* Gate */
#define RTU1__SEMA42_GATE9   RTU1__SEMA42.GATE[10].R       /* Gate */
#define RTU1__SEMA42_GATE8   RTU1__SEMA42.GATE[11].R       /* Gate */
#define RTU1__SEMA42_GATE15  RTU1__SEMA42.GATE[12].R       /* Gate */
#define RTU1__SEMA42_GATE14  RTU1__SEMA42.GATE[13].R       /* Gate */
#define RTU1__SEMA42_GATE13  RTU1__SEMA42.GATE[14].R       /* Gate */
#define RTU1__SEMA42_GATE12  RTU1__SEMA42.GATE[15].R       /* Gate */
#define RTU1__SEMA42_RSTGT_R RTU1__SEMA42.RSTGT.R.R        /* Reset Gate Read */
#define RTU1__SEMA42_RSTGT_W RTU1__SEMA42.RSTGT.W.R        /* Reset Gate Write */

/* RTU_XRDC */
#define RTU0__XRDC_CR        RTU0__XRDC.CR.R               /* Control */
#define RTU0__XRDC_HWCFG0    RTU0__XRDC.HWCFG0.R           /* Hardware Configuration 0 */
#define RTU0__XRDC_HWCFG1    RTU0__XRDC.HWCFG1.R           /* Hardware Configuration 1 */
#define RTU0__XRDC_MDACFG0   RTU0__XRDC.MDACFG[0].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG1   RTU0__XRDC.MDACFG[1].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG2   RTU0__XRDC.MDACFG[2].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG3   RTU0__XRDC.MDACFG[3].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG4   RTU0__XRDC.MDACFG[4].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG5   RTU0__XRDC.MDACFG[5].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG6   RTU0__XRDC.MDACFG[6].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG7   RTU0__XRDC.MDACFG[7].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG8   RTU0__XRDC.MDACFG[8].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG9   RTU0__XRDC.MDACFG[9].R        /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG10  RTU0__XRDC.MDACFG[10].R       /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG11  RTU0__XRDC.MDACFG[11].R       /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MDACFG12  RTU0__XRDC.MDACFG[12].R       /* Master Domain Assignment Configuration */
#define RTU0__XRDC_MRCFG0    RTU0__XRDC.MRCFG[0].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG1    RTU0__XRDC.MRCFG[1].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG2    RTU0__XRDC.MRCFG[2].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG3    RTU0__XRDC.MRCFG[3].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG4    RTU0__XRDC.MRCFG[4].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG5    RTU0__XRDC.MRCFG[5].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG6    RTU0__XRDC.MRCFG[6].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG7    RTU0__XRDC.MRCFG[7].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG8    RTU0__XRDC.MRCFG[8].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG9    RTU0__XRDC.MRCFG[9].R         /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG10   RTU0__XRDC.MRCFG[10].R        /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG11   RTU0__XRDC.MRCFG[11].R        /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG12   RTU0__XRDC.MRCFG[12].R        /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG13   RTU0__XRDC.MRCFG[13].R        /* Memory Region Configuration */
#define RTU0__XRDC_MRCFG14   RTU0__XRDC.MRCFG[14].R        /* Memory Region Configuration */
#define RTU0__XRDC_DERRLOC0  RTU0__XRDC.DERRLOC[0].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC1  RTU0__XRDC.DERRLOC[1].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC2  RTU0__XRDC.DERRLOC[2].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC3  RTU0__XRDC.DERRLOC[3].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC4  RTU0__XRDC.DERRLOC[4].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC5  RTU0__XRDC.DERRLOC[5].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC6  RTU0__XRDC.DERRLOC[6].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC7  RTU0__XRDC.DERRLOC[7].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC8  RTU0__XRDC.DERRLOC[8].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC9  RTU0__XRDC.DERRLOC[9].R       /* Domain Error Location */
#define RTU0__XRDC_DERRLOC10 RTU0__XRDC.DERRLOC[10].R      /* Domain Error Location */
#define RTU0__XRDC_DERRLOC11 RTU0__XRDC.DERRLOC[11].R      /* Domain Error Location */
#define RTU0__XRDC_DERRLOC12 RTU0__XRDC.DERRLOC[12].R      /* Domain Error Location */
#define RTU0__XRDC_DERRLOC13 RTU0__XRDC.DERRLOC[13].R      /* Domain Error Location */
#define RTU0__XRDC_DERRLOC14 RTU0__XRDC.DERRLOC[14].R      /* Domain Error Location */
#define RTU0__XRDC_DERRLOC15 RTU0__XRDC.DERRLOC[15].R      /* Domain Error Location */
#define RTU0__XRDC_DERR_W0_0 RTU0__XRDC.DERRW0[0].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_0 RTU0__XRDC.DERRW0[0].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_0 RTU0__XRDC.DERRW0[0].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_1 RTU0__XRDC.DERRW0[1].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_1 RTU0__XRDC.DERRW0[1].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_1 RTU0__XRDC.DERRW0[1].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_2 RTU0__XRDC.DERRW0[2].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_2 RTU0__XRDC.DERRW0[2].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_2 RTU0__XRDC.DERRW0[2].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_3 RTU0__XRDC.DERRW0[3].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_3 RTU0__XRDC.DERRW0[3].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_3 RTU0__XRDC.DERRW0[3].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_4 RTU0__XRDC.DERRW0[4].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_4 RTU0__XRDC.DERRW0[4].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_4 RTU0__XRDC.DERRW0[4].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_5 RTU0__XRDC.DERRW0[5].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_5 RTU0__XRDC.DERRW0[5].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_5 RTU0__XRDC.DERRW0[5].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_6 RTU0__XRDC.DERRW0[6].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_6 RTU0__XRDC.DERRW0[6].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_6 RTU0__XRDC.DERRW0[6].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_7 RTU0__XRDC.DERRW0[7].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_7 RTU0__XRDC.DERRW0[7].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_7 RTU0__XRDC.DERRW0[7].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_8 RTU0__XRDC.DERRW0[8].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_8 RTU0__XRDC.DERRW0[8].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_8 RTU0__XRDC.DERRW0[8].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_9 RTU0__XRDC.DERRW0[9].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_9 RTU0__XRDC.DERRW0[9].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_9 RTU0__XRDC.DERRW0[9].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_10 RTU0__XRDC.DERRW0[10].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_10 RTU0__XRDC.DERRW0[10].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_10 RTU0__XRDC.DERRW0[10].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_11 RTU0__XRDC.DERRW0[11].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_11 RTU0__XRDC.DERRW0[11].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_11 RTU0__XRDC.DERRW0[11].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_12 RTU0__XRDC.DERRW0[12].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_12 RTU0__XRDC.DERRW0[12].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_12 RTU0__XRDC.DERRW0[12].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_13 RTU0__XRDC.DERRW0[13].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_13 RTU0__XRDC.DERRW0[13].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_13 RTU0__XRDC.DERRW0[13].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_14 RTU0__XRDC.DERRW0[14].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_14 RTU0__XRDC.DERRW0[14].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_14 RTU0__XRDC.DERRW0[14].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_16 RTU0__XRDC.DERRW0[16].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_16 RTU0__XRDC.DERRW0[16].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_16 RTU0__XRDC.DERRW0[16].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_17 RTU0__XRDC.DERRW0[17].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_17 RTU0__XRDC.DERRW0[17].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_17 RTU0__XRDC.DERRW0[17].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_DERR_W0_18 RTU0__XRDC.DERRW0[18].DERR_W0.R /* Domain Error Word 0 */
#define RTU0__XRDC_DERR_W1_18 RTU0__XRDC.DERRW0[18].DERR_W1.R /* Domain Error Word 1 */
#define RTU0__XRDC_DERR_W3_18 RTU0__XRDC.DERRW0[18].DERR_W3.R /* Domain Error Word 3 */
#define RTU0__XRDC_MDA_W0_0_DFMT1 RTU0__XRDC.MDA_W0_0_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_1_DFMT1 RTU0__XRDC.MDA_W0_1_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_2_DFMT1 RTU0__XRDC.MDA_W0_2_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_3_DFMT1 RTU0__XRDC.MDA_W0_3_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_4_DFMT1 RTU0__XRDC.MDA_W0_4_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W1_4_DFMT1 RTU0__XRDC.MDA_W1_4_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W2_4_DFMT1 RTU0__XRDC.MDA_W2_4_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W3_4_DFMT1 RTU0__XRDC.MDA_W3_4_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W4_4_DFMT1 RTU0__XRDC.MDA_W4_4_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W5_4_DFMT1 RTU0__XRDC.MDA_W5_4_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W6_4_DFMT1 RTU0__XRDC.MDA_W6_4_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W7_4_DFMT1 RTU0__XRDC.MDA_W7_4_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_5_DFMT1 RTU0__XRDC.MDA_W0_5_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W1_5_DFMT1 RTU0__XRDC.MDA_W1_5_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W2_5_DFMT1 RTU0__XRDC.MDA_W2_5_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W3_5_DFMT1 RTU0__XRDC.MDA_W3_5_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W4_5_DFMT1 RTU0__XRDC.MDA_W4_5_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W5_5_DFMT1 RTU0__XRDC.MDA_W5_5_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W6_5_DFMT1 RTU0__XRDC.MDA_W6_5_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W7_5_DFMT1 RTU0__XRDC.MDA_W7_5_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_6_DFMT1 RTU0__XRDC.MDA_W0_6_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W1_6_DFMT1 RTU0__XRDC.MDA_W1_6_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W2_6_DFMT1 RTU0__XRDC.MDA_W2_6_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W3_6_DFMT1 RTU0__XRDC.MDA_W3_6_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W4_6_DFMT1 RTU0__XRDC.MDA_W4_6_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W5_6_DFMT1 RTU0__XRDC.MDA_W5_6_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W6_6_DFMT1 RTU0__XRDC.MDA_W6_6_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W7_6_DFMT1 RTU0__XRDC.MDA_W7_6_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_7_DFMT1 RTU0__XRDC.MDA_W0_7_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W1_7_DFMT1 RTU0__XRDC.MDA_W1_7_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W2_7_DFMT1 RTU0__XRDC.MDA_W2_7_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W3_7_DFMT1 RTU0__XRDC.MDA_W3_7_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W4_7_DFMT1 RTU0__XRDC.MDA_W4_7_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W5_7_DFMT1 RTU0__XRDC.MDA_W5_7_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W6_7_DFMT1 RTU0__XRDC.MDA_W6_7_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W7_7_DFMT1 RTU0__XRDC.MDA_W7_7_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_8_DFMT1 RTU0__XRDC.MDA_W0_8_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W1_8_DFMT1 RTU0__XRDC.MDA_W1_8_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W2_8_DFMT1 RTU0__XRDC.MDA_W2_8_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W3_8_DFMT1 RTU0__XRDC.MDA_W3_8_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W4_8_DFMT1 RTU0__XRDC.MDA_W4_8_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W5_8_DFMT1 RTU0__XRDC.MDA_W5_8_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W6_8_DFMT1 RTU0__XRDC.MDA_W6_8_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W7_8_DFMT1 RTU0__XRDC.MDA_W7_8_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_9_DFMT1 RTU0__XRDC.MDA_W0_9_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W1_9_DFMT1 RTU0__XRDC.MDA_W1_9_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W2_9_DFMT1 RTU0__XRDC.MDA_W2_9_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W3_9_DFMT1 RTU0__XRDC.MDA_W3_9_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W4_9_DFMT1 RTU0__XRDC.MDA_W4_9_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W5_9_DFMT1 RTU0__XRDC.MDA_W5_9_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W6_9_DFMT1 RTU0__XRDC.MDA_W6_9_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W7_9_DFMT1 RTU0__XRDC.MDA_W7_9_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_10_DFMT1 RTU0__XRDC.MDA_W0_10_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W1_10_DFMT1 RTU0__XRDC.MDA_W1_10_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W2_10_DFMT1 RTU0__XRDC.MDA_W2_10_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W3_10_DFMT1 RTU0__XRDC.MDA_W3_10_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W4_10_DFMT1 RTU0__XRDC.MDA_W4_10_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W5_10_DFMT1 RTU0__XRDC.MDA_W5_10_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W6_10_DFMT1 RTU0__XRDC.MDA_W6_10_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W7_10_DFMT1 RTU0__XRDC.MDA_W7_10_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_11_DFMT1 RTU0__XRDC.MDA_W0_11_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W1_11_DFMT1 RTU0__XRDC.MDA_W1_11_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W2_11_DFMT1 RTU0__XRDC.MDA_W2_11_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W3_11_DFMT1 RTU0__XRDC.MDA_W3_11_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W4_11_DFMT1 RTU0__XRDC.MDA_W4_11_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W5_11_DFMT1 RTU0__XRDC.MDA_W5_11_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W6_11_DFMT1 RTU0__XRDC.MDA_W6_11_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W7_11_DFMT1 RTU0__XRDC.MDA_W7_11_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_MDA_W0_12_DFMT1 RTU0__XRDC.MDA_W0_12_DFMT1.R /* Master Domain Assignment */
#define RTU0__XRDC_PDAC_W0_0 RTU0__XRDC.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_0 RTU0__XRDC.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_1 RTU0__XRDC.PDAC_SLOT[0].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_1 RTU0__XRDC.PDAC_SLOT[0].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_2 RTU0__XRDC.PDAC_SLOT[0].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_2 RTU0__XRDC.PDAC_SLOT[0].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_3 RTU0__XRDC.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_3 RTU0__XRDC.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_4 RTU0__XRDC.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_4 RTU0__XRDC.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_5 RTU0__XRDC.PDAC_SLOT[0].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_5 RTU0__XRDC.PDAC_SLOT[0].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_6 RTU0__XRDC.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_6 RTU0__XRDC.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_7 RTU0__XRDC.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_7 RTU0__XRDC.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_8 RTU0__XRDC.PDAC_SLOT[0].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_8 RTU0__XRDC.PDAC_SLOT[0].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_9 RTU0__XRDC.PDAC_SLOT[0].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_9 RTU0__XRDC.PDAC_SLOT[0].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_10 RTU0__XRDC.PDAC_SLOT[0].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_10 RTU0__XRDC.PDAC_SLOT[0].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_11 RTU0__XRDC.PDAC_SLOT[0].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_11 RTU0__XRDC.PDAC_SLOT[0].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_12 RTU0__XRDC.PDAC_SLOT[0].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_12 RTU0__XRDC.PDAC_SLOT[0].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_13 RTU0__XRDC.PDAC_SLOT[0].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_13 RTU0__XRDC.PDAC_SLOT[0].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_14 RTU0__XRDC.PDAC_SLOT[0].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_14 RTU0__XRDC.PDAC_SLOT[0].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_15 RTU0__XRDC.PDAC_SLOT[0].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_15 RTU0__XRDC.PDAC_SLOT[0].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_18 RTU0__XRDC.PDAC_SLOT[0].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_18 RTU0__XRDC.PDAC_SLOT[0].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_19 RTU0__XRDC.PDAC_SLOT[0].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_19 RTU0__XRDC.PDAC_SLOT[0].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_20 RTU0__XRDC.PDAC_SLOT[0].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_20 RTU0__XRDC.PDAC_SLOT[0].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_21 RTU0__XRDC.PDAC_SLOT[0].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_21 RTU0__XRDC.PDAC_SLOT[0].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_22 RTU0__XRDC.PDAC_SLOT[0].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_22 RTU0__XRDC.PDAC_SLOT[0].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_23 RTU0__XRDC.PDAC_SLOT[0].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_23 RTU0__XRDC.PDAC_SLOT[0].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_24 RTU0__XRDC.PDAC_SLOT[0].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_24 RTU0__XRDC.PDAC_SLOT[0].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_25 RTU0__XRDC.PDAC_SLOT[0].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_25 RTU0__XRDC.PDAC_SLOT[0].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_26 RTU0__XRDC.PDAC_SLOT[0].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_26 RTU0__XRDC.PDAC_SLOT[0].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_27 RTU0__XRDC.PDAC_SLOT[0].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_27 RTU0__XRDC.PDAC_SLOT[0].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_28 RTU0__XRDC.PDAC_SLOT[0].PDACN[28].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_28 RTU0__XRDC.PDAC_SLOT[0].PDACN[28].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_29 RTU0__XRDC.PDAC_SLOT[0].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_29 RTU0__XRDC.PDAC_SLOT[0].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_31 RTU0__XRDC.PDAC_SLOT[0].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_31 RTU0__XRDC.PDAC_SLOT[0].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_32 RTU0__XRDC.PDAC_SLOT[0].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_32 RTU0__XRDC.PDAC_SLOT[0].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_33 RTU0__XRDC.PDAC_SLOT[0].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_33 RTU0__XRDC.PDAC_SLOT[0].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_34 RTU0__XRDC.PDAC_SLOT[0].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_34 RTU0__XRDC.PDAC_SLOT[0].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_35 RTU0__XRDC.PDAC_SLOT[0].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_35 RTU0__XRDC.PDAC_SLOT[0].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_36 RTU0__XRDC.PDAC_SLOT[0].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_36 RTU0__XRDC.PDAC_SLOT[0].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_37 RTU0__XRDC.PDAC_SLOT[0].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_37 RTU0__XRDC.PDAC_SLOT[0].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_38 RTU0__XRDC.PDAC_SLOT[0].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_38 RTU0__XRDC.PDAC_SLOT[0].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_39 RTU0__XRDC.PDAC_SLOT[0].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_39 RTU0__XRDC.PDAC_SLOT[0].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_40 RTU0__XRDC.PDAC_SLOT[0].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_40 RTU0__XRDC.PDAC_SLOT[0].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_41 RTU0__XRDC.PDAC_SLOT[0].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_41 RTU0__XRDC.PDAC_SLOT[0].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_42 RTU0__XRDC.PDAC_SLOT[0].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_42 RTU0__XRDC.PDAC_SLOT[0].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_128 RTU0__XRDC.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_128 RTU0__XRDC.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_129 RTU0__XRDC.PDAC_SLOT[1].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_129 RTU0__XRDC.PDAC_SLOT[1].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_130 RTU0__XRDC.PDAC_SLOT[1].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_130 RTU0__XRDC.PDAC_SLOT[1].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_131 RTU0__XRDC.PDAC_SLOT[1].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_131 RTU0__XRDC.PDAC_SLOT[1].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_132 RTU0__XRDC.PDAC_SLOT[1].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_132 RTU0__XRDC.PDAC_SLOT[1].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_133 RTU0__XRDC.PDAC_SLOT[1].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_133 RTU0__XRDC.PDAC_SLOT[1].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_134 RTU0__XRDC.PDAC_SLOT[1].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_134 RTU0__XRDC.PDAC_SLOT[1].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_135 RTU0__XRDC.PDAC_SLOT[1].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_135 RTU0__XRDC.PDAC_SLOT[1].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_136 RTU0__XRDC.PDAC_SLOT[1].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_136 RTU0__XRDC.PDAC_SLOT[1].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_138 RTU0__XRDC.PDAC_SLOT[1].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_138 RTU0__XRDC.PDAC_SLOT[1].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_139 RTU0__XRDC.PDAC_SLOT[1].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_139 RTU0__XRDC.PDAC_SLOT[1].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_140 RTU0__XRDC.PDAC_SLOT[1].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_140 RTU0__XRDC.PDAC_SLOT[1].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_141 RTU0__XRDC.PDAC_SLOT[1].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_141 RTU0__XRDC.PDAC_SLOT[1].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_142 RTU0__XRDC.PDAC_SLOT[1].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_142 RTU0__XRDC.PDAC_SLOT[1].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_143 RTU0__XRDC.PDAC_SLOT[1].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_143 RTU0__XRDC.PDAC_SLOT[1].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_144 RTU0__XRDC.PDAC_SLOT[1].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_144 RTU0__XRDC.PDAC_SLOT[1].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_145 RTU0__XRDC.PDAC_SLOT[1].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_145 RTU0__XRDC.PDAC_SLOT[1].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_149 RTU0__XRDC.PDAC_SLOT[1].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_149 RTU0__XRDC.PDAC_SLOT[1].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_150 RTU0__XRDC.PDAC_SLOT[1].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_150 RTU0__XRDC.PDAC_SLOT[1].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_151 RTU0__XRDC.PDAC_SLOT[1].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_151 RTU0__XRDC.PDAC_SLOT[1].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_152 RTU0__XRDC.PDAC_SLOT[1].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_152 RTU0__XRDC.PDAC_SLOT[1].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_256 RTU0__XRDC.PDAC_SLOT[2].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_256 RTU0__XRDC.PDAC_SLOT[2].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_257 RTU0__XRDC.PDAC_SLOT[2].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_257 RTU0__XRDC.PDAC_SLOT[2].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_258 RTU0__XRDC.PDAC_SLOT[2].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_258 RTU0__XRDC.PDAC_SLOT[2].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_259 RTU0__XRDC.PDAC_SLOT[2].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_259 RTU0__XRDC.PDAC_SLOT[2].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_260 RTU0__XRDC.PDAC_SLOT[2].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_260 RTU0__XRDC.PDAC_SLOT[2].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_261 RTU0__XRDC.PDAC_SLOT[2].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_261 RTU0__XRDC.PDAC_SLOT[2].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_262 RTU0__XRDC.PDAC_SLOT[2].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_262 RTU0__XRDC.PDAC_SLOT[2].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_263 RTU0__XRDC.PDAC_SLOT[2].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_263 RTU0__XRDC.PDAC_SLOT[2].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_265 RTU0__XRDC.PDAC_SLOT[2].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_265 RTU0__XRDC.PDAC_SLOT[2].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_268 RTU0__XRDC.PDAC_SLOT[2].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_268 RTU0__XRDC.PDAC_SLOT[2].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_269 RTU0__XRDC.PDAC_SLOT[2].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_269 RTU0__XRDC.PDAC_SLOT[2].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_270 RTU0__XRDC.PDAC_SLOT[2].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_270 RTU0__XRDC.PDAC_SLOT[2].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_271 RTU0__XRDC.PDAC_SLOT[2].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_271 RTU0__XRDC.PDAC_SLOT[2].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_274 RTU0__XRDC.PDAC_SLOT[2].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_274 RTU0__XRDC.PDAC_SLOT[2].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_275 RTU0__XRDC.PDAC_SLOT[2].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_275 RTU0__XRDC.PDAC_SLOT[2].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_276 RTU0__XRDC.PDAC_SLOT[2].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_276 RTU0__XRDC.PDAC_SLOT[2].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_277 RTU0__XRDC.PDAC_SLOT[2].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_277 RTU0__XRDC.PDAC_SLOT[2].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_278 RTU0__XRDC.PDAC_SLOT[2].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_278 RTU0__XRDC.PDAC_SLOT[2].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_279 RTU0__XRDC.PDAC_SLOT[2].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_279 RTU0__XRDC.PDAC_SLOT[2].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_280 RTU0__XRDC.PDAC_SLOT[2].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_280 RTU0__XRDC.PDAC_SLOT[2].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_281 RTU0__XRDC.PDAC_SLOT[2].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_281 RTU0__XRDC.PDAC_SLOT[2].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_282 RTU0__XRDC.PDAC_SLOT[2].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_282 RTU0__XRDC.PDAC_SLOT[2].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_283 RTU0__XRDC.PDAC_SLOT[2].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_283 RTU0__XRDC.PDAC_SLOT[2].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_284 RTU0__XRDC.PDAC_SLOT[2].PDACN[28].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_284 RTU0__XRDC.PDAC_SLOT[2].PDACN[28].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_285 RTU0__XRDC.PDAC_SLOT[2].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_285 RTU0__XRDC.PDAC_SLOT[2].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_287 RTU0__XRDC.PDAC_SLOT[2].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_287 RTU0__XRDC.PDAC_SLOT[2].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_288 RTU0__XRDC.PDAC_SLOT[2].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_288 RTU0__XRDC.PDAC_SLOT[2].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_289 RTU0__XRDC.PDAC_SLOT[2].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_289 RTU0__XRDC.PDAC_SLOT[2].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_290 RTU0__XRDC.PDAC_SLOT[2].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_290 RTU0__XRDC.PDAC_SLOT[2].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_291 RTU0__XRDC.PDAC_SLOT[2].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_291 RTU0__XRDC.PDAC_SLOT[2].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_292 RTU0__XRDC.PDAC_SLOT[2].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_292 RTU0__XRDC.PDAC_SLOT[2].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_293 RTU0__XRDC.PDAC_SLOT[2].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_293 RTU0__XRDC.PDAC_SLOT[2].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_294 RTU0__XRDC.PDAC_SLOT[2].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_294 RTU0__XRDC.PDAC_SLOT[2].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_295 RTU0__XRDC.PDAC_SLOT[2].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_295 RTU0__XRDC.PDAC_SLOT[2].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_296 RTU0__XRDC.PDAC_SLOT[2].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_296 RTU0__XRDC.PDAC_SLOT[2].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_297 RTU0__XRDC.PDAC_SLOT[2].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_297 RTU0__XRDC.PDAC_SLOT[2].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_PDAC_W0_298 RTU0__XRDC.PDAC_SLOT[2].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU0__XRDC_PDAC_W1_298 RTU0__XRDC.PDAC_SLOT[2].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU0__XRDC_MRGD_W0_0 RTU0__XRDC.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_0 RTU0__XRDC.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_0 RTU0__XRDC.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_0 RTU0__XRDC.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_1 RTU0__XRDC.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_1 RTU0__XRDC.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_1 RTU0__XRDC.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_1 RTU0__XRDC.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_2 RTU0__XRDC.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_2 RTU0__XRDC.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_2 RTU0__XRDC.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_2 RTU0__XRDC.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_3 RTU0__XRDC.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_3 RTU0__XRDC.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_3 RTU0__XRDC.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_3 RTU0__XRDC.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_4 RTU0__XRDC.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_4 RTU0__XRDC.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_4 RTU0__XRDC.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_4 RTU0__XRDC.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_5 RTU0__XRDC.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_5 RTU0__XRDC.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_5 RTU0__XRDC.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_5 RTU0__XRDC.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_6 RTU0__XRDC.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_6 RTU0__XRDC.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_6 RTU0__XRDC.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_6 RTU0__XRDC.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_7 RTU0__XRDC.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_7 RTU0__XRDC.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_7 RTU0__XRDC.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_7 RTU0__XRDC.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_8 RTU0__XRDC.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_8 RTU0__XRDC.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_8 RTU0__XRDC.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_8 RTU0__XRDC.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_9 RTU0__XRDC.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_9 RTU0__XRDC.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_9 RTU0__XRDC.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_9 RTU0__XRDC.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_10 RTU0__XRDC.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_10 RTU0__XRDC.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_10 RTU0__XRDC.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_10 RTU0__XRDC.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_11 RTU0__XRDC.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_11 RTU0__XRDC.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_11 RTU0__XRDC.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_11 RTU0__XRDC.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_12 RTU0__XRDC.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_12 RTU0__XRDC.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_12 RTU0__XRDC.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_12 RTU0__XRDC.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_13 RTU0__XRDC.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_13 RTU0__XRDC.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_13 RTU0__XRDC.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_13 RTU0__XRDC.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_14 RTU0__XRDC.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_14 RTU0__XRDC.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_14 RTU0__XRDC.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_14 RTU0__XRDC.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_15 RTU0__XRDC.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_15 RTU0__XRDC.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_15 RTU0__XRDC.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_15 RTU0__XRDC.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_16 RTU0__XRDC.MRGDN[1][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_16 RTU0__XRDC.MRGDN[1][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_16 RTU0__XRDC.MRGDN[1][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_16 RTU0__XRDC.MRGDN[1][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_17 RTU0__XRDC.MRGDN[1][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_17 RTU0__XRDC.MRGDN[1][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_17 RTU0__XRDC.MRGDN[1][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_17 RTU0__XRDC.MRGDN[1][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_18 RTU0__XRDC.MRGDN[1][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_18 RTU0__XRDC.MRGDN[1][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_18 RTU0__XRDC.MRGDN[1][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_18 RTU0__XRDC.MRGDN[1][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_19 RTU0__XRDC.MRGDN[1][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_19 RTU0__XRDC.MRGDN[1][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_19 RTU0__XRDC.MRGDN[1][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_19 RTU0__XRDC.MRGDN[1][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_20 RTU0__XRDC.MRGDN[1][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_20 RTU0__XRDC.MRGDN[1][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_20 RTU0__XRDC.MRGDN[1][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_20 RTU0__XRDC.MRGDN[1][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_21 RTU0__XRDC.MRGDN[1][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_21 RTU0__XRDC.MRGDN[1][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_21 RTU0__XRDC.MRGDN[1][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_21 RTU0__XRDC.MRGDN[1][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_22 RTU0__XRDC.MRGDN[1][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_22 RTU0__XRDC.MRGDN[1][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_22 RTU0__XRDC.MRGDN[1][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_22 RTU0__XRDC.MRGDN[1][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_23 RTU0__XRDC.MRGDN[1][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_23 RTU0__XRDC.MRGDN[1][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_23 RTU0__XRDC.MRGDN[1][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_23 RTU0__XRDC.MRGDN[1][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_24 RTU0__XRDC.MRGDN[1][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_24 RTU0__XRDC.MRGDN[1][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_24 RTU0__XRDC.MRGDN[1][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_24 RTU0__XRDC.MRGDN[1][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_25 RTU0__XRDC.MRGDN[1][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_25 RTU0__XRDC.MRGDN[1][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_25 RTU0__XRDC.MRGDN[1][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_25 RTU0__XRDC.MRGDN[1][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_26 RTU0__XRDC.MRGDN[1][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_26 RTU0__XRDC.MRGDN[1][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_26 RTU0__XRDC.MRGDN[1][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_26 RTU0__XRDC.MRGDN[1][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_27 RTU0__XRDC.MRGDN[1][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_27 RTU0__XRDC.MRGDN[1][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_27 RTU0__XRDC.MRGDN[1][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_27 RTU0__XRDC.MRGDN[1][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_28 RTU0__XRDC.MRGDN[1][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_28 RTU0__XRDC.MRGDN[1][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_28 RTU0__XRDC.MRGDN[1][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_28 RTU0__XRDC.MRGDN[1][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_29 RTU0__XRDC.MRGDN[1][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_29 RTU0__XRDC.MRGDN[1][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_29 RTU0__XRDC.MRGDN[1][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_29 RTU0__XRDC.MRGDN[1][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_30 RTU0__XRDC.MRGDN[1][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_30 RTU0__XRDC.MRGDN[1][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_30 RTU0__XRDC.MRGDN[1][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_30 RTU0__XRDC.MRGDN[1][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_31 RTU0__XRDC.MRGDN[1][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_31 RTU0__XRDC.MRGDN[1][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_31 RTU0__XRDC.MRGDN[1][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_31 RTU0__XRDC.MRGDN[1][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_32 RTU0__XRDC.MRGDN[2][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_32 RTU0__XRDC.MRGDN[2][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_32 RTU0__XRDC.MRGDN[2][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_32 RTU0__XRDC.MRGDN[2][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_33 RTU0__XRDC.MRGDN[2][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_33 RTU0__XRDC.MRGDN[2][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_33 RTU0__XRDC.MRGDN[2][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_33 RTU0__XRDC.MRGDN[2][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_34 RTU0__XRDC.MRGDN[2][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_34 RTU0__XRDC.MRGDN[2][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_34 RTU0__XRDC.MRGDN[2][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_34 RTU0__XRDC.MRGDN[2][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_35 RTU0__XRDC.MRGDN[2][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_35 RTU0__XRDC.MRGDN[2][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_35 RTU0__XRDC.MRGDN[2][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_35 RTU0__XRDC.MRGDN[2][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_36 RTU0__XRDC.MRGDN[2][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_36 RTU0__XRDC.MRGDN[2][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_36 RTU0__XRDC.MRGDN[2][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_36 RTU0__XRDC.MRGDN[2][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_37 RTU0__XRDC.MRGDN[2][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_37 RTU0__XRDC.MRGDN[2][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_37 RTU0__XRDC.MRGDN[2][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_37 RTU0__XRDC.MRGDN[2][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_38 RTU0__XRDC.MRGDN[2][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_38 RTU0__XRDC.MRGDN[2][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_38 RTU0__XRDC.MRGDN[2][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_38 RTU0__XRDC.MRGDN[2][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_39 RTU0__XRDC.MRGDN[2][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_39 RTU0__XRDC.MRGDN[2][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_39 RTU0__XRDC.MRGDN[2][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_39 RTU0__XRDC.MRGDN[2][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_40 RTU0__XRDC.MRGDN[2][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_40 RTU0__XRDC.MRGDN[2][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_40 RTU0__XRDC.MRGDN[2][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_40 RTU0__XRDC.MRGDN[2][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_41 RTU0__XRDC.MRGDN[2][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_41 RTU0__XRDC.MRGDN[2][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_41 RTU0__XRDC.MRGDN[2][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_41 RTU0__XRDC.MRGDN[2][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_42 RTU0__XRDC.MRGDN[2][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_42 RTU0__XRDC.MRGDN[2][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_42 RTU0__XRDC.MRGDN[2][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_42 RTU0__XRDC.MRGDN[2][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_43 RTU0__XRDC.MRGDN[2][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_43 RTU0__XRDC.MRGDN[2][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_43 RTU0__XRDC.MRGDN[2][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_43 RTU0__XRDC.MRGDN[2][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_44 RTU0__XRDC.MRGDN[2][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_44 RTU0__XRDC.MRGDN[2][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_44 RTU0__XRDC.MRGDN[2][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_44 RTU0__XRDC.MRGDN[2][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_45 RTU0__XRDC.MRGDN[2][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_45 RTU0__XRDC.MRGDN[2][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_45 RTU0__XRDC.MRGDN[2][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_45 RTU0__XRDC.MRGDN[2][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_46 RTU0__XRDC.MRGDN[2][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_46 RTU0__XRDC.MRGDN[2][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_46 RTU0__XRDC.MRGDN[2][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_46 RTU0__XRDC.MRGDN[2][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_47 RTU0__XRDC.MRGDN[2][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_47 RTU0__XRDC.MRGDN[2][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_47 RTU0__XRDC.MRGDN[2][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_47 RTU0__XRDC.MRGDN[2][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_48 RTU0__XRDC.MRGDN[3][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_48 RTU0__XRDC.MRGDN[3][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_48 RTU0__XRDC.MRGDN[3][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_48 RTU0__XRDC.MRGDN[3][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_49 RTU0__XRDC.MRGDN[3][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_49 RTU0__XRDC.MRGDN[3][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_49 RTU0__XRDC.MRGDN[3][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_49 RTU0__XRDC.MRGDN[3][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_50 RTU0__XRDC.MRGDN[3][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_50 RTU0__XRDC.MRGDN[3][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_50 RTU0__XRDC.MRGDN[3][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_50 RTU0__XRDC.MRGDN[3][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_51 RTU0__XRDC.MRGDN[3][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_51 RTU0__XRDC.MRGDN[3][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_51 RTU0__XRDC.MRGDN[3][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_51 RTU0__XRDC.MRGDN[3][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_52 RTU0__XRDC.MRGDN[3][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_52 RTU0__XRDC.MRGDN[3][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_52 RTU0__XRDC.MRGDN[3][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_52 RTU0__XRDC.MRGDN[3][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_53 RTU0__XRDC.MRGDN[3][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_53 RTU0__XRDC.MRGDN[3][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_53 RTU0__XRDC.MRGDN[3][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_53 RTU0__XRDC.MRGDN[3][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_54 RTU0__XRDC.MRGDN[3][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_54 RTU0__XRDC.MRGDN[3][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_54 RTU0__XRDC.MRGDN[3][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_54 RTU0__XRDC.MRGDN[3][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_55 RTU0__XRDC.MRGDN[3][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_55 RTU0__XRDC.MRGDN[3][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_55 RTU0__XRDC.MRGDN[3][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_55 RTU0__XRDC.MRGDN[3][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_56 RTU0__XRDC.MRGDN[3][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_56 RTU0__XRDC.MRGDN[3][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_56 RTU0__XRDC.MRGDN[3][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_56 RTU0__XRDC.MRGDN[3][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_57 RTU0__XRDC.MRGDN[3][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_57 RTU0__XRDC.MRGDN[3][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_57 RTU0__XRDC.MRGDN[3][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_57 RTU0__XRDC.MRGDN[3][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_58 RTU0__XRDC.MRGDN[3][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_58 RTU0__XRDC.MRGDN[3][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_58 RTU0__XRDC.MRGDN[3][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_58 RTU0__XRDC.MRGDN[3][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_59 RTU0__XRDC.MRGDN[3][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_59 RTU0__XRDC.MRGDN[3][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_59 RTU0__XRDC.MRGDN[3][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_59 RTU0__XRDC.MRGDN[3][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_60 RTU0__XRDC.MRGDN[3][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_60 RTU0__XRDC.MRGDN[3][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_60 RTU0__XRDC.MRGDN[3][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_60 RTU0__XRDC.MRGDN[3][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_61 RTU0__XRDC.MRGDN[3][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_61 RTU0__XRDC.MRGDN[3][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_61 RTU0__XRDC.MRGDN[3][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_61 RTU0__XRDC.MRGDN[3][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_62 RTU0__XRDC.MRGDN[3][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_62 RTU0__XRDC.MRGDN[3][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_62 RTU0__XRDC.MRGDN[3][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_62 RTU0__XRDC.MRGDN[3][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_63 RTU0__XRDC.MRGDN[3][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_63 RTU0__XRDC.MRGDN[3][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_63 RTU0__XRDC.MRGDN[3][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_63 RTU0__XRDC.MRGDN[3][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_64 RTU0__XRDC.MRGDN[4][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_64 RTU0__XRDC.MRGDN[4][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_64 RTU0__XRDC.MRGDN[4][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_64 RTU0__XRDC.MRGDN[4][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_65 RTU0__XRDC.MRGDN[4][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_65 RTU0__XRDC.MRGDN[4][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_65 RTU0__XRDC.MRGDN[4][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_65 RTU0__XRDC.MRGDN[4][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_66 RTU0__XRDC.MRGDN[4][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_66 RTU0__XRDC.MRGDN[4][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_66 RTU0__XRDC.MRGDN[4][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_66 RTU0__XRDC.MRGDN[4][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_67 RTU0__XRDC.MRGDN[4][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_67 RTU0__XRDC.MRGDN[4][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_67 RTU0__XRDC.MRGDN[4][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_67 RTU0__XRDC.MRGDN[4][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_68 RTU0__XRDC.MRGDN[4][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_68 RTU0__XRDC.MRGDN[4][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_68 RTU0__XRDC.MRGDN[4][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_68 RTU0__XRDC.MRGDN[4][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_69 RTU0__XRDC.MRGDN[4][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_69 RTU0__XRDC.MRGDN[4][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_69 RTU0__XRDC.MRGDN[4][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_69 RTU0__XRDC.MRGDN[4][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_70 RTU0__XRDC.MRGDN[4][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_70 RTU0__XRDC.MRGDN[4][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_70 RTU0__XRDC.MRGDN[4][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_70 RTU0__XRDC.MRGDN[4][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_71 RTU0__XRDC.MRGDN[4][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_71 RTU0__XRDC.MRGDN[4][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_71 RTU0__XRDC.MRGDN[4][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_71 RTU0__XRDC.MRGDN[4][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_72 RTU0__XRDC.MRGDN[4][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_72 RTU0__XRDC.MRGDN[4][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_72 RTU0__XRDC.MRGDN[4][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_72 RTU0__XRDC.MRGDN[4][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_73 RTU0__XRDC.MRGDN[4][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_73 RTU0__XRDC.MRGDN[4][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_73 RTU0__XRDC.MRGDN[4][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_73 RTU0__XRDC.MRGDN[4][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_74 RTU0__XRDC.MRGDN[4][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_74 RTU0__XRDC.MRGDN[4][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_74 RTU0__XRDC.MRGDN[4][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_74 RTU0__XRDC.MRGDN[4][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_75 RTU0__XRDC.MRGDN[4][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_75 RTU0__XRDC.MRGDN[4][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_75 RTU0__XRDC.MRGDN[4][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_75 RTU0__XRDC.MRGDN[4][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_76 RTU0__XRDC.MRGDN[4][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_76 RTU0__XRDC.MRGDN[4][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_76 RTU0__XRDC.MRGDN[4][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_76 RTU0__XRDC.MRGDN[4][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_77 RTU0__XRDC.MRGDN[4][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_77 RTU0__XRDC.MRGDN[4][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_77 RTU0__XRDC.MRGDN[4][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_77 RTU0__XRDC.MRGDN[4][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_78 RTU0__XRDC.MRGDN[4][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_78 RTU0__XRDC.MRGDN[4][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_78 RTU0__XRDC.MRGDN[4][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_78 RTU0__XRDC.MRGDN[4][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_79 RTU0__XRDC.MRGDN[4][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_79 RTU0__XRDC.MRGDN[4][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_79 RTU0__XRDC.MRGDN[4][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_79 RTU0__XRDC.MRGDN[4][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_80 RTU0__XRDC.MRGDN[5][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_80 RTU0__XRDC.MRGDN[5][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_80 RTU0__XRDC.MRGDN[5][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_80 RTU0__XRDC.MRGDN[5][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_81 RTU0__XRDC.MRGDN[5][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_81 RTU0__XRDC.MRGDN[5][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_81 RTU0__XRDC.MRGDN[5][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_81 RTU0__XRDC.MRGDN[5][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_82 RTU0__XRDC.MRGDN[5][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_82 RTU0__XRDC.MRGDN[5][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_82 RTU0__XRDC.MRGDN[5][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_82 RTU0__XRDC.MRGDN[5][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_83 RTU0__XRDC.MRGDN[5][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_83 RTU0__XRDC.MRGDN[5][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_83 RTU0__XRDC.MRGDN[5][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_83 RTU0__XRDC.MRGDN[5][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_84 RTU0__XRDC.MRGDN[5][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_84 RTU0__XRDC.MRGDN[5][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_84 RTU0__XRDC.MRGDN[5][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_84 RTU0__XRDC.MRGDN[5][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_85 RTU0__XRDC.MRGDN[5][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_85 RTU0__XRDC.MRGDN[5][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_85 RTU0__XRDC.MRGDN[5][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_85 RTU0__XRDC.MRGDN[5][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_86 RTU0__XRDC.MRGDN[5][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_86 RTU0__XRDC.MRGDN[5][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_86 RTU0__XRDC.MRGDN[5][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_86 RTU0__XRDC.MRGDN[5][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_87 RTU0__XRDC.MRGDN[5][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_87 RTU0__XRDC.MRGDN[5][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_87 RTU0__XRDC.MRGDN[5][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_87 RTU0__XRDC.MRGDN[5][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_88 RTU0__XRDC.MRGDN[5][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_88 RTU0__XRDC.MRGDN[5][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_88 RTU0__XRDC.MRGDN[5][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_88 RTU0__XRDC.MRGDN[5][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_89 RTU0__XRDC.MRGDN[5][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_89 RTU0__XRDC.MRGDN[5][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_89 RTU0__XRDC.MRGDN[5][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_89 RTU0__XRDC.MRGDN[5][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_90 RTU0__XRDC.MRGDN[5][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_90 RTU0__XRDC.MRGDN[5][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_90 RTU0__XRDC.MRGDN[5][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_90 RTU0__XRDC.MRGDN[5][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_91 RTU0__XRDC.MRGDN[5][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_91 RTU0__XRDC.MRGDN[5][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_91 RTU0__XRDC.MRGDN[5][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_91 RTU0__XRDC.MRGDN[5][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_92 RTU0__XRDC.MRGDN[5][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_92 RTU0__XRDC.MRGDN[5][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_92 RTU0__XRDC.MRGDN[5][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_92 RTU0__XRDC.MRGDN[5][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_93 RTU0__XRDC.MRGDN[5][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_93 RTU0__XRDC.MRGDN[5][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_93 RTU0__XRDC.MRGDN[5][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_93 RTU0__XRDC.MRGDN[5][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_94 RTU0__XRDC.MRGDN[5][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_94 RTU0__XRDC.MRGDN[5][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_94 RTU0__XRDC.MRGDN[5][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_94 RTU0__XRDC.MRGDN[5][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_95 RTU0__XRDC.MRGDN[5][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_95 RTU0__XRDC.MRGDN[5][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_95 RTU0__XRDC.MRGDN[5][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_95 RTU0__XRDC.MRGDN[5][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_96 RTU0__XRDC.MRGDN[6][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_96 RTU0__XRDC.MRGDN[6][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_96 RTU0__XRDC.MRGDN[6][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_96 RTU0__XRDC.MRGDN[6][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_97 RTU0__XRDC.MRGDN[6][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_97 RTU0__XRDC.MRGDN[6][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_97 RTU0__XRDC.MRGDN[6][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_97 RTU0__XRDC.MRGDN[6][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_98 RTU0__XRDC.MRGDN[6][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_98 RTU0__XRDC.MRGDN[6][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_98 RTU0__XRDC.MRGDN[6][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_98 RTU0__XRDC.MRGDN[6][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_99 RTU0__XRDC.MRGDN[6][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_99 RTU0__XRDC.MRGDN[6][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_99 RTU0__XRDC.MRGDN[6][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_99 RTU0__XRDC.MRGDN[6][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_100 RTU0__XRDC.MRGDN[6][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_100 RTU0__XRDC.MRGDN[6][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_100 RTU0__XRDC.MRGDN[6][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_100 RTU0__XRDC.MRGDN[6][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_101 RTU0__XRDC.MRGDN[6][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_101 RTU0__XRDC.MRGDN[6][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_101 RTU0__XRDC.MRGDN[6][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_101 RTU0__XRDC.MRGDN[6][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_102 RTU0__XRDC.MRGDN[6][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_102 RTU0__XRDC.MRGDN[6][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_102 RTU0__XRDC.MRGDN[6][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_102 RTU0__XRDC.MRGDN[6][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_103 RTU0__XRDC.MRGDN[6][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_103 RTU0__XRDC.MRGDN[6][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_103 RTU0__XRDC.MRGDN[6][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_103 RTU0__XRDC.MRGDN[6][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_104 RTU0__XRDC.MRGDN[6][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_104 RTU0__XRDC.MRGDN[6][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_104 RTU0__XRDC.MRGDN[6][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_104 RTU0__XRDC.MRGDN[6][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_105 RTU0__XRDC.MRGDN[6][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_105 RTU0__XRDC.MRGDN[6][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_105 RTU0__XRDC.MRGDN[6][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_105 RTU0__XRDC.MRGDN[6][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_106 RTU0__XRDC.MRGDN[6][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_106 RTU0__XRDC.MRGDN[6][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_106 RTU0__XRDC.MRGDN[6][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_106 RTU0__XRDC.MRGDN[6][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_107 RTU0__XRDC.MRGDN[6][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_107 RTU0__XRDC.MRGDN[6][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_107 RTU0__XRDC.MRGDN[6][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_107 RTU0__XRDC.MRGDN[6][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_108 RTU0__XRDC.MRGDN[6][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_108 RTU0__XRDC.MRGDN[6][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_108 RTU0__XRDC.MRGDN[6][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_108 RTU0__XRDC.MRGDN[6][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_109 RTU0__XRDC.MRGDN[6][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_109 RTU0__XRDC.MRGDN[6][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_109 RTU0__XRDC.MRGDN[6][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_109 RTU0__XRDC.MRGDN[6][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_110 RTU0__XRDC.MRGDN[6][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_110 RTU0__XRDC.MRGDN[6][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_110 RTU0__XRDC.MRGDN[6][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_110 RTU0__XRDC.MRGDN[6][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_111 RTU0__XRDC.MRGDN[6][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_111 RTU0__XRDC.MRGDN[6][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_111 RTU0__XRDC.MRGDN[6][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_111 RTU0__XRDC.MRGDN[6][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_112 RTU0__XRDC.MRGDN[7][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_112 RTU0__XRDC.MRGDN[7][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_112 RTU0__XRDC.MRGDN[7][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_112 RTU0__XRDC.MRGDN[7][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_113 RTU0__XRDC.MRGDN[7][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_113 RTU0__XRDC.MRGDN[7][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_113 RTU0__XRDC.MRGDN[7][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_113 RTU0__XRDC.MRGDN[7][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_114 RTU0__XRDC.MRGDN[7][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_114 RTU0__XRDC.MRGDN[7][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_114 RTU0__XRDC.MRGDN[7][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_114 RTU0__XRDC.MRGDN[7][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_115 RTU0__XRDC.MRGDN[7][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_115 RTU0__XRDC.MRGDN[7][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_115 RTU0__XRDC.MRGDN[7][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_115 RTU0__XRDC.MRGDN[7][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_116 RTU0__XRDC.MRGDN[7][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_116 RTU0__XRDC.MRGDN[7][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_116 RTU0__XRDC.MRGDN[7][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_116 RTU0__XRDC.MRGDN[7][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_117 RTU0__XRDC.MRGDN[7][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_117 RTU0__XRDC.MRGDN[7][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_117 RTU0__XRDC.MRGDN[7][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_117 RTU0__XRDC.MRGDN[7][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_118 RTU0__XRDC.MRGDN[7][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_118 RTU0__XRDC.MRGDN[7][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_118 RTU0__XRDC.MRGDN[7][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_118 RTU0__XRDC.MRGDN[7][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_119 RTU0__XRDC.MRGDN[7][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_119 RTU0__XRDC.MRGDN[7][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_119 RTU0__XRDC.MRGDN[7][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_119 RTU0__XRDC.MRGDN[7][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_120 RTU0__XRDC.MRGDN[7][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_120 RTU0__XRDC.MRGDN[7][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_120 RTU0__XRDC.MRGDN[7][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_120 RTU0__XRDC.MRGDN[7][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_121 RTU0__XRDC.MRGDN[7][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_121 RTU0__XRDC.MRGDN[7][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_121 RTU0__XRDC.MRGDN[7][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_121 RTU0__XRDC.MRGDN[7][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_122 RTU0__XRDC.MRGDN[7][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_122 RTU0__XRDC.MRGDN[7][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_122 RTU0__XRDC.MRGDN[7][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_122 RTU0__XRDC.MRGDN[7][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_123 RTU0__XRDC.MRGDN[7][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_123 RTU0__XRDC.MRGDN[7][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_123 RTU0__XRDC.MRGDN[7][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_123 RTU0__XRDC.MRGDN[7][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_124 RTU0__XRDC.MRGDN[7][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_124 RTU0__XRDC.MRGDN[7][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_124 RTU0__XRDC.MRGDN[7][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_124 RTU0__XRDC.MRGDN[7][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_125 RTU0__XRDC.MRGDN[7][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_125 RTU0__XRDC.MRGDN[7][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_125 RTU0__XRDC.MRGDN[7][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_125 RTU0__XRDC.MRGDN[7][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_126 RTU0__XRDC.MRGDN[7][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_126 RTU0__XRDC.MRGDN[7][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_126 RTU0__XRDC.MRGDN[7][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_126 RTU0__XRDC.MRGDN[7][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_127 RTU0__XRDC.MRGDN[7][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_127 RTU0__XRDC.MRGDN[7][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_127 RTU0__XRDC.MRGDN[7][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_127 RTU0__XRDC.MRGDN[7][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_128 RTU0__XRDC.MRGDN[8][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_128 RTU0__XRDC.MRGDN[8][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_128 RTU0__XRDC.MRGDN[8][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_128 RTU0__XRDC.MRGDN[8][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_129 RTU0__XRDC.MRGDN[8][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_129 RTU0__XRDC.MRGDN[8][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_129 RTU0__XRDC.MRGDN[8][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_129 RTU0__XRDC.MRGDN[8][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_130 RTU0__XRDC.MRGDN[8][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_130 RTU0__XRDC.MRGDN[8][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_130 RTU0__XRDC.MRGDN[8][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_130 RTU0__XRDC.MRGDN[8][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_131 RTU0__XRDC.MRGDN[8][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_131 RTU0__XRDC.MRGDN[8][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_131 RTU0__XRDC.MRGDN[8][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_131 RTU0__XRDC.MRGDN[8][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_132 RTU0__XRDC.MRGDN[8][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_132 RTU0__XRDC.MRGDN[8][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_132 RTU0__XRDC.MRGDN[8][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_132 RTU0__XRDC.MRGDN[8][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_133 RTU0__XRDC.MRGDN[8][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_133 RTU0__XRDC.MRGDN[8][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_133 RTU0__XRDC.MRGDN[8][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_133 RTU0__XRDC.MRGDN[8][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_134 RTU0__XRDC.MRGDN[8][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_134 RTU0__XRDC.MRGDN[8][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_134 RTU0__XRDC.MRGDN[8][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_134 RTU0__XRDC.MRGDN[8][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_135 RTU0__XRDC.MRGDN[8][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_135 RTU0__XRDC.MRGDN[8][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_135 RTU0__XRDC.MRGDN[8][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_135 RTU0__XRDC.MRGDN[8][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_136 RTU0__XRDC.MRGDN[8][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_136 RTU0__XRDC.MRGDN[8][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_136 RTU0__XRDC.MRGDN[8][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_136 RTU0__XRDC.MRGDN[8][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_137 RTU0__XRDC.MRGDN[8][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_137 RTU0__XRDC.MRGDN[8][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_137 RTU0__XRDC.MRGDN[8][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_137 RTU0__XRDC.MRGDN[8][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_138 RTU0__XRDC.MRGDN[8][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_138 RTU0__XRDC.MRGDN[8][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_138 RTU0__XRDC.MRGDN[8][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_138 RTU0__XRDC.MRGDN[8][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_139 RTU0__XRDC.MRGDN[8][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_139 RTU0__XRDC.MRGDN[8][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_139 RTU0__XRDC.MRGDN[8][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_139 RTU0__XRDC.MRGDN[8][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_140 RTU0__XRDC.MRGDN[8][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_140 RTU0__XRDC.MRGDN[8][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_140 RTU0__XRDC.MRGDN[8][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_140 RTU0__XRDC.MRGDN[8][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_141 RTU0__XRDC.MRGDN[8][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_141 RTU0__XRDC.MRGDN[8][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_141 RTU0__XRDC.MRGDN[8][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_141 RTU0__XRDC.MRGDN[8][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_142 RTU0__XRDC.MRGDN[8][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_142 RTU0__XRDC.MRGDN[8][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_142 RTU0__XRDC.MRGDN[8][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_142 RTU0__XRDC.MRGDN[8][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_143 RTU0__XRDC.MRGDN[8][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_143 RTU0__XRDC.MRGDN[8][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_143 RTU0__XRDC.MRGDN[8][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_143 RTU0__XRDC.MRGDN[8][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_144 RTU0__XRDC.MRGDN[9][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_144 RTU0__XRDC.MRGDN[9][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_144 RTU0__XRDC.MRGDN[9][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_144 RTU0__XRDC.MRGDN[9][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_145 RTU0__XRDC.MRGDN[9][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_145 RTU0__XRDC.MRGDN[9][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_145 RTU0__XRDC.MRGDN[9][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_145 RTU0__XRDC.MRGDN[9][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_146 RTU0__XRDC.MRGDN[9][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_146 RTU0__XRDC.MRGDN[9][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_146 RTU0__XRDC.MRGDN[9][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_146 RTU0__XRDC.MRGDN[9][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_147 RTU0__XRDC.MRGDN[9][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_147 RTU0__XRDC.MRGDN[9][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_147 RTU0__XRDC.MRGDN[9][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_147 RTU0__XRDC.MRGDN[9][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_148 RTU0__XRDC.MRGDN[9][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_148 RTU0__XRDC.MRGDN[9][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_148 RTU0__XRDC.MRGDN[9][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_148 RTU0__XRDC.MRGDN[9][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_149 RTU0__XRDC.MRGDN[9][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_149 RTU0__XRDC.MRGDN[9][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_149 RTU0__XRDC.MRGDN[9][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_149 RTU0__XRDC.MRGDN[9][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_150 RTU0__XRDC.MRGDN[9][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_150 RTU0__XRDC.MRGDN[9][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_150 RTU0__XRDC.MRGDN[9][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_150 RTU0__XRDC.MRGDN[9][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_151 RTU0__XRDC.MRGDN[9][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_151 RTU0__XRDC.MRGDN[9][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_151 RTU0__XRDC.MRGDN[9][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_151 RTU0__XRDC.MRGDN[9][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_152 RTU0__XRDC.MRGDN[9][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_152 RTU0__XRDC.MRGDN[9][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_152 RTU0__XRDC.MRGDN[9][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_152 RTU0__XRDC.MRGDN[9][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_153 RTU0__XRDC.MRGDN[9][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_153 RTU0__XRDC.MRGDN[9][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_153 RTU0__XRDC.MRGDN[9][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_153 RTU0__XRDC.MRGDN[9][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_154 RTU0__XRDC.MRGDN[9][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_154 RTU0__XRDC.MRGDN[9][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_154 RTU0__XRDC.MRGDN[9][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_154 RTU0__XRDC.MRGDN[9][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_155 RTU0__XRDC.MRGDN[9][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_155 RTU0__XRDC.MRGDN[9][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_155 RTU0__XRDC.MRGDN[9][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_155 RTU0__XRDC.MRGDN[9][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_156 RTU0__XRDC.MRGDN[9][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_156 RTU0__XRDC.MRGDN[9][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_156 RTU0__XRDC.MRGDN[9][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_156 RTU0__XRDC.MRGDN[9][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_157 RTU0__XRDC.MRGDN[9][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_157 RTU0__XRDC.MRGDN[9][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_157 RTU0__XRDC.MRGDN[9][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_157 RTU0__XRDC.MRGDN[9][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_158 RTU0__XRDC.MRGDN[9][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_158 RTU0__XRDC.MRGDN[9][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_158 RTU0__XRDC.MRGDN[9][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_158 RTU0__XRDC.MRGDN[9][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_159 RTU0__XRDC.MRGDN[9][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_159 RTU0__XRDC.MRGDN[9][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_159 RTU0__XRDC.MRGDN[9][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_159 RTU0__XRDC.MRGDN[9][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_160 RTU0__XRDC.MRGDN[10][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_160 RTU0__XRDC.MRGDN[10][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_160 RTU0__XRDC.MRGDN[10][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_160 RTU0__XRDC.MRGDN[10][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_161 RTU0__XRDC.MRGDN[10][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_161 RTU0__XRDC.MRGDN[10][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_161 RTU0__XRDC.MRGDN[10][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_161 RTU0__XRDC.MRGDN[10][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_162 RTU0__XRDC.MRGDN[10][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_162 RTU0__XRDC.MRGDN[10][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_162 RTU0__XRDC.MRGDN[10][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_162 RTU0__XRDC.MRGDN[10][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_163 RTU0__XRDC.MRGDN[10][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_163 RTU0__XRDC.MRGDN[10][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_163 RTU0__XRDC.MRGDN[10][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_163 RTU0__XRDC.MRGDN[10][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_164 RTU0__XRDC.MRGDN[10][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_164 RTU0__XRDC.MRGDN[10][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_164 RTU0__XRDC.MRGDN[10][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_164 RTU0__XRDC.MRGDN[10][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_165 RTU0__XRDC.MRGDN[10][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_165 RTU0__XRDC.MRGDN[10][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_165 RTU0__XRDC.MRGDN[10][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_165 RTU0__XRDC.MRGDN[10][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_166 RTU0__XRDC.MRGDN[10][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_166 RTU0__XRDC.MRGDN[10][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_166 RTU0__XRDC.MRGDN[10][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_166 RTU0__XRDC.MRGDN[10][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_167 RTU0__XRDC.MRGDN[10][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_167 RTU0__XRDC.MRGDN[10][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_167 RTU0__XRDC.MRGDN[10][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_167 RTU0__XRDC.MRGDN[10][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_168 RTU0__XRDC.MRGDN[10][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_168 RTU0__XRDC.MRGDN[10][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_168 RTU0__XRDC.MRGDN[10][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_168 RTU0__XRDC.MRGDN[10][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_169 RTU0__XRDC.MRGDN[10][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_169 RTU0__XRDC.MRGDN[10][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_169 RTU0__XRDC.MRGDN[10][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_169 RTU0__XRDC.MRGDN[10][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_170 RTU0__XRDC.MRGDN[10][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_170 RTU0__XRDC.MRGDN[10][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_170 RTU0__XRDC.MRGDN[10][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_170 RTU0__XRDC.MRGDN[10][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_171 RTU0__XRDC.MRGDN[10][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_171 RTU0__XRDC.MRGDN[10][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_171 RTU0__XRDC.MRGDN[10][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_171 RTU0__XRDC.MRGDN[10][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_172 RTU0__XRDC.MRGDN[10][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_172 RTU0__XRDC.MRGDN[10][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_172 RTU0__XRDC.MRGDN[10][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_172 RTU0__XRDC.MRGDN[10][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_173 RTU0__XRDC.MRGDN[10][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_173 RTU0__XRDC.MRGDN[10][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_173 RTU0__XRDC.MRGDN[10][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_173 RTU0__XRDC.MRGDN[10][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_174 RTU0__XRDC.MRGDN[10][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_174 RTU0__XRDC.MRGDN[10][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_174 RTU0__XRDC.MRGDN[10][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_174 RTU0__XRDC.MRGDN[10][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_175 RTU0__XRDC.MRGDN[10][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_175 RTU0__XRDC.MRGDN[10][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_175 RTU0__XRDC.MRGDN[10][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_175 RTU0__XRDC.MRGDN[10][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_176 RTU0__XRDC.MRGDN[11][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_176 RTU0__XRDC.MRGDN[11][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_176 RTU0__XRDC.MRGDN[11][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_176 RTU0__XRDC.MRGDN[11][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_177 RTU0__XRDC.MRGDN[11][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_177 RTU0__XRDC.MRGDN[11][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_177 RTU0__XRDC.MRGDN[11][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_177 RTU0__XRDC.MRGDN[11][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_178 RTU0__XRDC.MRGDN[11][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_178 RTU0__XRDC.MRGDN[11][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_178 RTU0__XRDC.MRGDN[11][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_178 RTU0__XRDC.MRGDN[11][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_179 RTU0__XRDC.MRGDN[11][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_179 RTU0__XRDC.MRGDN[11][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_179 RTU0__XRDC.MRGDN[11][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_179 RTU0__XRDC.MRGDN[11][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_180 RTU0__XRDC.MRGDN[11][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_180 RTU0__XRDC.MRGDN[11][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_180 RTU0__XRDC.MRGDN[11][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_180 RTU0__XRDC.MRGDN[11][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_181 RTU0__XRDC.MRGDN[11][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_181 RTU0__XRDC.MRGDN[11][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_181 RTU0__XRDC.MRGDN[11][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_181 RTU0__XRDC.MRGDN[11][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_182 RTU0__XRDC.MRGDN[11][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_182 RTU0__XRDC.MRGDN[11][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_182 RTU0__XRDC.MRGDN[11][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_182 RTU0__XRDC.MRGDN[11][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_183 RTU0__XRDC.MRGDN[11][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_183 RTU0__XRDC.MRGDN[11][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_183 RTU0__XRDC.MRGDN[11][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_183 RTU0__XRDC.MRGDN[11][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_184 RTU0__XRDC.MRGDN[11][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_184 RTU0__XRDC.MRGDN[11][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_184 RTU0__XRDC.MRGDN[11][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_184 RTU0__XRDC.MRGDN[11][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_185 RTU0__XRDC.MRGDN[11][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_185 RTU0__XRDC.MRGDN[11][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_185 RTU0__XRDC.MRGDN[11][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_185 RTU0__XRDC.MRGDN[11][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_186 RTU0__XRDC.MRGDN[11][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_186 RTU0__XRDC.MRGDN[11][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_186 RTU0__XRDC.MRGDN[11][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_186 RTU0__XRDC.MRGDN[11][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_187 RTU0__XRDC.MRGDN[11][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_187 RTU0__XRDC.MRGDN[11][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_187 RTU0__XRDC.MRGDN[11][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_187 RTU0__XRDC.MRGDN[11][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_188 RTU0__XRDC.MRGDN[11][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_188 RTU0__XRDC.MRGDN[11][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_188 RTU0__XRDC.MRGDN[11][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_188 RTU0__XRDC.MRGDN[11][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_189 RTU0__XRDC.MRGDN[11][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_189 RTU0__XRDC.MRGDN[11][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_189 RTU0__XRDC.MRGDN[11][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_189 RTU0__XRDC.MRGDN[11][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_190 RTU0__XRDC.MRGDN[11][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_190 RTU0__XRDC.MRGDN[11][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_190 RTU0__XRDC.MRGDN[11][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_190 RTU0__XRDC.MRGDN[11][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_191 RTU0__XRDC.MRGDN[11][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_191 RTU0__XRDC.MRGDN[11][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_191 RTU0__XRDC.MRGDN[11][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_191 RTU0__XRDC.MRGDN[11][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_192 RTU0__XRDC.MRGDN[12][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_192 RTU0__XRDC.MRGDN[12][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_192 RTU0__XRDC.MRGDN[12][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_192 RTU0__XRDC.MRGDN[12][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_193 RTU0__XRDC.MRGDN[12][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_193 RTU0__XRDC.MRGDN[12][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_193 RTU0__XRDC.MRGDN[12][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_193 RTU0__XRDC.MRGDN[12][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_194 RTU0__XRDC.MRGDN[12][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_194 RTU0__XRDC.MRGDN[12][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_194 RTU0__XRDC.MRGDN[12][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_194 RTU0__XRDC.MRGDN[12][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_195 RTU0__XRDC.MRGDN[12][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_195 RTU0__XRDC.MRGDN[12][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_195 RTU0__XRDC.MRGDN[12][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_195 RTU0__XRDC.MRGDN[12][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_196 RTU0__XRDC.MRGDN[12][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_196 RTU0__XRDC.MRGDN[12][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_196 RTU0__XRDC.MRGDN[12][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_196 RTU0__XRDC.MRGDN[12][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_197 RTU0__XRDC.MRGDN[12][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_197 RTU0__XRDC.MRGDN[12][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_197 RTU0__XRDC.MRGDN[12][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_197 RTU0__XRDC.MRGDN[12][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_198 RTU0__XRDC.MRGDN[12][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_198 RTU0__XRDC.MRGDN[12][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_198 RTU0__XRDC.MRGDN[12][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_198 RTU0__XRDC.MRGDN[12][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_199 RTU0__XRDC.MRGDN[12][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_199 RTU0__XRDC.MRGDN[12][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_199 RTU0__XRDC.MRGDN[12][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_199 RTU0__XRDC.MRGDN[12][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_200 RTU0__XRDC.MRGDN[12][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_200 RTU0__XRDC.MRGDN[12][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_200 RTU0__XRDC.MRGDN[12][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_200 RTU0__XRDC.MRGDN[12][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_201 RTU0__XRDC.MRGDN[12][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_201 RTU0__XRDC.MRGDN[12][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_201 RTU0__XRDC.MRGDN[12][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_201 RTU0__XRDC.MRGDN[12][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_202 RTU0__XRDC.MRGDN[12][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_202 RTU0__XRDC.MRGDN[12][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_202 RTU0__XRDC.MRGDN[12][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_202 RTU0__XRDC.MRGDN[12][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_203 RTU0__XRDC.MRGDN[12][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_203 RTU0__XRDC.MRGDN[12][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_203 RTU0__XRDC.MRGDN[12][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_203 RTU0__XRDC.MRGDN[12][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_204 RTU0__XRDC.MRGDN[12][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_204 RTU0__XRDC.MRGDN[12][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_204 RTU0__XRDC.MRGDN[12][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_204 RTU0__XRDC.MRGDN[12][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_205 RTU0__XRDC.MRGDN[12][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_205 RTU0__XRDC.MRGDN[12][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_205 RTU0__XRDC.MRGDN[12][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_205 RTU0__XRDC.MRGDN[12][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_206 RTU0__XRDC.MRGDN[12][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_206 RTU0__XRDC.MRGDN[12][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_206 RTU0__XRDC.MRGDN[12][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_206 RTU0__XRDC.MRGDN[12][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_207 RTU0__XRDC.MRGDN[12][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_207 RTU0__XRDC.MRGDN[12][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_207 RTU0__XRDC.MRGDN[12][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_207 RTU0__XRDC.MRGDN[12][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_208 RTU0__XRDC.MRGDN[13][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_208 RTU0__XRDC.MRGDN[13][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_208 RTU0__XRDC.MRGDN[13][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_208 RTU0__XRDC.MRGDN[13][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_209 RTU0__XRDC.MRGDN[13][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_209 RTU0__XRDC.MRGDN[13][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_209 RTU0__XRDC.MRGDN[13][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_209 RTU0__XRDC.MRGDN[13][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_210 RTU0__XRDC.MRGDN[13][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_210 RTU0__XRDC.MRGDN[13][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_210 RTU0__XRDC.MRGDN[13][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_210 RTU0__XRDC.MRGDN[13][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_211 RTU0__XRDC.MRGDN[13][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_211 RTU0__XRDC.MRGDN[13][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_211 RTU0__XRDC.MRGDN[13][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_211 RTU0__XRDC.MRGDN[13][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_212 RTU0__XRDC.MRGDN[13][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_212 RTU0__XRDC.MRGDN[13][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_212 RTU0__XRDC.MRGDN[13][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_212 RTU0__XRDC.MRGDN[13][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_213 RTU0__XRDC.MRGDN[13][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_213 RTU0__XRDC.MRGDN[13][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_213 RTU0__XRDC.MRGDN[13][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_213 RTU0__XRDC.MRGDN[13][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_214 RTU0__XRDC.MRGDN[13][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_214 RTU0__XRDC.MRGDN[13][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_214 RTU0__XRDC.MRGDN[13][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_214 RTU0__XRDC.MRGDN[13][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_215 RTU0__XRDC.MRGDN[13][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_215 RTU0__XRDC.MRGDN[13][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_215 RTU0__XRDC.MRGDN[13][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_215 RTU0__XRDC.MRGDN[13][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_216 RTU0__XRDC.MRGDN[13][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_216 RTU0__XRDC.MRGDN[13][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_216 RTU0__XRDC.MRGDN[13][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_216 RTU0__XRDC.MRGDN[13][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_217 RTU0__XRDC.MRGDN[13][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_217 RTU0__XRDC.MRGDN[13][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_217 RTU0__XRDC.MRGDN[13][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_217 RTU0__XRDC.MRGDN[13][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_218 RTU0__XRDC.MRGDN[13][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_218 RTU0__XRDC.MRGDN[13][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_218 RTU0__XRDC.MRGDN[13][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_218 RTU0__XRDC.MRGDN[13][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_219 RTU0__XRDC.MRGDN[13][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_219 RTU0__XRDC.MRGDN[13][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_219 RTU0__XRDC.MRGDN[13][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_219 RTU0__XRDC.MRGDN[13][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_220 RTU0__XRDC.MRGDN[13][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_220 RTU0__XRDC.MRGDN[13][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_220 RTU0__XRDC.MRGDN[13][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_220 RTU0__XRDC.MRGDN[13][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_221 RTU0__XRDC.MRGDN[13][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_221 RTU0__XRDC.MRGDN[13][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_221 RTU0__XRDC.MRGDN[13][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_221 RTU0__XRDC.MRGDN[13][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_222 RTU0__XRDC.MRGDN[13][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_222 RTU0__XRDC.MRGDN[13][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_222 RTU0__XRDC.MRGDN[13][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_222 RTU0__XRDC.MRGDN[13][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_223 RTU0__XRDC.MRGDN[13][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_223 RTU0__XRDC.MRGDN[13][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_223 RTU0__XRDC.MRGDN[13][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_223 RTU0__XRDC.MRGDN[13][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_224 RTU0__XRDC.MRGDN[14][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_224 RTU0__XRDC.MRGDN[14][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_224 RTU0__XRDC.MRGDN[14][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_224 RTU0__XRDC.MRGDN[14][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_225 RTU0__XRDC.MRGDN[14][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_225 RTU0__XRDC.MRGDN[14][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_225 RTU0__XRDC.MRGDN[14][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_225 RTU0__XRDC.MRGDN[14][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_226 RTU0__XRDC.MRGDN[14][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_226 RTU0__XRDC.MRGDN[14][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_226 RTU0__XRDC.MRGDN[14][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_226 RTU0__XRDC.MRGDN[14][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_227 RTU0__XRDC.MRGDN[14][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_227 RTU0__XRDC.MRGDN[14][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_227 RTU0__XRDC.MRGDN[14][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_227 RTU0__XRDC.MRGDN[14][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_228 RTU0__XRDC.MRGDN[14][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_228 RTU0__XRDC.MRGDN[14][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_228 RTU0__XRDC.MRGDN[14][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_228 RTU0__XRDC.MRGDN[14][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_229 RTU0__XRDC.MRGDN[14][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_229 RTU0__XRDC.MRGDN[14][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_229 RTU0__XRDC.MRGDN[14][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_229 RTU0__XRDC.MRGDN[14][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_230 RTU0__XRDC.MRGDN[14][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_230 RTU0__XRDC.MRGDN[14][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_230 RTU0__XRDC.MRGDN[14][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_230 RTU0__XRDC.MRGDN[14][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_231 RTU0__XRDC.MRGDN[14][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_231 RTU0__XRDC.MRGDN[14][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_231 RTU0__XRDC.MRGDN[14][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_231 RTU0__XRDC.MRGDN[14][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_232 RTU0__XRDC.MRGDN[14][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_232 RTU0__XRDC.MRGDN[14][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_232 RTU0__XRDC.MRGDN[14][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_232 RTU0__XRDC.MRGDN[14][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_233 RTU0__XRDC.MRGDN[14][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_233 RTU0__XRDC.MRGDN[14][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_233 RTU0__XRDC.MRGDN[14][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_233 RTU0__XRDC.MRGDN[14][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_234 RTU0__XRDC.MRGDN[14][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_234 RTU0__XRDC.MRGDN[14][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_234 RTU0__XRDC.MRGDN[14][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_234 RTU0__XRDC.MRGDN[14][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_235 RTU0__XRDC.MRGDN[14][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_235 RTU0__XRDC.MRGDN[14][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_235 RTU0__XRDC.MRGDN[14][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_235 RTU0__XRDC.MRGDN[14][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_236 RTU0__XRDC.MRGDN[14][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_236 RTU0__XRDC.MRGDN[14][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_236 RTU0__XRDC.MRGDN[14][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_236 RTU0__XRDC.MRGDN[14][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_237 RTU0__XRDC.MRGDN[14][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_237 RTU0__XRDC.MRGDN[14][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_237 RTU0__XRDC.MRGDN[14][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_237 RTU0__XRDC.MRGDN[14][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_238 RTU0__XRDC.MRGDN[14][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_238 RTU0__XRDC.MRGDN[14][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_238 RTU0__XRDC.MRGDN[14][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_238 RTU0__XRDC.MRGDN[14][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU0__XRDC_MRGD_W0_239 RTU0__XRDC.MRGDN[14][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU0__XRDC_MRGD_W1_239 RTU0__XRDC.MRGDN[14][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU0__XRDC_MRGD_W2_239 RTU0__XRDC.MRGDN[14][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU0__XRDC_MRGD_W3_239 RTU0__XRDC.MRGDN[14][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

#define RTU1__XRDC_CR        RTU1__XRDC.CR.R               /* Control */
#define RTU1__XRDC_HWCFG0    RTU1__XRDC.HWCFG0.R           /* Hardware Configuration 0 */
#define RTU1__XRDC_HWCFG1    RTU1__XRDC.HWCFG1.R           /* Hardware Configuration 1 */
#define RTU1__XRDC_MDACFG0   RTU1__XRDC.MDACFG[0].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG1   RTU1__XRDC.MDACFG[1].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG2   RTU1__XRDC.MDACFG[2].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG3   RTU1__XRDC.MDACFG[3].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG4   RTU1__XRDC.MDACFG[4].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG5   RTU1__XRDC.MDACFG[5].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG6   RTU1__XRDC.MDACFG[6].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG7   RTU1__XRDC.MDACFG[7].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG8   RTU1__XRDC.MDACFG[8].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG9   RTU1__XRDC.MDACFG[9].R        /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG10  RTU1__XRDC.MDACFG[10].R       /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG11  RTU1__XRDC.MDACFG[11].R       /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MDACFG12  RTU1__XRDC.MDACFG[12].R       /* Master Domain Assignment Configuration */
#define RTU1__XRDC_MRCFG0    RTU1__XRDC.MRCFG[0].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG1    RTU1__XRDC.MRCFG[1].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG2    RTU1__XRDC.MRCFG[2].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG3    RTU1__XRDC.MRCFG[3].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG4    RTU1__XRDC.MRCFG[4].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG5    RTU1__XRDC.MRCFG[5].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG6    RTU1__XRDC.MRCFG[6].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG7    RTU1__XRDC.MRCFG[7].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG8    RTU1__XRDC.MRCFG[8].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG9    RTU1__XRDC.MRCFG[9].R         /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG10   RTU1__XRDC.MRCFG[10].R        /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG11   RTU1__XRDC.MRCFG[11].R        /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG12   RTU1__XRDC.MRCFG[12].R        /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG13   RTU1__XRDC.MRCFG[13].R        /* Memory Region Configuration */
#define RTU1__XRDC_MRCFG14   RTU1__XRDC.MRCFG[14].R        /* Memory Region Configuration */
#define RTU1__XRDC_DERRLOC0  RTU1__XRDC.DERRLOC[0].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC1  RTU1__XRDC.DERRLOC[1].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC2  RTU1__XRDC.DERRLOC[2].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC3  RTU1__XRDC.DERRLOC[3].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC4  RTU1__XRDC.DERRLOC[4].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC5  RTU1__XRDC.DERRLOC[5].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC6  RTU1__XRDC.DERRLOC[6].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC7  RTU1__XRDC.DERRLOC[7].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC8  RTU1__XRDC.DERRLOC[8].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC9  RTU1__XRDC.DERRLOC[9].R       /* Domain Error Location */
#define RTU1__XRDC_DERRLOC10 RTU1__XRDC.DERRLOC[10].R      /* Domain Error Location */
#define RTU1__XRDC_DERRLOC11 RTU1__XRDC.DERRLOC[11].R      /* Domain Error Location */
#define RTU1__XRDC_DERRLOC12 RTU1__XRDC.DERRLOC[12].R      /* Domain Error Location */
#define RTU1__XRDC_DERRLOC13 RTU1__XRDC.DERRLOC[13].R      /* Domain Error Location */
#define RTU1__XRDC_DERRLOC14 RTU1__XRDC.DERRLOC[14].R      /* Domain Error Location */
#define RTU1__XRDC_DERRLOC15 RTU1__XRDC.DERRLOC[15].R      /* Domain Error Location */
#define RTU1__XRDC_DERR_W0_0 RTU1__XRDC.DERRW0[0].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_0 RTU1__XRDC.DERRW0[0].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_0 RTU1__XRDC.DERRW0[0].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_1 RTU1__XRDC.DERRW0[1].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_1 RTU1__XRDC.DERRW0[1].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_1 RTU1__XRDC.DERRW0[1].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_2 RTU1__XRDC.DERRW0[2].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_2 RTU1__XRDC.DERRW0[2].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_2 RTU1__XRDC.DERRW0[2].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_3 RTU1__XRDC.DERRW0[3].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_3 RTU1__XRDC.DERRW0[3].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_3 RTU1__XRDC.DERRW0[3].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_4 RTU1__XRDC.DERRW0[4].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_4 RTU1__XRDC.DERRW0[4].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_4 RTU1__XRDC.DERRW0[4].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_5 RTU1__XRDC.DERRW0[5].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_5 RTU1__XRDC.DERRW0[5].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_5 RTU1__XRDC.DERRW0[5].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_6 RTU1__XRDC.DERRW0[6].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_6 RTU1__XRDC.DERRW0[6].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_6 RTU1__XRDC.DERRW0[6].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_7 RTU1__XRDC.DERRW0[7].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_7 RTU1__XRDC.DERRW0[7].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_7 RTU1__XRDC.DERRW0[7].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_8 RTU1__XRDC.DERRW0[8].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_8 RTU1__XRDC.DERRW0[8].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_8 RTU1__XRDC.DERRW0[8].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_9 RTU1__XRDC.DERRW0[9].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_9 RTU1__XRDC.DERRW0[9].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_9 RTU1__XRDC.DERRW0[9].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_10 RTU1__XRDC.DERRW0[10].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_10 RTU1__XRDC.DERRW0[10].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_10 RTU1__XRDC.DERRW0[10].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_11 RTU1__XRDC.DERRW0[11].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_11 RTU1__XRDC.DERRW0[11].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_11 RTU1__XRDC.DERRW0[11].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_12 RTU1__XRDC.DERRW0[12].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_12 RTU1__XRDC.DERRW0[12].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_12 RTU1__XRDC.DERRW0[12].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_13 RTU1__XRDC.DERRW0[13].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_13 RTU1__XRDC.DERRW0[13].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_13 RTU1__XRDC.DERRW0[13].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_14 RTU1__XRDC.DERRW0[14].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_14 RTU1__XRDC.DERRW0[14].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_14 RTU1__XRDC.DERRW0[14].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_16 RTU1__XRDC.DERRW0[16].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_16 RTU1__XRDC.DERRW0[16].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_16 RTU1__XRDC.DERRW0[16].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_17 RTU1__XRDC.DERRW0[17].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_17 RTU1__XRDC.DERRW0[17].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_17 RTU1__XRDC.DERRW0[17].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_DERR_W0_18 RTU1__XRDC.DERRW0[18].DERR_W0.R /* Domain Error Word 0 */
#define RTU1__XRDC_DERR_W1_18 RTU1__XRDC.DERRW0[18].DERR_W1.R /* Domain Error Word 1 */
#define RTU1__XRDC_DERR_W3_18 RTU1__XRDC.DERRW0[18].DERR_W3.R /* Domain Error Word 3 */
#define RTU1__XRDC_MDA_W0_0_DFMT1 RTU1__XRDC.MDA_W0_0_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_1_DFMT1 RTU1__XRDC.MDA_W0_1_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_2_DFMT1 RTU1__XRDC.MDA_W0_2_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_3_DFMT1 RTU1__XRDC.MDA_W0_3_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_4_DFMT1 RTU1__XRDC.MDA_W0_4_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W1_4_DFMT1 RTU1__XRDC.MDA_W1_4_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W2_4_DFMT1 RTU1__XRDC.MDA_W2_4_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W3_4_DFMT1 RTU1__XRDC.MDA_W3_4_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W4_4_DFMT1 RTU1__XRDC.MDA_W4_4_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W5_4_DFMT1 RTU1__XRDC.MDA_W5_4_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W6_4_DFMT1 RTU1__XRDC.MDA_W6_4_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W7_4_DFMT1 RTU1__XRDC.MDA_W7_4_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_5_DFMT1 RTU1__XRDC.MDA_W0_5_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W1_5_DFMT1 RTU1__XRDC.MDA_W1_5_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W2_5_DFMT1 RTU1__XRDC.MDA_W2_5_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W3_5_DFMT1 RTU1__XRDC.MDA_W3_5_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W4_5_DFMT1 RTU1__XRDC.MDA_W4_5_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W5_5_DFMT1 RTU1__XRDC.MDA_W5_5_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W6_5_DFMT1 RTU1__XRDC.MDA_W6_5_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W7_5_DFMT1 RTU1__XRDC.MDA_W7_5_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_6_DFMT1 RTU1__XRDC.MDA_W0_6_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W1_6_DFMT1 RTU1__XRDC.MDA_W1_6_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W2_6_DFMT1 RTU1__XRDC.MDA_W2_6_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W3_6_DFMT1 RTU1__XRDC.MDA_W3_6_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W4_6_DFMT1 RTU1__XRDC.MDA_W4_6_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W5_6_DFMT1 RTU1__XRDC.MDA_W5_6_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W6_6_DFMT1 RTU1__XRDC.MDA_W6_6_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W7_6_DFMT1 RTU1__XRDC.MDA_W7_6_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_7_DFMT1 RTU1__XRDC.MDA_W0_7_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W1_7_DFMT1 RTU1__XRDC.MDA_W1_7_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W2_7_DFMT1 RTU1__XRDC.MDA_W2_7_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W3_7_DFMT1 RTU1__XRDC.MDA_W3_7_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W4_7_DFMT1 RTU1__XRDC.MDA_W4_7_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W5_7_DFMT1 RTU1__XRDC.MDA_W5_7_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W6_7_DFMT1 RTU1__XRDC.MDA_W6_7_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W7_7_DFMT1 RTU1__XRDC.MDA_W7_7_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_8_DFMT1 RTU1__XRDC.MDA_W0_8_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W1_8_DFMT1 RTU1__XRDC.MDA_W1_8_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W2_8_DFMT1 RTU1__XRDC.MDA_W2_8_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W3_8_DFMT1 RTU1__XRDC.MDA_W3_8_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W4_8_DFMT1 RTU1__XRDC.MDA_W4_8_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W5_8_DFMT1 RTU1__XRDC.MDA_W5_8_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W6_8_DFMT1 RTU1__XRDC.MDA_W6_8_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W7_8_DFMT1 RTU1__XRDC.MDA_W7_8_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_9_DFMT1 RTU1__XRDC.MDA_W0_9_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W1_9_DFMT1 RTU1__XRDC.MDA_W1_9_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W2_9_DFMT1 RTU1__XRDC.MDA_W2_9_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W3_9_DFMT1 RTU1__XRDC.MDA_W3_9_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W4_9_DFMT1 RTU1__XRDC.MDA_W4_9_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W5_9_DFMT1 RTU1__XRDC.MDA_W5_9_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W6_9_DFMT1 RTU1__XRDC.MDA_W6_9_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W7_9_DFMT1 RTU1__XRDC.MDA_W7_9_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_10_DFMT1 RTU1__XRDC.MDA_W0_10_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W1_10_DFMT1 RTU1__XRDC.MDA_W1_10_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W2_10_DFMT1 RTU1__XRDC.MDA_W2_10_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W3_10_DFMT1 RTU1__XRDC.MDA_W3_10_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W4_10_DFMT1 RTU1__XRDC.MDA_W4_10_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W5_10_DFMT1 RTU1__XRDC.MDA_W5_10_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W6_10_DFMT1 RTU1__XRDC.MDA_W6_10_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W7_10_DFMT1 RTU1__XRDC.MDA_W7_10_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_11_DFMT1 RTU1__XRDC.MDA_W0_11_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W1_11_DFMT1 RTU1__XRDC.MDA_W1_11_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W2_11_DFMT1 RTU1__XRDC.MDA_W2_11_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W3_11_DFMT1 RTU1__XRDC.MDA_W3_11_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W4_11_DFMT1 RTU1__XRDC.MDA_W4_11_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W5_11_DFMT1 RTU1__XRDC.MDA_W5_11_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W6_11_DFMT1 RTU1__XRDC.MDA_W6_11_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W7_11_DFMT1 RTU1__XRDC.MDA_W7_11_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_MDA_W0_12_DFMT1 RTU1__XRDC.MDA_W0_12_DFMT1.R /* Master Domain Assignment */
#define RTU1__XRDC_PDAC_W0_0 RTU1__XRDC.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_0 RTU1__XRDC.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_1 RTU1__XRDC.PDAC_SLOT[0].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_1 RTU1__XRDC.PDAC_SLOT[0].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_2 RTU1__XRDC.PDAC_SLOT[0].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_2 RTU1__XRDC.PDAC_SLOT[0].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_3 RTU1__XRDC.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_3 RTU1__XRDC.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_4 RTU1__XRDC.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_4 RTU1__XRDC.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_5 RTU1__XRDC.PDAC_SLOT[0].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_5 RTU1__XRDC.PDAC_SLOT[0].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_6 RTU1__XRDC.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_6 RTU1__XRDC.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_7 RTU1__XRDC.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_7 RTU1__XRDC.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_8 RTU1__XRDC.PDAC_SLOT[0].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_8 RTU1__XRDC.PDAC_SLOT[0].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_9 RTU1__XRDC.PDAC_SLOT[0].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_9 RTU1__XRDC.PDAC_SLOT[0].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_10 RTU1__XRDC.PDAC_SLOT[0].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_10 RTU1__XRDC.PDAC_SLOT[0].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_11 RTU1__XRDC.PDAC_SLOT[0].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_11 RTU1__XRDC.PDAC_SLOT[0].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_12 RTU1__XRDC.PDAC_SLOT[0].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_12 RTU1__XRDC.PDAC_SLOT[0].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_13 RTU1__XRDC.PDAC_SLOT[0].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_13 RTU1__XRDC.PDAC_SLOT[0].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_14 RTU1__XRDC.PDAC_SLOT[0].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_14 RTU1__XRDC.PDAC_SLOT[0].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_15 RTU1__XRDC.PDAC_SLOT[0].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_15 RTU1__XRDC.PDAC_SLOT[0].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_18 RTU1__XRDC.PDAC_SLOT[0].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_18 RTU1__XRDC.PDAC_SLOT[0].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_19 RTU1__XRDC.PDAC_SLOT[0].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_19 RTU1__XRDC.PDAC_SLOT[0].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_20 RTU1__XRDC.PDAC_SLOT[0].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_20 RTU1__XRDC.PDAC_SLOT[0].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_21 RTU1__XRDC.PDAC_SLOT[0].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_21 RTU1__XRDC.PDAC_SLOT[0].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_22 RTU1__XRDC.PDAC_SLOT[0].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_22 RTU1__XRDC.PDAC_SLOT[0].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_23 RTU1__XRDC.PDAC_SLOT[0].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_23 RTU1__XRDC.PDAC_SLOT[0].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_24 RTU1__XRDC.PDAC_SLOT[0].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_24 RTU1__XRDC.PDAC_SLOT[0].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_25 RTU1__XRDC.PDAC_SLOT[0].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_25 RTU1__XRDC.PDAC_SLOT[0].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_26 RTU1__XRDC.PDAC_SLOT[0].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_26 RTU1__XRDC.PDAC_SLOT[0].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_27 RTU1__XRDC.PDAC_SLOT[0].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_27 RTU1__XRDC.PDAC_SLOT[0].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_28 RTU1__XRDC.PDAC_SLOT[0].PDACN[28].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_28 RTU1__XRDC.PDAC_SLOT[0].PDACN[28].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_29 RTU1__XRDC.PDAC_SLOT[0].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_29 RTU1__XRDC.PDAC_SLOT[0].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_31 RTU1__XRDC.PDAC_SLOT[0].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_31 RTU1__XRDC.PDAC_SLOT[0].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_32 RTU1__XRDC.PDAC_SLOT[0].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_32 RTU1__XRDC.PDAC_SLOT[0].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_33 RTU1__XRDC.PDAC_SLOT[0].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_33 RTU1__XRDC.PDAC_SLOT[0].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_34 RTU1__XRDC.PDAC_SLOT[0].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_34 RTU1__XRDC.PDAC_SLOT[0].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_35 RTU1__XRDC.PDAC_SLOT[0].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_35 RTU1__XRDC.PDAC_SLOT[0].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_36 RTU1__XRDC.PDAC_SLOT[0].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_36 RTU1__XRDC.PDAC_SLOT[0].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_37 RTU1__XRDC.PDAC_SLOT[0].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_37 RTU1__XRDC.PDAC_SLOT[0].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_38 RTU1__XRDC.PDAC_SLOT[0].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_38 RTU1__XRDC.PDAC_SLOT[0].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_39 RTU1__XRDC.PDAC_SLOT[0].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_39 RTU1__XRDC.PDAC_SLOT[0].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_40 RTU1__XRDC.PDAC_SLOT[0].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_40 RTU1__XRDC.PDAC_SLOT[0].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_41 RTU1__XRDC.PDAC_SLOT[0].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_41 RTU1__XRDC.PDAC_SLOT[0].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_42 RTU1__XRDC.PDAC_SLOT[0].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_42 RTU1__XRDC.PDAC_SLOT[0].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_128 RTU1__XRDC.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_128 RTU1__XRDC.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_129 RTU1__XRDC.PDAC_SLOT[1].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_129 RTU1__XRDC.PDAC_SLOT[1].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_130 RTU1__XRDC.PDAC_SLOT[1].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_130 RTU1__XRDC.PDAC_SLOT[1].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_131 RTU1__XRDC.PDAC_SLOT[1].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_131 RTU1__XRDC.PDAC_SLOT[1].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_132 RTU1__XRDC.PDAC_SLOT[1].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_132 RTU1__XRDC.PDAC_SLOT[1].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_133 RTU1__XRDC.PDAC_SLOT[1].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_133 RTU1__XRDC.PDAC_SLOT[1].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_134 RTU1__XRDC.PDAC_SLOT[1].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_134 RTU1__XRDC.PDAC_SLOT[1].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_135 RTU1__XRDC.PDAC_SLOT[1].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_135 RTU1__XRDC.PDAC_SLOT[1].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_136 RTU1__XRDC.PDAC_SLOT[1].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_136 RTU1__XRDC.PDAC_SLOT[1].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_138 RTU1__XRDC.PDAC_SLOT[1].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_138 RTU1__XRDC.PDAC_SLOT[1].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_139 RTU1__XRDC.PDAC_SLOT[1].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_139 RTU1__XRDC.PDAC_SLOT[1].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_140 RTU1__XRDC.PDAC_SLOT[1].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_140 RTU1__XRDC.PDAC_SLOT[1].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_141 RTU1__XRDC.PDAC_SLOT[1].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_141 RTU1__XRDC.PDAC_SLOT[1].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_142 RTU1__XRDC.PDAC_SLOT[1].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_142 RTU1__XRDC.PDAC_SLOT[1].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_143 RTU1__XRDC.PDAC_SLOT[1].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_143 RTU1__XRDC.PDAC_SLOT[1].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_144 RTU1__XRDC.PDAC_SLOT[1].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_144 RTU1__XRDC.PDAC_SLOT[1].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_145 RTU1__XRDC.PDAC_SLOT[1].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_145 RTU1__XRDC.PDAC_SLOT[1].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_149 RTU1__XRDC.PDAC_SLOT[1].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_149 RTU1__XRDC.PDAC_SLOT[1].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_150 RTU1__XRDC.PDAC_SLOT[1].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_150 RTU1__XRDC.PDAC_SLOT[1].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_151 RTU1__XRDC.PDAC_SLOT[1].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_151 RTU1__XRDC.PDAC_SLOT[1].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_152 RTU1__XRDC.PDAC_SLOT[1].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_152 RTU1__XRDC.PDAC_SLOT[1].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_256 RTU1__XRDC.PDAC_SLOT[2].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_256 RTU1__XRDC.PDAC_SLOT[2].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_257 RTU1__XRDC.PDAC_SLOT[2].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_257 RTU1__XRDC.PDAC_SLOT[2].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_258 RTU1__XRDC.PDAC_SLOT[2].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_258 RTU1__XRDC.PDAC_SLOT[2].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_259 RTU1__XRDC.PDAC_SLOT[2].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_259 RTU1__XRDC.PDAC_SLOT[2].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_260 RTU1__XRDC.PDAC_SLOT[2].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_260 RTU1__XRDC.PDAC_SLOT[2].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_261 RTU1__XRDC.PDAC_SLOT[2].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_261 RTU1__XRDC.PDAC_SLOT[2].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_262 RTU1__XRDC.PDAC_SLOT[2].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_262 RTU1__XRDC.PDAC_SLOT[2].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_263 RTU1__XRDC.PDAC_SLOT[2].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_263 RTU1__XRDC.PDAC_SLOT[2].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_265 RTU1__XRDC.PDAC_SLOT[2].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_265 RTU1__XRDC.PDAC_SLOT[2].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_268 RTU1__XRDC.PDAC_SLOT[2].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_268 RTU1__XRDC.PDAC_SLOT[2].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_269 RTU1__XRDC.PDAC_SLOT[2].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_269 RTU1__XRDC.PDAC_SLOT[2].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_270 RTU1__XRDC.PDAC_SLOT[2].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_270 RTU1__XRDC.PDAC_SLOT[2].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_271 RTU1__XRDC.PDAC_SLOT[2].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_271 RTU1__XRDC.PDAC_SLOT[2].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_274 RTU1__XRDC.PDAC_SLOT[2].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_274 RTU1__XRDC.PDAC_SLOT[2].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_275 RTU1__XRDC.PDAC_SLOT[2].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_275 RTU1__XRDC.PDAC_SLOT[2].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_276 RTU1__XRDC.PDAC_SLOT[2].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_276 RTU1__XRDC.PDAC_SLOT[2].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_277 RTU1__XRDC.PDAC_SLOT[2].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_277 RTU1__XRDC.PDAC_SLOT[2].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_278 RTU1__XRDC.PDAC_SLOT[2].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_278 RTU1__XRDC.PDAC_SLOT[2].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_279 RTU1__XRDC.PDAC_SLOT[2].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_279 RTU1__XRDC.PDAC_SLOT[2].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_280 RTU1__XRDC.PDAC_SLOT[2].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_280 RTU1__XRDC.PDAC_SLOT[2].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_281 RTU1__XRDC.PDAC_SLOT[2].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_281 RTU1__XRDC.PDAC_SLOT[2].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_282 RTU1__XRDC.PDAC_SLOT[2].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_282 RTU1__XRDC.PDAC_SLOT[2].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_283 RTU1__XRDC.PDAC_SLOT[2].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_283 RTU1__XRDC.PDAC_SLOT[2].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_284 RTU1__XRDC.PDAC_SLOT[2].PDACN[28].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_284 RTU1__XRDC.PDAC_SLOT[2].PDACN[28].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_285 RTU1__XRDC.PDAC_SLOT[2].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_285 RTU1__XRDC.PDAC_SLOT[2].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_287 RTU1__XRDC.PDAC_SLOT[2].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_287 RTU1__XRDC.PDAC_SLOT[2].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_288 RTU1__XRDC.PDAC_SLOT[2].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_288 RTU1__XRDC.PDAC_SLOT[2].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_289 RTU1__XRDC.PDAC_SLOT[2].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_289 RTU1__XRDC.PDAC_SLOT[2].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_290 RTU1__XRDC.PDAC_SLOT[2].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_290 RTU1__XRDC.PDAC_SLOT[2].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_291 RTU1__XRDC.PDAC_SLOT[2].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_291 RTU1__XRDC.PDAC_SLOT[2].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_292 RTU1__XRDC.PDAC_SLOT[2].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_292 RTU1__XRDC.PDAC_SLOT[2].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_293 RTU1__XRDC.PDAC_SLOT[2].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_293 RTU1__XRDC.PDAC_SLOT[2].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_294 RTU1__XRDC.PDAC_SLOT[2].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_294 RTU1__XRDC.PDAC_SLOT[2].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_295 RTU1__XRDC.PDAC_SLOT[2].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_295 RTU1__XRDC.PDAC_SLOT[2].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_296 RTU1__XRDC.PDAC_SLOT[2].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_296 RTU1__XRDC.PDAC_SLOT[2].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_297 RTU1__XRDC.PDAC_SLOT[2].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_297 RTU1__XRDC.PDAC_SLOT[2].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_PDAC_W0_298 RTU1__XRDC.PDAC_SLOT[2].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define RTU1__XRDC_PDAC_W1_298 RTU1__XRDC.PDAC_SLOT[2].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define RTU1__XRDC_MRGD_W0_0 RTU1__XRDC.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_0 RTU1__XRDC.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_0 RTU1__XRDC.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_0 RTU1__XRDC.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_1 RTU1__XRDC.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_1 RTU1__XRDC.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_1 RTU1__XRDC.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_1 RTU1__XRDC.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_2 RTU1__XRDC.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_2 RTU1__XRDC.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_2 RTU1__XRDC.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_2 RTU1__XRDC.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_3 RTU1__XRDC.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_3 RTU1__XRDC.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_3 RTU1__XRDC.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_3 RTU1__XRDC.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_4 RTU1__XRDC.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_4 RTU1__XRDC.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_4 RTU1__XRDC.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_4 RTU1__XRDC.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_5 RTU1__XRDC.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_5 RTU1__XRDC.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_5 RTU1__XRDC.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_5 RTU1__XRDC.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_6 RTU1__XRDC.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_6 RTU1__XRDC.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_6 RTU1__XRDC.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_6 RTU1__XRDC.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_7 RTU1__XRDC.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_7 RTU1__XRDC.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_7 RTU1__XRDC.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_7 RTU1__XRDC.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_8 RTU1__XRDC.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_8 RTU1__XRDC.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_8 RTU1__XRDC.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_8 RTU1__XRDC.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_9 RTU1__XRDC.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_9 RTU1__XRDC.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_9 RTU1__XRDC.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_9 RTU1__XRDC.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_10 RTU1__XRDC.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_10 RTU1__XRDC.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_10 RTU1__XRDC.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_10 RTU1__XRDC.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_11 RTU1__XRDC.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_11 RTU1__XRDC.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_11 RTU1__XRDC.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_11 RTU1__XRDC.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_12 RTU1__XRDC.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_12 RTU1__XRDC.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_12 RTU1__XRDC.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_12 RTU1__XRDC.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_13 RTU1__XRDC.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_13 RTU1__XRDC.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_13 RTU1__XRDC.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_13 RTU1__XRDC.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_14 RTU1__XRDC.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_14 RTU1__XRDC.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_14 RTU1__XRDC.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_14 RTU1__XRDC.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_15 RTU1__XRDC.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_15 RTU1__XRDC.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_15 RTU1__XRDC.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_15 RTU1__XRDC.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_16 RTU1__XRDC.MRGDN[1][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_16 RTU1__XRDC.MRGDN[1][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_16 RTU1__XRDC.MRGDN[1][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_16 RTU1__XRDC.MRGDN[1][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_17 RTU1__XRDC.MRGDN[1][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_17 RTU1__XRDC.MRGDN[1][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_17 RTU1__XRDC.MRGDN[1][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_17 RTU1__XRDC.MRGDN[1][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_18 RTU1__XRDC.MRGDN[1][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_18 RTU1__XRDC.MRGDN[1][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_18 RTU1__XRDC.MRGDN[1][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_18 RTU1__XRDC.MRGDN[1][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_19 RTU1__XRDC.MRGDN[1][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_19 RTU1__XRDC.MRGDN[1][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_19 RTU1__XRDC.MRGDN[1][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_19 RTU1__XRDC.MRGDN[1][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_20 RTU1__XRDC.MRGDN[1][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_20 RTU1__XRDC.MRGDN[1][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_20 RTU1__XRDC.MRGDN[1][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_20 RTU1__XRDC.MRGDN[1][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_21 RTU1__XRDC.MRGDN[1][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_21 RTU1__XRDC.MRGDN[1][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_21 RTU1__XRDC.MRGDN[1][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_21 RTU1__XRDC.MRGDN[1][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_22 RTU1__XRDC.MRGDN[1][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_22 RTU1__XRDC.MRGDN[1][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_22 RTU1__XRDC.MRGDN[1][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_22 RTU1__XRDC.MRGDN[1][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_23 RTU1__XRDC.MRGDN[1][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_23 RTU1__XRDC.MRGDN[1][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_23 RTU1__XRDC.MRGDN[1][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_23 RTU1__XRDC.MRGDN[1][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_24 RTU1__XRDC.MRGDN[1][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_24 RTU1__XRDC.MRGDN[1][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_24 RTU1__XRDC.MRGDN[1][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_24 RTU1__XRDC.MRGDN[1][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_25 RTU1__XRDC.MRGDN[1][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_25 RTU1__XRDC.MRGDN[1][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_25 RTU1__XRDC.MRGDN[1][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_25 RTU1__XRDC.MRGDN[1][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_26 RTU1__XRDC.MRGDN[1][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_26 RTU1__XRDC.MRGDN[1][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_26 RTU1__XRDC.MRGDN[1][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_26 RTU1__XRDC.MRGDN[1][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_27 RTU1__XRDC.MRGDN[1][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_27 RTU1__XRDC.MRGDN[1][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_27 RTU1__XRDC.MRGDN[1][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_27 RTU1__XRDC.MRGDN[1][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_28 RTU1__XRDC.MRGDN[1][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_28 RTU1__XRDC.MRGDN[1][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_28 RTU1__XRDC.MRGDN[1][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_28 RTU1__XRDC.MRGDN[1][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_29 RTU1__XRDC.MRGDN[1][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_29 RTU1__XRDC.MRGDN[1][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_29 RTU1__XRDC.MRGDN[1][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_29 RTU1__XRDC.MRGDN[1][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_30 RTU1__XRDC.MRGDN[1][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_30 RTU1__XRDC.MRGDN[1][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_30 RTU1__XRDC.MRGDN[1][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_30 RTU1__XRDC.MRGDN[1][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_31 RTU1__XRDC.MRGDN[1][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_31 RTU1__XRDC.MRGDN[1][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_31 RTU1__XRDC.MRGDN[1][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_31 RTU1__XRDC.MRGDN[1][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_32 RTU1__XRDC.MRGDN[2][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_32 RTU1__XRDC.MRGDN[2][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_32 RTU1__XRDC.MRGDN[2][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_32 RTU1__XRDC.MRGDN[2][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_33 RTU1__XRDC.MRGDN[2][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_33 RTU1__XRDC.MRGDN[2][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_33 RTU1__XRDC.MRGDN[2][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_33 RTU1__XRDC.MRGDN[2][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_34 RTU1__XRDC.MRGDN[2][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_34 RTU1__XRDC.MRGDN[2][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_34 RTU1__XRDC.MRGDN[2][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_34 RTU1__XRDC.MRGDN[2][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_35 RTU1__XRDC.MRGDN[2][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_35 RTU1__XRDC.MRGDN[2][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_35 RTU1__XRDC.MRGDN[2][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_35 RTU1__XRDC.MRGDN[2][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_36 RTU1__XRDC.MRGDN[2][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_36 RTU1__XRDC.MRGDN[2][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_36 RTU1__XRDC.MRGDN[2][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_36 RTU1__XRDC.MRGDN[2][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_37 RTU1__XRDC.MRGDN[2][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_37 RTU1__XRDC.MRGDN[2][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_37 RTU1__XRDC.MRGDN[2][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_37 RTU1__XRDC.MRGDN[2][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_38 RTU1__XRDC.MRGDN[2][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_38 RTU1__XRDC.MRGDN[2][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_38 RTU1__XRDC.MRGDN[2][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_38 RTU1__XRDC.MRGDN[2][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_39 RTU1__XRDC.MRGDN[2][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_39 RTU1__XRDC.MRGDN[2][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_39 RTU1__XRDC.MRGDN[2][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_39 RTU1__XRDC.MRGDN[2][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_40 RTU1__XRDC.MRGDN[2][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_40 RTU1__XRDC.MRGDN[2][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_40 RTU1__XRDC.MRGDN[2][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_40 RTU1__XRDC.MRGDN[2][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_41 RTU1__XRDC.MRGDN[2][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_41 RTU1__XRDC.MRGDN[2][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_41 RTU1__XRDC.MRGDN[2][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_41 RTU1__XRDC.MRGDN[2][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_42 RTU1__XRDC.MRGDN[2][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_42 RTU1__XRDC.MRGDN[2][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_42 RTU1__XRDC.MRGDN[2][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_42 RTU1__XRDC.MRGDN[2][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_43 RTU1__XRDC.MRGDN[2][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_43 RTU1__XRDC.MRGDN[2][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_43 RTU1__XRDC.MRGDN[2][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_43 RTU1__XRDC.MRGDN[2][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_44 RTU1__XRDC.MRGDN[2][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_44 RTU1__XRDC.MRGDN[2][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_44 RTU1__XRDC.MRGDN[2][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_44 RTU1__XRDC.MRGDN[2][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_45 RTU1__XRDC.MRGDN[2][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_45 RTU1__XRDC.MRGDN[2][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_45 RTU1__XRDC.MRGDN[2][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_45 RTU1__XRDC.MRGDN[2][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_46 RTU1__XRDC.MRGDN[2][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_46 RTU1__XRDC.MRGDN[2][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_46 RTU1__XRDC.MRGDN[2][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_46 RTU1__XRDC.MRGDN[2][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_47 RTU1__XRDC.MRGDN[2][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_47 RTU1__XRDC.MRGDN[2][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_47 RTU1__XRDC.MRGDN[2][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_47 RTU1__XRDC.MRGDN[2][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_48 RTU1__XRDC.MRGDN[3][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_48 RTU1__XRDC.MRGDN[3][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_48 RTU1__XRDC.MRGDN[3][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_48 RTU1__XRDC.MRGDN[3][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_49 RTU1__XRDC.MRGDN[3][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_49 RTU1__XRDC.MRGDN[3][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_49 RTU1__XRDC.MRGDN[3][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_49 RTU1__XRDC.MRGDN[3][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_50 RTU1__XRDC.MRGDN[3][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_50 RTU1__XRDC.MRGDN[3][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_50 RTU1__XRDC.MRGDN[3][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_50 RTU1__XRDC.MRGDN[3][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_51 RTU1__XRDC.MRGDN[3][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_51 RTU1__XRDC.MRGDN[3][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_51 RTU1__XRDC.MRGDN[3][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_51 RTU1__XRDC.MRGDN[3][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_52 RTU1__XRDC.MRGDN[3][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_52 RTU1__XRDC.MRGDN[3][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_52 RTU1__XRDC.MRGDN[3][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_52 RTU1__XRDC.MRGDN[3][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_53 RTU1__XRDC.MRGDN[3][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_53 RTU1__XRDC.MRGDN[3][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_53 RTU1__XRDC.MRGDN[3][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_53 RTU1__XRDC.MRGDN[3][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_54 RTU1__XRDC.MRGDN[3][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_54 RTU1__XRDC.MRGDN[3][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_54 RTU1__XRDC.MRGDN[3][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_54 RTU1__XRDC.MRGDN[3][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_55 RTU1__XRDC.MRGDN[3][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_55 RTU1__XRDC.MRGDN[3][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_55 RTU1__XRDC.MRGDN[3][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_55 RTU1__XRDC.MRGDN[3][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_56 RTU1__XRDC.MRGDN[3][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_56 RTU1__XRDC.MRGDN[3][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_56 RTU1__XRDC.MRGDN[3][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_56 RTU1__XRDC.MRGDN[3][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_57 RTU1__XRDC.MRGDN[3][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_57 RTU1__XRDC.MRGDN[3][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_57 RTU1__XRDC.MRGDN[3][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_57 RTU1__XRDC.MRGDN[3][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_58 RTU1__XRDC.MRGDN[3][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_58 RTU1__XRDC.MRGDN[3][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_58 RTU1__XRDC.MRGDN[3][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_58 RTU1__XRDC.MRGDN[3][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_59 RTU1__XRDC.MRGDN[3][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_59 RTU1__XRDC.MRGDN[3][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_59 RTU1__XRDC.MRGDN[3][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_59 RTU1__XRDC.MRGDN[3][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_60 RTU1__XRDC.MRGDN[3][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_60 RTU1__XRDC.MRGDN[3][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_60 RTU1__XRDC.MRGDN[3][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_60 RTU1__XRDC.MRGDN[3][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_61 RTU1__XRDC.MRGDN[3][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_61 RTU1__XRDC.MRGDN[3][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_61 RTU1__XRDC.MRGDN[3][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_61 RTU1__XRDC.MRGDN[3][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_62 RTU1__XRDC.MRGDN[3][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_62 RTU1__XRDC.MRGDN[3][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_62 RTU1__XRDC.MRGDN[3][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_62 RTU1__XRDC.MRGDN[3][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_63 RTU1__XRDC.MRGDN[3][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_63 RTU1__XRDC.MRGDN[3][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_63 RTU1__XRDC.MRGDN[3][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_63 RTU1__XRDC.MRGDN[3][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_64 RTU1__XRDC.MRGDN[4][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_64 RTU1__XRDC.MRGDN[4][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_64 RTU1__XRDC.MRGDN[4][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_64 RTU1__XRDC.MRGDN[4][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_65 RTU1__XRDC.MRGDN[4][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_65 RTU1__XRDC.MRGDN[4][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_65 RTU1__XRDC.MRGDN[4][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_65 RTU1__XRDC.MRGDN[4][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_66 RTU1__XRDC.MRGDN[4][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_66 RTU1__XRDC.MRGDN[4][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_66 RTU1__XRDC.MRGDN[4][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_66 RTU1__XRDC.MRGDN[4][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_67 RTU1__XRDC.MRGDN[4][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_67 RTU1__XRDC.MRGDN[4][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_67 RTU1__XRDC.MRGDN[4][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_67 RTU1__XRDC.MRGDN[4][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_68 RTU1__XRDC.MRGDN[4][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_68 RTU1__XRDC.MRGDN[4][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_68 RTU1__XRDC.MRGDN[4][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_68 RTU1__XRDC.MRGDN[4][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_69 RTU1__XRDC.MRGDN[4][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_69 RTU1__XRDC.MRGDN[4][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_69 RTU1__XRDC.MRGDN[4][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_69 RTU1__XRDC.MRGDN[4][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_70 RTU1__XRDC.MRGDN[4][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_70 RTU1__XRDC.MRGDN[4][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_70 RTU1__XRDC.MRGDN[4][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_70 RTU1__XRDC.MRGDN[4][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_71 RTU1__XRDC.MRGDN[4][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_71 RTU1__XRDC.MRGDN[4][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_71 RTU1__XRDC.MRGDN[4][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_71 RTU1__XRDC.MRGDN[4][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_72 RTU1__XRDC.MRGDN[4][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_72 RTU1__XRDC.MRGDN[4][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_72 RTU1__XRDC.MRGDN[4][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_72 RTU1__XRDC.MRGDN[4][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_73 RTU1__XRDC.MRGDN[4][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_73 RTU1__XRDC.MRGDN[4][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_73 RTU1__XRDC.MRGDN[4][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_73 RTU1__XRDC.MRGDN[4][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_74 RTU1__XRDC.MRGDN[4][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_74 RTU1__XRDC.MRGDN[4][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_74 RTU1__XRDC.MRGDN[4][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_74 RTU1__XRDC.MRGDN[4][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_75 RTU1__XRDC.MRGDN[4][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_75 RTU1__XRDC.MRGDN[4][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_75 RTU1__XRDC.MRGDN[4][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_75 RTU1__XRDC.MRGDN[4][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_76 RTU1__XRDC.MRGDN[4][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_76 RTU1__XRDC.MRGDN[4][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_76 RTU1__XRDC.MRGDN[4][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_76 RTU1__XRDC.MRGDN[4][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_77 RTU1__XRDC.MRGDN[4][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_77 RTU1__XRDC.MRGDN[4][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_77 RTU1__XRDC.MRGDN[4][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_77 RTU1__XRDC.MRGDN[4][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_78 RTU1__XRDC.MRGDN[4][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_78 RTU1__XRDC.MRGDN[4][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_78 RTU1__XRDC.MRGDN[4][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_78 RTU1__XRDC.MRGDN[4][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_79 RTU1__XRDC.MRGDN[4][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_79 RTU1__XRDC.MRGDN[4][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_79 RTU1__XRDC.MRGDN[4][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_79 RTU1__XRDC.MRGDN[4][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_80 RTU1__XRDC.MRGDN[5][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_80 RTU1__XRDC.MRGDN[5][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_80 RTU1__XRDC.MRGDN[5][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_80 RTU1__XRDC.MRGDN[5][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_81 RTU1__XRDC.MRGDN[5][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_81 RTU1__XRDC.MRGDN[5][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_81 RTU1__XRDC.MRGDN[5][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_81 RTU1__XRDC.MRGDN[5][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_82 RTU1__XRDC.MRGDN[5][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_82 RTU1__XRDC.MRGDN[5][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_82 RTU1__XRDC.MRGDN[5][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_82 RTU1__XRDC.MRGDN[5][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_83 RTU1__XRDC.MRGDN[5][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_83 RTU1__XRDC.MRGDN[5][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_83 RTU1__XRDC.MRGDN[5][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_83 RTU1__XRDC.MRGDN[5][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_84 RTU1__XRDC.MRGDN[5][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_84 RTU1__XRDC.MRGDN[5][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_84 RTU1__XRDC.MRGDN[5][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_84 RTU1__XRDC.MRGDN[5][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_85 RTU1__XRDC.MRGDN[5][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_85 RTU1__XRDC.MRGDN[5][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_85 RTU1__XRDC.MRGDN[5][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_85 RTU1__XRDC.MRGDN[5][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_86 RTU1__XRDC.MRGDN[5][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_86 RTU1__XRDC.MRGDN[5][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_86 RTU1__XRDC.MRGDN[5][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_86 RTU1__XRDC.MRGDN[5][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_87 RTU1__XRDC.MRGDN[5][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_87 RTU1__XRDC.MRGDN[5][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_87 RTU1__XRDC.MRGDN[5][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_87 RTU1__XRDC.MRGDN[5][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_88 RTU1__XRDC.MRGDN[5][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_88 RTU1__XRDC.MRGDN[5][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_88 RTU1__XRDC.MRGDN[5][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_88 RTU1__XRDC.MRGDN[5][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_89 RTU1__XRDC.MRGDN[5][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_89 RTU1__XRDC.MRGDN[5][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_89 RTU1__XRDC.MRGDN[5][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_89 RTU1__XRDC.MRGDN[5][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_90 RTU1__XRDC.MRGDN[5][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_90 RTU1__XRDC.MRGDN[5][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_90 RTU1__XRDC.MRGDN[5][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_90 RTU1__XRDC.MRGDN[5][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_91 RTU1__XRDC.MRGDN[5][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_91 RTU1__XRDC.MRGDN[5][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_91 RTU1__XRDC.MRGDN[5][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_91 RTU1__XRDC.MRGDN[5][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_92 RTU1__XRDC.MRGDN[5][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_92 RTU1__XRDC.MRGDN[5][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_92 RTU1__XRDC.MRGDN[5][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_92 RTU1__XRDC.MRGDN[5][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_93 RTU1__XRDC.MRGDN[5][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_93 RTU1__XRDC.MRGDN[5][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_93 RTU1__XRDC.MRGDN[5][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_93 RTU1__XRDC.MRGDN[5][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_94 RTU1__XRDC.MRGDN[5][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_94 RTU1__XRDC.MRGDN[5][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_94 RTU1__XRDC.MRGDN[5][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_94 RTU1__XRDC.MRGDN[5][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_95 RTU1__XRDC.MRGDN[5][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_95 RTU1__XRDC.MRGDN[5][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_95 RTU1__XRDC.MRGDN[5][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_95 RTU1__XRDC.MRGDN[5][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_96 RTU1__XRDC.MRGDN[6][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_96 RTU1__XRDC.MRGDN[6][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_96 RTU1__XRDC.MRGDN[6][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_96 RTU1__XRDC.MRGDN[6][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_97 RTU1__XRDC.MRGDN[6][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_97 RTU1__XRDC.MRGDN[6][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_97 RTU1__XRDC.MRGDN[6][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_97 RTU1__XRDC.MRGDN[6][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_98 RTU1__XRDC.MRGDN[6][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_98 RTU1__XRDC.MRGDN[6][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_98 RTU1__XRDC.MRGDN[6][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_98 RTU1__XRDC.MRGDN[6][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_99 RTU1__XRDC.MRGDN[6][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_99 RTU1__XRDC.MRGDN[6][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_99 RTU1__XRDC.MRGDN[6][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_99 RTU1__XRDC.MRGDN[6][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_100 RTU1__XRDC.MRGDN[6][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_100 RTU1__XRDC.MRGDN[6][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_100 RTU1__XRDC.MRGDN[6][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_100 RTU1__XRDC.MRGDN[6][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_101 RTU1__XRDC.MRGDN[6][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_101 RTU1__XRDC.MRGDN[6][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_101 RTU1__XRDC.MRGDN[6][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_101 RTU1__XRDC.MRGDN[6][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_102 RTU1__XRDC.MRGDN[6][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_102 RTU1__XRDC.MRGDN[6][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_102 RTU1__XRDC.MRGDN[6][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_102 RTU1__XRDC.MRGDN[6][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_103 RTU1__XRDC.MRGDN[6][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_103 RTU1__XRDC.MRGDN[6][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_103 RTU1__XRDC.MRGDN[6][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_103 RTU1__XRDC.MRGDN[6][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_104 RTU1__XRDC.MRGDN[6][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_104 RTU1__XRDC.MRGDN[6][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_104 RTU1__XRDC.MRGDN[6][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_104 RTU1__XRDC.MRGDN[6][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_105 RTU1__XRDC.MRGDN[6][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_105 RTU1__XRDC.MRGDN[6][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_105 RTU1__XRDC.MRGDN[6][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_105 RTU1__XRDC.MRGDN[6][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_106 RTU1__XRDC.MRGDN[6][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_106 RTU1__XRDC.MRGDN[6][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_106 RTU1__XRDC.MRGDN[6][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_106 RTU1__XRDC.MRGDN[6][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_107 RTU1__XRDC.MRGDN[6][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_107 RTU1__XRDC.MRGDN[6][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_107 RTU1__XRDC.MRGDN[6][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_107 RTU1__XRDC.MRGDN[6][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_108 RTU1__XRDC.MRGDN[6][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_108 RTU1__XRDC.MRGDN[6][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_108 RTU1__XRDC.MRGDN[6][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_108 RTU1__XRDC.MRGDN[6][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_109 RTU1__XRDC.MRGDN[6][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_109 RTU1__XRDC.MRGDN[6][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_109 RTU1__XRDC.MRGDN[6][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_109 RTU1__XRDC.MRGDN[6][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_110 RTU1__XRDC.MRGDN[6][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_110 RTU1__XRDC.MRGDN[6][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_110 RTU1__XRDC.MRGDN[6][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_110 RTU1__XRDC.MRGDN[6][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_111 RTU1__XRDC.MRGDN[6][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_111 RTU1__XRDC.MRGDN[6][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_111 RTU1__XRDC.MRGDN[6][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_111 RTU1__XRDC.MRGDN[6][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_112 RTU1__XRDC.MRGDN[7][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_112 RTU1__XRDC.MRGDN[7][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_112 RTU1__XRDC.MRGDN[7][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_112 RTU1__XRDC.MRGDN[7][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_113 RTU1__XRDC.MRGDN[7][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_113 RTU1__XRDC.MRGDN[7][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_113 RTU1__XRDC.MRGDN[7][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_113 RTU1__XRDC.MRGDN[7][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_114 RTU1__XRDC.MRGDN[7][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_114 RTU1__XRDC.MRGDN[7][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_114 RTU1__XRDC.MRGDN[7][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_114 RTU1__XRDC.MRGDN[7][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_115 RTU1__XRDC.MRGDN[7][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_115 RTU1__XRDC.MRGDN[7][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_115 RTU1__XRDC.MRGDN[7][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_115 RTU1__XRDC.MRGDN[7][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_116 RTU1__XRDC.MRGDN[7][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_116 RTU1__XRDC.MRGDN[7][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_116 RTU1__XRDC.MRGDN[7][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_116 RTU1__XRDC.MRGDN[7][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_117 RTU1__XRDC.MRGDN[7][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_117 RTU1__XRDC.MRGDN[7][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_117 RTU1__XRDC.MRGDN[7][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_117 RTU1__XRDC.MRGDN[7][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_118 RTU1__XRDC.MRGDN[7][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_118 RTU1__XRDC.MRGDN[7][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_118 RTU1__XRDC.MRGDN[7][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_118 RTU1__XRDC.MRGDN[7][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_119 RTU1__XRDC.MRGDN[7][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_119 RTU1__XRDC.MRGDN[7][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_119 RTU1__XRDC.MRGDN[7][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_119 RTU1__XRDC.MRGDN[7][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_120 RTU1__XRDC.MRGDN[7][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_120 RTU1__XRDC.MRGDN[7][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_120 RTU1__XRDC.MRGDN[7][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_120 RTU1__XRDC.MRGDN[7][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_121 RTU1__XRDC.MRGDN[7][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_121 RTU1__XRDC.MRGDN[7][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_121 RTU1__XRDC.MRGDN[7][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_121 RTU1__XRDC.MRGDN[7][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_122 RTU1__XRDC.MRGDN[7][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_122 RTU1__XRDC.MRGDN[7][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_122 RTU1__XRDC.MRGDN[7][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_122 RTU1__XRDC.MRGDN[7][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_123 RTU1__XRDC.MRGDN[7][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_123 RTU1__XRDC.MRGDN[7][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_123 RTU1__XRDC.MRGDN[7][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_123 RTU1__XRDC.MRGDN[7][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_124 RTU1__XRDC.MRGDN[7][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_124 RTU1__XRDC.MRGDN[7][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_124 RTU1__XRDC.MRGDN[7][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_124 RTU1__XRDC.MRGDN[7][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_125 RTU1__XRDC.MRGDN[7][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_125 RTU1__XRDC.MRGDN[7][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_125 RTU1__XRDC.MRGDN[7][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_125 RTU1__XRDC.MRGDN[7][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_126 RTU1__XRDC.MRGDN[7][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_126 RTU1__XRDC.MRGDN[7][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_126 RTU1__XRDC.MRGDN[7][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_126 RTU1__XRDC.MRGDN[7][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_127 RTU1__XRDC.MRGDN[7][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_127 RTU1__XRDC.MRGDN[7][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_127 RTU1__XRDC.MRGDN[7][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_127 RTU1__XRDC.MRGDN[7][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_128 RTU1__XRDC.MRGDN[8][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_128 RTU1__XRDC.MRGDN[8][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_128 RTU1__XRDC.MRGDN[8][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_128 RTU1__XRDC.MRGDN[8][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_129 RTU1__XRDC.MRGDN[8][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_129 RTU1__XRDC.MRGDN[8][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_129 RTU1__XRDC.MRGDN[8][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_129 RTU1__XRDC.MRGDN[8][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_130 RTU1__XRDC.MRGDN[8][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_130 RTU1__XRDC.MRGDN[8][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_130 RTU1__XRDC.MRGDN[8][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_130 RTU1__XRDC.MRGDN[8][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_131 RTU1__XRDC.MRGDN[8][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_131 RTU1__XRDC.MRGDN[8][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_131 RTU1__XRDC.MRGDN[8][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_131 RTU1__XRDC.MRGDN[8][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_132 RTU1__XRDC.MRGDN[8][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_132 RTU1__XRDC.MRGDN[8][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_132 RTU1__XRDC.MRGDN[8][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_132 RTU1__XRDC.MRGDN[8][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_133 RTU1__XRDC.MRGDN[8][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_133 RTU1__XRDC.MRGDN[8][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_133 RTU1__XRDC.MRGDN[8][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_133 RTU1__XRDC.MRGDN[8][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_134 RTU1__XRDC.MRGDN[8][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_134 RTU1__XRDC.MRGDN[8][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_134 RTU1__XRDC.MRGDN[8][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_134 RTU1__XRDC.MRGDN[8][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_135 RTU1__XRDC.MRGDN[8][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_135 RTU1__XRDC.MRGDN[8][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_135 RTU1__XRDC.MRGDN[8][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_135 RTU1__XRDC.MRGDN[8][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_136 RTU1__XRDC.MRGDN[8][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_136 RTU1__XRDC.MRGDN[8][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_136 RTU1__XRDC.MRGDN[8][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_136 RTU1__XRDC.MRGDN[8][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_137 RTU1__XRDC.MRGDN[8][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_137 RTU1__XRDC.MRGDN[8][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_137 RTU1__XRDC.MRGDN[8][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_137 RTU1__XRDC.MRGDN[8][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_138 RTU1__XRDC.MRGDN[8][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_138 RTU1__XRDC.MRGDN[8][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_138 RTU1__XRDC.MRGDN[8][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_138 RTU1__XRDC.MRGDN[8][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_139 RTU1__XRDC.MRGDN[8][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_139 RTU1__XRDC.MRGDN[8][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_139 RTU1__XRDC.MRGDN[8][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_139 RTU1__XRDC.MRGDN[8][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_140 RTU1__XRDC.MRGDN[8][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_140 RTU1__XRDC.MRGDN[8][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_140 RTU1__XRDC.MRGDN[8][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_140 RTU1__XRDC.MRGDN[8][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_141 RTU1__XRDC.MRGDN[8][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_141 RTU1__XRDC.MRGDN[8][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_141 RTU1__XRDC.MRGDN[8][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_141 RTU1__XRDC.MRGDN[8][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_142 RTU1__XRDC.MRGDN[8][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_142 RTU1__XRDC.MRGDN[8][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_142 RTU1__XRDC.MRGDN[8][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_142 RTU1__XRDC.MRGDN[8][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_143 RTU1__XRDC.MRGDN[8][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_143 RTU1__XRDC.MRGDN[8][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_143 RTU1__XRDC.MRGDN[8][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_143 RTU1__XRDC.MRGDN[8][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_144 RTU1__XRDC.MRGDN[9][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_144 RTU1__XRDC.MRGDN[9][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_144 RTU1__XRDC.MRGDN[9][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_144 RTU1__XRDC.MRGDN[9][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_145 RTU1__XRDC.MRGDN[9][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_145 RTU1__XRDC.MRGDN[9][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_145 RTU1__XRDC.MRGDN[9][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_145 RTU1__XRDC.MRGDN[9][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_146 RTU1__XRDC.MRGDN[9][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_146 RTU1__XRDC.MRGDN[9][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_146 RTU1__XRDC.MRGDN[9][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_146 RTU1__XRDC.MRGDN[9][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_147 RTU1__XRDC.MRGDN[9][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_147 RTU1__XRDC.MRGDN[9][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_147 RTU1__XRDC.MRGDN[9][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_147 RTU1__XRDC.MRGDN[9][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_148 RTU1__XRDC.MRGDN[9][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_148 RTU1__XRDC.MRGDN[9][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_148 RTU1__XRDC.MRGDN[9][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_148 RTU1__XRDC.MRGDN[9][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_149 RTU1__XRDC.MRGDN[9][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_149 RTU1__XRDC.MRGDN[9][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_149 RTU1__XRDC.MRGDN[9][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_149 RTU1__XRDC.MRGDN[9][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_150 RTU1__XRDC.MRGDN[9][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_150 RTU1__XRDC.MRGDN[9][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_150 RTU1__XRDC.MRGDN[9][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_150 RTU1__XRDC.MRGDN[9][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_151 RTU1__XRDC.MRGDN[9][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_151 RTU1__XRDC.MRGDN[9][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_151 RTU1__XRDC.MRGDN[9][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_151 RTU1__XRDC.MRGDN[9][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_152 RTU1__XRDC.MRGDN[9][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_152 RTU1__XRDC.MRGDN[9][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_152 RTU1__XRDC.MRGDN[9][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_152 RTU1__XRDC.MRGDN[9][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_153 RTU1__XRDC.MRGDN[9][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_153 RTU1__XRDC.MRGDN[9][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_153 RTU1__XRDC.MRGDN[9][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_153 RTU1__XRDC.MRGDN[9][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_154 RTU1__XRDC.MRGDN[9][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_154 RTU1__XRDC.MRGDN[9][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_154 RTU1__XRDC.MRGDN[9][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_154 RTU1__XRDC.MRGDN[9][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_155 RTU1__XRDC.MRGDN[9][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_155 RTU1__XRDC.MRGDN[9][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_155 RTU1__XRDC.MRGDN[9][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_155 RTU1__XRDC.MRGDN[9][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_156 RTU1__XRDC.MRGDN[9][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_156 RTU1__XRDC.MRGDN[9][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_156 RTU1__XRDC.MRGDN[9][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_156 RTU1__XRDC.MRGDN[9][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_157 RTU1__XRDC.MRGDN[9][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_157 RTU1__XRDC.MRGDN[9][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_157 RTU1__XRDC.MRGDN[9][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_157 RTU1__XRDC.MRGDN[9][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_158 RTU1__XRDC.MRGDN[9][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_158 RTU1__XRDC.MRGDN[9][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_158 RTU1__XRDC.MRGDN[9][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_158 RTU1__XRDC.MRGDN[9][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_159 RTU1__XRDC.MRGDN[9][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_159 RTU1__XRDC.MRGDN[9][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_159 RTU1__XRDC.MRGDN[9][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_159 RTU1__XRDC.MRGDN[9][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_160 RTU1__XRDC.MRGDN[10][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_160 RTU1__XRDC.MRGDN[10][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_160 RTU1__XRDC.MRGDN[10][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_160 RTU1__XRDC.MRGDN[10][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_161 RTU1__XRDC.MRGDN[10][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_161 RTU1__XRDC.MRGDN[10][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_161 RTU1__XRDC.MRGDN[10][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_161 RTU1__XRDC.MRGDN[10][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_162 RTU1__XRDC.MRGDN[10][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_162 RTU1__XRDC.MRGDN[10][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_162 RTU1__XRDC.MRGDN[10][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_162 RTU1__XRDC.MRGDN[10][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_163 RTU1__XRDC.MRGDN[10][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_163 RTU1__XRDC.MRGDN[10][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_163 RTU1__XRDC.MRGDN[10][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_163 RTU1__XRDC.MRGDN[10][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_164 RTU1__XRDC.MRGDN[10][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_164 RTU1__XRDC.MRGDN[10][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_164 RTU1__XRDC.MRGDN[10][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_164 RTU1__XRDC.MRGDN[10][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_165 RTU1__XRDC.MRGDN[10][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_165 RTU1__XRDC.MRGDN[10][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_165 RTU1__XRDC.MRGDN[10][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_165 RTU1__XRDC.MRGDN[10][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_166 RTU1__XRDC.MRGDN[10][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_166 RTU1__XRDC.MRGDN[10][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_166 RTU1__XRDC.MRGDN[10][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_166 RTU1__XRDC.MRGDN[10][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_167 RTU1__XRDC.MRGDN[10][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_167 RTU1__XRDC.MRGDN[10][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_167 RTU1__XRDC.MRGDN[10][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_167 RTU1__XRDC.MRGDN[10][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_168 RTU1__XRDC.MRGDN[10][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_168 RTU1__XRDC.MRGDN[10][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_168 RTU1__XRDC.MRGDN[10][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_168 RTU1__XRDC.MRGDN[10][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_169 RTU1__XRDC.MRGDN[10][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_169 RTU1__XRDC.MRGDN[10][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_169 RTU1__XRDC.MRGDN[10][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_169 RTU1__XRDC.MRGDN[10][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_170 RTU1__XRDC.MRGDN[10][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_170 RTU1__XRDC.MRGDN[10][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_170 RTU1__XRDC.MRGDN[10][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_170 RTU1__XRDC.MRGDN[10][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_171 RTU1__XRDC.MRGDN[10][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_171 RTU1__XRDC.MRGDN[10][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_171 RTU1__XRDC.MRGDN[10][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_171 RTU1__XRDC.MRGDN[10][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_172 RTU1__XRDC.MRGDN[10][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_172 RTU1__XRDC.MRGDN[10][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_172 RTU1__XRDC.MRGDN[10][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_172 RTU1__XRDC.MRGDN[10][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_173 RTU1__XRDC.MRGDN[10][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_173 RTU1__XRDC.MRGDN[10][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_173 RTU1__XRDC.MRGDN[10][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_173 RTU1__XRDC.MRGDN[10][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_174 RTU1__XRDC.MRGDN[10][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_174 RTU1__XRDC.MRGDN[10][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_174 RTU1__XRDC.MRGDN[10][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_174 RTU1__XRDC.MRGDN[10][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_175 RTU1__XRDC.MRGDN[10][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_175 RTU1__XRDC.MRGDN[10][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_175 RTU1__XRDC.MRGDN[10][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_175 RTU1__XRDC.MRGDN[10][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_176 RTU1__XRDC.MRGDN[11][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_176 RTU1__XRDC.MRGDN[11][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_176 RTU1__XRDC.MRGDN[11][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_176 RTU1__XRDC.MRGDN[11][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_177 RTU1__XRDC.MRGDN[11][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_177 RTU1__XRDC.MRGDN[11][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_177 RTU1__XRDC.MRGDN[11][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_177 RTU1__XRDC.MRGDN[11][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_178 RTU1__XRDC.MRGDN[11][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_178 RTU1__XRDC.MRGDN[11][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_178 RTU1__XRDC.MRGDN[11][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_178 RTU1__XRDC.MRGDN[11][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_179 RTU1__XRDC.MRGDN[11][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_179 RTU1__XRDC.MRGDN[11][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_179 RTU1__XRDC.MRGDN[11][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_179 RTU1__XRDC.MRGDN[11][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_180 RTU1__XRDC.MRGDN[11][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_180 RTU1__XRDC.MRGDN[11][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_180 RTU1__XRDC.MRGDN[11][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_180 RTU1__XRDC.MRGDN[11][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_181 RTU1__XRDC.MRGDN[11][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_181 RTU1__XRDC.MRGDN[11][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_181 RTU1__XRDC.MRGDN[11][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_181 RTU1__XRDC.MRGDN[11][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_182 RTU1__XRDC.MRGDN[11][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_182 RTU1__XRDC.MRGDN[11][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_182 RTU1__XRDC.MRGDN[11][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_182 RTU1__XRDC.MRGDN[11][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_183 RTU1__XRDC.MRGDN[11][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_183 RTU1__XRDC.MRGDN[11][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_183 RTU1__XRDC.MRGDN[11][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_183 RTU1__XRDC.MRGDN[11][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_184 RTU1__XRDC.MRGDN[11][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_184 RTU1__XRDC.MRGDN[11][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_184 RTU1__XRDC.MRGDN[11][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_184 RTU1__XRDC.MRGDN[11][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_185 RTU1__XRDC.MRGDN[11][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_185 RTU1__XRDC.MRGDN[11][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_185 RTU1__XRDC.MRGDN[11][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_185 RTU1__XRDC.MRGDN[11][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_186 RTU1__XRDC.MRGDN[11][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_186 RTU1__XRDC.MRGDN[11][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_186 RTU1__XRDC.MRGDN[11][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_186 RTU1__XRDC.MRGDN[11][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_187 RTU1__XRDC.MRGDN[11][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_187 RTU1__XRDC.MRGDN[11][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_187 RTU1__XRDC.MRGDN[11][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_187 RTU1__XRDC.MRGDN[11][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_188 RTU1__XRDC.MRGDN[11][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_188 RTU1__XRDC.MRGDN[11][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_188 RTU1__XRDC.MRGDN[11][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_188 RTU1__XRDC.MRGDN[11][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_189 RTU1__XRDC.MRGDN[11][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_189 RTU1__XRDC.MRGDN[11][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_189 RTU1__XRDC.MRGDN[11][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_189 RTU1__XRDC.MRGDN[11][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_190 RTU1__XRDC.MRGDN[11][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_190 RTU1__XRDC.MRGDN[11][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_190 RTU1__XRDC.MRGDN[11][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_190 RTU1__XRDC.MRGDN[11][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_191 RTU1__XRDC.MRGDN[11][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_191 RTU1__XRDC.MRGDN[11][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_191 RTU1__XRDC.MRGDN[11][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_191 RTU1__XRDC.MRGDN[11][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_192 RTU1__XRDC.MRGDN[12][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_192 RTU1__XRDC.MRGDN[12][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_192 RTU1__XRDC.MRGDN[12][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_192 RTU1__XRDC.MRGDN[12][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_193 RTU1__XRDC.MRGDN[12][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_193 RTU1__XRDC.MRGDN[12][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_193 RTU1__XRDC.MRGDN[12][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_193 RTU1__XRDC.MRGDN[12][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_194 RTU1__XRDC.MRGDN[12][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_194 RTU1__XRDC.MRGDN[12][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_194 RTU1__XRDC.MRGDN[12][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_194 RTU1__XRDC.MRGDN[12][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_195 RTU1__XRDC.MRGDN[12][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_195 RTU1__XRDC.MRGDN[12][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_195 RTU1__XRDC.MRGDN[12][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_195 RTU1__XRDC.MRGDN[12][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_196 RTU1__XRDC.MRGDN[12][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_196 RTU1__XRDC.MRGDN[12][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_196 RTU1__XRDC.MRGDN[12][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_196 RTU1__XRDC.MRGDN[12][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_197 RTU1__XRDC.MRGDN[12][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_197 RTU1__XRDC.MRGDN[12][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_197 RTU1__XRDC.MRGDN[12][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_197 RTU1__XRDC.MRGDN[12][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_198 RTU1__XRDC.MRGDN[12][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_198 RTU1__XRDC.MRGDN[12][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_198 RTU1__XRDC.MRGDN[12][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_198 RTU1__XRDC.MRGDN[12][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_199 RTU1__XRDC.MRGDN[12][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_199 RTU1__XRDC.MRGDN[12][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_199 RTU1__XRDC.MRGDN[12][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_199 RTU1__XRDC.MRGDN[12][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_200 RTU1__XRDC.MRGDN[12][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_200 RTU1__XRDC.MRGDN[12][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_200 RTU1__XRDC.MRGDN[12][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_200 RTU1__XRDC.MRGDN[12][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_201 RTU1__XRDC.MRGDN[12][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_201 RTU1__XRDC.MRGDN[12][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_201 RTU1__XRDC.MRGDN[12][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_201 RTU1__XRDC.MRGDN[12][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_202 RTU1__XRDC.MRGDN[12][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_202 RTU1__XRDC.MRGDN[12][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_202 RTU1__XRDC.MRGDN[12][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_202 RTU1__XRDC.MRGDN[12][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_203 RTU1__XRDC.MRGDN[12][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_203 RTU1__XRDC.MRGDN[12][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_203 RTU1__XRDC.MRGDN[12][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_203 RTU1__XRDC.MRGDN[12][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_204 RTU1__XRDC.MRGDN[12][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_204 RTU1__XRDC.MRGDN[12][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_204 RTU1__XRDC.MRGDN[12][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_204 RTU1__XRDC.MRGDN[12][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_205 RTU1__XRDC.MRGDN[12][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_205 RTU1__XRDC.MRGDN[12][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_205 RTU1__XRDC.MRGDN[12][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_205 RTU1__XRDC.MRGDN[12][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_206 RTU1__XRDC.MRGDN[12][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_206 RTU1__XRDC.MRGDN[12][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_206 RTU1__XRDC.MRGDN[12][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_206 RTU1__XRDC.MRGDN[12][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_207 RTU1__XRDC.MRGDN[12][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_207 RTU1__XRDC.MRGDN[12][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_207 RTU1__XRDC.MRGDN[12][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_207 RTU1__XRDC.MRGDN[12][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_208 RTU1__XRDC.MRGDN[13][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_208 RTU1__XRDC.MRGDN[13][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_208 RTU1__XRDC.MRGDN[13][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_208 RTU1__XRDC.MRGDN[13][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_209 RTU1__XRDC.MRGDN[13][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_209 RTU1__XRDC.MRGDN[13][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_209 RTU1__XRDC.MRGDN[13][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_209 RTU1__XRDC.MRGDN[13][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_210 RTU1__XRDC.MRGDN[13][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_210 RTU1__XRDC.MRGDN[13][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_210 RTU1__XRDC.MRGDN[13][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_210 RTU1__XRDC.MRGDN[13][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_211 RTU1__XRDC.MRGDN[13][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_211 RTU1__XRDC.MRGDN[13][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_211 RTU1__XRDC.MRGDN[13][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_211 RTU1__XRDC.MRGDN[13][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_212 RTU1__XRDC.MRGDN[13][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_212 RTU1__XRDC.MRGDN[13][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_212 RTU1__XRDC.MRGDN[13][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_212 RTU1__XRDC.MRGDN[13][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_213 RTU1__XRDC.MRGDN[13][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_213 RTU1__XRDC.MRGDN[13][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_213 RTU1__XRDC.MRGDN[13][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_213 RTU1__XRDC.MRGDN[13][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_214 RTU1__XRDC.MRGDN[13][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_214 RTU1__XRDC.MRGDN[13][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_214 RTU1__XRDC.MRGDN[13][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_214 RTU1__XRDC.MRGDN[13][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_215 RTU1__XRDC.MRGDN[13][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_215 RTU1__XRDC.MRGDN[13][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_215 RTU1__XRDC.MRGDN[13][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_215 RTU1__XRDC.MRGDN[13][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_216 RTU1__XRDC.MRGDN[13][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_216 RTU1__XRDC.MRGDN[13][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_216 RTU1__XRDC.MRGDN[13][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_216 RTU1__XRDC.MRGDN[13][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_217 RTU1__XRDC.MRGDN[13][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_217 RTU1__XRDC.MRGDN[13][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_217 RTU1__XRDC.MRGDN[13][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_217 RTU1__XRDC.MRGDN[13][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_218 RTU1__XRDC.MRGDN[13][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_218 RTU1__XRDC.MRGDN[13][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_218 RTU1__XRDC.MRGDN[13][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_218 RTU1__XRDC.MRGDN[13][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_219 RTU1__XRDC.MRGDN[13][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_219 RTU1__XRDC.MRGDN[13][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_219 RTU1__XRDC.MRGDN[13][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_219 RTU1__XRDC.MRGDN[13][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_220 RTU1__XRDC.MRGDN[13][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_220 RTU1__XRDC.MRGDN[13][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_220 RTU1__XRDC.MRGDN[13][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_220 RTU1__XRDC.MRGDN[13][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_221 RTU1__XRDC.MRGDN[13][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_221 RTU1__XRDC.MRGDN[13][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_221 RTU1__XRDC.MRGDN[13][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_221 RTU1__XRDC.MRGDN[13][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_222 RTU1__XRDC.MRGDN[13][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_222 RTU1__XRDC.MRGDN[13][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_222 RTU1__XRDC.MRGDN[13][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_222 RTU1__XRDC.MRGDN[13][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_223 RTU1__XRDC.MRGDN[13][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_223 RTU1__XRDC.MRGDN[13][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_223 RTU1__XRDC.MRGDN[13][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_223 RTU1__XRDC.MRGDN[13][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_224 RTU1__XRDC.MRGDN[14][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_224 RTU1__XRDC.MRGDN[14][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_224 RTU1__XRDC.MRGDN[14][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_224 RTU1__XRDC.MRGDN[14][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_225 RTU1__XRDC.MRGDN[14][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_225 RTU1__XRDC.MRGDN[14][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_225 RTU1__XRDC.MRGDN[14][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_225 RTU1__XRDC.MRGDN[14][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_226 RTU1__XRDC.MRGDN[14][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_226 RTU1__XRDC.MRGDN[14][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_226 RTU1__XRDC.MRGDN[14][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_226 RTU1__XRDC.MRGDN[14][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_227 RTU1__XRDC.MRGDN[14][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_227 RTU1__XRDC.MRGDN[14][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_227 RTU1__XRDC.MRGDN[14][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_227 RTU1__XRDC.MRGDN[14][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_228 RTU1__XRDC.MRGDN[14][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_228 RTU1__XRDC.MRGDN[14][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_228 RTU1__XRDC.MRGDN[14][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_228 RTU1__XRDC.MRGDN[14][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_229 RTU1__XRDC.MRGDN[14][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_229 RTU1__XRDC.MRGDN[14][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_229 RTU1__XRDC.MRGDN[14][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_229 RTU1__XRDC.MRGDN[14][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_230 RTU1__XRDC.MRGDN[14][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_230 RTU1__XRDC.MRGDN[14][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_230 RTU1__XRDC.MRGDN[14][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_230 RTU1__XRDC.MRGDN[14][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_231 RTU1__XRDC.MRGDN[14][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_231 RTU1__XRDC.MRGDN[14][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_231 RTU1__XRDC.MRGDN[14][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_231 RTU1__XRDC.MRGDN[14][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_232 RTU1__XRDC.MRGDN[14][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_232 RTU1__XRDC.MRGDN[14][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_232 RTU1__XRDC.MRGDN[14][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_232 RTU1__XRDC.MRGDN[14][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_233 RTU1__XRDC.MRGDN[14][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_233 RTU1__XRDC.MRGDN[14][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_233 RTU1__XRDC.MRGDN[14][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_233 RTU1__XRDC.MRGDN[14][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_234 RTU1__XRDC.MRGDN[14][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_234 RTU1__XRDC.MRGDN[14][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_234 RTU1__XRDC.MRGDN[14][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_234 RTU1__XRDC.MRGDN[14][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_235 RTU1__XRDC.MRGDN[14][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_235 RTU1__XRDC.MRGDN[14][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_235 RTU1__XRDC.MRGDN[14][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_235 RTU1__XRDC.MRGDN[14][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_236 RTU1__XRDC.MRGDN[14][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_236 RTU1__XRDC.MRGDN[14][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_236 RTU1__XRDC.MRGDN[14][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_236 RTU1__XRDC.MRGDN[14][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_237 RTU1__XRDC.MRGDN[14][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_237 RTU1__XRDC.MRGDN[14][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_237 RTU1__XRDC.MRGDN[14][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_237 RTU1__XRDC.MRGDN[14][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_238 RTU1__XRDC.MRGDN[14][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_238 RTU1__XRDC.MRGDN[14][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_238 RTU1__XRDC.MRGDN[14][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_238 RTU1__XRDC.MRGDN[14][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define RTU1__XRDC_MRGD_W0_239 RTU1__XRDC.MRGDN[14][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define RTU1__XRDC_MRGD_W1_239 RTU1__XRDC.MRGDN[14][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define RTU1__XRDC_MRGD_W2_239 RTU1__XRDC.MRGDN[14][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define RTU1__XRDC_MRGD_W3_239 RTU1__XRDC.MRGDN[14][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* RXLUT */
#define CE_RXLUT_RLCR        CE_RXLUT.RLCR.R               /* RXLUT Configuration */
#define CE_RXLUT_RLSR        CE_RXLUT.RLSR.R               /* RXLUT Status */
#define CE_RXLUT_RLSRIER     CE_RXLUT.RLSRIER.R            /* RXLUT Search Result Interrupt Enable */
#define CE_RXLUT_RLSRSR      CE_RXLUT.RLSRSR.R             /* RXLUT Search Result Status */
#define CE_RXLUT_RLFIMER     CE_RXLUT.RLFIMER.R            /* RXLUT Frame ID End */
#define CE_RXLUT_RLFISR      CE_RXLUT.RLFISR.R             /* RXLUT Frame ID Start */
#define CE_RXLUT_RLCIXR      CE_RXLUT.RLCIXR.R             /* RXLUT Channel ID AND Index */
#define CE_RXLUT_RLADRCR     CE_RXLUT.RLADRCR.R            /* RXLUT Address and Command */
#define CE_RXLUT_RLFIMESR    CE_RXLUT.RLFIMESR.R           /* RXLUT Frame ID End Status */
#define CE_RXLUT_RLFISSR     CE_RXLUT.RLFISSR.R            /* RXLUT Frame ID Start Status */
#define CE_RXLUT_RLCIXSR     CE_RXLUT.RLCIXSR.R            /* RXLUT Channel ID AND Index Status */
#define CE_RXLUT_RLADRSR     CE_RXLUT.RLADRSR.R            /* RXLUT Address Status */
#define CE_RXLUT_RLSICR      CE_RXLUT.RLSICR.R             /* RXLUT Search Input Command */
#define CE_RXLUT_RLSICFIR    CE_RXLUT.RLSICFIR.R           /* RXLUT Search Input Command Frame ID */
#define CE_RXLUT_RLSOIXR0    CE_RXLUT.RLSOIXR[0].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR0    CE_RXLUT.RLSOIXR[0].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR1    CE_RXLUT.RLSOIXR[1].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR1    CE_RXLUT.RLSOIXR[1].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR2    CE_RXLUT.RLSOIXR[2].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR2    CE_RXLUT.RLSOIXR[2].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR3    CE_RXLUT.RLSOIXR[3].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR3    CE_RXLUT.RLSOIXR[3].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR4    CE_RXLUT.RLSOIXR[4].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR4    CE_RXLUT.RLSOIXR[4].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR5    CE_RXLUT.RLSOIXR[5].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR5    CE_RXLUT.RLSOIXR[5].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR6    CE_RXLUT.RLSOIXR[6].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR6    CE_RXLUT.RLSOIXR[6].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR7    CE_RXLUT.RLSOIXR[7].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR7    CE_RXLUT.RLSOIXR[7].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR8    CE_RXLUT.RLSOIXR[8].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR8    CE_RXLUT.RLSOIXR[8].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR9    CE_RXLUT.RLSOIXR[9].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR9    CE_RXLUT.RLSOIXR[9].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR10   CE_RXLUT.RLSOIXR[10].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR10   CE_RXLUT.RLSOIXR[10].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSOIXR11   CE_RXLUT.RLSOIXR[11].RLSOIXR.R /* RXLUT Search Output */
#define CE_RXLUT_RLSOADR11   CE_RXLUT.RLSOIXR[11].RLSOADR.R /* RXLUT Search Output Address */
#define CE_RXLUT_RLSICFSR    CE_RXLUT.RLSICFSR.R           /* RXLUT Search Input Command FIFO Status */
#define CE_RXLUT_RLSOFSR0    CE_RXLUT.RLSOFSR0.R           /* RXLUT Search Output FIFO Status */
#define CE_RXLUT_RLSOFSR1    CE_RXLUT.RLSOFSR1.R           /* RXLUT Search Output FIFO Status */
#define CE_RXLUT_RLICEC0     CE_RXLUT.RLICEC0.R            /* RXLUT Input Command Entry Counter */
#define CE_RXLUT_RLICEC1     CE_RXLUT.RLICEC1.R            /* RXLUT Input Command Entry Counter */
#define CE_RXLUT_RLER        CE_RXLUT.RLER.R               /* RXLUT Error */
#define CE_RXLUT_RLOOR       CE_RXLUT.RLOOR.R              /* RXLUT Output Overflow */

/* SBSW */
#define SMU__SBSW_TMC_CONFIG_UNLOCK SMU__SBSW.TMC_CONFIG_UNLOCK.R /* TMC Configuration Unlock */
#define SMU__SBSW_TMWDP_CONFIG_UNLOCK SMU__SBSW.TMWDP_CONFIG_UNLOCK.R /* TMWDP Configuration Unlock */
#define SMU__SBSW_TMC_CONFIG_STATUS SMU__SBSW.TMC_CONFIG_STATUS.R /* TMC Configuration Status */
#define SMU__SBSW_TMWDP_CONFIG_STATUS SMU__SBSW.TMWDP_CONFIG_STATUS.R /* TMWDP Configuration Status */
#define SMU__SBSW_DEBUG_MODE SMU__SBSW.DEBUG_MODE.R        /* Debug Mode */
#define SMU__SBSW_TMC_FAULT_STATUS SMU__SBSW.TMC_FAULT_STATUS.R /* TMC Fault Status */
#define SMU__SBSW_TMWDP_FAULT_STATUS SMU__SBSW.TMWDP_FAULT_STATUS.R /* TMWDP Fault Status */
#define SMU__SBSW_TMC_0_CONFIG SMU__SBSW.TMC[0].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_0_DISTANCE SMU__SBSW.TMC[0].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_0_TIMEOUT SMU__SBSW.TMC[0].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_0_CONTROL SMU__SBSW.TMC[0].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_0_R0   SMU__SBSW.TMC[0].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_0_R1   SMU__SBSW.TMC[0].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_0_STATUS SMU__SBSW.TMC[0].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_0_TIMER SMU__SBSW.TMC[0].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_1_CONFIG SMU__SBSW.TMC[1].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_1_DISTANCE SMU__SBSW.TMC[1].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_1_TIMEOUT SMU__SBSW.TMC[1].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_1_CONTROL SMU__SBSW.TMC[1].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_1_R0   SMU__SBSW.TMC[1].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_1_R1   SMU__SBSW.TMC[1].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_1_STATUS SMU__SBSW.TMC[1].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_1_TIMER SMU__SBSW.TMC[1].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_2_CONFIG SMU__SBSW.TMC[2].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_2_DISTANCE SMU__SBSW.TMC[2].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_2_TIMEOUT SMU__SBSW.TMC[2].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_2_CONTROL SMU__SBSW.TMC[2].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_2_R0   SMU__SBSW.TMC[2].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_2_R1   SMU__SBSW.TMC[2].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_2_STATUS SMU__SBSW.TMC[2].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_2_TIMER SMU__SBSW.TMC[2].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_3_CONFIG SMU__SBSW.TMC[3].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_3_DISTANCE SMU__SBSW.TMC[3].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_3_TIMEOUT SMU__SBSW.TMC[3].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_3_CONTROL SMU__SBSW.TMC[3].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_3_R0   SMU__SBSW.TMC[3].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_3_R1   SMU__SBSW.TMC[3].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_3_STATUS SMU__SBSW.TMC[3].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_3_TIMER SMU__SBSW.TMC[3].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_4_CONFIG SMU__SBSW.TMC[4].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_4_DISTANCE SMU__SBSW.TMC[4].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_4_TIMEOUT SMU__SBSW.TMC[4].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_4_CONTROL SMU__SBSW.TMC[4].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_4_R0   SMU__SBSW.TMC[4].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_4_R1   SMU__SBSW.TMC[4].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_4_STATUS SMU__SBSW.TMC[4].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_4_TIMER SMU__SBSW.TMC[4].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_5_CONFIG SMU__SBSW.TMC[5].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_5_DISTANCE SMU__SBSW.TMC[5].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_5_TIMEOUT SMU__SBSW.TMC[5].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_5_CONTROL SMU__SBSW.TMC[5].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_5_R0   SMU__SBSW.TMC[5].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_5_R1   SMU__SBSW.TMC[5].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_5_STATUS SMU__SBSW.TMC[5].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_5_TIMER SMU__SBSW.TMC[5].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_6_CONFIG SMU__SBSW.TMC[6].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_6_DISTANCE SMU__SBSW.TMC[6].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_6_TIMEOUT SMU__SBSW.TMC[6].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_6_CONTROL SMU__SBSW.TMC[6].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_6_R0   SMU__SBSW.TMC[6].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_6_R1   SMU__SBSW.TMC[6].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_6_STATUS SMU__SBSW.TMC[6].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_6_TIMER SMU__SBSW.TMC[6].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_7_CONFIG SMU__SBSW.TMC[7].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_7_DISTANCE SMU__SBSW.TMC[7].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_7_TIMEOUT SMU__SBSW.TMC[7].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_7_CONTROL SMU__SBSW.TMC[7].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_7_R0   SMU__SBSW.TMC[7].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_7_R1   SMU__SBSW.TMC[7].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_7_STATUS SMU__SBSW.TMC[7].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_7_TIMER SMU__SBSW.TMC[7].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_8_CONFIG SMU__SBSW.TMC[8].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_8_DISTANCE SMU__SBSW.TMC[8].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_8_TIMEOUT SMU__SBSW.TMC[8].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_8_CONTROL SMU__SBSW.TMC[8].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_8_R0   SMU__SBSW.TMC[8].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_8_R1   SMU__SBSW.TMC[8].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_8_STATUS SMU__SBSW.TMC[8].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_8_TIMER SMU__SBSW.TMC[8].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_9_CONFIG SMU__SBSW.TMC[9].CONFIG.R   /* TMC Configuration */
#define SMU__SBSW_TMC_9_DISTANCE SMU__SBSW.TMC[9].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_9_TIMEOUT SMU__SBSW.TMC[9].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_9_CONTROL SMU__SBSW.TMC[9].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_9_R0   SMU__SBSW.TMC[9].R0.R         /* TMC R0 Data */
#define SMU__SBSW_TMC_9_R1   SMU__SBSW.TMC[9].R1.R         /* TMC R1 Data */
#define SMU__SBSW_TMC_9_STATUS SMU__SBSW.TMC[9].STATUS.R   /* TMC Status */
#define SMU__SBSW_TMC_9_TIMER SMU__SBSW.TMC[9].TIMER.R     /* TMC Timer */
#define SMU__SBSW_TMC_10_CONFIG SMU__SBSW.TMC[10].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_10_DISTANCE SMU__SBSW.TMC[10].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_10_TIMEOUT SMU__SBSW.TMC[10].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_10_CONTROL SMU__SBSW.TMC[10].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_10_R0  SMU__SBSW.TMC[10].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_10_R1  SMU__SBSW.TMC[10].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_10_STATUS SMU__SBSW.TMC[10].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_10_TIMER SMU__SBSW.TMC[10].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_11_CONFIG SMU__SBSW.TMC[11].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_11_DISTANCE SMU__SBSW.TMC[11].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_11_TIMEOUT SMU__SBSW.TMC[11].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_11_CONTROL SMU__SBSW.TMC[11].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_11_R0  SMU__SBSW.TMC[11].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_11_R1  SMU__SBSW.TMC[11].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_11_STATUS SMU__SBSW.TMC[11].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_11_TIMER SMU__SBSW.TMC[11].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_12_CONFIG SMU__SBSW.TMC[12].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_12_DISTANCE SMU__SBSW.TMC[12].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_12_TIMEOUT SMU__SBSW.TMC[12].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_12_CONTROL SMU__SBSW.TMC[12].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_12_R0  SMU__SBSW.TMC[12].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_12_R1  SMU__SBSW.TMC[12].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_12_STATUS SMU__SBSW.TMC[12].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_12_TIMER SMU__SBSW.TMC[12].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_13_CONFIG SMU__SBSW.TMC[13].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_13_DISTANCE SMU__SBSW.TMC[13].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_13_TIMEOUT SMU__SBSW.TMC[13].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_13_CONTROL SMU__SBSW.TMC[13].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_13_R0  SMU__SBSW.TMC[13].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_13_R1  SMU__SBSW.TMC[13].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_13_STATUS SMU__SBSW.TMC[13].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_13_TIMER SMU__SBSW.TMC[13].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_14_CONFIG SMU__SBSW.TMC[14].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_14_DISTANCE SMU__SBSW.TMC[14].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_14_TIMEOUT SMU__SBSW.TMC[14].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_14_CONTROL SMU__SBSW.TMC[14].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_14_R0  SMU__SBSW.TMC[14].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_14_R1  SMU__SBSW.TMC[14].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_14_STATUS SMU__SBSW.TMC[14].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_14_TIMER SMU__SBSW.TMC[14].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_15_CONFIG SMU__SBSW.TMC[15].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_15_DISTANCE SMU__SBSW.TMC[15].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_15_TIMEOUT SMU__SBSW.TMC[15].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_15_CONTROL SMU__SBSW.TMC[15].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_15_R0  SMU__SBSW.TMC[15].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_15_R1  SMU__SBSW.TMC[15].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_15_STATUS SMU__SBSW.TMC[15].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_15_TIMER SMU__SBSW.TMC[15].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_16_CONFIG SMU__SBSW.TMC[16].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_16_DISTANCE SMU__SBSW.TMC[16].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_16_TIMEOUT SMU__SBSW.TMC[16].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_16_CONTROL SMU__SBSW.TMC[16].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_16_R0  SMU__SBSW.TMC[16].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_16_R1  SMU__SBSW.TMC[16].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_16_STATUS SMU__SBSW.TMC[16].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_16_TIMER SMU__SBSW.TMC[16].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_17_CONFIG SMU__SBSW.TMC[17].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_17_DISTANCE SMU__SBSW.TMC[17].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_17_TIMEOUT SMU__SBSW.TMC[17].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_17_CONTROL SMU__SBSW.TMC[17].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_17_R0  SMU__SBSW.TMC[17].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_17_R1  SMU__SBSW.TMC[17].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_17_STATUS SMU__SBSW.TMC[17].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_17_TIMER SMU__SBSW.TMC[17].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_18_CONFIG SMU__SBSW.TMC[18].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_18_DISTANCE SMU__SBSW.TMC[18].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_18_TIMEOUT SMU__SBSW.TMC[18].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_18_CONTROL SMU__SBSW.TMC[18].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_18_R0  SMU__SBSW.TMC[18].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_18_R1  SMU__SBSW.TMC[18].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_18_STATUS SMU__SBSW.TMC[18].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_18_TIMER SMU__SBSW.TMC[18].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_19_CONFIG SMU__SBSW.TMC[19].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_19_DISTANCE SMU__SBSW.TMC[19].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_19_TIMEOUT SMU__SBSW.TMC[19].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_19_CONTROL SMU__SBSW.TMC[19].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_19_R0  SMU__SBSW.TMC[19].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_19_R1  SMU__SBSW.TMC[19].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_19_STATUS SMU__SBSW.TMC[19].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_19_TIMER SMU__SBSW.TMC[19].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_20_CONFIG SMU__SBSW.TMC[20].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_20_DISTANCE SMU__SBSW.TMC[20].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_20_TIMEOUT SMU__SBSW.TMC[20].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_20_CONTROL SMU__SBSW.TMC[20].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_20_R0  SMU__SBSW.TMC[20].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_20_R1  SMU__SBSW.TMC[20].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_20_STATUS SMU__SBSW.TMC[20].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_20_TIMER SMU__SBSW.TMC[20].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_21_CONFIG SMU__SBSW.TMC[21].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_21_DISTANCE SMU__SBSW.TMC[21].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_21_TIMEOUT SMU__SBSW.TMC[21].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_21_CONTROL SMU__SBSW.TMC[21].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_21_R0  SMU__SBSW.TMC[21].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_21_R1  SMU__SBSW.TMC[21].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_21_STATUS SMU__SBSW.TMC[21].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_21_TIMER SMU__SBSW.TMC[21].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_22_CONFIG SMU__SBSW.TMC[22].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_22_DISTANCE SMU__SBSW.TMC[22].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_22_TIMEOUT SMU__SBSW.TMC[22].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_22_CONTROL SMU__SBSW.TMC[22].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_22_R0  SMU__SBSW.TMC[22].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_22_R1  SMU__SBSW.TMC[22].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_22_STATUS SMU__SBSW.TMC[22].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_22_TIMER SMU__SBSW.TMC[22].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_23_CONFIG SMU__SBSW.TMC[23].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_23_DISTANCE SMU__SBSW.TMC[23].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_23_TIMEOUT SMU__SBSW.TMC[23].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_23_CONTROL SMU__SBSW.TMC[23].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_23_R0  SMU__SBSW.TMC[23].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_23_R1  SMU__SBSW.TMC[23].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_23_STATUS SMU__SBSW.TMC[23].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_23_TIMER SMU__SBSW.TMC[23].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_24_CONFIG SMU__SBSW.TMC[24].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_24_DISTANCE SMU__SBSW.TMC[24].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_24_TIMEOUT SMU__SBSW.TMC[24].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_24_CONTROL SMU__SBSW.TMC[24].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_24_R0  SMU__SBSW.TMC[24].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_24_R1  SMU__SBSW.TMC[24].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_24_STATUS SMU__SBSW.TMC[24].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_24_TIMER SMU__SBSW.TMC[24].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_25_CONFIG SMU__SBSW.TMC[25].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_25_DISTANCE SMU__SBSW.TMC[25].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_25_TIMEOUT SMU__SBSW.TMC[25].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_25_CONTROL SMU__SBSW.TMC[25].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_25_R0  SMU__SBSW.TMC[25].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_25_R1  SMU__SBSW.TMC[25].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_25_STATUS SMU__SBSW.TMC[25].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_25_TIMER SMU__SBSW.TMC[25].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_26_CONFIG SMU__SBSW.TMC[26].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_26_DISTANCE SMU__SBSW.TMC[26].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_26_TIMEOUT SMU__SBSW.TMC[26].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_26_CONTROL SMU__SBSW.TMC[26].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_26_R0  SMU__SBSW.TMC[26].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_26_R1  SMU__SBSW.TMC[26].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_26_STATUS SMU__SBSW.TMC[26].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_26_TIMER SMU__SBSW.TMC[26].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_27_CONFIG SMU__SBSW.TMC[27].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_27_DISTANCE SMU__SBSW.TMC[27].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_27_TIMEOUT SMU__SBSW.TMC[27].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_27_CONTROL SMU__SBSW.TMC[27].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_27_R0  SMU__SBSW.TMC[27].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_27_R1  SMU__SBSW.TMC[27].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_27_STATUS SMU__SBSW.TMC[27].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_27_TIMER SMU__SBSW.TMC[27].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_28_CONFIG SMU__SBSW.TMC[28].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_28_DISTANCE SMU__SBSW.TMC[28].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_28_TIMEOUT SMU__SBSW.TMC[28].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_28_CONTROL SMU__SBSW.TMC[28].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_28_R0  SMU__SBSW.TMC[28].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_28_R1  SMU__SBSW.TMC[28].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_28_STATUS SMU__SBSW.TMC[28].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_28_TIMER SMU__SBSW.TMC[28].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_29_CONFIG SMU__SBSW.TMC[29].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_29_DISTANCE SMU__SBSW.TMC[29].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_29_TIMEOUT SMU__SBSW.TMC[29].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_29_CONTROL SMU__SBSW.TMC[29].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_29_R0  SMU__SBSW.TMC[29].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_29_R1  SMU__SBSW.TMC[29].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_29_STATUS SMU__SBSW.TMC[29].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_29_TIMER SMU__SBSW.TMC[29].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_30_CONFIG SMU__SBSW.TMC[30].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_30_DISTANCE SMU__SBSW.TMC[30].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_30_TIMEOUT SMU__SBSW.TMC[30].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_30_CONTROL SMU__SBSW.TMC[30].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_30_R0  SMU__SBSW.TMC[30].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_30_R1  SMU__SBSW.TMC[30].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_30_STATUS SMU__SBSW.TMC[30].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_30_TIMER SMU__SBSW.TMC[30].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMC_31_CONFIG SMU__SBSW.TMC[31].CONFIG.R /* TMC Configuration */
#define SMU__SBSW_TMC_31_DISTANCE SMU__SBSW.TMC[31].DISTANCE.R /* TMC Distance */
#define SMU__SBSW_TMC_31_TIMEOUT SMU__SBSW.TMC[31].TIMEOUT.R /* TMC Timeout */
#define SMU__SBSW_TMC_31_CONTROL SMU__SBSW.TMC[31].CONTROL.R /* TMC Control */
#define SMU__SBSW_TMC_31_R0  SMU__SBSW.TMC[31].R0.R        /* TMC R0 Data */
#define SMU__SBSW_TMC_31_R1  SMU__SBSW.TMC[31].R1.R        /* TMC R1 Data */
#define SMU__SBSW_TMC_31_STATUS SMU__SBSW.TMC[31].STATUS.R /* TMC Status */
#define SMU__SBSW_TMC_31_TIMER SMU__SBSW.TMC[31].TIMER.R   /* TMC Timer */
#define SMU__SBSW_TMWDP_CONFIG_ADDR SMU__SBSW.TMWDP_CONFIG_ADDR.R /* TMWDP Configuration Address */
#define SMU__SBSW_TMWDP_CONTROL SMU__SBSW.TMWDP_CONTROL.R  /* TMWDP Control */
#define SMU__SBSW_TMWDP_STATUS SMU__SBSW.TMWDP_STATUS.R    /* TMWDP Status */
#define SMU__SBSW_TMWDP_AUTOMATA_STATUS SMU__SBSW.TMWDP_AUTOMATA_STATUS.R /* TMWDP Automata Status */
#define SMU__SBSW_TMWDP_AUTOMATA_ILLGL_TRANS SMU__SBSW.TMWDP_AUTOMATA_ILLGL_TRANS.R /* TMWDP Automata Illegal Transition */
#define SMU__SBSW_TMWDP_AUTOMATA_TIME_VIOLATION SMU__SBSW.TMWDP_AUTOMATA_TIME_VIOLATION.R /* TMWDP Automata Time Violation */
#define SMU__SBSW_AUT_0_STATUS SMU__SBSW.AUT[0].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_0_PRGS_REQ SMU__SBSW.AUT[0].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_1_STATUS SMU__SBSW.AUT[1].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_1_PRGS_REQ SMU__SBSW.AUT[1].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_2_STATUS SMU__SBSW.AUT[2].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_2_PRGS_REQ SMU__SBSW.AUT[2].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_3_STATUS SMU__SBSW.AUT[3].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_3_PRGS_REQ SMU__SBSW.AUT[3].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_4_STATUS SMU__SBSW.AUT[4].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_4_PRGS_REQ SMU__SBSW.AUT[4].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_5_STATUS SMU__SBSW.AUT[5].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_5_PRGS_REQ SMU__SBSW.AUT[5].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_6_STATUS SMU__SBSW.AUT[6].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_6_PRGS_REQ SMU__SBSW.AUT[6].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_7_STATUS SMU__SBSW.AUT[7].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_7_PRGS_REQ SMU__SBSW.AUT[7].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_8_STATUS SMU__SBSW.AUT[8].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_8_PRGS_REQ SMU__SBSW.AUT[8].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_9_STATUS SMU__SBSW.AUT[9].STATUS.R   /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_9_PRGS_REQ SMU__SBSW.AUT[9].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_10_STATUS SMU__SBSW.AUT[10].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_10_PRGS_REQ SMU__SBSW.AUT[10].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_11_STATUS SMU__SBSW.AUT[11].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_11_PRGS_REQ SMU__SBSW.AUT[11].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_12_STATUS SMU__SBSW.AUT[12].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_12_PRGS_REQ SMU__SBSW.AUT[12].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_13_STATUS SMU__SBSW.AUT[13].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_13_PRGS_REQ SMU__SBSW.AUT[13].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_14_STATUS SMU__SBSW.AUT[14].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_14_PRGS_REQ SMU__SBSW.AUT[14].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_15_STATUS SMU__SBSW.AUT[15].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_15_PRGS_REQ SMU__SBSW.AUT[15].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_16_STATUS SMU__SBSW.AUT[16].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_16_PRGS_REQ SMU__SBSW.AUT[16].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_17_STATUS SMU__SBSW.AUT[17].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_17_PRGS_REQ SMU__SBSW.AUT[17].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_18_STATUS SMU__SBSW.AUT[18].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_18_PRGS_REQ SMU__SBSW.AUT[18].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_19_STATUS SMU__SBSW.AUT[19].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_19_PRGS_REQ SMU__SBSW.AUT[19].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_20_STATUS SMU__SBSW.AUT[20].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_20_PRGS_REQ SMU__SBSW.AUT[20].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_21_STATUS SMU__SBSW.AUT[21].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_21_PRGS_REQ SMU__SBSW.AUT[21].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_22_STATUS SMU__SBSW.AUT[22].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_22_PRGS_REQ SMU__SBSW.AUT[22].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_23_STATUS SMU__SBSW.AUT[23].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_23_PRGS_REQ SMU__SBSW.AUT[23].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_24_STATUS SMU__SBSW.AUT[24].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_24_PRGS_REQ SMU__SBSW.AUT[24].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_25_STATUS SMU__SBSW.AUT[25].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_25_PRGS_REQ SMU__SBSW.AUT[25].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_26_STATUS SMU__SBSW.AUT[26].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_26_PRGS_REQ SMU__SBSW.AUT[26].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_27_STATUS SMU__SBSW.AUT[27].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_27_PRGS_REQ SMU__SBSW.AUT[27].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_28_STATUS SMU__SBSW.AUT[28].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_28_PRGS_REQ SMU__SBSW.AUT[28].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_29_STATUS SMU__SBSW.AUT[29].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_29_PRGS_REQ SMU__SBSW.AUT[29].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_30_STATUS SMU__SBSW.AUT[30].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_30_PRGS_REQ SMU__SBSW.AUT[30].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_AUT_31_STATUS SMU__SBSW.AUT[31].STATUS.R /* TMWDP Automaton Status */
#define SMU__SBSW_AUT_31_PRGS_REQ SMU__SBSW.AUT[31].PRGS_REQ.R /* TMWDP Automaton Progress Request */
#define SMU__SBSW_TC_ID      SMU__SBSW.TC_ID.R             /* TMWDP Core Domain ID */

/* SDA_AP */
#define SDA_AP_AUTHSTTS      SDA_AP.AUTHSTTS.R             /* Authentication status register */
#define SDA_AP_AUTHCTL       SDA_AP.AUTHCTL.R              /* Authentication Control Register */
#define SDA_AP_KEYCHAL0      SDA_AP.KEYCHAL0.R             /* Key Challenge 0 register */
#define SDA_AP_KEYCHAL1      SDA_AP.KEYCHAL1.R             /* Key Challenge 1 register */
#define SDA_AP_KEYCHAL2      SDA_AP.KEYCHAL2.R             /* Key Challenge 2 register */
#define SDA_AP_KEYCHAL3      SDA_AP.KEYCHAL3.R             /* Key Challenge 3 register */
#define SDA_AP_KEYCHAL4      SDA_AP.KEYCHAL4.R             /* Key Challenge 4 register */
#define SDA_AP_KEYCHAL5      SDA_AP.KEYCHAL5.R             /* Key Challenge 5 register */
#define SDA_AP_KEYCHAL6      SDA_AP.KEYCHAL6.R             /* Key Challenge 6 register */
#define SDA_AP_KEYCHAL7      SDA_AP.KEYCHAL7.R             /* Key Challenge 7 register */
#define SDA_AP_KEYRESP0      SDA_AP.KEYRESP0.R             /* Key Response 0 register */
#define SDA_AP_KEYRESP1      SDA_AP.KEYRESP1.R             /* Key Response 1 register */
#define SDA_AP_KEYRESP2      SDA_AP.KEYRESP2.R             /* Key Response 2 register */
#define SDA_AP_KEYRESP3      SDA_AP.KEYRESP3.R             /* Key Response 3 register */
#define SDA_AP_KEYRESP4      SDA_AP.KEYRESP4.R             /* Key Response 4 register */
#define SDA_AP_KEYRESP5      SDA_AP.KEYRESP5.R             /* Key Response 5 register */
#define SDA_AP_KEYRESP6      SDA_AP.KEYRESP6.R             /* Key Response 6 register */
#define SDA_AP_KEYRESP7      SDA_AP.KEYRESP7.R             /* Key Response 7 register */
#define SDA_AP_SDAUIDL       SDA_AP.SDAUIDL.R              /* UID Low Register */
#define SDA_AP_SDAUIDH       SDA_AP.SDAUIDH.R              /* UID High Register */
#define SDA_AP_SDASYSRSTS    SDA_AP.SDASYSRSTS.R           /* System reset status register */
#define SDA_AP_SDASYSRSTC    SDA_AP.SDASYSRSTC.R           /* System Reset Control Register */
#define SDA_AP_SDARSTCTRL    SDA_AP.SDARSTCTRL.R           /* Reset Control Register */
#define SDA_AP_SDARSTMASK    SDA_AP.SDARSTMASK.R           /* Reset Control Mask Register */
#define SDA_AP_IDR           SDA_AP.IDR.R                  /* Identification Register */
#define SDA_AP_DEVARCH       SDA_AP.DEVARCH.R              /* CoreSight Device Architecture Register */
#define SDA_AP_DEVTYPE       SDA_AP.DEVTYPE.R              /* CoreSight Device Type Identifier Register */
#define SDA_AP_PIDR4         SDA_AP.PIDR4.R                /* CoreSight Peripheral Identification Register 4 */
#define SDA_AP_PIDR0         SDA_AP.PIDR0.R                /* CoreSight Peripheral Identification Register 0 */
#define SDA_AP_PIDR1         SDA_AP.PIDR1.R                /* CoreSight Peripheral Identification Register 1 */
#define SDA_AP_PIDR2         SDA_AP.PIDR2.R                /* CoreSight Peripheral Identification Register 2 */
#define SDA_AP_PIDR3         SDA_AP.PIDR3.R                /* CoreSight Peripheral Identification Register 3 */
#define SDA_AP_CIDR0         SDA_AP.CIDR0.R                /* CoreSight Component Identification Register 0 */
#define SDA_AP_CIDR1         SDA_AP.CIDR1.R                /* CoreSight Component Identification Register 1 */
#define SDA_AP_CIDR2         SDA_AP.CIDR2.R                /* CoreSight Component Identification Register 2 */
#define SDA_AP_CIDR3         SDA_AP.CIDR3.R                /* CoreSight Component Identification Register 3 */

/* SEC_S250 */
#define SEC_MISCDAT0         SEC.MISCDAT0.R                /* Miscellaneous Data 0 */
#define SEC_HSE_DAT0         SEC.HSE_DAT0.R                /* HSE_H Data 0 */
#define SEC_HSE_DAT2         SEC.HSE_DAT[0].R              /* HSE_H Data n */
#define SEC_HSE_DAT3         SEC.HSE_DAT[1].R              /* HSE_H Data n */
#define SEC_HSE_DAT4         SEC.HSE_DAT[2].R              /* HSE_H Data n */
#define SEC_HSE_DAT5         SEC.HSE_DAT[3].R              /* HSE_H Data n */
#define SEC_HSE_DAT6         SEC.HSE_DAT[4].R              /* HSE_H Data n */
#define SEC_HSE_DAT7         SEC.HSE_DAT[5].R              /* HSE_H Data n */
#define SEC_HSE_DAT8         SEC.HSE_DAT[6].R              /* HSE_H Data n */
#define SEC_HSE_DAT9         SEC.HSE_DAT[7].R              /* HSE_H Data n */
#define SEC_HSE_DAT10        SEC.HSE_DAT[8].R              /* HSE_H Data n */
#define SEC_HSE_DAT11        SEC.HSE_DAT[9].R              /* HSE_H Data n */
#define SEC_HSE_QSPI0_DAT0   SEC.HSE_QSPI0_DAT0.R          /* QuadSPI_0 Data 0 */
#define SEC_HSE_QSPI0_DAT1   SEC.HSE_QSPI0_DAT1.R          /* QuadSPI_0 Data 1 */
#define SEC_HSE_QSPI1_DAT0   SEC.HSE_QSPI1_DAT0.R          /* QuadSPI_1 Data 0 */
#define SEC_HSE_QSPI1_DAT1   SEC.HSE_QSPI1_DAT1.R          /* QuadSPI_1 Data 1 */

/* SEMA42 */
#define SEMA42_3_GATE3       SEMA42_3.GATE[0].R            /* Gate */
#define SEMA42_3_GATE2       SEMA42_3.GATE[1].R            /* Gate */
#define SEMA42_3_GATE1       SEMA42_3.GATE[2].R            /* Gate */
#define SEMA42_3_GATE0       SEMA42_3.GATE[3].R            /* Gate */
#define SEMA42_3_GATE7       SEMA42_3.GATE[4].R            /* Gate */
#define SEMA42_3_GATE6       SEMA42_3.GATE[5].R            /* Gate */
#define SEMA42_3_GATE5       SEMA42_3.GATE[6].R            /* Gate */
#define SEMA42_3_GATE4       SEMA42_3.GATE[7].R            /* Gate */
#define SEMA42_3_GATE11      SEMA42_3.GATE[8].R            /* Gate */
#define SEMA42_3_GATE10      SEMA42_3.GATE[9].R            /* Gate */
#define SEMA42_3_GATE9       SEMA42_3.GATE[10].R           /* Gate */
#define SEMA42_3_GATE8       SEMA42_3.GATE[11].R           /* Gate */
#define SEMA42_3_GATE15      SEMA42_3.GATE[12].R           /* Gate */
#define SEMA42_3_GATE14      SEMA42_3.GATE[13].R           /* Gate */
#define SEMA42_3_GATE13      SEMA42_3.GATE[14].R           /* Gate */
#define SEMA42_3_GATE12      SEMA42_3.GATE[15].R           /* Gate */
#define SEMA42_3_RSTGT_R     SEMA42_3.RSTGT.R.R            /* Reset Gate Read */
#define SEMA42_3_RSTGT_W     SEMA42_3.RSTGT.W.R            /* Reset Gate Write */

#define SEMA42_1_GATE3       SEMA42_1.GATE[0].R            /* Gate */
#define SEMA42_1_GATE2       SEMA42_1.GATE[1].R            /* Gate */
#define SEMA42_1_GATE1       SEMA42_1.GATE[2].R            /* Gate */
#define SEMA42_1_GATE0       SEMA42_1.GATE[3].R            /* Gate */
#define SEMA42_1_GATE7       SEMA42_1.GATE[4].R            /* Gate */
#define SEMA42_1_GATE6       SEMA42_1.GATE[5].R            /* Gate */
#define SEMA42_1_GATE5       SEMA42_1.GATE[6].R            /* Gate */
#define SEMA42_1_GATE4       SEMA42_1.GATE[7].R            /* Gate */
#define SEMA42_1_GATE11      SEMA42_1.GATE[8].R            /* Gate */
#define SEMA42_1_GATE10      SEMA42_1.GATE[9].R            /* Gate */
#define SEMA42_1_GATE9       SEMA42_1.GATE[10].R           /* Gate */
#define SEMA42_1_GATE8       SEMA42_1.GATE[11].R           /* Gate */
#define SEMA42_1_GATE15      SEMA42_1.GATE[12].R           /* Gate */
#define SEMA42_1_GATE14      SEMA42_1.GATE[13].R           /* Gate */
#define SEMA42_1_GATE13      SEMA42_1.GATE[14].R           /* Gate */
#define SEMA42_1_GATE12      SEMA42_1.GATE[15].R           /* Gate */
#define SEMA42_1_RSTGT_R     SEMA42_1.RSTGT.R.R            /* Reset Gate Read */
#define SEMA42_1_RSTGT_W     SEMA42_1.RSTGT.W.R            /* Reset Gate Write */

#define SEMA42_2_GATE3       SEMA42_2.GATE[0].R            /* Gate */
#define SEMA42_2_GATE2       SEMA42_2.GATE[1].R            /* Gate */
#define SEMA42_2_GATE1       SEMA42_2.GATE[2].R            /* Gate */
#define SEMA42_2_GATE0       SEMA42_2.GATE[3].R            /* Gate */
#define SEMA42_2_GATE7       SEMA42_2.GATE[4].R            /* Gate */
#define SEMA42_2_GATE6       SEMA42_2.GATE[5].R            /* Gate */
#define SEMA42_2_GATE5       SEMA42_2.GATE[6].R            /* Gate */
#define SEMA42_2_GATE4       SEMA42_2.GATE[7].R            /* Gate */
#define SEMA42_2_GATE11      SEMA42_2.GATE[8].R            /* Gate */
#define SEMA42_2_GATE10      SEMA42_2.GATE[9].R            /* Gate */
#define SEMA42_2_GATE9       SEMA42_2.GATE[10].R           /* Gate */
#define SEMA42_2_GATE8       SEMA42_2.GATE[11].R           /* Gate */
#define SEMA42_2_GATE15      SEMA42_2.GATE[12].R           /* Gate */
#define SEMA42_2_GATE14      SEMA42_2.GATE[13].R           /* Gate */
#define SEMA42_2_GATE13      SEMA42_2.GATE[14].R           /* Gate */
#define SEMA42_2_GATE12      SEMA42_2.GATE[15].R           /* Gate */
#define SEMA42_2_RSTGT_R     SEMA42_2.RSTGT.R.R            /* Reset Gate Read */
#define SEMA42_2_RSTGT_W     SEMA42_2.RSTGT.W.R            /* Reset Gate Write */

#define SEMA42_0_GATE3       SEMA42_0.GATE[0].R            /* Gate */
#define SEMA42_0_GATE2       SEMA42_0.GATE[1].R            /* Gate */
#define SEMA42_0_GATE1       SEMA42_0.GATE[2].R            /* Gate */
#define SEMA42_0_GATE0       SEMA42_0.GATE[3].R            /* Gate */
#define SEMA42_0_GATE7       SEMA42_0.GATE[4].R            /* Gate */
#define SEMA42_0_GATE6       SEMA42_0.GATE[5].R            /* Gate */
#define SEMA42_0_GATE5       SEMA42_0.GATE[6].R            /* Gate */
#define SEMA42_0_GATE4       SEMA42_0.GATE[7].R            /* Gate */
#define SEMA42_0_GATE11      SEMA42_0.GATE[8].R            /* Gate */
#define SEMA42_0_GATE10      SEMA42_0.GATE[9].R            /* Gate */
#define SEMA42_0_GATE9       SEMA42_0.GATE[10].R           /* Gate */
#define SEMA42_0_GATE8       SEMA42_0.GATE[11].R           /* Gate */
#define SEMA42_0_GATE15      SEMA42_0.GATE[12].R           /* Gate */
#define SEMA42_0_GATE14      SEMA42_0.GATE[13].R           /* Gate */
#define SEMA42_0_GATE13      SEMA42_0.GATE[14].R           /* Gate */
#define SEMA42_0_GATE12      SEMA42_0.GATE[15].R           /* Gate */
#define SEMA42_0_RSTGT_R     SEMA42_0.RSTGT.R.R            /* Reset Gate Read */
#define SEMA42_0_RSTGT_W     SEMA42_0.RSTGT.W.R            /* Reset Gate Write */

#define SEMA42_4_GATE3       SEMA42_4.GATE[0].R            /* Gate */
#define SEMA42_4_GATE2       SEMA42_4.GATE[1].R            /* Gate */
#define SEMA42_4_GATE1       SEMA42_4.GATE[2].R            /* Gate */
#define SEMA42_4_GATE0       SEMA42_4.GATE[3].R            /* Gate */
#define SEMA42_4_GATE7       SEMA42_4.GATE[4].R            /* Gate */
#define SEMA42_4_GATE6       SEMA42_4.GATE[5].R            /* Gate */
#define SEMA42_4_GATE5       SEMA42_4.GATE[6].R            /* Gate */
#define SEMA42_4_GATE4       SEMA42_4.GATE[7].R            /* Gate */
#define SEMA42_4_GATE11      SEMA42_4.GATE[8].R            /* Gate */
#define SEMA42_4_GATE10      SEMA42_4.GATE[9].R            /* Gate */
#define SEMA42_4_GATE9       SEMA42_4.GATE[10].R           /* Gate */
#define SEMA42_4_GATE8       SEMA42_4.GATE[11].R           /* Gate */
#define SEMA42_4_GATE15      SEMA42_4.GATE[12].R           /* Gate */
#define SEMA42_4_GATE14      SEMA42_4.GATE[13].R           /* Gate */
#define SEMA42_4_GATE13      SEMA42_4.GATE[14].R           /* Gate */
#define SEMA42_4_GATE12      SEMA42_4.GATE[15].R           /* Gate */
#define SEMA42_4_RSTGT_R     SEMA42_4.RSTGT.R.R            /* Reset Gate Read */
#define SEMA42_4_RSTGT_W     SEMA42_4.RSTGT.W.R            /* Reset Gate Write */

#define SEMA42_5_GATE3       SEMA42_5.GATE[0].R            /* Gate */
#define SEMA42_5_GATE2       SEMA42_5.GATE[1].R            /* Gate */
#define SEMA42_5_GATE1       SEMA42_5.GATE[2].R            /* Gate */
#define SEMA42_5_GATE0       SEMA42_5.GATE[3].R            /* Gate */
#define SEMA42_5_GATE7       SEMA42_5.GATE[4].R            /* Gate */
#define SEMA42_5_GATE6       SEMA42_5.GATE[5].R            /* Gate */
#define SEMA42_5_GATE5       SEMA42_5.GATE[6].R            /* Gate */
#define SEMA42_5_GATE4       SEMA42_5.GATE[7].R            /* Gate */
#define SEMA42_5_GATE11      SEMA42_5.GATE[8].R            /* Gate */
#define SEMA42_5_GATE10      SEMA42_5.GATE[9].R            /* Gate */
#define SEMA42_5_GATE9       SEMA42_5.GATE[10].R           /* Gate */
#define SEMA42_5_GATE8       SEMA42_5.GATE[11].R           /* Gate */
#define SEMA42_5_GATE15      SEMA42_5.GATE[12].R           /* Gate */
#define SEMA42_5_GATE14      SEMA42_5.GATE[13].R           /* Gate */
#define SEMA42_5_GATE13      SEMA42_5.GATE[14].R           /* Gate */
#define SEMA42_5_GATE12      SEMA42_5.GATE[15].R           /* Gate */
#define SEMA42_5_RSTGT_R     SEMA42_5.RSTGT.R.R            /* Reset Gate Read */
#define SEMA42_5_RSTGT_W     SEMA42_5.RSTGT.W.R            /* Reset Gate Write */

/* SINC */
#define SINC_VERID           SINC.VERID.R                  /* Version ID */
#define SINC_PARAMETER       SINC.PARAMETER.R              /* Parameters */
#define SINC_MCR             SINC.MCR.R                    /* Main Control */
#define SINC_NIE             SINC.NIE.R                    /* Normal Interrupt Enable */
#define SINC_EIE             SINC.EIE.R                    /* Error Interrupt Enable */
#define SINC_FIFOIE          SINC.FIFOIE.R                 /* FIFO And CAD Error Interrupt Enable */
#define SINC_NIS             SINC.NIS.R                    /* Normal Interrupt Status */
#define SINC_EIS             SINC.EIS.R                    /* Error Interrupt Status */
#define SINC_FIFOIS          SINC.FIFOIS.R                 /* FIFO And CAD Error Interrupt Status */
#define SINC_SR              SINC.SR.R                     /* Status */
#define SINC_C0CR            SINC.CHANNEL[0].CCR.R         /* Channel 0 Control */
#define SINC_C0DR            SINC.CHANNEL[0].CDR.R         /* Channel 0 Data Rate */
#define SINC_C0CFR           SINC.CHANNEL[0].CCFR.R        /* Channel 0 Configuration */
#define SINC_C0PROT          SINC.CHANNEL[0].CPROT.R       /* Channel 0 Protection */
#define SINC_C0BIAS          SINC.CHANNEL[0].CBIAS.R       /* Channel 0 Bias */
#define SINC_C0LOLMT         SINC.CHANNEL[0].CLOLMT.R      /* Channel 0 Low Limit */
#define SINC_C0HILMT         SINC.CHANNEL[0].CHILMT.R      /* Channel 0 High Limit */
#define SINC_C0RDATA         SINC.CHANNEL[0].CRDATA.R      /* Channel 0 Result Data */
#define SINC_C0MPDATA        SINC.CHANNEL[0].CMPDATA.R     /* Channel 0 Multipurpose Data */
#define SINC_C0ACFR          SINC.CHANNEL[0].CACFR.R       /* Channel 0 Advanced Configuration */
#define SINC_C0SR            SINC.CHANNEL[0].CSR.R         /* Channel 0 Status */
#define SINC_C0DBGR          SINC.CHANNEL[0].CDBGR.R       /* Channel 0 Debug */
#define SINC_C1CR            SINC.CHANNEL[1].CCR.R         /* Channel 1 Control */
#define SINC_C1DR            SINC.CHANNEL[1].CDR.R         /* Channel 1 Data Rate */
#define SINC_C1CFR           SINC.CHANNEL[1].CCFR.R        /* Channel 1 Configuration */
#define SINC_C1PROT          SINC.CHANNEL[1].CPROT.R       /* Channel 1 Protection */
#define SINC_C1BIAS          SINC.CHANNEL[1].CBIAS.R       /* Channel 1 Bias */
#define SINC_C1LOLMT         SINC.CHANNEL[1].CLOLMT.R      /* Channel 1 Low Limit */
#define SINC_C1HILMT         SINC.CHANNEL[1].CHILMT.R      /* Channel 1 High Limit */
#define SINC_C1RDATA         SINC.CHANNEL[1].CRDATA.R      /* Channel 1 Result Data */
#define SINC_C1MPDATA        SINC.CHANNEL[1].CMPDATA.R     /* Channel 1 Multipurpose Data */
#define SINC_C1ACFR          SINC.CHANNEL[1].CACFR.R       /* Channel 1 Advanced Configuration */
#define SINC_C1SR            SINC.CHANNEL[1].CSR.R         /* Channel 1 Status */
#define SINC_C1DBGR          SINC.CHANNEL[1].CDBGR.R       /* Channel 1 Debug */
#define SINC_C2CR            SINC.CHANNEL[2].CCR.R         /* Channel 2 Control */
#define SINC_C2DR            SINC.CHANNEL[2].CDR.R         /* Channel 2 Data Rate */
#define SINC_C2CFR           SINC.CHANNEL[2].CCFR.R        /* Channel 2 Configuration */
#define SINC_C2PROT          SINC.CHANNEL[2].CPROT.R       /* Channel 2 Protection */
#define SINC_C2BIAS          SINC.CHANNEL[2].CBIAS.R       /* Channel 2 Bias */
#define SINC_C2LOLMT         SINC.CHANNEL[2].CLOLMT.R      /* Channel 2 Low Limit */
#define SINC_C2HILMT         SINC.CHANNEL[2].CHILMT.R      /* Channel 2 High Limit */
#define SINC_C2RDATA         SINC.CHANNEL[2].CRDATA.R      /* Channel 2 Result Data */
#define SINC_C2MPDATA        SINC.CHANNEL[2].CMPDATA.R     /* Channel 2 Multipurpose Data */
#define SINC_C2ACFR          SINC.CHANNEL[2].CACFR.R       /* Channel 2 Advanced Configuration */
#define SINC_C2SR            SINC.CHANNEL[2].CSR.R         /* Channel 2 Status */
#define SINC_C2DBGR          SINC.CHANNEL[2].CDBGR.R       /* Channel 2 Debug */
#define SINC_C3CR            SINC.CHANNEL[3].CCR.R         /* Channel 3 Control */
#define SINC_C3DR            SINC.CHANNEL[3].CDR.R         /* Channel 3 Data Rate */
#define SINC_C3CFR           SINC.CHANNEL[3].CCFR.R        /* Channel 3 Configuration */
#define SINC_C3PROT          SINC.CHANNEL[3].CPROT.R       /* Channel 3 Protection */
#define SINC_C3BIAS          SINC.CHANNEL[3].CBIAS.R       /* Channel 3 Bias */
#define SINC_C3LOLMT         SINC.CHANNEL[3].CLOLMT.R      /* Channel 3 Low Limit */
#define SINC_C3HILMT         SINC.CHANNEL[3].CHILMT.R      /* Channel 3 High Limit */
#define SINC_C3RDATA         SINC.CHANNEL[3].CRDATA.R      /* Channel 3 Result Data */
#define SINC_C3MPDATA        SINC.CHANNEL[3].CMPDATA.R     /* Channel 3 Multipurpose Data */
#define SINC_C3ACFR          SINC.CHANNEL[3].CACFR.R       /* Channel 3 Advanced Configuration */
#define SINC_C3SR            SINC.CHANNEL[3].CSR.R         /* Channel 3 Status */
#define SINC_C3DBGR          SINC.CHANNEL[3].CDBGR.R       /* Channel 3 Debug */

/* SIPI */
#define SIPI_0_CCR0          SIPI_0.CCR0.R                 /* SIPI Channel Control Register 0 */
#define SIPI_0_CSR0          SIPI_0.CSR0.R                 /* SIPI Channel Status Register 0 */
#define SIPI_0_CIR0          SIPI_0.CIR0.R                 /* SIPI Channel Interrupt Register 0 */
#define SIPI_0_CTOR0         SIPI_0.CTOR0.R                /* SIPI Channel Timeout Register 0 */
#define SIPI_0_CCRC0         SIPI_0.CCRC0.R                /* SIPI Channel CRC Register 0 */
#define SIPI_0_CAR0          SIPI_0.CAR0.R                 /* SIPI Channel Address Register 0 */
#define SIPI_0_CDR0          SIPI_0.CDR0.R                 /* SIPI Channel Data Register 0 */
#define SIPI_0_CCR1          SIPI_0.CCR1.R                 /* SIPI Channel Control Register 1 */
#define SIPI_0_CSR1          SIPI_0.CSR1.R                 /* SIPI Channel Status Register 1 */
#define SIPI_0_CIR1          SIPI_0.CIR1.R                 /* SIPI Channel Interrupt Register 1 */
#define SIPI_0_CTOR1         SIPI_0.CTOR1.R                /* SIPI Channel Timeout Register 1 */
#define SIPI_0_CCRC1         SIPI_0.CCRC1.R                /* SIPI Channel CRC Register 1 */
#define SIPI_0_CAR1          SIPI_0.CAR1.R                 /* SIPI Channel Address Register 1 */
#define SIPI_0_CDR1          SIPI_0.CDR1.R                 /* SIPI Channel Data Register 1 */
#define SIPI_0_CCR2          SIPI_0.CCR2.R                 /* SIPI Channel Control Register 2 */
#define SIPI_0_CSR2          SIPI_0.CSR2.R                 /* SIPI Channel Status Register 2 */
#define SIPI_0_CIR2          SIPI_0.CIR2.R                 /* SIPI Channel Interrupt Register 2 */
#define SIPI_0_CTOR2         SIPI_0.CTOR2.R                /* SIPI Channel Timeout Register 2 */
#define SIPI_0_CCRC2         SIPI_0.CCRC2.R                /* SIPI Channel CRC Register 2 */
#define SIPI_0_CAR2          SIPI_0.CAR2.R                 /* SIPI Channel Address Register 2 */
#define SIPI_0_CDR2_0        SIPI_0.CDR2[0].R              /* SIPI Channel Data Register 2 */
#define SIPI_0_CDR2_1        SIPI_0.CDR2[1].R              /* SIPI Channel Data Register 2 */
#define SIPI_0_CDR2_2        SIPI_0.CDR2[2].R              /* SIPI Channel Data Register 2 */
#define SIPI_0_CDR2_3        SIPI_0.CDR2[3].R              /* SIPI Channel Data Register 2 */
#define SIPI_0_CDR2_4        SIPI_0.CDR2[4].R              /* SIPI Channel Data Register 2 */
#define SIPI_0_CDR2_5        SIPI_0.CDR2[5].R              /* SIPI Channel Data Register 2 */
#define SIPI_0_CDR2_6        SIPI_0.CDR2[6].R              /* SIPI Channel Data Register 2 */
#define SIPI_0_CDR2_7        SIPI_0.CDR2[7].R              /* SIPI Channel Data Register 2 */
#define SIPI_0_CCR3          SIPI_0.CCR3.R                 /* SIPI Channel Control Register 3 */
#define SIPI_0_CSR3          SIPI_0.CSR3.R                 /* SIPI Channel Status Register 3 */
#define SIPI_0_CIR3          SIPI_0.CIR3.R                 /* SIPI Channel Interrupt Register 3 */
#define SIPI_0_CTOR3         SIPI_0.CTOR3.R                /* SIPI Channel Timeout Register 3 */
#define SIPI_0_CCRC3         SIPI_0.CCRC3.R                /* SIPI Channel CRC Register 3 */
#define SIPI_0_CAR3          SIPI_0.CAR3.R                 /* SIPI Channel Address Register 3 */
#define SIPI_0_CDR3          SIPI_0.CDR3.R                 /* SIPI Channel Data Register 3 */
#define SIPI_0_MCR           SIPI_0.MCR.R                  /* SIPI Module Configuration Register */
#define SIPI_0_SR            SIPI_0.SR.R                   /* SIPI Status Register */
#define SIPI_0_MAXCR         SIPI_0.MAXCR.R                /* SIPI Max Count Register */
#define SIPI_0_ARR           SIPI_0.ARR.R                  /* SIPI Address Reload Register */
#define SIPI_0_ACR           SIPI_0.ACR.R                  /* SIPI Address Count Register */
#define SIPI_0_ERR           SIPI_0.ERR.R                  /* SIPI Error Register */
#define SIPI_0_DEBUG         SIPI_0.DEBUG.R                /* SIPI Debug Register */
#define SIPI_0_PMR           SIPI_0.PMR.R                  /* SIPI Performance Monitor register */

#define SIPI_1_CCR0          SIPI_1.CCR0.R                 /* SIPI Channel Control Register 0 */
#define SIPI_1_CSR0          SIPI_1.CSR0.R                 /* SIPI Channel Status Register 0 */
#define SIPI_1_CIR0          SIPI_1.CIR0.R                 /* SIPI Channel Interrupt Register 0 */
#define SIPI_1_CTOR0         SIPI_1.CTOR0.R                /* SIPI Channel Timeout Register 0 */
#define SIPI_1_CCRC0         SIPI_1.CCRC0.R                /* SIPI Channel CRC Register 0 */
#define SIPI_1_CAR0          SIPI_1.CAR0.R                 /* SIPI Channel Address Register 0 */
#define SIPI_1_CDR0          SIPI_1.CDR0.R                 /* SIPI Channel Data Register 0 */
#define SIPI_1_CCR1          SIPI_1.CCR1.R                 /* SIPI Channel Control Register 1 */
#define SIPI_1_CSR1          SIPI_1.CSR1.R                 /* SIPI Channel Status Register 1 */
#define SIPI_1_CIR1          SIPI_1.CIR1.R                 /* SIPI Channel Interrupt Register 1 */
#define SIPI_1_CTOR1         SIPI_1.CTOR1.R                /* SIPI Channel Timeout Register 1 */
#define SIPI_1_CCRC1         SIPI_1.CCRC1.R                /* SIPI Channel CRC Register 1 */
#define SIPI_1_CAR1          SIPI_1.CAR1.R                 /* SIPI Channel Address Register 1 */
#define SIPI_1_CDR1          SIPI_1.CDR1.R                 /* SIPI Channel Data Register 1 */
#define SIPI_1_CCR2          SIPI_1.CCR2.R                 /* SIPI Channel Control Register 2 */
#define SIPI_1_CSR2          SIPI_1.CSR2.R                 /* SIPI Channel Status Register 2 */
#define SIPI_1_CIR2          SIPI_1.CIR2.R                 /* SIPI Channel Interrupt Register 2 */
#define SIPI_1_CTOR2         SIPI_1.CTOR2.R                /* SIPI Channel Timeout Register 2 */
#define SIPI_1_CCRC2         SIPI_1.CCRC2.R                /* SIPI Channel CRC Register 2 */
#define SIPI_1_CAR2          SIPI_1.CAR2.R                 /* SIPI Channel Address Register 2 */
#define SIPI_1_CDR2_0        SIPI_1.CDR2[0].R              /* SIPI Channel Data Register 2 */
#define SIPI_1_CDR2_1        SIPI_1.CDR2[1].R              /* SIPI Channel Data Register 2 */
#define SIPI_1_CDR2_2        SIPI_1.CDR2[2].R              /* SIPI Channel Data Register 2 */
#define SIPI_1_CDR2_3        SIPI_1.CDR2[3].R              /* SIPI Channel Data Register 2 */
#define SIPI_1_CDR2_4        SIPI_1.CDR2[4].R              /* SIPI Channel Data Register 2 */
#define SIPI_1_CDR2_5        SIPI_1.CDR2[5].R              /* SIPI Channel Data Register 2 */
#define SIPI_1_CDR2_6        SIPI_1.CDR2[6].R              /* SIPI Channel Data Register 2 */
#define SIPI_1_CDR2_7        SIPI_1.CDR2[7].R              /* SIPI Channel Data Register 2 */
#define SIPI_1_CCR3          SIPI_1.CCR3.R                 /* SIPI Channel Control Register 3 */
#define SIPI_1_CSR3          SIPI_1.CSR3.R                 /* SIPI Channel Status Register 3 */
#define SIPI_1_CIR3          SIPI_1.CIR3.R                 /* SIPI Channel Interrupt Register 3 */
#define SIPI_1_CTOR3         SIPI_1.CTOR3.R                /* SIPI Channel Timeout Register 3 */
#define SIPI_1_CCRC3         SIPI_1.CCRC3.R                /* SIPI Channel CRC Register 3 */
#define SIPI_1_CAR3          SIPI_1.CAR3.R                 /* SIPI Channel Address Register 3 */
#define SIPI_1_CDR3          SIPI_1.CDR3.R                 /* SIPI Channel Data Register 3 */
#define SIPI_1_MCR           SIPI_1.MCR.R                  /* SIPI Module Configuration Register */
#define SIPI_1_SR            SIPI_1.SR.R                   /* SIPI Status Register */
#define SIPI_1_MAXCR         SIPI_1.MAXCR.R                /* SIPI Max Count Register */
#define SIPI_1_ARR           SIPI_1.ARR.R                  /* SIPI Address Reload Register */
#define SIPI_1_ACR           SIPI_1.ACR.R                  /* SIPI Address Count Register */
#define SIPI_1_ERR           SIPI_1.ERR.R                  /* SIPI Error Register */
#define SIPI_1_DEBUG         SIPI_1.DEBUG.R                /* SIPI Debug Register */
#define SIPI_1_PMR           SIPI_1.PMR.R                  /* SIPI Performance Monitor register */

/* SIUL2_0 */
#define SIUL2_0_DISR0        SIUL2_0.DISR0.R               /* SIUL2 DMA/Interrupt Status Flag 0 */
#define SIUL2_0_DIRER0       SIUL2_0.DIRER0.R              /* SIUL2 DMA/Interrupt Request Enable 0 */
#define SIUL2_0_DIRSR0       SIUL2_0.DIRSR0.R              /* SIUL2 DMA/Interrupt Request Select 0 */
#define SIUL2_0_IREER0       SIUL2_0.IREER0.R              /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
#define SIUL2_0_IFEER0       SIUL2_0.IFEER0.R              /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
#define SIUL2_0_IFER0        SIUL2_0.IFER0.R               /* SIUL2 Interrupt Filter Enable 0 */
#define SIUL2_0_IFMCR0       SIUL2_0.IFMCR[0].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_0_IFMCR1       SIUL2_0.IFMCR[1].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_0_IFMCR2       SIUL2_0.IFMCR[2].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_0_IFMCR3       SIUL2_0.IFMCR[3].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_0_IFMCR4       SIUL2_0.IFMCR[4].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_0_IFMCR5       SIUL2_0.IFMCR[5].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_0_IFMCR6       SIUL2_0.IFMCR[6].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_0_IFMCR7       SIUL2_0.IFMCR[7].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_0_IFCPR        SIUL2_0.IFCPR.R               /* SIUL2 Interrupt Filter Clock Prescaler */
#define SIUL2_0_MSCR0        SIUL2_0.MSCR[0].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR1        SIUL2_0.MSCR[1].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR2        SIUL2_0.MSCR[2].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR3        SIUL2_0.MSCR[3].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR4        SIUL2_0.MSCR[4].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR5        SIUL2_0.MSCR[5].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR6        SIUL2_0.MSCR[6].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR7        SIUL2_0.MSCR[7].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR8        SIUL2_0.MSCR[8].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR9        SIUL2_0.MSCR[9].R             /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR10       SIUL2_0.MSCR[10].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR11       SIUL2_0.MSCR[11].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR12       SIUL2_0.MSCR[12].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR13       SIUL2_0.MSCR[13].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR14       SIUL2_0.MSCR[14].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR15       SIUL2_0.MSCR[15].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR16       SIUL2_0.MSCR[16].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR17       SIUL2_0.MSCR[17].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR18       SIUL2_0.MSCR[18].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR19       SIUL2_0.MSCR[19].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR20       SIUL2_0.MSCR[20].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR21       SIUL2_0.MSCR[21].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR22       SIUL2_0.MSCR[22].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR23       SIUL2_0.MSCR[23].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR24       SIUL2_0.MSCR[24].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR25       SIUL2_0.MSCR[25].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR26       SIUL2_0.MSCR[26].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR27       SIUL2_0.MSCR[27].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR28       SIUL2_0.MSCR[28].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR29       SIUL2_0.MSCR[29].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR30       SIUL2_0.MSCR[30].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR170      SIUL2_0.MSCR[170].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR171      SIUL2_0.MSCR[171].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR172      SIUL2_0.MSCR[172].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR173      SIUL2_0.MSCR[173].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR500      SIUL2_0.MSCR[500].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR501      SIUL2_0.MSCR[501].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_MSCR502      SIUL2_0.MSCR[502].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_0_IMCR0        SIUL2_0.IMCR[0].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR1        SIUL2_0.IMCR[1].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR2        SIUL2_0.IMCR[2].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR3        SIUL2_0.IMCR[3].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR4        SIUL2_0.IMCR[4].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR5        SIUL2_0.IMCR[5].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR6        SIUL2_0.IMCR[6].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR7        SIUL2_0.IMCR[7].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR8        SIUL2_0.IMCR[8].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR9        SIUL2_0.IMCR[9].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR10       SIUL2_0.IMCR[10].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR11       SIUL2_0.IMCR[11].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR12       SIUL2_0.IMCR[12].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR13       SIUL2_0.IMCR[13].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR14       SIUL2_0.IMCR[14].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR15       SIUL2_0.IMCR[15].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR16       SIUL2_0.IMCR[16].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR17       SIUL2_0.IMCR[17].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR18       SIUL2_0.IMCR[18].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR19       SIUL2_0.IMCR[19].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR20       SIUL2_0.IMCR[20].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR21       SIUL2_0.IMCR[21].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR22       SIUL2_0.IMCR[22].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR23       SIUL2_0.IMCR[23].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR24       SIUL2_0.IMCR[24].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR25       SIUL2_0.IMCR[25].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR26       SIUL2_0.IMCR[26].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR27       SIUL2_0.IMCR[27].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR28       SIUL2_0.IMCR[28].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR29       SIUL2_0.IMCR[29].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR30       SIUL2_0.IMCR[30].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR31       SIUL2_0.IMCR[31].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR32       SIUL2_0.IMCR[32].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR33       SIUL2_0.IMCR[33].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR34       SIUL2_0.IMCR[34].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR35       SIUL2_0.IMCR[35].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR36       SIUL2_0.IMCR[36].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR37       SIUL2_0.IMCR[37].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR38       SIUL2_0.IMCR[38].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR39       SIUL2_0.IMCR[39].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR40       SIUL2_0.IMCR[40].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR41       SIUL2_0.IMCR[41].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR42       SIUL2_0.IMCR[42].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR43       SIUL2_0.IMCR[43].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR44       SIUL2_0.IMCR[44].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR45       SIUL2_0.IMCR[45].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR46       SIUL2_0.IMCR[46].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR47       SIUL2_0.IMCR[47].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR48       SIUL2_0.IMCR[48].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR49       SIUL2_0.IMCR[49].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR50       SIUL2_0.IMCR[50].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR51       SIUL2_0.IMCR[51].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR55       SIUL2_0.IMCR[55].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR56       SIUL2_0.IMCR[56].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR57       SIUL2_0.IMCR[57].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR58       SIUL2_0.IMCR[58].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR59       SIUL2_0.IMCR[59].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR60       SIUL2_0.IMCR[60].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR61       SIUL2_0.IMCR[61].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR62       SIUL2_0.IMCR[62].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR63       SIUL2_0.IMCR[63].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR64       SIUL2_0.IMCR[64].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR65       SIUL2_0.IMCR[65].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR66       SIUL2_0.IMCR[66].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR67       SIUL2_0.IMCR[67].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR68       SIUL2_0.IMCR[68].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR69       SIUL2_0.IMCR[69].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR70       SIUL2_0.IMCR[70].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR71       SIUL2_0.IMCR[71].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR72       SIUL2_0.IMCR[72].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR73       SIUL2_0.IMCR[73].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR74       SIUL2_0.IMCR[74].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR75       SIUL2_0.IMCR[75].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR76       SIUL2_0.IMCR[76].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR77       SIUL2_0.IMCR[77].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR78       SIUL2_0.IMCR[78].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR79       SIUL2_0.IMCR[79].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR80       SIUL2_0.IMCR[80].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR81       SIUL2_0.IMCR[81].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR82       SIUL2_0.IMCR[82].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR83       SIUL2_0.IMCR[83].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR84       SIUL2_0.IMCR[84].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR85       SIUL2_0.IMCR[85].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR86       SIUL2_0.IMCR[86].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR87       SIUL2_0.IMCR[87].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR88       SIUL2_0.IMCR[88].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_IMCR89       SIUL2_0.IMCR[89].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_0_GPDO3        SIUL2_0.GPDO3.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO2        SIUL2_0.GPDO2.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO1        SIUL2_0.GPDO1.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO0        SIUL2_0.GPDO0.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO7        SIUL2_0.GPDO7.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO6        SIUL2_0.GPDO6.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO5        SIUL2_0.GPDO5.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO4        SIUL2_0.GPDO4.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO11       SIUL2_0.GPDO11.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO10       SIUL2_0.GPDO10.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO9        SIUL2_0.GPDO9.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO8        SIUL2_0.GPDO8.R               /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO15       SIUL2_0.GPDO15.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO14       SIUL2_0.GPDO14.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO13       SIUL2_0.GPDO13.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO12       SIUL2_0.GPDO12.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO19       SIUL2_0.GPDO19.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO18       SIUL2_0.GPDO18.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO17       SIUL2_0.GPDO17.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO16       SIUL2_0.GPDO16.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO23       SIUL2_0.GPDO23.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO22       SIUL2_0.GPDO22.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO21       SIUL2_0.GPDO21.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO20       SIUL2_0.GPDO20.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO27       SIUL2_0.GPDO27.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO26       SIUL2_0.GPDO26.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO25       SIUL2_0.GPDO25.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO24       SIUL2_0.GPDO24.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO30       SIUL2_0.GPDO30.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO29       SIUL2_0.GPDO29.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO28       SIUL2_0.GPDO28.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO171      SIUL2_0.GPDO171.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO170      SIUL2_0.GPDO170.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO173      SIUL2_0.GPDO173.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDO172      SIUL2_0.GPDO172.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_0_GPDI3        SIUL2_0.GPDI3.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI2        SIUL2_0.GPDI2.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI1        SIUL2_0.GPDI1.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI0        SIUL2_0.GPDI0.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI7        SIUL2_0.GPDI7.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI6        SIUL2_0.GPDI6.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI5        SIUL2_0.GPDI5.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI4        SIUL2_0.GPDI4.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI11       SIUL2_0.GPDI11.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI10       SIUL2_0.GPDI10.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI9        SIUL2_0.GPDI9.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI8        SIUL2_0.GPDI8.R               /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI15       SIUL2_0.GPDI15.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI14       SIUL2_0.GPDI14.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI13       SIUL2_0.GPDI13.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI12       SIUL2_0.GPDI12.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI19       SIUL2_0.GPDI19.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI18       SIUL2_0.GPDI18.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI17       SIUL2_0.GPDI17.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI16       SIUL2_0.GPDI16.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI23       SIUL2_0.GPDI23.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI22       SIUL2_0.GPDI22.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI21       SIUL2_0.GPDI21.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI20       SIUL2_0.GPDI20.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI27       SIUL2_0.GPDI27.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI26       SIUL2_0.GPDI26.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI25       SIUL2_0.GPDI25.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI24       SIUL2_0.GPDI24.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI30       SIUL2_0.GPDI30.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI29       SIUL2_0.GPDI29.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI28       SIUL2_0.GPDI28.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI171      SIUL2_0.GPDI171.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI170      SIUL2_0.GPDI170.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI173      SIUL2_0.GPDI173.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_GPDI172      SIUL2_0.GPDI172.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_0_PGPDO1       SIUL2_0.PGPDO1.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_0_PGPDO0       SIUL2_0.PGPDO0.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_0_PGPDO10      SIUL2_0.PGPDO10.R             /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_0_PGPDI1       SIUL2_0.PGPDI1.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_0_PGPDI0       SIUL2_0.PGPDI0.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_0_PGPDI10      SIUL2_0.PGPDI10.R             /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_0_MPGPDO0      SIUL2_0.MPGPDO[0].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_0_MPGPDO1      SIUL2_0.MPGPDO[1].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_0_MPGPDO10     SIUL2_0.MPGPDO[10].R          /* SIUL2 Masked Parallel GPIO Pad Data Out */

/* SIUL2_1 */
#define SIUL2_1_DISR0        SIUL2_1.DISR0.R               /* SIUL2 DMA/Interrupt Status Flag 0 */
#define SIUL2_1_DIRER0       SIUL2_1.DIRER0.R              /* SIUL2 DMA/Interrupt Request Enable 0 */
#define SIUL2_1_DIRSR0       SIUL2_1.DIRSR0.R              /* SIUL2 DMA/Interrupt Request Select 0 */
#define SIUL2_1_IREER0       SIUL2_1.IREER0.R              /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
#define SIUL2_1_IFEER0       SIUL2_1.IFEER0.R              /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
#define SIUL2_1_IFER0        SIUL2_1.IFER0.R               /* SIUL2 Interrupt Filter Enable 0 */
#define SIUL2_1_IFMCR0       SIUL2_1.IFMCR[0].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_1_IFMCR1       SIUL2_1.IFMCR[1].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_1_IFMCR2       SIUL2_1.IFMCR[2].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_1_IFMCR3       SIUL2_1.IFMCR[3].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_1_IFMCR4       SIUL2_1.IFMCR[4].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_1_IFMCR5       SIUL2_1.IFMCR[5].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_1_IFMCR6       SIUL2_1.IFMCR[6].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_1_IFMCR7       SIUL2_1.IFMCR[7].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_1_IFCPR        SIUL2_1.IFCPR.R               /* SIUL2 Interrupt Filter Clock Prescaler */
#define SIUL2_1_MSCR31       SIUL2_1.MSCR[31].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR32       SIUL2_1.MSCR[32].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR33       SIUL2_1.MSCR[33].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR34       SIUL2_1.MSCR[34].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR35       SIUL2_1.MSCR[35].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR36       SIUL2_1.MSCR[36].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR37       SIUL2_1.MSCR[37].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR38       SIUL2_1.MSCR[38].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR39       SIUL2_1.MSCR[39].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR40       SIUL2_1.MSCR[40].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR41       SIUL2_1.MSCR[41].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR42       SIUL2_1.MSCR[42].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR43       SIUL2_1.MSCR[43].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR44       SIUL2_1.MSCR[44].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR45       SIUL2_1.MSCR[45].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR46       SIUL2_1.MSCR[46].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR47       SIUL2_1.MSCR[47].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR48       SIUL2_1.MSCR[48].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR49       SIUL2_1.MSCR[49].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR50       SIUL2_1.MSCR[50].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR51       SIUL2_1.MSCR[51].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR52       SIUL2_1.MSCR[52].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR53       SIUL2_1.MSCR[53].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR54       SIUL2_1.MSCR[54].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR55       SIUL2_1.MSCR[55].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR56       SIUL2_1.MSCR[56].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR57       SIUL2_1.MSCR[57].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR58       SIUL2_1.MSCR[58].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR59       SIUL2_1.MSCR[59].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR60       SIUL2_1.MSCR[60].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR61       SIUL2_1.MSCR[61].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR62       SIUL2_1.MSCR[62].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR63       SIUL2_1.MSCR[63].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR64       SIUL2_1.MSCR[64].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR65       SIUL2_1.MSCR[65].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR66       SIUL2_1.MSCR[66].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR67       SIUL2_1.MSCR[67].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR68       SIUL2_1.MSCR[68].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR69       SIUL2_1.MSCR[69].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR70       SIUL2_1.MSCR[70].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR71       SIUL2_1.MSCR[71].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR72       SIUL2_1.MSCR[72].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR73       SIUL2_1.MSCR[73].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR74       SIUL2_1.MSCR[74].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR75       SIUL2_1.MSCR[75].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR76       SIUL2_1.MSCR[76].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR77       SIUL2_1.MSCR[77].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR78       SIUL2_1.MSCR[78].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR79       SIUL2_1.MSCR[79].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR80       SIUL2_1.MSCR[80].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR81       SIUL2_1.MSCR[81].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR82       SIUL2_1.MSCR[82].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR83       SIUL2_1.MSCR[83].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR84       SIUL2_1.MSCR[84].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR85       SIUL2_1.MSCR[85].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR86       SIUL2_1.MSCR[86].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR87       SIUL2_1.MSCR[87].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR88       SIUL2_1.MSCR[88].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR89       SIUL2_1.MSCR[89].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR90       SIUL2_1.MSCR[90].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR91       SIUL2_1.MSCR[91].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR500      SIUL2_1.MSCR[500].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR501      SIUL2_1.MSCR[501].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR502      SIUL2_1.MSCR[502].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_MSCR503      SIUL2_1.MSCR[503].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_1_IMCR127      SIUL2_1.IMCR[127].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR128      SIUL2_1.IMCR[128].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR129      SIUL2_1.IMCR[129].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR130      SIUL2_1.IMCR[130].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR131      SIUL2_1.IMCR[131].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR132      SIUL2_1.IMCR[132].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR133      SIUL2_1.IMCR[133].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR134      SIUL2_1.IMCR[134].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR135      SIUL2_1.IMCR[135].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR136      SIUL2_1.IMCR[136].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR137      SIUL2_1.IMCR[137].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR138      SIUL2_1.IMCR[138].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR139      SIUL2_1.IMCR[139].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR140      SIUL2_1.IMCR[140].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR141      SIUL2_1.IMCR[141].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR142      SIUL2_1.IMCR[142].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR143      SIUL2_1.IMCR[143].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR144      SIUL2_1.IMCR[144].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR145      SIUL2_1.IMCR[145].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR146      SIUL2_1.IMCR[146].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR147      SIUL2_1.IMCR[147].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR148      SIUL2_1.IMCR[148].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR149      SIUL2_1.IMCR[149].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR150      SIUL2_1.IMCR[150].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR151      SIUL2_1.IMCR[151].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR152      SIUL2_1.IMCR[152].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR153      SIUL2_1.IMCR[153].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR154      SIUL2_1.IMCR[154].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR155      SIUL2_1.IMCR[155].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR156      SIUL2_1.IMCR[156].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR157      SIUL2_1.IMCR[157].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR158      SIUL2_1.IMCR[158].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR159      SIUL2_1.IMCR[159].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR160      SIUL2_1.IMCR[160].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR161      SIUL2_1.IMCR[161].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR162      SIUL2_1.IMCR[162].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR163      SIUL2_1.IMCR[163].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR164      SIUL2_1.IMCR[164].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR165      SIUL2_1.IMCR[165].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR166      SIUL2_1.IMCR[166].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR167      SIUL2_1.IMCR[167].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR168      SIUL2_1.IMCR[168].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR169      SIUL2_1.IMCR[169].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR170      SIUL2_1.IMCR[170].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR171      SIUL2_1.IMCR[171].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR172      SIUL2_1.IMCR[172].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR173      SIUL2_1.IMCR[173].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR174      SIUL2_1.IMCR[174].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR175      SIUL2_1.IMCR[175].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR176      SIUL2_1.IMCR[176].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR177      SIUL2_1.IMCR[177].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR178      SIUL2_1.IMCR[178].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR179      SIUL2_1.IMCR[179].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR180      SIUL2_1.IMCR[180].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR181      SIUL2_1.IMCR[181].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR185      SIUL2_1.IMCR[185].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR186      SIUL2_1.IMCR[186].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR187      SIUL2_1.IMCR[187].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR191      SIUL2_1.IMCR[191].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR192      SIUL2_1.IMCR[192].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR193      SIUL2_1.IMCR[193].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR194      SIUL2_1.IMCR[194].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR195      SIUL2_1.IMCR[195].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR196      SIUL2_1.IMCR[196].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR197      SIUL2_1.IMCR[197].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR198      SIUL2_1.IMCR[198].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR199      SIUL2_1.IMCR[199].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR200      SIUL2_1.IMCR[200].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR201      SIUL2_1.IMCR[201].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR202      SIUL2_1.IMCR[202].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR203      SIUL2_1.IMCR[203].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR204      SIUL2_1.IMCR[204].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR205      SIUL2_1.IMCR[205].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR206      SIUL2_1.IMCR[206].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR207      SIUL2_1.IMCR[207].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR208      SIUL2_1.IMCR[208].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_IMCR209      SIUL2_1.IMCR[209].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_1_GPDO31       SIUL2_1.GPDO31.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO35       SIUL2_1.GPDO35.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO34       SIUL2_1.GPDO34.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO33       SIUL2_1.GPDO33.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO32       SIUL2_1.GPDO32.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO39       SIUL2_1.GPDO39.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO38       SIUL2_1.GPDO38.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO37       SIUL2_1.GPDO37.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO36       SIUL2_1.GPDO36.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO43       SIUL2_1.GPDO43.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO42       SIUL2_1.GPDO42.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO41       SIUL2_1.GPDO41.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO40       SIUL2_1.GPDO40.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO47       SIUL2_1.GPDO47.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO46       SIUL2_1.GPDO46.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO45       SIUL2_1.GPDO45.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO44       SIUL2_1.GPDO44.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO51       SIUL2_1.GPDO51.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO50       SIUL2_1.GPDO50.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO49       SIUL2_1.GPDO49.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO48       SIUL2_1.GPDO48.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO55       SIUL2_1.GPDO55.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO54       SIUL2_1.GPDO54.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO53       SIUL2_1.GPDO53.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO52       SIUL2_1.GPDO52.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO59       SIUL2_1.GPDO59.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO58       SIUL2_1.GPDO58.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO57       SIUL2_1.GPDO57.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO56       SIUL2_1.GPDO56.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO63       SIUL2_1.GPDO63.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO62       SIUL2_1.GPDO62.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO61       SIUL2_1.GPDO61.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO60       SIUL2_1.GPDO60.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO67       SIUL2_1.GPDO67.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO66       SIUL2_1.GPDO66.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO65       SIUL2_1.GPDO65.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO64       SIUL2_1.GPDO64.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO71       SIUL2_1.GPDO71.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO70       SIUL2_1.GPDO70.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO69       SIUL2_1.GPDO69.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO68       SIUL2_1.GPDO68.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO75       SIUL2_1.GPDO75.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO74       SIUL2_1.GPDO74.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO73       SIUL2_1.GPDO73.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO72       SIUL2_1.GPDO72.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO79       SIUL2_1.GPDO79.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO78       SIUL2_1.GPDO78.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO77       SIUL2_1.GPDO77.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO76       SIUL2_1.GPDO76.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO83       SIUL2_1.GPDO83.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO82       SIUL2_1.GPDO82.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO81       SIUL2_1.GPDO81.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO80       SIUL2_1.GPDO80.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO87       SIUL2_1.GPDO87.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO86       SIUL2_1.GPDO86.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO85       SIUL2_1.GPDO85.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO84       SIUL2_1.GPDO84.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO91       SIUL2_1.GPDO91.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO90       SIUL2_1.GPDO90.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO89       SIUL2_1.GPDO89.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDO88       SIUL2_1.GPDO88.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_1_GPDI31       SIUL2_1.GPDI31.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI35       SIUL2_1.GPDI35.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI34       SIUL2_1.GPDI34.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI33       SIUL2_1.GPDI33.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI32       SIUL2_1.GPDI32.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI39       SIUL2_1.GPDI39.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI38       SIUL2_1.GPDI38.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI37       SIUL2_1.GPDI37.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI36       SIUL2_1.GPDI36.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI43       SIUL2_1.GPDI43.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI42       SIUL2_1.GPDI42.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI41       SIUL2_1.GPDI41.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI40       SIUL2_1.GPDI40.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI47       SIUL2_1.GPDI47.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI46       SIUL2_1.GPDI46.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI45       SIUL2_1.GPDI45.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI44       SIUL2_1.GPDI44.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI51       SIUL2_1.GPDI51.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI50       SIUL2_1.GPDI50.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI49       SIUL2_1.GPDI49.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI48       SIUL2_1.GPDI48.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI55       SIUL2_1.GPDI55.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI54       SIUL2_1.GPDI54.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI53       SIUL2_1.GPDI53.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI52       SIUL2_1.GPDI52.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI59       SIUL2_1.GPDI59.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI58       SIUL2_1.GPDI58.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI57       SIUL2_1.GPDI57.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI56       SIUL2_1.GPDI56.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI63       SIUL2_1.GPDI63.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI62       SIUL2_1.GPDI62.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI61       SIUL2_1.GPDI61.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI60       SIUL2_1.GPDI60.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI67       SIUL2_1.GPDI67.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI66       SIUL2_1.GPDI66.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI65       SIUL2_1.GPDI65.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI64       SIUL2_1.GPDI64.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI71       SIUL2_1.GPDI71.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI70       SIUL2_1.GPDI70.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI69       SIUL2_1.GPDI69.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI68       SIUL2_1.GPDI68.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI75       SIUL2_1.GPDI75.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI74       SIUL2_1.GPDI74.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI73       SIUL2_1.GPDI73.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI72       SIUL2_1.GPDI72.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI79       SIUL2_1.GPDI79.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI78       SIUL2_1.GPDI78.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI77       SIUL2_1.GPDI77.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI76       SIUL2_1.GPDI76.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI83       SIUL2_1.GPDI83.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI82       SIUL2_1.GPDI82.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI81       SIUL2_1.GPDI81.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI80       SIUL2_1.GPDI80.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI87       SIUL2_1.GPDI87.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI86       SIUL2_1.GPDI86.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI85       SIUL2_1.GPDI85.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI84       SIUL2_1.GPDI84.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI91       SIUL2_1.GPDI91.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI90       SIUL2_1.GPDI90.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI89       SIUL2_1.GPDI89.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_GPDI88       SIUL2_1.GPDI88.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_1_PGPDO1       SIUL2_1.PGPDO1.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_1_PGPDO3       SIUL2_1.PGPDO3.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_1_PGPDO2       SIUL2_1.PGPDO2.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_1_PGPDO5       SIUL2_1.PGPDO5.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_1_PGPDO4       SIUL2_1.PGPDO4.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_1_PGPDI1       SIUL2_1.PGPDI1.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_1_PGPDI3       SIUL2_1.PGPDI3.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_1_PGPDI2       SIUL2_1.PGPDI2.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_1_PGPDI5       SIUL2_1.PGPDI5.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_1_PGPDI4       SIUL2_1.PGPDI4.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_1_MPGPDO1      SIUL2_1.MPGPDO[1].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_1_MPGPDO2      SIUL2_1.MPGPDO[2].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_1_MPGPDO3      SIUL2_1.MPGPDO[3].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_1_MPGPDO4      SIUL2_1.MPGPDO[4].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_1_MPGPDO5      SIUL2_1.MPGPDO[5].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */

/* SIUL2_3 */
#define SIUL2_3_DISR0        SIUL2_3.DISR0.R               /* SIUL2 DMA/Interrupt Status Flag 0 */
#define SIUL2_3_DIRER0       SIUL2_3.DIRER0.R              /* SIUL2 DMA/Interrupt Request Enable 0 */
#define SIUL2_3_DIRSR0       SIUL2_3.DIRSR0.R              /* SIUL2 DMA/Interrupt Request Select 0 */
#define SIUL2_3_IREER0       SIUL2_3.IREER0.R              /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
#define SIUL2_3_IFEER0       SIUL2_3.IFEER0.R              /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
#define SIUL2_3_IFER0        SIUL2_3.IFER0.R               /* SIUL2 Interrupt Filter Enable 0 */
#define SIUL2_3_IFCPR        SIUL2_3.IFCPR.R               /* SIUL2 Interrupt Filter Clock Prescaler */
#define SIUL2_3_IMCR0        SIUL2_3.IMCR[0].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR1        SIUL2_3.IMCR[1].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR2        SIUL2_3.IMCR[2].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR3        SIUL2_3.IMCR[3].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR4        SIUL2_3.IMCR[4].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR5        SIUL2_3.IMCR[5].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR6        SIUL2_3.IMCR[6].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR7        SIUL2_3.IMCR[7].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR8        SIUL2_3.IMCR[8].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR9        SIUL2_3.IMCR[9].R             /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR10       SIUL2_3.IMCR[10].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR11       SIUL2_3.IMCR[11].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR12       SIUL2_3.IMCR[12].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR13       SIUL2_3.IMCR[13].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR14       SIUL2_3.IMCR[14].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR15       SIUL2_3.IMCR[15].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR16       SIUL2_3.IMCR[16].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR17       SIUL2_3.IMCR[17].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR18       SIUL2_3.IMCR[18].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR19       SIUL2_3.IMCR[19].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR20       SIUL2_3.IMCR[20].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR21       SIUL2_3.IMCR[21].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR22       SIUL2_3.IMCR[22].R            /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_3_IMCR23       SIUL2_3.IMCR[23].R            /* SIUL2 Input Multiplexed Signal Configuration */

/* SIUL2_4 */
#define SIUL2_4_MIDR1        SIUL2_4.MIDR1.R               /* SIUL2 MCU ID 1 */
#define SIUL2_4_MIDR2        SIUL2_4.MIDR2.R               /* SIUL2 MCU ID 2 */
#define SIUL2_4_DISR0        SIUL2_4.DISR0.R               /* SIUL2 DMA/Interrupt Status Flag 0 */
#define SIUL2_4_DIRER0       SIUL2_4.DIRER0.R              /* SIUL2 DMA/Interrupt Request Enable 0 */
#define SIUL2_4_DIRSR0       SIUL2_4.DIRSR0.R              /* SIUL2 DMA/Interrupt Request Select 0 */
#define SIUL2_4_IREER0       SIUL2_4.IREER0.R              /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
#define SIUL2_4_IFEER0       SIUL2_4.IFEER0.R              /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
#define SIUL2_4_IFER0        SIUL2_4.IFER0.R               /* SIUL2 Interrupt Filter Enable 0 */
#define SIUL2_4_IFMCR0       SIUL2_4.IFMCR[0].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_4_IFMCR1       SIUL2_4.IFMCR[1].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_4_IFMCR2       SIUL2_4.IFMCR[2].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_4_IFMCR3       SIUL2_4.IFMCR[3].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_4_IFMCR4       SIUL2_4.IFMCR[4].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_4_IFMCR5       SIUL2_4.IFMCR[5].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_4_IFMCR6       SIUL2_4.IFMCR[6].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_4_IFMCR7       SIUL2_4.IFMCR[7].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_4_IFCPR        SIUL2_4.IFCPR.R               /* SIUL2 Interrupt Filter Clock Prescaler */
#define SIUL2_4_MIDR3        SIUL2_4.MIDR3.R               /* SIUL2 MCU ID 3 */
#define SIUL2_4_MIDR4        SIUL2_4.MIDR4.R               /* SIUL2 MCU ID 4 */
#define SIUL2_4_MSCR92       SIUL2_4.MSCR[92].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR93       SIUL2_4.MSCR[93].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR94       SIUL2_4.MSCR[94].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR95       SIUL2_4.MSCR[95].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR96       SIUL2_4.MSCR[96].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR97       SIUL2_4.MSCR[97].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR98       SIUL2_4.MSCR[98].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR99       SIUL2_4.MSCR[99].R            /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR100      SIUL2_4.MSCR[100].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR101      SIUL2_4.MSCR[101].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR102      SIUL2_4.MSCR[102].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR103      SIUL2_4.MSCR[103].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR104      SIUL2_4.MSCR[104].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR105      SIUL2_4.MSCR[105].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR106      SIUL2_4.MSCR[106].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR107      SIUL2_4.MSCR[107].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR108      SIUL2_4.MSCR[108].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR109      SIUL2_4.MSCR[109].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR110      SIUL2_4.MSCR[110].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR111      SIUL2_4.MSCR[111].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR112      SIUL2_4.MSCR[112].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR113      SIUL2_4.MSCR[113].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR114      SIUL2_4.MSCR[114].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR115      SIUL2_4.MSCR[115].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR116      SIUL2_4.MSCR[116].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR117      SIUL2_4.MSCR[117].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR118      SIUL2_4.MSCR[118].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR119      SIUL2_4.MSCR[119].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR120      SIUL2_4.MSCR[120].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR121      SIUL2_4.MSCR[121].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR122      SIUL2_4.MSCR[122].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR123      SIUL2_4.MSCR[123].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR124      SIUL2_4.MSCR[124].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR125      SIUL2_4.MSCR[125].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR126      SIUL2_4.MSCR[126].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR127      SIUL2_4.MSCR[127].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR128      SIUL2_4.MSCR[128].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR129      SIUL2_4.MSCR[129].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR130      SIUL2_4.MSCR[130].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR131      SIUL2_4.MSCR[131].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR132      SIUL2_4.MSCR[132].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR133      SIUL2_4.MSCR[133].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR134      SIUL2_4.MSCR[134].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR135      SIUL2_4.MSCR[135].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR136      SIUL2_4.MSCR[136].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR137      SIUL2_4.MSCR[137].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR138      SIUL2_4.MSCR[138].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR139      SIUL2_4.MSCR[139].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR140      SIUL2_4.MSCR[140].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR141      SIUL2_4.MSCR[141].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR142      SIUL2_4.MSCR[142].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR143      SIUL2_4.MSCR[143].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR144      SIUL2_4.MSCR[144].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_MSCR145      SIUL2_4.MSCR[145].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_4_IMCR256      SIUL2_4.IMCR[256].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR257      SIUL2_4.IMCR[257].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR258      SIUL2_4.IMCR[258].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR292      SIUL2_4.IMCR[292].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR293      SIUL2_4.IMCR[293].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR294      SIUL2_4.IMCR[294].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR295      SIUL2_4.IMCR[295].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR302      SIUL2_4.IMCR[302].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR303      SIUL2_4.IMCR[303].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR304      SIUL2_4.IMCR[304].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR305      SIUL2_4.IMCR[305].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR306      SIUL2_4.IMCR[306].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR307      SIUL2_4.IMCR[307].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR308      SIUL2_4.IMCR[308].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR309      SIUL2_4.IMCR[309].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR310      SIUL2_4.IMCR[310].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR311      SIUL2_4.IMCR[311].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR312      SIUL2_4.IMCR[312].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR316      SIUL2_4.IMCR[316].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR317      SIUL2_4.IMCR[317].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR318      SIUL2_4.IMCR[318].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR319      SIUL2_4.IMCR[319].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR320      SIUL2_4.IMCR[320].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR321      SIUL2_4.IMCR[321].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR322      SIUL2_4.IMCR[322].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR323      SIUL2_4.IMCR[323].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR324      SIUL2_4.IMCR[324].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR325      SIUL2_4.IMCR[325].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR326      SIUL2_4.IMCR[326].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR327      SIUL2_4.IMCR[327].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR328      SIUL2_4.IMCR[328].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR329      SIUL2_4.IMCR[329].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR330      SIUL2_4.IMCR[330].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR331      SIUL2_4.IMCR[331].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR332      SIUL2_4.IMCR[332].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR333      SIUL2_4.IMCR[333].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR334      SIUL2_4.IMCR[334].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR335      SIUL2_4.IMCR[335].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR336      SIUL2_4.IMCR[336].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR337      SIUL2_4.IMCR[337].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR338      SIUL2_4.IMCR[338].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR339      SIUL2_4.IMCR[339].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR340      SIUL2_4.IMCR[340].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR341      SIUL2_4.IMCR[341].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR342      SIUL2_4.IMCR[342].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR343      SIUL2_4.IMCR[343].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR344      SIUL2_4.IMCR[344].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR345      SIUL2_4.IMCR[345].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR346      SIUL2_4.IMCR[346].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR347      SIUL2_4.IMCR[347].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR348      SIUL2_4.IMCR[348].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR349      SIUL2_4.IMCR[349].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR350      SIUL2_4.IMCR[350].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR351      SIUL2_4.IMCR[351].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR352      SIUL2_4.IMCR[352].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR353      SIUL2_4.IMCR[353].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR354      SIUL2_4.IMCR[354].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR355      SIUL2_4.IMCR[355].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR356      SIUL2_4.IMCR[356].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR357      SIUL2_4.IMCR[357].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR358      SIUL2_4.IMCR[358].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR359      SIUL2_4.IMCR[359].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR360      SIUL2_4.IMCR[360].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR361      SIUL2_4.IMCR[361].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR362      SIUL2_4.IMCR[362].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR363      SIUL2_4.IMCR[363].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR364      SIUL2_4.IMCR[364].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR365      SIUL2_4.IMCR[365].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR366      SIUL2_4.IMCR[366].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR367      SIUL2_4.IMCR[367].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR368      SIUL2_4.IMCR[368].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR369      SIUL2_4.IMCR[369].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR370      SIUL2_4.IMCR[370].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_IMCR371      SIUL2_4.IMCR[371].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_4_GPDO95       SIUL2_4.GPDO95.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO94       SIUL2_4.GPDO94.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO93       SIUL2_4.GPDO93.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO92       SIUL2_4.GPDO92.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO99       SIUL2_4.GPDO99.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO98       SIUL2_4.GPDO98.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO97       SIUL2_4.GPDO97.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO96       SIUL2_4.GPDO96.R              /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO103      SIUL2_4.GPDO103.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO102      SIUL2_4.GPDO102.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO101      SIUL2_4.GPDO101.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO100      SIUL2_4.GPDO100.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO107      SIUL2_4.GPDO107.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO106      SIUL2_4.GPDO106.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO105      SIUL2_4.GPDO105.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO104      SIUL2_4.GPDO104.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO111      SIUL2_4.GPDO111.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO110      SIUL2_4.GPDO110.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO109      SIUL2_4.GPDO109.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO108      SIUL2_4.GPDO108.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO115      SIUL2_4.GPDO115.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO114      SIUL2_4.GPDO114.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO113      SIUL2_4.GPDO113.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO112      SIUL2_4.GPDO112.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO119      SIUL2_4.GPDO119.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO118      SIUL2_4.GPDO118.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO117      SIUL2_4.GPDO117.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO116      SIUL2_4.GPDO116.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO123      SIUL2_4.GPDO123.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO122      SIUL2_4.GPDO122.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO121      SIUL2_4.GPDO121.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO120      SIUL2_4.GPDO120.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO127      SIUL2_4.GPDO127.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO126      SIUL2_4.GPDO126.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO125      SIUL2_4.GPDO125.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO124      SIUL2_4.GPDO124.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO131      SIUL2_4.GPDO131.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO130      SIUL2_4.GPDO130.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO129      SIUL2_4.GPDO129.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO128      SIUL2_4.GPDO128.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO135      SIUL2_4.GPDO135.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO134      SIUL2_4.GPDO134.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO133      SIUL2_4.GPDO133.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO132      SIUL2_4.GPDO132.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO139      SIUL2_4.GPDO139.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO138      SIUL2_4.GPDO138.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO137      SIUL2_4.GPDO137.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO136      SIUL2_4.GPDO136.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO143      SIUL2_4.GPDO143.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO142      SIUL2_4.GPDO142.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO141      SIUL2_4.GPDO141.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO140      SIUL2_4.GPDO140.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO145      SIUL2_4.GPDO145.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDO144      SIUL2_4.GPDO144.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_4_GPDI95       SIUL2_4.GPDI95.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI94       SIUL2_4.GPDI94.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI93       SIUL2_4.GPDI93.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI92       SIUL2_4.GPDI92.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI99       SIUL2_4.GPDI99.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI98       SIUL2_4.GPDI98.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI97       SIUL2_4.GPDI97.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI96       SIUL2_4.GPDI96.R              /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI103      SIUL2_4.GPDI103.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI102      SIUL2_4.GPDI102.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI101      SIUL2_4.GPDI101.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI100      SIUL2_4.GPDI100.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI107      SIUL2_4.GPDI107.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI106      SIUL2_4.GPDI106.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI105      SIUL2_4.GPDI105.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI104      SIUL2_4.GPDI104.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI111      SIUL2_4.GPDI111.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI110      SIUL2_4.GPDI110.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI109      SIUL2_4.GPDI109.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI108      SIUL2_4.GPDI108.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI115      SIUL2_4.GPDI115.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI114      SIUL2_4.GPDI114.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI113      SIUL2_4.GPDI113.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI112      SIUL2_4.GPDI112.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI119      SIUL2_4.GPDI119.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI118      SIUL2_4.GPDI118.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI117      SIUL2_4.GPDI117.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI116      SIUL2_4.GPDI116.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI123      SIUL2_4.GPDI123.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI122      SIUL2_4.GPDI122.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI121      SIUL2_4.GPDI121.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI120      SIUL2_4.GPDI120.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI127      SIUL2_4.GPDI127.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI126      SIUL2_4.GPDI126.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI125      SIUL2_4.GPDI125.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI124      SIUL2_4.GPDI124.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI131      SIUL2_4.GPDI131.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI130      SIUL2_4.GPDI130.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI129      SIUL2_4.GPDI129.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI128      SIUL2_4.GPDI128.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI135      SIUL2_4.GPDI135.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI134      SIUL2_4.GPDI134.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI133      SIUL2_4.GPDI133.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI132      SIUL2_4.GPDI132.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI139      SIUL2_4.GPDI139.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI138      SIUL2_4.GPDI138.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI137      SIUL2_4.GPDI137.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI136      SIUL2_4.GPDI136.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI143      SIUL2_4.GPDI143.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI142      SIUL2_4.GPDI142.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI141      SIUL2_4.GPDI141.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI140      SIUL2_4.GPDI140.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI145      SIUL2_4.GPDI145.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_GPDI144      SIUL2_4.GPDI144.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_4_PGPDO5       SIUL2_4.PGPDO5.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_4_PGPDO7       SIUL2_4.PGPDO7.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_4_PGPDO6       SIUL2_4.PGPDO6.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_4_PGPDO9       SIUL2_4.PGPDO9.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_4_PGPDO8       SIUL2_4.PGPDO8.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_4_PGPDI5       SIUL2_4.PGPDI5.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_4_PGPDI7       SIUL2_4.PGPDI7.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_4_PGPDI6       SIUL2_4.PGPDI6.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_4_PGPDI9       SIUL2_4.PGPDI9.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_4_PGPDI8       SIUL2_4.PGPDI8.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_4_MPGPDO5      SIUL2_4.MPGPDO[5].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_4_MPGPDO6      SIUL2_4.MPGPDO[6].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_4_MPGPDO7      SIUL2_4.MPGPDO[7].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_4_MPGPDO8      SIUL2_4.MPGPDO[8].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_4_MPGPDO9      SIUL2_4.MPGPDO[9].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */

/* SIUL2_5 */
#define SIUL2_5_DISR0        SIUL2_5.DISR0.R               /* SIUL2 DMA/Interrupt Status Flag 0 */
#define SIUL2_5_DIRER0       SIUL2_5.DIRER0.R              /* SIUL2 DMA/Interrupt Request Enable 0 */
#define SIUL2_5_DIRSR0       SIUL2_5.DIRSR0.R              /* SIUL2 DMA/Interrupt Request Select 0 */
#define SIUL2_5_IREER0       SIUL2_5.IREER0.R              /* SIUL2 Interrupt Rising-Edge Event Enable 0 */
#define SIUL2_5_IFEER0       SIUL2_5.IFEER0.R              /* SIUL2 Interrupt Falling-Edge Event Enable 0 */
#define SIUL2_5_IFER0        SIUL2_5.IFER0.R               /* SIUL2 Interrupt Filter Enable 0 */
#define SIUL2_5_IFMCR0       SIUL2_5.IFMCR[0].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR1       SIUL2_5.IFMCR[1].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR2       SIUL2_5.IFMCR[2].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR3       SIUL2_5.IFMCR[3].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR4       SIUL2_5.IFMCR[4].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR5       SIUL2_5.IFMCR[5].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR6       SIUL2_5.IFMCR[6].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR7       SIUL2_5.IFMCR[7].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR8       SIUL2_5.IFMCR[8].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR9       SIUL2_5.IFMCR[9].R            /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR10      SIUL2_5.IFMCR[10].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR11      SIUL2_5.IFMCR[11].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR12      SIUL2_5.IFMCR[12].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR13      SIUL2_5.IFMCR[13].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR14      SIUL2_5.IFMCR[14].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR15      SIUL2_5.IFMCR[15].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR16      SIUL2_5.IFMCR[16].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR17      SIUL2_5.IFMCR[17].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR18      SIUL2_5.IFMCR[18].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFMCR19      SIUL2_5.IFMCR[19].R           /* SIUL2 Interrupt Filter Maximum Counter */
#define SIUL2_5_IFCPR        SIUL2_5.IFCPR.R               /* SIUL2 Interrupt Filter Clock Prescaler */
#define SIUL2_5_MSCR146      SIUL2_5.MSCR[146].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR147      SIUL2_5.MSCR[147].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR148      SIUL2_5.MSCR[148].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR149      SIUL2_5.MSCR[149].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR150      SIUL2_5.MSCR[150].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR151      SIUL2_5.MSCR[151].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR152      SIUL2_5.MSCR[152].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR153      SIUL2_5.MSCR[153].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR154      SIUL2_5.MSCR[154].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR155      SIUL2_5.MSCR[155].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR156      SIUL2_5.MSCR[156].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR157      SIUL2_5.MSCR[157].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR158      SIUL2_5.MSCR[158].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR159      SIUL2_5.MSCR[159].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR160      SIUL2_5.MSCR[160].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR161      SIUL2_5.MSCR[161].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR162      SIUL2_5.MSCR[162].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR163      SIUL2_5.MSCR[163].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR164      SIUL2_5.MSCR[164].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR165      SIUL2_5.MSCR[165].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR166      SIUL2_5.MSCR[166].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR167      SIUL2_5.MSCR[167].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR168      SIUL2_5.MSCR[168].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_MSCR169      SIUL2_5.MSCR[169].R           /* SIUL2 Multiplexed Signal Configuration Register */
#define SIUL2_5_IMCR449      SIUL2_5.IMCR[449].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR450      SIUL2_5.IMCR[450].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR451      SIUL2_5.IMCR[451].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR452      SIUL2_5.IMCR[452].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR453      SIUL2_5.IMCR[453].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR454      SIUL2_5.IMCR[454].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR462      SIUL2_5.IMCR[462].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR463      SIUL2_5.IMCR[463].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR464      SIUL2_5.IMCR[464].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR465      SIUL2_5.IMCR[465].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR466      SIUL2_5.IMCR[466].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR467      SIUL2_5.IMCR[467].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR468      SIUL2_5.IMCR[468].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR469      SIUL2_5.IMCR[469].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR470      SIUL2_5.IMCR[470].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR471      SIUL2_5.IMCR[471].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR472      SIUL2_5.IMCR[472].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR473      SIUL2_5.IMCR[473].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR474      SIUL2_5.IMCR[474].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR488      SIUL2_5.IMCR[488].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR489      SIUL2_5.IMCR[489].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR490      SIUL2_5.IMCR[490].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR491      SIUL2_5.IMCR[491].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR492      SIUL2_5.IMCR[492].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR493      SIUL2_5.IMCR[493].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR494      SIUL2_5.IMCR[494].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_IMCR495      SIUL2_5.IMCR[495].R           /* SIUL2 Input Multiplexed Signal Configuration */
#define SIUL2_5_GPDO147      SIUL2_5.GPDO147.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO146      SIUL2_5.GPDO146.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO151      SIUL2_5.GPDO151.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO150      SIUL2_5.GPDO150.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO149      SIUL2_5.GPDO149.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO148      SIUL2_5.GPDO148.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO155      SIUL2_5.GPDO155.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO154      SIUL2_5.GPDO154.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO153      SIUL2_5.GPDO153.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO152      SIUL2_5.GPDO152.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO159      SIUL2_5.GPDO159.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO158      SIUL2_5.GPDO158.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO157      SIUL2_5.GPDO157.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO156      SIUL2_5.GPDO156.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO163      SIUL2_5.GPDO163.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO162      SIUL2_5.GPDO162.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO161      SIUL2_5.GPDO161.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO160      SIUL2_5.GPDO160.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO167      SIUL2_5.GPDO167.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO166      SIUL2_5.GPDO166.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO165      SIUL2_5.GPDO165.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO164      SIUL2_5.GPDO164.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO169      SIUL2_5.GPDO169.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDO168      SIUL2_5.GPDO168.R             /* SIUL2 GPIO Pad Data Output */
#define SIUL2_5_GPDI147      SIUL2_5.GPDI147.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI146      SIUL2_5.GPDI146.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI151      SIUL2_5.GPDI151.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI150      SIUL2_5.GPDI150.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI149      SIUL2_5.GPDI149.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI148      SIUL2_5.GPDI148.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI155      SIUL2_5.GPDI155.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI154      SIUL2_5.GPDI154.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI153      SIUL2_5.GPDI153.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI152      SIUL2_5.GPDI152.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI159      SIUL2_5.GPDI159.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI158      SIUL2_5.GPDI158.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI157      SIUL2_5.GPDI157.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI156      SIUL2_5.GPDI156.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI163      SIUL2_5.GPDI163.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI162      SIUL2_5.GPDI162.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI161      SIUL2_5.GPDI161.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI160      SIUL2_5.GPDI160.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI167      SIUL2_5.GPDI167.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI166      SIUL2_5.GPDI166.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI165      SIUL2_5.GPDI165.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI164      SIUL2_5.GPDI164.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI169      SIUL2_5.GPDI169.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_GPDI168      SIUL2_5.GPDI168.R             /* SIUL2 GPIO Pad Data Input */
#define SIUL2_5_PGPDO9       SIUL2_5.PGPDO9.R              /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_5_PGPDO10      SIUL2_5.PGPDO10.R             /* SIUL2 Parallel GPIO Pad Data Out */
#define SIUL2_5_PGPDI9       SIUL2_5.PGPDI9.R              /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_5_PGPDI10      SIUL2_5.PGPDI10.R             /* SIUL2 Parallel GPIO Pad Data In */
#define SIUL2_5_MPGPDO9      SIUL2_5.MPGPDO[9].R           /* SIUL2 Masked Parallel GPIO Pad Data Out */
#define SIUL2_5_MPGPDO10     SIUL2_5.MPGPDO[10].R          /* SIUL2 Masked Parallel GPIO Pad Data Out */

/* SMU_L_VFCCU */
#define SMU__L_VFCCU_GDFHID_C0 SMU__L_VFCCU.GDFHID_C0.R    /* Global DID-FHID Map */
#define SMU__L_VFCCU_GDFHID_C1 SMU__L_VFCCU.GDFHID_C1.R    /* Global DID-FHID Map */
#define SMU__L_VFCCU_GFLTPO_C0 SMU__L_VFCCU.GFLTPO_C0.R    /* Global Fault Polarity */
#define SMU__L_VFCCU_GFLTPO_C1 SMU__L_VFCCU.GFLTPO_C1.R    /* Global Fault Polarity */
#define SMU__L_VFCCU_GFLTRC_C0 SMU__L_VFCCU.GFLTRC_C0.R    /* Global Fault Recovery */
#define SMU__L_VFCCU_GFLTRC_C1 SMU__L_VFCCU.GFLTRC_C1.R    /* Global Fault Recovery */
#define SMU__L_VFCCU_GFLTOVDC0 SMU__L_VFCCU.GFLTOVDC0.R    /* Global Fault Overflow Detection */
#define SMU__L_VFCCU_GFLTOVDC1 SMU__L_VFCCU.GFLTOVDC1.R    /* Global Fault Overflow Detection */
#define SMU__L_VFCCU_GSWFLODC SMU__L_VFCCU.GSWFLODC.R      /* Global Software Fault Overflow Detection Disable */
#define SMU__L_VFCCU_GCTRL   SMU__L_VFCCU.GCTRL.R          /* Global Space Control */
#define SMU__L_VFCCU_GINTOVFS SMU__L_VFCCU.GINTOVFS.R      /* Global DID FSM Status */
#define SMU__L_VFCCU_GDBGCFG SMU__L_VFCCU.GDBGCFG.R        /* Global Debug */
#define SMU__L_VFCCU_GDBGSTAT SMU__L_VFCCU.GDBGSTAT.R      /* Global Debug Status */
#define SMU__L_VFCCU_SWRPTDID0 SMU__L_VFCCU.SW_RKN[0].SWRPTDID.R /* Software Fault Reported DID */
#define SMU__L_VFCCU_SWRKSET_00 SMU__L_VFCCU.SW_RKN[0].SWRKSET_0.R /* Software Reaction Set */
#define SMU__L_VFCCU_SWRKCLR_00 SMU__L_VFCCU.SW_RKN[0].SWRKCLR_0.R /* Software Reaction Clear */
#define SMU__L_VFCCU_SWRPTDID1 SMU__L_VFCCU.SW_RKN[1].SWRPTDID.R /* Software Fault Reported DID */
#define SMU__L_VFCCU_SWRKSET_10 SMU__L_VFCCU.SW_RKN[1].SWRKSET_0.R /* Software Reaction Set */
#define SMU__L_VFCCU_SWRKCLR_10 SMU__L_VFCCU.SW_RKN[1].SWRKCLR_0.R /* Software Reaction Clear */
#define SMU__L_VFCCU_SWRPTDID2 SMU__L_VFCCU.SW_RKN[2].SWRPTDID.R /* Software Fault Reported DID */
#define SMU__L_VFCCU_SWRKSET_20 SMU__L_VFCCU.SW_RKN[2].SWRKSET_0.R /* Software Reaction Set */
#define SMU__L_VFCCU_SWRKCLR_20 SMU__L_VFCCU.SW_RKN[2].SWRKCLR_0.R /* Software Reaction Clear */
#define SMU__L_VFCCU_SWRPTDID3 SMU__L_VFCCU.SW_RKN[3].SWRPTDID.R /* Software Fault Reported DID */
#define SMU__L_VFCCU_SWRKSET_30 SMU__L_VFCCU.SW_RKN[3].SWRKSET_0.R /* Software Reaction Set */
#define SMU__L_VFCCU_SWRKCLR_30 SMU__L_VFCCU.SW_RKN[3].SWRKCLR_0.R /* Software Reaction Clear */
#define SMU__L_VFCCU_SWRPTDID4 SMU__L_VFCCU.SW_RKN[4].SWRPTDID.R /* Software Fault Reported DID */
#define SMU__L_VFCCU_SWRKSET_40 SMU__L_VFCCU.SW_RKN[4].SWRKSET_0.R /* Software Reaction Set */
#define SMU__L_VFCCU_SWRKCLR_40 SMU__L_VFCCU.SW_RKN[4].SWRKCLR_0.R /* Software Reaction Clear */
#define SMU__L_VFCCU_SWRPTDID5 SMU__L_VFCCU.SW_RKN[5].SWRPTDID.R /* Software Fault Reported DID */
#define SMU__L_VFCCU_SWRKSET_50 SMU__L_VFCCU.SW_RKN[5].SWRKSET_0.R /* Software Reaction Set */
#define SMU__L_VFCCU_SWRKCLR_50 SMU__L_VFCCU.SW_RKN[5].SWRKCLR_0.R /* Software Reaction Clear */
#define SMU__L_VFCCU_FHCFG0  SMU__L_VFCCU.FHCFG0.R         /* Fault Handler */
#define SMU__L_VFCCU_FHSRVDS0 SMU__L_VFCCU.FHSRVDS0.R      /* Fault Handler Status */
#define SMU__L_VFCCU_FHFLTENC00 SMU__L_VFCCU.FHFLTENC00.R  /* Fault Enable */
#define SMU__L_VFCCU_FHFLTENC01 SMU__L_VFCCU.FHFLTENC01.R  /* Fault Enable */
#define SMU__L_VFCCU_FHFLTS00 SMU__L_VFCCU.FHFLTS00.R      /* Fault Status */
#define SMU__L_VFCCU_FHFLTS01 SMU__L_VFCCU.FHFLTS01.R      /* Fault Status */
#define SMU__L_VFCCU_FHFLTRKC00 SMU__L_VFCCU.FHFLTRKC00.R  /* Fault Reaction Set Configuration */
#define SMU__L_VFCCU_FHFLTRKC01 SMU__L_VFCCU.FHFLTRKC01.R  /* Fault Reaction Set Configuration */
#define SMU__L_VFCCU_FHFLTRKC02 SMU__L_VFCCU.FHFLTRKC02.R  /* Fault Reaction Set Configuration */
#define SMU__L_VFCCU_FHFLTRKC03 SMU__L_VFCCU.FHFLTRKC03.R  /* Fault Reaction Set Configuration */
#define SMU__L_VFCCU_FHFLTRKC04 SMU__L_VFCCU.FHFLTRKC04.R  /* Fault Reaction Set Configuration */
#define SMU__L_VFCCU_FHFLTRKC05 SMU__L_VFCCU.FHFLTRKC05.R  /* Fault Reaction Set Configuration */
#define SMU__L_VFCCU_FHFLTRKC06 SMU__L_VFCCU.FHFLTRKC06.R  /* Fault Reaction Set Configuration */
#define SMU__L_VFCCU_FHIMRKC0_00 SMU__L_VFCCU.FHIMRKC0_00.R /* Immediate Reaction Configuration */
#define SMU__L_VFCCU_FHIMRKC0_10 SMU__L_VFCCU.FHIMRKC0_10.R /* Immediate Reaction Configuration */
#define SMU__L_VFCCU_FHIMRKC0_20 SMU__L_VFCCU.FHIMRKC0_20.R /* Immediate Reaction Configuration */
#define SMU__L_VFCCU_FHIMRKC0_30 SMU__L_VFCCU.FHIMRKC0_30.R /* Immediate Reaction Configuration */
#define SMU__L_VFCCU_FHIMRKC0_40 SMU__L_VFCCU.FHIMRKC0_40.R /* Immediate Reaction Configuration */
#define SMU__L_VFCCU_FHIMRKC0_50 SMU__L_VFCCU.FHIMRKC0_50.R /* Immediate Reaction Configuration */

/* SMU_MRU */
#define SMU__MRU_CH1_CFG0    SMU__MRU.CHXCONFIG[0].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH1_CFG1    SMU__MRU.CHXCONFIG[0].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH1_MBSTAT  SMU__MRU.CHXCONFIG[0].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH2_CFG0    SMU__MRU.CHXCONFIG[1].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH2_CFG1    SMU__MRU.CHXCONFIG[1].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH2_MBSTAT  SMU__MRU.CHXCONFIG[1].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH3_CFG0    SMU__MRU.CHXCONFIG[2].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH3_CFG1    SMU__MRU.CHXCONFIG[2].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH3_MBSTAT  SMU__MRU.CHXCONFIG[2].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH4_CFG0    SMU__MRU.CHXCONFIG[3].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH4_CFG1    SMU__MRU.CHXCONFIG[3].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH4_MBSTAT  SMU__MRU.CHXCONFIG[3].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH5_CFG0    SMU__MRU.CHXCONFIG[4].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH5_CFG1    SMU__MRU.CHXCONFIG[4].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH5_MBSTAT  SMU__MRU.CHXCONFIG[4].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH6_CFG0    SMU__MRU.CHXCONFIG[5].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH6_CFG1    SMU__MRU.CHXCONFIG[5].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH6_MBSTAT  SMU__MRU.CHXCONFIG[5].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH7_CFG0    SMU__MRU.CHXCONFIG[6].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH7_CFG1    SMU__MRU.CHXCONFIG[6].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH7_MBSTAT  SMU__MRU.CHXCONFIG[6].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH8_CFG0    SMU__MRU.CHXCONFIG[7].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH8_CFG1    SMU__MRU.CHXCONFIG[7].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH8_MBSTAT  SMU__MRU.CHXCONFIG[7].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH9_CFG0    SMU__MRU.CHXCONFIG[8].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH9_CFG1    SMU__MRU.CHXCONFIG[8].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH9_MBSTAT  SMU__MRU.CHXCONFIG[8].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH10_CFG0   SMU__MRU.CHXCONFIG[9].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH10_CFG1   SMU__MRU.CHXCONFIG[9].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH10_MBSTAT SMU__MRU.CHXCONFIG[9].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH11_CFG0   SMU__MRU.CHXCONFIG[10].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH11_CFG1   SMU__MRU.CHXCONFIG[10].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH11_MBSTAT SMU__MRU.CHXCONFIG[10].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_CH12_CFG0   SMU__MRU.CHXCONFIG[11].CH_CFG0.R /* Channel (x) Configuration 0 */
#define SMU__MRU_CH12_CFG1   SMU__MRU.CHXCONFIG[11].CH_CFG1.R /* Channel (x) Configuration 1 */
#define SMU__MRU_CH12_MBSTAT SMU__MRU.CHXCONFIG[11].CH_MBSTAT.R /* Channel (x) Mailbox Status */
#define SMU__MRU_NOTIFY0     SMU__MRU.NOTIFY0.R            /* Notification 0 Status */
#define SMU__MRU_NOTIFY1     SMU__MRU.NOTIFY1.R            /* Notification 1 Status */
#define SMU__MRU_CH1_MB0     SMU__MRU.CH1_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH1_MB1     SMU__MRU.CH1_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH1_MB2     SMU__MRU.CH1_MB2.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH1_MB3     SMU__MRU.CH1_MB3.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH2_MB0     SMU__MRU.CH2_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH2_MB1     SMU__MRU.CH2_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH2_MB2     SMU__MRU.CH2_MB2.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH2_MB3     SMU__MRU.CH2_MB3.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH3_MB0     SMU__MRU.CH3_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH3_MB1     SMU__MRU.CH3_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH3_MB2     SMU__MRU.CH3_MB2.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH3_MB3     SMU__MRU.CH3_MB3.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH4_MB0     SMU__MRU.CH4_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH4_MB1     SMU__MRU.CH4_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH4_MB2     SMU__MRU.CH4_MB2.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH4_MB3     SMU__MRU.CH4_MB3.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH5_MB0     SMU__MRU.CH5_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH5_MB1     SMU__MRU.CH5_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH6_MB0     SMU__MRU.CH6_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH6_MB1     SMU__MRU.CH6_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH7_MB0     SMU__MRU.CH7_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH7_MB1     SMU__MRU.CH7_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH8_MB0     SMU__MRU.CH8_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH8_MB1     SMU__MRU.CH8_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH9_MB0     SMU__MRU.CH9_MB0.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH9_MB1     SMU__MRU.CH9_MB1.R            /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH10_MB0    SMU__MRU.CH10_MB0.R           /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH10_MB1    SMU__MRU.CH10_MB1.R           /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH11_MB0    SMU__MRU.CH11_MB0.R           /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH11_MB1    SMU__MRU.CH11_MB1.R           /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH12_MB0    SMU__MRU.CH12_MB0.R           /* Channel (x) Mailbox (n) */
#define SMU__MRU_CH12_MB1    SMU__MRU.CH12_MB1.R           /* Channel (x) Mailbox (n) */

/* SMU_SEMA42 */
#define SMU__SEMA42_GATE3    SMU__SEMA42.GATE[0].R         /* Gate */
#define SMU__SEMA42_GATE2    SMU__SEMA42.GATE[1].R         /* Gate */
#define SMU__SEMA42_GATE1    SMU__SEMA42.GATE[2].R         /* Gate */
#define SMU__SEMA42_GATE0    SMU__SEMA42.GATE[3].R         /* Gate */
#define SMU__SEMA42_GATE7    SMU__SEMA42.GATE[4].R         /* Gate */
#define SMU__SEMA42_GATE6    SMU__SEMA42.GATE[5].R         /* Gate */
#define SMU__SEMA42_GATE5    SMU__SEMA42.GATE[6].R         /* Gate */
#define SMU__SEMA42_GATE4    SMU__SEMA42.GATE[7].R         /* Gate */
#define SMU__SEMA42_GATE11   SMU__SEMA42.GATE[8].R         /* Gate */
#define SMU__SEMA42_GATE10   SMU__SEMA42.GATE[9].R         /* Gate */
#define SMU__SEMA42_GATE9    SMU__SEMA42.GATE[10].R        /* Gate */
#define SMU__SEMA42_GATE8    SMU__SEMA42.GATE[11].R        /* Gate */
#define SMU__SEMA42_GATE15   SMU__SEMA42.GATE[12].R        /* Gate */
#define SMU__SEMA42_GATE14   SMU__SEMA42.GATE[13].R        /* Gate */
#define SMU__SEMA42_GATE13   SMU__SEMA42.GATE[14].R        /* Gate */
#define SMU__SEMA42_GATE12   SMU__SEMA42.GATE[15].R        /* Gate */
#define SMU__SEMA42_RSTGT_R  SMU__SEMA42.RSTGT.R.R         /* Reset Gate Read */
#define SMU__SEMA42_RSTGT_W  SMU__SEMA42.RSTGT.W.R         /* Reset Gate Write */

/* SMU_SRG_S */
#define SMU__SRG_S_CR0       SMU__SRG_S.CR0.R              /* Control 0 */
#define SMU__SRG_S_LCR0      SMU__SRG_S.LCR0.R             /* Lock Control 0 */
#define SMU__SRG_S_LCR2      SMU__SRG_S.LCR2.R             /* Lock Control 2 */
#define SMU__SRG_S_LCR3      SMU__SRG_S.LCR3.R             /* Lock Control 3 */
#define SMU__SRG_S_LCR4      SMU__SRG_S.LCR4.R             /* Lock Control 4 */
#define SMU__SRG_S_LCR7      SMU__SRG_S.LCR7.R             /* Lock Control 7 */
#define SMU__SRG_S_SR0       SMU__SRG_S.SR0.R              /* Status 0 */

/* SMU_XRDC */
#define SMU__XRDC_CR         SMU__XRDC.CR.R                /* Control */
#define SMU__XRDC_HWCFG0     SMU__XRDC.HWCFG0.R            /* Hardware Configuration 0 */
#define SMU__XRDC_HWCFG1     SMU__XRDC.HWCFG1.R            /* Hardware Configuration 1 */
#define SMU__XRDC_MDACFG0    SMU__XRDC.MDACFG[0].R         /* Master Domain Assignment Configuration */
#define SMU__XRDC_MDACFG1    SMU__XRDC.MDACFG[1].R         /* Master Domain Assignment Configuration */
#define SMU__XRDC_MRCFG0     SMU__XRDC.MRCFG[0].R          /* Memory Region Configuration */
#define SMU__XRDC_MRCFG1     SMU__XRDC.MRCFG[1].R          /* Memory Region Configuration */
#define SMU__XRDC_MRCFG2     SMU__XRDC.MRCFG[2].R          /* Memory Region Configuration */
#define SMU__XRDC_MRCFG3     SMU__XRDC.MRCFG[3].R          /* Memory Region Configuration */
#define SMU__XRDC_DERRLOC0   SMU__XRDC.DERRLOC[0].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC1   SMU__XRDC.DERRLOC[1].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC2   SMU__XRDC.DERRLOC[2].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC3   SMU__XRDC.DERRLOC[3].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC4   SMU__XRDC.DERRLOC[4].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC5   SMU__XRDC.DERRLOC[5].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC6   SMU__XRDC.DERRLOC[6].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC7   SMU__XRDC.DERRLOC[7].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC8   SMU__XRDC.DERRLOC[8].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC9   SMU__XRDC.DERRLOC[9].R        /* Domain Error Location */
#define SMU__XRDC_DERRLOC10  SMU__XRDC.DERRLOC[10].R       /* Domain Error Location */
#define SMU__XRDC_DERRLOC11  SMU__XRDC.DERRLOC[11].R       /* Domain Error Location */
#define SMU__XRDC_DERRLOC12  SMU__XRDC.DERRLOC[12].R       /* Domain Error Location */
#define SMU__XRDC_DERRLOC13  SMU__XRDC.DERRLOC[13].R       /* Domain Error Location */
#define SMU__XRDC_DERRLOC14  SMU__XRDC.DERRLOC[14].R       /* Domain Error Location */
#define SMU__XRDC_DERRLOC15  SMU__XRDC.DERRLOC[15].R       /* Domain Error Location */
#define SMU__XRDC_DERR_W0_0  SMU__XRDC.DERRW0[0].DERR_W0.R /* Domain Error Word 0 */
#define SMU__XRDC_DERR_W1_0  SMU__XRDC.DERRW0[0].DERR_W1.R /* Domain Error Word 1 */
#define SMU__XRDC_DERR_W3_0  SMU__XRDC.DERRW0[0].DERR_W3.R /* Domain Error Word 3 */
#define SMU__XRDC_DERR_W0_1  SMU__XRDC.DERRW0[1].DERR_W0.R /* Domain Error Word 0 */
#define SMU__XRDC_DERR_W1_1  SMU__XRDC.DERRW0[1].DERR_W1.R /* Domain Error Word 1 */
#define SMU__XRDC_DERR_W3_1  SMU__XRDC.DERRW0[1].DERR_W3.R /* Domain Error Word 3 */
#define SMU__XRDC_DERR_W0_2  SMU__XRDC.DERRW0[2].DERR_W0.R /* Domain Error Word 0 */
#define SMU__XRDC_DERR_W1_2  SMU__XRDC.DERRW0[2].DERR_W1.R /* Domain Error Word 1 */
#define SMU__XRDC_DERR_W3_2  SMU__XRDC.DERRW0[2].DERR_W3.R /* Domain Error Word 3 */
#define SMU__XRDC_DERR_W0_3  SMU__XRDC.DERRW0[3].DERR_W0.R /* Domain Error Word 0 */
#define SMU__XRDC_DERR_W1_3  SMU__XRDC.DERRW0[3].DERR_W1.R /* Domain Error Word 1 */
#define SMU__XRDC_DERR_W3_3  SMU__XRDC.DERRW0[3].DERR_W3.R /* Domain Error Word 3 */
#define SMU__XRDC_DERR_W0_16 SMU__XRDC.DERRW0[16].DERR_W0.R /* Domain Error Word 0 */
#define SMU__XRDC_DERR_W1_16 SMU__XRDC.DERRW0[16].DERR_W1.R /* Domain Error Word 1 */
#define SMU__XRDC_DERR_W3_16 SMU__XRDC.DERRW0[16].DERR_W3.R /* Domain Error Word 3 */
#define SMU__XRDC_DERR_W0_17 SMU__XRDC.DERRW0[17].DERR_W0.R /* Domain Error Word 0 */
#define SMU__XRDC_DERR_W1_17 SMU__XRDC.DERRW0[17].DERR_W1.R /* Domain Error Word 1 */
#define SMU__XRDC_DERR_W3_17 SMU__XRDC.DERRW0[17].DERR_W3.R /* Domain Error Word 3 */
#define SMU__XRDC_MDA_W0_0_DFMT1 SMU__XRDC.MDA_W0_0_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_MDA_W1_0_DFMT1 SMU__XRDC.MDA_W1_0_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_MDA_W2_0_DFMT1 SMU__XRDC.MDA_W2_0_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_MDA_W3_0_DFMT1 SMU__XRDC.MDA_W3_0_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_MDA_W4_0_DFMT1 SMU__XRDC.MDA_W4_0_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_MDA_W5_0_DFMT1 SMU__XRDC.MDA_W5_0_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_MDA_W6_0_DFMT1 SMU__XRDC.MDA_W6_0_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_MDA_W7_0_DFMT1 SMU__XRDC.MDA_W7_0_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_MDA_W0_1_DFMT1 SMU__XRDC.MDA_W0_1_DFMT1.R /* Master Domain Assignment */
#define SMU__XRDC_PDAC_W0_0  SMU__XRDC.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_0  SMU__XRDC.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_4  SMU__XRDC.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_4  SMU__XRDC.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_5  SMU__XRDC.PDAC_SLOT[0].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_5  SMU__XRDC.PDAC_SLOT[0].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_8  SMU__XRDC.PDAC_SLOT[0].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_8  SMU__XRDC.PDAC_SLOT[0].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_9  SMU__XRDC.PDAC_SLOT[0].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_9  SMU__XRDC.PDAC_SLOT[0].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_16 SMU__XRDC.PDAC_SLOT[0].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_16 SMU__XRDC.PDAC_SLOT[0].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_17 SMU__XRDC.PDAC_SLOT[0].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_17 SMU__XRDC.PDAC_SLOT[0].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_18 SMU__XRDC.PDAC_SLOT[0].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_18 SMU__XRDC.PDAC_SLOT[0].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_19 SMU__XRDC.PDAC_SLOT[0].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_19 SMU__XRDC.PDAC_SLOT[0].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_20 SMU__XRDC.PDAC_SLOT[0].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_20 SMU__XRDC.PDAC_SLOT[0].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_21 SMU__XRDC.PDAC_SLOT[0].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_21 SMU__XRDC.PDAC_SLOT[0].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_24 SMU__XRDC.PDAC_SLOT[0].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_24 SMU__XRDC.PDAC_SLOT[0].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_25 SMU__XRDC.PDAC_SLOT[0].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_25 SMU__XRDC.PDAC_SLOT[0].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_132 SMU__XRDC.PDAC_SLOT[1].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_132 SMU__XRDC.PDAC_SLOT[1].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_140 SMU__XRDC.PDAC_SLOT[1].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_140 SMU__XRDC.PDAC_SLOT[1].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_144 SMU__XRDC.PDAC_SLOT[1].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_144 SMU__XRDC.PDAC_SLOT[1].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_152 SMU__XRDC.PDAC_SLOT[1].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_152 SMU__XRDC.PDAC_SLOT[1].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_153 SMU__XRDC.PDAC_SLOT[1].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_153 SMU__XRDC.PDAC_SLOT[1].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_154 SMU__XRDC.PDAC_SLOT[1].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_154 SMU__XRDC.PDAC_SLOT[1].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_155 SMU__XRDC.PDAC_SLOT[1].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_155 SMU__XRDC.PDAC_SLOT[1].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_164 SMU__XRDC.PDAC_SLOT[1].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_164 SMU__XRDC.PDAC_SLOT[1].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_192 SMU__XRDC.PDAC_SLOT[1].PDACN[64].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_192 SMU__XRDC.PDAC_SLOT[1].PDACN[64].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_196 SMU__XRDC.PDAC_SLOT[1].PDACN[68].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_196 SMU__XRDC.PDAC_SLOT[1].PDACN[68].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_200 SMU__XRDC.PDAC_SLOT[1].PDACN[72].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_200 SMU__XRDC.PDAC_SLOT[1].PDACN[72].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_204 SMU__XRDC.PDAC_SLOT[1].PDACN[76].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_204 SMU__XRDC.PDAC_SLOT[1].PDACN[76].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_208 SMU__XRDC.PDAC_SLOT[1].PDACN[80].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_208 SMU__XRDC.PDAC_SLOT[1].PDACN[80].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_212 SMU__XRDC.PDAC_SLOT[1].PDACN[84].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_212 SMU__XRDC.PDAC_SLOT[1].PDACN[84].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_216 SMU__XRDC.PDAC_SLOT[1].PDACN[88].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_216 SMU__XRDC.PDAC_SLOT[1].PDACN[88].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_220 SMU__XRDC.PDAC_SLOT[1].PDACN[92].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_220 SMU__XRDC.PDAC_SLOT[1].PDACN[92].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_224 SMU__XRDC.PDAC_SLOT[1].PDACN[96].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_224 SMU__XRDC.PDAC_SLOT[1].PDACN[96].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_228 SMU__XRDC.PDAC_SLOT[1].PDACN[100].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_228 SMU__XRDC.PDAC_SLOT[1].PDACN[100].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_232 SMU__XRDC.PDAC_SLOT[1].PDACN[104].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_232 SMU__XRDC.PDAC_SLOT[1].PDACN[104].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_236 SMU__XRDC.PDAC_SLOT[1].PDACN[108].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_236 SMU__XRDC.PDAC_SLOT[1].PDACN[108].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_PDAC_W0_240 SMU__XRDC.PDAC_SLOT[1].PDACN[112].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define SMU__XRDC_PDAC_W1_240 SMU__XRDC.PDAC_SLOT[1].PDACN[112].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define SMU__XRDC_MRGD_W0_0  SMU__XRDC.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_0  SMU__XRDC.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_0  SMU__XRDC.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_0  SMU__XRDC.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_1  SMU__XRDC.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_1  SMU__XRDC.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_1  SMU__XRDC.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_1  SMU__XRDC.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_2  SMU__XRDC.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_2  SMU__XRDC.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_2  SMU__XRDC.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_2  SMU__XRDC.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_3  SMU__XRDC.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_3  SMU__XRDC.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_3  SMU__XRDC.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_3  SMU__XRDC.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_4  SMU__XRDC.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_4  SMU__XRDC.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_4  SMU__XRDC.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_4  SMU__XRDC.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_5  SMU__XRDC.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_5  SMU__XRDC.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_5  SMU__XRDC.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_5  SMU__XRDC.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_6  SMU__XRDC.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_6  SMU__XRDC.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_6  SMU__XRDC.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_6  SMU__XRDC.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_7  SMU__XRDC.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_7  SMU__XRDC.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_7  SMU__XRDC.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_7  SMU__XRDC.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_8  SMU__XRDC.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_8  SMU__XRDC.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_8  SMU__XRDC.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_8  SMU__XRDC.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_9  SMU__XRDC.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_9  SMU__XRDC.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_9  SMU__XRDC.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_9  SMU__XRDC.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_10 SMU__XRDC.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_10 SMU__XRDC.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_10 SMU__XRDC.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_10 SMU__XRDC.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_11 SMU__XRDC.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_11 SMU__XRDC.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_11 SMU__XRDC.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_11 SMU__XRDC.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_12 SMU__XRDC.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_12 SMU__XRDC.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_12 SMU__XRDC.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_12 SMU__XRDC.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_13 SMU__XRDC.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_13 SMU__XRDC.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_13 SMU__XRDC.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_13 SMU__XRDC.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_14 SMU__XRDC.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_14 SMU__XRDC.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_14 SMU__XRDC.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_14 SMU__XRDC.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_15 SMU__XRDC.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_15 SMU__XRDC.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_15 SMU__XRDC.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_15 SMU__XRDC.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_16 SMU__XRDC.MRGDN[1][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_16 SMU__XRDC.MRGDN[1][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_16 SMU__XRDC.MRGDN[1][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_16 SMU__XRDC.MRGDN[1][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_17 SMU__XRDC.MRGDN[1][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_17 SMU__XRDC.MRGDN[1][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_17 SMU__XRDC.MRGDN[1][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_17 SMU__XRDC.MRGDN[1][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_18 SMU__XRDC.MRGDN[1][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_18 SMU__XRDC.MRGDN[1][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_18 SMU__XRDC.MRGDN[1][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_18 SMU__XRDC.MRGDN[1][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_19 SMU__XRDC.MRGDN[1][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_19 SMU__XRDC.MRGDN[1][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_19 SMU__XRDC.MRGDN[1][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_19 SMU__XRDC.MRGDN[1][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_20 SMU__XRDC.MRGDN[1][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_20 SMU__XRDC.MRGDN[1][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_20 SMU__XRDC.MRGDN[1][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_20 SMU__XRDC.MRGDN[1][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_21 SMU__XRDC.MRGDN[1][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_21 SMU__XRDC.MRGDN[1][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_21 SMU__XRDC.MRGDN[1][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_21 SMU__XRDC.MRGDN[1][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_22 SMU__XRDC.MRGDN[1][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_22 SMU__XRDC.MRGDN[1][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_22 SMU__XRDC.MRGDN[1][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_22 SMU__XRDC.MRGDN[1][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_23 SMU__XRDC.MRGDN[1][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_23 SMU__XRDC.MRGDN[1][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_23 SMU__XRDC.MRGDN[1][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_23 SMU__XRDC.MRGDN[1][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_24 SMU__XRDC.MRGDN[1][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_24 SMU__XRDC.MRGDN[1][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_24 SMU__XRDC.MRGDN[1][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_24 SMU__XRDC.MRGDN[1][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_25 SMU__XRDC.MRGDN[1][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_25 SMU__XRDC.MRGDN[1][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_25 SMU__XRDC.MRGDN[1][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_25 SMU__XRDC.MRGDN[1][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_26 SMU__XRDC.MRGDN[1][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_26 SMU__XRDC.MRGDN[1][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_26 SMU__XRDC.MRGDN[1][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_26 SMU__XRDC.MRGDN[1][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_27 SMU__XRDC.MRGDN[1][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_27 SMU__XRDC.MRGDN[1][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_27 SMU__XRDC.MRGDN[1][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_27 SMU__XRDC.MRGDN[1][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_28 SMU__XRDC.MRGDN[1][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_28 SMU__XRDC.MRGDN[1][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_28 SMU__XRDC.MRGDN[1][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_28 SMU__XRDC.MRGDN[1][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_29 SMU__XRDC.MRGDN[1][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_29 SMU__XRDC.MRGDN[1][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_29 SMU__XRDC.MRGDN[1][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_29 SMU__XRDC.MRGDN[1][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_30 SMU__XRDC.MRGDN[1][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_30 SMU__XRDC.MRGDN[1][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_30 SMU__XRDC.MRGDN[1][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_30 SMU__XRDC.MRGDN[1][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_31 SMU__XRDC.MRGDN[1][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_31 SMU__XRDC.MRGDN[1][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_31 SMU__XRDC.MRGDN[1][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_31 SMU__XRDC.MRGDN[1][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_32 SMU__XRDC.MRGDN[2][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_32 SMU__XRDC.MRGDN[2][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_32 SMU__XRDC.MRGDN[2][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_32 SMU__XRDC.MRGDN[2][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_33 SMU__XRDC.MRGDN[2][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_33 SMU__XRDC.MRGDN[2][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_33 SMU__XRDC.MRGDN[2][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_33 SMU__XRDC.MRGDN[2][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_34 SMU__XRDC.MRGDN[2][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_34 SMU__XRDC.MRGDN[2][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_34 SMU__XRDC.MRGDN[2][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_34 SMU__XRDC.MRGDN[2][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_35 SMU__XRDC.MRGDN[2][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_35 SMU__XRDC.MRGDN[2][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_35 SMU__XRDC.MRGDN[2][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_35 SMU__XRDC.MRGDN[2][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_36 SMU__XRDC.MRGDN[2][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_36 SMU__XRDC.MRGDN[2][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_36 SMU__XRDC.MRGDN[2][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_36 SMU__XRDC.MRGDN[2][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_37 SMU__XRDC.MRGDN[2][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_37 SMU__XRDC.MRGDN[2][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_37 SMU__XRDC.MRGDN[2][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_37 SMU__XRDC.MRGDN[2][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_38 SMU__XRDC.MRGDN[2][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_38 SMU__XRDC.MRGDN[2][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_38 SMU__XRDC.MRGDN[2][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_38 SMU__XRDC.MRGDN[2][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_39 SMU__XRDC.MRGDN[2][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_39 SMU__XRDC.MRGDN[2][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_39 SMU__XRDC.MRGDN[2][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_39 SMU__XRDC.MRGDN[2][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_40 SMU__XRDC.MRGDN[2][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_40 SMU__XRDC.MRGDN[2][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_40 SMU__XRDC.MRGDN[2][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_40 SMU__XRDC.MRGDN[2][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_41 SMU__XRDC.MRGDN[2][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_41 SMU__XRDC.MRGDN[2][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_41 SMU__XRDC.MRGDN[2][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_41 SMU__XRDC.MRGDN[2][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_42 SMU__XRDC.MRGDN[2][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_42 SMU__XRDC.MRGDN[2][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_42 SMU__XRDC.MRGDN[2][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_42 SMU__XRDC.MRGDN[2][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_43 SMU__XRDC.MRGDN[2][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_43 SMU__XRDC.MRGDN[2][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_43 SMU__XRDC.MRGDN[2][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_43 SMU__XRDC.MRGDN[2][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_44 SMU__XRDC.MRGDN[2][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_44 SMU__XRDC.MRGDN[2][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_44 SMU__XRDC.MRGDN[2][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_44 SMU__XRDC.MRGDN[2][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_45 SMU__XRDC.MRGDN[2][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_45 SMU__XRDC.MRGDN[2][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_45 SMU__XRDC.MRGDN[2][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_45 SMU__XRDC.MRGDN[2][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_46 SMU__XRDC.MRGDN[2][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_46 SMU__XRDC.MRGDN[2][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_46 SMU__XRDC.MRGDN[2][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_46 SMU__XRDC.MRGDN[2][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_47 SMU__XRDC.MRGDN[2][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_47 SMU__XRDC.MRGDN[2][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_47 SMU__XRDC.MRGDN[2][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_47 SMU__XRDC.MRGDN[2][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_48 SMU__XRDC.MRGDN[3][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_48 SMU__XRDC.MRGDN[3][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_48 SMU__XRDC.MRGDN[3][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_48 SMU__XRDC.MRGDN[3][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_49 SMU__XRDC.MRGDN[3][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_49 SMU__XRDC.MRGDN[3][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_49 SMU__XRDC.MRGDN[3][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_49 SMU__XRDC.MRGDN[3][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_50 SMU__XRDC.MRGDN[3][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_50 SMU__XRDC.MRGDN[3][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_50 SMU__XRDC.MRGDN[3][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_50 SMU__XRDC.MRGDN[3][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_51 SMU__XRDC.MRGDN[3][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_51 SMU__XRDC.MRGDN[3][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_51 SMU__XRDC.MRGDN[3][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_51 SMU__XRDC.MRGDN[3][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_52 SMU__XRDC.MRGDN[3][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_52 SMU__XRDC.MRGDN[3][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_52 SMU__XRDC.MRGDN[3][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_52 SMU__XRDC.MRGDN[3][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_53 SMU__XRDC.MRGDN[3][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_53 SMU__XRDC.MRGDN[3][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_53 SMU__XRDC.MRGDN[3][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_53 SMU__XRDC.MRGDN[3][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_54 SMU__XRDC.MRGDN[3][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_54 SMU__XRDC.MRGDN[3][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_54 SMU__XRDC.MRGDN[3][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_54 SMU__XRDC.MRGDN[3][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_55 SMU__XRDC.MRGDN[3][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_55 SMU__XRDC.MRGDN[3][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_55 SMU__XRDC.MRGDN[3][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_55 SMU__XRDC.MRGDN[3][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_56 SMU__XRDC.MRGDN[3][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_56 SMU__XRDC.MRGDN[3][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_56 SMU__XRDC.MRGDN[3][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_56 SMU__XRDC.MRGDN[3][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_57 SMU__XRDC.MRGDN[3][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_57 SMU__XRDC.MRGDN[3][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_57 SMU__XRDC.MRGDN[3][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_57 SMU__XRDC.MRGDN[3][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_58 SMU__XRDC.MRGDN[3][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_58 SMU__XRDC.MRGDN[3][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_58 SMU__XRDC.MRGDN[3][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_58 SMU__XRDC.MRGDN[3][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_59 SMU__XRDC.MRGDN[3][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_59 SMU__XRDC.MRGDN[3][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_59 SMU__XRDC.MRGDN[3][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_59 SMU__XRDC.MRGDN[3][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_60 SMU__XRDC.MRGDN[3][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_60 SMU__XRDC.MRGDN[3][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_60 SMU__XRDC.MRGDN[3][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_60 SMU__XRDC.MRGDN[3][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_61 SMU__XRDC.MRGDN[3][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_61 SMU__XRDC.MRGDN[3][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_61 SMU__XRDC.MRGDN[3][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_61 SMU__XRDC.MRGDN[3][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_62 SMU__XRDC.MRGDN[3][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_62 SMU__XRDC.MRGDN[3][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_62 SMU__XRDC.MRGDN[3][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_62 SMU__XRDC.MRGDN[3][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define SMU__XRDC_MRGD_W0_63 SMU__XRDC.MRGDN[3][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define SMU__XRDC_MRGD_W1_63 SMU__XRDC.MRGDN[3][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define SMU__XRDC_MRGD_W2_63 SMU__XRDC.MRGDN[3][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define SMU__XRDC_MRGD_W3_63 SMU__XRDC.MRGDN[3][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* SPFU */
#define RTU0__SPFU_PF0_CFG   RTU0__SPFU.PFN[0].PF_CFG.R    /* Prefetch Configuration */
#define RTU0__SPFU_PF1_CFG   RTU0__SPFU.PFN[1].PF_CFG.R    /* Prefetch Configuration */
#define RTU0__SPFU_PF2_CFG   RTU0__SPFU.PFN[2].PF_CFG.R    /* Prefetch Configuration */
#define RTU0__SPFU_PF3_CFG   RTU0__SPFU.PFN[3].PF_CFG.R    /* Prefetch Configuration */
#define RTU0__SPFU_PF4_CFG   RTU0__SPFU.PFN[4].PF_CFG.R    /* Prefetch Configuration */

#define RTU1__SPFU_PF0_CFG   RTU1__SPFU.PFN[0].PF_CFG.R    /* Prefetch Configuration */
#define RTU1__SPFU_PF1_CFG   RTU1__SPFU.PFN[1].PF_CFG.R    /* Prefetch Configuration */
#define RTU1__SPFU_PF2_CFG   RTU1__SPFU.PFN[2].PF_CFG.R    /* Prefetch Configuration */
#define RTU1__SPFU_PF3_CFG   RTU1__SPFU.PFN[3].PF_CFG.R    /* Prefetch Configuration */
#define RTU1__SPFU_PF4_CFG   RTU1__SPFU.PFN[4].PF_CFG.R    /* Prefetch Configuration */

/* SPI */
#define SPI_2_MCR            SPI_2.MCR.R                   /* Module Configuration Register */
#define SPI_2_TCR            SPI_2.TCR.R                   /* Transfer Count Register */
#define SPI_2_CTAR0          SPI_2.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_2_CTAR1          SPI_2.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_2_CTAR2          SPI_2.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_2_CTAR3          SPI_2.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_2_CTAR4          SPI_2.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_2_CTAR5          SPI_2.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_2_CTAR0_SLAVE    SPI_2.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_2_SR             SPI_2.SR.R                    /* Status Register */
#define SPI_2_RSER           SPI_2.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_2_PUSHR          SPI_2.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_2_PUSHR_SLAVE    SPI_2.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_2_POPR           SPI_2.POPR.R                  /* POP RX FIFO Register */
#define SPI_2_TXFR0          SPI_2.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_2_TXFR1          SPI_2.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_2_TXFR2          SPI_2.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_2_TXFR3          SPI_2.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_2_TXFR4          SPI_2.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_2_RXFR0          SPI_2.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_2_RXFR1          SPI_2.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_2_RXFR2          SPI_2.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_2_RXFR3          SPI_2.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_2_RXFR4          SPI_2.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_2_CTARE0         SPI_2.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_2_CTARE1         SPI_2.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_2_CTARE2         SPI_2.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_2_CTARE3         SPI_2.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_2_CTARE4         SPI_2.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_2_CTARE5         SPI_2.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_2_SREX           SPI_2.SREX.R                  /* Status Register Extended */

#define SPI_3_MCR            SPI_3.MCR.R                   /* Module Configuration Register */
#define SPI_3_TCR            SPI_3.TCR.R                   /* Transfer Count Register */
#define SPI_3_CTAR0          SPI_3.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_3_CTAR1          SPI_3.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_3_CTAR2          SPI_3.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_3_CTAR3          SPI_3.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_3_CTAR4          SPI_3.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_3_CTAR5          SPI_3.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_3_CTAR0_SLAVE    SPI_3.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_3_SR             SPI_3.SR.R                    /* Status Register */
#define SPI_3_RSER           SPI_3.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_3_PUSHR          SPI_3.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_3_PUSHR_SLAVE    SPI_3.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_3_POPR           SPI_3.POPR.R                  /* POP RX FIFO Register */
#define SPI_3_TXFR0          SPI_3.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_3_TXFR1          SPI_3.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_3_TXFR2          SPI_3.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_3_TXFR3          SPI_3.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_3_TXFR4          SPI_3.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_3_RXFR0          SPI_3.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_3_RXFR1          SPI_3.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_3_RXFR2          SPI_3.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_3_RXFR3          SPI_3.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_3_RXFR4          SPI_3.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_3_CTARE0         SPI_3.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_3_CTARE1         SPI_3.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_3_CTARE2         SPI_3.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_3_CTARE3         SPI_3.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_3_CTARE4         SPI_3.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_3_CTARE5         SPI_3.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_3_SREX           SPI_3.SREX.R                  /* Status Register Extended */

#define SPI_4_MCR            SPI_4.MCR.R                   /* Module Configuration Register */
#define SPI_4_TCR            SPI_4.TCR.R                   /* Transfer Count Register */
#define SPI_4_CTAR0          SPI_4.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_4_CTAR1          SPI_4.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_4_CTAR2          SPI_4.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_4_CTAR3          SPI_4.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_4_CTAR4          SPI_4.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_4_CTAR5          SPI_4.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_4_CTAR0_SLAVE    SPI_4.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_4_SR             SPI_4.SR.R                    /* Status Register */
#define SPI_4_RSER           SPI_4.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_4_PUSHR          SPI_4.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_4_PUSHR_SLAVE    SPI_4.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_4_POPR           SPI_4.POPR.R                  /* POP RX FIFO Register */
#define SPI_4_TXFR0          SPI_4.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_4_TXFR1          SPI_4.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_4_TXFR2          SPI_4.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_4_TXFR3          SPI_4.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_4_TXFR4          SPI_4.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_4_RXFR0          SPI_4.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_4_RXFR1          SPI_4.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_4_RXFR2          SPI_4.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_4_RXFR3          SPI_4.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_4_RXFR4          SPI_4.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_4_CTARE0         SPI_4.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_4_CTARE1         SPI_4.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_4_CTARE2         SPI_4.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_4_CTARE3         SPI_4.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_4_CTARE4         SPI_4.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_4_CTARE5         SPI_4.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_4_SREX           SPI_4.SREX.R                  /* Status Register Extended */

#define SPI_0_MCR            SPI_0.MCR.R                   /* Module Configuration Register */
#define SPI_0_TCR            SPI_0.TCR.R                   /* Transfer Count Register */
#define SPI_0_CTAR0          SPI_0.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_0_CTAR1          SPI_0.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_0_CTAR2          SPI_0.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_0_CTAR3          SPI_0.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_0_CTAR4          SPI_0.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_0_CTAR5          SPI_0.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_0_CTAR0_SLAVE    SPI_0.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_0_SR             SPI_0.SR.R                    /* Status Register */
#define SPI_0_RSER           SPI_0.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_0_PUSHR          SPI_0.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_0_PUSHR_SLAVE    SPI_0.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_0_POPR           SPI_0.POPR.R                  /* POP RX FIFO Register */
#define SPI_0_TXFR0          SPI_0.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_0_TXFR1          SPI_0.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_0_TXFR2          SPI_0.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_0_TXFR3          SPI_0.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_0_TXFR4          SPI_0.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_0_RXFR0          SPI_0.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_0_RXFR1          SPI_0.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_0_RXFR2          SPI_0.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_0_RXFR3          SPI_0.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_0_RXFR4          SPI_0.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_0_CTARE0         SPI_0.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_0_CTARE1         SPI_0.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_0_CTARE2         SPI_0.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_0_CTARE3         SPI_0.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_0_CTARE4         SPI_0.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_0_CTARE5         SPI_0.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_0_SREX           SPI_0.SREX.R                  /* Status Register Extended */

#define SPI_1_MCR            SPI_1.MCR.R                   /* Module Configuration Register */
#define SPI_1_TCR            SPI_1.TCR.R                   /* Transfer Count Register */
#define SPI_1_CTAR0          SPI_1.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_1_CTAR1          SPI_1.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_1_CTAR2          SPI_1.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_1_CTAR3          SPI_1.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_1_CTAR4          SPI_1.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_1_CTAR5          SPI_1.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_1_CTAR0_SLAVE    SPI_1.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_1_SR             SPI_1.SR.R                    /* Status Register */
#define SPI_1_RSER           SPI_1.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_1_PUSHR          SPI_1.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_1_PUSHR_SLAVE    SPI_1.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_1_POPR           SPI_1.POPR.R                  /* POP RX FIFO Register */
#define SPI_1_TXFR0          SPI_1.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_1_TXFR1          SPI_1.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_1_TXFR2          SPI_1.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_1_TXFR3          SPI_1.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_1_TXFR4          SPI_1.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_1_RXFR0          SPI_1.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_1_RXFR1          SPI_1.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_1_RXFR2          SPI_1.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_1_RXFR3          SPI_1.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_1_RXFR4          SPI_1.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_1_CTARE0         SPI_1.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_1_CTARE1         SPI_1.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_1_CTARE2         SPI_1.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_1_CTARE3         SPI_1.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_1_CTARE4         SPI_1.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_1_CTARE5         SPI_1.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_1_SREX           SPI_1.SREX.R                  /* Status Register Extended */

#define SPI_5_MCR            SPI_5.MCR.R                   /* Module Configuration Register */
#define SPI_5_TCR            SPI_5.TCR.R                   /* Transfer Count Register */
#define SPI_5_CTAR0          SPI_5.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_5_CTAR1          SPI_5.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_5_CTAR2          SPI_5.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_5_CTAR3          SPI_5.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_5_CTAR4          SPI_5.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_5_CTAR5          SPI_5.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_5_CTAR0_SLAVE    SPI_5.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_5_SR             SPI_5.SR.R                    /* Status Register */
#define SPI_5_RSER           SPI_5.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_5_PUSHR          SPI_5.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_5_PUSHR_SLAVE    SPI_5.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_5_POPR           SPI_5.POPR.R                  /* POP RX FIFO Register */
#define SPI_5_TXFR0          SPI_5.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_5_TXFR1          SPI_5.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_5_TXFR2          SPI_5.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_5_TXFR3          SPI_5.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_5_TXFR4          SPI_5.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_5_RXFR0          SPI_5.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_5_RXFR1          SPI_5.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_5_RXFR2          SPI_5.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_5_RXFR3          SPI_5.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_5_RXFR4          SPI_5.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_5_CTARE0         SPI_5.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_5_CTARE1         SPI_5.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_5_CTARE2         SPI_5.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_5_CTARE3         SPI_5.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_5_CTARE4         SPI_5.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_5_CTARE5         SPI_5.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_5_SREX           SPI_5.SREX.R                  /* Status Register Extended */

#define SPI_6_MCR            SPI_6.MCR.R                   /* Module Configuration Register */
#define SPI_6_TCR            SPI_6.TCR.R                   /* Transfer Count Register */
#define SPI_6_CTAR0          SPI_6.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_6_CTAR1          SPI_6.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_6_CTAR2          SPI_6.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_6_CTAR3          SPI_6.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_6_CTAR4          SPI_6.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_6_CTAR5          SPI_6.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_6_CTAR0_SLAVE    SPI_6.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_6_SR             SPI_6.SR.R                    /* Status Register */
#define SPI_6_RSER           SPI_6.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_6_PUSHR          SPI_6.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_6_PUSHR_SLAVE    SPI_6.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_6_POPR           SPI_6.POPR.R                  /* POP RX FIFO Register */
#define SPI_6_TXFR0          SPI_6.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_6_TXFR1          SPI_6.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_6_TXFR2          SPI_6.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_6_TXFR3          SPI_6.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_6_TXFR4          SPI_6.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_6_RXFR0          SPI_6.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_6_RXFR1          SPI_6.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_6_RXFR2          SPI_6.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_6_RXFR3          SPI_6.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_6_RXFR4          SPI_6.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_6_CTARE0         SPI_6.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_6_CTARE1         SPI_6.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_6_CTARE2         SPI_6.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_6_CTARE3         SPI_6.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_6_CTARE4         SPI_6.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_6_CTARE5         SPI_6.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_6_SREX           SPI_6.SREX.R                  /* Status Register Extended */

#define SPI_7_MCR            SPI_7.MCR.R                   /* Module Configuration Register */
#define SPI_7_TCR            SPI_7.TCR.R                   /* Transfer Count Register */
#define SPI_7_CTAR0          SPI_7.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_7_CTAR1          SPI_7.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_7_CTAR2          SPI_7.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_7_CTAR3          SPI_7.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_7_CTAR4          SPI_7.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_7_CTAR5          SPI_7.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_7_CTAR0_SLAVE    SPI_7.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_7_SR             SPI_7.SR.R                    /* Status Register */
#define SPI_7_RSER           SPI_7.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_7_PUSHR          SPI_7.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_7_PUSHR_SLAVE    SPI_7.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_7_POPR           SPI_7.POPR.R                  /* POP RX FIFO Register */
#define SPI_7_TXFR0          SPI_7.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_7_TXFR1          SPI_7.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_7_TXFR2          SPI_7.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_7_TXFR3          SPI_7.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_7_TXFR4          SPI_7.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_7_RXFR0          SPI_7.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_7_RXFR1          SPI_7.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_7_RXFR2          SPI_7.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_7_RXFR3          SPI_7.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_7_RXFR4          SPI_7.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_7_CTARE0         SPI_7.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_7_CTARE1         SPI_7.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_7_CTARE2         SPI_7.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_7_CTARE3         SPI_7.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_7_CTARE4         SPI_7.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_7_CTARE5         SPI_7.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_7_SREX           SPI_7.SREX.R                  /* Status Register Extended */

#define SPI_8_MCR            SPI_8.MCR.R                   /* Module Configuration Register */
#define SPI_8_TCR            SPI_8.TCR.R                   /* Transfer Count Register */
#define SPI_8_CTAR0          SPI_8.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_8_CTAR1          SPI_8.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_8_CTAR2          SPI_8.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_8_CTAR3          SPI_8.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_8_CTAR4          SPI_8.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_8_CTAR5          SPI_8.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_8_CTAR0_SLAVE    SPI_8.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_8_SR             SPI_8.SR.R                    /* Status Register */
#define SPI_8_RSER           SPI_8.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_8_PUSHR          SPI_8.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_8_PUSHR_SLAVE    SPI_8.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_8_POPR           SPI_8.POPR.R                  /* POP RX FIFO Register */
#define SPI_8_TXFR0          SPI_8.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_8_TXFR1          SPI_8.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_8_TXFR2          SPI_8.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_8_TXFR3          SPI_8.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_8_TXFR4          SPI_8.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_8_RXFR0          SPI_8.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_8_RXFR1          SPI_8.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_8_RXFR2          SPI_8.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_8_RXFR3          SPI_8.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_8_RXFR4          SPI_8.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_8_CTARE0         SPI_8.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_8_CTARE1         SPI_8.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_8_CTARE2         SPI_8.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_8_CTARE3         SPI_8.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_8_CTARE4         SPI_8.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_8_CTARE5         SPI_8.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_8_SREX           SPI_8.SREX.R                  /* Status Register Extended */

#define SPI_9_MCR            SPI_9.MCR.R                   /* Module Configuration Register */
#define SPI_9_TCR            SPI_9.TCR.R                   /* Transfer Count Register */
#define SPI_9_CTAR0          SPI_9.MODE.CTAR[0].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_9_CTAR1          SPI_9.MODE.CTAR[1].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_9_CTAR2          SPI_9.MODE.CTAR[2].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_9_CTAR3          SPI_9.MODE.CTAR[3].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_9_CTAR4          SPI_9.MODE.CTAR[4].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_9_CTAR5          SPI_9.MODE.CTAR[5].R          /* Clock and Transfer Attributes Register (in Master mode) */
#define SPI_9_CTAR0_SLAVE    SPI_9.MODE.CTAR_SLAVE[0].R    /* Clock and Transfer Attributes Register (in Slave mode) */
#define SPI_9_SR             SPI_9.SR.R                    /* Status Register */
#define SPI_9_RSER           SPI_9.RSER.R                  /* DMA/Interrupt Request Select and Enable Register */
#define SPI_9_PUSHR          SPI_9.PUSHR.PUSHR.R           /* PUSH TX FIFO Register In Master Mode */
#define SPI_9_PUSHR_SLAVE    SPI_9.PUSHR.PUSHR_SLAVE.R     /* PUSH TX FIFO Register In Slave Mode */
#define SPI_9_POPR           SPI_9.POPR.R                  /* POP RX FIFO Register */
#define SPI_9_TXFR0          SPI_9.TXFR[0].R               /* Transmit FIFO Registers */
#define SPI_9_TXFR1          SPI_9.TXFR[1].R               /* Transmit FIFO Registers */
#define SPI_9_TXFR2          SPI_9.TXFR[2].R               /* Transmit FIFO Registers */
#define SPI_9_TXFR3          SPI_9.TXFR[3].R               /* Transmit FIFO Registers */
#define SPI_9_TXFR4          SPI_9.TXFR[4].R               /* Transmit FIFO Registers */
#define SPI_9_RXFR0          SPI_9.RXFR[0].R               /* Receive FIFO Registers */
#define SPI_9_RXFR1          SPI_9.RXFR[1].R               /* Receive FIFO Registers */
#define SPI_9_RXFR2          SPI_9.RXFR[2].R               /* Receive FIFO Registers */
#define SPI_9_RXFR3          SPI_9.RXFR[3].R               /* Receive FIFO Registers */
#define SPI_9_RXFR4          SPI_9.RXFR[4].R               /* Receive FIFO Registers */
#define SPI_9_CTARE0         SPI_9.CTARE[0].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_9_CTARE1         SPI_9.CTARE[1].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_9_CTARE2         SPI_9.CTARE[2].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_9_CTARE3         SPI_9.CTARE[3].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_9_CTARE4         SPI_9.CTARE[4].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_9_CTARE5         SPI_9.CTARE[5].R              /* Clock and Transfer Attributes Register Extended */
#define SPI_9_SREX           SPI_9.SREX.R                  /* Status Register Extended */

/* SRAMCTL */
#define CE_SRAMCTL_0_RAMCR   CE_SRAMCTL_0.RAMCR.R          /* RAM Control */
#define CE_SRAMCTL_0_RAMIAS  CE_SRAMCTL_0.RAMIAS.R         /* RAM Initialization Address Start */
#define CE_SRAMCTL_0_RAMIAE  CE_SRAMCTL_0.RAMIAE.R         /* RAM Initialization Address End */
#define CE_SRAMCTL_0_RAMSR   CE_SRAMCTL_0.RAMSR.R          /* RAM Status */
#define CE_SRAMCTL_0_RAMMEMA CE_SRAMCTL_0.RAMMEMA.R        /* RAM ECC Address */
#define CE_SRAMCTL_0_RAMSYSA CE_SRAMCTL_0.RAMSYSA.R        /* RAM System Address */
#define CE_SRAMCTL_0_RAMECCNT CE_SRAMCTL_0.RAMECCNT.R      /* RAM Correctable Error Count */
#define CE_SRAMCTL_0_RAMEID0 CE_SRAMCTL_0.RAMEID0.R        /* RAM Error Injection Data 0 */
#define CE_SRAMCTL_0_RAMEID1 CE_SRAMCTL_0.RAMEID1.R        /* RAM Error Injection Data 1 */
#define CE_SRAMCTL_0_RAMEIDC CE_SRAMCTL_0.RAMEIDC.R        /* RAM Error Injection Data Control */
#define CE_SRAMCTL_0_RAMEIA  CE_SRAMCTL_0.RAMEIA.R         /* RAM Error Injection Base Address */
#define CE_SRAMCTL_0_RAMEIAM CE_SRAMCTL_0.RAMEIAM.R        /* RAM Error Injection Address Mask */
#define CE_SRAMCTL_0_RAMMAXA CE_SRAMCTL_0.RAMMAXA.R        /* RAM Maximum-Value Address */
#define CE_SRAMCTL_0_RAMCR2  CE_SRAMCTL_0.RAMCR2.R         /* RAM Control 2 */

#define CE_SRAMCTL_1_RAMCR   CE_SRAMCTL_1.RAMCR.R          /* RAM Control */
#define CE_SRAMCTL_1_RAMIAS  CE_SRAMCTL_1.RAMIAS.R         /* RAM Initialization Address Start */
#define CE_SRAMCTL_1_RAMIAE  CE_SRAMCTL_1.RAMIAE.R         /* RAM Initialization Address End */
#define CE_SRAMCTL_1_RAMSR   CE_SRAMCTL_1.RAMSR.R          /* RAM Status */
#define CE_SRAMCTL_1_RAMMEMA CE_SRAMCTL_1.RAMMEMA.R        /* RAM ECC Address */
#define CE_SRAMCTL_1_RAMSYSA CE_SRAMCTL_1.RAMSYSA.R        /* RAM System Address */
#define CE_SRAMCTL_1_RAMECCNT CE_SRAMCTL_1.RAMECCNT.R      /* RAM Correctable Error Count */
#define CE_SRAMCTL_1_RAMEID0 CE_SRAMCTL_1.RAMEID0.R        /* RAM Error Injection Data 0 */
#define CE_SRAMCTL_1_RAMEID1 CE_SRAMCTL_1.RAMEID1.R        /* RAM Error Injection Data 1 */
#define CE_SRAMCTL_1_RAMEIDC CE_SRAMCTL_1.RAMEIDC.R        /* RAM Error Injection Data Control */
#define CE_SRAMCTL_1_RAMEIA  CE_SRAMCTL_1.RAMEIA.R         /* RAM Error Injection Base Address */
#define CE_SRAMCTL_1_RAMEIAM CE_SRAMCTL_1.RAMEIAM.R        /* RAM Error Injection Address Mask */
#define CE_SRAMCTL_1_RAMMAXA CE_SRAMCTL_1.RAMMAXA.R        /* RAM Maximum-Value Address */
#define CE_SRAMCTL_1_RAMCR2  CE_SRAMCTL_1.RAMCR2.R         /* RAM Control 2 */

#define CE_SRAMCTL_2_RAMCR   CE_SRAMCTL_2.RAMCR.R          /* RAM Control */
#define CE_SRAMCTL_2_RAMIAS  CE_SRAMCTL_2.RAMIAS.R         /* RAM Initialization Address Start */
#define CE_SRAMCTL_2_RAMIAE  CE_SRAMCTL_2.RAMIAE.R         /* RAM Initialization Address End */
#define CE_SRAMCTL_2_RAMSR   CE_SRAMCTL_2.RAMSR.R          /* RAM Status */
#define CE_SRAMCTL_2_RAMMEMA CE_SRAMCTL_2.RAMMEMA.R        /* RAM ECC Address */
#define CE_SRAMCTL_2_RAMSYSA CE_SRAMCTL_2.RAMSYSA.R        /* RAM System Address */
#define CE_SRAMCTL_2_RAMECCNT CE_SRAMCTL_2.RAMECCNT.R      /* RAM Correctable Error Count */
#define CE_SRAMCTL_2_RAMEID0 CE_SRAMCTL_2.RAMEID0.R        /* RAM Error Injection Data 0 */
#define CE_SRAMCTL_2_RAMEID1 CE_SRAMCTL_2.RAMEID1.R        /* RAM Error Injection Data 1 */
#define CE_SRAMCTL_2_RAMEIDC CE_SRAMCTL_2.RAMEIDC.R        /* RAM Error Injection Data Control */
#define CE_SRAMCTL_2_RAMEIA  CE_SRAMCTL_2.RAMEIA.R         /* RAM Error Injection Base Address */
#define CE_SRAMCTL_2_RAMEIAM CE_SRAMCTL_2.RAMEIAM.R        /* RAM Error Injection Address Mask */
#define CE_SRAMCTL_2_RAMMAXA CE_SRAMCTL_2.RAMMAXA.R        /* RAM Maximum-Value Address */
#define CE_SRAMCTL_2_RAMCR2  CE_SRAMCTL_2.RAMCR2.R         /* RAM Control 2 */

#define SMU__SRAMCTL_0_RAMCR SMU__SRAMCTL_0.RAMCR.R        /* RAM Control */
#define SMU__SRAMCTL_0_RAMIAS SMU__SRAMCTL_0.RAMIAS.R      /* RAM Initialization Address Start */
#define SMU__SRAMCTL_0_RAMIAE SMU__SRAMCTL_0.RAMIAE.R      /* RAM Initialization Address End */
#define SMU__SRAMCTL_0_RAMSR SMU__SRAMCTL_0.RAMSR.R        /* RAM Status */
#define SMU__SRAMCTL_0_RAMMEMA SMU__SRAMCTL_0.RAMMEMA.R    /* RAM ECC Address */
#define SMU__SRAMCTL_0_RAMSYSA SMU__SRAMCTL_0.RAMSYSA.R    /* RAM System Address */
#define SMU__SRAMCTL_0_RAMECCNT SMU__SRAMCTL_0.RAMECCNT.R  /* RAM Correctable Error Count */
#define SMU__SRAMCTL_0_RAMEID0 SMU__SRAMCTL_0.RAMEID0.R    /* RAM Error Injection Data 0 */
#define SMU__SRAMCTL_0_RAMEID1 SMU__SRAMCTL_0.RAMEID1.R    /* RAM Error Injection Data 1 */
#define SMU__SRAMCTL_0_RAMEIDC SMU__SRAMCTL_0.RAMEIDC.R    /* RAM Error Injection Data Control */
#define SMU__SRAMCTL_0_RAMEIA SMU__SRAMCTL_0.RAMEIA.R      /* RAM Error Injection Base Address */
#define SMU__SRAMCTL_0_RAMEIAM SMU__SRAMCTL_0.RAMEIAM.R    /* RAM Error Injection Address Mask */
#define SMU__SRAMCTL_0_RAMMAXA SMU__SRAMCTL_0.RAMMAXA.R    /* RAM Maximum-Value Address */
#define SMU__SRAMCTL_0_RAMCR2 SMU__SRAMCTL_0.RAMCR2.R      /* RAM Control 2 */

#define SMU__SRAMCTL_1_RAMCR SMU__SRAMCTL_1.RAMCR.R        /* RAM Control */
#define SMU__SRAMCTL_1_RAMIAS SMU__SRAMCTL_1.RAMIAS.R      /* RAM Initialization Address Start */
#define SMU__SRAMCTL_1_RAMIAE SMU__SRAMCTL_1.RAMIAE.R      /* RAM Initialization Address End */
#define SMU__SRAMCTL_1_RAMSR SMU__SRAMCTL_1.RAMSR.R        /* RAM Status */
#define SMU__SRAMCTL_1_RAMMEMA SMU__SRAMCTL_1.RAMMEMA.R    /* RAM ECC Address */
#define SMU__SRAMCTL_1_RAMSYSA SMU__SRAMCTL_1.RAMSYSA.R    /* RAM System Address */
#define SMU__SRAMCTL_1_RAMECCNT SMU__SRAMCTL_1.RAMECCNT.R  /* RAM Correctable Error Count */
#define SMU__SRAMCTL_1_RAMEID0 SMU__SRAMCTL_1.RAMEID0.R    /* RAM Error Injection Data 0 */
#define SMU__SRAMCTL_1_RAMEID1 SMU__SRAMCTL_1.RAMEID1.R    /* RAM Error Injection Data 1 */
#define SMU__SRAMCTL_1_RAMEIDC SMU__SRAMCTL_1.RAMEIDC.R    /* RAM Error Injection Data Control */
#define SMU__SRAMCTL_1_RAMEIA SMU__SRAMCTL_1.RAMEIA.R      /* RAM Error Injection Base Address */
#define SMU__SRAMCTL_1_RAMEIAM SMU__SRAMCTL_1.RAMEIAM.R    /* RAM Error Injection Address Mask */
#define SMU__SRAMCTL_1_RAMMAXA SMU__SRAMCTL_1.RAMMAXA.R    /* RAM Maximum-Value Address */
#define SMU__SRAMCTL_1_RAMCR2 SMU__SRAMCTL_1.RAMCR2.R      /* RAM Control 2 */

#define SMU__SRAMCTL_2_RAMCR SMU__SRAMCTL_2.RAMCR.R        /* RAM Control */
#define SMU__SRAMCTL_2_RAMIAS SMU__SRAMCTL_2.RAMIAS.R      /* RAM Initialization Address Start */
#define SMU__SRAMCTL_2_RAMIAE SMU__SRAMCTL_2.RAMIAE.R      /* RAM Initialization Address End */
#define SMU__SRAMCTL_2_RAMSR SMU__SRAMCTL_2.RAMSR.R        /* RAM Status */
#define SMU__SRAMCTL_2_RAMMEMA SMU__SRAMCTL_2.RAMMEMA.R    /* RAM ECC Address */
#define SMU__SRAMCTL_2_RAMSYSA SMU__SRAMCTL_2.RAMSYSA.R    /* RAM System Address */
#define SMU__SRAMCTL_2_RAMECCNT SMU__SRAMCTL_2.RAMECCNT.R  /* RAM Correctable Error Count */
#define SMU__SRAMCTL_2_RAMEID0 SMU__SRAMCTL_2.RAMEID0.R    /* RAM Error Injection Data 0 */
#define SMU__SRAMCTL_2_RAMEID1 SMU__SRAMCTL_2.RAMEID1.R    /* RAM Error Injection Data 1 */
#define SMU__SRAMCTL_2_RAMEIDC SMU__SRAMCTL_2.RAMEIDC.R    /* RAM Error Injection Data Control */
#define SMU__SRAMCTL_2_RAMEIA SMU__SRAMCTL_2.RAMEIA.R      /* RAM Error Injection Base Address */
#define SMU__SRAMCTL_2_RAMEIAM SMU__SRAMCTL_2.RAMEIAM.R    /* RAM Error Injection Address Mask */
#define SMU__SRAMCTL_2_RAMMAXA SMU__SRAMCTL_2.RAMMAXA.R    /* RAM Maximum-Value Address */
#define SMU__SRAMCTL_2_RAMCR2 SMU__SRAMCTL_2.RAMCR2.R      /* RAM Control 2 */

#define SMU__SRAMCTL_3_RAMCR SMU__SRAMCTL_3.RAMCR.R        /* RAM Control */
#define SMU__SRAMCTL_3_RAMIAS SMU__SRAMCTL_3.RAMIAS.R      /* RAM Initialization Address Start */
#define SMU__SRAMCTL_3_RAMIAE SMU__SRAMCTL_3.RAMIAE.R      /* RAM Initialization Address End */
#define SMU__SRAMCTL_3_RAMSR SMU__SRAMCTL_3.RAMSR.R        /* RAM Status */
#define SMU__SRAMCTL_3_RAMMEMA SMU__SRAMCTL_3.RAMMEMA.R    /* RAM ECC Address */
#define SMU__SRAMCTL_3_RAMSYSA SMU__SRAMCTL_3.RAMSYSA.R    /* RAM System Address */
#define SMU__SRAMCTL_3_RAMECCNT SMU__SRAMCTL_3.RAMECCNT.R  /* RAM Correctable Error Count */
#define SMU__SRAMCTL_3_RAMEID0 SMU__SRAMCTL_3.RAMEID0.R    /* RAM Error Injection Data 0 */
#define SMU__SRAMCTL_3_RAMEID1 SMU__SRAMCTL_3.RAMEID1.R    /* RAM Error Injection Data 1 */
#define SMU__SRAMCTL_3_RAMEIDC SMU__SRAMCTL_3.RAMEIDC.R    /* RAM Error Injection Data Control */
#define SMU__SRAMCTL_3_RAMEIA SMU__SRAMCTL_3.RAMEIA.R      /* RAM Error Injection Base Address */
#define SMU__SRAMCTL_3_RAMEIAM SMU__SRAMCTL_3.RAMEIAM.R    /* RAM Error Injection Address Mask */
#define SMU__SRAMCTL_3_RAMMAXA SMU__SRAMCTL_3.RAMMAXA.R    /* RAM Maximum-Value Address */
#define SMU__SRAMCTL_3_RAMCR2 SMU__SRAMCTL_3.RAMCR2.R      /* RAM Control 2 */

#define RTU0__SRAMCTL_D0_RAMCR RTU0__SRAMCTL_D0.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_D0_RAMIAS RTU0__SRAMCTL_D0.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_D0_RAMIAE RTU0__SRAMCTL_D0.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_D0_RAMSR RTU0__SRAMCTL_D0.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_D0_RAMMEMA RTU0__SRAMCTL_D0.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_D0_RAMSYSA RTU0__SRAMCTL_D0.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_D0_RAMECCNT RTU0__SRAMCTL_D0.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_D0_RAMEID0 RTU0__SRAMCTL_D0.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_D0_RAMEID1 RTU0__SRAMCTL_D0.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_D0_RAMEIDC RTU0__SRAMCTL_D0.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_D0_RAMEIA RTU0__SRAMCTL_D0.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_D0_RAMEIAM RTU0__SRAMCTL_D0.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_D0_RAMMAXA RTU0__SRAMCTL_D0.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_D0_RAMCR2 RTU0__SRAMCTL_D0.RAMCR2.R  /* RAM Control 2 */

#define RTU1__SRAMCTL_D0_RAMCR RTU1__SRAMCTL_D0.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_D0_RAMIAS RTU1__SRAMCTL_D0.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_D0_RAMIAE RTU1__SRAMCTL_D0.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_D0_RAMSR RTU1__SRAMCTL_D0.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_D0_RAMMEMA RTU1__SRAMCTL_D0.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_D0_RAMSYSA RTU1__SRAMCTL_D0.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_D0_RAMECCNT RTU1__SRAMCTL_D0.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_D0_RAMEID0 RTU1__SRAMCTL_D0.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_D0_RAMEID1 RTU1__SRAMCTL_D0.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_D0_RAMEIDC RTU1__SRAMCTL_D0.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_D0_RAMEIA RTU1__SRAMCTL_D0.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_D0_RAMEIAM RTU1__SRAMCTL_D0.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_D0_RAMMAXA RTU1__SRAMCTL_D0.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_D0_RAMCR2 RTU1__SRAMCTL_D0.RAMCR2.R  /* RAM Control 2 */

#define RTU0__SRAMCTL_D1_RAMCR RTU0__SRAMCTL_D1.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_D1_RAMIAS RTU0__SRAMCTL_D1.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_D1_RAMIAE RTU0__SRAMCTL_D1.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_D1_RAMSR RTU0__SRAMCTL_D1.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_D1_RAMMEMA RTU0__SRAMCTL_D1.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_D1_RAMSYSA RTU0__SRAMCTL_D1.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_D1_RAMECCNT RTU0__SRAMCTL_D1.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_D1_RAMEID0 RTU0__SRAMCTL_D1.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_D1_RAMEID1 RTU0__SRAMCTL_D1.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_D1_RAMEIDC RTU0__SRAMCTL_D1.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_D1_RAMEIA RTU0__SRAMCTL_D1.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_D1_RAMEIAM RTU0__SRAMCTL_D1.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_D1_RAMMAXA RTU0__SRAMCTL_D1.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_D1_RAMCR2 RTU0__SRAMCTL_D1.RAMCR2.R  /* RAM Control 2 */

#define RTU1__SRAMCTL_D1_RAMCR RTU1__SRAMCTL_D1.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_D1_RAMIAS RTU1__SRAMCTL_D1.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_D1_RAMIAE RTU1__SRAMCTL_D1.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_D1_RAMSR RTU1__SRAMCTL_D1.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_D1_RAMMEMA RTU1__SRAMCTL_D1.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_D1_RAMSYSA RTU1__SRAMCTL_D1.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_D1_RAMECCNT RTU1__SRAMCTL_D1.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_D1_RAMEID0 RTU1__SRAMCTL_D1.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_D1_RAMEID1 RTU1__SRAMCTL_D1.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_D1_RAMEIDC RTU1__SRAMCTL_D1.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_D1_RAMEIA RTU1__SRAMCTL_D1.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_D1_RAMEIAM RTU1__SRAMCTL_D1.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_D1_RAMMAXA RTU1__SRAMCTL_D1.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_D1_RAMCR2 RTU1__SRAMCTL_D1.RAMCR2.R  /* RAM Control 2 */

#define RTU0__SRAMCTL_D2_RAMCR RTU0__SRAMCTL_D2.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_D2_RAMIAS RTU0__SRAMCTL_D2.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_D2_RAMIAE RTU0__SRAMCTL_D2.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_D2_RAMSR RTU0__SRAMCTL_D2.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_D2_RAMMEMA RTU0__SRAMCTL_D2.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_D2_RAMSYSA RTU0__SRAMCTL_D2.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_D2_RAMECCNT RTU0__SRAMCTL_D2.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_D2_RAMEID0 RTU0__SRAMCTL_D2.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_D2_RAMEID1 RTU0__SRAMCTL_D2.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_D2_RAMEIDC RTU0__SRAMCTL_D2.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_D2_RAMEIA RTU0__SRAMCTL_D2.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_D2_RAMEIAM RTU0__SRAMCTL_D2.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_D2_RAMMAXA RTU0__SRAMCTL_D2.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_D2_RAMCR2 RTU0__SRAMCTL_D2.RAMCR2.R  /* RAM Control 2 */
#define RTU0__SRAMCTL_D2_RAMPC RTU0__SRAMCTL_D2.RAMPC.R    /* RAM Power Control */

#define RTU1__SRAMCTL_D2_RAMCR RTU1__SRAMCTL_D2.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_D2_RAMIAS RTU1__SRAMCTL_D2.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_D2_RAMIAE RTU1__SRAMCTL_D2.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_D2_RAMSR RTU1__SRAMCTL_D2.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_D2_RAMMEMA RTU1__SRAMCTL_D2.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_D2_RAMSYSA RTU1__SRAMCTL_D2.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_D2_RAMECCNT RTU1__SRAMCTL_D2.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_D2_RAMEID0 RTU1__SRAMCTL_D2.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_D2_RAMEID1 RTU1__SRAMCTL_D2.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_D2_RAMEIDC RTU1__SRAMCTL_D2.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_D2_RAMEIA RTU1__SRAMCTL_D2.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_D2_RAMEIAM RTU1__SRAMCTL_D2.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_D2_RAMMAXA RTU1__SRAMCTL_D2.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_D2_RAMCR2 RTU1__SRAMCTL_D2.RAMCR2.R  /* RAM Control 2 */
#define RTU1__SRAMCTL_D2_RAMPC RTU1__SRAMCTL_D2.RAMPC.R    /* RAM Power Control */

#define RTU0__SRAMCTL_C0_RAMCR RTU0__SRAMCTL_C0.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_C0_RAMIAS RTU0__SRAMCTL_C0.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_C0_RAMIAE RTU0__SRAMCTL_C0.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_C0_RAMSR RTU0__SRAMCTL_C0.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_C0_RAMMEMA RTU0__SRAMCTL_C0.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_C0_RAMSYSA RTU0__SRAMCTL_C0.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_C0_RAMECCNT RTU0__SRAMCTL_C0.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_C0_RAMEID0 RTU0__SRAMCTL_C0.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_C0_RAMEID1 RTU0__SRAMCTL_C0.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_C0_RAMEIDC RTU0__SRAMCTL_C0.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_C0_RAMEIA RTU0__SRAMCTL_C0.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_C0_RAMEIAM RTU0__SRAMCTL_C0.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_C0_RAMMAXA RTU0__SRAMCTL_C0.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_C0_RAMCR2 RTU0__SRAMCTL_C0.RAMCR2.R  /* RAM Control 2 */
#define RTU0__SRAMCTL_C0_RAMPC RTU0__SRAMCTL_C0.RAMPC.R    /* RAM Power Control */

#define RTU1__SRAMCTL_C0_RAMCR RTU1__SRAMCTL_C0.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_C0_RAMIAS RTU1__SRAMCTL_C0.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_C0_RAMIAE RTU1__SRAMCTL_C0.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_C0_RAMSR RTU1__SRAMCTL_C0.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_C0_RAMMEMA RTU1__SRAMCTL_C0.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_C0_RAMSYSA RTU1__SRAMCTL_C0.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_C0_RAMECCNT RTU1__SRAMCTL_C0.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_C0_RAMEID0 RTU1__SRAMCTL_C0.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_C0_RAMEID1 RTU1__SRAMCTL_C0.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_C0_RAMEIDC RTU1__SRAMCTL_C0.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_C0_RAMEIA RTU1__SRAMCTL_C0.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_C0_RAMEIAM RTU1__SRAMCTL_C0.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_C0_RAMMAXA RTU1__SRAMCTL_C0.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_C0_RAMCR2 RTU1__SRAMCTL_C0.RAMCR2.R  /* RAM Control 2 */
#define RTU1__SRAMCTL_C0_RAMPC RTU1__SRAMCTL_C0.RAMPC.R    /* RAM Power Control */

#define RTU0__SRAMCTL_C1_RAMCR RTU0__SRAMCTL_C1.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_C1_RAMIAS RTU0__SRAMCTL_C1.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_C1_RAMIAE RTU0__SRAMCTL_C1.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_C1_RAMSR RTU0__SRAMCTL_C1.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_C1_RAMMEMA RTU0__SRAMCTL_C1.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_C1_RAMSYSA RTU0__SRAMCTL_C1.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_C1_RAMECCNT RTU0__SRAMCTL_C1.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_C1_RAMEID0 RTU0__SRAMCTL_C1.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_C1_RAMEID1 RTU0__SRAMCTL_C1.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_C1_RAMEIDC RTU0__SRAMCTL_C1.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_C1_RAMEIA RTU0__SRAMCTL_C1.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_C1_RAMEIAM RTU0__SRAMCTL_C1.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_C1_RAMMAXA RTU0__SRAMCTL_C1.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_C1_RAMCR2 RTU0__SRAMCTL_C1.RAMCR2.R  /* RAM Control 2 */
#define RTU0__SRAMCTL_C1_RAMPC RTU0__SRAMCTL_C1.RAMPC.R    /* RAM Power Control */

#define RTU1__SRAMCTL_C1_RAMCR RTU1__SRAMCTL_C1.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_C1_RAMIAS RTU1__SRAMCTL_C1.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_C1_RAMIAE RTU1__SRAMCTL_C1.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_C1_RAMSR RTU1__SRAMCTL_C1.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_C1_RAMMEMA RTU1__SRAMCTL_C1.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_C1_RAMSYSA RTU1__SRAMCTL_C1.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_C1_RAMECCNT RTU1__SRAMCTL_C1.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_C1_RAMEID0 RTU1__SRAMCTL_C1.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_C1_RAMEID1 RTU1__SRAMCTL_C1.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_C1_RAMEIDC RTU1__SRAMCTL_C1.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_C1_RAMEIA RTU1__SRAMCTL_C1.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_C1_RAMEIAM RTU1__SRAMCTL_C1.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_C1_RAMMAXA RTU1__SRAMCTL_C1.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_C1_RAMCR2 RTU1__SRAMCTL_C1.RAMCR2.R  /* RAM Control 2 */
#define RTU1__SRAMCTL_C1_RAMPC RTU1__SRAMCTL_C1.RAMPC.R    /* RAM Power Control */

#define RTU0__SRAMCTL_C2_RAMCR RTU0__SRAMCTL_C2.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_C2_RAMIAS RTU0__SRAMCTL_C2.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_C2_RAMIAE RTU0__SRAMCTL_C2.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_C2_RAMSR RTU0__SRAMCTL_C2.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_C2_RAMMEMA RTU0__SRAMCTL_C2.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_C2_RAMSYSA RTU0__SRAMCTL_C2.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_C2_RAMECCNT RTU0__SRAMCTL_C2.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_C2_RAMEID0 RTU0__SRAMCTL_C2.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_C2_RAMEID1 RTU0__SRAMCTL_C2.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_C2_RAMEIDC RTU0__SRAMCTL_C2.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_C2_RAMEIA RTU0__SRAMCTL_C2.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_C2_RAMEIAM RTU0__SRAMCTL_C2.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_C2_RAMMAXA RTU0__SRAMCTL_C2.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_C2_RAMCR2 RTU0__SRAMCTL_C2.RAMCR2.R  /* RAM Control 2 */
#define RTU0__SRAMCTL_C2_RAMPC RTU0__SRAMCTL_C2.RAMPC.R    /* RAM Power Control */

#define RTU1__SRAMCTL_C2_RAMCR RTU1__SRAMCTL_C2.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_C2_RAMIAS RTU1__SRAMCTL_C2.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_C2_RAMIAE RTU1__SRAMCTL_C2.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_C2_RAMSR RTU1__SRAMCTL_C2.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_C2_RAMMEMA RTU1__SRAMCTL_C2.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_C2_RAMSYSA RTU1__SRAMCTL_C2.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_C2_RAMECCNT RTU1__SRAMCTL_C2.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_C2_RAMEID0 RTU1__SRAMCTL_C2.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_C2_RAMEID1 RTU1__SRAMCTL_C2.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_C2_RAMEIDC RTU1__SRAMCTL_C2.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_C2_RAMEIA RTU1__SRAMCTL_C2.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_C2_RAMEIAM RTU1__SRAMCTL_C2.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_C2_RAMMAXA RTU1__SRAMCTL_C2.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_C2_RAMCR2 RTU1__SRAMCTL_C2.RAMCR2.R  /* RAM Control 2 */
#define RTU1__SRAMCTL_C2_RAMPC RTU1__SRAMCTL_C2.RAMPC.R    /* RAM Power Control */

#define RTU0__SRAMCTL_C3_RAMCR RTU0__SRAMCTL_C3.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_C3_RAMIAS RTU0__SRAMCTL_C3.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_C3_RAMIAE RTU0__SRAMCTL_C3.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_C3_RAMSR RTU0__SRAMCTL_C3.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_C3_RAMMEMA RTU0__SRAMCTL_C3.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_C3_RAMSYSA RTU0__SRAMCTL_C3.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_C3_RAMECCNT RTU0__SRAMCTL_C3.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_C3_RAMEID0 RTU0__SRAMCTL_C3.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_C3_RAMEID1 RTU0__SRAMCTL_C3.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_C3_RAMEIDC RTU0__SRAMCTL_C3.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_C3_RAMEIA RTU0__SRAMCTL_C3.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_C3_RAMEIAM RTU0__SRAMCTL_C3.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_C3_RAMMAXA RTU0__SRAMCTL_C3.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_C3_RAMCR2 RTU0__SRAMCTL_C3.RAMCR2.R  /* RAM Control 2 */
#define RTU0__SRAMCTL_C3_RAMPC RTU0__SRAMCTL_C3.RAMPC.R    /* RAM Power Control */

#define RTU1__SRAMCTL_C3_RAMCR RTU1__SRAMCTL_C3.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_C3_RAMIAS RTU1__SRAMCTL_C3.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_C3_RAMIAE RTU1__SRAMCTL_C3.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_C3_RAMSR RTU1__SRAMCTL_C3.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_C3_RAMMEMA RTU1__SRAMCTL_C3.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_C3_RAMSYSA RTU1__SRAMCTL_C3.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_C3_RAMECCNT RTU1__SRAMCTL_C3.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_C3_RAMEID0 RTU1__SRAMCTL_C3.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_C3_RAMEID1 RTU1__SRAMCTL_C3.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_C3_RAMEIDC RTU1__SRAMCTL_C3.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_C3_RAMEIA RTU1__SRAMCTL_C3.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_C3_RAMEIAM RTU1__SRAMCTL_C3.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_C3_RAMMAXA RTU1__SRAMCTL_C3.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_C3_RAMCR2 RTU1__SRAMCTL_C3.RAMCR2.R  /* RAM Control 2 */
#define RTU1__SRAMCTL_C3_RAMPC RTU1__SRAMCTL_C3.RAMPC.R    /* RAM Power Control */

#define RTU0__SRAMCTL_C4_RAMCR RTU0__SRAMCTL_C4.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_C4_RAMIAS RTU0__SRAMCTL_C4.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_C4_RAMIAE RTU0__SRAMCTL_C4.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_C4_RAMSR RTU0__SRAMCTL_C4.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_C4_RAMMEMA RTU0__SRAMCTL_C4.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_C4_RAMSYSA RTU0__SRAMCTL_C4.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_C4_RAMECCNT RTU0__SRAMCTL_C4.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_C4_RAMEID0 RTU0__SRAMCTL_C4.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_C4_RAMEID1 RTU0__SRAMCTL_C4.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_C4_RAMEIDC RTU0__SRAMCTL_C4.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_C4_RAMEIA RTU0__SRAMCTL_C4.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_C4_RAMEIAM RTU0__SRAMCTL_C4.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_C4_RAMMAXA RTU0__SRAMCTL_C4.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_C4_RAMCR2 RTU0__SRAMCTL_C4.RAMCR2.R  /* RAM Control 2 */
#define RTU0__SRAMCTL_C4_RAMPC RTU0__SRAMCTL_C4.RAMPC.R    /* RAM Power Control */

#define RTU1__SRAMCTL_C4_RAMCR RTU1__SRAMCTL_C4.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_C4_RAMIAS RTU1__SRAMCTL_C4.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_C4_RAMIAE RTU1__SRAMCTL_C4.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_C4_RAMSR RTU1__SRAMCTL_C4.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_C4_RAMMEMA RTU1__SRAMCTL_C4.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_C4_RAMSYSA RTU1__SRAMCTL_C4.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_C4_RAMECCNT RTU1__SRAMCTL_C4.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_C4_RAMEID0 RTU1__SRAMCTL_C4.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_C4_RAMEID1 RTU1__SRAMCTL_C4.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_C4_RAMEIDC RTU1__SRAMCTL_C4.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_C4_RAMEIA RTU1__SRAMCTL_C4.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_C4_RAMEIAM RTU1__SRAMCTL_C4.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_C4_RAMMAXA RTU1__SRAMCTL_C4.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_C4_RAMCR2 RTU1__SRAMCTL_C4.RAMCR2.R  /* RAM Control 2 */
#define RTU1__SRAMCTL_C4_RAMPC RTU1__SRAMCTL_C4.RAMPC.R    /* RAM Power Control */

#define RTU0__SRAMCTL_C5_RAMCR RTU0__SRAMCTL_C5.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_C5_RAMIAS RTU0__SRAMCTL_C5.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_C5_RAMIAE RTU0__SRAMCTL_C5.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_C5_RAMSR RTU0__SRAMCTL_C5.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_C5_RAMMEMA RTU0__SRAMCTL_C5.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_C5_RAMSYSA RTU0__SRAMCTL_C5.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_C5_RAMECCNT RTU0__SRAMCTL_C5.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_C5_RAMEID0 RTU0__SRAMCTL_C5.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_C5_RAMEID1 RTU0__SRAMCTL_C5.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_C5_RAMEIDC RTU0__SRAMCTL_C5.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_C5_RAMEIA RTU0__SRAMCTL_C5.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_C5_RAMEIAM RTU0__SRAMCTL_C5.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_C5_RAMMAXA RTU0__SRAMCTL_C5.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_C5_RAMCR2 RTU0__SRAMCTL_C5.RAMCR2.R  /* RAM Control 2 */
#define RTU0__SRAMCTL_C5_RAMPC RTU0__SRAMCTL_C5.RAMPC.R    /* RAM Power Control */

#define RTU1__SRAMCTL_C5_RAMCR RTU1__SRAMCTL_C5.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_C5_RAMIAS RTU1__SRAMCTL_C5.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_C5_RAMIAE RTU1__SRAMCTL_C5.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_C5_RAMSR RTU1__SRAMCTL_C5.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_C5_RAMMEMA RTU1__SRAMCTL_C5.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_C5_RAMSYSA RTU1__SRAMCTL_C5.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_C5_RAMECCNT RTU1__SRAMCTL_C5.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_C5_RAMEID0 RTU1__SRAMCTL_C5.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_C5_RAMEID1 RTU1__SRAMCTL_C5.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_C5_RAMEIDC RTU1__SRAMCTL_C5.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_C5_RAMEIA RTU1__SRAMCTL_C5.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_C5_RAMEIAM RTU1__SRAMCTL_C5.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_C5_RAMMAXA RTU1__SRAMCTL_C5.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_C5_RAMCR2 RTU1__SRAMCTL_C5.RAMCR2.R  /* RAM Control 2 */
#define RTU1__SRAMCTL_C5_RAMPC RTU1__SRAMCTL_C5.RAMPC.R    /* RAM Power Control */

#define RTU0__SRAMCTL_C6_RAMCR RTU0__SRAMCTL_C6.RAMCR.R    /* RAM Control */
#define RTU0__SRAMCTL_C6_RAMIAS RTU0__SRAMCTL_C6.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU0__SRAMCTL_C6_RAMIAE RTU0__SRAMCTL_C6.RAMIAE.R  /* RAM Initialization Address End */
#define RTU0__SRAMCTL_C6_RAMSR RTU0__SRAMCTL_C6.RAMSR.R    /* RAM Status */
#define RTU0__SRAMCTL_C6_RAMMEMA RTU0__SRAMCTL_C6.RAMMEMA.R /* RAM ECC Address */
#define RTU0__SRAMCTL_C6_RAMSYSA RTU0__SRAMCTL_C6.RAMSYSA.R /* RAM System Address */
#define RTU0__SRAMCTL_C6_RAMECCNT RTU0__SRAMCTL_C6.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU0__SRAMCTL_C6_RAMEID0 RTU0__SRAMCTL_C6.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU0__SRAMCTL_C6_RAMEID1 RTU0__SRAMCTL_C6.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU0__SRAMCTL_C6_RAMEIDC RTU0__SRAMCTL_C6.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU0__SRAMCTL_C6_RAMEIA RTU0__SRAMCTL_C6.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU0__SRAMCTL_C6_RAMEIAM RTU0__SRAMCTL_C6.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU0__SRAMCTL_C6_RAMMAXA RTU0__SRAMCTL_C6.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU0__SRAMCTL_C6_RAMCR2 RTU0__SRAMCTL_C6.RAMCR2.R  /* RAM Control 2 */
#define RTU0__SRAMCTL_C6_RAMPC RTU0__SRAMCTL_C6.RAMPC.R    /* RAM Power Control */

#define RTU1__SRAMCTL_C6_RAMCR RTU1__SRAMCTL_C6.RAMCR.R    /* RAM Control */
#define RTU1__SRAMCTL_C6_RAMIAS RTU1__SRAMCTL_C6.RAMIAS.R  /* RAM Initialization Address Start */
#define RTU1__SRAMCTL_C6_RAMIAE RTU1__SRAMCTL_C6.RAMIAE.R  /* RAM Initialization Address End */
#define RTU1__SRAMCTL_C6_RAMSR RTU1__SRAMCTL_C6.RAMSR.R    /* RAM Status */
#define RTU1__SRAMCTL_C6_RAMMEMA RTU1__SRAMCTL_C6.RAMMEMA.R /* RAM ECC Address */
#define RTU1__SRAMCTL_C6_RAMSYSA RTU1__SRAMCTL_C6.RAMSYSA.R /* RAM System Address */
#define RTU1__SRAMCTL_C6_RAMECCNT RTU1__SRAMCTL_C6.RAMECCNT.R /* RAM Correctable Error Count */
#define RTU1__SRAMCTL_C6_RAMEID0 RTU1__SRAMCTL_C6.RAMEID0.R /* RAM Error Injection Data 0 */
#define RTU1__SRAMCTL_C6_RAMEID1 RTU1__SRAMCTL_C6.RAMEID1.R /* RAM Error Injection Data 1 */
#define RTU1__SRAMCTL_C6_RAMEIDC RTU1__SRAMCTL_C6.RAMEIDC.R /* RAM Error Injection Data Control */
#define RTU1__SRAMCTL_C6_RAMEIA RTU1__SRAMCTL_C6.RAMEIA.R  /* RAM Error Injection Base Address */
#define RTU1__SRAMCTL_C6_RAMEIAM RTU1__SRAMCTL_C6.RAMEIAM.R /* RAM Error Injection Address Mask */
#define RTU1__SRAMCTL_C6_RAMMAXA RTU1__SRAMCTL_C6.RAMMAXA.R /* RAM Maximum-Value Address */
#define RTU1__SRAMCTL_C6_RAMCR2 RTU1__SRAMCTL_C6.RAMCR2.R  /* RAM Control 2 */
#define RTU1__SRAMCTL_C6_RAMPC RTU1__SRAMCTL_C6.RAMPC.R    /* RAM Power Control */

/* SRX */
#define SRX_0_GBL_CTRL       SRX_0.GBL_CTRL.R              /* Global Control Register */
#define SRX_0_CHNL_EN        SRX_0.CHNL_EN.R               /* Channel Enable Register */
#define SRX_0_SLOW_MSG_UVF   SRX_0.SLOW_MSG_UVF.R          /* Slow Channel Status */
#define SRX_0_FMSG_RDY       SRX_0.FMSG_RDY.R              /* Fast Message Ready Status Register */
#define SRX_0_SMSG_RDY       SRX_0.SMSG_RDY.R              /* Slow Serial Message Ready Status Register */
#define SRX_0_CHNL_FIFO_EN   SRX_0.CHNL_FIFO_EN.R          /* Channel FIFO Enable */
#define SRX_0_DATA_CTRL1     SRX_0.DATA_CTRL1.R            /* Data Control Register 1 */
#define SRX_0_CHNL_OVF_UVF   SRX_0.CHNL_OVF_UVF.R          /* Fast Channel Status */
#define SRX_0_FDMA_CTRL      SRX_0.FDMA_CTRL.R             /* Fast Message DMA Control Register */
#define SRX_0_SDMA_CTRL      SRX_0.SDMA_CTRL.R             /* Slow Serial Message DMA Control Register */
#define SRX_0_CHNL_NB_REORD1 SRX_0.CHNL_NB_REORD1.R        /* Channel Nibble Reorder 1 */
#define SRX_0_FMSG_RDY_IE    SRX_0.FMSG_RDY_IE.R           /* Fast Message Ready Interrupt Control Register */
#define SRX_0_SMSG_RDY_IE    SRX_0.SMSG_RDY_IE.R           /* Slow Serial Message Ready Interrupt Enable Register */
#define SRX_0_CH0_CLK_CTRL   SRX_0.CNL[0].CH_CLK_CTRL.R    /* Channel '0' Clock Control Register */
#define SRX_0_CH0_STATUS     SRX_0.CNL[0].CH_STATUS.R      /* Channel '0' Status Register */
#define SRX_0_CH0_CONFIG     SRX_0.CNL[0].CH_CONFIG.R      /* Channel '0' Configuration Register */
#define SRX_0_CH0_FMSG_DATA  SRX_0.CNL[0].CH_FMSG_DATA.R   /* Channel '0' Fast Message Data Read Register */
#define SRX_0_CH0_FMSG_CRC   SRX_0.CNL[0].CH_FMSG_CRC.R    /* Channel '0' Fast Message CRC Read Register */
#define SRX_0_CH0_FMSG_TS    SRX_0.CNL[0].CH_FMSG_TS.R     /* Channel '0' Fast Message Time Stamp Read Register */
#define SRX_0_CH0_SMSG_BIT3  SRX_0.CNL[0].CH_SMSG_BIT3.R   /* Channel '0' Serial Message Read Register (Bit 3) */
#define SRX_0_CH0_SMSG_BIT2  SRX_0.CNL[0].CH_SMSG_BIT2.R   /* Channel '0' Serial Message Read Register (Bit 2) */
#define SRX_0_CH0_SMSG_TS    SRX_0.CNL[0].CH_SMSG_TS.R     /* Channel '0' Serial Message Time Stamp Read */
#define SRX_0_CH0_F_DMA_RD   SRX_0.CNL[0].CH_F_DMA_RD.R    /* Channel '0' DMA Fast Message Data Read */
#define SRX_0_CH0_F_DMA_CRC  SRX_0.CNL[0].CH_F_DMA_CRC.R   /* Channel '0' DMA Fast Message CRC Read */
#define SRX_0_CH0_F_DMA_TS   SRX_0.CNL[0].CH_F_DMA_TS.R    /* Channel '0' DMA Fast Message Time Stamp Read */
#define SRX_0_CH0_S_DMA_RD   SRX_0.CNL[0].CH_S_DMA_RD.R    /* Channel '0' DMA Slow Serial Message Bit3 Read */
#define SRX_0_CH0_S_DMA_CRC  SRX_0.CNL[0].CH_S_DMA_CRC.R   /* Channel '0' DMA Slow Serial Message Bit2 Read */
#define SRX_0_CH0_S_DMA_TS   SRX_0.CNL[0].CH_S_DMA_TS.R    /* Channel '0' DMA Slow Serial Message Time Stamp Read */
#define SRX_0_CHNL0_TIMESTAMP SRX_0.CNL[0].CHNL_TIMESTAMP.R /* Channel '0' Time Stamp */
#define SRX_0_CHNL0_COUNTER  SRX_0.CNL[0].CHNL_COUNTER.R   /* Channel '0' Counter */
#define SRX_0_CH1_CLK_CTRL   SRX_0.CNL[1].CH_CLK_CTRL.R    /* Channel '1' Clock Control Register */
#define SRX_0_CH1_STATUS     SRX_0.CNL[1].CH_STATUS.R      /* Channel '1' Status Register */
#define SRX_0_CH1_CONFIG     SRX_0.CNL[1].CH_CONFIG.R      /* Channel '1' Configuration Register */
#define SRX_0_CH1_FMSG_DATA  SRX_0.CNL[1].CH_FMSG_DATA.R   /* Channel '1' Fast Message Data Read Register */
#define SRX_0_CH1_FMSG_CRC   SRX_0.CNL[1].CH_FMSG_CRC.R    /* Channel '1' Fast Message CRC Read Register */
#define SRX_0_CH1_FMSG_TS    SRX_0.CNL[1].CH_FMSG_TS.R     /* Channel '1' Fast Message Time Stamp Read Register */
#define SRX_0_CH1_SMSG_BIT3  SRX_0.CNL[1].CH_SMSG_BIT3.R   /* Channel '1' Serial Message Read Register (Bit 3) */
#define SRX_0_CH1_SMSG_BIT2  SRX_0.CNL[1].CH_SMSG_BIT2.R   /* Channel '1' Serial Message Read Register (Bit 2) */
#define SRX_0_CH1_SMSG_TS    SRX_0.CNL[1].CH_SMSG_TS.R     /* Channel '1' Serial Message Time Stamp Read */
#define SRX_0_CH1_F_DMA_RD   SRX_0.CNL[1].CH_F_DMA_RD.R    /* Channel '1' DMA Fast Message Data Read */
#define SRX_0_CH1_F_DMA_CRC  SRX_0.CNL[1].CH_F_DMA_CRC.R   /* Channel '1' DMA Fast Message CRC Read */
#define SRX_0_CH1_F_DMA_TS   SRX_0.CNL[1].CH_F_DMA_TS.R    /* Channel '1' DMA Fast Message Time Stamp Read */
#define SRX_0_CH1_S_DMA_RD   SRX_0.CNL[1].CH_S_DMA_RD.R    /* Channel '1' DMA Slow Serial Message Bit3 Read */
#define SRX_0_CH1_S_DMA_CRC  SRX_0.CNL[1].CH_S_DMA_CRC.R   /* Channel '1' DMA Slow Serial Message Bit2 Read */
#define SRX_0_CH1_S_DMA_TS   SRX_0.CNL[1].CH_S_DMA_TS.R    /* Channel '1' DMA Slow Serial Message Time Stamp Read */
#define SRX_0_CHNL1_TIMESTAMP SRX_0.CNL[1].CHNL_TIMESTAMP.R /* Channel '1' Time Stamp */
#define SRX_0_CHNL1_COUNTER  SRX_0.CNL[1].CHNL_COUNTER.R   /* Channel '1' Counter */
#define SRX_0_CH2_CLK_CTRL   SRX_0.CNL[2].CH_CLK_CTRL.R    /* Channel '2' Clock Control Register */
#define SRX_0_CH2_STATUS     SRX_0.CNL[2].CH_STATUS.R      /* Channel '2' Status Register */
#define SRX_0_CH2_CONFIG     SRX_0.CNL[2].CH_CONFIG.R      /* Channel '2' Configuration Register */
#define SRX_0_CH2_FMSG_DATA  SRX_0.CNL[2].CH_FMSG_DATA.R   /* Channel '2' Fast Message Data Read Register */
#define SRX_0_CH2_FMSG_CRC   SRX_0.CNL[2].CH_FMSG_CRC.R    /* Channel '2' Fast Message CRC Read Register */
#define SRX_0_CH2_FMSG_TS    SRX_0.CNL[2].CH_FMSG_TS.R     /* Channel '2' Fast Message Time Stamp Read Register */
#define SRX_0_CH2_SMSG_BIT3  SRX_0.CNL[2].CH_SMSG_BIT3.R   /* Channel '2' Serial Message Read Register (Bit 3) */
#define SRX_0_CH2_SMSG_BIT2  SRX_0.CNL[2].CH_SMSG_BIT2.R   /* Channel '2' Serial Message Read Register (Bit 2) */
#define SRX_0_CH2_SMSG_TS    SRX_0.CNL[2].CH_SMSG_TS.R     /* Channel '2' Serial Message Time Stamp Read */
#define SRX_0_CH2_F_DMA_RD   SRX_0.CNL[2].CH_F_DMA_RD.R    /* Channel '2' DMA Fast Message Data Read */
#define SRX_0_CH2_F_DMA_CRC  SRX_0.CNL[2].CH_F_DMA_CRC.R   /* Channel '2' DMA Fast Message CRC Read */
#define SRX_0_CH2_F_DMA_TS   SRX_0.CNL[2].CH_F_DMA_TS.R    /* Channel '2' DMA Fast Message Time Stamp Read */
#define SRX_0_CH2_S_DMA_RD   SRX_0.CNL[2].CH_S_DMA_RD.R    /* Channel '2' DMA Slow Serial Message Bit3 Read */
#define SRX_0_CH2_S_DMA_CRC  SRX_0.CNL[2].CH_S_DMA_CRC.R   /* Channel '2' DMA Slow Serial Message Bit2 Read */
#define SRX_0_CH2_S_DMA_TS   SRX_0.CNL[2].CH_S_DMA_TS.R    /* Channel '2' DMA Slow Serial Message Time Stamp Read */
#define SRX_0_CHNL2_TIMESTAMP SRX_0.CNL[2].CHNL_TIMESTAMP.R /* Channel '2' Time Stamp */
#define SRX_0_CHNL2_COUNTER  SRX_0.CNL[2].CHNL_COUNTER.R   /* Channel '2' Counter */
#define SRX_0_CH3_CLK_CTRL   SRX_0.CNL[3].CH_CLK_CTRL.R    /* Channel '3' Clock Control Register */
#define SRX_0_CH3_STATUS     SRX_0.CNL[3].CH_STATUS.R      /* Channel '3' Status Register */
#define SRX_0_CH3_CONFIG     SRX_0.CNL[3].CH_CONFIG.R      /* Channel '3' Configuration Register */
#define SRX_0_CH3_FMSG_DATA  SRX_0.CNL[3].CH_FMSG_DATA.R   /* Channel '3' Fast Message Data Read Register */
#define SRX_0_CH3_FMSG_CRC   SRX_0.CNL[3].CH_FMSG_CRC.R    /* Channel '3' Fast Message CRC Read Register */
#define SRX_0_CH3_FMSG_TS    SRX_0.CNL[3].CH_FMSG_TS.R     /* Channel '3' Fast Message Time Stamp Read Register */
#define SRX_0_CH3_SMSG_BIT3  SRX_0.CNL[3].CH_SMSG_BIT3.R   /* Channel '3' Serial Message Read Register (Bit 3) */
#define SRX_0_CH3_SMSG_BIT2  SRX_0.CNL[3].CH_SMSG_BIT2.R   /* Channel '3' Serial Message Read Register (Bit 2) */
#define SRX_0_CH3_SMSG_TS    SRX_0.CNL[3].CH_SMSG_TS.R     /* Channel '3' Serial Message Time Stamp Read */
#define SRX_0_CH3_F_DMA_RD   SRX_0.CNL[3].CH_F_DMA_RD.R    /* Channel '3' DMA Fast Message Data Read */
#define SRX_0_CH3_F_DMA_CRC  SRX_0.CNL[3].CH_F_DMA_CRC.R   /* Channel '3' DMA Fast Message CRC Read */
#define SRX_0_CH3_F_DMA_TS   SRX_0.CNL[3].CH_F_DMA_TS.R    /* Channel '3' DMA Fast Message Time Stamp Read */
#define SRX_0_CH3_S_DMA_RD   SRX_0.CNL[3].CH_S_DMA_RD.R    /* Channel '3' DMA Slow Serial Message Bit3 Read */
#define SRX_0_CH3_S_DMA_CRC  SRX_0.CNL[3].CH_S_DMA_CRC.R   /* Channel '3' DMA Slow Serial Message Bit2 Read */
#define SRX_0_CH3_S_DMA_TS   SRX_0.CNL[3].CH_S_DMA_TS.R    /* Channel '3' DMA Slow Serial Message Time Stamp Read */
#define SRX_0_CHNL3_TIMESTAMP SRX_0.CNL[3].CHNL_TIMESTAMP.R /* Channel '3' Time Stamp */
#define SRX_0_CHNL3_COUNTER  SRX_0.CNL[3].CHNL_COUNTER.R   /* Channel '3' Counter */
#define SRX_0_CH4_CLK_CTRL   SRX_0.CNL[4].CH_CLK_CTRL.R    /* Channel '4' Clock Control Register */
#define SRX_0_CH4_STATUS     SRX_0.CNL[4].CH_STATUS.R      /* Channel '4' Status Register */
#define SRX_0_CH4_CONFIG     SRX_0.CNL[4].CH_CONFIG.R      /* Channel '4' Configuration Register */
#define SRX_0_CH4_FMSG_DATA  SRX_0.CNL[4].CH_FMSG_DATA.R   /* Channel '4' Fast Message Data Read Register */
#define SRX_0_CH4_FMSG_CRC   SRX_0.CNL[4].CH_FMSG_CRC.R    /* Channel '4' Fast Message CRC Read Register */
#define SRX_0_CH4_FMSG_TS    SRX_0.CNL[4].CH_FMSG_TS.R     /* Channel '4' Fast Message Time Stamp Read Register */
#define SRX_0_CH4_SMSG_BIT3  SRX_0.CNL[4].CH_SMSG_BIT3.R   /* Channel '4' Serial Message Read Register (Bit 3) */
#define SRX_0_CH4_SMSG_BIT2  SRX_0.CNL[4].CH_SMSG_BIT2.R   /* Channel '4' Serial Message Read Register (Bit 2) */
#define SRX_0_CH4_SMSG_TS    SRX_0.CNL[4].CH_SMSG_TS.R     /* Channel '4' Serial Message Time Stamp Read */
#define SRX_0_CH4_F_DMA_RD   SRX_0.CNL[4].CH_F_DMA_RD.R    /* Channel '4' DMA Fast Message Data Read */
#define SRX_0_CH4_F_DMA_CRC  SRX_0.CNL[4].CH_F_DMA_CRC.R   /* Channel '4' DMA Fast Message CRC Read */
#define SRX_0_CH4_F_DMA_TS   SRX_0.CNL[4].CH_F_DMA_TS.R    /* Channel '4' DMA Fast Message Time Stamp Read */
#define SRX_0_CH4_S_DMA_RD   SRX_0.CNL[4].CH_S_DMA_RD.R    /* Channel '4' DMA Slow Serial Message Bit3 Read */
#define SRX_0_CH4_S_DMA_CRC  SRX_0.CNL[4].CH_S_DMA_CRC.R   /* Channel '4' DMA Slow Serial Message Bit2 Read */
#define SRX_0_CH4_S_DMA_TS   SRX_0.CNL[4].CH_S_DMA_TS.R    /* Channel '4' DMA Slow Serial Message Time Stamp Read */
#define SRX_0_CHNL4_TIMESTAMP SRX_0.CNL[4].CHNL_TIMESTAMP.R /* Channel '4' Time Stamp */
#define SRX_0_CHNL4_COUNTER  SRX_0.CNL[4].CHNL_COUNTER.R   /* Channel '4' Counter */
#define SRX_0_CH5_CLK_CTRL   SRX_0.CNL[5].CH_CLK_CTRL.R    /* Channel '5' Clock Control Register */
#define SRX_0_CH5_STATUS     SRX_0.CNL[5].CH_STATUS.R      /* Channel '5' Status Register */
#define SRX_0_CH5_CONFIG     SRX_0.CNL[5].CH_CONFIG.R      /* Channel '5' Configuration Register */
#define SRX_0_CH5_FMSG_DATA  SRX_0.CNL[5].CH_FMSG_DATA.R   /* Channel '5' Fast Message Data Read Register */
#define SRX_0_CH5_FMSG_CRC   SRX_0.CNL[5].CH_FMSG_CRC.R    /* Channel '5' Fast Message CRC Read Register */
#define SRX_0_CH5_FMSG_TS    SRX_0.CNL[5].CH_FMSG_TS.R     /* Channel '5' Fast Message Time Stamp Read Register */
#define SRX_0_CH5_SMSG_BIT3  SRX_0.CNL[5].CH_SMSG_BIT3.R   /* Channel '5' Serial Message Read Register (Bit 3) */
#define SRX_0_CH5_SMSG_BIT2  SRX_0.CNL[5].CH_SMSG_BIT2.R   /* Channel '5' Serial Message Read Register (Bit 2) */
#define SRX_0_CH5_SMSG_TS    SRX_0.CNL[5].CH_SMSG_TS.R     /* Channel '5' Serial Message Time Stamp Read */
#define SRX_0_CH5_F_DMA_RD   SRX_0.CNL[5].CH_F_DMA_RD.R    /* Channel '5' DMA Fast Message Data Read */
#define SRX_0_CH5_F_DMA_CRC  SRX_0.CNL[5].CH_F_DMA_CRC.R   /* Channel '5' DMA Fast Message CRC Read */
#define SRX_0_CH5_F_DMA_TS   SRX_0.CNL[5].CH_F_DMA_TS.R    /* Channel '5' DMA Fast Message Time Stamp Read */
#define SRX_0_CH5_S_DMA_RD   SRX_0.CNL[5].CH_S_DMA_RD.R    /* Channel '5' DMA Slow Serial Message Bit3 Read */
#define SRX_0_CH5_S_DMA_CRC  SRX_0.CNL[5].CH_S_DMA_CRC.R   /* Channel '5' DMA Slow Serial Message Bit2 Read */
#define SRX_0_CH5_S_DMA_TS   SRX_0.CNL[5].CH_S_DMA_TS.R    /* Channel '5' DMA Slow Serial Message Time Stamp Read */
#define SRX_0_CHNL5_TIMESTAMP SRX_0.CNL[5].CHNL_TIMESTAMP.R /* Channel '5' Time Stamp */
#define SRX_0_CHNL5_COUNTER  SRX_0.CNL[5].CHNL_COUNTER.R   /* Channel '5' Counter */
#define SRX_0_CH6_CLK_CTRL   SRX_0.CNL[6].CH_CLK_CTRL.R    /* Channel '6' Clock Control Register */
#define SRX_0_CH6_STATUS     SRX_0.CNL[6].CH_STATUS.R      /* Channel '6' Status Register */
#define SRX_0_CH6_CONFIG     SRX_0.CNL[6].CH_CONFIG.R      /* Channel '6' Configuration Register */
#define SRX_0_CH6_FMSG_DATA  SRX_0.CNL[6].CH_FMSG_DATA.R   /* Channel '6' Fast Message Data Read Register */
#define SRX_0_CH6_FMSG_CRC   SRX_0.CNL[6].CH_FMSG_CRC.R    /* Channel '6' Fast Message CRC Read Register */
#define SRX_0_CH6_FMSG_TS    SRX_0.CNL[6].CH_FMSG_TS.R     /* Channel '6' Fast Message Time Stamp Read Register */
#define SRX_0_CH6_SMSG_BIT3  SRX_0.CNL[6].CH_SMSG_BIT3.R   /* Channel '6' Serial Message Read Register (Bit 3) */
#define SRX_0_CH6_SMSG_BIT2  SRX_0.CNL[6].CH_SMSG_BIT2.R   /* Channel '6' Serial Message Read Register (Bit 2) */
#define SRX_0_CH6_SMSG_TS    SRX_0.CNL[6].CH_SMSG_TS.R     /* Channel '6' Serial Message Time Stamp Read */
#define SRX_0_CH6_F_DMA_RD   SRX_0.CNL[6].CH_F_DMA_RD.R    /* Channel '6' DMA Fast Message Data Read */
#define SRX_0_CH6_F_DMA_CRC  SRX_0.CNL[6].CH_F_DMA_CRC.R   /* Channel '6' DMA Fast Message CRC Read */
#define SRX_0_CH6_F_DMA_TS   SRX_0.CNL[6].CH_F_DMA_TS.R    /* Channel '6' DMA Fast Message Time Stamp Read */
#define SRX_0_CH6_S_DMA_RD   SRX_0.CNL[6].CH_S_DMA_RD.R    /* Channel '6' DMA Slow Serial Message Bit3 Read */
#define SRX_0_CH6_S_DMA_CRC  SRX_0.CNL[6].CH_S_DMA_CRC.R   /* Channel '6' DMA Slow Serial Message Bit2 Read */
#define SRX_0_CH6_S_DMA_TS   SRX_0.CNL[6].CH_S_DMA_TS.R    /* Channel '6' DMA Slow Serial Message Time Stamp Read */
#define SRX_0_CHNL6_TIMESTAMP SRX_0.CNL[6].CHNL_TIMESTAMP.R /* Channel '6' Time Stamp */
#define SRX_0_CHNL6_COUNTER  SRX_0.CNL[6].CHNL_COUNTER.R   /* Channel '6' Counter */
#define SRX_0_CH7_CLK_CTRL   SRX_0.CNL[7].CH_CLK_CTRL.R    /* Channel '7' Clock Control Register */
#define SRX_0_CH7_STATUS     SRX_0.CNL[7].CH_STATUS.R      /* Channel '7' Status Register */
#define SRX_0_CH7_CONFIG     SRX_0.CNL[7].CH_CONFIG.R      /* Channel '7' Configuration Register */
#define SRX_0_CH7_FMSG_DATA  SRX_0.CNL[7].CH_FMSG_DATA.R   /* Channel '7' Fast Message Data Read Register */
#define SRX_0_CH7_FMSG_CRC   SRX_0.CNL[7].CH_FMSG_CRC.R    /* Channel '7' Fast Message CRC Read Register */
#define SRX_0_CH7_FMSG_TS    SRX_0.CNL[7].CH_FMSG_TS.R     /* Channel '7' Fast Message Time Stamp Read Register */
#define SRX_0_CH7_SMSG_BIT3  SRX_0.CNL[7].CH_SMSG_BIT3.R   /* Channel '7' Serial Message Read Register (Bit 3) */
#define SRX_0_CH7_SMSG_BIT2  SRX_0.CNL[7].CH_SMSG_BIT2.R   /* Channel '7' Serial Message Read Register (Bit 2) */
#define SRX_0_CH7_SMSG_TS    SRX_0.CNL[7].CH_SMSG_TS.R     /* Channel '7' Serial Message Time Stamp Read */
#define SRX_0_CH7_F_DMA_RD   SRX_0.CNL[7].CH_F_DMA_RD.R    /* Channel '7' DMA Fast Message Data Read */
#define SRX_0_CH7_F_DMA_CRC  SRX_0.CNL[7].CH_F_DMA_CRC.R   /* Channel '7' DMA Fast Message CRC Read */
#define SRX_0_CH7_F_DMA_TS   SRX_0.CNL[7].CH_F_DMA_TS.R    /* Channel '7' DMA Fast Message Time Stamp Read */
#define SRX_0_CH7_S_DMA_RD   SRX_0.CNL[7].CH_S_DMA_RD.R    /* Channel '7' DMA Slow Serial Message Bit3 Read */
#define SRX_0_CH7_S_DMA_CRC  SRX_0.CNL[7].CH_S_DMA_CRC.R   /* Channel '7' DMA Slow Serial Message Bit2 Read */
#define SRX_0_CH7_S_DMA_TS   SRX_0.CNL[7].CH_S_DMA_TS.R    /* Channel '7' DMA Slow Serial Message Time Stamp Read */
#define SRX_0_CHNL7_TIMESTAMP SRX_0.CNL[7].CHNL_TIMESTAMP.R /* Channel '7' Time Stamp */
#define SRX_0_CHNL7_COUNTER  SRX_0.CNL[7].CHNL_COUNTER.R   /* Channel '7' Counter */

#define SRX_1_GBL_CTRL       SRX_1.GBL_CTRL.R              /* Global Control Register */
#define SRX_1_CHNL_EN        SRX_1.CHNL_EN.R               /* Channel Enable Register */
#define SRX_1_SLOW_MSG_UVF   SRX_1.SLOW_MSG_UVF.R          /* Slow Channel Status */
#define SRX_1_FMSG_RDY       SRX_1.FMSG_RDY.R              /* Fast Message Ready Status Register */
#define SRX_1_SMSG_RDY       SRX_1.SMSG_RDY.R              /* Slow Serial Message Ready Status Register */
#define SRX_1_CHNL_FIFO_EN   SRX_1.CHNL_FIFO_EN.R          /* Channel FIFO Enable */
#define SRX_1_DATA_CTRL1     SRX_1.DATA_CTRL1.R            /* Data Control Register 1 */
#define SRX_1_CHNL_OVF_UVF   SRX_1.CHNL_OVF_UVF.R          /* Fast Channel Status */
#define SRX_1_FDMA_CTRL      SRX_1.FDMA_CTRL.R             /* Fast Message DMA Control Register */
#define SRX_1_SDMA_CTRL      SRX_1.SDMA_CTRL.R             /* Slow Serial Message DMA Control Register */
#define SRX_1_CHNL_NB_REORD1 SRX_1.CHNL_NB_REORD1.R        /* Channel Nibble Reorder 1 */
#define SRX_1_FMSG_RDY_IE    SRX_1.FMSG_RDY_IE.R           /* Fast Message Ready Interrupt Control Register */
#define SRX_1_SMSG_RDY_IE    SRX_1.SMSG_RDY_IE.R           /* Slow Serial Message Ready Interrupt Enable Register */
#define SRX_1_CH0_CLK_CTRL   SRX_1.CNL[0].CH_CLK_CTRL.R    /* Channel '0' Clock Control Register */
#define SRX_1_CH0_STATUS     SRX_1.CNL[0].CH_STATUS.R      /* Channel '0' Status Register */
#define SRX_1_CH0_CONFIG     SRX_1.CNL[0].CH_CONFIG.R      /* Channel '0' Configuration Register */
#define SRX_1_CH0_FMSG_DATA  SRX_1.CNL[0].CH_FMSG_DATA.R   /* Channel '0' Fast Message Data Read Register */
#define SRX_1_CH0_FMSG_CRC   SRX_1.CNL[0].CH_FMSG_CRC.R    /* Channel '0' Fast Message CRC Read Register */
#define SRX_1_CH0_FMSG_TS    SRX_1.CNL[0].CH_FMSG_TS.R     /* Channel '0' Fast Message Time Stamp Read Register */
#define SRX_1_CH0_SMSG_BIT3  SRX_1.CNL[0].CH_SMSG_BIT3.R   /* Channel '0' Serial Message Read Register (Bit 3) */
#define SRX_1_CH0_SMSG_BIT2  SRX_1.CNL[0].CH_SMSG_BIT2.R   /* Channel '0' Serial Message Read Register (Bit 2) */
#define SRX_1_CH0_SMSG_TS    SRX_1.CNL[0].CH_SMSG_TS.R     /* Channel '0' Serial Message Time Stamp Read */
#define SRX_1_CH0_F_DMA_RD   SRX_1.CNL[0].CH_F_DMA_RD.R    /* Channel '0' DMA Fast Message Data Read */
#define SRX_1_CH0_F_DMA_CRC  SRX_1.CNL[0].CH_F_DMA_CRC.R   /* Channel '0' DMA Fast Message CRC Read */
#define SRX_1_CH0_F_DMA_TS   SRX_1.CNL[0].CH_F_DMA_TS.R    /* Channel '0' DMA Fast Message Time Stamp Read */
#define SRX_1_CH0_S_DMA_RD   SRX_1.CNL[0].CH_S_DMA_RD.R    /* Channel '0' DMA Slow Serial Message Bit3 Read */
#define SRX_1_CH0_S_DMA_CRC  SRX_1.CNL[0].CH_S_DMA_CRC.R   /* Channel '0' DMA Slow Serial Message Bit2 Read */
#define SRX_1_CH0_S_DMA_TS   SRX_1.CNL[0].CH_S_DMA_TS.R    /* Channel '0' DMA Slow Serial Message Time Stamp Read */
#define SRX_1_CHNL0_TIMESTAMP SRX_1.CNL[0].CHNL_TIMESTAMP.R /* Channel '0' Time Stamp */
#define SRX_1_CHNL0_COUNTER  SRX_1.CNL[0].CHNL_COUNTER.R   /* Channel '0' Counter */
#define SRX_1_CH1_CLK_CTRL   SRX_1.CNL[1].CH_CLK_CTRL.R    /* Channel '1' Clock Control Register */
#define SRX_1_CH1_STATUS     SRX_1.CNL[1].CH_STATUS.R      /* Channel '1' Status Register */
#define SRX_1_CH1_CONFIG     SRX_1.CNL[1].CH_CONFIG.R      /* Channel '1' Configuration Register */
#define SRX_1_CH1_FMSG_DATA  SRX_1.CNL[1].CH_FMSG_DATA.R   /* Channel '1' Fast Message Data Read Register */
#define SRX_1_CH1_FMSG_CRC   SRX_1.CNL[1].CH_FMSG_CRC.R    /* Channel '1' Fast Message CRC Read Register */
#define SRX_1_CH1_FMSG_TS    SRX_1.CNL[1].CH_FMSG_TS.R     /* Channel '1' Fast Message Time Stamp Read Register */
#define SRX_1_CH1_SMSG_BIT3  SRX_1.CNL[1].CH_SMSG_BIT3.R   /* Channel '1' Serial Message Read Register (Bit 3) */
#define SRX_1_CH1_SMSG_BIT2  SRX_1.CNL[1].CH_SMSG_BIT2.R   /* Channel '1' Serial Message Read Register (Bit 2) */
#define SRX_1_CH1_SMSG_TS    SRX_1.CNL[1].CH_SMSG_TS.R     /* Channel '1' Serial Message Time Stamp Read */
#define SRX_1_CH1_F_DMA_RD   SRX_1.CNL[1].CH_F_DMA_RD.R    /* Channel '1' DMA Fast Message Data Read */
#define SRX_1_CH1_F_DMA_CRC  SRX_1.CNL[1].CH_F_DMA_CRC.R   /* Channel '1' DMA Fast Message CRC Read */
#define SRX_1_CH1_F_DMA_TS   SRX_1.CNL[1].CH_F_DMA_TS.R    /* Channel '1' DMA Fast Message Time Stamp Read */
#define SRX_1_CH1_S_DMA_RD   SRX_1.CNL[1].CH_S_DMA_RD.R    /* Channel '1' DMA Slow Serial Message Bit3 Read */
#define SRX_1_CH1_S_DMA_CRC  SRX_1.CNL[1].CH_S_DMA_CRC.R   /* Channel '1' DMA Slow Serial Message Bit2 Read */
#define SRX_1_CH1_S_DMA_TS   SRX_1.CNL[1].CH_S_DMA_TS.R    /* Channel '1' DMA Slow Serial Message Time Stamp Read */
#define SRX_1_CHNL1_TIMESTAMP SRX_1.CNL[1].CHNL_TIMESTAMP.R /* Channel '1' Time Stamp */
#define SRX_1_CHNL1_COUNTER  SRX_1.CNL[1].CHNL_COUNTER.R   /* Channel '1' Counter */
#define SRX_1_CH2_CLK_CTRL   SRX_1.CNL[2].CH_CLK_CTRL.R    /* Channel '2' Clock Control Register */
#define SRX_1_CH2_STATUS     SRX_1.CNL[2].CH_STATUS.R      /* Channel '2' Status Register */
#define SRX_1_CH2_CONFIG     SRX_1.CNL[2].CH_CONFIG.R      /* Channel '2' Configuration Register */
#define SRX_1_CH2_FMSG_DATA  SRX_1.CNL[2].CH_FMSG_DATA.R   /* Channel '2' Fast Message Data Read Register */
#define SRX_1_CH2_FMSG_CRC   SRX_1.CNL[2].CH_FMSG_CRC.R    /* Channel '2' Fast Message CRC Read Register */
#define SRX_1_CH2_FMSG_TS    SRX_1.CNL[2].CH_FMSG_TS.R     /* Channel '2' Fast Message Time Stamp Read Register */
#define SRX_1_CH2_SMSG_BIT3  SRX_1.CNL[2].CH_SMSG_BIT3.R   /* Channel '2' Serial Message Read Register (Bit 3) */
#define SRX_1_CH2_SMSG_BIT2  SRX_1.CNL[2].CH_SMSG_BIT2.R   /* Channel '2' Serial Message Read Register (Bit 2) */
#define SRX_1_CH2_SMSG_TS    SRX_1.CNL[2].CH_SMSG_TS.R     /* Channel '2' Serial Message Time Stamp Read */
#define SRX_1_CH2_F_DMA_RD   SRX_1.CNL[2].CH_F_DMA_RD.R    /* Channel '2' DMA Fast Message Data Read */
#define SRX_1_CH2_F_DMA_CRC  SRX_1.CNL[2].CH_F_DMA_CRC.R   /* Channel '2' DMA Fast Message CRC Read */
#define SRX_1_CH2_F_DMA_TS   SRX_1.CNL[2].CH_F_DMA_TS.R    /* Channel '2' DMA Fast Message Time Stamp Read */
#define SRX_1_CH2_S_DMA_RD   SRX_1.CNL[2].CH_S_DMA_RD.R    /* Channel '2' DMA Slow Serial Message Bit3 Read */
#define SRX_1_CH2_S_DMA_CRC  SRX_1.CNL[2].CH_S_DMA_CRC.R   /* Channel '2' DMA Slow Serial Message Bit2 Read */
#define SRX_1_CH2_S_DMA_TS   SRX_1.CNL[2].CH_S_DMA_TS.R    /* Channel '2' DMA Slow Serial Message Time Stamp Read */
#define SRX_1_CHNL2_TIMESTAMP SRX_1.CNL[2].CHNL_TIMESTAMP.R /* Channel '2' Time Stamp */
#define SRX_1_CHNL2_COUNTER  SRX_1.CNL[2].CHNL_COUNTER.R   /* Channel '2' Counter */
#define SRX_1_CH3_CLK_CTRL   SRX_1.CNL[3].CH_CLK_CTRL.R    /* Channel '3' Clock Control Register */
#define SRX_1_CH3_STATUS     SRX_1.CNL[3].CH_STATUS.R      /* Channel '3' Status Register */
#define SRX_1_CH3_CONFIG     SRX_1.CNL[3].CH_CONFIG.R      /* Channel '3' Configuration Register */
#define SRX_1_CH3_FMSG_DATA  SRX_1.CNL[3].CH_FMSG_DATA.R   /* Channel '3' Fast Message Data Read Register */
#define SRX_1_CH3_FMSG_CRC   SRX_1.CNL[3].CH_FMSG_CRC.R    /* Channel '3' Fast Message CRC Read Register */
#define SRX_1_CH3_FMSG_TS    SRX_1.CNL[3].CH_FMSG_TS.R     /* Channel '3' Fast Message Time Stamp Read Register */
#define SRX_1_CH3_SMSG_BIT3  SRX_1.CNL[3].CH_SMSG_BIT3.R   /* Channel '3' Serial Message Read Register (Bit 3) */
#define SRX_1_CH3_SMSG_BIT2  SRX_1.CNL[3].CH_SMSG_BIT2.R   /* Channel '3' Serial Message Read Register (Bit 2) */
#define SRX_1_CH3_SMSG_TS    SRX_1.CNL[3].CH_SMSG_TS.R     /* Channel '3' Serial Message Time Stamp Read */
#define SRX_1_CH3_F_DMA_RD   SRX_1.CNL[3].CH_F_DMA_RD.R    /* Channel '3' DMA Fast Message Data Read */
#define SRX_1_CH3_F_DMA_CRC  SRX_1.CNL[3].CH_F_DMA_CRC.R   /* Channel '3' DMA Fast Message CRC Read */
#define SRX_1_CH3_F_DMA_TS   SRX_1.CNL[3].CH_F_DMA_TS.R    /* Channel '3' DMA Fast Message Time Stamp Read */
#define SRX_1_CH3_S_DMA_RD   SRX_1.CNL[3].CH_S_DMA_RD.R    /* Channel '3' DMA Slow Serial Message Bit3 Read */
#define SRX_1_CH3_S_DMA_CRC  SRX_1.CNL[3].CH_S_DMA_CRC.R   /* Channel '3' DMA Slow Serial Message Bit2 Read */
#define SRX_1_CH3_S_DMA_TS   SRX_1.CNL[3].CH_S_DMA_TS.R    /* Channel '3' DMA Slow Serial Message Time Stamp Read */
#define SRX_1_CHNL3_TIMESTAMP SRX_1.CNL[3].CHNL_TIMESTAMP.R /* Channel '3' Time Stamp */
#define SRX_1_CHNL3_COUNTER  SRX_1.CNL[3].CHNL_COUNTER.R   /* Channel '3' Counter */
#define SRX_1_CH4_CLK_CTRL   SRX_1.CNL[4].CH_CLK_CTRL.R    /* Channel '4' Clock Control Register */
#define SRX_1_CH4_STATUS     SRX_1.CNL[4].CH_STATUS.R      /* Channel '4' Status Register */
#define SRX_1_CH4_CONFIG     SRX_1.CNL[4].CH_CONFIG.R      /* Channel '4' Configuration Register */
#define SRX_1_CH4_FMSG_DATA  SRX_1.CNL[4].CH_FMSG_DATA.R   /* Channel '4' Fast Message Data Read Register */
#define SRX_1_CH4_FMSG_CRC   SRX_1.CNL[4].CH_FMSG_CRC.R    /* Channel '4' Fast Message CRC Read Register */
#define SRX_1_CH4_FMSG_TS    SRX_1.CNL[4].CH_FMSG_TS.R     /* Channel '4' Fast Message Time Stamp Read Register */
#define SRX_1_CH4_SMSG_BIT3  SRX_1.CNL[4].CH_SMSG_BIT3.R   /* Channel '4' Serial Message Read Register (Bit 3) */
#define SRX_1_CH4_SMSG_BIT2  SRX_1.CNL[4].CH_SMSG_BIT2.R   /* Channel '4' Serial Message Read Register (Bit 2) */
#define SRX_1_CH4_SMSG_TS    SRX_1.CNL[4].CH_SMSG_TS.R     /* Channel '4' Serial Message Time Stamp Read */
#define SRX_1_CH4_F_DMA_RD   SRX_1.CNL[4].CH_F_DMA_RD.R    /* Channel '4' DMA Fast Message Data Read */
#define SRX_1_CH4_F_DMA_CRC  SRX_1.CNL[4].CH_F_DMA_CRC.R   /* Channel '4' DMA Fast Message CRC Read */
#define SRX_1_CH4_F_DMA_TS   SRX_1.CNL[4].CH_F_DMA_TS.R    /* Channel '4' DMA Fast Message Time Stamp Read */
#define SRX_1_CH4_S_DMA_RD   SRX_1.CNL[4].CH_S_DMA_RD.R    /* Channel '4' DMA Slow Serial Message Bit3 Read */
#define SRX_1_CH4_S_DMA_CRC  SRX_1.CNL[4].CH_S_DMA_CRC.R   /* Channel '4' DMA Slow Serial Message Bit2 Read */
#define SRX_1_CH4_S_DMA_TS   SRX_1.CNL[4].CH_S_DMA_TS.R    /* Channel '4' DMA Slow Serial Message Time Stamp Read */
#define SRX_1_CHNL4_TIMESTAMP SRX_1.CNL[4].CHNL_TIMESTAMP.R /* Channel '4' Time Stamp */
#define SRX_1_CHNL4_COUNTER  SRX_1.CNL[4].CHNL_COUNTER.R   /* Channel '4' Counter */
#define SRX_1_CH5_CLK_CTRL   SRX_1.CNL[5].CH_CLK_CTRL.R    /* Channel '5' Clock Control Register */
#define SRX_1_CH5_STATUS     SRX_1.CNL[5].CH_STATUS.R      /* Channel '5' Status Register */
#define SRX_1_CH5_CONFIG     SRX_1.CNL[5].CH_CONFIG.R      /* Channel '5' Configuration Register */
#define SRX_1_CH5_FMSG_DATA  SRX_1.CNL[5].CH_FMSG_DATA.R   /* Channel '5' Fast Message Data Read Register */
#define SRX_1_CH5_FMSG_CRC   SRX_1.CNL[5].CH_FMSG_CRC.R    /* Channel '5' Fast Message CRC Read Register */
#define SRX_1_CH5_FMSG_TS    SRX_1.CNL[5].CH_FMSG_TS.R     /* Channel '5' Fast Message Time Stamp Read Register */
#define SRX_1_CH5_SMSG_BIT3  SRX_1.CNL[5].CH_SMSG_BIT3.R   /* Channel '5' Serial Message Read Register (Bit 3) */
#define SRX_1_CH5_SMSG_BIT2  SRX_1.CNL[5].CH_SMSG_BIT2.R   /* Channel '5' Serial Message Read Register (Bit 2) */
#define SRX_1_CH5_SMSG_TS    SRX_1.CNL[5].CH_SMSG_TS.R     /* Channel '5' Serial Message Time Stamp Read */
#define SRX_1_CH5_F_DMA_RD   SRX_1.CNL[5].CH_F_DMA_RD.R    /* Channel '5' DMA Fast Message Data Read */
#define SRX_1_CH5_F_DMA_CRC  SRX_1.CNL[5].CH_F_DMA_CRC.R   /* Channel '5' DMA Fast Message CRC Read */
#define SRX_1_CH5_F_DMA_TS   SRX_1.CNL[5].CH_F_DMA_TS.R    /* Channel '5' DMA Fast Message Time Stamp Read */
#define SRX_1_CH5_S_DMA_RD   SRX_1.CNL[5].CH_S_DMA_RD.R    /* Channel '5' DMA Slow Serial Message Bit3 Read */
#define SRX_1_CH5_S_DMA_CRC  SRX_1.CNL[5].CH_S_DMA_CRC.R   /* Channel '5' DMA Slow Serial Message Bit2 Read */
#define SRX_1_CH5_S_DMA_TS   SRX_1.CNL[5].CH_S_DMA_TS.R    /* Channel '5' DMA Slow Serial Message Time Stamp Read */
#define SRX_1_CHNL5_TIMESTAMP SRX_1.CNL[5].CHNL_TIMESTAMP.R /* Channel '5' Time Stamp */
#define SRX_1_CHNL5_COUNTER  SRX_1.CNL[5].CHNL_COUNTER.R   /* Channel '5' Counter */
#define SRX_1_CH6_CLK_CTRL   SRX_1.CNL[6].CH_CLK_CTRL.R    /* Channel '6' Clock Control Register */
#define SRX_1_CH6_STATUS     SRX_1.CNL[6].CH_STATUS.R      /* Channel '6' Status Register */
#define SRX_1_CH6_CONFIG     SRX_1.CNL[6].CH_CONFIG.R      /* Channel '6' Configuration Register */
#define SRX_1_CH6_FMSG_DATA  SRX_1.CNL[6].CH_FMSG_DATA.R   /* Channel '6' Fast Message Data Read Register */
#define SRX_1_CH6_FMSG_CRC   SRX_1.CNL[6].CH_FMSG_CRC.R    /* Channel '6' Fast Message CRC Read Register */
#define SRX_1_CH6_FMSG_TS    SRX_1.CNL[6].CH_FMSG_TS.R     /* Channel '6' Fast Message Time Stamp Read Register */
#define SRX_1_CH6_SMSG_BIT3  SRX_1.CNL[6].CH_SMSG_BIT3.R   /* Channel '6' Serial Message Read Register (Bit 3) */
#define SRX_1_CH6_SMSG_BIT2  SRX_1.CNL[6].CH_SMSG_BIT2.R   /* Channel '6' Serial Message Read Register (Bit 2) */
#define SRX_1_CH6_SMSG_TS    SRX_1.CNL[6].CH_SMSG_TS.R     /* Channel '6' Serial Message Time Stamp Read */
#define SRX_1_CH6_F_DMA_RD   SRX_1.CNL[6].CH_F_DMA_RD.R    /* Channel '6' DMA Fast Message Data Read */
#define SRX_1_CH6_F_DMA_CRC  SRX_1.CNL[6].CH_F_DMA_CRC.R   /* Channel '6' DMA Fast Message CRC Read */
#define SRX_1_CH6_F_DMA_TS   SRX_1.CNL[6].CH_F_DMA_TS.R    /* Channel '6' DMA Fast Message Time Stamp Read */
#define SRX_1_CH6_S_DMA_RD   SRX_1.CNL[6].CH_S_DMA_RD.R    /* Channel '6' DMA Slow Serial Message Bit3 Read */
#define SRX_1_CH6_S_DMA_CRC  SRX_1.CNL[6].CH_S_DMA_CRC.R   /* Channel '6' DMA Slow Serial Message Bit2 Read */
#define SRX_1_CH6_S_DMA_TS   SRX_1.CNL[6].CH_S_DMA_TS.R    /* Channel '6' DMA Slow Serial Message Time Stamp Read */
#define SRX_1_CHNL6_TIMESTAMP SRX_1.CNL[6].CHNL_TIMESTAMP.R /* Channel '6' Time Stamp */
#define SRX_1_CHNL6_COUNTER  SRX_1.CNL[6].CHNL_COUNTER.R   /* Channel '6' Counter */
#define SRX_1_CH7_CLK_CTRL   SRX_1.CNL[7].CH_CLK_CTRL.R    /* Channel '7' Clock Control Register */
#define SRX_1_CH7_STATUS     SRX_1.CNL[7].CH_STATUS.R      /* Channel '7' Status Register */
#define SRX_1_CH7_CONFIG     SRX_1.CNL[7].CH_CONFIG.R      /* Channel '7' Configuration Register */
#define SRX_1_CH7_FMSG_DATA  SRX_1.CNL[7].CH_FMSG_DATA.R   /* Channel '7' Fast Message Data Read Register */
#define SRX_1_CH7_FMSG_CRC   SRX_1.CNL[7].CH_FMSG_CRC.R    /* Channel '7' Fast Message CRC Read Register */
#define SRX_1_CH7_FMSG_TS    SRX_1.CNL[7].CH_FMSG_TS.R     /* Channel '7' Fast Message Time Stamp Read Register */
#define SRX_1_CH7_SMSG_BIT3  SRX_1.CNL[7].CH_SMSG_BIT3.R   /* Channel '7' Serial Message Read Register (Bit 3) */
#define SRX_1_CH7_SMSG_BIT2  SRX_1.CNL[7].CH_SMSG_BIT2.R   /* Channel '7' Serial Message Read Register (Bit 2) */
#define SRX_1_CH7_SMSG_TS    SRX_1.CNL[7].CH_SMSG_TS.R     /* Channel '7' Serial Message Time Stamp Read */
#define SRX_1_CH7_F_DMA_RD   SRX_1.CNL[7].CH_F_DMA_RD.R    /* Channel '7' DMA Fast Message Data Read */
#define SRX_1_CH7_F_DMA_CRC  SRX_1.CNL[7].CH_F_DMA_CRC.R   /* Channel '7' DMA Fast Message CRC Read */
#define SRX_1_CH7_F_DMA_TS   SRX_1.CNL[7].CH_F_DMA_TS.R    /* Channel '7' DMA Fast Message Time Stamp Read */
#define SRX_1_CH7_S_DMA_RD   SRX_1.CNL[7].CH_S_DMA_RD.R    /* Channel '7' DMA Slow Serial Message Bit3 Read */
#define SRX_1_CH7_S_DMA_CRC  SRX_1.CNL[7].CH_S_DMA_CRC.R   /* Channel '7' DMA Slow Serial Message Bit2 Read */
#define SRX_1_CH7_S_DMA_TS   SRX_1.CNL[7].CH_S_DMA_TS.R    /* Channel '7' DMA Slow Serial Message Time Stamp Read */
#define SRX_1_CHNL7_TIMESTAMP SRX_1.CNL[7].CHNL_TIMESTAMP.R /* Channel '7' Time Stamp */
#define SRX_1_CHNL7_COUNTER  SRX_1.CNL[7].CHNL_COUNTER.R   /* Channel '7' Counter */

/* STM */
#define CE_STM_0_CR          CE_STM_0.CR.R                 /* Control */
#define CE_STM_0_CNT         CE_STM_0.CNT.R                /* Count */
#define CE_STM_0_CCR0        CE_STM_0.CHANNEL[0].CCR.R     /* Channel Control */
#define CE_STM_0_CIR0        CE_STM_0.CHANNEL[0].CIR.R     /* Channel Interrupt */
#define CE_STM_0_CMP0        CE_STM_0.CHANNEL[0].CMP.R     /* Channel Compare */
#define CE_STM_0_CCR1        CE_STM_0.CHANNEL[1].CCR.R     /* Channel Control */
#define CE_STM_0_CIR1        CE_STM_0.CHANNEL[1].CIR.R     /* Channel Interrupt */
#define CE_STM_0_CMP1        CE_STM_0.CHANNEL[1].CMP.R     /* Channel Compare */
#define CE_STM_0_CCR2        CE_STM_0.CHANNEL[2].CCR.R     /* Channel Control */
#define CE_STM_0_CIR2        CE_STM_0.CHANNEL[2].CIR.R     /* Channel Interrupt */
#define CE_STM_0_CMP2        CE_STM_0.CHANNEL[2].CMP.R     /* Channel Compare */
#define CE_STM_0_CCR3        CE_STM_0.CHANNEL[3].CCR.R     /* Channel Control */
#define CE_STM_0_CIR3        CE_STM_0.CHANNEL[3].CIR.R     /* Channel Interrupt */
#define CE_STM_0_CMP3        CE_STM_0.CHANNEL[3].CMP.R     /* Channel Compare */

#define CE_STM_1_CR          CE_STM_1.CR.R                 /* Control */
#define CE_STM_1_CNT         CE_STM_1.CNT.R                /* Count */
#define CE_STM_1_CCR0        CE_STM_1.CHANNEL[0].CCR.R     /* Channel Control */
#define CE_STM_1_CIR0        CE_STM_1.CHANNEL[0].CIR.R     /* Channel Interrupt */
#define CE_STM_1_CMP0        CE_STM_1.CHANNEL[0].CMP.R     /* Channel Compare */
#define CE_STM_1_CCR1        CE_STM_1.CHANNEL[1].CCR.R     /* Channel Control */
#define CE_STM_1_CIR1        CE_STM_1.CHANNEL[1].CIR.R     /* Channel Interrupt */
#define CE_STM_1_CMP1        CE_STM_1.CHANNEL[1].CMP.R     /* Channel Compare */
#define CE_STM_1_CCR2        CE_STM_1.CHANNEL[2].CCR.R     /* Channel Control */
#define CE_STM_1_CIR2        CE_STM_1.CHANNEL[2].CIR.R     /* Channel Interrupt */
#define CE_STM_1_CMP2        CE_STM_1.CHANNEL[2].CMP.R     /* Channel Compare */
#define CE_STM_1_CCR3        CE_STM_1.CHANNEL[3].CCR.R     /* Channel Control */
#define CE_STM_1_CIR3        CE_STM_1.CHANNEL[3].CIR.R     /* Channel Interrupt */
#define CE_STM_1_CMP3        CE_STM_1.CHANNEL[3].CMP.R     /* Channel Compare */

#define CE_STM_2_CR          CE_STM_2.CR.R                 /* Control */
#define CE_STM_2_CNT         CE_STM_2.CNT.R                /* Count */
#define CE_STM_2_CCR0        CE_STM_2.CHANNEL[0].CCR.R     /* Channel Control */
#define CE_STM_2_CIR0        CE_STM_2.CHANNEL[0].CIR.R     /* Channel Interrupt */
#define CE_STM_2_CMP0        CE_STM_2.CHANNEL[0].CMP.R     /* Channel Compare */
#define CE_STM_2_CCR1        CE_STM_2.CHANNEL[1].CCR.R     /* Channel Control */
#define CE_STM_2_CIR1        CE_STM_2.CHANNEL[1].CIR.R     /* Channel Interrupt */
#define CE_STM_2_CMP1        CE_STM_2.CHANNEL[1].CMP.R     /* Channel Compare */
#define CE_STM_2_CCR2        CE_STM_2.CHANNEL[2].CCR.R     /* Channel Control */
#define CE_STM_2_CIR2        CE_STM_2.CHANNEL[2].CIR.R     /* Channel Interrupt */
#define CE_STM_2_CMP2        CE_STM_2.CHANNEL[2].CMP.R     /* Channel Compare */
#define CE_STM_2_CCR3        CE_STM_2.CHANNEL[3].CCR.R     /* Channel Control */
#define CE_STM_2_CIR3        CE_STM_2.CHANNEL[3].CIR.R     /* Channel Interrupt */
#define CE_STM_2_CMP3        CE_STM_2.CHANNEL[3].CMP.R     /* Channel Compare */

#define SMU__STM_0_CR        SMU__STM_0.CR.R               /* Control */
#define SMU__STM_0_CNT       SMU__STM_0.CNT.R              /* Count */
#define SMU__STM_0_CCR0      SMU__STM_0.CHANNEL[0].CCR.R   /* Channel Control */
#define SMU__STM_0_CIR0      SMU__STM_0.CHANNEL[0].CIR.R   /* Channel Interrupt */
#define SMU__STM_0_CMP0      SMU__STM_0.CHANNEL[0].CMP.R   /* Channel Compare */
#define SMU__STM_0_CCR1      SMU__STM_0.CHANNEL[1].CCR.R   /* Channel Control */
#define SMU__STM_0_CIR1      SMU__STM_0.CHANNEL[1].CIR.R   /* Channel Interrupt */
#define SMU__STM_0_CMP1      SMU__STM_0.CHANNEL[1].CMP.R   /* Channel Compare */
#define SMU__STM_0_CCR2      SMU__STM_0.CHANNEL[2].CCR.R   /* Channel Control */
#define SMU__STM_0_CIR2      SMU__STM_0.CHANNEL[2].CIR.R   /* Channel Interrupt */
#define SMU__STM_0_CMP2      SMU__STM_0.CHANNEL[2].CMP.R   /* Channel Compare */
#define SMU__STM_0_CCR3      SMU__STM_0.CHANNEL[3].CCR.R   /* Channel Control */
#define SMU__STM_0_CIR3      SMU__STM_0.CHANNEL[3].CIR.R   /* Channel Interrupt */
#define SMU__STM_0_CMP3      SMU__STM_0.CHANNEL[3].CMP.R   /* Channel Compare */

#define SMU__STM_2_CR        SMU__STM_2.CR.R               /* Control */
#define SMU__STM_2_CNT       SMU__STM_2.CNT.R              /* Count */
#define SMU__STM_2_CCR0      SMU__STM_2.CHANNEL[0].CCR.R   /* Channel Control */
#define SMU__STM_2_CIR0      SMU__STM_2.CHANNEL[0].CIR.R   /* Channel Interrupt */
#define SMU__STM_2_CMP0      SMU__STM_2.CHANNEL[0].CMP.R   /* Channel Compare */
#define SMU__STM_2_CCR1      SMU__STM_2.CHANNEL[1].CCR.R   /* Channel Control */
#define SMU__STM_2_CIR1      SMU__STM_2.CHANNEL[1].CIR.R   /* Channel Interrupt */
#define SMU__STM_2_CMP1      SMU__STM_2.CHANNEL[1].CMP.R   /* Channel Compare */
#define SMU__STM_2_CCR2      SMU__STM_2.CHANNEL[2].CCR.R   /* Channel Control */
#define SMU__STM_2_CIR2      SMU__STM_2.CHANNEL[2].CIR.R   /* Channel Interrupt */
#define SMU__STM_2_CMP2      SMU__STM_2.CHANNEL[2].CMP.R   /* Channel Compare */
#define SMU__STM_2_CCR3      SMU__STM_2.CHANNEL[3].CCR.R   /* Channel Control */
#define SMU__STM_2_CIR3      SMU__STM_2.CHANNEL[3].CIR.R   /* Channel Interrupt */
#define SMU__STM_2_CMP3      SMU__STM_2.CHANNEL[3].CMP.R   /* Channel Compare */

#define RTU0__STM_0_CR       RTU0__STM_0.CR.R              /* Control */
#define RTU0__STM_0_CNT      RTU0__STM_0.CNT.R             /* Count */
#define RTU0__STM_0_CCR0     RTU0__STM_0.CHANNEL[0].CCR.R  /* Channel Control */
#define RTU0__STM_0_CIR0     RTU0__STM_0.CHANNEL[0].CIR.R  /* Channel Interrupt */
#define RTU0__STM_0_CMP0     RTU0__STM_0.CHANNEL[0].CMP.R  /* Channel Compare */
#define RTU0__STM_0_CCR1     RTU0__STM_0.CHANNEL[1].CCR.R  /* Channel Control */
#define RTU0__STM_0_CIR1     RTU0__STM_0.CHANNEL[1].CIR.R  /* Channel Interrupt */
#define RTU0__STM_0_CMP1     RTU0__STM_0.CHANNEL[1].CMP.R  /* Channel Compare */
#define RTU0__STM_0_CCR2     RTU0__STM_0.CHANNEL[2].CCR.R  /* Channel Control */
#define RTU0__STM_0_CIR2     RTU0__STM_0.CHANNEL[2].CIR.R  /* Channel Interrupt */
#define RTU0__STM_0_CMP2     RTU0__STM_0.CHANNEL[2].CMP.R  /* Channel Compare */
#define RTU0__STM_0_CCR3     RTU0__STM_0.CHANNEL[3].CCR.R  /* Channel Control */
#define RTU0__STM_0_CIR3     RTU0__STM_0.CHANNEL[3].CIR.R  /* Channel Interrupt */
#define RTU0__STM_0_CMP3     RTU0__STM_0.CHANNEL[3].CMP.R  /* Channel Compare */

#define RTU1__STM_0_CR       RTU1__STM_0.CR.R              /* Control */
#define RTU1__STM_0_CNT      RTU1__STM_0.CNT.R             /* Count */
#define RTU1__STM_0_CCR0     RTU1__STM_0.CHANNEL[0].CCR.R  /* Channel Control */
#define RTU1__STM_0_CIR0     RTU1__STM_0.CHANNEL[0].CIR.R  /* Channel Interrupt */
#define RTU1__STM_0_CMP0     RTU1__STM_0.CHANNEL[0].CMP.R  /* Channel Compare */
#define RTU1__STM_0_CCR1     RTU1__STM_0.CHANNEL[1].CCR.R  /* Channel Control */
#define RTU1__STM_0_CIR1     RTU1__STM_0.CHANNEL[1].CIR.R  /* Channel Interrupt */
#define RTU1__STM_0_CMP1     RTU1__STM_0.CHANNEL[1].CMP.R  /* Channel Compare */
#define RTU1__STM_0_CCR2     RTU1__STM_0.CHANNEL[2].CCR.R  /* Channel Control */
#define RTU1__STM_0_CIR2     RTU1__STM_0.CHANNEL[2].CIR.R  /* Channel Interrupt */
#define RTU1__STM_0_CMP2     RTU1__STM_0.CHANNEL[2].CMP.R  /* Channel Compare */
#define RTU1__STM_0_CCR3     RTU1__STM_0.CHANNEL[3].CCR.R  /* Channel Control */
#define RTU1__STM_0_CIR3     RTU1__STM_0.CHANNEL[3].CIR.R  /* Channel Interrupt */
#define RTU1__STM_0_CMP3     RTU1__STM_0.CHANNEL[3].CMP.R  /* Channel Compare */

#define RTU0__STM_1_CR       RTU0__STM_1.CR.R              /* Control */
#define RTU0__STM_1_CNT      RTU0__STM_1.CNT.R             /* Count */
#define RTU0__STM_1_CCR0     RTU0__STM_1.CHANNEL[0].CCR.R  /* Channel Control */
#define RTU0__STM_1_CIR0     RTU0__STM_1.CHANNEL[0].CIR.R  /* Channel Interrupt */
#define RTU0__STM_1_CMP0     RTU0__STM_1.CHANNEL[0].CMP.R  /* Channel Compare */
#define RTU0__STM_1_CCR1     RTU0__STM_1.CHANNEL[1].CCR.R  /* Channel Control */
#define RTU0__STM_1_CIR1     RTU0__STM_1.CHANNEL[1].CIR.R  /* Channel Interrupt */
#define RTU0__STM_1_CMP1     RTU0__STM_1.CHANNEL[1].CMP.R  /* Channel Compare */
#define RTU0__STM_1_CCR2     RTU0__STM_1.CHANNEL[2].CCR.R  /* Channel Control */
#define RTU0__STM_1_CIR2     RTU0__STM_1.CHANNEL[2].CIR.R  /* Channel Interrupt */
#define RTU0__STM_1_CMP2     RTU0__STM_1.CHANNEL[2].CMP.R  /* Channel Compare */
#define RTU0__STM_1_CCR3     RTU0__STM_1.CHANNEL[3].CCR.R  /* Channel Control */
#define RTU0__STM_1_CIR3     RTU0__STM_1.CHANNEL[3].CIR.R  /* Channel Interrupt */
#define RTU0__STM_1_CMP3     RTU0__STM_1.CHANNEL[3].CMP.R  /* Channel Compare */

#define RTU1__STM_1_CR       RTU1__STM_1.CR.R              /* Control */
#define RTU1__STM_1_CNT      RTU1__STM_1.CNT.R             /* Count */
#define RTU1__STM_1_CCR0     RTU1__STM_1.CHANNEL[0].CCR.R  /* Channel Control */
#define RTU1__STM_1_CIR0     RTU1__STM_1.CHANNEL[0].CIR.R  /* Channel Interrupt */
#define RTU1__STM_1_CMP0     RTU1__STM_1.CHANNEL[0].CMP.R  /* Channel Compare */
#define RTU1__STM_1_CCR1     RTU1__STM_1.CHANNEL[1].CCR.R  /* Channel Control */
#define RTU1__STM_1_CIR1     RTU1__STM_1.CHANNEL[1].CIR.R  /* Channel Interrupt */
#define RTU1__STM_1_CMP1     RTU1__STM_1.CHANNEL[1].CMP.R  /* Channel Compare */
#define RTU1__STM_1_CCR2     RTU1__STM_1.CHANNEL[2].CCR.R  /* Channel Control */
#define RTU1__STM_1_CIR2     RTU1__STM_1.CHANNEL[2].CIR.R  /* Channel Interrupt */
#define RTU1__STM_1_CMP2     RTU1__STM_1.CHANNEL[2].CMP.R  /* Channel Compare */
#define RTU1__STM_1_CCR3     RTU1__STM_1.CHANNEL[3].CCR.R  /* Channel Control */
#define RTU1__STM_1_CIR3     RTU1__STM_1.CHANNEL[3].CIR.R  /* Channel Interrupt */
#define RTU1__STM_1_CMP3     RTU1__STM_1.CHANNEL[3].CMP.R  /* Channel Compare */

#define RTU0__STM_2_CR       RTU0__STM_2.CR.R              /* Control */
#define RTU0__STM_2_CNT      RTU0__STM_2.CNT.R             /* Count */
#define RTU0__STM_2_CCR0     RTU0__STM_2.CHANNEL[0].CCR.R  /* Channel Control */
#define RTU0__STM_2_CIR0     RTU0__STM_2.CHANNEL[0].CIR.R  /* Channel Interrupt */
#define RTU0__STM_2_CMP0     RTU0__STM_2.CHANNEL[0].CMP.R  /* Channel Compare */
#define RTU0__STM_2_CCR1     RTU0__STM_2.CHANNEL[1].CCR.R  /* Channel Control */
#define RTU0__STM_2_CIR1     RTU0__STM_2.CHANNEL[1].CIR.R  /* Channel Interrupt */
#define RTU0__STM_2_CMP1     RTU0__STM_2.CHANNEL[1].CMP.R  /* Channel Compare */
#define RTU0__STM_2_CCR2     RTU0__STM_2.CHANNEL[2].CCR.R  /* Channel Control */
#define RTU0__STM_2_CIR2     RTU0__STM_2.CHANNEL[2].CIR.R  /* Channel Interrupt */
#define RTU0__STM_2_CMP2     RTU0__STM_2.CHANNEL[2].CMP.R  /* Channel Compare */
#define RTU0__STM_2_CCR3     RTU0__STM_2.CHANNEL[3].CCR.R  /* Channel Control */
#define RTU0__STM_2_CIR3     RTU0__STM_2.CHANNEL[3].CIR.R  /* Channel Interrupt */
#define RTU0__STM_2_CMP3     RTU0__STM_2.CHANNEL[3].CMP.R  /* Channel Compare */

#define RTU1__STM_2_CR       RTU1__STM_2.CR.R              /* Control */
#define RTU1__STM_2_CNT      RTU1__STM_2.CNT.R             /* Count */
#define RTU1__STM_2_CCR0     RTU1__STM_2.CHANNEL[0].CCR.R  /* Channel Control */
#define RTU1__STM_2_CIR0     RTU1__STM_2.CHANNEL[0].CIR.R  /* Channel Interrupt */
#define RTU1__STM_2_CMP0     RTU1__STM_2.CHANNEL[0].CMP.R  /* Channel Compare */
#define RTU1__STM_2_CCR1     RTU1__STM_2.CHANNEL[1].CCR.R  /* Channel Control */
#define RTU1__STM_2_CIR1     RTU1__STM_2.CHANNEL[1].CIR.R  /* Channel Interrupt */
#define RTU1__STM_2_CMP1     RTU1__STM_2.CHANNEL[1].CMP.R  /* Channel Compare */
#define RTU1__STM_2_CCR2     RTU1__STM_2.CHANNEL[2].CCR.R  /* Channel Control */
#define RTU1__STM_2_CIR2     RTU1__STM_2.CHANNEL[2].CIR.R  /* Channel Interrupt */
#define RTU1__STM_2_CMP2     RTU1__STM_2.CHANNEL[2].CMP.R  /* Channel Compare */
#define RTU1__STM_2_CCR3     RTU1__STM_2.CHANNEL[3].CCR.R  /* Channel Control */
#define RTU1__STM_2_CIR3     RTU1__STM_2.CHANNEL[3].CIR.R  /* Channel Interrupt */
#define RTU1__STM_2_CMP3     RTU1__STM_2.CHANNEL[3].CMP.R  /* Channel Compare */

#define RTU0__STM_3_CR       RTU0__STM_3.CR.R              /* Control */
#define RTU0__STM_3_CNT      RTU0__STM_3.CNT.R             /* Count */
#define RTU0__STM_3_CCR0     RTU0__STM_3.CHANNEL[0].CCR.R  /* Channel Control */
#define RTU0__STM_3_CIR0     RTU0__STM_3.CHANNEL[0].CIR.R  /* Channel Interrupt */
#define RTU0__STM_3_CMP0     RTU0__STM_3.CHANNEL[0].CMP.R  /* Channel Compare */
#define RTU0__STM_3_CCR1     RTU0__STM_3.CHANNEL[1].CCR.R  /* Channel Control */
#define RTU0__STM_3_CIR1     RTU0__STM_3.CHANNEL[1].CIR.R  /* Channel Interrupt */
#define RTU0__STM_3_CMP1     RTU0__STM_3.CHANNEL[1].CMP.R  /* Channel Compare */
#define RTU0__STM_3_CCR2     RTU0__STM_3.CHANNEL[2].CCR.R  /* Channel Control */
#define RTU0__STM_3_CIR2     RTU0__STM_3.CHANNEL[2].CIR.R  /* Channel Interrupt */
#define RTU0__STM_3_CMP2     RTU0__STM_3.CHANNEL[2].CMP.R  /* Channel Compare */
#define RTU0__STM_3_CCR3     RTU0__STM_3.CHANNEL[3].CCR.R  /* Channel Control */
#define RTU0__STM_3_CIR3     RTU0__STM_3.CHANNEL[3].CIR.R  /* Channel Interrupt */
#define RTU0__STM_3_CMP3     RTU0__STM_3.CHANNEL[3].CMP.R  /* Channel Compare */

#define RTU1__STM_3_CR       RTU1__STM_3.CR.R              /* Control */
#define RTU1__STM_3_CNT      RTU1__STM_3.CNT.R             /* Count */
#define RTU1__STM_3_CCR0     RTU1__STM_3.CHANNEL[0].CCR.R  /* Channel Control */
#define RTU1__STM_3_CIR0     RTU1__STM_3.CHANNEL[0].CIR.R  /* Channel Interrupt */
#define RTU1__STM_3_CMP0     RTU1__STM_3.CHANNEL[0].CMP.R  /* Channel Compare */
#define RTU1__STM_3_CCR1     RTU1__STM_3.CHANNEL[1].CCR.R  /* Channel Control */
#define RTU1__STM_3_CIR1     RTU1__STM_3.CHANNEL[1].CIR.R  /* Channel Interrupt */
#define RTU1__STM_3_CMP1     RTU1__STM_3.CHANNEL[1].CMP.R  /* Channel Compare */
#define RTU1__STM_3_CCR2     RTU1__STM_3.CHANNEL[2].CCR.R  /* Channel Control */
#define RTU1__STM_3_CIR2     RTU1__STM_3.CHANNEL[2].CIR.R  /* Channel Interrupt */
#define RTU1__STM_3_CMP2     RTU1__STM_3.CHANNEL[2].CMP.R  /* Channel Compare */
#define RTU1__STM_3_CCR3     RTU1__STM_3.CHANNEL[3].CCR.R  /* Channel Control */
#define RTU1__STM_3_CIR3     RTU1__STM_3.CHANNEL[3].CIR.R  /* Channel Interrupt */
#define RTU1__STM_3_CMP3     RTU1__STM_3.CHANNEL[3].CMP.R  /* Channel Compare */

/* SWT */
#define CE_SWT_0_CR          CE_SWT_0.CR.R                 /* Control */
#define CE_SWT_0_IR          CE_SWT_0.IR.R                 /* Interrupt */
#define CE_SWT_0_TO          CE_SWT_0.TO.R                 /* Timeout */
#define CE_SWT_0_WN          CE_SWT_0.WN.R                 /* Window */
#define CE_SWT_0_SR          CE_SWT_0.SR.R                 /* Service */
#define CE_SWT_0_CO          CE_SWT_0.CO.R                 /* Counter Output */
#define CE_SWT_0_SK          CE_SWT_0.SK.R                 /* Service Key */
#define CE_SWT_0_RRR         CE_SWT_0.RRR.R                /* Event Request */

#define CE_SWT_1_CR          CE_SWT_1.CR.R                 /* Control */
#define CE_SWT_1_IR          CE_SWT_1.IR.R                 /* Interrupt */
#define CE_SWT_1_TO          CE_SWT_1.TO.R                 /* Timeout */
#define CE_SWT_1_WN          CE_SWT_1.WN.R                 /* Window */
#define CE_SWT_1_SR          CE_SWT_1.SR.R                 /* Service */
#define CE_SWT_1_CO          CE_SWT_1.CO.R                 /* Counter Output */
#define CE_SWT_1_SK          CE_SWT_1.SK.R                 /* Service Key */
#define CE_SWT_1_RRR         CE_SWT_1.RRR.R                /* Event Request */

#define SMU__SWT_CR          SMU__SWT.CR.R                 /* Control */
#define SMU__SWT_IR          SMU__SWT.IR.R                 /* Interrupt */
#define SMU__SWT_TO          SMU__SWT.TO.R                 /* Timeout */
#define SMU__SWT_WN          SMU__SWT.WN.R                 /* Window */
#define SMU__SWT_SR          SMU__SWT.SR.R                 /* Service */
#define SMU__SWT_CO          SMU__SWT.CO.R                 /* Counter Output */
#define SMU__SWT_SK          SMU__SWT.SK.R                 /* Service Key */
#define SMU__SWT_RRR         SMU__SWT.RRR.R                /* Event Request */

#define RTU0__SWT_0_CR       RTU0__SWT_0.CR.R              /* Control */
#define RTU0__SWT_0_IR       RTU0__SWT_0.IR.R              /* Interrupt */
#define RTU0__SWT_0_TO       RTU0__SWT_0.TO.R              /* Timeout */
#define RTU0__SWT_0_WN       RTU0__SWT_0.WN.R              /* Window */
#define RTU0__SWT_0_SR       RTU0__SWT_0.SR.R              /* Service */
#define RTU0__SWT_0_CO       RTU0__SWT_0.CO.R              /* Counter Output */
#define RTU0__SWT_0_SK       RTU0__SWT_0.SK.R              /* Service Key */
#define RTU0__SWT_0_RRR      RTU0__SWT_0.RRR.R             /* Event Request */

#define RTU1__SWT_0_CR       RTU1__SWT_0.CR.R              /* Control */
#define RTU1__SWT_0_IR       RTU1__SWT_0.IR.R              /* Interrupt */
#define RTU1__SWT_0_TO       RTU1__SWT_0.TO.R              /* Timeout */
#define RTU1__SWT_0_WN       RTU1__SWT_0.WN.R              /* Window */
#define RTU1__SWT_0_SR       RTU1__SWT_0.SR.R              /* Service */
#define RTU1__SWT_0_CO       RTU1__SWT_0.CO.R              /* Counter Output */
#define RTU1__SWT_0_SK       RTU1__SWT_0.SK.R              /* Service Key */
#define RTU1__SWT_0_RRR      RTU1__SWT_0.RRR.R             /* Event Request */

#define RTU0__SWT_1_CR       RTU0__SWT_1.CR.R              /* Control */
#define RTU0__SWT_1_IR       RTU0__SWT_1.IR.R              /* Interrupt */
#define RTU0__SWT_1_TO       RTU0__SWT_1.TO.R              /* Timeout */
#define RTU0__SWT_1_WN       RTU0__SWT_1.WN.R              /* Window */
#define RTU0__SWT_1_SR       RTU0__SWT_1.SR.R              /* Service */
#define RTU0__SWT_1_CO       RTU0__SWT_1.CO.R              /* Counter Output */
#define RTU0__SWT_1_SK       RTU0__SWT_1.SK.R              /* Service Key */
#define RTU0__SWT_1_RRR      RTU0__SWT_1.RRR.R             /* Event Request */

#define RTU1__SWT_1_CR       RTU1__SWT_1.CR.R              /* Control */
#define RTU1__SWT_1_IR       RTU1__SWT_1.IR.R              /* Interrupt */
#define RTU1__SWT_1_TO       RTU1__SWT_1.TO.R              /* Timeout */
#define RTU1__SWT_1_WN       RTU1__SWT_1.WN.R              /* Window */
#define RTU1__SWT_1_SR       RTU1__SWT_1.SR.R              /* Service */
#define RTU1__SWT_1_CO       RTU1__SWT_1.CO.R              /* Counter Output */
#define RTU1__SWT_1_SK       RTU1__SWT_1.SK.R              /* Service Key */
#define RTU1__SWT_1_RRR      RTU1__SWT_1.RRR.R             /* Event Request */

#define RTU0__SWT_2_CR       RTU0__SWT_2.CR.R              /* Control */
#define RTU0__SWT_2_IR       RTU0__SWT_2.IR.R              /* Interrupt */
#define RTU0__SWT_2_TO       RTU0__SWT_2.TO.R              /* Timeout */
#define RTU0__SWT_2_WN       RTU0__SWT_2.WN.R              /* Window */
#define RTU0__SWT_2_SR       RTU0__SWT_2.SR.R              /* Service */
#define RTU0__SWT_2_CO       RTU0__SWT_2.CO.R              /* Counter Output */
#define RTU0__SWT_2_SK       RTU0__SWT_2.SK.R              /* Service Key */
#define RTU0__SWT_2_RRR      RTU0__SWT_2.RRR.R             /* Event Request */

#define RTU1__SWT_2_CR       RTU1__SWT_2.CR.R              /* Control */
#define RTU1__SWT_2_IR       RTU1__SWT_2.IR.R              /* Interrupt */
#define RTU1__SWT_2_TO       RTU1__SWT_2.TO.R              /* Timeout */
#define RTU1__SWT_2_WN       RTU1__SWT_2.WN.R              /* Window */
#define RTU1__SWT_2_SR       RTU1__SWT_2.SR.R              /* Service */
#define RTU1__SWT_2_CO       RTU1__SWT_2.CO.R              /* Counter Output */
#define RTU1__SWT_2_SK       RTU1__SWT_2.SK.R              /* Service Key */
#define RTU1__SWT_2_RRR      RTU1__SWT_2.RRR.R             /* Event Request */

#define RTU0__SWT_3_CR       RTU0__SWT_3.CR.R              /* Control */
#define RTU0__SWT_3_IR       RTU0__SWT_3.IR.R              /* Interrupt */
#define RTU0__SWT_3_TO       RTU0__SWT_3.TO.R              /* Timeout */
#define RTU0__SWT_3_WN       RTU0__SWT_3.WN.R              /* Window */
#define RTU0__SWT_3_SR       RTU0__SWT_3.SR.R              /* Service */
#define RTU0__SWT_3_CO       RTU0__SWT_3.CO.R              /* Counter Output */
#define RTU0__SWT_3_SK       RTU0__SWT_3.SK.R              /* Service Key */
#define RTU0__SWT_3_RRR      RTU0__SWT_3.RRR.R             /* Event Request */

#define RTU1__SWT_3_CR       RTU1__SWT_3.CR.R              /* Control */
#define RTU1__SWT_3_IR       RTU1__SWT_3.IR.R              /* Interrupt */
#define RTU1__SWT_3_TO       RTU1__SWT_3.TO.R              /* Timeout */
#define RTU1__SWT_3_WN       RTU1__SWT_3.WN.R              /* Window */
#define RTU1__SWT_3_SR       RTU1__SWT_3.SR.R              /* Service */
#define RTU1__SWT_3_CO       RTU1__SWT_3.CO.R              /* Counter Output */
#define RTU1__SWT_3_SK       RTU1__SWT_3.SK.R              /* Service Key */
#define RTU1__SWT_3_RRR      RTU1__SWT_3.RRR.R             /* Event Request */

#define RTU0__SWT_4_CR       RTU0__SWT_4.CR.R              /* Control */
#define RTU0__SWT_4_IR       RTU0__SWT_4.IR.R              /* Interrupt */
#define RTU0__SWT_4_TO       RTU0__SWT_4.TO.R              /* Timeout */
#define RTU0__SWT_4_WN       RTU0__SWT_4.WN.R              /* Window */
#define RTU0__SWT_4_SR       RTU0__SWT_4.SR.R              /* Service */
#define RTU0__SWT_4_CO       RTU0__SWT_4.CO.R              /* Counter Output */
#define RTU0__SWT_4_SK       RTU0__SWT_4.SK.R              /* Service Key */
#define RTU0__SWT_4_RRR      RTU0__SWT_4.RRR.R             /* Event Request */

#define RTU1__SWT_4_CR       RTU1__SWT_4.CR.R              /* Control */
#define RTU1__SWT_4_IR       RTU1__SWT_4.IR.R              /* Interrupt */
#define RTU1__SWT_4_TO       RTU1__SWT_4.TO.R              /* Timeout */
#define RTU1__SWT_4_WN       RTU1__SWT_4.WN.R              /* Window */
#define RTU1__SWT_4_SR       RTU1__SWT_4.SR.R              /* Service */
#define RTU1__SWT_4_CO       RTU1__SWT_4.CO.R              /* Counter Output */
#define RTU1__SWT_4_SK       RTU1__SWT_4.SK.R              /* Service Key */
#define RTU1__SWT_4_RRR      RTU1__SWT_4.RRR.R             /* Event Request */

/* SW_ETH_MAC_PORT0 */
#define NETC__SW0_ETH_MAC_PORT0_PM0_COMMAND_CONFIG NETC__SW0_ETH_MAC_PORT0.PM0_COMMAND_CONFIG.R /* Port MAC 0 Command and Configuration Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_MAC_ADDR_0 NETC__SW0_ETH_MAC_PORT0.PM0_MAC_ADDR_0.R /* Port MAC 0 MAC Address Register 0 */
#define NETC__SW0_ETH_MAC_PORT0_PM0_MAC_ADDR_1 NETC__SW0_ETH_MAC_PORT0.PM0_MAC_ADDR_1.R /* Port MAC 0 MAC Address Register 1 */
#define NETC__SW0_ETH_MAC_PORT0_PM0_MAXFRM NETC__SW0_ETH_MAC_PORT0.PM0_MAXFRM.R /* Port MAC 0 Maximum Frame Length Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_IEVENT NETC__SW0_ETH_MAC_PORT0.PM0_IEVENT.R /* Port MAC 0 Interrupt Event Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_IMASK NETC__SW0_ETH_MAC_PORT0.PM0_IMASK.R /* Port MAC 0 Interrupt Mask Register(INT_MASK) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_PAUSE_QUANTA NETC__SW0_ETH_MAC_PORT0.PM0_PAUSE_QUANTA.R /* Port MAC 0 Pause Quanta Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_PAUSE_THRESH NETC__SW0_ETH_MAC_PORT0.PM0_PAUSE_THRESH.R /* Port MAC 0 Pause Quanta Threshold Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RX_PAUSE_STATUS NETC__SW0_ETH_MAC_PORT0.PM0_RX_PAUSE_STATUS.R /* Port MAC 0 Receive Pause Status Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_SINGLE_STEP NETC__SW0_ETH_MAC_PORT0.PM0_SINGLE_STEP.R /* Port MAC 0 IEEE1588 Single-Step Control Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_HD_BACKOFF_ENTROPY NETC__SW0_ETH_MAC_PORT0.PM0_HD_BACKOFF_ENTROPY.R /* Port MAC 0 half-duplex backoff entropy register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_STATN_CONFIG NETC__SW0_ETH_MAC_PORT0.PM0_STATN_CONFIG.R /* Port MAC 0 Statistics Configuration Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_REOCTn NETC__SW0_ETH_MAC_PORT0.PM0_REOCTN.R /* Port MAC 0 Receive Ethernet Octets Counter(etherStatsOctetsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_ROCTn NETC__SW0_ETH_MAC_PORT0.PM0_ROCTN.R /* Port MAC 0 Receive Octets Counter(iflnOctetsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RXPFn NETC__SW0_ETH_MAC_PORT0.PM0_RXPFN.R /* Port MAC 0 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RFRMn NETC__SW0_ETH_MAC_PORT0.PM0_RFRMN.R /* Port MAC 0 Receive Frame Counter Register(aFramesReceivedOKn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RFCSn NETC__SW0_ETH_MAC_PORT0.PM0_RFCSN.R /* Port MAC 0 Receive Frame Check Sequence Error Counter Register() */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RVLANn NETC__SW0_ETH_MAC_PORT0.PM0_RVLANN.R /* Port MAC 0 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RERRn NETC__SW0_ETH_MAC_PORT0.PM0_RERRN.R /* Port MAC 0 Receive Frame Error Counter Register(ifInErrorsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RUCAn NETC__SW0_ETH_MAC_PORT0.PM0_RUCAN.R /* Port MAC 0 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RMCAn NETC__SW0_ETH_MAC_PORT0.PM0_RMCAN.R /* Port MAC 0 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RBCAn NETC__SW0_ETH_MAC_PORT0.PM0_RBCAN.R /* Port MAC 0 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RDRPn NETC__SW0_ETH_MAC_PORT0.PM0_RDRPN.R /* Port MAC 0 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RPKTn NETC__SW0_ETH_MAC_PORT0.PM0_RPKTN.R /* Port MAC 0 Receive Packets Counter Register(etherStatsPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RUNDn NETC__SW0_ETH_MAC_PORT0.PM0_RUNDN.R /* Port MAC 0 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_R64n NETC__SW0_ETH_MAC_PORT0.PM0_R64N.R /* Port MAC 0 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_R127n NETC__SW0_ETH_MAC_PORT0.PM0_R127N.R /* Port MAC 0 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_R255n NETC__SW0_ETH_MAC_PORT0.PM0_R255N.R /* Port MAC 0 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_R511n NETC__SW0_ETH_MAC_PORT0.PM0_R511N.R /* Port MAC 0 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_R1023n NETC__SW0_ETH_MAC_PORT0.PM0_R1023N.R /* Port MAC 0 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_R1522n NETC__SW0_ETH_MAC_PORT0.PM0_R1522N.R /* Port MAC 0 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_R1523Xn NETC__SW0_ETH_MAC_PORT0.PM0_R1523XN.R /* Port MAC 0 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_ROVRn NETC__SW0_ETH_MAC_PORT0.PM0_ROVRN.R /* Port MAC 0 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RJBRn NETC__SW0_ETH_MAC_PORT0.PM0_RJBRN.R /* Port MAC 0 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RFRGn NETC__SW0_ETH_MAC_PORT0.PM0_RFRGN.R /* Port MAC 0 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RCNPn NETC__SW0_ETH_MAC_PORT0.PM0_RCNPN.R /* Port MAC 0 Receive Control Packet Counter Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_RDRNTPn NETC__SW0_ETH_MAC_PORT0.PM0_RDRNTPN.R /* Port MAC 0 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TEOCTn NETC__SW0_ETH_MAC_PORT0.PM0_TEOCTN.R /* Port MAC 0 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TOCTn NETC__SW0_ETH_MAC_PORT0.PM0_TOCTN.R /* Port MAC 0 Transmit Octets Counter Register(ifOutOctetsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TXPFn NETC__SW0_ETH_MAC_PORT0.PM0_TXPFN.R /* Port MAC 0 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TFRMn NETC__SW0_ETH_MAC_PORT0.PM0_TFRMN.R /* Port MAC 0 Transmit Frame Counter Register(aFramesTransmittedOKn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TFCSn NETC__SW0_ETH_MAC_PORT0.PM0_TFCSN.R /* Port MAC 0 Transmit Frame Check Sequence Error Counter Register() */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TVLANn NETC__SW0_ETH_MAC_PORT0.PM0_TVLANN.R /* Port MAC 0 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TERRn NETC__SW0_ETH_MAC_PORT0.PM0_TERRN.R /* Port MAC 0 Transmit Frame Error Counter Register(ifOutErrorsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TUCAn NETC__SW0_ETH_MAC_PORT0.PM0_TUCAN.R /* Port MAC 0 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TMCAn NETC__SW0_ETH_MAC_PORT0.PM0_TMCAN.R /* Port MAC 0 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TBCAn NETC__SW0_ETH_MAC_PORT0.PM0_TBCAN.R /* Port MAC 0 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TPKTn NETC__SW0_ETH_MAC_PORT0.PM0_TPKTN.R /* Port MAC 0 Transmit Packets Counter Register(etherStatsPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TUNDn NETC__SW0_ETH_MAC_PORT0.PM0_TUNDN.R /* Port MAC 0 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_T64n NETC__SW0_ETH_MAC_PORT0.PM0_T64N.R /* Port MAC 0 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_T127n NETC__SW0_ETH_MAC_PORT0.PM0_T127N.R /* Port MAC 0 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_T255n NETC__SW0_ETH_MAC_PORT0.PM0_T255N.R /* Port MAC 0 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_T511n NETC__SW0_ETH_MAC_PORT0.PM0_T511N.R /* Port MAC 0 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_T1023n NETC__SW0_ETH_MAC_PORT0.PM0_T1023N.R /* Port MAC 0 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_T1522n NETC__SW0_ETH_MAC_PORT0.PM0_T1522N.R /* Port MAC 0 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_T1523Xn NETC__SW0_ETH_MAC_PORT0.PM0_T1523XN.R /* Port MAC 0 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TCNPn NETC__SW0_ETH_MAC_PORT0.PM0_TCNPN.R /* Port MAC 0 Transmit Control Packet Counter Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TDFRn NETC__SW0_ETH_MAC_PORT0.PM0_TDFRN.R /* Port MAC 0 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TMCOLn NETC__SW0_ETH_MAC_PORT0.PM0_TMCOLN.R /* Port MAC 0 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TSCOLn NETC__SW0_ETH_MAC_PORT0.PM0_TSCOLN.R /* Port MAC 0 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TLCOLn NETC__SW0_ETH_MAC_PORT0.PM0_TLCOLN.R /* Port MAC 0 Transmit Late Collision Counter(aLateCollisions) Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_TECOLn NETC__SW0_ETH_MAC_PORT0.PM0_TECOLN.R /* Port MAC 0 Transmit Excessive Collisions Counter Register */
#define NETC__SW0_ETH_MAC_PORT0_PM0_IF_MODE NETC__SW0_ETH_MAC_PORT0.PM0_IF_MODE.R /* Port MAC 0 Interface Mode Control Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_COMMAND_CONFIG NETC__SW0_ETH_MAC_PORT0.PM1_COMMAND_CONFIG.R /* Port MAC 1 Command and Configuration Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_MAC_ADDR_0 NETC__SW0_ETH_MAC_PORT0.PM1_MAC_ADDR_0.R /* Port MAC 1 MAC Address Register 0 */
#define NETC__SW0_ETH_MAC_PORT0_PM1_MAC_ADDR_1 NETC__SW0_ETH_MAC_PORT0.PM1_MAC_ADDR_1.R /* Port MAC 1 MAC Address Register 1 */
#define NETC__SW0_ETH_MAC_PORT0_PM1_MAXFRM NETC__SW0_ETH_MAC_PORT0.PM1_MAXFRM.R /* Port MAC 1 Maximum Frame Length Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_IEVENT NETC__SW0_ETH_MAC_PORT0.PM1_IEVENT.R /* Port MAC 1 Interrupt Event Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_IMASK NETC__SW0_ETH_MAC_PORT0.PM1_IMASK.R /* Port MAC 1 Interrupt Mask Register(INT_MASK) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_PAUSE_QUANTA NETC__SW0_ETH_MAC_PORT0.PM1_PAUSE_QUANTA.R /* Port MAC 1 Pause Quanta Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_PAUSE_THRESH NETC__SW0_ETH_MAC_PORT0.PM1_PAUSE_THRESH.R /* Port MAC 1 Pause Quanta Threshold Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RX_PAUSE_STATUS NETC__SW0_ETH_MAC_PORT0.PM1_RX_PAUSE_STATUS.R /* Port MAC 1 Receive Pause Status Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_SINGLE_STEP NETC__SW0_ETH_MAC_PORT0.PM1_SINGLE_STEP.R /* Port MAC 1 IEEE1588 Single-Step Control Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_HD_BACKOFF_ENTROPY NETC__SW0_ETH_MAC_PORT0.PM1_HD_BACKOFF_ENTROPY.R /* Port MAC 1 half-duplex backoff entropy register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_STATN_CONFIG NETC__SW0_ETH_MAC_PORT0.PM1_STATN_CONFIG.R /* Port MAC 1 Statistics Configuration Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_REOCTn NETC__SW0_ETH_MAC_PORT0.PM1_REOCTN.R /* Port MAC 1 Receive Ethernet Octets Counter(etherStatsOctetsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_ROCTn NETC__SW0_ETH_MAC_PORT0.PM1_ROCTN.R /* Port MAC 1 Receive Octets Counter(iflnOctetsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RXPFn NETC__SW0_ETH_MAC_PORT0.PM1_RXPFN.R /* Port MAC 1 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RFRMn NETC__SW0_ETH_MAC_PORT0.PM1_RFRMN.R /* Port MAC 1 Receive Frame Counter Register(aFramesReceivedOKn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RFCSn NETC__SW0_ETH_MAC_PORT0.PM1_RFCSN.R /* Port MAC 1 Receive Frame Check Sequence Error Counter Register() */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RVLANn NETC__SW0_ETH_MAC_PORT0.PM1_RVLANN.R /* Port MAC 1 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RERRn NETC__SW0_ETH_MAC_PORT0.PM1_RERRN.R /* Port MAC 1 Receive Frame Error Counter Register(ifInErrorsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RUCAn NETC__SW0_ETH_MAC_PORT0.PM1_RUCAN.R /* Port MAC 1 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RMCAn NETC__SW0_ETH_MAC_PORT0.PM1_RMCAN.R /* Port MAC 1 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RBCAn NETC__SW0_ETH_MAC_PORT0.PM1_RBCAN.R /* Port MAC 1 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RDRPn NETC__SW0_ETH_MAC_PORT0.PM1_RDRPN.R /* Port MAC 1 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RPKTn NETC__SW0_ETH_MAC_PORT0.PM1_RPKTN.R /* Port MAC 1 Receive Packets Counter Register(etherStatsPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RUNDn NETC__SW0_ETH_MAC_PORT0.PM1_RUNDN.R /* Port MAC 1 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_R64n NETC__SW0_ETH_MAC_PORT0.PM1_R64N.R /* Port MAC 1 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_R127n NETC__SW0_ETH_MAC_PORT0.PM1_R127N.R /* Port MAC 1 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_R255n NETC__SW0_ETH_MAC_PORT0.PM1_R255N.R /* Port MAC 1 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_R511n NETC__SW0_ETH_MAC_PORT0.PM1_R511N.R /* Port MAC 1 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_R1023n NETC__SW0_ETH_MAC_PORT0.PM1_R1023N.R /* Port MAC 1 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_R1522n NETC__SW0_ETH_MAC_PORT0.PM1_R1522N.R /* Port MAC 1 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_R1523Xn NETC__SW0_ETH_MAC_PORT0.PM1_R1523XN.R /* Port MAC 1 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_ROVRn NETC__SW0_ETH_MAC_PORT0.PM1_ROVRN.R /* Port MAC 1 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RJBRn NETC__SW0_ETH_MAC_PORT0.PM1_RJBRN.R /* Port MAC 1 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RFRGn NETC__SW0_ETH_MAC_PORT0.PM1_RFRGN.R /* Port MAC 1 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RCNPn NETC__SW0_ETH_MAC_PORT0.PM1_RCNPN.R /* Port MAC 1 Receive Control Packet Counter Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_RDRNTPn NETC__SW0_ETH_MAC_PORT0.PM1_RDRNTPN.R /* Port MAC 1 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TEOCTn NETC__SW0_ETH_MAC_PORT0.PM1_TEOCTN.R /* Port MAC 1 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TOCTn NETC__SW0_ETH_MAC_PORT0.PM1_TOCTN.R /* Port MAC 1 Transmit Octets Counter Register(ifOutOctetsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TXPFn NETC__SW0_ETH_MAC_PORT0.PM1_TXPFN.R /* Port MAC 1 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TFRMn NETC__SW0_ETH_MAC_PORT0.PM1_TFRMN.R /* Port MAC 1 Transmit Frame Counter Register(aFramesTransmittedOKn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TFCSn NETC__SW0_ETH_MAC_PORT0.PM1_TFCSN.R /* Port MAC 1 Transmit Frame Check Sequence Error Counter Register() */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TVLANn NETC__SW0_ETH_MAC_PORT0.PM1_TVLANN.R /* Port MAC 1 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TERRn NETC__SW0_ETH_MAC_PORT0.PM1_TERRN.R /* Port MAC 1 Transmit Frame Error Counter Register(ifOutErrorsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TUCAn NETC__SW0_ETH_MAC_PORT0.PM1_TUCAN.R /* Port MAC 1 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TMCAn NETC__SW0_ETH_MAC_PORT0.PM1_TMCAN.R /* Port MAC 1 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TBCAn NETC__SW0_ETH_MAC_PORT0.PM1_TBCAN.R /* Port MAC 1 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TPKTn NETC__SW0_ETH_MAC_PORT0.PM1_TPKTN.R /* Port MAC 1 Transmit Packets Counter Register(etherStatsPktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TUNDn NETC__SW0_ETH_MAC_PORT0.PM1_TUNDN.R /* Port MAC 1 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_T64n NETC__SW0_ETH_MAC_PORT0.PM1_T64N.R /* Port MAC 1 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_T127n NETC__SW0_ETH_MAC_PORT0.PM1_T127N.R /* Port MAC 1 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_T255n NETC__SW0_ETH_MAC_PORT0.PM1_T255N.R /* Port MAC 1 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_T511n NETC__SW0_ETH_MAC_PORT0.PM1_T511N.R /* Port MAC 1 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_T1023n NETC__SW0_ETH_MAC_PORT0.PM1_T1023N.R /* Port MAC 1 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_T1522n NETC__SW0_ETH_MAC_PORT0.PM1_T1522N.R /* Port MAC 1 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_T1523Xn NETC__SW0_ETH_MAC_PORT0.PM1_T1523XN.R /* Port MAC 1 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TCNPn NETC__SW0_ETH_MAC_PORT0.PM1_TCNPN.R /* Port MAC 1 Transmit Control Packet Counter Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TDFRn NETC__SW0_ETH_MAC_PORT0.PM1_TDFRN.R /* Port MAC 1 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TMCOLn NETC__SW0_ETH_MAC_PORT0.PM1_TMCOLN.R /* Port MAC 1 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TSCOLn NETC__SW0_ETH_MAC_PORT0.PM1_TSCOLN.R /* Port MAC 1 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TLCOLn NETC__SW0_ETH_MAC_PORT0.PM1_TLCOLN.R /* Port MAC 1 Transmit Late Collision Counter(aLateCollisions) Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_TECOLn NETC__SW0_ETH_MAC_PORT0.PM1_TECOLN.R /* Port MAC 1 Transmit Excessive Collisions Counter Register */
#define NETC__SW0_ETH_MAC_PORT0_PM1_IF_MODE NETC__SW0_ETH_MAC_PORT0.PM1_IF_MODE.R /* Port MAC 1 Interface Mode Control Register */
#define NETC__SW0_ETH_MAC_PORT0_MAC_MERGE_MMCSR NETC__SW0_ETH_MAC_PORT0.MAC_MERGE_MMCSR.R /* Port MAC Merge Control and Status Register */
#define NETC__SW0_ETH_MAC_PORT0_MAC_MERGE_MMFAECR NETC__SW0_ETH_MAC_PORT0.MAC_MERGE_MMFAECR.R /* Port MAC Merge Frame Assembly Error Count Register */
#define NETC__SW0_ETH_MAC_PORT0_MAC_MERGE_MMFSECR NETC__SW0_ETH_MAC_PORT0.MAC_MERGE_MMFSECR.R /* Port MAC Merge Frame SMD Error Count Register */
#define NETC__SW0_ETH_MAC_PORT0_MAC_MERGE_MMFAOCR NETC__SW0_ETH_MAC_PORT0.MAC_MERGE_MMFAOCR.R /* Port MAC Merge Frame Assembly OK Count Register */
#define NETC__SW0_ETH_MAC_PORT0_MAC_MERGE_MMFCRXR NETC__SW0_ETH_MAC_PORT0.MAC_MERGE_MMFCRXR.R /* Port MAC Merge Fragment Count RX Register */
#define NETC__SW0_ETH_MAC_PORT0_MAC_MERGE_MMFCTXR NETC__SW0_ETH_MAC_PORT0.MAC_MERGE_MMFCTXR.R /* Port MAC Merge Fragment Count TX Register */
#define NETC__SW0_ETH_MAC_PORT0_MAC_MERGE_MMHCR NETC__SW0_ETH_MAC_PORT0.MAC_MERGE_MMHCR.R /* Port MAC Merge Hold Count Register */
#define NETC__SW0_ETH_MAC_PORT0_PEMDIOCR NETC__SW0_ETH_MAC_PORT0.PEMDIOCR.R /* Port external MDIO configuration register */
#define NETC__SW0_ETH_MAC_PORT0_PEMDIOICR NETC__SW0_ETH_MAC_PORT0.PEMDIOICR.R /* Port external MDIO interface control register */
#define NETC__SW0_ETH_MAC_PORT0_PEMDIOIDR NETC__SW0_ETH_MAC_PORT0.PEMDIOIDR.R /* Port external MDIO interface data register */
#define NETC__SW0_ETH_MAC_PORT0_PEMDIORAR NETC__SW0_ETH_MAC_PORT0.PEMDIORAR.R /* Port external MDIO register address register */
#define NETC__SW0_ETH_MAC_PORT0_PEMDIOSR NETC__SW0_ETH_MAC_PORT0.PEMDIOSR.R /* Port external MDIO status register */
#define NETC__SW0_ETH_MAC_PORT0_PPSCR NETC__SW0_ETH_MAC_PORT0.PPSCR.R /* PHY status configuration register */
#define NETC__SW0_ETH_MAC_PORT0_PPSCTRLR NETC__SW0_ETH_MAC_PORT0.PPSCTRLR.R /* Port PHY status control register */
#define NETC__SW0_ETH_MAC_PORT0_PPSDR NETC__SW0_ETH_MAC_PORT0.PPSDR.R /* Port PHY status data register */
#define NETC__SW0_ETH_MAC_PORT0_PPSRAR NETC__SW0_ETH_MAC_PORT0.PPSRAR.R /* Port PHY status register address register */
#define NETC__SW0_ETH_MAC_PORT0_PPSER NETC__SW0_ETH_MAC_PORT0.PPSER.R /* Port PHY status event register */
#define NETC__SW0_ETH_MAC_PORT0_PPSMR NETC__SW0_ETH_MAC_PORT0.PPSMR.R /* Port PHY status mask register */

/* SW_ETH_MAC_PORT1 */
#define NETC__SW0_ETH_MAC_PORT1_PM0_COMMAND_CONFIG NETC__SW0_ETH_MAC_PORT1.PM0_COMMAND_CONFIG.R /* Port MAC 0 Command and Configuration Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_MAC_ADDR_0 NETC__SW0_ETH_MAC_PORT1.PM0_MAC_ADDR_0.R /* Port MAC 0 MAC Address Register 0 */
#define NETC__SW0_ETH_MAC_PORT1_PM0_MAC_ADDR_1 NETC__SW0_ETH_MAC_PORT1.PM0_MAC_ADDR_1.R /* Port MAC 0 MAC Address Register 1 */
#define NETC__SW0_ETH_MAC_PORT1_PM0_MAXFRM NETC__SW0_ETH_MAC_PORT1.PM0_MAXFRM.R /* Port MAC 0 Maximum Frame Length Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_IEVENT NETC__SW0_ETH_MAC_PORT1.PM0_IEVENT.R /* Port MAC 0 Interrupt Event Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_IMASK NETC__SW0_ETH_MAC_PORT1.PM0_IMASK.R /* Port MAC 0 Interrupt Mask Register(INT_MASK) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_PAUSE_QUANTA NETC__SW0_ETH_MAC_PORT1.PM0_PAUSE_QUANTA.R /* Port MAC 0 Pause Quanta Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_PAUSE_THRESH NETC__SW0_ETH_MAC_PORT1.PM0_PAUSE_THRESH.R /* Port MAC 0 Pause Quanta Threshold Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RX_PAUSE_STATUS NETC__SW0_ETH_MAC_PORT1.PM0_RX_PAUSE_STATUS.R /* Port MAC 0 Receive Pause Status Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_SINGLE_STEP NETC__SW0_ETH_MAC_PORT1.PM0_SINGLE_STEP.R /* Port MAC 0 IEEE1588 Single-Step Control Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_HD_BACKOFF_ENTROPY NETC__SW0_ETH_MAC_PORT1.PM0_HD_BACKOFF_ENTROPY.R /* Port MAC 0 half-duplex backoff entropy register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_STATN_CONFIG NETC__SW0_ETH_MAC_PORT1.PM0_STATN_CONFIG.R /* Port MAC 0 Statistics Configuration Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_REOCTn NETC__SW0_ETH_MAC_PORT1.PM0_REOCTN.R /* Port MAC 0 Receive Ethernet Octets Counter(etherStatsOctetsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_ROCTn NETC__SW0_ETH_MAC_PORT1.PM0_ROCTN.R /* Port MAC 0 Receive Octets Counter(iflnOctetsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RXPFn NETC__SW0_ETH_MAC_PORT1.PM0_RXPFN.R /* Port MAC 0 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RFRMn NETC__SW0_ETH_MAC_PORT1.PM0_RFRMN.R /* Port MAC 0 Receive Frame Counter Register(aFramesReceivedOKn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RFCSn NETC__SW0_ETH_MAC_PORT1.PM0_RFCSN.R /* Port MAC 0 Receive Frame Check Sequence Error Counter Register() */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RVLANn NETC__SW0_ETH_MAC_PORT1.PM0_RVLANN.R /* Port MAC 0 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RERRn NETC__SW0_ETH_MAC_PORT1.PM0_RERRN.R /* Port MAC 0 Receive Frame Error Counter Register(ifInErrorsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RUCAn NETC__SW0_ETH_MAC_PORT1.PM0_RUCAN.R /* Port MAC 0 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RMCAn NETC__SW0_ETH_MAC_PORT1.PM0_RMCAN.R /* Port MAC 0 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RBCAn NETC__SW0_ETH_MAC_PORT1.PM0_RBCAN.R /* Port MAC 0 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RDRPn NETC__SW0_ETH_MAC_PORT1.PM0_RDRPN.R /* Port MAC 0 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RPKTn NETC__SW0_ETH_MAC_PORT1.PM0_RPKTN.R /* Port MAC 0 Receive Packets Counter Register(etherStatsPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RUNDn NETC__SW0_ETH_MAC_PORT1.PM0_RUNDN.R /* Port MAC 0 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_R64n NETC__SW0_ETH_MAC_PORT1.PM0_R64N.R /* Port MAC 0 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_R127n NETC__SW0_ETH_MAC_PORT1.PM0_R127N.R /* Port MAC 0 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_R255n NETC__SW0_ETH_MAC_PORT1.PM0_R255N.R /* Port MAC 0 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_R511n NETC__SW0_ETH_MAC_PORT1.PM0_R511N.R /* Port MAC 0 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_R1023n NETC__SW0_ETH_MAC_PORT1.PM0_R1023N.R /* Port MAC 0 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_R1522n NETC__SW0_ETH_MAC_PORT1.PM0_R1522N.R /* Port MAC 0 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_R1523Xn NETC__SW0_ETH_MAC_PORT1.PM0_R1523XN.R /* Port MAC 0 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_ROVRn NETC__SW0_ETH_MAC_PORT1.PM0_ROVRN.R /* Port MAC 0 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RJBRn NETC__SW0_ETH_MAC_PORT1.PM0_RJBRN.R /* Port MAC 0 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RFRGn NETC__SW0_ETH_MAC_PORT1.PM0_RFRGN.R /* Port MAC 0 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RCNPn NETC__SW0_ETH_MAC_PORT1.PM0_RCNPN.R /* Port MAC 0 Receive Control Packet Counter Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_RDRNTPn NETC__SW0_ETH_MAC_PORT1.PM0_RDRNTPN.R /* Port MAC 0 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TEOCTn NETC__SW0_ETH_MAC_PORT1.PM0_TEOCTN.R /* Port MAC 0 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TOCTn NETC__SW0_ETH_MAC_PORT1.PM0_TOCTN.R /* Port MAC 0 Transmit Octets Counter Register(ifOutOctetsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TXPFn NETC__SW0_ETH_MAC_PORT1.PM0_TXPFN.R /* Port MAC 0 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TFRMn NETC__SW0_ETH_MAC_PORT1.PM0_TFRMN.R /* Port MAC 0 Transmit Frame Counter Register(aFramesTransmittedOKn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TFCSn NETC__SW0_ETH_MAC_PORT1.PM0_TFCSN.R /* Port MAC 0 Transmit Frame Check Sequence Error Counter Register() */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TVLANn NETC__SW0_ETH_MAC_PORT1.PM0_TVLANN.R /* Port MAC 0 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TERRn NETC__SW0_ETH_MAC_PORT1.PM0_TERRN.R /* Port MAC 0 Transmit Frame Error Counter Register(ifOutErrorsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TUCAn NETC__SW0_ETH_MAC_PORT1.PM0_TUCAN.R /* Port MAC 0 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TMCAn NETC__SW0_ETH_MAC_PORT1.PM0_TMCAN.R /* Port MAC 0 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TBCAn NETC__SW0_ETH_MAC_PORT1.PM0_TBCAN.R /* Port MAC 0 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TPKTn NETC__SW0_ETH_MAC_PORT1.PM0_TPKTN.R /* Port MAC 0 Transmit Packets Counter Register(etherStatsPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TUNDn NETC__SW0_ETH_MAC_PORT1.PM0_TUNDN.R /* Port MAC 0 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_T64n NETC__SW0_ETH_MAC_PORT1.PM0_T64N.R /* Port MAC 0 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_T127n NETC__SW0_ETH_MAC_PORT1.PM0_T127N.R /* Port MAC 0 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_T255n NETC__SW0_ETH_MAC_PORT1.PM0_T255N.R /* Port MAC 0 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_T511n NETC__SW0_ETH_MAC_PORT1.PM0_T511N.R /* Port MAC 0 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_T1023n NETC__SW0_ETH_MAC_PORT1.PM0_T1023N.R /* Port MAC 0 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_T1522n NETC__SW0_ETH_MAC_PORT1.PM0_T1522N.R /* Port MAC 0 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_T1523Xn NETC__SW0_ETH_MAC_PORT1.PM0_T1523XN.R /* Port MAC 0 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TCNPn NETC__SW0_ETH_MAC_PORT1.PM0_TCNPN.R /* Port MAC 0 Transmit Control Packet Counter Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TDFRn NETC__SW0_ETH_MAC_PORT1.PM0_TDFRN.R /* Port MAC 0 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TMCOLn NETC__SW0_ETH_MAC_PORT1.PM0_TMCOLN.R /* Port MAC 0 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TSCOLn NETC__SW0_ETH_MAC_PORT1.PM0_TSCOLN.R /* Port MAC 0 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TLCOLn NETC__SW0_ETH_MAC_PORT1.PM0_TLCOLN.R /* Port MAC 0 Transmit Late Collision Counter(aLateCollisions) Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_TECOLn NETC__SW0_ETH_MAC_PORT1.PM0_TECOLN.R /* Port MAC 0 Transmit Excessive Collisions Counter Register */
#define NETC__SW0_ETH_MAC_PORT1_PM0_IF_MODE NETC__SW0_ETH_MAC_PORT1.PM0_IF_MODE.R /* Port MAC 0 Interface Mode Control Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_COMMAND_CONFIG NETC__SW0_ETH_MAC_PORT1.PM1_COMMAND_CONFIG.R /* Port MAC 1 Command and Configuration Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_MAC_ADDR_0 NETC__SW0_ETH_MAC_PORT1.PM1_MAC_ADDR_0.R /* Port MAC 1 MAC Address Register 0 */
#define NETC__SW0_ETH_MAC_PORT1_PM1_MAC_ADDR_1 NETC__SW0_ETH_MAC_PORT1.PM1_MAC_ADDR_1.R /* Port MAC 1 MAC Address Register 1 */
#define NETC__SW0_ETH_MAC_PORT1_PM1_MAXFRM NETC__SW0_ETH_MAC_PORT1.PM1_MAXFRM.R /* Port MAC 1 Maximum Frame Length Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_IEVENT NETC__SW0_ETH_MAC_PORT1.PM1_IEVENT.R /* Port MAC 1 Interrupt Event Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_IMASK NETC__SW0_ETH_MAC_PORT1.PM1_IMASK.R /* Port MAC 1 Interrupt Mask Register(INT_MASK) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_PAUSE_QUANTA NETC__SW0_ETH_MAC_PORT1.PM1_PAUSE_QUANTA.R /* Port MAC 1 Pause Quanta Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_PAUSE_THRESH NETC__SW0_ETH_MAC_PORT1.PM1_PAUSE_THRESH.R /* Port MAC 1 Pause Quanta Threshold Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RX_PAUSE_STATUS NETC__SW0_ETH_MAC_PORT1.PM1_RX_PAUSE_STATUS.R /* Port MAC 1 Receive Pause Status Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_SINGLE_STEP NETC__SW0_ETH_MAC_PORT1.PM1_SINGLE_STEP.R /* Port MAC 1 IEEE1588 Single-Step Control Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_HD_BACKOFF_ENTROPY NETC__SW0_ETH_MAC_PORT1.PM1_HD_BACKOFF_ENTROPY.R /* Port MAC 1 half-duplex backoff entropy register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_STATN_CONFIG NETC__SW0_ETH_MAC_PORT1.PM1_STATN_CONFIG.R /* Port MAC 1 Statistics Configuration Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_REOCTn NETC__SW0_ETH_MAC_PORT1.PM1_REOCTN.R /* Port MAC 1 Receive Ethernet Octets Counter(etherStatsOctetsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_ROCTn NETC__SW0_ETH_MAC_PORT1.PM1_ROCTN.R /* Port MAC 1 Receive Octets Counter(iflnOctetsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RXPFn NETC__SW0_ETH_MAC_PORT1.PM1_RXPFN.R /* Port MAC 1 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RFRMn NETC__SW0_ETH_MAC_PORT1.PM1_RFRMN.R /* Port MAC 1 Receive Frame Counter Register(aFramesReceivedOKn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RFCSn NETC__SW0_ETH_MAC_PORT1.PM1_RFCSN.R /* Port MAC 1 Receive Frame Check Sequence Error Counter Register() */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RVLANn NETC__SW0_ETH_MAC_PORT1.PM1_RVLANN.R /* Port MAC 1 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RERRn NETC__SW0_ETH_MAC_PORT1.PM1_RERRN.R /* Port MAC 1 Receive Frame Error Counter Register(ifInErrorsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RUCAn NETC__SW0_ETH_MAC_PORT1.PM1_RUCAN.R /* Port MAC 1 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RMCAn NETC__SW0_ETH_MAC_PORT1.PM1_RMCAN.R /* Port MAC 1 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RBCAn NETC__SW0_ETH_MAC_PORT1.PM1_RBCAN.R /* Port MAC 1 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RDRPn NETC__SW0_ETH_MAC_PORT1.PM1_RDRPN.R /* Port MAC 1 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RPKTn NETC__SW0_ETH_MAC_PORT1.PM1_RPKTN.R /* Port MAC 1 Receive Packets Counter Register(etherStatsPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RUNDn NETC__SW0_ETH_MAC_PORT1.PM1_RUNDN.R /* Port MAC 1 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_R64n NETC__SW0_ETH_MAC_PORT1.PM1_R64N.R /* Port MAC 1 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_R127n NETC__SW0_ETH_MAC_PORT1.PM1_R127N.R /* Port MAC 1 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_R255n NETC__SW0_ETH_MAC_PORT1.PM1_R255N.R /* Port MAC 1 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_R511n NETC__SW0_ETH_MAC_PORT1.PM1_R511N.R /* Port MAC 1 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_R1023n NETC__SW0_ETH_MAC_PORT1.PM1_R1023N.R /* Port MAC 1 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_R1522n NETC__SW0_ETH_MAC_PORT1.PM1_R1522N.R /* Port MAC 1 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_R1523Xn NETC__SW0_ETH_MAC_PORT1.PM1_R1523XN.R /* Port MAC 1 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_ROVRn NETC__SW0_ETH_MAC_PORT1.PM1_ROVRN.R /* Port MAC 1 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RJBRn NETC__SW0_ETH_MAC_PORT1.PM1_RJBRN.R /* Port MAC 1 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RFRGn NETC__SW0_ETH_MAC_PORT1.PM1_RFRGN.R /* Port MAC 1 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RCNPn NETC__SW0_ETH_MAC_PORT1.PM1_RCNPN.R /* Port MAC 1 Receive Control Packet Counter Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_RDRNTPn NETC__SW0_ETH_MAC_PORT1.PM1_RDRNTPN.R /* Port MAC 1 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TEOCTn NETC__SW0_ETH_MAC_PORT1.PM1_TEOCTN.R /* Port MAC 1 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TOCTn NETC__SW0_ETH_MAC_PORT1.PM1_TOCTN.R /* Port MAC 1 Transmit Octets Counter Register(ifOutOctetsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TXPFn NETC__SW0_ETH_MAC_PORT1.PM1_TXPFN.R /* Port MAC 1 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TFRMn NETC__SW0_ETH_MAC_PORT1.PM1_TFRMN.R /* Port MAC 1 Transmit Frame Counter Register(aFramesTransmittedOKn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TFCSn NETC__SW0_ETH_MAC_PORT1.PM1_TFCSN.R /* Port MAC 1 Transmit Frame Check Sequence Error Counter Register() */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TVLANn NETC__SW0_ETH_MAC_PORT1.PM1_TVLANN.R /* Port MAC 1 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TERRn NETC__SW0_ETH_MAC_PORT1.PM1_TERRN.R /* Port MAC 1 Transmit Frame Error Counter Register(ifOutErrorsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TUCAn NETC__SW0_ETH_MAC_PORT1.PM1_TUCAN.R /* Port MAC 1 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TMCAn NETC__SW0_ETH_MAC_PORT1.PM1_TMCAN.R /* Port MAC 1 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TBCAn NETC__SW0_ETH_MAC_PORT1.PM1_TBCAN.R /* Port MAC 1 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TPKTn NETC__SW0_ETH_MAC_PORT1.PM1_TPKTN.R /* Port MAC 1 Transmit Packets Counter Register(etherStatsPktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TUNDn NETC__SW0_ETH_MAC_PORT1.PM1_TUNDN.R /* Port MAC 1 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_T64n NETC__SW0_ETH_MAC_PORT1.PM1_T64N.R /* Port MAC 1 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_T127n NETC__SW0_ETH_MAC_PORT1.PM1_T127N.R /* Port MAC 1 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_T255n NETC__SW0_ETH_MAC_PORT1.PM1_T255N.R /* Port MAC 1 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_T511n NETC__SW0_ETH_MAC_PORT1.PM1_T511N.R /* Port MAC 1 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_T1023n NETC__SW0_ETH_MAC_PORT1.PM1_T1023N.R /* Port MAC 1 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_T1522n NETC__SW0_ETH_MAC_PORT1.PM1_T1522N.R /* Port MAC 1 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_T1523Xn NETC__SW0_ETH_MAC_PORT1.PM1_T1523XN.R /* Port MAC 1 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TCNPn NETC__SW0_ETH_MAC_PORT1.PM1_TCNPN.R /* Port MAC 1 Transmit Control Packet Counter Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TDFRn NETC__SW0_ETH_MAC_PORT1.PM1_TDFRN.R /* Port MAC 1 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TMCOLn NETC__SW0_ETH_MAC_PORT1.PM1_TMCOLN.R /* Port MAC 1 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TSCOLn NETC__SW0_ETH_MAC_PORT1.PM1_TSCOLN.R /* Port MAC 1 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TLCOLn NETC__SW0_ETH_MAC_PORT1.PM1_TLCOLN.R /* Port MAC 1 Transmit Late Collision Counter(aLateCollisions) Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_TECOLn NETC__SW0_ETH_MAC_PORT1.PM1_TECOLN.R /* Port MAC 1 Transmit Excessive Collisions Counter Register */
#define NETC__SW0_ETH_MAC_PORT1_PM1_IF_MODE NETC__SW0_ETH_MAC_PORT1.PM1_IF_MODE.R /* Port MAC 1 Interface Mode Control Register */
#define NETC__SW0_ETH_MAC_PORT1_MAC_MERGE_MMCSR NETC__SW0_ETH_MAC_PORT1.MAC_MERGE_MMCSR.R /* Port MAC Merge Control and Status Register */
#define NETC__SW0_ETH_MAC_PORT1_MAC_MERGE_MMFAECR NETC__SW0_ETH_MAC_PORT1.MAC_MERGE_MMFAECR.R /* Port MAC Merge Frame Assembly Error Count Register */
#define NETC__SW0_ETH_MAC_PORT1_MAC_MERGE_MMFSECR NETC__SW0_ETH_MAC_PORT1.MAC_MERGE_MMFSECR.R /* Port MAC Merge Frame SMD Error Count Register */
#define NETC__SW0_ETH_MAC_PORT1_MAC_MERGE_MMFAOCR NETC__SW0_ETH_MAC_PORT1.MAC_MERGE_MMFAOCR.R /* Port MAC Merge Frame Assembly OK Count Register */
#define NETC__SW0_ETH_MAC_PORT1_MAC_MERGE_MMFCRXR NETC__SW0_ETH_MAC_PORT1.MAC_MERGE_MMFCRXR.R /* Port MAC Merge Fragment Count RX Register */
#define NETC__SW0_ETH_MAC_PORT1_MAC_MERGE_MMFCTXR NETC__SW0_ETH_MAC_PORT1.MAC_MERGE_MMFCTXR.R /* Port MAC Merge Fragment Count TX Register */
#define NETC__SW0_ETH_MAC_PORT1_MAC_MERGE_MMHCR NETC__SW0_ETH_MAC_PORT1.MAC_MERGE_MMHCR.R /* Port MAC Merge Hold Count Register */
#define NETC__SW0_ETH_MAC_PORT1_PEMDIOCR NETC__SW0_ETH_MAC_PORT1.PEMDIOCR.R /* Port external MDIO configuration register */
#define NETC__SW0_ETH_MAC_PORT1_PEMDIOICR NETC__SW0_ETH_MAC_PORT1.PEMDIOICR.R /* Port external MDIO interface control register */
#define NETC__SW0_ETH_MAC_PORT1_PEMDIOIDR NETC__SW0_ETH_MAC_PORT1.PEMDIOIDR.R /* Port external MDIO interface data register */
#define NETC__SW0_ETH_MAC_PORT1_PEMDIORAR NETC__SW0_ETH_MAC_PORT1.PEMDIORAR.R /* Port external MDIO register address register */
#define NETC__SW0_ETH_MAC_PORT1_PEMDIOSR NETC__SW0_ETH_MAC_PORT1.PEMDIOSR.R /* Port external MDIO status register */
#define NETC__SW0_ETH_MAC_PORT1_PPSCR NETC__SW0_ETH_MAC_PORT1.PPSCR.R /* PHY status configuration register */
#define NETC__SW0_ETH_MAC_PORT1_PPSCTRLR NETC__SW0_ETH_MAC_PORT1.PPSCTRLR.R /* Port PHY status control register */
#define NETC__SW0_ETH_MAC_PORT1_PPSDR NETC__SW0_ETH_MAC_PORT1.PPSDR.R /* Port PHY status data register */
#define NETC__SW0_ETH_MAC_PORT1_PPSRAR NETC__SW0_ETH_MAC_PORT1.PPSRAR.R /* Port PHY status register address register */
#define NETC__SW0_ETH_MAC_PORT1_PPSER NETC__SW0_ETH_MAC_PORT1.PPSER.R /* Port PHY status event register */
#define NETC__SW0_ETH_MAC_PORT1_PPSMR NETC__SW0_ETH_MAC_PORT1.PPSMR.R /* Port PHY status mask register */

/* SW_PORT0 */
#define NETC__SW0_PORT0_PCAPR NETC__SW0_PORT0.PCAPR.R      /* Port capability register */
#define NETC__SW0_PORT0_PMCAPR NETC__SW0_PORT0.PMCAPR.R    /* Port MAC capability register */
#define NETC__SW0_PORT0_PIOCAPR NETC__SW0_PORT0.PIOCAPR.R  /* Port I/O capability register */
#define NETC__SW0_PORT0_PCR  NETC__SW0_PORT0.PCR.R         /* Port configuration register */
#define NETC__SW0_PORT0_PMAR0 NETC__SW0_PORT0.PMAR0.R      /* Port MAC address register 0 */
#define NETC__SW0_PORT0_PMAR1 NETC__SW0_PORT0.PMAR1.R      /* Port MAC address register 1 */
#define NETC__SW0_PORT0_PTAR NETC__SW0_PORT0.PTAR.R        /* Port TPID acceptance register */
#define NETC__SW0_PORT0_PQOSMR NETC__SW0_PORT0.PQOSMR.R    /* Port QoS mode register */
#define NETC__SW0_PORT0_PQOR NETC__SW0_PORT0.PQOR.R        /* Port Queue Operational register */
#define NETC__SW0_PORT0_PPCR NETC__SW0_PORT0.PPCR.R        /* Port parser configuration register */
#define NETC__SW0_PORT0_PIPFCR NETC__SW0_PORT0.PIPFCR.R    /* Port ingress port filter configuration register */
#define NETC__SW0_PORT0_PSGCR NETC__SW0_PORT0.PSGCR.R      /* Port stream gate configuration register */
#define NETC__SW0_PORT0_POR  NETC__SW0_PORT0.PORr.R        /* Port operational register */
#define NETC__SW0_PORT0_PSR  NETC__SW0_PORT0.PSR.R         /* Port status register */
#define NETC__SW0_PORT0_PRXSDUOR NETC__SW0_PORT0.PRXSDUOR.R /* Port receive SDU overhead register */
#define NETC__SW0_PORT0_PTXSDUOR NETC__SW0_PORT0.PTXSDUOR.R /* Port transmit SDU overhead register */
#define NETC__SW0_PORT0_PTGSCR NETC__SW0_PORT0.PTGSCR.R    /* Port time gate scheduling control register */
#define NETC__SW0_PORT0_PTGAGLSR NETC__SW0_PORT0.PTGAGLSR.R /* Port time gate scheduling admin gate list status register */
#define NETC__SW0_PORT0_PTGAGLLR NETC__SW0_PORT0.PTGAGLLR.R /* Port time gate scheduling admin gate list length register */
#define NETC__SW0_PORT0_PTGOGLLR NETC__SW0_PORT0.PTGOGLLR.R /* Port time gating operational gate list length register */
#define NETC__SW0_PORT0_PTGSHAR NETC__SW0_PORT0.PTGSHAR.R  /* Port time gate scheduling hold advance register */
#define NETC__SW0_PORT0_PTGSRAR NETC__SW0_PORT0.PTGSRAR.R  /* Port time gate scheduling release advance register */
#define NETC__SW0_PORT0_PTGSHCR NETC__SW0_PORT0.PTGSHCR.R  /* Port time gate scheduling hold configuration register */
#define NETC__SW0_PORT0_PFPCR NETC__SW0_PORT0.PFPCR.R      /* Port frame preemption configuration register */
#define NETC__SW0_PORT0_PRXDCR NETC__SW0_PORT0.PRXDCR.R    /* Port Rx discard count register */
#define NETC__SW0_PORT0_PRXDCRR0 NETC__SW0_PORT0.PRXDCRR0.R /* Port Rx discard count reason register 0 */
#define NETC__SW0_PORT0_PRXDCRR1 NETC__SW0_PORT0.PRXDCRR1.R /* Port Rx discard count reason register 1 */
#define NETC__SW0_PORT0_PTXDCR NETC__SW0_PORT0.PTXDCR.R    /* Port Tx discard count register */
#define NETC__SW0_PORT0_PTXDCRR0 NETC__SW0_PORT0.PTXDCRR0.R /* Port Tx discard count reason register 0 */
#define NETC__SW0_PORT0_PTXDCRR1 NETC__SW0_PORT0.PTXDCRR1.R /* Port Tx discard count reason register 1 */
#define NETC__SW0_PORT0_PTGSTC0SR NETC__SW0_PORT0.TCT_NUM[0].PTGSTCSR.R /* Port time gate scheduling traffic class 0 status register */
#define NETC__SW0_PORT0_PTC0TMSDUR NETC__SW0_PORT0.TCT_NUM[0].PTCTMSDUR.R /* Port traffic class 0 transmit maximum SDU register */
#define NETC__SW0_PORT0_PTC0CBSR0 NETC__SW0_PORT0.TCT_NUM[0].PTCCBSR0.R /* Port transmit traffic class 0 credit based shaper register 0 */
#define NETC__SW0_PORT0_PTC0CBSR1 NETC__SW0_PORT0.TCT_NUM[0].PTCCBSR1.R /* Port traffic class 0 credit based shaper register 1 */
#define NETC__SW0_PORT0_PTGSTC1SR NETC__SW0_PORT0.TCT_NUM[1].PTGSTCSR.R /* Port time gate scheduling traffic class 1 status register */
#define NETC__SW0_PORT0_PTC1TMSDUR NETC__SW0_PORT0.TCT_NUM[1].PTCTMSDUR.R /* Port traffic class 1 transmit maximum SDU register */
#define NETC__SW0_PORT0_PTC1CBSR0 NETC__SW0_PORT0.TCT_NUM[1].PTCCBSR0.R /* Port transmit traffic class 1 credit based shaper register 0 */
#define NETC__SW0_PORT0_PTC1CBSR1 NETC__SW0_PORT0.TCT_NUM[1].PTCCBSR1.R /* Port traffic class 1 credit based shaper register 1 */
#define NETC__SW0_PORT0_PTGSTC2SR NETC__SW0_PORT0.TCT_NUM[2].PTGSTCSR.R /* Port time gate scheduling traffic class 2 status register */
#define NETC__SW0_PORT0_PTC2TMSDUR NETC__SW0_PORT0.TCT_NUM[2].PTCTMSDUR.R /* Port traffic class 2 transmit maximum SDU register */
#define NETC__SW0_PORT0_PTC2CBSR0 NETC__SW0_PORT0.TCT_NUM[2].PTCCBSR0.R /* Port transmit traffic class 2 credit based shaper register 0 */
#define NETC__SW0_PORT0_PTC2CBSR1 NETC__SW0_PORT0.TCT_NUM[2].PTCCBSR1.R /* Port traffic class 2 credit based shaper register 1 */
#define NETC__SW0_PORT0_PTGSTC3SR NETC__SW0_PORT0.TCT_NUM[3].PTGSTCSR.R /* Port time gate scheduling traffic class 3 status register */
#define NETC__SW0_PORT0_PTC3TMSDUR NETC__SW0_PORT0.TCT_NUM[3].PTCTMSDUR.R /* Port traffic class 3 transmit maximum SDU register */
#define NETC__SW0_PORT0_PTC3CBSR0 NETC__SW0_PORT0.TCT_NUM[3].PTCCBSR0.R /* Port transmit traffic class 3 credit based shaper register 0 */
#define NETC__SW0_PORT0_PTC3CBSR1 NETC__SW0_PORT0.TCT_NUM[3].PTCCBSR1.R /* Port traffic class 3 credit based shaper register 1 */
#define NETC__SW0_PORT0_PTGSTC4SR NETC__SW0_PORT0.TCT_NUM[4].PTGSTCSR.R /* Port time gate scheduling traffic class 4 status register */
#define NETC__SW0_PORT0_PTC4TMSDUR NETC__SW0_PORT0.TCT_NUM[4].PTCTMSDUR.R /* Port traffic class 4 transmit maximum SDU register */
#define NETC__SW0_PORT0_PTC4CBSR0 NETC__SW0_PORT0.TCT_NUM[4].PTCCBSR0.R /* Port transmit traffic class 4 credit based shaper register 0 */
#define NETC__SW0_PORT0_PTC4CBSR1 NETC__SW0_PORT0.TCT_NUM[4].PTCCBSR1.R /* Port traffic class 4 credit based shaper register 1 */
#define NETC__SW0_PORT0_PTGSTC5SR NETC__SW0_PORT0.TCT_NUM[5].PTGSTCSR.R /* Port time gate scheduling traffic class 5 status register */
#define NETC__SW0_PORT0_PTC5TMSDUR NETC__SW0_PORT0.TCT_NUM[5].PTCTMSDUR.R /* Port traffic class 5 transmit maximum SDU register */
#define NETC__SW0_PORT0_PTC5CBSR0 NETC__SW0_PORT0.TCT_NUM[5].PTCCBSR0.R /* Port transmit traffic class 5 credit based shaper register 0 */
#define NETC__SW0_PORT0_PTC5CBSR1 NETC__SW0_PORT0.TCT_NUM[5].PTCCBSR1.R /* Port traffic class 5 credit based shaper register 1 */
#define NETC__SW0_PORT0_PTGSTC6SR NETC__SW0_PORT0.TCT_NUM[6].PTGSTCSR.R /* Port time gate scheduling traffic class 6 status register */
#define NETC__SW0_PORT0_PTC6TMSDUR NETC__SW0_PORT0.TCT_NUM[6].PTCTMSDUR.R /* Port traffic class 6 transmit maximum SDU register */
#define NETC__SW0_PORT0_PTC6CBSR0 NETC__SW0_PORT0.TCT_NUM[6].PTCCBSR0.R /* Port transmit traffic class 6 credit based shaper register 0 */
#define NETC__SW0_PORT0_PTC6CBSR1 NETC__SW0_PORT0.TCT_NUM[6].PTCCBSR1.R /* Port traffic class 6 credit based shaper register 1 */
#define NETC__SW0_PORT0_PTGSTC7SR NETC__SW0_PORT0.TCT_NUM[7].PTGSTCSR.R /* Port time gate scheduling traffic class 7 status register */
#define NETC__SW0_PORT0_PTC7TMSDUR NETC__SW0_PORT0.TCT_NUM[7].PTCTMSDUR.R /* Port traffic class 7 transmit maximum SDU register */
#define NETC__SW0_PORT0_PTC7CBSR0 NETC__SW0_PORT0.TCT_NUM[7].PTCCBSR0.R /* Port transmit traffic class 7 credit based shaper register 0 */
#define NETC__SW0_PORT0_PTC7CBSR1 NETC__SW0_PORT0.TCT_NUM[7].PTCCBSR1.R /* Port traffic class 7 credit based shaper register 1 */
#define NETC__SW0_PORT0_PBPMCR0 NETC__SW0_PORT0.PBPMCR0.R  /* Port buffer pool mapping configuration register 0 */
#define NETC__SW0_PORT0_PBPMCR1 NETC__SW0_PORT0.PBPMCR1.R  /* Port buffer pool mapping configuration register 1 */
#define NETC__SW0_PORT0_PPCPDEIMR NETC__SW0_PORT0.PPCPDEIMR.R /* Port PCP DEI mapping register */
#define NETC__SW0_PORT0_PMCR NETC__SW0_PORT0.PMCR.R        /* Port mirror configuration register */
#define NETC__SW0_PORT0_PCTFCR NETC__SW0_PORT0.PCTFCR.R    /* Port cut through forwarding configuration register */
#define NETC__SW0_PORT0_PLANIDCR NETC__SW0_PORT0.PLANIDCR.R /* Port LANID configuration register */
#define NETC__SW0_PORT0_PISIDCR NETC__SW0_PORT0.PISIDCR.R  /* Port ingress stream identification configuration register */
#define NETC__SW0_PORT0_PFMCR NETC__SW0_PORT0.PFMCR.R      /* Port frame modification configuration register */
#define NETC__SW0_PORT0_PIPV2QMR0 NETC__SW0_PORT0.PIPV2QMR0.R /* Port IPV to queue mapping register 0 */
#define NETC__SW0_PORT0_PTCMINLR NETC__SW0_PORT0.PTCMINLR.R /* Port time capture minimum latency register */
#define NETC__SW0_PORT0_PTCMAXLR NETC__SW0_PORT0.PTCMAXLR.R /* Port time capture maximum latency register */
#define NETC__SW0_PORT0_BPCR NETC__SW0_PORT0.BPCR.R        /* Bridge port configuration register */
#define NETC__SW0_PORT0_BPDVR NETC__SW0_PORT0.BPDVR.R      /* Bridge port default VLAN register */
#define NETC__SW0_PORT0_BPSTGSR NETC__SW0_PORT0.BPSTGSR.R  /* Bridge port spanning tree group state register */
#define NETC__SW0_PORT0_BPSCR0 NETC__SW0_PORT0.BPSCR0.R    /* Bridge port storm control register 0 */
#define NETC__SW0_PORT0_BPSCR1 NETC__SW0_PORT0.BPSCR1.R    /* Bridge port storm control register 1 */
#define NETC__SW0_PORT0_BPOR NETC__SW0_PORT0.BPOR.R        /* Bridge port operational register */
#define NETC__SW0_PORT0_BPDCR NETC__SW0_PORT0.BPDCR.R      /* Bridge port discard count register */
#define NETC__SW0_PORT0_BPDCRR0 NETC__SW0_PORT0.BPDCRR0.R  /* Bridge port discard count reason register 0 */
#define NETC__SW0_PORT0_BPDCRR1 NETC__SW0_PORT0.BPDCRR1.R  /* Bridge port discard count reason register 1 */
#define NETC__SW0_PORT0_BPMLFSR NETC__SW0_PORT0.BPMLFSR.R  /* Bridge port MAC learning failure status register */

/* SW_PORT1 */
#define NETC__SW0_PORT1_PCAPR NETC__SW0_PORT1.PCAPR.R      /* Port capability register */
#define NETC__SW0_PORT1_PMCAPR NETC__SW0_PORT1.PMCAPR.R    /* Port MAC capability register */
#define NETC__SW0_PORT1_PIOCAPR NETC__SW0_PORT1.PIOCAPR.R  /* Port I/O capability register */
#define NETC__SW0_PORT1_PCR  NETC__SW0_PORT1.PCR.R         /* Port configuration register */
#define NETC__SW0_PORT1_PMAR0 NETC__SW0_PORT1.PMAR0.R      /* Port MAC address register 0 */
#define NETC__SW0_PORT1_PMAR1 NETC__SW0_PORT1.PMAR1.R      /* Port MAC address register 1 */
#define NETC__SW0_PORT1_PTAR NETC__SW0_PORT1.PTAR.R        /* Port TPID acceptance register */
#define NETC__SW0_PORT1_PQOSMR NETC__SW0_PORT1.PQOSMR.R    /* Port QoS mode register */
#define NETC__SW0_PORT1_PQOR NETC__SW0_PORT1.PQOR.R        /* Port Queue Operational register */
#define NETC__SW0_PORT1_PPCR NETC__SW0_PORT1.PPCR.R        /* Port parser configuration register */
#define NETC__SW0_PORT1_PIPFCR NETC__SW0_PORT1.PIPFCR.R    /* Port ingress port filter configuration register */
#define NETC__SW0_PORT1_PSGCR NETC__SW0_PORT1.PSGCR.R      /* Port stream gate configuration register */
#define NETC__SW0_PORT1_POR  NETC__SW0_PORT1.PORr.R        /* Port operational register */
#define NETC__SW0_PORT1_PSR  NETC__SW0_PORT1.PSR.R         /* Port status register */
#define NETC__SW0_PORT1_PRXSDUOR NETC__SW0_PORT1.PRXSDUOR.R /* Port receive SDU overhead register */
#define NETC__SW0_PORT1_PTXSDUOR NETC__SW0_PORT1.PTXSDUOR.R /* Port transmit SDU overhead register */
#define NETC__SW0_PORT1_PTGSCR NETC__SW0_PORT1.PTGSCR.R    /* Port time gate scheduling control register */
#define NETC__SW0_PORT1_PTGAGLSR NETC__SW0_PORT1.PTGAGLSR.R /* Port time gate scheduling admin gate list status register */
#define NETC__SW0_PORT1_PTGAGLLR NETC__SW0_PORT1.PTGAGLLR.R /* Port time gate scheduling admin gate list length register */
#define NETC__SW0_PORT1_PTGOGLLR NETC__SW0_PORT1.PTGOGLLR.R /* Port time gating operational gate list length register */
#define NETC__SW0_PORT1_PTGSHAR NETC__SW0_PORT1.PTGSHAR.R  /* Port time gate scheduling hold advance register */
#define NETC__SW0_PORT1_PTGSRAR NETC__SW0_PORT1.PTGSRAR.R  /* Port time gate scheduling release advance register */
#define NETC__SW0_PORT1_PTGSHCR NETC__SW0_PORT1.PTGSHCR.R  /* Port time gate scheduling hold configuration register */
#define NETC__SW0_PORT1_PFPCR NETC__SW0_PORT1.PFPCR.R      /* Port frame preemption configuration register */
#define NETC__SW0_PORT1_PRXDCR NETC__SW0_PORT1.PRXDCR.R    /* Port Rx discard count register */
#define NETC__SW0_PORT1_PRXDCRR0 NETC__SW0_PORT1.PRXDCRR0.R /* Port Rx discard count reason register 0 */
#define NETC__SW0_PORT1_PRXDCRR1 NETC__SW0_PORT1.PRXDCRR1.R /* Port Rx discard count reason register 1 */
#define NETC__SW0_PORT1_PTXDCR NETC__SW0_PORT1.PTXDCR.R    /* Port Tx discard count register */
#define NETC__SW0_PORT1_PTXDCRR0 NETC__SW0_PORT1.PTXDCRR0.R /* Port Tx discard count reason register 0 */
#define NETC__SW0_PORT1_PTXDCRR1 NETC__SW0_PORT1.PTXDCRR1.R /* Port Tx discard count reason register 1 */
#define NETC__SW0_PORT1_PTGSTC0SR NETC__SW0_PORT1.TCT_NUM[0].PTGSTCSR.R /* Port time gate scheduling traffic class 0 status register */
#define NETC__SW0_PORT1_PTC0TMSDUR NETC__SW0_PORT1.TCT_NUM[0].PTCTMSDUR.R /* Port traffic class 0 transmit maximum SDU register */
#define NETC__SW0_PORT1_PTC0CBSR0 NETC__SW0_PORT1.TCT_NUM[0].PTCCBSR0.R /* Port transmit traffic class 0 credit based shaper register 0 */
#define NETC__SW0_PORT1_PTC0CBSR1 NETC__SW0_PORT1.TCT_NUM[0].PTCCBSR1.R /* Port traffic class 0 credit based shaper register 1 */
#define NETC__SW0_PORT1_PTGSTC1SR NETC__SW0_PORT1.TCT_NUM[1].PTGSTCSR.R /* Port time gate scheduling traffic class 1 status register */
#define NETC__SW0_PORT1_PTC1TMSDUR NETC__SW0_PORT1.TCT_NUM[1].PTCTMSDUR.R /* Port traffic class 1 transmit maximum SDU register */
#define NETC__SW0_PORT1_PTC1CBSR0 NETC__SW0_PORT1.TCT_NUM[1].PTCCBSR0.R /* Port transmit traffic class 1 credit based shaper register 0 */
#define NETC__SW0_PORT1_PTC1CBSR1 NETC__SW0_PORT1.TCT_NUM[1].PTCCBSR1.R /* Port traffic class 1 credit based shaper register 1 */
#define NETC__SW0_PORT1_PTGSTC2SR NETC__SW0_PORT1.TCT_NUM[2].PTGSTCSR.R /* Port time gate scheduling traffic class 2 status register */
#define NETC__SW0_PORT1_PTC2TMSDUR NETC__SW0_PORT1.TCT_NUM[2].PTCTMSDUR.R /* Port traffic class 2 transmit maximum SDU register */
#define NETC__SW0_PORT1_PTC2CBSR0 NETC__SW0_PORT1.TCT_NUM[2].PTCCBSR0.R /* Port transmit traffic class 2 credit based shaper register 0 */
#define NETC__SW0_PORT1_PTC2CBSR1 NETC__SW0_PORT1.TCT_NUM[2].PTCCBSR1.R /* Port traffic class 2 credit based shaper register 1 */
#define NETC__SW0_PORT1_PTGSTC3SR NETC__SW0_PORT1.TCT_NUM[3].PTGSTCSR.R /* Port time gate scheduling traffic class 3 status register */
#define NETC__SW0_PORT1_PTC3TMSDUR NETC__SW0_PORT1.TCT_NUM[3].PTCTMSDUR.R /* Port traffic class 3 transmit maximum SDU register */
#define NETC__SW0_PORT1_PTC3CBSR0 NETC__SW0_PORT1.TCT_NUM[3].PTCCBSR0.R /* Port transmit traffic class 3 credit based shaper register 0 */
#define NETC__SW0_PORT1_PTC3CBSR1 NETC__SW0_PORT1.TCT_NUM[3].PTCCBSR1.R /* Port traffic class 3 credit based shaper register 1 */
#define NETC__SW0_PORT1_PTGSTC4SR NETC__SW0_PORT1.TCT_NUM[4].PTGSTCSR.R /* Port time gate scheduling traffic class 4 status register */
#define NETC__SW0_PORT1_PTC4TMSDUR NETC__SW0_PORT1.TCT_NUM[4].PTCTMSDUR.R /* Port traffic class 4 transmit maximum SDU register */
#define NETC__SW0_PORT1_PTC4CBSR0 NETC__SW0_PORT1.TCT_NUM[4].PTCCBSR0.R /* Port transmit traffic class 4 credit based shaper register 0 */
#define NETC__SW0_PORT1_PTC4CBSR1 NETC__SW0_PORT1.TCT_NUM[4].PTCCBSR1.R /* Port traffic class 4 credit based shaper register 1 */
#define NETC__SW0_PORT1_PTGSTC5SR NETC__SW0_PORT1.TCT_NUM[5].PTGSTCSR.R /* Port time gate scheduling traffic class 5 status register */
#define NETC__SW0_PORT1_PTC5TMSDUR NETC__SW0_PORT1.TCT_NUM[5].PTCTMSDUR.R /* Port traffic class 5 transmit maximum SDU register */
#define NETC__SW0_PORT1_PTC5CBSR0 NETC__SW0_PORT1.TCT_NUM[5].PTCCBSR0.R /* Port transmit traffic class 5 credit based shaper register 0 */
#define NETC__SW0_PORT1_PTC5CBSR1 NETC__SW0_PORT1.TCT_NUM[5].PTCCBSR1.R /* Port traffic class 5 credit based shaper register 1 */
#define NETC__SW0_PORT1_PTGSTC6SR NETC__SW0_PORT1.TCT_NUM[6].PTGSTCSR.R /* Port time gate scheduling traffic class 6 status register */
#define NETC__SW0_PORT1_PTC6TMSDUR NETC__SW0_PORT1.TCT_NUM[6].PTCTMSDUR.R /* Port traffic class 6 transmit maximum SDU register */
#define NETC__SW0_PORT1_PTC6CBSR0 NETC__SW0_PORT1.TCT_NUM[6].PTCCBSR0.R /* Port transmit traffic class 6 credit based shaper register 0 */
#define NETC__SW0_PORT1_PTC6CBSR1 NETC__SW0_PORT1.TCT_NUM[6].PTCCBSR1.R /* Port traffic class 6 credit based shaper register 1 */
#define NETC__SW0_PORT1_PTGSTC7SR NETC__SW0_PORT1.TCT_NUM[7].PTGSTCSR.R /* Port time gate scheduling traffic class 7 status register */
#define NETC__SW0_PORT1_PTC7TMSDUR NETC__SW0_PORT1.TCT_NUM[7].PTCTMSDUR.R /* Port traffic class 7 transmit maximum SDU register */
#define NETC__SW0_PORT1_PTC7CBSR0 NETC__SW0_PORT1.TCT_NUM[7].PTCCBSR0.R /* Port transmit traffic class 7 credit based shaper register 0 */
#define NETC__SW0_PORT1_PTC7CBSR1 NETC__SW0_PORT1.TCT_NUM[7].PTCCBSR1.R /* Port traffic class 7 credit based shaper register 1 */
#define NETC__SW0_PORT1_PBPMCR0 NETC__SW0_PORT1.PBPMCR0.R  /* Port buffer pool mapping configuration register 0 */
#define NETC__SW0_PORT1_PBPMCR1 NETC__SW0_PORT1.PBPMCR1.R  /* Port buffer pool mapping configuration register 1 */
#define NETC__SW0_PORT1_PPCPDEIMR NETC__SW0_PORT1.PPCPDEIMR.R /* Port PCP DEI mapping register */
#define NETC__SW0_PORT1_PMCR NETC__SW0_PORT1.PMCR.R        /* Port mirror configuration register */
#define NETC__SW0_PORT1_PCTFCR NETC__SW0_PORT1.PCTFCR.R    /* Port cut through forwarding configuration register */
#define NETC__SW0_PORT1_PLANIDCR NETC__SW0_PORT1.PLANIDCR.R /* Port LANID configuration register */
#define NETC__SW0_PORT1_PISIDCR NETC__SW0_PORT1.PISIDCR.R  /* Port ingress stream identification configuration register */
#define NETC__SW0_PORT1_PFMCR NETC__SW0_PORT1.PFMCR.R      /* Port frame modification configuration register */
#define NETC__SW0_PORT1_PIPV2QMR0 NETC__SW0_PORT1.PIPV2QMR0.R /* Port IPV to queue mapping register 0 */
#define NETC__SW0_PORT1_PTCMINLR NETC__SW0_PORT1.PTCMINLR.R /* Port time capture minimum latency register */
#define NETC__SW0_PORT1_PTCMAXLR NETC__SW0_PORT1.PTCMAXLR.R /* Port time capture maximum latency register */
#define NETC__SW0_PORT1_BPCR NETC__SW0_PORT1.BPCR.R        /* Bridge port configuration register */
#define NETC__SW0_PORT1_BPDVR NETC__SW0_PORT1.BPDVR.R      /* Bridge port default VLAN register */
#define NETC__SW0_PORT1_BPSTGSR NETC__SW0_PORT1.BPSTGSR.R  /* Bridge port spanning tree group state register */
#define NETC__SW0_PORT1_BPSCR0 NETC__SW0_PORT1.BPSCR0.R    /* Bridge port storm control register 0 */
#define NETC__SW0_PORT1_BPSCR1 NETC__SW0_PORT1.BPSCR1.R    /* Bridge port storm control register 1 */
#define NETC__SW0_PORT1_BPOR NETC__SW0_PORT1.BPOR.R        /* Bridge port operational register */
#define NETC__SW0_PORT1_BPDCR NETC__SW0_PORT1.BPDCR.R      /* Bridge port discard count register */
#define NETC__SW0_PORT1_BPDCRR0 NETC__SW0_PORT1.BPDCRR0.R  /* Bridge port discard count reason register 0 */
#define NETC__SW0_PORT1_BPDCRR1 NETC__SW0_PORT1.BPDCRR1.R  /* Bridge port discard count reason register 1 */
#define NETC__SW0_PORT1_BPMLFSR NETC__SW0_PORT1.BPMLFSR.R  /* Bridge port MAC learning failure status register */

/* SW_PORT2 */
#define NETC__SW0_PORT2_PCAPR NETC__SW0_PORT2.PCAPR.R      /* Port capability register */
#define NETC__SW0_PORT2_PMCAPR NETC__SW0_PORT2.PMCAPR.R    /* Port MAC capability register */
#define NETC__SW0_PORT2_PIOCAPR NETC__SW0_PORT2.PIOCAPR.R  /* Port I/O capability register */
#define NETC__SW0_PORT2_PCR  NETC__SW0_PORT2.PCR.R         /* Port configuration register */
#define NETC__SW0_PORT2_PMAR0 NETC__SW0_PORT2.PMAR0.R      /* Port MAC address register 0 */
#define NETC__SW0_PORT2_PMAR1 NETC__SW0_PORT2.PMAR1.R      /* Port MAC address register 1 */
#define NETC__SW0_PORT2_PTAR NETC__SW0_PORT2.PTAR.R        /* Port TPID acceptance register */
#define NETC__SW0_PORT2_PQOSMR NETC__SW0_PORT2.PQOSMR.R    /* Port QoS mode register */
#define NETC__SW0_PORT2_PQOR NETC__SW0_PORT2.PQOR.R        /* Port Queue Operational register */
#define NETC__SW0_PORT2_PPCR NETC__SW0_PORT2.PPCR.R        /* Port parser configuration register */
#define NETC__SW0_PORT2_PIPFCR NETC__SW0_PORT2.PIPFCR.R    /* Port ingress port filter configuration register */
#define NETC__SW0_PORT2_PSGCR NETC__SW0_PORT2.PSGCR.R      /* Port stream gate configuration register */
#define NETC__SW0_PORT2_POR  NETC__SW0_PORT2.PORr.R        /* Port operational register */
#define NETC__SW0_PORT2_PSR  NETC__SW0_PORT2.PSR.R         /* Port status register */
#define NETC__SW0_PORT2_PRXSDUOR NETC__SW0_PORT2.PRXSDUOR.R /* Port receive SDU overhead register */
#define NETC__SW0_PORT2_PTXSDUOR NETC__SW0_PORT2.PTXSDUOR.R /* Port transmit SDU overhead register */
#define NETC__SW0_PORT2_PTGSCR NETC__SW0_PORT2.PTGSCR.R    /* Port time gate scheduling control register */
#define NETC__SW0_PORT2_PTGAGLSR NETC__SW0_PORT2.PTGAGLSR.R /* Port time gate scheduling admin gate list status register */
#define NETC__SW0_PORT2_PTGAGLLR NETC__SW0_PORT2.PTGAGLLR.R /* Port time gate scheduling admin gate list length register */
#define NETC__SW0_PORT2_PTGOGLLR NETC__SW0_PORT2.PTGOGLLR.R /* Port time gating operational gate list length register */
#define NETC__SW0_PORT2_PRXDCR NETC__SW0_PORT2.PRXDCR.R    /* Port Rx discard count register */
#define NETC__SW0_PORT2_PRXDCRR0 NETC__SW0_PORT2.PRXDCRR0.R /* Port Rx discard count reason register 0 */
#define NETC__SW0_PORT2_PRXDCRR1 NETC__SW0_PORT2.PRXDCRR1.R /* Port Rx discard count reason register 1 */
#define NETC__SW0_PORT2_PTXDCR NETC__SW0_PORT2.PTXDCR.R    /* Port Tx discard count register */
#define NETC__SW0_PORT2_PTXDCRR0 NETC__SW0_PORT2.PTXDCRR0.R /* Port Tx discard count reason register 0 */
#define NETC__SW0_PORT2_PTXDCRR1 NETC__SW0_PORT2.PTXDCRR1.R /* Port Tx discard count reason register 1 */
#define NETC__SW0_PORT2_PTGSTC0SR NETC__SW0_PORT2.TCT_NUM[0].PTGSTCSR.R /* Port time gate scheduling traffic class 0 status register */
#define NETC__SW0_PORT2_PTC0TMSDUR NETC__SW0_PORT2.TCT_NUM[0].PTCTMSDUR.R /* Port traffic class 0 transmit maximum SDU register */
#define NETC__SW0_PORT2_PTC0CBSR0 NETC__SW0_PORT2.TCT_NUM[0].PTCCBSR0.R /* Port transmit traffic class 0 credit based shaper register 0 */
#define NETC__SW0_PORT2_PTC0CBSR1 NETC__SW0_PORT2.TCT_NUM[0].PTCCBSR1.R /* Port traffic class 0 credit based shaper register 1 */
#define NETC__SW0_PORT2_PTGSTC1SR NETC__SW0_PORT2.TCT_NUM[1].PTGSTCSR.R /* Port time gate scheduling traffic class 1 status register */
#define NETC__SW0_PORT2_PTC1TMSDUR NETC__SW0_PORT2.TCT_NUM[1].PTCTMSDUR.R /* Port traffic class 1 transmit maximum SDU register */
#define NETC__SW0_PORT2_PTC1CBSR0 NETC__SW0_PORT2.TCT_NUM[1].PTCCBSR0.R /* Port transmit traffic class 1 credit based shaper register 0 */
#define NETC__SW0_PORT2_PTC1CBSR1 NETC__SW0_PORT2.TCT_NUM[1].PTCCBSR1.R /* Port traffic class 1 credit based shaper register 1 */
#define NETC__SW0_PORT2_PTGSTC2SR NETC__SW0_PORT2.TCT_NUM[2].PTGSTCSR.R /* Port time gate scheduling traffic class 2 status register */
#define NETC__SW0_PORT2_PTC2TMSDUR NETC__SW0_PORT2.TCT_NUM[2].PTCTMSDUR.R /* Port traffic class 2 transmit maximum SDU register */
#define NETC__SW0_PORT2_PTC2CBSR0 NETC__SW0_PORT2.TCT_NUM[2].PTCCBSR0.R /* Port transmit traffic class 2 credit based shaper register 0 */
#define NETC__SW0_PORT2_PTC2CBSR1 NETC__SW0_PORT2.TCT_NUM[2].PTCCBSR1.R /* Port traffic class 2 credit based shaper register 1 */
#define NETC__SW0_PORT2_PTGSTC3SR NETC__SW0_PORT2.TCT_NUM[3].PTGSTCSR.R /* Port time gate scheduling traffic class 3 status register */
#define NETC__SW0_PORT2_PTC3TMSDUR NETC__SW0_PORT2.TCT_NUM[3].PTCTMSDUR.R /* Port traffic class 3 transmit maximum SDU register */
#define NETC__SW0_PORT2_PTC3CBSR0 NETC__SW0_PORT2.TCT_NUM[3].PTCCBSR0.R /* Port transmit traffic class 3 credit based shaper register 0 */
#define NETC__SW0_PORT2_PTC3CBSR1 NETC__SW0_PORT2.TCT_NUM[3].PTCCBSR1.R /* Port traffic class 3 credit based shaper register 1 */
#define NETC__SW0_PORT2_PTGSTC4SR NETC__SW0_PORT2.TCT_NUM[4].PTGSTCSR.R /* Port time gate scheduling traffic class 4 status register */
#define NETC__SW0_PORT2_PTC4TMSDUR NETC__SW0_PORT2.TCT_NUM[4].PTCTMSDUR.R /* Port traffic class 4 transmit maximum SDU register */
#define NETC__SW0_PORT2_PTC4CBSR0 NETC__SW0_PORT2.TCT_NUM[4].PTCCBSR0.R /* Port transmit traffic class 4 credit based shaper register 0 */
#define NETC__SW0_PORT2_PTC4CBSR1 NETC__SW0_PORT2.TCT_NUM[4].PTCCBSR1.R /* Port traffic class 4 credit based shaper register 1 */
#define NETC__SW0_PORT2_PTGSTC5SR NETC__SW0_PORT2.TCT_NUM[5].PTGSTCSR.R /* Port time gate scheduling traffic class 5 status register */
#define NETC__SW0_PORT2_PTC5TMSDUR NETC__SW0_PORT2.TCT_NUM[5].PTCTMSDUR.R /* Port traffic class 5 transmit maximum SDU register */
#define NETC__SW0_PORT2_PTC5CBSR0 NETC__SW0_PORT2.TCT_NUM[5].PTCCBSR0.R /* Port transmit traffic class 5 credit based shaper register 0 */
#define NETC__SW0_PORT2_PTC5CBSR1 NETC__SW0_PORT2.TCT_NUM[5].PTCCBSR1.R /* Port traffic class 5 credit based shaper register 1 */
#define NETC__SW0_PORT2_PTGSTC6SR NETC__SW0_PORT2.TCT_NUM[6].PTGSTCSR.R /* Port time gate scheduling traffic class 6 status register */
#define NETC__SW0_PORT2_PTC6TMSDUR NETC__SW0_PORT2.TCT_NUM[6].PTCTMSDUR.R /* Port traffic class 6 transmit maximum SDU register */
#define NETC__SW0_PORT2_PTC6CBSR0 NETC__SW0_PORT2.TCT_NUM[6].PTCCBSR0.R /* Port transmit traffic class 6 credit based shaper register 0 */
#define NETC__SW0_PORT2_PTC6CBSR1 NETC__SW0_PORT2.TCT_NUM[6].PTCCBSR1.R /* Port traffic class 6 credit based shaper register 1 */
#define NETC__SW0_PORT2_PTGSTC7SR NETC__SW0_PORT2.TCT_NUM[7].PTGSTCSR.R /* Port time gate scheduling traffic class 7 status register */
#define NETC__SW0_PORT2_PTC7TMSDUR NETC__SW0_PORT2.TCT_NUM[7].PTCTMSDUR.R /* Port traffic class 7 transmit maximum SDU register */
#define NETC__SW0_PORT2_PTC7CBSR0 NETC__SW0_PORT2.TCT_NUM[7].PTCCBSR0.R /* Port transmit traffic class 7 credit based shaper register 0 */
#define NETC__SW0_PORT2_PTC7CBSR1 NETC__SW0_PORT2.TCT_NUM[7].PTCCBSR1.R /* Port traffic class 7 credit based shaper register 1 */
#define NETC__SW0_PORT2_PBPMCR0 NETC__SW0_PORT2.PBPMCR0.R  /* Port buffer pool mapping configuration register 0 */
#define NETC__SW0_PORT2_PBPMCR1 NETC__SW0_PORT2.PBPMCR1.R  /* Port buffer pool mapping configuration register 1 */
#define NETC__SW0_PORT2_PPCPDEIMR NETC__SW0_PORT2.PPCPDEIMR.R /* Port PCP DEI mapping register */
#define NETC__SW0_PORT2_PMCR NETC__SW0_PORT2.PMCR.R        /* Port mirror configuration register */
#define NETC__SW0_PORT2_PLANIDCR NETC__SW0_PORT2.PLANIDCR.R /* Port LANID configuration register */
#define NETC__SW0_PORT2_PISIDCR NETC__SW0_PORT2.PISIDCR.R  /* Port ingress stream identification configuration register */
#define NETC__SW0_PORT2_PFMCR NETC__SW0_PORT2.PFMCR.R      /* Port frame modification configuration register */
#define NETC__SW0_PORT2_PIPV2QMR0 NETC__SW0_PORT2.PIPV2QMR0.R /* Port IPV to queue mapping register 0 */
#define NETC__SW0_PORT2_PTCMINLR NETC__SW0_PORT2.PTCMINLR.R /* Port time capture minimum latency register */
#define NETC__SW0_PORT2_PTCMAXLR NETC__SW0_PORT2.PTCMAXLR.R /* Port time capture maximum latency register */
#define NETC__SW0_PORT2_BPCR NETC__SW0_PORT2.BPCR.R        /* Bridge port configuration register */
#define NETC__SW0_PORT2_BPDVR NETC__SW0_PORT2.BPDVR.R      /* Bridge port default VLAN register */
#define NETC__SW0_PORT2_BPSTGSR NETC__SW0_PORT2.BPSTGSR.R  /* Bridge port spanning tree group state register */
#define NETC__SW0_PORT2_BPSCR0 NETC__SW0_PORT2.BPSCR0.R    /* Bridge port storm control register 0 */
#define NETC__SW0_PORT2_BPSCR1 NETC__SW0_PORT2.BPSCR1.R    /* Bridge port storm control register 1 */
#define NETC__SW0_PORT2_BPOR NETC__SW0_PORT2.BPOR.R        /* Bridge port operational register */
#define NETC__SW0_PORT2_BPDCR NETC__SW0_PORT2.BPDCR.R      /* Bridge port discard count register */
#define NETC__SW0_PORT2_BPDCRR0 NETC__SW0_PORT2.BPDCRR0.R  /* Bridge port discard count reason register 0 */
#define NETC__SW0_PORT2_BPDCRR1 NETC__SW0_PORT2.BPDCRR1.R  /* Bridge port discard count reason register 1 */
#define NETC__SW0_PORT2_BPMLFSR NETC__SW0_PORT2.BPMLFSR.R  /* Bridge port MAC learning failure status register */

/* SW_PSEUDO_MAC_PORT2 */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMSR NETC__SW0_PSEUDO_MAC_PORT2.PPMSR.R /* Port pseudo MAC status register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMROCR0 NETC__SW0_PSEUDO_MAC_PORT2.PPMROCR[0].R /* Port pseudo MAC receive octets counter */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMROCR1 NETC__SW0_PSEUDO_MAC_PORT2.PPMROCR[1].R /* Port pseudo MAC receive octets counter */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMRUFCR0 NETC__SW0_PSEUDO_MAC_PORT2.PPMRUFCR[0].R /* Port pseudo MAC receive unicast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMRUFCR1 NETC__SW0_PSEUDO_MAC_PORT2.PPMRUFCR[1].R /* Port pseudo MAC receive unicast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMRMFCR0 NETC__SW0_PSEUDO_MAC_PORT2.PPMRMFCR[0].R /* Port pseudo MAC receive multicast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMRMFCR1 NETC__SW0_PSEUDO_MAC_PORT2.PPMRMFCR[1].R /* Port pseudo MAC receive multicast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMRBFCR0 NETC__SW0_PSEUDO_MAC_PORT2.PPMRBFCR[0].R /* Port pseudo MAC receive broadcast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMRBFCR1 NETC__SW0_PSEUDO_MAC_PORT2.PPMRBFCR[1].R /* Port pseudo MAC receive broadcast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMTOCR0 NETC__SW0_PSEUDO_MAC_PORT2.PPMTOCR[0].R /* Port pseudo MAC transmit octets counter */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMTOCR1 NETC__SW0_PSEUDO_MAC_PORT2.PPMTOCR[1].R /* Port pseudo MAC transmit octets counter */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMTUFCR0 NETC__SW0_PSEUDO_MAC_PORT2.PPMTUFCR[0].R /* Port pseudo MAC transmit unicast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMTUFCR1 NETC__SW0_PSEUDO_MAC_PORT2.PPMTUFCR[1].R /* Port pseudo MAC transmit unicast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMTMFCR0 NETC__SW0_PSEUDO_MAC_PORT2.PPMTMFCR[0].R /* Port pseudo MAC transmit multicast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMTMFCR1 NETC__SW0_PSEUDO_MAC_PORT2.PPMTMFCR[1].R /* Port pseudo MAC transmit multicast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMTBFCR0 NETC__SW0_PSEUDO_MAC_PORT2.PPMTBFCR[0].R /* Port pseudo MAC transmit broadcast frame counter register */
#define NETC__SW0_PSEUDO_MAC_PORT2_PPMTBFCR1 NETC__SW0_PSEUDO_MAC_PORT2.PPMTBFCR[1].R /* Port pseudo MAC transmit broadcast frame counter register */

/* TIMERS */
#define CEVA_SPF2__TIMERS_TIMER0_CFG CEVA_SPF2__TIMERS.TIMER0_CFG.R /* TIMER0 CFG */
#define CEVA_SPF2__TIMERS_TIMER0_EW CEVA_SPF2__TIMERS.TIMER0_EW.R /* TIMER0 EW */
#define CEVA_SPF2__TIMERS_TIMER0_CC CEVA_SPF2__TIMERS.TIMER0_CC.R /* TIMER0 CC */
#define CEVA_SPF2__TIMERS_TIMER0_SC CEVA_SPF2__TIMERS.TIMER0_SC.R /* TIMER0 SC */
#define CEVA_SPF2__TIMERS_TIMER1_CFG CEVA_SPF2__TIMERS.TIMER1_CFG.R /* TIMER1 CFG */
#define CEVA_SPF2__TIMERS_TIMER1_EW CEVA_SPF2__TIMERS.TIMER1_EW.R /* TIMER1 EW */
#define CEVA_SPF2__TIMERS_TIMER1_CC CEVA_SPF2__TIMERS.TIMER1_CC.R /* TIMER1 CC */
#define CEVA_SPF2__TIMERS_TIMER1_SC CEVA_SPF2__TIMERS.TIMER1_SC.R /* TIMER1 SC */
#define CEVA_SPF2__TIMERS_TIMER2_CFG CEVA_SPF2__TIMERS.TIMER2_CFG.R /* TIMER2 CFG */
#define CEVA_SPF2__TIMERS_TIMER2_EW CEVA_SPF2__TIMERS.TIMER2_EW.R /* TIMER2 EW */
#define CEVA_SPF2__TIMERS_TIMER2_CC CEVA_SPF2__TIMERS.TIMER2_CC.R /* TIMER2 CC */
#define CEVA_SPF2__TIMERS_TIMER2_SC CEVA_SPF2__TIMERS.TIMER2_SC.R /* TIMER2 SC */
#define CEVA_SPF2__TIMERS_TIMER3_CFG CEVA_SPF2__TIMERS.TIMER3_CFG.R /* TIMER3 CFG */
#define CEVA_SPF2__TIMERS_TIMER3_EW CEVA_SPF2__TIMERS.TIMER3_EW.R /* TIMER3 EW */
#define CEVA_SPF2__TIMERS_TIMER3_CC CEVA_SPF2__TIMERS.TIMER3_CC.R /* TIMER3 CC */
#define CEVA_SPF2__TIMERS_TIMER3_SC CEVA_SPF2__TIMERS.TIMER3_SC.R /* TIMER3 SC */

/* TMR0_BASE */
#define NETC__TMR0_BASE_TMR_ID NETC__TMR0_BASE.TMR_ID.R    /* Module ID Register */
#define NETC__TMR0_BASE_TMR_CAPR NETC__TMR0_BASE.TMR_CAPR.R /* Timer Capability Register */
#define NETC__TMR0_BASE_TMR_FRT_L NETC__TMR0_BASE.TMR_FRT_L.R /* Timer free running time low register */
#define NETC__TMR0_BASE_TMR_FRT_H NETC__TMR0_BASE.TMR_FRT_H.R /* Timer free running time high register */
#define NETC__TMR0_BASE_TMR_SRT_L NETC__TMR0_BASE.TMR_SRT_L.R /* Timer synchronous time low register */
#define NETC__TMR0_BASE_TMR_SRT_H NETC__TMR0_BASE.TMR_SRT_H.R /* Timer synchronous time high register. */
#define NETC__TMR0_BASE_TMR_DEF_CNT_L NETC__TMR0_BASE.TMR_DEF_CNT_L.R /* Default ns timer counter low register */
#define NETC__TMR0_BASE_TMR_DEF_CNT_H NETC__TMR0_BASE.TMR_DEF_CNT_H.R /* Default ns timer counter high register */
#define NETC__TMR0_BASE_TMR_CTRL NETC__TMR0_BASE.TMR_CTRL.R /* Timer Control Register */
#define NETC__TMR0_BASE_TMR_TEVENT NETC__TMR0_BASE.TMR_TEVENT.R /* Timer Event Register */
#define NETC__TMR0_BASE_TMR_TEMASK NETC__TMR0_BASE.TMR_TEMASK.R /* Timer event mask register */
#define NETC__TMR0_BASE_TMR_STAT NETC__TMR0_BASE.TMR_STAT.R /* Timer status register */
#define NETC__TMR0_BASE_TMR_CNT_L NETC__TMR0_BASE.TMR_CNT_L.R /* Timer counter low register */
#define NETC__TMR0_BASE_TMR_CNT_H NETC__TMR0_BASE.TMR_CNT_H.R /* Timer counter high register */
#define NETC__TMR0_BASE_TMR_ADD NETC__TMR0_BASE.TMR_ADD.R  /* Timer addend register */
#define NETC__TMR0_BASE_TMR_ACC NETC__TMR0_BASE.TMR_ACC.R  /* Timer accumulator register */
#define NETC__TMR0_BASE_TMR_PRSC NETC__TMR0_BASE.TMR_PRSC.R /* Timer prescale register */
#define NETC__TMR0_BASE_TMR_ECTRL NETC__TMR0_BASE.TMR_ECTRL.R /* Extended timer control register */
#define NETC__TMR0_BASE_TMROFF_L NETC__TMR0_BASE.TMROFF_L.R /* Timer offset low register */
#define NETC__TMR0_BASE_TMROFF_H NETC__TMR0_BASE.TMROFF_H.R /* Timer offset high register */
#define NETC__TMR0_BASE_TMR_ALARM1_L NETC__TMR0_BASE.TMR_ALARM1_L.R /* Alarm 1 time comparator low register */
#define NETC__TMR0_BASE_TMR_ALARM1_H NETC__TMR0_BASE.TMR_ALARM1_H.R /* Alarm 1 time comparator high register */
#define NETC__TMR0_BASE_TMR_ALARM2_L NETC__TMR0_BASE.TMR_ALARM2_L.R /* Alarm 2 time comparator low register */
#define NETC__TMR0_BASE_TMR_ALARM2_H NETC__TMR0_BASE.TMR_ALARM2_H.R /* Alarm 2 time comparator high register */
#define NETC__TMR0_BASE_TMR_ALARM_CTRL NETC__TMR0_BASE.TMR_ALARM_CTRL.R /* Timer Alarm Control Register */
#define NETC__TMR0_BASE_TMR_FIPER1 NETC__TMR0_BASE.TMR_FIPER[0].R /* Timer 1 fixed interval period register */
#define NETC__TMR0_BASE_TMR_FIPER2 NETC__TMR0_BASE.TMR_FIPER[1].R /* Timer 2 fixed interval period register */
#define NETC__TMR0_BASE_TMR_FIPER3 NETC__TMR0_BASE.TMR_FIPER[2].R /* Timer 3 fixed interval period register */
#define NETC__TMR0_BASE_TMR_FIPER_CTRL NETC__TMR0_BASE.TMR_FIPER_CTRL.R /* Timer FIPER Control Register */
#define NETC__TMR0_BASE_TMR_ETTS1_L NETC__TMR0_BASE.TMR_ETTS1_L.R /* External trigger stamp register */
#define NETC__TMR0_BASE_TMR_ETTS1_H NETC__TMR0_BASE.TMR_ETTS1_H.R /* External trigger stamp register */
#define NETC__TMR0_BASE_TMR_ETTS2_L NETC__TMR0_BASE.TMR_ETTS2_L.R /* External trigger stamp register */
#define NETC__TMR0_BASE_TMR_ETTS2_H NETC__TMR0_BASE.TMR_ETTS2_H.R /* External trigger stamp register */
#define NETC__TMR0_BASE_TMR_CUR_TIME_L NETC__TMR0_BASE.TMR_CUR_TIME_L.R /* Timer current time low register */
#define NETC__TMR0_BASE_TMR_CUR_TIME_H NETC__TMR0_BASE.TMR_CUR_TIME_H.R /* Timer current time high register */
#define NETC__TMR0_BASE_TMR_PARAM NETC__TMR0_BASE.TMR_PARAM.R /* Timer parameter register */

/* TMU */
#define TMU_TMR              TMU.TMR.R                     /* Mode */
#define TMU_TSR              TMU.TSR.R                     /* Status */
#define TMU_TMSR             TMU.TMSR.R                    /* Monitor Site */
#define TMU_TMTMIR           TMU.TMTMIR.R                  /* Monitor Temperature Measurement Interval */
#define TMU_TIER             TMU.TIER.R                    /* Interrupt Enable */
#define TMU_TIDR             TMU.TIDR.R                    /* Interrupt Detect */
#define TMU_TIISCR           TMU.TIISCR.R                  /* Interrupt Immediate Site Capture */
#define TMU_TIASCR           TMU.TIASCR.R                  /* Interrupt Average Site Capture */
#define TMU_TICSCR           TMU.TICSCR.R                  /* Interrupt Critical Site Capture */
#define TMU_TMHTCR           TMU.TMHTCR.R                  /* Monitor High Temperature Capture */
#define TMU_TMLTCR           TMU.TMLTCR.R                  /* Monitor Low Temperature Capture */
#define TMU_TMRTRCR          TMU.TMRTRCR.R                 /* Monitor Rising Temperature Rate Capture */
#define TMU_TMFTRCR          TMU.TMFTRCR.R                 /* Monitor Falling Temperature Rate Capture */
#define TMU_TMHTITR          TMU.TMHTITR.R                 /* Monitor High Temperature Immediate Threshold */
#define TMU_TMHTATR          TMU.TMHTATR.R                 /* Monitor High Temperature Average Threshold */
#define TMU_TMHTACTR         TMU.TMHTACTR.R                /* Monitor High Temperature Average Critical Threshold */
#define TMU_TMLTITR          TMU.TMLTITR.R                 /* Monitor Low Temperature Immediate Threshold */
#define TMU_TMLTATR          TMU.TMLTATR.R                 /* Monitor Low Temperature Average Threshold */
#define TMU_TMLTACTR         TMU.TMLTACTR.R                /* Monitor Low Temperature Average Critical Threshold */
#define TMU_TMRTRCTR         TMU.TMRTRCTR.R                /* Monitor Rising Temperature Rate Critical Threshold */
#define TMU_TMFTRCTR         TMU.TMFTRCTR.R                /* Monitor Falling Temperature Rate Critical Threshold */
#define TMU_TTCFGR           TMU.TTCFGR.R                  /* Temperature Configuration */
#define TMU_TSCFGR           TMU.TSCFGR.R                  /* Sensor Configuration */
#define TMU_TRITSR0          TMU.TRITRATSR[0].TRITSR.R     /* Report Immediate Temperature at Site */
#define TMU_TRATSR0          TMU.TRITRATSR[0].TRATSR.R     /* Report Average Temperature at Site */
#define TMU_TRITSR1          TMU.TRITRATSR[1].TRITSR.R     /* Report Immediate Temperature at Site */
#define TMU_TRATSR1          TMU.TRITRATSR[1].TRATSR.R     /* Report Average Temperature at Site */
#define TMU_TRITSR2          TMU.TRITRATSR[2].TRITSR.R     /* Report Immediate Temperature at Site */
#define TMU_TRATSR2          TMU.TRITRATSR[2].TRATSR.R     /* Report Average Temperature at Site */
#define TMU_TRITSR3          TMU.TRITRATSR[3].TRITSR.R     /* Report Immediate Temperature at Site */
#define TMU_TRATSR3          TMU.TRITRATSR[3].TRATSR.R     /* Report Average Temperature at Site */
#define TMU_TRITSR4          TMU.TRITRATSR[4].TRITSR.R     /* Report Immediate Temperature at Site */
#define TMU_TRATSR4          TMU.TRITRATSR[4].TRATSR.R     /* Report Average Temperature at Site */
#define TMU_TCMCFG           TMU.TCMCFG.R                  /* Central Module Configuration */
#define TMU_TTRCR0           TMU.TTRCR[0].R                /* Temperature Range Control 0 */
#define TMU_TTRCR1           TMU.TTRCR[1].R                /* Temperature Range Control 1 */
#define TMU_TTRCR2           TMU.TTRCR[2].R                /* Temperature Range Control 2 */
#define TMU_TTRCR3           TMU.TTRCR[3].R                /* Temperature Range Control 3 */
#define TMU_TTRCR4           TMU.TTRCR[4].R                /* Temperature Range Control 4 */
#define TMU_TTRCR5           TMU.TTRCR[5].R                /* Temperature Range Control 5 */
#define TMU_TTRCR6           TMU.TTRCR[6].R                /* Temperature Range Control 6 */
#define TMU_TTRCR7           TMU.TTRCR[7].R                /* Temperature Range Control 7 */
#define TMU_TTRCR8           TMU.TTRCR[8].R                /* Temperature Range Control 8 */
#define TMU_TTRCR9           TMU.TTRCR[9].R                /* Temperature Range Control 9 */
#define TMU_TTRCR10          TMU.TTRCR[10].R               /* Temperature Range Control 10 */
#define TMU_TTRCR11          TMU.TTRCR[11].R               /* Temperature Range Control 11 */
#define TMU_TTRCR12          TMU.TTRCR[12].R               /* Temperature Range Control 12 */
#define TMU_TTRCR13          TMU.TTRCR[13].R               /* Temperature Range Control 13 */
#define TMU_TTRCR14          TMU.TTRCR[14].R               /* Temperature Range Control 14 */
#define TMU_TTRCR15          TMU.TTRCR[15].R               /* Temperature Range Control 15 */

/* TRGMUX_0 */
#define TRGMUX_0_REG_0       TRGMUX_0.REG_0.R              /* TRGMUX REG_0 */
#define TRGMUX_0_REG_1       TRGMUX_0.REG_1.R              /* TRGMUX REG_1 */
#define TRGMUX_0_REG_2       TRGMUX_0.REG_2.R              /* TRGMUX REG_2 */
#define TRGMUX_0_REG_3       TRGMUX_0.REG_3.R              /* TRGMUX REG_3 */
#define TRGMUX_0_REG_4       TRGMUX_0.REG_4.R              /* TRGMUX REG_4 */

/* TRGMUX_1 */
#define TRGMUX_1_REG_0       TRGMUX_1.REG_0.R              /* TRGMUX REG_0 */
#define TRGMUX_1_REG_1       TRGMUX_1.REG_1.R              /* TRGMUX REG_1 */
#define TRGMUX_1_REG_2       TRGMUX_1.REG_2.R              /* TRGMUX REG_2 */
#define TRGMUX_1_REG_3       TRGMUX_1.REG_3.R              /* TRGMUX REG_3 */
#define TRGMUX_1_REG_4       TRGMUX_1.REG_4.R              /* TRGMUX REG_4 */

/* TRGMUX_2 */
#define TRGMUX_2_TIM0_1_0    TRGMUX_2.TIM0_1_0.R           /* TRGMUX TIM0_1_0 */
#define TRGMUX_2_TIM0_3_2    TRGMUX_2.TIM0_3_2.R           /* TRGMUX TIM0_3_2 */
#define TRGMUX_2_TIM0_5_4    TRGMUX_2.TIM0_5_4.R           /* TRGMUX TIM0_5_4 */
#define TRGMUX_2_TIM0_7_6    TRGMUX_2.TIM0_7_6.R           /* TRGMUX TIM0_7_6 */
#define TRGMUX_2_TIM1_1_0    TRGMUX_2.TIM1_1_0.R           /* TRGMUX TIM1_1_0 */
#define TRGMUX_2_TIM1_3_2    TRGMUX_2.TIM1_3_2.R           /* TRGMUX TIM1_3_2 */
#define TRGMUX_2_TIM1_5_4    TRGMUX_2.TIM1_5_4.R           /* TRGMUX TIM1_5_4 */
#define TRGMUX_2_TIM1_7_6    TRGMUX_2.TIM1_7_6.R           /* TRGMUX TIM1_7_6 */
#define TRGMUX_2_TIM2_1_0    TRGMUX_2.TIM2_1_0.R           /* TRGMUX TIM2_1_0 */
#define TRGMUX_2_TIM2_3_2    TRGMUX_2.TIM2_3_2.R           /* TRGMUX TIM2_3_2 */
#define TRGMUX_2_TIM2_5_4    TRGMUX_2.TIM2_5_4.R           /* TRGMUX TIM2_5_4 */
#define TRGMUX_2_TIM2_7_6    TRGMUX_2.TIM2_7_6.R           /* TRGMUX TIM2_7_6 */
#define TRGMUX_2_TIO1_1_0    TRGMUX_2.TIO1_1_0.R           /* TRGMUX TIO1_1_0 */
#define TRGMUX_2_TIO1_3_2    TRGMUX_2.TIO1_3_2.R           /* TRGMUX TIO1_3_2 */
#define TRGMUX_2_TIO1_5_4    TRGMUX_2.TIO1_5_4.R           /* TRGMUX TIO1_5_4 */
#define TRGMUX_2_TIO1_7_6    TRGMUX_2.TIO1_7_6.R           /* TRGMUX TIO1_7_6 */
#define TRGMUX_2_TIO2_1_0    TRGMUX_2.TIO2_1_0.R           /* TRGMUX TIO2_1_0 */
#define TRGMUX_2_TIO2_3_2    TRGMUX_2.TIO2_3_2.R           /* TRGMUX TIO2_3_2 */
#define TRGMUX_2_TIO2_5_4    TRGMUX_2.TIO2_5_4.R           /* TRGMUX TIO2_5_4 */
#define TRGMUX_2_TIO2_7_6    TRGMUX_2.TIO2_7_6.R           /* TRGMUX TIO2_7_6 */
#define TRGMUX_2_TIO3_1_0    TRGMUX_2.TIO3_1_0.R           /* TRGMUX TIO3_1_0 */
#define TRGMUX_2_TIO3_3_2    TRGMUX_2.TIO3_3_2.R           /* TRGMUX TIO3_3_2 */
#define TRGMUX_2_TIO3_5_4    TRGMUX_2.TIO3_5_4.R           /* TRGMUX TIO3_5_4 */
#define TRGMUX_2_TIO3_7_6    TRGMUX_2.TIO3_7_6.R           /* TRGMUX TIO3_7_6 */

/* TRGMUX_3 */
#define TRGMUX_3_GTM_003_000 TRGMUX_3.GTM_003_000.R        /* TRGMUX GTM_003_000 */
#define TRGMUX_3_GTM_007_004 TRGMUX_3.GTM_007_004.R        /* TRGMUX GTM_007_004 */
#define TRGMUX_3_GTM_011_008 TRGMUX_3.GTM_011_008.R        /* TRGMUX GTM_011_008 */
#define TRGMUX_3_GTM_015_012 TRGMUX_3.GTM_015_012.R        /* TRGMUX GTM_015_012 */
#define TRGMUX_3_GTM_019_016 TRGMUX_3.GTM_019_016.R        /* TRGMUX GTM_019_016 */
#define TRGMUX_3_GTM_023_020 TRGMUX_3.GTM_023_020.R        /* TRGMUX GTM_023_020 */
#define TRGMUX_3_GTM_027_024 TRGMUX_3.GTM_027_024.R        /* TRGMUX GTM_027_024 */
#define TRGMUX_3_GTM_031_028 TRGMUX_3.GTM_031_028.R        /* TRGMUX GTM_031_028 */
#define TRGMUX_3_GTM_035_032 TRGMUX_3.GTM_035_032.R        /* TRGMUX GTM_035_032 */
#define TRGMUX_3_GTM_039_036 TRGMUX_3.GTM_039_036.R        /* TRGMUX GTM_039_036 */
#define TRGMUX_3_GTM_043_040 TRGMUX_3.GTM_043_040.R        /* TRGMUX GTM_043_040 */
#define TRGMUX_3_GTM_047_044 TRGMUX_3.GTM_047_044.R        /* TRGMUX GTM_047_044 */
#define TRGMUX_3_GTM_051_048 TRGMUX_3.GTM_051_048.R        /* TRGMUX GTM_051_048 */
#define TRGMUX_3_GTM_055_052 TRGMUX_3.GTM_055_052.R        /* TRGMUX GTM_055_052 */
#define TRGMUX_3_GTM_059_056 TRGMUX_3.GTM_059_056.R        /* TRGMUX GTM_059_056 */
#define TRGMUX_3_GTM_063_060 TRGMUX_3.GTM_063_060.R        /* TRGMUX GTM_063_060 */
#define TRGMUX_3_GTM_067_064 TRGMUX_3.GTM_067_064.R        /* TRGMUX GTM_067_064 */
#define TRGMUX_3_GTM_071_068 TRGMUX_3.GTM_071_068.R        /* TRGMUX GTM_071_068 */
#define TRGMUX_3_GTM_075_072 TRGMUX_3.GTM_075_072.R        /* TRGMUX GTM_075_072 */
#define TRGMUX_3_GTM_079_076 TRGMUX_3.GTM_079_076.R        /* TRGMUX GTM_079_076 */
#define TRGMUX_3_GTM_083_080 TRGMUX_3.GTM_083_080.R        /* TRGMUX GTM_083_080 */
#define TRGMUX_3_GTM_087_084 TRGMUX_3.GTM_087_084.R        /* TRGMUX GTM_087_084 */
#define TRGMUX_3_GTM_091_088 TRGMUX_3.GTM_091_088.R        /* TRGMUX GTM_091_088 */
#define TRGMUX_3_GTM_095_092 TRGMUX_3.GTM_095_092.R        /* TRGMUX GTM_095_092 */
#define TRGMUX_3_GTM_099_096 TRGMUX_3.GTM_099_096.R        /* TRGMUX GTM_099_096 */
#define TRGMUX_3_GTM_103_100 TRGMUX_3.GTM_103_100.R        /* TRGMUX GTM_103_100 */
#define TRGMUX_3_GTM_107_104 TRGMUX_3.GTM_107_104.R        /* TRGMUX GTM_107_104 */
#define TRGMUX_3_GTM_111_108 TRGMUX_3.GTM_111_108.R        /* TRGMUX GTM_111_108 */
#define TRGMUX_3_GTM_115_112 TRGMUX_3.GTM_115_112.R        /* TRGMUX GTM_115_112 */
#define TRGMUX_3_GTM_119_116 TRGMUX_3.GTM_119_116.R        /* TRGMUX GTM_119_116 */
#define TRGMUX_3_GTM_123_120 TRGMUX_3.GTM_123_120.R        /* TRGMUX GTM_123_120 */
#define TRGMUX_3_GTM_127_124 TRGMUX_3.GTM_127_124.R        /* TRGMUX GTM_127_124 */
#define TRGMUX_3_GTM_131_128 TRGMUX_3.GTM_131_128.R        /* TRGMUX GTM_131_128 */
#define TRGMUX_3_GTM_135_132 TRGMUX_3.GTM_135_132.R        /* TRGMUX GTM_135_132 */
#define TRGMUX_3_GTM_139_136 TRGMUX_3.GTM_139_136.R        /* TRGMUX GTM_139_136 */
#define TRGMUX_3_GTM_143_140 TRGMUX_3.GTM_143_140.R        /* TRGMUX GTM_143_140 */
#define TRGMUX_3_GTM_147_144 TRGMUX_3.GTM_147_144.R        /* TRGMUX GTM_147_144 */
#define TRGMUX_3_GTM_151_148 TRGMUX_3.GTM_151_148.R        /* TRGMUX GTM_151_148 */
#define TRGMUX_3_GTM_155_152 TRGMUX_3.GTM_155_152.R        /* TRGMUX GTM_155_152 */
#define TRGMUX_3_GTM_159_156 TRGMUX_3.GTM_159_156.R        /* TRGMUX GTM_159_156 */
#define TRGMUX_3_GTM_163_160 TRGMUX_3.GTM_163_160.R        /* TRGMUX GTM_163_160 */
#define TRGMUX_3_GTM_167_164 TRGMUX_3.GTM_167_164.R        /* TRGMUX GTM_167_164 */
#define TRGMUX_3_GTM_168     TRGMUX_3.GTM_168.R            /* TRGMUX GTM_168 */
#define TRGMUX_3_MSC_03_00   TRGMUX_3.MSC_03_00.R          /* TRGMUX MSC_03_00 */
#define TRGMUX_3_MSC_07_04   TRGMUX_3.MSC_07_04.R          /* TRGMUX MSC_07_04 */
#define TRGMUX_3_MSC_11_08   TRGMUX_3.MSC_11_08.R          /* TRGMUX MSC_11_08 */
#define TRGMUX_3_MSC_15_12   TRGMUX_3.MSC_15_12.R          /* TRGMUX MSC_15_12 */
#define TRGMUX_3_MSC_19_16   TRGMUX_3.MSC_19_16.R          /* TRGMUX MSC_19_16 */
#define TRGMUX_3_MSC_23_20   TRGMUX_3.MSC_23_20.R          /* TRGMUX MSC_23_20 */
#define TRGMUX_3_MSC_27_24   TRGMUX_3.MSC_27_24.R          /* TRGMUX MSC_27_24 */
#define TRGMUX_3_MSC_31_28   TRGMUX_3.MSC_31_28.R          /* TRGMUX MSC_31_28 */
#define TRGMUX_3_MSC_35_32   TRGMUX_3.MSC_35_32.R          /* TRGMUX MSC_35_32 */
#define TRGMUX_3_MSC_39_36   TRGMUX_3.MSC_39_36.R          /* TRGMUX MSC_39_36 */
#define TRGMUX_3_MSC_43_40   TRGMUX_3.MSC_43_40.R          /* TRGMUX MSC_43_40 */
#define TRGMUX_3_MSC_47_44   TRGMUX_3.MSC_47_44.R          /* TRGMUX MSC_47_44 */
#define TRGMUX_3_MSC_51_48   TRGMUX_3.MSC_51_48.R          /* TRGMUX MSC_51_48 */
#define TRGMUX_3_MSC_55_52   TRGMUX_3.MSC_55_52.R          /* TRGMUX MSC_55_52 */
#define TRGMUX_3_MSC_59_56   TRGMUX_3.MSC_59_56.R          /* TRGMUX MSC_59_56 */
#define TRGMUX_3_MSC_63_60   TRGMUX_3.MSC_63_60.R          /* TRGMUX MSC_63_60 */
#define TRGMUX_3_CTU_3_0     TRGMUX_3.CTU_3_0.R            /* TRGMUX CTU_3_0 */
#define TRGMUX_3_SINC_3_0    TRGMUX_3.SINC_3_0.R           /* TRGMUX SINC_3_0 */

/* USDHC */
#define USDHC_DS_ADDR        USDHC.DS_ADDR.R               /* DMA System Address */
#define USDHC_BLK_ATT        USDHC.BLK_ATT.R               /* Block Attributes */
#define USDHC_CMD_ARG        USDHC.CMD_ARG.R               /* Command Argument */
#define USDHC_CMD_XFR_TYP    USDHC.CMD_XFR_TYP.R           /* Command Transfer Type */
#define USDHC_CMD_RSP0       USDHC.CMD_RSP0.R              /* Command Response0 */
#define USDHC_CMD_RSP1       USDHC.CMD_RSP1.R              /* Command Response1 */
#define USDHC_CMD_RSP2       USDHC.CMD_RSP2.R              /* Command Response2 */
#define USDHC_CMD_RSP3       USDHC.CMD_RSP3.R              /* Command Response3 */
#define USDHC_DATA_BUFF_ACC_PORT USDHC.DATA_BUFF_ACC_PORT.R /* Data Buffer Access Port */
#define USDHC_PRES_STATE     USDHC.PRES_STATE.R            /* Present State */
#define USDHC_PROT_CTRL      USDHC.PROT_CTRL.R             /* Protocol Control */
#define USDHC_SYS_CTRL       USDHC.SYS_CTRL.R              /* System Control */
#define USDHC_INT_STATUS     USDHC.INT_STATUS.R            /* Interrupt Status */
#define USDHC_INT_STATUS_EN  USDHC.INT_STATUS_EN.R         /* Interrupt Status Enable */
#define USDHC_INT_SIGNAL_EN  USDHC.INT_SIGNAL_EN.R         /* Interrupt Signal Enable */
#define USDHC_AUTOCMD12_ERR_STATUS USDHC.AUTOCMD12_ERR_STATUS.R /* Auto CMD12 Error Status */
#define USDHC_HOST_CTRL_CAP  USDHC.HOST_CTRL_CAP.R         /* Host Controller Capabilities */
#define USDHC_WTMK_LVL       USDHC.WTMK_LVL.R              /* Watermark Level */
#define USDHC_MIX_CTRL       USDHC.MIX_CTRL.R              /* Mixer Control */
#define USDHC_FORCE_EVENT    USDHC.FORCE_EVENT.R           /* Force Event */
#define USDHC_ADMA_ERR_STATUS USDHC.ADMA_ERR_STATUS.R      /* ADMA Error Status */
#define USDHC_ADMA_SYS_ADDR  USDHC.ADMA_SYS_ADDR.R         /* ADMA System Address */
#define USDHC_DLL_CTRL       USDHC.DLL_CTRL.R              /* DLL (Delay Line) Control */
#define USDHC_DLL_STATUS     USDHC.DLL_STATUS.R            /* DLL Status */
#define USDHC_CLK_TUNE_CTRL_STATUS USDHC.CLK_TUNE_CTRL_STATUS.R /* CLK Tuning Control and Status */
#define USDHC_STROBE_DLL_CTRL USDHC.STROBE_DLL_CTRL.R      /* Strobe DLL control */
#define USDHC_STROBE_DLL_STATUS USDHC.STROBE_DLL_STATUS.R  /* Strobe DLL status */
#define USDHC_VEND_SPEC      USDHC.VEND_SPEC.R             /* Vendor Specific Register */
#define USDHC_MMC_BOOT       USDHC.MMC_BOOT.R              /* eMMC Boot */
#define USDHC_VEND_SPEC2     USDHC.VEND_SPEC2.R            /* Vendor Specific 2 Register */
#define USDHC_TUNING_CTRL    USDHC.TUNING_CTRL.R           /* Tuning Control */
#define USDHC_CQVER          USDHC.CQVER.R                 /* Command Queuing Version */
#define USDHC_CQCAP          USDHC.CQCAP.R                 /* Command Queuing Capabilities */
#define USDHC_CQCFG          USDHC.CQCFG.R                 /* Command Queuing Configuration */
#define USDHC_CQCTL          USDHC.CQCTL.R                 /* Command Queuing Control */
#define USDHC_CQIS           USDHC.CQIS.R                  /* Command Queuing Interrupt Status */
#define USDHC_CQISTE         USDHC.CQISTE.R                /* Command Queuing Interrupt Status Enable */
#define USDHC_CQISGE         USDHC.CQISGE.R                /* Command Queuing Interrupt Signal Enable */
#define USDHC_CQIC           USDHC.CQIC.R                  /* Command Queuing Interrupt Coalescing */
#define USDHC_CQTDLBA        USDHC.CQTDLBA.R               /* Command Queuing Task Descriptor List Base Address */
#define USDHC_CQTDLBAU       USDHC.CQTDLBAU.R              /* Command Queuing Task Descriptor List Base Address Upper 32 Bits */
#define USDHC_CQTDBR         USDHC.CQTDBR.R                /* Command Queuing Task Doorbell */
#define USDHC_CQTCN          USDHC.CQTCN.R                 /* Command Queuing Task Completion Notification */
#define USDHC_CQDQS          USDHC.CQDQS.R                 /* Command Queuing Device Queue Status */
#define USDHC_CQDPT          USDHC.CQDPT.R                 /* Command Queuing Device Pending Tasks */
#define USDHC_CQTCLR         USDHC.CQTCLR.R                /* Command Queuing Task Clear */
#define USDHC_CQSSC1         USDHC.CQSSC1.R                /* Command Queuing Send Status Configuration 1 */
#define USDHC_CQSSC2         USDHC.CQSSC2.R                /* Command Queuing Send Status Configuration 2 */
#define USDHC_CQCRDCT        USDHC.CQCRDCT.R               /* Command Queuing Command Response for Direct-Command Task */
#define USDHC_CQRMEM         USDHC.CQRMEM.R                /* Command Queuing Response Mode Error Mask */
#define USDHC_CQTERRI        USDHC.CQTERRI.R               /* Command Queuing Task Error Information */
#define USDHC_CQCRI          USDHC.CQCRI.R                 /* Command Queuing Command Response Index */
#define USDHC_CQCRA          USDHC.CQCRA.R                 /* Command Queuing Command Response Argument */

/* VIRT_WRAP */
#define VIRT_WRAP_3_REG_A15_0 VIRT_WRAP_3.REG_A[0].R       /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A31_16 VIRT_WRAP_3.REG_A[1].R      /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A47_32 VIRT_WRAP_3.REG_A[2].R      /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A63_48 VIRT_WRAP_3.REG_A[3].R      /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A79_64 VIRT_WRAP_3.REG_A[4].R      /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A95_80 VIRT_WRAP_3.REG_A[5].R      /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A111_96 VIRT_WRAP_3.REG_A[6].R     /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A127_112 VIRT_WRAP_3.REG_A[7].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A143_128 VIRT_WRAP_3.REG_A[8].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A159_144 VIRT_WRAP_3.REG_A[9].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A175_160 VIRT_WRAP_3.REG_A[10].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A191_176 VIRT_WRAP_3.REG_A[11].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A207_192 VIRT_WRAP_3.REG_A[12].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A223_208 VIRT_WRAP_3.REG_A[13].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A239_224 VIRT_WRAP_3.REG_A[14].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A255_240 VIRT_WRAP_3.REG_A[15].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A271_256 VIRT_WRAP_3.REG_A[16].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A287_272 VIRT_WRAP_3.REG_A[17].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A303_288 VIRT_WRAP_3.REG_A[18].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A319_304 VIRT_WRAP_3.REG_A[19].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A335_320 VIRT_WRAP_3.REG_A[20].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A351_336 VIRT_WRAP_3.REG_A[21].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A367_352 VIRT_WRAP_3.REG_A[22].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A383_368 VIRT_WRAP_3.REG_A[23].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A399_384 VIRT_WRAP_3.REG_A[24].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A415_400 VIRT_WRAP_3.REG_A[25].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A431_416 VIRT_WRAP_3.REG_A[26].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A447_432 VIRT_WRAP_3.REG_A[27].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A463_448 VIRT_WRAP_3.REG_A[28].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A479_464 VIRT_WRAP_3.REG_A[29].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A495_480 VIRT_WRAP_3.REG_A[30].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_A511_496 VIRT_WRAP_3.REG_A[31].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B527_512 VIRT_WRAP_3.REG_B[0].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B543_528 VIRT_WRAP_3.REG_B[1].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B559_544 VIRT_WRAP_3.REG_B[2].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B575_560 VIRT_WRAP_3.REG_B[3].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B591_576 VIRT_WRAP_3.REG_B[4].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B607_592 VIRT_WRAP_3.REG_B[5].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B623_608 VIRT_WRAP_3.REG_B[6].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B639_624 VIRT_WRAP_3.REG_B[7].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B655_640 VIRT_WRAP_3.REG_B[8].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B671_656 VIRT_WRAP_3.REG_B[9].R    /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B687_672 VIRT_WRAP_3.REG_B[10].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B703_688 VIRT_WRAP_3.REG_B[11].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B719_704 VIRT_WRAP_3.REG_B[12].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B735_720 VIRT_WRAP_3.REG_B[13].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B751_736 VIRT_WRAP_3.REG_B[14].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B767_752 VIRT_WRAP_3.REG_B[15].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B783_768 VIRT_WRAP_3.REG_B[16].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B799_784 VIRT_WRAP_3.REG_B[17].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B815_800 VIRT_WRAP_3.REG_B[18].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B831_816 VIRT_WRAP_3.REG_B[19].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B847_832 VIRT_WRAP_3.REG_B[20].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B863_848 VIRT_WRAP_3.REG_B[21].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B879_864 VIRT_WRAP_3.REG_B[22].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B895_880 VIRT_WRAP_3.REG_B[23].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B911_896 VIRT_WRAP_3.REG_B[24].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B927_912 VIRT_WRAP_3.REG_B[25].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B943_928 VIRT_WRAP_3.REG_B[26].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B959_944 VIRT_WRAP_3.REG_B[27].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B975_960 VIRT_WRAP_3.REG_B[28].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B991_976 VIRT_WRAP_3.REG_B[29].R   /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B1007_992 VIRT_WRAP_3.REG_B[30].R  /* Parameter_n Register */
#define VIRT_WRAP_3_REG_B1023_1008 VIRT_WRAP_3.REG_B[31].R /* Parameter_n Register */
#define VIRT_WRAP_3_REG_C1039_1024 VIRT_WRAP_3.REG_C[0].R  /* Parameter_n Register */

#define VIRT_WRAP_1_REG_A15_0 VIRT_WRAP_1.REG_A[0].R       /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A31_16 VIRT_WRAP_1.REG_A[1].R      /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A47_32 VIRT_WRAP_1.REG_A[2].R      /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A63_48 VIRT_WRAP_1.REG_A[3].R      /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A79_64 VIRT_WRAP_1.REG_A[4].R      /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A95_80 VIRT_WRAP_1.REG_A[5].R      /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A111_96 VIRT_WRAP_1.REG_A[6].R     /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A127_112 VIRT_WRAP_1.REG_A[7].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A143_128 VIRT_WRAP_1.REG_A[8].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A159_144 VIRT_WRAP_1.REG_A[9].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A175_160 VIRT_WRAP_1.REG_A[10].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A191_176 VIRT_WRAP_1.REG_A[11].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A207_192 VIRT_WRAP_1.REG_A[12].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A223_208 VIRT_WRAP_1.REG_A[13].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A239_224 VIRT_WRAP_1.REG_A[14].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A255_240 VIRT_WRAP_1.REG_A[15].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A271_256 VIRT_WRAP_1.REG_A[16].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A287_272 VIRT_WRAP_1.REG_A[17].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A303_288 VIRT_WRAP_1.REG_A[18].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A319_304 VIRT_WRAP_1.REG_A[19].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A335_320 VIRT_WRAP_1.REG_A[20].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A351_336 VIRT_WRAP_1.REG_A[21].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A367_352 VIRT_WRAP_1.REG_A[22].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A383_368 VIRT_WRAP_1.REG_A[23].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A399_384 VIRT_WRAP_1.REG_A[24].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A415_400 VIRT_WRAP_1.REG_A[25].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A431_416 VIRT_WRAP_1.REG_A[26].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A447_432 VIRT_WRAP_1.REG_A[27].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A463_448 VIRT_WRAP_1.REG_A[28].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A479_464 VIRT_WRAP_1.REG_A[29].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A495_480 VIRT_WRAP_1.REG_A[30].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_A511_496 VIRT_WRAP_1.REG_A[31].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B527_512 VIRT_WRAP_1.REG_B[0].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B543_528 VIRT_WRAP_1.REG_B[1].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B559_544 VIRT_WRAP_1.REG_B[2].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B575_560 VIRT_WRAP_1.REG_B[3].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B591_576 VIRT_WRAP_1.REG_B[4].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B607_592 VIRT_WRAP_1.REG_B[5].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B623_608 VIRT_WRAP_1.REG_B[6].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B639_624 VIRT_WRAP_1.REG_B[7].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B655_640 VIRT_WRAP_1.REG_B[8].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B671_656 VIRT_WRAP_1.REG_B[9].R    /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B687_672 VIRT_WRAP_1.REG_B[10].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B703_688 VIRT_WRAP_1.REG_B[11].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B719_704 VIRT_WRAP_1.REG_B[12].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B735_720 VIRT_WRAP_1.REG_B[13].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B751_736 VIRT_WRAP_1.REG_B[14].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B767_752 VIRT_WRAP_1.REG_B[15].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B783_768 VIRT_WRAP_1.REG_B[16].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B799_784 VIRT_WRAP_1.REG_B[17].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B815_800 VIRT_WRAP_1.REG_B[18].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B831_816 VIRT_WRAP_1.REG_B[19].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B847_832 VIRT_WRAP_1.REG_B[20].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B863_848 VIRT_WRAP_1.REG_B[21].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B879_864 VIRT_WRAP_1.REG_B[22].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B895_880 VIRT_WRAP_1.REG_B[23].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B911_896 VIRT_WRAP_1.REG_B[24].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B927_912 VIRT_WRAP_1.REG_B[25].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B943_928 VIRT_WRAP_1.REG_B[26].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B959_944 VIRT_WRAP_1.REG_B[27].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B975_960 VIRT_WRAP_1.REG_B[28].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B991_976 VIRT_WRAP_1.REG_B[29].R   /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B1007_992 VIRT_WRAP_1.REG_B[30].R  /* Parameter_n Register */
#define VIRT_WRAP_1_REG_B1023_1008 VIRT_WRAP_1.REG_B[31].R /* Parameter_n Register */
#define VIRT_WRAP_1_REG_C1039_1024 VIRT_WRAP_1.REG_C[0].R  /* Parameter_n Register */

#define VIRT_WRAP_0_REG_A15_0 VIRT_WRAP_0.REG_A[0].R       /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A31_16 VIRT_WRAP_0.REG_A[1].R      /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A47_32 VIRT_WRAP_0.REG_A[2].R      /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A63_48 VIRT_WRAP_0.REG_A[3].R      /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A79_64 VIRT_WRAP_0.REG_A[4].R      /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A95_80 VIRT_WRAP_0.REG_A[5].R      /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A111_96 VIRT_WRAP_0.REG_A[6].R     /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A127_112 VIRT_WRAP_0.REG_A[7].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A143_128 VIRT_WRAP_0.REG_A[8].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A159_144 VIRT_WRAP_0.REG_A[9].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A175_160 VIRT_WRAP_0.REG_A[10].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A191_176 VIRT_WRAP_0.REG_A[11].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A207_192 VIRT_WRAP_0.REG_A[12].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A223_208 VIRT_WRAP_0.REG_A[13].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A239_224 VIRT_WRAP_0.REG_A[14].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A255_240 VIRT_WRAP_0.REG_A[15].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A271_256 VIRT_WRAP_0.REG_A[16].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A287_272 VIRT_WRAP_0.REG_A[17].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A303_288 VIRT_WRAP_0.REG_A[18].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A319_304 VIRT_WRAP_0.REG_A[19].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A335_320 VIRT_WRAP_0.REG_A[20].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A351_336 VIRT_WRAP_0.REG_A[21].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A367_352 VIRT_WRAP_0.REG_A[22].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A383_368 VIRT_WRAP_0.REG_A[23].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A399_384 VIRT_WRAP_0.REG_A[24].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A415_400 VIRT_WRAP_0.REG_A[25].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A431_416 VIRT_WRAP_0.REG_A[26].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A447_432 VIRT_WRAP_0.REG_A[27].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A463_448 VIRT_WRAP_0.REG_A[28].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A479_464 VIRT_WRAP_0.REG_A[29].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A495_480 VIRT_WRAP_0.REG_A[30].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_A511_496 VIRT_WRAP_0.REG_A[31].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B527_512 VIRT_WRAP_0.REG_B[0].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B543_528 VIRT_WRAP_0.REG_B[1].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B559_544 VIRT_WRAP_0.REG_B[2].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B575_560 VIRT_WRAP_0.REG_B[3].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B591_576 VIRT_WRAP_0.REG_B[4].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B607_592 VIRT_WRAP_0.REG_B[5].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B623_608 VIRT_WRAP_0.REG_B[6].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B639_624 VIRT_WRAP_0.REG_B[7].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B655_640 VIRT_WRAP_0.REG_B[8].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B671_656 VIRT_WRAP_0.REG_B[9].R    /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B687_672 VIRT_WRAP_0.REG_B[10].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B703_688 VIRT_WRAP_0.REG_B[11].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B719_704 VIRT_WRAP_0.REG_B[12].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B735_720 VIRT_WRAP_0.REG_B[13].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B751_736 VIRT_WRAP_0.REG_B[14].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B767_752 VIRT_WRAP_0.REG_B[15].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B783_768 VIRT_WRAP_0.REG_B[16].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B799_784 VIRT_WRAP_0.REG_B[17].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B815_800 VIRT_WRAP_0.REG_B[18].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B831_816 VIRT_WRAP_0.REG_B[19].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B847_832 VIRT_WRAP_0.REG_B[20].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B863_848 VIRT_WRAP_0.REG_B[21].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B879_864 VIRT_WRAP_0.REG_B[22].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B895_880 VIRT_WRAP_0.REG_B[23].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B911_896 VIRT_WRAP_0.REG_B[24].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B927_912 VIRT_WRAP_0.REG_B[25].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B943_928 VIRT_WRAP_0.REG_B[26].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B959_944 VIRT_WRAP_0.REG_B[27].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B975_960 VIRT_WRAP_0.REG_B[28].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B991_976 VIRT_WRAP_0.REG_B[29].R   /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B1007_992 VIRT_WRAP_0.REG_B[30].R  /* Parameter_n Register */
#define VIRT_WRAP_0_REG_B1023_1008 VIRT_WRAP_0.REG_B[31].R /* Parameter_n Register */
#define VIRT_WRAP_0_REG_C1039_1024 VIRT_WRAP_0.REG_C[0].R  /* Parameter_n Register */

#define VIRT_WRAP_4_REG_A15_0 VIRT_WRAP_4.REG_A[0].R       /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A31_16 VIRT_WRAP_4.REG_A[1].R      /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A47_32 VIRT_WRAP_4.REG_A[2].R      /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A63_48 VIRT_WRAP_4.REG_A[3].R      /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A79_64 VIRT_WRAP_4.REG_A[4].R      /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A95_80 VIRT_WRAP_4.REG_A[5].R      /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A111_96 VIRT_WRAP_4.REG_A[6].R     /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A127_112 VIRT_WRAP_4.REG_A[7].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A143_128 VIRT_WRAP_4.REG_A[8].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A159_144 VIRT_WRAP_4.REG_A[9].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A175_160 VIRT_WRAP_4.REG_A[10].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A191_176 VIRT_WRAP_4.REG_A[11].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A207_192 VIRT_WRAP_4.REG_A[12].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A223_208 VIRT_WRAP_4.REG_A[13].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A239_224 VIRT_WRAP_4.REG_A[14].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A255_240 VIRT_WRAP_4.REG_A[15].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A271_256 VIRT_WRAP_4.REG_A[16].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A287_272 VIRT_WRAP_4.REG_A[17].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A303_288 VIRT_WRAP_4.REG_A[18].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A319_304 VIRT_WRAP_4.REG_A[19].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A335_320 VIRT_WRAP_4.REG_A[20].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A351_336 VIRT_WRAP_4.REG_A[21].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A367_352 VIRT_WRAP_4.REG_A[22].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A383_368 VIRT_WRAP_4.REG_A[23].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A399_384 VIRT_WRAP_4.REG_A[24].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A415_400 VIRT_WRAP_4.REG_A[25].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A431_416 VIRT_WRAP_4.REG_A[26].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A447_432 VIRT_WRAP_4.REG_A[27].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A463_448 VIRT_WRAP_4.REG_A[28].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A479_464 VIRT_WRAP_4.REG_A[29].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A495_480 VIRT_WRAP_4.REG_A[30].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_A511_496 VIRT_WRAP_4.REG_A[31].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B527_512 VIRT_WRAP_4.REG_B[0].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B543_528 VIRT_WRAP_4.REG_B[1].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B559_544 VIRT_WRAP_4.REG_B[2].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B575_560 VIRT_WRAP_4.REG_B[3].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B591_576 VIRT_WRAP_4.REG_B[4].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B607_592 VIRT_WRAP_4.REG_B[5].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B623_608 VIRT_WRAP_4.REG_B[6].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B639_624 VIRT_WRAP_4.REG_B[7].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B655_640 VIRT_WRAP_4.REG_B[8].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B671_656 VIRT_WRAP_4.REG_B[9].R    /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B687_672 VIRT_WRAP_4.REG_B[10].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B703_688 VIRT_WRAP_4.REG_B[11].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B719_704 VIRT_WRAP_4.REG_B[12].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B735_720 VIRT_WRAP_4.REG_B[13].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B751_736 VIRT_WRAP_4.REG_B[14].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B767_752 VIRT_WRAP_4.REG_B[15].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B783_768 VIRT_WRAP_4.REG_B[16].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B799_784 VIRT_WRAP_4.REG_B[17].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B815_800 VIRT_WRAP_4.REG_B[18].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B831_816 VIRT_WRAP_4.REG_B[19].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B847_832 VIRT_WRAP_4.REG_B[20].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B863_848 VIRT_WRAP_4.REG_B[21].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B879_864 VIRT_WRAP_4.REG_B[22].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B895_880 VIRT_WRAP_4.REG_B[23].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B911_896 VIRT_WRAP_4.REG_B[24].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B927_912 VIRT_WRAP_4.REG_B[25].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B943_928 VIRT_WRAP_4.REG_B[26].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B959_944 VIRT_WRAP_4.REG_B[27].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B975_960 VIRT_WRAP_4.REG_B[28].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B991_976 VIRT_WRAP_4.REG_B[29].R   /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B1007_992 VIRT_WRAP_4.REG_B[30].R  /* Parameter_n Register */
#define VIRT_WRAP_4_REG_B1023_1008 VIRT_WRAP_4.REG_B[31].R /* Parameter_n Register */
#define VIRT_WRAP_4_REG_C1039_1024 VIRT_WRAP_4.REG_C[0].R  /* Parameter_n Register */

#define VIRT_WRAP_5_REG_A15_0 VIRT_WRAP_5.REG_A[0].R       /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A31_16 VIRT_WRAP_5.REG_A[1].R      /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A47_32 VIRT_WRAP_5.REG_A[2].R      /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A63_48 VIRT_WRAP_5.REG_A[3].R      /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A79_64 VIRT_WRAP_5.REG_A[4].R      /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A95_80 VIRT_WRAP_5.REG_A[5].R      /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A111_96 VIRT_WRAP_5.REG_A[6].R     /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A127_112 VIRT_WRAP_5.REG_A[7].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A143_128 VIRT_WRAP_5.REG_A[8].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A159_144 VIRT_WRAP_5.REG_A[9].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A175_160 VIRT_WRAP_5.REG_A[10].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A191_176 VIRT_WRAP_5.REG_A[11].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A207_192 VIRT_WRAP_5.REG_A[12].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A223_208 VIRT_WRAP_5.REG_A[13].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A239_224 VIRT_WRAP_5.REG_A[14].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A255_240 VIRT_WRAP_5.REG_A[15].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A271_256 VIRT_WRAP_5.REG_A[16].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A287_272 VIRT_WRAP_5.REG_A[17].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A303_288 VIRT_WRAP_5.REG_A[18].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A319_304 VIRT_WRAP_5.REG_A[19].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A335_320 VIRT_WRAP_5.REG_A[20].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A351_336 VIRT_WRAP_5.REG_A[21].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A367_352 VIRT_WRAP_5.REG_A[22].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A383_368 VIRT_WRAP_5.REG_A[23].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A399_384 VIRT_WRAP_5.REG_A[24].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A415_400 VIRT_WRAP_5.REG_A[25].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A431_416 VIRT_WRAP_5.REG_A[26].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A447_432 VIRT_WRAP_5.REG_A[27].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A463_448 VIRT_WRAP_5.REG_A[28].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A479_464 VIRT_WRAP_5.REG_A[29].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A495_480 VIRT_WRAP_5.REG_A[30].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_A511_496 VIRT_WRAP_5.REG_A[31].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B527_512 VIRT_WRAP_5.REG_B[0].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B543_528 VIRT_WRAP_5.REG_B[1].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B559_544 VIRT_WRAP_5.REG_B[2].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B575_560 VIRT_WRAP_5.REG_B[3].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B591_576 VIRT_WRAP_5.REG_B[4].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B607_592 VIRT_WRAP_5.REG_B[5].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B623_608 VIRT_WRAP_5.REG_B[6].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B639_624 VIRT_WRAP_5.REG_B[7].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B655_640 VIRT_WRAP_5.REG_B[8].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B671_656 VIRT_WRAP_5.REG_B[9].R    /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B687_672 VIRT_WRAP_5.REG_B[10].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B703_688 VIRT_WRAP_5.REG_B[11].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B719_704 VIRT_WRAP_5.REG_B[12].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B735_720 VIRT_WRAP_5.REG_B[13].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B751_736 VIRT_WRAP_5.REG_B[14].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B767_752 VIRT_WRAP_5.REG_B[15].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B783_768 VIRT_WRAP_5.REG_B[16].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B799_784 VIRT_WRAP_5.REG_B[17].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B815_800 VIRT_WRAP_5.REG_B[18].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B831_816 VIRT_WRAP_5.REG_B[19].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B847_832 VIRT_WRAP_5.REG_B[20].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B863_848 VIRT_WRAP_5.REG_B[21].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B879_864 VIRT_WRAP_5.REG_B[22].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B895_880 VIRT_WRAP_5.REG_B[23].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B911_896 VIRT_WRAP_5.REG_B[24].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B927_912 VIRT_WRAP_5.REG_B[25].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B943_928 VIRT_WRAP_5.REG_B[26].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B959_944 VIRT_WRAP_5.REG_B[27].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B975_960 VIRT_WRAP_5.REG_B[28].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B991_976 VIRT_WRAP_5.REG_B[29].R   /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B1007_992 VIRT_WRAP_5.REG_B[30].R  /* Parameter_n Register */
#define VIRT_WRAP_5_REG_B1023_1008 VIRT_WRAP_5.REG_B[31].R /* Parameter_n Register */
#define VIRT_WRAP_5_REG_C1039_1024 VIRT_WRAP_5.REG_C[0].R  /* Parameter_n Register */

/* WATCHDOG */
#define CEVA_SPF2__WATCHDOG_WDOGACS CEVA_SPF2__WATCHDOG.WDOGACS.R /* Watchdog Access Control */
#define CEVA_SPF2__WATCHDOG_WDOGCFG CEVA_SPF2__WATCHDOG.WDOGCFG.R /* System Watchdog Configuration */
#define CEVA_SPF2__WATCHDOG_WDOGMIN CEVA_SPF2__WATCHDOG.WDOGMIN.R /* System Watchdog Minimum Threshold Configuration */
#define CEVA_SPF2__WATCHDOG_WDOGMAX CEVA_SPF2__WATCHDOG.WDOGMAX.R /* System Watchdog Maximum Threshold Configuration */
#define CEVA_SPF2__WATCHDOG_WDOGCNT CEVA_SPF2__WATCHDOG.WDOGCNT.R /* System Watchdog Count Value */
#define CEVA_SPF2__WATCHDOG_MPWDOGCFG CEVA_SPF2__WATCHDOG.MPWDOGCFG.R /* Master Port Watchdog Control */
#define CEVA_SPF2__WATCHDOG_MPWDOGT CEVA_SPF2__WATCHDOG.MPWDOGT.R /* DMSS Master Port Watchdog Threshold */
#define CEVA_SPF2__WATCHDOG_EPPWDOGC CEVA_SPF2__WATCHDOG.EPPWDOGC.R /* PMSS Master Port Watchdog Threshold */
#define CEVA_SPF2__WATCHDOG_EDPWDOGC CEVA_SPF2__WATCHDOG.EDPWDOGC.R /* EDP Master Port Count Value */
#define CEVA_SPF2__WATCHDOG_ICUWDOGCFG CEVA_SPF2__WATCHDOG.ICUWDOGCFG.R /* ICU Watchdog Configuration */
#define CEVA_SPF2__WATCHDOG_ICUWDOGT CEVA_SPF2__WATCHDOG.ICUWDOGT.R /* ICU Watchdog Threshold Configuration */
#define CEVA_SPF2__WATCHDOG_SYSWDOGSIG CEVA_SPF2__WATCHDOG.SYSWDOGSIG.R /* System Watchdog Signal */
#define CEVA_SPF2__WATCHDOG_ICUWDOGC CEVA_SPF2__WATCHDOG.ICUWDOGC.R /* ICU Watchdog Count Value */

/* XBIC */
#define CE_XBIC_MCR          CE_XBIC.MCR.R                 /* XBIC Module Control */
#define CE_XBIC_EIR          CE_XBIC.EIR.R                 /* XBIC Error Injection Attributes */
#define CE_XBIC_ESR          CE_XBIC.ESR.R                 /* XBIC Error Status Attributes */
#define CE_XBIC_EAR          CE_XBIC.EAR.R                 /* XBIC Error Address */

#define AES__FEED_XBIC_AHB_MCR AES__FEED_XBIC_AHB.MCR.R    /* XBIC Module Control */
#define AES__FEED_XBIC_AHB_EIR AES__FEED_XBIC_AHB.EIR.R    /* XBIC Error Injection Attributes */
#define AES__FEED_XBIC_AHB_ESR AES__FEED_XBIC_AHB.ESR.R    /* XBIC Error Status Attributes */
#define AES__FEED_XBIC_AHB_EAR AES__FEED_XBIC_AHB.EAR.R    /* XBIC Error Address */

#define AES__RESULT_XBIC_AHB_MCR AES__RESULT_XBIC_AHB.MCR.R /* XBIC Module Control */
#define AES__RESULT_XBIC_AHB_EIR AES__RESULT_XBIC_AHB.EIR.R /* XBIC Error Injection Attributes */
#define AES__RESULT_XBIC_AHB_ESR AES__RESULT_XBIC_AHB.ESR.R /* XBIC Error Status Attributes */
#define AES__RESULT_XBIC_AHB_EAR AES__RESULT_XBIC_AHB.EAR.R /* XBIC Error Address */

#define SMU__XBIC_MCR        SMU__XBIC.MCR.R               /* XBIC Module Control */
#define SMU__XBIC_EIR        SMU__XBIC.EIR.R               /* XBIC Error Injection Attributes */
#define SMU__XBIC_ESR        SMU__XBIC.ESR.R               /* XBIC Error Status Attributes */
#define SMU__XBIC_EAR        SMU__XBIC.EAR.R               /* XBIC Error Address */

/* XRDC_0 */
#define XRDC_0_CR            XRDC_0.CR.R                   /* Control */
#define XRDC_0_HWCFG0        XRDC_0.HWCFG0.R               /* Hardware Configuration 0 */
#define XRDC_0_HWCFG1        XRDC_0.HWCFG1.R               /* Hardware Configuration 1 */
#define XRDC_0_MDACFG0       XRDC_0.MDACFG[0].R            /* Master Domain Assignment Configuration */
#define XRDC_0_MDACFG1       XRDC_0.MDACFG[1].R            /* Master Domain Assignment Configuration */
#define XRDC_0_MDACFG2       XRDC_0.MDACFG[2].R            /* Master Domain Assignment Configuration */
#define XRDC_0_MDACFG3       XRDC_0.MDACFG[3].R            /* Master Domain Assignment Configuration */
#define XRDC_0_MDACFG4       XRDC_0.MDACFG[4].R            /* Master Domain Assignment Configuration */
#define XRDC_0_MRCFG0        XRDC_0.MRCFG[0].R             /* Memory Region Configuration */
#define XRDC_0_DERRLOC0      XRDC_0.DERRLOC[0].R           /* Domain Error Location */
#define XRDC_0_DERRLOC1      XRDC_0.DERRLOC[1].R           /* Domain Error Location */
#define XRDC_0_DERRLOC2      XRDC_0.DERRLOC[2].R           /* Domain Error Location */
#define XRDC_0_DERRLOC3      XRDC_0.DERRLOC[3].R           /* Domain Error Location */
#define XRDC_0_DERRLOC4      XRDC_0.DERRLOC[4].R           /* Domain Error Location */
#define XRDC_0_DERRLOC5      XRDC_0.DERRLOC[5].R           /* Domain Error Location */
#define XRDC_0_DERRLOC6      XRDC_0.DERRLOC[6].R           /* Domain Error Location */
#define XRDC_0_DERRLOC7      XRDC_0.DERRLOC[7].R           /* Domain Error Location */
#define XRDC_0_DERRLOC8      XRDC_0.DERRLOC[8].R           /* Domain Error Location */
#define XRDC_0_DERRLOC9      XRDC_0.DERRLOC[9].R           /* Domain Error Location */
#define XRDC_0_DERRLOC10     XRDC_0.DERRLOC[10].R          /* Domain Error Location */
#define XRDC_0_DERRLOC11     XRDC_0.DERRLOC[11].R          /* Domain Error Location */
#define XRDC_0_DERRLOC12     XRDC_0.DERRLOC[12].R          /* Domain Error Location */
#define XRDC_0_DERRLOC13     XRDC_0.DERRLOC[13].R          /* Domain Error Location */
#define XRDC_0_DERRLOC14     XRDC_0.DERRLOC[14].R          /* Domain Error Location */
#define XRDC_0_DERRLOC15     XRDC_0.DERRLOC[15].R          /* Domain Error Location */
#define XRDC_0_DERR_W0_0     XRDC_0.DERRW0[0].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_0_DERR_W1_0     XRDC_0.DERRW0[0].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_0_DERR_W3_0     XRDC_0.DERRW0[0].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_0_DERR_W0_16    XRDC_0.DERRW0[16].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_0_DERR_W1_16    XRDC_0.DERRW0[16].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_0_DERR_W3_16    XRDC_0.DERRW0[16].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_0_DERR_W0_17    XRDC_0.DERRW0[17].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_0_DERR_W1_17    XRDC_0.DERRW0[17].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_0_DERR_W3_17    XRDC_0.DERRW0[17].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_0_DERR_W0_18    XRDC_0.DERRW0[18].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_0_DERR_W1_18    XRDC_0.DERRW0[18].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_0_DERR_W3_18    XRDC_0.DERRW0[18].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_0_DERR_W0_19    XRDC_0.DERRW0[19].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_0_DERR_W1_19    XRDC_0.DERRW0[19].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_0_DERR_W3_19    XRDC_0.DERRW0[19].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_0_MDA_W0_0_DFMT1 XRDC_0.MDA_W0_0_DFMT1.R      /* Master Domain Assignment */
#define XRDC_0_MDA_W0_1_DFMT1 XRDC_0.MDA_W0_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_0_MDA_W0_2_DFMT1 XRDC_0.MDA_W0_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_0_MDA_W1_2_DFMT1 XRDC_0.MDA_W1_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_0_MDA_W2_2_DFMT1 XRDC_0.MDA_W2_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_0_MDA_W3_2_DFMT1 XRDC_0.MDA_W3_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_0_MDA_W0_3_DFMT1 XRDC_0.MDA_W0_3_DFMT1.R      /* Master Domain Assignment */
#define XRDC_0_MDA_W0_4_DFMT1 XRDC_0.MDA_W0_4_DFMT1.R      /* Master Domain Assignment */
#define XRDC_0_PDAC_W0_0     XRDC_0.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_0     XRDC_0.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_1     XRDC_0.PDAC_SLOT[0].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_1     XRDC_0.PDAC_SLOT[0].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_2     XRDC_0.PDAC_SLOT[0].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_2     XRDC_0.PDAC_SLOT[0].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_3     XRDC_0.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_3     XRDC_0.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_4     XRDC_0.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_4     XRDC_0.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_5     XRDC_0.PDAC_SLOT[0].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_5     XRDC_0.PDAC_SLOT[0].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_6     XRDC_0.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_6     XRDC_0.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_7     XRDC_0.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_7     XRDC_0.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_9     XRDC_0.PDAC_SLOT[0].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_9     XRDC_0.PDAC_SLOT[0].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_10    XRDC_0.PDAC_SLOT[0].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_10    XRDC_0.PDAC_SLOT[0].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_11    XRDC_0.PDAC_SLOT[0].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_11    XRDC_0.PDAC_SLOT[0].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_13    XRDC_0.PDAC_SLOT[0].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_13    XRDC_0.PDAC_SLOT[0].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_14    XRDC_0.PDAC_SLOT[0].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_14    XRDC_0.PDAC_SLOT[0].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_18    XRDC_0.PDAC_SLOT[0].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_18    XRDC_0.PDAC_SLOT[0].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_19    XRDC_0.PDAC_SLOT[0].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_19    XRDC_0.PDAC_SLOT[0].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_20    XRDC_0.PDAC_SLOT[0].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_20    XRDC_0.PDAC_SLOT[0].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_23    XRDC_0.PDAC_SLOT[0].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_23    XRDC_0.PDAC_SLOT[0].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_24    XRDC_0.PDAC_SLOT[0].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_24    XRDC_0.PDAC_SLOT[0].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_25    XRDC_0.PDAC_SLOT[0].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_25    XRDC_0.PDAC_SLOT[0].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_27    XRDC_0.PDAC_SLOT[0].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_27    XRDC_0.PDAC_SLOT[0].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_28    XRDC_0.PDAC_SLOT[0].PDACN[28].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_28    XRDC_0.PDAC_SLOT[0].PDACN[28].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_29    XRDC_0.PDAC_SLOT[0].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_29    XRDC_0.PDAC_SLOT[0].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_30    XRDC_0.PDAC_SLOT[0].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_30    XRDC_0.PDAC_SLOT[0].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_31    XRDC_0.PDAC_SLOT[0].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_31    XRDC_0.PDAC_SLOT[0].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_32    XRDC_0.PDAC_SLOT[0].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_32    XRDC_0.PDAC_SLOT[0].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_33    XRDC_0.PDAC_SLOT[0].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_33    XRDC_0.PDAC_SLOT[0].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_34    XRDC_0.PDAC_SLOT[0].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_34    XRDC_0.PDAC_SLOT[0].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_37    XRDC_0.PDAC_SLOT[0].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_37    XRDC_0.PDAC_SLOT[0].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_38    XRDC_0.PDAC_SLOT[0].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_38    XRDC_0.PDAC_SLOT[0].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_39    XRDC_0.PDAC_SLOT[0].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_39    XRDC_0.PDAC_SLOT[0].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_43    XRDC_0.PDAC_SLOT[0].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_43    XRDC_0.PDAC_SLOT[0].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_44    XRDC_0.PDAC_SLOT[0].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_44    XRDC_0.PDAC_SLOT[0].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_45    XRDC_0.PDAC_SLOT[0].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_45    XRDC_0.PDAC_SLOT[0].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_46    XRDC_0.PDAC_SLOT[0].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_46    XRDC_0.PDAC_SLOT[0].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_47    XRDC_0.PDAC_SLOT[0].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_47    XRDC_0.PDAC_SLOT[0].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_48    XRDC_0.PDAC_SLOT[0].PDACN[48].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_48    XRDC_0.PDAC_SLOT[0].PDACN[48].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_49    XRDC_0.PDAC_SLOT[0].PDACN[49].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_49    XRDC_0.PDAC_SLOT[0].PDACN[49].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_51    XRDC_0.PDAC_SLOT[0].PDACN[51].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_51    XRDC_0.PDAC_SLOT[0].PDACN[51].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_52    XRDC_0.PDAC_SLOT[0].PDACN[52].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_52    XRDC_0.PDAC_SLOT[0].PDACN[52].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_55    XRDC_0.PDAC_SLOT[0].PDACN[55].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_55    XRDC_0.PDAC_SLOT[0].PDACN[55].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_57    XRDC_0.PDAC_SLOT[0].PDACN[57].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_57    XRDC_0.PDAC_SLOT[0].PDACN[57].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_58    XRDC_0.PDAC_SLOT[0].PDACN[58].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_58    XRDC_0.PDAC_SLOT[0].PDACN[58].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_59    XRDC_0.PDAC_SLOT[0].PDACN[59].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_59    XRDC_0.PDAC_SLOT[0].PDACN[59].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_60    XRDC_0.PDAC_SLOT[0].PDACN[60].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_60    XRDC_0.PDAC_SLOT[0].PDACN[60].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_61    XRDC_0.PDAC_SLOT[0].PDACN[61].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_61    XRDC_0.PDAC_SLOT[0].PDACN[61].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_62    XRDC_0.PDAC_SLOT[0].PDACN[62].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_62    XRDC_0.PDAC_SLOT[0].PDACN[62].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_128   XRDC_0.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_128   XRDC_0.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_256   XRDC_0.PDAC_SLOT[2].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_256   XRDC_0.PDAC_SLOT[2].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_257   XRDC_0.PDAC_SLOT[2].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_257   XRDC_0.PDAC_SLOT[2].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_258   XRDC_0.PDAC_SLOT[2].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_258   XRDC_0.PDAC_SLOT[2].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_259   XRDC_0.PDAC_SLOT[2].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_259   XRDC_0.PDAC_SLOT[2].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_260   XRDC_0.PDAC_SLOT[2].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_260   XRDC_0.PDAC_SLOT[2].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_261   XRDC_0.PDAC_SLOT[2].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_261   XRDC_0.PDAC_SLOT[2].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_262   XRDC_0.PDAC_SLOT[2].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_262   XRDC_0.PDAC_SLOT[2].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_263   XRDC_0.PDAC_SLOT[2].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_263   XRDC_0.PDAC_SLOT[2].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_264   XRDC_0.PDAC_SLOT[2].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_264   XRDC_0.PDAC_SLOT[2].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_265   XRDC_0.PDAC_SLOT[2].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_265   XRDC_0.PDAC_SLOT[2].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_266   XRDC_0.PDAC_SLOT[2].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_266   XRDC_0.PDAC_SLOT[2].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_267   XRDC_0.PDAC_SLOT[2].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_267   XRDC_0.PDAC_SLOT[2].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_268   XRDC_0.PDAC_SLOT[2].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_268   XRDC_0.PDAC_SLOT[2].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_269   XRDC_0.PDAC_SLOT[2].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_269   XRDC_0.PDAC_SLOT[2].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_270   XRDC_0.PDAC_SLOT[2].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_270   XRDC_0.PDAC_SLOT[2].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_271   XRDC_0.PDAC_SLOT[2].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_271   XRDC_0.PDAC_SLOT[2].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_272   XRDC_0.PDAC_SLOT[2].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_272   XRDC_0.PDAC_SLOT[2].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_273   XRDC_0.PDAC_SLOT[2].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_273   XRDC_0.PDAC_SLOT[2].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_274   XRDC_0.PDAC_SLOT[2].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_274   XRDC_0.PDAC_SLOT[2].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_275   XRDC_0.PDAC_SLOT[2].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_275   XRDC_0.PDAC_SLOT[2].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_276   XRDC_0.PDAC_SLOT[2].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_276   XRDC_0.PDAC_SLOT[2].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_277   XRDC_0.PDAC_SLOT[2].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_277   XRDC_0.PDAC_SLOT[2].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_278   XRDC_0.PDAC_SLOT[2].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_278   XRDC_0.PDAC_SLOT[2].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_279   XRDC_0.PDAC_SLOT[2].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_279   XRDC_0.PDAC_SLOT[2].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_280   XRDC_0.PDAC_SLOT[2].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_280   XRDC_0.PDAC_SLOT[2].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_281   XRDC_0.PDAC_SLOT[2].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_281   XRDC_0.PDAC_SLOT[2].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_282   XRDC_0.PDAC_SLOT[2].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_282   XRDC_0.PDAC_SLOT[2].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_283   XRDC_0.PDAC_SLOT[2].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_283   XRDC_0.PDAC_SLOT[2].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_284   XRDC_0.PDAC_SLOT[2].PDACN[28].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_284   XRDC_0.PDAC_SLOT[2].PDACN[28].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_285   XRDC_0.PDAC_SLOT[2].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_285   XRDC_0.PDAC_SLOT[2].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_286   XRDC_0.PDAC_SLOT[2].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_286   XRDC_0.PDAC_SLOT[2].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_287   XRDC_0.PDAC_SLOT[2].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_287   XRDC_0.PDAC_SLOT[2].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_288   XRDC_0.PDAC_SLOT[2].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_288   XRDC_0.PDAC_SLOT[2].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_289   XRDC_0.PDAC_SLOT[2].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_289   XRDC_0.PDAC_SLOT[2].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_290   XRDC_0.PDAC_SLOT[2].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_290   XRDC_0.PDAC_SLOT[2].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_291   XRDC_0.PDAC_SLOT[2].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_291   XRDC_0.PDAC_SLOT[2].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_292   XRDC_0.PDAC_SLOT[2].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_292   XRDC_0.PDAC_SLOT[2].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_293   XRDC_0.PDAC_SLOT[2].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_293   XRDC_0.PDAC_SLOT[2].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_294   XRDC_0.PDAC_SLOT[2].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_294   XRDC_0.PDAC_SLOT[2].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_295   XRDC_0.PDAC_SLOT[2].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_295   XRDC_0.PDAC_SLOT[2].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_296   XRDC_0.PDAC_SLOT[2].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_296   XRDC_0.PDAC_SLOT[2].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_297   XRDC_0.PDAC_SLOT[2].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_297   XRDC_0.PDAC_SLOT[2].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_298   XRDC_0.PDAC_SLOT[2].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_298   XRDC_0.PDAC_SLOT[2].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_299   XRDC_0.PDAC_SLOT[2].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_299   XRDC_0.PDAC_SLOT[2].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_300   XRDC_0.PDAC_SLOT[2].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_300   XRDC_0.PDAC_SLOT[2].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_301   XRDC_0.PDAC_SLOT[2].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_301   XRDC_0.PDAC_SLOT[2].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_302   XRDC_0.PDAC_SLOT[2].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_302   XRDC_0.PDAC_SLOT[2].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_303   XRDC_0.PDAC_SLOT[2].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_303   XRDC_0.PDAC_SLOT[2].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_304   XRDC_0.PDAC_SLOT[2].PDACN[48].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_304   XRDC_0.PDAC_SLOT[2].PDACN[48].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_305   XRDC_0.PDAC_SLOT[2].PDACN[49].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_305   XRDC_0.PDAC_SLOT[2].PDACN[49].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_306   XRDC_0.PDAC_SLOT[2].PDACN[50].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_306   XRDC_0.PDAC_SLOT[2].PDACN[50].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_307   XRDC_0.PDAC_SLOT[2].PDACN[51].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_307   XRDC_0.PDAC_SLOT[2].PDACN[51].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_308   XRDC_0.PDAC_SLOT[2].PDACN[52].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_308   XRDC_0.PDAC_SLOT[2].PDACN[52].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_309   XRDC_0.PDAC_SLOT[2].PDACN[53].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_309   XRDC_0.PDAC_SLOT[2].PDACN[53].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_310   XRDC_0.PDAC_SLOT[2].PDACN[54].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_310   XRDC_0.PDAC_SLOT[2].PDACN[54].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_385   XRDC_0.PDAC_SLOT[3].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_385   XRDC_0.PDAC_SLOT[3].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_386   XRDC_0.PDAC_SLOT[3].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_386   XRDC_0.PDAC_SLOT[3].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_402   XRDC_0.PDAC_SLOT[3].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_402   XRDC_0.PDAC_SLOT[3].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_403   XRDC_0.PDAC_SLOT[3].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_403   XRDC_0.PDAC_SLOT[3].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_404   XRDC_0.PDAC_SLOT[3].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_404   XRDC_0.PDAC_SLOT[3].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_405   XRDC_0.PDAC_SLOT[3].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_405   XRDC_0.PDAC_SLOT[3].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_406   XRDC_0.PDAC_SLOT[3].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_406   XRDC_0.PDAC_SLOT[3].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_410   XRDC_0.PDAC_SLOT[3].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_410   XRDC_0.PDAC_SLOT[3].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_411   XRDC_0.PDAC_SLOT[3].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_411   XRDC_0.PDAC_SLOT[3].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_413   XRDC_0.PDAC_SLOT[3].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_413   XRDC_0.PDAC_SLOT[3].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_414   XRDC_0.PDAC_SLOT[3].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_414   XRDC_0.PDAC_SLOT[3].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_416   XRDC_0.PDAC_SLOT[3].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_416   XRDC_0.PDAC_SLOT[3].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_417   XRDC_0.PDAC_SLOT[3].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_417   XRDC_0.PDAC_SLOT[3].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_418   XRDC_0.PDAC_SLOT[3].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_418   XRDC_0.PDAC_SLOT[3].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_419   XRDC_0.PDAC_SLOT[3].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_419   XRDC_0.PDAC_SLOT[3].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_420   XRDC_0.PDAC_SLOT[3].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_420   XRDC_0.PDAC_SLOT[3].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_421   XRDC_0.PDAC_SLOT[3].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_421   XRDC_0.PDAC_SLOT[3].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_422   XRDC_0.PDAC_SLOT[3].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_422   XRDC_0.PDAC_SLOT[3].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_423   XRDC_0.PDAC_SLOT[3].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_423   XRDC_0.PDAC_SLOT[3].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_424   XRDC_0.PDAC_SLOT[3].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_424   XRDC_0.PDAC_SLOT[3].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_425   XRDC_0.PDAC_SLOT[3].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_425   XRDC_0.PDAC_SLOT[3].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_426   XRDC_0.PDAC_SLOT[3].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_426   XRDC_0.PDAC_SLOT[3].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_427   XRDC_0.PDAC_SLOT[3].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_427   XRDC_0.PDAC_SLOT[3].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_428   XRDC_0.PDAC_SLOT[3].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_428   XRDC_0.PDAC_SLOT[3].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_429   XRDC_0.PDAC_SLOT[3].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_429   XRDC_0.PDAC_SLOT[3].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_430   XRDC_0.PDAC_SLOT[3].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_430   XRDC_0.PDAC_SLOT[3].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_431   XRDC_0.PDAC_SLOT[3].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_431   XRDC_0.PDAC_SLOT[3].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_432   XRDC_0.PDAC_SLOT[3].PDACN[48].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_432   XRDC_0.PDAC_SLOT[3].PDACN[48].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_433   XRDC_0.PDAC_SLOT[3].PDACN[49].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_433   XRDC_0.PDAC_SLOT[3].PDACN[49].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_434   XRDC_0.PDAC_SLOT[3].PDACN[50].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_434   XRDC_0.PDAC_SLOT[3].PDACN[50].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_435   XRDC_0.PDAC_SLOT[3].PDACN[51].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_435   XRDC_0.PDAC_SLOT[3].PDACN[51].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_436   XRDC_0.PDAC_SLOT[3].PDACN[52].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_436   XRDC_0.PDAC_SLOT[3].PDACN[52].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_437   XRDC_0.PDAC_SLOT[3].PDACN[53].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_437   XRDC_0.PDAC_SLOT[3].PDACN[53].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_438   XRDC_0.PDAC_SLOT[3].PDACN[54].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_438   XRDC_0.PDAC_SLOT[3].PDACN[54].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_439   XRDC_0.PDAC_SLOT[3].PDACN[55].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_439   XRDC_0.PDAC_SLOT[3].PDACN[55].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_440   XRDC_0.PDAC_SLOT[3].PDACN[56].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_440   XRDC_0.PDAC_SLOT[3].PDACN[56].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_441   XRDC_0.PDAC_SLOT[3].PDACN[57].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_441   XRDC_0.PDAC_SLOT[3].PDACN[57].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_442   XRDC_0.PDAC_SLOT[3].PDACN[58].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_442   XRDC_0.PDAC_SLOT[3].PDACN[58].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_443   XRDC_0.PDAC_SLOT[3].PDACN[59].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_443   XRDC_0.PDAC_SLOT[3].PDACN[59].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_444   XRDC_0.PDAC_SLOT[3].PDACN[60].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_444   XRDC_0.PDAC_SLOT[3].PDACN[60].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_445   XRDC_0.PDAC_SLOT[3].PDACN[61].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_445   XRDC_0.PDAC_SLOT[3].PDACN[61].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_446   XRDC_0.PDAC_SLOT[3].PDACN[62].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_446   XRDC_0.PDAC_SLOT[3].PDACN[62].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_PDAC_W0_447   XRDC_0.PDAC_SLOT[3].PDACN[63].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_0_PDAC_W1_447   XRDC_0.PDAC_SLOT[3].PDACN[63].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_0_MRGD_W0_0     XRDC_0.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_0     XRDC_0.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_0     XRDC_0.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_0     XRDC_0.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_1     XRDC_0.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_1     XRDC_0.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_1     XRDC_0.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_1     XRDC_0.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_2     XRDC_0.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_2     XRDC_0.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_2     XRDC_0.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_2     XRDC_0.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_3     XRDC_0.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_3     XRDC_0.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_3     XRDC_0.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_3     XRDC_0.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_4     XRDC_0.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_4     XRDC_0.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_4     XRDC_0.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_4     XRDC_0.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_5     XRDC_0.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_5     XRDC_0.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_5     XRDC_0.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_5     XRDC_0.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_6     XRDC_0.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_6     XRDC_0.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_6     XRDC_0.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_6     XRDC_0.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_7     XRDC_0.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_7     XRDC_0.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_7     XRDC_0.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_7     XRDC_0.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_8     XRDC_0.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_8     XRDC_0.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_8     XRDC_0.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_8     XRDC_0.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_9     XRDC_0.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_9     XRDC_0.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_9     XRDC_0.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_9     XRDC_0.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_10    XRDC_0.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_10    XRDC_0.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_10    XRDC_0.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_10    XRDC_0.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_11    XRDC_0.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_11    XRDC_0.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_11    XRDC_0.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_11    XRDC_0.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_12    XRDC_0.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_12    XRDC_0.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_12    XRDC_0.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_12    XRDC_0.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_13    XRDC_0.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_13    XRDC_0.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_13    XRDC_0.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_13    XRDC_0.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_14    XRDC_0.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_14    XRDC_0.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_14    XRDC_0.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_14    XRDC_0.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_0_MRGD_W0_15    XRDC_0.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_0_MRGD_W1_15    XRDC_0.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_0_MRGD_W2_15    XRDC_0.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_0_MRGD_W3_15    XRDC_0.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* XRDC_1 */
#define XRDC_1_CR            XRDC_1.CR.R                   /* Control */
#define XRDC_1_HWCFG0        XRDC_1.HWCFG0.R               /* Hardware Configuration 0 */
#define XRDC_1_HWCFG1        XRDC_1.HWCFG1.R               /* Hardware Configuration 1 */
#define XRDC_1_MDACFG0       XRDC_1.MDACFG[0].R            /* Master Domain Assignment Configuration */
#define XRDC_1_MDACFG1       XRDC_1.MDACFG[1].R            /* Master Domain Assignment Configuration */
#define XRDC_1_MDACFG2       XRDC_1.MDACFG[2].R            /* Master Domain Assignment Configuration */
#define XRDC_1_MDACFG3       XRDC_1.MDACFG[3].R            /* Master Domain Assignment Configuration */
#define XRDC_1_MDACFG4       XRDC_1.MDACFG[4].R            /* Master Domain Assignment Configuration */
#define XRDC_1_MRCFG0        XRDC_1.MRCFG[0].R             /* Memory Region Configuration */
#define XRDC_1_DERRLOC0      XRDC_1.DERRLOC[0].R           /* Domain Error Location */
#define XRDC_1_DERRLOC1      XRDC_1.DERRLOC[1].R           /* Domain Error Location */
#define XRDC_1_DERRLOC2      XRDC_1.DERRLOC[2].R           /* Domain Error Location */
#define XRDC_1_DERRLOC3      XRDC_1.DERRLOC[3].R           /* Domain Error Location */
#define XRDC_1_DERRLOC4      XRDC_1.DERRLOC[4].R           /* Domain Error Location */
#define XRDC_1_DERRLOC5      XRDC_1.DERRLOC[5].R           /* Domain Error Location */
#define XRDC_1_DERRLOC6      XRDC_1.DERRLOC[6].R           /* Domain Error Location */
#define XRDC_1_DERRLOC7      XRDC_1.DERRLOC[7].R           /* Domain Error Location */
#define XRDC_1_DERRLOC8      XRDC_1.DERRLOC[8].R           /* Domain Error Location */
#define XRDC_1_DERRLOC9      XRDC_1.DERRLOC[9].R           /* Domain Error Location */
#define XRDC_1_DERRLOC10     XRDC_1.DERRLOC[10].R          /* Domain Error Location */
#define XRDC_1_DERRLOC11     XRDC_1.DERRLOC[11].R          /* Domain Error Location */
#define XRDC_1_DERRLOC12     XRDC_1.DERRLOC[12].R          /* Domain Error Location */
#define XRDC_1_DERRLOC13     XRDC_1.DERRLOC[13].R          /* Domain Error Location */
#define XRDC_1_DERRLOC14     XRDC_1.DERRLOC[14].R          /* Domain Error Location */
#define XRDC_1_DERRLOC15     XRDC_1.DERRLOC[15].R          /* Domain Error Location */
#define XRDC_1_DERR_W0_0     XRDC_1.DERRW0[0].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_1_DERR_W1_0     XRDC_1.DERRW0[0].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_1_DERR_W3_0     XRDC_1.DERRW0[0].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_1_DERR_W0_16    XRDC_1.DERRW0[16].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_1_DERR_W1_16    XRDC_1.DERRW0[16].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_1_DERR_W3_16    XRDC_1.DERRW0[16].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_1_DERR_W0_17    XRDC_1.DERRW0[17].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_1_DERR_W1_17    XRDC_1.DERRW0[17].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_1_DERR_W3_17    XRDC_1.DERRW0[17].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_1_DERR_W0_18    XRDC_1.DERRW0[18].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_1_DERR_W1_18    XRDC_1.DERRW0[18].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_1_DERR_W3_18    XRDC_1.DERRW0[18].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_1_DERR_W0_19    XRDC_1.DERRW0[19].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_1_DERR_W1_19    XRDC_1.DERRW0[19].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_1_DERR_W3_19    XRDC_1.DERRW0[19].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_1_MDA_W0_0_DFMT1 XRDC_1.MDA_W0_0_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W0_1_DFMT1 XRDC_1.MDA_W0_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W1_1_DFMT1 XRDC_1.MDA_W1_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W2_1_DFMT1 XRDC_1.MDA_W2_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W3_1_DFMT1 XRDC_1.MDA_W3_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W4_1_DFMT1 XRDC_1.MDA_W4_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W5_1_DFMT1 XRDC_1.MDA_W5_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W6_1_DFMT1 XRDC_1.MDA_W6_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W7_1_DFMT1 XRDC_1.MDA_W7_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W0_2_DFMT1 XRDC_1.MDA_W0_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W0_3_DFMT1 XRDC_1.MDA_W0_3_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_MDA_W0_4_DFMT1 XRDC_1.MDA_W0_4_DFMT1.R      /* Master Domain Assignment */
#define XRDC_1_PDAC_W0_0     XRDC_1.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_0     XRDC_1.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_1     XRDC_1.PDAC_SLOT[0].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_1     XRDC_1.PDAC_SLOT[0].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_3     XRDC_1.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_3     XRDC_1.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_4     XRDC_1.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_4     XRDC_1.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_5     XRDC_1.PDAC_SLOT[0].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_5     XRDC_1.PDAC_SLOT[0].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_6     XRDC_1.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_6     XRDC_1.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_7     XRDC_1.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_7     XRDC_1.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_8     XRDC_1.PDAC_SLOT[0].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_8     XRDC_1.PDAC_SLOT[0].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_9     XRDC_1.PDAC_SLOT[0].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_9     XRDC_1.PDAC_SLOT[0].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_13    XRDC_1.PDAC_SLOT[0].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_13    XRDC_1.PDAC_SLOT[0].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_18    XRDC_1.PDAC_SLOT[0].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_18    XRDC_1.PDAC_SLOT[0].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_19    XRDC_1.PDAC_SLOT[0].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_19    XRDC_1.PDAC_SLOT[0].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_20    XRDC_1.PDAC_SLOT[0].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_20    XRDC_1.PDAC_SLOT[0].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_21    XRDC_1.PDAC_SLOT[0].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_21    XRDC_1.PDAC_SLOT[0].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_23    XRDC_1.PDAC_SLOT[0].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_23    XRDC_1.PDAC_SLOT[0].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_24    XRDC_1.PDAC_SLOT[0].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_24    XRDC_1.PDAC_SLOT[0].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_25    XRDC_1.PDAC_SLOT[0].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_25    XRDC_1.PDAC_SLOT[0].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_29    XRDC_1.PDAC_SLOT[0].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_29    XRDC_1.PDAC_SLOT[0].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_128   XRDC_1.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_128   XRDC_1.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_257   XRDC_1.PDAC_SLOT[2].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_257   XRDC_1.PDAC_SLOT[2].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_258   XRDC_1.PDAC_SLOT[2].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_258   XRDC_1.PDAC_SLOT[2].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_259   XRDC_1.PDAC_SLOT[2].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_259   XRDC_1.PDAC_SLOT[2].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_260   XRDC_1.PDAC_SLOT[2].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_260   XRDC_1.PDAC_SLOT[2].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_261   XRDC_1.PDAC_SLOT[2].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_261   XRDC_1.PDAC_SLOT[2].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_385   XRDC_1.PDAC_SLOT[3].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_385   XRDC_1.PDAC_SLOT[3].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_386   XRDC_1.PDAC_SLOT[3].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_386   XRDC_1.PDAC_SLOT[3].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_402   XRDC_1.PDAC_SLOT[3].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_402   XRDC_1.PDAC_SLOT[3].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_403   XRDC_1.PDAC_SLOT[3].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_403   XRDC_1.PDAC_SLOT[3].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_404   XRDC_1.PDAC_SLOT[3].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_404   XRDC_1.PDAC_SLOT[3].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_405   XRDC_1.PDAC_SLOT[3].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_405   XRDC_1.PDAC_SLOT[3].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_406   XRDC_1.PDAC_SLOT[3].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_406   XRDC_1.PDAC_SLOT[3].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_410   XRDC_1.PDAC_SLOT[3].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_410   XRDC_1.PDAC_SLOT[3].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_413   XRDC_1.PDAC_SLOT[3].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_413   XRDC_1.PDAC_SLOT[3].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_414   XRDC_1.PDAC_SLOT[3].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_414   XRDC_1.PDAC_SLOT[3].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_416   XRDC_1.PDAC_SLOT[3].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_416   XRDC_1.PDAC_SLOT[3].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_417   XRDC_1.PDAC_SLOT[3].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_417   XRDC_1.PDAC_SLOT[3].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_418   XRDC_1.PDAC_SLOT[3].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_418   XRDC_1.PDAC_SLOT[3].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_419   XRDC_1.PDAC_SLOT[3].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_419   XRDC_1.PDAC_SLOT[3].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_420   XRDC_1.PDAC_SLOT[3].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_420   XRDC_1.PDAC_SLOT[3].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_421   XRDC_1.PDAC_SLOT[3].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_421   XRDC_1.PDAC_SLOT[3].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_422   XRDC_1.PDAC_SLOT[3].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_422   XRDC_1.PDAC_SLOT[3].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_423   XRDC_1.PDAC_SLOT[3].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_423   XRDC_1.PDAC_SLOT[3].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_424   XRDC_1.PDAC_SLOT[3].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_424   XRDC_1.PDAC_SLOT[3].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_425   XRDC_1.PDAC_SLOT[3].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_425   XRDC_1.PDAC_SLOT[3].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_426   XRDC_1.PDAC_SLOT[3].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_426   XRDC_1.PDAC_SLOT[3].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_427   XRDC_1.PDAC_SLOT[3].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_427   XRDC_1.PDAC_SLOT[3].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_428   XRDC_1.PDAC_SLOT[3].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_428   XRDC_1.PDAC_SLOT[3].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_429   XRDC_1.PDAC_SLOT[3].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_429   XRDC_1.PDAC_SLOT[3].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_430   XRDC_1.PDAC_SLOT[3].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_430   XRDC_1.PDAC_SLOT[3].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_PDAC_W0_431   XRDC_1.PDAC_SLOT[3].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_1_PDAC_W1_431   XRDC_1.PDAC_SLOT[3].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_1_MRGD_W0_0     XRDC_1.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_0     XRDC_1.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_0     XRDC_1.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_0     XRDC_1.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_1     XRDC_1.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_1     XRDC_1.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_1     XRDC_1.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_1     XRDC_1.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_2     XRDC_1.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_2     XRDC_1.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_2     XRDC_1.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_2     XRDC_1.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_3     XRDC_1.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_3     XRDC_1.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_3     XRDC_1.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_3     XRDC_1.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_4     XRDC_1.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_4     XRDC_1.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_4     XRDC_1.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_4     XRDC_1.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_5     XRDC_1.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_5     XRDC_1.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_5     XRDC_1.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_5     XRDC_1.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_6     XRDC_1.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_6     XRDC_1.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_6     XRDC_1.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_6     XRDC_1.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_7     XRDC_1.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_7     XRDC_1.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_7     XRDC_1.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_7     XRDC_1.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_8     XRDC_1.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_8     XRDC_1.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_8     XRDC_1.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_8     XRDC_1.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_9     XRDC_1.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_9     XRDC_1.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_9     XRDC_1.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_9     XRDC_1.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_10    XRDC_1.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_10    XRDC_1.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_10    XRDC_1.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_10    XRDC_1.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_11    XRDC_1.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_11    XRDC_1.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_11    XRDC_1.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_11    XRDC_1.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_12    XRDC_1.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_12    XRDC_1.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_12    XRDC_1.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_12    XRDC_1.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_13    XRDC_1.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_13    XRDC_1.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_13    XRDC_1.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_13    XRDC_1.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_14    XRDC_1.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_14    XRDC_1.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_14    XRDC_1.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_14    XRDC_1.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_1_MRGD_W0_15    XRDC_1.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_1_MRGD_W1_15    XRDC_1.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_1_MRGD_W2_15    XRDC_1.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_1_MRGD_W3_15    XRDC_1.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* XRDC_2 */
#define XRDC_2_CR            XRDC_2.CR.R                   /* Control */
#define XRDC_2_HWCFG0        XRDC_2.HWCFG0.R               /* Hardware Configuration 0 */
#define XRDC_2_HWCFG1        XRDC_2.HWCFG1.R               /* Hardware Configuration 1 */
#define XRDC_2_MDACFG0       XRDC_2.MDACFG[0].R            /* Master Domain Assignment Configuration */
#define XRDC_2_MDACFG1       XRDC_2.MDACFG[1].R            /* Master Domain Assignment Configuration */
#define XRDC_2_MDACFG2       XRDC_2.MDACFG[2].R            /* Master Domain Assignment Configuration */
#define XRDC_2_MRCFG0        XRDC_2.MRCFG[0].R             /* Memory Region Configuration */
#define XRDC_2_MRCFG1        XRDC_2.MRCFG[1].R             /* Memory Region Configuration */
#define XRDC_2_MRCFG2        XRDC_2.MRCFG[2].R             /* Memory Region Configuration */
#define XRDC_2_DERRLOC0      XRDC_2.DERRLOC[0].R           /* Domain Error Location */
#define XRDC_2_DERRLOC1      XRDC_2.DERRLOC[1].R           /* Domain Error Location */
#define XRDC_2_DERRLOC2      XRDC_2.DERRLOC[2].R           /* Domain Error Location */
#define XRDC_2_DERRLOC3      XRDC_2.DERRLOC[3].R           /* Domain Error Location */
#define XRDC_2_DERRLOC4      XRDC_2.DERRLOC[4].R           /* Domain Error Location */
#define XRDC_2_DERRLOC5      XRDC_2.DERRLOC[5].R           /* Domain Error Location */
#define XRDC_2_DERRLOC6      XRDC_2.DERRLOC[6].R           /* Domain Error Location */
#define XRDC_2_DERRLOC7      XRDC_2.DERRLOC[7].R           /* Domain Error Location */
#define XRDC_2_DERRLOC8      XRDC_2.DERRLOC[8].R           /* Domain Error Location */
#define XRDC_2_DERRLOC9      XRDC_2.DERRLOC[9].R           /* Domain Error Location */
#define XRDC_2_DERRLOC10     XRDC_2.DERRLOC[10].R          /* Domain Error Location */
#define XRDC_2_DERRLOC11     XRDC_2.DERRLOC[11].R          /* Domain Error Location */
#define XRDC_2_DERRLOC12     XRDC_2.DERRLOC[12].R          /* Domain Error Location */
#define XRDC_2_DERRLOC13     XRDC_2.DERRLOC[13].R          /* Domain Error Location */
#define XRDC_2_DERRLOC14     XRDC_2.DERRLOC[14].R          /* Domain Error Location */
#define XRDC_2_DERRLOC15     XRDC_2.DERRLOC[15].R          /* Domain Error Location */
#define XRDC_2_DERR_W0_0     XRDC_2.DERRW0[0].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_2_DERR_W1_0     XRDC_2.DERRW0[0].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_2_DERR_W3_0     XRDC_2.DERRW0[0].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_2_DERR_W0_1     XRDC_2.DERRW0[1].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_2_DERR_W1_1     XRDC_2.DERRW0[1].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_2_DERR_W3_1     XRDC_2.DERRW0[1].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_2_DERR_W0_2     XRDC_2.DERRW0[2].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_2_DERR_W1_2     XRDC_2.DERRW0[2].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_2_DERR_W3_2     XRDC_2.DERRW0[2].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_2_DERR_W0_16    XRDC_2.DERRW0[16].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_2_DERR_W1_16    XRDC_2.DERRW0[16].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_2_DERR_W3_16    XRDC_2.DERRW0[16].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_2_DERR_W0_17    XRDC_2.DERRW0[17].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_2_DERR_W1_17    XRDC_2.DERRW0[17].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_2_DERR_W3_17    XRDC_2.DERRW0[17].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_2_MDA_W0_0_DFMT1 XRDC_2.MDA_W0_0_DFMT1.R      /* Master Domain Assignment */
#define XRDC_2_MDA_W0_1_DFMT1 XRDC_2.MDA_W0_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_2_MDA_W0_2_DFMT1 XRDC_2.MDA_W0_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_2_PDAC_W0_0     XRDC_2.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_0     XRDC_2.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_3     XRDC_2.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_3     XRDC_2.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_4     XRDC_2.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_4     XRDC_2.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_6     XRDC_2.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_6     XRDC_2.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_7     XRDC_2.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_7     XRDC_2.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_8     XRDC_2.PDAC_SLOT[0].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_8     XRDC_2.PDAC_SLOT[0].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_10    XRDC_2.PDAC_SLOT[0].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_10    XRDC_2.PDAC_SLOT[0].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_11    XRDC_2.PDAC_SLOT[0].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_11    XRDC_2.PDAC_SLOT[0].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_12    XRDC_2.PDAC_SLOT[0].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_12    XRDC_2.PDAC_SLOT[0].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_13    XRDC_2.PDAC_SLOT[0].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_13    XRDC_2.PDAC_SLOT[0].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_14    XRDC_2.PDAC_SLOT[0].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_14    XRDC_2.PDAC_SLOT[0].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_15    XRDC_2.PDAC_SLOT[0].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_15    XRDC_2.PDAC_SLOT[0].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_PDAC_W0_128   XRDC_2.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_2_PDAC_W1_128   XRDC_2.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_2_MRGD_W0_0     XRDC_2.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_0     XRDC_2.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_0     XRDC_2.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_0     XRDC_2.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_1     XRDC_2.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_1     XRDC_2.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_1     XRDC_2.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_1     XRDC_2.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_2     XRDC_2.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_2     XRDC_2.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_2     XRDC_2.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_2     XRDC_2.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_3     XRDC_2.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_3     XRDC_2.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_3     XRDC_2.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_3     XRDC_2.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_4     XRDC_2.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_4     XRDC_2.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_4     XRDC_2.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_4     XRDC_2.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_5     XRDC_2.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_5     XRDC_2.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_5     XRDC_2.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_5     XRDC_2.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_6     XRDC_2.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_6     XRDC_2.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_6     XRDC_2.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_6     XRDC_2.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_7     XRDC_2.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_7     XRDC_2.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_7     XRDC_2.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_7     XRDC_2.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_8     XRDC_2.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_8     XRDC_2.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_8     XRDC_2.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_8     XRDC_2.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_9     XRDC_2.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_9     XRDC_2.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_9     XRDC_2.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_9     XRDC_2.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_10    XRDC_2.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_10    XRDC_2.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_10    XRDC_2.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_10    XRDC_2.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_11    XRDC_2.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_11    XRDC_2.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_11    XRDC_2.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_11    XRDC_2.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_12    XRDC_2.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_12    XRDC_2.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_12    XRDC_2.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_12    XRDC_2.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_13    XRDC_2.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_13    XRDC_2.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_13    XRDC_2.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_13    XRDC_2.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_14    XRDC_2.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_14    XRDC_2.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_14    XRDC_2.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_14    XRDC_2.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_15    XRDC_2.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_15    XRDC_2.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_15    XRDC_2.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_15    XRDC_2.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_16    XRDC_2.MRGDN[1][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_16    XRDC_2.MRGDN[1][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_16    XRDC_2.MRGDN[1][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_16    XRDC_2.MRGDN[1][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_17    XRDC_2.MRGDN[1][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_17    XRDC_2.MRGDN[1][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_17    XRDC_2.MRGDN[1][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_17    XRDC_2.MRGDN[1][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_18    XRDC_2.MRGDN[1][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_18    XRDC_2.MRGDN[1][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_18    XRDC_2.MRGDN[1][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_18    XRDC_2.MRGDN[1][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_19    XRDC_2.MRGDN[1][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_19    XRDC_2.MRGDN[1][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_19    XRDC_2.MRGDN[1][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_19    XRDC_2.MRGDN[1][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_20    XRDC_2.MRGDN[1][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_20    XRDC_2.MRGDN[1][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_20    XRDC_2.MRGDN[1][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_20    XRDC_2.MRGDN[1][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_21    XRDC_2.MRGDN[1][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_21    XRDC_2.MRGDN[1][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_21    XRDC_2.MRGDN[1][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_21    XRDC_2.MRGDN[1][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_22    XRDC_2.MRGDN[1][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_22    XRDC_2.MRGDN[1][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_22    XRDC_2.MRGDN[1][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_22    XRDC_2.MRGDN[1][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_23    XRDC_2.MRGDN[1][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_23    XRDC_2.MRGDN[1][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_23    XRDC_2.MRGDN[1][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_23    XRDC_2.MRGDN[1][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_24    XRDC_2.MRGDN[1][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_24    XRDC_2.MRGDN[1][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_24    XRDC_2.MRGDN[1][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_24    XRDC_2.MRGDN[1][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_25    XRDC_2.MRGDN[1][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_25    XRDC_2.MRGDN[1][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_25    XRDC_2.MRGDN[1][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_25    XRDC_2.MRGDN[1][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_26    XRDC_2.MRGDN[1][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_26    XRDC_2.MRGDN[1][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_26    XRDC_2.MRGDN[1][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_26    XRDC_2.MRGDN[1][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_27    XRDC_2.MRGDN[1][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_27    XRDC_2.MRGDN[1][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_27    XRDC_2.MRGDN[1][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_27    XRDC_2.MRGDN[1][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_28    XRDC_2.MRGDN[1][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_28    XRDC_2.MRGDN[1][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_28    XRDC_2.MRGDN[1][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_28    XRDC_2.MRGDN[1][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_29    XRDC_2.MRGDN[1][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_29    XRDC_2.MRGDN[1][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_29    XRDC_2.MRGDN[1][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_29    XRDC_2.MRGDN[1][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_30    XRDC_2.MRGDN[1][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_30    XRDC_2.MRGDN[1][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_30    XRDC_2.MRGDN[1][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_30    XRDC_2.MRGDN[1][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_31    XRDC_2.MRGDN[1][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_31    XRDC_2.MRGDN[1][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_31    XRDC_2.MRGDN[1][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_31    XRDC_2.MRGDN[1][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_32    XRDC_2.MRGDN[2][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_32    XRDC_2.MRGDN[2][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_32    XRDC_2.MRGDN[2][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_32    XRDC_2.MRGDN[2][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_33    XRDC_2.MRGDN[2][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_33    XRDC_2.MRGDN[2][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_33    XRDC_2.MRGDN[2][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_33    XRDC_2.MRGDN[2][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_34    XRDC_2.MRGDN[2][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_34    XRDC_2.MRGDN[2][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_34    XRDC_2.MRGDN[2][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_34    XRDC_2.MRGDN[2][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_35    XRDC_2.MRGDN[2][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_35    XRDC_2.MRGDN[2][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_35    XRDC_2.MRGDN[2][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_35    XRDC_2.MRGDN[2][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_36    XRDC_2.MRGDN[2][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_36    XRDC_2.MRGDN[2][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_36    XRDC_2.MRGDN[2][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_36    XRDC_2.MRGDN[2][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_37    XRDC_2.MRGDN[2][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_37    XRDC_2.MRGDN[2][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_37    XRDC_2.MRGDN[2][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_37    XRDC_2.MRGDN[2][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_38    XRDC_2.MRGDN[2][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_38    XRDC_2.MRGDN[2][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_38    XRDC_2.MRGDN[2][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_38    XRDC_2.MRGDN[2][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_39    XRDC_2.MRGDN[2][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_39    XRDC_2.MRGDN[2][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_39    XRDC_2.MRGDN[2][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_39    XRDC_2.MRGDN[2][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_40    XRDC_2.MRGDN[2][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_40    XRDC_2.MRGDN[2][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_40    XRDC_2.MRGDN[2][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_40    XRDC_2.MRGDN[2][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_41    XRDC_2.MRGDN[2][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_41    XRDC_2.MRGDN[2][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_41    XRDC_2.MRGDN[2][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_41    XRDC_2.MRGDN[2][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_42    XRDC_2.MRGDN[2][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_42    XRDC_2.MRGDN[2][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_42    XRDC_2.MRGDN[2][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_42    XRDC_2.MRGDN[2][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_43    XRDC_2.MRGDN[2][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_43    XRDC_2.MRGDN[2][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_43    XRDC_2.MRGDN[2][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_43    XRDC_2.MRGDN[2][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_44    XRDC_2.MRGDN[2][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_44    XRDC_2.MRGDN[2][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_44    XRDC_2.MRGDN[2][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_44    XRDC_2.MRGDN[2][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_45    XRDC_2.MRGDN[2][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_45    XRDC_2.MRGDN[2][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_45    XRDC_2.MRGDN[2][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_45    XRDC_2.MRGDN[2][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_46    XRDC_2.MRGDN[2][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_46    XRDC_2.MRGDN[2][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_46    XRDC_2.MRGDN[2][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_46    XRDC_2.MRGDN[2][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_2_MRGD_W0_47    XRDC_2.MRGDN[2][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_2_MRGD_W1_47    XRDC_2.MRGDN[2][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_2_MRGD_W2_47    XRDC_2.MRGDN[2][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_2_MRGD_W3_47    XRDC_2.MRGDN[2][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* XRDC_3A */
#define XRDC_3A_CR           XRDC_3A.CR.R                  /* Control */
#define XRDC_3A_HWCFG0       XRDC_3A.HWCFG0.R              /* Hardware Configuration 0 */
#define XRDC_3A_HWCFG1       XRDC_3A.HWCFG1.R              /* Hardware Configuration 1 */
#define XRDC_3A_MDACFG0      XRDC_3A.MDACFG[0].R           /* Master Domain Assignment Configuration */
#define XRDC_3A_MDACFG1      XRDC_3A.MDACFG[1].R           /* Master Domain Assignment Configuration */
#define XRDC_3A_MDACFG2      XRDC_3A.MDACFG[2].R           /* Master Domain Assignment Configuration */
#define XRDC_3A_MRCFG0       XRDC_3A.MRCFG[0].R            /* Memory Region Configuration */
#define XRDC_3A_DERRLOC0     XRDC_3A.DERRLOC[0].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC1     XRDC_3A.DERRLOC[1].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC2     XRDC_3A.DERRLOC[2].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC3     XRDC_3A.DERRLOC[3].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC4     XRDC_3A.DERRLOC[4].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC5     XRDC_3A.DERRLOC[5].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC6     XRDC_3A.DERRLOC[6].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC7     XRDC_3A.DERRLOC[7].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC8     XRDC_3A.DERRLOC[8].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC9     XRDC_3A.DERRLOC[9].R          /* Domain Error Location */
#define XRDC_3A_DERRLOC10    XRDC_3A.DERRLOC[10].R         /* Domain Error Location */
#define XRDC_3A_DERRLOC11    XRDC_3A.DERRLOC[11].R         /* Domain Error Location */
#define XRDC_3A_DERRLOC12    XRDC_3A.DERRLOC[12].R         /* Domain Error Location */
#define XRDC_3A_DERRLOC13    XRDC_3A.DERRLOC[13].R         /* Domain Error Location */
#define XRDC_3A_DERRLOC14    XRDC_3A.DERRLOC[14].R         /* Domain Error Location */
#define XRDC_3A_DERRLOC15    XRDC_3A.DERRLOC[15].R         /* Domain Error Location */
#define XRDC_3A_DERR_W0_0    XRDC_3A.DERRW0[0].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_3A_DERR_W1_0    XRDC_3A.DERRW0[0].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_3A_DERR_W3_0    XRDC_3A.DERRW0[0].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_3A_DERR_W0_16   XRDC_3A.DERRW0[16].DERR_W0.R  /* Domain Error Word 0 */
#define XRDC_3A_DERR_W1_16   XRDC_3A.DERRW0[16].DERR_W1.R  /* Domain Error Word 1 */
#define XRDC_3A_DERR_W3_16   XRDC_3A.DERRW0[16].DERR_W3.R  /* Domain Error Word 3 */
#define XRDC_3A_DERR_W0_17   XRDC_3A.DERRW0[17].DERR_W0.R  /* Domain Error Word 0 */
#define XRDC_3A_DERR_W1_17   XRDC_3A.DERRW0[17].DERR_W1.R  /* Domain Error Word 1 */
#define XRDC_3A_DERR_W3_17   XRDC_3A.DERRW0[17].DERR_W3.R  /* Domain Error Word 3 */
#define XRDC_3A_DERR_W0_18   XRDC_3A.DERRW0[18].DERR_W0.R  /* Domain Error Word 0 */
#define XRDC_3A_DERR_W1_18   XRDC_3A.DERRW0[18].DERR_W1.R  /* Domain Error Word 1 */
#define XRDC_3A_DERR_W3_18   XRDC_3A.DERRW0[18].DERR_W3.R  /* Domain Error Word 3 */
#define XRDC_3A_DERR_W0_19   XRDC_3A.DERRW0[19].DERR_W0.R  /* Domain Error Word 0 */
#define XRDC_3A_DERR_W1_19   XRDC_3A.DERRW0[19].DERR_W1.R  /* Domain Error Word 1 */
#define XRDC_3A_DERR_W3_19   XRDC_3A.DERRW0[19].DERR_W3.R  /* Domain Error Word 3 */
#define XRDC_3A_MDA_W0_0_DFMT1 XRDC_3A.MDA_W0_0_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W0_1_DFMT1 XRDC_3A.MDA_W0_1_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W1_1_DFMT1 XRDC_3A.MDA_W1_1_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W2_1_DFMT1 XRDC_3A.MDA_W2_1_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W3_1_DFMT1 XRDC_3A.MDA_W3_1_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W4_1_DFMT1 XRDC_3A.MDA_W4_1_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W5_1_DFMT1 XRDC_3A.MDA_W5_1_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W6_1_DFMT1 XRDC_3A.MDA_W6_1_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W7_1_DFMT1 XRDC_3A.MDA_W7_1_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W0_2_DFMT1 XRDC_3A.MDA_W0_2_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W1_2_DFMT1 XRDC_3A.MDA_W1_2_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W2_2_DFMT1 XRDC_3A.MDA_W2_2_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W3_2_DFMT1 XRDC_3A.MDA_W3_2_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W4_2_DFMT1 XRDC_3A.MDA_W4_2_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W5_2_DFMT1 XRDC_3A.MDA_W5_2_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W6_2_DFMT1 XRDC_3A.MDA_W6_2_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_MDA_W7_2_DFMT1 XRDC_3A.MDA_W7_2_DFMT1.R    /* Master Domain Assignment */
#define XRDC_3A_PDAC_W0_0    XRDC_3A.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_0    XRDC_3A.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_1    XRDC_3A.PDAC_SLOT[0].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_1    XRDC_3A.PDAC_SLOT[0].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_3    XRDC_3A.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_3    XRDC_3A.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_4    XRDC_3A.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_4    XRDC_3A.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_5    XRDC_3A.PDAC_SLOT[0].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_5    XRDC_3A.PDAC_SLOT[0].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_6    XRDC_3A.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_6    XRDC_3A.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_7    XRDC_3A.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_7    XRDC_3A.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_8    XRDC_3A.PDAC_SLOT[0].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_8    XRDC_3A.PDAC_SLOT[0].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_9    XRDC_3A.PDAC_SLOT[0].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_9    XRDC_3A.PDAC_SLOT[0].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_10   XRDC_3A.PDAC_SLOT[0].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_10   XRDC_3A.PDAC_SLOT[0].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_13   XRDC_3A.PDAC_SLOT[0].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_13   XRDC_3A.PDAC_SLOT[0].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_16   XRDC_3A.PDAC_SLOT[0].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_16   XRDC_3A.PDAC_SLOT[0].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_17   XRDC_3A.PDAC_SLOT[0].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_17   XRDC_3A.PDAC_SLOT[0].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_26   XRDC_3A.PDAC_SLOT[0].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_26   XRDC_3A.PDAC_SLOT[0].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_29   XRDC_3A.PDAC_SLOT[0].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_29   XRDC_3A.PDAC_SLOT[0].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_128  XRDC_3A.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_128  XRDC_3A.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_256  XRDC_3A.PDAC_SLOT[2].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_256  XRDC_3A.PDAC_SLOT[2].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_257  XRDC_3A.PDAC_SLOT[2].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_257  XRDC_3A.PDAC_SLOT[2].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_258  XRDC_3A.PDAC_SLOT[2].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_258  XRDC_3A.PDAC_SLOT[2].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_259  XRDC_3A.PDAC_SLOT[2].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_259  XRDC_3A.PDAC_SLOT[2].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_260  XRDC_3A.PDAC_SLOT[2].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_260  XRDC_3A.PDAC_SLOT[2].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_261  XRDC_3A.PDAC_SLOT[2].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_261  XRDC_3A.PDAC_SLOT[2].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_262  XRDC_3A.PDAC_SLOT[2].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_262  XRDC_3A.PDAC_SLOT[2].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_263  XRDC_3A.PDAC_SLOT[2].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_263  XRDC_3A.PDAC_SLOT[2].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_264  XRDC_3A.PDAC_SLOT[2].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_264  XRDC_3A.PDAC_SLOT[2].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_265  XRDC_3A.PDAC_SLOT[2].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_265  XRDC_3A.PDAC_SLOT[2].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_266  XRDC_3A.PDAC_SLOT[2].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_266  XRDC_3A.PDAC_SLOT[2].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_267  XRDC_3A.PDAC_SLOT[2].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_267  XRDC_3A.PDAC_SLOT[2].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_268  XRDC_3A.PDAC_SLOT[2].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_268  XRDC_3A.PDAC_SLOT[2].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_269  XRDC_3A.PDAC_SLOT[2].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_269  XRDC_3A.PDAC_SLOT[2].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_270  XRDC_3A.PDAC_SLOT[2].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_270  XRDC_3A.PDAC_SLOT[2].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_271  XRDC_3A.PDAC_SLOT[2].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_271  XRDC_3A.PDAC_SLOT[2].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_272  XRDC_3A.PDAC_SLOT[2].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_272  XRDC_3A.PDAC_SLOT[2].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_273  XRDC_3A.PDAC_SLOT[2].PDACN[17].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_273  XRDC_3A.PDAC_SLOT[2].PDACN[17].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_274  XRDC_3A.PDAC_SLOT[2].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_274  XRDC_3A.PDAC_SLOT[2].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_275  XRDC_3A.PDAC_SLOT[2].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_275  XRDC_3A.PDAC_SLOT[2].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_276  XRDC_3A.PDAC_SLOT[2].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_276  XRDC_3A.PDAC_SLOT[2].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_277  XRDC_3A.PDAC_SLOT[2].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_277  XRDC_3A.PDAC_SLOT[2].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_278  XRDC_3A.PDAC_SLOT[2].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_278  XRDC_3A.PDAC_SLOT[2].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_279  XRDC_3A.PDAC_SLOT[2].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_279  XRDC_3A.PDAC_SLOT[2].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_280  XRDC_3A.PDAC_SLOT[2].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_280  XRDC_3A.PDAC_SLOT[2].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_289  XRDC_3A.PDAC_SLOT[2].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_289  XRDC_3A.PDAC_SLOT[2].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_290  XRDC_3A.PDAC_SLOT[2].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_290  XRDC_3A.PDAC_SLOT[2].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_291  XRDC_3A.PDAC_SLOT[2].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_291  XRDC_3A.PDAC_SLOT[2].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_292  XRDC_3A.PDAC_SLOT[2].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_292  XRDC_3A.PDAC_SLOT[2].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_293  XRDC_3A.PDAC_SLOT[2].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_293  XRDC_3A.PDAC_SLOT[2].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_294  XRDC_3A.PDAC_SLOT[2].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_294  XRDC_3A.PDAC_SLOT[2].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_295  XRDC_3A.PDAC_SLOT[2].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_295  XRDC_3A.PDAC_SLOT[2].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_296  XRDC_3A.PDAC_SLOT[2].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_296  XRDC_3A.PDAC_SLOT[2].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_297  XRDC_3A.PDAC_SLOT[2].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_297  XRDC_3A.PDAC_SLOT[2].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_298  XRDC_3A.PDAC_SLOT[2].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_298  XRDC_3A.PDAC_SLOT[2].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_299  XRDC_3A.PDAC_SLOT[2].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_299  XRDC_3A.PDAC_SLOT[2].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_300  XRDC_3A.PDAC_SLOT[2].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_300  XRDC_3A.PDAC_SLOT[2].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_301  XRDC_3A.PDAC_SLOT[2].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_301  XRDC_3A.PDAC_SLOT[2].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_302  XRDC_3A.PDAC_SLOT[2].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_302  XRDC_3A.PDAC_SLOT[2].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_303  XRDC_3A.PDAC_SLOT[2].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_303  XRDC_3A.PDAC_SLOT[2].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_304  XRDC_3A.PDAC_SLOT[2].PDACN[48].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_304  XRDC_3A.PDAC_SLOT[2].PDACN[48].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_305  XRDC_3A.PDAC_SLOT[2].PDACN[49].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_305  XRDC_3A.PDAC_SLOT[2].PDACN[49].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_306  XRDC_3A.PDAC_SLOT[2].PDACN[50].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_306  XRDC_3A.PDAC_SLOT[2].PDACN[50].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_307  XRDC_3A.PDAC_SLOT[2].PDACN[51].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_307  XRDC_3A.PDAC_SLOT[2].PDACN[51].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_308  XRDC_3A.PDAC_SLOT[2].PDACN[52].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_308  XRDC_3A.PDAC_SLOT[2].PDACN[52].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_309  XRDC_3A.PDAC_SLOT[2].PDACN[53].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_309  XRDC_3A.PDAC_SLOT[2].PDACN[53].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_310  XRDC_3A.PDAC_SLOT[2].PDACN[54].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_310  XRDC_3A.PDAC_SLOT[2].PDACN[54].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_311  XRDC_3A.PDAC_SLOT[2].PDACN[55].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_311  XRDC_3A.PDAC_SLOT[2].PDACN[55].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_312  XRDC_3A.PDAC_SLOT[2].PDACN[56].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_312  XRDC_3A.PDAC_SLOT[2].PDACN[56].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_313  XRDC_3A.PDAC_SLOT[2].PDACN[57].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_313  XRDC_3A.PDAC_SLOT[2].PDACN[57].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_314  XRDC_3A.PDAC_SLOT[2].PDACN[58].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_314  XRDC_3A.PDAC_SLOT[2].PDACN[58].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_315  XRDC_3A.PDAC_SLOT[2].PDACN[59].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_315  XRDC_3A.PDAC_SLOT[2].PDACN[59].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_324  XRDC_3A.PDAC_SLOT[2].PDACN[68].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_324  XRDC_3A.PDAC_SLOT[2].PDACN[68].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_325  XRDC_3A.PDAC_SLOT[2].PDACN[69].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_325  XRDC_3A.PDAC_SLOT[2].PDACN[69].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_326  XRDC_3A.PDAC_SLOT[2].PDACN[70].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_326  XRDC_3A.PDAC_SLOT[2].PDACN[70].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_327  XRDC_3A.PDAC_SLOT[2].PDACN[71].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_327  XRDC_3A.PDAC_SLOT[2].PDACN[71].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_328  XRDC_3A.PDAC_SLOT[2].PDACN[72].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_328  XRDC_3A.PDAC_SLOT[2].PDACN[72].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_329  XRDC_3A.PDAC_SLOT[2].PDACN[73].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_329  XRDC_3A.PDAC_SLOT[2].PDACN[73].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_330  XRDC_3A.PDAC_SLOT[2].PDACN[74].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_330  XRDC_3A.PDAC_SLOT[2].PDACN[74].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_331  XRDC_3A.PDAC_SLOT[2].PDACN[75].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_331  XRDC_3A.PDAC_SLOT[2].PDACN[75].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_332  XRDC_3A.PDAC_SLOT[2].PDACN[76].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_332  XRDC_3A.PDAC_SLOT[2].PDACN[76].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_333  XRDC_3A.PDAC_SLOT[2].PDACN[77].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_333  XRDC_3A.PDAC_SLOT[2].PDACN[77].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_342  XRDC_3A.PDAC_SLOT[2].PDACN[86].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_342  XRDC_3A.PDAC_SLOT[2].PDACN[86].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_343  XRDC_3A.PDAC_SLOT[2].PDACN[87].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_343  XRDC_3A.PDAC_SLOT[2].PDACN[87].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_344  XRDC_3A.PDAC_SLOT[2].PDACN[88].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_344  XRDC_3A.PDAC_SLOT[2].PDACN[88].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_385  XRDC_3A.PDAC_SLOT[3].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_385  XRDC_3A.PDAC_SLOT[3].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_386  XRDC_3A.PDAC_SLOT[3].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_386  XRDC_3A.PDAC_SLOT[3].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_390  XRDC_3A.PDAC_SLOT[3].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_390  XRDC_3A.PDAC_SLOT[3].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_391  XRDC_3A.PDAC_SLOT[3].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_391  XRDC_3A.PDAC_SLOT[3].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_392  XRDC_3A.PDAC_SLOT[3].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_392  XRDC_3A.PDAC_SLOT[3].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_393  XRDC_3A.PDAC_SLOT[3].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_393  XRDC_3A.PDAC_SLOT[3].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_394  XRDC_3A.PDAC_SLOT[3].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_394  XRDC_3A.PDAC_SLOT[3].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_395  XRDC_3A.PDAC_SLOT[3].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_395  XRDC_3A.PDAC_SLOT[3].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_396  XRDC_3A.PDAC_SLOT[3].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_396  XRDC_3A.PDAC_SLOT[3].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_397  XRDC_3A.PDAC_SLOT[3].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_397  XRDC_3A.PDAC_SLOT[3].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_398  XRDC_3A.PDAC_SLOT[3].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_398  XRDC_3A.PDAC_SLOT[3].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_402  XRDC_3A.PDAC_SLOT[3].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_402  XRDC_3A.PDAC_SLOT[3].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_403  XRDC_3A.PDAC_SLOT[3].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_403  XRDC_3A.PDAC_SLOT[3].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_404  XRDC_3A.PDAC_SLOT[3].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_404  XRDC_3A.PDAC_SLOT[3].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_405  XRDC_3A.PDAC_SLOT[3].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_405  XRDC_3A.PDAC_SLOT[3].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_406  XRDC_3A.PDAC_SLOT[3].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_406  XRDC_3A.PDAC_SLOT[3].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_413  XRDC_3A.PDAC_SLOT[3].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_413  XRDC_3A.PDAC_SLOT[3].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_414  XRDC_3A.PDAC_SLOT[3].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_414  XRDC_3A.PDAC_SLOT[3].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_415  XRDC_3A.PDAC_SLOT[3].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_415  XRDC_3A.PDAC_SLOT[3].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_416  XRDC_3A.PDAC_SLOT[3].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_416  XRDC_3A.PDAC_SLOT[3].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_417  XRDC_3A.PDAC_SLOT[3].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_417  XRDC_3A.PDAC_SLOT[3].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_418  XRDC_3A.PDAC_SLOT[3].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_418  XRDC_3A.PDAC_SLOT[3].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_419  XRDC_3A.PDAC_SLOT[3].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_419  XRDC_3A.PDAC_SLOT[3].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_420  XRDC_3A.PDAC_SLOT[3].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_420  XRDC_3A.PDAC_SLOT[3].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_421  XRDC_3A.PDAC_SLOT[3].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_421  XRDC_3A.PDAC_SLOT[3].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_422  XRDC_3A.PDAC_SLOT[3].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_422  XRDC_3A.PDAC_SLOT[3].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_423  XRDC_3A.PDAC_SLOT[3].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_423  XRDC_3A.PDAC_SLOT[3].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_424  XRDC_3A.PDAC_SLOT[3].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_424  XRDC_3A.PDAC_SLOT[3].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_425  XRDC_3A.PDAC_SLOT[3].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_425  XRDC_3A.PDAC_SLOT[3].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_426  XRDC_3A.PDAC_SLOT[3].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_426  XRDC_3A.PDAC_SLOT[3].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_427  XRDC_3A.PDAC_SLOT[3].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_427  XRDC_3A.PDAC_SLOT[3].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_428  XRDC_3A.PDAC_SLOT[3].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_428  XRDC_3A.PDAC_SLOT[3].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_429  XRDC_3A.PDAC_SLOT[3].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_429  XRDC_3A.PDAC_SLOT[3].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_430  XRDC_3A.PDAC_SLOT[3].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_430  XRDC_3A.PDAC_SLOT[3].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_431  XRDC_3A.PDAC_SLOT[3].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_431  XRDC_3A.PDAC_SLOT[3].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_432  XRDC_3A.PDAC_SLOT[3].PDACN[48].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_432  XRDC_3A.PDAC_SLOT[3].PDACN[48].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_433  XRDC_3A.PDAC_SLOT[3].PDACN[49].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_433  XRDC_3A.PDAC_SLOT[3].PDACN[49].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_434  XRDC_3A.PDAC_SLOT[3].PDACN[50].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_434  XRDC_3A.PDAC_SLOT[3].PDACN[50].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_435  XRDC_3A.PDAC_SLOT[3].PDACN[51].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_435  XRDC_3A.PDAC_SLOT[3].PDACN[51].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_436  XRDC_3A.PDAC_SLOT[3].PDACN[52].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_436  XRDC_3A.PDAC_SLOT[3].PDACN[52].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_437  XRDC_3A.PDAC_SLOT[3].PDACN[53].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_437  XRDC_3A.PDAC_SLOT[3].PDACN[53].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_438  XRDC_3A.PDAC_SLOT[3].PDACN[54].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_438  XRDC_3A.PDAC_SLOT[3].PDACN[54].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_439  XRDC_3A.PDAC_SLOT[3].PDACN[55].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_439  XRDC_3A.PDAC_SLOT[3].PDACN[55].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_440  XRDC_3A.PDAC_SLOT[3].PDACN[56].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_440  XRDC_3A.PDAC_SLOT[3].PDACN[56].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_441  XRDC_3A.PDAC_SLOT[3].PDACN[57].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_441  XRDC_3A.PDAC_SLOT[3].PDACN[57].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_442  XRDC_3A.PDAC_SLOT[3].PDACN[58].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_442  XRDC_3A.PDAC_SLOT[3].PDACN[58].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_443  XRDC_3A.PDAC_SLOT[3].PDACN[59].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_443  XRDC_3A.PDAC_SLOT[3].PDACN[59].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_444  XRDC_3A.PDAC_SLOT[3].PDACN[60].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_444  XRDC_3A.PDAC_SLOT[3].PDACN[60].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_445  XRDC_3A.PDAC_SLOT[3].PDACN[61].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_445  XRDC_3A.PDAC_SLOT[3].PDACN[61].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_446  XRDC_3A.PDAC_SLOT[3].PDACN[62].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_446  XRDC_3A.PDAC_SLOT[3].PDACN[62].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_PDAC_W0_447  XRDC_3A.PDAC_SLOT[3].PDACN[63].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3A_PDAC_W1_447  XRDC_3A.PDAC_SLOT[3].PDACN[63].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3A_MRGD_W0_0    XRDC_3A.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_0    XRDC_3A.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_0    XRDC_3A.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_0    XRDC_3A.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_1    XRDC_3A.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_1    XRDC_3A.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_1    XRDC_3A.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_1    XRDC_3A.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_2    XRDC_3A.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_2    XRDC_3A.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_2    XRDC_3A.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_2    XRDC_3A.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_3    XRDC_3A.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_3    XRDC_3A.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_3    XRDC_3A.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_3    XRDC_3A.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_4    XRDC_3A.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_4    XRDC_3A.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_4    XRDC_3A.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_4    XRDC_3A.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_5    XRDC_3A.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_5    XRDC_3A.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_5    XRDC_3A.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_5    XRDC_3A.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_6    XRDC_3A.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_6    XRDC_3A.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_6    XRDC_3A.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_6    XRDC_3A.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_7    XRDC_3A.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_7    XRDC_3A.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_7    XRDC_3A.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_7    XRDC_3A.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_8    XRDC_3A.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_8    XRDC_3A.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_8    XRDC_3A.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_8    XRDC_3A.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_9    XRDC_3A.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_9    XRDC_3A.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_9    XRDC_3A.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_9    XRDC_3A.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_10   XRDC_3A.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_10   XRDC_3A.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_10   XRDC_3A.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_10   XRDC_3A.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_11   XRDC_3A.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_11   XRDC_3A.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_11   XRDC_3A.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_11   XRDC_3A.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_12   XRDC_3A.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_12   XRDC_3A.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_12   XRDC_3A.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_12   XRDC_3A.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_13   XRDC_3A.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_13   XRDC_3A.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_13   XRDC_3A.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_13   XRDC_3A.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_14   XRDC_3A.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_14   XRDC_3A.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_14   XRDC_3A.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_14   XRDC_3A.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_3A_MRGD_W0_15   XRDC_3A.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_3A_MRGD_W1_15   XRDC_3A.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_3A_MRGD_W2_15   XRDC_3A.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_3A_MRGD_W3_15   XRDC_3A.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* XRDC_3B */
#define XRDC_3B_CR           XRDC_3B.CR.R                  /* Control */
#define XRDC_3B_HWCFG0       XRDC_3B.HWCFG0.R              /* Hardware Configuration 0 */
#define XRDC_3B_HWCFG1       XRDC_3B.HWCFG1.R              /* Hardware Configuration 1 */
#define XRDC_3B_DERRLOC0     XRDC_3B.DERRLOC[0].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC1     XRDC_3B.DERRLOC[1].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC2     XRDC_3B.DERRLOC[2].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC3     XRDC_3B.DERRLOC[3].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC4     XRDC_3B.DERRLOC[4].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC5     XRDC_3B.DERRLOC[5].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC6     XRDC_3B.DERRLOC[6].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC7     XRDC_3B.DERRLOC[7].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC8     XRDC_3B.DERRLOC[8].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC9     XRDC_3B.DERRLOC[9].R          /* Domain Error Location */
#define XRDC_3B_DERRLOC10    XRDC_3B.DERRLOC[10].R         /* Domain Error Location */
#define XRDC_3B_DERRLOC11    XRDC_3B.DERRLOC[11].R         /* Domain Error Location */
#define XRDC_3B_DERRLOC12    XRDC_3B.DERRLOC[12].R         /* Domain Error Location */
#define XRDC_3B_DERRLOC13    XRDC_3B.DERRLOC[13].R         /* Domain Error Location */
#define XRDC_3B_DERRLOC14    XRDC_3B.DERRLOC[14].R         /* Domain Error Location */
#define XRDC_3B_DERRLOC15    XRDC_3B.DERRLOC[15].R         /* Domain Error Location */
#define XRDC_3B_DERR_W0_16   XRDC_3B.DERRW0[16].DERR_W0.R  /* Domain Error Word 0 */
#define XRDC_3B_DERR_W1_16   XRDC_3B.DERRW0[16].DERR_W1.R  /* Domain Error Word 1 */
#define XRDC_3B_DERR_W3_16   XRDC_3B.DERRW0[16].DERR_W3.R  /* Domain Error Word 3 */
#define XRDC_3B_DERR_W0_17   XRDC_3B.DERRW0[17].DERR_W0.R  /* Domain Error Word 0 */
#define XRDC_3B_DERR_W1_17   XRDC_3B.DERRW0[17].DERR_W1.R  /* Domain Error Word 1 */
#define XRDC_3B_DERR_W3_17   XRDC_3B.DERRW0[17].DERR_W3.R  /* Domain Error Word 3 */
#define XRDC_3B_DERR_W0_18   XRDC_3B.DERRW0[18].DERR_W0.R  /* Domain Error Word 0 */
#define XRDC_3B_DERR_W1_18   XRDC_3B.DERRW0[18].DERR_W1.R  /* Domain Error Word 1 */
#define XRDC_3B_DERR_W3_18   XRDC_3B.DERRW0[18].DERR_W3.R  /* Domain Error Word 3 */
#define XRDC_3B_PDAC_W0_1    XRDC_3B.PDAC_SLOT[0].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_1    XRDC_3B.PDAC_SLOT[0].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_2    XRDC_3B.PDAC_SLOT[0].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_2    XRDC_3B.PDAC_SLOT[0].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_3    XRDC_3B.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_3    XRDC_3B.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_4    XRDC_3B.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_4    XRDC_3B.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_6    XRDC_3B.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_6    XRDC_3B.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_10   XRDC_3B.PDAC_SLOT[0].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_10   XRDC_3B.PDAC_SLOT[0].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_11   XRDC_3B.PDAC_SLOT[0].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_11   XRDC_3B.PDAC_SLOT[0].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_12   XRDC_3B.PDAC_SLOT[0].PDACN[12].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_12   XRDC_3B.PDAC_SLOT[0].PDACN[12].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_14   XRDC_3B.PDAC_SLOT[0].PDACN[14].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_14   XRDC_3B.PDAC_SLOT[0].PDACN[14].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_15   XRDC_3B.PDAC_SLOT[0].PDACN[15].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_15   XRDC_3B.PDAC_SLOT[0].PDACN[15].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_128  XRDC_3B.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_128  XRDC_3B.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_256  XRDC_3B.PDAC_SLOT[2].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_256  XRDC_3B.PDAC_SLOT[2].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_3B_PDAC_W0_257  XRDC_3B.PDAC_SLOT[2].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_3B_PDAC_W1_257  XRDC_3B.PDAC_SLOT[2].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */

/* XRDC_4 */
#define XRDC_4_CR            XRDC_4.CR.R                   /* Control */
#define XRDC_4_HWCFG0        XRDC_4.HWCFG0.R               /* Hardware Configuration 0 */
#define XRDC_4_HWCFG1        XRDC_4.HWCFG1.R               /* Hardware Configuration 1 */
#define XRDC_4_MDACFG0       XRDC_4.MDACFG[0].R            /* Master Domain Assignment Configuration */
#define XRDC_4_MDACFG1       XRDC_4.MDACFG[1].R            /* Master Domain Assignment Configuration */
#define XRDC_4_MDACFG2       XRDC_4.MDACFG[2].R            /* Master Domain Assignment Configuration */
#define XRDC_4_MRCFG0        XRDC_4.MRCFG[0].R             /* Memory Region Configuration */
#define XRDC_4_MRCFG1        XRDC_4.MRCFG[1].R             /* Memory Region Configuration */
#define XRDC_4_DERRLOC0      XRDC_4.DERRLOC[0].R           /* Domain Error Location */
#define XRDC_4_DERRLOC1      XRDC_4.DERRLOC[1].R           /* Domain Error Location */
#define XRDC_4_DERRLOC2      XRDC_4.DERRLOC[2].R           /* Domain Error Location */
#define XRDC_4_DERRLOC3      XRDC_4.DERRLOC[3].R           /* Domain Error Location */
#define XRDC_4_DERRLOC4      XRDC_4.DERRLOC[4].R           /* Domain Error Location */
#define XRDC_4_DERRLOC5      XRDC_4.DERRLOC[5].R           /* Domain Error Location */
#define XRDC_4_DERRLOC6      XRDC_4.DERRLOC[6].R           /* Domain Error Location */
#define XRDC_4_DERRLOC7      XRDC_4.DERRLOC[7].R           /* Domain Error Location */
#define XRDC_4_DERRLOC8      XRDC_4.DERRLOC[8].R           /* Domain Error Location */
#define XRDC_4_DERRLOC9      XRDC_4.DERRLOC[9].R           /* Domain Error Location */
#define XRDC_4_DERRLOC10     XRDC_4.DERRLOC[10].R          /* Domain Error Location */
#define XRDC_4_DERRLOC11     XRDC_4.DERRLOC[11].R          /* Domain Error Location */
#define XRDC_4_DERRLOC12     XRDC_4.DERRLOC[12].R          /* Domain Error Location */
#define XRDC_4_DERRLOC13     XRDC_4.DERRLOC[13].R          /* Domain Error Location */
#define XRDC_4_DERRLOC14     XRDC_4.DERRLOC[14].R          /* Domain Error Location */
#define XRDC_4_DERRLOC15     XRDC_4.DERRLOC[15].R          /* Domain Error Location */
#define XRDC_4_DERR_W0_0     XRDC_4.DERRW0[0].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_4_DERR_W1_0     XRDC_4.DERRW0[0].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_4_DERR_W3_0     XRDC_4.DERRW0[0].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_4_DERR_W0_1     XRDC_4.DERRW0[1].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_4_DERR_W1_1     XRDC_4.DERRW0[1].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_4_DERR_W3_1     XRDC_4.DERRW0[1].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_4_DERR_W0_16    XRDC_4.DERRW0[16].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_4_DERR_W1_16    XRDC_4.DERRW0[16].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_4_DERR_W3_16    XRDC_4.DERRW0[16].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_4_DERR_W0_17    XRDC_4.DERRW0[17].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_4_DERR_W1_17    XRDC_4.DERRW0[17].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_4_DERR_W3_17    XRDC_4.DERRW0[17].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_4_DERR_W0_18    XRDC_4.DERRW0[18].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_4_DERR_W1_18    XRDC_4.DERRW0[18].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_4_DERR_W3_18    XRDC_4.DERRW0[18].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_4_DERR_W0_19    XRDC_4.DERRW0[19].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_4_DERR_W1_19    XRDC_4.DERRW0[19].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_4_DERR_W3_19    XRDC_4.DERRW0[19].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_4_MDA_W0_0_DFMT1 XRDC_4.MDA_W0_0_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W0_1_DFMT1 XRDC_4.MDA_W0_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W0_2_DFMT1 XRDC_4.MDA_W0_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W1_2_DFMT1 XRDC_4.MDA_W1_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W2_2_DFMT1 XRDC_4.MDA_W2_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W3_2_DFMT1 XRDC_4.MDA_W3_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W4_2_DFMT1 XRDC_4.MDA_W4_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W5_2_DFMT1 XRDC_4.MDA_W5_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W6_2_DFMT1 XRDC_4.MDA_W6_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_MDA_W7_2_DFMT1 XRDC_4.MDA_W7_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_4_PDAC_W0_0     XRDC_4.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_0     XRDC_4.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_1     XRDC_4.PDAC_SLOT[0].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_1     XRDC_4.PDAC_SLOT[0].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_3     XRDC_4.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_3     XRDC_4.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_4     XRDC_4.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_4     XRDC_4.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_5     XRDC_4.PDAC_SLOT[0].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_5     XRDC_4.PDAC_SLOT[0].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_6     XRDC_4.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_6     XRDC_4.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_7     XRDC_4.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_7     XRDC_4.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_9     XRDC_4.PDAC_SLOT[0].PDACN[9].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_9     XRDC_4.PDAC_SLOT[0].PDACN[9].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_10    XRDC_4.PDAC_SLOT[0].PDACN[10].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_10    XRDC_4.PDAC_SLOT[0].PDACN[10].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_11    XRDC_4.PDAC_SLOT[0].PDACN[11].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_11    XRDC_4.PDAC_SLOT[0].PDACN[11].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_13    XRDC_4.PDAC_SLOT[0].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_13    XRDC_4.PDAC_SLOT[0].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_18    XRDC_4.PDAC_SLOT[0].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_18    XRDC_4.PDAC_SLOT[0].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_19    XRDC_4.PDAC_SLOT[0].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_19    XRDC_4.PDAC_SLOT[0].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_20    XRDC_4.PDAC_SLOT[0].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_20    XRDC_4.PDAC_SLOT[0].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_21    XRDC_4.PDAC_SLOT[0].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_21    XRDC_4.PDAC_SLOT[0].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_23    XRDC_4.PDAC_SLOT[0].PDACN[23].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_23    XRDC_4.PDAC_SLOT[0].PDACN[23].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_24    XRDC_4.PDAC_SLOT[0].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_24    XRDC_4.PDAC_SLOT[0].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_25    XRDC_4.PDAC_SLOT[0].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_25    XRDC_4.PDAC_SLOT[0].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_29    XRDC_4.PDAC_SLOT[0].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_29    XRDC_4.PDAC_SLOT[0].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_30    XRDC_4.PDAC_SLOT[0].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_30    XRDC_4.PDAC_SLOT[0].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_31    XRDC_4.PDAC_SLOT[0].PDACN[31].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_31    XRDC_4.PDAC_SLOT[0].PDACN[31].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_128   XRDC_4.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_128   XRDC_4.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_256   XRDC_4.PDAC_SLOT[2].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_256   XRDC_4.PDAC_SLOT[2].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_257   XRDC_4.PDAC_SLOT[2].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_257   XRDC_4.PDAC_SLOT[2].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_258   XRDC_4.PDAC_SLOT[2].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_258   XRDC_4.PDAC_SLOT[2].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_259   XRDC_4.PDAC_SLOT[2].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_259   XRDC_4.PDAC_SLOT[2].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_260   XRDC_4.PDAC_SLOT[2].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_260   XRDC_4.PDAC_SLOT[2].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_261   XRDC_4.PDAC_SLOT[2].PDACN[5].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_261   XRDC_4.PDAC_SLOT[2].PDACN[5].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_262   XRDC_4.PDAC_SLOT[2].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_262   XRDC_4.PDAC_SLOT[2].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_263   XRDC_4.PDAC_SLOT[2].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_263   XRDC_4.PDAC_SLOT[2].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_264   XRDC_4.PDAC_SLOT[2].PDACN[8].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_264   XRDC_4.PDAC_SLOT[2].PDACN[8].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_272   XRDC_4.PDAC_SLOT[2].PDACN[16].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_272   XRDC_4.PDAC_SLOT[2].PDACN[16].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_274   XRDC_4.PDAC_SLOT[2].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_274   XRDC_4.PDAC_SLOT[2].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_276   XRDC_4.PDAC_SLOT[2].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_276   XRDC_4.PDAC_SLOT[2].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_280   XRDC_4.PDAC_SLOT[2].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_280   XRDC_4.PDAC_SLOT[2].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_281   XRDC_4.PDAC_SLOT[2].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_281   XRDC_4.PDAC_SLOT[2].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_283   XRDC_4.PDAC_SLOT[2].PDACN[27].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_283   XRDC_4.PDAC_SLOT[2].PDACN[27].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_385   XRDC_4.PDAC_SLOT[3].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_385   XRDC_4.PDAC_SLOT[3].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_386   XRDC_4.PDAC_SLOT[3].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_386   XRDC_4.PDAC_SLOT[3].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_402   XRDC_4.PDAC_SLOT[3].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_402   XRDC_4.PDAC_SLOT[3].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_403   XRDC_4.PDAC_SLOT[3].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_403   XRDC_4.PDAC_SLOT[3].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_404   XRDC_4.PDAC_SLOT[3].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_404   XRDC_4.PDAC_SLOT[3].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_405   XRDC_4.PDAC_SLOT[3].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_405   XRDC_4.PDAC_SLOT[3].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_406   XRDC_4.PDAC_SLOT[3].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_406   XRDC_4.PDAC_SLOT[3].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_410   XRDC_4.PDAC_SLOT[3].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_410   XRDC_4.PDAC_SLOT[3].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_413   XRDC_4.PDAC_SLOT[3].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_413   XRDC_4.PDAC_SLOT[3].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_414   XRDC_4.PDAC_SLOT[3].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_414   XRDC_4.PDAC_SLOT[3].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_416   XRDC_4.PDAC_SLOT[3].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_416   XRDC_4.PDAC_SLOT[3].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_417   XRDC_4.PDAC_SLOT[3].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_417   XRDC_4.PDAC_SLOT[3].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_418   XRDC_4.PDAC_SLOT[3].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_418   XRDC_4.PDAC_SLOT[3].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_419   XRDC_4.PDAC_SLOT[3].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_419   XRDC_4.PDAC_SLOT[3].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_420   XRDC_4.PDAC_SLOT[3].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_420   XRDC_4.PDAC_SLOT[3].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_421   XRDC_4.PDAC_SLOT[3].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_421   XRDC_4.PDAC_SLOT[3].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_422   XRDC_4.PDAC_SLOT[3].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_422   XRDC_4.PDAC_SLOT[3].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_423   XRDC_4.PDAC_SLOT[3].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_423   XRDC_4.PDAC_SLOT[3].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_424   XRDC_4.PDAC_SLOT[3].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_424   XRDC_4.PDAC_SLOT[3].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_425   XRDC_4.PDAC_SLOT[3].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_425   XRDC_4.PDAC_SLOT[3].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_426   XRDC_4.PDAC_SLOT[3].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_426   XRDC_4.PDAC_SLOT[3].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_427   XRDC_4.PDAC_SLOT[3].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_427   XRDC_4.PDAC_SLOT[3].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_428   XRDC_4.PDAC_SLOT[3].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_428   XRDC_4.PDAC_SLOT[3].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_429   XRDC_4.PDAC_SLOT[3].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_429   XRDC_4.PDAC_SLOT[3].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_430   XRDC_4.PDAC_SLOT[3].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_430   XRDC_4.PDAC_SLOT[3].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_PDAC_W0_431   XRDC_4.PDAC_SLOT[3].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_4_PDAC_W1_431   XRDC_4.PDAC_SLOT[3].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_4_MRGD_W0_0     XRDC_4.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_0     XRDC_4.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_0     XRDC_4.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_0     XRDC_4.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_1     XRDC_4.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_1     XRDC_4.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_1     XRDC_4.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_1     XRDC_4.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_2     XRDC_4.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_2     XRDC_4.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_2     XRDC_4.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_2     XRDC_4.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_3     XRDC_4.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_3     XRDC_4.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_3     XRDC_4.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_3     XRDC_4.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_4     XRDC_4.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_4     XRDC_4.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_4     XRDC_4.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_4     XRDC_4.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_5     XRDC_4.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_5     XRDC_4.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_5     XRDC_4.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_5     XRDC_4.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_6     XRDC_4.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_6     XRDC_4.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_6     XRDC_4.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_6     XRDC_4.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_7     XRDC_4.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_7     XRDC_4.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_7     XRDC_4.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_7     XRDC_4.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_8     XRDC_4.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_8     XRDC_4.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_8     XRDC_4.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_8     XRDC_4.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_9     XRDC_4.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_9     XRDC_4.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_9     XRDC_4.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_9     XRDC_4.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_10    XRDC_4.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_10    XRDC_4.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_10    XRDC_4.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_10    XRDC_4.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_11    XRDC_4.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_11    XRDC_4.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_11    XRDC_4.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_11    XRDC_4.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_12    XRDC_4.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_12    XRDC_4.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_12    XRDC_4.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_12    XRDC_4.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_13    XRDC_4.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_13    XRDC_4.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_13    XRDC_4.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_13    XRDC_4.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_14    XRDC_4.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_14    XRDC_4.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_14    XRDC_4.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_14    XRDC_4.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_15    XRDC_4.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_15    XRDC_4.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_15    XRDC_4.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_15    XRDC_4.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_16    XRDC_4.MRGDN[1][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_16    XRDC_4.MRGDN[1][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_16    XRDC_4.MRGDN[1][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_16    XRDC_4.MRGDN[1][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_17    XRDC_4.MRGDN[1][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_17    XRDC_4.MRGDN[1][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_17    XRDC_4.MRGDN[1][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_17    XRDC_4.MRGDN[1][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_18    XRDC_4.MRGDN[1][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_18    XRDC_4.MRGDN[1][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_18    XRDC_4.MRGDN[1][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_18    XRDC_4.MRGDN[1][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_4_MRGD_W0_19    XRDC_4.MRGDN[1][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_4_MRGD_W1_19    XRDC_4.MRGDN[1][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_4_MRGD_W2_19    XRDC_4.MRGDN[1][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_4_MRGD_W3_19    XRDC_4.MRGDN[1][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* XRDC_5 */
#define XRDC_5_CR            XRDC_5.CR.R                   /* Control */
#define XRDC_5_HWCFG0        XRDC_5.HWCFG0.R               /* Hardware Configuration 0 */
#define XRDC_5_HWCFG1        XRDC_5.HWCFG1.R               /* Hardware Configuration 1 */
#define XRDC_5_MDACFG0       XRDC_5.MDACFG[0].R            /* Master Domain Assignment Configuration */
#define XRDC_5_MDACFG1       XRDC_5.MDACFG[1].R            /* Master Domain Assignment Configuration */
#define XRDC_5_MDACFG2       XRDC_5.MDACFG[2].R            /* Master Domain Assignment Configuration */
#define XRDC_5_MRCFG0        XRDC_5.MRCFG[0].R             /* Memory Region Configuration */
#define XRDC_5_MRCFG1        XRDC_5.MRCFG[1].R             /* Memory Region Configuration */
#define XRDC_5_MRCFG2        XRDC_5.MRCFG[2].R             /* Memory Region Configuration */
#define XRDC_5_DERRLOC0      XRDC_5.DERRLOC[0].R           /* Domain Error Location */
#define XRDC_5_DERRLOC1      XRDC_5.DERRLOC[1].R           /* Domain Error Location */
#define XRDC_5_DERRLOC2      XRDC_5.DERRLOC[2].R           /* Domain Error Location */
#define XRDC_5_DERRLOC3      XRDC_5.DERRLOC[3].R           /* Domain Error Location */
#define XRDC_5_DERRLOC4      XRDC_5.DERRLOC[4].R           /* Domain Error Location */
#define XRDC_5_DERRLOC5      XRDC_5.DERRLOC[5].R           /* Domain Error Location */
#define XRDC_5_DERRLOC6      XRDC_5.DERRLOC[6].R           /* Domain Error Location */
#define XRDC_5_DERRLOC7      XRDC_5.DERRLOC[7].R           /* Domain Error Location */
#define XRDC_5_DERRLOC8      XRDC_5.DERRLOC[8].R           /* Domain Error Location */
#define XRDC_5_DERRLOC9      XRDC_5.DERRLOC[9].R           /* Domain Error Location */
#define XRDC_5_DERRLOC10     XRDC_5.DERRLOC[10].R          /* Domain Error Location */
#define XRDC_5_DERRLOC11     XRDC_5.DERRLOC[11].R          /* Domain Error Location */
#define XRDC_5_DERRLOC12     XRDC_5.DERRLOC[12].R          /* Domain Error Location */
#define XRDC_5_DERRLOC13     XRDC_5.DERRLOC[13].R          /* Domain Error Location */
#define XRDC_5_DERRLOC14     XRDC_5.DERRLOC[14].R          /* Domain Error Location */
#define XRDC_5_DERRLOC15     XRDC_5.DERRLOC[15].R          /* Domain Error Location */
#define XRDC_5_DERR_W0_0     XRDC_5.DERRW0[0].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_5_DERR_W1_0     XRDC_5.DERRW0[0].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_5_DERR_W3_0     XRDC_5.DERRW0[0].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_5_DERR_W0_1     XRDC_5.DERRW0[1].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_5_DERR_W1_1     XRDC_5.DERRW0[1].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_5_DERR_W3_1     XRDC_5.DERRW0[1].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_5_DERR_W0_2     XRDC_5.DERRW0[2].DERR_W0.R    /* Domain Error Word 0 */
#define XRDC_5_DERR_W1_2     XRDC_5.DERRW0[2].DERR_W1.R    /* Domain Error Word 1 */
#define XRDC_5_DERR_W3_2     XRDC_5.DERRW0[2].DERR_W3.R    /* Domain Error Word 3 */
#define XRDC_5_DERR_W0_16    XRDC_5.DERRW0[16].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_5_DERR_W1_16    XRDC_5.DERRW0[16].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_5_DERR_W3_16    XRDC_5.DERRW0[16].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_5_DERR_W0_17    XRDC_5.DERRW0[17].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_5_DERR_W1_17    XRDC_5.DERRW0[17].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_5_DERR_W3_17    XRDC_5.DERRW0[17].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_5_DERR_W0_18    XRDC_5.DERRW0[18].DERR_W0.R   /* Domain Error Word 0 */
#define XRDC_5_DERR_W1_18    XRDC_5.DERRW0[18].DERR_W1.R   /* Domain Error Word 1 */
#define XRDC_5_DERR_W3_18    XRDC_5.DERRW0[18].DERR_W3.R   /* Domain Error Word 3 */
#define XRDC_5_MDA_W0_0_DFMT1 XRDC_5.MDA_W0_0_DFMT1.R      /* Master Domain Assignment */
#define XRDC_5_MDA_W0_1_DFMT1 XRDC_5.MDA_W0_1_DFMT1.R      /* Master Domain Assignment */
#define XRDC_5_MDA_W0_2_DFMT1 XRDC_5.MDA_W0_2_DFMT1.R      /* Master Domain Assignment */
#define XRDC_5_PDAC_W0_0     XRDC_5.PDAC_SLOT[0].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_0     XRDC_5.PDAC_SLOT[0].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_1     XRDC_5.PDAC_SLOT[0].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_1     XRDC_5.PDAC_SLOT[0].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_3     XRDC_5.PDAC_SLOT[0].PDACN[3].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_3     XRDC_5.PDAC_SLOT[0].PDACN[3].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_4     XRDC_5.PDAC_SLOT[0].PDACN[4].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_4     XRDC_5.PDAC_SLOT[0].PDACN[4].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_6     XRDC_5.PDAC_SLOT[0].PDACN[6].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_6     XRDC_5.PDAC_SLOT[0].PDACN[6].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_7     XRDC_5.PDAC_SLOT[0].PDACN[7].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_7     XRDC_5.PDAC_SLOT[0].PDACN[7].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_13    XRDC_5.PDAC_SLOT[0].PDACN[13].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_13    XRDC_5.PDAC_SLOT[0].PDACN[13].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_18    XRDC_5.PDAC_SLOT[0].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_18    XRDC_5.PDAC_SLOT[0].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_19    XRDC_5.PDAC_SLOT[0].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_19    XRDC_5.PDAC_SLOT[0].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_20    XRDC_5.PDAC_SLOT[0].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_20    XRDC_5.PDAC_SLOT[0].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_22    XRDC_5.PDAC_SLOT[0].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_22    XRDC_5.PDAC_SLOT[0].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_24    XRDC_5.PDAC_SLOT[0].PDACN[24].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_24    XRDC_5.PDAC_SLOT[0].PDACN[24].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_25    XRDC_5.PDAC_SLOT[0].PDACN[25].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_25    XRDC_5.PDAC_SLOT[0].PDACN[25].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_26    XRDC_5.PDAC_SLOT[0].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_26    XRDC_5.PDAC_SLOT[0].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_128   XRDC_5.PDAC_SLOT[1].PDACN[0].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_128   XRDC_5.PDAC_SLOT[1].PDACN[0].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_257   XRDC_5.PDAC_SLOT[2].PDACN[1].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_257   XRDC_5.PDAC_SLOT[2].PDACN[1].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_258   XRDC_5.PDAC_SLOT[2].PDACN[2].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_258   XRDC_5.PDAC_SLOT[2].PDACN[2].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_274   XRDC_5.PDAC_SLOT[2].PDACN[18].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_274   XRDC_5.PDAC_SLOT[2].PDACN[18].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_275   XRDC_5.PDAC_SLOT[2].PDACN[19].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_275   XRDC_5.PDAC_SLOT[2].PDACN[19].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_276   XRDC_5.PDAC_SLOT[2].PDACN[20].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_276   XRDC_5.PDAC_SLOT[2].PDACN[20].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_277   XRDC_5.PDAC_SLOT[2].PDACN[21].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_277   XRDC_5.PDAC_SLOT[2].PDACN[21].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_278   XRDC_5.PDAC_SLOT[2].PDACN[22].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_278   XRDC_5.PDAC_SLOT[2].PDACN[22].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_282   XRDC_5.PDAC_SLOT[2].PDACN[26].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_282   XRDC_5.PDAC_SLOT[2].PDACN[26].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_285   XRDC_5.PDAC_SLOT[2].PDACN[29].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_285   XRDC_5.PDAC_SLOT[2].PDACN[29].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_286   XRDC_5.PDAC_SLOT[2].PDACN[30].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_286   XRDC_5.PDAC_SLOT[2].PDACN[30].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_288   XRDC_5.PDAC_SLOT[2].PDACN[32].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_288   XRDC_5.PDAC_SLOT[2].PDACN[32].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_289   XRDC_5.PDAC_SLOT[2].PDACN[33].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_289   XRDC_5.PDAC_SLOT[2].PDACN[33].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_290   XRDC_5.PDAC_SLOT[2].PDACN[34].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_290   XRDC_5.PDAC_SLOT[2].PDACN[34].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_291   XRDC_5.PDAC_SLOT[2].PDACN[35].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_291   XRDC_5.PDAC_SLOT[2].PDACN[35].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_292   XRDC_5.PDAC_SLOT[2].PDACN[36].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_292   XRDC_5.PDAC_SLOT[2].PDACN[36].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_293   XRDC_5.PDAC_SLOT[2].PDACN[37].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_293   XRDC_5.PDAC_SLOT[2].PDACN[37].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_294   XRDC_5.PDAC_SLOT[2].PDACN[38].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_294   XRDC_5.PDAC_SLOT[2].PDACN[38].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_295   XRDC_5.PDAC_SLOT[2].PDACN[39].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_295   XRDC_5.PDAC_SLOT[2].PDACN[39].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_296   XRDC_5.PDAC_SLOT[2].PDACN[40].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_296   XRDC_5.PDAC_SLOT[2].PDACN[40].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_297   XRDC_5.PDAC_SLOT[2].PDACN[41].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_297   XRDC_5.PDAC_SLOT[2].PDACN[41].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_298   XRDC_5.PDAC_SLOT[2].PDACN[42].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_298   XRDC_5.PDAC_SLOT[2].PDACN[42].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_299   XRDC_5.PDAC_SLOT[2].PDACN[43].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_299   XRDC_5.PDAC_SLOT[2].PDACN[43].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_300   XRDC_5.PDAC_SLOT[2].PDACN[44].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_300   XRDC_5.PDAC_SLOT[2].PDACN[44].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_301   XRDC_5.PDAC_SLOT[2].PDACN[45].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_301   XRDC_5.PDAC_SLOT[2].PDACN[45].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_302   XRDC_5.PDAC_SLOT[2].PDACN[46].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_302   XRDC_5.PDAC_SLOT[2].PDACN[46].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_PDAC_W0_303   XRDC_5.PDAC_SLOT[2].PDACN[47].PDAC_W0.R /* Peripheral Domain Access Control Word 0 */
#define XRDC_5_PDAC_W1_303   XRDC_5.PDAC_SLOT[2].PDACN[47].PDAC_W1.R /* Peripheral Domain Access Control Word 1 */
#define XRDC_5_MRGD_W0_0     XRDC_5.MRGDN[0][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_0     XRDC_5.MRGDN[0][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_0     XRDC_5.MRGDN[0][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_0     XRDC_5.MRGDN[0][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_1     XRDC_5.MRGDN[0][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_1     XRDC_5.MRGDN[0][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_1     XRDC_5.MRGDN[0][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_1     XRDC_5.MRGDN[0][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_2     XRDC_5.MRGDN[0][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_2     XRDC_5.MRGDN[0][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_2     XRDC_5.MRGDN[0][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_2     XRDC_5.MRGDN[0][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_3     XRDC_5.MRGDN[0][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_3     XRDC_5.MRGDN[0][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_3     XRDC_5.MRGDN[0][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_3     XRDC_5.MRGDN[0][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_4     XRDC_5.MRGDN[0][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_4     XRDC_5.MRGDN[0][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_4     XRDC_5.MRGDN[0][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_4     XRDC_5.MRGDN[0][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_5     XRDC_5.MRGDN[0][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_5     XRDC_5.MRGDN[0][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_5     XRDC_5.MRGDN[0][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_5     XRDC_5.MRGDN[0][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_6     XRDC_5.MRGDN[0][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_6     XRDC_5.MRGDN[0][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_6     XRDC_5.MRGDN[0][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_6     XRDC_5.MRGDN[0][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_7     XRDC_5.MRGDN[0][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_7     XRDC_5.MRGDN[0][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_7     XRDC_5.MRGDN[0][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_7     XRDC_5.MRGDN[0][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_8     XRDC_5.MRGDN[0][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_8     XRDC_5.MRGDN[0][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_8     XRDC_5.MRGDN[0][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_8     XRDC_5.MRGDN[0][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_9     XRDC_5.MRGDN[0][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_9     XRDC_5.MRGDN[0][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_9     XRDC_5.MRGDN[0][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_9     XRDC_5.MRGDN[0][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_10    XRDC_5.MRGDN[0][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_10    XRDC_5.MRGDN[0][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_10    XRDC_5.MRGDN[0][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_10    XRDC_5.MRGDN[0][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_11    XRDC_5.MRGDN[0][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_11    XRDC_5.MRGDN[0][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_11    XRDC_5.MRGDN[0][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_11    XRDC_5.MRGDN[0][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_12    XRDC_5.MRGDN[0][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_12    XRDC_5.MRGDN[0][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_12    XRDC_5.MRGDN[0][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_12    XRDC_5.MRGDN[0][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_13    XRDC_5.MRGDN[0][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_13    XRDC_5.MRGDN[0][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_13    XRDC_5.MRGDN[0][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_13    XRDC_5.MRGDN[0][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_14    XRDC_5.MRGDN[0][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_14    XRDC_5.MRGDN[0][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_14    XRDC_5.MRGDN[0][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_14    XRDC_5.MRGDN[0][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_15    XRDC_5.MRGDN[0][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_15    XRDC_5.MRGDN[0][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_15    XRDC_5.MRGDN[0][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_15    XRDC_5.MRGDN[0][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_16    XRDC_5.MRGDN[1][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_16    XRDC_5.MRGDN[1][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_16    XRDC_5.MRGDN[1][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_16    XRDC_5.MRGDN[1][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_17    XRDC_5.MRGDN[1][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_17    XRDC_5.MRGDN[1][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_17    XRDC_5.MRGDN[1][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_17    XRDC_5.MRGDN[1][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_18    XRDC_5.MRGDN[1][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_18    XRDC_5.MRGDN[1][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_18    XRDC_5.MRGDN[1][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_18    XRDC_5.MRGDN[1][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_19    XRDC_5.MRGDN[1][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_19    XRDC_5.MRGDN[1][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_19    XRDC_5.MRGDN[1][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_19    XRDC_5.MRGDN[1][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_20    XRDC_5.MRGDN[1][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_20    XRDC_5.MRGDN[1][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_20    XRDC_5.MRGDN[1][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_20    XRDC_5.MRGDN[1][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_21    XRDC_5.MRGDN[1][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_21    XRDC_5.MRGDN[1][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_21    XRDC_5.MRGDN[1][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_21    XRDC_5.MRGDN[1][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_22    XRDC_5.MRGDN[1][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_22    XRDC_5.MRGDN[1][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_22    XRDC_5.MRGDN[1][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_22    XRDC_5.MRGDN[1][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_23    XRDC_5.MRGDN[1][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_23    XRDC_5.MRGDN[1][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_23    XRDC_5.MRGDN[1][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_23    XRDC_5.MRGDN[1][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_24    XRDC_5.MRGDN[1][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_24    XRDC_5.MRGDN[1][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_24    XRDC_5.MRGDN[1][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_24    XRDC_5.MRGDN[1][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_25    XRDC_5.MRGDN[1][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_25    XRDC_5.MRGDN[1][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_25    XRDC_5.MRGDN[1][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_25    XRDC_5.MRGDN[1][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_26    XRDC_5.MRGDN[1][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_26    XRDC_5.MRGDN[1][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_26    XRDC_5.MRGDN[1][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_26    XRDC_5.MRGDN[1][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_27    XRDC_5.MRGDN[1][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_27    XRDC_5.MRGDN[1][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_27    XRDC_5.MRGDN[1][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_27    XRDC_5.MRGDN[1][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_28    XRDC_5.MRGDN[1][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_28    XRDC_5.MRGDN[1][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_28    XRDC_5.MRGDN[1][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_28    XRDC_5.MRGDN[1][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_29    XRDC_5.MRGDN[1][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_29    XRDC_5.MRGDN[1][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_29    XRDC_5.MRGDN[1][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_29    XRDC_5.MRGDN[1][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_30    XRDC_5.MRGDN[1][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_30    XRDC_5.MRGDN[1][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_30    XRDC_5.MRGDN[1][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_30    XRDC_5.MRGDN[1][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_31    XRDC_5.MRGDN[1][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_31    XRDC_5.MRGDN[1][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_31    XRDC_5.MRGDN[1][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_31    XRDC_5.MRGDN[1][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_32    XRDC_5.MRGDN[2][0].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_32    XRDC_5.MRGDN[2][0].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_32    XRDC_5.MRGDN[2][0].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_32    XRDC_5.MRGDN[2][0].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_33    XRDC_5.MRGDN[2][1].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_33    XRDC_5.MRGDN[2][1].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_33    XRDC_5.MRGDN[2][1].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_33    XRDC_5.MRGDN[2][1].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_34    XRDC_5.MRGDN[2][2].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_34    XRDC_5.MRGDN[2][2].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_34    XRDC_5.MRGDN[2][2].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_34    XRDC_5.MRGDN[2][2].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_35    XRDC_5.MRGDN[2][3].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_35    XRDC_5.MRGDN[2][3].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_35    XRDC_5.MRGDN[2][3].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_35    XRDC_5.MRGDN[2][3].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_36    XRDC_5.MRGDN[2][4].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_36    XRDC_5.MRGDN[2][4].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_36    XRDC_5.MRGDN[2][4].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_36    XRDC_5.MRGDN[2][4].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_37    XRDC_5.MRGDN[2][5].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_37    XRDC_5.MRGDN[2][5].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_37    XRDC_5.MRGDN[2][5].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_37    XRDC_5.MRGDN[2][5].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_38    XRDC_5.MRGDN[2][6].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_38    XRDC_5.MRGDN[2][6].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_38    XRDC_5.MRGDN[2][6].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_38    XRDC_5.MRGDN[2][6].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_39    XRDC_5.MRGDN[2][7].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_39    XRDC_5.MRGDN[2][7].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_39    XRDC_5.MRGDN[2][7].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_39    XRDC_5.MRGDN[2][7].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_40    XRDC_5.MRGDN[2][8].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_40    XRDC_5.MRGDN[2][8].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_40    XRDC_5.MRGDN[2][8].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_40    XRDC_5.MRGDN[2][8].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_41    XRDC_5.MRGDN[2][9].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_41    XRDC_5.MRGDN[2][9].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_41    XRDC_5.MRGDN[2][9].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_41    XRDC_5.MRGDN[2][9].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_42    XRDC_5.MRGDN[2][10].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_42    XRDC_5.MRGDN[2][10].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_42    XRDC_5.MRGDN[2][10].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_42    XRDC_5.MRGDN[2][10].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_43    XRDC_5.MRGDN[2][11].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_43    XRDC_5.MRGDN[2][11].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_43    XRDC_5.MRGDN[2][11].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_43    XRDC_5.MRGDN[2][11].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_44    XRDC_5.MRGDN[2][12].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_44    XRDC_5.MRGDN[2][12].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_44    XRDC_5.MRGDN[2][12].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_44    XRDC_5.MRGDN[2][12].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_45    XRDC_5.MRGDN[2][13].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_45    XRDC_5.MRGDN[2][13].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_45    XRDC_5.MRGDN[2][13].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_45    XRDC_5.MRGDN[2][13].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_46    XRDC_5.MRGDN[2][14].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_46    XRDC_5.MRGDN[2][14].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_46    XRDC_5.MRGDN[2][14].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_46    XRDC_5.MRGDN[2][14].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */
#define XRDC_5_MRGD_W0_47    XRDC_5.MRGDN[2][15].XRDC_MRGD_W0.R /* Memory Region Descriptor Word 0 */
#define XRDC_5_MRGD_W1_47    XRDC_5.MRGDN[2][15].XRDC_MRGD_W1.R /* Memory Region Descriptor Word 1 */
#define XRDC_5_MRGD_W2_47    XRDC_5.MRGDN[2][15].XRDC_MRGD_W2.R /* Memory Region Descriptor Word 2 */
#define XRDC_5_MRGD_W3_47    XRDC_5.MRGDN[2][15].XRDC_MRGD_W3.R /* Memory Region Descriptor Word 3 */

/* ierc_ierb */
#define NETC__IERC_IERB_F0_EC_CFH_DIDVID NETC__IERC_IERB.F0_EC_CFH_DIDVID.R /* Function 0 EC config header device ID and vendor ID register */
#define NETC__IERC_IERB_F0_EC_CFH_SIDSVID NETC__IERC_IERB.F0_EC_CFH_SIDSVID.R /* Function 0 EC config header subsystem ID and subsystem vendor ID register */

/* ierc_pci */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_DID_VID NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_DID_VID.R /* PCI device ID and vendor ID register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_CMD NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_CMD.R /* PCI command register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_STAT NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_STAT.R /* PCI status register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_REVID_CLASSCODE NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_REVID_CLASSCODE.R /* PCI revision ID and classcode register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_CL_SIZE NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_CL_SIZE.R /* PCI cache line size register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_HDR_TYPE NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_HDR_TYPE.R /* PCI header type register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_VID NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_VID.R /* PCI subsystem vendor ID register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_SUBSYS_ID NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_SUBSYS_ID.R /* PCI subsystem ID register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_CAP_PTR NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_CAP_PTR.R /* PCI capabilities pointer register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_INT_LINE NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_INT_LINE.R /* PCI interrupt line register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFH_INT_PIN NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFH_INT_PIN.R /* PCI interrupt pin register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP_LIST NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP_LIST.R /* PCI PCIe capabilities list register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_CAP NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_CAP.R /* PCI PCIe capabilities register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_CAP NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_CAP.R /* PCI PCIe device capabilities register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIE_DEV_STAT NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIE_DEV_STAT.R /* PCI PCIe device status register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP_LIST NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP_LIST.R /* PCI PCI-PM capabilities list register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CAP NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CAP.R /* PCI PCI-PM capabilities register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_CTL_STAT NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIPM_CTL_STAT.R /* PCI PCI-PM control and status register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCI_CFC_PCIPM_DATA NETC__IERC_F0_PCI_HDR_TYPE0.PCI_CFC_PCIPM_DATA.R /* PCI PCI-PM capabilities data register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR NETC__IERC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_EXT_CAP_HDR.R /* PCIe AER extended capability header */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_ROOT_ERR_CMD NETC__IERC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_ROOT_ERR_CMD.R /* PCIe AER root error command register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_ROOT_ERR_STAT NETC__IERC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_ROOT_ERR_STAT.R /* PCIe AER root error status register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCIE_CFC_AER_ERR_SRC_ID NETC__IERC_F0_PCI_HDR_TYPE0.PCIE_CFC_AER_ERR_SRC_ID.R /* PCIe AER error source identification register */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR NETC__IERC_F0_PCI_HDR_TYPE0.PCIE_CFC_RCEC_EPA_EXT_CAP_HDR.R /* PCIe RCEC Endpoint association extended capability header */
#define NETC__IERC_F0_PCI_HDR_TYPE0_PCIE_CFC_RCEC_EPA_BITMAP NETC__IERC_F0_PCI_HDR_TYPE0.PCIE_CFC_RCEC_EPA_BITMAP.R /* PCIe RCEC Endpoint association bitmap registerr */

#ifdef __MWERKS__
#pragma pop
#endif
#ifdef __ghs__
#pragma ghs endnowarning
#endif
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif
#ifdef  __cplusplus
}
#endif
#endif /* ifdef _S32Z2_H */
